--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75, 5724-E87, 5724-L68, 5724-L69
-- * (C) Copyright IBM Corp. 2001, 2005 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================


Create or Replace procedure Grant_sys as
   v_failures      VARCHAR2 (4000) := 'grant_syn: Failed for following objects : ';
   v_fail_flag     INTEGER         := 0;
   failure         EXCEPTION;
   sql_stmt        VARCHAR2 (200);
   v_table_name    user_tables.table_name%TYPE;
   v_object_name   user_objects.object_name%TYPE;
   v_syn_name      user_synonyms.synonym_name%TYPE;
   v_error         NUMBER;
   v_errmsg        VARCHAR2( 250 );
   v_schema        VARCHAR2( 100 )  := UPPER('$Schema_Name$');
   v_wpguser       VARCHAR2( 100 )  := UPPER('$WPGDBUser$');

------------------
-- Grant on Tables 
------------------
   
   PROCEDURE grant_table
   AS

      v_grant_cnt NUMBER := 0;
      
      CURSOR c_table
      IS
         SELECT table_name
           FROM user_tables;

   BEGIN
      OPEN c_table;
      
      LOOP
         FETCH c_table INTO v_table_name;
         EXIT WHEN c_table%NOTFOUND;

         sql_stmt := 'grant select on ' || v_table_name || ' to ' || v_schema || '_SEL';

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_table_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;

         sql_stmt := 'grant insert on ' || v_table_name || ' to ' || v_schema || '_INS';
         
         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_table_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;

         sql_stmt := 'grant update on ' || v_table_name || ' to ' || v_schema || '_UPD';

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_table_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;

         sql_stmt := 'grant delete on ' || v_table_name || ' to ' || v_schema || '_DEL';

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_table_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;
         
         v_grant_cnt := v_grant_cnt + 1;
      END LOOP;

      dbms_output.put_line( v_grant_cnt || ' Table Grants' );

      CLOSE c_table;
   END grant_table;

-------------------------------
-- Grant on views 
-------------------------------

   PROCEDURE grant_view
   AS
      
      v_grant_cnt NUMBER := 0;
      
      CURSOR c_view
      IS
         SELECT object_name
           FROM user_objects
          WHERE object_type = 'VIEW';
          
   BEGIN
      OPEN c_view;
      
      LOOP
         FETCH c_view INTO v_object_name;
         EXIT WHEN c_view%NOTFOUND;

         sql_stmt := 'grant select on ' || v_object_name || ' to ' || v_schema || '_SEL';

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_object_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;

         sql_stmt := 'grant insert on ' || v_object_name || ' to ' || v_schema || '_INS';

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_object_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;

         sql_stmt := 'grant update on ' || v_object_name || ' to ' || v_schema || '_UPD';

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_object_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;

         sql_stmt := 'grant delete on ' || v_object_name || ' to ' || v_schema || '_DEL';

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_object_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;

         v_grant_cnt := v_grant_cnt + 1;
      END LOOP;

      dbms_output.put_line( v_grant_cnt || ' View Grants' );

      CLOSE c_view;
   END grant_view;

-------------------------------
-- Grant on sequences
-------------------------------

   PROCEDURE grant_seq
   AS
      
      v_grant_cnt NUMBER := 0;
      
      CURSOR c_seq
      IS
         SELECT object_name
           FROM user_objects
          WHERE object_type = 'SEQUENCE' ;
          
   BEGIN
      OPEN c_seq;
      
      LOOP
         FETCH c_seq INTO v_object_name;
         EXIT WHEN c_seq%NOTFOUND;
         sql_stmt := 'grant select on ' || v_object_name || ' to ' || v_schema || '_SEL';

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_object_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;
         v_grant_cnt := v_grant_cnt + 1;
      END LOOP;

      dbms_output.put_line( v_grant_cnt || ' Sequence Grants' );

      CLOSE c_seq;
   END grant_seq;

   
--------------------------
-- Clean up synonyms
--------------------------

   PROCEDURE drop_synonyms
   AS

      v_syn_cnt NUMBER := 0;

      CURSOR c_objc
      IS
          select object_name 
            from all_objects
           where object_type = 'SYNONYM'
             and owner = v_wpguser;
   BEGIN
      OPEN c_objc;
      v_syn_cnt := 0;

      LOOP
         FETCH c_objc INTO v_syn_name;
         EXIT WHEN c_objc%NOTFOUND;
         
         sql_stmt := 'drop synonym ' || v_wpguser || '.' || v_syn_name;

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_syn_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;

      END LOOP;

      dbms_output.put_line( v_syn_cnt || ' Database User Synonyms dropped' );
      v_syn_cnt := 0;

      CLOSE c_objc;
   
   END drop_synonyms;

--------------------------
-- Create synonyms
--------------------------

   PROCEDURE make_synonyms
   AS

      v_syn_cnt NUMBER := 0;
   
      CURSOR c_obj
      IS
         SELECT object_name
           FROM user_objects
          WHERE object_type IN ( 'PROCEDURE'
                               , 'FUNCTION'
                               , 'PACKAGE'
                               , 'TABLE'
                               , 'VIEW'
                               , 'SEQUENCE'
                               , 'TYPE' )
            AND object_name not like 'BIN$%'
	;
            
   BEGIN
      OPEN c_obj;

      LOOP
         FETCH c_obj INTO v_object_name;
         EXIT WHEN c_obj%NOTFOUND;
         
         sql_stmt := 'create or replace synonym ' || v_wpguser || '.' || v_object_name || ' for ' || v_schema || '.' || v_object_name;

         BEGIN
            EXECUTE IMMEDIATE sql_stmt;
         EXCEPTION
            WHEN OTHERS
            THEN
               v_failures := v_failures || ' ' || v_object_name || ':' || SQLERRM;
               v_fail_flag := 1;
         END;

         v_syn_cnt := v_syn_cnt + 1;

      END LOOP;

      dbms_output.put_line( v_syn_cnt || ' Synonyms created' );

      CLOSE c_obj;
   END make_synonyms;
--------------------------
-- Grant Execute on Procs
--------------------------
	 PROCEDURE Grant_Procs ( in_username	VARCHAR2) AS
		CURSOR cur_procs IS
		SELECT	object_name
		FROM	all_objects
		WHERE	OWNER = (SELECT username from user_users)
		AND	OBJECT_TYPE IN ('PROCEDURE', 'FUNCTION')
		;

		v_cmdstr	VARCHAR2(500);
		v_count		NUMBER	:= 0;
	BEGIN
		dbms_output.put_line('Granting execute on Procedures to '|| in_username );

		FOR rec_procs IN cur_procs LOOP
		BEGIN
			v_cmdstr := 'Grant Execute On ' || rec_procs.object_name || ' To ' || in_username;
	
			EXECUTE IMMEDIATE v_cmdstr;
			v_count := v_count + 1;

		EXCEPTION
			WHEN Others THEN
				dbms_output.put_line(SQLERRM || ':' || rec_procs.object_name);
		END;
		END LOOP;

		IF v_count > 0 THEN
			dbms_output.put_line('Grants on '||TRIM(TO_CHAR(v_count)) || ' procs for '|| in_username );
		END IF;
	
	END grant_procs;

---------------------------------------------------------
--                     M  A  I  N                      --
---------------------------------------------------------
BEGIN
   grant_table;
   grant_view;
   grant_seq;
   drop_synonyms;
   make_synonyms;
   grant_procs ( v_wpguser );

   IF v_fail_flag > 0
   THEN
      RAISE failure;
   END IF;
EXCEPTION
   WHEN failure THEN
      raise_application_error( -20999, 20002 || '-' || v_failures );
END grant_sys;
/
Call grant_sys()
/
drop procedure grant_sys
/

Spool Off
EXIT;
