# Licensed Materials - Property of IBM
# 5724-E75, 5724-E87, 5724-L68, 5724-L69.
# (C) Copyright IBM Corporation 2001, 2007. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#
# REPROCESSING EVENTS AND BUSINESS DOCS THAT FAIL TO LOG TO THE DATABASE
# 
# When an event or doc in the DATALOGQ JMS queue fails 3 attempts to log to the database, it is inserted
# into the DATALOGERRORQ JMS queue to allow for later reprocessing when the cause of the problem has been fixed.
# 
# To reprocess these failed events and docs this manual utility is used:
# 
# reprocessDbLoggingErrors.sh
# 
# The utility dequeues all the events and docs from DATALOGERRORQ and reenqueues them into DATALOGQ so that the normal DocumentLogReceiver 
# will again attempt to log them to the database.	The utility stops when it has processed all the existing events and docs in DATALOGERRORQ. 
# 
# Any events and docs that again fail to log end up again in DATALOGERRORQ, but the utility ensures that they are reprocessed only once
# (i.e. it does not go into an endless loop with failing events and docs).
# 
# 
# To run the utility make sure that the env variables are correctly defined in reprocessDbLoggingErrors.sh on any router machine:
#
# REPROCESSOR_HOME=router installation root
# JAVA_HOME=$WAS_HOME/was/java
# LOG_REPROCESSOR_CLASSES=$REPROCESSOR_HOME/router/lib
#
# Then just run it on the command line:
#  ./reprocessDbLoggingErrors.sh
#

if [ $# -ne 1 ]
then 
echo The script needs the following parameters
echo 1. Router Servers bootstrap port

echo "USAGE  : reprocessDbLoggingErrors <RouterServer Bootstrap Port>"

echo "SAMPLE : reprocessDbLoggingErrors 56809"

exit 1
fi

BOOTSTRAP_PORT=$1
export BOOTSTRAP_PORT

# The $BCGHUB_HOME$ varaible is replaced during installation, to point to router installation root.
BCGHUB_HOME=$BCGHUB_HOME$
export BCGHUB_HOME

REPROCESSOR_HOME=$BCGHUB_HOME
export REPROCESSOR_HOME

# call the setupcmdline.sh file to set the required JAVA_HOME and WAS_HOME env varaibles.
. "$BCGHUB_HOME/bin/bcgSetupCmdLine.sh"

if [ "x$BACKCONFIG" = "x" ] ; then
	echo "If the location of ws_runtime.jar present in this exectable file is incorrect ,an error would be thrown. "
	echo "Edit this batch file and change the location if it is incorrect."
	echo "Press any key to continue."
read $BACKCONFIG
fi
WAS_HOME=$WAS_HOME$

export WAS_HOME


JAVA_HOME="$BCGHUB_WAS_HOME"/java

export JAVA_HOME

LOG_REPROCESSOR_LIB="$BCGHUB_HOME"/wasND/Profiles/"$RouterProfile"/installedApps/"$CellName"/BCGDocMgr.ear
#LOG_REPROCESSOR_CLASSES=$REPROCESSOR_HOME/router/lib

export LOG_REPROCESSOR_LIB

LOG_REPROCESSOR_CLASSES="$LOG_REPROCESSOR_LIB"/lib

export LOG_REPROCESSOR_CLASSES

WAS_PLUGIN_JAR="$BCGHUB_WAS_HOME"/deploytool/itp/plugins/com.ibm.websphere.v61_6.1.0/ws_runtime.jar:"$BCGHUB_WAS_HOME"/deploytool/itp/plugins/com.ibm.websphere.v61_6.1.0/classes/orbProperties.jar:"$BCGHUB_WAS_HOME"/plugins/com.ibm.ws.runtime_6.1.0.jar

export WAS_PLUGIN_JAR

WAS_JNDI_RUNTIME_JAR="$BCGHUB_WAS_HOME"/runtimes/com.ibm.ws.admin.client_6.1.0.jar

export WAS_JNDI_RUNTIME_JAR

JNDI_URL=corbaloc:iiop:localhost:$BOOTSTRAP_PORT
export JNDI_URL


LOG_REPROCESSOR_CLASSPATH=.:"$WAS_PLUGIN_JAR":"$LOG_REPROCESSOR_LIB"/ejb/bcgDocMgrEjb.jar:"$LOG_REPROCESSOR_CLASSES"/bcgCommon.jar:$LOG_REPROCESSOR_CLASSES/bcgServiceClient.jar:"$LOG_REPROCESSOR_CLASSES"/bcgService.jar:"$LOG_REPROCESSOR_CLASSES"/bcgsdk.jar:"$BCGHUB_WAS_HOME"/lib/j2ee.jar:"$WAS_JNDI_RUNTIME_JAR"

export LOG_REPROCESSOR_CLASSPATH


$JAVA_HOME/bin/java -Dcom.ibm.CORBA.Debug=true -Dcom.ibm.CORBA.CommTrace=true -classpath "$LOG_REPROCESSOR_CLASSPATH" com.ibm.bcg.server.serviceclient.eventEng.ReprocessEvent "$JNDI_URL"

