--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================
-- -- # ============================================================== # 
-- # Alter Tables 
-- -- # ============================================================== # 

-- 620FP1MigrationBase.sql

CREATE OR REPLACE PROCEDURE CRE_BP_RNSTATEDTL
AS

/*

In Oracle 11.1.0.7 tables having LOB COlumns have failed.  Reason is LOB Storage attribute 'next'
value is '0'

As Per Oracle Recommendation 'next' attribute value should be greater than chunk value.

In case Customer Installed 62 base in Oracle 11.1.0.7,  this script needs to be executed.

It sees whether tables having LOB columns ( 8 tables ) are present in dictionary.  IF not it 
recreates the table with new defnition.


	================================================================================
	Proc Name:       CRE_BP_RNSTATEDTL
	Create Date:     2009-06-11 00:10:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 BHARANI J        1.0 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

*/




v_flag integer;
v_sql varchar2(3000);
v_index1 varchar2(500);
v_index2 varchar2(500);
v_const1 varchar2(500);
v_tablename user_indexes.index_name%type;


BEGIN

	BEGIN

		SELECT table_name into v_tablename from user_tables 
				where table_name =  'BP_RNSTATEDTL';
                                  
		EXCEPTION

		WHEN NO_DATA_FOUND THEN
		v_flag := 1;
		dbms_output.put_line(' Table BP_RNSTATEDTL NOT exists in DB ');
	END;


	IF  v_tablename IS NULL THEN
	
	-- Create the table.  It is not found in dictionary.
	
		v_sql := ' CREATE TABLE BP_RNSTATEDTL 
		(
    		RNSTATEDTLID          NUMBER(10)    NOT NULL,
    		INITBUSINESSID        VARCHAR2(60)  NOT NULL,
    		PROCESSINSTANCEID     VARCHAR2(255) NOT NULL,
    		TRANSACTIONINSTANCEID VARCHAR2(255)     NULL,
    		ACTIONINSTANCEID      VARCHAR2(255)     NULL,
    		SIGNALINSTANCEID      VARCHAR2(255)     NULL,
    		MSGTRACKINGINSTANCEID VARCHAR2(255)     NULL,
    		GLBUSAGECD            VARCHAR2(10)      NULL,
    		ACTIONVER             VARCHAR2(15)      NULL,
    		SIGNALVER             VARCHAR2(15)      NULL,
    		GLBBUSACTIONCD        VARCHAR2(255)     NULL,
    		GLBBUSSIGNALCD        VARCHAR2(60)      NULL,
    		FRBUSINESSID          VARCHAR2(60)      NULL,
    		TOBUSINESSID          VARCHAR2(60)      NULL,
    		ACTIVITYNAME          VARCHAR2(255)     NULL,
    		MSGDATETIME           VARCHAR2(20)      NULL,
    		EXPIRETPATIME         TIMESTAMP(6)      NULL,
    		ATTEMPTCNT            NUMBER(3)         NULL,
    		ATTEMPTCNTCALC        NUMBER(3)         NULL,
    		ATTEMPTCNTTPA         NUMBER(3)         NULL,
    		FRSUPPLYCHAINCD       VARCHAR2(255)     NULL,
    		TOSUPPLYCHAINCD       VARCHAR2(255)     NULL,
    		DOCID                 VARCHAR2(255)     NULL,
    		DOCTS                 VARCHAR2(60)      NULL,
    		ACTIVITYID            VARCHAR2(65)  NOT NULL,
    		INBOUNDVUID           VARCHAR2(65)  NOT NULL,
    		OUTBOUNDVUID          VARCHAR2(65)      NULL,
    		CUSTOMMSGID01         VARCHAR2(255)     NULL,
    		CUSTOMMSGID02         VARCHAR2(65)      NULL,
    		ROWTS                 NUMBER(10)    DEFAULT 1 NOT NULL,
    		CREATEDATE            TIMESTAMP(6)  DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL,
    		UPDATEDATE            TIMESTAMP(6)  DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL,
    		MSGDIGEST             CLOB              NULL,
    		DIGITALSIGALGO        VARCHAR2(30)      NULL,
    		TimeToAck_TPA 	  Number(10)	    NULL,
    		TimeToPerform_TPA         Number(10)    NULL,
    		RetryCount_TPA            Number(10)    NULL,
    		GLOBALSUPPLYCHAINCD_TPA   VARCHAR2(255) NULL,
    		SyncAckRequired_TPA       VARCHAR2(1)   NULL 
		)
		LOB(MSGDIGEST) STORE AS SYS_LOB0000061169C00033$$
		(
		TABLESPACE TS_LOG
		STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
		PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
		ENABLE STORAGE IN ROW
		NOCACHE
		LOGGING
		CHUNK 16384
		PCTVERSION 10
		)
		TABLESPACE TS_LOG
		LOGGING
		PCTFREE 10
		PCTUSED 40
		INITRANS 1
		MAXTRANS 255
		STORAGE(FREELISTS 1
        	FREELIST GROUPS 1
        	BUFFER_POOL DEFAULT)
		NOPARALLEL ';
		
		v_index1 := 'CREATE INDEX AK_RNSTATEDTL_1
		    ON BP_RNSTATEDTL(INITBUSINESSID,PROCESSINSTANCEID)
		PCTFREE 10
		INITRANS 2
		MAXTRANS 255
		TABLESPACE TS_LOG
		STORAGE(FREELISTS 1
		        FREELIST GROUPS 1
		        BUFFER_POOL DEFAULT)
		LOGGING
		NOPARALLEL
		NOCOMPRESS';
		
		v_index2 := 'CREATE INDEX AK_RNSTATEDTL_2
		    ON BP_RNSTATEDTL(INBOUNDVUID)
		PCTFREE 10
		INITRANS 2
		MAXTRANS 255
		TABLESPACE TS_LOG
		STORAGE(FREELISTS 1
		        FREELIST GROUPS 1
		        BUFFER_POOL DEFAULT)
		LOGGING
		NOPARALLEL
		NOCOMPRESS';
		
		v_const1 := 'ALTER TABLE BP_RNSTATEDTL 
		    ADD CONSTRAINT PK_RNSTATEDTL
		PRIMARY KEY (RNSTATEDTLID)
		USING INDEX PCTFREE 10
		            INITRANS 2
		            MAXTRANS 255
		            TABLESPACE TS_LOG
		            STORAGE(FREELISTS 1
		                   FREELIST GROUPS 1
		                   BUFFER_POOL DEFAULT)
		LOGGING ENABLE VALIDATE';
	
		dbms_output.put_line('v_sql' || v_sql);
		
		-- Create table first
		
		EXECUTE IMMEDIATE (v_sql);
		
		-- Create Indexes and Constraints
		
		EXECUTE IMMEDIATE (v_index1);
		EXECUTE IMMEDIATE (v_index2);
		EXECUTE IMMEDIATE (v_const1);
		
	END IF;
	
EXCEPTION
	WHEN OTHERS THEN
	RAISE;

END CRE_BP_RNSTATEDTL;
/

CALL CRE_BP_RNSTATEDTL()
/

CREATE OR REPLACE PROCEDURE CRE_BP_RNSTATEDTL_HIST
AS

/*

In Oracle 11.1.0.7 tables having LOB COlumns have failed.  Reason is LOB Storage attribute 'next'
value is '0'

As Per Oracle Recommendation 'next' attribute value should be greater than chunk value.

In case Customer Installed 62 base in Oracle 11.1.0.7,  this script needs to be executed.

It sees whether tables having LOB columns ( 8 tables ) are present in dictionary.  IF not it 
recreates the table with new defnition.


	================================================================================
	Proc Name:       CRE_BP_RNSTATEDTL_HIST
	Create Date:     2009-06-11 00:10:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 BHARANI J        1.0 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

*/




v_flag integer;
v_sql varchar2(3000);
v_index1 varchar2(500);
v_index2 varchar2(500);
v_const1 varchar2(500);
v_tablename user_indexes.index_name%type;


BEGIN

	BEGIN

		SELECT table_name into v_tablename from user_tables 
				where table_name =  'BP_RNSTATEDTL_HIST';
                                  
		EXCEPTION

		WHEN NO_DATA_FOUND THEN
		v_flag := 1;
		dbms_output.put_line(' Table BP_RNSTATEDTL_HIST NOT exists in DB ');
	END;


	IF  v_tablename IS NULL THEN
	
	-- Create the table.  It is not found in dictionary.
	
		v_sql := ' CREATE TABLE BP_RNSTATEDTL_HIST 
			(
    			RNSTATEDTLID          NUMBER(10)    NOT NULL,
    			INITBUSINESSID        VARCHAR2(60)      NULL,
    			PROCESSINSTANCEID     VARCHAR2(255)     NULL,
    			TRANSACTIONINSTANCEID VARCHAR2(255)     NULL,
    			ACTIONINSTANCEID      VARCHAR2(255)     NULL,
    			SIGNALINSTANCEID      VARCHAR2(255)     NULL,
    			MSGTRACKINGINSTANCEID VARCHAR2(255)     NULL,
    			GLBUSAGECD            VARCHAR2(10)      NULL,
    			ACTIONVER             VARCHAR2(15)      NULL,
    			SIGNALVER             VARCHAR2(15)      NULL,
    			GLBBUSACTIONCD        VARCHAR2(255)     NULL,
    			GLBBUSSIGNALCD        VARCHAR2(60)      NULL,
    			FRBUSINESSID          VARCHAR2(20)      NULL,
    			TOBUSINESSID          VARCHAR2(20)      NULL,
    			ACTIVITYNAME          VARCHAR2(255)     NULL,
    			MSGDATETIME           VARCHAR2(20)      NULL,
    			EXPIRETPATIME         TIMESTAMP(6)      NULL,
    			ATTEMPTCNT            NUMBER(3)         NULL,
    			ATTEMPTCNTCALC        NUMBER(3)         NULL,
    			ATTEMPTCNTTPA         NUMBER(3)         NULL,
    			FRSUPPLYCHAINCD       VARCHAR2(255)     NULL,
    			TOSUPPLYCHAINCD       VARCHAR2(255)     NULL,
    			DOCID                 VARCHAR2(255)     NULL,
    			DOCTS                 VARCHAR2(60)      NULL,
    			ACTIVITYID            VARCHAR2(65)      NULL,
    			INBOUNDVUID           VARCHAR2(65)      NULL,
    			OUTBOUNDVUID          VARCHAR2(65)      NULL,
    			CUSTOMMSGID01         VARCHAR2(255)     NULL,
    			CUSTOMMSGID02         VARCHAR2(65)      NULL,
    			ROWTS                 NUMBER(10)        NULL,
    			CREATEDATE            TIMESTAMP(6)      NULL,
    			UPDATEDATE            TIMESTAMP(6)      NULL,
    			MSGDIGEST             CLOB              NULL,
    			DIGITALSIGALGO        VARCHAR2(30)      NULL,
    			TimeToAck_TPA 	  Number(10)	    NULL,
    			TimeToPerform_TPA         Number(10)    NULL,
    			RetryCount_TPA            Number(10)    NULL,
    			GLOBALSUPPLYCHAINCD_TPA   VARCHAR2(255) NULL,
    			SyncAckRequired_TPA       VARCHAR2(1)   NULL
			)
			LOB(MSGDIGEST) STORE AS SYS_LOB0000032280C00033$$
			(
			TABLESPACE TS_LOG
			STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
				PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
			ENABLE STORAGE IN ROW
			NOCACHE
			LOGGING
			CHUNK 16384
			PCTVERSION 10
			)
			TABLESPACE TS_LOG
			LOGGING
			PCTFREE 10
			PCTUSED 40
			INITRANS 1
			MAXTRANS 255
			STORAGE(FREELISTS 1
       			FREELIST GROUPS 1
        		BUFFER_POOL DEFAULT)
			NOPARALLEL ' ;


	
		dbms_output.put_line('v_sql' || v_sql);
		
		-- Create table first
		
		EXECUTE IMMEDIATE (v_sql);
		
	END IF;
EXCEPTION
	WHEN OTHERS THEN
	RAISE;

END CRE_BP_RNSTATEDTL_HIST;
/

CALL CRE_BP_RNSTATEDTL_HIST()
/

CREATE OR REPLACE PROCEDURE CRE_CF_BUSINESSLOGO
AS

/*

In Oracle 11.1.0.7 tables having LOB COlumns have failed.  Reason is LOB Storage attribute 'next'
value is '0'

As Per Oracle Recommendation 'next' attribute value should be greater than chunk value.

In case Customer Installed 62 base in Oracle 11.1.0.7,  this script needs to be executed.

It sees whether tables having LOB columns ( 8 tables ) are present in dictionary.  IF not it 
recreates the table with new defnition.


	================================================================================
	Proc Name:       CRE_CF_BUSINESSLOGO
	Create Date:     2009-06-11 00:10:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 BHARANI J        1.0 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

*/




v_flag integer;
v_sql varchar2(3000);
v_index1 varchar2(500);
v_index2 varchar2(500);
v_const1 varchar2(500);
v_tablename user_indexes.index_name%type;
v_indextype VARCHAR2(20);

BEGIN

	BEGIN

		SELECT table_name into v_tablename from user_tables 
				where table_name =  'CF_BUSINESSLOGO';
                                  
		EXCEPTION

		WHEN NO_DATA_FOUND THEN
		v_flag := 1;
		dbms_output.put_line(' Table CRE_CF_BUSINESSLOGO NOT exists in DB ');
	END;


	IF  v_tablename IS NULL THEN
	
	-- Create the table.  It is not found in dictionary.
	
	

		v_sql := ' CREATE TABLE CF_BUSINESSLOGO 
			(
	    		BUSINESSLOGOID          NUMBER(10)   NOT NULL,
	    		BUSINESSLOGONAME        VARCHAR2(30)     NULL,
	    		BRANDINGLOGOIMAGE       BLOB             NULL,
	    		BRANDINGBACKGROUNDIMAGE BLOB             NULL,
	    		ROWTS                   NUMBER(10)   DEFAULT 1 NOT NULL,
	    		CREATEDATE              TIMESTAMP(6)         DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL,
	    		UPDATEDATE              TIMESTAMP(6)         DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL
			)
			LOB(BRANDINGLOGOIMAGE) STORE AS SYS_LOB0000061204C00003$$
			(
			TABLESPACE TS_PROFILE
			STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
			PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
			ENABLE STORAGE IN ROW
			NOCACHE
			LOGGING
			CHUNK 16384
			PCTVERSION 10
			)
			LOB(BRANDINGBACKGROUNDIMAGE) STORE AS SYS_LOB0000061204C00004$$
			(
			TABLESPACE TS_PROFILE
			STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
			PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
			ENABLE STORAGE IN ROW
			NOCACHE
			LOGGING
			CHUNK 16384
			PCTVERSION 10
			)
			TABLESPACE TS_PROFILE
			LOGGING
			PCTFREE 10
			PCTUSED 40
			INITRANS 1
			MAXTRANS 255
			STORAGE(FREELISTS 1
	        	FREELIST GROUPS 1
	        	BUFFER_POOL DEFAULT)
			NOPARALLEL ' ;
	
		v_index1 := ' CREATE UNIQUE INDEX UK_BUSINESSLOGO_1
	    		ON CF_BUSINESSLOGO(BUSINESSLOGONAME)
			PCTFREE 10
			INITRANS 2
			MAXTRANS 255
			TABLESPACE TS_PROFILE
			STORAGE(FREELISTS 1
	        	FREELIST GROUPS 1
	        	BUFFER_POOL DEFAULT)
			LOGGING
			NOPARALLEL
			NOCOMPRESS ' ;
	
		v_index2 := ' ALTER TABLE CF_BUSINESSLOGO 
	    		ADD CONSTRAINT PK_BUSINESSLOGO
			PRIMARY KEY (BUSINESSLOGOID)
			USING INDEX PCTFREE 10
	            	INITRANS 2
	            	MAXTRANS 255
	            	TABLESPACE TS_PROFILE
	            	STORAGE(FREELISTS 1
	                   FREELIST GROUPS 1
	                   BUFFER_POOL DEFAULT)
			LOGGING ENABLE VALIDATE ' ;
	

		dbms_output.put_line('v_sql' || v_sql);
		
		-- Create table first
		
		EXECUTE IMMEDIATE (v_sql);
		EXECUTE IMMEDIATE (v_index1);
		EXECUTE IMMEDIATE (v_index2);
		
	END IF;
EXCEPTION
	WHEN OTHERS THEN
	RAISE;	

END CRE_CF_BUSINESSLOGO;
/

CALL CRE_CF_BUSINESSLOGO()
/

CREATE OR REPLACE PROCEDURE CRE_CF_CRL
AS

/*

In Oracle 11.1.0.7 tables having LOB COlumns have failed.  Reason is LOB Storage attribute 'next'
value is '0'

As Per Oracle Recommendation 'next' attribute value should be greater than chunk value.

In case Customer Installed 62 base in Oracle 11.1.0.7,  this script needs to be executed.

It sees whether tables having LOB columns ( 8 tables ) are present in dictionary.  IF not it 
recreates the table with new defnition.


	================================================================================
	Proc Name:       CRE_CF_CRL
	Create Date:     2009-06-11 00:10:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 BHARANI J        1.0 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

*/




v_flag integer;
v_sql varchar2(3000);
v_index1 varchar2(500);
v_index2 varchar2(500);
v_tablename user_indexes.index_name%type;
v_indextype VARCHAR2(20);

BEGIN

	BEGIN

		SELECT table_name into v_tablename from user_tables 
				where table_name =  'CF_CRL';
                                  
		EXCEPTION

		WHEN NO_DATA_FOUND THEN
		v_flag := 1;
		dbms_output.put_line(' Table CF_CRL NOT exists in DB ');
	END;


	IF  v_tablename IS NULL THEN
	
	-- Create the table.  It is not found in dictionary.

		v_sql := 'CREATE TABLE CF_CRL
			(
	    		CRLID     NUMBER(10)     NOT NULL,
	    		CRL   BLOB  NOT NULL,
	    		ISSUERDN    VARCHAR2(250)     NOT NULL,
	    		ROWTS          NUMBER(10)     DEFAULT 1 NOT NULL,
	    		CREATEDATE     TIMESTAMP(6)           DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL,
	    		UPDATEDATE     TIMESTAMP(6)           DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL
			)
			LOB(CRL) STORE AS SYS_LOB0000561493C00020$$
			(
			TABLESPACE TS_PROFILE
			STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
			PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
			ENABLE STORAGE IN ROW
			NOCACHE
			LOGGING
			CHUNK 16384
			PCTVERSION 10
			)
			TABLESPACE TS_PROFILE
			LOGGING
			PCTFREE 10
			PCTUSED 40
			INITRANS 1
			MAXTRANS 255
			STORAGE(FREELISTS 1
	        	FREELIST GROUPS 1
	        	BUFFER_POOL DEFAULT)
			NOPARALLEL  ' ;
	
		 v_index1 := ' ALTER TABLE CF_CRL
	    		       ADD CONSTRAINT PK_CRL
			       PRIMARY KEY (CRLID)
			       USING INDEX PCTFREE 10
	            	       INITRANS 2
	            	       MAXTRANS 255
	            	       TABLESPACE TS_PROFILE
	            	       STORAGE(FREELISTS 1
	                       FREELIST GROUPS 1
	                       BUFFER_POOL DEFAULT)
			       LOGGING ENABLE VALIDATE ' ;
	
		v_index2 := ' CREATE UNIQUE INDEX UK_CF_CRL_1
	    			ON CF_CRL(ISSUERDN)
				PCTFREE 10
				INITRANS 2
				MAXTRANS 255
				TABLESPACE TS_LOG
				STORAGE(FREELISTS 1
	        		FREELIST GROUPS 1
	        		BUFFER_POOL DEFAULT)
				LOGGING
				NOPARALLEL
				NOCOMPRESS ';

		dbms_output.put_line('v_sql' || v_sql);
		
		-- Create table first
		
		EXECUTE IMMEDIATE (v_sql);
		EXECUTE IMMEDIATE (v_index1);
		EXECUTE IMMEDIATE (v_index2);
		
	END IF;
EXCEPTION
	WHEN OTHERS THEN
	RAISE;
END CRE_CF_CRL;
/

CALL CRE_CF_CRL()
/

CREATE OR REPLACE PROCEDURE CRE_CF_GUIDELINEMAP
AS

/*

In Oracle 11.1.0.7 tables having LOB COlumns have failed.  Reason is LOB Storage attribute 'next'
value is '0'

As Per Oracle Recommendation 'next' attribute value should be greater than chunk value.

In case Customer Installed 62 base in Oracle 11.1.0.7,  this script needs to be executed.

It sees whether tables having LOB columns ( 8 tables ) are present in dictionary.  IF not it 
recreates the table with new defnition.


	================================================================================
	Proc Name:       CRE_CF_GUIDELINEMAP
	Create Date:     2009-06-11 00:10:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 BHARANI J        1.0 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

*/




v_flag integer;
v_sql varchar2(3000);
v_index1 varchar2(500);
v_index2 varchar2(500);
v_seq1 varchar2(500);
v_tablename user_indexes.index_name%type;
v_fk1 VARCHAR2(300);
v_fk2 VARCHAR2(300);

BEGIN

	BEGIN

		SELECT table_name into v_tablename from user_tables 
				where table_name =  'CF_GUIDELINEMAP';
                                  
		EXCEPTION

		WHEN NO_DATA_FOUND THEN
		v_flag := 1;
		dbms_output.put_line(' Table CF_GUIDELINEMAP NOT exists in DB ');
	END;


	IF  v_tablename IS NULL THEN
	
	-- Create the table.  It is not found in dictionary.

		v_sql := 'CREATE TABLE CF_GUIDELINEMAP 
		(
    		GUIDELINEMAPID       NUMBER(10)     NOT NULL,
    		GUIDELINEMAPFILENAME VARCHAR2(255)  NOT NULL,
    		GUIDELINEMAPDESC     VARCHAR2(2000)     NULL,
    		GUIDELINEMAP         BLOB               NULL,
    		COMMONMAPFLG         NUMBER(1)      DEFAULT 0     NULL,
    		ROWTS                NUMBER(10)     DEFAULT 1 NOT NULL,
    		CREATEDATE           TIMESTAMP(6)           DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL,
    		UPDATEDATE           TIMESTAMP(6)           DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL,
    		GUIDELINEMAPSIZE     NUMBER(10)         NULL,
    		groupname            VARCHAR2(255)      NULL
		)
		LOB(GUIDELINEMAP) STORE AS SYS_LOB0000059032C00004$$
		(
		TABLESPACE TS_PROFILE
		STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
		PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
		ENABLE STORAGE IN ROW
		NOCACHE
		LOGGING
		CHUNK 16384
		PCTVERSION 10
		)
		TABLESPACE TS_PROFILE
		LOGGING
		PCTFREE 10
		PCTUSED 40
		INITRANS 1
		MAXTRANS 255
		STORAGE(FREELISTS 1
        	FREELIST GROUPS 1
        	BUFFER_POOL DEFAULT)
		NOPARALLEL  ' ;
	
		 v_index1 := ' CREATE UNIQUE INDEX UK_GUIDELINEMAP_1
    				ON CF_GUIDELINEMAP(GUIDELINEMAPFILENAME,groupname)
				PCTFREE 10
				INITRANS 2
				MAXTRANS 255
				TABLESPACE TS_PROFILE
				STORAGE(FREELISTS 1
        			FREELIST GROUPS 1
        			BUFFER_POOL DEFAULT)
				LOGGING
				NOPARALLEL
				NOCOMPRESS ' ;
	
		v_index2 := ' ALTER TABLE CF_GUIDELINEMAP 
    			      ADD CONSTRAINT PK_GUIDLINEMAP
			      PRIMARY KEY (GUIDELINEMAPID)
			      USING INDEX PCTFREE 10
            		      INITRANS 2
            		      MAXTRANS 255
            		      TABLESPACE TS_PROFILE
            		      STORAGE(FREELISTS 1
                   	      FREELIST GROUPS 1
                              BUFFER_POOL DEFAULT)
			      LOGGING ENABLE VALIDATE ';

		dbms_output.put_line('v_sql' || v_sql);
		
		-- Before Adding below FK, delete the rows from CF_GUIDELINEMAP_LNK so that Validation 
		-- will be scuccessfull.
		
		DELETE FROM CF_GUIDELINEMAP_LNK;
		
		DELETE FROM PR_ROUTINGATTR_LNK;
		
		-- Later deleted rows will be added during Migration scripts
		
		v_fk1     :=  'ALTER TABLE CF_GUIDELINEMAP_LNK 
		    	       ADD CONSTRAINT FK_GDLNMAP_LK_2
		    	       FOREIGN KEY (GUIDELINEMAPID)
		               REFERENCES CF_GUIDELINEMAP (GUIDELINEMAPID)
   			       ENABLE ';
   			       
   		v_fk2     :=  'ALTER TABLE PR_ROUTINGATTR_LNK 
			       ADD CONSTRAINT FK_RTATTR_LK_6
			       FOREIGN KEY (GUIDELINEMAPID)
			       REFERENCES CF_GUIDELINEMAP (GUIDELINEMAPID)
    			       ENABLE ';
		
		-- Create table first
		
		EXECUTE IMMEDIATE (v_sql);
		EXECUTE IMMEDIATE (v_index1);
		EXECUTE IMMEDIATE (v_index2);
		EXECUTE IMMEDIATE (v_fk1);
		EXECUTE IMMEDIATE (v_fk2);
		
		
	END IF;
EXCEPTION
	WHEN OTHERS THEN
	RAISE;	

END CRE_CF_GUIDELINEMAP;
/

CALL CRE_CF_GUIDELINEMAP()
/

CREATE OR REPLACE PROCEDURE CRE_CF_TRANSFORMMAP
as
/*

In Oracle 11.1.0.7 tables having LOB COlumns have failed.  Reason is LOB Storage attribute 'next'
value is '0'

As Per Oracle Recommendation 'next' attribute value should be greater than chunk value.

In case Customer Installed 62 base in Oracle 11.1.0.7,  this script needs to be executed.

It sees whether tables having LOB columns ( 8 tables ) are present in dictionary.  IF not it 
recreates the table with new defnition.


	================================================================================
	Proc Name:       CRE_CF_TRANSFORMMAP
	Create Date:     2009-06-11 00:10:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 BHARANI J        1.0 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

*/




v_flag integer;
v_sql varchar2(3000);
v_index1 varchar2(500);
v_index2 varchar2(500);
v_seq1 varchar2(500);
v_tablename user_indexes.index_name%type;
v_fk1 VARCHAR2(300);
v_fk2 VARCHAR2(300);
v_fk3 VARCHAR2(300);

BEGIN

	BEGIN

		SELECT table_name into v_tablename from user_tables 
				where table_name =  'CF_TRANSFORMMAP';
                                  
		EXCEPTION

		WHEN NO_DATA_FOUND THEN
		v_flag := 1;
		dbms_output.put_line(' Table CRE_CF_TRANSFORMMAP NOT exists in DB ');
	END;


	IF  v_tablename IS NULL THEN
	
	-- Create the table.  It is not found in dictionary.
	
		v_sql := 'CREATE TABLE CF_TRANSFORMMAP 
			(
	   		 TRANSFORMMAPID       NUMBER(10)     NOT NULL,
	   		 TRANSFORMMAPFILENAME VARCHAR2(255)      NULL,
	    		 TRANSFORMMAPDESC     VARCHAR2(2000)     NULL,
	    		 TRANSFORMMAP         BLOB               NULL,
	    		 ROWTS                NUMBER(10)     DEFAULT 1 NOT NULL,
	    		 CREATEDATE           TIMESTAMP(6)           DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL,
	    		 UPDATEDATE           TIMESTAMP(6)           DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL
			)
			LOB(TRANSFORMMAP) STORE AS SYS_LOB0000059317C00004$$
			(
			TABLESPACE TS_PROFILE
			STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
			PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
			ENABLE STORAGE IN ROW
			PCTVERSION 10
			)
			TABLESPACE TS_PROFILE
			LOGGING
			PCTFREE 10
			PCTUSED 40
			INITRANS 1
			MAXTRANS 255
			STORAGE(FREELISTS 1
		        FREELIST GROUPS 1
		        BUFFER_POOL DEFAULT)
			NOPARALLEL  ' ;
	
		 v_index1 := ' CREATE UNIQUE INDEX UK_TRANSFORMMAP_1
	    		       ON CF_TRANSFORMMAP(TRANSFORMMAPFILENAME)
			       PCTFREE 10
			       INITRANS 2
				MAXTRANS 255
			       TABLESPACE TS_PROFILE
				STORAGE(FREELISTS 1
	       			FREELIST GROUPS 1
	        		BUFFER_POOL DEFAULT)
				LOGGING
				NOPARALLEL
				NOCOMPRESS ' ;
	
		v_index2 := ' ALTER TABLE CF_TRANSFORMMAP 
	    		      ADD CONSTRAINT PK_TRANSFORMMAP
			      PRIMARY KEY (TRANSFORMMAPID)
			      USING INDEX PCTFREE 10
	            	      INITRANS 2
	            	      MAXTRANS 255
	            	      TABLESPACE TS_PROFILE
	            	      STORAGE(FREELISTS 1
	                      FREELIST GROUPS 1
	                      BUFFER_POOL DEFAULT)
			      LOGGING ENABLE VALIDATE ';

		dbms_output.put_line('v_sql' || v_sql);
		
		v_fk1 := 'ALTER TABLE PR_ROUTINGCHANNEL 
		    	  ADD CONSTRAINT FK_RTCHNL_1
		    	  FOREIGN KEY (TRANSFORMMAPID)
		    	  REFERENCES CF_TRANSFORMMAP (TRANSFORMMAPID)
		    	  ON DELETE SET NULL ENABLE';
    		
    		v_fk2 := 'ALTER TABLE CF_TRANSFORMINTERACT_LNK 
    			  ADD CONSTRAINT FK_TRANFMINT_LK_1
    			  FOREIGN KEY (TRANSFORMMAPID)
    			  REFERENCES CF_TRANSFORMMAP (TRANSFORMMAPID)
   			  ON DELETE CASCADE
   			  ENABLE ';
   			  
   		v_fk3  := 'ALTER TABLE CF_TRANSFORMROUTINGOBJ_LNK ADD CONSTRAINT FK_CF_TRARO_LNK_2
			   FOREIGN KEY (TRANSFORMMAPID)
			   REFERENCES CF_TRANSFORMMAP (TRANSFORMMAPID)
                           ON DELETE CASCADE
			   ENABLE ';
		
		-- Create table first
		
		EXECUTE IMMEDIATE (v_sql);
		EXECUTE IMMEDIATE (v_index1);
		EXECUTE IMMEDIATE (v_index2);
		EXECUTE IMMEDIATE (v_fk1);
		EXECUTE IMMEDIATE (v_fk2);
		EXECUTE IMMEDIATE (v_fk3);
		
		
	END IF;
EXCEPTION
	WHEN OTHERS THEN
	RAISE;	

END CRE_CF_TRANSFORMMAP;
/

CALL CRE_CF_TRANSFORMMAP()
/

CREATE OR REPLACE PROCEDURE CRE_EDICSTX
AS

/*

In Oracle 11.1.0.7 tables having LOB COlumns have failed.  Reason is LOB Storage attribute 'next'
value is '0'

As Per Oracle Recommendation 'next' attribute value should be greater than chunk value.

In case Customer Installed 62 base in Oracle 11.1.0.7,  this script needs to be executed.

It sees whether tables having LOB columns ( 8 tables ) are present in dictionary.  IF not it 
recreates the table with new defnition.


	================================================================================
	Proc Name:       CRE_EDICSTX
	Create Date:     2009-06-11 00:10:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 BHARANI J        1.0 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

*/




v_flag integer;
v_sql varchar2(3000);
v_index1 varchar2(500);
v_index2 varchar2(500);
v_seq1 varchar2(500);
v_tablename user_indexes.index_name%type;
v_indextype VARCHAR2(20);

BEGIN

	BEGIN

		SELECT table_name into v_tablename from user_tables 
				where table_name =  'EDICSTX';
                                  
		EXCEPTION

		WHEN NO_DATA_FOUND THEN
		v_flag := 1;
		dbms_output.put_line(' Table EDICSTX NOT exists in DB ');
	END;


	IF  v_tablename IS NULL THEN
	
	-- Create the table.  It is not found in dictionary.

		v_sql := 'CREATE TABLE EDICSTX(
    			CSTYPE     NCHAR(4)           ,
    			MAPID      NCHAR(16)          ,
    			CSBYTES    INTEGER            ,
    			CSLINUM    INTEGER            ,
    			CSCONT     NCHAR(1)           ,
    			GENDATE    NCHAR(8)           ,
    			GENTIME    NCHAR(4)           ,
    			STRINGS    BLOB               
			)
			LOB(STRINGS) STORE AS SYS_LOB0000041632C00004$$
			(
			TABLESPACE TS_PROFILE
			STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
			PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
			ENABLE STORAGE IN ROW
			NOCACHE
			LOGGING
			CHUNK 16384
			PCTVERSION 10
			)
			TABLESPACE TS_PROFILE
			LOGGING
			PCTFREE 10
			PCTUSED 40
			INITRANS 1
			MAXTRANS 255
			STORAGE(FREELISTS 1
		        FREELIST GROUPS 1
		        BUFFER_POOL DEFAULT)
			NOPARALLEL
			' ;
	
		 v_index1 := ' CREATE UNIQUE INDEX EDICSTXX
    				ON EDICSTX(MAPID,CSTYPE,CSLINUM)
				PCTFREE 10
				INITRANS 2
				MAXTRANS 255
				TABLESPACE TS_PROFILE
				STORAGE(FREELISTS 1
    				FREELIST GROUPS 1
     				BUFFER_POOL DEFAULT)
				LOGGING
				NOPARALLEL
				NOCOMPRESS
				' ;
	
		
		
		EXECUTE IMMEDIATE (v_sql);
		EXECUTE IMMEDIATE (v_index1);
		
		
		
	END IF;
EXCEPTION
	WHEN OTHERS THEN
	RAISE;	

END CRE_EDICSTX;
/

CALL CRE_EDICSTX()
/

CREATE OR REPLACE PROCEDURE CRE_PR_CERT
AS

/*

In Oracle 11.1.0.7 tables having LOB COlumns have failed.  Reason is LOB Storage attribute 'next'
value is '0'

As Per Oracle Recommendation 'next' attribute value should be greater than chunk value.

In case Customer Installed 62 base in Oracle 11.1.0.7,  this script needs to be executed.

It sees whether tables having LOB columns ( 8 tables ) are present in dictionary.  IF not it 
recreates the table with new defnition.


	================================================================================
	Proc Name:       CRE_PR_CERT
	Create Date:     2009-06-11 00:10:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 BHARANI J        1.0 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

*/




v_flag integer;
v_sql varchar2(3000);
v_index1 varchar2(500);
v_index2 varchar2(500);
v_seq1 varchar2(500);
v_tablename user_indexes.index_name%type;
v_index3 VARCHAR2(500);
v_fk1 VARCHAR2(300);
v_fk2 VARCHAR2(300);
v_fk3 VARCHAR2(300);

BEGIN

	BEGIN

		SELECT table_name into v_tablename from user_tables 
				where table_name =  'PR_CERT';
                                  
		EXCEPTION

		WHEN NO_DATA_FOUND THEN
		v_flag := 1;
		dbms_output.put_line(' Table CRE_PR_CERT NOT exists in DB ');
	END;


	IF  v_tablename IS NULL THEN
	
	
	-- Create the table.  It is not found in dictionary.

		v_sql := 'CREATE TABLE  PR_CERT 
			(
	    		CERTID           NUMBER(10)     NOT NULL,
	    		PARTNERID        NUMBER(10)     NOT NULL,
	    		CERTNAME         VARCHAR2(60)       NULL,
	    		DEFFLG           NUMBER(1)      NOT NULL,
	    		CN               VARCHAR2(255)      NULL,
	    		L                VARCHAR2(255)      NULL,
	    		ST               VARCHAR2(255)      NULL,
	    		O                VARCHAR2(255)      NULL,
	    		OU               VARCHAR2(255)      NULL,
	    		C                VARCHAR2(255)      NULL,
	    		VALIDFR          TIMESTAMP(6)       NULL,
	    		VALIDTO          TIMESTAMP(6)       NULL,
	    		CERTVER          VARCHAR2(15)       NULL,
	    		FINGERPRINTALGO  VARCHAR2(30)       NULL,
	    		FINGERPRINT      VARCHAR2(255)      NULL,
	    		SERIALNBR        VARCHAR2(255)      NULL,
	    		PUBLICKEY        VARCHAR2(4000)     NULL,
	    		REVOKEDFLG       NUMBER(1)          NULL,
	    		REVOKEDDATE      TIMESTAMP(6)       NULL,
	    		CERT             BLOB               NULL,
	    		ROWTS            NUMBER(10)     DEFAULT 1 NOT NULL,
	    		CREATEDATE       TIMESTAMP(6)   DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL,
	    		UPDATEDATE       TIMESTAMP(6)   DEFAULT SYS_EXTRACT_UTC(SYSTIMESTAMP)     NULL,
	   		SSLCLIENTCERTFLG NUMBER(1)          NULL,
	    		DIGSIGCERTFLG    NUMBER(1)          NULL,
	    		ENCRYPTCERTFLG   NUMBER(1)          NULL,
	    		ENABLEDFLG       NUMBER(1)      DEFAULT 1     NULL,
	    		ROOTCERTFLG      NUMBER(1)          NULL,
	    		PRISECFLG        NUMBER(1)          DEFAULT 0 NULL,
	    		PASSWORD         RAW(100)           NULL,
	    		PRIVATEKEY       BLOB               NULL,
	    		CERTDESCRIPTION  VARCHAR2(255)      NULL,
	    		FTPFLAG          NUMBER(1)          NULL,
	    		SSLSERVERFLAG    NUMBER(1)          NULL,
	    		ISSUER_CN        VARCHAR2(255)      NULL,
	    		ISSUER_L         VARCHAR2(255)      NULL,
	    		ISSUER_ST        VARCHAR2(255)      NULL,
	    		ISSUER_O         VARCHAR2(255)      NULL,
	    		ISSUER_OU        VARCHAR2(255)      NULL,
	    		ISSUER_C         VARCHAR2(255)      NULL,
	    		DELETEFLAG       NUMBER(1)
			)
			LOB(CERT) STORE AS SYS_LOB0000061493C00020$$
			(
			TABLESPACE TS_PROFILE
			STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
			PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
			ENABLE STORAGE IN ROW
			NOCACHE
			LOGGING
			CHUNK 16384
			PCTVERSION 10
			)
			LOB(PRIVATEKEY) STORE AS SYS_LOB0000076392C00030$$
			(
			TABLESPACE TS_PROFILE
			STORAGE (INITIAL 64K NEXT 16K MINEXTENTS 1 MAXEXTENTS 2147483645
			PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1)
			DISABLE STORAGE IN ROW
			NOCACHE
			LOGGING
			CHUNK 16384
			PCTVERSION 10
			)
			TABLESPACE TS_PROFILE
			LOGGING
			PCTFREE 10
			PCTUSED 40
			INITRANS 1
			MAXTRANS 255
			STORAGE(FREELISTS 1
		        FREELIST GROUPS 1
		        BUFFER_POOL DEFAULT)
			NOPARALLEL
			NOCACHE' ;
	
		 v_index1 := ' CREATE INDEX  AK_CERT_1
	   			ON  PR_CERT(PARTNERID)
				PCTFREE 10
				INITRANS 2
				MAXTRANS 255
				TABLESPACE TS_PROFILE
				STORAGE(FREELISTS 1
	        		FREELIST GROUPS 1
	        		BUFFER_POOL DEFAULT)
				LOGGING
				NOPARALLEL
				NOCOMPRESS
				' ;
		v_index2 := ' ALTER TABLE  PR_CERT 
	    		      ADD CONSTRAINT PK_CERT
			      PRIMARY KEY (CERTID)
			      USING INDEX PCTFREE 10
	            	      INITRANS 2
	            	      MAXTRANS 255
	            	      TABLESPACE TS_PROFILE
	            	      STORAGE(FREELISTS 1
	                      FREELIST GROUPS 1
	                      BUFFER_POOL DEFAULT)
	                      LOGGING ENABLE VALIDATE ' ;
	                      
	        v_index3 := ' ALTER TABLE PR_CERT ADD (
			      CONSTRAINT UK_PRCERT_01 
			      UNIQUE (PARTNERID, CERTID) 
			   )';
			   
		DELETE FROM PR_CERTDESTTYPE;			   
						
		v_fk1 := '  ALTER TABLE PR_CERTDESTTYPE 
			    ADD CONSTRAINT FK_CERTDESTTYPE_2
			    FOREIGN KEY (CERTID)
			    REFERENCES PR_CERT (CERTID) ENABLE ' ;
    
		v_fk2 := ' ALTER TABLE PR_CERT 
		    	  ADD CONSTRAINT FK_CERT_1
		    	  FOREIGN KEY (PARTNERID)
		    	  REFERENCES PR_PARTNER (PARTNERID)
		    	  ENABLE ';
		    	  
		DELETE FROM PR_CERT_SET_LNK;
		    	  
		v_fk3  := 'ALTER TABLE PR_CERT_SET_LNK ADD (
 			   CONSTRAINT FK_CERTID 
 			   FOREIGN KEY (CERTID)
			   REFERENCES PR_CERT (CERTID))	';	    	  
		

		
		EXECUTE IMMEDIATE (v_sql);
		EXECUTE IMMEDIATE (v_index1);
		EXECUTE IMMEDIATE (v_index2);
		EXECUTE IMMEDIATE (v_index3);
		EXECUTE IMMEDIATE (v_fk1);
		EXECUTE IMMEDIATE (v_fk2);
		EXECUTE IMMEDIATE (v_fk3);
		
		
	END IF;
EXCEPTION
	WHEN OTHERS THEN
	RAISE;	

END CRE_PR_CERT;
/

CALL CRE_PR_CERT()
/

































      




































      

-- # ============================================================== # 
-- Create Views 
-- # ============================================================== # 

-- CF_PROTOCOLATTRTYPE_LOC_VW.sql

CREATE OR REPLACE VIEW CF_PROTOCOLATTRTYPE_LOC_VW 
(
    ATTRTYPEID,
    ATTRTYPEKEY,
    CONSTANTFLG,    
    REQUIREDFLG,    
    SEQNBR,         
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    ATTRTYPENAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   t.ATTRTYPEID,
   t.ATTRTYPEKEY,
   t.CONSTANTFLG,    
   t.REQUIREDFLG,    
   t.SEQNBR,         
   t.ROWTS,
   t.CREATEDATE,
   t.UPDATEDATE,
   l.LOCALE,
   l.ATTRTYPENAME,
   l.ROWTS AS LOC_ROWTS,
   l.CREATEDATE AS LOC_CREATEDATE,
   l.UPDATEDATE AS LOC_UPDATEDATE
FROM
    CF_PROTOCOLATTRTYPE t
    INNER JOIN CF_PROTOCOLATTRTYPE_Loc l ON l.ATTRTYPEID = t.ATTRTYPEID
/


-- CF_PROTOCOLATTRVALUETYPELOC_VW.sql

CREATE OR REPLACE VIEW CF_ProtocolAttrValueTypeLoc_VW 
(
    ATTRVALUETYPEID,        
    ATTRVALUETYPEKEY,       
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,                 
    ATTRVALUETYPENAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   t.ATTRVALUETYPEID,        
   t.ATTRVALUETYPEKEY,       
   t.ROWTS,
   t.CREATEDATE,
   t.UPDATEDATE,
   l.LOCALE,
   l.ATTRVALUETYPENAME,
   l.ROWTS AS LOC_ROWTS,
   l.CREATEDATE AS LOC_CREATEDATE,
   l.UPDATEDATE AS LOC_UPDATEDATE
FROM
    CF_ProtocolAttrValueType t
    INNER JOIN CF_ProtocolAttrValueType_Loc l ON l.ATTRVALUETYPEID = t.ATTRVALUETYPEID
/


-- CF_PROTOCOLDEFTYPE_LOC_VW.sql

CREATE OR REPLACE VIEW CF_PROTOCOLDEFTYPE_LOC_VW 
(
    METHODTYPEID,
    METHODTYPEKEY,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    METHODTYPENAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   t.METHODTYPEID,
   t.METHODTYPEKEY,
   t.ROWTS,
   t.CREATEDATE,
   t.UPDATEDATE,
   l.LOCALE,
   l.METHODTYPENAME,
   l.ROWTS AS LOC_ROWTS,
   l.CREATEDATE AS LOC_CREATEDATE,
   l.UPDATEDATE AS LOC_UPDATEDATE
FROM
    CF_PROTOCOLDEFTYPE t
    INNER JOIN CF_PROTOCOLDEFTYPE_LOC l ON l.METHODTYPEID = t.METHODTYPEID
/


-- CF_RNPIPMETADEF_LOC_VW.sql

CREATE OR REPLACE VIEW CF_RNPIPMETADEF_LOC_VW 
(
    METADEFID,
    METADEFKEY,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    METADEFNAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   t.MetaDefID,
   t.MetaDefKEY,
   t.ROWTS,
   t.CREATEDATE,
   t.UPDATEDATE,
   l.LOCALE,
   l.MetaDefNAME,
   l.ROWTS AS LOC_ROWTS,
   l.CREATEDATE AS LOC_CREATEDATE,
   l.UPDATEDATE AS LOC_UPDATEDATE
FROM
    CF_RNPIPMetaDef t
    INNER JOIN CF_RNPIPMetaDef_Loc l ON l.MetaDefID = t.MetaDefID
/


-- CF_ROUTINGATTROPT_LOC_VW.sql

CREATE OR REPLACE VIEW CF_ROUTINGATTROPT_LOC_VW 
(
    ATTROPTID,
    ROUTINGATTRID,
    ATTRVALUEKEY,
    ATTRSEQNBR,
    DEFAULTFLG,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    ATTRVALUE,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   a.ATTROPTID,
   a.ROUTINGATTRID,
   a.ATTRVALUEKEY,
   a.ATTRSEQNBR,
   a.DEFAULTFLG,
   a.ROWTS,
   a.CREATEDATE,
   a.UPDATEDATE,
   l.LOCALE,
   l.ATTRVALUE,
   l.ROWTS AS LOC_ROWTS,
   l.CREATEDATE AS LOC_CREATEDATE,
   l.UPDATEDATE AS LOC_UPDATEDATE
FROM
    CF_ROUTINGATTROPT a
    INNER JOIN CF_ROUTINGATTROPT_LOC l ON l.ATTROPTID = a.ATTROPTID
/


-- CF_ROUTINGATTR_LOC_VW.sql


CREATE OR REPLACE VIEW CF_ROUTINGATTR_LOC_VW 
(
    ROUTINGATTRID,
    ATTRIBUTEKEY,
    REQUIREDFLG,
    ATTRDEFLEVEL,
    ATTRSEQNBR,
    VISIBILITYCD,
    ROWTS,
    LOCALE,
    ATTRNAME,
    ATTRDESC,
    VALIDATIONRULE,
    LOC_ROWTS
)
AS
SELECT
   a.RoutingAttrId
   ,a.AttributeKey
   ,a.RequiredFlg
   ,a.AttrDefLevel
   ,a.AttrSeqNbr
   ,a.VisibilityCd
   ,a.RowTS
   ,l.LOCALE
   ,l.AttrName
   ,l.AttrDesc
   ,l.ValidationRule
   ,l.ROWTS AS LOC_ROWTS
FROM
    CF_RoutingAttr a
    INNER JOIN CF_RoutingAttr_Loc l ON l.RoutingAttrId = a.RoutingAttrId
/


-- CF_ROUTINGOBJECT05_VW.sql

CREATE OR REPLACE VIEW CF_ROUTINGOBJECT05_VW 
(
    ROUTINGOBJLEVEL,
    ROUTINGOBJTYPEKEY,
    ROUTINGOBJTYPEID,
    ROUTINGOBJID,
    ROUTINGOBJCD,
    ROUTINGOBJVER,
    ROUTINGOBJREFID,
    ROUTINGOBJPARENTREFID,
    GRPARENTOBJREFID,
    PARENTOBJLEVEL,
    PARENTOBJTYPEKEY,
    PARENTOBJID,
    PARENTOBJCD,
    PARENTOBJVER
)
AS
select /*+ FIRST_ROWS */
    D.RoutingObjLevel,
    D.RoutingObjTypeKey,
    R.RoutingObjTypeId,
    R.RoutingObjId,
    R.ROUTINGOBJCD,
    R.ROUTINGOBJVER    ,
    F.RoutingObjRefId,
    F.RoutingObjParentRefId,
    F2.RoutingObjParentRefId GrParentObjRefId,
    D2.RoutingObjLevel ParentObjLevel,
    D2.RoutingObjTypeKey ParentObjTypeKey,
    R2.RoutingObjId ParentObjId,
    R2.RoutingObjCd ParentObjCd,
    R2.RoutingObjVer ParentObjVer
from
    CF_RoutingObject R
    INNER JOIN CF_RoutingObjRef_Lnk F on F.RoutingObjId = R.RoutingObjId
    INNER JOIN CF_RoutingObjType D on D.RoutingObjTypeId = R.RoutingObjTypeId
    LEFT OUTER JOIN CF_RoutingObjRef_Lnk F2 on F2.RoutingObjRefId = F.RoutingObjParentRefId
    LEFT OUTER  JOIN CF_RoutingObject R2 on R2.RoutingObjId = F2.RoutingObjId
    LEFT OUTER  JOIN CF_RoutingObjType D2 on D2.RoutingObjTypeId = R2.RoutingObjTypeId
/


-- CF_ROUTINGOBJTYPE_LOC_VW.sql

CREATE OR REPLACE VIEW CF_ROUTINGOBJTYPE_LOC_VW 
(
    ROUTINGOBJTYPEID,
    ROUTINGOBJLEVEL,
    ROUTINGOBJSUBLEVEL,
    ROUTINGOBJATTRRANK,
    ROUTINGOBJTYPEKEY,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    ROUTINGOBJTYPECD,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   r.ROUTINGOBJTYPEID,
   r.ROUTINGOBJLEVEL,
   r.ROUTINGOBJSUBLEVEL,
   r.ROUTINGOBJATTRRANK,
   r.ROUTINGOBJTYPEKEY,
   r.ROWTS,
   r.CREATEDATE,
   r.UPDATEDATE,
   l.LOCALE,
   l.ROUTINGOBJTYPECD,
   l.ROWTS AS LOC_ROWTS,
   l.CREATEDATE AS LOC_CREATEDATE,
   l.UPDATEDATE AS LOC_UPDATEDATE
FROM
    CF_ROUTINGOBJTYPE r
    INNER JOIN CF_ROUTINGOBJTYPE_LOC l ON l.ROUTINGOBJTYPEID = r.ROUTINGOBJTYPEID
/


-- CF_STRUCTURETYPE_LOC_VW.sql

CREATE OR REPLACE VIEW CF_STRUCTURETYPE_LOC_VW 
(
    STRUCTURETYPEID,
    STRUCTURETYPEKEY,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    STRUCTURETYPENAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   t.STRUCTURETYPEID,
   t.STRUCTURETYPEKEY,
   t.ROWTS,
   t.CREATEDATE,
   t.UPDATEDATE,
   l.LOCALE,
   l.STRUCTURETYPENAME,
   l.ROWTS AS LOC_ROWTS,
   l.CREATEDATE AS LOC_CREATEDATE,
   l.UPDATEDATE AS LOC_UPDATEDATE
FROM
    CF_STRUCTURETYPE t
    INNER JOIN CF_STRUCTURETYPE_Loc l ON l.STRUCTURETYPEID = t.STRUCTURETYPEID
/


-- LG_ACTIVITY_DTL_CHILD_VW.sql

CREATE OR REPLACE VIEW LG_ACTIVITY_DTL_CHILD_VW
(ACTIVITYID, VUID, HDRFLG, MSGRAWLENGTH, MSGCONTENTLENGTH, 
 SYNCHRONOUSFLG, ORIGINALFILENAME, ENCODING,MessageStoreRequired)
AS 
SELECT ACTIVITYID,
 VUID,    HDRFLG,    MSGRAWLENGTH,    MSGCONTENTLENGTH,    SYNCHRONOUSFLG , ORIGINALFILENAME, ENCODING
 ,MessageStoreRequired
FROM LG_ACTIVITY_DTL WHERE HDRFLG = 0
/

-- LG_ACTIVITY_DTL_PARENT_VW.sql

CREATE OR REPLACE VIEW LG_ACTIVITY_DTL_PARENT_VW
(ACTIVITYID, VUID, HDRFLG, MSGRAWLENGTH, MSGCONTENTLENGTH, 
 SYNCHRONOUSFLG, ORIGINALFILENAME, ENCODING,MESSAGESTOREREQUIRED)
AS 
SELECT ACTIVITYID,
  VUID,    HDRFLG,    MSGRAWLENGTH,    MSGCONTENTLENGTH,
SYNCHRONOUSFLG, ORIGINALFILENAME, ENCODING,MESSAGESTOREREQUIRED FROM LG_ACTIVITY_DTL WHERE HDRFLG = 1
/

-- LG_SUMMARY_XREF_VW.sql

create or replace view LG_SUMMARY_XREF_VW
as select
 Lnk.SUMMARYID              
, Lnk.PROVSESSIONID
--
, part.PARTNER_SUMMARYID      
, part.INITPARTNERID          
, part.INITBUSINESSID         
, part.FRPARTNERID            
, part.FRBUSINESSID           
, part.TOPARTNERID            
, part.TOBUSINESSID           
, part.FRTRANSPORTTYPE        
, part.TOTRANSPORTTYPE        
, part.FRIPADDRESS            
, part.DESTTYPEID             
--
, prcs.PROCESS_SUMMARYID      
, prcs.FRPACKAGENAME          
, prcs.FRPACKAGEVER           
, prcs.TOPACKAGENAME          
, prcs.TOPACKAGEVER           
, prcs.FRPROTOCOLNAME         
, prcs.FRPROTOCOLVER          
, prcs.TOPROTOCOLNAME         
, prcs.TOPROTOCOLVER          
, prcs.FRPROCESSCD            
, prcs.FRPROCESSVER           
, prcs.TOPROCESSCD            
, prcs.TOPROCESSVER           
, prcs.ACTIVITYTYPECD 
, prcs.ActionFlg              
From LG_sum_xref_lnk Lnk
Inner join LG_sum_xref_part part on part.partner_summaryID = LNK.partner_summaryID
Inner join LG_sum_xref_prcs prcs on prcs.Process_summaryID = LNK.Process_summaryID
/

-- PR_ADDRESSTYPE_LOC_VW.sql

CREATE OR REPLACE VIEW PR_ADDRESSTYPE_LOC_VW 
(
    ADDRESSTYPEID,
    ADDRESSTYPENAME,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    ADDRESSTYPEDISPLAYNAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   a.ADDRESSTYPEID
   ,a.ADDRESSTYPENAME
   ,a.ROWTS
   ,a.CREATEDATE
   ,a.UPDATEDATE
   ,l.LOCALE
   ,l.ADDRESSTYPEDISPLAYNAME
   ,l.ROWTS        AS LOC_ROWTS
   ,l.CREATEDATE   AS LOC_CREATEDATE
   ,l.UPDATEDATE   AS LOC_UPDATEDATE
FROM PR_ADDRESSTYPE a
INNER JOIN  PR_ADDRESSTYPE_LOC l ON a.ADDRESSTYPEID = l.ADDRESSTYPEID
/


-- PR_BUSINESSIDTYPE_LOC_VW.sql

CREATE OR REPLACE VIEW PR_BUSINESSIDTYPE_LOC_VW 
(
    BUSINESSIDTYPE,
    BUSINESSIDNAME,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    VALIDATIONRULE,
    LOCALE,
    BUSINESSIDDISPLAYNAME,
    BUSINESSIDDESC,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   b.BUSINESSIDTYPE
   ,b.BUSINESSIDNAME
   ,b.ROWTS
   ,b.CREATEDATE
   ,b.UPDATEDATE
   ,b.VALIDATIONRULE
   ,l.LOCALE
   ,l.BUSINESSIDDISPLAYNAME
   ,l.BUSINESSIDDESC
   ,l.ROWTS         AS LOC_ROWTS
   ,l.CREATEDATE    AS LOC_CREATEDATE
   ,l.UPDATEDATE    AS LOC_UPDATEDATE
FROM PR_BUSINESSIDTYPE b
INNER JOIN PR_BUSINESSIDTYPE_LOC l ON b.BUSINESSIDTYPE = l.BUSINESSIDTYPE
/


-- PR_CLASSIFICATION_LOC_VW.sql

CREATE OR REPLACE VIEW PR_CLASSIFICATION_LOC_VW 
(
    CLASSIFICATIONID,
    CLASSIFICATIONNAME,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    CLASSIFICATIONDISPLAYNAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   c.CLASSIFICATIONID
   ,c.CLASSIFICATIONNAME
   ,c.ROWTS
   ,c.CREATEDATE
   ,c.UPDATEDATE
   ,l.LOCALE
   ,l.CLASSIFICATIONDISPLAYNAME
   ,l.ROWTS        AS LOC_ROWTS
   ,l.CREATEDATE   AS LOC_CREATEDATE
   ,l.UPDATEDATE   AS LOC_UPDATEDATE
FROM PR_CLASSIFICATION c
INNER JOIN PR_CLASSIFICATION_LOC l ON  c.CLASSIFICATIONID = l.CLASSIFICATIONID
/


-- PR_CONTACTTYPE_LOC_VW.sql

CREATE OR REPLACE VIEW PR_CONTACTTYPE_LOC_VW 
(
    CONTACTTYPEID,
    CONTACTTYPENAME,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    CONTACTTYPEDISPLAYNAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   c.CONTACTTYPEID
   ,c.CONTACTTYPENAME
   ,c.ROWTS
   ,c.CREATEDATE
   ,c.UPDATEDATE
   ,l.LOCALE
   ,l.CONTACTTYPEDISPLAYNAME
   ,l.ROWTS            AS LOC_ROWTS
   ,l.CREATEDATE       AS LOC_CREATEDATE
   ,l.UPDATEDATE       AS LOC_UPDATEDATE
From pr_contactType c
Inner Join pr_contactType_loc l on c.CONTACTTYPEID = l.CONTACTTYPEID
/


-- PR_DESTTYPE_LOC_VW.sql

CREATE OR REPLACE VIEW PR_DESTTYPE_LOC_VW 
(
    DESTTYPEID,
    DESTNAME,
    DEFAULTFLG,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    DIRFLG,
    LOCALE,
    DESTDISPLAYNAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   d.DESTTYPEID
   ,d.DESTNAME
   ,d.DEFAULTFLG
   ,d.ROWTS
   ,d.CREATEDATE
   ,d.UPDATEDATE
   ,d.DIRFLG
   ,l.LOCALE
   ,l.DESTDISPLAYNAME
   ,l.ROWTS          AS LOC_ROWTS
   ,l.CREATEDATE     AS LOC_CREATEDATE
   ,l.UPDATEDATE     AS LOC_UPDATEDATE
FROM PR_DESTTYPE d
INNER JOIN PR_DESTTYPE_LOC l ON d.DESTTYPEID = l.DESTTYPEID
/


-- PR_PARTNER01_VW.sql

CREATE OR REPLACE VIEW PR_PARTNER01_VW 
(
    PARTNERID,
    BUSINESSID,
    ENABLED,
    PARTNERTYPENAME,
    OWNERFLG,
    PARTNERNAME,
    GLOBALPARTNERID,
    PARENTPARTNERID,
    OLSID,
    LOCALE,
    URI,
    CLASSIFICATIONNAME
)
AS
select  /*+ FIRST_ROWS */
    P.PartnerId,
    B.BusinessId,
    P.Enabled,
    T.PartnerTypeName,
    T.OwnerFlg,
    P.PartnerName,
    P.GlobalPartnerId,
    P.ParentPartnerId,
    P.OLSID,
    P.Locale,
    P.URI,
    C.ClassificationName
From
    PR_Partner P
    LEFT JOIN PR_BusIdentifier B on B.PartnerId = P.PartnerId
    LEFT JOIN PR_PartnerType T on T.PartnerTypeId = P.PartnerTypeId
    LEFT JOIN PR_Classification C on C.ClassificationId = P.ClassificationId
/


-- PR_PARTNER02_VW.sql

CREATE OR REPLACE VIEW PR_PARTNER02_VW 
(
    PARTNERID,
    ENABLED,
    PARTNERTYPENAME,
    OWNERFLG,
    PARTNERNAME,
    GLOBALPARTNERID,
    PARENTPARTNERID,
    OLSID,
    LOCALE,
    URI,
    CLASSIFICATIONNAME
)
AS
select  /*+ FIRST_ROWS */
    P.PartnerId,
    P.Enabled,
    T.PartnerTypeName,
    T.OwnerFlg,
    P.PartnerName,
    P.GlobalPartnerId,
    P.ParentPartnerId,
    P.OLSID,
    P.Locale,
    P.URI,
    C.ClassificationName
From
    PR_Partner P
    LEFT JOIN PR_PartnerType T on T.PartnerTypeId = P.PartnerTypeId
    LEFT JOIN PR_Classification C on C.ClassificationId = P.ClassificationId
/


-- PR_PARTNERTYPE_LOC_VW.sql

CREATE OR REPLACE VIEW PR_PARTNERTYPE_LOC_VW 
(
    PARTNERTYPEID,
    PARTNERTYPENAME,
    OPERATORFLG,
    OWNERFLG,
    TYPEFLG,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    VISIBILITY,
    LOCALE,
    PARTNERTYPEDISPLAYNAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
    p.PARTNERTYPEID
   ,p.PARTNERTYPENAME
   ,p.OPERATORFLG
   ,p.OWNERFLG
   ,p.TYPEFLG
   ,p.ROWTS
   ,p.CREATEDATE
   ,p.UPDATEDATE
   ,p.VISIBILITY
   ,l.LOCALE
   ,l.PARTNERTYPEDISPLAYNAME
   ,l.ROWTS       AS LOC_ROWTS
   ,l.CREATEDATE  AS LOC_CREATEDATE
   ,l.UPDATEDATE  AS LOC_UPDATEDATE
From PR_PARTNERTYPE p
Inner Join  PR_PARTNERTYPE_Loc l on  p.PARTNERTYPEID = l.PARTNERTYPEID
/


-- PR_PERMISSION_LOC_VW.sql

CREATE OR REPLACE VIEW PR_PERMISSION_LOC_VW 
(
    PERMISSIONID,
    PERMISSIONNAME,
    ENABLEDFLG,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    LOCALE,
    PERMISSIONDESC,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   p.PERMISSIONID
   ,p.PERMISSIONNAME
   ,p.ENABLEDFLG
   ,p.ROWTS
   ,p.CREATEDATE
   ,p.UPDATEDATE
   ,pl.LOCALE
   ,pl.PERMISSIONDESC
   ,pl.ROWTS      AS Loc_RowTS
   ,pl.CREATEDATE AS Loc_CREATEDATE
   ,pl.UPDATEDATE AS Loc_UPDATEDATE
FROM PR_PERMISSION p
INNER JOIN PR_PERMISSION_LOC pl ON p.PERMISSIONID = pl.PERMISSIONID
/


-- PR_TRANSPORT_LOC_VW.sql

CREATE OR REPLACE VIEW PR_TRANSPORT_LOC_VW 
AS
SELECT
   t.TRANSPORTID
   ,t.TRANSPORTTYPE
   ,t.ROWTS
   ,t.CREATEDATE
   ,t.UPDATEDATE
   ,t.VISIBILITY
   ,t.UI_VISIBILITY
   ,l.LOCALE
   ,l.TRANSPORTTYPEDISPLAY
   ,l.ROWTS        AS LOC_ROWTS
   ,l.CREATEDATE   AS LOC_CREATEDATE
   ,l.UPDATEDATE   AS LOC_UPDATEDATE
FROM  PR_TRANSPORT t
Inner Join PR_TRANSPORT_LOC l on  t.TRANSPORTID = l.TRANSPORTID
/


-- PR_WORKFLOW_LOC_VW.sql

CREATE OR REPLACE VIEW PR_WORKFLOW_LOC_VW 
(
    WORKFLOWID,
    WORKFLOWNAME,
    WORKFLOWDEF,
    ENABLEDFLG,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    WORKFLOWTYPEID,
    LOCALE,
    WORKFLOWDISPLAYNAME,
    LOC_ROWTS,
    LOC_CREATEDATE,
    LOC_UPDATEDATE
)
AS
SELECT
   w.WORKFLOWID
   ,w.WORKFLOWNAME
   ,w.WORKFLOWDEF
   ,w.ENABLEDFLG
   ,w.ROWTS
   ,w.CREATEDATE
   ,w.UPDATEDATE
   ,w.WORKFLOWTYPEID
   ,l.LOCALE
   ,l.WORKFLOWDISPLAYNAME
   ,l.ROWTS AS LOC_ROWTS
   ,l.CREATEDATE AS LOC_CREATEDATE
   ,l.UPDATEDATE AS LOC_UPDATEDATE
FROM
    PR_WORKFLOW w
    INNER JOIN Pr_WorkFlow_Loc l ON l.WorkFlowId = w.WorkFlowId
/


-- CF_ROUTINGOBJECT01_VW.sql

CREATE OR REPLACE VIEW CF_ROUTINGOBJECT01_VW 
(
    ROUTINGOBJLEVEL,
    ROUTINGOBJTYPEKEY,
    ROUTINGOBJID,
    ROUTINGOBJTYPEID,
    ROUTINGOBJNAME,
    ROUTINGOBJCD,
    ROUTINGOBJVER,
    CHORGSEQNBR,
    ROUTINGOBJDESC,
    ROUTINGOBJATTRRANK
)
AS
select /*+ FIRST_ROWS */
    D.RoutingObjLevel,
    D.RoutingObjTypeKey,
    R.ROUTINGOBJID    ,
    R.ROUTINGOBJTYPEID    ,
    R.ROUTINGOBJNAME    ,
    R.ROUTINGOBJCD,
    R.ROUTINGOBJVER    ,
    R.CHORGSEQNBR,
    R.ROUTINGOBJDESC,
    D.RoutingObjAttrRank
from
    CF_RoutingObject R
    INNER JOIN CF_RoutingObjType D on D.RoutingObjTypeId = R.RoutingObjTypeId
/


-- CF_ROUTINGOBJECT02_VW.sql

CREATE OR REPLACE VIEW CF_ROUTINGOBJECT02_VW 
(
    ROUTINGOBJLEVEL,
    ROUTINGOBJTYPEKEY,
    ROUTINGOBJTYPEID,
    ROUTINGOBJID,
    ROUTINGOBJCD,
    ROUTINGOBJVER,
    ROUTINGOBJREFID,
    ROUTINGOBJPARENTREFID,
    GRPARENTOBJREFID,
    PARENTOBJLEVEL,
    PARENTOBJTYPEKEY,
    PARENTOBJID,
    PARENTOBJCD,
    PARENTOBJVER
)
AS
select /*+ FIRST_ROWS */
    D.RoutingObjLevel,
    D.RoutingObjTypeKey,
    R.RoutingObjTypeId,
    R.RoutingObjId,
    R.ROUTINGOBJCD,
    R.ROUTINGOBJVER    ,
    F.RoutingObjRefId,
    F.RoutingObjParentRefId,
    F2.RoutingObjParentRefId GrParentObjRefId,
    D2.RoutingObjLevel ParentObjLevel,
    D2.RoutingObjTypeKey ParentObjTypeKey,
    R2.RoutingObjId ParentObjId,
    R2.RoutingObjCd ParentObjCd,
    R2.RoutingObjVer ParentObjVer
from
    CF_RoutingObject R
    INNER JOIN CF_RoutingObjRef_Lnk F on F.RoutingObjId = R.RoutingObjId
    INNER JOIN CF_RoutingObjType D on D.RoutingObjTypeId = R.RoutingObjTypeId
    INNER JOIN CF_RoutingObjRef_Lnk F2 on F2.RoutingObjRefId = F.RoutingObjParentRefId
    INNER JOIN CF_RoutingObject R2 on R2.RoutingObjId = F2.RoutingObjId
    INNER JOIN CF_RoutingObjType D2 on D2.RoutingObjTypeId = R2.RoutingObjTypeId
/


-- CF_ROUTINGOBJECT03_VW.sql

CREATE OR REPLACE VIEW CF_ROUTINGOBJECT03_VW 
(
    ROUTINGOBJLEVEL,
    ROUTINGOBJTYPEKEY,
    ROUTINGOBJTYPEID,
    ROUTINGOBJID,
    ROUTINGOBJCD,
    ROUTINGOBJVER,
    ROUTINGOBJREFID,
    ROUTINGOBJPARENTREFID,
    GRPARENTOBJREFID,
    PARENTOBJLEVEL,
    PARENTOBJTYPEKEY,
    PARENTOBJID,
    PARENTOBJCD,
    PARENTOBJVER,
    GRROUTINGOBJREFID,
    GGPARENTOBJREFID,
    GRPARENTOBJLEVEL,
    GRPARENTOBJTYPEKEY,
    GRPARENTOBJID,
    GRPARENTOBJCD,
    GRPARENTOBJVER,
    GRCHORGSEQNBR,
    GRROUTINGOBJDESC
)
AS
select /*+ FIRST_ROWS */
    R.ROUTINGOBJLEVEL,
    R.ROUTINGOBJTYPEKEY,
    R.ROUTINGOBJTYPEID,
    R.ROUTINGOBJID,
    R.ROUTINGOBJCD,
    R.ROUTINGOBJVER,
    R.ROUTINGOBJREFID,
    R.ROUTINGOBJPARENTREFID,
    R.GRPARENTOBJREFID,
    R.PARENTOBJLEVEL,
    R.PARENTOBJTYPEKey,
    R.PARENTOBJID,
    R.PARENTOBJCD,
    R.PARENTOBJVER,
    F.RoutingObjRefId    GrRoutingObjRefId,
    F.RoutingObjParentRefId  GgParentObjRefId,
    D2.RoutingObjLevel   GrParentObjLevel,
    D2.RoutingObjTypeKey GrParentObjTypeKey,
    R2.RoutingObjId      GrParentObjId,
    R2.RoutingObjCd      GrParentObjCd,
    R2.RoutingObjVer     GrParentObjVer,
    R2.CHORGSEQNBR      GrChorgSeqNbr,
    R2.ROUTINGOBJDESC   GrRoutingObjDesc
from
    CF_RoutingObject02_vw R
    INNER JOIN CF_RoutingObjRef_Lnk F on F.RoutingObjRefId = R.GrParentObjRefId
    INNER JOIN CF_RoutingObject R2 on R2.RoutingObjId = F.RoutingObjId
    INNER JOIN CF_RoutingObjType D2 on D2.RoutingObjTypeId = R2.RoutingObjTypeId
/


-- CF_ROUTINGOBJECT04_VW.sql

CREATE OR REPLACE VIEW CF_ROUTINGOBJECT04_VW 
(
    ROUTINGOBJTYPEID,
    ROUTINGOBJLEVEL,
    ROUTINGOBJTYPEKEY,
    ROUTINGOBJID,
    ROUTINGOBJNAME,
    ROUTINGOBJCD,
    ROUTINGOBJVER,
    CHORGSEQNBR,
    ROUTINGOBJDESC,
    ACTIVESTATUSCD,
    ROUTINGOBJATTRRANK,
    ROUTINGOBJREFID,
    ROUTINGOBJPARENTREFID,
    DocumentFlg
)
AS
select /*+ FIRST_ROWS */
    D.RoutingObjTypeId, D.RoutingObjLevel,
    D.RoutingObjTypeKey,
    R.ROUTINGOBJID    ,
    R.ROUTINGOBJNAME    ,
    R.ROUTINGOBJCD,
    R.ROUTINGOBJVER    ,
    R.CHORGSEQNBR,
    R.ROUTINGOBJDESC,
    R.ACTIVESTATUSCD,
    D.RoutingObjAttrRank,
    L.RoutingObjRefId,
    L.RoutingObjParentRefId,
    R.DocumentFlg
from
    CF_RoutingObject R
    INNER JOIN CF_RoutingObjType D on D.RoutingObjTypeId = R.RoutingObjTypeId
    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
/


-- CF_VALIDOBJINTERACT01_VW.sql

CREATE OR REPLACE VIEW CF_VALIDOBJINTERACT01_VW 
(
    FRROUTINGOBJTYPEKEY,
    FRROUTINGOBJCD,
    FRROUTINGOBJVER,
    FRROUTINGOBJREFID,
    TOROUTINGOBJTYPEKEY,
    TOROUTINGOBJCD,
    TOROUTINGOBJVER,
    TOROUTINGOBJREFID
)
AS
select
    Rf.RoutingObjTypeKey FrRoutingObjTypeKey,
    Rf.RoutingObjCd FrRoutingObjCd,
    Rf.RoutingObjVer FrRoutingObjVer,
    Rf.RoutingObjRefId FrRoutingObjRefId,
    Rt.RoutingObjTypeKey ToRoutingObjTypeKey,
    Rt.RoutingObjCd ToRoutingObjCd,
    Rt.RoutingObjVer ToRoutingObjVer,
    Rt.RoutingObjRefId ToRoutingObjRefId
from
    CF_ValidObjInteract V
    INNER JOIN CF_RoutingObject04_vw Rf on Rf.RoutingObjRefId = V.FrRoutingObjRefId
    INNER JOIN CF_RoutingObject04_vw Rt on Rt.RoutingObjRefId = V.ToRoutingObjRefId
/


-- PR_CAPABILITY01_VW.sql

CREATE OR REPLACE VIEW PR_CAPABILITY01_VW 
(
    PARTNERID,
    CAPABILITYID,
    ROUTINGOBJREFID,
    ROUTINGOBJTYPEKEY,
    ROUTINGOBJPARENTREFID,
    ROUTINGOBJID,
    ROUTINGOBJCD,
    ROUTINGOBJVER,
    ActiveStatusCd
)
AS
select /*+ FIRST_ROWS */
    C.PartnerId,
    C.CapabilityId,
    C.RoutingObjRefId,
    R.RoutingObjTypeKey ,
    L.RoutingObjParentRefId,
    R.RoutingObjId,
    R.RoutingObjCd,
    R.RoutingObjVer,
    CASE WHEN C.ACTIVESTATUSCD=3 THEN 1 ELSE 0 END ActiveStatusCd
from
    PR_Capability C
    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjRefId = C.RoutingObjRefId
    INNER JOIN CF_RoutingObject01_vw R on R.RoutingObjId = L.RoutingObjId
/


-- PR_CAPABILITY02_VW.sql

CREATE OR REPLACE VIEW PR_CAPABILITY02_VW 
(
    PARTNERID,
    CAPABILITYID,
    ROUTINGOBJREFID,
    ROUTINGOBJTYPEKEY,
    ROUTINGOBJPARENTREFID,
    ROUTINGOBJID,
    ROUTINGOBJCD,
    ROUTINGOBJVER,
    PARENTCAPABILITYID,
    PARENTOBJTYPEKEY,
    GRPARENTOBJREFID,
    PARENTOBJID,
    PARENTOBJCD,
    PARENTOBJVER,
    ActiveStatusCd
)
AS
select /*+ FIRST_ROWS */
    C.PARTNERID,
    C.CAPABILITYID,
    C.ROUTINGOBJREFID,
    C.ROUTINGOBJTYPEKEY,
    C.ROUTINGOBJPARENTREFID,
    C.ROUTINGOBJID,
    C.ROUTINGOBJCD,
    C.ROUTINGOBJVER,
    cp.CAPABILITYID ParentCAPABILITYID,
    cp.ROUTINGOBJTYPEKEY  ParentOBJTYPEKEY,
    cp.ROUTINGOBJPARENTREFID GrParentOBJREFID,
    cp.ROUTINGOBJID ParentOBJID,
    cp.ROUTINGOBJCD ParentOBJCD,
    cp.ROUTINGOBJVER ParentOBJVER,
    cp.ActiveStatusCd
from
    PR_Capability01_vw C
    INNER JOIN PR_Capability01_vw Cp on Cp.RoutingObjRefId = C.RoutingObjParentRefId and Cp.PartnerId = C.PartnerId
/


-- PR_CAPABILITY03_VW.sql

CREATE OR REPLACE VIEW PR_CAPABILITY03_VW 
(
    PARTNERID,
    CAPABILITYID,
    ROUTINGOBJLEVEL,
    ROUTINGOBJTYPEKEY,
    ROUTINGOBJTYPEID,
    ROUTINGOBJID,
    ROUTINGOBJCD,
    ROUTINGOBJVER,
    ROUTINGOBJREFID,
    ROUTINGOBJPARENTREFID,
    GRPARENTOBJREFID,
    PARENTOBJLEVEL,
    PARENTOBJTYPEKEY,
    PARENTOBJID,
    PARENTOBJCD,
    PARENTOBJVER,
    GRROUTINGOBJREFID,
    GGPARENTOBJREFID,
    GRPARENTOBJLEVEL,
    GRPARENTOBJTYPEKEY,
    GRPARENTOBJID,
    GRPARENTOBJCD,
    GRPARENTOBJVER,
    GRCHORGSEQNBR,
    GRROUTINGOBJDESC
)
AS
select /*+ FIRST_ROWS */
    C.PARTNERID,
    C.CAPABILITYID,
    R.ROUTINGOBJLEVEL,
    R.ROUTINGOBJTYPEKEY,
    R.ROUTINGOBJTYPEID,
    R.ROUTINGOBJID,
    R.ROUTINGOBJCD,
    R.ROUTINGOBJVER,
    R.ROUTINGOBJREFID,
    R.ROUTINGOBJPARENTREFID,
    R.GRPARENTOBJREFID,
    R.PARENTOBJLEVEL,
    R.PARENTOBJTYPEKEY,
    R.PARENTOBJID,
    R.PARENTOBJCD,
    R.PARENTOBJVER,
    R.GRROUTINGOBJREFID,
    R.GGPARENTOBJREFID,
    R.GRPARENTOBJLEVEL,
    R.GRPARENTOBJTYPEKEY,
    R.GRPARENTOBJID,
    R.GRPARENTOBJCD,
    R.GRPARENTOBJVER,
    R.GRCHORGSEQNBR,
    R.GRROUTINGOBJDESC
from
    PR_Capability C
    INNER JOIN CF_RoutingObject03_vw R on R.RoutingObjRefId = C.RoutingObjRefId
/


-- PR_ROUTINGCHANNEL01_VW.sql

CREATE OR REPLACE VIEW PR_ROUTINGCHANNEL01_VW 
(
    CHANNELDIRECTION,
    CHANNELID,
    FRPARTNERID,
    TOPARTNERID,
    FRCAPABILITYID,
    TOCAPABILITYID,
    FRROUTINGOBJREFID,
    TOROUTINGOBJREFID,
    FRROUTINGOBJID,
    TOROUTINGOBJID,
    FRROUTINGOBJCD,
    FRROUTINGOBJVER,
    TOROUTINGOBJCD,
    TOROUTINGOBJVER,
    ENABLED,
    FRGATEWAYID,
    TOGATEWAYID,
    FRGATEWAYNAME,
    TOGATEWAYNAME,
    FRGATEWAYENABLED,
    TOGATEWAYENABLED,
    FRURI,
    FRURILOGIN,
    FRRUIPASSWORD,
    TOURI,
    TOURILOGIN,
    TOURIPASSWORD,
    WORKFLOWID,
    WORKFLOWNAME,
    WORKFLOWDEF,
    TRANSPORTTYPE,
    FRVALIDATECLIENTSSLCERT,
    FRVALIDATECLIENTIP,
    DESTTYPEID,
    TRANSFORMMAPID,
    PROFILEID
)
AS
select  /*+ FIRST_ROWS */
    'FR' ChannelDirection,
    C.ChannelId,
    C.FrPartnerId,
    C.ToPartnerId,
    Cfr.CapabilityId FrCapabilityId,
    Cto.CapabilityId ToCapabilityId,
    Cfr.RoutingObjRefId FrRoutingObjRefId,
    Cto.RoutingObjRefId ToRoutingObjRefId,
    Cfr.RoutingObjId FrRoutingObjId,
    Cto.RoutingObjId ToRoutingObjId,
    Cfr.RoutingObjCd FrRoutingObjCd,
    Cfr.RoutingObjVer FrRoutingObjVer,
    Cto.RoutingObjCd ToRoutingObjCd,
    Cto.RoutingObjVer ToRoutingObjVer,
    CASE WHEN C.ActiveStatusCd=63 Then 1 ELSE 0 END Enabled,
    FrGatewayId, ToGatewayId,
    Fg.GatewayName FrGatewayName,
    Tg.GatewayName ToGatewayName,
    Fg.EnabledFlg FrGatewayEnabled,
    Tg.EnabledFlg ToGatewayEnabled,
    Fg.URI FrURI,
    Fg.URILogin FrURILogin,
    Fg.URIPassword FrRUIPassword,
    Tg.URI ToURI,
    Tg.URILogin ToURILogin,
    Tg.URIPassword ToURIPassword,
    Wf.WorkFlowId,
    Wf.WorkFlowName,
    Wf.WorkFlowDef,
    T.TransportType,
    Fg.ValidateClientSSLCert FrValidateClientSSLCert,
    Fg.ValidateClientIP FrValidateClientIP,
    GL.DestTypeId,
    C.TransformMapId,
    C.ProfileID
From
    PR_RoutingChannel C
    INNER JOIN PR_Capability01_vw Cfr on Cfr.CapabilityId = C.FrCapabilityId and Cfr.PartnerId = C.FrPartnerId
    INNER JOIN PR_Capability01_vw Cto on Cto.CapabilityId = C.ToCapabilityId and Cto.PartnerId = C.ToPartnerId
    INNER JOIN PR_WorkFlow Wf on Wf.WorkflowId = C.WorkFlowId
    INNER JOIN PR_Channel_Gateway_Lnk GL on GL.ChannelId = C.ChannelId
    INNER JOIN PR_Gateway Fg on Fg.GatewayId = GL.FrGatewayId
    INNER JOIN PR_Gateway Tg on Tg.GatewayId = GL.ToGatewayId
    INNER JOIN PR_Transport T on T.TransportId = Tg.TransportId
UNION
select  /*+ FIRST_ROWS */
    'TO' ChannelDirection,
    C.ChannelId,
    C.ToPartnerId FrPartnerId,
    C.FrPartnerId ToPartnerId,
    Cfr.CapabilityId FrCapabilityId,
    Cto.CapabilityId ToCapabilityId,
    Cfr.RoutingObjRefId FrRoutingObjRefId,
    Cto.RoutingObjRefId ToRoutingObjRefId,
    Cfr.RoutingObjId FrRoutingObjId,
    Cto.RoutingObjId ToRoutingObjId,
    Cfr.RoutingObjCd FrRoutingObjCd,
    Cfr.RoutingObjVer FrRoutingObjVer,
    Cto.RoutingObjCd ToRoutingObjCd,
    Cto.RoutingObjVer ToRoutingObjVer,
    CASE WHEN C.ActiveStatusCd=63 Then 1 ELSE 0 END Enabled,
    ToGatewayId FrGatewayId,
    FrGatewayId ToGatewayId,
    Fg.GatewayName ToGatewayName,
    Tg.GatewayName FrGatewayName,
    Tg.EnabledFlg FrGatewayEnabled,
    Fg.EnabledFlg ToGatewayEnabled,
    Tg.URI FrURI,
    Tg.URILogin FrURILogin,
    Tg.URIPassword FrRUIPassword,
    Fg.URI ToURI,
    Fg.URILogin ToURILogin,
    Fg.URIPassword ToURIPassword,
    Wf.WorkFlowId,
    Wf.WorkFlowName,
    Wf.WorkFlowDef,
    T.TransportType,
    Fg.ValidateClientSSLCert FrValidateClientSSLCert,
    Fg.ValidateClientIP FrValidateClientIP,
    GL.DestTypeId,
    C.TransformMapId,
    C.ProfileId
From
    PR_RoutingChannel C
    INNER JOIN PR_Capability01_vw Cfr on Cfr.CapabilityId = C.FrCapabilityId and Cfr.PartnerId = C.FrPartnerId
    INNER JOIN PR_Capability01_vw Cto on Cto.CapabilityId = C.ToCapabilityId and Cto.PartnerId = C.ToPartnerId
    INNER JOIN PR_WorkFlow Wf on Wf.WorkflowId = C.WorkFlowId
    INNER JOIN PR_Channel_Gateway_Lnk GL on GL.ChannelId = C.ChannelId
    INNER JOIN PR_Gateway Fg on Fg.GatewayId = GL.FrGatewayId
    INNER JOIN PR_Gateway Tg on Tg.GatewayId = GL.ToGatewayId
    INNER JOIN PR_Transport T on T.TransportId = Tg.TransportId
/


-- PR_ROUTINGCHANNEL02_VW.sql

CREATE OR REPLACE VIEW PR_ROUTINGCHANNEL02_VW 
(
    CHANNELID,
    FRPARTNERID,
    TOPARTNERID,
    FRPROCESSCD,
    FRPROCESSVER,
    TOPROCESSCD,
    TOPROCESSVER,
    FRPROTOCOLNAME,
    FRPROTOCOLVER,
    TOPROTOCOLNAME,
    TOPROTOCOLVER,
    FRCAPABILITYID,
    TOCAPABILITYID,
    FRROUTINGOBJREFID,
    TOROUTINGOBJREFID,
    FRROUTINGOBJID,
    TOROUTINGOBJID,
    ENABLED
)
AS
select  /*+ FIRST_ROWS */
    C.ChannelId,
    C.FrPartnerId,
    C.ToPartnerId,
    Cpf.RoutingObjCd FrProcessCd,
    Cpf.RoutingObjVer FrProcessVer,
    Cpt.RoutingObjCd ToProcessCd,
    Cpt.RoutingObjVer ToProcessVer,
    Cpf.ParentObjCd FrProtocolName,
    Cpf.ParentObjVer FrProtocolVer,
    Cpt.ParentObjCd ToProtocolName,
    Cpt.ParentObjVer ToProtocolVer,
    Cpf.CapabilityId FrCapabilityId,
    Cpt.CapabilityId ToCapabilityId,
    Cpf.RoutingObjRefId FrRoutingObjRefId,
    Cpt.RoutingObjRefId ToRoutingObjRefId,
    Cpf.RoutingObjId FrRoutingObjId,
    Cpt.RoutingObjId ToRoutingObjId,
    CASE WHEN C.ActiveStatusCd=63 Then 1 ELSE 0 END Enabled
From
    PR_RoutingChannel C
    INNER JOIN PR_Capability02_vw Cpf on Cpf.CapabilityId = C.FrCapabilityId and Cpf.PartnerId = C.FrPartnerId
    INNER JOIN PR_Capability02_vw Cpt on Cpt.CapabilityId = C.ToCapabilityId and Cpt.PartnerId = C.ToPartnerId
/


-- PR_ROUTINGCHANNEL03_VW.sql

CREATE OR REPLACE VIEW PR_ROUTINGCHANNEL03_VW 
(
    CHANNELDIRECTION,
    CHANNELID,
    FRPARTNERID,
    TOPARTNERID,
    FRCAPABILITYID,
    TOCAPABILITYID,
    FRROUTINGOBJREFID,
    TOROUTINGOBJREFID,
    FRROUTINGOBJID,
    TOROUTINGOBJID,
    PROCESSCD,
    PROCESSVER,
    PROTOCOLNAME,
    PROTOCOLVER,
    PACKAGENAME,
    PACKAGEVER,
    PACKAGEREFID,
    ACTIVESTATUSCD,
    FRGATEWAYID,
    TOGATEWAYID,
    FRGATEWAYNAME,
    TOGATEWAYNAME,
    FRGATEWAYENABLED,
    TOGATEWAYENABLED,
    FRURI,
    FRURILOGIN,
    FRRUIPASSWORD,
    TOURI,
    TOURILOGIN,
    TOURIPASSWORD,
    WORKFLOWID,
    WORKFLOWNAME,
    WORKFLOWDEF,
    TRANSPORTTYPE,
    FRVALIDATECLIENTSSLCERT,
    FRVALIDATECLIENTIP,
    DESTTYPEID,
    TRANSFORMMAPID,
    CREATEDATE,
    UPDATEDATE
)
AS
select  /*+ FIRST_ROWS */
    'FR' ChannelDirection,
    C.ChannelId,
    C.FrPartnerId,
    C.ToPartnerId,
    Cfr.CapabilityId FrCapabilityId,
    Cto.CapabilityId ToCapabilityId,
    Cfr.RoutingObjRefId FrRoutingObjRefId,
    Cto.RoutingObjRefId ToRoutingObjRefId,
    Cfr.RoutingObjId FrRoutingObjId,
    Cto.RoutingObjId ToRoutingObjId,
    Cfr.RoutingObjCd ProcessCd,
    Cfr.RoutingObjVer ProcessVer,
    Cfr.ParentObjCd ProtocolName,
    Cfr.ParentObjVer ProtocolVer,
    Cfr.GrParentObjCd PackageName,
    Cfr.GrParentObjVer PackageVer,
    Cfr.GrParentObjRefId PackageRefId,
    CASE WHEN C.ActiveStatusCd=63 Then 1 ELSE 0 END ActiveStatusCd,
    FrGatewayId, ToGatewayId,
    Fg.GatewayName FrGatewayName,
    Tg.GatewayName ToGatewayName,
    Fg.EnabledFlg FrGatewayEnabled,
    Tg.EnabledFlg ToGatewayEnabled,
    Fg.URI FrURI,
    Fg.URILogin FrURILogin,
    Fg.URIPassword FrRUIPassword,
    Tg.URI ToURI,
    Tg.URILogin ToURILogin,
    Tg.URIPassword ToURIPassword,
    Wf.WorkFlowId,
    Wf.WorkFlowName,
    Wf.WorkFlowDef,
    T.TransportType,
    Fg.ValidateClientSSLCert FrValidateClientSSLCert,
    Fg.ValidateClientIP FrValidateClientIP,
    GL.DestTypeId,
    C.TransformMapId,
    C.CreateDate, C.UpdateDate
From
    PR_RoutingChannel C
    INNER JOIN PR_Capability03_vw Cfr on Cfr.CapabilityId = C.FrCapabilityId and Cfr.PartnerId = C.FrPartnerId
    INNER JOIN PR_Capability03_vw Cto on Cto.CapabilityId = C.ToCapabilityId and Cto.PartnerId = C.ToPartnerId
    INNER JOIN PR_WorkFlow Wf on Wf.WorkflowId = C.WorkFlowId
    INNER JOIN PR_Channel_Gateway_Lnk GL on GL.ChannelId = C.ChannelId
    INNER JOIN PR_Gateway Fg on Fg.GatewayId = GL.FrGatewayId
    INNER JOIN PR_Gateway Tg on Tg.GatewayId = GL.ToGatewayId
    INNER JOIN PR_Transport T on T.TransportId = Tg.TransportId
UNION
select  /*+ FIRST_ROWS */
    'TO' ChannelDirection,
    C.ChannelId,
    C.ToPartnerId FrPartnerId,
    C.FrPartnerId ToPartnerId,
    Cfr.CapabilityId FrCapabilityId,
    Cto.CapabilityId ToCapabilityId,
    Cfr.RoutingObjRefId FrRoutingObjRefId,
    Cto.RoutingObjRefId ToRoutingObjRefId,
    Cfr.RoutingObjId FrRoutingObjId,
    Cto.RoutingObjId ToRoutingObjId,
    Cto.RoutingObjCd ProcessCd,
    Cto.RoutingObjVer ProcessVer,
    Cto.ParentObjCd ProtocolName,
    Cto.ParentObjVer ProtocolVer,
    Cto.GrParentObjCd PackageName,
    Cto.GrParentObjVer PackageVer,
    Cto.GrParentObjRefId PackageRefId,
    CASE WHEN C.ActiveStatusCd=63 Then 1 ELSE 0 END ActiveStatusCd,
    ToGatewayId FrGatewayId,
    FrGatewayId ToGatewayId,
    Fg.GatewayName ToGatewayName,
    Tg.GatewayName FrGatewayName,
    Tg.EnabledFlg FrGatewayEnabled,
    Fg.EnabledFlg ToGatewayEnabled,
    Tg.URI FrURI,
    Tg.URILogin FrURILogin,
    Tg.URIPassword FrRUIPassword,
    Fg.URI ToURI,
    Fg.URILogin ToURILogin,
    Fg.URIPassword ToURIPassword,
    Wf.WorkFlowId,
    Wf.WorkFlowName,
    Wf.WorkFlowDef,
    T.TransportType,
    Fg.ValidateClientSSLCert FrValidateClientSSLCert,
    Fg.ValidateClientIP FrValidateClientIP,
    GL.DestTypeId,
    C.TransformMapId,
    C.CreateDate, C.UpdateDate
From
    PR_RoutingChannel C
    INNER JOIN PR_Capability03_vw Cfr on Cfr.CapabilityId = C.FrCapabilityId and Cfr.PartnerId = C.FrPartnerId
    INNER JOIN PR_Capability03_vw Cto on Cto.CapabilityId = C.ToCapabilityId and Cto.PartnerId = C.ToPartnerId
    INNER JOIN PR_WorkFlow Wf on Wf.WorkflowId = C.WorkFlowId
    INNER JOIN PR_Channel_Gateway_Lnk GL on GL.ChannelId = C.ChannelId
    INNER JOIN PR_Gateway Fg on Fg.GatewayId = GL.FrGatewayId
    INNER JOIN PR_Gateway Tg on Tg.GatewayId = GL.ToGatewayId
    INNER JOIN PR_Transport T on T.TransportId = Tg.TransportId
/


-- PR_ROUTINGATTR_LNK01_VW.sql

CREATE OR REPLACE VIEW PR_ROUTINGATTR_LNK01_VW 
(
    ROUTINGATTRLNKID,
    ATTRIBUTEKEY,
    PARTNERNAME,
    PARTNERID,
    CHANNELID,
    DESTTYPEID,
    ROUTINGOBJTYPEKEY,
    ROUTINGOBJCD,
    ROUTINGOBJVER,
    ROUTINGOBJREFID,
    ROUTINGATTRID,
    GUIDELINEMAPID,
    FAMAPID,
    PROFILEID,
    ATTROPTID,
    ATTRVALUEKEY,
    ATTRVALUE,
    CREATEDATE,
    UPDATEDATE
)
AS
select
    L.RoutingAttrLnkId,
    A.AttributeKey,
  ---  FN_Get_PartnerName(L.PartnerId) PartnerName,
    p.PartnerName,
    L.PartnerId,
    L.ChannelId,
    L.DestTypeId,
    R.RoutingObjTypeKey,
    R.RoutingObjCd,
    R.RoutingObjVer,
    R.RoutingObjRefId,
    L.RoutingAttrId,
    L.GuidelineMapId,
    L.FAMapId,
    L.ProfileId,
    L.AttrOptId,
    O.AttrValueKey,
    L.AttrValue,
    L.CreateDate,
    L.UpdateDate
from PR_RoutingAttr_Lnk L
INNER JOIN CF_RoutingObject04_vw R on R.RoutingObjRefId = L.RoutingObjRefId
INNER JOIN CF_RoutingAttr A on A.RoutingAttrId = L.RoutingAttrId
LEFT OUTER JOIN pr_Partner p on p.PartnerId = L.PartnerId
LEFT OUTER JOIN CF_RoutingAttrOpt O on O.AttrOptId = L.AttrOptId
/


-- DB_ACTIVITY_TIMING_VW.sql

CREATE OR REPLACE VIEW DB_ACTIVITY_TIMING_VW 
(
    ACTIVITYID,
    INITPARTNERIDHDR,
    INITPARTNERNAME,
    FRPARTNERIDHDR,
    FRPARTNERNAME,
    TOPARTNERIDHDR,
    TOPARTNERNAME,
    FRPROTOCOLNAME,
    TOPROTOCOLNAME,
    FRPROTOCOLVER,
    TOPROTOCOLVER,
    FRPROCESSCD,
    TOPROCESSCD,
    FRPROCESSVER,
    TOPROCESSVER,
    DOCID,
    DOCTS,
    CHANNELID,
    CHANNELSTATUSID,
    FRIPADDRESS,
    RCVDOCTS,
    PERSISTDOCTS,
    PROCESSSTARTDOCTS,
    PROCESSENDDOCTS,
    PROVSESSIONID,
    DESTTYPEID,
    ENDSTATEFLG,
    ENDSTATETS,
    DMDURATION,
    TRANSPORTDURATION,
    TRANSPORTATTEMPTCNT,
    GATEWAYATTEMPTCNT,
    PARENT_VUID,
    PARENT_HDRFLG,
    PARENT_MSGRAWLENGTH,
    PARENT_MSGCONTENTLENGTH,
    CHILD_VUID,
    CHILD_HDRFLG,
    CHILD_MSGRAWLENGTH,
    CHILD_MSGCONTENTLENGTH
)
AS
select /*+ FIRST_ROWS */
    a.ACTIVITYID
    , INITPARTNERIDHDR, initP.PartnerName   InitPartnerName
    , FRPARTNERIDHDR  , FrP.PartnerName     FrPartnerName
    , TOPARTNERIDHDR  , Top.PartnerName     ToPartnerName
    --
    , FRPROTOCOLNAME
    , TOPROTOCOLNAME
    , FRPROTOCOLVER
    , TOPROTOCOLVER
    , FRPROCESSCD
    , TOPROCESSCD
    , FRPROCESSVER
    , TOPROCESSVER
    , DOCID
    , DOCTS
    , CHANNELID
    , CHANNELSTATUSID
    , FRIPADDRESS
    -- Timing
    , RCVDOCTS
    , PERSISTDOCTS
    , PROCESSSTARTDOCTS
    , PROCESSENDDOCTS
    , PROVSESSIONID
    , DESTTYPEID
    -- EndState
    , ae.ENDSTATEFLG
    , ae.ENDSTATETS
    , ae.DMDURATION
    , ae.TRANSPORTDURATION
    , ae.TRANSPORTATTEMPTCNT
    , ae.GATEWAYATTEMPTCNT
    -- Dtl- parent
    , d1.VUID  Parent_vuid
    , d1.HDRFLG  Parent_HDRFLG
    , d1.MSGRAWLENGTH  Parent_MSGRAWLENGTH
    , d1.MSGCONTENTLENGTH  Parent_MSGCONTENTLENGTH
    -- Dtl- parent
    , d2.VUID     Child_vuid
    , d2.HDRFLG   Child_HDRFLG
    , d2.MSGRAWLENGTH    Child_MSGRAWLENGTH
    , d2.MSGCONTENTLENGTH Child_MSGCONTENTLENGTH
From Lg_Activity a
    Inner Join Lg_Activity_EndState ae on a.activityID = ae.activityID
    Inner Join Lg_Activity_Dtl_Parent_vw d1 on a.activityID = d1.activityID
    Left outer Join Lg_Activity_Dtl_child_vw d2 on a.activityID  = d2.activityID
    Left outer Join pr_partner initP on a.INITPARTNERIDHDR = initP.PartnerID
    Left outer Join pr_partner FrP   on a.FrPARTNERIDHDR   = FrP.PartnerID
    Left outer Join pr_partner ToP   on a.ToPARTNERIDHDR   = ToP.PartnerID
/


-- DB_PROCAUDITLOG_VW.sql

CREATE OR REPLACE VIEW DB_PROCAUDITLOG_VW 
(
    PROCNAME,
    CREATEDATE,
    ELAPSEDTIME,
    PARAMSTXT,
    STARTTS,
    EXCEPTIONFLG
)
AS
select
    PROCNAME,
    CREATEDATE,
    to_char(ELAPSEDTIME) ElapsedTime,
    PARAMSTXT,
    STARTTS,
    EXCEPTIONFLG
from DB_ProcAuditLog
/


-- DB_RNSTATEDTL_VW.sql

CREATE OR REPLACE VIEW DB_RNSTATEDTL_VW 
(
    RNSTATEDTLID,
    INITBUSINESSID,
    PROCESSINSTANCEID,
    TRANSACTIONINSTANCEID,
    ACTIONINSTANCEID,
    SIGNALINSTANCEID,
    MSGTRACKINGINSTANCEID,
    GLBUSAGECD,
    ACTIONVER,
    SIGNALVER,
    GLBBUSACTIONCD,
    GLBBUSSIGNALCD,
    FRBUSINESSID,
    TOBUSINESSID,
    ACTIVITYNAME,
    MSGDATETIME,
    EXPIRETPATIME,
    ATTEMPTCNT,
    ATTEMPTCNTCALC,
    ATTEMPTCNTTPA,
    FRSUPPLYCHAINCD,
    TOSUPPLYCHAINCD,
    DOCID,
    DOCTS,
    ACTIVITYID,
    INBOUNDVUID,
    OUTBOUNDVUID,
    CUSTOMMSGID01,
    CUSTOMMSGID02,
    ROWTS,
    CREATEDATE,
    UPDATEDATE,
    MSGDIGEST,
    DIGITALSIGALGO
)
AS
select
RNSTATEDTLID
,INITBUSINESSID
,PROCESSINSTANCEID
,TRANSACTIONINSTANCEID
,ACTIONINSTANCEID
,SIGNALINSTANCEID
,MSGTRACKINGINSTANCEID
,GLBUSAGECD
,ACTIONVER
,SIGNALVER
,GLBBUSACTIONCD
,GLBBUSSIGNALCD
,FRBUSINESSID
,TOBUSINESSID
,ACTIVITYNAME
,MSGDATETIME
,EXPIRETPATIME
,ATTEMPTCNT
,ATTEMPTCNTCALC
,ATTEMPTCNTTPA
,FRSUPPLYCHAINCD
,TOSUPPLYCHAINCD
,DOCID
,DOCTS
,ACTIVITYID
,INBOUNDVUID
,OUTBOUNDVUID
,CUSTOMMSGID01
,CUSTOMMSGID02
,ROWTS
,to_char(CREATEDATE, 'dd_MON-yyyy hh24:mi:ssxFF')CREATEDATE
,to_char(UPDATEDATE, 'dd_MON-yyyy hh24:mi:ssxFF')UPDATEDATE
,dbms_lob.substr(MSGDIGEST, dbms_lob.getlength(MSGDIGEST), 1) MSGDIGEST
,DIGITALSIGALGO
from bp_rnStateDtl
/


-- DB_RNSTATEHDRAUDITLOG_VW.sql

CREATE OR REPLACE VIEW DB_RNSTATEHDRAUDITLOG_VW 
(
    INITBUSINESSID,
    PROCESSINSTANCEID,
    RNSTATEDTLID,
    NEXTACTIONCD,
    MSGSENTSTATUSCD,
    CURRENTPIPSTATUSCD,
    ERRORREASONTEXT,
    CREATEDATE
)
AS
select
INITBUSINESSID
,PROCESSINSTANCEID
,RNSTATEDTLID
,NEXTACTIONCD
,MSGSENTSTATUSCD
,CURRENTPIPSTATUSCD
,substr(ERRORREASONTEXT,1,240)
,substr(to_char(createDate, 'yyyy-MM-DD hh24:mi:ssxFF'),1,24)
from  bp_rnstateHdrAuditLog
/


-- LG_EDI_OVERDUE_FA_VW.sql

--> overdue report view
--55937 - removed check for interchange and group control numbers
CREATE OR REPLACE VIEW lg_edi_overdue_fa_vw AS
   SELECT reconcile.createdate datesent, reconcile.originalactivityid, 
      partner.partnerid frompartnerid, partner2.partnerid topartnerid,
      partner.partnername frompartnername, partner2.partnername topartnername,
      activity.frprotocolname, activity.frprotocolver, 
      activity.toprotocolname, activity.toprotocolver,
      activity.frpackagename, activity.frpackagever,
      activity.topackagename, activity.topackagever,
      activity.frprocesscd, activity.frprocessver,
      activity.toprocesscd, activity.toprocessver,
      reconcile.intctlnum interchangecontrolnum,
      reconcile.grpctlnum groupcontrolnum,
      reconcile.txctlnum transactioncontrolnum,
      reconcile.fadueby,
      round(fn_timestampdiff((reconcile.createdate + (fadueby/1440)),fn_get_date())/(1000*60)) overdueby,
      CAST((reconcile.createdate + (fadueby/1440)) AS TIMESTAMP) AS expectedtime
   FROM lg_edi_fa_reconcile reconcile, pr_partner partner, 
      pr_partner partner2, lg_activity activity
   WHERE reconcile.frompartnerid = partner.partnerid AND
      reconcile.topartnerid = partner2.partnerid AND
      reconcile.originalactivityid = activity.activityid AND    
      round(fn_timestampdiff((reconcile.createdate + (fadueby/1440)), fn_get_date())/(1000*60)) > reconcile.fadueby AND
      reconcile.ignoredueby = 0 AND
      reconcile.txctlnum IS NOT NULL AND 
      reconcile.fadueby <> 0 AND
      reconcile.mostrecent = 1
/


-- LG_EDI_REJECTED_TX_VW.sql

-- rejected transaction report view
--55937 - removed check for interchange and group control numbers
CREATE OR REPLACE VIEW lg_edi_rejected_tx_vw AS
   SELECT ack.createdate, ack.ackactivityid,
      partner.partnerid frompartnerid, partner2.partnerid topartnerid,
      partner.partnername frompartnername, partner2.partnername topartnername,
      activity.frprotocolname, activity.frprotocolver, 
      activity.toprotocolname, activity.toprotocolver,
      activity.frpackagename, activity.frpackagever,
      activity.topackagename, activity.topackagever,
      activity.frprocesscd, activity.frprocessver,
      activity.toprocesscd, activity.toprocessver,
      ack.statuscode, ack.statustext,
      reconcile.intctlnum interchangecontrolnum,
      reconcile.grpctlnum groupcontrolnum,
      reconcile.txctlnum transactioncontrolnum
   FROM lg_ack ack, lg_activity activity, lg_edi_fa_reconcile reconcile,
      pr_partner partner, pr_partner partner2
   WHERE activity.frpartneridhdr = partner.partnerid AND
      activity.topartneridhdr = partner2.partnerid AND
      activity.activityid = reconcile.originalactivityid AND
      ack.originalactivityid = reconcile.originalactivityid AND
      --ack.ackactivityid = activity.activityid AND 
      ack.rejectedflg = 1 AND 
      reconcile.txctlnum IS NOT NULL AND
      reconcile.mostrecent = 1
/


-- LG_EDI_TX_DISPOSITION_VW.sql

CREATE OR REPLACE VIEW lg_edi_tx_disposition_vw AS
SELECT reconcile.originalactivityid,
   ack.ackactivityid,
   ack.statuscode,
   ack.statustext,
   ack.rejectedflg,
   partner1.partnerid frompartnerid,
   partner2.partnerid topartnerid,
   partner1.partnername frompartnername,
   partner2.partnername topartnername,
   reconcile.intctlnum,
   reconcile.grpctlnum,
   reconcile.txctlnum,
   activity.frprotocolname, 
   activity.frprotocolver, 
   activity.toprotocolname, 
   activity.toprotocolver,
   activity.frpackagename, 
   activity.frpackagever,
   activity.topackagename, 
   activity.topackagever,
   activity.frprocesscd, 
   activity.frprocessver,
   activity.toprocesscd, 
   activity.toprocessver,
   round(fn_timestampdiff((reconcile.createdate + (fadueby/1440)),fn_get_date())/(1000*60)) overdueby,
   reconcile.fadueby
FROM lg_edi_fa_reconcile reconcile
   inner join pr_partner  partner1 ON reconcile.frompartnerid = partner1.partnerid
   inner join pr_partner  partner2 ON reconcile.topartnerid = partner2.partnerid
   inner join lg_activity activity ON activity.activityid = reconcile.originalactivityid
   left outer join lg_ack ack ON reconcile.originalactivityid = ack.originalactivityid
/


-- CF_ROUTINGOBJECT06_VW.sql

CREATE OR REPLACE VIEW CF_ROUTINGOBJECT06_VW(
   ROUTINGOBJLEVEL,
   ROUTINGOBJTYPEKEY,
   ROUTINGOBJTYPEID,
   ROUTINGOBJID,
   ROUTINGOBJCD,
   ROUTINGOBJVER,
   ROUTINGOBJREFID,
   ROUTINGOBJPARENTREFID,
   GRPARENTOBJREFID,
   GRPARENTOBJCD,
   GRPARENTOBJVER,
   GRPARENTOBJREFID2,
	 GRPARENTOBJCD2,
   GRPARENTOBJVER2,
   GRPARENTOBJREFID3,
	 GRPARENTOBJCD3,
   GRPARENTOBJVER3,
   PARENTOBJLEVEL,
   PARENTOBJTYPEKEY,
   PARENTOBJID,
   PARENTOBJCD,
   PARENTOBJVER,
   ROUTINGOBJREFNAME
)
AS 
SELECT D.ROUTINGOBJLEVEL,
   D.ROUTINGOBJTYPEKEY,
   R.ROUTINGOBJTYPEID,
   R.ROUTINGOBJID,
   R.ROUTINGOBJCD,
   R.ROUTINGOBJVER,
   F.ROUTINGOBJREFID,
   F.ROUTINGOBJPARENTREFID,
   F2.ROUTINGOBJPARENTREFID GRPARENTOBJREFID,
   R3.ROUTINGOBJCD GRPARENTOBJCD,
   R3.ROUTINGOBJVER GRPARENTOBJVER,
   F3.ROUTINGOBJPARENTREFID GRPARENTOBJREFID2,
	 R4.ROUTINGOBJCD GRPARENTOBJCD2,
   R4.ROUTINGOBJVER GRPARENTOBJVER2,
   F4.ROUTINGOBJPARENTREFID GRPARENTOBJREFID3,
	 R5.ROUTINGOBJCD GRPARENTOBJCD3,
   R5.ROUTINGOBJVER GRPARENTOBJVER3,
   D2.ROUTINGOBJLEVEL PARENTOBJLEVEL,
   D2.ROUTINGOBJTYPEKEY PARENTOBJTYPEKEY,
   R2.ROUTINGOBJID PARENTOBJID,
   R2.ROUTINGOBJCD PARENTOBJCD,
   R2.ROUTINGOBJVER PARENTOBJVER,
   CASE D.ROUTINGOBJLEVEL
	                                when 1 THEN RTRIM(R.ROUTINGOBJCD)||'('||RTRIM(R.ROUTINGOBJVER)||')(L1)'
	                                when 2 THEN RTRIM(R2.ROUTINGOBJCD)||'('||RTRIM(R2.ROUTINGOBJVER)||')(L1)_'||RTRIM(R.ROUTINGOBJCD)||'('||RTRIM(R.ROUTINGOBJVER)||')(L2)'
	                                when 3 THEN RTRIM(R3.ROUTINGOBJCD)||'('||RTRIM(R3.ROUTINGOBJVER)||')(L1)_'||RTRIM(R2.ROUTINGOBJCD)||'('||RTRIM(R2.ROUTINGOBJVER)||')(L2)_'||RTRIM(R.ROUTINGOBJCD)||'('||RTRIM(R.ROUTINGOBJVER)||')(L3)'
	                                when 4 THEN RTRIM(R4.ROUTINGOBJCD)||'('||RTRIM(R4.ROUTINGOBJVER)||')(L1)_'||RTRIM(R3.ROUTINGOBJCD)||'('||RTRIM(R3.ROUTINGOBJVER)||')(L2)_'||RTRIM(R2.ROUTINGOBJCD)||'('||RTRIM(R2.ROUTINGOBJVER)||')(L3)_'||RTRIM(R.ROUTINGOBJCD)||'('||RTRIM(R.ROUTINGOBJVER)||')(L4)'
	                                WHEN 5 THEN RTRIM(R5.ROUTINGOBJCD)||'('||RTRIM(R5.ROUTINGOBJVER)||')(L1)_'||RTRIM(R4.ROUTINGOBJCD)||'('||RTRIM(R4.ROUTINGOBJVER)||')(L2)_'||RTRIM(R3.ROUTINGOBJCD)||'('||RTRIM(R3.ROUTINGOBJVER)||')(L3)_'||RTRIM(R2.ROUTINGOBJCD)||'('||RTRIM(R2.ROUTINGOBJVER)||')(L4)_'||RTRIM(R.ROUTINGOBJCD)||'('||RTRIM(R.ROUTINGOBJVER)||')(L5)'
	                                WHEN 6 THEN RTRIM(R6.ROUTINGOBJCD)||'('||RTRIM(R6.ROUTINGOBJVER)||')(L1)_'||RTRIM(R5.ROUTINGOBJCD)||'('||RTRIM(R5.ROUTINGOBJVER)||')(L2)_'||RTRIM(R4.ROUTINGOBJCD)||'('||RTRIM(R4.ROUTINGOBJVER)||')(L3)_'||RTRIM(R3.ROUTINGOBJCD)||'('||RTRIM(R3.ROUTINGOBJVER)||')(L4)_'||RTRIM(R2.ROUTINGOBJCD)||'('||RTRIM(R2.ROUTINGOBJVER)||')(L5)_'||RTRIM(R.ROUTINGOBJCD)||'('||RTRIM(R.ROUTINGOBJVER)||')(L6)'
	                                ELSE 'test'
                               END
FROM CF_ROUTINGOBJECT R 
INNER JOIN CF_ROUTINGOBJREF_LNK F ON F.ROUTINGOBJID = R.ROUTINGOBJID 
INNER JOIN CF_ROUTINGOBJTYPE D ON D.ROUTINGOBJTYPEID = R.ROUTINGOBJTYPEID 
LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F2 ON F2.ROUTINGOBJREFID = F.ROUTINGOBJPARENTREFID 
LEFT OUTER JOIN CF_ROUTINGOBJECT R2 ON R2.ROUTINGOBJID = F2.ROUTINGOBJID 
LEFT OUTER JOIN CF_ROUTINGOBJTYPE D2 ON D2.ROUTINGOBJTYPEID = R2.ROUTINGOBJTYPEID
LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F3 ON F3.ROUTINGOBJREFID = F2.ROUTINGOBJPARENTREFID
LEFT OUTER JOIN CF_ROUTINGOBJECT R3 ON R3.ROUTINGOBJID = F3.ROUTINGOBJID
LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F4 ON F4.ROUTINGOBJREFID = F3.ROUTINGOBJPARENTREFID
LEFT OUTER JOIN CF_ROUTINGOBJECT R4 ON R4.ROUTINGOBJID = F4.ROUTINGOBJID
LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F5 ON F5.ROUTINGOBJREFID = F4.ROUTINGOBJPARENTREFID
LEFT OUTER JOIN CF_ROUTINGOBJECT R5 ON R5.ROUTINGOBJID = F5.ROUTINGOBJID
LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F6 ON F6.ROUTINGOBJREFID = F5.ROUTINGOBJPARENTREFID
LEFT OUTER JOIN CF_ROUTINGOBJECT R6 ON R6.ROUTINGOBJID = F6.ROUTINGOBJID
/


-- BP_SPONSOR_STATE_VW.sql

CREATE OR REPLACE VIEW BP_SPONSOR_STATE_VW AS
SELECT	INITBUSINESSID
	, PROCESSINSTANCEID
	, NEXTACTIONCD
	, FRBUSINESSID
	, TOBUSINESSID
	, DOCID
	, CUSTOMMSGID01
	, UPDATESTATUSCD
	, GLBUSAGECD
	, DESTTYPEID
	, INBOUNDVUID
	, REFINBOUNDVUID
	, SYNCHRONOUSFLG
	, ROWTS
	, CREATEDATE
	, UPDATEDATE
	, 1 AS STATECD
FROM	BP_SPONSOR_STATE
UNION
SELECT	INITBUSINESSID
	, PROCESSINSTANCEID
	, NEXTACTIONCD
	, FRBUSINESSID
	, TOBUSINESSID
	, DOCID
	, CUSTOMMSGID01
	, UPDATESTATUSCD
	, GLBUSAGECD
	, DESTTYPEID
	, INBOUNDVUID
	, REFINBOUNDVUID
	, SYNCHRONOUSFLG
	, ROWTS
	, CREATEDATE
	, UPDATEDATE
	, 2 AS STATECD
FROM	BP_SPONSOR_STATE_EBMS
/

-- PR_PARTNER_HUBOWNER_VW.sql

CREATE OR REPLACE VIEW PR_PARTNER_HUBOWNER_VW
AS
SELECT partnerID 
FROM PR_PARTNER pi 
INNER JOIN PR_PARTNERTYPE pt ON pi.partnerTypeid = pt.partnerTypeId 
WHERE pt.PartnerTypeName = 'HubOwner'
/
--  -- # ============================================================== # !
--  # Create Views !
--  -- # ============================================================== # !

-- EDIMAPCSTDET.sql

CREATE OR REPLACE VIEW EDIMAPCSTDET AS                                          
  SELECT MAPID, CSTYPE, CSBYTES, CSLINUM, CSCONT, GENDATE, GENTIME,             
         STRINGS                                                                
  FROM EDICSTX                                                         
  WHERE CSTYPE='MP'
/                      

-- EDIMAPCSTHDR.sql

CREATE OR REPLACE VIEW EDIMAPCSTHDR AS                                          
    SELECT MAPID, GENDATE, GENTIME                                              
      FROM EDICSTX                                                     
      WHERE     CSLINUM=1 AND                                                   
                CSTYPE='MP'
/

-- EDIVCSTX.sql

CREATE OR REPLACE VIEW EDIVCSTX AS SELECT * FROM EDICSTX
/          

-- EDIVGLBV.sql

 CREATE OR REPLACE VIEW EDIVGLBV AS SELECT * FROM EDIMAPGBLVAR
/

-- EDIVPSAD.sql

CREATE OR REPLACE VIEW EDIVPSAD AS SELECT * FROM EDIPSAD
/

-- EDIVPSDI.sql

CREATE OR REPLACE VIEW EDIVPSDI AS SELECT * FROM EDIPSDI
/

-- EDIVPSTD.sql

CREATE OR REPLACE VIEW EDIVPSTD AS SELECT * FROM EDIPSTD
/
                                                     

-- EDIVPSTT.sql

CREATE OR REPLACE VIEW EDIVPSTT AS SELECT * FROM EDIPSTT
/                                

-- EDIVPSTV.sql

CREATE OR REPLACE VIEW EDIVPSTV AS SELECT * FROM EDIPSTV
/                                           

-- EDIVXMLNS.sql

CREATE OR REPLACE VIEW EDIVXMLNS AS SELECT * FROM EDIXMLNS
/  

-- # ============================================================== # 
-- Create Functions 
-- # ============================================================== # 

-- createdir.sql

CREATE OR REPLACE FUNCTION createdir (dir VARCHAR2)
   RETURN NUMBER AS LANGUAGE JAVA
   NAME 'com.ibm.bcg.db.archiver.udf.CreateDirectory.createDirectory(java.lang.String) return int';
/

-- FN_BUILD_WHERE.sql

CREATE OR REPLACE FUNCTION FN_BUILD_WHERE(
        P_COLUMN    VARCHAR2,
        P_DATATYPE  VARCHAR2,
        P_PARAM     VARCHAR2
        )
RETURN VARCHAR2
IS
/*
    =============================================================================
    Proc Name:      FN_BUILD_WHERECLAUSE
    Create Date:    2003-07-03
    Author:         V.Goel
    Purpose: This proc builds or add to the existing where clause for dynamic query.
    Data Definition:
        p_Column  : Name of the table column. should be prefixed with table alias
        p_DataType: Data type is not the same as that of Oracle. Currently it supports
                    NUMBER_LIST, NUMBER, VARCHAR2, VARCHAR2_LIKE. More can be added in future
        p_Param   : Parameter name in the calling procedure that relate to the column
        p_Where   : Variable storing the where clause
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------ ------ ---------------------------------------------
    DATE       NAME         VER     DESCRIPTION
    ---------- ------------ ------ ---------------------------------------------
    2003-07-03 V.Goel       3.5     Created
    =============================================================================
*/
v_Where VARCHAR2(4000) default '' ;
BEGIN
    IF p_Param IS NOT NULL THEN
	v_Where := 
	CASE p_DataType
                WHEN 'NUMBER_LIST'   THEN   ' AND '||p_Column||' IN '||p_Param
                WHEN 'NUMBER'        THEN   ' AND '||p_Column||' ='  ||p_Param
                WHEN 'VARCHAR2'      THEN   ' AND '||p_Column||' ='''||p_Param||''''
                WHEN 'VARCHAR2_LIKE' THEN   ' AND '||p_Column||' LIKE '''||p_Param||''''
                WHEN 'VARCHAR2_LIKE2' THEN   ' AND '||p_Column||' LIKE '''||p_Param||''||'%'''                
--                WHEN 'VARCHAR2_LIKE' THEN   ' AND '||UPPER(p_Column)||' LIKE '''||UPPER(p_Param)||''''
                ELSE  ''
            END;
    END IF;
    Return v_Where;
END;
/


-- FN_CALC_CPALIMITS.sql

CREATE OR REPLACE FUNCTION FN_Calc_CPALimits(P_CPAID VARCHAR2, P_PARTNERID1 INTEGER, P_PARTNERID2 INTEGER)
RETURN     PR_AGREEMENT_TBL PIPELINED IS
/*
    ================================================================================
    Proc Name:      FN_Calc_CPALimits
    Create Date:    2006-06-19
    Author:         S.Priani

    ================================================================================
*/
v_ReturnRecord	PR_AGREEMENT_REC := PR_AGREEMENT_REC(0, p_CPAID, p_PartnerID1, p_PartnerID2, 0, 0);
v_MSts_InProgress	INTEGER		DEFAULT 0;
v_MSts_OutOFSequence	INTEGER		DEFAULT 5;

BEGIN
	BEGIN
		SELECT	COALESCE(COUNT(G.PrcHdr), 0), COALESCE(SUM(G.NowOpen), 0)
		INTO	v_ReturnRecord.Invocations, v_ReturnRecord.OpenConversations
		FROM	(
				SELECT	hdr.ProcessHdrID PrcHDr
					,MAX(CASE WHEN MsgStatusCode IN (v_MSts_InProgress, v_MSts_OutOFSequence) THEN 1 ELSE 0 END) NowOpen
				FROM	LG_B2BPROCESS_HDR hdr
				INNER JOIN LG_B2BPROCESS_DTL dtl ON dtl.ProcessHdrId = hdr.ProcessHdrID
				WHERE	hdr.cpaid = p_CPAID
				AND	hdr.FromPartnerId IN (p_PartnerId1, p_PartnerId2)
				AND	hdr.ToPartnerId IN (p_PartnerId1, p_PartnerId2)
				GROUP BY hdr.ProcessHdrId
			)  G;

	EXCEPTION WHEN Others THEN
		v_ReturnRecord.Invocations := 0;
		v_ReturnRecord.OpenConversations := 0;
	END;

	PIPE ROW (v_ReturnRecord);
	RETURN;
END;
/

-- FN_CHK_CERT.sql

CREATE OR REPLACE FUNCTION FN_CHK_CERT(p_CertId in Number)
RETURN BOOLEAN IS
	v_Cnt Number(10);
BEGIN
    SELECT 1 INTO v_Cnt FROM LG_CERT_ARCHIVE
    WHERE ARCHIVECERTID = p_CertId;
    
    If v_Cnt = 0 then
    	RETURN TRUE;
    Else
    	RETURN FALSE;
    End if;
    
EXCEPTION
	WHEN No_Data_Found THEN 
		RETURN TRUE;
	WHEN Others THEN 
		RAISE;
END;
/


-- FN_CHK_DEFAULT_LOCALE.sql

CREATE OR REPLACE FUNCTION FN_CHK_DEFAULT_LOCALE(p_Locale IN VARCHAR2, p_LocTable IN VARCHAR2)
RETURN VARCHAR2 IS

/*
    ================================================================================
    Proc Name:      FN_CHK_DEFAULT_LOCALE
    Create Date:    2003-05-27
    Author:         L.Ling


                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-27 L.Ling        3.5    Returns the default LOCALE if p_Locale is null or
                                    p_Locale does not exist in p_LocTable table.
    ================================================================================
*/

v_Locale       PR_LOCALE.Locale%Type;
v_SqlStmt      VARCHAR2(500);
v_Count        NUMBER := -1;
v_Locale2      PR_LOCALE.Locale%Type;
v_Locale3      PR_LOCALE.Locale%Type;

PROCEDURE GET_DEFAULT
AS
BEGIN
    SELECT LOCALE
    INTO   v_Locale
    FROM   PR_LOCALE
    WHERE  DEFAULTFLG = 1;
END;

BEGIN
   IF p_Locale IS NULL THEN
       GET_DEFAULT;
   ELSE
       v_SqlStmt :=  'SELECT COUNT(*) FROM ' || p_LocTable || ' WHERE Locale = ' || '''' || p_Locale || '''';
       --DBMS_OUTPUT.PUT_LINE (v_SqlStmt);
       EXECUTE IMMEDIATE v_SqlStmt INTO v_Count;

       IF v_Count = 0 THEN -- 1st time no match
           SELECT SUBSTR(p_Locale, 1, INSTR(p_Locale, '_', -1) - 1) INTO v_Locale2 FROM dual;
           --DBMS_OUTPUT.PUT_LINE (v_Locale2);
           IF v_Locale2 IS NULL THEN
               GET_DEFAULT;
           ELSE
               v_SqlStmt :=  'SELECT COUNT(*) FROM ' || p_LocTable || ' WHERE Locale = ' || '''' || v_Locale2 || '''';
               EXECUTE IMMEDIATE v_SqlStmt INTO v_Count;

               IF v_Count = 0 THEN -- 2nd time no match
                   SELECT SUBSTR(v_Locale2, 1, INSTR(v_Locale2, '_', -1) - 1) INTO v_Locale3 FROM dual;
                   --DBMS_OUTPUT.PUT_LINE (v_Locale3);
                   v_SqlStmt :=  'SELECT COUNT(*) FROM ' || p_LocTable || ' WHERE Locale = ' || '''' || v_Locale3 || '''';
                   EXECUTE IMMEDIATE v_SqlStmt INTO v_Count;

                   IF v_Count = 0 THEN
                       GET_DEFAULT;
                   ELSE
                       v_Locale := v_Locale3;
                   END IF;
               ELSE
                   v_Locale := v_Locale2;
               END IF;
           END IF;
       ELSE
           v_Locale := p_Locale;
       END IF;
   END IF;

   RETURN v_Locale;

   EXCEPTION
   WHEN OTHERS THEN
       Raise_application_error (-20999,'20000 FN_CHK_DEFAULT_LOCALE ' ||SQLERRM);
END FN_CHK_DEFAULT_LOCALE;
/


-- FN_COMPARE_DB_OBJECT_COUNT.sql

CREATE OR REPLACE FUNCTION FN_COMP_DB_COUNT
(
P_DB_SCHEMA VARCHAR2
)
RETURN VARCHAR2
IS


/*
    ================================================================================

    Proc Name:      FN_COMP_DB_COUNT
    Create Date:    2008-08-19
    Author:         BHARANI J

    Description:    This function returns SUCESS OR FAILURE.  Compares the DB COUNT
    in db_object_count table and Dictionary Tables.  If Dictionary count is greater
    that db_object_count which was inserted by installer than Installation or Migration
    is fine.


                       ::::::::::::::::::::::::::::::

                             M O D I F I C A T I O N

                       ::::::::::::::::::::::::::::::


    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
   .
    ================================================================================
*/

   v_tablecount NUMBER(10);
    v_proccount NUMBER(10);
    v_viewcount NUMBER(10);
    v_functioncount NUMBER(10);


    v_tablecount_dict NUMBER(10);
     v_proccount_dict NUMBER(10);
    v_viewcount_dict NUMBER(10);
    v_functioncount_dict NUMBER(10);


    v_return VARCHAR2(30);

   BEGIN

    SELECT COALESCE(table_count,0),COALESCE(procedure_count,0),
    	   COALESCE(view_count,0),COALESCE(function_count,0)  INTO
    	   v_tablecount,v_proccount,v_viewcount,v_functioncount
    	   FROM DB_OBJECT_COUNT;


    SELECT COUNT(*) INTO v_tablecount_dict FROM user_tables;

    SELECT COUNT(*) INTO v_proccount_dict FROM user_objects where object_type='PROCEDURE';

    SELECT COUNT(*) INTO v_viewcount_dict FROM user_views;

    SELECT COUNT(*) INTO v_functioncount_dict FROM user_objects where object_type='FUNCTION';




   IF v_tablecount_dict >= v_tablecount   AND v_proccount_dict >= v_proccount
   AND v_viewcount_dict >= v_viewcount AND v_functioncount_dict >= v_functioncount
   
   THEN


    v_return :='SUCCESS';

   ELSE

    v_return :='FAILURE';

   END IF;

   Return v_return;

   END FN_COMP_DB_COUNT;
/

-- FN_FIND_SETNAME.sql

CREATE OR REPLACE FUNCTION FN_FIND_SETNAME
(
    p_certid INTEGER,
    p_partnerid INTEGER,
    p_searchflg INTEGER       -- 1-- InternalPartner 2 -- ExternalPartner
)
RETURN  VARCHAR IS

/*
	================================================================================
	Proc Name:       FN_FIND_SETNAME
	Create Date:     2007-09-25 15:16:24
	Author:          BHARANI J
	
	USED IN PR_CERTS_AH9

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-25 BHARANI J     6.1.1  Created for Multiple Certificate Feature

	================================================================================

*/



v_temp VARCHAR2(255);

BEGIN

BEGIN

SELECT 	b.CERTSETNAME into v_temp
FROM PR_CERT_SET_LNK l
right outer join pr_cert a on a.certid = l.certid
left  outer join pr_cert_set b on b.certsetid = l.certsetid
WHERE a.certid = p_certid
AND a.partnerid = p_partnerid
AND coalesce(a.deleteflag,0) <> 1;

EXCEPTION
  	WHEN NO_DATA_FOUND THEN
  	NULL;
END;

RETURN v_temp;

END FN_FIND_SETNAME;
/

-- FN_GET_ACKSTATUS.sql

CREATE OR REPLACE FUNCTION FN_GET_ACKSTATUS (p_MESSAGEID varchar2)
return number
as
/*
    ================================================================================
    Proc Name:      FN_GET_ACKSTATUS
    Create Date:    2009-02-05
    Author:         Reema kakodker
    Description:    This function will return the ACk status

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2009-01-05 Reema Kakodker 1.00   Created.

    ================================================================================

*/
v_AckStatus  Integer;

BEGIN
    Begin
        Select ackstatus into v_AckStatus from BP_RM_STATE_DTL where MESSAGEID = p_MESSAGEID;
        Exception
            WHEN others THEN
                v_AckStatus := -1;
    END;
    RETURN v_AckStatus;
END;
/



-- FN_GET_ATTRSCOPE.sql

CREATE OR REPLACE FUNCTION FN_Get_AttrScope
(
   p_AttrRank        Number,
   p_Option          Varchar2,   -- Values: ScopeKey,ScopeCd,LevelKey,LevelCd
   P_Locale          varchar2
)
RETURN Varchar2
as

/*
    ================================================================================
    Proc Name:      FN_Get_AttrScope
    Create Date:    2003-05-09
    Author:         J.Chen


                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-09 J.Chen        3.5    Returns where the attribute is inherited from
                                   (global,partner,channel or DestType) by given attribute rank.
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
*/


v_ScopeKey              varchar2(512);
v_ScopeRank             varchar2(30);
v_SumScopeRank          Integer;
v_AttrRank              Integer;
v_RoutingObjAttrRank    Integer;
v_LevelKey              varchar2(30);
v_LevelCd               varchar2(100);
v_ScopeCd               varchar2(100);

BEGIN

      v_AttrRank := p_AttrRank;

--------------------------------------------------------------
-- Get Routing object type  where the AttrValue inherited from
--------------------------------------------------------------

    Select    Max(RoutingObjAttrRank)
    Into      v_RoutingObjAttrRank
    From      CF_RoutingObjType
    Where     RoutingObjAttrRank <= v_AttrRank
    ;

    If p_Option = 'LevelKey' Then
        v_LevelKey := FN_GET_RTOBJTYPE(v_RoutingObjAttrRank,'Key',p_locale);
    Elsif p_Option = 'LevelCd' Then
        v_LevelCd := FN_GET_RTOBJTYPE(v_RoutingObjAttrRank,'Code',p_locale);
    End If;

-----------------------------------------------
-- Get scope where the AttrValue inherited from
-----------------------------------------------

    v_SumScopeRank := v_AttrRank - v_RoutingObjAttrRank;

    If v_SumScopeRank != 0 Then
        Select    Max(RoutingObjAttrRank)
        Into      v_ScopeRank
        From      CF_RoutingObjType
        Where     RoutingObjAttrRank <= v_SumScopeRank
        ;

        v_ScopeKey := FN_GET_RTOBJTYPE(v_ScopeRank,'Key',p_locale);
        v_ScopeCd := FN_GET_RTOBJTYPE(v_ScopeRank,'Code',p_locale);

     Else   -- v_SumScopeRanK = 0
        v_ScopeKey := '';

     End If;

     Case
        When p_Option = 'ScopeKey' Then
            RETURN v_ScopeKey;
        When p_Option = 'ScopeCd' Then
            RETURN v_ScopeCd;
        When p_Option = 'LevelKey' Then
            Return v_LevelKey;
        When p_Option = 'LevelCd' Then
            Return v_LevelCd;
        Else
            Return v_LevelKey;
     End Case;


     Exception
            When NO_DATA_FOUND THEN
                Return NULL;
                DBMS_OUTPUT.Put_Line('NDF');
END ;
/


-- FN_GET_DATE.sql

CREATE OR REPLACE FUNCTION FN_Get_Date
RETURN TIMESTAMP IS
Begin
   Return SYS_EXTRACT_UTC(CURRENT_TIMESTAMP);
End FN_Get_Date;
/

-- FN_GET_DEFAULT_LOC.sql

CREATE OR REPLACE FUNCTION FN_GET_DEFAULT_LOC
RETURN VARCHAR2 IS

/*
    ================================================================================
    Proc Name:      FN_DEFAULT_LOCALE
    Create Date:    2003-05-01
    Author:         L.Ling


                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-01 L.Ling        3.5    Returns the default LOCALE.
    ================================================================================
*/
                    
v_locale     PR_LOCALE.Locale%Type;

BEGIN
   SELECT  LOCALE 
   INTO    v_Locale
   FROM    PR_LOCALE
   WHERE   DEFAULTFLG = 1;
   Return  v_Locale;
End FN_GET_DEFAULT_LOC;
/


-- FN_GET_DESTTYPEID.sql

CREATE OR REPLACE FUNCTION FN_GET_DESTTYPEID (p_DestName varchar2)
return Integer
as
/*
    ================================================================================
    Proc Name:      FN_Get_DestTypeId
    Create Date:    2003-07-15
    Author:         V.Goel
    Description:    This function will return the DestTypeId based upon the
                    DestName.
    ================================================================================
*/
v_DestTypeId   pls_integer;

BEGIN
    Begin
        Select DestTypeId into v_DestTypeId from PR_DestType where DestName = nvl(p_DestName, 'Unknown');
        Exception
        
            WHEN others THEN
                RAISE_APPLICATION_ERROR (-20999,'20001 FN_Get_DestTypeId ' ||  ' '||SQLERRM);
    END;
    RETURN v_DestTypeId;  
END;
/


-- FN_GET_DIRLIST.sql

CREATE OR REPLACE FUNCTION fn_get_dirlist (
	p_ArchDest	IN VARCHAR2
	,p_FrDate	IN VARCHAR2
	,p_ToDate	IN VARCHAR2
	)
RETURN NUMBER
AS LANGUAGE JAVA
NAME 'com.ibm.bcg.db.archiver.udf.ListRestoreFiles.listRestoreFiles(java.lang.String, java.lang.String, java.lang.String) return int';
/


-- FN_GET_MDN_DISPTEXT.sql

CREATE OR REPLACE FUNCTION FN_Get_MDN_DispText (p_INITBUSINESSID  in Varchar2,
                           p_INITASMESSAGEID in Varchar2) Return VArchar2
    is
/*
    ================================================================================
    Proc Name:      FN_Get_MDN_DispText
    Create Date:    2003-05-28
    Author:         V.Goel
 
    This Function provides the DisplayTest (Status) for the first MDN only
    
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-28 V.Goel        3.5    DisplayTest (Status) for the first MDN only

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
*/    
   
v_DispText    LG_AS_Dtl.DispText%Type;
Begin
    Begin
        SELECT DispText INTO v_DispText
        FROM
        (
        SELECT DispText  From LG_AS_Dtl
        WHERE   INITBUSINESSID  = p_INITBUSINESSID
        AND     INITASMESSAGEID = p_INITASMESSAGEID
        AND     InitASMessageId <> ASMessageId
        Order By CreateDate
        )
        Where     RowNum <2
        ;
        DBMS_OUTPUT.Put_Line('DispText '||v_DispText);
        EXCEPTION
            WHEN NO_DATA_FOUND THEN  v_DispText := NULL;
    End;
    Return v_DispText;
End FN_Get_MDN_DispText;
/


-- FN_GET_NLS_SORT.sql

CREATE OR REPLACE FUNCTION FN_GET_NLS_SORT(p_Locale IN VARCHAR2)
RETURN VARCHAR2 IS
/*
    ================================================================================
    Proc Name:      FN_GET_NLS_SORT
    Create Date:    2003-09-25
    Author:         L.Ling
    ================================================================================
*/

v_NLSSORTCD PR_LOCALE.NLSSORTCD%Type;

BEGIN
   SELECT  NVL(NLSSORTCD, 'binary')
   INTO    v_NLSSORTCD 
   FROM    PR_LOCALE
   WHERE   LOCALE = p_Locale;
  
   RETURN  v_NLSSORTCD;  

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
       RETURN 'binary';
END FN_GET_NLS_SORT;
/


-- FN_GET_PARTNERID.sql

CREATE OR REPLACE FUNCTION FN_Get_PartnerId (p_BusinessId varchar2)
return number
as
/*
    ================================================================================
    Proc Name:      FN_Get_PartnerId
    Create Date:    2003-04-29
    Author:         B.Moffat
    Description:    This function will return the PartnerId based upon the
                    BusinessId.

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-29 B.Moffat      3.5    Created.

    ================================================================================

*/
v_PartnerId  PR_Partner.PartnerId%TYPE;

BEGIN
    Begin
        Select PartnerId into v_PartnerId from PR_Partner01_vw where BusinessId = p_BusinessId;
        Exception
            WHEN others THEN
                v_PartnerId := -1;
    END;
    RETURN v_PartnerId;
END;
/



-- FN_GET_PARTNERNAME.sql

CREATE OR REPLACE FUNCTION FN_Get_PartnerName (p_PartnerId Number)
return varchar2
as
/*
    ================================================================================
    Proc Name:      FN_Get_PartnerName
    Create Date:    2003-04-29
    Author:         B.Moffat
    Description:    This function will return the PartnerName based upon the
                    PartnerId.

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-29 B.Moffat      3.5    Created.

    ================================================================================

*/
v_PartnerName  PR_Partner.PartnerName%TYPE;

BEGIN
    Begin
        Select PartnerName into v_PartnerName from PR_Partner where PartnerId = p_PartnerId;
        Exception
            WHEN others THEN
                v_PartnerName := 'NotFound';
    END;
    RETURN v_PartnerName;
END;
/


-- FN_GET_ROBJREFID.sql

CREATE OR REPLACE FUNCTION FN_Get_RTObjRefId (p_RoutingObjId in Number, p_RoutingObjParentRefId in NUMBER)
return varchar2 as
/*
    ================================================================================
    Proc Name:      FN_Get_RTObjRefId
    Create Date:    2003-04-29
    Author:         B.Moffat
    Description:    This function will return the RoutingObjRefId based upon the
                    RoutingObjId and the RoutingObjParentRefId.

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-29 B.Moffat      3.5    Created.

    ================================================================================

*/
    v_RoutingObjRefId      CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE := 0;
BEGIN
    Begin
        IF p_RoutingObjParentRefId IS NULL THEN
            SELECT RoutingObjRefId INTO v_RoutingObjRefId
            FROM
               CF_RoutingObjRef_Lnk
            Where RoutingObjId = p_RoutingObjId and RoutingObjParentRefId IS NULL;
        ELSE
            SELECT RoutingObjRefId INTO v_RoutingObjRefId
            FROM
               CF_RoutingObjRef_Lnk
            Where RoutingObjId = p_RoutingObjId and RoutingObjParentRefId = p_RoutingObjParentRefId;
        END IF;
        Exception When NO_DATA_FOUND then null;
    END;
    return NVL(v_RoutingObjRefId, 0);
END FN_Get_RTObjRefId;
/


-- FN_GET_RTOBJATTRRK.sql

CREATE OR REPLACE FUNCTION FN_GET_RTOBJATTRRK
(
    p_RoutingObj      varchar2
)
return number
as
/*
    ================================================================================
    Proc Name:      FN_GET_RTOBJATTRRK
    Create Date:    2003-04-29
    Author:         B.Moffat
    Description:    This function will return the RoutingObjAttrRank based upon the
                    RoutingObjId.

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-29 B.Moffat      3.5    Created.

    ================================================================================

*/
v_RoutingObjAttrRank  number(6);


BEGIN
    IF FN_ISNumeric(p_RoutingObj) = 0 THEN
        select
            RoutingObjAttrRank into v_RoutingObjAttrRank
        FROM
            CF_RoutingObjType
        Where
            RoutingObjTypeKey = p_RoutingObj;
    ELSE
        select
            RoutingObjAttrRank into v_RoutingObjAttrRank
        FROM
            CF_RoutingObjType T
            INNER JOIN CF_RoutingObject R on R.RoutingObjTypeId = T.RoutingObjTypeId
        Where
            RoutingObjId = p_RoutingObj;
    END IF;
    -- DBMS_OUTPUT.Put_Line('p_RoutingObj='||p_RoutingObj||' Rank='||v_RoutingObjAttrRank);
    RETURN v_RoutingObjAttrRank;
    Exception
        When NO_DATA_FOUND THEN Return -1;
END FN_Get_RTObjAttrRk;
/



-- FN_GET_RTOBJATTRVL.sql

CREATE OR REPLACE FUNCTION FN_GET_RTOBJATTRVL
(
    p_RoutingObjId      number,
    p_RoutingObjRefId   number,
    p_PartnerId         number,
    p_ChannelId         number,
    p_DestTypeId        number,
    p_AttributeKey      varchar2,
    p_ValueOption       varchar2 -- values: AttrValue,AttrRank,AttrOptId,GuidelineMaplnkId,RoutingAttrLnkId,BusIdentifierId,FAMapId, ProfileId
)
RETURN Varchar2
as

/*
    ================================================================================
    Proc Name:      FN_GET_RTObjAttrVl
    Create Date:    2003-04-29
    Author:         B.Moffat
    Description:    This function will return either the highest ranking routing
                    object attribute value, or return the rank itself based up on the
                    p_ValueOption.


                    p_ValueOption
                    -------------
                    AttrValue - Return the highest ranking attribute value
                    AttrRank  - Return the highest rank itself
                    AttrOptId - Return the highest rank AttrOptId
                    GuidelineMapLnkId - Return the highest rank GuildelineMapLnkId
                    RoutingAttrLnkId - Return the highest rank RoutingAttrLnkId
                    BusIdentifierId - Return the highest rank BusIdentifierId

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
*/


    v_AttrValue             PR_RoutingAttr_Lnk.AttrValue%TYPE;
    v_RoutingObjAttrRank    CF_RoutingObjType.RoutingObjAttrRank%TYPE;
    v_RoutingAttrRank       CF_RoutingObjType.RoutingObjAttrRank%TYPE;
    v_AttrOptId             PR_RoutingAttr_Lnk.AttrOptId%TYPE;
    v_GuidelineMapId        PR_RoutingAttr_Lnk.GuidelineMapId%TYPE;
    v_FaMapID               PR_RoutingAttr_Lnk.FAMapId%TYPE;
    v_ProfileID             PR_RoutingAttr_Lnk.ProfileId%TYPE;
    v_RoutingAttrLnkId      PR_RoutingAttr_Lnk.RoutingAttrLnkId%Type;
    v_BusIdentifierId       PR_BusIdentifier.BusIdentifierId%TYPE;
    v_ScopeKey              varchar2(100);
    v_Levelkey              varchar2(100);
    v_LevelCd               varchar2(100);

BEGIN

   select FN_GET_RTOBJATTRRK(p_RoutingObjId) into v_RoutingObjAttrRank from dual;
   Select AttrValue,
           RoutingAttrLnkId,
           AttrOptId,
           GuidelineMapId,
           FAMapId,
           ProfileId,
           RoutingAttrRank,
           BusIdentifierId
    Into   v_AttrValue,
           v_RoutingAttrLnkId,
           v_AttrOptId,
           v_GuidelineMapId,
           v_FAMapId,
           v_ProfileId,
           v_RoutingAttrRank,
           v_BusIdentifierId
    From
        (select
                CASE
                    WHEN GuidelineMapId IS NOT NULL Then to_char(GuidelineMapId)
                    WHEN FAMapId IS NOT NULL Then to_char(FAMapId)
                    WHEN ProfileId IS NOT NULL Then to_char(ProfileId)
                    WHEN BusIdentifierId IS NOT NULL Then to_char(BusIdentifierId)
                    WHEN Ra.AttrOptId IS NULL THEN NVL(Ra.AttrValue, 'N/A')
                    ELSE NVL((Select AttrValueKey from CF_RoutingAttrOpt Ao Where Ao.AttrOptId = Ra.AttrOptId), 'N/A')
                END AttrValue,
                Ra.RoutingAttrLnkId,
                Ra.AttrOptId,
                Ra.GuidelineMapId,
                Ra.FaMapId,
                Ra.ProfileId,
                Ra.BusIdentifierId,
                TO_NUMBER(NVL((CASE
                    When Ra.PartnerId IS NULL AND Ra.ChannelId IS NULL and Ra.DestTypeId IS NULL
                            Then FN_GET_RTOBJATTRRK('Global')
                END),0)  +
                NVL((CASE When Ra.PartnerId = p_PartnerId And Ra.ChannelId IS NULL Then FN_GET_RTOBJATTRRK('Partner') END),0) +
                NVL((CASE When Ra.ChannelId = p_ChannelId And Ra.PartnerId = p_PartnerId And Ra.DestTypeId IS NULL Then FN_GET_RTOBJATTRRK('Channel')  END),0) +
                NVL((CASE When Ra.DestTypeId = p_DestTypeId and Ra.ChannelId = p_ChannelId
                                            Then FN_GET_RTOBJATTRRK('DestType') END),0))
                RoutingAttrRank

        FROM
            PR_RoutingAttr_Lnk  Ra
            INNER JOIN CF_RoutingAttr A on A.RoutingAttrId = Ra.RoutingAttrId
        Where
            Ra.RoutingObjRefId = p_RoutingObjRefId and A.AttributeKey = p_AttributeKey
        Order by RoutingAttrRank desc
        )
    Where rownum < 2 and RoutingAttrRank > 0;

    If NVL(v_RoutingAttrRank, 0) = 0 Then
        Return Null;
    Else
        v_RoutingAttrRank := v_RoutingAttrRank + v_RoutingObjAttrRank;

        Case
            When p_ValueOption = 'AttrValue' Then
                RETURN v_AttrValue;
            When p_ValueOption = 'AttrRank' Then
                RETURN to_char(v_RoutingAttrRank);
            When p_ValueOption = 'AttrOptId' Then
                Return v_AttrOptId;
            When p_ValueOption = 'GuidelineMapId' Then
                Return v_GuidelineMapId;
	    When p_ValueOption = 'FAMapId' Then
                Return v_FAMapId;
            When p_ValueOption = 'ProfileId' Then
                Return v_ProfileId;
            WHEN p_ValueOption = 'BusIdentifierId' THEN
                Return v_BusIdentifierId;
            When p_ValueOption = 'RoutingAttrLnkId' Then
                Return v_RoutingAttrLnkId;
         Else
            RETURN v_AttrValue;
        End Case;
    End If;

    Exception
        When NO_DATA_FOUND THEN
            Return NULL;

END ;
/


-- FN_GET_RTOBJID.sql

CREATE OR REPLACE FUNCTION FN_GET_RTOBJID
    (
    p_RoutingObjTypeKey  in varchar2,
    p_RoutingObjCd       in varchar2,
    p_RoutingObjVer      in varchar2
    )
return varchar2 is

/*
    ================================================================================
    Proc Name:      FN_GET_RTOBJID
    Create Date:    2003-04-29
    Author:         B.Moffat
    Description:    This function will return the RoutingObjId based upon the
                    RoutingObjTypeKey, RoutinbObjCd, and RoutingObjVer.



                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 B.Moffat      3.5    Created.

    ================================================================================
    
*/
    v_RoutingObjId      CF_RoutingObject.RoutingObjId%TYPE := 0;
BEGIN
    Begin
        SELECT RoutingObjId INTO v_RoutingObjId
        FROM
               CF_RoutingObject01_vw
        Where
            RoutingObjTypeKey  = p_RoutingObjTypeKey
            and RoutingObjCd = p_RoutingObjCd
            and RoutingObjVer = p_RoutingObjVer;
        Exception When NO_DATA_FOUND then null;
    END;
    return NVL(v_RoutingObjId, 0);
END FN_GET_RTOBJID;
/



-- FN_GET_RTOBJNAME.sql

CREATE OR REPLACE FUNCTION FN_GET_RTOBJNAME
(
   p_RoutingObjCD      IN VARCHAR2, 
   p_RoutingObjVer     IN VARCHAR2, 
   p_RoutingObjTypeKey IN VARCHAR2 
) Return Varchar2

as

/*
    ================================================================================
    Proc Name:      FN_GET_RTOBJNAME
    Create Date:    2003-05-21
    Author:         L.Ling


                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-21 L.Ling        3.5    Get routing object name.

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
*/

    v_RoutingObjName    CF_RoutingObject.RoutingObjName%type;
Begin
   Begin
       SELECT  RoutingObjName
       INTO    v_RoutingObjName
       FROM    CF_RoutingObject o
       INNER JOIN  CF_RoutingObjType t ON o.RoutingObjTypeId = t.RoutingObjTypeId AND t.RoutingObjTypeKey = p_RoutingObjTypeKey      
       WHERE   RoutingObjCD = p_RoutingObjCD
       AND     RoutingObjVer = p_RoutingObjVer; 

       Exception
           When NO_DATA_FOUND THEN v_RoutingObjName := NULL;
   End;            
   RETURN v_RoutingObjName;
END ;
/


-- FN_GET_RTOBJTYPE.sql

CREATE OR REPLACE FUNCTION FN_GET_RTOBJTYPE
(
    p_AttrRank Number,
    p_OptionFlg Varchar2,
    p_locale varchar2
) Return Varchar2

as

/*
    ================================================================================
    Proc Name:      FN_Get_RTObjType
    Create Date:    2003-05-09
    Author:         J.Chen


                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-09 J.Chen        3.5    Get_Routing Object Type Key or Code by given AttrRank
    2003-06-12 L.Ling        3.5    Changed view from CF_RoutingObjType_VW to CF_RoutingObjType_LOC_VW.
    2004-03-05 S.Narain      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
*/


--------------------------------------
-- Get_Routing Object Type Key or Code
--------------------------------------

    v_RoutingObjTypeKey CF_RoutingObjType.RoutingObjTypeKey%type;
    v_Resourcekey  CF_RoutingObjType.Resourcekey%type;

Begin
    Begin
         Select    RoutingObjTypeKey,
                   Resourcekey
         Into      v_RoutingObjTypeKey,
                   v_Resourcekey
         From      CF_RoutingObjType
         Where     RoutingObjAttrRank = p_AttrRank;
    End;
    Case
         When p_OptionFlg = 'Code' Then
              Return v_Resourcekey;
         When p_optionFlg = 'Key' Then
              Return v_RoutingObjTypeKey;
         Else
              Return v_RoutingObjTypeKey;
    End Case;


    Exception
        When NO_DATA_FOUND THEN
            Return NULL;
            DBMS_OUTPUT.Put_Line('NDF');
END ;
/


-- FN_GET_SIZECATID.sql

CREATE OR REPLACE FUNCTION FN_GET_SIZECATID
(p_Msglength in NUMBER) 
return varchar2 is
id  DB_MsgLength.SizeCategoryID%Type;
BEGIN
   IF p_Msglength > 0 then
   begin
       select SizeCategoryID into id from DB_MsgLength M
       where SizeCategory =
       (select MIN(SizeCategory) from DB_MsgLength  where SizeCategory >= p_MsgLength and SizeCategory >= 0)
       ;
   Exception
   when no_data_found then
       id := 'N/A';
   end;
   ELSE
       id := 'N/A';
   END IF;
   Return id;
END;
/

-- FN_INTERVAL2MS.sql

CREATE OR REPLACE FUNCTION FN_Interval2Ms (p_Interval in Interval Day To Second) return integer is
-- This function Figure out document duration based upon receiveTS and endState TS
--    Modified 2006-05-01 Shishir N      6.0   PMR 08997999846

   v_DocDuration    interval day(2) to second(6);
   v_DocDuration_ms    pls_integer;
   v_length            pls_integer;
   v_duration_String   varchar2(50);
   v_days              pls_integer;
   v_hours             pls_integer;
   v_minutes           pls_integer;
   v_seconds           pls_integer;
   v_millie            number;
Begin
   IF p_Interval is not null then
        v_days    :=                EXTRACT (DAY FROM p_Interval);
        v_hours   := v_days*24    + EXTRACT (HOUR FROM p_Interval);
        v_minutes := v_hours*60   + EXTRACT (MINUTE FROM p_Interval);
        v_seconds := v_minutes*60 + EXTRACT (SECOND FROM p_Interval);
        v_DocDuration_MS := v_seconds*1000;
    Else
       v_DocDuration_MS := 0;
    End If;
    Return v_DocDuration_MS;
End FN_Interval2Ms;
/


-- FN_ISNumeric.sql

CREATE OR REPLACE FUNCTION FN_ISNumeric (p_Var IN VARCHAR2)
/*
    =============================================================================
    Proc Name:      FN_ISNUMERIC
    Create Date:    2003-04-22
    Author:         B.Moffat
    Description:    This function will return 1=TRUE if the variable passed
                    is numeric, and 0=FALSE if it is not numeric.


                            ::::::::::::::
                            C H A N G E S
                            ::::::::::::::

        Example:  IF FN_ISNumeric('ABcde34') = 1 THEN
                      DBMS_OUTPUT.Put_Line('This is a number');
                  ELSE
                      DBMS_OUTPUT.Put_Line('This is NOT a number');
                  END IF;

    ---------- ------------ ------- -----------------------------------------------
    DATE       NAME         VER.    DESCRIPTION
    ---------- ------------ ------- -----------------------------------------------
    2003-04-22 B.Moffat     3.50    Created.


=============================================================================
*/
RETURN NUMBER IS
    v_Nbr NUMBER;
BEGIN
    v_Nbr :=TO_NUMBER(p_Var);
    RETURN 1;
    EXCEPTION
        WHEN OTHERS THEN RETURN 0;
END;
/


-- FN_PAGING_LOGIC.sql

CREATE OR REPLACE Function FN_PAGING_LOGIC(
        p_pageNumber IN PLS_INTEGER,
        p_pageSize   IN PLS_INTEGER,
        p_sqlstmt    IN VARCHAR2
        ) Return Varchar2 
        AS
/*
    =============================================================================
    Proc Name:      DB_PAGING_LOGIC
    Create Date:    2003-06-20
    Author:         V.Goel

    Purpose: This proc wraps around the paging logic to a sql statement

    Data Definition:
        p_pageNumber : Page number
        p_pageSize   : Page size
        p_sqlstmt    : Sql statement that need to be wrapped with paging logic

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------ ------ ---------------------------------------------
    DATE       NAME         VER     DESCRIPTION
    ---------- ------------ ------ ---------------------------------------------
    2003-06-20 V Goel       3.5     Created
    2003-07-07 V Goel       3.5     Change from procedure to function 
    =============================================================================
*/

v_startRow  BINARY_INTEGER Default 0;
v_endRow    BINARY_INTEGER Default 0;
v_sqlstmt Varchar2 (4000);
BEGIN
  IF NVL(p_pageNumber,0) IN (0, 1) THEN
    v_startRow := 1;
    v_endRow   := NVL(p_PageSize,10);
  ELSE
    v_startRow := ((NVL(p_PageSize,10) * p_pageNumber)+1) -  NVL(p_PageSize,10) ;
    v_endRow   :=   NVL(p_PageSize,10) * p_pageNumber;
  END IF;
    v_sqlstmt := 'SELECT E.* FROM (SELECT ROWNUM rnbr, F.*  FROM ('||p_sqlstmt||') F ) E WHERE rnbr BETWEEN '||v_startRow||' AND '||v_endRow;
    Return v_sqlstmt;
END FN_PAGING_LOGIC;
/


-- FN_SCHEMA_STATUS.sql

CREATE OR REPLACE FUNCTION FN_SCHEMA_STATUS
RETURN VARCHAR2
IS


/*
    ================================================================================

    Proc Name:      FN_COMP_DB_COUNT
    Create Date:    2008-08-19
    Author:         BHARANI J

    Description:    This function returns VALID OR INVALID.  CHECKS IF ANY OF THE 
    DBOBJECT is INVALID.  IF it is Invalid then return INVALID. ELSE VALID.


                       ::::::::::::::::::::::::::::::

                             M O D I F I C A T I O N

                       ::::::::::::::::::::::::::::::


    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
   .
    ================================================================================
*/

    v_invalid_count INTEGER;
    v_return VARCHAR2(30);
    

   BEGIN
    
    -- dbms_utility.compile_schema(P_SCHEMA);

    SELECT COUNT(1) into v_invalid_count FROM USER_OBJECTS where status='INVALID';
    	 
   IF v_invalid_count = 0 

   THEN


    v_return :='VALID';

   ELSE

    v_return :='INVALID';

   END IF;

   Return v_return;

   END FN_SCHEMA_STATUS;
/

-- FN_SET_DEFORNONDEF.sql

CREATE OR REPLACE FUNCTION FN_SET_DEFORNONDEF
(
    p_certsetid NUMBER,
    p_partnerid NUMBER,
    p_searchflg NUMBER       -- 0 - HubOperator 1-- InternalPartner 2 -- ExternalPartner
)
RETURN  NUMBER IS 

/*
	================================================================================
	Proc Name:       FN_SET_DEFORNONDEF
	Create Date:     2007-09-17 15:16:24
	Author:          BHARANI J 
	
	USED BY PR_CERTS_AH9,PR_CERT_SETS_AH1

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-25 Amit Kumar    6.1.1  Created for Multiple Certificate Feature

	================================================================================
	1 --> DEFAULT  0  --> NOT DEFAULT
*/

v_default NUMBER DEFAULT 0;
v_temp NUMBER DEFAULT 0;

BEGIN 


	IF p_searchflg = 1 THEN     -- INTERNAL PARTNER  

		SELECT COUNT(*) INTO v_temp FROM
		      	PR_CERT_SET_USAGE csu
		      	INNER JOIN PR_PARTNERPAIR_RTOBJPAIR_LNK prl on csu.partnerrtobjlnkid = prl.partnerrtobjlnkid 
		      	INNER JOIN PR_PARTNER_PAIR pp on prl.partnerpairid=pp.partnerpairid
		      	INNER JOIN PR_ROUTINGOBJ_PAIR RP on prl.routingobjpairid = rp.routingobjpairid 
		      	WHERE	pp.frpartnerid = p_partnerid AND
		      		    pp.topartnerid is null AND
		      		    rp.frroutingobjid is null AND
		      		    rp.toroutingobjid is null AND
		      		    (csu.certsetid_sslclient = p_certsetid OR
		      		    csu.certsetid_signing = p_certsetid)
		 ;
	      
	      
	END IF;


	IF p_searchflg = 2 THEN     -- EXTERNAL PARTNER 

    		SELECT COUNT(*) INTO v_temp FROM
	      		PR_CERT_SET_USAGE csu
	      		INNER JOIN PR_PARTNERPAIR_RTOBJPAIR_LNK prl on csu.partnerrtobjlnkid = prl.partnerrtobjlnkid 
	      		INNER JOIN PR_PARTNER_PAIR pp on prl.partnerpairid=pp.partnerpairid
	      		INNER JOIN PR_ROUTINGOBJ_PAIR RP on prl.routingobjpairid = rp.routingobjpairid 
	      		WHERE pp.frpartnerid is null AND
	      	    		pp.topartnerid = p_partnerid AND
	      	    		rp.frroutingobjid is null AND
	      	    		rp.toroutingobjid is null AND
	      	    		csu.certsetid_encryption = p_certsetid
	       ;
	      
	
	END IF;

	IF p_searchflg = 0 THEN -- Hub Operator

  		SELECT COUNT(*) INTO v_temp FROM
	      		PR_CERT_SET_USAGE csu
	      		INNER JOIN PR_PARTNERPAIR_RTOBJPAIR_LNK prl on csu.partnerrtobjlnkid = prl.partnerrtobjlnkid 
	      		INNER JOIN PR_PARTNER_PAIR pp on prl.partnerpairid=pp.partnerpairid
	      		INNER JOIN PR_ROUTINGOBJ_PAIR RP on prl.routingobjpairid = rp.routingobjpairid 
	      		WHERE pp.frpartnerid is null AND
	      	    		pp.topartnerid is null AND
	      	    		rp.frroutingobjid is null AND
	      	    		rp.toroutingobjid is null AND
	      	    		(csu.certsetid_sslclient = p_certsetid OR
	      	    		csu.certsetid_signing = p_certsetid)
	      	;
	      
	
	END IF;
IF v_temp > 0 THEN
		v_default := 1;
	ELSE
		v_default := 0;
END IF;
	      	
RETURN v_default;	
	
END FN_SET_DEFORNONDEF;
/

-- FN_SET_VALORINVAL.sql

CREATE OR REPLACE FUNCTION FN_SET_VALORINVAL
(
    p_certsetid INTEGER
)
RETURN INTEGER
AS

/*
	================================================================================
	Proc Name:       FN_SET_VALORINVAL
	Create Date:     2007-09-25 15:16:24
	Author:          BHARANI J

	USED BY : PR_CERT_SETS_AH1,PR_CERT_SETS_AH2
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-25 BHARANI J     6.1.1  Created for Multiple Certificate Feature

	================================================================================
	Return Code : 0 = VALID,
	1 = INVALID There is no primary Certificate in the set,
	2 = INVALID Primary Certificate in the set is disabled,
	3 = INVALID SET DISABLED
	4 = INVALID Primary Expired AND there is no secondary
	5 = INVALID Both Primary AND secondary Expired
	6 = INVALID Primary Valid AND secondary Expired.

*/

 v_valid INTEGER;
 v_temp VARCHAR2(10);

BEGIN

	-- Check whether the set has primary certificate.

	IF p_certsetid IS NOT NULL THEN

		BEGIN

		SELECT 'VALID' into v_temp
		FROM  pr_cert a ,pr_cert_set b,pr_cert_set_lnk l
		WHERE a.certid = l.certid
		AND b.certsetid = l.certsetid
		AND l.certsetid = p_certsetid
  		AND a.prisecflg = 1
  		AND coalesce(a.deleteflag,0) <> 1
  		;

  		EXCEPTION
  		WHEN NO_DATA_FOUND THEN
  		NULL;
  		END;

  		IF v_temp is null THEN
  		 	v_valid := 1;
		ELSE
			v_valid := 0;
			v_temp := null;
		END IF;

		IF v_valid = 0 THEN

			-- Check whether the Primary Certificate in the set is enabled.
			BEGIN

			SELECT 'VALID' into v_temp
			FROM  pr_cert a ,pr_cert_set b,pr_cert_set_lnk l
			WHERE a.certid = l.certid
			AND b.certsetid = l.certsetid
			AND l.certsetid = p_certsetid
	  		AND a.prisecflg = 1
	  		AND a.enabledflg=1
	  		AND coalesce(a.deleteflag,0) <> 1;
	  		
	  		EXCEPTION
		  	WHEN NO_DATA_FOUND THEN
		  	NULL;
  	        	END;

			IF v_temp is null THEN
  				 v_valid := 2;
			ELSE
				 v_valid := 0;
			 	v_temp := null;
			END IF;

		END IF;


		IF v_valid = 0 THEN

			-- Check whether the Primary Certificate in the set is enabled.
			BEGIN
			SELECT 'VALID' into v_temp
			FROM  pr_cert a ,pr_cert_set b,pr_cert_set_lnk l
			WHERE a.certid = l.certid
			AND b.certsetid = l.certsetid
			AND l.certsetid = p_certsetid
	  		AND a.prisecflg = 1
	  		AND a.enabledflg=1
	  		AND b.status=1
	  		AND coalesce(a.deleteflag,0) <> 1;
	  		
	  		EXCEPTION
			
			WHEN NO_DATA_FOUND THEN
			NULL;
  	        	END;

			IF v_temp is null THEN
  			 	v_valid := 3;
			ELSE
			 	v_valid := 0;
				 v_temp := null;
			END IF;
		END IF;


		-- Check whether the Primary Certificate expired.

		IF v_valid = 0 THEN

			BEGIN
			SELECT
			CASE when (a.validto - systimestamp) > (systimestamp-systimestamp) THEN 'VALID' else 'INVALID' END AS STATUS into v_temp
			FROM  pr_cert a ,pr_cert_set b,pr_cert_set_lnk l
			WHERE a.certid = l.certid
			AND b.certsetid = l.certsetid
			AND l.certsetid = p_certsetid
			AND a.prisecflg = 1
			AND coalesce(a.deleteflag,0) <> 1
			;
			
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
			NULL;
  	        	END;

		

			IF v_temp = 'INVALID' THEN

				BEGIN
			
				-- Check whether the secondart cert exists
				
				

				SELECT 'x'  into v_temp
				FROM  pr_cert a ,pr_cert_set b,pr_cert_set_lnk l
				WHERE a.certid = l.certid
				AND b.certsetid = l.certsetid
				AND l.certsetid = p_certsetid
				AND a.prisecflg = 2
				AND coalesce(a.deleteflag,0) <> 1;
										
				EXCEPTION
				WHEN NO_DATA_FOUND THEN
				v_valid := 4; 		  	
  	       			END;	       		
  	       		
  	      			IF v_temp <> 'x' THEN
  	      				v_valid := 4; 
				
				ELSE
			
					-- If sec cert exists THEN check whether it is expired or not.
				  	      			
					BEGIN
										
					SELECT
					CASE when (a.validto - systimestamp) > (systimestamp-systimestamp) THEN 'VALID' else 'INVALID' END AS STATUS INTO v_TEMP
					FROM  pr_cert a ,pr_cert_set b,pr_cert_set_lnk l
					WHERE a.certid = l.certid
					AND b.certsetid = l.certsetid
					AND l.certsetid = p_certsetid
					AND a.prisecflg = 2
					AND coalesce(a.deleteflag,0) <> 1
					;
									
									
										
					EXCEPTION
					WHEN NO_DATA_FOUND THEN
					NULL;
					END;
								
						IF v_temp = 'INVALID' THEN
											
							v_valid := 5;
					  	      				
						ELSE
				
							v_valid := 0;
						v_temp := null;
				
		   				END IF;
		   		END IF;

			ELSE
		
			 	v_valid := 0;
			 	v_temp := null;
			END IF;

		END IF;
	
	/* IF v_valid = 0 THEN -- This condition not required.

	-- CHECK

		BEGIN
		SELECT
		CASE when (a.validto - systimestamp) > (systimestamp-systimestamp) THEN 'VALID' else 'INVALID' END AS STATUS INTO v_TEMP
		FROM  pr_cert a ,pr_cert_set b,pr_cert_set_lnk l
		WHERE a.certid = l.certid
		AND b.certsetid = l.certsetid
		AND l.certsetid = p_certsetid
		AND a.prisecflg = 2
		AND coalesce(a.deleteflag,0) <> 1
		;
		
		EXCEPTION
		WHEN NO_DATA_FOUND THEN
		NULL;
  	        END;

			IF v_temp = 'INVALID' THEN
			
					 v_valid := 6;

			ELSE
					 v_valid := 0;
			END IF;
	END IF; */

	ELSE

	 v_valid := null ;

	END IF;
	
RETURN v_valid;

END FN_SET_VALORINVAL;
/

-- FN_TenMinuteDate.sql

Create or replace Function FN_TenMinuteDate 
            (p_ConvDate in Date) 
            return DATE 
            IS
/*
    =============================================================================
    Proc Name:      FN_TenMinuteDate
    Create Date:    2003-07-12
    Author:         V.Goel
    Purpose: This function will return the date at 10 minute intervals.  
             Example: 2003-01-20 15:24:00 = 2003-01-20 15:20:00
    =============================================================================
*/
Begin
    return (Trunc(p_ConvDate, 'HH24') + (trunc(to_number(to_char(p_ConvDate, 'MI'))/10)*10)/1440);
End FN_TenMinuteDate;
/



-- FN_TIMESTAMPDIFF.sql

CREATE OR REPLACE FUNCTION FN_TimeStampDiff (p_RecTS IN TIMESTAMP, p_EndStateTS IN TIMESTAMP) RETURN INTEGER IS
-- This function Figure out document duration based upon receiveTS and endState TS.
-- Note: receiveTS is a lower value and endStateTS is higher value
--	Modified 2006-05-01 Shishir N	6.0	PMR 08997999846

	v_DocDuration	interval day(2) to second(6);
	v_DocDuration_ms	NUMBER(12);
	v_length		pls_integer;
	v_days			pls_integer;
	v_hours			pls_integer;
	v_minutes		pls_integer;
	v_seconds		pls_integer;
	Numeric_Overflow	EXCEPTION;
	-- Fix for JR2731
	Precision_Too_Small     EXCEPTION;
	-- End of fix for JR27131
	PRAGMA EXCEPTION_INIT (Numeric_Overflow, -01426);
	-- Fix for JR2731	
	PRAGMA EXCEPTION_INIT (Precision_Too_Small, -01873);
	-- End of fix for JR27131	
Begin
	BEGIN
	IF p_EndStateTS is not null then
		v_DocDuration := p_EndStateTS - p_RecTS ;
		v_days	:= EXTRACT (DAY FROM v_DocDuration);
		v_hours	:= v_days*24 + EXTRACT (HOUR FROM v_DocDuration);
		v_minutes := v_hours*60	+ EXTRACT (MINUTE FROM v_DocDuration);
		v_seconds := v_minutes*60 + EXTRACT (SECOND FROM v_DocDuration);
		v_DocDuration_MS := v_seconds*1000;

	Else
		v_DocDuration_MS := 0;
	End If;
	EXCEPTION
		WHEN Numeric_Overflow THEN
			v_DocDuration_MS := 0;
		-- Fix for JR2731		
		WHEN Precision_Too_Small THEN
			v_DocDuration_MS := 0;
		-- End of fix for JR27131		
	END;
	Return v_DocDuration_MS;
End FN_TimeStampDiff;
/


-- FN_TS2Date.sql

CREATE OR REPLACE FUNCTION FN_TS2Date (p_TimeStamp in Timestamp default SYS_EXTRACT_UTC(CURRENT_TIMESTAMP) )
RETURN Date IS
v_date date := p_TimeStamp;
Begin
    Return v_date;
End FN_TS2Date;
/


-- # ============================================================== # 
-- Create Procedures 
-- # ============================================================== # 

-- AR_CREATE_DIRECTORY.sql

CREATE OR REPLACE PROCEDURE AR_CREATE_DIRECTORY
	(
	p_ReturnCd OUT	INTEGER
	,p_DebugFlg IN	INTEGER
	,RC1	OUT	DB_REF.RCT
	,p_dirObjName IN VARCHAR2
	,p_PathName	IN VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:	AR_CREATE_DIRECTORY
	Create Date:	2008-07-31 14:07:19
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-10-21 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------
	v_RowCnt	integer Default 0;
	v_BeginTS	timestamp Default FN_Get_Date;  -- Performance related variables
	v_ProcName	varchar2(30)	Default 'AR_CREATE_DIRECTORY';
	v_ExceptionFlg		INTEGER Default 0;
	v_Stmt		VARCHAR2(1024);

	v_ParamsTxt     varchar2(4000) Default
			'p_DebugFlg=>'||p_DebugFlg||CHR(10)
			 		||', p_dirObjName=>'||p_dirObjName||CHR(10)
			 		||', p_PathName=>'||p_PathName||CHR(10)
	;

BEGIN
	p_ReturnCd := 0;

	--- grant write permission on the path
	dbms_java.grant_permission(sys_context('USERENV', 'CURRENT_SCHEMA')
			   ,'SYS:java.io.FilePermission'
			   ,p_PathName
			   ,'read,write,delete,execute');

	-- create the directory
	v_RowCnt := createDir(p_PathName);

	--- create Oracle directory object
	v_stmt := 'CREATE OR REPLACE DIRECTORY ' || p_dirObjName || ' AS ''' || p_PathName || '''';

	EXECUTE IMMEDIATE v_stmt;

	OPEN RC1 FOR SELECT v_RowCnt FROM DUAL;
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END AR_CREATE_DIRECTORY;
/


-- AR_EXPORT_CERT.sql

CREATE OR REPLACE PROCEDURE AR_EXPORT_CERT
	(
	 p_ReturnCd 	OUT pls_INTEGER
	, p_DebugFlg 	IN pls_INTEGER
	, p_Date	IN VARCHAR2
	, p_userid IN 	VARCHAR2
	, p_ArchiveDest IN VARCHAR2
	, p_TaskID IN	NUMBER
	)

AS

/*
	================================================================================
	Name:	   AR_EXPORT_CERT
	Create Date:	2008-10-24
	Author	  Som Priani
	This Procedure exports data from  LG_CERT_ARCHIVE table

			 ::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
			 ::::::::::::::::::::::::::::::
	---------- ------------	-------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2008-10-24 S.Priani	6.2	F104161
	================================================================================

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_ProcName		VARCHAR2(30)	default 'AR_EXPORT_CERT';
	v_SqlText		Varchar2(4000);
	v_SqlTextStatic		Varchar2(3800)	DEFAULT '';
	v_BeginTS		TIMESTAMP	DEFAULT fn_Get_Date;

	v_ReturnCd		INTEGER;
	v_CutOffDate		VARCHAR2(10);
	v_TaskName		VARCHAR2(8)	DEFAULT 'CERTARCH';
	v_JobName		VARCHAR2(30)	DEFAULT TO_CHAR(v_BeginTS,'YYYYMMDDHH24MISSFF');
	v_Flag			INTEGER		DEFAULT 0;
	v_TableName		VARCHAR2(30)	DEFAULT 'WPGCER' || v_JobName;
	v_FileNamePrefix	Varchar2(255);
	v_Duration		NUMBER(9,2);
	v_StartTime		INTEGER;
	v_EndTime		INTEGER;
	
	v_RowCount		NUMBER;

	v_ExpDmpFile	VARCHAR2(255) ;
	v_DmpDir	VARCHAR2(30) := 'WPGDBDATA';
	NotSupported		EXCEPTION;
BEGIN
	BEGIN
		SELECT	1 INTO v_Flag
		FROM	v$version
		WHERE	banner LIKE '%9i%'
		AND	rownum < 2
		;
		RAISE NotSupported;
	EXCEPTION
		WHEN No_Data_Found THEN
			Null;

	END;
	p_ReturnCd := 1;
	v_StartTime := DBMS_UTILITY.get_time;
	v_DmpDir := UPPER(p_ArchiveDest);
	v_CutOffDate := REPLACE(p_Date,'-','');

	INSERT INTO lg_archiveinfo
		   ( actionname, userid,
			cutoffdate, partner,
			packagename,
			packagever,
			protocolname,
			protocolver,
			processname,
			processver
		   )
		VALUES ('CERT ARCHIVE Export', p_userid, v_BeginTS,
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			NULL
		  );

	v_FileNamePrefix := RTRIM(v_TaskName ||'.'|| v_CutOffDate ||'.'|| COALESCE(p_TaskID,0) || '.' || v_JobName);
	v_ExpDmpFile := v_FileNamePrefix || '.dmp';

	v_RowCount := 0;
	v_sqlText := 'SELECT COUNT(*) FROM LG_CERT_ARCHIVE';
	EXECUTE IMMEDIATE v_SqlText INTO v_RowCount;

	IF v_RowCount > 0 THEN
		-- create External table using Oracle DataPump access driver
		v_sqlText := 'CREATE TABLE ' || v_TableName 
				|| ' ORGANIZATION EXTERNAL ( TYPE ORACLE_DATAPUMP DEFAULT DIRECTORY ' || v_DmpDir
				|| ' LOCATION (''' || v_ExpDmpFile || ''') ) '
				|| ' AS SELECT  * '
				|| ' FROM LG_CERT_ARCHIVE '
		;

		-- this will export the data to a file
		EXECUTE IMMEDIATE v_SqlText;

		-- Drop the table
		EXECUTE IMMEDIATE 'DROP TABLE ' || v_TableName;
	END IF;
	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	dbms_output.put_line ( RPAD('LG_CERT_ARCHIVE', 30, ' ') || to_char( v_RowCount,'999999990' ) ||'   '|| SUBSTR('ARC',1,3)||'| '|| to_char( v_Duration, '999990.00' ));

	lg_arch_task_exportDbI(v_returncd, p_DebugFlg, p_TaskId, v_CutOffDate, 0, v_RowCount, v_FileNamePrefix, v_BeginTS);
	COMMIT;
	p_ReturnCd := 0;
	
EXCEPTION
	WHEN NotSupported THEN
		p_ReturnCd := 0;
	WHEN OTHERS THEN
		dbms_output.put_line('CertArch-Error:'||SQLERRM);
		ROLLBACK;
		RAISE_APPLICATION_ERROR (-20999, '20002 AR_EXPORT_CERT ' || SQLERRM);
END  AR_EXPORT_CERT;
/


-- AR_EXPORT_DATA.sql

CREATE OR REPLACE PROCEDURE AR_EXPORT_DATA(
	p_ReturnCd		OUT	pls_INTEGER
	,p_DebugFlg		IN	pls_INTEGER
	,p_SubVolName		IN	VARCHAR2
	,p_partner		IN	VARCHAR2
	,p_packagename		IN	VARCHAR2
	,p_packagever		IN	VARCHAR2
	,p_protocolname		IN	VARCHAR2
	,p_protocolver		IN	VARCHAR2
	,p_processname		IN	VARCHAR2
	,p_processver		IN	VARCHAR2
	,p_userid		IN	VARCHAR2
	,p_ArchiveDest		IN	VARCHAR2
	,p_TaskID		IN	NUMBER
	,p_ParameterID		IN	INTEGER
	)
AS

/*
	================================================================================
	Name:	   AR_EXPORT_DATA
	Create Date:	2008-10-24
	Author	  Som Priani
	This Procedure exports data from LG_MSG_ARCHIVE table

			 ::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
			 ::::::::::::::::::::::::::::::
	---------- ------------ -------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------  ------------ -------	---------------------------------------------
	2008-10-24 S.Priani	6.2	F104161
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_ProcName		Varchar2(30)	default 'AR_EXPORT_DATA';
	v_SqlText		Varchar2(4000);
	v_SqlTextStatic		Varchar2(3800)	DEFAULT '';
	v_BeginTS		TIMESTAMP	DEFAULT fn_Get_Date;

	v_ReturnCd		INTEGER;
	v_TaskName		VARCHAR2(8)	DEFAULT 'MSGARCH';
	v_JobName		VARCHAR2(20)	;
	v_Flag			INTEGER		DEFAULT 0;
	v_TableName		VARCHAR2(30)	;
	v_FileNamePrefix	VARCHAR2(255);
	v_Duration		NUMBER(9,2)	DEFAULT 0;
	v_StartTime		INTEGER;
	v_EndTime		INTEGER;

	v_Parallel		NUMBER(6)		DEFAULT 2;
	v_RowCount		NUMBER;

	v_ExpDmpFile	VARCHAR2(255) ;
	v_DmpDir	VARCHAR2(30) := 'WPGDBDATA';
	NotSupported		EXCEPTION;
BEGIN
	BEGIN
		SELECT	1 INTO v_Flag
		FROM	v$version
		WHERE	banner LIKE '%9i%'
		AND	rownum < 2
		;
		RAISE NotSupported;
	EXCEPTION
		WHEN No_Data_Found THEN
			Null;

	BEGIN
		SELECT	COALESCE(CAST(propvalue AS INT), v_Parallel)
		INTO	v_Parallel
		FROM	DB_SYSPROPERTIES
		WHERE	propname = 'bcg.archive.db.paralellism'
		;

	EXCEPTION
		WHEN No_Data_Found THEN
			Null;  --- USe the default files
	END;

	END;
	p_ReturnCd := 1;
	v_StartTime := DBMS_UTILITY.get_time;
	v_DmpDir := UPPER(p_ArchiveDest);

	INSERT INTO lg_archiveinfo
		   ( actionname, userid,
			cutoffdate, partner,
			packagename,
			packagever,
			protocolname,
			protocolver,
			processname,
			processver
		   )
		VALUES ('NON REP Export', p_userid, v_BeginTS,
			CASE p_partner WHEN 'allO0oxyz' then  'ALL' else  p_partner END,
			CASE p_packagename WHEN  'allO0oxyz' then  'ALL' else  p_packagename END,
			CASE p_packagever WHEN   'allO0oxyz' then  'ALL' else  p_packagever END,
			CASE p_protocolname WHEN  'allO0oxyz' then  'ALL' else  p_protocolname END,
			CASE p_protocolver WHEN   'allO0oxyz' then  'ALL' else  p_protocolver END,
			CASE p_processname WHEN  'allO0oxyz' then  'ALL' else  p_processname END,
			CASE p_processver WHEN  'allO0oxyz' then  'ALL' else  p_processver END
		);


	if p_partner is NOT NULL AND p_partner <> 'allO0oxyz' then
		 v_sqltextstatic := ' AND partnername = ''' || p_partner || '''';
		 v_flag := v_flag +1;
	end if;  
  
	if p_packagever is NOT NULL AND p_packagever <> 'allO0oxyz' then
		 v_sqltextstatic := v_sqltextstatic || ' AND packagever = ''' || p_packagever || '''';
		 v_flag := v_flag +1;
	end if;
  
	if p_packagename is NOT NULL AND p_packagename <> 'allO0oxyz' then
  		 v_sqltextstatic := v_sqltextstatic || ' AND packagename = ''' || p_packagename || '''';
		 v_flag := v_flag +1;
	end if;
  
	if p_protocolname is NOT NULL AND p_protocolname <> 'allO0oxyz' then
  		 v_sqltextstatic := v_sqltextstatic || ' AND protocolname = ''' || p_protocolname || '''';
		 v_flag := v_flag +1;
	end if;

	if p_protocolver is NOT NULL AND p_protocolver <> 'allO0oxyz' then
  		 v_sqltextstatic := v_sqltextstatic || ' AND protocolver = ''' || p_protocolver || '''';
		 v_flag := v_flag +1;
	end if;  
  
	if p_processname is NOT NULL AND p_processname <> 'allO0oxyz' then
  		 v_sqltextstatic := v_sqltextstatic || ' AND processname = '''  || p_processname || '''';
		 v_flag := v_flag +1;
	end if;  
  
	if p_processver is NOT NULL AND p_processver <> 'allO0oxyz' then
  		 v_sqltextstatic := v_sqltextstatic || ' AND processver = ''' || p_processver || '''';
		 v_flag := v_flag +1;
	end if;

	SELECT	TRIM(TO_CHAR(AR_EXPORT_SEQ.nextval))
	INTO	v_JobName
	FROM	dual;
	v_FileNamePrefix := RTRIM(v_TaskName ||'.'|| REPLACE(REPLACE(p_SubVolName,'/','.'),'\','.') ||'.'|| COALESCE(p_ParameterID,0) ||'.'|| v_JobName);
	v_ExpDmpFile := '';
	FOR v_ctr IN 1..v_Parallel LOOP
		v_ExpDmpFile := v_ExpDmpFile || '''' || v_FileNamePrefix || TRIM(TO_CHAR(v_ctr,'99')) || '.dmp'' ,';
	END LOOP;
	v_ExpDmpFile := RTRIM(v_ExpDmpFile, ',');

	v_RowCount := 0;
	v_sqlText := ' SELECT count(*) FROM LG_MSG_ARCHIVE WHERE SUBVOLNAME = ''' || p_SubVolName || ''' ' || v_sqlTextStatic;
	EXECUTE IMMEDIATE v_SqlText INTO v_RowCount;

	v_TableName := 'WPGEXP' || v_JobName;
	dbms_output.put_line ( RPAD(v_TableName, 30, ' ') || to_char( v_RowCount,'999999990' ) ||'   '|| SUBSTR('EXP',1,3)||'| '|| to_char( v_Duration, '999990.00' ));
	IF v_RowCount > 0 THEN

		-- create External table using Oracle DataPump access driver
		v_sqlText := 'CREATE TABLE ' || v_TableName 
				|| ' ORGANIZATION EXTERNAL ( TYPE ORACLE_DATAPUMP DEFAULT DIRECTORY ' || v_DmpDir
				|| ' LOCATION (' || v_ExpDmpFile || ') ) '
				|| ' PARALLEL ' || TO_CHAR(v_Parallel)
				|| ' AS SELECT  vuid,nrmediaid,upmediaid,subvolname,rowts,createdate,updatedate '
				|| ' ,digsigcertid,encryptcertid,digsigprikeycertid,ecryptprikeycertid '
				|| ' ,partnername,packagename,protocolname,processname,packagever,protocolver,processver,bdo '
				|| ' FROM LG_MSG_ARCHIVE WHERE SUBVOLNAME = ''' || p_SubVolName || ''' ' || v_sqlTextStatic
		;

		-- this will export the data to the file(s)
		EXECUTE IMMEDIATE v_SqlText;

		-- Drop the table
		EXECUTE IMMEDIATE 'DROP TABLE ' || v_TableName;
	END IF;
	lg_arch_task_exportDbI(v_returncd, p_DebugFlg, p_TaskId, p_SubVolName, COALESCE(p_ParameterID,0), v_RowCount, v_FileNamePrefix, v_BeginTS);
	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	dbms_output.put_line ( RPAD(v_TableName, 30, ' ') || to_char( v_RowCount,'999999990' ) ||'   '|| SUBSTR('IMP',1,3)||'| '|| to_char( v_Duration, '999990.00' ));
	COMMIT;

	p_ReturnCd := 0;

EXCEPTION
	WHEN NotSupported THEN
		p_ReturnCd := 0;
	WHEN OTHERS THEN
		dbms_output.put_line('Export error:'||SQLERRM);
		ROLLBACK;
		RAISE_APPLICATION_ERROR (-20999, '20002 AR_EXPORT_DATA ' || SQLERRM);
END AR_EXPORT_DATA;
/


-- AR_HISTORY_UPDATE.sql

CREATE OR REPLACE PROCEDURE         AR_HISTORY_UPDATE(
    p_TableName in Varchar2,
    p_Days      in Pls_Integer default 7)
    IS
/*
       ================================================================================
       Name:           AR_HISTORY_UPDATE 
       Create Date:    2003-09-30
       Author          Vipin Goel
        This Procedure Inserts data into <p_TableName>_HIST table for last <p_Days>
                        ::::::::::::::::::::::::::::::
                        M O D I F I C A T I O N
                        ::::::::::::::::::::::::::::::
       ---------  ------------ --------  ---------------------------------------------
       DATE       NAME         Version   DESCRIPTION
       ---------  ------------ --------  ---------------------------------------------
       2003-09-30 V.Goel                 Created
       ================================================================================

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_SqlText Varchar2(1000);
Begin
    v_SqlText :=
    'Insert Into '||p_TableName||'_Hist'||
    ' Select * From '||p_TableName||
    ' Where trunc(to_date(to_char(Createdate, ''yyyy-mm-dd''), ''yyyy-mm-dd'')) < Sysdate - '||p_Days ;
--    DBMS_OUTPUT.Put_Line(v_SqlText);
--    Note Create date is a TimeStamp in some cases, should be converted to date first
    Execute Immediate v_SqlText;
    Exception
        When No_Data_Found Then
            Null;
        When Others Then
        Rollback;
        Raise_Application_Error (-20999, '20001 AR_HISTORY_UPDATE '|| SQLERRM);
End AR_History_update;
/


-- AR_IMPORT_DATA.sql

CREATE OR REPLACE PROCEDURE AR_IMPORT_DATA(
	p_ReturnCd		OUT	pls_INTEGER
	,p_DebugFlg		IN	pls_INTEGER
	,p_TaskID		IN	NUMBER
	,p_ArchiveDest		IN	VARCHAR2
	,p_ArchiveFile		IN	VARCHAR2
	)
AS

/*
	================================================================================
	Name:	   AR_IMPORT_DATA
	Create Date:	2008-10-24
	Author	  Som Priani
	This Procedure imports data from external table (flat file) using Oracle datapump
	access driver. Inserts into LG_MSG_RESTORE or LG_CERT_ARCHIVE

			 ::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
			 ::::::::::::::::::::::::::::::
	---------- ------------ -------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------  ------------ -------	---------------------------------------------
	2008-10-31 S.Priani	6.2	F104161
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_ProcName		VARCHAR2(30)	default 'AR_IMPORT_DATA';
	v_SqlText		VARCHAR2(4000);
	v_BeginTS		TIMESTAMP	DEFAULT fn_Get_Date;

	v_ReturnCd		INTEGER;
	v_Cursor1		DB_REF.RCT;
	v_TaskName		VARCHAR2(8)	DEFAULT 'MSGARCH';
	v_JobName		VARCHAR2(30)	DEFAULT TO_CHAR(v_BeginTS,'YYYYMMDDHH24MISSFF');
	v_Flag			INTEGER		DEFAULT 0;
	v_TableName		VARCHAR2(30)	DEFAULT 'WPGIMP' || v_JobName;
	v_FileNamePrefix	VARCHAR2(255);
	v_subVolName		v_FileNamePrefix%TYPE;
	v_PosIdx1		NUMBER(4);
	v_PosIdx2		NUMBER(4);

	v_Duration		NUMBER(9,2)	DEFAULT 0;
	v_StartTime		INTEGER;
	v_EndTime		INTEGER;
	
	v_CommitCount		NUMBER(6)		DEFAULT 1000;
	v_RowCount		NUMBER;

	v_ExpDmpFile		VARCHAR2(255) ;
	v_DmpDir		VARCHAR2(30) := 'WPGRESTORE';
	cur_tmp			db_Ref.rct;

	rec_msgarch		lg_msg_restore%ROWTYPE;
	rec_certarch		lg_cert_archive%ROWTYPE;

	NotSupported		EXCEPTION;
BEGIN
	BEGIN
		SELECT	1 INTO v_PosIDX1
		FROM	v$version
		WHERE	banner LIKE '%9i%'
		AND	rownum < 2
		;
		RAISE NotSupported;
	EXCEPTION
		WHEN No_Data_Found THEN
			Null;
	END;
	p_ReturnCd := 1;
	v_StartTime := DBMS_UTILITY.get_time;

	BEGIN
		SELECT	COALESCE(CAST(propvalue AS INT), v_CommitCount)
		INTO	v_CommitCount
		FROM	DB_SYSPROPERTIES
		WHERE	propname = 'bcg.archive.db.rowsPerStatement'
		;

	EXCEPTION
		WHEN No_Data_Found THEN
			Null;  --- Commit on Default rows
	END;

	--- create a directory object for the path specified, will read data from there, write log file there.
	AR_CREATE_DIRECTORY(v_returncd, p_debugflg, v_cursor1, v_DmpDir, p_ArchiveDest);

	v_FileNamePrefix := UPPER(p_ArchiveFile);
	v_FileNamePrefix := RTRIM(REPLACE(v_FileNamePrefix,'.DMP',''));

	--- to get subvolume from filename of the format MSGARCH.yyyymmdd.a.b.TS23.dmp - subvol will be yyyymmdd/a
	IF UPPER(p_ArchiveDest) LIKE '%CERTIFICATE%' THEN
		v_TaskName := 'CERTARCH';
	ELSE
		v_TaskName := 'MSGARCH';
	END IF;
	IF v_FileNamePrefix LIKE v_TaskName || '%' THEN
		v_subVolName := REPLACE(v_FileNamePrefix, v_TaskName || '.','');
		v_PosIDX1 := INSTR( v_SubVolName, '.');
		v_PosIDX2 := INSTR( SUBSTR( v_subVolName, v_PosIdx1 +1 ) , '.');
		v_subVolName := SUBSTR( v_subVolName, 1, v_PosIDX1 + v_PosIDX2 - 1 );
	ELSE
		v_subVolName := 'Z' || SUBSTR( v_JobName, 1, 15);
	END IF;

	v_RowCount := 0;

	IF v_TASKNAME = 'MSGARCH' THEN
	----Importing data for LG_MSG_ARCHIVE
		BEGIN
		v_sqlText := 'CREATE TABLE ' || v_TableName 
				|| '( VUID		VARCHAR2(65) , '
				|| ' NRMEDIAID	  NUMBER(10)	, '
				|| ' UPMEDIAID	  NUMBER(10), '
				|| ' SUBVOLNAME	 VARCHAR2(16), '
				|| ' ROWTS		NUMBER(10)	, '
				|| ' CREATEDATE	 TIMESTAMP   , '
				|| ' UPDATEDATE	 TIMESTAMP   , '
				|| ' DIGSIGCERTID	NUMBER(10), '
				|| ' ENCRYPTCERTID	NUMBER(10), '
				|| ' DIGSIGPRIKEYCERTID NUMBER(10), '
				|| ' ECRYPTPRIKEYCERTID NUMBER(10), '
				|| ' PARTNERNAME	VARCHAR2(120), '
				|| ' PACKAGENAME	VARCHAR2(255), '
				|| ' PROTOCOLNAME	VARCHAR2(255), '
				|| ' PROCESSNAME	VARCHAR2(255), '
				|| ' PACKAGEVER	 VARCHAR2(60), '
				|| ' PROTOCOLVER	VARCHAR2(60), '
				|| ' PROCESSVER	 VARCHAR2(60), '
				|| ' BDO 		BLOB '
				|| ') ORGANIZATION EXTERNAL ( TYPE ORACLE_DATAPUMP DEFAULT DIRECTORY ' || v_DmpDir
				|| ' LOCATION (''' || p_ArchiveFile || ''') ) '
		;
		EXECUTE IMMEDIATE v_SqlText;
		COMMIT;
		END;

		OPEN cur_tmp FOR 'SELECT * FROM ' || v_TableName;
		LOOP
			FETCH cur_tmp INTO rec_msgarch;
			EXIT WHEN cur_tmp%NOTFOUND;
			BEGIN
			INSERT INTO lg_msg_restore (
				vuid
				,nrmediaid
				,upmediaid
				,subvolname
				,createdate
				,updatedate
				,digsigcertid
				,encryptcertid
				,digsigprikeycertid
				,ecryptprikeycertid
				,partnername
				,packagename
				,protocolname
				,processname
				,packagever
				,protocolver
				,processver
				,bdo
			) VALUES (
				rec_msgarch.vuid
				,rec_msgarch.nrmediaid
				,rec_msgarch.upmediaid
				,rec_msgarch.subvolname
				,rec_msgarch.createdate
				,rec_msgarch.updatedate
				,rec_msgarch.digsigcertid
				,rec_msgarch.encryptcertid
				,rec_msgarch.digsigprikeycertid
				,rec_msgarch.ecryptprikeycertid
				,rec_msgarch.partnername
				,rec_msgarch.packagename
				,rec_msgarch.protocolname
				,rec_msgarch.processname
				,rec_msgarch.packagever
				,rec_msgarch.protocolver
				,rec_msgarch.processver
				,rec_msgarch.bdo
			);

			v_RowCount := v_RowCount + 1;
			IF MOD(v_RowCount, v_CommitCount) = 0 THEN
				COMMIT;
			END IF;
			EXCEPTION
				WHEN Dup_Val_On_Index THEN
					Null;
			END;
		END LOOP;
		CLOSE cur_tmp;
	ELSE
		---- Importing certificates
		BEGIN
		v_sqlText := 'CREATE TABLE ' || v_TableName 
					|| ' (VUID             VARCHAR2(65)	, '
					|| ' ARCHIVECERTID    NUMBER(10)	, '
					|| ' CERTNAME         VARCHAR2(60)	, '
					|| ' DEFFLG           NUMBER(1)		, '
					|| ' CN               VARCHAR2(255)	, '
					|| ' L                VARCHAR2(255)	, '
					|| ' ST               VARCHAR2(255)	, '
					|| ' O                VARCHAR2(255)	, '
					|| ' OU               VARCHAR2(255)	, '
					|| ' C                VARCHAR2(255)	, '
					|| ' VALIDFR          TIMESTAMP(6)	, '
					|| ' VALIDTO          TIMESTAMP(6)	, '
					|| ' CERTVER          VARCHAR2(15)	, '
					|| ' FINGERPRINTALGO  VARCHAR2(30)	, '
					|| ' FINGERPRINT      VARCHAR2(255)	, '
					|| ' SERIALNBR        VARCHAR2(255)	, '
					|| ' PUBLICKEY        VARCHAR2(4000)	, '
					|| ' REVOKEDFLG       NUMBER(1)		, '
					|| ' REVOKEDDATE      TIMESTAMP(6)	, '
					|| ' CERT             BLOB		, '
					|| ' ROWTS            NUMBER(10)	, '
					|| ' CREATEDATE       TIMESTAMP(6)	, '
					|| ' UPDATEDATE       TIMESTAMP(6)	, '
					|| ' SSLCLIENTCERTFLG NUMBER(1)		, '
					|| ' DIGSIGCERTFLG    NUMBER(1)		, '
					|| ' ENCRYPTCERTFLG   NUMBER(1)		, '
					|| ' ENABLEDFLG       NUMBER(1)		, '
					|| ' ROOTCERTFLG      NUMBER(1)		, '
					|| ' PRISECFLG        NUMBER(1)		, '
					|| ' PASSWORD         RAW(100)		, '
					|| ' PRIVATEKEY       BLOB		 '
				|| ') ORGANIZATION EXTERNAL ( TYPE ORACLE_DATAPUMP DEFAULT DIRECTORY ' || v_DmpDir
				|| ' LOCATION (''' || p_ArchiveFile || ''') ) '
		;
		EXECUTE IMMEDIATE v_SqlText;
		COMMIT;
		END;

		OPEN cur_tmp FOR 'SELECT * FROM ' || v_TableName;
		LOOP
			FETCH cur_tmp INTO rec_certarch;
			EXIT WHEN cur_tmp%NOTFOUND;
			BEGIN
			INSERT INTO lg_cert_archive (
				VUID
				,ArchiveCERTID
				,CERTNAME
				,DEFFLG
				,CN
				,L
				,ST
				,O
				,OU
				,C
				,VALIDFR
				,VALIDTO
				,CERTVER
				,FINGERPRINTALGO
				,FINGERPRINT
				,SERIALNBR
				,PUBLICKEY
				,REVOKEDFLG
				,REVOKEDDATE
				,CERT
				,ROWTS
				,CREATEDATE
				,SSLCLIENTCERTFLG
				,DIGSIGCERTFLG
				,ENCRYPTCERTFLG
				,ENABLEDFLG
				,ROOTCERTFLG
				,PRISECFLG
				,PASSWORD
				,PRIVATEKEY
			) VALUES (
				rec_certarch.VUID
				,rec_certarch.ArchiveCERTID
				,rec_certarch.CERTNAME
				,rec_certarch.DEFFLG
				,rec_certarch.CN
				,rec_certarch.L
				,rec_certarch.ST
				,rec_certarch.O
				,rec_certarch.OU
				,rec_certarch.C
				,rec_certarch.VALIDFR
				,rec_certarch.VALIDTO
				,rec_certarch.CERTVER
				,rec_certarch.FINGERPRINTALGO
				,rec_certarch.FINGERPRINT
				,rec_certarch.SERIALNBR
				,rec_certarch.PUBLICKEY
				,rec_certarch.REVOKEDFLG
				,rec_certarch.REVOKEDDATE
				,rec_certarch.CERT
				,rec_certarch.ROWTS
				,rec_certarch.CREATEDATE
				,rec_certarch.SSLCLIENTCERTFLG
				,rec_certarch.DIGSIGCERTFLG
				,rec_certarch.ENCRYPTCERTFLG
				,rec_certarch.ENABLEDFLG
				,rec_certarch.ROOTCERTFLG
				,rec_certarch.PRISECFLG
				,rec_certarch.PASSWORD
				,rec_certarch.PRIVATEKEY
			);

			v_RowCount := v_RowCount + 1;
			IF MOD(v_RowCount, v_CommitCount) = 0 THEN
				COMMIT;
			END IF;
			EXCEPTION
				WHEN Dup_Val_On_Index THEN
					Null;
			END;
		END LOOP;
		CLOSE cur_tmp;
	END IF;

	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	dbms_output.put_line ( RPAD(v_TableName, 30, ' ') || to_char( v_RowCount,'999999990' ) ||'   '|| SUBSTR('IMP',1,3)||'| '|| to_char( v_Duration, '999990.00' ));
	lg_arch_task_exportDbI(v_returncd, p_DebugFlg, p_TaskId, v_SubVolName, TO_NUMBER(v_JobName), v_RowCount, v_FileNamePrefix, v_BeginTS);
	COMMIT;
	EXECUTE IMMEDIATE 'DROP TABLE ' || v_TableName;
	p_ReturnCd := 0;

EXCEPTION
	WHEN NotSupported THEN
		p_ReturnCd := 0;
	WHEN OTHERS THEN
		dbms_output.put_line('Import error:'||SQLERRM);
		ROLLBACK;
		RAISE_APPLICATION_ERROR (-20999, '20002 AR_IMPORT_DATA ' || SQLERRM);
END AR_IMPORT_DATA;
/


-- AR_NONREP_MAINTENANCE.sql

CREATE OR REPLACE PROCEDURE ar_nonrep_maintenance
(
		p_ReturnCd OUT	INTEGER
	,	p_DebugFlg IN	INTEGER
	,	p_Date	IN	VARCHAR2
	,	p_TASKID IN	NUMBER
	)
AS
/*
	================================================================================
	Name:	   AR_NONREP_MAINTENANCE
	Create Date:	2004-03-23
	Author	  Shishir Narain
	This Procedure removes data from LG_MSG_ARCHIVE table:

			 ::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
			 ::::::::::::::::::::::::::::::
	---------- ------------ -------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------  ------------ -------	---------------------------------------------
	2006-01-30 S.Narain	6.1	CR 56054: Improve Archiving Support
	2007-05-24 S.Priani	6.1.0.1	CR 104490
	2007-08-24 S.Priani	6.1.1	CR 104487
	2008-07-02 S.Priani	6.1.1.1	CR 106238
	2008-08-05 S.Priani	6.2	F106141
	================================================================================

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
	v_TSNextDay		TimeStamp;
	v_TableName		USER_TABLES.TABLE_NAME%TYPE 	DEFAULT 'LG_MSG_ARCHIVE';
	v_procname		VARCHAR2 (30)	DEFAULT 'AR_NONREP_MAINTENANCE';
	v_exceptionflg		INTEGER		DEFAULT 0;
	v_rowcnt		INTEGER		DEFAULT 0;
	v_DelChunkSize		NUMBER(10)	DEFAULT 5000;
	v_begints		TIMESTAMP	DEFAULT fn_get_date;
	v_sqltext		VARCHAR2 (4000);
	v_sqltextstatic		VARCHAR2 (2000);
	v_number		INTEGER;
	v_flag			PLS_INTEGER	DEFAULT 0; --will open RC2
	v_StartTime		INTEGER;
	v_EndTime		INTEGER;
	v_Duration		NUMBER(9,2);
	v_ServerTSDiff		INTERVAL DAY(2) TO SECOND(5);
	v_RC1			Db_REF.RCT;
	v_epoch			VARCHAR2(8)	DEFAULT '20010101';
	v_paramstxt		VARCHAR2 (4000)	:=	' p_Date=>' || p_date || CHR (10)
		;
BEGIN
	p_ReturnCd := 1;
	BEGIN
		SELECT	COALESCE(CAST(propvalue AS INT), v_DelChunkSize)
		INTO	v_DelChunkSize
		FROM	DB_SYSPROPERTIES
		WHERE	propname = 'bcg.archive.db.rowsPerStatement'
		;

	EXCEPTION
		WHEN No_Data_Found THEN
			Null;  --- delete 5000 rows by Default per statement
	END;

	dbms_output.put_line ( RPAD('TABLE NAME', 30, ' ') || ' Row Count   OP | Time(Secs)' );
	dbms_output.put_line ( RPAD('-', 60, '-')  );

	v_TSNextDay := to_TimeStamp(p_date,'YYYY-MM-DD') + 1 ;

	v_sqltext := 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ FROM LG_MSG_ARCHIVE '
			|| ' Where SUBVOLNAME < :ip_date '
			|| ' and rownum < ' || TO_CHAR(v_DelChunkSize + 1);
      
	IF v_flag = 0 THEN
	--- see if we can drop database partitions
	----	db_tabpart_drop(v_number, 1, v_rc1, 'LG_MSG_ARCHIVE', v_epoch, TO_CHAR(v_TSNextDay, 'YYYYMMDD'));
		null;
	END IF;

	BEGIN
		SELECT	lat.CREATEDATE - v_BeginTS
		INTO	v_ServerTSDiff
		FROM	lg_arch_task lat
		WHERE	lat.taskid = p_TASKID
		;
	END;
    
	v_StartTime := DBMS_UTILITY.get_time;
	v_RowCnt := 0;
	v_sqltext := v_sqltext || v_sqltextstatic;
	LOOP
		EXECUTE IMMEDIATE v_sqltext USING TO_CHAR(v_TSNextDay, 'YYYYMMDD');
		v_number := SQL%ROWCOUNT;
		COMMIT;
		v_RowCnt := v_RowCnt + v_number;
		EXIT WHEN v_number = 0;
	END LOOP;
	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	dbms_output.put_line ( RPAD('LG_MSG_ARCHIVE', 30, ' ') || to_char( v_RowCnt,'999999990' ) ||'   '|| SUBSTR('DEL',1,3)||'| '|| to_char( v_Duration, '999990.00' ));

	LG_ARCH_TASK_PURGEDBI (v_ExceptionFlg , 0, p_TASKID, v_TableName, v_TSNextDay, v_RowCnt, v_ProcName, 'Purge Successful', v_BeginTS + v_ServerTSDiff, sys_Extract_utc(systimestamp) + v_ServerTSDiff);
	COMMIT;
	p_ReturnCd := 0;
EXCEPTION
	WHEN Others THEN
		ROLLBACK;
		raise_application_error (-20999, '20002 AR_NONREP_MAINTENANCE ' || SQLERRM);
END ar_nonrep_maintenance;
/

-- AR_PURGE_ARCHIVE.sql

CREATE OR REPLACE PROCEDURE AR_PURGE_ARCHIVE(
    p_TableName in Varchar2,
    p_Date      in date)
    IS
/*
       ================================================================================
       Name:           AR_PURGE_ARCHIVE
       Create Date:    2004-03-15
       Author          Shishir Narain
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE	   NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2005-08-05 S.Narain		CR71945 Include FK in delete 	
	2005-10-06 S.Priani	4.2.2	BGTRK00073846
	2005-08-09 S.Priani	6.1	CR 102308
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
	v_SqlText	Varchar2(1000);
	v_RowLimit	NUMBER(10)		:= 5001;
	v_TmpCount	INTEGER(10);
	v_RowCount	INTEGER(10);
	v_StartTime	INTEGER			:= DBMS_UTILITY.get_time;
	v_EndTime	INTEGER;
	v_Duration	NUMBER(9,2);

BEGIN

IF p_TableName = 'LG_ACTIVITY_SUMMARY' THEN

	v_SqlText := 'DELETE /*+ FIRST_ROWS(5000)  */ FROM LG_ACTIVITY_SUMMARY las
		WHERE EXISTS (SELECT /*+ FIRST_ROWS(5000)  */ ''x''
			FROM	LG_SUMMARY ls
			WHERE	las.DAYHOUR < :ip_date 
			AND	las.SUMMARYID = ls.SUMMARYID
			AND	las.DAYHOUR = ls.DAYHOUR
			AND	las.ENDSTATEDAYHOUR = ls.ENDSTATEDAYHOUR
			AND	rownum < :IN_ROWS
			)';

ELSIF p_TableName = 'LG_EVENT_EVENTSUMMARY' THEN
			 
	v_SqlText := 'DELETE /*+ FIRST_ROWS(5000)  */ FROM LG_EVENT_EVENTSUMMARY les
		WHERE EXISTS (SELECT /*+ FIRST_ROWS(5000)  */ ''x''
			FROM	LG_EVENTSUMMARY le
			WHERE	le.DAYHOUR < :ip_date
			AND	le.EVENTSUMMARYID = les.EVENTSUMMARYID
			AND	le.DAYHOUR = les.DAYHOUR
			AND	rownum < :IN_ROWS
			)';

ELSIF p_TableName = 'LG_SUMMARY_MI' THEN

	v_SqlText := 'DELETE /*+ FIRST_ROWS(5000)  */ FROM LG_SUMMARY_MI Where DAYHOURMI < :ip_date
			  	 and rownum < :IN_ROWS';

ELSIF p_TableName IN ('LG_MSG_ARCHIVE','LG_CERT_ARCHIVE') THEN

	v_SqlText := 'DELETE /*+ FIRST_ROWS(5000)  */ FROM ' || p_TableName ||  '  Where CREATEDATE < :ip_date
			  	 and rownum < :IN_ROWS';
ELSE	

	v_SqlText := 'DELETE /*+ FIRST_ROWS(5000)  */ FROM ' || p_TableName ||  '  Where DAYHOUR < :ip_date
			  	 and rownum < :IN_ROWS'; 

END IF;

	v_RowCount:=0;
	LOOP
		EXECUTE IMMEDIATE v_SqlText USING p_Date + 1, v_RowLimit;
		v_TmpCount := SQL%RowCount;
		v_RowCount := v_RowCount + v_TmpCount;
		
		COMMIT;
		
		IF v_TmpCount < v_RowLimit - 1 THEN
			EXIT;
		END IF;
	END LOOP;
	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	dbms_output.put_line ( RPAD(p_TableName, 30, ' ') || to_char( v_RowCount,'999999990' ) || '   DEL| ' || to_char( v_Duration, '999990.00' ));
EXCEPTION
        WHEN No_Data_Found THEN
            NULL;
        WHEN Others THEN
        Rollback;
        Raise_Application_Error (-20999, '20002 AR_PURGE_ARCHIVE '|| SQLERRM);
End AR_Purge_Archive;
/


-- AR_PURGE_HEADERS.sql

CREATE OR REPLACE PROCEDURE Ar_Purge_Headers(
    p_Date      IN varchar2)
    IS
/*
       ================================================================================
       Name:           AR_PURGE_HEADERS
       Create Date:    2004-03-15
       Author          Shishir Narain
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2005-10-06 S.Priani	4.2.2	BGTRK00073846
	2005-10-10 B.Moffat	4.2.2	Modified(Viacore Inc.) - indexes and FKs are NOT dropped.
	2006-01-30 S.Narain	6.1	CR 56054:Improve Archiving support
	2006-08-09 S.Priani	6.1	CR 102308
	2006-09-07 S.Priani	6.1	CR 102673
	2007-09-07 S.Priani	6.1.1	CR 104487
	2008-10-20 Amit vijay	6.2	Defect 107019
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
	v_PurgeTS	TIMESTAMP(6)	;
	v_TableName	USER_TABLES.TABLE_NAME%TYPE;
	v_SqlText	VARCHAR2(1024);
	v_DelChunkSize	NUMBER(10)		:= 5000;
	v_RowsPerBatch	NUMBER(10)		:= 1000000;
	v_TmpCount	INTEGER(12);
	v_RowCount	INTEGER(12);
	v_DelCount	INTEGER(12);
	v_Stmt_DELETE	VARCHAR2(1024);
	v_StartTime	INTEGER;
	v_EndTime	INTEGER;
	v_Duration	NUMBER(9,2);

	v_Count_VUID	INTEGER;
	v_Count_ActID	INTEGER;
	v_Count_AS	INTEGER;
	v_Count_RN	INTEGER;
	v_Count_EBMS	INTEGER;
	v_Count_Evt	INTEGER;
PROCEDURE LogMesg(p_Object VARCHAR2
			,p_Operation VARCHAR2
			,p_RowCount	NUMBER
			,p_Duration	NUMBER
		) IS
BEGIN
	dbms_output.put_line ( RPAD(p_Object, 30, ' ') || to_char( p_RowCount,'999999990' ) ||'   '|| SUBSTR(p_Operation,1,3)||'| '|| to_char( p_Duration, '999990.00' ));
END;
PROCEDURE PurgeTable(p_sqltxt VARCHAR2, p_sql2 VARCHAR2, p_Limit NUMBER) IS
BEGIN
	v_StartTime := DBMS_UTILITY.get_time;
    
	v_RowCount := 0;
	v_DelCount := 0;
	LOOP
		v_SqlText := p_sqltxt ;
		EXECUTE IMMEDIATE v_SqlText USING v_RowCount + 1, v_RowCount + v_DelChunkSize;
		v_TmpCount := SQL%ROwCount;
		v_DelCount := v_DelCount + v_TmpCount;
		v_RowCount := v_RowCount + v_DelChunkSize;
		
		COMMIT;
		
		IF v_RowCount > p_Limit THEN
			EXIT;
		END IF;
	
	END LOOP;

	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	LogMesg(v_TableName, 'DEL', v_DelCount, v_Duration);
END PurgeTable;
PROCEDURE EmptyTable(p_TableName VARCHAR2) IS
	v_statement	VARCHAR2(500);
BEGIN
	v_Statement := 'TRUNCATE TABLE ' || p_tableName;
	EXECUTE IMMEDIATE v_Statement;
EXCEPTION
	WHEN Others THEN
		RAISE_APPLICATION_ERROR(-20991, SQLERRM);
END;
---Main

BEGIN
	v_PurgeTS := TO_TIMESTAMP( p_Date, 'YYYY-MM-DD') + 1;
	dbms_output.put_line ( RPAD('TABLE NAME', 30, ' ') || ' Row Count   OP | Time(Secs)' );
	dbms_output.put_line ( RPAD('-', 60, '-')  );


	BEGIN
		SELECT	COALESCE(CAST(propvalue AS INT), v_DelChunkSize)
		INTO	v_DelChunkSize
		FROM	DB_SYSPROPERTIES
		WHERE	propname = 'bcg.archive.db.rowsPerStatement'
		;

	EXCEPTION
		WHEN No_Data_Found THEN
			Null;  --- delete 5000 rows by Default per statement
	END;

	BEGIN
		SELECT	COALESCE(CAST(propvalue AS INT), v_RowsPerBatch)
		INTO	v_RowsPerBatch
		FROM	DB_SYSPROPERTIES
		WHERE	propname = 'bcg.archive.db.rowsPerBatch'
		;

	EXCEPTION
		WHEN No_Data_Found THEN
			Null;  ---  a million rows by Default per run
	END;

--- RN
	LOOP
		v_TableName := 'DB_PURGE_RN';
		EmptyTable(v_TableName);
		v_StartTime := DBMS_UTILITY.get_time;
		INSERT INTO DB_Purge_RN (RowCounter, CID, InitBusinessId, ProcessInstanceId)
			SELECT ROW_NUMBER() OVER (ORDER BY rcvdocts) AS rcnt, CID, INITBUSINESSID, PROCESSINSTANCEID
			FROM LG_Activity_RnHdr
			WHERE rcvDocTs < v_PurgeTS
			AND	rownum < v_RowsPerBatch + 1
			;
		v_RowCount := SQL%ROwCount;
		COMMIT;
		v_count_RN := v_rowCount;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg(v_TableName, 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_RN = 0;

		v_TableName := 'LG_ACTIVITY_RNDTL';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_activity_rndtl lard '
			|| ' WHERE ( CID,InitBusinessId, ProcessInstanceId) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ '
			|| '	 CID,InitBusinessId, ProcessInstanceId FROM DB_Purge_RN '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_RN );

		v_TableName := 'LG_ACTIVITY_RNHDR';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_activity_rnhdr '
			|| ' WHERE ( CID,InitBusinessId, ProcessInstanceId) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ '
			|| '	 CID,InitBusinessId, ProcessInstanceId FROM DB_Purge_RN '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_RN );

	END LOOP;
--- AS
	LOOP
		v_TableName := 'DB_PURGEASWRK';
		EmptyTable(v_TableName);
		v_StartTime := DBMS_UTILITY.get_time;

		INSERT INTO DB_PurgeASWrk (RowCounter, RcvDocTs, InitBusinessId, InitASMessageId, ActivityId)
			SELECT ROW_NUMBER() OVER (ORDER BY rcvdocts) AS rcnt, RcvDocTs, INITBUSINESSID, INITASMESSAGEID, ActivityId
			FROM (SELECT DISTINCT a.RcvDocTs,
				    d.INITBUSINESSID,
				    d.INITASMESSAGEID,
				    a.ActivityId
				FROM LG_AS_Dtl d
				INNER JOIN LG_Activity a on a.ActivityId = d.ActivityId
				WHERE rcvDocTs < v_PurgeTS
				AND	rownum < v_RowsPerBatch + 1)
			;
		v_RowCount := SQL%ROwCount;
		COMMIT;
		v_count_AS := v_rowCount;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg(v_TableName, 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_AS = 0;

		v_TableName := 'LG_AS_DTL';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_as_dtl lasd '
			|| ' WHERE (InitBusinessId, InitASMessageId) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ '
			|| '	InitBusinessId, InitASMessageId FROM DB_PurgeASWrk '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_AS );

		v_TableName := 'LG_AS_HDR';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_as_hdr lash '
			|| ' WHERE (InitBusinessId, InitASMessageId) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ '
			|| '	InitBusinessId, InitASMessageId FROM DB_PurgeASWrk '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_count_AS );
	END LOOP;
--- ebXML
	LOOP
		v_TableName := 'DB_PURGEEBMS';
		EmptyTable(v_TableName);
		v_StartTime := DBMS_UTILITY.get_time;
		INSERT INTO DB_PurgeEBMS (RowCounter, ProcessHdrID)
			SELECT ROW_NUMBER() OVER (ORDER BY ProcessHdrID) AS rcnt, PROCESSHdrID
			FROM lg_b2bprocess_hdr
			WHERE rcvDocTs < v_PurgeTS
			AND	rownum < v_RowsPerBatch + 1
			;
		v_RowCount := SQL%ROwCount;
		v_count_ebms := v_rowCount;
		COMMIT;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg(v_TableName, 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_ebms = 0;

		v_TableName := 'LG_STATUS_ACTIVITY';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_status_Activity lbd '
			|| ' WHERE ProcessHDrID IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ProcessHDrID FROM DB_PurgeEBMS '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_count_EBMS );

		v_TableName := NULL;
		v_TableName := 'LG_B2BPROCESS_DTL';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_b2bprocess_dtl lbd '
			|| ' WHERE ProcessHDrID IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ProcessHDrID FROM DB_PurgeEBMS '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_count_EBMS );

		v_TableName := 'LG_B2BPROCESS_HDR';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_b2bprocess_hdr lbh '
			|| ' WHERE ProcessHDrID IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ProcessHDrID FROM DB_PurgeEBMS '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_count_EBMS );
	END LOOP;


-- Activity
	LOOP
		v_TableName := 'DB_PURGEACTID';
		EmptyTable(v_TableName);
		v_StartTime := DBMS_UTILITY.get_time;
		INSERT INTO DB_PurgeActID( rowcounter, rcvdocts, activityid)
			SELECT ROW_NUMBER() OVER(ORDER BY rcvdocts) AS rcnt, RcvDocTs, ActivityId
			FROM	LG_Activity a
			WHERE	rcvDocTs < v_PurgeTS
			AND	rownum < v_RowsPerBatch + 1
			;
		v_RowCount := SQL%ROwCount;
		COMMIT;
		v_count_ActID := v_rowCount;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg(v_TableName, 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_ActID = 0;


		v_TableName := 'LG_VALIDATION';
		v_Stmt_DELETE := 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_validation lv '
			|| ' WHERE ActivityId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ActivityID FROM DB_PurgeActID '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_ActID );

		v_TableName := 'LG_ACTIVITY_DTL';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_activity_dtl lad '
			|| ' WHERE ActivityId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ActivityID FROM DB_PurgeActID '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_count_ActID );

		v_TableName := 'LG_ACTIVITY_ENDSTATE';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_activity_endstate lae '
			|| ' WHERE ActivityId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ActivityID FROM DB_PurgeActID '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_ActID );
		
		v_TableName := 'LG_SYNCH_REQ_RESP';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from LG_SYNCH_REQ_RESP ls '
			|| ' WHERE ActivityId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ActivityID FROM DB_PurgeActID '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_ActID );

		v_TableName := 'LG_DELIVERY_LOG';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_delivery_log lad '
			|| ' WHERE ActivityId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ActivityID FROM DB_PurgeActID '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_ActID );

		v_TableName := 'BP_PROCESS_LOG';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_process_log lad '
			|| ' WHERE ActivityId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ActivityID FROM DB_PurgeActID '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_ActID );

		v_TableName := 'LG_ACTIVITY';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_activity '
			|| ' WHERE ActivityId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ ActivityID FROM DB_PurgeActID '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_ActID );
	END LOOP;

-- Event
	LOOP
		v_TableName := 'DB_PURGEEventWrk';
		EmptyTable(v_TableName);
		v_StartTime := DBMS_UTILITY.get_time;
		INSERT INTO DB_PurgeEventWrk (RowCounter, eventTs,  EventId)
			SELECT	ROW_NUMBER() OVER (ORDER BY eventTs) AS rcnt, e.eventTs,  e.EventId
			FROM	LG_Event e
			WHERE	eventTs < v_PurgeTS 
			AND	rownum < v_RowsPerBatch + 1
			;
		v_RowCount := SQL%ROwCount;
		COMMIT;
		v_count_Evt := v_rowCount;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg(v_TableName, 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_Evt = 0;

		v_TableName := 'LG_STACKTRACE';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_stacktrace las '
			|| ' WHERE EventID IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ EventID FROM DB_PURGEEVENTWRK '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_Evt );

		v_TableName := 'LG_EVENT';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from lg_event le '
			|| ' WHERE EventID IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ EventID FROM DB_PURGEEVENTWRK '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')';
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_Count_Evt );
	END LOOP;

EXCEPTION
        WHEN OTHERS THEN
	        ROLLBACK;
	        RAISE_APPLICATION_ERROR (-20999, '20002' || v_TableName || SQLERRM);
END Ar_Purge_Headers;
/


-- AR_PURGE_TABLE.sql

CREATE OR REPLACE PROCEDURE ar_purge_table (
   p_tablename   IN   VARCHAR2,
   p_days        IN   PLS_INTEGER DEFAULT 7
)
IS
/*
       ================================================================================
       Name:           AR_PURGE_TABLE
       Create Date:    2003-09-30
       Author          Vipin Goel
        This Procedure deletes data From <p_TableName> table for last <p_Days>
                        ::::::::::::::::::::::::::::::
                        M O D I F I C A T I O N
                        ::::::::::::::::::::::::::::::
	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2005-08-05 S.Narain		CR71945 Include FK in delete 	
	2005-10-14 S.Priani	4.2.2	BGTRK00073846
	2005-08-09 S.Priani	6.1	CR 102308

	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
	v_sqltext	VARCHAR2 (1000);
	v_PurgeTS	bp_state_hdr.createdate%TYPE := TRUNC(SYS_EXTRACT_UTC(SYSTIMESTAMP), 'DD') - p_Days;
	v_RowLimit	NUMBER(10)		:= 5001;
	v_TmpCount	INTEGER(10);
	v_RowCount	INTEGER(10);
	v_Stmt_DELETE	VARCHAR2(1024);
	v_StartTime	INTEGER			:= DBMS_UTILITY.get_time;
	v_EndTime	INTEGER;
	v_Duration	NUMBER(9,2);
BEGIN
   IF UPPER(p_tablename) IN ('BP_AS_STATE_DTL', 'BP_AS_STATE_HDR') THEN
      v_sqltext :=
            'DELETE /*+ FIRST_ROWS(5000)  */ from '
		|| p_tablename ||' bas
    	    	     WHERE EXISTS ( Select /*+ FIRST_ROWS(5000)  */ ''x''
			from BP_State_Hdr bsh
			Where bas.STATEHDRID = bsh.STATEHDRID
			and bsh.CREATEDATE < :IN_TS
			)';

   ELSE
      v_sqltext :=
            'DELETE /*+ FIRST_ROWS(5000)  */ from '
         || p_tablename
         || ' Where Createdate < :IN_TS';
		 
   END IF;

	v_RowCount := 0;
	LOOP
		v_Stmt_DELETE := v_sqltext || ' AND rownum < :IN_Limit';
		EXECUTE IMMEDIATE v_Stmt_DELETE USING v_PurgeTS, v_RowLimit;
		v_TmpCount := SQL%RowCount;
		v_RowCount := v_RowCount + v_TmpCount;
		
		COMMIT;
		
		IF v_TmpCount < v_RowLimit - 1 THEN
			EXIT;
		END IF;
	
	END LOOP;
	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	dbms_output.put_line ( RPAD(p_TableName, 30, ' ') || to_char( v_RowCount,'999999990' ) || ' DEL| ' || to_char( v_Duration, '999990.00' ));
EXCEPTION
   WHEN NO_DATA_FOUND THEN
      NULL;
   WHEN OTHERS THEN
      ROLLBACK;
      raise_application_error (-20999, '20002 AR_PURGE_TABLE ' || SQLERRM);
END ar_purge_table;
/


-- AR_STATEENGINE_MAINTENANCE.sql

CREATE OR REPLACE PROCEDURE AR_StateEngine_Maintenance
    (
    p_Date in Varchar2
    )
    IS
/*
       ================================================================================
       Name:           AR_RNStateEngine_Maintenance
       Create Date:    2003-09-30
       Author          Vipin Goel
        This Procedure Moves data from the Table to _Hist Table and the table:

        Table                  History Table         Action
        -----                  -------------         ------
RosettaNet:
        BP_rnStateHdr          BP_rnStateHdr_Hist    Purge
        BP_rnStateDtl          BP_rnStateDtl_Hist    Purge
        BP_rnStateHdrAuditLog                        Truncate
        BP_Sponsor_State			     Purge

AS1/AS2:
        BP_State_Hdr           BP_State_Hdr_Hist     Purge
        BP_AS_State_Hdr        BP_AS_State_Hdr_Hist  Purge
        BP_AS_State_Dtl        BP_AS_State_Dtl_Hist  Purge

ebXML:
        BP_RM_STATE_HDR        RM_STATE_HDR_HIST     Purge
        BP_RM_STATE_DTL        RM_STATE_DTL_HIST     Purge
        BP_Sponsor_State_EBMS			     Purge

                        ::::::::::::::::::::::::::::::
                        M O D I F I C A T I O N
                        ::::::::::::::::::::::::::::::
	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2005-10-06 S.Priani	4.2.2	BGTRK00073846
	2005-10-10 B.Moffat	4.2.2	Modified(Viacore Inc.) - indexes and FKs are NOT dropped.
	2006-01-30 S.Narain	6.1	CR 56054:Improve Archiving support
	2006-08-09 S.Priani	6.1	CR 102308
	2006-09-07 S.Priani	6.1	CR 102673
	2007-09-07 S.Priani	6.1.1	CR 104487

       ================================================================================

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
	v_Days		pls_integer :=  FLOOR( sysdate - TO_DATE( p_Date,'YYYY-MM-DD'));
	v_PurgeTS	bp_state_hdr.createdate%TYPE := TO_TIMESTAMP( p_Date, 'YYYY-MM-DD') + 1;
	v_TableName	USER_TABLES.TABLE_NAME%TYPE;
	v_SqlText	VARCHAR2(1024);
	v_DelChunkSize	NUMBER(10)		DEFAULT 5000;
	v_RowsPerBatch	NUMBER(10)		DEFAULT 1000000;
	v_TmpCount	INTEGER(12);
	v_RowCount	INTEGER(12);
	v_DelCount	INTEGER(12);
	v_Stmt_DELETE	VARCHAR2(1024);
	v_Stmt_SELECT	VARCHAR2(10);
	v_StartTime	INTEGER;
	v_EndTime	INTEGER;
	v_Duration	NUMBER(9,2);
	p_archive	PLS_INTEGER DEFAULT 0; --- can be an input parameter
						--- archiving is OFF by default
	v_Count_AS	NUMBER;
	v_Count_RN	NUMBER;
	v_LowestID	NUMBER;
	v_HighestID	NUMBER;

PROCEDURE LogMesg(p_Object VARCHAR2
			,p_Operation VARCHAR2
			,p_RowCount	NUMBER
			,p_Duration	NUMBER
		) IS
BEGIN
	dbms_output.put_line ( RPAD(p_Object, 30, ' ') || to_char( p_RowCount,'999999990' ) ||'   '|| SUBSTR(p_Operation,1,3)||'| '|| to_char( p_Duration, '999990.00' ));
END;
PROCEDURE PurgeTable(p_sqltxt VARCHAR2, p_sql2 VARCHAR2, p_Start NUMBER, p_Limit NUMBER) IS
BEGIN
	v_StartTime := DBMS_UTILITY.get_time;
    
	v_RowCount := COALESCE(p_Start -1, 0);
	v_DelCount := 0;
	LOOP
		v_SqlText := p_sqltxt ;
		EXECUTE IMMEDIATE v_SqlText USING v_RowCount + 1, v_RowCount + v_DelChunkSize;
		v_TmpCount := SQL%ROwCount;
		v_DelCount := v_DelCount + v_TmpCount;
		v_RowCount := v_RowCount + v_DelChunkSize;
		COMMIT;

		IF v_RowCount > p_Limit THEN
			EXIT;
		END IF;
	
	END LOOP;

	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	LogMesg(v_TableName, 'DEL', v_DelCount, v_Duration);
END PurgeTable;
PROCEDURE EmptyTable(p_TableName VARCHAR2) IS
	v_statement	VARCHAR2(500);
BEGIN
	v_Statement := 'TRUNCATE TABLE ' || p_tableName;
	EXECUTE IMMEDIATE v_Statement;
EXCEPTION
	WHEN Others THEN
		RAISE_APPLICATION_ERROR(-20991, SQLERRM);
END;
---Main

BEGIN
	v_PurgeTS := TO_TIMESTAMP( p_Date, 'YYYY-MM-DD') + 1;
	dbms_output.put_line ( RPAD('TABLE NAME', 30, ' ') || ' Row Count   OP | Time(Secs)' );
	dbms_output.put_line ( RPAD('-', 60, '-')  );


	BEGIN
		SELECT	COALESCE(CAST(propvalue AS INT), v_DelChunkSize)
		INTO	v_DelChunkSize
		FROM	DB_SYSPROPERTIES
		WHERE	propname = 'bcg.archive.db.rowsPerStatement'
		;

	EXCEPTION
		WHEN No_Data_Found THEN
			Null;  --- delete 5000 rows by Default per statement
	END;

	BEGIN
		SELECT	COALESCE(CAST(propvalue AS INT), v_RowsPerBatch)
		INTO	v_RowsPerBatch
		FROM	DB_SYSPROPERTIES
		WHERE	propname = 'bcg.archive.db.rowsPerBatch'
		;

	EXCEPTION
		WHEN No_Data_Found THEN
			Null;  ---  a million rows by Default per run
	END;

--- RN
	LOOP
		v_TableName := 'DB_PURGE_BPRN';
		EmptyTable(v_TableName);

		v_StartTime := DBMS_UTILITY.get_time;
		INSERT INTO DB_Purge_BPRN (RowCounter, InitBusinessId, ProcessInstanceId)
			SELECT ROW_NUMBER() OVER (ORDER BY updatedate) AS rcnt, INITBUSINESSID, PROCESSINSTANCEID
			FROM BP_RNSTATEHDR
			WHERE UPDATEDATE < v_PurgeTS
			AND	rownum < v_RowsPerBatch + 1
		;
		v_RowCount := SQL%ROwCount;
		COMMIT;
		v_count_RN := v_rowCount;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg('BPRN_WRK', 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_RN = 0;

		IF p_archive = 1 THEN
			AR_HISTORY_UPDATE('BP_rnStateDtl', v_Days);
		END IF;
		v_TableName := 'BP_RNSTATEDTL';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_rnstatedtl rnd '
		|| ' WHERE (InitBusinessID, ProcessInstanceID) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
		|| '	InitBusinessId, ProcessInstanceID '
		|| '	FROM	DB_PURGE_BPRN '
		|| ' 	WHERE RowCounter BETWEEN :low AND :high '
		|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_Count_RN);
    --
		IF p_archive = 1 THEN
			AR_HISTORY_UPDATE('BP_rnStateHdr', v_Days);
		END IF;
		v_TableName := 'BP_RNSTATEHDR';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_rnstatehdr rnh '
		|| ' WHERE (InitBusinessID, ProcessInstanceID) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
		|| '		InitBusinessId, ProcessInstanceID '
		|| '	FROM	DB_PURGE_BPRN '
		|| ' 	WHERE RowCounter BETWEEN :low AND :high '
		|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_Count_RN);
    --
		v_TableName := 'BP_RNSTATEHDRAUDITLOG';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_rnstatehdrauditlog rnl '
		|| ' WHERE (InitBusinessID, ProcessInstanceID) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
		|| '		InitBusinessId, ProcessInstanceID '
		|| '	FROM	DB_PURGE_BPRN '
		|| ' 	WHERE RowCounter BETWEEN :low AND :high '
		|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_Count_RN);
    --
		v_TableName := 'BP_SPONSOR_STATE';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_SPONSOR_STATE rnl '
		|| ' WHERE (InitBusinessID, ProcessInstanceID) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
		|| '		InitBusinessId, ProcessInstanceID '
		|| '	FROM	DB_PURGE_BPRN '
		|| ' 	WHERE RowCounter BETWEEN :low AND :high '
		|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_Count_RN);
	END LOOP;


-- AS1/AS2   :
	LOOP
		v_TableName := 'DB_PURGEBPWRK';
		EmptyTable(v_TableName);
	
		v_StartTime := DBMS_UTILITY.get_time;
		INSERT INTO DB_PurgeBPWRK (RowCounter, StateHdrId)
			SELECT ROW_NUMBER() OVER (ORDER BY createdate) AS rcnt, STATEHDRID
			FROM BP_STATE_HDR
			WHERE UPDATEDATE < v_PurgeTS
			AND	rownum < v_RowsPerBatch + 1
			;
		v_RowCount := SQL%ROwCount;
		COMMIT;
		v_count_AS := v_rowCount;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg('BP_AS_WRK', 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_AS = 0;
		IF p_archive = 1 THEN
			AR_HISTORY_UPDATE('BP_AS_State_Dtl', v_Days);
		END IF;
		v_TableName := 'BP_AS_STATE_DTL';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_as_state_dtl asd '
			|| ' WHERE StateHDrId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
			|| '	StateHDrID '
			|| '	FROM	DB_PURGEBPWRK '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_Count_AS);
	    --
		IF p_archive = 1 THEN
			AR_HISTORY_UPDATE('BP_AS_State_Hdr', v_Days);
		END IF;
		v_TableName := 'BP_AS_STATE_HDR';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_as_state_hdr ash '
			|| ' WHERE StateHDrId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
			|| '	StateHDrID '
			|| '	FROM	DB_PURGEBPWRK '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_Count_AS);
	    --
		IF p_archive = 1 THEN
			AR_HISTORY_UPDATE('BP_State_Hdr'   , v_Days);
		END IF;
		v_TableName := 'BP_STATE_HDR';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_state_hdr sh '
			|| ' WHERE StateHDrId IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
			|| '	StateHDrID '
			|| '	FROM	DB_PURGEBPWRK '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_Count_AS);
	END LOOP;
	
	-- ebXML   :
	SELECT	MIN(RMStateHdrId)
		, MAX(RMStateHdrId)
	INTO	v_LowestID
		,v_HighestID
	FROM	bp_RM_State_Hdr
	WHERE	UpdateDate < v_PurgeTS
	;
	IF v_LowestID IS NOT NULL AND v_HighestID IS NOT NULL THEN
		v_TableName := 'BP_RM_STATE_DTL';
		IF p_archive = 1 THEN
			AR_HISTORY_UPDATE(v_TableName, v_Days);
		END IF;
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_rm_state_dtl rmd '
			|| ' WHERE RMStateHdrId BETWEEN :low AND :high '
		;
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_LowestID -1, v_HighestID);
	    --
		v_TableName := 'BP_RM_STATE_HDR';
		IF p_archive = 1 THEN
			AR_HISTORY_UPDATE(v_TableName, v_Days);
		END IF;
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_rm_state_hdr rmh '
			|| ' WHERE RMStateHdrId BETWEEN :low AND :high '
		;
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_LowestID -1, v_HighestID);
	END IF;
	
	LOOP
		v_TableName := 'DB_PURGE_BPRN';
		EmptyTable(v_TableName);

		v_StartTime := DBMS_UTILITY.get_time;
		INSERT INTO DB_Purge_BPRN (RowCounter, InitBusinessId, ProcessInstanceId)
			SELECT ROW_NUMBER() OVER (ORDER BY createdate) AS rcnt, INITBUSINESSID, ProcessInstanceID
			FROM BP_SPONSOR_STATE_EBMS
			WHERE UPDATEDATE < v_PurgeTS
			AND	rownum < v_RowsPerBatch + 1
			;
		v_RowCount := SQL%ROwCount;
		COMMIT;
		v_count_RN := v_rowCount;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg('BP_ebXML_SPWRK', 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_RN = 0;
	
		v_TableName := 'BP_SPONSOR_STATE_EBMS';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_SPONSOR_STATE_ebms rnl '
			|| ' WHERE (InitBusinessID, ProcessInstanceID) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
			|| '		InitBusinessId, ProcessInstanceID '
			|| '	FROM	DB_PURGE_BPRN '
			|| ' 	WHERE RowCounter BETWEEN :low AND :high '
			|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_Count_RN);
	END LOOP;

	v_TableName := 'BP_DUPCHECK';
	SELECT	MIN(dupCheckId)
		, MAX(dupCheckId)
	INTO	v_LowestID
		,v_HighestID
	FROM	bp_dupcheck
	WHERE	createdate < v_PurgeTS
	;
	IF v_LowestID IS NOT NULL AND v_HighestID IS NOT NULL THEN
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from bp_dupcheck bdc '
			|| ' WHERE DupCheckId BETWEEN :low AND :high '
		;
		PurgeTable ( v_Stmt_DELETE, v_TableName, v_LowestID -1, v_HighestID);
	END IF;
EXCEPTION
        WHEN OTHERS THEN
	        ROLLBACK;
	        RAISE_APPLICATION_ERROR (-20999, '20002 AR_STATEENGINE_MAINTENANCE '|| SQLERRM);
End	AR_StateEngine_Maintenance;
/


-- AR_SUMMARY_MAINTENANCE.sql

CREATE OR REPLACE PROCEDURE AR_Summary_Maintenance
(
		p_ReturnCd OUT	INTEGER
	,	p_DebugFlg IN	INTEGER
	,	p_Date	IN	VARCHAR2
	,	p_TASKID IN	INTEGER
)
    IS
/*
	================================================================================
	Name:	   AR_Summary_Maintenance
	Create Date:    2004-03-23
	Author	  Shishir Narain
	This Procedure removes data from table:

	Table		  Action
	-----		  ------

  	LG_EVENT_EVENTSUMMARY		Purge
  	LG_EVENT_SUMMARY		Purge
  	LG_ACTIVITY_SUMMARY		Purge
  	LG_SUMMARY			Purge
  	LG_SUMMARY_RN			Purge
  	LG_SUMMARY_RN_MI		Purge
  	LG_SUMMARY_MI			Purge

			::::::::::::::::::::::::::::::
			M O D I F I C A T I O N
			::::::::::::::::::::::::::::::
	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2006-01-30 S.Narain	6.1	CR 56054:Improve Archiving support
	2007-09-07 S.Priani	6.1.1	CR 104487
	2008-08-05 S.Priani	6.2	F106141
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
	v_PurgeTS	TIMESTAMP(6)	;
	v_TableName	USER_TABLES.TABLE_NAME%TYPE;
	v_ProcName	USER_TABLES.TABLE_NAME%TYPE	:= 'AR_SUMMARY_MAINTENANCE';
	v_ExceptionFlg	NUMBER(2);
	v_DelChunkSize	NUMBER(10)		:= 5000;
	v_RowsPerBatch	NUMBER(10)		:= 1000000;
	v_begints		TIMESTAMP	DEFAULT fn_get_date;
	v_SqlText	VARCHAR2(1024);
	v_TmpCount	INTEGER(12);
	v_RowCount	INTEGER(12);
	v_DelCount	INTEGER(12);
	v_Stmt_DELETE	VARCHAR2(1024);
	v_Stmt_SELECT	VARCHAR2(10);
	v_StartTime	INTEGER;
	v_EndTime	INTEGER;
	v_Duration	NUMBER(9,2);
	v_ServerTSDiff		INTERVAL DAY(2) TO SECOND(5);
	v_Count_AS	NUMBER;
	v_count_summ	NUMBER;

PROCEDURE LogMesg(p_Object VARCHAR2
			,p_Operation VARCHAR2
			,p_RowCount	NUMBER
			,p_Duration	NUMBER
		) IS
BEGIN
	dbms_output.put_line ( RPAD(p_Object, 30, ' ') || to_char( p_RowCount,'999999990' ) ||'   '|| SUBSTR(p_Operation,1,3)||'| '|| to_char( p_Duration, '999990.00' ));
END;
PROCEDURE PurgeTable(p_sqltxt VARCHAR2, p_sql2 VARCHAR2, p_Start NUMBER, p_Limit NUMBER) IS
BEGIN
	v_StartTime := DBMS_UTILITY.get_time;
    
	v_RowCount := COALESCE(p_Start -1, 0);
	v_DelCount := 0;
	LOOP
		v_SqlText := p_sqltxt ;
		EXECUTE IMMEDIATE v_SqlText USING v_RowCount + 1, v_RowCount + v_DelChunkSize;
		v_TmpCount := SQL%ROwCount;
		v_DelCount := v_DelCount + v_TmpCount;
		v_RowCount := v_RowCount + v_DelChunkSize;
		COMMIT;

		IF v_RowCount > p_Limit THEN
			EXIT;
		END IF;
	
	END LOOP;

	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	LogMesg(v_TableName, 'DEL', v_DelCount, v_Duration);
	IF v_DelCount > 0 THEN
		LG_ARCH_TASK_PURGEDBI (v_ExceptionFlg , 0, p_TASKID, v_TableName, v_PurgeTS, v_DelCount, v_ProcName, 'Purge Successful', v_BeginTS + v_ServerTSDiff, sys_Extract_utc(systimestamp) + v_ServerTSDiff);
	END IF;
	COMMIT;
END PurgeTable;
PROCEDURE PurgeTableByDate(p_sqltxt VARCHAR2, p_TS TimeStamp, p_ChunkSize NUMBER) IS
BEGIN
	v_StartTime := DBMS_UTILITY.get_time;
    
	v_DelCount := 0;
	LOOP
		v_SqlText := p_sqltxt ;
		EXECUTE IMMEDIATE v_SqlText USING p_TS, p_ChunkSize;
		v_TmpCount := SQL%ROwCount;
		v_DelCount := v_DelCount + v_TmpCount;
		COMMIT;

		IF v_TmpCount = 0 THEN
			EXIT;
		END IF;
	
	END LOOP;

	v_EndTime := DBMS_UTILITY.get_time;
	v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
	LogMesg(v_TableName, 'DEL', v_DelCount, v_Duration);
	IF v_DelCount > 0 THEN
		LG_ARCH_TASK_PURGEDBI (v_ExceptionFlg , 0, p_TASKID, v_TableName, v_PurgeTS, v_DelCount, v_ProcName, 'Purge Successful', v_BeginTS + v_ServerTSDiff, sys_Extract_utc(systimestamp) + v_ServerTSDiff);
	END IF;
	COMMIT;
END PurgeTableByDate;
PROCEDURE EmptyTable(p_TableName VARCHAR2) IS
	v_statement	VARCHAR2(500);
BEGIN
	v_Statement := 'TRUNCATE TABLE ' || p_tableName;
	EXECUTE IMMEDIATE v_Statement;
EXCEPTION
	WHEN Others THEN
		RAISE_APPLICATION_ERROR(-20991, SQLERRM);
END;
---Main

BEGIN
	v_PurgeTS := TO_TIMESTAMP( p_Date, 'YYYY-MM-DD') + 1;
	dbms_output.put_line ( RPAD('TABLE NAME', 30, ' ') || ' Row Count   OP | Time(Secs)' );
	dbms_output.put_line ( RPAD('-', 60, '-')  );

	p_ReturnCd := 1;

	BEGIN
		SELECT	COALESCE(CAST(propvalue AS INT), v_DelChunkSize)
		INTO	v_DelChunkSize
		FROM	DB_SYSPROPERTIES
		WHERE	propname = 'bcg.archive.db.rowsPerStatement'
		;

	EXCEPTION
		WHEN No_Data_Found THEN
			Null;  --- delete 5000 rows by Default per statement
	END;

	BEGIN
		SELECT	COALESCE(CAST(propvalue AS INT), v_RowsPerBatch)
		INTO	v_RowsPerBatch
		FROM	DB_SYSPROPERTIES
		WHERE	propname = 'bcg.archive.db.rowsPerBatch'
		;

	EXCEPTION
		WHEN No_Data_Found THEN
			Null;  ---  a million rows by Default per run
	END;
	BEGIN
		SELECT	lat.CREATEDATE - v_BeginTS
		INTO	v_ServerTSDiff
		FROM	lg_arch_task lat
		WHERE	lat.taskid = p_TASKID
		;
	END;

--- EventSummary
	LOOP
		v_TableName := 'DB_PURGE_SUMMARY';
		EmptyTable(v_TableName);
		v_StartTime := DBMS_UTILITY.get_time;
		INSERT INTO DB_Purge_Summary (RowCounter, SummaryID, DayHour)
			SELECT ROW_NUMBER() OVER (ORDER BY DayHour) AS rcnt, EventSummaryID, DayHour
			FROM	LG_EVENTSUMMARY
			WHERE	DAYHOUR < v_PurgeTS
			AND	rownum < v_RowsPerBatch + 1
		;
		v_RowCount := SQL%ROwCount;
		COMMIT;
		v_count_summ := v_rowCount;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg(v_TableName||'EVT', 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_summ = 0;

		v_TableName := 'LG_EVENT_EVENTSUMMARY';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from LG_EVENT_EVENTSUMMARY rnd '
		|| ' WHERE (EventSummaryID, DayHour) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
		|| '	SummaryID, DayHour '
		|| '	FROM	DB_PURGE_SUMMARY '
		|| ' 	WHERE RowCounter BETWEEN :low AND :high '
		|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_count_summ);
    --
		v_TableName := 'LG_EVENTSUMMARY';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from LG_EVENTSUMMARY rnd '
		|| ' WHERE (EventSummaryID, DayHour) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
		|| '	SummaryID, DayHour '
		|| '	FROM	DB_PURGE_SUMMARY '
		|| ' 	WHERE RowCounter BETWEEN :low AND :high '
		|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_count_summ);
    --
	END LOOP;
--- ActivitySummary
	LOOP
		v_TableName := 'DB_PURGE_SUMMARY';
		EmptyTable(v_TableName);
		v_StartTime := DBMS_UTILITY.get_time;
		INSERT INTO DB_Purge_Summary (RowCounter, SummaryID, DayHour)
			SELECT ROW_NUMBER() OVER (ORDER BY DayHour) AS rcnt, SummaryID, DayHour
			FROM	LG_SUMMARY
			WHERE	DAYHOUR < v_PurgeTS
			AND	rownum < v_RowsPerBatch + 1
		;
		v_RowCount := SQL%ROwCount;
		COMMIT;
		v_count_summ := v_rowCount;
		v_EndTime := DBMS_UTILITY.get_time;
		v_Duration := MOD (v_EndTime - v_StartTime + POWER (2, 32), POWER (2, 32)) / 100;
		LogMesg(v_TableName||'Act', 'INS', v_RowCount, v_Duration);
	EXIT WHEN v_count_summ = 0;

		v_TableName := 'LG_SUMMARY_EVENT';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from LG_SUMMARY_EVENT rnd '
		|| ' WHERE (SummaryID, DayHour) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
		|| '	SummaryID, DayHour '
		|| '	FROM	DB_PURGE_SUMMARY '
		|| ' 	WHERE RowCounter BETWEEN :low AND :high '
		|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_count_summ);
    --
		v_TableName := 'LG_ACTIVITY_SUMMARY';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from LG_ACTIVITY_SUMMARY rnd '
		|| ' WHERE (SummaryID, DayHour) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
		|| '	SummaryID, DayHour '
		|| '	FROM	DB_PURGE_SUMMARY '
		|| ' 	WHERE RowCounter BETWEEN :low AND :high '
		|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_count_summ);
    --
		v_TableName := 'LG_SUMMARY';
		v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from LG_SUMMARY rnd '
		|| ' WHERE (SummaryID, DayHour) IN (SELECT /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ') */ '
		|| '	SummaryID, DayHour '
		|| '	FROM	DB_PURGE_SUMMARY '
		|| ' 	WHERE RowCounter BETWEEN :low AND :high '
		|| ')'
		;
		PurgeTable ( v_Stmt_DELETE, v_Stmt_SELECT, 0, v_count_summ);
    --
	END LOOP;
	
	v_TableName := 'LG_SUMMARY_MI';
	v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from LG_SUMMARY_MI su '
		|| ' WHERE	DayHourMI < :TS '
		|| ' AND	rownum < :RC '
		;
	PurgeTableByDate ( v_Stmt_DELETE, v_PurgeTS, v_DelChunkSize);
    --
	v_TableName := 'LG_SUMMARY_RN';
	v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from LG_SUMMARY_RN su '
		|| ' WHERE	DayHour < :TS '
		|| ' AND	rownum < :RC '
		;
	PurgeTableByDate ( v_Stmt_DELETE, v_PurgeTS, v_DelChunkSize);
    --
	v_TableName := 'LG_SUMMARY_RN_MI';
	v_Stmt_DELETE:= 'DELETE /*+ FIRST_ROWS(' || TO_CHAR(v_DelChunkSize) || ')  */ from LG_SUMMARY_RN_MI su '
		|| ' WHERE	DayHourMI < :TS '
		|| ' AND	rownum < :RC '
		;
	PurgeTableByDate ( v_Stmt_DELETE, v_PurgeTS, v_DelChunkSize);
    --
	p_ReturnCd := 0;
End AR_Summary_Maintenance;
/


-- AR_SUMMARY_MAINTENANCE_AH1.sql

CREATE OR REPLACE PROCEDURE AR_Summary_Maintenance_AH1
    (
    p_Date IN varchar2
    )
    IS
/*
	================================================================================
	Name:	   AR_Summary_Maintenance_AH1
	Create Date:    2008-01-01
	Author	  BHARANI J
	
	Creating new procedure same as AR_Summary_Maintenance.
	
	This Procedure removes data from table:

	Table		  Action
	-----		  ------

  	LG_ACTIVITY_SUMMARY		Purge
  	LG_EVENT_EVENTSUMMARY		Purge
  	LG_EVENT_SUMMARY		Purge
  	LG_SUMMARY_RN			Purge
  	LG_SUMMARY_MI			Purge
  	LG_SUMMARY			Purge

			::::::::::::::::::::::::::::::
			M O D I F I C A T I O N
			::::::::::::::::::::::::::::::
	---------- ------------ --------  ---------------------------------------------
	DATE	NAME		 Version   DESCRIPTION
	---------- ------------ --------  ---------------------------------------------
	2008-01-01 Bharani J    6.1.1    105521
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_Date Date;

Begin
select to_date(p_date,'YYYY-MM-DD') + 1 into v_Date from dual;

	dbms_output.put_line ( RPAD('TABLE NAME', 30, ' ') || ' Row Count   OP | Time(Secs)' );
	dbms_output.put_line ( RPAD('-', 60, '-')  );
--- moving all the following to AR_PURGE_HEADERS due to dependency

	AR_Purge_Archive('LG_ACTIVITY_SUMMARY', v_Date);

	AR_Purge_Archive('LG_EVENT_EVENTSUMMARY', v_Date);
	
	AR_Purge_Archive('LG_EVENTSUMMARY', v_Date);

	AR_Purge_Archive('LG_SUMMARY_RN', v_Date);

	AR_Purge_Archive('LG_SUMMARY_MI', v_Date);

	AR_Purge_Archive('LG_SUMMARY'   , v_Date);

End AR_Summary_Maintenance_AH1;
/


-- AR_TRUNCATE_TABLE.sql

CREATE OR REPLACE PROCEDURE AR_TRUNCATE_TABLE(
	p_ReturnCd		OUT	pls_INTEGER
	,p_DebugFlg		IN	pls_INTEGER
	,p_TableName		IN	VARCHAR2
)
    IS
/*
	================================================================================
	Name:	    AR_TRUNCATE_TABLE
	Create Date:    2003-09-30
	Author	   Vipin Goel
	 This Procedure Truncates Table <p_TableName>
			   ::::::::::::::::::::::::::::::
			   M O D I F I C A T I O N
			   ::::::::::::::::::::::::::::::
	---------- ------------ ------- ---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2003-09-30 V.Goel	4.2.2.0	Created
	================================================================================

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

    v_SqlText Varchar2(1000);
Begin
    v_SqlText := 'Truncate Table '||p_TableName ;

    DBMS_OUTPUT.Put_Line(v_SqlText);
    Execute Immediate v_SqlText;
	p_ReturnCd := 0;
Exception
	When Others Then
	Rollback;
	Raise_Application_Error (-20999, '20002 AR_TRUNCATE_TABLE '|| SQLERRM);
End AR_Truncate_Table;
/


-- BP_AS_State_HdrI_AH1.sql

CREATE OR REPLACE PROCEDURE         BP_AS_STATE_HDRI_AH1
    (
    p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    ,p_SQLERRM out varchar2
    ,p_STATEHDRID out NUMBER -- If Provided, only create AS_State_Dtl record
    ,p_AS_STATE_DTLID OUT NUMBER
    ,P_TPA_FILLFLG in pls_integer
    ,p_WorkGroupName     in Varchar2
    ,p_StateInstanceName in Varchar2
    ,p_CHANNELID in NUMBER
    ,p_STATESTATUSCD in pls_integer -- 0 = Successful, 1 = Fatal Error
    ,p_RESTARTFLG in NUMBER
    ,p_PROCESSINSTANCEID in VARCHAR2
    ,p_INITBUSINESSID in VARCHAR2
    ,p_FRBUSINESSID in VARCHAR2
    ,p_TOBUSINESSID in VARCHAR2
    ,p_CURRENTSTATECD in VARCHAR2
    ,p_NEXTACTIONCD in VARCHAR2
    ,p_MSGSENTSTATUSCD in VARCHAR2
    ,p_EXPECTACKFLG in NUMBER
    ,p_ACKRCVFLG in NUMBER
    ,p_ACKSENTFLG in NUMBER
    ,p_DESTName in VARCHAR2
    ,p_UPDATEPGMNAME in VARCHAR2
    ,p_PackageVer   in VARCHAR2     -- was p_AS_VER
    ,p_MDNREQUIREDFLG in NUMBER
    ,p_MDNSIGREQUIREDFLG in NUMBER
    ,p_MDNASYNFLG in NUMBER
    ,p_MDNSIGNATUREALGORITHM in VARCHAR2
    ,p_COMPRESSEDFLG in NUMBER
-- AS State Detail --------------------------------------
    ,p_TARGETURL in VARCHAR2
    ,p_MSGDIGEST in VARCHAR2
    ,p_MSGDIGESTALGORITHM in VARCHAR2
    ,p_EXTERNALMSGID in VARCHAR2
    ,p_DOCTIMESTAMP in Timestamp
    ,p_DISPOSITIONSTATUSCD in VARCHAR2
    ,p_INBOUNDVUID in VARCHAR2
    ,p_OUTBOUNDVUID in VARCHAR2
    ,p_DTL_FRBUSINESSID in VARCHAR2
    ,p_DTL_TOBUSINESSID in VARCHAR2
    ,p_PackageName In Varchar2    -- Hdr
    ,p_TPA_MaxDocRetryCnt in NUMBER
    ,p_TPA_TIMETOACK in NUMBER
    ,p_RESPONSESERVLETURI     in varchar2
    )
AS
/*
    ================================================================================
    Proc Name:       BP_AS_State_HdrI
    Create Date:     2003-03-10 11:55:56
    Author:          V.Goel

                     Check to see if a Header Exists based upon InitBusinessID and ProcessInstanceID.
                     No  :Insert record in BP_STATE_HDR and BP_StateWorkGroup Using BP_STATE_HDRI_AH1
                          Insert BP_AS_State_Hdr/Dtl records
                     Yes :Update BP_STATE_HDR and Insert in BP_AS_State_Dtl

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_destTypeID            PLS_INTEGER default FN_GET_DESTTYPEID (p_DestName);

    v_PK_Value              number default 0;
    v_RowCnt                integer default 0;
    v_BeginTS               timestamp default FN_Get_Date; -- Performance related variables
    v_ProcName              varchar(30) default 'BP_AS_STATE_HDRI_AH1';
    v_ExceptionFlg          pls_integer default 0;
    v_ReturnCd              pls_integer;
    v_Insert_Update         Varchar2(12);
    v_MsgDigest             BP_AS_State_Dtl.MsgDigest%Type;
    v_CurrentStateCd        BP_State_Hdr.CurrentStateCd%TYPE;
    v_RetryCnt              pls_integer := Null;  --BP_State_Hdr.RetryCnt%TYPE;
    v_MsgSentStatusCd       BP_State_Hdr.MsgSentStatusCd%TYPE := null;
    v_TPA_MaxDocRetryCnt    pls_integer;   -- must be varchar2 when using with channelDef
    v_TPA_TimeToAck         pls_integer;   -- must be varchar2 when using with channelDef
    v_TPA_TimeToPerform     pls_integer;   -- must be varchar2 when using with channelDef
    v_BatchDocProcessedCnt  BP_State_Hdr.BatchDocProcessedCnt%TYPE;
    v_RC1                   db_ref.RCT;
---------
v_ParamsTxt       varchar(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', P_TPA_FILLFLG=>'||P_TPA_FILLFLG||CHR(10)
-- BP_State Hdr
        ||', p_WorkGroupName=>'||p_WorkGroupName||CHR(10)
        ||', p_StateInstanceName=>'||  p_StateInstanceName||CHR(10)
        ||', p_CHANNELID=>'|| p_CHANNELID||CHR(10)
        ||', p_STATESTATUSCD=>'||p_STATESTATUSCD||CHR(10)
        ||', p_RESTARTFLG=>'||p_RESTARTFLG||CHR(10)
        ||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
        ||', p_INITBUSINESSID=>'||p_INITBUSINESSID ||CHR(10)
        ||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
        ||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
        ||', p_CURRENTSTATECD=>'||p_CURRENTSTATECD||CHR(10)
        ||', p_NEXTACTIONCD=>'||p_NEXTACTIONCD||CHR(10)
        ||', p_MSGSENTSTATUSCD=>'||p_MSGSENTSTATUSCD||CHR(10)
        ||', p_EXPECTACKFLG=>'||p_EXPECTACKFLG||CHR(10)
        ||', p_ACKRCVFLG=>'||p_ACKRCVFLG||CHR(10)
        ||', p_ACKSENTFLG=>'||p_ACKSENTFLG ||CHR(10)
        ||', p_DESTName=>'||p_DESTName||CHR(10)
        ||', p_UPDATEPGMNAME=>'||p_UPDATEPGMNAME||CHR(10)
-- BP_AS_State_Hdr
        ||', p_STATEHDRID=>'||p_STATEHDRID||CHR(10)
        ||', p_AS_STATE_DTLID=>'||p_AS_STATE_DTLID||CHR(10)
        ||', p_PackageVer=>'||p_PackageVer||CHR(10)
        ||', p_MDNREQUIREDFLG=>'||p_MDNREQUIREDFLG||CHR(10)
        ||', p_MDNSIGREQUIREDFLG=>'||p_MDNSIGREQUIREDFLG||CHR(10)
        ||', p_MDNASYNFLG=>'||p_MDNASYNFLG||CHR(10)
        ||', p_MDNSIGNATUREALGORITHM=>'||p_MDNSIGNATUREALGORITHM||CHR(10)
        ||', p_COMPRESSEDFLG=>'||p_COMPRESSEDFLG||CHR(10)
        --------------------------------------------------------------------
        ||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
        ||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
        ||', p_MSGSENTSTATUSCD=>'||p_MSGSENTSTATUSCD||CHR(10)
        ||', p_TARGETURL=>'||p_TARGETURL||CHR(10)
        ||', p_MSGDIGEST=>'||p_MSGDIGEST||CHR(10)
        ||', p_MSGDIGESTALGORITHM=>'||p_MSGDIGESTALGORITHM||CHR(10)
        ||', p_EXTERNALMSGID=>'||p_EXTERNALMSGID||CHR(10)
        ||', p_DOCTIMESTAMP=>'||p_DOCTIMESTAMP||CHR(10)
        ||', p_DISPOSITIONSTATUSCD=>'||p_DISPOSITIONSTATUSCD||CHR(10)
        ||', p_INBOUNDVUID=>'||p_INBOUNDVUID||CHR(10)
        ||', p_OUTBOUNDVUID=>'||p_OUTBOUNDVUID||CHR(10)
        ||', p_PackageName=>'|| p_PackageName||CHR(10)
        ||', p_RESPONSESERVLETURI=>'|| p_RESPONSESERVLETURI||CHR(10)
        ;
--
-- Procedures
--
Procedure Get_MsgDigest;
--
Procedure InsertDtl AS
BEGIN
    Insert into BP_AS_STATE_DTL(
        AS_STATEDTLID
        ,STATEHDRID
        ,FRBUSINESSID
        ,TOBUSINESSID
        ,MSGSENTSTATUSCD
        ,TARGETURL
        ,MSGDIGEST
        ,MSGDIGESTALGORITHM
        ,EXTERNALMSGID
        ,DOCTIMESTAMP
        ,DISPOSITIONSTATUSCD
        ,INBOUNDVUID
        ,OUTBOUNDVUID
        ,PackageName
        ,PackageVer
        ,RESPONSESERVLETURI
    ) Values (
        nvl(p_AS_STATE_DTLID, BP_AS_STATE_DTL_SEQ.nextval)
        ,p_STATEHDRID
        ,p_DTL_FRBUSINESSID
        ,p_DTL_TOBUSINESSID
        ,p_MSGSENTSTATUSCD
        ,p_TARGETURL
        ,p_MSGDIGEST
        ,p_MSGDIGESTALGORITHM
        ,p_EXTERNALMSGID
        ,p_DOCTIMESTAMP
        ,p_DISPOSITIONSTATUSCD
        ,p_INBOUNDVUID
        ,p_OUTBOUNDVUID
        ,p_PackageName
        ,p_PackageVer
        ,p_RESPONSESERVLETURI
    )Returning AS_STATEDTLID Into p_AS_STATE_DTLID;
    v_RowCnt := sql%ROWCOUNT;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
            RAISE_APPLICATION_ERROR(-20999,'20100 - BP_AS_State_HdrI: No Data Found');
        When DUP_VAL_ON_INDEX then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            p_returnCd := 2;
            RAISE_APPLICATION_ERROR(-20999,'20100 - BP_AS_State_HdrI: Duplicate Row');
        When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            p_SQLERRM := SQLERRM;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||', '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - BP_AS_State_HdrI: '||SQLERRM);
END InsertDtl;
--
Procedure BP_AS_State_HdrI is
Begin
    Insert into BP_AS_STATE_HDR(
        STATEHDRID
        ,AS_STATE_DTLID
        ,MDNREQUIREDFLG
        ,MDNSIGREQUIREDFLG
        ,MDNASYNFLG
        ,MDNSIGNATUREALGORITHM
        ,COMPRESSEDFLG
    ) Values (
        p_STATEHDRID
        ,BP_AS_STATE_DTL_SEQ.nextval
        ,p_MDNREQUIREDFLG
        ,p_MDNSIGREQUIREDFLG
        ,p_MDNASYNFLG
        ,p_MDNSIGNATUREALGORITHM
        ,p_COMPRESSEDFLG
    ) returning AS_State_DtlID into p_AS_STATE_DTLID;
    v_RowCnt := sql%ROWCOUNT;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    InsertDtl;
    v_MsgDigest := p_MsgDigest;
    EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
            RAISE_APPLICATION_ERROR(-20999,'20100 - BP_AS_State_HdrI: No Data Found');
        When DUP_VAL_ON_INDEX then
            InsertDtl;
--            Get_MsgDigest;
        When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            p_SQLERRM := SQLERRM;
--            DB_ProcAuditLogI(v_ProcName, v_BeginTS, p_SQLERRM, p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||', '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - BP_AS_State_HdrI: '||SQLERRM);
End BP_AS_State_HdrI;
--
Procedure BP_AS_State_HdrU is
Begin
    Update BP_AS_State_Hdr
        Set AS_STATE_DTLID = BP_AS_STATE_DTL_SEQ.nextval
    Where STATEHDRID = p_STATEHDRID
    Returning AS_State_DtlID into p_AS_STATE_DTLID;
    DBMS_OUTPUT.PUT_LINE(' Update AS State Hdr  AS_StateDTLID '||p_AS_STATE_DTLID);
    EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
            RAISE_APPLICATION_ERROR(-20999,'20100 - BP_AS_State_HdrU: No Data Found');
        When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            p_SQLERRM := SQLERRM;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||', '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - BP_AS_State_HdrU: '||SQLERRM);
End BP_AS_State_HdrU;
--

--
Procedure BP_State_HdrI is
Begin
     v_RetryCnt := 0;
     v_CurrentStateCd     := p_CurrentStateCd;
     v_MsgSentStatusCd    := p_MsgSentStatusCd;
    BP_STATE_HDRI_AH1(
    P_RETURNCD              => v_ReturnCd
    ,P_DEBUGFLG             => 0
    ,P_SQLERRM             => P_SQLERRM
        ,P_STATEHDRID           => P_STATEHDRID
    ,P_TPA_FILLFLG          => 1  -- AutoFill
    ,P_WORKGROUPNAME        => P_WORKGROUPNAME
    ,P_STATEINSTANCENAME    => P_STATEINSTANCENAME
    ,P_CHANNELID            => P_CHANNELID  --  ?????
    ,P_STATESTATUSCD        => P_STATESTATUSCD
    ,P_RESTARTFLG           => P_RESTARTFLG
    ,P_PROCESSINSTANCEID    => P_PROCESSINSTANCEID
    ,P_PROCESSEXPIREDATE    => NULL -- To_Date('2003-03-13 10:49:21', 'YYYY-MM-DD HH24:MI:SS')
    ,P_PROCESSSTARTDATE     => v_BeginTS -- To_Date('2003-03-13 10:49:21', 'YYYY-MM-DD HH24:MI:SS')
    ,P_INITBUSINESSID       => P_INITBUSINESSID
    ,P_FRBUSINESSID         => P_FRBUSINESSID
    ,P_TOBUSINESSID         => P_TOBUSINESSID
    ,P_CURRENTSTATECD       => P_CURRENTSTATECD
    ,P_LASTSTATECD          => ''
    ,P_NEXTACTIONCD         => P_NEXTACTIONCD
    ,P_MSGSENTSTATUSCD      => ''
    ,P_RETRYCNT             => 0
    ,P_TPA_MAXDOCRETRYCNT   => Nvl(p_TPA_MaxDocRetryCnt, 3)   -- *******Temporary *******************
    ,P_TPA_TIMETOACK        => Nvl(p_TPA_TimeToAck, 2)        -- *******Temporary *******************
    ,P_TPA_TIMETOPERFORM    => NULL
    ,P_NEXTRETRYDATE        => NULL -- To_Date('2003-03-13 10:49:21', 'YYYY-MM-DD HH24:MI:SS')
    ,P_EXPECTACKFLG         => P_EXPECTACKFLG
    ,P_ACKRCVFLG            => P_ACKRCVFLG
    ,P_ACKSENTFLG           => P_ACKSENTFLG
    ,P_OUTOFSEQFLG          => NULL
    ,P_EXPECTRESPDOCFLG     => NULL
    ,P_PROCESSRETRYDATE     => v_BeginTS + (p_TPA_TimeToAck/1440) -- To_Date('2003-03-13 10:49:21', 'YYYY-MM-DD HH24:MI:SS')
    ,P_RESPONSERCVFLG       => NULL
    ,P_STATESCENARIOID      => NULL
    ,P_SCENARIOREQIREDFLG   => NULL
    ,P_BATCHCNT             => NULL
    ,P_BATCHDOCPROCESSEDCNT => NULL
    ,P_DESTTYPEID           => v_DESTTYPEID
    ,P_UPDATEPGMNAME        => P_UPDATEPGMNAME
    ,P_ERRORCD              => ''
    ,P_ERRORREASONTEXT      => ''
    );
End BP_State_HdrI;
--
Procedure BP_State_HdrU is
    Begin
    BP_STATE_HDRU_AH2(
    P_RETURNCD              => P_RETURNCD
    ,P_DEBUGFLG             => P_DEBUGFLG
    ,P_SQLERRM              => P_SQLERRM
    ,RC1                    => v_RC1
    ,P_WORKGROUPNAME        => P_WORKGROUPNAME
    ,P_STATEHDRID           => P_STATEHDRID
    ,p_STATEInstanceName    => p_STATEInstanceName
    ,P_CHANNELID            => P_CHANNELID
    ,P_STATESTATUSCD        => P_STATESTATUSCD
    ,P_RESTARTFLG           => P_RESTARTFLG
    ,P_PROCESSINSTANCEID    => P_PROCESSINSTANCEID
    ,P_PROCESSEXPIREDATE    => Null   --P_PROCESSEXPIREDATE
    ,P_PROCESSSTARTDATE     => Null   --P_PROCESSSTARTDATE
    ,P_INITBUSINESSID       => P_INITBUSINESSID
    ,P_FRBUSINESSID         => P_FrBUSINESSID
    ,P_TOBUSINESSID         => P_ToBUSINESSID
    ,P_CURRENTSTATECD       => P_CURRENTSTATECD
    ,P_LASTSTATECD          => Null   --P_LASTSTATECD
    ,P_NEXTACTIONCD         => P_NEXTACTIONCD
    ,P_MSGSENTSTATUSCD      => P_MSGSENTSTATUSCD
    ,P_RETRYCNT             => Null   --P_RETRYCNT
    ,P_TPA_MAXDOCRETRYCNT   => Null   --P_TPA_MAXDOCRETRYCNT
    ,P_TPA_TIMETOACK        => Null   --P_TPA_TIMETOACK
    ,P_TPA_TIMETOPERFORM    => Null   --P_TPA_TIMETOPERFORM
    ,P_NEXTRETRYDATE        => Null   --P_NEXTRETRYDATE
    ,P_EXPECTACKFLG         => P_EXPECTACKFLG
    ,P_ACKRCVFLG            => P_ACKRCVFLG
    ,P_ACKSENTFLG           => P_ACKSENTFLG
    ,P_OUTOFSEQFLG          => Null   --P_OUTOFSEQFLG
    ,P_EXPECTRESPDOCFLG     => Null   --P_EXPECTRESPDOCFLG
    ,P_PROCESSRETRYDATE     => v_BeginTS + (p_TPA_TimeToAck/1440)   --P_PROCESSRETRYDATE
    ,P_RESPONSERCVFLG       => Null   --P_RESPONSERCVFLG
    ,P_STATESCENARIOID      => Null   --P_STATESCENARIOID
    ,P_SCENARIOREQIREDFLG   => Null   --P_SCENARIOREQIREDFLG
    ,P_BATCHCNT             => Null   --P_BATCHCNT
    ,P_BATCHDOCPROCESSEDCNT => Null   --P_BATCHDOCPROCESSEDCNT
    ,P_DESTNAME             => p_DESTName
    ,P_UPDATEPGMNAME        => P_UPDATEPGMNAME
    ,P_ERRORCD              => Null   --P_ERRORCD
    ,P_ERRORREASONTEXT      => Null   --P_ERRORREASONTEXT
    );
-- Open RC1
    Fetch v_RC1 into  v_CurrentStateCd,
                    v_TPA_MaxDocRetryCnt,
                    v_RetryCnt,
                    v_MsgSentStatusCd,
                    v_BatchDocProcessedCnt,  -- Added B.Moffat 2003-04-19
                    v_TPA_TIMETOACK;          -- Added B.Moffat 2003-04-19
    Close v_RC1;
End BP_State_HdrU;
--
Procedure Get_StateHdrID(p2_INITBUSINESSID IN Varchar2,
                         p2_PROCESSINSTANCEID IN Varchar2,
                         p2_WorkGroupName   in Varchar2,
                         p2_StateInstanceName   in Varchar2,
                         p2_StateHdrID out Number)
                         is
Begin
    Select StateHdrID INTO p2_StateHdrID
    From BP_State_Hdr
    Where INITBUSINESSID    = p2_INITBUSINESSID
    And   PROCESSINSTANCEID = p2_PROCESSINSTANCEID
    And   WorkGroupName     = p2_WorkGroupName
    And   StateInstanceName = p2_StateInstanceName
    ;
    Exception
            When NO_DATA_FOUND THEN
               p2_StateHdrID := Null;
            When OTHERS Then
                p_ReturnCd     := 1;
                v_ExceptionFlg := 1;
                p_SQLERRM := SQLERRM;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                RAISE_APPLICATION_ERROR(-20999,'20100 - BP_AS_State_HdrI: '||SQLERRM);
End Get_StateHdrID;
--
Procedure Get_MsgDigest is
Begin
    Select MsgDigest Into v_MsgDigest
    From BP_AS_State_Dtl
    Where STATEHDRID = p_STATEHDRID
    And   AS_StateDtlId = (Select MIN(AS_StateDtlId) From BP_AS_State_Dtl
                                   Where STATEHDRID = p_STATEHDRID)
    ;
    Exception
            When NO_DATA_FOUND THEN
               v_MsgDigest := Null;
            When OTHERS Then
                p_ReturnCd     := 1;
                v_ExceptionFlg := 1;
                p_SQLERRM := SQLERRM;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End Get_MsgDigest;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    p_ReturnCd := 0;
    Get_StateHdrID(p_INITBUSINESSID,
                    p_PROCESSINSTANCEID,
                    p_WorkGroupName,
                    nvl(p_StateInstanceName, -1),
                    p_StateHdrID);
    DBMS_OUTPUT.PUT_LINE('Return cd '||p_ReturnCd||' StateHdrID '||p_StateHdrID);
    IF p_ReturnCd = 0 Then            -- ReturnCD
        If p_StateHdrID is null Then  ---- v_StateHdrID
            BP_State_HdrI;
            v_Insert_Update := 'INSERT';
            IF p_ReturnCd = 0 Then
                BP_AS_State_HdrI;
            END IF;
        Else
            v_Insert_Update := 'UPDATE';
            BP_State_HdrU;
            Get_MsgDigest;
            IF p_ReturnCd = 0 Then
                BP_AS_State_HdrU;
                InsertDtl ;             -- Insert Detail record
            END IF;
        End If;                       ----v_StateHdrID
    END IF;                           -- ReturnCD
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for select
                    v_CurrentStateCd      CurrentStateCd,
                    v_TPA_MaxDocRetryCnt  TPA_MaxDocRetryCnt,
                    v_RetryCnt            RetryCnt,
                    v_MsgSentStatusCd     MsgSentStatusCd,
                    v_MsgDigest           MsgDigest,
                    v_Insert_Update       Insert_Update
                  From Dual;
END BP_AS_State_HdrI_AH1;
/


-- BP_DUPCHECKI_AH1.sql

CREATE OR REPLACE PROCEDURE         BP_DUPCHECKI_AH1 (p_returnCd out pls_integer, p_debugFlg in
pls_integer, RC1 out db_ref.RCT
,p_ACTIVITYID in VARCHAR2
,p_VUID in VARCHAR2
,p_KEY01 in VARCHAR2 default '-1'
,p_KEY02 in VARCHAR2 default '-1'
,p_KEY03 in VARCHAR2 default '-1'
,p_KEY04 in VARCHAR2 default '-1'
,p_KEY05 in VARCHAR2 default '-1'
,p_KEY06 in VARCHAR2 default '-1'
,p_KEY07 in VARCHAR2 default '-1'
,p_KEY08 in VARCHAR2 default '-1'
,p_KEY09 in VARCHAR2 default '-1'
,p_KEY10 in VARCHAR2 default '-1'
) as
/*
================================================================================
Proc Name:      BP_DUPCHECKI_AH1
Create Date:    02/19/02 02:00:34 PM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
v_ActivityId bp_dupCheck.ActivityId%type;
v_Vuid       bp_dupCheck.Vuid%type;
v_Createdate bp_dupCheck.Createdate%type;
--
-- Procedures
--
Procedure ProcAudit is
begin
If p_debugFlg =1 Then
DB_ProcAuditLogI_Ah1('BP_DUPCHECKI_AH1',v_beginTS, 'p_returnCd'||','||p_debugFlg
||','||'RC1'
||','||p_ACTIVITYID
||','||p_VUID
||','||p_KEY01
||','||p_KEY02
||','||p_KEY03
||','||p_KEY04
||','||p_KEY05
||','||p_KEY06
||','||p_KEY07
||','||p_KEY08
||','||p_KEY09
||','||p_KEY10
);
End if ;
End ProcAudit;
Procedure Display_param is
begin
dbms_output.put_line('Record '
     ||p_KEY01
||','||p_KEY02
||','||p_KEY03
||','||p_KEY04
||','||p_KEY05
||','||p_KEY06
||','||p_KEY07
||','||p_KEY08
||','||p_KEY09
||','||p_KEY10  );
end;
--
-- MAin
--
BEGIN
 p_returnCd := 0;
 Begin
    Insert into BP_DUPCHECK(
    DUPCHECKID
,   ACTIVITYID
,   VUID
,   KEY01
,   KEY02
,   KEY03
,   KEY04
,   KEY05
,   KEY06
,   KEY07
,   KEY08
,   KEY09
,   KEY10, createDate
) Values (
BP_DUPCHECK_SEQ.nextval
,   p_ACTIVITYID
,   p_VUID
,   nvl(p_KEY01,'-1')
,   nvl(p_KEY02,'-1')
,   nvl(p_KEY03,'-1')
,   nvl(p_KEY04,'-1')
,   nvl(p_KEY05,'-1')
,   nvl(p_KEY06,'-1')
,   nvl(p_KEY07,'-1')
,   nvl(p_KEY08,'-1')
,   nvl(p_KEY09,'-1')
,   nvl(p_KEY10,'-1')
,  v_BeginTS
) returning ACTIVITYID, VUID, CreateDate into v_ACTIVITYID, v_VUID, v_CreateDate;
ProcAudit;
EXCEPTION
  When Dup_Val_On_Index Then
  begin
    Select x.ACTIVITYID, x.VUID, x.CREATEDATE
    into v_ACTIVITYID, v_VUID, v_CreateDate
    From BP_DUPCHECK x
    Where
    x.KEY01= nvl(p_KEY01,'-1') and
    x.KEY02= nvl(p_KEY02,'-1') and
    x.KEY03= nvl(p_KEY03,'-1') and
    x.KEY04= nvl(p_KEY04,'-1') and
    x.KEY05= nvl(p_KEY05,'-1') and
    x.KEY06= nvl(p_KEY06,'-1') and
    x.KEY07= nvl(p_KEY07,'-1') and
    x.KEY08= nvl(p_KEY08,'-1') and
    x.KEY09= nvl(p_KEY09,'-1') and
    x.KEY10= nvl(p_KEY10,'-1')
    ;
  Exception
   when no_data_found then Display_param ;
   Raise_application_error (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
  end;
  When Others Then
   Display_param ;
   ProcAudit;
   p_returnCd := 1;
   Raise_application_error (-20999,'20001' || SQLCODE|| ' '||SQLERRM);
 End;
    if RC1%IsOpen then close RC1; end if;
    Open RC1 FOR
        Select v_ACTIVITYID ACTIVITYID,
               v_VUID VUID,
               v_CREATEDATE CREATEDATE
        From Dual;

END BP_DUPCHECKI_AH1;
/


-- BP_PROCESS_LOGD.sql

CREATE OR REPLACE Procedure BP_PROCESS_LOGD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,P_Activityid in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_PROCESS_LOGD
	Create Date:     2008-08-12 15:48:55
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_PROCESS_LOGD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', P_Activityid=>'||P_Activityid||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	DELETE FROM BP_PROCESS_LOG
	WHERE ACTIVITYID = P_ACTIVITYID
	;
	
	v_RowCnt := SQL%ROWCOUNT;
	
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	
	p_ReturnCd := 0;
	
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 BP_PROCESS_LOGD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End BP_PROCESS_LOGD;
/

-- BP_PROCESS_LOGI_AH1.sql

CREATE OR REPLACE Procedure BP_PROCESS_LOGI_AH1
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ACTIVITYID in VARCHAR2
	,p_STATUSCD in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       BP_PROCESS_LOGI_AH1
	Create Date:     2003-10-06 08:48:31
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-10-06 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_PROCESS_LOGI_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_STATUSCD=>'||p_STATUSCD||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into BP_PROCESS_LOG(
		ACTIVITYID
		,STATUSCD
	) Values (
		p_ACTIVITYID
		,p_STATUSCD
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 BP_PROCESS_LOGI_AH1' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
--			RAISE_APPLICATION_ERROR (-20999,'20001 BP_PROCESS_LOGI_AH1' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
--			RAISE_APPLICATION_ERROR (-20999,'20002 BP_PROCESS_LOGI_AH1' || SQLCODE|| ' '||SQLERRM);
END BP_PROCESS_LOGI_AH1;
/


-- BP_PROCESS_LOGI_AH2.sql

CREATE OR REPLACE PROCEDURE BP_PROCESS_LOGI_AH2
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
    ,RC1 out db_ref.RCT
	,p_ACTIVITYID in VARCHAR2
	,p_STATUSCD in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       BP_PROCESS_LOGI_AH2
	Create Date:     2003-10-06 08:48:31
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-10-06 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_PROCESS_LOGI_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_STATUSCD=>'||p_STATUSCD||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into BP_PROCESS_LOG(
		ACTIVITYID
		,STATUSCD
	) Values (
		p_ACTIVITYID
		,p_STATUSCD
	);
	p_ReturnCd := 0;
    If RC1%IsOpen Then Close RC1; End If;
    Open RC1 FOR Select 1 From Dual;
    
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 BP_PROCESS_LOGI_AH2' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
--			RAISE_APPLICATION_ERROR (-20999,'20001 BP_PROCESS_LOGI_AH2' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
--			RAISE_APPLICATION_ERROR (-20999,'20002 BP_PROCESS_LOGI_AH2' || SQLCODE|| ' '||SQLERRM);
END BP_PROCESS_LOGI_AH2;
/


-- BP_PROCESS_LOGS.sql

CREATE OR REPLACE PROCEDURE BP_PROCESS_LOGS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ACTIVITYID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_PROCESS_LOGS
	Create Date:     2003-10-06 08:48:32
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-10-06 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_PROCESS_LOGS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ACTIVITYID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		RTRIM(ACTIVITYID) ACTIVITYID
		,STATUSCD
		,CREATEDATE
	FROM
		BP_PROCESS_LOG
	Where
		ACTIVITYID = p_ACTIVITYID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		RTRIM(ACTIVITYID) ACTIVITYID
		,STATUSCD
		,CREATEDATE
	FROM
		BP_PROCESS_LOG
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End BP_PROCESS_LOGS;
/


-- BP_PROCESS_LOGU_AH1.sql

CREATE OR REPLACE Procedure BP_PROCESS_LOGU_AH1
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
--	, RC1 out db_ref.RCT
	,p_ACTIVITYID in VARCHAR2
	,p_STATUSCD in NUMBER
) as

/*
	================================================================================
	Proc Name:       BP_PROCESS_LOGU_AH1
	Create Date:     2003-10-06 08:48:32
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-10-06 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_PROCESS_LOGU_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_STATUSCD=>'||p_STATUSCD||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update BP_PROCESS_LOG
	SET
		STATUSCD     = p_STATUSCD
		, UPDATEDATE = v_BeginTS
	Where
		ACTIVITYID = p_ACTIVITYID
        ;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN ZeroUpdate then
            p_ReturnCd := 3;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - BP_PROCESS_LOGU_AH1: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - BP_PROCESS_LOGU_AH1:'||SQLERRM);
End BP_PROCESS_LOGU_AH1;
/


-- BP_PROCESS_LOGU_AH2.sql

CREATE OR REPLACE PROCEDURE BP_PROCESS_LOGU_AH2
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_ACTIVITYID in VARCHAR2
	,p_STATUSCD in NUMBER
) as

/*
	================================================================================
	Proc Name:       BP_PROCESS_LOGU_AH2
	Create Date:     2003-10-06 08:48:32
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-10-06 SYSTEM           3.5 Created
                                    Only diff from BP_Process_LogU_ah1 is RC1 as input for console.                    
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_PROCESS_LOGU_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_STATUSCD=>'||p_STATUSCD||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update BP_PROCESS_LOG
	SET
		STATUSCD     = p_STATUSCD
		, UPDATEDATE = v_BeginTS
	Where
		ACTIVITYID = p_ACTIVITYID
        ;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	p_ReturnCd := 0;
    If RC1%IsOpen Then Close RC1; End If;
    Open RC1 FOR Select 1 From Dual;
    
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN ZeroUpdate then
            p_ReturnCd := 3;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			--RAISE_APPLICATION_ERROR(-20999,'20100 - BP_PROCESS_LOGU_AH2: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - BP_PROCESS_LOGU_AH2:'||SQLERRM);
End BP_PROCESS_LOGU_AH2;
/


-- BP_RM_STATE_DTLD.sql

--> EXECUTE DB_GENPROC.GENPROC_DELETE('BP_RM_STATE_DTL');
CREATE OR REPLACE PROCEDURE BP_RM_STATE_DTLD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_RMSTATEDTLID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_DTLD
	Create Date:     2006-06-07 18:02:55
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_DTLD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RMSTATEDTLID=>'||p_RMSTATEDTLID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From BP_RM_STATE_DTL
	Where 
		RMSTATEDTLID = p_RMSTATEDTLID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End BP_RM_STATE_DTLD;
/
--> SPOOL OFF


-- BP_RM_STATE_DTLI.sql

--> EXECUTE DB_GENPROC.GENPROC_INSERT('BP_RM_STATE_DTL', TRUE, TRUE );
CREATE OR REPLACE PROCEDURE BP_RM_STATE_DTLI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_RMSTATEHDRID in NUMBER
	,p_MESSAGEID in VARCHAR2
	,p_SEQUENCENUMBER in NUMBER
	,p_FRBUSINESSID in VARCHAR2
	,p_TOBUSINESSID in VARCHAR2
	,p_RCVDOCTS in TIMESTAMP
	,p_FRPROTOCOLNAME in VARCHAR2
	,p_FRPROTOCOLVER in VARCHAR2
	,p_FRPROCESSCD in VARCHAR2
	,p_FRPROCESSVER in VARCHAR2
	,p_TOPROTOCOLNAME in VARCHAR2
	,p_TOPROTOCOLVER in VARCHAR2
	,p_TOPROCESSCD in VARCHAR2
	,p_TOPROCESSVER in VARCHAR2
	,p_DESTTYPE in NUMBER
	,p_TIMETOLIVE in TIMESTAMP
	,p_RETRYINTERVAL in NUMBER
	,p_RETRYCOUNT in NUMBER
	,p_MAXRETRYCOUNT in NUMBER
	,p_PERSISTDURATION in TIMESTAMP
	,p_ACKREQUESTED in NUMBER
	,p_ACKSTATUS in NUMBER
	,p_SEQSTATUS in NUMBER
	,p_SIGNEDACK in NUMBER
	,p_DOCINTOFLOW in NUMBER
	,p_VUID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_DTLI
	Create Date:     2006-06-07 18:02:55
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_DTLI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RMSTATEHDRID=>'||p_RMSTATEHDRID||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_SEQUENCENUMBER=>'||p_SEQUENCENUMBER||CHR(10)
		||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
		||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
		||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
		||', p_FRPROTOCOLNAME=>'||p_FRPROTOCOLNAME||CHR(10)
		||', p_FRPROTOCOLVER=>'||p_FRPROTOCOLVER||CHR(10)
		||', p_FRPROCESSCD=>'||p_FRPROCESSCD||CHR(10)
		||', p_FRPROCESSVER=>'||p_FRPROCESSVER||CHR(10)
		||', p_TOPROTOCOLNAME=>'||p_TOPROTOCOLNAME||CHR(10)
		||', p_TOPROTOCOLVER=>'||p_TOPROTOCOLVER||CHR(10)
		||', p_TOPROCESSCD=>'||p_TOPROCESSCD||CHR(10)
		||', p_TOPROCESSVER=>'||p_TOPROCESSVER||CHR(10)
		||', p_DESTTYPE=>'||p_DESTTYPE||CHR(10)
		||', p_TIMETOLIVE=>'||p_TIMETOLIVE||CHR(10)
		||', p_RETRYINTERVAL=>'||p_RETRYINTERVAL||CHR(10)
		||', p_RETRYCOUNT=>'||p_RETRYCOUNT||CHR(10)
		||', p_MAXRETRYCOUNT=>'||p_MAXRETRYCOUNT||CHR(10)
		||', p_PERSISTDURATION=>'||p_PERSISTDURATION||CHR(10)
		||', p_ACKREQUESTED=>'||p_ACKREQUESTED||CHR(10)
		||', p_ACKSTATUS=>'||p_ACKSTATUS||CHR(10)
		||', p_SEQSTATUS=>'||p_SEQSTATUS||CHR(10)
		||', p_SIGNEDACK=>'||p_SIGNEDACK||CHR(10)
		||', p_DOCINTOFLOW=>'||p_DOCINTOFLOW||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into BP_RM_STATE_DTL(
		RMSTATEDTLID
		,RMSTATEHDRID
		,MESSAGEID
		,SEQUENCENUMBER
		,FRBUSINESSID
		,TOBUSINESSID
		,RCVDOCTS
		,FRPROTOCOLNAME
		,FRPROTOCOLVER
		,FRPROCESSCD
		,FRPROCESSVER
		,TOPROTOCOLNAME
		,TOPROTOCOLVER
		,TOPROCESSCD
		,TOPROCESSVER
		,DESTTYPE
		,TIMETOLIVE
		,RETRYINTERVAL
		,RETRYCOUNT
		,MAXRETRYCOUNT
		,PERSISTDURATION
		,ACKREQUESTED
		,ACKSTATUS
		,SEQSTATUS
		,SIGNEDACK
		,DOCINTOFLOW
		,VUID
	) Values (
		BP_RM_STATE_DTL_SEQ.nextval
		,p_RMSTATEHDRID
		,p_MESSAGEID
		,p_SEQUENCENUMBER
		,p_FRBUSINESSID
		,p_TOBUSINESSID
		,p_RCVDOCTS
		,p_FRPROTOCOLNAME
		,p_FRPROTOCOLVER
		,p_FRPROCESSCD
		,p_FRPROCESSVER
		,p_TOPROTOCOLNAME
		,p_TOPROTOCOLVER
		,p_TOPROCESSCD
		,p_TOPROCESSVER
		,p_DESTTYPE
		,p_TIMETOLIVE
		,p_RETRYINTERVAL
		,p_RETRYCOUNT
		,p_MAXRETRYCOUNT
		,p_PERSISTDURATION
		,p_ACKREQUESTED
		,p_ACKSTATUS
		,p_SEQSTATUS
		,p_SIGNEDACK
		,p_DOCINTOFLOW
		,p_VUID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		BP_RM_STATE_DTL_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END BP_RM_STATE_DTLI;
/
--> SPOOL OFF


-- BP_RM_STATE_DTLS.sql

--> EXECUTE DB_GENPROC.GENPROC_SELECT('BP_RM_STATE_DTL', FALSE );
CREATE OR REPLACE PROCEDURE BP_RM_STATE_DTLS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_RMSTATEDTLID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_DTLS
	Create Date:     2006-06-07 18:02:56
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_DTLS';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RMSTATEDTLID=>'||p_RMSTATEDTLID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_RMSTATEDTLID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		RMSTATEDTLID
		,RMSTATEHDRID
		,RTRIM(MESSAGEID) MESSAGEID
		,SEQUENCENUMBER
		,RTRIM(FRBUSINESSID) FRBUSINESSID
		,RTRIM(TOBUSINESSID) TOBUSINESSID
		,RCVDOCTS
		,RTRIM(FRPROTOCOLNAME) FRPROTOCOLNAME
		,RTRIM(FRPROTOCOLVER) FRPROTOCOLVER
		,RTRIM(FRPROCESSCD) FRPROCESSCD
		,RTRIM(FRPROCESSVER) FRPROCESSVER
		,RTRIM(TOPROTOCOLNAME) TOPROTOCOLNAME
		,RTRIM(TOPROTOCOLVER) TOPROTOCOLVER
		,RTRIM(TOPROCESSCD) TOPROCESSCD
		,RTRIM(TOPROCESSVER) TOPROCESSVER
		,DESTTYPE
		,TIMETOLIVE
		,RETRYINTERVAL
		,RETRYCOUNT
		,MAXRETRYCOUNT
		,PERSISTDURATION
		,ACKREQUESTED
		,ACKSTATUS
		,SEQSTATUS
		,SIGNEDACK
		,DOCINTOFLOW
		,RTRIM(VUID) VUID
		,ROWTS
		,CREATEDATE
	FROM 
		BP_RM_STATE_DTL
	Where 
		RMSTATEDTLID = p_RMSTATEDTLID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		RMSTATEDTLID
		,RMSTATEHDRID
		,RTRIM(MESSAGEID) MESSAGEID
		,SEQUENCENUMBER
		,RTRIM(FRBUSINESSID) FRBUSINESSID
		,RTRIM(TOBUSINESSID) TOBUSINESSID
		,RCVDOCTS
		,RTRIM(FRPROTOCOLNAME) FRPROTOCOLNAME
		,RTRIM(FRPROTOCOLVER) FRPROTOCOLVER
		,RTRIM(FRPROCESSCD) FRPROCESSCD
		,RTRIM(FRPROCESSVER) FRPROCESSVER
		,RTRIM(TOPROTOCOLNAME) TOPROTOCOLNAME
		,RTRIM(TOPROTOCOLVER) TOPROTOCOLVER
		,RTRIM(TOPROCESSCD) TOPROCESSCD
		,RTRIM(TOPROCESSVER) TOPROCESSVER
		,DESTTYPE
		,TIMETOLIVE
		,RETRYINTERVAL
		,RETRYCOUNT
		,MAXRETRYCOUNT
		,PERSISTDURATION
		,ACKREQUESTED
		,ACKSTATUS
		,SEQSTATUS
		,SIGNEDACK
		,DOCINTOFLOW
		,RTRIM(VUID) VUID
		,ROWTS
		,CREATEDATE
	FROM 
		BP_RM_STATE_DTL
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End BP_RM_STATE_DTLS;
/
--> SPOOL OFF


-- BP_RM_STATE_DTLU.sql

--> EXECUTE DB_GENPROC.GENPROC_UPDATE('BP_RM_STATE_DTL');
CREATE OR REPLACE PROCEDURE BP_RM_STATE_DTLU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_RMSTATEDTLID in NUMBER
	,p_RMSTATEHDRID in NUMBER
	,p_MESSAGEID in VARCHAR2
	,p_SEQUENCENUMBER in NUMBER
	,p_FRBUSINESSID in VARCHAR2
	,p_TOBUSINESSID in VARCHAR2
	,p_RCVDOCTS in TIMESTAMP
	,p_FRPROTOCOLNAME in VARCHAR2
	,p_FRPROTOCOLVER in VARCHAR2
	,p_FRPROCESSCD in VARCHAR2
	,p_FRPROCESSVER in VARCHAR2
	,p_TOPROTOCOLNAME in VARCHAR2
	,p_TOPROTOCOLVER in VARCHAR2
	,p_TOPROCESSCD in VARCHAR2
	,p_TOPROCESSVER in VARCHAR2
	,p_DESTTYPE in NUMBER
	,p_TIMETOLIVE in TIMESTAMP
	,p_RETRYINTERVAL in NUMBER
	,p_RETRYCOUNT in NUMBER
	,p_MAXRETRYCOUNT in NUMBER
	,p_PERSISTDURATION in TIMESTAMP
	,p_ACKREQUESTED in NUMBER
	,p_ACKSTATUS in NUMBER
	,p_SEQSTATUS in NUMBER
	,p_SIGNEDACK in NUMBER
	,p_DOCINTOFLOW in NUMBER
	,p_VUID in VARCHAR2
	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_DTLU
	Create Date:     2006-06-07 18:02:56
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_DTLU';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RMSTATEDTLID=>'||p_RMSTATEDTLID||CHR(10)
		||', p_RMSTATEHDRID=>'||p_RMSTATEHDRID||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_SEQUENCENUMBER=>'||p_SEQUENCENUMBER||CHR(10)
		||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
		||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
		||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
		||', p_FRPROTOCOLNAME=>'||p_FRPROTOCOLNAME||CHR(10)
		||', p_FRPROTOCOLVER=>'||p_FRPROTOCOLVER||CHR(10)
		||', p_FRPROCESSCD=>'||p_FRPROCESSCD||CHR(10)
		||', p_FRPROCESSVER=>'||p_FRPROCESSVER||CHR(10)
		||', p_TOPROTOCOLNAME=>'||p_TOPROTOCOLNAME||CHR(10)
		||', p_TOPROTOCOLVER=>'||p_TOPROTOCOLVER||CHR(10)
		||', p_TOPROCESSCD=>'||p_TOPROCESSCD||CHR(10)
		||', p_TOPROCESSVER=>'||p_TOPROCESSVER||CHR(10)
		||', p_DESTTYPE=>'||p_DESTTYPE||CHR(10)
		||', p_TIMETOLIVE=>'||p_TIMETOLIVE||CHR(10)
		||', p_RETRYINTERVAL=>'||p_RETRYINTERVAL||CHR(10)
		||', p_RETRYCOUNT=>'||p_RETRYCOUNT||CHR(10)
		||', p_MAXRETRYCOUNT=>'||p_MAXRETRYCOUNT||CHR(10)
		||', p_PERSISTDURATION=>'||p_PERSISTDURATION||CHR(10)
		||', p_ACKREQUESTED=>'||p_ACKREQUESTED||CHR(10)
		||', p_ACKSTATUS=>'||p_ACKSTATUS||CHR(10)
		||', p_SEQSTATUS=>'||p_SEQSTATUS||CHR(10)
		||', p_SIGNEDACK=>'||p_SIGNEDACK||CHR(10)
		||', p_DOCINTOFLOW=>'||p_DOCINTOFLOW||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update BP_RM_STATE_DTL
	SET
		RMSTATEHDRID=p_RMSTATEHDRID
		,      MESSAGEID=p_MESSAGEID
		,      SEQUENCENUMBER=p_SEQUENCENUMBER
		,      FRBUSINESSID=p_FRBUSINESSID
		,      TOBUSINESSID=p_TOBUSINESSID
		,      RCVDOCTS=p_RCVDOCTS
		,      FRPROTOCOLNAME=p_FRPROTOCOLNAME
		,      FRPROTOCOLVER=p_FRPROTOCOLVER
		,      FRPROCESSCD=p_FRPROCESSCD
		,      FRPROCESSVER=p_FRPROCESSVER
		,      TOPROTOCOLNAME=p_TOPROTOCOLNAME
		,      TOPROTOCOLVER=p_TOPROTOCOLVER
		,      TOPROCESSCD=p_TOPROCESSCD
		,      TOPROCESSVER=p_TOPROCESSVER
		,      DESTTYPE=p_DESTTYPE
		,      TIMETOLIVE=p_TIMETOLIVE
		,      RETRYINTERVAL=p_RETRYINTERVAL
		,      RETRYCOUNT=p_RETRYCOUNT
		,      MAXRETRYCOUNT=p_MAXRETRYCOUNT
		,      PERSISTDURATION=p_PERSISTDURATION
		,      ACKREQUESTED=p_ACKREQUESTED
		,      ACKSTATUS=p_ACKSTATUS
		,      SEQSTATUS=p_SEQSTATUS
		,      SIGNEDACK=p_SIGNEDACK
		,      DOCINTOFLOW=p_DOCINTOFLOW
		,      VUID=p_VUID
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		RMSTATEDTLID = p_RMSTATEDTLID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End BP_RM_STATE_DTLU;
/
--> SPOOL OFF


-- BP_RM_STATE_HDRD.sql

--> EXECUTE DB_GENPROC.GENPROC_DELETE('BP_RM_STATE_HDR');
CREATE OR REPLACE PROCEDURE BP_RM_STATE_HDRD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_RMSTATEHDRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_HDRD
	Create Date:     2006-06-07 18:03:04
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_HDRD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RMSTATEHDRID=>'||p_RMSTATEHDRID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From BP_RM_STATE_HDR
	Where 
		RMSTATEHDRID = p_RMSTATEHDRID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End BP_RM_STATE_HDRD;
/
--> SPOOL OFF


-- BP_RM_STATE_HDRI.sql

--> EXECUTE DB_GENPROC.GENPROC_INSERT('BP_RM_STATE_HDR', TRUE, TRUE );
CREATE OR REPLACE PROCEDURE BP_RM_STATE_HDRI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CONVERSATIONID in VARCHAR2
	,p_INITBUSINESSID in VARCHAR2
	,p_WORKGROUPNAME in VARCHAR2
	,p_STATEINSTANCENAME in VARCHAR2
	,p_CPAID in VARCHAR2
	,p_PROTOCOLNAME in VARCHAR2
	,p_PROTOCOLVER in VARCHAR2
	,p_PROCESSCD in VARCHAR2
	,p_PROCESSVER in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_HDRI
	Create Date:     2006-06-07 18:03:04
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_HDRI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_WORKGROUPNAME=>'||p_WORKGROUPNAME||CHR(10)
		||', p_STATEINSTANCENAME=>'||p_STATEINSTANCENAME||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME||CHR(10)
		||', p_PROTOCOLVER=>'||p_PROTOCOLVER||CHR(10)
		||', p_PROCESSCD=>'||p_PROCESSCD||CHR(10)
		||', p_PROCESSVER=>'||p_PROCESSVER||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into BP_RM_STATE_HDR(
		RMSTATEHDRID
		,CONVERSATIONID
		,INITBUSINESSID
		,WORKGROUPNAME
		,STATEINSTANCENAME
		,CPAID
		,PROTOCOLNAME
		,PROTOCOLVER
		,PROCESSCD
		,PROCESSVER
	) Values (
		BP_RM_STATE_HDR_SEQ.nextval
		,p_CONVERSATIONID
		,p_INITBUSINESSID
		,p_WORKGROUPNAME
		,p_STATEINSTANCENAME
		,p_CPAID
		,p_PROTOCOLNAME
		,p_PROTOCOLVER
		,p_PROCESSCD
		,p_PROCESSVER
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		BP_RM_STATE_HDR_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END BP_RM_STATE_HDRI;
/
--> SPOOL OFF


-- BP_RM_STATE_HDRI_AH1.sql

CREATE OR REPLACE PROCEDURE BP_RM_STATE_HDRI_AH1
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CONVERSATIONID in VARCHAR2
	,p_INITBUSINESSID in VARCHAR2
	,p_FRBUSINESSID in VARCHAR2
	,p_TOBUSINESSID in VARCHAR2
	,p_CPAID in VARCHAR2
	,p_FRPROTOCOLNAME in VARCHAR2
	,p_FRPROTOCOLVER in VARCHAR2
	,p_FRPROCESSCD in VARCHAR2
	,p_FRPROCESSVER in VARCHAR2
	,p_TOPROTOCOLNAME in VARCHAR2
	,p_TOPROTOCOLVER in VARCHAR2
	,p_TOPROCESSCD in VARCHAR2
	,p_TOPROCESSVER in VARCHAR2
	,p_DESTNAME in VARCHAR2
	,p_MESSAGEID in VARCHAR2
	,p_SEQUENCENUMBER in NUMBER
	,p_TIMETOLIVE in TIMESTAMP
	,p_PERSISTDURATION in TIMESTAMP
	,p_RETRYCOUNT in NUMBER
	,p_MAXRETRYCOUNT in NUMBER
	,p_RETRYINTERVAL in NUMBER
	,p_ACKREQUESTED in NUMBER
	,p_ACKSTATUS in NUMBER
	,p_SEQSTATUS in NUMBER
	,p_SIGNEDACK in NUMBER
	,p_RCVDOCTS in TIMESTAMP
	,p_VUID in VARCHAR2
	,p_WORKGROUPNAME in VARCHAR2
	,p_ACTIVITYID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_HDRI_AH1
	Create Date:     2006-03-29 16:28:38
	Author:          SYSTEM 

Inserts data into state tables for Reliable messaging, performs sequence check.
Out-Of-Sequence messages (received earlier than this one) are updated to be introduced 
into the flow as a result of processing the current message.
	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-29 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_HDRI_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
		||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_FRPROTOCOLNAME=>'||p_FRPROTOCOLNAME||CHR(10)
		||', p_FRPROTOCOLVER=>'||p_FRPROTOCOLVER||CHR(10)
		||', p_FRPROCESSCD=>'||p_FRPROCESSCD||CHR(10)
		||', p_FRPROCESSVER=>'||p_FRPROCESSVER||CHR(10)
		||', p_TOPROTOCOLNAME=>'||p_TOPROTOCOLNAME||CHR(10)
		||', p_TOPROTOCOLVER=>'||p_TOPROTOCOLVER||CHR(10)
		||', p_TOPROCESSCD=>'||p_TOPROCESSCD||CHR(10)
		||', p_TOPROCESSVER=>'||p_TOPROCESSVER||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_SEQUENCENUMBER=>'||p_SEQUENCENUMBER||CHR(10)
		||', p_TIMETOLIVE=>'||p_TIMETOLIVE||CHR(10)
		||', p_PERSISTDURATION=>'||p_PERSISTDURATION||CHR(10)
		||', p_RETRYCOUNT=>'||p_RETRYCOUNT||CHR(10)
		||', p_MAXRETRYCOUNT=>'||p_MAXRETRYCOUNT||CHR(10)
		||', p_RETRYINTERVAL=>'||p_RETRYINTERVAL||CHR(10)
		||', p_ACKREQUESTED=>'||p_ACKREQUESTED||CHR(10)
		||', p_ACKSTATUS=>'||p_ACKSTATUS||CHR(10)
		||', p_SEQSTATUS=>'||p_SEQSTATUS||CHR(10)
		||', p_SIGNEDACK=>'||p_SIGNEDACK||CHR(10)
		||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_WORKGROUPNAME=>'||p_WORKGROUPNAME||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		;

v_RMStateHdrId		bp_rm_state_hdr.rmstatehdrid%TYPE;
v_SequenceNumber	bp_rm_state_dtl.SequenceNumber%TYPE;
v_RetryMinutes		bp_rm_state_dtl.RetryInterval%TYPE := COALESCE(p_RETRYINTERVAL, 5);
v_SeqFlg		bp_rm_state_dtl.SeqStatus%TYPE;
v_FirstMessage		PLS_INTEGER;
v_DestTypeID		PLS_INTEGER	DEFAULT FN_GET_DESTTYPEID (p_DestName);
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	BEGIN
		SELECT	hdr.RMSTATEHDRID
		INTO	v_RMstatehdrid
		FROM	bp_rm_state_hdr hdr
		WHERE	hdr.ConversationId = p_ConversationId
		AND	hdr.CPAID = p_CPAID
		;
		v_FirstMessage := 0;  --- Found a Conversation ->This message is NOT the first message in the conversation
		BEGIN
			--- Check for previous message which is in sequence
			SELECT	MAX(dtl.SequenceNumber)
			INTO	v_SequenceNumber
			FROM	bp_rm_state_dtl dtl
			WHERE	dtl.RMStateHdrId = v_RMStateHdrId
			AND	dtl.SequenceNumber < p_SequenceNumber
			AND	dtl.SeqStatus = 1
			;

		EXCEPTION
			WHEN No_Data_Found THEN
				v_SequenceNumber := -1;
		END;
	EXCEPTION
		WHEN No_Data_Found THEN
			v_FirstMessage := 1;
			Insert into BP_RM_STATE_HDR(
				RMSTATEHDRID
				,CONVERSATIONID
				,INITBUSINESSID
				,CPAID
				,WORKGROUPNAME
			) Values (
				BP_RM_STATE_HDR_SEQ.nextval
				,p_CONVERSATIONID
				,p_INITBUSINESSID
				,p_CPAID
				,p_WORKGROUPNAME
			);
			SELECT BP_RM_STATE_HDR_SEQ.currval INTO v_RMstateHdrId FROM DUAL;
		WHEN Others THEN
			v_ProcName := v_ProcName || '-hdr';
			RAISE;
	END;
	BEGIN
		v_SeqFlg:= CASE 
				WHEN p_SequenceNumber = 0	--- This message has Seq=0 
					THEN 1			--- InSequence
				WHEN v_FirstMessage = 0		--- Ongoing Conversation
				AND	v_SequenceNumber + 1 = p_SequenceNumber --- New Message is next in sequence 
					THEN 1  --- InSequence
				ELSE 0  --- Message is Out of Sequence
			END;
		Insert into BP_RM_STATE_DTL(
			RMSTATEDTLID
			,RMSTATEHDRID
			,MESSAGEID
			,SEQUENCENUMBER
			,FRBUSINESSID
			,TOBUSINESSID
			,RCVDOCTS
			,FRPROTOCOLNAME
			,FRPROTOCOLVER
			,FRPROCESSCD
			,FRPROCESSVER
			,TOPROTOCOLNAME
			,TOPROTOCOLVER
			,TOPROCESSCD
			,TOPROCESSVER
			,DESTTYPE
			,TIMETOLIVE
			,RETRYINTERVAL
			,RETRYCOUNT
			,MAXRETRYCOUNT
			,PERSISTDURATION
			,ACKREQUESTED
			,ACKSTATUS
			,SEQSTATUS
			,SIGNEDACK
			,DOCINTOFLOW
			,VUID
			,ACTIVITYID
			,NEXTRETRYTS
		) Values (
			BP_RM_STATE_DTL_SEQ.nextval
			,v_RMSTATEHDRID
			,p_MESSAGEID
			,p_SEQUENCENUMBER
			,p_FRBUSINESSID
			,p_TOBUSINESSID
			,p_RCVDOCTS
			,p_FRPROTOCOLNAME
			,p_FRPROTOCOLVER
			,p_FRPROCESSCD
			,p_FRPROCESSVER
			,p_TOPROTOCOLNAME
			,p_TOPROTOCOLVER
			,p_TOPROCESSCD
			,p_TOPROCESSVER
			,v_DestTypeID
			,p_TIMETOLIVE
			,v_RetryMinutes
			,p_RETRYCOUNT
			,p_MAXRETRYCOUNT
			,p_PERSISTDURATION
			,p_ACKREQUESTED
			,p_ACKSTATUS
			,v_SeqFlg
			,p_SIGNEDACK
			,0  --- No
			,p_VUID
			,p_ACTIVITYID
			,v_BeginTS + NUMTODSINTERVAL( v_RetryMinutes, 'MINUTE')
		);
		--- See if this message completes the sequence for subsequent messages
		IF v_SeqFlg = 1 THEN
			UPDATE	bp_rm_state_dtl
			SET	DocIntoFlow = 1
				,UpdateDate= v_BeginTS
			WHERE	RMStateHdrId = v_RMStateHdrID
			AND	SequenceNumber = p_SequenceNumber + 1
			AND	SeqStatus = 0
			;
		END IF;
	EXCEPTION
		WHEN Dup_Val_On_Index THEN
			v_ProcName := v_ProcName || '-dtl';
			RAISE;
	END;
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		BP_RM_STATE_DTL_SEQ.currval ID, 1 RowTS, v_SeqFlg SeqStatus From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN No_Data_Found THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	WHEN Dup_Val_On_Index THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	WHEN Others THEN
		Rollback work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		RAISE;
END BP_RM_STATE_HDRI_AH1;
/


-- BP_RM_STATE_HDRI_AH2.sql

CREATE OR REPLACE PROCEDURE BP_RM_STATE_HDRI_AH2
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CONVERSATIONID in VARCHAR2
	,p_INITBUSINESSID in VARCHAR2
	,p_FRBUSINESSID in VARCHAR2
	,p_TOBUSINESSID in VARCHAR2
	,p_CPAID in VARCHAR2
	,p_FRPROTOCOLNAME in VARCHAR2
	,p_FRPROTOCOLVER in VARCHAR2
	,p_FRPROCESSCD in VARCHAR2
	,p_FRPROCESSVER in VARCHAR2
	,p_TOPROTOCOLNAME in VARCHAR2
	,p_TOPROTOCOLVER in VARCHAR2
	,p_TOPROCESSCD in VARCHAR2
	,p_TOPROCESSVER in VARCHAR2
	,p_DESTNAME in VARCHAR2
	,p_MESSAGEID in VARCHAR2
	,p_SEQUENCENUMBER in NUMBER
	,p_TIMETOLIVE in TIMESTAMP
	,p_PERSISTDURATION in TIMESTAMP
	,p_RETRYCOUNT in NUMBER
	,p_MAXRETRYCOUNT in NUMBER
	,p_RETRYINTERVAL in INTEGER
	,p_ACKREQUESTED in NUMBER
	,p_ACKSTATUS in NUMBER
	,p_SEQSTATUS in NUMBER
	,p_SIGNEDACK in NUMBER
	,p_RCVDOCTS in TIMESTAMP
	,p_VUID in VARCHAR2
	,p_WORKGROUPNAME in VARCHAR2
	,p_ACTIVITYID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_HDRI_AH2
	Create Date:     2006-03-29 16:28:38
	Author:          SYSTEM 

Inserts data into state tables for Reliable messaging for given values.
Checks whether Conversation (hdr) exists. Unless values are specified :
	*AckStatus defaults to 0 (Ack not recd)
	*SeqStatus defaults to 1 (Mesg is in sequence)

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-29 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_HDRI_AH2';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
		||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_FRPROTOCOLNAME=>'||p_FRPROTOCOLNAME||CHR(10)
		||', p_FRPROTOCOLVER=>'||p_FRPROTOCOLVER||CHR(10)
		||', p_FRPROCESSCD=>'||p_FRPROCESSCD||CHR(10)
		||', p_FRPROCESSVER=>'||p_FRPROCESSVER||CHR(10)
		||', p_TOPROTOCOLNAME=>'||p_TOPROTOCOLNAME||CHR(10)
		||', p_TOPROTOCOLVER=>'||p_TOPROTOCOLVER||CHR(10)
		||', p_TOPROCESSCD=>'||p_TOPROCESSCD||CHR(10)
		||', p_TOPROCESSVER=>'||p_TOPROCESSVER||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_SEQUENCENUMBER=>'||p_SEQUENCENUMBER||CHR(10)
		||', p_TIMETOLIVE=>'||p_TIMETOLIVE||CHR(10)
		||', p_PERSISTDURATION=>'||p_PERSISTDURATION||CHR(10)
		||', p_RETRYCOUNT=>'||p_RETRYCOUNT||CHR(10)
		||', p_MAXRETRYCOUNT=>'||p_MAXRETRYCOUNT||CHR(10)
		||', p_RETRYINTERVAL=>'||p_RETRYINTERVAL||CHR(10)
		||', p_ACKREQUESTED=>'||p_ACKREQUESTED||CHR(10)
		||', p_ACKSTATUS=>'||p_ACKSTATUS||CHR(10)
		||', p_SEQSTATUS=>'||p_SEQSTATUS||CHR(10)
		||', p_SIGNEDACK=>'||p_SIGNEDACK||CHR(10)
		||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_WORKGROUPNAME=>'||p_WORKGROUPNAME||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		;
v_RMStateHdrId		bp_rm_state_hdr.rmstatehdrid%TYPE;
v_FirstMessage		PLS_INTEGER;
v_DestTypeID		PLS_INTEGER	DEFAULT FN_GET_DESTTYPEID (p_DestName);
v_RetryMinutes		bp_rm_state_dtl.RetryInterval%TYPE := COALESCE(p_RETRYINTERVAL, 5);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	BEGIN
		SELECT	hdr.RMSTATEHDRID
		INTO	v_RMstatehdrid
		FROM	bp_rm_state_hdr hdr
		WHERE	hdr.ConversationId = p_ConversationId
		AND	hdr.CPAID = p_CPAID
		;
	EXCEPTION
		WHEN No_Data_Found THEN
			v_FirstMessage := 1;
			Insert into BP_RM_STATE_HDR(
				RMSTATEHDRID
				,CONVERSATIONID
				,INITBUSINESSID
				,CPAID
				,WORKGROUPNAME
			) Values (
				BP_RM_STATE_HDR_SEQ.nextval
				,p_CONVERSATIONID
				,p_INITBUSINESSID
				,p_CPAID
				,p_WORKGROUPNAME
			);
			SELECT BP_RM_STATE_HDR_SEQ.currval INTO v_RMstateHdrId FROM DUAL;
		WHEN Others THEN
			v_ProcName := v_ProcName || '-hdr';
			RAISE;
	END;
	BEGIN
		Insert into BP_RM_STATE_DTL(
			RMSTATEDTLID
			,RMSTATEHDRID
			,MESSAGEID
			,SEQUENCENUMBER
			,FRBUSINESSID
			,TOBUSINESSID
			,RCVDOCTS
			,FRPROTOCOLNAME
			,FRPROTOCOLVER
			,FRPROCESSCD
			,FRPROCESSVER
			,TOPROTOCOLNAME
			,TOPROTOCOLVER
			,TOPROCESSCD
			,TOPROCESSVER
			,DestType
			,TIMETOLIVE
			,RETRYINTERVAL
			,RETRYCOUNT
			,MAXRETRYCOUNT
			,PERSISTDURATION
			,ACKREQUESTED
			,ACKSTATUS
			,SEQSTATUS
			,SIGNEDACK
			,DOCINTOFLOW
			,VUID
			,ACTIVITYID
			,NEXTRETRYTS
		) Values (
			BP_RM_STATE_DTL_SEQ.nextval
			,v_RMSTATEHDRID
			,p_MESSAGEID
			,p_SEQUENCENUMBER
			,p_FRBUSINESSID
			,p_TOBUSINESSID
			,p_RCVDOCTS
			,p_FRPROTOCOLNAME
			,p_FRPROTOCOLVER
			,p_FRPROCESSCD
			,p_FRPROCESSVER
			,p_TOPROTOCOLNAME
			,p_TOPROTOCOLVER
			,p_TOPROCESSCD
			,p_TOPROCESSVER
			,v_DestTypeID
			,p_TIMETOLIVE
			,v_RetryMinutes
			,p_RETRYCOUNT
			,p_MAXRETRYCOUNT
			,p_PERSISTDURATION
			,p_ACKREQUESTED
			,COALESCE( p_ACKSTATUS, 0)
			,COALESCE( p_SEQSTATUS, 1)
			,p_SIGNEDACK
			,0
			,p_VUID
			,p_ACTIVITYID
			,v_BeginTS + NUMTODSINTERVAL(v_RetryMinutes, 'MINUTE')
		);
	EXCEPTION
		WHEN Dup_Val_On_Index THEN
			v_ProcName := v_ProcName || '-dtl';
			RAISE;
	END;
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		BP_RM_STATE_DTL_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN No_Data_Found THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	WHEN Dup_Val_On_Index THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	WHEN Others THEN
		Rollback work;
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		RAISE;
END BP_RM_STATE_HDRI_AH2;
/


-- BP_RM_STATE_HDRS.sql

--> EXECUTE DB_GENPROC.GENPROC_SELECT('BP_RM_STATE_HDR', FALSE );
CREATE OR REPLACE PROCEDURE BP_RM_STATE_HDRS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_RMSTATEHDRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_HDRS
	Create Date:     2006-06-07 18:03:04
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_HDRS';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RMSTATEHDRID=>'||p_RMSTATEHDRID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_RMSTATEHDRID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		RMSTATEHDRID
		,RTRIM(CONVERSATIONID) CONVERSATIONID
		,RTRIM(INITBUSINESSID) INITBUSINESSID
		,RTRIM(WORKGROUPNAME) WORKGROUPNAME
		,RTRIM(STATEINSTANCENAME) STATEINSTANCENAME
		,RTRIM(CPAID) CPAID
		,RTRIM(PROTOCOLNAME) PROTOCOLNAME
		,RTRIM(PROTOCOLVER) PROTOCOLVER
		,RTRIM(PROCESSCD) PROCESSCD
		,RTRIM(PROCESSVER) PROCESSVER
		,ROWTS
		,CREATEDATE
	FROM 
		BP_RM_STATE_HDR
	Where 
		RMSTATEHDRID = p_RMSTATEHDRID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		RMSTATEHDRID
		,RTRIM(CONVERSATIONID) CONVERSATIONID
		,RTRIM(INITBUSINESSID) INITBUSINESSID
		,RTRIM(WORKGROUPNAME) WORKGROUPNAME
		,RTRIM(STATEINSTANCENAME) STATEINSTANCENAME
		,RTRIM(CPAID) CPAID
		,RTRIM(PROTOCOLNAME) PROTOCOLNAME
		,RTRIM(PROTOCOLVER) PROTOCOLVER
		,RTRIM(PROCESSCD) PROCESSCD
		,RTRIM(PROCESSVER) PROCESSVER
		,ROWTS
		,CREATEDATE
	FROM 
		BP_RM_STATE_HDR
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End BP_RM_STATE_HDRS;
/
--> SPOOL OFF


-- BP_RM_STATE_HDRS_AH1.sql

CREATE OR REPLACE PROCEDURE BP_RM_STATE_HDRS_AH1
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_RowCnt		in pls_integer
	,p_WorkGroupName	in  VARCHAR2
	,p_StateInstanceName	in  VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_HDRS_AH1
	Create Date:     2006-03-29 16:28:40
	Author:          SYSTEM 

Invoked from server to get work for Reliable messaging engine
	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-29 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_HDRS_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RowCnt=>'||p_RowCnt||CHR(10)
		||', p_WorkGroupName=>'||p_WorkGroupName||CHR(10)
		||', p_StateInstanceName =>'||p_StateInstanceName||CHR(10)
		;

v_ReturnCD	integer;
v_RowTS		integer :=0;
v_RowID		bp_rm_state_que.row_id%TYPE;
v_UpdateDate	bp_rm_state_que.UpdateDate%TYPE;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	BEGIN
		DELETE FROM bp_rm_State_Que 
		WHERE ROWNUM < 2 
		RETURNING row_id, updateDate INTO v_RowId, v_updateDate;
		v_RowCnt := SQL%RowCount;
		COMMIT;
	EXCEPTION
		WHEN Others THEN
		        v_RowCnt := 0;
	END;
	IF v_RowCnt = 0 THEN
		v_RowId := Null;
		BP_RM_STATE_QUEI_AH1
			(
			P_RETURNCD            => v_ReturnCD
			, p_DebugFlg          => p_DebugFlg
			, P_ROWCNT            => v_RowCnt
			, P_WORKGROUPNAME     => p_WorkGroupName
			, P_STATEINSTANCENAME => p_StateInstanceName
			);
		
	ELSE
		UPDATE	bp_rm_state_dtl rmd
		SET	NextRetryTS = v_BeginTS + NUMTODSINTERVAL(rmd.RetryInterval, 'MINUTE')
		WHERE	rmd.rowid = v_RowID;
	END IF;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 FOR
	SELECT	rmh.ConversationId
		,rmd.MessageID
		,rmd.VUID
		,rmd.ACTIVITYID
		,rmd.FrBusinessID
		,rmd.ToBusinessID
		,rmd.RCVDOCTS
		,rmd.TimeToLive
		,rmd.AckStatus
		,rmd.RetryCount
		,rmd.MaxRetryCount
		,Dest.DestName  /* PMR 12053,370,000 */
		,rmd.RowTS
		,v_BeginTS CurrentTS
	FROM	bp_rm_state_dtl rmd
	INNER JOIN bp_rm_State_Hdr rmh ON rmh.RMStateHdrId = rmd.RMStateHdrId
	Left Outer Join pr_DestType	 Dest  on Dest.DestTypeID   = rmd.DESTTYPE   /* PMR 12053,370,000 */
	WHERE	rmd.rowId = v_RowId
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN TOO_MANY_ROWS THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
	WHEN ZeroUpdate THEN
		v_ExceptionFlg := 1;
		p_ReturnCd := 3;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
	WHEN Others THEN
		ROLLBACK work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
End BP_RM_STATE_HDRS_AH1;
/


-- BP_RM_STATE_HDRU.sql

--> EXECUTE DB_GENPROC.GENPROC_UPDATE('BP_RM_STATE_HDR');
CREATE OR REPLACE PROCEDURE BP_RM_STATE_HDRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_RMSTATEHDRID in NUMBER
	,p_CONVERSATIONID in VARCHAR2
	,p_INITBUSINESSID in VARCHAR2
	,p_WORKGROUPNAME in VARCHAR2
	,p_STATEINSTANCENAME in VARCHAR2
	,p_CPAID in VARCHAR2
	,p_PROTOCOLNAME in VARCHAR2
	,p_PROTOCOLVER in VARCHAR2
	,p_PROCESSCD in VARCHAR2
	,p_PROCESSVER in VARCHAR2
	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_HDRU
	Create Date:     2006-06-07 18:03:05
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_HDRU';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RMSTATEHDRID=>'||p_RMSTATEHDRID||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_WORKGROUPNAME=>'||p_WORKGROUPNAME||CHR(10)
		||', p_STATEINSTANCENAME=>'||p_STATEINSTANCENAME||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME||CHR(10)
		||', p_PROTOCOLVER=>'||p_PROTOCOLVER||CHR(10)
		||', p_PROCESSCD=>'||p_PROCESSCD||CHR(10)
		||', p_PROCESSVER=>'||p_PROCESSVER||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update BP_RM_STATE_HDR
	SET
		CONVERSATIONID=p_CONVERSATIONID
		,      INITBUSINESSID=p_INITBUSINESSID
		,      WORKGROUPNAME=p_WORKGROUPNAME
		,      STATEINSTANCENAME=p_STATEINSTANCENAME
		,      CPAID=p_CPAID
		,      PROTOCOLNAME=p_PROTOCOLNAME
		,      PROTOCOLVER=p_PROTOCOLVER
		,      PROCESSCD=p_PROCESSCD
		,      PROCESSVER=p_PROCESSVER
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		RMSTATEHDRID = p_RMSTATEHDRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End BP_RM_STATE_HDRU;
/
--> SPOOL OFF


-- BP_RM_STATE_HDRU_AH1.sql

CREATE OR REPLACE PROCEDURE BP_RM_STATE_HDRU_AH1
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_CONVERSATIONID in VARCHAR2
	,p_INITBUSINESSID in VARCHAR2
	,p_FRBUSINESSID in VARCHAR2
	,p_TOBUSINESSID in VARCHAR2
	,p_CPAID in VARCHAR2
	,p_FRPROTOCOLNAME in VARCHAR2
	,p_FRPROTOCOLVER in VARCHAR2
	,p_FRPROCESSCD in VARCHAR2
	,p_FRPROCESSVER in VARCHAR2
	,p_TOPROTOCOLNAME in VARCHAR2
	,p_TOPROTOCOLVER in VARCHAR2
	,p_TOPROCESSCD in VARCHAR2
	,p_TOPROCESSVER in VARCHAR2
	,p_DESTNAME in VARCHAR2
	,p_MESSAGEID in VARCHAR2
	,p_SEQUENCENUMBER in NUMBER
	,p_TIMETOLIVE in TIMESTAMP
	,p_PERSISTDURATION in TIMESTAMP
	,p_RETRYCOUNT in NUMBER
	,p_MAXRETRYCOUNT in NUMBER
	,p_RETRYINTERVAL in NUMBER
	,p_ACKREQUESTED in NUMBER
	,p_ACKSTATUS in NUMBER
	,p_SEQSTATUS in NUMBER
	,p_SIGNEDACK in NUMBER
	,p_RCVDOCTS in TIMESTAMP
	,p_VUID in VARCHAR2
	,p_WORKGROUPNAME in VARCHAR2
---	,in p_ROWTS INTEGER
	,p_ACTIVITYID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_RM_STATE_HDRU_AH1
	Create Date:     2006-03-29 16:28:40
	Author:          SYSTEM 

Update the state of the message in Reliable messaging table
	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-29 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_Ackstatus1          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'BP_RM_STATE_HDRU_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
		||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_FRPROTOCOLNAME=>'||p_FRPROTOCOLNAME||CHR(10)
		||', p_FRPROTOCOLVER=>'||p_FRPROTOCOLVER||CHR(10)
		||', p_FRPROCESSCD=>'||p_FRPROCESSCD||CHR(10)
		||', p_FRPROCESSVER=>'||p_FRPROCESSVER||CHR(10)
		||', p_TOPROTOCOLNAME=>'||p_TOPROTOCOLNAME||CHR(10)
		||', p_TOPROTOCOLVER=>'||p_TOPROTOCOLVER||CHR(10)
		||', p_TOPROCESSCD=>'||p_TOPROCESSCD||CHR(10)
		||', p_TOPROCESSVER=>'||p_TOPROCESSVER||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_SEQUENCENUMBER=>'||p_SEQUENCENUMBER||CHR(10)
		||', p_TIMETOLIVE=>'||p_TIMETOLIVE||CHR(10)
		||', p_PERSISTDURATION=>'||p_PERSISTDURATION||CHR(10)
		||', p_RETRYCOUNT=>'||p_RETRYCOUNT||CHR(10)
		||', p_MAXRETRYCOUNT=>'||p_MAXRETRYCOUNT||CHR(10)
		||', p_RETRYINTERVAL=>'||p_RETRYINTERVAL||CHR(10)
		||', p_ACKREQUESTED=>'||p_ACKREQUESTED||CHR(10)
		||', p_ACKSTATUS=>'||p_ACKSTATUS||CHR(10)
		||', p_SEQSTATUS=>'||p_SEQSTATUS||CHR(10)
		||', p_SIGNEDACK=>'||p_SIGNEDACK||CHR(10)
		||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		;
V_ACKSTATUS integer;
v_RowTS      integer :=0;
v_SequenceNumber	bp_rm_state_dtl.SequenceNumber%TYPE := 0;
v_RMStateHdrID		bp_rm_state_dtl.RMStateHdrId%TYPE := 0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_Ackstatus1 := FN_GET_ACKSTATUS(p_MESSAGEID);
    
    v_ackstatus := p_ACKSTATUS;

	If v_Ackstatus1 != 3 AND v_ACKSTATUS = 0 then
	   v_ACKSTATUS := NULL;
	end if;

	v_RowCnt := 0;
	Update BP_RM_STATE_DTL dtl
	SET
		      SEQUENCENUMBER = COALESCE ( p_SEQUENCENUMBER, dtl.SEQUENCENUMBER)
		,      FRBUSINESSID = COALESCE ( p_FRBUSINESSID, dtl.FRBUSINESSID)
		,      TOBUSINESSID = COALESCE ( p_TOBUSINESSID, dtl.TOBUSINESSID)
		,      RCVDOCTS = COALESCE ( p_RCVDOCTS, dtl.RCVDOCTS)
		,      FRPROTOCOLNAME = COALESCE ( p_FRPROTOCOLNAME, dtl.FRPROTOCOLNAME)
		,      FRPROTOCOLVER = COALESCE ( p_FRPROTOCOLVER, dtl.FRPROTOCOLVER)
		,      FRPROCESSCD = COALESCE ( p_FRPROCESSCD, dtl.FRPROCESSCD)
		,      FRPROCESSVER = COALESCE ( p_FRPROCESSVER, dtl.FRPROCESSVER)
		,      TOPROTOCOLNAME = COALESCE ( p_TOPROTOCOLNAME, dtl.TOPROTOCOLNAME)
		,      TOPROTOCOLVER = COALESCE ( p_TOPROTOCOLVER, dtl.TOPROTOCOLVER)
		,      TOPROCESSCD = COALESCE ( p_TOPROCESSCD, dtl.TOPROCESSCD)
		,      TOPROCESSVER = COALESCE ( p_TOPROCESSVER, dtl.TOPROCESSVER)
		,      TIMETOLIVE = COALESCE ( p_TIMETOLIVE, dtl.TIMETOLIVE)
		,      RETRYINTERVAL = COALESCE ( p_RETRYINTERVAL, dtl.RETRYINTERVAL)
		,      RETRYCOUNT = COALESCE(dtl.RETRYCOUNT,0) + COALESCE(p_RetryCount,0)
		,      MAXRETRYCOUNT = COALESCE ( p_MAXRETRYCOUNT, dtl.MAXRETRYCOUNT)
		,      PERSISTDURATION = COALESCE ( p_PERSISTDURATION, dtl.PERSISTDURATION)
		,      ACKREQUESTED = COALESCE ( p_ACKREQUESTED, dtl.ACKREQUESTED)
		,      ACKSTATUS = COALESCE ( v_ACKSTATUS, dtl.ACKSTATUS)
		,      SEQSTATUS = COALESCE ( p_SEQSTATUS, dtl.SEQSTATUS)
		,      SIGNEDACK = COALESCE ( p_SIGNEDACK, dtl.SIGNEDACK)
		,      VUID = COALESCE( p_VUID, dtl.VUID)
		,      ACTIVITYID = COALESCE( p_ACTIVITYID, dtl.ACTIVITYID)
		,      DOCINTOFLOW=COALESCE(CASE WHEN p_SEQSTATUS=1 THEN 0 ELSE NULL END, dtl.DOCINTOFLOW)
		,      ROWTS= RowTS + 1
		,      UPDATEDATE= v_BeginTS
	WHERE	MESSAGEID = p_MESSAGEID
	---And RowTS = p_RowTS
	RETURNING ROWTS, RMStateHdrId INTO v_ROWTS, v_RMStateHdrId
	;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;

	--- Introduce next message in the flow 
	IF p_SeqStatus = 1 THEN
		UPDATE	bp_rm_state_dtl
		SET	DocIntoFlow = 1
			,UpdateDate= v_BeginTS
		WHERE	RMStateHdrId = v_RMStateHdrID
		AND	SequenceNumber = p_SequenceNumber + 1
		AND	SeqStatus = 0
		;
	END IF;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN ZeroUpdate then
		v_ExceptionFlg := 1;
		p_ReturnCd := 3;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
	WHEN OTHERS then
		ROLLBACK work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
End BP_RM_STATE_HDRU_AH1;
/


-- BP_RM_STATE_QUEI_AH1.sql

CREATE OR REPLACE PROCEDURE BP_RM_STATE_QUEI_AH1
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_RowCnt		in  pls_integer
	,p_WorkGroupName	in  VARCHAR2
	,p_StateInstanceName	in  VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:      BP_RM_STATE_QUEI_AH1
	Create Date:    2006-03-21 14:53:38
	Author:		SYSTEM 

Inserts work into the Reliable messaging Q table
			   ::::::::::::::::::::::::::::::
			      M O D I F I C A T I O N
			   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-21 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value	number := 0;

v_RowCnt	  INTEGER Default 0;
v_BeginTS	 TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName	VARCHAR2(30) Default 'BP_RM_STATE_QUEI_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||'p_RowCnt=>'||p_RowCnt||CHR(10)
		||',p_WorkGroupName=>'||p_WorkGroupName||CHR(10)
		||',p_StateInstanceName =>'||p_StateInstanceName||CHR(10)
		;

---------------------------------------------------------------------------
--			 M    A    I    N			      --
---------------------------------------------------------------------------

BEGIN
	LOCK TABLE BP_RM_STATE_QUE IN EXCLUSIVE MODE;
	Insert into BP_RM_STATE_QUE(
		ROW_ID
		,updatedate
	)  (SELECT dtl.ROWID, dtl.updatedate
		FROM	bp_rm_state_dtl dtl
		INNER JOIN bp_rm_state_hdr hdr ON hdr.rmstatehdrid = dtl.rmstatehdrid
		WHERE	hdr.WorkGroupName = p_WorkGroupName
--		AND	hdr.StateInstanceName = p_StateInstanceName
		AND
			(
				(DocIntoFlow = 1  --- Needs to be introduced into the flow
				AND SeqStatus = 0 --- Document is NOT in correct sequence
				)
			OR
				(	AckStatus = 0             --- Ack not obtained
				)
			)
		AND NextRetryTS <= v_BeginTS  --- RetryTime is past
	);
	v_RowCnt := sql%RowCount;
	COMMIT;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Dup_Val_On_Index THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	WHEN Others THEN
		Rollback work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		RAISE;
END BP_RM_STATE_QUEI_AH1;
/


-- BP_RNMSGDIGESTI_AH1.sql

CREATE OR REPLACE PROCEDURE BP_RNMSGDIGESTI_AH1
    (
    p_ReturnCd              out Integer
    ,p_DebugFlg             in Pls_Integer
    ,p_INITBUSINESSID       in VARCHAR2
    ,p_PROCESSINSTANCEID    in VARCHAR2
    ,p_MSGTRACKINGINSTANCEID    in VARCHAR2
    ,p_MSGDIGEST            in VARCHAR2
    )
as
/*
    ================================================================================
    Proc Name:        BP_RNMSGDIGESTI_AH1
    Create Date:     2002-10-04 16:01:06
    Author:            SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2003-09-26 SYSTEM	3.5	Created
	2007-06-28 S.Priani	6.1	Added MSGTRACKINGINSTANCEID

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt    integer     :=0;
v_BeginTS   timestamp := FN_Get_Date; -- Performance related variables
v_pk_value  number := 0;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('BP_RNMSGDIGESTI_AH1',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||p_INITBUSINESSID
    ||','||p_PROCESSINSTANCEID
    ||','||p_MSGTRACKINGINSTANCEID
    ||','||p_MSGDIGEST
    );
End ProcAudit;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    Insert into BP_RNMSGDIGEST(
            INITBUSINESSID
        ,    PROCESSINSTANCEID
        ,    MSGTRACKINGINSTANCEID
        ,    MSGDIGEST
        ) Values (
            p_INITBUSINESSID
        ,    p_PROCESSINSTANCEID
        ,    p_MSGTRACKINGINSTANCEID
        ,    p_MSGDIGEST
        );
    ProcAudit;
    EXCEPTION
    When NO_DATA_FOUND Then Null;
    When Dup_val_on_index then

        update bp_RNMSGDigest
        set msgdigest = p_MSGDIGEST
        where InitBusinessId = p_INITBUSINESSID
        and   ProcessInstanceId = p_PROCESSINSTANCEID
        and   MSGTRACKINGINSTANCEID = p_MSGTRACKINGINSTANCEID
    ;

        p_returnCd := 2;
    When Others Then
        p_ReturnCd := 1;
        ProcAudit;
        Raise_application_error (-20999,'20000BP_RNMSGDIGESTI_AH1' || SQLCODE|| ' '||SQLERRM);

END BP_RNMSGDIGESTI_AH1;
/

-- BP_RNMSGDIGESTS.sql

CREATE OR REPLACE PROCEDURE BP_RNMSGDIGESTS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_INITBUSINESSID IN VARCHAR2
	, p_PROCESSINSTANCEID IN VARCHAR2
	, p_MSGTRACKINGINSTANCEID    in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_RNMSGDIGESTS
	Create Date:     2003-09-26 00:07:11
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2003-09-26 SYSTEM	3.5	Created
	2007-06-28 S.Priani	6.1	Added MSGTRACKINGINSTANCEID

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_RNMSGDIGESTS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
		||', p_MSGTRACKINGINSTANCEID=>'||p_MSGTRACKINGINSTANCEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_INITBUSINESSID IS NOT NULL
	 AND p_PROCESSINSTANCEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		RTRIM(INITBUSINESSID) INITBUSINESSID
		,RTRIM(PROCESSINSTANCEID) PROCESSINSTANCEID
		,RTRIM(MSGTRACKINGINSTANCEID) MSGTRACKINGINSTANCEID
		,RTRIM(MSGDIGEST) MSGDIGEST
		,CREATEDATE
	FROM
		BP_RNMSGDIGEST
	Where
		INITBUSINESSID = p_INITBUSINESSID
		AND PROCESSINSTANCEID = p_PROCESSINSTANCEID
		AND  MSGTRACKINGINSTANCEID = p_MSGTRACKINGINSTANCEID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		RTRIM(INITBUSINESSID) INITBUSINESSID
		,RTRIM(PROCESSINSTANCEID) PROCESSINSTANCEID
		,RTRIM(MSGTRACKINGINSTANCEID) MSGTRACKINGINSTANCEID
		,RTRIM(MSGDIGEST) MSGDIGEST
		,CREATEDATE
	FROM
		BP_RNMSGDIGEST
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End BP_RNMSGDIGESTS;
/



-- BP_RNSTATEHDRAUDITLOGI.sql

CREATE OR REPLACE Procedure BP_RNSTATEHDRAUDITLOGI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_INITBUSINESSID in VARCHAR2
	,p_PROCESSINSTANCEID in VARCHAR2
	,p_RNSTATEDTLID in NUMBER
	,p_NEXTACTIONCD in VARCHAR2
	,p_MSGSENTSTATUSCD in VARCHAR2
	,p_CURRENTPIPSTATUSCD in VARCHAR2
	,p_ERRORREASONTEXT in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       BP_RNSTATEHDRAUDITLOGI
	Create Date:     2003-09-26 00:03:38
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_RNSTATEHDRAUDITLOGI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
		||', p_RNSTATEDTLID=>'||p_RNSTATEDTLID||CHR(10)
		||', p_NEXTACTIONCD=>'||p_NEXTACTIONCD||CHR(10)
		||', p_MSGSENTSTATUSCD=>'||p_MSGSENTSTATUSCD||CHR(10)
		||', p_CURRENTPIPSTATUSCD=>'||p_CURRENTPIPSTATUSCD||CHR(10)
		||', p_ERRORREASONTEXT=>'||p_ERRORREASONTEXT||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into BP_RNSTATEHDRAUDITLOG(
		INITBUSINESSID
		,PROCESSINSTANCEID
		,RNSTATEDTLID
		,NEXTACTIONCD
		,MSGSENTSTATUSCD
		,CURRENTPIPSTATUSCD
		,ERRORREASONTEXT
	) Values (
		p_INITBUSINESSID
		,p_PROCESSINSTANCEID
		,p_RNSTATEDTLID
		,p_NEXTACTIONCD
		,p_MSGSENTSTATUSCD
		,p_CURRENTPIPSTATUSCD
		,p_ERRORREASONTEXT
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 BP_RNSTATEHDRAUDITLOGI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 BP_RNSTATEHDRAUDITLOGI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 BP_RNSTATEHDRAUDITLOGI' || SQLCODE|| ' '||SQLERRM);
END BP_RNSTATEHDRAUDITLOGI;
/


-- BP_RNSTATEHDRI_AH1.sql

CREATE OR REPLACE PROCEDURE BP_RNSTATEHDRI_AH1
(
    p_ReturnCd                    OUT       INTEGER,
    p_DebugFlg                    IN        PLS_INTEGER,
    p_SqlErrm                     OUT       VARCHAR2,
    p_RNSTATEDTLID                OUT       Number,
    p_NEXTACTIONCD                OUT       VARCHAR2,
    p_MSGSENTSTATUSCD             OUT       VARCHAR2,
    p_CURRENTPIPSTATUSCD          OUT       VARCHAR2,
    p_out_ATTEMPTCNT              OUT       Number,
    p_MaxATTEMPTCNT               OUT       Number,    -- (Per TPA from pr_channelDef)
    p_rowTS                       OUT       Number,
    p_RAStatus                    OUT        varchar2,
    p_channelID                   IN        PLS_INTEGER,
    p_INITBUSINESSID              IN        VARCHAR2,
    p_PROCESSINSTANCEID           IN        VARCHAR2,
    p_ACTIVITYID                  IN        VARCHAR2, -- DTL
    p_InboundVuid                 IN        VARCHAR2, -- DTL
    p_FrPROTOCOLNAME              IN        VARCHAR2,
    p_FrPROTOCOLVER               IN        VARCHAR2,
    p_PROCESSCD                   IN        VARCHAR2,
    p_PROCESSVER                  IN        VARCHAR2,
----- DTL -----
    p_TRANSACTIONINSTANCEID       IN        VARCHAR2,
    p_ACTIONINSTANCEID            IN        VARCHAR2,
    p_SIGNALINSTANCEID            IN        VARCHAR2,
    p_MSGTRACKINGINSTANCEID       IN        VARCHAR2,
    p_GLBUSAGECD                  IN        VARCHAR2,
    p_ACTIONVER                   IN        VARCHAR2,
    p_SIGNALVER                   IN        VARCHAR2,
    p_GLBBUSACTIONCD              IN        VARCHAR2,
    p_GLBBUSSIGNALCD              IN        VARCHAR2,
    p_FRBUSINESSID                IN        VARCHAR2,
    p_TOBUSINESSID                IN        VARCHAR2,
    p_ACTIVITYNAME                IN        VARCHAR2,
    p_MSGDATETIME                 IN        VARCHAR2,
    p_ATTEMPTCNT                  IN        PLS_INTEGER,
-- Hdr
    p_FRGLBPARTNERCLASSCD        IN         VARCHAR2,
    p_TOGLBPARTNERCLASSCD        IN         VARCHAR2,
    p_MSGDIGEST                  IN         CLOB,                 -- LONG,
    p_DIGITALSIGALGO             IN         VARCHAR2,
    p_CUSTOMMSGID01              IN         VARCHAR2,     -- DTL
    p_ToPROTOCOLNAME             IN         VARCHAR2,     -- To determine req_rn_protocolver
    p_ToPROTOCOLVER              IN         VARCHAR2,
    p_reStartFlg                 IN         PLS_INTEGER, -- 0 = ignore the flag, 1 = it is a reStart
    p_SponsorFlg                 IN         PLS_INTEGER,  -- 0 = insert  1= Do Not Insert
    p_ToProcessCd                in         varchar2,
    p_ToProcessVer               in         varchar2,
    --p_destTypeID                 in         pls_integer,
    p_DestName                   in         VARCHAR2,
    p_dupProcessInstanceId       in         VARCHAR2,
    p_SingleActionFlg            in         PLS_INTEGER, -- 1= true 0= false
    p_SYNCHRONOUSFLG             in         PLS_INTEGER, -- 1= true 0= false
    p_RESPONSEURL                in         VARCHAR2,    -- (255)
    p_REPLYTOMESSAGEID           in         VARCHAR2,    -- (255)
    p_FRPACKAGENAME              in         VARCHAR2,  --(30)
    p_FRPACKAGEVER               in         VARCHAR2,  --(15)
    p_TOPACKAGENAME              in         VARCHAR2,  --(30)
    p_TOPACKAGEVER               in         VARCHAR2,  --(15)
    p_DocEncryptionCd            in         PLS_Integer,
--
    p_TimeToAck_TPA           in         PLS_Integer,
    p_TimeToPerform_TPA       in         PLS_Integer,
    p_RetryCount_TPA          in         PLS_Integer,
    p_GLOBALSUPPLYCHAINCD_TPA in         VARCHAR2,
    p_SyncAckRequired_TPA     in         VARCHAR2,
    p_REQ_ENCRYPTALGO         in         Varchar2,
    p_MSGSTANDARD             IN	VARCHAR2,
    p_MSGSTANDARD_VER         IN	VARCHAR2,
    p_PIPPAYLOADBINDINGID     IN	VARCHAR2,
    P_SENDERLOCATIONID        IN	VARCHAR2,
    P_RECEIVERLOCATIONID      IN	VARCHAR2
    )
AS
/*
	================================================================================
    Proc Name:      BP_RNSTATEHDRI_AH1
    Create Date:    2002-04-23 12:04:11
    Author:         V Goel
        ---------- ------------	------- ---------------------------------------------
        DATE       NAME		VER    DESCRIPTION
        ---------- ------------	------- ---------------------------------------------
        2003-11-10 SYSTEM	3.5	Created
        2006-01-10 S.Priani	6.3	JR23074
        2006-07-25 S.Priani	6.3	102154
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

    v_destTypeID     PLS_INTEGER default FN_GET_DESTTYPEID (p_DestName);
    v_rowcnt         INTEGER := 0;
    v_BeginTS        Timestamp := FN_Get_Date;    -- Performance related variables
    v_ExceptionFlg   Integer := 0;
    v_id             pls_integer := 0;

    V_TIMETOACKTPA   pls_integer := Null;   -- bp_rnStateHdr.EXPIRETPATIME%type;  --
    v_ATTEMPTCNTTPA  bp_rnStateHdr.ATTEMPTCNTTPA%type := Null;
    v_TimeToPerform  pls_integer := Null;
    v_GlobalSupplyChainCode BP_RNStateDTL.TOSUPPLYCHAINCD%Type := Null;
    v_EncryptionTPA  pls_Integer Default null;
    v_RetryCount_TPA PLS_Integer;
    v_ATTEMPTCNTCALC pls_integer ;
    v_TimeToResponse varchar2(10);
    v2_EXPIRETPATIME bp_rnStateHdr.EXPIRETPATIME%type ;
    v_SyncAckRequired bp_rnStateHdr.SyncAckRequired%Type;
    v_SynchronouSFlg pls_integer;
    v_ChannelID      pls_integer;
    v_getTPA_Flg     pls_integer default 0;
    v_InsertHdr      pls_integer default 0;
    v_FrProtocolName bp_rnStateHdr.ProtocolName%TYPE;
    v_ToProtocolName bp_rnStateHdr.ToProtocolName%TYPE;

    amount           pls_integer;
    position         pls_integer := 1;
    v_buffer         varchar2(32767);
    v_req_rn_protocolver bp_rnStateHdr.req_rnprotocolver%type;
    v_do_dtl_flag    Boolean := TRUE;
    v_ProcessCd      bp_rnstatehdr.processcd%type;
    v_ProcessVer     bp_Rnstatehdr.processVer%type;
    v_ProcessInstanceId varchar2(255);
    v_InitBusinessId    varchar2(20);
    v_ProcessCd2     bp_rnstatehdr.processcd%type;
    v_ProcName       varchar2(30) := 'BP_RNSTATEHDRI_AH1';

	v_RC1		DB_REF.RCT;
--
v_ParamsTxt       varchar(4000) :=
        ' p_DebugFlg=>'||p_DebugFlg||CHR(10)
    ||', p_channelID  =>'|| p_channelID ||CHR(10)
    ||', p_INITBUSINESSID =>'|| p_INITBUSINESSID||CHR(10)
    ||', p_PROCESSINSTANCEID =>'||p_PROCESSINSTANCEID  ||CHR(10)
    ||', p_ACTIVITYID       =>'|| p_ACTIVITYID ||CHR(10)
    ||', p_InboundVuid      =>'||p_InboundVuid  ||CHR(10)
    ||', p_FrPROTOCOLNAME   =>'||p_FrPROTOCOLNAME ||CHR(10)
    ||', p_FrPROTOCOLVER    =>'||p_FrPROTOCOLVER ||CHR(10)
    ||', p_PROCESSCD        =>'||p_PROCESSCD    ||CHR(10)
    ||', p_PROCESSVER       =>'||p_PROCESSVER   ||CHR(10)
    ||', p_TRANSACTIONINSTANCEID=>'|| p_TRANSACTIONINSTANCEID ||CHR(10)
    ||', p_ACTIONINSTANCEID=>'||  p_ACTIONINSTANCEID ||CHR(10)
    ||', p_SIGNALINSTANCEID =>'|| p_SIGNALINSTANCEID  ||CHR(10)
    ||', p_MSGTRACKINGINSTANCEID=>'|| p_MSGTRACKINGINSTANCEID||CHR(10)
    ||', p_GLBUSAGECD  =>'|| p_GLBUSAGECD ||CHR(10)
    ||', p_ACTIONVER  =>'|| p_ACTIONVER   ||CHR(10)
    ||', p_SIGNALVER   =>'||p_SIGNALVER   ||CHR(10)
    ||', p_GLBBUSACTIONCD  =>'||p_GLBBUSACTIONCD  ||CHR(10)
    ||', p_GLBBUSSIGNALCD  =>'||p_GLBBUSSIGNALCD  ||CHR(10)
    ||', p_FRBUSINESSID    =>'|| p_FRBUSINESSID   ||CHR(10)
    ||', p_TOBUSINESSID  =>'|| p_TOBUSINESSID      ||CHR(10)
    ||', p_ACTIVITYNAME   =>'||p_ACTIVITYNAME   ||CHR(10)
    ||', p_MSGDATETIME   =>'|| p_MSGDATETIME     ||CHR(10)
    ||', p_ATTEMPTCNT =>'||p_ATTEMPTCNT||CHR(10)
    ||', p_FRGLBPARTNERCLASSCD=>'||p_FRGLBPARTNERCLASSCD  ||CHR(10)
    ||', p_TOGLBPARTNERCLASSCD  =>'|| p_TOGLBPARTNERCLASSCD ||CHR(10)
    ||', p_MSGDIGEST          =>'|| p_MSGDIGEST  ||CHR(10)
    ||', p_DIGITALSIGALGO   =>'|| p_DIGITALSIGALGO   ||CHR(10)
    ||', p_CUSTOMMSGID01    =>'||  p_CUSTOMMSGID01   ||CHR(10)
    ||', p_ToPROTOCOLNAME   =>'|| p_ToPROTOCOLNAME   ||CHR(10)
    ||', p_ToPROTOCOLVER    =>'||p_ToPROTOCOLVER    ||CHR(10)
    ||', p_reStartFlg      =>'||p_reStartFlg   ||CHR(10)
    ||', p_SponsorFlg     =>'|| p_SponsorFlg    ||CHR(10)
    ||', p_ToProcessCd    =>'|| p_ToProcessCd   ||CHR(10)
    ||', p_ToProcessVer   =>'|| p_ToProcessVer   ||CHR(10)
    ||', p_destName     =>'|| p_destName   ||CHR(10)
    ||', p_dupProcessInstanceId =>'|| p_dupProcessInstanceId||CHR(10)
    ||', p_SingleActionFlg=>'|| p_SingleActionFlg||CHR(10)
    ||', p_SYNCHRONOUSFLG=>' || p_SYNCHRONOUSFLG||CHR(10)
    ||', p_RESPONSEURL=>'    || p_RESPONSEURL||CHR(10)
    ||', p_REPLYTOMESSAGEID=>'||p_REPLYTOMESSAGEID||CHR(10)
    ||', p_FRPACKAGENAME=>'||p_FRPACKAGENAME||CHR(10)
    ||', p_FRPACKAGEVER=>'||p_FRPACKAGEVER||CHR(10)
    ||', p_TOPACKAGENAME=>'||p_TOPACKAGENAME||CHR(10)
    ||', p_TOPACKAGEVER=>'||p_TOPACKAGEVER||CHR(10)
    ||', p_DocEncryptionCd=>'|| p_DocEncryptionCd||CHR(10)
    ||', p_TimeToAck_TPA =>'|| p_TimeToAck_TPA||CHR(10)
    ||', p_TimeToPerform_TPA=>'|| p_TimeToPerform_TPA||CHR(10)
    ||', p_RetryCount_TPA=>'|| p_RetryCount_TPA||CHR(10)
    ||', p_GLOBALSUPPLYCHAINCD_TPA=>'|| p_GLOBALSUPPLYCHAINCD_TPA||CHR(10)
    ||', p_SyncAckRequired_TPA=>'|| p_SyncAckRequired_TPA||CHR(10)
    ||', p_REQ_ENCRYPTALGO=>'||p_REQ_ENCRYPTALGO||CHR(10)
    ||', p_MSGSTANDARD=>'||p_MSGSTANDARD||chr(10)
    ||', p_MSGSTANDARD_VER=>'||p_MSGSTANDARD_VER||chr(10)
    ||', p_PIPPAYLOADBINDINGID=>'||p_PIPPAYLOADBINDINGID||chr(10)
    ||', P_SENDERLOCATIONID=>'||P_SENDERLOCATIONID||chr(10)
    ||', P_RECEIVERLOCATIONID=>'||P_RECEIVERLOCATIONID||chr(10) ;


--  Procedures
--FUNCTION  processid RETURN INTEGER;
--
    PROCEDURE write_auditLog is
        r_code  pls_integer;
        d_flag pls_integer := 0;
    BEGIN
        BP_RNSTATEHDRAUDITLOGI (
        p_ReturnCd => r_code
        ,p_DebugFlg => d_flag
        ,p_INITBUSINESSID     => p_INITBUSINESSID
        ,p_PROCESSINSTANCEID  => p_PROCESSINSTANCEID
        ,p_RNSTATEDTLID       => p_RNSTATEDTLID
        ,p_NEXTACTIONCD       => p_NEXTACTIONCD
        ,p_MSGSENTSTATUSCD    => p_MSGSENTSTATUSCD
        ,p_CURRENTPIPSTATUSCD => p_CURRENTPIPSTATUSCD
        ,p_ERRORREASONTEXT    => 'Insert...'|| ' TS '||to_char(FN_Get_Date, 'dd_MON-yyyy hh24:mi:ssxFF')
        );
    END write_auditLog;
--
    PROCEDURE ProcAudit
    IS
    BEGIN
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    END ProcAudit;
--
    PROCEDURE Get_detail_for_restart is
        v_RNSTATEDTLID pls_integer := 0;
    BEGIN
        select  RNSTATEDTLID into v_RNSTATEDTLID
        from bp_rnStateDTL
        Where InboundVuid  = p_InboundVuid;
        p_RNSTATEDTLID := v_RNSTATEDTLID;
        v_do_dtl_flag := False;
    Exception
        When NO_DATA_FOUND Then
        Null;
        When Others Then
            ROLLBACK WORK;
            v_ExceptionFlg := 1;
		ProcAudit;
            DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, 'BP_RNSTATEHDRI_AH1--Get_detail_for_restart - SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
            p_ReturnCd := 1;
            Raise;

    END Get_detail_for_restart;
--
    PROCEDURE RNSTATEDTLi IS
    BEGIN
    INSERT INTO BP_RNSTATEDTL
        (
            RNSTATEDTLID,
            INITBUSINESSID,
            PROCESSINSTANCEID,
            TRANSACTIONINSTANCEID,
            ACTIONINSTANCEID,
            SIGNALINSTANCEID,
            MSGTRACKINGINSTANCEID,
            GLBUSAGECD,
            ACTIONVER,
            SIGNALVER,
            GLBBUSACTIONCD,
            GLBBUSSIGNALCD,
            FRBUSINESSID,
            TOBUSINESSID,
            ACTIVITYNAME,
            MSGDATETIME,
            EXPIRETPATIME,
            ATTEMPTCNT,
            ATTEMPTCNTTPA,
            ACTIVITYID,
            InBoundVUID,
            CUSTOMMSGID01,
            MSGDIGEST ,
            DIGITALSIGALGO
            ,TimeToAck_TPA
            ,TimeToPerform_TPA
            ,RetryCount_TPA
            ,GLOBALSUPPLYCHAINCD_TPA
            ,SyncAckRequired_TPA
        )
        VALUES
        (
            BP_RNSTATEDTL_SEQ.nextval,
            p_INITBUSINESSID,
            p_PROCESSINSTANCEID,
            p_TRANSACTIONINSTANCEID,
            p_ACTIONINSTANCEID,
            p_SIGNALINSTANCEID,
            p_MSGTRACKINGINSTANCEID,
            p_GLBUSAGECD,
            p_ACTIONVER,
            p_SIGNALVER,
            p_GLBBUSACTIONCD,
            p_GLBBUSSIGNALCD,
            p_FRBUSINESSID,
            p_TOBUSINESSID,
            p_ACTIVITYNAME,
            p_MSGDATETIME,
            v2_EXPIRETPATIME,
            p_ATTEMPTCNT,
            p_MAXATTEMPTCNT,
            p_ACTIVITYID,
            p_InboundVuid,
            p_CustomMsgID01,
            p_MSGDIGEST,
            p_DIGITALSIGALGO,
            p_TimeToAck_TPA,
            p_TimeToPerform_TPA,
            v_RetryCount_TPA,
            p_GLOBALSUPPLYCHAINCD_TPA,
            p_SyncAckRequired_TPA
        )   returning RNSTATEDTLID into p_RNSTATEDTLID;
    -- ProcAudit;  Duplicate call
    EXCEPTION
    WHEN NO_DATA_FOUND THEN NULL;
    WHEN DUP_VAL_ON_INDEX THEN NULL;
    WHEN OTHERS    THEN
        ROLLBACK WORK;
        ProcAudit;
            DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, 'BP_RNSTATEHDRI_AH1--detailI - SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
        p_ReturnCd := 1;
        Raise;
    END RNSTATEDTLi;
--
--
    PROCEDURE Insert_Sponsor_state is
        rcode pls_integer;
    Begin
        IF p_SponsorFlg = 0 then
            BP_SPONSOR_STATEI_AH1 (
             p_ReturnCd          => rcode
            ,p_DebugFlg          => 0
		,RC1 => v_RC1
            ,p_INITBUSINESSID    => p_INITBUSINESSID
            ,p_PROCESSINSTANCEID => p_PROCESSINSTANCEID
            ,p_NEXTACTIONCD      => p_NEXTACTIONCD
            ,p_FRBUSINESSID      => p_FRBUSINESSID
            ,p_TOBUSINESSID      => p_TOBUSINESSID
            ,p_DOCID             => null
            ,p_CUSTOMMSGID01     => p_CUSTOMMSGID01
            ,p_UPDATESTATUSCD    => 1
            ,P_GLBUSAGECD        => P_GLBUSAGECD
            ,p_destName          => p_DestName
            ,p_InboundVuid       => p_InboundVuid
            ,p_REFINBOUNDVUID    => Null
            ,p_SynchronousFlg    => NVL(v_SynchronouSFlg, p_SynchronousFlg)
            ) ;
          If rcode >0 then
            Raise_application_error (-20999, '20005 BP_RNSTATEHDRI_AH1-Insert_Sponsor_state - Error in inserting ');
          end if;
        END IF;
     EXCEPTION
      WHEN OTHERS    THEN
            ROLLBACK WORK;
            v_ExceptionFlg := 1;
		ProcAudit;
            DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, 'BP_RNSTATEHDRI_AH1--InsertSponsorState - SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
            p_ReturnCd := 1;
            Raise;
    end Insert_Sponsor_state;
--
--  Get_TPA_parameters
--
    Procedure Get_TPA_parameters  is
        v_ROUTINGCAPABILITY_TBL PR_ROUTINGCAPABILITY_TBL ;
        v_RETURNCD        pls_integer;
        v_Count           pls_integer;
        v_PARTNERID       pls_integer;
        v_CAPABILITYID    pls_integer;
        v_ROUTINGOBJREFID pls_integer;
        v_TimeToResponse2 pls_integer;
    Begin
        Begin
            If nvl(v_FrProtocolName, p_FrProtocolName) = 'RosettaNet' Then

                SELECT  rc.FrPartnerID, rc.FrCapabilityID, c.RoutingObjRefId
                INTO    v_PartnerID,    v_CapabilityID, v_RoutingObjRefId
                FROM    pr_routingChannel rc
                INNER JOIN pr_Capability c ON c.CAPABILITYID = rc.FrCAPABILITYID
                WHERE   Channelid = p_ChannelID
                ;
            Elsif nvl(v_ToProtocolName, p_ToProtocolName) = 'RosettaNet' Then
                SELECT  rc.ToPartnerID, rc.ToCapabilityID, c.RoutingObjRefId
                INTO    v_PartnerID,    v_CapabilityID, v_RoutingObjRefId
                FROM    pr_routingChannel rc
                INNER JOIN pr_Capability c ON c.CAPABILITYID = rc.ToCAPABILITYID
                WHERE   Channelid = p_ChannelID
                ;
            End If;
            EXCEPTION
                WHEN NO_DATA_FOUND THEN NULL;
        End;

        PR_CAPABILITYS_AH5(
        P_RETURNCD               => v_RETURNCD
        ,P_DEBUGFLG              => 0  -- 0 to supress dbms output, 1 to print dbms output
        ,P_PARTNERID             => v_PARTNERID
        ,P_ROUTINGOBJREFID       => v_ROUTINGOBJREFID
        ,P_ROUTINGCAPABILITY_TBL => v_ROUTINGCAPABILITY_TBL
        ,p_StatusOption => 0
        );


        For Cx in
            (select /*+ FIRST_ROWS */  distinct
                A.AttributeKey,T.*,
                TO_NUMBER(NVL(FN_Get_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId, v_PARTNERID, p_ChannelID, 1, A.AttributeKey, 'AttrRank'), 0)) AttrRank,
                FN_Get_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId, v_PARTNERID, p_ChannelID, 1, A.AttributeKey, 'AttrValue') AttrValue
            from
                TABLE(CAST (v_RoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
                INNER JOIN PR_RoutingAttr_Lnk  Ra on Ra.RoutingObjRefId = T.RoutingObjRefId
                INNER JOIN CF_RoutingAttr A on Ra.RoutingAttrId = A.RoutingAttrId
                WHERE (
                    (Ra.partnerid = v_PartnerID or Ra.channelid = p_ChannelID or Ra.destTypeID = v_destTypeID)
                OR  (Ra.partnerid is null AND Ra.channelid is null AND Ra.destTypeID is null)
                )
                AND A.AttributeKey in ('TimeToAcknowledge', 'TimeToPerform', 'RetryCount', 'GlobalSupplyChainCode', 'SyncAckRequired', 'Encryption')
            Order by AttributeKey, AttrRank
            )
        Loop
            IF Cx.AttrRank > 0 THEN
                Case Cx.AttributeKey
                    When   'RetryCount'             Then v_ATTEMPTCNTTPA            := TO_NUMBER(Cx.AttrValue);
                    When   'TimeToAcknowledge'      Then V_TIMETOACKTPA             := TO_NUMBER(Cx.AttrValue);
                    When   'TimeToPerform'          Then V_TIMETOPERFORM            := TO_NUMBER(Cx.AttrValue);
                    When   'GlobalSupplyChainCode'  Then v_GlobalSupplyChainCode    := Cx.AttrValue  ;    -- it is a varchar2
                    When   'SyncAckRequired'        Then v_SyncAckRequired          := Cx.AttrValue  ;    -- it is a VARCHAR
                    When   'Encryption'             Then v_EncryptionTPA            := TO_NUMBER(Cx.AttrValue);
                    Else
                        null;
                End Case;
                DBMS_OUTPUT.Put_line(' Attr Key '||Cx.AttributeKey||
                                     ' Attr Val '||Cx.AttrValue||
                                     ' Attr Rank '||Cx.AttrRank||
                                     ' Obj Type '||Cx.ROUTINGOBJTYPEKEY||
                                     ' Obj Code '||Cx.ROUTINGOBJCD);
            END IF;
        End Loop;
-- The TTP for one way PIP is Null,
-- However, for sync RN one way pip TTP should have a value, but, RetryCount = 0  p_SynchronousFlg

    IF  nvl(v_SynchronouSFlg, p_SynchronousFlg) = 1 Then
        v_ATTEMPTCNTTPA := 0; -- RetryCount
    END IF;
    IF nvl(v_ATTEMPTCNTTPA, 0) = 0 AND
        V_TIMETOACKTPA  Is Null    AND
        V_TIMETOPERFORM Is Null    AND
        v_GlobalSupplyChainCode Is Null AND
        v_SyncAckRequired is null
    Then
        p_ReturnCD := 2;
        p_SqlErrm := 'All the TPAs are Null';
    END IF;
        DBMS_OUTPUT.Put_line('RetryCount '           ||v_ATTEMPTCNTTPA  ||
                            ' TimeToAcknowledge '    ||V_TIMETOACKTPA   ||
                            ' TimeToPerform '        ||V_TIMETOPERFORM  ||
                            ' GlobalSupplyChainCode '||v_GlobalSupplyChainCode ||
                            ' SyncAckRequired '      || v_SyncAckRequired
                            );
        Exception
        WHEN OTHERS    THEN
            ROLLBACK WORK;
            v_ExceptionFlg := 1;
		ProcAudit;
            DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, 'BP_RNSTATEHDRI_AH1--Get_TPAParams - SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
            p_ReturnCd := 1;
            Raise;
    End Get_TPA_parameters;
--
--
    PROCEDURE Get_Hdr_data is
    BEGIN


       SELECT NEXTACTIONCD,   MSGSENTSTATUSCD,   CURRENTPIPSTATUSCD,   ATTEMPTCNTCALC, rowTS, RAStatus, ATTEMPTCNTTPA
       INTO   p_NEXTACTIONCD, p_MSGSENTSTATUSCD, p_CURRENTPIPSTATUSCD, p_out_ATTEMPTCNT, p_rowTS, p_RAStatus, p_MaxATTEMPTCNT
       FROM  BP_RNSTATEHDR
       WHERE INITBUSINESSID = p_INITBUSINESSID
       AND   PROCESSINSTANCEID = p_PROCESSINSTANCEID;
     EXCEPTION
        WHEN OTHERS    THEN
            ProcAudit;
            ROLLBACK WORK;
            p_ReturnCd := 1;
            Raise;
    END Get_Hdr_data;

--------------------------------------------------------------------
--                             M a i n
--------------------------------------------------------------------
BEGIN
    write_auditLog;
    p_ReturnCd := 0;

-- Check duplicate ProcessInstanceId, Stop process if dup found with ProcessCd different from p_ProcessCd.
-- Then the proc will be called again with the dupProcessInstanceId
-- which will be inserted into Bp_rnstatehdr and a GE will be generated against it
-- Check the process code based upon direction
-- Check for process CD
--
   BEGIN
         SELECT     InitBusinessId, ProcessInstanceId,
                Case when protocolName = 'RosettaNet' Then  ProcessCd
                     When ToProtocolName = 'RosettaNet' Then  ToProcessCd
                Else '-2' End as ProcessCd, SynchronouSFlg, ChannelID, ProtocolName, ToProtocolName                              -- cause it to fail
         INTO v_InitBusinessId, v_ProcessInstanceid, v_ProcessCd2, v_SynchronouSFlg, v_ChannelID, v_FrProtocolName, v_ToProtocolName
         FROM       BP_RNStateHdr
         WHERE      InitBusinessId = p_INITBUSINESSID
         AND        ProcessInstanceId = p_PROCESSINSTANCEID;

         IF p_channelID IS NOT NULL AND p_channelID <> v_channelID THEN
             v_getTPA_Flg := 1;
         END IF;

         EXCEPTION
         WHEN NO_DATA_FOUND THEN
               v_getTPA_Flg := 1;
              v_InsertHdr  := 1;
   END;

   IF v_ProcessCd2 IS NOT NULL
    AND RTRIM(v_ProcessCd2) <> Case When p_frProtocolName = 'RosettaNet' Then RTRIM(p_ProcessCd)
                                    When p_ToProtocolName = 'RosettaNet' Then RTRIM(p_ToProcessCd)
                                Else '-1' End
    THEN
       p_ReturnCd := 3;
       p_SqlErrm := 'ProcessCD Comparision Error: DB: '||RTRIM(v_ProcessCd2)||' Vs Input:'||RTRIM(p_ProcessCd);
       GoTo Exit_Now;
   END IF;

-- Determine Rosettanet protocol version
    If  p_ToPROTOCOLNAME = 'RosettaNet'
    then
        v_req_rn_protocolver := p_ToPROTOCOLVER;
        v_ProcessCd := p_ToProcessCd;
        v_ProcessVer := p_ToProcessVer;
    Elsif
        p_FrPROTOCOLNAME = 'RosettaNet'
    then
        v_req_rn_protocolver := p_FrPROTOCOLVER;
        v_ProcessCd := p_ProcessCd;
        v_ProcessVer := p_ProcessVer;
    End if;
--
    IF v_getTPA_Flg = 1 THEN
        If nvl(v_SynchronousFlg, p_SynchronousFlg) = 1 Then
            v_RetryCount_TPA := 0;
        Else
            v_RetryCount_TPA := p_RetryCount_TPA;  
        End If;
    End IF;
    

    p_MaxATTEMPTCNT := TO_NUMBER(v_ATTEMPTCNTTPA);
    v2_EXPIRETPATIME := v_BeginTS ;  -- set Expire TPA Time
--
    Insert_Sponsor_state;
    IF v_InsertHdr = 1 Then
       INSERT INTO BP_RNSTATEHDR
        (
        INITBUSINESSID,
        PROCESSINSTANCEID,
        PROTOCOLNAME,
        PROTOCOLVER,
        REQ_RNPROTOCOLVER,
        PROCESSCD,
        PROCESSVER,
        RNSTATEDTLID,
        EXPIRETPATIME,
        ATTEMPTCNTCALC,
        ATTEMPTCNTTPA,
        FRGLBPARTNERCLASSCD,
        TOGLBPARTNERCLASSCD,
        TIMETOACKTPA,
        TimeToPerform,
        ToProtocolName,
        ToProtocolVer,
        ToProcessCd,
        ToProcessVer,
        destTypeID,
        DupProcessInstanceId,
        SingleActionFlg,
        GlobalSupplyChainCDTPA,
        SYNCHRONOUSFLG,
        RESPONSEURL,
        REPLYTOMESSAGEID,
        FRPACKAGENAME,
        FRPACKAGEVER,
        TOPACKAGENAME,
        TOPACKAGEVER,
        CHANNELID,
        SYNCACKREQUIRED,
        REQ_ACTIONINSTANCEID,
        REQ_MSGTRACKINGINSTANCEID,
        DocEncryptionCd,
        REQ_ENCRYPTALGO
	,MSGSTANDARD
	,MSGSTANDARD_VER
	,PIPPAYLOADBINDINGID
	,SENDERLOCATIONID
	,RECEIVERLOCATIONID
        )
        VALUES
        (
        p_INITBUSINESSID,
        p_PROCESSINSTANCEID,
        p_FrPROTOCOLNAME,
        p_FrPROTOCOLVER,
        v_req_rn_protocolver,
        p_PROCESSCD,
        p_PROCESSVER,
        p_RNSTATEDTLID,
        null,                               --v2_EXPIRETPATIME,
        1,                                   --              p_ATTEMPTCNTCALC
        v_RetryCount_TPA,    --p_MaxATTEMPTCNT,
        p_FRGLBPARTNERCLASSCD,
        p_TOGLBPARTNERCLASSCD,
        p_TimeToAck_TPA,     --v_TIMETOACKTPA,
        p_TimeToPerform_TPA, -- v_TimeToPerform,
        p_ToPROTOCOLNAME,
        p_ToPROTOCOLVER,
        p_toProcessCd,
        p_toProcessVer,
        v_destTypeID,
        p_dupProcessInstanceId,
        p_SingleActionFlg,
        p_GLOBALSUPPLYCHAINCD_TPA,  -- v_GlobalSupplyChainCode,
        p_SYNCHRONOUSFLG,
        p_RESPONSEURL,
        p_REPLYTOMESSAGEID,
        P_FRPACKAGENAME,
        P_FRPACKAGEVER,
        P_TOPACKAGENAME,
        P_TOPACKAGEVER,
        p_channelID,
        p_SyncAckRequired_TPA, -- v_SYNCACKREQUIRED,
        P_ACTIONINSTANCEID,
        P_MSGTRACKINGINSTANCEID,
        p_DocEncryptionCd,
        p_REQ_ENCRYPTALGO
	,p_MSGSTANDARD
	,p_MSGSTANDARD_VER
	,p_PIPPAYLOADBINDINGID
	,P_SENDERLOCATIONID
	,P_RECEIVERLOCATIONID
          )    Returning ATTEMPTCNTCALC, ROWTS into p_out_ATTEMPTCNT, p_rowTS;
    ELSE    -- IF v_InsertHdr = 1
        IF v_getTPA_Flg = 1 THEN
            Update     BP_RNSTATEHDR
                SET   TIMETOACKTPA           = nvl(p_TIMETOACK_TPA, TIMETOACKTPA)
                    , ATTEMPTCNTTPA          = nvl(v_RetryCount_TPA, ATTEMPTCNTTPA)
                    , GLOBALSUPPLYCHAINCDTPA = nvl(p_GLOBALSUPPLYCHAINCD_TPA, GLOBALSUPPLYCHAINCDTPA)
                    , SyncAckRequired        = nvl(p_SyncAckRequired_TPA, SyncAckRequired)
                    , ChannelID              = nvl(p_ChannelID, ChannelID)
                    , UpdateDate             = v_BeginTS
             WHERE      InitBusinessId    = p_INITBUSINESSID
             AND        ProcessInstanceId = p_PROCESSINSTANCEID;
         END IF ;
        If p_reStartFlg = 1 Then
            Get_detail_for_restart;
        End if;

         Get_Hdr_data;
    END IF;     -- IF v_InsertHdr = 1
    RNSTATEDTLi;

<<Exit_Now>>
    ProcAudit;

EXCEPTION
     When DUP_VAL_ON_INDEX then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            p_returnCd := 2;
    WHEN OTHERS    THEN
            ROLLBACK WORK;
            v_ExceptionFlg := 1;
		ProcAudit;
            DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, 'BP_RNSTATEHDRI_AH1- - SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
            p_ReturnCd := 1;
            Raise;
END;
/


-- BP_RNSTATEHDRS_0A1PIP.sql

CREATE OR REPLACE PROCEDURE         BP_RNSTATEHDRS_0A1PIP (
    p_ReturnCd                  OUT      INTEGER,
    p_DebugFlg                  IN       PLS_INTEGER,
    P_InitBusinessId            OUT      Varchar2,
    p_out_ProcessInstanceId     Out      VARCHAR2,
    p_HdrRowTS                  OUT      PLS_INTEGER,
    p_RNSTATEDTLID              OUT      PLS_INTEGER,
    p_GLBBUSACTIONCD            OUT      varchar2,
    p_ACTIONVER                 OUT      Varchar2,
    p_processcd                 OUT      Varchar2,
    p_processVer                OUT      Varchar2,
    p_NEXTACTIONCD              OUT      Varchar2,
    p_MSGSENTSTATUSCD           OUT      Varchar2,
    p_CURRENTPIPSTATUSCD        OUT      Varchar2,
    p_FrBusinessId              IN       VARCHAR2,
    p_ToBusinessId              IN       VARCHAR2,
    p_DocId                     IN       Varchar2
--

)
AS
/*
================================================================================
Proc Name:        BP_RNSTATEHDRS_0A1PIP
Create Date:    2002-04-23 12:04:11
Author:            SYSTEM
================================================================================
    Return code  = 0  Good
                   1  Something went wrong
                   2  No data found
                   3  Dup
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_rowcnt INTEGER := 0;
    v_BeginTS timestamp := FN_Get_Date;    -- Performance related variables
    v_InitBusinessId    BP_RNSTATEHDR.InitBusinessId%type;
    v_ProcessInstanceId BP_RNSTATEHDR.ProcessInstanceId%type;
    whereSql varchar2(1000);
    stmtSql  varchar2(2000);
    PROCEDURE ProcAudit
    IS
    BEGIN
        DB_ProcAuditLogI_Ah1 (
            'BP_RNSTATEHDRS_0A1PIP',
             v_beginTS,
            'p_ReturnCd'      || ',' ||
             p_DebugFlg       || ',' ||
             p_FrBusinessId   || ',' ||
             p_ToBusinessId   || ',' ||
             p_DocId
        );
    END ProcAudit;
Procedure Too_many_rows_proc is
    counter pls_integer;
Begin
--
-- Check to make sure that there is only one header
--
    select count(*) INTO counter from
    (select /*+ INDEX (D AK_RNSTATEDTL_1) */ distinct H.ProcessInstanceId  -- 7/26/2002
     from BP_RNSTATEHdr H
     inner join BP_RNSTATEDTL D on H.InitBusinessId    = d.InitBusinessId
                              and H.ProcessInstanceId = d.ProcessInstanceId
            where d.docid = p_DocID
            and   H.INITBUSINESSID in (p_FrBusinessId, p_ToBusinessId)
    );
    If counter >1 then
       p_ReturnCd := 3;
    Else
        select /*+ INDEX (D AK_RNSTATEDTL_1) */ d.InitBusinessId, d.ProcessInstanceId, d.RNSTATEDTLID,
               d.GLBBUSACTIONCD, d.ACTIONVER
               , H.processcd, H.processVer, H.NEXTACTIONCD, H.MSGSENTSTATUSCD, H.CURRENTPIPSTATUSCD,H.rowTS
        INTO p_InitBusinessId, p_out_ProcessInstanceId,
             p_RNSTATEDTLID, p_GLBBUSACTIONCD, p_ACTIONVER,  p_processcd,
             p_processVer, p_NEXTACTIONCD, p_MSGSENTSTATUSCD, p_CURRENTPIPSTATUSCD, p_HDRrowTS
        from   BP_RNSTATEDTL D
        inner Join BP_RNSTATEHDR H on H.InitBusinessId    = d.InitBusinessId
                                  and H.ProcessInstanceId = d.ProcessInstanceId
        where H.INITBUSINESSID in (p_FrBusinessId, p_ToBusinessId)
        and   DocID =p_DocID
        and  d.RNSTATEDTLID = ( select /*+ INDEX (DTL AK_RNSTATEDTL_1) */ max(dtl.RNSTATEDTLID)
                                       from BP_RNSTATEDTL dtl
                                 where  dtl.INITBUSINESSID in (p_FrBusinessId, p_ToBusinessId)
                                 and    dtl.DocID          = p_DocID
                                 )
--       and H.ProcessInstanceId =
--            (select distinct H.ProcessInstanceId from BP_RNSTATEHdr H
--                                            inner join BP_RNSTATEDTL D on H.InitBusinessId    = d.InitBusinessId
--                                                                      and H.ProcessInstanceId = d.ProcessInstanceId
--            where d.docid = p_DocID
--            and   H.INITBUSINESSID in (p_FrBusinessId, p_ToBusinessId)
--            )
        ;
   end if;
   Exception
    when no_data_found then
        p_ReturnCd := 2;
    WHEN Too_many_rows then
        p_ReturnCd := 3;
    WHEN OTHERS
    THEN
        ProcAudit;
        dbms_output.put_line('Sqlcode '||SQLCODE);
        raise_application_error (-20999, '20100 - BP_RNSTATEHDRS_0A1PIP:Too many Rows Proc' || SQLERRM);
        p_ReturnCd := 1;
end Too_many_rows_proc;
BEGIN
       p_ReturnCd := 0;
    procAudit;
  Begin
    select /*+ INDEX (D AK_RNSTATEDTL_1) */  d.InitBusinessId, d.ProcessInstanceId, d.RNSTATEDTLID,
           d.GLBBUSACTIONCD, d.ACTIONVER
           , H.processcd, H.processVer, H.NEXTACTIONCD, H.MSGSENTSTATUSCD, H.CURRENTPIPSTATUSCD,H.rowTS
    INTO p_InitBusinessId, p_out_ProcessInstanceId,
         p_RNSTATEDTLID, p_GLBBUSACTIONCD, p_ACTIONVER,  p_processcd,
         p_processVer, p_NEXTACTIONCD, p_MSGSENTSTATUSCD, p_CURRENTPIPSTATUSCD, p_HDRrowTS
    from BP_RNSTATEDTL d
    inner Join BP_RNSTATEHdr H on H.InitBusinessId    = d.InitBusinessId
                              and H.ProcessInstanceId = d.ProcessInstanceId
    where H.INITBUSINESSID in (p_FrBusinessId, p_ToBusinessId)
    and   DocID = p_DocID;
   Exception
    when no_data_found then
        p_ReturnCd := 2;
    WHEN Too_many_rows then
        Too_many_rows_proc;
    WHEN OTHERS
    THEN
        ProcAudit;
        raise_application_error (-20999, '20200 - BP_RNSTATEHDRS_0A1PIP:' || SQLERRM);
        p_ReturnCd := 1;
  End;

END BP_RNSTATEHDRS_0A1PIP;
/


-- BP_RNSTATEHDRS_AH1.sql

CREATE OR REPLACE PROCEDURE BP_RNStateHdrS_AH1(
    p_ReturnCd               OUT     pls_INTEGER,
    p_DebugFlg               IN      PLS_INTEGER,
    RC1                      OUT     db_ref.RCT,
    p_BatchSize              IN      pls_Integer
)
AS
/*
    ================================================================================
    Proc Name:        BP_RNStateHdrS_AH1
    Create Date:    2002-04-23 12:04:11
    Author:            B.Moffat
	---------- ------------	------- ---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- ---------------------------------------------
	2003-11-10 SYSTEM	3.5	Created
	2006-01-10 S.Priani	6.0.0.3	JR23074
	2006-09-12 S.Priani	6.1	PMR, CR100297
        2006-09-27 Amit M		JR24457

    ================================================================================
    Legend: NA  = NextActionCD
            MSS = MsgSentStatusCD
            SC  = Service Content
            TP  = Trading Partner
            SP  = Hub Sponsor
    RosettaNet Engine:
        1. Send right away
            NA = RA + MSS= REQSENT/RESPSENT => RA               :TP to SP Req Receipt Ack generated
            NA = RAE/GE + MSS <> RAESENT/GESENT => RAE/GE       :TP to SP Req Failed validation, Receipt Ack/General Exception generated
            NA = 0A1 + MSS = RASENT => 0A1                      :Sp to TP PiP cancelled, 0A1 started -- One way/Two way Pip
        2. Wait for TPA to expire before re-try
            NA = REQ/RESP + MSS = REQSENT/RESPSENT   (update ExpireTPATime in BP_RNStateHdr_U)
        3. Retry
            NA = REQ/RESP + MSS = REQSENT/RESPSENT + AckTPA time Expired => SC retry     :Sp to TP Req sent No Receipt Ack, Re-send Req
            NA = REQ/RESP + MSS = REQSENT/RESPSENT + AckTPA time Expired + Max retries reached + not 0A1 => 0A1
                                                            ::SP to TP Req Sent No Ack, Retries Exceeded, 0A1 Started
        4. Retries Failed
            NA = REQ/RESP + MSS = REQFAILED/RESPFAILED + AckTPA time Expired => SC retry :SP to TP Req/Resp failed, Retry attempted
            NA = REQ/RESP + MSS = REQFAILED/RESPFAILED + AckTPA time Expired + Max retries reached + not 0A1 +
                 TimeToPerForm Exceeded=> 0A1
                                                                               ::SP to TP Req failed max retries, 0A1 started
        5. Receipt Ack Retry
            NA = RA +   MSS = REQACKFAILED/RESPACKFAILED
        6. Failed delivery to SP
            NA = 0A1 + MSS = REQFAILED/RESPFAILED

        7. Time To Perform Exceeded (Lost Document)
            NA = RAREC + MSS = REQSENT + Time2perform Exceeded
        8  Out of Sequence
            NA = RESP + RAStatus is null => issue oA1

        -- overview
           update one row at a time.
           1) select all the records meet the update criteria through For Loop.
           2) fetch the first row from the cursor doing "select for update Skip Locked" on this rowid.
              If succeed, call Update_RnHeader to update the row.
              if failed to lock the row, loop to fetch the next row from the cursor.
           3) The update routine re-evaluate the update criteria(ExpirationTPATime) because the row could be updated by other threads
              after the cursor select. If update succeed, return the updated row to the calling thread by refcursor.
              If no row to update, go back to step 2).

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
--
-- Cursor
--
--  Cursor c_hdr is select rowid from BP_RNStateHdr01_VW order by UpdateDate;
------------------
    v_RowCnt                INTEGER := 0;
    v_BeginTS               timestamp := FN_Get_Date();    -- Performance related variables
    v_RowID                 ROWID;
    v_count                 pls_integer;
    v_UpdateSuccessfulFlg   boolean := False;
    v_ProcName              varchar2(30) := 'BP_RNSTATEHDRS_AH1';
    v_NewDocFlg             integer := 0;
    v_BatchSize             integer := p_BatchSize + 1;
    v_BatchRows             integer := 0;
    v_Rows                  integer := 0;
    v_Rows2                 integer := 0;
    j                       integer := 0;
    v_Msg                   varchar2(1024);
    v_RETURNCD              pls_integer;
---
    v_TIMETOACKTPA          pls_integer;
    v_INITBUSINESSID        BP_RNStateHdr.INITBUSINESSID%Type := null;
    v_PROCESSINSTANCEID     BP_RNStateHdr.PROCESSINSTANCEID%Type;
    v_RNSTATEDTLID          BP_RNStateHdr.RNSTATEDTLID%Type;
    v_NEXTACTIONCD          BP_RNStateHdr.NEXTACTIONCD%Type;
    v_MSGSENTSTATUSCD       BP_RNStateHdr.MSGSENTSTATUSCD%Type;
    v_CURRENTPIPSTATUSCD    BP_RNStateHdr.CURRENTPIPSTATUSCD%Type;
    v_EXPIRETPATIME         BP_RNStateHdr.EXPIRETPATIME%Type;
    V_ACTIONTIME	    BP_RNStateHdr.ACTIONTIME%Type;
    v_RowTS                 BP_RNStateHdr.RowTS%Type;
    v_UpdateStatusCD        BP_RNStateHdr.UpdateStatusCD%Type;
    v_ParamsTxt             varchar2(1024) := 'p_ReturnCd,' || p_DebugFlg|| ', RC1,' || p_BatchSize;
    Type t_RowID            is table of ROWID;
    Type t_UpdateDate       is table of TIMESTAMP;
    array_RowID             t_RowID  := t_RowID();
    array_UpdateDate        t_UpdateDate  := t_UpdateDate();

---
-------------------------------------------------
-------------------------------------------------
Procedure InsertLog as
    v_date date ;
BEGIN

    Begin
        v_date := v_EXPIRETPATIME;
        v_Msg := 'Select_ah1...'|| ' ExpireTPA '||
                 to_char(v_Date , 'dd_mon-yy hh24:mi:ss')||' Status '||v_UpdateStatusCD;
    Exception
        When Others Then Null;             
    End;
    If v_PROCESSINSTANCEID is not null then
     Begin
     BP_RNStateHdrAUDITLOGI(
       P_RETURNCD            => V_RETURNCD
      ,P_DEBUGFLG           => 0
      ,P_INITBUSINESSID     => v_INITBUSINESSID
      ,P_PROCESSINSTANCEID  => v_PROCESSINSTANCEID
      ,P_RNSTATEDTLID       => v_RNSTATEDTLID
      ,P_NEXTACTIONCD       => v_NEXTACTIONCD
      ,P_MSGSENTSTATUSCD    => v_MSGSENTSTATUSCD
      ,P_CURRENTPIPSTATUSCD => v_CURRENTPIPSTATUSCD
      ,P_ERRORREASONTEXT    => v_Msg
        );
    EXCEPTION
        When NO_DATA_FOUND Then Null;
        When Others Then
            Null;
      End;
    End if;
END InsertLog;
---
---------------------------------------------------------------
---                    M   A   I   N                        ---
---------------------------------------------------------------

BEGIN
	BEGIN
        	DELETE FROM BP_RNStateHdr_Que
		Where rownum < 2
		Returning Row_ID  into v_RowID;
		v_RowCnt := SQL%ROWCOUNT;
		COMMIT;
	END;

	IF v_RowCnt = 0 THEN
		BP_RNStateHdr_QueI_AH1 (p_ReturnCd, v_Rowcnt);
	ELSE
	BEGIN
            update BP_RNStateHdr H /*+ FIRST_ROWS */
            set
                H.UpdateStatusCd = 2,
                H.UpdateDate = FN_Get_Date(),
                H.ExpireTPATime = FN_Get_Date() + NUMTODSINTERVAL(COALESCE(H.TIMETOACKTPA,5), 'MINUTE')   -- set up time for retry based upon TPA interval, if null, use 5 minutes
            WHERE
                H.rowid = v_RowID
                and
                (
                    (H.ExpireTPATime  <= FN_Get_Date() or H.ExpireTPATime is Null)
                     OR
                    (UpdateStatusCd = 1 and NextActionCd IN ('RA', '0A1', 'RAE', 'GE', 'RESP') )
                 )
            RETURNING
                INITBUSINESSID,
                PROCESSINSTANCEID,
                RNSTATEDTLID,
                NEXTACTIONCD,
                MSGSENTSTATUSCD,
                CURRENTPIPSTATUSCD,
                EXPIRETPATIME,
                TIMETOACKTPA,
                ACTIONTIME,
                RowTS,
                UpdateStatusCD
            INTO
               V_INITBUSINESSID    ,
               V_PROCESSINSTANCEID ,
               V_RNSTATEDTLID      ,
               V_NEXTACTIONCD      ,
               V_MSGSENTSTATUSCD   ,
               V_CURRENTPIPSTATUSCD,
               V_EXPIRETPATIME     ,
               v_TIMETOACKTPA      ,
               V_ACTIONTIME	,
               v_RowTS,
               v_UpdateStatusCD ;
            If sql%rowCount > 0  Then
                v_UpdateSuccessfulFlg := True;
            End if;
            Exception
                WHEN NO_DATA_FOUND THEN
                    ROLLBACK;
                    v_RowCnt := 0;
	END;
	END IF;

    p_ReturnCd := 0;
    IF RC1%ISOPEN THEN
        Close RC1;
    END IF;
    if NOT (v_UpdateSuccessfulFlg) then
        v_RowID := null;
    end if;
    open RC1 for
        select /*+ FIRST_ROWS */
            Case
                when
                    H.UpdateStatusCd = 2  and                 -- retries reached max, issue "0A1"
                    H.nextActionCD in ('REQ', 'RESP')  and H.MSGSENTSTATUSCD in ('REQSENT', 'REQFAILED','RESPSENT','RESPFAILED', 'REQACKSENT','REQACKFAILED' )
                    and H.AttemptCntCalc > H.AttemptCntTPA
                    and H.processcd != '0A1'
                    and ((H.SINGLEACTIONFLG = 0) or
                            (h.SINGLEACTIONFLG = 1 and (FN_TS2Date() - (FN_TS2Date(H.CreateDate)))*24*60 > (( h.AttemptCntTPA + 1 ) * h.TimeToAckTPA)))    -- one way PIP
                    Then '0A1'
                When
                    H.UpdateStatusCd = 2                  -- retries NOT reached max, But, Time to Perform Exceeded , issue "0A1"
                    and H.nextActionCD in ('REQ', 'RESP')
                    and H.MSGSENTSTATUSCD in ('REQSENT', 'REQFAILED','RESPSENT','RESPFAILED', 'REQACKSENT', 'REQACKFAILED' )
                    And H.TimeToPerform is not null
                    and (FN_TS2Date() - (FN_TS2Date(H.CreateDate)))*24*60 > h.TimeToPerform
                    And H.SINGLEACTIONFLG = 0   -- two way PIP
                    Then '0A1'
                When
                    H.UpdateStatusCd = 2  and                 -- Time to perform Exceeded, issue "0A1"
                    H.nextActionCD = 'RAREC' and H.MSGSENTSTATUSCD = 'REQSENT'
                    and H.TimeToPerform is not null and (FN_TS2Date() - (FN_TS2Date(H.CreateDate)))*24*60 > H.TimeToPerform
                    Then '0A1'
                When
                    UpdateStatusCd =2                       -- Out-Of-Sequence
                    and nextActionCD = 'RESP'   --  and MSGSENTSTATUSCD = 'REQSENT'
                    and RAStatus is null
                    Then 'RA'
            Else
                H.NEXTACTIONCD
            END AS RN_State_Action_CD,
            H.INITBUSINESSID,
            H.PROCESSINSTANCEID,
            D.ACTIVITYID,
            D.OutBoundVUID     VUID,
            H.PROTOCOLNAME,
            --JR24457 starts
            H.CREATEDATE,
            --JR24457 ends
            H.PROTOCOLVER,
            H.PROCESSCD,
            H.PROCESSVER,
            H.REQ_RNPROTOCOLVER,
            H.NEXTACTIONCD,
            H.MSGSENTSTATUSCD,
            H.CURRENTPIPSTATUSCD,
            H.REFINITBUSINESSID,
            H.REFPROCESSINSTANCEID,
            H.RNSTATEDTLID,
            H.EXPIRETPATIME,
            H.ATTEMPTCNTCALC,
            H.ATTEMPTCNTTPA,
            Case
                When
                    (H.UpdateStatusCd = 2  and                 -- retries reached max, issue "0A1"
                     H.nextActionCD in ('REQ', 'RESP')  and H.MSGSENTSTATUSCD in ('REQSENT', 'REQFAILED','RESPSENT','RESPFAILED', 'REQACKSENT' )
                     and H.AttemptCntCalc > H.AttemptCntTPA
                     and H.processcd != '0A1'
                    )
                Then  'Retries exhausted'
                When
                (
                  ( H.UpdateStatusCd = 2                   -- retries NOT reached max, But, Time to Perform Exceeded , issue "0A1"
                    and H.nextActionCD in ('REQ', 'RESP')
                    and H.MSGSENTSTATUSCD in ('REQSENT', 'REQFAILED','RESPSENT','RESPFAILED', 'REQACKSENT' )
                    and H.processcd != '0A1'
                    And H.TimeToPerform is not null
                    and (FN_TS2Date() - (FN_TS2Date(H.CreateDate)))*24*60 > h.TimeToPerform    -- two way PIP
                  )
                OR
                  (
                    H.UpdateStatusCd = 2                  -- Time to perform Exceeded, issue "0A1"
                    and H.nextActionCD = 'RAREC'    and H.MSGSENTSTATUSCD = 'REQSENT'
                    and H.TimeToPerform is not null and (FN_TS2Date() - (FN_TS2Date(H.CreateDate)))*24*60 > H.TimeToPerform
                  )
                )

                Then 'Time to Perform has Exceeded'
            Else
                H.ERRORREASONTEXT
            END AS ERRORREASONTEXT,
            H.FRGLBPARTNERCLASSCD,
            H.TOGLBPARTNERCLASSCD,
            Rtrim(dbms_lob.substr(D.MSGDIGEST, 4000,1)) MSGDIGEST,
            D.DIGITALSIGALGO,
            H.RNERRORCD,
            H.RAStatus,
            H.TOPROTOCOLNAME,
            H.TOPROTOCOLVER,
            H.TOPROCESSCD,
            H.TOPROCESSVER,
            Dest.DestName,    -- H.DestTypeId,
            H.DupProcessInstanceId,
            H.FRPACKAGENAME,
            H.FRPACKAGEVER,
            H.TOPACKAGENAME,
            H.TOPACKAGEVER,
            H.ChannelID,
            H.SYNCHRONOUSFLG,
            H.RESPONSEURL,
            H.REPLYTOMESSAGEID,
            H.SyncAckRequired,
            H.REQ_ACTIONINSTANCEID,
            H.REQ_MSGTRACKINGINSTANCEID,
            H.DocEncryptionCd,
            H.REQ_ENCRYPTALGO,
	 H.MSGSTANDARD
	, H.MSGSTANDARD_VER
	, H.PIPPAYLOADBINDINGID
	, H.SENDERLOCATIONID
	, H.ACTIONTIME ACKEXPIRYTIME
	, H.RECEIVERLOCATIONID,
           -----------------------
            D.TRANSACTIONINSTANCEID,
            D.ACTIONINSTANCEID,
            D.SIGNALINSTANCEID,
            D.MSGTRACKINGINSTANCEID,
            D.GLBUSAGECD,
            D.ACTIONVER,
            D.SIGNALVER,
            D.GLBBUSACTIONCD,
            D.GLBBUSSIGNALCD,
            D.FRBUSINESSID,
            D.TOBUSINESSID,
            D.ACTIVITYNAME,
            D.MSGDATETIME,
            nvl(D.FRSUPPLYCHAINCD, H.GLOBALSUPPLYCHAINCDTPA) FRSUPPLYCHAINCD,
            nvl(D.TOSUPPLYCHAINCD, H.GLOBALSUPPLYCHAINCDTPA) TOSUPPLYCHAINCD,
            D.DOCID,
            D.DOCTS,
            D.CUSTOMMSGID01,
            H.CUSTOMMSGID02,                       -- from Hdr
            frpT.ownerFlg FrPARTNEROwnerFlg,
            TopT.ownerFlg ToPARTNEROwnerFlg,
            frp.PARTNERID FRPARTNERID,
            Top.PARTNERID TOPARTNERID
		,COALESCE(frBRt.RN0A1FLG, 0) frRN0A1Flg
		,COALESCE(toBRt.RN0A1FLG, 0) toRN0A1Flg
		,COALESCE(frBRt.PartnerEventFlg, 0) frPartnerEventFlg
		,COALESCE(toBRt.PartnerEventFlg, 0) toPartnerEventFlg
        from
            BP_RNStateHdr H
            --Modified for JR25359
            Left Outer JOIN BP_RNSTATEDTL D on D.RNSTATEDTLID = Case When H.UpdateStatusCd = 2 and h.nextActionCD = 'RESP' and h.RAStatus is null
                Then h.OrigRNStateDtlId
              Else h.RNSTATEDTLID END
             --Ended modification for JR25359
            Left Outer Join pr_Busidentifier frBus on D.frBusinessid    = frBus.Businessid
            Left Outer Join pr_BusIdRouting  frBRt on frBRt.BusIdentifierId   = frBus.BusIdentifierId
            Left Outer Join pr_partner       frp   on frBus.partnerid   = frp.partnerid
            Left Outer Join pr_partnerType   frpT  on frp.PARTNERTYPEID = frpT.PARTNERTYPEID
            Left Outer Join pr_Busidentifier ToBus on D.ToBusinessid    = ToBus.Businessid
            Left Outer Join pr_BusIdRouting  toBRt on toBRt.BusIdentifierId   = toBus.BusIdentifierId
            Left Outer Join pr_partner       Top   on ToBus.partnerid   = Top.partnerid
            Left Outer Join pr_partnerType   TopT  on Top.PARTNERTYPEID = TopT.PARTNERTYPEID
            Left Outer Join pr_DestType      Dest  on Dest.DestTypeID   = H.DestTypeID
        Where H.rowid = v_RowID;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_Ah1 (v_ProcName, v_BeginTS, v_ParamsTxt, 0);
    InsertLog;
END;
/


-- BP_RNSTATEHDRS_AH3.sql

CREATE OR REPLACE PROCEDURE         BP_RNSTATEHDRS_AH3
    (
    p_ReturnCd          OUT     PLS_INTEGER,
    p_DebugFlg          IN      PLS_INTEGER,
    RC1                 OUT     DB_REF.rct,
    P_InitBusinessId    IN      Varchar2,
    p_ProcessInstanceId IN      VARCHAR2
    )
AS
/*
================================================================================
Proc Name:        BP_RNSTATEHDRS_AH3
Create Date:    2003-07-12 12:04:11
Author:          V.Goel
================================================================================
    Return code  = 0  Good, 1 = Something went wrong, 2 = Dup, 3 = No data found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_RNSTATEHDRS_AH3';
v_ExceptionFlg    integer Default 0;

v_ParamsTxt       varchar2(4000) :=
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_INITBUSINESSID=>'   ||p_INITBUSINESSID   ||CHR(10)
		||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10);

BEGIN
       p_ReturnCd := 0;


  OPEN RC1 for
    SELECT
    SYNCHRONOUSFLG
    , RESPONSEURL
    , REPLYTOMESSAGEID
    FROM BP_RNSTATEHdr H
    WHERE  H.InitBusinessId    = p_InitBusinessId
    AND    H.ProcessInstanceId = p_ProcessInstanceId;
   EXCEPTION
    WHEN NO_DATA_FOUND THEN
        p_ReturnCd := 3;
    WHEN OTHERS
    THEN
        p_ReturnCd := 1;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        raise_application_error (-20999, '20100 - BP_RNSTATEHDRS_AH2:' || SQLERRM);

--/+ INDEX (H AK_RNSTATEDTL_1) /

END BP_RNSTATEHDRS_AH3;
/


-- BP_RNSTATEHDRS_HDRID.sql

CREATE OR REPLACE PROCEDURE BP_RNSTATEHDRS_HDRID (
    p_ReturnCd                  OUT      INTEGER,
    p_DebugFlg                  IN       PLS_INTEGER,
    P_out_InitBusinessId        OUT      Varchar2,
    p_ProcessInstanceId         OUT      VARCHAR2,
    p_TRANSACTIONINSTANCEID     OUT      Varchar2,
    p_ACTIONINSTANCEID          OUT      Varchar2,
    p_ACTIONVER                 OUT      Varchar2,
    p_MSGTRACKINGINSTANCEID     OUT      Varchar2,
    p_GLBBUSACTIONCD            OUT      Varchar2,
    p_GLBUSAGECD                OUT      Varchar2,
    p_ACTIVITYNAME              OUT      Varchar2,
    p_SYNCHRONOUSFLG 			OUT		 pls_INTEGER,
    p_RESPONSEURL 				OUT		 VARCHAR2,
    p_REPLYTOMESSAGEID    		OUT		 VARCHAR2,
    p_DIGITALSIGALGO    		OUT		 VARCHAR2, 
    p_REQ_ENCRYPTALGO           OUT     Varchar2,   	
    p_InitBusinessId            IN       VARCHAR2,
    p_DocId                     IN       Varchar2
)
AS
/*
================================================================================
Proc Name:        BP_RNSTATEHDRS_HdrId
Create Date:    2002-04-23 12:04:11
Author:            SYSTEM
================================================================================
    Return code  = 0  Good
                   1  Something went wrong
                   2  No data found
                   3  Dup
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_rowcnt INTEGER := 0;
    v_BeginTS timestamp := FN_Get_Date;    -- Performance related variables
    v_InitBusinessId    BP_RNSTATEHDR.InitBusinessId%type;
    v_ProcessInstanceId BP_RNSTATEHDR.ProcessInstanceId%type;
    whereSql varchar2(1000);
    stmtSql  varchar2(2000);
    PROCEDURE ProcAudit
    IS
    BEGIN
            DB_ProcAuditLogI_Ah1 (
                'BP_RNSTATEHDRS_HdrId',
                 v_beginTS,
                'p_ReturnCd'      || ',' ||
                 p_DebugFlg       || ',' ||
                 p_InitBusinessId   || ',' ||
                 p_DocId
            );
    END ProcAudit;
    Procedure Process_too_many_rows is
        counter  pls_integer;
    Begin
        --
-- Check to make sure that there is only one header
--
    select count(*) INTO counter from
    (select /*+ INDEX (D AK_RNSTATEDTL_1) */ distinct H.ProcessInstanceId
     from BP_RNSTATEHdr H
     inner join BP_RNSTATEDTL D on H.InitBusinessId    = d.InitBusinessId
                              and H.ProcessInstanceId = d.ProcessInstanceId
            where d.docid = p_DocID
            and   H.INITBUSINESSID = p_InitBusinessId
    );
    If counter >1 then
       p_ReturnCd := 3;
    Else
        select /*+ INDEX (D AK_RNSTATEDTL_1) */ d.InitBusinessId,
           d.ProcessInstanceId,
           d.TRANSACTIONINSTANCEID,
           d.ACTIONINSTANCEID,
           d.ACTIONVER,
           d.MSGTRACKINGINSTANCEID,
           d.GLBBUSACTIONCD,
           d.GLBUSAGECD,
           d.ACTIVITYNAME,
		   H.SYNCHRONOUSFLG,
		   H.RESPONSEURL,
		   H.REPLYTOMESSAGEID,
           d.DIGITALSIGALGO,
           H.REQ_ENCRYPTALGO 
    INTO p_out_InitBusinessId,
	     p_ProcessInstanceId,
		 p_TRANSACTIONINSTANCEID,
         p_ACTIONINSTANCEID,
		 p_ACTIONVER,
		 p_MSGTRACKINGINSTANCEID,
		 p_GLBBUSACTIONCD,
         p_GLBUSAGECD,
		 p_ACTIVITYNAME,
		 p_SYNCHRONOUSFLG,
		 p_RESPONSEURL,
		 p_REPLYTOMESSAGEID,
         p_DIGITALSIGALGO,
         p_REQ_ENCRYPTALGO
    from BP_RNSTATEDTL d
    inner Join BP_RNSTATEHdr H on H.RNSTATEDTLID    = d.RNSTATEDTLID
    where D.INITBUSINESSID = p_InitBusinessId
    and   d.DocID = p_DocID
    and  d.RNSTATEDTLID = ( select /*+ INDEX (DTL AK_RNSTATEDTL_1) */ max(dtl.RNSTATEDTLID)
                                 from BP_RNSTATEDTL dtl
                                 where  dtl.INITBUSINESSID = p_InitBusinessId
                                 and    dtl.DocID          = p_DocID
                                 )
        ;
   end if;
   Exception
    when no_data_found then
        p_ReturnCd := 2;
    WHEN Too_many_rows then
        p_ReturnCd := 3;
    WHEN OTHERS
    THEN
        ProcAudit;
        dbms_output.put_line('Sqlcode '||SQLCODE);
        raise_application_error (-20999, '20100 - BP_RNSTATEHDRS_HdrId:Too many Rows Proc ' || SQLERRM);
        p_ReturnCd := 1;
    End Process_too_many_rows;
--------------------------------------------------------------------
--              M A I N
--------------------------------------------------------------------
BEGIN
       p_ReturnCd := 0;
    procAudit;
  Begin
    select /*+ INDEX (D AK_RNSTATEDTL_1) */
           d.InitBusinessId,
           d.ProcessInstanceId,
           d.TRANSACTIONINSTANCEID,
           d.ACTIONINSTANCEID,
           d.ACTIONVER,
           d.MSGTRACKINGINSTANCEID,
           d.GLBBUSACTIONCD,
           d.GLBUSAGECD,
           d.ACTIVITYNAME,
		   H.SYNCHRONOUSFLG,
		   H.RESPONSEURL,
		   H.REPLYTOMESSAGEID,
           d.DIGITALSIGALGO,
           H.REQ_ENCRYPTALGO 
    INTO p_out_InitBusinessId,
	     p_ProcessInstanceId,
		 p_TRANSACTIONINSTANCEID,
         p_ACTIONINSTANCEID,
		 p_ACTIONVER,
		 p_MSGTRACKINGINSTANCEID,
		 p_GLBBUSACTIONCD,
         p_GLBUSAGECD,
		 p_ACTIVITYNAME,
		 p_SYNCHRONOUSFLG,
		 p_RESPONSEURL,
		 p_REPLYTOMESSAGEID,
         p_DIGITALSIGALGO,
         p_REQ_ENCRYPTALGO 
    from BP_RNSTATEDTL d
        inner Join BP_RNSTATEHdr H on H.InitBusinessId    = d.InitBusinessId
                                  and H.ProcessInstanceId = d.ProcessInstanceId
--    inner Join BP_RNSTATEHdr H on H.RNSTATEDTLID    = d.RNSTATEDTLID
    where D.INITBUSINESSID = p_InitBusinessId
    and   d.DocID = p_DocID;
   Exception
    when no_data_found then
        p_ReturnCd := 2;
    When too_many_rows Then
        Process_too_many_rows;
    WHEN OTHERS
    THEN
        ProcAudit;
        raise_application_error (-20999, '20101 - BP_RNSTATEHDRS__HdrId:' || SQLERRM);
        p_ReturnCd := 1;
  End;
END BP_RNSTATEHDRS_HdrId;
/


-- BP_RNSTATEHDRS_HDRID2.sql

CREATE OR REPLACE PROCEDURE BP_RNSTATEHDRS_HDRID2 (
    p_ReturnCd                  OUT      INTEGER,
    p_DebugFlg                  IN       PLS_INTEGER,
    P_out_InitBusinessId        OUT      Varchar2,
    p_TRANSACTIONINSTANCEID     OUT      Varchar2,
    p_ACTIONINSTANCEID          OUT      Varchar2,
    p_ACTIONVER                 OUT      Varchar2,
    p_MSGTRACKINGINSTANCEID     OUT      Varchar2,
    p_GLBBUSACTIONCD            OUT      Varchar2,
    p_GLBUSAGECD                OUT      Varchar2,
    p_ACTIVITYNAME              OUT      Varchar2,
    p_SYNCHRONOUSFLG 		OUT	 pls_INTEGER,
    p_RESPONSEURL 		OUT	 VARCHAR2,
    p_REPLYTOMESSAGEID    	OUT	 VARCHAR2,
    p_DIGITALSIGALGO    	OUT	 VARCHAR2, 
    p_REQ_ENCRYPTALGO           OUT     Varchar2,   	
    p_InitBusinessId            IN       VARCHAR2,
    p_DocId_ProcInstId		IN       Varchar2
)
AS
/*
	================================================================================
	Proc Name:		BP_RNSTATEHDRS_HdrId
	Create Date:		2002-04-23 12:04:11
	Author:			S.Priani
                        ::::::::::::::::::::::::::::::
                        M O D I F I C A T I O N
                        ::::::::::::::::::::::::::::::
	---------- ------------	-------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-08-16 S.Priani	6.1	CR 102346

	================================================================================
	Return code  = 0:Good  1:Something went wrong  2:No data found  3:Dup
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_rowcnt INTEGER := 0;
    v_BeginTS timestamp := FN_Get_Date;    -- Performance related variables
    v_InitBusinessId    BP_RNSTATEHDR.InitBusinessId%type;
    v_ProcessInstanceId BP_RNSTATEHDR.ProcessInstanceId%type;
    whereSql varchar2(1000);
    stmtSql  varchar2(2000);
    PROCEDURE ProcAudit
    IS
    BEGIN
            DB_ProcAuditLogI_Ah1 (
                'BP_RNSTATEHDRS_HdrId',
                 v_beginTS,
                'p_ReturnCd'      || ',' ||
                 p_DebugFlg       || ',' ||
                 p_InitBusinessId   || ',' ||
                 p_DocId_ProcInstId
            );
    END ProcAudit;
    Procedure Process_too_many_rows is
        counter  pls_integer;
    Begin
        --
-- Check to make sure that there is only one header
--
    select count(*) INTO counter
     from BP_RNSTATEHdr H
            where H.ProcessInstanceId = p_DocID_ProcInstId
            and   H.INITBUSINESSID = p_InitBusinessId
		and MsgSentStatusCd IN ( 'RESPSENT');
    If counter >0 then
       p_ReturnCd := 3;
    Else
        select /*+ INDEX (D AK_RNSTATEDTL_1) */ d.InitBusinessId,
           d.TRANSACTIONINSTANCEID,
           d.ACTIONINSTANCEID,
           d.ACTIONVER,
           d.MSGTRACKINGINSTANCEID,
           d.GLBBUSACTIONCD,
           d.GLBUSAGECD,
           d.ACTIVITYNAME,
		   H.SYNCHRONOUSFLG,
		   H.RESPONSEURL,
		   H.REPLYTOMESSAGEID,
           d.DIGITALSIGALGO,
           H.REQ_ENCRYPTALGO 
    INTO p_out_InitBusinessId,
		 p_TRANSACTIONINSTANCEID,
         p_ACTIONINSTANCEID,
		 p_ACTIONVER,
		 p_MSGTRACKINGINSTANCEID,
		 p_GLBBUSACTIONCD,
         p_GLBUSAGECD,
		 p_ACTIVITYNAME,
		 p_SYNCHRONOUSFLG,
		 p_RESPONSEURL,
		 p_REPLYTOMESSAGEID,
         p_DIGITALSIGALGO,
         p_REQ_ENCRYPTALGO
    from BP_RNSTATEDTL d
    inner Join BP_RNSTATEHdr H on H.RNSTATEDTLID    = d.RNSTATEDTLID
    where D.INITBUSINESSID = p_InitBusinessId
      and d.ProcessInstanceId = p_DocID_ProcInstID
      and d.RNSTATEDTLID = ( select
                                            max(dtl.RNSTATEDTLID)
                                       from BP_RNSTATEDTL dtl
                                      where  dtl.INITBUSINESSID = p_InitBusinessId
              				and dtl.ProcessInstanceId = p_DocID_ProcInstID
					and dtl.docid is not null
                                        )
    ;
   end if;
   Exception
    when no_data_found then
        p_ReturnCd := 2;
    WHEN Too_many_rows then
        p_ReturnCd := 3;
    WHEN OTHERS
    THEN
        ProcAudit;
        dbms_output.put_line('Sqlcode '||SQLCODE);
        raise_application_error (-20999, '20100 - BP_RNSTATEHDRS_HdrId:Too many Rows Proc ' || SQLERRM);
        p_ReturnCd := 1;
    End Process_too_many_rows;
--------------------------------------------------------------------
--              M A I N
--------------------------------------------------------------------
BEGIN
       p_ReturnCd := 0;
    procAudit;
  Begin
    select /*+ INDEX (D AK_RNSTATEDTL_1) */
           d.InitBusinessId,
           d.TRANSACTIONINSTANCEID,
           d.ACTIONINSTANCEID,
           d.ACTIONVER,
           d.MSGTRACKINGINSTANCEID,
           d.GLBBUSACTIONCD,
           d.GLBUSAGECD,
           d.ACTIVITYNAME,
		   H.SYNCHRONOUSFLG,
		   H.RESPONSEURL,
		   H.REPLYTOMESSAGEID,
           d.DIGITALSIGALGO,
           H.REQ_ENCRYPTALGO 
    INTO p_out_InitBusinessId,
		 p_TRANSACTIONINSTANCEID,
         p_ACTIONINSTANCEID,
		 p_ACTIONVER,
		 p_MSGTRACKINGINSTANCEID,
		 p_GLBBUSACTIONCD,
         p_GLBUSAGECD,
		 p_ACTIVITYNAME,
		 p_SYNCHRONOUSFLG,
		 p_RESPONSEURL,
		 p_REPLYTOMESSAGEID,
         p_DIGITALSIGALGO,
         p_REQ_ENCRYPTALGO 
    from BP_RNSTATEDTL d
        inner Join BP_RNSTATEHdr H on H.InitBusinessId    = d.InitBusinessId
                                  and H.ProcessInstanceId = d.ProcessInstanceId
    where D.INITBUSINESSID = p_InitBusinessId
       and d.ProcessInstanceID = p_docId_ProcInstId
;
   Exception
    when no_data_found then
        p_ReturnCd := 2;
    When too_many_rows Then
        Process_too_many_rows;
    WHEN OTHERS
    THEN
        ProcAudit;
        raise_application_error (-20999, '20101 - BP_RNSTATEHDRS__HdrId:' || SQLERRM);
        p_ReturnCd := 1;
  End;
END BP_RNSTATEHDRS_HdrId2;
/


-- BP_RNSTATEHDRS_PID.SQL

CREATE OR REPLACE PROCEDURE         BP_RNSTATEHDRS_PID (
    p_ReturnCd                  OUT      INTEGER,
    p_DebugFlg                  IN       PLS_INTEGER,
    p_checkflg                  out      integer,  -- 1 = found, 0 = not found
    p_InitBusinessId            IN       varchar2,
    p_ProcessInstanceId         IN       VARCHAR2

)
AS
/*
================================================================================
Proc Name:        BP_RNSTATEHDRS_PID
Create Date:    2002-12-11
Author:            J.Chen
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_rowcnt INTEGER := 0;
    v_BeginTS timestamp := FN_Get_Date;    -- Performance related variables
    v_checkflg integer;

PROCEDURE ProcAudit
IS
BEGIN
            DB_ProcAuditLogI_Ah1 (
                'BP_RNSTATEHDRS_PID',
                 v_beginTS,
                'p_ReturnCd'      || ',' ||
                 p_DebugFlg       || ',' ||
                 p_initbusinessid || ',' ||
                 p_ProcessInstanceId
            );
END ProcAudit;


-----------------------------------------------------------------------------
--                                     M A I N
-----------------------------------------------------------------------------
BEGIN
       p_ReturnCd := 0;
    procAudit;
  Begin
    select 1 INTO v_checkflg
    from BP_RNSTATEHDR
    where INITBUSINESSID = p_InitBusinessId
    and   PROCESSINSTANCEID = p_PROCESSINSTANCEID
    and   rownum < 2;
   Exception
   When no_data_found Then
        null;
    WHEN OTHERS
    THEN
        ProcAudit;
        raise_application_error (-20999, '20100 - BP_RNSTATEHDRS_PID:' || SQLERRM);
        p_ReturnCd := 1;
  End;

  If v_checkflg = 1 Then
     p_checkflg := v_checkflg;
  else
     p_checkflg := 0;
  end if;


END BP_RNSTATEHDRS_PID;
/


-- BP_RNSTATEHDRS_PROCINSTANCEID.sql

CREATE OR REPLACE PROCEDURE         BP_RNSTATEHDRS_PROCINSTANCEID (
    p_ReturnCd                  OUT        INTEGER,
    p_DebugFlg                  IN         PLS_INTEGER,
--    RC1                            OUT         db_ref.RCT,
    P_InitBusinessId            OUT        Varchar2,
    p_out_ProcessInstanceId     Out        VARCHAR2,
    p_rowTS                     OUT        PLS_INTEGER,
    p_FrBusinessId              IN         VARCHAR2,
    p_ToBusinessId              IN         VARCHAR2,
    p_ProcessInstanceId         IN         VARCHAR2,
    p_TransactionInstanceId     IN         VARCHAR2,         --(RNIF 1.1 only)
    p_ActionInstanceId          IN         VARCHAR2,
    p_MSGTRACKINGINSTANCEID     IN         VARCHAR2          --(RNIF 2.0 only)

)
AS
/*
================================================================================
Proc Name:        BP_RNSTATEHDRS_PROCINSTANCEID
Create Date:    2002-04-23 12:04:11
Author:            SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_rowcnt INTEGER := 0;
    v_BeginTS timestamp := FN_Get_Date;    -- Performance related variables
    v_InitBusinessId    BP_RNSTATEHDR.InitBusinessId%type;
    v_ProcessInstanceId BP_RNSTATEHDR.ProcessInstanceId%type;
    whereSql varchar2(1000);
    stmtSql  varchar2(2000);
    PROCEDURE ProcAudit
    IS
    BEGIN
        DB_ProcAuditLogI_Ah1 (
            'BP_RNSTATEHDRS_PROCINSTANCEID',
             v_beginTS,
            'p_ReturnCd'      || ',' ||
             p_DebugFlg       || ',' ||
             'RC1'            || ',' ||
             p_FrBusinessId   || ',' ||
             p_ToBusinessId   || ',' ||
             p_ProcessInstanceId           || ',' ||
             p_TransactionInstanceId       || ',' ||
             p_ActionInstanceId            || ',' ||
             p_MSGTRACKINGINSTANCEID
        );
    END ProcAudit;
/*
    Procedure build_whereClause(p_column in varchar2, p_dataType in varchar2, p_param in varchar2, p_where in out varchar2) is
    begin
     if p_param is not null then
       if p_dataType = 'NUMBER_LIST' then
         p_where := p_where||' and '||p_column||' in '||p_param;
       end if;
       if p_dataType = 'NUMBER' then
         p_where := p_where||' and '||p_column||' ='||to_char(p_param) ;
       end if;
       if p_dataType = 'VARCHAR2' then
         p_where := p_where||' and '||p_column||' ='''||p_param||'''' ;
       end if;
       if p_dataType = 'VARCHAR2_LIKE' then
         p_where := p_where||' and '||p_column||' LIKE '''||p_param||'''' ;
       end if;
     end if;
end build_whereClause;
*/
    PROCEDURE DrillDown_More is
    BEGIN

    WhereSql :=
      FN_Build_Where('d.TransactionInstanceId',    'VARCHAR2', p_TransactionInstanceId)
      || FN_Build_Where('d.ActionInstanceId',      'VARCHAR2', p_ActionInstanceId)
      || FN_Build_Where('d.MsgTrackingInstanceId', 'VARCHAR2', p_MsgTrackingInstanceId);

      StmtSql := 'select InitBusinessId, ProcessInstanceId, h.rowTS
                  from BP_RNSTATEHdr h
                  Inner Join BP_RNSTATEDtl d on h.INITBUSINESSID    = d.INITBUSINESSID
                                            AND h.PROCESSINSTANCEID = d.PROCESSINSTANCEID;
                  where (h.INITBUSINESSID = p_FrBusinessId or h.INITBUSINESSID = p_ToBusinessId)
                  and   h.PROCESSINSTANCEID = p_PROCESSINSTANCEID '||WhereSql;

      Execute Immediate StmtSql INTO p_InitBusinessId, p_out_ProcessInstanceId, p_rowTS;
     EXCEPTION
        When no_data_found Then
            p_ReturnCd := 2;
        WHEN OTHERS    THEN
            ProcAudit;
            raise_application_error (-20999, '20100 - BP_RNSTATEHDRS_PROCINSTANCEID: DrillDown_More' || SQLERRM);
            ROLLBACK WORK;
            p_ReturnCd := 1;
    END DrillDown_More;
-----------------------------------------------------------------------------
--                                     M A I N
-----------------------------------------------------------------------------
BEGIN
       p_ReturnCd := 0;
    procAudit;
  Begin
    select InitBusinessId, ProcessInstanceId, rowTS INTO p_InitBusinessId, p_out_ProcessInstanceId, p_rowTS
    from BP_RNSTATEHDR h
    where (INITBUSINESSID = p_FrBusinessId or INITBUSINESSID = p_ToBusinessId)
    and   PROCESSINSTANCEID = p_PROCESSINSTANCEID;
   Exception
   When no_data_found Then
        p_ReturnCd := 2;
    When too_many_rows Then   DrillDown_More;
    WHEN OTHERS
    THEN
        ProcAudit;
        raise_application_error (-20999, '20100 - BP_RNSTATEHDRS_PROCINSTANCEID:' || SQLERRM);
        p_ReturnCd := 1;
  End;

END BP_RNSTATEHDRS_PROCINSTANCEID;
/


-- BP_RNSTATEHDRU_AH1.sql

CREATE OR REPLACE PROCEDURE BP_RNSTATEHDRU_AH1
(
    p_ReturnCd               OUT     PLS_INTEGER,
    p_DebugFlg               IN      PLS_INTEGER,
    p_INITBUSINESSID         IN      VARCHAR2,
    p_PROCESSINSTANCEID      IN      VARCHAR2,
    p_NEXTACTIONCD           IN      VARCHAR2,
    p_MSGSENTSTATUSCD        IN      VARCHAR2,
    p_CURRENTPIPSTATUSCD     IN      VARCHAR2,
    p_RNSTATEDTLID           IN      NUMBER,
    p_ERRORREASONTEXT        IN      VARCHAR2,
    p_FRSUPPLYCHAINCD        IN      VARCHAR2,
    p_TOSUPPLYCHAINCD        IN      VARCHAR2,
    p_DOCID                  IN      VARCHAR2,
    p_DOCTS                  IN      VARCHAR2,
    p_RnErrorCD              IN      VARCHAR2,
    p_outboundVuid           IN      VARCHAR2,
    p_SponsorFlg             IN      PLS_INTEGER, -- 0 = Update  1= Do Not Update
    p_UpdatestatusCD         IN      PLS_INTEGER,  -- update if not null, else ignore
    p_CUSTOMMSGID02          IN      Varchar2,      -- -- Event Message
    p_HdrUpdateFlg           IN      PLS_INTEGER,  -- null =do not update rnstatedtlid in hdr, not null = do it
    p_REFINITBUSINESSID      IN      VARCHAR2,
    p_REFPROCESSINSTANCEID   IN      VARCHAR2,
    p_inboundVuid            IN      VARCHAR2,
    p_reStartFlg             IN      PLS_INTEGER,   -- 0 = ignore the flag, 1 = it is a reStart
    p_RAStatus               IN      VARCHAR2
)
AS
/*
================================================================================
Proc Name:        BP_RNSTATEHDRU_AH1
Create Date:    2002-04-23 12:04:11
Author:            SYSTEM
2003-06-30 V Goel        3.5      Comment out "write_auditLog".Changing ExpireTPATime to TIMESTAMP has caused problem
                                  Look for  "*********************** Temporary ***************************"
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_rowcnt         INTEGER := 0;
    v_BeginTS        timestamp := FN_GET_DATE();    -- Performance related variables
    v_rowTS          INTEGER := 0;
    v_ProcName       varchar2(30) := 'BP_RNSTATEHDRU_AH1';
    zeroUpdate       EXCEPTION;
    v_RNStateDTLId   bp_rnstatedtl.RNSTATEDTLID%type;
    v2_rowTS         pls_integer;
    v_ExpireTPAtime  bp_rnStateHdr.ExpireTPAtime%type;
    v_UpdateFlg     Boolean:= TRUE;
    v_Msg           varchar2(1024);
    PROCEDURE ProcAudit
    IS
    BEGIN
            DB_ProcAuditLogI_Ah1 (
                v_ProcName,
                v_beginTS,
                'p_ReturnCd'         ||',' ||
                p_DebugFlg           ||',' ||
                p_INITBUSINESSID     ||',' ||
                p_PROCESSINSTANCEID  ||',' ||
                p_NEXTACTIONCD       ||',' ||
                p_MSGSENTSTATUSCD    ||',' ||
                p_CURRENTPIPSTATUSCD ||',' ||
                p_RNSTATEDTLID       ||',' ||
                substr(p_ERRORREASONTEXT,1,250)    ||',' ||
                p_FRSUPPLYCHAINCD    ||',' ||
                p_TOSUPPLYCHAINCD    ||',' ||
                p_DOCID              ||',' ||
                p_DOCTS              ||',' ||
                p_RnErrorCD          ||',' ||
                p_OutBoundvuid       ||',' ||
                p_SponsorFlg         ||',' ||
                p_UpdatestatusCD     ||',' ||
                p_CUSTOMMSGID02      ||',' ||
                p_HdrUpdateFlg       ||',' ||
                p_REFINITBUSINESSID  ||',' ||
                p_REFPROCESSINSTANCEID ||',' ||
                p_inboundVuid        ||',' ||
                p_reStartFlg
            );

    END ProcAudit;

    PROCEDURE write_auditLog is
     r_code             pls_integer;
     d_flag             pls_integer := 0;
     v_Date             Date        ;
    BEGIN
        Begin
            v_date := v_EXPIRETPATIME;
            v_Msg := 'Update...ExpireTPA '||
                     to_char(v_Date , 'dd_mon-yy hh24:mi:ss');
        Exception
            When Others Then Null;
        End;

        BP_RNSTATEHDRAUDITLOGI (
        p_ReturnCd => r_code
        ,p_DebugFlg => d_flag
        ,p_INITBUSINESSID     => p_INITBUSINESSID
        ,p_PROCESSINSTANCEID  => p_PROCESSINSTANCEID
        ,p_RNSTATEDTLID       => v_RNSTATEDTLID
        ,p_NEXTACTIONCD       => p_NEXTACTIONCD
        ,p_MSGSENTSTATUSCD    => p_MSGSENTSTATUSCD
        ,p_CURRENTPIPSTATUSCD => p_CURRENTPIPSTATUSCD
        ,p_ERRORREASONTEXT    => rtrim(v_Msg)
        );
        EXCEPTION
            WHEN OTHERS THEN NULL;
    END write_auditLog;
--

    Procedure Update_sponsor_state is
        R_code pls_integer := 0;
        DataFound Boolean := TRUE;
        v_refInboundVuid   bp_sponsor_state.refInboundVuid%Type;
    BEGIN
      Begin -- select
        Select rowts into v2_rowts from  bp_sponsor_state
        Where  INITBUSINESSID    =  P_INITBUSINESSID
        And    PROCESSINSTANCEID =  P_PROCESSINSTANCEID ;
      Exception
        When no_data_found then
            DataFound := FALSE;
        When Others then
         raise_application_error (-20999, '20110 - '||v_ProcName||':  Update_sponsor_state -- No Data Found');
      End; -- select
      IF DataFound then
            BP_SPONSOR_STATEU_AH2(
                  P_RETURNCD          => R_code
                , P_DEBUGFLG          =>  0
                , P_INITBUSINESSID    => p_INITBUSINESSID
                , P_PROCESSINSTANCEID => p_PROCESSINSTANCEID
                , P_NEXTACTIONCD      => p_NEXTACTIONCD
                , P_DOCID             => p_DOCID
                , p_refInboundVuid    => v_refInboundVuid
                , P_ROWTS             => v2_RowTS
                );
        END IF;
        If r_code  > 0 then
            raise_application_error (-20999, '20111 - '||v_ProcName||':  Update_sponsor_state -- Update Failed');
        End if;
     Exception
     WHEN OTHERS  THEN
        ProcAudit;
        raise_application_error (-20999, '20112- '||v_ProcName||':  Update_sponsor_state ' || SQLERRM);
        ROLLBACK WORK;
        p_ReturnCd := 1;
    END Update_sponsor_state;
--
    PROCEDURE Update_RnHeader is
    Begin
    if p_RNSTATEDTLID = 0 then -- java can not send null for  integer
        v_RNSTATEDTLID := null;
    else
        v_RNSTATEDTLID := p_RNSTATEDTLID;
    end if;
    UPDATE BP_RNSTATEHDR
        SET
             NEXTACTIONCD       = nvl(p_NEXTACTIONCD, NEXTACTIONCD),
             MSGSENTSTATUSCD    = nvl(p_MSGSENTSTATUSCD, MSGSENTSTATUSCD),
             CURRENTPIPSTATUSCD = nvl(p_CURRENTPIPSTATUSCD, CURRENTPIPSTATUSCD),
             RNSTATEDTLID       = nvl2(p_HdrUpdateFlg, nvl(v_RNSTATEDTLID, RNSTATEDTLID), RNSTATEDTLID),
             ERRORREASONTEXT    = nvl(p_ERRORREASONTEXT, ERRORREASONTEXT),
             RnErrorCD          = nvl(p_RnErrorCD, RnErrorCD),
             UpdatestatusCD     = nvl(p_UpdatestatusCD, UpdatestatusCD),
             ExpireTPATime      = Case When
                                            NEXTACTIONCD   in   ('REQ',     'RESP')
                                       and p_MSGSENTSTATUSCD in ('REQSENT', 'RESPSENT', 'REQFAILED', 'RESPFAILED')
                                       then
                                            (v_BeginTS + (TIMETOACKTPA/1440))
                                       When p_NEXTACTIONCD = '0A1'
                                       then
                                            v_BeginTS
                                  Else
                                    ExpireTPATime End,
             ACTIONTIME		= Case When COALESCE(p_NEXTACTIONCD, 'x')  IN  ('RA', 'GE', 'RAE') then
	     					v_BeginTS + NUMTODSINTERVAL(COALESCE(TIMETOACKTPA,5), 'MINUTE')
	     				Else	ACTIONTIME End,
	     				
             REFINITBUSINESSID   = nvl(p_REFINITBUSINESSID, REFINITBUSINESSID),
             REFPROCESSINSTANCEID= nvl(p_REFPROCESSINSTANCEID, REFPROCESSINSTANCEID),
             RAStatus            = nvl(p_RAStatus, RAStatus),
             CUSTOMMSGID02      = nvl(p_CUSTOMMSGID02, CUSTOMMSGID02),
             ROWTS              = (ROWTS + 1),
             UPDATEDATE         = v_BeginTS,
            --Added for JR25359
             ORIGRNSTATEDTLID   = CASE WHEN COALESCE(p_NEXTACTIONCD, 'x') = 'RA' AND v_RNSTATEDTLID <> 0 AND ORIGRNSTATEDTLID IS NULL
					THEN v_RNSTATEDTLID 
					ELSE ORIGRNSTATEDTLID END
	    --Ended addition for JR25359
     WHERE INITBUSINESSID     = p_INITBUSINESSID
        AND PROCESSINSTANCEID = p_PROCESSINSTANCEID
--        AND (CURRENTPIPSTATUSCD is NULL OR CURRENTPIPSTATUSCD NOT IN ('0A1'))
        AND (p_NEXTACTIONCD != '0A1' OR CURRENTPIPSTATUSCD != '0A1' OR NEXTACTIONCD != '0A1STARTED')
--
    RETURNING ROWTS, ExpireTPATime INTO v_rowTS, v_ExpireTPAtime;
    v_rowcnt := SQL%ROWCOUNT;
      write_auditLog;
--    IF (v_rowcnt = 0)
--    THEN
--        RAISE ZeroUpdate;
--    END IF;
EXCEPTION
    WHEN TOO_MANY_ROWS
    THEN
        p_ReturnCd := 1;
        ProcAudit;
        raise_application_error (-20999, '20113 - '||v_ProcName||': Too Many Rows');
    WHEN ZeroUpdate
    THEN
        p_ReturnCd := 3;
        ProcAudit;
        raise_application_error (-20999, '20114 '||TO_CHAR(P_RNSTATEDTLID)||'- '||v_ProcName||': No Data Found');
    WHEN OTHERS
    THEN
        ProcAudit;
        p_ReturnCd := 1;
        raise_application_error (-20999, '20115 - '||v_ProcName||': UpdateHdr '||
        ' ErrorReasonText Length '||length(p_ERRORREASONTEXT) || SQLERRM);
    End Update_RnHeader;
---
    Procedure Update_RnDtl is
    Begin
            UPDATE BP_RNSTATEDTL SET
                 FRSUPPLYCHAINCD = nvl(p_FRSUPPLYCHAINCD, FRSUPPLYCHAINCD),
                 TOSUPPLYCHAINCD = nvl(p_TOSUPPLYCHAINCD, TOSUPPLYCHAINCD),
                 DOCID           = nvl(p_DOCID, DOCID),
                 DOCTS           = nvl(p_DOCTS, DOCTS),
                 OutBoundVuid    = nvl(p_OutBoundVuid, OutBoundVuid),
                 CUSTOMMSGID02   = nvl(p_CUSTOMMSGID02, CUSTOMMSGID02),
                 UPDATEDATE      = v_BeginTS
            WHERE INITBUSINESSID     = p_INITBUSINESSID
            AND PROCESSINSTANCEID    = p_PROCESSINSTANCEID
            AND RNSTATEDTLID         = p_RNSTATEDTLID;
             v_rowcnt := SQL%ROWCOUNT;
            /* IF (v_rowcnt = 0) THEN
                RAISE ZeroUpdate;
            END IF;
			*/
EXCEPTION
    WHEN TOO_MANY_ROWS
    THEN
        p_ReturnCd := 1;
        ProcAudit;
        raise_application_error (-20999, '20116 - '||v_ProcName||':  Update DTL: Too Many Rows');
    WHEN ZeroUpdate
    THEN
        p_ReturnCd := 3;
        ProcAudit;
        raise_application_error (-20999, '20117 '||TO_CHAR(P_RNSTATEDTLID)
                                                 ||'- '||v_ProcName||':  Update DTL: No Data Found');
    WHEN OTHERS
    THEN
        ProcAudit;
        raise_application_error (-20999, '20118 '||TO_CHAR(P_RNSTATEDTLID)
                                                 ||'- '||v_ProcName||':  Update DTL:' || SQLERRM);
        p_ReturnCd := 1;
    End Update_RnDtl;
----
    PROCEDURE Set_update_flag_vuid is
        v_createdate    BP_RnStateDTL.createdate%tYPE;
        v_updatedate    BP_RnStateDTL.updatedate%tYPE;
    Begin
        Select createdate, updatedate into v_createdate, v_updatedate
        From BP_RnStateDTL
        where inboundVUID = p_INBOUNDVUID;
        if v_createDate <> v_updateDate then
            v_UpdateFlg := FALSE;
        end if;
    EXCEPTION
    WHEN NO_DATA_FOUND
    THEN
        v_UpdateFlg := FALSE;
    WHEN TOO_MANY_ROWS
    THEN
        v_UpdateFlg := FALSE;
    End Set_update_flag_vuid;
--
    PROCEDURE Set_update_flag_pip is
        Counter pls_integer := 0;
    Begin
        select 1 into counter
        From bp_rnStateHdr
        Where  INITBUSINESSID    =  P_INITBUSINESSID
        And    PROCESSINSTANCEID =  P_PROCESSINSTANCEID
        And    CURRENTPIPSTATUSCD = '0A1';
        If counter > 0 Then
            v_UpdateFlg := FALSE;
        End if;
        Exception
        WHEN NO_DATA_FOUND THEN
            v_UpdateFlg := FALSE;
    End Set_update_flag_pip;
---------------------------------------------------------------------------
--                       M  A  I  N
---------------------------------------------------------------------------

BEGIN
    v_rowcnt := 0;
--
-- Re-start Flag Processing
--
    If p_reStartFlg = 1 Then
        IF p_CURRENTPIPSTATUSCD = '0A1' Then
            Set_update_flag_pip;
        Elsif p_INBOUNDVUID is null then
            v_UpdateFlg := FALSE;
        Else
            Set_update_flag_vuid;
        End if;
    End if;
--
-- Update Rnstate header, Detail and Sponsor State records
--
    If ( v_UpdateFlg ) Then
        Update_RnHeader;        --  UPDATE BP_RNSTATE HEADER
	IF v_RowCnt = 0 THEN RAISE ZeroUpdate; END IF;
        IF P_RNSTATEDTLID > 0 THEN
            Update_RnDtl;         --  UPDATE BP_RNSTATEDTL
		IF v_RowCnt = 0 THEN RAISE ZeroUpdate; END IF;
        END IF;
        IF p_SponsorFlg =0 AND v_RowCnt > 0 THEN
            Update_sponsor_state;  -- Update Sponsor State table
        END IF;
    End If ;
--
    p_ReturnCd := 0;
    ProcAudit;

EXCEPTION 
	WHEN ZeroUpdate THEN
		p_ReturnCd := 3;
END;
/


-- BP_RNSTATEHDRU_CALC.sql

CREATE OR REPLACE PROCEDURE         BP_RNSTATEHDRU_calc
(
    p_ReturnCd          out integer
    , p_DebugFlg         in  pls_integer
    --, RC1                out db_ref.RCT
    ,p_INITBUSINESSID    in  VARCHAR2
    ,p_PROCESSINSTANCEID in  VARCHAR2
) as

/*
================================================================================
Proc Name:              BP_RNSTATEHDRU
Create Date:    2002-11-11 11:21:51
Author:                 V Goel

================================================================================
Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer         :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_rowTS integer :=0;
zeroUpdate  Exception;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('BP_RNSTATEHDRU',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_INITBUSINESSID
    ||','||p_PROCESSINSTANCEID
    );
End ProcAudit;

BEGIN
    v_rowcnt := 0;
    Update BP_RNSTATEHDR SET
         ATTEMPTCNTCALC= ATTEMPTCNTCALC + 1
     ,   rowTS         = RowTS + 1
     ,   UPDATEDATE    = sysdate
    Where INITBUSINESSID  = p_INITBUSINESSID
    and PROCESSINSTANCEID = p_PROCESSINSTANCEID
    ;
    v_rowcnt := SQL%rowcount;
    if (v_rowcnt = 0) then
        raise ZeroUpdate ;
    end if;
    p_ReturnCd := 0;
    ProcAudit;
EXCEPTION
    WHEN TOO_MANY_ROWS then ProcAudit;
        raise_application_error(-20999, '20100 - BP_RNSTATEHDRU: Too Many Rows');
    WHEN ZeroUpdate then ProcAudit;
        raise_application_error(-20999,'20100 - BP_RNSTATEHDRU: No Data Found');
    WHEN OTHERS then
        ProcAudit;
        p_ReturnCd := 1;
        raise_application_error(-20999,'20100 - BP_RNSTATEHDRU:'||sqlerrm);

End BP_RNSTATEHDRU_calc;
/


-- BP_RNSTATEHDR_QUEI_AH1.sql

CREATE OR REPLACE PROCEDURE BP_RNStateHdr_QueI_AH1(
    p_ReturnCd               OUT     INTEGER,
    p_RowCnt                 OUT     INTEGER
)
AS
/*
    ================================================================================
    Proc Name:        BP_RNStateHdr_QueI_AH1
    Create Date:    2002-04-23 12:04:11
    Author:            B.Moffat
    ================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
--
-- Cursor
--
--  Cursor c_hdr is select rowid from BP_RNStateHdr01_VW order by UpdateDate;
------------------
    v_BeginTS               timestamp := FN_Get_Date();    -- Performance related variables
    v_rowid                 ROWID;
    v_count                 pls_integer;
    v_UpdateSuccessfulFlg   boolean := False;
    v_ProcName              varchar2(30) := 'BP_RNSTATEHDR_QUEI_AH1';
    v_NewDocFlg             integer := 0;
    v_BatchRows             integer := 0;

    p_RowCnt2                 integer := 0;
    j                       integer := 0;
    v_Msg                   varchar2(1024);
    v_RETURNCD              pls_integer;
---
    v_TIMETOACKTPA          pls_integer;
    v_INITBUSINESSID        BP_RNStateHdr.INITBUSINESSID%Type := null;
    v_PROCESSINSTANCEID     BP_RNStateHdr.PROCESSINSTANCEID%Type;
    v_RNSTATEDTLID          BP_RNStateHdr.RNSTATEDTLID%Type;
    v_NEXTACTIONCD          BP_RNStateHdr.NEXTACTIONCD%Type;
    v_MSGSENTSTATUSCD       BP_RNStateHdr.MSGSENTSTATUSCD%Type;
    v_CURRENTPIPSTATUSCD    BP_RNStateHdr.CURRENTPIPSTATUSCD%Type;
    v_EXPIRETPATIME         BP_RNStateHdr.EXPIRETPATIME%Type;
    v_RowTS                 BP_RNStateHdr.RowTS%Type;
    v_UpdateStatusCD        BP_RNStateHdr.UpdateStatusCD%Type;
    v_ParamsTxt             varchar2(1024) := 'p_ReturnCd,';
    Type t_RowID            is table of ROWID;
    Type t_UpdateDate       is table of TIMESTAMP;
    array_RowID             t_RowID  := t_RowID();
    array_UpdateDate        t_UpdateDate  := t_UpdateDate();


---
PROCEDURE InsertQue is
Begin

    BEGIN
        select
            Row_ID, UpdateDate
            BULK COLLECT INTO array_RowID, array_UpdateDate
        from
            (
            select /*+ FIRST_ROWS INDEX(H2 AK_RNSTA) */
                rowid Row_ID, UpdateDate
            from
                BP_RNStateHdr H2
            WHERE
                UpdateStatusCd = 1 and                                      -- Non TPA
                  nextActionCD = 'RA' and MSGSENTSTATUSCD in ('REQSENT', 'RESPSENT', 'REQACKFAILED', 'RESPACKFAILED')
                  AND ((SYNCHRONOUSFLG = 0) OR  (SYNCHRONOUSFLG = 1 And  SINGLEACTIONFLG = 1))

            UNION
            select /*+ FIRST_ROWS INDEX(H2 AK_RNSTA) */
                rowid Row_ID, UpdateDate
            from
                BP_RNStateHdr H2
            WHERE
                (UpdateStatusCd = 1 and nextActionCD = '0A1'
                and MSGSENTSTATUSCD in ('REQSENT', 'RESPSENT', 'REQACKSENT', 'RESPACKSENT', 'REQFAILED', 'RESPFAILED') and FN_Get_Date() - 1/1440 >= UpdateDate )


            UNION
            select /*+ FIRST_ROWS INDEX(H2 AK_RNSTATEHDR_3) */
                rowid Row_ID, UpdateDate
            from
                BP_RNStateHdr H2
            WHERE
                UpdateStatusCd = 1 
            and (nextActionCD in ('RAE', 'GE') AND NOT (nvl(CURRENTPIPSTATUSCD,'X') = 'RESP' AND SYNCHRONOUSFLG = 1 And  SINGLEACTIONFLG = 0)  )
            UNION
            select /*+ FIRST_ROWS */
                rowid Row_ID, UpdateDate
            from
                BP_RNStateHdr H2
            WHERE
                ( UpdateStatusCd in(1, 2) and nextActionCD in ('REQ', 'RESP')                       -- Retry
                  and MSGSENTSTATUSCD in ('REQSENT', 'REQFAILED','RESPSENT','RESPFAILED', 'REQACKSENT','REQACKFAILED' )
                  and CURRENTPIPSTATUSCD not in ('RESPACK','RESPEX')
                  and ExpireTPATime <= FN_Get_Date()
                  and AttemptCntCalc <= AttemptCntTPA
                 )
            UNION
            select /*+ FIRST_ROWS */
                rowid Row_ID, UpdateDate
            from
                BP_RNStateHdr H2
            WHERE
                (UpdateStatusCd in (1, 2)                   -- retries reached max issue 0A1 if one way PIP or wait for TimeToPerform exceeded then issue 0A1 if two way PIP
                 and nextActionCD in ('REQ', 'RESP') 
                 and MSGSENTSTATUSCD in ('REQSENT', 'REQFAILED','RESPSENT','RESPFAILED', 'REQACKSENT','REQACKFAILED' )
                 and CURRENTPIPSTATUSCD not in ('RESPACK','RESPEX')
                 and ExpireTPATime <= FN_Get_Date()
                 and AttemptCntCalc > AttemptCntTPA
                 and processcd != '0A1'
                 and (
                      (SINGLEACTIONFLG = 0) --TimeToPerform is not null and (FN_Get_Date() - (to_date(to_char(CreateDate, 'YYYY-MM-DD HH24:MI:SS'), 'YYYY-MM-DD HH24:MI:SS')))*24*60 > TimeToPerform)                            -- two way PIP
                      or (SINGLEACTIONFLG = 1 and (FN_TS2Date()  - (FN_TS2Date(CreateDate)))*24*60 > (( AttemptCntTPA + 1 ) * TimeToAckTPA)))    -- one way PIP
                 )
            UNION
            select /*+ FIRST_ROWS INDEX(H2 AK_RNSTATEHDR_3) */
                rowid Row_ID, UpdateDate
            from
                BP_RNStateHdr H2
            WHERE
                (UpdateStatusCd =1  and                 -- Time to perform Exceeded (Lost Doc), issue "0A1"
                 nextActionCD = 'RAREC' and MSGSENTSTATUSCD = 'REQSENT'
                 AND SINGLEACTIONFLG = 0
                 and (FN_TS2Date() - (FN_TS2Date(CreateDate)))*24*60 > TimeToPerform
                 )
            UNION
            select /*+ FIRST_ROWS INDEX(H2 AK_RNSTATEHDR_3) */
                rowid Row_ID, UpdateDate
            from
                BP_RNStateHdr H2
            WHERE
                (UpdateStatusCd =1                       -- Out-Of-Sequence
                and nextActionCD = 'RESP'   --  and MSGSENTSTATUSCD = 'REQSENT'
                AND SYNCHRONOUSFLG = 0
                and RAStatus is null
                )

            ) T1
            order by UpdateDAte;
            p_RowCnt := SQL%ROWCOUNT;
        EXCEPTION
            WHEN NO_DATA_FOUND then p_RowCnt := 0;
            WHEN OTHERS then
                p_ReturnCd := 1;
                p_RowCnt := 0;
    END;

    IF NVL(p_RowCnt, 0) > 0 THEN
        BEGIN
            BEGIN
                Lock table BP_RNStateHdr_Que IN EXCLUSIVE MODE NOWAIT;
                p_RowCnt := 1;
                EXCEPTION WHEN OTHERS THEN
                    p_RowCnt := 0;
            END;
            IF p_RowCnt > 0 THEN
                BEGIN
                    DB_ProcAuditLogI_Ah1 (v_ProcName, v_BeginTS, 'Rows found = '||to_char(array_RowID.count), 0);
                    p_RowCnt := array_RowID.count;
                    FORALL  i in 1 .. array_RowID.count
                        INSERT into BP_RNStateHdr_Que
                        VALUES (array_UpdateDate(i), array_RowID(i));
                        commit;
                    EXCEPTION
                        WHEN NO_DATA_FOUND then p_RowCnt := 0;
                        WHEN DUP_VAL_ON_INDEX then p_RowCnt := 0;
                END;
            END IF;
        END;
    END IF;

End InsertQue;

---------------------------------------------------------------
---                    M   A   I   N                        ---
---------------------------------------------------------------

BEGIN

    p_ReturnCd := 0;
    p_RowCnt := 0;
    BEGIN
        select 1 into p_RowCnt from BP_RNStateHdr_Que where rownum < 2;
        Exception
            WHEN NO_DATA_FOUND THEN p_RowCnt := 0;
            WHEN OTHERS THEN p_ReturnCd := 1;
    END;

    IF p_RowCnt = 0 THEN
        InsertQue;
    END IF;

    IF p_RowCnt = 0 THEN
        v_ParamsTxt := v_ParamsTxt || ' -- No rows found --';
    END IF;

    DB_ProcAuditLogI_Ah1 (v_ProcName, v_BeginTS, v_ParamsTxt, 0);
END;
/


-- BP_SPONSOR_STATEI_AH1.sql

CREATE OR REPLACE Procedure BP_SPONSOR_STATEI_AH1
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_INITBUSINESSID in VARCHAR2
	,p_PROCESSINSTANCEID in VARCHAR2
	,p_NEXTACTIONCD in VARCHAR2
	,p_FRBUSINESSID in VARCHAR2
	,p_TOBUSINESSID in VARCHAR2
	,p_DOCID in VARCHAR2
	,p_CUSTOMMSGID01 in VARCHAR2
	,p_UPDATESTATUSCD in NUMBER
	,p_GLBUSAGECD in VARCHAR2
	,p_DESTNAME in VARCHAR2
	,p_INBOUNDVUID in VARCHAR2
	,p_REFINBOUNDVUID in VARCHAR2
	,p_SynchronousFlg in pls_integer
	)
AS

/*
	================================================================================
	Proc Name:       BP_SPONSOR_STATEI_AH1
	Create Date:     2003-07-02 10:45:01
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2003-07-02 SYSTEM	3.5	Created
	2006-07-17 S.Priani	6.1	CR102082

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'BP_SPONSOR_STATEI_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
		||', p_NEXTACTIONCD=>'||p_NEXTACTIONCD||CHR(10)
		||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
		||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
		||', p_DOCID=>'||p_DOCID||CHR(10)
		||', p_CUSTOMMSGID01=>'||p_CUSTOMMSGID01||CHR(10)
		||', p_UPDATESTATUSCD=>'||p_UPDATESTATUSCD||CHR(10)
		||', p_GLBUSAGECD=>'||p_GLBUSAGECD||CHR(10)
		||', p_DESTNAME=>'||p_DESTNAME||CHR(10)
		||', p_INBOUNDVUID=>'||p_INBOUNDVUID||CHR(10)
		||', p_REFINBOUNDVUID=>'||p_REFINBOUNDVUID||CHR(10)
		||', p_SynchronousFlg=>'||p_SynchronousFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into BP_SPONSOR_STATE(
		INITBUSINESSID
		,PROCESSINSTANCEID
		,NEXTACTIONCD
		,FRBUSINESSID
		,TOBUSINESSID
		,DOCID
		,CUSTOMMSGID01
		,UPDATESTATUSCD
		,GLBUSAGECD
		,DESTTYPEID
		,INBOUNDVUID
		,REFINBOUNDVUID
		,SynchronousFlg
	) Values (
		p_INITBUSINESSID
		,p_PROCESSINSTANCEID
		,p_NEXTACTIONCD
		,p_FRBUSINESSID
		,p_TOBUSINESSID
		,p_DOCID
		,p_CUSTOMMSGID01
		,p_UPDATESTATUSCD
		,p_GLBUSAGECD
		,FN_GET_DESTTYPEID(p_DESTNAME)
		,p_INBOUNDVUID
		,p_REFINBOUNDVUID
		,p_SynchronousFlg
	);
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 BP_SPONSOR_STATEI_AH1' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 BP_SPONSOR_STATEI_AH1' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			Rollback work;
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 BP_SPONSOR_STATEI_AH1' || SQLCODE|| ' '||SQLERRM);
END BP_SPONSOR_STATEI_AH1;
/


-- BP_SPONSOR_STATES_AH1.sql

CREATE OR REPLACE PROCEDURE BP_SPONSOR_STATES_AH1
(
    p_ReturnCd                 OUT     pls_INTEGER,
    p_DebugFlg                 IN      PLS_INTEGER,
    RC1                        OUT     db_ref.RCT,
    p_BatchSize                IN      pls_Integer
)
AS
/*
    ================================================================================
    Proc Name:        BP_SPONSOR_STATES_AH1
    Create Date:    2002-05-15 12:04:11
    Author:            B.Moffat

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE	   NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-08-10 S.Priani	6.1	102524 - ebXML support
	2006-09-29 S.Priani	6.1	102827 - ebXML support
	2006-10-19 S.Priani	6.1	102915 - ebXML support
	2006-01-17 S.Priani	6.1	103664 - union view prb
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_rowcnt INTEGER := 0;
    v_BeginTS timestamp := FN_GET_DATE();    -- Performance related variables
    v_rowid   ROWID;
    v_count   pls_integer;
    v_ProcName              varchar2(30) := 'BP_SPONSOR_STATES_AH1';
    v_NewDocFlg             integer := 0;
    v_BatchSize             integer := NVL(p_BatchSize,10) + 1;
    v_BatchRows             integer := 0;
    v_Rows                  integer := 0;
    v_Rows2                 integer := 0;
    j                       integer := 0;
    v_Msg                   varchar2(1024);
    v_BUSINESSIDTYPE        PR_BusinessidType.BUSINESSIDTYPE%Type;
    Type v_RNSTATE_REC_Type is record
        (HDR_Row_ID           dbms_sql.varchar2_table,
        RN_STATE_Action_cd    dbms_sql.varchar2_table,
        INITBUSINESSID        dbms_sql.varchar2_table,
        PROCESSINSTANCEID     dbms_sql.varchar2_table,
        NEXTACTIONCD          dbms_sql.varchar2_table,
        FRBUSINESSID          dbms_sql.varchar2_table,
        TOBUSINESSID          dbms_sql.varchar2_table,
        DOCID                 dbms_sql.varchar2_table,
        CUSTOMMSGID01         dbms_sql.varchar2_table,
        GLBUSAGECD            dbms_sql.varchar2_table,
        DestTypeID            dbms_sql.Number_table,
        INboundVUID           dbms_sql.varchar2_table,
        RefINboundVUID        dbms_sql.varchar2_table,
        SynchronousFlg          dbms_sql.Number_table
    ) ;

    v_RNSTATE_REC    v_RNSTATE_REC_Type;
    v_BP_STATE_TBL BP_Sponsor_STATE_TBL := BP_Sponsor_STATE_TBL();
Procedure InsertLog;
Procedure ProcAudit (p_ProcName IN varchar2, p_ParamsTxt IN varchar2);
Procedure GetDocBatch;
Procedure InsertLog as
BEGIN
    dbms_output.put_Line('Start Log Insert = '||' ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||']' );
    v_Msg := 'Select... Sponsor State Management Engine ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||'] ';
    FORALL  i in 1 .. v_RNSTATE_REC.INITBUSINESSID.count
        Insert into BP_RNSTATEHDRAUDITLOG(
                INITBUSINESSID
        ,       PROCESSINSTANCEID
        ,       RNSTATEDTLID
        ,       NEXTACTIONCD
        ,       MSGSENTSTATUSCD
        ,       CURRENTPIPSTATUSCD
        ,       ERRORREASONTEXT)
        values
        (
        v_RNSTATE_REC.INITBUSINESSID (i)
        ,  v_RNSTATE_REC.PROCESSINSTANCEID (i)
        ,  null
        ,  v_RNSTATE_REC.NEXTACTIONCD (i)
        ,  null
        ,  null
        ,  v_Msg
        );
    EXCEPTION
        When NO_DATA_FOUND Then Null;
        When Others Then
            Raise_application_error (-20999,' 20003 - '||v_ProcName ||'  '|| SQLCODE|| ' '||SQLERRM);
            p_ReturnCd := 1;
END InsertLog;
Procedure GetDocBatch IS
BEGIN
    BEGIN
        update BP_Sponsor_STATE /*+ FIRST_ROWS */
        set
            UpdateStatusCd = 2,
            UpdateDate = FN_GET_DATE()
        WHERE	UpdateStatusCd = 1
        AND	NextActionCD in ('RAREC', 'RAEREC', 'GEREC', '0A1STARTED', 'UNP.SCON.VALERR'  ---
		,'WAITACK', 'ACKOB', 'ACKNOTOB' ,'MSGFAILED', 'MSGNOTSENT')   -- for ebXML
	AND	rownum < v_BatchSize
        RETURNING
             rowid,
             INITBUSINESSID,
             PROCESSINSTANCEID,
             NEXTACTIONCD,
             FRBUSINESSID,
             TOBUSINESSID,
             DOCID,
             CUSTOMMSGID01,
             GLBUSAGECD,
             DestTypeid,
             INboundVUID,
             RefINboundVUID,
             SynchronousFlg
        BULK COLLECT INTO
            v_RNSTATE_REC.HDR_Row_ID,
            v_RNSTATE_REC.INITBUSINESSID,
            v_RNSTATE_REC.PROCESSINSTANCEID,
            v_RNSTATE_REC.NEXTACTIONCD,
            v_RNSTATE_REC.FRBUSINESSID,
            v_RNSTATE_REC.TOBUSINESSID,
            v_RNSTATE_REC.DOCID,
            v_RNSTATE_REC.CUSTOMMSGID01,
            v_RNSTATE_REC.GLBUSAGECD,
            v_RNSTATE_REC.DestTypeID,
            v_RNSTATE_REC.INboundVUID,
            v_RNSTATE_REC.RefINboundVUID,
            v_RNSTATE_REC.SynchronousFlg
            ;
        v_Rows := SQL%ROWCOUNT;
        EXCEPTION
            When NO_DATA_FOUND Then
                v_Rows := 0;
            When Others Then
                Raise;
                p_ReturnCd := -1;
    END;
    IF v_Rows > 0 and NVL(v_RNSTATE_REC.INITBUSINESSID.count, 0) > 0 THEN
        FOR i in 1 .. v_RNSTATE_REC.INITBUSINESSID.count LOOP
            v_BP_STATE_TBL.extend;
            v_BP_STATE_TBL(i) := BP_Sponsor_STATE_REC(
                                    v_RNSTATE_REC.HDR_Row_ID(i),
                                    v_RNSTATE_REC.INITBUSINESSID(i),
                                    v_RNSTATE_REC.PROCESSINSTANCEID(i),
                                    v_RNSTATE_REC.NEXTACTIONCD(i),
                                    v_RNSTATE_REC.FRBUSINESSID(i),
                                    v_RNSTATE_REC.TOBUSINESSID(i),
                                    v_RNSTATE_REC.DOCID(i),
                                    v_RNSTATE_REC.CUSTOMMSGID01(i),
                                    v_RNSTATE_REC.GLBUSAGECD(i),
                                    v_RNSTATE_REC.DestTYpeID(i),
                                    v_RNSTATE_REC.INboundVUID(i),
                                    v_RNSTATE_REC.RefINboundVUID(i),
                                    v_RNSTATE_REC.SynchronousFlg(i)
                                    );
		j := i;
            ProcAudit (v_ProcName, v_Msg);
          END LOOP;
    END IF;

	---- this has been added for ebXML
	---- Sponsor state messages for ebXML may get lower priority if both RNIF and ebMS are running
	v_BatchSize := v_BatchSize - j ;
    BEGIN
        update BP_Sponsor_STATE_ebms /*+ FIRST_ROWS */
        set
            UpdateStatusCd = 2,
            UpdateDate = FN_GET_DATE()
        WHERE	UpdateStatusCd = 1
        AND	NextActionCD in ('RAREC', 'RAEREC', 'GEREC', '0A1STARTED', 'UNP.SCON.VALERR'  ---
		,'WAITACK', 'ACKOB', 'ACKNOTOB' ,'MSGFAILED', 'MSGNOTSENT')   -- for ebXML
	AND	rownum < v_BatchSize
        RETURNING
             rowid,
             INITBUSINESSID,
             PROCESSINSTANCEID,
             NEXTACTIONCD,
             FRBUSINESSID,
             TOBUSINESSID,
             DOCID,
             CUSTOMMSGID01,
             GLBUSAGECD,
             DestTypeid,
             INboundVUID,
             RefINboundVUID,
             SynchronousFlg
        BULK COLLECT INTO
            v_RNSTATE_REC.HDR_Row_ID,
            v_RNSTATE_REC.INITBUSINESSID,
            v_RNSTATE_REC.PROCESSINSTANCEID,
            v_RNSTATE_REC.NEXTACTIONCD,
            v_RNSTATE_REC.FRBUSINESSID,
            v_RNSTATE_REC.TOBUSINESSID,
            v_RNSTATE_REC.DOCID,
            v_RNSTATE_REC.CUSTOMMSGID01,
            v_RNSTATE_REC.GLBUSAGECD,
            v_RNSTATE_REC.DestTypeID,
            v_RNSTATE_REC.INboundVUID,
            v_RNSTATE_REC.RefINboundVUID,
            v_RNSTATE_REC.SynchronousFlg
            ;
        v_Rows := SQL%ROWCOUNT;
        EXCEPTION
            When NO_DATA_FOUND Then
                v_Rows := 0;
            When Others Then
                Raise;
                p_ReturnCd := -1;
    END;
    IF v_Rows > 0 and NVL(v_RNSTATE_REC.INITBUSINESSID.count, 0) > 0 THEN
        FOR i in 1 .. v_RNSTATE_REC.INITBUSINESSID.count LOOP
            v_BP_STATE_TBL.extend;
            v_BP_STATE_TBL(j + i) := BP_Sponsor_STATE_REC(
                                    v_RNSTATE_REC.HDR_Row_ID(i),
                                    v_RNSTATE_REC.INITBUSINESSID(i),
                                    v_RNSTATE_REC.PROCESSINSTANCEID(i),
                                    v_RNSTATE_REC.NEXTACTIONCD(i),
                                    v_RNSTATE_REC.FRBUSINESSID(i),
                                    v_RNSTATE_REC.TOBUSINESSID(i),
                                    v_RNSTATE_REC.DOCID(i),
                                    v_RNSTATE_REC.CUSTOMMSGID01(i),
                                    v_RNSTATE_REC.GLBUSAGECD(i),
                                    v_RNSTATE_REC.DestTYpeID(i),
                                    v_RNSTATE_REC.INboundVUID(i),
                                    v_RNSTATE_REC.RefINboundVUID(i),
                                    v_RNSTATE_REC.SynchronousFlg(i)
                                    );
            ProcAudit (v_ProcName, v_Msg);
          END LOOP;
    END IF;
   select count(*) INTO v_count
   from table(cast(v_BP_STATE_TBL as BP_Sponsor_STATE_TBL)) T;
END GetDocBatch;
PROCEDURE ProcAudit (p_ProcName IN varchar2, p_ParamsTxt IN varchar2)
IS
BEGIN
        DB_ProcAuditLogI_Ah1 (
            p_ProcName,
            v_BeginTS,
            p_ParamsTxt
            );
END ProcAudit;
Procedure Get_Duns_BusinessIdType
is
Begin
    SELECT BUSINESSIDTYPE INTO v_BUSINESSIDTYPE
    FROM pr_businessidType
    WHERE BusinessIdName = 'DUNS';
End Get_Duns_BusinessIdType;
---------------------------------------------------------------
---                    M   A   I   N                        ---
---------------------------------------------------------------

BEGIN
    BEGIN
        Get_Duns_BusinessIdType;
        select 1 into v_NewDocFlg
        from BP_Sponsor_STATE_vw
        where UpdateStatusCd = 1
        and rownum < 2;
        EXCEPTION
          When NO_DATA_FOUND Then v_NewDocFlg := 0;
    END;
    IF v_NewDocFlg > 0 THEN
        v_Msg := 'New Docs Found';
        ProcAudit (v_ProcName, v_Msg);
        GetDocBatch;
    END IF;
    p_ReturnCd := 0;
    IF RC1%ISOPEN THEN
        Close RC1;
    END IF;
      IF NVL(v_BP_STATE_TBL.count, 0) > 0 THEN
        v_Msg := 'No Docs Found';
      ELSE
        v_Msg := 'Output for new Docs Found';
      END IF;
        ProcAudit (v_ProcName, v_Msg);
        open RC1 for
            select
                T.NEXTACTIONCD,
                T.INITBUSINESSID,
                T.PROCESSINSTANCEID,
                T.FRBUSINESSID,
                T.TOBUSINESSID,
                T.DOCID,
                T.CUSTOMMSGID01,
                T.GLBUSAGECD,
                Dest.DestName,    -- T.DestTYpeID,
                T.INboundVUID,
                T.RefINboundVUID,
                T.SynchronousFlg,
                nvl(br.PARTNEREVENTFLG,0)  "frPARTNEREVENTFLG",
                frpT.ownerflg as FrPartnerOwnerFlg,
                topT.ownerflg as ToPartnerOwnerFlg
             from
                TABLE(cast(v_BP_STATE_TBL as BP_Sponsor_STATE_TBL)) T
                Left Outer Join pr_busIdentifier b on T.FRBUSINESSID = b.BUSINESSID
                Left Outer Join pr_BusIdRouting br  on b.BUSIDENTIFIERID = br.BUSIDENTIFIERID
                Left Outer Join pr_Busidentifier frBus on t.frBusinessid    = frBus.Businessid
                Left Outer Join pr_Busidentifier ToBus on t.ToBusinessid    = ToBus.Businessid
                Left Outer Join pr_partner       frp   on frBus.partnerid   = frp.partnerid
                Left Outer Join pr_partner       Top   on ToBus.partnerid   = Top.partnerid
                Left Outer Join pr_partnerType   frpT  on frp.PARTNERTYPEID = frpT.PARTNERTYPEID
                Left Outer Join pr_partnerType   TopT  on Top.PARTNERTYPEID = TopT.PARTNERTYPEID
                Left Outer Join pr_DestType      Dest  on Dest.DestTypeID   = T.DestTypeID    ;
    ProcAudit (v_ProcName, 'ReturnOutput');
    p_ReturnCd := 0;
    v_Msg := 'p_ReturnCd,' || p_DebugFlg|| ', RC1,' || p_BatchSize;
    ProcAudit (v_ProcName, v_Msg);
END;
/


-- BP_SPONSOR_STATEU_AH1.sql

CREATE OR REPLACE PROCEDURE BP_SPONSOR_STATEU_AH1
(
     p_ReturnCd          out integer
    ,p_DebugFlg          in pls_integer
    ,p_INITBUSINESSID    in VARCHAR2
    ,p_PROCESSINSTANCEID in VARCHAR2
    ,p_updatestatusCD    in pls_integer
    ,p_NEXTACTIONCD      in varchar2
) as
/*
================================================================================
Proc Name:        BP_SPONSOR_STATEU_AH1
Create Date:     2002-05-14 16:30:31
Author:            V Goel
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_GET_DATE(); -- Performance related variables
v_rowTS integer :=0;
zeroUpdate  Exception;
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('BP_SPONSOR_STATEU_AH1',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||p_INITBUSINESSID
    ||','||p_PROCESSINSTANCEID
    ||','||p_updatestatusCD
    );
End ProcAudit;
BEGIN
    v_rowcnt := 0;
    Update BP_SPONSOR_STATE SET
          updatestatusCD= nvl(p_updatestatusCD, updatestatusCD)
    ,       NEXTACTIONCD = nvl(p_NEXTACTIONCD, NEXTACTIONCD)
    ,      ROWTS        = ROWTS + 1
    ,      UPDATEDATE   = v_BeginTS
    Where INITBUSINESSID     = p_INITBUSINESSID
    and PROCESSINSTANCEID    = p_PROCESSINSTANCEID;

    v_rowcnt := SQL%rowcount;
    if (v_rowcnt = 0) then  raise ZeroUpdate ; end if;

    p_ReturnCd := 0;
    ProcAudit;
EXCEPTION
    WHEN TOO_MANY_ROWS then
        ProcAudit;
        raise_application_error(-20999, '20100 - BP_SPONSOR_STATEU_AH1: Too Many Rows');
    WHEN ZeroUpdate then 
        ProcAudit;
    WHEN OTHERS then ProcAudit;
        raise_application_error(-20999,'20102 - BP_SPONSOR_STATEU_AH1:'||sqlerrm);
        p_ReturnCd := 1;
End BP_SPONSOR_STATEU_AH1;
/


-- BP_SPONSOR_STATEU_AH2.sql

CREATE OR REPLACE PROCEDURE BP_SPONSOR_STATEU_AH2 (
    p_ReturnCd out integer
    ,p_DebugFlg in pls_integer
    ,p_INITBUSINESSID in VARCHAR2
    ,p_PROCESSINSTANCEID in VARCHAR2
    ,p_NEXTACTIONCD in VARCHAR2
    ,p_docid        in varchar2
    ,p_refInboundVuid in Varchar2
    ,p_ROWTS in NUMBER
    ) as
/*
================================================================================
Proc Name:        BP_SPONSOR_STATEU_AH2
Create Date:      2002-05-14 16:30:31
Author:            V Goel
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
---------- ------------- ------ ---------------------------------------------
DATE         NAME        Ver     DESCRIPTION
---------- ------------- ------ ---------------------------------------------
2002-05-14 V Goel        3.2    Created
2003-01-23 J.Chen        3.4    Added p_refInboundVuid as input
2002-05-14 V Goel        3.2    Add UpdateStatusCD to update statement Defect # 1-1B932
2003-07-09 B.Moffat      3.5    Renamed from BP_SPONSOR_STATE_U to the current name.

================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_GET_DATE(); -- Performance related variables
v_rowTS integer :=0;
v_UPDATESTATUSCD    BP_SPONSOR_STATE.UPDATESTATUSCD%Type := Null;
v_ProcName          varchar2(30) := 'BP_SPONSOR_STATEU_AH2';
v_ExceptionFlg      integer := 0;
v_ReturnCd          pls_integer := 0;
v_ParamsTxt         varchar2(4000) :=
            'p_ReturnCd'||','||p_DebugFlg
            ||','||p_INITBUSINESSID
            ||','||p_PROCESSINSTANCEID
            ||','||p_NEXTACTIONCD
            ||','||p_docid
            ||','||p_refInboundVuid
            ||','||p_ROWTS;


zeroUpdate  Exception;

BEGIN
    v_rowcnt := 0;
    If p_NEXTACTIONCD in ('RAREC', 'RAEREC', 'GEREC', '0A1STARTED', 'UNP.SCON.VALERR') Then
        v_UPDATESTATUSCD := 1;
    End If;
    Update BP_SPONSOR_STATE SET
          NEXTACTIONCD= nvl(p_NEXTACTIONCD, NEXTACTIONCD)
    ,     DOCID       = nvl(p_DOCID, DOCID)
    ,     refInboundVuid = nvl(p_refInboundVuid, refInboundVuid)
    ,     UPDATESTATUSCD = nvl(v_UPDATESTATUSCD, UPDATESTATUSCD)
    ,      ROWTS       = p_ROWTS + 1
    ,      UPDATEDATE  = v_BeginTS
    Where INITBUSINESSID     = p_INITBUSINESSID
    and PROCESSINSTANCEID    = p_PROCESSINSTANCEID
    And ROWTS                = p_ROWTS
        returning ROWTS into v_rowTS;
    v_rowcnt := SQL%rowcount;
    if (v_rowcnt = 0) then  raise ZeroUpdate ; end if;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
    WHEN TOO_MANY_ROWS then DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
        raise_application_error(-20999, '20100 - BP_SPONSOR_STATE_U: Too Many Rows');
    WHEN ZeroUpdate then DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
        raise_application_error(-20999,'20100 - BP_SPONSOR_STATE_U: No Data Found');
    WHEN OTHERS then DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
        raise_application_error(-20999,'20100 - BP_SPONSOR_STATE_U:'||sqlerrm);
        ROLLBACK work;
        p_ReturnCd := 1;
End;
/


-- BP_SPONSOR_STATE_EBMSI_AH1.sql

CREATE OR REPLACE Procedure BP_SPONSOR_STATE_EBMSI_AH1
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_INITBUSINESSID in VARCHAR2
	,p_PROCESSINSTANCEID in VARCHAR2
	,p_NEXTACTIONCD in VARCHAR2
	,p_FRBUSINESSID in VARCHAR2
	,p_TOBUSINESSID in VARCHAR2
	,p_DOCID in VARCHAR2
	,p_CUSTOMMSGID01 in VARCHAR2
	,p_UPDATESTATUSCD in NUMBER
	,p_GLBUSAGECD in VARCHAR2
	,p_DESTNAME in VARCHAR2
	,p_INBOUNDVUID in VARCHAR2
	,p_REFINBOUNDVUID in VARCHAR2
	,p_SynchronousFlg in pls_integer
	)
AS

/*
	================================================================================
	Proc Name:       BP_SPONSOR_STATE_EBMSI_AH1
	Create Date:     2006-09-02 10:45:01
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-10-17 S.Priani	6.1	Created:CR102915

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'BP_SPONSOR_STATE_EBMSI_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
		||', p_NEXTACTIONCD=>'||p_NEXTACTIONCD||CHR(10)
		||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
		||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
		||', p_DOCID=>'||p_DOCID||CHR(10)
		||', p_CUSTOMMSGID01=>'||p_CUSTOMMSGID01||CHR(10)
		||', p_UPDATESTATUSCD=>'||p_UPDATESTATUSCD||CHR(10)
		||', p_GLBUSAGECD=>'||p_GLBUSAGECD||CHR(10)
		||', p_DESTNAME=>'||p_DESTNAME||CHR(10)
		||', p_INBOUNDVUID=>'||p_INBOUNDVUID||CHR(10)
		||', p_REFINBOUNDVUID=>'||p_REFINBOUNDVUID||CHR(10)
		||', p_SynchronousFlg=>'||p_SynchronousFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into BP_SPONSOR_STATE_EBMS(
		INITBUSINESSID
		,PROCESSINSTANCEID
		,NEXTACTIONCD
		,FRBUSINESSID
		,TOBUSINESSID
		,DOCID
		,CUSTOMMSGID01
		,UPDATESTATUSCD
		,GLBUSAGECD
		,DESTTYPEID
		,INBOUNDVUID
		,REFINBOUNDVUID
		,SynchronousFlg
	) Values (
		p_INITBUSINESSID
		,p_PROCESSINSTANCEID
		,p_NEXTACTIONCD
		,p_FRBUSINESSID
		,p_TOBUSINESSID
		,p_DOCID
		,p_CUSTOMMSGID01
		,p_UPDATESTATUSCD
		,p_GLBUSAGECD
		,FN_GET_DESTTYPEID(p_DESTNAME)
		,p_INBOUNDVUID
		,p_REFINBOUNDVUID
		,p_SynchronousFlg
	);
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE;
		When OTHERS Then
			Rollback work;
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
END BP_SPONSOR_STATE_EBMSI_AH1;
/


-- BP_STATE_HDRI_AH1.sql

CREATE OR REPLACE PROCEDURE         BP_STATE_HDRI_AH1
    (p_ReturnCd out PLS_Integer
    ,p_DebugFlg in pls_integer
    ,p_SqlErrm     out varchar2
    ,p_STATEHDRID  out pls_integer
    ,P_TPA_FILLFLG in pls_integer  -- 0= Provided 1= Auto Filled
    ,p_WorkGroupName     in varchar2
    ,p_StateInstanceName in varchar2
    ,p_CHANNELID in NUMBER
    ,p_STATESTATUSCD in pls_integer
    ,p_RESTARTFLG in NUMBER
    ,p_PROCESSINSTANCEID in VARCHAR2
    ,p_PROCESSEXPIREDATE in Timestamp
    ,p_PROCESSSTARTDATE in Timestamp
    ,p_INITBUSINESSID in VARCHAR2
    ,p_FRBUSINESSID in VARCHAR2
    ,p_TOBUSINESSID in VARCHAR2
    ,p_CURRENTSTATECD in VARCHAR2
    ,p_LASTSTATECD in VARCHAR2
    ,p_NEXTACTIONCD in VARCHAR2
    ,p_MSGSENTSTATUSCD in VARCHAR2
    ,p_RETRYCNT in NUMBER
    ,p_TPA_MaxDocRetryCnt in NUMBER
    ,p_TPA_TIMETOACK in NUMBER
    ,p_TPA_TimeToPerform in Number
    ,p_NEXTRETRYDATE in Timestamp
    ,p_EXPECTACKFLG in NUMBER
    ,p_ACKRCVFLG in NUMBER
    ,p_ACKSENTFLG in NUMBER
    ,p_OUTOFSEQFLG in NUMBER
    ,p_EXPECTRESPDOCFLG in NUMBER
    ,p_PROCESSRETRYDATE in Timestamp
    ,p_RESPONSERCVFLG in NUMBER
    ,p_STATESCENARIOID in NUMBER
    ,p_SCENARIOREQIREDFLG in NUMBER
    ,p_BATCHCNT in NUMBER
    ,p_BATCHDOCPROCESSEDCNT in NUMBER
    ,p_DESTTYPEID in NUMBER
    ,p_UPDATEPGMNAME in VARCHAR2
    ,p_ERRORCD in VARCHAR2
    ,p_ERRORREASONTEXT in VARCHAR2
     )
AS

/*
    ================================================================================
    Proc Name:       BP_STATE_HDRI_AH1
    Create Date:     2003-03-04 16:42:45
    Author:          SYSTEM

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar(30) := 'BP_STATE_HDRI_AH1';
v_ExceptionFlg    integer := 0;
v_STATEINSTANCENAME bp_state_hdr.STATEINSTANCENAME%Type := nvl(P_STATEINSTANCENAME, -1);
------
v_ParamsTxt       varchar(4000) :=
        'p_DEBUGFLG=>'||p_DEBUGFLG||CHR(10)
        ||',p_TPA_FILLFLG=>'||p_TPA_FILLFLG||CHR(10)
        ||', p_WORKGROUPNAME=>'||p_WORKGROUPNAME||CHR(10)
        ||', p_STATEINSTANCENAME=>'||p_STATEINSTANCENAME||CHR(10)
        ||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
        ||', p_STATESTATUSCD=>'||p_STATESTATUSCD||CHR(10)
        ||', p_RESTARTFLG=>'||p_RESTARTFLG||CHR(10)
        ||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
        ||', p_PROCESSEXPIREDATE=>'||p_PROCESSEXPIREDATE||CHR(10)
        ||', p_PROCESSSTARTDATE=>'||p_PROCESSSTARTDATE||CHR(10)
        ||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
        ||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
        ||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
        ||', p_CURRENTSTATECD=>'||p_CURRENTSTATECD||CHR(10)
        ||', p_LASTSTATECD=>'||p_LASTSTATECD||CHR(10)
        ||', p_NEXTACTIONCD=>'||p_NEXTACTIONCD||CHR(10)
        ||', p_MSGSENTSTATUSCD=>'||p_MSGSENTSTATUSCD||CHR(10)
        ||', p_RETRYCNT=>'||p_RETRYCNT||CHR(10)
        ||', p_TPA_MaxDocRetryCnt=>'||p_TPA_MaxDocRetryCnt||CHR(10)
        ||', p_TPA_TimeToPerform=>'||p_TPA_TimeToPerform||CHR(10)
        ||', p_TPA_TIMETOACK=>'||p_TPA_TIMETOACK||CHR(10)
        ||', p_NEXTRETRYDATE=>'||p_NEXTRETRYDATE||CHR(10)
        ||', p_EXPECTACKFLG=>'||p_EXPECTACKFLG||CHR(10)
        ||', p_ACKRCVFLG=>'||p_ACKRCVFLG||CHR(10)
        ||', p_ACKSENTFLG=>'||p_ACKSENTFLG||CHR(10)
        ||', p_OUTOFSEQFLG=>'||p_OUTOFSEQFLG||CHR(10)
        ||', p_EXPECTRESPDOCFLG=>'||p_EXPECTRESPDOCFLG||CHR(10)
        ||', p_PROCESSRETRYDATE=>'||p_PROCESSRETRYDATE||CHR(10)
        ||', p_RESPONSERCVFLG=>'||p_RESPONSERCVFLG||CHR(10)
        ||', p_STATESCENARIOID=>'||p_STATESCENARIOID||CHR(10)
        ||', p_SCENARIOREQIREDFLG=>'||p_SCENARIOREQIREDFLG||CHR(10)
        ||', p_BATCHCNT=>'||p_BATCHCNT||CHR(10)
        ||', p_BATCHDOCPROCESSEDCNT=>'||p_BATCHDOCPROCESSEDCNT||CHR(10)
        ||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
        ||', p_UPDATEPGMNAME=>'||p_UPDATEPGMNAME||CHR(10)
        ||', p_ERRORCD=>'||p_ERRORCD||CHR(10)
        ||', p_ERRORREASONTEXT=>'||p_ERRORREASONTEXT||CHR(10)
        ;

v_PK_Value        number := 0;
v_Exception       varchar2(4000);
--
-- BP_STATE_HDR
--
Procedure Get_StateHdrId is
Begin
    Select STATEHDRID Into p_STATEHDRID
    From BP_STATE_HDR
    Where INITBUSINESSID    = p_INITBUSINESSID
    And   PROCESSINSTANCEID = p_PROCESSINSTANCEID
    ;
    EXCEPTION
        When NO_DATA_FOUND THEN
            p_SqlErrm := 'INITBUSINESSID=>'|| p_INITBUSINESSID||' PROCESSINSTANCEID=>'|| p_PROCESSINSTANCEID;
            p_ReturnCd := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 1);
        When DUP_VAL_ON_INDEX then
            p_SqlErrm := 'INITBUSINESSID=>'|| p_INITBUSINESSID||' PROCESSINSTANCEID=>'|| p_PROCESSINSTANCEID;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, 1);
            p_returnCd := 2;
        When OTHERS Then
            p_SqlErrm := SQLERRM ;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, p_SqlErrm||' '||v_ParamsTxt, p_DebugFlg, 1);
End Get_StateHdrId;
Procedure BP_STATE_HDRi is
    BEGIN
        Insert into BP_STATE_HDR(
            STATEHDRID
            ,WORKGROUPNAME
            ,STATEINSTANCENAME
            ,CHANNELID
            ,STATESTATUSCD
            ,RESTARTFLG
            ,PROCESSINSTANCEID
            ,PROCESSEXPIREDATE
            ,PROCESSSTARTDATE
            ,INITBUSINESSID
            ,FRBUSINESSID
            ,TOBUSINESSID
            ,CURRENTSTATECD
            ,LASTSTATECD
            ,NEXTACTIONCD
            ,MSGSENTSTATUSCD
            ,RETRYCNT
            ,TPA_MaxDocRetryCnt
            ,TPA_TIMETOACK
            ,TPA_TimeToPerform
            ,NEXTRETRYDATE
            ,EXPECTACKFLG
            ,ACKRCVFLG
            ,ACKSENTFLG
            ,OUTOFSEQFLG
            ,EXPECTRESPDOCFLG
            ,PROCESSRETRYDATE
            ,RESPONSERCVFLG
            ,STATESCENARIOID
            ,SCENARIOREQIREDFLG
            ,BATCHCNT
            ,BATCHDOCPROCESSEDCNT
            ,DESTTYPEID
            ,UPDATEPGMNAME
            ,ERRORCD
            ,ERRORREASONTEXT
        ) Values (
            BP_STATE_HDR_SEQ.Nextval
            ,p_WORKGROUPNAME
            ,v_STATEINSTANCENAME
            ,nvl(p_CHANNELID, -1)
            ,p_STATESTATUSCD
            ,p_RESTARTFLG
            ,p_PROCESSINSTANCEID
            ,p_PROCESSEXPIREDATE
            ,p_PROCESSSTARTDATE
            ,p_INITBUSINESSID
            ,p_FRBUSINESSID
            ,p_TOBUSINESSID
            ,p_CURRENTSTATECD
            ,p_LASTSTATECD
            ,p_NEXTACTIONCD
            ,p_MSGSENTSTATUSCD
            ,p_RETRYCNT
            ,p_TPA_MaxDocRetryCnt
            ,p_TPA_TIMETOACK
            ,p_TPA_TimeToPerform
            ,p_NEXTRETRYDATE
            ,p_EXPECTACKFLG
            ,p_ACKRCVFLG
            ,p_ACKSENTFLG
            ,p_OUTOFSEQFLG
            ,p_EXPECTRESPDOCFLG
            ,p_PROCESSRETRYDATE
            ,p_RESPONSERCVFLG
            ,p_STATESCENARIOID
            ,p_SCENARIOREQIREDFLG
            ,p_BATCHCNT
            ,p_BATCHDOCPROCESSEDCNT
            ,p_DESTTYPEID
            ,p_UPDATEPGMNAME
            ,p_ERRORCD
            ,p_ERRORREASONTEXT
        ) Returning STATEHDRID into p_STATEHDRID;
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 0);
        p_ReturnCd := 0;
        EXCEPTION
            When NO_DATA_FOUND THEN
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 1);
            When DUP_VAL_ON_INDEX then
                Get_StateHdrId;
            When OTHERS Then
                p_SqlErrm := SQLERRM ;
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, p_SqlErrm||' '||v_ParamsTxt, p_DebugFlg, 1);
    END BP_STATE_HDRI;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'Proc Input '||v_ParamsTxt, p_DebugFlg, 0);
   BP_STATE_HDRI;
END BP_STATE_HDRI_AH1;
/


-- BP_STATE_HDRS_AH1.sql

CREATE OR REPLACE PROCEDURE         BP_STATE_HDRS_AH1
    (
    p_ReturnCd              OUT PLS_INTEGER
    , p_DebugFlg            IN PLS_INTEGER
    , RC1                   OUT DB_REF.RCT
    , p_WORKGROUPNAME       IN Varchar2
    , p_STATEINSTANCENAME   IN Varchar2
    , p_INITBUSINESSID      IN Varchar2
    , p_PROCESSINSTANCEID   IN Varchar2
    )
AS

/*
    ================================================================================
    Proc Name:       BP_STATE_HDRS_AH1
    Create Date:     2003-05-09 14:08:53
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-09 V.Goel           3.5 Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'BP_STATE_HDRS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_WORKGROUPNAME=>'||p_WORKGROUPNAME||CHR(10)
        ||', p_STATEINSTANCENAME=>'|| p_STATEINSTANCENAME||CHR(10)
        ||', p_INITBUSINESSID=>'||  p_INITBUSINESSID||CHR(10)
        ||', p_PROCESSINSTANCEID=>'|| p_PROCESSINSTANCEID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    SELECT
             H.CHANNELID
            ,D.DestName   --DESTTYPEID
    FROM
            BP_STATE_HDR H
            Inner Join PR_DestType D ON D.DestTypeID = H.DestTypeID

    Where
             H.WORKGROUPNAME     =  p_WORKGROUPNAME
        And H.STATEINSTANCENAME  = p_STATEINSTANCENAME
        And H.PROCESSINSTANCEID  = p_PROCESSINSTANCEID
        And H.INITBUSINESSID     = p_INITBUSINESSID
        ;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End BP_STATE_HDRS_AH1;
/


-- BP_STATE_HDRS_AH2.sql

CREATE OR REPLACE PROCEDURE BP_STATE_HDRS_AH2
    (
    p_ReturnCd              OUT PLS_INTEGER
    , p_DebugFlg            IN PLS_INTEGER
    , RC1                   OUT DB_REF.RCT
    , p_WORKGROUPNAME       IN Varchar2
    , p_STATEINSTANCENAME   IN Varchar2
    , p_INITBUSINESSID      IN Varchar2
    , p_PROCESSINSTANCEID   IN Varchar2
    )
AS

/*
    ================================================================================
    Proc Name:       BP_STATE_HDRS_AH2
    Create Date:     2003-05-09 14:08:53
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-09 V.Goel           3.5 Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'BP_STATE_HDRS_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_WORKGROUPNAME=>'||p_WORKGROUPNAME||CHR(10)
        ||', p_STATEINSTANCENAME=>'|| p_STATEINSTANCENAME||CHR(10)
        ||', p_INITBUSINESSID=>'||  p_INITBUSINESSID||CHR(10)
        ||', p_PROCESSINSTANCEID=>'|| p_PROCESSINSTANCEID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    SELECT
		H.CHANNELID
		,H.STATEHDRID           
		,H.WORKGROUPNAME        
		,H.STATEINSTANCENAME    
		,H.STATESTATUSCD        
		,H.RESTARTFLG           
		,H.PROCESSINSTANCEID    
		,H.PROCESSEXPIREDATE    
		,H.PROCESSSTARTDATE     
		,H.INITBUSINESSID       
		,H.FRBUSINESSID         
		,H.TOBUSINESSID         
		,H.CURRENTSTATECD       
		,H.LASTSTATECD          
		,H.NEXTACTIONCD         
		,H.MSGSENTSTATUSCD      
		,H.RETRYCNT             
		,H.TPA_MAXDOCRETRYCNT   
		,H.TPA_TIMETOACK        
		,H.TPA_TIMETOPERFORM    
		,H.NEXTRETRYDATE        
		,H.EXPECTACKFLG         
		,H.ACKRCVFLG            
		,H.ACKSENTFLG           
		,H.OUTOFSEQFLG          
		,H.EXPECTRESPDOCFLG     
		,H.PROCESSRETRYDATE     
		,H.RESPONSERCVFLG       
		,H.STATESCENARIOID      
		,H.SCENARIOREQIREDFLG   
		,H.BATCHCNT             
		,H.BATCHDOCPROCESSEDCNT 
		,H.DESTTYPEID           
		,H.UPDATEPGMNAME        
		,H.ERRORCD              
		,H.ERRORREASONTEXT      
		,H.ROWTS                
		,H.CREATEDATE           
		,H.UPDATEDATE           
		,D.DestName   --DESTTYPEID
    FROM
            BP_STATE_HDR H
            Inner Join PR_DestType D ON D.DestTypeID = H.DestTypeID

    Where
             H.WORKGROUPNAME     =  p_WORKGROUPNAME
        And H.STATEINSTANCENAME  = p_STATEINSTANCENAME
        And H.PROCESSINSTANCEID  = p_PROCESSINSTANCEID
        And H.INITBUSINESSID     = p_INITBUSINESSID
        ;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End BP_STATE_HDRS_AH2;
/


-- BP_STATE_HDRU_AH1.sql

CREATE OR REPLACE PROCEDURE BP_STATE_HDRU_AH1
(
     p_ReturnCd out PLS_Integer
    ,p_DebugFlg in PLS_Integer
    ,RC1 out DB_Ref.RCT
    ,p_SqlErrm out VARCHAR2
    ,p_BatchSize in PLS_Integer
    ,p_WorkGroupName in VARCHAR2
    ,p_StateInstanceName in VARCHAR2
) as

/*
    ================================================================================
    Proc Name:       BP_STATE_HDRU
    Create Date:     2003-03-04 16:42:39
    Author:          SYSTEM
    NOTE: p_BatchSize hard coded to 1

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

    Selection Criterian
    1   Initial Try
        Rows with StateStatusCd    = 1 are selected
                  And ExpectAckFLG = 1

    2   Retry
        Rows with StateStatusCd       = 3
                And ProcessRetryDATE  <=  FN_GET_DATE()
                And ExpectAckFLG      =  1 And AckRcvFLG = 0
                And RETRYCNT          <= TPA_MAXDOCRETRYCNT

    3   Max retries reached
        Rows with StateStatusCd       = 3
                And ProcessRetryDATE  <=  FN_GET_DATE()
                And ExpectAckFLG      =  1 And AckRcvFLG = 0
                And RETRYCNT          > TPA_MAXDOCRETRYCNT

    4   TTP (Time to Perform) Expired for response
        Rows with StateStatusCd      = 3
                And ExpectRespDocFLG = 1 And ResponseRcvFLG  = 0
                And ProcessExpireDATE <= FN_GET_DATE()

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_ProcName             varchar2(30) := 'BP_STATE_HDRU_AH1';
v_RowCnt                pls_integer :=0;
v_RowCnt_Update         pls_integer :=0;
v_RowCnt_Delete         pls_integer :=0;

v_BeginTS               timestamp := FN_GET_DATE(); -- Performance related variables
v_ExceptionFlg          integer := 0;
v_RowTS                 integer :=0;
i                       pls_integer;
v_ParamsTxt             Varchar2(4000) :=
  'p_DebugFlg=>'         ||p_DebugFlg      ||CHR(10)
  ||', p_DebugFlg=>'     ||p_DebugFlg      ||CHR(10)
  ||', ,p_BatchSize=>'   ||p_BatchSize     ||CHR(10)
  ||', p_WorkGroupName=>'||p_WorkGroupName ||CHR(10)
  ||', p_StateInstanceName=>'||p_StateInstanceName ||CHR(10);
---
--zeroUpdate              Exception;
v_ReturnCd              pls_integer := 0;
v_StateHdrId            pls_integer;
v_RowId                 rowid := Null;
v_updatedate           Timestamp;
Procedure ProcAudit is
Begin
    DB_ProcAuditLogI_ah1(v_ProcName,v_beginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End ProcAudit;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
  Begin
    DELETE from BP_State_Hdr_Que Where rownum < 2 Returning row_id, updateDate into v_RowId, v_updateDate;
    v_RowCnt_Delete := SQL%RowCount;
    Commit;
    Exception
    when Others Then
        Null;
--    DBMS_OUTPUT.PUT_LINE('Exception: DeleteFlg :'||v_RowCnt_Delete|| ' Row ID :'||v_RowId);
  end;
--    DBMS_OUTPUT.PUT_LINE('DeleteFlg :'||v_RowCnt_Delete|| ' Row ID :'||v_RowId);
    If v_RowCnt_Delete = 0 Then
        v_RowId := Null;
        BP_STATE_HDR_QUEI_AH1
        (
        P_RETURNCD            => v_ReturnCD
        , P_ROWCNT            => v_RowCnt
        , P_WORKGROUPNAME     => p_WorkGroupName
        , P_STATEINSTANCENAME => p_StateInstanceName
        );
    Else
        BEGIN
            update BP_State_Hdr
            set
                StateStatusCd = Case
                                   When  StateStatusCd = 1 Then  2
                                   When  StateStatusCd = 3 Then  3
                                   End
                ,UpdatePgmName = p_WorkGroupName
                ,UpdateDate = v_BeginTS
                ,ProcessRetryDATE = v_BeginTS + (TPA_TIMETOACK/1440)  --(RN ok, AS2 ok, EDI -no need)
                ,RowTS = RowTS + 1
            where
                rowid = v_RowId
                and updatedate = v_updateDate;

            v_RowCnt_Update := SQL%RowCount;
            EXCEPTION
            When Others Then
                p_ReturnCd := 1;
                p_SqlErrm := SQLERRM;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);
        END;
    End if ; -- v_RowCnt_delete > 0

-- Finally Open Cursor -- if Update is successful

    If  v_RowCnt_Update = 0 Then
        v_RowId := Null;
    End If;
    If RC1%IsOpen Then Close RC1; End If;

    CASE p_WorkGroupName
    When 'AS' Then
        Open RC1 for Select  /*+ First_Rows */
            Case
                When StateStatusCd = 3 And H.ExpectAckFLG = 1 And H.AckRcvFLG = 0 And H.RETRYCNT < H.TPA_MAXDOCRETRYCNT
                Then 'Retry'
                When StateStatusCd = 3 And H.ExpectAckFLG = 1 And H.AckRcvFLG = 0 And H.RETRYCNT >= H.TPA_MAXDOCRETRYCNT
                Then 'RetryMax'
                When ExpectRespDocFLG = 1 And ResponseRcvFLG = 0 And ProcessExpireDATE <= FN_GET_DATE()
                Then 'TTPExc'
                Else 'Normal'
            End  as ConditionCode,
            H.StateHdrId,
            H.WORKGROUPNAME,
            H.STATEINSTANCENAME,
            H.CHANNELID,
            H.STATESTATUSCD,
            H.RESTARTFLG,
            H.PROCESSINSTANCEID,
            H.PROCESSEXPIREDATE,
            H.PROCESSSTARTDATE,
            H.INITBUSINESSID,
            H.FRBUSINESSID,
            H.TOBUSINESSID,
            H.CURRENTSTATECD,
            H.LASTSTATECD,
            H.NEXTACTIONCD,
            H.MSGSENTSTATUSCD,
            H.RETRYCNT,
            H.TPA_MAXDOCRETRYCNT,
            H.TPA_TIMETOACK,
            H.TPA_TIMETOPERFORM,
            H.NEXTRETRYDATE,
            H.EXPECTACKFLG,
            H.ACKRCVFLG,
            H.ACKSENTFLG,
            H.OUTOFSEQFLG,
            H.EXPECTRESPDOCFLG,
            H.PROCESSRETRYDATE,
            H.RESPONSERCVFLG,
            H.STATESCENARIOID,
            H.SCENARIOREQIREDFLG,
            H.BATCHCNT,
            H.BATCHDOCPROCESSEDCNT,
            Dest.DestName,
            H.UPDATEPGMNAME,
            H.ERRORCD,
            H.ERRORREASONTEXT,
-- AS_State_Hdr
            asH.MDNREQUIREDFLG,
            asH.MDNSIGREQUIREDFLG,
            asH.MDNASYNFLG,
            asH.MDNSIGNATUREALGORITHM,
            asH.COMPRESSEDFLG,
-- AS_State_Dtl
            asD.PackageName,
            asD.PackageVer,      --AS_VER,
            asD.FRBUSINESSID  DTL_FRBUSINESSID,
            asD.TOBUSINESSID  DTL_TOBUSINESSID,
            asD.MSGSENTSTATUSCD,
            asD.TARGETURL,
            asD.MSGDIGEST,
            asD.MSGDIGESTALGORITHM,
            asD.EXTERNALMSGID,
            asD.DOCTIMESTAMP,
            asD.DISPOSITIONSTATUSCD,
            asD.INBOUNDVUID,
            asD.OUTBOUNDVUID,
            asD.ResponseServletURI,
--
            frpT.ownerFlg "FrPARTNEROwnerFlg" ,
            TopT.ownerFlg "ToPARTNEROwnerFlg",
            frp.PARTNERID "FRPARTNERID",
            Top.PARTNERID "TOPARTNERID"
        From
            BP_State_Hdr H
            Inner Join BP_AS_State_Hdr asH on asH.StateHdrID = H.StateHdrID
            Inner Join BP_AS_State_Dtl asD on asD.AS_StateDtlID = asH.AS_State_DtlID
            Left Outer Join pr_Busidentifier frBus on H.frBusinessid    = frBus.Businessid
            Left Outer Join pr_partner       frp   on frBus.partnerid   = frp.partnerid
            Left Outer Join pr_partnerType   frpT  on frp.PARTNERTYPEID = frpT.PARTNERTYPEID
            Left Outer Join pr_Busidentifier ToBus on H.ToBusinessid    = ToBus.Businessid
            Left Outer Join pr_partner       Top   on ToBus.partnerid   = Top.partnerid
            Left Outer Join pr_partnerType   TopT  on Top.PARTNERTYPEID = TopT.PARTNERTYPEID
            Left Outer Join pr_DestType      Dest  on Dest.DestTypeID   = H.DestTypeID
        Where
            H.RowId = v_RowId ;  --H.StateHdrId = v_StateHdrId;
    Else
        Null;

    End Case;

        EXCEPTION
        When NO_DATA_FOUND Then Null;
        When Others Then
            p_ReturnCd := 1;
            p_SqlErrm := SQLERRM;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);
/*
        Select
            Row_ID, StateHdrId
            INTO
            v_RowId, v_StateHdrId
        from
        (
        select /  FIRST_ROWS INDEX(H AK_STATEHDR_1) /
            rowid Row_ID, StateHdrId
        from
            BP_State_Hdr H
        WHERE
            UpdateDate > v_BeginTS - 100  -- Note: this search argument forces index/sort usage
            And WorkGroupName = p_WorkGroupName
            And StateInstanceName = NVL(p_StateInstanceName, -1)
        AND (    -- AND
                (StateStatusCd        = 1            -- Initial try
                And ExpectAckFLG      = 1
                )
            Or
                (StateStatusCd        = 3             -- Retry and Max retries reached
                And ProcessRetryDATE  <=  v_BeginTS
                And ExpectAckFLG      =  1 And AckRcvFLG = 0
                )
            Or
                (StateStatusCd = 3             -- Response  TTP Expired
                And ExpectRespDocFLG = 1 And ResponseRcvFLG  = 0
                And ProcessExpireDATE <= v_BeginTS
                )
            )-- AND
        ) T1
        Where ROWNUM < 2                --  (p_BatchSize + 1)
        FOR UPDATE Skip Locked;
*/

End;
/


-- BP_STATE_HDRU_AH2.sql

CREATE OR REPLACE PROCEDURE  BP_STATE_HDRU_AH2
 (
    p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_SQLERRM out varchar2
    ,p_WorkGroupName in Varchar2
    ,p_STATEHDRID in NUMBER
    ,p_STATEInstanceName in Varchar2
    ,p_CHANNELID in NUMBER
    ,p_STATESTATUSCD in pls_integer
    ,p_RESTARTFLG in NUMBER
    ,p_PROCESSINSTANCEID in VARCHAR2
    ,p_PROCESSEXPIREDATE in Timestamp
    ,p_PROCESSSTARTDATE in Timestamp
    ,p_INITBUSINESSID in VARCHAR2
    ,p_FRBUSINESSID in VARCHAR2
    ,p_TOBUSINESSID in VARCHAR2
    ,p_CURRENTSTATECD in VARCHAR2
    ,p_LASTSTATECD in VARCHAR2
    ,p_NEXTACTIONCD in VARCHAR2
    ,p_MSGSENTSTATUSCD in VARCHAR2
    ,p_RETRYCNT in NUMBER
    ,p_TPA_MAXDOCRETRYCNT in NUMBER
    ,p_TPA_TIMETOACK in NUMBER
    ,p_TPA_TIMETOPERFORM in NUMBER
    ,p_NEXTRETRYDATE in Timestamp
    ,p_EXPECTACKFLG in NUMBER
    ,p_ACKRCVFLG in NUMBER
    ,p_ACKSENTFLG in NUMBER
    ,p_OUTOFSEQFLG in NUMBER
    ,p_EXPECTRESPDOCFLG in NUMBER
    ,p_PROCESSRETRYDATE in Timestamp
    ,p_RESPONSERCVFLG in NUMBER
    ,p_STATESCENARIOID in NUMBER
    ,p_SCENARIOREQIREDFLG in NUMBER
    ,p_BATCHCNT in NUMBER
    ,p_BATCHDOCPROCESSEDCNT in NUMBER
    ,p_DESTName in VARCHAR2
    ,p_UPDATEPGMNAME in VARCHAR2
    ,p_ERRORCD in VARCHAR2
    ,p_ERRORREASONTEXT in VARCHAR2
) as

/*
===============================================================================
 Proc Name:       BP_STATE_HDRU_AH2
 Create Date:     2003-03-11 09:42:06
 Author:          SYSTEM

                    ::::::::::::::::::::::::::::::
                          M O D I F I C A T I O N
                    ::::::::::::::::::::::::::::::

===============================================================================
 Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar(30) := 'BP_STATE_HDRU_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar(4000) :=
  'p_DebugFlg=>'||p_DebugFlg||CHR(10)
  ||', p_WorkGroupName=>'||p_WorkGroupName||CHR(10)
  ||', p_STATEHDRID=>'||p_STATEHDRID||CHR(10)
  ||', p_STATEInstanceName=>'||p_STATEInstanceName||CHR(10)
  ||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
  ||', p_STATESTATUSCD=>'||p_STATESTATUSCD||CHR(10)
  ||', p_RESTARTFLG=>'||p_RESTARTFLG||CHR(10)
  ||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
  ||', p_PROCESSEXPIREDATE=>'||p_PROCESSEXPIREDATE||CHR(10)
  ||', p_PROCESSSTARTDATE=>'||p_PROCESSSTARTDATE||CHR(10)
  ||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
  ||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
  ||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
  ||', p_CURRENTSTATECD=>'||p_CURRENTSTATECD||CHR(10)
  ||', p_LASTSTATECD=>'||p_LASTSTATECD||CHR(10)
  ||', p_NEXTACTIONCD=>'||p_NEXTACTIONCD||CHR(10)
  ||', p_MSGSENTSTATUSCD=>'||p_MSGSENTSTATUSCD||CHR(10)
  ||', p_RETRYCNT=>'||p_RETRYCNT||CHR(10)
  ||', p_TPA_MAXDOCRETRYCNT=>'||p_TPA_MAXDOCRETRYCNT||CHR(10)
  ||', p_TPA_TIMETOACK=>'||p_TPA_TIMETOACK||CHR(10)
  ||', p_TPA_TIMETOPERFORM=>'||p_TPA_TIMETOPERFORM||CHR(10)
  ||', p_NEXTRETRYDATE=>'||p_NEXTRETRYDATE||CHR(10)
  ||', p_EXPECTACKFLG=>'||p_EXPECTACKFLG||CHR(10)
  ||', p_ACKRCVFLG=>'||p_ACKRCVFLG||CHR(10)
  ||', p_ACKSENTFLG=>'||p_ACKSENTFLG||CHR(10)
  ||', p_OUTOFSEQFLG=>'||p_OUTOFSEQFLG||CHR(10)
  ||', p_EXPECTRESPDOCFLG=>'||p_EXPECTRESPDOCFLG||CHR(10)
  ||', p_PROCESSRETRYDATE=>'||p_PROCESSRETRYDATE||CHR(10)
  ||', p_RESPONSERCVFLG=>'||p_RESPONSERCVFLG||CHR(10)
  ||', p_STATESCENARIOID=>'||p_STATESCENARIOID||CHR(10)
  ||', p_SCENARIOREQIREDFLG=>'||p_SCENARIOREQIREDFLG||CHR(10)
  ||', p_BATCHCNT=>'||p_BATCHCNT||CHR(10)
  ||', p_BATCHDOCPROCESSEDCNT=>'||p_BATCHDOCPROCESSEDCNT||CHR(10)
  ||', p_DESTNAME=>'||p_DESTNAME||CHR(10)
  ||', p_UPDATEPGMNAME=>'||p_UPDATEPGMNAME||CHR(10)
  ||', p_ERRORCD=>'||p_ERRORCD||CHR(10)
  ||', p_ERRORREASONTEXT=>'||p_ERRORREASONTEXT||CHR(10);

v_RowTS      integer :=0;
zeroUpdate  Exception;
-- State Hdr
v_CurrentStateCd        BP_STATE_HDR.CurrentStateCd%Type;
v_TPA_MaxDocRetryCnt    BP_STATE_HDR.TPA_MaxDocRetryCnt%Type;
v_RetryCnt              BP_STATE_HDR.RetryCnt%Type;
v_MsgSentStatusCd       BP_STATE_HDR.MsgSentStatusCd%Type;
v_BatchDocProcessedCnt  BP_STATE_HDR.BatchDocProcessedCnt%Type;
v_TPA_TIMETOACK         BP_STATE_HDR.TPA_TIMETOACK%Type;
--Detail
v_MsgDigest             BP_AS_State_Dtl.MsgDigest%Type;
v_StateHdrId            pls_integer;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    If p_WorkGroupName = 'AS' Then
        IF p_STATEHDRID is Null Then
            Select STATEHDRID into v_StateHdrId
            From BP_State_Hdr
            Where InitBusinessId    = p_INITBUSINESSID
            And   ProcessInstanceId = p_PROCESSINSTANCEID
            AND   STATEInstanceName = p_STATEInstanceName
            AND   WorkGroupName     = 'AS';
        Else
            v_StateHdrId  := p_StateHdrId ;
        End If;
    Else
        v_StateHdrId  := p_StateHdrId ;
    End IF;
    Update BP_STATE_HDR
    SET
        CHANNELID=nvl(p_CHANNELID, CHANNELID)
        ,      STATESTATUSCD        = nvl(p_STATESTATUSCD, STATESTATUSCD)

        ,      PROCESSEXPIREDATE    = nvl(p_PROCESSEXPIREDATE, PROCESSEXPIREDATE)
        ,      PROCESSSTARTDATE     = nvl(p_PROCESSSTARTDATE, PROCESSSTARTDATE)

        ,      FRBUSINESSID         = nvl(p_FRBUSINESSID, FRBUSINESSID)
        ,      TOBUSINESSID         = nvl(p_TOBUSINESSID, TOBUSINESSID)
        ,      CURRENTSTATECD       = nvl(p_CURRENTSTATECD, CURRENTSTATECD)
        ,      LASTSTATECD          = nvl(p_LASTSTATECD, LASTSTATECD)
        ,      NEXTACTIONCD         = nvl(p_NEXTACTIONCD, NEXTACTIONCD)
        ,      MSGSENTSTATUSCD      = nvl(p_MSGSENTSTATUSCD, MSGSENTSTATUSCD)
        ,      RETRYCNT             = nvl(p_RETRYCNT, RETRYCNT)
        ,      TPA_MAXDOCRETRYCNT   = nvl(p_TPA_MAXDOCRETRYCNT, TPA_MAXDOCRETRYCNT)
        ,      TPA_TIMETOACK        = nvl(p_TPA_TIMETOACK, TPA_TIMETOACK)
        ,      TPA_TIMETOPERFORM    = nvl(p_TPA_TIMETOPERFORM, TPA_TIMETOPERFORM)
        ,      NEXTRETRYDATE        = nvl(p_NEXTRETRYDATE, NEXTRETRYDATE)
        ,      EXPECTACKFLG         = nvl(p_EXPECTACKFLG, EXPECTACKFLG)
        ,      ACKRCVFLG            = nvl(p_ACKRCVFLG, ACKRCVFLG)
        ,      ACKSENTFLG           = nvl(p_ACKSENTFLG, ACKSENTFLG)
        ,      OUTOFSEQFLG          = nvl(p_OUTOFSEQFLG, OUTOFSEQFLG)
        ,      EXPECTRESPDOCFLG     = nvl(p_EXPECTRESPDOCFLG, EXPECTRESPDOCFLG)
        ,      PROCESSRETRYDATE     = nvl(p_PROCESSRETRYDATE, PROCESSRETRYDATE)
        ,      RESPONSERCVFLG       = nvl(p_RESPONSERCVFLG, RESPONSERCVFLG)
        ,      STATESCENARIOID      = nvl(p_STATESCENARIOID, STATESCENARIOID)
        ,      SCENARIOREQIREDFLG   = nvl(p_SCENARIOREQIREDFLG, SCENARIOREQIREDFLG)
        ,      BATCHCNT             = nvl(p_BATCHCNT, BATCHCNT)
        ,      BATCHDOCPROCESSEDCNT = nvl(p_BATCHDOCPROCESSEDCNT, 0) + BATCHDOCPROCESSEDCNT
        ,      UPDATEPGMNAME        = nvl(p_UPDATEPGMNAME, UPDATEPGMNAME)
        ,      ERRORCD              = nvl(p_ERRORCD, ERRORCD)
        ,      ERRORREASONTEXT      = nvl(p_ERRORREASONTEXT, ERRORREASONTEXT)
        ,      ROWTS                = RowTS + 1
        ,      UPDATEDATE           = v_BeginTS
    Where
        STATEHDRID = v_STATEHDRID
    Returning
        CurrentStateCd,
        TPA_MaxDocRetryCnt,
        RetryCnt,
        MsgSentStatusCd,
        BatchDocProcessedCnt,
        TPA_TIMETOACK
    Into
        v_CurrentStateCd,
        v_TPA_MaxDocRetryCnt,
        v_RetryCnt,
        v_MsgSentStatusCd,
        v_BatchDocProcessedCnt,
        v_TPA_TIMETOACK;

    v_RowCnt := SQL%rowcount;
    if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
    If RC1%IsOpen Then Close RC1; End If;
    Open RC1 For
    Select
        v_CurrentStateCd       CurrentStateCd,
        v_TPA_MaxDocRetryCnt   TPA_MaxDocRetryCnt,
        v_RetryCnt             RetryCnt,
        v_MsgSentStatusCd      MsgSentStatusCd,
        v_BatchDocProcessedCnt BatchDocProcessedCnt,
        v_TPA_TIMETOACK        TPA_TIMETOACK
    From  Dual;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);

    EXCEPTION
        WHEN TOO_MANY_ROWS Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt,p_DebugFlg, 1);
        WHEN ZeroUpdate then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt,p_DebugFlg, 1);
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            p_SQLERRM := SQLERRM;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);

End;
/


-- BP_STATE_HDRU_AH3.sql

CREATE OR REPLACE PROCEDURE  BP_STATE_HDRU_AH3
 (
    p_ReturnCd out integer
    ,p_DebugFlg in pls_integer
    --,RC1 out db_ref.RCT
    ,p_WorkGroupName in Varchar2
    ,p_STATEInstanceName in Varchar2
    ,p_STATESTATUSCD in pls_integer
    ,p_PROCESSINSTANCEID in VARCHAR2
    ,p_INITBUSINESSID in VARCHAR2
    ,p_EXPECTACKFLG in NUMBER
    ) as

/*
===============================================================================
 Proc Name:       BP_STATE_HDRU_AH3
 Create Date:     2007-08-17 16:24:06
 Author:          Amit Kumar Vijay

                    ::::::::::::::::::::::::::::::
                          M O D I F I C A T I O N
                    ::::::::::::::::::::::::::::::

===============================================================================
 Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar(30) := 'BP_STATE_HDRU_AH3';
v_ExceptionFlg    integer := 0;
p_SQLERRM 		varchar(4000);
v_ParamsTxt       varchar(4000) :=
  'p_DebugFlg=>'||p_DebugFlg||CHR(10)
  ||', p_WorkGroupName=>'||p_WorkGroupName||CHR(10)
  ||', p_STATEInstanceName=>'||p_STATEInstanceName||CHR(10)
  ||', p_STATESTATUSCD=>'||p_STATESTATUSCD||CHR(10)
  ||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
  ||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
  ||', p_EXPECTACKFLG=>'||p_EXPECTACKFLG||CHR(10);

  
v_RowTS      integer :=0;
zeroUpdate  Exception;
-- State Hdr

--Detail
v_StateHdrId            pls_integer;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    Update BP_STATE_HDR
    SET
        STATESTATUSCD        = nvl(p_STATESTATUSCD, STATESTATUSCD)
        ,      EXPECTACKFLG         = nvl(p_EXPECTACKFLG, EXPECTACKFLG)
        ,      PROCESSRETRYDATE     = v_BeginTS + (TPA_TimeToAck/1440)
        ,      ROWTS                = RowTS + 1
        ,      UPDATEDATE           = v_BeginTS
    Where
         InitBusinessId    = p_INITBUSINESSID
         And    ProcessInstanceId = p_PROCESSINSTANCEID
        AND	STATEInstanceName=p_STATEInstanceName
        AND	WorkGroupName='AS';
    
    v_RowCnt := SQL%rowcount;
    if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
    
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);

    EXCEPTION
        WHEN TOO_MANY_ROWS Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt,p_DebugFlg, 1);
        WHEN ZeroUpdate then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt,p_DebugFlg, 1);
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            p_SQLERRM := SQLERRM;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);

End;
/


-- BP_STATE_HDR_QUEI_AH1.sql

CREATE OR REPLACE PROCEDURE BP_STATE_HDR_QUEI_AH1
        (
        p_ReturnCd              out pls_integer
        , p_RowCnt              out pls_integer
        , p_WorkGroupName       in  VARCHAR2
        , p_StateInstanceName   in  VARCHAR2
        )
AS

/*
        ================================================================================
        Proc Name:       BP_STATE_HDR_QUEI_AH1
        Create Date:     2003-11-10 10:11:25
        Author:          V.Goel

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-11-10 SYSTEM           3.5 Created
        

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'BP_STATE_HDR_QUEI_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_RowCnt=>'||p_RowCnt||CHR(10)
                ||',p_WorkGroupName=>'||p_WorkGroupName||CHR(10)
                ||',p_StateInstanceName =>'||p_StateInstanceName||CHR(10)
                ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    LOCK TABLE BP_State_Hdr_Que IN EXCLUSIVE MODE;
    
    INSERT into BP_State_Hdr_Que (UPDATEDATE, ROW_ID)
        select UpdateDate,rowId
        from BP_State_Hdr H
        WHERE 
        WorkGroupName = p_WorkGroupName
                    And StateInstanceName = p_StateInstanceName
                    AND 
                        (StateStatusCd        = 3             -- Retry and Max retries reached
                         And ProcessRetryDATE  <=  v_BeginTS
                         And ExpectAckFLG      =  1 And AckRcvFLG = 0
                        )
        ;
    p_RowCnt := sql%RowCount;   
    COMMIT;

        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, 0, v_ExceptionFlg);
        EXCEPTION
                When NO_DATA_FOUND THEN 
                    Null;
                When DUP_VAL_ON_INDEX then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, 0, v_ExceptionFlg);
                        p_returnCd := 2;
---                        RAISE_APPLICATION_ERROR (-20999,'20001 BP_STATE_HDR_QUEI_AH1' || SQLCODE|| ' '||SQLERRM);
                When OTHERS Then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, 0, 1);
                        RAISE_APPLICATION_ERROR (-20999,'20002 BP_STATE_HDR_QUEI_AH1' || SQLCODE|| ' '||SQLERRM);
END BP_STATE_HDR_QUEI_AH1;
/


-- CF_ARCHIVER_CONFIGS.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_CONFIGS
	(
	p_ReturnCd  OUT  Integer
	, p_DebugFlg IN  Integer
	, RC1  OUT  DB_REF.RCT
	, p_CONFIGNAME  IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:	CF_ARCHIVER_CONFIGS
	Create Date:	2008-07-25 15:52:09
	Author:		SYSTEM

			  ::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
			  ::::::::::::::::::::::::::::::

	---------- ------------ -------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------ -------	---------------------------------------------
	2008-07-25 SYSTEM	6.1.1.x	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			M    A    I    N			     --
---------------------------------------------------------------------------

	v_RowCnt	integer Default 0;
	v_BeginTS	TimeStamp  ; -- Performance related variables
	v_ProcName	varchar2(30)  Default 'CF_ARCHIVER_CONFIGS';
	v_ExceptionFlg  integer  Default 0;
	v_ExceptionCode varchar2(30);
	v_PK_Value	Integer Default 0;
	v_SQLErrm	varchar2(255) ;
	v_RowTS		Integer Default 0;
	
	v_ParamsTxt       varchar2(4000) Default
					'p_DebugFlg=>'||p_DebugFlg||CHR(10)
					||', p_CONFIGNAME=>'||p_CONFIGNAME||CHR(10)
		;
	
BEGIN
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	IF p_CONFIGNAME IS NOT NULL THEN
		OPEN RC1 For
			SELECT
				CONFIGID
				,STATUSFLAG
				,KEEP_NUMDAYS
				,ARCHIVEFLAG
				,SCHED_TYPE
				,RTRIM(DAYOFMONTH) DAYOFMONTH
				,RTRIM(DAYOFWEEK) DAYOFWEEK
				,RTRIM(HOUROPT) HOUROPT
				,RTRIM(MINUTEOPT) MINUTEOPT
				,FREQDAYOFMONTH
				,FREQDAYOFWEEK
				,ROWTS
				,SCHEDULERTASKID
				,RTRIM(CONFIGNAME) CONFIGNAME
				,RTRIM(CONFIGDESC) CONFIGDESC
				,RTRIM(ARCHDEST_FL) ARCHDEST_FL
				,RTRIM(ARCHDEST_DB) ARCHDEST_DB
				,LASTEXECTS
				,NEXTEXECTS
				,CREATEDATE
				,RTRIM(RESTORE_FRDATE) RESTORE_FRDATE
				,RTRIM(RESTORE_TODATE) RESTORE_TODATE
			FROM	CF_ARCHIVER_CONFIG
		WHERE	CONFIGNAME = p_CONFIGNAME
		;
	ELSE
		OPEN RC1 For
			SELECT
				CONFIGID
				,STATUSFLAG
				,KEEP_NUMDAYS
				,ARCHIVEFLAG
				,SCHED_TYPE
				,RTRIM(DAYOFMONTH) DAYOFMONTH
				,RTRIM(DAYOFWEEK) DAYOFWEEK
				,RTRIM(HOUROPT) HOUROPT
				,RTRIM(MINUTEOPT) MINUTEOPT
				,FREQDAYOFMONTH
				,FREQDAYOFWEEK
				,ROWTS
				,SCHEDULERTASKID
				,RTRIM(CONFIGNAME) CONFIGNAME
				,RTRIM(CONFIGDESC) CONFIGDESC
				,RTRIM(ARCHDEST_FL) ARCHDEST_FL
				,RTRIM(ARCHDEST_DB) ARCHDEST_DB
				,LASTEXECTS
				,NEXTEXECTS
				,CREATEDATE
				,RTRIM(RESTORE_FRDATE) RESTORE_FRDATE
				,RTRIM(RESTORE_TODATE) RESTORE_TODATE
			FROM	CF_ARCHIVER_CONFIG
		;
	END IF;

	p_ReturnCD :=0;

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

EXCEPTION
	WHEN OTHERS THEN
		RAISE_APPLICATION_ERROR(-20991, SQLERRM);
END CF_ARCHIVER_CONFIGS;
/


-- CF_ARCHIVER_CONFIGS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_CONFIGS_AH1
	(
	p_ReturnCd  OUT  Integer
	, p_DebugFlg IN  Integer
	, RC1  OUT  DB_REF.RCT
	, p_CONFIGNAME  IN VARCHAR2
	, p_LOCKFLAG	IN	INTEGER
	)
AS

/*
	================================================================================
	Proc Name:	CF_ARCHIVER_CONFIGS_AH1
	Create Date:	2008-07-25 15:52:09
	Author:		SYSTEM

			  ::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
			  ::::::::::::::::::::::::::::::

	---------- ------------ -------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------ -------	---------------------------------------------
	2008-07-25 SYSTEM	6.1.1.x	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			M    A    I    N			     --
---------------------------------------------------------------------------

	v_RowCnt	integer Default 0;
	v_BeginTS	TimeStamp  ; -- Performance related variables
	v_ProcName	varchar2(30)  Default 'CF_ARCHIVER_CONFIGS_AH1';
	v_ExceptionFlg  integer  Default 0;
	v_ExceptionCode varchar2(30);
	v_PK_Value	Integer Default 0;
	v_SQLErrm	varchar2(255) ;
	v_RowTS		Integer Default 0;
	
	v_ParamsTxt       varchar2(4000) Default
					'p_DebugFlg=>'||p_DebugFlg||CHR(10)
					||', p_CONFIGNAME=>'||p_CONFIGNAME||CHR(10)
		;
	
BEGIN
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	--- lockFlag = 1 (locked) , 0 (unlocked)
	--- request to lock
	IF p_LOCKFLAG IS NOT NULL THEN
		UPDATE	cf_archiver_config
		SET	lockflag = p_lockflag
		WHERE	lockflag <> p_lockflag
		AND	configname = p_configname;
		COMMIT;
		IF p_ReturnCd = 3 THEN
			--- can not lock/unlock
			p_ReturnCd := 4;
		END IF;
	END IF;
	
	IF p_CONFIGNAME IS NOT NULL THEN
		OPEN RC1 For
			SELECT
				CONFIGID
				,STATUSFLAG
				,KEEP_NUMDAYS
				,ARCHIVEFLAG
				,SCHED_TYPE
				,RTRIM(DAYOFMONTH) DAYOFMONTH
				,RTRIM(DAYOFWEEK) DAYOFWEEK
				,RTRIM(HOUROPT) HOUROPT
				,RTRIM(MINUTEOPT) MINUTEOPT
				,FREQDAYOFMONTH
				,FREQDAYOFWEEK
				,LOCKFLAG
				,ROWTS
				,SCHEDULERTASKID
				,RTRIM(CONFIGNAME) CONFIGNAME
				,RTRIM(CONFIGDESC) CONFIGDESC
				,RTRIM(ARCHDEST_FL) ARCHDEST_FL
				,RTRIM(ARCHDEST_DB) ARCHDEST_DB
				,LASTEXECTS
				,NEXTEXECTS
				,CREATEDATE
				,RTRIM(RESTORE_FRDATE) RESTORE_FRDATE
				,RTRIM(RESTORE_TODATE) RESTORE_TODATE
			FROM	CF_ARCHIVER_CONFIG
		WHERE	CONFIGNAME = p_CONFIGNAME
		;
	ELSE
		OPEN RC1 For
			SELECT
				CONFIGID
				,STATUSFLAG
				,KEEP_NUMDAYS
				,ARCHIVEFLAG
				,SCHED_TYPE
				,RTRIM(DAYOFMONTH) DAYOFMONTH
				,RTRIM(DAYOFWEEK) DAYOFWEEK
				,RTRIM(HOUROPT) HOUROPT
				,RTRIM(MINUTEOPT) MINUTEOPT
				,FREQDAYOFMONTH
				,FREQDAYOFWEEK
				,LOCKFLAG
				,ROWTS
				,SCHEDULERTASKID
				,RTRIM(CONFIGNAME) CONFIGNAME
				,RTRIM(CONFIGDESC) CONFIGDESC
				,RTRIM(ARCHDEST_FL) ARCHDEST_FL
				,RTRIM(ARCHDEST_DB) ARCHDEST_DB
				,LASTEXECTS
				,NEXTEXECTS
				,CREATEDATE
				,RTRIM(RESTORE_FRDATE) RESTORE_FRDATE
				,RTRIM(RESTORE_TODATE) RESTORE_TODATE
			FROM	CF_ARCHIVER_CONFIG
		;
	END IF;

	p_ReturnCD :=0;

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

EXCEPTION
	WHEN OTHERS THEN
		RAISE_APPLICATION_ERROR(-20991, SQLERRM);
END CF_ARCHIVER_CONFIGS_AH1;
/


-- CF_ARCHIVER_CONFIGU.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_CONFIGU
(
	 p_ReturnCd  out integer
	,p_DebugFlg in Integer
	,RC1 out db_ref.RCT
	, p_CONFIGID in INTEGER
	, p_STATUSFLAG in PLS_INTEGER
	, p_KEEP_NUMDAYS  in PLS_INTEGER
	, p_ARCHIVEFLAG in PLS_INTEGER
	, p_SCHED_TYPE in PLS_INTEGER
	, p_DAYOFMONTH in VARCHAR2
	, p_DAYOFWEEK in VARCHAR2
	, p_HOUROPT in VARCHAR2
	, p_MINUTEOPT in VARCHAR2
	, p_FREQDAYOFMONTH in PLS_INTEGER
	, p_FREQDAYOFWEEK in PLS_INTEGER
	, p_ROWTS in INTEGER
	, p_SCHEDULERTASKID in VARCHAR2
	, p_CONFIGNAME in VARCHAR2 
	, p_CONFIGDESC in VARCHAR2 
	, p_ARCHDEST_FL in VARCHAR2 
	, p_ARCHDEST_DB in VARCHAR 
	, p_LASTEXECTS in TIMESTAMP
	, p_NEXTEXECTS in TIMESTAMP
	, p_RESTORE_FRDATE in VARCHAR2
	, p_RESTORE_TODATE in VARCHAR2
)
AS
/*
	================================================================================
	Proc Name:	CF_ARCHIVER_CONFIGU
	Create Date:     2008-07-25 15:50:17
	Author:	   SYSTEM

			     ::::::::::::::::::::::::::::::
				    M O D I F I C A T I O N
			     ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	   NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-25 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			    M    A    I    N				  --
---------------------------------------------------------------------------

	 v_RowCnt	   integer Default 0;
	 v_BeginTS	  TimeStamp  ; -- Performance related variables
	 v_ProcName	 VARCHAR2(30)  Default 'CF_ARCHIVER_CONFIGU';
	 v_ExceptionFlg    integer  Default 0;
	 v_ExceptionCode   VARCHAR2(30);
	 v_PK_Value	 Integer Default 0;
	 v_SQLErrm	  VARCHAR2(255) ;
	 v_chkpt	    Char(1) default '0';
	 v_msg	      VARCHAR2(500);
	 v_RowTS	    Integer Default 0;
	 ZEROUPDATE EXCEPTION;
	
	 v_ParamsTxt       varchar2(4000) Default
		 			 	'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		 			 	||', p_CONFIGID=>'||p_CONFIGID||CHR(10)
		 			 	||', p_STATUSFLAG=>'||p_STATUSFLAG||CHR(10)
		 			 	||', p_KEEP_NUMDAYS=>'||p_KEEP_NUMDAYS||CHR(10)
		 			 	||', p_ARCHIVEFLAG=>'||p_ARCHIVEFLAG||CHR(10)
		 			 	||', p_SCHED_TYPE=>'||p_SCHED_TYPE||CHR(10)
		 			 	||', p_DAYOFMONTH=>'||p_DAYOFMONTH||CHR(10)
		 			 	||', p_DAYOFWEEK=>'||p_DAYOFWEEK||CHR(10)
		 			 	||', p_HOUROPT=>'||p_HOUROPT||CHR(10)
		 			 	||', p_MINUTEOPT=>'||p_MINUTEOPT||CHR(10)
		 			 	||', p_FREQDAYOFMONTH=>'||p_FREQDAYOFMONTH||CHR(10)
		 			 	||', p_FREQDAYOFWEEK=>'||p_FREQDAYOFWEEK||CHR(10)
		 			 	||', p_SCHEDULERTASKID=>'||p_SCHEDULERTASKID||CHR(10)
		 			 	||', p_CONFIGNAME=>'||p_CONFIGNAME||CHR(10)
		 			 	||', p_CONFIGDESC=>'||p_CONFIGDESC||CHR(10)
		 			 	||', p_ARCHDEST_FL=>'||p_ARCHDEST_FL||CHR(10)
		 			 	||', p_ARCHDEST_DB=>'||p_ARCHDEST_DB||CHR(10)
		 			 	||', p_LASTEXECTS=>'||p_LASTEXECTS||CHR(10)
		 			 	||', p_NEXTEXECTS=>'||p_NEXTEXECTS||CHR(10)
		 			 	||', p_RESTORE_FRDATE=>'||p_RESTORE_FRDATE||CHR(10)
		 			 	||', p_RESTORE_TODATE=>'||p_RESTORE_TODATE||CHR(10)
	 		;
	
	
	BEGIN
	
	UPDATE	CF_ARCHIVER_CONFIG ux
	SET	ux.ROWTS = p_RowTS + 1
		, ux.STATUSFLAG = COALESCE( p_STATUSFLAG, ux.STATUSFLAG)
		, ux.KEEP_NUMDAYS = COALESCE( p_KEEP_NUMDAYS, ux.KEEP_NUMDAYS)
		, ux.ARCHIVEFLAG = COALESCE( p_ARCHIVEFLAG, ux.ARCHIVEFLAG)
		, ux.SCHED_TYPE = COALESCE( p_SCHED_TYPE, ux.SCHED_TYPE)
		, ux.DAYOFMONTH = p_DAYOFMONTH
		, ux.DAYOFWEEK = p_DAYOFWEEK
		, ux.HOUROPT = p_HOUROPT
		, ux.MINUTEOPT = p_MINUTEOPT
		, ux.FREQDAYOFMONTH = p_FREQDAYOFMONTH
		, ux.FREQDAYOFWEEK = p_FREQDAYOFWEEK
		, ux.SCHEDULERTASKID = COALESCE( p_SCHEDULERTASKID, ux.SCHEDULERTASKID)
		, ux.CONFIGNAME = COALESCE( p_CONFIGNAME, ux.CONFIGNAME)
		, ux.CONFIGDESC = COALESCE( p_CONFIGDESC, ux.CONFIGDESC)
		, ux.ARCHDEST_FL = p_ARCHDEST_FL
		, ux.ARCHDEST_DB = p_ARCHDEST_DB
		, ux.LASTEXECTS = COALESCE( p_LASTEXECTS, ux.LASTEXECTS)
		, ux.NEXTEXECTS = COALESCE( p_NEXTEXECTS, ux.NEXTEXECTS)
		, ux.RESTORE_FRDATE = p_RESTORE_FRDATE
		, ux.RESTORE_TODATE = p_RESTORE_TODATE
		, ux.UPDATEDATE= v_BeginTS
	WHERE	CONFIGID = p_CONFIGID
	AND	RowTS = p_RowTS ;

	v_rowcnt := SQL%rowcount;
	IF (v_rowcnt = 0) then  raise ZeroUpdate ; end if;

	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select v_rowTS ROWTS From Dual;

	p_ReturnCd := 0;

	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

EXCEPTION
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
		RAISE_APPLICATION_ERROR (-20999,'20001 CF_ARCHIVER_CONFIGU' || SQLCODE|| ' '||SQLERRM);
	
	When ZEROUPDATE Then
		p_ReturnCd := 3;
		raise_application_error(-20999,'20101 - CF_ARCHIVER_CONFIGU: ZERO RECORDS UPDATED ');

End CF_ARCHIVER_CONFIGU;
/


-- CF_ARCHIVER_CONFIGU_AH1.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_CONFIGU_AH1
(
	 	p_ReturnCd	out	integer
	,	p_DebugFlg	in	Integer
	,       RC1             out     db_ref.RCT
	,	p_STATUSFLAG	in	SMALLINT
	,	p_SCHEDULERTASKID in	VARCHAR2
	,	p_TASKID	in	INTEGER
	,	p_LASTEXECTS	in	TIMESTAMP
	,	p_NEXTEXECTS	in	TIMESTAMP
	,	p_TASKENDTS	in	TIMESTAMP
)
AS

/*
	================================================================================
	Proc Name:	CF_ARCHIVER_CONFIGU_AH1
	Create Date:     2008-07-25 15:50:17
	Author:	   S.Priani
	Description: Change status of archiver based on value of p_StatusFlag
			0 = SCHEDULED		*
			1 = BEING SCHEDULED
			2 = RUNNING		*
			3 = STOPPING
			4 = DISABLED_FAILED	*
			5 = DISABLING
			6 = DISABLED		*
			7 = STOPPED		*
			8 = SUCCESSFUL		*
		* -> possible input values from server
		If 0 THEN if p_taskid is null then create new cfg
		If 2 THEN INSERT INTO LG_ARCH_TASK
		If 4 THEN UPDATE in LG_ARCH_TASK(2 to 4) for given TASKID
		If 6 THEN UPDATE in LG_ARCH_TASK(2 to 7) for given TASKID
		If 7 THEN UPDATE in LG_ARCH_TASK(3 to 7) for given TASKID
		If 8 THEN UPDATE in LG_ARCH_TASK(2 to 8) for given TASKID

			     ::::::::::::::::::::::::::::::
				    M O D I F I C A T I O N
			     ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	   NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-08-05 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			    M    A    I    N				  --
---------------------------------------------------------------------------

	v_RowCnt		integer Default 0;
	v_BeginTS		TimeStamp  ; -- Performance related variables
	v_ProcName		varchar2(30)  Default 'CF_ARCHIVER_CONFIGU_AH1';
	v_ExceptionFlg		integer  Default 0;
	v_ExceptionCode 	varchar2(30);
	v_PK_Value		Integer Default 0;
	v_SQLErrm	 	varchar2(255) ;
	v_chkpt			Char(1) default '0';
	v_sch_sts_SCHEDULED	INTEGER	 DEFAULT 0;
	v_sch_sts_SCHEDULING	INTEGER	 DEFAULT 1;
	v_sch_sts_RUNNING	INTEGER	 DEFAULT 2;
	v_sch_sts_STOPPING	INTEGER	 DEFAULT 3;
	v_sch_sts_FAILED	INTEGER	 DEFAULT 4;
	v_sch_sts_DISABLING	INTEGER	 DEFAULT 5;
	v_sch_sts_DISABLED	INTEGER	 DEFAULT 6;
	v_sch_sts_STOPPED	INTEGER	 DEFAULT 7;
	v_sch_sts_SUCCESS	INTEGER	 DEFAULT 8;

	v_RowTS			Integer Default 0;
	v_NewTaskId		INTEGER;
	v_ArchCfgID		INTEGER;
	v_SchedulerTaskID	varchar2(60);
	NO_CONFIG_ERROR		EXCEPTION;
		
	v_ParamsTxt		varchar2(4000) Default
	 			 	'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 			 	||', p_STATUSFLAG=>'||p_STATUSFLAG||CHR(10)
	 			 	||', p_SCHEDULERTASKID=>'||p_SCHEDULERTASKID||CHR(10)
	 			 	||', p_TASKID=>'||p_TASKID||CHR(10)
	 			 	||', p_LASTEXECTS=>'||p_LASTEXECTS||CHR(10)
	 			 	||', p_NEXTEXECTS=>'||p_NEXTEXECTS||CHR(10)
	 			 	||', p_TASKENDTS=>'||p_TASKENDTS||CHR(10)
	;		
		
	Cursor cur_cfg  IS
		SELECT	ArchCfgID 
		FROM	lg_arch_cfg
		WHERE	schedulertaskId = v_SchedulerTaskId 
		ORDER BY archcfgid DESC
	;
BEGIN 
	UPDATE	CF_ARCHIVER_CONFIG ux
	SET	ux.ROWTS = ux.RowTS + 1
		, ux.STATUSFLAG = CASE
					WHEN p_StatusFlag IN (v_sch_sts_SCHEDULED, v_sch_sts_STOPPED, v_sch_sts_SUCCESS)
						THEN v_sch_sts_SCHEDULED
					WHEN p_StatusFlag IN (v_sch_sts_RUNNING, v_sch_sts_FAILED, v_sch_sts_DISABLED)
						THEN p_StatusFlag
					ELSE
						ux.StatusFlag
				END
		, ux.SCHEDULERTASKID = COALESCE( p_SCHEDULERTASKID, ux.SCHEDULERTASKID)
		, ux.LASTEXECTS = COALESCE( p_LASTEXECTS, ux.LASTEXECTS)
		, ux.NEXTEXECTS = COALESCE( p_NEXTEXECTS, ux.NEXTEXECTS)
		, ux.UPDATEDATE= v_BeginTS
	WHERE	ux.CONFIGNAME = 'ARCHIVER'
	RETURNING 	schedulertaskid INTO	v_schedulertaskid
	;

	IF p_TASKID IS NULL THEN
		IF p_StatusFlag = v_sch_sts_SCHEDULED THEN
			v_chkpt:='2';
			-- get new archiver config for scheduling a new task
			FOR rec_cfg IN 
				(SELECT	*
				FROM	CF_ARCHIVER_CONFIG WHERE configname = 'ARCHIVER')
				LOOP
					v_chkpt:='3';
					INSERT INTO LG_ARCH_CFG(
						ARCHCFGID
						,SCHEDULERTASKID
						,STATUSFLAG
						,KEEP_NUMDAYS
						,ARCHIVEFLAG
						,SCHED_TYPE
						,DAYOFMONTH
						,DAYOFWEEK
						,HOUROPT
						,MINUTEOPT
						,FREQDAYOFMONTH
						,FREQDAYOFWEEK
						,ARCHDEST_FL
						,ARCHDEST_DB
						,CREATEDATE
					) VALUES (
						LG_ARCH_CFG_SEQ.NEXTVAL
						,v_SCHEDULERTASKID
						,p_STATUSFLAG
						,rec_cfg.KEEP_NUMDAYS
						,rec_cfg.ARCHIVEFLAG
						,rec_cfg.SCHED_TYPE
						,rec_cfg.DAYOFMONTH
						,rec_cfg.DAYOFWEEK
						,rec_cfg.HOUROPT
						,rec_cfg.MINUTEOPT
						,rec_cfg.FREQDAYOFMONTH
						,rec_cfg.FREQDAYOFWEEK
						,rec_cfg.ARCHDEST_FL
						,rec_cfg.ARCHDEST_DB
						,v_BeginTS
					)
					RETURNING ARCHCFGID INTO v_ArchCfgID;
					v_chkpt:='3';

					BEGIN
						INSERT INTO lg_arch_cfg_param(
							CFGPARAMID
							,ARCHCFGID
							,PARTNERID
							,PACKAGE_RO_ID
							,PROTOCOL_RO_ID
							,PROCESS_RO_ID
							,PARTNER
							,PACKAGENAME
							,PACKAGEVER
							,PROTOCOLNAME
							,PROTOCOLVER
							,PROCESSNAME
							,PROCESSVER
						) SELECT
							LG_ARCH_CFG_PARAM_SEQ.NEXTVAL
							,v_ARCHCFGID
							,PARTNERID
							,PACKAGE_RO_ID
							,PROTOCOL_RO_ID
							,PROCESS_RO_ID
							,PARTNER
							,PACKAGENAME
							,PACKAGEVER
							,PROTOCOLNAME
							,PROTOCOLVER
							,PROCESSNAME
							,PROCESSVER
						FROM	cf_archiver_param
						;
					END;
				END LOOP;
			ELSIF p_STATUSFLAG = v_sch_sts_RUNNING THEN
				 v_chkpt:='4';
				-- Initiate a new run
				OPEN cur_cfg;
				FETCH cur_cfg INTO v_ArchCfgId;
				CLOSE cur_cfg;
				INSERT INTO LG_ARCH_TASK(
					TASKID
					,ARCHCFGID
					,SCHEDULERTASKID
					,STATUSFLAG
					,CREATEDATE
				) VALUES (
					LG_ARCH_TASK_SEQ.nextval
					,v_ARCHCFGID
					,v_SCHEDULERTASKID
					,p_STATUSFLAG
					,COALESCE(p_LastExecTS, v_BeginTS)
				) returning TASKID INTO v_NewTaskID;
				
				IF v_NewTaskID IS NULL THEN
					RAISE NO_CONFIG_ERROR;
				ELSE
					OPEN RC1 For
							SELECT	tk.TASKID
								,cfg.ARCHCFGID
								,cfg.KEEP_NUMDAYS
								,cfg.ARCHIVEFLAG
								,cfg.DAYOFMONTH
								,cfg.DAYOFWEEK
								,cfg.HOUROPT
								,cfg.MINUTEOPT
								,cfg.FREQDAYOFMONTH
								,cfg.FREQDAYOFWEEK
								,cfg.SCHEDULERTASKID
								,cfg.ARCHDEST_FL
								,cfg.ARCHDEST_DB
								,cfg.CREATEDATE
							FROM	lg_arch_cfg cfg
							INNER JOIN lg_arch_task tk ON tk.archCfgID = cfg.ArchCfgId
							WHERE	tk.TASKID = v_NewTaskID
	;
				END IF;
			END IF;
	ELSE
		--- update status of the given task
		 v_chkpt:='5';
		UPDATE	lg_arch_task ux
		SET	ux.statusflag = CASE 
					WHEN p_StatusFlag IN (v_sch_sts_FAILED, v_sch_sts_DISABLED, v_sch_sts_STOPPED, v_sch_sts_SUCCESS) 
					AND ux.statusFlag = v_sch_sts_RUNNING -- change state for RUNNING task only
						THEN p_StatusFlag
					ELSE
						ux.StatusFlag
					END
			,ux.TASKENDTS = CASE
					WHEN p_StatusFlag IN (v_sch_sts_FAILED, v_sch_sts_DISABLED, v_sch_sts_STOPPED, v_sch_sts_SUCCESS) 
					AND ux.statusFlag = v_sch_sts_RUNNING -- endTS applies to RUNNING only
						THEN p_TASKENDTS
					ELSE
						null
					END
			,ux.UPDATEDATE = v_BeginTS
		WHERE	ux.TASKID = p_TASKID
		;

	END IF;
	p_ReturnCd := 0;
	v_chkpt :='6';
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN NO_CONFIG_ERROR THEN 
		raise_application_error(-20999,'20101 - CF_ARCHIVER_CONFIGU_AH1: ARCHIVER CONFIGURATION NOT FOUND ');
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		RAISE_APPLICATION_ERROR (-20901,'CF_ARCHIVER_CONFIGU_AH1' || SQLCODE|| ' '||SQLERRM);
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
		RAISE_APPLICATION_ERROR (-20902,'CF_ARCHIVER_CONFIGU_AH1' || SQLCODE|| ' '||SQLERRM);
	When OTHERS then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
		RAISE_APPLICATION_ERROR (-20999,'CF_ARCHIVER_CONFIGU_AH1:chkpt-' || v_chkpt || ' '||SQLERRM);
End CF_ARCHIVER_CONFIGU_AH1;
/



-- CF_ARCHIVER_CONFIGU_AH2.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_CONFIGU_AH2
(
	 	p_ReturnCd	out	integer
	,	p_DebugFlg	in	Integer
	,       RC1             out     db_ref.RCT
	,	p_STATUSFLAG	in	SMALLINT
	,	p_SCHEDULERTASKID in	VARCHAR2
	,	p_TASKID	in	INTEGER
	,	p_LASTEXECTS	in	TIMESTAMP
	,	p_NEXTEXECTS	in	TIMESTAMP
	,	p_TASKENDTS	in	TIMESTAMP
)
AS

/*
	================================================================================
	Proc Name:	CF_ARCHIVER_CONFIGU_AH2
	Create Date:     2008-07-25 15:50:17
	Author:	   S.Priani
	Description: Change status of restore based on value of p_StatusFlag
			0 = SCHEDULED		*
			1 = BEING SCHEDULED
			2 = RUNNING		*
			3 = STOPPING
			4 = DISABLED_FAILED	*
			5 = DISABLING
			6 = DISABLED		*
			7 = STOPPED		*
			8 = SUCCESSFUL		*
		* -> possible input values from server
		If 0 THEN if p_taskid is null then create new cfg
		If 2 THEN INSERT INTO LG_ARCH_TASK
		If 4 THEN UPDATE in LG_ARCH_TASK(2 to 4) for given TASKID
		If 6 THEN UPDATE in LG_ARCH_TASK(2 to 7) for given TASKID
		If 7 THEN UPDATE in LG_ARCH_TASK(3 to 7) for given TASKID
		If 8 THEN UPDATE in LG_ARCH_TASK(2 to 8) for given TASKID

			     ::::::::::::::::::::::::::::::
				    M O D I F I C A T I O N
			     ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	   NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-08-05 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			    M    A    I    N				  --
---------------------------------------------------------------------------

	v_RowCnt		integer Default 0;
	v_BeginTS		TimeStamp  ; -- Performance related variables
	v_ProcName		varchar2(30)  Default 'CF_ARCHIVER_CONFIGU_AH2';
	v_ExceptionFlg		integer  Default 0;
	v_ExceptionCode 	varchar2(30);
	v_PK_Value		Integer Default 0;
	v_SQLErrm	 	varchar2(255) ;
	v_chkpt			Char(1) default '0';
	v_sch_sts_SCHEDULED	INTEGER	 DEFAULT 0;
	v_sch_sts_SCHEDULING	INTEGER	 DEFAULT 1;
	v_sch_sts_RUNNING	INTEGER	 DEFAULT 2;
	v_sch_sts_STOPPING	INTEGER	 DEFAULT 3;
	v_sch_sts_FAILED	INTEGER	 DEFAULT 4;
	v_sch_sts_DISABLING	INTEGER	 DEFAULT 5;
	v_sch_sts_DISABLED	INTEGER	 DEFAULT 6;
	v_sch_sts_STOPPED	INTEGER	 DEFAULT 7;
	v_sch_sts_SUCCESS	INTEGER	 DEFAULT 8;

	v_RowTS			Integer Default 0;
	v_NewTaskId		INTEGER;
	v_ArchCfgID		INTEGER;
	v_SchedulerTaskID	varchar2(60)	DEFAULT 'RESTORE';
	NO_CONFIG_ERROR		EXCEPTION;
		
	v_ParamsTxt		varchar2(4000) Default
	 			 	'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 			 	||', p_STATUSFLAG=>'||p_STATUSFLAG||CHR(10)
	 			 	||', p_SCHEDULERTASKID=>'||p_SCHEDULERTASKID||CHR(10)
	 			 	||', p_TASKID=>'||p_TASKID||CHR(10)
	 			 	||', p_LASTEXECTS=>'||p_LASTEXECTS||CHR(10)
	 			 	||', p_NEXTEXECTS=>'||p_NEXTEXECTS||CHR(10)
	 			 	||', p_TASKENDTS=>'||p_TASKENDTS||CHR(10)
	;		
		
	Cursor cur_cfg  IS
		SELECT	ArchCfgID 
		FROM	lg_arch_cfg
		WHERE	schedulertaskId = v_SchedulerTaskId 
		ORDER BY archcfgid DESC
	;
BEGIN 
	UPDATE	CF_ARCHIVER_CONFIG ux
	SET	ux.ROWTS = ux.RowTS + 1
		, ux.STATUSFLAG = p_StatusFlag
		, ux.SCHEDULERTASKID = COALESCE( p_SCHEDULERTASKID, ux.SCHEDULERTASKID)
		, ux.LASTEXECTS = COALESCE( p_LASTEXECTS, ux.LASTEXECTS)
		, ux.NEXTEXECTS = COALESCE( p_NEXTEXECTS, ux.NEXTEXECTS)
		, ux.UPDATEDATE= v_BeginTS
	WHERE	ux.CONFIGNAME = 'RESTORE'
	;

	IF p_TASKID IS NULL THEN
		IF p_StatusFlag = v_sch_sts_RUNNING THEN
			v_chkpt:='2';
			-- get new archiver config for scheduling a new task
			FOR rec_cfg IN 
				(SELECT	*
					FROM	CF_ARCHIVER_CONFIG WHERE CONFIGNAME = 'RESTORE')
				LOOP
					v_chkpt:='3';
					INSERT INTO LG_ARCH_CFG(
						ARCHCFGID
						,SCHEDULERTASKID
						,STATUSFLAG
						,KEEP_NUMDAYS
						,ARCHIVEFLAG
						,SCHED_TYPE
						,DAYOFMONTH
						,DAYOFWEEK
						,HOUROPT
						,MINUTEOPT
						,FREQDAYOFMONTH
						,FREQDAYOFWEEK
						,ARCHDEST_FL
						,ARCHDEST_DB
						,RESTORE_FRDATE
						,RESTORE_TODATE
						,CREATEDATE
					) VALUES (
						LG_ARCH_CFG_SEQ.NEXTVAL
						,v_SCHEDULERTASKID
						,p_STATUSFLAG
						,rec_cfg.KEEP_NUMDAYS
						,rec_cfg.ARCHIVEFLAG
						,rec_cfg.SCHED_TYPE
						,rec_cfg.DAYOFMONTH
						,rec_cfg.DAYOFWEEK
						,rec_cfg.HOUROPT
						,rec_cfg.MINUTEOPT
						,rec_cfg.FREQDAYOFMONTH
						,rec_cfg.FREQDAYOFWEEK
						,rec_cfg.ARCHDEST_FL
						,rec_cfg.ARCHDEST_DB
						,rec_cfg.RESTORE_FRDATE
						,rec_cfg.RESTORE_TODATE
						,v_BeginTS
					)
					RETURNING ARCHCFGID INTO v_ArchCfgID;

			END LOOP;
			 v_chkpt:='4';
			-- Initiate a new run
			OPEN cur_cfg;
			FETCH cur_cfg INTO v_ArchCfgId;
			CLOSE cur_cfg;
			INSERT INTO LG_ARCH_TASK(
				TASKID
				,ARCHCFGID
				,SCHEDULERTASKID
				,STATUSFLAG
				,CREATEDATE
			) VALUES (
				LG_ARCH_TASK_SEQ.nextval
				,v_ARCHCFGID
				,v_SCHEDULERTASKID
				,p_STATUSFLAG
				,COALESCE(p_LastExecTS, v_BeginTS)
			) returning TASKID INTO v_NewTaskID;
				
			IF v_NewTaskID IS NULL THEN
				RAISE NO_CONFIG_ERROR;
			ELSE
				OPEN RC1 For
					SELECT	tk.TASKID
						,cfg.ARCHCFGID
						,cfg.KEEP_NUMDAYS
						,cfg.ARCHIVEFLAG
						,cfg.DAYOFMONTH
						,cfg.DAYOFWEEK
						,cfg.HOUROPT
						,cfg.MINUTEOPT
						,cfg.FREQDAYOFMONTH
						,cfg.FREQDAYOFWEEK
						,cfg.SCHEDULERTASKID
						,cfg.ARCHDEST_FL
						,cfg.ARCHDEST_DB
						,cfg.RESTORE_FRDATE
						,cfg.RESTORE_TODATE
						,cfg.CREATEDATE
					FROM	lg_arch_cfg cfg
					INNER JOIN lg_arch_task tk ON tk.archCfgID = cfg.ArchCfgId
					WHERE	tk.TASKID = v_NewTaskID;
			END IF;
		END IF;
	ELSE
		--- update status of the given task
		 v_chkpt:='5';
		UPDATE	lg_arch_task ux
		SET	ux.statusflag = CASE 
					WHEN p_StatusFlag IN (v_sch_sts_FAILED, v_sch_sts_DISABLED, v_sch_sts_STOPPED, v_sch_sts_SUCCESS) 
					AND ux.statusFlag = v_sch_sts_RUNNING -- change state for RUNNING task only
						THEN p_StatusFlag
					ELSE
						ux.StatusFlag
					END
			,ux.TASKENDTS = CASE
					WHEN p_StatusFlag IN (v_sch_sts_FAILED, v_sch_sts_DISABLED, v_sch_sts_STOPPED, v_sch_sts_SUCCESS) 
					AND ux.statusFlag = v_sch_sts_RUNNING -- endTS applies to RUNNING only
						THEN p_TASKENDTS
					ELSE
						null
					END
			,ux.UPDATEDATE = v_BeginTS
		WHERE	ux.TASKID = p_TASKID
		;

	END IF;
	p_ReturnCd := 0;
	v_chkpt :='6';
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN NO_CONFIG_ERROR THEN 
		raise_application_error(-20999,'20101 - CF_ARCHIVER_CONFIGU_AH2: ARCHIVER CONFIGURATION NOT FOUND ');
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		RAISE_APPLICATION_ERROR (-20901,'CF_ARCHIVER_CONFIGU_AH2' || SQLCODE|| ' '||SQLERRM);
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
		RAISE_APPLICATION_ERROR (-20902,'CF_ARCHIVER_CONFIGU_AH2' || SQLCODE|| ' '||SQLERRM);
	When OTHERS then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
		RAISE_APPLICATION_ERROR (-20999,'CF_ARCHIVER_CONFIGU_AH2:chkpt-' || v_chkpt || ' '||SQLERRM);
End CF_ARCHIVER_CONFIGU_AH2;
/



-- CF_ARCHIVER_PARAMD.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_PARAMD
	( p_ReturnCd OUT	INTEGER
	, p_DebugFlg IN INTEGER
	, p_PARAMETERID	IN INTEGER
	)

AS

/*
	================================================================================
	Proc Name:	CF_ARCHIVER_PARAMD
	Create Date:	2008-07-31 15:44:50
	Author:		SYSTEM

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 SYSTEM	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	 v_RowCnt		integer Default 0;
	 v_BeginTS	 TimeStamp	; -- Performance related variables
	 v_ProcName	varchar2(30)	Default 'CF_ARCHIVER_PARAMD';
	 v_ExceptionFlg	integer	Default 0;
	 v_ExceptionCode	varchar2(30);
	 v_PK_Value	INTEGER Default 0;
	 v_SQLErrm	 Varchar2(255) ;
	 v_chkpt		Char(1) default '0';
	 v_msg		Varchar2(500);
	 v_RowTS		INTEGER Default 0;	
	  v_ParamsTxt	varchar(4000) DEFAULT
			' p_DebugFlg=>'||Coalesce(Rtrim(CHR(p_DebugFlg)),'')
			||', p_PARAMETERID=>'||COALESCE(RTRIM(CHR(p_PARAMETERID)),'')
		;
		
	zeroDelete  Exception;		
	
BEGIN

	v_BeginTS := FN_Get_Date();
	v_chkpt :='1';
	
	DELETE FROM CF_ARCHIVER_PARAM
	WHERE PARAMETERID = p_PARAMETERID
	;
	
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	
Exception
	When ZeroDelete Then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 CF_ARCHIVER_PARAMD: No Data Found');
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
	
END CF_ARCHIVER_PARAMD;
/


-- CF_ARCHIVER_PARAMI.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_PARAMI
	(
		p_ReturnCd OUT	INTEGER
	,	p_DebugFlg IN	INTEGER
	, 	RC1 out db_ref.RCT
	,	p_CONFIGID IN INTEGER
	,	p_PARTNERID IN INTEGER
	,	p_PACKAGE_RO_ID IN INTEGER
	,	p_PROTOCOL_RO_ID IN INTEGER
	,	p_PROCESS_RO_ID IN INTEGER
	,	p_PARTNER IN VARCHAR2
	,       p_PACKAGENAME IN VARCHAR2
	,	p_PACKAGEVER IN VARCHAR2
	,	p_PROTOCOLNAME IN VARCHAR2
	,	p_PROTOCOLVER IN VARCHAR
	,	p_PROCESSNAME IN VARCHAR
	,	p_PROCESSVER IN VARCHAR
	,	p_ENABLED IN INTEGER
	)
AS


/*
	================================================================================
	Proc Name:	CF_ARCHIVER_PARAMI
	Create Date:	2008-07-31 15:43:52
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	 v_RowCnt		integer Default 0;
	 v_BeginTS         timestamp Default FN_Get_Date;  -- Performance related variables
	 v_ProcName	varchar(30)	Default 'CF_ARCHIVER_PARAMI';
	 v_ExceptionFlg	integer	Default 0;
	 v_ExceptionCode	varchar(30);
	 v_PK_Value	INTEGER Default 0;
	 v_SQLErrm	 Varchar(255) ;
	 v_chkpt		Char(1) default '0';
	 v_msg		Varchar(500);
	 v_RowTS		INTEGER Default 0;
	 
	 v_ParamsTxt       varchar2(4000) Default
	 			 	'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 			 	||', p_CONFIGID=>'||p_CONFIGID||CHR(10)
	 			 	||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
	 			 	||', p_PACKAGE_RO_ID=>'||p_PACKAGE_RO_ID||CHR(10)
	 			 	||', p_PROTOCOL_RO_ID=>'||p_PROTOCOL_RO_ID||CHR(10)
	 			 	||', p_PROCESS_RO_ID=>'||p_PROCESS_RO_ID||CHR(10)
	 			 	||', p_PARTNER=>'||p_PARTNER||CHR(10)
	 			 	||', p_PACKAGENAME=>'||p_PACKAGENAME||CHR(10)
	 			 	||', p_PACKAGEVER=>'||p_PACKAGEVER||CHR(10)
	 			 	||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME||CHR(10)
	 			 	||', p_PROTOCOLVER=>'||p_PROTOCOLVER||CHR(10)
	 			 	||', p_PROCESSVER=>'||p_PROCESSVER||CHR(10)
	 			 	||', p_ENABLED=>'||p_ENABLED||CHR(10)
	 		;
	 		
	 		
	 		
	BEGIN	 		
	 
	p_ReturnCd := 0;

	v_chkpt:='1';
	
	INSERT INTO CF_ARCHIVER_PARAM(
		PARAMETERID
		,CONFIGID
		,PARTNERID
		,PACKAGE_RO_ID
		,PROTOCOL_RO_ID
		,PROCESS_RO_ID
		,PARTNER
		,PACKAGENAME
		,PACKAGEVER
		,PROTOCOLNAME
		,PROTOCOLVER
		,PROCESSNAME
		,PROCESSVER
		,ENABLED
		,CREATEDATE
		,UPDATEDATE
	) VALUES (
		CF_ARCHIVER_PARAM_SEQ.NEXTVAL
		,p_CONFIGID
		,p_PARTNERID
		,p_PACKAGE_RO_ID
		,p_PROTOCOL_RO_ID
		,p_PROCESS_RO_ID
		,p_PARTNER
		,p_PACKAGENAME
		,p_PACKAGEVER
		,p_PROTOCOLNAME
		,p_PROTOCOLVER
		,p_PROCESSNAME
		,p_PROCESSVER
		,p_ENABLED
		,v_BeginTS
		,v_BeginTS
	);
	v_chkpt:='2';
	
	IF RC1%isOpen THEN Close RC1; END IF;
				
	Open RC1 FOR SELECT CF_ARCHIVER_PARAM_SEQ.currval ID,
			    1 RowTS From Dual;

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END CF_ARCHIVER_PARAMI;
/


-- CF_ARCHIVER_PARAMS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_PARAMS_AH1
	(
		p_ReturnCd OUT	INTEGER
	,	p_DebugFlg IN	INTEGER
	,       RC1 out db_ref.RCT
	,       RC2 out db_ref.RCT
	,	p_sortkey IN	VARCHAR2
	,	p_PageSizeNbr IN INTEGER
	,	p_PageNbr IN	INTEGER
	,	p_CONFIGID IN	INTEGER
	)
AS
/*
	================================================================================
	Proc Name:	CF_ARCHIVER_PARAMS_AH1
	Create Date:	2008-07-31 15:43:52
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	v_RowCnt	integer Default 0;
	v_BeginTS       timestamp Default FN_Get_Date;  -- Performance related variables
	v_ProcName	varchar2(30)	Default 'CF_ARCHIVER_PARAMS_AH1';
	v_ExceptionFlg	integer	Default 0;
	v_ExceptionCode	varchar2(30);
	v_PK_Value	INTEGER Default 0;
	v_SQLErrm	Varchar2(255) ;
	v_chkpt		Char(1) default '0';
	v_RowTS		INTEGER Default 0;

	v_where		varchar2(1000);
	v_count		Integer	DEFAULT 0;
	v_sortKey	varchar2(1000);
	v_sqlStmt	varchar2(4000);

	v_ParamsTxt     varchar2(4000) Default
	 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 		||', p_CONFIGID=>'||p_CONFIGID||CHR(10)
		;

BEGIN

	p_ReturnCd := 0;
	v_Chkpt:='1';
	v_where := ' Where 1 = 1 '||
		FN_Build_Where('ap.CONFIGID', 'NUMBER', p_CONFIGID)
	;
	v_SQLStmt :=  'SELECT '
			|| 'ap.PARAMETERID '
			|| ',ap.CONFIGID '
			|| ',ap.PARTNERID '
			|| ',ap.PACKAGE_RO_ID '
			|| ',ap.PROTOCOL_RO_ID '
			|| ',ap.PROCESS_RO_ID '
			|| ',RTRIM(ap.PARTNER) PARTNER '
			|| ',RTRIM(ap.PACKAGENAME) PACKAGENAME '
			|| ',RTRIM(ap.PACKAGEVER) PACKAGEVER '
			|| ',RTRIM(ap.PROTOCOLNAME) PROTOCOLNAME '
			|| ',RTRIM(ap.PROTOCOLVER) PROTOCOLVER '
			|| ',RTRIM(ap.PROCESSNAME) PROCESSNAME '
			|| ',RTRIM(ap.PROCESSVER) PROCESSVER '
			|| ',ap.ENABLED '
			|| ',ap.ROWTS '
			|| ',ap.CREATEDATE '
		|| 'FROM	CF_ARCHIVER_PARAM ap '
		|| v_where
	;
	-------------------------
	-- Total row count Cursor
	-------------------------
	IF COALESCE(p_pageNbr,0) in (0,1) THEN
		v_Chkpt :='2';
		EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM ('||v_Sqlstmt ||') ' INTO v_Count;
		OPEN RC2 For
			SELECT v_count TotalRowsNbr FROM DUAL;
	END IF;

	IF p_SortKey IS NOT NULL THEN
		 v_Chkpt :='3';
		 v_SortKey := UPPER(p_SortKey);
		 v_sqlstmt := v_sqlstmt ||' order by ' ||v_sortkey; 
	END IF;	

	---------------
	-- Paging logic
	---------------
	IF p_PageSizeNbr >= 0 THEN
		v_Chkpt :='4';
		v_SQLstmt := FN_PAGING_LOGIC(p_pageNbr, p_pageSizeNbr, v_Sqlstmt);
	END IF;
		
	v_Chkpt :='5';
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 for v_SQLstmt;

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END CF_ARCHIVER_PARAMS_AH1;
/


-- CF_ARCHIVER_PARAMS_AH2.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_PARAMS_AH2
	(
		p_ReturnCd OUT	INTEGER
	,	p_DebugFlg IN	INTEGER
	,       RC1 out db_ref.RCT
	,	p_ParameterType	IN	INTEGER --- 2=Partner, 16=Package, 32=Protocol, 64=Process
	,	p_ObjectId	IN	INTEGER --- PartnerId Or RoutingObjectId
	)
AS


/*
	================================================================================
	Proc Name:	CF_ARCHIVER_PARAMS_AH2
	Create Date:	2008-07-31 15:43:52
	Author:		S.Priani

	Desc:	WhereUsed  Partner OR RoutingObjects - to srch in Archiver Parameters
				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-09-11 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	 v_RowCnt		integer Default 0;
	 v_BeginTS         timestamp Default FN_Get_Date;  -- Performance related variables
	 v_ProcName	varchar2(30)	Default 'CF_ARCHIVER_PARAMS_AH2';
	 v_ExceptionFlg	integer	Default 0;
	 v_ExceptionCode	VARCHAR2(30);
	 v_PK_Value	INTEGER Default 0;
	 v_SQLErrm	Varchar2(255) ;
	 v_chkpt		VARCHAR2(1) default '0';
	 v_RowTS		INTEGER Default 0;

	 v_where		varCHAR2(1000);
	 v_count		Integer;
	 v_count2	Integer;
	 v_sortKey	varchar2(1000);
	 v_sqlStmt	varchar2(4000);
	 v_sqlStmt2	varchar2(4000);
	 v_DySQL		VARCHAR2(1000);
	 
	 C1  db_ref.RCT;
	 v_ParamsTxt       varchar2(4000) Default
	 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 		||', p_ParameterType=>'||p_ParameterType||CHR(10)
	 		||', p_ObjectID=>'||p_ObjectID||CHR(10)
		;
	 v_stmt varchar2(4000);		

	BEGIN
	IF p_ParameterType = 2 THEN
		v_where := 'WHERE ap.PartnerID = ' || TO_CHAR(p_ObjectID);
	ELSE
		v_where := 'WHERE ap.Package_RO_ID = ' || TO_CHAR(p_ObjectID)
			|| ' OR ap.Protocol_RO_ID = ' || TO_CHAR(p_ObjectID)
			|| ' OR ap.Process_RO_ID = ' || TO_CHAR(p_ObjectID);
	END IF;
	
	p_ReturnCd := 0;
	v_Chkpt:='1';
	v_SQLStmt :=  'SELECT '
			|| 'ap.PARAMETERID '
			|| ',ap.CONFIGID '
			|| ',ap.PARTNERID '
			|| ',ap.PACKAGE_RO_ID '
			|| ',ap.PROTOCOL_RO_ID '
			|| ',ap.PROCESS_RO_ID '
			|| ',RTRIM(ap.PARTNER) PARTNER '
			|| ',RTRIM(ap.PACKAGENAME) PACKAGENAME '
			|| ',RTRIM(ap.PACKAGEVER) PACKAGEVER '
			|| ',RTRIM(ap.PROTOCOLNAME) PROTOCOLNAME '
			|| ',RTRIM(ap.PROTOCOLVER) PROTOCOLVER '
			|| ',RTRIM(ap.PROCESSNAME) PROCESSNAME '
			|| ',RTRIM(ap.PROCESSVER) PROCESSVER '
			|| ',ap.ENABLED '
			|| ',ap.ROWTS '
			|| ',ap.CREATEDATE '
		|| 'FROM	CF_ARCHIVER_PARAM ap '
		|| v_where
	;

	BEGIN
		  IF RC1%isOpen THEN Close RC1; END IF;
	    	  Open RC1 for v_SQLstmt;
	END;

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END CF_ARCHIVER_PARAMS_AH2;
/


-- CF_ARCHIVER_PARAMS_AH3.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_PARAMS_AH3
	(
		p_ReturnCd OUT	INTEGER
	,	p_DebugFlg IN	INTEGER
	,       RC1 out db_ref.RCT
	,	p_PARAMETERID IN INTEGER
	,	p_PARTNERID IN INTEGER
	,	p_PACKAGE_RO_ID IN INTEGER
	,	p_PROTOCOL_RO_ID IN INTEGER
	,	p_PROCESS_RO_ID IN INTEGER
	,	p_PARTNER IN VARCHAR2
	,       p_PACKAGENAME IN VARCHAR2
	,	p_PACKAGEVER IN VARCHAR2
	,	p_PROTOCOLNAME IN VARCHAR2
	,	p_PROTOCOLVER IN VARCHAR
	,	p_PROCESSNAME IN VARCHAR
	,	p_PROCESSVER IN VARCHAR
	)
AS

/*
	================================================================================
	Proc Name:	CF_ARCHIVER_PARAMS_AH3
	Create Date:	2008-07-31 15:43:52
	Author:		S.Priani

	Desc:	srch in Archiver Parameters if this value belongs in Or engulfs any other
		-1  --- Member of 
		0 --- Exact Match
		+1  --- OverRides
				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-09-11 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	v_RowCnt		integer Default 0;
	v_BeginTS         timestamp Default FN_Get_Date;  -- Performance related variables
	v_ProcName	varchar2(30)	Default 'CF_ARCHIVER_PARAMS_AH3';
	v_ExceptionFlg	integer	Default 0;
	v_ExceptionCode	varchar(30);
	v_PK_Value	INTEGER Default 0;
	v_SQLErrm	Varchar(255) ;
	v_chkpt		Char(1) default '0';
	v_msg		Varchar(500);
	v_RowTS		INTEGER Default 0;
	v_AnyValue	INTEGER Default -1;
	v_Ignore	INTEGER Default 0;
		
	v_ParamsTxt       varchar2(4000) Default
	 			 	'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 			 	||', p_ParameterID=>'||p_ParameterID||CHR(10)
	 			 	||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
	 			 	||', p_PACKAGE_RO_ID=>'||p_PACKAGE_RO_ID||CHR(10)
	 			 	||', p_PROTOCOL_RO_ID=>'||p_PROTOCOL_RO_ID||CHR(10)
	 			 	||', p_PROCESS_RO_ID=>'||p_PROCESS_RO_ID||CHR(10)
	 			 	||', p_PARTNER=>'||p_PARTNER||CHR(10)
	 			 	||', p_PACKAGENAME=>'||p_PACKAGENAME||CHR(10)
	 			 	||', p_PACKAGEVER=>'||p_PACKAGEVER||CHR(10)
	 			 	||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME||CHR(10)
	 			 	||', p_PROTOCOLVER=>'||p_PROTOCOLVER||CHR(10)
	 			 	||', p_PROCESSVER=>'||p_PROCESSVER||CHR(10)
	;
BEGIN
	IF RC1%isOpen THEN Close RC1; END IF;
    	Open RC1 for 
	SELECT PARAMETERID
		,CONFIGID
		,PARTNERID
		,PACKAGE_RO_ID
		,PROTOCOL_RO_ID
		,PROCESS_RO_ID
		,RTRIM(PARTNER) PARTNER
		,RTRIM(PACKAGENAME) PACKAGENAME
		,RTRIM(PACKAGEVER) PACKAGEVER
		,RTRIM(PROTOCOLNAME) PROTOCOLNAME
		,RTRIM(PROTOCOLVER) PROTOCOLVER
		,RTRIM(PROCESSNAME) PROCESSNAME
		,RTRIM(PROCESSVER) PROCESSVER
		,ENABLED
		,ROWTS
		,CASE 
			WHEN	p_PARTNERID =	CASE 
						WHEN ovr.PARTNERID = v_AnyValue 
							THEN p_PARTNERID 
						WHEN ovr.PartnerID = p_PartnerID
							AND ovr.Partner = COALESCE(p_Partner, ovr.Partner)
							THEN ovr.PARTNERID  
						ELSE v_Ignore
						END
			AND	p_PACKAGE_RO_ID = CASE
						WHEN ovr.PACKAGE_RO_ID = v_AnyValue
							THEN p_PACKAGE_RO_ID
						WHEN ovr.Package_RO_ID = p_Package_RO_ID
							AND ovr.PackageName = COALESCE(p_PackageName , ovr.PackageName )
							AND ovr.PackageVer = COALESCE(p_PackageVer, ovr.PackageVer)
							THEN ovr.PACKAGE_RO_ID 
						ELSE v_Ignore
						END
			AND	p_PROTOCOL_RO_ID = CASE
						WHEN ovr.PROTOCOL_RO_ID = v_AnyValue
							THEN p_PROTOCOL_RO_ID
						WHEN ovr.PROTOCOL_RO_ID = p_PROTOCOL_RO_ID
							AND ovr.PROTOCOLName = COALESCE(p_PROTOCOLName , ovr.PROTOCOLName )
							AND ovr.PROTOCOLVer = COALESCE(p_PROTOCOLVer, ovr.PROTOCOLVer)
							THEN ovr.PROTOCOL_RO_ID 
						ELSE v_Ignore
						END
			AND	p_PROCESS_RO_ID = CASE 
						WHEN ovr.PROCESS_RO_ID = v_AnyValue
							THEN p_PROCESS_RO_ID
						WHEN ovr.PROCESS_RO_ID = p_PROCESS_RO_ID
							AND ovr.PROCESSName = COALESCE(p_PROCESSName , ovr.PROCESSName )
							AND ovr.PROCESSVer = COALESCE(p_PROCESSVer, ovr.PROCESSVer)
							THEN ovr.PROCESS_RO_ID 
						ELSE v_Ignore
						END
				THEN -1
			ELSE 0
		END  + 
		CASE
			WHEN	ovr.PARTNERID =	CASE 
						WHEN p_PARTNERID = v_AnyValue 
							THEN ovr.PARTNERID 
						WHEN p_PartnerID = ovr.PartnerID
							AND p_Partner = COALESCE(ovr.Partner, p_Partner)
							THEN p_PARTNERID  
						ELSE v_Ignore
						END
			AND	ovr.PACKAGE_RO_ID = CASE
						WHEN p_PACKAGE_RO_ID = v_AnyValue
							THEN ovr.PACKAGE_RO_ID
						WHEN p_Package_RO_ID = ovr.Package_RO_ID
							AND p_PackageName = COALESCE(ovr.PackageName , p_PackageName )
							AND p_PackageVer = COALESCE(ovr.PackageVer, p_PackageVer)
							THEN p_PACKAGE_RO_ID 
						ELSE v_Ignore
						END
			AND	ovr.PROTOCOL_RO_ID = CASE
						WHEN p_PROTOCOL_RO_ID = v_AnyValue
							THEN ovr.PROTOCOL_RO_ID
						WHEN p_PROTOCOL_RO_ID = ovr.PROTOCOL_RO_ID
							AND p_PROTOCOLName = COALESCE(ovr.PROTOCOLName , p_PROTOCOLName )
							AND p_PROTOCOLVer = COALESCE(ovr.PROTOCOLVer, p_PROTOCOLVer)
							THEN p_PROTOCOL_RO_ID 
						ELSE v_Ignore
						END
			AND	ovr.PROCESS_RO_ID = CASE 
						WHEN p_PROCESS_RO_ID = v_AnyValue
							THEN ovr.PROCESS_RO_ID
						WHEN p_PROCESS_RO_ID = ovr.PROCESS_RO_ID
							AND p_PROCESSName = COALESCE(ovr.PROCESSName , p_PROCESSName )
							AND p_PROCESSVer = COALESCE(ovr.PROCESSVer, p_PROCESSVer)
							THEN p_PROCESS_RO_ID 
						ELSE v_Ignore
						END
				THEN +1
			ELSE 0
		END	OverRideType
	FROM	CF_ARCHIVER_PARAM ovr
	WHERE	ovr.PARAMETERID <> p_PARAMETERID
	AND	((
				p_PARTNERID =	CASE 
						WHEN ovr.PARTNERID = v_AnyValue 
							THEN p_PARTNERID 
						WHEN ovr.PartnerID = p_PartnerID
							AND ovr.Partner = COALESCE(p_Partner, ovr.Partner)
							THEN ovr.PARTNERID  
						ELSE v_Ignore
						END
			AND	p_PACKAGE_RO_ID = CASE
						WHEN ovr.PACKAGE_RO_ID = v_AnyValue
							THEN p_PACKAGE_RO_ID
						WHEN ovr.Package_RO_ID = p_Package_RO_ID
							AND ovr.PackageName = COALESCE(p_PackageName , ovr.PackageName )
							AND ovr.PackageVer = COALESCE(p_PackageVer, ovr.PackageVer)
							THEN ovr.PACKAGE_RO_ID 
						ELSE v_Ignore
						END
			AND	p_PROTOCOL_RO_ID = CASE
						WHEN ovr.PROTOCOL_RO_ID = v_AnyValue
							THEN p_PROTOCOL_RO_ID
						WHEN ovr.PROTOCOL_RO_ID = p_PROTOCOL_RO_ID
							AND ovr.PROTOCOLName = COALESCE(p_PROTOCOLName , ovr.PROTOCOLName )
							AND ovr.PROTOCOLVer = COALESCE(p_PROTOCOLVer, ovr.PROTOCOLVer)
							THEN ovr.PROTOCOL_RO_ID 
						ELSE v_Ignore
						END
			AND	p_PROCESS_RO_ID = CASE 
						WHEN ovr.PROCESS_RO_ID = v_AnyValue
							THEN p_PROCESS_RO_ID
						WHEN ovr.PROCESS_RO_ID = p_PROCESS_RO_ID
							AND ovr.PROCESSName = COALESCE(p_PROCESSName , ovr.PROCESSName )
							AND ovr.PROCESSVer = COALESCE(p_PROCESSVer, ovr.PROCESSVer)
							THEN ovr.PROCESS_RO_ID 
						ELSE v_Ignore
						END
		) OR (
				ovr.PARTNERID =	CASE 
						WHEN p_PARTNERID = v_AnyValue 
							THEN ovr.PARTNERID 
						WHEN p_PartnerID = ovr.PartnerID
							AND p_Partner = COALESCE(ovr.Partner, p_Partner)
							THEN p_PARTNERID  
						ELSE v_Ignore
						END
			AND	ovr.PACKAGE_RO_ID = CASE
						WHEN p_PACKAGE_RO_ID = v_AnyValue
							THEN ovr.PACKAGE_RO_ID
						WHEN p_Package_RO_ID = ovr.Package_RO_ID
							AND p_PackageName = COALESCE(ovr.PackageName , p_PackageName )
							AND p_PackageVer = COALESCE(ovr.PackageVer, p_PackageVer)
							THEN p_PACKAGE_RO_ID 
						ELSE v_Ignore
						END
			AND	ovr.PROTOCOL_RO_ID = CASE
						WHEN p_PROTOCOL_RO_ID = v_AnyValue
							THEN ovr.PROTOCOL_RO_ID
						WHEN p_PROTOCOL_RO_ID = ovr.PROTOCOL_RO_ID
							AND p_PROTOCOLName = COALESCE(ovr.PROTOCOLName , p_PROTOCOLName )
							AND p_PROTOCOLVer = COALESCE(ovr.PROTOCOLVer, p_PROTOCOLVer)
							THEN p_PROTOCOL_RO_ID 
						ELSE v_Ignore
						END
			AND	ovr.PROCESS_RO_ID = CASE 
						WHEN p_PROCESS_RO_ID = v_AnyValue
							THEN ovr.PROCESS_RO_ID
						WHEN p_PROCESS_RO_ID = ovr.PROCESS_RO_ID
							AND p_PROCESSName = COALESCE(ovr.PROCESSName , p_PROCESSName )
							AND p_PROCESSVer = COALESCE(ovr.PROCESSVer, p_PROCESSVer)
							THEN p_PROCESS_RO_ID 
						ELSE v_Ignore
						END

		))
	;

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END CF_ARCHIVER_PARAMS_AH3;
/


-- CF_ARCHIVER_PARAMU.sql

CREATE OR REPLACE PROCEDURE CF_ARCHIVER_PARAMU
(
	 	p_ReturnCd OUT	integer
	,	p_DebugFlg IN INTEGER
	,       RC1 out db_ref.RCT
	,	p_PARAMETERID IN INTEGER
	,	p_CONFIGID IN INTEGER
	,	p_PARTNERID  IN INTEGER
	,	p_PACKAGE_RO_ID IN INTEGER
	,	p_PROTOCOL_RO_ID IN INTEGER
	,	p_PROCESS_RO_ID IN INTEGER
	,	p_PARTNER IN VARCHAR2
	,	p_PACKAGENAME IN VARCHAR2 
	,	p_PACKAGEVER IN VARCHAR2
	,	p_PROTOCOLNAME IN VARCHAR2
	,	p_PROTOCOLVER IN VARCHAR2 
	,	p_PROCESSNAME IN VARCHAR2
	,	p_PROCESSVER IN VARCHAR2
	,	p_ENABLED IN INTEGER
	,	p_ROWTS In INTEGER
)
AS



/*
	================================================================================
	Proc Name:	CF_ARCHIVER_PARAMU
	Create Date:	2008-07-31 15:43:53
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	 v_RowCnt		integer Default 0;
	 v_BeginTS	 TimeStamp	; -- Performance related variables
	 v_ProcName	varchar(30)	Default 'CF_ARCHIVER_PARAMU';
	 v_ExceptionFlg	integer	Default 0;
	 v_ExceptionCode	varchar(30);
	 v_PK_Value	INTEGER Default 0;
	 v_SQLErrm	 Varchar(255) ;
	 v_chkpt		Char(1) default '0';
	 v_msg		Varchar(500);
	 v_RowTS		INTEGER Default 0;
	 v_ParamsTxt       varchar2(4000) Default
	 	 		 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 	 			 	||', p_PARAMETERID=>'||p_PARAMETERID||CHR(10)
	 	 			 	||', p_CONFIGID=>'||p_CONFIGID||CHR(10)
	 	 			 	||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
	 	 			 	||', p_PACKAGE_RO_ID=>'||p_PACKAGE_RO_ID||CHR(10)
	 	 			 	||', p_PROCESS_RO_ID=>'||p_PROCESS_RO_ID||CHR(10)
	 	 			 	||', p_PARTNER=>'||p_PARTNER||CHR(10)
	 	 			 	||', p_PACKAGENAME=>'||p_PACKAGENAME||CHR(10)
	 	 			 	||', p_PACKAGEVER=>'||p_PACKAGEVER||CHR(10)
	 	 			 	||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME||CHR(10)
	 	 			 	||', p_PROTOCOLVER=>'||p_PROTOCOLVER||CHR(10)
	 	 			 	||', p_PROCESSNAME=>'||p_PROCESSNAME||CHR(10)
	 	 			 	||', p_PROCESSVER=>'||p_PROCESSVER||CHR(10)
	 	 			 	||', p_ENABLED=>'||p_ENABLED||CHR(10)
	 	 			 	||', p_ROWTS=>'||p_ROWTS||CHR(10)
	 		;
	
	BEGIN
	
	 p_ReturnCd := 0;
	 v_BeginTS := FN_Get_Date();
	
	 v_chkpt :='1';
	Update CF_ARCHIVER_PARAM ux SET
		ux.CONFIGID = COALESCE(p_CONFIGID, ux.CONFIGID)
		, ux.PARTNERID = COALESCE( p_PARTNERID, ux.PARTNERID)
		, ux.PACKAGE_RO_ID = COALESCE( p_PACKAGE_RO_ID, ux.PACKAGE_RO_ID)
		, ux.PROTOCOL_RO_ID = COALESCE( p_PROTOCOL_RO_ID, ux.PROTOCOL_RO_ID)
		, ux.PROCESS_RO_ID = COALESCE( p_PROCESS_RO_ID, ux.PROCESS_RO_ID)
		, ux.PARTNER = COALESCE( p_PARTNER, ux.PARTNER)
		, ux.PACKAGENAME = COALESCE( p_PACKAGENAME, ux.PACKAGENAME)
		, ux.PACKAGEVER = COALESCE( p_PACKAGEVER, ux.PACKAGEVER)
		, ux.PROTOCOLNAME = COALESCE( p_PROTOCOLNAME, ux.PROTOCOLNAME)
		, ux.PROTOCOLVER = COALESCE( p_PROTOCOLVER, ux.PROTOCOLVER)
		, ux.PROCESSNAME = COALESCE( p_PROCESSNAME, ux.PROCESSNAME)
		, ux.PROCESSVER = COALESCE( p_PROCESSVER, ux.PROCESSVER)
		, ux.ENABLED = COALESCE( p_ENABLED, ux.ENABLED)
		, ux.ROWTS = COALESCE(  p_RowTS + 1, ux.ROWTS)
		, ux.UPDATEDATE = COALESCE(  v_BeginTS, ux.UPDATEDATE)
	WHERE	PARAMETERID = p_PARAMETERID
	AND	RowTS = p_RowTS ;
	
	 v_RowTS := p_RowTS + 1;
	 
	 v_chkpt :='2';
	OPEN RC1 FOR Select
	v_rowTS ROWTS From Dual;
	
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
	v_ExceptionFlg := 1;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
	RAISE_APPLICATION_ERROR (-20999,'20000 CF_ARCHIVER_CONFIGU' || SQLCODE|| ' '||SQLERRM);

	When DUP_VAL_ON_INDEX then
	v_ExceptionFlg := 1;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_returnCd := 2;
	RAISE_APPLICATION_ERROR (-20999,'20001 CF_ARCHIVER_CONFIGU' || SQLCODE|| ' '||SQLERRM);
	
End CF_ARCHIVER_PARAMU;
/


-- CF_BATCHCONTENTSD.sql

Create or Replace Procedure CF_BATCHCONTENTSD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer   
	,p_BATCHID in VARCHAR2
	,p_VUID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_BATCHCONTENTSD
	Create Date:     2004-08-12 10:24:20
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BATCHCONTENTSD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_BATCHCONTENTS
	Where 
		BATCHID = p_BATCHID
		AND VUID = p_VUID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_BATCHCONTENTSD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_BATCHCONTENTSD;
/


-- CF_BATCHCONTENTSI.sql

Create or Replace Procedure CF_BATCHCONTENTSI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_BATCHID in VARCHAR2
	,p_VUID in VARCHAR2
   ,p_ORDERING in NUMBER
	,p_STATUS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_BATCHCONTENTSI
	Create Date:     2004-08-12 10:24:53
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BATCHCONTENTSI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
      ||', p_BATCHID=>'||p_BATCHID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_STATUS=>'||p_STATUS||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_BATCHCONTENTS(
		BATCHID
		,VUID
		,ORDERING
		,STATUS
	) Values (
		p_BATCHID
		,p_VUID
		,p_ORDERING
		,p_STATUS
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_BATCHCONTENTSI' || SQLCODE|| ' '||SQLERRM);
END CF_BATCHCONTENTSI;
/


-- CF_BATCHCONTENTSI_AH1.sql

CREATE OR REPLACE PROCEDURE CF_BATCHCONTENTSI_AH1
	(
	p_ReturnCd	OUT PLS_INTEGER
	,p_DebugFlg	IN PLS_INTEGER
	,RC1		OUT DB_REF.RCT
	,p_BATCHID	IN VARCHAR2
	,p_VUID		IN VARCHAR2
	,p_ORDERING	in NUMBER
	,p_STATUS	in NUMBER
	,p_BATCHSTATUS	IN NUMBER
	,p_BATCHCOUNT	IN NUMBER
	,p_PARENTBATCHID	in VARCHAR2
	,p_PARENTTXNVUID	in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_BATCHCONTENTSI
	Create Date:     2004-08-12 10:25:12
	Author:	  SYSTEM

			   ::::::::::::::::::::::::::::::
				 M O D I F I C A T I O N
			   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-08-12 SYSTEM	3.5	Created
	2004-10-27 S.Priani	4.3	BGTRK00054219
	2005-05-07 S.Priani	4.3	BGTRK00048595
	2006-06-06 S.Narain 6.1 JR23845
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_BeginTS	 timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName	varchar2(30) Default 'CF_BATCHCONTENTSI_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_STATUS=>'||p_STATUS||CHR(10)
		||', p_PARENTBATCHID=>'||p_PARENTBATCHID||CHR(10)
		||', p_BATCHSTATUS=>'||p_PARENTBATCHID||CHR(10)
	      	||', p_BATCHCOUNT=>'||p_PARENTBATCHID||CHR(10)
		||', p_PARENTBATCHID=>'||p_PARENTBATCHID||CHR(10)
		||', p_PARENTTXNVUID=>'||p_PARENTTXNVUID||CHR(10)
		;
v_batchid		VARCHAR2(65);
v_separator		VARCHAR2(1)	DEFAULT '.';
v_orderprefix		cf_batch.OrderPrefix%TYPE;
v_refcursor		db_ref.RCT;
v_retcd			pls_integer;

BatchNotFound		EXCEPTION;
PRAGMA EXCEPTION_INIT ( BatchNotFound, -02291 );
---------------------------------------------------------------------------
--			 M    A    I    N			      --
---------------------------------------------------------------------------

BEGIN
	BEGIN
		Insert into CF_BATCHCONTENTS(
			 BATCHID
		 	,VUID
			,ORDERING
	   		,STATUS
	   		,CREATEDATE
	   		,UPDATEDATE
			)
		 Values (
			p_BATCHID
	   		,p_VUID
		 	,p_ORDERING
			,p_STATUS
			,v_BeginTS
			,v_BeginTS
			);
	EXCEPTION
		WHEN BatchNotFound THEN

		IF p_PARENTBATCHID IS NOT NULL AND p_PARENTTXNVUID IS NOT NULL THEN
			SELECT	COALESCE( orderprefix, '')
			INTO	v_orderprefix
			FROM	cf_batch
			WHERE	batchid = p_PARENTBATCHID
			;
			SELECT	LTRIM( TRIM(v_orderprefix) || v_separator || TRIM(TO_CHAR( ordering )), v_separator)
			INTO	v_orderprefix
			FROM	cf_batchContents
			WHERE	vuid = p_PARENTTXNVUID
			AND	batchid = p_PARENTBATCHID
			;
		ELSE
			v_orderprefix := NULL; -- Batch is a RootBatch
		END IF;
	
	--do the batch insert first
	--and attempt the batchContents insert again
		CF_BATCHI(
			p_ReturnCd 	=>	v_retcd
			,p_DebugFlg 	=>	p_DebugFlg
			,RC1 		=>	v_refcursor
			,p_BATCHID	=>	p_BATCHID
			,p_PARENTBATCHID=>	p_PARENTBATCHID
			,p_BATCHCOUNT	=>	p_BATCHCOUNT
			,p_BATCHSTATUS	=>	p_BATCHSTATUS
			,p_ORDERPREFIX	=>	v_orderprefix
			);

		Insert into CF_BATCHCONTENTS(
			 BATCHID
		 	,VUID
			,ORDERING
	   		,STATUS
	   		,CREATEDATE
	   		,UPDATEDATE
			)
		 Values (
			p_BATCHID
	   		,p_VUID
		 	,p_ORDERING
			,p_STATUS
			,v_BeginTS
			,v_BeginTS
			);
	END;
  IF p_BATCHCOUNT IS NULL THEN
	UPDATE	cf_batch
	SET	batchCount = batchCount +1
		,UPDATEDATE = v_BeginTS
	WHERE	batchId = p_BATCHID
	;
	END IF;

	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		1 RowTS From Dual;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 CF_BATCHCONTENTSI' || SQLCODE|| ' '||SQLERRM);
END CF_BATCHCONTENTSI_AH1;
/


-- CF_BATCHCONTENTSS.sql

CREATE OR REPLACE PROCEDURE CF_BATCHCONTENTSS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_BATCHID IN VARCHAR2
	, p_VUID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_BATCHCONTENTSS
	Create Date:     2004-08-12 10:25:34
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BATCHCONTENTSS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

IF p_VUID IS NOT NULL THEN
	OPEN RC1 FOR 
	SELECT
		RTRIM(BATCHID) BATCHID
		,RTRIM(VUID) VUID
		,ORDERING
		,STATUS
		,ROWTS
		,CREATEDATE
	FROM 
		CF_BATCHCONTENTS
	Where 
		BATCHID = p_BATCHID
		AND VUID = p_VUID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		RTRIM(BATCHID) BATCHID
		,RTRIM(VUID) VUID
		,ORDERING
		,STATUS
		,ROWTS
		,CREATEDATE
	FROM 
		CF_BATCHCONTENTS
   Where 
		BATCHID = p_BATCHID
	;
	END IF;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_BATCHCONTENTSS;
/


-- CF_BATCHCONTENTSU.sql

CREATE OR REPLACE PROCEDURE CF_BATCHCONTENTSU
	(
	 p_ReturnCd out INTEGER
	,p_DebugFlg in PLS_INTEGER
	,RC1 out db_ref.RCT
	,p_BATCHID in VARCHAR2
	,p_VUID in VARCHAR2
	,p_ORDERING in NUMBER
	,p_STATUS in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_BATCHCONTENTSU
	Create Date:     2004-08-12 10:25:55
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2004-08-12 SYSTEM	4.3	Created
	2005-04-29 S.Priani	4.3	BGTRK00048595 - ReWrite
  2006-06-07 S.Narain	6.1	JR23845
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BATCHCONTENTSU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_STATUS=>'||p_STATUS||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS            INTEGER := 0;
zeroUpdate         EXCEPTION;
Not_Complete_Batch EXCEPTION;

v_BatchID		VARCHAR2(65);
v_BatchComplete		INTEGER DEFAULT 1;
v_BatchInvalidated	INTEGER DEFAULT 2;
v_CheckParents		BOOLEAN DEFAULT TRUE;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;

   --update the original record
	UPDATE	CF_BATCHCONTENTS cfb
	SET	ORDERING= NVL(p_ORDERING, cfb.ORDERING)
	,      STATUS=p_STATUS
	,      ROWTS= NVL(p_RowTS, cfb.ROWTS) + 1
   	,      UPDATEDATE= v_BeginTS
	WHERE BATCHID = p_BATCHID AND 
         VUID = p_VUID ;
  v_ROWTS := NVL(p_RowTS, 0) + 1;

	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	COMMIT;

   --update status of the Batch and it's parent heirarchy
	v_BatchID := p_BatchId;

	WHILE v_CheckParents = TRUE
        	AND (p_STATUS = v_BatchComplete OR p_STATUS = v_BatchInvalidated)
	LOOP
		IF p_STATUS = v_BatchComplete THEN

			UPDATE	cf_batch
			SET	batchstatus = v_BatchComplete
				,UPDATEDATE= v_BeginTS
			WHERE	batchid = v_BatchID
			--all child batches are complete
			AND	NOT EXISTS (SELECT 'x' 
					FROM	cf_batch
					WHERE	parentbatchid = v_BatchID
					AND	batchstatus <> v_BatchComplete)
			--all transactions are complete
			AND	EXISTS (SELECT 'x'
					FROM	cf_batch
					WHERE	batchid = v_BatchID
					AND 	((batchcount = 0)
						 OR
						 (batchcount = (SELECT count(*)
								FROM	cf_batchcontents
								WHERE	batchid = v_BatchID
								AND	status = v_BatchComplete
								)
						 )
						)
					)
			;
	
		ELSIF p_STATUS = v_BatchInvalidated THEN
			--if one part fails the entire batch fails
			UPDATE cf_batch
			SET	batchstatus = v_BatchInvalidated
				,UPDATEDATE= v_BeginTS
			WHERE	batchid = v_BatchID
			;
	
		END IF;

		COMMIT;

		--get the parent id of the current batch and complete the process
		SELECT	parentbatchid
		INTO	v_BatchID
		FROM	cf_batch
		WHERE	batchid = v_BatchID
		;

		--if no parent then flip the switch to exit the loop
		IF (v_BatchID IS NULL OR LTRIM(RTRIM(v_BatchID)) = '') THEN
			v_CheckParents := FALSE;
		END IF;

	END LOOP;

	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	
   EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_BATCHCONTENTSU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_BATCHCONTENTSU: No Data Found');      
      WHEN Not_Complete_Batch THEN
         p_ReturnCd := 0;
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_BATCHCONTENTSU:'||SQLERRM);
End CF_BATCHCONTENTSU;
/


-- CF_BATCHD.sql

Create or Replace Procedure CF_BATCHD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer   
	,p_BATCHID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_BATCHD
	Create Date:     2004-08-12 10:26:23
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BATCHD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_BATCH
	Where 
		BATCHID = p_BATCHID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_BATCHD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_BATCHD;
/


-- CF_BATCHI.sql

Create or Replace Procedure CF_BATCHI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_BATCHID in VARCHAR2
	,p_PARENTBATCHID in VARCHAR2
	,p_BATCHCOUNT in NUMBER
	,p_BATCHSTATUS in NUMBER
	,p_ORDERPREFIX in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_BATCHI
	Create Date:     2004-08-12 10:26:49
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2004-10-27 S.Priani	4.3	BGTRK00054219
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BATCHI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		||', p_PARENTBATCHID=>'||p_PARENTBATCHID||CHR(10)
		||', p_BATCHCOUNT=>'||p_BATCHCOUNT||CHR(10)
		||', p_BATCHSTATUS=>'||p_BATCHSTATUS||CHR(10)
		||', p_ORDERPREFIX=>'||p_ORDERPREFIX||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_BATCH(
		BATCHID
		,PARENTBATCHID
		,BATCHCOUNT
		,BATCHSTATUS
		,ORDERPREFIX
		,CREATEDATE
		,UPDATEDATE
	) Values (
		p_BATCHID
		,p_PARENTBATCHID
		,p_BATCHCOUNT
		,p_BATCHSTATUS
		,p_ORDERPREFIX
		,v_BeginTS
		,v_BeginTS
	);
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 CF_BATCHI' || SQLCODE|| ' '||SQLERRM);
END CF_BATCHI;
/


-- CF_BATCHS.sql

CREATE OR REPLACE PROCEDURE CF_BATCHS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_BATCHID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_BATCHS
	Create Date:     2004-08-12 10:27:06
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2004-10-27 S.Priani	4.3	BGTRK00054219
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BATCHS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	IF p_BATCHID IS NOT NULL THEN
	OPEN RC1 FOR 
	SELECT
		RTRIM(BATCHID) BATCHID
		,RTRIM(PARENTBATCHID) PARENTBATCHID
		,BATCHCOUNT
		,BATCHSTATUS
		,ORDERPREFIX
		,ROWTS    
	FROM 
		CF_BATCH
	Where 
		BATCHID = p_BATCHID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		RTRIM(BATCHID) BATCHID
		,RTRIM(PARENTBATCHID) PARENTBATCHID
		,BATCHCOUNT
		,BATCHSTATUS
		,ORDERPREFIX
		,ROWTS    
	FROM 
		CF_BATCH
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_BATCHS;
/


-- CF_BATCHU.sql

Create or Replace Procedure CF_BATCHU
	(
	 p_ReturnCd out integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_BATCHID in VARCHAR2
	,p_PARENTBATCHID in VARCHAR2
	,p_BATCHCOUNT in NUMBER
	,p_BATCHSTATUS in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_BATCHU
	Create Date:     2004-08-12 10:27:24
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BATCHU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		||', p_PARENTBATCHID=>'||p_PARENTBATCHID||CHR(10)
		||', p_BATCHCOUNT=>'||p_BATCHCOUNT||CHR(10)
		||', p_BATCHSTATUS=>'||p_BATCHSTATUS||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_BATCH
	SET
		PARENTBATCHID=p_PARENTBATCHID
		,      BATCHCOUNT=p_BATCHCOUNT
		,      BATCHSTATUS=p_BATCHSTATUS
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		BATCHID = p_BATCHID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_BATCHU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_BATCHU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_BATCHU:'||SQLERRM);
End CF_BATCHU;
/



-- CF_BUSINESSLOGOI_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_BusinessLogoI_ah1
    (
    p_ReturnCd out integer
    ,p_DebugFlg in pls_integer
--    ,p_SQLERRM out varchar2
    ,RC1 out db_ref.RCT
    ,p_BUSINESSLOGONAME in VARCHAR2
    ,p_BRANDINGLOGOIMAGE in BLOB
    ,p_BRANDINGBACKGROUNDIMAGE in BLOB
    )
AS

/*
    ===============================================================================
=
    Proc Name:       CF_BusinessLogoI_ah1
    Create Date:     2003-04-11 14:04:33
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-11 SYSTEM           3.5 Created

    ===============================================================================
=
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar(30) := 'CF_BusinessLogoI_ah1';
v_BUSINESSLOGOID            CF_BusinessLogo.BUSINESSLOGOID%Type;
v_BRANDINGLOGOIMAGE         CF_BusinessLogo.BRANDINGLOGOIMAGE%Type;
v_BRANDINGBACKGROUNDIMAGE   CF_BusinessLogo.BRANDINGBACKGROUNDIMAGE%Type;
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_BUSINESSLOGONAME=>'||p_BUSINESSLOGONAME||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    Insert into CF_BusinessLogo(
        BUSINESSLOGOID
        ,BUSINESSLOGONAME
        ,BRANDINGLOGOIMAGE
        ,BRANDINGBACKGROUNDIMAGE
    ) Values (
        CF_BUSINESSLOGO_SEQ.nextval
        ,p_BUSINESSLOGONAME
        ,p_BRANDINGLOGOIMAGE
        ,p_BRANDINGBACKGROUNDIMAGE
    )RETURNING BUSINESSLOGOID, BRANDINGlogoIMAGE, BRANDINGbackgroundIMAGE
    INTO
    v_BUSINESSLOGOID, v_BRANDINGlogoIMAGE, v_BRANDINGbackgroundIMAGE
    ;
    p_ReturnCd := 0;
    v_RowCnt := sql%ROWCOUNT;
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 FOR Select
        v_BUSINESSLOGOID            BUSINESSLOGOID,
        v_BRANDINGlogoIMAGE         BRANDINGlogoIMAGE,
        v_BRANDINGbackgroundIMAGE   BRANDINGbackgroundIMAGE,
        1 RowTS
        From Dual;
--        CF_BusinessLogo_SEQ.currval ID,
    p_ReturnCd := 0;
 --   Commit;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
            RAISE_APPLICATION_ERROR (-20999,'20000 CF_BusinessLogoI_ah1 NO DATA FOUND ' ||SQLERRM);
        When DUP_VAL_ON_INDEX then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            p_returnCd := 2;
            RAISE_APPLICATION_ERROR (-20999,'20001 CF_BusinessLogoI_ah1 Primary Key Violation ' ||SQLERRM);
        When OTHERS Then
            Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR (-20999,'20002 CF_BusinessLogoI_ah1 ' ||SQLERRM);

END CF_BusinessLogoI_ah1;
/


-- CF_BUSINESSLOGOS.sql

CREATE OR REPLACE PROCEDURE CF_BUSINESSLOGOS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_BUSINESSLOGOID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_BUSINESSLOGOS
	Create Date:     2003-09-26 00:07:09
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BUSINESSLOGOS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSINESSLOGOID=>'||p_BUSINESSLOGOID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_BUSINESSLOGOID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		BUSINESSLOGOID
		,RTRIM(BUSINESSLOGONAME) BUSINESSLOGONAME
		,BRANDINGLOGOIMAGE
		,BRANDINGBACKGROUNDIMAGE
		,ROWTS
		,CREATEDATE
	FROM
		CF_BUSINESSLOGO
	Where
		BUSINESSLOGOID = p_BUSINESSLOGOID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		BUSINESSLOGOID
		,RTRIM(BUSINESSLOGONAME) BUSINESSLOGONAME
		,BRANDINGLOGOIMAGE
		,BRANDINGBACKGROUNDIMAGE
		,ROWTS
		,CREATEDATE
	FROM
		CF_BUSINESSLOGO
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_BUSINESSLOGOS;
/


-- CF_BUSINESSLOGOU.sql

CREATE OR REPLACE Procedure CF_BUSINESSLOGOU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_BUSINESSLOGOID in NUMBER
	,p_BUSINESSLOGONAME in VARCHAR2
	,p_BRANDINGLOGOIMAGE in BLOB
	,p_BRANDINGBACKGROUNDIMAGE in BLOB
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_BUSINESSLOGOU
	Create Date:     2003-09-26 08:20:46
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_BUSINESSLOGOU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSINESSLOGOID=>'||p_BUSINESSLOGOID||CHR(10)
		||', p_BUSINESSLOGONAME=>'||p_BUSINESSLOGONAME||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_BUSINESSLOGO
	SET
		BUSINESSLOGONAME=p_BUSINESSLOGONAME
		,      BRANDINGLOGOIMAGE=p_BRANDINGLOGOIMAGE
		,      BRANDINGBACKGROUNDIMAGE=p_BRANDINGBACKGROUNDIMAGE
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		BUSINESSLOGOID = p_BUSINESSLOGOID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_BUSINESSLOGOU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_BUSINESSLOGOU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_BUSINESSLOGOU:'||SQLERRM);
End CF_BUSINESSLOGOU;
/


-- CF_CONTENTTYPEFILTERD.sql

CREATE OR REPLACE Procedure CF_CONTENTTYPEFILTERD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_CONTENTTYPEFILTERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTENTTYPEFILTERD
	Create Date:     2003-09-26 08:23:53
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTENTTYPEFILTERD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONTENTTYPEFILTERID=>'||p_CONTENTTYPEFILTERID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_CONTENTTYPEFILTER
	Where
		CONTENTTYPEFILTERID = p_CONTENTTYPEFILTERID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_CONTENTTYPEFILTERD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_CONTENTTYPEFILTERD;
/


-- CF_CONTENTTYPEFILTERI.sql

Create or Replace Procedure CF_CONTENTTYPEFILTERI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGOBJID in NUMBER
	,p_FILTERTYPECD in NUMBER
	,p_CONTENTTYPE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTENTTYPEFILTERI
	Create Date:     2003-11-21 14:19:36
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-11-21 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTENTTYPEFILTERI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		||', p_FILTERTYPECD=>'||p_FILTERTYPECD||CHR(10)
		||', p_CONTENTTYPE=>'||p_CONTENTTYPE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_CONTENTTYPEFILTER(
		CONTENTTYPEFILTERID
		,ROUTINGOBJID
		,FILTERTYPECD
		,CONTENTTYPE
	) Values (
		CF_CONTENTTYPEFILTER_SEQ.nextval
		,p_ROUTINGOBJID
		,p_FILTERTYPECD
		,p_CONTENTTYPE
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_CONTENTTYPEFILTER_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_CONTENTTYPEFILTERI' || SQLCODE|| ' '||SQLERRM);
END CF_CONTENTTYPEFILTERI;
/


-- CF_CONTENTTYPEFILTERS.sql

CREATE OR REPLACE PROCEDURE CF_CONTENTTYPEFILTERS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CONTENTTYPEFILTERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTENTTYPEFILTERS
	Create Date:     2003-09-26 00:07:07
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTENTTYPEFILTERS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONTENTTYPEFILTERID=>'||p_CONTENTTYPEFILTERID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_CONTENTTYPEFILTERID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		CONTENTTYPEFILTERID
		,ROUTINGOBJID
		,FILTERTYPECD
		,RTRIM(CONTENTTYPE) CONTENTTYPE
		,ROWTS
		,CREATEDATE
	FROM
		CF_CONTENTTYPEFILTER
	Where
		CONTENTTYPEFILTERID = p_CONTENTTYPEFILTERID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		CONTENTTYPEFILTERID
		,ROUTINGOBJID
		,FILTERTYPECD
		,RTRIM(CONTENTTYPE) CONTENTTYPE
		,ROWTS
		,CREATEDATE
	FROM
		CF_CONTENTTYPEFILTER
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_CONTENTTYPEFILTERS;
/


-- CF_CONTENTTYPEFILTERS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_CONTENTTYPEFILTERS_AH1
	(
	p_ReturnCd          OUT PLS_INTEGER
	, p_DebugFlg        IN  PLS_INTEGER
	, RC1               OUT DB_REF.RCT
	, p_ROUTINGOBJID    IN PLS_INTEGER
    , p_FILTERTYPECD    IN PLS_INTEGER  -- 1 = Include  2 = Exclude
    , p_CONTENTTYPE     IN Varchar2     -- Supports wild card
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTENTTYPEFILTERS_AH1
	Create Date:     2003-07-12 14:13:25
	Author:          V.Goel
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTENTTYPEFILTERS_AH1';
v_ExceptionFlg    integer Default 0;
v_Where           Varchar2(200);
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
        ||', p_FILTERTYPECD=>'||p_FILTERTYPECD||CHR(10)
        ||', p_CONTENTTYPE=>'||p_CONTENTTYPE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_Where := ' Where 1 = 1'
            ||FN_BUILD_WHERE('C.ROUTINGOBJID', 'NUMBER', p_ROUTINGOBJID)
            ||FN_BUILD_WHERE('C.FILTERTYPECD', 'NUMBER', p_FILTERTYPECD)
            ||FN_BUILD_WHERE('C.CONTENTTYPE' , 'VARCHAR2_LIKE', p_CONTENTTYPE);

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
	'SELECT
		CONTENTTYPEFILTERID
		,C.ROUTINGOBJID
		,FILTERTYPECD
		,RTRIM(CONTENTTYPE) CONTENTTYPE
        ,RTRIM(ROUTINGOBJCD) ROUTINGOBJCD
        ,RTRIM(ROUTINGOBJVER) ROUTINGOBJVER
	FROM  CF_ContentTypeFilter  C
    Inner Join CF_RoutingObject R ON C.ROUTINGOBJID = R.ROUTINGOBJID '||v_Where ||' Order By C.ROUTINGOBJID';
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_CONTENTTYPEFILTERS_AH1;
/


-- CF_CONTENTTYPEFILTERU.sql

CREATE OR REPLACE Procedure CF_CONTENTTYPEFILTERU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_CONTENTTYPEFILTERID in NUMBER
	,p_ROUTINGOBJID in NUMBER
	,p_FILTERTYPECD in NUMBER
	,p_CONTENTTYPE in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_CONTENTTYPEFILTERU
	Create Date:     2003-09-26 08:20:48
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTENTTYPEFILTERU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONTENTTYPEFILTERID=>'||p_CONTENTTYPEFILTERID||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		||', p_FILTERTYPECD=>'||p_FILTERTYPECD||CHR(10)
		||', p_CONTENTTYPE=>'||p_CONTENTTYPE||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_CONTENTTYPEFILTER
	SET
		ROUTINGOBJID=p_ROUTINGOBJID
		,      FILTERTYPECD=p_FILTERTYPECD
		,      CONTENTTYPE=p_CONTENTTYPE
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		CONTENTTYPEFILTERID = p_CONTENTTYPEFILTERID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_CONTENTTYPEFILTERU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_CONTENTTYPEFILTERU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_CONTENTTYPEFILTERU:'||SQLERRM);
End CF_CONTENTTYPEFILTERU;
/


-- CF_CONTROLNUMD.sql

Create or Replace Procedure CF_CONTROLNUMD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_ControlNumber in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTROLNUMD
	Create Date:     2004-08-09 13:17:28
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTROLNUMD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ControlNumber=>'||p_ControlNumber||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_CONTROLNUM
	Where 
		controlnumberid = p_ControlNumber
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_CONTROLNUMD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_CONTROLNUMD;
/


-- CF_CONTROLNUMI.sql

Create or Replace Procedure CF_CONTROLNUMI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
   ,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_DOCTYPE in VARCHAR2
	,p_INTCTLNUM in VARCHAR2
	,p_GRPCTLNUM in VARCHAR2
	,p_TXCTLNUM in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTROLNUMI
	Create Date:     2004-08-09 13:17:02
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTROLNUMI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
      ||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
      ||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_DOCTYPE=>'||p_DOCTYPE||CHR(10)
		||', p_INTCTLNUM=>'||p_INTCTLNUM||CHR(10)
		||', p_GRPCTLNUM=>'||p_GRPCTLNUM||CHR(10)
		||', p_TXCTLNUM=>'||p_TXCTLNUM||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_CONTROLNUM(
		CONTROLNUMBERID
		,FROMPARTNERID
		,TOPARTNERID
		,DOCTYPE
		,INTCTLNUM
		,GRPCTLNUM
		,TXCTLNUM
      ,UPDATEDATE
		,CREATEDATE
	) Values (
      CF_CONTROLNUM_SEQ.nextval
      ,p_FROMPARTNERID
		,p_TOPARTNERID
		,p_DOCTYPE
		,p_INTCTLNUM
		,p_GRPCTLNUM
		,p_TXCTLNUM
      ,v_BeginTS
		,v_BeginTS
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_CONTROLNUM_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_CONTROLNUMI' || SQLCODE|| ' '||SQLERRM);
END CF_CONTROLNUMI;
/


-- CF_CONTROLNUMS.sql

CREATE OR REPLACE PROCEDURE CF_CONTROLNUMS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ControlNumber IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTROLNUMS
	Create Date:     2004-08-09 13:17:45
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTROLNUMS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ControlNumber=>'||p_ControlNumber||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ControlNumber IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		CONTROLNUMBERID
		,FROMPARTNERID
		,TOPARTNERID
		,RTRIM(DOCTYPE) DOCTYPE
		,RTRIM(INTCTLNUM) INTCTLNUM
		,RTRIM(GRPCTLNUM) GRPCTLNUM
		,RTRIM(TXCTLNUM) TXCTLNUM
		,ROWTS
		,CREATEDATE
	FROM 
		CF_CONTROLNUM
	Where 
		controlnumberid = p_ControlNumber
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		CONTROLNUMBERID
		,FROMPARTNERID
		,TOPARTNERID
		,RTRIM(DOCTYPE) DOCTYPE
		,RTRIM(INTCTLNUM) INTCTLNUM
		,RTRIM(GRPCTLNUM) GRPCTLNUM
		,RTRIM(TXCTLNUM) TXCTLNUM
		,ROWTS
		,CREATEDATE
	FROM 
		CF_CONTROLNUM
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_CONTROLNUMS;
/

-- CF_CONTROLNUMS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_CONTROLNUMS_AH1
	(
	  p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
   , p_FromPartnerID IN NUMBER
   , p_ToPartnerID   IN NUMBER
   , p_DocType       IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTROLNUMS_AH1
	Create Date:     2004-08-09 13:21:25
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTROLNUMS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FromPartnerID=>'||p_FromPartnerID||CHR(10)
      ||', p_ToPartnerID=>'||p_ToPartnerID||CHR(10)
      ||', p_DocType=>'||p_DocType||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_DocType IS NULL
 THEN
	OPEN RC1 FOR 
	SELECT controlnumberid, frompartnerid, topartnerid, doctype, intctlnum, grpctlnum, txctlnum, rowts
   FROM cf_controlnum
   WHERE doctype IS NULL AND
      frompartnerid = p_FromPartnerID AND
      topartnerid = p_ToPartnerID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT controlnumberid, frompartnerid, topartnerid, doctype, intctlnum, grpctlnum, txctlnum, rowts
   FROM cf_controlnum
   WHERE frompartnerid = p_FromPartnerID AND
      topartnerid = p_ToPartnerID AND 
      doctype = p_DocType
   ;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_CONTROLNUMS_AH1;
/


-- CF_CONTROLNUMS_AH2.sql

CREATE OR REPLACE PROCEDURE CF_CONTROLNUMS_AH2
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FromPartnerID IN NUMBER
   , p_ToPartnerID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTROLNUMS_AH2
	Create Date:     2004-08-09 13:21:35
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTROLNUMS_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FromPartnerID=>'||p_FromPartnerID||CHR(10)
      ||', p_ToPartnerID=>'||p_ToPartnerID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

v_Where  VARCHAR2(4000) default '' ;
stmtSql  varchar2(2000);

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;    
   
   v_Where := ' WHERE 1=1 '
               ||' '||FN_BUILD_WHERE('frompartnerid', 'NUMBER', TO_CHAR(p_FromPartnerID))
               ||' '||FN_BUILD_WHERE('topartnerid', 'NUMBER', TO_CHAR(p_ToPartnerID));

   OPEN RC1 FOR 'SELECT controlnumberid, frompartnerid, topartnerid, doctype, intctlnum, grpctlnum, txctlnum, rowts'
              ||' FROM cf_controlnum '
              ||v_Where;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_CONTROLNUMS_AH2;
/


-- CF_CONTROLNUMS_AH3.sql

CREATE OR REPLACE PROCEDURE CF_CONTROLNUMS_AH3
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
   , RC2  OUT  DB_REF.RCT
   , p_sortkey IN VARCHAR2
   , p_PageSizeNbr IN NUMBER
   , p_PageNbr IN NUMBER
	, p_FromPartnerID IN NUMBER
   , p_ToPartnerID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_CONTROLNUMS_AH3
	Create Date:     2004-08-09 13:21:35
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTROLNUMS_AH3';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FromPartnerID=>'||p_FromPartnerID||CHR(10)
      ||', p_ToPartnerID=>'||p_ToPartnerID||CHR(10)
      ||', p_sortkey=>'||p_sortkey||CHR(10)
      ||', p_PageSizeNbr=>'||p_PageSizeNbr||CHR(10)
      ||', p_PageNbr=>'||p_PageNbr||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

v_where    VARCHAR2(1000);
v_sqlstmt  VARCHAR2(2000);
v_sqlstmtT VARCHAR2(4000);
v_sqlstmtF VARCHAR2(4000);
--needed for paging
v_sortKey        VARCHAR2(4000);
sql_inner        VARCHAR2(4000);
v_count          INTEGER;
--

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;    
   
   v_where := ' WHERE 1=1 '
               ||' '||FN_BUILD_WHERE('partnerid', 'NUMBER', TO_CHAR(p_FromPartnerID))
               ||' '||FN_BUILD_WHERE('topartnerid', 'NUMBER', TO_CHAR(p_ToPartnerID));
   
   --v_sqlstmt := 'SELECT controlnumberid, frompartnerid, topartnerid, doctype, intctlnum, grpctlnum, txctlnum, rowts'
   --            ||' FROM cf_controlnum ';
   --v_sqlstmt := v_sqlstmt||v_where;

   v_sqlstmtT := 'SELECT p.partnername tname, c.toPartnerId, c.controlnumberid toStmtCNid, c.docType, c.intctlnum, c.grpctlnum, c.txctlnum, c.rowts '
                       || ' FROM cf_controlnum c, pr_partner p '
                       || ' WHERE c.topartnerid=p.partnerid ' 
                       ||FN_BUILD_WHERE('c.topartnerid', 'NUMBER', TO_CHAR(p_ToPartnerID));
   v_sqlstmtF := 'SELECT p.partnername fname, c.fromPartnerId, c.controlnumberid fromStmtCNid '
                       || ' FROM cf_controlnum c, pr_partner p '
                       || ' WHERE c.frompartnerid=p.partnerid ' 
                       ||FN_BUILD_WHERE('c.frompartnerid', 'NUMBER', TO_CHAR(p_FromPartnerID));

   v_sqlstmt := 'SELECT tname toPartnerName,fname fromPartnerName, toPartnerId, fromPartnerId, toStmtCNid controlNumberId, docType, intctlnum, grpctlnum, txctlnum, rowts from ( '||v_sqlstmtT
             || ')  toSTMT, (' || v_sqlstmtF || ')  fromSTMT WHERE 1=1 AND toStmtCNid=fromStmtCNid';
   --paging stuff
   v_sqlstmt := v_sqlstmt ||' ORDER BY UPPER( fromPartnerName ) ASC, UPPER( toPartnerName) ASC,  UPPER( docType) ASC';
   
   v_sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, v_sqlstmt);
   sql_inner := 'FROM CF_CONTROLNUM';
   --

   IF RC1%isOpen THEN CLOSE RC1; END IF;
   OPEN RC1 FOR v_sqlstmt;

   --more paging
   IF RC2%ISOPEN THEN CLOSE RC2; END IF;
   IF NVL(P_PAGENBR,0) IN (0,1) THEN
       Execute IMMEDIATE 'SELECT COUNT(*)  '||sql_inner ||v_where INTO v_count;
       OPEN RC2 FOR SELECT v_count TotalRowsNbr FROM dual;
   END IF;
   --
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_CONTROLNUMS_AH3;
/


-- CF_CONTROLNUMU.sql

Create or Replace Procedure CF_CONTROLNUMU
	(
	 p_ReturnCd out integer
	,p_DebugFlg in pls_integer
   ,p_ControlNumber in NUMBER
	,p_FromPartnerID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_DOCTYPE in VARCHAR2
	,p_INTCTLNUM in VARCHAR2
	,p_GRPCTLNUM in VARCHAR2
	,p_TXCTLNUM in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_CONTROLNUMU
	Create Date:     2004-08-09 13:17:59
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_CONTROLNUMU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ControlNumber=>'||p_ControlNumber||CHR(10)
		||', p_FromPartnerID=>'||p_FromPartnerID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_DOCTYPE=>'||p_DOCTYPE||CHR(10)
		||', p_INTCTLNUM=>'||p_INTCTLNUM||CHR(10)
		||', p_GRPCTLNUM=>'||p_GRPCTLNUM||CHR(10)
		||', p_TXCTLNUM=>'||p_TXCTLNUM||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
--if the control number isn't passed in then create the row
--ow check the doctype for the null/not null syntax
IF p_ControlNumber IS NULL THEN
   INSERT INTO cf_controlnum 
      (controlnumberid,frompartnerid, topartnerid, doctype, intctlnum, grpctlnum, txctlnum,UPDATEDATE,CREATEDATE)
   VALUES 
      (CF_CONTROLNUM_SEQ.nextval,p_FromPartnerID, p_ToPartnerID, p_DocType, p_IntCtlNum, p_GrpCtlNum, p_TxCtlNum,v_BeginTS,v_BeginTS);
ELSE
   v_RowCnt := 0;
	
      Update CF_CONTROLNUM
	   SET  
                INTCTLNUM=p_INTCTLNUM
		   ,      GRPCTLNUM=p_GRPCTLNUM
		   ,      TXCTLNUM=p_TXCTLNUM
		   ,      ROWTS= p_RowTS + 1
		   ,      UPDATEDATE= v_BeginTS
      WHERE controlnumberid = p_ControlNumber AND
         RowTS = p_RowTS returning RowTS into v_RowTS;
   
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
END IF;
   
   
   --IF RC1%isOpen THEN Close RC1; END IF;
	--OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_CONTROLNUMU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_CONTROLNUMU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_CONTROLNUMU:'||SQLERRM);
End CF_CONTROLNUMU;
/


-- CF_EDIFAMAPD.sql

Create or Replace Procedure CF_EDIFAMAPD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_FAMAPID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPD
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FAMAPID=>'||p_FAMAPID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_EDIFAMAP
	Where 
		FAMAPID = p_FAMAPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_EDIFAMAPD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_EDIFAMAPD;
/


-- CF_EDIFAMAPI.sql

Create or Replace Procedure CF_EDIFAMAPI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_FAMAPFILENAME in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_FAMAP in BLOB
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPI
	Create Date:     2004-09-08 10:48:14
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FAMAPFILENAME=>'||p_FAMAPFILENAME||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Insert into CF_EDIFAMAP(
		FAMAPID
		,FAMAPFILENAME
		,RESOURCEKEY
		,DESCRIPTION
		,FAMAP
	) Values (
		CF_EDIFAMAP_SEQ.nextval
		,p_FAMAPFILENAME
		,p_RESOURCEKEY
		,p_DESCRIPTION
		,p_FAMAP
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_EDIFAMAP_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_EDIFAMAPI' || SQLCODE|| ' '||SQLERRM);
END CF_EDIFAMAPI;
/


-- CF_EDIFAMAPS.sql

CREATE OR REPLACE PROCEDURE CF_EDIFAMAPS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FAMAPID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPS
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FAMAPID=>'||p_FAMAPID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_FAMAPID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		FAMAPID
		,ROWTS
		,CREATEDATE
		,RTRIM(FAMAPFILENAME) FAMAPFILENAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,FAMAP
	FROM 
		CF_EDIFAMAP
	Where 
		FAMAPID = p_FAMAPID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		FAMAPID
		,ROWTS
		,CREATEDATE
		,RTRIM(FAMAPFILENAME) FAMAPFILENAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,FAMAP
	FROM 
		CF_EDIFAMAP
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_EDIFAMAPS;
/


-- CF_EDIFAMAPS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_EDIFAMAPS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FAMAPID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPS_AH1
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FAMAPID=>'||p_FAMAPID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_FAMAPID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		FAMAPID
		,RTRIM(FAMAPFILENAME) FAMAPFILENAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,ROWTS
	FROM 
		CF_EDIFAMAP
	Where 
		FAMAPID = p_FAMAPID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		FAMAPID
		,RTRIM(FAMAPFILENAME) FAMAPFILENAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,ROWTS
	FROM 
		CF_EDIFAMAP
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_EDIFAMAPS_AH1;
/


-- CF_EDIFAMAPS_AH2.sql

CREATE OR REPLACE PROCEDURE CF_EDIFAMAPS_AH2
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, RC2 out  db_ref.RCT
	, p_sortkey         in  varchar2
	, p_PageSizeNbr     in  pls_Integer
	, p_PageNbr         in  pls_Integer
	, p_FAMAPFILENAME IN varchar2
	, p_DESC IN varchar2
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPS_AH2
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPS_AH2';
v_ExceptionFlg    integer Default 0;

v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(4000);
  v_where            varchar2(4000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(4000);
  sql_inner          varchar2(4000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(4000);
v_ParamsTxt       varchar(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||',p_sortkey=>'|| p_sortkey||CHR(10)
        ||',p_PageSizeNbr=>'|| p_PageSizeNbr||CHR(10)
        ||',p_PageNbr=>'|| p_PageNbr||CHR(10)
        ||',p_DESC=>'|| p_DESC||CHR(10)
        ||',p_FAMAPFILENAME=>'|| p_FAMAPFILENAME||CHR(10);
Procedure where_clause as
begin
    v_where := ' Where 1=1'||
    FN_Build_Where('FAMAPFILENAME', 'VARCHAR2_LIKE'   , p_FAMAPFILENAME) ||
    FN_Build_Where('DESCRIPTION',     'VARCHAR2_LIKE'   , p_DESC);
end where_clause;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	p_ReturnCd := 0;
	    where_clause;
	
	    sqlstmt := 'SELECT  FAMAPID ,RTRIM(FAMAPFILENAME) FAMAPFILENAME, '
	    		    ||'RTRIM(RESOURCEKEY) RESOURCEKEY, RTRIM(DESCRIPTION) DESCRIPTION, '
		    ||'ROWTS FROM CF_EDIFAMAP ' || v_where;
	    
	  IF NVL(INSTR(p_sortkey,'ASC'),0) >  0 THEN
	    v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'ASC')-1);
	    sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ')'; 
	   ELSIF INSTR(upper(p_sortkey),'DESC') > 0 THEN
	      v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'DESC')-1);
	      sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ') DESC'; 
	   ELSE
	       sqlstmt := sqlstmt ||' order by FAMAPFILENAME DESC'; 
	  END IF; 
	        
	   sql_inner := 'FROM CF_EDIFAMAP';
	
	    -- total row count Cursor
	
	    IF RC2%ISOPEN THEN CLOSE RC2; END IF;
	    IF NVL(P_PAGENBR,0) IN (0,1) THEN
	      Execute Immediate 'select count(*)  '||sql_inner ||v_where INTO v_count;
	  	       open RC2 for select v_count TotalRowsNbr from dual;
	     end if;
	-- Paging logic
	     sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
	 --  Set_page_parameters(p_pageNbr, p_pageSizeNbr, sqlstmt);
	
	    IF RC1%isOpen THEN Close RC1; END IF;
	    Open RC1 for sqlstmt;
	      p_ReturnCd := 0;
	      DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	 Exception
	    When NO_DATA_FOUND THEN Null;
	    When OTHERS Then
	                p_ReturnCd     := 1;
	                v_ExceptionFlg := 1;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_EDIFAMAPS_AH2;
/


-- CF_EDIFAMAPS_AH3.sql

CREATE OR REPLACE PROCEDURE CF_EDIFAMAPS_AH3
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, RC2 out  db_ref.RCT
	, p_sortkey         in  varchar2
	, p_PageSizeNbr     in  pls_Integer
	, p_PageNbr         in  pls_Integer
	, p_FAMAPFILENAME IN varchar2
	, p_DESCRIPTION in varchar2
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPS_AH3
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPS_AH3';
v_ExceptionFlg    integer Default 0;

v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(4000);
  v_where            varchar2(4000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(4000);
  sql_inner          varchar2(4000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(4000);
v_ParamsTxt       varchar(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||',p_sortkey=>'|| p_sortkey||CHR(10)
        ||',p_PageSizeNbr=>'|| p_PageSizeNbr||CHR(10)
        ||',p_PageNbr=>'|| p_PageNbr||CHR(10)
        ||',p_DESCRIPTION=>'|| p_DESCRIPTION||CHR(10)
        ||',p_FAMAPFILENAME=>'|| p_FAMAPFILENAME||CHR(10);
Procedure where_clause as
begin
    v_where := ' Where 1=1'||
    FN_Build_Where('p_FAMAPFILENAME', 'VARCHAR2_LIKE'   , p_FAMAPFILENAME) ||
    FN_Build_Where('p_DESCRIPTION',     'VARCHAR2_LIKE'   , p_DESCRIPTION);
end where_clause;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	p_ReturnCd := 0;
	    where_clause;
	
	    sqlstmt := 'SELECT  FAMAPID ,RTRIM(FAMAPFILENAME) FAMAPFILENAME, '
	    		    ||'RTRIM(RESOURCEKEY) RESOURCEKEY, RTRIM(DESCRIPTION) DESCRIPTION, '
		    ||'ROWTS FROM CF_EDIFAMAP ' || v_where;
	    
	  IF NVL(INSTR(p_sortkey,'ASC'),0) >  0 THEN
	    v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'ASC')-1);
	    sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ')'; 
	   ELSIF INSTR(upper(p_sortkey),'DESC') > 0 THEN
	      v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'DESC')-1);
	      sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ') DESC'; 
	   ELSE
	       sqlstmt := sqlstmt ||' order by FAMAPFILENAME DESC'; 
	  END IF; 
	        
	   sql_inner := 'FROM CF_EDIFAMAP';
	
	    -- total row count Cursor
	
	    IF RC2%ISOPEN THEN CLOSE RC2; END IF;
	    IF NVL(P_PAGENBR,0) IN (0,1) THEN
	      Execute Immediate 'select count(*)  '||sql_inner ||v_where INTO v_count;
	  	       open RC2 for select v_count TotalRowsNbr from dual;
	     end if;
	-- Paging logic
	     sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
	 --  Set_page_parameters(p_pageNbr, p_pageSizeNbr, sqlstmt);
	
	    IF RC1%isOpen THEN Close RC1; END IF;
	    Open RC1 for sqlstmt;
	      p_ReturnCd := 0;
	      DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	 Exception
	    When NO_DATA_FOUND THEN Null;
	    When OTHERS Then
	                p_ReturnCd     := 1;
	                v_ExceptionFlg := 1;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_EDIFAMAPS_AH3;
/


-- CF_EDIFAMAPS_AH4.sql

CREATE OR REPLACE PROCEDURE CF_EDIFAMAPS_AH4
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_RoutingObjId IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPS_AH4
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	2006-03-21 Shishir N        6.1 60217-Partner Migration Changes	
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPS_AH4';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RoutingObjId=>'||p_RoutingObjId||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
  IF p_RoutingObjId IS NOT NULL
		 THEN
	OPEN RC1 FOR 
			SELECT
					CEF.FAMAPID FAMAPID
					,CEF.FAMAPFILENAME
					,RTRIM(CEF.RESOURCEKEY) RESOURCEKEY
					,RTRIM(CEF.DESCRIPTION) DESCRIPTION		
					,CEF.ROWTS ROWTS
					,CEF.FAMAP FAMAP
				FROM 
					CF_EDIFAMAP CEF,
					CF_EDIFAMAP_LNK CEL
				WHERE 
					CEL.ROUTINGOBJID = p_RoutingObjId
		and CEL.FAMAPID = CEF.FAMAPID;
	ELSE
	OPEN RC1 FOR 
	SELECT
			FAMAPID
			,RTRIM(FAMAPFILENAME) FAMAPFILENAME	
			,RTRIM(RESOURCEKEY) RESOURCEKEY
			,RTRIM(DESCRIPTION) DESCRIPTION		
			,ROWTS
			,FAMAP
		FROM 
			CF_EDIFAMAP;
    end if;										
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_EDIFAMAPS_AH4;
/


-- CF_EDIFAMAPS_AH5.sql

CREATE OR REPLACE PROCEDURE CF_EDIFAMAPS_AH5
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FAMAPID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPS_AH5
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPS_AH5';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FAMAPID=>'||p_FAMAPID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR 
	 SELECT  
                cro.routingobjid ROUTINGOBJID,
                cro.routingobjname ROUTINGOBJNAME,
                cro.routingobjcd ROUTINGOBJCD,
                cro.routingobjver ROUTINGOBJVER,
                cro2.routingobjname ROUTINGOBJPARENTNAME,
                cro3.routingobjname ROUTINGOBJGRPARENTNAME
        FROM 
                cf_routingobject cro
        INNER JOIN cf_edifamap_lnk cel on cel.routingobjid = cro.routingobjid
        INNER JOIN cf_routingobject05_vw crv on crv.routingobjid = cro.routingobjid 
        left outer JOIN cf_routingobject cro2 on cro2.routingobjid = crv.parentobjid
        left outer JOIN cf_routingobjref_lnk crl on crl.routingobjrefid = crv.grparentobjrefid
        left outer JOIN  cf_routingobject cro3 on cro3.routingobjid = crl.routingobjid
        WHERE
		CEL.FAMAPID = p_FAMAPID;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_EDIFAMAPS_AH5;
/

-- CF_EDIFAMAPS_AH6.sql

CREATE OR REPLACE PROCEDURE CF_EDIFAMAPS_AH6
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FAMAPID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPS_AH6
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPS_AH6';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FAMAPID=>'||p_FAMAPID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR 
	  SELECT
	 	                 crv.routingobjid ROUTINGOBJID,
	 	                 cro.routingobjname ROUTINGOBJNAME,
	 	                 crv.routingobjcd ROUTINGOBJCD,
                     crv.routingobjrefid ROUTINGOBJREFID,	 	                 
	 	                 crv.routingobjver ROUTINGOBJVER,
	 	                 cro2.routingobjname ROUTINGOBJPARENTNAME,
	 	                 cro3.routingobjname ROUTINGOBJGRPARENTNAME
	 	         FROM
	 	                 cf_routingobject05_vw crv
	 	         INNER JOIN pr_routingattr_lnk prl on prl.routingobjrefid = crv.routingobjrefid
	 	         INNER JOIN cf_routingobject cro on crv.routingobjid = cro.routingobjid
	 	         left outer JOIN cf_routingobject cro2 on cro2.routingobjid = crv.parentobjid
	 	         left outer JOIN cf_routingobjref_lnk crl on crl.routingobjrefid = crv.grparentobjrefid
	 	         left outer JOIN  cf_routingobject cro3 on cro3.routingobjid = crl.routingobjid
	 	         WHERE
		prl.FAMAPID = p_FAMAPID;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_EDIFAMAPS_AH6;
/

-- CF_EDIFAMAPU.sql

Create or Replace Procedure CF_EDIFAMAPU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_FAMAPID in NUMBER
	,p_ROWTS in NUMBER
	,p_FAMAPFILENAME in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_FAMAP in BLOB
) as
/*
	================================================================================
	Proc Name:       CF_EDIFAMAPU
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAPU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FAMAPID=>'||p_FAMAPID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_FAMAPFILENAME=>'||p_FAMAPFILENAME||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;
v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_RowCnt := 0;
	Update CF_EDIFAMAP
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      FAMAPFILENAME=p_FAMAPFILENAME
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
		,      FAMAP=p_FAMAP
	Where 
		FAMAPID = p_FAMAPID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_EDIFAMAPU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_EDIFAMAPU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_EDIFAMAPU:'||SQLERRM);
End CF_EDIFAMAPU;
/


-- CF_EDIFAMAP_LNKD.sql

Create or Replace Procedure CF_EDIFAMAP_LNKD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_EDIFAMAPLNKID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAP_LNKD
	Create Date:     2004-09-08 10:48:14
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAP_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_EDIFAMAPLNKID=>'||p_EDIFAMAPLNKID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_EDIFAMAP_LNK
	Where 
		EDIFAMAPLNKID = p_EDIFAMAPLNKID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_EDIFAMAP_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_EDIFAMAP_LNKD;
/


-- CF_EDIFAMAP_LNKD_AH1.sql

Create or Replace Procedure CF_EDIFAMAP_LNKD_AH1
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_FAMAPID in NUMBER
	,p_ROUTINGOBJID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAP_LNKD_AH1
	Create Date:     2008-09-22 19:18:14
	Author:          AMIT KUMAR VIJAY 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2008-09-22 AMIT VIJAY    6.2    Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAP_LNKD_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		||', p_FAMAPID=>'||p_FAMAPID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_EDIFAMAP_LNK
	Where 
		FAMAPID = p_FAMAPID
	and	ROUTINGOBJID = p_ROUTINGOBJID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_EDIFAMAP_LNKD_AH1: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_EDIFAMAP_LNKD_AH1;
/


-- CF_EDIFAMAP_LNKI.sql

Create or Replace Procedure CF_EDIFAMAP_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGOBJID in NUMBER
	,p_FAMAPID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_EDIFAMAP_LNKI
	Create Date:     2004-09-08 10:48:14
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_EDIFAMAP_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FAMAPID=>'||p_FAMAPID||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Insert into CF_EDIFAMAP_LNK(
		EDIFAMAPLNKID
		,FAMAPID
		,ROUTINGOBJID
	) Values (
		CF_EDIFAMAP_LNK_SEQ.nextval
		,p_FAMAPID
		,p_ROUTINGOBJID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_EDIFAMAP_LNK_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_EDIFAMAP_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_EDIFAMAP_LNKI;
/


-- CF_ENVELOPEQD.sql

Create or Replace Procedure CF_ENVELOPEQD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_VUID in VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQD
	Create Date:     2004-09-07 17:49:01
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-07 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_ENVELOPEQ
	Where 
		VUID = p_VUID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ENVELOPEQD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_ENVELOPEQD;
/


-- CF_ENVELOPEQI.sql

CREATE OR REPLACE PROCEDURE CF_ENVELOPEQI
	(p_ReturnCd		out	pls_integer
	,p_DebugFlg		in	pls_integer
	)
AS
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQI
	Create Date:     2004-09-07 17:49:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2005-05-09 S.Priani	4.3	BGTRK00048595
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

	/* This procedure is a NOP to be in sync with DB2 code
	To prevent lock escalation in DB2
	We are using temporary table to hold the data and then transfer it to CF_ENVELOPEQ
	*/
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 CF_ENVELOPEQI' || SQLCODE|| ' '||SQLERRM);
END CF_ENVELOPEQI;
/


-- CF_ENVELOPEQS.sql

CREATE OR REPLACE PROCEDURE CF_ENVELOPEQS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_VUID IN VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQS
	Create Date:     2004-09-07 17:49:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2004-10-27 S.Priani	4.3	BGTRK00054219
	2004-12-07 S.Priani	4.3	BGTRK00060863
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_VUID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		RTRIM(VUID) VUID
		,RTRIM(ROOTBATCHID) ROOTBATCHID
		,RTRIM(ACTIVITYID) ACTIVITYID
		,RTRIM(ORDERING) ORDERING
		,FROMPARTNERID
		,TOPARTNERID
		,ROWTS
		,ENVELOPEPROFILEID
		,CONNECTIONPROFILEID
		,CREATEDATE
	FROM 
		CF_ENVELOPEQ
	Where 
		VUID = p_VUID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		RTRIM(VUID) VUID
		,RTRIM(ROOTBATCHID) ROOTBATCHID
		,RTRIM(ACTIVITYID) ACTIVITYID
		,RTRIM(ORDERING) ORDERING
		,FROMPARTNERID
		,TOPARTNERID
		,ROWTS
		,ENVELOPEPROFILEID
		,CONNECTIONPROFILEID
		,CREATEDATE
	FROM 
		CF_ENVELOPEQ
	ORDER BY ROOTBATCHID, ORDERING, FROMPARTNERID, TOPARTNERID, ENVELOPEPROFILEID, CONNECTIONPROFILEID
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ENVELOPEQS;
/


-- CF_ENVELOPEQS_BATCH.sql

CREATE OR REPLACE PROCEDURE CF_ENVELOPEQS_BATCH
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	)
AS
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQS_BATCH
	Create Date:     2004-09-07 17:49:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2004-10-27 S.Priani	4.3	BGTRK00054219
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQS_BATCH';
v_ExceptionFlg    integer Default 0;
v_CompleteBatchStatus Integer Default 1;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	-- select batchid from cf_batch with status COMPLETE and is a rootBatch

	SELECT 
		BATCHID AS ROOTBATCHID
	FROM 
		CF_BATCH b
	WHERE 
		b.BATCHSTATUS = v_CompleteBatchStatus
	AND EXISTS (SELECT 'x' FROM CF_ENVELOPEQ q WHERE q.ROOTBATCHID = b.BATCHID)
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ENVELOPEQS_BATCH;
/


-- CF_ENVELOPEQS_PARTNERPAIR.sql

CREATE OR REPLACE PROCEDURE CF_ENVELOPEQS_PARTNERPAIR
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	)
AS
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQS_PARTNERPAIR
	Create Date:     2004-09-07 17:49:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-07 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQS_PARTNERPAIR';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT 
			 distinct FROMPARTNERID, TOPARTNERID
		FROM 
			CF_ENVELOPEQ
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ENVELOPEQS_PARTNERPAIR;
/


-- CF_ENVELOPEQS_TX.sql

CREATE OR REPLACE PROCEDURE CF_ENVELOPEQS_TX
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FROMPARTNERID IN INTEGER
	, p_TOPARTNERID IN INTEGER
	, p_BATCHID IN VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQS_TX
	Create Date:     2004-09-07 17:49:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2004-10-27 S.Priani	4.3	BGTRK00054219
	2004-12-07 S.Priani	4.3	BGTRK00060863
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQS_TX';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	IF p_BATCHID IS NOT NULL THEN
		OPEN RC1 FOR 
		SELECT	RTRIM(VUID) VUID
			,ROOTBATCHID
			,RTRIM(ACTIVITYID) ACTIVITYID
			,FROMPARTNERID
			,TOPARTNERID
			,ROWTS
			,ENVELOPEPROFILEID
			,CONNECTIONPROFILEID
			,CREATEDATE
		FROM	CF_ENVELOPEQ
		WHERE	FROMPARTNERID = p_FROMPARTNERID
		AND     TOPARTNERID = p_TOPARTNERID
		AND     ROOTBATCHID = p_BATCHID
		ORDER BY	ROOTBATCHID, ORDERING
		;
	ELSE
		OPEN RC1 FOR 
		SELECT	RTRIM(VUID) VUID
			,ROOTBATCHID
			,RTRIM(ACTIVITYID) ACTIVITYID
			,FROMPARTNERID
			,TOPARTNERID
			,ROWTS
			,ENVELOPEPROFILEID
			,CONNECTIONPROFILEID
			,CREATEDATE
			FROM	CF_ENVELOPEQ
		WHERE	FROMPARTNERID = p_FROMPARTNERID
		AND     TOPARTNERID = p_TOPARTNERID
		AND     ROOTBATCHID IS NULL
		ORDER BY	ROOTBATCHID, ORDERING
		;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ENVELOPEQS_TX;
/


-- CF_ENVELOPEQS_TX_NOBATCH.sql

CREATE OR REPLACE PROCEDURE CF_ENVELOPEQS_TX_NOBATCH
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FROMPARTNERID IN INTEGER
	, p_TOPARTNERID IN INTEGER
	)
AS
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQS_TX_NOBATCH
	Create Date:     2004-09-07 17:49:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2004-10-27 S.Priani	4.3	BGTRK00054219
	2004-12-07 S.Priani	4.3	BGTRK00060863
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQS_TX_NOBATCH';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT 
				RTRIM(VUID) VUID
				,ROOTBATCHID
				,RTRIM(ACTIVITYID) ACTIVITYID
				,FROMPARTNERID
				,TOPARTNERID
				,ROWTS
				,ENVELOPEPROFILEID
				,CONNECTIONPROFILEID
				,CREATEDATE
			FROM 
				CF_ENVELOPEQ
			WHERE 
				FROMPARTNERID = p_FROMPARTNERID
			AND     TOPARTNERID = p_TOPARTNERID
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ENVELOPEQS_TX_NOBATCH;
/


-- CF_ENVELOPEQU.sql

Create or Replace Procedure CF_ENVELOPEQU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_VUID in VARCHAR2
	,p_BATCHID in VARCHAR2
	,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_ROWTS in NUMBER
	,p_ENVELOPEPROFILEID in NUMBER
	,p_CONNECTIONPROFILEID in NUMBER
) as
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQU
	Create Date:     2004-09-07 17:49:01
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2004-10-27 S.Priani	4.3	BGTRK00054219
	2004-12-07 S.Priani	4.3	BGTRK00060863
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ENVELOPEPROFILEID=>'||p_ENVELOPEPROFILEID||CHR(10)
		||', p_CONNECTIONPROFILEID=>'||p_CONNECTIONPROFILEID||CHR(10)
		;
v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_RowCnt := 0;
	Update CF_ENVELOPEQ
	SET
		ROOTBATCHID=p_BATCHID
		,      FROMPARTNERID=p_FROMPARTNERID
		,      TOPARTNERID=p_TOPARTNERID
		,      ROWTS= p_RowTS + 1
		,      ENVELOPEPROFILEID=p_ENVELOPEPROFILEID
		,      CONNECTIONPROFILEID=p_CONNECTIONPROFILEID
		,      UPDATEDATE= v_BeginTS
	Where 
		VUID = p_VUID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_ENVELOPEQU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_ENVELOPEQU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_ENVELOPEQU:'||SQLERRM);
End CF_ENVELOPEQU;
/


-- CF_ENVELOPEQ_TEMPI.sql

CREATE OR REPLACE PROCEDURE CF_ENVELOPEQ_TEMPI
	(p_ReturnCd		out	pls_integer
	,p_DebugFlg		in	pls_integer
	,p_VUID			in	VARCHAR2
	,p_BATCHID		in	VARCHAR2
	,p_ACTIVITYID		in	VARCHAR2
	,p_FROMPARTNERID	in	NUMBER
	,p_TOPARTNERID		in	NUMBER
	,p_ORDERING		in	VARCHAR2
	,p_ENVELOPEPROFILEID	in	NUMBER
	,p_CONNECTIONPROFILEID	in	NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQ_TEMPI
	Create Date:     2004-09-07 17:49:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2004-10-27 S.Priani	4.3	BGTRK00054219
	2004-12-07 S.Priani	4.3	BGTRK00060863
	2004-12-07 S.Priani	4.3	BGTRK00047894
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQ_TEMPI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_BATCHID=>'||p_BATCHID||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_ENVELOPEPROFILEID=>'||p_ENVELOPEPROFILEID||CHR(10)
		||', p_CONNECTIONPROFILEID=>'||p_CONNECTIONPROFILEID||CHR(10)
		;

CURSOR	rootb_cur (pi_batchId IN VARCHAR2) IS
	SELECT	parentBatchId, OrderPrefix
	FROM	cf_batch
	WHERE	batchId = pi_batchid
	;

v_firstTime		BOOLEAN				DEFAULT TRUE;
v_OrderPrefix		cf_Batch.OrderPrefix%TYPE;
v_Ordering		cf_EnvelopeQ.Ordering%TYPE;
v_ParentBatchId		cf_Batch.ParentBatchId%TYPE;
v_BatchId		cf_Batch.BatchId%TYPE;
v_separator		VARCHAR2(1)			DEFAULT '.';
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

	v_ParentBatchId := p_BatchId;

	WHILE (v_ParentBatchID IS NOT NULL OR LTRIM(RTRIM(v_ParentBatchID)) != '' )
	LOOP
		v_batchId := v_ParentBatchId;

		IF rootb_cur%ISOPEN THEN
			CLOSE rootb_cur;
		END IF;
		OPEN rootb_cur ( v_batchId );
		FETCH rootb_cur INTO v_ParentBatchId, v_OrderPrefix;
		IF rootb_cur%NOTFOUND THEN
			RAISE No_Data_Found;
		END IF;
		CLOSE rootb_cur;

		IF v_firstTime = TRUE THEN
			v_firstTime := FALSE;
			v_ordering := COALESCE(v_OrderPrefix,'');
		END IF;

	END LOOP;

	Insert into CF_ENVELOPEQ(
		VUID
		,ROOTBATCHID
		,ACTIVITYID
		,FROMPARTNERID
		,TOPARTNERID
		,ENVELOPEPROFILEID
		,CONNECTIONPROFILEID
		,ORDERING
		,CREATEDATE
		,UPDATEDATE
	) Values (
		p_VUID
		,v_BatchId
		,p_ACTIVITYID
		,p_FROMPARTNERID
		,p_TOPARTNERID
		,p_ENVELOPEPROFILEID
		,p_CONNECTIONPROFILEID
		,LTRIM( v_ordering || v_separator || p_ORDERING , v_separator)
		,v_BeginTS
		,v_BeginTS
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 CF_ENVELOPEQ_TEMPI' || SQLCODE|| ' '||SQLERRM);
END CF_ENVELOPEQ_TEMPI;
/


-- CF_ENVELOPEQ_TEMP_INIT.sql

CREATE OR REPLACE PROCEDURE CF_ENVELOPEQ_TEMP_INIT
	(p_ReturnCd		out	pls_integer
	,p_DebugFlg		in	pls_integer
	)
AS
/*
	================================================================================
	Proc Name:       CF_ENVELOPEQ_TEMP_INIT
	Create Date:     2004-09-07 17:49:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-09-07 SYSTEM	4.3	Created
	2005-05-09 S.Priani	4.3	BGTRK00048595
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ENVELOPEQ_TEMP_INIT';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

	/* This procedure is a NOP to be in sync with DB2 code
	To prevent lock escalation in DB2
	We are using temporary table to hold the data and then transfer it to CF_ENVELOPEQ
	*/
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 CF_ENVELOPEQ_TEMP_INIT' || SQLCODE|| ' '||SQLERRM);
END CF_ENVELOPEQ_TEMP_INIT;
/


-- CF_FIXEDWORKFLOWHANDLER_XREFD.sql

CREATE OR REPLACE Procedure CF_FIXEDWORKFLOWHANDLER_XREFD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_FIXEDWOKFLOWSTEPID in VARCHAR
	,p_HANDLERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_FIXEDWOKFLOWHANDLER_XREFD
	Create Date:     2004-03-14 15:35:08
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWOKFLOWHANDLER_XREFD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_where := ' WHERE 1=1 ' ;
    v_where := v_where || FN_BUILD_WHERE('FIXEDWOKFLOWSTEPID', 'VARCHAR2', p_FIXEDWOKFLOWSTEPID) ;
	 v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	
    v_sqlstmt := 'Delete From CF_FIXEDWORKFLOWHANDLER_XREF ' || v_where ;
	
	execute immediate v_sqlstmt ;
		
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FIXEDWORKFLOWHANDLER_XREFD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_FIXEDWORKFLOWHANDLER_XREFD;
/


-- CF_FIXEDWORKFLOWHANDLER_XREFI.sql

CREATE OR REPLACE Procedure CF_FIXEDWORKFLOWHANDLER_XREFI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT   
	,p_FIXEDWOKFLOWSTEPID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_ORDERING in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_FIXEDWORKFLOWHANDLER_XREFI
	Create Date:     2004-03-14 15:35:45
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWORKFLOWHANDLER_XREFI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_FIXEDWORKFLOWHANDLER_XREF(
		FIXEDWOKFLOWSTEPID
		,HANDLERID
		,ORDERING
	) Values (
		p_FIXEDWOKFLOWSTEPID
		,p_HANDLERID
		,p_ORDERING
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FIXEDWORKFLOWHANDLER_XREFI' || SQLCODE|| ' '||SQLERRM);
END CF_FIXEDWORKFLOWHANDLER_XREFI;
/


-- CF_FIXEDWORKFLOWHANDLER_XREFS.sql

CREATE OR REPLACE PROCEDURE CF_FIXEDWORKFLOWHANDLER_XREFS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FIXEDWOKFLOWSTEPID IN VARCHAR2
	, p_HANDLERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_FIXEDWORKFLOWHANDLER_XREFS
	Create Date:     2004-03-14 15:36:13
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWORKFLOWHANDLER_XREFS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;           
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('FIXEDWOKFLOWSTEPID', 'VARCHAR2', p_FIXEDWOKFLOWSTEPID) ||
       FN_Build_Where('rx.HANDLERID', 'NUMBER', p_HANDLERID);
	   
	v_sqlstmt := 
       'SELECT '
        ||'FIXEDWOKFLOWSTEPID'
		||',rx.HANDLERID'
		||',ORDERING'
		||',PROVIDERFLG'
		||',RESOURCEKEY'
		||',DESCRIPTION'
		||',CLASSNAME'
		||',rx.ROWTS'
		||',rx.CREATEDATE '
		||'FROM '
		||'CF_FIXEDWORKFLOWHANDLER_XREF rx INNER JOIN PR_HANDLER h ' 
		||'ON rx.HANDLERID = h.HANDLERID '
		||v_where 
      ||' ORDER BY FIXEDWOKFLOWSTEPID, ORDERING '
		;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_FIXEDWORKFLOWHANDLER_XREFS;
/


-- CF_FIXEDWORKFLOWHNDLRATTR_LNKD.sql

CREATE OR REPLACE Procedure CF_FIXEDWORKFLOWHNDLRATTR_LNKD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_FIXEDWOKFLOWSTEPID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_HANDLERATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_FIXEDWORKFLOWHANDLERATTR_LNKD
	Create Date:     2004-03-14 17:20:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWORKFLOWHNDLRATTR_LNKD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   
   v_where := ' WHERE 1=1 ' ;
   v_where := v_where || FN_BUILD_WHERE('FIXEDWOKFLOWSTEPID', 'VARCHAR2', p_FIXEDWOKFLOWSTEPID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;
	
   v_sqlstmt := 'Delete From CF_FIXEDWORKFLOWHNDLRATTR_LNK ' || v_where ;
	
	execute immediate v_sqlstmt ;

	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FIXEDWORKFLOWHNDLRATTR_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_FIXEDWORKFLOWHNDLRATTR_LNKD;
/


-- CF_FIXEDWORKFLOWHNDLRATTR_LNKI.sql

CREATE OR REPLACE Procedure CF_FIXEDWORKFLOWHNDLRATTR_LNKI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT 
	,p_FIXEDWOKFLOWSTEPID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_HANDLERATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_FIXEDWORKFLOWHNDLRATTR_LNKI
	Create Date:     2004-03-14 17:22:19
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWORKFLOWHNDLRATTR_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_FIXEDWORKFLOWHNDLRATTR_LNK(
		FIXEDWOKFLOWSTEPID
		,HANDLERID
		,HANDLERATTRID
		,ATTRVALUE
	) Values (
		p_FIXEDWOKFLOWSTEPID
		,p_HANDLERID
		,p_HANDLERATTRID
		,p_ATTRVALUE
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FIXEDWORKFLOWHNDLRATTR_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_FIXEDWORKFLOWHNDLRATTR_LNKI;
/


-- CF_FIXEDWORKFLOWHNDLRATTR_LNKS.sql

CREATE OR REPLACE PROCEDURE CF_FIXEDWORKFLOWHNDLRATTR_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FIXEDWOKFLOWSTEPID IN VARCHAR2
	, p_HANDLERID IN NUMBER
	, p_HANDLERATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_FIXEDWORKFLOWHNDLRATTR_LNKS
	Create Date:     2004-03-14 17:21:53
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created
   2004-05-24 PAD           4.2.2  Added v_where_inner logic
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
   
   **NOTE - if no handler or stepid is given ALL handler default
            handler attributes will be returned
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWORKFLOWHNDLRATTR_LNKS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;        
v_where_inner     varchar2(4000) ;        
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' WHERE 1=1 ' ;
    
    if p_HANDLERATTRID is not null then
       v_where := v_where || FN_BUILD_WHERE('FIXEDWOKFLOWSTEPID', 'VARCHAR2', p_FIXEDWOKFLOWSTEPID) ;
       v_where := v_where || FN_BUILD_WHERE('RA.HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where := v_where || FN_BUILD_WHERE('RA.HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_sqlstmt :=
       'SELECT '
	   ||'FIXEDWOKFLOWSTEPID'
	   ||',RA.HANDLERID'
	   ||',RA.HANDLERATTRID'
       ||',RTRIM(ATTRNAME) ATTRNAME'
       ||',RTRIM(ATTRVALUE) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',RA.ROWTS'
       ||',RA.CREATEDATE '
       ||'FROM '
       ||'CF_FIXEDWORKFLOWHNDLRATTR_LNK RA INNER JOIN PR_HANDLERATTR HA '
       ||'ON RA.HANDLERATTRID = HA.HANDLERATTRID '
       || v_where
	   || ' ORDER BY FIXEDWOKFLOWSTEPID, RA.HANDLERID' 
	   ;
    ELSE
       v_where_inner := ' WHERE 1=1 ' ;
       v_where_inner := v_where_inner || FN_BUILD_WHERE('FIXEDWOKFLOWSTEPID', 'VARCHAR2', p_FIXEDWOKFLOWSTEPID) ;
       v_where_inner := v_where_inner || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where_inner := v_where_inner || FN_BUILD_WHERE('HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;
       v_where := v_where || FN_BUILD_WHERE('HA.HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where := v_where || FN_BUILD_WHERE('HA.HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_sqlstmt :=
       'SELECT '
       ||'FIXEDWOKFLOWSTEPID'
	   ||',NVL(RA1.HANDLERID, ha.HANDLERID) HANDLERID '
       ||',NVL(ra1.HANDLERATTRID, ha.HANDLERATTRID) HANDLERATTRID '
       ||',RTRIM(attrname) ATTRNAME'
       ||',RTRIM(NVL(attrvalue, defaultattrvalue)) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',ra1.ROWTS'
       ||',ra1.CREATEDATE '
       ||'from '
       ||'(select FIXEDWOKFLOWSTEPID, handlerid, handlerattrid, attrvalue, rowts, createdate from '
       ||' cf_FIXEDWORKFLOWHNDLRattr_lnk RA '||v_where_inner||') RA1 '
       ||'right outer join pr_handlerattr HA '
       ||'on RA1.handlerattrid = HA.handlerattrid '
       ||v_where
	   ||' ORDER BY FIXEDWOKFLOWSTEPID, HANDLERID' 
       ;
    end if ;     
	
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_FIXEDWORKFLOWHNDLRATTR_LNKS;
/


-- CF_FIXEDWORKFLOWHNDLRATTR_LNKU.sql

CREATE OR REPLACE Procedure CF_FIXEDWORKFLOWHNDLRATTR_LNKU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_FIXEDWOKFLOWSTEPID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_HANDLERATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_FIXEDWORKFLOWHNDLRATTR_LNKU
	Create Date:     2004-03-14 17:21:33
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWORKFLOWHNDLRATTR_LNKU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_FIXEDWORKFLOWHNDLRATTR_LNK
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      ATTRVALUE=p_ATTRVALUE
	Where 
		FIXEDWOKFLOWSTEPID = p_FIXEDWOKFLOWSTEPID
		AND HANDLERID = p_HANDLERID
		AND HANDLERATTRID = p_HANDLERATTRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_FIXEDWORKFLOWHNDLRATTR_LNKU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_FIXEDWORKFLOWHNDLRATTR_LNKU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_FIXEDWORKFLOWHNDLRATTR_LNKU:'||SQLERRM);
End CF_FIXEDWORKFLOWHNDLRATTR_LNKU;
/


-- CF_FIXEDWORKFLOWSTEP_XREFD.sql

CREATE OR REPLACE Procedure CF_FIXEDWORKFLOWSTEP_XREFD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_FIXEDWORKFLOWID in NUMBER
	,p_FIXEDWOKFLOWSTEPID in VARCHAR
	)
AS

/*
	================================================================================
	Proc Name:       CF_FIXEDWORKFLOWSTEP_XREFD
	Create Date:     2004-03-14 15:35:08
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWORKFLOWSTEP_XREFD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWORKFLOWID=>'||p_FIXEDWORKFLOWID||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   
   v_where := ' WHERE 1=1 ' ;
   v_where := v_where || FN_BUILD_WHERE('FIXEDWORKFLOWID', 'NUMBER', p_FIXEDWORKFLOWID) ;
	v_where := v_where || FN_BUILD_WHERE('FIXEDWOKFLOWSTEPID', 'VARCHAR2', p_FIXEDWOKFLOWSTEPID) ;
	
   v_sqlstmt := 'Delete From CF_FIXEDWORKFLOWSTEP_XREF ' || v_where ;
	
	execute immediate v_sqlstmt ;
		
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FIXEDWORKFLOWSTEP_XREFD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_FIXEDWORKFLOWSTEP_XREFD;
/


-- CF_FIXEDWORKFLOWSTEP_XREFI.sql

CREATE OR REPLACE Procedure CF_FIXEDWORKFLOWSTEP_XREFI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT   
	,p_FIXEDWORKFLOWID in NUMBER
	,p_FIXEDWOKFLOWSTEPID in VARCHAR
	,p_ORDERING in NUMBER

	)
AS

/*
	================================================================================
	Proc Name:       CF_FIXEDWORKFLOWSTEP_XREFI
	Create Date:     2004-03-14 15:35:45
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWORKFLOWSTEP_XREFI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWORKFLOWID=>'||p_FIXEDWORKFLOWID||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_FIXEDWORKFLOWSTEP_XREF(
		FIXEDWORKFLOWID
		,FIXEDWOKFLOWSTEPID
		,ORDERING
	) Values (
		p_FIXEDWORKFLOWID
		,p_FIXEDWOKFLOWSTEPID
		,p_ORDERING
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FIXEDWORKFLOWSTEP_XREFI' || SQLCODE|| ' '||SQLERRM);
END CF_FIXEDWORKFLOWSTEP_XREFI;
/


-- CF_FIXEDWORKFLOWSTEP_XREFS.sql

CREATE OR REPLACE PROCEDURE CF_FIXEDWORKFLOWSTEP_XREFS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FIXEDWORKFLOWID IN NUMBER
	, p_FIXEDWOKFLOWSTEPID IN VARCHAR
	)
AS

/*
	================================================================================
	Proc Name:       CF_FIXEDWORKFLOWSTEP_XREFS
	Create Date:     2004-03-14 15:36:13
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FIXEDWORKFLOWSTEP_XREFS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;           
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWORKFLOWID=>'||p_FIXEDWORKFLOWID||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('FIXEDWORKFLOWID', 'NUMBER', p_FIXEDWORKFLOWID) ||
       FN_Build_Where('rx.FIXEDWOKFLOWSTEPID', 'VARCHAR2', p_FIXEDWOKFLOWSTEPID);
	   
	v_sqlstmt := 
       'SELECT '
        ||'FIXEDWORKFLOWID'
		||',rx.FIXEDWOKFLOWSTEPID'
		||',ORDERING'
		||',CONFIGURABLE'
		||',PROVIDERFLAG'
		||',CLASSNAME'
		||',RESOURCEKEY'
		||',DESCRIPTION'
		||',rx.ROWTS'
		||',rx.CREATEDATE '
		||'FROM '
		||'CF_FIXEDWORKFLOWSTEP_XREF rx INNER JOIN PR_FIXEDWORKFLOWSTEP h ' 
		||'ON rx.FIXEDWOKFLOWSTEPID = h.FIXEDWOKFLOWSTEPID '
		||v_where
		||' ORDER BY rx.ORDERING'
		;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_FIXEDWORKFLOWSTEP_XREFS;
/


-- CF_FTPConfigD.SQL

CREATE OR REPLACE Procedure CF_FTPCONFIGD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_FTPCONFIGID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_FTPCONFIGD
	Create Date:     2003-09-26 08:23:54
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTPCONFIGD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FTPCONFIGID=>'||p_FTPCONFIGID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_FTPCONFIG
	Where
		FTPCONFIGID = p_FTPCONFIGID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FTPCONFIGD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_FTPCONFIGD;
/


-- CF_FTPCONFIGI.SQL

Create or Replace Procedure CF_FTPCONFIGI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_FTPUSERNAME in VARCHAR2
	,p_PARTNERBUSID in VARCHAR2
	,p_ENABLEDFLG in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_FTPCONFIGI
	Create Date:     2003-11-21 14:22:18
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-11-21 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTPCONFIGI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_FTPUSERNAME=>'||p_FTPUSERNAME||CHR(10)
		||', p_PARTNERBUSID=>'||p_PARTNERBUSID||CHR(10)
		||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_FTPCONFIG(
		FTPCONFIGID
		,PARTNERID
		,FTPUSERNAME
		,PARTNERBUSID
		,ENABLEDFLG
	) Values (
		CF_FTPCONFIG_SEQ.nextval
		,p_PARTNERID
		,p_FTPUSERNAME
		,p_PARTNERBUSID
		,p_ENABLEDFLG
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_FTPCONFIG_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FTPCONFIGI' || SQLCODE|| ' '||SQLERRM);
END CF_FTPCONFIGI;
/


-- CF_FTPCONFIGS_AH1.SQL

CREATE OR REPLACE PROCEDURE         CF_FTPCONFIGS_AH1
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_PartnerId IN NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_FTPCONFIGS_AH1
    Create Date:     2003-05-03 12:12:52
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-03 M.Mount       3.5    Search by Partnerid

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_FTPCONFIGS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_PartnerId=>'||p_PartnerId||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PartnerId IS NOT NULL
 THEN
    OPEN RC1 FOR
    SELECT
        FTPCONFIGID
        ,PARTNERID
        ,RTRIM(FTPUSERNAME) FTPUSERNAME
        ,RTRIM(PARTNERBUSID) PARTNERBUSID
        ,ENABLEDFLG
        ,ROWTS
        ,CREATEDATE
    FROM
        CF_FTPCONFIG
    Where
        Partnerid = p_PartnerId
    ;
    ELSE
    OPEN RC1 FOR
    SELECT
        FTPCONFIGID
        ,PARTNERID
        ,RTRIM(FTPUSERNAME) FTPUSERNAME
        ,RTRIM(PARTNERBUSID) PARTNERBUSID
        ,ENABLEDFLG
        ,ROWTS
        ,CREATEDATE
    FROM
        CF_FTPCONFIG
    ;
    END IF;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_FTPCONFIGS_AH1;
/


-- CF_FTPCONFIGU.SQL

CREATE OR REPLACE Procedure CF_FTPCONFIGU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_FTPCONFIGID in NUMBER
	,p_PARTNERID in NUMBER
	,p_FTPUSERNAME in VARCHAR2
	,p_PARTNERBUSID in VARCHAR2
	,p_ENABLEDFLG in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_FTPCONFIGU
	Create Date:     2003-09-26 08:20:49
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTPCONFIGU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FTPCONFIGID=>'||p_FTPCONFIGID||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_FTPUSERNAME=>'||p_FTPUSERNAME||CHR(10)
		||', p_PARTNERBUSID=>'||p_PARTNERBUSID||CHR(10)
		||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_FTPCONFIG
	SET
		PARTNERID=p_PARTNERID
		,      FTPUSERNAME=p_FTPUSERNAME
		,      PARTNERBUSID=p_PARTNERBUSID
		,      ENABLEDFLG=p_ENABLEDFLG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		FTPCONFIGID = p_FTPCONFIGID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_FTPCONFIGU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_FTPCONFIGU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_FTPCONFIGU:'||SQLERRM);
End CF_FTPCONFIGU;
/


-- CF_FTP_IP_RESTRICTORD.sql

Create or Replace Procedure CF_FTP_IP_RESTRICTORD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer   
	,p_IPPATTERN in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_FTP_IP_RESTRICTORD
	Create Date:     2007-08-29 17:20:23
	Author:          AMIT KUMAR VIJAY 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-08-29 AMIT V.       6.1.1  Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTP_IP_RESTRICTORD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_IPPATTERN=>'||p_IPPATTERN||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
  IF p_IPPATTERN IS NOT NULL
  THEN
	Delete From CF_FTP_IP_RESTRICTOR
	Where 
		IPPATTERN = p_IPPATTERN
	;
	v_RowCnt := SQL%ROWCOUNT;
	
  ELSE
	Delete From CF_FTP_IP_RESTRICTOR;
  	v_RowCnt := SQL%ROWCOUNT;
  END IF;
  
  	
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FTP_IP_RESTRICTORD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_FTP_IP_RESTRICTORD;
/


-- CF_FTP_IP_RESTRICTORI.sql

Create or Replace Procedure CF_FTP_IP_RESTRICTORI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_IPPATTERN in VARCHAR2
	,p_IPPERMISSION in NUMBER
	,p_IPORDER in NUMBER
	
	)
AS

/*
	================================================================================
	Proc Name:       CF_FTP_IP_RESTRICTORI
	Create Date:     2007-08-29 17:12:55
	Author:          AMIT KUMAR VIJAY

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2007-07-19 AMIT V.	6.1.1	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTP_IP_RESTRICTORI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_IPPATTERN=>'||p_IPPATTERN||CHR(10)
		||', p_IPPERMISSION=>'||p_IPPERMISSION||CHR(10)
		||', p_IPORDER=>'||p_IPORDER||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_FTP_IP_RESTRICTOR(
		FTPIPRESTRICTID
		,IPPATTERN
		,IPPERMISSION
		,IPORDER
		,UPDATEDATE
		,CREATEDATE
	) Values (
		CF_FTP_IP_RESTRICTOR_SEQ.nextval
		,p_IPPATTERN
		,p_IPPERMISSION
		,p_IPORDER
		,v_BeginTS
		,v_BeginTS
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_FTP_IP_RESTRICTOR_SEQ.currval ID,1 ROWTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 CF_FTP_IP_RESTRICTORI' || SQLCODE|| ' '||SQLERRM);
END CF_FTP_IP_RESTRICTORI;
/


-- CF_FTP_IP_RESTRICTORS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_FTP_IP_RESTRICTORS_AH1
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_IPPATTERN in VARCHAR2
    )
AS

/*
    ================================================================================
    Proc Name:       CF_FTP_IP_RESTRICTORS_AH1
    Create Date:     2007-08-29 15:46:49
    Author:          AMIT KUMAR VIJAY

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-29 AMIT V.       6.1.1  Created(Search by IPPATTERN)

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_FTP_IP_RESTRICTORS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_IPPATTERN=>'||p_IPPATTERN||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_IPPATTERN IS NOT NULL
 THEN
    OPEN RC1 FOR
    SELECT
    	IPORDER
        ,RTRIM(IPPATTERN)	IPPATTERN
        ,IPPERMISSION
        FROM
        CF_FTP_IP_RESTRICTOR
    Where
        IPPATTERN = p_IPPATTERN
        ORDER BY IPORDER
    ;
    ELSE
    OPEN RC1 FOR
    SELECT
        IPORDER
        ,RTRIM(IPPATTERN)	IPPATTERN
        ,IPPERMISSION
    FROM
        CF_FTP_IP_RESTRICTOR
        ORDER BY IPORDER
    ;
    END IF;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_FTP_IP_RESTRICTORS_AH1;
/


-- CF_FTP_IP_RESTRICTORU.sql

Create or Replace Procedure CF_FTP_IP_RESTRICTORU
	(
	 p_ReturnCd out integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_IPPATTERN in VARCHAR2
	,p_IPPERMISSION in NUMBER
	,p_IPORDER in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_FTP_IP_RESTRICTORU
	Create Date:     2007-08-29 16:44:24
	Author:          AMIT KUMAR VIJAY 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-08-29 AMIT V.       6.1.1  Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_ROWCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTP_IP_RESTRICTORU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_IPPATTERN=>'||p_IPPATTERN||CHR(10)
		||', p_IPPERMISSION=>'||p_IPPERMISSION||CHR(10)
		||', p_IPORDER=>'||p_IPORDER||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_ROWTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_ROWCnt := 0;
	Update CF_FTP_IP_RESTRICTOR
	SET
		IPPERMISSION=p_IPPERMISSION
		,      IPORDER=p_IPORDER
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		IPPATTERN = p_IPPATTERN
		And ROWTS = p_ROWTS returning ROWTS into v_ROWTS;
	v_ROWCnt := SQL%rowcount;
	if (v_ROWCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_ROWTS ROWTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_FTP_IP_RESTRICTORU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_FTP_IP_RESTRICTORU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_FTP_IP_RESTRICTORU:'||SQLERRM);
End CF_FTP_IP_RESTRICTORU;
/



-- CF_FTP_SERVERD_AH1.sql

Create or Replace Procedure CF_FTP_SERVERD_AH1

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	)
AS

/*
	================================================================================
	Proc Name:       CF_FTP_SERVERD_AH1
	Create Date:     2007-07-26 11:07:21
	Author:          AMIT KUMAR VIJAY 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-07-26 AMIT V.       6.1.1  Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTP_SERVERD_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_FTP_SERVER
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FTP_SERVERD_AH1: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_FTP_SERVERD_AH1;
/


-- CF_FTP_SERVERI.sql

Create or Replace Procedure CF_FTP_SERVERI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_HOSTNAME in VARCHAR2
	,p_PORTNUMBER in NUMBER
	,p_INSTALLLOCATION in VARCHAR2
	,p_VERSIONINFORMATION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_FTP_SERVERI
	Create Date:     2007-07-19 10:26:49
	Author:          AMIT KUMAR VIJAY

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2007-07-19 AMIT V.	6.1.1	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTP_SERVERI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_HOSTNAME=>'||p_HOSTNAME||CHR(10)
		||', p_PORTNUMBER=>'||p_PORTNUMBER||CHR(10)
		||', p_INSTALLLOCATION=>'||p_INSTALLLOCATION||CHR(10)
		||', p_VERSIONINFORMATION=>'||p_VERSIONINFORMATION||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_FTP_SERVER(
		FTPHOSTID
		,HOSTNAME
		,PORTNUMBER
		,INSTALLLOCATION
		,VERSIONINFORMATION
		,UPDATEDATE
		,CREATEDATE
	) Values (
		CF_FTP_SERVER_SEQ.nextval
		,p_HOSTNAME
		,p_PORTNUMBER
		,p_INSTALLLOCATION
		,p_VERSIONINFORMATION
		,v_BeginTS
		,v_BeginTS
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_FTP_SERVER_SEQ.currval ID,1 ROWTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 CF_FTP_SERVERI' || SQLCODE|| ' '||SQLERRM);
END CF_FTP_SERVERI;
/


-- CF_FTP_SERVERS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_FTP_SERVERS_AH1
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_HOSTNAME in VARCHAR2
    )
AS

/*
    ================================================================================
    Proc Name:       CF_FTP_SERVERS_AH1
    Create Date:     2007-07-23 16:00:00
    Author:          AMIT KUMAR VIJAY

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-07-23 AMIT V.       6.1.1  Created(Search by Hostname)

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_FTP_SERVERS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_HOSTNAME=>'||p_HOSTNAME||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_HOSTNAME IS NOT NULL
 THEN
    OPEN RC1 FOR
    SELECT
        FTPHOSTID
        ,RTRIM(HOSTNAME)	HOSTNAME
        ,PORTNUMBER
        ,RTRIM(INSTALLLOCATION) INSTALLLOCATION
        ,RTRIM(VERSIONINFORMATION) VERSIONINFORMATION
        ,UPDATEDATE
        ,CREATEDATE
        ,ROWTS
    FROM
        CF_FTP_SERVER
    Where
        HOSTNAME = p_HOSTNAME
    ;
    ELSE
    OPEN RC1 FOR
    SELECT
        FTPHOSTID
	,RTRIM(HOSTNAME)	HOSTNAME
	,PORTNUMBER
	,RTRIM(INSTALLLOCATION) INSTALLLOCATION
	,RTRIM(VERSIONINFORMATION) VERSIONINFORMATION
	,UPDATEDATE
	,CREATEDATE
        ,ROWTS
    FROM
        CF_FTP_SERVER
    ;
    END IF;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_FTP_SERVERS_AH1;
/


-- CF_FTP_USERD.sql

Create or Replace Procedure CF_FTP_USERD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer   
	,p_USERID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_FTP_USERD
	Create Date:     2007-07-19 10:26:23
	Author:          AMIT KUMAR VIJAY 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-07-19 AMIT V.       6.1.1  Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTP_USERD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_USERID=>'||p_USERID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_FTP_USER
	Where 
		USERNAME = p_USERID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_FTP_USERD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_FTP_USERD;
/


-- CF_FTP_USERI.sql

Create or Replace Procedure CF_FTP_USERI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_USERNAME in VARCHAR2
	,p_HOMEDIRECTORY in VARCHAR2
	,p_ENABLEFLAG in NUMBER
	,p_WRITEPERMISSION in NUMBER
	,p_IDLETIME in NUMBER
	,p_UPLOADRATE in NUMBER
	,p_DOWNLOADRATE in NUMBER
	,p_MAXLOGINNUMBER in NUMBER
	,p_MAXLOGINPERIP in NUMBER
	,p_MODIFYDIR in NUMBER
	
	)
AS

/*
	================================================================================
	Proc Name:       CF_FTP_USERI
	Create Date:     2007-07-19 10:26:49
	Author:          AMIT KUMAR VIJAY

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2007-07-19 AMIT V.	6.1.1	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTP_USERI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_USERNAME=>'||p_USERNAME||CHR(10)
		||', p_HOMEDIRECTORY=>'||p_HOMEDIRECTORY||CHR(10)
		||', p_ENABLEFLAG=>'||p_ENABLEFLAG||CHR(10)
		||', p_WRITEPERMISSION=>'||p_WRITEPERMISSION||CHR(10)
		||', p_IDLETIME=>'||p_IDLETIME||CHR(10)
		||', p_UPLOADRATE=>'||p_UPLOADRATE||CHR(10)
		||', p_DOWNLOADRATE=>'||p_DOWNLOADRATE||CHR(10)
		||', p_MAXLOGINNUMBER=>'||p_MAXLOGINNUMBER||CHR(10)
		||', p_MAXLOGINPERIP=>'||p_MAXLOGINPERIP||CHR(10)
		||', p_MODIFYDIR=>'||p_MODIFYDIR||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_FTP_USER(
		FTPUSERID
		,USERNAME
		,HOMEDIRECTORY
		,ENABLEFLAG
		,WRITEPERMISSION
		,IDLETIME
		,UPLOADRATE
		,DOWNLOADRATE
		,MAXLOGINNUMBER
		,MAXLOGINPERIP
		,MODIFYDIR
		,UPDATEDATE
		,CREATEDATE
	) Select 
		CF_FTP_USER_SEQ.nextval
		,p_USERNAME
		,p_HOMEDIRECTORY
		,p_ENABLEFLAG
		,p_WRITEPERMISSION
		,p_IDLETIME
		,p_UPLOADRATE
		,p_DOWNLOADRATE
		,p_MAXLOGINNUMBER
		,p_MAXLOGINPERIP
		,p_MODIFYDIR
		,v_BeginTS
		,v_BeginTS 
	 From Dual 
	 Where Exists (Select * from PR_USER P where P.USERNAME=p_USERNAME);
	
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_FTP_USER_SEQ.currval ID,1 ROWTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 CF_FTP_USERI' || SQLCODE|| ' '||SQLERRM);
END CF_FTP_USERI;
/


-- CF_FTP_USERS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_FTP_USERS_AH1
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_USERNAME in VARCHAR2
    )
AS

/*
    ================================================================================
    Proc Name:       CF_FTP_USERS_AH1
    Create Date:     2007-07-23 16:00:00
    Author:          AMIT KUMAR VIJAY

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-07-23 AMIT V.       6.1.1  Created(Search by USERNAME)

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_FTP_USERS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_USERNAME=>'||p_USERNAME||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_USERNAME IS NOT NULL
 THEN
    OPEN RC1 FOR
    SELECT
        C.FTPUSERID		FTPUSERID
        ,RTRIM(C.USERNAME)	USERNAME
        ,RTRIM(P.PASSWORD)	PASSWORD
        ,RTRIM(C.HOMEDIRECTORY)	HOMEDIRECTORY
        ,C.ENABLEFLAG		ENABLEFLAG
        ,C.WRITEPERMISSION	WRITEPERMISSION
        ,C.IDLETIME		IDLETIME
        ,C.UPLOADRATE		UPLOADRATE
        ,C.DOWNLOADRATE		DOWNLOADRATE
        ,C.MAXLOGINNUMBER	MAXLOGINNUMBER
        ,C.MAXLOGINPERIP	MAXLOGINPERIP
        ,C.MODIFYDIR		MODIFYDIR
        ,C.UPDATEDATE		UPDATEDATE
        ,C.CREATEDATE		CREATEDATE
        ,C.ROWTS		ROWTS
    FROM
        CF_FTP_USER C, PR_USER P
    Where
        C.USERNAME=P.USERNAME
        AND C.USERNAME LIKE p_USERNAME
    ;
    ELSE
    OPEN RC1 FOR
    SELECT
        C.FTPUSERID		FTPUSERID
        ,RTRIM(C.USERNAME)	USERNAME
        ,RTRIM(P.PASSWORD)	PASSWORD
        ,RTRIM(C.HOMEDIRECTORY)	HOMEDIRECTORY
        ,C.ENABLEFLAG		ENABLEFLAG
        ,C.WRITEPERMISSION	WRITEPERMISSION
        ,C.IDLETIME		IDLETIME
        ,C.UPLOADRATE		UPLOADRATE
        ,C.DOWNLOADRATE		DOWNLOADRATE
        ,C.MAXLOGINNUMBER	MAXLOGINNUMBER
        ,C.MAXLOGINPERIP	MAXLOGINPERIP
        ,C.MODIFYDIR		MODIFYDIR
        ,C.UPDATEDATE		UPDATEDATE
        ,C.CREATEDATE		CREATEDATE
        ,C.ROWTS		ROWTS
    FROM
        CF_FTP_USER C, PR_USER P
    Where
        C.USERNAME=P.USERNAME
    ;
    END IF;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_FTP_USERS_AH1;
/


-- CF_FTP_USERU.sql

Create or Replace Procedure CF_FTP_USERU
	(
	 p_ReturnCd out integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_USERNAME in VARCHAR2
	,p_HOMEDIRECTORY in VARCHAR2
	,p_ENABLEFLAG in NUMBER
	,p_WRITEPERMISSION in NUMBER
	,p_IDLETIME in NUMBER
	,p_UPLOADRATE in NUMBER
	,p_DOWNLOADRATE in NUMBER
	,p_MAXLOGINNUMBER in NUMBER
	,p_MAXLOGINPERIP in NUMBER
	,p_ROWTS in NUMBER
	,p_MODIFYDIR in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_FTP_USERU
	Create Date:     2007-07-19 10:27:24
	Author:          AMIT KUMAR VIJAY 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-07-19 AMIT V.       6.1.1  Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_ROWCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_FTP_USERU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_USERNAME=>'||p_USERNAME||CHR(10)
		||', p_HOMEDIRECTORY=>'||p_HOMEDIRECTORY||CHR(10)
		||', p_ENABLEFLAG=>'||p_ENABLEFLAG||CHR(10)
		||', p_WRITEPERMISSION=>'||p_WRITEPERMISSION||CHR(10)
		||', p_IDLETIME=>'||p_IDLETIME||CHR(10)
		||', p_UPLOADRATE=>'||p_UPLOADRATE||CHR(10)
		||', p_DOWNLOADRATE=>'||p_DOWNLOADRATE||CHR(10)
		||', p_MAXLOGINNUMBER=>'||p_MAXLOGINNUMBER||CHR(10)
		||', p_MAXLOGINPERIP=>'||p_MAXLOGINPERIP||CHR(10)
		||', p_MODIFYDIR=>'||p_MODIFYDIR||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_ROWTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_ROWCnt := 0;
	Update CF_FTP_USER
	SET
		HOMEDIRECTORY=p_HOMEDIRECTORY
		,      ENABLEFLAG=p_ENABLEFLAG
		,      WRITEPERMISSION=p_WRITEPERMISSION
		,      IDLETIME=p_IDLETIME
		,      UPLOADRATE=p_UPLOADRATE
		,      DOWNLOADRATE=p_DOWNLOADRATE
		,      MAXLOGINNUMBER=p_MAXLOGINNUMBER
		,      MAXLOGINPERIP=p_MAXLOGINPERIP
		,      MODIFYDIR=p_MODIFYDIR
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		USERNAME = p_USERNAME
		And ROWTS = p_ROWTS returning ROWTS into v_ROWTS;
	v_ROWCnt := SQL%rowcount;
	if (v_ROWCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_ROWTS ROWTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_FTP_USERU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_FTP_USERU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_FTP_USERU:'||SQLERRM);
End CF_FTP_USERU;
/



-- CF_GuidelineMapD.SQL

CREATE OR REPLACE Procedure CF_GUIDELINEMAPD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_GUIDELINEMAPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_GUIDELINEMAPD
	Create Date:     2003-09-26 08:23:55
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GUIDELINEMAPD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GUIDELINEMAPID=>'||p_GUIDELINEMAPID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_GUIDELINEMAP
	Where
		GUIDELINEMAPID = p_GUIDELINEMAPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_GUIDELINEMAPD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_GUIDELINEMAPD;
/


-- CF_GUIDELINEMAPD_AH1.sql

CREATE OR REPLACE PROCEDURE CF_GUIDELINEMAPD_AH1 
(
   p_returncd         OUT      PLS_INTEGER,
   p_debugflg         IN       PLS_INTEGER,
   p_guidelinemapid   IN       NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       CF_GUIDELINEMAPD_AH1
    Create Date:     2007-08-20 08:23:55
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-20 BHARANI J           1 Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_rowcnt         INTEGER         DEFAULT 0;
   v_begints        TIMESTAMP       DEFAULT fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   DEFAULT 'CF_GUIDELINEMAPD_AH1';
   v_exceptionflg   INTEGER         DEFAULT 0;
   v_paramstxt      VARCHAR2 (4000) DEFAULT    'p_DebugFlg=>' || p_debugflg || CHR (10) ||
                                               ', p_GUIDELINEMAPID=>' || p_guidelinemapid  || CHR (10)
                                               ;
   zerodelete       EXCEPTION;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
   pr_routingattr_lnku_ah3 (p_returncd            => p_returncd,
                            p_debugflg            => p_debugflg,
                            p_guidelinemapid      => p_guidelinemapid
                           );

   DELETE FROM cf_guidelinemap_lnk
         WHERE guidelinemapid = p_guidelinemapid;

   DELETE FROM cf_guidelinemap
         WHERE guidelinemapid = p_guidelinemapid;

   v_rowcnt := SQL%ROWCOUNT;

   IF (v_rowcnt = 0)
   THEN
      RAISE zerodelete;
   END IF;

   db_procauditlogi_ah1 (v_procname, v_begints, v_paramstxt, p_debugflg, 1);
   p_returncd := 0; 
   
EXCEPTION
   WHEN zerodelete
   THEN
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            'Zero Delete,' || v_paramstxt,
                            p_debugflg,
                            1
                           );
      raise_application_error (-20999,
                               '20002 CF_GUIDELINEMAPD_AH1: No Data Found'
                              );
                                
   WHEN OTHERS
   THEN
      p_returncd := 1;
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            SQLERRM || CHR (10) || v_paramstxt,
                            p_debugflg,
                            1
                           );
      raise_application_error (-20999, '20000 ' || SQLCODE || ' ' || SQLERRM);
END cf_guidelinemapd_ah1;
/

-- CF_GUIDELINEMAPI.sql

CREATE OR REPLACE Procedure CF_GUIDELINEMAPI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_GUIDELINEMAPFILENAME in VARCHAR2
	,p_GUIDELINEMAPDESC in VARCHAR2
	,p_GUIDELINEMAP in BLOB
	,p_COMMONMAPFLG in NUMBER
	,p_GUIDELINEMAPSIZE in NUMBER
   ,p_groupname IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_GUIDELINEMAPI
	Create Date:     2003-09-26 00:01:11
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GUIDELINEMAPI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GUIDELINEMAPFILENAME=>'||p_GUIDELINEMAPFILENAME||CHR(10)
		||', p_GUIDELINEMAPDESC=>'||p_GUIDELINEMAPDESC||CHR(10)
		||', p_COMMONMAPFLG=>'||p_COMMONMAPFLG||CHR(10)
		||', p_GUIDELINEMAPSIZE=>'||p_GUIDELINEMAPSIZE||CHR(10)
      ||', p_groupname=>'||p_groupname||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_GUIDELINEMAP(
		GUIDELINEMAPID
		,GUIDELINEMAPFILENAME
		,GUIDELINEMAPDESC
		,GUIDELINEMAP
		,COMMONMAPFLG
		,GUIDELINEMAPSIZE
      ,groupname
	) Values (
		CF_GUIDELINEMAP_SEQ.nextval
		,p_GUIDELINEMAPFILENAME
		,p_GUIDELINEMAPDESC
		,p_GUIDELINEMAP
		,p_COMMONMAPFLG
		,p_GUIDELINEMAPSIZE
      ,p_groupname
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_GUIDELINEMAP_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;			
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_GUIDELINEMAPI' || SQLCODE|| ' '||SQLERRM);
END CF_GUIDELINEMAPI;
/


-- CF_GuidelineMapS.SQL

CREATE OR REPLACE PROCEDURE CF_GUIDELINEMAPS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_GUIDELINEMAPID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_GUIDELINEMAPS
	Create Date:     2003-09-26 00:07:06
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GUIDELINEMAPS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GUIDELINEMAPID=>'||p_GUIDELINEMAPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_GUIDELINEMAPID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		GUIDELINEMAPID
		,RTRIM(GUIDELINEMAPFILENAME) GUIDELINEMAPFILENAME
		,RTRIM(GUIDELINEMAPDESC) GUIDELINEMAPDESC
		,GUIDELINEMAP
		,COMMONMAPFLG
		,ROWTS
		,CREATEDATE
		,GUIDELINEMAPSIZE
      ,groupname
	FROM
		CF_GUIDELINEMAP
	Where
		GUIDELINEMAPID = p_GUIDELINEMAPID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		GUIDELINEMAPID
		,RTRIM(GUIDELINEMAPFILENAME) GUIDELINEMAPFILENAME
		,RTRIM(GUIDELINEMAPDESC) GUIDELINEMAPDESC
		,GUIDELINEMAP
		,COMMONMAPFLG
		,ROWTS
		,CREATEDATE
		,GUIDELINEMAPSIZE
      ,groupname
	FROM
		CF_GUIDELINEMAP
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_GUIDELINEMAPS;
/


-- CF_GuideLineMapsS_AH3.SQL

CREATE OR REPLACE PROCEDURE CF_GuideLineMapsS_AH3
    (
      p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1 out db_ref.RCT
    , RC2 out  db_ref.RCT
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_GUIDELINEMAPFILENAME in varchar2
    , p_GUIDELINEMAPDESC in varchar2
    , p_groupname IN VARCHAR2
    )
AS
    /*
    ================================================================================
    Proc Name:       CF_GuideLineMapsS_AH3
    Create Date:     2003-04-21 14:58:42
    Author:          M.Mount
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-21 M.Mount       3.5    Created  Returns ALL Maps with paging logic
    2003-05-30 M.Mount       3.5    Fixed p_sortkey parameter.
    2003-06-16 V.Goel        3.5    Defect 1-1EXET -- Hard coded sort param
    2003-06-20 M.Mount       3.5    Use DB_Paging_Logic and DB_Build_WhereClause
    2003-07-07 V Goel        3.5    Use FN_Paging_Logic and FN_Build_Where Functions
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar(30) := 'CF_GuideLineMapsS_AH3';
v_ExceptionFlg    integer := 0;

--paging variables

  v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(4000);
  v_where            varchar2(4000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(4000);
  sql_inner          varchar2(4000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(4000);
v_ParamsTxt       varchar(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||',p_sortkey=>'|| p_sortkey||CHR(10)
        ||',p_PageSizeNbr=>'|| p_PageSizeNbr||CHR(10)
        ||',p_PageNbr=>'|| p_PageNbr||CHR(10)
        ||',p_GUIDELINEMAPFILENAME=>'|| p_GUIDELINEMAPFILENAME||CHR(10)
        ||',p_GUIDELINEMAPDESC=>'|| p_GUIDELINEMAPDESC||CHR(10)
        ||',p_groupname=>'|| p_groupname||CHR(10);
Procedure where_clause as
begin
    v_where := ' Where 1=1'||
    FN_Build_Where('GUIDELINEMAPFILENAME', 'VARCHAR2_LIKE'   , p_GUIDELINEMAPFILENAME) ||
    FN_Build_Where('GUIDELINEMAPDESC',     'VARCHAR2_LIKE'   , p_GUIDELINEMAPDESC) ||
    FN_Build_Where('groupname',            'VARCHAR2_LIKE'   , p_groupname);
end where_clause;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    p_ReturnCd := 0;
    where_clause;

    sqlstmt := 'SELECT
           guidelINEMAPID,
           RTRIM(GUIDELINEMAPFILENAME) AS GuideLineMapFileName,
           RTRIM(GUIDELINEMAPDESC) AS GuideLineMapDesc,
        guidelINEMAP AS GUIDELINEMAP,
        groupname,
        ROWTS,
        CREATEDATE,
        UPDATEDATE,
        GUIDELINEMAPSIZE,
        COMMONMAPFLG
    FROM CF_GUIDELINEMAP '||v_where;
    
  DBMS_OUTPUT.PUT_LINE('Output 1------'||p_sortkey);
  IF NVL(INSTR(p_sortkey,'ASC'),0) >  0 THEN
    v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'ASC')-1);
    sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ')'; 
   ELSIF INSTR(upper(p_sortkey),'DESC') > 0 THEN
      v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'DESC')-1);
      sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ') DESC'; 
   ELSE
      sqlstmt := sqlstmt ||' order by GUIDELINEMAPFILENAME DESC, groupname desc';
      --sqlstmt := sqlstmt ||' order by GUIDELINEMAPFILENAME DESC'; 
  END IF; 
        
   DB_Print('Sql paging 1', sqlstmt);

   sql_inner := 'FROM CF_GUIDELINEMAP';

    -- total row count Cursor

    IF RC2%ISOPEN THEN CLOSE RC2; END IF;
    IF NVL(P_PAGENBR,0) IN (0,1) THEN
      Execute Immediate 'select count(*)  '||sql_inner ||v_where INTO v_count;
   --   DB_Print('Sql_inner ', sql_inner);
   --   DB_Print('Sql_where ', v_where);
       open RC2 for select v_count TotalRowsNbr from dual;
     end if;
-- Paging logic
     sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
 --  Set_page_parameters(p_pageNbr, p_pageSizeNbr, sqlstmt);

    DB_Print('Sql paging', sqlstmt);

    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for sqlstmt;
      p_ReturnCd := 0;
      DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 Exception
    When NO_DATA_FOUND THEN Null;
    When OTHERS Then
                p_ReturnCd     := 1;
                v_ExceptionFlg := 1;

End CF_GuideLineMapsS_AH3;
/


-- CF_GuideLineMapsS_AH5.SQL

CREATE OR REPLACE PROCEDURE         CF_GuideLineMapsS_AH5
    (
     p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1 out db_ref.RCT
    , p_guidelineMapFileName in varchar2
    , p_groupname IN VARCHAR2
    )
AS
/*
    ================================================================================
    Proc Name:       CF_GuideLineMapsS_AH5
    Create Date:     2003-04-21 14:58:42
    Author:          M.Mount
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-21 M.Mount       3.5 Created  --Returns for a specific MapFileName provided
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar(30) := 'CF_GuideLineMapsS_AH5';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_guidelineMapfileName=>'||p_guidelineMapFileName||CHR(10)
        ||', p_groupname=>'||p_groupname||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    
    IF p_groupname IS NULL THEN
      Open RC1 for
      SELECT
        guidelINEMAPID AS GUIDELINEMAPID,
           RTRIM(GUIDELINEMAPFILENAME) As GUIDELINEMAPFILENAME,
           RTRIM(GUIDELINEMAPDESC) As GUIDELINEMAPDESC,
        guidelINEMAP AS GUIDELINEMAP,
        ROWTS,
        CREATEDATE,
        UPDATEDATE,
        GUIDELINEMAPSIZE,
        COMMONMAPFLG,
        groupname
      FROM CF_GUIDELINEMAP
      WHERE
        GUIDELINEMAPFILENAME =  p_guidelineMapFileName;
        p_ReturnCd := 0;
    ELSE
      Open RC1 for
      SELECT
        guidelINEMAPID AS GUIDELINEMAPID,
           RTRIM(GUIDELINEMAPFILENAME) As GUIDELINEMAPFILENAME,
           RTRIM(GUIDELINEMAPDESC) As GUIDELINEMAPDESC,
        guidelINEMAP AS GUIDELINEMAP,
        ROWTS,
        CREATEDATE,
        UPDATEDATE,
        GUIDELINEMAPSIZE,
        COMMONMAPFLG,
        groupname
      FROM CF_GUIDELINEMAP
      WHERE
        GUIDELINEMAPFILENAME =  p_guidelineMapFileName
        AND groupname = p_groupname;
        p_ReturnCd := 0;
    END IF;

 DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 Exception
    When NO_DATA_FOUND THEN Null;
    When OTHERS Then
                p_ReturnCd     := 1;
                v_ExceptionFlg := 1;
End CF_GuideLineMapsS_AH5;
/


-- CF_GUIDELINEMAPS_AH6.sql

CREATE OR REPLACE PROCEDURE         CF_GUIDELINEMAPS_AH6
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_RoutingObjId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_GUIDELINEMAPS_AH6
    Create Date:     2003-05-15 11:16:01
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-15 M.Mount          3.5 Search by Routing Object Id
	  2006-03-21 Shishir N        6.1 60217-Partner Migration Changes
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_GUIDELINEMAPS_AH6';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_RoutingObjId=>'||p_RoutingObjId||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    IF p_RoutingObjId IS NOT NULL
		 THEN
    Open RC1 for
          select m.guidelINEMAPID AS GUIDELINEMAPID,
					               m.GUIDELINEMAPFILENAME AS GUIDELINEMAPFILENAME ,
					               RTRIM(m.GUIDELINEMAPDESC) AS GUIDELINEMAPDESC,
					            m.guidelINEMAP AS GUIDELINEMAP,
					            m.CREATEDATE,
					            m.UPDATEDATE,
					            m.GUIDELINEMAPSIZE,
					            m.COMMONMAPFLG,
                           m.groupname,
					            m.ROWTS AS MAPROWTS
					        From
					            CF_GUIDELINEMAP m
					            INNER JOIN cf_guidelineMap_lnk l on m.guidelinemapid  = l.guidelinemapid
					            INNER JOIN cf_RoutingObject r on l.routingObjId = r.routingObjId
					        Where
            r.routingObjId = p_RoutingObjId;
   
    ELSE
	OPEN RC1 FOR
	          select  GUIDELINEMAPID,
													 RTRIM(GUIDELINEMAPFILENAME) AS GUIDELINEMAPFILENAME ,
													 RTRIM(GUIDELINEMAPDESC) AS GUIDELINEMAPDESC,
												GUIDELINEMAP,
												CREATEDATE,
												UPDATEDATE,
												GUIDELINEMAPSIZE,
												COMMONMAPFLG,
                                    groupname,
												ROWTS
										From
												CF_GUIDELINEMAP;
   END IF;																	
         p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

 Exception
    When NO_DATA_FOUND THEN Null;
    When OTHERS Then
                p_ReturnCd     := 1;
                v_ExceptionFlg := 1;
End CF_GUIDELINEMAPS_AH6;
/


-- CF_GUIDELINEMAPU.SQL

CREATE OR REPLACE Procedure CF_GUIDELINEMAPU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_GUIDELINEMAPID in NUMBER
	,p_GUIDELINEMAPFILENAME in VARCHAR2
	,p_GUIDELINEMAPDESC in VARCHAR2
	,p_GUIDELINEMAP in BLOB
	,p_COMMONMAPFLG in NUMBER
	,p_ROWTS in NUMBER
	,p_GUIDELINEMAPSIZE in NUMBER
   ,p_groupname IN VARCHAR2
) as

/*
	================================================================================
	Proc Name:       CF_GUIDELINEMAPU
	Create Date:     2003-09-26 08:20:51
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GUIDELINEMAPU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GUIDELINEMAPID=>'||p_GUIDELINEMAPID||CHR(10)
		||', p_GUIDELINEMAPFILENAME=>'||p_GUIDELINEMAPFILENAME||CHR(10)
		||', p_GUIDELINEMAPDESC=>'||p_GUIDELINEMAPDESC||CHR(10)
		||', p_COMMONMAPFLG=>'||p_COMMONMAPFLG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_GUIDELINEMAPSIZE=>'||p_GUIDELINEMAPSIZE||CHR(10)
      ||', p_groupname=>'||p_groupname||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_GUIDELINEMAP
	SET
		GUIDELINEMAPFILENAME=p_GUIDELINEMAPFILENAME
		,      GUIDELINEMAPDESC=p_GUIDELINEMAPDESC
		,      GUIDELINEMAP=p_GUIDELINEMAP
		,      COMMONMAPFLG=p_COMMONMAPFLG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      GUIDELINEMAPSIZE=p_GUIDELINEMAPSIZE
      ,      groupname = p_groupname
	Where
		GUIDELINEMAPID = p_GUIDELINEMAPID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_GUIDELINEMAPU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_GUIDELINEMAPU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_GUIDELINEMAPU:'||SQLERRM);
End CF_GUIDELINEMAPU;
/


-- CF_GUIDELINEMAP_LNKD_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_GUIDELINEMAP_LNKD_AH1

    (p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,p_GuidelineMapId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_GUIDELINEMAP_LNKD
    Create Date:     2003-05-17 15:40:39
    Author:          SYSTEM
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_GUIDELINEMAP_LNKD';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_GuidelineMapId=>'||p_GuidelineMapId||CHR(10)
        ;

--zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    Delete From CF_GUIDELINEMAP_LNK
    Where
        GUIDELINEMAPID = p_GuidelineMapId
    ;
--    v_RowCnt := SQL%ROWCOUNT;
--    if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    p_ReturnCd := 0;
    Exception
--        When ZeroDelete Then
--            v_ExceptionFlg := 1;
--            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
        When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_GUIDELINEMAP_LNKD_AH1;
/


-- CF_GUIDELINEMAP_LNKD_AH2.sql

CREATE OR REPLACE PROCEDURE         CF_GUIDELINEMAP_LNKD_AH2

    (p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,p_GuidelineMapId in NUMBER
    ,p_ROUTINGOBJID in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_GUIDELINEMAP_LNKD_AH2
    Create Date:     2008-09-22 19:11:39
    Author:          AMIT KUMAR VIJAY
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_GUIDELINEMAP_LNKD_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
        ||', p_GuidelineMapId=>'||p_GuidelineMapId||CHR(10)
        ;

--zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    Delete From CF_GUIDELINEMAP_LNK
    Where
        GUIDELINEMAPID = p_GuidelineMapId
    and ROUTINGOBJID   = p_ROUTINGOBJID
    ;
--    v_RowCnt := SQL%ROWCOUNT;
--    if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    p_ReturnCd := 0;
    Exception
--        When ZeroDelete Then
--            v_ExceptionFlg := 1;
--            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
        When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_GUIDELINEMAP_LNKD_AH2;
/


-- CF_GUIDELINEMAP_LNKI.SQL

CREATE OR REPLACE Procedure CF_GUIDELINEMAP_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGOBJID in NUMBER
	,p_GUIDELINEMAPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_GUIDELINEMAP_LNKI
	Create Date:     2003-09-26 00:01:13
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GUIDELINEMAP_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		||', p_GUIDELINEMAPID=>'||p_GUIDELINEMAPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_GUIDELINEMAP_LNK(
		GUIDELINEMAPLNKID
		,ROUTINGOBJID
		,GUIDELINEMAPID
	) Values (
		CF_GUIDELINEMAP_LNK_SEQ.nextval
		,p_ROUTINGOBJID
		,p_GUIDELINEMAPID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_GUIDELINEMAP_LNK_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 CF_GUIDELINEMAP_LNKI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 CF_GUIDELINEMAP_LNKI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_GUIDELINEMAP_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_GUIDELINEMAP_LNKI;
/


-- CF_GWYHANDLERATTR_LNKD.sql

CREATE OR REPLACE Procedure CF_GWYHANDLERATTR_LNKD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_GATEWAYID in NUMBER  
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_HANDLERATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_GWYHANDLERATTR_LNKD
	Create Date:     2004-03-14 17:20:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GWYHANDLERATTR_LNKD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   
   v_where := ' WHERE 1=1 ' ;
   v_where := v_where || FN_BUILD_WHERE('GATEWAYID', 'NUMBER', p_GATEWAYID) ;
	v_where := v_where || FN_BUILD_WHERE('CFGPOINTID', 'VARCHAR2',p_CFGPOINTID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;
	
   v_sqlstmt := 'Delete From CF_GWYHANDLERATTR_LNK ' || v_where ;
	
	execute immediate v_sqlstmt ;

	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			--v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			--RAISE_APPLICATION_ERROR (-20999,'20002 CF_GWYHANDLERATTR_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_GWYHANDLERATTR_LNKD;
/


-- CF_GWYHANDLERATTR_LNKI.sql

CREATE OR REPLACE Procedure CF_GWYHANDLERATTR_LNKI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT 
	,p_GATEWAYID in NUMBER
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_HANDLERATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_GWYHANDLERATTR_LNKI
	Create Date:     2004-03-14 17:22:19
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GWYHANDLERATTR_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_GWYHANDLERATTR_LNK(
		GATEWAYID
		,HANDLERID
		,CFGPOINTID
		,HANDLERATTRID
		,ATTRVALUE
	) Values (
		p_GATEWAYID
		,p_HANDLERID
		,p_CFGPOINTID
		,p_HANDLERATTRID
		,p_ATTRVALUE
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_GWYHANDLERATTR_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_GWYHANDLERATTR_LNKI;
/


-- CF_GWYHANDLERATTR_LNKS.sql

CREATE OR REPLACE PROCEDURE CF_GWYHANDLERATTR_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_GATEWAYID IN NUMBER
	, p_CFGPOINTID IN VARCHAR2
	, p_HANDLERID IN NUMBER
	, p_HANDLERATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_GWYHANDLERATTR_LNKS
	Create Date:     2004-03-14 17:21:53
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created
   2004-05-24 PAD           4.2.2  Added v_where_inner logic
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
   
   **NOTE - if no handler or gateway or cfgpoint is given ALL handler default
            handler attributes will be returned
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GWYHANDLERATTR_LNKS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;        
v_where_inner     varchar2(4000) ;        
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' WHERE 1=1 ' ;
    
	
    if p_HANDLERATTRID is not null then
       v_where := v_where || FN_BUILD_WHERE('RA.HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where := v_where || FN_BUILD_WHERE('RA.HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_sqlstmt :=
       'SELECT '
	    ||'GATEWAYID'
	    ||',CFGPOINTID'
	    ||',RA.HANDLERID'
	    ||',RA.HANDLERATTRID'
       ||',RTRIM(ATTRNAME) ATTRNAME'
       ||',RTRIM(ATTRVALUE) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',RA.ROWTS'
       ||',RA.CREATEDATE '
       ||'FROM '
       ||'CF_GWYHANDLERATTR_LNK RA INNER JOIN PR_HANDLERATTR HA '
       ||'ON RA.HANDLERATTRID = HA.HANDLERATTRID '
       ||' AND RA.HANDLERID = HA.HANDLERID '
       || v_where ;
       
    ELSE
       v_where_inner := ' WHERE 1=1 ' ;
       v_where_inner := v_where_inner || FN_BUILD_WHERE('GATEWAYID', 'NUMBER', p_GATEWAYID) ;
       v_where_inner := v_where_inner || FN_BUILD_WHERE('CFGPOINTID', 'VARCHAR2',p_CFGPOINTID) ;
       v_where_inner := v_where_inner || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where_inner := v_where_inner || FN_BUILD_WHERE('HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;
       v_where := v_where || FN_BUILD_WHERE('HA.HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where := v_where || FN_BUILD_WHERE('HA.HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_sqlstmt :=
       'SELECT '
       ||'GATEWAYid'
   	 ||',CFGPOINTID'
	    ||',NVL(RA1.HANDLERID, ha.HANDLERID) HANDLERID '
       ||',NVL(ra1.HANDLERATTRID, ha.HANDLERATTRID) HANDLERATTRID '
       ||',RTRIM(attrname) ATTRNAME'
       ||',RTRIM(NVL(attrvalue, defaultattrvalue)) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',ra1.ROWTS'
       ||',ra1.CREATEDATE '
       ||'from '
       ||'(select GATEWAYid, cfgpointid, handlerid, handlerattrid, attrvalue, rowts, createdate '
       ||' from cf_GWYHANDLERattr_lnk RA '||v_where_inner||') RA1 '
       ||'right outer join pr_handlerattr HA '
       ||'on RA1.handlerattrid = HA.handlerattrid'
       ||v_where
       ;
    end if ;     
	
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_GWYHANDLERATTR_LNKS;
/


-- CF_GWYHANDLERATTR_LNKU.sql

CREATE OR REPLACE Procedure CF_GWYHANDLERATTR_LNKU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_GATEWAYID in NUMBER
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_HANDLERATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_GWYHANDLERATTR_LNKU
	Create Date:     2004-03-14 17:21:33
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GWYHANDLERATTR_LNKU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_GWYHANDLERATTR_LNK
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      ATTRVALUE=p_ATTRVALUE
	Where 
		GATEWAYID = p_GATEWAYID
		AND CFGPOINTID = p_CFGPOINTID
		AND HANDLERID = p_HANDLERID
		AND HANDLERATTRID = p_HANDLERATTRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_GWYHANDLERATTR_LNKU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_GWYHANDLERATTR_LNKU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_GWYHANDLERATTR_LNKU:'||SQLERRM);
End CF_GWYHANDLERATTR_LNKU;
/


-- CF_GWYHANDLER_XREFD.sql

CREATE OR REPLACE Procedure CF_GWYHANDLER_XREFD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_GATEWAYID in NUMBER
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_GWYHANDLER_XREFD
	Create Date:     2004-03-14 15:35:08
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GWYHANDLER_XREFD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   
   v_where := ' WHERE 1=1 ' ;
   v_where := v_where || FN_BUILD_WHERE('GATEWAYID', 'NUMBER', p_GATEWAYID) ;
	v_where := v_where || FN_BUILD_WHERE('CFGPOINTID', 'VARCHAR2',p_CFGPOINTID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	
   v_sqlstmt := 'Delete From CF_GWYHANDLER_XREF ' || v_where ;
	
	execute immediate v_sqlstmt ;
		
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			--v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			--RAISE_APPLICATION_ERROR (-20999,'20002 CF_GWYHANDLER_XREFD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_GWYHANDLER_XREFD;
/


-- CF_GWYHANDLER_XREFI.sql

CREATE OR REPLACE Procedure CF_GWYHANDLER_XREFI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT   
	,p_GATEWAYID in NUMBER
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_ORDERING in NUMBER

	)
AS

/*
	================================================================================
	Proc Name:       CF_GWYHANDLER_XREFI
	Create Date:     2004-03-14 15:35:45
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GWYHANDLER_XREFI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_GWYHANDLER_XREF(
		GATEWAYID
		,HANDLERID
		,ORDERING
		,CFGPOINTID
	) Values (
		p_GATEWAYID
		,p_HANDLERID
		,p_ORDERING
		,p_CFGPOINTID
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_GWYHANDLER_XREFI' || SQLCODE|| ' '||SQLERRM);
END CF_GWYHANDLER_XREFI;
/


-- CF_GWYHANDLER_XREFS.sql

CREATE OR REPLACE PROCEDURE CF_GWYHANDLER_XREFS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_GATEWAYID IN NUMBER
	, p_CFGPOINTID IN VARCHAR2
	, p_HANDLERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_GWYHANDLER_XREFS
	Create Date:     2004-03-14 15:36:13
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_GWYHANDLER_XREFS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;           
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('GATEWAYID', 'NUMBER', p_GATEWAYID) ||
       FN_Build_Where('CFGPOINTID', 'VARCHAR2', p_CFGPOINTID) ||
       FN_Build_Where('rx.HANDLERID', 'NUMBER', p_HANDLERID);
	   
	v_sqlstmt := 
       'SELECT '
        ||'GATEWAYID'
		||',RTRIM(CFGPOINTID) CFGPOINTID'
		||',rx.HANDLERID'
		||',ORDERING'
		||',PROVIDERFLG'
		||',RESOURCEKEY'
		||',DESCRIPTION'
		||',CLASSNAME'
		||',rx.ROWTS'
		||',rx.CREATEDATE '
		||'FROM '
		||'CF_GWYHANDLER_XREF rx INNER JOIN PR_HANDLER h ' 
		||'ON rx.HANDLERID = h.HANDLERID '
		||v_where 
      ||' ORDER BY ORDERING ASC '
		;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_GWYHANDLER_XREFS;
/


-- CF_LOCALE_MGMTI_AH1.sql

CREATE OR REPLACE PROCEDURE CF_LOCALE_MGMTI_AH1
        (
        p_ReturnCd out pls_integer
        ,p_DebugFlg in pls_integer
        ,RC1 out db_ref.RCT
        ,p_LOCALE in VARCHAR2
        ,p_BUNDLE1 in BLOB
        ,p_BUNDLE2 in BLOB
        ,p_BUNDLE3 in BLOB
        ,p_BUNDLE4 in BLOB
        ,p_BUNDLE5 in BLOB
        ,p_BUNDLE6 in BLOB
        ,p_BANNER in BLOB
        ,p_LOGO in BLOB
        ,p_CSS in BLOB
        ,p_HELP in BLOB
        ,p_ENABLEFLG in NUMBER
        )
AS

/*
        ================================================================================
        Proc Name:       CF_LOCALE_MGMTI_AH1
        Create Date:     2003-09-26 14:37:08
        Author:          SYSTEM

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-09-26 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_LOCALE_MGMTI_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_LOCALE=>'||p_LOCALE||CHR(10)
                ||', p_ENABLEFLG=>'||p_ENABLEFLG||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        --  Add new locale into pr_locale table    
	    Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg)
	    select p_locale, p_locale, 0 from Dual
    Where Not Exists (select 'x' from PR_LOCALE where LOCALE = p_locale);
        
        Insert into CF_LOCALE_MGMT(
                LOCALEMGMTID
                ,LOCALE
                ,BUNDLE1
                ,BUNDLE2
                ,BUNDLE3
                ,BUNDLE4
                ,BUNDLE5
                ,BUNDLE6
                ,BANNER
                ,LOGO
                ,CSS
                ,HELP
                ,ENABLEFLG
        ) Values (
                CF_LOCALE_MGMT_SEQ.nextval
                ,p_LOCALE
                ,p_BUNDLE1
                ,p_BUNDLE2
                ,p_BUNDLE3
                ,p_BUNDLE4
                ,p_BUNDLE5
                ,p_BUNDLE6
                ,p_BANNER
                ,p_LOGO
                ,p_CSS
                ,p_HELP
                ,p_ENABLEFLG
        );
        p_ReturnCd := 0;
        v_RowCnt := sql%ROWCOUNT;
        IF RC1%isOpen THEN Close RC1; END IF;
        Open RC1 FOR Select
                CF_LOCALE_MGMT_SEQ.currval ID, 1 RowTS From Dual;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        EXCEPTION
                When NO_DATA_FOUND THEN
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
                        RAISE_APPLICATION_ERROR (-20999,'20000 CF_LOCALE_MGMTI_AH1' || SQLCODE|| ' '||SQLERRM);
                When DUP_VAL_ON_INDEX then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        p_returnCd := 2;
                        RAISE_APPLICATION_ERROR (-20999,'20001 CF_LOCALE_MGMTI_AH1' || SQLCODE|| ' '||SQLERRM);
                When OTHERS Then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR (-20999,'20002 CF_LOCALE_MGMTI_AH1' || SQLCODE|| ' '||SQLERRM);
END CF_LOCALE_MGMTI_AH1;
/


-- CF_LOCALE_MGMTS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_LOCALE_MGMTS_AH1
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        )
AS

/*
        ================================================================================
        Proc Name:       CF_LOCALE_MGMTS_AH1
        Create Date:     2003-09-25 08:18:40
        Author:          V.Goel
            List all the Locale indicating which blob has value

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-09-25 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_LOCALE_MGMTS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        IF RC1%ISOPEN THEN CLOSE RC1; END IF;
        OPEN RC1 FOR
        SELECT
                LOCALEMGMTID
                ,RTRIM(M.LOCALE) LOCALE
                ,Case When Length(M.BUNDLE1)> 0 Then 1 Else 0 end  BUNDLE1Flg
                ,Case When Length(M.BUNDLE2)> 0 Then 1 Else 0 end  BUNDLE2Flg
                ,Case When Length(M.BUNDLE3)> 0 Then 1 Else 0 end  BUNDLE3Flg
                ,Case When Length(M.BUNDLE4)> 0 Then 1 Else 0 end  BUNDLE4Flg
                ,Case When Length(M.BUNDLE5)> 0 Then 1 Else 0 end  BUNDLE5Flg
                ,Case When Length(M.BUNDLE6)> 0 Then 1 Else 0 end  BUNDLE6Flg
                ,Case When Length(M.BANNER) > 0 Then 1 Else 0 end  BANNERFlg
                ,Case When Length(M.LOGO)   > 0 Then 1 Else 0 end  LOGOFlg
                ,Case When Length(M.CSS)    > 0 Then 1 Else 0 end  CSSFlg
                ,Case When Length(M.HELP)   > 0 Then 1 Else 0 end  HELPFlg
                ,M.ENABLEFLG
                ,M.ROWTS
                ,M.CREATEDATE
                ,M.UpdateDate
                ,L.DEFAULTFLG
        FROM
                CF_LOCALE_MGMT M
                left Outer Join PR_Locale L on L.LOCALE = M.LOCALE
        Order by M.Locale;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_LOCALE_MGMTS_AH1;
/


-- CF_LOCALE_MGMTS_AH2.sql

CREATE OR REPLACE PROCEDURE CF_LOCALE_MGMTS_AH2
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        , p_LOCALEMGMTID IN NUMBER
        )
AS

/*
        ================================================================================
        Proc Name:       CF_LOCALE_MGMTS_AH2
        Create Date:     2003-09-25 08:18:40
        Author:          SYSTEM

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-09-25 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_LOCALE_MGMTS_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_LOCALEMGMTID=>'||p_LOCALEMGMTID||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        IF RC1%ISOPEN THEN CLOSE RC1; END IF;
        OPEN RC1 FOR
        SELECT
                LOCALEMGMTID
                ,RTRIM(M.LOCALE) LOCALE
--
                ,Case When Length(M.BUNDLE1)> 0 Then 1 Else 0 end  BUNDLE1Flg
                ,Case When Length(M.BUNDLE2)> 0 Then 1 Else 0 end  BUNDLE2Flg
                ,Case When Length(M.BUNDLE3)> 0 Then 1 Else 0 end  BUNDLE3Flg
                ,Case When Length(M.BUNDLE4)> 0 Then 1 Else 0 end  BUNDLE4Flg
                ,Case When Length(M.BUNDLE5)> 0 Then 1 Else 0 end  BUNDLE5Flg
                ,Case When Length(M.BUNDLE6)> 0 Then 1 Else 0 end  BUNDLE6Flg
                ,Case When Length(M.BANNER) > 0 Then 1 Else 0 end  BANNERFlg
                ,Case When Length(M.LOGO)   > 0 Then 1 Else 0 end  LOGOFlg
                ,Case When Length(M.CSS)    > 0 Then 1 Else 0 end  CSSFlg
                ,Case When Length(M.HELP)   > 0 Then 1 Else 0 end  HELPFlg
--
                ,M.BUNDLE1
                ,M.BUNDLE2
                ,M.BUNDLE3
                ,M.BUNDLE4
                ,M.BUNDLE5
                ,M.BUNDLE6
                ,M.BANNER
                ,M.LOGO
                ,M.CSS
                ,M.HELP
                ,M.ENABLEFLG
                ,M.ROWTS
                ,M.CREATEDATE
                ,M.UpdateDate
                ,L.DEFAULTFLG
        FROM
                CF_LOCALE_MGMT M
                left Outer Join PR_Locale L on L.LOCALE = M.LOCALE
        WHERE
                M.LOCALEMGMTID = p_LOCALEMGMTID;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_LOCALE_MGMTS_AH2;
/


-- CF_LOCALE_MGMTU_AH1.sql

CREATE OR REPLACE PROCEDURE CF_LOCALE_MGMTU_AH1
        (
         p_ReturnCd out pls_integer
        , p_DebugFlg in pls_integer
        , RC1 out db_ref.RCT
        ,p_LOCALEMGMTID in NUMBER
        ,p_LOCALE in VARCHAR2
        ,p_BUNDLE1 in BLOB
        ,p_BUNDLE2 in BLOB
        ,p_BUNDLE3 in BLOB
        ,p_BUNDLE4 in BLOB
        ,p_BUNDLE5 in BLOB
        ,p_BUNDLE6 in BLOB
        ,p_BANNER in BLOB
        ,p_LOGO in BLOB
        ,p_CSS in BLOB
        ,p_HELP in BLOB
        ,p_ENABLEFLG in NUMBER
        ,p_ROWTS in NUMBER
) as

/*
        ================================================================================
        Proc Name:       CF_LOCALE_MGMTU_AH1
        Create Date:     2003-09-25 08:11:37
        Author:          SYSTEM

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-09-25 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_LOCALE_MGMTU_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_LOCALEMGMTID=>'||p_LOCALEMGMTID||CHR(10)
                ||', p_LOCALE=>'||p_LOCALE||CHR(10)
                ||', p_ENABLEFLG=>'||p_ENABLEFLG||CHR(10)
                ||', p_ROWTS=>'||p_ROWTS||CHR(10)
                ;

v_RowTS      integer :=0;
zeroUpdate  Exception;
/*
                ,      BUNDLE1 = Case When Length(p_BUNDLE1) > 0 Then p_BUNDLE1 Else BUNDLE1 end
                ,      BUNDLE2 = Case When Length(p_BUNDLE2) > 0 Then p_BUNDLE2 Else BUNDLE2 end
                ,      BUNDLE3 = Case When Length(p_BUNDLE3) > 0 Then p_BUNDLE3 Else BUNDLE3 end
                ,      BUNDLE4 = Case When Length(p_BUNDLE4) > 0 Then p_BUNDLE4 Else BUNDLE4 end
                ,      BUNDLE_DB = Case When Length(p_BUNDLE_DB) > 0 Then p_BUNDLE_DB Else BUNDLE_DB end
                ,      BANNER  = Case When Length(p_BANNER)  > 0 Then p_BANNER  Else BANNER  end
                ,      LOGO    = Case When Length(p_LOGO)    > 0 Then p_LOGO    Else LOGO    end
                ,      CSS     = Case When Length(p_CSS)     > 0 Then p_CSS     Else CSS     end
                ,      HELP    = Case When Length(p_HELP)    > 0 Then p_HELP    Else HELP    end

*/                        

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
        v_RowCnt := 0;
        
        Update CF_LOCALE_MGMT
        SET
                LOCALE=p_LOCALE
                ,      BUNDLE1   = nvl(p_BUNDLE1, BUNDLE1) 
                ,      BUNDLE2   = nvl(p_BUNDLE2, BUNDLE2)
                ,      BUNDLE3   = nvl(p_BUNDLE3, BUNDLE3)
                ,      BUNDLE4   = nvl(p_BUNDLE4, BUNDLE4)
                ,      BUNDLE5   = nvl(p_BUNDLE5, BUNDLE5)
                ,      BUNDLE6   = nvl(p_BUNDLE6, BUNDLE6)
                ,      BANNER    = nvl(p_BANNER, BANNER)
                ,      LOGO      = nvl(p_LOGO, LOGO)
                ,      CSS       = nvl(p_CSS, CSS)
                ,      HELP      = nvl(p_HELP, HELP)
                ,      ENABLEFLG = p_ENABLEFLG
                ,      ROWTS     = p_RowTS + 1
                ,      UPDATEDATE= FN_Get_Date
        Where
                LOCALEMGMTID = p_LOCALEMGMTID
                And RowTS    = p_RowTS 
        Returning RowTS into v_RowTS;
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
        IF RC1%isOpen THEN Close RC1; END IF;
        OPEN RC1 for select v_RowTS RowTS from Dual;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        EXCEPTION
                WHEN TOO_MANY_ROWS Then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999, '20100 - CF_LOCALE_MGMTU_AH1: Too Many Rows');
                WHEN ZeroUpdate then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - CF_LOCALE_MGMTU_AH1: No Data Found');
                WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - CF_LOCALE_MGMTU_AH1:'||SQLERRM);
End CF_LOCALE_MGMTU_AH1;
/


-- CF_PROFILEATTRD.sql

Create or Replace Procedure CF_PROFILEATTRD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_PROFILEATTRID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILEATTRD
	Create Date:     2004-09-06 18:06:04
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEATTRD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEATTRID=>'||p_PROFILEATTRID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_PROFILEATTR
	Where 
		PROFILEATTRID = p_PROFILEATTRID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILEATTRD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_PROFILEATTRD;
/


-- CF_PROFILEATTRI.sql

Create or Replace Procedure CF_PROFILEATTRI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ATTRNAME in VARCHAR2
	,p_PROFILETYPEID in NUMBER
	,p_EDITFLAG in NUMBER
	,p_MAXLEN in NUMBER
	,p_DEFAULTATTRVALUE in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROFILEATTRI
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEATTRI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILETYPEID=>'||p_PROFILETYPEID||CHR(10)
		||', p_MAXLEN=>'||p_MAXLEN||CHR(10)
		||', p_EDITFLAG=>'||p_EDITFLAG||CHR(10)
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)
		||', p_DEFAULTATTRVALUE=>'||p_DEFAULTATTRVALUE||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_PROFILEATTR(
		PROFILEATTRID
		,PROFILETYPEID
		,MAXLEN
		,EDITFLAG
		,ATTRNAME
		,DEFAULTATTRVALUE
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		CF_PROFILEATTR_SEQ.nextval
		,p_PROFILETYPEID
		,p_MAXLEN
		,p_EDITFLAG
		,p_ATTRNAME
		,p_DEFAULTATTRVALUE
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_PROFILEATTR_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILEATTRI' || SQLCODE|| ' '||SQLERRM);
END CF_PROFILEATTRI;
/


-- CF_PROFILEATTRS.sql

CREATE OR REPLACE PROCEDURE CF_PROFILEATTRS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROFILEATTRID IN NUMBER
	, p_PROFILETYPEID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILEATTRS
	Create Date:     2004-09-06 18:06:04
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	2006-05-11 S.Priani	 6.1	CR60181
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEATTRS';
v_ExceptionFlg    integer Default 0;
v_Where           Varchar2 (1000);
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEATTRID=>'||p_PROFILEATTRID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_where := ' Where 1 = 1 '
		                ||FN_BUILD_WHERE('PROFILEATTRID','NUMBER',p_PROFILEATTRID)
		                ||FN_BUILD_WHERE('PROFILETYPEID','NUMBER',p_PROFILETYPEID);
		
			IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	OPEN RC1 FOR 
	'SELECT a.PROFILEATTRID ,a.PROFILETYPEID ,a.EDITFLAG ,RTRIM(a.ATTRNAME) ATTRIBUTENAME'
	        	    ||', RTRIM(a.DEFAULTATTRVALUE) ATTRIBUTEVALUE,MAXLEN MAXLENGTH '
			    ||', RTRIM(a.RESOURCEKEY) RESOURCEKEY, RTRIM(a.DESCRIPTION) DESCRIPTION'
			    ||', a.ROWTS'
              		||' ,g.ProfileAttr_GrpId'
			    ||'  FROM '
			    ||'  CF_PROFILEATTR a'
	||' Inner Join CF_PROFILEATTR_GRP g ON g.profileattr_grpid = a.profileattr_grpid'
		    || v_where
;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End CF_PROFILEATTRS;
/


-- CF_PROFILEATTRU.sql

Create or Replace Procedure CF_PROFILEATTRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PROFILEATTRID in NUMBER
	,p_PROFILETYPEID in NUMBER
	,p_ATTRNAME in VARCHAR2	
	,p_EDITFLAG in NUMBER	
	,p_MAXLEN in NUMBER
	,p_DEFAULTATTRVALUE in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2	
	,p_ROWTS in NUMBER
) as
/*
	================================================================================
	Proc Name:       CF_PROFILEATTRU
	Create Date:     2004-09-06 18:06:04
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEATTRU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEATTRID=>'||p_PROFILEATTRID||CHR(10)
		||', p_PROFILETYPEID=>'||p_PROFILETYPEID||CHR(10)
		||', p_MAXLEN=>'||p_MAXLEN||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_EDITFLAG=>'||p_EDITFLAG||CHR(10)
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)
		||', p_DEFAULTATTRVALUE=>'||p_DEFAULTATTRVALUE||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;
v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_RowCnt := 0;
	Update CF_PROFILEATTR
	SET
		PROFILETYPEID=p_PROFILETYPEID
		,      MAXLEN=p_MAXLEN
		,      ROWTS= p_RowTS + 1
		,      EDITFLAG=p_EDITFLAG
		,      UPDATEDATE= v_BeginTS
		,      ATTRNAME=p_ATTRNAME
		,      DEFAULTATTRVALUE=p_DEFAULTATTRVALUE
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
	Where 
		PROFILEATTRID = p_PROFILEATTRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_PROFILEATTRU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILEATTRU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILEATTRU:'||SQLERRM);
End CF_PROFILEATTRU;
/


-- CF_PROFILEATTR_GRPD.sql

-- > EXECUTE DB_GENPROC.GENPROC_DELETE('CF_PROFILEATTR_GRP');
CREATE OR REPLACE PROCEDURE CF_PROFILEATTR_GRPD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_PROFILEATTR_GRPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROFILEATTR_GRPD
	Create Date:     2006-05-09 13:43:14
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_PROFILEATTR_GRPD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEATTR_GRPID=>'||p_PROFILEATTR_GRPID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_PROFILEATTR_GRP
	Where 
		PROFILEATTR_GRPID = p_PROFILEATTR_GRPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_PROFILEATTR_GRPD;
/
-- > SPOOL OFF


-- CF_PROFILEATTR_GRPI.sql

-- > EXECUTE DB_GENPROC.GENPROC_INSERT('CF_PROFILEATTR_GRP', TRUE, TRUE );
CREATE OR REPLACE PROCEDURE CF_PROFILEATTR_GRPI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROFILEATTR_GRPKEY in VARCHAR2
	,p_ORDERING in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROFILEATTR_GRPI
	Create Date:     2006-05-09 13:43:13
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_PROFILEATTR_GRPI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEATTR_GRPKEY=>'||p_PROFILEATTR_GRPKEY||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_PROFILEATTR_GRP(
		PROFILEATTR_GRPID
		,PROFILEATTR_GRPKEY
		,ORDERING
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		CF_PROFILEATTR_GRP_SEQ.nextval
		,p_PROFILEATTR_GRPKEY
		,p_ORDERING
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_PROFILEATTR_GRP_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END CF_PROFILEATTR_GRPI;
/
-- > SPOOL OFF


-- CF_PROFILEATTR_GRPS.sql

CREATE OR REPLACE PROCEDURE CF_PROFILEATTR_GRPS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROFILEATTR_GRPID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROFILEATTR_GRPS
	Create Date:     2006-05-09 13:43:14
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_PROFILEATTR_GRPS';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEATTR_GRPID=>'||p_PROFILEATTR_GRPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PROFILEATTR_GRPID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		PROFILEATTR_GRPID
		,RTRIM(PROFILEATTR_GRPKEY) PROFILEATTR_GRPKEY
		,ORDERING
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,ROWTS
		,CREATEDATE
	FROM 
		CF_PROFILEATTR_GRP
	Where 
		PROFILEATTR_GRPID = p_PROFILEATTR_GRPID
	ORDER BY Ordering
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		PROFILEATTR_GRPID
		,RTRIM(PROFILEATTR_GRPKEY) PROFILEATTR_GRPKEY
		,ORDERING
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,ROWTS
		,CREATEDATE
	FROM 
		CF_PROFILEATTR_GRP
	ORDER BY Ordering
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROFILEATTR_GRPS;
/


-- CF_PROFILEATTR_GRPU.sql

-- > EXECUTE DB_GENPROC.GENPROC_UPDATE('CF_PROFILEATTR_GRP');
CREATE OR REPLACE PROCEDURE CF_PROFILEATTR_GRPU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PROFILEATTR_GRPID in NUMBER
	,p_PROFILEATTR_GRPKEY in VARCHAR2
	,p_ORDERING in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROFILEATTR_GRPU
	Create Date:     2006-05-09 13:43:15
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_PROFILEATTR_GRPU';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEATTR_GRPID=>'||p_PROFILEATTR_GRPID||CHR(10)
		||', p_PROFILEATTR_GRPKEY=>'||p_PROFILEATTR_GRPKEY||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_PROFILEATTR_GRP
	SET
		PROFILEATTR_GRPKEY=p_PROFILEATTR_GRPKEY
		,      ORDERING=p_ORDERING
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		PROFILEATTR_GRPID = p_PROFILEATTR_GRPID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End CF_PROFILEATTR_GRPU;
/
-- > SPOOL OFF


-- CF_PROFILEATTR_LNKD.sql

Create or Replace Procedure CF_PROFILEATTR_LNKD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_PROFILEID in NUMBER
	,p_PROFILEATTRID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILEATTR_LNKD
	Create Date:     2004-09-06 18:06:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEATTR_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		||', p_PROFILEATTRID=>'||p_PROFILEATTRID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_PROFILEATTR_LNK
	Where 
		PROFILEID = p_PROFILEID
		AND PROFILEATTRID = p_PROFILEATTRID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILEATTR_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_PROFILEATTR_LNKD;
/


-- CF_PROFILEATTR_LNKI.sql

Create or Replace Procedure CF_PROFILEATTR_LNKI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_PROFILEID in NUMBER
	,p_PROFILEATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILEATTR_LNKI
	Create Date:     2004-09-06 18:06:03
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           3.5 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEATTR_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		||', p_PROFILEATTRID=>'||p_PROFILEATTRID||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Insert into CF_PROFILEATTR_LNK(
		PROFILEID
		,PROFILEATTRID
		,ATTRVALUE
	) Values (
		p_PROFILEID
		,p_PROFILEATTRID
		,p_ATTRVALUE
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILEATTR_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_PROFILEATTR_LNKI;
/


-- CF_PROFILEATTR_LNKS.sql

CREATE OR REPLACE PROCEDURE CF_PROFILEATTR_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, RC2  OUT DB_REF.RCT
	, p_PROFILEID IN NUMBER
	, p_PROFILEATTRID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILEATTR_LNKS
	Create Date:     2004-09-06 18:06:03
	Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEATTR_LNKS';
v_ExceptionFlg    integer Default 0;
v_Where           Varchar2 (1000);
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		||', p_PROFILEATTRID=>'||p_PROFILEATTRID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_where := ' Where 1 = 1 '
	                ||FN_Build_Where('x.ATTRID', 'NUMBER', p_PROFILEATTRID)
	                ||FN_Build_Where('x.PROFILEID'  , 'NUMBER'  , p_PROFILEID);

		IF RC1%ISOPEN THEN CLOSE RC1; END IF;

		OPEN RC1 FOR
		'SELECT x.PROFILEID PROFILEID,x.ATTRID  PROFILEATTRID, x.EDITFLAG EDITFLAG'
	        	      ||' ,x.ATTRNAME ATTRIBUTENAME, x.MAXLENGTH MAXLENGTH, x.RESOURCEKEY RESOURCEKEY '
	        	      ||' ,x.DESCRIPTION DESCRIPTION, RTRIM(coalesce(l.ATTRVALUE, x.ATTRVALUE)) ATTRIBUTEVALUE'
	                      ||' ,L.ROWTS ROWTS FROM (SELECT  p.PROFILEID,C.PROFILEATTRID  ATTRID, DEFAULTATTRVALUE ATTRVALUE'
	                      ||' ,C.EDITFLAG EDITFLAG,C.ATTRNAME ATTRNAME , C.MAXLEN MAXLENGTH'
	                      ||' ,C.RESOURCEKEY RESOURCEKEY,C.DESCRIPTION DESCRIPTION'
	                      ||' FROM CF_PROFILE P'
	                      ||' INNER JOIN CF_PROFILEATTR C on p.profiletypeid = c.profiletypeid ) x'
	                      ||' left outer JOIN CF_PROFILEATTR_LNK L ON L.PROFILEATTRID = x.ATTRID'
	                      ||' and l.profileid = x.profileid '
                              || v_where ;
		IF RC2%ISOPEN THEN CLOSE RC2; END IF;
		IF p_profileid IS NULL and p_profileattrid IS NOT NULL THEN
			OPEN RC2 FOR
		         	SELECT PROFILEID, PROFILENAME
	            		FROM
	                	CF_PROFILE
	            		Where profileid in (select profileid from cf_profile
                                  	where profiletypeid = (select profiletypeid 
                                                            from cf_profileattr
                                                            where profileattrid = p_profileattrid));
		ELSE
					
			OPEN RC2 FOR
   			 	select PROFILEID, PROFILENAME
	            		from
	                	CF_PROFILE
	            		Where profileid = coalesce(p_profileid, profileid);
   				 
   		END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		              WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
		              WHEN OTHERS then
		                   p_ReturnCd := 1;
		                   v_ExceptionFlg := 1;
		                   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                   RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILEATTR_LNKS:'||SQLERRM);
End CF_PROFILEATTR_LNKS;
/

-- CF_PROFILEATTR_LNKS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_PROFILEATTR_LNKS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROFILENAME IN VARCHAR2
	, p_PROFILEATTRID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILEATTR_LNKS_AH1
	Create Date:     2004-09-06 18:06:03
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEATTR_LNKS_AH1';
v_ExceptionFlg    integer Default 0;
v_Where           Varchar2 (1000);
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILENAME=>'||p_PROFILENAME||CHR(10)
		||', p_PROFILEATTRID=>'||p_PROFILEATTRID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_where := ' Where 1 = 1 '
	                ||FN_Build_Where('x.ATTRID', 'NUMBER', p_PROFILEATTRID)
	                ||FN_Build_Where('x.PROFILENAME'  , 'VARCHAR2'  , p_PROFILENAME);
	
		IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
		OPEN RC1 FOR
		'SELECT x.PROFILEID PROFILEID,x.ATTRID  PROFILEATTRID, x.EDITFLAG EDITFLAG, x.PROFILENAME PROFILENAME'
	        	      ||' ,x.ATTRNAME ATTRIBUTENAME, x.MAXLENGTH MAXLENGTH, x.RESOURCEKEY RESOURCEKEY '
	        	      ||' ,x.DESCRIPTION DESCRIPTION, RTRIM(coalesce(l.ATTRVALUE, x.ATTRVALUE)) ATTRIBUTEVALUE'
	                      ||' ,L.ROWTS ROWTS FROM (SELECT  p.PROFILEID, p.PROFILENAME, C.PROFILEATTRID  ATTRID, DEFAULTATTRVALUE ATTRVALUE'
	                      ||' ,C.EDITFLAG EDITFLAG,C.ATTRNAME ATTRNAME , C.MAXLEN MAXLENGTH'
	                      ||' ,C.RESOURCEKEY RESOURCEKEY,C.DESCRIPTION DESCRIPTION'
	                      ||' FROM CF_PROFILE P'
	                      ||' INNER JOIN CF_PROFILEATTR C on p.profiletypeid = c.profiletypeid ) x'
	                      ||' left outer JOIN CF_PROFILEATTR_LNK L ON L.PROFILEATTRID = x.ATTRID'
	                      ||' and l.profileid = x.profileid '
                              || v_where ;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		              WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
		              WHEN OTHERS then
		                   p_ReturnCd := 1;
		                   v_ExceptionFlg := 1;
		                   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                   RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILEATTR_LNKS_AH1:'||SQLERRM);
End CF_PROFILEATTR_LNKS_AH1;
/


-- CF_PROFILEATTR_LNKU.sql

Create or Replace Procedure CF_PROFILEATTR_LNKU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PROFILEID in NUMBER
	,p_PROFILEATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	,p_ROWTS in NUMBER
) as
/*
	================================================================================
	Proc Name:       CF_PROFILEATTR_LNKU
	Create Date:     2004-09-06 18:06:02
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEATTR_LNKU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		||', p_PROFILEATTRID=>'||p_PROFILEATTRID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;
v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_RowCnt := 0;
	Update CF_PROFILEATTR_LNK
	SET
		ROWTS= p_RowTS + 1
		,      ATTRVALUE=p_ATTRVALUE
		,      UPDATEDATE= v_BeginTS
	Where 
		PROFILEID = p_PROFILEID
		AND PROFILEATTRID = p_PROFILEATTRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_PROFILEATTR_LNKU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILEATTR_LNKU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILEATTR_LNKU:'||SQLERRM);
End CF_PROFILEATTR_LNKU;
/


-- CF_PROFILED.sql

Create or Replace Procedure CF_PROFILED
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_PROFILEID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILED
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILED';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_PROFILE
	Where 
		PROFILEID = p_PROFILEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILED: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_PROFILED;
/


-- CF_PROFILEI.sql

Create or Replace Procedure CF_PROFILEI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROFILETYPEID in NUMBER	
	,p_OVERRIDEPROFILEID in NUMBER
	,p_PROFILENAME in VARCHAR2
	,p_VISIBLE in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILEI
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_OVERRIDEPROFILEID=>'||p_OVERRIDEPROFILEID||CHR(10)
		||', p_PROFILETYPEID=>'||p_PROFILETYPEID||CHR(10)
		||', p_VISIBLE=>'||p_VISIBLE||CHR(10)
		||', p_PROFILENAME=>'||p_PROFILENAME||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Insert into CF_PROFILE(
		PROFILEID
		,OVERRIDEPROFILEID
		,PROFILETYPEID
		,VISIBLE
		,PROFILENAME
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		CF_PROFILE_SEQ.nextval
		,p_OVERRIDEPROFILEID
		,p_PROFILETYPEID
		,p_VISIBLE
		,p_PROFILENAME
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_PROFILE_SEQ.currval ProfileID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILEI' || SQLCODE|| ' '||SQLERRM);
END CF_PROFILEI;
/


-- CF_PROFILEI_COPY.sql

CREATE OR REPLACE PROCEDURE CF_PROFILEI_COPY
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 OUT Db_Ref.RCT
	,p_PROFILEID IN NUMBER	
	,p_PROFILENAME IN VARCHAR2
	,p_NewDESCRIPTION IN VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILEI_COPY
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        NUMBER := 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROFILEI_COPY';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		||', p_PROFILENAME=>'||p_PROFILENAME||CHR(10)
		||', p_NewDESCRIPTION=>'||p_NewDESCRIPTION||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	INSERT INTO CF_PROFILE(
		PROFILEID
		,OVERRIDEPROFILEID
		,PROFILETYPEID
		,VISIBLE
		,PROFILENAME
		,RESOURCEKEY
		,DESCRIPTION
	) SELECT
				CF_PROFILE_SEQ.NEXTVAL
				,OVERRIDEPROFILEID
				,PROFILETYPEID
				,VISIBLE
				,p_PROFILENAME
				,RESOURCEKEY
				,p_NewDESCRIPTION
				FROM
				CF_PROFILE
					WHERE
				PROFILEID = p_PROFILEID;
				
	v_RowCnt := SQL%ROWCOUNT;
	IF RC1%isOpen THEN CLOSE RC1; END IF;
	OPEN RC1 FOR SELECT 
		CF_PROFILE_SEQ.CURRVAL PROFILEID, 1 RowTS FROM Dual;
		
	INSERT INTO CF_PROFILEATTR_LNK 
	               (PROFILEID,
	                PROFILEATTRID,
	                ATTRVALUE)
		SELECT	 
			 CF_PROFILE_SEQ.CURRVAL,
			 PROFILEATTRID,
			 ATTRVALUE
	         FROM
		               CF_PROFILEATTR_LNK
		               WHERE
			       profileID = p_profileID;	
		p_ReturnCd := 0;
	Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN DUP_VAL_ON_INDEX THEN
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN OTHERS THEN
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILEI_COPY' || SQLCODE|| ' '||SQLERRM);
END CF_PROFILEI_COPY;
/

-- CF_PROFILES.sql

CREATE OR REPLACE PROCEDURE CF_PROFILES
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROFILEID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILES
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2004-09-06 SYSTEM	4.3	Created
	2006-06-23 S.Priani	6.1	Changes for ebXML
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILES';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PROFILEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		C.PROFILEID
		,C.PROFILETYPEID
		,P.PROFILETYPENAME PROFILETYPENAME
		,RTRIM(C.PROFILENAME) PROFILENAME
		,RTRIM(C.RESOURCEKEY) RESOURCEKEY
		,RTRIM(C.DESCRIPTION) DESCRIPTION
		,C.ROWTS
		,C.CREATEDATE
		,C.VISIBLE
		,C.OVERRIDEPROFILEID
		,C.CanDelete
	FROM 
		CF_PROFILE C,
		CF_PROFILETYPE P
	WHERE 
		PROFILEID = p_PROFILEID
		and C.PROFILETYPEID = P.PROFILETYPEID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		C.PROFILEID
		,C.PROFILETYPEID
		,P.PROFILETYPENAME PROFILETYPENAME
		,RTRIM(C.PROFILENAME) PROFILENAME
		,RTRIM(C.RESOURCEKEY) RESOURCEKEY
		,RTRIM(C.DESCRIPTION) DESCRIPTION
		,C.ROWTS
		,C.CREATEDATE
		,C.VISIBLE
		,C.OVERRIDEPROFILEID
		,C.CanDelete
	FROM 
		CF_PROFILE C,
		CF_PROFILETYPE P
	WHERE 
		 C.PROFILETYPEID = P.PROFILETYPEID
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		              WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
		              WHEN OTHERS then
		                   p_ReturnCd := 1;
		                   v_ExceptionFlg := 1;
		                   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                   RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILES:'||SQLERRM);
End CF_PROFILES;
/


-- CF_PROFILES_AH1.sql

CREATE OR REPLACE PROCEDURE CF_PROFILES_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROFILEID IN NUMBER
	, p_PROFILETYPEID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILES_AH1
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2004-09-06 SYSTEM	4.3	Created
	2006-06-23 S.Priani	6.1	Changes for ebXML
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILES_AH1';
v_ExceptionFlg    integer Default 0;
v_Where           Varchar2 (1000);
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		||', p_PROFILETYPEID=>'||p_PROFILETYPEID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_where := ' Where C.PROFILETYPEID=P.PROFILETYPEID'
	|| FN_BUILD_WHERE('C.PROFILEID','NUMBER',p_PROFILEID)
        || FN_BUILD_WHERE('C.PROFILETYPEID','NUMBER',p_PROFILETYPEID);
        
		
			IF RC1%ISOPEN THEN CLOSE RC1; END IF;
		
		OPEN RC1 FOR
		'SELECT C.PROFILEID ,C.PROFILETYPEID ,P.PROFILETYPENAME, RTRIM(C.PROFILENAME) PROFILENAME'
			    ||', RTRIM(C.RESOURCEKEY) RESOURCEKEY, RTRIM(C.DESCRIPTION) DESCRIPTION'
			    ||', C.ROWTS, C.CREATEDATE, C.VISIBLE, C.OVERRIDEPROFILEID, C.CANDELETE'
		    ||'  FROM '
			    ||'  CF_PROFILE C, '
			    ||'  CF_PROFILETYPE P '
		    || v_where;       
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROFILES_AH1;
/


-- CF_PROFILES_AH3.sql

CREATE OR REPLACE PROCEDURE CF_PROFILES_AH3
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  Db_Ref.RCT
	, p_ROUTINGOBJID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILES_AH3
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2004-09-06 SYSTEM	4.3	Created
	2006-06-23 S.Priani	6.1	Changes for ebXML
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

================================================================================*/
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROFILES_AH3';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ROUTINGOBJID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
			C.PROFILEID
			,C.PROFILETYPEID
			,P.PROFILETYPENAME
			,C.PROFILENAME
			,C.RESOURCEKEY
			,C.DESCRIPTION
			,C.CANDELETE
		FROM
			CF_PROFILE_LNK CPL
			INNER JOIN CF_PROFILE C ON CPL.PROFILEID = C.PROFILEID
			INNER JOIN CF_PROFILETYPE P ON P.PROFILETYPEID = C.PROFILETYPEID
		WHERE
		CPL.ROUTINGOBJID = p_ROUTINGOBJID;
	END IF;		
	p_ReturnCd := 0;
	Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
	              WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
	              WHEN OTHERS THEN
	                   p_ReturnCd := 1;
	                   v_ExceptionFlg := 1;
	                   Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                   RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILES_AH3:'||SQLERRM);
END CF_PROFILES_AH3;
/


-- CF_PROFILES_AH4.sql

CREATE OR REPLACE PROCEDURE CF_PROFILES_AH4
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  Db_Ref.RCT
	, p_PROFILEID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILES_AH4
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROFILES_AH4';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

OPEN RC1 FOR 
	select 
    crv.routingobjid ROUTINGOBJID,
    cro.routingobjname ROUTINGOBJNAME,
    crv.routingobjcd ROUTINGOBJCD,
    crv.routingobjrefid ROUTINGOBJREFID,    
    crv.routingobjver ROUTINGOBJVER,
    cro2.routingobjname ROUTINGOBJPARENTNAME,
    cro3.routingobjname ROUTINGOBJGRPARENTNAME
from
    cf_routingobject05_vw crv
    INNER JOIN pr_routingattr_lnk prl on prl.routingobjrefid = crv.routingobjrefid
    INNER JOIN cf_routingobject cro ON cro.ROUTINGOBJID = crv.ROUTINGOBJID
    LEFT OUTER JOIN cf_routingobject cro2 on cro2.ROUTINGOBJID = crv.PARENTOBJID
    LEFT OUTER JOIN cf_routingobjref_lnk crl on crl.ROUTINGOBJREFID = crv.GRPARENTOBJREFID
    LEFT OUTER JOIN cf_routingobject cro3 on cro3.ROUTINGOBJID = crl.ROUTINGOBJID
    WHERE
		 prl.profileid = p_PROFILEID;
	p_ReturnCd := 0;
	Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
	              WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
	              WHEN OTHERS THEN
	                   p_ReturnCd := 1;
	                   v_ExceptionFlg := 1;
	                   Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                   RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILES_AH4:'||SQLERRM);
END CF_PROFILES_AH4;
/

-- CF_PROFILES_EBXML.sql

CREATE OR REPLACE PROCEDURE CF_PROFILES_EBXML
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROFILES_EBXML
	Create Date:     2006-07-03 11:17:48
	Author:          SYSTEM 

Returns the Default ebXML synchronous and async profiles for Console
	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-07-03 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_PROFILES_EBXML';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
		SELECT	profileid, profiletypeid, profilename, description, rowts, createdate, updatedate, canDelete
		FROM	cf_profile
		WHERE	profilename IN ('WPG-ebMS-Synchronous','WPG-ebMS-ASynchronous')
		AND	canDelete = 1
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End CF_PROFILES_EBXML;
/


-- CF_PROFILETYPED.sql

Create or Replace Procedure CF_PROFILETYPED
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_PROFILETYPEID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILETYPED
	Create Date:     2004-09-06 18:06:03
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILETYPED';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILETYPEID=>'||p_PROFILETYPEID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_PROFILETYPE
	Where 
		PROFILETYPEID = p_PROFILETYPEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILETYPED: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_PROFILETYPED;
/


-- CF_PROFILETYPEI.sql

Create or Replace Procedure CF_PROFILETYPEI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROFILETYPEID in NUMBER
	,p_PROFILETYPENAME in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILETYPEI
	Create Date:     2004-09-06 18:06:04
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILETYPEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILETYPEID=>'||p_PROFILETYPEID||CHR(10)
		||', p_PROFILETYPENAME=>'||p_PROFILETYPENAME||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Insert into CF_PROFILETYPE(
		PROFILETYPEID
		,PROFILETYPENAME
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		p_PROFILETYPEID
		,p_PROFILETYPENAME
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILETYPEI' || SQLCODE|| ' '||SQLERRM);
END CF_PROFILETYPEI;
/


-- CF_PROFILETYPES.sql

CREATE OR REPLACE PROCEDURE CF_PROFILETYPES
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROFILETYPEID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILETYPES
	Create Date:     2004-09-06 18:06:04
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILETYPES';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILETYPEID=>'||p_PROFILETYPEID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PROFILETYPEID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		PROFILETYPEID
		,ROWTS
		,CREATEDATE
		,RTRIM(PROFILETYPENAME) PROFILETYPENAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
	FROM 
		CF_PROFILETYPE
	Where 
		PROFILETYPEID = p_PROFILETYPEID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		PROFILETYPEID
		,ROWTS
		,CREATEDATE
		,RTRIM(PROFILETYPENAME) PROFILETYPENAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
	FROM 
		CF_PROFILETYPE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROFILETYPES;
/


-- cf_profiletypeS_ah1.sql

CREATE OR REPLACE PROCEDURE cf_profiletypeS_ah1
	(
	  p_ReturnCd  OUT PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1         OUT DB_REF.RCT
	)
AS

/*
   This procedure lists EDI Standards

	================================================================================
	Proc Name:       CF_PROFILETYPES_ah1
	Create Date:     2005-12-09 10:04:32
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2005-12-09 SYSTEM	3.5	Created
	2006-06-23 S.Priani	6.1	Changes for ebXML

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
================================================================================*/


v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROFILETYPES_ah1';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR
		SELECT
		 profiletypeid STANDARDID
		,RTRIM(profiletypename) STANDARDNAME
		,RTRIM(resourcekey) NAMERESKEY
	FROM cf_profiletype
	WHERE profiletypename NOT IN  ('CONNECTION','EDI','ebMS')
	;

   p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END CF_PROFILETYPES_ah1;
/


-- CF_PROFILETYPEU.sql

Create or Replace Procedure CF_PROFILETYPEU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PROFILETYPEID in NUMBER
	,p_PROFILETYPENAME in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_ROWTS in NUMBER
) as
/*
	================================================================================
	Proc Name:       CF_PROFILETYPEU
	Create Date:     2004-09-06 18:06:03
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILETYPEU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILETYPEID=>'||p_PROFILETYPEID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_PROFILETYPENAME=>'||p_PROFILETYPENAME||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;
v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_RowCnt := 0;
	Update CF_PROFILETYPE
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      PROFILETYPENAME=p_PROFILETYPENAME
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
	Where 
		PROFILETYPEID = p_PROFILETYPEID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_PROFILETYPEU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILETYPEU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILETYPEU:'||SQLERRM);
End CF_PROFILETYPEU;
/


-- CF_PROFILEU.sql

Create or Replace Procedure CF_PROFILEU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PROFILEID in NUMBER
	,p_PROFILENAME in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_ROWTS in NUMBER
) as
/*
	================================================================================
	Proc Name:       CF_PROFILEU
	Create Date:     2004-09-06 18:06:06
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILEU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_PROFILENAME=>'||p_PROFILENAME||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;
v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	v_RowCnt := 0;
	Update CF_PROFILE
		SET
	
			       PROFILENAME=p_PROFILENAME
			,      RESOURCEKEY=p_RESOURCEKEY
			,      DESCRIPTION=p_DESCRIPTION
			,      ROWTS= p_RowTS + 1
			,      UPDATEDATE= v_BeginTS
		WHERE 
			PROFILEID = p_PROFILEID
			And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_PROFILEU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILEU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROFILEU:'||SQLERRM);
End CF_PROFILEU;
/


-- CF_PROFILE_LNKD.sql

Create or Replace Procedure CF_PROFILE_LNKD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_PROFILELNKID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILE_LNKD
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILE_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILELNKID=>'||p_PROFILELNKID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_PROFILE_LNK
	Where 
		PROFILELNKID = p_PROFILELNKID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILE_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_PROFILE_LNKD;
/


-- CF_PROFILE_LNKI.sql

Create or Replace Procedure CF_PROFILE_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGOBJID in NUMBER
	,p_PROFILEID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILE_LNKI
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROFILE_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Insert into CF_PROFILE_LNK(
		PROFILELNKID
		,PROFILEID
		,ROUTINGOBJID
	) Values (
		CF_PROFILE_LNK_SEQ.nextval
		,p_PROFILEID
		,p_ROUTINGOBJID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_PROFILE_LNK_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILE_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_PROFILE_LNKI;
/


-- CF_PROFILE_LNKI_AH1.sql

CREATE OR REPLACE PROCEDURE CF_PROFILE_LNKI_AH1
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,p_PROFILEID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILE_LNKI_AH1
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        NUMBER := 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROFILE_LNKI_AH1';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		;
v_routingobjid    INTEGER DEFAULT 0;		
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
CURSOR c_routingobj IS
	              SELECT 
	                   DISTINCT routingobjid
	              FROM 
                CF_PROFILE_LNK;

BEGIN
	OPEN c_routingobj;
	LOOP
	FETCH c_routingobj INTO v_routingobjid;
	EXIT WHEN c_routingobj%NOTFOUND;
	INSERT INTO CF_PROFILE_LNK(
		            		ProfileLnkId
		            		,ProfileId
		            		,RoutingObjId
		            		,RowTS)
		            VALUES
		            		(
		            		CF_PROFILE_LNK_SEQ.NEXTVAL
		            		,p_PROFILEID
		            		,v_routingobjid
	            		,1);
	END LOOP;
	
	p_ReturnCd := 0;
		Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN DUP_VAL_ON_INDEX THEN
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN OTHERS THEN
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILE_LNKI_AH1' || SQLCODE|| ' '||SQLERRM);
END CF_PROFILE_LNKI_AH1;
/

-- CF_PROFILE_LNKI_AH2.sql

CREATE OR REPLACE PROCEDURE CF_PROFILE_LNKI_AH2
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,p_SRCDICTNAME IN NCHAR
	)
AS
/*
	================================================================================
	Proc Name:       CF_PROFILE_LNKI_AH2
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        NUMBER := 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROFILE_LNKI_AH2';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_SRCDICTNAME=>'||p_SRCDICTNAME||CHR(10)
		;
v_profileid    INTEGER DEFAULT 0;		
v_RoutingobjID  INTEGER DEFAULT 0;	
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

CURSOR c_profiles IS
	              SELECT 
	                   profileid
	              FROM 
                cf_profile p
                WHERE profiletypeid != 1;

BEGIN
	SELECT routingobjid INTO v_RoutingobjID FROM CF_RoutingObject o 
			                   inner join CF_RoutingObjType t ON t.RoutingObjTypeId = o.RoutingObjTypeId
			                   WHERE o.RoutingObjCd = p_SRCDICTNAME
			                  AND   o.Routingobjver = 'ALL'
		                   AND   t.routingobjtypekey = 'Protocol';
	OPEN c_profiles;
	        LOOP
	        FETCH c_profiles INTO v_profileid;
	        EXIT WHEN c_profiles%NOTFOUND;            
	            INSERT INTO cf_profile_lnk(
	            		ProfileLnkId
	            		,ProfileId
	            		,RoutingObjId
	            		,RowTS)
	            VALUES
	            		(
	            		cf_profile_lnk_SEQ.NEXTVAL
	            		,v_profileid
	            		,v_ROUTINGOBJID
	            		,1);
	        
	        END LOOP;
    CLOSE c_profiles;
	p_ReturnCd := 0;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				v_ExceptionFlg := 1;
				Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 3;
			WHEN DUP_VAL_ON_INDEX THEN
				v_ExceptionFlg := 1;
				Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
				p_returnCd := 2;
			WHEN OTHERS THEN
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROFILE_LNKI_AH2' || SQLCODE|| ' '||SQLERRM);
	END CF_PROFILE_LNKI_AH2;
/	

-- CF_PROTOCOLATTRD.sql

CREATE OR REPLACE Procedure CF_PROTOCOLATTRD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_PROTOCOLATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROTOCOLATTRD
	Create Date:     2003-09-26 08:23:57
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROTOCOLATTRD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROTOCOLATTRID=>'||p_PROTOCOLATTRID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_PROTOCOLATTR
	Where
		PROTOCOLATTRID = p_PROTOCOLATTRID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROTOCOLATTRD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_PROTOCOLATTRD;
/


-- CF_PROTOCOLATTRI.sql

CREATE OR REPLACE Procedure CF_PROTOCOLATTRI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROTOCOLDEFID in NUMBER
	,p_ATTRVALUETYPEID in NUMBER
	,p_ATTRTYPEID in NUMBER
	,p_ATTRTYPEVALUE in VARCHAR2
   ,p_namespace IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROTOCOLATTRI
	Create Date:     2003-09-26 00:01:14
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROTOCOLATTRI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROTOCOLDEFID=>'||p_PROTOCOLDEFID||CHR(10)
		||', p_ATTRVALUETYPEID=>'||p_ATTRVALUETYPEID||CHR(10)
		||', p_ATTRTYPEID=>'||p_ATTRTYPEID||CHR(10)
		||', p_ATTRTYPEVALUE=>'||p_ATTRTYPEVALUE||CHR(10)
      ||', p_namespace=>'||p_namespace||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_PROTOCOLATTR(
		PROTOCOLATTRID
		,PROTOCOLDEFID
		,ATTRVALUETYPEID
		,ATTRTYPEID
		,ATTRTYPEVALUE
      ,namespace
	) Values (
		CF_PROTOCOLATTR_SEQ.nextval
		,p_PROTOCOLDEFID
		,p_ATTRVALUETYPEID
		,p_ATTRTYPEID
		,p_ATTRTYPEVALUE
      ,p_namespace
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_PROTOCOLATTR_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;			
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROTOCOLATTRI' || SQLCODE|| ' '||SQLERRM);
END CF_PROTOCOLATTRI;
/


-- CF_PROTOCOLATTRS.sql

CREATE OR REPLACE PROCEDURE CF_PROTOCOLATTRS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROTOCOLATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROTOCOLATTRS
	Create Date:     2003-09-26 00:07:04
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROTOCOLATTRS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROTOCOLATTRID=>'||p_PROTOCOLATTRID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PROTOCOLATTRID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		PROTOCOLATTRID
		,PROTOCOLDEFID
		,ATTRVALUETYPEID
		,ATTRTYPEID
		,RTRIM(ATTRTYPEVALUE) ATTRTYPEVALUE
      ,rtrim(namespace) namespace
		,ROWTS
		,CREATEDATE
	FROM
		CF_PROTOCOLATTR
	Where
		PROTOCOLATTRID = p_PROTOCOLATTRID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		PROTOCOLATTRID
		,PROTOCOLDEFID
		,ATTRVALUETYPEID
		,ATTRTYPEID
		,RTRIM(ATTRTYPEVALUE) ATTRTYPEVALUE
      ,rtrim(namespace) namespace
		,ROWTS
		,CREATEDATE
	FROM
		CF_PROTOCOLATTR
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROTOCOLATTRS;
/


-- CF_PROTOCOLATTRS_AH2.SQL

CREATE OR REPLACE PROCEDURE         CF_PROTOCOLATTRS_AH2
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_ProtocolDefId IN NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_PROTOCOLATTRS_AH2
    Create Date:     2003-05-07 09:16:50
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-08 M.Mount       3.5 Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_PROTOCOLATTRS_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_ProtocolDefId=>'||p_ProtocolDefId||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ProtocolDefId IS NOT NULL
 THEN
    OPEN RC1 FOR
    SELECT
        PROTOCOLATTRID
        ,PROTOCOLDEFID
        ,ATTRVALUETYPEID
        ,ATTRTYPEID
        ,RTRIM(ATTRTYPEVALUE) ATTRTYPEVALUE
        ,rtrim(namespace) namespace
        ,ROWTS
        ,CREATEDATE
    FROM
        CF_PROTOCOLATTR
    Where
        ProtocolDefId = p_ProtocolDefId
    ;
    ELSE
    OPEN RC1 FOR
    SELECT
        PROTOCOLATTRID
        ,PROTOCOLDEFID
        ,ATTRVALUETYPEID
        ,ATTRTYPEID
        ,RTRIM(ATTRTYPEVALUE) ATTRTYPEVALUE
        ,rtrim(namespace) namespace
        ,ROWTS
        ,CREATEDATE
    FROM
        CF_PROTOCOLATTR
    ;
    END IF;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROTOCOLATTRS_AH2;
/


-- cf_protocolattrtypeI.sql

CREATE OR REPLACE PROCEDURE CF_PROTOCOLATTRTYPEI
	(p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN  PLS_INTEGER                                         
	,p_ATTRTYPEKEY  IN VARCHAR2                                                     
	,p_CONSTANTFLG  IN NUMBER                                                       
	,p_REQUIREDFLG  IN NUMBER                                                       
	,p_SEQNBR IN NUMBER                                                            
	,p_RESOURCEKEY  IN VARCHAR2                                                     
	,p_DESCRIPTION  IN VARCHAR2                                                     
	,p_PROVIDERFLAG IN NUMBER                                                      
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       CF_PROTOCOLATTRTYPEI                                          
	Create Date:     2006-02-24 12:52:48                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2006-02-24 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             
v_PK_Value        NUMBER := 0;                                                  

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROTOCOLATTRTYPEI';                  
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_ATTRTYPEKEY=>'||p_ATTRTYPEKEY||CHR(10)                                 
		||', p_CONSTANTFLG=>'||p_CONSTANTFLG||CHR(10)                                 
		||', p_REQUIREDFLG=>'||p_REQUIREDFLG||CHR(10)                                 
		||', p_SEQNBR=>'||p_SEQNBR||CHR(10)                                           
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)                                 
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)                                 
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)                               
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	INSERT INTO cf_protocolattrtype(                                               
		ATTRTYPEID                                                                    
		,ATTRTYPEKEY                                                                  
		,CONSTANTFLG                                                                  
		,REQUIREDFLG                                                                  
		,SEQNBR                                                                       
		,RESOURCEKEY                                                                  
		,DESCRIPTION                                                                  
		,PROVIDERFLAG                                                                 
	) VALUES (                                                                     
		CF_PROTOCOLATTRTYPE_SEQ.nextval                                               
		,p_ATTRTYPEKEY                                                                
		,p_CONSTANTFLG                                                                
		,p_REQUIREDFLG                                                                
		,cf_protocolattrtype_sqnbr_seq.NEXTVAL
		,p_RESOURCEKEY                                                                
		,p_DESCRIPTION                                                                
		,p_PROVIDERFLAG                                                               
	);                                                                             
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);
	EXCEPTION                                                                      
		WHEN NO_DATA_FOUND THEN                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt,p_DebugFlg, 0);
			p_returnCd := 3;                                                             
		WHEN DUP_VAL_ON_INDEX then                                                    
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;                                                             
		WHEN OTHERS THEN
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 cf_protocolattrtypeI' || SQLCODE|| ' '||SQLERRM);                                                                    
END CF_PROTOCOLATTRTYPEI;
/


-- CF_PROTOCOLATTRTYPES_AH2.sql

CREATE OR REPLACE PROCEDURE CF_PROTOCOLATTRTYPES_AH2
    (
     p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_Locale    In  varchar2
    , p_ATTRTYPEID IN NUMBER
    , p_providerflag IN NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_PROTOCOLATTRTYPES_AH2
    Create Date:     2003-05-21
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-21 J.Chen        3.5    Returns ProtocolAttrType by input
    2003-05-28 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_PROTOCOLATTRTYPES_AH2';
v_ExceptionFlg    integer := 0;
v_Where           Varchar2(2000);
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_Locale=>'||p_Locale||CHR(10)
        ||', p_ATTRTYPEID=>'||p_ATTRTYPEID||CHR(10)
        ||', p_providerflag=>'||p_providerflag||CHR(10)
        ;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------


BEGIN
   v_Where := ' Where 1 = 1'
            ||FN_BUILD_WHERE('ATTRTYPEID', 'NUMBER', p_ATTRTYPEID)
            ||FN_BUILD_WHERE('providerflag', 'NUMBER', p_providerflag)
            ;

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR ' SELECT
                     ATTRTYPEID
                    ,RESOURCEKEY
                    ,CONSTANTFLG
                    ,REQUIREDFLG
                    ,SEQNBR
                    ,ROWTS
                    ,CREATEDATE
                    ,DESCRIPTION
                FROM
                CF_PROTOCOLATTRTYPE '||v_where;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --  COMMIT;
End CF_PROTOCOLATTRTYPES_AH2;
/


-- CF_PROTOCOLATTRTYPE_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE CF_PROTOCOLATTRTYPE_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_ATTRTYPEID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROTOCOLATTRTYPE_LOC_VWS
	Create Date:     2003-09-26 00:10:47
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
    	2004-03-05 S.Narain      4.2.2  Removed NLS
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROTOCOLATTRTYPE_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ATTRTYPEID=>'||p_ATTRTYPEID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ATTRTYPEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		ATTRTYPEID
		,RTRIM(ATTRTYPEKEY) ATTRTYPEKEY
		,CONSTANTFLG
		,REQUIREDFLG
		,SEQNBR
		,ROWTS
		,CREATEDATE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,DESCRIPTION
	FROM
		CF_PROTOCOLATTRTYPE
	Where
		ATTRTYPEID = p_ATTRTYPEID;

	ELSE
	OPEN RC1 FOR
	SELECT
		ATTRTYPEID
		,RTRIM(ATTRTYPEKEY) ATTRTYPEKEY
		,CONSTANTFLG
		,REQUIREDFLG
		,SEQNBR
		,ROWTS
		,CREATEDATE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,DESCRIPTION
	FROM
		CF_PROTOCOLATTRTYPE;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROTOCOLATTRTYPE_LOC_VWS;
/


-- CF_PROTOCOLATTRU_AH1.sql

CREATE OR REPLACE PROCEDURE CF_PROTOCOLATTRU_AH1
    (
     p_ReturnCd out integer
    ,p_DebugFlg in pls_integer
    ,RC1 out db_ref.RCT
    ,p_PROTOCOLATTRID in NUMBER
    ,p_PROTOCOLDEFID in NUMBER
    ,p_ATTRVALUETYPEID in NUMBER
    ,p_ATTRTYPEID in NUMBER
    ,p_ATTRTYPEVALUE in VARCHAR2
    ,p_ROWTS in NUMBER
    ,p_namespace IN VARCHAR2
) as

/*
    ================================================================================
    Proc Name:       CF_PROTOCOLATTRU_AH1
    Create Date:     2003-05-07 09:16:51
    Author:          SYSTEM

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'CF_PROTOCOLATTRU_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_PROTOCOLATTRID=>'||p_PROTOCOLATTRID||CHR(10)
        ||', p_PROTOCOLDEFID=>'||p_PROTOCOLDEFID||CHR(10)
        ||', p_ATTRVALUETYPEID=>'||p_ATTRVALUETYPEID||CHR(10)
        ||', p_ATTRTYPEID=>'||p_ATTRTYPEID||CHR(10)
        ||', p_ATTRTYPEVALUE=>'||p_ATTRTYPEVALUE||CHR(10)
        ||', p_ROWTS=>'||p_ROWTS||CHR(10)
        ||', p_namespace=>'||p_namespace||CHR(10)
        ;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_RowCnt := 0;
    Update CF_PROTOCOLATTR
    SET
        PROTOCOLDEFID=p_PROTOCOLDEFID
        ,      ATTRVALUETYPEID=p_ATTRVALUETYPEID
        ,      ATTRTYPEID=p_ATTRTYPEID
        ,      ATTRTYPEVALUE=p_ATTRTYPEVALUE
        ,      ROWTS= p_RowTS + 1
        ,      UPDATEDATE= FN_GET_DATE()
        ,      namespace = p_namespace
    Where
        PROTOCOLATTRID = p_PROTOCOLATTRID
        And RowTS = p_RowTS returning RowTS into v_RowTS;
    v_RowCnt := SQL%rowcount;
    if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
    IF RC1%isOpen THEN Close RC1; END IF;
    OPEN RC1 for select v_RowTS RowTS from Dual;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        WHEN TOO_MANY_ROWS Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999, '20100 - CF_PROTOCOLATTRU_AH1: Too Many Rows');
        WHEN ZeroUpdate then
            IF RC1%isOpen THEN Close RC1; END IF;
            OPEN RC1 for select 0 RowTS from Dual;
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROTOCOLATTRU_AH1:'||SQLERRM);
End CF_PROTOCOLATTRU_AH1;
/


-- CF_ProtocolAttrValueTypeS_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_PROTOCOLATTRVALUETYPES_AH1
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
     , p_Locale    In  varchar2
    , p_ATTRVALUETYPEID IN NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_PROTOCOLATTRVALUETYPES
    Create Date:     2003-05-21
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-21 J.Chen        3.5    Returns ProtocolAttrvalueType (XPath/Constant) by input
    2003-05-28 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2004-03-03 S.Priani	    4.2.2   Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_PROTOCOLATTRVALUETYPES';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_Locale => '|| p_Locale ||chr(10)
        ||', p_ATTRVALUETYPEID=>'||p_ATTRVALUETYPEID||CHR(10)
        ;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ATTRVALUETYPEID IS NOT NULL
 THEN
    OPEN RC1 FOR
    SELECT
        ATTRVALUETYPEID
        ,RESOURCEKEY
        ,ATTRVALUETYPEKEY
        ,ROWTS
        ,CREATEDATE
        ,DESCRIPTION
    FROM
        CF_ProtocolAttrValueType
    Where
        ATTRVALUETYPEID = p_ATTRVALUETYPEID
    ;
    ELSE
    OPEN RC1 FOR
    SELECT
        ATTRVALUETYPEID
        ,RESOURCEKEY
        ,ATTRVALUETYPEKEY
        ,ROWTS
        ,CREATEDATE
        ,DESCRIPTION
    FROM
        CF_ProtocolAttrValueType
    ;
    END IF;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --   Commit;

    EXCEPTION
         When OTHERS Then
            RAISE_APPLICATION_ERROR (-20999,'20000-CF_PROTOCOLATTRVALUETYPES_AH1' || SQLCODE|| ' '||SQLERRM);
            Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
End CF_PROTOCOLATTRVALUETYPES_AH1;
/


-- CF_PROTOCOLDEFD.sql

CREATE OR REPLACE Procedure CF_PROTOCOLDEFD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_PROTOCOLDEFID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROTOCOLDEFD
	Create Date:     2003-09-26 08:23:58
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROTOCOLDEFD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROTOCOLDEFID=>'||p_PROTOCOLDEFID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_PROTOCOLDEF
	Where
		PROTOCOLDEFID = p_PROTOCOLDEFID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROTOCOLDEFD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_PROTOCOLDEFD;
/


-- CF_PROTOCOLDEFI.sql

CREATE OR REPLACE Procedure CF_PROTOCOLDEFI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGOBJID in NUMBER
	,p_METHODTYPEID in NUMBER
	,p_IDENTIFIERVALUE in VARCHAR2
	,p_STRUCTURETYPEID in NUMBER
   ,p_srcdoctype IN VARCHAR2
   ,p_srcdocver  IN VARCHAR2
   ,p_protocoldeffamilyid IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROTOCOLDEFI
	Create Date:     2003-09-26 00:01:16
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        NUMBER := 0;

v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROTOCOLDEFI';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		||', p_METHODTYPEID=>'||p_METHODTYPEID||CHR(10)
		||', p_IDENTIFIERVALUE=>'||p_IDENTIFIERVALUE||CHR(10)
		||', p_STRUCTURETYPEID=>'||p_STRUCTURETYPEID||CHR(10)
      ||', p_srcdoctype=>'||p_srcdoctype||CHR(10)
      ||', p_srcdocver=>'||p_srcdocver||CHR(10)
      ||', p_protocoldeffamilyid=>'||p_protocoldeffamilyid||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_PROTOCOLDEF(
		PROTOCOLDEFID
		,ROUTINGOBJID
		,METHODTYPEID
		,IDENTIFIERVALUE
		,STRUCTURETYPEID
      ,srcdoctype
      ,srcdocver
      ,protocoldeffamilyid
	) Values (
		CF_PROTOCOLDEF_SEQ.nextval
		,p_ROUTINGOBJID
		,p_METHODTYPEID
		,p_IDENTIFIERVALUE
		,p_STRUCTURETYPEID
      ,p_srcdoctype
      ,p_srcdocver
      ,p_protocoldeffamilyid
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_PROTOCOLDEF_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;			
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROTOCOLDEFI' || SQLCODE|| ' '||SQLERRM);
END CF_PROTOCOLDEFI;
/


-- CF_PROTOCOLDEFS.sql

CREATE OR REPLACE PROCEDURE CF_PROTOCOLDEFS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROTOCOLDEFID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROTOCOLDEFS
	Create Date:     2003-09-26 00:07:02
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROTOCOLDEFS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROTOCOLDEFID=>'||p_PROTOCOLDEFID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PROTOCOLDEFID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		PROTOCOLDEFID
		,ROUTINGOBJID
		,METHODTYPEID
		,RTRIM(IDENTIFIERVALUE) IDENTIFIERVALUE
		,ROWTS
		,CREATEDATE
		,STRUCTURETYPEID
      ,srcdoctype
      ,srcdocver
      ,protocoldeffamilyid
	FROM
		CF_PROTOCOLDEF
	Where
		PROTOCOLDEFID = p_PROTOCOLDEFID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		PROTOCOLDEFID
		,ROUTINGOBJID
		,METHODTYPEID
		,RTRIM(IDENTIFIERVALUE) IDENTIFIERVALUE
		,ROWTS
		,CREATEDATE
		,STRUCTURETYPEID
      ,srcdoctype
      ,srcdocver
      ,protocoldeffamilyid
	FROM
		CF_PROTOCOLDEF
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROTOCOLDEFS;
/


-- CF_PROTOCOLDEFS_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_ProtocolDefS_AH1
    (
    p_ReturnCd    OUT  INTEGER
    , p_DebugFlg  IN   PLS_INTEGER
    , RC1  OUT  db_ref.RCT
    , RC2  OUT  db_ref.RCT
    , RC3  OUT  db_ref.RCT
    )
AS

/*
    ================================================================================
    Proc Name:       CF_ProtocolDefS_AH1
    Create Date:     2003-04-26 13:33:54
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-26 J.Chen        3.5    Created
    2003-05-03 J.Chen        3.5    Changes made on CF_RoutingObjRef_Lnk
    2003-06-05 J.Chen        3.5    Only returns protocol level routing objects in rc1
    2003-06-17 J.Chen        3.5    Added RTrim on RoutingObjName/CD/ver
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) := 'CF_ProtocolDefS_AH1';
v_ExceptionFlg    INTEGER := 0;
v_ParamsTxt       VARCHAR2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;

OPEN RC1 FOR
    -- Protocol level routing objects
    SELECT    d.RoutingObjId,
              RTRIM(r.RoutingObjName) AS RoutingObjName,
              RTRIM(r.RoutingObjCd) AS RoutingObjCd,
              RTRIM(r.RoutingObjVer) AS RoutingObjVer,
              rt.RoutingObjTypeKey,
              d.srcdoctype,
              d.srcdocver,
              d.protocoldeffamilyid
    FROM      CF_ProtocolDef d
         Inner Join CF_StructureType t ON t.StructureTypeId = d.StructureTypeId
         Inner Join CF_RoutingObject r ON r.RoutingObjId = d.RoutingObjId
         Inner Join CF_RoutingObjType rt ON rt.RoutingObjTypeId = r.RoutingObjTypeId
    WHERE     t.StructureTypeKey = 'XML'
    ;

/*    Open RC1 for
    -- Protocol level routing objects
    Select    d.RoutingObjId,
              r.RoutingObjName,
              r.RoutingObjCd,
              r.RoutingObjVer,
              rt.RoutingObjTypeKey,
              pl.RoutingObjId As RefRoutingObjId
    From      CF_ProtocolDef d
         Inner Join CF_StructureType t On t.StructureTypeId = d.StructureTypeId
         Inner Join CF_RoutingObject r On r.RoutingObjId = d.RoutingObjId
         Inner Join CF_RoutingObjRef_Lnk cl On cl.RoutingObjId = r.RoutingObjId
         Inner Join CF_RoutingObjRef_Lnk pl On pl.RoutingObjRefId = cl.RoutingObjParentRefId
         Inner Join CF_RoutingObjType rt On rt.RoutingObjTypeId = r.RoutingObjTypeId
    Where     t.StructureTypeKey = 'XML'
    Union   -- Process level RoutingObjects
    Select    o.RoutingObjId,
              o.RoutingObjName,
              o.RoutingObjCd,
              o.RoutingObjVer,
              rt.RoutingObjTypeKey,
              pl.RoutingObjId As RefRoutingObjId
    From      CF_RoutingObject o
         Inner Join CF_RoutingObjRef_Lnk cl On cl.RoutingObjId = o.RoutingObjId
         Inner Join CF_RoutingObjRef_Lnk pl On pl.RoutingObjRefId = cl.RoutingObjParentRefId
         Inner Join CF_RoutingObjType rt On rt.RoutingObjTypeId = o.RoutingObjTypeId
    Where     cl.RoutingObjParentRefId In (Select   d.RoutingObjId
                                            From     CF_ProtocolDef d
                                                 Inner Join CF_StructureType t On t.StructureTypeId = d.StructureTypeId
                                                 Inner Join CF_RoutingObject r On r.RoutingObjId = d.RoutingObjId
                                            Where     t.StructureTypeKey = 'XML'
                                           )
    ;
*/

-- Return method type and identifier value
    IF RC2%isOpen THEN Close RC2; END IF;

    OPEN rc2 FOR
    SELECT  d.ProtocolDefId
            ,d.RoutingObjId
            ,t.MethodTypeKey
            ,d.IdentifierValue
            ,s.StructureTypeKey
            ,d.srcdoctype
            ,d.srcdocver
            ,d.protocoldeffamilyid
    FROM    CF_ProtocolDef d
        Inner Join CF_ProtocolDefType t  ON t.MethodTypeId       = d.MethodTypeId
        Inner Join CF_StructureType s    ON s.StructureTypeId    = d.StructureTypeId
    ;


-- Return Attribute type and value
    IF RC3%isOpen THEN Close RC3; END IF;

    OPEN rc3 FOR
    SELECT      d.ProtocolDefId
                ,i.AttrValueTypeKey AS AttrValueType
                ,a.AttrTypeValue
                ,t.AttrTypeKey
                ,d.srcdoctype
                ,d.srcdocver
                ,d.protocoldeffamilyid
    FROM        CF_ProtocolAttr a
        Inner Join CF_ProtocolDef d ON d.ProtocolDefId = a.ProtocolDefId
        Inner Join CF_ProtocolAttrValueType i   ON i.AttrValueTypeId    = a.AttrValueTypeId
        Inner Join CF_ProtocolAttrType t    ON t.AttrTypeId             = a.AttrTypeId
    ;


    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION
         When OTHERS THEN
            RAISE_APPLICATION_ERROR (-20999,'20000-CF_ProtocolDefS_AH1' || SQLCODE|| ' '||SQLERRM);
            ROLLBACK WORK;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            COMMIT;
END CF_ProtocolDefS_AH1;
/


-- CF_ProtocolDefS_AH3.SQL

CREATE OR REPLACE PROCEDURE CF_ProtocolDefS_AH3
    (
      p_ReturnCd   OUT  INTEGER
    , p_DebugFlg  IN   PLS_INTEGER
    , RC1         OUT  db_ref.RCT
    , p_MethodTypeId IN  NUMBER
    , p_StructureTypeId IN NUMBER
    , p_IdentifierValue IN VARCHAR2
    , p_srcdoctype      IN VARCHAR2
    , p_srcdocver       IN VARCHAR2
    , p_protocoldeffamilyid IN NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_ProtocolDefS_AH3
    Create Date:     2003-04-26 13:33:54
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-08 M.Mount       3.5    Created.
    2003-05-09 M.mount       3.5    Added StrucutureTypeId to result set.
    2003-07-08 V.Goel        3.5    Use FN_Build_Where
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) := 'CF_ProtocolDefS_AH3';
v_ExceptionFlg    INTEGER := 0;
v_ParamsTxt       VARCHAR2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||',  p_MethodTypeId=>'|| p_MethodTypeId||CHR(10)
        ||',  p_StructureTypeId=>'|| p_StructureTypeId||CHR(10)
        ||',  p_IdentifierValue=>'|| p_IdentifierValue||CHR(10)
        ||',  p_srcdoctype=>'|| p_srcdoctype||CHR(10)
        ||',  p_srcdocver=>'|| p_srcdocver||CHR(10)
        ||',  p_protocoldeffamilyid=>'|| p_protocoldeffamilyid||CHR(10)
        ;
  v_where            VARCHAR2(1000);
  sqlstmt            VARCHAR2(4000);
  sql_Column         VARCHAR2(2000);
  sql_inner          VARCHAR2(1000); -- includes from clause, inner joins and some outer joins that effect where clause


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    p_ReturnCd := 0;

    v_where := ' Where 1=1'||
    FN_Build_Where('MethodTypeId',    'NUMBER'        , p_MethodTypeId)||
    FN_Build_Where('StructureTypeId',  'NUMBER'        , p_StructureTypeId)||
    FN_Build_Where('IdentifierValue', 'VARCHAR2_LIKE' , p_IdentifierValue)||
    FN_BUILD_WHERE('srcdoctype','VARCHAR2',p_srcdoctype)||
    FN_BUILD_WHERE('srcdocver','VARCHAR2',p_srcdocver)||
    FN_BUILD_WHERE('protocoldeffamilyid','NUMBER',RTRIM(to_char(p_protocoldeffamilyid)));


  sqlstmt := 'select
        PROTOCOLDEFID
        ,ROUTINGOBJID
        ,METHODTYPEID
        ,RTRIM(IDENTIFIERVALUE) IDENTIFIERVALUE
        ,STRUCTURETYPEID
        ,srcdoctype
        ,srcdocver
        ,protocoldeffamilyid
        ,ROWTS
        ,CREATEDATE
    From
        CF_ProtocolDef '||v_where || ' order by METHODTYPEID';

        sql_inner := 'FROM CF_ProtocolDef';

    DB_Print('Sql paging', sqlstmt);

    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for sqlstmt;
      p_ReturnCd := 0;
      DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    Exception
    When NO_DATA_FOUND THEN Null;
    When OTHERS Then
                p_ReturnCd     := 1;
                v_ExceptionFlg := 1;

End CF_ProtocolDefS_AH3;
/


-- CF_PROTOCOLDEFTYPE_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE CF_PROTOCOLDEFTYPE_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_METHODTYPEID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROTOCOLDEFTYPE_LOC_VWS
	Create Date:     2003-09-26 00:10:45
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROTOCOLDEFTYPE_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_METHODTYPEID=>'||p_METHODTYPEID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_METHODTYPEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		METHODTYPEID
		,RTRIM(METHODTYPEKEY) METHODTYPEKEY
		,ROWTS
		,CREATEDATE
		,RTRIM(RESOURCEKEY)
		,DESCRIPTION
	FROM
		CF_PROTOCOLDEFTYPE
	Where
		METHODTYPEID = p_METHODTYPEID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		METHODTYPEID
		,RTRIM(METHODTYPEKEY) METHODTYPEKEY
		,ROWTS
		,CREATEDATE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,DESCRIPTION
	FROM
		CF_PROTOCOLDEFTYPE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROTOCOLDEFTYPE_LOC_VWS;
/


-- CF_PROTOCOLDEFU.sql

CREATE OR REPLACE Procedure CF_PROTOCOLDEFU
	(
	 p_ReturnCd OUT INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 OUT db_ref.RCT
	,p_PROTOCOLDEFID IN NUMBER
	,p_ROUTINGOBJID IN NUMBER
	,p_METHODTYPEID IN NUMBER
	,p_IDENTIFIERVALUE IN VARCHAR2
	,p_ROWTS IN NUMBER
	,p_STRUCTURETYPEID IN NUMBER
   ,p_srcdoctype      IN VARCHAR2
   ,p_srcdocver       IN VARCHAR2
   ,p_protocoldeffamilyid IN NUMBER
) AS

/*
	================================================================================
	Proc Name:       CF_PROTOCOLDEFU
	Create Date:     2003-09-26 08:20:53
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROTOCOLDEFU';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROTOCOLDEFID=>'||p_PROTOCOLDEFID||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		||', p_METHODTYPEID=>'||p_METHODTYPEID||CHR(10)
		||', p_IDENTIFIERVALUE=>'||p_IDENTIFIERVALUE||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_STRUCTURETYPEID=>'||p_STRUCTURETYPEID||CHR(10)
      ||',  p_srcdoctype=>'|| p_srcdoctype||CHR(10)
      ||',  p_srcdocver=>'|| p_srcdocver||CHR(10)
      ||',  p_protocoldeffamilyid=>'|| p_protocoldeffamilyid||CHR(10)
		;

v_RowTS     INTEGER :=0;
zeroUpdate  EXCEPTION;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	UPDATE CF_PROTOCOLDEF
	SET
		ROUTINGOBJID=p_ROUTINGOBJID
		,      METHODTYPEID=p_METHODTYPEID
		,      IDENTIFIERVALUE=p_IDENTIFIERVALUE
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      STRUCTURETYPEID=p_STRUCTURETYPEID
      ,      srcdoctype = p_srcdoctype
      ,      srcdocver = p_srcdocver
      ,      protocoldeffamilyid = p_protocoldeffamilyid
	WHERE
		PROTOCOLDEFID = p_PROTOCOLDEFID
		AND RowTS = p_RowTS returning RowTS INTO v_RowTS;
	v_RowCnt := SQL%rowcount;
	IF (v_RowCnt = 0) THEN RAISE ZeroUpdate ; END IF;
	IF RC1%isOpen THEN CLOSE RC1; END IF;
	OPEN RC1 FOR SELECT v_RowTS RowTS FROM Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_PROTOCOLDEFU: Too Many Rows');
		WHEN ZeroUpdate THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROTOCOLDEFU: No Data Found');
		WHEN OTHERS THEN
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROTOCOLDEFU:'||SQLERRM);
END CF_PROTOCOLDEFU;
/


-- cf_protocoldef_familyD.sql

CREATE OR REPLACE PROCEDURE cf_protocoldef_familyD
                             
	(p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER   
	,p_protocolfamilydefid IN NUMBER                                               
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       CF_PROTOCOLDEF_FAMILYD                                        
	Create Date:     2006-02-21 12:35:55                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2006-02-21 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROTOCOLDEF_FAMILYD';                
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_protocolfamilydefid=>'||p_protocolfamilydefid||CHR(10)                 
		;
                                                                            
zeroDelete  Exception;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	
   --fk_protocoldef_4 will take care of this - on delete cascade
   --delete the formats associated with the family
   --DELETE FROM cf_protocoldef
   --WHERE protocoldeffamilyid = p_protocolfamilydefid
   --;

   --then delete the family itself
	DELETE FROM cf_protocoldef_family                                              
   WHERE protocoldeffamilyid = p_protocolfamilydefid
	;                                                                              
	
   v_RowCnt := SQL%ROWCOUNT;                                                      
	
   IF (v_RowCnt = 0) THEN RAISE ZeroDelete ; END IF;                             
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;                                                               

	EXCEPTION                                                                      
		WHEN ZeroDelete THEN                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20002 cf_protocoldef_familyD: No Data Found');                                                                        
		WHEN OTHERS THEN                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);         
End cf_protocoldef_familyD;
/


-- cf_protocoldef_familyI.sql

CREATE OR REPLACE PROCEDURE CF_PROTOCOLDEF_FAMILYI
    (p_ReturnCd       OUT PLS_INTEGER
    ,p_DebugFlg       IN PLS_INTEGER
   ,RC1              OUT DB_REF.RCT
   ,p_FAMILYNAME     IN VARCHAR2
    ,p_METHODTYPEID   IN NUMBER
   ,p_PROTOCOLID     IN NUMBER
   ,p_IDENT_STRING   IN VARCHAR2
   ,p_OPTIONCD     IN NUMBER
    )
AS                                                                           

/*                                                                             
    ===============================================================================
=                                                                               
    Proc Name:       CF_PROTOCOLDEF_FAMILYI                                        
    Create Date:     2006-02-21 12:35:55                                           
    Author:          SYSTEM 
                                                      
                       ::::::::::::::::::::::::::::::                              
                             M O D I F I C A T I O N                               
                       ::::::::::::::::::::::::::::::
                             
    ---------- ------------- ------ ---------------------------------------------  
    DATE       NAME          Ver    DESCRIPTION                                    
    ---------- ------------- ------ ---------------------------------------------  
    2006-02-21 SYSTEM           3.5 Created
                                       
    ===============================================================================
=                                                                               
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             
v_PK_Value        NUMBER := 0;                                                  

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROTOCOLDEF_FAMILYI';                
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
        ||', p_METHODTYPEID=>'||p_METHODTYPEID||CHR(10)                               
        ||', p_FAMILYNAME=>'||p_FAMILYNAME||CHR(10)                                   
        ||', p_IDENT_STRING=>'||p_IDENT_STRING||CHR(10)                               
        ||', p_PROTOCOLID=>'||p_PROTOCOLID||CHR(10)
        ||', p_OPTIONCD=>'||p_OPTIONCD||CHR(10)
        ;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
    INSERT INTO cf_protocoldef_family(                                             
        PROTOCOLDEFFAMILYID                                                           
        ,METHODTYPEID                                                                 
        ,FAMILYNAME                                                                   
        ,IDENT_STRING                                                                 
        ,PROTOCOLID 
        ,OPTIONCD
    ) VALUES (                                                                     
        CF_PROTOCOLDEF_FAMILY_SEQ.nextval                                             
        ,p_METHODTYPEID                                                               
        ,p_FAMILYNAME                                                                 
        ,p_IDENT_STRING                                                               
        ,p_PROTOCOLID 
        ,p_OPTIONCD
    );                                                                             

    p_ReturnCd := 0;                                                               
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);

   IF RC1%ISOPEN THEN CLOSE RC1; END IF;
   OPEN RC1 FOR 
    SELECT CF_PROTOCOLDEF_FAMILY_SEQ.CURRVAL ID, 1 rowts FROM dual
   ;

    EXCEPTION                                                                      
        WHEN NO_DATA_FOUND THEN                                                       
            v_ExceptionFlg := 1;                                                         
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
            p_returnCd := 3;                                                             
        WHEN DUP_VAL_ON_INDEX THEN                                                    
            v_ExceptionFlg := 1;                                                         
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            p_returnCd := 2;                                                             
        WHEN OTHERS THEN                                                              
            p_ReturnCd := 1;                                                             
            v_ExceptionFlg := 1;                                                         
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR (-20999,'20002 cf_protocoldef_familyI' || SQLCODE|| ''||SQLERRM);                                                                    
END CF_PROTOCOLDEF_FAMILYI;
/


-- cf_protocoldef_familyS.sql

CREATE OR REPLACE PROCEDURE cf_protocoldef_familyS                              
    (
      p_ReturnCd  OUT PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1         OUT DB_REF.RCT                                                                      
    , p_PROTOCOLDEFFAMILYID IN NUMBER                                              
    )                                                                              
AS                                                                              

/*                                                                             
    ===============================================================================
=                                                                               
    Proc Name:       CF_PROTOCOLDEF_FAMILYS                                        
    Create Date:     2006-02-21 12:35:55                                           
    Author:          SYSTEM 
                                                      
                       ::::::::::::::::::::::::::::::                              
                             M O D I F I C A T I O N                               
                       ::::::::::::::::::::::::::::::
                             
    ---------- ------------- ------ ---------------------------------------------  
    DATE       NAME          Ver    DESCRIPTION                                    
    ---------- ------------- ------ ---------------------------------------------  
    2006-02-21 SYSTEM           3.5 Created
                                       
    ===============================================================================
=                                                                               
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROTOCOLDEF_FAMILYS';                
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
        ||', p_PROTOCOLDEFFAMILYID=>'||p_PROTOCOLDEFFAMILYID||CHR(10)                 
        ;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
IF p_PROTOCOLDEFFAMILYID IS NOT NULL 
 THEN                                     
    OPEN RC1 FOR 
    SELECT                                                          
        PROTOCOLDEFFAMILYID                                                           
        ,METHODTYPEID                                                                 
        ,RTRIM(FAMILYNAME) FAMILYNAME                                                 
        ,RTRIM(IDENT_STRING) IDENT_STRING                                             
        ,PROTOCOLID
        ,OPTIONCD
        ,ROWTS                                                                        
        ,CREATEDATE                                                                   
    FROM 
        cf_protocoldef_family                                                  
    WHERE 
        PROTOCOLDEFFAMILYID = p_PROTOCOLDEFFAMILYID                           
    ;                                                                              
    ELSE                                                                           
    OPEN RC1 FOR 
    SELECT                                                          
        PROTOCOLDEFFAMILYID                                                           
        ,METHODTYPEID                                                                 
        ,RTRIM(FAMILYNAME) FAMILYNAME                                                 
        ,RTRIM(IDENT_STRING) IDENT_STRING                                             
        ,PROTOCOLID
        ,OPTIONCD
        ,ROWTS                                                                        
        ,CREATEDATE                                                                   
    FROM 
        cf_protocoldef_family                                                  
    ;                                                                              
    END IF;                                                                        
    p_ReturnCd := 0;
    
   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END cf_protocoldef_familyS;
/

-- cf_protocoldef_familyS_ah1.sql

CREATE OR REPLACE PROCEDURE cf_protocoldef_familys_ah1
    (
    p_ReturnCd    OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
   , RC2  OUT  DB_REF.RCT
   , p_sortkey   IN VARCHAR2
   , p_PageSizeNbr IN NUMBER
   , p_PageNbr   IN NUMBER
   , p_pattern   IN VARCHAR
   , p_methodtype IN NUMBER
   , p_protocolid IN NUMBER
   , p_optioncd IN NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       cf_protocoldef_familys_ah1
    Create Date:     2004-08-09 13:21:35
    Author:          SYSTEM 

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-08-09 SYSTEM           3.5 Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'cf_protocoldef_familys_ah1';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_pattern=>'||p_pattern||CHR(10)
      ||', p_methodtype=>'||p_methodtype||CHR(10)
      ||', p_sortkey=>'||p_sortkey||CHR(10)
      ||', p_PageSizeNbr=>'||p_PageSizeNbr||CHR(10)
      ||', p_PageNbr=>'||p_PageNbr||CHR(10)
        ;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

v_where    VARCHAR2(1000);
v_sqlstmt  VARCHAR2(2000);
v_sqlstmtT VARCHAR2(4000);
v_sqlstmtF VARCHAR2(4000);
--needed for paging
v_sortKey        VARCHAR2(4000);
sql_inner        VARCHAR2(4000);
v_count          INTEGER;
--

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;    
   
   IF p_sortkey IS NULL THEN
      v_sortkey := 'familyname';
   ELSE 
      v_sortkey := p_sortkey;
   END IF;

   v_where := ' WHERE 1=1 '
               ||' '||FN_BUILD_WHERE('familyname', 'VARCHAR2_LIKE', TO_CHAR(p_pattern))
               ||' '||FN_BUILD_WHERE('methodtypeid', 'NUMBER', TO_CHAR(p_methodtype))
               ||' '||FN_BUILD_WHERE('protocolid', 'NUMBER', TO_CHAR(p_protocolid))
               ||' '||FN_BUILD_WHERE('optioncd', 'NUMBER', TO_CHAR(p_optioncd));

   v_sqlstmt := 'SELECT ' ||
      ' PROTOCOLDEFFAMILYID ' ||
      ' , METHODTYPEID '||
        ' , FAMILYNAME' ||
      ' , ident_string' ||
      ' , protocolid' ||      
      ' , optioncd' ||      
      ' , ROWTS' ||      
        ' , CREATEDATE'   ||
      ' , UPDATEDATE'   ||
      ' FROM ' ||
        ' cf_protocoldef_family ' ||
      v_where||
       ' ORDER BY '||v_sortkey;
   --paging stuff
   
   v_sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, v_sqlstmt);
   sql_inner := 'FROM cf_protocoldef_family';
   --

   IF RC1%isOpen THEN CLOSE RC1; END IF;
   OPEN RC1 FOR v_sqlstmt;

   --more paging
   IF RC2%ISOPEN THEN CLOSE RC2; END IF;

   IF NVL(P_PAGENBR,0) IN (0,1) THEN
       Execute IMMEDIATE 'SELECT COUNT(*)  '||sql_inner ||v_where INTO v_count;
       OPEN RC2 FOR SELECT v_count TotalRowsNbr FROM dual;
   END IF;
   --
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End cf_protocoldef_familys_ah1;
/


-- cf_protocoldef_familyU.sql

CREATE OR REPLACE PROCEDURE cf_protocoldef_familyU
    (
     p_ReturnCd OUT INTEGER
    ,p_DebugFlg IN PLS_INTEGER
    -- ,RC1        OUT db_ref.RCT
    ,p_PROTOCOLDEFFAMILYID IN NUMBER
    ,p_METHODTYPEID        IN NUMBER
    ,p_PROTOCOLID          IN NUMBER
    ,p_IDENT_STRING        IN VARCHAR2
    ,p_OPTIONCD            IN NUMBER
    ,p_FAMILYNAME          IN VARCHAR2
    ,p_ROWTS               IN NUMBER
) AS

/*                                                                             
    ===============================================================================
=                                                                               
    Proc Name:       CF_PROTOCOLDEF_FAMILYU                                        
    Create Date:     2006-02-21 12:35:55                                           
    Author:          SYSTEM 
                                                      
                       ::::::::::::::::::::::::::::::                              
                             M O D I F I C A T I O N                               
                       ::::::::::::::::::::::::::::::
                             
    ---------- ------------- ------ ---------------------------------------------  
    DATE       NAME          Ver    DESCRIPTION                                    
    ---------- ------------- ------ ---------------------------------------------  
    2006-02-21 SYSTEM           3.5 Created
                                       
    ===============================================================================
=                                                                               
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_PROTOCOLDEF_FAMILYU';                
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
        ||', p_PROTOCOLDEFFAMILYID=>'||p_PROTOCOLDEFFAMILYID||CHR(10)                 
        ||', p_METHODTYPEID=>'||p_METHODTYPEID||CHR(10)                               
        ||', p_FAMILYNAME=>'||p_FAMILYNAME||CHR(10)                                   
        ||', p_IDENT_STRING=>'||p_IDENT_STRING||CHR(10)                               
        ||', p_PROTOCOLID=>'||p_PROTOCOLID||CHR(10)                                   
        ||', p_OPTIONCD=>'||p_OPTIONCD||CHR(10)                                   
        ||', p_ROWTS=>'||p_ROWTS||CHR(10)                                             
        ;
                                                                            
v_RowTS     INTEGER :=0;                                                       
zeroUpdate  EXCEPTION;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
    v_RowCnt := 0;                                                                 
    UPDATE cf_protocoldef_family
    SET                                              
        METHODTYPEID=p_METHODTYPEID                                                   
        ,      FAMILYNAME=p_FAMILYNAME                                                
        ,      IDENT_STRING=p_IDENT_STRING                                            
        ,      PROTOCOLID=p_PROTOCOLID                                                
        ,      OPTIONCD=p_OPTIONCD                                                
        ,      ROWTS= p_RowTS + 1                                                     
        ,      UPDATEDATE= v_BeginTS                                                  
    WHERE
        PROTOCOLDEFFAMILYID = p_PROTOCOLDEFFAMILYID                           
        AND RowTS = p_RowTS returning RowTS INTO v_RowTS;
       
   v_RowCnt := SQL%rowcount;                                                      
    
   IF (v_RowCnt = 0) THEN RAISE ZeroUpdate ; END IF;                             

    -- IF RC1%isOpen THEN CLOSE RC1; END IF;                                          
    -- OPEN RC1 for select v_RowTS RowTS from Dual;                                   
    
   p_ReturnCd := 0;                                                               
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION                                                                      
        WHEN TOO_MANY_ROWS THEN
            v_ExceptionFlg := 1;                                                         
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999, '20100 - cf_protocoldef_familyU: Too Many Rows');
        WHEN ZeroUpdate THEN                                                          
            v_ExceptionFlg := 1;                                                         
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt,p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - cf_protocoldef_familyU: No Data Found');
        WHEN OTHERS then                                                              
            p_ReturnCd := 1;                                                             
            v_ExceptionFlg := 1;                                                         
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR(-20999,'20100 - cf_protocoldef_familyU:'||SQLERRM);  
END CF_PROTOCOLDEF_FAMILYU;
/


-- CF_PROXY_GATEWAY_LNKD.sql

Create or Replace Procedure CF_PROXY_GATEWAY_LNKD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_GWYPROXYLNKID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROXY_GATEWAY_LNKD
	Create Date:     2005-03-10 16:40:12
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROXY_GATEWAY_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GWYPROXYLNKID=>'||p_GWYPROXYLNKID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_PROXY_GATEWAY_LNK
	Where
		GWYPROXYLNKID = p_GWYPROXYLNKID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROXY_GATEWAY_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_PROXY_GATEWAY_LNKD;
/


-- CF_PROXY_GATEWAY_LNKD_AH1.sql

CREATE OR REPLACE PROCEDURE CF_PROXY_GATEWAY_LNKD_AH1 (
   p_returncd    OUT      PLS_INTEGER,
   p_debugflg    IN       PLS_INTEGER,
   p_gatewayid   IN       NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       CF_PROXY_GATEWAY_LNKD_AH1
    Create Date:     2007-08-20 16:40:12
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-20 BHARANI J         1  Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

   v_begints        TIMESTAMP       DEFAULT fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   DEFAULT 'CF_PROXY_GATEWAY_LNKD_AH1';
   v_exceptionflg   INTEGER         DEFAULT 0;
   v_paramstxt      VARCHAR2 (4000)
      DEFAULT    'p_DebugFlg=>'
              || p_debugflg
              || CHR (10)
              || ', p_Gatewayid=>'
              || p_gatewayid
              || CHR (10);

BEGIN

   DELETE FROM cf_proxy_gateway_lnk
         WHERE gatewayid = p_gatewayid;       
          

   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
   
    p_returncd := 0; 
   
EXCEPTION
   WHEN OTHERS
   THEN
      p_returncd := 1;
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            SQLERRM || CHR (10) || v_paramstxt,
                            p_debugflg,
                            1
                           );
      raise_application_error (-20999, '20000 ' || SQLCODE || ' ' || SQLERRM);
      
END cf_proxy_gateway_lnkd_ah1;
/

-- CF_PROXY_GATEWAY_LNKI.sql

Create or Replace Procedure CF_PROXY_GATEWAY_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_GATEWAYID in NUMBER
	,p_PROXYID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROXY_GATEWAY_LNKI
	Create Date:     2005-03-10 16:40:13
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROXY_GATEWAY_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_PROXY_GATEWAY_LNK(
		GWYPROXYLNKID
		,GATEWAYID
		,PROXYID
	) Values (
		CF_PROXY_GATEWAY_LNK_SEQ.nextval
		,p_GATEWAYID
		,p_PROXYID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_PROXY_GATEWAY_LNK_SEQ.currval GWYPROXYLNKID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_PROXY_GATEWAY_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_PROXY_GATEWAY_LNKI;
/


-- CF_PROXY_GATEWAY_LNKS.sql

CREATE OR REPLACE PROCEDURE CF_PROXY_GATEWAY_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_GATEWAYID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROXY_GATEWAY_LNKS
	Create Date:     2005-03-10 16:40:15
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROXY_GATEWAY_LNKS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
	SELECT
		cfgl.GWYPROXYLNKID
		,pr.PROXYID
		,pr.TRANSPORTID
		,RTRIM(pr.PROXYNAME) PROXYNAME
		,RTRIM(pr.DESCRIPTION) DESCRIPTION
		,pr.DEFAULTFLG
		,cfgl.ROWTS
		,pr.CREATEDATE
		,pr.UPDATEDATE
	FROM
		PR_PROXY   pr
		INNER JOIN CF_PROXY_GATEWAY_LNK cfgl ON pr.PROXYID = cfgl.PROXYID
	Where
		GATEWAYID = p_GATEWAYID
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROXY_GATEWAY_LNKS;
/


-- CF_PROXY_GATEWAY_LNKS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_PROXY_GATEWAY_LNKS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROXYID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_PROXY_GATEWAY_LNKS
	Create Date:     2005-03-10 16:40:15
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROXY_GATEWAY_LNKS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
	SELECT
		cfgl.GWYPROXYLNKID
		,pr.PROXYID
		,pr.TRANSPORTID
		,cfgl.GATEWAYID
		,RTRIM(pr.PROXYNAME) PROXYNAME
		,RTRIM(pr.DESCRIPTION) DESCRIPTION
		,pr.DEFAULTFLG
		,pr.ROWTS
		,pr.CREATEDATE
		,pr.UPDATEDATE
		,pg.gatewayname
		,prt.partnername
	FROM
		PR_PROXY   pr
		INNER JOIN CF_PROXY_GATEWAY_LNK cfgl ON pr.PROXYID = cfgl.PROXYID
		INNER JOIN PR_GATEWAY pg on pg.gatewayid = cfgl.gatewayid
		INNER JOIN pr_partner prt on prt.partnerid = pg.partnerid
	Where
		pr.PROXYID = p_PROXYID
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_PROXY_GATEWAY_LNKS_AH1;
/

-- CF_PROXY_GATEWAY_LNKU.sql

Create or Replace Procedure CF_PROXY_GATEWAY_LNKU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_GWYPROXYLNKID in NUMBER
	,p_GATEWAYID in NUMBER
	,p_PROXYID in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_PROXY_GATEWAY_LNKU
	Create Date:     2005-03-10 16:40:16
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_PROXY_GATEWAY_LNKU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GWYPROXYLNKID=>'||p_GWYPROXYLNKID||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_PROXY_GATEWAY_LNK
	SET
		GATEWAYID=p_GATEWAYID
		,      PROXYID=p_PROXYID
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where
		GWYPROXYLNKID = p_GWYPROXYLNKID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_PROXY_GATEWAY_LNKU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROXY_GATEWAY_LNKU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_PROXY_GATEWAY_LNKU:'||SQLERRM);
End CF_PROXY_GATEWAY_LNKU;
/


-- CF_RCVHANDLERATTR_LNKD.sql

CREATE OR REPLACE Procedure CF_RCVHANDLERATTR_LNKD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_RCVCONFIGID in NUMBER
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_HANDLERATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RCVHANDLERATTR_LNKD
	Create Date:     2004-03-14 17:20:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RCVHANDLERATTR_LNKD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   
   v_where := ' WHERE 1=1 ' ;
   v_where := v_where || FN_BUILD_WHERE('RCVCONFIGID', 'NUMBER', p_RCVCONFIGID) ;
	v_where := v_where || FN_BUILD_WHERE('CFGPOINTID', 'VARCHAR2',p_CFGPOINTID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;
	
   v_sqlstmt := 'Delete From CF_RCVHANDLERATTR_LNK ' || v_where ;
	
	execute immediate v_sqlstmt ;

	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_RCVHANDLERATTR_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_RCVHANDLERATTR_LNKD;
/


-- CF_RCVHANDLERATTR_LNKI.sql

CREATE OR REPLACE Procedure CF_RCVHANDLERATTR_LNKI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT 
	,p_RCVCONFIGID in NUMBER
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_HANDLERATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_RCVHANDLERATTR_LNKI
	Create Date:     2004-03-14 17:22:19
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RCVHANDLERATTR_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_RCVHANDLERATTR_LNK(
		RCVCONFIGID
		,HANDLERID
		,CFGPOINTID
		,HANDLERATTRID
		,ATTRVALUE
	) Values (
		p_RCVCONFIGID
		,p_HANDLERID
		,p_CFGPOINTID
		,p_HANDLERATTRID
		,p_ATTRVALUE
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_RCVHANDLERATTR_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_RCVHANDLERATTR_LNKI;
/


-- CF_RCVHANDLERATTR_LNKS.sql

CREATE OR REPLACE PROCEDURE CF_RCVHANDLERATTR_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_RCVCONFIGID IN NUMBER
	, p_CFGPOINTID IN VARCHAR2
	, p_HANDLERID IN NUMBER
	, p_HANDLERATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RCVHANDLERATTR_LNKS
	Create Date:     2004-03-14 17:21:53
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created
   2004-05-5  PAD           4.2.2  Added join on handlerid, returning invalid data
	2004-05-24 PAD           4.2.2  Added v_where_inner logic
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
   
   **NOTE - if no handler or rcvconfig or config is given ALL handler default
            handler attributes will be returned
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RCVHANDLERATTR_LNKS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;        
v_where_inner     varchar2(4000) ;        
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' WHERE 1=1 ' ;
    
    if p_HANDLERATTRID is not null then
       v_where := v_where || FN_BUILD_WHERE('RA.HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where := v_where || FN_BUILD_WHERE('RA.HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_sqlstmt :=
       'SELECT '
	   ||'RCVCONFIGID'
	   ||',CFGPOINTID'
	   ||',RA.HANDLERID'
	   ||',RA.HANDLERATTRID'
       ||',RTRIM(ATTRNAME) ATTRNAME'
       ||',RTRIM(ATTRVALUE) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',RA.ROWTS'
       ||',RA.CREATEDATE '
       ||'FROM '
       ||'CF_RCVHANDLERATTR_LNK RA INNER JOIN PR_HANDLERATTR HA '
       ||'ON RA.HANDLERATTRID = HA.HANDLERATTRID '
       ||'AND RA.HANDLERID = HA.HANDLERID '
       || v_where ;
    ELSE
       v_where_inner := ' WHERE 1=1 ' ;
       v_where_inner := v_where_inner || FN_BUILD_WHERE('RCVCONFIGID', 'NUMBER', p_RCVCONFIGID) ;
	    v_where_inner := v_where_inner || FN_BUILD_WHERE('CFGPOINTID', 'VARCHAR2',p_CFGPOINTID) ;  
       v_where_inner := v_where_inner || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where_inner := v_where_inner || FN_BUILD_WHERE('HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_where := v_where || FN_BUILD_WHERE('HA.HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where := v_where || FN_BUILD_WHERE('HA.HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_sqlstmt :=
       'SELECT '
       ||'rcvconfigid'
   	 ||',CFGPOINTID'
	    ||',NVL(RA1.HANDLERID, ha.HANDLERID) HANDLERID '
       ||',NVL(ra1.HANDLERATTRID, ha.HANDLERATTRID) HANDLERATTRID '
       ||',RTRIM(attrname) ATTRNAME'
       ||',RTRIM(NVL(attrvalue, defaultattrvalue)) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',ra1.ROWTS'
       ||',ra1.CREATEDATE '
       ||'from '
       ||'(select rcvconfigid, cfgpointid, handlerid, handlerattrid, attrvalue, rowts, createdate '
       ||' from cf_rcvhandlerattr_lnk RA '||v_where_inner||') RA1 '
       ||'right outer join pr_handlerattr HA '
       ||'on RA1.handlerattrid = HA.handlerattrid '
       ||v_where
       ;
    end if ;     
	
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_RCVHANDLERATTR_LNKS;
/


-- CF_RCVHANDLERATTR_LNKU.sql

CREATE OR REPLACE Procedure CF_RCVHANDLERATTR_LNKU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_RCVCONFIGID in NUMBER
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_HANDLERATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_RCVHANDLERATTR_LNKU
	Create Date:     2004-03-14 17:21:33
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RCVHANDLERATTR_LNKU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_RCVHANDLERATTR_LNK
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      ATTRVALUE=p_ATTRVALUE
	Where 
		RCVCONFIGID = p_RCVCONFIGID
		AND CFGPOINTID = p_CFGPOINTID
		AND HANDLERID = p_HANDLERID
		AND HANDLERATTRID = p_HANDLERATTRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_RCVHANDLERATTR_LNKU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_RCVHANDLERATTR_LNKU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_RCVHANDLERATTR_LNKU:'||SQLERRM);
End CF_RCVHANDLERATTR_LNKU;
/


-- CF_RCVHANDLER_XREFD.sql

CREATE OR REPLACE Procedure CF_RCVHANDLER_XREFD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_RCVCONFIGID in NUMBER
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RCVHANDLER_XREFD
	Create Date:     2004-03-14 15:35:08
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75, 5724-E87, 5724-L68, 5724-L69.
-- * (C) Copyright IBM Corp. 2001, 2007 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RCVHANDLER_XREFD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   
   v_where := ' WHERE 1=1 ' ;
   v_where := v_where || FN_BUILD_WHERE('RCVCONFIGID', 'NUMBER', p_RCVCONFIGID) ;
	v_where := v_where || FN_BUILD_WHERE('CFGPOINTID', 'VARCHAR2',p_CFGPOINTID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	
   v_sqlstmt := 'Delete From CF_RCVHANDLER_XREF ' || v_where ;
	
	execute immediate v_sqlstmt ;
		
	v_RowCnt := SQL%ROWCOUNT;
	--if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		--When ZeroDelete Then
		--	--v_ExceptionFlg := 1;
		--	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
		--	--RAISE_APPLICATION_ERROR (-20999,'20002 CF_RCVHANDLER_XREFD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_RCVHANDLER_XREFD;
/


-- CF_RCVHANDLER_XREFI.sql

CREATE OR REPLACE Procedure CF_RCVHANDLER_XREFI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT   
	,p_RCVCONFIGID in NUMBER
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	,p_ORDERING in NUMBER

	)
AS

/*
	================================================================================
	Proc Name:       CF_RCVHANDLER_XREFI
	Create Date:     2004-03-14 15:35:45
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RCVHANDLER_XREFI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_RCVHANDLER_XREF(
		RCVCONFIGID
		,HANDLERID
		,ORDERING
		,CFGPOINTID
	) Values (
		p_RCVCONFIGID
		,p_HANDLERID
		,p_ORDERING
		,p_CFGPOINTID
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_RCVHANDLER_XREFI' || SQLCODE|| ' '||SQLERRM);
END CF_RCVHANDLER_XREFI;
/


-- CF_RCVHANDLER_XREFS.sql

CREATE OR REPLACE PROCEDURE CF_RCVHANDLER_XREFS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_RCVCONFIGID IN NUMBER
	, p_CFGPOINTID IN VARCHAR2
	, p_HANDLERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RCVHANDLER_XREFS
	Create Date:     2004-03-14 15:36:13
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RCVHANDLER_XREFS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;           
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('RCVCONFIGID', 'NUMBER', p_RCVCONFIGID) ||
       FN_Build_Where('CFGPOINTID', 'VARCHAR2', p_CFGPOINTID) ||
       FN_Build_Where('rx.HANDLERID', 'NUMBER', p_HANDLERID);
	   
	v_sqlstmt := 
       'SELECT '
        ||'RCVCONFIGID'
		||',RTRIM(CFGPOINTID) CFGPOINTID'
		||',rx.HANDLERID'
		||',ORDERING'
		||',PROVIDERFLG'
		||',RESOURCEKEY'
		||',DESCRIPTION'
		||',CLASSNAME'
		||',rx.ROWTS'
		||',rx.CREATEDATE '
		||'FROM '
		||'CF_RCVHANDLER_XREF rx INNER JOIN PR_HANDLER h ' 
		||'ON rx.HANDLERID = h.HANDLERID '
		||v_where 
      ||' ORDER BY ordering '
		;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_RCVHANDLER_XREFS;
/


-- CF_RNPIPATTR_GRPD.sql

-- > EXECUTE DB_GENPROC.GENPROC_DELETE('CF_RNPIPATTR_GRP');
CREATE OR REPLACE PROCEDURE CF_RNPIPATTR_GRPD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_RNPIPATTR_GRPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RNPIPATTR_GRPD
	Create Date:     2006-05-09 13:43:16
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_RNPIPATTR_GRPD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RNPIPATTR_GRPID=>'||p_RNPIPATTR_GRPID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_RNPIPATTR_GRP
	Where 
		RNPIPATTR_GRPID = p_RNPIPATTR_GRPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_RNPIPATTR_GRPD;
/
-- > SPOOL OFF


-- CF_RNPIPATTR_GRPI.sql

-- > EXECUTE DB_GENPROC.GENPROC_INSERT('CF_RNPIPATTR_GRP', TRUE, TRUE );
CREATE OR REPLACE PROCEDURE CF_RNPIPATTR_GRPI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_RNPIPATTR_GRPKEY in VARCHAR2
	,p_ORDERING in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_RNPIPATTR_GRPI
	Create Date:     2006-05-09 13:43:16
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_RNPIPATTR_GRPI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RNPIPATTR_GRPKEY=>'||p_RNPIPATTR_GRPKEY||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_RNPIPATTR_GRP(
		RNPIPATTR_GRPID
		,RNPIPATTR_GRPKEY
		,ORDERING
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		CF_RNPIPATTR_GRP_SEQ.nextval
		,p_RNPIPATTR_GRPKEY
		,p_ORDERING
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_RNPIPATTR_GRP_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END CF_RNPIPATTR_GRPI;
/
-- > SPOOL OFF


-- CF_RNPIPATTR_GRPS.sql

CREATE OR REPLACE PROCEDURE CF_RNPIPATTR_GRPS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_RNPIPATTR_GRPID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RNPIPATTR_GRPS
	Create Date:     2006-05-09 13:43:16
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_RNPIPATTR_GRPS';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RNPIPATTR_GRPID=>'||p_RNPIPATTR_GRPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_RNPIPATTR_GRPID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		RNPIPATTR_GRPID
		,RTRIM(RNPIPATTR_GRPKEY) RNPIPATTR_GRPKEY
		,ORDERING
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,ROWTS
		,CREATEDATE
	FROM 
		CF_RNPIPATTR_GRP
	Where 
		RNPIPATTR_GRPID = p_RNPIPATTR_GRPID
	ORDER BY Ordering
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		RNPIPATTR_GRPID
		,RTRIM(RNPIPATTR_GRPKEY) RNPIPATTR_GRPKEY
		,ORDERING
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,ROWTS
		,CREATEDATE
	FROM 
		CF_RNPIPATTR_GRP
	ORDER BY Ordering
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_RNPIPATTR_GRPS;
/
-- > SPOOL OFF


-- CF_RNPIPATTR_GRPU.sql

-- > EXECUTE DB_GENPROC.GENPROC_UPDATE('CF_RNPIPATTR_GRP');
CREATE OR REPLACE PROCEDURE CF_RNPIPATTR_GRPU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_RNPIPATTR_GRPID in NUMBER
	,p_RNPIPATTR_GRPKEY in VARCHAR2
	,p_ORDERING in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RNPIPATTR_GRPU
	Create Date:     2006-05-09 13:43:17
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_RNPIPATTR_GRPU';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RNPIPATTR_GRPID=>'||p_RNPIPATTR_GRPID||CHR(10)
		||', p_RNPIPATTR_GRPKEY=>'||p_RNPIPATTR_GRPKEY||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_RNPIPATTR_GRP
	SET
		RNPIPATTR_GRPKEY=p_RNPIPATTR_GRPKEY
		,      ORDERING=p_ORDERING
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		RNPIPATTR_GRPID = p_RNPIPATTR_GRPID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End CF_RNPIPATTR_GRPU;
/
-- > SPOOL OFF


-- CF_RNPIPMETADEFS.sql

CREATE OR REPLACE PROCEDURE CF_RNPIPMETADEFS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_METADEFID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RNPIPMETADEFS
	Create Date:     2003-09-26 00:07:00
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RNPIPMETADEFS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_METADEFID=>'||p_METADEFID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_METADEFID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		METADEFID
		,RTRIM(METADEFKEY) METADEFKEY
		,ROWTS
		,CREATEDATE
	FROM
		CF_RNPIPMETADEF
	Where
		METADEFID = p_METADEFID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		METADEFID
		,RTRIM(METADEFKEY) METADEFKEY
		,ROWTS
		,CREATEDATE
	FROM
		CF_RNPIPMETADEF
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_RNPIPMETADEFS;
/


-- CF_RNPIPMETADEF_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE CF_RNPIPMETADEF_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_METADEFID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RNPIPMETADEF_LOC_VWS
	Create Date:     2003-09-26 00:10:43
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-03-03 S.Priani	 4.2.2	Removed NLS
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RNPIPMETADEF_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_METADEFID=>'||p_METADEFID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_METADEFID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		METADEFID
		,RTRIM(METADEFKEY) METADEFKEY
		,ROWTS
		,CREATEDATE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RNPIPATTR_GRPID
	FROM
		CF_RNPIPMETADEF
	Where
		METADEFID = p_METADEFID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		METADEFID
		,RTRIM(METADEFKEY) METADEFKEY
		,ROWTS
		,CREATEDATE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RNPIPATTR_GRPID
	FROM
		CF_RNPIPMETADEF
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End CF_RNPIPMETADEF_LOC_VWS;
/


-- CF_RNPIPMETAVALUED.sql

CREATE OR REPLACE Procedure CF_RNPIPMETAVALUED

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_METAVALUEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RNPIPMETAVALUED
	Create Date:     2003-09-26 08:23:59
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RNPIPMETAVALUED';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_METAVALUEID=>'||p_METAVALUEID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_RNPIPMETAVALUE
	Where
		METAVALUEID = p_METAVALUEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_RNPIPMETAVALUED: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_RNPIPMETAVALUED;
/


-- CF_RNPIPMETAVALUEI.sql

CREATE OR REPLACE Procedure CF_RNPIPMETAVALUEI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROCESSOBJID in NUMBER
	,p_ACTIVITYOBJID in NUMBER
	,p_ACTIONOBJID in NUMBER
	,p_METADEFID in NUMBER
	,p_METADEFVALUE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_RNPIPMETAVALUEI
	Create Date:     2003-09-26 00:01:17
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RNPIPMETAVALUEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSOBJID=>'||p_PROCESSOBJID||CHR(10)
		||', p_ACTIVITYOBJID=>'||p_ACTIVITYOBJID||CHR(10)
		||', p_ACTIONOBJID=>'||p_ACTIONOBJID||CHR(10)
		||', p_METADEFID=>'||p_METADEFID||CHR(10)
		||', p_METADEFVALUE=>'||p_METADEFVALUE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_RNPIPMETAVALUE(
		METAVALUEID
		,PROCESSOBJID
		,ACTIVITYOBJID
		,ACTIONOBJID
		,METADEFID
		,METADEFVALUE
	) Values (
		CF_RNPIPMETAVALUE_SEQ.nextval
		,p_PROCESSOBJID
		,p_ACTIVITYOBJID
		,p_ACTIONOBJID
		,p_METADEFID
		,p_METADEFVALUE
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_RNPIPMETAVALUE_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;			
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_RNPIPMETAVALUEI' || SQLCODE|| ' '||SQLERRM);
END CF_RNPIPMETAVALUEI;
/


-- CF_RNPIPMETAVALUES.sql

CREATE OR REPLACE PROCEDURE CF_RNPIPMETAVALUES
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_METAVALUEID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_RNPIPMETAVALUES
	Create Date:     2003-09-26 00:06:58
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RNPIPMETAVALUES';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_METAVALUEID=>'||p_METAVALUEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_METAVALUEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		METAVALUEID
		,PROCESSOBJID
		,ACTIVITYOBJID
		,ACTIONOBJID
		,METADEFID
		,RTRIM(METADEFVALUE) METADEFVALUE
		,ROWTS
		,CREATEDATE
	FROM
		CF_RNPIPMETAVALUE
	Where
		METAVALUEID = p_METAVALUEID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		METAVALUEID
		,PROCESSOBJID
		,ACTIVITYOBJID
		,ACTIONOBJID
		,METADEFID
		,RTRIM(METADEFVALUE) METADEFVALUE
		,ROWTS
		,CREATEDATE
	FROM
		CF_RNPIPMETAVALUE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_RNPIPMETAVALUES;
/


-- CF_RNPIPMETAVALUES_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_RNPipMetaValueS_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , RC2  out  db_ref.RCT
    )
AS
/*
    ================================================================================
    Proc Name:       CF_RNPipMetaValueS_AH1
    Create Date:     2003-04-26 13:33:53
    Author:          J.Chen
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-26 J.Chen        3.5    Returns all RosettaNet Objects at Process,Activity
                                    and Action level with RefRoutingObjId
    2003-05-03 J.Chen        3.5    Changes made on CF_RoutingObjRef_Lnk table
    2003-05-13 J.Chen        3.5    Changed RoutingObjId to RoutingObjRefId in CF_ProcessDef for rc2
    2003-05-23 J.Chen        3.5    Changed rc2 using CF_RNPipMetaValue and return RoutingObjID (Process,activity,action)
    2003-06-17 J.Chen        3.5    Added RTRIM on RoutingObjname/Cd/ver
    2003-07-16 J.Chen        3.5    changed RoutingobjId to RoutingObjRefId in connect by prior
    2004-10-11 S.Priani      4.3    ReWrite for Ora-3113 on 9204
    2006-05-11 S.Priani      6.1    CR60181
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_RNPipMetaValueS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 For
	Select	T2.RoutingObjId,
		tp.RoutingObjTypeKey,
		RTRIM(ob.RoutingObjName) As RoutingObjName,
		RTRIM(ob.RoutingObjCd) As RoutingObjCd,
		RTRIM(ob.RoutingObjVer) As RoutingObjVer,
		T2.RefRoutingObjId
	From
		(
	        Select	T3.RoutingObjId,
			T3.RefRoutingObjId,
			T3.RoutingObjRefId,
			T3.RoutingObjParentRefId
		From	(
	                Select	cl.RoutingObjId,
				pl.RoutingObjId As RefRoutingObjId,
				cl.RoutingObjRefId,
				cl.RoutingObjParentRefId
			From	CF_RoutingObjRef_Lnk cl
				,CF_RoutingobjRef_Lnk pl 
			WHERE pl.RoutingObjRefId = cl.RoutingObjParentRefId
			) T3
	            Connect by  T3.RoutingObjRefId != T3.RoutingObjParentRefId
	            	And Prior T3.RoutingObjRefId = T3.RoutingObjParentRefId
	            Start With  T3.RoutingObjParentRefId in
			(
		        Select  pli.RoutingObjRefId
	        	From    CF_RoutingObjRef_Lnk pli
			WHERE	EXISTS (SELECT 'x'
					FROM	CF_RoutingObjRef_Lnk cli 
					WHERE	cli.RoutingObjParentRefId = pli.RoutingObjRefId
					AND	cli.RoutingObjId IN 
						(SELECT o.RoutingObjId
						FROM	CF_RoutingObject o
						Where   o.RoutingObjTypeId = 
							(SELECT t.RoutingObjTypeId
				                        FROM    CF_RoutingObjType t
			        	                WHERE t.RoutingObjTypeKey = 'Process')
						)
					)
			AND	EXISTS (SELECT 'x'
					FROM	CF_RoutingObject r 
					WHERE	r.RoutingObjId = pli.RoutingObjId
					AND	r.RoutingObjCd = 'RosettaNet')
			)
	         ) T2
		,CF_RoutingObject ob 
		,CF_RoutingObjType tp 
	WHERE	tp.RoutingObjTypeId = ob.RoutingObjTypeId 
	AND	ob.RoutingObjId = t2.RoutingObjId
    Union
    Select  T.RoutingObjId,
            T.routingobjtypekey,
            RTRIM(T.RoutingObjName) As RoutingObjName,
            RTRIM(T.RoutingObjCd) As RoutingObjCd,
            RTRIM(T.RoutingObjVer) As RoutingObjVer,
            T.refRoutingObjId
          --  ,T.RoutingObjRefId,
          --  T.RoutingObjParentRefId
    From
         (
            Select  o.RoutingObjId,
                    o.RoutingObjName,
                    o.RoutingObjCd,
                    o.RoutingObjVer,
                    t.RoutingObjTypeKey,
                    pl.RoutingObjId As RefRoutingObjId,
                    cl.RoutingObjRefId,
                    cl.RoutingObjParentRefId
            From    CF_RoutingObject o
                Inner Join CF_RoutingObjType t On t.RoutingObjTypeId = o.RoutingObjTypeId
                Inner Join CF_RoutingObjRef_Lnk cl On cl.RoutingObjId = o.RoutingObjId
                Inner Join CF_RoutingObjRef_Lnk pl On pl.RoutingObjRefId = cl.RoutingobjParentRefId
            Where     t.RoutingObjTypeKey in ('Process') -- ,'Activity','Action')
         ) T
         Inner Join CF_RoutingObject r on r.RoutingObjId = T.refRoutingObjId
    Where r.RoutingObjCd = 'RosettaNet';

    Open rc2 For
    Select    d.ActionObjId,
              d.ActivityObjId,
              d.ProcessObjId,
              t.MetaDefKey,
              d.MetaDefValue
              ,g.RNPIPAttr_GrpId
    From      CF_RNPipMetaValue d
              Inner Join CF_RNPipMetaDef t On t.MetaDefId = d.MetaDefId
              Inner Join CF_RNPipAttr_Grp g On g.RNPIPAttr_GrpID = t.RNPIPAttr_GrpID
	;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End CF_RNPipMetaValueS_AH1;
/


-- CF_RNPIPMETAVALUES_AH2.sql

CREATE OR REPLACE PROCEDURE         CF_RNPipMetaValueS_AH2
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
     , p_Locale  in varchar2
    , p_METAValueID IN NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_RNPIPMETAVALUES_AH2
    Create Date:     2003-05-23 16:09:54
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-23 SYSTEM        3.5    Created.
    2003-05-28 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2004-03-05 S.Priani      4.2.2  Removed NLS
    2006-05-11 S.Priani      6.1    CR60181
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_RNPIPMETAVALUES_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_Locale=>'||p_Locale||CHR(10)
        ||', p_METAVALUEID=>'||p_METAVALUEID||CHR(10)
        ;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_METAVALUEID IS NOT NULL
 THEN
    OPEN RC1 FOR
    SELECT
        v.METAVALUEID
        ,v.PROCESSOBJID
        ,v.ACTIVITYOBJID
        ,v.ACTIONOBJID
        ,v.METADEFID
        ,RTRIM(v.METADEFVALUE) METADEFVALUE
        ,l.MetaDefKey
        ,l.RESOURCEKEY
        ,l.DESCRIPTION
        ,v.ROWTS
        ,v.CREATEDATE
              ,g.RNPIPAttr_GrpId
    FROM
        CF_RNPipMetaValue v
        Inner join CF_RNPipMetaDef l On v.MetaDefId = l.MetaDefId
              Inner Join CF_RNPipAttr_Grp g On g.RNPIPAttr_GrpID = l.RNPIPAttr_GrpID
    Where
        v.METAVALUEID = p_METAVALUEID
    ;
    ELSE
    OPEN RC1 FOR
    SELECT
        v.METAVALUEID
        ,v.PROCESSOBJID
        ,v.ACTIVITYOBJID
        ,v.ACTIONOBJID
        ,v.METADEFID
        ,RTRIM(v.METADEFVALUE) METADEFVALUE
        ,l.MetaDefKey
        ,l.RESOURCEKEY
        ,l.DESCRIPTION
        ,v.ROWTS
        ,v.CREATEDATE
              ,g.RNPIPAttr_GrpId
    FROM
        CF_RNPipMetaValue v
        Inner join CF_RNPipMetaDef l On v.MetaDefId = l.MetaDefId
              Inner Join CF_RNPipAttr_Grp g On g.RNPIPAttr_GrpID = l.RNPIPAttr_GrpID
    ;
    END IF;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End CF_RNPipMetaValueS_AH2;
/


-- CF_RNPIPMETAVALUES_AH3.sql

CREATE OR REPLACE PROCEDURE         CF_RNPipMetaValueS_AH3
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
     , p_Locale  in varchar2
    , p_ProcessObjId in number
    , p_ActivityObjId in number
    , p_ActionObjId in number
    , p_MetaDefKey in varchar2  -- nullable
    )
AS

/*
    ================================================================================
    Proc Name:       CF_RNPIPMETAVALUES_AH3
    Create Date:     2003-05-23 16:09:54
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-23 SYSTEM        3.5    Created.
    2003-05-28 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2004-03-05 S.Priani      4.2.2  Removed NLS 
    2006-05-11 S.Priani      6.1    CR60181
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/



v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_RNPIPMETAVALUES_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_Locale=>'||p_Locale||CHR(10)
        ||', p_ProcessObjId=>'||p_ProcessObjId||CHR(10)
        ||', p_ActivityObjId=>'||p_ActivityObjId||CHR(10)
        ||', p_ActionObjId=>'||p_ActionObjId||CHR(10)
        ||', p_MetaDefKey=>'|| p_MetaDefKey ||CHR(10)
        ;

--- v_Locale PR_Locale.Locale%Type := FN_CHK_DEFAULT_LOCALE(p_Locale, 'CF_RNPIPMETADEF_LOC');

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;

    IF p_MetaDefKey IS NOT NULL Then
        OPEN RC1 FOR
        SELECT
            v.METAVALUEID
            ,v.PROCESSOBJID
            ,v.ACTIVITYOBJID
            ,v.ACTIONOBJID
            ,v.METADEFID
            ,RTRIM(v.METADEFVALUE) METADEFVALUE
            ,l.MetaDefKey
            ,l.RESOURCEKEY
            ,l.DESCRIPTION
            ,v.ROWTS
            ,v.CREATEDATE
              ,g.RNPIPAttr_GrpId
        FROM
            CF_RNPipMetaValue v
            Inner join CF_RNPipMetaDef l On v.MetaDefId = l.MetaDefId
              Inner Join CF_RNPipAttr_Grp g On g.RNPIPAttr_GrpID = l.RNPIPAttr_GrpID
        Where
            ProcessObjId = p_ProcessObjId
        And ActivityObjId = p_ActivityObjId
        And ActionObjId = p_ActionObjId
        And MetaDefKey = p_MetaDefKey
        ;
    Else
        Open RC1 For
        Select
             v.METAVALUEID
            ,v.PROCESSOBJID
            ,v.ACTIVITYOBJID
            ,v.ACTIONOBJID
            ,v.METADEFID
            ,RTRIM(v.METADEFVALUE) METADEFVALUE
            ,l.MetaDefKey
            ,l.RESOURCEKEY
            ,l.DESCRIPTION
            ,v.ROWTS
            ,v.CREATEDATE
              ,g.RNPIPAttr_GrpId
        FROM
            CF_RNPipMetaValue v
            Inner join CF_RNPipMetaDef l On v.MetaDefId = l.MetaDefId
              Inner Join CF_RNPipAttr_Grp g On g.RNPIPAttr_GrpID = l.RNPIPAttr_GrpID
        Where
            ProcessObjId = p_ProcessObjId
        And ActivityObjId = p_ActivityObjId
        And ActionObjId = p_ActionObjId
        ;
    End If;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End CF_RNPipMetaValueS_AH3;
/


-- CF_RNPIPMETAVALUES_AH4.sql

CREATE OR REPLACE PROCEDURE         CF_RNPipMetaValueS_AH4
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , RC2  OUT  DB_REF.RCT
    , p_MetaDefValue  IN Varchar2  -- Always provided
    , p_ResponseFlg   IN pls_integer -- = 1 True 0 = False
    )
AS

/*
    ================================================================================
    Proc Name:       CF_RNPipMetaValueS_AH4
    Create Date:     2003-05-30 16:09:54
    Author:          V.Goel
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RNPIPMETAVALUES_AH4';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_MetaDefValue=>'||p_MetaDefValue||CHR(10)
        ||', p_ResponseFlg=>'||  p_ResponseFlg||CHR(10);
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    Begin
      IF p_ResponseFlg = 1 THEN
        OPEN RC1 FOR
        SELECT distinct *
        FROM
            CF_RnPipMetaValue
        WHERE
        (ProcessObjId, ActionObjId)  =
            (SELECT ProcessObjId, ActionObjId  FROM CF_RnPipMetaValue WHERE MetaDefValue = p_MetaDefValue);

      ELSE
          OPEN RC1 FOR
          SELECT distinct *
          FROM
              CF_RnPipMetaValue
          WHERE
              (ProcessObjId, ACTIVITYOBJID)  =
              (SELECT ProcessObjId, ACTIVITYOBJID  FROM CF_RnPipMetaValue WHERE MetaDefValue = p_MetaDefValue)
          ORDER BY ProcessObjId, ACTIVITYOBJID, ActionObjId, METADEFID;
       END IF;
        EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            p_ReturnCd := 3;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'RC1 - NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
    End;
    IF RC2%ISOPEN THEN CLOSE RC2; END IF;
    Begin
        Open RC2 For
        SELECT
          meta.MetaDefValue,
                meta.ActivityObjId,
                meta.ActionObjId,
                ro.RoutingObjId,
                RTRIM(ro.RoutingObjName) As RoutingObjName
        FROM
          CF_RNPipMetaValue meta
          Inner Join CF_RoutingObject ro  ON ro.RoutingObjId=meta.ActivityObjId
                                          OR ro.RoutingObjId=meta.ActionObjId
        WHERE
            meta.MetaDefValue = p_MetaDefValue
--        and (ro.RoutingObjId=meta.ActivityObjId or ro.RoutingObjId=meta.ActionObjId)
        ;
        EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            p_ReturnCd := 3;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'RC2 - NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
    End;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_RNPipMetaValueS_AH4;
/


-- CF_RNPIPMETAVALUEU.sql

CREATE OR REPLACE Procedure CF_RNPIPMETAVALUEU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_METAVALUEID in NUMBER
	,p_PROCESSOBJID in NUMBER
	,p_ACTIVITYOBJID in NUMBER
	,p_ACTIONOBJID in NUMBER
	,p_METADEFID in NUMBER
	,p_METADEFVALUE in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_RNPIPMETAVALUEU
	Create Date:     2003-09-26 08:20:55
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_RNPIPMETAVALUEU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_METAVALUEID=>'||p_METAVALUEID||CHR(10)
		||', p_PROCESSOBJID=>'||p_PROCESSOBJID||CHR(10)
		||', p_ACTIVITYOBJID=>'||p_ACTIVITYOBJID||CHR(10)
		||', p_ACTIONOBJID=>'||p_ACTIONOBJID||CHR(10)
		||', p_METADEFID=>'||p_METADEFID||CHR(10)
		||', p_METADEFVALUE=>'||p_METADEFVALUE||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_RNPIPMETAVALUE
	SET
		PROCESSOBJID=p_PROCESSOBJID
		,      ACTIVITYOBJID=p_ACTIVITYOBJID
		,      ACTIONOBJID=p_ACTIONOBJID
		,      METADEFID=p_METADEFID
		,      METADEFVALUE=p_METADEFVALUE
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		METAVALUEID = p_METAVALUEID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_RNPIPMETAVALUEU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_RNPIPMETAVALUEU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_RNPIPMETAVALUEU:'||SQLERRM);
End CF_RNPIPMETAVALUEU;
/


-- CF_ROUTINGATTROPTD.sql

Create or Replace Procedure CF_ROUTINGATTROPTD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_ATTROPTID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_ROUTINGATTROPTD
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGATTROPTD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ATTROPTID=>'||p_ATTROPTID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_ROUTINGATTROPT
	Where 
		ATTROPTID = p_ATTROPTID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGATTROPTD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_ROUTINGATTROPTD;
/


-- CF_ROUTINGATTROPTI.sql

Create or Replace Procedure CF_ROUTINGATTROPTI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ROUTINGATTRID in NUMBER
	,p_ATTRVALUEKEY in VARCHAR2
	,p_ATTRSEQNBR in NUMBER
	,p_DEFAULTFLG in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_ROUTINGATTROPTI
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGATTROPTI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTRID=>'||p_ROUTINGATTRID||CHR(10)
		||', p_ATTRVALUEKEY=>'||p_ATTRVALUEKEY||CHR(10)
		||', p_ATTRSEQNBR=>'||p_ATTRSEQNBR||CHR(10)
		||', p_DEFAULTFLG=>'||p_DEFAULTFLG||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Insert into CF_ROUTINGATTROPT(
		ATTROPTID
		,ROUTINGATTRID
		,ATTRVALUEKEY
		,ATTRSEQNBR
		,DEFAULTFLG
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		CF_ROUTINGATTROPT_SEQ.nextval
		,p_ROUTINGATTRID
		,p_ATTRVALUEKEY
		,p_ATTRSEQNBR
		,p_DEFAULTFLG
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGATTROPTI' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGATTROPTI;
/


-- CF_ROUTINGATTROPTS.sql

Create or Replace Procedure CF_ROUTINGATTROPTS
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1  OUT  DB_REF.RCT		
	,p_ROUTINGATTRID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_ROUTINGATTROPTS
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
  2006-04-07 Shishir N        6.1 60217: Partner Migration
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGATTROPTS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTRID=>'||p_ROUTINGATTRID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR
	SELECT
			ATTROPTID    
			,ROUTINGATTRID
			,ATTRVALUEKEY 
			,ATTRSEQNBR   
			,DEFAULTFLG   
			RESOURCEKEY  
			,DESCRIPTION	 
			,ROWTS        
			,CREATEDATE   
			,UPDATEDATE   
		FROM
			CF_ROUTINGATTROPT
		WHERE
		  ROUTINGATTRID = p_ROUTINGATTRID
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGATTROPTS' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGATTROPTS;
/


-- CF_ROUTINGATTRS_AH2.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGATTRS_AH2
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , RC2  out  db_ref.RCT
    , p_Locale in Varchar2
    , p_RoutingObjRefId in number
    )
AS

/*
    ================================================================================
    Proc Name:       CF_RoutingAttrS_AH2
    Create Date:     2003-09-25
    Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2  Removed NLS    
	2006-05-11 S.Priani      6.1    CR60181
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_RoutingAttrS_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||'p_Locale=>'|| p_Locale || CHR(10)
        ||'p_RoutingObjRefId=>'|| p_RoutingObjRefId || CHR(10)
        ;

v_RoutingAttrId   number(10);
v_value varchar2(50);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
--    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(v_Locale);

    --for testing--
    --dbms_output.put_line('v_Locale = ' || v_Locale);
    --select value into v_value from v$nls_parameters where parameter = 'NLS_SORT';
    --dbms_output.put_line('The NLS_SORT = ' || v_value);

    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 For
    Select
               v.RoutingAttrId
	      ,v.Resourcekey
	      ,v.AttributeKey
	      ,v.Description
	      ,v.AttrDefLevel
	      ,v.RequiredFlg
	      ,v.VisibilityCd
	      ,v.RowTS
	      ,v.attrdesckey   
	      ,v.attrdescription 
	      ,v.validationrulekey 
              ,v.validescription 
              ,g.RoutingAttr_GrpId
    From
              CF_RoutingObjAttr_lnk l
              Inner join CF_RoutingAttr v On v.RoutingAttrId = l.RoutingAttrId
              Inner join CF_RoutingAttr_Grp g On v.RoutingAttr_GrpId = g.RoutingAttr_GrpId
    Where     l.RoutingObjRefId = p_RoutingObjRefId
		;

    Open rc2 For
    Select
              RoutingAttrId
              ,AttrOptId
              ,AttrValueKey
              ,DefaultFlg
              ,AttrSeqNbr
              ,Resourcekey
              ,RowTS
              ,Description
    From
              CF_RoutingAttrOpt
    Where     RoutingAttrId in ( Select     a.RoutingAttrId
                                 From       CF_RoutingObjAttr_lnk l
                                            Inner join CF_RoutingAttr a On a.RoutingAttrId = l.RoutingAttrId
                                  Where     l.RoutingObjRefId = p_RoutingObjRefId
                               )
    ORDER BY  RoutingAttrId, attrseqnbr;


    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		RAISE;
END CF_ROUTINGATTRS_AH2;
/


-- CF_ROUTINGATTR_GRPD.sql

-- > EXECUTE DB_GENPROC.GENPROC_DELETE('CF_ROUTINGATTR_GRP');
CREATE OR REPLACE PROCEDURE CF_ROUTINGATTR_GRPD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ROUTINGATTR_GRPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGATTR_GRPD
	Create Date:     2006-05-09 13:43:18
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_ROUTINGATTR_GRPD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTR_GRPID=>'||p_ROUTINGATTR_GRPID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_ROUTINGATTR_GRP
	Where 
		ROUTINGATTR_GRPID = p_ROUTINGATTR_GRPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_ROUTINGATTR_GRPD;
/
-- > SPOOL OFF


-- CF_ROUTINGATTR_GRPI.sql

-- > EXECUTE DB_GENPROC.GENPROC_INSERT('CF_ROUTINGATTR_GRP', TRUE, TRUE );
CREATE OR REPLACE PROCEDURE CF_ROUTINGATTR_GRPI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGATTR_GRPKEY in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGATTR_GRPI
	Create Date:     2006-05-09 13:43:17
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_ROUTINGATTR_GRPI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTR_GRPKEY=>'||p_ROUTINGATTR_GRPKEY||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_ROUTINGATTR_GRP(
		ROUTINGATTR_GRPID
		,ROUTINGATTR_GRPKEY
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		CF_ROUTINGATTR_GRP_SEQ.nextval
		,p_ROUTINGATTR_GRPKEY
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_ROUTINGATTR_GRP_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END CF_ROUTINGATTR_GRPI;
/
-- > SPOOL OFF


-- CF_ROUTINGATTR_GRPS.sql

-- > EXECUTE DB_GENPROC.GENPROC_SELECT('CF_ROUTINGATTR_GRP', FALSE );
CREATE OR REPLACE PROCEDURE CF_ROUTINGATTR_GRPS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ROUTINGATTR_GRPID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGATTR_GRPS
	Create Date:     2006-05-09 13:43:18
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_ROUTINGATTR_GRPS';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTR_GRPID=>'||p_ROUTINGATTR_GRPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ROUTINGATTR_GRPID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		ROUTINGATTR_GRPID
		,RTRIM(ROUTINGATTR_GRPKEY) ROUTINGATTR_GRPKEY
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,ROWTS
		,CREATEDATE
	FROM 
		CF_ROUTINGATTR_GRP
	Where 
		ROUTINGATTR_GRPID = p_ROUTINGATTR_GRPID
	ORDER BY ORDERING
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		ROUTINGATTR_GRPID
		,RTRIM(ROUTINGATTR_GRPKEY) ROUTINGATTR_GRPKEY
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,ROWTS
		,CREATEDATE
	FROM 
		CF_ROUTINGATTR_GRP
	ORDER BY ORDERING
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ROUTINGATTR_GRPS;
/
-- > SPOOL OFF


-- CF_ROUTINGATTR_GRPU.sql

-- > EXECUTE DB_GENPROC.GENPROC_UPDATE('CF_ROUTINGATTR_GRP');
CREATE OR REPLACE PROCEDURE CF_ROUTINGATTR_GRPU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_ROUTINGATTR_GRPID in NUMBER
	,p_ROUTINGATTR_GRPKEY in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGATTR_GRPU
	Create Date:     2006-05-09 13:43:19
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_ROUTINGATTR_GRPU';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTR_GRPID=>'||p_ROUTINGATTR_GRPID||CHR(10)
		||', p_ROUTINGATTR_GRPKEY=>'||p_ROUTINGATTR_GRPKEY||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_ROUTINGATTR_GRP
	SET
		ROUTINGATTR_GRPKEY=p_ROUTINGATTR_GRPKEY
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		ROUTINGATTR_GRPID = p_ROUTINGATTR_GRPID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End CF_ROUTINGATTR_GRPU;
/
-- > SPOOL OFF


-- CF_ROUTINGATTR_LNKI_CPA.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGATTR_LNKI_CPA
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGOBJRefID	IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGATTR_LNKI_CPA
	Create Date:     2006-05-02 15:44:51
	Author:          SYSTEM 

Creates links to ebXML routing attributes for a Routing Object (given RoutingObjRefID).
Used by console during CPA import.
	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-02 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_DupCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'CF_ROUTINGATTR_LNKI_CPA';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;
CURSOR	c_ebms_attrs IS
	SELECT	RoutingAttrId, AttributeKey
	FROM	CF_ROUTINGATTR
	WHERE	AttributeKey IN ('x'
		,'TimeToAcknowledge'
		,'TimeToPerform'
		,'RetryCount'
		,'DigitalSignatureRequired'
		,'NonRepudiationRequired'
		,'NonRepudiationOfReceipt'
	)
	OR	AttributeKey LIKE 'bcg.ro.ebxml%';
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	FOR r_ebms_attrs IN c_ebms_attrs LOOP
	BEGIN
		v_RowCnt := v_RowCnt + 1;
		INSERT INTO cf_routingobjattr_lnk(
			routingobjrefid
			,routingattrid
			)
		VALUES 	(p_routingObjRefId
			,r_ebms_attrs.RoutingAttrID
		);
	EXCEPTION
		WHEN Dup_Val_On_Index THEN
			v_DupCnt := v_dupCnt + 1;
	END;
		
	END LOOP;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		v_RowCnt AttrCnt, v_DupCnt ExistCnt From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		Rollback work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		RAISE;
END CF_ROUTINGATTR_LNKI_CPA;
/


-- CF_ROUTINGATTR_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGATTR_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_ROUTINGATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGATTR_LOC_VWS
	Create Date:     2003-09-26 00:10:41
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
    	2004-03-05 S.Narain       4.2.2 Removed NLS
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGATTR_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTRID=>'||p_ROUTINGATTRID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ROUTINGATTRID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		ROUTINGATTRID
		,RTRIM(ATTRIBUTEKEY) ATTRIBUTEKEY
		,REQUIREDFLG
		,ATTRDEFLEVEL
		,ATTRSEQNBR
		,VISIBILITYCD
		,ROWTS
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(ATTRDESCKEY) ATTRDESCKEY
		,RTRIM(VALIDATIONRULEKEY) VALIDATIONRULEKEY
		,DESCRIPTION
	FROM
		CF_ROUTINGATTR
	Where
		ROUTINGATTRID = p_ROUTINGATTRID;
	ELSE
	OPEN RC1 FOR
	SELECT
		ROUTINGATTRID
		,RTRIM(ATTRIBUTEKEY) ATTRIBUTEKEY
		,REQUIREDFLG
		,ATTRDEFLEVEL
		,ATTRSEQNBR
		,VISIBILITYCD
		,ROWTS
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(ATTRDESCKEY) ATTRDESCKEY
		,RTRIM(VALIDATIONRULEKEY) VALIDATIONRULEKEY
		,DESCRIPTION
	FROM
		CF_ROUTINGATTR;

	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ROUTINGATTR_LOC_VWS;
/


-- CF_ROUTINGOBJATTRDEF_LNKI.sql

CREATE OR REPLACE PROCEDURE cf_routingobjattrdef_lnki (
   p_returncd          OUT      PLS_INTEGER,
   p_debugflg          IN       PLS_INTEGER,
   rc1                 OUT      db_ref.rct,
   p_routingobjrefid   IN       NUMBER
)
AS
/*
   ================================================================================
   Proc Name:       CF_ROUTINGOBJATTRDEF_LNKI
   Create Date:     2003-09-26 00:03:39
   Author:          SYSTEM

                      ::::::::::::::::::::::::::::::
                            M O D I F I C A T I O N
                      ::::::::::::::::::::::::::::::

   ---------- ------------- ------ ---------------------------------------------
   DATE       NAME          Ver    DESCRIPTION
   ---------- ------------- ------ ---------------------------------------------
   2006-05-31 Shishir N        6.1 Created - 100903

   ================================================================================
   Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================*/
   v_pk_value       NUMBER          := 0;
   v_rowcnt         INTEGER         DEFAULT 0;
   v_begints        TIMESTAMP       DEFAULT fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   DEFAULT 'CF_ROUTINGOBJATTRDEF_LNKI';
   v_exceptionflg   INTEGER         DEFAULT 0;
   v_paramstxt      VARCHAR2 (4000)
      DEFAULT    'p_DebugFlg=>'
              || p_debugflg
              || CHR (10)
              || ', p_ROUTINGOBJREFID=>'
              || p_routingobjrefid
              || CHR (10);
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
   -- associate the default Non Repudation Required with the new RO
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
  Select p_RoutingobjrefId,RoutingAttrId  from cf_routingAttr where AttributeKey = 'NonRepudiationRequired' 
;      

   p_returncd := 0;
   v_rowcnt := SQL%ROWCOUNT;

   IF rc1%ISOPEN
   THEN
      CLOSE rc1;
   END IF;

   Open RC1 FOR Select
		 1 RowTS From Dual;

   p_returncd := 0;
   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
EXCEPTION
   WHEN NO_DATA_FOUND
   THEN
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            'NO_DATA_FOUND, ' || v_paramstxt,
                            p_debugflg,
                            0
                           );
      p_returncd := 3;
   WHEN DUP_VAL_ON_INDEX
   THEN
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            'DUP_VAL_ON_INDEX, ' || v_paramstxt,
                            p_debugflg,
                            v_exceptionflg
                           );
      p_returncd := 2;
   WHEN OTHERS
   THEN
      p_returncd := 1;
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            SQLERRM || CHR (10) || v_paramstxt,
                            p_debugflg,
                            1
                           );
      raise_application_error (-20999,
                                  '20002 CF_ROUTINGOBJATTRDEF_LNKI'
                               || SQLCODE
                               || ' '
                               || SQLERRM
                              );
END cf_routingobjattrdef_lnki;
/


-- CF_ROUTINGOBJATTR_LNKD.sql

CREATE OR REPLACE Procedure CF_ROUTINGOBJATTR_LNKD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_ROUTINGOBJREFID in NUMBER
	,p_ROUTINGATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGOBJATTR_LNKD
	Create Date:     2003-09-26 08:24:01
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJATTR_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJREFID=>'||p_ROUTINGOBJREFID||CHR(10)
		||', p_ROUTINGATTRID=>'||p_ROUTINGATTRID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_ROUTINGOBJATTR_LNK
	Where
		ROUTINGOBJREFID = p_ROUTINGOBJREFID
		AND ROUTINGATTRID = p_ROUTINGATTRID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJATTR_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_ROUTINGOBJATTR_LNKD;
/


-- CF_ROUTINGOBJATTR_LNKI.sql

CREATE OR REPLACE Procedure CF_ROUTINGOBJATTR_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGOBJREFID in NUMBER
	,p_ROUTINGATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGOBJATTR_LNKI
	Create Date:     2003-09-26 00:03:39
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJATTR_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJREFID=>'||p_ROUTINGOBJREFID||CHR(10)
		||', p_ROUTINGATTRID=>'||p_ROUTINGATTRID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_ROUTINGOBJATTR_LNK(
		ROUTINGOBJREFID
		,ROUTINGATTRID
	) Values (
		p_ROUTINGOBJREFID
		,p_ROUTINGATTRID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;			
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJATTR_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJATTR_LNKI;
/


-- CF_ROUTINGOBJECT06_VWS_AH1.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJECT06_VWS_AH1
(
 	p_ReturnCd out pls_integer
 	,p_DebugFlg in pls_integer
	,RC1  OUT  DB_REF.RCT	
 	,p_RoutingObjRefId in NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       CF_ROUTINGOBJECT06_VWS_AH1
    Author:          Shishir N
    Returns the routing object info by RoutingObjRefId

    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECT06_VWS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RoutingObjRefId=>'||p_RoutingObjRefId||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR
  SELECT ROUTINGOBJREFNAME from CF_ROUTINGOBJECT06_VW
  where RoutingObjRefId = p_RoutingObjRefId;
  
  p_ReturnCd := 0;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		EXCEPTION
			When NO_DATA_FOUND THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 3;
			When OTHERS Then
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJECT06_VWS_AH1' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJECT06_VWS_AH1;
/

-- CF_ROUTINGOBJECT06_VWS_AH2.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJECT06_VWS_AH2
(
 	 p_ReturnCd out pls_integer
 	,p_DebugFlg in pls_integer
	,RC1  OUT  DB_REF.RCT	 	
 	,p_Routobjrefname in VARCHAR2
)
AS
/*
    ================================================================================
    Proc Name:       CF_ROUTINGOBJECT06_VWS_AH2
    Author:          Shishir N
    Returns the routing object info by RoutingObjRefId

    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECT06_VWS_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_Routobjrefname=>'||p_Routobjrefname||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR
   SELECT 
   RoutingObjRefId
   , RoutingobjId 
   from CF_ROUTINGOBJECT06_VW
  where ROUTINGOBJREFNAME = p_Routobjrefname;
  
  p_ReturnCd := 0;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		EXCEPTION
			When NO_DATA_FOUND THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 3;
			When OTHERS Then
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJECT06_VWS_AH2' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJECT06_VWS_AH2;
/

-- CF_ROUTINGOBJECT06_VWS_AH3.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJECT06_VWS_AH3
(
 	 p_ReturnCd out pls_integer
 	,p_DebugFlg in pls_integer
	,RC1  OUT  DB_REF.RCT	 	
 	,p_RoutingObjId in NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       CF_ROUTINGOBJECT06_VWS_AH3
    Author:          Shishir N
    Returns the routing object info by RoutingObjRefId

    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECT06_VWS_AH3';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RoutingObjId=>'||p_RoutingObjId||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR
    SELECT ROUTINGOBJCD||'(BCG_RO_SEPERATOR)'||ROUTINGOBJVER from CF_ROUTINGOBJECT06_VW
  where RoutingObjId = p_RoutingObjId;
  
  p_ReturnCd := 0;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		EXCEPTION
			When NO_DATA_FOUND THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 3;
			When OTHERS Then
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJECT06_VWS_AH3' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJECT06_VWS_AH3;
/

-- CF_ROUTINGOBJECT06_VWS_AH4.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJECT06_VWS_AH4
(
 	 p_ReturnCd out pls_integer
 	,p_DebugFlg in pls_integer
	,RC1  OUT  DB_REF.RCT		
 	,p_RoutingObjName in VARCHAR2
)
AS
/*
    ================================================================================
    Proc Name:       CF_ROUTINGOBJECT06_VWS_AH4
    Author:          Shishir N
    Returns the routing object info by RoutingObjRefId

    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECT06_VWS_AH4';
v_ExceptionFlg    integer Default 0;
v_RoutingObjcd  varchar2(255);
v_Routingobjver varchar2(255);
v_Length        Integer;
v_StringLocn    Integer;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RoutingObjName=>'||p_RoutingObjName||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN		
  IF RC1%ISOPEN THEN CLOSE RC1; END IF;
  
	    v_Length := Length(RTRIM(p_RoutingObjName));
	    v_StringLocn := INSTR(p_RoutingObjName,'(BCG_RO_SEPERATOR)');
	    
	    v_RoutingObjcd := SUBSTR(p_RoutingObjName, 1, (v_StringLocn-1));
      v_Routingobjver := SUBSTR(p_RoutingObjName, (v_StringLocn+18));
    
	OPEN RC1 FOR
  SELECT ROUTINGOBJID from CF_ROUTINGOBJECT06_VW
	  where RoutingObjCd = v_RoutingObjcd
  and Routingobjver = v_Routingobjver;
  
  p_ReturnCd := 0;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		EXCEPTION
			When NO_DATA_FOUND THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 3;
			When OTHERS Then
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJECT06_VWS_AH4' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJECT06_VWS_AH4;
/

-- CF_ROUTINGOBJECT06_VWS_AH5.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJECT06_VWS_AH5
(
 	 p_ReturnCd out pls_integer
 	,p_DebugFlg in pls_integer
	,RC1  OUT  DB_REF.RCT		
)
AS
/*
    ================================================================================
    Proc Name:       CF_ROUTINGOBJECT06_VWS_AH5
    Author:          Shishir N
    Returns the routing object info 
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECT06_VWS_AH5';
v_ExceptionFlg    integer Default 0;
v_RoutingObjcd  varchar2(255);
v_Routingobjver varchar2(255);
v_Length        Integer;
v_StringLocn    Integer;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN		
  IF RC1%ISOPEN THEN CLOSE RC1; END IF;
  
	OPEN RC1 FOR
  SELECT ROUTINGOBJREFID, ROUTINGOBJREFNAME from CF_ROUTINGOBJECT06_VW;
  
  p_ReturnCd := 0;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		EXCEPTION
			When NO_DATA_FOUND THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 3;
			When OTHERS Then
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJECT06_VWS_AH5' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJECT06_VWS_AH5;
/

-- CF_ROUTINGOBJECTD_AH1.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJECTD_AH1 
(
   p_returncd             OUT      INTEGER,
   p_debugflg             IN       PLS_INTEGER,
   p_routingobjectrefid   IN       NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       cf_routingobjectd_ah1
    Create Date:     2007-08-20 16:40:12
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-10 BHARANI J         1  Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_routingobjrefid         INTEGER;
   v_routingobjparentrefid   INTEGER;
   v_inobjrefid              INTEGER;
   v_rc                      INTEGER;
   v_rowcnt                  INTEGER;
   v_child                   INTEGER Default 0;
   temp_routingobjrefid      INTEGER;
   v_count                   INTEGER;
   temp_routingobjid         INTEGER;
   v_PK_Value        number := 0;
   
   v_RowCnt          integer Default 0;
   v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
   v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECTD_AH1';
   v_ExceptionFlg    integer Default 0;
   v_ParamsTxt       varchar2(4000) Default
   		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
   		||', p_routingobjectrefid=>'||p_routingobjectrefid||CHR(10)
		;
   
   
BEGIN

   FOR xx IN
      ((SELECT     routingobjrefid, routingobjparentrefid
              FROM cf_routingobjref_lnk
        START WITH routingobjrefid IN (
                      SELECT routingobjrefid
                        FROM cf_routingobjref_lnk
                       WHERE routingobjid IN (
                                 SELECT routingobjid
                                   FROM cf_routingobject
                                  WHERE routingobjrefid =
                                                         p_routingobjectrefid))
        CONNECT BY routingobjparentrefid = PRIOR routingobjrefid)
       ORDER BY routingobjrefid DESC)
       
   LOOP
   
 
   
      v_routingobjrefid := xx.routingobjrefid;
      v_routingobjparentrefid := xx.routingobjparentrefid;
      
      DBMS_OUTPUT.put_line (v_routingobjrefid);

      --delete attributes for parent relationships and children relationships
      --#1 cf_routingobjattr_lnk

      --delete from cf_transformroutingobj_lnk for from routing object
      DELETE FROM cf_transformroutingobj_lnk
            WHERE fromroutingobjrefid = v_routingobjrefid;

      v_rc := '1';

      --delete from  cf_transformroutingobj_lnk for to routing object
      DELETE FROM cf_transformroutingobj_lnk
            WHERE toroutingobjrefid = v_routingobjrefid;

      v_rc := '2';

      --delete from pr_workflowinteract_lnk for from routing object
      DELETE FROM pr_workflowinteract_lnk
            WHERE interactid IN (SELECT interactid
                                   FROM cf_validobjinteract
                                  WHERE frroutingobjrefid = v_routingobjrefid);

      v_rc := '3';

      --delete from pr_workflowinteract_lnk for to routing object
      DELETE FROM pr_workflowinteract_lnk
            WHERE interactid IN (SELECT interactid
                                   FROM cf_validobjinteract
                                  WHERE toroutingobjrefid = v_routingobjrefid);

      v_rc := '4';

      --delete from cf_transforminteract_lnk for the from routing object
      DELETE FROM cf_transforminteract_lnk
            WHERE interactid IN (SELECT interactid
                                   FROM cf_validobjinteract
                                  WHERE frroutingobjrefid = v_routingobjrefid);

      v_rc := '5';

      --delete from cf_transforminteract_lnk for the to routing object
      DELETE FROM cf_transforminteract_lnk
            WHERE interactid IN (SELECT interactid
                                   FROM cf_validobjinteract
                                  WHERE toroutingobjrefid = v_routingobjrefid);

      v_rc := '6';

      --delete from cf_validobjinteract for from routing object
      --#4
      DELETE FROM cf_validobjinteract
            WHERE frroutingobjrefid = v_routingobjrefid;

      v_rc := '7';

      --delete from cf_validobjinteract for to routing object
      --#4
      DELETE FROM cf_validobjinteract
            WHERE toroutingobjrefid = v_routingobjrefid;

      v_rc := '8';

      --delete from pr_routingattr_lnk for routingobjrefid
      --#5
      DELETE FROM pr_routingattr_lnk
            WHERE routingobjrefid = v_routingobjrefid;

      v_rc := '9';

      --delete from pr_routingattr_lnk for channelid for from capability id
      --#6
      DELETE FROM pr_routingattr_lnk
            WHERE channelid IN (
                     SELECT channelid
                       FROM pr_routingchannel
                      WHERE frcapabilityid IN (
                                    SELECT capabilityid
                                      FROM pr_capability
                                     WHERE routingobjrefid =
                                                            v_routingobjrefid));

      v_rc := '10';

      --delete from pr_routingattr_lnk for channelid for to capability id
      --#6
      DELETE FROM pr_routingattr_lnk
            WHERE channelid IN (
                     SELECT channelid
                       FROM pr_routingchannel
                      WHERE tocapabilityid IN (
                                    SELECT capabilityid
                                      FROM pr_capability
                                     WHERE routingobjrefid =
                                                            v_routingobjrefid));

      v_rc := '11';

      --delete from pr_channel_gateway_lnk for from capability
      --#7
      DELETE FROM pr_channel_gateway_lnk
            WHERE channelid IN (
                     SELECT channelid
                       FROM pr_routingchannel
                      WHERE frcapabilityid IN (
                                    SELECT capabilityid
                                      FROM pr_capability
                                     WHERE routingobjrefid =
                                                            v_routingobjrefid));

      v_rc := '12';

      --delete from pr_channel_gateway_lnk for to capability
      --#7
      DELETE FROM pr_channel_gateway_lnk
            WHERE channelid IN (
                     SELECT channelid
                       FROM pr_routingchannel
                      WHERE tocapabilityid IN (
                                    SELECT capabilityid
                                      FROM pr_capability
                                     WHERE routingobjrefid =
                                                            v_routingobjrefid));

      v_rc := '13';

      --delete from pr_routingchannel for from capability id
      --#8
      DELETE FROM pr_routingchannel
            WHERE frcapabilityid IN (
                                    SELECT capabilityid
                                      FROM pr_capability
                                     WHERE routingobjrefid =
                                                            v_routingobjrefid);

      v_rc := '14';

      --delete from pr_routingchannel for to capability id
      --#8
      DELETE FROM pr_routingchannel
            WHERE tocapabilityid IN (
                                    SELECT capabilityid
                                      FROM pr_capability
                                     WHERE routingobjrefid =
                                                            v_routingobjrefid);

      v_rc := '15';

      --delete from pr_capability
      --#9
      DELETE FROM pr_capability
            WHERE routingobjrefid = v_routingobjrefid;

      v_rc := '16';

      DELETE FROM cf_contenttypefilter
            WHERE routingobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                 (SELECT routingobjid
                                    FROM cf_routingobjref_lnk
                                   WHERE routingobjrefid = v_routingobjrefid));

      v_rc := '17';

      --delete from cf_edifamap_lnk
      DELETE FROM cf_edifamap_lnk
            WHERE routingobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                 (SELECT routingobjid
                                    FROM cf_routingobjref_lnk
                                   WHERE routingobjrefid = v_routingobjrefid));

      v_rc := '18';

      --delete from cf_guidelinemap_lnk
      DELETE FROM cf_guidelinemap_lnk
            WHERE routingobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                 (SELECT routingobjid
                                    FROM cf_routingobjref_lnk
                                   WHERE routingobjrefid = v_routingobjrefid));

      v_rc := '19';

      --delete from cf_profile_lnk
      DELETE FROM cf_profile_lnk
            WHERE routingobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                 (SELECT routingobjid
                                    FROM cf_routingobjref_lnk
                                   WHERE routingobjrefid = v_routingobjrefid));

      v_rc := '20';
     
      DELETE FROM cf_routingobjattr_lnk
            WHERE routingobjrefid = v_routingobjrefid;

      
      v_rc := '21';

      DELETE FROM cf_rnpipmetavalue
            WHERE processobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                  (SELECT routingobjid
                                     FROM cf_routingobjref_lnk
                                    WHERE routingobjrefid = v_routingobjrefid));
                                    
      DELETE FROM cf_rnpipmetavalue
            WHERE actionobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                  (SELECT routingobjid
                                     FROM cf_routingobjref_lnk
                                    WHERE routingobjrefid = v_routingobjrefid));  
                                    
      DELETE FROM cf_rnpipmetavalue
            WHERE activityobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                  (SELECT routingobjid
                                     FROM cf_routingobjref_lnk
                                    WHERE routingobjrefid = v_routingobjrefid));                                     

      v_rc := '22';

      --delete from protocolattr
      DELETE FROM cf_protocoldef_family
            WHERE protocolid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                  (SELECT routingobjid
                                     FROM cf_routingobjref_lnk
                                    WHERE routingobjrefid = v_routingobjrefid));

      v_rc := '23';

      DELETE FROM cf_protocolattr
            WHERE protocoldefid IN (
                     SELECT protocoldefid
                       FROM cf_protocoldef
                      WHERE routingobjid IN (
                               SELECT routingobjid
                                 FROM cf_routingobject
                                WHERE routingobjid =
                                         (SELECT routingobjid
                                            FROM cf_routingobjref_lnk
                                           WHERE routingobjrefid =
                                                             v_routingobjrefid)));

      v_rc := '24';

      --delete from protocoldef
      DELETE FROM cf_protocoldef
            WHERE routingobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                  (SELECT routingobjid
                                     FROM cf_routingobjref_lnk
                                    WHERE routingobjrefid = v_routingobjrefid));
                                    
	DELETE FROM cf_webservice_attrvalue 
		WHERE wsroutingobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                  (SELECT routingobjid
                                     FROM cf_routingobjref_lnk
                                    WHERE routingobjrefid = v_routingobjrefid));        
                                    
	DELETE FROM pr_errorflow 
		WHERE wsroutingobjid IN (
                     SELECT routingobjid
                       FROM cf_routingobject
                      WHERE routingobjid =
                                  (SELECT routingobjid
                                     FROM cf_routingobjref_lnk
                                    WHERE routingobjrefid = v_routingobjrefid));    
                                    
	-- ADDED FOR MULTIPLE CERTIFICATE FEATURE                                    
                                    
       
       DELETE FROM PR_CERT_SET_USAGE 
       		WHERE PARTNERRTOBJLNKID IN (
       		SELECT PARTNERRTOBJLNKID 
       		FROM PR_PARTNERPAIR_RTOBJPAIR_LNK
       		WHERE ROUTINGOBJPAIRID IN (
       			SELECT ROUTINGOBJPAIRID FROM PR_PARTNERPAIR_RTOBJPAIR_LNK
       			WHERE ROUTINGOBJPAIRID IN (
			       			SELECT ROUTINGOBJPAIRID 
			       			FROM  PR_ROUTINGOBJ_PAIR
			       			WHERE FRROUTINGOBJID IN (
			                        	SELECT routingobjid
			                          	FROM cf_routingobject
			                         	WHERE routingobjid =
			                                     	(SELECT routingobjid
			                                        FROM cf_routingobjref_lnk
			                                    	WHERE routingobjrefid = v_routingobjrefid))
			                           OR  TOROUTINGOBJID IN (
			                        	SELECT routingobjid
			                          	FROM cf_routingobject
			                         	WHERE routingobjid =
			                                     	(SELECT routingobjid
			                                        FROM cf_routingobjref_lnk
			                                    	WHERE routingobjrefid = v_routingobjrefid)))))   	
                                    	;  
       		
       		
       		
       
       DELETE FROM PR_PARTNERPAIR_RTOBJPAIR_LNK 
       		WHERE ROUTINGOBJPAIRID IN (
       			SELECT ROUTINGOBJPAIRID 
       			FROM  PR_ROUTINGOBJ_PAIR
       			WHERE FRROUTINGOBJID IN (
                        	SELECT routingobjid
                          	FROM cf_routingobject
                         	WHERE routingobjid =
                                     	(SELECT routingobjid
                                        FROM cf_routingobjref_lnk
                                    	WHERE routingobjrefid = v_routingobjrefid))
                           OR  TOROUTINGOBJID IN (
                        	SELECT routingobjid
                          	FROM cf_routingobject
                         	WHERE routingobjid =
                                     	(SELECT routingobjid
                                        FROM cf_routingobjref_lnk
                                    	WHERE routingobjrefid = v_routingobjrefid)))   	
                                    	;  
       				
       		
       
       
                                    
       DELETE FROM PR_ROUTINGOBJ_PAIR
               WHERE FRROUTINGOBJID IN (
                        SELECT routingobjid
                          FROM cf_routingobject
                         WHERE routingobjid =
                                     (SELECT routingobjid
                                        FROM cf_routingobjref_lnk
                                    WHERE routingobjrefid = v_routingobjrefid));  
                                    
	DELETE FROM PR_ROUTINGOBJ_PAIR
               WHERE TOROUTINGOBJID IN (
                        SELECT routingobjid
                          FROM cf_routingobject
                         WHERE routingobjid =
                                     (SELECT routingobjid
                                        FROM cf_routingobjref_lnk
                                    WHERE routingobjrefid = v_routingobjrefid));                                     

--  First cf_routingobjref_lnk entry needs to be deleted.  Then cf_routingobject needs to be
--  deleted.


      temp_routingobjrefid:=v_routingobjrefid;
             
             
    SELECT COUNT (1)
      INTO v_count
      FROM (SELECT routingobjid
              FROM cf_routingobjref_lnk
             WHERE routingobjid = (SELECT routingobjid
                                     FROM cf_routingobjref_lnk
                                WHERE routingobjrefid = temp_routingobjrefid));
                                    
                                    
      SELECT routingobjid
             INTO temp_routingobjid
             FROM cf_routingobjref_lnk
             WHERE routingobjrefid = temp_routingobjrefid;                           
      
            DELETE FROM cf_routingobjref_lnk
                   WHERE routingobjrefid = v_routingobjrefid;
      
            v_rc := '26';         
                                    
            if v_count = 1 then
      
            DELETE FROM cf_routingobject
                  WHERE  routingobjid = temp_routingobjid ;
                                    
            end if;                           
  
       
   END LOOP;
   
   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   
   p_returncd := 0;
   
--   v_rc := 'Success';
EXCEPTION
   
      
   WHEN NO_DATA_FOUND
   THEN
   DBMS_OUTPUT.put_line (   'The program failed at '
                              || v_rc
                              || '. Error message '
                              || SQLERRM
                           );
   
   WHEN OTHERS
      THEN
      ROLLBACK WORK;
   
      DBMS_OUTPUT.put_line (   'The program failed at '
                            || v_rc
                            || '. Error message '
                            || SQLERRM
                           );
END cf_routingobjectd_ah1;
/

-- CF_RoutingObjectI.SQL

Create or Replace Procedure CF_ROUTINGOBJECTI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGOBJTYPEID in NUMBER
	,p_ROUTINGOBJNAME in VARCHAR2
	,p_ROUTINGOBJCD in VARCHAR2
	,p_ROUTINGOBJVER in VARCHAR2
	,p_CHORGSEQNBR in NUMBER
	,p_ROUTINGOBJDESC in VARCHAR2
	,p_ACTIVESTATUSCD in NUMBER
	,p_VISIBILITYCD in NUMBER
	,p_DOCUMENTFLG in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGOBJECTI
	Create Date:     2003-11-21 14:10:29
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-11-21 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECTI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJTYPEID=>'||p_ROUTINGOBJTYPEID||CHR(10)
		||', p_ROUTINGOBJNAME=>'||p_ROUTINGOBJNAME||CHR(10)
		||', p_ROUTINGOBJCD=>'||p_ROUTINGOBJCD||CHR(10)
		||', p_ROUTINGOBJVER=>'||p_ROUTINGOBJVER||CHR(10)
		||', p_CHORGSEQNBR=>'||p_CHORGSEQNBR||CHR(10)
		||', p_ROUTINGOBJDESC=>'||p_ROUTINGOBJDESC||CHR(10)
		||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
		||', p_VISIBILITYCD=>'||p_VISIBILITYCD||CHR(10)
		||', p_DOCUMENTFLG=>'||p_DOCUMENTFLG||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_ROUTINGOBJECT(
		ROUTINGOBJID
		,ROUTINGOBJTYPEID
		,ROUTINGOBJNAME
		,ROUTINGOBJCD
		,ROUTINGOBJVER
		,CHORGSEQNBR
		,ROUTINGOBJDESC
		,ACTIVESTATUSCD
		,VISIBILITYCD
		,DOCUMENTFLG
	) Values (
		CF_ROUTINGOBJECT_SEQ.nextval
		,p_ROUTINGOBJTYPEID
		,p_ROUTINGOBJNAME
		,p_ROUTINGOBJCD
		,p_ROUTINGOBJVER
		,p_CHORGSEQNBR
		,p_ROUTINGOBJDESC
		,p_ACTIVESTATUSCD
		,p_VISIBILITYCD
		,p_DOCUMENTFLG
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		CF_ROUTINGOBJECT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJECTI' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJECTI;
/


-- CF_ROUTINGOBJECTI_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_RoutingObjectI_AH1
    (
     p_ReturnCd out pls_integer
    ,p_DebugFlg  in pls_integer
    ,RC1 out db_ref.RCT
    ,p_ROUTINGOBJTYPEID in NUMBER
    ,p_ROUTINGOBJNAME in VARCHAR2
    ,p_ROUTINGOBJCD in VARCHAR2
    ,p_ROUTINGOBJVER in VARCHAR2
    ,p_CHORGSEQNBR in NUMBER
    ,p_ROUTINGOBJDESC in VARCHAR2
    ,p_ACTIVESTATUSCD in NUMBER
    ,p_VISIBILITYCD in NUMBER
    ,p_ParentRoutingObjId in NUMBER
    ,p_DocumentFlg in number
    )
AS
/*
================================================================================
Proc Name:      CF_RoutingObjectI_AH1
Create Date:    12/19/01 04:16:23 PM
    Insert record Into CF_RoutingObjectI and CF_RoutingObject_lnk table, one exists in the
    CF_RoutingObject table then insert into CF_RoutingObject_lnk table
Author          SYSTEM
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME          Ver      DESCRIPTION
---------  --------------------  ---------------------------------------------
2003-04-23 M.Mount       3.5      Insert Proc for CF_RoutingObject and lnk table
2003-05-03 M.Mount       3.5      Changed p_RefRoutingObjId to p_ROUTINGOBJPARENTREFID
2003-05-06 J.Chen        3.5      Resolve the p_ParentRoutingObjId to RoutingObjParentRefId for Insert
2003-05-07 M.Mount       3.5      Made both insert procedure calls and return value in RC1 everytime.
2003-05-08 M.Mount       3.5      Changed the select for p.RoutingObjRefId to do self join.
2003-07-01 M.Mounnt      3.5      Changed return_code to p_returnCd and debug to p_debugflg
2003-07-19 J.Chen        3.5      Added DocumentFlg as input
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RoutingObjId    number := 0;
v_RoutingObjRefId number := 0;
v_RoutingObjParentRefId number;
v_PK_Value        number := 0;
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar(30) := 'CF_ROUTINGOBJECTI';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar(4000) :=
        'p_DebugFlg Flg=>'||p_DebugFlg ||CHR(10)
        ||', p_ROUTINGOBJTYPEID=>'||p_ROUTINGOBJTYPEID||CHR(10)
        ||', p_ROUTINGOBJNAME=>'||p_ROUTINGOBJNAME||CHR(10)
        ||', p_ROUTINGOBJCD=>'||p_ROUTINGOBJCD||CHR(10)
        ||', p_ROUTINGOBJVER=>'||p_ROUTINGOBJVER||CHR(10)
        ||', p_CHORGSEQNBR=>'||p_CHORGSEQNBR||CHR(10)
        ||', p_ROUTINGOBJDESC=>'||p_ROUTINGOBJDESC||CHR(10)
        ||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
        ||', p_VISIBILITYCD=>'||p_VISIBILITYCD||CHR(10)
        ||', p_ParentROUTINGOBJID=>'||p_ParentROUTINGOBJID||CHR(10)
        ||', p_DocumentFlg=>'||p_DocumentFlg
        ;

Procedure Get_RoutingObject_ID is
Begin
    Select ROUTINGOBJID into v_RoutingObjId
    From CF_RoutingObject
    Where ROUTINGOBJTYPEID  = p_ROUTINGOBJTYPEID
    AND ROUTINGOBJCD = p_ROUTINGOBJCD
    AND ROUTINGOBJVER = p_ROUTINGOBJVER;
End Get_RoutingObject_ID;


Procedure CF_RoutingObjI is
BEGIN
  Insert into CF_RoutingObject(
        ROUTINGOBJID
        ,ROUTINGOBJTYPEID
        ,ROUTINGOBJNAME
        ,ROUTINGOBJCD
        ,ROUTINGOBJVER
        ,CHORGSEQNBR
        ,ROUTINGOBJDESC
        ,ACTIVESTATUSCD
        ,VISIBILITYCD
        ,DocumentFlg
    ) Values (
        CF_ROUTINGOBJECT_SEQ.nextval
        ,p_ROUTINGOBJTYPEID
        ,p_ROUTINGOBJNAME
        ,p_ROUTINGOBJCD
        ,p_ROUTINGOBJVER
        ,p_CHORGSEQNBR
        ,p_ROUTINGOBJDESC
        ,p_ACTIVESTATUSCD
        ,p_VISIBILITYCD
        ,p_DocumentFlg
    )
    RETURNING RoutingObjId INTO v_RoutingObjId;

   --   dbms_output.put_line(v_RoutingObjId);
    p_ReturnCd := 0;
    v_RowCnt := sql%ROWCOUNT;
    p_ReturnCd := 0;

    Select    p.RoutingObjRefId
    Into      v_RoutingObjParentRefId
    From      CF_RoutingObjRef_Lnk p
    inner join CF_RoutingObjRef_Lnk c on p.RoutingObjParentRefId = c.RoutingObjRefId
    Where     p.RoutingObjId = p_ParentRoutingObjId;


 --   CF_RoutingObjRegI;  -- call to lnk table insert

    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,v_ParamsTxt,p_DebugFlg , v_ExceptionFlg);
 --   Commit;

      EXCEPTION
        When NO_DATA_FOUND THEN NULL;
        When DUP_VAL_ON_INDEX then
            Get_RoutingObject_ID;
  --          CF_RoutingObjRegI;
        When OTHERS Then
            Rollback work;
            p_ReturnCd  := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,v_ParamsTxt, p_DebugFlg , v_ExceptionFlg);
            Commit;
               RAISE_APPLICATION_ERROR (-20999,'20000 CF_RoutingObjectI_AH1' || SQLCODE|| ' '||SQLERRM);
End CF_RoutingObjI;

Procedure CF_RoutingObjRegI is
BEGIN
        Insert into CF_RoutingObjRef_lnk(
                 RoutingObjRefId
                ,RoutingObjId
                ,ROUTINGOBJPARENTREFID
        ) Values (
                 CF_RoutingObjRef_lnk_SEQ.nextVal
                ,v_RoutingObjId
                ,v_RoutingObjParentRefId
                        )
                returning  RoutingObjRefId into v_RoutingObjRefId;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg , v_ExceptionFlg);
  --      Commit;

        EXCEPTION
                When NO_DATA_FOUND THEN
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg , 0);
                When DUP_VAL_ON_INDEX then
                        Null;
                When OTHERS Then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,v_ParamsTxt, p_DebugFlg ,  v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR (-20999,'20000 CF_RoutingObjectI_AH1' || SQLCODE|| ''||SQLERRM);
End CF_RoutingObjRegI;

--


--                                        M  A  I  N
BEGIN
     v_RoutingObjId := NULL;
     v_RoutingObjRefId := NULL;

     CF_RoutingObjI;  -- do insert in Routing Object Table

     CF_RoutingObjRegI;  -- do insert into lnk table

     IF RC1%isOpen THEN Close RC1; END IF;
        Open RC1 FOR Select NVL(v_RoutingObjRefId,CF_RoutingObjRef_lnk_SEQ.currval) RefLnkId, NVL(v_RoutingObjId,CF_ROUTINGOBJECT_SEQ.currval) RoutingObjId, 1 ROWTS From Dual;

END CF_RoutingObjectI_AH1;
/


-- CF_ROUTINGOBJECTI_AH2.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJECTI_AH2
	(
	p_ReturnCd           OUT pls_integer
	,p_DebugFlg          IN pls_integer
	,RC1                 OUT db_ref.RCT
	,p_ROUTINGOBJTYPEID  IN NUMBER
	,p_ROUTINGOBJNAME    IN VARCHAR2
	,p_ROUTINGOBJCD      IN VARCHAR2
	,p_ROUTINGOBJVER     IN VARCHAR2
	,p_CHORGSEQNBR       IN NUMBER
	,p_ROUTINGOBJDESC    IN VARCHAR2
	,p_ACTIVESTATUSCD    IN NUMBER
	,p_VISIBILITYCD      IN NUMBER
	,p_DOCUMENTFLG       IN NUMBER
   ,p_PROTOCOLTYPE      IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGOBJECTI_AH2
	Create Date:     2003-11-21 14:10:29
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-11-21 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        NUMBER := 0;

v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_ROUTINGOBJECTI_AH2';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJTYPEID=>'||p_ROUTINGOBJTYPEID||CHR(10)
		||', p_ROUTINGOBJNAME=>'||p_ROUTINGOBJNAME||CHR(10)
		||', p_ROUTINGOBJCD=>'||p_ROUTINGOBJCD||CHR(10)
		||', p_ROUTINGOBJVER=>'||p_ROUTINGOBJVER||CHR(10)
		||', p_CHORGSEQNBR=>'||p_CHORGSEQNBR||CHR(10)
		||', p_ROUTINGOBJDESC=>'||p_ROUTINGOBJDESC||CHR(10)
		||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
		||', p_VISIBILITYCD=>'||p_VISIBILITYCD||CHR(10)
		||', p_DOCUMENTFLG=>'||p_DOCUMENTFLG||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	INSERT INTO CF_ROUTINGOBJECT(
		ROUTINGOBJID
		,ROUTINGOBJTYPEID
		,ROUTINGOBJNAME
		,ROUTINGOBJCD
		,ROUTINGOBJVER
		,CHORGSEQNBR
		,ROUTINGOBJDESC
		,ACTIVESTATUSCD
		,VISIBILITYCD
		,DOCUMENTFLG
      ,PROTOCOLTYPE
	) VALUES (
		CF_ROUTINGOBJECT_SEQ.nextval
		,p_ROUTINGOBJTYPEID
		,p_ROUTINGOBJNAME
		,p_ROUTINGOBJCD
		,p_ROUTINGOBJVER
		,p_CHORGSEQNBR
		,p_ROUTINGOBJDESC
		,p_ACTIVESTATUSCD
		,p_VISIBILITYCD
		,p_DOCUMENTFLG
      ,p_PROTOCOLTYPE
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN CLOSE RC1; END IF;
	OPEN RC1 FOR SELECT 
		CF_ROUTINGOBJECT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJECTI_AH2' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJECTI_AH2;
/


-- CF_ROUTINGOBJECTI_AH3.sql

CREATE OR REPLACE PROCEDURE CF_RoutingObjectI_AH3
	(
	p_ROUTINGOBJTYPEKEY in VARCHAR2
	,p_ROUTINGOBJCD in VARCHAR2
	,p_ROUTINGOBJVER in VARCHAR2
	,p_ROUTINGOBJNAME in VARCHAR2
	,p_ROUTINGOBJDESC in VARCHAR2
	,p_CHORGSEQNBR in NUMBER
	,p_ACTIVESTATUSCD in NUMBER
	,p_VISIBILITYCD in NUMBER
	,p_DocumentFlg in number
	,p_PARENTROTYPEKEY in VARCHAR2
	,p_PARENTROCD in VARCHAR2
	,p_PARENTROVER in VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_RoutingObjectI_AH3
	Create Date:     2007-01-03 15:24:38
	Author:          S.Priani 

	Desc:	*Inserts metadata for RoutingObject into  CF_RoutingObject
		*Inserts metadata into CF_RoutingObjRef_Lnk such that
		- Links the RO to given parentRO 
		- needs to be invoked for each "RO -> ParentRO" combination
		*Server Output lists the RO and its ParentRefIds

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-01-03 S.Priani	 6.1	CR#103555

	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

	v_RoutingObjId	number := Null;
	v_RoutingObjRefId number := Null;
	v_RoutingObjParentRefId number;


BEGIN

	BEGIN
		SELECT	ROUTINGOBJID
		INTO	v_RoutingObjId
		From	CF_RoutingObject o
		INNER JOIN CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
		Where	t.ROUTINGOBJTYPEKEY  = p_RoutingObjTYPEKEY
		AND	o.ROUTINGOBJCD = p_ROUTINGOBJCD
		AND	o.ROUTINGOBJVER = p_ROUTINGOBJVER;
	EXCEPTION
		When No_Data_Found then
			Insert into CF_RoutingObject(
				ROUTINGOBJID
				,ROUTINGOBJTYPEID
				,ROUTINGOBJNAME
				,ROUTINGOBJCD
				,ROUTINGOBJVER
				,CHORGSEQNBR
				,ROUTINGOBJDESC
				,ACTIVESTATUSCD
				,VISIBILITYCD
				,DocumentFlg
			) (SELECT
				CF_ROUTINGOBJECT_SEQ.nextval
				,ROUTINGOBJTYPEID
				,p_ROUTINGOBJNAME
				,p_ROUTINGOBJCD
				,p_ROUTINGOBJVER
				,p_CHORGSEQNBR
				,p_ROUTINGOBJDESC
				,p_ACTIVESTATUSCD
				,p_VISIBILITYCD
				,p_DocumentFlg
				FROM	cf_RoutingObjType
				WHERE	RoutingObjTypeKey = p_ROUTINGOBJTYPEKEY
			);

			SELECT	 CF_ROUTINGOBJECT_SEQ.currval INTO v_RoutingObjId FROM DUAL;

	END;

	dbms_output.put_line('ROId:'||NVL(TO_CHAR(v_RoutingObjId),'NULL')||':'||p_RoutingObjTYPEKEY||'$'||p_ROUTINGOBJCD||'@'||p_ROUTINGOBJVER);

	IF p_ParentROTypeKey IS NOT NULL THEN
		FOR rX IN (
			Select	l.RoutingObjRefId
			From	CF_RoutingObjRef_Lnk l
			INNER JOIN CF_RoutingObject o on l.RoutingObjId = o.RoutingObjId
			INNER JOIN CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
			Where	t.ROUTINGOBJTYPEKEY  = p_PARENTROTYPEKEY
			AND	o.ROUTINGOBJCD = p_PARENTROCD
			AND	o.ROUTINGOBJVER = p_PARENTROVER) 
		LOOP
			v_RoutingObjParentRefId := rX.RoutingObjRefId;
		BEGIN
			INSERT INTO CF_RoutingObjRef_lnk(
				 RoutingObjRefId
				,RoutingObjId
				,ROUTINGOBJPARENTREFID
			) VALUES (
				 CF_RoutingObjRef_lnk_SEQ.nextVal
				,v_RoutingObjId
				,v_RoutingObjParentRefId
			)
			RETURNING  RoutingObjRefId into v_RoutingObjRefId;
		EXCEPTION
			When DUP_VAL_ON_INDEX THEN
				SELECT  RoutingObjRefId INTO v_RoutingObjRefId
				FROM	CF_RoutingObjRef_lnk
				WHERE	RoutingObjId = v_RoutingObjId
				AND	ROUTINGOBJPARENTREFID = v_RoutingObjParentRefId
				;
		END;
			dbms_output.put_line('- RefID:' || v_RoutingObjRefID || '->ParentRef:'||v_RoutingObjParentRefId);

		END LOOP;
	ELSE
		v_RoutingObjParentRefId := Null;
		BEGIN
			INSERT INTO CF_RoutingObjRef_lnk(
				 RoutingObjRefId
				,RoutingObjId
				,ROUTINGOBJPARENTREFID
			) VALUES (
				 CF_RoutingObjRef_lnk_SEQ.nextVal
				,v_RoutingObjId
				,v_RoutingObjParentRefId
			)
			RETURNING  RoutingObjRefId into v_RoutingObjRefId;
		EXCEPTION
			When DUP_VAL_ON_INDEX THEN
				SELECT  RoutingObjRefId INTO v_RoutingObjRefId
				FROM	CF_RoutingObjRef_lnk
				WHERE	RoutingObjId = v_RoutingObjId
				AND	ROUTINGOBJPARENTREFID IS NULL
				;
		END;
		dbms_output.put_line('-- RefId:' || v_RoutingObjRefId);
	END IF;

END CF_RoutingObjectI_AH3;
/


-- CF_ROUTINGOBJECTS_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_RoutingObjectS_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in Varchar2
    , p_RoutingObjTypeKey in Varchar2
    )
AS
/*
    ================================================================================
    Proc Name:       CF_RoutingObjectS_AH1
    Create Date:     2003-04-22
    Author:          J.Chen
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns all routing objects by given RoutingObjType
    2003-05-02 J.Chen        3.5    Changed number to varchar2 for p_RoutingObjTypeKey
    2003-05-06 J.Chen        3.5    Changed p_locale to v_locale for null checking on p_locale.
    2003-05-29 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12 J.Chen        3.5    Changed CF_RoutingObjType_vw to CF_RoutingObjType_Loc_vw
    2003-07-19 J.Chen        3.5    Added DocumentFlg column
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_RoutingObjectS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||','||p_Locale||CHR(10)
        ||', p_ROUTINGOBJTypeKey=>'||p_ROUTINGOBJTypeKey||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
         l.RoutingObjRefId
         ,l.RoutingObjParentRefId
         ,b.ROUTINGOBJID
         ,v.RoutingObjLevel
         ,v.RESOURCEKEY
         ,v.RoutingObjTypeId
         ,v.RoutingObjTypeKey
         ,v.Description
         ,RTRIM(b.ROUTINGOBJNAME) ROUTINGOBJNAME
         ,RTRIM(b.ROUTINGOBJCD) ROUTINGOBJCD
         ,RTRIM(b.ROUTINGOBJVER) ROUTINGOBJVER
         ,b.CHORGSEQNBR
         ,RTRIM(b.ROUTINGOBJDESC) ROUTINGOBJDESC
         ,b.ACTIVESTATUSCD
         ,b.VISIBILITYCD
         ,b.DocumentFlg
         ,b.RowTS
    From
         CF_ROUTINGOBJECT b
         Inner Join CF_RoutingObjRef_Lnk l On l.RoutingObjId = b.RoutingObjId
         Inner Join CF_RoutingObjType v On b.RoutingObjTypeId = v.RoutingObjTypeId
    Where
         v.ROUTINGOBJTypeKey = p_ROUTINGOBJTypeKey
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --  Commit;
    EXCEPTION
         When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR (-20999,'20000-CF_RoutingObjectS_AH1' || SQLCODE|| ' '||SQLERRM);
End CF_RoutingObjectS_AH1;
/


-- CF_ROUTINGOBJECTS_AH10.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJECTS_AH10
        (
        p_ReturnCd          OUT  PLS_INTEGER
        , p_DebugFlg        IN  PLS_INTEGER
        , RC1               OUT  DB_REF.RCT
        , p_FrProtocolName  IN Varchar2
        , p_FrProtocolVer   IN Varchar2
        , p_FrProcessCD     IN Varchar2
        , p_FrActionCD1     IN Varchar2
        , p_FrActionCD2     IN Varchar2
        )
AS
/*
    ================================================================================
    Proc Name:       CF_ROUTINGOBJECTS_AH10
    Create Date:     2003-07-25 14:30:16
    Author:          V.Goel
    This Procedure finds a web service process, activity, action and optional `OneWay?
    attribute associated with the activity (operation) for SOAP channel parsing.
    Sometimes the action has its own namespace but often the namespace is only
    in the process routing object and the action inherits it.
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ROUTINGOBJECTS_AH10';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
                'p_DebugFlg=>'          ||p_DebugFlg      ||CHR(10)
                ||', p_FrProtocolName=>'||p_FrProtocolName||CHR(10)
                ||', p_FrProtocolVer=>' ||p_FrProtocolVer ||CHR(10)
                ||', p_FrProcessCD=>'   ||p_FrProcessCD   ||CHR(10)
                ||', p_FrActionCD1=>'   ||p_FrActionCD1   ||CHR(10)
                ||', p_FrActionCD2=>'   ||p_FrActionCD2   ||CHR(10)
                ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    select
        ro_process.routingobjcd  as ProcessCode,
        ro_process.routingobjver as ProcessVersion,
        ro_activity.routingobjcd as ActivityCode,
        ro_action.routingobjcd   as ActionCode,
        rao.attrvaluekey         as IsOneWay
    from cf_routingObject ro_protocol
        inner join cf_routingobjType rot_protocol    on ro_protocol.routingobjtypeid       = rot_protocol.routingobjtypeid
        inner join cf_routingobjref_lnk rol_protocol on ro_protocol.routingobjid           = rol_protocol.routingobjid
        inner join cf_routingobjref_lnk rol_process  on rol_process.routingobjparentrefid  = rol_protocol.routingobjrefid
        inner join cf_routingObject ro_process       on ro_process.routingobjid            = rol_process.routingobjid
        inner join cf_routingobjType rot_process     on ro_process.routingobjtypeid        = rot_process.routingobjtypeid
        inner join cf_routingobjref_lnk rol_activity on rol_activity.routingobjparentrefid = rol_process.routingobjrefid
        inner join cf_routingObject ro_activity      on ro_activity.routingobjid           = rol_activity.routingobjid
        inner join cf_routingobjType rot_activity    on ro_activity.routingobjtypeid       = rot_activity.routingobjtypeid
        inner join cf_routingobjref_lnk rol_action   on rol_action.routingobjparentrefid   = rol_activity.routingobjrefid
        inner join cf_routingObject ro_action        on ro_action.routingobjid             = rol_action.routingobjid
        inner join cf_routingobjType rot_action      on ro_action.routingobjtypeid         = rot_action.routingobjtypeid
        left outer join pr_routingattr_lnk ral       on ral.routingobjrefid                = rol_activity.routingobjrefid
        left outer join cf_routingattr ra            on ral.routingattrid                  = ra.routingattrid
                                                    and ra.attributekey                    = 'OneWay'
        left outer join cf_routingattropt rao        on rao.attroptid = ral.attroptid
    where rot_protocol.routingobjtypekey     = 'Protocol'
        and   rot_process.routingobjtypekey  = 'Process'
        and   rot_activity.routingobjtypekey = 'Activity'
        and   rot_action.routingobjtypekey   = 'Action'
        and   ro_protocol.routingobjcd       = p_FrProtocolName
        and ro_protocol.routingobjver        = p_FrProtocolVer
        and (
                (ro_process.routingobjcd like p_FrProcessCD and ro_action.routingobjcd = p_FrActionCD1)
              or ro_action.routingobjcd  = p_FrActionCD2
            )
        ;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ROUTINGOBJECTS_AH10;
/


-- CF_ROUTINGOBJECTS_AH11.sql

CREATE OR REPLACE PROCEDURE CF_RoutingObjectS_AH11(
    p_ReturnCd                OUT INTEGER
    , p_DebugFlg              IN  PLS_INTEGER
    , RC1                     OUT db_ref.RCT
    , p_Locale                IN  VARCHAR2
    , p_ParentRoutingObjRefId IN  NUMBER
    )
AS
/*
    ================================================================================
    Proc Name:       CF_RoutingObjectS_AH11
    Create Date:     2003-04-22 10:07:31
    Author:          J.Chen
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE        NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22  J.Chen        3.5    Returns the direct children of given routing objects.
    2003-05-02  J.Chen        3.5    Changed p_ParentRoutingObjId to p_ParentRoutingObjRefId.
    2003-05-06  J.Chen        3.5    Changed p_locale to v_locale for null checking on p_locale.
    2003-05-29  L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12  J.Chen        3.5    Changed CF_RoutingObjType_vw to CF_RoutingObjType_Loc_vw
    2003-07-19  J.Chen        3.5    Added DocumentFlg column in result set
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          INTEGER :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) := 'CF_RoutingObjectS_AH11';
v_ExceptionFlg    INTEGER := 0;
v_ParamsTxt       VARCHAR2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||'p_Locale=>'|| p_Locale || CHR(10)
        ||', p_ParentRoutingObjRefId=>'||p_ParentRoutingObjRefId||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%isOpen THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    SELECT
        l.RoutingObjRefId
        ,l.RoutingObjParentRefId
        ,b.ROUTINGOBJID
        ,v.RoutingObjLevel
        ,v.RoutingObjTypeId
        ,v.ROUTINGOBJTYPEKey
        ,v.RESOURCEKEY
        ,v.Description
        ,RTRIM(b.ROUTINGOBJNAME) ROUTINGOBJNAME
        ,RTRIM(b.ROUTINGOBJCD) ROUTINGOBJCD
        ,RTRIM(b.ROUTINGOBJVER) ROUTINGOBJVER
        ,b.CHORGSEQNBR
        ,RTRIM(b.ROUTINGOBJDESC) ROUTINGOBJDESC
        ,b.ACTIVESTATUSCD
        ,b.VISIBILITYCD
        ,b.DocumentFlg
        ,b.RowTS
        ,b.protocoltype
    FROM
        CF_RoutingObjRef_Lnk l
        Inner Join CF_ROUTINGOBJECT b ON b.RoutingObjId = l.RoutingObjId
        Inner Join CF_RoutingObjType v ON v.RoutingObjTypeId = b.RoutingObjTypeId
    WHERE 
          l.RoutingObjParentRefId = p_ParentRoutingObjRefId
    AND   l.RoutingObjRefId != RoutingObjParentRefId
    AND   b.protocoltype = 1;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --  Commit;
    EXCEPTION
         WHEN OTHERS THEN
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR (-20999,'20000-CF_RoutingObjectS_AH11' || SQLCODE|| ' '||SQLERRM);
END CF_RoutingObjectS_AH11;
/


-- cf_routingobjects_ah12.sql

 CREATE OR REPLACE PROCEDURE cf_routingobjects_ah12
	(
	  p_ReturnCd     OUT PLS_INTEGER
	, p_DebugFlg     IN  PLS_INTEGER
	, RC1            OUT DB_REF.RCT                                                                      
	, p_syntax       IN  VARCHAR2
	)                                                                              
AS                                                                              

/* 
   This proc gets packages based on syntax
                                                                            
	================================================================================
	Proc Name:       cf_routingobjects_ah12                                             
	Create Date:     2005-12-05 13:30:03                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-05 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'cf_routingobjects_ah12';                     
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_syntax=>'||p_syntax||CHR(10)                               
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          

   OPEN RC1 FOR 
	SELECT ro.routingobjname
      ,ro.routingobjver
		,ro.routingobjid
      ,NVL((CASE WHEN (SELECT pkgid 
                       FROM dis_imp_edi_syntax_def_pkg def
                       WHERE def.pkgid = ro.routingobjid
                           AND def.syntax = p_syntax) 
                 IS NOT NULL THEN (SELECT 1 FROM dual)END), 0) defaultflag
	FROM cf_routingobject ro,
        cf_routingobjtype rotype
	WHERE ro.routingobjid NOT IN (SELECT DISTINCT pkgid
                                 FROM dis_imp_edi_syntax_pkg_lnk lnk
                                 WHERE lnk.syntax = p_syntax)
   AND rotype.routingobjtypeid = ro.routingobjtypeid
   AND rotype.routingobjtypekey ='Package'
	;                                                                              

   --hardcoded sql above because p_syntax will never be null
	  
   p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                    
END cf_routingobjects_ah12;
/


-- cf_routingobjectS_ah13.sql

CREATE OR REPLACE PROCEDURE cf_routingobjectS_ah13                                   
	(
     p_ReturnCd     OUT PLS_INTEGER
	, p_DebugFlg     IN  PLS_INTEGER
	, RC1            OUT DB_REF.RCT                                                                      
	, p_syntax       IN  NUMBER                                                     
	)                                                                              
AS                                                                              

/*                                                                             
   This proc lists protocols given a syntax

	================================================================================
	Proc Name:       cf_routingobjectS_ah13                                             
	Create Date:     2005-12-09 10:08:05                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-09 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'cf_routingobjectS_ah13';                     
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_syntax=>'||p_syntax||CHR(10)                               
		;
v_sqlstmt         VARCHAR2(500);
v_where           VARCHAR2(500);
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

--assuming that the protocoltypeid column in CF_ROUTINGOBJECT contains the syntax

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          

   v_sqlstmt := 'SELECT routingobjid, routingobjname, routingobjver '||
                'FROM cf_routingobject ';
   v_where   := 'WHERE routingobjtypeid = 2 '
                ||FN_BUILD_WHERE('PROTOCOLTYPE', 'NUMBER', TO_CHAR(p_syntax));

   v_sqlstmt := v_sqlstmt || v_where;

   OPEN RC1 FOR v_sqlstmt;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt,p_DebugFlg, v_ExceptionFlg);
END cf_routingobjectS_ah13;
/


-- cf_routingobjectS_ah14.sql

CREATE OR REPLACE PROCEDURE cf_routingobjectS_ah14                                   
	(
	  p_ReturnCd  OUT PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1         OUT DB_REF.RCT                                                                      
	, p_syntax    IN NUMBER                                                     
	)                                                                              
AS                                                                              

/*                                                                             
   This procedure list doctypes given a protocol id

	================================================================================
	Proc Name:       cf_routingobjectS_ah14                                             
	Create Date:     2005-12-09 10:16:11                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-09 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'cf_routingobjectS_ah14';                     
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_syntax=>'||p_syntax||CHR(10)                               
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          

   --assuming that the protocoltypeid column in CF_ROUTINGOBJECT contains the syntax
   --lnk1 gets the refid for the routingobject of p_syntax
   --lnk2 gets the refid for the routingobjects who have p_syntax as a parent
   --lnk3 gets the routingobjids for the routingobjects who have p_syntax as a parent

	OPEN RC1 FOR 
      SELECT routingobjid, routingobjname, routingobjver
      FROM cf_routingobject
      WHERE routingobjid IN (SELECT routingobjid
                             FROM cf_routingobjref_lnk lnk3
                             WHERE lnk3.routingobjrefid IN (SELECT lnk2.routingobjrefid
                                                            FROM cf_routingobjref_lnk lnk2
                                                            WHERE lnk2.routingobjparentrefid IN (SELECT lnk1.routingobjrefid
                                                                                                 FROM cf_routingobjref_lnk lnk1
                                                                                                 WHERE lnk1.routingobjid = p_syntax)));
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END cf_routingobjectS_ah14;
/


-- CF_ROUTINGOBJECTS_AH3.sql

CREATE OR REPLACE PROCEDURE         CF_RoutingObjectS_AH3(
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in Varchar2
    , p_ParentRoutingObjRefId in NUMBER
    )
AS
/*
    ================================================================================
    Proc Name:       CF_RoutingObjectS_AH3
    Create Date:     2003-04-22 10:07:31
    Author:          J.Chen
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE        NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22  J.Chen        3.5    Returns the direct children of given routing objects.
    2003-05-02  J.Chen        3.5    Changed p_ParentRoutingObjId to p_ParentRoutingObjRefId.
    2003-05-06  J.Chen        3.5    Changed p_locale to v_locale for null checking on p_locale.
    2003-05-29  L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12  J.Chen        3.5    Changed CF_RoutingObjType_vw to CF_RoutingObjType_Loc_vw
    2003-07-19  J.Chen        3.5    Added DocumentFlg column in result set
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_RoutingObjectS_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||'p_Locale=>'|| p_Locale || CHR(10)
        ||', p_ParentRoutingObjRefId=>'||p_ParentRoutingObjRefId||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 For
    Select
        l.RoutingObjRefId
        ,l.RoutingObjParentRefId
        ,b.ROUTINGOBJID
        ,v.RoutingObjLevel
        ,v.RoutingObjTypeId
        ,v.ROUTINGOBJTYPEKey
        ,v.RESOURCEKEY
        ,v.Description
        ,RTRIM(b.ROUTINGOBJNAME) ROUTINGOBJNAME
        ,RTRIM(b.ROUTINGOBJCD) ROUTINGOBJCD
        ,RTRIM(b.ROUTINGOBJVER) ROUTINGOBJVER
        ,b.CHORGSEQNBR
        ,RTRIM(b.ROUTINGOBJDESC) ROUTINGOBJDESC
        ,b.ACTIVESTATUSCD
        ,b.VISIBILITYCD
        ,b.DocumentFlg
        ,b.RowTS
    From
        CF_RoutingObjRef_Lnk l
        Inner Join CF_ROUTINGOBJECT b On b.RoutingObjId = l.RoutingObjId
        Inner Join CF_RoutingObjType v On v.RoutingObjTypeId = b.RoutingObjTypeId
    Where 
          l.RoutingObjParentRefId = p_ParentRoutingObjRefId
    And   l.RoutingObjRefId != RoutingObjParentRefId;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --  Commit;
    EXCEPTION
         When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR (-20999,'20000-CF_RoutingObjectS_AH3' || SQLCODE|| ' '||SQLERRM);
End CF_RoutingObjectS_AH3;
/


-- CF_ROUTINGOBJECTS_AH5.sql

CREATE OR REPLACE PROCEDURE CF_RoutingObjectS_AH5(
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in Varchar2
    , p_RoutingObjRefId in NUMBER
    )
AS
/*
    ================================================================================
    Proc Name:       CF_RoutingObjectS_AH5
    Create Date:     2003-05-02 10:07:31
    Author:          J.Chen
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-02 J.Chen        3.5    Returns the routing object info by RoutingObjRefId
    2003-05-06 J.Chen        3.5    Changed p_locale to v_locale for null checking on p_locale.
    2003-05-29 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12 J.Chen        3.5    Changed CF_RoutingObjType_vw to CF_RoutingObjType_Loc_vw
    2003-07-19 J.Chen        3.5    Added DocumentFlg to result set
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_RoutingObjectS_AH5';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||'p_Locale=>'|| p_Locale || CHR(10)
        ||', p_RoutingObjRefId=>'||p_RoutingObjRefId||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 For
    Select
        l.RoutingObjRefId
        ,l.RoutingObjParentRefId
        ,b.ROUTINGOBJID
        ,v.RoutingObjLevel
        ,v.RoutingObjTypeId
        ,v.ROUTINGOBJTYPEKey
        ,v.RESOURCEKEY
        ,v.Description
        ,RTRIM(b.ROUTINGOBJNAME) ROUTINGOBJNAME
        ,RTRIM(b.ROUTINGOBJCD) ROUTINGOBJCD
        ,RTRIM(b.ROUTINGOBJVER) ROUTINGOBJVER
        ,b.CHORGSEQNBR
        ,RTRIM(b.ROUTINGOBJDESC) ROUTINGOBJDESC
        ,b.ACTIVESTATUSCD
        ,b.VISIBILITYCD
        ,b.DocumentFlg
        ,b.RowTS
    From
        CF_ROUTINGOBJECT b
        Inner Join CF_RoutingObjRef_Lnk l On l.RoutingObjId = b.RoutingObjId
        Inner Join CF_RoutingObjType v On v.RoutingObjTypeId = b.RoutingObjTypeId
    Where    l.RoutingObjRefId = p_RoutingObjRefId;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --   Commit;
    EXCEPTION
         When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR (-20999,'20000-CF_RoutingObjectS_AH5' || SQLCODE|| ' '||SQLERRM);
End CF_RoutingObjectS_AH5;
/


-- CF_ROUTINGOBJECTS_AH6.sql

CREATE OR REPLACE PROCEDURE CF_RoutingObjectS_AH6
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in Varchar2
    , p_RoutingObjId in NUMBER
    )
AS
/*
    ================================================================================
    Proc Name:       CF_RoutingObjectS_AH6
    Create Date:     2003-05-02 10:07:31
    Author:          J.Chen
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-06 J.Chen        3.5    Returns the routing object Ref info by RoutingObjId
    2003-05-29 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12 J.Chen        3.5    Changed CF_RoutingObjType_vw to CF_RoutingObjType_Loc_vw
    2003-07-19 J.Chen        3.5    Added Documentflg in result set
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_RoutingObjectS_AH6';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||'p_Locale=>'|| p_Locale || CHR(10)
        ||', p_RoutingObjId=>'||p_RoutingObjId||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 For
    Select
        l.RoutingObjRefId
        ,l.RoutingObjParentRefId
        ,b.ROUTINGOBJID
        ,v.RoutingObjLevel
        ,v.RoutingObjTypeId
        ,v.ROUTINGOBJTYPEKey
        ,v.Resourcekey
        ,v.Description
        ,RTRIM(b.ROUTINGOBJNAME) ROUTINGOBJNAME
        ,RTRIM(b.ROUTINGOBJCD) ROUTINGOBJCD
        ,RTRIM(b.ROUTINGOBJVER) ROUTINGOBJVER
        ,b.CHORGSEQNBR
        ,RTRIM(b.ROUTINGOBJDESC) ROUTINGOBJDESC
        ,b.ACTIVESTATUSCD
        ,b.VISIBILITYCD
        ,b.DocumentFlg
        ,b.RowTS
    From
        CF_ROUTINGOBJECT b
        Inner Join CF_RoutingObjRef_Lnk l On l.RoutingObjId = b.RoutingObjId
        Inner Join CF_RoutingObjType v On v.RoutingObjTypeId = b.RoutingObjTypeId
    Where   l.RoutingObjId = p_RoutingObjId
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --  Commit;
    EXCEPTION
         When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR (-20999,'20000-CF_RoutingObjectS_AH6' || SQLCODE|| ' '||SQLERRM);
End CF_RoutingObjectS_AH6;
/


-- CF_ROUTINGOBJECTS_AH7.sql

CREATE OR REPLACE PROCEDURE         CF_ROUTINGOBJECTS_AH7
    (
    p_ReturnCd  out  integer
    ,p_DebugFlg  in  pls_integer
    ,RC1  out  db_ref.RCT
    ,p_Locale in varchar2
    ,p_ROUTINGOBJID in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_ROUTINGOBJECTS_AH7
    Create Date:     2003-04-22 10:07:31
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 SYSTEM        3.5    Created.
    2003-05-29 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12 J.Chen        3.5    Changed CF_RoutingObjType_vw to CF_RoutingObjType_Loc_vw
    2004-03-05 S.Narain      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ROUTINGOBJECTS_AH7';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
        o.ROUTINGOBJID
        ,o.ROUTINGOBJTYPEID
        ,RTRIM(o.ROUTINGOBJNAME) ROUTINGOBJNAME
        ,RTRIM(o.ROUTINGOBJCD) ROUTINGOBJCD
        ,RTRIM(o.ROUTINGOBJVER) ROUTINGOBJVER
        ,o.CHORGSEQNBR
        ,RTRIM(o.ROUTINGOBJDESC) ROUTINGOBJDESC
        ,o.ACTIVESTATUSCD
        ,o.VISIBILITYCD
        ,o.DOCUMENTFLG
        ,o.ROWTS
        ,v.RoutingObjLevel
        ,v.RoutingObjTypeKey
        ,v.RESOURCEKEY
        ,v.DESCRIPTION
    From
        CF_ROUTINGOBJECT o
        Inner Join CF_RoutingObjType v On v.RoutingObjTypeId = o.RoutingObjTypeId
    Where
        ROUTINGOBJID = p_ROUTINGOBJID
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ROUTINGOBJECTS_AH7;
/


-- CF_ROUTINGOBJECTS_AH8.sql

CREATE OR REPLACE PROCEDURE         CF_ROUTINGOBJECTS_AH8
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_GuidelineMapId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_ROUTINGOBJECTS_AH8
    Create Date:     2003-04-22 10:07:31
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 SYSTEM           3.5 Returns list of objects Id's based on MapId passed

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ROUTINGOBJECTS_AH8';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_GuidelineMapId=>'||p_GuidelineMapId||CHR(10)
        ;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
        o.ROUTINGOBJID
      From
        CF_ROUTINGOBJECT o
        Inner Join CF_GuidelineMap_lnk l on l.RoutingObjId = o.RoutingObjId
    Where
        l.GuidelineMapId = p_GuidelineMapId
  --  And v.Locale = v_Locale
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ROUTINGOBJECTS_AH8;
/


-- CF_ROUTINGOBJECTS_AH9.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJECTS_AH9
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        , p_Locale   In Varchar2
        , p_ROUTINGOBJTypeKey IN Varchar2
        )
AS
/*
        ================================================================================
        Proc Name:       CF_ROUTINGOBJECTS_AH9
        Create Date:     2003-05-27 14:30:16
        Author:          SYSTEM
                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::
        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-05-27 SYSTEM           3.5 Created
        2004-03-05 S.Narain       4.2.2  Removed NLS
        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ROUTINGOBJECTS_AH9';
v_ExceptionFlg    integer := 0;
--v_Locale          PR_Locale.locale%Type:= FN_CHK_Default_Locale(p_Locale , 'CF_RoutingObjType_Loc');
v_ParamsTxt       varchar2(4000) :=
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ROUTINGOBJTypeKey=>'||p_ROUTINGOBJTypeKey||CHR(10)
                ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
        IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ROUTINGOBJTypeKey IS NOT NULL
 THEN
        OPEN RC1 FOR
        SELECT
                ROUTINGOBJID
                ,R.ROUTINGOBJTYPEID
                ,RTRIM(ROUTINGOBJNAME) ROUTINGOBJNAME
                ,RTRIM(ROUTINGOBJCD) ROUTINGOBJCD
                ,RTRIM(ROUTINGOBJVER) ROUTINGOBJVER
                ,CHORGSEQNBR
                ,RTRIM(ROUTINGOBJDESC) ROUTINGOBJDESC
                ,ACTIVESTATUSCD
                ,VISIBILITYCD
                ,R.DocumentFlg
                ,R.RowTS
                ,Typ.ROUTINGOBJTypeKey
                ,Typ.RESOURCEKEY
                ,Typ.routingObjLevel
                ,Typ.Description
        FROM
                CF_ROUTINGOBJECT R
                Inner Join CF_RoutingObjType Typ ON R.ROUTINGOBJTYPEID = Typ.ROUTINGOBJTYPEID
        Where
                Typ.ROUTINGOBJTypeKey = p_ROUTINGOBJTypeKey
        ;
        ELSE
        OPEN RC1 FOR
        SELECT
                ROUTINGOBJID
                ,R.ROUTINGOBJTYPEID
                ,RTRIM(ROUTINGOBJNAME) ROUTINGOBJNAME
                ,RTRIM(ROUTINGOBJCD) ROUTINGOBJCD
                ,RTRIM(ROUTINGOBJVER) ROUTINGOBJVER
                ,CHORGSEQNBR
                ,RTRIM(ROUTINGOBJDESC) ROUTINGOBJDESC
                ,ACTIVESTATUSCD
                ,VISIBILITYCD
                ,R.DocumentFlg
                ,R.RowTS
                ,Typ.ROUTINGOBJTypeKey
                ,Typ.RESOURCEKEY
                ,Typ.routingObjLevel
                ,Typ.Description
        FROM
                CF_ROUTINGOBJECT R
                Inner Join CF_RoutingObjType Typ ON R.ROUTINGOBJTYPEID = Typ.ROUTINGOBJTYPEID
        ;
        END IF;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --  Commit;
End CF_ROUTINGOBJECTS_AH9;
/


-- CF_ROUTINGOBJECTS_WEBSRV.sql

CREATE OR REPLACE Procedure CF_ROUTINGOBJECTS_WEBSRV
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	)
AS

/*


	================================================================================
	Proc Name:       CF_ROUTINGOBJECTS_WEBSRV
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECTS_WEBSRV';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) ;

v_where           varchar2(200);
v_sqlstmt	  varchar2(500);


BEGIN


		OPEN RC1 FOR
		 SELECT    
                o.ROUTINGOBJID,
                o.ROUTINGOBJCD Routingobjectname
		FROM cf_routingobject06_vw o
		inner join CF_WEBSERVICE_ATTRVALUE v1 ON v1.wsroutingobjid = o.routingobjid
		inner join CF_WEBSERVICE_ATTR a1 ON v1.wsattrid = a1.wsattrid
		inner join CF_WEBSERVICE_ATTRVALUE v2 ON v2.wsroutingobjid = o.routingobjid
		inner join CF_WEBSERVICE_ATTR a2 ON v2.wsattrid = a2.wsattrid
		WHERE      a1.wsattrkey = 'bcg.webservice.attr.operationType'
		AND        v1.wsattrvalue = '1'
		AND        a2.wsattrkey = 'bcg.webservice.attr.parameterType'
		AND        v2.wsattrvalue IN ('String','ErrorFlowDocumentType')
		AND        o.GRPARENTOBJCD2='Web Service' 
		AND        o.GRPARENTOBJCD3='None';

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 CF_ROUTINGOBJECTS_WEBSRV' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJECTS_WEBSRV' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJECTS_WEBSRV;
/

-- CF_ROUTINGOBJECTU.sql

CREATE OR REPLACE Procedure CF_ROUTINGOBJECTU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_ROUTINGOBJID in NUMBER
	,p_ROUTINGOBJTYPEID in NUMBER
	,p_ROUTINGOBJNAME in VARCHAR2
	,p_ROUTINGOBJCD in VARCHAR2
	,p_ROUTINGOBJVER in VARCHAR2
	,p_CHORGSEQNBR in NUMBER
	,p_ROUTINGOBJDESC in VARCHAR2
	,p_ACTIVESTATUSCD in NUMBER
	,p_VISIBILITYCD in NUMBER
	,p_ROWTS in NUMBER
	,p_DOCUMENTFLG in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_ROUTINGOBJECTU
	Create Date:     2003-09-26 08:20:56
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2008-09-12 Amit Kumar Vijay 6.2	Defect 47980, 106065 and 106066

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECTU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		||', p_ROUTINGOBJTYPEID=>'||p_ROUTINGOBJTYPEID||CHR(10)
		||', p_ROUTINGOBJNAME=>'||p_ROUTINGOBJNAME||CHR(10)
		||', p_ROUTINGOBJCD=>'||p_ROUTINGOBJCD||CHR(10)
		||', p_ROUTINGOBJVER=>'||p_ROUTINGOBJVER||CHR(10)
		||', p_CHORGSEQNBR=>'||p_CHORGSEQNBR||CHR(10)
		||', p_ROUTINGOBJDESC=>'||p_ROUTINGOBJDESC||CHR(10)
		||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
		||', p_VISIBILITYCD=>'||p_VISIBILITYCD||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_DOCUMENTFLG=>'||p_DOCUMENTFLG||CHR(10)
		;

v_RowTS      integer :=0;
v_ACTIVESTATUSCD number;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	
	
	IF p_ACTIVESTATUSCD=0 then 

		Update pr_capability
			set Activestatuscd= CASE WHEN ACTIVESTATUSCD in (2,3) THEN Activestatuscd-2 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		Where capabilityid in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
											chld.ROUTINGOBJPARENTREFID,
											chld.GRPARENTOBJREFID,
											chld.GRPARENTOBJREFID2,
											chld.GRPARENTOBJREFID3));
						
		Update cf_validobjinteract
			set ENABLEDCD =CASE WHEN ENABLEDCD in (2,3,6,7) THEN ENABLEDCD-2 ELSE ENABLEDCD END
				,	updatedate=v_BeginTS
		WHERE INTERACTID in (SELECT INTERACTID FROM CF_VALIDOBJINTERACT cvi, 
					CF_ROUTINGOBJECT06_VW crl1, CF_ROUTINGOBJECT06_VW chld
				     WHERE crl1.routingobjid=p_ROUTINGOBJID
					and cvi.TOROUTINGOBJREFID=chld.routingobjrefid
				     and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));			
			
		Update cf_validobjinteract
			set ENABLEDCD =CASE WHEN ENABLEDCD in (4,5,6,7) THEN ENABLEDCD-4 ELSE ENABLEDCD END
			,	updatedate=v_BeginTS
		WHERE INTERACTID in (SELECT INTERACTID FROM CF_VALIDOBJINTERACT cvi, 
					CF_ROUTINGOBJECT06_VW crl1, CF_ROUTINGOBJECT06_VW chld
			 	     WHERE crl1.routingobjid=p_ROUTINGOBJID
					and cvi.FRROUTINGOBJREFID=chld.routingobjrefid
				     and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));
			

		Update pr_routingchannel
			set ACTIVESTATUSCD=CASE WHEN ACTIVESTATUSCD in (8,9,10,11,12,13,14,15,24,25,26,27,28,29,30,31,40,41,42,43,44,45,46,47,56,57,58,59,60,61,62,63) THEN ACTIVESTATUSCD-8 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		WHERE TOCAPABILITYID in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
		Update pr_routingchannel
			set ACTIVESTATUSCD=CASE WHEN ACTIVESTATUSCD in (16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63) THEN ACTIVESTATUSCD-16 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		WHERE FRCAPABILITYID in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
	ELSE
		
		Update pr_capability
			set Activestatuscd= CASE WHEN ACTIVESTATUSCD in (0,1) THEN Activestatuscd+2 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		Where capabilityid in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
						
		Update cf_validobjinteract
			set ENABLEDCD =CASE WHEN ENABLEDCD in (0,1,4,5) THEN ENABLEDCD+2 ELSE ENABLEDCD END
			,	updatedate=v_BeginTS
		WHERE INTERACTID in (SELECT INTERACTID FROM CF_VALIDOBJINTERACT cvi, 
					CF_ROUTINGOBJECT06_VW crl1, CF_ROUTINGOBJECT06_VW chld
				     WHERE crl1.routingobjid=p_ROUTINGOBJID
					and cvi.TOROUTINGOBJREFID=chld.routingobjrefid
				     and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));			
			
		Update cf_validobjinteract
			set ENABLEDCD =CASE WHEN ENABLEDCD in (0,1,2,3) THEN ENABLEDCD+4 ELSE ENABLEDCD END
			,	updatedate=v_BeginTS
		WHERE INTERACTID in (SELECT distinct(INTERACTID) FROM CF_VALIDOBJINTERACT cvi, 
					CF_ROUTINGOBJECT06_VW crl1, CF_ROUTINGOBJECT06_VW chld
				     WHERE crl1.routingobjid=p_ROUTINGOBJID
					and cvi.FRROUTINGOBJREFID=chld.routingobjrefid
				     and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));
			

			
		Update pr_routingchannel
			set ACTIVESTATUSCD=CASE WHEN ACTIVESTATUSCD in (0,1,2,3,4,5,6,7,16,17,18,19,20,21,22,23,32,33,34,35,36,37,38,39,48,49,50,51,52,53,54,55) THEN ACTIVESTATUSCD+8 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		WHERE TOCAPABILITYID in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
		Update pr_routingchannel
			set ACTIVESTATUSCD=CASE WHEN ACTIVESTATUSCD in (0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47) THEN ACTIVESTATUSCD+16 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		WHERE FRCAPABILITYID in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
	END IF;
	
	Update CF_ROUTINGOBJECT
	SET
		ROUTINGOBJTYPEID=p_ROUTINGOBJTYPEID
		,      ROUTINGOBJNAME=p_ROUTINGOBJNAME
		,      ROUTINGOBJCD=p_ROUTINGOBJCD
		,      ROUTINGOBJVER=p_ROUTINGOBJVER
		,      CHORGSEQNBR=p_CHORGSEQNBR
		,      ROUTINGOBJDESC=p_ROUTINGOBJDESC
		,      ACTIVESTATUSCD=p_ACTIVESTATUSCD
		,      VISIBILITYCD=p_VISIBILITYCD
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      DOCUMENTFLG=p_DOCUMENTFLG
	Where
		ROUTINGOBJID = p_ROUTINGOBJID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_ROUTINGOBJECTU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_ROUTINGOBJECTU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_ROUTINGOBJECTU:'||SQLERRM);
End CF_ROUTINGOBJECTU;
/


-- CF_ROUTINGOBJECTU_AH1.sql

CREATE OR REPLACE Procedure CF_ROUTINGOBJECTU_AH1
	(
	 p_ReturnCd          OUT INTEGER
	,p_DebugFlg          IN PLS_INTEGER
	,RC1                 OUT db_ref.RCT
	,p_ROUTINGOBJID      IN NUMBER
	,p_ROUTINGOBJTYPEID  IN NUMBER
	,p_ROUTINGOBJNAME    IN VARCHAR2
	,p_ROUTINGOBJCD      IN VARCHAR2
	,p_ROUTINGOBJVER     IN VARCHAR2
	,p_CHORGSEQNBR       IN NUMBER
	,p_ROUTINGOBJDESC    IN VARCHAR2
	,p_ACTIVESTATUSCD    IN NUMBER
	,p_VISIBILITYCD      IN NUMBER
	,p_ROWTS             IN NUMBER
	,p_DOCUMENTFLG       IN NUMBER
   ,p_PROTOCOLTYPE      IN NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_ROUTINGOBJECTU_AH1
	Create Date:     2003-09-26 08:20:56
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2008-09-12 Amit Kumar Vijay 6.2	Defect 47980, 106065 and 106066

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJECTU_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		||', p_ROUTINGOBJTYPEID=>'||p_ROUTINGOBJTYPEID||CHR(10)
		||', p_ROUTINGOBJNAME=>'||p_ROUTINGOBJNAME||CHR(10)
		||', p_ROUTINGOBJCD=>'||p_ROUTINGOBJCD||CHR(10)
		||', p_ROUTINGOBJVER=>'||p_ROUTINGOBJVER||CHR(10)
		||', p_CHORGSEQNBR=>'||p_CHORGSEQNBR||CHR(10)
		||', p_ROUTINGOBJDESC=>'||p_ROUTINGOBJDESC||CHR(10)
		||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
		||', p_VISIBILITYCD=>'||p_VISIBILITYCD||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_DOCUMENTFLG=>'||p_DOCUMENTFLG||CHR(10)
		;

v_RowTS      integer :=0;
v_ACTIVESTATUSCD number;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	
	
	IF p_ACTIVESTATUSCD=0 then 

		Update pr_capability
			set Activestatuscd= CASE WHEN ACTIVESTATUSCD in (2,3) THEN Activestatuscd-2 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		Where capabilityid in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
											chld.ROUTINGOBJPARENTREFID,
											chld.GRPARENTOBJREFID,
											chld.GRPARENTOBJREFID2,
											chld.GRPARENTOBJREFID3));
						
		Update cf_validobjinteract
			set ENABLEDCD =CASE WHEN ENABLEDCD in (2,3,6,7) THEN ENABLEDCD-2 ELSE ENABLEDCD END
				,	updatedate=v_BeginTS
		WHERE INTERACTID in (SELECT INTERACTID FROM CF_VALIDOBJINTERACT cvi, 
					CF_ROUTINGOBJECT06_VW crl1, CF_ROUTINGOBJECT06_VW chld
				     WHERE crl1.routingobjid=p_ROUTINGOBJID
					and cvi.TOROUTINGOBJREFID=chld.routingobjrefid
				     and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));			
			
		Update cf_validobjinteract
			set ENABLEDCD =CASE WHEN ENABLEDCD in (4,5,6,7) THEN ENABLEDCD-4 ELSE ENABLEDCD END
			,	updatedate=v_BeginTS
		WHERE INTERACTID in (SELECT INTERACTID FROM CF_VALIDOBJINTERACT cvi, 
					CF_ROUTINGOBJECT06_VW crl1, CF_ROUTINGOBJECT06_VW chld
			 	     WHERE crl1.routingobjid=p_ROUTINGOBJID
					and cvi.FRROUTINGOBJREFID=chld.routingobjrefid
				     and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));
			

		Update pr_routingchannel
			set ACTIVESTATUSCD=CASE WHEN ACTIVESTATUSCD in (8,9,10,11,12,13,14,15,24,25,26,27,28,29,30,31,40,41,42,43,44,45,46,47,56,57,58,59,60,61,62,63) THEN ACTIVESTATUSCD-8 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		WHERE TOCAPABILITYID in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
		Update pr_routingchannel
			set ACTIVESTATUSCD=CASE WHEN ACTIVESTATUSCD in (16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63) THEN ACTIVESTATUSCD-16 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		WHERE FRCAPABILITYID in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
	ELSE
		
		Update pr_capability
			set Activestatuscd= CASE WHEN ACTIVESTATUSCD in (0,1) THEN Activestatuscd+2 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		Where capabilityid in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
						
		Update cf_validobjinteract
			set ENABLEDCD =CASE WHEN ENABLEDCD in (0,1,4,5) THEN ENABLEDCD+2 ELSE ENABLEDCD END
			,	updatedate=v_BeginTS
		WHERE INTERACTID in (SELECT INTERACTID FROM CF_VALIDOBJINTERACT cvi, 
					CF_ROUTINGOBJECT06_VW crl1, CF_ROUTINGOBJECT06_VW chld
				     WHERE crl1.routingobjid=p_ROUTINGOBJID
					and cvi.TOROUTINGOBJREFID=chld.routingobjrefid
				     and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));			
			
		Update cf_validobjinteract
			set ENABLEDCD =CASE WHEN ENABLEDCD in (0,1,2,3) THEN ENABLEDCD+4 ELSE ENABLEDCD END
			,	updatedate=v_BeginTS
		WHERE INTERACTID in (SELECT distinct(INTERACTID) FROM CF_VALIDOBJINTERACT cvi, 
					CF_ROUTINGOBJECT06_VW crl1, CF_ROUTINGOBJECT06_VW chld
				     WHERE crl1.routingobjid=p_ROUTINGOBJID
					and cvi.FRROUTINGOBJREFID=chld.routingobjrefid
				     and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));
			

			
		Update pr_routingchannel
			set ACTIVESTATUSCD=CASE WHEN ACTIVESTATUSCD in (0,1,2,3,4,5,6,7,16,17,18,19,20,21,22,23,32,33,34,35,36,37,38,39,48,49,50,51,52,53,54,55) THEN ACTIVESTATUSCD+8 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		WHERE TOCAPABILITYID in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
		Update pr_routingchannel
			set ACTIVESTATUSCD=CASE WHEN ACTIVESTATUSCD in (0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47) THEN ACTIVESTATUSCD+16 ELSE ACTIVESTATUSCD END
			,	updatedate=v_BeginTS
		WHERE FRCAPABILITYID in (select c.capabilityid from PR_capability c, CF_ROUTINGOBJECT06_VW chld, CF_ROUTINGOBJECT06_VW crl1 
						WHERE crl1.routingobjid=p_ROUTINGOBJID
					and c.routingobjrefid=chld.routingobjrefid
						and crl1.routingobjrefid IN (chld.ROUTINGOBJREFID,
										chld.ROUTINGOBJPARENTREFID,
										chld.GRPARENTOBJREFID,
										chld.GRPARENTOBJREFID2,
										chld.GRPARENTOBJREFID3));
		
	END IF;
	
	Update CF_ROUTINGOBJECT
	SET
		ROUTINGOBJTYPEID=p_ROUTINGOBJTYPEID
		,      ROUTINGOBJNAME=p_ROUTINGOBJNAME
		,      ROUTINGOBJCD=p_ROUTINGOBJCD
		,      ROUTINGOBJVER=p_ROUTINGOBJVER
		,      CHORGSEQNBR=p_CHORGSEQNBR
		,      ROUTINGOBJDESC=p_ROUTINGOBJDESC
		,      ACTIVESTATUSCD=p_ACTIVESTATUSCD
		,      VISIBILITYCD=p_VISIBILITYCD
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      DOCUMENTFLG=p_DOCUMENTFLG
      ,      PROTOCOLTYPE=p_PROTOCOLTYPE
	Where
		ROUTINGOBJID = p_ROUTINGOBJID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_ROUTINGOBJECTU_AH1: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_ROUTINGOBJECTU_AH1: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_ROUTINGOBJECTU_AH1:'||SQLERRM);
End CF_ROUTINGOBJECTU_AH1;
/


-- CF_ROUTINGOBJREF_LNKD.sql

CREATE OR REPLACE Procedure CF_ROUTINGOBJREF_LNKD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_ROUTINGOBJREFID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGOBJREF_LNKD
	Create Date:     2003-09-26 08:24:02
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJREF_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJREFID=>'||p_ROUTINGOBJREFID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_ROUTINGOBJREF_LNK
	Where
		ROUTINGOBJREFID = p_ROUTINGOBJREFID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJREF_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_ROUTINGOBJREF_LNKD;
/


-- CF_ROUTINGOBJREF_LNKI.sql

CREATE OR REPLACE Procedure CF_ROUTINGOBJREF_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ROUTINGOBJID in NUMBER
	,p_ROUTINGOBJPARENTREFID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGOBJREF_LNKI
	Create Date:     2003-09-26 00:01:22
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJREF_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJID=>'||p_ROUTINGOBJID||CHR(10)
		||', p_ROUTINGOBJPARENTREFID=>'||p_ROUTINGOBJPARENTREFID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_ROUTINGOBJREF_LNK(
		ROUTINGOBJREFID
		,ROUTINGOBJID
		,ROUTINGOBJPARENTREFID
	) Values (
		CF_ROUTINGOBJREF_LNK_SEQ.nextval
		,p_ROUTINGOBJID
		,p_ROUTINGOBJPARENTREFID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_ROUTINGOBJREF_LNK_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;			
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_ROUTINGOBJREF_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJREF_LNKI;
/


-- CF_ROUTINGOBJTYPES_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_ROUTINGOBJTYPES_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in Varchar2
    )
AS

/*
    ================================================================================
    Proc Name:       CF_RoutingObjTypeS_AH1
    Create Date:     2003-05-06 10:07:31
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-06 J.Chen        3.5    Returns all routing object types ordered by level.
    2003-05-28 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
                                    Changed CF_ROUTINGOBJTYPE_VW to CF_ROUTINGOBJTYPE_LOC_VW.
    2004-03-05 S.Narain      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_RoutingObjTypeS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||'p_Locale=>'|| p_Locale || CHR(10);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 For
    Select
              ROUTINGOBJTYPEID,
              ROUTINGOBJLEVEL,
              ROUTINGOBJSUBLEVEL,
              ROUTINGOBJATTRRANK,
              ROUTINGOBJTYPEKEY,
              RESOURCEKEY,
              DESCRIPTION,
              RowTS
    From
              CF_ROUTINGOBJTYPE
    Where
             ROUTINGOBJLEVEL != -1
    Order by  ROUTINGOBJLEVEL;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);


    EXCEPTION
         When OTHERS Then
            Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR (-20999,'20000-CF_RoutingObjTypeS_AH1' || SQLCODE|| ' '||SQLERRM);
END CF_ROUTINGOBJTYPES_AH1;
/


-- CF_ROUTINGOBJTYPE_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE CF_ROUTINGOBJTYPE_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_ROUTINGOBJTYPEID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_ROUTINGOBJTYPE_LOC_VWS
	Create Date:     2003-09-26 00:10:39
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-03-05 S.Narain      4.2.2  Removed NLS
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_ROUTINGOBJTYPE_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGOBJTYPEID=>'||p_ROUTINGOBJTYPEID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ROUTINGOBJTYPEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		ROUTINGOBJTYPEID
		,ROUTINGOBJLEVEL
		,ROUTINGOBJSUBLEVEL
		,ROUTINGOBJATTRRANK
		,RTRIM(ROUTINGOBJTYPEKEY) ROUTINGOBJTYPEKEY
		,ROWTS
		,CREATEDATE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
	FROM
		CF_ROUTINGOBJTYPE
	Where
		ROUTINGOBJTYPEID = p_ROUTINGOBJTYPEID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		ROUTINGOBJTYPEID
		,ROUTINGOBJLEVEL
		,ROUTINGOBJSUBLEVEL
		,ROUTINGOBJATTRRANK
		,RTRIM(ROUTINGOBJTYPEKEY) ROUTINGOBJTYPEKEY
		,ROWTS
		,CREATEDATE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
	FROM
		CF_ROUTINGOBJTYPE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_ROUTINGOBJTYPE_LOC_VWS;
/


-- CF_STRUCTURETYPE_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE CF_STRUCTURETYPE_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_STRUCTURETYPEID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_STRUCTURETYPE_LOC_VWS
	Create Date:     2003-09-26 00:10:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-03-03 S.Priani	 4.2.2	Removed NLS

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_STRUCTURETYPE_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_STRUCTURETYPEID=>'||p_STRUCTURETYPEID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_STRUCTURETYPEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		STRUCTURETYPEID
		,RTRIM(STRUCTURETYPEKEY) STRUCTURETYPEKEY
		,ROWTS
		,CREATEDATE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		FROM
		CF_STRUCTURETYPE
	Where
		STRUCTURETYPEID = p_STRUCTURETYPEID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		STRUCTURETYPEID
		,RTRIM(STRUCTURETYPEKEY) STRUCTURETYPEKEY
		,ROWTS
		,CREATEDATE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
	FROM
		CF_STRUCTURETYPE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_STRUCTURETYPE_LOC_VWS;
/


-- CF_TRANSFORMINTERACT_LNKD.sql

CREATE OR REPLACE Procedure CF_TRANSFORMINTERACT_LNKD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_INTERACTID in NUMBER
	,p_TRANSFORMMAPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_TRANSFORMINTERACT_LNKD
	Create Date:     2003-09-26 08:24:03
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMINTERACT_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_INTERACTID=>'||p_INTERACTID||CHR(10)
		||', p_TRANSFORMMAPID=>'||p_TRANSFORMMAPID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_TRANSFORMINTERACT_LNK
	Where
		INTERACTID = p_INTERACTID
		AND TRANSFORMMAPID = NVL(p_TRANSFORMMAPID, TRANSFORMMAPID)
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_TRANSFORMINTERACT_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_TRANSFORMINTERACT_LNKD;
/


-- CF_TRANSFORMINTERACT_LNKI.sql

CREATE OR REPLACE Procedure CF_TRANSFORMINTERACT_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out
db_ref.RCT
	,p_INTERACTID in NUMBER
	,p_TRANSFORMMAPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_TRANSFORMINTERACT_LNKI
	Create Date:     2003-09-26 00:03:41
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMINTERACT_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_INTERACTID=>'||p_INTERACTID||CHR(10)
		||', p_TRANSFORMMAPID=>'||p_TRANSFORMMAPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_TRANSFORMINTERACT_LNK(
		INTERACTID
		,TRANSFORMMAPID
	) Values (
		p_INTERACTID
		,p_TRANSFORMMAPID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;			
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_TRANSFORMINTERACT_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_TRANSFORMINTERACT_LNKI;
/


-- CF_TRANSFORMINTERACT_LNKS_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_TransformInteract_LnkS_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_InteractId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_TransformInteract_LnkS_AH1
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns TransformMapId by given Fr/To RoutingObjId
    2003-05-02 J.Chen        3.5    Replaced Fr/ToRoutingObjId to Fr/ToRoutingObjRefId as input
    2003-05-06 J.Chen        3.5    Changed input Fr/ToRoutingObjRefId to p_InteractId
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_TransformInteract_LnkS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_InteractId=>'||p_InteractId||CHR(10)
        ;
v_InteractId  Number(10);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

         IF RC1%isOpen THEN Close RC1; END IF;

         Open RC1 for
         Select    l.TransformMapId
                   ,m.TransformMapDesc
                   ,m.TransformMapFileName
                   ,l.RowTS
         From      CF_TransformInteract_Lnk l
                   Inner Join CF_TransformMap m On m.TransformMapId = l.TransformMapId
         Where     InteractId = p_InteractId
         ;

         p_ReturnCd := 0;
         DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --       Commit;

         EXCEPTION
              When OTHERS Then
                   Rollback work;
              p_ReturnCd := 1;
              v_ExceptionFlg := 1;
              DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
              Commit;
              RAISE_APPLICATION_ERROR (-20999,'20000-CF_TransformInteract_LnkS_AH1' || SQLCODE|| ' '||SQLERRM);
End CF_TransformInteract_LnkS_AH1;
/


-- CF_TRANSFORMMAPD.SQL

CREATE OR REPLACE Procedure CF_TRANSFORMMAPD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_TRANSFORMMAPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_TRANSFORMMAPD
	Create Date:     2003-09-26 08:24:05
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMMAPD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSFORMMAPID=>'||p_TRANSFORMMAPID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From CF_TRANSFORMMAP
	Where
		TRANSFORMMAPID = p_TRANSFORMMAPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_TRANSFORMMAPD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_TRANSFORMMAPD;
/


-- CF_TRANSFORMMAPI.SQL

CREATE OR REPLACE Procedure CF_TRANSFORMMAPI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_TRANSFORMMAPFILENAME in VARCHAR2
	,p_TRANSFORMMAPDESC in VARCHAR2
	,p_TRANSFORMMAP in BLOB
	)
AS

/*
	================================================================================
	Proc Name:       CF_TRANSFORMMAPI
	Create Date:     2003-09-26 00:01:23
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMMAPI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSFORMMAPFILENAME=>'||p_TRANSFORMMAPFILENAME||CHR(10)
		||', p_TRANSFORMMAPDESC=>'||p_TRANSFORMMAPDESC||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_TRANSFORMMAP(
		TRANSFORMMAPID
		,TRANSFORMMAPFILENAME
		,TRANSFORMMAPDESC
		,TRANSFORMMAP
	) Values (
		CF_TRANSFORMMAP_SEQ.nextval
		,p_TRANSFORMMAPFILENAME
		,p_TRANSFORMMAPDESC
		,p_TRANSFORMMAP
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_TRANSFORMMAP_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_TRANSFORMMAPI' || SQLCODE|| ' '||SQLERRM);
END CF_TRANSFORMMAPI;
/


-- CF_TRANSFORMMAPS.SQL

CREATE OR REPLACE PROCEDURE CF_TRANSFORMMAPS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_TRANSFORMMAPID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_TRANSFORMMAPS
	Create Date:     2003-09-26 00:06:57
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMMAPS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSFORMMAPID=>'||p_TRANSFORMMAPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_TRANSFORMMAPID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		TRANSFORMMAPID
		,RTRIM(TRANSFORMMAPFILENAME) TRANSFORMMAPFILENAME
		,RTRIM(TRANSFORMMAPDESC) TRANSFORMMAPDESC
		,TRANSFORMMAP
		,ROWTS
		,CREATEDATE
	FROM
		CF_TRANSFORMMAP
	Where
		TRANSFORMMAPID = p_TRANSFORMMAPID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		TRANSFORMMAPID
		,RTRIM(TRANSFORMMAPFILENAME) TRANSFORMMAPFILENAME
		,RTRIM(TRANSFORMMAPDESC) TRANSFORMMAPDESC
		,TRANSFORMMAP
		,ROWTS
		,CREATEDATE
	FROM
		CF_TRANSFORMMAP
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_TRANSFORMMAPS;
/


-- CF_TRANSFORMMAPSS_AH1.SQL

CREATE OR REPLACE PROCEDURE         CF_TRANSFORMMAPSS_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_TRANSFORMMAPFILENAME in Varchar2
    )
AS

/*
    ================================================================================
    Proc Name:       CF_TRANSFORMMAPSS_AH1
    Create Date:     2003-04-23 17:12:32
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-23 SYSTEM           3.5 Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_TRANSFORMMAPSS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_TRANSFORMMAPFILENAME=>'||p_TRANSFORMMAPFILENAME||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
        TRANSFORMMAPID
        ,RTRIM(TRANSFORMMAPFILENAME) TRANSFORMMAPFILENAME
           ,RTRIM(TRANSFORMMAPDESC) TRANSFORMMAPDESC
        ,TRANSFORMMAP
        ,ROWTS
        ,CREATEDATE
    From
        CF_TRANSFORMMAP
    Where
        TRANSFORMMAPFILENAME = p_TRANSFORMMAPFILENAME
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - CF_TRANSFORMMAPSS_AH1:'||SQLERRM);
End CF_TRANSFORMMAPSS_AH1;
/


-- CF_TRANSFORMMAPS_AH2.sql

CREATE OR REPLACE PROCEDURE         CF_TRANSFORMMAPS_AH2
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_TRANSFORMMAPID in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_TRANSFORMMAPS_AH2
    Create Date:     2003-05-07
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-07 J.Chen        3.5    Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_TRANSFORMMAPS_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_TRANSFORMMAPID=>'||p_TRANSFORMMAPID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
        TRANSFORMMAPID
        ,RTRIM(TRANSFORMMAPFILENAME) TRANSFORMMAPFILENAME
        ,RTRIM(TRANSFORMMAPDESC) TRANSFORMMAPDESC
        ,ROWTS
        ,TRANSFORMMAP
    From
        CF_TRANSFORMMAP
    Where
        TRANSFORMMAPID = p_TRANSFORMMAPID
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - CF_TRANSFORMMAPS_AH2:'||SQLERRM);
End CF_TRANSFORMMAPS_AH2;
/


-- CF_TRANSFORMMAPS_AH3.sql

CREATE OR REPLACE PROCEDURE         CF_TRANSFORMMAPS_AH3
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_TransformMapFileName in varchar2
    )
AS

/*
    ================================================================================
    Proc Name:       CF_TRANSFORMMAPS_AH3
    Create Date:     2003-05-07
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-07 J.Chen        3.5    Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_TRANSFORMMAPS_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_TRANSFORMMAPFileName=>'||p_TransformMapFileName||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
        TRANSFORMMAPID
        ,RTRIM(TRANSFORMMAPFILENAME) TRANSFORMMAPFILENAME
        ,RTRIM(TRANSFORMMAPDESC) TRANSFORMMAPDESC
        ,ROWTS
    From
        CF_TRANSFORMMAP
    Where
        TRANSFORMMAPFileName = p_TRANSFORMMAPFileName
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - CF_TRANSFORMMAPS_AH3:'||SQLERRM);
End CF_TRANSFORMMAPS_AH3;
/


-- CF_TRANSFORMMAPS_AH4.sql

CREATE OR REPLACE PROCEDURE CF_TRANSFORMMAPS_AH4
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, RC2 out  db_ref.RCT
	, p_sortkey         in  varchar2
	, p_PageSizeNbr     in  pls_Integer
	, p_PageNbr         in  pls_Integer
	, p_TRANSFORMMAPFILENAME IN varchar2
	, p_DESCRIPTION in varchar2
	)
AS
/*
	================================================================================
	Proc Name:       CF_TRANSFORMMAPS_AH4
	Create Date:     2004-09-08 10:48:13
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMMAPS_AH4';
v_ExceptionFlg    integer Default 0;

v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(4000);
  v_where            varchar2(4000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(4000);
  sql_inner          varchar2(4000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(4000);
v_ParamsTxt       varchar(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||',p_sortkey=>'|| p_sortkey||CHR(10)
        ||',p_PageSizeNbr=>'|| p_PageSizeNbr||CHR(10)
        ||',p_PageNbr=>'|| p_PageNbr||CHR(10)
        ||',p_DESCRIPTION=>'|| p_DESCRIPTION||CHR(10)
        ||',p_TRANSFORMMAPFILENAME=>'|| p_TRANSFORMMAPFILENAME||CHR(10);
Procedure where_clause as
begin
    v_where := ' Where 1=1'||
    FN_Build_Where('TRANSFORMMAPFILENAME', 'VARCHAR2_LIKE'   , p_TRANSFORMMAPFILENAME) ||
    FN_Build_Where('DESCRIPTION',     'VARCHAR2_LIKE'   , p_DESCRIPTION);
end where_clause;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	p_ReturnCd := 0;
	    where_clause;
	
	    sqlstmt := 'SELECT  TRANSFORMMAPID ,RTRIM(TRANSFORMMAPFILENAME) TRANSFORMMAPFILENAME, '
		    		    ||'TRANSFORMMAP, TRANSFORMMAPDESC, ROWTS FROM CF_TRANSFORMMAP ' || v_where;
	    
	  IF NVL(INSTR(p_sortkey,'ASC'),0) >  0 THEN
	    v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'ASC')-1);
	    sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ')'; 
	   ELSIF INSTR(upper(p_sortkey),'DESC') > 0 THEN
	      v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'DESC')-1);
	      sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ') DESC'; 
	   ELSE
	       sqlstmt := sqlstmt ||' order by TRANSFORMMAPFILENAME DESC'; 
	  END IF; 
	        
	   sql_inner := 'FROM CF_TRANSFORMMAP';
	
	    -- total row count Cursor
	
	    IF RC2%ISOPEN THEN CLOSE RC2; END IF;
	    IF NVL(P_PAGENBR,0) IN (0,1) THEN
	      Execute Immediate 'select count(*)  '||sql_inner ||v_where INTO v_count;
	  	       open RC2 for select v_count TotalRowsNbr from dual;
	     end if;
	-- Paging logic
	     sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
	 --  Set_page_parameters(p_pageNbr, p_pageSizeNbr, sqlstmt);
	
	    IF RC1%isOpen THEN Close RC1; END IF;
	    Open RC1 for sqlstmt;
	      p_ReturnCd := 0;
	      DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	 Exception
	    When NO_DATA_FOUND THEN Null;
	    When OTHERS Then
	                p_ReturnCd     := 1;
	                v_ExceptionFlg := 1;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_TRANSFORMMAPS_AH4;
/


-- CF_TRANSFORMMAPS_AH5.sql

CREATE OR REPLACE PROCEDURE CF_TRANSFORMMAPS_AH5
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_TransformMapID   IN INTEGER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_TRANSFORMMAPS_AH5
    Create Date:     2003-05-07
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-07 J.Chen        3.5    Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
    ================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_TRANSFORMMAPS_AH5';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_TransformMapID=>'||p_TransformMapID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    SELECT
		prc.CHANNELID, prc.FRPARTNERID,prc.TOPARTNERID
		,prt1.PARTNERNAME FROMPARTNERNAME,prt2.PARTNERNAME TOPARTNERNAME
    From
	
	CF_VALIDOBJINTERACT cvo
	LEFT OUTER JOIN CF_TRANSFORMINTERACT_LNK ctl on ctl.interactid = cvo.interactid	
	INNER JOIN PR_CAPABILITY pcf on pcf.routingobjrefid = cvo.FRROUTINGOBJREFID
	INNER JOIN PR_CAPABILITY pct on pct.routingobjrefid = cvo.TOROUTINGOBJREFID
	INNER JOIN PR_ROUTINGCHANNEL prc on prc.FRCAPABILITYID = pcf.capabilityid
	INNER JOIN PR_PARTNER prt1 on prt1.partnerid = prc.FRPARTNERID
	INNER JOIN pr_partner prt2 on prt2.partnerid = prc.topartnerid
    Where
		prc.TRANSFORMMAPID = p_TransformMapID
		and prc.tocapabilityid = pct.capabilityid
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - CF_TRANSFORMMAPS_AH5:'||SQLERRM);
End CF_TRANSFORMMAPS_AH5;
/


-- cf_transformmapS_ah6.sql

CREATE OR REPLACE PROCEDURE cf_transformmapS_ah6
	(
	  p_ReturnCd   OUT  PLS_INTEGER
	, p_DebugFlg   IN  PLS_INTEGER
	, RC1          OUT DB_REF.RCT                                                                      
	, p_toobjectid IN  NUMBER
   , p_frobjectid IN  NUMBER
	)                                                                              
AS                                                                              

/*                                                                             
   This proc gets transformation maps give source and target routing object ids (not ref ids)

	================================================================================
	Proc Name:       CF_TRANSFORMMAPS_ah6                                              
	Create Date:     2005-12-09 10:19:44                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-09 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'CF_TRANSFORMMAPS_ah6';                      
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_toobjectid=>'||p_toobjectid||CHR(10)
      ||', p_frobjectid=>'||p_frobjectid||CHR(10)
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          

	OPEN RC1 FOR 
		SELECT tmap.transformmapid, tmap.transformmapfilename, tmap.transformmapdesc
   FROM cf_transformmap tmap,
        cf_transforminteract_lnk tilnk,
        cf_validobjinteract objinteract,
        cf_routingobjref_lnk frreflnk,
        cf_routingobjref_lnk toreflnk
   WHERE tmap.transformmapid = tilnk.transformmapid
      AND tilnk.interactid = objinteract.interactid
      AND objinteract.frroutingobjrefid = frreflnk.routingobjrefid
      AND objinteract.toroutingobjrefid = toreflnk.routingobjrefid
      AND toreflnk.routingobjid = p_toobjectid
      AND frreflnk.routingobjid = p_frobjectid
	;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END cf_transformmapS_ah6;
/


-- CF_TRANSFORMMAPS_AH7.sql

CREATE OR REPLACE PROCEDURE CF_TRANSFORMMAPS_AH7
(
   p_returncd          OUT      INTEGER,
   p_debugflg          IN       PLS_INTEGER,
   rc1                 OUT      db_ref.rct,
   p_routingobjrefid   IN       INTEGER
)
AS
/*
    ================================================================================
    Proc Name:       CF_TRANSFORMMAPS_AH7
    Create Date:     2007-08-20 16:40:12
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-10 BHARANI J        1    Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
    ================================================================================*/
  
   v_begints        TIMESTAMP       := fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   := 'CF_TRANSFORMMAPS_AH7';
   v_exceptionflg   INTEGER         := 0;
   v_paramstxt      VARCHAR2 (4000)
      :=    'p_DebugFlg=>'
         || p_debugflg
         || CHR (10)
         || ', p_routingobjrefid=>'
         || p_routingobjrefid
         || CHR (10);

BEGIN
   IF rc1%ISOPEN
   THEN
      CLOSE rc1;
   END IF;

   OPEN rc1 FOR
      SELECT distinct a.transformmapid,
             RTRIM (transformmapfilename) transformmapfilename,
             RTRIM (transformmapdesc) transformmapdesc
        FROM cf_transformmap a,
             cf_transforminteract_lnk b,
             cf_validobjinteract c,
             pr_capability d
       WHERE a.transformmapid = b.transformmapid
         AND b.interactid = c.interactid
         AND (   c.frroutingobjrefid = d.routingobjrefid
              OR c.toroutingobjrefid = d.routingobjrefid
             )
         AND d.routingobjrefid = p_routingobjrefid;

   
   
   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
                        
  p_returncd := 0;                        
EXCEPTION
   WHEN OTHERS
   THEN
      p_returncd := 1;
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            v_paramstxt,
                            p_debugflg,
                            v_exceptionflg
                           );
                           
      raise_application_error (-20999,
                               '20100 - CF_TRANSFORMMAPS_AH7:' || SQLERRM);
                               
END CF_TRANSFORMMAPS_AH7;
/

-- CF_TRANSFORMMAPS_WORKFLOW_AH1.sql

CREATE OR REPLACE PROCEDURE CF_TRANSFORMMAPS_WORKFLOW_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_WORKFLOWID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_TRANSFORMMAPS_WORKFLOW_AH1
	Create Date:     2004-03-13 00:10:40
	Author:          SYSTEM 

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2003-03-31 T.Dang	4.2.2	Created
	2005-10-04 S.Priani	6.0.0.2	BGTRK00049412
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMMAPS_WORKFLOW_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR 
	SELECT
		ct.TRANSFORMMAPID
		,RTRIM(TRANSFORMMAPFILENAME) TRANSFORMMAPFILENAME
	FROM 
		PR_WORKFLOW w 
		INNER JOIN PR_WORKFLOWINTERACT_LNK wi ON w.WORKFLOWID = wi.WORKFLOWID 
		INNER JOIN CF_TRANSFORMINTERACT_LNK ti ON wi.INTERACTID = ti.INTERACTID 
		INNER JOIN CF_TRANSFORMMAP ct ON ti.TRANSFORMMAPID = ct.TRANSFORMMAPID  
	Where 
		w.WORKFLOWID = p_WORKFLOWID
	;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

END CF_TRANSFORMMAPS_WORKFLOW_AH1;
/


-- CF_TRANSFORMMAPU.SQL

CREATE OR REPLACE Procedure CF_TRANSFORMMAPU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_TRANSFORMMAPID in NUMBER
	,p_TRANSFORMMAPFILENAME in VARCHAR2
	,p_TRANSFORMMAPDESC in VARCHAR2
	,p_TRANSFORMMAP in BLOB
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_TRANSFORMMAPU
	Create Date:     2003-09-26 08:20:59
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMMAPU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSFORMMAPID=>'||p_TRANSFORMMAPID||CHR(10)
		||', p_TRANSFORMMAPFILENAME=>'||p_TRANSFORMMAPFILENAME||CHR(10)
		||', p_TRANSFORMMAPDESC=>'||p_TRANSFORMMAPDESC||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_TRANSFORMMAP
	SET
		TRANSFORMMAPFILENAME=p_TRANSFORMMAPFILENAME
		,      TRANSFORMMAPDESC=p_TRANSFORMMAPDESC
		,      TRANSFORMMAP=p_TRANSFORMMAP
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		TRANSFORMMAPID = p_TRANSFORMMAPID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_TRANSFORMMAPU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_TRANSFORMMAPU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_TRANSFORMMAPU:'||SQLERRM);
End CF_TRANSFORMMAPU;
/


-- CF_TRANSFORMROUTINGOBJ_LNKD.sql

Create or Replace Procedure CF_TRANSFORMROUTINGOBJ_LNKD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_TRANSFORMMAPROLNKID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_TRANSFORMROUTINGOBJ_LNKD
	Create Date:     2004-09-08 10:48:14
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMROUTINGOBJ_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSFORMMAPROLNKID=>'||p_TRANSFORMMAPROLNKID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Delete From CF_TRANSFORMROUTINGOBJ_LNK
	Where 
		TRANSFORMMAPROLNKID = p_TRANSFORMMAPROLNKID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_TRANSFORMROUTINGOBJ_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_TRANSFORMROUTINGOBJ_LNKD;
/


-- CF_TRANSFORMROUTINGOBJ_LNKI.sql

Create or Replace Procedure CF_TRANSFORMROUTINGOBJ_LNKI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_TRANSFORMMAPID in NUMBER
	,p_FROMROUTINGOBJREFID in NUMBER
	,p_TOROUTINGOBJREFID in NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_TRANSFORMROUTINGOBJ_LNKI
	Create Date:     2004-09-08 10:48:14
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMROUTINGOBJ_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSFORMMAPID=>'||p_TRANSFORMMAPID||CHR(10)
		||', p_FROMROUTINGOBJREFID=>'||p_FROMROUTINGOBJREFID||CHR(10)
		||', p_TOROUTINGOBJREFID=>'||p_TOROUTINGOBJREFID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	Insert into CF_TRANSFORMROUTINGOBJ_LNK(
		TRANSFORMMAPROLNKID
		,TRANSFORMMAPID
		,FROMROUTINGOBJREFID
		,TOROUTINGOBJREFID
	) Values (
		CF_TRANSFORMROUTINGOBJ_LNK_SEQ.nextval
		,p_TRANSFORMMAPID
		,p_FROMROUTINGOBJREFID
		,p_TOROUTINGOBJREFID
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_TRANSFORMROUTINGOBJ_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_TRANSFORMROUTINGOBJ_LNKI;
/


-- CF_TRANSFORMROUTINGOBJ_LNKS.sql

CREATE OR REPLACE PROCEDURE CF_TRANSFORMROUTINGOBJ_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_TRANSFORMMAPID IN NUMBER
	, p_FROMROUTINGOBJREFID IN NUMBER
	, p_TOROUTINGOBJREFID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       CF_TRANSFORMROUTINGOBJ_LNKS
	Create Date:     2004-09-08 10:48:14
	Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-08 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMROUTINGOBJ_LNKS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSFORMMAPID=>'||p_TRANSFORMMAPID||CHR(10)
		||', p_FROMROUTINGOBJREFID=>'||p_FROMROUTINGOBJREFID||CHR(10)
		||', p_TOROUTINGOBJREFID=>'||p_TOROUTINGOBJREFID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

		OPEN RC1 FOR
		SELECT L.TRANSFORMMAPID TRANSFORMMAPID
	,T.TRANSFORMMAPFILENAME TRANSFORMMAPFILENAME
	, L.FROMROUTINGOBJREFID FROMROUTINGOBJREFID
	,L.TOROUTINGOBJREFID TOROUTINGOBJREFID
	, L.ROWTS ROWTS
FROM CF_TRANSFORMROUTINGOBJ_LNK L
	,CF_TRANSFORMMAP T
WHERE L.TRANSFORMMAPID = T.TRANSFORMMAPID
AND ((p_TRANSFORMMAPID IS NULL) 
  OR (p_TRANSFORMMAPID IS NOT NULL AND L.TRANSFORMMAPID = p_TRANSFORMMAPID)
    )
AND ((p_FROMROUTINGOBJREFID IS NULL) 
  OR (p_FROMROUTINGOBJREFID IS NOT NULL AND
	L.FROMROUTINGOBJREFID IN (
	SELECT ROUTINGOBJREFID
	FROM CF_ROUTINGOBJECT03_VW
	WHERE  (ROUTINGOBJID, PARENTOBJID)
		IN (
		 SELECT ROUTINGOBJID, PARENTOBJID
		 FROM CF_ROUTINGOBJECT03_VW
		 WHERE ROUTINGOBJREFID = p_FROMROUTINGOBJREFID
		)
	)
     )
    )
AND ((p_TOROUTINGOBJREFID IS NULL) 
  OR (p_TOROUTINGOBJREFID IS NOT NULL AND
	L.TOROUTINGOBJREFID IN (
	SELECT ROUTINGOBJREFID
	FROM CF_ROUTINGOBJECT03_VW
	WHERE  (ROUTINGOBJID, PARENTOBJID)
		IN (
		 SELECT ROUTINGOBJID, PARENTOBJID
		 FROM CF_ROUTINGOBJECT03_VW
		 WHERE ROUTINGOBJREFID = p_TOROUTINGOBJREFID
		)
	)
     )
    )
;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_TRANSFORMROUTINGOBJ_LNKS;
/

-- CF_VALIDOBJINTERACTI.sql

CREATE OR REPLACE Procedure CF_VALIDOBJINTERACTI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_FRROUTINGOBJREFID in NUMBER
	,p_TOROUTINGOBJREFID in NUMBER
	,p_ENABLEDCD in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_VALIDOBJINTERACTI
	Create Date:     2003-09-26 00:01:25
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2008-09-12 Amit Kumar Vijay 6.2 Defect 47980, 106065 and 106066
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_VALIDOBJINTERACTI';
v_ExceptionFlg    integer Default 0;
v_FRENABLEDCD	  integer Default 0;
v_TOENABLEDCD	  integer Default 0;
v_FrRORefId	  CF_ROUTINGOBJREF_LNK.ROUTINGOBJREFID%TYPE;
v_ToRORefId	  CF_ROUTINGOBJREF_LNK.ROUTINGOBJREFID%TYPE;
V_INTERACTOBJID      CF_VALIDOBJINTERACT.INTERACTID%TYPE;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FRROUTINGOBJREFID=>'||p_FRROUTINGOBJREFID||CHR(10)
		||', p_TOROUTINGOBJREFID=>'||p_TOROUTINGOBJREFID||CHR(10)
		||', p_ENABLEDCD=>'||p_ENABLEDCD||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

v_FrRORefId:=p_FRROUTINGOBJREFID;
v_ToRORefId:=p_TOROUTINGOBJREFID;
	
	select count(*) into v_FRENABLEDCD from cf_routingobject cr 
		inner join cf_routingobjref_lnk crl on crl.routingobjid=cr.routingobjid
	WHERE crl.routingobjrefid in(SELECT prnt.routingobjrefid FROM CF_ROUTINGOBJECT06_VW prnt
					,CF_ROUTINGOBJECT06_VW slf
				     WHERE prnt.ROUTINGOBJREFID IN ( slf.ROUTINGOBJREFID
									,slf.ROUTINGOBJPARENTREFID
									,slf.GRPARENTOBJREFID
									,slf.GRPARENTOBJREFID2
									,slf.GRPARENTOBJREFID3)
				    and slf.ROUTINGOBJREFID=v_FrRORefId)
	and cr.activestatuscd=0;
			
			
	select count(*) into v_TOENABLEDCD from cf_routingobject cr 
		inner join cf_routingobjref_lnk crl on crl.routingobjid=cr.routingobjid
	WHERE crl.routingobjrefid in(SELECT prnt.routingobjrefid FROM CF_ROUTINGOBJECT06_VW prnt
					,CF_ROUTINGOBJECT06_VW slf
				     WHERE prnt.ROUTINGOBJREFID IN ( slf.ROUTINGOBJREFID
									,slf.ROUTINGOBJPARENTREFID
									,slf.GRPARENTOBJREFID
									,slf.GRPARENTOBJREFID2
									,slf.GRPARENTOBJREFID3)
	  			     and slf.ROUTINGOBJREFID=v_ToRORefId)
	and cr.activestatuscd=0;
	
	IF v_FRENABLEDCD=0 and v_TOENABLEDCD=0 Then
		  BEGIN
			INSERT INTO CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
				SELECT	CF_ValidobjInterAct_Seq.nextval
					,v_FrRoRefId
					,v_ToROrefId
					,7
				FROM	dual;
			  EXCEPTION
				WHEN Dup_Val_On_Index THEN
					SELECT	InteractId
					INTO	v_InterActObjId
					FROM	CF_ValidObjInteract
					WHERE	FrRoutingObjRefId = v_FrRoRefId
					AND	ToRoutingObjRefId = v_ToROrefId;
		 END;		
	ELSIF v_FRENABLEDCD=0 and v_TOENABLEDCD<>0 Then
		 BEGIN
			INSERT INTO CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
				SELECT	CF_ValidobjInterAct_Seq.nextval
					,v_FrRoRefId
					,v_ToROrefId
					,5
				FROM	dual;
			 EXCEPTION
				WHEN Dup_Val_On_Index THEN
					SELECT	InteractId
					INTO	v_InterActObjId
					FROM	CF_ValidObjInteract
					WHERE	FrRoutingObjRefId = v_FrRoRefId
					AND	ToRoutingObjRefId = v_ToROrefId;
		 END;		
	ELSIF v_FRENABLEDCD<>0 and v_TOENABLEDCD=0 Then
		 BEGIN
			INSERT INTO CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
				SELECT	CF_ValidobjInterAct_Seq.nextval
					,v_FrRoRefId
					,v_ToROrefId
					,3
				FROM	dual;
			 EXCEPTION
				WHEN Dup_Val_On_Index THEN
					SELECT	InteractId
					INTO	v_InterActObjId
					FROM	CF_ValidObjInteract
					WHERE	FrRoutingObjRefId = v_FrRoRefId
					AND	ToRoutingObjRefId = v_ToROrefId;
		 END;
	ELSE 
		 BEGIN
			INSERT INTO CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
				SELECT	CF_ValidobjInterAct_Seq.nextval
					,v_FrRoRefId
					,v_ToROrefId
					,1
				FROM	dual;
			 EXCEPTION
				WHEN Dup_Val_On_Index THEN
					SELECT	InteractId
					INTO	v_InterActObjId
					FROM	CF_ValidObjInteract
					WHERE	FrRoutingObjRefId = v_FrRoRefId
					AND	ToRoutingObjRefId = v_ToROrefId;
		 END;
	END IF;
			
			
	
		
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		CF_VALIDOBJINTERACT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 CF_VALIDOBJINTERACTI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 CF_VALIDOBJINTERACTI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_VALIDOBJINTERACTI' || SQLCODE|| ' '||SQLERRM);
END CF_VALIDOBJINTERACTI;
/


-- CF_VALIDOBJINTERACTI_AH1.sql

CREATE OR REPLACE PROCEDURE CF_ValidObjInteractI_AH1
		(p_FrROHeir	VARCHAR2
		,p_ToROHeir	VARCHAR2
		,p_WorkFlow	VARCHAR2
		)
AS
/*
	================================================================================
	Proc Name:       CF_ValidObjInteractI_AH1
	Create Date:     2007-01-03 15:24:38
	Author:          S.Priani 

	Desc:	*Inserts metadata for linking a pair of RoutingObjects into CF_ValidobjInteract
		- ROHeir accepts Routing Object Heirarchy upto 5 levels (sDl is Separator):
			Package sDl Protocol sDl Process sDl Activity sDl Action
		- Each ROHeir is string of format:
			ROType sTp ROCode sVr ROVersion
			(sTp is delimiter for RO Type, sVr is delimiter for ROCode)
		- String can contain SQL wildcards ("%") and uses LIKE operator
		- ROHeir should uniquely identify a RoutingObjectRefId

		*Inserts metadata into Pr_WorkFlowInteract_lnk such that
		- Links the Interaction to WorkFlow
		- Workflow parameter accepts the workflowname (string)

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-01-03 S.Priani	 6.1	CR#103555
	2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066

	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
	-- Delimiter and Separator declaration and initialization
	-- These can be any 3 distinct characters, which should not occur in the strings for:
		-- Routing Object Type
		-- Routing Object Code
		-- Routing Object Version
		-- And should not be reserved in SQL for string comparison
	-- Note that the same characters neet to be used(in input parameters) while invoking the procedure
	sTp	VARCHAR2(1)	 := '$';
	sVr	VARCHAR2(1)	 := '@';
	sDl	VARCHAR2(1)	 := '+';
	v_FRENABLEDCD integer default 0;
	v_TOENABLEDCD integer default 0;
	v_RowCnt          integer Default 0;
	CURSOR cX(pi_RO_HEIR_NAME VARCHAR2) IS
		SELECT	*
		FROM	(
		SELECT	R.ROUTINGOBJLEVEL,
			R.ROUTINGOBJTYPEKEY,
			R.ROUTINGOBJTYPEID,
			R.ROUTINGOBJID,
			R.ROUTINGOBJCD,
			R.ROUTINGOBJVER,
			F.ROUTINGOBJREFID,
			F.ROUTINGOBJPARENTREFID,
			CASE R.ROUTINGOBJLEVEL
				WHEN 1 THEN RTRIM(R.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R.ROUTINGOBJCD)||sVr||RTRIM(R.ROUTINGOBJVER)
				WHEN 2 THEN RTRIM(R2.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R2.ROUTINGOBJCD)||sVr||RTRIM(R2.ROUTINGOBJVER) ||sDl
						||RTRIM(R.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R.ROUTINGOBJCD)||sVr||RTRIM(R.ROUTINGOBJVER)
				WHEN 3 THEN RTRIM(R3.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R3.ROUTINGOBJCD)||sVr||RTRIM(R3.ROUTINGOBJVER)||sDl
						||RTRIM(R2.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R2.ROUTINGOBJCD)||sVr||RTRIM(R2.ROUTINGOBJVER) || sDl 
						|| RTRIM(R.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R.ROUTINGOBJCD) ||sVr||RTRIM(R.ROUTINGOBJVER)
				WHEN 4 THEN RTRIM(R4.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R4.ROUTINGOBJCD)||sVr||RTRIM(R4.ROUTINGOBJVER) ||sDl
						||RTRIM(R3.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R3.ROUTINGOBJCD)||sVr||RTRIM(R3.ROUTINGOBJVER)||sDl
						||RTRIM(R2.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R2.ROUTINGOBJCD)||sVr||RTRIM(R2.ROUTINGOBJVER) || sDl 
						|| RTRIM(R.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R.ROUTINGOBJCD) ||sVr||RTRIM(R.ROUTINGOBJVER)
				WHEN 5 THEN RTRIM(R5.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R5.ROUTINGOBJCD)||sVr||RTRIM(R5.ROUTINGOBJVER) ||sDl
						||RTRIM(R4.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R4.ROUTINGOBJCD)||sVr||RTRIM(R4.ROUTINGOBJVER) ||sDl
						||RTRIM(R3.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R3.ROUTINGOBJCD)||sVr||RTRIM(R3.ROUTINGOBJVER)||sDl
						||RTRIM(R2.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R2.ROUTINGOBJCD)||sVr||RTRIM(R2.ROUTINGOBJVER) || sDl 
						|| RTRIM(R.ROUTINGOBJTYPEKEY) || sTp || RTRIM(R.ROUTINGOBJCD) ||sVr||RTRIM(R.ROUTINGOBJVER)
				ELSE 'UnDefined'
			END RO_HEIR_NAME
		FROM cf_routingobject01_vw R
		INNER JOIN CF_ROUTINGOBJREF_LNK F ON F.ROUTINGOBJID = R.ROUTINGOBJID
		LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F2 ON F2.ROUTINGOBJREFID = F.ROUTINGOBJPARENTREFID
		LEFT OUTER JOIN cf_routingobject01_vw R2 ON R2.ROUTINGOBJID = F2.ROUTINGOBJID
		LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F3 ON F3.ROUTINGOBJREFID = F2.ROUTINGOBJPARENTREFID
		LEFT OUTER JOIN cf_routingobject01_vw R3 ON R3.ROUTINGOBJID = F3.ROUTINGOBJID
		LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F4 ON F4.ROUTINGOBJREFID = F3.ROUTINGOBJPARENTREFID
		LEFT OUTER JOIN cf_routingobject01_vw R4 ON R4.ROUTINGOBJID = F4.ROUTINGOBJID
		LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F5 ON F5.ROUTINGOBJREFID = F4.ROUTINGOBJPARENTREFID
		LEFT OUTER JOIN cf_routingobject01_vw R5 ON R5.ROUTINGOBJID = F5.ROUTINGOBJID
		) vw
		WHERE	RO_HEIR_NAME LIKE pi_RO_HEIR_NAME;

		v_InterActObjId		CF_ValidObjInteract.InteractID%TYPE;
		v_FrRORefId		CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
		v_ToRORefId		CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
		v_ctr			INTEGER;
BEGIN
		BEGIN
			v_ctr := 0;
			dbms_output.put_line('Fr:'||p_FrROHeir);
			FOR rX IN cX(p_FrROHeir) LOOP
				v_FrRORefId := rX.ROUTINGOBJREFID;
				v_ctr := v_ctr +1;
			END LOOP;
			IF v_ctr <> 1 THEN RAISE No_Data_Found; END IF;
		EXCEPTION
			WHEN No_Data_Found THEN
				IF cX%ISOPEN THEN CLOSE cX; END IF;
				dbms_output.put_line('ValidInteract: Fr Routing Object not found-'||v_ctr);
				RAISE;
		END;
		BEGIN
			v_ctr := 0;
			dbms_output.put_line('To:'||p_ToROHeir);
			FOR rX IN cX(p_ToROHeir) LOOP
				v_ToRORefId := rX.ROUTINGOBJREFID;
				v_ctr := v_ctr +1;
			END LOOP;
			IF v_ctr <> 1 THEN RAISE No_Data_Found; END IF;
		EXCEPTION
			WHEN No_Data_Found THEN
				IF cX%ISOPEN THEN CLOSE cX; END IF;
				dbms_output.put_line('ValidInteract: To Routing Object not found-'||v_ctr);
				RAISE;
		END;
		BEGIN
		
			select count(*) into v_FRENABLEDCD from cf_routingobject cr 
				inner join cf_routingobjref_lnk crl on crl.routingobjid=cr.routingobjid
			WHERE crl.routingobjrefid in(SELECT prnt.routingobjrefid FROM CF_ROUTINGOBJECT06_VW prnt
							,CF_ROUTINGOBJECT06_VW slf
						     WHERE prnt.ROUTINGOBJREFID IN ( slf.ROUTINGOBJREFID
											,slf.ROUTINGOBJPARENTREFID
											,slf.GRPARENTOBJREFID
											,slf.GRPARENTOBJREFID2
											,slf.GRPARENTOBJREFID3)
						    and slf.ROUTINGOBJREFID=v_FrRORefId)
			and cr.activestatuscd=0;
			
			
			select count(*) into v_TOENABLEDCD from cf_routingobject cr 
				inner join cf_routingobjref_lnk crl on crl.routingobjid=cr.routingobjid
			WHERE crl.routingobjrefid in(SELECT prnt.routingobjrefid FROM CF_ROUTINGOBJECT06_VW prnt
							,CF_ROUTINGOBJECT06_VW slf
						     WHERE prnt.ROUTINGOBJREFID IN ( slf.ROUTINGOBJREFID
											,slf.ROUTINGOBJPARENTREFID
											,slf.GRPARENTOBJREFID
											,slf.GRPARENTOBJREFID2
											,slf.GRPARENTOBJREFID3)
			  			     and slf.ROUTINGOBJREFID=v_ToRORefId)
			and cr.activestatuscd=0;
	
			IF v_FRENABLEDCD=0 and v_TOENABLEDCD=0 Then
			  BEGIN
				INSERT INTO CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
					SELECT	CF_ValidobjInterAct_Seq.nextval
						,v_FrRoRefId
						,v_ToROrefId
						,7
					FROM	dual;

					SELECT	CF_ValidobjInterAct_Seq.currval
					INTO	v_InterActObjId
					FROM	DUAL;
			  EXCEPTION
				WHEN Dup_Val_On_Index THEN
					SELECT	InteractId
					INTO	v_InterActObjId
					FROM	CF_ValidObjInteract
					WHERE	FrRoutingObjRefId = v_FrRoRefId
					AND	ToRoutingObjRefId = v_ToROrefId;
			 END;		
			ELSIF v_FRENABLEDCD=0 and v_TOENABLEDCD<>0 Then
			 BEGIN
				INSERT INTO CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
					SELECT	CF_ValidobjInterAct_Seq.nextval
						,v_FrRoRefId
						,v_ToROrefId
						,5
					FROM	dual;

					SELECT	CF_ValidobjInterAct_Seq.currval
					INTO	v_InterActObjId
					FROM	DUAL;
			 EXCEPTION
				WHEN Dup_Val_On_Index THEN
					SELECT	InteractId
					INTO	v_InterActObjId
					FROM	CF_ValidObjInteract
					WHERE	FrRoutingObjRefId = v_FrRoRefId
					AND	ToRoutingObjRefId = v_ToROrefId;
			 END;		
			ELSIF v_FRENABLEDCD<>0 and v_TOENABLEDCD=0 Then
			 BEGIN
				INSERT INTO CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
					SELECT	CF_ValidobjInterAct_Seq.nextval
						,v_FrRoRefId
						,v_ToROrefId
						,3
					FROM	dual;

					SELECT	CF_ValidobjInterAct_Seq.currval
					INTO	v_InterActObjId
					FROM	DUAL;
			 EXCEPTION
				WHEN Dup_Val_On_Index THEN
					SELECT	InteractId
					INTO	v_InterActObjId
					FROM	CF_ValidObjInteract
					WHERE	FrRoutingObjRefId = v_FrRoRefId
					AND	ToRoutingObjRefId = v_ToROrefId;
			 END;
			ELSE 
			 BEGIN
				INSERT INTO CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
					SELECT	CF_ValidobjInterAct_Seq.nextval
						,v_FrRoRefId
						,v_ToROrefId
						,1
					FROM	dual;

					SELECT	CF_ValidobjInterAct_Seq.currval
					INTO	v_InterActObjId
					FROM	DUAL;
			 EXCEPTION
				WHEN Dup_Val_On_Index THEN
					SELECT	InteractId
					INTO	v_InterActObjId
					FROM	CF_ValidObjInteract
					WHERE	FrRoutingObjRefId = v_FrRoRefId
					AND	ToRoutingObjRefId = v_ToROrefId;
			 END;
			END IF;
		END;
			

		dbms_output.put_line('- InteractId' || v_InterActObjId || '-:'|| v_FrRoRefId || '->'|| v_ToRORefId);
		dbms_output.put_line('-- WorkFlow:'|| p_WorkFlow);
		BEGIN
			INSERT INTO Pr_WorkFlowInteract_lnk(WorkflowId,InterActId)
				SELECT	WorkflowId, v_InterActObjId 
				FROM	pr_workflow
				WHERE	WorkFlowName = p_workflow;

		EXCEPTION
			When DUP_VAL_ON_INDEX THEN
				NULL;
			When No_Data_Found THEN
				dbms_output.put_line('ERROR: workflow not found');
				
		END;
END CF_ValidObjInteractI_AH1;
/

-- CF_VALIDOBJINTERACTS.sql

CREATE OR REPLACE PROCEDURE CF_VALIDOBJINTERACTS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_INTERACTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_VALIDOBJINTERACTS
	Create Date:     2003-09-26 00:06:55
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2008-09-12 Amit Kumar Vijay 6.2	Defect 47980, 106065 and 106066
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_VALIDOBJINTERACTS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_INTERACTID=>'||p_INTERACTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_INTERACTID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		INTERACTID
		,FRROUTINGOBJREFID
		,TOROUTINGOBJREFID
		,CASE WHEN ENABLEDCD=7 Then 1 Else 0 END ENABLEDCD
		,ROWTS
		,CREATEDATE
	FROM
		CF_VALIDOBJINTERACT
	Where
		INTERACTID = p_INTERACTID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		INTERACTID
		,FRROUTINGOBJREFID
		,TOROUTINGOBJREFID
		,CASE WHEN ENABLEDCD=7 Then 1 Else 0 END ENABLEDCD
		,ROWTS
		,CREATEDATE
	FROM
		CF_VALIDOBJINTERACT
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_VALIDOBJINTERACTS;
/


-- CF_VALIDOBJINTERACTS_AH1.sql

CREATE OR REPLACE PROCEDURE         CF_ValidObjInteractS_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , RC2  out  db_ref.rct
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_InitRoutingObjRefId in Number
    , p_RecvRoutingObjRefId in Number
    , p_EnabledCd in varchar2  -- list of codes (0,1,2) : 0=disabled, 1=enabled, 2=deleted
    )
AS

/*
    ================================================================================
    Proc Name:       CF_ValidObjInteractS_AH1
    Create Date:     2003-05-06
    Author:          J.Chen

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	
	2008-09-12 Amit Kumar 	 6.2	Defect 47980, 106065 and 106066

    
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
--v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ValidObjInteractS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_sortkey=>'||p_sortkey||CHR(10)
        ||', p_PageSizeNbr=>'||p_PageSizeNbr||CHR(10)
        ||', p_PageNbr=>'||p_PageNbr||CHR(10)
        ||', p_InitRoutingObjRefId=> '||p_InitRoutingObjRefId ||CHR(10)
        ||', p_RecvRoutingObjRefId=>'||p_RecvRoutingObjRefId||CHR(10)
        ||', p_EnabledCd=>'||p_EnabledCd||CHR(10)
        ;

  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
  v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(2000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  v_StartWith        varchar2(1000);
  v_Select           varchar2(1000);
  v_Select2          varchar2(1000);
  v_where2           varchar2(1000);
  v_GetAllFlg        Integer := 0;

----- Types
Type char65Tab is table of varchar2(65);
Type num10Tab  is table of number(10);
array_ChannelId char65Tab := char65Tab();
array_rnbr       num10Tab  := num10Tab();
--
ChannelIdTab  char65_Table := char65_Table();


----------------
-- Build_WhereIn
----------------
PROCEDURE Build_WhereIn(p_column in varchar2, p_dataType in varchar2, p_param in varchar2) IS
BEGIN
    IF p_param IS NOT NULL THEN

         IF p_dataType = 'NUMBER_LIST' THEN
              v_where := v_where||' and '||p_column||' in '||p_param;
              If p_param = 'CASE WHEN v.ENABLEDCD=7 Then 1 Else 0 END' Then
                   v_where2 := v_where2 || ' and '||p_column||' in '||p_param;
              End If;
         END IF;

         IF p_dataType = 'NUMBER' THEN
              v_where := v_where||' and '||p_column||' ='||to_char(p_param) ;
         END IF;

         IF p_dataType = 'VARCHAR2' THEN
              v_where := v_where||' and '||p_column||' ='''||p_param||'''' ;
         END IF;

         IF p_dataType = 'VARCHAR2_LIKE' THEN
              v_where := v_where||' and '||p_column||' LIKE '''||p_param||'''' ;
         END IF;
    END IF;
END Build_WhereIn;

---------------
-- Where_Clause
---------------
PROCEDURE Where_Clause AS
BEGIN

    Build_WhereIn('v.FrRoutingObjRefId', 'NUMBER', p_InitRoutingObjRefId);
    Build_WhereIn('v.ToRoutingObjRefId', 'NUMBER', p_RecvRoutingObjRefId);
    Build_WhereIn('CASE WHEN v.ENABLEDCD=7 Then 1 Else 0 END', 'NUMBER_LIST', p_EnabledCd);

END Where_Clause;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    Case
         When p_InitRoutingObjRefId Is Not Null And p_RecvRoutingObjRefId Is Not Null Then

              sql_inner := ' From
                                 CF_ValidObjInteract v ,
                                 (select * from cf_Routingobjref_lnk fl
                                        Connect by PRIOR fl.RoutingObjRefId = fl.RoutingObjParentRefId
                                        Start with fl.RoutingObjParentRefId = '|| p_InitRoutingObjRefId ||
                                      ' Union
                                        Select * From cf_RoutingObjRef_lnk
                                        Where RoutingObjRefId = '|| p_InitRoutingObjRefId || ') fl,'||

                                 '(select * from cf_Routingobjref_lnk tl
                                        Connect by PRIOR tl.RoutingObjRefId = tl.RoutingObjParentRefId
                                        Start with tl.RoutingObjParentRefId = '|| p_RecvRoutingObjRefId ||
                                      ' Union
                                        Select * From cf_RoutingObjRef_lnk
                                        Where RoutingObjRefId = '|| p_RecvRoutingObjRefId || ') tl '||
                            ' where v.FrRoutingObjRefId = fl.RoutingObjRefId
                              and   v.ToRoutingObjRefId = tl.RoutingObjRefId ';


         When p_InitRoutingObjRefId Is Not Null Then
                sql_inner := ' From
                                 CF_ValidObjInteract v ,
                                 (select * from cf_Routingobjref_lnk fl
                                        Connect by PRIOR fl.RoutingObjRefId = fl.RoutingObjParentRefId
                                        Start with fl.RoutingObjParentRefId = '|| p_InitRoutingObjRefId ||
                                      ' Union
                                        Select * From cf_RoutingObjRef_lnk
                                        Where RoutingObjRefId = '|| p_InitRoutingObjRefId || ') fl '||
                                ' where v.FrRoutingObjRefId = fl.RoutingObjRefId ';


         When p_RecvRoutingObjRefId Is Not Null Then
                 sql_inner := ' From
                                 CF_ValidObjInteract v ,
                                 (select * from cf_Routingobjref_lnk tl
                                        Connect by PRIOR tl.RoutingObjRefId = tl.RoutingObjParentRefId
                                        Start with tl.RoutingObjParentRefId = '|| p_RecvRoutingObjRefId ||
                                      ' Union
                                        Select * From cf_RoutingObjRef_lnk
                                        Where RoutingObjRefId = '|| p_RecvRoutingObjRefId ||') tl '||
                                ' where  v.ToRoutingObjRefId = tl.RoutingObjRefId ';

         Else
              v_GetAllFlg := 1;
    End Case;


----------------------
--  Build Where clause
----------------------

    v_where := ' Where 1=1 ';
    v_where2 := ' Where 1=1 ';
    where_clause;

    v_Select := 'select  v.InteractId,
                         v.FrRoutingObjRefId,
                         v.ToRoutingObjRefId,
                         CASE WHEN v.ENABLEDCD=7 Then 1 Else 0 END EnabledCd,
                         v.RowTS ';

    v_Select2 := 'select  t.InteractId,
                          t.FrRoutingObjRefId,
                          t.ToRoutingObjRefId,
                          t.EnabledCd,
                          t.RowTS ';

    If v_GetAllFlg = 0 Then
         sqlstmt := v_Select2 ||
               ' From ( '|| chr(10)||
                        v_Select || sql_inner || ') t ' || v_where2 ;  --||chr(10)||
              -- ' Union '|| chr(10)||
              --  v_Select || chr(10)||
              -- ' From   CF_ValidObjInteract v'|| chr(10) || v_where;
     Else  -- Get all valid Fr/ToRoutingObject interact
         sqlstmt := v_Select ||
                   ' From CF_ValidObjInteract v'||
                   v_Where;
     End If;


     db_Print('Sql before paging', sqlstmt);

--------------
-- Add SortKey
--------------
    If p_sortkey Is Not Null Then
         v_sortkey := upper(p_sortkey);
     --  v_sortkey := replace(v_sortkey,'FRPARTNERNAME','frp.partnername');
     --  v_sortkey := replace(v_sortkey,'TOPARTNERNAME','top.partnername');
         sqlstmt := sqlstmt ||' order by '||v_sortkey;
    End If;

    --db_Print('SqlSTMT', sqlSTMT);

-------------------------
-- Total row count Cursor
-------------------------
    If RC2%IsOpen Then Close RC2; End If;
    If NVL(p_pageNbr,0) In (0,1) Then
         Execute Immediate 'select count(*) From ('|| sqlstmt || ')' INTO v_count;
         --   db_Print('Sql_inner ', sql_inner);
         --   db_Print('Sql_totalrows ', sqlstmt);
         open RC2 for select v_count TotalRowsNbr from dual;
    End if;
-------------------
-- Add Paging logic
-------------------
--    Set_page_parameters(p_pageNbr, p_pageSizeNbr, sqlstmt);
    sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);

    db_Print('Sql paging', sqlstmt);
  --   dbms_output.put_line('sql pageing= '|| sqlstmt);

--------------
-- Result Set
--------------
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for sqlstmt;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - CF_ValidObjInteractS_AH1:'||SQLERRM);
------------
-- Set pages
------------
/*
Procedure Set_page_parameters(p_pageNumber in pls_integer,
                              p_pageSize   in pls_integer,
                              p_sqlstmt    in out varchar2) as
StartRow binary_integer:=0;
EndRow   binary_integer:=0;
Begin
  If nvl(p_pageNumber,0) in (0, 1) Then
    StartRow := 1;
    EndRow   := nvl(p_PageSize,10);
  else
    StartRow := ((nvl(p_PageSize,10) * p_pageNumber)+1) -  nvl(p_PageSize,10) ;
    EndRow   :=   nvl(p_PageSize,10) * p_pageNumber;
  end if;
    p_sqlstmt := 'select E.* from (select rownum rnbr, F.*  from ('||p_sqlstmt||') F ) E where rnbr between '||startRow||' and '||endRow;
End Set_page_parameters;
*/


End CF_ValidObjInteractS_AH1;
/


-- CF_VALIDOBJINTERACTS_AH2.sql

CREATE OR REPLACE PROCEDURE CF_ValidObjInteractS_AH2
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_RoutingObjRefId in NUMBER
    , p_RoleTypeCd in NUMBER      -- 1=Initiator, 2=Reciever,3=Both, -1=All
    )
AS

/*
    ================================================================================
    Proc Name:       CF_ValidObjInteractS_AH2
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns Fr/To RoutingObjId by given RoutingObjId
    2003-05-02 J.Chen        3.5    Replaced RoutingObjId with RoutingObjRefId
    2006-04-12 Shishir N     6.1    60217 Partner Migration
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ValidObjInteractS_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_RoutingObjRefId =>'||p_RoutingObjRefId||CHR(10)
        ||', p_RoleTypeCd =>'||p_RoleTypeCd||CHR(10)
        ;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
         IF RC1%isOpen THEN Close RC1; END IF;
         IF p_RoleTypeCd = 1 OR p_RoleTypeCd = 2 Then
              Open rc1 For
              Select
                        InteractId,
                        FrRoutingObjRefId,
                        ToRoutingObjRefId,
                        CASE WHEN ENABLEDCD=7 Then 1 Else 0 END ENABLEDCD,
                        RowTS
              From      CF_ValidObjInteract
              Where     p_RoutingObjRefId = Case When p_RoleTypeCd = 1 Then FrRoutingObjRefId
                                                 When p_RoleTypeCd = 2 Then ToRoutingObjRefId
                                            End
              And       CASE WHEN ENABLEDCD=7 Then 1 Else 0 END = 1
              ;
         End If;

         IF p_RoleTypeCd = 3 Then
              Open rc1 For
              Select
                        InteractId,
                        FrRoutingObjRefId,
                        ToRoutingObjRefId,
                        CASE WHEN ENABLEDCD=7 Then 1 Else 0 END ENABLEDCD,
                        RowTS
              From      CF_ValidObjInteract
              Where     (FrRoutingObjRefId = p_RoutingObjRefId
              Or        ToRoutingObjRefId = p_RoutingObjRefId)
              And       CASE WHEN ENABLEDCD=7 Then 1 Else 0 END = 1
              ;
          End If;
				IF p_RoleTypeCd = -1 Then
              Open rc1 For
              Select
							                        InteractId,
							                        FrRoutingObjRefId,
							                        ToRoutingObjRefId,
							                        CASE WHEN ENABLEDCD=7 Then 1 Else 0 END ENABLEDCD,
							                        RowTS
							              From      CF_ValidObjInteract
              Where     CASE WHEN ENABLEDCD=7 Then 1 Else 0 END = 1;
          End If;

         p_ReturnCd := 0;
         DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --       Commit;

         EXCEPTION
              When OTHERS Then
              p_ReturnCd := 1;
              v_ExceptionFlg := 1;
              DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
              RAISE_APPLICATION_ERROR (-20999,'20000-CF_ValidObjInteractS_AH2' || SQLCODE|| ' '||SQLERRM);
End CF_ValidObjInteractS_AH2;
/


-- CF_VALIDOBJINTERACTS_AH3.sql

CREATE OR REPLACE PROCEDURE         CF_ValidObjInteractS_AH3
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_FrRoutingObjRefId in NUMBER
    , p_ToRoutingObjRefId in NUMBER      -- 0 = from, 1 = to, 2 = both
    )
AS

/*
    ================================================================================
    Proc Name:       CF_ValidObjInteractS_AH3
    Create Date:     2003-05-06
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-06 J.Chen        3.5    Returns a valid interaction by given Fr/ToRoutingObjRefId
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ValidObjInteractS_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_FrRoutingObjRefId =>'||p_FrRoutingObjRefId||CHR(10)
        ||', p_ToRoutingObjRefId =>'||p_ToRoutingObjRefId||CHR(10)
        ;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
         IF RC1%isOpen THEN Close RC1; END IF;
              Open rc1 For
              Select
                        InteractId,
                        FrRoutingObjRefId,
                        ToRoutingObjRefId,
                        CASE WHEN ENABLEDCD=7 Then 1 Else 0 END ENABLEDCD,
                        RowTS
              From      CF_ValidObjInteract
              Where     FrRoutingObjRefId = p_FrRoutingObjRefId
              And       ToRoutingObjRefId = p_ToRoutingObjRefId
              ;


         p_ReturnCd := 0;
         DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    --     Commit;

         EXCEPTION
              When OTHERS Then
                   Rollback work;
              p_ReturnCd := 1;
              v_ExceptionFlg := 1;
              DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
              RAISE_APPLICATION_ERROR (-20999,'20000-CF_ValidObjInteractS_AH3' || SQLCODE|| ' '||SQLERRM);
End CF_ValidObjInteractS_AH3;
/


-- CF_VALIDOBJINTERACTS_AH4.sql

CREATE OR REPLACE PROCEDURE         CF_ValidObjInteractS_AH4
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_TransformMapId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_ValidObjInteractS_AH2
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-28 J.Chen        3.5    Returns valid interact by given TransforMapId
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ValidObjInteractS_AH4';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_TransformMapId =>'||p_TransformMapId||CHR(10)
        ;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
         IF RC1%isOpen THEN Close RC1; END IF;

              Open rc1 For
              Select
                        v.InteractId,
                        v.FrRoutingObjRefId,
                        v.ToRoutingObjRefId,
                        CASE WHEN v.ENABLEDCD=7 Then 1 Else 0 END ENABLEDCD,
                        v.RowTS
              From
                        CF_ValidObjInteract v
                        Inner Join CF_TransformInteract_lnk t on t.InteractId = v.InteractId
              Where
                        t.TransformMapId = p_TransformMapId;


         p_ReturnCd := 0;
         DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --       Commit;

         EXCEPTION
              When No_data_Found Then
                    p_ReturnCd := 3;
              When OTHERS Then
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                RAISE_APPLICATION_ERROR (-20999,'20000-CF_ValidObjInteractS_AH4' || SQLCODE|| ' '||SQLERRM);
End CF_ValidObjInteractS_AH4;
/


-- CF_VALIDOBJINTERACTS_AH5.sql

CREATE OR REPLACE PROCEDURE CF_VALIDOBJINTERACTS_AH5
    (
     p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , RC2  out  db_ref.RCT
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_routingobjrefid in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_ValidObjInteractS_AH5
    Create Date:     2007-04-08
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-04-08 BHARANI J        1   Returns valid interaction by given Interactid
    2008-09-12 Amit Kumar Vijay	6.2 Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/



v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ValidObjInteractS_AH5';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_routingobjrefid =>'||p_routingobjrefid||CHR(10)
        ||', p_sortkey=>'||p_sortkey||CHR(10)
	||', p_PageSizeNbr=>'||p_PageSizeNbr||CHR(10)
        ||', p_PageNbr=>'||p_PageNbr||CHR(10)
        ;
sqlstmt            varchar2(4000);
v_Column         varchar2(4000);
v_sqlstmt          varchar2(4000); -- includes from clause, inner joins and some outer joins that effect where clause
v_sortkey          varchar2(4000);
v_where            varchar2(4000);


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	v_where := ' (SELECT RoutingObjRefId '
			|| 'FROM	CF_ROUTINGOBJECT06_VW '
			|| 'WHERE	ROUTINGOBJREFID = ' || '''' || p_routingobjrefid || ''''
			|| 'OR ROUTINGOBJPARENTREFID = ' || '''' || p_routingobjrefid || ''''
			|| 'OR GRPARENTOBJREFID = ' || '''' || p_routingobjrefid || ''''
			|| 'OR GRPARENTOBJREFID2 = ' || '''' || p_routingobjrefid || ''''
			|| 'OR GRPARENTOBJREFID3 = ' || '''' || p_routingobjrefid || ''''
			|| ') '
	;

  sqlstmt := 'SELECT  v.InteractId '
                  ||', v.FrRoutingObjRefId '
                  ||', v.ToRoutingObjRefId '
                  ||', CASE WHEN v. ENABLEDCD=7 Then 1 Else 0 END ENABLEDCD '
                  ||' From CF_ValidObjInteract v '
                  || ' WHERE v.frroutingobjrefid IN '|| v_where
	          || ' OR v.toroutingobjrefid In ' || v_where;

  if RC2%IsOpen then close RC2; end if;
         if NVL(p_pageNbr,0) in (0,1) then
            open RC2 for 'select count(*) TotalRowsNbr from (' || sqlstmt || ')' ;
    end if;

 --------------
 -- Add SortKey
 --------------

   -- Order by Logic
       if p_sortkey is not null then
           sqlstmt := sqlstmt ||' order by '||p_sortkey;
       end if;

  -------------------
  -- Add Paging logic
  -------------------

    sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);

    IF RC1%isOpen THEN Close RC1; END IF;
	    Open RC1 for sqlstmt ;





   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

   p_ReturnCd := 0;
         EXCEPTION
              When No_data_Found Then
                    p_ReturnCd := 3;
              When OTHERS Then
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                RAISE_APPLICATION_ERROR (-20999,'20000-CF_ValidObjInteractS_AH5' || SQLCODE|| ' '||SQLERRM);
End CF_ValidObjInteractS_AH5;
/

-- CF_VALIDOBJINTERACTS_AH6.sql

CREATE OR REPLACE PROCEDURE CF_VALIDOBJINTERACTS_AH6
    (
     p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , RC2  out  db_ref.RCT
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_interactid in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       CF_VALIDOBJINTERACTS_AH6
    Create Date:     2007-08-20
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-04-08 BHARANI J        1   Returns valid interaction by given Interactid
    2008-09-12 Amit Kumar 	 6.2	Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'CF_ValidObjInteractS_AH6';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_interactid =>'||p_interactid||CHR(10)
        ||', p_sortkey=>'||p_sortkey||CHR(10)
	||', p_PageSizeNbr=>'||p_PageSizeNbr||CHR(10)
        ||', p_PageNbr=>'||p_PageNbr||CHR(10)
        ;
sqlstmt            varchar2(4000);
sql_Column         varchar2(4000);
v_column          varchar2(4000); -- includes from clause, inner joins and some outer joins that effect where clause
v_sortkey          varchar2(4000);
v_where            varchar2(4000);


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN



   v_where := ' Where 1=1'||
   FN_Build_Where('INTERACTID', 'NUMBER'   , p_interactid);
  
  v_column := 'SELECT v.FrRoutingObjRefId 
             , v.ToRoutingObjRefId 
             , (select partnername from pr_partner where partnerid=c.frpartnerid) FROMPARTNERNAME 
             , (select partnername from pr_partner where partnerid=c.topartnerid) TOPARTNERNAME 
             ,  CASE WHEN c.activestatuscd=63 THEN 1 ELSE 0 END ACTIVESTATUSCD
              From CF_ValidObjInteract v
              INNER JOIN pr_capability pf ON v.FRROUTINGOBJREFID=pf.routingobjrefid
              INNER JOIN pr_capability pt ON v.TOROUTINGOBJREFID=pt.routingobjrefid
              INNER JOIN pr_routingchannel c ON c.FRCAPABILITYID= pf.CAPABILITYID AND c.TOCAPABILITYID=pt.CAPABILITYID ';

	sqlstmt := 	v_column || v_where;

	if RC2%IsOpen then close RC2; end if;
		if NVL(p_pageNbr,0) in (0,1) then
		open RC2 for 'select count(*) TotalRowsNbr from (' || sqlstmt || ')';
	end if;
   --------------
   -- Add SortKey
   --------------

     -- Order by Logic
	if p_sortkey is not null then
             sqlstmt := sqlstmt ||' order by '||p_sortkey;
	end if;

  -------------------
  -- Add Paging logic
  -------------------
  --    Set_page_parameters(p_pageNbr, p_pageSizeNbr, sqlstmt);
	sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);

	db_Print('Sql paging', sqlstmt);

	IF RC1%isOpen THEN Close RC1; END IF;
		Open RC1 for sqlstmt;

	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;

  --       Commit;

         EXCEPTION
              When No_data_Found Then
                    p_ReturnCd := 3;
              When OTHERS Then
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                RAISE_APPLICATION_ERROR (-20999,'20000-CF_ValidObjInteractS_AH6' || SQLCODE|| ' '||SQLERRM);
End CF_ValidObjInteractS_AH6;
/

-- CF_VALIDOBJINTERACTU_AH1.sql

CREATE OR REPLACE PROCEDURE  CF_VALIDOBJINTERACTU_AH1
    (
     p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    ,p_INTERACTID in NUMBER
    ,p_ENABLEDCD in NUMBER
    ,p_ROWTS in NUMBER
) as

/*
    ================================================================================
    Proc Name:       CF_VALIDOBJINTERACTU_AH1
    Create Date:     2003-05-06 15:46:31
    Author:          J.Chen

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------

	2008-09-12 Amit Kumar 	 6.2	Defect 47980, 106065 and 106066

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'CF_VALIDOBJINTERACTU_AH1';
v_ExceptionFlg    integer := 0;
v_ENABLEDCD	  number;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_INTERACTID=>'||p_INTERACTID||CHR(10)
        ||', p_ENABLEDCD=>'||p_ENABLEDCD||CHR(10)
        ||', p_ROWTS=>'||p_ROWTS||CHR(10)
        ;

v_RowTS      integer :=0;
zeroUpdate  Exception;

CURSOR CHANNEL_UPDATE 
	IS
SELECT ACTIVESTATUSCD, UPDATEDATE, CHANNELID from pr_RoutingChannel
Where
	         CHANNELID in (  Select c.ChannelId
	                            From    cf_ValidObjInteract v
	                            Inner join Pr_Capability f on v.FrRoutingObjRefId = f.RoutingObjRefId
	                            Inner join Pr_Capability t on v.ToRoutingObjRefId = t.RoutingObjRefId
	                            Inner join pr_RoutingChannel c on c.frCapabilityId = f.CapabilityId
	                                                          and c.ToCapabilityId = t.CapabilityId
	                            Where   v.InteractId = p_InteractId
                            ) FOR UPDATE;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_RowCnt := 0;
    BEGIN
    	SELECT ENABLEDCD into v_ENABLEDCD 
	    	from CF_VALIDOBJINTERACT
	    WHERE INTERACTID = P_INTERACTID
	        AND ROWTS = P_ROWTS;
	EXCEPTION
		When NO_DATA_FOUND THEN
		v_RowCnt := 0;
    END;

    
    IF p_ENABLEDCD=0 THEN
    
    	FOR CU in CHANNEL_UPDATE LOOP
    		BEGIN
    			Update PR_ROUTINGCHANNEL
	    			SET ACTIVESTATUSCD=CASE WHEN cu.ACTIVESTATUSCD in (4,5,6,7,12,13,14,15,20,21,22,23,28,29,30,31,36,37,38,39,44,45,46,47,52,53,54,55,60,61,62,63) THEN cu.ACTIVESTATUSCD-4 ELSE cu.ACTIVESTATUSCD END
	      	        	,	UPDATEDATE = v_BeginTS
	    		Where
	         		CHANNELID = CU.CHANNELID;
	         	
		END;
	END LOOP;
    	
    	Update CF_VALIDOBJINTERACT
    	    SET    ENABLEDCD=6
    	    ,      ROWTS = p_RowTS + 1
    	    ,      UPDATEDATE = FN_GET_DATE()
    	Where
    	    INTERACTID = p_INTERACTID
    	    and ENABLEDCD = 7
    	    And RowTS = p_RowTS returning RowTS into v_RowTS;
    	v_RowCnt := SQL%rowcount;
    	
    ELSE
    	
        FOR CU in CHANNEL_UPDATE LOOP
        	BEGIN
        		Update PR_ROUTINGCHANNEL
    				SET ACTIVESTATUSCD=CASE WHEN cu.ACTIVESTATUSCD in (0,1,2,3,8,9,10,11,16,17,18,19,24,25,26,27,32,33,34,35,40,41,42,43,48,49,50,51,56,57,58,59) THEN cu.ACTIVESTATUSCD+4 ELSE cu.ACTIVESTATUSCD END
    	      	        	,      UPDATEDATE = v_BeginTS
    	    		Where
    	         		CHANNELID = CU.CHANNELID;
    	         	
		END;
	END LOOP;
	
	Update CF_VALIDOBJINTERACT
		SET    ENABLEDCD=7
	   	,      ROWTS = p_RowTS + 1
	    	,      UPDATEDATE = FN_GET_DATE()
	Where
	    INTERACTID = p_INTERACTID
	    and ENABLEDCD = 6
	    And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
		
    END IF;
    	
    IF RC1%isOpen THEN Close RC1; END IF;
    
    if (v_RowCnt = 0) then 
    	OPEN RC1 for select p_RowTS RowTS from Dual;
    else
    	OPEN RC1 for select v_RowTS RowTS from Dual;
    end if;
    
    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        WHEN TOO_MANY_ROWS Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999, '20100 - CF_VALIDOBJINTERACTU_AH1: Too Many Rows');
        WHEN ZeroUpdate then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - CF_VALIDOBJINTERACTU_AH1: No Data Found');
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - CF_VALIDOBJINTERACTU_AH1:'||SQLERRM);
End CF_VALIDOBJINTERACTU_AH1;
/


-- CF_VALIDOBJINTERACTU_AH2.sql

CREATE OR REPLACE PROCEDURE CF_VALIDOBJINTERACTU_AH2
(
   p_returncd     OUT      INTEGER,
   p_debugflg     IN       PLS_INTEGER,
   p_interactid   IN       NUMBER
)
AS
/*================================================================================
    Proc Name:       CF_VALIDOBJINTERACTU_AH2
    Create Date:     2007-08-20
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-04-08 BHARANI J        1   Returns valid interaction by given Interactid
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

   v_rowcnt         INTEGER         := 0;
   v_begints        TIMESTAMP       := fn_get_date ();
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   := 'CF_VALIDOBJINTERACTU_AH2';
   v_exceptionflg   INTEGER         := 0;
   v_paramstxt      VARCHAR2 (4000) := 'p_DebugFlg=>' || p_debugflg || CHR (10) ||
                                       ', p_INTERACTID=>' || p_interactid || CHR (10)
                                       ;

   zeroDelete  Exception;

BEGIN

	DELETE FROM pr_workflowinteract_lnk p
	WHERE p.interactid = p_interactid;

	DELETE FROM cf_transforminteract_lnk t
	WHERE t.interactid = p_interactid;

	DELETE FROM PR_CHANNEL_GATEWAY_LNK pc
	WHERE pc.channelid IN(
		SELECT frca.channelid
		FROM cf_validobjinteract v
                        INNER JOIN cf_routingobjref_lnk FRRO ON v.frroutingobjrefid=FRRO.routingobjrefid
                        INNER JOIN cf_routingobjref_lnk TORO ON v.frroutingobjrefid=TORO.routingobjrefid
                        INNER JOIN pr_capability p ON p.routingobjrefid = FRRO.routingobjrefid
                        AND p.routingobjrefid = TORO.routingobjrefid
                        INNER JOIN pr_routingchannel frca ON p.capabilityid = frca.frcapabilityid
                        INNER JOIN pr_routingchannel toca ON p.capabilityid = toca.frcapabilityid
                       WHERE v.interactid =  p_interactid );

	DELETE FROM pr_routingattr_lnk prl
	WHERE prl.channelid IN (
		SELECT frca.channelid
		FROM cf_validobjinteract v
                        INNER JOIN cf_routingobjref_lnk FRRO ON v.frroutingobjrefid=FRRO.routingobjrefid
                        INNER JOIN cf_routingobjref_lnk TORO ON v.frroutingobjrefid=TORO.routingobjrefid
                        INNER JOIN pr_capability p ON p.routingobjrefid = FRRO.routingobjrefid
                        AND p.routingobjrefid = TORO.routingobjrefid
                        INNER JOIN pr_routingchannel frca ON p.capabilityid = frca.frcapabilityid
                        INNER JOIN pr_routingchannel toca ON p.capabilityid = toca.frcapabilityid
                       WHERE v.interactid =  p_interactid );

	DELETE FROM pr_routingchannel pr
	WHERE pr.channelid IN (
		SELECT frca.channelid
		FROM cf_validobjinteract v
                        INNER JOIN cf_routingobjref_lnk FRRO ON v.frroutingobjrefid=FRRO.routingobjrefid
                        INNER JOIN cf_routingobjref_lnk TORO ON v.frroutingobjrefid=TORO.routingobjrefid
                        INNER JOIN pr_capability p ON p.routingobjrefid = FRRO.routingobjrefid
                        AND p.routingobjrefid = TORO.routingobjrefid
                        INNER JOIN pr_routingchannel frca ON p.capabilityid = frca.frcapabilityid
                        INNER JOIN pr_routingchannel toca ON p.capabilityid = toca.frcapabilityid
                       WHERE v.interactid =  p_interactid );


	DELETE FROM cf_validobjinteract v
	WHERE v.interactid = p_interactid;

	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;

	db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            SQLERRM || CHR (10) || v_paramstxt,
                            p_debugflg,
                            1
                           );


	p_returncd := 0;

	EXCEPTION
	When ZERODELETE THEN
   			v_ExceptionFlg := 1;
   			p_ReturnCd := 3;
   			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_GATEWAYD: No Data Found');
	WHEN OTHERS  THEN
  		p_returncd := 1;
      		v_exceptionflg := 1;
      		db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            SQLERRM || CHR (10) || v_paramstxt,
                            p_debugflg,
                            1
                           );
        ROLLBACK WORK;
	raise_application_error (-20999,
                               '20100 - CF_VALIDOBJINTERACTU_AH2:' || SQLERRM
                              );
END cf_validobjinteractu_ah2;
/

-- CF_VARWORKFLOWHANDLERATTR_LNKD.sql

CREATE OR REPLACE Procedure CF_VARWORKFLOWHANDLERATTR_LNKD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_WORKFLOWID in NUMBER
	,p_HANDLERID in NUMBER
   ,p_ORDERING in NUMBER
	,p_HANDLERATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_VARWORKFLOWHANDLERATTR_LNKD
	Create Date:     2004-03-14 17:20:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_VARWORKFLOWHANDLERATTR_LNKD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
  
   v_where := ' WHERE 1=1 ' ;
   v_where := v_where || FN_BUILD_WHERE('WORKFLOWID', 'NUMBER', p_WORKFLOWID) ;
	v_where := v_where || FN_BUILD_WHERE('ORDERING', 'NUMBER',p_ORDERING) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;
	
   v_sqlstmt := 'Delete From CF_VARWORKFLOWHANDLERATTR_LNK ' || v_where ;
	
	execute immediate v_sqlstmt ;

	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_VARWORKFLOWHANDLERATTR_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_VARWORKFLOWHANDLERATTR_LNKD;
/


-- CF_VARWORKFLOWHANDLERATTR_LNKI.sql

CREATE OR REPLACE Procedure CF_VARWORKFLOWHANDLERATTR_LNKI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT 
	,p_WORKFLOWID in NUMBER
	,p_HANDLERID in NUMBER
    ,p_ORDERING in NUMBER
	,p_HANDLERATTRID in NUMBER
 
	,p_ATTRVALUE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       CF_VARWORKFLOWHANDLERATTR_LNKI
	Create Date:     2004-03-14 17:22:19
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_VARWORKFLOWHANDLERATTR_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_VARWORKFLOWHANDLERATTR_LNK(
		WORKFLOWID
		,HANDLERID
		,ORDERING
		,HANDLERATTRID
		,ATTRVALUE
	) Values (
		p_WORKFLOWID
		,p_HANDLERID
		,p_ORDERING
		,p_HANDLERATTRID
		,p_ATTRVALUE
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_VARWORKFLOWHANDLERATTR_LNKI' || SQLCODE|| ' '||SQLERRM);
END CF_VARWORKFLOWHANDLERATTR_LNKI;
/


-- CF_VARWORKFLOWHANDLERATTR_LNKS.sql

CREATE OR REPLACE PROCEDURE CF_VARWORKFLOWHANDLERATTR_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_WORKFLOWID IN NUMBER
	, p_HANDLERID IN NUMBER
   , p_ORDERING IN NUMBER
	, p_HANDLERATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_VARWORKFLOWHANDLERATTR_LNKS
	Create Date:     2004-03-14 17:21:53
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created
   2004-04-26 PAD           4.2.2  Added join on handlerid, returning invalid data
	2004-05-24 PAD           4.2.2  Added v_where_inner logic
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
   
   **NOTE - if no handler or workflow is given ALL handler default
            handler attributes will be returned
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_VARWORKFLOWHANDLERATTR_LNKS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;        
v_where_inner     varchar2(4000) ;        
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' WHERE 1=1 ' ;
    
    if p_HANDLERATTRID is not null then
       v_where := v_where || FN_BUILD_WHERE('RA.HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where := v_where || FN_BUILD_WHERE('RA.HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_sqlstmt :=
       'SELECT '
	   ||'WORKFLOWID'
	   ||',RA.HANDLERID'
   	   ||',ORDERING'
	   ||',RA.HANDLERATTRID'
       ||',RTRIM(ATTRNAME) ATTRNAME'
       ||',RTRIM(ATTRVALUE) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',RA.ROWTS'
       ||',RA.CREATEDATE '
       ||'FROM '
       ||'CF_VARWORKFLOWHANDLERATTR_LNK RA INNER JOIN PR_HANDLERATTR HA '
       ||'ON RA.HANDLERATTRID = HA.HANDLERATTRID '
       ||'AND RA.HANDLERID = HA.HANDLERID '
       || v_where
	   || ' ORDER BY WORKFLOWID, RA.HANDLERID, ORDERING' 
	   ;
    ELSE
       v_where_inner := ' WHERE 1=1 ' ;
       v_where_inner := v_where_inner || FN_BUILD_WHERE('WORKFLOWID', 'NUMBER', p_WORKFLOWID) ;
       v_where_inner := v_where_inner || FN_BUILD_WHERE('ORDERING', 'NUMBER',p_ORDERING) ;
       v_where_inner := v_where_inner || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where_inner := v_where_inner || FN_BUILD_WHERE('HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_where := v_where || FN_BUILD_WHERE('HA.HANDLERID', 'NUMBER', p_HANDLERID) ;
	    v_where := v_where || FN_BUILD_WHERE('HA.HANDLERATTRID', 'NUMBER', p_HANDLERATTRID) ;

       v_sqlstmt :=
       'SELECT '
       ||'WORKFLOWid'
	    ||',NVL(RA1.HANDLERID, ha.HANDLERID) HANDLERID '
   	 ||',ORDERING'
       ||',NVL(ra1.HANDLERATTRID, ha.HANDLERATTRID) HANDLERATTRID '
       ||',RTRIM(attrname) ATTRNAME'
       ||',RTRIM(NVL(attrvalue, defaultattrvalue)) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',ra1.ROWTS'
       ||',ra1.CREATEDATE '
       ||'from '
       ||'(select WORKFLOWid, ordering, handlerid, handlerattrid, attrvalue, rowts, createdate '
       ||' from cf_VARWORKFLOWHANDLERattr_lnk RA'||v_where_inner||') RA1 '
       ||'right outer join pr_handlerattr HA '
       ||'on RA1.handlerattrid = HA.handlerattrid '
       ||v_where
	    ||' ORDER BY WORKFLOWID, HANDLERID, ORDERING' 
       ;
    end if ;     
	
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_VARWORKFLOWHANDLERATTR_LNKS;
/


-- CF_VARWORKFLOWHANDLERATTR_LNKU.sql

CREATE OR REPLACE Procedure CF_VARWORKFLOWHANDLERATTR_LNKU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_WORKFLOWID in NUMBER
	,p_HANDLERID in NUMBER
	,p_ORDERING in NUMBER
	,p_HANDLERATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       CF_VARWORKFLOWHANDLERATTR_LNKU
	Create Date:     2004-03-14 17:21:33
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_VARWORKFLOWHANDLERATTR_LNKU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update CF_VARWORKFLOWHANDLERATTR_LNK
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      ATTRVALUE=p_ATTRVALUE
	Where 
		WORKFLOWID = p_WORKFLOWID
		AND ORDERING = p_ORDERING
		AND HANDLERID = p_HANDLERID
		AND HANDLERATTRID = p_HANDLERATTRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - CF_VARWORKFLOWHANDLERATTR_LNKU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_VARWORKFLOWHANDLERATTR_LNKU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - CF_VARWORKFLOWHANDLERATTR_LNKU:'||SQLERRM);
End CF_VARWORKFLOWHANDLERATTR_LNKU;
/


-- CF_VARWORKFLOWHANDLER_XREFD.sql

CREATE OR REPLACE Procedure CF_VARWORKFLOWHANDLER_XREFD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
   ,p_WORKFLOWID in NUMBER
	,p_HANDLERID in NUMBER
	,p_ORDERING in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_VARWORKFLOWHANDLER_XREFD
	Create Date:     2004-03-14 15:35:08
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_VARWORKFLOWHANDLER_XREFD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   
   v_where := ' WHERE 1=1 ' ;
   v_where := v_where || FN_BUILD_WHERE('WORKFLOWID', 'NUMBER', p_WORKFLOWID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	v_where := v_where || FN_BUILD_WHERE('ORDERING', 'NUMBER',p_ORDERING) ;
	
   v_sqlstmt := 'Delete From CF_VARWORKFLOWHANDLER_XREF ' || v_where ;
	
	execute immediate v_sqlstmt ;
		
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_VARWORKFLOWHANDLER_XREFD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_VARWORKFLOWHANDLER_XREFD;
/


-- CF_VARWORKFLOWHANDLER_XREFI.sql

CREATE OR REPLACE Procedure CF_VARWORKFLOWHANDLER_XREFI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT   
	,p_WORKFLOWID in NUMBER
	,p_HANDLERID in NUMBER
	,p_ORDERING in NUMBER

	)
AS

/*
	================================================================================
	Proc Name:       CF_VARWORKFLOWHANDLER_XREFI
	Create Date:     2004-03-14 15:35:45
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_VARWORKFLOWHANDLER_XREFI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into CF_VARWORKFLOWHANDLER_XREF(
		WORKFLOWID
		,HANDLERID
		,ORDERING
	) Values (
		p_WORKFLOWID
		,p_HANDLERID
		,p_ORDERING
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_VARWORKFLOWHANDLER_XREFI' || SQLCODE|| ' '||SQLERRM);
END CF_VARWORKFLOWHANDLER_XREFI;
/


-- CF_VARWORKFLOWHANDLER_XREFS.sql

CREATE OR REPLACE PROCEDURE CF_VARWORKFLOWHANDLER_XREFS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_WORKFLOWID IN NUMBER
	, p_HANDLERID IN NUMBER
	, p_ORDERING IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       CF_VARWORKFLOWHANDLER_XREFS
	Create Date:     2004-03-14 15:36:13
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-14 SYSTEM           3.5 Created
	2004-04-14 S.Priani       4.2.2 BGTRK00046224 Removed ordering on HandlerId
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_VARWORKFLOWHANDLER_XREFS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;           
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		||', p_ORDERING=>'||p_ORDERING||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('WORKFLOWID', 'NUMBER', p_WORKFLOWID) ||
       FN_Build_Where('ORDERING', 'NUMBER', p_ORDERING) ||
       FN_Build_Where('rx.HANDLERID', 'NUMBER', p_HANDLERID);
	   
	v_sqlstmt := 
       'SELECT '
        ||'WORKFLOWID'
		||',rx.HANDLERID'
		||',ORDERING'
		||',PROVIDERFLG'
		||',RESOURCEKEY'
		||',DESCRIPTION'
		||',CLASSNAME'
		||',rx.ROWTS'
		||',rx.CREATEDATE '
		||'FROM '
		||'CF_VARWORKFLOWHANDLER_XREF rx INNER JOIN PR_HANDLER h ' 
		||'ON rx.HANDLERID = h.HANDLERID '
		||v_where 
		||' ORDER BY WORKFLOWID, ORDERING'
		;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End CF_VARWORKFLOWHANDLER_XREFS;
/


-- CF_WDI_EDIROATTRIBUTES.sql

CREATE OR REPLACE PROCEDURE CF_WDI_EDIROAttributes
	(p_ReturnCd OUT PLS_INTEGER
	,p_SRCDICTNAME IN NCHAR
	,p_SRCDOCNAME IN NCHAR
	)
AS
/*
	================================================================================
	Proc Name:       CF_WDI_EDIROAttributes
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        NUMBER := 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_WDI_EDIROAttributes';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
 ' p_SRCDICTNAME=>'||p_SRCDICTNAME||CHR(10)
 ||', p_SRCDOCNAME=>'||p_SRCDOCNAME||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

--55533 - this attribute only gets associated with the proccess types for EDI
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 	
		SELECT  RoutingobjrefId,(SELECT RoutingAttrId
    	                         FROM cf_routingAttr 
                               WHERE AttributeKey = 	'bcg.ro.edi.farequired'	)
     	FROM cf_routingobject03_vw crv
     	WHERE crv.routingobjcd = p_SRCDOCNAME AND
    	      crv.routingobjver = 'ALL' AND
     		   crv.parentobjcd = p_SRCDICTNAME
   	AND NOT EXISTS (SELECT crl.routingobjrefid, crl.routingattrid 
                      FROM CF_RoutingObjAttr_Lnk crl, cf_routingattr, cf_routingobject03_vw crv
    			          WHERE crl.ROUTINGOBJREFID = crv.ROUTINGOBJREFID AND
    			                crv.routingobjcd = p_SRCDOCNAME AND
    			                crv.routingobjver = 'ALL' AND
     			                crv.parentobjcd = p_SRCDICTNAME AND
	 		                   cf_routingattr.AttributeKey = 	'bcg.ro.edi.farequired'	 AND
     			                cf_routingattr.routingAttrid = crl.ROUTINGATTRID);
	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.EDISEGDLM'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.EDISEGDLM'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.EDIDEDLM'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.EDIDEDLM'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.EDISEDLM'	)
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.EDISEDLM'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.EDIRLSCHAR'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.EDIRLSCHAR'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.EDIDESEP'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.EDIDESEP'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.EDIDECNOT'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.EDIDECNOT'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.SEGOUTPUT'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.SEGOUTPUT'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.ALLOWDUPLICATES'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.ALLOWDUPLICATES'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.TRANSFORMATIONERRLEVEL'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.TRANSFORMATIONERRLEVEL'	);
                                                                              
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.FAMAPNAME'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.FAMAPNAME'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.EnvelopeProfile'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.EnvelopeProfile'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.VALERRLEVEL'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALERRLEVEL'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.VALLEVEL'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALLEVEL'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.VALCHARSET'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALCHARSET'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.VALALPHANUM'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALALPHANUM'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.GRPLVLFA'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.GRPLVLFA'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.VALCTRLYY'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALCTRLYY'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.VALSERVSEGVAL'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALSERVSEGVAL'	);	 
                                                                              
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.CONNECTIONQUALIFIER'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.CONNECTIONQUALIFIER'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.INTERCHANGEQUAL'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.INTERCHANGEQUAL'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.INTERCHANGEID'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.INTERCHANGEID'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.ICHGUSAGEINDICATOR'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.ICHGUSAGEINDICATOR'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.GRPAPPSENDERID'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.GRPAPPSENDERID'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.GRPAPPRECEIVERID'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.GRPAPPRECEIVERID'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.ICHGREVROUTE'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.ICHGREVROUTE'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.ICHGROUTEADDR'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.ICHGROUTEADDR'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.GRPAPPSENDERQUAL'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.GRPAPPSENDERQUAL'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.GRPAPPRECEIVERQUAL'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.GRPAPPRECEIVERQUAL'	);	 
INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.edi.GRPAPPPASSWORD'	) 
     FROM cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
      WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
                                                                              WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.GRPAPPPASSWORD'	);
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.EnvelopeProfile'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDOCNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.EnvelopeProfile'	);	 
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.VALERRLEVEL'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDOCNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALERRLEVEL'	);	 

Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.VALLEVEL'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDOCNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALLEVEL'	);	 

Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.VALCHARSET'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDOCNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALCHARSET'	);	 

Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.VALALPHANUM'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDOCNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.VALALPHANUM'	);	 

Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.CONNECTIONQUALIFIER'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDOCNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.CONNECTIONQUALIFIER'	);	 

Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'GuidelineMap'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDOCNAME 
									      AND a.AttributeKey = 	'GuidelineMap' );
-- add default attribute values
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, '10' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALCTRLYY'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALCTRLYY'
               and prl.AttrValue = '10'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));
                                         
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, 'CHARSET' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALCHARSET'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALCHARSET'
               and prl.AttrValue = 'CHARSET'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));

Insert into PR_RoutingAttr_Lnk ( RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, 'ALPHANUM' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALALPHANUM'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALALPHANUM'
               and prl.AttrValue = 'ALPHANUM'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));
 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.SEGOUTPUT'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.SEGOUTPUT'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));
 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));
 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.GRPLVLFA'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.GRPLVLFA'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));
 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.DISCARDONERROR'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.DISCARDONERROR'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));

Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, 1, v_ExceptionFlg);
			EXCEPTION
					WHEN NO_DATA_FOUND THEN
						v_ExceptionFlg := 1;
						Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, 1, 0);
						p_returnCd := 3;
					WHEN DUP_VAL_ON_INDEX THEN
						v_ExceptionFlg := 1;
						Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, 1, v_ExceptionFlg);
						p_returnCd := 2;
					WHEN OTHERS THEN
						p_ReturnCd := 1;
						v_ExceptionFlg := 1;
						Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, 1, 1);
					RAISE_APPLICATION_ERROR (-20999,'20002 CF_WDI_EDIROAttributes' || SQLCODE|| ' '||SQLERRM);
END CF_WDI_EDIROAttributes;
/

-- CF_WDI_INSERTMAP.sql

CREATE OR REPLACE PROCEDURE CF_WDI_InsertMap
	(p_ReturnCd OUT PLS_INTEGER
	,p_MAPID IN NCHAR
	,p_MAPTYPE IN NCHAR
	,p_DESCRIPT IN NCHAR
	,p_SRCDICTNAME IN NCHAR
	,p_TRGDICTNAME IN NCHAR
	,p_SRCDOCNAME IN NCHAR
	,p_TRGDOCNAME IN NCHAR
	,p_SMAPPKGNAME IN VARCHAR2
	,p_TMAPPKGNAME IN VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       CF_WDI_InsertMap
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        NUMBER := 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_WDI_InsertMap';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		' p_MAPID=>'||p_MAPID||CHR(10)
		||', p_MAPTYPE=>'||p_MAPTYPE||CHR(10)
		||', p_DESCRIPT=>'||p_DESCRIPT||CHR(10)
		||', p_SRCDOCNAME=>'||p_SRCDOCNAME||CHR(10)
		||', p_TRGDOCNAME=>'||p_TRGDOCNAME||CHR(10)
		||', p_SRCDICTNAME=>'||p_SRCDICTNAME||CHR(10)
		||', p_TRGDICTNAME=>'||p_TRGDICTNAME||CHR(10)
		||', p_SMAPPKGNAME=>'||p_SMAPPKGNAME||CHR(10)
		||', p_TMAPPKGNAME=>'||p_TMAPPKGNAME||CHR(10)
		;
v_TRANSFORMMAPID  Integer Default 0;
v_GUIDELINEMAPID  Integer Default 0;
v_FAMAPID  	  Integer Default 0;
v_FROMROUTINGOBJREFID  Integer Default 0;
v_TOROUTINGOBJREFID  Integer Default 0;
v_aFROMROUTINGOBJREFID  Integer Default 0;
v_aTOROUTINGOBJREFID  Integer Default 0;
v_ROUTINGOBJID  	  Integer Default 0;
v_INTERACTID  	  Integer Default 0;
v_count           Integer Default 0;
c_wrongrouting exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
	--Transformation Map
	BEGIN
	if p_MAPTYPE = 'A' THEN
	   BEGIN
	   	   		---Get v_FromRoutingobjRefID
					Select routingobjrefid into v_FROMROUTINGOBJREFID from cf_routingobject03_vw where 
					routingobjtypekey = 'Process' and
					routingobjcd = p_SRCDOCNAME and 
					Routingobjver = 'ALL' and
					parentobjtypekey = 'Protocol' and
					parentobjcd = p_SRCDICTNAME and
					parentobjver = 'ALL' and
        				GRPARENTOBJTYPEKEY = 'Package' and
        				GRPARENTOBJCD = p_SMAPPKGNAME;
					
			---Get v_ToRoutingobjRefID	
				Select routingobjrefid into v_TOROUTINGOBJREFID from cf_routingobject03_vw where 
					routingobjtypekey = 'Process' and
					routingobjcd = p_TRGDOCNAME and 
					Routingobjver = 'ALL' and
					parentobjtypekey = 'Protocol' and
					parentobjcd = p_TRGDICTNAME and
					parentobjver = 'ALL'and
        				GRPARENTOBJTYPEKEY = 'Package' and
        				GRPARENTOBJCD = p_TMAPPKGNAME;

	   Select count(*) into v_count From CF_TRANSFORMMAP
	                   where TRANSFORMMAPFILENAME = p_MAPID;
	   If v_count > 0 Then
	   BEGIN
	   			select FROMROUTINGOBJREFID
	   			,TOROUTINGOBJREFID
	   			into
	   			v_aFROMROUTINGOBJREFID
	   			,v_aTOROUTINGOBJREFID
	   			from
	   			CF_TRANSFORMROUTINGOBJ_LNK ctl
	   			inner join CF_TRANSFORMMAP ct on ctl.TRANSFORMMAPID = ct.TRANSFORMMAPID
	   			where ct.TRANSFORMMAPFILENAME = p_MAPID;
	   		if ((v_aFROMROUTINGOBJREFID = v_FROMROUTINGOBJREFID) and (v_aTOROUTINGOBJREFID = v_TOROUTINGOBJREFID))
		then
	          Update CF_TRANSFORMMAP
	          SET TRANSFORMMAPDESC = p_DESCRIPT
	          ,      ROWTS = RowTS + 1
	          where TRANSFORMMAPFILENAME = p_MAPID;
	          else
		          	RAISE c_wrongrouting;
		          	end if;
        	end;
       else
			  INSERT INTO CF_TRANSFORMMAP(
					TRANSFORMMAPID
					,TRANSFORMMAPFILENAME
					,TRANSFORMMAPDESC
					,TRANSFORMMAP
				) VALUES (
					CF_TRANSFORMMAP_SEQ.NextVal
					,p_MAPID
					,p_DESCRIPT
					,UTL_RAW.CAST_TO_RAW(p_MAPID)
				);
		SELECT CF_TRANSFORMMAP_SEQ.CURRVAL into v_TRANSFORMMAPID from dual;



	INSERT INTO CF_TRANSFORMROUTINGOBJ_LNK(
		TRANSFORMMAPROLNKID
		,TRANSFORMMAPID
		,FROMROUTINGOBJREFID
		,TOROUTINGOBJREFID
	) VALUES (
		CF_TRANSFORMROUTINGOBJ_LNK_SEQ.NextVal
		,v_TRANSFORMMAPID
		,v_FROMROUTINGOBJREFID
		,v_TOROUTINGOBJREFID
	);
	end if;
	end;
	end if;
--############################################################################################
--Validation Map
if p_MAPTYPE = 'V' THEN
BEGIN
	Select count(*) into v_count From CF_GUIDELINEMAP
		                   where GUIDELINEMAPFILENAME = p_MAPID;

			If v_count > 0 Then
		        Update CF_GUIDELINEMAP
		        	SET GUIDELINEMAPDESC = p_DESCRIPT
		        	,      ROWTS = RowTS + 1
		        	where GUIDELINEMAPFILENAME = p_MAPID;
        	else
	INSERT INTO CF_GUIDELINEMAP(
		GUIDELINEMAPID
		,GUIDELINEMAPFILENAME
		,GUIDELINEMAPDESC
		,GUIDELINEMAP
	) VALUES (
		CF_GUIDELINEMAP_SEQ.NEXTVAL
		,p_MAPID
		,p_DESCRIPT
		,UTL_RAW.CAST_TO_RAW(p_MAPID)
	);
	SELECT CF_GUIDELINEMAP_SEQ.CURRVAL into v_GUIDELINEMAPID from dual;
--	Get v_RoutingobjID
	Select routingobjid into v_RoutingobjID From CF_RoutingObject o
		                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
		                   Where o.RoutingObjCd = p_SRCDOCNAME
		                  and   o.Routingobjver = 'ALL'
		                   and   t.routingobjtypekey = 'Process';


	INSERT INTO CF_GUIDELINEMAP_LNK(
		GUIDELINEMAPLNKID
		,ROUTINGOBJID
		,GUIDELINEMAPID
	) VALUES (
		CF_GUIDELINEMAP_LNK_SEQ.NEXTVAL
		,v_ROUTINGOBJID
		,v_GUIDELINEMAPID
	);
end if;
end;
end if;
--############################################################################################
--FA Map
if p_MAPTYPE = 'K' THEN
BEGIN
	 Select count(*) into v_count From CF_EDIFAMAP
		                   where FAMAPFILENAME = p_MAPID;
				If v_count > 0
			Then
		        Update CF_EDIFAMAP
		        	SET DESCRIPTION = p_DESCRIPT
		        	,      ROWTS = RowTS + 1
		        	where FAMAPFILENAME = p_MAPID;
        	else
INSERT INTO CF_EDIFAMAP(
		FAMAPID
		,FAMAPFILENAME
		,DESCRIPTION
		,FAMAP
	) VALUES (
		CF_EDIFAMAP_SEQ.NEXTVAL
		,p_MAPID
		,p_DESCRIPT
		,UTL_RAW.CAST_TO_RAW(p_MAPID)
	);
	SELECT CF_EDIFAMAP_SEQ.CURRVAL into  v_FAMAPID from dual;
---Get v_RoutingobjID using
	Select routingobjid into v_RoutingobjID From CF_RoutingObject o
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = p_SRCDOCNAME
                  and   o.Routingobjver = 'ALL'
                  and   t.routingobjtypekey = 'Process';



INSERT INTO CF_EDIFAMAP_LNK(
		EDIFAMAPLNKID
		,FAMAPID
		,ROUTINGOBJID
	) VALUES (
		CF_EDIFAMAP_LNK_SEQ.NEXTVAL
		,v_FAMAPID
		,v_ROUTINGOBJID
	);
 end if;
 end;
 end if;
 p_ReturnCd := 0;
 			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, 1, v_ExceptionFlg);
 		EXCEPTION
 			When c_wrongrouting THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Wrong Routing Information, '||v_ParamsTxt, 1, 0);
				p_returnCd := 4;
 			WHEN NO_DATA_FOUND THEN
 				v_ExceptionFlg := 1;
 				Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, 1, 0);
 				p_returnCd := 3;
 			WHEN DUP_VAL_ON_INDEX THEN
 				v_ExceptionFlg := 1;
 				Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, 1, v_ExceptionFlg);
 				p_returnCd := 2;
 			WHEN OTHERS THEN
 				p_ReturnCd := 1;
 				v_ExceptionFlg := 1;
 				Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, 1, 1);
 				RAISE_APPLICATION_ERROR (-20999,'20002 CF_WDI_InsertMap' || SQLCODE|| ' '||SQLERRM);
 	END CF_WDI_InsertMap;
/

-- CF_WDI_MAPD.sql

Create or Replace Procedure CF_WDI_MAPD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_MAPID in NCHAR
	,p_MAPTYPE in NCHAR
	)
AS
/*
	================================================================================
	Proc Name:       CF_WDI_MAPD
	Create Date:     2004-08-30 11:18:37
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	----------	-------------	------	---------------------------------------------
	DATE      	NAME         	Ver   	DESCRIPTION
	----------	-------------	------	---------------------------------------------
	2004-08-30	SYSTEM		4.3	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMMAPD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MAPTYPE=>'||p_MAPTYPE||CHR(10)
		||', p_MAPID=>'||p_MAPID||CHR(10)
		;
v_count integer default 0;
zeroDelete  Exception;
c_wrongmaptype Exception;
c_mapinuse Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
p_ReturnCd := 0;
if p_MAPTYPE NOT IN ('A', 'V', 'K') then 
		  RAISE c_wrongmaptype;
	  end if;
	--Transformation Map
	if p_MAPTYPE = 'A' THEN 
	--check if it is in use
		select count(*) into v_count from pr_routingchannel where transformmapid = 
		(select transformmapid from cf_transformmap where transformmapfilename = p_MAPID );
		if v_count > 0
		then
		RAISE c_mapinuse;
		else
		delete from cf_transforminteract_lnk where 
		transformmapid = (select transformmapid from cf_transformmap where transformmapfilename = p_MAPID);
		delete from cf_transformmap where transformmapfilename = p_MAPID;
		end if;
	
	end if;
	--Validation Map
	if p_MAPTYPE = 'V' THEN 
		--
	--check if it is in use
			select count(*) into v_count from pr_routingattr_lnk where guidelinemapid = 
			(select guidelinemapid from cf_guidelinemap where guidelinemapfilename = p_MAPID);
			if v_count > 0
			then
			RAISE c_mapinuse;
			else
			delete from cf_guidelinemap_lnk where guidelinemapid =
			(select guidelinemapid from cf_guidelinemap where guidelinemapfilename = p_MAPID );
			delete from cf_guidelinemap where guidelinemapfilename = p_MAPID;
			end if;
	
	end if;
	--FA Map
	if p_MAPTYPE = 'K' THEN 
	--check if it is in use
			select count(*) into v_count from pr_routingattr_lnk where famapid = 
			(select famapid from cf_edifamap where famapfilename = p_MAPID );
			if v_count > 0
			then
			RAISE c_mapinuse;
			else
			delete from cf_edifamap_lnk where famapid =
			(select famapid from cf_edifamap where famapfilename = p_MAPID );
			delete from cf_edifamap where famapfilename = p_MAPID;
		end if;
	end if;
	Exception
		When c_wrongmaptype Then
					v_ExceptionFlg := 1;
					DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Wrong Map Type,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_WDI_MAPD: Wrong Map Type');
		When c_mapinuse Then
					v_ExceptionFlg := 1;
					DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Map in Use,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_WDI_MAPD: Map in Use');
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_WDI_MAPD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End CF_WDI_MAPD;
/

-- CF_WDI_MAPI.sql

CREATE OR REPLACE Procedure CF_WDI_MAPI
	(p_ReturnCd    OUT PLS_INTEGER
	,p_DebugFlg    IN  PLS_INTEGER
	,p_MAPID       IN  NCHAR
	,p_MAPTYPE     IN  NCHAR
	,p_DESCRIPT    IN  NCHAR
	,p_SRCPKGNAME  IN  NCHAR
	,p_SRCSYNTAX   IN  NCHAR
	,p_SRCDICTNAME IN  NCHAR
	,p_SRCDOCNAME  IN  NCHAR
	,p_SRCDESCRIPT IN  NCHAR
	,p_TRGPKGNAME  IN  NCHAR
	,p_TRGSYNTAX   IN  NCHAR
	,p_TRGDICTNAME IN  NCHAR
	,p_TRGDOCNAME  IN  NCHAR
	,p_TRGDESCRIPT IN  NCHAR
	)
AS
/*
	================================================================================
	Proc Name:       CF_WDI_MAPI
	Create Date:     08/30/2004
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	08/30/2004 SYSTEM           4.3  Created
	05/09/2008 AMIT KUMAR VIJAY 6.2  Modified for WTX


	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_TRANSFORMMAPI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MAPID=>'||p_MAPID||CHR(10)
		||', p_MAPTYPE=>'||p_MAPTYPE||CHR(10)
		||', p_DESCRIPT=>'||p_DESCRIPT||CHR(10)
		||', p_SRCPKGNAME=>'||p_SRCPKGNAME||CHR(10)
		||', p_SRCSYNTAX=>'||p_SRCSYNTAX||CHR(10)
		||', p_SRCDOCNAME=>'||p_SRCDOCNAME||CHR(10)
		||', p_SRCDESCRIPT=>'||p_SRCDESCRIPT||CHR(10)
		||', p_TRGPKGNAME=>'||p_TRGPKGNAME||CHR(10)
		||', p_TRGSYNTAX=>'||p_TRGSYNTAX||CHR(10)
		||', p_TRGDICTNAME=>'||p_TRGDICTNAME||CHR(10)
		||', p_TRGDOCNAME=>'||p_TRGDOCNAME||CHR(10)
		||', p_TRGDESCRIPT=>'||p_TRGDESCRIPT||CHR(10)
		;
c_wrongmaptype exception;
c_wrongsrcsyntax exception;
c_wrongtrgsyntax exception;
c_missingtrginfo exception;
c_wrongrouting exception;
v_returncd integer;
c_nullparm exception;
v_SRCPKGNAME      nchar(60);
v_TRGPKGNAME      nchar(60);
v_SMAPPKGNAME     varchar2(255);
v_TMAPPKGNAME     varchar2(255);

--Wizard vars
v_SRCOBJVERSION   VARCHAR2(60);
v_SRCOBJCD        VARCHAR2(60);
v_TRGOBJVERSION   VARCHAR2(60);
v_TRGOBJCD        VARCHAR2(60);
v_MAPID           INTEGER;
v_exists          INTEGER;
--end Wizard vars
-- FOR Backend integration variable
v_SRCPKGNAME_BI      nchar(60);
v_TRGPKGNAME_BI      nchar(60);
v_SMAPPKGNAME_BI     varchar2(255);
v_TMAPPKGNAME_BI     varchar2(255);
v_SRCOBJVERSION_BI   VARCHAR2(60);
v_SRCOBJCD_BI        VARCHAR2(60);
v_TRGOBJVERSION_BI   VARCHAR2(60);
v_TRGOBJCD_BI        VARCHAR2(60);
-- End Backend integration variable
BEGIN
	  if LENGTH(LTRIM(RTRIM(p_SRCPKGNAME))) = 0 then
	  	 v_SRCPKGNAME:= null;
	  	 else
	  	 v_SRCPKGNAME := p_SRCPKGNAME;
	  end if;
	  if LENGTH(LTRIM(RTRIM(p_TRGPKGNAME))) = 0 then
	  	 v_TRGPKGNAME:= null;
	  	 else
	  	 v_TRGPKGNAME := p_TRGPKGNAME;
	  end if;
	  if p_MAPTYPE is null  or p_srcsyntax is null
	      or p_srcdictname is null  or p_srcdocname is null then
	      RAISE c_nullparm;
    	  end if;

	  if p_MAPTYPE NOT IN ('A', 'V', 'K') then
	  RAISE c_wrongmaptype;
	  end if;
	  
	  if p_SRCSYNTAX NOT IN ('F', 'X', 'E') then
	  	  RAISE c_wrongsrcsyntax;
	  end if;
	  
	  if p_MAPTYPE in ('A', 'K') then
	      		if p_TRGSYNTAX NOT IN ('F', 'X', 'E') then
	      	  	  RAISE c_wrongtrgsyntax;
	  		end if;
	  		if p_TRGDICTNAME is null OR p_TRGDOCNAME is null then
	  		RAISE c_missingtrginfo;
	  		end if;
	  end if;

	  CF_WDI_ROInsertSrc(v_ReturnCd, v_SRCPKGNAME ,p_SRCSYNTAX, p_SRCDICTNAME ,p_SRCDOCNAME ,p_SRCDESCRIPT );

	  if p_SRCSYNTAX = 'E' then
	  CF_WDI_EDIROAttributes(v_ReturnCd, p_SRCDICTNAME ,p_SRCDOCNAME );
	  end if;

	  if p_SRCSYNTAX = 'X' then
	  	  CF_WDI_XMLROAttributes(v_ReturnCd, p_SRCDICTNAME,p_SRCDOCNAME  );
	  end if;

          if p_SRCSYNTAX = 'F' then
	  	  INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
	       		SELECT  RoutingobjrefId,(SELECT RoutingAttrId  FROM cf_routingAttr WHERE AttributeKey = 	'bcg.ro.TRANSFORMATIONERRLEVEL'	) 
	       		FROM cf_RoutingObjRef_lnk l 
	        		Inner Join cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId 
	        		WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND NOT EXISTS
	        							(SELECT l.RoutingObjRefId FROM CF_RoutingObjAttr_Lnk l 
	                                                          INNER JOIN cf_RoutingObjref_lnk rl ON rl.RoutingObjrefId = l.RoutingObjRefId 
	                                                          INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
	                                                          INNER JOIN cf_routingAttr a ON a.RoutingAttrId = l.RoutingAttrId 
	                                                          WHERE r.RoutingObjCd = p_SRCDICTNAME AND r.RoutingObjver = 'ALL' AND R.RoutingobjName = p_SRCDICTNAME 
	                                                          AND a.AttributeKey = 	'bcg.ro.TRANSFORMATIONERRLEVEL'	);
	                                                          
	  	  Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
	  		  Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
	                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
	                    where r.RoutingObjId = l.RoutingObjId
	                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
	                    AND a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
	                    and cro.routingattrid = a.routingattrid
	                    and cro.defaultflg = 1
	  		  And Not Exists(                                 
	                                      Select * from PR_RoutingAttr_Lnk prl
	  				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
	  				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
	  				                   Where  a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
	  				                   and cro.defaultflg = 1
	  				                   and cro.routingattrid = a.routingattrid
	  				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
	  				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
	  				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));
	  
	  
	  end if;
          
	  if p_MAPTYPE in ('A', 'K') THEN
	  CF_WDI_ROInsertTrg(v_ReturnCd, v_TRGPKGNAME ,p_TRGSYNTAX, p_TRGDICTNAME ,p_TRGDOCNAME ,p_TRGDESCRIPT );

		  if p_TRGSYNTAX = 'E' then
			  CF_WDI_EDIROAttributes(v_ReturnCd, p_TRGDICTNAME ,p_TRGDOCNAME );

			  end if;
			  if p_TRGSYNTAX = 'X' then
				  CF_WDI_XMLROAttributes(v_ReturnCd, p_TRGDICTNAME ,p_TRGDOCNAME );

		  end if;
	  end if;
	  if v_SRCPKGNAME is NULL then
	  	  	  		if p_SRCSYNTAX = 'F' then v_SMAPPKGNAME := 'None'; end if;
	  	  	  		if p_SRCSYNTAX = 'E' then v_SMAPPKGNAME := 'N/A'; v_SMAPPKGNAME_BI := 'Backend Integration'; end if;
	  	  	  		if p_SRCSYNTAX = 'X' then  v_SMAPPKGNAME := 'None'; end if;
	  	 else
	  	  	    v_SMAPPKGNAME := v_SRCPKGNAME ;
	  	  	    v_SMAPPKGNAME_BI := v_SRCPKGNAME ;
	  	      end if;
	 if v_TRGPKGNAME is NULL then
	  	  	  	  		if p_TRGSYNTAX = 'F' then v_TMAPPKGNAME := 'None'; end if;
	  	  	  	  		if p_TRGSYNTAX = 'E' then v_TMAPPKGNAME := 'N/A'; v_TMAPPKGNAME_BI := 'Backend Integration'; end if;
	  	  	  	  		if p_TRGSYNTAX = 'X' then v_TMAPPKGNAME := 'None'; end if;
	  	 else
	  	  	     v_TMAPPKGNAME := v_TRGPKGNAME ;
	  	  	     v_TMAPPKGNAME_BI := v_TRGPKGNAME ;
	       end if;
	  CF_WDI_InsertMap(v_ReturnCd ,p_MAPID ,p_MAPTYPE,p_DESCRIPT ,p_SRCDICTNAME, p_TRGDICTNAME, p_SRCDOCNAME ,p_TRGDOCNAME, v_SMAPPKGNAME, v_TMAPPKGNAME);
	  if v_ReturnCd = 4 then raise c_wrongrouting; end if;

	  CF_PROFILE_LNKI_AH2(v_ReturnCd ,1 ,p_SRCDICTNAME);

   --Wizard specific code
   DECLARE
      notable EXCEPTION;
      PRAGMA EXCEPTION_INIT (notable, -942);
   BEGIN

      IF p_MAPTYPE = 'A' THEN
         SELECT TRANSFORMMAPID
         INTO v_MAPID
         FROM CF_TRANSFORMMAP
         WHERE TRANSFORMMAPFILENAME = p_mapid;
      END IF;
      IF p_MAPTYPE = 'V' THEN
         SELECT GUIDELINEMAPID
         INTO v_MAPID
         FROM CF_GUIDELINEMAP
         WHERE GUIDELINEMAPFILENAME = p_mapid;
      END IF;
      IF p_MAPTYPE = 'K' THEN
         SELECT FAMAPID
         INTO v_MAPID
         FROM CF_EDIFAMAP
         WHERE FAMAPFILENAME = p_mapid;
      END IF;

      IF p_SRCPKGNAME is NULL then
		   IF p_SRCSYNTAX = 'F' then 
                    v_SRCPKGNAME := 'None';
				   	  v_SRCOBJCD   := 'None';
					     v_SRCOBJVERSION := 'N/A';
   		ELSIF
	   	p_SRCSYNTAX = 'E' then 
                    v_SRCPKGNAME := 'N/A';
			   	     v_SRCOBJCD   := 'N/A';
				        v_SRCOBJVERSION := 'N/A';
		    v_SRCPKGNAME_BI := 'Backend Integration';
		    		     v_SRCOBJCD_BI   := 'Backend Integration';
				        v_SRCOBJVERSION_BI   := '1.0';
   		ELSIF
	   	p_SRCSYNTAX = 'X' then 
                    v_SRCPKGNAME := 'None';
			   	     v_SRCOBJCD   := 'None';
				        v_SRCOBJVERSION := 'N/A';
   		END IF;
      ELSE
         v_srcobjversion := '1.0';
         v_SRCPKGNAME := to_char(p_SRCPKGNAME);
         v_srcobjversion_BI := '1.0';
         v_SRCPKGNAME_BI := 'Backend Integration';
      END IF;

      IF p_TRGPKGNAME is NULL then
		   IF p_TRGSYNTAX = 'F' then 
                    v_TRGPKGNAME := 'None';
				   	  v_TRGOBJCD   := 'None';
					     v_TRGOBJVERSION := 'N/A';
   		ELSIF
	   	p_TRGSYNTAX = 'E' then 
                    v_TRGPKGNAME := 'N/A';
			   	     v_TRGOBJCD   := 'N/A';
				        v_TRGOBJVERSION := 'N/A';
		    v_TRGPKGNAME_BI := 'Backend Integration';
				     v_TRGOBJCD_BI   := 'Backend Integration';
				        v_TRGOBJVERSION_BI   := '1.0';
   		ELSIF
	   	p_TRGSYNTAX = 'X' then 
                    v_TRGPKGNAME := 'None';
			   	     v_TRGOBJCD   := 'None';
				        v_TRGOBJVERSION := 'N/A';
   		END IF;
      ELSE
         v_trgobjversion := '1.0';
         v_TRGPKGNAME := to_char(p_TRGPKGNAME);
         v_trgobjversion := '1.0';
         v_TRGPKGNAME := 'Backend Integration';
      END IF;

      --SELECT count(*) INTO v_exists
      --FROM eifimporthistory
      --WHERE ToPkgName = v_TRGPKGNAME
      --   AND ToPkgVer = v_TRGOBJVERSION
      --   AND ToProtName = to_char(p_TRGDICTNAME)
      --   AND ToDocTypeName = to_char(p_TRGDOCNAME)
      --   AND FrPkgName = v_SRCPKGNAME
      --   AND FrPkgVer = v_SRCOBJVERSION
      --   AND FrProtName = to_char(p_SRCDICTNAME)
      --   AND FrDocTypeName = to_char(p_SRCDOCNAME)
      --   AND MapID = v_mapid
      --   AND MapName = to_char(p_MAPID)
      --   AND MapType = to_char(p_MAPTYPE)
      --   AND FrSyntax = to_char(p_SRCSYNTAX)
      --   AND ToSyntax = to_char(p_TRGSYNTAX)
      --;

      --IF v_exists = 0 THEN
         INSERT INTO EIFImportHistory
         (ToPkgName,
          ToPkgVer,
          ToProtName,
          ToDocTypeName,
          FrPkgName,
          FrPkgVer,
          FrProtName,
          FrDocTypeName,
          MapID,
          MapName,
          MapType,
          FrSyntax,
          ToSyntax)
         VALUES
         (v_TRGPKGNAME,
          v_TRGOBJVERSION,
          rtrim(to_char(p_TRGDICTNAME)),
          rtrim(to_char(p_TRGDOCNAME)),
          v_SRCPKGNAME,
          v_SRCOBJVERSION,
          rtrim(to_char(p_SRCDICTNAME)),
          rtrim(to_char(p_SRCDOCNAME)),
          v_mapid,
          rtrim(to_char(p_MAPID)),
          rtrim(to_char(p_MAPTYPE)),
          rtrim(to_char(p_SRCSYNTAX)),
          rtrim(to_char(p_TRGSYNTAX)));
      --END IF;
   EXCEPTION
      --when the row already exists eat the dup_val_on_index
      --when the tables don't exist eat the
      WHEN DUP_VAL_ON_INDEX THEN
            v_ExceptionFlg := 1;
            p_returnCd := 2;
      WHEN notable THEN
            v_ExceptionFlg := 1;
            p_returnCd := 2;
   END;

   --Wizard specific code


	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
			When c_wrongmaptype THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Wrong Map Type, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 4;
			When c_wrongsrcsyntax THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Wrong Source Syntax, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 5;
			When c_wrongtrgsyntax THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Wrong Target Syntax, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 6;
			When c_missingtrginfo THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Missing Target Information, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 7;
			When c_nullparm THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Required Parameters are Null, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 8;
			When c_wrongrouting THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Missing Target Information, '||v_ParamsTxt, 1, 0);
				p_returnCd := 2;
			When NO_DATA_FOUND THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 3;
			When OTHERS Then
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_WDI_MAPI' || SQLCODE|| ' '||SQLERRM);
END CF_WDI_MAPI;
/

-- CF_WDI_ROINSERTSRC.sql

CREATE OR REPLACE Procedure CF_WDI_ROINSERTSRC
	(p_ReturnCd  out Integer
	,p_SRCPKGNAME IN NCHAR
	,p_SRCSYNTAX IN NCHAR
	,p_SRCDICTNAME IN NCHAR
	,p_SRCDOCNAME IN NCHAR
	,p_SRCDESCRIPT IN NCHAR
	)
AS
/*
	================================================================================
	Proc Name:       CF_WDI_ROINSERTSRC
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	05/09/2008 AMIT KUMAR VIJAY 6.2   Modified for WTX
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        NUMBER := 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_WDI_ROINSERTSRC';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		  'p_SRCPKGNAME=>'||p_SRCPKGNAME||CHR(10) 
		  ||',p_SRCSYNTAX=>'||p_SRCSYNTAX||CHR(10)
		  ||',p_SRCDICTNAME=>'||p_SRCDICTNAME||CHR(10) 
		  ||',p_SRCDOCNAME=>'||p_SRCDOCNAME||CHR(10)
		  ||',p_SRCDESCRIPT=>'||p_SRCDESCRIPT||CHR(10)
		;

v_SRCPKGNAME      Varchar2(60);
v_SRCOBJCD        Varchar2(60);
v_SRCOBJVERSION   Varchar2(60);
-- For Backend integration variable
v_SRCPKGNAME_BI      Varchar2(60);
v_SRCOBJCD_BI        Varchar2(60);
v_SRCOBJVERSION_BI   Varchar2(60);
-- End Backend integration variable
v_count           Integer Default 0;
v_chkpoint        integer default 0;

v_srcdictname	  cf_routingobject.RoutingObjCd%TYPE := p_srcdictname;
v_srcdocname	  cf_routingobject.RoutingObjCd%TYPE := p_srcdocname;
v_srcdescript	  cf_routingobject.RoutingObjDesc%TYPE := p_srcdescript;

cursor CUR_CFROUTINGOBJREFLNK
	IS
((Select (select RoutingObjId
                   from CF_RoutingObject o
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = v_srcdocname
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Process') RoutingObjId
                , L.RoutingObjRefId RoutingObjPARENTRefId
             from
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
             where
                   T.RoutingObjTypeKey = 'Protocol'
                    and R.RoutingObjName = v_srcdictname
                    and R.RoutingObjCd = v_srcdictname
                    and R.RoutingObjVer = 'ALL')
             MINUS	(Select v.ROUTINGOBJID, ROUTINGOBJPARENTREFID From CF_ROUTINGOBJECT02_VW v
                            inner join cf_routingobject o on o.RoutingobjId = v.PARENTOBJID
                            where v.RoutingObjCd = v_srcdocname
                            and v.Routingobjver = 'ALL'
                            and v.routingobjtypekey = 'Process'
                            and  v.PARENTOBJTYPEKEY = 'Protocol'
                            and o.RoutingObjName = v_srcdictname
                            and v.PARENTOBJCD = v_srcdictname
                            and v.PARENTOBJVER = 'ALL'));
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

	if p_SRCPKGNAME is NULL then
		if p_SRCSYNTAX = 'F' then v_SRCPKGNAME := 'None';
					  v_SRCOBJCD   := 'None';
					  v_SRCOBJVERSION   := 'N/A';end if;
		if
		p_SRCSYNTAX = 'E' then v_SRCPKGNAME := 'N/A';
				       v_SRCOBJCD   := 'N/A';
				       v_SRCOBJVERSION   := 'N/A';
				       -- For Backend integration 
				       v_SRCPKGNAME_BI := 'Backend Integration';
				       v_SRCOBJCD_BI   := 'Backend Integration';
				       v_SRCOBJVERSION_BI   := '1.0';
				       -- End Backend integration 
				       end if;
		if
		p_SRCSYNTAX = 'X' then v_SRCPKGNAME := 'None';
				       v_SRCOBJCD   := 'None';
				       v_SRCOBJVERSION   := 'N/A';end if;
	else
	begin
	Select count(*) into v_count From CF_RoutingObject o
                   						inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   						Where o.RoutingObjCd = NVL(p_SRCPKGNAME,v_SRCPKGNAME)
                   						and   o.Routingobjver = '1.0'
                   						and   t.routingobjtypekey = 'Package';
        If v_count > 0 Then
        Update CF_RoutingObject
        	SET RoutingObjDesc = v_srcdescript
        	,      ROWTS = RowTS + 1
        	where RoutingObjCd = NVL(p_SRCPKGNAME,v_SRCPKGNAME)
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Package')
        	and Routingobjver = '1.0';
			        	else
			
	Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc,
                               ActiveStatusCd, VisibilityCd, DocumentFlg)
                        (select CF_RoutingObject_Seq.nextval,
                               RoutingObjTypeId,  NVL(p_SRCPKGNAME,v_SRCPKGNAME) , NVL(p_SRCPKGNAME,v_SRCPKGNAME),
                               '1.0', 1, v_srcdescript, 1, 7, 0
     					from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Package');
			
Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId)
    select  CF_RoutingObjRef_Lnk_Seq.nextval, RoutingObjId
    from  CF_RoutingObject r
    inner join CF_RoutingObjType t on t.RoutingObjTypeId = r.RoutingObjTypeId
    WHERE t.routingobjtypekey ='Package'
    AND r.RoutingObjCd = NVL(p_SRCPKGNAME,v_SRCPKGNAME)
    AND r.RoutingObjName = NVL(p_SRCPKGNAME,v_SRCPKGNAME)
    AND NOT Exists(Select * from cf_routingObjref_lnk l
                   inner join cf_routingobject r on r.RoutingObjId = l.RoutingObjId
                   inner join cf_routingobjtype t on t.RoutingObjTypeId = r.RoutingObjTypeId
                   WHERE t.routingobjtypekey = 'Package'
                   AND r.RoutingObjCd = NVL(p_SRCPKGNAME,v_SRCPKGNAME)
                   AND r.RoutingObjName = NVL(p_SRCPKGNAME,v_SRCPKGNAME));
							   
                   end if;
                   end;
                   end if;

---Insert Protocol
	Select count(*) into v_count From CF_RoutingObject o
			   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
			   Where o.RoutingObjCd = v_srcdictname
			   and   o.Routingobjver = 'ALL'
			   and   t.routingobjtypekey = 'Protocol';
	IF v_count > 0 Then
      IF p_SRCSYNTAX = 'E' THEN
         Update CF_RoutingObject
        	SET RoutingObjDesc = v_srcdescript
        	,      ROWTS = RowTS + 1
         ,      protocoltype = 1
        	where RoutingObjCd = v_srcdictname
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Protocol')
        	and Routingobjver = 'ALL';
      ELSE
         Update CF_RoutingObject
        	SET RoutingObjDesc = v_srcdescript
        	,      ROWTS = RowTS + 1
         ,      protocoltype = 0
        	where RoutingObjCd = v_srcdictname
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Protocol')
        	and Routingobjver = 'ALL';
      END IF;
   ELSE
      IF p_SRCSYNTAX = 'E' THEN
         Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg, protocoltype)
         (select CF_RoutingObject_Seq.nextval,
         RoutingObjTypeId, v_srcdictname, v_srcdictname, 'ALL', 1, v_srcdescript, 1, 7, 0, 1
         from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
         );
      ELSE
         Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg, protocoltype)
         (select CF_RoutingObject_Seq.nextval,
         RoutingObjTypeId, v_srcdictname, v_srcdictname, 'ALL', 1, v_srcdescript, 1, 7, 0, 0
         from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
         );
      END IF;
	 			
 end if;
 			
Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId)
             select CF_RoutingObjRef_Lnk_Seq.nextval,
                (select RoutingObjId
                   from CF_RoutingObject o
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = v_srcdictname
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Protocol')
                , L.RoutingObjRefId
             from
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
             where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = NVL(p_SRCPKGNAME,v_SRCPKGNAME)
                    and R.RoutingObjCd = NVL(p_SRCPKGNAME,v_SRCOBJCD)
                    and R.RoutingObjVer = NVL(v_SRCOBJVERSION, '1.0')
             AND Not exists(Select v.ROUTINGOBJID, ROUTINGOBJPARENTREFID From CF_ROUTINGOBJECT02_VW v
                            inner join cf_routingobject o on o.RoutingobjId = v.PARENTOBJID
                            where v.RoutingObjCd = v_srcdictname
                            and v.Routingobjver = 'ALL'
                            and v.routingobjtypekey = 'Protocol'
                            and  v.PARENTOBJTYPEKEY = 'Package'
                            and o.RoutingObjName = NVL(p_SRCPKGNAME,v_SRCPKGNAME)
                            and v.PARENTOBJCD = NVL(p_SRCPKGNAME,v_SRCOBJCD)
                            and v.PARENTOBJVER = NVL(v_SRCOBJVERSION, '1.0'));
                            
-- For Backend integration                            
IF p_SRCPKGNAME is NULL and p_SRCSYNTAX = 'E' THEN                            
	Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId)
             select CF_RoutingObjRef_Lnk_Seq.nextval,
                (select RoutingObjId
                   from CF_RoutingObject o
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = v_srcdictname
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Protocol')
                , L.RoutingObjRefId
             from
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
             where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = NVL(p_SRCPKGNAME,v_SRCPKGNAME_BI)
                    and R.RoutingObjCd = NVL(p_SRCPKGNAME,v_SRCOBJCD_BI)
                    and R.RoutingObjVer = NVL(v_SRCOBJVERSION_BI, '1.0')
             AND Not exists(Select v.ROUTINGOBJID, ROUTINGOBJPARENTREFID From CF_ROUTINGOBJECT02_VW v
                            inner join cf_routingobject o on o.RoutingobjId = v.PARENTOBJID
                            where v.RoutingObjCd = v_srcdictname
                            and v.Routingobjver = 'ALL'
                            and v.routingobjtypekey = 'Protocol'
                            and  v.PARENTOBJTYPEKEY = 'Package'
                            and o.RoutingObjName = NVL(p_SRCPKGNAME,v_SRCPKGNAME_BI)
                            and v.PARENTOBJCD = NVL(p_SRCPKGNAME,v_SRCOBJCD_BI)
                            and v.PARENTOBJVER = NVL(v_SRCOBJVERSION_BI, '1.0'));
END IF;
-- END Backend integration
                            
--Insert Process
Select count(*) into v_count From CF_RoutingObject o
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = v_srcdocname
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Process';
	IF v_count > 0 Then
      IF p_SRCSYNTAX = 'E' THEN
         Update CF_RoutingObject
        	SET RoutingObjDesc = v_srcdescript
        	,      ROWTS = RowTS + 1
         ,      protocoltype = 1
        	where RoutingObjCd = v_srcdocname
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Process')
        	and Routingobjver = 'ALL';
      ELSE
         Update CF_RoutingObject
        	SET RoutingObjDesc = v_srcdescript
        	,      ROWTS = RowTS + 1
         ,      protocoltype = 0
        	where RoutingObjCd = v_srcdocname
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Process')
        	and Routingobjver = 'ALL';
      END IF;
   ELSE
      IF p_SRCSYNTAX = 'E' THEN
         Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg, protocoltype)
         (select CF_RoutingObject_Seq.nextval,
         RoutingObjTypeId,  v_srcdocname, v_srcdocname, 'ALL', 1, v_srcdescript, 1, 7, 1, 1
         from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Process');
      ELSE
         Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg, protocoltype)
         (select CF_RoutingObject_Seq.nextval,
         RoutingObjTypeId,  v_srcdocname, v_srcdocname, 'ALL', 1, v_srcdescript, 1, 7, 1, 0
         from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Process');
      END IF;
   END if;
	  			
  FOR PROCESS_ROUTINGREFLNK in CUR_CFROUTINGOBJREFLNK Loop
  	BEGIN
  		Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId 
  						,RoutingObjId
  						,RoutingObjParentRefId)
             				VALUES( CF_RoutingObjRef_Lnk_Seq.nextval
             					,PROCESS_ROUTINGREFLNK.RoutingObjId
             					,PROCESS_ROUTINGREFLNK.RoutingObjPARENTRefId
             					);
        END;
  END LOOP;
			
--SOURCE ENDS
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, 1, v_ExceptionFlg);
END ;
/

-- CF_WDI_ROINSERTTRG.sql

CREATE OR REPLACE Procedure CF_WDI_ROINSERTTRG
	(p_ReturnCd  out Integer
	,p_TRGPKGNAME IN NCHAR
	,p_TRGSYNTAX IN NCHAR
	,p_TRGDICTNAME IN NCHAR
	,p_TRGDOCNAME IN NCHAR
	,p_TRGDESCRIPT IN NCHAR
	)
AS
/*
	================================================================================
	Proc Name:       CF_WDI_ROINSERTTRG
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	05/09/2008 AMIT KUMAR VIJAY 6.2  Modified for WTX
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        NUMBER := 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_WDI_ROINSERTTRG';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		  'p_TRGPKGNAME=>'||p_TRGPKGNAME||CHR(10) 
		  ||',p_TRGSYNTAX=>'||p_TRGSYNTAX||CHR(10)
		  ||',p_TRGDICTNAME=>'||p_TRGDICTNAME||CHR(10) 
		  ||',p_TRGDOCNAME=>'||p_TRGDOCNAME||CHR(10)
		  ||',p_TRGDESCRIPT=>'||p_TRGDESCRIPT||CHR(10)
		;

v_TRGPKGNAME      Varchar2(60);
v_TRGOBJCD        Varchar2(60);
v_TRGOBJVERSION   Varchar2(60);
-- For Backend integration variable
v_TRGPKGNAME_BI      Varchar2(60);
v_TRGOBJCD_BI        Varchar2(60);
v_TRGOBJVERSION_BI   Varchar2(60);
-- End Backend integration variable
v_count           Integer Default 0;
v_chkpoint        integer default 0;

v_TRGdictname	  cf_routingobject.RoutingObjCd%TYPE := p_TRGdictname;
v_TRGdocname	  cf_routingobject.RoutingObjCd%TYPE := p_TRGdocname;
v_TRGdescript	  cf_routingobject.RoutingObjDesc%TYPE := p_TRGdescript;

cursor CUR_CFROUTINGOBJREFLNK
	IS
((Select (select RoutingObjId
                   from CF_RoutingObject o
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = v_TRGdocname
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Process') RoutingObjId
                , L.RoutingObjRefId RoutingObjPARENTRefId
             from
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
             where
                   T.RoutingObjTypeKey = 'Protocol'
                    and R.RoutingObjName = v_TRGdictname
                    and R.RoutingObjCd = v_TRGdictname
                    and R.RoutingObjVer = 'ALL')
             MINUS	(Select v.ROUTINGOBJID, ROUTINGOBJPARENTREFID From CF_ROUTINGOBJECT02_VW v
                            inner join cf_routingobject o on o.RoutingobjId = v.PARENTOBJID
                            where v.RoutingObjCd = v_TRGdocname
                            and v.Routingobjver = 'ALL'
                            and v.routingobjtypekey = 'Process'
                            and  v.PARENTOBJTYPEKEY = 'Protocol'
                            and o.RoutingObjName = v_TRGdictname
                            and v.PARENTOBJCD = v_TRGdictname
                            and v.PARENTOBJVER = 'ALL'));

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

	if p_TRGPKGNAME is NULL then
		if p_TRGSYNTAX = 'F' then v_TRGPKGNAME := 'None';
					  v_TRGOBJCD   := 'None';
					  v_TRGOBJVERSION   := 'N/A';end if;
		if
		p_TRGSYNTAX = 'E' then v_TRGPKGNAME := 'N/A';
				       v_TRGOBJCD   := 'N/A';
				       v_TRGOBJVERSION   := 'N/A';
				       -- For Backend integration 
				       v_TRGPKGNAME_BI := 'Backend Integration';
				       v_TRGOBJCD_BI   := 'Backend Integration';
				       v_TRGOBJVERSION_BI   := '1.0';
				       -- End Backend integration 
				       end if;
		if
		p_TRGSYNTAX = 'X' then v_TRGPKGNAME := 'None';
				       v_TRGOBJCD   := 'None';
				       v_TRGOBJVERSION   := 'N/A';end if;
	else
	begin
	Select count(*) into v_count From CF_RoutingObject o
                   						inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   						Where o.RoutingObjCd = NVL(p_TRGPKGNAME,v_TRGPKGNAME)
                   						and   o.Routingobjver = '1.0'
                   						and   t.routingobjtypekey = 'Package';
        If v_count > 0 Then
        Update CF_RoutingObject
        	SET RoutingObjDesc = v_TRGdescript
        	,      ROWTS = RowTS + 1
        	where RoutingObjCd = NVL(p_TRGPKGNAME,v_TRGPKGNAME)
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Package')
        	and Routingobjver = '1.0';
			        	else
			
	Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc,
                               ActiveStatusCd, VisibilityCd, DocumentFlg)
                        (select CF_RoutingObject_Seq.nextval,
                               RoutingObjTypeId,  NVL(p_TRGPKGNAME,v_TRGPKGNAME) , NVL(p_TRGPKGNAME,v_TRGPKGNAME),
                               '1.0', 1, v_TRGdescript, 1, 7, 0
     					from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Package');
			
Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId)
    select  CF_RoutingObjRef_Lnk_Seq.nextval, RoutingObjId
    from  CF_RoutingObject r
    inner join CF_RoutingObjType t on t.RoutingObjTypeId = r.RoutingObjTypeId
    WHERE t.routingobjtypekey ='Package'
    AND r.RoutingObjCd = NVL(p_TRGPKGNAME,v_TRGPKGNAME)
    AND r.RoutingObjName = NVL(p_TRGPKGNAME,v_TRGPKGNAME)
    AND NOT Exists(Select * from cf_routingObjref_lnk l
                   inner join cf_routingobject r on r.RoutingObjId = l.RoutingObjId
                   inner join cf_routingobjtype t on t.RoutingObjTypeId = r.RoutingObjTypeId
                   WHERE t.routingobjtypekey = 'Package'
                   AND r.RoutingObjCd = NVL(p_TRGPKGNAME,v_TRGPKGNAME)
                   AND r.RoutingObjName = NVL(p_TRGPKGNAME,v_TRGPKGNAME));
							   
                   end if;
                   end;
                   end if;

---Insert Protocol
	Select count(*) into v_count From CF_RoutingObject o
			   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
			   Where o.RoutingObjCd = v_TRGdictname
			   and   o.Routingobjver = 'ALL'
			   and   t.routingobjtypekey = 'Protocol';
	IF v_count > 0 THEN
				
      IF p_TRGSYNTAX = 'E' THEN
         Update CF_RoutingObject
        	SET RoutingObjDesc = v_TRGdescript
        	,      ROWTS = RowTS + 1
         ,      protocoltype = 1
        	where RoutingObjCd = v_TRGdictname
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Protocol')
        	and Routingobjver = 'ALL';
      ELSE
         Update CF_RoutingObject
        	SET RoutingObjDesc = v_TRGdescript
        	,      ROWTS = RowTS + 1
         ,      protocoltype = 0
        	where RoutingObjCd = v_TRGdictname
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Protocol')
        	and Routingobjver = 'ALL';
      END IF;
						
   ELSE
		IF p_TRGSYNTAX = 'E' THEN				
        	Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg, protocoltype)
         (select CF_RoutingObject_Seq.nextval,
         RoutingObjTypeId, v_TRGdictname, v_TRGdictname, 'ALL', 1, v_TRGdescript, 1, 7, 0, 1
         from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
         );
      ELSE
         Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg, protocoltype)
         (select CF_RoutingObject_Seq.nextval,
         RoutingObjTypeId, v_TRGdictname, v_TRGdictname, 'ALL', 1, v_TRGdescript, 1, 7, 0, 0
         from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
         );
      END IF;
	 			
 end if;
 			
Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId)
             select CF_RoutingObjRef_Lnk_Seq.nextval,
                (select RoutingObjId
                   from CF_RoutingObject o
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = v_TRGdictname
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Protocol')
                , L.RoutingObjRefId
             from
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
             where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = NVL(p_TRGPKGNAME,v_TRGPKGNAME)
                    and R.RoutingObjCd = NVL(p_TRGPKGNAME,v_TRGOBJCD)
                    and R.RoutingObjVer = NVL(v_TRGOBJVERSION, '1.0')
             AND Not exists(Select v.ROUTINGOBJID, ROUTINGOBJPARENTREFID From CF_ROUTINGOBJECT02_VW v
                            inner join cf_routingobject o on o.RoutingobjId = v.PARENTOBJID
                            where v.RoutingObjCd = v_TRGdictname
                            and v.Routingobjver = 'ALL'
                            and v.routingobjtypekey = 'Protocol'
                            and  v.PARENTOBJTYPEKEY = 'Package'
                            and o.RoutingObjName = NVL(p_TRGPKGNAME,v_TRGPKGNAME)
                            and v.PARENTOBJCD = NVL(p_TRGPKGNAME,v_TRGOBJCD)
                            and v.PARENTOBJVER = NVL(v_TRGOBJVERSION, '1.0'));
                            
-- For Backend integration                            
IF p_TRGPKGNAME is NULL and p_TRGSYNTAX = 'E' THEN                            
	Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId)
             select CF_RoutingObjRef_Lnk_Seq.nextval,
                (select RoutingObjId
                   from CF_RoutingObject o
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = v_TRGdictname
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Protocol')
                , L.RoutingObjRefId
             from
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
             where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = NVL(p_TRGPKGNAME,v_TRGPKGNAME_BI)
                    and R.RoutingObjCd = NVL(p_TRGPKGNAME,v_TRGOBJCD_BI)
                    and R.RoutingObjVer = NVL(v_TRGOBJVERSION_BI, '1.0')
             AND Not exists(Select v.ROUTINGOBJID, ROUTINGOBJPARENTREFID From CF_ROUTINGOBJECT02_VW v
                            inner join cf_routingobject o on o.RoutingobjId = v.PARENTOBJID
                            where v.RoutingObjCd = v_TRGdictname
                            and v.Routingobjver = 'ALL'
                            and v.routingobjtypekey = 'Protocol'
                            and  v.PARENTOBJTYPEKEY = 'Package'
                            and o.RoutingObjName = NVL(p_TRGPKGNAME,v_TRGPKGNAME_BI)
                            and v.PARENTOBJCD = NVL(p_TRGPKGNAME,v_TRGOBJCD_BI)
                            and v.PARENTOBJVER = NVL(v_TRGOBJVERSION_BI, '1.0'));
END IF;
-- END Backend integration                          
			
--Insert Process
Select count(*) into v_count From CF_RoutingObject o
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = v_TRGdocname
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Process';
	If v_count > 0 Then
      
      IF p_TRGSYNTAX = 'E' THEN
         Update CF_RoutingObject
        	SET RoutingObjDesc = v_TRGdescript
        	,      ROWTS = RowTS + 1
         ,      protocoltype = 1
        	where RoutingObjCd = v_TRGdocname
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Process')
        	and Routingobjver = 'ALL';
      ELSE
         Update CF_RoutingObject
        	SET RoutingObjDesc = v_TRGdescript
        	,      ROWTS = RowTS + 1
         ,      protocoltype = 0
        	where RoutingObjCd = v_TRGdocname
        	AND RoutingObjTypeID = (SELECT  t.RoutingObjTypeID FROM CF_ROUTINGOBJTYPE t WHERE t.ROUTINGOBJTYPEKEY = 'Process')
        	and Routingobjver = 'ALL';
      END IF;
						
   ELSE
      IF p_TRGSYNTAX = 'E' THEN
         Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg, protocoltype)
         (select CF_RoutingObject_Seq.nextval,
         RoutingObjTypeId,  v_TRGdocname, v_TRGdocname, 'ALL', 1, v_TRGdescript, 1, 7, 1, 1
         from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Process');
      ELSE
         Insert into CF_RoutingObject (RoutingObjId,RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg, protocoltype)
         (select CF_RoutingObject_Seq.nextval,
         RoutingObjTypeId,  v_TRGdocname, v_TRGdocname, 'ALL', 1, v_TRGdescript, 1, 7, 1, 0
         from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Process');
      END IF;
   END if;
	  			
  FOR PROCESS_ROUTINGREFLNK in CUR_CFROUTINGOBJREFLNK Loop
    	BEGIN
    		Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId 
    						,RoutingObjId
    						,RoutingObjParentRefId)
               				VALUES( CF_RoutingObjRef_Lnk_Seq.nextval
               					,PROCESS_ROUTINGREFLNK.RoutingObjId
               					,PROCESS_ROUTINGREFLNK.RoutingObjPARENTRefId
               					);
          END;
  END LOOP;
			
--TARGET ENDS
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, 1, v_ExceptionFlg);
END ;
/

-- CF_WDI_XMLROATTRIBUTES.sql

CREATE OR REPLACE PROCEDURE CF_WDI_XMLROAttributes
	(p_ReturnCd OUT PLS_INTEGER
	,p_SRCDICTNAME IN NCHAR
	,p_SRCDOCNAME IN NCHAR
	)
	AS
/*
	================================================================================
	Proc Name:       CF_WDI_XMLROAttributes
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
			   ::::::::::::::::::::::::::::::
				 M O D I F I C A T I O N
			   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        NUMBER := 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'CF_WDI_XMLROAttributes';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
				' p_SRCDICTNAME=>'||p_SRCDICTNAME||CHR(10)
				||', p_SRCDOCNAME=>'||p_SRCDOCNAME||CHR(10)
		;	
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

	BEGIN
	Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.SEGOUTPUT'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.SEGOUTPUT'	);	 
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.TRANSFORMATIONERRLEVEL'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.TRANSFORMATIONERRLEVEL'	);	 
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.xml.XMLNSACTIVE'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDICTNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.xml.XMLNSACTIVE'	);
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.xml.XMLNSACTIVE'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDOCNAME 
                                                                              AND a.AttributeKey = 	'bcg.ro.xml.XMLNSACTIVE'	);                                                                              
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'GuidelineMap'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = p_SRCDOCNAME and r.RoutingObjver = 'ALL' and R.RoutingobjName = p_SRCDOCNAME 
                                                                              AND a.AttributeKey = 	'GuidelineMap'	);
-- add default attribute values

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.SEGOUTPUT'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.SEGOUTPUT'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));
 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = p_SRCDICTNAME and r.RoutingObjver = 'ALL'));


	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, 1, v_ExceptionFlg);
END CF_WDI_XMLROAttributes;
/

-- CF_WEBSERVICE_ATTRS_AH1.sql

CREATE OR REPLACE Procedure CF_WEBSERVICE_ATTRS_AH1
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	)
AS

/*


	================================================================================
	Proc Name:       CF_WEBSERVICE_ATTRS_AH1
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_WEBSERVICE_ATTRS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;


BEGIN
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	OPEN RC1 FOR
		SELECT WSATTRID,
			WSATTRKEY,
			WSATTRDESC,
			RESOURCEKEY
		FROM CF_WEBSERVICE_ATTR 
		;
		
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 CF_WEBSERVICE_ATTRS_AH1' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_WEBSERVICE_ATTRS_AH1' || SQLCODE|| ' '||SQLERRM);
END CF_WEBSERVICE_ATTRS_AH1;
/

-- CF_WEBSERVICE_ATTRVALUEI.sql

CREATE OR REPLACE Procedure CF_WEBSERVICE_ATTRVALUEI
(
	p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_WSAttrID		in		INTEGER
	, P_WSROUTINGOBJID      in              INTEGER
	, p_WSAttrValue		in		VARCHAR2
)
AS

/*


	================================================================================
	Proc Name:       CF_WEBSERVICE_ATTRVALUEI
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_WEBSERVICE_ATTRVALUEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

v_wsattrvalueid    INTEGER;


BEGIN
	
	
	INSERT INTO CF_WEBSERVICE_ATTRVALUE
			(
			WSATTRVALUEID,		
			WSATTRID,		
			WSROUTINGOBJID,		
			WSATTRVALUE,
			CREATEDATE,
			UPDATEDATE
			)
			VALUES
			(
			CF_WEBSERVICE_ATTRVALUE_SEQ.nextval,
			p_WSAttrID,
			P_WSROUTINGOBJID,
			p_WSAttrValue,
			v_BeginTS,
			v_BeginTS
		)RETURNING WSATTRVALUEID INTO v_wsattrvalueid
		;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	OPEN RC1 FOR
		SELECT v_wsattrvalueid,
			1 ROWTS
		FROM DUAL 
		;
		
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 CF_WEBSERVICE_ATTRVALUEI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_WEBSERVICE_ATTRVALUEI' || SQLCODE|| ' '||SQLERRM);
END CF_WEBSERVICE_ATTRVALUEI;
/

-- CF_WEBSERVICE_ATTRVALUES_AH1.sql

CREATE OR REPLACE Procedure CF_WEBSERVICE_ATTRVALUES_AH1
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_wsroutingobjid	in		INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       CF_WEBSERVICE_ATTRVALUES_AH1
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_WEBSERVICE_ATTRVALUES_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;


BEGIN
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	OPEN RC1 FOR
		SELECT 	a.WSATTRID,
					a.WSATTRKEY,
					a.WSATTRDESC,
					a.RESOURCEKEY,
					b.wsattrvalue,
					c.ROUTINGOBJTYPEID,
					c.ROUTINGOBJNAME,
					c.ROUTINGOBJCD
				FROM CF_WEBSERVICE_ATTR a,  CF_WEBSERVICE_ATTRVALUE b, cf_routingobject c
				WHERE a.wsattrid = b.wsattrid
				AND b.wsroutingobjid = c.routingobjid	
				AND b.wsroutingobjid = p_wsroutingobjid	
		;               
	p_ReturnCd := 0;
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 CF_WEBSERVICE_ATTRVALUES_AH1' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_WEBSERVICE_ATTRVALUES_AH1' || SQLCODE|| ' '||SQLERRM);
END CF_WEBSERVICE_ATTRVALUES_AH1;
/

-- CF_WEBSERVICE_ATTRVALUEU.sql

CREATE OR REPLACE Procedure CF_WEBSERVICE_ATTRVALUEU
(
	 p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_WSAttrID		in		INTEGER
	, p_wsroutingobjid 	in		INTEGER
	, p_WSAttrValue		in		VARCHAR2
	, p_wsattrvalueid	in 		INTEGER
	, p_RowTS		in		INTEGER
)
AS

/*


	================================================================================
	Proc Name:       CF_WEBSERVICE_ATTRVALUEU
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'CF_WEBSERVICE_ATTRVALUEU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000);
v_RowTS INTEGER;


v_wsattrvalueid    INTEGER;
ZeroUpdate EXCEPTION;


BEGIN
	
	
	v_RowCnt := 0;
		Update CF_WEBSERVICE_ATTRVALUE
		SET
			WSATTRID = p_WSAttrID,
			wsroutingobjid = p_wsroutingobjid,
			WSATTRVALUE = p_WSAttrValue,			
			ROWTS= p_RowTS + 1,
			UPDATEDATE= v_BeginTS
		Where
			WSATTRVALUEID = p_WSAttrValueID
			And RowTS = p_RowTS returning RowTS into v_RowTS;
			
		v_RowCnt := SQL%rowcount;
		
		if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
		
		IF RC1%isOpen THEN Close RC1; END IF;
		OPEN RC1 for select v_RowTS RowTS from Dual;
		p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 CF_WEBSERVICE_ATTRVALUEU' || SQLCODE|| ' '||SQLERRM);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GWYATTRU: No Data Found');
					
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 CF_WEBSERVICE_ATTRVALUEU' || SQLCODE|| ' '||SQLERRM);
END CF_WEBSERVICE_ATTRVALUEU;
/

-- DB_BLOB2VARIABLE.sql

CREATE OR REPLACE PROCEDURE DB_Blob2Variable (
    p_Table_Name in varchar2,
    p_Column_Name in Varchar2,
    p_WhereClause in Varchar2)
    is
/*
    =============================================================================
    Proc Name:      
    Create Date:   
    Author:        

    =============================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_Length pls_integer;
Pointer pls_integer := 1;
i pls_integer := 0;
TempRaw raw (120);
ChunkSize pls_integer := 16000;
Segment   pls_integer := 0;
Length_ToDO pls_integer := 0;
First Boolean := TRUE;
Begin
    Execute Immediate 'Select  dbms_lob.GetLength('||p_Column_Name||')  From '||p_Table_name||'  Where '||p_WhereClause Into v_Length;
    Pointer := 1;
 While v_Length > 0
    Loop
        i       := 0;
        Segment := Segment + 1;
        First   := TRUE;
        DBMS_OUTPUT.Put_Line(p_Column_Name||'_'||segment||' raw (32676) := ');
        IF v_Length > ChunkSize Then
            Length_ToDO := ChunkSize;
        Else
            Length_ToDO := v_Length;
        End IF;
--        DBMS_OUTPUT.Put_Line('Length to process '|| Length_ToDO);
        While i <=  Length_ToDO
        Loop
            Execute Immediate 'Select DBMS_LOB.Substr('||p_Column_Name||','|| Least(60,(Length_ToDO - i))||','||Pointer||') From '
            ||p_Table_name||'  Where '||p_WhereClause INTO TempRaw;
--            If Length_ToDO < 16000 Then
--                DBMS_OUTPUT.Put_Line('Pointer '||Pointer||' SubStr Size '||Least(60,(Length_ToDO - i))  );
--            End if;
            If First Then
                First := False;
                DBMS_OUTPUT.Put_Line( '''' ||  TempRaw||'''' );
            Else
                DBMS_OUTPUT.Put_Line(  '||'||'''' ||  TempRaw||'''' );
            End If;
            Pointer := Pointer + Least(60,(Length_ToDO - i));
            i := i + 60;
        End Loop;
        v_Length := v_Length - Length_ToDO;
        DBMS_OUTPUT.Put_Line(';');
    End Loop;
end DB_Blob2Variable;
/


-- DB_EXCEPTION_HANDLER.sql

CREATE OR REPLACE PROCEDURE DB_EXCEPTION_HANDLER
                            (P_ReturnCd out pls_integer,
                            p_DebugFlg In pls_integer,
                            p_ProcName in Varchar2,
                            p_ParamsTxt in Varchar2,
                            p_beginTS   IN TimeStamp,
                            p_SQLCODE in Varchar2,
                            p_SQLERRM in Varchar2,
                            p_ExceptionCode In Varchar2  -- values are ZeroUpdate, ZeroDelete
                            ) is
ZeroDelete  Exception;
ZeroUpdate  Exception;
TooManyRows Exception;
AllOthers   Exception;
v_ExceptionFlg pls_integer;

/*
    =============================================================================
    Proc Name:      
    Create Date:   
    Author:        

    =============================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

Begin
    DBMS_OUTPUT.Put_Line(' Sql Code: '||p_SQLCODE);
Case
    When p_SQLCODE = -1422 Then Raise TooManyRows;
    When p_SQLCODE = 1 And p_ExceptionCode = 'ZeroDelete' Then Raise ZeroDelete;
    When p_SQLCODE = 1 And p_ExceptionCode = 'ZeroUpdate' Then Raise ZeroUpdate;
    Else
        Raise AllOthers;
End Case;

Exception
    WHEN TooManyRows Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(p_ProcName, p_BeginTS,'TOO_MANY_ROWS,'||p_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_AH1(p_ProcName, p_BeginTS, p_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999, '20100 : Too Many Rows in '||p_ProcName);
    When ZeroDelete Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(p_ProcName, p_BeginTS, 'Zero Delete,'||p_ParamsTxt, p_DebugFlg, 1);
            Commit;
            RAISE_APPLICATION_ERROR (-20999,'20002 : Zero Delete - No Data Found in '||p_ProcName);

    WHEN ZeroUpdate then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(p_ProcName, p_BeginTS,'ZeroUpdate, '||p_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_AH1(p_ProcName, p_BeginTS, p_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 : ZeroUpdate - No Data Found in '||p_ProcName);
    WHEN AllOthers then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(p_ProcName, p_BeginTS, p_SQLERRM||CHR(10)||p_ParamsTxt, p_DebugFlg, 1);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 :'||p_ProcName||' '||' Sql Code: '||p_SQLCODE||' Error: '||p_SQLERRM);
End DB_EXCEPTION_HANDLER;
/


-- DB_INSTALL_CFG_AH1.sql

--================================================================================
-- *
-- *
--================================================================================

CREATE OR REPLACE PROCEDURE DB_INSTALL_CFG_Insert
(
    p_product_key         in  varchar2
    , p_Version        in    varchar2
    , p_BeanId            in  varchar2
    , p_Field       in    varchar2
    , p_Value        in    varchar2
    )
AS
	v_SqlText Varchar2(1000);
	v_rowcnt integer    :=0;
	zeroUpdate  Exception;
begin

	update DB_INSTALL_CFG
	set 
		value=p_Value
	where
		Product_Key = p_product_key and
		BeanId = p_BeanId and
		Field = p_Field and 
			exists ( 
			select Product_key 
			from DB_INSTALL_CFG
			where 
				Product_Key = p_product_key and
				BeanId = p_BeanId and
				Field = p_Field );
	v_RowCnt := SQL%rowcount;
	DBMS_OUTPUT.Put_Line('records updated = ' || v_RowCnt);
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	Exception
        When ZeroUpdate Then
	BEGIN
		-- No record found to update, we need to insert this record.
   		DBMS_OUTPUT.Put_Line('no existing record found, creating a new record');
			insert into DB_INSTALL_CFG 
				(Product_Key, Version, BeanId, Field, Value)
			values (p_product_key, p_Version, p_BeanId, p_Field, p_Value);
			Exception 
			When Others then
       				Raise;
	END;
        When Others Then
        	Raise_Application_Error (-20999, '20001 DB_INSTALL_CFG_Insert error '|| SQLERRM);
End DB_INSTALL_CFG_Insert;
/



-- DB_LENGTHCHECK_ACTIVITY.sql

CREATE OR REPLACE PROCEDURE DB_LENGTHCHECK_ACTIVITY (
     P_INITBUSINESSIDHDR   IN OUT VARCHAR2
   , P_FRBUSINESSIDHDR     IN OUT VARCHAR2
   , P_TOBUSINESSIDHDR     IN OUT VARCHAR2
   , P_INITBUSINESSIDBDY   IN OUT VARCHAR2
   , P_FRBUSINESSIDBDY     IN OUT VARCHAR2
   , P_TOBUSINESSIDBDY     IN OUT VARCHAR2
   , P_FRPROTOCOLNAME      IN OUT VARCHAR2
   , P_TOPROTOCOLNAME      IN OUT VARCHAR2
   , P_FRPROTOCOLVER       IN OUT VARCHAR2
   , P_TOPROTOCOLVER       IN OUT VARCHAR2
   , P_FRPROCESSCD         IN OUT VARCHAR2
   , P_TOPROCESSCD         IN OUT VARCHAR2
   , P_FRPROCESSVER        IN OUT VARCHAR2
   , P_TOPROCESSVER        IN OUT VARCHAR2
   , P_DOCID               IN OUT VARCHAR2
   , P_DOCTS               IN OUT VARCHAR2
   , P_FRIPADDRESS         IN OUT VARCHAR2
   ) is
/*
    ================================================================================
    Proc Name:       DB_LENGTHCHECK_ACTIVITY
    Create Date:     2003-06-18 11:55:56
        This proc returns length of LG_Activity columns
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-06-18 V.Goel           3.5 Created
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   L_BUSINESSID    PLS_INTEGER ;
   L_PROTOCOLNAME  PLS_INTEGER ;
   L_PROTOCOLVER   PLS_INTEGER ;
   L_PROCESSCD     PLS_INTEGER ;
   L_PROCESSVER    PLS_INTEGER ;
   L_IPADDRESS     PLS_INTEGER ;
   L_BusIDType     PLS_INTEGER ;

BEGIN
    DB_LENGTHCHECK_COMMOM(
        P_BUSINESSID    => L_BUSINESSID
        ,P_PROTOCOLNAME => L_PROTOCOLNAME
        ,P_PROTOCOLVER  => L_PROTOCOLVER
        ,P_PROCESSCD    => L_PROCESSCD
        ,P_PROCESSVER   => L_PROCESSVER
        ,P_IPADDRESS    => L_IPADDRESS
        ,P_BUSIDTYPE    => L_BUSIDTYPE
    );
--       p_ACTIVITYID                       := RTRIM(substr(p_ACTIVITYID,1,L_VUID);
       p_DOCID                            := RTRIM(substr(p_DOCID,1,255));
       p_DOCTS                            := RTRIM(substr(p_DOCTS,1,60));
       p_FRIPADDRESS                      := RTRIM(substr(p_FRIPADDRESS,1,L_IPADDRESS));

       p_FRBUSINESSIDBDY                  := RTRIM(substr(p_FRBUSINESSIDBDY,1,L_BUSINESSID));
       p_FRBUSINESSIDHDR                  := RTRIM(substr(p_FRBUSINESSIDHDR,1,L_BUSINESSID));
       p_INITBUSINESSIDBDY                := RTRIM(substr(p_INITBUSINESSIDBDY,1,L_BUSINESSID));
       p_INITBUSINESSIDHDR                := RTRIM(substr(p_INITBUSINESSIDHDR,1,L_BUSINESSID));
       p_TOBUSINESSIDBDY                  := RTRIM(substr(p_TOBUSINESSIDBDY,1,L_BUSINESSID));
       p_TOBUSINESSIDHDR                  := RTRIM(substr(p_TOBUSINESSIDHDR,1,L_BUSINESSID));


       p_FRPROCESSCD                      := RTRIM(substr(p_FRPROCESSCD,1,L_PROCESSCD));
       p_FRPROCESSVER                     := RTRIM(substr(p_FRPROCESSVER,1,L_PROCESSVER));
       p_TOPROCESSCD                      := RTRIM(substr(p_TOPROCESSCD,1,L_PROCESSCD));
       p_TOPROCESSVER                     := RTRIM(substr(p_TOPROCESSVER,1,L_PROCESSVER));

       p_FRPROTOCOLNAME                   := RTRIM(substr(p_FRPROTOCOLNAME,1,L_PROTOCOLNAME));
       p_FRPROTOCOLVER                    := RTRIM(substr(p_FRPROTOCOLVER,1,L_PROTOCOLVER));
       p_TOPROTOCOLNAME                   := RTRIM(substr(p_TOPROTOCOLNAME,1,L_PROTOCOLNAME));
       p_TOPROTOCOLVER                    := RTRIM(substr(p_TOPROTOCOLVER,1,L_PROTOCOLVER));

END DB_LENGTHCHECK_ACTIVITY;
/


-- DB_LENGTHCHECK_ACTIVITY_RNDTL.sql

CREATE OR REPLACE PROCEDURE DB_LENGTHCHECK_ACTIVITY_RNDTL (
     P_ACTSIGTRACKINSTANCEID  IN OUT VARCHAR2
   , P_TRANSACTIONINSTANCEID  IN OUT VARCHAR2
   , P_GLBUSAGECD             IN OUT VARCHAR2
   , P_MSGDATETIME            IN OUT VARCHAR2
   , P_FRBUSIDTYPE            IN OUT VARCHAR2
   , P_TOBUSIDTYPE            IN OUT VARCHAR2
   , P_IRTGLBBUSACTIONCD      IN OUT VARCHAR2
   , P_IRTINSTANCEID          IN OUT VARCHAR2
   , P_GLBBUSACTIONCD         IN OUT VARCHAR2
   , P_SECURETRANSREQD        IN OUT VARCHAR2
   , P_FRSERVICE              IN OUT VARCHAR2
   , P_TOSERVICE              IN OUT VARCHAR2
   , P_GLBDOCFUNCTIONCD       IN OUT VARCHAR2
   , P_FRGLBPARTNERCLASSCD    IN OUT VARCHAR2
   , P_TOGLBPARTNERCLASSCD    IN OUT VARCHAR2
   , P_GLBBUSSIGNALCD         IN OUT VARCHAR2
   , P_FRROLE                 IN OUT VARCHAR2
   , P_TOROLE                 IN OUT VARCHAR2
   , P_ACTIVITYNAME           IN OUT VARCHAR2
   , P_GLBPROCESSCD           IN OUT VARCHAR2
   ) is
/*
    ================================================================================
    Proc Name:       DB_LENGTHCHECK_ACTIVITY_RNDTL
    Create Date:     2003-06-18 11:55:56
        This proc returns length of LG_Activity_rnDTL columns
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-06-18 V.Goel           3.5 Created
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   L_BUSINESSID    PLS_INTEGER ;
   L_PROTOCOLNAME  PLS_INTEGER ;
   L_PROTOCOLVER   PLS_INTEGER ;
   L_PROCESSCD     PLS_INTEGER ;
   L_PROCESSVER    PLS_INTEGER ;
   L_IPADDRESS     PLS_INTEGER ;
   L_BusIDType     PLS_INTEGER ;

BEGIN
    DB_LENGTHCHECK_COMMOM(
        P_BUSINESSID    => L_BUSINESSID
        ,P_PROTOCOLNAME => L_PROTOCOLNAME
        ,P_PROTOCOLVER  => L_PROTOCOLVER
        ,P_PROCESSCD    => L_PROCESSCD
        ,P_PROCESSVER   => L_PROCESSVER
        ,P_IPADDRESS    => L_IPADDRESS
        ,P_BUSIDTYPE    => L_BUSIDTYPE
    );

--   p_INITBUSINESSID                   := RTRIM(substr(p_INITBUSINESSID,1,20);
--   p_PROCESSINSTANCEID                := RTRIM(substr(p_PROCESSINSTANCEID,1,255);
   p_ACTSIGTRACKINSTANCEID            := RTRIM(substr(p_ACTSIGTRACKINSTANCEID,1,512));
   p_TRANSACTIONINSTANCEID            := RTRIM(substr(p_TRANSACTIONINSTANCEID,1,255));
   p_GLBUSAGECD                       := RTRIM(substr(p_GLBUSAGECD,1,10));
   p_MSGDATETIME                      := RTRIM(substr(p_MSGDATETIME,1,20));
   p_FRBUSIDTYPE                      := RTRIM(substr(p_FRBUSIDTYPE,1,L_BusIDType));
   p_TOBUSIDTYPE                      := RTRIM(substr(p_TOBUSIDTYPE,1,L_BusIDType));
   p_IRTGLBBUSACTIONCD                := RTRIM(substr(p_IRTGLBBUSACTIONCD,1,255));
   p_IRTINSTANCEID                    := RTRIM(substr(p_IRTINSTANCEID,1,255));
   p_GLBBUSACTIONCD                   := RTRIM(substr(p_GLBBUSACTIONCD,1,255));
   p_SECURETRANSREQD                  := RTRIM(substr(p_SECURETRANSREQD,1,10));
   p_FRSERVICE                        := RTRIM(substr(p_FRSERVICE,1,255));
   p_TOSERVICE                        := RTRIM(substr(p_TOSERVICE,1,255));
   p_GLBDOCFUNCTIONCD                 := RTRIM(substr(p_GLBDOCFUNCTIONCD,1,60));
   p_FRGLBPARTNERCLASSCD              := RTRIM(substr(p_FRGLBPARTNERCLASSCD,1,256));
   p_TOGLBPARTNERCLASSCD              := RTRIM(substr(p_TOGLBPARTNERCLASSCD,1,256));
   p_GLBBUSSIGNALCD                   := RTRIM(substr(p_GLBBUSSIGNALCD,1,60));
   p_FRROLE                           := RTRIM(substr(p_FRROLE,1,255));
   p_TOROLE                           := RTRIM(substr(p_TOROLE,1,255));
   p_ACTIVITYNAME                     := RTRIM(substr(p_ACTIVITYNAME,1,255));
   p_GLBPROCESSCD                     := RTRIM(substr(p_GLBPROCESSCD,1,255));

End DB_LENGTHCHECK_ACTIVITY_RNDTL;
/


-- DB_LENGTHCHECK_ACTIVITY_RNHDR.sql

CREATE OR REPLACE PROCEDURE DB_LENGTHCHECK_ACTIVITY_RNHDR (
     P_INITBUSINESSID     IN OUT VARCHAR2
   , P_PROCESSINSTANCEID  IN OUT VARCHAR2
   , P_INITBUSIDTYPE      IN OUT VARCHAR2
   , P_PROTOCOLNAME       IN OUT VARCHAR2
   , P_PROTOCOLVER        IN OUT VARCHAR2
   , P_PROCESSCD          IN OUT VARCHAR2
   , P_PROCESSVER         IN OUT VARCHAR2
   ) is
/*
    ================================================================================
    Proc Name:       DB_LENGTHCHECK_ACTIVITY_RNHDR
    Create Date:     2003-06-18 11:55:56
        This proc returns length of LG_Activity_rnHdr columns
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-06-18 V.Goel           3.5 Created
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   L_BUSINESSID    PLS_INTEGER ;
   L_PROTOCOLNAME  PLS_INTEGER ;
   L_PROTOCOLVER   PLS_INTEGER ;
   L_PROCESSCD     PLS_INTEGER ;
   L_PROCESSVER    PLS_INTEGER ;
   L_IPADDRESS     PLS_INTEGER ;
   L_BusIDType     PLS_INTEGER ;

BEGIN
    DB_LENGTHCHECK_COMMOM(
        P_BUSINESSID    => L_BUSINESSID
        ,P_PROTOCOLNAME => L_PROTOCOLNAME
        ,P_PROTOCOLVER  => L_PROTOCOLVER
        ,P_PROCESSCD    => L_PROCESSCD
        ,P_PROCESSVER   => L_PROCESSVER
        ,P_IPADDRESS    => L_IPADDRESS
        ,P_BUSIDTYPE    => L_BUSIDTYPE
    );

   p_INITBUSINESSID                   := RTRIM(substr(p_INITBUSINESSID,1,L_BUSINESSID));
   p_PROCESSINSTANCEID                := RTRIM(substr(p_PROCESSINSTANCEID,1,255));
   p_INITBUSIDTYPE                    := RTRIM(substr(p_INITBUSIDTYPE,1,L_BusIDType));
   p_PROTOCOLNAME                     := RTRIM(substr(p_PROTOCOLNAME,1,L_PROTOCOLNAME));
   p_PROTOCOLVER                      := RTRIM(substr(p_PROTOCOLVER,1,L_PROTOCOLVER));
   p_PROCESSCD                        := RTRIM(substr(p_PROCESSCD,1,L_PROCESSCD));
   p_PROCESSVER                       := RTRIM(substr(p_PROCESSVER,1,L_PROCESSVER));
End DB_LENGTHCHECK_ACTIVITY_RNHDR;
/


-- DB_LENGTHCHECK_COMMOM.sql

CREATE OR REPLACE PROCEDURE DB_LENGTHCHECK_COMMOM
    (
    p_BUSINESSID    OUT PLS_INTEGER ,
    p_PROTOCOLNAME  OUT PLS_INTEGER ,
    p_PROTOCOLVER   OUT PLS_INTEGER ,
    p_PROCESSCD     OUT PLS_INTEGER ,
    p_PROCESSVER    OUT PLS_INTEGER ,
    p_IPADDRESS     OUT PLS_INTEGER ,
    p_BusIDType     OUT PLS_INTEGER
   )
   AS
/*
    ================================================================================
    Proc Name:       DB_LENGTHCHECK_COMMOM
    Create Date:     2003-06-18 11:55:56
        This proc returns length of common columns
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-06-18 V.Goel           3.5 Created
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   L_BUSINESSID    PLS_INTEGER := 60;
   L_PROTOCOLNAME  PLS_INTEGER := 60;
   L_PROTOCOLVER   PLS_INTEGER := 30;
   L_PROCESSCD     PLS_INTEGER := 255;
   L_PROCESSVER    PLS_INTEGER := 30;
   L_IPADDRESS     PLS_INTEGER := 45;
   L_BusIDType     PLS_INTEGER := 30;

BEGIN
    p_BUSINESSID   := L_BUSINESSID   ;
    p_PROTOCOLNAME := L_PROTOCOLNAME ;
    p_PROTOCOLVER  := L_PROTOCOLVER  ;
    p_PROCESSCD    := L_PROCESSCD    ;
    p_PROCESSVER   := L_PROCESSVER   ;
    p_IPADDRESS    := L_IPADDRESS    ;
    p_BusIDType    := L_BusIDType    ;

END DB_LENGTHCHECK_COMMOM;
/


-- DB_LENGTHCHECK_EVENT.sql

CREATE OR REPLACE PROCEDURE DB_LENGTHCHECK_EVENT
    (
     P_SOURCECLASS    IN OUT VARCHAR2
    , P_HOSTIPADDRESS  IN OUT VARCHAR2
    , P_ARGS           IN OUT VARCHAR2
    ) IS

/*
    ================================================================================
    Proc Name:       DB_LENGTHCHECK_ACTIVITY
    Create Date:     2003-06-18 11:55:56
        This proc returns length of LG_Activity columns
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-06-18 V.Goel           3.5 Created
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   L_BUSINESSID    PLS_INTEGER ;
   L_PROTOCOLNAME  PLS_INTEGER ;
   L_PROTOCOLVER   PLS_INTEGER ;
   L_PROCESSCD     PLS_INTEGER ;
   L_PROCESSVER    PLS_INTEGER ;
   L_IPADDRESS     PLS_INTEGER ;
   L_BusIDType     PLS_INTEGER ;

BEGIN
    DB_LENGTHCHECK_COMMOM(
        P_BUSINESSID    => L_BUSINESSID
        ,P_PROTOCOLNAME => L_PROTOCOLNAME
        ,P_PROTOCOLVER  => L_PROTOCOLVER
        ,P_PROCESSCD    => L_PROCESSCD
        ,P_PROCESSVER   => L_PROCESSVER
        ,P_IPADDRESS    => L_IPADDRESS
        ,P_BUSIDTYPE    => L_BUSIDTYPE
    );

--   p_EVENTID                          := RTRIM(substr(p_EVENTID,1,65);
   p_SOURCECLASS                      := RTRIM(substr(p_SOURCECLASS,1,255));
   p_HOSTIPADDRESS                    := RTRIM(substr(p_HOSTIPADDRESS,1,L_IPADDRESS));
   p_ARGS                             := RTRIM(substr(p_ARGS,1,1024));

END DB_LENGTHCHECK_EVENT;
/


-- DB_LENGTHCHECK_SUMMARY_XREF.sql

CREATE OR REPLACE PROCEDURE DB_LENGTHCHECK_SUMMARY_XREF (
     P_INITBUSINESSID     IN OUT VARCHAR2
   , P_FRBUSINESSID       IN OUT VARCHAR2
   , P_TOBUSINESSID       IN OUT VARCHAR2
   , P_FRPROTOCOLNAME     IN OUT VARCHAR2
   , P_TOPROTOCOLNAME     IN OUT VARCHAR2
   , P_FRPROCESSCD        IN OUT VARCHAR2
   , P_FRPROCESSVER       IN OUT VARCHAR2
   , P_TOPROCESSCD        IN OUT VARCHAR2
   , P_TOPROCESSVER       IN OUT VARCHAR2
   , P_FRIPADDRESS        IN OUT VARCHAR2
   , P_FRPROTOCOLVER      IN OUT VARCHAR2
   , P_TOPROTOCOLVER      IN OUT VARCHAR2
   ) is
/*
    ================================================================================
    Proc Name:       DB_LENGTHCHECK_SUMMARY_XREF
    Create Date:     2003-06-18 11:55:56
        This proc returns length of LG_Summary_Xref columns
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-06-18 V.Goel           3.5 Created
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   L_BUSINESSID    PLS_INTEGER ;
   L_PROTOCOLNAME  PLS_INTEGER ;
   L_PROTOCOLVER   PLS_INTEGER ;
   L_PROCESSCD     PLS_INTEGER ;
   L_PROCESSVER    PLS_INTEGER ;
   L_IPADDRESS     PLS_INTEGER ;
   L_BusIDType     PLS_INTEGER ;

BEGIN
    DB_LENGTHCHECK_COMMOM(
        P_BUSINESSID    => L_BUSINESSID
        ,P_PROTOCOLNAME => L_PROTOCOLNAME
        ,P_PROTOCOLVER  => L_PROTOCOLVER
        ,P_PROCESSCD    => L_PROCESSCD
        ,P_PROCESSVER   => L_PROCESSVER
        ,P_IPADDRESS    => L_IPADDRESS
        ,P_BUSIDTYPE    => L_BUSIDTYPE
    );

   p_INITBUSINESSID                   := RTRIM(substr(p_INITBUSINESSID,1,L_BUSINESSID));
   p_FRBUSINESSID                     := RTRIM(substr(p_FRBUSINESSID,1,L_BUSINESSID));
   p_TOBUSINESSID                     := RTRIM(substr(p_TOBUSINESSID,1,L_BUSINESSID));
   p_FRPROTOCOLNAME                   := RTRIM(substr(p_FRPROTOCOLNAME,1,L_PROTOCOLNAME));
   p_TOPROTOCOLNAME                   := RTRIM(substr(p_TOPROTOCOLNAME,1,L_PROTOCOLNAME));
   p_FRPROCESSCD                      := RTRIM(substr(p_FRPROCESSCD,1,L_PROCESSCD));
   p_FRPROCESSVER                     := RTRIM(substr(p_FRPROCESSVER,1,L_PROCESSVER));
   p_TOPROCESSCD                      := RTRIM(substr(p_TOPROCESSCD,1,L_PROCESSCD));
   p_TOPROCESSVER                     := RTRIM(substr(p_TOPROCESSVER,1,L_PROCESSVER));
   p_FRIPADDRESS                      := RTRIM(substr(p_FRIPADDRESS,1,L_IPADDRESS));
   p_FRPROTOCOLVER                    := RTRIM(substr(p_FRPROTOCOLVER,1,L_PROTOCOLVER));
   p_TOPROTOCOLVER                    := RTRIM(substr(p_TOPROTOCOLVER,1,L_PROTOCOLVER));
End DB_LENGTHCHECK_SUMMARY_XREF;
/


-- DB_LENGTHCHECK_VALIDATION.sql

CREATE OR REPLACE PROCEDURE DB_LENGTHCHECK_VALIDATION (
     P_FIELD       IN OUT VARCHAR2
   , P_VALUE       IN OUT VARCHAR2
   , P_ERROR       IN OUT VARCHAR2
) is
/*
    ================================================================================
    Proc Name:       DB_LENGTHCHECK_VALIDATION
    Create Date:     2003-06-18 11:55:56
        This proc returns length of LG_Validation columns
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-06-18 V.Goel           3.5 Created
    ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   L_BUSINESSID    PLS_INTEGER ;
   L_PROTOCOLNAME  PLS_INTEGER ;
   L_PROTOCOLVER   PLS_INTEGER ;
   L_PROCESSCD     PLS_INTEGER ;
   L_PROCESSVER    PLS_INTEGER ;
   L_IPADDRESS     PLS_INTEGER ;
   L_BusIDType     PLS_INTEGER ;

BEGIN
/*
    DB_LENGTHCHECK_COMMOM(
        P_BUSINESSID    => L_BUSINESSID
        ,P_PROTOCOLNAME => L_PROTOCOLNAME
        ,P_PROTOCOLVER  => L_PROTOCOLVER
        ,P_PROCESSCD    => L_PROCESSCD
        ,P_PROCESSVER   => L_PROCESSVER
        ,P_IPADDRESS    => L_IPADDRESS
        ,P_BUSIDTYPE    => L_BUSIDTYPE
    );
*/
   p_FIELD                            := RTRIM(substr(p_FIELD,1,1024));
   p_VALUE                            := RTRIM(substr(p_VALUE,1,1024));
   p_ERROR                            := RTRIM(substr(p_ERROR,1,2048));
End DB_LENGTHCHECK_VALIDATION;
/


-- DB_LIST_DIRECTORY.sql

CREATE OR REPLACE PROCEDURE db_list_directory (
	p_returnCd 	OUT NUMBER
	,p_DebugFlg 	IN NUMBER
	,RC1		OUT db_Ref.RCT
	,p_ArchDest	IN VARCHAR2
	,p_FrDate	IN VARCHAR2
	,p_ToDate	IN VARCHAR2
)
AS
/*
	================================================================================
	Proc Name:	AR_CREATE_DIRECTORY
	Create Date:	2008-11-08 14:07:19
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-11-08 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------
	v_dirList	NUMBER(1)	DEFAULT 0;
	v_BeginTS	timestamp Default FN_Get_Date;  -- Performance related variables
	v_ProcName	varchar2(30)	Default 'DB_LIST_DIRECTORY';

	v_ParamsTxt     varchar2(4000) Default
			'p_DebugFlg=>'||p_DebugFlg||CHR(10)
			 		||', p_ArchDest=>'||p_ArchDest||CHR(10)
			 		||', p_FrDate=>'||p_FrDate||CHR(10)
			 		||', p_ToDate=>'||p_ToDate||CHR(10)
	;

BEGIN
	--- grant read permission on the path and all subfolders

	v_dirList := fn_get_dirlist(p_ArchDest, p_FrDate, p_ToDate);

	p_ReturnCd := v_dirlist;
	IF v_dirList = 0 THEN
		OPEN RC1 FOR
		SELECT FileDir, FileName, ModifiedDate	
		FROM	AR_RESTORE_LIST
		ORDER BY MODIFIEDDATE
		;
	END IF;
END;
/


-- DB_METADATA_LOAD.sql

CREATE OR REPLACE PROCEDURE DB_MetaData_Load
    (
     p_ReturnCd  Out Integer
     ,p_DebugFlg in integer
     ,p_LocaleTable in varchar2
     ,p_ParentTable in varchar2
     ,p_KeyValue in varchar2
     ,p_KeyColumn in varchar2
     ,p_IdColumn in varchar2
     ,p_DisplayValue in varchar2
     ,p_DisplayColumn in varchar2
     ,p_Locale in varchar2
     ,p_AttributeKey in varchar2
    )
AS
 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar2(30)  Default 'DB_METADATA_LOAD';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_ParamsTxt       varchar2(4000) ;
 v_PK_Value        number(10) Default 0;
 v_SQLErrm         Varchar2(255) ;
 v_msg             Varchar2(500);
 v_RowTS           Integer Default 0;
 v_ID              number(10);
 v_From            varchar2(1000);
 v_KeyColumn       varchar2(50);
 v_Sqlstmt         varchar2(4000);
 v_sqlstmt_ID      varchar2(4000);
 v_DisplayColumnCnt number(1) Default 0;
 v_DisplayColumn2  varchar2(1000) Default '';
 v_AttrName        varchar2(1000) Default '';
 
Begin
/*
	================================================================================
	Proc Name:       DB_METADATA_LOAD
	Create Date:     2003-10-02 15:15:47
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-10-02 SYSTEM           3.5 Load locale metadata by given input

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

   
	p_ReturnCd := 0;
--  Add new locale into pr_locale table    
    Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg)
    select p_locale, p_locale, 0 from Dual
    Where Not Exists (select 'x' from PR_LOCALE where LOCALE = p_locale);
    
    SELECT INSTR(NVL(p_DisplayColumn, ''), '+', 1, 1) INTO v_DisplayColumnCnt FROM DUAL;
    
    IF v_DisplayColumnCnt <> 0 THEN  -- Got combined field. 
        SELECT SUBSTR(p_DisplayColumn, 1, v_DisplayColumnCnt - 1) INTO v_AttrName FROM DUAL;
        SELECT SUBSTR(p_DisplayColumn, v_DisplayColumnCnt + 1, LENGTH(p_DisplayColumn) - v_DisplayColumnCnt) INTO v_DisplayColumn2 FROM DUAL;
    ELSE
        v_DisplayColumn2 := p_DisplayColumn;        
    END IF;    
    
    db_print('p_DisplayColumn', p_DisplayColumn);
    db_print('v_AttrName', v_AttrName);
    db_print('v_DisplayColumn2', v_DisplayColumn2);
    db_print('p_localetable', p_localetable);
    
    IF p_AttributeKey IS NOT NULL THEN
        v_sqlStmt := ' Select p.'||p_IDColumn|| ' From '||
                          p_LocaleTable || ' l inner join '|| p_ParentTable || ' p on l.'|| p_IDColumn || ' = p.'|| p_IDColumn ||
                         ' Inner join cf_routingAttr a on a.routingattrid = p.routingattrid '||
                         ' Where l.locale = '''||p_locale ||''' And p.'||p_KeyColumn ||' = '''||p_KeyValue||''''||
                         ' And a.attributekey = ''' || p_AttributeKey || '''';
    ELSE
        v_sqlStmt := ' Select p.'||p_IDColumn|| ' From '||
                          p_LocaleTable || ' l inner join '|| p_ParentTable || ' p on l.'|| p_IDColumn || ' = p.'|| p_IDColumn ||
                         ' Where l.locale = '''||p_locale ||''' And p.'||p_KeyColumn ||' = '''||p_KeyValue||'''';
    END If;
    
    db_print('v_sqlStmt1 : ', v_sqlStmt);
    execute immediate v_sqlstmt into v_ID;

    If v_ID is not null Then
       v_Sqlstmt := 'Update '||p_localeTable||
                    ' Set '|| v_DisplayColumn2 ||' = '''|| p_DisplayValue ||''', Updatedate = FN_get_date()'||
                    ' Where '|| p_IDColumn || ' = '|| v_ID || ' And locale = '''||p_Locale ||'''' ;
    End If;
    
    db_print('v_sqlStmt2 : ', v_sqlStmt);
	Execute immediate v_sqlstmt;

    Exception
        when NO_DATA_FOUND then
            Begin
                If p_AttributeKey is not null Then
                    v_sqlstmt_ID := ' Select '|| p_IDColumn|| ' From '|| p_ParentTable || ' o ' ||
                                ' Inner join cf_routingAttr a on a.routingattrid = o.routingattrid '||
                                ' Where '|| p_KeyColumn ||' = '''||p_KeyValue||''''||
                                ' And a.attributekey = ''' || p_AttributeKey || '''';
                else
                    v_sqlstmt_ID := ' Select '|| p_IDColumn|| ' From '|| p_ParentTable ||  ' Where '|| p_KeyColumn ||' = '''||p_KeyValue||'''';
                end if;
                

                db_print('v_sqlstmt_ID : ', v_sqlstmt_ID);
                execute immediate v_sqlstmt_ID into v_ID;
            
                IF v_DisplayColumnCnt <> 0 THEN
                    v_sqlstmt := 'INSERT INTO '|| p_LocaleTable || '('|| p_IDColumn || ', Locale, ' || v_AttrName || ',' || v_DisplayColumn2 ||
	                            ') VALUES ('|| v_ID ||','''|| p_Locale ||''','''|| p_DisplayValue ||''','''|| p_DisplayValue ||''')';                
                ELSE            
                    v_sqlstmt := 'INSERT INTO '|| p_LocaleTable || '('|| p_IDColumn || ', Locale, ' || v_DisplayColumn2 ||
	                            ') VALUES ('|| v_ID ||','''|| p_Locale ||''','''|| p_DisplayValue ||''')';
                END IF;                                

                db_print('v_sqlStmt3 : ', v_sqlStmt);
                execute immediate v_sqlstmt;

                Exception
                    When NO_DATA_FOUND then
	                    p_ReturnCd := 3;
                    when others then
                        p_ReturnCd := 1;
                        Raise_application_error (-20999,'20000 db_metadata_load: within exception_handler '||SQLERRM);
            End;
        when others then
            p_ReturnCd := 1;
            Raise_application_error (-20999,'20000 db_metadata_load '||SQLERRM);
End;
/


-- DB_PRINT.sql

CREATE OR REPLACE PROCEDURE DB_PRINT(
        P_MSG VARCHAR2,
        P_STMNT VARCHAR2
        ) IS
/*
    =============================================================================
    Proc Name:      DB_PRINT
    Create Date:    2003-06-20
    Author:         V.Goel

    Purpose: This proc prints a statement  of any size (Larger than 255 characters)

    Data Definition:
        P_MSG   : It is an ID to distinguish a message if several messages are being printed. Max Length =255
        P_STMNT : Statement that needed to be printed Less than 32K

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------ ------ ---------------------------------------------
    DATE       NAME         VER     DESCRIPTION
    ---------- ------------ ------ ---------------------------------------------
    2003-06-20 V Goel       3.5     Created

    =============================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

       offset        INTEGER DEFAULT 1;
       buffer_size   INTEGER DEFAULT 200;
BEGIN
   DBMS_OUTPUT.Put_Line(p_msg);
    LOOP
        DBMS_OUTPUT.Put_Line(SUBSTR(p_Stmnt, offset, buffer_size));
        offset := offset + buffer_size;
        EXIT WHEN SUBSTR(p_Stmnt, offset, buffer_size) IS NULL;
    END LOOP;
END DB_PRINT;
/


-- DB_PROCAUDITLOGI_AH1.sql

CREATE OR REPLACE PROCEDURE DB_ProcAuditLogI_AH1
(
   p_ProcName            IN varchar2,
   p_StartTS            IN TimeStamp,
   p_ParamsTxt        IN varchar2 DEFAULT NULL,
   p_DebugFlg           IN number DEFAULT NULL,
   p_ExceptionFlg       IN number DEFAULT NULL
)
AS

/*
    =============================================================================
    Proc Name:      DB_ProcAuditLogI_AH1
    Create Date:    2003-02-17
    Author:         B.Moffat

         Data Definition:

         p_ProcName          = Proc Name
         p_StartTS            = Start time of the proc
         p_ParamsTxt        = Free form parameter text
         p_DebugFlg          = 0, 1 : off/on audit logging
         p_ExceptionFlg      = 0, 1 : denote exception or not


         1.  AuditFlg:  0 = Do not Insert an audit row in DB_ProcAuditLog
                        1 = Insert an audit row in DB_ProcAuditLog. If this
                            is set and FileOutputFlg = 1, then a line will be
                            written to the audit output file. AuditFlg always
                            takes presidence though.
         2.  FileOutpugFlg:  0 = Do not write an audit line to the output file
                             1 = Write an audit line to the output file
         3.  ProcAuditName = '/NONE' - this is the default directory entry in
                             DB_ProcAuditLog to denote a valid output directory
                             has not been entered for auditing.

    Procedure Logic:
    ---------------
    1.   If p_DebugFlg = 1 or DB_ProcAuditNames.AuditFlg = 1 then an audit row
         will be written to DB_ProcAuditLog.
    2.   If the p_ProcName passed in is not found in DB_ProcAuditNames, the
         row will be inserted one time with AuditFlg = 0.
    3.   There is a reserved row for audit file output where the name begins with "/".
         If there is no row with the ProcName like '/%', then the row will be
         inserted one time with '/NONE'. The row can be updated later with the directory
         path for auidt file output.
    4.   To write to the output audit file, the reserved file output audit row will be
         checked for FileAuditFlg = 1 as well as the row for the specific proc.
    5.   To write a row to DB_ProcAuditLog, but no audit file output:
              ProcName = 'PR_PARTNERI' AND AuditFlg = 1 and FileOutputFlg = 0
    6.   To write a row to DB_ProcAuditLog along with audit file output:
              ProcName = '/tmp'  AND FileAuditFlg = 1
              AND ProcName = 'PR_PARTNERI' AND AuditFlg = 1 and FileOutputFlg = 1
    7.   To turn off audit file output for a proc
              ProcName = 'PR_PARTNERI'  AND FileAuditFlg = 0
    8.   To globally turn off audit file output
              ProcName = '/tmp'  AND FileAuditFlg = 0


                           :::::::::::::::::::::::::
                           M O D I F I C A T I O N S
                           :::::::::::::::::::::::::

    ---------- ------------ ----- -----------------------------------------------
    DATE       NAME         Ver.  DESCRIPTION
    ---------- ------------ ----- -----------------------------------------------
    2003-02-17 B.Moffat     3.5.0 Created
    2003-04-04 B.Moffat     3.5.0 Making inserting to table independent of writing to disk file
    2003-04-18 B.Moffat     3.5.0 p_ParamsTxt will be substringed to 1024 because
                                  DB_ProcAuditLog.ParmsTxt is 1024.
    2003-04-18 V.Goel       3.5.0 Add dup_val_on_index to InsertProcName
    2003-07-02 V.Goel       3.5.0 Move commit
    =============================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

    v_F                 utl_file.file_type;
    v_EndTS             Timestamp(6) := FN_Get_Date;
    v_Instance          varchar2(30) ;
    v_Dir               varchar2(512);
    v_ProcName          varchar2(30) := UPPER(p_ProcName);
    v_File              varchar(128) := 'ProcAudit-'||USER||'.txt';
    v_Msg               varchar2(32000) := RPAD(v_ProcName, 31, ' ')
                                ||RPAD(to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS'), 21, ' ')
                                ||RPAD(to_char(v_EndTS - p_StartTS), 8)
                                ||'>'||substr(p_ParamsTxt, 1, 29000)||'<';
    --v_AuditFlg          number(1) := p_DebugFlg;
    v_AuditFlg          number(1) := 0;
    --v_FileAuditFlg      number(1) := p_DebugFlg;
    v_FileAuditFlg      number(1) := 0;
    v_FileOutputFlg     number(1) := 0;

    pragma AUTONOMOUS_TRANSACTION;  -- Specifying Autonomous Transaction

Procedure InsertProcName (p_ProcName2 IN varchar2)
IS

BEGIN
    Insert into DB_ProcAuditNames (ProcName, AuditFlg) values (p_ProcName2, 0);
    Exception
        When Dup_Val_On_Index Then Null;
        When others then raise_application_error (-20999, '20001, DB_ProcAuditNames Insert Error'||substr(sqlerrm,1,80));
END InsertProcName;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF NVL(p_DebugFlg, 0) = 0 THEN
         BEGIN
              SELECT AuditFlg, FileAuditFlg INTO v_AuditFlg, v_FileAuditFlg
              FROM DB_ProcAuditNames WHERE ProcName = Upper(v_ProcName);
              Exception
                   When NO_DATA_FOUND THEN
                        InsertProcName (UPPER(p_ProcName));
                        v_AuditFlg := 0;
                   When OTHERS then v_AuditFlg := 0;
         END;
    END IF;

    IF v_AuditFlg = 1 THEN
         BEGIN
               INSERT into DB_ProcAuditLog (ProcName, CreateDate, StartTS, ParamsTxt, ElapsedTime, ExceptionFlg)
               values (substr(UPPER(p_ProcName), 1, 30), FN_Get_Date, p_startTS, substr(p_ParamsTxt, 1, 1024), (v_EndTS - p_startTS), NVL(p_ExceptionFlg, 0) );
               Exception
                   when others then raise_application_error (-20999, '20002, DB_ProcAuditLog Insert Error: '||substr(sqlerrm,1,80));
         END;
         BEGIN
              select ProcName, AuditFlg into v_Dir, v_FileOutputFlg from DB_ProcAuditNames where ProcName like '/%';
              Exception
                   When NO_DATA_FOUND THEN
                        v_Dir := '/NONE';
                        InsertProcName (v_Dir);
                   When OTHERS THEN
                        v_Dir := '/NONE';
         END;
    END IF;  --v_AuditFlg = 1
    Commit;
    IF v_FileAuditFlg = 1 THEN
         IF v_Dir != '/NONE' AND v_FileAuditFlg = 1 THEN
              utl_file.fclose_all;
              BEGIN
                 v_F := utl_file.fopen( v_Dir, v_File, 'A', 4000 );
                 EXCEPTION
                     WHEN utl_file.invalid_path THEN
                         v_F := utl_file.fopen( v_dir, v_File, 'W' );
              END;
              utl_file.put_line( v_F, v_Msg );
              utl_file.fclose( v_F );
        END IF;

    END IF;
END DB_ProcAuditLogI_AH1;
/


-- DB_STATS.sql

CREATE OR REPLACE PROCEDURE DB_Stats (p_Owner varchar2 default null)
as
/*
    =============================================================================
    Proc Name:      
    Create Date:   
    Author:        

    =============================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

BEGIN
    dbms_stats.gather_schema_stats(ownname => NVL(p_owner, user),
    estimate_percent => dbms_stats.auto_sample_size,
    method_opt=>'for all columns size auto', cascade=>true, degree=>8);
END;
/


-- DB_SYSPROPERTIESS.sql

CREATE OR REPLACE PROCEDURE DB_SYSPROPERTIESS
	(
	p_ReturnCd	OUT	INTEGER
	,p_DebugFlg	IN	PLS_INTEGER
	,RC1		OUT	DB_REF.RCT
	,p_SysPropID	IN	NUMBER
	) AS

/*
	================================================================================
	Proc Name:       DB_SYSPROPERTIESS

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

v_RowCnt          integer Default 0;
v_BeginTS         TimeStamp  ; -- Performance related variables
v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIES';
v_ExceptionFlg    integer  Default 0;
v_ExceptionCode   varchar2(30);
v_PK_Value        Integer Default 0;
v_SQLErrm         Varchar2(255) ;
v_chkpt           Char(1) default '0';
v_msg             Varchar2(500);
v_RowTS           Integer Default 0;

v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_SYSPROPID=>'||p_SYSPROPID||CHR(10)
		;
BEGIN
IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_SysPropID IS NOT NULL THEN
    OPEN RC1 FOR
	SELECT
		 SYSPROPID
		,RTRIM(PROPNAME) PROPNAME
		,PROPVALUE
		,COMPONENT
		,SYSPROPGRPID
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,ROWTS
	FROM
		DB_SYSPROPERTIES
	WHERE
		SYSPROPID = p_SysPropID	;
ELSE
    OPEN RC1 FOR
	SELECT
		 SYSPROPID
		,RTRIM(PROPNAME) PROPNAME
		,PROPVALUE
		,COMPONENT
		,SYSPROPGRPID		
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,ROWTS
	FROM
		DB_SYSPROPERTIES ;
END IF;
p_returncd := 0;
db_procauditlogi_ah1(v_procname, v_begints, v_paramstxt, p_debugflg, v_exceptionflg);
END;
/


-- DB_SYSPROPERTIESS_AH1.sql

CREATE OR REPLACE PROCEDURE DB_SYSPROPERTIESS_AH1
	(
	p_ReturnCd	OUT	INTEGER
	,p_DebugFlg	IN	PLS_INTEGER
	,RC1		OUT	DB_REF.RCT
	,p_PropName	IN	VARCHAR2
	) AS

/*
	================================================================================
	Proc Name:       DB_SYSPROPERTIESS

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIES';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_PK_Value        Integer Default 0;
 v_SQLErrm         Varchar2(255) ;
 v_chkpt           Char(1) default '0';
 v_msg             Varchar(500);
 v_RowTS           Integer Default 0;
 v_ParamsTxt	   varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PropName=>'||p_PropName||CHR(10)
		;

BEGIN
IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PropName IS NOT NULL THEN
	OPEN RC1 FOR
	SELECT
		 SYSPROPID
		,RTRIM(PROPNAME) PROPNAME
		,PROPVALUE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,ROWTS
	FROM
		DB_SYSPROPERTIES
	WHERE
		PROPNAME LIKE p_PropName || '%'	;
ELSE
	OPEN RC1 FOR
	SELECT
		 SYSPROPID
		,RTRIM(PROPNAME) PROPNAME
		,PROPVALUE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,ROWTS
	FROM
		DB_SYSPROPERTIES ;
END IF;
db_procauditlogi_ah1(v_procname, v_begints, v_paramstxt, p_debugflg, v_exceptionflg);
END;
/


-- DB_SYSPROPERTIESS_AH2.sql

CREATE OR REPLACE PROCEDURE DB_SYSPROPERTIESS_AH2
	(
	p_ReturnCd	OUT	INTEGER
	,p_DebugFlg	IN	PLS_INTEGER
	,RC1		OUT	DB_REF.RCT
	,p_GroupName	IN	VARCHAR2
	) AS

/*
	================================================================================
	Proc Name:       DB_SYSPROPERTIESS_AH2

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIES_AH2';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_PK_Value        Integer Default 0;
 v_SQLErrm         Varchar2(255) ;
 v_chkpt           Char(1) default '0';
 v_msg             Varchar(500);
 v_RowTS           Integer Default 0;
 v_Where           Varchar2 (1000);
 v_ParamsTxt	   varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_Groupname=>'||p_Groupname||CHR(10)
		;

BEGIN
IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_GROUPNAME IS NOT NULL THEN
OPEN RC1 For
	SELECT
		SYSPROPID
		,RTRIM(PROPNAME) PROPNAME
		,PROPVALUE
		,COMPONENT
		,ds.SYSPROPGRPID
		,RTRIM(ds.RESOURCEKEY) RESOURCEKEY
		,ds.ROWTS
	FROM
		DB_SYSPROPERTIES ds, DB_SYSPROPERTIES_GROUP dsg
	WHERE
		ds.SYSPROPGRPID = dsg.SYSPROPGRPID	
		and dsg.groupname = p_GROUPNAME;
		else
OPEN RC1 FOR
	SELECT
			SYSPROPID
			,RTRIM(PROPNAME) PROPNAME
			,PROPVALUE
			,COMPONENT
			,ds.SYSPROPGRPID
			,RTRIM(ds.RESOURCEKEY) RESOURCEKEY
			,ds.ROWTS
		FROM
			DB_SYSPROPERTIES ds, DB_SYSPROPERTIES_GROUP dsg
		WHERE
			ds.SYSPROPGRPID = dsg.SYSPROPGRPID;
END IF;			
	p_ReturnCd := 0;

db_procauditlogi_ah1(v_procname, v_begints, v_paramstxt, p_debugflg, v_exceptionflg);
END;
/


-- DB_SYSPROPERTIESS_AH3.sql

CREATE OR REPLACE PROCEDURE db_syspropertiess_ah3 (
   p_returncd    OUT      INTEGER,
   p_debugflg    IN       PLS_INTEGER,
   rc1           OUT      db_ref.rct,
   p_propname    IN       VARCHAR2,
   p_component   IN       INTEGER
)
AS
/*
   ================================================================================
   Proc Name:       DB_SYSPROPERTIESS_AH3

   ================================================================================
   Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

   ---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
   v_rowcnt          INTEGER         DEFAULT 0;
   v_begints         TIMESTAMP;              -- Performance related variables
   v_procname        VARCHAR2 (30)   DEFAULT 'DB_SYSPROPERTIESS_AH3';
   v_exceptionflg    INTEGER         DEFAULT 0;
   v_exceptioncode   VARCHAR2 (30);
   v_pk_value        INTEGER         DEFAULT 0;
   v_sqlerrm         VARCHAR2 (255);
   v_chkpt           CHAR (1)        DEFAULT '0';
   v_msg             VARCHAR (500);
   v_rowts           INTEGER         DEFAULT 0;
   v_where           VARCHAR2 (1000);
   v_where_or        VARCHAR2 (400)   DEFAULT '';
   v_paramstxt       VARCHAR2 (4000)
      DEFAULT    'p_DebugFlg=>'
              || p_debugflg
              || CHR (10)
              || ', p_PropName=>'
              || p_propname
              || CHR (10)
              || ', p_Component=>'
              || p_component
              || CHR (10);
BEGIN
   v_where :=
         ' Where 1=1 '
      || fn_build_where ('PropName', 'VARCHAR2_LIKE', p_propname);
         -- 0 is common. When we say component id 1 it means return for component id 1 and 0 (as its common to 1 also)
			   -- same for component id 2,3 and 4. They will have the common proerties too.
         -- for component id 2(receiver) and 3(router), properties for component 4(edi) are also required.
   CASE p_component 
       WHEN 0 THEN v_where_or := ' AND COMPONENT = 0';
       WHEN 1 THEN v_where_or := ' AND ((COMPONENT = 1 ) OR (COMPONENT = 0))';
       WHEN 2 THEN v_where_or := ' AND ((COMPONENT = 2 ) OR (COMPONENT = 0) OR (COMPONENT = 4))';
       WHEN 3 THEN v_where_or := ' AND ((COMPONENT = 3 ) OR (COMPONENT = 0) OR (COMPONENT = 4))';
       WHEN 4 THEN v_where_or := ' AND ((COMPONENT = 4 ) OR (COMPONENT = 0))';
	   WHEN null THEN v_where_or := '';
       ELSE
       null;
   END CASE;
   

   IF rc1%ISOPEN
   THEN
      CLOSE rc1;
   END IF;

   OPEN rc1 FOR    'SELECT SYSPROPID'
                || ' ,RTRIM(PROPNAME) PROPNAME ,PROPVALUE	,COMPONENT '
                || ' ,SYSPROPGRPID ,RTRIM(RESOURCEKEY) RESOURCEKEY ,ROWTS '
                || ' FROM	DB_SYSPROPERTIES '
                || v_where
                || v_where_or;

   p_returncd := 0;
   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
END;
/

-- DB_SYSPROPERTIESS_AH4.sql

CREATE OR REPLACE PROCEDURE DB_SYSPROPERTIESS_AH4
	(
	p_ReturnCd	OUT	INTEGER
	,p_DebugFlg	IN	PLS_INTEGER
	,RC1		OUT	DB_REF.RCT
	,p_COMPONENT	IN	NUMBER
	) AS

/*
	================================================================================
	Proc Name:       DB_SYSPROPERTIESS_AH4

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIES_COMPONENT';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_PK_Value        Integer Default 0;
 v_SQLErrm         Varchar2(255) ;
 v_chkpt           Char(1) default '0';
 v_msg             Varchar(500);
 v_RowTS           Integer Default 0;
 v_Where           Varchar2 (1000);
 v_ParamsTxt	   varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_COMPONENT=>'||p_COMPONENT||CHR(10)
		;

BEGIN
IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_COMPONENT IS NOT NULL THEN
OPEN RC1 For
	SELECT
		SYSPROPID
		,RTRIM(PROPNAME) PROPNAME
		,PROPVALUE
		,COMPONENT
		,ds.SYSPROPGRPID
		,RTRIM(ds.RESOURCEKEY) RESOURCEKEY
		,ds.ROWTS
	FROM
		DB_SYSPROPERTIES ds
	WHERE
		ds.COMPONENT = p_COMPONENT
		ORDER BY PROPNAME;
else
OPEN RC1 FOR
	SELECT
			SYSPROPID
			,RTRIM(PROPNAME) PROPNAME
			,PROPVALUE
			,COMPONENT
			,ds.SYSPROPGRPID
			,RTRIM(ds.RESOURCEKEY) RESOURCEKEY
			,ds.ROWTS
		FROM
			DB_SYSPROPERTIES ds
			ORDER BY PROPNAME;
END IF;			
	p_ReturnCd := 0;

db_procauditlogi_ah1(v_procname, v_begints, v_paramstxt, p_debugflg, v_exceptionflg);
END;
/


-- DB_SYSPROPERTIESS_AH5.sql

CREATE OR REPLACE PROCEDURE DB_SYSPROPERTIESS_AH5
	(
	p_ReturnCd	OUT	INTEGER
	,p_DebugFlg	IN	PLS_INTEGER
	,RC1		OUT	DB_REF.RCT
	,p_GroupName	IN	VARCHAR2
	) AS

/*
	================================================================================
	Proc Name:       DB_SYSPROPERTIESS_AH5

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
												::::::::::::::::::::::::::::::
                        		M O D I F I C A T I O N
                        ::::::::::::::::::::::::::::::
       ---------  ------------ --------  ---------------------------------------------
       DATE       NAME         Version   DESCRIPTION
       ---------  ------------ --------  ---------------------------------------------
       2008-09-30 AMIT VIJAY     6.2       Partner Migration
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIES_AH5';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_PK_Value        Integer Default 0;
 v_SQLErrm         Varchar2(255) ;
 v_chkpt           Char(1) default '0';
 v_msg             Varchar(500);
 v_RowTS           Integer Default 0;
 v_Where           Varchar2 (1000);
 v_ParamsTxt	   varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_Groupname=>'||p_Groupname||CHR(10)
		;

BEGIN
IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_GROUPNAME IS NOT NULL THEN
OPEN RC1 For
	SELECT
		SYSPROPID
		,RTRIM(PROPNAME) PROPNAME
		,PROPVALUE
		,COMPONENT
		,ds.SYSPROPGRPID
		,dsg.GROUPNAME
		,RTRIM(ds.RESOURCEKEY) RESOURCEKEY
		,ds.ROWTS
	FROM
		DB_SYSPROPERTIES ds, DB_SYSPROPERTIES_GROUP dsg
	WHERE
		ds.SYSPROPGRPID = dsg.SYSPROPGRPID	
		and dsg.groupname = p_GROUPNAME;
		else
OPEN RC1 FOR
	SELECT
			SYSPROPID
			,RTRIM(PROPNAME) PROPNAME
			,PROPVALUE
			,COMPONENT
			,ds.SYSPROPGRPID
			,dsg.GROUPNAME
			,RTRIM(ds.RESOURCEKEY) RESOURCEKEY
			,ds.ROWTS
		FROM
			DB_SYSPROPERTIES ds 
		left outer join DB_SYSPROPERTIES_GROUP dsg
		on ds.SYSPROPGRPID = dsg.SYSPROPGRPID;
END IF;			
	p_ReturnCd := 0;

db_procauditlogi_ah1(v_procname, v_begints, v_paramstxt, p_debugflg, v_exceptionflg);
END;
/


-- DB_SYSPROPERTIESS_COMPONENT.sql

CREATE OR REPLACE PROCEDURE DB_SYSPROPERTIESS_COMPONENT
	(
	p_ReturnCd	OUT	INTEGER
	,p_DebugFlg	IN	PLS_INTEGER
	,RC1		OUT	DB_REF.RCT
	,p_COMPONENT	IN	NUMBER
	) AS

/*
	================================================================================
	Proc Name:       DB_SYSPROPERTIESS_COMPONENT

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIES_COMPONENT';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_PK_Value        Integer Default 0;
 v_SQLErrm         Varchar2(255) ;
 v_chkpt           Char(1) default '0';
 v_msg             Varchar(500);
 v_RowTS           Integer Default 0;
 v_Where           Varchar2 (1000);
 v_ParamsTxt	   varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_COMPONENT=>'||p_COMPONENT||CHR(10)
		;

BEGIN
IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_COMPONENT IS NOT NULL THEN
OPEN RC1 For
	SELECT
		SYSPROPID
		,RTRIM(PROPNAME) PROPNAME
		,PROPVALUE
		,COMPONENT
		,ds.SYSPROPGRPID
		,RTRIM(ds.RESOURCEKEY) RESOURCEKEY
		,ds.ROWTS
	FROM
		DB_SYSPROPERTIES ds
	WHERE
		ds.COMPONENT = p_COMPONENT;
else
OPEN RC1 FOR
	SELECT
			SYSPROPID
			,RTRIM(PROPNAME) PROPNAME
			,PROPVALUE
			,COMPONENT
			,ds.SYSPROPGRPID
			,RTRIM(ds.RESOURCEKEY) RESOURCEKEY
			,ds.ROWTS
		FROM
			DB_SYSPROPERTIES ds;
END IF;			
	p_ReturnCd := 0;

db_procauditlogi_ah1(v_procname, v_begints, v_paramstxt, p_debugflg, v_exceptionflg);
END;
/


-- DB_SYSPROPERTIESU.sql

CREATE OR REPLACE PROCEDURE DB_SYSPROPERTIESU
	(
	 p_ReturnCd	OUT	integer
	,p_DebugFlg	IN	PLS_Integer
	,RC1		OUT	DB_REF.RCT
	,p_SysPropID	IN	NUMBER
	,p_PropName	IN	VARCHAR2
	,p_PropValue	IN	VARCHAR2
	,p_ResourceKey	IN	VARCHAR2
	,p_ROWTS	IN	NUMBER
) AS
/*
	================================================================================
	Proc Name:       DB_SYSPROPERTIESU
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIESU';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_PK_Value        Integer Default 0;
 v_SQLErrm         Varchar2(255) ;
 v_chkpt           Char(1) default '0';
 v_msg             Varchar2(500);
 v_RowTS           Integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_SysPropID=>'||p_SysPropID||CHR(10)
                ||', p_PropName=>'||p_PropName||CHR(10)
                ||', p_PropValue=>'||p_PropValue||CHR(10)
                ||', p_ResourceKey=>'||p_ResourceKey||CHR(10)
                ||', p_RowTS=>'||p_RowTS||CHR(10)
                ;
ZeroUpdate	EXCEPTION;
BEGIN
	Update DB_SYSPROPERTIES
	SET
		      PROPVALUE=p_PROPVALUE
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	WHERE 
		SYSPROPID = p_SysPropID
		And RowTS = p_RowTS ;

        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;

        p_ReturnCd := 0;

	If RC1%IsOpen Then Close RC1; End If;
	Open RC1 FOR Select 1 From Dual;
    
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
                WHEN ZeroUpdate then
                    p_ReturnCd := 3;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||': No Data Found');
                WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||':'||SQLERRM);
END;
/


-- DB_SYSPROPERTIESU_AH1.sql

CREATE OR REPLACE PROCEDURE DB_SYSPROPERTIESU_AH1
	(
	 p_ReturnCd	OUT	integer
	,p_DebugFlg	IN	PLS_Integer
	,RC1		OUT	DB_REF.RCT
	,p_PropName	IN	VARCHAR2
	,p_PropValue	IN	VARCHAR2
	,p_ROWTS	IN	NUMBER
) AS
/*
	================================================================================
	Proc Name:       DB_SYSPROPERTIESU_AH1
	Create Date:     2007-11-22 17:30:55
	Author:          AMIT KUMAR VIJAY 
		
	            ::::::::::::::::::::::::::::::
	               M O D I F I C A T I O N
	            ::::::::::::::::::::::::::::::
		
	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2007-11-22 AMIT KUMAR	6.1.1	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIESU_AH1';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_PK_Value        Integer Default 0;
 v_SQLErrm         Varchar2(255) ;
 v_chkpt           Char(1) default '0';
 v_propvalue	   DB_SYSPROPERTIES.PROPVALUE%Type DEFAULT NULL;
 v_msg             Varchar2(500);
 v_RowTS           Integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_PropName=>'||p_PropName||CHR(10)
                ||', p_PropValue=>'||p_PropValue||CHR(10)
                ||', p_RowTS=>'||p_RowTS||CHR(10)
                ;
ZeroUpdate	EXCEPTION;
BEGIN
IF p_PropName='bcg.config.ftpmanagement.allowedIPs' THEN 

	Select PropValue into v_PropValue from db_sysproperties where PropName='bcg.config.ftpmanagement.allowedIPs';
	
	IF v_PropValue='*' then 
		Update DB_SYSPROPERTIES
		SET
		  PropValue=p_PropValue
		  ,ROWTS= p_RowTS + 1
		  ,UPDATEDATE= v_BeginTS
	WHERE 
		PropName='bcg.config.ftpmanagement.allowedIPs';
		
	ELSE 
		Update DB_SYSPROPERTIES
		SET
		  PropValue= PropValue || ',' || p_PropValue
		  ,ROWTS= p_RowTS + 1
		  ,UPDATEDATE= v_BeginTS
	WHERE 
		PropName='bcg.config.ftpmanagement.allowedIPs';
		
	END IF;
END IF;

	
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;

        p_ReturnCd := 0;

	If RC1%IsOpen Then Close RC1; End If;
	Open RC1 FOR Select 1 From Dual;
    
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
                WHEN ZeroUpdate then
                    p_ReturnCd := 3;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||': No Data Found');
                WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||':'||SQLERRM);
END;
/


-- DB_SYSPROPERTIESU_HUBINIT.sql

CREATE OR REPLACE PROCEDURE DB_SYSPROPERTIESU_HUBINIT
	(
	 p_ReturnCd	OUT	integer
	,p_DebugFlg	IN	PLS_Integer
	,p_CommonLoc	IN	VARCHAR2
) AS
/*
	================================================================================
	Proc Name:       DB_SYSPROPERTIESU_HUBINIT
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  := FN_GET_DATE;
 v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIESU_HUBINIT';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_PK_Value        Integer Default 0;
 v_SQLErrm         Varchar2(255) ;
 v_chkpt           Char(1) default '0';
 v_msg             Varchar2(500);
 v_RowTS           Integer Default 0;
	v_CommonLoc	db_SysProperties.PropValue%TYPE := p_CommonLoc;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                || 'p_CommonLoc=>'||p_CommonLoc||CHR(10)
                ;
ZeroUpdate	EXCEPTION;
BEGIN
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.install.commonloc';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/receiver/config', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.receiver.configpath';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/receiver/reject', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.vms_receiver_reject_dir';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/receiver/tmp', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.vms_receiver_tmp_dir';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/gateways', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.delivery.gatewayDirectory';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/sync_in', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.delivery.responseDir';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/security/crl/', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.CRLDir';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/security/vtp', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.certs.vtp.CertificateDir';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/gateways', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.console.outbound.gatewayDirectory';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/security/vtp', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.console.certs.vtp.CertificateDir';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/router_in', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.console.vtp.RouterIn';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/router_in/', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.receiver.persistpath';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/sync_in', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.receiver.sync.persistpath';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/signal_in', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.receiver.signal.persistpath';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/router_in', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.vms_inbound_directory.main';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/data', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.bpe_temp_directory.main';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/signal_in', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.vms_inbound_directory.signal';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/data', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.bpe_temp_directory.signal';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/sync_in', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.vms_inbound_directory.synchronous';
	UPDATE	db_sysProperties p
	SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/data', '{SHARED_DATA_DIR}', v_CommonLoc)
		,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.bpe_temp_directory.synchronous';
	-- Added for 106815
	UPDATE	db_sysProperties p
		SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/maps', '{SHARED_DATA_DIR}', v_CommonLoc)
			,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.wtx.mapLocation';
	-- Added for 107358
	UPDATE	db_sysProperties p
		SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/sftp', '{SHARED_DATA_DIR}', v_CommonLoc)
			,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.receiver.common.sftp';
	-- Added for 107401
	UPDATE	db_sysProperties p
		SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/receiver', '{SHARED_DATA_DIR}', v_CommonLoc)
			,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.receiver.common.receiver';
	
	UPDATE	db_sysProperties p
			SET	p.PropValue = REPLACE('{SHARED_DATA_DIR}/dataRestore', '{SHARED_DATA_DIR}', v_CommonLoc)
				,p.UpdateDate = v_BeginTS
	WHERE	p.PROPNAME = 'bcg.archiver.dataRestorePath';
	
	P_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
                WHEN ZeroUpdate then
                    p_ReturnCd := 3;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||': No Data Found');
                WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||':'||SQLERRM);
END;
/


-- db_syspropertiesU_soapAPI.sql

CREATE or Replace PROCEDURE DB_SYSPROPERTIESU_SOAPAPI(P_RETURNCD OUT INTEGER,
P_Propertyvalue IN VARCHAR2, RC1 OUT DB_REF.RCT)
AS
v_BeginTS         TimeStamp := FN_Get_Date ; -- Performance related variables
v_ProcName        varchar2(30)  Default 'DB_SYSPROPERTIESU_SOAPAPI';
v_ExceptionFlg    integer  Default 0;
p_DebugFlg Integer default 1;
v_ExceptionCode   varchar2(30);
v_ParamsTxt       varchar2(4000) ;
v_SQLErrm         Varchar2(255) ;
v_chkpt           Char(1) default '0';
v_RowTS           Integer Default 0;
v_RowCnt          integer Default 0;
V_PROPVALUE VARCHAR2(60);
ZeroUpdate EXCEPTION;

BEGIN

IF UPPER(P_PROPERTYVALUE)='ON' THEN
 UPDATE DB_SYSPROPERTIES
 SET PROPVALUE='true'
 ,ROWTS =ROWTS+1
 ,UPDATEDATE=v_BeginTS
 WHERE
  UPPER(PROPNAME)=UPPER('publicapi.soapBasedApi');
ELSIF UPPER(P_PROPERTYVALUE)='OFF' THEN
 UPDATE DB_SYSPROPERTIES
 SET PROPVALUE='false'
 ,ROWTS=ROWTS+1
 ,UPDATEDATE=v_BeginTS
 WHERE
  UPPER(PROPNAME)=UPPER('publicapi.soapBasedApi');
END IF;
 v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
  p_ReturnCd := 0;
 If RC1%IsOpen Then Close RC1; End If;
 Open RC1 FOR Select 1 From Dual;
 DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
     WHEN ZeroUpdate then
                    p_ReturnCd := 3;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||': No Data Found');
     WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||':'||SQLERRM);
END;
/


-- DB_TABPART_ADD.sql

CREATE OR REPLACE PROCEDURE DB_TABPART_ADD (
	p_returncd		OUT	PLS_INTEGER
	,p_debugflg		IN	PLS_INTEGER
	,rc1			OUT	db_ref.rct
	,p_TableName		IN	VARCHAR2
	,p_RangeEnd		IN	VARCHAR2
	,p_RangeSize		IN	INTEGER
)
AS
/*
	================================================================================
	Proc Name:	DB_TABPART_ADD
	Create Date:	2007-08-17 12:04:11
	Author:		S.Priani
	================================================================================
	Return code = 0	Good, 1 = Something went wrong, 2 = Dup, 3 = No data found
				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::
	---------- ------------ -------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------- ------------ -------	---------------------------------------------
	2007-08-24 S.Priani	6.1.1	CR 104487
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
-- Performance related variables
	v_rowcnt		INTEGER		DEFAULT 0;
	v_begints		TIMESTAMP	DEFAULT fn_get_date;
	v_sqltext		VARCHAR2 (4000);
	v_sqltextstatic		VARCHAR2 (2000);
	v_flag			PLS_INTEGER	DEFAULT 0; --will open RC2
	v_procname		VARCHAR2 (30)	DEFAULT 'DB_TABPART_ADD';
	v_exceptionflg		INTEGER		DEFAULT 0;
	v_rc1			db_ref.rct;
	v_returncd		INTEGER		DEFAULT 0;
	tp_rec			db_tabpart_work%ROWTYPE;
	v_part_tosplit		VARCHAR2(30);
	v_last_partition	VARCHAR2(8);
	v_next_partition	VARCHAR2(8);
	v_stmt			VARCHAR2(200);
	v_paramstxt		VARCHAR2 (4000)	:=	'p_DebugFlg=>' || p_debugflg || CHR (10)
	;
BEGIN
	db_tabpart_list(v_returncd, p_DebugFlg, v_rc1, p_TableName, p_RangeEnd, p_RangeEnd, NULL);
	IF v_returncd = 0 THEN
	BEGIN
		FETCH v_rc1 INTO tp_rec;
		--- If the listed partition is the last one then we can split it to add more partitions
		IF tp_rec.h_vl = 'MAXVALUE' THEN
			v_part_tosplit := tp_rec.p_nm;
			--- get the upper bound for the last partition before MAXVALUE
			db_tabpart_list(v_returncd, p_DebugFlg, v_rc1, p_TableName, Null, Null, tp_rec.P_Id-1);
			FETCH v_rc1 INTO tp_rec;
			v_last_partition := REPLACE(dbms_lob.substr(tp_rec.h_vl,10,1),'''');
			dbms_output.put_line('Adding partitions from ' || v_last_partition || ' every ' || p_RangeSize || ' days');
			v_next_partition := TO_CHAR(TO_DATE(v_last_partition, 'YYYYMMDD') + p_RangeSize, 'YYYYMMDD');
			--- add partitions till the p_RangeEnd value belongs in the last but one partition
			--- atleast one partition may be added
			LOOP
				dbms_output.put_line('Adding partition ending at ' || v_next_partition);
				v_stmt := 'alter table '|| p_TableName 
						||' split partition '|| v_part_tosplit 
						||' at ( '''|| v_next_partition ||''' ) into (partition MSGARCH' || v_next_partition 
						||', partition ' || v_part_tosplit || ')';
				dbms_output.put_line(v_stmt);
				BEGIN
					EXECUTE IMMEDIATE v_stmt;
				EXCEPTION
					WHEN Others THEN
						--- if any error then stop adding partitions, jump out of the loop
						EXIT;
				END;
				IF v_next_partition > p_RangeEnd THEN
					EXIT;
				END IF;
				v_next_partition := TO_CHAR(TO_DATE(v_next_partition, 'YYYYMMDD') + p_RangeSize, 'YYYYMMDD');
			END LOOP;
			--- list the added partitions
			db_tabpart_list(v_returncd, p_DebugFlg, v_rc1, p_TableName, v_last_partition, p_RangeEnd, Null);
			--- OPEN RC1 by associating the result from the procedure call to output parameter
			RC1 := v_rc1; 
		ELSE
			OPEN RC1 FOR SELECT sysdate, 'Cannot add partitions -- Range too low' Message from dual;
		END IF;
	p_returncd := 0;
	EXCEPTION
		WHEN Invalid_Cursor THEN
			dbms_output.put_line('No table');
			RAISE;
	END;
	ELSE
		OPEN RC1 FOR SELECT sysdate TimeStamp, 'Table not found OR is not partitioned' Message from dual;
		p_returncd := 3;
	END IF;
	COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
		p_returncd := 3;
	WHEN OTHERS THEN
		p_returncd := 1;
		db_procauditlogi_ah1 (v_procname,
				v_begints,
				v_paramstxt,
				p_debugflg,
				v_exceptionflg
				);
		raise_application_error (-20999, '20100 - DB_TABPART_ADD:' || SQLERRM);
END DB_TABPART_ADD;
/


-- DB_TABPART_DROP.sql

CREATE OR REPLACE PROCEDURE DB_TABPART_DROP (
	p_returncd		OUT	PLS_INTEGER
	,p_debugflg		IN	PLS_INTEGER
	,rc1			OUT	db_ref.rct
	,p_TableName		IN	VARCHAR2
	,p_RangeBegin		IN	VARCHAR2
	,p_RangeEnd		IN	VARCHAR2
)
AS
/*
	================================================================================
	Proc Name:	DB_TABPART_DROP
	Create Date:	2007-08-17 12:04:11
	Author:		S.Priani
	================================================================================
	Return code = 0	Good, 1 = Something went wrong, 2 = Dup, 3 = No data found
				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::
	---------- ------------ -------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------- ------------ -------	---------------------------------------------
	2007-08-24 S.Priani	6.1.1	CR 104487
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
-- Performance related variables
	v_rowcnt		INTEGER		DEFAULT 0;
	v_begints		TIMESTAMP	DEFAULT fn_get_date;
	v_sqltext		VARCHAR2 (4000);
	v_sqltextstatic		VARCHAR2 (2000);
	v_flag			PLS_INTEGER	DEFAULT 0; --will open RC2
	v_procname		VARCHAR2 (30)	DEFAULT 'DB_TABPART_DROP';
	v_exceptionflg		INTEGER		DEFAULT 0;
	v_rc1			db_ref.rct;
	v_returncd		INTEGER		DEFAULT 0;
	tp_rec			db_tabpart_work%ROWTYPE;
	v_part_todrop		VARCHAR2(30);
	v_next_partition	VARCHAR2(8);
	v_stmt			VARCHAR2(200);
	v_paramstxt		VARCHAR2 (4000)	:=	'p_DebugFlg=>' || p_debugflg || CHR (10)
	;
BEGIN
	db_tabpart_list(v_returncd, p_DebugFlg, v_rc1, p_TableName, p_RangeBegin, p_RangeEnd, NULL);
	IF v_returncd = 0 THEN
		BEGIN
			LOOP
				FETCH v_rc1 INTO tp_rec;
				EXIT WHEN v_rc1%NOTFOUND;
				--- drop all the partitions listed except the one with MAXVALUE
				IF tp_rec.h_vl <> 'MAXVALUE' THEN
					v_part_todrop := tp_rec.p_nm;
					v_next_partition := REPLACE(dbms_lob.substr(tp_rec.h_vl,10,1),'''');
					dbms_output.put_line('Dropping partition ending at ' || v_next_partition);
					v_stmt := 'alter table '|| p_TableName 
								||' drop partition '|| v_part_todrop 
					;
					BEGIN
						EXECUTE IMMEDIATE v_stmt;
						v_rowcnt := v_rowcnt +1;
						dbms_output.put_line(v_stmt || ' ...Success');
					EXCEPTION
						WHEN Others THEN
							--- if any error then stop dropping partitions, jump out of the loop
							dbms_output.put_line(v_stmt || ' ...Failed');
							EXIT;
					END;
				END IF;
			END LOOP;
			IF v_rowcnt <> 0 THEN
				db_tabpart_list(v_returncd, p_DebugFlg, v_rc1, p_TableName, p_RangeBegin, p_RangeEnd, Null);
				--- OPEN RC1 by associating the result from the procedure call to output parameter
				RC1 := v_rc1; 
			ELSE
				OPEN RC1 FOR SELECT sysdate Timestamp, 'Cannot drop partitions -- Range too small' Message from dual;
			END IF;
		EXCEPTION
			WHEN Invalid_Cursor THEN
				dbms_output.put_line('No table');
				RAISE;
		END;
		p_returncd := 0;
	ELSE
		OPEN RC1 FOR SELECT sysdate Timestamp, 'Table does not exist or is not partitioned' Message from dual;
		p_returncd := 3;
	END IF;
	COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
		p_returncd := 3;
	WHEN OTHERS THEN
		p_returncd := 1;
		db_procauditlogi_ah1 (v_procname,
				v_begints,
				v_paramstxt,
				p_debugflg,
				v_exceptionflg
				);
		raise_application_error (-20999, '20100 - DB_TABPART_DROP:' || SQLERRM);
END DB_TABPART_DROP;
/


-- DB_TABPART_LIST.sql

CREATE OR REPLACE PROCEDURE DB_TABPART_LIST (
	p_returncd		OUT	PLS_INTEGER
	,p_debugflg		IN	PLS_INTEGER
	,rc1			OUT	db_ref.rct
	,p_TableName		IN	VARCHAR2
	,p_RangeBegin		IN	VARCHAR2
	,p_RangeEnd		IN	VARCHAR2
	,p_PartitionID		IN	NUMBER
)
AS
/*
	================================================================================
	Proc Name:	DB_TABPART_LIST
	Create Date:	2007-08-17 12:04:11
	Author:		S.Priani
	================================================================================
	Return code = 0	Good, 1 = Something went wrong, 2 = Dup, 3 = No data found
				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::
	---------- ------------ -------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------- ------------ -------	---------------------------------------------
	2007-08-24 S.Priani	6.1.1	CR 104487
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
-- Performance related variables
	v_rowcnt		INTEGER		DEFAULT 0;
	v_begints		TIMESTAMP	DEFAULT fn_get_date;
	v_sqltext		VARCHAR2(4000);
	v_sqltextstatic		VARCHAR2(2000);
	v_flag			PLS_INTEGER	DEFAULT 0; --will open RC2
	v_procname		VARCHAR2(30)	DEFAULT 'DB_TABPART_LIST';
	Unhandled		EXCEPTION;
	v_exceptionflg		INTEGER		DEFAULT 0;
	v_exception_code	INTEGER		DEFAULT 0;
	v_exception_mesg	VARCHAR2(200)	DEFAULT '';
	v_TableName		VARCHAR2(30)	:= UPPER(p_TableName);
	v_paramstxt		VARCHAR2(4000)	:=	'p_DebugFlg=>' || p_debugflg || CHR (10)
	;

BEGIN
	LOCK TABLE DB_TABPART_WORK IN EXCLUSIVE MODE;

	DELETE FROM DB_TABPART_WORK
	;
	-- insert into temp table to convert LONG to LOB
	INSERT INTO DB_TABPART_WORK (P_ID, P_NM, TSPNM, R_CNT, H_VL)
	(SELECT partition_position, partition_name, tablespace_name, num_rows, TO_LOB(high_value)
		FROM	user_tab_partitions
		WHERE	table_name = v_TableName
	);
	IF SQL%ROWCOUNT = 0 THEN
		v_exception_mesg := 'Table does not exist Or Table is not Partitioned';
		Raise No_Data_Found;
	END IF;
	---list subvolume folders upto given Cutoff date
	IF p_PartitionID > 0 THEN  -- get data for given partition ONLY
		OPEN RC1 FOR
			SELECT	p_id partition_position, p_nm partition_name, tspnm tablespace, r_cnt num_rows, h_vl high_value
			FROM	db_tabpart_work
			WHERE	p_id = p_PartitionID;

	ELSIF p_PartitionId = -1 THEN -- get data for all partitions in the table
		OPEN RC1 FOR
			SELECT	p_id partition_position, p_nm partition_name, tspnm tablespace, r_cnt num_rows, h_vl high_value
			FROM	db_tabpart_work;
	ELSE -- range of partitions
		OPEN RC1 FOR
			SELECT	p_id partition_position, p_nm partition_name, tspnm tablespace, r_cnt num_rows, h_vl high_value
			FROM	db_tabpart_work
			WHERE	p_id BETWEEN (
					SELECT	MIN(P_ID)
					FROM	db_TABPART_WORK
					WHERE	REPLACE(dbms_lob.substr(h_vl,10,1),'''') > p_RangeBegin
				)
				AND (
					SELECT	MAX(P_ID)+1
					FROM	db_TABPART_WORK
					WHERE	REPLACE(dbms_lob.substr(h_vl,10,1),'''') < p_RangeEnd
				)
		;
	END IF;
	p_returncd := 0;
	COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
		p_returncd := 3;
		OPEN RC1 FOR
			SELECT sysdate TimeStamp, v_exception_mesg MESSAGE from dual;
	WHEN OTHERS THEN
		p_returncd := 1;
		db_procauditlogi_ah1 (v_procname,
				v_begints,
				v_paramstxt,
				p_debugflg,
				v_exceptionflg
				);
		RAISE_APPLICATION_ERROR (-20999, 'DB_TABPART_LIST:' || SQLERRM);
END DB_TABPART_LIST;
/


-- dis_imp_edi_import_lockD.sql

CREATE OR REPLACE PROCEDURE dis_imp_edi_import_lockD
                           
	(p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN  PLS_INTEGER   
	)
AS                                                                           

/*                                                                             
   This proc deletes from the dis_imp_edi_import_lock table which contains the sessionid of a user using a wizard

	================================================================================
	Proc Name:       DIS_IMP_EDI_IMPORT_LOCKD                                      
	Create Date:     2005-12-05 15:08:27                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-05 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'DIS_IMP_EDI_IMPORT_LOCKD';              
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                               
		;
                                                                            
zeroDelete  EXCEPTION;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	DELETE FROM dis_imp_edi_import_lock                                            
	;                                                                              
	v_RowCnt := SQL%ROWCOUNT;                                                      

   --don't raise the zerodelete exception
	--IF (v_RowCnt = 0) THEN  RAISE ZeroDelete ; END IF;                             

	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);
	p_ReturnCd := 0;                                                               
	EXCEPTION
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt,p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 dis_imp_edi_import_lockD: No Data Found');
		When OTHERS Then                                                              
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);         
End dis_imp_edi_import_lockD;
/


-- dis_imp_edi_import_lockI.sql

CREATE OR REPLACE PROCEDURE DIS_IMP_EDI_IMPORT_LOCKI
	(p_ReturnCd  OUT PLS_INTEGER
	,p_DebugFlg  IN PLS_INTEGER
	,p_SESSIONID IN VARCHAR2
	)
AS                                                                           

/* 
   This proc inserts a value into dis_imp_edi_import_lock which contains the sessionid of a user using a wizard
                                                                            
	================================================================================
	Proc Name:       DIS_IMP_EDI_IMPORT_LOCKI                                      
	Create Date:     2005-12-05 15:08:27                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-05 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             
v_PK_Value        NUMBER := 0;                                                  

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'DIS_IMP_EDI_IMPORT_LOCKI';              
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_SESSIONID=>'||p_SESSIONID||CHR(10)                                     
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	INSERT INTO dis_imp_edi_import_lock(                                           
		SESSIONID                                                                     
	) VALUES (                                                                     
		p_SESSIONID                                                                   
	);                                                                             
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION                                                                      
		When NO_DATA_FOUND THEN                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt,p_DebugFlg, 0);
			p_returnCd := 3;                                                             
		When DUP_VAL_ON_INDEX then                                                    
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;                                                             
		When OTHERS Then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 dis_imp_edi_import_lockI' || SQLCODE||' '||SQLERRM);
END DIS_IMP_EDI_IMPORT_LOCKI;
/


-- dis_imp_edi_import_lockI_ah1.sql

CREATE OR REPLACE PROCEDURE DIS_IMP_EDI_IMPORT_LOCKI_AH1
	(p_ReturnCd  OUT PLS_INTEGER
	,p_DebugFlg  IN PLS_INTEGER
	,p_SESSIONID IN VARCHAR2
	)
AS                                                                           

/* 
   contains the sessionid of a user using a wizard
   This proc takes a sessionid as input
   -if the input sessionid does not have the lock then an exception is thrown
   -if the input sessionid does have the lock the proc returns
   -if no sessionid owns the lock then that sessionid is inserted
                                                                            
	================================================================================
	Proc Name:       DIS_IMP_EDI_IMPORT_LOCKI_AH1                                      
	Create Date:     2005-12-05 15:08:27                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-05 SYSTEM           3.5 Created
                                       
	================================================================================
   
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             
v_PK_Value        NUMBER := 0;                                                  

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'DIS_IMP_EDI_IMPORT_LOCKI_AH1';              
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_SESSIONID=>'||p_SESSIONID||CHR(10)                                     
		;
v_SessionID       VARCHAR2(255);
c_anotherwizard   EXCEPTION;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN

   --not need where clause because the table should only have 1 row
   SELECT sessionid
   INTO v_SessionID
   FROM dis_imp_edi_import_lock;

   IF v_sessionid IS NOT NULL THEN
      --if they don't match then someone else has the lock, but if they do match then don't do anything
      IF v_sessionid <> p_sessionid THEN
         RAISE c_anotherwizard;
      END IF;
   ELSE
      INSERT INTO dis_imp_edi_import_lock (sessionid)
      VALUES (p_sessionid);
   END IF;

	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   EXCEPTION
      WHEN c_anotherwizard THEN
         v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Another user is using the wizard'||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;      
		When NO_DATA_FOUND THEN                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt,p_DebugFlg, 0);
			p_returnCd := 0;
         --now do the insert
         INSERT INTO dis_imp_edi_import_lock (sessionid)
         VALUES (p_sessionid);
		WHEN DUP_VAL_ON_INDEX THEN
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;                                                             
		WHEN OTHERS THEN
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 dis_imp_edi_import_lockI' || SQLCODE||' '||SQLERRM);
END DIS_IMP_EDI_IMPORT_LOCKI_AH1;
/


-- dis_imp_edi_import_lockS.sql

CREATE OR REPLACE PROCEDURE dis_imp_edi_import_lockS                            
	(
	  p_ReturnCd  OUT PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1         OUT DB_REF.RCT                                                                      
	)                                                                              
AS                                                                              

/* 
   This proc gets the sessionid from the DIS_IMP_EDI_IMPORT_LOCK table which contains the sessionid of a user using a wizard
                                                                            
	================================================================================
	Proc Name:       DIS_IMP_EDI_IMPORT_LOCKS                                      
	Create Date:     2005-12-05 15:08:27                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-05 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'DIS_IMP_EDI_IMPORT_LOCKS';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10);
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
   
	OPEN RC1 FOR 
	SELECT                                                          
		RTRIM(SESSIONID) SESSIONID                                                    
	FROM 
		dis_imp_edi_import_lock                                                
   ;
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END dis_imp_edi_import_lockS;
/


-- dis_imp_edi_import_lockU.sql

CREATE OR REPLACE PROCEDURE dis_imp_edi_import_lockU
	(
	  p_ReturnCd  OUT INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1         OUT db_ref.RCT                      
	, p_SESSIONID IN  VARCHAR2                                                       
) AS

/*                                                                             
   This proc updates the value in dis_imp_edi_import_lock which contains the sessionid for a user running a wizard

	================================================================================
	Proc Name:       DIS_IMP_EDI_IMPORT_LOCKU                                      
	Create Date:     2005-12-05 15:08:27                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-05 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'DIS_IMP_EDI_IMPORT_LOCKU';              
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_SESSIONID=>'||p_SESSIONID||CHR(10)                                     
		;
                                                                            
v_RowTS     INTEGER :=0;                                                       
zeroUpdate  EXCEPTION;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	v_RowCnt := 0;                                                                 
	UPDATE dis_imp_edi_import_lock
	SET SESSIONID=p_SESSIONID;  
	
   v_RowCnt := SQL%rowcount;                                                      
	
   IF (v_RowCnt = 0) THEN  RAISE ZeroUpdate ; END IF;
	IF RC1%isOpen THEN CLOSE RC1; END IF;                                          
	OPEN RC1 FOR SELECT v_RowTS RowTS FROM Dual;                                   
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);                                                                
	EXCEPTION
		WHEN TOO_MANY_ROWS THEN                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt,p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - dis_imp_edi_import_lockU: Too Many Rows');
		WHEN ZeroUpdate then                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - dis_imp_edi_import_lockU: No Data Found');
		WHEN OTHERS then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - dis_imp_edi_import_lockU:'||SQLERRM);
END DIS_IMP_EDI_IMPORT_LOCKU;
/


-- dis_imp_edi_syntax_action_lnkS.sql

CREATE OR REPLACE PROCEDURE dis_imp_edi_syntax_action_lnkS                      
	(
	  p_ReturnCd  OUT PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1         OUT DB_REF.RCT                                                                      
	)                                                                              
AS                                                                              

/*                                                                             
   This proc selects actions that should not be shown for each syntax

	================================================================================                                                                               
	Proc Name:       DIS_IMP_EDI_SYNTAX_ACTION_LNKS                                
	Create Date:     2005-12-06 10:16:45                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-06 SYSTEM           3.5 Created
                                       
	================================================================================                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'DIS_IMP_EDI_SYNTAX_ACTION_LNKS';        
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10);
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
   OPEN RC1 FOR 
	SELECT                                                          
		RTRIM(FROMSYNTAX) FROMSYNTAX                                                  
		,RTRIM(TOSYNTAX) TOSYNTAX                                                     
		,ACTIONID                                                                     
	FROM 
		dis_imp_edi_syntax_action_lnk                                          
	;                                                                              
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END dis_imp_edi_syntax_action_lnkS;
/


-- dis_imp_edi_syntax_def_actnS.sql

CREATE OR REPLACE PROCEDURE dis_imp_edi_syntax_def_actnS
	(
	  p_ReturnCd  OUT PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1         OUT DB_REF.RCT                                                                      
	)                                                                              
AS                                                                              

/*                                                                             
   This proc selects from the dis_imp_edi_syntax_def_actn - which contains default actions for a syntax

	================================================================================                                                                               
	Proc Name:       dis_imp_edi_syntax_def_actnS                                
	Create Date:     2005-12-06 10:16:45                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-06 SYSTEM           3.5 Created
                                       
	================================================================================                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'dis_imp_edi_syntax_def_actnS';        
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10);
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
   OPEN RC1 FOR 
	SELECT                                                          
		 rtrim(fromsyntax) fromsyntax
      ,rtrim(tosyntax) tosyntax
		,actionid
	FROM 
		dis_imp_edi_syntax_def_actn
	;                                                                              
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END dis_imp_edi_syntax_def_actnS;
/


-- dis_imp_edi_syntax_def_pkgS.sql

CREATE OR REPLACE PROCEDURE dis_imp_edi_syntax_def_pkgS
	(
	  p_ReturnCd  OUT PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1         OUT DB_REF.RCT                                                                      
	)                                                                              
AS                                                                              

/*                                                                             
   This proc selects the default package for a syntax

	================================================================================                                                                               
	Proc Name:       dis_imp_edi_syntax_def_pkgS                                
	Create Date:     2005-12-06 10:16:45                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-06 SYSTEM           3.5 Created
                                       
	================================================================================                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'dis_imp_edi_syntax_def_pkgS';        
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10);
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
   OPEN RC1 FOR 
	SELECT                                                          
		 syntax syntax
		,pkgid
	FROM 
		dis_imp_edi_syntax_def_pkg
	;                                                                              
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END dis_imp_edi_syntax_def_pkgS;
/


-- dis_imp_edi_syntax_pkg_lnkS.sql

CREATE OR REPLACE PROCEDURE dis_imp_edi_syntax_pkg_lnkS                         
	(
	  p_ReturnCd  OUT PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1         OUT DB_REF.RCT                                                                      
	)                                                                              
AS                                                                              

/*                                                                             
   This proc selects unsupported packages for a syntax

	================================================================================                                                                               
	Proc Name:       DIS_IMP_EDI_SYNTAX_PKG_LNKS                                   
	Create Date:     2005-12-06 13:27:00                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-06 SYSTEM           3.5 Created
                                       
	================================================================================                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'DIS_IMP_EDI_SYNTAX_PKG_LNKS';           
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          

	OPEN RC1 FOR 
	SELECT                                                                         
		RTRIM(SYNTAX) SYNTAX                                                  
     ,PKGID                                                                     
	FROM 
		dis_imp_edi_syntax_pkg_lnk                                          
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt,p_DebugFlg, v_ExceptionFlg);
END dis_imp_edi_syntax_pkg_lnkS;
/


-- edi_standard_package_lnkS.sql

CREATE OR REPLACE PROCEDURE edi_standard_package_lnkS                           
	(
	  p_ReturnCd   OUT PLS_INTEGER
	, p_DebugFlg   IN  PLS_INTEGER
	, RC1          OUT DB_REF.RCT                                                                      
   , p_standardid IN  NUMBER
	)                                                                              
AS                                                                              

/*                                                                             
   This proc gets EDI protocol and doctypes based on an EDI Standard

	================================================================================
	Proc Name:       EDI_STANDARD_PACKAGE_LNKS                                     
	Create Date:     2005-12-09 10:29:54                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-09 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'EDI_STANDARD_PACKAGE_LNKS';             
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
      ||'p_standardid=>'||p_standardid||CHR(10)
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          

	OPEN RC1 FOR 
	SELECT lnk.protocolid,
          prot.routingobjname protocolname,
          prot.routingobjver  protocolversion,
          lnk.doctypeid,
          doc.routingobjname doctypename,
          doc.routingobjver  doctypeversion
	FROM 
		edi_standard_package_lnk lnk,
      cf_routingobject prot,
      cf_routingobject doc
   WHERE lnk.protocolid = prot.routingobjid
      AND lnk.doctypeid = doc.routingobjid
      AND lnk.profiletypeid = p_standardid
	;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt,p_DebugFlg, v_ExceptionFlg);
END edi_standard_package_lnkS;
/


-- eifimporthistoryD.sql

CREATE OR REPLACE PROCEDURE eifimporthistoryD
	(p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN  PLS_INTEGER 
                      
	)
AS                                                                           

/*                                                                             
   This proc deletes data from the EIFImportHistory table

	================================================================================
	Proc Name:       EIFIMPORTHISTORYD                                             
	Create Date:     2005-12-06 11:57:00                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-06 SYSTEM           3.5 Created
                                       
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'EIFIMPORTHISTORYD';                     
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10);
                                                                            
zeroDelete  EXCEPTION;
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	DELETE FROM eifimporthistory                                                   
	;                                                                              
	v_RowCnt := SQL%ROWCOUNT;                                                      

   --do not do anything if there is nothing to delete
	--IF (v_RowCnt = 0) THEN  RAISE ZeroDelete ; END IF;                             
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);
	p_ReturnCd := 0;                                                               
	EXCEPTION                                                                      
		WHEN ZeroDelete THEN                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 eifimporthistoryD: No Data Found');   
		WHEN OTHERS THEN                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);         
END eifimporthistoryD;
/


-- eifimporthistoryS.sql

CREATE OR REPLACE PROCEDURE eifimporthistoryS                                   
	(
	  p_ReturnCd  OUT PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1         OUT DB_REF.RCT                                                                      
   , p_MapType   IN  VARCHAR2
	)                                                                              
AS                                                                              

/*                                                                             
   This proc selects data from the EIFImportHistory table
   1st RC1 is for validation maps
   2nd RC1 is for non-validation maps

	================================================================================                                                                               
	Proc Name:       EIFIMPORTHISTORYS                                             
	Create Date:     2005-12-06 11:57:01                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-06 SYSTEM           3.5 Created
                                       
	================================================================================                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) DEFAULT 'EIFIMPORTHISTORYS';                     
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10);
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          

   --put p_maptype in where instead of building w/ the where fxn because there will always be a value for 
   --maptype being passed in
   IF p_maptype = 'V' THEN
	   OPEN RC1 FOR 
         SELECT                                                                         
         RTRIM(frPkgName)     FrPkgName,
         frpkg.routingobjid   FrPkgID,
         RTRIM(frPkgVer)      FrPkgVer,
         RTRIM(frProtName)    FrProtName,
         frprot.routingobjid  FrProtID,
         'ALL'                FrProtVer,
         RTRIM(frDocTypeName) FrDocTypeName,
         frdoc.routingobjid   FrDocID,
         'ALL'                FrDocTypeVer,
         hist.mapid           MapID,
         RTRIM(MapName)       MapName,
         RTRIM(MapType)       MapType,
         RTRIM(FrSyntax)      FrSyntax,
         RTRIM(ToSyntax)      ToSyntax
   	FROM 
	   	EIFImportHistory hist,
         cf_routingobject frpkg,
         cf_routingobject frprot,
         cf_routingobject frdoc
      WHERE rtrim(hist.frpkgname)    = frpkg.RoutingObjCd
      AND rtrim(hist.frprotname)     = frprot.RoutingObjCd
      AND rtrim(hist.frdoctypename)  = frdoc.RoutingObjCd
      AND rtrim(hist.maptype) = p_maptype
      ORDER BY frpkgname, frprotname, frdoctypename
   ;
	ELSE                                                                           
	   OPEN RC1 FOR 
         SELECT                                                                         
         RTRIM(ToPkgName)     ToPkgName,
         topkg.routingobjid   ToPkgID,
         RTRIM(ToPkgVer)      ToPkgVer,
         RTRIM(ToProtName)    ToProtName,
         toprot.routingobjid  ToProtID,
         'ALL'                ToProtVer,
         RTRIM(ToDocTypeName) ToDocTypeName,
         todoc.routingobjid   ToDocID,
         'ALL'                ToDocTypeVer,
         RTRIM(FrPkgName)     FrPkgName,
         frpkg.routingobjid   FrPkgID,
         RTRIM(FrPkgVer)      FrPkgVer,
         RTRIM(FrProtName)    FrProtName,
         frprot.routingobjid  FrProtID,
         'ALL'                FrProtVer,
         RTRIM(FrDocTypeName) FrDocTypeName,
         frdoc.routingobjid   FrDocID,
         'ALL'                FrDocTypeVer,
         hist.mapid           MapID,
         RTRIM(MapName)       MapName,
         RTRIM(MapType)       MapType,
         RTRIM(FrSyntax)      FrSyntax,
         RTRIM(ToSyntax)      ToSyntax
   	FROM 
	   	EIFImportHistory hist,
         cf_routingobject topkg,
         cf_routingobject toprot,
         cf_routingobject todoc,
         cf_routingobject frpkg,
         cf_routingobject frprot,
         cf_routingobject frdoc
      WHERE rtrim(hist.topkgname)    = topkg.RoutingObjCd
      AND rtrim(hist.toprotname)     = toprot.RoutingObjCd
      AND rtrim(hist.todoctypename)  = todoc.RoutingObjCd
      AND rtrim(hist.frpkgname)      = frpkg.RoutingObjCd
      AND rtrim(hist.frprotname)     = frprot.RoutingObjCd
      AND rtrim(hist.frdoctypename)  = frdoc.RoutingObjCd
      AND rtrim(hist.maptype) = p_maptype
      ORDER BY frpkgname, frprotname, frdoctypename
   ;
	
	END IF;
      
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt,p_DebugFlg, v_ExceptionFlg);
END eifimporthistoryS;
/


-- LG_ACCESS_CTLM.sql

CREATE OR REPLACE PROCEDURE LG_Access_CTLM
    (p_ReturnCd     out integer
    ,p_DebugFlg     in pls_integer
    ,p_HOSTNAME     in VARCHAR2
    ,p_LOGTYPECD    in VARCHAR2
    ,p_FILENAME     in VARCHAR2
    ,p_LOGLINENBR   in pls_integer
    ,p_RCVDOCTS     in TIMESTAMP
    )
AS

/*
    ================================================================================
    Proc Name:       LG_Access_CTLM
    Create Date:     2003-01-31
    Author:          V Goel

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt    integer     :=0;
v_BeginTS   timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default ' LG_Access_CTLM';
v_ExceptionFlg    integer Default 0;
v_PK_Value  number := 0;
v_RowTS     integer :=0;
zeroUpdate  Exception;

Procedure ProcAudit is
Begin
    DB_ProcAuditLogI_ah1('LG_Access_CTLM',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
        ||','||'RC1'
        ||','||p_HOSTNAME
        ||','||p_LOGTYPECD
        ||','||p_FILENAME
        ||','||p_LOGLINENBR
        ||','||p_RCVDOCTS
        );
End ProcAudit;

Procedure DoUpdate is
Begin
   Update LG_Access_CTL AC
       Set
          ac.HostName =  p_HostName
        , ac.LogTypeCd = p_LogTypeCd
        , ac.FileName = p_FileName
        , ac.LogLineNbr = p_LogLineNbr
        , ac.RcvDocTs = p_RcvDocTs
        , ac.UpdateDate = v_BeginTS
      Where ac.HostName = p_HostName  and ac.LogTypeCd = p_LogTypeCd;
    v_RowCnt := SQL%rowcount;
    if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
    p_ReturnCd := 0;
    ProcAudit;
    EXCEPTION
            WHEN ZeroUpdate then
                p_ReturnCd := 3;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    Raise_application_error (-20999,'20001 LG_Access_CTLM '||SQLERRM);
End DoUpdate;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
 v_RowCnt := 0;
 p_ReturnCd := 0;
 DoUpdate;
  if (v_RowCnt = 0) then
     Insert into LG_Access_CTL
           (
              HOSTNAME
           ,  LOGTYPECD
           ,  FILENAME
           ,  LOGLINENBR
           ,  RCVDOCTS
           ,  CREATEDATE
           ,  UPDATEDATE
           )
              Values
           (
             p_HostName
           , p_LogTypeCd
           , p_FileName
           , p_LogLineNbr
           , p_RcvDocTS
           , v_BeginTS
           , v_BeginTS
            );
    end if;

 p_ReturnCd :=0;
 ProcAudit;
     EXCEPTION
              WHEN Dup_val_on_index THEN
                 DoUpdate;
              When Others Then
                   p_ReturnCd := 1;
                   ProcAudit;
                   Raise_application_error (-20999,'20001 LG_Access_CTLM '||SQLERRM);

END LG_Access_CTLM;
/


-- LG_ACKI.SQL

Create or Replace Procedure LG_ACKI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ORIGINALACTIVITYID in VARCHAR2
	,p_ACKACTIVITYID in VARCHAR2
	,p_STATUSCODE in VARCHAR2
	,p_STATUSTEXT in VARCHAR2
   ,p_RejectedFlg IN PLS_INTEGER
	)
AS

/*
	================================================================================
	Proc Name:       LG_ACKI
	Create Date:     2004-09-02 12:32:22
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_ACKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ORIGINALACTIVITYID=>'||p_ORIGINALACTIVITYID||CHR(10)
		||', p_ACKACTIVITYID=>'||p_ACKACTIVITYID||CHR(10)
		||', p_STATUSCODE=>'||p_STATUSCODE||CHR(10)
		||', p_STATUSTEXT=>'||p_STATUSTEXT||CHR(10)
      ||', p_rejectedFlg=>'||p_rejectedFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_ACK(
		ORIGINALACTIVITYID
		,ACKACTIVITYID
		,STATUSCODE
		,STATUSTEXT
      ,rejectedflg
	) Values (
		p_ORIGINALACTIVITYID
		,p_ACKACTIVITYID
		,p_STATUSCODE
		,p_STATUSTEXT
      ,p_rejectedflg
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_ACKI' || SQLCODE|| ' '||SQLERRM);
END LG_ACKI;
/


-- LG_ACKS.SQL

CREATE OR REPLACE PROCEDURE LG_ACKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ORIGINALACTIVITYID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_ACKS
	Create Date:     2004-09-02 12:34:36
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_ACKS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ORIGINALACTIVITYID=>'||p_ORIGINALACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	IF p_ORIGINALACTIVITYID IS NOT NULL THEN
	OPEN RC1 FOR
	SELECT
		RTRIM(ORIGINALACTIVITYID) ORIGINALACTIVITYID
		,RTRIM(ACKACTIVITYID) ACKACTIVITYID
		,ROWTS
		,CREATEDATE
		,RTRIM(STATUSCODE) STATUSCODE
		,RTRIM(STATUSTEXT) STATUSTEXT
      ,RejectedFlg rejectedflg
	FROM
		LG_ACK
	Where
		ORIGINALACTIVITYID = p_ORIGINALACTIVITYID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		RTRIM(ORIGINALACTIVITYID) ORIGINALACTIVITYID
		,RTRIM(ACKACTIVITYID) ACKACTIVITYID
		,ROWTS
		,CREATEDATE
		,RTRIM(STATUSCODE) STATUSCODE
		,RTRIM(STATUSTEXT) STATUSTEXT
      ,RejectedFlg rejectedflg
	FROM
		LG_ACK
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_ACKS;
/


-- LG_ACKS_AH1.SQL

CREATE OR REPLACE PROCEDURE LG_ACKS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ACKACTIVITYID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_ACKS_AH1
	Create Date:     2004-09-02 12:34:36
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_ACKS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ACKACTIVITYID=>'||p_ACKACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	IF p_ACKACTIVITYID IS NOT NULL THEN
	OPEN RC1 FOR
	SELECT
		RTRIM(ORIGINALACTIVITYID) ORIGINALACTIVITYID
		,RTRIM(ACKACTIVITYID) ACKACTIVITYID
		,ROWTS
		,CREATEDATE
		,RTRIM(STATUSCODE) STATUSCODE
		,RTRIM(STATUSTEXT) STATUSTEXT
	FROM
		LG_ACK
	Where
		ACKACTIVITYID = p_ACKACTIVITYID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		RTRIM(ORIGINALACTIVITYID) ORIGINALACTIVITYID
		,RTRIM(ACKACTIVITYID) ACKACTIVITYID
		,ROWTS
		,CREATEDATE
		,RTRIM(STATUSCODE) STATUSCODE
		,RTRIM(STATUSTEXT) STATUSTEXT
	FROM
		LG_ACK
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_ACKS_AH1;
/


-- LG_ACTIVITYI_RECEIVE.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITYi_receive (
p_returnCD            in out integer
, p_debugFlg          in pls_integer
, p_ACTIVITYID        in VARCHAR2
, p_RCVDOCTS          in Timestamp
, p_FRIPADDRESS       in out VARCHAR2
,P_PROVSESSIONID      in pls_integer

) as
/*
================================================================================
Proc Name:      LG_ACTIVITYi_receive
Create Date:    2003-01-15
Author          V Goel
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
v_returnCD pls_integer;

-- for length check
    v_ACTIVITYID        LG_ACTIVITY.ACTIVITYID%Type := p_ACTIVITYID;
--
   v_INITBUSINESSIDHDR    Lg_Activity.INITBUSINESSIDHDR%Type;
   v_FRBUSINESSIDHDR      Lg_Activity.FRBUSINESSIDHDR%Type;
   v_TOBUSINESSIDHDR      Lg_Activity.TOBUSINESSIDHDR%Type;
   v_INITBUSINESSIDBDY    Lg_Activity.INITBUSINESSIDBDY%Type;
   v_FRBUSINESSIDBDY      Lg_Activity.FRBUSINESSIDBDY%Type;
   v_TOBUSINESSIDBDY      Lg_Activity.TOBUSINESSIDBDY%Type;
   v_FRPROTOCOLNAME       Lg_Activity.FRPROTOCOLNAME%Type;
   v_TOPROTOCOLNAME       Lg_Activity.TOPROTOCOLNAME%Type;
   v_FRPROTOCOLVER        Lg_Activity.FRPROTOCOLVER%Type;
   v_TOPROTOCOLVER        Lg_Activity.TOPROTOCOLVER%Type;
   v_FRPROCESSCD          Lg_Activity.FRPROCESSCD%Type;
   v_TOPROCESSCD          Lg_Activity.TOPROCESSCD%Type;
   v_FRPROCESSVER         Lg_Activity.FRPROCESSVER%Type;
   v_TOPROCESSVER         Lg_Activity.TOPROCESSVER%Type;
   v_DOCID                Lg_Activity.DOCID%Type;
   v_DOCTS                Lg_Activity.DOCTS%Type;
   v_FRIPADDRESS          LG_ACTIVITY.FRIPADDRESS%Type;
--
-- ProcAudit
--
Procedure ProcAudit is
begin
DB_ProcAuditLogI_Ah1('LG_ACTIVITYi_receive',v_beginTS,
p_ACTIVITYID||','||
p_RCVDOCTS||','||p_FRIPADDRESS||','||p_debugFlg
);
End ProcAudit;


Procedure Check_length is
begin
--dbms_output.put_line ('Check length next');
   db_LengthCheck_ACTIVITY (
     P_INITBUSINESSIDHDR   => v_INITBUSINESSIDHDR
   , P_FRBUSINESSIDHDR     => v_FRBUSINESSIDHDR
   , P_TOBUSINESSIDHDR     => v_TOBUSINESSIDHDR
   , P_INITBUSINESSIDBDY   => v_INITBUSINESSIDBDY
   , P_FRBUSINESSIDBDY     => v_FRBUSINESSIDBDY
   , P_TOBUSINESSIDBDY     => v_TOBUSINESSIDBDY
   , P_FRPROTOCOLNAME      => v_FRPROTOCOLNAME
   , P_TOPROTOCOLNAME      => v_TOPROTOCOLNAME
   , P_FRPROTOCOLVER       => v_FRPROTOCOLVER
   , P_TOPROTOCOLVER       => v_TOPROTOCOLVER
   , P_FRPROCESSCD         => v_FRPROCESSCD
   , P_TOPROCESSCD         => v_TOPROCESSCD
   , P_FRPROCESSVER        => v_FRPROCESSVER
   , P_TOPROCESSVER        => v_TOPROCESSVER
   , P_DOCID               => v_DOCID
   , P_DOCTS               => v_DOCTS
   , P_FRIPADDRESS         => P_FRIPADDRESS
   ) ;
--   dbms_output.put_line ('Check length done');
   v_FRIPADDRESS := p_FRIPADDRESS;
Exception
   When Others Then
       Raise_application_error (-20999,'20001 LG_ACTIVITYi_receive-- Check Length '||SQLERRM);
end check_length;
--
-- Main
--
BEGIN

Check_length;
Begin
    Insert into LG_ACTIVITY(
        ACTIVITYID
    ,   RCVDOCTS
    ,   FRIPADDRESS
    ,   PROVSESSIONID
    ,   DESTTYPEID
    ) Values (
        v_ACTIVITYID
    ,   p_RCVDOCTS         -- date
    ,   v_FRIPADDRESS
    ,   nvl(P_PROVSESSIONID, -1)
    ,   FN_GET_DESTTYPEID('Unknown')
    );
    EXCEPTION
        When Dup_Val_On_Index Then
            p_returnCD := 2;
        When Others Then
            ProcAudit;
            p_returnCD := 1;
            Raise_application_error (-20999,'20002 LG_ACTIVITYi_receive' || SQLCODE|| ' '||SQLERRM);

END;
--
-- Run summary proc
--
--LG_DOCPROCESSING_SUMM(
--P_RETURNCD         => v_returnCD
--,P_DEBUGFLG        => p_debugFlg
--,P_RCVDOCTS        => p_RCVDOCTS
--,P_FRIPADDRESS     => v_FRIPADDRESS
--,P_RCVDOCCNT       => 1
--,P_PERSISTDOCCNT   => 1
--,P_PROCESSSTARTCNT => 0
--,P_PROCESSENDCNT   => 0
--,P_PROVSESSIONID   => P_PROVSESSIONID
--,P_SUCCESSCNT      => 0
--,P_FAILCNT         => 0
--);

BEGIN
INSERT INTO LG_UNPROCESSED_DOCPROC
	(RCVDOCTS
	,FRIPADDRESS
	,PROVSESSIONID
	,RCVDOCCNT
	,PERSISTDOCCNT
	,PROCESSSTARTCNT
	,PROCESSENDCNT
	,SUCCESSCNT
	,FAILCNT
	)
VALUES
	(P_RCVDOCTS
	,P_FRIPADDRESS
	,P_PROVSESSIONID
	,1     -- rcvdoccnt
	,1     -- persistdoccnt
	,0     -- processstartcnt
	,0     -- processendcnt
	,0     -- successcnt
	,0);   -- FAILCNT
EXCEPTION
	WHEN Others THEN
		p_ReturnCd := 1;
		ProcAudit;
		Raise_application_error (-20999,'20000 LG_ACTIVITY_ENDSTATEI_AH1 - Ins Unprocessed ' || SQLCODE|| ' '||SQLERRM);
END;

ProcAudit;
EXCEPTION
When NO_DATA_FOUND Then Null;
When Others Then
ProcAudit;
p_returnCD := 1;
Raise_application_error (-20999,'20000 LG_ACTIVITYi_receive' || SQLCODE|| ' '||SQLERRM);


END LG_ACTIVITYi_receive;
/


-- LG_ACTIVITYI_RECEIVE_AH1.sql


CREATE OR REPLACE PROCEDURE LG_ACTIVITYI_RECEIVE_AH1
(
    p_returnCD		OUT			pls_integer
    ,p_debugFlg 	IN			pls_integer
    ,RC1		out			db_ref.RCT
    ,p_ACTIVITYID	IN			VARCHAR2
    ,p_RCVDOCTS		IN			Timestamp
    ,p_FRIPADDRESS	IN			VARCHAR2
    ,P_PROVSESSIONID	IN			NUMBER
    ,P_FTPUSERID	IN			NUMBER
    ,P_ORIGINALFILENAME	IN			VARCHAR2
    ,P_LASTMODIFIEDTIME	IN			VARCHAR2

)
AS

/*
================================================================================
Proc Name:      LG_ACTIVITYI_RECEIVE_AH1
Create Date:    2007-09-15
Author          AMIT KUMAR VIJAY
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_RowCnt          integer :=0;
  v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
  v_ProcName        varchar2(30) := 'LG_ACTIVITYI_RECEIVE_AH1';
  v_ExceptionFlg    integer := 0;
  v_ParamsTxt       varchar2(4000) :=
         'p_DebugFlg=>'||p_DebugFlg||CHR(10)
         ||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
         ||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
         ||', p_FRIPADDRESS=>'||p_FRIPADDRESS||CHR(10)
         ||', P_PROVSESSIONID=>'||P_PROVSESSIONID||CHR(10)
         ||', P_FTPUSERID=>'||P_FTPUSERID||CHR(10)
         ||', P_ORIGINALFILENAME=>'||P_ORIGINALFILENAME||CHR(10)
         ||', P_LASTMODIFIEDTIME=>'||P_LASTMODIFIEDTIME||CHR(10)
         ;
  v_RowTS        		integer :=0;
  
   v_pk_value integer DEFAULT 0;
   v_returnCD integer;
   
-- for length check
    v_ACTIVITYID        LG_ACTIVITY.ACTIVITYID%Type := p_ACTIVITYID;
--
   v_INITBUSINESSIDHDR    Lg_Activity.INITBUSINESSIDHDR%Type;
      v_FRBUSINESSIDHDR      Lg_Activity.FRBUSINESSIDHDR%Type;
      v_TOBUSINESSIDHDR      Lg_Activity.TOBUSINESSIDHDR%Type;
      v_INITBUSINESSIDBDY    Lg_Activity.INITBUSINESSIDBDY%Type;
      v_FRBUSINESSIDBDY      Lg_Activity.FRBUSINESSIDBDY%Type;
      v_TOBUSINESSIDBDY      Lg_Activity.TOBUSINESSIDBDY%Type;
      v_FRPROTOCOLNAME       Lg_Activity.FRPROTOCOLNAME%Type;
      v_TOPROTOCOLNAME       Lg_Activity.TOPROTOCOLNAME%Type;
      v_FRPROTOCOLVER        Lg_Activity.FRPROTOCOLVER%Type;
      v_TOPROTOCOLVER        Lg_Activity.TOPROTOCOLVER%Type;
      v_FRPROCESSCD          Lg_Activity.FRPROCESSCD%Type;
      v_TOPROCESSCD          Lg_Activity.TOPROCESSCD%Type;
      v_FRPROCESSVER         Lg_Activity.FRPROCESSVER%Type;
      v_TOPROCESSVER         Lg_Activity.TOPROCESSVER%Type;
      v_DOCID                Lg_Activity.DOCID%Type;
      v_DOCTS                Lg_Activity.DOCTS%Type;
      v_FRIPADDRESS          LG_ACTIVITY.FRIPADDRESS%Type;
   
      v_FTPEVENTID	     LG_EVENT_FTP.FTPEVENTID%TYPE;
   
--Procedure Check_length is
--begin
--dbms_output.put_line ('Check length next');
--   db_LengthCheck_ACTIVITY (
--     P_INITBUSINESSIDHDR   => v_INITBUSINESSIDHDR
--   , P_FRBUSINESSIDHDR     => v_FRBUSINESSIDHDR
--   , P_TOBUSINESSIDHDR     => v_TOBUSINESSIDHDR
--   , P_INITBUSINESSIDBDY   => v_INITBUSINESSIDBDY
--   , P_FRBUSINESSIDBDY     => v_FRBUSINESSIDBDY
--   , P_TOBUSINESSIDBDY     => v_TOBUSINESSIDBDY
--   , P_FRPROTOCOLNAME      => v_FRPROTOCOLNAME
--   , P_TOPROTOCOLNAME      => v_TOPROTOCOLNAME
--   , P_FRPROTOCOLVER       => v_FRPROTOCOLVER
--   , P_TOPROTOCOLVER       => v_TOPROTOCOLVER
--   , P_FRPROCESSCD         => v_FRPROCESSCD
--   , P_TOPROCESSCD         => v_TOPROCESSCD
--   , P_FRPROCESSVER        => v_FRPROCESSVER
--   , P_TOPROCESSVER        => v_TOPROCESSVER
--   , P_DOCID               => v_DOCID
--   , P_DOCTS               => v_DOCTS
--   , P_FRIPADDRESS         => P_FRIPADDRESS
--   ) ;
--   dbms_output.put_line ('Check length done');
--   v_FRIPADDRESS := p_FRIPADDRESS;
--Exception
--   When Others Then
--       Raise_application_error (-20999,'20001 LG_ACTIVITYi_receive_ah1-- Check Length '||SQLERRM);
--end check_length;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	
--
-- Main
--

--Check_length;
Begin
    Insert into LG_ACTIVITY(
        ACTIVITYID
    ,   RCVDOCTS
    ,   FRIPADDRESS
    ,   PROVSESSIONID
    ,   DESTTYPEID
    ) Values (
        v_ACTIVITYID
    ,   p_RCVDOCTS         -- date
    ,   v_FRIPADDRESS
    ,   nvl(P_PROVSESSIONID, -1)
    ,   FN_GET_DESTTYPEID('Unknown')
    );
    EXCEPTION
        When Dup_Val_On_Index Then
            p_returnCD := 2;
        When Others Then
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            p_returnCD := 1;
            Raise_application_error (-20999,'20002 LG_ACTIVITYi_receive_ah1' || SQLCODE|| ' '||SQLERRM);

END;

BEGIN
INSERT INTO LG_UNPROCESSED_DOCPROC
	(RCVDOCTS
	,FRIPADDRESS
	,PROVSESSIONID
	,RCVDOCCNT
	,PERSISTDOCCNT
	,PROCESSSTARTCNT
	,PROCESSENDCNT
	,SUCCESSCNT
	,FAILCNT
	)
VALUES
	(P_RCVDOCTS
	,P_FRIPADDRESS
	,P_PROVSESSIONID
	,1     -- rcvdoccnt
	,1     -- persistdoccnt
	,0     -- processstartcnt
	,0     -- processendcnt
	,0     -- successcnt
	,0);   -- FAILCNT
EXCEPTION
	WHEN Others THEN
		p_ReturnCd := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		Raise_application_error (-20999,'20000 LG_ACTIVITY_ENDSTATEI_AH1 - Ins Unprocessed ' || SQLCODE|| ' '||SQLERRM);
END;

IF P_FTPUSERID IS NOT NULL AND P_ORIGINALFILENAME IS NOT NULL AND P_LASTMODIFIEDTIME IS NOT NULL THEN 
	BEGIN
	   		
	   				Select L.FTPEVENTID INTO v_FTPEVENTID FROM LG_EVENT_FTP L 
	    				WHERE L.ORIGINALFILENAME=p_ORIGINALFILENAME
	    				AND L.FTPUSERID=P_FTPUSERID
					AND L.LASTMODIFIEDTIME=p_LASTMODIFIEDTIME;
					
					EXCEPTION
					When NO_DATA_FOUND THEN
						v_FTPEVENTID := NULL;
	END;
	
	
		        	
			
	
	
	IF v_FTPEVENTID IS NULL
	THEN 	
		Begin
        			INSERT INTO LG_EVENT_FTP (
        					FTPEVENTID
        					,FTPUSERID
        					,ORIGINALFILENAME
        					,LASTMODIFIEDTIME
        					,VUID
        					,UPDATEDATE
        					,CREATEDATE
        					)
        				    VALUES (
						LG_EVENT_FTP_SEQ.nextval
						,p_FTPUSERID
						,p_ORIGINALFILENAME
						,p_LASTMODIFIEDTIME
						,p_ACTIVITYID
						,v_BeginTS
						,v_BeginTS
						); 
						
				EXCEPTION
				        When Dup_Val_On_Index Then
				            p_returnCD := 2;
				        When Others Then
				            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
				            p_returnCD := 1;
				            Raise_application_error (-20999,'20002 LG_ACTIVITYi_receive_ah1' || SQLCODE|| ' '||SQLERRM);
				
		END;
        ELSE
					
				UPDATE LG_EVENT_FTP
				  SET 
					VUID=p_ACTIVITYID 
					WHERE P_FTPUSERID=P_FTPUSERID
					AND ORIGINALFILENAME=P_ORIGINALFILENAME
					AND LASTMODIFIEDTIME=P_LASTMODIFIEDTIME;
					
	END IF;
	
END IF;
	
	EXCEPTION
				
				WHEN OTHERS then
					ROLLBACK work;
					p_ReturnCd := 1;
					v_ExceptionFlg := 1;
					DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
					Commit;
					RAISE_APPLICATION_ERROR(-20999,'20100 - LG_ACTIVITYI_RECEIVE_AH1:'||SQLERRM);
	
	    	    
	    	    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		
END LG_ACTIVITYI_RECEIVE_AH1;
/	




-- LG_ACTIVITYS_ACTIVITYID.sql

CREATE OR REPLACE PROCEDURE LG_ActivityS_ActivityId (
      p_ReturnCd    out  pls_INTEGER
    , p_DebugFlg    in      pls_INTEGER
     , RC1          out  db_ref.rct          -- rows
     , p_Locale     in      Varchar2
    , p_ActivityId  in      varchar2
    )
AS
/*
================================================================================
    Proc Name:      LG_SUMMARYS_activityId
    Create Date:    01/04/2002
    Author          Vipin Goel
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------    ---------------------------------------------
	DATE       NAME          Ver       DESCRIPTION
	---------- ------------- ------    ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2     Removed NLS
    	2006-02-20 S.Nair	 6.1.0     56000
    	2006-06-09 jriordan      610       101012 added package name and ver
    	2008-01-01 BHARANI J     611       102199 
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_BeginTS          timestamp(6):= FN_Get_Date;
    v_ProcName          varchar2(30) := 'LG_ActivityS_Vuid2';
    v_ExceptionFlg      integer := 0;
    v_ParamsTxt         varchar2(4000) :=
                                'p_ReturnCd'||','||p_DebugFlg
                                ||','||p_Locale
                                ||','||'RC1'
                                ||','||P_ActivityId;
    Parent_hdrflg      pls_integer := 1 ; -- lg_activity_dtl.HDRFLG = 1
    Child_hdrflg       pls_integer := 0 ; -- lg_activity_dtl.HDRFLG = 0
--
-- Main
--
BEGIN
    p_ReturnCd := 0;
    if RC1%IsOpen then close RC1; end if;
    open RC1 for
    SELECT
        frp.partnername FRPARTNERNAME,
        a.FRPARTNERIDHdr FRPARTNERID,
        a.FRBUSINESSIDHDR FRBUSINESSID,
        Top.partnername TOPARTNERNAME,
        a.TOPARTNERIDHdr TOPARTNERID,
        a.ToBUSINESSIDHdr ToBUSINESSID ,
        FRPACKAGENAME,
	FRPACKAGEVER,
	TOPACKAGENAME,
        TOPACKAGEVER,
        FRPROTOCOLNAME,
        FRPROTOCOLVer,
        TOPROTOCOLNAME,
        TOPROTOCOLVer,
        --frproc.RoutingObjName FrPROCESSName,
        RTRIM(FN_GET_RTOBJNAME(a.FrProcessCd, a.FrProcessVer, 'Process')) FrProcessName,
        a.FRPROCESSVer,
        a.FrPROCESSCD,
        --Toproc.RoutingObjName TOPROCESSName,
        RTRIM(FN_GET_RTOBJNAME(a.ToProcessCd, a.ToProcessVer, 'Process')) ToProcessName,
        a.TOPROCESSVer,
        a.ToPROCESSCD,
        a.DocId,
        a.DocTS,
        a.ActivityId,
        a.RcvDocTS,
        a.CHANNELSTATUSID,
        fradtl.vuid frVuid,
        toadtl.vuid ToVuid,
        fradtl.MSGRAWLENGTH  FrMSGRAWLENGTH,
        fradtl.MSGCONTENTLENGTH FrMSGCONTENTLENGTH,
        Toadtl.MSGRAWLENGTH  ToMSGRAWLENGTH,
        Toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH,
               fradtl.ORIGINALFILENAME frFILENAME,
               toadtl.ORIGINALFILENAME toFILENAME,
               fradtl.ENCODING frENCODING,
               toadtl.ENCODING toENCODING,
               fradtl.MessageStoreRequired frmesgstorereqd,
               toadtl.MessageStorerequired tomesgstorereqd,
        ae.EndStateTS,
        ae.EndStateFlg,
        wf.Resourcekey WfResourcekey,
        wf.Description WfDescription,
        t.Resourcekey Resourcekey,
        t.Description Description,
        -- 56000 New fields added to the select list.
	a.USER01,
	a.USER02,
	a.USER03,
	a.USER04,
	a.USER05,
	a.USER06,
	a.USER07,
	a.USER08,
	a.USER09,
	a.USER10,
	bpd.key01,
	bpd.key02,
	bpd.key03,
	bpd.key04,
	bpd.key05,
	bpd.key06,
	bpd.key07,
	bpd.key08,
	bpd.key09,
	bpd.key10,
	lsrr.VUID_RESP
    FROM
       LG_Activity a
       Left Outer Join lg_activity_dtl_parent_vw fradtl    ON a.activityId = fradtl.activityId
       Left Outer Join lg_activity_dtl_child_vw toadtl ON a.activityId = Toadtl.activityId
       Left Outer Join lg_activity_endState ae         ON a.activityId = ae.activityId
       Left Outer Join pr_partner frp                  ON a.FRPARTNERIDHdr = frp.partnerId
       Left Outer Join pr_partner Top                  ON a.TOPARTNERIDHdr = top.partnerId
       LEFT OUTER JOIN PR_RoutingChannel ch            ON a.ChannelID = ch.ChannelID
       LEFT OUTER JOIN PR_WorkFlow wf           ON ch.workflowID = wf.workflowID
       LEFT OUTER JOIN PR_DestType t        ON t.desttypeid = a.desttypeid
       left outer join bp_dupcheck bpd      on bpd.activityid   = a.activityid
       left outer join LG_SYNCH_REQ_RESP lsrr on lsrr.activityid = a.activityid
   WHERE a.activityId     = p_activityId;

   DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

   EXCEPTION
   WHEN OTHERS THEN
       DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
       p_ReturnCd := 1;
       Raise_application_error (-20999,'20000 - LG_ActivityS_activityId ' || SQLCODE|| ' '||SQLERRM);
END ;
/


-- LG_ACTIVITYS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITYS_AH1
(
      p_ReturnCd       out pls_INTEGER
    , p_DebugFlg        in  pls_INTEGER
    , RC1               out db_ref.RCT      -- rows
    , RC2               out db_ref.rct_count          -- count
    , p_Locale          in  Varchar2
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_FROMPARTNERID   in  varchar2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_TOPARTNERID     in  varchar2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_STARTDATE       in  DATE                    --    (Date and Time)
    , p_ENDDATE         in  DATE                    --    (Date and Time)
    , p_SRCTRGFLG       in  pls_Integer     default 0  --  (Source or Target Protocol/Process/Ver Indicator: 0 = Src, 1 =Trg)
    , p_PACKAGENAME     in  varchar2                --(From or To based on SRCTRGFLG)
    , p_PACKAGEVER      in  varchar2                --(From or To based on SRCTRGFLG)
    , p_PROTOCOLNAME    in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROTOCOLVer     in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROCESSCD       in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROCESSVER      in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_CHANNELSTATUSID in  varchar2               -- Passing a list as (1,2,3,... etc.)
    , p_EndStateFlg     in  varchar2             -- pass in a list (lg_activity_endState.EndStateFlg: 1=Success,2=Failure
    , p_OriginalFileName in  varchar2                -- LG_ACTIVITY_DTL.ORIGINALFILENAME
    , p_DocId           in  varchar2                -- LG_ACTIVITY.docid
    , p_eventCD         in  pls_integer   default null -- added 2002/05/07
    , p_DestTypeId      in  number
    , p_vuid            in  varchar2
    , p_FRIPADDRESS     in  varchar2
    , p_SYNCHRONOUSFLG  in  pls_Integer   -- = 1 is sync 0 = non-sync = null all
    , P_FROMPARTNERTYPE   in varchar2 
    , P_TOPARTNERTYPE   in varchar2 
)
AS
/*
================================================================================
    Proc Name:      LG_ActivityS_AH1
    Create Date:    01/04/2002
    Author          Vipin Goel
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2  Removed NLS
    	2004-08-25 S.Priani      4.2.2  FP2:BGTRK00058859
    	2004-09-23 S.Priani      4.2.2  FP2:BGTRK00059981
    	2004-11-03 S.Priani      4.3	BGTRK00060472-back out ReSend changes
	2005-09-27 S.Priani	 6.0.0.2	BGTRK00055921
	2005-08-29 S.Narain	 6.1    	JR24677
	2007-08-25 Bharani J     6.1.1  For Multiple Internal Partner Feature
	2008-10-20 Amit vijay	 6.2	Defect 107019
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
  v_startdate        varchar2(20) := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
  v_enddate          varchar2(20) := to_char(nvl(p_EndDATE,sysdate)    , 'dd-mm-yy hh24:mi:ss');
  v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(2000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
--  sql_outer          varchar2(1000);
 
  v_partners_int	 varchar2(500);
  v_target_int	         varchar2(500);

  frPartnerName      pr_partner.partnerName%type;
  ToPartnerName      pr_partner.partnerName%type;
  Parent_hdrflg      pls_integer := 1 ; -- lg_activity_dtl.HDRFLG = 1
  Child_hdrflg       pls_integer := 0 ; -- lg_activity_dtl.HDRFLG = 0
----- Types
Type char65Tab is table of varchar2(65);
Type num10Tab  is table of number(10);
array_activityId char65Tab := char65Tab();
array_rnbr       num10Tab  := num10Tab();
--
activityIdTab  char65_Table := char65_Table();
----------
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
  DB_ProcAuditLogI_AH1('LG_ActivityS_AH1',v_beginTS,
    'p_ReturnCd'  ||','||
    p_DebugFlg          ||','||
    p_Locale       ||','||
    'RC1'          ||','||
    'RC2'          ||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_FROMPARTNERID||','||
    p_TOPARTNERID  ||','||
    To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
    p_SRCTRGFLG    ||','||
    p_PACKAGENAME  ||','||
    p_PACKAGEVER   ||','||
    p_PROTOCOLNAME ||','||
    p_PROTOCOLVer  ||','||
    p_PROCESSCD    ||','||
    p_PROCESSVER   ||','||
    p_CHANNELSTATUSID ||','||
    p_EndStateFlg  ||','||
    p_OriginalFileName        ||','||
    p_DocId        ||','||
    p_eventCD      ||','||
    p_DestTypeId   ||','||
    p_vuid         ||','||
    p_FRIPADDRESS  ||','||
    p_SYNCHRONOUSFLG ||','||
    p_frompartnertype ||','||
    p_topartnertype 
    ,0,0);
End ProcAudit;
Procedure where_clause as
begin

	v_where := ' WHERE 1=1 ';
   
	IF upper(p_frompartnertype) ='INTERNAL PARTNERS' then
   
		         		
		v_Partners_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';
		v_where := v_where || ' AND FRPARTNERIDHDR IN ' || v_Partners_Int ;	
             		
		IF p_FROMPARTNERID is not null then		       
			v_where := v_where || ' OR FRPARTNERIDHDR IN ' || p_FROMPARTNERID ;		       
		END IF;
		
		v_where := v_where || ' ) ';             		
          		
	END IF;
   
	IF upper(p_topartnertype)='INTERNAL PARTNERS' then
	
		
		v_target_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';       
		v_where := v_where || ' AND TOPARTNERIDHDR IN ' || v_target_Int ;
           
		IF p_TOPARTNERID is not null then
			v_where := v_where || ' OR TOPARTNERIDHDR IN ' || p_TOPARTNERID ;
		END IF;
          		
		v_where := v_where || ' ) ';    		
	END IF;
    
    
	IF p_frompartnertype IS NULL AND p_FROMPARTNERID is not null then
		v_where := v_where || ' AND FRPARTNERIDHDR IN ' || p_FROMPARTNERID ;
	END IF;
     
	IF p_topartnertype IS NULL AND p_TOPARTNERID is not null then
		v_where := v_where || ' AND TOPARTNERIDHDR IN ' || p_TOPARTNERID ;
	END IF;   
         
         
	v_where := v_where ||
		' AND  a.RCVDOCTS between '||chr(10)||' to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and'||
                                             chr(10)||' to_date('''|| v_endDate||''',''dd-mm-yy hh24:mi:ss'')'||
	FN_Build_Where('a.CHANNELSTATUSID','NUMBER_LIST', p_CHANNELSTATUSID )||
	FN_Build_Where('DESTTYPEID',       'NUMBER'     , p_DESTTYPEID      )||
	FN_Build_Where('e.EVENTCD',        'NUMBER'     , p_eventCD         )||
	FN_Build_Where('DOCID',            'VARCHAR2_LIKE'   , p_DOCID      )||
	FN_Build_Where('a.FRIPADDRESS',    'VARCHAR2'   , p_FRIPADDRESS     )||
	FN_Build_Where('d.Vuid',           'VARCHAR2_LIKE', p_vuid          )||
	FN_Build_Where('d.OriginalFileName','VARCHAR2_LIKE', p_OriginalFileName          );

   CASE
       WHEN p_SYNCHRONOUSFLG = 1 THEN
         v_where :=  v_where ||            ' and EXISTS (Select 1 from lg_activity_dtl lad where SYNCHRONOUSFLG = 1 and lad.activityId = a.activityid )';
       WHEN p_SYNCHRONOUSFLG = 0 THEN
        v_where :=  v_where ||        ' and NOT EXISTS (Select 1 from lg_activity_dtl lad where SYNCHRONOUSFLG = 1 and lad.activityId = a.activityid )';
    ELSE
        Null;
   END CASE;

  IF p_SRCTRGFLG = 0 THEN -- From
    v_where := v_where ||
    FN_Build_Where('FRPACKAGENAME',  'VARCHAR2', p_PACKAGENAME        )||
    FN_Build_Where('FRPACKAGEVER',   'VARCHAR2', p_PACKAGEVER         )||
    FN_Build_Where('FrPROTOCOLNAME', 'VARCHAR2', p_PROTOCOLNAME       )||
    FN_Build_Where('FrPROTOCOLVER',  'VARCHAR2', p_PROTOCOLVER        )||
    FN_Build_Where('FrPROCESSCD',    'VARCHAR2', p_PROCESSCD          )||
    FN_Build_Where('FrPROCESSVER',   'VARCHAR2', p_PROCESSVER         );
  ELSIF p_SRCTRGFLG = 1 then -- To
    v_where := v_where ||
    FN_Build_Where('ToPACKAGENAME',  'VARCHAR2', p_PACKAGENAME        )||
    FN_Build_Where('ToPACKAGEVER',   'VARCHAR2', p_PACKAGEVER         )||
    FN_Build_Where('ToPROTOCOLNAME', 'VARCHAR2', p_PROTOCOLNAME       )||
    FN_Build_Where('ToPROTOCOLVER',  'VARCHAR2', p_PROTOCOLVER        )||
    FN_Build_Where('ToPROCESSCD',    'VARCHAR2', p_PROCESSCD          )||
    FN_Build_Where('ToPROCESSVER',   'VARCHAR2', p_PROCESSVER         );
  END IF; 
END where_clause;
--                                 M  a  i  n
BEGIN
    p_ReturnCd := 0;
    where_clause;
    --DB_Print('Where Clause', v_Where);
    If p_ENDSTATEFLG is not null then
    if instr(p_ENDSTATEFLG,'0') = 0 then
       v_where := v_where ||
       ' and (EXISTS (Select 1 from lg_activity_endState ae where ae.activityid = a.activityid and ae.ENDSTATEFLG in '|| p_ENDSTATEFLG ||' ))';
    end if;

    if instr(p_ENDSTATEFLG, '0') > 0
    and instr(p_ENDSTATEFLG, '1') = 0
    and instr(p_ENDSTATEFLG,'2') = 0 then
       v_where := v_where||
        ' and NOT EXISTS (select 1 from lg_activity_endstate lge where lge.activityid = a.activityid)';
    end if;

    if instr(p_ENDSTATEFLG, '0') > 0
    and (instr(p_ENDSTATEFLG, '1') > 0
    or instr(p_ENDSTATEFLG,'2') > 0 ) then
       v_where := v_where ||
        ' and (EXISTS (Select 1 from lg_activity_endState ae where ae.activityid = a.activityid and ae.ENDSTATEFLG in '|| p_ENDSTATEFLG ||' )'||
        ' or NOT EXISTS (select 1 from lg_activity_endstate lge where lge.activityid = a.activityid) )';
    end if;
    end if;

    sql_inner :=
      'from lg_activity a ' ||
     -- ' left outer join lg_activity_endState ae on a.activityId     =  ae.activityId ' ||
     -- ' left outer join lg_activity_dtl_parent_vw fradtl on a.activityId = fradtl.activityId ' ||
     -- ' left outer join lg_activity_dtl_child_vw  toadtl on a.activityId = toadtl.activityId ' ||
      ' left outer join pr_partner frp on a.FRPARTNERIDHdr = frp.partnerId ' ||
      ' left outer join pr_partner top on a.ToPARTNERIDHdr = top.partnerId ' ;

IF p_eventCD is not null then
    sql_inner := sql_inner ||
              'left outer Join Lg_Event e on a.activityId =  e.activityId';
END IF;


IF p_vuid is not null OR p_OriginalFileName is not null then
    sql_inner := sql_inner ||
                  ' left outer Join Lg_Activity_dtl d on a.activityId =  d.activityId ';
END IF;


sqlstmt := 'select a.activityid '||sql_inner||' '||v_where ;
    if p_sortkey is not null then
        --EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);
        v_sortkey := upper(p_sortkey);
        v_sortkey := replace(v_sortkey,'FRPARTNERNAME','frp.partnername');
        v_sortkey := replace(v_sortkey,'TOPARTNERNAME','top.partnername');
        sqlstmt := sqlstmt ||' order by '||v_sortkey;
    end if;

-- Remove duplicate values while retaining the sorting: BGTRK00059848, BGTRK00059891
	sqlstmt := 'SELECT activityid from (select rownum, activityid from ('
			 || sqlstmt || ')) group by activityid order by min(rownum)';
-- total row count Cursor
    if RC2%IsOpen then close RC2; end if;
    if NVL(p_pageNbr,0) in (0,1) then
        Execute Immediate 'select count(*) from ('|| sqlstmt ||')' INTO v_count;
        open RC2 for select v_count TotalRowsNbr from dual;
     end if;
-- Paging logic
   sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
     --DB_Print('Sql paging', sqlstmt);

-- Open Cursor
    if RC1%IsOpen then close RC1; end if;
    execute immediate sqlstmt Bulk Collect INTO array_rnbr, array_ActivityId;

--    dbms_output.put_line('Last rnbr '||array_rnbr.LAST );
-- Transfer activityID from array to table
    For i IN 1..array_ActivityId.count LOOP
      activityIdTab.extend;
      activityIdTab(i) := char65_rec(array_rnbr(i), array_ActivityId(i));
      dbms_output.put_line('ActivityID '||array_ActivityId(i) );
    End Loop;

    open RC1 for
    select   /*+ First_Rows */
        rtrim(frp.partnername)    "FRPARTNERNAME",
        a.FRPARTNERIDHdr          "FRPARTNERID",
        --rtrim(a.FRBUSINESSIDHDR)  "FRBUSINESSID",
        a.FRBUSINESSIDHDR  "FRBUSINESSID",
        rtrim(Top.partnername)    "TOPARTNERNAME",
        a.TOPARTNERIDHdr         "TOPARTNERID",
        a.ToBUSINESSIDHdr        "ToBUSINESSID",
        rtrim(a.FRPACKAGENAME)   FRPACKAGENAME,
        rtrim(a.FRPACKAGEVER)    FRPACKAGEVER,
        rtrim(a.TOPACKAGENAME)   TOPACKAGENAME,
        rtrim(a.TOPACKAGEVER)    TOPACKAGEVER,
        a.FRPROTOCOLNAME	FRPROTOCOLNAME,
        a.FRPROTOCOLVER		FRPROTOCOLVER,
        a.TOPROTOCOLNAME	TOPROTOCOLNAME,
        a.TOPROTOCOLVER		TOPROTOCOLVER,
        RTRIM(FN_GET_RTOBJNAME(a.FrPROCESSCD, a.FrPROCESSVER, 'Process')) FrProcessName,
        rtrim(a.FrPROCESSCD) FrPROCESSCD,
        rtrim(a.FRPROCESSVER) FRPROCESSVER,
        RTRIM(FN_GET_RTOBJNAME(a.ToPROCESSCD, a.ToPROCESSVER, 'Process')) ToProcessName,
        rtrim(a.ToPROCESSCD)  ToPROCESSCD,
        rtrim(a.TOPROCESSVER)  TOPROCESSVER,
        rtrim(a.DocId) DocId,
        rtrim(a.DocTS) DocTS,
        a.ActivityId,
        a.RcvDocTS,
        a.CHANNELSTATUSID,
             CASE
		WHEN fradtl.SYNCHRONOUSFLG IS NULL THEN 0
		WHEN fradtl.SYNCHRONOUSFLG = 0 AND COALESCE(toadtl.SYNCHRONOUSFLG, 0) = 0 THEN 0
		ELSE 1
             END SYNCHRONOUSFLG,
        rtrim(t.Resourcekey) Resourcekey,
        rtrim(t.Description) Description,
        fradtl.vuid frVuid,
        toadtl.vuid ToVuid,
        fradtl.MSGRAWLENGTH frMSGRAWLENGTH,
        toadtl.MSGRAWLENGTH ToMSGRAWLENGTH,
        fradtl.MSGCONTENTLENGTH frMSGCONTENTLENGTH,
        toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH,
               fradtl.ORIGINALFILENAME frFILENAME,
               toadtl.ORIGINALFILENAME toFILENAME,
               fradtl.ENCODING frENCODING,
               toadtl.ENCODING toENCODING,
        ae.EndStateTS,
        nvl(ae.EndStateFlg,0) EndStateFlg,
        rtrim(wf.Resourcekey) WfResourcekey,
        rtrim(wf.Description) WfDescription
    from
        TABLE(cast(activityIdTab as char65_Table)) Y
        left outer join lg_activity a                    on Y.id             = a.activityId
        left outer join lg_activity_endState ae          on a.activityId     =  ae.activityId
        left outer join lg_activity_dtl_parent_vw fradtl on a.activityId     = fradtl.activityId
        left outer join lg_activity_dtl_child_vw  toadtl on a.activityId     = toadtl.activityId
        left outer join pr_partner frp                   on a.FRPARTNERIDHdr = frp.partnerId
        left outer join pr_partner top                   on a.ToPARTNERIDHdr = top.partnerId
        LEFT OUTER JOIN PR_RoutingChannel ch        ON a.ChannelID               = ch.ChannelID
        LEFT OUTER JOIN PR_WorkFlow wf       on ch.workflowID             = wf.workflowID
        LEFT OUTER JOIN PR_DestType t        on t.desttypeid     = a.desttypeid
    order by Y.seq;



    ProcAudit;
    Exception
        when others then
            ProcAudit;
            p_ReturnCd := 1;
END ;
/


-- LG_ACTIVITYS_AH2.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITYS_AH2
(
      p_ReturnCd        out pls_INTEGER
    , p_DebugFlg        in  pls_INTEGER
    , RC1               out db_ref.RCT      -- rows
    , RC2               out db_ref.rct_count          -- count
    , p_Locale          in  Varchar2
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_FROMPARTNERID   in  varchar2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_TOPARTNERID     in  varchar2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_STARTDATE       in  DATE                    --    (Date and Time)
    , p_ENDDATE         in  DATE                    --    (Date and Time)
    , p_SRCTRGFLG       in  pls_Integer     default 0  --  (Source or Target Protocol/Process/Ver Indicator: 0 = Src, 1 =Trg)
    , p_PACKAGENAME     in  varchar2                --(From or To based on SRCTRGFLG)
    , p_PACKAGEVER      in  varchar2                --(From or To based on SRCTRGFLG)
    , p_PROTOCOLNAME    in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROTOCOLVer     in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROCESSCD       in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROCESSVER      in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_CHANNELSTATUSID in  varchar2               -- Passing a list as (1,2,3,... etc.)
    , p_EndStateFlg     in  varchar2             -- pass in a list (lg_activity_endState.EndStateFlg: 1=Success,2=Failure
    , p_OriginalFileName in  varchar2                -- LG_ACTIVITY_DTL.ORIGINALFILENAME
    , p_DocId           in  varchar2                -- LG_ACTIVITY.docid
    , p_eventCD         in  pls_integer   default null -- added 2002/05/07
    , p_DestTypeId      in  number
    , p_vuid            in  varchar2
    , p_FRIPADDRESS     in  varchar2
    , p_SYNCHRONOUSFLG  in  pls_Integer   -- = 1 is sync 0 = non-sync = null all
    -- 56000 New fields added to the procedure
    , P_FROMPARTNERTYPE in varchar2 
    , P_TOPARTNERTYPE   in varchar2
    , p_USER01		in VARCHAR2
    , p_USER02		in VARCHAR2
    , p_USER03		in VARCHAR2
    , p_USER04		in VARCHAR2
    , p_USER05		in VARCHAR2
    , p_USER06		in VARCHAR2
    , p_USER07		in VARCHAR2
    , p_USER08		in VARCHAR2
    , p_USER09		in VARCHAR2
    , p_USER10		in VARCHAR2
    )
AS
/*
================================================================================
    Proc Name:      LG_ActivityS_AH2
    Create Date:    09/02/2006
    Author          
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------   ---------------------------------------------
	DATE       NAME          Ver      DESCRIPTION
	---------- ------------- ------   ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2    Removed NLS    
    	2004-08-25 S.Priani      4.2.2    FP2:BGTRK00058859    
    	2004-09-23 S.Priani      4.2.2    FP2:BGTRK00059981   
    	2004-11-03 S.Priani      4.3	  BGTRK00060472-back out ReSend changes   
	2005-09-27 S.Priani	 6.0.0.2  BGTRK00055921
	2006-02-20 S.Nair	 6.1.0    56000
	2005-08-29 S.Narain	 6.1      JR24677
        2007-08-25 Bharani J     6.1.1    For Multiple Internal Partner Feature
        2008-10-20 Amit vijay	 6.2	  Defect 107019
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(4000);
  v_startdate        varchar2(20) := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
  v_enddate          varchar2(20) := to_char(nvl(p_EndDATE,sysdate)    , 'dd-mm-yy hh24:mi:ss');
  v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(2000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
--  sql_outer          varchar2(1000);
  frPartnerName      pr_partner.partnerName%type;
  ToPartnerName      pr_partner.partnerName%type;
  Parent_hdrflg      pls_integer := 1 ; -- lg_activity_dtl.HDRFLG = 1
  Child_hdrflg       pls_integer := 0 ; -- lg_activity_dtl.HDRFLG = 0
  
  v_partners_int	varchar2(500);
  v_target_int	varchar2(500);

----- Types
Type char65Tab is table of varchar2(65);
Type num10Tab  is table of number(10);
array_activityId char65Tab := char65Tab();
array_rnbr       num10Tab  := num10Tab();
--
activityIdTab  char65_Table := char65_Table();
----------
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
  DB_ProcAuditLogI_AH1('LG_ActivityS_AH1',v_beginTS,
    'p_ReturnCd'  ||','||
    p_DebugFlg          ||','||
    p_Locale       ||','||
    'RC1'          ||','||
    'RC2'          ||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_FROMPARTNERID||','||
    p_TOPARTNERID  ||','||
    To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
    p_SRCTRGFLG    ||','||
    p_PACKAGENAME  ||','||
    p_PACKAGEVER   ||','||
    p_PROTOCOLNAME ||','||
    p_PROTOCOLVer  ||','||
    p_PROCESSCD    ||','||
    p_PROCESSVER   ||','||
    p_CHANNELSTATUSID ||','||
    p_EndStateFlg  ||','||
    p_OriginalFileName        ||','||
    p_DocId        ||','||
    p_eventCD      ||','||
    p_DestTypeId   ||','||
    p_vuid         ||','||
    p_FRIPADDRESS  ||','||
    p_SYNCHRONOUSFLG  ||','|| 
    p_frompartnertype  ||','||   
    p_topartnertype  ||','||   
    p_USER01	   ||','||
    p_USER02	   ||','||
    p_USER03	   ||','||    
    p_USER04	   ||','||
    p_USER05	   ||','||
    p_USER06	   ||','||
    p_USER07	   ||','||
    p_USER08	   ||','||
    p_USER09	   ||','||
    p_USER10
    ,0,0);
End ProcAudit;
Procedure where_clause as
begin

	v_where := ' WHERE 1=1 ';
      
	IF upper(p_frompartnertype)='INTERNAL PARTNERS' then
		          		
		v_Partners_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';
		v_where := v_where || ' AND FRPARTNERIDHDR IN ' || v_Partners_Int ;	
                		
		IF p_FROMPARTNERID is not null then		       
			v_where := v_where || ' OR FRPARTNERIDHDR IN ' || p_FROMPARTNERID ;		       
		END IF;
		v_where := v_where || ' ) ';             		
             		
	END IF;
      
	IF upper(p_topartnertype)='INTERNAL PARTNERS' then
			
		v_target_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';	           
		v_where := v_where || ' AND TOPARTNERIDHDR IN ' || v_target_Int ;
              
		IF p_TOPARTNERID is not null then
			v_where := v_where || ' OR TOPARTNERIDHDR IN ' || p_TOPARTNERID ;
		END IF;
             		
		v_where := v_where || ' ) ';    		
	END IF;
       
       
	IF p_frompartnertype IS NULL AND p_FROMPARTNERID is not null then
		v_where := v_where || ' AND FRPARTNERIDHDR IN ' || p_FROMPARTNERID ;
        END IF;
        
	IF p_topartnertype IS NULL AND p_TOPARTNERID is not null then
		v_where := v_where || ' AND TOPARTNERIDHDR IN ' || p_TOPARTNERID ;
	END IF;  
     
     
	v_where := v_where ||
		' AND  a.RCVDOCTS between '||chr(10)||' to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and'||
                                             chr(10)||' to_date('''|| v_endDate||''',''dd-mm-yy hh24:mi:ss'')'||
	FN_Build_Where('a.CHANNELSTATUSID','NUMBER_LIST', p_CHANNELSTATUSID )||
	FN_Build_Where('DESTTYPEID',       'NUMBER'     , p_DESTTYPEID      )||
	FN_Build_Where('e.EVENTCD',        'NUMBER'     , p_eventCD         )||
	FN_Build_Where('DOCID',            'VARCHAR2_LIKE'   , p_DOCID      )||
	FN_Build_Where('a.FRIPADDRESS',    'VARCHAR2'   , p_FRIPADDRESS     )||
	FN_Build_Where('d.Vuid',           'VARCHAR2_LIKE', p_vuid          )||
	FN_Build_Where('d.OriginalFileName','VARCHAR2_LIKE', p_OriginalFileName );      
      -- 56000 New fields added to FN_Build_Where
      IF p_USER01 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER01',    	 'VARCHAR2'   , COALESCE(p_USER01,''));
      END IF;
      IF p_USER02 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER02',    	 'VARCHAR2'   , COALESCE(p_USER02,''));
      END IF;
      IF p_USER03 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER03',    	 'VARCHAR2'   , COALESCE(p_USER03,''));
      END IF;
      IF p_USER04 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER04',    	 'VARCHAR2'   , COALESCE(p_USER04,''));
      END IF;
      IF p_USER05 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER05',    	 'VARCHAR2'   , COALESCE(p_USER05,''));
      END IF;
      IF p_USER06 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER06',    	 'VARCHAR2'   , COALESCE(p_USER06,''));
      END IF;
      IF p_USER07 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER07',    	 'VARCHAR2'   , COALESCE(p_USER07,''));
      END IF;
      IF p_USER08 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER08',    	 'VARCHAR2'   , COALESCE(p_USER08,''));
      END IF;
      IF p_USER09 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER09',    	 'VARCHAR2'   , COALESCE(p_USER09,''));
      END IF;
      IF p_USER10 is NOT NULL then
	v_where := v_where ||    FN_Build_Where('a.USER10',    	 'VARCHAR2'   , COALESCE(p_USER10,''));
      END IF;




   CASE
       WHEN p_SYNCHRONOUSFLG = 1 THEN
         v_where :=  v_where ||            ' and EXISTS (Select 1 from lg_activity_dtl lad where SYNCHRONOUSFLG = 1 and lad.activityId = a.activityid )';
       WHEN p_SYNCHRONOUSFLG = 0 THEN
        v_where :=  v_where ||        ' and NOT EXISTS (Select 1 from lg_activity_dtl lad where SYNCHRONOUSFLG = 1 and lad.activityId = a.activityid )';
    ELSE
        Null;
   END CASE;

  IF p_SRCTRGFLG = 0 THEN -- From
    v_where := v_where ||
    FN_Build_Where('FRPACKAGENAME',  'VARCHAR2', p_PACKAGENAME        )||
    FN_Build_Where('FRPACKAGEVER',   'VARCHAR2', p_PACKAGEVER         )||
    FN_Build_Where('FrPROTOCOLNAME', 'VARCHAR2', p_PROTOCOLNAME       )||
    FN_Build_Where('FrPROTOCOLVER',  'VARCHAR2', p_PROTOCOLVER        )||
    FN_Build_Where('FrPROCESSCD',    'VARCHAR2', p_PROCESSCD          )||
    FN_Build_Where('FrPROCESSVER',   'VARCHAR2', p_PROCESSVER         );
  ELSIF p_SRCTRGFLG = 1 then -- To
    v_where := v_where ||
    FN_Build_Where('ToPACKAGENAME',  'VARCHAR2', p_PACKAGENAME        )||
    FN_Build_Where('ToPACKAGEVER',   'VARCHAR2', p_PACKAGEVER         )||
    FN_Build_Where('ToPROTOCOLNAME', 'VARCHAR2', p_PROTOCOLNAME       )||
    FN_Build_Where('ToPROTOCOLVER',  'VARCHAR2', p_PROTOCOLVER        )||
    FN_Build_Where('ToPROCESSCD',    'VARCHAR2', p_PROCESSCD          )||
    FN_Build_Where('ToPROCESSVER',   'VARCHAR2', p_PROCESSVER         );
  END IF;
  
END where_clause;
--                                 M  a  i  n
BEGIN
    p_ReturnCd := 0;
    where_clause;
    --DB_Print('Where Clause', v_Where);
    If p_ENDSTATEFLG is not null then
    if instr(p_ENDSTATEFLG,'0') = 0 then
       v_where := v_where || 
       ' and (EXISTS (Select 1 from lg_activity_endState ae where ae.activityid = a.activityid and ae.ENDSTATEFLG in '|| p_ENDSTATEFLG ||' ))';
    end if;
    
    if instr(p_ENDSTATEFLG, '0') > 0 
    and instr(p_ENDSTATEFLG, '1') = 0 
    and instr(p_ENDSTATEFLG,'2') = 0 then
       v_where := v_where||
        ' and NOT EXISTS (select 1 from lg_activity_endstate lge where lge.activityid = a.activityid)';
    end if;
    
    if instr(p_ENDSTATEFLG, '0') > 0 
    and (instr(p_ENDSTATEFLG, '1') > 0 
    or instr(p_ENDSTATEFLG,'2') > 0 ) then        
       v_where := v_where ||
        ' and (EXISTS (Select 1 from lg_activity_endState ae where ae.activityid = a.activityid and ae.ENDSTATEFLG in '|| p_ENDSTATEFLG ||' )'||
        ' or NOT EXISTS (select 1 from lg_activity_endstate lge where lge.activityid = a.activityid) )';
    end if;
    end if;

    sql_inner :=
      'from lg_activity a ' ||
     -- ' left outer join lg_activity_endState ae on a.activityId     =  ae.activityId ' ||
     -- ' left outer join lg_activity_dtl_parent_vw fradtl on a.activityId = fradtl.activityId ' ||
     -- ' left outer join lg_activity_dtl_child_vw  toadtl on a.activityId = toadtl.activityId ' ||
      ' left outer join pr_partner frp on a.FRPARTNERIDHdr = frp.partnerId ' ||
      ' left outer join pr_partner top on a.ToPARTNERIDHdr = top.partnerId ' ;

IF p_eventCD is not null then
    sql_inner := sql_inner ||
              'left outer Join Lg_Event e on a.activityId =  e.activityId ';
END IF;


IF p_vuid is not null OR p_OriginalFileName is not null then
    sql_inner := sql_inner ||
                  ' left outer Join Lg_Activity_dtl d on a.activityId =  d.activityId ';
END IF;


sqlstmt := 'select a.activityid '||sql_inner||' '||v_where ;
    if p_sortkey is not null then
        --EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);
        v_sortkey := upper(p_sortkey);
        v_sortkey := replace(v_sortkey,'FRPARTNERNAME','frp.partnername');
        v_sortkey := replace(v_sortkey,'TOPARTNERNAME','top.partnername');
        sqlstmt := sqlstmt ||' order by '||v_sortkey;
    end if;

-- Remove duplicate values while retaining the sorting: BGTRK00059848, BGTRK00059891
	sqlstmt := 'SELECT activityid from (select rownum, activityid from ('
			 || sqlstmt || ')) group by activityid order by min(rownum)';
-- total row count Cursor
    if RC2%IsOpen then close RC2; end if;
    if NVL(p_pageNbr,0) in (0,1) then
        Execute Immediate 'select count(*) from ('|| sqlstmt ||')' INTO v_count;
        open RC2 for select v_count TotalRowsNbr from dual;
     end if;
-- Paging logic
   sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
     --DB_Print('Sql paging', sqlstmt);

-- Open Cursor
    if RC1%IsOpen then close RC1; end if;
    execute immediate sqlstmt Bulk Collect INTO array_rnbr, array_ActivityId;

--    dbms_output.put_line('Last rnbr '||array_rnbr.LAST );
-- Transfer activityID from array to table
    For i IN 1..array_ActivityId.count LOOP
      activityIdTab.extend;
      activityIdTab(i) := char65_rec(array_rnbr(i), array_ActivityId(i));
      dbms_output.put_line('ActivityID '||array_ActivityId(i) );
    End Loop;

    open RC1 for
    select   /*+ First_Rows */
        rtrim(frp.partnername)    "FRPARTNERNAME",
        a.FRPARTNERIDHdr          "FRPARTNERID",
        --rtrim(a.FRBUSINESSIDHDR)  "FRBUSINESSID",
        a.FRBUSINESSIDHDR  "FRBUSINESSID",
        rtrim(Top.partnername)    "TOPARTNERNAME",
        a.TOPARTNERIDHdr         "TOPARTNERID",
        a.ToBUSINESSIDHdr        "ToBUSINESSID",
        rtrim(a.FRPACKAGENAME)   FRPACKAGENAME,
        rtrim(a.FRPACKAGEVER)    FRPACKAGEVER,
        rtrim(a.TOPACKAGENAME)   TOPACKAGENAME,
        rtrim(a.TOPACKAGEVER)    TOPACKAGEVER,
        a.FRPROTOCOLNAME	FRPROTOCOLNAME,
        a.FRPROTOCOLVER		FRPROTOCOLVER,
        a.TOPROTOCOLNAME	TOPROTOCOLNAME,
        a.TOPROTOCOLVER		TOPROTOCOLVER,
        RTRIM(FN_GET_RTOBJNAME(a.FrPROCESSCD, a.FrPROCESSVER, 'Process')) FrProcessName,
        rtrim(a.FrPROCESSCD) FrPROCESSCD,
        rtrim(a.FRPROCESSVER) FRPROCESSVER,
        RTRIM(FN_GET_RTOBJNAME(a.ToPROCESSCD, a.ToPROCESSVER, 'Process')) ToProcessName,
        rtrim(a.ToPROCESSCD)  ToPROCESSCD,
        rtrim(a.TOPROCESSVER)  TOPROCESSVER,
        rtrim(a.DocId) DocId,
        rtrim(a.DocTS) DocTS,
        a.ActivityId,
        a.RcvDocTS,
        a.CHANNELSTATUSID,
             CASE 
		WHEN fradtl.SYNCHRONOUSFLG IS NULL THEN 0
		WHEN fradtl.SYNCHRONOUSFLG = 0 AND COALESCE(toadtl.SYNCHRONOUSFLG, 0) = 0 THEN 0 
		ELSE 1
             END SYNCHRONOUSFLG,
        rtrim(t.Resourcekey) Resourcekey,
        rtrim(t.Description) Description,
        fradtl.vuid frVuid,
        toadtl.vuid ToVuid,
        fradtl.MSGRAWLENGTH frMSGRAWLENGTH,
        toadtl.MSGRAWLENGTH ToMSGRAWLENGTH,
        fradtl.MSGCONTENTLENGTH frMSGCONTENTLENGTH,
        toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH,
               fradtl.ORIGINALFILENAME frFILENAME,
               toadtl.ORIGINALFILENAME toFILENAME,
               fradtl.ENCODING frENCODING,
               toadtl.ENCODING toENCODING,
               fradtl.MessageStoreRequired frmesgstorereqd,
               toadtl.MessageStorerequired tomesgstorereqd,  
        ae.EndStateTS,
        nvl(ae.EndStateFlg,0) EndStateFlg,
        rtrim(wf.Resourcekey) WfResourcekey,
        rtrim(wf.Description) WfDescription,
        -- 56000 New fields added to the select list
        a.USER01,
	a.USER02,
	a.USER03,
	a.USER04,
	a.USER05,
	a.USER06,
	a.USER07,
	a.USER08,
	a.USER09,
	a.USER10,
	bpd.key01,
	bpd.key02,
	bpd.key03,
	bpd.key04,
	bpd.key05,
	bpd.key06,
	bpd.key07,
	bpd.key08,
	bpd.key09,
	bpd.key10,
	'dummy' VUID_RESP
    from
        TABLE(cast(activityIdTab as char65_Table)) Y
        left outer join lg_activity a                    on Y.id             = a.activityId
        left outer join lg_activity_endState ae          on a.activityId     =  ae.activityId
        left outer join lg_activity_dtl_parent_vw fradtl on a.activityId     = fradtl.activityId
        left outer join lg_activity_dtl_child_vw  toadtl on a.activityId     = toadtl.activityId
        left outer join pr_partner frp                   on a.FRPARTNERIDHdr = frp.partnerId
        left outer join pr_partner top                   on a.ToPARTNERIDHdr = top.partnerId
        LEFT OUTER JOIN PR_RoutingChannel ch        ON a.ChannelID               = ch.ChannelID
        LEFT OUTER JOIN PR_WorkFlow wf       on ch.workflowID             = wf.workflowID
        LEFT OUTER JOIN PR_DestType t        on t.desttypeid     = a.desttypeid
        left outer join bp_dupcheck bpd      on bpd.activityid   = a.activityid
       -- left outer join LG_SYNCH_REQ_RESP lsrr on lsrr.activityid = a.activityid
    order by Y.seq;



    ProcAudit;
    Exception
        when others then
            ProcAudit;
            p_ReturnCd := 1;
END ;
/


-- LG_ACTIVITYS_DOCID.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITYS_DOCID
	(
	  p_ReturnCd       OUT   PLS_INTEGER
	, p_DebugFlg       IN    PLS_INTEGER
	, RC1              OUT   DB_REF.RCT                                                                      
	, P_DOCID 	   IN    VARCHAR2
	, P_INITBUSINESSID IN    VARCHAR2
	)                                                                              
AS                                                                              

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       LG_ACTIVITYS_DOCID                                             
	Create Date:     2006-05-26 10:40:58                                           
	Author:          S.Nair 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                      M O D I F I C A T I O N                                  
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------	-------	---------------------------------------------  
	DATE       NAME		VER	DESCRIPTION                                               
	---------- ------------	-------	---------------------------------------------  
	2006-05-26 S.Nair	6.1	Created
                                                 
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER Default 0;                                           
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        VARCHAR2(30) Default 'LG_ACTIVITYS_DOCID';                     
v_ExceptionFlg    INTEGER Default 0;                                            
v_ParamsTxt       VARCHAR2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', P_DOCID=>'||P_DOCID||CHR(10)
		||', P_INITBUSINESSID=>'||P_INITBUSINESSID||CHR(10)
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	
IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          

	OPEN RC1 FOR 
	SELECT                                                          
		ACTIVITYID
	FROM 
		LG_ACTIVITY
	Where 
		DOCID = p_DOCID
		AND INITBUSINESSIDHDR = P_INITBUSINESSID
	;

p_ReturnCd := 0;
DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

EXCEPTION
	WHEN No_Data_Found THEN
	    p_ReturnCd     := 3;
	WHEN Others Then
		Raise_Application_Error (-20999, '20002 LG_ACTIVITYS_DOCID '|| SQLERRM);
End LG_ACTIVITYS_DOCID;
/

-- LG_ACTIVITYS_DOCLATENCY.sql

CREATE OR REPLACE PROCEDURE         LG_ACTIVITYS_DOCLATENCY
(
    p_ReturnCd      out pls_integer
    , p_debugflg    in pls_integer
    , RC1           out db_ref.RCT
    , p_STARTDATE   in DATE                --    (Date and Time)
    , p_ENDDATE     in DATE                    --    (Date and Time)
) As
/*
================================================================================
Proc Name:      LG_ACTIVITYS_DOCLATENCY
Create Date:    07/26/02
Author          S.HUTCHINSON
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME                  DESCRIPTION
---------  --------------------  ---------------------------------------------
07/26/02   S.Hutchinson          Created
01/28/03   J.Chen                Calling Interval2Millisecons to get interval in milliseconds
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_ACTIVITYS_DOCLATENCY',v_beginTS,'RETURN_CODE'||','||p_DEBUGflg||','
    ||'RC1'||','||P_STARTDATE||','||P_ENDDATE);
End ProcAudit;
BEGIN
IF RC1%ISOPEN THEN CLOSE RC1; END IF;
Open RC1 for
SELECT /*+ INDEX(AE PK_ACTIVITY_ENDSTATE) */
      A.INITBUSINESSIDHDR INITBUSINESSID,
      A.FRBUSINESSIDHDR FRBUSINESSID,
      A.TOBUSINESSIDHDR TOBUSINESSID,
      RTRIM(P1.PARTNERNAME) INITPARTNER,
      RTRIM(P2.PARTNERNAME) FRPARTNER,
      RTRIM(P3.PARTNERNAME) TOPARTNER,
      CASE
          WHEN AE.ENDSTATEFLG = 1 THEN 'SUCCESS'
          WHEN AE.ENDSTATEFLG = 2 THEN 'FAIL'
      END ENDSTATE,
      MIN(A.RCVDOCTS) MIN_RCVDOCTS,
      MAX(AE.ENDSTATETS) MAX_ENDSTATETS,
      MIN(FN_TimeStampDiff(A.RCVDOCTS, AE.ENDSTATETS)) MIN_PROC_TIME_MS,
      MAX(FN_TimeStampDiff(A.RCVDOCTS, AE.ENDSTATETS)) MAX_PROC_TIME_MS,
      AVG(FN_TimeStampDiff(A.RCVDOCTS, AE.ENDSTATETS)) AVG_PROC_TIME_MS,
      COUNT(*) DOC_COUNT
FROM
      LG_ACTIVITY A
      INNER JOIN LG_ACTIVITY_ENDSTATE AE ON A.ACTIVITYID = AE.ACTIVITYID
      INNER JOIN PR_PARTNER P1 ON A.INITPARTNERIDHDR = P1.PARTNERID
      INNER JOIN PR_PARTNER P2 ON A.FRPARTNERIDHDR = P2.PARTNERID
      INNER JOIN PR_PARTNER P3 ON A.TOPARTNERIDHDR = P3.PARTNERID
WHERE
      A.RCVDOCTS BETWEEN P_STARTDATE AND P_ENDDATE
GROUP BY
      A.INITBUSINESSIDHDR,
      A.FRBUSINESSIDHDR,
      A.TOBUSINESSIDHDR,
      P1.PARTNERNAME,
      P2.PARTNERNAME,
      P3.PARTNERNAME,
      CASE
          WHEN AE.ENDSTATEFLG = 1 THEN 'SUCCESS'
          WHEN AE.ENDSTATEFLG = 2 THEN 'FAIL'
      END
ORDER BY
      1, 2, 3, 4;
p_ReturnCd := 0;
procAudit;
End LG_ACTIVITYS_DOCLATENCY;
/


-- LG_ACTIVITYS_DOCLIST.sql

CREATE OR REPLACE PROCEDURE         LG_ACTIVITYS_DOCLIST(
      p_ReturnCd       in out pls_INTEGER
    , p_DebugFlg        in     pls_INTEGER
    , RC1               out    db_ref.RCT      -- weak cursor   --rows
    , p_FROMPARTNERID   in     PLS_Integer     --   single partner id
    , p_TOPARTNERID     in     PLS_Integer     --    single partner id
    , p_STARTDATE       in     TimeStamp       --    (Date and Time)
    , p_ENDDATE         in     TimeStamp       --    (Date and Time)
    , p_DocId           in     varchar2        -- LG_ACTIVITY.docid is a list ('id1','id2',......)
    )
AS
/*
================================================================================
Proc Name:      LG_ACTIVITYS_DOCLIST
Create Date:    2003-01-24
Author          Vipin Goel
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------   -------- --------  ---------------------------------------------
DATE        NAME     VERSION   DESCRIPTION
---------   -------- --------  ---------------------------------------------
2003-01-24  V.Goel   3.4       New proc created
2003-04-22  V.Goel   3.5       Globalization
2003-05-15  V.Goel   3.5       Replace pr_process by new CF_RoutingObject,
                               pr_channel by PR_RoutingChannel and
                               PR_WorkFlow by PR_WorkFlow_Loc_vw.
2003-05-21  L.Ling   3.5       Added FN_Get_RoutingObjName().
                               Removed CF_RoutingObject and CF_RoutingObjType tables.
2003-05-30  L.Ling   3.5       Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
2003-07-08  V.Goel   3.5       Use FN_Build_Where
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
  sqlStmt            varchar2(4000);

----------
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
-- If p_debugFlg =1  Then
  DB_ProcAuditLogI_ah1('LG_ACTIVITYS_DOCLIST',v_beginTS,'p_ReturnCd'||','||
    p_DEBUGFlg||','||
    'RC1'    ||','||
    p_FROMPARTNERID||','||
    p_TOPARTNERID  ||','||
    To_char(p_STARTDATE,'DD-MON-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,  'DD-MON-YY HH24:mi:ss')||','||
    p_DocId,  p_DEBUGFlg
    );
--End if ;
End ProcAudit;

--
Procedure where_clause as
begin
    v_where := ' Where 1=1'||
    FN_Build_Where('a.FRPARTNERIDHDR',  'NUMBER',        p_FROMPARTNERID)||
    FN_Build_Where('a.TOPARTNERIDHDR',  'NUMBER',        p_ToPARTNERID)  ||
    FN_Build_Where('a.DOCID',           'VARCHAR2_LIST', p_DOCID);

end where_clause;
--                                 M  a  i  n
BEGIN
    p_ReturnCd := 0;
  --  v_where := v_where||' Where  a.RCVDOCTS between '||chr(10)||' to_date('''||v_startDate||''',''dd-mon-yy hh24:mi:ss'') and'||
  --                                                  chr(10)||' to_date('''|| v_endDate||''',''dd-mon-yy hh24:mi:ss'')';

    where_clause;
    DB_Print('Where ', v_where);



-- Open Cursor
    if RC1%IsOpen then close RC1; end if;

     sqlStmt :='SELECT
               frp.partnername    "FRPARTNERNAME",
               a.FRPARTNERIDHdr   "FRPARTNERID",
               a.FRBUSINESSIDHDR  "FRBUSINESSID",
               Top.partnername    "TOPARTNERNAME",
               a.TOPARTNERIDHdr   "TOPARTNERID",
               a.ToBUSINESSIDHdr  "ToBUSINESSID",
               a.FRPROTOCOLNAME,
               a.FRPROTOCOLVER,
               a.TOPROTOCOLNAME,
               a.TOPROTOCOLVER,
               --frproc.RoutingObjName  "FrPROCESSName",
               RTRIM(FN_Get_RoutingObjName(a.FrProcessCd, a.FrProcessVer, ''Process'')) FrProcessName,
               a.FrPROCESSCD,
               a.FRPROCESSVER,
               --Toproc.RoutingObjName "ToPROCESSName",
               RTRIM(FN_Get_RoutingObjName(a.ToProcessCd, a.ToProcessVer, ''Process'')) ToProcessName,
               a.ToPROCESSCD,
               a.TOPROCESSVER,
               a.DocId,
               a.DocTS,
               a.ActivityId,
               a.RcvDocTS,
               a.CHANNELSTATUSID,
               t.Resourcekey,
               t.Description,
               fradtl.vuid frVuid,
               toadtl.vuid ToVuid,
               fradtl.MSGRAWLENGTH frMSGRAWLENGTH,
               toadtl.MSGRAWLENGTH ToMSGRAWLENGTH,
               fradtl.MSGCONTENTLENGTH frMSGCONTENTLENGTH,
               toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH,
               fradtl.ORIGINALFILENAME frFILENAME,
               toadtl.ORIGINALFILENAME toFILENAME,
               fradtl.ENCODING frENCODING,
               toadtl.ENCODING toENCODING,
               ae.EndStateTS,
               nvl(ae.EndStateFlg,0) EndStateFlg
             FROM
             lg_activity a
             left outer join lg_activity_endState ae          on a.activityId      =  ae.activityId
             left outer join lg_activity_dtl_parent_vw fradtl on a.activityId      = fradtl.activityId
             left outer join lg_activity_dtl_child_vw  toadtl on a.activityId      = toadtl.activityId
             left outer join pr_partner frp                   on a.FRPARTNERIDHdr  = frp.partnerId
             left outer join pr_partner top                   on a.ToPARTNERIDHdr  = top.partnerId
           LEFT OUTER JOIN PR_RoutingChannel ch        ON a.ChannelID             = ch.ChannelID
           LEFT OUTER JOIN PR_WorkFlow wf       on ch.workflowID           = wf.workflowID
           left outer join PR_DestType t       on t.desttypeid            = a.desttypeid
           WHERE
            a.RCVDOCTS between nvl(:STARTDATE,(FN_Get_Date-1)) And nvl(:EndDate,FN_Get_Date)
           '
           ||v_where||' ORDER BY a.RcvDocTS'
           ;
           DB_Print('Sql ', sqlstmt);

           open RC1 for sqlStmt using  p_STARTDATE, p_EndDATE;
-- proc Audit
       ProcAudit;

       Exception
       when others then
           ProcAudit;
           p_ReturnCd := 1;
--    db_Print('Sql ', sqlstmt);
/*
     dbms_output.put_line(substr(sqlstmt,1,250));
     dbms_output.put_line(substr(sqlstmt,251,250));
     dbms_output.put_line(substr(sqlstmt,501,250));
     dbms_output.put_line(substr(sqlstmt,751,250));
     dbms_output.put_line(substr(sqlstmt,1001,250));
*/
END ;
/


-- LG_ACTIVITYS_INOUTPERF.sql

CREATE OR REPLACE PROCEDURE         LG_ACTIVITYS_INOUTPERF
(
    p_ReturnCd out integer
    , p_debugflg in pls_integer
    , RC1 out db_ref.RCT
    , RC2 out db_ref.RCT
    , p_STARTDATE in DATE                --    (Date and Time)
    , p_ENDDATE in DATE                    --    (Date and Time)
) As
/*
================================================================================
Proc Name:      LG_ACTIVITYS_INOUTPERF
Create Date:    07/26/02
Author          S.HUTCHINSON
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit
is
begin
    DB_ProcAuditLogI_ah1('LG_ACTIVITYS_INOUTPERF',v_beginTS,'RETURN_CODE'||','||p_DEBUGflg||','
    ||'RC1'||','||P_STARTDATE||','||P_ENDDATE);
End ProcAudit;
BEGIN
IF RC1%ISOPEN THEN CLOSE RC1; END IF;
Open RC1 for
       SELECT
             DAYHOURMI  "RCVTIME",
             RCVDOCCNT  DOCUMENTCNT,
             RCVDOCCNT /60 "DocPerSecond"
       FROM
             LG_SUMMARY_MI
       WHERE
             DAYHOURMI BETWEEN P_STARTDATE AND P_ENDDATE ;
IF RC2%ISOPEN THEN CLOSE RC2; END IF;
OPEN RC2 FOR
       SELECT
            ENDSTATEDAYHOURMI "SNTTIME",
            SNTDOCCNT "SuccessCnt",
            FAILDOCCNT  "FailCnt" ,
            SNTDOCCNT/60 "Success/Second",
            FAILDOCCNT/60 "Failed/Second"
       FROM
             LG_SUMMARY_MI
       WHERE
             ENDSTATEDAYHOURMI BETWEEN P_STARTDATE AND P_ENDDATE ;
p_ReturnCd := 0;
procAudit;
End LG_ACTIVITYS_INOUTPERF;
/


-- LG_ACTIVITYS_RNDTL_ACTIVITYID.sql

CREATE OR REPLACE PROCEDURE  LG_ActivityS_RNDtl_ActivityID (
      p_ReturnCd    out pls_INTEGER
    , p_DebugFlg    in  pls_INTEGER
    , RC1           out db_ref.rct          -- rows
    , p_ActivityId  in  varchar2
    )
AS
/*
================================================================================
    Proc Name:      LG_ActivityS_rndtl_activityID
    Create Date:    01/04/2002
    Author          Vipin Goel
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_BeginTS          timestamp(6):= FN_Get_Date;
v_ProcName          varchar2(30) := 'LG_ActivityS_RNDtl_ActivityID';
v_ExceptionFlg      integer := 0;
v_ParamsTxt         varchar2(4000) :=
                            'p_ReturnCd'||','||p_DebugFlg||
                            ','||'RC1'||','||
                            p_ActivityId;
--
-- Main
--
BEGIN
    p_ReturnCd := 0;
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;

    OPEN RC1 FOR
    SELECT
        RTRIM(frp.partnername)     FRPARTNERNAME,
        a.FRPARTNERIDHdr           FRPARTNERID,
        RTRIM(a.FRBUSINESSIDHDR)   FRBUSINESSID,
        RTRIM(Top.partnername)     TOPARTNERNAME,
        a.TOPARTNERIDHdr           TOPARTNERID,
        RTRIM(a.ToBUSINESSIDHdr)   ToBUSINESSID ,
        RTRIM(FRPROTOCOLNAME)      FRPROTOCOLNAME ,
        RTRIM(TOPROTOCOLNAME)      TOPROTOCOLNAME,
        a.FrPROCESSCD,
        RTRIM(FN_GET_RTOBJNAME(a.FrProcessCd, a.FrProcessVer, 'Process')) FrProcessName,
        RTRIM(a.FRPROCESSVer)      FRPROCESSVer,
        a.ToPROCESSCD,
        RTRIM(FN_GET_RTOBJNAME(a.ToProcessCd, a.ToProcessVer, 'Process')) ToProcessName,
        RTRIM(a.TOPROCESSVer)      TOPROCESSVer,
        RTRIM(a.DocId) DocId,
        a.DocTS,
        RTRIM(a.ActivityId)        ACTIVITYID,
        a.RcvDocTS,
        a.CHANNELSTATUSID,
        RTRIM(fradtl.vuid)         frVuid,
        RTRIM(toadtl.vuid)         ToVuid,
        fradtl.MSGRAWLENGTH  FrMSGRAWLENGTH, fradtl.MSGCONTENTLENGTH FrMSGCONTENTLENGTH,
        Toadtl.MSGRAWLENGTH  ToMSGRAWLENGTH, Toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH,
               fradtl.ORIGINALFILENAME frFILENAME,
               toadtl.ORIGINALFILENAME toFILENAME,
               fradtl.ENCODING frENCODING,
               toadtl.ENCODING toENCODING,
        ae.EndStateTS,
        ae.EndStateFlg
       ,   rndtl.CID
       ,   RTRIM(rndtl.INITBUSINESSID)         INITBUSINESSID
       ,   RTRIM(rndtl.PROCESSINSTANCEID)      PROCESSINSTANCEID
       ,   RTRIM(rndtl.ACTSIGTRACKINSTANCEID)  ACTSIGTRACKINSTANCEID
       ,   rndtl.INITPARTNERID
       ,   RTRIM(rndtl.TRANSACTIONINSTANCEID)  TRANSACTIONINSTANCEID
       ,   RTRIM(rndtl.GLBUSAGECD)             GLBUSAGECD
       ,   RTRIM(rndtl.MSGDATETIME)            MSGDATETIME
       ,   RTRIM(rndtl.FRBUSIDTYPE)            FRBUSIDTYPE
       ,   RTRIM(rndtl.TOBUSIDTYPE)            TOBUSIDTYPE
       ,   RTRIM(rndtl.IRTGLBBUSACTIONCD)      IRTGLBBUSACTIONCD
       ,   RTRIM(rndtl.IRTINSTANCEID)          IRTINSTANCEID
       ,   RTRIM(rndtl.GLBBUSACTIONCD)         GLBBUSACTIONCD
       ,   RTRIM(rndtl.SECURETRANSREQD)        SECURETRANSREQD
       ,   rndtl.ATTEMPTCNT
       ,   RTRIM(rndtl.FRSERVICE)              FRSERVICE
       ,   RTRIM(rndtl.TOSERVICE)              TOSERVICE
       ,   RTRIM(rndtl.GLBDOCFUNCTIONCD)       GLBDOCFUNCTIONCD
       ,   RTRIM(rndtl.FRGLBPARTNERCLASSCD)    FRGLBPARTNERCLASSCD
       ,   RTRIM(rndtl.TOGLBPARTNERCLASSCD)    TOGLBPARTNERCLASSCD
       ,   RTRIM(rndtl.GLBBUSSIGNALCD)         GLBBUSSIGNALCD
       ,   RTRIM(rndtl.FRROLE)  FRROLE
       ,   RTRIM(rndtl.TOROLE)  TOROLE
       ,   RTRIM(rndtl.ACTIVITYNAME)           ACTIVITYNAME
       ,   RTRIM(rndtl.GLBPROCESSCD)           GLBPROCESSCD
       ,   rndtl.ATTACHMENTNBR
    FROM lg_activity a
        inner join lg_activity_rndtl rndtl          on a.activityId    = rndtl.activityId
        Left Outer Join lg_activity_dtl_parent_vw fradtl on a.activityId    = fradtl.activityId
        Left Outer Join lg_activity_dtl_child_vw toadtl  on a.activityId    = Toadtl.activityId
        Left Outer Join lg_activity_endState ae     on a.activityId    = ae.activityId
        Left Outer Join pr_partner frp              on a.FRPARTNERIDHdr = frp.partnerId
        Left Outer Join pr_partner Top              on a.TOPARTNERIDHdr = top.partnerId
    WHERE   a.ActivityId     = p_ActivityId ;

   DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   Exception
   when others then
       DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
       p_ReturnCd := 1;
       Raise_application_error (-20999,'20000 - LG_ActivityS_rndtl_activityID ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_ACTIVITYS_RNDTL_CID.sql

CREATE OR REPLACE PROCEDURE         LG_ActivityS_rndtl_CID (
      p_ReturnCd  out pls_INTEGER
    , p_DebugFlg        in  pls_INTEGER
    , RC1          out db_ref.rct          -- rows
    , p_cid        in  pls_INTEGER
    )
AS
/*
================================================================================
    Proc Name:      LG_ActivityS_rndtl_CID
    Create Date:    01/04/2002
    Author          Vipin Goel
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_beginTS          timestamp(6):= FN_Get_Date;
v_ProcName          varchar2(30) := 'LG_ActivityS_rndtl_CID';
v_ExceptionFlg      integer := 0;
v_ParamsTxt         varchar2(4000) :=
                            'p_ReturnCd'||','||p_DebugFlg||
                            ','||'RC1'||','||
                            p_cid;
--
-- Main
--
BEGIN
    p_ReturnCd := 0;
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;

    OPEN RC1 FOR
    SELECT
        frp.partnername InitPARTNERNAME, a.InitPARTNERIDHdr InitPARTNERID, a.InitBUSINESSIDHDR InitBUSINESSID,
        a.frpartneridHdr FrPartnerid, frp2.PartnerName FrPartnerName,
        Top.partnername TOPARTNERNAME, a.TOPARTNERIDHdr TOPARTNERID, a.ToBUSINESSIDHdr ToBUSINESSID ,
        FRPROTOCOLNAME, TOPROTOCOLNAME,
        RTRIM(FN_GET_RTOBJNAME(a.FrProcessCd, a.FrProcessVer, 'Process')) FrProcessName,
        a.FrPROCESSCD, a.FRPROCESSVer,
        RTRIM(FN_GET_RTOBJNAME(a.ToProcessCd, a.ToProcessVer, 'Process')) ToProcessName,
        a.ToPROCESSCD, a.TOPROCESSVer,
        a.DocId, a.DocTS, a.ActivityId, a.RcvDocTS,a.CHANNELSTATUSID,
        fradtl.vuid frVuid, toadtl.vuid ToVuid,
        fradtl.MSGRAWLENGTH  FrMSGRAWLENGTH, fradtl.MSGCONTENTLENGTH FrMSGCONTENTLENGTH,fradtl.MESSAGESTOREREQUIRED frmesgstorereqd,
        Toadtl.MSGRAWLENGTH  ToMSGRAWLENGTH, Toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH,toadtl.MESSAGESTOREREQUIRED tomesgstorereqd,
        ae.EndStateTS, ae.EndStateFlg
	,RTRIM(fradtl.ORIGINALFILENAME) frFILENAME
	,RTRIM(toadtl.ORIGINALFILENAME) toFILENAME
	,RTRIM(fradtl.ENCODING) frENCODING
	,RTRIM(toadtl.ENCODING) toENCODING
       , rndtl.ATTEMPTCNT  ATTEMPTCount
        , rndtl.INITPARTNERID
        , rndtl.ACTSIGTRACKINSTANCEID
        , rndtl.GLBBUSACTIONCD
        , rndtl.GLBBUSSIGNALCD
    FROM
        lg_activity a
        inner join lg_activity_rndtl rndtl     on a.activityId  = rndtl.activityId
        Left Outer Join lg_activity_dtl fradtl on a.activityId  = fradtl.activityId
                                          and fradtl.hdrflg = 1
        Left Outer Join lg_activity_dtl Toadtl  on a.activityId = Toadtl.activityId
                                          and Toadtl.hdrflg = 0
        Left Outer Join lg_activity_endState ae on a.activityId = ae.activityId
        Left Outer Join pr_partner frp          on a.InitPARTNERIDHdr = frp.partnerId
        Left Outer Join pr_partner Top          on a.TOPARTNERIDHdr = top.partnerId
        Left Outer Join pr_partner frp2         on a.FrPARTNERIDHdr = frp2.partnerId
    WHERE
        rndtl.cid     = p_cid
    ORDER BY a.RcvDocTS;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    Exception
        when others then
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
        p_ReturnCd := 1;
        Raise_application_error (-20999,'20000 - LG_ActivityS_rndtl_CID ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_ACTIVITYS_SYNCH.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITYS_SYNCH
        (
        p_ReturnCd      OUT  PLS_INTEGER
        , p_DebugFlg    IN  PLS_INTEGER
        , RC1           OUT  DB_REF.RCT
        , p_VUID IN VARCHAR2
        )
AS

/*
        ================================================================================
        Proc Name:       LG_ACTIVITYS_SYNCH
        Create Date:     2003-10-22 08:59:37
        Author:          SYSTEM 

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-10-22 SYSTEM           3.5 Created
        2004-09-20 S.Priani	 4.2.2	FP2:BGTRK00059887
        2004-09-24 S.Priani	 4.2.2	FP2:BGTRK00047091

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; 
v_ProcName        varchar2(30) Default 'LG_ACTIVITYS_SYNCH';
v_ExceptionFlg    integer Default 0;
v_ActivityID      varchar2(65);
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_VUID=>'||p_VUID||CHR(10)
                ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
--    Get_ActivityID;
        IF RC1%ISOPEN THEN CLOSE RC1; END IF;
        OPEN RC1 FOR 
        SELECT	DISTINCT ---- BGTRK00059887
                RTRIM(a.ACTIVITYID) ACTIVITYID ---- BGTRK00047091
                ,RTRIM(Frp.PartnerName) FrPartnerName
                ,FRPARTNERIDHDR
                ,RTRIM(Top.PartnerName) ToPartnerName
                ,TOPARTNERIDHDR
                ,RTRIM(FRPROTOCOLNAME) FRPROTOCOLNAME
                ,RTRIM(FRPROTOCOLVER) FRPROTOCOLVER
                ,RTRIM(TOPROTOCOLNAME) TOPROTOCOLNAME
                ,RTRIM(TOPROTOCOLVER) TOPROTOCOLVER
                ,RTRIM(FN_GET_RTOBJNAME(a.FrPROCESSCD, a.FrPROCESSVER, 'Process')) FrProcessName
                ,RTRIM(FRPROCESSCD) FRPROCESSCD
                ,RTRIM(FRPROCESSVER) FRPROCESSVER
                ,RTRIM(FN_GET_RTOBJNAME(a.ToPROCESSCD, a.ToPROCESSVER, 'Process')) ToProcessName
                ,RTRIM(TOPROCESSCD) TOPROCESSCD
                ,RTRIM(TOPROCESSVER) TOPROCESSVER
                ,RCVDOCTS
                ,RTRIM(DOCID) DOCID                
                ,RTRIM(FRPACKAGENAME) FRPACKAGENAME
                ,RTRIM(FRPACKAGEVER) FRPACKAGEVER
                ,RTRIM(TOPACKAGENAME) TOPACKAGENAME
                ,RTRIM(TOPACKAGEVER) TOPACKAGEVER
                ,RTRIM(fradtl.vuid) frVuid
                ,RTRIM(toadtl.vuid) ToVuid
			,RTRIM(fradtl.ORIGINALFILENAME) frFILENAME
			,RTRIM(toadtl.ORIGINALFILENAME) toFILENAME
			,RTRIM(fradtl.ENCODING) frENCODING
			,RTRIM(toadtl.ENCODING) toENCODING
                ,ae.EndStateTS
                ,nvl(ae.EndStateFlg,0) EndStateFlg                                
        FROM 
                LG_Synch_Req_Resp S    
                Inner join  LG_ACTIVITY a on (a.ACTIVITYID = S.VUID_RESP or a.ACTIVITYID = S.ACTIVITYID)
                Left Outer Join PR_Partner Frp                   ON a.FRPARTNERIDHDR = Frp.PartnerID
                Left Outer Join PR_Partner Top                   ON a.ToPARTNERIDHDR = Top.PartnerID
                left outer join lg_activity_endState ae          ON a.activityId     =  ae.activityId
                left outer join lg_activity_dtl_parent_vw fradtl ON a.activityId     = fradtl.activityId
                left outer join lg_activity_dtl_child_vw  toadtl ON a.activityId     = toadtl.activityId
                
        Where 
                s.ACTIVITYID = p_Vuid or s.VUID_RESP = p_Vuid
        Order By RCVDOCTS;
        
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);      
    
    Exception
      WHEN NO_DATA_FOUND THEN
         p_ReturnCd := 3;
      WHEN OTHERS THEN
        p_ReturnCd := 1;
        v_ExceptionFlg := 1;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
        RAISE_APPLICATION_ERROR (-20999,'20001 Get_ActivityID_From_VUID'||v_ProcName ||' '|| SQLCODE|| ' '||SQLERRM);                    
    
End LG_ACTIVITYS_SYNCH;
/


-- LG_ACTIVITYS_VUID.sql

CREATE OR REPLACE PROCEDURE         LG_ActivityS_Vuid (
      p_ReturnCd       out   pls_INTEGER
    , p_DebugFlg       in    pls_INTEGER
    , RC1              out  db_ref.rct          -- rows
    , p_Vuid           in   varchar2
    )
AS
/*
================================================================================
Proc Name:      LG_ActivityS_Vuid
Create Date:    04/18/2002
Author          Vipin Goel
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
Procedure ProcAudit is
v_diffTS Timestamp;
begin
  DB_ProcAuditLogI_ah1('LG_ActivityS_Vuid',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'||','||p_Vuid);
End ProcAudit;
--
-- Main
--
BEGIN
    p_ReturnCd := 0;
    if RC1%IsOpen then close RC1; end if;
    open RC1 for select
    a.FRPARTNERIDHdr  FRPARTNERID,
    a.FRBUSINESSIDHDR FRBUSINESSID,
    frp.partnername   FRPARTNERNAME,
    a.TOPARTNERIDHdr  TOPARTNERID,
    a.ToBUSINESSIDHdr ToBUSINESSID ,
    Top.partnername   TOPARTNERNAME,
    a.FRPROTOCOLNAME,
    a.FrPROTOCOLVer,
    a.FrProcessCD,
    a.FrProcessVer,
    a.DocId,
    a.DocTS,
    a.RcvDocTS,
    ae.EndStateTS,
    ae.EndStateFlg,
    dtl.MSGRAWLENGTH,
    dtl.MSGCONTENTLENGTH
    ,dtl.ORIGINALFILENAME
    ,dtl.ENCODING
--    dtl.msgsize
    From lg_activity a
    Left Outer JOIN lg_activity_dtl dtl     ON a.activityID     = dtl.activityID
    Left Outer JOIN lg_activity_endState ae ON a.activityID     = ae.activityID
    LEFT OUTER JOIN pr_partner frp     ON a.FRPARTNERIDHdr = frp.partnerId
    LEFT OUTER JOIN pr_partner top     ON a.TOPARTNERIDHdr = top.partnerId
    where dtl.vuid  = p_Vuid
    ;
-- proc Audit
  ProcAudit;
Exception
 when others then
  ProcAudit;
  p_ReturnCd := 1;
  Raise_application_error (-20999,'20000 - LG_ActivityS_Vuid ' || SQLCODE|| ' '||SQLERRM);
END ;
/


-- LG_ACTIVITYS_VUID2.sql

CREATE OR REPLACE PROCEDURE         LG_ActivityS_Vuid2 (
   p_ReturnCD     out pls_INTEGER
   , p_DebugFlg         in  pls_INTEGER
   , RC1           out db_ref.rct          -- rows
   , p_Vuid        in  varchar2
    )
AS
/*
================================================================================
    Proc Name:      LG_ActivityS_Vuid
    Create Date:    2003-01-13
    Author          Vipin Goel
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_beginTS          timestamp(6):= FN_Get_Date;
v_ProcName          varchar2(30) := 'LG_ActivityS_Vuid2';
v_ExceptionFlg      integer := 0;
v_ParamsTxt         varchar2(4000) :=
                                'RETURN_CODE'||','||p_DebugFlg||
                                    ','||'RC1'||','||p_Vuid;

--
-- Main
--
BEGIN
    p_ReturnCD := 0;
    if RC1%IsOpen then close RC1; end if;
    open RC1 for select
    a.activityID,
    a.FRPARTNERIDHdr  FRPARTNERID,
    a.FRBUSINESSIDHDR FRBUSINESSID,
    frp.partnername   FRPARTNERNAME,
    a.TOPARTNERIDHdr  TOPARTNERID,
    a.ToBUSINESSIDHdr ToBUSINESSID ,
    Top.partnername   TOPARTNERNAME,
    a.FRPROTOCOLNAME,
    a.FrPROTOCOLVer,
    a.FrProcessCD,
    a.FrProcessVer,
    a.TOPROTOCOLNAME,
    a.TOPROTOCOLVer,
    a.ToPROCESSCD,
    a.TOPROCESSVer,
    a.DocId,
    a.DocTS,
    a.RcvDocTS,
--
   ae.EndStateTS,
   ae.EndStateFlg,
   ae.DMDURATION,
   ae.TRANSPORTDURATION,
   ae.TRANSPORTATTEMPTCNT,
   ae.GATEWAYATTEMPTCNT,
--
   RTRIM(FN_GET_RTOBJNAME(a.FrPRocessCd, a.FrProcessVer, 'Process')) FrProcessName,
   RTRIM(FN_GET_RTOBJNAME(a.ToPRocessCd, a.ToProcessVer, 'Process')) ToProcessName,
--
    dtl.MSGRAWLENGTH,
    dtl.MSGCONTENTLENGTH
    ,dtl.ORIGINALFILENAME
    ,dtl.ENCODING
--
    From lg_activity a
    Left Outer JOIN lg_activity_dtl dtl     ON a.activityID     = dtl.activityID
    Left Outer JOIN lg_activity_endState ae ON a.activityID     = ae.activityID
    LEFT OUTER JOIN pr_partner frp      ON a.FRPARTNERIDHdr = frp.partnerId
    LEFT OUTER JOIN pr_partner top      ON a.TOPARTNERIDHdr = top.partnerId
    where dtl.vuid  = p_Vuid;
   DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   Exception
   when others then
       DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
       p_ReturnCD := 1;
       Raise_application_error (-20999,'20000 - LG_ActivityS_Vuid ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_ACTIVITY_DTLI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITY_DTLI_AH1
(
    p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,p_ACTIVITYID in VARCHAR2
    ,p_VUID in VARCHAR2
    ,p_HDRFLG in NUMBER
    ,p_MSGRAWLENGTH in NUMBER
    ,p_MSGCONTENTLENGTH in NUMBER
    ,p_DOCTYPECD in VARCHAR2
    ,p_SYNCHRONOUSFLG in NUMBER
	,p_GATEWAYID in NUMBER
	,p_DOCUMENTDIRECTION in NUMBER
    ,p_ORIGINALFILENAME in VARCHAR2
    ,p_ENCODING in VARCHAR2
    ,p_MsgStoreReqd  in NUMBER
) AS
/*
================================================================================
Proc Name:  LG_ACTIVITY_DTLI_AH1
Create Date:  2002-11-11 09:34:31
Author:   SYSTEM
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-06-28 S.Priani	     4.3	Added GATEWAYID, DOCUMENTDIRECTION
    2008-01-23 BHARANI J     6.1.1      CR102199
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_ExceptionFlg      Integer :=0;
v_RowCnt            Integer :=0;
v_ProcName          varchar2(30) := 'LG_ACTIVITY_DTLI_AH1';
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
v_RowTS             integer :=0;
zeroUpdate          Exception;

Procedure ProcAudit is
begin
 --  If p_DebugFlg =1 Then
       DB_ProcAuditLogI_ah1(v_ProcName,v_beginTS,
       'p_ReturnCd'||','||p_DebugFlg
       ||','||p_ACTIVITYID
       ||','||p_VUID
       ||','||p_HDRFLG
       ||','||p_MSGRAWLENGTH
       ||','||p_MSGCONTENTLENGTH
       ||','||p_DOCTYPECD
       ||','||p_GATEWAYID
       ||','||p_DOCUMENTDIRECTION
	||','||p_ORIGINALFILENAME
	||','||p_ENCODING
	||','||p_MsgStoreReqd
       );
 -- End if ;
End ProcAudit;

Procedure DoUpdate is
begin
        Update LG_ACTIVITY_DTL d
        Set
            HdrFlg = p_HdrFlg
            , d.MsgRawLength = case when p_MSGRAWLENGTH > 0 then p_MSGRAWLENGTH     Else d.MSGRAWLENGTH      End
            , d.MsgContentLength = case when p_MSGCONTENTLENGTH > 0 then p_MSGCONTENTLENGTH Else d.MSGCONTENTLENGTH  End
            , d.DocTypeCd = p_DocTypeCd
            , d.SynchronousFlg = COALESCE(p_SYNCHRONOUSFLG, SYNCHRONOUSFLG)
		, d.GateWayId = COALESCE(p_GATEWAYID, GATEWAYID)
		, d.DocumentDirection = COALESCE(p_DOCUMENTDIRECTION ,DOCUMENTDIRECTION)
            , d.OriginalFileName = COALESCE(p_OriginalFileName, OriginalFileName)
            , d.Encoding = COALESCE(p_Encoding, Encoding)
            , d.RowTS = d.RowTS + 1
            , d.UpdateDate = v_BeginTS
            , d.MessageStoreRequired=p_MsgStoreReqd
        Where d.ACTIVITYID = p_ACTIVITYID and d.VUID = p_VUID;
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate; end if;
        p_ReturnCd := 0;
        ProcAudit;

        EXCEPTION
            WHEN ZeroUpdate then
                p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' DoUpdate '||SQLERRM);
end DoUpdate;

----------------------------------------------------------------------
--                             M  A  I  N
----------------------------------------------------------------------
BEGIN

 DoUpdate;

 if (v_RowCnt = 0) then
     Insert Into LG_ACTIVITY_DTL D
      (
           D.ACTIVITYID
       ,   D.VUID
       ,   D.HDRFLG
       ,   D.MSGRAWLENGTH
       ,   D.MSGCONTENTLENGTH
       ,   D.DOCTYPECD
       ,   D.CREATEDATE
       ,   D.UPDATEDATE
       ,   D.SYNCHRONOUSFLG
	 ,	D.GATEWAYID
	 ,	D.DOCUMENTDIRECTION
       ,   D.ORIGINALFILENAME
       ,   D.ENCODING
       ,   D.MessageStoreRequired
       ) Values (
           p_ACTIVITYID
       ,   p_VUID
       ,   p_HDRFLG
       ,   p_MSGRAWLENGTH
       ,   p_MSGCONTENTLENGTH
       ,   p_DOCTYPECD
       ,   v_BeginTS
       ,   v_BeginTS
       ,   p_SYNCHRONOUSFLG
	 ,	p_GATEWAYID
	 ,	p_DOCUMENTDIRECTION
       ,   p_ORIGINALFILENAME
       ,   p_ENCODING
       ,   p_MsgStoreReqd
     );
 end if;

 p_ReturnCd := 0;
 ProcAudit;

EXCEPTION
   When Dup_val_on_index Then
      DoUpdate;
   When NO_DATA_FOUND Then Null;
   When Others Then
       ProcAudit;
       Raise_application_error (-20999,'20000 LG_ACTIVITY_DTLI_AH1 - Insert '||SQLERRM);
       p_ReturnCd := 1;
END LG_ACTIVITY_DTLI_AH1;
/



-- LG_ACTIVITY_ENDSTATEI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITY_ENDSTATEI_AH1
(
    p_ReturnCd out pls_integer
    ,p_DebugFlg             in pls_integer
    ,p_ACTIVITYID          in VARCHAR2
    ,p_ENDSTATETS          in TimeStamp
    ,p_ENDSTATEFLG         in NUMBER
    ,p_DMDURATION          in NUMBER
    ,p_TRANSPORTDURATION   in NUMBER
    ,p_TRANSPORTATTEMPTCNT in NUMBER
    ,p_GATEWAYATTEMPTCNT   in NUMBER
    ,p_RcvDocTS            in TimeStamp
    ,p_FrIPAddress         in VARCHAR2
    ,p_ProvSessionID       in pls_integer
    ,p_ENDSTATETYPECD      in pls_integer
	,p_FAILINGCOMPONENT	in	NUMBER

) as

/*
================================================================================
Proc Name:        LG_ACTIVITY_ENDSTATEI_AH1
Create Date:     2002-11-13 15:43:50
Author:            SYSTEM
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-03-11 S.Priani	     4.2.2  Replaced call to Summary procs by Insert
    2004-06-28 S.Priani	     4.3	Added FAILINGCOMPONENT, update first then insert
 ================================================================================
Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
v_returnCD pls_integer :=0;
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_ACTIVITY_ENDSTATEI_AH1 ',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||p_ACTIVITYID
    ||','||p_ENDSTATEFLG
    ||','||p_ENDSTATETS
    ||','||p_DMDURATION
    ||','||p_TRANSPORTDURATION
    ||','||p_TRANSPORTATTEMPTCNT
    ||','||p_GATEWAYATTEMPTCNT
    --
    ||','||p_RcvDocTS
    ||','||p_FrIPAddress
    ||','||p_ProvSessionID
    ||','||p_ENDSTATETYPECD
    ||','||p_FAILINGCOMPONENT
    );

End ProcAudit;
--
Procedure DoUpdate IS
v_RowTS      integer :=0;
BEGIN
	v_RowCnt := 0;
	Update LG_ACTIVITY_ENDSTATE
	SET
		ENDSTATEFLG = COALESCE ( p_ENDSTATEFLG, ENDSTATEFLG)
		,ENDSTATETS = COALESCE ( p_ENDSTATETS, ENDSTATETS)
		,DMDURATION = COALESCE ( p_DMDURATION, DMDURATION)
		,TRANSPORTDURATION = COALESCE ( p_TRANSPORTDURATION, TRANSPORTDURATION)
		,TRANSPORTATTEMPTCNT = COALESCE ( p_TRANSPORTATTEMPTCNT, TRANSPORTATTEMPTCNT)
		,GATEWAYATTEMPTCNT = COALESCE ( p_GATEWAYATTEMPTCNT, GATEWAYATTEMPTCNT)
		,ENDSTATETYPECD = COALESCE ( p_ENDSTATETYPECD, ENDSTATETYPECD)
		,FAILINGCOMPONENT = COALESCE ( p_FAILINGCOMPONENT, FAILINGCOMPONENT)
	Where
		ACTIVITYID = p_ACTIVITYID
		;
	v_RowCnt := SQL%rowcount;
End DoUpdate;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
DoUpdate;

if (v_RowCnt = 0) then

Insert into LG_ACTIVITY_ENDSTATE(
ACTIVITYID
,    ENDSTATEFLG
,    ENDSTATETS
,    DMDURATION
,    TRANSPORTDURATION
,    TRANSPORTATTEMPTCNT
,    GATEWAYATTEMPTCNT
,    ENDSTATETYPECD
,	FAILINGCOMPONENT
) Values (
p_ACTIVITYID
,    p_ENDSTATEFLG
,    p_ENDSTATETS
,    p_DMDURATION
,    p_TRANSPORTDURATION
,    p_TRANSPORTATTEMPTCNT
,    p_GATEWAYATTEMPTCNT
,    p_ENDSTATETYPECD
,	p_FAILINGCOMPONENT
);

end if;
--
-- Run summary proc
--
BEGIN
INSERT INTO LG_UNPROCESSED_DOCPROC
	(RCVDOCTS
	,FRIPADDRESS
	,PROVSESSIONID
	,RCVDOCCNT
	,PERSISTDOCCNT
	,PROCESSSTARTCNT
	,PROCESSENDCNT
	,SUCCESSCNT
	,FAILCNT
	)
VALUES
	(P_RCVDOCTS
	,P_FRIPADDRESS
	,P_PROVSESSIONID
	,0
	,0
	,0
	,0
	,case when p_ENDSTATEFLG = 1 Then 1 else 0 end   -- success P_SUCCESSCNT
	,case when p_ENDSTATEFLG = 2 Then 1 else 0 end);   -- Failure P_FAILCNT
EXCEPTION
	WHEN Others THEN
		p_ReturnCd := 1;
		ProcAudit;
		Raise_application_error (-20999,'20000 LG_ACTIVITY_ENDSTATEI_AH1 - Ins Unprocessed ' || SQLCODE|| ' '||SQLERRM);
END;
ProcAudit;
EXCEPTION
    When NO_DATA_FOUND Then Null;
    When Dup_val_on_index then p_returnCd := 2;
    When Others Then
    ProcAudit;
    Raise_application_error (-20999,'20000 LG_ACTIVITY_ENDSTATEI_AH1  ' || SQLCODE|| ' '||SQLERRM);
    p_ReturnCd := 1;
END LG_ACTIVITY_ENDSTATEI_AH1 ;
--/
/


-- LG_ACTIVITY_ENDSTATES_AH1.sql

CREATE OR REPLACE PROCEDURE lg_activity_endstates_ah1 (
   p_returncd      OUT      PLS_INTEGER,
   p_debugflg      IN       PLS_INTEGER,
   rc1             OUT      db_ref.rct,
   p_endstateflg   IN       PLS_INTEGER
)
AS
/*
   ================================================================================
   Proc Name:       LG_ACTIVITY_ENDSTATES_AH1
   Create Date:     2006-06-03 12:34:36
   Author:          Shishir N

                      ::::::::::::::::::::::::::::::
                            M O D I F I C A T I O N
                      ::::::::::::::::::::::::::::::

   ---------- ------------- ------ ---------------------------------------------
   DATE       NAME          Ver    DESCRIPTION
   ---------- ------------- ------ ---------------------------------------------
   2006-06-03 Shishir N        6.1  Created

   ================================================================================
   Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================*/
   v_rowcnt         INTEGER         DEFAULT 0;
   v_begints        TIMESTAMP       DEFAULT fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   DEFAULT 'LG_ACTIVITY_ENDSTATES_AH1';
   v_exceptionflg   INTEGER         DEFAULT 0;
   v_paramstxt      VARCHAR2 (4000)
      DEFAULT    'p_DebugFlg=>'
              || p_debugflg
              || CHR (10)
              || ', p_ENDSTATEFLG=>'
              || p_endstateflg
              || CHR (10);
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
   IF rc1%ISOPEN
   THEN
      CLOSE rc1;
   END IF;

   IF p_endstateflg IS NOT NULL
   THEN
      OPEN rc1 FOR
         SELECT activityid, endstateflg, createdate, endstatets, dmduration,
                transportduration, transportattemptcnt, gatewayattemptcnt,
                endstatetypecd, failingcomponent
           FROM lg_activity_endstate
          WHERE endstateflg = p_endstateflg;
   ELSE
      OPEN rc1 FOR
         SELECT activityid, endstateflg, createdate, endstatets, dmduration,
                transportduration, transportattemptcnt, gatewayattemptcnt,
                endstatetypecd, failingcomponent
           FROM lg_activity_endstate;
   END IF;

   p_returncd := 0;
   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
END lg_activity_endstates_ah1;
/

-- LG_ACTIVITY_EVENTS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITY_EVENTS_AH1
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        , p_ACTIVITYID IN VARCHAR2
        , p_VUID IN VARCHAR2
        , p_EVENTID IN VARCHAR2
        )
AS

/*
        ================================================================================
        Proc Name:       LG_ACTIVITY_EVENTS_AH1
        Create Date:     2003-07-02 14:21:33
        Author:          SYSTEM

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-07-02 SYSTEM           3.5 Created
        2008-10-20 Amit vijay	    6.2	Defect 107019

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_where           varchar2(1000);
v_RowCnt          integer        default 0;
v_BeginTS         timestamp      default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30)   default 'LG_ACTIVITY_EVENTS_AH1';
v_ExceptionFlg    integer        default 0;
v_ParamsTxt       varchar2(4000) default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
                ||', p_VUID=>'||p_VUID||CHR(10)
                ||', p_EVENTID=>'||p_EVENTID||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    v_where := ' Where 1 = 1 '
        ||FN_BUILD_WHERE('ACTIVITYID', 'VARCHAR2', p_ACTIVITYID)
        ||FN_BUILD_WHERE('VUID',       'VARCHAR2', p_VUID)
        ||FN_BUILD_WHERE('EVENTID',    'VARCHAR2', p_EVENTID);

    OPEN RC1 FOR
    'SELECT
            RTRIM(ACTIVITYID) ACTIVITYID
            ,RTRIM(VUID) VUID
            ,RTRIM(EVENTID) EVENTID
            ,CREATEDATE
    FROM
            LG_EVENT'||v_where||' and ACTIVITYID IS NOT NULL'
    ;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_ACTIVITY_EVENTS_AH1;
/


-- LG_ACTIVITY_EVENTS_AH2.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITY_EVENTS_AH2
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        , p_ACTIVITYID IN VARCHAR2
        , p_VUID IN VARCHAR2
        , p_EVENTID IN VARCHAR2
        )
AS

/*
        ================================================================================
        Proc Name:       LG_ACTIVITY_EVENTS_AH2
        Create Date:     2007-09-15 14:21:33
        Author:          AMIT KUMAR VIJAY

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2007-09-21 AMIT KUMAR    6.1.1   Created
        2008-10-20 Amit vijay	 6.2	 Defect 107019

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_where           varchar2(1000);
v_where1           varchar2(1000);
v_RowCnt          integer        default 0;
v_BeginTS         timestamp      default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30)   default 'LG_ACTIVITY_EVENTS_AH2';
v_ExceptionFlg    integer        default 0;
v_ParamsTxt       varchar2(4000) default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
                ||', p_VUID=>'||p_VUID||CHR(10)
                ||', p_EVENTID=>'||p_EVENTID||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    v_where := ' Where 1 = 1 '
        ||FN_BUILD_WHERE('ACTIVITYID', 'VARCHAR2', p_ACTIVITYID)
        ||FN_BUILD_WHERE('VUID',       'VARCHAR2', p_VUID)
        ||FN_BUILD_WHERE('EVENTID',    'VARCHAR2', p_EVENTID);
    
    v_where1 := ' Where 1 = 1 '
    	||FN_BUILD_WHERE('X.EVENTID',    'VARCHAR2', p_EVENTID);

    OPEN RC1 FOR
    'SELECT
            RTRIM(ACTIVITYID) ACTIVITYID
            ,RTRIM(VUID) VUID
            ,RTRIM(EVENTID) EVENTID
            ,CREATEDATE
    FROM
            LG_EVENT'||v_where ||' and ACTIVITYID IS NOT NULL'||
              
              'UNION '||
	      'SELECT RTRIM(L.VUID) ACTIVITYID ' ||
                     ',RTRIM(L.VUID) VUID '       ||
                     ',RTRIM(X.EVENTID) EVENTID ' ||
                     ',lCREATEDATE '             ||
                     'FROM   LG_EVENT_FTP L, LG_EVENT_FTP_XREF X'        || ' WHERE L.FTPEVENTID=X.FTPEVENTID '|| v_where1
            ;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_ACTIVITY_EVENTS_AH2;
/


-- LG_ACTIVITY_MERGE.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITY_MERGE
(
    p_ReturnCd in out pls_integer
    , p_DebugFlg in         pls_integer
    ,p_ACTIVITYID        in VARCHAR2
    ,p_INITBUSINESSIDHDR in out VARCHAR2
    ,p_FRBUSINESSIDHDR   in out VARCHAR2
    ,p_TOBUSINESSIDHDR   in out VARCHAR2
    ,p_INITPARTNERIDHDR  in NUMBER
    ,p_FRPARTNERIDHDR    in NUMBER
    ,p_TOPARTNERIDHDR    in NUMBER
    ,p_INITBUSINESSIDBDY in out VARCHAR2
    ,p_FRBUSINESSIDBDY   in out VARCHAR2
    ,p_TOBUSINESSIDBDY   in out VARCHAR2
    ,p_INITPARTNERIDBDY  in NUMBER
    ,p_FRPARTNERIDBDY    in NUMBER
    ,p_TOPARTNERIDBDY    in NUMBER
    ,p_FRPROTOCOLNAME    in out VARCHAR2
    ,p_TOPROTOCOLNAME    in out VARCHAR2
    ,p_FRPROTOCOLVER     in out VARCHAR2
    ,p_TOPROTOCOLVER     in out VARCHAR2
    ,p_FRPROCESSCD       in out VARCHAR2
    ,p_TOPROCESSCD       in out VARCHAR2
    ,p_FRPROCESSVER      in out VARCHAR2
    ,p_TOPROCESSVER      in out VARCHAR2
    ,p_RCVDOCTS          in Timestamp
    ,p_DOCID             in out VARCHAR2
    ,p_DOCTS             in out VARCHAR2
    ,p_CHANNELID         in NUMBER
    ,p_CHANNELSTATUSName in VARCHAR2
    ,p_FRIPADDRESS       in out VARCHAR2
    ,p_DestName          in Varchar2
    --,p_DestTypeId        in pls_integer
    ,p_PersistDocTS      in Timestamp
    ,p_ProcessStartDocTS in Timestamp
    ,p_ProcessEndDocTS   in Timestamp
    ,P_PROVSESSIONID     in pls_integer
    --4/16/2003
    ,P_ACTIVITYTYPECD   in VARCHAR2  -- BATCH, DOC
    ,P_FRPACKAGENAME    in VARCHAR2  -- AS2, AS1, Null
    ,P_TOPACKAGENAME    in VARCHAR2  -- AS2, AS1, Null
    ,P_FRPACKAGEVER     in VARCHAR2
    ,P_TOPACKAGEVER     in VARCHAR2
    ,P_FRPACKAGECD      in VARCHAR2  -- AS, Null
    ,P_TOPACKAGECD      in VARCHAR2  -- AS, Null
    ,P_FRCONTENTCD      in VARCHAR2  -- RN, EDI, FF
    ,P_TOCONTENTCD      in VARCHAR2  -- RN, EDI, FF
    ,p_FRTRANSPORTTYPE  in VARCHAR2
    ,p_TOTRANSPORTTYPE  in VARCHAR2
    ,P_RCVDOCCNT        in pls_integer
    ,P_EVENTCD          in pls_integer
    ,P_MSGRAWLENGTH     in NUMBER
    ,P_MSGCONTENTLENGTH in NUMBER
    ,p_SYNCHRONOUSFLG   in pls_integer -- 1 = sync, 0 = Not sync
    ,p_ActionFlg        in pls_integer -- 1 = Action, 0 = Signal
    ,p_insertFlg        in pls_integer -- 1= insert first, 0=update first
    -- 56000 New fields added to the procedure
    ,p_USER01		in VARCHAR2
    ,p_USER02		in VARCHAR2
    ,p_USER03		in VARCHAR2
    ,p_USER04		in VARCHAR2
    ,p_USER05		in VARCHAR2
    ,p_USER06		in VARCHAR2
    ,p_USER07		in VARCHAR2
    ,p_USER08		in VARCHAR2
    ,p_USER09		in VARCHAR2
    ,p_USER10		in VARCHAR2    
) as
/*
================================================================================
Proc Name:      LG_ACTIVITY_merge
Create Date:    01/11/02 05:55:54 PM
Author          SYSTEM
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------    ---------------------------------------------
    DATE       NAME          Ver       DESCRIPTION
    ---------- ------------- ------    ---------------------------------------------
    2004-03-11 S.Priani	     4.2.2     Replaced call to Summary procs by Insert
    2006-02-20 S.Nair	     6.1.0     56000
    
 ================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
p_BUSINESSIDTYPE pls_integer ; --*************** temp ******************

    v_destTypeID     PLS_INTEGER default FN_GET_DESTTYPEID (p_DestName);
    v_rowcnt integer    :=0;
    v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
    v_pk_value number := 0;
    v_returnCD pls_integer;
    v_ProcessStartCnt pls_integer := 0;
    v_ProcessEndCnt   pls_integer := 0;
    v_InsertDup       varchar2(3);  -- yes/no

    v_INITPARTNERIDHDR  pr_partner.partnerid%type;
    v_FRPARTNERIDHDR    pr_partner.partnerid%type;
    v_TOPARTNERIDHDR    pr_partner.partnerid%type;
-- for length check
    v_ACTIVITYID        LG_ACTIVITY.ACTIVITYID%Type;
    v_INITBUSINESSIDHDR LG_ACTIVITY.INITBUSINESSIDHDR%Type;
    v_FRBUSINESSIDHDR   LG_ACTIVITY.FRBUSINESSIDHDR%Type;
    v_TOBUSINESSIDHDR   LG_ACTIVITY.TOBUSINESSIDHDR%Type;
    v_INITBUSINESSIDBDY LG_ACTIVITY.INITBUSINESSIDBDY%Type;
    v_FRBUSINESSIDBDY   LG_ACTIVITY.FRBUSINESSIDBDY%Type;
    v_TOBUSINESSIDBDY   LG_ACTIVITY.TOBUSINESSIDBDY%Type;
    v_INITPARTNERIDBDY  LG_ACTIVITY.INITPARTNERIDBDY%Type;
    v_FRPROTOCOLNAME    LG_ACTIVITY.FRPROTOCOLNAME%Type;
    v_TOPROTOCOLNAME    LG_ACTIVITY.TOPROTOCOLNAME%Type;
    v_FRPROTOCOLVER     LG_ACTIVITY.FRPROTOCOLVER%Type;
    v_TOPROTOCOLVER     LG_ACTIVITY.TOPROTOCOLVER%Type;
    v_FRPROCESSCD       LG_ACTIVITY.FRPROCESSCD%Type;
    v_TOPROCESSCD       LG_ACTIVITY.TOPROCESSCD%Type;
    v_FRPROCESSVER      LG_ACTIVITY.FRPROCESSVER%Type;
    v_TOPROCESSVER      LG_ACTIVITY.TOPROCESSVER%Type;
    v_DOCID             LG_ACTIVITY.DOCID%Type;
    v_DOCTS             LG_ACTIVITY.DOCTS%Type;
    v_FRIPADDRESS       LG_ACTIVITY.FRIPADDRESS%Type;
    v_CHANNELSTATUSName Varchar2(100) ; --PR_CHANNEL_STATUS.STATUSNAME%type;
    v_channelStatusId   pls_integer ; --PR_CHANNEL_STATUS.STATUSID%type;
    zeroUpdate          Exception;
    v_ProcName          varchar2(30) := 'LG_ACTIVITY_MERGE';
--
-- ProcAudit
--
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_ah1('LG_ACTIVITY_merge',v_beginTS,
    'Return_code'||','||p_DebugFlg||','||p_ACTIVITYID||','||p_INITBUSINESSIDHDR||','||p_FRBUSINESSIDHDR||','||
    p_TOBUSINESSIDHDR||','||p_INITPARTNERIDHDR||','||p_FRPARTNERIDHDR||','||p_TOPARTNERIDHDR||','||
    p_INITBUSINESSIDBDY||','||p_FRBUSINESSIDBDY||','||p_TOBUSINESSIDBDY||','||p_INITPARTNERIDBDY||','||
    p_FRPARTNERIDBDY||','||p_TOPARTNERIDBDY||','||p_FRPROTOCOLNAME||','||
    p_TOPROTOCOLNAME||','||p_FRPROTOCOLVER||','||p_TOPROTOCOLVER||','||p_FRPROCESSCD
    ||','||p_TOPROCESSCD||','||p_FRPROCESSVER||','||p_TOPROCESSVER||','||p_RCVDOCTS||','||
    p_DOCID||','||p_DOCTS||','||p_CHANNELID
    ||','||p_CHANNELSTATUSNAME
    ||','||p_FRIPADDRESS
    ||','||p_DestName
    ||','||p_PersistDocTS
    ||','||p_ProcessStartDocTS
    ||','||p_ProcessEndDocTS
    ||','||P_PROVSESSIONID
    --
    ||','||P_ACTIVITYTYPECD
    ||','||P_FRPACKAGENAME
    ||','||P_TOPACKAGENAME
    ||','||P_FRPACKAGEVER
    ||','||P_TOPACKAGEVER
    ||','||P_FRPACKAGECD
    ||','||P_TOPACKAGECD
    ||','||P_FRCONTENTCD
    ||','||P_TOCONTENTCD
    ||','||p_FRTRANSPORTTYPE
    ||','||p_TOTRANSPORTTYPE
    ||','||P_RCVDOCCNT
    ||','||P_EVENTCD
    ||','||P_MSGRAWLENGTH
    ||','||P_MSGCONTENTLENGTH
    ||','||p_SYNCHRONOUSFLG
    ||','||p_ActionFlg
    ||','||p_insertFlg
    ||','||p_USER01
    ||','||p_USER02
    ||','||p_USER03
    ||','||p_USER04
    ||','||p_USER05
    ||','||p_USER06
    ||','||p_USER07
    ||','||p_USER08
    ||','||p_USER09
    ||','||p_USER10
    );
End ProcAudit;
--
-- Get partnerid from businessid
--
Function get_partnerid_from_businessid(p_businessId in varchar2) return number is
v_partnerid    pr_busIdentifier.partnerid%type;
begin
 begin
   select partnerid into v_partnerid  from pr_busIdentifier
   where businessid = p_businessid
   and   BUSINESSIDTYPE = nvl(p_BUSINESSIDTYPE,1) ;
  exception
   when no_data_found then v_partnerid := -1;
   When Others Then
   ProcAudit;
   Raise_application_error (-20999,'20002-LG_ACTIVITY_merge-get_partnerid_from_businessid-' || SQLCODE|| ' '||SQLERRM);
 end;
 return v_partnerid;
end get_partnerid_from_businessid;
--
Procedure Check_length is
begin
   db_LengthCheck_activity (
     P_INITBUSINESSIDHDR   => p_INITBUSINESSIDHDR
   , P_FRBUSINESSIDHDR     => p_FRBUSINESSIDHDR
   , P_TOBUSINESSIDHDR     => p_TOBUSINESSIDHDR
   , P_INITBUSINESSIDBDY   => p_INITBUSINESSIDBDY
   , P_FRBUSINESSIDBDY     => p_FRBUSINESSIDBDY
   , P_TOBUSINESSIDBDY     => p_TOBUSINESSIDBDY
   , P_FRPROTOCOLNAME      => p_FRPROTOCOLNAME
   , P_TOPROTOCOLNAME      => p_TOPROTOCOLNAME
   , P_FRPROTOCOLVER       => p_FRPROTOCOLVER
   , P_TOPROTOCOLVER       => p_TOPROTOCOLVER
   , P_FRPROCESSCD         => p_FRPROCESSCD
   , P_TOPROCESSCD         => p_TOPROCESSCD
   , P_FRPROCESSVER        => p_FRPROCESSVER
   , P_TOPROCESSVER        => p_TOPROCESSVER
   , P_DOCID               => p_DOCID
   , P_DOCTS               => p_DOCTS
   , P_FRIPADDRESS         => P_FRIPADDRESS
   ) ;

end check_length;

PROCEDURE Update_Lg_Activity AS
BEGIN
    Update Lg_Activity
      Set
           INITBUSINESSIDHDR   = p_INITBUSINESSIDHDR  -- p_INITPARTNERIDHDR
       ,   FRBUSINESSIDHDR     = p_FRBUSINESSIDHDR    -- p_FrPARTNERIDHDR
       ,   TOBUSINESSIDHDR     = p_TOBUSINESSIDHDR    -- p_ToPARTNERIDHDR
       ,   INITPARTNERIDHDR    = p_INITPARTNERIDHDR
       ,   FRPARTNERIDHDR      = p_FRPARTNERIDHDR
       ,   TOPARTNERIDHDR      = p_TOPARTNERIDHDR
       ,   INITBUSINESSIDBDY   = p_INITBUSINESSIDBDY
       ,   FRBUSINESSIDBDY     = p_FRBUSINESSIDBDY
       ,   TOBUSINESSIDBDY     = p_TOBUSINESSIDBDY
       ,   INITPARTNERIDBDY    = p_INITPARTNERIDBDY
       ,   FRPARTNERIDBDY      = p_FRPARTNERIDBDY
       ,   TOPARTNERIDBDY      = p_TOPARTNERIDBDY
       ,   FRPROTOCOLNAME      = p_FRPROTOCOLNAME
       ,   TOPROTOCOLNAME      = p_TOPROTOCOLNAME
       ,   FRPROTOCOLVER       = p_FRPROTOCOLVER
       ,   TOPROTOCOLVER       = p_TOPROTOCOLVER
       ,   FRPROCESSCD         = p_FRPROCESSCD
       ,   TOPROCESSCD         = p_TOPROCESSCD
       ,   FRPROCESSVER        = p_FRPROCESSVER
       ,   TOPROCESSVER        = p_TOPROCESSVER
       ,   RCVDOCTS            = nvl(p_RCVDOCTS, RCVDOCTS)
       ,   DOCID               = p_DOCID
       ,   DOCTS               = p_DOCTS
       ,   CHANNELID           = p_CHANNELID
       ,   CHANNELSTATUSID     = v_channelStatusId   --p_CHANNELSTATUSName
       ,   FRIPADDRESS         = nvl(p_FRIPADDRESS, FRIPADDRESS)
       ,   DestTypeId          = nvl(v_DestTypeId, DestTypeId)
       ,   PersistDocTS        = p_PersistDocTS
       ,   ProcessStartDocTS   = p_ProcessStartDocTS
       ,   ProcessEndDocTS     = p_ProcessEndDocTS
       ,   PROVSESSIONID       = nvl(p_PROVSESSIONID, PROVSESSIONID)
       ,   ACTIVITYTYPECD      = nvl(p_ACTIVITYTYPECD, ACTIVITYTYPECD) -- 4/16/2003
       ,   FRPACKAGENAME       = nvl(p_FRPACKAGENAME, FRPACKAGENAME)
       ,   TOPACKAGENAME       = nvl(p_TOPACKAGENAME, TOPACKAGENAME)
       ,   FRPACKAGEVER        = nvl(p_FRPACKAGEVER, FRPACKAGEVER)
       ,   TOPACKAGEVER        = nvl(p_TOPACKAGEVER, TOPACKAGEVER)
       ,   FRPACKAGECD         = nvl(p_FRPACKAGECD, FRPACKAGECD)
       ,   TOPACKAGECD         = nvl(p_TOPACKAGECD, TOPACKAGECD)
       ,   FRCONTENTCD         = nvl(p_FRCONTENTCD, FRCONTENTCD)
       ,   TOCONTENTCD         = nvl(p_TOCONTENTCD, TOCONTENTCD)
       ,   FRTRANSPORTTYPE     = nvl(p_FRTRANSPORTTYPE, FRTRANSPORTTYPE)
       ,   TOTRANSPORTTYPE     = nvl(p_TOTRANSPORTTYPE, TOTRANSPORTTYPE)
       ,   ActionFlg           = p_ActionFlg
       ,   USER01	       = nvl(p_USER01, USER01)
       ,   USER02	       = nvl(p_USER02, USER02)
       ,   USER03 	       = nvl(p_USER03, USER03)
       ,   USER04	       = nvl(p_USER04, USER04)
       ,   USER05	       = nvl(p_USER05, USER05)
       ,   USER06	       = nvl(p_USER06, USER06)
       ,   USER07	       = nvl(p_USER07, USER07)
       ,   USER08	       = nvl(p_USER08, USER08)
       ,   USER09	       = nvl(p_USER09, USER09)
       ,   USER10              = nvl(p_USER10, USER10)       
       ,   UPDATEDATE          = v_BeginTS
    Where ActivityID = p_activityID;
    v_RowCnt := SQL%rowcount;
--    if (v_RowCnt = 0) then  raise ZeroUpdate; end if;
    p_ReturnCd := 0;
    ProcAudit;
    EXCEPTION
--            WHEN ZeroUpdate then p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' Insert_LG_EVENTSUMMARY '||SQLERRM);
END Update_Lg_Activity;
--
Procedure Insert_LG_ACtivity is
Begin
        INSERT INTO LG_Activity
        (
           ACTIVITYID
        ,   INITBUSINESSIDHDR
        ,   FRBUSINESSIDHDR
        ,   TOBUSINESSIDHDR
        ,   INITPARTNERIDHDR
        ,   FRPARTNERIDHDR
        ,   TOPARTNERIDHDR
        ,   INITBUSINESSIDBDY
        ,   FRBUSINESSIDBDY
        ,   TOBUSINESSIDBDY
        ,   INITPARTNERIDBDY
        ,   FRPARTNERIDBDY
        ,   TOPARTNERIDBDY
        ,   FRPROTOCOLNAME
        ,   TOPROTOCOLNAME
        ,   FRPROTOCOLVER
        ,   TOPROTOCOLVER
        ,   FRPROCESSCD
        ,   TOPROCESSCD
        ,   FRPROCESSVER
        ,   TOPROCESSVER
        ,   RCVDOCTS
        ,   DOCID
        ,   DOCTS
        ,   CHANNELID
        ,   CHANNELSTATUSID
        ,   FRIPADDRESS
        ,   DestTypeId
        ,   PersistDocTS
        ,   ProcessStartDocTS
        ,   ProcessEndDocTS
        ,   PROVSESSIONID
        ,   ACTIVITYTYPECD
        ,   FRPACKAGENAME
        ,   TOPACKAGENAME
        ,   FRPACKAGEVER
        ,   TOPACKAGEVER
        ,   FRPACKAGECD
        ,   TOPACKAGECD
        ,   FRCONTENTCD
        ,   TOCONTENTCD
        ,   FRTRANSPORTTYPE
        ,   ToTRANSPORTTYPE
        ,   CREATEDATE
        ,   UPDATEDATE
        ,   ActionFlg
        -- 56000 New fields added
        ,   USER01
        ,   USER02
        ,   USER03
	,   USER04
	,   USER05
	,   USER06
        ,   USER07
        ,   USER08
        ,   USER09
        ,   USER10        
            )
      Values
       (
            p_ACTIVITYID
        ,   p_INITBUSINESSIDHDR
        ,   p_FRBUSINESSIDHDR
        ,   p_TOBUSINESSIDHDR
        ,   p_InitPARTNERIDHDR   -- p_INITPARTNERIDHDR
        ,   p_FrPARTNERIDHDR     -- p_FRPARTNERIDHDR
        ,   p_ToPARTNERIDHDR     -- p_TOPARTNERIDHDR
        ,   p_INITBUSINESSIDBDY
        ,   p_FRBUSINESSIDBDY
        ,   p_TOBUSINESSIDBDY
        ,   p_INITPARTNERIDBDY
        ,   p_FRPARTNERIDBDY
        ,   p_TOPARTNERIDBDY
        ,   p_FRPROTOCOLNAME
        ,   p_TOPROTOCOLNAME
        ,   p_FRPROTOCOLVER
        ,   p_TOPROTOCOLVER
        ,   p_FRPROCESSCD
        ,   p_TOPROCESSCD
        ,   p_FRPROCESSVER
        ,   p_TOPROCESSVER
        ,   p_RCVDOCTS         -- date
        ,   p_DOCID
        ,   p_DOCTS
        ,   p_CHANNELID        -- number
        ,   v_channelStatusId  --p_CHANNELSTATUSName  -- number
        ,   p_FRIPADDRESS
        ,   v_DestTypeId
        ,   p_PersistDocTS
        ,   p_ProcessStartDocTS
        ,   p_ProcessEndDocTS
        ,   nvl(p_PROVSESSIONID, -1)
        ,   p_ACTIVITYTYPECD
        ,   p_FRPACKAGENAME
        ,   p_TOPACKAGENAME
        ,   p_FRPACKAGEVER
        ,   p_TOPACKAGEVER
        ,   p_FRPACKAGECD
        ,   p_TOPACKAGECD
        ,   p_FRCONTENTCD
        ,   p_TOCONTENTCD
        ,   p_FRTRANSPORTTYPE
        ,   p_ToTRANSPORTTYPE
        ,   v_BeginTS
        ,   v_BeginTS
        ,   p_ActionFlg
        ,   p_USER01
        ,   p_USER02
        ,   p_USER03
	,   p_USER04
	,   p_USER05
	,   p_USER06
        ,   p_USER07
        ,   p_USER08
        ,   p_USER09
        ,   p_USER10        
    );
--  end if;

        EXCEPTION
        When Dup_val_on_index Then
           v_InsertDup := 1;  --Update_Lg_Activity;
        When Others Then
           p_ReturnCd := 1;
           ProcAudit;
           Raise_application_error (-20999,'20001  Update_Lg_Activity '||SQLERRM);

end Insert_LG_Activity;
----------------------------------------------------------------
--                        M  A  I  N
----------------------------------------------------------------
BEGIN
    Check_length;
    -- if init, fr and to partner id is null then fetch it from pr_busIdentifier table
    v_InitPARTNERIDHDR := nvl(p_INITPARTNERIDHDR, get_partnerid_from_businessid(p_INITBUSINESSIDHDR) );
    v_FrPARTNERIDHDR   := nvl(p_FrPARTNERIDHDR  , get_partnerid_from_businessid(p_FrBUSINESSIDHDR)   );
    v_ToPARTNERIDHDR   := nvl(p_ToPARTNERIDHDR  , get_partnerid_from_businessid(p_ToBUSINESSIDHDR)   );
    v_channelStatusId  := -1  ;  --get_channelStatusId (p_CHANNELSTATUSName);

--
    p_ReturnCd := 0;
    IF p_InsertFlg = 1 Then
        Insert_LG_Activity;
        if v_InsertDup = 1 Then
            Update_Lg_Activity;
        end if;
    ELSE
    Update_Lg_Activity;
    if (v_RowCnt = 0) then
        Insert_LG_Activity;
    end if;
    END IF;
--
-- Do summary
--
If p_ProcessStartDocTS is not null then v_ProcessStartCnt := 1; End If;
If p_ProcessEndDocTS   is not null then v_ProcessEndCnt   := 1; End If ;
/*
LG_DOCPROCESSING_SUMM(
P_RETURNCD         => v_returnCD
	,p_DebugFlg        => p_DebugFlg
	,P_RCVDOCTS        => p_RCVDOCTS
	,P_FRIPADDRESS     => P_FRIPADDRESS
	,P_RCVDOCCNT       => 0
	,P_PERSISTDOCCNT   => 0
	,P_PROCESSSTARTCNT => v_ProcessStartCnt
	,P_PROCESSENDCNT   => v_ProcessEndCnt
	,P_PROVSESSIONID   => P_PROVSESSIONID
	,P_SUCCESSCNT      => 0
	,P_FAILCNT         => 0
);
*/
-- Insert into LG_UNPROCESSED_ACTIVITY
-----LG_SUMMARYI_AH1
Begin
        INSERT INTO LG_Unprocessed_Activity
        (
       	VUID
	,RCVDOCTS
	,SNTDOCTS
	,INITBUSINESSID
	,FRBUSINESSID
	,TOBUSINESSID
	,FRPROTOCOLNAME
	,TOPROTOCOLNAME
	,FRPROCESSCD
	,FRPROCESSVER
	,TOPROCESSCD
	,TOPROCESSVER
	,FRIPADDRESS
	,CHANNELSTATUSNAME
	,RCVDOCCNT
	,SNTDOCCNT
	,FAILDOCCNT
	,EVENTCD
	,MSGRAWLENGTH
	,MSGCONTENTLENGTH
	,FRPROTOCOLVER
	,TOPROTOCOLVER
	,DESTNAME
	,DMDURATION_MS
	,TRANSPORTDURATION_MS
	,TRANSPORTATTEMPTCNT
	,GATEWAYATTEMPTCNT
	,PERSISTDOCTS
	,BPESTARTTS
	,BPEENDTS
	,PROVSESSIONID
	,FRTRANSPORTTYPE
	,TOTRANSPORTTYPE
	,FRPACKAGENAME
	,FRPACKAGEVER
	,TOPACKAGENAME
	,TOPACKAGEVER
	,SYNCHRONOUSFLG
	,ACTIONFLG
	)
	VALUES
	(
	P_ACTIVITYID
	,P_RCVDOCTS
	,NULL
	,P_INITBUSINESSIDHDR
	,P_FRBUSINESSIDHDR
	,P_TOBUSINESSIDHDR
	,P_FRPROTOCOLNAME
	,P_TOPROTOCOLNAME
	,P_FRPROCESSCD
	,P_FRPROCESSVER
	,P_TOPROCESSCD
	,P_TOPROCESSVER
	,P_FRIPADDRESS
	, null   --P_CHANNELSTATUSNAME
	,P_RCVDOCCNT
	,0
	,0
	,P_EVENTCD
	,P_MSGRAWLENGTH
	,P_MSGCONTENTLENGTH
	,P_FRPROTOCOLVER
	,P_TOPROTOCOLVER
	,P_DESTNAME
	,0
	,0
	,0
	,0
	,P_PERSISTDOCTS
	,P_PROCESSSTARTDOCTS
	,P_PROCESSENDDOCTS
	,P_PROVSESSIONID
	,P_FRTRANSPORTTYPE
	,P_TOTRANSPORTTYPE
	,P_FRPACKAGENAME
	,P_FRPACKAGEVER
	,P_TOPACKAGENAME
	,P_TOPACKAGEVER
	,p_SYNCHRONOUSFLG
	,p_ActionFlg
	);
EXCEPTION When Others Then
           p_ReturnCd := 1;
           ProcAudit;
           Raise_application_error (-20999,'20001  Insert_LG_Unprocessed_Activity '||SQLERRM);
END;

ProcAudit;
p_ReturnCd := 0;
EXCEPTION
-- When NO_DATA_FOUND Then Null;
When Others Then
ProcAudit;
Raise_application_error (-20999,'20000 LG_ACTIVITY_merge' || SQLCODE|| ' '||SQLERRM);
p_ReturnCd := 1;

END;
/


-- LG_ACTIVITY_RNDTLS_AH1.sql

CREATE OR REPLACE PROCEDURE  LG_ACTIVITY_RNDTLS_AH1
(
      p_ReturnCd        out pls_INTEGER
    , p_DebugFlg        in    pls_INTEGER
    , RC1               out db_ref.RCT                -- rows
    , RC2               out db_ref.rct_count          -- count
    , p_sortkey         in  varchar2               -- RcvDocTS only
    , p_PageSizeNbr         in    pls_Integer
    , p_PageNbr             in  pls_Integer
    , p_FROMPARTNERID       in    varchar2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_TOPARTNERID         in    varchar2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_STARTDATE           in    DATE                    --    (Date and Time)
    , p_ENDDATE             in    DATE                    --    (Date and Time)
    , p_PROTOCOLVER         in  VARCHAR2
    , p_PROCESSCD           in    VARCHAR2
    , p_PROCESSVER          in    VARCHAR2
    , p_PROCESSINSTANCEID   in varchar2
    , p_INITBUSINESSID      in varchar2
    , p_CHANNELSTATUSID     in    pls_Integer             -- from lg_activity -- Ignore it 05/16/02
    , p_level               in integer                  -- indicate query level 1=summary 2=detail
    --, p_Locale      in varchar2
)
AS
/*
================================================================================
Proc Name:      LG_ACTIVITY_RNDTLs_AH1
Create Date:    2002-08-09
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME         Version   DESCRIPTION
---------  --------------------  ---------------------------------------------
2002-08-09  S.Hutchinson 3.2      Created
2002-08-21  J.Chen       3.2      Optimization - moved where clause into inner query
                                                 added index hints
                                                 added ar.cid = ah.cid to pick up index ak_activity_rndtl_1
2002-08-23  J.Chen       3.2      Limit the result set to 500 rows.
2003-06-20  V Goel       3.5      Use DB_Paging_Logic and DB_Build_WhereClause
2003-07-01  M.Mount      3.5      Changed return_code to p_returncd and debug to p_debugflg
2003-07-07  V Goel       3.5      Use FN_Paging_Logic and FN_Build_Where Functions
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
  v_startdate        varchar2(20) := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
  v_enddate          varchar2(20) := to_char(nvl(p_EndDATE,sysdate)    , 'dd-mm-yy hh24:mi:ss');
  v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(32000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(2000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  frPartnerName      pr_partner.partnerName%type;
  ToPartnerName      pr_partner.partnerName%type;
  v_order            varchar2(200);
  v_where2           varchar2(1000);
--  Parent_hdrflg      pls_integer := 1 ; -- lg_activity_dtl.HDRFLG = 1
-- Child_hdrflg       pls_integer := 0 ; -- lg_activity_dtl.HDRFLG = 0
----- Typ
Type char18Tab is table of varchar2(18);
Type char20Tab is table of varchar2(20);
Type char255Tab is table of varchar2(255);
Type num10Tab  is table of number(10);
array_ibusid    char20Tab  := char20Tab();
array_piid      char255Tab := char255Tab();
array_cid       num10Tab   := num10Tab();
array_rnbr      num10Tab   := num10Tab();
--array_rowid     char18Tab  := char18Tab();
--rndtlTab       Row_ID_TBL := Row_ID_TBL();
----------
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin

  DB_ProcAuditLogI_AH1('LG_ACTIVITY_RNDTLS_AH1',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'||','||'RC2'||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_FROMPARTNERID||','||
    p_TOPARTNERID  ||','||
    To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
    p_PROTOCOLVER  ||','||
    p_PROCESSCD    ||','||
    p_PROCESSVER   ||','||
    p_PROCESSINSTANCEID ||','||
    p_INITBUSINESSID ||','||
    p_CHANNELSTATUSID
    ,0,0);
End ProcAudit;
--
Procedure where_clause as
begin
    v_where := v_where||' Where AH.RCVDOCTS between '||chr(10)||' to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and'||
                                                         chr(10)||' to_date('''|| v_endDate||''',''dd-mm-yy hh24:mi:ss'') '||

  FN_Build_Where('AH.InitPARTNERID',  'NUMBER_LIST', p_FROMPARTNERID          ) ||
  FN_Build_Where('AH.ToPARTNERID',    'NUMBER_LIST', p_ToPARTNERID            ) ||
  FN_Build_Where('AH.PROTOCOLNAME',   'VARCHAR2'   , 'RosettaNet'             ) ||
  FN_Build_Where('AH.PROTOCOLVER',    'VARCHAR2'   , p_PROTOCOLVER            ) ||
  FN_Build_Where('AH.PROCESSCD',      'VARCHAR2'   , p_PROCESSCD              ) ||
  FN_Build_Where('AH.PROCESSVER',     'VARCHAR2'   , p_PROCESSVER             ) ||
  FN_Build_Where('AH.PROCESSINSTANCEID', 'VARCHAR2_LIKE', p_PROCESSINSTANCEID ) ||
  FN_Build_Where('AH.INITBUSINESSID', 'VARCHAR2'   , p_INITBUSINESSID         );
end where_clause;
--
-- Main
--
BEGIN
   -- dbms_output.enable(1000000);
    p_ReturnCd := 0;
  -- WHERE  RN.PROCESSBEGINTS BETWEEN TO_DATE('2002-08-02 12:00:00','YYYY-MM-DD HH24:MI:SS') AND TO_DATE('2002-08-02 14:30:00','YYYY-MM-DD HH24:MI:SS')
     where_clause;
--    dbms_output.put_line(substr(v_where,1,250));
--    dbms_output.put_line(substr(v_where,251,250));
    if p_sortkey is not null then
        --EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);
        v_sortkey := upper(p_sortkey);
--       v_sortkey := replace(v_sortkey,'FRPARTNERNAME','frp.partnername');
--       v_sortkey := replace(v_sortkey,'TOPARTNERNAME','top.partnername');
    --   sqlstmt := sqlstmt ||' order by '||v_sortkey;
        v_order := ' order by '|| v_sortkey;
    end if;
--     db_general_pkg.db_dbms_output('Sql ', sqlstmt);
if p_level = 1 then  --summary
--open rc1 for
sqlstmt :=
'select
       RN.INITPARTNERNAME,
       RN.TRADINGPARTNERNAME,
       RN.INITPARTNERID,
       RN.TRADINGPARTNERID,
          SUM(RN.DOCCNT) AS DOCCNT,
         SUM(RN.SEG1ACTCNT) AS SEG1ACTCNT,
          SUM(RN.SEG1SIGCNT) AS SEG1SIGCNT,
          SUM(RN.SEG2ACTCNT) AS SEG2ACTCNT,
          SUM(RN.SEG2SIGCNT) AS SEG2SIGCNT,
         SUM(RN.AA) AS AACNT,
         SUM(RN.GE) AS GECNT,
         SUM(RN.RA) AS RACNT,
         SUM(RN.RAE) AS RAECNT
FROM
  (SELECT /*+ INDEX(AH AK_ACTIVITY_RNHDR_1, pk_activity_rnhdr) index(AR ak_activity_rndtl_1, pk_activity_rndtl)  index(AE) index(A) */
         AR.INITPARTNERID,
         P1.PARTNERNAME AS INITPARTNERNAME,
         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID THEN AR.TOPARTNERID
             WHEN AR.INITPARTNERID = AR.TOPARTNERID THEN AR.FRPARTNERID
         END AS TRADINGPARTNERID,
         CASE
             WHEN AR.INITPARTNERID = AR.FRPARTNERID THEN P2.PARTNERNAME
             WHEN AR.INITPARTNERID = AR.TOPARTNERID THEN P3.PARTNERNAME
         END AS TRADINGPARTNERNAME,
         AR.INITBUSINESSID,
         AH.RCVDOCTS AS PROCESSBEGINTS,
         1 AS DOCCNT,
         AE.ENDSTATEFLG,
         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN 1
             ELSE 0
         END AS SEG1ACTCNT,
         CASE
                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN 1
             ELSE 0
         END AS SEG1SIGCNT,
         CASE
                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN 1
             ELSE 0
         END AS SEG2ACTCNT,
         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN 1
             ELSE 0
         END AS SEG2SIGCNT,
         CASE
                WHEN AR.GLBBUSSIGNALCD = ''Acceptance Acknowledgment'' OR AR.GLBBUSSIGNALCD = ''Acceptance Acknowledge'' THEN 1
             ELSE 0
         END AS AA,
         CASE
                WHEN AR.GLBBUSSIGNALCD = ''Exception'' OR AR.GLBBUSSIGNALCD = ''General Exception'' THEN 1
             ELSE 0
         END AS GE,
         CASE
                WHEN AR.GLBBUSSIGNALCD = ''Receipt Acknowledge'' OR AR.GLBBUSSIGNALCD = ''Receipt Acknowledgment'' THEN 1
             ELSE 0
         END AS RA,
         CASE
                WHEN AR.GLBBUSSIGNALCD = ''Receipt Acknowledgement Exception'' THEN 1
             ELSE 0
         END AS RAE
  FROM   LG_ACTIVITY_RNDTL AR
            INNER JOIN LG_ACTIVITY_RNHDR AH ON AR.INITBUSINESSID = AH.INITBUSINESSID
                                        AND AR.PROCESSINSTANCEID = AH.PROCESSINSTANCEID
                                        AND AR.CID = AH.CID
           INNER JOIN PR_PARTNER P1 ON P1.PARTNERID = AH.INITPARTNERID
            INNER JOIN PR_PARTNER P2 ON P2.PARTNERID = AR.TOPARTNERID
            INNER JOIN PR_PARTNER P3 ON P3.PARTNERID = AR.FRPARTNERID
           INNER JOIN LG_ACTIVITY A ON A.ACTIVITYID = AR.ACTIVITYID
           LEFT OUTER JOIN LG_ACTIVITY_ENDSTATE AE ON A.ACTIVITYID = AE.ACTIVITYID '|| v_where || ') RN ' ||
 ' GROUP BY
       RN.INITPARTNERNAME,
       RN.TRADINGPARTNERNAME,
       RN.INITPARTNERID,
       RN.TRADINGPARTNERID '
       || v_order;
end if;
if p_level=2 then     --detail
sqlstmt := 'SELECT * FROM
       (SELECT
       RN.CID,
       RN.INITPARTNERNAME,
        RN.TRADINGPARTNERNAME,
       RN.INITPARTNERID,
        RN.TRADINGPARTNERID,
         RN.PROCESSCD,
         RN.PROCESSINSTANCEID,
         SUM(RN.DOCCNT) AS DOCCNT,
         SUM(RN.SEG1ACTCNT) AS SEG1ACTCNT,
       SUM(RN.SEG1SIGCNT) AS SEG1SIGCNT,
       SUM(RN.SEG2ACTCNT) AS SEG2ACTCNT,
       SUM(RN.SEG2SIGCNT) AS SEG2SIGCNT,
         SUM(RN.AA) AS AACNT,
         SUM(RN.GE) AS GECNT,
         SUM(RN.RA) AS RACNT,
         SUM(RN.RAE) AS RAECNT,
       RN.PROCESSBEGINTS,
       Null AS SEG1ACTRCVTS,
       Null AS SEG2ACTRCVTS
/*
       ,CASE
                WHEN ENDSTATEFLG = 1 AND SEG1ACTCNT = 1 THEN RN.SEG1ACTRCVTS
             ELSE NULL
         END AS SEG1ACTRCVTS,
         CASE
                WHEN ENDSTATEFLG = 1 AND SEG2ACTCNT = 1 THEN RN.SEG2ACTRCVTS
             ELSE NULL
         END AS SEG2ACTRCVTS
*/
  FROM
  (SELECT /*+ INDEX(AH AK_ACTIVITY_RNHDR_1, pk_activity_rnhdr) index(AR ak_activity_rndtl_1, pk_activity_rndtl)  index(AE) index(A) */
       AH.CID,
         AR.INITPARTNERID,
         P1.PARTNERNAME AS INITPARTNERNAME,
         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID THEN AR.TOPARTNERID
             WHEN AR.INITPARTNERID = AR.TOPARTNERID THEN AR.FRPARTNERID
         END AS TRADINGPARTNERID,
         CASE
           WHEN AR.INITPARTNERID = AR.FRPARTNERID THEN P2.PARTNERNAME
             WHEN AR.INITPARTNERID = AR.TOPARTNERID THEN P3.PARTNERNAME
         END AS TRADINGPARTNERNAME,
         AH.PROCESSCD,
         AR.INITBUSINESSID,
         AR.PROCESSINSTANCEID,
         AH.RCVDOCTS AS PROCESSBEGINTS,
         1 AS DOCCNT,
         AE.ENDSTATEFLG,
         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN 1
             ELSE 0
         END AS SEG1ACTCNT,
         CASE
                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN 1
             ELSE 0
         END AS SEG1SIGCNT,
         CASE
                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN 1
             ELSE 0
         END AS SEG2ACTCNT,
         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN 1
             ELSE 0
         END AS SEG2SIGCNT,
         CASE
                WHEN AR.GLBBUSSIGNALCD = ''Acceptance Acknowledgment'' OR AR.GLBBUSSIGNALCD = ''Acceptance Acknowledge'' THEN 1
             ELSE 0
         END AS AA,
         CASE
                WHEN AR.GLBBUSSIGNALCD = ''Exception'' OR AR.GLBBUSSIGNALCD = ''General Exception'' THEN 1
             ELSE 0
         END AS GE,
         CASE
                WHEN AR.GLBBUSSIGNALCD = ''Receipt Acknowledge'' OR AR.GLBBUSSIGNALCD = ''Receipt Acknowledgment'' THEN 1
             ELSE 0
         END AS RA,
         CASE
                WHEN AR.GLBBUSSIGNALCD = ''Receipt Acknowledgement Exception'' THEN 1
             ELSE 0
         END AS RAE --,
/*         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN (AE.SNTDOCTS - A.RCVDOCTS)*24*60*60
             ELSE 0
         END AS SEG1ACTDURATION,
         CASE
                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN (AE.SNTDOCTS - A.RCVDOCTS)*24*60*60
             ELSE 0
         END AS SEG1SIGDURATION,
         CASE
                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN (AE.SNTDOCTS - A.RCVDOCTS)*24*60*60
             ELSE 0
         END AS SEG2ACTDURATION,
         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN (AE.SNTDOCTS - A.RCVDOCTS)*24*60*60
             ELSE 0
         END AS SEG2SIGDURATION,
*/
--         CASE
--               WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN A.RCVDOCTS
--             ELSE NULL
--         END AS SEG1ACTRCVTS,
/*        CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN AE.SNTDOCTS
             ELSE NULL
         END AS SEG1ACTSNTTS,
        CASE
                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN A.RCVDOCTS
             ELSE NULL
         END AS SEG1SIGRCVTS,
         CASE
                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN AE.SNTDOCTS
             ELSE NULL
         END AS SEG1SIGSNTTS,
*/
--         CASE
--                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN A.RCVDOCTS
--             ELSE NULL
--         END AS SEG2ACTRCVTS,
/*        CASE
                WHEN AR.INITPARTNERID = AR.TOPARTNERID AND AR.GLBBUSACTIONCD IS NOT NULL THEN AE.SNTDOCTS
             ELSE NULL
         END AS SEG2ACTSNTTS,
         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN A.RCVDOCTS
             ELSE NULL
         END AS SEG2SIGRCVTS,
         CASE
                WHEN AR.INITPARTNERID = AR.FRPARTNERID AND AR.GLBBUSSIGNALCD IS NOT NULL THEN AE.SNTDOCTS
             ELSE NULL
        END AS SEG2SIGSNTTS
*/
  FROM   LG_ACTIVITY_RNDTL AR
         INNER JOIN LG_ACTIVITY_RNHDR AH ON AR.INITBUSINESSID = AH.INITBUSINESSID
                                       AND AR.PROCESSINSTANCEID = AH.PROCESSINSTANCEID
                                       AND AR.CID = AH.CID
         INNER JOIN PR_PARTNER P1 ON P1.PARTNERID = AH.INITPARTNERID
       INNER JOIN PR_PARTNER P2 ON P2.PARTNERID = AR.TOPARTNERID
       INNER JOIN PR_PARTNER P3 ON P3.PARTNERID = AR.FRPARTNERID
         INNER JOIN LG_ACTIVITY A ON A.ACTIVITYID = AR.ACTIVITYID
         LEFT OUTER JOIN LG_ACTIVITY_ENDSTATE AE ON A.ACTIVITYID = AE.ACTIVITYID '|| v_where || ') RN ' ||
  ' GROUP BY
       RN.CID,
         RN.INITPARTNERNAME,
        RN.TRADINGPARTNERNAME,
       RN.INITPARTNERID,
        RN.TRADINGPARTNERID,
         RN.PROCESSCD,
         RN.PROCESSINSTANCEID,
         RN.PROCESSBEGINTS,
       Null,Null'|| v_order ||') where rownum <= 500';
end if;
    db_print('Sql ', sqlstmt);

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR sqlstmt;

    --IF p_sortkey is not null THEN
    --EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';
    --END IF;

    EXCEPTION
    WHEN OTHERS THEN
        ProcAudit;
        p_ReturnCd := 1;
    end;
-- proc Audit
    ProcAudit;
END;
/


-- LG_ACTIVITY_RNHDRI_AH1.sql

CREATE OR REPLACE PROCEDURE         LG_ACTIVITY_RNHDRi_ah1 (
    p_ReturnCd out integer
    ,p_DebugFlg in pls_integer
--    ,RC1 out db_ref.RCT
    -- RnHDR
    ,p_INITBUSINESSID        in out VARCHAR2
    ,p_PROCESSINSTANCEID     in out varCHAR2
    ,p_ref_INITBUSINESSID    in VARCHAR2    -- do not map to table(null except for inbound 0A1)
    ,p_ref_PROCESSINSTANCEID in varCHAR2    -- do not map to table(null except for inbound 0A1)
    ,p_InitPARTNERID         in pls_integer
    ,p_FrPARTNERID           in pls_integer
    ,p_ToPARTNERID           in pls_integer
    ,p_INITBUSIDTYPE         in out VARCHAR2
    ,p_PROTOCOLNAME          in out VARCHAR2
    ,p_PROTOCOLVER           in out VARCHAR2
    ,p_PROCESSCD             in out VARCHAR2
    ,p_PROCESSVER            in out VARCHAR2
    ,p_RCVDOCTS              in TimeStamp            -- DATE
    ,p_STATUSFLG             in NUMBER
    -- RnDTl
    ,p_ACTIVITYID            in VARCHAR2
    ,p_VUID                  in VARCHAR2
    --,p_CID in NUMBER
    --,p_INITBUSINESSID in VARCHAR2
    --,p_PROCESSINSTANCEID in CHAR
    ,p_ACTSIGTRACKINSTANCEID in out VARCHAR2
    ,p_TRANSACTIONINSTANCEID in out VARCHAR2
    ,p_GLBUSAGECD            in out VARCHAR2
    ,p_MSGDATETIME           in out VARCHAR2
    ,p_FRBUSIDTYPE           in out VARCHAR2
    ,p_TOBUSIDTYPE           in out VARCHAR2
    ,p_IRTGLBBUSACTIONCD     in out VARCHAR2
    ,p_IRTINSTANCEID         in out VARCHAR2
    ,p_GLBBUSACTIONCD        in out VARCHAR2
    ,p_SECURETRANSREQD       in out VARCHAR2
    ,p_ATTEMPTCNT            in NUMBER
    ,p_FRSERVICE             in out VARCHAR2
    ,p_TOSERVICE             in out VARCHAR2
    ,p_GLBDOCFUNCTIONCD      in out VARCHAR2
    ,p_FRGLBPARTNERCLASSCD   in out VARCHAR2
    ,p_TOGLBPARTNERCLASSCD   in out VARCHAR2
    ,p_GLBBUSSIGNALCD        in out VARCHAR2
    ,p_FRROLE                in out VARCHAR2
    ,p_TOROLE                in out VARCHAR2
    ,p_ACTIVITYNAME          in out VARCHAR2
    ,p_GLBPROCESSCD          in out VARCHAR2
    ,p_ATTACHMENTNBR         in NUMBER
    --
    , P_FRBUSINESSID          in out VARCHAR2
    , P_TOBUSINESSID          in out VARCHAR2
    , p_OtherScrFlg           in varchar2 -- 'FROM'/'TO'
    , P_OtherPROTOCOLNAME     in VARCHAR2
    , P_OtherPROTOCOLVer      in VARCHAR2
    , P_OtherPROCESSCD        in VARCHAR2
    , P_OtherPROCESSVER       in VARCHAR2
    , P_FRIPADDRESS           in out VARCHAR2
    , P_CHANNELSTATUSNAME     in VARCHAR2
    --, P_DestTypeId            in number
    , P_DestName              in VARCHAR2
    , p_endstateTS            in TimeStamp
    , P_PROVSESSIONID         in pls_integer
    , p_FrTransportType       in VARCHAR2
    , p_ToTransportType       in VARCHAR2
    , p_FRPACKAGENAME       in  VARCHAR2
    , p_FRPACKAGEVER        in  VARCHAR2
    , p_TOPACKAGENAME       in  VARCHAR2
    , p_TOPACKAGEVER        in  VARCHAR2
    , p_SYNCHRONOUSFLG      in  Pls_Integer
    , p_ActionFlg           in  Pls_Integer
) as
/*

================================================================================
Proc Name:      LG_ACTIVITY_RNHDRi_ah1
Create Date:    04/18/02 09:04:41 AM
Author          V Goel
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2004-03-11 S.Priani	4.2.2	Replaced call to Summary procs by Insert
	2006-05-01 Shishir N	6.0	PMR 08997999846
	2006-09-07 S.Priani	6.1	JR22621
================================================================================*/
v_destTypeID        PLS_INTEGER default FN_GET_DESTTYPEID (p_DestName);
v_rowcnt            integer    :=0;
v_BeginTS           timestamp := FN_Get_Date; -- Performance related variables
v_pk_value          number := 0;
v_cid                LG_ACTIVITY_RNHDR.cid%type := null;
v2_INITBUSINESSID    LG_ACTIVITY_RNHDR.INITBUSINESSID%type;
v2_PROCESSINSTANCEID LG_ACTIVITY_RNHDR.PROCESSINSTANCEID%type;
v_summaryID          Lg_sum_xref_lnk.summaryID%type;
v_Ref_summaryID      Lg_sum_xref_lnk.summaryID%type; -- belongs to ref_initBusinessID and ref_processInstanceID
v_ref_RcvDocTS       LG_ACTIVITY_RNHDR.RcvDocTS%type;
v2_Ref_summaryID     Lg_sum_xref_lnk.summaryID%type;
v2_ref_RcvDocTS      LG_ACTIVITY_RNHDR.RcvDocTS%type;
v_rnHdr_RcvDocTS     LG_ACTIVITY_RNHDR.RcvDocTS%type;
V_endstateTS         Lg_Activity_endState.EndstateTS%Type ; -- end state TS for request used in Time2perform
V_endstateTS_date    Date;
V_RCVDOCTS           Date := p_RCVDOCTS;
v_0A1_Flag           Boolean := FALSE;
v_rnDTL_Dup_Flag     Boolean := FALSE;
v_dummy              varchar2(100);
--
v_ProcessSummaryID       Lg_ProcessSummary_Xref.PROCESSSUMMARYID%Type;
v_Ref_ProcessSummaryID   Lg_ProcessSummary_Xref.PROCESSSUMMARYID%Type;
v2_Ref_ProcessSummaryID  Lg_ProcessSummary_Xref.PROCESSSUMMARYID%Type;
-- Lg summary processing
    v_rnProcessCnt       pls_integer := 0;
    v_rnActionCnt        pls_integer := 0;
    v_rnSignalCnt        pls_integer := 0;
    v_UnknownCnt         pls_integer := 0;
    v_Action_duration_MS pls_integer := 0;
    v_Signal_duration_MS pls_integer := 0;
    v_Time2perform       pls_integer := 0;

--
    v_AACNT             pls_integer := 0;
    v_GECNT             pls_integer := 0;
    v_RACNT             pls_integer := 0;
    v_RAECNT            pls_integer := 0;
    v_AAECNT            pls_integer := 0;

--
-- Procedures
--
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_ah1('LG_ACTIVITY_RNHDRi_ah1',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
--    ||','||'RC1'
    ||','||p_INITBUSINESSID
    ||','||p_PROCESSINSTANCEID
    ||','||p_INITBUSIDTYPE
    ||','||p_PROTOCOLNAME
    ||','||p_PROTOCOLVER
    ||','||p_PROCESSCD
    ||','||p_PROCESSVER
    ||','||p_RCVDOCTS
    ||','||p_STATUSFLG
    ||','||p_ACTIVITYID
    ||','||p_VUID
    --||','||p_CID
    --||','||p_INITBUSINESSID
    --||','||p_PROCESSINSTANCEID
    ||','||p_ACTSIGTRACKINSTANCEID
    ||','||p_TRANSACTIONINSTANCEID
    ||','||p_GLBUSAGECD
    ||','||p_MSGDATETIME
    ||','||p_FRBUSIDTYPE
    ||','||p_TOBUSIDTYPE
    ||','||p_IRTGLBBUSACTIONCD
    ||','||p_IRTINSTANCEID
    ||','||p_GLBBUSACTIONCD
    ||','||p_SECURETRANSREQD
    ||','||p_ATTEMPTCNT
    ||','||p_FRSERVICE
    ||','||p_TOSERVICE
    ||','||p_GLBDOCFUNCTIONCD
    ||','||p_FRGLBPARTNERCLASSCD
    ||','||p_TOGLBPARTNERCLASSCD
    ||','||p_GLBBUSSIGNALCD
    ||','||p_FRROLE
    ||','||p_TOROLE
    ||','||p_ACTIVITYNAME
    ||','||p_GLBPROCESSCD
    ||','||p_ATTACHMENTNBR
    ||','||P_FRBUSINESSID
    ||','||P_TOBUSINESSID
    ||','||P_OTHERSCRFLG
    ||','||P_OTHERPROTOCOLNAME
    ||','||P_OTHERPROTOCOLVER
    ||','||P_OTHERPROCESSCD
    ||','||P_OTHERPROCESSVER
    ||','||P_FRIPADDRESS
    ||','||P_CHANNELSTATUSNAME
    ||','||p_FrTransportType
    ||','||p_ToTransportType
    ||','||p_SYNCHRONOUSFLG
    ||','||p_ActionFlg
    );
End ProcAudit;
--
Procedure Check_length is
Begin
-- ACTIVITY_RNHDR
   DB_LENGTHCHECK_ACTIVITY_RNHDR(
       P_INITBUSINESSID     => P_INITBUSINESSID
       ,P_PROCESSINSTANCEID => P_PROCESSINSTANCEID
       ,P_INITBUSIDTYPE     => P_INITBUSIDTYPE
       ,P_PROTOCOLNAME      => P_PROTOCOLNAME
       ,P_PROTOCOLVER       => P_PROTOCOLVER
       ,P_PROCESSCD         => P_PROCESSCD
       ,P_PROCESSVER        => P_PROCESSVER
   );

-- ACTIVITY_RNDTL
   DB_LENGTHCHECK_ACTIVITY_RNDTL(
        P_ACTSIGTRACKINSTANCEID => P_ACTSIGTRACKINSTANCEID
       ,P_TRANSACTIONINSTANCEID => P_TRANSACTIONINSTANCEID
       ,P_GLBUSAGECD            => P_GLBUSAGECD
       ,P_MSGDATETIME           => P_MSGDATETIME
       ,P_FRBUSIDTYPE           => P_FRBUSIDTYPE
       ,P_TOBUSIDTYPE           => P_TOBUSIDTYPE
       ,P_IRTGLBBUSACTIONCD     => P_IRTGLBBUSACTIONCD
       ,P_IRTINSTANCEID         => P_IRTINSTANCEID
       ,P_GLBBUSACTIONCD        => P_GLBBUSACTIONCD
       ,P_SECURETRANSREQD       => P_SECURETRANSREQD
       ,P_FRSERVICE             => P_FRSERVICE
       ,P_TOSERVICE             => P_TOSERVICE
       ,P_GLBDOCFUNCTIONCD      => P_GLBDOCFUNCTIONCD
       ,P_FRGLBPARTNERCLASSCD   => P_FRGLBPARTNERCLASSCD
       ,P_TOGLBPARTNERCLASSCD   => P_TOGLBPARTNERCLASSCD
       ,P_GLBBUSSIGNALCD        => P_GLBBUSSIGNALCD
       ,P_FRROLE                => P_FRROLE
       ,P_TOROLE                => P_TOROLE
       ,P_ACTIVITYNAME          => P_ACTIVITYNAME
       ,P_GLBPROCESSCD          => P_GLBPROCESSCD
   );

-- Summary_xref  : length is checked in procedure lg_summary_xrefi_ah1

End Check_length;
--

Procedure Insert_xref is
    p_ReturnCd             pls_integer;
    v_FrProtocolName   LG_Activity_Rnhdr.ProtocolName%Type;
    v_ToProtocolName   LG_Activity_Rnhdr.ProtocolName%Type;
    v_FrProtocolVer    LG_Activity_Rnhdr.ProtocolVer%Type;
    v_ToProtocolVer    LG_Activity_Rnhdr.ProtocolVer%Type;
    v_FrProcessCD      LG_Activity_Rnhdr.ProcessCD%Type;
    v_ToProcessCD      LG_Activity_Rnhdr.ProcessCD%Type;
    v_FrProcessVer     LG_Activity_Rnhdr.ProcessVer%Type;
    v_ToProcessVer     LG_Activity_Rnhdr.ProcessVer%Type;
    v_initPartnerID    pr_partner.partnerID%Type;
    v_ToPartnerID      pr_partner.partnerID%Type;

Begin
        If p_OtherScrflg = 'FROM' Then
            v_FrProtocolName := P_OtherPROTOCOLNAME;
            v_FRPROTOCOLVer  := P_OtherPROTOCOLVer;
            v_ToProtocolName := P_PROTOCOLNAME;
            v_TOPROTOCOLVer  := P_PROTOCOLVer;
            v_FrProcessCD    := P_OtherPROCESSCD;
            v_FrProcessVer   := P_OtherPROCESSVER;
            v_ToProcessCD    := P_PROCESSCD;
            v_ToProcessVer   := P_PROCESSVER;
        Elsif p_OtherScrflg = 'TO' Then
            v_FrProtocolName := P_PROTOCOLNAME;
            v_FRPROTOCOLVer  := P_PROTOCOLVer;
            v_ToProtocolName := P_OtherPROTOCOLNAME;
            v_TOPROTOCOLVer  := P_OtherPROTOCOLVer;
            v_FrProcessCD    := P_PROCESSCD;
            v_FrProcessVer   := P_PROCESSVER;
            v_ToProcessCD    := P_OtherPROCESSCD;
            v_ToProcessVer   := P_OtherPROCESSVER;
        End if;
    LG_SUMMARY_XREFI_AH1(
    P_ReturnCd        => p_ReturnCd
    ,P_DEBUGFLG           => p_DebugFlg
    ,P_SUMMARYID         => v_summaryid       -- out
    ,P_INITBUSINESSID    => P_INITBUSINESSID
    ,P_FRBUSINESSID      => P_FRBUSINESSID
    ,P_TOBUSINESSID      => P_TOBUSINESSID
    ,P_FRPROTOCOLNAME    => v_FRPROTOCOLNAME
    ,P_FRPROTOCOLVER     => v_FRPROTOCOLVer
    ,P_TOPROTOCOLNAME    => v_TOPROTOCOLNAME
    ,P_TOPROTOCOLVer     => v_TOPROTOCOLVer
    ,P_FRPROCESSCD       => v_FRPROCESSCD
    ,P_FRPROCESSVER      => v_FRPROCESSVER
    ,P_TOPROCESSCD       => v_TOPROCESSCD
    ,P_TOPROCESSVER      => v_TOPROCESSVER
    ,P_FRIPADDRESS       => P_FRIPADDRESS
    ,P_CHANNELSTATUSNAME => P_CHANNELSTATUSNAME
    ,P_DESTTypeId        => v_DestTypeId
    ,P_PROVSESSIONID     => P_PROVSESSIONID
    ,p_FrTransportType   => p_FrTransportType
    ,p_ToTransportType   => p_ToTransportType
    ,p_FRPACKAGENAME   => nvl(p_FRPACKAGENAME, '-1')
    ,p_FRPACKAGEVER    => nvl(p_FRPACKAGEVER, '-1')
    ,p_TOPACKAGENAME   => nvl(p_TOPACKAGENAME, '-1')
    ,p_TOPACKAGEVER    => nvl(p_TOPACKAGEVER, '-1')
    ,p_SYNCHRONOUSFLG  => nvl(p_SYNCHRONOUSFLG,0)
    ,p_ActionFlg       => nvl(p_ActionFlg,-1)

);
--
-- Lg_ProcessSummary_Xref
--
LG_PROCESSSUMMARY_XREFI_AH1(
P_RETURNCD          => p_ReturnCd
,P_DEBUGFLG         => p_DebugFlg
,P_PROCESSSUMMARYID => v_PROCESSSUMMARYID  -- out
,P_INITPARTNERID    => P_INITPARTNERID
,P_TOPARTNERID      => Case When P_initPartnerID = P_frPartnerID Then p_TOPARTNERID Else p_frPartnerID End
,P_INITBUSIDTYPE    => P_INITBUSIDTYPE
,P_PROTOCOLNAME     => P_PROTOCOLNAME
,P_PROTOCOLVER      => P_PROTOCOLVER
,P_PROCESSCD        => P_PROCESSCD
,P_PROCESSVER       => P_PROCESSVER
,P_IPADDRESS        => P_FrIPADDRESS
--,P_SUMMARYID        => v_summaryid
,P_DESTTYPEID       => v_DestTypeId
,P_SYNCHRONOUSFLG    => nvl(P_SYNCHRONOUSFLG, 0)
);
--
-- Lg_Summary_ProcessSummary_Xref (Link between Lg_ProcessSummary_Xref and Lg_Summary_Xref)
--
LG_SUMMARY_PROCESSSUMMARYI_AH1(
P_RETURNCD          => p_ReturnCd
,P_DEBUGFLG         => p_DebugFlg
,P_SUMMARYID        => v_summaryid
,P_PROCESSSUMMARYID => v_PROCESSSUMMARYID
);
end Insert_xref;
--
Procedure Insert_ACTIVITY_RNDTL as
BEGIN
Insert into LG_ACTIVITY_RNDTL(
ACTIVITYID
,   VUID
,   CID
,   INITBUSINESSID
,   PROCESSINSTANCEID
,   ACTSIGTRACKINSTANCEID
,   INITPARTNERID
,   FRPARTNERID
,   TOPARTNERID
,   TRANSACTIONINSTANCEID
,   GLBUSAGECD
,   MSGDATETIME
,   FRBUSIDTYPE
,   TOBUSIDTYPE
,   IRTGLBBUSACTIONCD
,   IRTINSTANCEID
,   GLBBUSACTIONCD
,   SECURETRANSREQD
,   ATTEMPTCNT
,   FRSERVICE
,   TOSERVICE
,   GLBDOCFUNCTIONCD
,   FRGLBPARTNERCLASSCD
,   TOGLBPARTNERCLASSCD
,   GLBBUSSIGNALCD
,   FRROLE
,   TOROLE
,   ACTIVITYNAME
,   GLBPROCESSCD
,   ATTACHMENTNBR
) Values (
    p_ACTIVITYID
,   p_VUID
,   v_CID
,   p_INITBUSINESSID
,   p_PROCESSINSTANCEID
,   P_ACTSIGTRACKINSTANCEID
,   p_INITPARTNERID
,   p_FRPARTNERID
,   p_TOPARTNERID
,   p_TRANSACTIONINSTANCEID
,   p_GLBUSAGECD
,   p_MSGDATETIME
,   p_FRBUSIDTYPE
,   p_TOBUSIDTYPE
,   p_IRTGLBBUSACTIONCD
,   p_IRTINSTANCEID
,   p_GLBBUSACTIONCD
,   p_SECURETRANSREQD
,   p_ATTEMPTCNT
,   p_FRSERVICE
,   p_TOSERVICE
,   p_GLBDOCFUNCTIONCD
,   p_FRGLBPARTNERCLASSCD
,   p_TOGLBPARTNERCLASSCD
,   p_GLBBUSSIGNALCD
,   p_FRROLE
,   p_TOROLE
,   p_ACTIVITYNAME
,   p_GLBPROCESSCD
,   p_ATTACHMENTNBR
);
   If p_glbDocFunctionCD = 'Request'  and
      nvl(p_attemptCnt, 1) = 1
    Then
       if v_0A1_Flag = FALSE then
          v_rnProcessCnt := 1;
       End if;
   End if;
EXCEPTION
    When NO_DATA_FOUND Then Null;
    When dup_val_on_index then
        v_rnDTL_Dup_Flag := TRUE ;
    When Others Then
    ProcAudit;
    Raise_application_error (-20999,'20000 LG_ACTIVITY_RNGDRi_ah1  DTLi' || SQLCODE|| ' '||SQLERRM);
    p_ReturnCd := 1;
end Insert_ACTIVITY_RNDTL;
--
Procedure Get_CID (p_ibusid    in varchar2,
                   p_procInsId in varchar2,
                   p_rnhdr_cid       out pls_integer,
                   p_rnhdr_rcvDocTS  out date,
                   p_rnhdr_summaryID out pls_integer,
                   p_rnhdr_ProcessSummaryID out pls_integer
                   )is
begin
  select   CID, summaryID, RcvDocTS, ProcessSummaryID INTO
         p_rnhdr_cid, p_rnhdr_summaryID, p_rnhdr_rcvDocTS, p_rnhdr_ProcessSummaryID
  from LG_ACTIVITY_RNHDR
  where INITBUSINESSID    = p_ibusid
  and   PROCESSINSTANCEID = p_procInsId  ;

  Exception
  When Others Then
    ProcAudit;
    Raise_application_error (-20999,'20001 LG_ACTIVITY_RNHDRi_ah1 Get_CID Ref IDs ' ||p_ibusid|| ' '||
                             p_procInsId||' '||SQLCODE|| ' '||SQLERRM);
    p_ReturnCd := 1;
end Get_CID;
--
Procedure Insert_ACTIVITY_RNHDR is
Begin
Insert into LG_ACTIVITY_RNHDR(
    CID
,   INITBUSINESSID
,   PROCESSINSTANCEID
,   INITPARTNERID
,   TOPARTNERID
,   INITBUSIDTYPE
,   PROTOCOLNAME
,   PROTOCOLVER
,   PROCESSCD
,   PROCESSVER
,   RCVDOCTS
,   SUMMARYID
,   DESTTYPEID
,   PROCESSSUMMARYID
,   SYNCHRONOUSFLG
) Values (
    nvl(v_cid, CID_SEQ.nextval)
,   p_INITBUSINESSID
,   p_PROCESSINSTANCEID
,   p_INITPARTNERID
,   Case When P_initPartnerID = P_frPartnerID
    Then
        p_TOPARTNERID
    Else
        p_frPartnerID End
,   p_INITBUSIDTYPE
,   p_PROTOCOLNAME
,   p_PROTOCOLVER
,   p_PROCESSCD
,   p_PROCESSVER
,   p_RCVDOCTS
,   v_SUMMARYID
,   v_DestTypeId
,   v_PROCESSSUMMARYID
,   nvl(p_SYNCHRONOUSFLG, 0)
) returning CID, RcvDocTs into v_cid, v_rnHdr_RcvDocTS;
Insert_ACTIVITY_RNDTL;

EXCEPTION
    when dup_val_on_index then
      Get_CID(p_INITBUSINESSID,
              p_PROCESSINSTANCEID,
              v_cid,
              v_rnhdr_rcvDocTS,
              v_ref_summaryID,
              v_ref_ProcessSummaryID) ;
      Insert_ACTIVITY_RNDTL;
    When NO_DATA_FOUND Then Null;
    When Others Then
    ProcAudit;
    Raise_application_error (-20999,'20002 LG_ACTIVITY_RNHDRi_ah1--HDRi'|| SQLCODE||' '||SQLERRM);
    p_ReturnCd := 1;
end Insert_ACTIVITY_RNHDR;

--
Procedure Get_Time2Perform is

Begin
   IF v_cid is not null And
      nvl(p_ATTEMPTCNT,1) = 1
   Then
     Begin
       select min(ae.endstateTS) Into V_endstateTS
       from lg_activity_rnDTL d
       inner join Lg_activity_endstate ae on d.activityid = ae.activityid
       where d.cid= v_cid
       and   d.GlbDocFunctionCD = 'Request'
       and   ae.endstateFlg = 1;
     Exception
       When no_data_found Then V_endstateTS := Null;
       When Others then
       Raise_application_error (-20999,'20002 LG_ACTIVITY_RNHDRi_ah1-Get Time2Perform '|| SQLCODE||' '||SQLERRM);
     End;
     If V_endstateTS is not null then
       V_endstateTS_date := V_endstateTS;  -- Convert to date prior to computing Time2Perform
       v_Time2perform := (V_RCVDOCTS - V_endstateTS_date)*24*60*60; -- V_RCVDOCTS for response and V_endstateTS for request
  --     v_Time2perform := (V_RCVDOCTS - V_endstateTS)*24*60*60; -- V_RCVDOCTS for response and V_endstateTS for request
       If v_Time2perform < 0 Then
            v_Time2perform := 0;
       End If;
     End if;
   End IF;
 Null;
End Get_Time2Perform;
--
Procedure Do_Exception is
Begin
   IF    p_GLBBUSSIGNALCD in ('Acceptance Acknowledgment', 'Acceptance Acknowledge')   Then v_AACnt := 1;
   ELSIF p_GLBBUSSIGNALCD in ('Exception', 'General Exception')                        Then v_GECNT := 1;
   ELSIF p_GLBBUSSIGNALCD in ('Receipt Acknowledgment', 'Receipt Acknowledge')         Then v_RACNT := 1;
   ELSIF p_GLBBUSSIGNALCD =   'Receipt Acknowledgement Exception'                      Then v_RAECNT := 1;
   ELSIF p_GLBBUSSIGNALCD =   'Acceptance Acknowledgement Exception'                   Then v_AAECNT := 1;
   ELSE
       v_UnknownCnt := 1;
   END IF ;
End Do_Exception;
--
PROCEDURE Update_lg_summary is

    p_ReturnCd              pls_integer;
--- ProcessSummary
    v_REQACTIONCNT         pls_integer := 0;
    v_RESPACTIONCNT        pls_integer := 0;
    v_REQSIGNALCNT         pls_integer := 0;
    v_RESPSIGNALCNT        pls_integer := 0;
    v_REQUESTDURATIONSUM   pls_integer := 0;
    v_RESPONSEDURATIONSUM  pls_integer := 0;
    v_RAREQDURATIONSUM     pls_integer := 0;
    v_RARESPDURATIONSUM    pls_integer := 0;

Begin

If p_InitPartnerID  =  p_FrPartnerID Then  --   (from ===> To)
      If  p_GlbBusActionCD is not null then   -- Request
            v_ReqActionCNT       := 1;
            v_rnActionCnt        := 1;
            v_Action_duration_MS := FN_TIMESTAMPDIFF(p_RcvDocTS, p_endstateTS);
            v_RequestDuratioNSUM := v_Action_duration_MS;

      Elsif p_GLBBUSSIGNALCD is not null then  -- RA-Resp
            v_RespSignalCNT      := 1;
            v_rnSignalCnt        := 1;
            v_Signal_duration_MS := FN_TIMESTAMPDIFF(p_RcvDocTS, p_endstateTS);
            v_RARespDurationSUM  := v_Signal_duration_MS;

            Do_Exception;  --  Finish parameters setting for Lg summary Processing

      Else  --p_GlbBusActionCD is null and p_GLBBUSSIGNALCD is null
            v_UnknownCnt := 1;
      End if;
ElsIf  p_InitPartnerID = p_ToPartnerID Then  --   (To ===> From)
      If p_GlbBusActionCD is not null then    -- Response
            v_RespActionCNT       := 1;
            v_rnActionCnt         := 1;
            v_Action_duration_MS  := FN_TIMESTAMPDIFF(p_RcvDocTS, p_endstateTS);
            v_ResponseDurationSUM := v_Action_duration_MS;
            Get_Time2Perform;  -- TTP

      Elsif p_GLBBUSSIGNALCD is not null then  -- RA-Req
            v_ReqSignalCNT       := 1;
            v_rnSignalCnt        := 1;
            v_Signal_duration_MS := FN_TIMESTAMPDIFF(p_RcvDocTS, p_endstateTS);
            v_RAReqDurationSUM   := v_Signal_duration_MS;

            Do_Exception;
       Else  --p_GlbBusActionCD is null and p_GLBBUSSIGNALCD is null
            v_UnknownCnt := 1;
      End if;
Else  -- p_InitPartnerID Not equal to p_ToPartnerID or p_FrPartnerID
            v_UnknownCnt := 1;
End if;

-- Setup for 0A1 count
   Case
       When p_PROCESSCD='0A1'
       and  p_initPartnerId = p_frPartnerid    --p_GLBDOCFUNCTIONCD = 'Request'
       and  p_ref_INITBUSINESSID    is NOT null
       and  p_ref_PROCESSINSTANCEID is NOT null
   Then
           v2_ref_ProcessSummaryID := v_ref_ProcessSummaryID;
           v2_Ref_summaryID        := v_Ref_summaryID ;
           v2_ref_RcvDocTS         := v_ref_RcvDocTS ;
   Else
       v2_ref_ProcessSummaryID := Null;
       v2_Ref_summaryID        := Null;
       v2_ref_RcvDocTS         := Null;
   End Case;
	BEGIN
		INSERT INTO LG_UNPROCESSED_SUMMARY_RN
			(
			RCVDOCTS
			,DOCCNT
			,RCVPROCESSCNT
			,ACTIONCNT
			,SIGNALCNT
			,ACTIONDURATIONSUM
			,SIGNALDURATIONSUM
			,AACNT
			,GECNT
			,RACNT
			,RAECNT
			,AAECNT
			,TIMETOPERFORM
			,REFSUMMARYID
			,REFRCVDOCTS
			,SUMMARYID
			,UNKNOWNCNT
			)
		VALUES
			(
			v_rnhdr_rcvDocTS
			,1
			,v_rnProcessCnt
			,v_rnActionCnt
			,v_rnSignalCnt
			,v_Action_duration_MS
			,v_Signal_duration_MS
			,v_AACnt
			,v_GECnt
			,v_RACnt
			,v_RAECnt
			,v_AAECnt
			,nvl(v_time2perform, 0)
			,v2_ref_SummaryID
			,v2_ref_RcvDocTS
			,v_summaryID
			,v_UnknownCnt
			);
	EXCEPTION
		WHEN Others THEN
			p_ReturnCd := 1;
			ProcAudit;
			Raise_application_error (-20999,'20000 LG_ACTIVITY_RNHDRI_AH1 - Ins Unprocessed ' || SQLCODE|| ' '||SQLERRM);
	END;
--
BEGIN
INSERT INTO LG_UNPROCESSED_RN
(
	PROCESSSUMMARYID
,	RCVDOCTS
,	DOCCNT
,	RCVPROCESSCNT
,	REQACTIONCNT
,	RESPACTIONCNT
,	REQSIGNALCNT
,	RESPSIGNALCNT
,	REQDURATIONSUM
,	RESPDURATIONSUM
,	RAREQDURATIONSUM
,	RARESPDURATIONSUM
,	AACNT
,	GECNT
,	RACNT
,	RAECNT
,	AAECNT
,	REALTIMETOPERFORM
,	UNKNOWNCNT
,	REFPROCESSSUMMID
,	REFRCVDOCTS
)
VALUES
(
	v_PROCESSSUMMARYID
,	v_rnhdr_rcvDocTS
,	1
,	v_rnProcessCnt
,	v_ReqActionCNT
,	v_RespActionCNT
,	v_ReqSignalCNT
,	v_RespSignalCNT
,	v_RequestDuratioNSUM
,	v_ResponseDurationSUM
,	v_RAReqDurationSUM
,	v_RARespDurationSUM
,	v_AACnt
,	v_GECnt
,	v_RACnt
,	v_RAECnt
,	v_AAECnt
,	nvl(v_time2perform, 0)
,	v_UnknownCnt
,	v2_Ref_ProcessSummaryID
,	v2_ref_RcvDocTS
);
EXCEPTION
	WHEN Others THEN
		p_ReturnCd := 1;
		ProcAudit;
		Raise_application_error (-20999,'20000 LG_ACTIVITY_RNHDRI_AH1 - Ins Unprocessed ' || SQLCODE|| ' '||SQLERRM);
END;
End Update_lg_summary;
-------------------------------------------------------------------------------
--                                  M  A  I  N                               --
-------------------------------------------------------------------------------
BEGIN
	Check_length;
    IF p_ref_INITBUSINESSID    is NOT null   and
       p_ref_PROCESSINSTANCEID is NOT null
    then
      Get_CID(p_ref_INITBUSINESSID,
              p_ref_PROCESSINSTANCEID,
              v_cid,
              v_ref_rcvDocTS,
              v_ref_summaryID,
              v_ref_ProcessSummaryID) ;
      v_0A1_Flag := TRUE;
    END IF;
--
---    Insert_Xref;              -- seq 1 gives summaryID, ProcessSummaryID
    Insert_ACTIVITY_RNHDR;    -- seq 2 uses summaryID and ProcessSummaryID, gives processCnt
    IF v_rnDTL_Dup_Flag = FALSE Then
---        Update_lg_summary;        -- seq 3 uses summaryid, ProcessSummaryID and processCnt
	null;
    End If;

    p_ReturnCd := 0;
    ProcAudit;
EXCEPTION
    When NO_DATA_FOUND Then Null;
    When Others Then
    ProcAudit;
    Raise_application_error (-20999,'20003 LG_ACTIVITY_RNHDRi_ah1' || SQLCODE|| ' '||SQLERRM);
    p_ReturnCd := 1;
END LG_ACTIVITY_RNHDRI_ah1;
/


-- LG_ACTIVITY_RNHDRS_AH1.sql

CREATE OR REPLACE PROCEDURE         LG_Activity_RNHdrS_AH1 (
      p_ReturnCd     out pls_INTEGER
    , p_DebugFlg     in  pls_INTEGER
    , RC1              out db_ref.RCT          -- rows
    , RC2              out db_ref.rct_count    -- count
    , p_Locale         in  Varchar2
    , p_sortkey        in  varchar2            -- RcvDocTS only
    , p_PageSizeNbr    in  pls_Integer
    , p_PageNbr        in  pls_Integer
    , p_FROMPartnerId  in  varchar2            -- (All = NULL, One, Comma Seperated list of Id's)
    , p_TOPartnerId    in  varchar2            -- (All = NULL, One, Comma Seperated list of Id's)
    , p_STARTDATE      in  DATE                -- (Date and Time)
    , p_ENDDATE        in  DATE                -- (Date and Time)
    , p_PROTOCOLVER    in  VARCHAR2
    , p_PROCESSCD      in  VARCHAR2
    , p_PROCESSVER     in  VARCHAR2
    , p_PROCESSINSTANCEID  in varchar2
    , p_INITBUSINESSID     in  varchar2
    , p_CHANNELSTATUSID    in  pls_Integer     -- from LG_activity -- Ignore it 05/16/02
    , p_DESTTYPEID         in  pls_Integer
    , p_SYNCHRONOUSFLG     in  pls_Integer   -- = 1 is sync 0 = non-sync = null all
    , P_INITPARTNERTYPE   in varchar2 
    , P_TRADINGPARTNERTYPE   in varchar2 
    )
AS
/*
================================================================================
    Proc Name:      LG_Activity_RNHdrS_AH1
    Create Date:    01/04/2002
    Author          Vipin Goel
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2  Removed NLS 
    	2007-08-25 Bharani J     6.1.1  For Multiple Internal Partner Feature
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_BEGINTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
  v_startdate        varchar2(20) := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
  v_enddate          varchar2(20) := to_char(nvl(p_EndDATE,sysdate)    , 'dd-mm-yy hh24:mi:ss');
  v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(2000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  frPartnerName      PR_partner.partnerName%type;
  ToPartnerName      PR_partner.partnerName%type;
----- Types
Type char20Tab is table of varchar2(20);
Type char255Tab is table of varchar2(255);
Type num10Tab  is table of number(10);
array_ibusid    char20Tab  := char20Tab();
array_piid      char255Tab := char255Tab();
array_cid       num10Tab   := num10Tab();
array_rnbr      num10Tab   := num10Tab();
--
rnhdrTab       actRnHdr_Table := actRnHdr_Table();

   v_partners_int varchar2(500);
   v_target_int	varchar2(500);
----------
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
BEGIN
--    If p_DebugFlg =1  Then
        DB_ProcAuditLogI_AH1('LG_ACTIVITY_RNHDRS_AH1',v_BEGINTS,'Return_Code'||','||
        p_DebugFlg||','||
        p_Locale ||','||
        'RC1'||','||
        'RC2'||','||
        P_SortKey      ||','||
        P_PageSizeNbr  ||','||
        P_PageNbr      ||','||
        p_FROMPartnerId||','||
        p_TOPartnerId  ||','||
        To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
        To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
        p_PROTOCOLVER  ||','||
        p_PROCESSCD    ||','||
        p_PROCESSVER   ||','||
        p_PROCESSINSTANCEID ||','||
        p_INITBUSINESSID    ||','||
        p_CHANNELSTATUSID   ||','||
        p_DestTypeId        ||','||
        p_SYNCHRONOUSFLG    ||','||
        p_initpartnertype    ||','||
        p_tradingpartnertype
        ,0,0);
--        commit;
--    END if ;
END ProcAudit;
Procedure Where_Clause as
BEGIN
	v_where := ' WHERE 1=1 ';
              
	IF upper(p_InitPartnerType)='INTERNAL PARTNERS' then      
		       		
		v_Partners_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';
		v_where := v_where || ' AND InitPartnerId IN ' || v_Partners_Int ;	
                        		
		IF p_FROMPARTNERID is not null then		       
                       v_where := v_where || ' OR InitPartnerId IN ' || p_FROMPARTNERID ;		       
		END IF;
		v_where := v_where || ' ) ';             		
                     		
	END IF;
              
	IF upper(p_tradingpartnertype)='INTERNAL PARTNERS' then
			
		v_target_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';
                        		           
		v_where := v_where || ' AND TOPARTNERID IN ' || v_target_Int ;
                      
		IF p_TOPARTNERID is not null then
           	    v_where := v_where || ' OR TOPARTNERID IN ' || p_TOPARTNERID ;
           	END IF;                     		
		v_where := v_where || ' ) ';    		
	END IF;
               
               
	IF p_InitPartnerType IS NULL AND p_FROMPARTNERID is not null then
		v_where := v_where || ' AND InitPartnerId IN ' || p_FROMPARTNERID ;
	END IF;
                
	IF p_tradingpartnertype IS NULL AND p_TOPARTNERID is not null then
		v_where := v_where || ' AND TOPARTNERID IN ' || p_TOPARTNERID ;
	END IF;   
     
     
	v_where := v_where ||
	' AND  hdr.RCVDOCTS between '||chr(10)||' to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and'||
                                                chr(10)||' to_date('''|| v_endDate ||''',''dd-mm-yy hh24:mi:ss'')'||
	FN_Build_Where('hdr.PROTOCOLNAME',   'VARCHAR2'   , 'RosettaNet') ||
	FN_Build_Where('hdr.PROTOCOLVER',    'VARCHAR2'   , p_PROTOCOLVER) ||
	FN_Build_Where('hdr.PROCESSCD',      'VARCHAR2'   , p_PROCESSCD) ||
	FN_Build_Where('hdr.PROCESSVER',     'VARCHAR2'   , p_PROCESSVER) ||
	FN_Build_Where('hdr.PROCESSINSTANCEID', 'VARCHAR2_LIKE', p_PROCESSINSTANCEID) ||
	FN_Build_Where('hdr.INITBUSINESSID'   , 'VARCHAR2'     , p_INITBUSINESSID)    ||
	FN_Build_Where('hdr.DESTTYPEID'       , 'NUMBER'       , p_DESTTYPEID)        ||
	FN_Build_Where('hdr.SYNCHRONOUSFLG'   , 'NUMBER'       , p_SYNCHRONOUSFLG)
	;   
END Where_Clause;
--
-- Main
--
BEGIN
    p_ReturnCd:= 0;
     Where_Clause;
sql_inner :=
           'from LG_Activity_RNHdr hdr '
          ;
sql_outer :=
            'left outer join PR_partner frp   on hdr.InitPartnerId = frp.PartnerId
             left outer join PR_partner top   on hdr.ToPartnerId   = top.PartnerId';
             
sqlstmt := 'select hdr.cid,hdr.initbusinessid, hdr.PROCESSINSTANCEID '||sql_inner||' '||v_where ;
    if p_sortkey is not null then
        v_sortkey := upper(p_sortkey);
        sqlstmt := sqlstmt ||' order by '||v_sortkey;
    END if;
-- total row count Cursor
    if RC2%IsOpen then close RC2; END if;
    if NVL(p_pageNbr,0) in (0,1) then
        DB_Print('Sql_inner ', sql_inner);
        DB_Print('Sql_where ', v_where);
        EXECUTE IMMEDIATE 'select count(*)  '||sql_inner ||v_where INTO v_count;
        open RC2 for select v_count TotalRowsNbr from dual;
    END if;
-- Paging logic
   sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
     DB_Print('Sql ', sqlstmt);
-- Open Cursor
    if RC1%IsOpen then close RC1; END if;
    EXECUTE IMMEDIATE sqlstmt Bulk Collect INTO array_rnbr, array_cid, array_ibusid, array_piid;
-- Transfer activityID from array to table
       For i IN 1..array_rnbr.count LOOP
           RnHdrTab.extend;
           RnHdrTab(i) := actRnHdr_rec(array_rnbr(i), array_CId(i), array_ibusid(i), array_piid(i));
       END Loop;
       OPEN RC1 FOR
       SELECT /*+ FIRST_ROWS INDEX(hdr) */
           hdr.cid,
           hdr.INITPartnerId,
           frp.partnername FRPARTNERNAME,
           hdr.TOPartnerId,
           Top.partnername TOPARTNERNAME,
           hdr.PROCESSINSTANCEID,
           hdr.RCVDOCTS,
           hdr.PROTOCOLNAME,
           hdr.PROTOCOLVER,
           hdr.PROCESSCD,
           hdr.PROCESSVER,
           hdr.SYNCHRONOUSFLG,
           RTRIM(FN_GET_RTOBJNAME(hdr.ProcessCd, hdr.ProcessVer, 'Process')) ProcessName,
           t.Resourcekey,
           t.Description
        FROM
           TABLE(cast(rnhdrTab as actRnHdr_Table)) Y
           inner join LG_Activity_RNHdr hdr ON hdr.cid = Y.cid
                                       AND hdr.initbusinessID = y.initbusinessID
                                       AND hdr.PROCESSINSTANCEID = Y.PROCESSINSTANCEID
           left outer join PR_partner frp ON hdr.InitPartnerId = frp.PartnerId
           left outer join PR_partner top ON hdr.ToPartnerId = top.PartnerId
           left outer join PR_desttype t ON hdr.DestTypeId = t.DestTypeId
        ORDER BY Y.seq;
   ProcAudit;
   EXCEPTION
   WHEN OTHERS THEN
       ProcAudit;
       p_ReturnCd:= 1;
END ;
/


-- LG_ACTIVITY_RNHDRS_CID.sql

CREATE OR REPLACE PROCEDURE LG_ACTIVITY_RNHDRS_CID
    (p_ReturnCd    out integer
    , p_DebugFlg         in pls_integer
    , RC1           out db_ref.RCT
    , p_Locale      in Varchar2
    , p_CID         in pls_integer
    )
As
/*
================================================================================
    Proc Name:      LG_ACTIVITY_RNHDRS_CID
    Create Date:    04/18/02 11:46:11 AM
    Author          V.Goel
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2  Removed NLS    
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt   integer    :=0;
v_BeginTS  timestamp := FN_Get_Date; -- Performance related variables
v_ProcName          varchar2(30) := 'LG_ACTIVITY_RNHDRS_CID';
v_ExceptionFlg      integer := 0;
v_ParamsTxt         varchar2(4000) :=
                                        'p_ReturnCd'||','||p_DebugFlg
                                        ||','||p_Locale
                                        ||','||'RC1'
                                        ||','||p_CID;
BEGIN
   OPEN RC1 FOR
   SELECT
        h.cid
        ,   h.INITPARTNERID             INITPARTNERID
        ,   RTRIM(h.PROCESSINSTANCEID)  PROCESSINSTANCEID
        ,   RTRIM(h.PROCESSCD)          PROCESSCD
        ,   RTRIM(FN_GET_RTOBJNAME(h.ProcessCd, h.ProcessVer, 'Process')) ProcessName
        ,   RTRIM(h.PROCESSVER)         PROCESSVER
        ,   RTRIM(h.PROTOCOLNAME)       PROTOCOLNAME
        ,   RTRIM(h.PROTOCOLVER)        PROTOCOLVER
        ,   h.INITPARTNERID             INITPARTNERID
        ,   RTRIM(h.ToPartnerID)        ToPartnerID
        ,   RTRIM(frp.partnerName)      InitPartnerName
        ,   RTRIM(top.partnerName)      TradingPartnerName
        ,   h.RCVDOCTS
        ,   RTRIM(t.Resourcekey)        Resourcekey
        ,   t.Description				Description
		,   t.destname                  DESTDisplayNAME
   FROM
        LG_ACTIVITY_RNHDR h
        LEFT OUTER JOIN pr_Partner frp         on h.InitPartnerID = frp.partnerID
        LEFT OUTER JOIN pr_Partner top         on h.ToPartnerID   = top.partnerID
        LEFT outer join pr_desttype t   on h.desttypeid    = t.desttypeid
   WHERE h.CID    = p_CID
 ;

   p_ReturnCd := 0;
   DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

   EXCEPTION
       WHEN NO_DATA_FOUND THEN NULL;
       WHEN OTHERS THEN
           DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
           Raise_application_error (-20999,'20003 LG_ACTIVITY_RNHDRi_ah1' || SQLCODE|| ' '||SQLERRM);
           ROLLBACK WORK;
           p_ReturnCd := 1;
END;
/



-- LG_ACTIVITY_SUMMARYS_AH1.sql

CREATE OR REPLACE PROCEDURE         LG_ACTIVITY_SUMMARYS_AH1
(
      p_ReturnCd       out pls_INTEGER
    , p_DebugFlg        in  pls_INTEGER
    , RC1               out db_ref.RCT      -- rows
    , RC2               out db_ref.rct      -- count
    , p_Locale          in  Varchar2
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_FROMPARTNERID   in  Integer
    , p_TOPARTNERID     in  Integer
    , p_STARTDATE       in  TIMESTAMP
    , p_ENDDATE         in  TIMESTAMP
-- From
    , p_FrPACKAGENAME     in  varchar2
    , p_FrPACKAGEVER      in  varchar2
    , p_FrPROTOCOLNAME    in  VARCHAR2
    , p_FrPROTOCOLVer     in  VARCHAR2
    , p_FrPROCESSCD       in  VARCHAR2
    , p_FrPROCESSVER      in  VARCHAR2
-- To
    , p_ToPACKAGENAME     in  varchar2
    , p_ToPACKAGEVER      in  varchar2
    , p_ToPROTOCOLNAME    in  VARCHAR2
    , p_ToPROTOCOLVer     in  VARCHAR2
    , p_ToPROCESSCD       in  VARCHAR2
    , p_ToPROCESSVER      in  VARCHAR2
    , p_EndStateFlg       IN  Integer  -- lg_activity_endState.EndStateFlg: o= in process, 1=Success, 2=Failure, null= all)
    , p_eventCd           IN  Integer
    , p_DESTTYPEID        IN  Integer
)
AS
/*
================================================================================
    Proc Name:      LG_ACTIVITY_SUMMARYS_AH1
    Create Date:    01/04/2002
    Author          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2  Removed NLS    
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
  v_sortKey          varchar2(1000);
  sqlstmt            varchar2(4000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
----- Types
Type char65Tab is table of varchar2(65);
Type num10Tab  is table of number(10);
array_activityId char65Tab := char65Tab();
array_rnbr       num10Tab  := num10Tab();
--
activityIdTab  char65_Table := char65_Table();
----------
--
-- Procedures
--
Procedure ProcAudit is
begin
  DB_ProcAuditLogI_AH1('LG_ACTIVITY_SUMMARYS_AH1',v_beginTS,
    'p_ReturnCd'   ||','||
    p_DebugFlg     ||','||
    p_Locale       ||','||
    'RC1'          ||','||
    'RC2'          ||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_FROMPARTNERID||','||
    p_TOPARTNERID  ||','||
    To_char(p_STARTDATE,'DD-MON-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,'DD-MON-YY HH24:mi:ss')  ||','||
--    p_SRCTRGFLG    ||','||
    p_FrPACKAGENAME  ||','||
    p_FrPACKAGEVER   ||','||
    p_FrPROTOCOLNAME ||','||
    p_FrPROTOCOLVer  ||','||
    p_FrPROCESSCD    ||','||
    p_FrPROCESSVER   ||','||
    p_ToPACKAGENAME  ||','||
    p_ToPACKAGEVER   ||','||
    p_ToPROTOCOLNAME ||','||
    p_ToPROTOCOLVer  ||','||
    p_ToPROCESSCD    ||','||
    p_ToPROCESSVER   ||','||
    p_EndStateFlg    ||','||
    p_eventCd
    ,0,0);
End ProcAudit;
Procedure where_clause as
BEGIN
    v_where := ' Where  Las.DayHour between :startDate and :endDate'||
      FN_Build_Where('x.FRPARTNERID',   'NUMBER', p_FROMPARTNERID   )||
      FN_Build_Where('x.TOPARTNERID',   'NUMBER', p_ToPARTNERID     );
--  if p_SRCTRGFLG = 0 then -- From
    v_where := v_where ||
    FN_Build_Where('x.FRPACKAGENAME',  'VARCHAR2', p_FrPACKAGENAME        )||
    FN_Build_Where('x.FRPACKAGEVER',   'VARCHAR2', p_FrPACKAGEVER         )||
    FN_Build_Where('x.FrPROTOCOLNAME', 'VARCHAR2', p_FrPROTOCOLNAME       )||
    FN_Build_Where('x.FrPROTOCOLVER',  'VARCHAR2', p_FrPROTOCOLVER        )||
    FN_Build_Where('x.FrPROCESSCD',    'VARCHAR2', p_FrPROCESSCD          )||
    FN_Build_Where('x.FrPROCESSVER',   'VARCHAR2', p_FrPROCESSVER         );
--  elsif p_SRCTRGFLG = 1 then -- To
    v_where := v_where ||
    FN_Build_Where('x.ToPACKAGENAME',  'VARCHAR2', p_ToPACKAGENAME        )||
    FN_Build_Where('x.ToPACKAGEVER',   'VARCHAR2', p_ToPACKAGEVER         )||
    FN_Build_Where('x.ToPROTOCOLNAME', 'VARCHAR2', p_ToPROTOCOLNAME       )||
    FN_Build_Where('x.ToPROTOCOLVER',  'VARCHAR2', p_ToPROTOCOLVER        )||
    FN_Build_Where('x.ToPROCESSCD',    'VARCHAR2', p_ToPROCESSCD          )||
    FN_Build_Where('x.ToPROCESSVER',   'VARCHAR2', p_ToPROCESSVER         )||
    FN_Build_Where('se.eventCd',       'NUMBER'  , p_eventCd              )||
    FN_Build_Where('x.DESTTYPEID',     'NUMBER'  , p_DESTTYPEID);

    IF P_ENDSTATEFLG IS NOT NULL THEN
        IF P_ENDSTATEFLG = 0 THEN
            v_where := v_where|| ' and a.activityId  NOT in (select activityId from lg_activity_endstate) ';
        END IF;
        IF (P_ENDSTATEFLG = 1  OR P_ENDSTATEFLG = 2) THEN
            v_where := v_where || ' and ae.ENDSTATEFLG = '|| p_ENDSTATEFLG;
        END IF;
    END IF;
END where_clause;
--                                 M  a  i  n
BEGIN
    p_ReturnCd := 0;
    where_clause;
    DB_Print('Where Clause', v_Where);

    sql_inner := ' from LG_Summary_Xref_vw x INNER JOIN LG_Summary s  on x.summaryID = s.summaryID'||
                 ' Inner join LG_Activity_Summary Las on s.summaryID = Las.summaryID and s.dayHour = Las.DayHour'||
                 ' Left Outer  join LG_Summary_Event se on s.summaryID = se.summaryID and s.dayHour = se.DayHour'||
                 ' Inner Join LG_Activity a on a.activityId =  Las.VuId '||
                 ' left outer join lg_activity_endState ae on a.activityId     =  ae.activityId'||
                 ' left outer join pr_partner frp  on a.FRPARTNERIDHdr = frp.partnerId '||
                 ' left outer join pr_partner top  on a.ToPARTNERIDHdr = top.partnerId ';

sqlstmt := 'SELECT DISTINCT T.ActivityId FROM ( ' || 'select a.activityid '||sql_inner||' '||v_where ;

    if p_sortkey is not null then
       v_sortkey := upper(p_sortkey);
       v_sortkey := replace(v_sortkey,'FRPARTNERNAME','FRP.PARTNERNAME');
       v_sortkey := replace(v_sortkey,'TOPARTNERNAME','TOP.PARTNERNAME');
       v_sortkey := replace(v_sortkey,'FRPROCESSCD'  ,'x.FRPROCESSCD');
       sqlstmt := sqlstmt ||' ORDER BY '|| NVL(v_sortkey, '');
    end if;
-- total row count Cursor
    if RC2%IsOpen then close RC2; end if;
    if NVL(p_pageNbr,0) in (0,1) then
        OPEN RC2 for 'SELECT COUNT(DISTINCT a.activityid) TotalRowsNbr '||sql_inner ||v_where Using p_StartDate, p_EndDate;
     end if;
-- Paging logic
   sqlstmt := sqlstmt || ' ) T';
   sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
     DB_Print('Sql paging', sqlstmt);
-- Open Cursor
    execute immediate sqlstmt Bulk Collect INTO array_rnbr, array_ActivityId Using p_StartDate, p_EndDate;
--    dbms_output.put_line('Last rnbr '||array_rnbr.LAST );
-- Transfer activityID from array to table
    For i IN 1..array_ActivityId.count LOOP
      activityIdTab.extend;
      activityIdTab(i) := char65_rec(array_rnbr(i), array_ActivityId(i));
      dbms_output.put_line('ActivityID '||array_ActivityId(i) );
    End Loop;

    dbms_output.put_line('Temporary table (Arrays) done ' );
    if RC1%IsOpen then close RC1; end if;

    open RC1 for
    select   /*+ First_Rows */
        rtrim(frp.partnername)    "FRPARTNERNAME",
        rtrim(a.FRPARTNERIDHdr)   "FRPARTNERID",
        rtrim(a.FRBUSINESSIDHDR)  "FRBUSINESSID",
        rtrim(Top.partnername)    "TOPARTNERNAME",
        a.TOPARTNERIDHdr         "TOPARTNERID",
        a.ToBUSINESSIDHdr        "ToBUSINESSID",
        rtrim(a.FRPACKAGENAME)   FRPACKAGENAME,
        rtrim(a.FRPACKAGEVER)    FRPACKAGEVER,
        rtrim(a.TOPACKAGENAME)   TOPACKAGENAME,
        rtrim(a.TOPACKAGEVER)    TOPACKAGEVER,
        a.FRPROTOCOLNAME,
        a.FRPROTOCOLVER,
        a.TOPROTOCOLNAME,
        a.TOPROTOCOLVER,
        RTRIM(FN_GET_RTOBJNAME(a.FrPROCESSCD, a.FrPROCESSVER, 'Process')) FrProcessName,
        rtrim(a.FrPROCESSCD) FrPROCESSCD,
        rtrim(a.FRPROCESSVER) FRPROCESSVER,
        RTRIM(FN_GET_RTOBJNAME(a.ToPROCESSCD, a.ToPROCESSVER, 'Process')) ToProcessName,
        rtrim(a.ToPROCESSCD)  ToPROCESSCD,
        rtrim(a.TOPROCESSVER)  TOPROCESSVER,
        rtrim(a.DocId) DocId,
        rtrim(a.DocTS) DocTS,
        a.ActivityId,
        a.RcvDocTS,
        a.CHANNELSTATUSID,
        --a.SYNCHRONOUSFLG,
        CASE WHEN fradtl.SYNCHRONOUSFLG = 0 AND COALESCE(toadtl.SYNCHRONOUSFLG, 0) = 0 THEN 0 ELSE 1
        END SYNCHRONOUSFLG,
        rtrim(t.Resourcekey) Resourcekey,
        rtrim(t.Description) Description,
        fradtl.vuid frVuid,
        toadtl.vuid ToVuid,
        fradtl.MSGRAWLENGTH frMSGRAWLENGTH,
        toadtl.MSGRAWLENGTH ToMSGRAWLENGTH,
        fradtl.MSGCONTENTLENGTH frMSGCONTENTLENGTH,
        toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH,
               fradtl.ORIGINALFILENAME frFILENAME,
               toadtl.ORIGINALFILENAME toFILENAME,
               fradtl.ENCODING frENCODING,
               toadtl.ENCODING toENCODING,
        ae.EndStateTS,
        nvl(ae.EndStateFlg,0) EndStateFlg,
        rtrim(wf.Resourcekey) WfResourckey,
        rtrim(wf.Description) WfDescription
    from
        TABLE(cast(activityIdTab as char65_Table)) Y
        left outer join lg_activity a                    on Y.id             = a.activityId
        left outer join lg_activity_endState ae          on a.activityId     =  ae.activityId
        left outer join lg_activity_dtl_parent_vw fradtl on a.activityId     = fradtl.activityId
        left outer join lg_activity_dtl_child_vw  toadtl on a.activityId     = toadtl.activityId
        left outer join pr_partner frp                   on a.FRPARTNERIDHdr = frp.partnerId
        left outer join pr_partner top                   on a.ToPARTNERIDHdr = top.partnerId
        LEFT OUTER JOIN PR_RoutingChannel ch        ON a.ChannelID               = ch.ChannelID
        LEFT OUTER JOIN PR_WorkFlow wf       on ch.workflowID             = wf.workflowID
        LEFT OUTER JOIN PR_DestType t        on t.desttypeid     = a.desttypeid
    order by Y.seq;


    ProcAudit;
    Exception
        when others then
            ProcAudit;
            p_ReturnCd := 1;
            dbms_output.put_line('Exception Happened ' );
END LG_ACTIVITY_SUMMARYS_AH1;
/


-- LG_ARCH_CFG_PARAMS.sql

CREATE OR REPLACE PROCEDURE LG_ARCH_CFG_PARAMS
	(
	 p_ReturnCd	OUT	INTEGER
	,  p_DebugFlg	IN	INTEGER
	,  RC1 out db_ref.RCT
	,  p_ARCHTASKID	IN INTEGER
	,  p_CFGPARAMID	IN INTEGER
	)
AS

/*
	================================================================================
	Proc Name:	LG_ARCH_CFG_PARAMS
	Create Date:	2008-07-31 14:07:22
	Author:		SYSTEM

	Desc:	Parameters used for a particular archiver task
				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 SYSTEM	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	 v_RowCnt		integer Default 0;
	 v_BeginTS	 TimeStamp	; -- Performance related variables
	 v_ProcName	varchar(30)	Default 'LG_ARCH_CFG_PARAMS';
	 v_ExceptionFlg	integer	Default 0;
	 v_ExceptionCode	varchar(30);
	 v_PK_Value	INTEGER Default 0;
	 v_SQLErrm	 Varchar(255) ;
	 v_chkpt		Char(1) default '0';
	 v_msg		Varchar(500);
	 v_RowTS		INTEGER Default 0;
	 
	 v_ParamsTxt       varchar2(4000) Default
	 	 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 	 		||', p_ARCHTASKID=>'||p_ARCHTASKID||CHR(10)
	 	 		||', p_CFGPARAMID=>'||p_CFGPARAMID||CHR(10)
		;
	 
	BEGIN
	
	IF p_ARCHTASKID IS NOT NULL AND p_CFGPARAMID IS NOT NULL THEN
		OPEN RC1 FOR
		SELECT	prm.CFGPARAMID
			,prm.ARCHCFGID
			,prm.PARTNERID
			,prm.PACKAGE_RO_ID
			,prm.PROTOCOL_RO_ID
			,prm.PROCESS_RO_ID
			,RTRIM(PARTNER) PARTNER
			,RTRIM(PACKAGENAME) PACKAGENAME
			,RTRIM(PACKAGEVER) PACKAGEVER
			,RTRIM(PROTOCOLNAME) PROTOCOLNAME
			,RTRIM(PROTOCOLVER) PROTOCOLVER
			,RTRIM(PROCESSNAME) PROCESSNAME
			,RTRIM(PROCESSVER) PROCESSVER
			,prm.STATUSFLAG
			,prm.ROWTS
			,prm.CREATEDATE
		FROM	LG_ARCH_CFG_PARAM prm
		INNER JOIN lg_arch_task tk ON tk.archCfgID = prm.ArchCfgId
		WHERE	tk.TASKID = p_ArchTaskID
		AND	prm.CFGPARAMID = p_CFGPARAMID
		;
				
	ELSIF p_ARCHTASKID IS NOT NULL THEN
		OPEN RC1 FOR
		SELECT	prm.CFGPARAMID
			,prm.ARCHCFGID
			,prm.PARTNERID
			,prm.PACKAGE_RO_ID
			,prm.PROTOCOL_RO_ID
			,prm.PROCESS_RO_ID
			,RTRIM(PARTNER) PARTNER
			,RTRIM(PACKAGENAME) PACKAGENAME
			,RTRIM(PACKAGEVER) PACKAGEVER
			,RTRIM(PROTOCOLNAME) PROTOCOLNAME
			,RTRIM(PROTOCOLVER) PROTOCOLVER
			,RTRIM(PROCESSNAME) PROCESSNAME
			,RTRIM(PROCESSVER) PROCESSVER
			,prm.STATUSFLAG
			,prm.ROWTS
			,prm.CREATEDATE
		FROM	LG_ARCH_CFG_PARAM prm
		INNER JOIN lg_arch_task tk ON tk.archCfgID = prm.ArchCfgId
		WHERE	tk.TASKID = p_ArchTaskID
		;
	
	END IF;
	 DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END LG_ARCH_CFG_PARAMS;
/


-- LG_ARCH_CFG_PARAMS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_ARCH_CFG_PARAMS_AH1
(
		p_ReturnCd OUT	INTEGER
	,	p_DebugFlg IN	INTEGER
	,       RC1 out db_ref.RCT
	,       RC2 out db_ref.RCT
	,	p_sortkey IN	VARCHAR2
	,	p_PageSizeNbr IN	INTEGER
	,	p_PageNbr IN	INTEGER
	,	p_ARCHTASKID IN	INTEGER
)
AS


/*
	================================================================================
	Proc Name:	LG_ARCH_CFG_PARAMS_AH1
	Create Date:	2008-07-31 14:07:22
	Author:		SYSTEM

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	v_RowCnt	integer Default 0;
	v_BeginTS       timestamp Default FN_Get_Date;  -- Performance related variables
	 v_ProcName	varchar2(30)	Default 'LG_ARCH_CFG_PARAMS_AH1';
	v_ExceptionFlg	integer	Default 0;
	v_ExceptionCode	varchar2(30);
	v_PK_Value	INTEGER Default 0;
	v_SQLErrm	Varchar2(255) ;
	v_chkpt		Char(1) default '0';
	v_RowTS		INTEGER Default 0;

	v_where		varchar2(1000);
	v_count		Integer	DEFAULT 0;
	v_sortKey	varchar2(1000);
	v_sqlStmt	varchar2(4000);

	v_ParamsTxt     varchar2(4000) Default
	 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 			 	||', p_ARCHTASKID=>'||p_ARCHTASKID||CHR(10)
		;

BEGIN

	p_ReturnCd := 0;
	v_Chkpt:='1';
	v_where := ' Where 1 = 1 '||
		FN_Build_Where('tk.TASKID', 'NUMBER', p_ARCHTASKID) 
	;
	 v_SQLStmt :=  'SELECT '
			|| 'tp.CFGPARAMID '
			|| ',tp.ARCHCFGID '
			|| ',tp.PARTNERID '
			|| ',tp.PACKAGE_RO_ID '
			|| ',tp.PROTOCOL_RO_ID '
			|| ',tp.PROCESS_RO_ID '
			|| ',RTRIM(tp.PARTNER) PARTNER '
			|| ',RTRIM(tp.PACKAGENAME) PACKAGENAME '
			|| ',RTRIM(tp.PACKAGEVER) PACKAGEVER '
			|| ',RTRIM(tp.PROTOCOLNAME) PROTOCOLNAME '
			|| ',RTRIM(tp.PROTOCOLVER) PROTOCOLVER '
			|| ',RTRIM(tp.PROCESSNAME) PROCESSNAME '
			|| ',RTRIM(tp.PROCESSVER) PROCESSVER '
			|| ',tp.STATUSFLAG '
			|| ',tp.ROWTS '
			|| ',tp.CREATEDATE '
		|| 'FROM	LG_ARCH_CFG_PARAM tp '
		|| 'INNER JOIN lg_arch_task tk ON tk.ArchCfgID = tp.ArchCfgId '
		|| v_where
	;
	-------------------------
	-- Total row count Cursor
	-------------------------
	IF COALESCE(p_pageNbr,0) in (0,1) THEN
		v_Chkpt :='2';
		EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM ('||v_Sqlstmt ||') ' INTO v_Count;
		OPEN RC2 For
			SELECT v_count TotalRowsNbr FROM DUAL;
	END IF;

	IF p_SortKey IS NOT NULL THEN
		 v_Chkpt :='3';
		 v_SortKey := UPPER(p_SortKey);
		 v_sqlstmt := v_sqlstmt ||' order by ' ||v_sortkey; 
	END IF;	

	---------------
	-- Paging logic
	---------------
	IF p_PageSizeNbr >= 0 THEN
		v_Chkpt :='4';
		v_SQLstmt := FN_PAGING_LOGIC(p_pageNbr, p_pageSizeNbr, v_Sqlstmt);
	END IF;
		
	v_Chkpt :='5';
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 for v_SQLstmt;

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END LG_ARCH_CFG_PARAMS_AH1;
/


-- LG_ARCH_TASKS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_ARCH_TASKS_AH1
	(
	p_ReturnCd OUT	INTEGER
	,p_DebugFlg IN	INTEGER
	,RC1 out db_ref.RCT
	,RC2 out db_ref.RCT
	,p_sortkey IN	VARCHAR2
	,p_PageSizeNbr IN INTEGER
	,p_PageNbr IN	INTEGER
	,p_TASKID IN	INTEGER
	,p_CONFIG	IN VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:	LG_ARCH_TASKS_AH1
	Create Date:	2008-07-31 14:07:19
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	v_RowCnt	integer Default 0;
	v_BeginTS       timestamp Default FN_Get_Date;  -- Performance related variables
	v_ProcName	varchar2(30)	Default 'LG_ARCH_TASKS_AH1';
	v_ExceptionFlg	integer	Default 0;
	v_ExceptionCode	varchar2(30);
	v_PK_Value	INTEGER Default 0;
	v_SQLErrm	Varchar2(255) ;
	v_chkpt		Char(1) default '0';
	v_RowTS		INTEGER Default 0;

	v_where		varchar2(1000);
	v_count		Integer	DEFAULT 0;
	v_sortKey	varchar2(1000);
	v_sqlStmt	varchar2(4000);

	v_ParamsTxt     varchar2(4000) Default
	 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 			 		||', p_TASKID=>'||p_TASKID||CHR(10)
		;

BEGIN

	p_ReturnCd := 0;
	v_Chkpt:='1';
	v_where := ' Where 1 = 1 '||
		FN_Build_Where('TASKID', 'NUMBER', p_TASKID) 
	;
	 v_SQLStmt :=  'SELECT '
			|| 'tk.TASKID '
			|| ',tk.SCHEDULERTASKID '
			|| ',tk.STATUSFLAG '
			|| ',tk.CREATEDATE '
			|| ',tk.TASKENDTS '
			|| ',tk.UPDATEDATE '
			|| ',tk.ROWTS '
			|| ',cf.KEEP_NUMDAYS '
			|| ',cf.ARCHIVEFLAG '
			|| ',RTRIM(cf.ARCHDEST_FL) ARCHDEST_FL '
			|| ',RTRIM(cf.ARCHDEST_DB) ARCHDEST_DB '
			|| ',cf.DAYOFMONTH '
			|| ',cf.DAYOFWEEK '
			|| ',cf.HOUROPT '
			|| ',cf.MINUTEOPT '
			|| ',cf.FREQDAYOFMONTH '
			|| ',cf.FREQDAYOFWEEK '
			|| ',cf.SCHED_TYPE '
		|| 'FROM	LG_ARCH_TASK tk '
		|| 'INNER JOIN	LG_ARCH_CFG cf ON cf.ARCHCFGID = tk.ARCHCFGID '
		|| v_where
	;
	IF p_CONFIG = 'RESTORE' THEN
		v_SQlStmt := v_SQLStmt || ' AND cf.SCHEDULERTASKID = ''RESTORE'' ';
	ELSE
		v_SQlStmt := v_SQLStmt || ' AND cf.SCHEDULERTASKID <> ''RESTORE'' ';
	END IF;
	-------------------------
	-- Total row count Cursor
	-------------------------
	IF COALESCE(p_pageNbr,0) in (0,1) THEN
		v_Chkpt :='2';
		EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM ('||v_Sqlstmt ||') ' INTO v_Count;
		OPEN RC2 For
			SELECT v_count TotalRowsNbr FROM DUAL;
	END IF;

	IF p_SortKey IS NOT NULL THEN
		 v_Chkpt :='3';
		 v_SortKey := UPPER(p_SortKey);
		 v_SortKey := REPLACE(v_sortKey, 'CREATEDATE', 'tk.CREATEDATE');
		 v_sqlstmt := v_sqlstmt ||' order by ' ||v_sortkey; 
	END IF;	

	---------------
	-- Paging logic
	---------------
	IF p_PageSizeNbr >= 0 THEN
		v_Chkpt :='4';
		v_SQLstmt := FN_PAGING_LOGIC(p_pageNbr, p_pageSizeNbr, v_Sqlstmt);
	END IF;
		
	v_Chkpt :='5';
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 for v_SQLstmt;

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END LG_ARCH_TASKS_AH1;
/


-- LG_ARCH_TASK_COMMFLSYSI.sql

CREATE OR REPLACE PROCEDURE LG_ARCH_TASK_COMMFLSYSI
	(
		p_ReturnCd OUT	INTEGER
	,	p_DebugFlg IN	INTEGER
	,	p_TASKID IN	NUMBER
	,	p_SUBVOLNAME IN	VARCHAR2 
	,	p_TASKTYPEID IN	INTEGER
	,	p_NUMFILES IN	INTEGER
	,	p_StartTS IN	TIMESTAMP
	,	p_EndTS	IN	TIMESTAMP
	)
AS

/*
	================================================================================
	Proc Name:	LG_ARCH_TASK_COMMFLSYSI
	Create Date:	2008-07-31 14:07:20
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	 v_RowCnt		integer Default 0;
	 v_BeginTS	 TimeStamp	; -- Performance related variables
	 v_ProcName	varchar2(30)	Default 'LG_ARCH_TASK_COMMFLSYSI';
	 v_ExceptionFlg	integer	Default 0;
	 v_ExceptionCode	varchar2(30);
	 v_PK_Value	INTEGER Default 0;
	 v_SQLErrm	 varchar2(255) ;
	 v_chkpt		Char(1) default '0';
	 v_msg		varchar2(500);
	 v_RowTS		INTEGER Default 0;
	 
	 v_ParamsTxt       varchar2(4000) Default
	 	 	 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 	 	 		||', p_TASKID=>'||p_TASKID||CHR(10)
	 	 	 		||', p_SUBVOLNAME=>'||p_SUBVOLNAME||CHR(10)
	 	 	 		||', p_TASKTYPEID=>'||p_TASKTYPEID||CHR(10)
	 	 	 		||', p_NUMFILES=>'||p_NUMFILES||CHR(10)
	 	 	 		||', p_StartTS=>'||p_StartTS||CHR(10)
	 	 	 		||', p_EndTS=>'||p_EndTS||CHR(10)
	 	 	 		;
	
BEGIN
	v_chkpt :='1';

	UPDATE	LG_ARCH_TASK_COMMFLSYS ux
	SET	
		ux.NUMFILES = ux.NUMFILES + COALESCE(p_NUMFILES,0)
		,ux.RowTS = RowTS + 1
		,ux.UpdateDate = p_EndTS
	WHERE	ux.TASKID = p_TASKID
	AND	ux.SUBVOLNAME = p_SUBVOLNAME
	AND	ux.TASKTYPEID = p_TASKTYPEID
	;

	v_RowCnt := SQL%ROWCOUNT;
	IF v_RowCnt = 0 THEN
		p_ReturnCd := 0;
		v_chkpt :='2';
		
		INSERT INTO LG_ARCH_TASK_COMMFLSYS(
			CFSTASKID
			,TASKID
			,SUBVOLNAME
			,TASKTYPEID
			,NUMFILES
			,CREATEDATE
			,UPDATEDATE
		) VALUES (
			LG_ARCH_TASK_COMMFLSYS_SEQ.nextval
			,p_TASKID
			,p_SUBVOLNAME
			,p_TASKTYPEID
			,p_NUMFILES
			,p_StartTS
			,COALESCE(p_EndTS, p_StartTS)
		);
		
	END IF;

	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		RAISE_APPLICATION_ERROR (-20999,'20000 LG_ARCH_TASK_COMMFLSYSI' || SQLCODE|| ' '||SQLERRM);
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
		RAISE_APPLICATION_ERROR (-20999,'20001 LG_ARCH_TASK_COMMFLSYSI' || SQLCODE|| ' '||SQLERRM);
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 LG_ARCH_TASK_COMMFLSYSI' || SQLCODE|| ' '||SQLERRM);		
END LG_ARCH_TASK_COMMFLSYSI;
/


-- LG_ARCH_TASK_EXPORTDBI.sql

CREATE OR REPLACE PROCEDURE LG_ARCH_TASK_EXPORTDBI
	(
	p_ReturnCd OUT	INTEGER
	,p_DebugFlg IN	INTEGER
	,p_TASKID IN	INTEGER
	,p_SUBVOLNAME IN	varchar2
	,p_TASKPARAMID IN INTEGER
	,p_NUMROWS IN	INTEGER
	,p_ARCHFILENAME	IN varchar2
	,p_StartTS IN	TIMESTAMP
	)
AS

/*
	================================================================================
	Proc Name:	LG_ARCH_TASK_EXPORTDBI
	Create Date:	2008-07-31 14:07:20
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	 v_RowCnt		integer Default 0;
	 v_ProcName	varchar2(30)	Default 'LG_ARCH_TASK_EXPORTDBI';
	 v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
	 v_ExceptionFlg	integer	Default 0;
	 v_ExceptionCode	varchar2(30);
	 v_PK_Value	INTEGER Default 0;
	 v_SQLErrm	 varchar2(255) ;
	 v_chkpt		Char(1) default '0';
	 v_msg		varchar2(500);
	 v_RowTS		INTEGER Default 0;
	 v_ParamsTxt       varchar2(4000) Default
	 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	 		||', p_TASKID=>'||p_TASKID||CHR(10)
	 		||', p_SUBVOLNAME=>'||p_SUBVOLNAME||CHR(10)
	 		||', p_TASKPARAMID=>'||p_TASKPARAMID||CHR(10)
	 		||', p_NUMROWS=>'||p_NUMROWS||CHR(10)
	 		||', p_ARCHFILENAME=>'||p_ARCHFILENAME||CHR(10)
	 		||', p_StartTS=>'||p_StartTS||CHR(10)
		;
	
BEGIN
	
	p_ReturnCd := 0;
	
	v_chkpt:='1';
	
	INSERT INTO LG_ARCH_TASK_EXPORTDB(
		EXPORTTASKID
		,TASKID
		,SUBVOLNAME
		,TASKPARAMID
		,NUMROWS
		,ARCHFILENAME
		,CREATEDATE
		,UPDATEDATE
	) VALUES (
		LG_ARCH_TASK_EXPORTDB_SEQ.nextval
		,p_TASKID
		,p_SUBVOLNAME
		,p_TASKPARAMID
		,COALESCE(p_NUMROWS, -1)
		,p_ARCHFILENAME
		,p_StartTS
		,v_BeginTS
	) ;
	v_chkpt:='2';
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		RAISE_APPLICATION_ERROR (-20999,'20000 LG_ARCH_TASK_EXPORTDBI' || SQLCODE|| ' '||SQLERRM);
	When DUP_VAL_ON_INDEX then
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 LG_ARCH_TASK_EXPORTDBI' || SQLCODE|| ' '||SQLERRM);		
END LG_ARCH_TASK_EXPORTDBI;
/


-- LG_ARCH_TASK_EXPORTDBS.sql

CREATE	OR REPLACE PROCEDURE LG_ARCH_TASK_EXPORTDBS
	(
	p_ReturnCd OUT	INTEGER
	,p_DebugFlg IN	INTEGER
	,RC1 out db_ref.RCT
	,p_EXPORTTASKID	IN INTEGER
	)
AS
/*
	================================================================================
	Proc Name:	LG_ARCH_TASK_EXPORTDBS
	Create Date:	2008-07-31 14:07:20
	Author:		SYSTEM

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 SYSTEM	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	v_RowCnt	integer Default 0;
	v_BeginTS	timestamp Default FN_Get_Date; -- Performance related variables
	v_ProcName	varchar2(30)	Default 'LG_ARCH_TASK_EXPORTDBS';
	v_ExceptionFlg	integer	Default 0;
	v_ExceptionCode	varchar2(30);
	v_ParamsTxt	varchar2(4000) ;
	v_PK_Value	INTEGER Default 0;
	v_SQLErrm	varchar2(255) ;
	v_msg		varchar2(500);
	v_RowTS		INTEGER Default 0;

BEGIN

	p_ReturnCd := 0;

	IF p_EXPORTTASKID IS NOT NULL THEN
		OPEN RC1 For
			SELECT
				EXPORTTASKID
				,TASKID
						,RTRIM(SUBVOLNAME) SUBVOLNAME
				,TASKPARAMID
		,RESTOREFLAG
				,ROWTS
				,RTRIM(ARCHFILENAME) ARCHFILENAME
				,CREATEDATE
			FROM	LG_ARCH_TASK_EXPORTDB
			WHERE	EXPORTTASKID = p_EXPORTTASKID;
	ELSE
		 OPEN RC1 For
			SELECT
				EXPORTTASKID
				,TASKID
				,RTRIM(SUBVOLNAME) SUBVOLNAME
				,TASKPARAMID
		,RESTOREFLAG
				,ROWTS
				,RTRIM(ARCHFILENAME) ARCHFILENAME
				,CREATEDATE
			FROM	LG_ARCH_TASK_EXPORTDB;
	END IF;
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END LG_ARCH_TASK_EXPORTDBS;
/


-- LG_ARCH_TASK_EXPORTDBU.sql

CREATE OR REPLACE PROCEDURE LG_ARCH_TASK_EXPORTDBU
	(
	        p_ReturnCd out	integer
	,       p_DebugFlg in INTEGER
	,       RC1 out db_ref.RCT
	,	p_EXPORTTASKID IN INTEGER
	,	p_TASKID IN INTEGER
	,	p_SUBVOLNAME IN VARCHAR2
	,	p_TASKPARAMID IN INTEGER
	,	p_RESTOREFLAG IN INTEGER
	,	p_ROWTS IN INTEGER
	,	p_ARCHFILENAME IN VARCHAR2 
	)
AS

/*
	================================================================================
	Proc Name:	LG_ARCH_TASK_EXPORTDBU
	Create Date:	2008-07-31 14:07:21
	Author:		SYSTEM

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	 v_RowCnt		integer Default 0;
	 v_BeginTS		TimeStamp	; -- Performance related variables
	 v_ProcName		varchar2(30)	Default 'LG_ARCH_TASK_EXPORTDBU';
	 v_ExceptionFlg		integer	Default 0;
	 v_ExceptionCode	varchar2(30);
	 
	 v_PK_Value		INTEGER Default 0;
	 v_SQLErrm		Varchar2(255) ;
	 v_chkpt		Char(1) default '0';
	 v_msg			Varchar2(500);
	 v_RowTS		INTEGER Default 0;
	 v_ParamsTxt		varchar2(4000) Default
		 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		 		||', p_TASKID=>'||p_TASKID||CHR(10)
		 		||', p_SUBVOLNAME=>'||p_SUBVOLNAME||CHR(10)
		 		||', p_TASKPARAMID=>'||p_TASKPARAMID||CHR(10)
		 		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		 		||', p_ARCHFILENAME=>'||p_ARCHFILENAME||CHR(10)
	;
	 
BEGIN 
	v_chkpt :='1';
	
	Update LG_ARCH_TASK_EXPORTDB ux
	SET
		 ux.TASKID = COALESCE( p_TASKID, ux.TASKID)
		, ux.SUBVOLNAME = COALESCE( p_SUBVOLNAME, ux.SUBVOLNAME)
		, ux.TASKPARAMID = COALESCE( p_TASKPARAMID, ux.TASKPARAMID)
		, ux.RESTOREFLAG = COALESCE( p_RESTOREFLAG, ux.RESTOREFLAG)
		, ux.ARCHFILENAME = COALESCE( p_ARCHFILENAME, ux.ARCHFILENAME)
		, ux.RowTS = p_RowTS + 1
		, ux.UPDATEDATE= v_BeginTS
	WHERE	ux.EXPORTTASKID = p_EXPORTTASKID
	And	ux.RowTS = p_RowTS ;
		
	v_RowTS := p_RowTS + 1;
	v_chkpt :='2';
	
	Open RC1 FOR Select
		p_TASKID id,v_rowTS ROWTS From Dual;
	
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		RAISE_APPLICATION_ERROR (-20999,'20000 CF_ARCHIVER_CONFIGU' || SQLCODE|| ' '||SQLERRM);
	
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
		RAISE_APPLICATION_ERROR (-20999,'20001 CF_ARCHIVER_CONFIGU' || SQLCODE|| ' '||SQLERRM);
End LG_ARCH_TASK_EXPORTDBU;
/





-- LG_ARCH_TASK_PURGEDBI.sql

CREATE OR REPLACE PROCEDURE LG_ARCH_TASK_PURGEDBI
(
	 p_ReturnCd OUT	INTEGER
	,p_DebugFlg IN	INTEGER
	,p_TASKID IN	INTEGER
	,p_TABLENAME IN	VARCHAR2
	,p_CUTOFFDATE IN	TIMESTAMP
	,p_ROWSPURGED IN	INTEGER
	,p_PROCNAME IN	VARCHAR2
	,p_LOGMSG IN	VARCHAR2
	,p_PURGESTARTDATE IN	TIMESTAMP
	,p_PURGEENDDATE	IN TIMESTAMP
)
AS


/*
	================================================================================
	Proc Name:	LG_ARCH_TASK_PURGEDBI
	Create Date:	2008-07-31 14:07:22
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created
	2008-10-20 Amit vijay	6.2	Defect 107019

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	v_RowCnt	integer Default 0;
	v_BeginTS	TimeStamp	; -- Performance related variables
	v_ProcName	varchar(30)	Default 'LG_ARCH_TASK_PURGEDBI';
	v_ExceptionFlg	integer	Default 0;
	v_ExceptionCode	varchar(30);
	v_PK_Value	INTEGER Default 0;
	v_SQLErrm	VARCHAR(255) ;
	v_chkpt		Char(1) default '0';
	v_msg		Varchar(500);
	v_RowTS		INTEGER Default 0;
	 
	v_ParamsTxt	varchar2(4000) Default
			 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
			 		||', p_TASKID=>'||p_TASKID||CHR(10)
			 		||', p_TABLENAME=>'||p_TABLENAME||CHR(10)
			 		||', p_CUTOFFDATE=>'||p_CUTOFFDATE||CHR(10)
			 		||', p_ROWSPURGED=>'||p_ROWSPURGED||CHR(10)
			 		||', p_PROCNAME=>'||p_PROCNAME||CHR(10)
			 		||', p_PURGESTARTDATE=>'||p_PURGESTARTDATE||CHR(10)
			 		||', p_PURGEENDDATE=>'||p_PURGEENDDATE||CHR(10)
	;

	
BEGIN

	p_ReturnCd := 0;
	v_BeginTS := FN_Get_Date();
	v_chkpt:='1';

	UPDATE	LG_ARCH_TASK_PURGEDB ux SET
		ux.CUTOFFDATE = COALESCE(p_CUTOFFDATE, ux.CUTOFFDATE)
		,ux.ROWSPURGED = ux.ROWSPURGED + COALESCE(p_ROWSPURGED,0)
		,ux.PROCNAME = COALESCE(p_PROCNAME, ux.PROCNAME)
		,ux.LOGMSG = COALESCE(p_LOGMSG, ux.LOGMSG)
		,ux.PURGEENDDATE = p_PURGEENDDATE
		,ux.RowTS = RowTS + 1
	WHERE	ux.TASKID = p_TASKID
	AND	ux.TABLENAME = p_TABLENAME
	;
	
	v_RowCnt := SQL%ROWCOUNT;
	IF v_RowCnt = 0 THEN
		 p_ReturnCd := 0;
		 v_chkpt :='2';
		INSERT INTO LG_ARCH_TASK_PURGEDB(
			DBTASKID
			,TASKID
			,TABLENAME
			,TABLETYPE
			,CUTOFFDATE
			,ROWSPURGED
			,PROCNAME
			,LOGMSG
			,PURGESTARTDATE
			,PURGEENDDATE
		) VALUES (
			LG_ARCH_TASK_PURGEDB_SEQ.NEXTVAL
			,p_TASKID
			,p_TABLENAME
			,CASE
				WHEN p_TABLENAME IN ('LG_MSG_ARCHIVE'
					) THEN 1
				WHEN p_TABLENAME IN ('LG_ACTIVITY_RNDTL'
				,'LG_ACTIVITY_RNHDR'
				,'LG_AS_DTL'
				,'LG_AS_HDR'
				,'LG_STATUS_ACTIVITY'
				,'LG_B2BPROCESS_DTL'
				,'LG_B2BPROCESS_HDR'
				,'LG_VALIDATION'
				,'LG_ACTIVITY_DTL'
				,'LG_ACTIVITY_ENDSTATE'
				,'BP_PROCESS_LOG'
				,'LG_DELIVERY_LOG'
				,'LG_SYNCH_REQ_RESP'
				,'LG_ACTIVITY'
				,'LG_STACKTRACE'
				,'LG_EVENT'
				,'LG_ACK'
				,'LG_CHAIN'
				,'LG_EDI_FA_RECONCILE'
				,'LG_ENVELOPE_INCLUSION'
					) THEN 2
				WHEN p_TABLENAME IN ('BP_RNSTATEHDRAUDITLOG'
				,'BP_RNMSGDIGEST'
				,'BP_SPONSOR_STATE'
				,'BP_RNSTATEDTL'
				,'BP_RNSTATEHDR'
				,'BP_AS_STATE_DTL'
				,'BP_AS_STATE_HDR'
				,'BP_STATE_HDR'
				,'BP_RM_STATE_DTL'
				,'BP_RM_STATE_HDR'
				,'BP_SPONSOR_STATE_EBMS'
				,'BP_DUPCHECK'
					) THEN 3
				WHEN p_TABLENAME IN ('LG_SUMMARY_MI'
				,'LG_SUMMARY_RN'
				,'LG_SUMMARY_RN_MI'
				,'LG_SUMMARY_EVENT'
				,'LG_EVENT_EVENTSUMMARY'
				,'LG_EVENTSUMMARY'
				,'LG_ACTIVITY_SUMMARY'
				,'LG_SUMMARY'
					) THEN 4
				ELSE 0
			END
			,p_CUTOFFDATE
			,p_ROWSPURGED
			,p_PROCNAME
			,p_LOGMSG
			,p_PURGESTARTDATE
			,p_PURGEENDDATE
		);
	END IF;	
		

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END LG_ARCH_TASK_PURGEDBI;
/


-- LG_ARCH_TASK_PURGEDBS_AH1.sql

CREATE	OR REPLACE PROCEDURE LG_ARCH_TASK_PURGEDBS_AH1
	(
		p_ReturnCd	OUT	INTEGER
	,	p_DebugFlg	IN	INTEGER
	,	RC1		out	db_ref.RCT
	,	RC2		out	db_ref.RCT
	,	p_sortkey	IN	VARCHAR2
	,	p_PageSizeNbr	IN	INTEGER
	,	p_PageNbr	IN	INTEGER
	,	p_TASKID	IN	INTEGER
	,	p_TYPEID	IN	INTEGER		-- 1=PurgeDB, 2=ExportDB, 3=CommFileSys
	)
AS

/*
	================================================================================
	Proc Name:	LG_ARCH_TASK_PURGEDBS_AH1
	Create Date:	2008-07-31 14:07:22
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	v_RowCnt	integer Default 0;
	v_BeginTS	timestamp Default FN_Get_Date;  -- Performance related variables
	v_ProcName	varchar(30)	Default 'LG_ARCH_TASK_PURGEDBS_AH1';
	v_ExceptionFlg	integer	Default 0;
	v_ExceptionCode	varchar2(30);
	v_PK_Value	INTEGER Default 0;
	v_SQLErrm	Varchar2(255) ;
	v_chkpt		Char(1) default '0';
	v_RowTS		INTEGER Default 0;

	v_where		varchar2(1000);
	v_count		Integer	DEFAULT 0;
	v_sortKey	varchar2(1000);
	v_sqlStmt	varchar2(4000);

	v_ParamsTxt     varchar2(4000) Default
	 		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		 		||', p_TASKID=>'||p_TASKID||CHR(10)
	;

BEGIN

	p_ReturnCd := 0;
	v_Chkpt :='1';
	v_where := ' Where 1 = 1'||
		FN_Build_Where('TASKID', 'NUMBER', p_TASKID) 
	;
	---- default is the set of tables being purged
		v_SQLStmt :=  'SELECT '
				|| 'DBTASKID '
				|| ',TASKID '
				|| ',RTRIM(TABLENAME) TABLENAME '
				|| ',CUTOFFDATE '
				|| ',TABLETYPE '
				|| ',ROWSPURGED '
				|| ',RTRIM(PROCNAME) PROCNAME '
				|| ',RTRIM(LOGMSG) LOGMSG '
				|| ',PURGESTARTDATE '
				|| ',PURGEENDDATE '
			|| 'FROM  LG_ARCH_TASK_PURGEDB '
			|| v_where
		;
	IF p_TYPEID = 2 THEN
		v_SQLStmt :=  'SELECT '
				|| 'EXPORTTASKID '
				|| ',TASKID '
				|| ',RTRIM(SUBVOLNAME) SUBVOLNAME '
				|| ',TASKPARAMID '
				|| ',NUMROWS '
				|| ',RTRIM(ARCHFILENAME) ARCHFILENAME '
				|| ',RESTOREFLAG '
				|| ',CREATEDATE EXPSTARTDATE '
				|| ',UPDATEDATE EXPENDDATE '
			|| 'FROM  LG_ARCH_TASK_EXPORTDB '
			|| v_where
		;
	END IF;
	IF p_TYPEID = 3 THEN
		v_SQLStmt :=  'SELECT '
				|| 'CFSTASKID '
				|| ',TASKID '
				|| ',RTRIM(SUBVOLNAME) SUBVOLNAME '
				|| ',TASKTYPEID '
				|| ',NUMFILES '
			|| 'FROM  LG_ARCH_TASK_COMMFLSYS '
			|| v_where
		;
	END IF;
	-------------------------
	-- Total row count Cursor
	-------------------------
	IF COALESCE(p_pageNbr,0) in (0,1) THEN
		v_Chkpt :='2';
		EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM ('||v_Sqlstmt ||') ' INTO v_Count;
		OPEN RC2 For
			SELECT v_count TotalRowsNbr FROM DUAL;
	END IF;

	IF p_SortKey IS NOT NULL THEN
		 v_Chkpt :='3';
		 v_SortKey := UPPER(p_SortKey);
		 v_sqlstmt := v_sqlstmt ||' order by ' ||v_sortkey; 
	END IF;	

	---------------
	-- Paging logic
	---------------
	IF p_PageSizeNbr >= 0 THEN
		v_Chkpt :='4';
		v_SQLstmt := FN_PAGING_LOGIC(p_pageNbr, p_pageSizeNbr, v_Sqlstmt);
	END IF;
		
	v_Chkpt :='5';
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 for v_SQLstmt;

	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END LG_ARCH_TASK_PURGEDBS_AH1;
/


-- LG_AS_DTLS_VUID.sql

CREATE OR REPLACE PROCEDURE LG_AS_DTLS_VUID (
      p_ReturnCd       out   pls_INTEGER
    , p_DebugFlg       in    pls_INTEGER
    , RC1  	       OUT  DB_REF.RCT
    , p_activityid     in   varchar2
    , p_Vuid           in   varchar2
    )
AS
/*
================================================================================
Proc Name:      LG_AS_DTLS_VUID
Create Date:    10/18/2004
Author          Shishir Narain
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_ProcName        varchar2(30) Default 'LG_AS_DTLS_VUID';
  v_ExceptionFlg    integer Default 0;
  v_Where           Varchar2 (1000);
  v_sqlstmt         varchar2(3600);
Procedure ProcAudit is
v_diffTS Timestamp;
begin
  DB_ProcAuditLogI_ah1('LG_AS_DTLS_VUID',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
   ','||p_activityid||','||p_Vuid);
End ProcAudit;
--
-- Main
--
BEGIN
	IF p_activityid is not null then
	v_where := 'WHERE activityID = '''|| p_activityid||'''	   AND   lg_as_dtl.initasmessageid != lg_as_dtl.asmessageid';
	ELSE
	v_where := 'WHERE vuid = p_Vuid';
    END IF;
			IF RC1%ISOPEN THEN CLOSE RC1; END IF;
		--	dbms_output.put_line(v_sqlstmt);
      v_sqlstmt := 'SELECT ACTIVITYID as ACKACTIVITYID'
	 		    ||'  FROM '
			    ||'  LG_AS_DTL '
		    	||   v_where;
				dbms_output.put_line(v_sqlstmt);
	OPEN RC1 FOR v_sqlstmt;
				
	
	p_ReturnCd := 0;
	ProcAudit;
END ;
/

-- LG_AS_HDRI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_AS_HDRI_AH1
        (p_ReturnCd out pls_integer
        ,p_DebugFlg in pls_integer
        ,p_SQLERRM out varchar2
-- Hdr
        ,p_INITBUSINESSID in VARCHAR2
        ,p_INITASMESSAGEID in VARCHAR2  -- null for parent
        ,p_FRBUSINESSID in VARCHAR2
        ,p_TOBUSINESSID in VARCHAR2
        ,p_RCVDOCTS     in DATE
-- Dtl
        ,p_PACKAGENAME  in VARCHAR2
        ,p_PACKAGEVER   in VARCHAR2
        ,p_ACTIVITYID in VARCHAR2
        ,p_VUID in VARCHAR2
        ,p_ASMESSAGEID in VARCHAR2
        ,p_DISPTEXT in VARCHAR2
        ,p_MDNURI in VARCHAR2
        ,p_MDNRequiredFlg in pls_integer  -- Hdr
        ,p_MDNSTATEKEY in pls_integer

    )
AS

/*
    ================================================================================
    Proc Name:       LG_AS_HDRI_AH1
    Create Date:     2003-04-09 14:53:25
    Author:          V.Goel

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ---------- ---------------------------------------------
    DATE       NAME          Ver	DESCRIPTION
    ---------- ------------- ---------- ---------------------------------------------
    2003-04-16 V.Goel        3.5	completed
    2003-04-18 V.Goel        3.5	Add MDNRequiredFlg
    2003-04-29 V.Goel        3.5	Move PackageName/Ver from Hdr to DTL
    2004-03-11 S.Priani	     4.2.2	Replaced call to Summary procs by Insert
    2006-09-05 S.Priani      6.1	CR 102607
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar(30) := 'LG_AS_HDRI_AH1';
v_ExceptionFlg    integer := 0;
-- Summary
	v_DOCCNT          Pls_Integer default 0;
	v_RCVPROCESSCNT   Pls_Integer default 0;
	v_MDNCNT          Pls_Integer default 0;
	v_DISPWARNINGCNT  Pls_Integer default 0;
	v_DISPERRORCNT    Pls_Integer default 0;
	v_DISPFAILURECNT  Pls_Integer default 0;
	v_SUCCESSCNT      Pls_Integer default 0;
--
v_ParamsTxt       varchar(4000) :=
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
--Hdr
                ||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
                ||', p_INITASMESSAGEID=>'||p_INITASMESSAGEID||CHR(10)
                ||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
                ||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
--Dtl
                ||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
                ||', p_VUID=>'||p_VUID||CHR(10)
                ||', p_FRBUSINESSID=>'||p_FRBUSINESSID||CHR(10)
                ||', p_ASMESSAGEID=>'||p_ASMESSAGEID||CHR(10)
                ||', p_DISPTEXT=>'||p_DISPTEXT||CHR(10)
                ||', p_MDNURI=>'||p_MDNURI||CHR(10)
                ||', p_MDNRequiredFlg=>'||p_MDNRequiredFlg
                ||', p_MDNStateKey=>'||p_MDNStateKey
                ;

Procedure LG_AS_DTLI;
--
Procedure Do_Summary IS
v_returnCD pls_integer;

Begin
	 v_DOCCNT := 1;
	IF p_INITBUSINESSID = p_TOBUSINESSID Then
	   v_MDNCNT := 1;
	END IF;
	 Case
        When Instr(upper(p_DispText), 'FAILURE',1,1) > 0 Then v_DISPFAILURECNT := 1;
        When Instr(upper(p_DispText), 'WARNING',1,1) > 0 Then v_DISPWARNINGCNT := 1;
        When Instr(upper(p_DispText), 'ERROR'  ,1,1) > 0 Then v_DISPERRORCNT   := 1;
        Else v_SUCCESSCNT := 1;
	End Case;

BEGIN
INSERT INTO LG_UNPROCESSED_AS
	(INITBUSINESSID
	,INITASMESSAGEID
	,TOBUSINESSID
	,RCVDOCTS
	,DOCCNT
	,RCVPROCESSCNT
	,MDNCNT
	,DISPWARNGCNT
	,DISPERRORCNT
	,DISPFAILURECNT
	,SUCCESSCNT
	)
VALUES
	(P_INITBUSINESSID
	,P_INITASMESSAGEID
	,P_TOBUSINESSID
	,P_RCVDOCTS
	,v_DOCCNT
	,v_RCVPROCESSCNT   --  done in procedure LG_AS_Hdri
	,v_MDNCNT
	,v_DISPWARNINGCNT
	,v_DISPERRORCNT
	,v_DISPFAILURECNT
	,v_SUCCESSCNT
	);
EXCEPTION
	WHEN Others THEN
		p_ReturnCd := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);
		Raise_application_error (-20999,'20000 LG_AS_HDRI_AH1 - Ins Unprocessed ' || SQLCODE|| ' '||SQLERRM);
END;

end Do_Summary;

--
procedure LG_AS_Hdri IS
Begin
        Insert into lg_as_Hdr(
                INITBUSINESSID
                ,INITASMESSAGEID
                ,TOBUSINESSID
                ,RCVDOCTS
                ,MDNRequiredFlg
                ,MDNSTATEKEY
        ) Values (
                p_INITBUSINESSID
                ,nvl(p_INITASMESSAGEID, p_ASMESSAGEID)
                ,Case
                    When p_INITBUSINESSID = p_FRBUSINESSID Then p_TOBUSINESSID
                    Else p_FRBUSINESSID
                 End
                ,p_RCVDOCTS
                ,p_MDNRequiredFlg
                ,nvl(p_MDNSTATEKEY
                ,CASE
                	When p_MDNRequiredFlg = 1 THEN 2
                	When p_MDNRequiredFlg = 0 THEN 1
                	Else NULL
                End)
        );
	v_RCVPROCESSCNT := 1;	-- Summary
    p_ReturnCd := 0;
    LG_AS_DTLI;   -- also add a detail row
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);
EXCEPTION
        When NO_DATA_FOUND THEN
            p_SQLERRM := SQLERRM;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt,p_DebugFlg, 0);
        When DUP_VAL_ON_INDEX then
            LG_AS_DTLI;
        When OTHERS Then
            Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            p_SQLERRM := SQLERRM;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);

End LG_AS_Hdri;
--
Procedure LG_AS_DTLI
AS
BEGIN
        Insert into lg_as_dtl(
                ACTIVITYID
                ,VUID
                ,INITBUSINESSID
                ,INITASMESSAGEID
                ,FRBUSINESSID
                ,TOBUSINESSID
                ,ASMESSAGEID
                ,DISPTEXT
                ,MDNURI
                ,PACKAGENAME
                ,PACKAGEVER
        ) Values (
                p_ACTIVITYID
                ,p_VUID
                ,p_INITBUSINESSID
                ,p_INITASMESSAGEID
                ,p_FRBUSINESSID
                ,p_TOBUSINESSID
                ,p_ASMESSAGEID
                ,p_DISPTEXT
                ,p_MDNURI
                ,p_PACKAGENAME
                ,p_PACKAGEVER
        );
	IF p_MDNSTATEKEY IS NOT NULL THEN
		UPDATE	lg_as_hdr
		SET	mdnstatekey     = p_mdnstatekey  
           		,updatedate      = v_BeginTS
    		WHERE	initbusinessid = p_initbusinessid
       		AND	initasmessageid = p_initasmessageid;
	END IF;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);

	When DUP_VAL_ON_INDEX then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        p_returnCd := 2;

	When OTHERS Then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        p_SQLERRM := SQLERRM;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
						RAISE_APPLICATION_ERROR (-20999,'20000 LG_AS_DtlI' || SQLCODE|| ' '||SQLERRM);

END LG_AS_DTLI;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    LG_AS_Hdri;
	--Do_Summary;

END LG_AS_HDRI_AH1;
/


-- LG_AS_HDRS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_AS_HDRS_AH1
(
      p_RETURNCD        out pls_INTEGER
    , p_DebugFlg        in  pls_INTEGER
    , RC1               out  db_ref.RCT              -- rows
    , RC2               out  db_ref.rct_count          -- count
    , p_Locale          in  VARCHAR2
    , p_sortkey         in  VARCHAR2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_FRPARTNERID     in  VARCHAR2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_TOPARTNERID     in  VARCHAR2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_ASInitBusinessID  in  VARCHAR2
    , p_ASMessageID     in  VARCHAR2
    , p_DOCID           in  VARCHAR2
    , p_STARTDATE       in  DATE                    --    (Date and Time)
    , p_ENDDATE         in  DATE                    --    (Date and Time)
    , p_SRCTRGFLG       in  pls_Integer     default 0  --  (Source or Target Protocol/Process/Ver Indicator: 0 = Src, 1 = Trg)
    , p_PACKAGENAME     in  varchar2                --(From or To based on SRCTRGFLG)
    , p_PACKAGEVER      in  varchar2                --(From or To based on SRCTRGFLG)
    , p_PROTOCOLNAME    in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROTOCOLVer     in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROCESSCD       in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROCESSVER      in  VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_DestTypeId      in  pls_Integer
    , p_PayloadInitBusinessID in varchar2
    , p_MDNSTATEKEY_LIST in VARCHAR2
    , p_SYNCHRONOUSFLG  in  pls_Integer   -- = 1 is sync 0 = non-sync = null all , p_initpartnertype   in varchar2 
    , P_FROMPARTNERTYPE   in varchar2 
    , P_TOPARTNERTYPE   in varchar2 
    
)
AS
/*
================================================================================
    Proc Name:      LG_AS_HDRS_AH1
    Create Date:    01/04/2002
    Author          Vipin Goel
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	--------------------------------------------
	DATE	   NAME		Ver	DESCRIPTION
	---------- ------------	-------	--------------------------------------------
    	2004-03-05 S.Narain	4.2.2	Removed NLS 
	2005-09-28 S.Narain	6FP2	BGTRK00049107 Inconsistent MDN Disp Text
	2006-09-18 S.Priani	6.1	102766
	2006-09-25 S.Priani	6.1	102833
	2006-10-18 jriordan	6.1	102891f_1
	2006-11-28 jriordan	6.1	103448
	2007-08-25 BHARANI J    6.1.1  Feature 101102
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_beginTS          timestamp(6):= FN_Get_Date;
    v_where            varchar2(4000);
    v_count            pls_integer;
    v_count2           pls_integer;
    v_sortKey          varchar2(1000);
    StartRow           pls_integer;
    EndRow             pls_integer;
    i                  pls_integer;
    sqlstmt            varchar2(4000);
    sqlstmt2           varchar2(4000);
    sql_inner          varchar2(4000); -- includes from clause, inner joins and some outer joins that effect where clause
    v_FrPartnerID      pr_partner.partnerID%type;
    frPartnerName      pr_partner.partnerName%type;
    ToPartnerName      pr_partner.partnerName%type;
    v_ProcName        varchar(30) := 'LG_AS_HDRS_AH1';
    v_ExceptionFlg    integer := 0;
    v_DispText         LG_AS_Dtl.DISPTEXT%Type;
    v_ACTIVITYID       LG_AS_Dtl.ACTIVITYID%Type; 
    
    v_partners_int	varchar(500);
    v_target_int	varchar(500);
    

    v_ParamsTxt       varchar(4000) :=
            'p_DebugFlg=>'||p_DebugFlg||CHR(10)
            ||', RC1=>RC1'||CHR(10)
            ||', RC1=>RC1'||CHR(10)
            ||', p_sortkey=>'||p_sortkey||CHR(10)
            ||', p_PageSizeNbr=>'||p_PageSizeNbr  ||CHR(10)
            ||', p_PageNbr=>'||p_PageNbr  ||CHR(10)
            ||', p_FRPARTNERID=>'||p_FRPARTNERID  ||CHR(10)
            ||', p_TOPARTNERID=>'||p_TOPARTNERID  ||CHR(10)
            ||', p_ASInitBusinessID=>'||p_ASInitBusinessID  ||CHR(10)
            ||', p_STARTDATE=>'||To_Char(p_STARTDATE, 'YY-MM-DD HH24:MI:SS')  ||CHR(10)
            ||', p_ENDDATE=>'||To_Char(p_ENDDATE, 'YY-MM-DD HH24:MI:SS')  ||CHR(10)
            ||', p_SRCTRGFLG=>'||p_SRCTRGFLG  ||CHR(10)
            ||', p_PACKAGENAME=>'||p_PACKAGENAME  ||CHR(10)
            ||', p_PACKAGEVER=>'||p_PACKAGEVER  ||CHR(10)
            ||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME  ||CHR(10)
            ||', p_PROTOCOLVer=>'||p_PROTOCOLVer  ||CHR(10)
            ||', p_PROCESSCD=>'||p_PROCESSCD  ||CHR(10)
            ||', p_PROCESSVER=>'||p_PROCESSVER  ||CHR(10)
            ||', p_DestTypeId=>'||p_DestTypeId  ||CHR(10)
            ||', p_DocId=>'||p_DocId  ||CHR(10)
            ||', p_PayloadInitBusinessID=>'||p_PayloadInitBusinessID  ||CHR(10)            
            ||', p_SYNCHRONOUSFLG=>'|| p_SYNCHRONOUSFLG ||CHR(10)
            ||', p_frompartnertype=>'|| p_frompartnertype ||CHR(10)
            ||', p_topartnertype=>'|| p_topartnertype ||CHR(10);

-- Type
Type char65Tab is table of varchar2(65);
Type num10Tab  is table of number(10);
Type smallintTab is table of SMALLINT;
Type timestampTab is table of TIMESTAMP(6); 
Type char255Tab is table of VARCHAR2(255);
Type char60Tab is table of varchar2(60);
Type char998Tab is table of VARCHAR2(998);
---
array_docId char65Tab := char65Tab();
array_seq       num10Tab  := num10Tab();
array_SYNCHRONOUSFLG smallintTab := smallintTab();
array_RCVDOCTS timestampTab := timestampTab();
array_initBusinessID char60Tab := char60Tab();
array_InitASMessageId char998Tab := char998Tab();
array_DispText char255Tab := char255Tab();
array_MdnState smallintTab := smallintTab();
array_FRPROCESSCD char255Tab := char255Tab();
array_TOPROCESSCD char255Tab := char255Tab();


---
activityIdTab  ASSYNC_TABLE := ASSYNC_TABLE();



Procedure WHERE_CLAUSE as
BEGIN
	
	IF upper(p_frompartnertype)='INTERNAL PARTNERS' then
		        		
		v_Partners_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';
		v_where := v_where || ' AND FRPARTNERIDHDR IN ' || v_Partners_Int ;	
                   		
		IF p_FRPARTNERID is not null then		       
			v_where := v_where || ' OR FRPARTNERIDHDR IN ' || p_FRPARTNERID ;		       
		END IF;
		v_where := v_where || ' ) ';             		
                		
	END IF;
         
	IF upper(p_topartnertype)='INTERNAL PARTNERS' then
		
		v_target_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';     		           
		v_where := v_where || ' AND TOPARTNERIDHDR IN ' || v_target_Int ;
                 
		IF p_TOPARTNERID is not null then
			v_where := v_where || ' OR TOPARTNERIDHDR IN ' || p_TOPARTNERID ;
		END IF;
                		
		v_where := v_where || ' ) ';    		
	END IF;
          
          
	IF p_frompartnertype IS NULL AND p_FRPARTNERID is not null then
		v_where := v_where || ' AND FRPARTNERIDHDR IN ' || p_FRPARTNERID ;
	END IF;
           
	IF p_topartnertype IS NULL AND p_TOPARTNERID is not null then
		v_where := v_where || ' AND TOPARTNERIDHDR IN ' || p_TOPARTNERID ;
	END IF; 
     

	v_where := v_where ||
	FN_Build_Where('a.DESTTYPEID',       'NUMBER'     , p_DESTTYPEID)  ||
    FN_Build_Where('asd.ASMessageID',      'VARCHAR2_LIKE2'   , p_ASMessageID) ||
	FN_Build_Where('asd.InitBusinessID',   'VARCHAR2'   , p_ASInitBusinessID)||
	FN_Build_Where('h.MDNSTATEKEY',   'NUMBER_LIST'   , p_MDNSTATEKEY_LIST)||
	FN_Build_Where('a.INITBUSINESSIDHDR',   'VARCHAR2'   , p_PayloadInitBusinessID)||
	FN_Build_Where('a.DOCID',   'VARCHAR2_LIKE'   , p_DOCID)||
	FN_Build_Where('DTL.SYNCHRONOUSFLG', 'NUMBER'     , p_SYNCHRONOUSFLG);

    IF p_SRCTRGFLG = 0 THEN -- From
        v_where := v_where ||
        FN_Build_Where('a.FRPACKAGENAME',  'VARCHAR2', p_PACKAGENAME) ||
        FN_Build_Where('a.FRPACKAGEVER',   'VARCHAR2', p_PACKAGEVER)  ||
        FN_Build_Where('a.FrPROTOCOLNAME', 'VARCHAR2', p_PROTOCOLNAME)||
        FN_Build_Where('a.FrPROTOCOLVER',  'VARCHAR2', p_PROTOCOLVER) ||
        FN_Build_Where('a.FrPROCESSCD',    'VARCHAR2', p_PROCESSCD)   ||
        FN_Build_Where('a.FrPROCESSVER',   'VARCHAR2', p_PROCESSVER);
    ELSIF p_SRCTRGFLG = 1 THEN -- To
        v_where := v_where ||
        FN_Build_Where('a.ToPACKAGENAME',  'VARCHAR2', p_PACKAGENAME) ||
        FN_Build_Where('a.ToPACKAGEVER',   'VARCHAR2', p_PACKAGEVER)  ||
        FN_Build_Where('a.ToPROTOCOLNAME', 'VARCHAR2', p_PROTOCOLNAME)||
        FN_Build_Where('a.ToPROTOCOLVER',  'VARCHAR2', p_PROTOCOLVER) ||
        FN_Build_Where('a.ToPROCESSCD',    'VARCHAR2', p_PROCESSCD)   ||
        FN_Build_Where('a.ToPROCESSVER',   'VARCHAR2', p_PROCESSVER);
    END IF;
     
END WHERE_CLAUSE;

--                           M A I N
BEGIN
    p_RETURNCD := 0;
    WHERE_CLAUSE;
    
    sqlstmt :=
        ' select ' || 
        ' h.InitBusinessID, ' ||
	' h.mdnstatekey, ' ||
	' a.FrProcessCd, ' ||
	' a.ToProcessCd, ' ||
	' a.DocId,  ' ||
        ' h.InitASMessageId, ' ||
        ' case when sum(dtl.SYNCHRONOUSFLG) = 0 then 0 else 1 end AS SyncFlg,  ' ||
        ' h.RCVDOCTS ' ||
        ' from  ' ||
        ' lg_activity a  ' ||
        ' inner join lg_activity_dtl dtl on a.activityid = dtl.activityid ' ||
        ' inner join lg_as_dtl asd on asd.ActivityId = DTL.ActivityId ' ||
                              ' and asd.vuid       = DTL.Vuid ' ||
        ' INNER JOIN LG_AS_Hdr h ON asd.InitBusinessID = h.InitBusinessID ' ||
                         ' AND asd.InitASMessageId = h.InitASMessageId ' ||
                         ' AND asd.INITASMESSAGEID = asd.ASMESSAGEID  ' ||                       
        ' WHERE h.RcvDocTS BETWEEN :startDate and :enddate '
        || v_where ||
        ' group by ' || 
        ' h.InitBusinessID, ' || 
        ' h.InitASMessageId, ' ||
	' h.mdnstatekey, ' ||
	' a.FrProcessCd, ' ||
	' a.ToProcessCd, ' ||
	' a.DocId,  ' ||        
        ' h.RCVDOCTS ';
        
    Sql_Inner :=  ' From ('|| sqlstmt || ')';       

    if p_sortkey is not null then
       v_sortkey := upper(p_sortkey);
       --v_sortkey := replace(v_sortkey,'FRPARTNERNAME','frp.partnername');
       --v_sortkey := replace(v_sortkey,'TOPARTNERNAME','top.partnername');
       v_sortkey := REPLACE(v_sortkey,'RCVDOCTS','h.RCVDOCTS');
       v_sortkey := REPLACE(v_sortkey,'FRPROCESSCD','a.FRPROCESSCD');
       v_sortkey := REPLACE(v_sortkey,'TOPROCESSCD','a.TOPROCESSCD');
       v_sortkey := REPLACE(v_sortkey,'INITASMESSAGEID','h.InitASMessageId');
       v_sortkey := REPLACE(v_sortkey,'MDNSTATEKEY','h.MDNSTATEKEY');
       v_sortkey := REPLACE(v_sortkey,'DOCID','a.DOCID');              
       sqlstmt := sqlstmt ||' order by '||v_sortkey;
    end if;

    
    IF RC2%IsOpen THEN CLOSE RC2; END IF;
    IF NVL(p_pageNbr,0) IN (0,1) THEN
        EXECUTE IMMEDIATE
            'select count(*)  '|| sql_inner
            INTO v_count USING P_STARTDATE, P_ENDDATE;
        OPEN RC2 FOR SELECT v_count TotalRowsNbr FROM Dual;
    END IF;
    
    sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);

-- Open Cursor
    if RC1%IsOpen then close RC1; end if;
    --DB_PRINT('sqlstmt = ', sqlstmt);   
    execute immediate sqlstmt Bulk Collect INTO array_seq
					, array_InitBusinessID
					, array_MdnState
					, array_FRPROCESSCD
					, array_TOPROCESSCD
					, array_DocId
					, array_InitASMessageId
					, array_SYNCHRONOUSFLG
					, array_RCVDOCTS 
		using P_STARTDATE, P_ENDDATE;

    For i IN 1..array_seq.Count 
    LOOP
        BEGIN
            
            -- reset the v_DispText
            v_DispText := '';
            
            SELECT DispText  
            INTO v_DispText
            FROM   LG_AS_Dtl
            WHERE   INITBUSINESSID  = array_InitBusinessID(i)
            AND     INITASMESSAGEID = array_INITASMESSAGEID(i)
            AND     InitASMessageId <> ASMessageId
            AND     ROWNUM < 2
            ORDER BY CreateDate;
            
            EXCEPTION
            WHEN NO_DATA_FOUND THEN NULL;
        END;
        
        BEGIN
            SELECT dt.ACTIVITYID INTO v_ACTIVITYID
            FROM LG_AS_DTL dt 
            WHERE dt.InitBusinessID = array_InitBusinessID(i) 
            AND dt.InitASMessageId = array_InitASMessageId(i)
			AND     InitASMessageId = ASMessageId
            AND   ROWNUM < 2
            ORDER BY CREATEDATE; 
            
            EXCEPTION
            WHEN NO_DATA_FOUND THEN NULL;
        END;
                        
        activityIdTab.extend;
        activityIdTab(i) := ASSYNC_REC(array_seq(i), v_activityId, array_InitBusinessID(i), array_InitASMessageId(i), array_SYNCHRONOUSFLG(i), v_DispText, array_MdnState(i), array_FRPROCESSCD(i), array_TOPROCESSCD(i),array_DocID(i)  );
       
        --dbms_output.put_line('activityId: ' || v_activityId || ' InitBusinessID: '||array_InitBusinessID(i) || ' InitASMessageId: ' || array_InitASMessageId(i) || ' DispText: ' || v_DispText);
    END LOOP;

--
-- Result set
--
    OPEN RC1 FOR
        SELECT DISTINCT
            Init.partnername    "InitPARTNERNAME",
            a.InitPARTNERIDHdr   "InitPARTNERID",
            H.InitBUSINESSID  "InitBUSINESSID",
            Top.partnername    "TOPARTNERNAME",
            a.TOPARTNERIDHdr   "TOPARTNERID",
            d.ToBUSINESSID  "ToBUSINESSID",
            a.FRPROTOCOLNAME,
            a.FRPROTOCOLVER,
            FN_GET_RTOBJNAME(a.FrPRocessCd, a.FrProcessVer, 'Process')  "FrPROCESSName",
            --frproc.RoutingObjName  "FrPROCESSName",
            a.FrPROCESSCD ,
            a.FRPROCESSVER,
            rtrim(a.FRPACKAGENAME)   FRPACKAGENAME,
            rtrim(a.FRPACKAGEVER)    FRPACKAGEVER,
            rtrim(a.TOPACKAGENAME)   TOPACKAGENAME,
            rtrim(a.TOPACKAGEVER)    TOPACKAGEVER,
            a.ActivityId,
            a.DocId,
            Y.SYNCHRONOUSFLG,
            h.RcvDocTS,
            h.InitASMessageID,
            t.Resourcekey,
            t.Description,
            h.MDNREQUIREDFLG,
            h.MDNSTATEKEY,
            h.TRANSRETRYEXFLG,
            --FN_Get_MDN_DispText (h.INITBUSINESSID, H.INITASMESSAGEID) DISPTEXT,
            Y.DISPTEXT,
            d.MDNURI,
            Y.seq
        FROM
            TABLE(cast(activityIdTab AS ASSYNC_TABLE)) Y
            INNER JOIN lg_activity a  ON Y.id            = a.activityId
            INNER JOIN LG_AS_Dtl d         ON a.ActivityId = d.ActivityId
            INNER JOIN LG_AS_Hdr h         ON d.InitBusinessID = h.InitBusinessID
                                        AND d.InitASMessageId = h.InitASMessageId
                                        AND d.INITASMESSAGEID = d.ASMESSAGEID
            LEFT OUTER JOIN LG_Activity_dtl_parent_vw fradtl    on a.activityId = fradtl.activityId 
            LEFT OUTER JOIN LG_Activity_dtl_child_vw  toadtl    on a.activityId = toadtl.activityId 
            LEFT OUTER JOIN PR_BusIdentifier B1 ON B1.BusinessID = H.InitBUSINESSID
            LEFT OUTER JOIN PR_BusIdentifier B2 ON B2.BusinessID = D.ToBUSINESSID
            LEFT OUTER JOIN pr_partner Init     ON B1.PARTNERID = Init.partnerId
            LEFT OUTER JOIN pr_partner top      ON B2.PARTNERID = top.partnerId
            LEFT OUTER JOIN pr_desttype t ON t.desttypeid = a.desttypeid
    ORDER BY Y.seq;    
END;
/


-- LG_AS_HDRS_AH2.sql

CREATE OR REPLACE PROCEDURE   LG_AS_HDRS_AH2
    (
    p_ReturnCd  out  pls_integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale  in Varchar2
    , p_INITBUSINESSID in VARCHAR2
    , p_INITASMESSAGEID in VARCHAR2
    )
AS

/*
    ================================================================================
    Proc Name:       LG_AS_HDRS_AH2
    Create Date:     2003-04-24 10:12:44
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2003-03-04 S.Narain	4.2.2	Removed NLS
	2005-10-04 S.Priani	6.0.0.2	BGTRK00049412
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'LG_AS_HDRS_AH2';
v_ExceptionFlg    integer := 0;
v_DispText          LG_AS_Dtl.DispText%Type;

--v_MDNURI            LG_AS_Dtl.MDNURI%Type;
v_ParamsTxt       varchar2(4000) :=
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||'p_Locale=>'||p_Locale||CHR(10)
                ||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
                ||', p_INITASMESSAGEID=>'||p_INITASMESSAGEID||CHR(10) ;
Procedure Get_MDN_DispText is
Begin
    SELECT DispText INTO v_DispText
    FROM
    (
    SELECT DispText  From LG_AS_Dtl
    WHERE   INITBUSINESSID  = p_INITBUSINESSID
    AND     INITASMESSAGEID = p_INITASMESSAGEID
    AND     InitASMessageId <> ASMessageId
    Order By CreateDate
    )
    Where     RowNum <2;
    DBMS_OUTPUT.Put_Line('DispText '||v_DispText);
EXCEPTION
    WHEN NO_DATA_FOUND THEN   NULL;
End Get_MDN_DispText;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    Get_MDN_DispText;
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    SELECT  DISTINCT
-- Hdr
       RTRIM(h.INITASMESSAGEID) INITASMESSAGEID
       ,a.RCVDOCTS
       ,MDNREQUIREDFLG
--Dtl
       ,v_DISPTEXT  "DISPTEXT"
       ,d.MDNURI    "MDNURI"
-- Others
       ,Init.partnername    "InitPARTNERNAME"
       ,a.InitPARTNERIDHdr   "InitPARTNERID"
       ,h.InitBUSINESSID  "InitBUSINESSID"
       ,Top.partnername    "TOPARTNERNAME"
       ,a.ToPARTNERIDHdr   "TOPARTNERID"
       ,d.ToBUSINESSID  "ToBUSINESSID"
       ,RTRIM(a.FRPROTOCOLNAME) FRPROTOCOLNAME
       ,RTRIM(a.FRPROTOCOLVER) FRPROTOCOLVER
       ,RTRIM(FN_GET_RTOBJNAME(a.FrPRocessCd, a.FrProcessVer, 'Process')) FrProcessName
       ,a.FrPROCESSCD
       ,RTRIM(a.FRPROCESSVER) FRPROCESSVER
       ,RTRIM(a.FrPackageName) FrPackageName
       ,RTRIM(a.FrPackageVer) FrPackageVer
       ,t.Resourcekey
       ,t.Description
    From
       LG_ACTIVITY a
       Inner Join LG_AS_Dtl d             On a.ActivityId    = d.ActivityId
                                         And InitASMessageId = ASMessageId   -- It is non-MDN
       Inner Join LG_AS_Hdr h             On d.InitBusinessID = h.InitBusinessID
                                      And d.InitASMessageId = h.InitASMessageId
        Left Outer Join PR_BusIdentifier B1    On B1.BusinessID = H.InitBUSINESSID
        Left Outer Join PR_BusIdentifier B2    On B2.BusinessID = D.ToBUSINESSID
        left outer join pr_partner Init       On B1.PARTNERID = Init.partnerId
        left outer join pr_partner top        On B2.PARTNERID = top.partnerId

       left outer join PR_DESTTYPE t  On t.desttypeid     = a.desttypeid
    Where
            d.INITBUSINESSID  = p_INITBUSINESSID
       AND  d.INITASMESSAGEID = p_INITASMESSAGEID  ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End;
/


-- LG_AS_HDRS_AH3.sql

CREATE OR REPLACE PROCEDURE         LG_AS_HDRS_AH3
    (
    p_ReturnCd  out  pls_integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale  in Varchar2
    , p_INITBUSINESSID in VARCHAR2
    , p_INITASMESSAGEID in VARCHAR2
    )
AS

/*
    ================================================================================
    Proc Name:       LG_AS_HDRS_AH3
    Create Date:     2003-04-24 10:12:44
    Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables

v_ProcName        varchar2(30) := 'LG_AS_HDRS_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||'p_Locale=>'||p_Locale||CHR(10)
                ||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
                ||', p_INITASMESSAGEID=>'||p_INITASMESSAGEID||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select /*+ FIRST_ROWS */
        frp.partnername    "FRPARTNERNAME",
        a.FRPARTNERIDHdr   "FRPARTNERID",
        d.FRBUSINESSID  "FRBUSINESSID",
        Top.partnername    "TOPARTNERNAME",
        a.TOPARTNERIDHdr   "TOPARTNERID",
        d.ToBUSINESSID  "ToBUSINESSID",
        a.FRPROTOCOLNAME,
        a.FRPROTOCOLVER,
        a.TOPROTOCOLNAME,
        a.TOPROTOCOLVER,
        FN_GET_RTOBJNAME(a.FrPRocessCd, a.FrProcessVer, 'Process')  "FrPROCESSName",
        a.FrPROCESSCD ,
        a.FRPROCESSVER,
        FN_GET_RTOBJNAME(a.ToPRocessCd, a.ToProcessVer, 'Process') "ToPROCESSName",
        a.ToPROCESSCD,
        a.TOPROCESSVER,
        a.DocId,
        a.DocTS,
        a.ActivityId,
        a.RcvDocTS,
        a.CHANNELSTATUSID,
        RTRIM(a.FrPackageName) FrPackageName,
        RTRIM(a.FrPackageVer) FrPackageVer,
        RTRIM(a.ToPackageName) ToPackageName,
        RTRIM(a.ToPackageVer) ToPackageVer,
        t.Resourcekey ,
        t.Description,
        fradtl.vuid frVuid,
        toadtl.vuid ToVuid,
        fradtl.MSGRAWLENGTH frMSGRAWLENGTH,
        toadtl.MSGRAWLENGTH ToMSGRAWLENGTH,
        fradtl.MSGCONTENTLENGTH frMSGCONTENTLENGTH,
        toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH,
        fradtl.MESSAGESTOREREQUIRED frmesgstorereqd,
        toadtl.MESSAGESTOREREQUIRED tomesgstorereqd,
        ae.EndStateTS,
        nvl(ae.EndStateFlg,0) EndStateFlg,
        wf.Resourcekey  wfResourcekey,
        wf.Description  wfDescription
    FROM
        LG_ACTIVITY a
        INNER JOIN LG_AS_Dtl d   On a.ActivityId = d.ActivityId
        INNER JOIN LG_AS_Hdr h   On d.InitBusinessID = h.InitBusinessID And d.InitASMessageId = h.InitASMessageId
        LEFT OUTER JOIN LG_Activity_endState ae             on a.activityId     =  ae.activityId
        LEFT OUTER JOIN LG_Activity_dtl_parent_vw fradtl    on a.activityId     = fradtl.activityId
        LEFT OUTER JOIN LG_Activity_dtl_child_vw  toadtl    on a.activityId     = toadtl.activityId
        Left Outer Join PR_BusIdentifier B1  On B1.BusinessID = D.FrBUSINESSID
        Left Outer Join PR_BusIdentifier B2  On B2.BusinessID = D.ToBUSINESSID
        left outer join pr_partner Frp       On B1.PARTNERID = Frp.partnerId
        left outer join pr_partner top       On B2.PARTNERID = top.partnerId
        LEFT OUTER JOIN PR_RoutingChannel ch ON a.ChannelID      = ch.ChannelID
        LEFT OUTER JOIN PR_WorkFlow wf on ch.workflowID    = wf.workflowID 
        LEFT OUTER JOIN PR_DestType t on t.desttypeid    = a.desttypeid 
    Where
        h.INITBUSINESSID = p_INITBUSINESSID
        AND h.INITASMESSAGEID = p_INITASMESSAGEID
    Order By a.RcvDocTS;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End;
/


-- LG_AS_HDRU_AH1.sql

CREATE OR REPLACE Procedure LG_AS_HDRU_AH1
        (
         p_ReturnCd out integer
        , p_DebugFlg in pls_integer
        ,p_INITBUSINESSID in VARCHAR2
        ,p_INITASMESSAGEID in VARCHAR2
        ,p_TRANSRETRYEXFLG in NUMBER
        ,p_MDNSTATEKEY     in NUMBER

) as

/*
        ================================================================================
        Proc Name:       LG_AS_HDRU_AH1
        Create Date:     2003-07-23 12:25:05
        Author:          V Goel
        This proc updates the TRANSRETRYEXFLG (max retries exceeded)
        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_AS_HDRU_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
                ||', p_INITASMESSAGEID=>'||p_INITASMESSAGEID||CHR(10)
                ||', p_MDNSTATEKEY=>'||p_MDNSTATEKEY||CHR(10)                
                ||', p_TRANSRETRYEXFLG=>'||p_TRANSRETRYEXFLG||CHR(10)
                ;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        v_RowCnt := 0;
        Update LG_AS_HDR
        SET
                TRANSRETRYEXFLG  = p_TRANSRETRYEXFLG
		,MDNSTATEKEY     = p_MDNSTATEKEY                  
                ,UPDATEDATE      = v_BeginTS
        Where
                INITBUSINESSID = p_INITBUSINESSID
                AND INITASMESSAGEID = p_INITASMESSAGEID
                ;
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        EXCEPTION
                WHEN TOO_MANY_ROWS Then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999, '20100 - LG_AS_HDRU_AH1: Too Many Rows');
                WHEN ZeroUpdate then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_AS_HDRU_AH1: No Data Found');
                WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_AS_HDRU_AH1:'||SQLERRM);
End LG_AS_HDRU_AH1;
/


-- LG_B2BPROCESS_DTLD.sql

--> EXECUTE DB_GENPROC.GENPROC_DELETE('LG_B2BPROCESS_DTL');
CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_DTLD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_MESSAGEID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_DTLD
	Create Date:     2006-06-07 18:03:15
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_DTLD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From LG_B2BPROCESS_DTL
	Where 
		MESSAGEID = p_MESSAGEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_B2BPROCESS_DTLD;
/
--> SPOOL OFF


-- LG_B2BPROCESS_DTLI.sql

--> EXECUTE DB_GENPROC.GENPROC_INSERT('LG_B2BPROCESS_DTL', TRUE, TRUE );
CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_DTLI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROCESSHDRID in NUMBER
	,p_MESSAGEID in VARCHAR2
	,p_ACTIVITYID in VARCHAR2
	,p_VUID in VARCHAR2
	,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_SERVICETYPE in VARCHAR2
	,p_ACTION in VARCHAR2
	,p_MSGDATETIME in TIMESTAMP
	,p_SEQUENCENUMBER in NUMBER
	,p_SYNCHRONOUSFLG in NUMBER
	,p_MSGSTATUSCODE in NUMBER
	,p_CONTENTTYPE in VARCHAR2
	,p_ERRORCODE in VARCHAR2
	,p_SEVERITY in NUMBER
	,p_ERRORLOCATION in VARCHAR2
	,p_ERRORDESCRIPTION in VARCHAR2
	,p_REFMESSAGEID in VARCHAR2
	,p_SIGNEDACK in NUMBER
	,p_TIMETOLIVE in TIMESTAMP
	,p_FRROLE in VARCHAR2
	,p_TOROLE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_DTLI
	Create Date:     2006-06-07 18:03:15
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_DTLI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_SERVICETYPE=>'||p_SERVICETYPE||CHR(10)
		||', p_ACTION=>'||p_ACTION||CHR(10)
		||', p_MSGDATETIME=>'||p_MSGDATETIME||CHR(10)
		||', p_SEQUENCENUMBER=>'||p_SEQUENCENUMBER||CHR(10)
		||', p_SYNCHRONOUSFLG=>'||p_SYNCHRONOUSFLG||CHR(10)
		||', p_MSGSTATUSCODE=>'||p_MSGSTATUSCODE||CHR(10)
		||', p_CONTENTTYPE=>'||p_CONTENTTYPE||CHR(10)
		||', p_ERRORCODE=>'||p_ERRORCODE||CHR(10)
		||', p_SEVERITY=>'||p_SEVERITY||CHR(10)
		||', p_ERRORLOCATION=>'||p_ERRORLOCATION||CHR(10)
		||', p_ERRORDESCRIPTION=>'||p_ERRORDESCRIPTION||CHR(10)
		||', p_REFMESSAGEID=>'||p_REFMESSAGEID||CHR(10)
		||', p_SIGNEDACK=>'||p_SIGNEDACK||CHR(10)
		||', p_TIMETOLIVE=>'||p_TIMETOLIVE||CHR(10)
		||', p_FRROLE=>'||p_FRROLE||CHR(10)
		||', p_TOROLE=>'||p_TOROLE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_B2BPROCESS_DTL(
		PROCESSHDRID
		,MESSAGEID
		,ACTIVITYID
		,VUID
		,FROMPARTNERID
		,TOPARTNERID
		,SERVICETYPE
		,ACTION
		,MSGDATETIME
		,SEQUENCENUMBER
		,SYNCHRONOUSFLG
		,MSGSTATUSCODE
		,CONTENTTYPE
		,ERRORCODE
		,SEVERITY
		,ERRORLOCATION
		,ERRORDESCRIPTION
		,REFMESSAGEID
		,SIGNEDACK
		,TIMETOLIVE
		,FRROLE
		,TOROLE
	) Values (
		p_PROCESSHDRID
		,p_MESSAGEID
		,p_ACTIVITYID
		,p_VUID
		,p_FROMPARTNERID
		,p_TOPARTNERID
		,p_SERVICETYPE
		,p_ACTION
		,p_MSGDATETIME
		,p_SEQUENCENUMBER
		,p_SYNCHRONOUSFLG
		,p_MSGSTATUSCODE
		,p_CONTENTTYPE
		,p_ERRORCODE
		,p_SEVERITY
		,p_ERRORLOCATION
		,p_ERRORDESCRIPTION
		,p_REFMESSAGEID
		,p_SIGNEDACK
		,p_TIMETOLIVE
		,p_FRROLE
		,p_TOROLE
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		p_ProcessHdrId, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END LG_B2BPROCESS_DTLI;
/
--> SPOOL OFF


-- LG_B2BPROCESS_DTLS.sql

--> EXECUTE DB_GENPROC.GENPROC_SELECT('LG_B2BPROCESS_DTL', FALSE );
CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_DTLS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_MESSAGEID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_DTLS
	Create Date:     2006-06-07 18:03:15
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_DTLS';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_MESSAGEID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		PROCESSHDRID
		,RTRIM(MESSAGEID) MESSAGEID
		,RTRIM(ACTIVITYID) ACTIVITYID
		,RTRIM(VUID) VUID
		,FROMPARTNERID
		,TOPARTNERID
		,RTRIM(SERVICETYPE) SERVICETYPE
		,RTRIM(ACTION) ACTION
		,MSGDATETIME
		,SEQUENCENUMBER
		,SYNCHRONOUSFLG
		,MSGSTATUSCODE
		,RTRIM(CONTENTTYPE) CONTENTTYPE
		,RTRIM(ERRORCODE) ERRORCODE
		,SEVERITY
		,RTRIM(ERRORLOCATION) ERRORLOCATION
		,RTRIM(ERRORDESCRIPTION) ERRORDESCRIPTION
		,RTRIM(REFMESSAGEID) REFMESSAGEID
		,SIGNEDACK
		,TIMETOLIVE
		,RTRIM(FRROLE) FRROLE
		,RTRIM(TOROLE) TOROLE
		,ROWTS
		,CREATEDATE
	FROM 
		LG_B2BPROCESS_DTL
	Where 
		MESSAGEID = p_MESSAGEID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		PROCESSHDRID
		,RTRIM(MESSAGEID) MESSAGEID
		,RTRIM(ACTIVITYID) ACTIVITYID
		,RTRIM(VUID) VUID
		,FROMPARTNERID
		,TOPARTNERID
		,RTRIM(SERVICETYPE) SERVICETYPE
		,RTRIM(ACTION) ACTION
		,MSGDATETIME
		,SEQUENCENUMBER
		,SYNCHRONOUSFLG
		,MSGSTATUSCODE
		,RTRIM(CONTENTTYPE) CONTENTTYPE
		,RTRIM(ERRORCODE) ERRORCODE
		,SEVERITY
		,RTRIM(ERRORLOCATION) ERRORLOCATION
		,RTRIM(ERRORDESCRIPTION) ERRORDESCRIPTION
		,RTRIM(REFMESSAGEID) REFMESSAGEID
		,SIGNEDACK
		,TIMETOLIVE
		,RTRIM(FRROLE) FRROLE
		,RTRIM(TOROLE) TOROLE
		,ROWTS
		,CREATEDATE
	FROM 
		LG_B2BPROCESS_DTL
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_B2BPROCESS_DTLS;
/
--> SPOOL OFF


-- LG_B2BPROCESS_DTLS_ACTIVITYID.sql

CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_DTLS_ACTIVITYID
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ACTIVITYID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_DTLS_ACTIVITYID
	Create Date:     2006-03-10 16:33:55
	Author:	  SYSTEM 

			   ::::::::::::::::::::::::::::::
			      M O D I F I C A T I O N
			   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-10 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt	  INTEGER Default 0;
v_BeginTS	 TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName	VARCHAR2(30) Default 'LG_B2BPROCESS_DTLS_ACTIVITYID';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--			 M    A    I    N			      --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
	SELECT	psdtl.ProcessHdrId PROCESSHDRID
		, RTRIM(frp.partnername)     FRPARTNERNAME
		, la.FRPARTNERIDHdr	   FRPARTNERID
		, RTRIM(la.FRBUSINESSIDHDR)   FRBUSINESSID
		, RTRIM(Top.partnername)     TOPARTNERNAME
		, la.TOPARTNERIDHdr	   TOPARTNERID
		, RTRIM(la.ToBUSINESSIDHdr)   ToBUSINESSID 
		, RTRIM(FRPROTOCOLNAME)      FRPROTOCOLNAME 
		, RTRIM(TOPROTOCOLNAME)      TOPROTOCOLNAME
		, la.FrPROCESSCD
		, RTRIM(FN_GET_RTOBJNAME(la.FrProcessCd, la.FrProcessVer, 'Process')) FrProcessName
		, RTRIM(la.FRPROCESSVer)      FRPROCESSVer
		, la.ToPROCESSCD
		, RTRIM(FN_GET_RTOBJNAME(la.ToProcessCd, la.ToProcessVer, 'Process')) ToProcessName
		, RTRIM(la.TOPROCESSVer)      TOPROCESSVer
		, RTRIM(la.DocId) DocId
		, la.DocTS
		, RTRIM(la.ActivityId)	ACTIVITYID
		, la.RcvDocTS
		, la.CHANNELSTATUSID
		, RTRIM(fradtl.vuid)	 frVuid
		, RTRIM(toadtl.vuid)	 ToVuid
		, fradtl.MSGRAWLENGTH  FrMSGRAWLENGTH, fradtl.MSGCONTENTLENGTH FrMSGCONTENTLENGTH
		, Toadtl.MSGRAWLENGTH  ToMSGRAWLENGTH, Toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH
		, fradtl.ORIGINALFILENAME frFILENAME
		, toadtl.ORIGINALFILENAME toFILENAME
		, fradtl.ENCODING frENCODING
		, toadtl.ENCODING toENCODING
		, ae.EndStateTS
		, COALESCE( ae.EndStateFlg, psdtl.MsgStatusCode) EndStateFlg
		, psdtl.ProcessHdrId
		, RTRIM(pshdr.InitBusinessID)	 INITBUSINESSIDHDR
		, RTRIM(pshdr.ConversationID)    CONVERSATIONID
		, psdtl.FROMPARTNERID
		, RTRIM(psdtl.Action) Action
		, RTRIM(psdtl.FRROLE) FRROLE
		, RTRIM(psdtl.TOROLE) TOROLE
		, frp.partnername InitPARTNERNAME
		, la.InitPARTNERIDHdr InitPARTNERID
		, la.InitBUSINESSIDHDR InitBUSINESSID
	FROM	lg_activity la
		INNER JOIN lg_b2bprocess_dtl psdtl	  ON la.activityId    = psdtl.activityId
		INNER JOIN lg_b2bprocess_hdr pshdr	  ON psdtl.processhdrid    = pshdr.processhdrid
		LEFT OUTER JOIN lg_activity_dtl_parent_vw fradtl ON la.activityId    = fradtl.activityId
		LEFT OUTER JOIN lg_activity_dtl_child_vw toadtl  ON la.activityId    = Toadtl.activityId
		LEFT OUTER JOIN lg_activity_endState ae     ON la.activityId    = ae.activityId
		LEFT OUTER JOIN pr_partner frp	      ON la.FRPARTNERIDHdr = frp.partnerId
		LEFT OUTER JOIN pr_partner Top	      ON la.TOPARTNERIDHdr = top.partnerId
	WHERE   la.ActivityId     = p_ActivityId 
UNION
	SELECT	psdtl.ProcessHdrId PROCESSHDRID
		, RTRIM(frp.partnername)     FRPARTNERNAME
		, la.FRPARTNERIDHdr	   FRPARTNERID
		, RTRIM(la.FRBUSINESSIDHDR)   FRBUSINESSID
		, RTRIM(Top.partnername)     TOPARTNERNAME
		, la.TOPARTNERIDHdr	   TOPARTNERID
		, RTRIM(la.ToBUSINESSIDHdr)   ToBUSINESSID 
		, RTRIM(FRPROTOCOLNAME)      FRPROTOCOLNAME 
		, RTRIM(TOPROTOCOLNAME)      TOPROTOCOLNAME
		, la.FrPROCESSCD
		, RTRIM(FN_GET_RTOBJNAME(la.FrProcessCd, la.FrProcessVer, 'Process')) FrProcessName
		, RTRIM(la.FRPROCESSVer)      FRPROCESSVer
		, la.ToPROCESSCD
		, RTRIM(FN_GET_RTOBJNAME(la.ToProcessCd, la.ToProcessVer, 'Process')) ToProcessName
		, RTRIM(la.TOPROCESSVer)      TOPROCESSVer
		, RTRIM(la.DocId) DocId
		, la.DocTS
		, RTRIM(la.ActivityId)	ACTIVITYID
		, la.RcvDocTS
		, la.CHANNELSTATUSID
		, RTRIM(fradtl.vuid)	 frVuid
		, RTRIM(toadtl.vuid)	 ToVuid
		, fradtl.MSGRAWLENGTH  FrMSGRAWLENGTH, fradtl.MSGCONTENTLENGTH FrMSGCONTENTLENGTH
		, Toadtl.MSGRAWLENGTH  ToMSGRAWLENGTH, Toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH
		, fradtl.ORIGINALFILENAME frFILENAME
		, toadtl.ORIGINALFILENAME toFILENAME
		, fradtl.ENCODING frENCODING
		, toadtl.ENCODING toENCODING
		, ae.EndStateTS
		, ae.EndStateFlg
		, psdtl.ProcessHdrId
		, RTRIM(pshdr.InitBusinessID)	 INITBUSINESSIDHDR
		, RTRIM(pshdr.ConversationID)    CONVERSATIONID
		, psdtl.FRPARTNERID
		, RTRIM(psdtl.Action) Action
		, '' FRROLE
		, '' TOROLE
		, frp.partnername InitPARTNERNAME
		, la.InitPARTNERIDHdr InitPARTNERID
		, la.InitBUSINESSIDHDR InitBUSINESSID
	FROM	lg_activity la
		INNER JOIN lg_status_activity psdtl	  ON la.activityId    = psdtl.activityId
		INNER JOIN lg_b2bprocess_hdr pshdr	  ON psdtl.processhdrid    = pshdr.processhdrid
		LEFT OUTER JOIN lg_activity_dtl_parent_vw fradtl ON la.activityId    = fradtl.activityId
		LEFT OUTER JOIN lg_activity_dtl_child_vw toadtl  ON la.activityId    = Toadtl.activityId
		LEFT OUTER JOIN lg_activity_endState ae     ON la.activityId    = ae.activityId
		LEFT OUTER JOIN pr_partner frp	      ON la.FRPARTNERIDHdr = frp.partnerId
		LEFT OUTER JOIN pr_partner Top	      ON la.TOPARTNERIDHdr = top.partnerId
	WHERE   la.ActivityId     = p_ActivityId ;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End LG_B2BPROCESS_DTLS_ACTIVITYID;
/


-- LG_B2BPROCESS_DTLS_HDRID.sql

CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_DTLS_HDRID
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROCESSHDRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_DTLS_HDRID
	Create Date:     2006-03-10 16:33:55
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-10 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_DTLS_HDRID';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		;

---------------------------------------------------------------------------
--			 M    A   I    N			      --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT	psdtl.ProcessHdrId
		, frp.partnername InitPARTNERNAME
		, la.InitPARTNERIDHdr InitPARTNERID
		, la.InitBUSINESSIDHDR InitBUSINESSID
		, la.frpartneridHdr FrPartnerid
		, frp2.PartnerName FrPartnerName
		, Top.partnername TOPARTNERNAME
		, la.TOPARTNERIDHdr TOPARTNERID
		, la.ToBUSINESSIDHdr ToBUSINESSID 
		, FRPROTOCOLNAME, TOPROTOCOLNAME
		, RTRIM(FN_GET_RTOBJNAME(la.FrProcessCd, la.FrProcessVer, 'Process')) FrProcessName
		, la.FrPROCESSCD
		, la.FRPROCESSVer
		, RTRIM(FN_GET_RTOBJNAME(la.ToProcessCd, la.ToProcessVer, 'Process')) ToProcessName
		, la.ToPROCESSCD
		, la.TOPROCESSVer
		, la.DocId
		, la.DocTS
		, la.ActivityId
		, la.RcvDocTS
		, la.CHANNELSTATUSID
		, fradtl.vuid frVuid
		, toadtl.vuid ToVuid
		, fradtl.MSGRAWLENGTH  FrMSGRAWLENGTH
		, fradtl.MSGCONTENTLENGTH FrMSGCONTENTLENGTH
  		, fradtl.MESSAGESTOREREQUIRED frmesgstorereqd
		, Toadtl.MSGRAWLENGTH  ToMSGRAWLENGTH
		, Toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH
  		, toadtl.MESSAGESTOREREQUIRED tomesgstorereqd
		, ae.EndStateTS
		, COALESCE( ae.EndStateFlg, psdtl.MsgStatusCode) EndStateFlg
		, RTRIM(fradtl.ORIGINALFILENAME) frFILENAME
		, RTRIM(toadtl.ORIGINALFILENAME) toFILENAME
		, RTRIM(fradtl.ENCODING) frENCODING
		, RTRIM(toadtl.ENCODING) toENCODING
		, psdtl.FROMPARTNERID
		, RTRIM(psdtl.Action) Action
		,la.FrPackageName
		,la.ToPackageName
	FROM	lg_activity la
		INNER JOIN lg_b2bprocess_dtl psdtl     on la.activityId  = psdtl.activityId
		Left Outer Join lg_activity_dtl fradtl on la.activityId  = fradtl.activityId
						  and fradtl.hdrflg = 1
		Left Outer Join lg_activity_dtl Toadtl  on la.activityId = Toadtl.activityId
						  and Toadtl.hdrflg = 0
		Left Outer Join lg_activity_endState ae on la.activityId = ae.activityId
		Left Outer Join pr_partner frp	  on la.InitPARTNERIDHdr = frp.partnerId
		Left Outer Join pr_partner Top	  on la.TOPARTNERIDHdr = top.partnerId
		Left Outer Join pr_partner frp2	 on la.FrPARTNERIDHdr = frp2.partnerId
	WHERE
		psdtl.PROCESSHDRID = p_PROCESSHDRID
	UNION
	SELECT	psdtl.ProcessHdrId
		, frp.partnername InitPARTNERNAME
		, la.InitPARTNERIDHdr InitPARTNERID
		, la.InitBUSINESSIDHDR InitBUSINESSID
		, la.frpartneridHdr FrPartnerid
		, frp2.PartnerName FrPartnerName
		, Top.partnername TOPARTNERNAME
		, la.TOPARTNERIDHdr TOPARTNERID
		, la.ToBUSINESSIDHdr ToBUSINESSID 
		, FRPROTOCOLNAME, TOPROTOCOLNAME
		, RTRIM(FN_GET_RTOBJNAME(la.FrProcessCd, la.FrProcessVer, 'Process')) FrProcessName
		, la.FrPROCESSCD
		, la.FRPROCESSVer
		, RTRIM(FN_GET_RTOBJNAME(la.ToProcessCd, la.ToProcessVer, 'Process')) ToProcessName
		, la.ToPROCESSCD
		, la.TOPROCESSVer
		, la.DocId
		, la.DocTS
		, la.ActivityId
		, la.RcvDocTS
		, la.CHANNELSTATUSID
		, fradtl.vuid frVuid
		, toadtl.vuid ToVuid
		, fradtl.MSGRAWLENGTH  FrMSGRAWLENGTH
		, fradtl.MSGCONTENTLENGTH FrMSGCONTENTLENGTH
	  	, fradtl.MESSAGESTOREREQUIRED frmesgstorereqd
		, Toadtl.MSGRAWLENGTH  ToMSGRAWLENGTH
		, Toadtl.MSGCONTENTLENGTH ToMSGCONTENTLENGTH
  		, toadtl.MESSAGESTOREREQUIRED tomesgstorereqd
		, ae.EndStateTS
		, ae.EndStateFlg
		, RTRIM(fradtl.ORIGINALFILENAME) frFILENAME
		, RTRIM(toadtl.ORIGINALFILENAME) toFILENAME
		, RTRIM(fradtl.ENCODING) frENCODING
		, RTRIM(toadtl.ENCODING) toENCODING
		, psdtl.FRPARTNERID
		, RTRIM(psdtl.Action) Action
		,la.FrPackageName
		,la.ToPackageName
	FROM	lg_activity la
		INNER JOIN lg_status_activity psdtl     on la.activityId  = psdtl.activityId
		Left Outer Join lg_activity_dtl fradtl on la.activityId  = fradtl.activityId
						  and fradtl.hdrflg = 1
		Left Outer Join lg_activity_dtl Toadtl  on la.activityId = Toadtl.activityId
						  and Toadtl.hdrflg = 0
		Left Outer Join lg_activity_endState ae on la.activityId = ae.activityId
		Left Outer Join pr_partner frp	  on la.InitPARTNERIDHdr = frp.partnerId
		Left Outer Join pr_partner Top	  on la.TOPARTNERIDHdr = top.partnerId
		Left Outer Join pr_partner frp2	 on la.FrPARTNERIDHdr = frp2.partnerId
	WHERE
		psdtl.PROCESSHDRID = p_PROCESSHDRID
	ORDER BY RcvDocTS;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End LG_B2BPROCESS_DTLS_HDRID;
/


-- LG_B2BPROCESS_DTLU.sql

--> EXECUTE DB_GENPROC.GENPROC_UPDATE('LG_B2BPROCESS_DTL');
CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_DTLU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PROCESSHDRID in NUMBER
	,p_MESSAGEID in VARCHAR2
	,p_ACTIVITYID in VARCHAR2
	,p_VUID in VARCHAR2
	,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_SERVICETYPE in VARCHAR2
	,p_ACTION in VARCHAR2
	,p_MSGDATETIME in TIMESTAMP
	,p_SEQUENCENUMBER in NUMBER
	,p_SYNCHRONOUSFLG in NUMBER
	,p_MSGSTATUSCODE in NUMBER
	,p_CONTENTTYPE in VARCHAR2
	,p_ERRORCODE in VARCHAR2
	,p_SEVERITY in NUMBER
	,p_ERRORLOCATION in VARCHAR2
	,p_ERRORDESCRIPTION in VARCHAR2
	,p_REFMESSAGEID in VARCHAR2
	,p_SIGNEDACK in NUMBER
	,p_TIMETOLIVE in TIMESTAMP
	,p_FRROLE in VARCHAR2
	,p_TOROLE in VARCHAR2
	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_DTLU
	Create Date:     2006-06-07 18:03:16
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_DTLU';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_SERVICETYPE=>'||p_SERVICETYPE||CHR(10)
		||', p_ACTION=>'||p_ACTION||CHR(10)
		||', p_MSGDATETIME=>'||p_MSGDATETIME||CHR(10)
		||', p_SEQUENCENUMBER=>'||p_SEQUENCENUMBER||CHR(10)
		||', p_SYNCHRONOUSFLG=>'||p_SYNCHRONOUSFLG||CHR(10)
		||', p_MSGSTATUSCODE=>'||p_MSGSTATUSCODE||CHR(10)
		||', p_CONTENTTYPE=>'||p_CONTENTTYPE||CHR(10)
		||', p_ERRORCODE=>'||p_ERRORCODE||CHR(10)
		||', p_SEVERITY=>'||p_SEVERITY||CHR(10)
		||', p_ERRORLOCATION=>'||p_ERRORLOCATION||CHR(10)
		||', p_ERRORDESCRIPTION=>'||p_ERRORDESCRIPTION||CHR(10)
		||', p_REFMESSAGEID=>'||p_REFMESSAGEID||CHR(10)
		||', p_SIGNEDACK=>'||p_SIGNEDACK||CHR(10)
		||', p_TIMETOLIVE=>'||p_TIMETOLIVE||CHR(10)
		||', p_FRROLE=>'||p_FRROLE||CHR(10)
		||', p_TOROLE=>'||p_TOROLE||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update LG_B2BPROCESS_DTL
	SET
		PROCESSHDRID=p_PROCESSHDRID
		,      ACTIVITYID=p_ACTIVITYID
		,      VUID=p_VUID
		,      FROMPARTNERID=p_FROMPARTNERID
		,      TOPARTNERID=p_TOPARTNERID
		,      SERVICETYPE=p_SERVICETYPE
		,      ACTION=p_ACTION
		,      MSGDATETIME=p_MSGDATETIME
		,      SEQUENCENUMBER=p_SEQUENCENUMBER
		,      SYNCHRONOUSFLG=p_SYNCHRONOUSFLG
		,      MSGSTATUSCODE=p_MSGSTATUSCODE
		,      CONTENTTYPE=p_CONTENTTYPE
		,      ERRORCODE=p_ERRORCODE
		,      SEVERITY=p_SEVERITY
		,      ERRORLOCATION=p_ERRORLOCATION
		,      ERRORDESCRIPTION=p_ERRORDESCRIPTION
		,      REFMESSAGEID=p_REFMESSAGEID
		,      SIGNEDACK=p_SIGNEDACK
		,      TIMETOLIVE=p_TIMETOLIVE
		,      FRROLE=p_FRROLE
		,      TOROLE=p_TOROLE
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		MESSAGEID = p_MESSAGEID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End LG_B2BPROCESS_DTLU;
/
--> SPOOL OFF


-- LG_B2BPROCESS_HDRD.sql

--> EXECUTE DB_GENPROC.GENPROC_DELETE('LG_B2BPROCESS_HDR');
CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_HDRD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_PROCESSHDRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_HDRD
	Create Date:     2006-06-07 18:03:22
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_HDRD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From LG_B2BPROCESS_HDR
	Where 
		PROCESSHDRID = p_PROCESSHDRID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_B2BPROCESS_HDRD;
/
--> SPOOL OFF


-- LG_B2BPROCESS_HDRI.sql

--> EXECUTE DB_GENPROC.GENPROC_INSERT('LG_B2BPROCESS_HDR', TRUE, TRUE );
CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_HDRI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CONVERSATIONID in VARCHAR2
	,p_RCVDOCTS in TIMESTAMP
	,p_INITBUSINESSID in VARCHAR2
	,p_CPAID in VARCHAR2
	,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_DESTTYPEID in NUMBER
	,p_PROTOCOLNAME in VARCHAR2
	,p_PROTOCOLVER in VARCHAR2
	,p_PROCESSCD in VARCHAR2
	,p_PROCESSVER in VARCHAR2
	,p_CONVERSATIONIDEXPIRES in TIMESTAMP
	,p_ENDDATE in TIMESTAMP
	,p_SYNCREPLYMODE in NUMBER
	,p_PRCSTATUSCODE in NUMBER
	,p_PRCSTATUSDESC in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_HDRI
	Create Date:     2006-06-07 18:03:22
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_HDRI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME||CHR(10)
		||', p_PROTOCOLVER=>'||p_PROTOCOLVER||CHR(10)
		||', p_PROCESSCD=>'||p_PROCESSCD||CHR(10)
		||', p_PROCESSVER=>'||p_PROCESSVER||CHR(10)
		||', p_CONVERSATIONIDEXPIRES=>'||p_CONVERSATIONIDEXPIRES||CHR(10)
		||', p_ENDDATE=>'||p_ENDDATE||CHR(10)
		||', p_SYNCREPLYMODE=>'||p_SYNCREPLYMODE||CHR(10)
		||', p_PRCSTATUSCODE=>'||p_PRCSTATUSCODE||CHR(10)
		||', p_PRCSTATUSDESC=>'||p_PRCSTATUSDESC||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_B2BPROCESS_HDR(
		PROCESSHDRID
		,CONVERSATIONID
		,RCVDOCTS
		,INITBUSINESSID
		,CPAID
		,FROMPARTNERID
		,TOPARTNERID
		,DESTTYPEID
		,PROTOCOLNAME
		,PROTOCOLVER
		,PROCESSCD
		,PROCESSVER
		,CONVERSATIONIDEXPIRES
		,ENDDATE
		,SYNCREPLYMODE
		,PRCSTATUSCODE
		,PRCSTATUSDESC
	) Values (
		LG_B2BPROCESS_HDR_SEQ.nextval
		,p_CONVERSATIONID
		,p_RCVDOCTS
		,p_INITBUSINESSID
		,p_CPAID
		,p_FROMPARTNERID
		,p_TOPARTNERID
		,p_DESTTYPEID
		,p_PROTOCOLNAME
		,p_PROTOCOLVER
		,p_PROCESSCD
		,p_PROCESSVER
		,p_CONVERSATIONIDEXPIRES
		,p_ENDDATE
		,p_SYNCREPLYMODE
		,p_PRCSTATUSCODE
		,p_PRCSTATUSDESC
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		LG_B2BPROCESS_HDR_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END LG_B2BPROCESS_HDRI;
/
--> SPOOL OFF


-- LG_B2BPROCESS_HDRI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_HDRI_AH1
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CONVERSATIONID in VARCHAR2
	,p_INITBUSINESSID in VARCHAR2
	,p_CPAID in VARCHAR2
	,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_DESTNAME in VARCHAR2
	,p_PROTOCOLNAME in VARCHAR2
	,p_PROTOCOLVER in VARCHAR2
	,p_PROCESSCD in VARCHAR2
	,p_PROCESSVER in VARCHAR2
	,p_SYNCREPLYMODE in NUMBER
	,p_STATUSCODE in NUMBER
	,p_STATUSDESC in VARCHAR2
--- Detail
	,p_ACTIVITYID in VARCHAR2
	,p_VUID in VARCHAR2
	,p_SERVICETYPE in VARCHAR2
	,p_ACTION in VARCHAR2
	,p_MSGDATETIME in TIMESTAMP
	,p_SEQUENCENUMBER in NUMBER
	,p_SYNCHRONOUSFLG in NUMBER
	,p_CONTENTTYPE in VARCHAR2
	,p_ERRORCODE in VARCHAR2
	,p_SEVERITY in NUMBER
	,p_ERRORLOCATION in VARCHAR2
	,p_ERRORDESCRIPTION in VARCHAR2
	,p_REFMESSAGEID in VARCHAR2
	,p_SIGNEDACK in NUMBER
	,p_TIMETOLIVE in TIMESTAMP
	,p_FRROLE in VARCHAR2
	,p_TOROLE in VARCHAR2
	,p_NEXTSTATUS in VARCHAR2
	,p_MESSAGEID in VARCHAR2
	,p_STATUSID in NUMBER
	,p_REFMSGSTATUSCD in NUMBER
	,p_TIMETOPERFORM in NUMBER
	,p_RESPONSEURL in VARCHAR2
	,p_PARENTVUID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_HDRI_AH1
	Create Date:     2006-03-20 16:22:22
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-20 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_HDRI_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_DESTNAME=>'||p_DESTNAME||CHR(10)
		||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME||CHR(10)
		||', p_PROTOCOLVER=>'||p_PROTOCOLVER||CHR(10)
		||', p_PROCESSCD=>'||p_PROCESSCD||CHR(10)
		||', p_PROCESSVER=>'||p_PROCESSVER||CHR(10)
		||', p_SYNCREPLYMODE=>'||p_SYNCREPLYMODE||CHR(10)
		||', p_STATUSCODE=>'||p_STATUSCODE||CHR(10)
		||', p_STATUSDESC=>'||p_STATUSDESC||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_SERVICETYPE=>'||p_SERVICETYPE||CHR(10)
		||', p_ACTION=>'||p_ACTION||CHR(10)
		||', p_MSGDATETIME=>'||p_MSGDATETIME||CHR(10)
		||', p_SEQUENCENUMBER=>'||p_SEQUENCENUMBER||CHR(10)
		||', p_SYNCHRONOUSFLG=>'||p_SYNCHRONOUSFLG||CHR(10)
		||', p_CONTENTTYPE=>'||p_CONTENTTYPE||CHR(10)
		||', p_ERRORCODE=>'||p_ERRORCODE||CHR(10)
		||', p_SEVERITY=>'||p_SEVERITY||CHR(10)
		||', p_ERRORLOCATION=>'||p_ERRORLOCATION||CHR(10)
		||', p_ERRORDESCRIPTION=>'||p_ERRORDESCRIPTION||CHR(10)
		||', p_REFMESSAGEID=>'||p_REFMESSAGEID||CHR(10)
		||', p_SIGNEDACK=>'||p_SIGNEDACK||CHR(10)
		||', p_TIMETOLIVE=>'||p_TIMETOLIVE||CHR(10)
		||', p_FRROLE=>'||p_FRROLE||CHR(10)
		||', p_TOROLE=>'||p_TOROLE||CHR(10)
		||', p_NEXTSTATUS=>'||p_NEXTSTATUS||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_STATUSID=>'||p_STATUSID||CHR(10)
		||', p_REFMSGSTATUSCD=>'||p_REFMSGSTATUSCD||CHR(10)
		||', p_TIMETOPERFORM=>'||p_TIMETOPERFORM||CHR(10)
		||', p_RESPONSEURL=>'||p_RESPONSEURL||CHR(10)
		||', p_PARENTVUID=>'||p_PARENTVUID||CHR(10)
		;
v_processhdrid			lg_b2bprocess_hdr.processhdrid%TYPE;
v_actiontypeid			lg_status_activity.actiontypeid%TYPE;
v_destTypeID			lg_b2bprocess_hdr.desttypeid%TYPE  DEFAULT FN_GET_DESTTYPEID (p_DestName);
v_actionPing			lg_status_activity.action%TYPE		DEFAULT	'PING';
v_actionPong			lg_status_activity.action%TYPE		DEFAULT	'PONG';
v_actionStsReq			lg_status_activity.action%TYPE		DEFAULT	'STATUSREQUEST';
v_actionStsResp			lg_status_activity.action%TYPE		DEFAULT	'STATUSRESPONSE';
v_PrcStatus			lg_b2bprocess_hdr.PrcStatusCode%TYPE;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_PrcStatus := CASE WHEN p_StatusCode = 5 THEN 0 ELSE p_StatusCode END;
	v_ActionTypeID := CASE 
				WHEN Upper(p_Action) = v_ActionPing THEN 1
				WHEN Upper(p_Action) = v_ActionPong THEN 2
				WHEN Upper(p_Action) = v_ActionStsReq THEN 3
				WHEN Upper(p_Action) = v_ActionStsResp THEN 4
				ELSE -1
			END;

	BEGIN
		IF v_ActionTypeId IN ( -1, 2) THEN  --- update ConversationStatus for Business messages and Pong only

			UPDATE	lg_b2bprocess_hdr hdr
			SET	hdr.Prcstatuscode = COALESCE( v_PrcStatus, hdr.PRCstatuscode)
				,hdr.Prcstatusdesc = COALESCE( p_STATUSDESC, hdr.Prcstatusdesc)
				,hdr.NextStatus = COALESCE( p_NextStatus, hdr.NextStatus)
				,      ROWTS= RowTS + 1
				,      UPDATEDATE= v_BeginTS
			WHERE	hdr.conversationid = p_ConversationId
			AND	hdr.CPAID = p_CPAID
			RETURNING	hdr.processhdrid INTO	v_processhdrid
			;
			IF SQL%NotFound THEN 
				RAISE No_Data_Found;
			END IF;
		ELSE
			SELECT	hdr.ProcessHdrId
			INTO	v_ProcessHdrID
			FROM	lg_b2bprocess_hdr hdr
			WHERE	hdr.conversationid = p_ConversationId
			AND	hdr.CPAID = p_CPAID
			;
		END IF;
	EXCEPTION
		WHEN No_Data_Found THEN
			Insert into LG_B2BPROCESS_HDR(
				PROCESSHDRID
				,CONVERSATIONID
				,INITBUSINESSID
				,CPAID
				,FROMPARTNERID
				,TOPARTNERID
				,RCVDOCTS
				,DESTTYPEID
				,PROTOCOLNAME
				,PROTOCOLVER
				,PROCESSCD
				,PROCESSVER
				,SYNCREPLYMODE
				,PRCSTATUSCODE
				,PRCSTATUSDESC
				,NEXTSTATUS
				,CREATEDATE
				,UPDATEDATE
			) Values (
				LG_B2BPROCESS_HDR_SEQ.nextval
				,p_CONVERSATIONID
				,p_INITBUSINESSID
				,p_CPAID
				,p_FROMPARTNERID
				,p_TOPARTNERID
				,p_MSGDATETIME
				,v_DESTTYPEID
				,p_PROTOCOLNAME
				,p_PROTOCOLVER
				,p_PROCESSCD
				,p_PROCESSVER
				,p_SYNCREPLYMODE
				,v_PrcStatus
				,p_STATUSDESC
				,p_NEXTSTATUS
				,v_BeginTS
				,v_BeginTS
			);
			SELECT LG_B2BPROCESS_HDR_SEQ.currval INTO v_processhdrid FROM DUAL;
	END;
	IF v_ActionTypeId BETWEEN 1 AND 4 THEN
		IF p_STATUSID IS NOT NULL THEN
			Update LG_STATUS_ACTIVITY
			SET
			      FRPARTNERID = COALESCE( p_FROMPARTNERID, FRPARTNERID)
				,	PROCESSHDRID = v_ProcessHDRID
				,	MESSAGEID = p_MESSAGEID
				,      TOPARTNERID = COALESCE( p_TOPARTNERID, TOPARTNERID)
				,      ACTION = COALESCE( p_ACTION, ACTION)
				,      ACTIONTYPEID = COALESCE( v_ACTIONTYPEID, ACTIONTYPEID)
				,      REFMESSAGEID = COALESCE( p_REFMESSAGEID, REFMESSAGEID)
				,      ACTIVITYID = COALESCE( p_ACTIVITYID, ACTIVITYID)
				,      VUID = COALESCE( p_VUID, VUID)
				,      STATUSCODE = COALESCE( p_STATUSCODE, STATUSCODE)
				,      STATUSDESC = COALESCE( p_STATUSDESC, STATUSDESC)
				,      ERRORCODE = COALESCE( p_ERRORCODE, ERRORCODE)
				,      SEVERITY = COALESCE( p_SEVERITY, SEVERITY)
				,      ERRORLOCATION = COALESCE( p_ERRORLOCATION, ERRORLOCATION)
				,      ERRORDESCRIPTION = COALESCE( p_ERRORDESCRIPTION, ERRORDESCRIPTION)
				,      REFMSGSTATUSCD = COALESCE( p_REFMSGSTATUSCD, REFMSGSTATUSCD)
				,      TIMETOPERFORM = COALESCE( p_TIMETOPERFORM, TIMETOPERFORM)
				,      ROWTS= RowTS + 1
				,      UPDATEDATE= v_BeginTS
				,      MESSAGETS= SYSTIMESTAMP
			WHERE	STATUSID = p_STATUSID;
			v_RowCnt := sql%ROWCOUNT;
			IF v_RowCnt = 0 THEN
				RAISE_APPLICATION_ERROR(-20901, v_ProcName || '- No Row updated for Ping/StatusRequest');
			END IF;
		ELSE
			BEGIN
				Insert into LG_STATUS_ACTIVITY(
					STATUSID
					,PROCESSHDRID
					,MESSAGEID
					,FRPARTNERID
					,TOPARTNERID
					,ACTIONTYPEID
					,TIMETOPERFORM
					,ACTION
					,REFMESSAGEID
					,ACTIVITYID
					,VUID
					,STATUSCODE
					,STATUSDESC
					,ERRORCODE
					,SEVERITY
					,ERRORLOCATION
					,ERRORDESCRIPTION
					,REFMSGSTATUSCD
					,MESSAGETS
				) Values (
					LG_STATUS_ACTIVITY_SEQ.nextval
					,v_PROCESSHDRID
					,p_MESSAGEID
					,p_FROMPARTNERID
					,p_TOPARTNERID
					,v_ACTIONTYPEID
					,p_TIMETOPERFORM
					,p_ACTION
					,p_REFMESSAGEID
					,p_ACTIVITYID
					,p_VUID
					,p_STATUSCODE
					,p_STATUSDESC
					,p_ERRORCODE
					,p_SEVERITY
					,p_ERRORLOCATION
					,p_ERRORDESCRIPTION
					,p_REFMSGSTATUSCD
					,SYSTIMESTAMP
				);
					v_RowCnt := sql%ROWCOUNT;
			EXCEPTION
				WHEN Dup_Val_On_Index THEN
					Update LG_STATUS_ACTIVITY
					SET
						      FRPARTNERID = COALESCE( p_FROMPARTNERID, FRPARTNERID)
						,      TOPARTNERID = COALESCE( p_TOPARTNERID, TOPARTNERID)
						,      ACTION = COALESCE( p_ACTION, ACTION)
						,      ACTIONTYPEID = COALESCE( v_ACTIONTYPEID, ACTIONTYPEID)
						,      REFMESSAGEID = COALESCE( p_REFMESSAGEID, REFMESSAGEID)
						,      ACTIVITYID = COALESCE( p_ACTIVITYID, ACTIVITYID)
						,      VUID = COALESCE( p_VUID, VUID)
						,      STATUSCODE = COALESCE( p_STATUSCODE, STATUSCODE)
						,      STATUSDESC = COALESCE( p_STATUSDESC, STATUSDESC)
						,      ERRORCODE = COALESCE( p_ERRORCODE, ERRORCODE)
						,      SEVERITY = COALESCE( p_SEVERITY, SEVERITY)
						,      ERRORLOCATION = COALESCE( p_ERRORLOCATION, ERRORLOCATION)
						,      ERRORDESCRIPTION = COALESCE( p_ERRORDESCRIPTION, ERRORDESCRIPTION)
						,      REFMSGSTATUSCD = COALESCE( p_REFMSGSTATUSCD, REFMSGSTATUSCD)
						,      TIMETOPERFORM = COALESCE( p_TIMETOPERFORM, TIMETOPERFORM)
						,      ROWTS= RowTS + 1
						,      UPDATEDATE= v_BeginTS
						,      MESSAGETS= SYSTIMESTAMP
					Where	PROCESSHDRID = v_ProcessHDRID
					AND	MESSAGEID=p_MESSAGEID;
					v_RowCnt := sql%ROWCOUNT;
			END;
		END IF;
	ELSE   ---- Not a Ping/Pong/StatusReq/StatusResp --> Business message/signal
		BEGIN
			Insert into LG_B2BPROCESS_DTL(
				PROCESSHDRID
				,MESSAGEID
				,ACTIVITYID
				,VUID
				,FROMPARTNERID
				,TOPARTNERID
				,TIMETOPERFORM
				,SERVICETYPE
				,ACTION
				,MSGDATETIME
				,SEQUENCENUMBER
				,SYNCHRONOUSFLG
				,MSGSTATUSCODE
				,CONTENTTYPE
				,ERRORCODE
				,SEVERITY
				,ERRORLOCATION
				,ERRORDESCRIPTION
				,REFMESSAGEID
				,SIGNEDACK
				,TIMETOLIVE
				,FRROLE
				,TOROLE
				,RESPONSEURL
				,PARENTVUID
			) Values (
				v_processhdrid
				,p_MESSAGEID
				,p_ACTIVITYID
				,p_VUID
				,p_FROMPARTNERID
				,p_TOPARTNERID
				,p_TIMETOPERFORM
				,p_SERVICETYPE
				,p_ACTION
				,p_MSGDATETIME
				,p_SEQUENCENUMBER
				,p_SYNCHRONOUSFLG
				,p_STATUSCODE
				,p_CONTENTTYPE
				,p_ERRORCODE
				,p_SEVERITY
				,p_ERRORLOCATION
				,p_ERRORDESCRIPTION
				,p_REFMESSAGEID
				,p_SIGNEDACK
				,p_TIMETOLIVE
				,p_FRROLE
				,p_TOROLE
				,p_RESPONSEURL
				,p_PARENTVUID
			);
				v_RowCnt := sql%ROWCOUNT;
		EXCEPTION
			WHEN Dup_Val_On_Index THEN
				Update LG_B2BPROCESS_DTL dtl
				SET
					dtl.ACTIVITYID= COALESCE(p_ACTIVITYID, ACTIVITYID)
					, dtl.VUID = COALESCE(p_VUID, VUID)
					, dtl.FROMPARTNERID = COALESCE(p_FROMPARTNERID, FROMPARTNERID)
					, dtl.TOPARTNERID = COALESCE(p_TOPARTNERID, TOPARTNERID)
					, dtl.SERVICETYPE = COALESCE(p_SERVICETYPE, SERVICETYPE)
					, dtl.ACTION = COALESCE(p_ACTION, ACTION)
					, dtl.MSGDATETIME = COALESCE(p_MSGDATETIME, MSGDATETIME)
					, dtl.SEQUENCENUMBER = COALESCE(p_SEQUENCENUMBER, SEQUENCENUMBER)
					, dtl.SYNCHRONOUSFLG = COALESCE(p_SYNCHRONOUSFLG, SYNCHRONOUSFLG)
					, dtl.MSGSTATUSCODE = COALESCE(p_STATUSCODE, MSGSTATUSCODE)
					, dtl.CONTENTTYPE = COALESCE(p_CONTENTTYPE, CONTENTTYPE)
					, dtl.ERRORCODE = COALESCE(p_ERRORCODE, ERRORCODE)
					, dtl.SEVERITY = COALESCE(p_SEVERITY, SEVERITY)
					, dtl.ERRORLOCATION = COALESCE(p_ERRORLOCATION, ERRORLOCATION)
					, dtl.ERRORDESCRIPTION = COALESCE(p_ERRORDESCRIPTION, ERRORDESCRIPTION)
					, dtl.REFMESSAGEID = COALESCE(p_REFMESSAGEID, REFMESSAGEID)
					, dtl.SIGNEDACK = COALESCE(p_SIGNEDACK, SIGNEDACK)
					, dtl.TIMETOLIVE = COALESCE(p_TIMETOLIVE, TIMETOLIVE)
					, dtl.FRROLE = COALESCE(p_FRROLE, FRROLE)
					, dtl.TOROLE = COALESCE(p_TOROLE, TOROLE)
					, dtl.TIMETOPERFORM = COALESCE( p_TIMETOPERFORM, TIMETOPERFORM)
					, dtl.RESPONSEURL = COALESCE( p_RESPONSEURL, RESPONSEURL)
					, dtl.PARENTVUID = COALESCE( p_PARENTVUID, PARENTVUID)
					,      ROWTS= RowTS + 1
					,      UPDATEDATE= v_BeginTS
				Where 
					PROCESSHDRID = v_PROCESSHDRID
				AND	MESSAGEID = p_MESSAGEID
				;
				v_RowCnt := sql%ROWCOUNT;
		END;
	END IF;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
			v_processhdrid ID, v_RowCnt RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN No_Data_Found THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	WHEN Others THEN
		Rollback work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		RAISE;
END LG_B2BPROCESS_HDRI_AH1;
/


-- LG_B2BPROCESS_HDRS.sql

--> EXECUTE DB_GENPROC.GENPROC_SELECT('LG_B2BPROCESS_HDR', FALSE );
CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_HDRS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROCESSHDRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_HDRS
	Create Date:     2006-06-07 18:03:22
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_HDRS';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PROCESSHDRID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		PROCESSHDRID
		,RTRIM(CONVERSATIONID) CONVERSATIONID
		,RCVDOCTS
		,RTRIM(INITBUSINESSID) INITBUSINESSID
		,RTRIM(CPAID) CPAID
		,FROMPARTNERID
		,TOPARTNERID
		,DESTTYPEID
		,RTRIM(PROTOCOLNAME) PROTOCOLNAME
		,RTRIM(PROTOCOLVER) PROTOCOLVER
		,RTRIM(PROCESSCD) PROCESSCD
		,RTRIM(PROCESSVER) PROCESSVER
		,CONVERSATIONIDEXPIRES
		,ENDDATE
		,SYNCREPLYMODE
		,PRCSTATUSCODE
		,RTRIM(PRCSTATUSDESC) PRCSTATUSDESC
		,ROWTS
		,CREATEDATE
	FROM 
		LG_B2BPROCESS_HDR
	Where 
		PROCESSHDRID = p_PROCESSHDRID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		PROCESSHDRID
		,RTRIM(CONVERSATIONID) CONVERSATIONID
		,RCVDOCTS
		,RTRIM(INITBUSINESSID) INITBUSINESSID
		,RTRIM(CPAID) CPAID
		,FROMPARTNERID
		,TOPARTNERID
		,DESTTYPEID
		,RTRIM(PROTOCOLNAME) PROTOCOLNAME
		,RTRIM(PROTOCOLVER) PROTOCOLVER
		,RTRIM(PROCESSCD) PROCESSCD
		,RTRIM(PROCESSVER) PROCESSVER
		,CONVERSATIONIDEXPIRES
		,ENDDATE
		,SYNCREPLYMODE
		,PRCSTATUSCODE
		,RTRIM(PRCSTATUSDESC) PRCSTATUSDESC
		,ROWTS
		,CREATEDATE
	FROM 
		LG_B2BPROCESS_HDR
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_B2BPROCESS_HDRS;
/
--> SPOOL OFF


-- LG_B2BPROCESS_HDRS_ACTION.sql

CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_HDRS_ACTION
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_FRPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_PACKAGENAME in VARCHAR2
	,p_PROTOCOLNAME in VARCHAR2
	,p_PROTOCOLVER in VARCHAR2
	,p_PROCESSCD in VARCHAR2
	,p_PROCESSVER in VARCHAR2
	,p_ACTIVITYNAME in VARCHAR2
	,p_ACTIONNAME in VARCHAR2
	,p_CONVERSATIONID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_HDRS_ACTION
	Create Date:     2006-03-20 16:22:22
	Author:          S.Priani 

Return RS with:
List of all possible actions for given Partnerpair,package,protocol,process,activity
Remove the given action
Remove the actions performed for given Conversation

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-08-20 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_HDRS_ACTION';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FRPARTNERID=>'||p_FRPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_PACKAGENAME=>'||p_PACKAGENAME||CHR(10)
		||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME||CHR(10)
		||', p_PROTOCOLVER=>'||p_PROTOCOLVER||CHR(10)
		||', p_PROCESSCD=>'||p_PROCESSCD||CHR(10)
		||', p_PROCESSVER=>'||p_PROCESSVER||CHR(10)
		||', p_ACTIVITYNAME=>'||p_ACTIVITYNAME||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		;
v_processhdrid			lg_b2bprocess_hdr.processhdrid%TYPE;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR
		SELECT	ro1.RoutingObjCd	Action
		FROM	pr_RoutingChannel rc
		INNER JOIN pr_Capability c1 ON c1.capabilityId = rc.FrCapabilityId
		INNER JOIN cf_RoutingObject06_vw ro1 ON ro1.RoutingObjRefId = c1.RoutingObjRefId
		WHERE	ro1.routingObjTypeKey = 'Action'
		AND	rc.FrPartnerId = p_ToPartnerId
		AND	rc.ToPartnerId = p_FrPartnerId
		AND	ro1.ParentObjCd = p_ActivityName
		AND	ro1.GrParentObjCd = p_ProcessCd
		AND	ro1.GrParentObjVer = p_ProcessVer
		AND	ro1.GrParentObjCd2 = p_ProtocolName
		AND	ro1.GrParentObjVer2 = p_ProtocolVer
--		AND	ro1.GrParentObjCd3 = p_PackageName
		AND	ro1.RoutingObjcd <> p_ActionName
	MINUS
		SELECT	dtl.Action
		FROM	lg_b2bProcess_dtl dtl
		INNER JOIN	lg_b2bprocess_hdr hdr ON hdr.ProcessHdrId = dtl.ProcessHdrId
		WHERE	hdr.ConversationId = p_ConversationId
		;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN No_Data_Found THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	WHEN Others THEN
		Rollback work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		RAISE;
END LG_B2BPROCESS_HDRS_ACTION;
/


-- LG_B2BPROCESS_HDRS_AH2.sql

CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_HDRS_AH2
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CONVERSATIONID IN VARCHAR2
	, p_MESSAGEID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_HDRS_AH2
	Create Date:     2006-03-20 16:22:23
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-20 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_HDRS_AH2';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	IF p_MESSAGEID IS NOT NULL THEN
	OPEN RC1 FOR 
		SELECT
			hdr.PROCESSHDRID
			,RTRIM(hdr.CONVERSATIONID) CONVERSATIONID
			,RTRIM(hdr.INITBUSINESSID) INITBUSINESSID
			,RTRIM(hdr.CPAID) CPAID
			,hdr.FROMPARTNERID
			,hdr.TOPARTNERID
			,hdr.RCVDOCTS
			,hdr.DESTTYPEID
			,RTRIM(hdr.PROTOCOLNAME) PROTOCOLNAME
			,RTRIM(hdr.PROTOCOLVER) PROTOCOLVER
			,RTRIM(hdr.PROCESSCD) PROCESSCD
			,RTRIM(hdr.PROCESSVER) PROCESSVER
			,hdr.CONVERSATIONIDEXPIRES
			,hdr.ENDDATE
			,hdr.SYNCREPLYMODE
			,hdr.PRCSTATUSCODE
			,RTRIM(hdr.PRCSTATUSDESC) PRCSTATUSDESC
			,hdr.ROWTS
			,hdr.CREATEDATE
			,RTRIM(dtl.MESSAGEID) MESSAGEID
			,RTRIM(dtl.ACTIVITYID) ACTIVITYID
			,RTRIM(dtl.VUID) VUID
			,dtl.FROMPARTNERID
			,dtl.TOPARTNERID
			,RTRIM(dtl.SERVICETYPE) SERVICETYPE
			,RTRIM(dtl.ACTION) ACTION
			,dtl.MSGDATETIME
			,dtl.SEQUENCENUMBER
			,dtl.SYNCHRONOUSFLG
			,dtl.MSGSTATUSCODE
			,RTRIM(dtl.CONTENTTYPE) CONTENTTYPE
			,RTRIM(dtl.ERRORCODE) ERRORCODE
			,dtl.SEVERITY
			,RTRIM(dtl.ERRORLOCATION) ERRORLOCATION
			,RTRIM(dtl.ERRORDESCRIPTION) ERRORDESCRIPTION
			,RTRIM(dtl.REFMESSAGEID) REFMESSAGEID
			,dtl.SIGNEDACK
			,dtl.TIMETOLIVE
			,RTRIM(dtl.FRROLE) FRROLE
			,RTRIM(dtl.TOROLE) TOROLE
			,RTRIM(dtl.RESPONSEURL) RESPONSEURL
			,RTRIM(dtl.PARENTVUID) PARENTVUID
			,dtl.ROWTS
			,dtl.CREATEDATE
		FROM	LG_B2BPROCESS_DTL dtl
			INNER JOIN LG_B2BPROCESS_HDR hdr ON hdr.PROCESSHDRID = dtl.PROCESSHDRID
		Where	MESSAGEID = p_MESSAGEID
	;

	ELSIF p_CONVERSATIONID IS NOT NULL THEN
	OPEN RC1 FOR 
		SELECT
			hdr.PROCESSHDRID
			,RTRIM(hdr.CONVERSATIONID) CONVERSATIONID
			,RTRIM(hdr.INITBUSINESSID) INITBUSINESSID
			,RTRIM(hdr.CPAID) CPAID
			,hdr.FROMPARTNERID
			,hdr.TOPARTNERID
			,hdr.RCVDOCTS
			,hdr.DESTTYPEID
			,RTRIM(hdr.PROTOCOLNAME) PROTOCOLNAME
			,RTRIM(hdr.PROTOCOLVER) PROTOCOLVER
			,RTRIM(hdr.PROCESSCD) PROCESSCD
			,RTRIM(hdr.PROCESSVER) PROCESSVER
			,hdr.CONVERSATIONIDEXPIRES
			,hdr.ENDDATE
			,hdr.SYNCREPLYMODE
			,hdr.PRCSTATUSCODE
			,RTRIM(hdr.PRCSTATUSDESC) PRCSTATUSDESC
			,hdr.ROWTS
			,hdr.CREATEDATE
			,RTRIM(dtl.MESSAGEID) MESSAGEID
			,RTRIM(dtl.ACTIVITYID) ACTIVITYID
			,RTRIM(dtl.VUID) VUID
			,dtl.FROMPARTNERID
			,dtl.TOPARTNERID
			,RTRIM(dtl.SERVICETYPE) SERVICETYPE
			,RTRIM(dtl.ACTION) ACTION
			,dtl.MSGDATETIME
			,dtl.SEQUENCENUMBER
			,dtl.SYNCHRONOUSFLG
			,dtl.MSGSTATUSCODE
			,RTRIM(dtl.CONTENTTYPE) CONTENTTYPE
			,RTRIM(dtl.ERRORCODE) ERRORCODE
			,dtl.SEVERITY
			,RTRIM(dtl.ERRORLOCATION) ERRORLOCATION
			,RTRIM(dtl.ERRORDESCRIPTION) ERRORDESCRIPTION
			,RTRIM(dtl.REFMESSAGEID) REFMESSAGEID
			,dtl.SIGNEDACK
			,dtl.TIMETOLIVE
			,RTRIM(dtl.FRROLE) FRROLE
			,RTRIM(dtl.TOROLE) TOROLE
			,RTRIM(dtl.RESPONSEURL) RESPONSEURL
			,RTRIM(dtl.PARENTVUID) PARENTVUID
			,dtl.ROWTS
			,dtl.CREATEDATE
		FROM	LG_B2BPROCESS_DTL dtl
			INNER JOIN LG_B2BPROCESS_HDR hdr ON hdr.PROCESSHDRID = dtl.PROCESSHDRID
		Where	CONVERSATIONID = p_CONVERSATIONID
		ORDER BY dtl.sequencenumber
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End LG_B2BPROCESS_HDRS_AH2;
/


-- LG_B2BPROCESS_HDRS_HDRID.sql

CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_HDRS_HDRID
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROCESSHDRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_HDRS_HDRID
	Create Date:     2006-03-10 16:33:58
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-10 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_HDRS_HDRID';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT
		h.PROCESSHDRID
		,RTRIM(h.CONVERSATIONID) CONVERSATIONID
		,h.FROMPARTNERID
		,h.TOPARTNERID
		,RTRIM(FN_GET_RTOBJNAME(h.ProcessCd, h.ProcessVer, 'Process')) ProcessName
		,RTRIM(h.PROTOCOLNAME) PROTOCOLNAME
		,RTRIM(h.PROTOCOLVER) PROTOCOLVER
		,RTRIM(h.PROCESSCD) PROCESSCD
		,RTRIM(h.PROCESSVER) PROCESSVER
		,h.CONVERSATIONIDEXPIRES
		,h.ENDDATE
		,RTRIM(frp.partnerName)      InitPartnerName
		,RTRIM(top.partnerName)      TradingPartnerName
		,h.RCVDOCTS
		,h.ROWTS
		,h.CREATEDATE
		,h.SYNCREPLYMODE AS SYNCHRONOUSFLG
		,RTRIM(t.Resourcekey)        Resourcekey
		,t.Description		Description
		,h.PrcStatusCode		StatusCode
		,h.NextStatus		StatusDescription
	FROM	LG_B2BPROCESS_HDR h
		LEFT OUTER JOIN pr_Partner frp         on h.FromPartnerID = frp.partnerID
		LEFT OUTER JOIN pr_Partner top         on h.ToPartnerID   = top.partnerID
		LEFT outer join pr_desttype t   on h.desttypeid    = t.desttypeid
	WHERE	PROCESSHDRID = p_PROCESSHDRID
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End LG_B2BPROCESS_HDRS_HDRID;
/


-- LG_B2BPROCESS_HDRS_SRCH.sql

CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_HDRS_SRCH
	(
	p_ReturnCd		OUT		PLS_INTEGER
	, p_DebugFlg		IN		PLS_INTEGER
	, RC1			OUT		DB_REF.RCT
	, RC2			OUT		DB_REF.RCT
	, p_Sortkey		in		VARCHAR2
	, p_PageSizeNbr		in		PLS_INTEGER
	, p_PageNbr		in		PLS_INTEGER
	, p_FROMPARTNERID	in		VARCHAR2
	, p_TOPARTNERID		in		VARCHAR2
	, p_STARTDATE		in		DATE
	, p_ENDDATE		in		DATE
	, p_ConversationID	in		VARCHAR2
	, p_InitBusinessID	in		VARCHAR2
	, p_ProtocolName	in		VARCHAR2
	, p_ProtocolVer		in		VARCHAR2
	, p_ProcessCd		in		VARCHAR2
	, p_ProcessVer		in		VARCHAR2
	, p_DestTypeID		in		PLS_INTEGER
	, p_SynchronousFlg	in		PLS_INTEGER
	, P_FROMPARTNERTYPE     in              VARCHAR2 
        , P_TOPARTNERTYPE       in              VARCHAR2 
	)
AS

/*
	================================================================================
	Proc Name:	LG_B2BPROCESS_HDRS_SRCH
	Create Date:	2006-03-10 16:33:58
	Author:		SYSTEM 

				::::::::::::::::::::::::::::::
				 M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-10 SYSTEM	6.1	Created
        2007-08-25 BHARANI J    6.1.1   Feature 101102
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt	INTEGER Default 0;
v_BeginTS	TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName	VARCHAR2(30) Default 'LG_B2BPROCESS_HDRS_SRCH';
v_ExceptionFlg	INTEGER Default 0;
v_where		VARCHAR2(1000);
v_startdate	VARCHAR2(20) := TO_CHAR(NVL(p_startdate,SYSDATE-1), 'DD-mm-YY HH24:MI:SS');
v_enddate	VARCHAR2(20) := TO_CHAR(NVL(p_enddate,SYSDATE)	, 'DD-mm-YY HH24:MI:SS');
v_count		PLS_INTEGER;
v_count2	PLS_INTEGER;
v_sortKey	VARCHAR2(1000);
v_iter		PLS_INTEGER;
sqlstmt		VARCHAR2(4000);
sql_inner	VARCHAR2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
sql_outer	VARCHAR2(1000);

----- Types
TYPE char20Tab IS TABLE OF VARCHAR2(20);
TYPE char255Tab IS TABLE OF VARCHAR2(255);
TYPE num10Tab  IS TABLE OF NUMBER(10);
array_ibusid    char20Tab  := char20Tab();
array_piid      char255Tab := char255Tab();
array_cid	num10Tab   := num10Tab();
array_rnbr      num10Tab   := num10Tab();

v_partners_int	varchar2(500);
v_target_int	varchar2(500);
--
psHdrTab	actRnHdr_Table := actRnHdr_Table();
----------
--
v_ParamsTxt	VARCHAR2(4000) Default
	p_DebugFlg||','||
	'RC1'||','||
	'RC2'||','||
	P_SortKey      ||','||
	P_PageSizeNbr  ||','||
	P_PageNbr      ||','||
	p_FROMPartnerId||','||
	p_TOPartnerId  ||','||
	To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
	To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
	p_PROTOCOLNAME  ||','||
	p_PROTOCOLVER  ||','||
	p_PROCESSCD    ||','||
	p_PROCESSVER   ||','||
	p_ConversationId ||','||
	p_INITBUSINESSID    ||','||
	p_DestTypeId	 ||','||
	p_SYNCHRONOUSFLG ||','||
	p_frompartnertype ||','||
	p_topartnertype 
		;

PROCEDURE Where_Clause IS
BEGIN

	v_where := ' WHERE 1=1 ';
	   
	IF upper(p_frompartnertype)='INTERNAL PARTNERS' then	   
		          		
		v_Partners_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';							
		v_where := v_where || ' AND FromPartnerId IN ' || v_Partners_Int ;	
	             		
	        IF p_FROMPARTNERID is not null then		       
			v_where := v_where || ' OR FromPartnerId IN ' || p_FROMPARTNERID ;		       
	        END IF;
		v_where := v_where || ' ) ';             		
	          		
	END IF;
	   
	IF upper(p_topartnertype)='INTERNAL PARTNERS' then
			
		v_target_Int := ' (SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW ';
		v_where := v_where || ' AND ToPartnerId IN ' || v_target_Int ;
	           
		IF p_TOPARTNERID is not null then
			v_where := v_where || ' OR ToPartnerId IN ' || p_TOPARTNERID ;
		END IF;		
		v_where := v_where || ' ) ';    		
	END IF;
	    
	    
	IF p_frompartnertype IS NULL AND p_FROMPARTNERID is not null then
		v_where := v_where || ' AND FromPartnerId IN ' || p_FROMPARTNERID ;
	END IF;
	     
	IF p_topartnertype IS NULL AND p_TOPARTNERID is not null then
		v_where := v_where || ' AND TOPARTNERID IN ' || p_TOPARTNERID ;
	END IF; 
             
             
	v_where := v_where ||
		' AND  hdr.RCVDOCTS BETWEEN '||CHR(10)||' TO_DATE('''||v_startDate||''',''DD-mm-YY HH24:MI:SS'') AND'||
						CHR(10)||' TO_DATE('''|| v_endDate ||''',''DD-mm-YY HH24:MI:SS'')'||
	FN_Build_Where('hdr.PROTOCOLNAME',   'VARCHAR2'   , p_PROTOCOLNAME) ||
	FN_Build_Where('hdr.PROTOCOLVER',	'VARCHAR2'   , p_PROTOCOLVER) ||
	FN_Build_Where('hdr.PROCESSCD',	'VARCHAR2'   , p_PROCESSCD) ||
	FN_Build_Where('hdr.PROCESSVER',	'VARCHAR2'   , p_PROCESSVER) ||
	FN_Build_Where('hdr.CONVERSATIONID', 'VARCHAR2_LIKE', p_CONVERSATIONID) ||
	FN_Build_Where('hdr.DESTTYPEID'       , 'NUMBER'       , p_DESTTYPEID)        ||
	FN_Build_Where('hdr.INITBUSINESSID'   , 'VARCHAR2'	, p_INITBUSINESSID)	||
	FN_Build_Where('hdr.SYNCREPLYMODE'   , 'NUMBER'       , p_SYNCHRONOUSFLG)
	;
    
END Where_Clause;

----------------------------------------------------------------------------------
--			M	A	I	N				--
----------------------------------------------------------------------------------

BEGIN
	p_ReturnCd := 0;
	Where_clause;

	sql_inner := 'from LG_B2BPROCESS_HDR hdr '
		;
	sql_outer := 'left outer join PR_partner frp   on hdr.FromPartnerId = frp.PartnerId '
		|| 'left outer join PR_partner top   on hdr.ToPartnerId   = top.PartnerId '
		;
	sqlstmt := 'SELECT hdr.processhdrid, hdr.initbusinessid, hdr.conversationid '||sql_inner||' '||v_where ;
	IF p_sortkey IS NOT NULL THEN
		v_sortkey := UPPER(p_sortkey);
		sqlstmt := sqlstmt ||' ORDER BY '||v_sortkey;
	END if;
-- total row count Cursor
	IF RC2%IsOpen THEN CLOSE RC2; END IF;
	IF NVL(p_pageNbr,0) IN (0,1) THEN
		EXECUTE IMMEDIATE 'SELECT COUNT(*) '||sql_inner ||v_where INTO v_count;
		OPEN RC2 FOR SELECT v_count TotalRowsNbr FROM DUAL;
	END IF;
-- Paging logic
	sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
-- Open Cursor
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	EXECUTE IMMEDIATE sqlstmt Bulk Collect INTO array_rnbr, array_cid, array_ibusid, array_piid;
-- Transfer activityID from array to table
	FOR v_iter IN 1..array_rnbr.count LOOP
		psHdrTab.extend;
		psHdrTab(v_iter) := actRnHdr_rec(array_rnbr(v_iter), array_CId(v_iter), array_ibusid(v_iter), array_piid(v_iter));
	END LOOP;
	OPEN RC1 FOR
	SELECT /*+ FIRST_ROWS INDEX(hdr) */
		hdr.ProcessHdrId,
		hdr.FromPartnerId,
		frp.partnername FRPARTNERNAME,
		hdr.TOPartnerId,
		Top.partnername TOPARTNERNAME,
		hdr.ConversationId,
		hdr.RCVDOCTS,
		hdr.PROTOCOLNAME,
		hdr.PROTOCOLVER,
		hdr.PROCESSCD,
		hdr.PROCESSVER,
		hdr.SYNCREPLYMODE AS SYNCHRONOUSFLG,
		RTRIM(FN_GET_RTOBJNAME(hdr.ProcessCd, hdr.ProcessVer, 'Process')) ProcessName
		,t.Resourcekey
		,t.Description
		, hdr.PRCSTATUSCODE STATUSCODE
		, hdr.NEXTSTATUS STATUSDESCRIPTION
	 FROM
		TABLE(cast(psHdrTab as actRnHdr_Table)) Y
		INNER JOIN LG_B2BProcess_HDr hdr ON hdr.ProcessHdrID = Y.CID
						AND hdr.initbusinessID = y.initbusinessID
						AND hdr.ConversationId = Y.ProcessInstanceId
		LEFT OUTER JOIN PR_partner frp ON hdr.FromPartnerId = frp.PartnerId
		LEFT OUTER JOIN PR_partner top ON hdr.ToPartnerId = top.PartnerId
		LEFT OUTER JOIN PR_desttype t ON hdr.DestTypeId = t.DestTypeId
	ORDER BY Y.seq;

	DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End LG_B2BPROCESS_HDRS_SRCH;
/


-- LG_B2BPROCESS_HDRU.sql

--> EXECUTE DB_GENPROC.GENPROC_UPDATE('LG_B2BPROCESS_HDR');
CREATE OR REPLACE PROCEDURE LG_B2BPROCESS_HDRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PROCESSHDRID in NUMBER
	,p_CONVERSATIONID in VARCHAR2
	,p_RCVDOCTS in TIMESTAMP
	,p_INITBUSINESSID in VARCHAR2
	,p_CPAID in VARCHAR2
	,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_DESTTYPEID in NUMBER
	,p_PROTOCOLNAME in VARCHAR2
	,p_PROTOCOLVER in VARCHAR2
	,p_PROCESSCD in VARCHAR2
	,p_PROCESSVER in VARCHAR2
	,p_CONVERSATIONIDEXPIRES in TIMESTAMP
	,p_ENDDATE in TIMESTAMP
	,p_SYNCREPLYMODE in NUMBER
	,p_PRCSTATUSCODE in NUMBER
	,p_PRCSTATUSDESC in VARCHAR2
	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_B2BPROCESS_HDRU
	Create Date:     2006-06-07 18:03:23
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_B2BPROCESS_HDRU';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		||', p_PROTOCOLNAME=>'||p_PROTOCOLNAME||CHR(10)
		||', p_PROTOCOLVER=>'||p_PROTOCOLVER||CHR(10)
		||', p_PROCESSCD=>'||p_PROCESSCD||CHR(10)
		||', p_PROCESSVER=>'||p_PROCESSVER||CHR(10)
		||', p_CONVERSATIONIDEXPIRES=>'||p_CONVERSATIONIDEXPIRES||CHR(10)
		||', p_ENDDATE=>'||p_ENDDATE||CHR(10)
		||', p_SYNCREPLYMODE=>'||p_SYNCREPLYMODE||CHR(10)
		||', p_PRCSTATUSCODE=>'||p_PRCSTATUSCODE||CHR(10)
		||', p_PRCSTATUSDESC=>'||p_PRCSTATUSDESC||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update LG_B2BPROCESS_HDR
	SET
		CONVERSATIONID=p_CONVERSATIONID
		,      RCVDOCTS=p_RCVDOCTS
		,      INITBUSINESSID=p_INITBUSINESSID
		,      CPAID=p_CPAID
		,      FROMPARTNERID=p_FROMPARTNERID
		,      TOPARTNERID=p_TOPARTNERID
		,      DESTTYPEID=p_DESTTYPEID
		,      PROTOCOLNAME=p_PROTOCOLNAME
		,      PROTOCOLVER=p_PROTOCOLVER
		,      PROCESSCD=p_PROCESSCD
		,      PROCESSVER=p_PROCESSVER
		,      CONVERSATIONIDEXPIRES=p_CONVERSATIONIDEXPIRES
		,      ENDDATE=p_ENDDATE
		,      SYNCREPLYMODE=p_SYNCREPLYMODE
		,      PRCSTATUSCODE=p_PRCSTATUSCODE
		,      PRCSTATUSDESC=p_PRCSTATUSDESC
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		PROCESSHDRID = p_PROCESSHDRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End LG_B2BPROCESS_HDRU;
/
--> SPOOL OFF


-- LG_CERT_ARCHIVEI.sql

CREATE OR REPLACE Procedure LG_CERT_ARCHIVEI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_VUID in VARCHAR2
	,p_CERTID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_CERT_ARCHIVEI
	Create Date:     2004-03-10 00:01:40
	Author:          Shishir N
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_CERT_ARCHIVEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_CERTID=>'||p_CERTID||CHR(10)
	;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_CERT_ARCHIVE(
	        VUID
		,ARCHIVECERTID
		,CERTNAME
		,DEFFLG
		,CN
		,L
		,ST
		,O
		,OU
		,C
		,VALIDFR
		,VALIDTO
		,CERTVER
		,FINGERPRINTALGO
		,FINGERPRINT
		,SERIALNBR
		,PUBLICKEY
		,REVOKEDFLG
		,REVOKEDDATE
		,CERT
		,SSLCLIENTCERTFLG
		,DIGSIGCERTFLG
		,ENCRYPTCERTFLG
		,ENABLEDFLG
		,ROOTCERTFLG
		,PRISECFLG
		,PASSWORD
		,PRIVATEKEY)
	(SELECT
			p_VUID
			,CERTID
			,RTRIM(CERTNAME) CERTNAME
			,DEFFLG
			,RTRIM(CN) CN
			,RTRIM(L) L
			,RTRIM(ST) ST
			,RTRIM(O) O
			,RTRIM(OU) OU
			,RTRIM(C) C
			,VALIDFR
			,VALIDTO
			,RTRIM(CERTVER) CERTVER
			,RTRIM(FINGERPRINTALGO) FINGERPRINTALGO
			,RTRIM(FINGERPRINT) FINGERPRINT
			,RTRIM(SERIALNBR) SERIALNBR
			,RTRIM(PUBLICKEY) PUBLICKEY
			,REVOKEDFLG
			,REVOKEDDATE
			,CERT
			,SSLCLIENTCERTFLG
			,DIGSIGCERTFLG
			,ENCRYPTCERTFLG
			,ENABLEDFLG
			,ROOTCERTFLG
			,PRISECFLG 
			,PASSWORD
			,PRIVATEKEY

		FROM
			PR_CERT
		Where
			CERTID = p_CERTID
			AND coalesce(deleteflag,0) <> 1)
	;	
	
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 LG_CERT_ARCHIVEI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 LG_CERT_ARCHIVEI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_CERT_ARCHIVEI' || SQLCODE|| ' '||SQLERRM);
END LG_CERT_ARCHIVEI;
/


-- LG_CERT_ARCHIVES.sql

CREATE OR REPLACE PROCEDURE LG_CERT_ARCHIVES
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CERTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_CERT_ARCHIVES
	Create Date:     2003-09-26 00:06:43
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-06-28 Shishir N	    4.3	PKEE-5VC24T: Added column PRISECFLG
	2007-09-14 Amit V.	  6.1.1	Multiple certificate
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_CERT_ARCHIVES';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CERTID=>'||p_CERTID||CHR(10)
		;
v_certsetid 	  pr_cert_set.certsetid%TYPE;
v_partnerid         pr_partner.partnerid%TYPE;
v_searchflg         INTEGER;
 

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_CERTID IS NOT NULL
 THEN
 
 
 
	OPEN RC1 FOR
	SELECT
		ArchiveCERTID
		,RTRIM(CERTNAME) CERTNAME
		,DEFFLG
		,RTRIM(CN) CN
		,RTRIM(L) L
		,RTRIM(ST) ST
		,RTRIM(O) O
		,RTRIM(OU) OU
		,RTRIM(C) C
		,VALIDFR
		,VALIDTO
		,RTRIM(CERTVER) CERTVER
		,RTRIM(FINGERPRINTALGO) FINGERPRINTALGO
		,RTRIM(FINGERPRINT) FINGERPRINT
		,RTRIM(SERIALNBR) SERIALNBR
		,RTRIM(PUBLICKEY) PUBLICKEY
		,REVOKEDFLG
		,REVOKEDDATE
		,CERT
		,ROWTS
		,CREATEDATE
		,SSLCLIENTCERTFLG
		,DIGSIGCERTFLG
		,ENCRYPTCERTFLG
		,ENABLEDFLG
		,ROOTCERTFLG
		,PRISECFLG
		,PASSWORD
		,PRIVATEKEY
	FROM	lg_cert_archive
	WHERE	ArchiveCERTID = p_CERTID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		ArchiveCERTID
		,RTRIM(CERTNAME) CERTNAME
		,DEFFLG
		,RTRIM(CN) CN
		,RTRIM(L) L
		,RTRIM(ST) ST
		,RTRIM(O) O
		,RTRIM(OU) OU
		,RTRIM(C) C
		,VALIDFR
		,VALIDTO
		,RTRIM(CERTVER) CERTVER
		,RTRIM(FINGERPRINTALGO) FINGERPRINTALGO
		,RTRIM(FINGERPRINT) FINGERPRINT
		,RTRIM(SERIALNBR) SERIALNBR
		,RTRIM(PUBLICKEY) PUBLICKEY
		,REVOKEDFLG
		,REVOKEDDATE
		,CERT
		,ROWTS
		,CREATEDATE
		,SSLCLIENTCERTFLG
		,DIGSIGCERTFLG
		,ENCRYPTCERTFLG
		,ENABLEDFLG
		,ROOTCERTFLG
		,PRISECFLG
		,PASSWORD
		,PRIVATEKEY
	FROM	lg_cert_archive
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_CERT_ARCHIVES;
/


-- LG_CHAIND.sql

Create or Replace Procedure LG_CHAIND
        (p_ReturnCd out pls_integer
        ,p_DebugFlg in pls_integer
        ,p_PARENTACTIVITYID in VARCHAR2
        ,p_CHILDACTIVITYID in VARCHAR2
        )
AS

/*
        ================================================================================
        Proc Name:       LG_CHAIND
        Create Date:     2004-09-10 12:41:55
        Author:          SYSTEM

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2004-09-10 SYSTEM           4.3 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_CHAIND';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_PARENTACTIVITYID=>'||p_PARENTACTIVITYID||CHR(10)
                ||', p_CHILDACTIVITYID=>'||p_CHILDACTIVITYID||CHR(10)
                ;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        Delete From LG_CHAIN
        Where
                PARENTACTIVITYID = p_PARENTACTIVITYID
                AND CHILDACTIVITYID = p_CHILDACTIVITYID
        ;
        v_RowCnt := SQL%ROWCOUNT;
        if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        p_ReturnCd := 0;
        Exception
                When ZeroDelete Then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR (-20999,'20002 LG_CHAIND: No Data Found');
                When OTHERS Then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_CHAIND;
/


-- LG_CHAINI.SQL

Create or Replace Procedure LG_CHAINI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_PARENTACTIVITYID in VARCHAR2
	,p_CHILDACTIVITYID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_CHAINI
	Create Date:     2004-09-02 12:50:29
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_CHAINI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARENTACTIVITYID=>'||p_PARENTACTIVITYID||CHR(10)
		||', p_CHILDACTIVITYID=>'||p_CHILDACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_CHAIN(
		PARENTACTIVITYID
		,CHILDACTIVITYID
	) Values (
		p_PARENTACTIVITYID
		,p_CHILDACTIVITYID
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_CHAINI' || SQLCODE|| ' '||SQLERRM);
END LG_CHAINI;
/


-- LG_CHAINS_DRILLDOWN.SQL

CREATE OR REPLACE PROCEDURE lg_chainS_DRILLDOWN
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, RC2               out db_ref.rct_count          -- count
        , p_Locale          in  VARCHAR2
        , p_sortkey         in  VARCHAR2
        , p_PageSizeNbr     in  pls_Integer
        , p_PageNbr         in  pls_Integer
	, p_PARENTACTIVITYID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_chainS_DRILLDOWN
	Create Date:     2004-09-02 15:04:48
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_chainS_DRILLDOWN';
v_ExceptionFlg    integer Default 0;
sqlstmt            varchar2(4000);
v_sortKey          varchar2(4000);
v_where            varchar2(4000);
sql_inner          varchar2(100);
v_count            integer;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
	    ||',p_sortkey=>'|| p_sortkey||CHR(10)
        ||',p_PageSizeNbr=>'|| p_PageSizeNbr||CHR(10)
        ||',p_PageNbr=>'|| p_PageNbr||CHR(10)
		||', p_PARENTACTIVITYID=>'||p_PARENTACTIVITYID||CHR(10)
		;
Procedure where_clause as
begin
    v_where := ' Where 1=1'||
    FN_Build_Where('PARENTACTIVITYID', 'VARCHAR2'   , p_PARENTACTIVITYID) ;
end where_clause;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	where_clause;
	sqlstmt := 'SELECT 	RTRIM(PARENTACTIVITYID) PARENTACTIVITYID '
		|| ',RTRIM(CHILDACTIVITYID) CHILDACTIVITYID ,ROWTS ,CREATEDATE '
		|| ' FROM lg_chain'
		|| v_where;

	  IF NVL(INSTR(p_sortkey,'ASC'),0) >  0 THEN
	    v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'ASC')-1);
	    sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ')';
	   ELSIF INSTR(upper(p_sortkey),'DESC') > 0 THEN
	      v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'DESC')-1);
	      sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ') DESC';
	   ELSE
	       sqlstmt := sqlstmt ||' order by PARENTACTIVITYID DESC';
	  END IF;

	   sql_inner := 'FROM LG_CHAIN';
	   
	   IF RC2%ISOPEN THEN CLOSE RC2; END IF;
	    IF NVL(P_PAGENBR,0) IN (0,1) THEN
	      Execute Immediate 'select count(*)  '||sql_inner ||v_where INTO v_count;
	  	       open RC2 for select v_count TotalRowsNbr from dual;
	     end if;
	-- Paging logic
	     sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
	 --  Set_page_parameters(p_pageNbr, p_pageSizeNbr, sqlstmt);

	    IF RC1%isOpen THEN Close RC1; END IF;
	    Open RC1 for sqlstmt;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End lg_chainS_DRILLDOWN;
/


-- LG_CHAINS_DRILLUP.SQL

CREATE OR REPLACE PROCEDURE lg_chainS_DRILLUP
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CHILDACTIVITYID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_chainS_DRILLUP
	Create Date:     2004-09-02 15:04:48
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_chainS_DRILLUP';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CHILDACTIVITYID=>'||p_CHILDACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	IF p_CHILDACTIVITYID IS NOT NULL THEN
	OPEN RC1 FOR
	SELECT
		RTRIM(PARENTACTIVITYID) PARENTACTIVITYID
		,RTRIM(CHILDACTIVITYID) CHILDACTIVITYID
		,ROWTS
		,CREATEDATE
	FROM
		lg_chain
	Where
		CHILDACTIVITYID = p_CHILDACTIVITYID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		RTRIM(PARENTACTIVITYID) PARENTACTIVITYID
		,RTRIM(CHILDACTIVITYID) CHILDACTIVITYID
		,ROWTS
		,CREATEDATE
	FROM
		lg_chain
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End lg_chainS_DRILLUP;
/


-- LG_DAE_DOC_LOCKD_AH1.sql

CREATE OR REPLACE PROCEDURE LG_DAE_DOC_LOCKD_AH1
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ID in VARCHAR2
	,p_TOKENID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_DAE_DOC_LOCKD_AH1
	Create Date:     2006-03-13 14:50:11
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-13 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_DAE_DOC_LOCKD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ID=>'||p_ID||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	p_ReturnCd := 0;

	Delete From LG_DAE_DOC_LOCK
	Where
		ID = p_ID
	AND	TOKENID = p_TOKENID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
         p_ReturnCd := 3 ;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
--			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_DAE_DOC_LOCKD_AH1;
/


-- LG_DAE_DOC_LOCKI_TOKEN.sql

CREATE OR REPLACE PROCEDURE LG_DAE_DOC_LOCKI_TOKEN
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ID in VARCHAR2
	,p_TOKENID in VARCHAR2
	,p_Max_Age in Integer  -- in milliseconds 
	)
AS

/*
	================================================================================
	Proc Name:	   LG_DAE_DOC_LOCKI_TOKEN
	Create Date:	 2006-03-13 14:58:12
	Author:	  SYSTEM

			   ::::::::::::::::::::::::::::::
				  M O D I F I C A T I O N
			   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE	   NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-13 SYSTEM	6.1	Created
	2006-11-13 S.Priani	6.1	JR24762

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value	number := 0;

v_RowCnt	  INTEGER Default 0;
v_BeginTS	 TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName	VARCHAR2(30) Default 'LG_DAE_DOC_LOCKI_TOKEN';
v_ExceptionFlg	INTEGER Default 0;
v_Id		LG_DAE_DOC_LOCK.ID%Type;
v_TokenId	LG_DAE_DOC_LOCK.TokenID%Type;
v_RowTS		LG_DAE_DOC_LOCK.RowTS%Type;
v_ParamsTxt	   VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ID=>'||p_ID||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
		||', p_Max_Age=>'||p_Max_Age||CHR(10)
		;

---------------------------------------------------------------------------
--			 M	A	I	N				  --
---------------------------------------------------------------------------

BEGIN

   -- first try to insert the lock - if there's a problem then we'll check in the exception block to see
   --if the lock has expired
			Insert into LG_DAE_DOC_LOCK(
				ID
				,TOKENID
			) Values (
				p_ID
				,p_TOKENID
			);
			v_ExceptionFlg := 0;      
			p_returnCd := 0;      

	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Dup_Val_On_Index THEN
	         -- The TokenID represents the DeliveryMgr instance that holds the lock
	         -- First, find out if we already have the lock registered
		BEGIN

			UPDATE	lg_dae_doc_lock DM
         		Set	updatedate = v_BeginTS
		         	,TOKENID = p_TokenID
      		   		,RowTS = RowTS + 1
			Where	ID = p_ID 
			AND EXISTS	(SELECT	TL.tokenid
					FROM	LG_DAE_TOKEN_LOCK TL 
	        			WHERE	DM.TOKENID = TL.TOKENID
	       				AND	FN_TimestampDiff(tl.UpdateDate, v_BeginTS) > p_Max_Age
				);
			IF SQL%RowCount > 0 THEN
				p_returnCd := 6;  -- updated the lock, previous token has expired
			ELSE
				p_returnCd := 2;  -- could not update
			END IF;
		END;
		
WHEN Others THEN
	--Rollback work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		RAISE;
END LG_DAE_DOC_LOCKI_TOKEN;
/


-- LG_DAE_TOKEN_LOCKD.sql

CREATE OR REPLACE PROCEDURE LG_DAE_TOKEN_LOCKD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_TOKENID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_DAE_TOKEN_LOCKD
	Create Date:     2006-03-13 14:46:21
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-13 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_DAE_TOKEN_LOCKD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	p_ReturnCd := 0;

	Delete From LG_DAE_TOKEN_LOCK
	Where
		TOKENID = p_TOKENID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	Exception
		WHEN ZeroDelete THEN
         p_ReturnCd := 3 ;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			--RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_DAE_TOKEN_LOCKD;
/


-- LG_DAE_TOKEN_LOCKI.sql

CREATE OR REPLACE PROCEDURE lg_dae_token_lockI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_TOKENID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       lg_dae_token_lockI
	Create Date:     2006-03-13 14:47:51
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-13 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'lg_dae_token_lockI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	p_ReturnCd := 0;

	Insert into lg_dae_token_lock(
		TOKENID
	) Values (
		p_TOKENID
	);
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END lg_dae_token_lockI;
/


-- LG_DAE_TOKEN_LOCKU_AH1.sql

CREATE OR REPLACE PROCEDURE LG_DAE_TOKEN_LOCKU_AH1
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
--	, RC1 out db_ref.RCT
	,p_TOKENID in VARCHAR2
--	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_DAE_TOKEN_LOCKU_AH1
	Create Date:     2006-03-13 14:54:35
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-13 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_DAE_TOKEN_LOCKU_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
--		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update LG_DAE_TOKEN_LOCK
	SET
		UPDATEDATE= v_BeginTS
		,      ROWTS= ROWTS + 1
	Where
		TOKENID = p_TOKENID ;
--		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
--	IF RC1%isOpen THEN Close RC1; END IF;
--	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End LG_DAE_TOKEN_LOCKU_AH1;
/


-- LG_DELIVERY_LOGI.sql

CREATE OR REPLACE Procedure LG_DELIVERY_LOGI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ACTIVITYID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_DELIVERY_LOGI
	Create Date:     2003-10-06 08:48:31
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-10-06 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_DELIVERY_LOGI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_DELIVERY_LOG(
		ACTIVITYID
	) Values (
		p_ACTIVITYID
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 LG_DELIVERY_LOGI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 LG_DELIVERY_LOGI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_DELIVERY_LOGI' || SQLCODE|| ' '||SQLERRM);
END LG_DELIVERY_LOGI;
/


-- LG_DELIVERY_LOGS.sql

CREATE OR REPLACE PROCEDURE LG_DELIVERY_LOGS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ACTIVITYID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_DELIVERY_LOGS
	Create Date:     2003-10-06 08:48:32
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-10-06 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_DELIVERY_LOGS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ACTIVITYID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		RTRIM(ACTIVITYID) ACTIVITYID
		,CREATEDATE
	FROM
		LG_DELIVERY_LOG
	Where
		ACTIVITYID = p_ACTIVITYID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		RTRIM(ACTIVITYID) ACTIVITYID
		,CREATEDATE
	FROM
		LG_DELIVERY_LOG
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_DELIVERY_LOGS;
/


-- LG_DM_DOC_LOCKD.sql

Create or Replace Procedure LG_DM_DOC_LOCKD

        (p_ReturnCd out pls_integer
        ,p_DebugFlg in pls_integer 
        ,p_ID in VARCHAR2
        )
AS

/*
        ================================================================================
        Proc Name:       LG_DM_DOC_LOCKD
        Create Date:     2003-11-18 16:06:25
        Author:          SYSTEM 

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-11-18 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_DM_DOC_LOCKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ID=>'||p_ID||CHR(10)
                ;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        Delete From LG_DM_DOC_LOCK
        Where 
                ID = p_ID
        ;
        v_RowCnt := SQL%ROWCOUNT;
        if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        p_ReturnCd := 0;
        Exception
                When ZeroDelete Then
                        p_ReturnCd := 3;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR (-20999,'20002 LG_DM_DOC_LOCKD: No Data Found');
                When OTHERS Then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_DM_DOC_LOCKD;
/


-- LG_DM_DOC_LOCKD_AH1.sql

CREATE OR REPLACE PROCEDURE LG_DM_DOC_LOCKD_AH1
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ID in VARCHAR2
	,p_TOKENID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_DM_DOC_LOCKD_AH1
	Create Date:     2006-03-13 14:50:11
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-13 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_DM_DOC_LOCKD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ID=>'||p_ID||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	p_ReturnCd := 0;

	Delete From LG_DM_DOC_LOCK
	Where
		ID = p_ID
	AND	TOKENID = p_TOKENID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
         p_ReturnCd := 3 ;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
--			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_DM_DOC_LOCKD_AH1;
/


-- LG_DM_DOC_LOCKI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_DM_DOC_LOCKI_AH1
        (p_ReturnCd out pls_integer
        ,p_DebugFlg in pls_integer
        ,p_ID       in VARCHAR2
        ,p_Max_Age  in pls_integer   -- in seconds
        )
AS

/*
        ================================================================================
        Proc Name:       LG_DM_DOC_LOCKI_AH1
        Create Date:     2003-11-18 15:42:31
        Author:          V.Goel

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-11-18 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;
v_updateDate      timestamp;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_DM_DOC_LOCKI_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ID=>'||p_ID||CHR(10)
                ||', p_Max_Age=>'||p_Max_Age||CHR(10)
                ;

Procedure Get_row 
is
Begin
    Select updatedate into v_updateDate
    From LG_DM_DOC_LOCK
    Where ID = p_ID;
Exception
    When NO_DATA_FOUND Then Null;
End Get_row;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    Get_row;
    If v_updateDate is Null Then
        p_ReturnCd := 0;
        INSERT INTO LG_DM_DOC_LOCK(
                ID
        ) VALUES (
                p_ID
        );

    ELSE
        --48339 removed multiplication of p_Max_Age by 1000 - parm comes in as miliseconds
        --so there's no need to do the conversion here
        If FN_TimeStampDiff(v_updateDate, v_BeginTS) <= p_Max_Age Then
             p_ReturnCd := 2;
             DBMS_Output.Put_Line( 'Ret Code ;' ||p_ReturnCd);
        Else
            Update LG_DM_DOC_LOCK
            Set updatedate = v_BeginTS
            Where ID = p_ID ;
            p_ReturnCd := 0;
        End if;
    End If;
    Commit;
        
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        EXCEPTION
            When NO_DATA_FOUND THEN
                p_ReturnCd := 3;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
                --RAISE_APPLICATION_ERROR (-20999,'20000 LG_DM_DOC_LOCKI_AH1' || SQLCODE|| ' '||SQLERRM);
            When DUP_VAL_ON_INDEX then
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                p_returnCd := 2;
                --RAISE_APPLICATION_ERROR (-20999,'20001 LG_DM_DOC_LOCKI_AH1' || SQLCODE|| ' '||SQLERRM);
            When OTHERS Then
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                RAISE_APPLICATION_ERROR (-20999,'20002 LG_DM_DOC_LOCKI_AH1' || SQLCODE|| ' '||SQLERRM);
END LG_DM_DOC_LOCKI_AH1;
/


-- LG_DM_DOC_LOCKI_TOKEN.sql

CREATE OR REPLACE PROCEDURE LG_DM_DOC_LOCKI_TOKEN
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ID in VARCHAR2
	,p_TOKENID in VARCHAR2
	,p_Max_Age in Integer  -- in milliseconds 
	)
AS

/*
	================================================================================
	Proc Name:	   LG_DM_DOC_LOCKI_TOKEN
	Create Date:	 2006-03-13 14:58:12
	Author:	  SYSTEM

			   ::::::::::::::::::::::::::::::
				  M O D I F I C A T I O N
			   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-11 TTD		6.1	Created
	2006-08-29 S.Priani	6.1	CR 102461
	2006-11-13 S.Priani	6.1	JR24762

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value	number := 0;

v_RowCnt	  INTEGER Default 0;
v_BeginTS	 TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName	VARCHAR2(30) Default 'LG_DM_DOC_LOCKI_TOKEN';
v_ExceptionFlg	INTEGER Default 0;
v_Id		LG_DM_DOC_LOCK.ID%Type;
v_TokenId	LG_DM_DOC_LOCK.TokenID%Type;
v_UpdateDate	LG_DM_DOC_LOCK.UpdateDate%Type;
v_RowTS		LG_DM_DOC_LOCK.RowTS%Type;
v_ParamsTxt	   VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ID=>'||p_ID||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
		||', p_Max_Age=>'||p_Max_Age||CHR(10)
		;

---------------------------------------------------------------------------
--			 M	A	I	N			  --
---------------------------------------------------------------------------

BEGIN

   -- first try to insert the lock - if there's a problem then we'll check in the exception block to see
   --if the lock has expired
	Insert into LG_DM_DOC_LOCK(
		ID
		,TOKENID
	) Values (
		p_ID
		,p_TOKENID
	);
	v_ExceptionFlg := 0;      
	p_returnCd := 0;      

	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Dup_Val_On_Index THEN
        	BEGIN
			-- 102461 update with subQ to prevent serializing issues
			UPDATE	LG_DM_DOC_LOCK DM
			SET	updatedate = v_BeginTS
				,TOKENID = p_TokenID
				,RowTS = RowTS + 1
			WHERE	ID = p_ID 
			AND EXISTS	(SELECT	'x'
					FROM	LG_TOKEN_LOCK TL 
					WHERE	DM.TOKENID = TL.TOKENID
	       				AND	FN_TimestampDiff(tl.UpdateDate, v_BeginTS) > p_Max_Age
					)
			;

			IF SQL%RowCount = 0 THEN
		      	   	p_ReturnCd := 2; -- existing lock has not timed out. deny the lock request
		      	ELSE
		      		p_ReturnCd := 6; -- lock updated
			END IF;
        	END;
	WHEN Others THEN
			--Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END LG_DM_DOC_LOCKI_TOKEN;
/


-- LG_DM_DOC_LOCKU_AH1.sql

CREATE OR REPLACE PROCEDURE LG_DM_DOC_LOCKU_AH1
        (
         p_ReturnCd out integer
        , p_DebugFlg in pls_integer
--        , RC1 out db_ref.RCT
        ,p_ID in VARCHAR2
) as

/*
        ================================================================================
        Proc Name:       LG_DM_DOC_LOCKU_AH1
        Create Date:     2003-11-18 15:58:23
        Author:          V Goel

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-11-18 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_DM_DOC_LOCKU_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ID=>'||p_ID||CHR(10)
                ;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        v_RowCnt := 0;
        Update LG_DM_DOC_LOCK
        SET
                UPDATEDATE= v_BeginTS
                ,      ROWTS= RowTS + 1
        Where
                ID = p_ID;
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
--        IF RC1%isOpen THEN Close RC1; END IF;
--        OPEN RC1 for select v_RowTS RowTS from Dual;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        EXCEPTION
                WHEN ZeroUpdate then
                    p_ReturnCd := 3;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_DM_DOC_LOCKU_AH1: No Data Found');
                WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_DM_DOC_LOCKU_AH1:'||SQLERRM);
End LG_DM_DOC_LOCKU_AH1;
/


-- LG_DOCPROCESSING_SUMM.sql

CREATE OR REPLACE PROCEDURE LG_DOCPROCESSING_SUMM
(
    p_ReturnCd         out integer
    ,p_DebugFlg        in pls_integer
    ,p_RCVDOCTS        in TIMESTAMP
    ,p_FRIPADDRESS     in VARCHAR2
    ,p_RCVDOCCNT       in pls_integer
    ,p_PERSISTDOCCNT   in pls_integer
    ,p_PROCESSSTARTCNT in pls_integer
    ,p_PROCESSENDCNT   in pls_integer
    ,p_PROVSESSIONID   in pls_integer
    ,p_SUCCESSCNT      in pls_integer
    ,p_FAILCNT         in pls_integer
) as

/*
    ================================================================================
    Proc Name:       LG_DocProcessing_SumM
    Create Date:     2003-01-21 09:11:13
    Author:          SYSTEM
    NOTE:  Important - there should be NO Commit in this proc. The commit will
           be controled by the application.
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt                integer     := 0;
v_BeginTS               timestamp   := FN_Get_Date; -- Performance related variables
v_rowTS                 integer     := 0;
v_DupFlg                pls_integer := 0;
v_count                 pls_integer := 0;
v_ProcName              Varchar2(30) := 'LG_DOCPROCESSING_SUMM';
zeroUpdate              Exception;
v_RcvDocTS_10Min        DATE := FN_TenMinuteDate(p_RcvDocTS);

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_DocProcessing_SumM',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||p_RCVDOCTS
    ||','||p_FRIPADDRESS
    ||','||p_RCVDOCCNT
    ||','||p_PERSISTDOCCNT
    ||','||p_PROCESSSTARTCNT
    ||','||p_PROCESSENDCNT
    ||','||p_PROVSESSIONID
    ||','||p_SUCCESSCNT
    ||','||p_FAILCNT
    , p_DebugFlg
    );
End ProcAudit;
--
Procedure Update_docProcessing_sum is
Begin

     Update LG_DocProcessing_Sum DP
      Set
         DP.RcvDocCnt       = DP.RcvDocCnt       + nvl(p_RcvDocCnt,0),
         DP.PersistDocCnt   = DP.PersistDocCnt   + nvl(p_PersistDocCnt,0),
         DP.ProcessStartCnt = DP.ProcessStartCnt + nvl(p_ProcessStartCnt,0),
         DP.ProcessEndCnt   = DP.ProcessEndCnt   + nvl(p_ProcessEndCnt,0),
         DP.SUCCESSCNT      = DP.SUCCESSCNT      + nvl(p_SUCCESSCNT,0),
         DP.FAILCNT         = DP.FAILCNT         + nvl(p_FAILCNT,0),
         DP.UpdateDate = v_BeginTS
      Where DP.RcvDocTS = p_RcvDocTs and DP.FRIPAddress = nvl(p_FRIPAddress, -1) and DP.PROVSESSIONID = nvl(p_PROVSESSIONID, -1);
      v_RowCnt := SQL%rowcount;
      if (v_RowCnt = 0) then  raise ZeroUpdate; end if;
      p_ReturnCd := 0;
      ProcAudit;
    EXCEPTION
            WHEN ZeroUpdate then
                p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' Insert_LG_EVENTSUMMARY '||SQLERRM);
End Update_docProcessing_sum;

------------------------------------------------------------------------------
--                               M   A   I   N
------------------------------------------------------------------------------
BEGIN
 Update_docProcessing_sum;
   Begin
     if (v_RowCnt = 0) then
        INSERT INTO LG_DocProcessing_Sum DP
         (
         DP.RcvDocTS,
         DP.FRIPAddress,
         DP.RcvDocCnt,
         DP.PersistDocCnt,
         DP.ProcessStartCnt,
         DP.ProcessEndCnt,
         DP.PROVSESSIONID,
         DP.SUCCESSCNT,
         DP.FAILCNT
         )
         Values
         (
         p_RcvDocTS,
         coalesce(p_FRIPAddress, '-1'),
         nvl(p_RcvDocCnt,0),
         nvl(p_PersistDocCnt,0),
         nvl(p_ProcessStartCnt,0),
         nvl(p_ProcessEndCnt,0),
         nvl(p_PROVSESSIONID, -1),
         nvl(p_SUCCESSCNT,0),
         nvl(p_FAILCNT,0)
         );
   end if;
--
    EXCEPTION
        WHEN DUP_VAL_ON_INDEX Then
            v_DupFlg := 1;
            v_Count := v_Count + 1;
   end;

    p_ReturnCd := 0;

    IF v_DupFlg = 1 THEN
        v_DupFlg := 0;
        If v_Count < 3 Then
            Update_docProcessing_sum;
        Else
            Raise ZeroUpdate;
        End If;
    END IF;
    ProcAudit;
    EXCEPTION
        WHEN ZeroUpdate then
              ProcAudit;
            raise_application_error(-20999,'20100 - LG_DocProcessing_SumM:--ZeroUpdate'||sqlerrm
            ||' p_RCVDOCTS ('||to_char(p_RCVDOCTS,'YYYY-Mon-DD HH24:MI:SS')||')'
            ||' p_FRIPADDRESS ('||p_FRIPADDRESS||')'
            ||' p_PROVSESSIONID ('||p_PROVSESSIONID||')' );
            p_ReturnCd := 1;
        WHEN OTHERS then
              ProcAudit;
            raise_application_error(-20999,'20100 - LG_DocProcessing_SumM:'||sqlerrm
            ||' p_RCVDOCTS ('||to_char(p_RCVDOCTS,'YYYY-Mon-DD HH24:MI:SS')||')'
            ||' p_FRIPADDRESS ('||p_FRIPADDRESS||')'
            ||' p_PROVSESSIONID ('||p_PROVSESSIONID||')' );
            p_ReturnCd := 1;
End LG_DocProcessing_SumM;
/


-- LG_EDI_FA_RECONCILEI.SQL

Create or Replace Procedure LG_EDI_FA_RECONCILEI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ORIGINALACTIVITYID in VARCHAR2
	,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_INTCTLNUM in VARCHAR2
	,p_GRPCTLNUM in VARCHAR2
	,p_TXCTLNUM in VARCHAR2
   ,p_fadueby IN PLS_INTEGER
   ,p_ignoredueby IN PLS_INTEGER
	)
AS

/*
	================================================================================
	Proc Name:       LG_EDI_FA_RECONCILEI
	Create Date:     2004-09-02 12:58:06
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EDI_FA_RECONCILEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ORIGINALACTIVITYID=>'||p_ORIGINALACTIVITYID||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_INTCTLNUM=>'||p_INTCTLNUM||CHR(10)
		||', p_GRPCTLNUM=>'||p_GRPCTLNUM||CHR(10)
		||', p_TXCTLNUM=>'||p_TXCTLNUM||CHR(10)
      ||', p_fadueby=>'||p_fadueby||CHR(10)
      ||', p_ignoredueby=>'||p_ignoredueby||CHR(10)
		;
v_fadueby         INTEGER := 0;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

   IF p_fadueby IS NULL THEN
      v_fadueby := 0;
   ELSE
      v_fadueby := p_fadueby;
   END IF;

	Insert into lg_edi_fa_reconcile(
		ORIGINALACTIVITYID
		,FROMPARTNERID
		,TOPARTNERID
		,INTCTLNUM
		,GRPCTLNUM
		,TXCTLNUM
      ,fadueby
      ,ignoredueby
      ,mostrecent
	) Values (
		p_ORIGINALACTIVITYID
		,p_FROMPARTNERID
		,p_TOPARTNERID
		,p_INTCTLNUM
		,p_GRPCTLNUM
		,p_TXCTLNUM
      ,v_fadueby
      ,p_ignoredueby
      ,1
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
         --change the DUP_VAL_ON_INDEX handler for 56039 - update existing row's MostRecent col value to 0 and redo the insert
			UPDATE lg_edi_fa_reconcile
         SET mostrecent = 0
         WHERE ORIGINALACTIVITYID = p_ORIGINALACTIVITYID;
      
         INSERT INTO LG_EDI_FA_RECONCILE(
      		ORIGINALACTIVITYID
	      	,FROMPARTNERID
   		   ,TOPARTNERID
      		,CREATEDATE
	      	,INTCTLNUM
   		   ,GRPCTLNUM
      		,TXCTLNUM
            ,fadueby
            ,ignoredueby
            ,MostRecent
	      ) VALUES (
		      p_ORIGINALACTIVITYID
		      ,p_FROMPARTNERID
      		,p_TOPARTNERID
	      	,v_BeginTS
		      ,p_INTCTLNUM
   		   ,p_GRPCTLNUM
   	   	,p_TXCTLNUM
            ,p_fadueby
            ,p_ignoredueby
            ,1
   	   );    
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 lg_edi_fa_reconcileI' || SQLCODE|| ' '||SQLERRM);
END LG_EDI_FA_RECONCILEI;
/


-- LG_EDI_FA_RECONCILES.sql

CREATE OR REPLACE PROCEDURE LG_EDI_FA_RECONCILES
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	)
AS

/*
	================================================================================
	Proc Name:       LG_EDI_FA_RECONCILES
	Create Date:     2008-10-23 16:28:40
	Author:          Amit Kumar Vijay 


	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2008-10-23 Amit Vijay	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_EDI_FA_RECONCILES';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

v_ReturnCD	integer;
v_RowTS		integer :=0;
zeroUpdate  Exception;
v_ReconcileActivityID LG_EDI_FA_RECONCILE_QUE.ReconcileActivityID%TYPE;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	
	Select count(*) into v_RowCnt FROM LG_EDI_FA_RECONCILE_QUE
	;
		
	
	IF v_RowCnt = 0 THEN
	
			LOCK TABLE LG_EDI_FA_RECONCILE_QUE IN EXCLUSIVE MODE;
			
			Insert into LG_EDI_FA_RECONCILE_QUE
				( ReconcileActivityID
				)
			(select reconcile.OriginalActivityID
			 FROM lg_edi_fa_reconcile reconcile
			 	Inner join lg_edi_overdue_fa_vw overdue on overdue.OriginalActivityID=reconcile.OriginalActivityID
			 WHERE notifyStatus<>1
			 );
		
			Commit;
	
	END IF;
	
	Begin
	
	select ReconcileActivityID into v_ReconcileActivityID
		from LG_EDI_FA_RECONCILE_QUE
		where RowNum<2;
		
	EXCEPTION 
		WHEN NO_DATA_FOUND THEN 
			v_ReconcileActivityID:='-1';
	END;
	
		
		Delete from LG_EDI_FA_RECONCILE_QUE
		Where ReconcileActivityID=v_ReconcileActivityID;
	
		
	commit;
	
	IF RC1%isOpen THEN Close RC1; END IF;
	
	OPEN RC1 FOR
	Select reconcile.OriginalActivityID
		,la.FRBUSINESSIDHDR FRPartnerBusId
		,la.TOBUSINESSIDHDR ToPartnerBusId
		,overdue.frompartnerid frPartnerId
		,overdue.topartnerid toPartnerId
		,reconcile.INTCTLNUM INTCTLNUM
		,reconcile.GRPCTLNUM GRPCTLNUM
		,reconcile.TXCTLNUM TXCTLNUM
		,overdue.fadueby
		,overdue.overdueby
		,overdue.datesent
		,overdue.expectedtime
	FROM lg_edi_fa_reconcile reconcile
		Inner join lg_edi_overdue_fa_vw overdue on overdue.OriginalActivityID=reconcile.OriginalActivityID
	Inner join LG_ACTIVITY la on la.ActivityID=reconcile.OriginalActivityID
		WHERE notifyStatus<>1
	and reconcile.OriginalActivityID=v_ReconcileActivityID
	;
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN TOO_MANY_ROWS THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
	WHEN ZeroUpdate THEN
		v_ExceptionFlg := 1;
		p_ReturnCd := 3;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
	WHEN Others THEN
		ROLLBACK work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
End LG_EDI_FA_RECONCILES;
/


-- LG_EDI_FA_RECONCILES_AH1.SQL

CREATE OR REPLACE PROCEDURE lg_edi_fa_reconcileS_ah1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_INTCTLNUM in VARCHAR2
	,p_GRPCTLNUM in VARCHAR2
	,p_TXCTLNUM in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_EDI_FA_RECONCILES_ah1
	Create Date:     2004-09-02 14:00:57
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EDI_FA_RECONCILES_ah1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_INTCTLNUM=>'||p_INTCTLNUM||CHR(10)
		||', p_GRPCTLNUM=>'||p_GRPCTLNUM||CHR(10)
		||', p_TXCTLNUM=>'||p_TXCTLNUM||CHR(10)
		;
v_sqlstmt            varchar2(4000);

FUNCTION	Where_Clause(
        P_COLUMN    VARCHAR2
        ,P_PARAM    VARCHAR2
        )
RETURN VARCHAR2
IS
	v_where		VARCHAR2(2000) DEFAULT '';
BEGIN
	IF P_PARAM IS NOT NULL THEN
		v_where := ' AND '||p_Column||' ='''||p_Param||'''';
	END IF;
	RETURN v_where;
END Where_Clause;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_sqlstmt := 'SELECT
		RTRIM(ORIGINALACTIVITYID) ORIGINALACTIVITYID
		,FROMPARTNERID
		,TOPARTNERID
		,RTRIM(INTCTLNUM) INTCTLNUM
		,RTRIM(GRPCTLNUM) GRPCTLNUM
		,RTRIM(TXCTLNUM) TXCTLNUM
	FROM
		lg_edi_fa_reconcile
	Where
		FROMPARTNERID= COALESCE(' || RTRIM(TO_CHAR(p_FROMPARTNERID)) || ', FROMPARTNERID) ' ||
		' AND	TOPARTNERID= COALESCE(' || RTRIM(TO_CHAR(p_TOPARTNERID)) || ', TOPARTNERID)' ;

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt
		||	 Where_Clause( 'INTCTLNUM', p_INTCTLNUM)
		||	 Where_Clause( 'GRPCTLNUM', p_GRPCTLNUM)
		||	 Where_Clause( 'TXCTLNUM', p_TXCTLNUM)
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End lg_edi_fa_reconcileS_ah1;
/


-- LG_EDI_FA_RECONCILES_AH2.SQL

CREATE OR REPLACE PROCEDURE lg_edi_fa_reconcileS_ah2
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	,p_FROMPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_INTCTLNUM in VARCHAR2
	,p_GRPCTLNUM in VARCHAR2
	,p_TXCTLNUM in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_EDI_FA_RECONCILES_ah2
	Create Date:     2004-09-02 14:00:57
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EDI_FA_RECONCILES_ah2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FROMPARTNERID=>'||p_FROMPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_INTCTLNUM=>'||p_INTCTLNUM||CHR(10)
		||', p_GRPCTLNUM=>'||p_GRPCTLNUM||CHR(10)
		||', p_TXCTLNUM=>'||p_TXCTLNUM||CHR(10)
		;
v_sqlstmt            varchar2(4000);

FUNCTION	Where_Clause(
        P_COLUMN    VARCHAR2
        ,P_PARAM    VARCHAR2
        )
RETURN VARCHAR2
IS
	v_where		VARCHAR2(2000) DEFAULT '';
BEGIN
	IF P_PARAM IS NOT NULL THEN
		v_where := ' AND '||p_Column||' ='''||p_Param||'''';
	ELSE
		v_where := ' AND '||p_Column||' IS NULL';
	END IF;
	RETURN v_where;
END;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_sqlstmt := 'SELECT
		RTRIM(ORIGINALACTIVITYID) ORIGINALACTIVITYID
		,FROMPARTNERID
		,TOPARTNERID
		,RTRIM(INTCTLNUM) INTCTLNUM
		,RTRIM(GRPCTLNUM) GRPCTLNUM
		,RTRIM(TXCTLNUM) TXCTLNUM
	FROM
		lg_edi_fa_reconcile
	Where
		FROMPARTNERID= COALESCE(' || RTRIM(TO_CHAR(p_FROMPARTNERID)) || ', FROMPARTNERID) ' ||
		' AND	TOPARTNERID= COALESCE(' || RTRIM(TO_CHAR(p_TOPARTNERID)) || ', TOPARTNERID)' ;

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt
		||	 Where_Clause( 'INTCTLNUM', p_INTCTLNUM)
		||	 Where_Clause( 'GRPCTLNUM', p_GRPCTLNUM)
		||	 Where_Clause( 'TXCTLNUM', p_TXCTLNUM)
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End lg_edi_fa_reconcileS_ah2;
/


-- LG_EDI_FA_RECONCILEU.sql

Create or Replace Procedure LG_EDI_FA_RECONCILEU
	(
	 p_ReturnCd out integer
	,p_DebugFlg in pls_integer
	,p_OriginalActivityID in VARCHAR2
	,p_notifyStatus in NUMBER
	) as

/*
	================================================================================
	Proc Name:       LG_EDI_FA_RECONCILEU
	Create Date:     2008-10-23 10:27:24
	Author:          AMIT KUMAR VIJAY 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2008-10-23 Amit Vijay    6.2    Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_ROWCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EDI_FA_RECONCILEU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_OriginalActivityID=>'||p_OriginalActivityID||CHR(10)
		||', p_notifyStatus=>'||p_notifyStatus||CHR(10)
		;

v_ROWTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_ROWCnt := 0;

	UPDATE lg_edi_fa_reconcile
	      SET  notifyStatus = p_notifyStatus
	WHERE originalactivityid=p_OriginalActivityID;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - LG_EDI_FA_RECONCILEU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - LG_EDI_FA_RECONCILEU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - LG_EDI_FA_RECONCILEU:'||SQLERRM);
End LG_EDI_FA_RECONCILEU;
/



-- LG_EDI_FA_RECONCILEU_AH1.sql

CREATE OR REPLACE Procedure LG_EDI_FA_RECONCILEU_AH1
        (
         p_ReturnCd           OUT NUMBER
	     ,p_DebugFlg           IN  NUMBER
	     ,p_OriginalActivityID IN  VARCHAR2
) AS

/*
        ================================================================================
        Proc Name:       LG_EDI_FA_RECONCILEU_AH1
        Create Date:     2003-07-23 12:25:05
        Author:          PAD
        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'LG_EDI_FA_RECONCILEU_AH1';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
                  'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                  ||', p_OriginalActivityID=>'||p_OriginalActivityID||CHR(10)
                  ;

v_RowTS      INTEGER :=0;
zeroUpdate   EXCEPTION;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        v_RowCnt := 0;
        
        UPDATE lg_edi_fa_reconcile
        SET IgnoreDueBy = 1
        WHERE originalactivityid = p_OriginalActivityID;

        v_RowCnt := SQL%rowcount;
        IF (v_RowCnt = 0) THEN  RAISE ZeroUpdate ; END IF;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        EXCEPTION
                WHEN TOO_MANY_ROWS Then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999, '20100 - LG_EDI_FA_RECONCILEU_AH1: Too Many Rows');
                WHEN ZeroUpdate then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_EDI_FA_RECONCILEU_AH1: No Data Found');
                WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_EDI_FA_RECONCILEU_AH1:'||SQLERRM);
END LG_EDI_FA_RECONCILEU_AH1;
/


-- LG_EDI_OVERDUE_FA_VWS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_EDI_OVERDUE_FA_VWS_AH1
	(
	  p_ReturnCd      OUT PLS_INTEGER
	, p_DebugFlg      IN  PLS_INTEGER
   , RC1             OUT DB_REF.RCT
   , RC2             OUT DB_REF.RCT
   , p_Locale        IN  VARCHAR2
	, p_sortkey       IN  VARCHAR2
   , p_PageSizeNbr   IN  PLS_INTEGER
   , p_PageNbr       IN  PLS_INTEGER
   , p_STARTDATE     IN  TIMESTAMP -- (Date and Time)
   , p_ENDDATE       IN  TIMESTAMP -- (Date and Time)
   , p_SRCTRGFLG     IN  PLS_INTEGER -- (Source or Target Protocol/Process/Ver Indicator: 0 = Src, 1 =Trg)
   , p_FROMPARTNERID IN  VARCHAR2 -- (All = NULL, One, Comma Seperated list of Id's)
   , p_TOPARTNERID   IN  VARCHAR2 -- (All = NULL, One, Comma Seperated list of Id's)
   , p_PACKAGENAME   IN  VARCHAR2 -- (From or To based on SRCTRGFLG)
   , p_PACKAGEVER    IN  VARCHAR2 -- (From or To based on SRCTRGFLG)
   , p_PROTOCOLNAME  IN  VARCHAR2 -- (From or To based on SRCTRGFLG)
   , p_PROTOCOLVer   IN  VARCHAR2 -- (From or To based on SRCTRGFLG)
   , p_PROCESSCD     IN  VARCHAR2 -- (From or To based on SRCTRGFLG)
   , p_PROCESSVER    IN  VARCHAR2 -- (From or To based on SRCTRGFLG)
   , p_DocId         IN  VARCHAR2 -- LG_ACTIVITY.docid
	)
/*
	================================================================================
	Proc Name:       LG_EDI_OVERDUE_FA_VWS_AH1
	Create Date:     2004-09-09 16:15:12
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-09 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001, 2005 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

AS
v_RowCnt          PLS_INTEGER Default 0;
v_BeginTS         TimeStamp  ; -- Performance related variables
v_ProcName        VARCHAR(30)  Default 'LG_EDI_OVERDUE_FA_VWS_AH1';
v_ExceptionFlg    PLS_INTEGER Default 0;
v_ExceptionCode   VARCHAR(30);
v_PK_Value        PLS_INTEGER Default 0;
v_SQLErrm         VARCHAR(255) ;
v_chkpt           CHAR(1) default '0';
v_msg             VARCHAR(500);
v_RowTS           PLS_INTEGER Default 0;
v_count           PLS_INTEGER;
v_sqlstmt         VARCHAR2(4000) DEFAULT '';
v_sqlstmt2        VARCHAR2(4000) DEFAULT '';
v_where           VARCHAR2(1000) DEFAULT '';
v_startdate       VARCHAR2(20);
v_enddate         VARCHAR2(20);
v_sortKey         VARCHAR2(4000);
v_srctrgflg       PLS_INTEGER;
v_ParamsTxt       VARCHAR(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||',p_sortkey=>'|| p_sortkey||CHR(10)
        ||',p_PageSizeNbr=>'|| p_PageSizeNbr||CHR(10)
        ||',p_PageNbr=>'|| p_PageNbr||CHR(10)
        ;

BEGIN

--set default for sortcol to sort by longest delay
IF coalesce(p_sortkey,'') = '' THEN
   v_sortkey := 'datesent';
ELSE
   v_sortkey := p_sortkey;
END IF;

   v_BeginTS := FN_Get_Date();
   --v_startdate := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
   v_startdate   := to_char(coalesce(p_StartDATE,FN_Get_Date()), 'YYYY-MM-DD HH24:MI:SS');
   v_enddate     := to_char(coalesce(p_EndDATE,FN_Get_Date()), 'YYYY-MM-DD HH24:MI:SS');

   --double check these guys
   v_where := ' WHERE  datesent BETWEEN '||
        ' to_date('''||v_startDate||''',''YYYY-MM-DD HH24:MI:SS'') and'||
        ' to_date('''||v_endDate  ||''',''YYYY-MM-DD HH24:MI:SS'')'||
        FN_Build_Where('originalactivityid', 'VARCHAR2',p_DocId) ||
        FN_Build_Where('frompartnerid','NUMBER_LIST', p_FROMPARTNERID) ||
        FN_Build_Where('topartnerid','NUMBER_LIST', p_TOPARTNERID);


   v_SRCTRGFLG := COALESCE(p_SRCTRGFLG, 0);

   IF v_SRCTRGFLG = 0 THEN     -- From
      v_where := v_where ||
      FN_Build_Where('frpackagename',  'VARCHAR2', p_PACKAGENAME        )||
      FN_Build_Where('frpackagever',   'VARCHAR2', p_PACKAGEVER         )||
      FN_Build_Where('frprotocolname', 'VARCHAR2', p_PROTOCOLNAME       )||
      FN_Build_Where('frprotocolver',  'VARCHAR2', p_PROTOCOLVER        )||
      FN_Build_Where('frprocesscd',    'VARCHAR2', p_PROCESSCD          )||
      FN_Build_Where('frprocessver',   'VARCHAR2', p_PROCESSVER         );
   ELSIF v_SRCTRGFLG = 1 THEN -- To
      v_where := v_where ||
      FN_Build_Where('topackagename',  'VARCHAR2', p_PACKAGENAME        )||
      FN_Build_Where('topackagever',   'VARCHAR2', p_PACKAGEVER         )||
      FN_Build_Where('toprotocolname', 'VARCHAR2', p_PROTOCOLNAME       )||
      FN_Build_Where('toprotocolver',  'VARCHAR2', p_PROTOCOLVER        )||
      FN_Build_Where('toprocesscd',    'VARCHAR2', p_PROCESSCD          )||
      FN_Build_Where('toprocessver',   'VARCHAR2', p_PROCESSVER         );
   END if;

   v_where := v_where ||' AND originalactivityid NOT IN (SELECT DISTINCT originalactivityid FROM lg_ack) ';

	v_chkpt := '2';
   v_sqlstmt := 
      'SELECT ' ||
			' datesent ' ||
			' , originalactivityid' ||
			' , frompartnername' ||      
			' , topartnername'   ||
         ' , frprotocolname'  ||
         ' , frprotocolver'   ||
         ' , toprotocolname'  ||
         ' , toprotocolver'   ||
         ' , frpackagename'   ||
         ' , frpackagever'    ||
         ' , topackagename'   ||
         ' , topackagever'    ||
         ' , frprocesscd'     ||
         ' , frprocessver'    ||
         ' , toprocesscd'     ||
         ' , toprocessver'    ||
         ' , interchangecontrolnum' ||
         ' , groupcontrolnum' ||
         ' , transactioncontrolnum' ||
         ' , overdueby '||
         ' , expectedtime '||
      ' FROM ' ||
			' lg_edi_overdue_fa_vw ' ||      
      v_where ||
	   ' ORDER BY '||v_sortkey;

   --dbms_output.put_line('v_where is '||v_where);
   --dbms_output.put_line('v_sortkey is '||v_sortkey);

   --paging logic
   v_sqlstmt2 := FN_Paging_logic(p_pageNbr, p_pageSizeNbr, v_sqlstmt);

	v_chkpt := '3';
	--PREPARE v_stmt FROM v_sqlstmt;
   IF RC1%isOpen THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt2;

   -------------------------
   -- Total row count Cursor
   -------------------------
    IF coalesce (p_pageNbr,0) In (0,1) Then
        v_sqlstmt2 := v_sqlstmt;
        
        v_chkpt := '4';
        EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM ('||v_sqlstmt||')' INTO v_count;
        v_chkpt := '5';
        
        IF RC2%ISOPEN THEN CLOSE RC2; END IF;
            OPEN RC2 FOR SELECT v_count TotalRowsNbr FROM dual;
        
    END IF;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END LG_EDI_OVERDUE_FA_VWS_AH1;
/


-- LG_EDI_REJECTED_TX_VWS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_EDI_REJECTED_TX_VWS_AH1
	(
	 p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg    IN INTEGER
   ,RC1           OUT DB_REF.RCT
   ,RC2           OUT DB_REF.RCT
   ,p_Locale      IN VARCHAR2
	,p_sortkey     IN VARCHAR2
   ,p_PageSizeNbr IN PLS_INTEGER
   ,p_PageNbr     IN PLS_INTEGER
   ,p_STARTDATE   IN TIMESTAMP  -- (Date and Time)
   ,p_ENDDATE     IN TIMESTAMP  -- (Date and Time)
   ,p_SRCTRGFLG     PLS_INTEGER -- (Source or Target Protocol/Process/Ver Indicator: 0 = Src, 1 =Trg)
   ,p_FROMPARTNERID IN VARCHAR2 -- (All = NULL, One, Comma Seperated list of Id's)
   ,p_TOPARTNERID   IN VARCHAR2 -- (All = NULL, One, Comma Seperated list of Id's)
   ,p_PACKAGENAME   IN VARCHAR2 -- (From or To based on SRCTRGFLG)
   ,p_PACKAGEVER    IN VARCHAR2 -- (From or To based on SRCTRGFLG)
   ,p_PROTOCOLNAME  IN VARCHAR2 -- (From or To based on SRCTRGFLG)
   ,p_PROTOCOLVer   IN VARCHAR2 -- (From or To based on SRCTRGFLG)
   ,p_PROCESSCD     IN VARCHAR2 -- (From or To based on SRCTRGFLG)
   ,p_PROCESSVER    IN VARCHAR2 -- (From or To based on SRCTRGFLG)
   ,p_DocId         IN VARCHAR2 -- LG_ACTIVITY.docid
	)
AS
/*
	================================================================================
	Proc Name:       LG_EDI_REJECTED_TX_VWS_AH1
	Create Date:     2004-09-09 16:15:12
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-09 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001, 2005 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

v_RowCnt          PLS_INTEGER Default 0;
v_BeginTS         TimeStamp  ; -- Performance related variables
v_ProcName        VARCHAR(30)  Default 'LG_EDI_REJECTED_TX_VWS_AH1';
v_ExceptionFlg    PLS_INTEGER Default 0;
v_ExceptionCode   VARCHAR(30);
v_PK_Value        PLS_INTEGER Default 0;
v_SQLErrm         VARCHAR(255) ;
v_chkpt           CHAR(1) default '0';
v_msg             VARCHAR(500);
v_RowTS           PLS_INTEGER Default 0;
v_count           PLS_INTEGER;
v_sqlstmt         VARCHAR(4000) DEFAULT '';
v_sqlstmt2        VARCHAR(4000) DEFAULT '';
v_where           VARCHAR(1000) DEFAULT '';
v_startdate       VARCHAR(20);
v_enddate         VARCHAR(20);
v_sortKey         VARCHAR2(4000);
v_srctrgflg       PLS_INTEGER;
v_ParamsTxt       VARCHAR(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||',p_sortkey=>'|| p_sortkey||CHR(10)
        ||',p_PageSizeNbr=>'|| p_PageSizeNbr||CHR(10)
        ||',p_PageNbr=>'|| p_PageNbr||CHR(10)
        ;

BEGIN

--set default for sortcol to sort by longest delay
IF coalesce(p_sortkey,'') = '' THEN
   v_sortkey := 'createdate';
ELSE
   v_sortkey := p_sortkey;
END IF;

   v_BeginTS := FN_Get_Date();
   --v_startdate := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
   v_startdate   := to_char(coalesce(p_StartDATE,FN_Get_Date()), 'YYYY-MM-DD HH24:MI:SS');
   v_enddate     := to_char(coalesce(p_EndDATE,FN_Get_Date()), 'YYYY-MM-DD HH24:MI:SS');

   --double check these guys
   v_where := ' WHERE  createdate BETWEEN '||
        ' to_date('''||v_startDate||''',''YYYY-MM-DD HH24:MI:SS'') and'||
        ' to_date('''||v_endDate  ||''',''YYYY-MM-DD HH24:MI:SS'')'||
        FN_Build_Where('ackactivityid', 'VARCHAR2',p_DocId) ||
        FN_Build_Where('frompartnerid', 'NUMBER_LIST',p_frompartnerid) ||
        FN_Build_Where('topartnerid', 'NUMBER_LIST',p_topartnerid);
        

   v_SRCTRGFLG := COALESCE(p_SRCTRGFLG, 0);

   IF v_SRCTRGFLG = 0 then     -- From
      v_where := v_where ||
      FN_Build_Where('frpackagename',  'VARCHAR2', p_PACKAGENAME        )||
      FN_Build_Where('frpackagever',   'VARCHAR2', p_PACKAGEVER         )||
      FN_Build_Where('frprotocolname', 'VARCHAR2', p_PROTOCOLNAME       )||
      FN_Build_Where('frprotocolver',  'VARCHAR2', p_PROTOCOLVER        )||
      FN_Build_Where('frprocesscd',    'VARCHAR2', p_PROCESSCD          )||
      FN_Build_Where('frprocessver',   'VARCHAR2', p_PROCESSVER         );
   ELSIF v_SRCTRGFLG = 1 then -- To
      v_where := v_where ||
      FN_Build_Where('topackagename',  'VARCHAR2', p_PACKAGENAME        )||
      FN_Build_Where('topackagever',   'VARCHAR2', p_PACKAGEVER         )||
      FN_Build_Where('toprotocolname', 'VARCHAR2', p_PROTOCOLNAME       )||
      FN_Build_Where('toprotocolver',  'VARCHAR2', p_PROTOCOLVER        )||
      FN_Build_Where('toprocesscd',    'VARCHAR2', p_PROCESSCD          )||
      FN_Build_Where('toprocessver',   'VARCHAR2', p_PROCESSVER         );
   END if;

	v_chkpt := '2';
   v_sqlstmt := 
      'SELECT ' ||
			' createdate ' ||
			' , ackactivityid' ||
         --' , originalactivityid' ||
			' , frompartnername' ||      
			' , topartnername'   ||
         ' , frprotocolname'  ||
         ' , frprotocolver'   ||
         ' , toprotocolname'  ||
         ' , toprotocolver'   ||
         ' , frpackagename'   ||
         ' , frpackagever'    ||
         ' , topackagename'   ||
         ' , topackagever'    ||
         ' , frprocesscd'     ||
         ' , frprocessver'    ||
         ' , toprocesscd'     ||
         ' , toprocessver'    ||
         ' , statuscode' ||
         ' , statustext' ||
         ' , interchangecontrolnum' ||
         ' , groupcontrolnum' ||
         ' , transactioncontrolnum' ||
      ' FROM ' ||
			' lg_edi_rejected_tx_vw ' ||
      v_where||
	   ' ORDER BY '||p_sortkey;

   --paging logic
   v_sqlstmt2 := FN_Paging_logic(p_pageNbr, p_pageSizeNbr, v_sqlstmt);

	v_chkpt := '3';
	IF RC1%isOpen THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt2;

   -------------------------
   -- Total row count Cursor
   -------------------------
    IF coalesce (p_pageNbr,0) In (0,1) Then
        v_sqlstmt2 := v_sqlstmt;
        
        v_chkpt := '4';
        EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM ('||v_sqlstmt||')' INTO v_count;
        v_chkpt := '5';
        
        IF RC2%ISOPEN THEN CLOSE RC2; END IF;
            OPEN RC2 FOR SELECT v_count TotalRowsNbr FROM dual;
        
    END IF;

    p_ReturnCd := 0;
	 DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END LG_EDI_REJECTED_TX_VWS_AH1;
/


-- LG_ENVELOPE_INCLUSIOND.SQL

Create or Replace Procedure lg_envelope_inclusionD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_PARENTACTIVITYID in VARCHAR2
	,p_CHILDACTIVITYID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_ENVELOPE_INCLUSIOND
	Create Date:     2004-09-02 13:02:27
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_ENVELOPE_INCLUSIOND';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARENTACTIVITYID=>'||p_PARENTACTIVITYID||CHR(10)
		||', p_CHILDACTIVITYID=>'||p_CHILDACTIVITYID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From lg_envelope_inclusion
	Where
		PARENTACTIVITYID = p_PARENTACTIVITYID
		AND CHILDACTIVITYID = p_CHILDACTIVITYID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 lg_envelope_inclusionD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End lg_envelope_inclusionD;
/


-- LG_ENVELOPE_INCLUSIONI.SQL

Create or Replace Procedure LG_ENVELOPE_INCLUSIONI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_PARENTACTIVITYID in VARCHAR2
	,p_CHILDACTIVITYID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_ENVELOPE_INCLUSIONI
	Create Date:     2004-09-02 13:01:44
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_ENVELOPE_INCLUSIONI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARENTACTIVITYID=>'||p_PARENTACTIVITYID||CHR(10)
		||', p_CHILDACTIVITYID=>'||p_CHILDACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into lg_envelope_inclusion(
		PARENTACTIVITYID
		,CHILDACTIVITYID
	) Values (
		p_PARENTACTIVITYID
		,p_CHILDACTIVITYID
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 lg_envelope_inclusionI' || SQLCODE|| ' '||SQLERRM);
END LG_ENVELOPE_INCLUSIONI;
/


-- LG_ENVELOPE_INCLUSIONS_DRILLDN.SQL

CREATE OR REPLACE PROCEDURE lg_envelope_inclusionS_DRILLDN
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, RC2               out db_ref.rct_count          -- count
        , p_Locale          in  Varchar2
        , p_sortkey         in  varchar2
        , p_PageSizeNbr     in  pls_Integer
        , p_PageNbr         in  pls_Integer
	, p_PARENTACTIVITYID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_ENVELOPE_inclusionS_DRILLDN
	Create Date:     2004-09-02 14:57:40
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_ENVELOPE_inclusionS_DRILLDN';
v_ExceptionFlg    integer Default 0;
sqlstmt            varchar2(4000);
v_sortKey          varchar2(4000);
v_where            varchar2(4000);
sql_inner          varchar2(100);
v_count            integer;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||',p_sortkey=>'|| p_sortkey||CHR(10)
        ||',p_PageSizeNbr=>'|| p_PageSizeNbr||CHR(10)
        ||',p_PageNbr=>'|| p_PageNbr||CHR(10)
		||', p_PARENTACTIVITYID=>'||p_PARENTACTIVITYID||CHR(10)
		;
Procedure where_clause as
begin
    v_where := ' Where 1=1'||
    FN_Build_Where('PARENTACTIVITYID', 'VARCHAR2'   , p_PARENTACTIVITYID) ;
end where_clause;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	where_clause;
	sqlstmt := 'SELECT RTRIM(PARENTACTIVITYID) PARENTACTIVITYID '
		|| ' ,RTRIM(CHILDACTIVITYID) CHILDACTIVITYID '
		|| ' ,ROWTS ,CREATEDATE '
		|| ' FROM lg_envelope_inclusion'
		|| v_where;

	  IF NVL(INSTR(p_sortkey,'ASC'),0) >  0 THEN
	    v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'ASC')-1);
	    sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ')';
	   ELSIF INSTR(upper(p_sortkey),'DESC') > 0 THEN
	      v_sortkey := substr(p_sortkey,1,instr(upper(p_sortkey),'DESC')-1);
	      sqlstmt := sqlstmt ||' order by upper(' ||v_sortkey|| ') DESC';
	   ELSE
	       sqlstmt := sqlstmt ||' order by PARENTACTIVITYID DESC';
	  END IF;

	   sql_inner := 'FROM lg_envelope_inclusion';
	   
	   IF RC2%ISOPEN THEN CLOSE RC2; END IF;
	    IF NVL(P_PAGENBR,0) IN (0,1) THEN
	      Execute Immediate 'select count(*)  '||sql_inner ||v_where INTO v_count;
	  	       open RC2 for select v_count TotalRowsNbr from dual;
	     end if;
	-- Paging logic
	     sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
	 --  Set_page_parameters(p_pageNbr, p_pageSizeNbr, sqlstmt);

	    IF RC1%isOpen THEN Close RC1; END IF;
	    Open RC1 for sqlstmt;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End lg_envelope_inclusionS_DRILLDN;
/


-- LG_ENVELOPE_INCLUSIONS_DRILLUP.SQL

CREATE OR REPLACE PROCEDURE lg_envelope_inclusionS_DRILLUP
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CHILDACTIVITYID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_ENVELOPE_inclusionS_DRILLUP
	Create Date:     2004-09-02 14:57:40
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-02 SYSTEM        4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_ENVELOPE_inclusionS_DRILLUP';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CHILDACTIVITYID=>'||p_CHILDACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	IF p_CHILDACTIVITYID IS NOT NULL THEN
	OPEN RC1 FOR
	SELECT
		RTRIM(PARENTACTIVITYID) PARENTACTIVITYID
		,RTRIM(CHILDACTIVITYID) CHILDACTIVITYID
		,ROWTS
		,CREATEDATE
	FROM
		lg_envelope_inclusion
	Where
		CHILDACTIVITYID = p_CHILDACTIVITYID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		RTRIM(PARENTACTIVITYID) PARENTACTIVITYID
		,RTRIM(CHILDACTIVITYID) CHILDACTIVITYID
		,ROWTS
		,CREATEDATE
	FROM
		lg_envelope_inclusion
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End lg_envelope_inclusionS_DRILLUP;
/


-- LG_EVENTCDD.sql

CREATE OR REPLACE Procedure LG_EVENTCDD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_EVENTCODE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_EVENTCDD
	Create Date:     2004-03-26 15:32:41
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENTCDD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_EVENTCODE=>'||p_EVENTCODE||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From LG_EVENTCD
	Where 
		EVENTCODE = p_EVENTCODE
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_EVENTCDD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_EVENTCDD;
/


-- LG_EVENTCDI.sql

CREATE OR REPLACE Procedure LG_EVENTCDI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_EVENTCODE in VARCHAR2
	,p_EVENTNAME in VARCHAR2
	,p_PROVIDERFLAG in NUMBER
	,p_INTERNALDESC in VARCHAR2
	,p_EVENTNAMEKEY in VARCHAR2
	,p_TEXTKEY in VARCHAR2
	,p_VISIBILITY in NUMBER
	,p_SEVERITY in NUMBER
	,p_EVENTNAMERESOURCEBUNDLEID in NUMBER
	,p_ALERTABLEFLG in NUMBER
	,p_TEXTKEYRESOURCEBUNDLEID in NUMBER
	,p_EXTERNALDESC in VARCHAR2
	,p_EXTERNALDESCRESOURCEBUNDLEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_EVENTCDI
	Create Date:     2004-03-26 15:33:16
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENTCDI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_EVENTCODE=>'||p_EVENTCODE||CHR(10)
		||', p_EVENTNAME=>'||p_EVENTNAME||CHR(10)
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_INTERNALDESC=>'||p_INTERNALDESC||CHR(10)
		||', p_EVENTNAMEKEY=>'||p_EVENTNAMEKEY||CHR(10)
		||', p_TEXTKEY=>'||p_TEXTKEY||CHR(10)
		||', p_VISIBILITY=>'||p_VISIBILITY||CHR(10)
		||', p_SEVERITY=>'||p_SEVERITY||CHR(10)
		||', p_EVENTNAMERESOURCEBUNDLEID=>'||p_EVENTNAMERESOURCEBUNDLEID||CHR(10)
		||', p_ALERTABLEFLG=>'||p_ALERTABLEFLG||CHR(10)
		||', p_TEXTKEYRESOURCEBUNDLEID=>'||p_TEXTKEYRESOURCEBUNDLEID||CHR(10)
		||', p_EXTERNALDESC=>'||p_EXTERNALDESC||CHR(10)
		||', p_EXTERNALDESCRESOURCEBUNDLEID=>'||p_EXTERNALDESCRESOURCEBUNDLEID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_EVENTCD(
		EVENTCD
		,EVENTCODE
		,EVENTNAME
		,PROVIDERFLAG
		,INTERNALDESC
		,EVENTNAMEKEY
		,TEXTKEY
		,VISIBILITY
		,SEVERITY
		,EVENTNAMERESOURCEBUNDLEID
		,ALERTABLEFLG
		,TEXTKEYRESOURCEBUNDLEID
		,EXTERNALDESC
		,EXTERNALDESCRESOURCEBUNDLEID
	) Values (
		LG_EVENTCD_SEQ.nextval
		,p_EVENTCODE
		,p_EVENTNAME
		,p_PROVIDERFLAG
		,p_INTERNALDESC
		,p_EVENTNAMEKEY
		,p_TEXTKEY
		,p_VISIBILITY
		,p_SEVERITY
		,p_EVENTNAMERESOURCEBUNDLEID
		,p_ALERTABLEFLG
		,p_TEXTKEYRESOURCEBUNDLEID
		,p_EXTERNALDESC
		,p_EXTERNALDESCRESOURCEBUNDLEID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		LG_EVENTCD_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_EVENTCDI' || SQLCODE|| ' '||SQLERRM);
END LG_EVENTCDI;
/


-- LG_EVENTCDS.sql

CREATE OR REPLACE PROCEDURE LG_EVENTCDS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_EVENTCD IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_EVENTCDS
	Create Date:     2003-09-26 00:06:53
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENTCDS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_EVENTCD=>'||p_EVENTCD||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_EVENTCD IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		EVENTCD
		,RTRIM(EVENTNAME) EVENTNAME
		,RTRIM(INTERNALDESC) INTERNALDESC
		,RTRIM(EVENTNAMEKEY) EVENTNAMEKEY
		,RTRIM(TEXTKEY) TEXTKEY
		,VISIBILITY
		,SEVERITY
		,ALERTABLEFLG
		,C.ROWTS
		,C.CREATEDATE
		,RTRIM(EXTERNALDESC) EXTERNALDESC		
		,RTRIM(EVENTCODE) EVENTCODE
		,C.PROVIDERFLAG
		,B1.RESOURCEBUNDLEID EVENTNAMERESBUNDLEID
		,RTRIM(B1.RESBUNDLEFILENAME) EVENTNAMERESBUNDLE
		,B2.RESOURCEBUNDLEID TEXTRESBUNDLEID
		,RTRIM(B2.RESBUNDLEFILENAME) TEXTRESBUNDLE
		,B3.RESOURCEBUNDLEID EXTERNALDESCRESBUNDLEID
		,RTRIM(B3.RESBUNDLEFILENAME) EXTERNALDESCRESBUNDLE
	FROM
		LG_EVENTCD C 
		INNER JOIN PR_RESOURCEBUNDLES B1 ON C.EVENTNAMERESOURCEBUNDLEID    = B1.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B2 ON C.TEXTKEYRESOURCEBUNDLEID      = B2.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B3 ON C.EXTERNALDESCRESOURCEBUNDLEID = B3.RESOURCEBUNDLEID
	Where
		C.EVENTCD = p_EVENTCD
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		EVENTCD
		,RTRIM(EVENTNAME) EVENTNAME
		,RTRIM(INTERNALDESC) INTERNALDESC
		,RTRIM(EVENTNAMEKEY) EVENTNAMEKEY
		,RTRIM(TEXTKEY) TEXTKEY
		,VISIBILITY
		,SEVERITY
		,ALERTABLEFLG
		,C.ROWTS
		,C.CREATEDATE
		,RTRIM(EXTERNALDESC) EXTERNALDESC
		,RTRIM(EVENTCODE) EVENTCODE
		,C.PROVIDERFLAG
		,B1.RESOURCEBUNDLEID EVENTNAMERESBUNDLEID
		,RTRIM(B1.RESBUNDLEFILENAME) EVENTNAMERESBUNDLE
		,B2.RESOURCEBUNDLEID TEXTRESBUNDLEID
		,RTRIM(B2.RESBUNDLEFILENAME) TEXTRESBUNDLE
		,B3.RESOURCEBUNDLEID EXTERNALDESCRESBUNDLEID
		,RTRIM(B3.RESBUNDLEFILENAME) EXTERNALDESCRESBUNDLE
	FROM
		LG_EVENTCD C
		INNER JOIN PR_RESOURCEBUNDLES B1 ON C.EVENTNAMERESOURCEBUNDLEID    = B1.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B2 ON C.TEXTKEYRESOURCEBUNDLEID      = B2.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B3 ON C.EXTERNALDESCRESOURCEBUNDLEID = B3.RESOURCEBUNDLEID
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_EVENTCDS;
/


-- LG_EVENTCDS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_EVENTCDS_AH1
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_EVENTCD IN NUMBER
    , p_Severity IN PLS_INTEGER
    , p_Visibility IN Varchar2  -- 1=hub Owner  2=sponcer  4= trading partner
    , p_EventName IN Varchar2
    )
AS

/*
    ================================================================================
    Proc Name:       LG_EVENTCDS_AH1
    Create Date:     2003-07-02 10:45:25
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-07-02 V.Goel           3.5 Created
    2003-07-08 V.Goel           3.5 p_Visibility is a list
    2004-05-07 PaulD         4.2.2  Added Order By
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENTCDS_AH1';
v_ExceptionFlg    integer Default 0;
v_where           varchar2(1000);
v_ParamsTxt       varchar2(4000) Default
        'p_DebugFlg=>'      || p_DebugFlg  ||CHR(10)
        ||', p_EVENTCD=>'   || p_EVENTCD   ||CHR(10)
        ||', p_Severity=>'  || p_Severity  ||CHR(10)
        ||', p_Visibility=>'|| p_Visibility||CHR(10)
        ||', p_EventName=>' || p_EventName ||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_where := 'Where 1 = 1'
        ||FN_BUILD_WHERE('EVENTCD',     'NUMBER', p_EVENTCD)
        ||FN_BUILD_WHERE('SEVERITY',    'NUMBER', p_Severity)
        ||FN_BUILD_WHERE('VISIBILITY',  'NUMBER_LIST', p_Visibility)
        ||FN_BUILD_WHERE('EVENTNAME',   'VARCHAR2_LIKE', p_EventName);
    DBMS_OUTPUT.Put_Line('where clause :'||v_where);

    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    'SELECT
        EVENTCD
        ,RTRIM(EVENTNAME) EVENTNAME
        ,RTRIM(INTERNALDESC) INTERNALDESC
        ,RTRIM(EVENTNAMEKEY) EVENTNAMEKEY
        ,RTRIM(TEXTKEY) TEXTKEY
        ,VISIBILITY
        ,SEVERITY
        ,ALERTABLEFLG
        ,C.ROWTS
        ,C.CREATEDATE
        ,RTRIM(EXTERNALDESC) EXTERNALDESC
		,RTRIM(EVENTCODE) EVENTCODE
		,C.PROVIDERFLAG
		,B1.RESOURCEBUNDLEID EVENTNAMERESBUNDLEID
		,RTRIM(B1.RESBUNDLEFILENAME) EVENTNAMERESBUNDLE
		,B2.RESOURCEBUNDLEID TEXTRESBUNDLEID
		,RTRIM(B2.RESBUNDLEFILENAME) TEXTRESBUNDLE
		,B3.RESOURCEBUNDLEID EXTERNALDESCRESBUNDLEID
		,RTRIM(B3.RESBUNDLEFILENAME) EXTERNALDESCRESBUNDLE
    FROM
        LG_EVENTCD C 
		INNER JOIN PR_RESOURCEBUNDLES B1 ON C.EVENTNAMERESOURCEBUNDLEID    = B1.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B2 ON C.TEXTKEYRESOURCEBUNDLEID      = B2.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B3 ON C.EXTERNALDESCRESOURCEBUNDLEID = B3.RESOURCEBUNDLEID '
        ||v_where||
    ' ORDER BY C.EVENTCODE '
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_EVENTCDS_AH1;
/


-- LG_EVENTCDS_AH2.sql

CREATE OR REPLACE PROCEDURE LG_EVENTCDS_AH2
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_EVENTCODE IN VARCHAR2
    , p_Severity IN PLS_INTEGER
    , p_Visibility IN Varchar2  -- 1=hub Owner  2=sponcer  4= trading partner
    , p_EventName IN Varchar2
    )
AS

/*
    ================================================================================
    Proc Name:       LG_EVENTCDS_AH1
    Create Date:     2003-07-02 10:45:25
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-03-26 T. Dang       4.2.2  Created
    2004-05-07 PaulD         4.2.2  Added Order By
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENTCDS_AH2';
v_ExceptionFlg    integer Default 0;
v_where           varchar2(1000);
v_ParamsTxt       varchar2(4000) Default
        'p_DebugFlg=>'      || p_DebugFlg  ||CHR(10)
        ||', p_EVENTCODE=>' || p_EVENTCODE ||CHR(10)
        ||', p_Severity=>'  || p_Severity  ||CHR(10)
        ||', p_Visibility=>'|| p_Visibility||CHR(10)
        ||', p_EventName=>' || p_EventName ||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_where := 'Where 1 = 1'
        ||FN_BUILD_WHERE('EVENTCODE', 'VARCHAR2', p_EVENTCODE)
        ||FN_BUILD_WHERE('SEVERITY',  'NUMBER', p_Severity)
        ||FN_BUILD_WHERE('VISIBILITY','NUMBER_LIST', p_Visibility)
        ||FN_BUILD_WHERE('EVENTNAME', 'VARCHAR2_LIKE', p_EventName);
    DBMS_OUTPUT.Put_Line('where clause :'||v_where);

    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    'SELECT
        EVENTCD
        ,RTRIM(EVENTNAME) EVENTNAME
        ,RTRIM(INTERNALDESC) INTERNALDESC
        ,RTRIM(EVENTNAMEKEY) EVENTNAMEKEY
        ,RTRIM(TEXTKEY) TEXTKEY
        ,VISIBILITY
        ,SEVERITY
        ,ALERTABLEFLG
        ,C.ROWTS
        ,C.CREATEDATE
        ,RTRIM(EXTERNALDESC) EXTERNALDESC
		,RTRIM(EVENTCODE) EVENTCODE
		,C.PROVIDERFLAG
		,B1.RESOURCEBUNDLEID EVENTNAMERESBUNDLEID
		,RTRIM(B1.RESBUNDLEFILENAME) EVENTNAMERESBUNDLE
		,B2.RESOURCEBUNDLEID TEXTRESBUNDLEID
		,RTRIM(B2.RESBUNDLEFILENAME) TEXTRESBUNDLE
		,B3.RESOURCEBUNDLEID EXTERNALDESCRESBUNDLEID
		,RTRIM(B3.RESBUNDLEFILENAME) EXTERNALDESCRESBUNDLE
    FROM
        LG_EVENTCD C 
		INNER JOIN PR_RESOURCEBUNDLES B1 ON C.EVENTNAMERESOURCEBUNDLEID    = B1.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B2 ON C.TEXTKEYRESOURCEBUNDLEID      = B2.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B3 ON C.EXTERNALDESCRESOURCEBUNDLEID = B3.RESOURCEBUNDLEID '
        ||v_where||
    ' ORDER BY C.EVENTCODE '
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

END LG_EVENTCDS_AH2 ;
/


-- LG_EVENTCDS_AH3.sql

CREATE OR REPLACE PROCEDURE LG_EVENTCDS_AH3
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_EVENTCODE IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_EVENTCDS
	Create Date:     2003-09-26 00:06:53
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-26 T. Dang        4.2.2 Created
   2004-05-07 PaulD         4.2.2  Added Order By
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENTCDS_AH3';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_EVENTCODE=>'||p_EVENTCODE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_EVENTCODE IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		EVENTCD
		,RTRIM(EVENTNAME) EVENTNAME
		,RTRIM(INTERNALDESC) INTERNALDESC
		,RTRIM(EVENTNAMEKEY) EVENTNAMEKEY
		,RTRIM(TEXTKEY) TEXTKEY
		,VISIBILITY
		,SEVERITY
		,ALERTABLEFLG
		,C.ROWTS
		,C.CREATEDATE
		,RTRIM(EXTERNALDESC) EXTERNALDESC		
		,RTRIM(EVENTCODE) EVENTCODE
		,C.PROVIDERFLAG
		,B1.RESOURCEBUNDLEID EVENTNAMERESBUNDLEID
		,RTRIM(B1.RESBUNDLEFILENAME) EVENTNAMERESBUNDLE
		,B2.RESOURCEBUNDLEID TEXTRESBUNDLEID
		,RTRIM(B2.RESBUNDLEFILENAME) TEXTRESBUNDLE
		,B3.RESOURCEBUNDLEID EXTERNALDESCRESBUNDLEID
		,RTRIM(B3.RESBUNDLEFILENAME) EXTERNALDESCRESBUNDLE
	FROM
		LG_EVENTCD C 
		INNER JOIN PR_RESOURCEBUNDLES B1 ON C.EVENTNAMERESOURCEBUNDLEID    = B1.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B2 ON C.TEXTKEYRESOURCEBUNDLEID      = B2.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B3 ON C.EXTERNALDESCRESOURCEBUNDLEID = B3.RESOURCEBUNDLEID
	Where
		C.EVENTCODE = p_EVENTCODE
   ORDER BY
      C.EVENTCODE
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		EVENTCD
		,RTRIM(EVENTNAME) EVENTNAME
		,RTRIM(INTERNALDESC) INTERNALDESC
		,RTRIM(EVENTNAMEKEY) EVENTNAMEKEY
		,RTRIM(TEXTKEY) TEXTKEY
		,VISIBILITY
		,SEVERITY
		,ALERTABLEFLG
		,C.ROWTS
		,C.CREATEDATE
		,RTRIM(EXTERNALDESC) EXTERNALDESC
		,RTRIM(EVENTCODE) EVENTCODE
		,C.PROVIDERFLAG
		,B1.RESOURCEBUNDLEID EVENTNAMERESBUNDLEID
		,RTRIM(B1.RESBUNDLEFILENAME) EVENTNAMERESBUNDLE
		,B2.RESOURCEBUNDLEID TEXTRESBUNDLEID
		,RTRIM(B2.RESBUNDLEFILENAME) TEXTRESBUNDLE
		,B3.RESOURCEBUNDLEID EXTERNALDESCRESBUNDLEID
		,RTRIM(B3.RESBUNDLEFILENAME) EXTERNALDESCRESBUNDLE
	FROM
		LG_EVENTCD C
		INNER JOIN PR_RESOURCEBUNDLES B1 ON C.EVENTNAMERESOURCEBUNDLEID    = B1.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B2 ON C.TEXTKEYRESOURCEBUNDLEID      = B2.RESOURCEBUNDLEID
		INNER JOIN PR_RESOURCEBUNDLES B3 ON C.EXTERNALDESCRESOURCEBUNDLEID = B3.RESOURCEBUNDLEID
   ORDER BY
      C.EVENTCODE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_EVENTCDS_AH3;
/


-- LG_EVENTCDU.sql

CREATE OR REPLACE Procedure LG_EVENTCDU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_EVENTCD in NUMBER
	,p_EVENTNAME in VARCHAR2
	,p_INTERNALDESC in VARCHAR2
	,p_EVENTNAMEKEY in VARCHAR2
	,p_TEXTKEY in VARCHAR2
	,p_VISIBILITY in NUMBER
	,p_SEVERITY in NUMBER
	,p_ALERTABLEFLG in NUMBER
	,p_ROWTS in NUMBER
	,p_EXTERNALDESC in VARCHAR2
	,p_EVENTCODE in VARCHAR2
	,p_PROVIDERFLAG in NUMBER
	,p_EVENTNAMERESOURCEBUNDLEID in NUMBER
	,p_TEXTKEYRESOURCEBUNDLEID in NUMBER
	,p_EXTERNALDESCRESOURCEBUNDLEID in NUMBER
) as

/*
	================================================================================
	Proc Name:       LG_EVENTCDU
	Create Date:     2003-09-26 08:21:00
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENTCDU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_EVENTCD=>'||p_EVENTCD||CHR(10)
		||', p_EVENTNAME=>'||p_EVENTNAME||CHR(10)
		||', p_INTERNALDESC=>'||p_INTERNALDESC||CHR(10)
		||', p_EVENTNAMEKEY=>'||p_EVENTNAMEKEY||CHR(10)
		||', p_TEXTKEY=>'||p_TEXTKEY||CHR(10)
		||', p_VISIBILITY=>'||p_VISIBILITY||CHR(10)
		||', p_SEVERITY=>'||p_SEVERITY||CHR(10)
		||', p_ALERTABLEFLG=>'||p_ALERTABLEFLG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_EXTERNALDESC=>'||p_EXTERNALDESC||CHR(10)
		||', p_EVENTCODE=>'||p_EVENTCODE||CHR(10)		
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_EVENTNAMERESOURCEBUNDLEID=>'||p_EVENTNAMERESOURCEBUNDLEID||CHR(10)	
		||', p_TEXTKEYRESOURCEBUNDLEID=>'||p_TEXTKEYRESOURCEBUNDLEID||CHR(10)
		||', p_EXTERNALDESCRESOURCEBUNDLEID=>'||p_EXTERNALDESCRESOURCEBUNDLEID||CHR(10)					
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
    IF p_EVENTCD IS NOT NULL THEN
	Update LG_EVENTCD
	SET
		EVENTNAME=p_EVENTNAME
		,      INTERNALDESC=p_INTERNALDESC
		,      EVENTNAMEKEY=p_EVENTNAMEKEY
		,      TEXTKEY=p_TEXTKEY
		,      VISIBILITY=p_VISIBILITY
		,      SEVERITY=p_SEVERITY
		,      ALERTABLEFLG=p_ALERTABLEFLG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      EXTERNALDESC=p_EXTERNALDESC
		,      PROVIDERFLAG=p_PROVIDERFLAG
		,      EVENTNAMERESOURCEBUNDLEID=p_EVENTNAMERESOURCEBUNDLEID
		,      TEXTKEYRESOURCEBUNDLEID=p_TEXTKEYRESOURCEBUNDLEID
		,      EXTERNALDESCRESOURCEBUNDLEID=p_EXTERNALDESCRESOURCEBUNDLEID
	Where
		EVENTCD = p_EVENTCD
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	ELSE
	Update LG_EVENTCD
	SET
		EVENTNAME=p_EVENTNAME
		,      INTERNALDESC=p_INTERNALDESC
		,      EVENTNAMEKEY=p_EVENTNAMEKEY
		,      TEXTKEY=p_TEXTKEY
		,      VISIBILITY=p_VISIBILITY
		,      SEVERITY=p_SEVERITY
		,      ALERTABLEFLG=p_ALERTABLEFLG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      EXTERNALDESC=p_EXTERNALDESC
		,      PROVIDERFLAG=p_PROVIDERFLAG
		,      EVENTNAMERESOURCEBUNDLEID=p_EVENTNAMERESOURCEBUNDLEID
		,      TEXTKEYRESOURCEBUNDLEID=p_TEXTKEYRESOURCEBUNDLEID
		,      EXTERNALDESCRESOURCEBUNDLEID=p_EXTERNALDESCRESOURCEBUNDLEID
	Where
		EVENTCODE = p_EVENTCODE
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	END IF ;
	
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - LG_EVENTCDU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - LG_EVENTCDU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - LG_EVENTCDU:'||SQLERRM);
End LG_EVENTCDU;
/


-- LG_EVENTI_AH1.sql

CREATE OR REPLACE PROCEDURE         LG_EVENTI_AH1
(
   p_ReturnCd         out integer
   ,p_DebugFlg        in pls_integer
   ,p_EVENTID         in VARCHAR2
   ,p_EVENTCD         in NUMBER
   ,p_EVENTTS         in TimeStamp
   ,p_FAULTTYPE       in NUMBER
   ,p_SOURCECLASS     in out VARCHAR2
   ,p_HOSTIPADDRESS   in out VARCHAR2
   ,p_PARTNERID1      in NUMBER
   ,p_PARTNERID2      in NUMBER
   ,p_ARGS            in out VARCHAR2
   ,p_USERID          in NUMBER
   ,P_BUSINESSID1     IN VARCHAR2         -- only provided if partnerId is null
   ,P_BUSINESSID2     IN VARCHAR2         -- only provided if partnerId is null
   ,p_TEXTID          IN VARCHAR2
) as
/*
================================================================================
Proc Name:        LG_EVENTI_AH1
Create Date:     2002-07-08 10:28:00
Author:            SYSTEM
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
---------- ------------- ------ ---------------------------------------------
DATE         NAME        Ver     DESCRIPTION
---------- ------------- ------ ---------------------------------------------
2002-07-08 SYSTEM        3.3      Created
2002-08-27 V Goel        3.3      Check Length of ARGS Defect # 1-VXRP
2002-09-26 V Goel        3.3      Check Length of sourceclass, hostipaddress Defect # 1-Y12A
2002-10-28 J.Chen        3.3      Added businessId as input incase partnerid is not available
2003-01-16 V Goel        3.4      Change EventTS data type to TimeStamp
2003-01-30 V Goel        3.4      Use db_lengthCheck.event to check/truncate length, change
                                  "in" to "in out" for parameters candidate for length check
2003-04-28 V Goel        3.5      Re-name the proc from LG_EVENTI to LG_EVENTI_AH1
2003-06-19 V Goel        3.5      Replace Pkg DB_LENGTHCHECK.EVENT by Proc DB_LENGTHCHECK_EVENT
2003-07-08 V Goel        3.5      Use FN_Get_PartnerId Function to get PartnerID from BusinessId
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt   integer     :=0;
v_BeginTS  timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
v_sourceclass     lg_event.sourceclass%TYPE;
v_hostipaddress   lg_event.hostipaddress%TYPE;
v_args     lg_event.args%Type;
v_partnerId1 lg_event.partnerId1%Type;
v_partnerid2 lg_event.partnerId2%Type;
--
-- Procedures
--
Procedure ProcAudit is
begin
--If p_DebugFlg =1 Then
DB_ProcAuditLogI_ah1('LG_EVENTI_AH1',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
||','||p_EVENTID
||','||p_EVENTCD
||','||p_EVENTTS
||','||p_FAULTTYPE
||','||p_SOURCECLASS
||','||p_HOSTIPADDRESS
||','||p_PARTNERID1
||','||p_PARTNERID2
||','||p_ARGS
||','||p_USERID
||','||p_BusinessId1
||','||p_BusinessId2
||','||p_TEXTID
);
--End if ;
End ProcAudit;

Procedure Check_length is
begin
   DB_LENGTHCHECK_EVENT(
    P_SOURCECLASS   => P_SOURCECLASS
   ,P_HOSTIPADDRESS => P_HOSTIPADDRESS
   ,P_ARGS          => P_ARGS
   );
end Check_length;
-------------------------------------------------------------------------
--                       M A I N                                       --
-------------------------------------------------------------------------
BEGIN
    p_ReturnCd := 0;
    Check_length;
    If p_PartnerId1 is null then
       v_PartnerId1 := FN_Get_PartnerId(p_BusinessId1);
    else
       v_PartnerId1 := p_PartnerId1;
    end if;

     If p_PartnerId2 is null then
       v_PartnerId2 := FN_Get_PartnerId(p_BusinessId2);
    else
       v_PartnerId2 := p_PartnerId2;
    end if;

Insert into LG_EVENT(
    EVENTID
,    EVENTCD
,    EVENTTS
,    FAULTTYPE
,    SOURCECLASS
,    HOSTIPADDRESS
,    PARTNERID1
,    PARTNERID2
,    ARGS
,    USERID
,    CREATEDATE
,    TEXTID
) Values (
    p_EVENTID
,    p_EVENTCD
,    p_EVENTTS
,    p_FAULTTYPE
,    p_SOURCECLASS
,    p_HOSTIPADDRESS
,    v_PARTNERID1
,    v_PARTNERID2
,    p_ARGS
,    p_USERID
,    v_BeginTS
,    p_TEXTID
);
ProcAudit;
EXCEPTION
When NO_DATA_FOUND Then Null;
When Others Then
ProcAudit;
   Raise_application_error (-20999,'20002 LG_EVENTI_AH1' || SQLCODE|| ' '||SQLERRM);
   p_ReturnCd := 1;
END LG_EVENTI_AH1;
/


-- LG_EVENTI_AH2.sql

CREATE OR REPLACE PROCEDURE LG_EVENTI_AH2
(
   p_ReturnCd         out integer
   ,p_DebugFlg        in pls_integer
   ,p_EVENTID         in VARCHAR2
   ,p_EVENTCODE       in VARCHAR2
   ,p_EVENTTS         in TimeStamp
   ,p_FAULTTYPE       in NUMBER
   ,p_SOURCECLASS     in out VARCHAR2
   ,p_HOSTIPADDRESS   in out VARCHAR2
   ,p_FRIPADDRESS     in VARCHAR2
   ,p_PARTNERID1      in NUMBER
   ,p_PARTNERID2      in NUMBER
   ,p_ARGS            in out VARCHAR2
   ,p_USERID          in NUMBER
   ,P_BUSINESSID1     IN VARCHAR2         -- only provided if partnerId is null
   ,P_BUSINESSID2     IN VARCHAR2         -- only provided if partnerId is null
   ,p_TEXTID          IN VARCHAR2
   ,p_ACTIVITYID     IN VARCHAR2
   ,p_VUID           IN VARCHAR2
) as
/*
================================================================================
Proc Name:        LG_EVENTI_AH1
Create Date:     2002-07-08 10:28:00
Author:            SYSTEM
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
---------- ------------- ------ ---------------------------------------------
DATE         NAME        Ver     DESCRIPTION
---------- ------------- ------ ---------------------------------------------
2002-07-08 SYSTEM        3.3      Created
2002-08-27 V Goel        3.3      Check Length of ARGS Defect # 1-VXRP
2002-09-26 V Goel        3.3      Check Length of sourceclass, hostipaddress Defect # 1-Y12A
2002-10-28 J.Chen        3.3      Added businessId as input incase partnerid is not available
2003-01-16 V Goel        3.4      Change EventTS data type to TimeStamp
2003-01-30 V Goel        3.4      Use db_lengthCheck.event to check/truncate length, change
                                  "in" to "in out" for parameters candidate for length check
2003-04-28 V Goel        3.5      Re-name the proc from LG_EVENTI to LG_EVENTI_AH1
2003-06-19 V Goel        3.5      Replace Pkg DB_LENGTHCHECK.EVENT by Proc DB_LENGTHCHECK_EVENT
2003-07-08 V Goel        3.5      Use FN_Get_PartnerId Function to get PartnerID from BusinessId
2004-03-30 T. Dang     4.2.2      Use alphanumeric EVENT CODE
2008-10-20 Amit vijay	 6.2	  Defect 107019
2008-11-17 BHARABI J     6.2      Defect 107624
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_EVENTCD  integer ;
v_rowcnt   integer     :=0;
v_BeginTS  timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
v_sourceclass     lg_event.sourceclass%TYPE;
v_hostipaddress   lg_event.hostipaddress%TYPE;
v_args     lg_event.args%Type;
v_partnerId1 lg_event.partnerId1%Type;
v_partnerid2 lg_event.partnerId2%Type;

--
-- Procedures
--
Procedure ProcAudit is
begin
--If p_DebugFlg =1 Then
DB_ProcAuditLogI_ah1('LG_EVENTI_AH2',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
||','||p_EVENTID
||','||p_EVENTCODE
||','||p_EVENTTS
||','||p_FAULTTYPE
||','||p_SOURCECLASS
||','||p_HOSTIPADDRESS
||','||p_PARTNERID1
||','||p_PARTNERID2
||','||p_ARGS
||','||p_USERID
||','||p_BusinessId1
||','||p_BusinessId2
||','||p_TEXTID
||','||p_ACTIVITYID
||','||p_VUID
);
--End if ;
End ProcAudit;

Procedure Check_length is
begin
   DB_LENGTHCHECK_EVENT(
    P_SOURCECLASS   => P_SOURCECLASS
   ,P_HOSTIPADDRESS => P_HOSTIPADDRESS
   ,P_ARGS          => P_ARGS
   );
end Check_length;
-------------------------------------------------------------------------
--                       M A I N                                       --
-------------------------------------------------------------------------
BEGIN
    p_ReturnCd := 0;
    Check_length;
    v_PartnerId1 := nvl(p_PartnerId1, FN_Get_PartnerId(p_BusinessId1) );
    v_PartnerId2 := nvl(p_PartnerId2, FN_Get_PartnerId(p_BusinessId2) );
Begin  -- LG_Event
    IF LENGTH (TRIM (TRANSLATE (p_EVENTCODE, ' +-.0123456789', ' ') ) ) IS NULL THEN
	   v_EVENTCD := TO_NUMBER(p_EVENTCODE)  ;
	ELSE
	   select EVENTCD into v_EVENTCD from LG_EVENTCD where EVENTCODE = p_EVENTCODE 
	   ;
	END IF ;
IF p_Vuid IS NOT NULL AND p_ActivityID IS NOT NULL and p_EventID IS NOT NULL THEN	
    Insert into LG_EVENT(
        EVENTID
    ,    EVENTCD
    ,    EVENTTS
    ,    FAULTTYPE
    ,    SOURCECLASS
    ,    HOSTIPADDRESS
    ,    PARTNERID1
    ,    PARTNERID2
    ,    ARGS
    ,    USERID
    ,    CREATEDATE
    ,    TEXTID
    ,    ACTIVITYID
    ,    VUID
    
    ) Values (
        p_EVENTID
    ,    v_EVENTCD
    ,    p_EVENTTS
    ,    p_FAULTTYPE
    ,    p_SOURCECLASS
    ,    p_HOSTIPADDRESS
    ,    v_PARTNERID1
    ,    v_PARTNERID2
    ,    p_ARGS
    ,    p_USERID
    ,    v_BeginTS
    ,    p_TEXTID
    ,    p_ACTIVITYID
    ,    p_VUID
    );
ELSE
    Insert into LG_EVENT(
            EVENTID
        ,    EVENTCD
        ,    EVENTTS
        ,    FAULTTYPE
        ,    SOURCECLASS
        ,    HOSTIPADDRESS
        ,    PARTNERID1
        ,    PARTNERID2
        ,    ARGS
        ,    USERID
        ,    CREATEDATE
        ,    TEXTID
        ) Values (
            p_EVENTID
        ,    v_EVENTCD
        ,    p_EVENTTS
        ,    p_FAULTTYPE
        ,    p_SOURCECLASS
        ,    p_HOSTIPADDRESS
        ,    v_PARTNERID1
        ,    v_PARTNERID2
        ,    p_ARGS
        ,    p_USERID
        ,    v_BeginTS
        ,    p_TEXTID
        );
END IF;
    ProcAudit;
    EXCEPTION
        When NO_DATA_FOUND Then 
        Null;
        When DUP_VAL_ON_INDEX then
	  	p_returnCd := 2;
        When Others Then
            p_ReturnCd := 1;
            ProcAudit;
            Raise_application_error (-20999,'20002 LG_EVENTI_AH2' || SQLCODE|| ' '||SQLERRM);

End; -- LG_Event

END LG_EVENTI_AH2;
/


-- LG_EVENTI_AH3.sql

CREATE OR REPLACE PROCEDURE LG_EVENTI_AH3
(
   p_ReturnCd		out pls_integer
   ,p_DebugFlg		in pls_integer
   ,p_EVENTID		in VARCHAR2
   ,p_EVENTCODE		in VARCHAR2
   ,p_EVENTTS		in TimeStamp
   ,p_FAULTTYPE		IN NUMBER
   ,p_SOURCECLASS	in VARCHAR2
   ,p_HOSTIPADDRESS	in VARCHAR2
   ,p_FRIPADDRESS	in VARCHAR2
   ,p_PARTNERID1	in NUMBER
   ,p_PARTNERID2	in NUMBER
   ,p_ARGS		in VARCHAR2
   ,p_USERID		in NUMBER
   ,P_BUSINESSID1	IN VARCHAR2         -- only provided if partnerId is null
   ,P_BUSINESSID2	IN VARCHAR2         -- only provided if partnerId is null
   ,p_TEXTID		IN VARCHAR2
   ,p_ACTIVITYID	IN VARCHAR2
   ,p_VUID		IN VARCHAR2
   ,p_USERNAME		IN VARCHAR2
   ,p_ORIGINALFILENAME	IN VARCHAR2
   ,p_LASTMODIFIEDTIME	IN VARCHAR2
)

AS

/*
	================================================================================
	Proc Name:       LG_EVENTI_AH3
	Create Date:     2007-09-03 10:26:49
	Author:          AMIT KUMAR VIJAY

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2007-09-03 AMIT V.	6.1.1	Created
	2008-10-20 Amit vijay	6.2	Defect 107019
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_Pk_Value number := 0;

v_RowCnt		integer Default 0;
v_BeginTS		timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName		varchar2(30) Default 'LG_EVENTI_AH3';
v_EVENTCD		integer ;
v_sourceclass		lg_event.sourceclass%TYPE := p_sourceclass;
v_hostipaddress		lg_event.hostipaddress%TYPE := p_hostipaddress;
v_partnerId1		lg_event.partnerId1%Type;
v_partnerid2		lg_event.partnerId2%Type;
v_FTPUSERID		CF_FTP_USER.FTPUSERID%Type DEFAULT NULL;
v_FTPEVENTID		LG_EVENT_FTP.FTPEVENTID%Type DEFAULT NULL;
v_ORIGINALFILENAME	LG_EVENT_FTP.ORIGINALFILENAME%Type DEFAULT NULL;
v_LASTMODIFIEDTIME	LG_EVENT_FTP.LASTMODIFIEDTIME%Type DEFAULT NULL;
v_ExceptionFlg		integer Default 0;
v_ARGS			VARCHAR2(4000) := P_ARGS;
v_RC1			db_ref.RCT;

v_ParamsTxt		varchar2(4000) Default
			'p_DebugFlg=>'||p_DebugFlg||CHR(10)
			||', p_EVENTID=>'||p_EVENTID||CHR(10)
			||', p_EVENTCODE=>'||p_EVENTCODE||CHR(10)
			||', p_EVENTTS=>'||p_EVENTTS||CHR(10)
			||', p_FAULTTYPE=>'||p_FAULTTYPE||CHR(10)
			||', p_SOURCECLASS=>'||p_SOURCECLASS||CHR(10)
			||', p_HOSTIPADDRESS=>'||p_HOSTIPADDRESS||CHR(10)
			||', p_FRIPADDRESS=>'||p_FRIPADDRESS||CHR(10)
			||', p_PARTNERID1=>'||p_PARTNERID1||CHR(10)
			||', p_PARTNERID2=>'||p_PARTNERID2||CHR(10)
			||', p_ARGS=>'||p_ARGS||CHR(10)		
			||', p_USERID=>'||p_USERID||CHR(10)
			||', P_BUSINESSID1=>'||P_BUSINESSID1||CHR(10)
			||', P_BUSINESSID2=>'||P_BUSINESSID2||CHR(10)
			||', p_TEXTID=>'||p_TEXTID||CHR(10)
			||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
			||', p_VUID=>'||p_VUID||CHR(10)
			||', p_USERNAME=>'||p_USERNAME||CHR(10)
			||', p_ORIGINALFILENAME=>'||p_ORIGINALFILENAME||CHR(10)
			||', p_LASTMODIFIEDTIME=>'||p_LASTMODIFIEDTIME||CHR(10)
			;
		
		
Procedure Check_length is
begin
   	DB_LENGTHCHECK_EVENT(
	v_SOURCECLASS
   	,v_HOSTIPADDRESS
   	,v_ARGS
   	);
end Check_length;		
		
		
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------


BEGIN
    	p_ReturnCd := 0;
    	Check_length;
    	v_PartnerId1 := coalesce(p_PartnerId1, FN_Get_PartnerId(p_BusinessId1) );
    	v_PartnerId2 := coalesce(p_PartnerId2, FN_Get_PartnerId(p_BusinessId2) );		
    
    
    
	Begin  -- LG_Event
    	
	    	IF LENGTH (TRIM (TRANSLATE (p_EVENTCODE, ' +-.0123456789', ' ') ) ) IS NULL THEN
		   v_EVENTCD := TO_NUMBER(p_EVENTCODE)  ;
		ELSE
		   select EVENTCD into v_EVENTCD from LG_EVENTCD where EVENTCODE = p_EVENTCODE;
		END IF ;
    	
		IF p_Vuid IS NOT NULL AND p_ActivityID IS NOT NULL and p_EventID IS NOT NULL THEN	
		    Insert into LG_EVENT(
		        EVENTID
		    ,    EVENTCD
		    ,    EVENTTS
		    ,    FAULTTYPE
		    ,    SOURCECLASS
		    ,    HOSTIPADDRESS
		    ,    PARTNERID1
		    ,    PARTNERID2
		    ,    ARGS
		    ,    USERID
		    ,    CREATEDATE
		    ,    TEXTID
		    ,    ACTIVITYID
		    ,    VUID
		    
		    ) Values (
		        p_EVENTID
		    ,    v_EVENTCD
		    ,    p_EVENTTS
		    ,    p_FAULTTYPE
		    ,    v_SOURCECLASS
		    ,    v_HOSTIPADDRESS
		    ,    v_PARTNERID1
		    ,    v_PARTNERID2
		    ,    v_ARGS
		    ,    p_USERID
		    ,    v_BeginTS
		    ,    p_TEXTID
		    ,    p_ACTIVITYID
		    ,    p_VUID
		    );
		ELSE
		    Insert into LG_EVENT(
		        EVENTID
		    ,    EVENTCD
		    ,    EVENTTS
		    ,    FAULTTYPE
		    ,    SOURCECLASS
        	    ,    HOSTIPADDRESS
   		    ,    PARTNERID1
   		    ,    PARTNERID2
   		    ,    ARGS
   		    ,    USERID
        	    ,    CREATEDATE
        	    ,    TEXTID
        	) Values (
        	        p_EVENTID
        	    ,    v_EVENTCD
        	    ,    p_EVENTTS
        	    ,    p_FAULTTYPE
        	    ,    v_SOURCECLASS
        	    ,    v_HOSTIPADDRESS
        	    ,    v_PARTNERID1
        	    ,    v_PARTNERID2
        	    ,    v_ARGS
        	    ,    p_USERID
        	    ,    v_BeginTS
        	    ,    p_TEXTID
                );
                END IF;

END;

	IF p_USERNAME IS NOT NULL THEN
    	   		
    		Select FTPUSERID INTO v_FTPUSERID from CF_FTP_USER c where c.USERNAME=p_USERNAME;
    	   	
    	END IF;
    
    
    	IF p_ORIGINALFILENAME IS NOT NULL AND p_LASTMODIFIEDTIME IS NOT NULL THEN
   		BEGIN
   		
   				Select L.FTPEVENTID INTO v_FTPEVENTID FROM LG_EVENT_FTP L 
    				WHERE L.ORIGINALFILENAME=p_ORIGINALFILENAME
				AND L.LASTMODIFIEDTIME=p_LASTMODIFIEDTIME;
				
				EXCEPTION
				When NO_DATA_FOUND THEN
					v_FTPEVENTID := NULL;
		END;

     		IF v_FTPEVENTID IS NOT NULL THEN
	
			LG_EVENT_FTP_XREFI
   				(
   				p_ReturnCd
   			     	,p_DebugFlg
   			     	,v_RC1 
   			     	,v_FTPEVENTID
   			     	,P_EVENTID
   			     	);
  		ELSE
			
  		        LG_EVENT_FTPI
   		    		(
   		     		p_ReturnCd
   				,p_DebugFlg
   				,v_RC1 
   				,P_EVENTID
   				,v_FTPUSERID
  	 			,p_ORIGINALFILENAME
  	 			,p_LASTMODIFIEDTIME
  	 			);
  	 	END IF; 
   		
   		p_ReturnCd := 0;
			v_RowCnt := sql%ROWCOUNT;
			p_ReturnCd := 0;
	
   	END IF;
   	
   	
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_EVENTI_AH3' || SQLCODE|| ' '||SQLERRM);
	
END LG_EVENTI_AH3;
/


-- LG_EVENTSUMMARYI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_EVENTSUMMARYI_AH1
(
      p_returnCd        out pls_integer
    , p_DebugFlg        in  pls_integer
    , p_EVENTID         in  VARCHAR2
    , p_EVENTTS         in  TIMESTAMP
    , p_EVENTCODE       in  VARCHAR2
    , p_FAULTTYPE       in  pls_integer
    , p_HOSTIPADDRESS   in  VARCHAR2
    , p_FRIPADDRESS     in  VARCHAR2
    , p_PARTNERID1      in  pls_integer
    , p_PARTNERID2      in  pls_integer
    , p_USERID          in  pls_integer
)
AS
/*
================================================================================
Proc Name:      LG_EVENTSUMMARYI_AH1
Create Date:    01/17/2002
Author          Vipin Goel
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
 v_EVENTCD           integer ;
 v_createDate        date := FN_GET_DATE();
 v_update_summary    boolean := False;
 v_ProcName          varchar2(30) := 'LG_EVENTSUMMARYI_AH1';
 v_beginTS           Timestamp(6):= FN_GET_DATE();
 v_ExceptionFlg      Integer :=0;
 v_RowCnt            Integer :=0;
 v_EVENTSUMMARYID    lg_eventSummary_xref.EVENTSUMMARYID%type;
 v_EventTS           Date := p_EventTS;                                 -- Convert TS to Date
 v_RowTS             integer :=0;
 zeroUpdate          Exception;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1(v_ProcName,v_beginTS,'p_returnCd'||','||p_DebugFlg||','||
    P_EVENTID ||','||
    to_char(P_EventTS,'dd-mon-yy hh24:mi:ss')||','||
    p_EVENTCODE ||','||
    P_FAULTTYPE ||','||
    P_HOSTIPADDRESS ||','||
    P_FRIPADDRESS||','||
    P_PARTNERID1||','||
    P_PARTNERID2 || ','||
    P_USERID
    );
End ProcAudit;
--
PROCEDURE Update_LG_EVENTSUMMARY AS
BEGIN
    Update LG_EVENTSUMMARY S
      Set
          s.EventCnt = s.EventCnt + 1
        , s.UpdateDate = v_CreateDate
      Where s.EventSummaryId = v_EVENTSUMMARYID  and  s.DayHour = trunc(v_EventTs,'HH24');
      v_RowCnt := SQL%rowcount;
--      if (v_RowCnt = 0) then  raise ZeroUpdate; end if;

--       ProcAudit;
    EXCEPTION
--            WHEN ZeroUpdate then
--                p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' Insert_LG_EVENTSUMMARY '||SQLERRM);
END Update_LG_EVENTSUMMARY;
---
Procedure Get_summaryID is
Begin
    select
        EVENTsummaryId into v_EVENTsummaryId
    from
        LG_EVENTSUMMARY_XREF
    where EVENTCD       = v_EVENTCD
        and   FAULTTYPE     = p_FAULTTYPE
        and   HOSTIPADDRESS = p_HOSTIPADDRESS
        and   FRIPADDRESS   = p_FRIPADDRESS
        and   PARTNERID1    = NVL(p_PARTNERID1,-1)
        and   PARTNERID2    = NVL(p_PARTNERID2,-1)
        and   USERID        = NVL(p_USERID,-1);

    Exception
        when NO_DATA_FOUND then
            p_returnCd :=3;
            raise_application_error(-20999,20005||v_ProcName||' - Get_SummaryID '||SQLERRM);
        when others then
            p_returnCd := 1;
            raise_application_error(-20999,20006||v_ProcName||' - Get_SummaryID '||SQLERRM);
END Get_summaryID;
------
procedure Insert_lg_eventSummary_xref as
Begin
 Insert into LG_EVENTSUMMARY_XREF
   (EVENTSUMMARYID
   , EVENTCD
   , FAULTTYPE
   , HOSTIPADDRESS
   , FRIPADDRESS
   , PARTNERID1
   , PARTNERID2
   , CREATEDATE
   , USERID)
values
(lg_eventSummary_xref_SEQ.nextval
   , v_EVENTCD
   , p_FAULTTYPE
   , p_HOSTIPADDRESS
   , p_FRIPADDRESS
   , nvl(p_PARTNERID1,-1)
   , nvl(p_PARTNERID2,-1)
   , v_createDate
   , NVL(p_USERID,-1)) returning EVENTSUMMARYID into v_EVENTSUMMARYID;

    Exception
    when dup_val_on_index   then
        Get_summaryID;
    When Others Then
        p_returnCd := 1;
        raise_application_error(-20999,20007||v_ProcName||' -  Insert_LG_EventSUMMARY_XREF '||SQLERRM);
End Insert_lg_eventSummary_xref;
-----
procedure insert_LG_EVENT_EVENTSUMMARY as
Begin
 Insert into LG_EVENT_EVENTSUMMARY (EventSUMMARYID, DAYHOUR, EventID, CREATEDATE)
    values( v_EventSUMMARYID,trunc(v_EventTS,'HH24'), P_EventID, v_createdate);
 exception
  when dup_val_on_index   then  null;
  When Others then
    raise_application_error(-20999,20009||v_ProcName||' -  Insert_LG_EVENT_EVENTSUMMARY '||SQLERRM);
End insert_LG_EVENT_EVENTSUMMARY;
---------------------------------
-- Main
---------------------------------
BEGIN
    p_returnCd := 0;
    IF LENGTH (TRIM (TRANSLATE (p_EVENTCODE, ' +-.0123456789', ' ') ) ) IS NULL THEN
	   v_EVENTCD := TO_NUMBER(p_EVENTCODE)  ;
	ELSE
	   select EVENTCD into v_EVENTCD from LG_EVENTCD where EVENTCODE = p_EVENTCODE 
	   ;
	END IF ;	
    Insert_lg_eventSummary_xref;
    Update_LG_EVENTSUMMARY;

    if (v_RowCnt = 0) then
        INSERT INTO LG_EVENTSUMMARY
        (
         EVENTSUMMARYID
        ,DAYHOUR
        ,EVENTCNT
        ,CREATEDATE
        ,UPDATEDATE
        )
       VALUES
        (
         v_EVENTSUMMARYID
        ,trunc(v_EventTs,'HH24')
        ,1
        ,v_CreateDate
        ,v_CreateDate
       );
     end if;
 Insert_LG_EVENT_EVENTSUMMARY;
 ProcAudit;
     EXCEPTION
        When Dup_val_on_index Then
           Update_LG_EVENTSUMMARY;
        When Others Then
           p_ReturnCd := 1;
           ProcAudit;
           Raise_application_error (-20999,'20001  LG_EVENTSUMMARYI_AH1 '||SQLERRM);
END;
/


-- LG_EVENTSUMMARYS_AH2.sql

CREATE OR REPLACE PROCEDURE LG_EVENTSUMMARYS_AH2 (
      p_returnCd        out pls_integer
    , p_DebugFlg        in  pls_integer
    , RC1               out db_ref.rct          -- rows
    , p_STARTDATE       in  TIMESTAMP                 --    (Date and Time)
    , p_ENDDATE         in  TIMESTAMP                  --    (Date and Time)
    , p_PartnerIdList   in  varchar2
    , p_EventCodeList   in  varchar2
    , p_Severity        in  pls_Integer             --    table LG_EVENTCD
    , p_Threshold       in  pls_Integer
    , p_VISIBILITY      in  varchar2      -- List (comma seperated) 1=  3=  7=
    , p_partner_VISIBILITY in pls_integer
    )
AS
/*
================================================================================
Proc Name:      LG_EventSUMMARYs_AH2
Create Date:    09/05/2002
Author          J.Chen
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(2000);
  v_startdate        varchar2(20) := to_char(trunc(nvl(p_STARTDATE,current_timeStamp-1), 'HH24'), 'dd-mm-yy hh24:mi:ss');
  v_enddate          varchar2(20) := to_char(trunc(nvl(p_EndDATE,current_timeStamp)     ,'HH24'), 'dd-mm-yy hh24:mi:ss');
  v_Threshold        pls_integer;
  v_errnum           varchar2(500);
  v_errmsg           varchar2(500);
  sqlstmt            varchar2(4000);
  sql_Column         varchar2(1000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
--
-- Procedures
--
-----------
--ProcAudit
-----------
Procedure ProcAudit is
v_diffTS Timestamp;
begin
  DB_ProcAuditLogI_Ah1('LG_EVENTSUMMARYS_AH2',v_beginTS,'p_returnCd'||','||p_DebugFlg||
    ','||'RC1'||','||
    To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
    p_SEVERITY    ||','||
    p_Threshold   ||','||
    p_VISIBILITY  ||','||
    p_partner_VISIBILITY
    );

End ProcAudit;
----------------
-- Where_clause
----------------
Procedure where_clause as
begin
    v_where := ' where  s.dayHour between to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and to_date('''||
                          v_endDate||''',''dd-mm-yy hh24:mi:ss'')'||
  FN_Build_Where('e.SEVERITY',    'NUMBER'     , p_SEVERITY)||
  FN_Build_Where('e.EVENTCD',     'NUMBER_LIST', p_EventCodeList)||
  FN_Build_Where('e.VISIBILITY',  'NUMBER_LIST', p_VISIBILITY);
end where_clause;
--------------------------------------------------------------------
--                              M a i n
--------------------------------------------------------------------
BEGIN
    p_returnCd := 0;

    where_clause;

    IF p_PartnerIdList IS NOT NULL THEN
       v_where := v_where || ' and (x.PartnerId1 in '|| p_PartnerIdList ||
                             ' or x.PartnerId2 in '|| p_PartnerIdList ||')';
    END IF;
    IF p_partner_VISIBILITY = 1 then
       v_where := v_where || ' and pp1.visibility = 1 ' ; --(pp1.visibility = 1 or pp2.visibility = 1)
    END IF;
--  dbms_output.put_line(substr(v_where,1,250));
--  dbms_output.put_line(substr(v_where,251,250));
    sql_column :=
    'select e.EventCd AS EventCode,
            e.EventNameKey,
            e.EventName,
            e.SEVERITY,
            sum(eventCNT) AS EventCount, e.eventnameresourcebundleid ';
    sql_inner :=
    'from lg_Eventsummary_xref x
    inner join  lg_Eventsummary s  on x.EventsummaryID     = s.EventsummaryID
    inner join lg_eventcd   e      on x.eventcd  = e.eventcd
    inner join pr_partner   p1     on x.partnerid1 = p1.partnerid
    Inner Join pr_partnerType pp1  on p1.PARTNERTYPEID = pp1.PARTNERTYPEID';
--    inner join pr_partner   p2     on x.partnerid2 = p2.partnerid
--    Inner Join pr_partnerType pp2  on p2.PARTNERTYPEID = pp2.PARTNERTYPEID

    If p_Threshold IS NOT NULL THEN
       v_Threshold := p_Threshold + 1;
       sqlstmt := 'select * from ('||sql_column||' '||sql_inner||' '||v_where||
                   ' group by  e.EventCd, e.EventNameKey, e.EventName, e.Severity, e.eventnameresourcebundleid ' ||
                   ' order by e.Severity, EventCount DESC ) Where rownum < '|| v_Threshold;
    ELSE
       sqlstmt := sql_column||' '||sql_inner||' '||v_where||
                ' group by  e.EventCd, e.EventNameKey, e.EventName, e.Severity, e.eventnameresourcebundleid ' ||
                ' order by e.Severity, EventCount DESC';
    END IF;
   DB_Print('Sql for rc1 ', sqlstmt);
-- Open Cursor
    if RC1%IsOpen then close RC1; end if;
    open RC1 for sqlstmt;
-- proc Audit
  ProcAudit;
Exception
 when others then
  ProcAudit;
  --dbms_output.put_line('error occurred in lg_EventSummaryS_Ah2:'||SQLERRM);
  raise_application_error(-20999,20002||'-'||'Error occurred in lg_EventSummaryS_Ah2:'||SQLERRM);
  p_returnCd := 1;
END ;
/


-- LG_EVENTS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_EVENTS_AH1
(
      p_ReturnCd      in out pls_INTEGER
    , p_DebugFlg                in       pls_INTEGER
    , RC1               in  out  db_ref.rct          -- rows
    , RC2               in  out  db_ref.rct_count          -- count
    , p_sortkey         in  varchar2
    , p_PageSizeNbr        in    pls_Integer
    , p_PageNbr            in  pls_Integer
    , p_PARTNERID       in    varchar2                --    (All = NULL, One, Comma Seperated list of Id's) for partner1 or partner2
    , p_STARTDATE        in    DATE                    --    (Date and Time) lg_event.EventTS between p_startdate and p_enddate
    , p_ENDDATE            in    DATE                    --    (Date and Time)
    , p_SEVERITY        in  pls_Integer  -- debug = 0, info = 1, warning = 3, critical = 9 (future types later)
    , p_faultType        in  pls_integer  --unknown = 0, source = 1, target = 2, system = 3
    , p_HOSTIPADDRESS   in  varchar2
    , p_Eventcode       in  pls_Integer
    , p_VISIBILITY      in  varchar2      -- 1=  3=  7=
    , p_EventId         in  varchar2
    , p_initpartnertype   in varchar2     -- For ALL Internal Partners 
    )
AS
/*
================================================================================
Proc Name:      LG_EventS_AH1
Create Date:    01/04/2002
Author          Vipin Goel
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME          REL      DESCRIPTION
---------  --------------------  ---------------------------------------------
01/04/2002   Vipin                Created
01/29/2002   Vipin                took out ref to lg_class and lg_eventcd_class tables
01/30/2002   Vipin                The p_startDate and p_endDate is compared to Lg_Event.EventTS
01/30/2002   Vipin                Add visibility to input parameter list
02/26/2002   Vipin                Modified sqlstmt for tuning
03/20/2002   Vipin       3.1      Removed week cursor and added srtong cursor
03/26/02     Vipin       3.1      Fixed sort problem
2002/05/29   Vipin       3.2      added logic in where clause for p_partnerid to be either partnerid1 or 2
2002-07-09   J.Chen      3.3      Added UserId and UserName to the result set
2002-07-24   Vipin       3.3      For P-severity it should be >= (currently it is  =)
2002-07-25   Vipin       3.3      Added hints to sql statements and use rowid instead of eventid for open rc1
2002/08/02   Vipin       3.2      Partnerid logic in where clause is done twice, first one taken out Defect #1-U5C9
2002-09-16   J.Chen      3.3      Changed inner join on userid to left outer join and added userid to sql_outer
                                  Partnerid 1 and 2 should be using left outer join
2002-12-16   J.Chen      3.3.2    Added p_eventId as input   -- may use wildcard for input values
2003-05-14   Vipin       3.5      Remove reference to db_ref.rct_event  (Strong cursor)
2003-06-20   V Goel      3.5      Use DB_Paging_Logic and DB_Build_WhereClause
2003-07-07   V Goel      3.5      Use FN_Paging_Logic and FN_Build_Where Functions
2007-28-25   Bharani J   6.1.1    For Multiple Internal Partner Feature
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  v_where            varchar2(2000);
--  v2_where           varchar2(2000);
  sql_Column         varchar2(1000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  v_startdate        varchar2(20) := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
  v_enddate          varchar2(20) := to_char(nvl(p_EndDATE,sysdate)    , 'dd-mm-yy hh24:mi:ss');
  StartRow           pls_integer;
  EndRow             pls_integer;
  v_count            Number;
  i                  pls_integer;
  v_min_rowid        varchar2(18);
  v_max_rowid        varchar2(18);
  v_min_eventid     varchar2(65);
  v_max_eventid     varchar2(65);
 
  v_PARTNERS_INT	varchar(500);
  v_PARTNERS_LIST	varchar(500);
------- Type
Type ar_eventid       is table of varchar2(65);
Type ar_rnbr          is table of number(10);
Type ar_rowid         is table of varchar2(18);
array_eventid        ar_eventid       := ar_eventid();
array_rnbr           ar_rnbr        := ar_rnbr();
array_rowid          ar_rowid       := ar_rowid();
--
eventTable char65_Table := char65_Table();
eventtable2 ROWID_TABLE := ROWID_TABLE();
-------
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
-- If debug =1  Then
  DB_ProcAuditLogI_AH1('LG_EVENTs_Ah1',v_beginTS,'RETURN_CODE'||','||p_DebugFlg||
    ','||'RC1'||','||'RC2'||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_PARTNERID    ||','||
    To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
    p_SEVERITY     ||','||
    p_faultType    ||','||
    p_HOSTIPADDRESS||','||
    p_Eventcode    ||','||
    p_VISIBILITY   ||','||
    p_eventId
    ,0,0);
--commit;
--End if ;
End ProcAudit;
Procedure where_clause as
begin

    
    v_where :=
    FN_Build_Where('LG_EVENT.FAULTTYPE',     'NUMBER'     , p_FAULTTYPE)||
    FN_Build_Where('LG_EVENT.HOSTIPADDRESS', 'VARCHAR2'   , p_HOSTIPADDRESS)||
    FN_Build_Where('LG_EVENTCD.Eventcd',     'NUMBER'     , p_Eventcode)||
    FN_Build_Where('LG_EVENTCD.VISIBILITY',  'NUMBER_LIST', p_VISIBILITY)||
    FN_Build_Where('LG_EVENT.EVENTID','VARCHAR2_LIKE', p_eventid);
    
    
    If p_SEVERITY is NOT null then
    v_where := v_where||' and LG_EVENTCD.SEVERITY >= '||p_SEVERITY;
    End if;
    
    
    v_Partners_List := ' (lg_event.partnerID1 in '|| p_partnerid ||
   				 ' or  lg_event.partnerID2 in ' || p_partnerid ||')';
   	IF upper(p_InitPartnerType)='INTERNAL PARTNERS' then	
		v_Partners_Int := '(SELECT partnerID FROM PR_PARTNER_HUBOWNER_VW)';  												
		v_where := v_where || ' AND ( ' || 'lg_event.partnerID1 IN ' || v_Partners_Int
   					|| ' OR lg_event.partnerID2 IN ' || v_Partners_Int;
   		IF p_PARTNERID is not null then
   			 v_where := v_where ||' OR ' || v_Partners_List;
   		END IF;
   		v_where := v_where || ' ) ';
   	ELSE
   		IF p_PARTNERID is not null then
   			 v_where := v_where ||' and ' || v_Partners_List;
   		end if;
	end if;
    
    

end where_clause;
--
-- Main
--
BEGIN
    p_ReturnCd:= 0;
--    v_where := v_where||' Where  LG_EVENT.EVENTTS between '||chr(10)||' to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and'||
--                                                     chr(10)||' to_date('''|| v_endDate||''',''dd-mm-yy hh24:mi:ss'')';
    where_clause;
--    dbms_output.put_line(substr(v_where,1,250));
--    dbms_output.put_line(substr(v_where,251,250));
sql_inner :=
        ' from    LG_EVENT
        inner Join LG_EVENTCD    on LG_EVENT.EVENTCD         = LG_EVENTCD.EVENTCD
        ';
sql_outer :=
       'Left outer Join PR_PARTNER p1 on LG_EVENT.PARTNERID1      = p1.PARTNERID
        Left outer join PR_PARTNER p2 on LG_EVENT.PARTNERID2      = p2.PARTNERID
        left outer join PR_USER U     on LG_EVENT.USERID     = U.USERID';       
-- total row count Cursor
    if RC2%IsOpen then close RC2; end if;
    if NVL(p_pageNbr,0) in (0,1) then
--     Execute Immediate  'select /*+ index(LG_EVENT AK_LG_EVENT_2)*/ count(*)  '|| sql_inner||' '||v_where INTO v_count ; 
    Execute Immediate  'select /*+ index(LG_EVENT AK_LG_EVENT_2)*/ count(*)  '|| sql_inner||' '||
    ' Where  LG_EVENT.EVENTTS between :startDate and  :enddate'||' '||v_where INTO v_count using P_STARTDATE, P_ENDDATE;
    open RC2 for select v_count TotalRowsNbr from dual;
     end if;
-- Sort Logic
--    sqlstmt2 := 'select /*+ index(LG_EVENT AK_LG_EVENT_2)*/ eventid '||sql_inner||' '||sql_outer||' '||v_where;
    sqlstmt2 := 'select /*+ index(LG_EVENT AK_LG_EVENT_2)*/ eventid, LG_EVENT.rowid row_id '||sql_inner||' '||sql_outer||' '||
               ' Where  LG_EVENT.EVENTTS between :startDate and  :enddate'||' '||v_where;
-- , LG_EVENT.rowid
    if p_sortkey is not null then
       sqlstmt2 := sqlstmt2 ||' order by '||p_sortkey;
    end if;
-- Paging logic
    sqlstmt2 := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt2);
           DB_Print('Sql ', sqlstmt2);
-- get eventid for number of rows = p_pageSizeNbr
    Begin
        execute immediate sqlstmt2 Bulk Collect into array_rnbr, array_eventid, array_rowid using P_STARTDATE, P_ENDDATE;
    Exception
        When Others then
            Raise_application_error (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
    End;
-- migrate eventcd to eventTable
    For i IN 1..array_eventid.count LOOP
      eventTable2.extend;
      eventTable2(i) := rowid_rec (array_eventid(i), array_rowid(i), array_rnbr(i));
      dbms_output.put_line(' eventid '||array_eventid(i)||' Rowid '||array_rowid(i));
    End Loop;
--  select min(Y.row_id),max(Y.row_id) into v_min_rowid, v_max_rowid from TABLE(cast(eventTable2 as rowid_table )) Y;
 select min(Y.id),max(Y.id) into v_min_eventid, v_max_eventid from TABLE(cast(eventTable2 as rowid_table )) Y;
  dbms_output.put_line('Min '||v_min_eventid||' Max '|| v_max_eventid);
-- Open Cursor
--        /*+ Rowid (E)*/
--        /*+ index(E PK_EVENT) */
    if RC1%IsOpen then close RC1; end if;
        OPEN rc1 for select
       /*+ index(E PK_EVENT) */
          E.EVENTID
        , E.EVENTCD
        , E.EVENTTS
        , E.FAULTTYPE
        , E.HOSTIPADDRESS
        , E.SOURCECLASS
        , E.PARTNERID1
        , P1.PARTNERNAME      PARTNERNAME1
        , E.PARTNERID2
        , P2.PARTNERNAME      PARTNERNAME2
        , LG_EVENTCD.EVENTNAME
        , LG_EVENTCD.EVENTNAMEKEY
        , LG_EVENTCD.INTERNALDESC
        ,CASE WHEN E.TEXTID IS NOT NULL THEN RTRIM(E.TEXTID) ELSE RTRIM(LG_EVENTCD.TEXTKEY) END AS TEXTKEY
        , LG_EVENTCD.VISIBILITY
        , LG_EVENTCD.SEVERITY
        , LG_EVENTCD.ALERTABLEFLG
        , E.ARGS
        , E.USERID
        , U.USERNAME
		, LG_EVENTCD.EVENTCODE
		, LG_EVENTCD.EVENTNAMERESOURCEBUNDLEID
		, LG_EVENTCD.TEXTKEYRESOURCEBUNDLEID
		, LG_EVENTCD.EXTERNALDESCRESOURCEBUNDLEID		
         from   LG_EVENT  E
         Inner join  TABLE(cast(eventTable2 as rowid_table )) Y  on E.eventid = Y.id
         Inner Join LG_EVENTCD         on E.EVENTCD         = LG_EVENTCD.EVENTCD
         Left outer join PR_PARTNER p1 on E.PARTNERID1      = p1.PARTNERID
         Left outer join PR_PARTNER p2 on E.PARTNERID2      = p2.PARTNERID
         Left outer join PR_USER U     on E.USERID          = U.USERID
         where E.eventid between v_min_eventid and v_max_eventid
         order by Y.seq;

  ProcAudit;
Exception
 when others then
  ProcAudit;
  p_ReturnCd:= 1;
END ;
/


-- LG_EVENTS_AH2.sql

CREATE OR REPLACE PROCEDURE LG_EVENTS_AH2
(
   p_ReturnCd       in out pls_INTEGER
   , p_DebugFlg        in    pls_INTEGER
   , RC1               in  out  db_ref.rct          -- rows
   , RC2               in  out  db_ref.rct_count          -- count
   , p_sortkey         in  varchar2
   , p_PageSizeNbr        in    pls_Integer
   , p_PageNbr            in  pls_Integer
   , p_PARTNERID       in    varchar2                --    (All = NULL, One, Comma Seperated list of Id's) for partner1 or partner2
   , p_STARTDATE        in    DATE                    --    (Date and Time) lg_event.EventTS between p_startdate and p_enddate
   , p_ENDDATE            in    DATE                    --    (Date and Time)
   , p_SEVERITY        in  pls_Integer  -- p_DebugFlg = 0, info = 1, warning = 3, critical = 9 (future types later)
   , p_faultType        in  pls_integer  --unknown = 0, source = 1, target = 2, system = 3
   , p_HOSTIPADDRESS   in  varchar2
   , p_Eventcode       in  pls_Integer
   , p_VISIBILITY      in  varchar2      -- 1=  3=  7=
   , p_EventName       in  varchar2
)
AS
/*
================================================================================
Proc Name:      LG_EventS_AH2
Create Date:    01/04/2002
Author          Vipin Goel
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME          REL      DESCRIPTION
---------  --------------------  ---------------------------------------------
01/04/2002   Vipin                Created
01/29/2002   Vipin                took out ref to lg_class and lg_eventcd_class tables
01/30/2002   Vipin                The p_startDate and p_endDate is compared to Lg_Event.EventTS
01/30/2002   Vipin                Add visibility to input parameter list
02/26/2002   Vipin                Modified sqlstmt for tuning
03/20/2002   Vipin       3.1      Removed week cursor and added srtong cursor
03/26/02     Vipin       3.1      Fixed sort problem
2002/05/29   Vipin       3.2      added logic in where clause for p_partnerid to be either partnerid1 or 2
2002-07-09   J.Chen      3.3      Added UserId and UserName to the result set
2002-07-24   Vipin       3.3      For P-severity it should be >= (currently it is  =)
2002-07-25   Vipin       3.3      Added hints to sql statements and use rowid instead of eventid for open rc1
2002/08/02   Vipin       3.2      Partnerid logic in where clause is done twice, first one taken out Defect #1-U5C9
2002-09-16   J.Chen      3.3      Added eventname as input and changed inner join on userid to left outer
                                  Changed left outer to inner join in sqlstmt2 to match rc1 query
2003-05-14   Vipin       3.5      Remove reference to db_ref.rct_event  (Strong cursor)
2003-06-20   V Goel      3.5      Use DB_Paging_Logic and DB_Build_WhereClause
2003-07-07   V Goel      3.5      Use FN_Paging_Logic and FN_Build_Where Functions
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  v_where            varchar2(2000);
  sql_Column         varchar2(1000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  v_startdate        varchar2(20) := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
  v_enddate          varchar2(20) := to_char(nvl(p_EndDATE,sysdate)    , 'dd-mm-yy hh24:mi:ss');
  StartRow           pls_integer;
  EndRow             pls_integer;
  v_count            Number;
  i                  pls_integer;
------- Type
Type ar_eventid       is table of varchar2(65);
Type ar_rnbr          is table of number(10);
Type ar_rowid         is table of varchar2(18);
array_eventid        ar_eventid       := ar_eventid();
array_rnbr           ar_rnbr        := ar_rnbr();
array_rowid          ar_rowid       := ar_rowid();
--
eventTable char65_Table := char65_Table();
eventtable2 ROWID_TABLE := ROWID_TABLE();
-------
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
-- If p_DebugFlg =1  Then
  DB_ProcAuditLogI_AH1('LG_EVENTs_Ah2',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'||','||'RC2'||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_PARTNERID    ||','||
    To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
    p_SEVERITY     ||','||
    p_faultType    ||','||
    p_HOSTIPADDRESS||','||
    p_Eventcode    ||','||
    p_VISIBILITY   ||','||
    p_EventName
    ,0,0);
--commit;
--End if ;
End ProcAudit;
Procedure where_clause as
begin
    v_where :=
    FN_Build_Where('LG_EVENT.FAULTTYPE',     'NUMBER'     , p_FAULTTYPE)    ||
    FN_Build_Where('LG_EVENT.HOSTIPADDRESS', 'VARCHAR2'   , p_HOSTIPADDRESS)||
    FN_Build_Where('LG_EVENTCD.Eventcd',     'NUMBER'     , p_Eventcode)    ||
    FN_Build_Where('LG_EVENTCD.VISIBILITY',  'NUMBER_LIST', p_VISIBILITY)   ||
    FN_Build_Where('LG_EVENTCD.EVENTNAME',   'VARCHAR2_LIKE', p_EventName);
--
    If p_SEVERITY IS NOT NULL THEN
    v_where := v_where||' and LG_EVENTCD.SEVERITY >= '||p_SEVERITY;
    END IF;
    IF p_PARTNERID IS NOT NULL THEN
     v_where := v_where||' and (lg_event.partnerID1 in '||p_partnerid ||' or  lg_event.partnerID2 in '||p_partnerid||')';
    END IF;

--  If p_PARTNERID is NOT null then
--    v_where := v_where||' and LG_EVENT.PARTNERID1 in '||p_PARTNERID ||' or LG_EVENT.PARTNERID2 in '||p_PARTNERID;
--  End if;
end where_clause;
--
-- Main
--
BEGIN
    p_ReturnCd := 0;
--    v_where := v_where||' Where  LG_EVENT.EVENTTS between '||chr(10)||' to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and'||
--                                                     chr(10)||' to_date('''|| v_endDate||''',''dd-mm-yy hh24:mi:ss'')';
    where_clause;
--    dbms_output.put_line(substr(v_where,1,250));
--    dbms_output.put_line(substr(v_where,251,250));
sql_inner :=
        ' from    LG_EVENT
        inner Join LG_EVENTCD    on LG_EVENT.EVENTCD         = LG_EVENTCD.EVENTCD
        inner join PR_PARTNER p1 on LG_EVENT.PARTNERID1      = p1.PARTNERID
        inner join PR_PARTNER p2 on LG_EVENT.PARTNERID2      = p2.PARTNERID';
sql_outer :=
        ' left outer join PR_USER U     on LG_EVENT.USERID     = U.USERID ';

-- total row count Cursor
    if RC2%IsOpen then close RC2; end if;
    if NVL(p_pageNbr,0) in (0,1) then
--     Execute Immediate  'select /*+ index(LG_EVENT AK_LG_EVENT_2)*/ count(*)  '|| sql_inner||' '||v_where INTO v_count ;
    Execute Immediate  'select /*+ index(LG_EVENT AK_LG_EVENT_2)*/ count(*)  '|| sql_inner||' '||
    ' Where  LG_EVENT.EVENTTS between :startDate and  :enddate'||' '||v_where INTO v_count using P_STARTDATE, P_ENDDATE;
    open RC2 for select v_count TotalRowsNbr from dual;
     end if;
-- Sort Logic
--    sqlstmt2 := 'select /*+ index(LG_EVENT AK_LG_EVENT_2)*/ eventid '||sql_inner||' '||sql_outer||' '||v_where;
    sqlstmt2 := 'select /*+ index(LG_EVENT AK_LG_EVENT_2)*/ eventid, LG_EVENT.rowid row_id '||sql_inner||' '||sql_outer||' '||
               ' Where  LG_EVENT.EVENTTS between :startDate and  :enddate'||' '||v_where;
-- , LG_EVENT.rowid
    if p_sortkey is not null then
       sqlstmt2 := sqlstmt2 ||' order by '||p_sortkey;
    end if;
-- Paging logic
    sqlstmt2 := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt2);
           DB_Print('Sql ', sqlstmt2);
-- get eventid for number of rows = p_pageSizeNbr
    Begin
        execute immediate sqlstmt2 Bulk Collect into array_rnbr, array_eventid, array_rowid using P_STARTDATE, P_ENDDATE;
    Exception
        When Others then
            Raise_application_error (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
    End;
-- migrate eventcd to eventTable
/*
    For i IN 1..array_eventid.count LOOP
     eventTable.extend;
     eventTable(i) := char65_rec (array_rnbr(i),array_eventid(i));
--     lg_event_rec2(array_eventid(i));
--     dbms_output.put_line(' eventid '||array_eventid(i)||' Rowid '||array_rowid(i));
    End Loop;
*/
    For i IN 1..array_eventid.count LOOP
      eventTable2.extend;
      eventTable2(i) := rowid_rec (array_eventid(i), array_rowid(i), array_rnbr(i));
--      dbms_output.put_line(' eventid '||array_eventid(i)||' Rowid '||array_rowid(i));
    End Loop;
-- Open Cursor
    if RC1%IsOpen then close RC1; end if;
        OPEN rc1 for select /*+ index(E PK_EVENT) */
        E.EVENTID
        , E.EVENTCD
        , E.EVENTTS
        , E.FAULTTYPE
        , E.HOSTIPADDRESS
        , E.SOURCECLASS
        , E.PARTNERID1
        , P1.PARTNERNAME      PARTNERNAME1
        , E.PARTNERID2
        , P2.PARTNERNAME      PARTNERNAME2
        , LG_EVENTCD.EVENTNAME
        , LG_EVENTCD.EVENTNAMEKEY
        , LG_EVENTCD.INTERNALDESC
        --, LG_EVENTCD.TEXTKEY
        , CASE WHEN E.TEXTID IS NOT NULL THEN RTRIM(E.TEXTID) ELSE RTRIM(LG_EVENTCD.TEXTKEY) END AS TEXTKEY
        , LG_EVENTCD.VISIBILITY
        , LG_EVENTCD.SEVERITY
        , LG_EVENTCD.ALERTABLEFLG
        , E.ARGS
        , E.USERID
        , U.USERNAME
		, LG_EVENTCD.EVENTCODE
		, LG_EVENTCD.EVENTNAMERESOURCEBUNDLEID
		, LG_EVENTCD.TEXTKEYRESOURCEBUNDLEID
		, LG_EVENTCD.EXTERNALDESCRESOURCEBUNDLEID 		
         from   LG_EVENT  E
         Inner join  TABLE(cast(eventTable2 as rowid_table )) Y  on E.rowid = Y.row_id
         Inner Join LG_EVENTCD         on E.EVENTCD         = LG_EVENTCD.EVENTCD
         Inner join PR_PARTNER p1 on E.PARTNERID1      = p1.PARTNERID
         Inner join PR_PARTNER p2 on E.PARTNERID2      = p2.PARTNERID
         Left outer join PR_USER U     on E.USERID          = U.USERID
         order by Y.seq;

  ProcAudit;
Exception
 when others then
  ProcAudit;
  p_ReturnCd := 1;
END ;
/


-- LG_EVENTS_AH3.sql

CREATE OR REPLACE PROCEDURE LG_EVENTS_AH3
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        , p_EVENTCD IN pls_integer
        , p_EventTS   IN TimeStamp
        )
AS

/*
        ================================================================================
        Proc Name:       LG_EVENTS
        Create Date:     2003-07-21 16:41:51
        Author:          V Goel

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENTS_AH3';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_EVENTCD=>'||p_EVENTCD||CHR(10)
                ||', p_EVENTTS=>'||p_EVENTTS||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        IF RC1%ISOPEN THEN CLOSE RC1; END IF;
        OPEN RC1 FOR
        SELECT
                EVENTTS
        FROM
                LG_EVENT
        Where
                EVENTCD = p_EVENTCD
        And     EventTS > p_EventTS
        ORDER BY EVENTTS DESC;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_EVENTS_AH3;
/


-- LG_EVENTS_EVENTID.sql

CREATE OR REPLACE PROCEDURE LG_EVENTS_EVENTID
(
   p_ReturnCd in out integer
   ,p_DebugFlg in pls_integer
   ,RC1 in out db_ref.RCT
   ,p_EVENTID in VARCHAR2
)
AS
/*
================================================================================
Proc Name:      LG_EVENTs_eventid
Create Date:    01/10/02 02:30:04 PM
Author          SYSTEM
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_EVENTs_eventid',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg||','||'RC1'
    ||','||p_EVENTID);
End ProcAudit;
BEGIN
Open RC1 for select
          LG_EVENT.EVENTID
        , LG_EVENT.EVENTCD
        , LG_EVENT.EVENTTS
        , LG_EVENT.FAULTTYPE
        , LG_EVENT.HOSTIPADDRESS
        , LG_EVENT.SOURCECLASS
        , LG_EVENT.PARTNERID1
        , P1.PARTNERNAME PARTNERNAME1
        , LG_EVENT.PARTNERID2
        , P2.PARTNERNAME PARTNERNAME2
        , LG_EVENTCD.EVENTNAME
        , LG_EVENTCD.EVENTNAMEKEY
        , LG_EVENTCD.INTERNALDESC
        , CASE WHEN LG_EVENT.TEXTID IS NOT NULL THEN RTRIM(LG_EVENT.TEXTID) ELSE RTRIM(LG_EVENTCD.TEXTKEY) END AS TEXTKEY
        , LG_EVENTCD.VISIBILITY
        , LG_EVENTCD.SEVERITY
        , LG_EVENTCD.ALERTABLEFLG
        , LG_EVENT.ARGS
        , LG_EVENT.USERID
        , u.USERNAME
		, LG_EVENTCD.EVENTCODE
		, LG_EVENTCD.EVENTNAMERESOURCEBUNDLEID
		, LG_EVENTCD.TEXTKEYRESOURCEBUNDLEID
		, LG_EVENTCD.EXTERNALDESCRESOURCEBUNDLEID
from    LG_EVENT
        Left outer join PR_PARTNER p1 on LG_EVENT.PARTNERID1      = p1.PARTNERID
        Left outer join PR_PARTNER p2 on LG_EVENT.PARTNERID2      = p2.PARTNERID
        Inner Join LG_EVENTCD         on LG_EVENT.EVENTCD         = LG_EVENTCD.EVENTCD
        Left outer join pr_user u on LG_EVENT.USERID = u.userId
Where   LG_EVENT.EVENTID = p_eventid ;
p_ReturnCd := 0;
procAudit;
Exception
    When NO_DATA_FOUND Then Null;
    When Others Then
    ProcAudit;
    p_ReturnCd := 1;
    Raise_application_error (-20999,'20000 - LG_EVENTs_eventid' || SQLCODE|| ' '||SQLERRM);
End LG_EVENTS_EVENTID;
/


-- LG_EVENTS_VUID.sql

CREATE OR REPLACE PROCEDURE LG_EVENTS_VUID
(
      p_ReturnCd       out pls_INTEGER
    , p_DebugFlg        in  pls_INTEGER
    , RC1               out  db_ref.rct          -- rows
    , RC2               out  db_ref.rct          -- count
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_VUID            in  varchar2
    , p_ActivityID      in  varchar2
    , p_VISIBILITY      in  varchar2      -- 1=hub Owner  2=sponcer  4= trading partner
)
AS
/*
================================================================================
Proc Name:      LG_EventS_VUID
Create Date:    01/04/2002
Author          Vipin Goel
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  -----------------  ---------------------------------------------
DATE       NAME         VER     DESCRIPTION
---------  -----------------  ---------------------------------------------
01/04/02   V. Goel            Created
01/25/02   V. Goel            add activityId to parameter and logic
01/30/02   V. Goel            Remove ref to lg_class
01/30/02   V. Goel            Add visibility to input parameter list
           V. Goel      3.3   Add userid and user name
2003-06-20 V. Goel      3.5   Use DB_Paging_Logic and DB_Build_WhereClause
2003-07-01 M.Mount      3.5   Changed input parameters to standard
2003-07-07 V Goel       3.5   Use FN_Paging_Logic and FN_Build_Where Functions
2008-10-20 Amit vijay	6.2   Defect 107019
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
--  v_startdate        varchar2(20) := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
--  v_enddate          varchar2(20) := to_char(nvl(p_EndDATE,sysdate)    , 'dd-mm-yy hh24:mi:ss');
  StartRow           pls_integer;
  EndRow             pls_integer;
  sqlstmt            varchar2(4000);
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
-- If p_DebugFlg =1  Then
  DB_ProcAuditLogI_AH1('LG_Events_VUID',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'||','||'RC2'||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_VUID         ||','||
    p_ActivityID   ||','||
    p_VISIBILITY
    ,0,0);
--commit;
--End if ;
End ProcAudit;
Procedure where_clause as
begin
    v_where := ' where 1=1'||
    FN_Build_Where('LG_EVENT.ACTIVITYID','VARCHAR2'   , p_ACTIVITYID)||
    FN_Build_Where('LG_EVENT.VUID'      ,'VARCHAR2'   , p_VUID)||
    FN_Build_Where('VISIBILITY',    'NUMBER_LIST'   , p_VISIBILITY);
  end where_clause;
--
-- Main
--
BEGIN
    p_ReturnCd := 0;
    where_clause;
    sqlstmt := 'select
          LG_EVENT.EVENTID
        , LG_EVENT.EVENTCD
        , LG_EVENT.EVENTTS
        , LG_EVENT.FAULTTYPE
        , LG_EVENT.HOSTIPADDRESS
        , LG_EVENT.SOURCECLASS
        , LG_EVENT.PARTNERID1
        , P1.PARTNERNAME PARTNERNAME1
        , LG_EVENT.PARTNERID2
        , P2.PARTNERNAME PARTNERNAME2
        , LG_EVENTCD.EVENTNAME
        , LG_EVENTCD.EVENTNAMEKEY
        , LG_EVENTCD.INTERNALDESC
        , CASE WHEN LG_EVENT.TEXTID IS NOT NULL THEN RTRIM(LG_EVENT.TEXTID) ELSE RTRIM(LG_EVENTCD.TEXTKEY) END AS TEXTKEY
        , LG_EVENTCD.VISIBILITY
        , LG_EVENTCD.SEVERITY
        , LG_EVENTCD.ALERTABLEFLG
        , LG_EVENT.ARGS
        , lg_event.USERID
        , u.USERNAME
		, LG_EVENTCD.EVENTCODE
		, LG_EVENTCD.EVENTNAMERESOURCEBUNDLEID
		, LG_EVENTCD.TEXTKEYRESOURCEBUNDLEID
		, LG_EVENTCD.EXTERNALDESCRESOURCEBUNDLEID 				
       from    LG_EVENT
           LEFT OUTER JOIN PR_PARTNER p1        on LG_EVENT.PARTNERID1 = p1.PARTNERID
           LEFT OUTER JOIN PR_PARTNER p2        on LG_EVENT.PARTNERID2 = p2.PARTNERID
           LEFT OUTER JOIN PR_USER U            ON LG_EVENT.USERID = u.USERID
           INNER JOIN LG_EVENTCD                on LG_EVENT.EVENTCD = LG_EVENTCD.EVENTCD' ||' '||v_where||' and LG_EVENT.ACTIVITYID is not Null ';

    if p_sortkey is not null then
       sqlstmt := sqlstmt ||' order by '||p_sortkey;
    end if;
-- DB_Print('Sql ', sqlstmt);
-- total row count Cursor
    if RC2%IsOpen then close RC2; end if;
    if NVL(p_pageNbr,0) in (0,1) then
      open RC2 for 'select count(*) TotalRowsNbr from ('||sqlstmt||')';
     end if;
-- Paging logic
     sqlstmt := FN_Paging_Logic(p_pagenbr, p_pagesizenbr,sqlstmt);
     DB_Print('Sql ', sqlstmt);
-- Open Cursor
    if RC1%IsOpen then close RC1; end if;
    open RC1 for  sqlstmt;

  ProcAudit;
Exception
 when others then
  ProcAudit;
  p_ReturnCd := 1;
END ;
/


-- LG_EVENTS_VUID_AH1.sql

CREATE OR REPLACE PROCEDURE LG_EVENTS_VUID_AH1
(
      p_ReturnCd       out pls_INTEGER
    , p_DebugFlg        in  pls_INTEGER
    , RC1               out  db_ref.rct          -- rows
    , RC2               out  db_ref.rct          -- count
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_VUID            in  varchar2
    , p_ActivityID      in  varchar2
    , p_VISIBILITY      in  varchar2      -- 1=hub Owner  2=sponcer  4= trading partner
)
AS
/*
================================================================================
Proc Name:      LG_EventS_VUID_AH1
Create Date:    21/09/2007
Author          AMIT KUMAR VIJAY
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  -----------------  ---------------------------------------------
DATE       NAME         VER     DESCRIPTION
---------  -----------------  ---------------------------------------------
21/09/2007 AMIT KUMAR  6.1.1  CREATED
2008-10-20 Amit vijay  6.2    Defect 107019
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
  v_where1            varchar2(1000);
--  v_startdate        varchar2(20) := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
--  v_enddate          varchar2(20) := to_char(nvl(p_EndDATE,sysdate)    , 'dd-mm-yy hh24:mi:ss');
  StartRow           pls_integer;
  EndRow             pls_integer;
  sqlstmt            varchar2(4000);
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
-- If p_DebugFlg =1  Then
  DB_ProcAuditLogI_AH1('LG_Events_VUID',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'||','||'RC2'||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_VUID         ||','||
    p_ActivityID   ||','||
    p_VISIBILITY
    ,0,0);
--commit;
--End if ;
End ProcAudit;
Procedure where_clause as
begin
    v_where := ' where 1=1'||
    FN_Build_Where('LG_EVENT.ACTIVITYID','VARCHAR2'   , p_ACTIVITYID)||
    FN_Build_Where('LG_EVENT.VUID'      ,'VARCHAR2'   , p_VUID)||
    FN_Build_Where('VISIBILITY',    'NUMBER_LIST'   , p_VISIBILITY);
    
    v_where1 := ' where 1=1'||
            FN_Build_Where('ef.VUID',	'VARCHAR2', p_VUID)||
            FN_Build_Where('ef.FTPEVENTID'      ,'NUMBER'   , 'xf.FTPEVENTID')||
            FN_Build_Where('LG_EVENT.EVENTID',    'VARCHAR2'   , 'xf.EVENTID');

  end where_clause;
--
-- Main
--
BEGIN
    p_ReturnCd := 0;
    where_clause;
    sqlstmt := 'select
          LG_EVENT.EVENTID
        , LG_EVENT.EVENTCD
        , LG_EVENT.EVENTTS
        , LG_EVENT.FAULTTYPE
        , LG_EVENT.HOSTIPADDRESS
        , LG_EVENT.SOURCECLASS
        , LG_EVENT.PARTNERID1
        , P1.PARTNERNAME PARTNERNAME1
        , LG_EVENT.PARTNERID2
        , P2.PARTNERNAME PARTNERNAME2
        , LG_EVENTCD.EVENTNAME
        , LG_EVENTCD.EVENTNAMEKEY
        , LG_EVENTCD.INTERNALDESC
        , CASE WHEN LG_EVENT.TEXTID IS NOT NULL THEN RTRIM(LG_EVENT.TEXTID) ELSE RTRIM(LG_EVENTCD.TEXTKEY) END AS TEXTKEY
        , LG_EVENTCD.VISIBILITY
        , LG_EVENTCD.SEVERITY
        , LG_EVENTCD.ALERTABLEFLG
        , LG_EVENT.ARGS
        , lg_event.USERID
        , u.USERNAME
		, LG_EVENTCD.EVENTCODE
		, LG_EVENTCD.EVENTNAMERESOURCEBUNDLEID
		, LG_EVENTCD.TEXTKEYRESOURCEBUNDLEID
		, LG_EVENTCD.EXTERNALDESCRESOURCEBUNDLEID 				
       from    LG_EVENT
           LEFT OUTER JOIN PR_PARTNER p1        on LG_EVENT.PARTNERID1 = p1.PARTNERID
           LEFT OUTER JOIN PR_PARTNER p2        on LG_EVENT.PARTNERID2 = p2.PARTNERID
           LEFT OUTER JOIN PR_USER U            ON LG_EVENT.USERID = u.USERID
           INNER JOIN LG_EVENTCD                on LG_EVENT.EVENTCD = LG_EVENTCD.EVENTCD' ||' '||v_where||' and LG_EVENT.ACTIVITYID is not Null '
       ||'  UNION '||
       'select ' ||
         ' LG_EVENT.EVENTID ' ||
        ', LG_EVENT.EVENTCD ' ||
        ', LG_EVENT.EVENTTS ' ||
        ', LG_EVENT.FAULTTYPE ' ||
        ', LG_EVENT.HOSTIPADDRESS ' ||
        ', LG_EVENT.SOURCECLASS ' ||
        ', LG_EVENT.PARTNERID1 ' ||
        ', P1.PARTNERNAME PARTNERNAME1 ' ||
        ', LG_EVENT.PARTNERID2 ' ||
        ', P2.PARTNERNAME PARTNERNAME2 ' ||
        ', LG_EVENTCD.EVENTNAME' ||
        ', LG_EVENTCD.EVENTNAMEKEY ' ||
        ', LG_EVENTCD.INTERNALDESC ' ||
        ', CASE WHEN LG_EVENT.TEXTID IS NOT NULL THEN RTRIM(LG_EVENT.TEXTID) ELSE RTRIM(LG_EVENTCD.TEXTKEY) END AS TEXTKEY ' ||
        ', LG_EVENTCD.VISIBILITY ' ||
        ', LG_EVENTCD.SEVERITY ' ||
        ', LG_EVENTCD.ALERTABLEFLG ' ||
        ', LG_EVENT.ARGS ' ||
        ', lg_event.USERID ' ||
        ', u.USERNAME ' ||
		', LG_EVENTCD.EVENTCODE' ||
		', LG_EVENTCD.EVENTNAMERESOURCEBUNDLEID' ||
		', LG_EVENTCD.TEXTKEYRESOURCEBUNDLEID' ||
		', LG_EVENTCD.EXTERNALDESCRESOURCEBUNDLEID ' ||
       'from    LG_EVENT , LG_EVENT_FTP_XREF fx, LG_EVENT_FTP ef' ||
       '    LEFT OUTER JOIN PR_PARTNER p1        on LG_EVENT.PARTNERID1 = p1.PARTNERID ' ||
       '    LEFT OUTER JOIN PR_PARTNER p2        on LG_EVENT.PARTNERID2 = p2.PARTNERID ' ||
       '    LEFT OUTER JOIN PR_USER U            ON LG_EVENT.USERID = u.USERID ' ||
       '    INNER JOIN LG_EVENTCD                on LG_EVENT.EVENTCD = LG_EVENTCD.EVENTCD ' ||' '||v_where1;

    if p_sortkey is not null then
       sqlstmt := sqlstmt ||' order by '||p_sortkey;
    end if;
-- DB_Print('Sql ', sqlstmt);
-- total row count Cursor
    if RC2%IsOpen then close RC2; end if;
    if NVL(p_pageNbr,0) in (0,1) then
      open RC2 for 'select count(*) TotalRowsNbr from ('||sqlstmt||')';
     end if;
-- Paging logic
     sqlstmt := FN_Paging_Logic(p_pagenbr, p_pagesizenbr,sqlstmt);
     DB_Print('Sql ', sqlstmt);
-- Open Cursor
    if RC1%IsOpen then close RC1; end if;
    open RC1 for  sqlstmt;

  ProcAudit;
Exception
 when others then
  ProcAudit;
  p_ReturnCd := 1;
END ;
/


-- LG_EVENT_FTPI.sql

Create or Replace Procedure LG_EVENT_FTPI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_EVENTID in VARCHAR2
	,p_FTPUSERID in NUMBER
	,p_ORIGINALFILENAME in VARCHAR2
	,p_LASTMODIFIEDTIME in VARCHAR2
	
	)
AS

/*
	================================================================================
	Proc Name:       LG_EVENT_FTPI
	Create Date:     2007-08-29 10:26:49
	Author:          AMIT KUMAR VIJAY

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2007-08-29 AMIT V.	6.1.1	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENT_FTPI';
v_FTPEVENTID	  LG_EVENT_FTP.FTPEVENTID%type;
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_EVENTID=>'||p_EVENTID||CHR(10)
		||', p_FTPUSERID=>'||p_FTPUSERID||CHR(10)
		||', p_ORIGINALFILENAME=>'||p_ORIGINALFILENAME||CHR(10)
		||', p_LASTMODIFIEDTIME=>'||p_LASTMODIFIEDTIME||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_EVENT_FTP(
		FTPEVENTID
		,FTPUSERID
		,ORIGINALFILENAME
		,LASTMODIFIEDTIME
		,UPDATEDATE
		,CREATEDATE
	) Values (
		LG_EVENT_FTP_SEQ.nextval
		,p_FTPUSERID
		,p_ORIGINALFILENAME
		,p_LASTMODIFIEDTIME
		,v_BeginTS
		,v_BeginTS
	) returning FTPEVENTID into v_FTPEVENTID;

	LG_EVENT_FTP_XREFI(
		p_ReturnCd
            	,p_DebugFlg
            	,RC1
            	,v_FTPEVENTID
            	,p_EVENTID
            	);
            	
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		LG_EVENT_FTP_SEQ.currval ID,1 ROWTS From Dual;
	p_ReturnCd := 0; 
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 LG_EVENT_FTPI' || SQLCODE|| ' '||SQLERRM);
END LG_EVENT_FTPI;
/


-- LG_EVENT_FTP_XREFI.sql

Create or Replace Procedure LG_EVENT_FTP_XREFI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_FTPEVENTID in NUMBER
	,p_EVENTID in VARCHAR2
		
	)
AS

/*
	================================================================================
	Proc Name:       LG_EVENT_FTP_XREFI
	Create Date:     2007-08-29 10:26:49
	Author:          AMIT KUMAR VIJAY

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME		Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2007-08-29 AMIT V.	6.1.1	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_EVENT_FTP_XREFI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FTPEVENTID=>'||p_FTPEVENTID||CHR(10)
		||', p_EVENTID=>'||p_EVENTID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_EVENT_FTP_XREF(
		FTPEVENTXREFID
		,FTPEVENTID
		,EVENTID
		,UPDATEDATE
		,CREATEDATE
	) Values (
		LG_EVENT_FTP_XREF_SEQ.nextval
		,p_FTPEVENTID
		,p_EVENTID
		,v_BeginTS
		,v_BeginTS
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		LG_EVENT_FTP_XREF_SEQ.currval ID,1 ROWTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR (-20999,'20002 LG_EVENT_XREFI' || SQLCODE|| ' '||SQLERRM);
END LG_EVENT_FTP_XREFI;
/


-- LG_LOCKD.sql

Create or Replace Procedure LG_LOCKD

        (p_ReturnCd out pls_integer
        ,p_DebugFlg in pls_integer 
        ,p_ID in VARCHAR2
        ,p_ComponentName IN VARCHAR2
        )
AS

/*
        ================================================================================
        Proc Name:       LG_LOCKD
        Create Date:     2003-11-18 16:06:25
        Author:          SYSTEM 

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-11-18 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_LOCKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ID=>'||p_ID||CHR(10)
                ;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

   --48808 added if lockid is null then don't do anything
   IF p_ID IS NOT NULL THEN
        --58747 added DAE component
        IF upper(p_ComponentName) = 'EDIENV' THEN
            DELETE FROM LG_LOCK_EDIENV
            WHERE lockid = p_ID;
        ELSIF upper(p_ComponentName) = 'FTPSCRIPT' THEN
            DELETE FROM LG_LOCK_FTPSCRIPT
            WHERE lockid = p_ID;
        ELSIF upper(p_ComponentName) = 'DAE' THEN
            DELETE FROM LG_LOCK_DAE
            WHERE lockid = p_ID;
        END IF;
   END IF;

        v_RowCnt := SQL%ROWCOUNT;
        if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        p_ReturnCd := 0;
        
        Exception
                When ZeroDelete Then
                        --48808 - make zero delete exception do nothing
                        p_ReturnCd := 0;
                        --v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
                        --RAISE_APPLICATION_ERROR (-20999,'20002 LG_LOCKD: No Data Found');
                When OTHERS Then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_LOCKD;
/


-- LG_LOCKI.sql

CREATE OR REPLACE PROCEDURE LG_LOCKI
        (p_ReturnCd        OUT pls_integer
        ,p_DebugFlg        IN pls_integer
        ,RC1               OUT db_ref.RCT   
        ,p_ComponentName   IN VARCHAR2
        ,p_ID              IN VARCHAR2
        ,p_Max_Age         IN PLS_INTEGER   -- in seconds
        ,p_ProcessType     IN VARCHAR2
        ,p_MaxReserveTime  IN PLS_INTEGER   -- in seconds
        
        )
AS

/*
        ================================================================================
        Proc Name:       LG_LOCKI
        Create Date:     2003-11-18 15:42:31
        Author:          V.Goel

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-11-18 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_updateDate     timestamp;
v_RowCnt         INTEGER DEFAULT 0;
v_ParamsTxt      VARCHAR2(4000) DEFAULT
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ID=>'||p_ID||CHR(10)
                ||', p_Max_Age=>'||p_Max_Age||CHR(10)
                ;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30)  DEFAULT 'LG_LOCKI';
v_ExceptionFlg    INTEGER  DEFAULT 0;
v_ExceptionCode   VARCHAR2(30);
v_PK_Value        INTEGER DEFAULT 0;
v_SQLErrm         VARCHAR2(255) ;
v_chkpt           VARCHAR2(1) DEFAULT '0';
v_msg             VARCHAR2(500);
v_RowTS           INTEGER DEFAULT 0;
v_updateDate      TimeStamp  DEFAULT NULL;
--DUP_VAL_ON_INDEX CONDITION FOR SQLSTATE '23505';
v_SelectStmt      VARCHAR2(4000);
v_InsertStmt      VARCHAR2(4000);
--needed for ftpscript queue logic
--return codes
-- 4 = lock request successful
-- 5 = lock request unsuccessful
-- 6 = lock request added to queue
-- 7 = process already owns lock
--**note that p_MaxReserveTime is the max time anyone can be in the queue
--**note that p_Max_Age is the max time anyone can hold a lock
v_LockTableName        VARCHAR2(30);
v_QueueTableName       VARCHAR2(30);
v_QueueLockID          VARCHAR2(60) DEFAULT NULL;
v_QueueProcess         VARCHAR2(60) DEFAULT NULL;
v_QueueUpdateDate      TimeStamp;
v_LockOwnerProcess     VARCHAR2(60) DEFAULT NULL;
v_LockID               VARCHAR2(60) DEFAULT NULL;
v_LockUpdateDate       TimeStamp  default null;
v_RCSuccess            INTEGER DEFAULT 0;
v_RCCouldNotGetLock    INTEGER DEFAULT 5;
v_RCAddedtoQ           INTEGER DEFAULT 6;
v_RCProcessOwnsLock    INTEGER DEFAULT 7;
v_ComponentNoQueue     INTEGER DEFAULT 8;
v_InQueue              VARCHAR2(1) DEFAULT 'N';
v_sqlstmt              VARCHAR2(4000);
v_DynamicSQL           VARCHAR2(4000);
BadComponent           EXCEPTION;
BadComponentProcesstype EXCEPTION;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
--create proc to check contents of lock table
PROCEDURE checkLockTable(tablename IN VARCHAR2, 
                         pv_ID IN VARCHAR2,
                         pv_LockUpdateDate OUT timestamp,
                         pv_LockOwnerProcess OUT VARCHAR2)
IS

BEGIN
   EXECUTE IMMEDIATE 
         ' SELECT updatedate,processtypename '
       ||' FROM '||tablename
       ||' WHERE lockid = :p1'
   INTO pv_LockUpdateDate, pv_LockOwnerProcess
   USING pv_ID;
EXCEPTION
    WHEN NO_DATA_FOUND THEN NULL;
END checkLockTable;

--create proc to check contents of queue
PROCEDURE CheckQueueTable(tablename IN VARCHAR2,
                          pv_ID IN VARCHAR2, 
                          pv_QueueLockID OUT VARCHAR2,
                          pv_QueueProcess OUT VARCHAR2, 
                          pv_QueueUpdateDate OUT TimeStamp)
IS
BEGIN
   EXECUTE IMMEDIATE
         ' SELECT lockid, processtypename, updatedate '
       ||' FROM '||tablename
       ||' WHERE lockid = :p2'
   INTO pv_QueueLockID, pv_QueueProcess, pv_QueueUpdateDate
   USING pv_ID;
EXCEPTION
   WHEN NO_DATA_FOUND THEN NULL;
END CheckQueueTable;

BEGIN
   p_ReturnCd := v_RCSuccess;
   v_chkpt := '1';

   --48339 -> this procedure is ok for 48339 because p_Max_Age and p_MaxReserveTime
   --both come into the procedure as miliseconds.  FN_TimestampDiff returns miliseconds
   --so as a result the procedure is working fine

   --set the variables for the table names
   IF upper(p_ComponentName) = 'EDIENV' THEN
      
      v_LockTableName := 'LG_LOCK_EDIENV';
      --leave this blank since there's no q for enveloper, then we'll go to the
      --sql exception handler
      v_QueueTableName := '';
      
      IF p_ProcessType IS NOT NULL THEN
         RAISE BadComponentProcesstype;
      END IF;

   ELSIF upper(p_ComponentName) = 'FTPSCRIPT' THEN

      v_LockTableName := 'LG_LOCK_FTPSCRIPT';
      v_QueueTableName := 'LG_LOCK_FTPSCRIPT_Q';

   --58747 added DAE component
   ELSIF upper(p_ComponentName) = 'DAE' THEN

      v_LockTableName := 'LG_LOCK_DAE';
      v_QueueTableName := '';


   ELSE
      --custom error handler for case where an invalid component was passed in
      RAISE BadComponent;
   END IF;
   
   --call a private proc to get the contents of the lock table
   checkLockTable(v_LockTableName, p_ID, v_LockUpdateDate, v_LockOwnerProcess);

   --if there should be q logic then check the contents of the q for the lock id
   IF p_ProcessType IS NOT NULL THEN
      CheckQueueTable(v_QueueTableName,p_ID,v_QueueLockId,v_QueueProcess,v_QueueUpdateDate);
   END IF;

   --now check to if there is no lock, or the current lock has expired
   IF (v_LockUpdateDate IS NULL 
   OR FN_TimestampDiff(v_LockUpdateDate,v_BeginTS) > p_Max_Age) 
   AND p_ReturnCd <> v_RCCouldNotGetLock THEN

      --do we need to check the q?
      IF p_ProcessType IS NOT NULL THEN
      
         --if process is not in q, but the process in the q has expired
         --OR this process is in the q
         --OR the q is empty
         IF (v_QueueProcess IS NULL) 
         OR
            (v_QueueProcess <> p_ProcessType AND
             FN_TimestampDiff(v_QueueUpdateDate,v_BeginTS) > p_MaxReserveTime)
         OR (v_QueueProcess = p_ProcessType) THEN

         --if the q is not empty then 
         --delete the record from the queue, then get the lock
            IF v_QueueProcess IS NOT NULL THEN
               
               EXECUTE IMMEDIATE 'DELETE FROM '||v_QueueTableName
                                  ||' WHERE lockid = '''||p_ID||'''';
               
            END IF;
      
         --if there's a lock available but someone else is in the q
         --and their time hasn't expired then we can't get the lock
         ELSIF v_QueueProcess <> p_ProcessType 
         AND FN_TimestampDiff(v_QueueUpdateDate,v_BeginTS) < p_MaxReserveTime THEN
         
            p_ReturnCd := v_RCCouldNotGetLock;
            --SET p_ReturnCd = 53;
         END IF;
      END IF;

      --if there is no lock insert it
      IF v_LockUpdateDate IS NULL AND p_ReturnCd <> v_RCCouldNotGetLock THEN
         
         --EXECUTE IMMEDIATE v_DynamicSQL;
         EXECUTE IMMEDIATE 'INSERT INTO '||v_LockTableName
                    ||' (lockid,UPDATEDATE,CREATEDATE,RowTS, processtypename) '
                    ||' VALUES(:p3 , :p4 , :p5 , 1, :6)'
         USING p_ID, v_BeginTS, v_BeginTS, p_ProcessType; 
      
         p_ReturnCd := v_RCSuccess;

         --added for 47803 to return ROWTS in case of success
         IF RC1%isOpen THEN Close RC1; END IF;
	         Open RC1 FOR Select 1 RowTS From Dual;
         --SET p_ReturnCd = 12;
         
      --if there is a lock that has expired get the lock   
      ELSIF FN_TimestampDiff(v_LockUpdateDate,v_BeginTS) > p_Max_Age 
      AND p_ReturnCd <> v_RCCouldNotGetLock THEN

         EXECUTE IMMEDIATE 'UPDATE '||v_LockTableName
                ||' SET updatedate = :p6 , '
                ||' processtypename = :p7 ,'
                ||' rowts = rowts + 1'
                ||' WHERE lockid = :p8'
         USING v_BeginTS, p_ProcessType, p_ID;
      
         p_ReturnCd := v_RCSuccess;

         --added for 47803 to return ROWTS in case of success
         --55063 put v_BeginTS in quotes as well
         --had to add NVL for proccesstype being null
         IF RC1%isOpen THEN Close RC1; END IF;
            v_sqlstmt := 'SELECT rowts RowTS'
                      ||' FROM '|| v_LockTableName
                      ||' WHERE updatedate = '''||v_BeginTS||''''
                      ||' AND NVL(processtypename,''UNKNOWNPSTYPE'') = NVL( '''||p_ProcessType||''',''UNKNOWNPSTYPE'')'
                      ||' AND lockid = '''||p_ID||'''';
            Open RC1 FOR v_sqlstmt;

         --SET p_ReturnCd = 13;

      END IF;

--if there is NOT a lock available
ELSE

   --if there is no q then we can't get the lock
   IF p_ProcessType IS NULL THEN
      p_ReturnCd := v_RCCouldNotGetLock;
      --SET p_ReturnCd = 54;

   --if there is q logic check to see if we can be added to the q
   ELSE
      --does this process already own the lock?
      IF v_LockOwnerProcess = p_ProcessType THEN
         p_ReturnCd := v_RCProcessOwnsLock;

      ELSE
         --if the q is empty add to q
         IF v_QueueProcess IS NULL THEN
            EXECUTE IMMEDIATE ' INSERT INTO '||v_QueueTableName
                            ||' (lockid, processtypename, createdate, updatedate, rowts)'
                            ||' VALUES(:p9 , :p10 , :p11 , :p12 , 1)'
            USING p_ID, p_ProcessType, v_BeginTS, v_BeginTS   ;
            
            p_ReturnCd := v_RCAddedtoQ;
            --SET p_ReturnCd = 61;
            
         --if the process requesting the lock is in the q update the update
         --date and return
         ELSIF v_QueueProcess = p_ProcessType THEN

            EXECUTE IMMEDIATE 'UPDATE '||v_QueueTableName
                            ||' SET updatedate = :p13 '
                            ||' WHERE processtypename = :p14'
                            ||' AND lockid = :p15 '
            USING v_BeginTS, p_ProcessType, p_ID   ;
            
            p_ReturnCd := v_RCAddedtoQ;
            --SET p_ReturnCd = 62;

         --if the process requesting the lock IS NOT in the q check
         --who is, if their time is up remove them and add self
         --ow can't get lock
         ELSIF v_QueueProcess <> p_ProcessType THEN

            --has the current record in the q expired?
            IF FN_TimeStampDiff(v_QueueUpdateDate,v_BeginTS) > p_MaxReserveTime THEN
                     
               EXECUTE IMMEDIATE 'UPDATE '||v_QueueTableName
                               ||' SET updatedate = :p16 , '
                               ||' processtypename = :p17 '
                               ||' WHERE lockid = :p18 '
               USING v_BeginTS, p_ProcessType, p_ID ;

               p_ReturnCd := v_RCAddedtoQ;
               --SET p_ReturnCd = 63;
            
            ELSE
               --can't get the lock because another process has the lock
               --and it hasn't expired yet
               p_ReturnCd := v_RCCouldNotGetLock;
               --SET p_ReturnCd = 55;
            END IF;
         END IF;
      END IF;
   END IF;
END IF;
    
        EXCEPTION
            When NO_DATA_FOUND THEN
                p_ReturnCd := 3;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
                --RAISE_APPLICATION_ERROR (-20999,'20000 LG_LOCKI' || SQLCODE|| ' '||SQLERRM);
            When DUP_VAL_ON_INDEX then
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                p_returnCd := 2;
                --RAISE_APPLICATION_ERROR (-20999,'20001 LG_LOCKI' || SQLCODE|| ' '||SQLERRM);
            When BadComponent THEN
                v_ExceptionFlg := 1;
                p_ReturnCd := v_RCCouldNotGetLock;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Invalid Component Entered, '||v_ParamsTxt, p_DebugFlg, 0);
            When BadComponentProcesstype THEN
                v_ExceptionFlg := 1;
                p_ReturnCd := v_ComponentNoQueue;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'This Component Does Not Support Queueing, '||v_ParamsTxt, p_DebugFlg, 0);
            When OTHERS Then
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                RAISE_APPLICATION_ERROR (-20999,'20002 LG_LOCKI' || SQLCODE|| ' '||SQLERRM);
END LG_LOCKI;
/


-- LG_LOCKU.sql

CREATE OR REPLACE PROCEDURE LG_LOCKU
        (
         p_ReturnCd out integer
        ,p_DebugFlg in pls_integer
        ,RC1 out db_ref.RCT
        ,p_ID in VARCHAR2
        ,p_ComponentName IN VARCHAR2
        ,p_rowts IN NUMBER
) as

/*
        ================================================================================
        Proc Name:       LG_LOCKU
        Create Date:     2003-11-18 15:58:23
        Author:          V Goel

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-11-18 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_LOCKU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ID=>'||p_ID||CHR(10)
                ;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        v_RowCnt := 0;

        --58747 added DAE component
        IF upper(p_ComponentName) = 'EDIENV' THEN
            Update LG_LOCK_EDIENV
            SET UPDATEDATE= v_BeginTS,
               ROWTS= RowTS + 1
            Where
               lockid = p_ID
               AND rowts = p_rowts returning RowTS into v_RowTS;
        ELSIF upper(p_ComponentName) = 'FTPSCRIPT' THEN
            Update LG_LOCK_FTPSCRIPT
            SET UPDATEDATE= v_BeginTS,
               ROWTS= RowTS + 1
            Where
               lockid = p_ID
               AND rowts = p_rowts returning RowTS into v_RowTS;
        ELSIF upper(p_ComponentName) = 'DAE' THEN
            Update LG_LOCK_DAE
            SET UPDATEDATE= v_BeginTS,
               ROWTS= RowTS + 1
            Where
               lockid = p_ID
               AND rowts = p_rowts returning RowTS into v_RowTS;
        END IF;


        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;

        p_ReturnCd := 0;

        IF RC1%isOpen THEN Close RC1; END IF;
	         OPEN RC1 for select v_RowTS RowTS from Dual;

        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        EXCEPTION
                WHEN ZeroUpdate then
                    p_ReturnCd := 3;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_LOCKU: No Data Found');
                WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_LOCKU:'||SQLERRM);
End LG_LOCKU;
/


-- LG_MEDIAD.sql

CREATE OR REPLACE Procedure LG_MEDIAD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_MEDIAID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_MEDIAD
	Create Date:     2003-09-26 08:24:06
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_MEDIAD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MEDIAID=>'||p_MEDIAID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From LG_MEDIA
	Where
		MEDIAID = p_MEDIAID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_MEDIAD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_MEDIAD;
/


-- LG_MEDIAI.sql

CREATE OR REPLACE Procedure LG_MEDIAI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_LINKMEDIAID in NUMBER
	,p_URI in VARCHAR2
	,p_TYPEFLG in NUMBER
	,p_PURGEFLG in NUMBER
	,p_ARCHIVEFLG in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_MEDIAI
	Create Date:     2003-09-26 00:01:26
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_MEDIAI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_LINKMEDIAID=>'||p_LINKMEDIAID||CHR(10)
		||', p_URI=>'||p_URI||CHR(10)
		||', p_TYPEFLG=>'||p_TYPEFLG||CHR(10)
		||', p_PURGEFLG=>'||p_PURGEFLG||CHR(10)
		||', p_ARCHIVEFLG=>'||p_ARCHIVEFLG||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_MEDIA(
		MEDIAID
		,LINKMEDIAID
		,URI
		,TYPEFLG
		,PURGEFLG
		,ARCHIVEFLG
	) Values (
		LG_MEDIA_SEQ.nextval
		,p_LINKMEDIAID
		,p_URI
		,p_TYPEFLG
		,p_PURGEFLG
		,p_ARCHIVEFLG
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		LG_MEDIA_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 LG_MEDIAI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 LG_MEDIAI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_MEDIAI' || SQLCODE|| ' '||SQLERRM);
END LG_MEDIAI;
/


-- LG_MEDIAS.sql

CREATE OR REPLACE PROCEDURE LG_MEDIAS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_MEDIAID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_MEDIAS
	Create Date:     2003-09-26 00:06:49
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_MEDIAS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MEDIAID=>'||p_MEDIAID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_MEDIAID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		MEDIAID
		,LINKMEDIAID
		,RTRIM(URI) URI
		,TYPEFLG
		,PURGEFLG
		,ARCHIVEFLG
		,ROWTS
		,CREATEDATE
	FROM
		LG_MEDIA
	Where
		MEDIAID = p_MEDIAID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		MEDIAID
		,LINKMEDIAID
		,RTRIM(URI) URI
		,TYPEFLG
		,PURGEFLG
		,ARCHIVEFLG
		,ROWTS
		,CREATEDATE
	FROM
		LG_MEDIA
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_MEDIAS;
/


-- LG_MEDIAU.sql

CREATE OR REPLACE Procedure LG_MEDIAU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_MEDIAID in NUMBER
	,p_LINKMEDIAID in NUMBER
	,p_URI in VARCHAR2
	,p_TYPEFLG in NUMBER
	,p_PURGEFLG in NUMBER
	,p_ARCHIVEFLG in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       LG_MEDIAU
	Create Date:     2003-09-26 08:21:02
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_MEDIAU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MEDIAID=>'||p_MEDIAID||CHR(10)
		||', p_LINKMEDIAID=>'||p_LINKMEDIAID||CHR(10)
		||', p_URI=>'||p_URI||CHR(10)
		||', p_TYPEFLG=>'||p_TYPEFLG||CHR(10)
		||', p_PURGEFLG=>'||p_PURGEFLG||CHR(10)
		||', p_ARCHIVEFLG=>'||p_ARCHIVEFLG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update LG_MEDIA
	SET
		LINKMEDIAID=p_LINKMEDIAID
		,      URI=p_URI
		,      TYPEFLG=p_TYPEFLG
		,      PURGEFLG=p_PURGEFLG
		,      ARCHIVEFLG=p_ARCHIVEFLG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		MEDIAID = p_MEDIAID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - LG_MEDIAU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - LG_MEDIAU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - LG_MEDIAU:'||SQLERRM);
End LG_MEDIAU;
/


-- LG_MEDIA_CFGI.sql

CREATE OR REPLACE Procedure LG_MEDIA_CFGI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_MEDIAID in NUMBER
	,p_TYPEFLG in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_MEDIA_CFGI
	Create Date:     2003-09-26 00:01:28
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_MEDIA_CFGI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MEDIAID=>'||p_MEDIAID||CHR(10)
		||', p_TYPEFLG=>'||p_TYPEFLG||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_MEDIA_CFG(
		MEDIACFGID
		,MEDIAID
		,TYPEFLG
	) Values (
		LG_MEDIA_CFG_SEQ.nextval
		,p_MEDIAID
		,p_TYPEFLG
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		LG_MEDIA_CFG_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 LG_MEDIA_CFGI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 LG_MEDIA_CFGI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_MEDIA_CFGI' || SQLCODE|| ' '||SQLERRM);
END LG_MEDIA_CFGI;
/


-- LG_MEDIA_CFGS.sql

CREATE OR REPLACE PROCEDURE LG_MEDIA_CFGS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_MEDIACFGID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_MEDIA_CFGS
	Create Date:     2003-09-26 00:06:51
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_MEDIA_CFGS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MEDIACFGID=>'||p_MEDIACFGID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_MEDIACFGID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		MEDIACFGID
		,MEDIAID
		,TYPEFLG
		,ROWTS
		,CREATEDATE
	FROM
		LG_MEDIA_CFG
	Where
		MEDIACFGID = p_MEDIACFGID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		MEDIACFGID
		,MEDIAID
		,TYPEFLG
		,ROWTS
		,CREATEDATE
	FROM
		LG_MEDIA_CFG
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_MEDIA_CFGS;
/


-- LG_MEDIA_CFGU.sql

CREATE OR REPLACE Procedure LG_MEDIA_CFGU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_MEDIACFGID in NUMBER
	,p_MEDIAID in NUMBER
	,p_TYPEFLG in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       LG_MEDIA_CFGU
	Create Date:     2003-09-26 08:21:04
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_MEDIA_CFGU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MEDIACFGID=>'||p_MEDIACFGID||CHR(10)
		||', p_MEDIAID=>'||p_MEDIAID||CHR(10)
		||', p_TYPEFLG=>'||p_TYPEFLG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update LG_MEDIA_CFG
	SET
		MEDIAID=p_MEDIAID
		,      TYPEFLG=p_TYPEFLG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		MEDIACFGID = p_MEDIACFGID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - LG_MEDIA_CFGU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - LG_MEDIA_CFGU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - LG_MEDIA_CFGU:'||SQLERRM);
End LG_MEDIA_CFGU;
/


-- LG_MSGLENGTH_SUMMARYI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_MSGLENGTH_SUMMARYI_AH1
    (
    p_ReturnCd out integer
    ,p_DebugFlg      in pls_integer
    ,p_MsgRAWLength_OUT in NUMBER
    ,p_SUMMARYID    in NUMBER
    ,p_DAYHOUR      in TIMESTAMP
    ,p_MsgRAWLength_IN  in NUMBER
) as

/*
================================================================================
Proc Name:        LG_MSGLENGTH_SUMMARYI_AH1
Create Date:     2002-11-15 13:45:55
Author:            V Goel
================================================================================
Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer     :=0;
v_ProcName  varchar2(30) := 'LG_MSGLENGTH_SUMMARYI_AH1';
v_BeginTS timestamp := FN_GET_DATE(); -- Performance related variables
v_pk_value number := 0;
v_SIZECATEGORYID_in   DB_MsgLength.SizeCategoryID%Type;
v_SIZECATEGORYID_out  DB_MsgLength.SizeCategoryID%Type;
v_DAYHOUR Date := p_DAYHOUR;  -- convert TS to date
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_AH1(v_ProcName,v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||p_SUMMARYID
    ||','||p_DAYHOUR
    ||','||p_MsgRAWLength_IN
    ||','||p_MsgRAWLength_OUT
    );
End ProcAudit;

Procedure InsertUpd_In_Out is
Begin
       Update LG_MSGLENGTH_SUMMARY s
                Set
                      s.RAWSIZECATEGORY_IN_CNT = s.RAWSIZECATEGORY_IN_CNT + 1
                    , s.RAWSIZECATEGORY_OUT_CNT = s.RAWSIZECATEGORY_OUT_CNT + 1
                    , s.UpdateDate =  v_BeginTS
                Where s.SIZECATEGORYID = v_SIZECATEGORYID_in and s.SummaryId =  p_SUMMARYID   and  s.DayHour = p_DayHour;

        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then
           INSERT INTO  LG_MSGLENGTH_SUMMARY
              (
                    SIZECATEGORYID
               ,    SUMMARYID
               ,    DAYHOUR
               ,    RAWSIZECATEGORY_IN_CNT
               ,    RAWSIZECATEGORY_OUT_CNT
               ) Values (
                   v_SIZECATEGORYID_in
               ,   p_SUMMARYID
               ,   p_DayHour
               ,    1
               ,    1
               );
         end if;


    EXCEPTION
        When NO_DATA_FOUND Then p_returnCd := 3;
        When Dup_val_on_index then
             p_returnCd := 2;
             Update LG_MSGLENGTH_SUMMARY s
                 Set
                    s.RAWSIZECATEGORY_IN_CNT = s.RAWSIZECATEGORY_IN_CNT + 1
                    , s.RAWSIZECATEGORY_OUT_CNT = s.RAWSIZECATEGORY_OUT_CNT + 1
                    , s.UpdateDate =  v_BeginTS
                 Where s.SIZECATEGORYID = v_SIZECATEGORYID_in and s.SummaryId =  p_SUMMARYID   and  s.DayHour = p_DayHour;
        When Others Then
            ProcAudit;
            Raise_application_error (-20999,'20000 LG_MSGLENGTH_SUMMARYI_AH1- InsertUpd_In_Out' || SQLCODE|| ' '||SQLERRM);
            p_ReturnCd := 1;
End  InsertUpd_In_Out;
--
procedure InsertUpd_In is
begin
     Update LG_MSGLENGTH_SUMMARY s
       Set
          s.RAWSIZECATEGORY_IN_CNT = s.RAWSIZECATEGORY_IN_CNT + 1
         , s.UpdateDate =  v_BeginTS
       Where s.SIZECATEGORYID = v_SIZECATEGORYID_in and s.SummaryId =  p_SUMMARYID   and  s.DayHour = p_DayHour;

       v_RowCnt := SQL%rowcount;
       if (v_RowCnt = 0) then

            INSERT INTO LG_MSGLENGTH_SUMMARY
               (
                    SIZECATEGORYID
               ,    SUMMARYID
               ,    DAYHOUR
               ,    RAWSIZECATEGORY_IN_CNT
               ,    RAWSIZECATEGORY_OUT_CNT
               ,    CreateDate
               ,    UpdateDate
               ) Values (
                   v_SIZECATEGORYID_in
               ,   p_SUMMARYID
               ,   p_DayHour
               ,    1
               ,    0
               ,    v_BeginTS
               ,    v_BeginTS
               );
       end if;

    EXCEPTION
        When NO_DATA_FOUND Then p_returnCd := 3;
        When Dup_val_on_index then
             p_returnCd := 2;
             Update LG_MSGLENGTH_SUMMARY s
              Set
                s.RAWSIZECATEGORY_IN_CNT = s.RAWSIZECATEGORY_IN_CNT + 1
               , s.UpdateDate =  v_BeginTS
              Where s.SIZECATEGORYID = v_SIZECATEGORYID_in and s.SummaryId =  p_SUMMARYID   and  s.DayHour = p_DayHour;
        When Others Then
            ProcAudit;
            Raise_application_error (-20999,'20000 LG_MSGLENGTH_SUMMARYI_AH1- InsertUpd_In' || SQLCODE|| ' '||SQLERRM);
            p_ReturnCd := 1;
end InsertUpd_In;
--
Procedure InsertUpd_out is

Begin
   --  DBMS_OUTPUT.put_line('v_SIZECATEGORYID_out '||v_SIZECATEGORYID_out);
       Update LG_MSGLENGTH_SUMMARY s
                Set
                     s.RAWSIZECATEGORY_OUT_CNT = s.RAWSIZECATEGORY_OUT_CNT + 1
                    , s.UpdateDate =  v_BeginTS
                Where s.SIZECATEGORYID = v_SIZECATEGORYID_in and s.SummaryId =  p_SUMMARYID   and  s.DayHour = p_DayHour;

       v_RowCnt := SQL%rowcount;
       if (v_RowCnt = 0) then

            INSERT INTO LG_MSGLENGTH_SUMMARY
               (
                    SIZECATEGORYID
               ,    SUMMARYID
               ,    DAYHOUR
               ,    RAWSIZECATEGORY_IN_CNT
               ,    RAWSIZECATEGORY_OUT_CNT
               ,    CreateDate
               ,    UpdateDate
               ) Values (
                  v_SIZECATEGORYID_in
               ,  p_SUMMARYID
               ,  p_DayHour
               ,  0
               ,  1
               ,  v_BeginTS
               ,  v_BeginTS);
       end if;

       EXCEPTION
         When NO_DATA_FOUND Then p_returnCd := 3;
         When Dup_val_on_index then
              p_returnCd := 2;
              Update LG_MSGLENGTH_SUMMARY s
               Set
                  s.RAWSIZECATEGORY_OUT_CNT = s.RAWSIZECATEGORY_OUT_CNT + 1
                , s.UpdateDate =  v_BeginTS
               Where s.SIZECATEGORYID = v_SIZECATEGORYID_in and s.SummaryId =  p_SUMMARYID   and  s.DayHour = p_DayHour;
         When Others Then
            ProcAudit;
            Raise_application_error (-20999,'20000 LG_MSGLENGTH_SUMMARYI_AH1- InsertUpd_out' || SQLCODE|| ' '||SQLERRM);
            p_ReturnCd := 1;


End InsertUpd_out;
----------------------------------------------------------------------
--                             M  A  I  N
----------------------------------------------------------------------
BEGIN
   v_SIZECATEGORYID_in  := FN_GET_SIZECATID(p_MsgRAWLength_IN);
   v_SIZECATEGORYID_out := FN_GET_SIZECATID(p_MsgRAWLength_OUT);

--   in/out Message Length fall in same size category

   IF v_SIZECATEGORYID_in = v_SIZECATEGORYID_out and v_SIZECATEGORYID_in != 'N/A' THEN
      InsertUpd_In_Out;
   ELSE
      IF v_SIZECATEGORYID_in != 'N/A' THEN  --   In Message Length has a size category
          InsertUpd_In;
      END IF;
      IF v_SIZECATEGORYID_out != 'N/A'  THEN --   Out Message Length has a size category
          InsertUpd_out;
      END IF;  -- OUT
   END IF;  -- Both
ProcAudit;
EXCEPTION
    When NO_DATA_FOUND Then Null;
    When Dup_val_on_index then p_returnCd := 2;
    When Others Then
        ProcAudit;
        Raise_application_error (-20999,'20003 LG_MSGLENGTH_SUMMARYI_AH1' || SQLCODE|| ' '||SQLERRM);
        p_ReturnCd := 1;
END LG_MSGLENGTH_SUMMARYI_AH1;
/


-- LG_MSG_ARCHIVEI.sql

CREATE OR REPLACE Procedure LG_MSG_ARCHIVEI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_VUID in VARCHAR2
	,p_NRMEDIAID in NUMBER
	,p_UPMEDIAID in NUMBER
	,p_SUBVOLNAME in VARCHAR2
	,p_DIGSIGCERTID in NUMBER
	,p_ENCRYPTCERTID in NUMBER
	,p_DIGSIGPRIKEYCERTID in NUMBER
	,p_ECRYPTPRIKEYCERTID in NUMBER
   ,p_businessid  IN VARCHAR2
   ,p_packagename  IN VARCHAR2
   ,p_protocolname IN VARCHAR2
   ,p_processname IN VARCHAR2
   ,p_packagever IN VARCHAR2
   ,p_protocolver IN VARCHAR2
   ,p_processver IN VARCHAR2
   ,p_BDO        IN BLOB
	)
AS

/*
	================================================================================
	Proc Name:       LG_MSG_ARCHIVEI
	Create Date:     2003-09-26 00:03:44
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
        2007-03-07 BHARANI          3.6 Modified for CR 104336
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_MSG_ARCHIVEI';
v_ExceptionFlg    integer Default 0;
v_partnername     VARCHAR2(120) DEFAULT NULL;

v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_NRMEDIAID=>'||p_NRMEDIAID||CHR(10)
		||', p_UPMEDIAID=>'||p_UPMEDIAID||CHR(10)
		||', p_SUBVOLNAME=>'||p_SUBVOLNAME||CHR(10)
		||', p_DIGSIGCERTID=>'||p_DIGSIGCERTID||CHR(10)
		||', p_ENCRYPTCERTID=>'||p_ENCRYPTCERTID||CHR(10)
		||', p_DIGSIGPRIKEYCERTID=>'||p_DIGSIGPRIKEYCERTID||CHR(10)
		||', p_ECRYPTPRIKEYCERTID=>'||p_ECRYPTPRIKEYCERTID||CHR(10)
      ||', p_businessid=>'||p_businessid||CHR(10)
      ||', p_packagename=>'||p_packagename||CHR(10)
      ||', p_protocolname=>'||p_protocolname||CHR(10)
      ||', p_processname=>'||p_processname||CHR(10)
      ||', p_packagever=>'||p_packagever||CHR(10)
      ||', p_protocolver=>'||p_protocolver||CHR(10)
      ||', p_processver=>'||p_processver||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   --100776
   BEGIN
   IF p_businessid IS NOT NULL THEN
      SELECT partnername INTO v_partnername
      FROM pr_partner partner, pr_busidentifier busid
      WHERE partner.partnerid = busid.partnerid
         AND busid.businessid = p_businessid;
   END IF;         
   EXCEPTION
   		When NO_DATA_FOUND THEN
           v_partnername := '';
     END; 
 

	Insert into LG_MSG_ARCHIVE(
		VUID
		,NRMEDIAID
		,UPMEDIAID
		,SUBVOLNAME
		,DIGSIGCERTID
		,ENCRYPTCERTID
		,DIGSIGPRIKEYCERTID
		,ECRYPTPRIKEYCERTID
      ,partnername
      ,packagename
      ,protocolname
      ,processname
      ,packagever
      ,protocolver
      ,processver
      ,bdo
	) Values (
		p_VUID
		,p_NRMEDIAID
		,p_UPMEDIAID
		,p_SUBVOLNAME
		,p_DIGSIGCERTID
		,p_ENCRYPTCERTID
		,p_DIGSIGPRIKEYCERTID
		,p_ECRYPTPRIKEYCERTID
      ,v_partnername
      ,p_packagename
      ,p_protocolname
      ,p_processname
      ,p_packagever
      ,p_protocolver
      ,p_processver
      ,p_BDO
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
--			RAISE_APPLICATION_ERROR (-20999,'20000 LG_MSG_ARCHIVEI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
--			RAISE_APPLICATION_ERROR (-20999,'20001 LG_MSG_ARCHIVEI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_MSG_ARCHIVEI' || SQLCODE|| ' '||SQLERRM);
END LG_MSG_ARCHIVEI;
/


-- LG_MSG_ARCHIVES_AH3.sql

CREATE OR REPLACE PROCEDURE LG_MSG_ARCHIVES_AH3 (
	p_returncd		OUT	PLS_INTEGER,
	p_debugflg		IN	PLS_INTEGER,
	rc1			OUT	db_ref.rct,
	rc2			OUT	db_ref.rct,
	p_date			IN	DATE,
	p_partner		IN	VARCHAR2,
	p_packagename		IN	VARCHAR2,
	p_packagever		IN	VARCHAR2,
	p_protocolname		IN	VARCHAR2,
	p_protocolver		IN	VARCHAR2,
	p_processname		IN	VARCHAR2,
	p_processver		IN	VARCHAR2,
	p_userid		IN	VARCHAR2,
	p_archdestination	IN	VARCHAR2
)
AS
/*
	================================================================================
	Proc Name:	LG_MSG_ARCHIVES_AH3
	Create Date:	2007-08-17 12:04:11
	Author:		S.Priani
	================================================================================
	Return code = 0	Good, 1 = Something went wrong, 2 = Dup, 3 = No data found
				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::
	---------- ------------ -------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------- ------------ -------	---------------------------------------------
	2007-08-24 S.Priani	6.1.1	CR 104487
	================================================================================*/
-- Performance related variables
	v_rowcnt		INTEGER		DEFAULT 0;
	v_begints		TIMESTAMP	DEFAULT fn_get_date;
	v_sqltext		VARCHAR2 (4000);
	v_sqltextstatic		VARCHAR2 (2000);
	v_flag			PLS_INTEGER	DEFAULT 0; --will open RC2
	v_subvolume_max		VARCHAR2 (8)	DEFAULT TO_CHAR(p_Date + 1, 'YYYYMMDD'); -- take the next day
	v_procname		VARCHAR2 (30)	DEFAULT 'LG_MSG_ARCHIVES_AH3';
	v_exceptionflg		INTEGER		DEFAULT 0;
	v_paramstxt		VARCHAR2 (4000)	:=	'p_DebugFlg=>' || p_debugflg || CHR (10)
		|| ', p_Date=>' || p_date || CHR (10)
		|| ', p_Partner=>' || p_partner || CHR (10)
		|| ', p_PackageNamer=>' || p_packagename || CHR (10)
		|| ', p_PackageVersion=>' || p_packagever || CHR (10)
		|| ', p_ProtocolName=>' || p_protocolname || CHR (10)
		|| ', p_ProtocolVersion=>' || p_protocolver || CHR (10)
		|| ', p_ProcessName=>' || p_processname || CHR (10)
		|| ', p_ProcessVersion=>' || p_processver || CHR (10)
		|| ', p_UserId=>' || p_userid || CHR (10)
		|| ', p_ArchiveDestination=>' || p_archdestination || CHR (10)
		;
BEGIN
	-- add into archive information table
	INSERT INTO lg_archiveinfo
		 (archiveid, actionname, userid,
			cutoffdate, partner,
			packagename,
			packagever,
			protocolname,
			protocolver,
			processname,
			processver,
			archdestination
		 )
	 VALUES (lg_archiveinfo_seq.NEXTVAL, 'NonRep Archive', p_userid, p_date,
			CASE p_partner WHEN 'allO0oxyz' then  'ALL' else  p_partner END, 
			CASE p_packagename WHEN  'allO0oxyz' then  'ALL' else  p_packagename END, 
			CASE p_packagever WHEN   'allO0oxyz' then  'ALL' else  p_packagever END, 
			CASE p_protocolname WHEN  'allO0oxyz' then  'ALL' else  p_protocolname END, 
			CASE p_protocolver WHEN   'allO0oxyz' then  'ALL' else  p_protocolver END, 
			CASE p_processname WHEN  'allO0oxyz' then  'ALL' else  p_processname END, 
			CASE p_processver WHEN  'allO0oxyz' then  'ALL' else  p_processver END, 
			p_archdestination
		 );

	if p_partner is NOT NULL AND p_partner <> 'allO0oxyz' then
		v_sqltextstatic := ' AND partnername = ''' || p_partner || '''';
		v_flag := v_flag +1;
	end if;

	if p_packagever is NOT NULL AND p_packagever <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND packagever = ''' || p_packagever || '''';
		v_flag := v_flag +1;
	end if;

	if p_packagename is NOT NULL AND p_packagename <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND packagename = ''' || p_packagename || '''';
		v_flag := v_flag +1;
	end if;

	if p_protocolname is NOT NULL AND p_protocolname <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND protocolname = ''' || p_protocolname || '''';
		v_flag := v_flag +1;
	end if;

	if p_protocolver is NOT NULL AND p_protocolver <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND protocolver = ''' || p_protocolver || '''';
		v_flag := v_flag +1;
	end if;

	if p_processname is NOT NULL AND p_processname <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND processname = ''' || p_processname || '''';
		v_flag := v_flag +1;
	end if;

	if p_processver is NOT NULL AND p_processver <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND processver = ''' || p_processver || '''';
		v_flag := v_flag +1;
	end if;

	IF v_flag = 0 THEN
	---list subvolume folders upto given Cutoff date
		OPEN RC2 FOR
			SELECT	count(m.vuid) vuid_count
				,c.uri
				,m.subvolname
			FROM	lg_msg_archive m
			INNER JOIN lg_media c ON m.nrmediaid = c.mediaid
			WHERE	m.subvolname < v_subvolume_max
			GROUP BY c.uri, m.subvolname
			;

	ELSE
	---list vuids by given filters
		v_sqltext := 'SELECT	m.vuid, c.uri, m.subvolname '
			|| ' FROM	lg_msg_archive m, lg_media c '
			|| ' WHERE	m.nrmediaid = c.mediaid '
			|| ' AND	m.subvolname < :IP_Limit '
			|| v_sqltextstatic;

		OPEN rc1 FOR v_sqltext USING v_subvolume_max;
	END IF;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
		p_returncd := 3;
	WHEN OTHERS THEN
		p_returncd := 1;
		db_procauditlogi_ah1 (v_procname,
				v_begints,
				v_paramstxt,
				p_debugflg,
				v_exceptionflg
				);
		raise_application_error (-20999, '20100 - LG_MSG_ARCHIVES_AH3:' || SQLERRM);
END lg_msg_archives_AH3;
/

-- LG_MSG_ARCHIVES_AH4.sql

CREATE OR REPLACE PROCEDURE LG_MSG_ARCHIVES_AH4 (
	p_returncd		OUT	PLS_INTEGER,
	p_debugflg		IN	PLS_INTEGER,
	rc1			OUT	db_ref.rct,
	rc2			OUT	db_ref.rct,
	p_date			IN	DATE,
	p_partner		IN	VARCHAR2,
	p_packagename		IN	VARCHAR2,
	p_packagever		IN	VARCHAR2,
	p_protocolname		IN	VARCHAR2,
	p_protocolver		IN	VARCHAR2,
	p_processname		IN	VARCHAR2,
	p_processver		IN	VARCHAR2,
	p_userid		IN	VARCHAR2,
	p_archdestination	IN	VARCHAR2
)
AS
/*
	================================================================================
	Proc Name:	LG_MSG_ARCHIVES_AH4
	Create Date:	2007-08-17 12:04:11
	Author:		S.Priani
	================================================================================
	Return code = 0	Good, 1 = Something went wrong, 2 = Dup, 3 = No data found
				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::
	---------- ------------ -------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------- ------------ -------	---------------------------------------------
	2007-08-24 S.Priani	6.1.1	CR 104487
	================================================================================*/
-- Performance related variables
	v_rowcnt		INTEGER		DEFAULT 0;
	v_begints		TIMESTAMP	DEFAULT fn_get_date;
	v_sqltext		VARCHAR2 (4000);
	v_sqltextstatic		VARCHAR2 (2000);
	v_flag			PLS_INTEGER	DEFAULT 0; --will open RC2
	v_subvolume_max		VARCHAR2 (8)	DEFAULT TO_CHAR(p_Date + 1, 'YYYYMMDD'); -- take the next day
	v_procname		VARCHAR2 (30)	DEFAULT 'LG_MSG_ARCHIVES_AH4';
	v_exceptionflg		INTEGER		DEFAULT 0;
	v_paramstxt		VARCHAR2 (4000)	:=	'p_DebugFlg=>' || p_debugflg || CHR (10)
		|| ', p_Date=>' || p_date || CHR (10)
		|| ', p_Partner=>' || p_partner || CHR (10)
		|| ', p_PackageNamer=>' || p_packagename || CHR (10)
		|| ', p_PackageVersion=>' || p_packagever || CHR (10)
		|| ', p_ProtocolName=>' || p_protocolname || CHR (10)
		|| ', p_ProtocolVersion=>' || p_protocolver || CHR (10)
		|| ', p_ProcessName=>' || p_processname || CHR (10)
		|| ', p_ProcessVersion=>' || p_processver || CHR (10)
		|| ', p_UserId=>' || p_userid || CHR (10)
		|| ', p_ArchiveDestination=>' || p_archdestination || CHR (10)
		;
BEGIN
	-- add into archive information table
	INSERT INTO lg_archiveinfo
		 (archiveid, actionname, userid,
			cutoffdate, partner,
			packagename,
			packagever,
			protocolname,
			protocolver,
			processname,
			processver,
			archdestination
		 )
	 VALUES (lg_archiveinfo_seq.NEXTVAL, 'MessageStore Archive', p_userid,
			p_date, DECODE (p_partner, 'allO0oxyz', 'ALL', p_partner),
			DECODE (p_packagename, 'allO0oxyz', 'ALL', p_packagename),
			DECODE (p_packagever, 'allO0oxyz', 'ALL', p_packagever),
			DECODE (p_protocolname, 'allO0oxyz', 'ALL', p_protocolname),
			DECODE (p_protocolver, 'allO0oxyz', 'ALL', p_protocolver),
			DECODE (p_processname, 'allO0oxyz', 'ALL', p_processname),
			DECODE (p_processver, 'allO0oxyz', 'ALL', p_processver),
			p_archdestination
		 );

	if p_partner is NOT NULL AND p_partner <> 'allO0oxyz' then
		v_sqltextstatic := ' AND partnername = ''' || p_partner || '''';
		v_flag := v_flag +1;
	end if;

	if p_packagever is NOT NULL AND p_packagever <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND packagever = ''' || p_packagever || '''';
		v_flag := v_flag +1;
	end if;

	if p_packagename is NOT NULL AND p_packagename <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND packagename = ''' || p_packagename || '''';
		v_flag := v_flag +1;
	end if;

	if p_protocolname is NOT NULL AND p_protocolname <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND protocolname = ''' || p_protocolname || '''';
		v_flag := v_flag +1;
	end if;

	if p_protocolver is NOT NULL AND p_protocolver <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND protocolver = ''' || p_protocolver || '''';
		v_flag := v_flag +1;
	end if;

	if p_processname is NOT NULL AND p_processname <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND processname = ''' || p_processname || '''';
		v_flag := v_flag +1;
	end if;

	if p_processver is NOT NULL AND p_processver <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND processver = ''' || p_processver || '''';
		v_flag := v_flag +1;
	end if;

	IF v_flag = 0 THEN
	---list subvolume folders upto given Cutoff date
		OPEN RC2 FOR
			SELECT	count(m.vuid) vuid_count
				,c.uri
				,m.subvolname
			FROM	lg_msg_archive m
			INNER JOIN lg_media c ON m.upmediaid = c.mediaid
			WHERE	m.subvolname < v_subvolume_max
			GROUP BY c.uri, m.subvolname
			;

	ELSE
	---list vuids by given filters
		v_sqltext := 'SELECT	m.vuid, c.uri, m.subvolname '
			|| ' FROM	lg_msg_archive m, lg_media c '
			|| ' WHERE	m.upmediaid = c.mediaid '
			|| ' AND	m.subvolname < :IP_Limit '
			|| v_sqltextstatic;

		OPEN rc1 FOR v_sqltext USING v_subvolume_max;
	END IF;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
		p_returncd := 3;
	WHEN OTHERS THEN
		p_returncd := 1;
		db_procauditlogi_ah1 (v_procname,
				v_begints,
				v_paramstxt,
				p_debugflg,
				v_exceptionflg
				);
		raise_application_error (-20999, '20100 - LG_MSG_ARCHIVES_AH4:' || SQLERRM);
END lg_msg_archives_AH4;
/

-- LG_MSG_ARCHIVES_AH5.sql

CREATE OR REPLACE PROCEDURE LG_MSG_ARCHIVES_AH5 (
	p_returncd		OUT	PLS_INTEGER,
	p_debugflg		IN	PLS_INTEGER,
	rc1			OUT	db_ref.rct,
	p_date			IN	DATE,
	p_partner		IN	VARCHAR2,
	p_packagename		IN	VARCHAR2,
	p_packagever		IN	VARCHAR2,
	p_protocolname		IN	VARCHAR2,
	p_protocolver		IN	VARCHAR2,
	p_processname		IN	VARCHAR2,
	p_processver		IN	VARCHAR2,
	p_userid		IN	VARCHAR2,
	p_archdestination	IN	VARCHAR2
)
AS
/*
	================================================================================
	Proc Name:	LG_MSG_ARCHIVES_AH5
	Create Date:	2007-08-17 12:04:11
	Author:		S.Priani
	================================================================================
	Return code = 0	Good, 1 = Something went wrong, 2 = Dup, 3 = No data found
				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::
	---------- ------------ -------	---------------------------------------------
	DATE	   NAME		Version	DESCRIPTION
	---------- ------------ -------	---------------------------------------------
	2007-08-24 S.Priani	6.1.1	CR 104487
	2008-08-24 S.Priani	6.1.1.x	F104161
	================================================================================*/
-- Performance related variables
	v_rowcnt		INTEGER		DEFAULT 0;
	v_begints		TIMESTAMP	DEFAULT fn_get_date;
	v_sqltext		VARCHAR2 (4000);
	v_sqltextstatic		VARCHAR2 (2000);
	v_flag			PLS_INTEGER	DEFAULT 0; --will open RC2
	v_subvolume_max		VARCHAR2 (8)	DEFAULT TO_CHAR(p_Date + 1, 'YYYYMMDD'); -- take the next day
	v_procname		VARCHAR2 (30)	DEFAULT 'LG_MSG_ARCHIVES_AH5';
	v_exceptionflg		INTEGER		DEFAULT 0;
	v_paramstxt		VARCHAR2 (4000)	:=	'p_DebugFlg=>' || p_debugflg || CHR (10)
		|| ', p_Date=>' || p_date || CHR (10)
		|| ', p_Partner=>' || p_partner || CHR (10)
		|| ', p_PackageNamer=>' || p_packagename || CHR (10)
		|| ', p_PackageVersion=>' || p_packagever || CHR (10)
		|| ', p_ProtocolName=>' || p_protocolname || CHR (10)
		|| ', p_ProtocolVersion=>' || p_protocolver || CHR (10)
		|| ', p_ProcessName=>' || p_processname || CHR (10)
		|| ', p_ProcessVersion=>' || p_processver || CHR (10)
		|| ', p_UserId=>' || p_userid || CHR (10)
		|| ', p_ArchiveDestination=>' || p_archdestination || CHR (10)
		;
BEGIN
	-- add into archive information table
	INSERT INTO lg_archiveinfo
		 (archiveid, actionname, userid,
			cutoffdate, partner,
			packagename,
			packagever,
			protocolname,
			protocolver,
			processname,
			processver,
			archdestination
		 )
	 VALUES (lg_archiveinfo_seq.NEXTVAL, 'Archive', p_userid,
			p_date, DECODE (p_partner, 'allO0oxyz', 'ALL', p_partner),
			DECODE (p_packagename, 'allO0oxyz', 'ALL', p_packagename),
			DECODE (p_packagever, 'allO0oxyz', 'ALL', p_packagever),
			DECODE (p_protocolname, 'allO0oxyz', 'ALL', p_protocolname),
			DECODE (p_protocolver, 'allO0oxyz', 'ALL', p_protocolver),
			DECODE (p_processname, 'allO0oxyz', 'ALL', p_processname),
			DECODE (p_processver, 'allO0oxyz', 'ALL', p_processver),
			p_archdestination
		 );

	if p_partner is NOT NULL AND p_partner <> 'allO0oxyz' then
		v_sqltextstatic := ' AND partnername = ''' || p_partner || '''';
		v_flag := v_flag +1;
	end if;

	if p_packagever is NOT NULL AND p_packagever <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND packagever = ''' || p_packagever || '''';
		v_flag := v_flag +1;
	end if;

	if p_packagename is NOT NULL AND p_packagename <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND packagename = ''' || p_packagename || '''';
		v_flag := v_flag +1;
	end if;

	if p_protocolname is NOT NULL AND p_protocolname <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND protocolname = ''' || p_protocolname || '''';
		v_flag := v_flag +1;
	end if;

	if p_protocolver is NOT NULL AND p_protocolver <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND protocolver = ''' || p_protocolver || '''';
		v_flag := v_flag +1;
	end if;

	if p_processname is NOT NULL AND p_processname <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND processname = ''' || p_processname || '''';
		v_flag := v_flag +1;
	end if;

	if p_processver is NOT NULL AND p_processver <> 'allO0oxyz' then
		v_sqltextstatic := v_sqltextstatic || ' AND processver = ''' || p_processver || '''';
		v_flag := v_flag +1;
	end if;

	IF v_flag = 0 THEN
	---list subvolume folders upto given Cutoff date
		OPEN RC1 FOR
			SELECT	m.subvolname
				,m.nrmediaid nonrep_mediaid
				,m.upmediaid msgstore_mediaid
				,'' VUID
				,count(m.vuid) vuid_count
			FROM	lg_msg_archive m
			WHERE	m.subvolname < v_subvolume_max
			GROUP BY m.subvolname
				,m.nrmediaid
				,m.upmediaid;

	ELSE
	---list vuids by given filters
		v_sqltext := 'SELECT m.subvolname, m.nrmediaid nonrep_mediaid, m.upmediaid msgstore_mediaid, m.vuid, 1 vuid_count '
			|| ' FROM lg_msg_archive m '
			|| ' WHERE m.subvolname < :IP_Limit '
			|| v_sqltextstatic;

		OPEN rc1 FOR v_sqltext USING v_subvolume_max;
	END IF;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
		p_returncd := 3;
	WHEN OTHERS THEN
		p_returncd := 1;
		db_procauditlogi_ah1 (v_procname,
				v_begints,
				v_paramstxt,
				p_debugflg,
				v_exceptionflg
				);
		raise_application_error (-20999, '20100 - LG_MSG_ARCHIVES_AH5:' || SQLERRM);
END lg_msg_archives_AH5;
/

-- LG_MSG_ARCHIVES_VUID.sql

CREATE OR REPLACE PROCEDURE         LG_MSG_ARCHIVES_VUID
(
    p_ReturnCd      out pls_integer
    , p_DebugFlg    in  pls_integer
    , RC1           out db_ref.RCT
    , p_VUID        in  VARCHAR2
) As
/*
================================================================================
Proc Name:      LG_MSG_ARCHIVES_VUID
Create Date:    01/11/02 01:52:49 PM
Author          S.HUTCHINSON
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
DB_ProcAuditLogI_Ah1('LG_MSG_ARCHIVES_VUID',v_beginTS,
'p_ReturnCd'||','||p_DebugFlg||','||'RC1'
||','||p_VUID);
End ProcAudit;
BEGIN
Open RC1 for
SELECT
    RTRIM(A.VUID) VUID
,   A.NRMEDIAID
,   RTRIM(M1.URI) NRURI
,   A.UPMEDIAID
,   RTRIM(M2.URI) UPURI
,   A.ROWTS
,   A.CREATEDATE
,   A.SUBVOLNAME
,   A. DIGSIGCERTID
,   A. ENCRYPTCERTID
--,   A. DIGSIGPUBLICCERTID
--,   A. ENCRYPTPUBLICCERTID
,   A. DIGSIGPRIKEYCERTID
,   A. ECRYPTPRIKEYCERTID
,   A.BDO
FROM
    LG_MSG_ARCHIVE A
    LEFT OUTER JOIN LG_MEDIA M1 ON A.NRMEDIAID = M1.MEDIAID
    LEFT OUTER JOIN LG_MEDIA M2 ON A.UPMEDIAID = M2.MEDIAID
WHERE
    A.VUID = p_VUID;
p_ReturnCd := 0;
procAudit;
End LG_MSG_ARCHIVES_VUID;
/


-- LG_MSG_ARCHIVEU_AH1.sql

CREATE OR REPLACE PROCEDURE LG_MSG_ARCHIVEU_AH1
        (
         p_ReturnCd     out pls_integer
        , p_DebugFlg    in pls_integer
--        , RC1           out db_ref.RCT
        ,p_VUID         in VARCHAR2
        ,p_NRMEDIAID    in NUMBER
        ,p_UPMEDIAID    in NUMBER
        ,p_SUBVOLNAME   in VARCHAR2
        ,p_ROWTS        in NUMBER
        ,p_DIGSIGCERTID in NUMBER
        ,p_ENCRYPTCERTID        in NUMBER
        ,p_DIGSIGPRIKEYCERTID   in NUMBER
        ,p_ECRYPTPRIKEYCERTID   in NUMBER
        ,p_businessid   IN VARCHAR2
        ,p_packagename  IN VARCHAR2
        ,p_protocolname IN VARCHAR2
        ,p_processname  IN VARCHAR2
        ,p_packagever   IN VARCHAR2
        ,p_protocolver  IN VARCHAR2
        ,p_processver   IN VARCHAR2
        ,p_BDO          IN BLOB
) as

/*
        ================================================================================
        Proc Name:       LG_MSG_ARCHIVEU_AH1
        Create Date:     2003-07-28 14:14:46
        Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------

        2004-03-02 S.Narain	 4.2.2  Archive Management Changes.
        2005-06-10 S.Narain      6FP2   BGTRK00049391
        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_MSG_ARCHIVEU_AH1';
v_ExceptionFlg    integer Default 0;
v_partnername     VARCHAR2(120) DEFAULT NULL;

v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_VUID=>'||p_VUID||CHR(10)
                ||', p_NRMEDIAID=>'||p_NRMEDIAID||CHR(10)
                ||', p_UPMEDIAID=>'||p_UPMEDIAID||CHR(10)
                ||', p_SUBVOLNAME=>'||p_SUBVOLNAME||CHR(10)
                ||', p_ROWTS=>'||p_ROWTS||CHR(10)
                ||', p_DIGSIGCERTID=>'||p_DIGSIGCERTID||CHR(10)
                ||', p_ENCRYPTCERTID=>'||p_ENCRYPTCERTID||CHR(10)
                ||', p_DIGSIGPRIKEYCERTID=>'||p_DIGSIGPRIKEYCERTID||CHR(10)
                ||', p_ECRYPTPRIKEYCERTID=>'||p_ECRYPTPRIKEYCERTID||CHR(10)
                ||', p_businessid=>'||p_businessid||CHR(10)
                ||', p_packagename=>'||p_packagename||CHR(10)
                ||', p_protocolname=>'||p_protocolname||CHR(10)
                ||', p_processname=>'||p_processname||CHR(10)
                ||', p_packagever=>'||p_packagever||CHR(10)
                ||', p_protocolver=>'||p_protocolver||CHR(10)
                ||', p_processver=>'||p_processver||CHR(10)  
                ;

v_RowTS      integer :=0;
v_ReturnCd   pls_integer :=0;
v_RC1	     db_ref.RCT;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        --100776
        BEGIN
        IF p_businessid IS NOT NULL THEN
            SELECT partnername INTO v_partnername
            FROM pr_partner partner, pr_busidentifier busid
            WHERE partner.partnerid = busid.partnerid
               AND busid.businessid = p_businessid;
        END IF;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
        null;
         END;
        v_RowCnt := 0;
	IF p_BDO IS NOT NULL AND dbms_lob.getlENGTH(p_BDO) > 0 THEN
	        Update LG_MSG_ARCHIVE
	        SET
	                NRMEDIAID                 = nvl(p_NRMEDIAID, NRMEDIAID)
	                ,      UPMEDIAID          = nvl(p_UPMEDIAID, UPMEDIAID)
	                ,      SUBVOLNAME         = nvl(p_SUBVOLNAME, SUBVOLNAME)
	                ,      ROWTS              = RowTS + 1
	                ,      UPDATEDATE         = v_BeginTS    --FN_GET_DATE()
	                ,      DIGSIGCERTID       = nvl(p_DIGSIGCERTID, DIGSIGCERTID)
	                ,      ENCRYPTCERTID      = nvl(p_ENCRYPTCERTID, ENCRYPTCERTID)
	                ,      DIGSIGPRIKEYCERTID = nvl(p_DIGSIGPRIKEYCERTID, DIGSIGPRIKEYCERTID)
	                ,      ECRYPTPRIKEYCERTID = nvl(p_ECRYPTPRIKEYCERTID, ECRYPTPRIKEYCERTID)
	                ,      partnername        = nvl(v_partnername, partnername)
	                ,      packagename        = nvl(p_packagename, packagename)
	                ,      protocolname       = nvl(p_protocolname, protocolname)
	                ,      processname        = nvl(p_processname, processname)
	                ,      packagever         = nvl(p_packagever, packagever)
	                ,      protocolver        = nvl(p_protocolver, protocolver)
	                ,      processver         = nvl(p_processver, processver)
	                ,      bdo		  = p_BDO
	        Where
	                VUID = p_VUID;
	ELSE
	        Update LG_MSG_ARCHIVE
	        SET
	                NRMEDIAID                 = nvl(p_NRMEDIAID, NRMEDIAID)
	                ,      UPMEDIAID          = nvl(p_UPMEDIAID, UPMEDIAID)
	                ,      SUBVOLNAME         = nvl(p_SUBVOLNAME, SUBVOLNAME)
	                ,      ROWTS              = RowTS + 1
	                ,      UPDATEDATE         = v_BeginTS    --FN_GET_DATE()
	                ,      DIGSIGCERTID       = nvl(p_DIGSIGCERTID, DIGSIGCERTID)
	                ,      ENCRYPTCERTID      = nvl(p_ENCRYPTCERTID, ENCRYPTCERTID)
	                ,      DIGSIGPRIKEYCERTID = nvl(p_DIGSIGPRIKEYCERTID, DIGSIGPRIKEYCERTID)
	                ,      ECRYPTPRIKEYCERTID = nvl(p_ECRYPTPRIKEYCERTID, ECRYPTPRIKEYCERTID)
	                ,      partnername        = nvl(v_partnername, partnername)
	                ,      packagename        = nvl(p_packagename, packagename)
	                ,      protocolname       = nvl(p_protocolname, protocolname)
	                ,      processname        = nvl(p_processname, processname)
	                ,      packagever         = nvl(p_packagever, packagever)
	                ,      protocolver        = nvl(p_protocolver, protocolver)
	                ,      processver         = nvl(p_processver, processver)
	        Where
	                VUID = p_VUID;
	END IF;

                 
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;

          if (p_DIGSIGCERTID is not null AND FN_CHK_CERT(p_DIGSIGCERTID)) then lg_cert_archivei(v_ReturnCd,1,v_RC1, p_VUID, p_DIGSIGCERTID);end if;
          if (p_ENCRYPTCERTID is not null AND FN_CHK_CERT(p_ENCRYPTCERTID)) then lg_cert_archivei(v_ReturnCd,1,v_RC1, p_VUID, p_ENCRYPTCERTID);end if;
          if (p_DIGSIGPRIKEYCERTID is not null AND FN_CHK_CERT(p_DIGSIGPRIKEYCERTID)) then lg_cert_archivei(v_ReturnCd,1,v_RC1, p_VUID, p_DIGSIGPRIKEYCERTID);end if;
          if (p_ECRYPTPRIKEYCERTID is not null AND FN_CHK_CERT(p_ECRYPTPRIKEYCERTID)) then lg_cert_archivei(v_ReturnCd,1,v_RC1, p_VUID, p_ECRYPTPRIKEYCERTID);end if;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        EXCEPTION
                WHEN TOO_MANY_ROWS Then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999, '20100 - LG_MSG_ARCHIVEU_AH1: Too Many Rows');
                WHEN ZeroUpdate then
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_MSG_ARCHIVEU_AH1: Zero Update');
                WHEN OTHERS then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR(-20999,'20100 - LG_MSG_ARCHIVEU_AH1:'||SQLERRM);
End LG_MSG_ARCHIVEU_AH1;
/


-- LG_MSG_RESTORES_AH1.sql

CREATE OR REPLACE PROCEDURE LG_MSG_RESTORES_AH1
	(
	p_ReturnCd		OUT	pls_INTEGER
	,p_DebugFlg		IN	pls_INTEGER
	,RC1			out	db_ref.RCT
	,RC2			out	db_ref.RCT
	,p_sortkey		IN	VARCHAR2
	,p_PageSizeNbr		IN	INTEGER
	,p_PageNbr		IN	INTEGER
	,p_FrDate		IN	VARCHAR2
	,p_ToDate		IN	VARCHAR2
	,p_partner		IN	VARCHAR2
	,p_packagename		IN	VARCHAR2
	,p_packagever		IN	VARCHAR2
	,p_protocolname		IN	VARCHAR2
	,p_protocolver		IN	VARCHAR2
	,p_processname		IN	VARCHAR2
	,p_processver		IN	VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:	LG_MSG_RESTORES_AH1
	Create Date:	2008-07-31 14:07:19
	Author:		S.Priani

				::::::::::::::::::::::::::::::
				M O D I F I C A T I O N
				::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE	NAME		VER	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2008-07-31 S.Priani	6.2	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
---------------------------------------------------------------------------
--			 M	A	I	N				--
---------------------------------------------------------------------------

	v_RowCnt	integer Default 0;
	v_BeginTS       timestamp Default FN_Get_Date;  -- Performance related variables
	v_ProcName	varchar2(30)	Default 'LG_MSG_RESTORES_AH1';
	v_ExceptionFlg	integer	Default 0;
	v_ExceptionCode	varchar2(30);
	v_PK_Value	INTEGER Default 0;
	v_SQLErrm	Varchar2(255) ;
	v_chkpt		Char(1) default '0';
	v_RowTS		INTEGER Default 0;

	v_where		VARCHAR2(1000);
	v_count		INTEGER		DEFAULT 0;
	v_flag		INTEGER		DEFAULT 0;
	v_sortKey	VARCHAR2(1000);
	v_sqlStmt	VARCHAR2(4000);
	v_subVolMax	VARCHAR2(10);

	msgvuidtab	CHAR65_TABLE	:= CHAR65_TABLE();
	TYPE char65Tab IS TABLE OF VARCHAR2(65);
	TYPE num10Tab  IS TABLE OF NUMBER(10);
	arr_vuid	char65Tab	:= char65Tab();
	arr_rnbr	num10Tab	:= num10Tab();

	v_ParamsTxt     VARCHAR2(4000) Default
	 		'p_Sortkey=>'||p_Sortkey||CHR(10)
	 		|| 'p_FrDate=>'||p_FrDate||CHR(10)
	 		|| 'p_ToDate=>'||p_ToDate||CHR(10)
	 		|| 'p_Partner=>'||p_Partner||CHR(10)
	 		|| 'p_PackageName=>'||p_PackageName||CHR(10)
	 		|| 'p_PackageVer=>'||p_PackageVer||CHR(10)
	 		|| 'p_ProtocolName=>'||p_ProtocolName||CHR(10)
	 		|| 'p_ProtocolVer=>'||p_ProtocolVer||CHR(10)
	 		|| 'p_ProcessName=>'||p_ProcessName||CHR(10)
	 		|| 'p_ProcessVer=>'||p_ProcessVer||CHR(10)
		;

BEGIN
	if p_partner is NOT NULL AND p_partner <> 'allO0oxyz' then
		 v_where := ' AND m.partnername = ''' || p_partner || '''';
		 v_flag := v_flag +1;
	end if;  
  
	if p_packagever is NOT NULL AND p_packagever <> 'allO0oxyz' then
		 v_where := v_where || ' AND m.packagever = ''' || p_packagever || '''';
		 v_flag := v_flag +1;
	end if;
  
	if p_packagename is NOT NULL AND p_packagename <> 'allO0oxyz' then
  		 v_where := v_where || ' AND m.packagename = ''' || p_packagename || '''';
		 v_flag := v_flag +1;
	end if;
  
	if p_protocolname is NOT NULL AND p_protocolname <> 'allO0oxyz' then
  		 v_where := v_where || ' AND m.protocolname = ''' || p_protocolname || '''';
		 v_flag := v_flag +1;
	end if;

	if p_protocolver is NOT NULL AND p_protocolver <> 'allO0oxyz' then
  		 v_where := v_where || ' AND m.protocolver = ''' || p_protocolver || '''';
		 v_flag := v_flag +1;
	end if;  
  
	if p_processname is NOT NULL AND p_processname <> 'allO0oxyz' then
  		 v_where := v_where || ' AND m.processname = '''  || p_processname || '''';
		 v_flag := v_flag +1;
	end if;  
  
	if p_processver is NOT NULL AND p_processver <> 'allO0oxyz' then
  		 v_where := v_where || ' AND m.processver = ''' || p_processver || '''';
		 v_flag := v_flag +1;
	end if;

	--- subolname has data in the format yyyymmdd/a, need to convert the p_ToDate to nextday
	v_SubVolMax := TO_CHAR(TO_DATE(p_ToDate, 'YYYYMMDD') + 1, 'YYYYMMDD');
	v_Chkpt:='1';
	v_SQLstmt := 'SELECT m.vuid '
			|| ' FROM lg_msg_restore m '
			|| ' WHERE m.subvolname BETWEEN :FrSubVol AND :ToSubVol '
			|| v_where
	;
	-------------------------
	-- Total row count Cursor
	-------------------------
	IF COALESCE(p_pageNbr,0) in (0,1) THEN
		v_Chkpt :='2';
		EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM ('||v_Sqlstmt ||') ' INTO v_Count USING p_FrDate, v_SubVolMax;
		OPEN RC2 For
			SELECT v_count TotalRowsNbr FROM DUAL;
	END IF;

	IF p_SortKey IS NOT NULL THEN
		 v_Chkpt :='4';
		 v_SortKey := UPPER(p_SortKey);
		 v_SortKey := REPLACE(v_sortKey, 'CREATEDATE', 'm.CREATEDATE');
		 v_sqlstmt := v_sqlstmt ||' order by ' ||v_sortkey; 
	END IF;	

	---------------
	-- Paging logic
	---------------
	IF p_PageSizeNbr >= 0 THEN
		v_Chkpt :='5';
		v_SQLstmt := FN_PAGING_LOGIC(p_pageNbr, p_pageSizeNbr, v_Sqlstmt);
	END IF;

	EXECUTE IMMEDIATE v_SQLStmt BULK COLLECT INTO arr_rnbr, arr_VUID USING p_FrDate, v_SubVolMax;

	FOR i IN 1..arr_VUID.count LOOP
		MsgVuidTab.EXTEND;
		MsgVuidTab(i) := char65_rec(arr_rnbr(i), arr_VUID(i));
	END LOOP;

	v_Chkpt :='5';
	IF RC1%isOpen THEN Close RC1; END IF;
		Open RC1 FOR
			SELECT m.vuid, m.nrmediaid nonrep_mediaid, m.upmediaid msgstore_mediaid, m.subvolname 
				 , m.digsigcertid, m.encryptcertid, m.digsigprikeycertid, m.ecryptprikeycertid 
				 , m.partnername, m.packagename, m.packagever, m.protocolname, m.protocolver, m.processname, m.processver 
				 , m.createdate, m.updatedate 
				 , RTRIM(enc.certname) certname_encr
				 , RTRIM(sig.certname) certname_sign
				 , RTRIM(encpk.certname) certname_encrpk
				 , RTRIM(sigpk.certname) certname_signpk
			FROM	lg_msg_restore m 
			INNER JOIN TABLE(CAST(MsgVuidTab AS CHAR65_TABLE)) qq ON qq.ID = m.vuid
			LEFT OUTER JOIN lg_cert_archive enc ON enc.archivecertid = m.encryptcertid
			LEFT OUTER JOIN lg_cert_archive sig ON sig.archivecertid = m.digsigcertid
			LEFT OUTER JOIN lg_cert_archive encpk ON encpk.archivecertid = m.ecryptprikeycertid
			LEFT OUTER JOIN lg_cert_archive sigpk ON sigpk.archivecertid = m.digsigprikeycertid
		;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END LG_MSG_RESTORES_AH1;
/


-- LG_MSG_RESTORES_VUID.sql

CREATE OR REPLACE PROCEDURE  LG_MSG_RESTORES_VUID
(
    p_ReturnCd      out pls_integer
    , p_DebugFlg    in  pls_integer
    , RC1           out db_ref.RCT
    , p_VUID        in  VARCHAR2
) As
/*
================================================================================
Proc Name:      LG_MSG_RESTORES_VUID
Create Date:    30/10/08
Author          S.Priani
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
DB_ProcAuditLogI_Ah1('LG_MSG_ARCHIVES_VUID',v_beginTS,
'p_ReturnCd'||','||p_DebugFlg||','||'RC1'
||','||p_VUID);
End ProcAudit;
BEGIN
Open RC1 for
	SELECT
	  RTRIM(m.VUID) VUID
	,	m.NRMEDIAID
	,	m.UPMEDIAID
	,	m.ROWTS
	,	m.CREATEDATE
	,	m.SUBVOLNAME
	,	m.DIGSIGCERTID
	,	m.ENCRYPTCERTID
	,	m.DIGSIGPRIKEYCERTID
	,	m.ECRYPTPRIKEYCERTID
	,	m.partnername, m.packagename, m.packagever, m.protocolname, m.protocolver, m.processname, m.processver 
	,	m.BDO
	FROM	LG_MSG_RESTORE m
	WHERE	m.VUID = p_VUID
	;
p_ReturnCd := 0;
procAudit;
End LG_MSG_RESTORES_VUID;
/


-- LG_PROCESSSUMMARY_ASI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_PROCESSSUMMARY_ASI_AH1
    (p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,p_INITBUSINESSID in VARCHAR2
    ,p_INITASMESSAGEID in VARCHAR2
    ,p_TOBUSINESSID in VARCHAR2
    ,p_RCVDOCTS in DATE
    ,p_DOCCNT in NUMBER
    ,p_RCVPROCESSCNT in NUMBER
    ,p_MDNCNT in NUMBER
    ,p_DISPWARNINGCNT in NUMBER
    ,p_DISPERRORCNT in NUMBER
    ,p_DISPFAILURECNT in NUMBER
    ,p_SuccessCnt in NUMBER
    )
AS
/*
    ================================================================================
    Proc Name:       LG_PROCESSSUMMARY_ASI_AH1
    Create Date:     2003-05-02 11:29:39
    Author:          V Goel

    Description:    This procedure summarizes the AS process as well as exception counts

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATe(); -- Performance related variables
v_ProcName        varchar2(30) := 'LG_PROCESSSUMMARY_ASI_AH1';
v_ExceptionFlg    integer := 0;
v_ASPROCESSSUMMARYID pls_integer;
zeroUpdate          Exception;
v_ParamsTxt       varchar2(4000) :=
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
                ||', p_INITASMESSAGEID=>'||p_INITASMESSAGEID||CHR(10)
                ||', p_TOBUSINESSID=>'||p_TOBUSINESSID||CHR(10)
                ||', p_RCVDOCTS=>'||p_RCVDOCTS||CHR(10)
                ||', p_DOCCNT=>'||p_DOCCNT||CHR(10)
                ||', p_RCVPROCESSCNT=>'||p_RCVPROCESSCNT||CHR(10)
                ||', p_MDNCNT=>'||p_MDNCNT||CHR(10)
                ||', p_DISPWARNINGCNT=>'||p_DISPWARNINGCNT||CHR(10)
                ||', p_DISPERRORCNT=>'||p_DISPERRORCNT||CHR(10)
                ||', p_DISPFAILURECNT=>'||p_DISPFAILURECNT||CHR(10)
                ||', p_SuccessCnt=>'||p_SuccessCnt||CHR(10)
                ;
--

Procedure Get_AS_SummaryID is
Begin
    SELECT ASPROCESSSUMMARYID INTO v_ASPROCESSSUMMARYID
    FROM LG_ProcessSummary_as_XREF
    WHERE INITBUSINESSID  = p_INITBUSINESSID
    AND   INITASMESSAGEID = p_INITASMESSAGEID
--    AND   TOBUSINESSID    = p_TOBUSINESSID
    ;
    EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND Get_AS_SummaryID, '||v_ParamsTxt, p_DebugFlg, 0);
            RAISE_APPLICATION_ERROR (-20999,'20000 LG_ProcessSummary_as_XREFI Get_AS_SummaryID ' ||SQLERRM);
        When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR (-20999,'20000 LG_ProcessSummary_as_XREFI Get_AS_SummaryID ' ||SQLERRM);
End Get_AS_SummaryID;


Procedure LG_ProcessSummary_as_XREFI is
BEGIN
    Insert into LG_ProcessSummary_as_XREF(
            ASPROCESSSUMMARYID
            ,INITBUSINESSID
            ,INITASMESSAGEID
            ,TOBUSINESSID
    ) Values (
            LG_ProcessSummary_as_XREF_SEQ.nextval
            ,nvl(p_INITBUSINESSID, -1)
            ,p_INITASMESSAGEID
            ,nvl(p_TOBUSINESSID, -1)
    ) Returning ASPROCESSSUMMARYID INTO v_ASPROCESSSUMMARYID
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND LG_ProcessSummary_as_XREFI, '||v_ParamsTxt, p_DebugFlg, 0);
            RAISE_APPLICATION_ERROR (-20999,'20000 LG_ProcessSummary_as_XREFI' || SQLCODE|| ' '||SQLERRM);
        When DUP_VAL_ON_INDEX then
            Get_AS_SummaryID;

        When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR (-20999,'20000 LG_ProcessSummary_as_XREFI' || SQLCODE|| ' '||SQLERRM);
END LG_PROCESSSUMMARY_AS_XREFI;
--
Procedure Upd_LG_Processsummary_AS is
Begin
     Update LG_Processsummary_AS S
        Set
                  s.DOCCNT          =p_DOCCNT       +  s.DOCCNT
            ,   s.RCVPROCESSCNT   = p_RCVPROCESSCNT  +  s.RCVPROCESSCNT
            ,   s.MDNCNT          = p_MDNCNT         +  s.MDNCNT
            ,   s.DISPWARNINGCNT  = p_DISPWARNINGCNT +  s.DISPWARNINGCNT
            ,   s.DISPERRORCNT    = p_DISPERRORCNT   +  s.DISPERRORCNT
            ,   s.DISPFAILURECNT  = p_DISPFAILURECNT +  s.DISPFAILURECNT
            ,   s.SUCCESSCNT      = p_SuccessCnt     +  s.SUCCESSCNT
            ,   s.Updatedate      = v_BeginTS
        Where (S.ASProcessSummaryID = v_ASProcessSummaryID and  S.dayHour = Trunc(P_RCVDOCTS, 'HH24'));
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate; end if;
        p_ReturnCd := 0;

        EXCEPTION
            WHEN ZeroUpdate then
                p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' Upd_LG_Processsummary_AS '||SQLERRM);

End Upd_LG_Processsummary_AS;
--
Procedure  Upd_LG_Processsummary_AS_MI is
Begin
     Update  LG_Processsummary_AS_MI s
        Set
            s.DOCCNT          = p_DOCCNT         +  s.DOCCNT
            ,   s.RCVPROCESSCNT   = p_RCVPROCESSCNT  +  s.RCVPROCESSCNT
            ,   s.MDNCNT          = p_MDNCNT         +  s.MDNCNT
            ,   s.DISPWARNINGCNT  = p_DISPWARNINGCNT +  s.DISPWARNINGCNT
            ,   s.DISPERRORCNT    = p_DISPERRORCNT   +  s.DISPERRORCNT
            ,   s.DISPFAILURECNT  = p_DISPFAILURECNT +  s.DISPFAILURECNT
            ,   s.Updatedate      = v_BeginTS
        Where (S.ASProcessSummaryID = v_ASProcessSummaryID and  S.dayHourMI =  Trunc(P_RCVDOCTS, 'HH24'));
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate; end if;
        p_ReturnCd := 0;

        EXCEPTION
            WHEN ZeroUpdate then
                p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' Upd_LG_Processsummary '||SQLERRM);

End  Upd_LG_Processsummary_AS_MI;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    LG_PROCESSSUMMARY_AS_XREFI;
    Upd_LG_Processsummary_AS;
    BEGIN
         if (v_RowCnt = 0) then
            INSERT INTO LG_Processsummary_AS
              (
                 ASPROCESSSUMMARYID
                ,DAYHOUR
                ,DOCCNT
                ,RCVPROCESSCNT
                ,MDNCNT
                ,DISPWARNINGCNT
                ,DISPERRORCNT
                ,DISPFAILURECNT
                ,SuccessCnt
              )
            Values
              (
                 v_ASPROCESSSUMMARYID
                ,Trunc(P_RCVDOCTS, 'HH24')
                ,p_DOCCNT
                ,p_RCVPROCESSCNT
                ,p_MDNCNT
                ,p_DISPWARNINGCNT
                ,p_DISPERRORCNT
                ,p_DISPFAILURECNT
                ,p_SuccessCnt
             );
         end if;
         p_ReturnCd := 0;
         DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 0);

         EXCEPTION
            When Dup_val_on_index Then
               Upd_LG_Processsummary_AS;
            When Others Then
               p_ReturnCd := 1;
               Raise_application_error (-20999,v_Procname||SQLERRM);
    END;

    Upd_LG_Processsummary_AS_MI;

    BEGIN
         if (v_RowCnt = 0) then
            INSERT INTO LG_Processsummary_AS_MI
              (
                 ASPROCESSSUMMARYID
                ,DAYHOURMI
                ,DOCCNT
                ,RCVPROCESSCNT
                ,MDNCNT
                ,DISPWARNINGCNT
                ,DISPERRORCNT
                ,DISPFAILURECNT
              )
           Values
              (
                 v_ASPROCESSSUMMARYID
                ,Trunc(P_RCVDOCTS, 'HH24')
                ,p_DOCCNT
                ,p_RCVPROCESSCNT
                ,p_MDNCNT
                ,p_DISPWARNINGCNT
                ,p_DISPERRORCNT
                ,p_DISPFAILURECNT
              );
         end if;
         p_ReturnCd := 0;
         DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 0);

         EXCEPTION
            When Dup_val_on_index Then
               Upd_LG_Processsummary_AS_MI;
            When Others Then
               p_ReturnCd := 1;
               Raise_application_error (-20999,v_Procname||SQLERRM);
    END;
END LG_PROCESSSUMMARY_ASI_AH1;
/


-- Lg_PROCESSSUMMARY_RNI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_ProcessSummary_RNI_AH1
(
    p_ReturnCd              out integer,
    p_DebugFlg              in pls_integer
    -- RN
    ,p_ProcessSummaryID     in pls_integer
    ,P_RCVDOCTS             in DATE
    ,p_DOCCNT               in pls_integer
    ,p_RCVPROCESSCNT        in pls_integer
    ,p_ReqACTIONCNT         in pls_integer
    ,p_RespACTIONCNT        in pls_integer
    ,p_ReqSIGNALCNT         in pls_integer
    ,p_respSIGNALCNT        in pls_integer
    ,p_RequestDURATIONSUM   in pls_integer
    ,p_ResponseDURATIONSUM  in pls_integer
    ,p_RAReqDURATIONSUM     in pls_integer
    ,p_RARespDURATIONSUM    in pls_integer
    ,p_AACNT                in pls_integer
    ,p_GECNT                in pls_integer
    ,p_RACNT                in pls_integer
    ,p_RAECNT               in pls_integer
    ,p_AAECNT               in pls_integer
    ,p_TIMETOPERFORM        in pls_integer
    ,p_UnknownCnt           in pls_integer
    ,p_ref_ProcessSummaryID in pls_integer  -- for zeroA1 count
    ,p_ref_RcvDocTS         in date         -- for zeroA1 count
) AS
/*
================================================================================
Proc Name:        LG_ProcessSummary_RNI_AH1
Create Date:     2003-02-17
Author:          V Goel
================================================================================
Return Code : 0 = OK,
              1 = Not Good,
              2 = Dup,
              3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt     integer     :=0;
v_BeginTS    timestamp := FN_Get_Date; -- Performance related variables
v_pk_value   pls_integer := 0;
v_summaryid  lg_sum_xref_Lnk.summaryid%type ;
v_ProcName        varchar2(30) := 'LG_ProcessSummary_RNI_AH1';
v_ExceptionFlg    integer := 0;
v_rcode      pls_integer;
--
zeroUpdate  Exception;
--
-- Procedures
--

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_ProcessSummary_RNI_AH1',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    --
    ||','||P_RCVDOCTS
    ||','||p_DOCCNT
    ||','||p_RCVPROCESSCNT
    ||','||p_ReqACTIONCNT
    ||','||p_RespACTIONCNT
    ||','||p_ReqSIGNALCNT
    ||','||p_RespSIGNALCNT
    ||','||p_RequestDURATIONSUM
    ||','||p_ResponseDURATIONSUM
    ||','||p_RAReqDURATIONSUM
    ||','||p_RARespDURATIONSUM
    ||','||p_AACNT
    ||','||p_GECNT
    ||','||p_RACNT
    ||','||p_RAECNT
    ||','||p_TIMETOPERFORM
    --||';'||P_PROVSESSIONID
    ||','||p_ref_ProcessSummaryID
    ||','||p_UnknownCnt
    , p_DebugFlg
    );
End ProcAudit;
--

Procedure Do_0A1A is
begin
--
-- Lg_summary_rn
--

  Update  lg_processSummary_rn S
        Set
            s.zeroA1CNT = nvl(s.zeroA1CNT,0) + 1
          , s.Updatedate = v_BeginTS
        where (S.ProcessSummaryID = p_ref_ProcessSummaryID and  S.dayHour = Trunc(P_Ref_RCVDOCTS, 'HH24'));
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate; end if;
        p_ReturnCd := 0;

        EXCEPTION
            WHEN ZeroUpdate then
                p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' Upd_LG_Processsummary '||SQLERRM);
END  Do_0A1A;

Procedure Do_0A1B is
begin
--
-- Lg_summary_rn_MI
--
    update lg_Processsummary_rn_MI S
        set
            s.zeroA1CNT = nvl(s.zeroA1CNT,0) + 1
           ,s.Updatedate = v_BeginTS
        where (S.ProcessSummaryID = p_ref_ProcessSummaryID and  S.dayHourMI = FN_tenMinuteDate(P_Ref_RCVDOCTS));
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate; end if;
        p_ReturnCd := 0;

        EXCEPTION
            WHEN ZeroUpdate then
                p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' Upd_LG_Processsummary '||SQLERRM);

end Do_0A1B;
--
Procedure Upd_lg_summary_rn_mi is
Begin
     Update  lg_Processsummary_rn_MI S
        Set
                s.DOCCNT              = s.DOCCNT              +   p_DOCCNT
           ,    s.RCVPROCESSCNT       = s.RCVPROCESSCNT       +   p_RCVPROCESSCNT
           ,    s.ReqActionCNT        = s.ReqActionCNT        +   p_ReqActionCNT
           ,    s.RespActionCNT       = s.RespActionCNT       +   p_RespActionCNT
           ,    s.ReqSignalCNT        = s.ReqSignalCNT        +   p_ReqSignalCNT
           ,    s.RespSignalCNT       = s.RespSignalCNT       +   p_RespSignalCNT
           ,    s.RequestDurationSUM  = s.RequestDurationSUM  + p_RequestDurationSUM
           ,    s.ResponseDurationSUM = s.ResponseDurationSUM + p_ResponseDurationSUM
           ,    s.RAReqDurationSUM    = s.RAReqDurationSUM    + p_RAReqDurationSUM
           ,    s.RARespDurationSUM   = s.RARespDurationSUM   + p_RARespDurationSUM
           ,    s.AACNT               = s.AACNT               +   p_AACNT
           ,    s.GECNT               = s.GECNT               +   p_GECNT
           ,    s.RAECNT              = s.RAECNT              +   p_RAECNT
           ,    s.TIMETOPERFORMSum    = s.TIMETOPERFORMSum    + p_TIMETOPERFORM
           ,    s.UnknownCnt          = s.UnknownCnt          +   p_UnknownCnt
           ,    s.Updatedate = v_BeginTS
         where (S.ProcessSummaryID = p_ProcessSummaryID and  S.dayHourMi = FN_tenMinuteDate(P_RCVDOCTS));
         v_RowCnt := SQL%rowcount;
         if (v_RowCnt = 0) then  raise ZeroUpdate; end if;
         p_ReturnCd := 0;

        EXCEPTION
            WHEN ZeroUpdate then
                p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' Upd_LG_Processsummary '||SQLERRM);

End Upd_lg_summary_rn_mi;

Procedure  Upd_lg_processSummary_rn is
Begin
     Update  lg_processSummary_rn S
        Set
                s.DOCCNT            = s.DOCCNT          +   p_DOCCNT
           ,    s.RCVPROCESSCNT     = s.RCVPROCESSCNT   +   p_RCVPROCESSCNT
           ,    s.ReqActionCNT        = s.ReqActionCNT        +   p_ReqActionCNT
           ,    s.RespActionCNT       = s.RespActionCNT       +   p_RespActionCNT
           ,    s.ReqSignalCNT        = s.ReqSignalCNT        +   p_ReqSignalCNT
           ,    s.RespSignalCNT       = s.RespSignalCNT       +   p_RespSignalCNT
           ,    s.RequestDurationSUM  = s.RequestDurationSUM  + p_RequestDurationSUM
           ,    s.ResponseDurationSUM = s.ResponseDurationSUM + p_ResponseDurationSUM
           ,    s.RAReqDurationSUM    = s.RAReqDurationSUM    + p_RAReqDurationSUM
           ,    s.RARespDurationSUM   = s.RARespDurationSUM   + p_RARespDurationSUM
           ,    s.AACNT             = s.AACNT           +   p_AACNT
           ,    s.GECNT             = s.GECNT           +   p_GECNT
           ,    s.RAECNT            = s.RAECNT          +   p_RAECNT
           ,    s.TIMETOPERFORMSum = s.TIMETOPERFORMSum + p_TIMETOPERFORM
           ,    s.UnknownCnt        = s.UnknownCnt      +   p_UnknownCnt
           ,    s.Updatedate = v_BeginTS
        where  (S.ProcessSummaryID = p_ProcessSummaryID and  S.dayHour = Trunc(P_RCVDOCTS, 'HH24'));
        v_RowCnt := SQL%rowcount;
        if (v_RowCnt = 0) then  raise ZeroUpdate; end if;
        p_ReturnCd := 0;

        EXCEPTION
            WHEN ZeroUpdate then
                p_ReturnCd := 2;
            WHEN OTHERS then
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    Raise_application_error (-20999,20001||v_ProcName||' Upd_LG_Processsummary '||SQLERRM);
END Upd_lg_processSummary_rn;

--------------------------------------------------------------------------
--                 M  A  I  N
--------------------------------------------------------------------------
BEGIN
    p_ReturnCd := 0;
    Upd_lg_processSummary_rn;
    BEGIN
       if (v_RowCnt = 0) then
          INSERT INTO lg_processSummary_rn s
            (
                    s.ProcessSummaryID
               ,    s.DAYHOUR
               ,    s.DOCCNT
               ,    s.RCVPROCESSCNT
               ,    s.ReqActionCNT
               ,    s.RespActionCNT
               ,    s.ReqSignalCNT
               ,    s.RespSignalCNT
               ,    s.RequestDurationSUM
               ,    s.ResponseDurationSUM
               ,    s.RAReqDurationSUM
               ,    s.RARespDurationSUM
               ,    s.AACNT
               ,    s.GECNT
               ,    s.RAECNT
               ,    s.AAECNT
               ,    s.TIMETOPERFORMSum
               ,    s.UnknownCnt
            )
        Values
            (
                    p_ProcessSummaryID
               ,    Trunc(P_RCVDOCTS, 'HH24')
               ,    p_DOCCNT
               ,    p_RCVPROCESSCNT
               ,    p_ReqActionCNT
               ,    p_RespActionCNT
               ,    p_ReqSignalCNT
               ,    p_RespSignalCNT
               ,    p_RequestDurationSUM
               ,    p_ResponseDurationSUM
               ,    p_RAReqDurationSUM
               ,    p_RARespDurationSUM
               ,    p_AACNT
               ,    p_GECNT
               ,    p_RAECNT
               ,    p_AAECNT
               ,    p_TIMETOPERFORM
               ,    p_UnknownCnt
            );
        end if;
        p_ReturnCd := 0;
        ProcAudit;

       EXCEPTION
          When Dup_val_on_index Then
             Upd_lg_processSummary_rn;
          When Others Then
             p_ReturnCd := 1;
             Raise_application_error (-20999,v_Procname||SQLERRM);
    END;

   Upd_Lg_Summary_rn_MI;
   Begin
      if (v_RowCnt = 0) then
          INSERT INTO lg_Processsummary_rn_MI S
           (
                    s.ProcessSummaryID
               ,    s.DAYHOURmi
               ,    s.DOCCNT
               ,    s.RCVPROCESSCNT
               ,    s.ReqActionCNT
               ,    s.RespActionCNT
               ,    s.ReqSignalCNT
               ,    s.RespSignalCNT
               ,    s.RequestDurationSUM
               ,    s.ResponseDurationSUM
               ,    s.RAReqDurationSUM
               ,    s.RARespDurationSUM
               ,    s.AACNT
               ,    s.GECNT
               ,    s.RAECNT
               ,    s.AAECNT
               ,    s.TIMETOPERFORMSum
               ,    s.UnknownCnt
               )
           Values
               (
                    p_ProcessSummaryID
               ,    FN_tenMinuteDate(P_RCVDOCTS)
               ,    p_DOCCNT
               ,    p_RCVPROCESSCNT
               ,    p_ReqActionCNT
               ,    p_RespActionCNT
               ,    p_ReqSignalCNT
               ,    p_RespSignalCNT
               ,    p_RequestDurationSUM
               ,    p_ResponseDurationSUM
               ,    p_RAReqDurationSUM
               ,    p_RARespDurationSUM
               ,    p_AACNT
               ,    p_GECNT
               ,    p_RAECNT
               ,    p_AAECNT
               ,    p_TIMETOPERFORM
               ,    p_UnknownCnt
              );
        end if;

        p_ReturnCd := 0;
        ProcAudit;

        EXCEPTION
          When Dup_val_on_index Then
             Upd_Lg_Summary_rn_MI;
          When Others Then
             p_ReturnCd := 1;
             Raise_application_error (-20999,v_Procname||SQLERRM);
    END;

--
-- If ref_ProcessSummaryID is provided then do the following processing
--
    IF p_ref_ProcessSummaryID  is not null and p_ref_RcvDocTS is not null then
        Do_0A1A;
        begin
         if (v_RowCnt = 0) then
          INSERT INTO lg_processSummary_rn s
             (
                 s.ProcessSummaryID
               , s.DAYHOUR
               , s.zeroA1CNT
             )
         Values
             (
                 p_ref_ProcessSummaryID
               , Trunc(P_Ref_RCVDOCTS, 'HH24')
               , 1
             );
         end if;

         EXCEPTION
           When Dup_val_on_index Then
              Do_0A1A;
           When Others Then
             p_ReturnCd := 1;
             Raise_application_error (-20999,v_Procname||SQLERRM);
        end;

        Do_0A1B;
        begin
          if (v_Rowcnt = 0) then
            INSERT INTO lg_Processsummary_rn_MI S
             (
                s.ProcessSummaryID
               ,s.DAYHOURMI
               ,s.zeroA1CNT
             )
           Values (
               p_ref_ProcessSummaryID
               ,FN_tenMinuteDate(P_Ref_RCVDOCTS)
               ,1
             );
          end if;

          EXCEPTION
           When Dup_val_on_index Then
             Do_0A1B;
           When Others Then
           p_ReturnCd := 1;
           Raise_application_error (-20999,v_Procname||SQLERRM);
       end;

    END IF;
    ProcAudit;


END LG_ProcessSummary_RNI_AH1;
/


-- LG_PROCESSSUMMARY_XREFI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_ProcessSummary_XREFI_ah1
    (p_ReturnCd         out integer
    ,p_DebugFlg         in pls_integer
    ,p_PROCESSSUMMARYID out pls_integer
    ,p_INITPARTNERID    in pls_integer
    ,p_TOPARTNERID      in pls_integer
    ,p_INITBUSIDTYPE    in VARCHAR2
    ,p_PROTOCOLNAME     in VARCHAR2
    ,p_PROTOCOLVER      in VARCHAR2
    ,p_PROCESSCD        in VARCHAR2
    ,p_PROCESSVER       in VARCHAR2
    ,p_IPADDRESS        in VARCHAR2
    ,p_DESTTYPEID       in pls_integer
    ,p_SYNCHRONOUSFLG    in pls_integer
     ) as

/*
    ================================================================================
    Proc Name:       LG_ProcessSummary_XREFI_ah1
    Create Date:     2003-02-18 07:56:59
    Author:          V Goel
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
v_PROCESSSUMMARYID  LG_ProcessSummary_XREF.PROCESSSUMMARYID%type;
v_parameter varchar2(4000);

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_ProcessSummary_XREFI_ah1',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||p_INITPARTNERID
    ||','||p_TOPARTNERID
    ||','||p_INITBUSIDTYPE
    ||','||p_PROTOCOLNAME
    ||','||p_PROTOCOLVER
    ||','||p_PROCESSCD
    ||','||p_PROCESSVER
    ||','||p_IPADDRESS
    ||','||p_DESTTYPEID
    ||','||p_SYNCHRONOUSFLG
    );
End ProcAudit;
--
Procedure Output_parameters is
Begin
    v_parameter := (
      ' INITPARTNERID '||p_INITPARTNERID
    ||' TOPARTNERID '  ||p_TOPARTNERID
    ||' INITBUSIDTYPE '||p_INITBUSIDTYPE
    ||' PROTOCOLNAME ' ||p_PROTOCOLNAME
    ||' PROTOCOLVER '  ||p_PROTOCOLVER
    ||' PROCESSCD '    ||p_PROCESSCD
    ||' PROCESSVER'    ||p_PROCESSVER
    ||' IPADDRESS '    ||p_IPADDRESS
--    ||' SUMMARYID '    ||p_SUMMARYID
    ||' DESTTYPEID '   ||p_DESTTYPEID
    ||' SYNCHRONOUSFLG '||p_SYNCHRONOUSFLG
    );
End Output_parameters;
--
Procedure Get_ProcessSummaryID is
Begin
    select ProcessSummaryID Into P_ProcessSummaryID
    from LG_PROCESSSUMMARY_XREF
    where
        INITPARTNERID = p_INITPARTNERID
    AND TOPARTNERID   = p_TOPARTNERID
--    AND INITBUSIDTYPE = p_INITBUSIDTYPE
    AND PROTOCOLNAME  = p_PROTOCOLNAME
    AND PROTOCOLVER   = p_PROTOCOLVER
    AND PROCESSCD     = p_PROCESSCD
    AND PROCESSVER    = p_PROCESSVER
    AND IPADDRESS     =  coalesce(p_IPADDRESS,'-1')
    AND DESTTYPEID    = p_DESTTYPEID
    AND SYNCHRONOUSFLG = nvl(p_SYNCHRONOUSFLG, 0);
Exception
    when NO_DATA_FOUND then
        p_ReturnCd := 3;
        Output_parameters;
      raise_application_error(-20999,20001||' LG_ProcessSummary_XREFI_ah1 - Get_ProcessSummaryID '
                                          ||SQLERRM||v_parameter);
    when others then
        Output_parameters;
        p_ReturnCd := 1;
        raise_application_error(-20999,20002||' LG_ProcessSummary_XREFI_ah1 - Get_ProcessSummaryID '
                                            ||SQLERRM||v_parameter);

end Get_ProcessSummaryID ;

---------------------------------------------------------------
--                           M A I N
---------------------------------------------------------------
BEGIN
    Insert into LG_ProcessSummary_XREF(
    PROCESSSUMMARYID
    ,    INITPARTNERID
    ,    TOPARTNERID
    ,    INITBUSIDTYPE
    ,    PROTOCOLNAME
    ,    PROTOCOLVER
    ,    PROCESSCD
    ,    PROCESSVER
    ,    IPADDRESS
    ,    DESTTYPEID
    ,    SYNCHRONOUSFLG
    ) Values (
    LG_PROCESSSUMMARY_XREF_SEQ.nextval
    ,    p_INITPARTNERID
    ,    p_TOPARTNERID
    ,    p_INITBUSIDTYPE
    ,    p_PROTOCOLNAME
    ,    p_PROTOCOLVER
    ,    p_PROCESSCD
    ,    p_PROCESSVER
    ,    coalesce(p_IPADDRESS,'-1')
    ,    p_DESTTYPEID
    ,    nvl(p_SYNCHRONOUSFLG, 0)
    ) Returning PROCESSSUMMARYID into p_PROCESSSUMMARYID;
ProcAudit;
EXCEPTION
    When NO_DATA_FOUND Then Null;
    When Dup_val_on_index then
    Get_ProcessSummaryID;
    When Others Then
    ProcAudit;
    Raise_application_error (-20999,'20000 LG_ProcessSummary_XREFI_ah1' ||SQLERRM);
    p_ReturnCd := 1;
END LG_ProcessSummary_XREFI_ah1;
/


-- LG_REQRESPI.sql

Create or Replace Procedure LG_REQRESPI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_REQACTIVITYID in VARCHAR2
	,P_RESPACTIVITYID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_REQRESPI
	Create Date:     2006-05-25 12:50:29
	Author:          S.Nair

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2006-05-25 S.Nair        6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_REQRESPI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_REQACTIVITYID=>'||p_REQACTIVITYID||CHR(10)
		||', P_RESPACTIVITYID=>'||P_RESPACTIVITYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_REQRESP(
		REQACTIVITYID
		,RESPACTIVITYID
	) Values (
		p_REQACTIVITYID
		,P_RESPACTIVITYID
	);
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_REQRESPI' || SQLCODE|| ' '||SQLERRM);
END LG_REQRESPI;
/


-- LG_REQ_RESPS.sql

CREATE OR REPLACE PROCEDURE LG_REQ_RESPS
        (
        p_ReturnCd      OUT  PLS_INTEGER
        , p_DebugFlg    IN  PLS_INTEGER
        , RC1           OUT  DB_REF.RCT
        , p_VUID IN VARCHAR2
        )
AS

/*
        ================================================================================
        Proc Name:       LG_REQ_RESPS
        Create Date:     2006-05-17 08:59:37
        Author:          S.Nair 

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2006-05-17 S.Nair        6.1    Created
        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; 
v_ProcName        varchar2(30) Default 'LG_REQ_RESPS';
v_ExceptionFlg    integer Default 0;
v_ActivityID      varchar2(65);
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_VUID=>'||p_VUID||CHR(10)
                ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
--    Get_ActivityID;
        IF RC1%ISOPEN THEN CLOSE RC1; END IF;
        OPEN RC1 FOR 
        SELECT	DISTINCT ---- BGTRK00059887
                RTRIM(a.ACTIVITYID) ACTIVITYID ---- BGTRK00047091
                ,RTRIM(Frp.PartnerName) FrPartnerName
                ,FRPARTNERIDHDR
                ,RTRIM(Top.PartnerName) ToPartnerName
                ,TOPARTNERIDHDR
                ,RTRIM(FRPROTOCOLNAME) FRPROTOCOLNAME
                ,RTRIM(FRPROTOCOLVER) FRPROTOCOLVER
                ,RTRIM(TOPROTOCOLNAME) TOPROTOCOLNAME
                ,RTRIM(TOPROTOCOLVER) TOPROTOCOLVER
                ,RTRIM(FN_GET_RTOBJNAME(a.FrPROCESSCD, a.FrPROCESSVER, 'Process')) FrProcessName
                ,RTRIM(FRPROCESSCD) FRPROCESSCD
                ,RTRIM(FRPROCESSVER) FRPROCESSVER
                ,RTRIM(FN_GET_RTOBJNAME(a.ToPROCESSCD, a.ToPROCESSVER, 'Process')) ToProcessName
                ,RTRIM(TOPROCESSCD) TOPROCESSCD
                ,RTRIM(TOPROCESSVER) TOPROCESSVER
                ,RCVDOCTS
                ,RTRIM(DOCID) DOCID                
                ,RTRIM(FRPACKAGENAME) FRPACKAGENAME
                ,RTRIM(FRPACKAGEVER) FRPACKAGEVER
                ,RTRIM(TOPACKAGENAME) TOPACKAGENAME
                ,RTRIM(TOPACKAGEVER) TOPACKAGEVER
                ,RTRIM(fradtl.vuid) frVuid
                ,RTRIM(toadtl.vuid) ToVuid
			,RTRIM(fradtl.ORIGINALFILENAME) frFILENAME
			,RTRIM(toadtl.ORIGINALFILENAME) toFILENAME
			,RTRIM(fradtl.ENCODING) frENCODING
			,RTRIM(toadtl.ENCODING) toENCODING
                ,ae.EndStateTS
                ,nvl(ae.EndStateFlg,0) EndStateFlg                                
        FROM 
                LG_REQRESP S
                Inner join  LG_ACTIVITY a on (a.ACTIVITYID = S.RESPACTIVITYID or a.ACTIVITYID = S.REQACTIVITYID)
                Left Outer Join PR_Partner Frp                   ON a.FRPARTNERIDHDR = Frp.PartnerID
                Left Outer Join PR_Partner Top                   ON a.ToPARTNERIDHDR = Top.PartnerID
                left outer join lg_activity_endState ae          ON a.activityId     =  ae.activityId
                left outer join lg_activity_dtl_parent_vw fradtl ON a.activityId     = fradtl.activityId
                left outer join lg_activity_dtl_child_vw  toadtl ON a.activityId     = toadtl.activityId                
        Where 
                s.REQACTIVITYID = p_Vuid or s.RESPACTIVITYID = p_Vuid
        Order By RCVDOCTS;
        
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);      
    
    Exception
      WHEN NO_DATA_FOUND THEN
         p_ReturnCd := 3;
      WHEN OTHERS THEN
        p_ReturnCd := 1;
        v_ExceptionFlg := 1;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
        RAISE_APPLICATION_ERROR (-20999,'20001 Get_ActivityID_From_VUID'||v_ProcName ||' '|| SQLCODE|| ' '||SQLERRM);                    
    
End LG_REQ_RESPS;
/

-- LG_RESENDD.sql

Create or Replace Procedure LG_RESENDD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_RESENDID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_RESENDD
	Create Date:     2004-06-23 17:08:21
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	----------	-------------	------	---------------------------------------------
	DATE      	NAME         	Ver   	DESCRIPTION
	----------	-------------	------	---------------------------------------------
	2004-06-23	SYSTEM		4.3	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_RESENDD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RESENDID=>'||TO_CHAR(p_RESENDID)||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From LG_RESEND
	Where
		RESENDID = p_RESENDID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_RESENDD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_RESENDD;
/


-- LG_RESENDI.sql

Create or Replace Procedure LG_RESENDI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_VUID in VARCHAR2
	)
AS

/*
	=======================================================================================
	Proc Name:       LG_RESENDI
	Create Date:     2004-06-23 16:35:20
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	-----------	-------------	------	---------------------------------------------
	DATE      	NAME         	Ver   	DESCRIPTION
	-----------	-------------	------	---------------------------------------------
	2004-06-23	SYSTEM       	4.3	Created
	2004-10-26	S.Priani       	4.3	BGTRK00047286
	=======================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_RESENDI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_RESEND(
		RESENDID
		,VUID
	) (
		SELECT	LG_RESEND_SEQ.NEXTVAL
			,p_VUID
		FROM	DUAL
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_RESENDI' || SQLCODE|| ' '||SQLERRM);
END LG_RESENDI;
/


-- LG_RESENDS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_RESENDS_AH1 (
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	)
AS

/*
	================================================================================
	Proc Name:       LG_RESENDS_AH1
	Create Date:     2004-06-23 17:08:52
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	----------	-------------	------	---------------------------------------------
	DATE      	NAME         	Ver   	DESCRIPTION
	----------	-------------	------	---------------------------------------------
	2004-06-23	SYSTEM		4.3	Created
	2006-09-26	S.Priani	6.1	CR 102885

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_RESENDS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

Unable_To_Lock		Exception;
PRAGMA EXCEPTION_INIT( Unable_To_Lock, -54);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
		SELECT	res.resendid AS RESENDID
			,res.vuid AS VUID
			, res.updatedate AS UPDATEDATE
			, dtl.gatewayid AS GATEWAYID
			, dtl.documentdirection AS DOCUMENTDIRECTION
			,dtl.synchronousflg AS SYNCHRONOUSFLG
			, ends.endstateflg AS ENDSTATEFLG
			, ends.failingcomponent AS FAILINGCOMPONENT
		FROM	lg_resend res
		INNER JOIN lg_activity_dtl dtl
			ON	res.vuid = dtl.vuid
		INNER JOIN lg_activity_endstate ends
			ON	dtl.activityid = ends.activityid
		FOR UPDATE OF res.isLocked NOWAIT;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Unable_To_Lock THEN
		p_ReturnCd := 4;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
	WHEN Others THEN
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
End LG_RESENDS_AH1;
/


-- LG_STACKTRACEI.sql

CREATE OR REPLACE Procedure LG_STACKTRACEI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
--	,RC1 out db_ref.RCT
	,p_EVENTID in VARCHAR2
	,p_EVENTTEXT in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_STACKTRACEI
	Create Date:     2003-09-26 00:03:45
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_STACKTRACEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_EVENTID=>'||p_EVENTID||CHR(10)
		||', p_EVENTTEXT=>'||p_EVENTTEXT||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into LG_STACKTRACE(
		EVENTID
		,EVENTTEXT
	) Values (
		p_EVENTID
		,p_EVENTTEXT
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
--	IF RC1%isOpen THEN Close RC1; END IF;
--	Open RC1 FOR Select 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 LG_STACKTRACEI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 LG_STACKTRACEI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_STACKTRACEI' || SQLCODE|| ' '||SQLERRM);
END LG_STACKTRACEI;
/


-- LG_STATUS_ACTIVITYD.sql

--> EXECUTE DB_GENPROC.GENPROC_DELETE('LG_STATUS_ACTIVITY');
CREATE OR REPLACE PROCEDURE LG_STATUS_ACTIVITYD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_STATUSID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_STATUS_ACTIVITYD
	Create Date:     2006-06-07 18:03:29
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_STATUS_ACTIVITYD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_STATUSID=>'||p_STATUSID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From LG_STATUS_ACTIVITY
	Where 
		STATUSID = p_STATUSID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_STATUS_ACTIVITYD;
/
--> SPOOL OFF


-- LG_STATUS_ACTIVITYI.sql

CREATE OR REPLACE PROCEDURE LG_STATUS_ACTIVITYI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROCESSHDRID in NUMBER
	,p_MESSAGEID in VARCHAR2
	,p_FRPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_ACTIONTYPEID in NUMBER
	,p_ACTION in VARCHAR2
	,p_REFMESSAGEID in VARCHAR2
	,p_ACTIVITYID in VARCHAR2
	,p_VUID in VARCHAR2
	,p_STATUSCODE in NUMBER
	,p_STATUSDESC in VARCHAR2
	,p_ERRORCODE in VARCHAR2
	,p_SEVERITY in NUMBER
	,p_ERRORLOCATION in VARCHAR2
	,p_ERRORDESCRIPTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_STATUS_ACTIVITYI
	Create Date:     2006-06-07 18:03:29
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_STATUS_ACTIVITYI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_FRPARTNERID=>'||p_FRPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_ACTIONTYPEID=>'||p_ACTIONTYPEID||CHR(10)
		||', p_ACTION=>'||p_ACTION||CHR(10)
		||', p_REFMESSAGEID=>'||p_REFMESSAGEID||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_STATUSCODE=>'||p_STATUSCODE||CHR(10)
		||', p_STATUSDESC=>'||p_STATUSDESC||CHR(10)
		||', p_ERRORCODE=>'||p_ERRORCODE||CHR(10)
		||', p_SEVERITY=>'||p_SEVERITY||CHR(10)
		||', p_ERRORLOCATION=>'||p_ERRORLOCATION||CHR(10)
		||', p_ERRORDESCRIPTION=>'||p_ERRORDESCRIPTION||CHR(10)
		;
v_actiontypeid			lg_status_activity.actiontypeid%TYPE;
v_actionPing			lg_status_activity.action%TYPE		DEFAULT	'PING';
v_actionPong			lg_status_activity.action%TYPE		DEFAULT	'PONG';
v_actionStsReq			lg_status_activity.action%TYPE		DEFAULT	'STATUSREQUEST';
v_actionStsResp			lg_status_activity.action%TYPE		DEFAULT	'STATUSRESPONSE';

ActionNotValid		EXCEPTION;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_ActionTypeID := CASE 
				WHEN Upper(p_Action) = v_ActionPing THEN 1
				WHEN Upper(p_Action) = v_ActionPong THEN 2
				WHEN Upper(p_Action) = v_ActionStsReq THEN 3
				WHEN Upper(p_Action) = v_ActionStsResp THEN 4
				ELSE -1
			END;
	IF v_ActionTypeId <> -1 THEN
		Insert into LG_STATUS_ACTIVITY(
			STATUSID
			,PROCESSHDRID
			,MESSAGEID
			,FRPARTNERID
			,TOPARTNERID
			,ACTIONTYPEID
			,ACTION
			,REFMESSAGEID
			,ACTIVITYID
			,VUID
			,STATUSCODE
			,STATUSDESC
			,ERRORCODE
			,SEVERITY
			,ERRORLOCATION
			,ERRORDESCRIPTION
			,CREATEDATE
			,UPDATEDATE
		) Values (
			LG_STATUS_ACTIVITY_SEQ.nextval
			,p_PROCESSHDRID
			,p_MESSAGEID
			,p_FRPARTNERID
			,p_TOPARTNERID
			,p_ACTIONTYPEID
			,p_ACTION
			,p_REFMESSAGEID
			,p_ACTIVITYID
			,p_VUID
			,p_STATUSCODE
			,p_STATUSDESC
			,p_ERRORCODE
			,p_SEVERITY
			,p_ERRORLOCATION
			,p_ERRORDESCRIPTION
			,v_BeginTS
			,v_BeginTS
		);
	ELSE
		Raise ActionNotValid;
	END IF;
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		LG_STATUS_ACTIVITY_SEQ.currval ID, 1 RowTS, v_BeginTS UpdateDate From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN ActionNotValid THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Action is not one of Ping/Pong/StatusRequest/StatusResponse, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	WHEN Dup_Val_On_Index THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	WHEN Others THEN
		Rollback work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		RAISE;
END LG_STATUS_ACTIVITYI;
/


-- LG_STATUS_ACTIVITYS.sql

--> EXECUTE DB_GENPROC.GENPROC_SELECT('LG_STATUS_ACTIVITY', FALSE );
CREATE OR REPLACE PROCEDURE LG_STATUS_ACTIVITYS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_STATUSID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_STATUS_ACTIVITYS
	Create Date:     2006-06-07 18:03:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_STATUS_ACTIVITYS';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_STATUSID=>'||p_STATUSID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_STATUSID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		STATUSID
		,PROCESSHDRID
		,RTRIM(MESSAGEID) MESSAGEID
		,FRPARTNERID
		,TOPARTNERID
		,ACTIONTYPEID
		,RTRIM(ACTION) ACTION
		,RTRIM(REFMESSAGEID) REFMESSAGEID
		,RTRIM(ACTIVITYID) ACTIVITYID
		,RTRIM(VUID) VUID
		,STATUSCODE
		,RTRIM(STATUSDESC) STATUSDESC
		,RTRIM(ERRORCODE) ERRORCODE
		,SEVERITY
		,RTRIM(ERRORLOCATION) ERRORLOCATION
		,RTRIM(ERRORDESCRIPTION) ERRORDESCRIPTION
		,ROWTS
		,CREATEDATE
	FROM 
		LG_STATUS_ACTIVITY
	Where 
		STATUSID = p_STATUSID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		STATUSID
		,PROCESSHDRID
		,RTRIM(MESSAGEID) MESSAGEID
		,FRPARTNERID
		,TOPARTNERID
		,ACTIONTYPEID
		,RTRIM(ACTION) ACTION
		,RTRIM(REFMESSAGEID) REFMESSAGEID
		,RTRIM(ACTIVITYID) ACTIVITYID
		,RTRIM(VUID) VUID
		,STATUSCODE
		,RTRIM(STATUSDESC) STATUSDESC
		,RTRIM(ERRORCODE) ERRORCODE
		,SEVERITY
		,RTRIM(ERRORLOCATION) ERRORLOCATION
		,RTRIM(ERRORDESCRIPTION) ERRORDESCRIPTION
		,ROWTS
		,CREATEDATE
	FROM 
		LG_STATUS_ACTIVITY
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_STATUS_ACTIVITYS;
/
--> SPOOL OFF


-- LG_STATUS_ACTIVITYS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_STATUS_ACTIVITYS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ConversationID IN VARCHAR2
	, p_MESSAGEID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_STATUS_ACTIVITYS_AH1
	Create Date:     2006-06-23 11:17:48
	Author:          SYSTEM 

Get ChannelID from activity tables for Ping/Status Request to send the response back
on the same channel
	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-23 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_STATUS_ACTIVITYS_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ConversationID=>'||p_ConversationID||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
		SELECT	req.STATUSID
			, req.ProcessHdrId
			, req.RefMessageId
			, req.MessageTS RequestTS
			, ah.ChannelID
			, ah.ChannelStatusID
			, ah.FrIPAddress
			, ad.ActivityID
			, ad.VUID
			, ad.GatewayID
		FROM	lg_status_activity req
		INNER JOIN lg_b2bprocess_hdr bph ON bph.ProcessHdrId = req.ProcessHdrId
		INNER JOIN lg_Activity_dtl ad ON ad.ActivityId = req.ActivityId AND ad.VUID = req.VUID
		INNER JOIN lg_Activity ah ON ah.ActivityId = ad.ActivityId
		WHERE	bph.ConversationId = p_ConversationId
		AND	req.Messageid = p_MessageId
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End LG_STATUS_ACTIVITYS_AH1;
/


-- LG_STATUS_ACTIVITYS_MESGID.sql

CREATE OR REPLACE PROCEDURE LG_STATUS_ACTIVITYS_MESGID
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROCESSHDRID IN NUMBER
	, p_MESSAGEID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_STATUS_ACTIVITYS_MESGID
	Create Date:     2006-04-06 11:17:48
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-04-06 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_STATUS_ACTIVITYS_MESGID';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT * FROM (
		SELECT	req.STATUSID
			, req.MessageId
			, req.ProcessHdrId
			, req.RefMessageId
			, COALESCE( resp.REFMSGSTATUSCD, req.STATUSCODE) STATUSCODE
			, COALESCE( resp.STATUSDESC, req.STATUSDESC) STATUSDESC
			, COALESCE( resp.MessageTS, req.MessageTS) ResponseTS
			, req.MessageTS RequestTS
		FROM	lg_status_activity req
		LEFT OUTER JOIN lg_status_activity resp ON resp.REFMESSAGEID = req.MESSAGEID
							AND	resp.ACTIONTypeid = 4 -- MsgStatusResponse
		WHERE	req.ACTIONTypeid = 3 -- MsgStatusRequest
		AND	req.ProcessHdrId = p_ProcessHdrId
		AND	req.refmessageid = p_MessageId
		ORDER BY COALESCE(resp.MessageTS, req.MessageTS) DESC
		)
	WHERE	rownum < 2
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End LG_STATUS_ACTIVITYS_MESGID;
/


-- LG_STATUS_ACTIVITYS_PING.sql

CREATE OR REPLACE PROCEDURE LG_STATUS_ACTIVITYS_PING
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	,p_FRPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_ACTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_STATUS_ACTIVITYS_PING
	Create Date:     2006-04-06 11:17:48
	Author:          S.Priani 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-04-06 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


DEFAULTPINGTIMEOUT	INTEGER Default 5;
v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_STATUS_ACTIVITYS_PING';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FRPARTNERID=>'||p_FRPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)		
		||', p_ACTION=>'||p_ACTION||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT
		lsa.STATUSID
		, lsa.PROCESSHDRID
		, RTRIM(lsa.MESSAGEID) MESSAGEID
		, lsa.FRPARTNERID
		, lsa.TOPARTNERID
		, RTRIM(lsa.ACTION) ACTION
		, RTRIM(lsa.REFMESSAGEID) REFMESSAGEID
		, RTRIM(lsa.ACTIVITYID) ACTIVITYID
		, RTRIM(lsa.VUID) VUID
		, CASE 
			WHEN lsa.ActionTypeID = 1 AND lsa.STATUSCODE = 1  --- Ping sent succesfully
			AND lsa.MessageTS + NUMTODSINTERVAL(COALESCE(lsa.TimeToPerform,DEFAULTPINGTIMEOUT), 'MINUTE') < SysTimeStamp
				THEN 6  --- Timed out
			ELSE lsa.STATUSCODE
		  END STATUSCODE
		, RTRIM(lsa.STATUSDESC) STATUSDESC
		, RTRIM(lsa.ERRORCODE) ERRORCODE
		, lsa.SEVERITY
		, lsa.ROWTS
		, lsa.MessageTS UpdateDate
		, lsa.ActionTypeID
		, lsa.TimeToPerform
	FROM	lg_status_activity lsa
	WHERE	lsa.statusID IN (
		SELECT * FROM ( ---Get Last Ping request to this partner
			SELECT	req.STATUSID
			FROM	lg_status_activity req
			WHERE	req.ActionTypeid = 1 -- Ping
			AND	req.frpartnerid = p_FRPARTNERID
			AND	req.topartnerid = p_TOPARTNERID
			ORDER BY req.MessageTS DESC
			)
		WHERE	rownum < 2
		UNION
		SELECT * FROM ( ---Get Last Pong response from this partner
			SELECT	resp.STATUSID
			FROM	lg_status_activity resp
			WHERE	resp.ActionTypeid = 2 -- Pong
			AND	resp.frpartnerid = p_TOPARTNERID
			AND	resp.topartnerid = p_FRPARTNERID			
			ORDER BY resp.MessageTS DESC
			)
		WHERE	rownum < 2
	)
	ORDER BY lsa.MessageTS DESC
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End LG_STATUS_ACTIVITYS_PING;
/


-- LG_STATUS_ACTIVITYU.sql

--> EXECUTE DB_GENPROC.GENPROC_UPDATE('LG_STATUS_ACTIVITY');
CREATE OR REPLACE PROCEDURE LG_STATUS_ACTIVITYU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_STATUSID in NUMBER
	,p_PROCESSHDRID in NUMBER
	,p_MESSAGEID in VARCHAR2
	,p_FRPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_ACTIONTYPEID in NUMBER
	,p_ACTION in VARCHAR2
	,p_REFMESSAGEID in VARCHAR2
	,p_ACTIVITYID in VARCHAR2
	,p_VUID in VARCHAR2
	,p_STATUSCODE in NUMBER
	,p_STATUSDESC in VARCHAR2
	,p_ERRORCODE in VARCHAR2
	,p_SEVERITY in NUMBER
	,p_ERRORLOCATION in VARCHAR2
	,p_ERRORDESCRIPTION in VARCHAR2
	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_STATUS_ACTIVITYU
	Create Date:     2006-06-07 18:03:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-07 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_STATUS_ACTIVITYU';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_STATUSID=>'||p_STATUSID||CHR(10)
		||', p_PROCESSHDRID=>'||p_PROCESSHDRID||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_FRPARTNERID=>'||p_FRPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_ACTIONTYPEID=>'||p_ACTIONTYPEID||CHR(10)
		||', p_ACTION=>'||p_ACTION||CHR(10)
		||', p_REFMESSAGEID=>'||p_REFMESSAGEID||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_STATUSCODE=>'||p_STATUSCODE||CHR(10)
		||', p_STATUSDESC=>'||p_STATUSDESC||CHR(10)
		||', p_ERRORCODE=>'||p_ERRORCODE||CHR(10)
		||', p_SEVERITY=>'||p_SEVERITY||CHR(10)
		||', p_ERRORLOCATION=>'||p_ERRORLOCATION||CHR(10)
		||', p_ERRORDESCRIPTION=>'||p_ERRORDESCRIPTION||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update LG_STATUS_ACTIVITY
	SET
		PROCESSHDRID=p_PROCESSHDRID
		,      MESSAGEID=p_MESSAGEID
		,      FRPARTNERID=p_FRPARTNERID
		,      TOPARTNERID=p_TOPARTNERID
		,      ACTIONTYPEID=p_ACTIONTYPEID
		,      ACTION=p_ACTION
		,      REFMESSAGEID=p_REFMESSAGEID
		,      ACTIVITYID=p_ACTIVITYID
		,      VUID=p_VUID
		,      STATUSCODE=p_STATUSCODE
		,      STATUSDESC=p_STATUSDESC
		,      ERRORCODE=p_ERRORCODE
		,      SEVERITY=p_SEVERITY
		,      ERRORLOCATION=p_ERRORLOCATION
		,      ERRORDESCRIPTION=p_ERRORDESCRIPTION
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      MESSAGETS= SYSTIMESTAMP
	Where 
		STATUSID = p_STATUSID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End LG_STATUS_ACTIVITYU;
/
--> SPOOL OFF


-- LG_STATUS_ACTIVITYU_AH1.sql

CREATE OR REPLACE PROCEDURE LG_STATUS_ACTIVITYU_AH1
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_CONVERSATIONID in VARCHAR2
	,p_MESSAGEID in VARCHAR2
	,p_FRPARTNERID in NUMBER
	,p_TOPARTNERID in NUMBER
	,p_ACTIONTYPEID in NUMBER
	,p_ACTION in VARCHAR2
	,p_REFMESSAGEID in VARCHAR2
	,p_ACTIVITYID in VARCHAR2
	,p_VUID in VARCHAR2
	,p_STATUSCODE in NUMBER
	,p_STATUSDESC in VARCHAR2
	,p_ERRORCODE in VARCHAR2
	,p_SEVERITY in NUMBER
	,p_ERRORLOCATION in VARCHAR2
	,p_ERRORDESCRIPTION in VARCHAR2
	,p_ROWTS in NUMBER
	,p_TIMETOPERFORM in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_STATUS_ACTIVITYU_AH1
	Create Date:     2006-05-11 18:11:42
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-11 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_STATUS_ACTIVITYU_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONVERSATIONID=>'||p_CONVERSATIONID||CHR(10)
		||', p_MESSAGEID=>'||p_MESSAGEID||CHR(10)
		||', p_FRPARTNERID=>'||p_FRPARTNERID||CHR(10)
		||', p_TOPARTNERID=>'||p_TOPARTNERID||CHR(10)
		||', p_ACTIONTYPEID=>'||p_ACTIONTYPEID||CHR(10)
		||', p_ACTION=>'||p_ACTION||CHR(10)
		||', p_REFMESSAGEID=>'||p_REFMESSAGEID||CHR(10)
		||', p_ACTIVITYID=>'||p_ACTIVITYID||CHR(10)
		||', p_VUID=>'||p_VUID||CHR(10)
		||', p_STATUSCODE=>'||p_STATUSCODE||CHR(10)
		||', p_STATUSDESC=>'||p_STATUSDESC||CHR(10)
		||', p_ERRORCODE=>'||p_ERRORCODE||CHR(10)
		||', p_SEVERITY=>'||p_SEVERITY||CHR(10)
		||', p_ERRORLOCATION=>'||p_ERRORLOCATION||CHR(10)
		||', p_ERRORDESCRIPTION=>'||p_ERRORDESCRIPTION||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_TIMETOPERFORM=>'||p_TIMETOPERFORM||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update LG_STATUS_ACTIVITY sa
	SET
                 FRPARTNERID = COALESCE( p_FRPARTNERID, sa.FRPARTNERID )
                , TOPARTNERID = COALESCE( p_TOPARTNERID, sa.TOPARTNERID )
                , ACTIONTYPEID = COALESCE( p_ACTIONTYPEID, sa.ACTIONTYPEID )
                , ACTION = COALESCE( p_ACTION, sa.ACTION )
                , REFMESSAGEID = COALESCE( p_REFMESSAGEID, sa.REFMESSAGEID )
                , ACTIVITYID = COALESCE( p_ACTIVITYID, sa.ACTIVITYID )
                , VUID = COALESCE( p_VUID, sa.VUID )
                , STATUSCODE = COALESCE( p_STATUSCODE, sa.STATUSCODE )
                , STATUSDESC = COALESCE( p_STATUSDESC, sa.STATUSDESC )
                , ERRORCODE = COALESCE( p_ERRORCODE, sa.ERRORCODE )
                , SEVERITY = COALESCE( p_SEVERITY, sa.SEVERITY )
                , ERRORLOCATION = COALESCE( p_ERRORLOCATION, sa.ERRORLOCATION )
                , ERRORDESCRIPTION = COALESCE( p_ERRORDESCRIPTION, sa.ERRORDESCRIPTION )
		,      TIMETOPERFORM = COALESCE( p_TIMETOPERFORM, TIMETOPERFORM)
		,      ROWTS= COALESCE(p_RowTS,sa.RowTS) + 1
		,      UPDATEDATE= v_BeginTS
		,      MESSAGETS= SYSTIMESTAMP
	Where 
                MESSAGEID = p_MESSAGEID ;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN ZeroUpdate then
		v_ExceptionFlg := 1;
		p_ReturnCd := 3;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
	WHEN OTHERS then
		ROLLBACK work;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
End LG_STATUS_ACTIVITYU_AH1;
/


-- LG_SUMMARYENGINELOCK.SQL

CREATE OR REPLACE PROCEDURE LG_SUMMARYENGINELOCK (
		p_ReturnCd	in out	pls_integer
		,p_DebugFlg	in	pls_integer
		,p_LockFlg	in	pls_integer	-- 1 => Lock Table
							-- 0 => Update LastRunTS
) AS
/*
	================================================================================
	Proc Name:      LG_SUMMARYENGINELOCK
	Create Date:    12/03/04
	Author          S.Priani

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
                       ::::::::::::::::::::::::::::::

	----------	-------------	------	---------------------------------------------
	DATE      	NAME         	Ver   	DESCRIPTION
	----------	-------------	------	---------------------------------------------
	2004-03-12	S.Priani	4.2.2	Created
	2006-11-06	S.Priani	6.1	CR 103283
	================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_BeginTS	timestamp	:= FN_Get_Date; -- Performance related variables
	v_ReturnCd	pls_integer;
	Unable_To_Lock	Exception;
	Pragma Exception_Init (Unable_To_Lock, -054);

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_SummaryEngineLock',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    , p_DebugFlg
    );
End ProcAudit;

--------------------------------------------------------------------------
--                 M  A  I  N
--------------------------------------------------------------------------
BEGIN
	IF p_LockFlg = 1 THEN
		LOCK TABLE lg_summaryengine IN EXCLUSIVE MODE NOWAIT;
	ELSE
		UPDATE	lg_summaryengine
		SET	LASTRUNTS = SYS_EXTRACT_UTC(SYSTIMESTAMP)
		;
	END IF;
	ProcAudit;
	p_ReturnCd := 0;
EXCEPTION
	When Unable_To_Lock Then
		ProcAudit;
		p_ReturnCd := 4;
	When Others Then
		ProcAudit;
		p_ReturnCd := 1;
		Raise_application_error (-20999,'20000 LG_SUMMARYENGINELOCK ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_SUMMARYENGINES.SQL

CREATE OR REPLACE PROCEDURE LG_SUMMARYENGINES (
		p_ReturnCd	in out	pls_integer
		,p_DebugFlg	in	pls_integer
		,RC1		OUT	DB_REF.RCT
) AS
/*
================================================================================
Proc Name:      LG_SUMMARYENGINES
Create Date:    12/03/04
Author          S.Priani

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2004-03-12 SYSTEM	4.2.2	Created
	2005-09-19 S.Priani	6.0.0.2	BGTRK00055949 - return system time also
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_BeginTS	timestamp	:= FN_Get_Date; -- Performance related variables
	v_ReturnCd	pls_integer;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_SummaryEngineS',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    , p_DebugFlg
    );
End ProcAudit;

--------------------------------------------------------------------------
--                 M  A  I  N
--------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR
		SELECT	LASTRUNTS
			,SYS_EXTRACT_UTC(SYSTIMESTAMP) CURRENTTS
		FROM	lg_summaryengine
	;
	ProcAudit;
	p_ReturnCd := 0;
EXCEPTION
	When Others Then
		ProcAudit;
		p_ReturnCd := 1;
		Raise_application_error (-20999,'20000 LG_SUMMARYENGINES ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_SUMMARYI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_SUMMARYI_AH1(
      p_ReturnCd      out pls_integer
    , p_DebugFlg       in pls_integer
    , P_VUID           in   VARCHAR2
    , P_RCVDOCTS       in   Timestamp
    , P_SNTDOCTS       in   Timestamp
    , P_INITBUSINESSID in out  VARCHAR2
    , P_FrBUSINESSID   in out  VARCHAR2
    , P_ToBUSINESSID   in out  VARCHAR2
    , P_FRPROTOCOLNAME in out  VARCHAR2
    , P_TOPROTOCOLNAME in out  VARCHAR2
    , P_FRPROCESSCD    in out  VARCHAR2
    , p_FRPROCESSVER   in out  VARCHAR2
    , P_TOPROCESSCD    in out  VARCHAR2
    , p_TOPROCESSVER   in out  VARCHAR2
    , P_FRIPADDRESS    in out  VARCHAR2
    , p_CHANNELSTATUSName in  VARCHAR2
--
    , P_RCVDOCCNT         in   pls_integer
    , P_SNTDOCCNT         in   pls_integer
    , P_FAILDOCCNT        in   pls_integer
    , P_EVENTCD           in   pls_integer  -- 3.2
    , p_MSGRAWLENGTH      in   NUMBER
    , p_MSGCONTENTLENGTH  in   NUMBER
    , P_FRPROTOCOLVer     in out  VARCHAR2
    , P_TOPROTOCOLVer     in out  VARCHAR2
    , p_DestName          in   VARCHAR2
    , P_dmDuration_MS     in   pls_integer   -- MS -- millisecond
    , P_TRANSPORTDURATION_MS   in   pls_integer
    , P_TransportATTEMPTCNT    in   pls_integer
    , P_GATEWAYATTEMPTCNT      in   pls_integer
-- 1/23/2003
    , p_PERSISTDOCTS           in   Timestamp
    , p_BPEStartTS      in   Timestamp
    , p_BPEEndTS        in   Timestamp
    , p_PROVSESSIONID          in   pls_integer
    , p_FrTransportType     in VARCHAR2
    , p_ToTransportType     in VARCHAR2
    , p_FRPACKAGENAME   in  VARCHAR2
    , p_FRPACKAGEVER    in  VARCHAR2
    , p_TOPACKAGENAME   in  VARCHAR2
    , p_TOPACKAGEVER    in  VARCHAR2
    , p_SYNCHRONOUSFLG  in  pls_integer
    ,p_ActionFlg        in pls_integer -- 1 = Action, 0 = Signal
    )
AS
/*
================================================================================
Proc Name:      LG_SUMMARYI_AH1
Create Date:    01/03/2002
Author          Vipin Goel

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2006-05-01 Shishir N         6.0   PMR 08997999846
--================================================================================
--================================================================================*/
   v_return_code        pls_integer;
   v_SUMMARYID         LG_SUM_XREF_lnk.SUMMARYID%type;
   v_ProcName           varchar2(30) := 'LG_SUMMARYI_AH1';
   v_vuid              LG_activity_summary.vuid%type;
   v_createDate date := FN_GET_DATE();
   v_update_summary boolean := False;
   v_beginTS    Timestamp(6):= FN_Get_Date;
   v_MSGRAWLENGTH_IN        LG_SUMMARY.MSGRAWLENGTHSUM_IN%type := 0;
   v_MSGCONTENTLENGTH_IN    LG_SUMMARY.MSGCONTENTLENGTHSUM_IN%type := 0;
   v_MSGRAWLENGTH_OUT       LG_SUMMARY.MSGRAWLENGTHSUM_OUT%type := 0;
   v_MSGCONTENTLENGTH_OUT   LG_SUMMARY.MSGCONTENTLENGTHSUM_OUT%type := 0;
   v_MSGRAWLENGTH_FAIL       LG_SUMMARY.MSGRAWLENGTHSUM_FAIL%type := 0;
   v_MSGCONTENTLENGTH_FAIL   LG_SUMMARY.MSGCONTENTLENGTHSUM_FAIL%type := 0;
   v_recDocDate  Date := to_date(to_char(p_rcvdocts, 'dd-mon-yy hh24:mi:ss'),'dd-mon-yy hh24:mi:ss' );
   V_SntDocDate  Date := to_date(to_char(p_sntdocts, 'dd-mon-yy hh24:mi:ss'),'dd-mon-yy hh24:mi:ss' );
--
   v_DocDuration_ms    pls_integer;
   v_PersistQueTime_ms pls_integer;
   v_BpeQueTime_ms     pls_integer;
   v_BPEProcessTime_ms pls_integer;
   v_DMQueTime_ms      pls_integer;
   v_PROVSESSIONID     LG_SUM_XREF_Lnk.PROVSESSIONID%type := nvl(P_PROVSESSIONID, -1);
   v_DestTypeId        PR_DESTTYPE.DestTypeId%type;
   v_UpdCount          pls_integer;

--
-- Procedure
--
-- Declaration
procedure Update_LG_SUMMARY;
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
    DB_ProcAuditLogI_Ah1(v_ProcName,v_beginTS,'p_ReturnCd'||','||p_DebugFlg||','||
    P_VUID||','||
    to_char(P_RCVDOCTS,'dd-mon-yy hh24:mi:ssxFF')||','||
    to_char(P_SNTDOCTS,'dd-mon-yy hh24:mi:ssxFF')||','||
    P_INITBUSINESSID   ||','||
    P_FrBUSINESSID     ||','||
    P_ToBUSINESSID     ||','||
    P_FRPROTOCOLNAME   ||','||
    P_TOPROTOCOLNAME   ||','||
    P_FRPROCESSCD      ||','||
    P_FRPROCESSVER     ||','||
    P_TOPROCESSCD      ||','||
    P_TOPROCESSVER     ||','||
    P_FRIPADDRESS      ||','||
    P_CHANNELSTATUSName||','||
    P_RCVDOCCNT        ||','||
    P_SNTDOCCNT        ||','||
    P_FAILDOCCNT       ||','||
    P_EVENTCD          ||','||
    p_MSGRAWLENGTH     ||','||
    p_MSGCONTENTLENGTH ||','||
    P_FRPROTOCOLVer    ||','||
    P_TOPROTOCOLVer    ||','||
    p_PERSISTDOCTS     ||','||
    p_BPEStartTS ||','||
    p_BPEEndTS  ||','||
    p_PROVSESSIONID
    ||','||p_FrTransportType
    ||','||p_ToTransportType
    ||','||p_FRPACKAGENAME
    ||','||p_FRPACKAGEVER
    ||','||p_TOPACKAGENAME
    ||','||p_TOPACKAGEVER
    ||','||p_SYNCHRONOUSFLG
    ||','||p_ActionFlg
    );

End ProcAudit;
--
Procedure Update_MSGLENGTH_SUMMARY is
Begin
    LG_MSGLENGTH_SUMMARYI_AH1(
    P_RETURNCD          => p_ReturnCD
    ,P_DEBUGFLG         => p_DebugFlg
    ,P_MSGRAWLENGTH_OUT => v_MSGRAWLENGTH_OUT
    ,P_SUMMARYID        => v_SUMMARYID
    ,P_DAYHOUR          => trunc(v_recDocDate,'HH24')
    ,P_MSGRAWLENGTH_IN  => v_MSGRAWLENGTH_IN
    );
End Update_MSGLENGTH_SUMMARY;
--
Procedure Insert_lg_summary_xref is
Begin

v_DestTypeId := FN_GET_DESTTYPEID(p_DestName);

   LG_SUMMARY_XREFI_AH1(
   p_ReturnCd          => p_ReturnCd
   ,p_DebugFlg         => p_DebugFlg
   ,P_SUMMARYID         => v_SUMMARYID
   ,P_INITBUSINESSID    => P_INITBUSINESSID
   ,P_FRBUSINESSID      => P_FrBUSINESSID
   ,P_TOBUSINESSID      => P_ToBUSINESSID
   ,P_FRPROTOCOLNAME    => P_FRPROTOCOLNAME
   ,P_FRPROTOCOLVER     => P_FRPROTOCOLVer
   ,P_TOPROTOCOLNAME    => P_TOPROTOCOLNAME
   ,P_TOPROTOCOLVER     => P_TOPROTOCOLVer
   ,P_FRPROCESSCD       => P_FRPROCESSCD
   ,P_FRPROCESSVER      => p_FRPROCESSVER
   ,P_TOPROCESSCD       => P_TOPROCESSCD
   ,P_TOPROCESSVER      => p_TOPROCESSVER
   ,P_FRIPADDRESS       => P_FRIPADDRESS
   ,P_CHANNELSTATUSNAME => p_CHANNELSTATUSName
   ,p_DestTypeId        => v_DestTypeId
   ,p_PROVSESSIONID     => v_PROVSESSIONID
   ,p_FrTransportType   => p_FrTransportType
   ,p_ToTransportType   => p_ToTransportType
   ,p_FRPACKAGENAME   => p_FRPACKAGENAME
   ,p_FRPACKAGEVER    => p_FRPACKAGEVER
   ,p_TOPACKAGENAME   => p_TOPACKAGENAME
   ,p_TOPACKAGEVER    => p_TOPACKAGEVER
   ,p_SYNCHRONOUSFLG  => p_SYNCHRONOUSFLG
   ,p_ActionFlg       => p_ActionFlg
);
end Insert_lg_summary_xref;
--
Procedure Upd_LGSummary is
Begin
        update lg_summary S set
                          s.RcvDocCnt               = nvl(s.RcvDocCnt,0)               + nvl(p_RCVDOCCNT,0)
                        , s.SntDocCnt               = nvl(s.SntDocCnt,0)               + nvl(p_SNTDOCCNT,0)
                        , s.FailDocCnt              = nvl(s.FailDocCnt,0)              + nvl(p_FAILDOCCNT,0)
                        , s.MsgRawLengthSum_IN      = nvl(s.MSGRAWLENGTHSUM_IN,0)      + nvl(v_MSGRAWLENGTH_IN,0)
                        , s.MsgContentLengthSum_IN  = nvl(s.MSGCONTENTLENGTHSUM_IN,0)  + nvl(v_MSGCONTENTLENGTH_IN,0)
                        , s.MsgRawLengthSum_OUT     = nvl(s.MSGRAWLENGTHSUM_OUT,0)     + nvl(v_MSGRAWLENGTH_OUT,0)
                        , s.MsgContentLengthSum_OUT = nvl(s.MSGCONTENTLENGTHSUM_OUT,0) + nvl(v_MSGCONTENTLENGTH_OUT,0)
                        , s.MsgRawLengthSum_FAIL    = nvl(s.MSGRAWLENGTHSUM_FAIL,0)     +nvl(v_MSGRAWLENGTH_FAIL,0)
                        , s.MsgContentLengthSum_FAIL= nvl(s.MSGCONTENTLENGTHSUM_FAIL,0) +nvl(v_MSGCONTENTLENGTH_FAIL,0)
                        , s.dmDurationSum           = nvl(s.dmDurationSum,0)           + nvl(P_dmDuration_MS,0)
                        , s.TRANSPORTDURATIONSum    = nvl(s.TRANSPORTDURATIONSum,0)    + nvl(P_TRANSPORTDURATION_MS,0)
                        , s.TransportATTEMPTCNT     = nvl(s.TransportATTEMPTCNT,0)     + nvl(P_TransportATTEMPTCNT,0)
                        , s.GATEWAYATTEMPTCNT       = nvl(s.GATEWAYATTEMPTCNT,0)       + nvl(P_GATEWAYATTEMPTCNT,0)
                        , s.MSGRAWLENGTH_IN_MIN     = least   (nvl(s.MSGRAWLENGTH_IN_MIN, v_MSGRAWLENGTH_IN),   nvl(v_MSGRAWLENGTH_IN,s.MSGRAWLENGTH_IN_MIN))
                        , s.MSGRAWLENGTH_IN_Max     = greatest(nvl(s.MSGRAWLENGTH_IN_Max, v_MSGRAWLENGTH_IN),   nvl(v_MSGRAWLENGTH_IN,0))
                        , s.MSGRAWLENGTH_out_MIN    = least   (nvl(s.MSGRAWLENGTH_out_MIN, v_MSGRAWLENGTH_OUT), nvl(v_MSGRAWLENGTH_OUT, s.MSGRAWLENGTH_out_MIN))
                        , s.MSGRAWLENGTH_out_Max    = greatest(nvl(s.MSGRAWLENGTH_out_Max, v_MSGRAWLENGTH_OUT), nvl(v_MSGRAWLENGTH_OUT,0))
                        , s.DOCDURATION             = nvl(s.DOCDURATION,0)    + v_DocDuration_MS
                        , s.PERSISTQUESUM           = nvl(s.PERSISTQUESUM, 0) + v_PersistQueTime_ms
                        , s.BPEQUESUM               = nvl(s.BPEQUESUM,0)      + v_BpeQueTime_ms
                        , s.BPEPROCESSSUM           = nvl(s.BPEPROCESSSUM,0)  + v_BPEProcessTime_ms
                        , s.DMQUESUM                = nvl(s.DMQUESUM,0)       + v_DMQueTime_ms
                        , s.Updatedate = FN_GET_DATE()
        Where
            S.summaryid = v_summaryId
            and  S.dayHour        = trunc(v_recDocDate,'HH24')
            and s.EndStateDayHour = trunc(nvl(V_SntDocDate, v_recDocDate ),'HH24');

            v_UpdCount := sql%RowCount;
Exception
    when others then
        p_ReturnCd := 1;
        raise_application_error(-20999, 20007||' LG_SUMMARYI_AH1 - Upd_LGSummary '||SQLERRM);
End Upd_LGSummary;
--
Procedure Upd_LGSummaryMi is
Begin
        update lg_summary_mi S set
                          s.RcvDocCnt               = nvl(s.RcvDocCnt,0)               + nvl(p_RCVDOCCNT,0)
                        , s.SntDocCnt               = nvl(s.SntDocCnt,0)               + nvl(p_SNTDOCCNT,0)
                        , s.FailDocCnt              = nvl(s.FailDocCnt,0)              + nvl(p_FAILDOCCNT,0)
                        , s.MsgRawLengthSum_IN      = nvl(s.MSGRAWLENGTHSUM_IN,0)      + nvl(v_MSGRAWLENGTH_IN,0)
                        , s.MsgContentLengthSum_IN  = nvl(s.MSGCONTENTLENGTHSUM_IN,0)  + nvl(v_MSGCONTENTLENGTH_IN,0)
                        , s.MsgRawLengthSum_OUT     = nvl(s.MSGRAWLENGTHSUM_OUT,0)     + nvl(v_MSGRAWLENGTH_OUT,0)
                        , s.MsgContentLengthSum_OUT = nvl(s.MSGCONTENTLENGTHSUM_OUT,0) + nvl(v_MSGCONTENTLENGTH_OUT,0)
                        , s.MsgRawLengthSum_FAIL    = nvl(s.MSGRAWLENGTHSUM_FAIL,0)     + nvl(v_MSGRAWLENGTH_FAIL,0)
                        , s.MsgContentLengthSum_FAIL = nvl(s.MSGCONTENTLENGTHSUM_FAIL,0) + nvl(v_MSGCONTENTLENGTH_FAIL,0)                       
                        , s.dmDurationSum           = nvl(s.dmDurationSum,0)           + nvl(P_dmDuration_MS,0)
                        , s.TRANSPORTDURATIONSum    = nvl(s.TRANSPORTDURATIONSum,0)    + nvl(P_TRANSPORTDURATION_MS,0)
                        , s.TransportATTEMPTCNT     = nvl(s.TransportATTEMPTCNT,0)     + nvl(P_TransportATTEMPTCNT,0)
                        , s.GATEWAYATTEMPTCNT       = nvl(s.GATEWAYATTEMPTCNT,0)       + nvl(P_GATEWAYATTEMPTCNT,0)
                        , s.MSGRAWLENGTH_IN_MIN     = least   (nvl(s.MSGRAWLENGTH_IN_MIN, v_MSGRAWLENGTH_IN),   nvl(v_MSGRAWLENGTH_IN,s.MSGRAWLENGTH_IN_MIN))
                        , s.MSGRAWLENGTH_IN_Max     = greatest(nvl(s.MSGRAWLENGTH_IN_Max, v_MSGRAWLENGTH_IN),   nvl(v_MSGRAWLENGTH_IN,0))
                        , s.MSGRAWLENGTH_out_MIN    = least   (nvl(s.MSGRAWLENGTH_out_MIN, v_MSGRAWLENGTH_OUT), nvl(v_MSGRAWLENGTH_OUT, s.MSGRAWLENGTH_out_MIN))
                        , s.MSGRAWLENGTH_out_Max    = greatest(nvl(s.MSGRAWLENGTH_out_Max, v_MSGRAWLENGTH_OUT), nvl(v_MSGRAWLENGTH_OUT,0))
                        , s.DOCDURATION             = nvl(s.DOCDURATION,0)    + v_DocDuration_MS
                        , s.PERSISTQUESUM           = nvl(s.PERSISTQUESUM, 0) + v_PersistQueTime_ms
                        , s.BPEQUESUM               = nvl(s.BPEQUESUM,0)      + v_BpeQueTime_ms
                        , s.BPEPROCESSSUM           = nvl(s.BPEPROCESSSUM,0)  + v_BPEProcessTime_ms
                        , s.DMQUESUM                = nvl(s.DMQUESUM,0)       + v_DMQueTime_ms
                        , s.Updatedate = FN_GET_DATE()
        Where
            S.summaryid = v_summaryId
            and  S.dayHourMi        = FN_tenMinuteDate(v_recDocDate)
            and s.EndStateDayHourMi = FN_tenMinuteDate(nvl(V_SntDocDate, v_recDocDate ));

            v_UpdCount := sql%RowCount;
Exception
    when others then
        p_ReturnCd := 1;
        raise_application_error(-20999, 20007||' LG_SUMMARYI_AH1 - Upd_LGSummaryMi '||SQLERRM);
End Upd_LGSummaryMi;
--
Procedure Update_lg_summary_mi is
Begin
        Upd_LGSummaryMi;
        IF v_UpdCount = 0 Then
        Begin
            Insert into lg_summary_mi
            (
            SUMMARYID
            , DAYHOURMI
            , CREATEDATE
            , RCVDOCCNT
            , SNTDOCCNT
            , FAILDOCCNT
            , MSGRAWLENGTHSUM_IN
            , MSGCONTENTLENGTHSUM_IN
            , MSGRAWLENGTHSUM_OUT
            , MSGCONTENTLENGTHSUM_OUT
            , MSGRAWLENGTHSUM_FAIL
            , MSGCONTENTLENGTHSUM_FAIL
            , ENDSTATEDAYHOURMI
            , DMDURATIONSUM
            , TRANSPORTDURATIONSUM
            , TRANSPORTATTEMPTCNT
            , GATEWAYATTEMPTCNT
            , MSGRAWLENGTH_IN_MIN
            , MSGRAWLENGTH_IN_MAX
            , MSGRAWLENGTH_OUT_MIN
            , MSGRAWLENGTH_OUT_MAX
            , PERSISTQUESUM
            , BPEQUESUM
            , BPEPROCESSSUM
            , DMQUESUM
            )
        VALUES
            (
            v_SUMMARYID
            , FN_tenMinuteDate(v_recDocDate)
            , v_createdate
            , nvl(p_RCVDOCCNT,0)
            , nvl(p_SNTDOCCNT,0)
            , nvl(p_FAILDOCCNT,0)
            , nvl(v_MSGRAWLENGTH_IN,0)
            , nvl(v_MSGCONTENTLENGTH_IN,0)
            , nvl(v_MSGRAWLENGTH_OUT,0)
            , nvl(v_MSGCONTENTLENGTH_OUT,0)
            , nvl(v_MSGRAWLENGTH_FAIL,0)
            , nvl(v_MSGCONTENTLENGTH_FAIL,0)
            , FN_tenMinuteDate(nvl(V_SntDocDate, v_recDocDate ))
            , p_DMDURATION_MS
            , p_TRANSPORTDURATION_MS
            , p_TRANSPORTATTEMPTCNT
            , p_GATEWAYATTEMPTCNT
            , v_MSGRAWLENGTH_IN
            , v_MSGRAWLENGTH_IN
            , v_MSGRAWLENGTH_OUT
            , v_MSGRAWLENGTH_OUT
            , v_PersistQueTime_ms
            , v_BpeQueTime_ms
            , v_BPEProcessTime_ms
            , v_DMQueTime_ms
            );

--   dbms_output.put_line('merge count lg_summary_mi '||sql%rowcount);
Exception
    When Dup_Val_on_Index Then
        Upd_LGSummaryMi;
    when others then
        p_ReturnCd := 1;
        raise_application_error(-20999,20007||' LG_SUMMARYI_AH1 - Update_lg_summary_mi '||SQLERRM);
End;
End if;
End Update_lg_summary_mi;
--
procedure Update_LG_SUMMARY as
Begin
-- Set up in/out/Fail lengths
    IF p_RCVDOCCNT > 0 THEN                            -- InBound
        v_MSGRAWLENGTH_IN      := nvl(p_MSGRAWLENGTH,0);
        v_MSGCONTENTLENGTH_IN  := nvl(p_MSGCONTENTLENGTH,0);
    ELSIF  (p_SNTDOCCNT > 0)   THEN -- OutBound
        v_MSGRAWLENGTH_OUT     := nvl(p_MSGRAWLENGTH,0);
        v_MSGCONTENTLENGTH_OUT := nvl(p_MSGCONTENTLENGTH,0);
    ELSIF (p_FAILDOCCNT > 0) THEN 
        v_MSGRAWLENGTH_FAIL   := nvl(p_MSGRAWLENGTH,0);
        v_MSGCONTENTLENGTH_FAIL := nvl(p_MSGCONTENTLENGTH,0);
    END IF;
   
    dbms_output.put_line('Update_LG_SUMMARY-MSGRAWLENGTH_IN '||v_MSGRAWLENGTH_IN||
                                         ' MSGRAWLENGTH_OUT '||v_MSGRAWLENGTH_OUT||
                                         ' MSGRAWLENGTH_FAIL '||v_MSGRAWLENGTH_FAIL);
--
-- Figure out document duration
--
   IF P_SNTDOCTS IS NOT NULL THEN
      v_DocDuration_ms := FN_TIMESTAMPDIFF(P_RcvDOCTS, P_SNTDOCTS);
      v_DMQueTime_ms   := FN_TIMESTAMPDIFF(p_BPEEndTS, P_SNTDOCTS) - P_dmDuration_MS; --???
   END IF;
--
-- compute Persist Queue, BPE Queue, BPE Process timing When the proc is called at Lg_activity Update Time
--
   IF (p_PERSISTDOCTS IS NOT NULL AND p_BPEStartTS   IS NOT NULL AND
       p_BPEEndTS     IS NOT NULL AND P_SNTDOCTS     IS     NULL) THEN
       v_PersistQueTime_ms := FN_TIMESTAMPDIFF(P_RCVDOCTS, p_PERSISTDOCTS);
       v_BpeQueTime_ms     := FN_TIMESTAMPDIFF(p_PERSISTDOCTS, p_BPEStartTS);
       v_BPEProcessTime_ms := FN_TIMESTAMPDIFF(p_BPEStartTS, p_BPEEndTS);
   END IF;

-- From RcvDocTS perspective Update receive/sent/failed doc count and summarize msg length

        Upd_LGSummary;
        IF v_UpdCount = 0 Then
        Begin
            Insert into lg_summary
            (
            SUMMARYID
            , DAYHOUR
            , CREATEDATE
            , RCVDOCCNT
            , SNTDOCCNT
            , FAILDOCCNT
            , MSGRAWLENGTHSUM_IN
            , MSGCONTENTLENGTHSUM_IN
            , MSGRAWLENGTHSUM_OUT
            , MSGCONTENTLENGTHSUM_OUT
            , MSGRAWLENGTHSUM_FAIL
            , MSGCONTENTLENGTHSUM_FAIL
            , ENDSTATEDAYHOUR
            , DMDURATIONSUM
            , TRANSPORTDURATIONSUM
            , TRANSPORTATTEMPTCNT
            , GATEWAYATTEMPTCNT
            , MSGRAWLENGTH_IN_MIN
            , MSGRAWLENGTH_IN_MAX
            , MSGRAWLENGTH_OUT_MIN
            , MSGRAWLENGTH_OUT_MAX
            , PERSISTQUESUM
            , BPEQUESUM
            , BPEPROCESSSUM
            , DMQUESUM
            )
        VALUES
            (
            v_SUMMARYID
            , trunc(v_recDocDate,'HH24')
            , v_createdate
            , nvl(p_RCVDOCCNT,0)
            , nvl(p_SNTDOCCNT,0)
            , nvl(p_FAILDOCCNT,0)
            , nvl(v_MSGRAWLENGTH_IN,0)
            , nvl(v_MSGCONTENTLENGTH_IN,0)
            , nvl(v_MSGRAWLENGTH_OUT,0)
            , nvl(v_MSGCONTENTLENGTH_OUT,0)
            , nvl(v_MSGRAWLENGTH_FAIL,0)
            , nvl(v_MSGCONTENTLENGTH_OUT,0)
            , trunc(nvl(V_SntDocDate, v_recDocDate ),'HH24')
            , p_DMDURATION_MS
            , p_TRANSPORTDURATION_MS
            , p_TRANSPORTATTEMPTCNT
            , p_GATEWAYATTEMPTCNT
            , v_MSGRAWLENGTH_IN
            , v_MSGRAWLENGTH_IN
            , v_MSGRAWLENGTH_OUT
            , v_MSGRAWLENGTH_OUT
            , v_PersistQueTime_ms
            , v_BpeQueTime_ms
            , v_BPEProcessTime_ms
            , v_DMQueTime_ms
            );

--   dbms_output.put_line('merge count lg_summary_mi '||sql%rowcount);
Exception
    When Dup_Val_on_Index Then
        Upd_LGSummary;
    when others then
        p_ReturnCd := 1;
        raise_application_error(-20999,20007||' LG_SUMMARYI_AH1 - Update_lg_summary '||SQLERRM);
End;
End if;

End update_LG_SUMMARY;
--
procedure insert_LG_Activity as
Begin
    Insert into lg_activity_summary (
        SUMMARYID,
        DAYHOUR,
        VUID,
        CREATEDATE,
        ENDSTATEDAYHOUR)
    values(
        v_SUMMARYID,
        trunc(v_recDocDate,'HH24'),
        P_VUID,
        v_createdate,
        trunc(nvl(V_SntDocDate, v_recDocDate ),'HH24')
        );
exception
    when dup_val_on_index   then  null;
    When Others then
        raise_application_error(-20999,20008||' LG_SUMMARYI_AH1 - Insert_LG_Activity '||SQLERRM);
End insert_LG_Activity;
--
-- insert/update lg_summary_event
--
procedure update_LG_SUMMARY_event as
Begin
    Insert into lg_summary_event (SUMMARYID,DAYHOUR , EVENTCD, EVENTCNT, CREATEDATE)
    values (v_SUMMARYID,trunc(v_recDocDate,'HH24'), nvl(p_EVENTCD,0),  p_FAILDOCCNT, v_createdate);
Exception
    when dup_val_on_index then
    begin -- update event
        update lg_summary_event set EVENTCNT  = EVENTCNT  + p_FAILDOCCNT
        where summaryid = v_summaryId
        and   dayHour   = trunc(v_recDocDate, 'HH24')
        and   eventcd   = nvl(p_eventcd,0) ;
    exception
        when others then
        p_ReturnCd := 1;
        raise_application_error(-20999,20009||' LG_SUMMARYI_AH1 - Update_LG_SUMMARY_event '||SQLERRM);
    end; -- update event
    when others then
        p_ReturnCd := 1;
        raise_application_error(-20999,20010||' LG_SUMMARYI_AH1 - Update_LG_SUMMARY_event '||SQLERRM);
End update_LG_SUMMARY_event ;
------------------------------------------------------------------------------
--                                  M  A  I  N
------------------------------------------------------------------------------
 BEGIN
    p_ReturnCd := 0;

    Insert_LG_SUMMARY_XREF;
    -- Dbms_output.put_line('summaryID '||to_char(v_SUMMARYID)  );
    update_LG_SUMMARY;
    Update_lg_summary_mi;
    Update_MSGLENGTH_SUMMARY;
    insert_LG_Activity;
   IF (p_FAILDOCCNT > 0) then   --P_EVENTCD is NOT null and
       update_LG_SUMMARY_event;
   END IF;
   ProcAudit;
   Exception
        when others then
        ProcAudit;
        p_ReturnCd := 1;
        raise_application_error(-20999,20011||' LG_SUMMARYI_AH1 - Main '||SQLERRM);
 END ;
/


-- LG_SUMMARYS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_SUMMARYS_AH1
(
      p_ReturnCd       out   pls_INTEGER
    , p_DebugFlg       in    pls_INTEGER
    , RC1              out  db_ref.rct          -- rows
    , RC2              out  db_ref.rct          -- count
    , p_sortkey        in   varchar2
    , p_PageSizeNbr    in    pls_Integer
    , p_PageNbr        in    pls_Integer
    , p_Detail_Level   in    pls_integer             -- 1 = higher level of detail, 2= lower level of detail
    , p_FROMPARTNERID  in    varchar2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_TOPARTNERID    in    varchar2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_STARTDATE      in    DATE                    --    (Date and Time)
    , p_ENDDATE        in    DATE                    --    (Date and Time)
    , p_SRCTRGFLG      in    pls_Integer     default 0  --  (Source or Target Protocol/Process/Ver Indicator: 0 = Src, 1 = Trg)
    , p_PACKAGENAME    in    varchar2                --(From or To based on SRCTRGFLG)
    , p_PACKAGEVER     in    varchar2                --(From or To based on SRCTRGFLG)
    , p_PROTOCOLNAME   in    VARCHAR2                --(From or To based on SRCTRGFLG)
--  , p_PROTOCOLVER    in    VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROCESSCD      in    VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_PROCESSVER     in    VARCHAR2                --(From or To based on SRCTRGFLG)
    , p_DESTTypeId     in  number
    , p_MI_toggle      in  varchar2 default null   -- = 'MI' -- use Lg_summary_mi else use Lg_summary
    , p_Locale         in varchar2
)
AS
/*
================================================================================
Proc Name:      LG_SUMMARYS_AH1
Create Date:    01/04/2002
Author          Vipin Goel
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME         Version   DESCRIPTION
---------  --------------------  ---------------------------------------------
2005-11-09 S.Narain     6FP2      BGTRK00049515
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(4000);
  v_startdate        varchar2(20) := to_char(trunc(nvl(p_STARTDATE,sysdate-1), 'HH24'), 'dd-mm-yy hh24:mi:ss');
  v_enddate          varchar2(20) := to_char(trunc(nvl(p_EndDATE,sysdate)     ,'HH24'), 'dd-mm-yy hh24:mi:ss');
  StartRow           pls_integer;
  EndRow             pls_integer;
  sqlstmt            varchar2(8000);
  sql_Column         varchar2(2000);
  sql_inner          varchar2(4000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  sql_group          varchar2(1000);
  sql_group_count    varchar2(1000);
  frPartnerName      pr_partner.partnerName%type;
  ToPartnerName      pr_partner.partnerName%type;
  High               pls_integer := 1;
  Medium             pls_integer := 2;
  v_tableName        varchar2(30) ;
  v_dayHour          varchar2(30) ;
  v_ObjTypeKey       varchar2(10) := 'Process';
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
  DB_ProcAuditLogI_AH1('LG_SUMMARYs_Ah1',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'||','||'RC2'||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_FROMPARTNERID||','||
    p_TOPARTNERID  ||','||
    To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
    p_SRCTRGFLG    ||','||
    p_PACKAGENAME  ||','||
    p_PACKAGEVER   ||','||
    p_PROTOCOLNAME ||','||
--  p_PROTOCOLVER  ||','||
    p_PROCESSCD    ||','||
    p_PROCESSVER   ||','||
    p_DESTTYPEID
    ,0,0);
End ProcAudit;

Procedure where_clause as
begin
    v_where := ' Where '||v_dayHour||' between to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and to_date('''||
                          v_endDate||''',''dd-mm-yy hh24:mi:ss'')'||
  FN_Build_Where('FRPARTNERID',     'NUMBER_LIST', p_FROMPARTNERID)||
  FN_Build_Where('TOPARTNERID',     'NUMBER_LIST', p_ToPARTNERID)||
  FN_Build_Where('x.DESTTYPEID',      'NUMBER'   , p_DESTTYPEID);
  if nvl(p_SRCTRGFLG,0) = 0 then
  v_where := v_where ||
    FN_Build_Where('x.FRPACKAGENAME',  'VARCHAR2', p_PACKAGENAME)||
    FN_Build_Where('x.FRPACKAGEVER',   'VARCHAR2', p_PACKAGEVER)||
    FN_Build_Where('FrPROTOCOLNAME', 'VARCHAR2', p_PROTOCOLNAME)||
--  FN_Build_Where('FrPROTOCOLVER', 'VARCHAR2', p_PROTOCOLVER);
    FN_Build_Where('FrPROCESSCD',    'VARCHAR2', p_PROCESSCD)||
    FN_Build_Where('FrPROCESSVER',   'VARCHAR2', p_PROCESSVER);
  else
    v_where := v_where ||
    FN_Build_Where('x.ToPACKAGENAME',  'VARCHAR2', p_PACKAGENAME)||
    FN_Build_Where('x.ToPACKAGEVER',   'VARCHAR2', p_PACKAGEVER)||
    FN_Build_Where('ToPROTOCOLNAME', 'VARCHAR2', p_PROTOCOLNAME)||
--  FN_Build_Where('ToPROTOCOLVER', 'VARCHAR2', p_PROTOCOLVER);
    FN_Build_Where('ToPROCESSCD',    'VARCHAR2', p_PROCESSCD)||
    FN_Build_Where('ToPROCESSVER',   'VARCHAR2', p_PROCESSVER);
  end if;
end where_clause;
--
-- Main
--
BEGIN
   case p_MI_toggle
       when 'MI' then
           v_tableName := 'LG_SUMMARY_MI';
           v_dayHour   := 's.DAYHOURMI';
       else
           v_tableName := 'LG_SUMMARY'   ;
           v_dayHour   := 's.DAYHOUR';
  end case;
    p_ReturnCd := 0;

--    v_where := v_where||' Where s.dayHour between to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and to_date('''||v_endDate||''',''dd-mm-yy hh24:mi:ss'')';

    where_clause;
--    dbms_output.put_line(substr(v_where,1,250));
--    dbms_output.put_line(substr(v_where,251,250));
  If p_Detail_Level = High then
    sql_column :=
        'select FRPARTNERID,frp.partnername FRPARTNERNAME,TOPARTNERID,Top.partnername TOPARTNERNAME,
        FrPackageVer FrPackageVersion,FRPROCESSVER,ToPackageVer ToPackageVersion,TOPROCESSVER,
        FrPackageName,FRPROTOCOLNAME,FRPROCESSCD,FN_GET_RTOBJNAME(x.FrPRocessCd, x.FrProcessVer, ''Process'') FrProcessName,
        ToPackageName,TOPROTOCOLNAME,TOPROCESSCD,FN_GET_RTOBJNAME(x.TOPRocessCd, x.ToProcessVer, ''Process'') ToProcessName,
        sum(RCVDOCCNT) RCVDOCCNT,sum(SNTDOCCNT) SNTDOCCNT,sum(FAILDOCCNT) FAILDOCCNT,case when sum(RCVDOCCNT-(SNTDOCCNT + FAILDOCCNT))  > 0 then sum(RCVDOCCNT-(SNTDOCCNT + FAILDOCCNT)) else 0 end  INPROCESSCNT,
        to_char(SUM(nvl(MSGRAWLENGTHSUM_IN,0))) MSGRAWLENGTHSUM_IN ,
        to_char(SUM(nvl(MSGCONTENTLENGTHSUM_IN,0))) MSGCONTENTLENGTHSUM_IN,
        to_char(SUM(nvl(MSGRAWLENGTHSUM_OUT,0))) MSGRAWLENGTHSUM_OUT,
        to_char(SUM(nvl(MSGCONTENTLENGTHSUM_OUT,0))) MSGCONTENTLENGTHSUM_OUT ,
        to_char(SUM(nvl(MSGRAWLENGTHSUM_FAIL,0))) MSGRAWLENGTHSUM_FAIL,
        to_char(SUM(nvl(MSGCONTENTLENGTHSUM_FAIL,0))) MSGCONTENTLENGTHSUM_FAIL,
        case when sum(RCVDOCCNT-(SNTDOCCNT + FAILDOCCNT))>0 then to_CHAR(sum(MSGRAWLENGTHSUM_IN-(MSGRAWLENGTHSUM_OUT + MSGRAWLENGTHSUM_FAIL))) else to_CHAR(0) end  MSGRAWLENGTHSUM_INPROC ,
        case when sum(RCVDOCCNT-(SNTDOCCNT + FAILDOCCNT))>0 then to_CHAR(sum(MSGCONTENTLENGTHSUM_IN-(MSGCONTENTLENGTHSUM_OUT + MSGCONTENTLENGTHSUM_FAIL))) else to_CHAR(0) end  MSGCONTENTLENGTHSUM_INPROC ';
-- remove from columns  FrPackageVer, FRPROCESSVER, ToPackageVer, TOPROCESSVER,
    sql_group := 'FRPARTNERID, frp.partnername, TOPARTNERID, Top.partnername, FrPackageVer, FRPROCESSVER, ToPackageVer, TOPROCESSVER,
                FrPackageName,  FRPROTOCOLNAME, FRPROCESSCD,  FN_GET_RTOBJNAME(x.FrPRocessCd, x.FrProcessVer, ''Process''),
                ToPackageName,  TOPROTOCOLNAME, TOPROCESSCD,  FN_GET_RTOBJNAME(x.TOPRocessCd, x.ToProcessVer, ''Process'')' ;
-- Removed FRBUSINESSID ,ToBUSINESSID, x.DestTypeId from  sql_column, sql_group and sql_group_count
--  FrPackageVer, FRPROCESSVER, ToPackageVer, TOPROCESSVER,
    sql_group_count := 'FRPARTNERID, FrPackageName, FRPROTOCOLNAME, FRPROCESSCD, TOPARTNERID, ToPackageName, TOPROTOCOLNAME, TOPROCESSCD,
                       FrPackageVer, FRPROCESSVER, ToPackageVer, TOPROCESSVER';
-- Remove   FrPackageVer, FRPROCESSVER, ToPackageVer, TOPROCESSVER
    Elsif p_Detail_Level = Medium then
    sql_column :=
      'select FRPARTNERID,
      	      frp.partnername FRPARTNERNAME,
      	      TOPARTNERID,
      	      Top.partnername TOPARTNERNAME,
              sum(RCVDOCCNT) RCVDOCCNT,
              sum(SNTDOCCNT) SNTDOCCNT,
              sum(FAILDOCCNT) FAILDOCCNT,
              case when sum(RCVDOCCNT-(SNTDOCCNT + FAILDOCCNT))  > 0 then sum(RCVDOCCNT-(SNTDOCCNT + FAILDOCCNT)) else 0 end  INPROCESSCNT,
              to_char(SUM(nvl(MSGRAWLENGTHSUM_IN,0))) MSGRAWLENGTHSUM_IN,
              to_char(SUM(nvl(MSGCONTENTLENGTHSUM_IN,0))) MSGCONTENTLENGTHSUM_IN,
              to_char(SUM(nvl(MSGRAWLENGTHSUM_OUT,0))) MSGRAWLENGTHSUM_OUT,
              to_char(SUM(nvl(MSGCONTENTLENGTHSUM_OUT,0))) MSGCONTENTLENGTHSUM_OUT,
              to_char(SUM(nvl(MSGRAWLENGTHSUM_FAIL,0))) MSGRAWLENGTHSUM_FAIL,
              to_char(SUM(nvl(MSGCONTENTLENGTHSUM_FAIL,0))) MSGCONTENTLENGTHSUM_FAIL,
              case when sum(RCVDOCCNT-(SNTDOCCNT + FAILDOCCNT))>0 then to_CHAR(sum(MSGRAWLENGTHSUM_IN-(MSGRAWLENGTHSUM_OUT + MSGRAWLENGTHSUM_FAIL))) else to_CHAR(0) end  MSGRAWLENGTHSUM_INPROC ,
              case when sum(RCVDOCCNT-(SNTDOCCNT + FAILDOCCNT))>0 then to_CHAR(sum(MSGCONTENTLENGTHSUM_IN-(MSGCONTENTLENGTHSUM_OUT + MSGCONTENTLENGTHSUM_FAIL))) else to_CHAR(0) end  MSGCONTENTLENGTHSUM_INPROC ';
    sql_group := 'FRPARTNERID, frp.partnername, TOPARTNERID, Top.partnername';
    sql_group_count := 'FRPARTNERID, TOPARTNERID';
  End if;
  case p_MI_toggle when 'MI' then v_tableName := 'LG_SUMMARY_MI';
                             else v_tableName := 'LG_SUMMARY'   ;
  end case;
  sql_inner :=
    'from lg_summary_xref_vw x Inner Join '||v_tableName||' s  on x.summaryID = s.summaryID';
    dbms_output.put_line ( 'Sql_Inner : '||sql_inner);
--     Inner Join lg_summary s         on x.summaryID    = s.summaryID';
  sql_outer :=
    'Left Outer Join pr_partner Frp  on x.FRPARTNERID  = frp.partnerId
    Left Outer Join pr_partner Top  on x.ToPARTNERID  = Top.partnerId
   ';

  sqlstmt := sql_column||' '||sql_inner||' '||sql_outer||' '||v_where||
    ' group by  '||sql_group;
-- total row count Cursor
/*
    dbms_output.put_line ( 'Open RC2....');
    dbms_output.put_line ( 'select count(*) TotalRowsNbr from (select count(*) '||sql_inner);
    dbms_output.put_line ( v_where);
    dbms_output.put_line (' group by  '||sql_group_count||')' );
*/
    if RC2%IsOpen then close RC2; end if;
    if NVL(p_pageNbr,0) in (0,1) then
      open RC2 for 'select count(*) TotalRowsNbr from (select count(*) '||sql_inner||v_where||
      ' group by  '||sql_group_count||')';
    end if;
-- Order by Logic
    if p_sortkey is not null then
        --EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);
        sqlstmt := sqlstmt ||' order by '||p_sortkey;
    end if;
    DB_Print('sqlstmt: ', sqlstmt);

-- Paging logic
      sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);

-- Open Cursor
--    dbms_output.put_line ( 'Open RC1....');

    if RC1%IsOpen then close RC1; end if;
    open RC1 for sqlstmt;

    --IF p_sortkey is not null THEN
        --EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';
    --END IF;

-- proc Audit
    ProcAudit;
    Exception
        when others then
        ProcAudit;
        p_ReturnCd := 1;
END ;
/


-- LG_SUMMARYS_AH2.sql

CREATE OR REPLACE PROCEDURE LG_SUMMARYS_AH2 (
      p_ReturnCd           out pls_INTEGER
    , p_DebugFlg                in  pls_INTEGER
    , RC1                   out db_ref.RCT       -- rows
    , p_PartnerIdList       in  varchar2                --    (All = NULL, One, Comma Seperated list of Id's)
    , p_STARTDate           in  DATE                    --    (Date and Time)
    , p_ENDDate             in  DATE                    --    (Date and Time)
    , p_Threshold           in  pls_integer
--    , p_VISIBILITY     in pls_integer
    , p_partner_VISIBILITY  in pls_integer
    )
AS
/*
================================================================================
Proc Name:      LG_SUMMARYS_AH2
Create Date:    2002-09-06
Author          J.Chen
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME         Version   DESCRIPTION
---------  --------------------  ---------------------------------------------
2002-09-06 J.Chen       3.3       Created
2002-09-11 V.Goel       3.3       Gave alias to column names ie. sum(ReceivedCount) as ReceivedCount
2002-10-24 J.Chen       3.3       Switch to lg_summary instead of lg_activity
2003-02-04 V Goel       3.4       Add p_partner_visibility, remove p_VISIBILITY (unused)
2003-07-08 V.Goel       3.5       Use FN_Build_Where
2005-11-09 S.Narain     6FP2      BGTRK00049515
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
  v_startdate        varchar2(20) := to_char(trunc(nvl(p_STARTDATE,sysdate-1), 'HH24'), 'dd-mm-yy hh24:mi:ss');
  v_enddate          varchar2(20) := to_char(trunc(nvl(p_EndDATE,sysdate)     ,'HH24'), 'dd-mm-yy hh24:mi:ss');
  v_count            pls_integer;
  v_count2           pls_integer;
  v_threshold        integer;
  v_sortKey          varchar2(1000);
  sqlstmt            varchar2(4000);
  sql_Column         varchar2(2000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  frPartnerName      pr_partner.partnerName%type;
  ToPartnerName      pr_partner.partnerName%type;
----------
---------------
-- Procedures
---------------
------------
-- ProcAudit
------------
Procedure ProcAudit is
v_diffTS Timestamp;
begin
-- If p_DebugFlg=1  Then
  DB_ProcAuditLogI_ah1('LG_SUMMARYS_AH2',v_beginTS,'p_ReturnCd'||','||p_DEBUGflg||
    ','||'RC1'||','||
    p_PartnerIdList ||','||
    To_char(p_STARTDATE,'DD-mm-YY HH24:mi:ss')||','||
    To_char(p_ENDDATE,'DD-mm-YY HH24:mi:ss')  ||','||
    p_Threshold ||','||
    p_partner_VISIBILITY
    );

--End if ;
End ProcAudit;
---------------
-- Where_clause
---------------
Procedure where_clause as
begin
    v_where := v_where||FN_Build_Where('x.FRPARTNERID',  'NUMBER_LIST', p_PARTNERIDList);
end where_clause;
--                                M  A  I  N
BEGIN
    p_ReturnCd := 0;
    v_where := v_where||' Where  s.dayhour between '||chr(10)||' to_date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and'||
                                                       chr(10)||' to_date('''|| v_endDate||''',''dd-mm-yy hh24:mi:ss'')';
   IF p_partner_VISIBILITY = 1 then
       v_where := v_where || ' and Frpt.visibility = 1 ' ;  --(Frpt.visibility = 1 or Topt.visibility = 1)
    END IF;
    where_clause;
    dbms_output.put_line(substr(v_where,1,250));
    dbms_output.put_line(substr(v_where,251,250));
sql_inner :=
             'from lg_summary s
             inner join lg_summary_xref x    on s.summaryId       = x.summaryId
             left outer join pr_partner Frp  on x.FRPARTNERID     = Frp.partnerId
             Inner Join pr_partnerType Frpt  on Frp.PARTNERTYPEID = Frpt.PARTNERTYPEID';
-- left outer join pr_partner Top  on x.ToPARTNERID     = Top.partnerId
-- Inner Join pr_partnerType Topt  on Top.PARTNERTYPEID = Topt.PARTNERTYPEID
IF p_Threshold IS NOT NULL THEN
   v_threshold := p_threshold + 1;
   sqlstmt := 'select * from ('||
              ' select x.FrPartnerId AS PartnerId, '||
                     ' frp.PartnerName, '||
                     ' sum(s.RCVDOCCNT) as ReceivedCount, '||
                     ' sum(s.SNTDOCCNT)  as SuccessCount, '||
                     ' sum(s.FailDOCCNT) as FailedCount, '||
                     ' sum(greatest(s.RCVDOCCNT - (s.SNTDOCCNT + s.FailDOCCNT),0)) AS InProcessCount '||
              sql_inner||' '||v_where ||' group by x.FrPARTNERID, frp.PartnerName order by ReceivedCount DESC) '||
            ' Where rownum < '|| v_threshold ;
ELSE
    sqlstmt := ' select x.FrPartnerId As PartnerId, '||
                     ' frp.PartnerName, '||
                     ' sum(s.RCVDOCCNT) as ReceivedCount, '||
                     ' sum(s.SNTDOCCNT)  as SuccessCount, '||
                     ' sum(s.FailDOCCNT) as FailedCount, '||
                     ' sum(greatest(s.RCVDOCCNT - (s.SNTDOCCNT + s.FailDOCCNT),0)) AS InProcessCount '||
              sql_inner||' '||v_where ||' group by x.FrPARTNERID, frp.PartnerName order by ReceivedCount DESC';
END IF;
 DB_Print('Sql for rc1 ', sqlstmt);
-- Open Cursor
    if RC1%IsOpen then close RC1; end if;
    open rc1 for sqlstmt ;
-- proc Audit
  ProcAudit;
Exception
 when others then
  ProcAudit;
  raise_application_error(-20999,20002||'-'||'Error occurred in LG_SUMMARYS_AH2:'||SQLERRM);
  p_ReturnCd := 1;
END;
--/
/


-- LG_SUMMARY_ACTIVITY.SQL

CREATE OR REPLACE PROCEDURE LG_SUMMARY_ACTIVITY (
		p_ReturnCd	in out	pls_integer
		,p_DebugFlg	in	pls_integer
) AS
/*
================================================================================
Proc Name:      LG_SUMMARY_ACTIVITY
Create Date:    12/03/04
Author          S.Priani

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2004-03-22 S.Priani	4.2.2	Created
	2005-09-19 S.Priani	6.0.0.2	BGTRK00055950
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_BeginTS	timestamp	:= FN_Get_Date; -- Performance related variables
	v_ReturnCd	pls_integer;
	CURSOR cur_lg_unp_act IS
		SELECT	lua.ROWID, lua.*
		FROM	lg_unprocessed_activity lua;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_DocProcessing_SumM',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    , p_DebugFlg
    );
End ProcAudit;

--------------------------------------------------------------------------
--                 M  A  I  N
--------------------------------------------------------------------------
BEGIN
	FOR rec_lg_unp_act IN cur_lg_unp_act LOOP
		BEGIN
		LG_SUMMARYI_AH1
			(
			v_ReturnCd
			,p_DebugFlg
		       	,rec_lg_unp_act.VUID
			,rec_lg_unp_act.RCVDOCTS
			,rec_lg_unp_act.SNTDOCTS
			,rec_lg_unp_act.INITBUSINESSID
			,rec_lg_unp_act.FRBUSINESSID
			,rec_lg_unp_act.TOBUSINESSID
			,rec_lg_unp_act.FRPROTOCOLNAME
			,rec_lg_unp_act.TOPROTOCOLNAME
			,rec_lg_unp_act.FRPROCESSCD
			,rec_lg_unp_act.FRPROCESSVER
			,rec_lg_unp_act.TOPROCESSCD
			,rec_lg_unp_act.TOPROCESSVER
			,rec_lg_unp_act.FRIPADDRESS
			,rec_lg_unp_act.CHANNELSTATUSNAME
			,rec_lg_unp_act.RCVDOCCNT
			,rec_lg_unp_act.SNTDOCCNT
			,rec_lg_unp_act.FAILDOCCNT
			,rec_lg_unp_act.EVENTCD
			,rec_lg_unp_act.MSGRAWLENGTH
			,rec_lg_unp_act.MSGCONTENTLENGTH
			,rec_lg_unp_act.FRPROTOCOLVER
			,rec_lg_unp_act.TOPROTOCOLVER
			,rec_lg_unp_act.DESTNAME
			,rec_lg_unp_act.DMDURATION_MS
			,rec_lg_unp_act.TRANSPORTDURATION_MS
			,rec_lg_unp_act.TRANSPORTATTEMPTCNT
			,rec_lg_unp_act.GATEWAYATTEMPTCNT
			,rec_lg_unp_act.PERSISTDOCTS
			,rec_lg_unp_act.BPESTARTTS
			,rec_lg_unp_act.BPEENDTS
			,rec_lg_unp_act.PROVSESSIONID
			,rec_lg_unp_act.FRTRANSPORTTYPE
			,rec_lg_unp_act.TOTRANSPORTTYPE
			,rec_lg_unp_act.FRPACKAGENAME
			,rec_lg_unp_act.FRPACKAGEVER
			,rec_lg_unp_act.TOPACKAGENAME
			,rec_lg_unp_act.TOPACKAGEVER
			,rec_lg_unp_act.SYNCHRONOUSFLG
			,rec_lg_unp_act.ACTIONFLG
			);

			DELETE	LG_UNPROCESSED_ACTIVITY
			WHERE	ROWID = rec_lg_unp_act.ROWID;
		EXCEPTION
			When Others Then
				p_ReturnCd := 1;
				Raise;
		END;
	END LOOP;
	ProcAudit;
	p_ReturnCd := 0;
	COMMIT;
EXCEPTION
	When Others Then
		ProcAudit;
		p_ReturnCd := 1;
		Raise_application_error (-20999,'20000 LG_SUMMARY_ACTIVITY ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_SUMMARY_AS_PRC.SQL

CREATE OR REPLACE PROCEDURE LG_SUMMARY_AS_PRC (
		p_ReturnCd	in out	pls_integer
		,p_DebugFlg	in	pls_integer
) AS
/*
================================================================================
Proc Name:      LG_SUMMARY_AS_PRC
Create Date:    12/03/04
Author          S.Priani

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2004-03-22 S.Priani	4.2.2	Created
	2005-09-19 S.Priani	6.0.0.2	BGTRK00055950
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_BeginTS	timestamp	:= FN_Get_Date; -- Performance related variables
	v_ReturnCd	pls_integer;
	CURSOR cur_lg_unp_as IS
		SELECT	lua.ROWID, lua.*
		FROM	lg_unprocessed_as lua;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_SUMMARY_AS_PRC',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    , p_DebugFlg
    );
End ProcAudit;

--------------------------------------------------------------------------
--                 M  A  I  N
--------------------------------------------------------------------------
BEGIN
	FOR rec_lg_unp_as IN cur_lg_unp_as LOOP
		BEGIN
		LG_PROCESSSUMMARY_ASI_AH1
			(
			v_ReturnCd
			,p_DebugFlg
			,rec_lg_unp_as.INITBUSINESSID   
			,rec_lg_unp_as.INITASMESSAGEID
			,rec_lg_unp_as.TOBUSINESSID   
			,rec_lg_unp_as.RCVDOCTS       
			,rec_lg_unp_as.DOCCNT         
			,rec_lg_unp_as.RCVPROCESSCNT  
			,rec_lg_unp_as.MDNCNT         
			,rec_lg_unp_as.DISPWARNGCNT   
			,rec_lg_unp_as.DISPERRORCNT   
			,rec_lg_unp_as.DISPFAILURECNT 
			,rec_lg_unp_as.SUCCESSCNT     
			);

			DELETE	lg_unprocessed_as
			WHERE	ROWID = rec_lg_unp_as.ROWID;
		EXCEPTION
			When Others Then
				p_ReturnCd := 1;
				Raise;
		END;
	END LOOP;
	ProcAudit;
	p_ReturnCd := 0;
	COMMIT;
EXCEPTION
	When Others Then
		ProcAudit;
		p_ReturnCd := 1;
		Raise_application_error (-20999,'20000 LG_SUMMARY_AS_PRC ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_SUMMARY_DOCPROC.SQL

CREATE OR REPLACE PROCEDURE LG_SUMMARY_DOCPROC (
		p_ReturnCd	in out	pls_integer
		,p_DebugFlg	in	pls_integer
) AS
/*
================================================================================
Proc Name:      LG_SUMMARY_DOCPROC
Create Date:    12/03/04
Author          S.Priani

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2004-03-22 S.Priani	4.2.2	Created
	2005-09-19 S.Priani	6.0.0.2	BGTRK00055950
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_BeginTS	timestamp	:= FN_Get_Date; -- Performance related variables
	v_ReturnCd	pls_integer;
	CURSOR cur_lg_unp_docproc IS
		SELECT	lua.ROWID, lua.*
		FROM	lg_unprocessed_docproc lua;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_Summary_DocProc',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    , p_DebugFlg
    );
End ProcAudit;

--------------------------------------------------------------------------
--                 M  A  I  N
--------------------------------------------------------------------------
BEGIN
	FOR rec_lg_unp_docproc IN cur_lg_unp_docproc LOOP
		BEGIN
		LG_DOCPROCESSING_SUMM
			(
			v_ReturnCd
			,p_DebugFlg
		       	,rec_lg_unp_docproc.RCVDOCTS
		       	,rec_lg_unp_docproc.FRIPADDRESS
		       	,rec_lg_unp_docproc.PROVSESSIONID
		       	,rec_lg_unp_docproc.RCVDOCCNT
		       	,rec_lg_unp_docproc.PERSISTDOCCNT
		       	,rec_lg_unp_docproc.PROCESSSTARTCNT
		       	,rec_lg_unp_docproc.PROCESSENDCNT
		       	,rec_lg_unp_docproc.SUCCESSCNT
		       	,rec_lg_unp_docproc.FAILCNT
			);

			DELETE	lg_unprocessed_docproc
			WHERE	ROWID = rec_lg_unp_docproc.ROWID;
		EXCEPTION
			When Others Then
				p_ReturnCd := 1;
				Raise;
		END;
	END LOOP;
	ProcAudit;
	p_ReturnCd := 0;
	COMMIT;
EXCEPTION
	When Others Then
		ProcAudit;
		p_ReturnCd := 1;
		Raise_application_error (-20999,'20000 LG_SUMMARY_DOCPROC ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_SUMMARY_EVENTS_AH1.sql

CREATE OR REPLACE PROCEDURE LG_SUMMARY_EVENTS_AH1
(
p_ReturnCd         OUT   integer
, p_DebugFlg        IN    pls_integer
, RC1               OUT   db_ref.RCT
, p_STARTDATE       IN    TimeStamp                    --required (Date and Time
, p_ENDDATE         IN    Timestamp
, P_FROMPARTNERID   IN    INTEGER      --required List
, P_TOPARTNERID     IN    INTEGER      --required List
-- From
, p_FrPACKAGENAME   IN    varchar  --255)
, p_FrPACKAGEVER    IN    varchar  --60)
, p_FrPROTOCOLNAME  IN    VARCHAR  --255)
, p_FrPROTOCOLVer   IN    VARCHAR  --60)
, p_FrPROCESSCD     IN    VARCHAR  --255)
, p_FrPROCESSVER    IN    VARCHAR  --60)
-- To
, p_ToPACKAGENAME   IN    varchar  --255)
, p_ToPACKAGEVER    IN    varchar  --60)
, p_ToPROTOCOLNAME  IN    VARCHAR  --255)
, p_ToPROTOCOLVer   IN    VARCHAR  --60)
, p_ToPROCESSCD     IN    VARCHAR  --255)
, p_ToPROCESSVER    IN    VARCHAR  --60)
) As
/*
================================================================================
Proc Name:      LG_SUMMARY_EVENTS_AH1
Create Date:    2002/06/19 12:56:47 PM
Author          V Goel
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_error            integer     := 0;
    v_errmsg           varchar2(255);
    v_rowcnt           integer    :=0;
    v_BeginTS          timestamp := FN_Get_Date; -- Performance related variables
    sqlWhere           varchar2(1000);
    v_startdate        varchar2(20) := to_char(nvl(p_STARTDATE,sysdate-1), 'dd-mm-yy hh24:mi:ss');
    v_enddate          varchar2(20) := to_char(nvl(p_EndDATE,sysdate)    , 'dd-mm-yy hh24:mi:ss');
    v_ToPACKAGENAME    varchar(255);
    v_ToPACKAGEVER     varchar(60);

Procedure ProcAudit is
begin
        DB_ProcAuditLogI_ah1('LG_SUMMARY_EVENTS_AH1',v_beginTS,
        'p_ReturnCd'||','||p_DebugFlg||','||'RC1'
        ||','||To_TimeStamp(p_STARTDATE,'DD-mm-YY HH24:mi:ssxFF')
        ||','||To_TimeStamp(p_ENDDATE,  'DD-mm-YY HH24:mi:ssxFF')
        ||','||p_fromPartnerId
        ||','||p_ToPartnerId
        ||','||p_FrPACKAGENAME
		||','||p_FrPACKAGEVer
        ||','||p_FrProtocolName
		||','||p_FrProtocolVer
        ||','||p_FrProcessCD
        ||','||p_FrPROCESSVER
        ||','||p_ToPACKAGENAME
		||','||p_ToPACKAGEVer
        ||','||p_ToProtocolName
		||','||p_ToProtocolVer
        ||','||p_ToProcessCD
        ||','||p_ToProcessVer
        );
End ProcAudit;
--
Procedure where_clause as
begin
    sqlwhere := ' Where  se.DayHour between '||
                chr(10)||' to_Date('''||v_startDate||''',''dd-mm-yy hh24:mi:ss'') and'||
                chr(10)||' to_Date('''||v_endDate||''',''dd-mm-yy hh24:mi:ss'')'||
    FN_Build_Where('x.FRPARTNERID',     'NUMBER', p_FROMPARTNERID)||
    FN_Build_Where('x.TOPARTNERID',     'NUMBER', p_ToPARTNERID)  ;

    sqlwhere := sqlwhere ||
	FN_Build_Where('x.FrPACKAGENAME',  'VARCHAR2', p_FrPACKAGENAME)||
	FN_Build_Where('x.FrPACKAGEVer',   'VARCHAR2', p_FrPACKAGEVer)||
    FN_Build_Where('x.FrPROTOCOLNAME', 'VARCHAR2', p_FrPROTOCOLNAME)||
	FN_Build_Where('x.FrPROTOCOLVER',  'VARCHAR2', p_FrPROTOCOLVer)||
    FN_Build_Where('x.FrPROCESSCD',    'VARCHAR2', p_FrPROCESSCD)   ||
    FN_Build_Where('x.FrPROCESSVER',   'VARCHAR2', p_FrPROCESSVER);

    -- Match router input data
    IF p_ToPACKAGENAME = 'Unknown' THEN
        v_ToPACKAGENAME := 'AND x.ToPACKAGENAME IN (''Unknown'', ''-1'')';
    ELSE
        SELECT FN_Build_Where('x.ToPACKAGENAME',  'VARCHAR2', p_ToPACKAGENAME) INTO v_ToPACKAGENAME FROM DUAL;
    END IF;

    IF p_ToPACKAGEVER = 'Unknown' THEN
        v_ToPACKAGEVER := 'AND x.ToPACKAGEVER IN (''Unknown'', ''-1'')';
    ELSE
        SELECT FN_Build_Where('x.ToPACKAGEVER',   'VARCHAR2', p_ToPACKAGEVER) INTO v_ToPACKAGEVER FROM DUAL;
    END IF;

    sqlwhere := sqlwhere ||
    v_ToPACKAGENAME ||
    v_ToPACKAGEVER ||
    FN_Build_Where('x.ToPROTOCOLNAME', 'VARCHAR2', p_ToPROTOCOLNAME)||
	FN_Build_Where('x.ToPROTOCOLVER',  'VARCHAR2', p_ToPROTOCOLVer)||
    FN_Build_Where('x.ToPROCESSCD',    'VARCHAR2', p_ToPROCESSCD)   ||
    FN_Build_Where('x.ToPROCESSVER',   'VARCHAR2', p_ToPROCESSVER);


end where_clause;
-------------------------------------------------------------------------
--                              M A I N
-------------------------------------------------------------------------
BEGIN
    p_ReturnCd := 0;
    where_clause;
    DB_Print('Where Clause', sqlWhere );
    Open RC1 for
    'select se.EVENTCD
    	,   RTRIM(cd.EVENTNAMEKEY) EVENTNAMEKEY
    	,   sum(se.EVENTCNT) DOCUMENTCOUNT
		,   cd.eventnameresourcebundleid 
    From LG_SUMMARY_XREF_vw x
    inner Join LG_SUMMARY_EVENT se   on x.summaryID = se.summaryID
    Left Outer join lg_eventCD           cd  on se.eventcd      = cd.eventcd '  -- 11/21/2002
    ||sqlWhere|| ' group by se.EVENTCD, RTRIM(cd.EVENTNAMEKEY), cd.eventnameresourcebundleid';


EXCEPTION
    When NO_DATA_FOUND Then Null;
    When Others Then
        ProcAudit;
        p_ReturnCd := 1;
        Raise_application_error (-20999,'20000 LG_SUMMARY_EVENTS_AH1' || SQLCODE|| ' '||SQLERRM);

End LG_SUMMARY_EVENTS_AH1;
/


-- LG_SUMMARY_PROCESSSUMMARYI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_SUMMARY_PROCESSSUMMARYI_AH1
        (p_ReturnCd out integer
        ,p_DebugFlg in pls_integer
        ,p_SUMMARYID in NUMBER
        ,p_PROCESSSUMMARYID in NUMBER
        )
AS

/*
        ================================================================================
        Proc Name:       LG_SUMMARY_PROCESSSUMMARYI_AH1
        Create Date:     2003-02-19 15:20:47
        Author:          SYSTEM

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ExceptionFlg    integer := 0;
v_PK_Value        number := 0;

Procedure ProcAudit is
Begin
        DB_ProcAuditLogI_Ah1('LG_SUMMARY_PROCESSSUMMARYI_AH1',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
                ||','||'RC1'
                ||','||p_SUMMARYID
                ||','||p_PROCESSSUMMARYID
                , p_DebugFlg);
End ProcAudit;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        Insert into lg_summary_processSummary(
                SUMMARYID
                ,PROCESSSUMMARYID
        ) Values (
                p_SUMMARYID
                ,p_PROCESSSUMMARYID
        );
        ProcAudit;
        EXCEPTION
                When NO_DATA_FOUND THEN
                        ProcAudit;
                When DUP_VAL_ON_INDEX then
                        ProcAudit;
                        p_returnCd := 0;
                When OTHERS Then
                        RAISE_APPLICATION_ERROR (-20999,'20000 LG_SUMMARY_PROCESSSUMMARYI_AH1' || SQLCODE|| ' '||SQLERRM);
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        ProcAudit;
END LG_SUMMARY_PROCESSSUMMARYI_AH1;
/


-- LG_SUMMARY_RNI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_SUMMARY_RNI_AH1
(
    p_ReturnCd           out integer,
    p_DebugFlg           in pls_integer
    -- RN
    ,P_RCVDOCTS          in   DATE
    ,p_DOCCNT            in pls_integer
    ,p_RCVPROCESSCNT     in pls_integer
    ,p_ACTIONCNT         in pls_integer
    ,p_SIGNALCNT         in pls_integer
    ,p_ACTIONDURATIONSUM in pls_integer
    ,p_SIGNALDURATIONSUM in pls_integer
    ,p_AACNT             in pls_integer
    ,p_GECNT             in pls_integer
    ,p_RACNT             in pls_integer
    ,p_RAECNT            in pls_integer
    ,p_AAECNT            in pls_integer
    ,p_TIMETOPERFORM     in pls_integer
    ,p_ref_summaryID     in pls_integer
    ,p_ref_RcvDocTS      in date
    ,p_summaryID         in pls_integer
    ,p_UnknownCnt        in pls_integer
    ) as
/*
================================================================================
Proc Name:        LG_Summary_RNI_AH1
Create Date:     2002-09-27 09:55:23
Author:            SYSTEM
================================================================================
Return Code : 0 = OK,
              1 = Not Good,
              2 = Dup,
              3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt     integer     :=0;
v_BeginTS    timestamp := FN_GET_DATE(); -- Performance related variables
v_pk_value   pls_integer := 0;
v_summaryid  lg_sum_xref_Lnk.summaryid%type ;
v_rcode      pls_integer;
v_ProcName   varchar2(30) := 'LG_SUMMARY_RNI_AH1';
v_UpdCount    pls_integer;
--
zeroUpdate  Exception;
--
-- Procedures
--

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1(v_ProcName,v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||P_RCVDOCTS
    ||','||p_DOCCNT
    ||','||p_RCVPROCESSCNT
    ||','||p_ACTIONCNT
    ||','||p_SIGNALCNT
    ||','||p_ACTIONDURATIONSUM
    ||','||p_SIGNALDURATIONSUM
    ||','||p_AACNT
    ||','||p_GECNT
    ||','||p_RACNT
    ||','||p_RAECNT
    ||','||p_TIMETOPERFORM
    ||','||p_ref_summaryID
    ||','||p_UnknownCnt
    , p_DebugFlg
    );
End ProcAudit;
--
--

Procedure Do_0A1 is
begin
--
-- Lg_summary_rn
--
    Begin
        Update lg_summary_rn S
            Set s.zeroA1CNT = nvl(s.zeroA1CNT,0) + 1
          ,    s.Updatedate = FN_GET_DATE()
        Where
            S.summaryid = p_ref_summaryID
            and  S.dayHour = Trunc(P_Ref_RCVDOCTS, 'HH24')
        ;
        IF sql%RowCount = 0 Then
            Insert into lg_summary_rn s
            (
                s.SUMMARYID
           ,    s.DAYHOUR
           ,   s.zeroA1CNT
            ) Values (
                 p_ref_summaryID
           ,     Trunc(P_Ref_RCVDOCTS, 'HH24')
           ,     1
           );
        End If;
    End;
--
-- Lg_summary_rn_MI
--
    Begin
        Update lg_summary_rn_MI S
            Set s.zeroA1CNT = nvl(s.zeroA1CNT,0) + 1
          ,    s.Updatedate = FN_GET_DATE()
        Where
            S.summaryid = p_ref_summaryID
            and  S.dayHourMI = FN_tenMinuteDate(P_Ref_RCVDOCTS)
        ;
        IF sql%RowCount = 0 Then
            Insert into lg_summary_rn_MI s
            (
                s.SUMMARYID
           ,    s.DAYHOURMI
           ,   s.zeroA1CNT
            ) Values (
                 p_ref_summaryID
           ,     FN_tenMinuteDate(P_Ref_RCVDOCTS)
           ,     1
           );
        End If;
    End;
EXCEPTION
When Others Then
    ProcAudit;
    p_ReturnCd := 1;
    Raise_application_error (-20999,'20000 LG_Summary_RNI_AH1 - Do_0A1: ' ||SQLERRM);
end Do_0A1  ;
--
Procedure Upd_LGSummaryRn is
Begin
     UPDATE lg_summary_rn S set
            s.DOCCNT            = s.DOCCNT          +   nvl(p_DOCCNT,0)
       ,    s.RCVPROCESSCNT     = s.RCVPROCESSCNT   +   nvl(p_RCVPROCESSCNT,0)
       ,    s.ACTIONCNT         = s.ACTIONCNT       +   nvl(p_ACTIONCNT,0)
       ,    s.SIGNALCNT         = s.SIGNALCNT       +   nvl(p_SIGNALCNT,0)
       ,    s.ACTIONDURATIONSUM = s.ACTIONDURATIONSUM + nvl(p_ACTIONDURATIONSUM,0)
       ,    s.SIGNALDURATIONSUM = s.SIGNALDURATIONSUM + nvl(p_SIGNALDURATIONSUM,0)
       ,    s.AACNT             = s.AACNT           +   nvl(p_AACNT,0)
       ,    s.GECNT             = s.GECNT           +   nvl(p_GECNT,0)
       ,    s.RACNT             = s.RACNT           +   nvl(p_RACNT,0)
       ,    s.RAECNT            = s.RAECNT          +   nvl(p_AAECNT,0)
       ,    s.REALTIMETOPERFORM = s.REALTIMETOPERFORM + nvl(p_TIMETOPERFORM,0)
       ,    s.UnknownCnt        = s.UnknownCnt      +   nvl(p_UnknownCnt,0)
       ,    s.Updatedate = FN_GET_DATE()
    WHERE
        S.summaryid = p_summaryId and  S.dayHour = Trunc(P_RCVDOCTS, 'HH24');

    v_UpdCount := sql%RowCount;

EXCEPTION
When Others Then
    ProcAudit;
    p_ReturnCd := 1;
    Raise_application_error (-20999,'20000 LG_Summary_RNI_AH1 - Upd_LgSummaryRn: ' ||SQLERRM);

end Upd_LGSummaryRn;
--
Procedure Upd_LGSummaryRnMi is
Begin
     UPDATE lg_summary_rn_MI S set
            s.DOCCNT            = s.DOCCNT          +   nvl(p_DOCCNT,0)
       ,    s.RCVPROCESSCNT     = s.RCVPROCESSCNT   +   nvl(p_RCVPROCESSCNT,0)
       ,    s.ACTIONCNT         = s.ACTIONCNT       +   nvl(p_ACTIONCNT,0)
       ,    s.SIGNALCNT         = s.SIGNALCNT       +   nvl(p_SIGNALCNT,0)
       ,    s.ACTIONDURATIONSUM = s.ACTIONDURATIONSUM + nvl(p_ACTIONDURATIONSUM,0)
       ,    s.SIGNALDURATIONSUM = s.SIGNALDURATIONSUM + nvl(p_SIGNALDURATIONSUM,0)
       ,    s.AACNT             = s.AACNT           +   nvl(p_AACNT,0)
       ,    s.GECNT             = s.GECNT           +   nvl(p_GECNT,0)
       ,    s.RACNT             = s.RACNT           +   nvl(p_RACNT,0)
       ,    s.RAECNT            = s.RAECNT          +   nvl(p_AAECNT,0)
       ,    s.REALTIMETOPERFORM = s.REALTIMETOPERFORM + nvl(p_TIMETOPERFORM,0)
       ,    s.UnknownCnt        = s.UnknownCnt      +   nvl(p_UnknownCnt,0)
       ,    s.Updatedate = FN_GET_DATE()
    WHERE
        S.summaryid = p_summaryId and  S.dayHourMi = FN_tenMinuteDate(P_RCVDOCTS);

    v_UpdCount := sql%RowCount;

EXCEPTION
When Others Then
    ProcAudit;
    p_ReturnCd := 1;
    Raise_application_error (-20999,'20000 LG_Summary_RNI_AH1 - Upd_LgSummaryRnMi: ' ||SQLERRM);

end Upd_LGSummaryRnMi;
Procedure Update_lg_summary_rn_mi is
Begin
    Upd_LGSummaryRnMi;
    IF v_UpdCount = 0 Then
    Begin
         Insert into lg_summary_rn_mi s
         (
            s.SUMMARYID
        ,    s.DAYHOURmi
        ,    s.DOCCNT
        ,    s.RCVPROCESSCNT
        ,    s.ACTIONCNT
        ,    s.SIGNALCNT
        ,    s.ACTIONDURATIONSUM
        ,    s.SIGNALDURATIONSUM
        ,    s.AACNT
        ,    s.GECNT
        ,    s.RACNT
        ,    s.RAECNT
        ,    s.AAECNT
        ,    s.REALTIMETOPERFORM
        ,    s.UnknownCnt
        )
        Values
        (
            p_summaryId
       ,    FN_tenMinuteDate(P_RCVDOCTS)
       ,    nvl(p_DOCCNT,0)
       ,    nvl(p_RCVPROCESSCNT,0)
       ,    nvl(p_ACTIONCNT,0)
       ,    nvl(p_SIGNALCNT,0)
       ,    nvl(p_ACTIONDURATIONSUM,0)
       ,    nvl(p_SIGNALDURATIONSUM,0)
       ,    nvl(p_AACNT,0)
       ,    nvl(p_GECNT,0)
       ,    nvl(p_RACNT,0)
       ,    nvl(p_RAECNT,0)
       ,    nvl(p_AAECNT,0)
       ,    nvl(p_TIMETOPERFORM,0)
       ,    nvl(p_UnknownCnt,0)
        );
Exception
    When Dup_Val_on_Index Then
        Upd_LGSummaryRnMi;
    when others then
        ProcAudit;
        p_ReturnCd := 1;
        raise_application_error(-20999,'20000 LG_Summary_RNI_AH1 - Update_lg_summary_mi '||SQLERRM);
End;
End if;

End Update_Lg_Summary_rn_MI;
--
Procedure Update_lg_summary_rn is
Begin
    Upd_LGSummaryRn;
    IF v_UpdCount = 0 Then
    Begin
         Insert into lg_summary_rn s
         (
            s.SUMMARYID
        ,    s.DAYHOUR
        ,    s.DOCCNT
        ,    s.RCVPROCESSCNT
        ,    s.ACTIONCNT
        ,    s.SIGNALCNT
        ,    s.ACTIONDURATIONSUM
        ,    s.SIGNALDURATIONSUM
        ,    s.AACNT
        ,    s.GECNT
        ,    s.RACNT
        ,    s.RAECNT
        ,    s.AAECNT
        ,    s.REALTIMETOPERFORM
        ,    s.UnknownCnt
        )
        Values
        (
            p_summaryId
       ,    Trunc(P_RCVDOCTS, 'HH24')
       ,    nvl(p_DOCCNT,0)
       ,    nvl(p_RCVPROCESSCNT,0)
       ,    nvl(p_ACTIONCNT,0)
       ,    nvl(p_SIGNALCNT,0)
       ,    nvl(p_ACTIONDURATIONSUM,0)
       ,    nvl(p_SIGNALDURATIONSUM,0)
       ,    nvl(p_AACNT,0)
       ,    nvl(p_GECNT,0)
       ,    nvl(p_RACNT,0)
       ,    nvl(p_RAECNT,0)
       ,    nvl(p_AAECNT,0)
       ,    nvl(p_TIMETOPERFORM,0)
       ,    nvl(p_UnknownCnt,0)
        );
Exception
    When Dup_Val_on_Index Then
        Upd_LGSummaryRn;
    when others then
        ProcAudit;
        p_ReturnCd := 1;
        raise_application_error(-20999,'20000 LG_Summary_RNI_AH1 - Update_lg_summary_mi '||SQLERRM);
End;
End if;

End Update_Lg_Summary_rn;
--------------------------------------------------------------------------
--                 M  A  I  N
--------------------------------------------------------------------------
BEGIN
    p_ReturnCd := 0;
   Update_Lg_Summary_rn;
   Update_Lg_Summary_rn_MI ;
--
-- If ref_SummaryID is provided then do the following processing
--
    IF p_ref_summaryID is not null then
        Do_0A1;
    END IF;
    ProcAudit;
EXCEPTION
When Others Then
    ProcAudit;
    p_ReturnCd := 1;
    Raise_application_error (-20999,'20001 '||v_ProcName||'  '|| SQLCODE|| ' '||SQLERRM
                               ||' Summary ID '||p_summaryID
                               ||' RcvDocTS '||to_char(p_rcvDocTS, 'YYYY-MM-DD hh24:mi:ss')
                               ||' Ref Summary ID '||p_ref_summaryID
                               ||' Ref RcvDocTS '||to_char(p_Ref_rcvDocTS, 'YYYY-MM-DD hh24:mi:ss'));
/*
     MERGE into lg_summary_rn_MI S Using ( Select
             p_summaryID              M_SUMMARYID
       ,    FN_tenMinuteDate(P_RCVDOCTS) M_DayHour
       ,    nvl(p_DOCCNT,0)                  M_DOCCNT
       ,    nvl(p_RCVPROCESSCNT,0)           M_RCVPROCESSCNT
       ,    nvl(p_ACTIONCNT,0)               M_ACTIONCNT
       ,    nvl(p_SIGNALCNT,0)               M_SIGNALCNT
       ,    nvl(p_ACTIONDURATIONSUM,0)       M_ACTIONDURATIONSUM
       ,    nvl(p_SIGNALDURATIONSUM,0)       M_SIGNALDURATIONSUM
       ,    nvl(p_AACNT,0)                   M_AACNT
       ,    nvl(p_GECNT,0)                   M_GECNT
       ,    nvl(p_RACNT,0)                   M_RACNT
       ,    nvl(p_RAECNT,0)                  M_RAECNT
       ,    nvl(p_AAECNT,0)                  M_AAECNT
       ,    nvl(p_TIMETOPERFORM,0)           M_TIMETOPERFORM
       ,    nvl(p_UnknownCnt,0)              M_UnknownCnt
     From Dual) E
     ON (S.summaryid = E.M_summaryId and  S.dayHourMi = E.M_DayHour)

*/
/*
     MERGE into lg_summary_rn S Using ( Select
             p_summaryID              M_SUMMARYID
       ,    Trunc(P_RCVDOCTS, 'HH24') M_DayHour
       ,    nvl(p_DOCCNT,0)                  M_DOCCNT
       ,    nvl(p_RCVPROCESSCNT,0)           M_RCVPROCESSCNT
       ,    nvl(p_ACTIONCNT,0)               M_ACTIONCNT
       ,    nvl(p_SIGNALCNT,0)               M_SIGNALCNT
       ,    nvl(p_ACTIONDURATIONSUM,0)       M_ACTIONDURATIONSUM
       ,    nvl(p_SIGNALDURATIONSUM,0)       M_SIGNALDURATIONSUM
       ,    nvl(p_AACNT,0)                   M_AACNT
       ,    nvl(p_GECNT,0)                   M_GECNT
       ,    nvl(p_RACNT,0)                   M_RACNT
       ,    nvl(p_RAECNT,0)                  M_RAECNT
       ,    nvl(p_AAECNT,0)                  M_AAECNT
       ,    nvl(p_TIMETOPERFORM,0)           M_TIMETOPERFORM
       ,    nvl(p_UnknownCnt,0)              M_UnknownCnt
     From Dual) E
     ON (S.summaryid = E.M_summaryId and  S.dayHour = E.M_DayHour)
     When Matched then update set
            s.DOCCNT            = s.DOCCNT          +   E.M_DOCCNT
       ,    s.RCVPROCESSCNT     = s.RCVPROCESSCNT   +   E.M_RCVPROCESSCNT
       ,    s.ACTIONCNT         = s.ACTIONCNT       +   E.M_ACTIONCNT
       ,    s.SIGNALCNT         = s.SIGNALCNT       +   E.M_SIGNALCNT
       ,    s.ACTIONDURATIONSUM = s.ACTIONDURATIONSUM + E.M_ACTIONDURATIONSUM
       ,    s.SIGNALDURATIONSUM = s.SIGNALDURATIONSUM + E.M_SIGNALDURATIONSUM
       ,    s.AACNT             = s.AACNT           +   E.M_AACNT
       ,    s.GECNT             = s.GECNT           +   E.M_GECNT
       ,    s.RACNT             = s.RACNT           +   E.M_RACNT
       ,    s.RAECNT            = s.RAECNT          +   E.M_RAECNT
       ,    s.REALTIMETOPERFORM = s.REALTIMETOPERFORM + E.M_TIMETOPERFORM
       ,    s.UnknownCnt        = s.UnknownCnt      +   E.M_UnknownCnt
       ,    s.Updatedate = FN_GET_DATE()
     When Not Matched Then Insert (
            s.SUMMARYID
       ,    s.DAYHOUR
       ,    s.DOCCNT
       ,    s.RCVPROCESSCNT
       ,    s.ACTIONCNT
       ,    s.SIGNALCNT
       ,    s.ACTIONDURATIONSUM
       ,    s.SIGNALDURATIONSUM
       ,    s.AACNT
       ,    s.GECNT
       ,    s.RACNT
       ,    s.RAECNT
       ,    s.AAECNT
       ,    s.REALTIMETOPERFORM
       ,    s.UnknownCnt
       ) Values (
            E.M_SUMMARYID
       ,    E.M_DayHour
       ,    E.M_DOCCNT
       ,    E.M_RCVPROCESSCNT
       ,    E.M_ACTIONCNT
       ,    E.M_SIGNALCNT
       ,    E.M_ACTIONDURATIONSUM
       ,    E.M_SIGNALDURATIONSUM
       ,    E.M_AACNT
       ,    E.M_GECNT
       ,    E.M_RACNT
       ,    E.M_RAECNT
       ,    E.M_AAECNT
       ,    E.M_TIMETOPERFORM
       ,    E.M_UnknownCnt
);

*/
/*
 MERGE into lg_summary_rn S Using ( Select
             p_ref_summaryID              M_SUMMARYID
       ,    Trunc(P_Ref_RCVDOCTS, 'HH24') M_DayHour
       ,    1                             M_zeroA1CNT
       From Dual) E
       ON (S.summaryid = E.M_summaryId and  S.dayHour = E.M_DayHour)
       When Matched then update set
           s.zeroA1CNT = nvl(s.zeroA1CNT,0) + E.M_zeroA1CNT
      ,    s.Updatedate = FN_GET_DATE()
      When Not Matched Then Insert (
            s.SUMMARYID
       ,    s.DAYHOUR
       ,   s.zeroA1CNT
        ) Values (
             E.M_SUMMARYID
       ,     E.M_DayHour
       ,     E.M_zeroA1CNT
       );
*/
/*
 MERGE into lg_summary_rn_MI S Using ( Select
             p_ref_summaryID               M_SUMMARYID
       ,    FN_tenMinuteDate(P_Ref_RCVDOCTS) M_DayHour
       ,    1                             M_zeroA1CNT
       From Dual) E
       ON (S.summaryid = E.M_summaryId and  S.dayHourMI = E.M_DayHour)
       When Matched then update set
           s.zeroA1CNT = nvl(s.zeroA1CNT,0) + E.M_zeroA1CNT
      ,    s.Updatedate = FN_GET_DATE()
      When Not Matched Then Insert (
            s.SUMMARYID
       ,    s.DAYHOURMI
       ,   s.zeroA1CNT
        ) Values (
             E.M_SUMMARYID
       ,     E.M_DayHour
       ,     E.M_zeroA1CNT
       );
*/

END LG_Summary_RNI_AH1;
/


-- LG_SUMMARY_RN_PRC.SQL

CREATE OR REPLACE PROCEDURE LG_SUMMARY_RN_PRC (
		p_ReturnCd	in out	pls_integer
		,p_DebugFlg	in	pls_integer
) AS
/*
================================================================================
Proc Name:      LG_SUMMARY_RN_PRC
Create Date:    12/03/04
Author          S.Priani

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2004-03-22 S.Priani	4.2.2	Created
	2005-09-19 S.Priani	6.0.0.2	BGTRK00055950
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_BeginTS	timestamp	:= FN_Get_Date; -- Performance related variables
	v_ReturnCd	pls_integer;
	CURSOR cur_lg_unp_rn IS
		SELECT	lur.ROWID, lur.*
		FROM	lg_unprocessed_rn lur;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_SUMMARY_RN_PRC',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    , p_DebugFlg
    );
End ProcAudit;

--------------------------------------------------------------------------
--                 M  A  I  N
--------------------------------------------------------------------------
BEGIN
	FOR rec_lg_unp_rn IN cur_lg_unp_rn LOOP
		BEGIN
		LG_PROCESSSUMMARY_RNI_AH1
			(
			v_ReturnCd
			,p_DebugFlg
			,rec_lg_unp_rn.PROCESSSUMMARYID
			,rec_lg_unp_rn.RCVDOCTS
			,rec_lg_unp_rn.DOCCNT
			,rec_lg_unp_rn.RCVPROCESSCNT
			,rec_lg_unp_rn.REQACTIONCNT
			,rec_lg_unp_rn.RESPACTIONCNT
			,rec_lg_unp_rn.REQSIGNALCNT
			,rec_lg_unp_rn.RESPSIGNALCNT
			,rec_lg_unp_rn.REQDURATIONSUM
			,rec_lg_unp_rn.RESPDURATIONSUM
			,rec_lg_unp_rn.RAREQDURATIONSUM
			,rec_lg_unp_rn.RARESPDURATIONSUM
			,rec_lg_unp_rn.AACNT
			,rec_lg_unp_rn.GECNT
			,rec_lg_unp_rn.RACNT
			,rec_lg_unp_rn.RAECNT
			,rec_lg_unp_rn.AAECNT
			,rec_lg_unp_rn.REALTIMETOPERFORM
			,rec_lg_unp_rn.UNKNOWNCNT
			,rec_lg_unp_rn.REFPROCESSSUMMID
			,rec_lg_unp_rn.REFRCVDOCTS
			);

			DELETE	lg_unprocessed_rn
			WHERE	ROWID = rec_lg_unp_rn.ROWID;
		EXCEPTION
			When Others Then
				p_ReturnCd := 1;
				Raise;
		END;
	END LOOP;
	ProcAudit;
	p_ReturnCd := 0;
	COMMIT;
EXCEPTION
	When Others Then
		ProcAudit;
		p_ReturnCd := 1;
		Raise_application_error (-20999,'20000 LG_SUMMARY_RN_PRC ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_SUMMARY_XREFI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_SUMMARY_XrefI_AH1 (
      p_ReturnCd       out pls_integer
    , p_DebugFlg       in pls_integer
    , p_summaryID      OUT pls_integer
--
    , P_INITBUSINESSID in OUT  VARCHAR2
    , P_FrBUSINESSID   in OUT  VARCHAR2
    , P_ToBUSINESSID   in OUT  VARCHAR2
    , P_FRPROTOCOLNAME in OUT  VARCHAR2
    , P_FRPROTOCOLVER  in OUT  VARCHAR2
    , P_TOPROTOCOLNAME in OUT  VARCHAR2
    , P_TOPROTOCOLVER  in OUT  VARCHAR2
    , P_FRPROCESSCD    in OUT  VARCHAR2
    , p_FRPROCESSVER   in OUT  VARCHAR2
    , P_TOPROCESSCD    in OUT  VARCHAR2
    , p_TOPROCESSVER   in OUT  VARCHAR2
    , P_FRIPADDRESS    in OUT  VARCHAR2
    , p_CHANNELSTATUSName in  VARCHAR2
    , p_DestTypeId     in   pls_integer  -- if null default to zero
    , P_ProvSessionID  in   pls_integer
    , p_FrTransportType in  VARCHAR2
    , p_ToTransportType in  VARCHAR2
    , p_FRPACKAGENAME   in  VARCHAR2
    , p_FRPACKAGEVER    in  VARCHAR2
    , p_TOPACKAGENAME   in  VARCHAR2
    , p_TOPACKAGEVER    in  VARCHAR2
    , p_SYNCHRONOUSFLG  in  pls_integer
    , p_ActionFlg        in pls_integer -- 1 = Action, 0 = Signal
    )
AS
/*

================================================================================
Proc Name:      LG_SUMMARY_XrefI_AH1
Create Date:    07/11/2003
Author          Vipin Goel
    This main purpose of this proc is to provide SummaryID.
    1   First it tries to get the Partner_summaryID,
        if not found, then Insert into LG_SUM_XREF_Part table generating a  Partner_summaryID
        If due to multi-threading, Dup condition is found, it selects again

    2   Second  it tries to get the Process_summaryID,
        if not found, then Insert into LG_SUM_XREF_Prcs table generating a  Process_summaryID
        If due to multi-threading, Dup condition is found, it selects again

    3   Third it tries to get the summaryID,
        if not found, then Insert into LG_sum_Xref_Lnk table generating a  summaryID
        If due to multi-threading, Dup condition is found, it selects again
====================================================================================
Return Code = 0  OK
              1  Error Condition
              2  Duplicate
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
    v_SUMMARYID         LG_Sum_Xref_Lnk.SUMMARYID%type;
    v_PARTNER_SUMMARYID LG_SUM_XREF_Part.PARTNER_SUMMARYID%type;
    v_Process_SUMMARYID LG_SUM_XREF_Prcs.Process_SUMMARYID%type;
    v_ACTIVITYTYPECD    LG_SUM_XREF_Prcs.ACTIVITYTYPECD%Type;

    v_InitPartnerId     LG_SUM_XREF_Part.INITPARTNERID%type;
    v_FrPartnerId       LG_SUM_XREF_Part.FrPARTNERID%type;
    v_ToPartnerId       LG_SUM_XREF_Part.ToPARTNERID%type;

     v_desttypeID        LG_SUM_XREF_Part.DestTypeID%type := P_desttypeID  ;
     v_createDate        date Default sysdate;
     v_update_summary    boolean Default False;
     v_beginTS           Timestamp(6) Default FN_Get_Date;
--     v_ChannelStatusId   pls_integer;  --LG_SUMMARY_XREF.ChannelStatusId%type;
     v_ProvSessionID     LG_Sum_Xref_Lnk.ProvSessionID%type Default nvl(p_ProvSessionID, -1);
    -- v_DestName          LG_SUMMARY_XREF.DestName%type;
    v_ParamsTxt       varchar2(4000) Default
      'INITBUSINESSID =>'  || p_INITBUSINESSID||         CHR(10)||
     ' ,FrPARTNERID =>'     || to_char(v_FrPartnerId)||  CHR(10)||
     ' ,TPPARTNERID =>'     || to_char(v_ToPartnerId)||  CHR(10)||
     ' ,ToBUSINESSID =>'    || p_ToBUSINESSID||          CHR(10)||
     ' ,INITPARTNERID =>'   || to_char(v_InitPartnerId)||CHR(10)||
     ' ,FRPROTOCOLNAME=>'   || p_FRPROTOCOLNAME||        CHR(10)||
     ' ,FRPROTOCOLVer=>'    || p_FRPROTOCOLVER||         CHR(10)||
     ' ,TOPROTOCOLNAME=>'   || p_TOPROTOCOLNAME||        CHR(10)||
     ' ,TOPROTOCOLVer=>'    || p_TOPROTOCOLVER||         CHR(10)||
     ' ,FRPROCESSCD=>'      || p_FRPROCESSCD||           CHR(10)||
     ' ,FRPROCESSVER=>'     || p_FRPROCESSVER||          CHR(10)||
     ' ,TOPROCESSCD=>'      || p_TOPROCESSCD||           CHR(10)||
     ' ,TOPROCESSVER=>'     || p_TOPROCESSVER||          CHR(10)||
     ' ,FRIPADDRESS=>'      || p_FRIPADDRESS||           CHR(10)||
     ' ,DESTTypeID =>'      || to_char(v_DESTTYPEID)||   CHR(10)||
     ' ,ProvSessionID =>'   || to_char(v_ProvSessionID)||CHR(10)||
     ' ,FrTransportType =>' || p_FrTransportType ||      CHR(10)||
     ' ,ToTransportType =>' || p_ToTransportType ||      CHR(10)||
     ' ,p_FRPACKAGENAME  =>' ||p_FRPACKAGENAME||         CHR(10)||
     ' ,p_FRPACKAGEVER  =>'  ||p_FRPACKAGEVER||          CHR(10)||
     ' ,p_TOPACKAGENAME =>'  ||p_TOPACKAGENAME||         CHR(10)||
     ' ,p_TOPACKAGEVER =>'   || p_TOPACKAGEVER||         CHR(10)||
     ' ,p_SYNCHRONOUSFLG =>' || p_SYNCHRONOUSFLG||       CHR(10)||
     ' ,p_ActionFlg =>'      ||p_ActionFlg ;
-- ' CHANNELSTATUSID =>' || to_char( v_CHANNELSTATUSID)||CHR(10)||

--
-- Procedure
--
Procedure ProcAudit is
--v_diffTS Timestamp;
begin
    DB_ProcAuditLogI_ah1('LG_SUMMARY_XrefI_AH1',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||','||
    P_INITBUSINESSID||','||
    P_FrBUSINESSID||','||
    P_ToBUSINESSID||','||
    P_FRPROTOCOLNAME||','||
    P_FRPROTOCOLVer||','||
    P_TOPROTOCOLNAME||','||
    P_TOPROTOCOLVer||','||
    P_FRPROCESSCD||','||
    P_FRPROCESSVER||','||
    P_TOPROCESSCD||','||
    P_TOPROCESSVER||','||
    P_FRIPADDRESS||','||
    P_CHANNELSTATUSName||','||
    P_DestTypeId||','||
    P_ProvSessionID||','||
    p_FrTransportType||','||
    p_ToTransportType||','||
    p_FRPACKAGENAME ||','||
    p_FRPACKAGEVER ||','||
    p_TOPACKAGENAME ||','||
    p_TOPACKAGEVER
    );
End ProcAudit;
--

--
Procedure Check_length is
begin
       DB_LENGTHCHECK_SUMMARY_XREF(
   P_INITBUSINESSID  => P_INITBUSINESSID
   ,P_FRBUSINESSID   => P_FRBUSINESSID
   ,P_TOBUSINESSID   => P_TOBUSINESSID
   ,P_FRPROTOCOLNAME => P_FRPROTOCOLNAME
   ,P_TOPROTOCOLNAME => P_TOPROTOCOLNAME
   ,P_FRPROCESSCD    => P_FRPROCESSCD
   ,P_FRPROCESSVER   => P_FRPROCESSVER
   ,P_TOPROCESSCD    => P_TOPROCESSCD
   ,P_TOPROCESSVER   => P_TOPROCESSVER
   ,P_FRIPADDRESS    => P_FRIPADDRESS
   ,P_FRPROTOCOLVER  => P_FRPROTOCOLVER
   ,P_TOPROTOCOLVER  => P_TOPROTOCOLVER
   );

end check_length;
--
Procedure Get_Partner_SummaryID is
Begin
    select Partner_summaryId into v_Partner_summaryId
    from LG_SUM_XREF_Part
    where
        INITPARTNERID     = v_InitPartnerId
    and INITBUSINESSID    = nvl(p_INITBUSINESSID,  '-1')
    and FrPARTNERID       = v_FrPartnerId
    and FrBUSINESSID      = nvl(p_FrBUSINESSID,    '-1')
    and ToPARTNERID       = v_ToPartnerId
    and ToBUSINESSID      = nvl(p_ToBUSINESSID,    '-1')
    and FrTransportType   = nvl(p_FrTransportType, '-1')
    and ToTransportType   = nvl(p_ToTransportType, '-1')
    and FRIPADDRESS       = nvl(p_FRIPADDRESS,     '-1')
    and DESTTypeId        = v_DestTypeId;

    Exception
    when NO_DATA_FOUND then
         v_Partner_summaryId := -1;
    when others then
        p_ReturnCd := 1;
        raise_application_error(-20999,20005||' LG_SUMMARY_XrefI_AH1 - Get_Partner_SummaryID '||SQLERRM||v_ParamsTxt );
END Get_Partner_SummaryID ;
--
Procedure Get_Process_SummaryID is
Begin
    select Process_summaryId into v_Process_summaryId
    from LG_SUM_XREF_Prcs
    where
        FRPACKAGENAME     = nvl(p_FRPACKAGENAME,  '-1')
    and FRPACKAGEVER      = nvl(p_FRPACKAGEVER,   '-1')
    and TOPACKAGENAME     = nvl(p_TOPACKAGENAME,  '-1')
    and TOPACKAGEVER      = nvl(p_TOPACKAGEVER,   '-1')
    and FRPROTOCOLNAME    = nvl(p_FRPROTOCOLNAME, '-1')
    and FRPROTOCOLVer     = nvl(p_FRPROTOCOLVer,  '-1')
    and TOPROTOCOLNAME    = nvl(p_TOPROTOCOLNAME, '-1')
    and TOPROTOCOLVer     = nvl(p_TOPROTOCOLVer,  '-1')
    and FRPROCESSCD       = nvl(p_FRPROCESSCD,    '-1')
    and FRPROCESSVER      = nvl(p_FRPROCESSVER,   '-1')
    and TOPROCESSCD       = nvl(p_TOPROCESSCD,    '-1')
    and TOPROCESSVER      = nvl(p_TOPROCESSVER,   '-1')
    And ACTIVITYTYPECD    = nvl(v_ACTIVITYTYPECD, '-1')  -- Not being used
    And SYNCHRONOUSFLG    = nvl(p_SYNCHRONOUSFLG,0)
    And ActionFlg         = nvl(p_ActionFlg, -1);

    Exception
        when NO_DATA_FOUND then
            v_Process_summaryId := -1;
        when others then
            p_ReturnCd := 1;
            raise_application_error(-20999,20005||' LG_SUMMARY_XrefI_AH1 - Get_Process_SummaryID '||SQLERRM||v_ParamsTxt );
END Get_Process_SummaryID ;
--
Procedure Get_SummaryID is
Begin
 select summaryId into v_summaryId
 from LG_Sum_Xref_Lnk
 where
        PARTNER_SUMMARYID =  v_PARTNER_SUMMARYID
    And PROCESS_SUMMARYID =  v_PROCESS_SUMMARYID
    And PROVSESSIONID     =  v_ProvSessionID;
Exception
when NO_DATA_FOUND then
  v_summaryId := -1;
--  raise_application_error(-20999,20004||'LG_SUMMARY_XrefI_AH1 - Get_SummaryID '||SQLERRM||v_ParamsTxt );
when others then
    p_ReturnCd := 1;
    raise_application_error(-20999,20005||' LG_SUMMARY_XrefI_AH1 - Get_SummaryID '||SQLERRM||v_ParamsTxt );
END Get_SummaryID ;

--
procedure Insert_LG_SUM_XREF_Part as
Begin

 Insert into LG_SUM_XREF_Part
   (PARTNER_SUMMARYID
   ,INITPARTNERID
   ,INITBUSINESSID
   ,FrPARTNERID
   ,FrBUSINESSID
   ,ToPARTNERID
   ,ToBUSINESSID
   ,FrTransportType
   ,ToTransportType
   ,FRIPADDRESS
   ,DESTTypeId
   ,CREATEDATE
    )
   values
   (LG_SUM_XREF_Part_SEQ.nextval
   , v_InitPartnerId
   , nvl(p_INITBUSINESSID, -1)
   , v_FrPartnerId
   , nvl(p_FrBUSINESSID, -1)
   , v_ToPartnerId
   , nvl(p_ToBUSINESSID, -1)
   , nvl(p_FrTransportType, '-1')
   , nvl(p_ToTransportType, '-1')
   , nvl(p_FRIPADDRESS, -1)
   , v_DestTypeId
   , v_createDate
   ) returning PARTNER_SUMMARYID into v_PARTNER_SUMMARYID ;
dbms_output.put_line('PARTNER_summaryid '||to_char(v_PARTNER_SUMMARYID)  );
Exception
when dup_val_on_index   then
--  p_ReturnCd := 2;             -- Dup
  Get_Partner_SummaryID ;
when others then
    p_ReturnCd := 1;
    raise_application_error(-20999,20005||' LG_SUMMARY_XrefI_AH1 - Insert_LG_SUM_XREF_Part '||SQLERRM|| v_ParamsTxt );
End Insert_LG_SUM_XREF_Part;
--
procedure Insert_LG_SUM_XREF_Prcs is
Begin
    Insert into LG_SUM_XREF_Prcs
    (Process_SUMMARYID
   ,FRPACKAGENAME
   ,FRPACKAGEVER
   ,TOPACKAGENAME
   ,TOPACKAGEVER
   ,FRPROTOCOLNAME
   ,FRPROTOCOLVER
   ,TOPROTOCOLNAME
   ,TOPROTOCOLVER
   ,FRPROCESSCD
   ,FRPROCESSVER
   ,TOPROCESSCD
   ,TOPROCESSVER
   ,ACTIVITYTYPECD
   ,CREATEDATE
   ,SYNCHRONOUSFLG
   ,ActionFlg
    )
    Values
    (
    LG_SUM_XREF_Prcs_SEQ.nextval
   , nvl(p_FRPACKAGENAME,  '-1')
   , nvl(p_FRPACKAGEVER,   '-1')
   , nvl(p_TOPACKAGENAME,  '-1')
   , nvl(p_TOPACKAGEVER,   '-1')
   , nvl(p_FRPROTOCOLNAME, '-1')
   , nvl(p_FRPROTOCOLVER,  '-1')
   , nvl(p_TOPROTOCOLNAME, '-1')
   , nvl(p_TOPROTOCOLVER,  '-1')
   , nvl(p_FRPROCESSCD,    '-1')
   , nvl(p_FRPROCESSVER,   '-1')
   , nvl(p_TOPROCESSCD,    '-1')
   , nvl(p_TOPROCESSVER,   '-1')
   , nvl(v_ACTIVITYTYPECD, '-1')
   , v_createDate
   , nvl(p_SYNCHRONOUSFLG,0) -- 0 = Not Sync , 1 = Sync
   , nvl(p_ActionFlg, -1)    -- 0 = Signal, 1 = Action
   ) returning Process_SUMMARYID into v_Process_SUMMARYID ;

dbms_output.put_line('Process_summaryid '||to_char(v_Process_SUMMARYID)  );
Exception
when dup_val_on_index   then
--  p_ReturnCd := 2;             -- Dup
  Get_Process_SummaryID ;
when others then
    p_ReturnCd := 1;
    raise_application_error(-20999,20005||' LG_SUMMARY_XrefI_AH1 - Insert_LG_SUM_XREF_Prcs '||SQLERRM|| v_ParamsTxt );
END Insert_LG_SUM_XREF_Prcs;
--
Procedure Insert_LG_Sum_Xref_Lnk
IS
Begin
Insert Into LG_Sum_Xref_Lnk
    (SUMMARYID
    ,PARTNER_SUMMARYID
    ,PROCESS_SUMMARYID
    ,PROVSESSIONID
    ,CREATEDATE
    )
    Values
    (LG_Sum_Xref_Lnk_SEQ.nextval
    ,v_PARTNER_SUMMARYID
    ,v_PROCESS_SUMMARYID
    ,v_PROVSESSIONID
    ,v_CREATEDATE
    ) returning SUMMARYID into v_SUMMARYID ;
dbms_output.put_line('summaryid '||to_char(v_SUMMARYID)  );
Exception
when dup_val_on_index   then
--  p_ReturnCd := 2;             -- Dup
  Get_SummaryID ;
when others then
    p_ReturnCd := 1;
    raise_application_error(-20999,20005||' LG_SUMMARY_XrefI_AH1 - Insert_LG_Sum_Xref_Lnk '||SQLERRM|| v_ParamsTxt );

End Insert_LG_Sum_Xref_Lnk;
-----------------------------------------------------------------------------
-- Main
------------------------------------------------------------------------------
    BEGIN
        Check_length;
        p_ReturnCd := 0;
	    v_initpartnerid := FN_get_partnerId(P_INITBUSINESSID);
	    v_FrPartnerId   := FN_get_partnerId(P_FrBUSINESSID)  ;
	    v_ToPartnerId   := FN_get_partnerId(P_TOBUSINESSID)  ;
-- Partner_SummaryID
        Get_Partner_SummaryID;
        IF v_Partner_SummaryID = -1 Then
            Insert_LG_SUM_XREF_Part;
        END IF;
-- Process_SummaryID
        Get_Process_SummaryID;
        IF v_Process_SummaryID = -1 Then
            Insert_LG_SUM_XREF_Prcs;
        END IF;
-- SummaryID
        Get_SummaryID;
        IF v_SummaryID = -1 Then
            Insert_LG_Sum_Xref_Lnk;
        END IF;
--
        p_summaryID := v_summaryId;        -- return summary id
        ProcAudit;
     Exception
     when others then
        ProcAudit;
        p_ReturnCd := 1;
        raise_application_error(-20999,20011||' LG_SUMMARY_XrefI_AH1 - Main '||SQLERRM);
    END ;
/


-- LG_SYNCH_REQ_RESPI_AH1.sql

CREATE OR REPLACE PROCEDURE LG_SYNCH_REQ_RESPI_AH1
        (p_ReturnCd         out pls_integer
        ,p_DebugFlg         in pls_integer
        ,p_Vuid_Req         in VARCHAR2
        ,p_VUID_RESP        in VARCHAR2
        ,p_Vuid_STATUS      in pls_integer
        ,p_DocID_Req        in varchar2
        ,p_FrPartnerID_req  in pls_integer
        )
AS

/*
        ================================================================================
        Proc Name:       LG_SYNCH_REQ_RESPI_AH1
        Create Date:     2003-10-22 10:36:16
        Author:          V Goel
            Case 1 input p_VUID_REQ is supplied
             Or  2 input P_DOCID_REQ and p_FrPartnerID_req both must be supplied
                         p_VUID_REQ is fetched from LG_Activity table using docid and FrpartnerID

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-10-22 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_SYNCH_REQ_RESPI_AH1';
v_ExceptionFlg    integer Default 0;
v_Vuid            LG_Activity.ActivityID%Type;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_Vuid_Req=>'||p_Vuid_Req||CHR(10)
                ||', p_VUID_RESP=>'||p_VUID_RESP||CHR(10)
                ||', p_Vuid_STATUS=>'||p_Vuid_STATUS||CHR(10)
                ||', p_DocID_Req=>'||p_DocID_Req||CHR(10)
                ||', p_FrPartnerID_req=>'||p_FrPartnerID_req||CHR(10)
                ;



---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF p_VUID_REQ IS NULL THEN
		
		INSERT INTO LG_SYNCH_REQ_RESP(
				ACTIVITYID
				,VUID_RESP
				,STATUS)
			SELECT  ActivityID
			 	,p_Vuid_Resp
		                ,p_Vuid_STATUS
		        FROM LG_Activity
        		WHERE DocID = p_DocID_Req
       			AND   FrPartnerIdHdr = p_FrPartnerID_req
        		;
			    
       		 	v_RowCnt := SQL%rowcount;
       		
					 
        
	ELSE
		v_Vuid := p_VUID_REQ;
		
		INSERT INTO LG_SYNCH_REQ_RESP(
		                ACTIVITYID
		                ,VUID_RESP
		                ,STATUS
		        ) Values (
		                v_VUID
		                ,p_Vuid_Resp
		                ,p_Vuid_STATUS
       		 );
       		 
       		  v_RowCnt := SQL%rowcount;
	END IF;
        
        IF v_RowCnt > 1 THEN        
        	p_ReturnCd := 2;
	ELSE
		p_ReturnCd := 0;
	END IF;
	
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
        EXCEPTION
		When NO_DATA_FOUND Then
		            p_ReturnCd := 3;
		            v_ExceptionFlg := 1;
		            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		            RETURN;
		            --RAISE_APPLICATION_ERROR (-20999,'20011 LG_SYNCH_REQ_RESPI_AH1 - Get Vuid ' || SQLCODE|| ' '||SQLERRM);

                When DUP_VAL_ON_INDEX then
                          Null;
--                        v_ExceptionFlg := 1;
--                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
--                        p_returnCd := 2;
--                        RAISE_APPLICATION_ERROR (-20999,'20001 LG_SYNCH_REQ_RESPI_AH1' || SQLCODE|| ' '||SQLERRM);
                When OTHERS Then
                        p_ReturnCd := 1;
                        v_ExceptionFlg := 1;
                        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                        RAISE_APPLICATION_ERROR (-20999,'20002 LG_SYNCH_REQ_RESPI_AH1' || SQLCODE|| ' '||SQLERRM);
END LG_SYNCH_REQ_RESPI_AH1;
/

-- LG_TOKEN_LOCKD.sql

CREATE OR REPLACE PROCEDURE LG_TOKEN_LOCKD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_TOKENID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_TOKEN_LOCKD
	Create Date:     2006-03-13 14:46:21
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-13 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_TOKEN_LOCKD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	p_ReturnCd := 0;

	Delete From LG_TOKEN_LOCK
	Where
		TOKENID = p_TOKENID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	Exception
		WHEN ZeroDelete THEN
         p_ReturnCd := 3 ;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			--RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_TOKEN_LOCKD;
/


-- LG_TOKEN_LOCKI.sql

CREATE OR REPLACE PROCEDURE LG_TOKEN_LOCKI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_TOKENID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_TOKEN_LOCKI
	Create Date:     2006-03-13 14:47:51
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-13 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_TOKEN_LOCKI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	p_ReturnCd := 0;

	Insert into LG_TOKEN_LOCK(
		TOKENID
	) Values (
		p_TOKENID
	);
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END LG_TOKEN_LOCKI;
/


-- LG_TOKEN_LOCKU_AH1.sql

CREATE OR REPLACE PROCEDURE LG_TOKEN_LOCKU_AH1
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
--	, RC1 out db_ref.RCT
	,p_TOKENID in VARCHAR2
--	,p_ROWTS in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       LG_TOKEN_LOCKU_AH1
	Create Date:     2006-03-13 14:54:35
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-13 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'LG_TOKEN_LOCKU_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TOKENID=>'||p_TOKENID||CHR(10)
--		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update LG_TOKEN_LOCK
	SET
		UPDATEDATE= v_BeginTS
		,      ROWTS= ROWTS + 1
	Where
		TOKENID = p_TOKENID ;
--		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
--	IF RC1%isOpen THEN Close RC1; END IF;
--	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End LG_TOKEN_LOCKU_AH1;
/


-- LG_UNPROCESSED_ACTIVITYI.sql

CREATE OR REPLACE PROCEDURE LG_UNPROCESSED_ACTIVITYI(
      p_ReturnCd      out pls_integer
    , p_DebugFlg       in pls_integer
    , P_VUID           in   VARCHAR2
    , P_RCVDOCTS       in   Timestamp
    , P_SNTDOCTS       in   Timestamp
    , P_INITBUSINESSID in out  VARCHAR2
    , P_FrBUSINESSID   in out  VARCHAR2
    , P_ToBUSINESSID   in out  VARCHAR2
    , P_FRPROTOCOLNAME in out  VARCHAR2
    , P_TOPROTOCOLNAME in out  VARCHAR2
    , P_FRPROCESSCD    in out  VARCHAR2
    , p_FRPROCESSVER   in out  VARCHAR2
    , P_TOPROCESSCD    in out  VARCHAR2
    , p_TOPROCESSVER   in out  VARCHAR2
    , P_FRIPADDRESS    in out  VARCHAR2
    , p_CHANNELSTATUSName in  VARCHAR2
    , P_RCVDOCCNT         in   pls_integer
    , P_SNTDOCCNT         in   pls_integer
    , P_FAILDOCCNT        in   pls_integer
    , P_EVENTCD           in   pls_integer  -- 3.2
    , p_MSGRAWLENGTH      in   NUMBER
    , p_MSGCONTENTLENGTH  in   NUMBER
    , P_FRPROTOCOLVer     in out  VARCHAR2
    , P_TOPROTOCOLVer     in out  VARCHAR2
    , p_DestName          in   VARCHAR2
    , P_dmDuration_MS     in   pls_integer   -- MS -- millisecond
    , P_TRANSPORTDURATION_MS   in   pls_integer
    , P_TransportATTEMPTCNT    in   pls_integer
    , P_GATEWAYATTEMPTCNT      in   pls_integer
    , p_PERSISTDOCTS           in   Timestamp
    , p_BPEStartTS      in   Timestamp
    , p_BPEEndTS        in   Timestamp
    , p_PROVSESSIONID          in   pls_integer
    , p_FrTransportType     in VARCHAR2
    , p_ToTransportType     in VARCHAR2
    , p_FRPACKAGENAME   in  VARCHAR2
    , p_FRPACKAGEVER    in  VARCHAR2
    , p_TOPACKAGENAME   in  VARCHAR2
    , p_TOPACKAGEVER    in  VARCHAR2
    , p_SYNCHRONOUSFLG  in  pls_integer
    ,p_ActionFlg        in pls_integer -- 1 = Action, 0 = Signal
    )
AS
/*
================================================================================
Proc Name:      LG_UNPROCESSED_ACTIVITYI
Create Date:    05/27/2004
Author          S.Priani
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_ProcName           varchar2(30) := 'LG_UNPROCESSED_ACTIVITYI';
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
	'P_DEBUGFLG=>'         ||NVL(RTRIM(TO_CHAR(P_DEBUGFLG)),'')        ||CHR(10)
                ||',P_VUID=>'                ||NVL(P_VUID,'')                         ||CHR(10)
                ||',P_RCVDOCTS=>'            ||NVL(RTRIM(TO_CHAR(P_RCVDOCTS)),'')        ||CHR(10)
                ||',P_SNTDOCTS=>'            ||NVL(RTRIM(TO_CHAR(P_SNTDOCTS)),'')        ||CHR(10)
                ||',P_INITBUSINESSID=>'      ||NVL(P_INITBUSINESSID,'')               ||CHR(10)
                ||',P_FRBUSINESSID=>'        ||NVL(P_FRBUSINESSID,'')                 ||CHR(10)
                ||',P_TOBUSINESSID=>'        ||NVL(P_TOBUSINESSID,'')                 ||CHR(10)
                ||',P_FRPROTOCOLNAME=>'      ||NVL(P_FRPROTOCOLNAME,'')               ||CHR(10)
                ||',P_TOPROTOCOLNAME=>'      ||NVL(P_TOPROTOCOLNAME,'')               ||CHR(10)
                ||',P_FRPROCESSCD=>'         ||NVL(P_FRPROCESSCD,'')                  ||CHR(10)
                ||',P_FRPROCESSVER=>'        ||NVL(P_FRPROCESSVER,'')                 ||CHR(10)
                ||',P_TOPROCESSCD=>'         ||NVL(P_TOPROCESSCD,'')                  ||CHR(10)
                ||',P_TOPROCESSVER=>'        ||NVL(P_TOPROCESSVER,'')                 ||CHR(10)
                ||',P_FRIPADDRESS=>'         ||NVL(P_FRIPADDRESS,'')                  ||CHR(10)
                ||',P_CHANNELSTATUSNAME=>'   ||NVL(P_CHANNELSTATUSNAME,'')            ||CHR(10)
                ||',P_RCVDOCCNT=>'           ||NVL(RTRIM(TO_CHAR(P_RCVDOCCNT)),'')       ||CHR(10)
                ||',P_SNTDOCCNT=>'           ||NVL(RTRIM(TO_CHAR(P_SNTDOCCNT)),'')       ||CHR(10)
                ||',P_FAILDOCCNT=>'          ||NVL(RTRIM(TO_CHAR(P_FAILDOCCNT)),'')      ||CHR(10)
                ||',P_EVENTCD=>'             ||NVL(RTRIM(TO_CHAR(P_EVENTCD)),'')         ||CHR(10)
                ||',P_MSGRAWLENGTH=>'        ||NVL(RTRIM(TO_CHAR(P_MSGRAWLENGTH)),'')    ||CHR(10)
                ||',P_MSGCONTENTLENGTH=>'    ||NVL(RTRIM(TO_CHAR(P_MSGCONTENTLENGTH)),'')||CHR(10)
                ||',P_FRPROTOCOLVER=>'       ||NVL(P_FRPROTOCOLVER,'')                ||CHR(10)
                ||',P_TOPROTOCOLVER=>'       ||NVL(P_TOPROTOCOLVER ,'')               ||CHR(10)
                ||',P_DESTNAME=>'            ||NVL(P_DESTNAME,'')                     ||CHR(10)
                ||',P_DMDURATION_MS=>'       ||NVL(RTRIM(TO_CHAR(P_DMDURATION_MS)),'')   ||CHR(10)
                ||',P_TRANSPORTDURATION_MS=>'||NVL(RTRIM(TO_CHAR(P_TRANSPORTDURATION_MS)),'')||CHR(10)
                ||',P_TRANSPORTATTEMPTCNT=>' ||NVL(RTRIM(TO_CHAR(P_TRANSPORTATTEMPTCNT)),'') ||CHR(10)
                ||',P_GATEWAYATTEMPTCNT=>'   ||NVL(RTRIM(TO_CHAR(P_GATEWAYATTEMPTCNT)),'')   ||CHR(10)
                ||',P_PERSISTDOCTS=>'        ||NVL(RTRIM(TO_CHAR(P_PERSISTDOCTS)),'')        ||CHR(10)
                ||',P_BPESTARTTS=>'          ||NVL(RTRIM(TO_CHAR(P_BPESTARTTS)),'')      ||CHR(10)
                ||',P_BPEENDTS=>'            ||NVL(RTRIM(TO_CHAR(P_BPEENDTS)),'')        ||CHR(10)
                ||',P_PROVSESSIONID=>'       ||NVL(RTRIM(TO_CHAR(P_PROVSESSIONID)),'')   ||CHR(10)
                ||',P_FRTRANSPORTTYPE=>'     ||NVL(P_FRTRANSPORTTYPE,'')              ||CHR(10)
                ||',P_TOTRANSPORTTYPE=>'     ||NVL(P_TOTRANSPORTTYPE,'')              ||CHR(10)
                ||',P_FRPACKAGENAME=>'       ||NVL(P_FRPACKAGENAME,'')                ||CHR(10)
                ||',P_FRPACKAGEVER=>'        ||NVL(P_FRPACKAGEVER,'')                 ||CHR(10)
                ||',P_TOPACKAGENAME=>'       ||NVL(P_TOPACKAGENAME,'')                ||CHR(10)
                ||',P_TOPACKAGEVER=>'        ||NVL(P_TOPACKAGEVER,'')                 ||CHR(10)
                ||',p_SYNCHRONOUSFLG=>'      ||NVL(RTRIM(TO_CHAR(p_SYNCHRONOUSFLG)),'')  ||CHR(10)
                ||',p_ActionFlg=>'           ||NVL(RTRIM(TO_CHAR(p_ActionFlg)),'')  ||CHR(10) ;
------------------------------------------------------------------------------
--                                  M  A  I  N
------------------------------------------------------------------------------
BEGIN

        INSERT INTO LG_Unprocessed_Activity
        (
       	VUID
	,RCVDOCTS
	,SNTDOCTS
	,INITBUSINESSID
	,FRBUSINESSID
	,TOBUSINESSID
	,FRPROTOCOLNAME
	,TOPROTOCOLNAME
	,FRPROCESSCD
	,FRPROCESSVER
	,TOPROCESSCD
	,TOPROCESSVER
	,FRIPADDRESS
	,CHANNELSTATUSNAME
	,RCVDOCCNT
	,SNTDOCCNT
	,FAILDOCCNT
	,EVENTCD
	,MSGRAWLENGTH
	,MSGCONTENTLENGTH
	,FRPROTOCOLVER
	,TOPROTOCOLVER
	,DESTNAME
	,DMDURATION_MS
	,TRANSPORTDURATION_MS
	,TRANSPORTATTEMPTCNT
	,GATEWAYATTEMPTCNT
	,PERSISTDOCTS
	,BPESTARTTS
	,BPEENDTS
	,PROVSESSIONID
	,FRTRANSPORTTYPE
	,TOTRANSPORTTYPE
	,FRPACKAGENAME
	,FRPACKAGEVER
	,TOPACKAGENAME
	,TOPACKAGEVER
	,SYNCHRONOUSFLG
	,ACTIONFLG
	)
	VALUES
	(
	P_VUID
	,P_RCVDOCTS
	,P_SNTDOCTS
	,P_INITBUSINESSID
	,P_FRBUSINESSID
	,P_TOBUSINESSID
	,P_FRPROTOCOLNAME
	,P_TOPROTOCOLNAME
	,P_FRPROCESSCD
	,P_FRPROCESSVER
	,P_TOPROCESSCD
	,P_TOPROCESSVER
	,P_FRIPADDRESS
	,P_CHANNELSTATUSNAME
	,P_RCVDOCCNT
	,P_SNTDOCCNT
	,P_FAILDOCCNT
	,P_EVENTCD
	,P_MSGRAWLENGTH
	,P_MSGCONTENTLENGTH
	,P_FRPROTOCOLVER
	,P_TOPROTOCOLVER
	,P_DESTNAME
	,P_DMDURATION_MS
	,P_TRANSPORTDURATION_MS
	,P_TRANSPORTATTEMPTCNT
	,P_GATEWAYATTEMPTCNT
	,P_PERSISTDOCTS
	,P_BPESTARTTS
	,P_BPEENDTS
	,P_PROVSESSIONID
	,P_FRTRANSPORTTYPE
	,P_TOTRANSPORTTYPE
	,P_FRPACKAGENAME
	,P_FRPACKAGEVER
	,P_TOPACKAGENAME
	,P_TOPACKAGEVER
	,p_SYNCHRONOUSFLG
	,NVL(p_ActionFlg, -1)
	);

	v_RowCnt := sql%ROWCOUNT;
   	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	When NO_DATA_FOUND THEN
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
		p_returnCd := 3;
	When DUP_VAL_ON_INDEX then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_returnCd := 2;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
       		RAISE_APPLICATION_ERROR(-20999,20011||' LG_UNPROCESSED_ACTIVITYI '||SQLERRM);
END ;
/


-- LG_UNP_SUMMARY_RN_PRC.SQL

CREATE OR REPLACE PROCEDURE LG_UNP_SUMMARY_RN_PRC (
		p_ReturnCd	in out	pls_integer
		,p_DebugFlg	in	pls_integer
) AS
/*
================================================================================
Proc Name:      LG_UNP_SUMMARY_RN_PRC
Create Date:    25/06/04
Author          S.Priani

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2004-03-22 S.Priani	4.2.2	Created
	2005-09-19 S.Priani	6.0.0.2	BGTRK00055950
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_BeginTS	timestamp	:= FN_Get_Date; -- Performance related variables
	v_ReturnCd	pls_integer;
	CURSOR cur_lg_unp_rn IS
		SELECT	lur.ROWID, lur.*
		FROM	lg_unprocessed_summary_rn lur;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_UNP_SUMMARY_RN_PRC',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    , p_DebugFlg
    );
End ProcAudit;

--------------------------------------------------------------------------
--                 M  A  I  N
--------------------------------------------------------------------------
BEGIN
	FOR rec_lg_unp_rn IN cur_lg_unp_rn LOOP
		BEGIN
		LG_SUMMARY_RNI_AH1
			(
			v_ReturnCd
			,p_DebugFlg
			,rec_lg_unp_rn.RCVDOCTS
			,rec_lg_unp_rn.DOCCNT
			,rec_lg_unp_rn.RCVPROCESSCNT
			,rec_lg_unp_rn.ACTIONCNT
			,rec_lg_unp_rn.SIGNALCNT
			,rec_lg_unp_rn.ACTIONDURATIONSUM
			,rec_lg_unp_rn.SIGNALDURATIONSUM
			,rec_lg_unp_rn.AACNT
			,rec_lg_unp_rn.GECNT
			,rec_lg_unp_rn.RACNT
			,rec_lg_unp_rn.RAECNT
			,rec_lg_unp_rn.AAECNT
			,rec_lg_unp_rn.TIMETOPERFORM
			,rec_lg_unp_rn.REFSUMMARYID
			,rec_lg_unp_rn.REFRCVDOCTS
			,rec_lg_unp_rn.SUMMARYID
			,rec_lg_unp_rn.UNKNOWNCNT
			);

			DELETE	lg_unprocessed_summary_rn
			WHERE	ROWID = rec_lg_unp_rn.ROWID;
		EXCEPTION
			When Others Then
				p_ReturnCd := 1;
				Raise;
		END;
	END LOOP;
	ProcAudit;
	p_ReturnCd := 0;
	COMMIT;
EXCEPTION
	When Others Then
		ProcAudit;
		p_ReturnCd := 1;
		Raise_application_error (-20999,'20000 LG_UNP_SUMMARY_RN_PRC ' || SQLCODE|| ' '||SQLERRM);
END;
/


-- LG_VALIDATIONI_AH1.sql

CREATE OR REPLACE PROCEDURE         LG_VALIDATIONI_ah1 (
p_ReturnCd         out integer
, p_DebugFlg       in pls_integer
,p_SORTSEQUENCE    in NUMBER
,p_VUID            in VARCHAR2
,p_ACTIVITYID      in VARCHAR2
,p_FIELD           in out VARCHAR2
,p_VALUE           in out VARCHAR2
,p_ERROR           in out VARCHAR2
) as
/*
================================================================================
Proc Name:        LG_VALIDATIONI
Create Date:     2002-05-23 12:02:25
Author:            SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
v_field     LG_VALIDATION.field%type;
v_value     LG_VALIDATION.value%type;
v_error     LG_VALIDATION.error%type;
Procedure ProcAudit is
begin
--If p_DebugFlg =1 Then
DB_ProcAuditLogI_Ah1('LG_VALIDATIONI',v_beginTS,
'p_ReturnCd'||','||p_DebugFlg
||','||p_SORTSEQUENCE
||','||p_VUID
||','||p_ACTIVITYID
||','||p_FIELD
||','||p_VALUE
||','||p_ERROR
);
--End if ;
End ProcAudit;
--

Procedure Check_length is
begin
   DB_LENGTHCHECK_VALIDATION(
   P_FIELD  => P_FIELD
   ,P_VALUE => P_VALUE
   ,P_ERROR => P_ERROR
   );

end check_length;
-------------------------------------------------------------------
-- MAIN
-------------------------------------------------------------------
BEGIN
Check_length;
Insert into LG_VALIDATION(
    VALIDATIONID
,    SORTSEQUENCE
,    VUID
,    ACTIVITYID
,    FIELD
,    VALUE
,    ERROR
) Values (
    LG_VALIDATION_SEQ.nextval
,    p_SORTSEQUENCE
,    p_VUID
,    p_ACTIVITYID
,    p_FIELD
,    p_VALUE
,    p_ERROR
);
ProcAudit;
EXCEPTION
When NO_DATA_FOUND Then Null;
When Others Then
ProcAudit;
Raise_application_error (-20999,'20000LG_VALIDATIONI' || SQLCODE|| '
'||SQLERRM);
Rollback work;
p_ReturnCd := 1;
END LG_VALIDATIONI_ah1;
/


-- LG_VALIDATIONS_VUID.sql

CREATE OR REPLACE PROCEDURE         LG_VALIDATIONs_VUID
(
    p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_VUID in VARCHAR2
) As
/*
================================================================================
Proc Name:      LG_VALIDATIONs
Create Date:    01/10/02 11:49:22 AM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('LG_VALIDATIONs',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg||','||'RC1'
    ||','||p_VUID);
End ProcAudit;
BEGIN
Open RC1 for select
--    sortSequence
    RTRIM(FIELD)  FIELD
,   RTRIM(VALUE)  VALUE
,   RTRIM(ERROR)  ERROR
,   CREATEDATE
From LG_VALIDATION
Where  VUID = p_VUID
order by sortSequence
;
p_ReturnCd := 0;
procAudit;
End LG_VALIDATIONs_VUID;
/


-- LG_VTP_STATUSD.sql

CREATE OR REPLACE Procedure LG_VTP_STATUSD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_CID in NUMBER
	,p_INITBUSINESSID in VARCHAR2
	,p_PROCESSINSTANCEID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_VTP_STATUSD
	Create Date:     2003-09-26 08:24:07
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_VTP_STATUSD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CID=>'||p_CID||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From LG_VTP_STATUS
	Where
		CID = p_CID
		AND INITBUSINESSID = p_INITBUSINESSID
		AND PROCESSINSTANCEID = p_PROCESSINSTANCEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 LG_VTP_STATUSD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End LG_VTP_STATUSD;
/


-- LG_VTP_STATUSI_AH1.sql

CREATE OR REPLACE PROCEDURE         LG_VTP_STATUSI_AH1
    (p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,p_CID in NUMBER  --  CID is Expected to be Null
    ,p_INITBUSINESSID in VARCHAR2
    ,p_PROCESSINSTANCEID in VARCHAR2
    ,p_RESPONDSTATUSFLG in NUMBER  -- Expecting zero
    ,p_FrBusinessId in Varchar2
    ,p_ToBusinessId in Varchar2
    ,p_GLBBUSACTIONCD in varchar2
    ,p_EndStateFlg in number
    )
AS

/*
    ================================================================================
    Proc Name:       LG_VTP_STATUSI_AH1
    Create Date:     2003-03-19 13:59:20
    Author:          SYSTEM
    09-29-2003  M.Mount 4.2.1  Removed Rollback

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_PK_Value number := 0;
v_CID      pls_integer;
--
Procedure ProcAudit is
Begin
        DB_ProcAuditLogI_Ah1('LG_VTP_STATUSI_AH1',v_beginTS,
'p_ReturnCd'||','||p_DebugFlg
            ||','||'RC1'
            ||','||p_CID
            ||','||p_INITBUSINESSID
            ||','||p_PROCESSINSTANCEID
            ||','||p_RESPONDSTATUSFLG
        );
End ProcAudit;
--
Procedure Get_CID (p2_ibusid    in varchar2,
                   p2_procInsId in varchar2,
                   p2_rnhdr_cid out pls_integer
                   )is
begin
  select   CID INTO
         p2_rnhdr_cid
  from LG_ACTIVITY_RNHDR
  where INITBUSINESSID    = p2_ibusid
  and   PROCESSINSTANCEID = p2_procInsId  ;

  Exception
  When Others Then
    ProcAudit;
    Raise_application_error (-20999,'20001 LG_VTP_STATUSI_AH1 Get_CID Ref IDs ' ||p2_ibusid|| ' '||
                             p2_procInsId||' '||SQLERRM);
    Rollback work;
    p_ReturnCd  := 1;
end Get_CID;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    p_returnCd := 0;
    If p_CID is Null Then
        Get_CID (p_INITBUSINESSID, p_PROCESSINSTANCEID, v_CID);
    Else
        v_CID := p_CID ;
    End If;

--  Delete the request document record if the response document is successful
    If p_InitBusinessId = p_ToBusinessId and p_InitBusinessId <> p_FrBusinessId and p_GLBBUSACTIONCD is NOT null and p_EndStateFlg = 1 Then
        Delete Lg_VTP_Status
        Where InitBusinessId = p_InitBusinessId
        And   ProcessInstanceId = p_ProcessInstanceId
        And   Cid = v_Cid;

    Else
        Begin
            Insert into LG_VTP_STATUS(
                CID
                ,INITBUSINESSID
                ,PROCESSINSTANCEID
                ,RESPONDSTATUSFLG
            ) Values (
                v_CID
                ,p_INITBUSINESSID
                ,p_PROCESSINSTANCEID
                ,p_RESPONDSTATUSFLG         -- decode(p_VTP_STATUS, 'OFF', 0, 0)

            );

            ProcAudit;
            EXCEPTION
                When NO_DATA_FOUND Then Null;
                When DUP_VAL_ON_INDEX then
                    -- Ignore dups if the response document failed
                    If p_InitBusinessId = p_ToBusinessId and p_InitBusinessId <> p_FrBusinessId and p_GLBBUSACTIONCD is NOT null and p_EndStateFlg = 2 Then
                       Null;
                    Else
                        p_returnCd := 2;
                    End If;
                When OTHERS Then
                ProcAudit;
                Raise_application_error (-20999,'20000 LG_VTP_STATUSI_AH1 '||SQLERRM);
 --               Rollback work;
                p_ReturnCd := 1;
        End;
    End If;

END LG_VTP_STATUSI_Ah1;
/


-- LG_VTP_STATUSS.sql

CREATE OR REPLACE PROCEDURE LG_VTP_STATUSS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CID IN NUMBER
	, p_INITBUSINESSID IN VARCHAR2
	, p_PROCESSINSTANCEID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       LG_VTP_STATUSS
	Create Date:     2003-09-26 00:06:48
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'LG_VTP_STATUSS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CID=>'||p_CID||CHR(10)
		||', p_INITBUSINESSID=>'||p_INITBUSINESSID||CHR(10)
		||', p_PROCESSINSTANCEID=>'||p_PROCESSINSTANCEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_CID IS NOT NULL
	 AND p_INITBUSINESSID IS NOT NULL
	 AND p_PROCESSINSTANCEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		CID
		,RTRIM(INITBUSINESSID) INITBUSINESSID
		,RTRIM(PROCESSINSTANCEID) PROCESSINSTANCEID
		,RESPONDSTATUSFLG
		,ROWTS
		,CREATEDATE
	FROM
		LG_VTP_STATUS
	Where
		CID = p_CID
		AND INITBUSINESSID = p_INITBUSINESSID
		AND PROCESSINSTANCEID = p_PROCESSINSTANCEID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		CID
		,RTRIM(INITBUSINESSID) INITBUSINESSID
		,RTRIM(PROCESSINSTANCEID) PROCESSINSTANCEID
		,RESPONDSTATUSFLG
		,ROWTS
		,CREATEDATE
	FROM
		LG_VTP_STATUS
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End LG_VTP_STATUSS;
/


-- LG_VTP_STATUSS_AH1.sql

CREATE OR REPLACE PROCEDURE         LG_VTP_STATUSS_AH1
    (
    p_ReturnCd  out  pls_integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    )
AS

/*
    ===============================================================================
=
    Proc Name:       LG_VTP_STATUSS_AH1
    Create Date:     2003-03-21 14:15:48
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-03-21 SYSTEM           3.5 Created
    2003-04-29 v Goel           3.5 Remove p_SQLErrm
    2003-05-14 v Goel           3.5 Add ActivityID and Vuid
    ===============================================================================
=
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar(30) := 'LG_VTP_STATUSS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar(4000) :=
        'p_DebugFlg=>'||p_DebugFlg
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
        S.CID
        ,RTRIM(S.INITBUSINESSID) INITBUSINESSID
        ,RTRIM(S.PROCESSINSTANCEID) PROCESSINSTANCEID
        ,S.RESPONDSTATUSFLG
        ,D.ActivityID
        ,D.Vuid
    From
        LG_VTP_STATUS S
        Inner Join LG_Activity_RnDtl D  ON S.CID = D.CID
                                        AND S.INITBUSINESSID    = D.INITBUSINESSID
                                        AND S.PROCESSINSTANCEID = D.PROCESSINSTANCEID
    Where
        RESPONDSTATUSFLG = 0
        And GLBDOCFUNCTIONCD = 'Request'
        And nvl(ATTEMPTCNT,1) = 1
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    Exception
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
--            p_SQLERRM := SQLERRM;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
End LG_VTP_STATUSS_AH1;
/


-- PR_ADDRESSD.sql

CREATE OR REPLACE Procedure PR_ADDRESSD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_ADDRESSID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_ADDRESSD
	Create Date:     2003-09-26 08:24:09
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ADDRESSD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ADDRESSID=>'||p_ADDRESSID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_ADDRESS
	Where
		ADDRESSID = p_ADDRESSID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ADDRESSD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_ADDRESSD;
/


-- PR_ADDRESSI.sql

CREATE OR REPLACE Procedure PR_ADDRESSI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_ADDRESSTYPEID in NUMBER
	,p_ADDRESS in VARCHAR2
	,p_PHONEBUS in VARCHAR2
	,p_PHONEMBL in VARCHAR2
	,p_PHONEPGR in VARCHAR2
	,p_PHONEFAX in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_ADDRESSI
	Create Date:     2003-09-26 00:01:30
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ADDRESSI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_ADDRESSTYPEID=>'||p_ADDRESSTYPEID||CHR(10)
		||', p_ADDRESS=>'||p_ADDRESS||CHR(10)
		||', p_PHONEBUS=>'||p_PHONEBUS||CHR(10)
		||', p_PHONEMBL=>'||p_PHONEMBL||CHR(10)
		||', p_PHONEPGR=>'||p_PHONEPGR||CHR(10)
		||', p_PHONEFAX=>'||p_PHONEFAX||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_ADDRESS(
		ADDRESSID
		,PARTNERID
		,ADDRESSTYPEID
		,ADDRESS
		,PHONEBUS
		,PHONEMBL
		,PHONEPGR
		,PHONEFAX
	) Values (
		PR_ADDRESS_SEQ.nextval
		,p_PARTNERID
		,p_ADDRESSTYPEID
		,p_ADDRESS
		,p_PHONEBUS
		,p_PHONEMBL
		,p_PHONEPGR
		,p_PHONEFAX
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_ADDRESS_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ADDRESSI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_ADDRESSI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ADDRESSI' || SQLCODE|| ' '||SQLERRM);
END PR_ADDRESSI;
/


-- PR_ADDRESSS.sql

CREATE OR REPLACE PROCEDURE PR_ADDRESSS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ADDRESSID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_ADDRESSS
	Create Date:     2003-09-26 00:06:46
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ADDRESSS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ADDRESSID=>'||p_ADDRESSID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ADDRESSID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		ADDRESSID
		,PARTNERID
		,ADDRESSTYPEID
		,RTRIM(ADDRESS) ADDRESS
		,RTRIM(PHONEBUS) PHONEBUS
		,RTRIM(PHONEMBL) PHONEMBL
		,RTRIM(PHONEPGR) PHONEPGR
		,RTRIM(PHONEFAX) PHONEFAX
		,ROWTS
		,CREATEDATE
	FROM
		PR_ADDRESS
	Where
		ADDRESSID = p_ADDRESSID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		ADDRESSID
		,PARTNERID
		,ADDRESSTYPEID
		,RTRIM(ADDRESS) ADDRESS
		,RTRIM(PHONEBUS) PHONEBUS
		,RTRIM(PHONEMBL) PHONEMBL
		,RTRIM(PHONEPGR) PHONEPGR
		,RTRIM(PHONEFAX) PHONEFAX
		,ROWTS
		,CREATEDATE
	FROM
		PR_ADDRESS
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_ADDRESSS;
/


-- PR_ADDRESSS_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_ADDRESSS_AH1
(
   p_ReturnCd  OUT  PLS_INTEGER
   ,p_DebugFlg  IN  PLS_INTEGER
   ,RC1  OUT  DB_REF.RCT
   ,p_Locale IN VARCHAR2
   ,p_ADDRESSID IN NUMBER
)
AS

/*
    ================================================================================
    Proc Name:       PR_ADDRESSS
    Create Date:     2003-07-02 10:45:29
    Author:          M.Mount

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-07-30 M.Mount       3.5    Created
   2003-08-21 L.Ling        3.5    ECR: 1-1L1VD. Combined Address1, Address2, Address3 to Address.
    2004-03-03 S.Priani	     4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_ADDRESSS';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
                       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                       ||', p_Locale=>'||p_Locale||CHR(10)
                       ||', p_ADDRESSID=>'||p_ADDRESSID||CHR(10);


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ADDRESSID IS NOT NULL
 THEN
    OPEN RC1 FOR
    SELECT
        a.ADDRESSID
        ,a.PARTNERID
        ,a.ADDRESSTYPEID
        ,RTRIM(a.ADDRESS) ADDRESS
        ,RTRIM(a.PHONEBUS) PHONEBUS
        ,RTRIM(a.PHONEMBL) PHONEMBL
        ,RTRIM(a.PHONEPGR) PHONEPGR
        ,RTRIM(a.PHONEFAX) PHONEFAX
        ,a.ROWTS
        ,a.CREATEDATE
        ,RTRIM(t.RESOURCEKEY) RESOURCEKEY
        ,t.Description
    FROM
        PR_ADDRESS a
        INNER JOIN PR_AddressType t ON t.AddressTypeId = a.AddressTypeId
    WHERE
        ADDRESSID = p_ADDRESSID
	;
    ELSE
    OPEN RC1 FOR
    SELECT
        a.ADDRESSID
        ,a.PARTNERID
        ,a.ADDRESSTYPEID
        ,RTRIM(a.ADDRESS) ADDRESS
        ,RTRIM(a.PHONEBUS) PHONEBUS
        ,RTRIM(a.PHONEMBL) PHONEMBL
        ,RTRIM(a.PHONEPGR) PHONEPGR
        ,RTRIM(a.PHONEFAX) PHONEFAX
        ,a.ROWTS
        ,a.CREATEDATE
        ,RTRIM(t.RESOURCEKEY) RESOURCEKEY
        ,t.Description
    FROM
        PR_ADDRESS a
        INNER JOIN PR_AddressType t ON t.AddressTypeId = a.AddressTypeId
	;
END IF;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_ADDRESSS_AH1;
/


-- PR_ADDRESSS_PARTNERID.sql

CREATE OR REPLACE PROCEDURE PR_ADDRESSS_PARTNERID
(
   p_ReturnCd out integer
   ,p_DebugFlg in pls_integer
   ,RC1 out db_ref.RCT
   ,p_Locale in Varchar2
   ,p_PARTNERID in NUMBER
) As

/*
   ================================================================================
   Proc Name:       PR_ADDRESSS_PARTNERID
   Create Date:     2003-01-20 11:50:11
   Author:          L.Ling

                   ::::::::::::::::::::::::::::::
                         M O D I F I C A T I O N
                   ::::::::::::::::::::::::::::::

   ---------- ------------- ------ ---------------------------------------------
   DATE       NAME          Ver    DESCRIPTION
   ---------- ------------- ------ ---------------------------------------------
   2003-01-20 L.Ling        3.5    Created
   2003-08-21 L.Ling        3.5    ECR: 1-1L1VD. Combined Address1, Address2, Address3 to Address.
   2004-03-03 S.Priani	    4.2.2  Removed NLS
   2006-04-12 Shishir N     6.1    60217 Partner Migration
   
   ================================================================================
   Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

PROCEDURE ProcAudit IS
BEGIN
   DB_ProcAuditLogI_Ah1('PR_ADDRESSS_PARTNERID',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
   ||','||p_Locale
   ||','||'RC1'
   ||','||p_PARTNERID
   );
END ProcAudit;

BEGIN
   IF RC1%isOpen THEN Close RC1; END IF;
   IF p_PARTNERID is not null 
   THEN
   OPEN RC1 FOR
   SELECT
	           a.ADDRESSID
	         ,  a.PARTNERID
	         ,  a.ADDRESSTYPEID
	         ,  RTRIM(t.RESOURCEKEY) RESOURCEKEY
	         ,  RTRIM(t.DESCRIPTION) DESCRIPTION
	         ,  RTRIM(a.ADDRESS)  ADDRESS
	         ,  RTRIM(a.PHONEBUS)  PHONEBUS
	         ,  RTRIM(a.PHONEMBL)  PHONEMBL
	         ,  RTRIM(a.PHONEPGR)  PHONEPGR
	         ,  RTRIM(a.PHONEFAX)  PHONEFAX
	         ,  a.ROWTS
	         ,  a.CREATEDATE
	         ,  pp.PARTNERLOGIN
	         FROM PR_ADDRESS a
	         LEFT OUTER JOIN PR_ADDRESSTYPE t ON a.ADDRESSTYPEID = t.ADDRESSTYPEID
	         INNER JOIN PR_PARTNER pp ON a.PARTNERID = pp.PARTNERID
        WHERE a.PARTNERID = p_PARTNERID;
	ELSE
	       OPEN RC1 FOR
	       SELECT
				 		          a.ADDRESSID
				 		        ,  a.PARTNERID
				 		        ,  a.ADDRESSTYPEID
				 		        ,  RTRIM(t.RESOURCEKEY) RESOURCEKEY
				 		        ,  RTRIM(t.DESCRIPTION) DESCRIPTION
				 		        ,  RTRIM(a.ADDRESS)  ADDRESS
				 		        ,  RTRIM(a.PHONEBUS)  PHONEBUS
				 		        ,  RTRIM(a.PHONEMBL)  PHONEMBL
				 		        ,  RTRIM(a.PHONEPGR)  PHONEPGR
				 		        ,  RTRIM(a.PHONEFAX)  PHONEFAX
				 		        ,  a.ROWTS
				 		        ,  a.CREATEDATE
				 		        ,  pp.PARTNERLOGIN
				 		        FROM PR_ADDRESS a
				 		        LEFT OUTER JOIN PR_ADDRESSTYPE t ON a.ADDRESSTYPEID = t.ADDRESSTYPEID
		        INNER JOIN PR_PARTNER pp ON a.PARTNERID = pp.PARTNERID;
	END IF;

   p_ReturnCd := 0;
   procAudit;
END PR_ADDRESSS_PARTNERID;
/


-- PR_ADDRESSTYPES_ALL.sql

CREATE OR REPLACE PROCEDURE         PR_ADDRESSTYPES_ALL
(
p_ReturnCd out integer
, p_DebugFlg in pls_integer
, RC1 out db_ref.RCT
, p_Locale in Varchar2
)
AS

/*
    ================================================================================
    Proc Name:       PR_ADDRESSTYPES_ALL
    Create Date:     2003-01-22
    Author:          L.Ling
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-03 S.Priani	 4.2.2	Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_value varchar2(50);

PROCEDURE ProcAudit IS
BEGIN
   DB_ProcAuditLogI_Ah1('PR_ADDRESSTYPES_ALL',v_beginTS,
   'p_ReturnCd'||','||p_DebugFlg
   ||','||p_Locale
   ||','||'RC1'
   );
END ProcAudit;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
---    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(v_Locale);

    --for testing--
    --dbms_output.put_line('v_Locale = ' || v_Locale);
    --select value into v_value from v$nls_parameters where parameter = 'NLS_SORT';
    --dbms_output.put_line('The NLS_SORT = ' || v_value);

   IF RC1%isOpen THEN Close RC1; END IF;

   OPEN RC1 FOR
   SELECT
       ADDRESSTYPEID
   ,   RTRIM(RESOURCEKEY)  RESOURCEKEY
   ,   description
   FROM PR_ADDRESSTYPE
	;

---   EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';
   p_ReturnCd := 0;
   procAudit;
END PR_ADDRESSTYPES_ALL;
/


-- PR_ADDRESSU.sql

CREATE OR REPLACE Procedure PR_ADDRESSU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_ADDRESSID in NUMBER
	,p_PARTNERID in NUMBER
	,p_ADDRESSTYPEID in NUMBER
	,p_ADDRESS in VARCHAR2
	,p_PHONEBUS in VARCHAR2
	,p_PHONEMBL in VARCHAR2
	,p_PHONEPGR in VARCHAR2
	,p_PHONEFAX in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_ADDRESSU
	Create Date:     2003-09-26 08:21:08
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ADDRESSU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ADDRESSID=>'||p_ADDRESSID||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_ADDRESSTYPEID=>'||p_ADDRESSTYPEID||CHR(10)
		||', p_ADDRESS=>'||p_ADDRESS||CHR(10)
		||', p_PHONEBUS=>'||p_PHONEBUS||CHR(10)
		||', p_PHONEMBL=>'||p_PHONEMBL||CHR(10)
		||', p_PHONEPGR=>'||p_PHONEPGR||CHR(10)
		||', p_PHONEFAX=>'||p_PHONEFAX||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_ADDRESS
	SET
		PARTNERID=p_PARTNERID
		,      ADDRESSTYPEID=p_ADDRESSTYPEID
		,      ADDRESS=p_ADDRESS
		,      PHONEBUS=p_PHONEBUS
		,      PHONEMBL=p_PHONEMBL
		,      PHONEPGR=p_PHONEPGR
		,      PHONEFAX=p_PHONEFAX
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		ADDRESSID = p_ADDRESSID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_ADDRESSU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ADDRESSU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ADDRESSU:'||SQLERRM);
End PR_ADDRESSU;
/


-- PR_AGREEMENTD.sql

-- >  EXECUTE DB_GENPROC.GENPROC_DELETE('PR_AGREEMENT');
CREATE OR REPLACE PROCEDURE PR_AGREEMENTD
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_AGREEMENTID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_AGREEMENTD
	Create Date:     2006-05-09 16:04:00
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_AGREEMENTD';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_AGREEMENTID=>'||p_AGREEMENTID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_AGREEMENT
	Where 
		AGREEMENTID = p_AGREEMENTID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		WHEN ZeroDelete THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
		WHEN Others THEN
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_AGREEMENTD;
/
-- >  SPOOL OFF


-- PR_AGREEMENTI.sql

-- >  EXECUTE DB_GENPROC.GENPROC_INSERT('PR_AGREEMENT', TRUE, TRUE );
CREATE OR REPLACE PROCEDURE PR_AGREEMENTI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CPAID in VARCHAR2
	,p_PARTNERID1 in NUMBER
	,p_PARTNERID2 in NUMBER
	,p_VERSION in VARCHAR2
	,p_STATUS in NUMBER
	,p_STATUSREASON in VARCHAR2
	,p_STARTDATE in TIMESTAMP
	,p_ENDDATE in TIMESTAMP
	,p_INVOCATIONLIMIT in NUMBER
	,p_CONCURRENTCONVERSATIONS in NUMBER
	,p_ORIGINALFILECONTENTS in BLOB
	)
AS

/*
	================================================================================
	Proc Name:       PR_AGREEMENTI
	Create Date:     2006-05-09 16:03:59
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_AGREEMENTI';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_PARTNERID1=>'||p_PARTNERID1||CHR(10)
		||', p_PARTNERID2=>'||p_PARTNERID2||CHR(10)
		||', p_VERSION=>'||p_VERSION||CHR(10)
		||', p_STATUS=>'||p_STATUS||CHR(10)
		||', p_STATUSREASON=>'||p_STATUSREASON||CHR(10)
		||', p_STARTDATE=>'||p_STARTDATE||CHR(10)
		||', p_ENDDATE=>'||p_ENDDATE||CHR(10)
		||', p_INVOCATIONLIMIT=>'||p_INVOCATIONLIMIT||CHR(10)
		||', p_CONCURRENTCONVERSATIONS=>'||p_CONCURRENTCONVERSATIONS||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_AGREEMENT(
		AGREEMENTID
		,CPAID
		,PARTNERID1
		,PARTNERID2
		,VERSION
		,STATUS
		,STATUSREASON
		,STARTDATE
		,ENDDATE
		,INVOCATIONLIMIT
		,CONCURRENTCONVERSATIONS
		,ORIGINALFILECONTENTS
	) Values (
		PR_AGREEMENT_SEQ.nextval
		,p_CPAID
		,p_PARTNERID1
		,p_PARTNERID2
		,p_VERSION
		,p_STATUS
		,p_STATUSREASON
		,p_STARTDATE
		,p_ENDDATE
		,p_INVOCATIONLIMIT
		,p_CONCURRENTCONVERSATIONS
		,p_ORIGINALFILECONTENTS
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		PR_AGREEMENT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN No_Data_Found THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN Dup_Val_On_Index THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN Others THEN
			Rollback work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			RAISE;
END PR_AGREEMENTI;
/
-- >  SPOOL OFF


-- PR_AGREEMENTS.sql

CREATE OR REPLACE PROCEDURE PR_AGREEMENTS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_AGREEMENTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_AGREEMENTS
	Create Date:     2006-05-09 16:04:00
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_AGREEMENTS';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_AGREEMENTID=>'||p_AGREEMENTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_AGREEMENTID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		AGREEMENTID
		,RTRIM(CPAID) CPAID
		,PARTNERID1
		,PARTNERID2
		,RTRIM(VERSION) VERSION
		,STATUS
		,RTRIM(STATUSREASON) STATUSREASON
		,STARTDATE
		,ENDDATE
		,INVOCATIONLIMIT
		,CONCURRENTCONVERSATIONS
		,ROWTS
		,CREATEDATE
		,ORIGINALFILECONTENTS
	FROM 
		PR_AGREEMENT
	Where 
		AGREEMENTID = p_AGREEMENTID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		AGREEMENTID
		,RTRIM(CPAID) CPAID
		,PARTNERID1
		,PARTNERID2
		,RTRIM(VERSION) VERSION
		,STATUS
		,RTRIM(STATUSREASON) STATUSREASON
		,STARTDATE
		,ENDDATE
		,INVOCATIONLIMIT
		,CONCURRENTCONVERSATIONS
		,ROWTS
		,CREATEDATE
		,ORIGINALFILECONTENTS
	FROM 
		PR_AGREEMENT
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_AGREEMENTS;
/


-- PR_AGREEMENTS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_AGREEMENTS_AH1
(
	p_ReturnCd		IN OUT		INTEGER
	, p_DebugFlg		IN		INTEGER
	, RC1			OUT		DB_REF.RCT
	, RC2			OUT		DB_REF.RCT
	, p_Sortkey		IN		VARCHAR2
	, p_PageSizeNbr		IN		PLS_INTEGER
	, p_PageNbr		IN		PLS_INTEGER
	, p_CPAID		IN		VARCHAR2
	, p_PartnerId1		IN		INTEGER
	, p_PartnerId2		IN		INTEGER
	, p_Status		IN		INTEGER
)
AS

/*
	================================================================================
	Proc Name:       PR_AGREEMENTS_AH1
	Create Date:     2006-03-10 16:34:04
	Author:          SYSTEM

Returns RSet to console for searched CPAs. Search criteria interprets p_Status as:
 = 1 for notStarted, expired, exhausted (invocationLimit reached)
 = 0 for valid CPA
	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-10 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_AGREEMENTS_AH1';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_PARTNERID1=>'||p_PARTNERID1||CHR(10)
		||', p_PARTNERID2=>'||p_PARTNERID2||CHR(10)
		;

CURSOR cX IS	SELECT	AGREEMENTID
		 	, RTRIM(CPAID) AS CPAID
		 	, PartnerId1
		 	, PartnerId2
		FROM	PR_AGREEMENT  agr
		WHERE	agr.CPAID = COALESCE(p_CPAID, agr.CPAID)
		AND	(agr.PartnerID1 = COALESCE(p_PartnerID1, agr.PartnerID1) OR agr.PartnerID2 = COALESCE(p_PartnerID1, agr.PartnerId2))
		AND	(agr.PartnerID1 = COALESCE(p_PartnerID2, agr.PartnerID1) OR agr.PartnerID2 = COALESCE(p_PartnerID2, agr.PartnerId2))
		AND	CASE	
				WHEN p_Status = 0 AND (agr.StartDate > v_BeginTS OR agr.EndDate < v_BeginTS) THEN 1
				WHEN p_Status = 1 AND (agr.StartDate <=  v_BeginTS AND agr.EndDate >= v_BeginTS) THEN 1
				ELSE 0
			END = 0
		;
TYPE char60Tab IS TABLE OF VARCHAR2(60);
TYPE num10Tab  IS TABLE OF NUMBER(10);
array_AId	num10Tab   := num10Tab();
array_CPAID	char60Tab  := char60Tab();
array_PID1	num10Tab   := num10Tab();
array_PID2	num10Tab   := num10Tab();
array_InvCnt	num10Tab   := num10Tab();
array_ConCnt	num10Tab   := num10Tab();

prAgrTab	pr_agreement_tbl := pr_agreement_tbl();

v_sqlstmt		VARCHAR2(4000);
v_sortKey		VARCHAR2(500);
v_Count			NUMBER(10);

v_InvCnt	INTEGER;
v_OpenCnt	INTEGER;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	OPEN cX;
	FETCH cX BULK COLLECT INTO array_aid, array_cpaid, array_pid1, array_pid2;

	FOR v_iter IN 1..array_aid.count LOOP
		SELECT	Invocations, OpenConversations
		INTO	v_InvCnt, v_OpenCnt
		FROM	TABLE( CAST( FN_CALC_CPALimits(array_cpaid(v_iter), array_pid1(v_iter), array_pid2(v_iter)) AS PR_AGREEMENT_TBL))
		;
		prAgrTab.extend;
		prAgrTab(v_iter) := pr_agreement_rec( array_aid(v_iter)
						, array_cpaid(v_iter)
						,array_pid1(v_iter)
						,array_pid2(v_iter)
						,v_InvCnt
						,v_OpenCnt
						);
	END LOOP;

	v_sqlStmt := 'SELECT	agr.AGREEMENTID '
		||',RTRIM(agr.CPAID) CPAID '
		||',agr.PARTNERID1 '
		||',agr.PARTNERID2 '
		||',P1.PARTNERName AS Partner1'
		||',P2.PARTNERName AS Partner2'
		||',RTRIM(agr.VERSION) AS VERSION '
		||',agr.STATUS '
		||',RTRIM(agr.STATUSREASON) AS STATUSREASON '
		||',agr.STARTDATE '
		||',agr.ENDDATE '
		||',agr.INVOCATIONLIMIT '
		||',agr.CONCURRENTCONVERSATIONS '
		||',ht.INVOCATIONS '
		||',ht.OPENCONVERSATIONS '
		||',agr.ROWTS '
		||',agr.CREATEDATE '
		||',CASE WHEN agr.StartDate > :DateNow THEN 2 '
		||'	WHEN agr.EndDate < :DateNow THEN 1 '
		||'	WHEN COALESCE(agr.InvocationLimit, ht.Invocations +1) <= ht.Invocations THEN 1 '
		||'	ELSE 0 END AS CPASTATUS '
		||',SYS_EXTRACT_UTC( SYSTIMESTAMP ) CURRENT_TS '
		||'FROM	TABLE( CAST( :Tab AS PR_Agreement_Tbl)) ht '
		||'INNER JOIN PR_AGREEMENT agr ON agr.AgreementID = ht.AgreementID '
		||'LEFT OUTER JOIN PR_PARTNER P1 ON P1.PartnerID = agr.PartnerId1 '
		||'LEFT OUTER JOIN PR_PARTNER P2 ON P2.PartnerID = agr.PartnerId2 '
		||'WHERE 1=1 '
		||'AND	0 = CASE WHEN :InStatus = 0 AND COALESCE(agr.InvocationLimit, ht.Invocations +1) <= ht.Invocations THEN 1 '
		||'		ELSE 0 '
		||'	END '
	;

-- Get RowCount
	EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM (' || v_sqlstmt || ')' INTO v_count USING v_BeginTS, v_BeginTS, prAgrTab, p_Status;

-- Sort RSet if required
	IF p_sortkey IS NOT NULL THEN
		v_sortkey := UPPER(p_sortkey);
		v_sqlstmt := v_sqlstmt ||' ORDER BY '|| v_sortkey;
	END if;

-- Paging logic
	v_sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, v_sqlstmt);

-- Row count Cursor
	IF RC2%IsOpen THEN CLOSE RC2; END IF;
	IF NVL(p_pageNbr,0) IN (0,1) THEN
		OPEN RC2 FOR SELECT v_count TotalRowsNbr FROM DUAL;
	END IF;

-- Open Cursor
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_SqlStmt USING v_BeginTS, v_BeginTS, prAgrTab, p_Status;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		RAISE;
End PR_AGREEMENTS_AH1;
/


-- PR_AGREEMENTS_AH2.sql

CREATE OR REPLACE PROCEDURE PR_AGREEMENTS_AH2
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PartnerName1 IN VARCHAR2
	, p_PartnerName2 IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_AGREEMENTS_AH2
	Create Date:     2006-05-09 16:04:00
	Author:          SYSTEM 

	To search for CPA between a given partnerPair (PartnerNames lookup)
	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_AGREEMENTS_AH2';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PartnerName1=>'||p_PartnerName1||CHR(10)
		||', p_PartnerName2=>'||p_PartnerName2||CHR(10)
		;

v_PartnerID1	PR_Partner.PartnerID%TYPE;
v_PartnerID2	PR_Partner.PartnerID%TYPE;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	BEGIN
		SELECT	partnerID
		INTO	v_PartnerID1
		FROM	pr_partner
		WHERE	partnerName = p_PartnerName1
		AND	enabled <> 2  --- Partner is not deleted
		;
	EXCEPTION
		WHEN No_Data_Found THEN
			v_PartnerID1 := -1;
	END;
	BEGIN
		SELECT	partnerID
		INTO	v_PartnerID2
		FROM	pr_partner
		WHERE	partnerName = p_PartnerName2
		AND	enabled <> 2  --- Partner is not deleted
		;
	EXCEPTION
		WHEN No_Data_Found THEN
			v_PartnerID2 := -1;
	END;
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT
		AGREEMENTID
		,RTRIM(CPAID) CPAID
		,PARTNERID1
		,PARTNERID2
		,RTRIM(VERSION) VERSION
		,STATUS
		,RTRIM(STATUSREASON) STATUSREASON
		,STARTDATE
		,ENDDATE
		,INVOCATIONLIMIT
		,CONCURRENTCONVERSATIONS
		,ROWTS
		,CREATEDATE
		,ORIGINALFILECONTENTS
	FROM	PR_AGREEMENT agr
	WHERE	1 = 1
	AND	(agr.PartnerID1 = v_PartnerId1 OR agr.PartnerID2 = v_PartnerId1)
	AND	(agr.PartnerID1 = v_PartnerId2 OR agr.PartnerID2 = v_PartnerId2)
	;
	p_ReturnCd := 0;
		
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		RAISE;
End PR_AGREEMENTS_AH2;
/


-- PR_AGREEMENTS_AH3.sql

CREATE OR REPLACE PROCEDURE PR_AGREEMENTS_AH3
(
	p_ReturnCd		IN OUT		INTEGER
	, p_DebugFlg		IN		INTEGER
	, RC1			OUT		DB_REF.RCT
	, p_AgreementID		IN		INTEGER
)
AS

/*
	================================================================================
	Proc Name:       PR_AGREEMENTS_AH3
	Create Date:     2006-03-10 16:34:04
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-10 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_AGREEMENTS_AH3';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_AgreementID=>'||p_AgreementID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

-- Open Cursor
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT	agr.AGREEMENTID 
		,RTRIM(agr.CPAID) CPAID 
		,agr.PARTNERID1 
		,agr.PARTNERID2 
		,P1.PARTNERName AS Partner1
		,P2.PARTNERName AS Partner2
		,RTRIM(agr.VERSION) AS VERSION 
		,agr.STATUS 
		,RTRIM(agr.STATUSREASON) AS STATUSREASON 
		,agr.STARTDATE 
		,agr.ENDDATE 
		,agr.INVOCATIONLIMIT 
		,agr.CONCURRENTCONVERSATIONS 
		,COALESCE(ht.INVOCATIONS ,0) INVOCATIONS
		,COALESCE(ht.OPENCONVERSATIONS , 0) OPENCONVERSATIONS
		,agr.ROWTS 
		,agr.CREATEDATE 
		,CASE WHEN agr.StartDate > v_BeginTS THEN 2 
			WHEN agr.EndDate < v_BeginTS THEN 1 
			WHEN COALESCE(agr.InvocationLimit, ht.Invocations +1) <= ht.Invocations THEN 1 
			ELSE 0 END AS CPASTATUS 
		,EXTRACT( DAY FROM (agr.ENDDATE - SYS_EXTRACT_UTC( SYSTIMESTAMP ))) DAYS_LEFT 
	FROM	PR_AGREEMENT agr
	LEFT OUTER JOIN PR_PARTNER P1 ON P1.PartnerID = agr.PartnerId1 
	LEFT OUTER JOIN PR_PARTNER P2 ON P2.PartnerID = agr.PartnerId2 
	LEFT OUTER JOIN TABLE(CAST(FN_CALC_CPALIMITS(agr.CPAID, agr.PartnerId1, agr.PartnerId2) AS PR_AGREEMENT_TBL)) ht 
			ON ht.PartnerId1 = agr.PartnerID1 AND ht.PartnerId2 = agr.PartnerID2
	WHERE	agr.AgreementID = p_AgreementId
	;


	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		RAISE;
End PR_AGREEMENTS_AH3;
/


-- PR_AGREEMENTS_AH4.sql

CREATE OR REPLACE PROCEDURE PR_AGREEMENTS_AH4
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PartnerID1 IN NUMBER
	, p_PartnerID2 IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_AGREEMENTS_AH4
	Create Date:     2006-05-09 16:04:00
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_AGREEMENTS_AH4';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PartnerID1=>'||p_PartnerID1||CHR(10)
		||', p_PartnerID2=>'||p_PartnerID2||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT
		AGREEMENTID
		,RTRIM(CPAID) CPAID
		,PARTNERID1
		,PARTNERID2
		,RTRIM(VERSION) VERSION
		,STATUS
		,RTRIM(STATUSREASON) STATUSREASON
		,STARTDATE
		,ENDDATE
		,INVOCATIONLIMIT
		,CONCURRENTCONVERSATIONS
		,ROWTS
		,CREATEDATE
		,ORIGINALFILECONTENTS
	FROM	PR_AGREEMENT agr
	WHERE	1 = 1
	AND	(agr.PartnerID1 = COALESCE(p_PartnerId1, agr.PartnerID1) OR agr.PartnerID2 = COALESCE(p_PartnerId1, agr.PartnerId2))
	AND	(agr.PartnerID1 = COALESCE(p_PartnerId2, agr.PartnerID1) OR agr.PartnerID2 = COALESCE(p_PartnerId2, agr.PartnerId2))
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		RAISE;
End PR_AGREEMENTS_AH4;
/


-- PR_AGREEMENTS_CPALIMIT.sql

CREATE OR REPLACE PROCEDURE PR_AGREEMENTS_CPALIMIT
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CPAID IN VARCHAR2
	, p_PartnerID1 IN NUMBER
	, p_PartnerID2 IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_AGREEMENTS_CPALIMIT
	Create Date:     2006-03-10 16:34:04
	Author:          SYSTEM 

Used by server to validate the CPA. Returns RSet having the CPA status as:
0=Valid, 1=StartDate < Current, 2=EndDate > Current, 3=Invocations Max'd
4=ConcurrentLimit reached, 5=No Data for CPA, 6=Error, 7=PartnerPair and CPA mismatch

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-03-10 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_AGREEMENTS_CPALIMIT';
v_ExceptionFlg    INTEGER Default 0;
v_CPA_Status		INTEGER;
CPAStatus_IsValid	INTEGER		DEFAULT 0;
CPAStatus_NotStarted	INTEGER		DEFAULT 1;
CPAStatus_Expired	INTEGER		DEFAULT 2;
CPAStatus_Exhausted	INTEGER		DEFAULT 3;
CPAStatus_TxnLimited	INTEGER		DEFAULT 4;
CPAStatus_NotFound	INTEGER		DEFAULT 5;
CPAStatus_Errors	INTEGER		DEFAULT 6;
CPAStatus_Partner	INTEGER		DEFAULT 7;
v_STARTDATE		pr_agreement.STARTDATE%TYPE;
v_ENDDATE		pr_agreement.ENDDATE%TYPE;
v_INVOCATIONLIMIT	pr_agreement.INVOCATIONLIMIT%TYPE;
v_CONCURRENT		pr_agreement.CONCURRENTCONVERSATIONS%TYPE;
v_ConversationCnt	INTEGER;
v_OpenConversationCnt	INTEGER;
CPAProblem		EXCEPTION;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		;
CURSOR cX IS	SELECT
			PARTNERID1
			,PARTNERID2
			,STARTDATE
			,ENDDATE
			,INVOCATIONLIMIT
			,CONCURRENTCONVERSATIONS
		FROM	PR_AGREEMENT
		Where	CPAID = p_CPAID
	;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_CPA_Status := CPAStatus_NOTFOUND;   --- Assume CPA is not there
	FOR rX in cX LOOP
		v_CPA_Status := CPAStatus_Partner;  --- Assume CPA is not for PartnerPair

		IF (p_PartnerID1 = rX.PartnerID1 AND p_PartnerID2 = rX.PartnerID2)
			OR (p_PartnerID1 = rX.PartnerID2 AND p_PartnerID2 = rX.PartnerID1) THEN

			v_CPA_Status := CPAStatus_IsValid;  --- Found CPA for given PatnerPair
			v_StartDate := rX.StartDate;
			v_EndDate := rX.EndDate;
			v_InvocationLimit := rX.InvocationLimit;
			v_Concurrent := rX.ConcurrentConversations;
			EXIT;
		END IF;
	END LOOP;

	IF v_CPA_Status != CPAStatus_IsValid THEN
		RAISE CPAProblem;
	END IF;
	
	IF v_StartDate IS NOT NULL THEN
		IF v_BeginTS < v_StartDate THEN
			v_CPA_Status := CPAStatus_NotStarted;
			RAISE CPAProblem;
		END IF;
	END IF;
	IF v_EndDate IS NOT NULL THEN
		IF v_BeginTS > v_EndDate THEN
			v_CPA_Status := CPAStatus_Expired;
			RAISE CPAProblem;
		END IF;
	END IF;

	BEGIN
		SELECT	Invocations, OpenConversations
		INTO	v_ConversationCnt,v_OpenConversationCnt
		FROM	TABLE( CAST( FN_CALC_CPALIMITS( p_CPAID, p_PartnerID1, p_PartnerID2) AS PR_AGREEMENT_TBL))
		;
	EXCEPTION
		WHEN No_Data_Found THEN
		v_ConversationCnt := 0;
		v_OpenConversationCnt := 0;
	END;

	IF v_InvocationLimit IS NOT NULL THEN
		IF v_ConversationCnt >= v_InvocationLimit THEN
			v_CPA_Status := CPAStatus_Exhausted;
			RAISE CPAProblem;
		END IF;
	END IF;

	IF v_Concurrent IS NOT NULL THEN
		IF v_OpenConversationCnt >= v_Concurrent THEN
			v_CPA_Status := CPAStatus_TxnLimited;
			RAISE CPAProblem;
		END IF;
	END IF;
	RAISE CPAProblem;

EXCEPTION
	WHEN CPAProblem THEN
		p_ReturnCd := 0;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		IF RC1%ISOPEN THEN CLOSE RC1; END IF;
		OPEN RC1 FOR 
			SELECT v_CPA_Status AS CPAStatus
				,v_ConversationCnt InvocationsMade
				,v_OpenConversationCnt OpenConversations
			FROM dual
		;

	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		RAISE;
End PR_AGREEMENTS_CPALIMIT;
/


-- PR_AGREEMENTU.sql

-- >  EXECUTE DB_GENPROC.GENPROC_UPDATE('PR_AGREEMENT');
CREATE OR REPLACE PROCEDURE PR_AGREEMENTU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_AGREEMENTID in NUMBER
	,p_CPAID in VARCHAR2
	,p_PARTNERID1 in NUMBER
	,p_PARTNERID2 in NUMBER
	,p_VERSION in VARCHAR2
	,p_STATUS in NUMBER
	,p_STATUSREASON in VARCHAR2
	,p_STARTDATE in TIMESTAMP
	,p_ENDDATE in TIMESTAMP
	,p_INVOCATIONLIMIT in NUMBER
	,p_CONCURRENTCONVERSATIONS in NUMBER
	,p_ROWTS in NUMBER
	,p_ORIGINALFILECONTENTS in BLOB
	)
AS

/*
	================================================================================
	Proc Name:       PR_AGREEMENTU
	Create Date:     2006-05-09 16:04:01
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-05-09 SYSTEM	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_AGREEMENTU';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_AGREEMENTID=>'||p_AGREEMENTID||CHR(10)
		||', p_CPAID=>'||p_CPAID||CHR(10)
		||', p_PARTNERID1=>'||p_PARTNERID1||CHR(10)
		||', p_PARTNERID2=>'||p_PARTNERID2||CHR(10)
		||', p_VERSION=>'||p_VERSION||CHR(10)
		||', p_STATUS=>'||p_STATUS||CHR(10)
		||', p_STATUSREASON=>'||p_STATUSREASON||CHR(10)
		||', p_STARTDATE=>'||p_STARTDATE||CHR(10)
		||', p_ENDDATE=>'||p_ENDDATE||CHR(10)
		||', p_INVOCATIONLIMIT=>'||p_INVOCATIONLIMIT||CHR(10)
		||', p_CONCURRENTCONVERSATIONS=>'||p_CONCURRENTCONVERSATIONS||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_AGREEMENT
	SET
		CPAID=p_CPAID
		,      PARTNERID1=p_PARTNERID1
		,      PARTNERID2=p_PARTNERID2
		,      VERSION=p_VERSION
		,      STATUS=p_STATUS
		,      STATUSREASON=p_STATUSREASON
		,      STARTDATE=p_STARTDATE
		,      ENDDATE=p_ENDDATE
		,      INVOCATIONLIMIT=p_INVOCATIONLIMIT
		,      CONCURRENTCONVERSATIONS=p_CONCURRENTCONVERSATIONS
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      ORIGINALFILECONTENTS=p_ORIGINALFILECONTENTS
	Where 
		AGREEMENTID = p_AGREEMENTID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			ROLLBACK work;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End PR_AGREEMENTU;
/
-- >  SPOOL OFF


-- PR_ALERTD_ALERTID.sql

CREATE OR REPLACE PROCEDURE         PR_ALERTD_ALERTID
(p_ReturnCd out integer, p_DebugFlg in pls_integer
,p_ALERTID in NUMBER
) As
/*
================================================================================
Proc Name:      PR_ALERTd_ALERTID
Create Date:    03/08/02
Author          S.Hutchinson
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_ALERTd_ALERTID',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_ALERTID
    );
End ProcAudit;
BEGIN
    Delete From PR_ALERT_CONTACT
    Where ALERTID = p_ALERTID;

    Delete From PR_NOTIFICATION
    Where ALERTID = p_ALERTID;

    Delete From PR_BATCH_LOG
    Where ALERTID = p_ALERTID;

    Delete From PR_ALERTEVAL_LOG
    Where ALERTID = p_ALERTID;

    Delete From PR_ALERT
    Where ALERTID = p_ALERTID;

    v_rowcnt := SQL%ROWCOUNT;
    ProcAudit;

    p_ReturnCd := 0;
    Exception
        When NO_DATA_FOUND Then ProcAudit;
            Raise_application_error (-20999,'20002 PR_ALERTd_ALERTID: No Data Found');
        When Others Then ProcAudit;
            Raise_application_error (-20999,'20000 PR_ALERTd_ALERTID:' || SQLCODE|| ' '||SQLERRM);
        p_ReturnCd := 1;
End PR_ALERTd_ALERTID;
/


-- PR_ALERTI.sql

Create or Replace Procedure PR_ALERTI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_ALERTNAME in VARCHAR2
	,p_ALERTPARTNERID in NUMBER
	,p_ALERTYPE in NUMBER
	,p_ROUTINGOBJREFID in NUMBER
	,p_EVENTCD in NUMBER
	,p_VOLUMETYPE in NUMBER
	,p_VOLUME in NUMBER
	,p_PCTDEV in NUMBER
	,p_VOLMIN in NUMBER
	,p_VOLMAX in NUMBER
	,p_FREQTYPE in NUMBER
	,p_FREQBEGHOUR in VARCHAR2
	,p_FREQENDHOUR in VARCHAR2
	,p_FREQWEEKDAY in VARCHAR2
	,p_FREQMONTHDAY in VARCHAR2
	,p_ENABLEDFLG in NUMBER
	,p_MODENAME in VARCHAR2
	,p_ALERTCUSTOMBUSTEXT in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_ALERTI
	Create Date:     2003-10-20 09:31:27
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-10-20 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt            integer Default 0;
v_BeginTS           timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName          varchar2(30) Default 'PR_ALERTI';
v_ExceptionFlg      integer Default 0;
v_AlertNotifyModeID Integer Default 0;
v_ParamsTxt         varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_ALERTNAME=>'||p_ALERTNAME||CHR(10)
		||', p_ALERTPARTNERID=>'||p_ALERTPARTNERID||CHR(10)
		||', p_ALERTYPE=>'||p_ALERTYPE||CHR(10)
		||', p_ROUTINGOBJREFID=>'||p_ROUTINGOBJREFID||CHR(10)
		||', p_EVENTCD=>'||p_EVENTCD||CHR(10)
		||', p_VOLUMETYPE=>'||p_VOLUMETYPE||CHR(10)
		||', p_MODENAME=>'||p_MODENAME||CHR(10)
		||', p_VOLUME=>'||p_VOLUME||CHR(10)
		||', p_PCTDEV=>'||p_PCTDEV||CHR(10)
		||', p_VOLMIN=>'||p_VOLMIN||CHR(10)
		||', p_VOLMAX=>'||p_VOLMAX||CHR(10)
		||', p_FREQTYPE=>'||p_FREQTYPE||CHR(10)
		||', p_FREQBEGHOUR=>'||p_FREQBEGHOUR||CHR(10)
		||', p_FREQENDHOUR=>'||p_FREQENDHOUR||CHR(10)
		||', p_FREQWEEKDAY=>'||p_FREQWEEKDAY||CHR(10)
		||', p_FREQMONTHDAY=>'||p_FREQMONTHDAY||CHR(10)
		||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
		||', p_ALERTCUSTOMBUSTEXT=>'||p_ALERTCUSTOMBUSTEXT||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	Begin
	
	Select Id into v_AlertNotifyModeID from PR_AlertNotifyMode 
					   Where ModeName = p_ModeName;
					   
	EXCEPTION	
		WHEN NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ALERTI' || SQLCODE|| ' '||SQLERRM);		
		WHEN OTHERS THEN
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ALERTI' || SQLCODE|| ' '||SQLERRM);		
	End;

	Insert into PR_ALERT(
		ALERTID
		,PARTNERID
		,ALERTNAME
		,ALERTPARTNERID
		,ALERTYPE
		,ROUTINGOBJREFID
		,EVENTCD
		,VOLUMETYPE
		,VOLUME
		,PCTDEV
		,VOLMIN
		,VOLMAX
		,FREQTYPE
		,FREQBEGHOUR
		,FREQENDHOUR
		,FREQWEEKDAY
		,ALERTNOTIFYMODEID
		,FREQMONTHDAY
		,ENABLEDFLG
		,ALERTCUSTOMBUSTEXT
	) Values (
		PR_ALERT_SEQ.nextval
		,p_PARTNERID
		,p_ALERTNAME
		,p_ALERTPARTNERID
		,p_ALERTYPE
		,p_ROUTINGOBJREFID
		,p_EVENTCD
		,p_VOLUMETYPE
		,p_VOLUME
		,p_PCTDEV
		,p_VOLMIN
		,p_VOLMAX
		,p_FREQTYPE
		,p_FREQBEGHOUR
		,p_FREQENDHOUR
		,p_FREQWEEKDAY
		,v_AlertNotifyModeID
		,p_FREQMONTHDAY
		,p_ENABLEDFLG
		,p_ALERTCUSTOMBUSTEXT
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		PR_ALERT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ALERTI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_ALERTI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ALERTI' || SQLCODE|| ' '||SQLERRM);
END PR_ALERTI;
/


-- PR_ALERTS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_ALERTS_AH1
(
      p_ReturnCd            out  pls_INTEGER
    , p_DebugFlg            in   pls_INTEGER
    , RC1                   out  db_ref.rct          -- rows
    , RC2                   out  db_ref.rct          -- count
    , p_sortkey             in   varchar2
    , p_PageSizeNbr         in   pls_Integer
    , p_PageNbr             in   pls_Integer
    , p_PartnerId           in   pls_Integer          -- alert owner
    , p_AlertType           in   pls_Integer
    , p_AlertNameWildcard   in   varchar2
    , p_EnabledFlg          in   pls_integer
    , p_SubscribedContact   in   integer     -- 1=SubscribedContact 0=non-subscribedContact null=both
    , p_AlertPartnerId      in   pls_Integer          -- trigger partner,  0 = ANY trigger partner
    , p_Locale              in   varchar2
)
AS
/*
================================================================================
Proc Name:      LG_ALERTS_AH1
Create Date:    02/28/2002
Author          J.CHEN
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME         Ver       DESCRIPTION
---------  --------------------  ---------------------------------------------
2002-02-28 J.CHEN       3.1         Created
2002-03-20 J Chen       3.1         Added p_AlertPartnerId as input
2003-01-02 J.Chen       3.3.2       Call where_clause for p_partnerId and p_AlertPartnerId
                                    Pass in p_AlertPartnerId = 0 means ANY trigger partner
                                    defect 1-154KA, 1-154K6
2003-06-23   V Goel      3.5      Use DB_Paging_Logic and DB_Build_WhereClause
2003-07-01   M.Mount     3.5      Changed parameters to standard.
2003-07-07   V Goel      3.5      Use FN_Paging_Logic and FN_Build_Where Functions
2006-10-18 jriordan	6.1	  103131
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2007 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  v_where            varchar2(2000);
  sql_Column         varchar2(1000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  v_true             boolean := TRUE;
  v_false            boolean := FALSE;
--
-- Procedures
--
------------
-- ProcAudit
------------
Procedure ProcAudit is
v_diffTS Timestamp;
begin
-- If p_DebugFlg =1  Then
  DB_ProcAuditLogI_AH1('LG_ALERTS_AH1',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'||','||'RC2'||','||
    P_SortKey      ||','||
    P_PageSizeNbr  ||','||
    P_PageNbr      ||','||
    p_AlertPartnerId  ||','||
    p_AlertType    ||','||
    p_AlertNameWildcard   ||','||
    p_EnabledFlg
    ,0,0);

--End if ;
End ProcAudit;
---------------
-- Where_clause
---------------
Procedure where_clause as
begin
    v_where := 'WHERE 1=1 '||
  FN_Build_Where('a.PartnerId',       'NUMBER', p_PartnerId)||
  FN_Build_Where('a.AlertPartnerId',  'NUMBER', p_AlertPartnerId)||
  FN_Build_Where('a.AlerType',        'NUMBER', p_AlertType)||
  FN_Build_Where('upper(a.Alertname)','VARCHAR2_LIKE', upper(p_AlertNameWildcard))||
  FN_Build_Where('a.EnabledFlg',      'NUMBER', p_EnabledFlg);
end where_clause;
----------
-- Main
----------
BEGIN
    p_ReturnCd := 0;
-------------------------------
-- Build sql statement for rc1
-------------------------------

    where_clause;
--    dbms_output.put_line(substr(v_where,1,250));
--    dbms_output.put_line(substr(v_where,251,250));
    sql_column := 'SELECT /*+ INDEX(a) */ DISTINCT a.AlertId,  '|| chr(10) ||
                         'a.AlertName,'|| chr(10) ||
                         'a.AlerType,' || chr(10) ||
						 'a.alertNotifyModeId alertNotifyModeId,' || chr(10) ||
                         'n.BatchType,'|| chr(10) ||
                         'p2.PartnerName,'|| chr(10) ||
                         'a.AlertPartnerId,'|| chr(10) ||
                         'p1.PartnerName As OwnerName,' || chr(10) ||
                         'a.PartnerId,'|| chr(10) ||
                         'NVL2(ac.ContactId, 1, 0) AS SubscribedContacts,' || chr(10) ||
                         'a.EnabledFlg, '||chr(10) ||
                         'm.MODENAME AS ALERTNOTIFYMODE, '||chr(10) ||
                         'a.ALERTCUSTOMBUSTEXT '||chr(10);
    sql_inner :=
        'FROM  pr_Alert a '||chr(10)||
        'INNER JOIN pr_Notification n on a.AlertId = n.AlertId '|| chr(10) ||
        'INNER JOIN pr_AlertNotifyMode m on a.AlertNotifyModeId = m.Id ';
    sql_outer :=
        'LEFT OUTER JOIN pr_Alert_Contact ac on a.AlertId = ac.AlertId '|| chr(10) ||
        'LEFT OUTER JOIN pr_Partner p1 on a.PartnerId = p1.PartnerId '|| chr(10) ||
        'LEFT OUTER JOIN pr_Partner p2 on a.AlertPartnerId = p2.PartnerId';
    sqlstmt := sql_column|| chr(10) ||
               sql_inner || chr(10) ||
               sql_outer || chr(10) ||
               v_where;
    sqlstmt2 := sql_inner || chr(10) ||
                sql_outer || chr(10) ||
                v_where;

    IF nvl(p_SubscribedContact,-1) = 2 THEN
       sqlstmt := 'SELECT * FROM ('|| chr(10) || sqlstmt || ') t' || chr(10) ||
                  'WHERE t.alertNotifyModeId = 2 ';
    ELSIF p_SubscribedContact IS NOT NULL THEN
       sqlstmt := 'SELECT * FROM ('|| chr(10) || sqlstmt || ') t' || chr(10) ||
                  'WHERE t.SubscribedContacts = '|| p_SubscribedContact || chr(10) ||
				  ' AND t.alertNotifyModeId = 1 ';
    END IF;
--  DB_Print('sqlstmt=',sqlstmt);
--  DB_Print('v_where=',v_where);
------------------------------------
-- Open rc2 : Total row count Cursor
------------------------------------
    IF RC2%IsOpen THEN close RC2; END IF;
    IF NVL(p_pageNbr,0) in (0,1) then
       open RC2 for 'select count(*) TotalRowsNbr from( '||sqlstmt ||')';
    END IF;
--    DB_Print('rc2 =','select count(*) TotalRowsNbr from( '||sqlstmt||')');
--------------
-- Add sortkey
--------------
    IF p_sortkey is not null THEN
        --EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);
        sqlstmt := sqlstmt ||' order by '||p_sortkey ;
    END IF;
 --   IF p_DebugFlg = 1 Then
        DB_Print('Sqlstmt with order by', sqlstmt);
 --   END IF;
---------------
-- Paging logic
---------------
     sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
     IF p_DebugFlg = 1 Then
        DB_Print('Sqlstmt with paging ', sqlstmt);
     END IF;
--------------
-- Open rc1
--------------
   IF RC1%IsOpen THEN close RC1; END IF;
   BEGIN
        open RC1 for  sqlstmt;

        --IF p_sortkey is not null THEN
            --EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';
        --END IF;
--------------
-- proc Audit
--------------
      --ProcAudit;
        EXCEPTION
        WHEN OTHERS THEN
            ProcAudit;
            p_ReturnCd := 1;
        END;

        IF p_DebugFlg = 1 THEN
           --dbms_output.put_line('Start '||StartRow||' EndRow '||endrow);
           DB_Print('sqlstmt=',sqlstmt);
        END IF;
END ;
/


-- PR_ALERTS_AH2.sql

CREATE OR REPLACE PROCEDURE PR_ALERTS_AH2 (
      p_ReturnCd        out  pls_INTEGER
    , p_DebugFlg        in   pls_INTEGER
    , RC1               out  db_ref.rct          -- rows
    , p_EventCode         in   varchar2
    , p_AlertPartnerId  in   pls_Integer
    , p_AlertTargetPartnerid in pls_integer
  )
AS
/*
================================================================================
Proc Name:      PR_ALERTS_AH2
Purpose:        Returns all event alerts that match input criteria.
Create Date:    03/04/2002
Author          J.CHEN
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  -------------------  ---------------------------------------------
DATE       NAME                  DESCRIPTION
---------  -------------------  ---------------------------------------------
03/04/02   J.CHEN               Created
03/26/04   T. Dang              Added resourcebundleid column for lg_eventcd
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  v_where            varchar2(2000);
  sql_Column         varchar2(1000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  v_true             boolean := TRUE;
  v_false            boolean := FALSE;
  v_alertpartnerid    integer;
  v_alerttargetpartnerid integer;
  v_alertpar_list varchar2(100);
--
-- Procedures
--
------------
-- ProcAudit
------------
Procedure ProcAudit is
v_diffTS Timestamp;
begin
 --If p_DebugFlg =1  Then
  DB_ProcAuditLogI_AH1('PR_ALERTS_AH2',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'||','||
    p_EventCode  ||','||
    p_AlertPartnerId
    );

--End if ;
End ProcAudit;
----------
-- Main
----------
BEGIN
    p_ReturnCd := 0;
-------------------------------
-- Build sql statement for rc1
-------------------------------
--  Event Alert Only

    v_alertpartnerid := coalesce(p_AlertPartnerId,0);
    v_alerttargetpartnerid := coalesce(p_AlertTargetPartnerid,0);
    v_alertpar_list := '(' || to_char(v_alertpartnerid) || ',' || to_char(v_alerttargetpartnerid) || ',' || to_char(0) || ')' ;

    v_where := 'WHERE a.AlerType = 1 AND a.EnabledFlg = 1 '||
                FN_Build_Where('c.EventCode', 'VARCHAR2', p_EventCode);

    v_where := v_where || ' AND a.AlertPartnerId IN ' ||
        			v_alertpar_list ;
    					
    
--    dbms_output.put_line(substr(v_where,1,250));
--    dbms_output.put_line(substr(v_where,251,250));
    sql_column := 'SELECT /*+ INDEX(pr_Alert) */ a.AlertId,  '|| chr(10) ||
                         'a.PARTNERID, '|| chr(10) ||
                         'p.PartnerName, '|| chr(10) ||
                         'a.ALERTNAME, '|| chr(10) ||
                         'a.ALERTPARTNERID, '|| chr(10) ||
			 's.ModeName, '|| chr(10) ||
                         'ap.PartnerName As AlertPartnerName, '|| chr(10) ||
                         'a.ALERTYPE, '|| chr(10) ||
                         'o.RoutingObjCd As PROCESSCD, '|| chr(10) ||
                         'a.EVENTCD, '|| chr(10) ||
                         'c.EventName, '|| chr(10) ||
                         'c.InternalDesc, '|| chr(10) ||
                         'a.FREQTYPE, '|| chr(10) ||
                         'a.FREQBEGHOUR, '||chr(10) ||
                         'a.FREQENDHOUR, '||chr(10) ||
                         'a.FREQWEEKDAY, '||chr(10) ||
                         'a.FREQMONTHDAY, '||chr(10) ||
                         'a.ENABLEDFLG, '||chr(10)||
                         'n.BatchType, ' || chr(10) ||
                         'n.Frequency, ' || chr(10) ||
                         'n.BatchCnt '||chr(10) ||
                         ', pra.RESBUNDLEFILENAME as eventnameRESBUNDLEFILENAME '  || chr(10) ||
                         ', c.EVENTNAMEKEY '  || chr(10) ||
                         ', c.TEXTKEY '  || chr(10) ||
                         ', c.EVENTCODE '  || chr(10) ||
                         ', prb.RESBUNDLEFILENAME as textkeyRESBUNDLEFILENAME '  || chr(10) ||
                         ', c.EXTERNALDESCRESOURCEBUNDLEID '   || chr(10) ||
                         ', a.ALERTCUSTOMBUSTEXT ' 
			 ;
    sql_inner :=
        'FROM  pr_Alert a' || chr(10) ||
        'Left Outer Join cf_RoutingObjRef_Lnk l on l.RoutingObjRefId = a.RoutingObjRefId '||chr(10) ||
        'Left Outer Join cf_RoutingObject o on o.RoutingobjId = l.RoutingObjId '||chr(10) ||
        'LEFT OUTER JOIN pr_Partner p on p.PartnerId = a.PartnerId '|| chr(10) ||
        'LEFT OUTER JOIN pr_Partner ap on ap.PartnerId = a.AlertPartnerId ' ||chr(10)||
        'LEFT OUTER JOIN pr_Notification n on n.AlertId = a.AlertId '||chr(10)||
        'LEFT OUTER JOIN pr_AlertNotifyMode s on s.Id = a.AlertNotifyModeId '||chr(10)||
        'LEFT OUTER JOIN lg_EventCd c on c.EventCD = a.EventCD '||chr(10)||
        'LEFT OUTER JOIN pr_resourcebundles pra on pra.resourcebundleid = c.eventnameresourcebundleid ' ||chr(10)||
        'LEFT OUTER JOIN pr_resourcebundles prb on prb.resourcebundleid = c.textkeyresourcebundleid ';
    sqlstmt := sql_column|| chr(10) ||
               sql_inner || chr(10) ||
               v_where;
                             
--------------
-- Open rc1
--------------
   IF RC1%IsOpen THEN close RC1; END IF;
   BEGIN
        open RC1 for  sqlstmt;
--------------
-- proc Audit
--------------
        ProcAudit;
        EXCEPTION
        WHEN OTHERS THEN
            ProcAudit;
            p_ReturnCd := 1;
        END;
    IF p_DebugFlg = 1 THEN
       DB_Print('sqlstmt for rc1=',sqlstmt);
    END IF;

END ;
/


-- PR_ALERTS_AH3.sql

CREATE OR REPLACE PROCEDURE         PR_ALERTS_AH3 (
      p_ReturnCd            out  pls_INTEGER
    , p_DebugFlg                 in      pls_INTEGER
    , RC1                    out  db_ref.rct          -- rows
     )
AS
/*
================================================================================
Proc Name:      LG_ALERTS_AH3
Purpose:        Returns list of current enabled volume alerts.
Create Date:    03/04/2002
Author          J.CHEN
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  sqlstmt            varchar2(4000);
  v_where            varchar2(2000);
  sql_Column         varchar2(1000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
--
-- Procedures
--
------------
-- ProcAudit
------------
Procedure ProcAudit is
v_diffTS Timestamp;
begin
  DB_ProcAuditLogI_Ah1('PR_ALERTS_AH3',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'
     );
End ProcAudit;
-------------------
--     Main
-------------------
BEGIN
    p_ReturnCd := 0;
-------------------------------
-- Build sql statement for rc1
-------------------------------
--  Enabled Volume Alert Only
    v_where := 'WHERE a.AlerType = 2 AND a.EnabledFlg = 1 ';
    sql_column := 'SELECT a.AlertId,  '|| chr(10) ||
                         'a.PARTNERID, '|| chr(10) ||
                         'p.PartnerName, '|| chr(10) ||
                         'a.ALERTNAME, '|| chr(10) ||
                         'a.ALERTPARTNERID, '|| chr(10) ||
                         'ap.PartnerName As AlertPartnerName, '|| chr(10) ||
                         'a.ALERTYPE, '|| chr(10) ||
                         'o.RoutingObjCd As ProcessCD, '|| chr(10) ||
                         'a.EVENTCD, '|| chr(10) ||
                         'a.VOLUMETYPE, '|| chr(10) ||
                         'a.VOLUME, '|| chr(10) ||
                         'a.PCTDEV, '|| chr(10) ||
                         'a.VOLMIN, '|| chr(10) ||
                         'a.VOLMAX, '|| chr(10) ||
                         'a.FREQTYPE, '|| chr(10) ||
                         'a.FREQBEGHOUR, '||chr(10) ||
                         'a.FREQENDHOUR, '||chr(10) ||
                         'a.FREQWEEKDAY, '||chr(10) ||
                         'a.FREQMONTHDAY, '||chr(10) ||
                         'a.ENABLEDFLG, '||chr(10) ||
                         'e.LastEvalDT, '||chr(10) ||
                         'm.MODENAME, '||chr(10) ||
                         'a.ALERTCUSTOMBUSTEXT '||chr(10) ;
    sql_inner :=
        'FROM  pr_Alert a '||chr(10)||
        'Inner join PR_ALERTNOTIFYMODE m on m.ID = a.ALERTNOTIFYMODEID '||chr(10)||
        'Left Outer Join cf_RoutingObjRef_Lnk l on l.RoutingObjRefId = a.RoutingObjRefId '||chr(10) ||
        'Inner join cf_RoutingObject o on o.RoutingobjId = l.RoutingObjId '||  chr(10) ||      
        'LEFT OUTER JOIN pr_AlertEval_Log e on a.AlertId = e.AlertId ' || chr(10) ||
        'LEFT OUTER JOIN pr_Partner p on p.PartnerId = a.PartnerId '||chr(10)||
        'LEFT OUTER JOIN pr_Partner ap on ap.PartnerId = a.AlertPartnerId ';
    sqlstmt := sql_column|| chr(10) ||
               sql_inner || chr(10) ||
               v_where;
--------------
-- Open rc1
--------------
   IF RC1%IsOpen THEN close RC1; END IF;
   BEGIN
       open RC1 for  sqlstmt;
--------------
-- proc Audit
--------------
        ProcAudit;
        EXCEPTION
        WHEN OTHERS THEN
            ProcAudit;
            p_ReturnCd := 1;
        END;
    IF p_DebugFlg = 1 THEN
       --dbms_output.put_line('Start '||StartRow||' EndRow '||endrow);
       db_Print('sqlstmt for rc1=',sqlstmt);
    END IF;
END ;
/


-- PR_ALERTS_AH4.sql

CREATE OR REPLACE PROCEDURE PR_ALERTS_AH4 (
	p_ReturnCd		out	pls_INTEGER
	, p_DebugFlg		in	pls_INTEGER
	, RC1			out	db_ref.rct		-- rows
	, p_AlertId		in	pls_Integer		--required
	, p_EvalDateStart	in	Timestamp		--required
	, p_EvalDateEnd		in	Timestamp		--required
	)
AS
/*
================================================================================
Proc Name:	LG_ALERTS_AH4
Purpose:	Returns success/fail status and actual volume found for volume alerts.
Create Date:	03/04/2002
Author		J.CHEN
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
	v_beginTS		timestamp(6):= FN_Get_Date;
	v_ProcName		varchar2(30)	:= 'PR_ALERTS_AH4';
	sqlstmt			varchar2(4000);
	v_where			varchar2(2000);
	sql_Column		varchar2(1000);
	sql_inner		varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
	sql_outer		varchar2(1000);
	v_EvalDate		timestamp(6);
	v_RoutingObjRefId	integer;
	v_VolumeType		integer;
	v_Volume		number(10);
	v_PctDev		number(10);
	v_volMin		number(10);
	v_volMax		number(10);
	v_errmsg		varchar2(1000);
	v_errnum		integer;
	v_startdate		varchar2(20) := to_char(p_EvalDateStart, 'dd-mm-yy hh24:mi:ss');
	v_enddate		varchar2(20) := to_char(p_EvalDateEnd, 'dd-mm-yy hh24:mi:ss');
	v_rowcnt		number(10);
	v_ParamsTxt		VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_AlertID=>'||p_AlertID||CHR(10)
		||', p_EvalDateStart=>'||p_EvalDateStart||CHR(10)
		||', p_EvalDateEnd=>'||p_EvalDateEnd||CHR(10)
		;
	Unable_To_Lock		EXCEPTION;
	Skip_Locked		EXCEPTION;
	PRAGMA EXCEPTION_INIT( Unable_To_Lock, -54);
--
-- Procedures
--
------------
-- ProcAudit
------------
Procedure ProcAudit is
BEGIN
	DB_ProcAuditLogI_Ah1('LG_ALERTS_AH4',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
	','||'RC1'||','||
	p_AlertId	||','||
	p_EvalDateStart || ','||
	p_EvalDateEnd
	);
End ProcAudit;
-------------------------------
--		Main
-------------------------------
BEGIN
	p_ReturnCd := 0;
	BEGIN
		SELECT	LastEvalDT 
		INTO	v_EvalDate 
		FROM	pr_AlertEval_Log
		WHERE	AlertId = p_AlertId
		FOR UPDATE NOWAIT;

	EXCEPTION
		WHEN Unable_To_Lock THEN
			RAISE Skip_Locked; --- another thread is working
		WHEN No_Data_Found THEN
			INSERT INTO pr_AlertEval_Log(AlertId, LastEvalDT, CREATEDATE)
			VALUES (p_AlertId,p_EvalDateEnd,sysTimeStamp);
	END;
	IF v_EvalDate >= p_EvalDateEnd THEN  --already updated by another thread
		RAISE Skip_Locked;
	END IF;
-------------------------
-- Get alert definition
-------------------------
	BEGIN
		SELECT RoutingObjRefId,
			VOLUMETYPE,
			VOLUME,
			PCTDEV,
			VOLMIN,
			VOLMAX
		INTO	v_RoutingObjRefId,
			v_VolumeType,
			v_Volume,
			v_PctDev,
			v_volMin,
			v_volMax
		FROM	pr_alert
		WHERE	AlertId = p_AlertId;

	END;

	IF v_RoutingObjRefId IS NULL THEN
		sql_inner :=
		' FROM lg_summary s '||
		'inner join lg_summary_xref x2 on s.summaryid = x2.summaryId '||
		'inner join pr_alert a2 on (x2.FRPartnerid = a2.AlertPartnerId) ' ||
		'Inner join pr_partner p on p.partnerid = a2.PartnerId ' ||
		'Inner join pr_partnertype pt on pt.partnertypeId = p.partnertypeId ';
	ELSE
		sql_inner :=
		' FROM lg_summary s '||
		'inner join lg_summary_xref_vw x2 on s.summaryid = x2.summaryId '||
		'inner join pr_alert a2 on (a2.AlertPartnerId = x2.FRPartnerId) '||
		'inner join CF_RoutingObject03_vw o on o.RoutingobjCd = x2.FRProcessCd '||
						' and o.RoutingObjVer = x2.FrProcessVer '||
						' and o.ParentObjCd = x2.FrProtocolname '||
						' and o.ParentObjVer = x2.FrProtocolVer '||
						' and o.GRParentObjCd = x2.FrPackageName '||
						' and o.GRParentObjVer = x2.FrPackageVer '||
		'inner join CF_RoutingObjRef_lnk l on l.RoutingobjRefId = a2.RoutingObjRefId And l.RoutingObjId = o.RoutingObjId '||
		'Inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId ' ||
		'Inner join pr_partner p on p.partnerid = a2.PartnerId ' ||
		'Inner join pr_partnertype pt on pt.partnertypeId = p.partnertypeId ';
	END IF;

	sqlstmt :=	'SELECT '|| chr(10) ||
				'CASE '||v_volumeType || chr(10) ||
				-- expected
				'WHEN 1 THEN '|| chr(10) ||
					'CASE WHEN b.rcvDocCnt <= '||v_Volume||' + '||v_volume * (v_PctDev/100)||' and b.rcvDocCnt >= '||v_Volume||' - ' ||v_volume * (v_PctDev/100)|| ' THEN '||chr(10)||
						'''Success'''||chr(10)||
					' ELSE '||chr(10)||
						'''Fail'''||chr(10)||
				' END '||chr(10)||
				--Range
				'WHEN 2 THEN '||chr(10)||
					'CASE WHEN b.rcvDocCnt >= '||v_VolMin||' and b.rcvDocCnt <= '||v_VolMax||' THEN '||chr(10)||
							'''Success'''||chr(10)||
					' ELSE '|| chr(10) ||
							'''Fail'''||chr(10)||
					' END '||chr(10)||
				-- Zero Volume
				'WHEN 3 THEN '||chr(10)||
					'CASE WHEN b.rcvDocCnt > 0 THEN '||chr(10)||
							'''Success'''||chr(10)||
					' ELSE '||chr(10)||
							'''Fail'''||chr(10)||
					' END '||chr(10)||
				' END AS Status, '||chr(10)||
				'b.rcvDocCnt As ActualVolume '||chr(10)||
		'FROM	'||chr(10)||
			'(select NVL(sum(s.rcvdoccnt - s.faildoccnt),0) as rcvdoccnt '||chr(10)||
			 sql_inner ||
			 'WHERE a2.AlertId = '||p_AlertId || chr(10) ||
			 'AND s.DayHour BETWEEN :TSstart AND :TSend ' || chr(10) ||
			 'AND x2.InitPartnerId = x2.frPartnerId AND x2.actionflg=1 ' || chr(10) ||
			 ' AND ((pt.partnertypeName != ' || '''HubPartner''' || ') OR (pt.partnertypeName = ' || '''HubPartner''' || ' AND (a2.PartnerId = x2.ToPartnerId or a2.PartnerId = x2.FrPartnerId)))) b';

---------------------------
-- Update pr_AlertEval_Log
---------------------------
		UPDATE	pr_AlertEval_Log
		SET	LastEvalDT = p_EvalDateEnd
		WHERE	AlertId = p_AlertId;
--------------
-- Open rc1
--------------
	db_print('SQLSTMT', sqlstmt);

	IF RC1%IsOpen THEN close RC1; END IF;
	BEGIN
		OPEN RC1 FOR sqlstmt USING p_EvalDateStart, p_EvalDateEnd;
		ProcAudit;
	END;
EXCEPTION
	WHEN Dup_Val_On_Index THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt || ' Init', p_DebugFlg, 0);
	WHEN Skip_Locked THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt || ' AnotherThread', p_DebugFlg, 0);
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		RAISE;	
END;
/



-- PR_ALERTS_AH5.sql

CREATE OR REPLACE PROCEDURE PR_ALERTS_AH5
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER	
	, RC1  OUT  DB_REF.RCT
	)
AS

/*
	================================================================================
	Proc Name:       PR_ALERTS_AH5
	Create Date:     2006-07-27 00:06:46
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2006-07-27 SYSTEM        6.1 	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ALERTS_AH5';
v_ExceptionFlg    integer Default 0;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
	SELECT
		EVENTCD
	FROM
		LG_EVENTCD
	WHERE
		ALERTABLEFLG = 1
	;
	p_ReturnCd := 1;
	--DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_DebugFlg, v_ExceptionFlg);
	
Exception

    WHEN OTHERS  THEN
        p_ReturnCd := -1;
        Raise_application_error (-20999, '20000 PR_ALERTS_AH5' || SQLERRM);
	
End PR_ALERTS_AH5;
/


-- PR_ALERTS_AH6.sql

CREATE OR REPLACE PROCEDURE PR_ALERTS_AH6 (
      p_ReturnCd   out  pls_INTEGER
    , p_DebugFlg   in   pls_INTEGER
    , RC1  out  db_ref.rct     
    ,p_alertownerid IN PLS_INTEGER
    ,p_alertpartnerid IN PLS_INTEGER
    ,p_alertype IN PLS_INTEGER
     )
AS
/*
================================================================================
Proc Name:      PR_ALERTS_AH6
Purpose:        Returns list of Alert details used for partner Migration feature.
Create Date:    08/09/2007
Author          BHARANI J
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  sqlstmt            varchar2(4000);
  v_where            varchar2(2000);
  sql_Column         varchar2(1000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
--
-- Procedures
--
------------
-- ProcAudit
------------
Procedure ProcAudit is
v_diffTS Timestamp;
begin
  DB_ProcAuditLogI_Ah1('PR_ALERTS_AH6',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'
     );
End ProcAudit;
-------------------
--     Main
-------------------
BEGIN
    p_ReturnCd := 0;
-------------------------------
-- Build sql statement for rc1
-------------------------------
--  Enabled Volume Alert Only

      v_where := 'WHERE 1=1 '||
       FN_Build_Where('a.PartnerId',       'NUMBER', p_alertownerid)||
       FN_Build_Where('a.AlertPartnerId',  'NUMBER', p_AlertPartnerId)||
       FN_Build_Where('a.AlerType',        'NUMBER', p_alertype);
     
  
    sql_column := 'SELECT a.AlertId,  '|| chr(10) ||
                         'a.PARTNERID, '|| chr(10) ||                         
                         'a.ALERTNAME, '|| chr(10) ||
                         'a.ALERTPARTNERID, '|| chr(10) ||
                         'a.alertype, ' || chr(10)||
                         'a.routingobjrefid, ' || chr(10) ||     
                         'a.eventcd, ' || chr(10) ||
                         'a.volumetype, ' || chr(10) ||
                         'a.volume,' || chr(10) ||
                         'a.pctdev, ' || chr(10) ||
                         'a.volmin, ' || chr(10) ||
                         'a.volmax, ' || chr(10) ||
                         'a.freqtype,' || chr(10) ||                    
                         'RTRIM (freqbeghour) freqbeghour, ' || chr(10) || 
                         'RTRIM (freqendhour) freqendhour, ' || chr(10) ||
                         'RTRIM (freqweekday) freqweekday, ' || chr(10) ||
                         'RTRIM (freqmonthday) freqmonthday, ' || chr(10) ||
                         'a.enabledflg, ' || chr(10) ||
                         'a.rowts,' || chr(10) ||
                         'a.alertnotifymodeid, ' || chr(10) ||
                         'a.createdate, ' || chr(10) ||                        
                         'an.modename, ' || chr(10) ||
                         'a.ALERTCUSTOMBUSTEXT ' || chr(10) ;                   
    sql_inner :=
        'FROM  pr_Alert a '||chr(10)||        
        'Left Outer Join pr_AlertNotifyMode an on an.id = a.alertnotifymodeid '
        ;
        
        
    sqlstmt := sql_column|| chr(10) ||
               sql_inner || chr(10) ||
               v_where;
               
 -- dbms_output.put_line(sqlstmt);
 
--------------
-- Open rc1
--------------
   IF RC1%IsOpen THEN close RC1; END IF;
   BEGIN
       open RC1 for  sqlstmt;
--------------
-- proc Audit
--------------
        ProcAudit;
        EXCEPTION
        WHEN OTHERS THEN
            ProcAudit;
            p_ReturnCd := 1;
        END;
    IF p_DebugFlg = 1 THEN
       --dbms_output.put_line('Start '||StartRow||' EndRow '||endrow);
       db_Print('sqlstmt for rc1=',sqlstmt);
    END IF;
END ;
/

-- PR_ALERTS_CONTACTID.SQL

CREATE OR REPLACE PROCEDURE         PR_ALERTS_ContactId (
      p_ReturnCd            out  pls_INTEGER
    , p_DebugFlg            in   pls_INTEGER
    , RC1                   out  db_ref.rct
    , p_ContactId           in   number
    )
AS
/*
================================================================================
Proc Name:      PR_ALERTs_ContactId
Create Date:    2002-12-05
Author          J.Chen
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME         Ver       DESCRIPTION
---------  --------------------  ---------------------------------------------
2002-12-05 J.Chen       3.3.2    Display list of alerts to which the user/contact is subscribed.
2003-07-01 M.Mount      3.5      Changed input parameters to standard
2003-07-08 V.Goel       3.5      Use FN_Build_Where
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  v_where            varchar2(2000);
  sql_Column         varchar2(1000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  v_true             boolean := TRUE;
  v_false            boolean := FALSE;
--
-- Procedures
--
------------
-- ProcAudit
------------
Procedure ProcAudit is
v_diffTS Timestamp;
begin
-- If p_DebugFlg =1  Then
  DB_ProcAuditLogI_ah1('LG_ALERTS_ContactId',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||
    ','||'RC1'||','||'RC2'||','||
    p_contactid
    );

--End if ;
End ProcAudit;
---------------
-- Where_clause
---------------
Procedure where_clause as
begin
    v_where := 'where 1=1 '||
    FN_Build_Where('contactId', 'NUMBER', p_ContactId);
end where_clause;
-------------------------------------------------------------------
--                             M A I N
-------------------------------------------------------------------
BEGIN
    --dbms_output.enable(1000000);
    p_ReturnCd := 0;
-------------------------------
-- Build sql statement for rc1
-------------------------------

    IF p_ContactId IS NOT NULL THEN
       where_clause;
    END IF;


--    dbms_output.put_line(substr(v_where,1,250));
--    dbms_output.put_line(substr(v_where,251,250));
    sql_column := 'SELECT /*+ INDEX(a) */ DISTINCT a.AlertId,  '|| chr(10) ||
                         'a.AlertName,'|| chr(10) ||
                         'a.AlerType,' || chr(10) ||
                         'n.BatchType,'|| chr(10) ||
                         'p2.PartnerName,'|| chr(10) ||
                         'a.AlertPartnerId,'|| chr(10) ||
                         'p1.PartnerName As OwnerName,' || chr(10) ||
                         'a.PartnerId,'|| chr(10) ||
                         --'NVL2(ac.ContactId, 1, 0) AS SubscribedContacts,' || chr(10) ||
                         'a.EnabledFlg, '||chr(10) ||
                         'a.ALERTCUSTOMBUSTEXT ' ||chr(10) ;
    sql_inner :=
        'FROM  pr_Alert a '||chr(10)||
        'INNER JOIN pr_Notification n on a.AlertId = n.AlertId ';
    sql_outer :=
        'LEFT OUTER JOIN pr_Alert_Contact ac on a.AlertId = ac.AlertId '|| chr(10) ||
        'LEFT OUTER JOIN pr_Partner p1 on a.PartnerId = p1.PartnerId '|| chr(10) ||
        'LEFT OUTER JOIN pr_Partner p2 on a.AlertPartnerId = p2.PartnerId';
    sqlstmt := sql_column|| chr(10) ||
               sql_inner || chr(10) ||
               sql_outer || chr(10) ||
               v_where;

   -- IF p_SubscribedContact IS NOT NULL THEN
   --    sqlstmt := 'SELECT * FROM ('|| chr(10) || sqlstmt || ') t' || chr(10) ||
   --               'WHERE t.SubscribedContacts = '|| p_SubscribedContact ;
   -- END IF;
--  DB_Print('sqlstmt=',sqlstmt);
--  DB_Print('v_where=',v_where);

--------------
-- Open rc1
--------------
   IF RC1%IsOpen THEN close RC1; END IF;
   BEGIN
       open RC1 for  sqlstmt;
--------------
-- proc Audit
--------------
      --ProcAudit;
        EXCEPTION
        WHEN OTHERS THEN
            ProcAudit;
            p_ReturnCd := 1;
        END;
--    IF p_DebugFlg = 1 THEN
       --dbms_output.put_line('Start '||StartRow||' EndRow '||endrow);
       DB_Print('sqlstmt=',sqlstmt);
--    END IF;
END ;
/


-- PR_ALERTS_PNAME.sql

CREATE OR REPLACE PROCEDURE PR_ALERTS_PNAME
(
   p_ReturnCd out integer,
   p_DebugFlg in pls_integer,
   RC1 out db_ref.RCT,
   p_ALERTID in NUMBER
) As
/*
================================================================================
Proc Name:      PR_ALERTs_Pname
Create Date:    03/07/02
Author          SYSTEM
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_ALERTs_Pname',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_ALERTID
    );
End ProcAudit;
BEGIN
    Open RC1 for
    select
    ALERTID
    ,   a.PARTNERID, RTRIM(p.PartnerName) PartnerName
    ,   RTRIM(ALERTNAME)  ALERTNAME
    ,   a.ALERTPARTNERID, RTRIM(ap.PartnerName) AlertPartnerName
    ,   ALERTYPE
    ,   a.RoutingObjRefId
    ,   a.EVENTCD, RTRIM(e.EVENTNAME) EVENTNAME
    ,   VOLUMETYPE
    ,   VOLUME
    ,   PCTDEV
    ,   VOLMIN
    ,   VOLMAX
    ,   FREQTYPE
    ,   RTRIM(FREQBEGHOUR)  FREQBEGHOUR
    ,   RTRIM(FREQENDHOUR)  FREQENDHOUR
    ,   RTRIM(FREQWEEKDAY)  FREQWEEKDAY
    ,   RTRIM(FREQMONTHDAY)  FREQMONTHDAY
    ,   a.ENABLEDFLG
    ,   a.ROWTS
    ,   a.CREATEDATE
	,   e.EVENTCODE
	,   e.EVENTNAMERESOURCEBUNDLEID
   ,   e.EVENTNAMEKEY
   ,   m.MODENAME ALERTNOTIFYMODE
   ,   a.ALERTCUSTOMBUSTEXT 
    From PR_ALERT a
    inner join PR_AlertNotifyMode m on a.AlertNotifyModeId = m.Id
    Left outer join pr_partner p  on  p.partnerid = a.partnerid
    Left outer join pr_partner ap on ap.partnerid = a.AlertPartnerid    -- 2000-03-19 JC PartnerId -> AlertPartnerId
    Left outer join Lg_EventCD e  on a.EVENTCD    = e.EVENTCD
    Where ALERTID = p_ALERTID
    ;

    p_ReturnCd := 0;
    procAudit;
EXCEPTION
When Others Then
p_ReturnCd := 1;
Rollback work;
ProcAudit;
Raise_application_error (-20999,'20000 PR_ALERTs_Pname' || SQLCODE|| ' '||SQLERRM);
End PR_ALERTs_Pname;
/


-- PR_ALERTU.sql

CREATE OR REPLACE Procedure PR_ALERTU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_ALERTID in NUMBER
	,p_PARTNERID in NUMBER
	,p_ALERTNAME in VARCHAR2
	,p_ALERTPARTNERID in NUMBER
	,p_ALERTYPE in NUMBER
	,p_ROUTINGOBJREFID in NUMBER
	,p_EVENTCD in NUMBER
	,p_VOLUMETYPE in NUMBER
	,p_VOLUME in NUMBER
	,p_PCTDEV in NUMBER
	,p_VOLMIN in NUMBER
	,p_VOLMAX in NUMBER
	,p_FREQTYPE in NUMBER
	,p_FREQBEGHOUR in VARCHAR2
	,p_FREQENDHOUR in VARCHAR2
	,p_FREQWEEKDAY in VARCHAR2
	,p_FREQMONTHDAY in VARCHAR2
	,p_ENABLEDFLG in NUMBER
	,p_ROWTS in NUMBER
	,p_MODENAME in VARCHAR2
	,p_ALERTCUSTOMBUSTEXT in VARCHAR2
) as

/*
	================================================================================
	Proc Name:       PR_ALERTU
	Create Date:     2003-09-26 08:21:10
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ALERTU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ALERTID=>'||p_ALERTID||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_ALERTNAME=>'||p_ALERTNAME||CHR(10)
		||', p_ALERTPARTNERID=>'||p_ALERTPARTNERID||CHR(10)
		||', p_ALERTYPE=>'||p_ALERTYPE||CHR(10)
		||', p_ROUTINGOBJREFID=>'||p_ROUTINGOBJREFID||CHR(10)
		||', p_EVENTCD=>'||p_EVENTCD||CHR(10)
		||', p_VOLUMETYPE=>'||p_VOLUMETYPE||CHR(10)
		||', p_VOLUME=>'||p_VOLUME||CHR(10)
		||', p_PCTDEV=>'||p_PCTDEV||CHR(10)
		||', p_VOLMIN=>'||p_VOLMIN||CHR(10)
		||', p_VOLMAX=>'||p_VOLMAX||CHR(10)
		||', p_FREQTYPE=>'||p_FREQTYPE||CHR(10)
		||', p_FREQBEGHOUR=>'||p_FREQBEGHOUR||CHR(10)
		||', p_FREQENDHOUR=>'||p_FREQENDHOUR||CHR(10)
		||', p_FREQWEEKDAY=>'||p_FREQWEEKDAY||CHR(10)
		||', p_FREQMONTHDAY=>'||p_FREQMONTHDAY||CHR(10)
		||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_MODENAME=>'||p_MODENAME||CHR(10)
		||', p_ALERTCUSTOMBUSTEXT=>'||p_ALERTCUSTOMBUSTEXT||CHR(10)
		;
v_AlertNotifyModeID integer Default 0;
v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	
	Select Id into v_AlertNotifyModeID from PR_AlertNotifyMode 
	Where ModeName = p_ModeName;
	
	Update PR_ALERT
	SET
		ALERTID=p_ALERTID
		,      PARTNERID=p_PARTNERID
		,      ALERTNAME=p_ALERTNAME
		,      ALERTPARTNERID=p_ALERTPARTNERID
		,      ALERTYPE=p_ALERTYPE
		,      ROUTINGOBJREFID=p_ROUTINGOBJREFID
		,      EVENTCD=p_EVENTCD
		,      VOLUMETYPE=p_VOLUMETYPE
		,      VOLUME=p_VOLUME
		,      PCTDEV=p_PCTDEV
		,      VOLMIN=p_VOLMIN
		,      VOLMAX=p_VOLMAX
		,      FREQTYPE=p_FREQTYPE
		,      FREQBEGHOUR=p_FREQBEGHOUR
		,      FREQENDHOUR=p_FREQENDHOUR
		,      FREQWEEKDAY=p_FREQWEEKDAY
		,      FREQMONTHDAY=p_FREQMONTHDAY
		,      ENABLEDFLG=p_ENABLEDFLG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      ALERTNOTIFYMODEID=v_AlertNotifyModeID
		,      ALERTCUSTOMBUSTEXT=p_ALERTCUSTOMBUSTEXT
		Where  AlertId = p_AlertId
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_ALERTU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ALERTU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ALERTU:'||SQLERRM);
End PR_ALERTU;
/


-- PR_ALERT_CONTACTD_AH1.sql

CREATE OR REPLACE PROCEDURE PR_ALERT_CONTACTD_AH1
(p_ReturnCd out integer, p_DebugFlg in pls_integer
,p_ALERTID in NUMBER
) As
/*
================================================================================
Proc Name:      PR_ALERT_CONTACTD_AH1
Create Date:    05/23/03 11:11:08 AM
Author          SYSTEM
================================================================================

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
zeroDelete  Exception;
Procedure ProcAudit is
begin
DB_ProcAuditLogI_Ah1('PR_ALERT_CONTACTD_AH1',v_beginTS,
'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
||','||p_ALERTID
);
End ProcAudit;
BEGIN
   Delete From PR_ALERT_CONTACT
   Where ALERTID = p_ALERTID;

   v_rowcnt := SQL%ROWCOUNT;
    if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
   ProcAudit;
   p_ReturnCd := 0;

   Exception
   When zeroDelete Then ProcAudit;
       --Raise_application_error (-20999,'20002 PR_ALERT_CONTACTD_AH1: No Data Found');
   When Others Then ProcAudit;
       p_ReturnCd := 1;
       Raise_application_error (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
End PR_ALERT_CONTACTD_AH1;
/


-- PR_ALERT_CONTACTI.sql

CREATE OR REPLACE Procedure PR_ALERT_CONTACTI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ALERTID in NUMBER
	,p_CONTACTID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_ALERT_CONTACTI
	Create Date:     2003-09-26 00:03:47
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ALERT_CONTACTI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ALERTID=>'||p_ALERTID||CHR(10)
		||', p_CONTACTID=>'||p_CONTACTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_ALERT_CONTACT(
		ALERTID
		,CONTACTID
	) Values (
		p_ALERTID
		,p_CONTACTID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ALERT_CONTACTI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_ALERT_CONTACTI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ALERT_CONTACTI' || SQLCODE|| ' '||SQLERRM);
END PR_ALERT_CONTACTI;
/


-- PR_ALERT_CONTACTS_ALERTID.sql

CREATE OR REPLACE PROCEDURE PR_ALERT_CONTACTS_ALERTID
(
   p_ReturnCd out integer,
   p_DebugFlg in pls_integer,
   RC1 out db_ref.RCT,
   p_AlertID in NUMBER
) AS
/*
================================================================================
Proc Name:      PR_Alert_CONTACTs_alertId
Create Date:    02/28/02 11:22:34 AM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
DB_ProcAuditLogI_Ah1('PR_CONTACTs',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
||','||p_AlertID
);
End ProcAudit;
BEGIN
   Open RC1 for
   SELECT
    c.CONTACTID
    ,   c.PARTNERID, RTRIM(p.PartnerName) PartnerName
    ,   RTRIM(NAME1)  NAME1
    ,   RTRIM(NAME2)  NAME2
    ,   RTRIM(NAME3)  NAME3
    ,   RTRIM(TITLE)  TITLE
    ,   RTRIM(EMAIL)  EMAIL
    ,   RTRIM(ADDRESS)  ADDRESS
    ,   RTRIM(PHONEBUS)  PHONEBUS
    ,   RTRIM(PHONEMBL)  PHONEMBL
    ,   RTRIM(PHONEPGR)  PHONEPGR
    ,   RTRIM(PHONEFAX)  PHONEFAX
    ,   NOTIFYFLG
    ,   VISIBILITYFLG
    ,   RTRIM(LanguageLocale)  LanguageLocale
    ,   RTRIM(FormatLocale)  FormatLocale
    ,   RTRIM(TimeZone)  TimeZone
   From PR_CONTACT c
    Inner Join pr_alert_contact ac on c.contactID  = ac.contactID
    Left outer join pr_partner  p  on p.partnerid  = c.partnerid
   Where ac.AlertID = p_AlertID;

   p_ReturnCd := 0;
   procAudit;
End PR_Alert_CONTACTs_alertId;
/


-- PR_BATCH_LOGD_NOTIFYFLG.sql

CREATE OR REPLACE PROCEDURE PR_BATCH_LOGD_NOTIFYFLG
(
   p_ReturnCd out integer,
   p_DebugFlg in pls_integer
   ,p_TimeStamp in Timestamp
) As
/*
================================================================================
Proc Name:      PR_BATCH_LOGd
Create Date:    03/01/02 04:40:02 PM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_NotifyFlg_true pls_integer := 1;

Procedure ProcAudit is
begin
   DB_ProcAuditLogI_Ah1('PR_BATCH_LOGd_notifyFlg',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
   ||','||'RC1'
   ||','||p_TimeStamp);
End ProcAudit;

BEGIN
   p_ReturnCd := 0;

   Delete From PR_BATCH_LOG
   Where UpdateDate < = p_TimeStamp
   and   NotifyFlg  = v_NotifyFlg_true;

   ProcAudit;

   Exception
       When NO_DATA_FOUND Then ProcAudit;
       When Others Then ProcAudit;
       Raise_application_error (-20999,'20001 PR_BATCH_LOGd_notifyFlg:' || SQLCODE|| ' '||SQLERRM);
       p_ReturnCd := 1;
End PR_BATCH_LOGd_notifyFlg;
/



-- PR_BATCH_LOGI_AH1.sql

CREATE OR REPLACE PROCEDURE PR_BATCH_LOGI_AH1
(
    p_ReturnCd out integer,
    p_DebugFlg in pls_integer
    ,p_ALERTID in NUMBER
    ,p_TS in Timestamp
) as
/*
================================================================================
Proc Name:      PR_BATCH_LOGi
Create Date:    03/01/02 04:40:01 PM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_GET_DATE(); -- Performance related variables
v_pk_value number := 0;
v_ProcName varchar2(30) := 'PR_BATCH_LOGI_AH1';
v_NOTIFYFLG_False pls_integer := 0;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1(v_ProcName,v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||p_ALERTID
    ||','||p_TS
    );
End ProcAudit;
--
Procedure Insert_batch_log is
begin
    Insert into PR_BATCH_LOG(
        BATCHID
        ,   ALERTID
        ,   BATCHCNT
        ,   BEGINTS
        ,   NOTIFYFLG
    ) Values (
        PR_BATCH_LOG_SEQ.nextval
        ,   p_ALERTID
        ,   1
        ,   p_TS
        ,   v_NOTIFYFLG_False
    );
    EXCEPTION
        When NO_DATA_FOUND Then Null;
        When Others Then
            ProcAudit;
            Raise_application_error (-20999,'20000 '||v_ProcName||' -- Insert_batch_log ' || SQLCODE|| ' '||SQLERRM);
end Insert_batch_log;

--------------------------------------
--          M A I N
--------------------------------------
BEGIN
    Update PR_BATCH_LOG   set   BATCHCNT = BATCHCNT + 1
        ,     ENDTS    = p_TS
        , UpdateDate = FN_GET_DATE()
    where
        ALERTID   = p_ALERTID
        and   NOTIFYFLG = v_NOTIFYFLG_False;
    v_rowcnt := sql%rowcount;
    If v_rowcnt = 0 then
        Insert_batch_log;
    end if;
    ProcAudit;
    EXCEPTION
        When NO_DATA_FOUND Then Null;
        When Others Then
            ProcAudit;
            Raise_application_error (-20999,'20001 '||v_ProcName||' '|| SQLCODE|| ' '||SQLERRM);
            p_ReturnCd := 1;
END;
/


-- PR_BATCH_LOGS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_BATCH_LOGS_AH1
(
    p_returncd out pls_integer
    , p_debugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_EndMinute in pls_integer -- number of minutes since Dec'70
) As
/*
================================================================================
Proc Name:      PR_BATCH_LOGS_AH1
Create Date:    03/01/02 04:40:02 PM
Purpose    :    This procedure retrieves rows fron pr_batch_log table based upon
                the spec's defined in pr_notification table. If a match is found
                the alert id along with other data is returned.
                The notification Flag for selected data is turned on, so this data
                is not selected again by any other thread in a multi-thread processing env.
                Commit or rollback is controlled by calling Program.

IMPORTANT:      Even though this proc is named as a select proc, IT DOES PERFORM AN UPDATE.

Author          V.Goel
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt          integer    :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_NOTIFYFLG_True  pls_integer := 1;
v_NOTIFYFLG_False pls_integer := 0;
v_ProcName        varchar2(30) := 'PR_BATCH_LOGS_AH1';
-------
batch_log   batch_table := batch_table();
Type num10_table is table of number(10);
alertid_num10_table num10_table:= num10_table();
BatchId_num10_table num10_table:= num10_table();
BatchCNT_num10_table num10_table:= num10_table();
Type dateTable is table of date;
BeginTS_table dateTable := dateTable();
EndTS_table   dateTable := dateTable();
v_BatchId         pr_batch_log.BatchId%Type;
i                 pls_integer:=1;
------
------
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_AH1(v_ProcName,v_beginTS, 'p_returncd'||','||p_debugFlg
    ||','||'RC1'
    ||','||p_EndMinute
    );
End ProcAudit;

-----------------------------------------------------------------
--                    M   A   I   N
-----------------------------------------------------------------

BEGIN
    begin
        update pr_batch_log
            set NOTIFYFLG  = v_NOTIFYFLG_True
            , updateDate = FN_GET_DATE()
        where
            BatchID    in
                (
                Select b.batchId
                From
                    pr_notification  n
                    Inner join pr_Batch_Log b on b.alertId = n.AlertId
                Where
                    b.NOTIFYFLG  = v_NOTIFYFLG_False
                    and    ((n.BATCHTYPE = 3 and mod(p_EndMinute, n.Frequency)=0) or  -- Time Based
                    (n.BATCHTYPE = 4 and (b.BATCHCNT >= n.BATCHCNT or mod(p_EndMinute, n.Frequency)=0)))
                    )
        returning AlertID, batchCnt, beginTS, endTS bulk collect
        into alertid_num10_table, BatchCNT_num10_table, BeginTS_table, EndTS_table;
        Exception
            When Others Then
            ProcAudit;
            Raise_application_error (-20999,'20000 PR_BATCH_LOGs_AH1 --Update ' || SQLCODE|| ' '||SQLERRM);
    end;
    begin
        For i IN 1..AlertId_num10_table.count LOOP
            batch_log.extend;
            batch_log(i) := BATCH_REC(AlertId_num10_table(i),batchCNT_num10_table(i), BeginTS_table(i), EndTS_table(i));
        End Loop;
    end;
    ---
    Begin
        Open RC1 for
            select /*+ FIRST_ROWS INDEX(a PK_ALERT) */
                y.AlertId
                ,   y.BATCHCNT
                ,   y.BEGINTS
                ,   y.ENDTS
                ,   a.PARTNERID, RTRIM(p.PartnerName) PartnerName
                ,   RTRIM(a.ALERTNAME)  ALERTNAME
                ,   a.ALERTPARTNERID
                ,   s.ModeName
                ,   RTRIM(ap.PartnerName) AlertPartnerName
                ,   a.ALERTYPE
                ,   RTRIM(o.RoutingObjCd) As PROCESSCD
                ,   a.EVENTCD, RTRIM(e.EVENTNAME) EVENTNAME, RTRIM(e.INTERNALDESC) INTERNALDESC
                ,   a.VOLUMETYPE
                ,   a.VOLUME
                ,   a.PCTDEV
                ,   a.VOLMIN
                ,   a.VOLMAX
                ,   a.FREQTYPE
                ,   RTRIM(a.FREQBEGHOUR)  FREQBEGHOUR
                ,   RTRIM(a.FREQENDHOUR)  FREQENDHOUR
                ,   RTRIM(a.FREQWEEKDAY)  FREQWEEKDAY
                ,   RTRIM(a.FREQMONTHDAY)  FREQMONTHDAY
                ,   a.ENABLEDFLG
		,   e.EVENTCODE
       		,   e.TEXTKEY
       		,   e.EVENTNAMEKEY
       		,   pra.RESBUNDLEFILENAME as EVENTNAMERESBUNDLEFILENAME
       		,   prb.RESBUNDLEFILENAME as TEXTKEYRESBUNDLEFILENAME
       		,   a.ALERTCUSTOMBUSTEXT
            	From
                Table(cast(Batch_Log as batch_table))y
                Inner join pr_alert a    on a.alertid    = y.alertid
                Left Outer Join cf_RoutingObjRef_Lnk l on l.RoutingObjRefId = a.RoutingObjRefId
                Left Outer Join cf_RoutingObject o on o.RoutingobjId = l.RoutingObjId
                Left Outer Join pr_partner p  on p.partnerid  = a.partnerid
                Left Outer Join pr_partner ap on ap.partnerid = a.AlertPartnerid
                LEFT OUTER JOIN pr_AlertNotifyMode s on s.Id = a.AlertNotifyModeId 
                Left Outer Join Lg_EventCD e  on a.EVENTCD    = e.EVENTCD
                LEFT OUTER JOIN pr_resourcebundles pra on pra.resourcebundleid = e.eventnameresourcebundleid
      		LEFT OUTER JOIN pr_resourcebundles prb on prb.resourcebundleid = e.textkeyresourcebundleid;
            Exception
                When Others Then
                    ProcAudit;
                    Raise_application_error (-20999,'20001 PR_BATCH_LOGs_AH1 --Update ' || SQLCODE|| ' '||SQLERRM);
    end;
    p_returncd := 0;
    procAudit;
    Exception
        When Others Then
            ProcAudit;
            Raise_application_error (-20999,'20002 PR_BATCH_LOGs_AH1  ' || SQLCODE|| ' '||SQLERRM);
End;
/


-- PR_BUSIDENTIFIERD.sql

CREATE OR REPLACE Procedure PR_BUSIDENTIFIERD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_BUSIDENTIFIERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_BUSIDENTIFIERD
	Create Date:     2003-09-26 08:24:10
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDENTIFIERD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSIDENTIFIERID=>'||p_BUSIDENTIFIERID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_BUSIDENTIFIER
	Where
		BUSIDENTIFIERID = p_BUSIDENTIFIERID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_BUSIDENTIFIERD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_BUSIDENTIFIERD;
/


-- PR_BUSIDENTIFIERD_AH1.sql

CREATE OR REPLACE Procedure PR_BUSIDENTIFIERD_AH1

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_BUSIDENTIFIERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_BUSIDENTIFIERD_AH1
	Create Date:     2007-04-10 08:24:10
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-04-10 SYSTEM        6.1.1  Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDENTIFIERD_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSIDENTIFIERID=>'||p_BUSIDENTIFIERID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	Delete From PR_ROUTINGATTR_LNK a
	Where EXISTS ( SELECT 'x' from CF_ROUTINGATTR b, PR_BUSIDENTIFIER c
	      WHERE 	b.ATTRIBUTEKEY  = 'ASBusinessId'
		  AND 	    a.ROUTINGATTRID = b.ROUTINGATTRID
		  AND       c.BUSIDENTIFIERID = p_BUSIDENTIFIERID
		  AND       a.ATTRVALUE	= c.BUSINESSID)	
	;

	Delete From PR_BUSIDENTIFIER
	Where
		BUSIDENTIFIERID = p_BUSIDENTIFIERID
	;
	
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_BUSIDENTIFIERD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_BUSIDENTIFIERD_AH1;
/


-- PR_BUSIDENTIFIERI.sql

CREATE OR REPLACE Procedure PR_BUSIDENTIFIERI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_BUSINESSID in VARCHAR2
	,p_BUSINESSIDTYPE in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_BUSIDENTIFIERI
	Create Date:     2003-09-26 00:01:35
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDENTIFIERI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_BUSINESSID=>'||p_BUSINESSID||CHR(10)
		||', p_BUSINESSIDTYPE=>'||p_BUSINESSIDTYPE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_BUSIDENTIFIER(
		BUSIDENTIFIERID
		,PARTNERID
		,BUSINESSID
		,BUSINESSIDTYPE
	) Values (
		PR_BUSIDENTIFIER_SEQ.nextval
		,p_PARTNERID
		,p_BUSINESSID
		,p_BUSINESSIDTYPE
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_BUSIDENTIFIER_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
--			RAISE_APPLICATION_ERROR (-20999,'20000 PR_BUSIDENTIFIERI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
--			RAISE_APPLICATION_ERROR (-20999,'20001 PR_BUSIDENTIFIERI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_BUSIDENTIFIERI' || SQLCODE|| ' '||SQLERRM);
END PR_BUSIDENTIFIERI;
/


-- PR_BUSIDENTIFIERS_BUSINESSID.sql

CREATE OR REPLACE PROCEDURE PR_BUSIDENTIFIERS_BUSINESSID
	(
	  p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER	
	, RC1  OUT  DB_REF.RCT	
	, p_BUSINESSID IN  VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_BUSIDENTIFIERS_BUSINESSID
	Create Date:     2006-08-23  00:06:46
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2006-08-23 Sujit Nair       1.0 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/



v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDENTIFIERS_BUSINESSID';
v_ExceptionFlg    integer Default 0;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	IF p_BUSINESSID IS NOT NULL THEN
		OPEN RC1 FOR
			SELECT
				BUSIDENTIFIERID
				,PARTNERID
				,BUSINESSID
				,BUSINESSIDTYPE
				,ROWTS
			FROM
				PR_BUSIDENTIFIER
			WHERE
				BUSINESSID = p_BUSINESSID
			;
	ELSE
		OPEN RC1 FOR
			SELECT
				BUSIDENTIFIERID
				,PARTNERID
				,BUSINESSID
				,BUSINESSIDTYPE
				,ROWTS
			FROM
				PR_BUSIDENTIFIER
			;
			
	END IF;

	p_ReturnCd := 0;
	
	--DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, p_DebugFlg, v_ExceptionFlg);
	
Exception

    WHEN OTHERS  THEN
        p_ReturnCd := 1;
        Raise_application_error (-20999, '20000 PR_BUSIDENTIFIERS_BUSINESSID' || SQLERRM);
	
End PR_BUSIDENTIFIERS_BUSINESSID;
/

-- PR_BUSIDENTIFIERS_HUBOWNER.sql

CREATE OR REPLACE PROCEDURE PR_BUSIDENTIFIERS_HUBOWNER
(p_ReturnCd out integer
, p_DebugFlg in pls_integer
, RC1 out db_ref.RCT
) As
/*
================================================================================
Proc Name:      PR_BUSIDENTIFIERS_HubOwner
Create Date:    2002-05-09 15:26:59
Author          V Goel
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE        NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-04-07  S.Narain      4.2.2  BGTRK00041845
    2007-08-25  BHARANI J     6.1.1  Feature 101102
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt   integer         default 0;
v_BeginTS  timestamp default FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
begin
DB_ProcAuditLogI_Ah1('PR_BUSIDENTIFIERS_HubOwner',v_beginTS,
'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
);
End ProcAudit;
BEGIN
    Open RC1 for
    select b.businessid
    from
        pr_partner p
        Inner join pr_partnerType pt on p.partnerTypeID = pt.partnerTypeID
        Inner join pr_BusIdentifier b on p.partnerID = b.partnerID
        Inner join PR_COMMUNITYMANAGERDEFAULT c on p.partnerid=c.partnerid
    where
        pt.partnerTypeName = 'HubOwner'
    ;
   p_ReturnCd := 0;
   procAudit;
End PR_BUSIDENTIFIERS_HubOwner;
/


-- PR_BUSIDENTIFIERS_PARTNERID.sql

CREATE OR REPLACE PROCEDURE         PR_BUSIDENTIFIERS_PARTNERID
(p_ReturnCd out integer
, p_DebugFlg in pls_integer
, RC1 out db_ref.RCT
, p_Locale in Varchar2
, p_PartnerID in NUMBER
) As
/*
================================================================================
Proc Name:      PR_BUSIDENTIFIERS_partnerid
Create Date:    2002-05-09 15:26:59
Author:         V Goel

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-03-05 S.Priani      4.2.2  Removed NLS
    2007-01-02 jriordan      6.1.0  102018
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt   integer         :=0;
v_BeginTS  timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
begin
DB_ProcAuditLogI_Ah1('PR_BUSIDENTIFIERS_partnerid',v_beginTS,
'p_ReturnCd'||','||p_DebugFlg
||','||p_Locale
||','||'RC1'
||','||p_PartnerID
);
End ProcAudit;
BEGIN
   Open RC1 for select
   --        b.BUSIDENTIFIERID  BUSIDENTIFIERID,
            RTRIM(b.PARTNERID)       PARTNERID
   ,       RTRIM(BUSINESSID)  "BUSINESSID"
   ,       b.BusinessIdType
   ,       RTRIM(Resourcekey) RESOURCEKEY
   ,       b.BUSIDENTIFIERID "BusinessIdentifierId"
   ,       RTRIM(bt.BUSINESSIDDESCKEY) BUSINESSIDDESCKEY
   ,       RTRIM(bt.VALIDATIONRULE) "VALIDATIONRULE"
   From PR_BUSIDENTIFIER b
   Inner JOin PR_BusinessIdType bt on bt.BUSINESSIDTYPE = b.BUSINESSIDTYPE
   Where PARTNERID = p_PARTNERID
   Order by BUSINESSID ASC
   ;
   p_ReturnCd := 0;
   procAudit;
End PR_BUSIDENTIFIERS_PARTNERID;
/


-- PR_BUSIDENTIFIER_EMAILS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_BUSIDENTIFIER_EMAILS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_EMAILADDRESS IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_BUSIDENTIFIER_EMAILS_AH1
	Create Date:     2006-09-01 18:13:02
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDENTIFIER_EMAILS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_EMAILADDRESS=>'||p_EMAILADDRESS||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
	SELECT	e.BUSINESSEMAILID
		,b.BUSIDENTIFIERID
		,b.BUSINESSID
		,b.PARTNERID
		,RTRIM(EMAILADDRESS) EMAILADDRESS
	FROM	PR_EMAIL e
	INNER JOIN PR_BUSIDENTIFIER_EMAIL_LNK l ON l.BUSINESSEMAILID = e.BUSINESSEMAILID
	INNER JOIN PR_BUSIDENTIFIER b ON b.BUSIDENTIFIERID = l.BUSIDENTIFIERID
	WHERE	EMAILADDRESS = LOWER(p_EMAILADDRESS)
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_BUSIDENTIFIER_EMAILS_AH1;
/


-- PR_BUSIDENTIFIER_EMAIL_LNKD.sql

Create or Replace Procedure PR_BUSIDENTIFIER_EMAIL_LNKD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_BUSIDENTIFIERID in NUMBER
	,p_BUSINESSEMAILID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_EMAILD
	Create Date:     2006-09-01 18:13:01
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDENTIFIER_EMAIL_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSIDENTIFIERID=>'||p_BUSIDENTIFIERID||CHR(10)
		||', p_BUSINESSEMAILID=>'||p_BUSINESSEMAILID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_BUSIDENTIFIER_EMAIL_LNK
	Where	BUSINESSEMAILID = p_BUSINESSEMAILID
	AND	BUSIDENTIFIERID = p_BUSIDENTIFIERID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
Exception
	When ZeroDelete Then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
End PR_BUSIDENTIFIER_EMAIL_LNKD;
/


-- PR_BUSIDENTIFIER_EMAIL_LNKI.sql

CREATE OR REPLACE PROCEDURE PR_BUSIDENTIFIER_EMAIL_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_BUSIDENTIFIERID in NUMBER
	,p_BUSINESSEMAILID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_BUSIDENTIFIER_EMAIL_LNKI
	Create Date:     2006-09-01 18:13:01
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDENTIFIER_EMAIL_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSIDENTIFIERID=>'||p_BUSIDENTIFIERID||CHR(10)
		||', p_BUSINESSEMAILID=>'||p_BUSINESSEMAILID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_BUSIDENTIFIER_EMAIL_LNK(
		BUSIDENTIFIERID
		,BUSINESSEMAILID
	) Values (
		p_BUSIDENTIFIERID
		,p_BUSINESSEMAILID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		p_BUSIDENTIFIERID BUSID, p_BUSINESSEMAILID EMAILID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
END PR_BUSIDENTIFIER_EMAIL_LNKI;
/


-- PR_BUSIDENTIFIER_EMAIL_LNKS.sql

CREATE OR REPLACE PROCEDURE PR_BUSIDENTIFIER_EMAIL_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_BUSIDENTIFIERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_BUSIDENTIFIER_EMAIL_LNKS
	Create Date:     2006-09-01 18:13:02
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDENTIFIER_EMAIL_LNKS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSIDENTIFIERID=>'||p_BUSIDENTIFIERID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_BUSIDENTIFIERID IS NOT NULL THEN
	OPEN RC1 FOR 
	SELECT
		BUSINESSEMAILID
		,BUSIDENTIFIERID
		,ROWTS
		,CREATEDATE
	FROM 
		PR_BUSIDENTIFIER_EMAIL_LNK
	Where	BUSIDENTIFIERID = p_BUSIDENTIFIERID
	;
ELSE
	OPEN RC1 FOR 
	SELECT
		BUSINESSEMAILID
		,BUSIDENTIFIERID
		,ROWTS
		,CREATEDATE
	FROM 
		PR_BUSIDENTIFIER_EMAIL_LNK
	;
END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_BUSIDENTIFIER_EMAIL_LNKS;
/


-- PR_BUSIDENTIFIER_EMAIL_LNKU.sql

CREATE OR REPLACE PROCEDURE PR_BUSIDENTIFIER_EMAIL_LNKU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_BUSINESSEMAILID in NUMBER
	,p_BUSIDENTIFIERID in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_BUSIDENTIFIER_EMAIL_LNKU
	Create Date:     2006-09-01 18:13:02
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDENTIFIER_EMAIL_LNKU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSINESSEMAILID=>'||p_BUSINESSEMAILID||CHR(10)
		||', p_BUSIDENTIFIERID=>'||p_BUSIDENTIFIERID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_BUSIDENTIFIER_EMAIL_LNK
	SET      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where BUSINESSEMAILID = p_BUSINESSEMAILID
	AND	BUSIDENTIFIERID=p_BUSIDENTIFIERID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN ZeroUpdate then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
	WHEN OTHERS then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
End PR_BUSIDENTIFIER_EMAIL_LNKU;
/


-- PR_BUSIDROUTINGD_PARTNERID.sql

CREATE OR REPLACE PROCEDURE         PR_BUSIDROUTINGD_Partnerid
(p_ReturnCd out integer
, p_DebugFlg in pls_integer
,p_partnerid in NUMBER
) As
/*
================================================================================
Proc Name:        PR_BUSIDROUTINGD_Partnerid
Create Date:     2002-05-31 11:24:29
Author:            SYSTEM
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2005-10-04 S.Priani	6.0.0.2	BGTRK00049412
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_BUSIDROUTINGD_Partnerid',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_partnerid
    );
End ProcAudit;
BEGIN
    Delete
    From PR_BUSIDROUTING B
    where exists (select BUSIDROUTINGID
    From PR_BUSIDROUTING B
    Inner Join pr_busidentifier bi  on bi.BUSIDENTIFIERID = B.BUSIDENTIFIERID
    Inner Join pr_businessIdType bt on bi.BusinessIDType = bt.BusinessIDType
----    inner join pr_partner p         on bi.partnerid = p.partnerid
    Where PartnerID = p_PartnerID)
    ;
    v_rowcnt := SQL%ROWCOUNT;
    ProcAudit;
    p_ReturnCd := 0;
    Exception
        When NO_DATA_FOUND Then ProcAudit;
        Raise_application_error (-20999,'20002 PR_BUSIDROUTINGD_Partnerid: No Data Found');
        When Others Then ProcAudit;
        Raise_application_error (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
        p_ReturnCd := 1;
End PR_BUSIDROUTINGD_Partnerid;
/


-- PR_BUSIDROUTINGI.sql

CREATE OR REPLACE Procedure PR_BUSIDROUTINGI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_BUSIDENTIFIERID in NUMBER
	,p_RN0A1FLG in NUMBER
	,p_PARTNEREVENTFLG in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_BUSIDROUTINGI
	Create Date:     2003-09-26 00:01:36
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSIDROUTINGI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSIDENTIFIERID=>'||p_BUSIDENTIFIERID||CHR(10)
		||', p_RN0A1FLG=>'||p_RN0A1FLG||CHR(10)
		||', p_PARTNEREVENTFLG=>'||p_PARTNEREVENTFLG||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_BUSIDROUTING(
		BUSIDROUTINGID
		,BUSIDENTIFIERID
		,RN0A1FLG
		,PARTNEREVENTFLG
	) Values (
		PR_BUSIDROUTING_SEQ.nextval
		,p_BUSIDENTIFIERID
		,p_RN0A1FLG
		,p_PARTNEREVENTFLG
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_BUSIDROUTING_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_BUSIDROUTINGI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_BUSIDROUTINGI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_BUSIDROUTINGI' || SQLCODE|| ' '||SQLERRM);
END PR_BUSIDROUTINGI;
/


-- PR_BUSIDROUTINGS_BUSINESSID.sql

CREATE OR REPLACE PROCEDURE         PR_BUSIDROUTINGS_BusinessID
(p_ReturnCd out integer, p_DebugFlg in pls_integer, RC1 out db_ref.RCT
, p_BusinessID in varchar2
) As
/*
================================================================================
Proc Name:        PR_BUSIDROUTINGS_BusinessID
Create Date:     2002-05-31 10:08:18
Author:            V Goel
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_BUSIDROUTINGS_BusinessID',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_BusinessID
    );
End ProcAudit;
BEGIN
    Open RC1 for select
        B.BUSIDROUTINGID
    ,    nvl(B.RN0A1FLG, 0) RN0A1FLG
    ,    nvl(B.PARTNEREVENTFLG, 0) PARTNEREVENTFLG
    From PR_BUSIDROUTING B
    Inner Join pr_busidentifier bi  on bi.BUSIDENTIFIERID = B.BUSIDENTIFIERID
    Where bi.BusinessID = p_BusinessID
    ;
    p_ReturnCd := 0;
    procAudit;
End PR_BUSIDROUTINGS_BusinessID;
/


-- PR_BUSIDROUTINGS_PARTNERID.sql

CREATE OR REPLACE PROCEDURE         PR_BUSIDROUTINGS_PARTNERID
(p_ReturnCd out integer
, p_DebugFlg in pls_integer
, RC1 out db_ref.RCT
, p_Locale in Varchar2
, p_PartnerID in NUMBER
) As
/*
================================================================================
Proc Name:        PR_BUSIDROUTINGS_Partnerid
Create Date:     2002-05-31 10:08:18
Author:            SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

	---------- ------------	------- --------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	------- --------------------------------------------
	2003-03-04 S.Narain	4.2.2	Removed NLS
	2005-10-04 S.Priani	6.0.0.2	BGTRK00049412
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt   integer     :=0;
v_BeginTS  timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
begin
DB_ProcAuditLogI_Ah1('PR_BUSIDROUTINGS_Partnerid',v_beginTS,
'p_ReturnCd'
||','||p_DebugFlg
||','||p_Locale
||','||'RC1'
||','||p_PartnerID
);
End ProcAudit;
BEGIN
   Open RC1 for select
        B.BUSIDROUTINGID
   ,    B.BUSIDENTIFIERID
   ,    B.RN0A1FLG
   ,    B.PARTNEREVENTFLG
   ,    B.ROWTS
   ,    B.CREATEDATE
   ,   bi.BusinessID
   ,   bi.BusinessIDType
   ,   bt.Resourcekey
   ,   bt.BusinessIDDescKey
   ,   bt.Description
   From PR_BUSIDROUTING B
   Inner Join pr_busidentifier bi  on bi.BUSIDENTIFIERID = B.BUSIDENTIFIERID
   Inner Join pr_businessIdType bt on bi.BusinessIDType = bt.BusinessIDType
---   inner join pr_partner p         on bi.partnerid = p.partnerid
   Where PartnerID = p_PartnerID
   ;
   p_ReturnCd := 0;
   procAudit;
End PR_BUSIDROUTINGS_PARTNERID;
/


-- PR_BUSINESSIDTYPES_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_BUSINESSIDTYPES_AH1
(
   p_ReturnCd out integer
   , p_DebugFlg in pls_integer
   , RC1 out db_ref.RCT
   , p_Locale in Varchar2
   , p_BusinessIdName In Varchar2
) As
/*
================================================================================
Proc Name:        PR_BUSINESSIDTYPES_AH1
Create Date:     2002-07-12 16:24:43
Author:            SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2  Removed NLS
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
begin
--   If p_DebugFlg =1 Then
   DB_ProcAuditLogI_Ah1('PR_BUSINESSIDTYPES_AH1',v_beginTS,
   'p_ReturnCd'
   ||','||p_DebugFlg
   ||','||p_Locale
   ||','||'RC1'
   ||','||p_BusinessIdName
   );
 --  End if ;
End ProcAudit;
BEGIN
   OPEN RC1 FOR SELECT
        BUSINESSIDTYPE
   ,    RTRIM(Resourcekey)  Resourcekey
   ,    RTRIM(BUSINESSIDDESCKey)  BUSINESSIDDESCKey
   ,    RTRIM(VALIDATIONRULE)  VALIDATIONRULE
   ,    Description
   FROM PR_BUSINESSIDTYPE
   WHERE BusinessIdName = p_BusinessIdName
   ;
   p_ReturnCd := 0;
   procAudit;
End PR_BUSINESSIDTYPES_AH1;
/


-- PR_BUSINESSIDTYPE_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE PR_BUSINESSIDTYPE_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_BUSINESSIDTYPE IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_BUSINESSIDTYPE_LOC_VWS
	Create Date:     2003-09-26 00:10:35
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-03-05 S.Priani      4.2.2  Removed NLS

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_BUSINESSIDTYPE_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSINESSIDTYPE=>'||p_BUSINESSIDTYPE||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_BUSINESSIDTYPE IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		BUSINESSIDTYPE
		,RTRIM(BUSINESSIDNAME) BUSINESSIDNAME
		,ROWTS
		,CREATEDATE
		,RTRIM(VALIDATIONRULE) VALIDATIONRULE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(BUSINESSIDDESCKEY) BUSINESSIDDESCKEY
		,RTRIM(BUSIDDESCRIPTION) BUSIDDESCRIPTION
	FROM
		PR_BUSINESSIDTYPE
	Where
		BUSINESSIDTYPE = p_BUSINESSIDTYPE
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		BUSINESSIDTYPE
		,RTRIM(BUSINESSIDNAME) BUSINESSIDNAME
		,ROWTS
		,CREATEDATE
		,RTRIM(VALIDATIONRULE) VALIDATIONRULE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(BUSINESSIDDESCKEY) BUSINESSIDDESCKEY
		,RTRIM(BUSIDDESCRIPTION) BUSIDDESCRIPTION
	FROM
		PR_BUSINESSIDTYPE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_BUSINESSIDTYPE_LOC_VWS;
/


-- PR_CAPABILITYI.sql

CREATE OR REPLACE Procedure PR_CAPABILITYI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_ROUTINGOBJREFID in NUMBER
	,p_ROLETYPECD in NUMBER
	,p_DEFAULTFLG in NUMBER
	,p_ACTIVESTATUSCD in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CAPABILITYI
	Create Date:     2003-09-26 00:01:38
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2008-09-12 Amit Kumar Vijay 6.2 Defect 47980, 106065 and 106066
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CAPABILITYI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_ROUTINGOBJREFID=>'||p_ROUTINGOBJREFID||CHR(10)
		||', p_ROLETYPECD=>'||p_ROLETYPECD||CHR(10)
		||', p_DEFAULTFLG=>'||p_DEFAULTFLG||CHR(10)
		||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_CAPABILITY(
		CAPABILITYID
		,PARTNERID
		,ROUTINGOBJREFID
		,ROLETYPECD
		,DEFAULTFLG
		,ACTIVESTATUSCD
	) Values (
		PR_CAPABILITY_SEQ.nextval
		,p_PARTNERID
		,p_ROUTINGOBJREFID
		,p_ROLETYPECD
		,p_DEFAULTFLG
		,CASE WHEN p_ACTIVESTATUSCD=1 then 3 else 2 END
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_CAPABILITY_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_CAPABILITYI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_CAPABILITYI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CAPABILITYI' || SQLCODE|| ' '||SQLERRM);
END PR_CAPABILITYI;
/


-- PR_CAPABILITYS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CapabilityS_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
     , RC1  out  db_ref.RCT
     , p_Locale in Varchar2
    , p_PartnerId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_CapabilityS_AH1
    Create Date:     2003-04-22 10:07:31
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns a list of root capabilities by a given partner
    2003-05-02 J.Chen        3.5    Replaced RoutingObjId with RoutingObjRefId.
    2003-05-30 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12 L.Ling        3.5    Changed view from CF_RoutingObjType_VW to CF_RoutingObjType_LOC_VW.
    2003-06-17 J.Chen        3.5    Added RTRIM on RoutingObjName/Cd/ver
    2003-09-22 M.Mount       4.2.1  Removed Commit
    2004-03-03 S.Priani	     4.2.2  Removed NLS
    2008-09-12 Amit Kumar 	 6.2	Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CapabilityS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||','||p_Locale||CHR(10)
        ||', p_PartnerId=>'||p_PartnerId||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
         c.CAPABILITYID
         ,c.ROUTINGOBJRefID
         ,RTRIM(v.RESOURCEKEY) RESOURCEKEY
         ,RTRIM(b.ROUTINGOBJNAME) As RoutingObjName
         ,RTRIM(b.ROUTINGOBJCD) As RoutingObjCd
         ,RTRIM(b.ROUTINGOBJVER) As RoutingObjVer
         ,p.PARTNERID
         ,p.PartnerName
         ,c.ROLETYPECD
         ,c.DEFAULTFLG
         ,CASE WHEN c.ActiveStatusCd=3 THEN 1 ELSE 0 END ActiveStatusCd
         ,v.RoutingObjTypeKey
         ,c.RowTS
         ,v.Description
    From
         PR_CAPABILITY c
         Inner Join CF_RoutingObjRef_Lnk l on l.RoutingObjRefId = c.RoutingObjRefId
         Inner Join CF_RoutingObject b On b.RoutingObjId = l.RoutingObjId
         Inner Join CF_RoutingObjType v On b.RoutingObjTypeId = v.RoutingObjTypeId
         Inner Join PR_Partner p       On p.PartnerId = c.PartnerId
    Where
         c.PartnerId = p_PartnerId
    And  l.RoutingObjParentRefId Is Null
    ORDER BY b.ROUTINGOBJCD;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION
         When OTHERS Then
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_CapabilityS_AH1' || SQLCODE|| ' '||SQLERRM);
 --           Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --           Commit;
End PR_CapabilityS_AH1;
/


-- PR_CAPABILITYS_AH2.sql

CREATE OR REPLACE PROCEDURE         PR_CapabilityS_AH2
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in Varchar2
    , p_CapabilityId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_CapabilityS_AH2
    Create Date:     2003-04-22 10:07:31
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns capability info by a given capabilityId.
    2003-05-02 J.Chen        3.5    Replaced ParentCapabilityId and RoutingObjId with
                                    RoutingObjRefId.
    2003-05-30 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12 L.Ling        3.5    Changed view from CF_RoutingObjType_VW to CF_RoutingObjType_LOC_VW.
    2003-06-17 J.Chen        3.5    Added RTRIM on RoutingObjName/cd/ver
    2003-09-22 M.Mount       4.2.1  Removed commit
    2004-03-03 S.Priani      4.2.2  Removed NLS
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CapabilityS_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||','||p_Locale||CHR(10)
        ||', p_CapabilityId=>'||p_CapabilityId||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
         c.CAPABILITYID
         ,c.RoutingObjRefId
         ,RTRIM(v.RESOURCEKEY) RESOURCEKEY
         ,RTRIM(b.ROUTINGOBJNAME) As RoutingObjName
         ,RTRIM(b.ROUTINGOBJCD) As RoutingObjCd
         ,RTRIM(b.ROUTINGOBJVER) As RoutingObjVer
         ,p.PARTNERID
         ,p.PartnerName
         ,c.ROLETYPECD
         ,c.DEFAULTFLG
         ,CASE WHEN c.ActiveStatusCd=3 THEN 1 ELSE 0 END ActiveStatusCd
         ,v.RoutingObjTypeKey
         ,c.RowTS
         ,v.Description
    From
         PR_CAPABILITY c
         Inner Join CF_RoutingObjRef_Lnk l On l.RoutingObjRefId = c.RoutingObjRefId
         Inner Join CF_RoutingObject b On b.RoutingObjId = l.RoutingObjId
         Inner Join CF_RoutingObjType v On b.RoutingObjTypeId = v.RoutingObjTypeId
         Inner Join PR_Partner p       On p.PartnerId = c.PartnerId
    Where
         c.CapabilityId = p_CapabilityId
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION
         When OTHERS Then
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_CapabilityS_AH2' || SQLCODE|| ' '||SQLERRM);
   --         Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   --         Commit;
End PR_CapabilityS_AH2;
/


-- PR_CAPABILITYS_AH3.sql

CREATE OR REPLACE PROCEDURE         PR_CapabilityS_AH3
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in Varchar2
    , p_CapabilityId in NUMBER
    )
AS
/*
    ================================================================================
    Proc Name:       PR_CapabilityS_AH3
    Create Date:     2003-04-22 10:07:31
    Author:          J.Chen
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns a list of direct children for given capability.
    2003-05-02 J.Chen        3.5    Repalced RoutingObjId with RoutingObjRefId
    2003-05-08 J.Chen        3.5    Added PartnerId to inner join on pr_capability.
    2003-05-30 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12 L.Ling        3.5    Changed view from CF_RoutingObjType_VW to CF_RoutingObjType_LOC_VW.
    2003-06-17 J.Chen        3.5    Added RTRIM on RoutingObjName/cd/ver
    2003-09-22 M.Mount       4.2.1  Removed commit.
    2004-03-03 S.Priani      4.2.2  Removed NLS
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CapabilityS_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||','||p_Locale||CHR(10)
        ||', p_CapabilityId=>'||p_CapabilityId||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
         T.CAPABILITYID
         ,T.RoutingObjRefId
         ,b.ROUTINGOBJID
         ,RTRIM(v.RESOURCEKEY) RESOURCEKEY
         ,RTRIM(b.ROUTINGOBJNAME) As RoutingObjName
         ,RTRIM(b.ROUTINGOBJCD) As RoutingObjCd
         ,RTRIM(b.ROUTINGOBJVER) As RoutingObjVer
         ,p.PARTNERID
         ,p.PartnerName
         ,T.ROLETYPECD
         ,T.DEFAULTFLG
         ,T.ActiveStatusCd
         ,v.RoutingObjTypeKey
         ,T.RowTS
    From
       (
        Select
              cc.CapabilityId
              ,cc.RoutingObjRefId
              ,l.RoutingObjId
              ,cc.PartnerId
              ,cc.RoleTypeCd
              ,cc.DefaultFlg
              ,CASE WHEN cc.ActiveStatusCd=3 THEN 1 ELSE 0 END ActiveStatusCd
              ,cc.RowTS
        From
              PR_Capability pc
        Inner Join CF_RoutingObjRef_Lnk l On l.RoutingObjParentRefId = pc.RoutingObjRefId
        Inner Join PR_Capability cc On cc.RoutingObjRefId = l.RoutingObjRefId
                                   And cc.PartnerId = pc.PartnerId
        Where pc.CapabilityId = p_CapabilityId
        And   l.RoutingObjParentRefId Is Not Null
        ) T
        Inner Join CF_RoutingObject b On b.RoutingObjId = T.RoutingObjId
        Inner Join CF_RoutingObjType v On v.RoutingObjTypeId = b.RoutingObjTypeId
        Inner Join PR_Partner p       On p.PartnerId = T.PartnerId
	;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
         When OTHERS Then
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_CapabilityS_AH3' || SQLCODE|| ' '||SQLERRM);
   --         Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CapabilityS_AH3;
/


-- PR_CAPABILITYS_AH4.sql

CREATE OR REPLACE PROCEDURE PR_CapabilityS_AH4
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in Varchar2
    , p_PartnerId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_CapabilityS_AH4
    Create Date:     2003-05-02 10:07:31
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-02 J.Chen        3.5    Returns all capabilities by a given partner
    2003-05-30 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12 L.Ling        3.5    Changed view from CF_RoutingObjType_VW to CF_RoutingObjType_LOC_VW.
    2003-06-17 J.Chen        3.5    Added RTRIM on RoutingObjName/cd/ver
    2003-09-22 M.Mount       4.2.1  Removed commit and rollback
    2004-03-03 S.Priani      4.2.2  Removed NLS
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CapabilityS_AH4';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||','||p_Locale||CHR(10)
        ||', p_PartnerId=>'||p_PartnerId||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
         c.CAPABILITYID
         ,c.ROUTINGOBJRefID
         ,RTRIM(v.RESOURCEKEY) RESOURCEKEY
         ,v.Description
         ,RTRIM(b.ROUTINGOBJNAME) As RoutingObjName
         ,RTRIM(b.ROUTINGOBJCD) As RoutingObjCd
         ,RTRIM(b.ROUTINGOBJVER) As RoutingObjVer
         ,p.PARTNERID
         ,p.PartnerName
         ,c.ROLETYPECD
         ,c.DEFAULTFLG
         ,CASE WHEN c.ActiveStatusCd=3 THEN 1 ELSE 0 END ActiveStatusCd
         ,v.RoutingObjTypeKey
         ,c.RowTS
    From
         PR_CAPABILITY c
         Inner Join CF_RoutingObjRef_Lnk l on l.RoutingObjRefId = c.RoutingObjRefId
         Inner Join CF_RoutingObject b On b.RoutingObjId = l.RoutingObjId
         Inner Join CF_RoutingObjType v On b.RoutingObjTypeId = v.RoutingObjTypeId
         Inner Join PR_Partner p       On p.PartnerId = c.PartnerId
    Where
         c.PartnerId = p_PartnerId
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --   COMMIT;

    EXCEPTION
         When OTHERS Then
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_CapabilityS_AH4' || SQLCODE|| ' '||SQLERRM);
 --           Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --          Commit;
End PR_CapabilityS_AH4;
/


-- PR_CAPABILITYS_AH5.sql

CREATE OR REPLACE PROCEDURE PR_CapabilityS_AH5
    (
    p_ReturnCd            out integer
    ,p_DebugFlg           in pls_integer
    ,p_PartnerId          in number
    ,p_RoutingObjRefId    in number
    ,p_RoutingCapability_Tbl out PR_RoutingCapability_Tbl
    ,p_StatusOption       in number  -- 1 = active capability, 0 = ignore status
    )
is

/*
    ================================================================================
    Proc Name:      PR_CapabilityS_AH5
    Create Date:    2003-05-07
    Author:         B.Moffat
    Description:    Retrieve Partner Capability data into
                    p_RoutingCapability_Tbl (output varrary)

                        PartnerId,
                        RoutingObjLevel,
                        RoutingObjId,
                        RoutingObjTypeKey,
                        RoutingObjCd,
                        RoutingObjVer,
                        RoutingObjRefId,
                        CapabilityId


                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-07 B.Moffat         3.5 Check Channel for the router.
    2003-06-02 J.Chen           3.5 Added ActiveStatusCd = 1 to exclude disabled capabilities
                                    if p_StatusOption = 1 .
    2003-09-22 M.Mount          4.2.1 Removed commit.
    2008-09-12 Amit Kumar 	6.2 Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

TYPE v_RoutingCapability_Rec is Record      -- NOTE:  This record must look exactly like PR_RoutingCapability_Rec
(
    PartnerId               number(10),
    RoutingObjLevel         number(6),
    RoutingObjId            number(10),
    RoutingObjTypeKey       varchar2(60),
    RoutingObjCd            varchar2(255),
    RoutingObjVer           varchar2(60),
    RoutingObjRefId         number(10),
    CapabilityId            number(10)
);


TYPE v_RoutingCapability_T is table of v_RoutingCapability_Rec index by pls_integer;
v_RoutingCapability_Tbl v_RoutingCapability_T;

v_RowCnt            integer :=0;
v_BeginTS           timestamp := FN_Get_Date; -- Performance related variables
v_ProcName          varchar2(30) := 'PR_CapabilityS_AH5';
v_ExceptionFlg      integer := 0;
v_ParamsTxt         varchar2(4000) :=
       ' p_DebugFlg=>'||p_DebugFlg||CHR(10)
    ||',p_ReturnCd=>'||p_ReturnCd||CHR(10)
    ||',p_PartnerId=>'||p_PartnerId||CHR(10)
    ||',p_RoutingObjRefId=>'||p_RoutingObjRefId||CHR(10)
    ||',p_RoutingCapability_Tbl => PR_RoutingCapability_Tbl'  ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------



BEGIN

    p_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();

    Select distinct
        PartnerId,
        RoutingObjLevel,
        RoutingObjId,
        RoutingObjTypeKey,
        RoutingObjCd,
        RoutingObjVer,
        RoutingObjRefId,
        CapabilityId
    BULK COLLECT INTO v_RoutingCapability_Tbl
    from
        (
        select /*+ FIRST_ROWS */
                Cfr1.PartnerId,
                Rfr1.RoutingObjLevel,
                Rfr1.RoutingObjId,
                Rfr1.RoutingObjTypeKey,
                Rfr1.RoutingObjCd,
                Rfr1.RoutingObjVer,
                Cfr1.RoutingObjRefId,
                Cfr1.CapabilityId
            from
                CF_RoutingObject04_vw Rfr1
                INNER JOIN PR_Capability Cfr1 on Cfr1.RoutingObjRefId = Rfr1.RoutingObjRefId and Cfr1.PartnerId = p_PartnerId
            where Rfr1.Activestatuscd = 1 AND
            Cfr1.ActiveStatusCd = Case when p_StatusOption = 1 then 3 else Cfr1.ActiveStatusCd End
            Start with Rfr1.RoutingObjRefId = p_RoutingObjRefId
            connect by
                Rfr1.RoutingObjRefId = PRIOR Rfr1.RoutingObjParentRefId
            UNION
            select /*+ FIRST_ROWS */
                Cfr2.PartnerId,
                Rfr2.RoutingObjLevel,
                Rfr2.RoutingObjId,
                Rfr2.RoutingObjTypeKey,
                Rfr2.RoutingObjCd,
                Rfr2.RoutingObjVer,
                Cfr2.RoutingObjRefId,
                Cfr2.CapabilityId
            from
                CF_RoutingObject04_vw Rfr2
                INNER JOIN PR_Capability Cfr2 on Cfr2.RoutingObjRefId = Rfr2.RoutingObjRefId and Cfr2.PartnerId = p_PartnerId
            where Rfr2.Activestatuscd = 1 AND
            Cfr2.ActiveStatusCd = Case when p_StatusOption = 1 then 3 else Cfr2.ActiveStatusCd End
            Start with Rfr2.RoutingObjRefId = p_RoutingObjRefId
            connect by
                PRIOR Rfr2.RoutingObjRefId = Rfr2.RoutingObjParentRefId
        ) T;

    IF p_DebugFlg = 1 THEN DBMS_OUTPUT.Put_Line('---------------------------------------------------------'); END IF;
    For i IN 1..v_RoutingCapability_Tbl.count LOOP
        p_RoutingCapability_Tbl.extend;
        p_RoutingCapability_Tbl(i) := PR_RoutingCapability_Rec
                                        (
                                        v_RoutingCapability_Tbl(i).PartnerId,
                                        v_RoutingCapability_Tbl(i).RoutingObjLevel,
                                        v_RoutingCapability_Tbl(i).RoutingObjId,
                                        v_RoutingCapability_Tbl(i).RoutingObjTypeKey,
                                        v_RoutingCapability_Tbl(i).RoutingObjCd,
                                        v_RoutingCapability_Tbl(i).RoutingObjVer,
                                        v_RoutingCapability_Tbl(i).RoutingObjRefId,
                                        v_RoutingCapability_Tbl(i).CapabilityId
                                        );
        IF p_DebugFlg = 1 THEN
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, CHR(10)||
                'PartnerId         =>'||v_RoutingCapability_Tbl(i).PartnerId||CHR(10)||
                'RoutingObjLevel   =>'||v_RoutingCapability_Tbl(i).RoutingObjLevel||CHR(10)||
                'RoutingObjId      =>'||v_RoutingCapability_Tbl(i).RoutingObjId||CHR(10)||
                'RoutingObjTypeKey =>'||v_RoutingCapability_Tbl(i).RoutingObjTypeKey||CHR(10)||
                'RoutingObjCd      =>'||v_RoutingCapability_Tbl(i).RoutingObjCd||CHR(10)||
                'RoutingObjVer     =>'||v_RoutingCapability_Tbl(i).RoutingObjVer||CHR(10)||
                'RoutingObjRefId   =>'||v_RoutingCapability_Tbl(i).RoutingObjRefId||CHR(10)||
                'CapabilityId      =>'||v_RoutingCapability_Tbl(i).CapabilityId
                , p_DebugFlg, v_ExceptionFlg);
        END IF;

    End Loop;
    IF p_DebugFlg = 1 THEN DBMS_OUTPUT.Put_Line('---------------------------------------------------------'); END IF;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --   commit;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            p_ReturnCd := 3;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND: '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   --         commit;
        WHEN OTHERS then
            RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||': '||SQLERRM);
  --          ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
--            Commit;

END;
/


-- PR_CAPABILITYS_AH6.sql

CREATE OR REPLACE PROCEDURE         PR_CapabilityS_AH6
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in Varchar2
    , p_PartnerId in Number
    , p_RoutingObjRefId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_CapabilityS_AH6
    Create Date:     2003-04-22 10:07:31
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns capability info by given capabilityId and partnerId.
    2003-05-30 L.Ling        3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-06-12 L.Ling        3.5    Changed view from CF_RoutingObjType_VW to CF_RoutingObjType_LOC_VW.
    2003-06-17 J.Chen        3.5    Added RTRIM on RoutingObjName/cd/ver
    2003-09-22 M.Mount       4.2.1  Removed commit.
    2004-03-03 S.Priani      4.2.2  Removed NLS
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CapabilityS_AH6';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||','||p_Locale||CHR(10)
        ||', p_RoutingObjRefId=>'||p_RoutingObjRefId||CHR(10)
        ||', p_PartnerId=>'||p_PartnerId||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
         c.CAPABILITYID
         ,c.RoutingObjRefId
         ,RTRIM(v.RESOURCEKEY) RESOURCEKEY
         ,v.Description
         ,RTRIM(b.ROUTINGOBJNAME) As RoutingObjName
         ,RTRIM(b.ROUTINGOBJCD) As RoutingObjCd
         ,RTRIM(b.ROUTINGOBJVER) As RoutingObjVer
         ,p.PARTNERID
         ,p.PartnerName
         ,c.ROLETYPECD
         ,c.DEFAULTFLG
         ,CASE WHEN c.ActiveStatusCd=3 THEN 1 ELSE 0 END ActiveStatusCd
         ,v.RoutingObjTypeKey
         ,c.RowTS
    From
         PR_CAPABILITY c
         Inner Join CF_RoutingObjRef_Lnk l On l.RoutingObjRefId = c.RoutingObjRefId
         Inner Join CF_RoutingObject b On b.RoutingObjId = l.RoutingObjId
         Inner Join CF_RoutingObjType v On b.RoutingObjTypeId = v.RoutingObjTypeId
         Inner Join PR_Partner p       On p.PartnerId = c.PartnerId
    Where
         c.PartnerId = p_PartnerId
    And  c.RoutingObjRefId = p_RoutingObjRefId
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --   COMMIT;

    EXCEPTION
         When OTHERS Then
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_CapabilityS_AH6' || SQLCODE|| ' '||SQLERRM);
 --           Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --         Commit;
End PR_CapabilityS_AH6;
/


-- PR_CAPABILITYS_AH7.sql

CREATE OR REPLACE PROCEDURE PR_CAPABILITYS_AH7 
(
   p_returncd          OUT      INTEGER,
   p_debugflg          IN       PLS_INTEGER,
   rc1                 OUT      db_ref.rct,
   p_routingobjrefid   IN       NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       PR_CapabilityS_AH7
    Create Date:     2007-08-20 08:24:13
    Author:          Bharani J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-09 PR_CapabilityS_AH7 1  Created
    2008-09-12 Amit Kumar 	6.2  Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_rowcnt         INTEGER         := 0;
   v_begints        TIMESTAMP       := fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   := 'PR_CapabilityS_AH7';
   v_exceptionflg   INTEGER         := 0;
   v_paramstxt      VARCHAR2 (4000)
      :=    'p_DebugFlg=>'
         || p_debugflg
         || CHR (10)
         || ', p_RoutingObjRefId=>'
         || p_routingobjrefid
         || CHR (10);
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF rc1%ISOPEN THEN
		CLOSE rc1;
	END IF;

	OPEN rc1 FOR
	SELECT 
	c.capabilityid,
	p.PARTNERNAME,
	c.routingobjrefid,
	c.roletypecd,
	c.defaultflg,
	CASE WHEN c.ActiveStatusCd=3 THEN 1 ELSE 0 END ActiveStatusCd
        FROM pr_capability c,pr_partner p
	WHERE c.partnerid = p.partnerid 
	AND c.routingobjrefid = p_routingobjrefid
	AND c.activestatuscd =3 ;

   
   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
                        
   p_returncd := 0;
   
EXCEPTION
   WHEN OTHERS
   THEN
      raise_application_error (-20999,
                                  '20000-PR_CapabilityS_AH7'
                               || SQLCODE
                               || ' '
                               || SQLERRM
                              );
      p_returncd := 1;
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            v_paramstxt,
                            p_debugflg,
                            v_exceptionflg
                           );
END pr_capabilitys_ah7;
/

-- PR_CAPABILITYU_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CapabilityU_AH1
    (
     p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_CAPABILITYID in NUMBER
    , p_ACTIVESTATUSCD in NUMBER
    , p_RoleTypeCd in number
    , p_ROWTS in NUMBER
) as

/*
    ================================================================================
    Proc Name:       PR_CapabilityU_AH1
    Create Date:     2003-04-22 10:07:31
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Update capability ActiveStatusCd
    2003-05-02 J.Chen        3.5    Added p_RoleTypeCd as input
    2003-09-23 M.Mount       4.2.1  Removed Commits.
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CAPABILITYU_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_CAPABILITYID=>'||p_CAPABILITYID||CHR(10)
        ||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
        ||', p_RoleTypeCd=>'||p_RoleTypeCd||CHR(10)
        ||', p_ROWTS=>'||p_ROWTS||CHR(10)
        ;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_RowCnt := 0;
    
    IF p_ACTIVESTATUSCD=0 
    Then
    			update PR_ROUTINGCHANNEL
    			SET ACTIVESTATUSCD = CASE WHEN ACTIVESTATUSCD in (1,3,5,7,9,11,13,15,17,19,21,23,25,27,29,31,33,35,37,39,41,43,45,47,49,51,53,55,57,59,61,63) THEN ACTIVESTATUSCD-1 ELSE ACTIVESTATUSCD END
    			,   UPDATEDATE = v_BeginTS
    			WHERE TOCAPABILITYID in (select c.capabilityid from pr_capability c
							inner join CF_ROUTINGOBJECT06_VW chld on c.ROUTINGOBJREFID=chld.routingobjrefid
						inner join pr_capability cc on cc.capabilityid=cc.capabilityid
							WHERE cc.capabilityid=p_capabilityid
						and cc.partnerid=c.partnerid
							and cc.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));
    	
    			update PR_ROUTINGCHANNEL
		    	SET ACTIVESTATUSCD = CASE WHEN ACTIVESTATUSCD in (2,3,6,7,10,11,14,15,18,19,22,23,26,27,30,31,34,35,38,39,42,43,46,47,50,51,54,55,58,59,62,63) THEN ACTIVESTATUSCD-2 ELSE ACTIVESTATUSCD END
		    	,   UPDATEDATE = v_BeginTS
			WHERE FRCAPABILITYID  in (select c.capabilityid from pr_capability c
							inner join CF_ROUTINGOBJECT06_VW chld on c.ROUTINGOBJREFID=chld.routingobjrefid
						inner join pr_capability cc on cc.capabilityid=cc.capabilityid
							WHERE cc.capabilityid=p_capabilityid
						and cc.partnerid=c.partnerid
							and cc.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));
    		Update PR_CAPABILITY
    			SET
    			    ROWTS= p_RowTS + 1
    			    ,      UPDATEDATE= v_BeginTS
    			    ,      ACTIVESTATUSCD = 2
    			    ,      RoleTypeCd = p_RoleTypeCd
    			Where
    			    CAPABILITYID = p_CAPABILITYID
    			    and ACTIVESTATUSCD in (2,3)
    			    And RowTS = p_RowTS returning RowTS into v_RowTS;
			    v_RowCnt := SQL%rowcount;
		
    
    ELSE
    
    			update PR_ROUTINGCHANNEL
			SET ACTIVESTATUSCD = CASE WHEN ACTIVESTATUSCD in  (0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30,32,34,36,38,40,42,44,46,48,50,52,54,56,58,60,62) THEN ACTIVESTATUSCD+1 ELSE ACTIVESTATUSCD END
			,   UPDATEDATE = v_BeginTS
			WHERE TOCAPABILITYID  in (select c.capabilityid from pr_capability c
							inner join CF_ROUTINGOBJECT06_VW chld on c.ROUTINGOBJREFID=chld.routingobjrefid
						inner join pr_capability cc on cc.capabilityid=cc.capabilityid
							WHERE cc.capabilityid=p_capabilityid
						and cc.partnerid=c.partnerid
							and cc.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));
    	
	    	
			update PR_ROUTINGCHANNEL
		    	SET ACTIVESTATUSCD = CASE WHEN ACTIVESTATUSCD in (0,1,4,5,8,9,12,13,16,17,20,21,24,25,28,29,32,33,36,37,40,41,44,45,48,49,52,53,56,57,60,61) THEN ACTIVESTATUSCD+2 ELSE ACTIVESTATUSCD END
		    	,   UPDATEDATE = v_BeginTS
			WHERE FRCAPABILITYID in (select c.capabilityid from pr_capability c
							inner join CF_ROUTINGOBJECT06_VW chld on c.ROUTINGOBJREFID=chld.routingobjrefid
						inner join pr_capability cc on cc.capabilityid=cc.capabilityid
							WHERE cc.capabilityid=p_capabilityid
						and cc.partnerid=c.partnerid
							and cc.routingobjrefid IN (chld.ROUTINGOBJREFID,
									chld.ROUTINGOBJPARENTREFID,
									chld.GRPARENTOBJREFID,
									chld.GRPARENTOBJREFID2,
									chld.GRPARENTOBJREFID3));
		Update PR_CAPABILITY
			SET
			    ROWTS= p_RowTS + 1
			    ,      UPDATEDATE= v_BeginTS
			    ,      ACTIVESTATUSCD =3
			    ,      RoleTypeCd = p_RoleTypeCd
			Where
			    CAPABILITYID = p_CAPABILITYID
			    and ACTIVESTATUSCD in (2,3)
			    And RowTS = p_RowTS returning RowTS into v_RowTS;
			    v_RowCnt := SQL%rowcount;
		
    END IF;
    
    IF RC1%isOpen THEN Close RC1; END IF;
    
    if (v_RowCnt = 0) then
    	OPEN RC1 for select p_RowTS RowTS from Dual;
    else
    	OPEN RC1 for select v_RowTS RowTS from Dual;
    end if;
    
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        WHEN TOO_MANY_ROWS Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --           Commit;
            RAISE_APPLICATION_ERROR(-20999, '20100 - PR_CapabilityU_AH1: Too Many Rows');
        WHEN ZeroUpdate then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --           Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CapabilityU_AH1: No Data Found');
        WHEN OTHERS then
 --           ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
 --           Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CapabilityU_AH1:'||SQLERRM);
End PR_CapabilityU_AH1;
/


-- PR_CERTDESTTYPED.sql

CREATE OR REPLACE Procedure PR_CERTDESTTYPED

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_CERTDESTTYPEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CERTDESTTYPED
	Create Date:     2003-09-26 08:24:11
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTDESTTYPED';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CERTDESTTYPEID=>'||p_CERTDESTTYPEID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_CERTDESTTYPE
	Where
		CERTDESTTYPEID = p_CERTDESTTYPEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CERTDESTTYPED: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_CERTDESTTYPED;
/


-- PR_CERTDESTTYPEI.sql

CREATE OR REPLACE Procedure PR_CERTDESTTYPEI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CERTID in NUMBER
	,p_DESTTYPEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CERTDESTTYPEI
	Create Date:     2003-09-26 00:01:42
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTDESTTYPEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CERTID=>'||p_CERTID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_CERTDESTTYPE(
		CERTDESTTYPEID
		,CERTID
		,DESTTYPEID
	) Values (
		PR_CERTDESTTYPE_SEQ.nextval
		,p_CERTID
		,p_DESTTYPEID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_CERTDESTTYPE_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_CERTDESTTYPEI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_CERTDESTTYPEI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CERTDESTTYPEI' || SQLCODE|| ' '||SQLERRM);
END PR_CERTDESTTYPEI;
/


-- PR_CERTDESTTYPES_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_CERTDESTTYPES_AH1
(
p_ReturnCd out integer
, p_DebugFlg in pls_integer
, RC1 out db_ref.RCT
, p_Locale in Varchar2
, p_CERTID in NUMBER
) As

/*
================================================================================
Proc Name:        PR_CERTDESTTYPES
Create Date:     2002-10-10 16:47:09
Author:            SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2  Removed NLS
================================================================================
Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
    begin
    DB_ProcAuditLogI_Ah1('PR_CERTDESTTYPES',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg ||','||p_Locale||','||'RC1'
    ||','||p_CERTID
    );
End ProcAudit;
BEGIN
   IF RC1%isOpen THEN Close RC1; END IF;
   Open RC1 for select
        c.CERTDESTTYPEID
   ,    c.CERTID
   ,    c.DESTTYPEID
   ,    c.ROWTS
   ,    c.CREATEDATE
   ,   dt.Resourcekey
   ,   dt.Description
   From PR_CERTDESTTYPE c
   INNER JOIN PR_DESTTYPE dt ON dt.DestTypeId = c.DestTypeId
   Where c.CERTID = p_CERTID
   ;

   p_ReturnCd := 0;
   procAudit;
END PR_CERTDESTTYPES_AH1;
/


-- PR_CERTD_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERTD_AH1
	(
	 p_ReturnCd OUT pls_integer
	,p_DebugFlg IN  pls_integer
	,P_CERTID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_CERTD_AH1
	Create Date:     2007-09-14 15:16:24
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-25 BHARANI J       6.1.1  Created for Multiple Certificate Feature

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTD_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', P_CERTID=>'||P_CERTID||CHR(10)
		;

zeroDelete  Exception;

BEGIN
			
	
	DELETE FROM PR_CERT_SET_LNK 
	WHERE CERTID=P_CERTID
	;
	
	DELETE FROM PR_CERTdesttype
		WHERE CERTID=P_CERTID
	;
	
	
	Update PR_CERT SET
	deleteflag=1
	WHERE CERTID=P_CERTID;
	
		
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
		
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CERTD_AH1: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_CERTD_AH1;
/


-- PR_CERTI.sql

CREATE OR REPLACE Procedure PR_CERTI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_CERTNAME in VARCHAR2
	,p_DEFFLG in NUMBER
	,p_CN in VARCHAR2
	,p_L in VARCHAR2
	,p_ST in VARCHAR2
	,p_O in VARCHAR2
	,p_OU in VARCHAR2
	,p_C in VARCHAR2
	,p_VALIDFR in TIMESTAMP
	,p_VALIDTO in TIMESTAMP
	,p_CERTVER in VARCHAR2
	,p_FINGERPRINTALGO in VARCHAR2
	,p_FINGERPRINT in VARCHAR2
	,p_SERIALNBR in VARCHAR2
	,p_PUBLICKEY in VARCHAR2
	,p_REVOKEDFLG in NUMBER
	,p_REVOKEDDATE in TIMESTAMP
	,p_CERT in BLOB
	,p_SSLCLIENTCERTFLG in NUMBER
	,p_DIGSIGCERTFLG in NUMBER
	,p_ENCRYPTCERTFLG in NUMBER
	,p_ENABLEDFLG in NUMBER
	,p_ROOTCERTFLG in NUMBER
	,p_PRISECFLG in NUMBER
	,p_PASSWORD in RAW
	,p_PRIVATEKEY in BLOB
	,P_CERTDESCRIPTION in VARCHAR2
	,P_FTPFLAG in NUMBER
	,P_SSLSERVERFLAG in NUMBER
	,P_ISSUER_CN in VARCHAR2
	,P_ISSUER_L in VARCHAR2
	,P_ISSUER_ST in VARCHAR2
	,P_ISSUER_O in VARCHAR2
	,P_ISSUER_OU in VARCHAR2
	,P_ISSUER_C in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_CERTI
	Create Date:     2003-09-26 00:01:40
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
        2004-06-28 Shishir N	    4.3	PKEE-5VC24T: Added column PRISECFLG
        2007-09-25 BHARANI J       6.1.1 Multiple Certificate Feature
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_CERTNAME=>'||p_CERTNAME||CHR(10)
		||', p_DEFFLG=>'||p_DEFFLG||CHR(10)
		||', p_CN=>'||p_CN||CHR(10)
		||', p_L=>'||p_L||CHR(10)
		||', p_ST=>'||p_ST||CHR(10)
		||', p_O=>'||p_O||CHR(10)
		||', p_OU=>'||p_OU||CHR(10)
		||', p_C=>'||p_C||CHR(10)
		||', p_VALIDFR=>'||p_VALIDFR||CHR(10)
		||', p_VALIDTO=>'||p_VALIDTO||CHR(10)
		||', p_CERTVER=>'||p_CERTVER||CHR(10)
		||', p_FINGERPRINTALGO=>'||p_FINGERPRINTALGO||CHR(10)
		||', p_FINGERPRINT=>'||p_FINGERPRINT||CHR(10)
		||', p_SERIALNBR=>'||p_SERIALNBR||CHR(10)
		||', p_PUBLICKEY=>'||p_PUBLICKEY||CHR(10)
		||', p_REVOKEDFLG=>'||p_REVOKEDFLG||CHR(10)
		||', p_REVOKEDDATE=>'||p_REVOKEDDATE||CHR(10)
		||', p_SSLCLIENTCERTFLG=>'||p_SSLCLIENTCERTFLG||CHR(10)
		||', p_DIGSIGCERTFLG=>'||p_DIGSIGCERTFLG||CHR(10)
		||', p_ENCRYPTCERTFLG=>'||p_ENCRYPTCERTFLG||CHR(10)
		||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
		||', p_ROOTCERTFLG=>'||p_ROOTCERTFLG||CHR(10)
		||', p_PRISECFLG=>'||p_PRISECFLG||CHR(10)
		||', p_PASSWORD=>'||p_PASSWORD||CHR(10)
		||', P_CERTDESCRIPTION=>'||P_CERTDESCRIPTION||CHR(10)
		||', P_FTPFLAG=>'||P_FTPFLAG||CHR(10)
		||', P_SSLSERVERFLAG=>'||P_SSLSERVERFLAG||CHR(10)
		||', P_ISSUER_CN=>'||P_ISSUER_CN||CHR(10)
		||', P_ISSUER_L=>'||P_ISSUER_L||CHR(10)
		||', P_ISSUER_ST=>'||P_ISSUER_ST||CHR(10)
		||', P_ISSUER_O=>'||P_ISSUER_O||CHR(10)
		||', P_ISSUER_OU=>'||P_ISSUER_OU||CHR(10)
		||', P_ISSUER_C=>'||P_ISSUER_C||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_CERT(
		CERTID
		,PARTNERID
		,CERTNAME
		,DEFFLG
		,CN
		,L
		,ST
		,O
		,OU
		,C
		,VALIDFR
		,VALIDTO
		,CERTVER
		,FINGERPRINTALGO
		,FINGERPRINT
		,SERIALNBR
		,PUBLICKEY
		,REVOKEDFLG
		,REVOKEDDATE
		,CERT
		,SSLCLIENTCERTFLG
		,DIGSIGCERTFLG
		,ENCRYPTCERTFLG
		,ENABLEDFLG
		,ROOTCERTFLG
		,PRISECFLG
		,PASSWORD
		,PRIVATEKEY
		,CERTDESCRIPTION
		,FTPFLAG
		,SSLSERVERFLAG
		,ISSUER_CN
		,ISSUER_L
		,ISSUER_ST
		,ISSUER_O
		,ISSUER_OU
		,ISSUER_C
	) Values (
		PR_CERT_SEQ.nextval
		,p_PARTNERID
		,p_CERTNAME
		,p_DEFFLG
		,p_CN
		,p_L
		,p_ST
		,p_O
		,p_OU
		,p_C
		,p_VALIDFR
		,p_VALIDTO
		,p_CERTVER
		,p_FINGERPRINTALGO
		,p_FINGERPRINT
		,p_SERIALNBR
		,p_PUBLICKEY
		,p_REVOKEDFLG
		,p_REVOKEDDATE
		,p_CERT
		,p_SSLCLIENTCERTFLG
		,p_DIGSIGCERTFLG
		,p_ENCRYPTCERTFLG
		,p_ENABLEDFLG
		,p_ROOTCERTFLG
		,p_PRISECFLG
		,p_PASSWORD
		,p_PRIVATEKEY
		,P_CERTDESCRIPTION
		,P_FTPFLAG
		,P_SSLSERVERFLAG
		,P_ISSUER_CN
		,P_ISSUER_L
		,P_ISSUER_ST
		,P_ISSUER_O
		,P_ISSUER_OU
		,P_ISSUER_C
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_CERT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_CERTI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_CERTI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CERTI' || SQLCODE|| ' '||SQLERRM);
END PR_CERTI;
/


-- PR_CERTI_AH1.sql

CREATE OR REPLACE Procedure PR_CERTI_AH1
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CERTID in NUMBER
	,p_PRISECFLG in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CERTI_AH1
	Create Date:     2006-10-16 00:01:40
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
        2004-06-28 Shishir N	    4.3	PKEE-5VC24T: Added column PRISECFLG
        2006-10-16 Sujit Nair       6.1 102872f_1
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTI_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CERTID=>'||p_CERTID||CHR(10)
		||', p_PRISECFLG=>'||p_PRISECFLG||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_CERT(CERTID
		,PARTNERID
		,CERTNAME
		,DEFFLG
		,CN
		,L
		,ST
		,O
		,OU
		,C
		,VALIDFR
		,VALIDTO
		,CERTVER
		,FINGERPRINTALGO
		,FINGERPRINT
		,SERIALNBR
		,PUBLICKEY
		,REVOKEDFLG
		,REVOKEDDATE
		,CERT
		,CREATEDATE
		,SSLCLIENTCERTFLG
		,DIGSIGCERTFLG
		,ENCRYPTCERTFLG
		,ENABLEDFLG
		,ROOTCERTFLG
		,PRISECFLG
		,PASSWORD
		,PRIVATEKEY)
	Select 
	 	PR_CERT_SEQ.nextval
	 	,PARTNERID
	 	,CERTNAME
	 	,DEFFLG
	 	,CN
	 	,L
	 	,ST
	 	,O
	 	,OU
	 	,C
	 	,VALIDFR
		,VALIDTO
		,CERTVER
		,FINGERPRINTALGO
		,FINGERPRINT
		,SERIALNBR
		,PUBLICKEY
		,REVOKEDFLG
		,REVOKEDDATE
		,CERT
		,CREATEDATE
		,SSLCLIENTCERTFLG
		,DIGSIGCERTFLG
		,ENCRYPTCERTFLG
		,ENABLEDFLG
		,ROOTCERTFLG
		,p_PRISECFLG
		,PASSWORD
		,PRIVATEKEY 
	From
		PR_CERT	
	Where   CERTID = p_CERTID
	AND coalesce(deleteflag,0) <> 1;

	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_CERT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_CERTI_AH1' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_CERTI_AH1' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CERTI_AH1' || SQLCODE|| ' '||SQLERRM);
END PR_CERTI_AH1;
/


-- PR_CERTS.sql

CREATE OR REPLACE PROCEDURE PR_CERTS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CERTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CERTS
	Create Date:     2003-09-26 00:06:43
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-06-28 Shishir N	    4.3	PKEE-5VC24T: Added column PRISECFLG
	2007-09-14 Amit V.	  6.1.1	Multiple certificate
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CERTID=>'||p_CERTID||CHR(10)
		;
v_certsetid 	  pr_cert_set.certsetid%TYPE;
v_partnerid         pr_partner.partnerid%TYPE;
v_searchflg         INTEGER;
 

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_CERTID IS NOT NULL
 THEN
 
 
 	SELECT pc.partnerid,
 	CASE ppt.PARTNERTYPENAME 
 	WHEN 'HubOperator' THEN 0
 	WHEN 'HubOwner' THEN 1
 	WHEN 'HubPartner' THEN 2
 	END into v_partnerid,v_searchflg 
 	from pr_cert pc ,pr_partner pp , pr_partnertype ppt
 	WHERE pc.partnerid = pp.partnerid 
 	AND pp.partnertypeid = ppt.partnertypeid
 	AND pc.certid = p_CERTID ;
 	
 	BEGIN
 		SELECT b.certsetid INTO v_certsetid
 		from PR_CERT a
 		INNER JOIN pr_cert_set_lnk b on a.certid = b.certid
 		INNER JOIN pr_cert_set c on b.certsetid = c.certsetid
 		AND COALESCE(a.deleteflag,0) <> 1
 		AND a.certid = p_CERTID;
 		
	EXCEPTION 
	
	WHEN NO_DATA_FOUND THEN
	v_certsetid := -1;
 	
 	END;

 
 
	OPEN RC1 FOR
	SELECT
		CERTID
		,PARTNERID
		,RTRIM(CERTNAME) CERTNAME
		,FN_SET_DEFORNONDEF(v_certsetid,v_partnerid,v_searchflg) DEFFLG
		,RTRIM(CN) CN
		,RTRIM(L) L
		,RTRIM(ST) ST
		,RTRIM(O) O
		,RTRIM(OU) OU
		,RTRIM(C) C
		,VALIDFR
		,VALIDTO
		,RTRIM(CERTVER) CERTVER
		,RTRIM(FINGERPRINTALGO) FINGERPRINTALGO
		,RTRIM(FINGERPRINT) FINGERPRINT
		,RTRIM(SERIALNBR) SERIALNBR
		,RTRIM(PUBLICKEY) PUBLICKEY
		,REVOKEDFLG
		,REVOKEDDATE
		,CERT
		,ROWTS
		,CREATEDATE
		,SSLCLIENTCERTFLG
		,DIGSIGCERTFLG
		,ENCRYPTCERTFLG
		,ENABLEDFLG
		,ROOTCERTFLG
		,PRISECFLG
		,PASSWORD
		,PRIVATEKEY
		,CERTDESCRIPTION
		,FTPFLAG
		,SSLSERVERFLAG
		,ISSUER_CN
		,ISSUER_L
		,ISSUER_ST
		,ISSUER_O
		,ISSUER_OU
		,ISSUER_C
	FROM
		PR_CERT
	Where
		CERTID = p_CERTID
		AND coalesce(deleteflag,0) <> 1	;
	ELSE
	OPEN RC1 FOR
	SELECT
		CERTID
		,PARTNERID
		,RTRIM(CERTNAME) CERTNAME
		,DEFFLG
		,RTRIM(CN) CN
		,RTRIM(L) L
		,RTRIM(ST) ST
		,RTRIM(O) O
		,RTRIM(OU) OU
		,RTRIM(C) C
		,VALIDFR
		,VALIDTO
		,RTRIM(CERTVER) CERTVER
		,RTRIM(FINGERPRINTALGO) FINGERPRINTALGO
		,RTRIM(FINGERPRINT) FINGERPRINT
		,RTRIM(SERIALNBR) SERIALNBR
		,RTRIM(PUBLICKEY) PUBLICKEY
		,REVOKEDFLG
		,REVOKEDDATE
		,CERT
		,ROWTS
		,CREATEDATE
		,SSLCLIENTCERTFLG
		,DIGSIGCERTFLG
		,ENCRYPTCERTFLG
		,ENABLEDFLG
		,ROOTCERTFLG
		,PRISECFLG
		,PASSWORD
		,PRIVATEKEY
		,CERTDESCRIPTION
		,FTPFLAG
		,SSLSERVERFLAG
		,ISSUER_CN
		,ISSUER_L
		,ISSUER_ST
		,ISSUER_O
		,ISSUER_OU
		,ISSUER_C		
	FROM
		PR_CERT
	Where coalesce(deleteflag,0) <> 1
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CERTS;
/


-- PR_CERTSETU_DEFAULT.sql

CREATE OR REPLACE PROCEDURE PR_CERTSETU_DEFAULT
(
	p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1			out		DB_REF.RCT
	, p_CertID		in		INTEGER
	, p_CertSetID		in		INTEGER
) AS
/*
	================================================================================
	Proc Name:       PR_CERTSETU_DEFAULT
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
        2008-01-01  BHARANI J     6.1.1  105492.
        2008-02-15  BHARANI J     6.1.1  105855
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_RowCnt          integer Default 0;
	v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
	v_ProcName        varchar2(30) Default 'PR_USERS_AH3';
	v_ExceptionFlg    integer Default 0;
	v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CertID=>'||p_CertID||CHR(10)
		||', p_CertSetID=>'||p_CertSetID||CHR(10)
		;

	v_RowTS			integer :=0;
	v_certSetID_new		PR_CERT_SET.CertSetID%TYPE := -1;
	v_certSetID_enc		PR_CERT_SET.CertSetID%TYPE;
	v_certSetID_ssl		PR_CERT_SET.CertSetID%TYPE;
	v_certSetID_sig		PR_CERT_SET.CertSetID%TYPE;
	v_sslclientflg		PR_CERT.sslclientcertflg%TYPE;
	v_digsigflg		PR_CERT.digsigcertflg%TYPE;
	v_encryptflg		PR_CERT.encryptcertflg%TYPE;
	v_DestTypeID		PR_DESTTYPE.desttypeid%TYPE;
	v_CertSetUsageID	PR_CERT_SET_USAGE.certsetusageid%TYPE;
	v_RC1			DB_REF.RCT;
	v_Rc2                   DB_REF.RCT;
	v_PartnerID		PR_PARTNER.partnerid%TYPE := -1;
	v_PartnerPairID		PR_PARTNER_PAIR.partnerpairid%TYPE := -1;
	v_RoutingObjPairID	PR_ROUTINGOBJ_PAIR.routingobjpairid%TYPE := -1;
	v_partnerrtobjlnkid	pr_partnerpair_rtobjpair_lnk.partnerrtobjlnkid%TYPE := -1;
	v_certenableflg		pr_cert.enabledflg%TYPE := 0; -- disable
	v_pricertid		pr_cert.certid%TYPE;
	v_seccertid		pr_cert.certid%TYPE;
	v_id			VARCHAR2(10)		:= TO_CHAR(SYSDATE, 'SSHH24MIMMYY');
	v_certsetnm		pr_cert_set.certsetname%TYPE	:= 'DEF';
	v_certsetdesc		pr_cert_set.certsetdesc%TYPE	:= 'System Generated Default Set';
	v_debugflg      integer;
    	v_returncd      integer;
    	V_ROUTINGOBJTYPEID integer;
    	v_splitset  INTEGER;
        v_digsigflg_oldsec INTEGER;
        v_sslclientflg_oldsec INTEGER;
        v_dont_move_sec INTEGER;

--------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF p_CertSetID IS NULL THEN



	--- new primary certificate being added

		BEGIN
			SELECT	partnerid
				,sslclientcertflg
				,digsigcertflg
				,encryptcertflg
			INTO	v_partnerid
				,v_sslclientflg
				,v_digsigflg
				,v_encryptflg
			FROM	pr_cert
			WHERE	certid = p_CertID
			AND coalesce(deleteflag,0) <> 1
			AND	prisecflg = 1;
		EXCEPTION
			WHEN No_Data_Found THEN
				p_ReturnCd := 4;
		END;

		--- Create new set for this certificate
		IF v_certsetid_new = -1 THEN
		--- create certset
			INSERT INTO pr_cert_set(certsetid, certsetname, certsetdesc, partnerid, digsigflg, sslclientflg, encryptflg,status)
				(SELECT	pr_cert_set_seq.NEXTVAL, v_certsetnm || pr_cert_set_seq.CURRVAL || v_id, v_certsetdesc, v_partnerid, v_digsigflg, v_sslclientflg, v_encryptflg,1
				FROM dual);
			SELECT	pr_cert_set_seq.CURRVAL INTO v_certsetid_new FROM DUAL;
		--- link the primary cert
			INSERT INTO pr_cert_set_lnk(certsetlnkid, certsetid, certid)
				(SELECT	pr_cert_set_lnk_seq.NEXTVAL, v_certsetid_new, p_certid
				FROM dual);
		END IF;

		BEGIN
			SELECT	partnerpairID
			INTO	v_partnerPairID
			FROM	pr_partner_pair pp
			WHERE	pp.frpartnerid IS NULL AND pp.topartnerid IS NULL;

			EXCEPTION

			WHEN NO_Data_Found THEN
			pr_partner_pairi(v_returncd,v_debugflg,v_rc1,null,null);
	     		Fetch v_rc1 into v_partnerpairid,v_rowts;
		END;
			SELECT Routingobjtypeid INTO v_routingobjtypeid from cf_routingobjtype
			WHERE Routingobjtypekey='Package';

		BEGIN
			SELECT	routingobjpairid
			INTO	v_routingobjpairid
			FROM	pr_routingobj_pair rp
			WHERE	rp.frroutingobjid IS NULL AND rp.toroutingobjid IS NULL;

			EXCEPTION
			WHEN NO_Data_Found THEN
               		PR_ROUTINGOBJ_PAIRI(v_returncd, v_debugflg, v_RC2, v_routingobjtypeid,null,null);

               		Fetch v_RC2 into v_routingobjpairid,v_rowts;
		END;

		BEGIN
			SELECT	partnerrtobjlnkid
			INTO	v_partnerrtobjlnkid
			FROM	pr_partnerpair_rtobjpair_lnk ppl
			WHERE	ppl.PartnerPairID = v_PartnerPairID
			AND	ppl.RoutingObjPairID = v_RoutingObjPairID;

			EXCEPTION
			WHEN NO_Data_Found THEN
			--- need to create default partner/package pair link
			INSERT INTO pr_partnerpair_rtobjpair_lnk (partnerrtobjlnkid, PartnerPairID, RoutingObjPairID )				(SELECT	PR_PARPAIR_RTOBJPAIR_LNK_SEQ.NEXTVAL, v_PartnerPairID, v_RoutingObjPairID
			FROM	dual);
			SELECT	PR_PARPAIR_RTOBJPAIR_LNK_SEQ.currval INTO v_partnerrtobjlnkid FROM dual;
		END;

		--- look for hubOperator default CertSets for all destinationType(s) for this certificate
		FOR cX IN (SELECT	pdt.destTypeID
					,COALESCE( pcu.CERTSETUSAGEID, -1) CertSetUsageID
					,COALESCE( pcu.CERTSETID_ENCRYPTION, -1) certsetid_encryption
					,COALESCE( pcu.CERTSETID_SIGNING, -2) certsetid_signing
					,COALESCE( pcu.CERTSETID_SSLCLIENT, -4) certsetid_sslclient
				FROM	pr_certdesttype pdt
				LEFT OUTER JOIN	pr_cert_set_usage pcu ON pcu.desttypeid = pdt.desttypeid
						AND	pcu.partnerrtobjlnkid = v_partnerrtobjlnkid
				WHERE	pdt.certid = p_CertID
				)



		LOOP

			IF cX.CertSetUsageID = -1 THEN
			--- No default certset to use
			--- use the CertSet for current destination as default for hubOperator
				--- confgure this new set as default for the destination
			INSERT INTO pr_cert_set_usage(certsetusageid, partnerrtobjlnkid, desttypeid
					, certsetid_encryption, certsetid_signing, certsetid_sslclient)
					(SELECT	pr_cert_set_usage_seq.NEXTVAL, v_partnerrtobjlnkid, cX.DestTypeID
						, CASE WHEN v_encryptflg = 1 THEN v_certsetid_new ELSE NULL END
						, CASE WHEN v_digsigflg = 1 THEN v_certsetid_new ELSE NULL END
						, CASE WHEN v_sslclientflg = 1 THEN v_certsetid_new ELSE NULL END
					FROM dual);
			ELSE --- default certsets are configured
				--- update the usage with New certset, using it for the flgs defined on the cert

				--- If the first cert is of type SSL and DIGSIG and the second
				--- is of either SSL or DIGSIG, then we have to split the set.

				IF cX.certsetid_signing = cX.certsetid_sslclient AND cX.certsetid_sslclient <> -1
				AND v_digsigflg <> v_sslclientflg THEN
					--- need to update
					--- update the usage with New certset, using it for the flgs defined on the cert
					v_splitset := 1;
				ELSE
					v_splitset := 0;
				END IF;

				UPDATE	pr_cert_set_usage psu
				SET	RowTS = RowTS + 1
					,certsetid_encryption = CASE WHEN v_encryptflg = 1 THEN v_certsetid_new ELSE psu.certsetid_encryption END
					,certsetid_signing = CASE WHEN v_digsigflg = 1 THEN v_certsetid_new ELSE psu.certsetid_signing END
					,certsetid_sslclient = CASE WHEN v_sslclientflg = 1 THEN v_certsetid_new ELSE psu.certsetid_sslclient END
				WHERE	CertSetUsageID = cX.CertSetUsageID
				;


				--- get cert sets which got overwritten
				SELECT	CASE WHEN v_encryptflg = 1 THEN cX.certsetid_encryption ELSE -1 END
					, CASE WHEN v_digsigflg = 1 THEN cX.certsetid_signing ELSE -2 END
					, CASE WHEN v_sslclientflg = 1 THEN cX.certsetid_sslclient ELSE -4 END
				INTO	v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl
				FROM	dual;

               			IF v_splitset = 1 THEN

					-- The old set should not be used. so updated the old set usages to null. Added for Defect 105855
					UPDATE	pr_cert_set_usage psu
					SET	RowTS = RowTS + 1
					,certsetid_encryption = CASE WHEN v_encryptflg = 1 THEN v_certsetid_new ELSE NULL END
					,certsetid_signing = CASE WHEN v_digsigflg = 1 THEN v_certsetid_new ELSE NULL END
					,certsetid_sslclient = CASE WHEN v_sslclientflg = 1 THEN v_certsetid_new ELSE NULL END
					WHERE	CertSetUsageID = cX.CertSetUsageID
					AND desttypeid = cX.destTypeID;


					-- Old SET needs to be removed.  So SET usage should be removed first.

					UPDATE	pr_cert_set_usage psu
					SET	RowTS = ROWTS + 1
					,certsetid_encryption =  NULL
					WHERE
					certsetid_encryption IN (v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl)
                    			;

                   			 UPDATE	pr_cert_set_usage psu
					SET	RowTS = ROWTS + 1
					,certsetid_signing =  NULL
					WHERE
					certsetid_signing IN (v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl)
                    			;

                    			UPDATE	pr_cert_set_usage psu
					SET	RowTS = ROWTS + 1
					,certsetid_sslclient =  NULL
					WHERE
					certsetid_sslclient IN (v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl)
                                        ;

				END IF;

				UPDATE	pr_cert
				SET	RowTS = RowTS + 1
				,enabledflg = v_certenableflg
				,prisecflg = -1
				,SSLCLIENTCERTFLG = 0
				,DIGSIGCERTFLG = 0
				,ENCRYPTCERTFLG = 0
				,ROOTCERTFLG  = 0
				,FTPFLAG = 0
				WHERE	certid IN (SELECT	certID
						FROM	pr_cert_set_lnk
						WHERE	certsetid IN (v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl)
						AND PRISECFLG = 1
					)
				AND coalesce(deleteflag,0) <> 1
				;

				-- remove the desttype

				Delete from PR_CERTDESTTYPE
				WHERE	certid IN (SELECT	a.certID
				FROM	pr_cert_set_lnk a, pr_cert b
				WHERE	certsetid IN (v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl)
				AND a.certid = b.certid AND prisecflg = -1 )
				;
                
                --  Start of Fix.  Special Case
                
                --  Take the property of old secondary cert.  
                
                BEGIN
                
                select nvl(b.DIGSIGCERTFLG,-1),nvl(b.SSLCLIENTCERTFLG,-1)
                INTO v_digsigflg_oldsec,v_sslclientflg_oldsec 
                FROM pr_cert_set_lnk a, pr_cert b
		WHERE a.certid = b.certid AND prisecflg = 2
                AND certsetid IN (v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl);
                
                EXCEPTION
			WHEN NO_Data_Found THEN
			NULL;
			
			WHEN TOO_MANY_ROWS THEN
			-- In this case secondary cert should not be moved.
			v_dont_move_sec:=1;
			
		END;			
               
                IF (v_digsigflg_oldsec <> v_sslclientflg_oldsec AND v_digsigflg = v_sslclientflg) OR
                   (v_digsigflg_oldsec = v_sslclientflg_oldsec AND v_digsigflg <> v_sslclientflg)
                THEN
                      v_dont_move_sec:=1;
		END IF;  
                      
		IF v_dont_move_sec = 1 THEN
		
		 -- Do not link sec cert of type of SSL OR SIGN to new set which is of type SSL AND SIGN
		 -- Make the old sec cert usage to null
		
			UPDATE pr_cert SET 
			RowTS = RowTS + 1
			,enabledflg = v_certenableflg
			,prisecflg = -1
			,SSLCLIENTCERTFLG = 0
			,DIGSIGCERTFLG = 0
			,ENCRYPTCERTFLG = 0
			,ROOTCERTFLG  = 0
			,FTPFLAG = 0
			WHERE	certid IN (select a.certid FROM pr_cert_set_lnk a, pr_cert b
				    where a.certid = b.certid AND prisecflg = 2)
			AND coalesce(deleteflag,0) <> 1;
			
		-- Remove the old secondary destination type also
		
			Delete from PR_CERTDESTTYPE
			WHERE	certid IN
			(select a.certid FROM pr_cert_set_lnk a, pr_cert b
			where a.certid = b.certid AND prisecflg = -1)
			;
					
                ELSE
			UPDATE pr_cert_set_lnk SET certsetid=v_certsetid_new
			WHERE certsetid IN (v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl)
			AND certid IN (select a.certid FROM pr_cert_set_lnk a, pr_cert b
			WHERE a.certid = b.certid AND prisecflg = 2
                        );
                END IF;     
                
                --  End of Fix.
                    
               

				DELETE FROM pr_cert_set_lnk
				WHERE	certsetid IN (v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl)
				;

				BEGIN


				DELETE FROM pr_cert_set
				WHERE	certsetid IN (v_certsetid_enc, v_certsetid_sig, v_certsetid_ssl)
				;

				EXCEPTION

				WHEN OTHERS THEN
				NULL;

				END;


			END IF;

		END LOOP;

	ELSE
	--- secondary certificate needs to be added to existing set
		BEGIN
			SELECT	cs.sslclientflg
				,cs.digsigflg
				,cs.encryptflg
				,MAX(COALESCE(c1.certid, -1))
				,MAX(COALESCE(c2.certid, -1))
			INTO	v_sslclientflg
				,v_digsigflg
				,v_encryptflg
				,v_pricertid
				,v_seccertid
			FROM	pr_cert_set cs
			LEFT OUTER JOIN pr_cert_set_lnk csl ON csl.certsetid = cs.certsetid
			LEFT OUTER JOIN pr_cert c1 ON csl.certid = c1.certid AND c1.prisecflg = 1 AND coalesce(c1.deleteflag,0) <> 1
			LEFT OUTER JOIN pr_cert c2 ON csl.certid = c2.certid AND c2.prisecflg <> 1 AND coalesce(c2.deleteflag,0) <> 1
			WHERE	cs.certsetid = p_CertsetID
			GROUP BY cs.sslclientflg,cs.digsigflg,cs.encryptflg
			;
			--- if set does not contain primary then error
			IF v_pricertid = -1 THEN
				Raise No_Data_Found;
			END IF;

			--- existing secondary cert will be disabled, and unlinked from the set
			IF v_seccertid <> -1 THEN
				UPDATE	pr_cert
				SET	RowTS = RowTS + 1
				, enabledflg = v_certenableflg
				, prisecflg = -1
				, SSLCLIENTCERTFLG = 0
				, DIGSIGCERTFLG = 0
				, ENCRYPTCERTFLG = 0
				, ROOTCERTFLG  = 0
				, FTPFLAG = 0
				WHERE	certid = v_seccertid
				AND coalesce(deleteflag,0) <> 1;

			-- remove the secondary desttype

				DELETE FROM pr_certdesttype
				WHERE certid=v_seccertid;


				DELETE FROM pr_cert_set_lnk
				WHERE	certid = v_seccertid AND certsetid = p_certsetid ;
			END IF;

			--- update the new secondary to match the attribs from the set/primary cert
			UPDATE	pr_cert
			SET	sslclientcertflg = v_sslclientflg
				,digsigcertflg = v_digsigflg
				,encryptcertflg = v_encryptflg
				,prisecflg = 2
			WHERE	certid = p_certid
			AND coalesce(deleteflag,0) <> 1
			;

			INSERT INTO pr_certdesttype(certdesttypeid, certid, desttypeid)
				(SELECT	pr_certdesttype_seq.NEXTVAL, p_certid, desttypeid
				FROM	pr_certdesttype aa
				WHERE	aa.certid = v_pricertid
				AND	NOT EXISTS (SELECT 'x' FROM pr_certdesttype b WHERE b.certID = p_certID AND b.desttypeid = aa.desttypeid)
				);

			INSERT INTO pr_cert_set_lnk(certsetlnkid, certsetid, certid)
				(SELECT	pr_cert_set_lnk_seq.NEXTVAL, p_certsetid, p_certid
				FROM dual);

		EXCEPTION
			WHEN No_Data_Found THEN
				p_ReturnCd := 4 ;
		END;

	END IF;


	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 FOR SELECT CertSetId, CertSetName, RowTS
		FROM	pr_cert_set
		WHERE	CertSetID IN (v_CertSetId_new, p_CertSetID);
	p_returncd := 0;

EXCEPTION
	WHEN OTHERS then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
END;
/

-- PR_CERTS_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_CERTS_AH1
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        , p_ValidTo IN TimeStamp
        )
AS

/*
    ================================================================================
    Proc Name:       PR_CERTS_AH1
    Create Date:     2003-07-21
    Author:          V Goel
            This Proc select all the certs based upon Valid To date
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2004-06-28 Shishir N	    4.3	PKEE-5VC24T: Added column PRISECFLG            
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
    JR23211	: Enabledflg should be compared to 0 and not 3.
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_ValidTo=>'||p_ValidTo||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        IF RC1%ISOPEN THEN CLOSE RC1; END IF;
        OPEN RC1 FOR
        SELECT
--                CERTID
                c.PARTNERID
                ,P.PartnerName
                ,RTRIM(CERTNAME) CERTNAME
--                ,DEFFLG
                ,RTRIM(CN) CN
                ,RTRIM(L) L
                ,RTRIM(ST) ST
                ,RTRIM(O) O
                ,RTRIM(OU) OU
                ,RTRIM(C) C
                ,VALIDFR
                ,VALIDTO
--                ,RTRIM(CERTVER) CERTVER
--                ,RTRIM(FINGERPRINTALGO) FINGERPRINTALGO
--                ,RTRIM(FINGERPRINT) FINGERPRINT
--                ,RTRIM(SERIALNBR) SERIALNBR
--                ,RTRIM(PUBLICKEY) PUBLICKEY
--                ,REVOKEDFLG
--                ,REVOKEDDATE
                ,CERT
--                ,ROWTS
--                ,CREATEDATE
                ,SSLCLIENTCERTFLG
                ,DIGSIGCERTFLG
                ,ENCRYPTCERTFLG
                ,ENABLEDFLG
                ,ROOTCERTFLG
                ,PRISECFLG
                ,privatekey
--                ,PASSWORD
        FROM
                PR_CERT c
                Inner Join Pr_Partner P On P.PartnerID = c.partnerID
        Where
                c.ValidTo <= p_ValidTo
        And     c.RevokedFlg = 0
--        And     c.EnabledFlg <> 3
        And     c.EnabledFlg <> 0
        AND coalesce(c.deleteflag,0) <> 1
        ;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CERTS_AH1;
/


-- PR_CERTS_AH10.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_AH10
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 out db_ref.RCT
	,P_CERTNAME IN VARCHAR2
	,p_PARTNERID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_CERTS_AH10
	Create Date:     2007-09-19 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-12-05 BHARANI J    6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	USED BY CONSOLE TO AVOID INSERTING DUPLICATE RECORD
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERTS_AH10';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', P_CERTNAME=>'||P_CERTNAME||CHR(10)
       ||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
       ;

BEGIN

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	Open RC1 FOR 	SELECT
    				a.certid,
				a.certname
			FROM
				PR_CERT a
			WHERE
				a.CERTNAME = P_CERTNAME
				AND a.partnerid = p_PARTNERID
				AND a.deleteflag IS NULL;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

	EXCEPTION
			WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERTS_AH10:'||SQLERRM);

END PR_CERTS_AH10;
/

-- PR_CERTS_AH11.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_AH11
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 out db_ref.RCT
	,p_certsetid IN NUMBER
	)
AS

/*


	================================================================================
	Proc Name:       PR_CERTS_AH7
	Create Date:     2007-09-19 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2008-12-17  Anurag Panda   6.2.0.x  
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	Return the primary certificate for the given certSetId.
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERTS_AH11';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_certsetid=>'||p_certsetid||CHR(10);


BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	Open RC1 FOR 	select a.CERT 
		FROM  pr_cert a ,pr_cert_set_lnk l 
		where a.certid = l.certid
	    	and l.certsetid = p_certsetid
                and a.priSecFlg = 1
	    	AND coalesce(a.deleteflag,0) <> 1;
	    	
   	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   	
	EXCEPTION
		WHEN OTHERS then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERTS_AH7:'||SQLERRM);
			        
END PR_CERTS_AH11;
/
                     

-- PR_CERTS_AH2.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_AH2
        (
        p_ReturnCd      OUT  PLS_INTEGER
        , p_DebugFlg    IN  PLS_INTEGER
        , RC1           OUT  DB_REF.RCT
        , p_CN          IN   VARCHAR2
        , p_L           IN   VARCHAR2
        , p_ST          IN   VARCHAR2
        , p_O           IN   VARCHAR2
        , p_OU          IN   VARCHAR2
        , p_C           IN   VARCHAR2
        )
AS
/*
    ================================================================================
    Proc Name:       PR_CERTS_AH2
    Create Date:     2003-07-25
    Author:          V Goel
            This Proc select BusinessID. Note If input parameter as well as the data,
            both are null they are considered equal
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

        ---------  ------------ -------	---------------------------------------------
        DATE       NAME         Version	DESCRIPTION
        ---------  ------------ -------	---------------------------------------------
	2007-08-24 S.Priani	6.1.0.1	CR 104489

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTS_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_CN=>'  ||p_CN||CHR(10)
                ||', p_L=>'   ||p_L ||CHR(10)
                ||', p_ST=>'  ||p_ST||CHR(10)
                ||', p_O=>'   ||p_O ||CHR(10)
                ||', p_OU=>'  ||p_OU||CHR(10)
                ||', p_C=>'   ||p_C ||CHR(10)
                ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
        select b.businessid
		,b.partnerid
        from pr_partner p
        inner join pr_busidentifier b on b.partnerid = p.partnerid
        inner join pr_cert c on c.partnerid = p.partnerid AND coalesce(c.deleteflag,0) <> 1
        where nvl(c.CN, 'A') = nvl(p_CN, 'A')
        and   nvl(c.L , 'A') = nvl(p_L , 'A')
        and   nvl(c.ST, 'A') = nvl(p_ST, 'A')
        and   nvl(c.O , 'A') = nvl(p_O , 'A')
        and   nvl(c.OU, 'A') = nvl(p_OU, 'A')
        and   nvl(c.C , 'A') = nvl(p_C , 'A')
        ;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CERTS_AH2;
/


-- PR_CERTS_AH3.sql

CREATE OR REPLACE PROCEDURE         PR_CERTS_AH3
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        , p_PartnerTypeNameList  VARCHAR2
        ,  p_ExcludeFlg  IN INTEGER
        )
AS

/*
    ================================================================================
     Proc Name:       PR_CERTS_AH3
        Create Date:     2003-07-29 11:35:49
        Author:          V.Goel
        COMMENTS:
        For excludeFlg = 0 -- This Procedure returns all the Certs for PartnerTypes in the List
        For excludeFlg = 1 -- This Procedure returns all the Certs for all the PartnerTypes Not in the List
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2004-06-28 Shishir N	    4.3	PKEE-5VC24T: Added column PRISECFLG        
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTS_AH3';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000);
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_PartnerTypeNameList=>'||p_PartnerTypeNameList||CHR(10)
                ||', p_ExcludeFlg=>'||p_ExcludeFlg||CHR(10);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

   v_sqlstmt :=
    'SELECT '|| chr(10) ||
    ' CERTID, '|| chr(10) ||
    ' c.PARTNERID, '|| chr(10) ||
    ' DEFFLG, '|| chr(10) ||
    ' RTRIM(CN) CN, '|| chr(10) ||
    ' VALIDFR, '|| chr(10) ||
    ' VALIDTO, '|| chr(10) ||
    ' RTRIM(SERIALNBR) SERIALNBR,'|| chr(10) ||
    ' REVOKEDFLG, '|| chr(10) ||
    ' CERT, '|| chr(10) ||
    ' SSLCLIENTCERTFLG, '|| chr(10) ||
    ' DIGSIGCERTFLG, '|| chr(10) ||
    ' ENCRYPTCERTFLG, '|| chr(10) ||
    ' ENABLEDFLG, '|| chr(10) ||
    ' ROOTCERTFLG, '|| chr(10) ||
    ' PRISECFLG, ' || chr(10) ||
    ' PRIVATEKEY '|| chr(10) ||
    ' FROM PR_CERT C '|| chr(10) ||
    ' Inner Join PR_Partner P  On C.PartnerID = P.PartnerID'|| chr(10) ||
    ' Inner Join PR_PartnerType PT  On P.PartnerTypeID = PT.PartnerTypeID'|| chr(10) ||
    ' WHERE coalesce(c.deleteflag,0) <> 1
      AND PartnerTypeName '|| Case When p_ExcludeFlg = 1 Then ' NOT IN '
                                    Else ' IN ' End || p_PartnerTypeNameList;


--------------
-- Open rc1
--------------
 IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for v_sqlstmt;
      p_ReturnCd := 0;
      DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 Exception
    When NO_DATA_FOUND THEN p_ReturnCd     := 3;
    When OTHERS Then
                p_ReturnCd     := 1;
                v_ExceptionFlg := 1;

End PR_CERTS_AH3;
/


-- PR_CERTS_AH5.sql

CREATE OR REPLACE PROCEDURE pr_certs_ah5 (
   p_returncd   OUT      INTEGER,
   p_debugflg   IN       PLS_INTEGER,
   rc1          OUT      Db_Ref.rct,
   p_lastupdttime   IN       TIMESTAMP
)
AS
/*
================================================================================
Proc Name:        PR_CERTS_AH5
Create Date:      2004-06-20
Author:           Shishir Narain
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_rowcnt       INTEGER   := 0;
   v_begints      TIMESTAMP := Fn_Get_Date;  -- Performance related variables
   v_updatedate   TIMESTAMP;

   PROCEDURE procaudit
   IS
   BEGIN
      Db_Procauditlogi_Ah1 ('PR_CERTS_AH5',
                            v_begints,
                               'p_ReturnCd'
                            || ','
                            || p_debugflg
                            || ','
                            || 'RC1'
                            || ','
                            || p_lastupdttime
                           );
   END procaudit;
BEGIN
--v_updateDate := nvl(p_UpdateDate, To_Timestamp('01-01-1900 00:00:00.0', 'DD-MM-YYYY HH24:mi:ssXFF'));
   IF p_lastupdttime IS NOT NULL
   THEN
      OPEN rc1
       FOR
          SELECT c.certid, c.partnerid, sslclientcertflg, digsigcertflg,
                 encryptcertflg, defflg, RTRIM (cn) cn, validfr, validto,
                 RTRIM (serialnbr) serialnbr, revokedflg, revokeddate, cert,
                 c.createdate, c.updatedate, c.enabledflg, rootcertflg,
                 PASSWORD, privatekey,c.certname,c.fingerprint
            FROM pr_cert c 
           WHERE c.updatedate > p_lastupdttime
             AND c.validfr < SYS_EXTRACT_UTC (SYSTIMESTAMP)
             AND c.validto > SYS_EXTRACT_UTC (SYSTIMESTAMP)
             AND c.enabledflg = 1
             AND c.revokedflg = 0
             AND c.rootcertflg = 1
             AND coalesce(c.deleteflag,0) <> 1;
   ELSE
      OPEN rc1
       FOR
          SELECT c.certid, c.partnerid, sslclientcertflg, digsigcertflg,
                 encryptcertflg, defflg, RTRIM (cn) cn, validfr, validto,
                 RTRIM (serialnbr) serialnbr, revokedflg, revokeddate, cert,
                 c.createdate, c.updatedate, c.enabledflg, rootcertflg,
                 PASSWORD, privatekey,c.certname,c.fingerprint
            FROM pr_cert c 
           WHERE c.validfr < SYS_EXTRACT_UTC (SYSTIMESTAMP)
             AND c.validto > SYS_EXTRACT_UTC (SYSTIMESTAMP)
             AND c.enabledflg = 1
             AND c.revokedflg = 0
             AND c.rootcertflg = 1
             AND coalesce(c.deleteflag,0) <> 1;
   END IF;

   p_returncd := 0;
   procaudit;
END pr_certs_ah5;
/

-- PR_CERTS_AH6.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_AH6 (
   p_returncd   OUT      INTEGER,
   p_debugflg   IN       PLS_INTEGER,
   rc1          OUT      Db_Ref.rct,
   p_FINGERPRINT   IN       VARCHAR2,
   p_SERIALNBR     IN       VARCHAR2
)
AS
/*
================================================================================
Proc Name:        PR_CERTS_AH6
Create Date:      2004-06-29
Author:           Shishir Narain
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_rowcnt       INTEGER   := 0;
   v_begints      TIMESTAMP := Fn_Get_Date;  -- Performance related variables
   v_updatedate   TIMESTAMP;

   PROCEDURE procaudit
   IS
   BEGIN
      Db_Procauditlogi_Ah1 ('PR_CERTS_AH6',
                            v_begints,
                               'p_ReturnCd'
                            || ','
                            || p_debugflg
                            || ','
                            || 'RC1'
                            || ','
                            || p_fingerprint
                            || ','
                            || p_serialnbr
                           );
   END procaudit;
BEGIN
--v_updateDate := nvl(p_UpdateDate, To_Timestamp('01-01-1900 00:00:00.0', 'DD-MM-YYYY HH24:mi:ssXFF'));
         OPEN rc1
       FOR
          SELECT 
          	  c.certid
          	, partnerid
          	, sslclientcertflg
          	, digsigcertflg
          	, encryptcertflg
                , defflg
                , RTRIM (cn) cn
                , validfr, validto
                , RTRIM (serialnbr) serialnbr
                , revokedflg
                , revokeddate
                , cert
                , c.createdate
                , c.updatedate
                , enabledflg
                , rootcertflg
                , PASSWORD
                , privatekey
                ,c.ROWTS
                ,c.SERIALNBR
            FROM pr_cert c
           where c.prisecflg <> -1
             AND enabledflg = 1
             AND revokedflg = 0
             AND c.SERIALNBR = p_serialnbr
             AND  FINGERPRINT = p_fingerprint
             AND coalesce(c.deleteflag,0) <> 1
;

   p_returncd := 0;
   procaudit;
END PR_CERTS_AH6;
/

-- PR_CERTS_AH7.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_AH7
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 out db_ref.RCT
	,p_certsetid IN NUMBER
	)
AS

/*


	================================================================================
	Proc Name:       PR_CERTS_AH7
	Create Date:     2007-09-19 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-19 BHARANI J    6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	List and show set of certificates (both primary and secondary) by given Certificate-Set. 
	The result set will list contents from PR_CERT by joining with PR_CERT_SET_LNK.
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERTS_AH7';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_certsetid=>'||p_certsetid||CHR(10);


BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	Open RC1 FOR 	SELECT 	a.CERTID,
				a.CERTNAME,
				a.prisecflg
		  	FROM  pr_cert a ,pr_cert_set_lnk l 
			where a.certid = l.certid
			AND coalesce(a.deleteflag,0) <> 1
			and l.certsetid = p_certsetid
			;
   
   	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   	
	EXCEPTION
		WHEN OTHERS then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERTS_AH7:'||SQLERRM);
			        
END PR_CERTS_AH7;
/
                     

-- PR_CERTS_AH8.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_AH8
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 out db_ref.RCT
	,p_partnerid IN NUMBER
	,p_prisecflg IN NUMBER
	,p_sslclientcertflg IN NUMBER
	,p_digsigcertflg IN NUMBER
	,p_encryptcertflg IN NUMBER
	)
AS

/*


	================================================================================
	Proc Name:       PR_CERTS_AH8
	Create Date:     2007-09-19 00:01:44
	Author:          BHARANI J 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-09-19  BHARANI J   6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	List certificates by given Certificate-usage. 
	The result set will list contents from PR_CERT by joining with PR_CERT_SET_LNK. 
	Certificates already part of a set will not be returned
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/



v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERTS_AH8';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_partnerid=>'||p_partnerid||CHR(10)
       ||', p_prisecflg=>'||p_prisecflg||CHR(10)
       ||', p_sslclientcertflg=>'||p_sslclientcertflg||CHR(10)
       ||', p_digsigcertflg=>'||p_digsigcertflg||CHR(10)
       ||', p_encryptcertflg=>'||p_encryptcertflg||CHR(10);

BEGIN

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	Open RC1 FOR 	SELECT 	a.certid,
				a.certname	
			FROM pr_cert a
			where a.partnerid = p_partnerid
			AND coalesce(a.deleteflag,0) <> 1
			AND a.prisecflg = p_prisecflg
			AND a.sslclientcertflg = p_sslclientcertflg
			AND a.digsigcertflg = p_digsigcertflg
			AND a.encryptcertflg = p_encryptcertflg
			AND NOT EXISTS 
				(SELECT 'x' from PR_CERT_SET_LNK b
				WHERE a.certid = b.certid)
			AND a.enabledflg=1
			AND a.validto > SYS_EXTRACT_UTC (SYSTIMESTAMP)
			;				
			
   
   	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   	
	EXCEPTION
	WHEN OTHERS then
					p_ReturnCd := 1;
					v_ExceptionFlg := 1;
					DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
					RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERTS_AH8:'||SQLERRM);
			        
	END PR_CERTS_AH8;
/





-- PR_CERTS_AH9.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_AH9
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 out db_ref.RCT
	,p_partnerid IN NUMBER
	,p_searchflg IN NUMBER -- Expects 1 or 2 
	)
AS

/*


	================================================================================
	Proc Name:       PR_CERTS_AH9
	Create Date:     2007-09-19 00:01:44
	Author:          AMIT KUMAR VIJAY

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-09-19 AMIT KUMAR    6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	List certificates by given partnerid. 
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERTS_AH9';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_partnerid=>'||p_partnerid||CHR(10)
       ||', p_searchflg=>'||p_searchflg||CHR(10)
       ;


    
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN    
	
		IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	Open RC1 FOR 	SELECT  a.certid,
			a.certname,
			a.sslclientcertflg,
			a.digsigcertflg,
			a.encryptcertflg,
			a.rootcertflg,
			a.ftpflag,
			a.sslserverflag,			
			FN_FIND_SETNAME(a.certid,P_PARTNERID,P_SEARCHFLG) CERTSETNAME,
			a.validfr,
			a.validto,
			a.prisecflg,
			FN_SET_DEFORNONDEF(b.certsetid,P_PARTNERID,P_SEARCHFLG) DEFAULTFLG,
			a.enabledflg		
			FROM PR_CERT_SET_LNK l
			right outer join pr_cert a on a.certid = l.certid 
			left  outer join pr_cert_set b on b.certsetid = l.certsetid
			WHERE a.partnerid = p_partnerid
			AND coalesce(a.deleteflag,0) <> 1
	;
	
	p_returncd := 0;
	
EXCEPTION
   			  			
   		WHEN OTHERS then
   			p_ReturnCd := 1;
   			v_ExceptionFlg := 1;
   			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
   			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERTS_AH9:'||SQLERRM);
   			
   			
END PR_CERTS_AH9;
/

   			   			
	
	
	
                 

-- PR_CERTS_ALL.sql

CREATE OR REPLACE PROCEDURE         PR_CERTS_ALL
(
    p_ReturnCd out integer,
    p_DebugFlg in pls_integer,
    RC1 out db_ref.RCT,
    p_UpdateDate in Timestamp default To_Timestamp('01-01-1900 00:00:00.0', 'DD-MM-YYYY HH24:mi:ssXFF')
) As
/*
================================================================================
Proc Name:        PR_CERTS_ALL
Create Date:     2002-05-08
Author:            S.Hutchinson
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2004-06-28 Shishir N	    4.3	PKEE-5VC24T: Added column PRISECFLG
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_updatedate Timestamp;
Procedure ProcAudit is
begin
--If p_DebugFlg =1 Then
DB_ProcAuditLogI_ah1('PR_CERTS_ALL',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
||','||p_UpdateDate);
--End if ;
End ProcAudit;
BEGIN
v_updateDate := nvl(p_UpdateDate, To_Timestamp('01-01-1900 00:00:00.0', 'DD-MM-YYYY HH24:mi:ssXFF'));
Open RC1 for
SELECT
    C.CERTID
,    C.PARTNERID
,   SSLCLIENTCERTFLG
,   DIGSIGCERTFLG
,   ENCRYPTCERTFLG
,    DEFFLG
,    RTRIM(CN)  CN
,    VALIDFR
,    VALIDTO
,    RTRIM(SERIALNBR)  SERIALNBR
,    REVOKEDFLG
,   REVOKEDDATE
,    CERT
,    C.CREATEDATE
,   C.UPDATEDATE
,C.ENABLEDFLG
,ROOTCERTFLG
,PRISECFLG
,PASSWORD
,PRIVATEKEY
From PR_CERT C
Inner Join PR_Partner P       On C.PartnerID = P.PartnerID
Inner Join PR_PartnerType PT  On P.PartnerTypeID = PT.PartnerTypeID
Where c.UpdateDate > v_UpdateDate
And   pt.OperatorFlg <> 1
AND coalesce(deleteflag,0) <> 1;
p_ReturnCd := 0;
procAudit;
--,    RTRIM(CERTNAME)  CERTNAME
--,   TYPEFLG
--,    RTRIM(L)  L
--,    RTRIM(ST)  ST
--,    RTRIM(O)  O
--,    RTRIM(OU)  OU
--,    RTRIM(C)  C
--,    RTRIM(CERTVER)  CERTVER
--,    RTRIM(FINGERPRINTALGO)  FINGERPRINTALGO
--,    RTRIM(FINGERPRINT)  FINGERPRINT
--,    RTRIM(PUBLICKEY)  PUBLICKEY
--,    ROWTS

End PR_CERTS_ALL;
/


-- PR_CERTS_FTP_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_FTP_AH1
(
	p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1			out		DB_REF.RCT
) AS
/*
	================================================================================
	Proc Name:       PR_CERTS_FTP_AH1
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_RowCnt          integer Default 0;
	v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
	v_ProcName        varchar2(30) Default 'PR_CERTS_FTP_AH1';
	v_ExceptionFlg    integer Default 0;
	v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 FOR
		SELECT	CERTID
			,PARTNERID
			,CERTNAME
			,DEFFLG
			,CN
			,L
			,ST
			,O
			,OU
			,C
			,VALIDFR
			,VALIDTO
			,CERTVER
			,FINGERPRINTALGO
			,FINGERPRINT
			,SERIALNBR
			,PUBLICKEY
			,REVOKEDFLG
			,REVOKEDDATE
			,CERT
			,ROWTS
			,CREATEDATE
			,UPDATEDATE
			,SSLCLIENTCERTFLG
			,DIGSIGCERTFLG
			,ENCRYPTCERTFLG
			,ENABLEDFLG
			,ROOTCERTFLG
			,PRISECFLG
			,PASSWORD
			,PRIVATEKEY
			,CERTDESCRIPTION
			,FTPFLAG
			,SSLSERVERFLAG
			,ISSUER_CN
			,ISSUER_L
			,ISSUER_ST
			,ISSUER_O
			,ISSUER_OU
			,ISSUER_C
		FROM	PR_CERT
		WHERE	SSLSERVERFLAG = 1
		AND coalesce(deleteflag,0) <> 1;

	p_returncd := 0;

EXCEPTION
	WHEN Others THEN
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
END;
/


-- PR_CERTS_FTP_AH2.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_FTP_AH2
(
	p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1			out		DB_REF.RCT
) AS
/*
	================================================================================
	Proc Name:       PR_CERTS_FTP_AH2
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_RowCnt          integer Default 0;
	v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
	v_ProcName        varchar2(30) Default 'PR_CERTS_FTP_AH2';
	v_ExceptionFlg    integer Default 0;
	v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 FOR
		SELECT	CERTID
			,c.PARTNERID
			,CERTNAME
			,DEFFLG
			,CN
			,L
			,ST
			,O
			,OU
			,C
			,VALIDFR
			,VALIDTO
			,CERTVER
			,FINGERPRINTALGO
			,FINGERPRINT
			,SERIALNBR
			,PUBLICKEY
			,REVOKEDFLG
			,REVOKEDDATE
			,CERT
			,c.ROWTS
			,c.CREATEDATE
			,c.UPDATEDATE
			,SSLCLIENTCERTFLG
			,DIGSIGCERTFLG
			,ENCRYPTCERTFLG
			,ENABLEDFLG
			,ROOTCERTFLG
			,PRISECFLG
			,PASSWORD
			,PRIVATEKEY
			,CERTDESCRIPTION
			,FTPFLAG
			,SSLSERVERFLAG
			,ISSUER_CN
			,ISSUER_L
			,ISSUER_ST
			,ISSUER_O
			,ISSUER_OU
			,ISSUER_C
		FROM	PR_CERT c
		INNER JOIN PR_PARTNER p ON c.partnerid = p.partnerid
		INNER JOIN PR_PARTNERTYPE pt ON pt.partnertypeid = p.partnertypeid
		WHERE	c.SSLCLIENTCERTFLG = 1
		AND coalesce(c.deleteflag,0) <> 1
		AND	c.FTPFLAG = 1
		AND	pt.partnertypeName = 'HubPartner'
		;

	p_returncd := 0;

EXCEPTION
	WHEN Others THEN
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
END;
/


-- PR_CERTS_FTP_AH3.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_FTP_AH3
(
	p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1			out		DB_REF.RCT
) AS
/*
	================================================================================
	Proc Name:       PR_CERTS_FTP_AH3
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_RowCnt          integer Default 0;
	v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
	v_ProcName        varchar2(30) Default 'PR_CERTS_FTP_AH3';
	v_ExceptionFlg    integer Default 0;
	v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 FOR
		SELECT	CERTID
			,c.PARTNERID
			,CERTNAME
			,DEFFLG
			,CN
			,L
			,ST
			,O
			,OU
			,C
			,VALIDFR
			,VALIDTO
			,CERTVER
			,FINGERPRINTALGO
			,FINGERPRINT
			,SERIALNBR
			,PUBLICKEY
			,REVOKEDFLG
			,REVOKEDDATE
			,CERT
			,c.ROWTS
			,c.CREATEDATE
			,c.UPDATEDATE
			,SSLCLIENTCERTFLG
			,DIGSIGCERTFLG
			,ENCRYPTCERTFLG
			,ENABLEDFLG
			,ROOTCERTFLG
			,PRISECFLG
			,PASSWORD
			,PRIVATEKEY
			,CERTDESCRIPTION
			,FTPFLAG
			,SSLSERVERFLAG
			,ISSUER_CN
			,ISSUER_L
			,ISSUER_ST
			,ISSUER_O
			,ISSUER_OU
			,ISSUER_C
		FROM	PR_CERT c
		INNER JOIN PR_PARTNER p ON c.partnerid = p.partnerid
		INNER JOIN PR_PARTNERTYPE pt ON pt.partnertypeid = p.partnertypeid
		WHERE	c.ROOTCERTFLG = 1
		AND coalesce(c.deleteflag,0) <> 1
		AND	pt.OperatorFlg = 1
		;

	p_returncd := 0;

EXCEPTION
	WHEN Others THEN
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
END;
/


-- PR_CERTS_FTP_AH4.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_FTP_AH4
(
	p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1			out		DB_REF.RCT
	, P_ISENCRYPT           in              INTEGER
	, P_ISSIGN              in 		INTEGER
	, P_ISSSSL              in              INTEGER
	, P_PRISECFLAG          in              INTEGER
	, P_DestName            in    		Varchar2
) AS
/*
	================================================================================
	Proc Name:       PR_CERTS_FTP_AH4
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_RowCnt          integer Default 0;
	v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
	v_ProcName        varchar2(30) Default 'PR_CERTS_FTP_AH4';
	v_ExceptionFlg    integer Default 0;
	v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', P_ISENCRYPT=>'||P_ISENCRYPT||CHR(10)
		||', P_ISSIGN=>'||P_ISSIGN||CHR(10)
		||', P_ISSSSL=>'||P_ISSSSL||CHR(10)
		||', P_PRISECFLAG=>'||P_PRISECFLAG||CHR(10)
		||', P_DestName=>'||P_DestName||CHR(10)	
		;
	v_partnerid       integer Default 0;	
	v_CERTSETNAME     varchar2(120);
	v_DestTypeId      integer;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%isOpen THEN Close RC1; END IF;
	
	
	SELECT partnerid into v_partnerid from pr_partner p 
	WHERE p.partnername='Hub Operator';
	
	v_DestTypeId := FN_GET_DESTTYPEID(P_DestName);
	
	OPEN RC1 FOR
		SELECT	 C.CERTID
			,C.PARTNERID
			,S.CERTSETID CERTSETID
			,S.CERTSETNAME CERTSETNAME
			,CERTNAME
			,DEFFLG
			,CN
			,L
			,ST
			,O
			,OU
			,C
			,VALIDFR
			,VALIDTO
			,CERTVER
			,FINGERPRINTALGO
			,FINGERPRINT
			,SERIALNBR
			,PUBLICKEY
			,REVOKEDFLG
			,REVOKEDDATE
			,CERT
			,C.ROWTS
			,C.CREATEDATE
			,C.UPDATEDATE
			,SSLCLIENTCERTFLG
			,DIGSIGCERTFLG
			,ENCRYPTCERTFLG
			,ENABLEDFLG
			,ROOTCERTFLG
			,PRISECFLG
			,PASSWORD
			,PRIVATEKEY
			,CERTDESCRIPTION
			,FTPFLAG
			,SSLSERVERFLAG
			,ISSUER_CN
			,ISSUER_L
			,ISSUER_ST
			,ISSUER_O
			,ISSUER_OU
			,ISSUER_C
		FROM	PR_CERT c
		LEFT OUTER JOIN PR_CERT_SET_LNK l ON c.certid=l.certid
		RIGHT OUTER JOIN PR_CERT_SET s ON l.certsetid=s.certsetid
		LEFT OUTER JOIN PR_CERTDESTTYPE d ON c.CERTID=d.CERTID
		WHERE	SSLCLIENTCERTFLG = P_ISSSSL 
		AND coalesce(c.deleteflag,0) <> 1
		AND c.DIGSIGCERTFLG = P_ISSIGN
		AND c.ENCRYPTCERTFLG = P_ISENCRYPT
		AND d.desttypeid=v_DestTypeId 
		AND c.PARTNERID = v_partnerid
		AND c.PRISECFLG = P_PRISECFLAG;

	p_returncd := 0;
	


EXCEPTION
	WHEN Others THEN
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
END;
/


-- PR_CERTS_OPERATOR.sql

CREATE OR REPLACE PROCEDURE         PR_CERTS_OPERATOR
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        )
AS

/*
        ================================================================================
        Proc Name:       PR_CERTS
        Create Date:     2003-07-21 11:53:54
        Author:          V.Goel
        Returns all the Certs for the 'Operator'
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2004-06-28 Shishir N	    4.3	PKEE-5VC24T: Added column PRISECFLG        
        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTS_OPERATOR';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    SELECT
        c.CERTID
        ,c.PARTNERID
        ,DEFFLG
        ,RTRIM(CN) CN
        ,VALIDFR
        ,VALIDTO
        ,RTRIM(SERIALNBR) SERIALNBR
        ,REVOKEDFLG
        ,CERT
        ,SSLCLIENTCERTFLG
        ,DIGSIGCERTFLG
        ,ENCRYPTCERTFLG
        ,ENABLEDFLG
        ,ROOTCERTFLG
        ,PRISECFLG
        ,PASSWORD
        ,D.DestName
        ,c.PrivateKey
        ,C.ROWTS
    FROM
        PR_CERT C
        Inner Join PR_Partner P       On C.PartnerID = P.PartnerID
        Inner Join PR_PartnerType PT  On P.PartnerTypeID = PT.PartnerTypeID
        Left Outer Join PR_CertDestType CD on c.CertID = CD.certID
        Left Outer Join PR_DestType D      on d.destTypeID = CD.destTypeID
    Where
            coalesce(c.deleteflag,0) <> 1
            AND pt.OperatorFlg = 1;       --PartnerTypeName = 'HubOperator'
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CERTS_OPERATOR;
/


-- PR_CERTS_OPERATOR_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_OPERATOR_AH1
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  Db_Ref.RCT
        , p_lastUpdttime IN TIMESTAMP
        , p_certsetid IN INTEGER
	, p_desttypename IN VARCHAR
	, p_prisecflg IN INTEGER
	, p_partnerid IN INTEGER
	, p_businessid IN VARCHAR2
	)
/*


	================================================================================
	Proc Name:       PR_CERTS_OPERATOR_AH1
	Create Date:     2002-04-24 16:12:30
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------ ------- ---------------------------------------------
	DATE       NAME         Ver	DESCRIPTION
	---------- ------------ ------- ---------------------------------------------
	2007-08-25 BHARANI J	6.1.1	Feature 55795
	2007-10-20 S.Priani	6.1.1 	105091f_1
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
AS
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'PR_CERTS_OPERATOR_AH1';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_lastUpdttime=>'||p_lastUpdttime||CHR(10)
                ;
v_desttypeid INTEGER;
v_partnerid INTEGER;
v_partnertypeid INTEGER;
v_Where      Varchar2(500);
v_where1     Varchar2(500);
v_sql_stmt   varchar2(4000) DEFAULT '';
v_sql_stmt1 varchar2(2000) DEFAULT '';
v_prisecflg INTEGER;
v_huboperatorid INTEGER;
v_join varchar2(2000) DEFAULT '';


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF p_desttypename IS NOT NULL THEN
		v_desttypeid := FN_GET_DESTTYPEID(p_desttypename);
	END IF;
	IF p_businessid IS NOT NULL THEN
		BEGIN
			SELECT	partnerid
			INTO	v_partnerid
			FROM	pr_busidentifier
			WHERE	businessid = p_businessid;
		Exception WHEN No_data_found then
			v_partnerid := p_partnerid;
		END;
	ELSE
		v_partnerid := p_partnerid;
	END IF;

/*
Fix as suggested by Jayp

When Certsetid is passed then there is a set and usage .  Look the data from 
pr_cert_set_usage table. 

When Certsetid is not passed then there is not set and usage. Certificates destination
details can be seen from pr_certdesttype table.

*/


	IF p_certsetid IS NOT NULL THEN
	
		v_join := ' FROM
       			 PR_CERT C
        		 LEFT OUTER JOIN PR_CERT_SET_LNK L ON c.certid = l.certid
			 LEFT OUTER JOIN PR_CERT_SET b ON b.certsetid = l.certsetid
        		 INNER JOIN PR_PARTNER P       ON C.PartnerID = P.PartnerID
        		 LEFT OUTER JOIN PR_CERT_SET_USAGE PCU ON b.certsetid = pcu.certsetid_sslclient
                                 	OR b.certsetid = pcu.certsetid_signing
                                 	OR b.certsetid = pcu.certsetid_encryption
       			 LEFT OUTER JOIN PR_DESTTYPE D      ON d.destTypeID = PCU.destTypeID 
   			 WHERE
	    		 c.enabledflg = 1
       	    		 AND c.revokedflg = 0 
       	    		 AND coalesce(c.deleteflag,0) <> 1 ';
       	    
		v_where := 
			FN_BUILD_WHERE('b.certsetid', 'NUMBER', p_certsetid)
			||FN_BUILD_WHERE('prisecflg' , 'NUMBER', p_prisecflg)
		 	;
	ELSE
		v_prisecflg := -1;
		
		v_join := ' FROM
        		PR_CERT C
        		LEFT OUTER JOIN PR_CERT_SET_LNK L ON c.certid = l.certid
			LEFT OUTER JOIN PR_CERT_SET b ON b.certsetid = l.certsetid
        		INNER JOIN PR_PARTNER P       ON C.PartnerID = P.PartnerID
        		LEFT OUTER JOIN PR_CERTDESTTYPE CD ON c.CertID = CD.certID
        		LEFT OUTER JOIN PR_DESTTYPE D      ON d.destTypeID = CD.destTypeID
   		WHERE
	    		c.enabledflg = 1
       	   		AND c.revokedflg = 0 
       	   		AND coalesce(c.deleteflag,0) <> 1 ';
		
		v_where := 
			FN_BUILD_WHERE('d.desttypeid', 'NUMBER', v_desttypeid)
			||FN_BUILD_WHERE('prisecflg' , 'NUMBER', v_prisecflg)
			||FN_BUILD_WHERE(' p.partnerid', 'NUMBER', v_partnerid)
		;
		
	
		
	END IF;

	-- v_sql_stmt is to pick partner certificate
	
	
   	v_sql_stmt := ' SELECT
         c.CERTID
        ,c.PARTNERID
        ,c.DEFFLG
        ,RTRIM(CN) CN
        ,VALIDFR
        ,VALIDTO
        ,RTRIM(SERIALNBR) SERIALNBR
        ,REVOKEDFLG
        ,CERT
        ,SSLCLIENTCERTFLG
        ,DIGSIGCERTFLG
        ,ENCRYPTCERTFLG
        ,ENABLEDFLG
        ,ROOTCERTFLG
        ,PASSWORD
        ,D.DestName
        ,c.PrivateKey
        ,c.ROWTS
	,c.PRISECFLG
    		' ;

	v_sql_stmt :=  v_sql_stmt || v_join || v_where;
	
	-- v_sql_stmt1 is to pick Hub Operator certificate. Pick only if passed set id is null
	
	IF p_certsetid IS NULL THEN
	
	v_sql_stmt1 := ' UNION ALL SELECT
         c.CERTID
        ,c.PARTNERID
        ,c.DEFFLG
        ,RTRIM(CN) CN
        ,VALIDFR
        ,VALIDTO
        ,RTRIM(SERIALNBR) SERIALNBR
        ,REVOKEDFLG
        ,CERT
        ,SSLCLIENTCERTFLG
        ,DIGSIGCERTFLG
        ,ENCRYPTCERTFLG
        ,ENABLEDFLG
        ,ROOTCERTFLG
        ,PASSWORD
        ,D.DestName
        ,c.PrivateKey
        ,c.ROWTS
	,c.PRISECFLG
    FROM
        PR_CERT C
        LEFT OUTER JOIN PR_CERT_SET_LNK L ON c.certid = l.certid
	LEFT OUTER JOIN PR_CERT_SET b ON b.certsetid = l.certsetid
        Inner Join PR_PARTNER P       ON C.PartnerID = P.PartnerID
        Left Outer Join PR_CERTDESTTYPE CD ON c.CertID = CD.certID
        Left Outer Join PR_DESTTYPE D      ON d.destTypeID = CD.destTypeID
   WHERE
	coalesce(c.deleteflag,0) <> 1
	AND c.enabledflg = 1
       	AND c.revokedflg = 0 ' ;
       	
       	SELECT  p.PARTNERTYPEid INTO v_partnertypeid FROM pr_partner p, pr_partnertype pt
		WHERE p.partnertypeid = pt.partnertypeid
	AND pt.partnertypename='HubOperator';
	
	V_Where1 := FN_BUILD_WHERE('d.desttypeid', 'NUMBER', v_desttypeid)
		|| FN_BUILD_WHERE('p.partnertypeid', 'NUMBER', v_partnertypeid) ;
	
	v_sql_stmt1 := v_sql_stmt1 || V_WHERE1;
	
	END IF;
	
	v_sql_stmt := v_sql_stmt || v_sql_stmt1;

   	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sql_stmt ;


    p_ReturnCd := 0;
    Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END PR_CERTS_OPERATOR_AH1;
/

-- PR_CERTS_OPERATOR_AH2.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_OPERATOR_AH2
(
	p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1			out		DB_REF.RCT
	, p_ChannelId           in              INTEGER
	, p_DestTypeName	in		VARCHAR2
	, p_EncryptFlg		in		INTEGER
	, p_DigSigFlg		in		INTEGER
	, p_SSLClientFlg	in		INTEGER
	, p_PriSecFlg		in		INTEGER
) AS
/*
	================================================================================
	Proc Name:       PR_CERTS_OPERATOR_AH2
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
        2008-02-27  BHARANI J     6.1.1  105966
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_RowCnt          integer Default 0;
	v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
	v_ProcName        varchar2(30) Default 'PR_USERS_AH3';
	v_ExceptionFlg    integer Default 0;
	v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ChannelId=>'||p_ChannelId||CHR(10)
		||', p_DestTypeName=>'||p_DestTypeName||CHR(10)
		||', p_EncryptFlg=>'||p_EncryptFlg||CHR(10)
		||', p_DigSigFlg=>'||p_DigSigFlg||CHR(10)
		||', p_SSLClientFlg=>'||p_SSLClientFlg||CHR(10)
		||', p_PriSecFlg=>'||p_PriSecFlg||CHR(10)
		;

	v_RowTS			integer :=0;
	v_frpartnerid		PR_PARTNER.PARTNERID%TYPE;
	v_topartnerid		PR_PARTNER.PARTNERID%TYPE;
	v_frroutingobjid	CF_ROUTINGOBJECT.ROUTINGOBJID%TYPE;
	v_toroutingobjid	CF_ROUTINGOBJECT.ROUTINGOBJID%TYPE;
	v_desttypeid            PR_DESTTYPE.desttypeid%TYPE;
	v_frroutingobjname      CF_ROUTINGOBJECT.ROUTINGOBJNAME%TYPE;
	v_toroutingobjname      CF_ROUTINGOBJECT.ROUTINGOBJNAME%TYPE;
	v_temppartnerid        NUMBER;
	v_temproutingobjid     NUMBER;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	v_DestTypeId := FN_GET_DESTTYPEID(p_DestTypeName);

	WITH RTOBJVW AS
			(SELECT	R.ROUTINGOBJLEVEL,
				R.ROUTINGOBJTYPEID,
				R.ROUTINGOBJID,
				F.ROUTINGOBJREFID,
				F.ROUTINGOBJPARENTREFID,
				CASE R.ROUTINGOBJLEVEL
					WHEN 1 THEN R.ROUTINGOBJID
					WHEN 2 THEN R2.ROUTINGOBJID
					WHEN 3 THEN R3.ROUTINGOBJID
					WHEN 4 THEN R4.ROUTINGOBJID
					WHEN 5 THEN R5.ROUTINGOBJID
					WHEN 6 THEN R6.ROUTINGOBJID
					ELSE -1
				END PackageID
			FROM	cf_routingobject01_vw R
			INNER JOIN CF_ROUTINGOBJREF_LNK F ON F.ROUTINGOBJID = R.ROUTINGOBJID
			LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F2 ON F2.ROUTINGOBJREFID = F.ROUTINGOBJPARENTREFID
			LEFT OUTER JOIN cf_routingobject01_vw R2 ON R2.ROUTINGOBJID = F2.ROUTINGOBJID
			LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F3 ON F3.ROUTINGOBJREFID = F2.ROUTINGOBJPARENTREFID
			LEFT OUTER JOIN cf_routingobject01_vw R3 ON R3.ROUTINGOBJID = F3.ROUTINGOBJID
			LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F4 ON F4.ROUTINGOBJREFID = F3.ROUTINGOBJPARENTREFID
			LEFT OUTER JOIN cf_routingobject01_vw R4 ON R4.ROUTINGOBJID = F4.ROUTINGOBJID
			LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F5 ON F5.ROUTINGOBJREFID = F4.ROUTINGOBJPARENTREFID
			LEFT OUTER JOIN cf_routingobject01_vw R5 ON R5.ROUTINGOBJID = F5.ROUTINGOBJID
			LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F6 ON F6.ROUTINGOBJREFID = F5.ROUTINGOBJPARENTREFID
			LEFT OUTER JOIN cf_routingobject01_vw R6 ON R6.ROUTINGOBJID = F6.ROUTINGOBJID
			)
		SELECT	pc.frpartnerid
			, pc.topartnerid
			, frrol.Packageid as frroutingobjid
			, torol.Packageid as toroutingobjid
		INTO	v_topartnerid
			,v_frpartnerid
			,v_toroutingobjid
			,v_frroutingobjid
		FROM	pr_routingchannel pc	
		INNER JOIN pr_capability frcap ON frcap.capabilityid = pc.frcapabilityid
		INNER JOIN pr_capability tocap ON tocap.capabilityid = pc.tocapabilityid
		INNER JOIN rtobjvw frrol ON frcap.routingobjrefid = frrol.routingobjrefid
		INNER JOIN rtobjvw torol ON tocap.routingobjrefid = torol.routingobjrefid
		WHERE	pc.channelid = p_channelid
	;
	
	SELECT routingobjname into v_frroutingobjname from cf_routingobject01_vw where 
					  routingobjid = v_frroutingobjid;
					  
	SELECT routingobjname INTO v_toroutingobjname from cf_routingobject01_vw where 
					  routingobjid = v_toroutingobjid;
	
	-- FIX FOR RNIF 2 WAY PIP.
	
	IF v_frroutingobjname ='RNIF' AND v_toroutingobjname='Backend Integration' THEN
		
			v_temppartnerid := v_frpartnerid;
			v_frpartnerid := v_topartnerid;
			v_topartnerid := v_temppartnerid;
			v_temproutingobjid := v_frroutingobjid;
			v_frroutingobjid := v_toroutingobjid;
			v_toroutingobjid := v_temproutingobjid;
		
	END IF;

	

	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 FOR
		WITH	CSLVL  AS
		(SELECT	desttypeid
			,CASE WHEN p_encryptflg = 1 THEN pcu.certsetid_encryption ELSE -1 END As certsetid_encryption
			,CASE WHEN p_digsigflg = 1 THEN pcu.certsetid_signing ELSE -1 END As certsetid_signing
			,CASE WHEN p_sslclientflg = 1 THEN pcu.certsetid_sslclient ELSE -1 END As certsetid_sslclient
			,case when pp.frpartnerid is null then 0 else 1000 end
			 + case when pp.topartnerid is null then 0 else 100 end
			 + case when rp.toroutingobjid is null then 0 else 10 end
			 + case when rp.toroutingobjid is null then 0 else 1 end AS sortlevel
		FROM	pr_cert_set_usage pcu
		INNER JOIN pr_partnerpair_rtobjpair_lnk ppl ON pcu.partnerrtobjlnkid = ppl.partnerrtobjlnkid
		INNER JOIN pr_partner_pair pp on pp.partnerpairid = ppl.Partnerpairid
		INNER JOIN pr_routingobj_pair rp on rp.routingobjpairid = ppl.routingobjpairid
		WHERE	COALESCE(pp.frpartnerid,v_frpartnerid) = v_frpartnerid
		AND	COALESCE(pp.topartnerid,v_topartnerid) = v_topartnerid
		AND	COALESCE(rp.frroutingobjid,v_frroutingobjid) = v_frroutingobjid
		AND	COALESCE(rp.toroutingobjid,v_toroutingobjid) = v_toroutingobjid
		AND	 pcu.desttypeid = v_desttypeid
		)
		SELECT	c.CERTID
			,c.PARTNERID
			,CERTNAME
			,DEFFLG
			,CN
			,L
			,ST
			,O
			,OU
			,C
			,VALIDFR
			,VALIDTO
			,CERTVER
			,FINGERPRINTALGO
			,FINGERPRINT
			,SERIALNBR
			,PUBLICKEY
			,REVOKEDFLG
			,REVOKEDDATE
			,CERT
			,c.ROWTS
			,c.CREATEDATE
			,c.UPDATEDATE
			,SSLCLIENTCERTFLG
			,DIGSIGCERTFLG
			,ENCRYPTCERTFLG
			,ENABLEDFLG
			,ROOTCERTFLG
			,PRISECFLG
			,PASSWORD
			,PRIVATEKEY
			,CERTDESCRIPTION
			,FTPFLAG
			,SSLSERVERFLAG
			,ISSUER_CN
			,ISSUER_L
			,ISSUER_ST
			,ISSUER_O
			,ISSUER_OU
			,ISSUER_C
			,pcs.certsetname
			,pcs.certsetid
		FROM	PR_CERT c
		INNER JOIN pr_cert_set_lnk pcsl ON pcsl.certid = c.certid
		INNER JOIN (SELECT	b.desttypeid
				,b.ENC AS encryption_flg
				,x.certsetid_encryption
				,b.SGN AS signing_flg
				,y.certsetid_signing
				,b.SSL AS sslclient_flg
				,z.certsetid_sslclient
			FROM	(SELECT	desttypeid, 
					MAX(CASE WHEN certsetid_encryption IS NOT NULL THEN sortlevel ELSE -1 END) AS ENC
					,MAX(CASE WHEN certsetid_signing IS NOT NULL THEN sortlevel ELSE -1 END) AS SGN
					,MAX(CASE WHEN certsetid_sslclient IS NOT NULL THEN sortlevel ELSE -1 END) AS SSL
				FROM	CSLVL
				GROUP BY desttypeid) B
			LEFT OUTER JOIN CSLVL x ON x.sortlevel = b.ENC AND x.desttypeid = b.desttypeid
			LEFT OUTER JOIN CSLVL y ON y.sortlevel = b.SGN AND y.desttypeid = b.desttypeid
			LEFT OUTER JOIN CSLVL z ON z.sortlevel = b.SSL AND z.desttypeid = b.desttypeid
			) cs ON pcsl.certsetid IN (cs.certsetid_encryption, cs.certsetid_signing, cs.certsetid_sslclient)
		INNER JOIN pr_cert_set pcs ON pcsl.certsetid = pcs.certsetid
		WHERE c.prisecflg = p_PriSecFlg
		AND coalesce(c.deleteflag,0) <> 1
		AND c.enabledflg = 1
	;

	p_returncd := 0;
    	
EXCEPTION
	WHEN OTHERS then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
END;
/


-- PR_CERTS_PARTNERID.sql

CREATE OR REPLACE PROCEDURE         PR_CERTS_partnerid
(p_ReturnCd out integer, p_DebugFlg in pls_integer, RC1 out db_ref.RCT
, p_Partnerid in NUMBER
) As
/*
================================================================================
Proc Name:        PR_CERTS_partnerid
Create Date:     2002-04-24 16:12:30
Author:            SYSTEM
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
---------- ------------- ------ ---------------------------------------------
DATE         NAME             Ver     DESCRIPTION
---------- ------------- ------ ---------------------------------------------
2002-04-29 Vgoel             3.2      Created
2002-05-09 Vgoel             3.2      Added Type Flag
2002-07-11 Vgoel             3.3      Add order by
2002-10-14 J.Chen            3.3      removed DestTypeId in resultset
2004-06-28 Shishir N	     4.3      PKEE-5VC24T: Added column PRISECFLG
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
--If p_DebugFlg =1 Then
DB_ProcAuditLogI_ah1('PR_CERTS_partnerid',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
||','||p_Partnerid
);
--End if ;
End ProcAudit;
BEGIN
Open RC1 for select
    c.CERTID
,    PARTNERID
,    RTRIM(CERTNAME)  CERTNAME
--,   TYPEFLG
,   DIGSIGCERTFLG
,   ENCRYPTCERTFLG
,    DEFFLG
,    RTRIM(CN)  CN
,    RTRIM(L)  L
,    RTRIM(ST)  ST
,    RTRIM(O)  O
,    RTRIM(OU)  OU
,    RTRIM(C)  C
,    VALIDFR
,    VALIDTO
,    RTRIM(CERTVER)  CERTVER
,    RTRIM(FINGERPRINTALGO)  FINGERPRINTALGO
,    RTRIM(FINGERPRINT)  FINGERPRINT
,    RTRIM(SERIALNBR)  SERIALNBR
,    RTRIM(PUBLICKEY)  PUBLICKEY
,   SSLCLIENTCERTFLG
,    REVOKEDFLG
,   REVOKEDDATE
,    CERT
,    c.ROWTS
,   ENABLEDFLG
,   ROOTCERTFLG
,   PRISECFLG
,   PASSWORD
,   PrivateKey
--,   dt.DestTypeId
From PR_CERT c
--Inner join pr_certDestType  d on c.CERTID = d.CERTID
--inner join pr_destType      dt on d.desttypeID = dt.desttypeID
Where Partnerid = p_Partnerid
AND coalesce(deleteflag,0) <> 1
Order by Upper(CERTNAME)
;
p_ReturnCd := 0;
procAudit;
/*
From PR_CERT c
Where c.CERTID = p_CERTID
*/
End PR_CERTS_partnerid;
/


-- PR_CERTS_PARTNERID_AH1.sql

CREATE OR REPLACE PROCEDURE pr_certs_partnerid_ah1 (
   p_returncd       OUT      INTEGER,
   p_debugflg       IN       PLS_INTEGER,
   rc1              OUT      Db_Ref.rct,
   p_partnerid      IN       NUMBER,
   p_lastupdttime   IN       TIMESTAMP,
   p_destypename   IN       VARCHAR
)
AS
/*


	================================================================================
	Proc Name:       PR_CERTS_PARTNERID_AH1
	Create Date:     2002-04-24 16:12:30
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_rowcnt    INTEGER   := 0;
   v_begints   TIMESTAMP := Fn_Get_Date;     -- Performance related variables
   v_desttypeid INTEGER;

   PROCEDURE procaudit
   IS
   BEGIN
--If p_DebugFlg =1 Then
      Db_Procauditlogi_Ah1 ('PR_CERTS_partnerid',
                            v_begints,
                               'p_ReturnCd'
                            || ','
                            || p_debugflg
                            || ','
                            || 'RC1'
                            || ','
                            || p_partnerid
                            || ','
                            || p_lastupdttime
                           );
--End if ;
   END procaudit;
BEGIN

BEGIN
select desttypeid into v_desttypeid from pr_desttype where destname=p_destypename;
EXCEPTION WHEN OTHERS THEN 
NULL;
END;


IF p_lastUpdttime IS NOT NULL THEN
   OPEN rc1
    FOR
       SELECT     c.certid
                , t.Destname
                , partnerid
                , RTRIM (certname) certname
                , digsigcertflg
		, encryptcertflg
		, defflg
		, RTRIM (cn) cn
                , RTRIM (l) l
		, RTRIM (st) st
		, RTRIM (o) o
		, RTRIM (ou) ou
                , RTRIM (c) c
		, validfr
		, validto
		, RTRIM (certver) certver
                , RTRIM (fingerprintalgo) fingerprintalgo
                , RTRIM (fingerprint) fingerprint
		, RTRIM (serialnbr) serialnbr
                , RTRIM (publickey) publickey
		, sslclientcertflg
		, revokedflg
                , revokeddate
		, cert
		, c.rowts
		, enabledflg
		, rootcertflg
                , PASSWORD
		, privatekey
		,c.ROWTS
		,c.PRISECFLG
       FROM     PR_CERT c
       Left Outer JOIN pr_CertDestType d on d.CertId = c.Certid
       Left Outer JOIN pr_DestType t on t.desttypeid = d.desttypeid
       WHERE    partnerid = p_partnerid
       AND c.updatedate > p_lastupdttime
       AND d.desttypeid =v_desttypeid
       AND c.enabledflg = 1
       AND c.revokedflg = 0
       AND c.prisecflg <> -1
       AND coalesce(c.deleteflag,0) <> 1
       ORDER BY c.prisecflg ;
ELSE
	OPEN rc1
	    FOR
	       SELECT     c.certid
	      	 	, t.Destname
			, partnerid
			, RTRIM (certname) certname
			, digsigcertflg
			, encryptcertflg
			, defflg
			, RTRIM (cn) cn
			, RTRIM (l) l
			, RTRIM (st) st
			, RTRIM (o) o
			, RTRIM (ou) ou
			, RTRIM (c) c
			, validfr
			, validto
			, RTRIM (certver) certver
			, RTRIM (fingerprintalgo) fingerprintalgo
			, RTRIM (fingerprint) fingerprint
			, RTRIM (serialnbr) serialnbr
			, RTRIM (publickey) publickey
			, sslclientcertflg
			, revokedflg
			, revokeddate
			, cert
			, c.rowts
			, enabledflg
			, rootcertflg
			, PASSWORD
			, privatekey
			,c.ROWTS
			,c.PRISECFLG
	       FROM     PR_CERT c
	       Left Outer JOIN pr_CertDestType d on d.CertId = c.Certid
       	 Left Outer JOIN pr_DestType t on t.desttypeid = d.desttypeid
	       WHERE    partnerid = p_partnerid
		AND d.desttypeid = v_desttypeid
	       AND c.enabledflg = 1
	       AND c.revokedflg = 0
	       AND c.prisecflg <> -1
	       AND coalesce(c.deleteflag,0) <> 1
       	ORDER BY c.prisecflg ;
END IF;
   p_returncd := 0;
   procaudit;
END pr_certs_partnerid_ah1;
/

-- PR_CERTS_PARTNERID_AH2.sql

CREATE OR REPLACE PROCEDURE PR_CERTS_PARTNERID_AH2 
(
   p_returncd       OUT      INTEGER,
   p_debugflg       IN       PLS_INTEGER,
   rc1              OUT      Db_Ref.rct,
   p_partnerid      IN       NUMBER,       
   p_lastupdttime   IN       TIMESTAMP,
   p_destypename   IN       VARCHAR,       --  Can be null or not null
   p_certsetid     IN       INTEGER        --  Can be null or not null
)
AS
/*                                                                             
    ===============================================================================
                                                                             
    Proc Name:       PR_CERTS_PARTNERID_AH2                                        
    Create Date:     2007-10-04 12:25:13                                           
    Author:          BHARANI J 
                                                      
                       ::::::::::::::::::::::::::::::                              
                             M O D I F I C A T I O N                               
                       ::::::::::::::::::::::::::::::
                             
    ---------- ------------- ------ ---------------------------------------------  
    DATE       NAME          VER    DESCRIPTION                                    
    ---------- ------------- ------ ---------------------------------------------  
    2006-02-16 BHARANI J      6.1.1 ERROR FLOW Feature
                                       
    ===============================================================================
                                                                              
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found         
    Used By Server
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_rowcnt    INTEGER   := 0;
   v_begints   TIMESTAMP := Fn_Get_Date;     -- Performance related variables
   v_desttypeid INTEGER;
   v_Where      Varchar2(500);
   v_sql_stmt   varchar2(1500);

 
BEGIN

	BEGIN
		SELECT desttypeid into v_desttypeid from pr_desttype 
		WHERE destname=p_destypename;
		
	EXCEPTION WHEN OTHERS THEN
		NULL;
	END;
	
	
	IF p_certsetid IS NOT NULL THEN
		v_where := FN_BUILD_WHERE('ps.certsetid' , 'NUMBER', p_certsetid);
	ELSE
		v_where := FN_BUILD_WHERE('c.partnerid', 'NUMBER', p_partnerid)
			||FN_BUILD_WHERE('d.desttypeid', 'NUMBER', v_desttypeid);
	END IF;
		
       
	v_sql_stmt := 
		' SELECT     c.certid
                , t.Destname
                , c.partnerid
                , RTRIM (certname) certname
                , digsigcertflg
		, encryptcertflg
		, sslclientcertflg
		, c.cert
		, c.rowts
		, c.PRISECFLG
	FROM     PR_CERT c
	Left Outer JOIN pr_CertDestType d on d.CertId = c.Certid
	Left Outer JOIN pr_DestType t on t.desttypeid = d.desttypeid
	LEFT OUTER JOIN pr_cert_set_lnk pcs on pcs.certid = c.certid
	LEFT OUTER JOIN pr_cert_set ps on pcs.CERTSETID = ps.CERTSETID 
	WHERE COALESCE(c.deleteflag,0) <> 1 
	AND c.enabledflg =1 ';
	
	v_sql_stmt := v_sql_stmt || v_Where;
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sql_stmt ;
     
	p_returncd := 0;
   
END PR_CERTS_PARTNERID_AH2;
/

-- PR_CERTU.sql

CREATE OR REPLACE PROCEDURE PR_CERTU
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_certid in number
	,p_PARTNERID in NUMBER
	,p_CERTNAME in VARCHAR2
	,p_DEFFLG in NUMBER
	,p_CN in VARCHAR2
	,p_L in VARCHAR2
	,p_ST in VARCHAR2
	,p_O in VARCHAR2
	,p_OU in VARCHAR2
	,p_C in VARCHAR2
	,p_VALIDFR in TIMESTAMP
	,p_VALIDTO in TIMESTAMP
	,p_CERTVER in VARCHAR2
	,p_FINGERPRINTALGO in VARCHAR2
	,p_FINGERPRINT in VARCHAR2
	,p_SERIALNBR in VARCHAR2
	,p_PUBLICKEY in VARCHAR2
	,p_REVOKEDFLG in NUMBER
	,p_REVOKEDDATE in TIMESTAMP
	,p_CERT in BLOB
	,p_SSLCLIENTCERTFLG in NUMBER
	,p_DIGSIGCERTFLG in NUMBER
	,p_ENCRYPTCERTFLG in NUMBER
	,p_ROWTS in NUMBER
	,p_ENABLEDFLG in NUMBER
	,p_ROOTCERTFLG in NUMBER
	,p_PRISECFLG in NUMBER
	,p_PASSWORD in RAW
	,p_PRIVATEKEY in BLOB
	,P_CERTDESCRIPTION in VARCHAR2
	,P_FTPFLAG in NUMBER
	,P_SSLSERVERFLAG in NUMBER
	,P_ISSUER_CN in VARCHAR2
	,P_ISSUER_L in VARCHAR2
	,P_ISSUER_ST in VARCHAR2
	,P_ISSUER_O in VARCHAR2
	,P_ISSUER_OU in VARCHAR2
	,P_ISSUER_C in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_CERTU
	Create Date:     2008-04-03 00:01:40
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2008-04-03 BHARANI J     1      Created for partner Migration
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CERTU';
v_ExceptionFlg    integer Default 0;
v_rowTS integer :=0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CERTID =>'||p_CERTID||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_CERTNAME=>'||p_CERTNAME||CHR(10)
		||', p_DEFFLG=>'||p_DEFFLG||CHR(10)
		||', p_CN=>'||p_CN||CHR(10)
		||', p_L=>'||p_L||CHR(10)
		||', p_ST=>'||p_ST||CHR(10)
		||', p_O=>'||p_O||CHR(10)
		||', p_OU=>'||p_OU||CHR(10)
		||', p_C=>'||p_C||CHR(10)
		||', p_VALIDFR=>'||p_VALIDFR||CHR(10)
		||', p_VALIDTO=>'||p_VALIDTO||CHR(10)
		||', p_CERTVER=>'||p_CERTVER||CHR(10)
		||', p_FINGERPRINTALGO=>'||p_FINGERPRINTALGO||CHR(10)
		||', p_FINGERPRINT=>'||p_FINGERPRINT||CHR(10)
		||', p_SERIALNBR=>'||p_SERIALNBR||CHR(10)
		||', p_PUBLICKEY=>'||p_PUBLICKEY||CHR(10)
		||', p_REVOKEDFLG=>'||p_REVOKEDFLG||CHR(10)
		||', p_REVOKEDDATE=>'||p_REVOKEDDATE||CHR(10)
		||', p_SSLCLIENTCERTFLG=>'||p_SSLCLIENTCERTFLG||CHR(10)
		||', p_DIGSIGCERTFLG=>'||p_DIGSIGCERTFLG||CHR(10)
		||', p_ENCRYPTCERTFLG=>'||p_ENCRYPTCERTFLG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
		||', p_ROOTCERTFLG=>'||p_ROOTCERTFLG||CHR(10)
		||', p_PRISECFLG=>'||p_PRISECFLG||CHR(10)
		||', p_PASSWORD=>'||p_PASSWORD||CHR(10)
		||', P_CERTDESCRIPTION=>'||P_CERTDESCRIPTION||CHR(10)
		||', P_FTPFLAG=>'||P_FTPFLAG||CHR(10)
		||', P_SSLSERVERFLAG=>'||P_SSLSERVERFLAG||CHR(10)
		||', P_ISSUER_CN=>'||P_ISSUER_CN||CHR(10)
		||', P_ISSUER_L=>'||P_ISSUER_L||CHR(10)
		||', P_ISSUER_ST=>'||P_ISSUER_ST||CHR(10)
		||', P_ISSUER_O=>'||P_ISSUER_O||CHR(10)
		||', P_ISSUER_OU=>'||P_ISSUER_OU||CHR(10)
		||', P_ISSUER_C=>'||P_ISSUER_C||CHR(10)
		;

zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Update PR_CERT SET
		PARTNERID = p_PARTNERID
		,CERTNAME = p_CERTNAME
		,DEFFLG = p_DEFFLG
		,CN = p_CN
		,L = p_L
		,ST = p_ST
		,O = p_O
		,OU = p_OU
		,C = p_C
		,VALIDFR = p_VALIDFR
		,VALIDTO = p_VALIDTO
		,CERTVER = p_CERTVER
		,FINGERPRINTALGO = p_FINGERPRINTALGO
		,FINGERPRINT = p_FINGERPRINT
		,SERIALNBR = p_SERIALNBR
		,PUBLICKEY =  p_PUBLICKEY
		,REVOKEDFLG = p_REVOKEDFLG
		,REVOKEDDATE = p_REVOKEDDATE
		,CERT = p_CERT
		,SSLCLIENTCERTFLG = p_SSLCLIENTCERTFLG
		,DIGSIGCERTFLG = p_DIGSIGCERTFLG
		,ENCRYPTCERTFLG = p_ENCRYPTCERTFLG
		,ENABLEDFLG = p_ENABLEDFLG
		,ROOTCERTFLG = p_ROOTCERTFLG
		,PRISECFLG = p_PRISECFLG
		,PASSWORD = p_PASSWORD
		,PRIVATEKEY = p_PRIVATEKEY
		,CERTDESCRIPTION = p_CERTDESCRIPTION
		,FTPFLAG = p_FTPFLAG
		,SSLSERVERFLAG = p_SSLSERVERFLAG
		,ISSUER_CN = p_ISSUER_CN
		,ISSUER_L = p_ISSUER_L
		,ISSUER_ST = p_ISSUER_ST
		,ISSUER_O = p_ISSUER_O
		,ISSUER_OU = p_ISSUER_OU
		,ISSUER_C = p_ISSUER_C
		,ROWTS            =   p_ROWTS + 1
		 Where CERTID = p_CERTID
		 AND coalesce(deleteflag,0) <> 1
    		 And ROWTS = p_ROWTS returning ROWTS into v_rowTS;
    
     v_rowcnt := SQL%rowcount;
     IF (v_rowcnt = 0) then  raise ZeroUpdate ; end if;
		
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		p_CERTID id,v_rowTS ROWTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_CERTU' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_CERTU' || SQLCODE|| ' '||SQLERRM);
		When ZEROUPDATE Then
			p_ReturnCd := 3;
			raise_application_error(-20999,'20101 - PR_CERTU_name: No Data Found');
END PR_CERTU;
/

-- PR_CERTU_NAME.sql

CREATE OR REPLACE PROCEDURE PR_CERTU_name
(
    p_ReturnCd          out integer
    ,p_DebugFlg         in pls_integer
    ,RC1                out db_ref.RCT
    ,p_CERTID           in pls_integer
    ,p_CERTNAME         in VARCHAR2
    ,p_SSLCLIENTCERTFLG In pls_integer
    ,p_DIGSIGCERTFLG    In pls_integer
    ,p_ENCRYPTCERTFLG   In pls_integer
    ,p_ROWTS            in pls_integer
    ,p_ENABLEDFLG       in pls_integer
    ,p_ROOTCERTFLG      in pls_integer
    ,p_PRISECFLG        in pls_integer
    ,P_CERTDESC         in VARCHAR2
    ,P_FTPFLAG          in pls_integer
    ,P_SSLSERVERFLAG    in pls_integer
 --   ,p_PASSWORD         in RAW
--,p_DestName  in Varchar2
) as
/*
================================================================================
Proc Name:        PR_CERTU_name
Create Date:     2002-05-02 16:35:14
Author:            SYSTEM
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
---------- ------------- ------ ---------------------------------------------
DATE         NAME             Ver     DESCRIPTION
---------- ------------- ------ ---------------------------------------------
2002-05-03 V Goel          3.2      Created, only name can change
2002-05-09 V Goel          3.2      Added TypeFlg
2002-10-08 V Goel          3.3      Remove TypeFlg and add three flags
2003-09-23 M. Mount        4.2.1    Removed commit. Fixed update date to v_BeginTS
2004-06-28 Shishir N	   4.3      PKEE-5VC24T: Added column PRISECFLG
2007-08-25 BHARANI J       6.1.1    Added certdesc
2008-01-01 BHARANI J       6.1.1    Added 2 new parameter for defect 105492
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_GET_DATE(); -- Performance related variables
v_rowTS integer :=0;
zeroUpdate  Exception;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_CERTID=>'||p_CERTID||CHR(10)
                ||', p_CERTNAME=>'||p_CERTNAME||CHR(10)
                ||', p_ROWTS=>'||p_ROWTS||CHR(10)
                ||', p_SSLCLIENTCERTFLG=>'||p_SSLCLIENTCERTFLG||CHR(10)
                ||', p_DIGSIGCERTFLG=>'||p_DIGSIGCERTFLG||CHR(10)
                ||', p_ENCRYPTCERTFLG=>'||p_ENCRYPTCERTFLG||CHR(10)
                ||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
                ||', p_PRISECFLG=>'||p_PRISECFLG||CHR(10)
                ||', p_ROOTCERTFLG=>'||p_ROOTCERTFLG||CHR(10)
                ||', P_CERTDESC=>'||P_CERTDESC||CHR(10)
                ||', P_FTPFLAG=>'||P_FTPFLAG||CHR(10)
                ||', P_SSLSERVERFLAG=>'||P_SSLSERVERFLAG||CHR(10);

--
-- procedures
--
Procedure ProcAudit is
begin

DB_ProcAuditLogI_ah1('PR_CERTU_name',v_beginTS, v_ParamsTxt, p_DebugFlg);
/*
'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
||','||p_CERTID
||','||p_CERTNAME
||','||p_SSLCLIENTCERTFLG
||','||p_DIGSIGCERTFLG
||','||p_ENCRYPTCERTFLG
||','||p_ROWTS
);
*/

End ProcAudit;
--
Procedure Update_Cert is
BEGIN
    v_rowcnt := 0;
    Update PR_CERT SET
           CERTNAME  = p_CERTNAME
    ,      SSLCLIENTCERTFLG =   p_SSLCLIENTCERTFLG
    ,      DIGSIGCERTFLG    =   p_DIGSIGCERTFLG
    ,      ENCRYPTCERTFLG   =   p_ENCRYPTCERTFLG
    ,      ENABLEDFLG       =   p_ENABLEDFLG
    ,      ROOTCERTFLG      =   p_ROOTCERTFLG
    ,      PRISECFLG        =   p_PRISECFLG
 --   ,      PASSWORD         =   p_PASSWORD
    ,      ROWTS            =   p_ROWTS + 1
    ,      UPDATEDATE       = v_BeginTS
    ,      certdescription  =  P_CERTDESC
    ,      ftpflag          =  P_FTPFLAG
    ,      sslserverflag    =  P_SSLSERVERFLAG 
    Where CERTID = p_CERTID
    AND coalesce(deleteflag,0) <> 1
    And ROWTS = p_ROWTS returning ROWTS into v_rowTS;
    v_rowcnt := SQL%rowcount;
    if (v_rowcnt = 0) then  raise ZeroUpdate ; end if;
EXCEPTION
    WHEN TOO_MANY_ROWS then ProcAudit;
        raise_application_error(-20999, '20100 - PR_CERTU_name: Too Many Rows');
    WHEN ZeroUpdate then ProcAudit;
        raise_application_error(-20999,'20101 - PR_CERTU_name: No Data Found');
    WHEN OTHERS then
        ProcAudit;
    ROLLBACK work;
        p_ReturnCd := 1;
        raise_application_error(-20999,'20102 - PR_CERTU_name:'||sqlerrm);
end Update_Cert;
--
/*
Function GetdestTypeID (p_Name in varchar2) return Number is
    v_destTypeID PR_DestType.DestTypeID%type;
Begin
    select destTypeID into v_destTypeID from pr_destTYpe
    where destName = p_destName;
    return v_destTypeID;
EXCEPTION
    WHEN NO_DATA_FOUND then
    ProcAudit;
    raise_application_error(-20999, '20100 - PR_CERTU_name - GetdestTypeID: DestName Does Not Exist');
    WHEN OTHERS then
    ProcAudit;
    ROLLBACK work;
    p_ReturnCd := 1;
    raise_application_error(-20999,'20100 - PR_CERTU_name - GetdestTypeID:'||sqlerrm);
end GetdestTypeID;
--
Procedure Update_certdestType is
v2_destTypeID    PR_DestType.DestTypeID%type;
v_CertDestTypeID PR_CertDestType.CertDestTypeID%type;
v_destName       PR_DestType.DestName%type;
v_rowTS          PR_DestType.RowTS%type;
Begin
    v2_destTypeID := GetdestTypeID (p_destName);
    select CertdestTypeID, rowTS into v_CertdestTypeID, v_rowTS
    from pr_CertDestType
    Where Certid = p_certid;
    if v_destName <> p_destName then
        Update pr_certDestType set desttypeID = v2_destTypeID
        where certid = p_certid;
    end if;
EXCEPTION
    WHEN NO_DATA_FOUND then
    ProcAudit;
    raise_application_error(-20999, '20100 - PR_CERTU_name - Update_certDestType: DestName Does Not Exist');
    WHEN OTHERS then
    ProcAudit;
    ROLLBACK work;
    p_ReturnCd := 1;
    raise_application_error(-20999,'20100 - PR_CERTU_name - GetdestTypeID:'||sqlerrm);
end Update_certdestType;
*/
--------------------------------------------------------
--              M  A  I  N
--------------------------------------------------------
Begin
    Update_Cert;
 --   Commit;
    OPEN RC1 for select v_rowTS ROWTS from Dual;
    p_ReturnCd := 0;
    ProcAudit;
End PR_CERTU_name;
/


-- PR_CERTU_PROMOTE.sql

CREATE OR REPLACE PROCEDURE PR_CERTU_PROMOTE
	(
	 p_ReturnCd   OUT   INTEGER
	,p_DebugFlg   IN   INTEGER
	,P_CERTSETID  INTEGER    
	)
AS


/*


	================================================================================
	Proc Name:       PR_CERTU_PROMOTE
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
        2008-08-29  BHARANI J     611FP1 106377
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


 v_BEGINTS           TIMESTAMP;   -- Performance related variables
 v_ExceptionFlg      INTEGER      DEFAULT 0;
 v_ProcName          VARCHAR (30) DEFAULT 'PR_CERTU_PROMOTE';
 v_sqlerrm           VARCHAR(255);
 v_ParamsTxt         VARCHAR(4000);
 v_Chkpt             CHAR(1)      DEFAULT '0';
 v_Msg              VARCHAR(1024);
 v_delpricertid     INTEGER;
 v_RowCnt          INTEGER Default 0;
 zeroUpdate  Exception;
 

BEGIN
     
     v_BEGINTS := FN_Get_Date();
     
     -- Store the primary certid into v_delpricertid to be deleted
         
        SELECT a.certid INTO v_delpricertid
                 FROM
                      PR_CERT a , PR_CERT_SET b , PR_CERT_SET_lnk l
                 WHERE
                  COALESCE(a.deleteflag,0) <> 1
                  AND b.certsetid = l.certsetid
     		  AND a.certid = l.certid
                  AND b.certsetid = P_CERTSETID
                  AND a.prisecflg = 1;
     
        UPDATE PR_CERT SET prisecflg=0,enabledflg=0 where certid = v_delpricertid;
         
   	DELETE FROM pr_cert_set_lnk where certid = v_delpricertid ;
	
	UPDATE PR_CERT SET prisecflg=1 where certid =
	(SELECT           
				l.certid
			FROM
		        PR_CERT a, PR_CERT_SET b, PR_CERT_SET_lnk l
			where
			a.certid = l.certid 
			AND coalesce(a.deleteflag,0) <> 1
			and b.certsetid = l.certsetid
			and b.certsetid = P_CERTSETID 
			and a.prisecflg = 2
			);
			
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;			
			
			
	p_ReturnCd := 0;			
			
   EXCEPTION
   
   	WHEN ZeroUpdate then
				v_ExceptionFlg := 1;
				p_ReturnCd := 3;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
   	
   	WHEN OTHERS then
   						ROLLBACK work;
   						p_ReturnCd := 1;
   						v_ExceptionFlg := 1;
   						DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
   						Commit;
				RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERTU_PROMOTE:'||SQLERRM);
END PR_CERTU_PROMOTE;
/

-- PR_CERTU_REVOKE.sql

CREATE OR REPLACE PROCEDURE PR_CERTU_REVOKE
	(
	 p_ReturnCd OUT INTEGER
	, p_DebugFlg IN PLS_INTEGER
	, RC1 OUT Db_Ref.RCT
	,p_CERTID IN NUMBER
	,p_ROWTS IN NUMBER
) AS

/*
	================================================================================
	Proc Name:       PR_CERTU_REVOKE
	Create Date:     2004-06-21 08:21:10
	Author:          Shishir Narain

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'PR_CERTU_REVOKE';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CERTID=>'||p_CERTID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      INTEGER :=0;
zeroUpdate  EXCEPTION;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	UPDATE PR_CERT
	SET
		CERTID=p_CERTID
		,      REVOKEDFLG=1
		,      ENABLEDFLG=0
		,      REVOKEDDATE=Fn_Get_Date
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= Fn_Get_Date
		WHERE  CertId = p_CertId
		AND RowTS = p_RowTS returning RowTS INTO v_RowTS;
	v_RowCnt := SQL%rowcount;
	IF (v_RowCnt = 0) THEN  RAISE ZeroUpdate ; END IF;
	IF RC1%isOpen THEN CLOSE RC1; END IF;
	OPEN RC1 FOR SELECT v_RowTS RowTS FROM Dual;
	p_ReturnCd := 0;
	Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS THEN
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_CERTU_REVOKE: Too Many Rows');
		WHEN ZeroUpdate THEN
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERTU_REVOKE: No Data Found');
		WHEN OTHERS THEN
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERTU_REVOKE:'||SQLERRM);
END PR_CERTU_REVOKE;
/


-- PR_CERTU_USAGE.sql

CREATE OR REPLACE PROCEDURE PR_CERTU_USAGE
(
    p_ReturnCd          out integer
    ,p_DebugFlg         in pls_integer
    ,RC1                out db_ref.RCT
    ,p_CERTID           in pls_integer
    ,p_ROWTS            in pls_integer
    ,p_PRISECFLG      in pls_integer
    ,p_ENABLEDFLG     in pls_integer
) as
/*
================================================================================
Proc Name:        PR_CERTU_USAGE
Create Date:     2004-06-25 16:35:14
Author:          Shishir N
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_GET_DATE(); -- Performance related variables
v_rowTS integer :=0;
zeroUpdate  Exception;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_CERTID=>'||p_CERTID||CHR(10)
                ||', p_ROWTS=>'||p_ROWTS||CHR(10)
                ||', p_PRISECFLG=>'||p_PRISECFLG||CHR(10)
                ||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10);

--
-- procedures
--
Procedure ProcAudit is
begin

DB_ProcAuditLogI_ah1('PR_CERTU_USAGE',v_beginTS, v_ParamsTxt, p_DebugFlg);
/*
'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
||','||p_CERTID
||','||p_CERTNAME
||','||p_SSLCLIENTCERTFLG
||','||p_DIGSIGCERTFLG
||','||p_ENCRYPTCERTFLG
||','||p_ROWTS
);
*/

End ProcAudit;
--
Procedure Update_Cert is
BEGIN
    v_rowcnt := 0;
    Update PR_CERT SET
           PRISECFLG      =   p_PRISECFLG
    ,      ENABLEDFLG    = p_ENABLEDFLG       
    ,      ROWTS            =   p_ROWTS + 1
    ,      UPDATEDATE       = v_BeginTS
    Where CERTID = p_CERTID
    And ROWTS = p_ROWTS returning ROWTS into v_rowTS;
    v_rowcnt := SQL%rowcount;
    if (v_rowcnt = 0) then  raise ZeroUpdate ; end if;
EXCEPTION
    WHEN TOO_MANY_ROWS then ProcAudit;
        raise_application_error(-20999, '20100 - PR_CERTU_USAGE: Too Many Rows');
    WHEN ZeroUpdate then ProcAudit;
        raise_application_error(-20999,'20101 - PR_CERTU_USAGE: No Data Found');
    WHEN OTHERS then
        ProcAudit;
    ROLLBACK work;
        p_ReturnCd := 1;
        raise_application_error(-20999,'20102 - PR_CERTU_USAGE:'||sqlerrm);
end Update_Cert;
--

--------------------------------------------------------
--              M  A  I  N
--------------------------------------------------------
Begin
    Update_Cert;
 --   Commit;
    OPEN RC1 for select v_rowTS ROWTS from Dual;
    p_ReturnCd := 0;
    ProcAudit;
End PR_CERTU_USAGE;
/


-- PR_CERT_SETD_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SETD_AH1
	(
	 p_ReturnCd  OUT PLS_INTEGER
	,p_DebugFlg IN  PLS_INTEGER
	,P_CERTSETID IN NUMBER
	,P_SEARCHFLG IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_CERT_SETD_AH1
	Create Date:     2007-09-25 15:16:24
	Author:          BHARANI J 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-25 BHARANI J     6.1.1  Created for Multiple Certificate Feature

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

 v_RowCnt          integer Default 0;
 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar2(30)  Default 'PR_CERT_SETD_AH1';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar2(30);
 v_ParamsTxt       varchar2(4000) :=
         'p_DebugFlg=>'||p_DebugFlg||CHR(10)
         ||', P_CERTSETID=>'||P_CERTSETID||CHR(10)
       ;
 v_SQLErrm         Varchar2(255) ;
 v_msg             Varchar2(500);
 v_RowTS           Integer Default 0;


			
BEGIN
	
	
	
	UPDATE PR_CERT_SET_USAGE SET
	certsetid_encryption=null
	Where certsetid_encryption=P_CERTSETID;
	
	UPDATE PR_CERT_SET_USAGE SET
	certsetid_signing=null
	Where certsetid_signing=P_CERTSETID;
	
	UPDATE PR_CERT_SET_USAGE SET
	certsetid_sslclient=null
	Where certsetid_sslclient=P_CERTSETID;
	
	Delete from PR_CERT_SET_USAGE where
	certsetid_encryption is null
	AND certsetid_signing is null
	AND certsetid_sslclient is null;
	
	
	UPDATE PR_CHANNEL_CERTSET_LNK SET
	certsetid_encryption=null
	where certsetid_encryption=P_CERTSETID;
		
	UPDATE PR_CHANNEL_CERTSET_LNK SET
	certsetid_signing=null
	Where certsetid_signing=P_CERTSETID;
		
	UPDATE PR_CHANNEL_CERTSET_LNK SET
	certsetid_sslclient=null
	Where certsetid_sslclient=P_CERTSETID;
		
	Delete from PR_CHANNEL_CERTSET_LNK where
	certsetid_encryption is null
	AND certsetid_signing is null
	AND certsetid_sslclient is null;
	
	IF P_SEARCHFLG = 0 THEN
		
		DELETE FROM PR_CERTdesttype
		WHERE CERTID IN 
		(select certid from PR_CERT_SET_LNK
		Where certsetid = P_CERTSETID);
		
		Update PR_CERT SET	         
		RowTS = RowTS + 1
		,enabledflg = 0
		,prisecflg = -1
		,SSLCLIENTCERTFLG = 0
		,DIGSIGCERTFLG = 0
		,ENCRYPTCERTFLG = 0
		,ROOTCERTFLG  = 0
		,FTPFLAG = 0
		WHERE CERTID IN
		(select certid from PR_CERT_SET_LNK
		Where certsetid = P_CERTSETID);
	
	END IF;
	
	DELETE FROM PR_CERT_SET_LNK 
	WHERE CERTSETID=P_CERTSETID
	;
	
	
	
	DELETE FROM PR_CERT_SET 
	WHERE CERTSETID=P_CERTSETID
	;
	
	
	p_returncd := 0;
	
	EXCEPTION
	   		WHEN NO_DATA_FOUND Then
	   			p_ReturnCd := 3;
	   			v_ExceptionFlg := 1;
	   			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'FOR NOT FOUND,'||v_ParamsTxt, p_DebugFlg, 1);
	   			  			
	   		WHEN OTHERS then
	   			p_ReturnCd := 1;
	   			v_ExceptionFlg := 1;
	   			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
	   			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SETD_AH1:'||SQLERRM);
   			
	
	
END PR_CERT_SETD_AH1;

/


-- PR_CERT_SETI.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SETI
	(
	p_ReturnCd OUT pls_integer
	,p_DebugFlg IN pls_integer
	,RC1 out db_ref.RCT
	,p_certsetname IN varchar2
	,p_certsetdesc IN varchar2
	,p_partnerid IN NUMBER
	,p_status IN NUMBER
	,p_encryptflg IN NUMBER
	,p_digsigflg IN NUMBER
	,p_sslclientflg IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_CERT_SETI
	Create Date:     2007-09-13 15:16:24
	Author:          Amit Kumar Vijay 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-13 BHARANI J       6.1.1  Created for Multiple Certificate Feature

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/



v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30)  Default 'PR_CERT_SETI';
v_ExceptionFlg    integer  Default 0;
v_function        VARCHAR2(50);
v_certsetid       integer;

v_ParamsTxt       varchar2(4000) Default
			'p_DebugFlg=>'||p_DebugFlg||CHR(10)
			||', p_certsetname=>'||p_certsetname||CHR(10)
			||', p_certsetdesc=>'||p_certsetdesc||CHR(10)
			||', p_partnerid=>'||p_partnerid||CHR(10)
			||', p_status=>'||p_status||CHR(10)
			||', p_encryptflg=>'||p_encryptflg||CHR(10)
			||', p_digsigflg=>'||p_digsigflg||CHR(10)
			||', p_sslclientflg=>'||p_sslclientflg||CHR(10)
			;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN



Insert into PR_CERT_SET
			(
			CERTSETID,	
			CERTSETNAME,
			CERTSETDESC,
			PARTNERID,
			STATUS,
			ENCRYPTFLG,
			DIGSIGFLG,
			SSLCLIENTFLG			
			) 
		Values 	
			(
			PR_CERT_SET_SEQ.nextval,
			p_certsetname,
			p_certsetdesc,
			p_partnerid,
			p_status,
			p_encryptflg,
			p_digsigflg,
			p_sslclientflg
			) returning CERTSETID into v_CERTSETID;
		v_RowCnt := sql%ROWCOUNT;
		
		
		
		p_ReturnCd := 0;
		
	IF RC1%isOpen THEN Close RC1; END IF;
		Open RC1 FOR Select
			v_certsetid CERTSETID,
       			1 ROWTS,
       			p_returncd Returnstatus From Dual; 
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
			When NO_DATA_FOUND THEN
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
				p_returnCd := 3;
			When DUP_VAL_ON_INDEX then
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
				p_returnCd := 2;
				Open RC1 FOR Select
				v_certsetid CERTSETID,
	     			1 ROWTS,
       				p_returncd Returnstatus From Dual; 
			When OTHERS Then
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				RAISE_APPLICATION_ERROR (-20999,'20002 PR_CERT_SETI' || SQLCODE|| ' '||SQLERRM);
		END PR_CERT_SETI;
/

-- PR_CERT_SETI_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SETI_AH1
	(
	p_ReturnCd OUT PLS_Integer
	,p_DebugFlg IN PLS_Integer
	,RC1 out db_ref.RCT
	,p_certsetname IN varchar2
	,p_certsetdesc IN varchar2
	,p_partnerid IN NUMBER
	,p_status IN NUMBER
	,p_encryptflg IN NUMBER
	,p_digsigflg IN NUMBER
	,p_sslclientflg IN NUMBER
	,p_pricertid IN NUMBER
	,p_seccertid IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CERT_SETI_AH1
	Create Date:     2007-09-19 15:16:24
	Author:          BHARANI J 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-19 BHARANI J    6.1.1  Created for Multiple Certificate Feature

	================================================================================
	-- This will call 2 procedure PR_CERT_SETI and PR_CERT_SET_LNKI_AH1.   
	-- PR_CERT_SETI procedure will populate pr_cert_set table 
        -- PR_CERT_SET_LNKI_AH1 procedure will populate pr_cert_set_lnk table.
	
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	4 CERTIFICATE EXISTS IN SOME OTHER SET 5 CERT AND SET DO NOT HAVE SAME PROPERTY
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERT_SETI_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_certsetname=>'||p_certsetname||CHR(10)
       ||', p_certsetdesc=>'||p_certsetdesc||CHR(10)
       ||', p_partnerid=>'||p_partnerid||CHR(10)
       ||', p_status=>'||p_status||CHR(10)
       ||', p_encryptflg=>'||p_encryptflg||CHR(10)
       ||', p_digsigflg=>'||p_digsigflg||CHR(10)
       ||', p_sslclientflg=>'||p_sslclientflg||CHR(10)
       ||', p_pricertid=>'||p_pricertid||CHR(10)
       ||', p_seccertid=>'||p_seccertid||CHR(10)
       ;
v_RowTS        integer :=0;

CERT_BELONGS_TO_ANOTHER_SET Exception;
CERT_SET_PROPERTY_MISMATCH Exception;

v_certsetlnkid1   PR_CERT_SET_LNK.CERTSETLNKID%TYPE DEFAULT NULL;
v_certsetlnkid2   PR_CERT_SET_LNK.CERTSETLNKID%TYPE DEFAULT NULL;

v_certsetid 	PR_CERT_SET.CERTSETID%TYPE DEFAULT NULL;
v_returncd        INTEGER;
v_set_returncd    INTEGER;
v_set_lnkreturncd INTEGER;
v_RC1 			db_ref.RCT;
v_RC2 			db_ref.RCT;


BEGIN
	
	PR_CERT_SETI(p_returncd, p_DebugFlg, v_RC1, p_certsetname,p_certsetdesc,
	                  p_partnerid,p_status,p_encryptflg,p_digsigflg,
	                  p_sslclientflg);	                  
	                  
	Fetch v_RC1 into v_certsetid,
        		 v_rowts,
        		 v_set_returncd;		 
	
	IF v_set_returncd = 2 THEN
		v_rowts := null;
		RAISE DUP_VAL_ON_INDEX;
	END IF;
	
	p_returncd := v_set_returncd;
	
	IF p_returncd = 0 AND v_certsetid IS NOT NULL THEN
	    
		PR_CERT_SET_LNKI_AH1(v_ReturnCd, p_DebugFlg, v_RC2, v_certsetid ,p_partnerid, p_pricertid ,
				     p_seccertid,p_encryptflg,p_digsigflg,p_sslclientflg);
				  
		
        	Fetch v_RC2 into v_certsetlnkid1,
        		 	 v_certsetlnkid2,
        		 	 v_set_lnkreturncd
        			 ;

			IF v_set_lnkreturncd = 4 THEN
				v_certsetid := null;
				v_rowts := null;
				RAISE CERT_BELONGS_TO_ANOTHER_SET;
			END IF;
	
			IF v_set_lnkreturncd = 5 THEN
				v_certsetid := null;
				v_rowts := null;
				RAISE CERT_SET_PROPERTY_MISMATCH;
			END IF;
	
		 p_returncd := v_set_lnkreturncd;
		 
		 IF RC1%ISOPEN THEN CLOSE RC1; END IF;
		 		    Open RC1 FOR SELECT v_certsetid  CERTSETID,
		 		    			v_rowts RowTS ,
		 		   			v_certsetlnkid1 CERTSETLNKPRIMARYID,
		 		   			v_certsetlnkid2 CERTSETLNKSECONDARYID
				 	FROM Dual;
				 
		ELSE
		
		IF RC1%ISOPEN THEN CLOSE RC1; END IF;
				    Open RC1 FOR SELECT v_certsetid  CERTSETID,
				    			v_rowts RowTS ,
				   			v_certsetlnkid1 CERTSETLNKPRIMARYID,
				   			v_certsetlnkid2 CERTSETLNKSECONDARYID
				    FROM Dual;
	        		 
		
	 END IF;
	
EXCEPTION
		WHEN CERT_BELONGS_TO_ANOTHER_SET Then
			p_ReturnCd := 4;
			v_ExceptionFlg :=1 ;
			ROLLBACK WORK;
			Open RC1 FOR SELECT v_certsetid  CERTSETID,
					    v_rowts RowTS ,
					    v_certsetlnkid1 CERTSETLNKPRIMARYID,
				            v_certsetlnkid2 CERTSETLNKSECONDARYID
				 	FROM Dual;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'CERT_BELONGS_TO_ANOTHER_SET,'||v_ParamsTxt, p_DebugFlg, 1);
			            
		WHEN CERT_SET_PROPERTY_MISMATCH then
			p_ReturnCd := 5;
			v_ExceptionFlg := 1;
			ROLLBACK WORK;
			Open RC1 FOR SELECT v_certsetid  CERTSETID,
					    v_rowts RowTS ,
					    v_certsetlnkid1 CERTSETLNKPRIMARYID,
					    v_certsetlnkid2 CERTSETLNKSECONDARYID
				 	FROM Dual;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'CERT_SET_PROPERTY_MISMATCH, '||v_ParamsTxt, p_DebugFlg, 1);
			
                
		WHEN DUP_VAL_ON_INDEX Then
			p_ReturnCd := 2;
		        v_ExceptionFlg :=1 ;
		        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, 1);
			            
		WHEN OTHERS then
			ROLLBACK work;
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			Commit;
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SETI_AH1:'||SQLERRM);
			        

END PR_CERT_SETI_AH1;
/


-- PR_CERT_SETS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SETS_AH1
	(
	 p_ReturnCd OUT     PLS_INTEGER
	,p_DebugFlg IN     PLS_INTEGER
	,RC1 out db_ref.RCT
	,P_PARTNERID IN   NUMBER
	,P_SEARCHFLG IN   NUMBER    --  1,2,3
	)
AS
/*


	================================================================================
	Proc Name:       PR_CERT_SETS_AH1
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	
	LIST SET OF CERTIFICATES BY GIVEN PARTNERID
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


 v_BEGINTS           TIMESTAMP;   -- Performance related variables
 v_ExceptionFlg      INTEGER      DEFAULT 0;
 v_ProcName          VARCHAR (30) DEFAULT 'PR_CERT_SETS_AH1';
 v_sqlerrm           VARCHAR(255);
 v_ParamsTxt         VARCHAR(4000);
 v_Chkpt             CHAR(1)      DEFAULT '0';
 v_Msg              VARCHAR(1024);

 v_sslclientflg INTEGER;
 v_digsigflg INTEGER;
 v_encryptflg INTEGER;

    BEGIN
    
     v_BEGINTS := FN_Get_Date();
    	   	
    	OPEN RC1  FOR
    		SELECT CERTSETID,
    		CERTSETNAME,
    		MAX(pricertid) PRICERTID,
    		MAX(seccertid) SECCERTID,
    		MAX(pricertname) pricertname,
    		MAX(seccertname) seccertname,
    		sslclientflg,
		digsigflg,
		encryptflg,
                STATUS,
                VALIDITY,
                DEFAULTFLG
                FROM
		(SELECT  b.certsetid,
			 b.certsetname,
			 case when a.prisecflg=1 then a.certid end AS pricertid ,
			 case when a.prisecflg=2 then a.certid end AS seccertid ,
			 case when a.prisecflg=1 then a.CERTNAME END AS pricertname,
			 case when a.prisecflg=2 then a.CERTNAME end AS seccertname,
		    	 b.sslclientflg,
		    	 b.digsigflg,
		    	 b.encryptflg,		    			    		
		    	 b.status,
			 FN_SET_VALORINVAL(b.certsetid)  VALIDITY,
			 FN_SET_DEFORNONDEF(b.certsetid,P_PARTNERID,P_SEARCHFLG) DEFAULTFLG
		 FROM pr_cert_set_lnk l
		 right outer join pr_cert_set b on b.certsetid = l.certsetid
		 left outer join pr_cert a on a.certid = l.certid AND coalesce(a.deleteflag,0) <> 1
		 where b.partnerid = P_PARTNERID
		 ) 
		GROUP BY CERTSETID,CERTSETNAME,sslclientflg,
		digsigflg,encryptflg,STATUS,VALIDITY,DEFAULTFLG
 	;
    	
    	p_ReturnCd := 0;
    	
      EXCEPTION
       
    		WHEN NO_DATA_FOUND then
       			p_ReturnCd := 3;
       			v_ExceptionFlg := 1;
       			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'FOR NOT FOUND,'||v_ParamsTxt, p_DebugFlg, 1);
       			  			
       		WHEN OTHERS then
       			p_ReturnCd := 1;
       			v_ExceptionFlg := 1;
       			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
       			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SETS_AH1:'||SQLERRM);
       			
       			
END PR_CERT_SETS_AH1;

/    	

-- PR_CERT_SETS_AH2.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SETS_AH2
	(
	 p_ReturnCd   OUT   PLS_INTEGER
	,p_DebugFlg  IN    PLS_INTEGER
	,RC1 out db_ref.RCT
	,P_PARTNERID IN  INTEGER
	,P_SEARCHFLG  IN INTEGER    --  1(ssl),2(sig),3(encryption)
	)
AS

/*


	================================================================================
	Proc Name:       PR_CERT_SETS_AH2
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


 v_BEGINTS           TIMESTAMP;   -- Performance related variables
 v_ExceptionFlg      INTEGER      DEFAULT 0;
 v_ProcName          VARCHAR (30) DEFAULT 'PR_CERT_SETS_AH2';
 v_sqlerrm           VARCHAR(255);
 v_ParamsTxt         VARCHAR(4000);
 v_Chkpt             CHAR(1)      DEFAULT '0';
 v_Msg              VARCHAR(1024);

 v_sslclientflg INTEGER;
 v_digsigflg INTEGER;
 v_encryptflg INTEGER;


BEGIN

	p_ReturnCd := 0;
	v_BEGINTS := FN_Get_Date();


    	IF P_SEARCHFLG = 1 THEN
    	
    		BEGIN
    	
    		OPEN RC1 FOR
			SELECT 	CERTSETID,
	    		CERTSETNAME
	    		FROM PR_CERT_SET 
	    	WHERE SSLCLIENTFLG = 1
	    	AND FN_SET_VALORINVAL(CERTSETID) = 0
	    	AND partnerid=P_PARTNERID;
	
		END;
	
	END IF;
	
	IF P_SEARCHFLG = 2 THEN
	    	
	    		BEGIN
	    	
	    		OPEN RC1 FOR
				SELECT 	CERTSETID,
		    		CERTSETNAME
		    		FROM PR_CERT_SET 
		    	WHERE digsigflg = 1
		    	AND FN_SET_VALORINVAL(CERTSETID) = 0
		    	AND partnerid=P_PARTNERID;
		    	
			
		
			END;
		
	END IF;
	
	IF P_SEARCHFLG = 3 THEN
		    	
		    		BEGIN
		    	
		    		OPEN RC1  FOR
					SELECT 	CERTSETID,
			    		CERTSETNAME
			    		FROM PR_CERT_SET 
			    	WHERE encryptflg = 1
			    	AND FN_SET_VALORINVAL(CERTSETID) = 0
			    	AND partnerid=P_PARTNERID;
			    	
				
			
				END;
			
		END IF;
	    
	    
    	 p_ReturnCd := 0;
    	 
 EXCEPTION
       
    		WHEN NO_DATA_FOUND then
       			p_ReturnCd := 3;
       			v_ExceptionFlg := 1;
       			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'FOR NOT FOUND,'||v_ParamsTxt, p_DebugFlg, 1);
       			  			
       		WHEN OTHERS then
       			p_ReturnCd := 1;
       			v_ExceptionFlg := 1;
       			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
       			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SETS_AH2:'||SQLERRM);
       			    	 
    	
END PR_CERT_SETS_AH2;
/

-- PR_CERT_SETS_AH3.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SETS_AH3
	(
	 p_ReturnCd    OUT  PLS_INTEGER
	, p_DebugFlg    IN  INTEGER
	, RC1 out db_ref.RCT
	, P_PARTNERID   IN INTEGER	
	, P_PRISECFLG  IN INTEGER    
	, P_SSLCLIENTCERTFLG IN INTEGER
	, P_DIGSIGCERTFLG IN INTEGER
	, P_ENCRYPTCERTFLG IN INTEGER
	, P_SEARCHFLG  IN INTEGER        -- 0 HubOperator (dispaly all sets) , 1 -- Partners
	)
AS
/*


	================================================================================
	Proc Name:       PR_CERT_SETS_AH3
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
        2008-01-01  BHARANI J     6.1.1  105492
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


 v_BEGINTS           TIMESTAMP;   -- Performance related variables
 v_ExceptionFlg      INTEGER      DEFAULT 0;
 v_ProcName          VARCHAR (30) DEFAULT 'PR_CERT_SETS_AH3';
 v_sqlerrm           VARCHAR(255);
 v_ParamsTxt         VARCHAR(4000);
 v_Chkpt             CHAR(1)      DEFAULT '0';
 v_Msg              VARCHAR(1024);
 v_where            VARCHAR(1000);
 v_where1            VARCHAR(1000);
 sql_stmt	   VARCHAR(2000);
 sql_group          VARCHAR(100);



BEGIN

     p_ReturnCd := 0;
     v_BEGINTS := FN_Get_Date();

	v_where := ' where 1=1 ';
	
	IF P_SEARCHFLG = 0 then     -- Hub Operator No Need to check any condition . Return all sets.
			v_where := v_where ||
			' AND a.prisecflg <> 2 ' ||
			FN_Build_Where('b.PARTNERID',  'NUMBER', P_PARTNERID);
	END IF;	
		
	IF P_SEARCHFLG = 1 then -- To
		 v_where := v_where ||
		FN_Build_Where('SSLCLIENTFLG',  'NUMBER', P_SSLCLIENTCERTFLG)||
		FN_Build_Where('DIGSIGFLG',   'NUMBER', P_DIGSIGCERTFLG )||
		FN_Build_Where('ENCRYPTFLG', 'NUMBER', P_ENCRYPTCERTFLG ) ||
		FN_Build_Where('b.PARTNERID',   'NUMBER', P_PARTNERID );
    	END IF;
    	
    	IF P_PRISECFLG = 1 THEN 
		    	
		    	 v_where1 :=  ' AND  b.certsetid NOT IN
					( select b.certsetid  from pr_cert_set_lnk l
					 right outer join pr_cert_set b on b.certsetid = l.certsetid
					 inner join pr_cert a on a.certid = l.certid and a.prisecflg = 1 AND coalesce(a.deleteflag,0) <> 1'|| 
					 FN_Build_Where('SSLCLIENTFLG',  'NUMBER', P_SSLCLIENTCERTFLG )||
				         FN_Build_Where('DIGSIGFLG',   'NUMBER', P_DIGSIGCERTFLG)||
					 FN_Build_Where('ENCRYPTFLG', 'NUMBER', P_ENCRYPTCERTFLG) ||
					 FN_Build_Where('b.PARTNERID',   'NUMBER', P_PARTNERID) || ')';
					 
			END IF;
	    	
	    	IF P_PRISECFLG = 2 THEN 
	    	
	    	 v_where1 :=  ' AND a.prisecflg <> 2
	    	 		AND  b.certsetid NOT IN
				( select b.certsetid  from pr_cert_set_lnk l
				 right outer join pr_cert_set b on b.certsetid = l.certsetid
				 inner join pr_cert a on a.certid = l.certid and a.prisecflg = 2 AND coalesce(a.deleteflag,0) <> 1'||
				 FN_Build_Where('SSLCLIENTFLG',  'NUMBER', P_SSLCLIENTCERTFLG )||
			         FN_Build_Where('DIGSIGFLG',   'NUMBER', P_DIGSIGCERTFLG )||
				 FN_Build_Where('ENCRYPTFLG', 'NUMBER', P_ENCRYPTCERTFLG) ||
				 FN_Build_Where('b.PARTNERID',   'NUMBER', P_PARTNERID ) || ')';
				 
	END IF;   
    	
    	IF P_PRISECFLG = 1 and P_SEARCHFLG <> 0 THEN
			 v_where := v_where || v_where1 ;
		END IF;
		
		IF P_PRISECFLG = 1 and P_SEARCHFLG = 0 THEN
			 v_where := v_where || v_where1 ;
		END IF;
		 
		 
		IF P_PRISECFLG = 2 and P_SEARCHFLG <> 0 THEN
		 	 v_where := v_where || v_where1 ;
		END IF;	 
		
		IF P_PRISECFLG = 2 and P_SEARCHFLG = 0 THEN
			 v_where := v_where || v_where1  ;
	END IF;	 
    		
    	 	
	 sql_stmt  :=
	
	'SELECT CERTSETID,
    		CERTSETNAME,
    		MAX(pricertid) PRICERTID,
    		MAX(seccertid) SECCERTID,
    		MAX(pricertname) pricertname,
    		MAX(seccertname) seccertname,
    		MAX(privalidto) privalidto,
    		MAX(secvalidto) secvalidto
                FROM
		(SELECT  b.certsetid,
			 b.certsetname,
			 case when a.prisecflg=1 then a.certid end AS pricertid ,
			 case when a.prisecflg=2 then a.certid end AS seccertid ,
			 case when a.prisecflg=1 then a.CERTNAME END AS pricertname,
			 case when a.prisecflg=2 then a.CERTNAME end AS seccertname,	    			    		
		    	 case when a.prisecflg=1 then a.validto end AS privalidto,
		    	 case when a.prisecflg=2 then a.validto end AS secvalidto
		 FROM pr_cert_set_lnk l
		 right outer join pr_cert_set b on b.certsetid = l.certsetid
		 left outer join pr_cert a on a.certid = l.certid AND coalesce(a.deleteflag,0) <> 1'
		 ;
        	
	 sql_group := ' )  A
        	GROUP BY CERTSETID,CERTSETNAME '
        ;
        	
	 sql_stmt := sql_stmt || v_where || sql_group;
	
	IF rc1%ISOPEN 	   THEN
		CLOSE rc1;
	END IF;
	
	OPEN rc1 FOR sql_stmt;
	
	EXCEPTION
           
        		WHEN NO_DATA_FOUND then
           			p_ReturnCd := 3;
           			v_ExceptionFlg := 1;
           			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'FOR NOT FOUND,'||v_ParamsTxt, p_DebugFlg, 1);
           			  			
           		WHEN OTHERS then
           			p_ReturnCd := 1;
           			v_ExceptionFlg := 1;
           			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
       			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SETS_AH3:'||SQLERRM);

END PR_CERT_SETS_AH3;
/

-- PR_CERT_SETU_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SETU_AH1
	(
	 p_ReturnCd OUT pls_integer
	,p_DebugFlg IN pls_integer
	,RC1 out db_ref.RCT
	,p_certsetname IN varchar2
	,p_certsetdesc IN varchar2
	,p_partnerid IN NUMBER
	,p_status IN NUMBER
	,p_encryptflg IN NUMBER
	,p_digsigflg IN NUMBER
	,p_sslclientflg IN NUMBER
	,p_rowts IN NUMBER
	,p_pricertid IN NUMBER
	,p_seccertid IN NUMBER
	,p_certsetid IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_CERT_SETU_AH1
	Create Date:     2007-09-18 15:16:24
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-18 AMIT KUMAR    6.1.1  Created for Multiple Certificate Feature

	================================================================================
	
	-- 1. This will Update pr_cert_set table 
	-- 2. If the certsetid already present in the pr_cert_set_lnk table they will be deleted.
	-- 3. New Rows will be created in the pr_cert_set_lnk for the primary certid and secondary
	--    certid.
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= ROWTS_NOT_MATCHING
	4 CERTIFICATE EXISTS IN SOME OTHER SET 5 CERT AND SET DO NOT HAVE SAME PROPERTY
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERT_SETU_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_certsetname=>'||p_certsetname||CHR(10)
       ||', p_certsetdesc=>'||p_certsetdesc||CHR(10)
       ||', p_partnerid=>'||p_partnerid||CHR(10)
       ||', p_status=>'||p_status||CHR(10)
       ||', p_encryptflg=>'||p_encryptflg||CHR(10)
       ||', p_digsigflg=>'||p_digsigflg||CHR(10)
       ||', p_sslclientflg=>'||p_sslclientflg||CHR(10)
       ||', p_rowts=>'||p_rowts||CHR(10)
       ||', p_pricertid=>'||p_pricertid||CHR(10)
       ||', p_seccertid=>'||p_seccertid||CHR(10)
       ||', p_certsetid=>'||p_certsetid||CHR(10)
       ;
v_RowTS        integer :=0;

CERT_BELONGS_TO_ANOTHER_SET Exception;
CERT_SET_PROPERTY_MISMATCH Exception;
ROWTS_NOT_MATCHING Exception;

v_certsetlnkid1   INTEGER;
v_certsetlnkid2   INTEGER;

v_certsetid INTEGER;

v_exists_pri   varchar2(1);
v_exists_sec   varchar2(1);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	
	
	-- First Update pr_cert_set table
	BEGIN
	
	select rowts into v_rowts from pr_cert_set
	where certsetid = p_certsetid;
	
	EXCEPTION
	       	When NO_DATA_FOUND THEN
	       	p_RETURNCD := 3; 
	       	raise;
        END;
	
	IF v_rowts = p_rowts THEN
	
	UPDATE pr_cert_set set
	certsetname = p_certsetname,
	certsetdesc = p_certsetdesc,
	encryptflg  = p_encryptflg,
	digsigflg = p_digsigflg,
	sslclientflg = p_sslclientflg,
	status = p_status,
	partnerid = p_partnerid,
	rowts = p_rowts + 1
	WHERE
	certsetid = p_certsetid
	AND rowts = p_rowts ;
	
	v_rowts := p_rowts + 1 ;
	
	ELSE 
		raise ROWTS_NOT_MATCHING;
	END IF;
		
	
		
	-- Delete the data in pr_cert_set_lnk table for given certsetid. 
	-- Later it will be inseted with the new details.
	
	DELETE FROM pr_cert_set_lnk 
	WHERE certsetid = p_certsetid;
	
		
	-- Insert two rows in pr_cert_set_lnk table.
	
	-- Before Inserting do all Validation.
	-- a) 1.  The certificate to be inserted should not be a part of another set
	-- b) 2.  The Certificate to be inserted have same usages and property.
	
	-- Validation 1
	
	-- Check the Primary Certificate belongs to some other Set.
		
	BEGIN
	
	SELECT certsetid 
	INTO v_certsetid
	FROM pr_cert a, pr_cert_set_lnk l
	WHERE a.certid = l.certid
	AND coalesce(a.deleteflag,0) <> 1
	AND l.certid = COALESCE(P_Pricertid,0)
	AND a.prisecflg = 1
	;
	
	EXCEPTION
	        	When NO_DATA_FOUND THEN
	        	NULL;        
        END;
	        
	        
	IF v_certsetid <> P_CERTSETID THEN
		raise  CERT_BELONGS_TO_ANOTHER_SET;
	END IF;
	
	-- Check the Secondary Certificate belongs to some other Set.
	BEGIN
	
	SELECT certsetid 
	INTO v_certsetid
	FROM pr_cert a, pr_cert_set_lnk l
	WHERE a.certid = l.certid
	AND coalesce(a.deleteflag,0) <> 1
	AND l.certid = COALESCE(P_SECCERTID,0)
	AND a.prisecflg = 2
	;
	
	EXCEPTION
	        	When NO_DATA_FOUND THEN
	        	NULL;        
	        
        END;
	       
	
	IF v_certsetid <> P_CERTSETID THEN
		raise  CERT_BELONGS_TO_ANOTHER_SET;
		-- The secondary Certificate belongs to some other set		
	END IF
	;
	-- Don't Insert in pr_cert_set_lnk table if Primary Certificateid id NULL
	
	IF P_Pricertid IS NOT NULL THEN
	      	
		-- Validation 2a
	
		-- For the given Primary Certid check the following
		-- a) Is it really a Primary Certificate
		-- b) Is the Primary Certificate property and set property matches
		-- c) Is the Primary Certificate and Set belongs to same Partner. 
		
		BEGIN
		
		SELECT 'x' into v_exists_pri
		FROM pr_cert
		WHERE certid = P_Pricertid
		AND coalesce(deleteflag,0) <> 1
		AND SSLCLIENTCERTFLG = p_sslclientflg
		AND DIGSIGCERTFLG = p_digsigflg
		AND ENCRYPTCERTFLG = p_encryptflg
		AND PARTNERID = p_partnerid
		AND PRISECFLG = 1;
		
		 EXCEPTION
		        	When NO_DATA_FOUND THEN
		        	NULL;        
		        
        	END;
		
		IF v_exists_pri = 'x' THEN
	       	
			Insert into PR_CERT_SET_LNK
				(
	       			CERTSETLNKID	
	       			,CERTSETID	
	       			,CERTID			
	       			) 
	       			Values 	
	       			(
	       			PR_CERT_SET_LNK_SEQ.nextval
	       			,P_CERTSETID
	       			,P_Pricertid
	       			)returning CERTSETLNKID into v_certsetlnkid1
				;
		ELSE
		
		raise  CERT_SET_PROPERTY_MISMATCH;
		
		END IF;
		
	END IF;
	
	-- Don't Insert in pr_cert_set_lnk table if Secondary Certificateid id NULL
		
	IF P_SECCERTID IS NOT NULL THEN
		
		
		-- For the given Secondary Certid check the following
		-- a) Is it really a Secondary Certificate
		-- b) Is the Secondary Certificate property and set property matches
		-- c) Is the Secondary Certificate and Set belongs to same Partner.
		
		BEGIN
		
		SELECT 'y' into v_exists_sec
		FROM pr_cert
		WHERE certid = P_SECCERTID
		AND coalesce(deleteflag,0) <> 1
		AND SSLCLIENTCERTFLG = p_sslclientflg
		AND DIGSIGCERTFLG = p_digsigflg
		AND ENCRYPTCERTFLG = p_encryptflg
		AND PARTNERID = p_partnerid
		AND PRISECFLG = 2;	
		
		 EXCEPTION
		        	When NO_DATA_FOUND THEN
		        	NULL;        
		        
        	END;
		
	       	IF v_exists_sec = 'y' THEN
	       	

	        	Insert into PR_CERT_SET_LNK
				(
		       		CERTSETLNKID	
		       		,CERTSETID	
		       		,CERTID			
		       		) 
		       		Values 	
		       		(
		       		PR_CERT_SET_LNK_SEQ.nextval
		       		,P_CERTSETID
		       		,P_SECCERTID
		       		)returning CERTSETLNKID into v_certsetlnkid2
			;
		ELSE
	        
	        	raise  CERT_SET_PROPERTY_MISMATCH;
	        
	        END IF;
	        
	END IF;
	        
	p_ReturnCd := 0;
		    v_RowCnt := sql%ROWCOUNT;
		    Open RC1 FOR SELECT p_certsetid CERTSETID,
					v_rowts RowTS ,
					v_certsetlnkid1 CERTSETLNKID1,
					v_certsetlnkid2 CERTSETLNKID2
				 FROM Dual;
	    p_ReturnCd := 0;
	    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	
	EXCEPTION
			WHEN CERT_BELONGS_TO_ANOTHER_SET Then
				p_ReturnCd := 4;
				v_ExceptionFlg :=1 ;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'CERT_BELONGS_TO_ANOTHER_SET,'||v_ParamsTxt, p_DebugFlg, 1);				
				ROLLBACK WORK;
				RAISE_APPLICATION_ERROR(-20999, '20100 - PR_CERT_SETU_AH1: CERT_BELONGS_TO_ANOTHER_SET');
				                
			WHEN CERT_SET_PROPERTY_MISMATCH then
				p_ReturnCd := 5;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'CERT_SET_PROPERTY_MISMATCH, '||v_ParamsTxt, p_DebugFlg, 1);
				ROLLBACK WORK;
				RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SETU_AH1: CERT_SET_PROPERTY_MISMATCH');
				
	                WHEN ROWTS_NOT_MATCHING Then
				p_ReturnCd := 3;
				v_ExceptionFlg :=1 ;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ROWTS_NOT_MATCHING,'||v_ParamsTxt, p_DebugFlg, 1);
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
				ROLLBACK WORK;
				RAISE_APPLICATION_ERROR(-20999, '20100 - PR_CERT_SETU_AH1: ROWTS_NOT_MATCHING'); 
				                
			WHEN DUP_VAL_ON_INDEX Then
				p_ReturnCd := 2;
			        v_ExceptionFlg :=1 ;
			        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, 1);
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
				ROLLBACK WORK;
				RAISE_APPLICATION_ERROR(-20999, '20100 - PR_CERT_SETU_AH1: DUP_VAL_ON_INDEX');     
				                
			WHEN OTHERS then
				ROLLBACK work;
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				Commit;
				RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SETU_AH1:'||SQLERRM);
		        
END PR_CERT_SETU_AH1;
/

-- PR_CERT_SET_LNKI_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SET_LNKI_AH1
	(
	p_ReturnCd  OUT PLS_Integer
	,p_DebugFlg IN PLS_Integer
	,RC1 out db_ref.RCT
	,P_CERTSETID IN NUMBER
	,p_partnerid IN NUMBER
	,P_Pricertid IN NUMBER
	,P_SECCERTID IN NUMBER
	,p_encryptflg IN NUMBER
	,p_digsigflg IN NUMBER
	,p_sslclientflg IN NUMBER	
	)
AS

/*
	================================================================================
	Proc Name:       PR_CERT_SET_LNKI_AH1
	Create Date:     2007-09-19 15:16:24
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-19 BHARANI J    6.1.1  Created for Multiple Certificate Feature

	================================================================================
	
	-- This procedure inserts 2 records in table.  One is primary certificate of the set
        -- and another is secondary certificate of the set.
	
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found 
	4 CERTIFICATE EXISTS IN SOME OTHER SET 5 CERT AND SET DO NOT HAVE SAME PROPERTY
	
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERT_SET_LNKI_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', P_CERTSETID=>'||P_CERTSETID||CHR(10)
       ||', P_Pricertid=>'||P_Pricertid||CHR(10)
       ||', p_partnerid=>'||p_partnerid||CHR(10)
       ||', P_Pricertid=>'||P_Pricertid||CHR(10)
       ||', P_SECCERTID=>'||P_SECCERTID||CHR(10)
       ||', p_encryptflg=>'||p_encryptflg||CHR(10)
       ||', p_digsigflg=>'||p_digsigflg||CHR(10)
       ||', p_sslclientflg=>'||p_sslclientflg||CHR(10)
       ;
v_RowTS        integer :=0;

v_certsetlnkid1   INTEGER;
v_certsetlnkid2   INTEGER;

v_certsetid INTEGER;

v_exists_pri   varchar2(1);
v_exists_sec   varchar2(1);

CERT_BELONGS_TO_ANOTHER_SET Exception;
CERT_SET_PROPERTY_MISMATCH Exception;
ROWTS_NOT_MATCHING Exception;

BEGIN

	-- Insert two rows in pr_cert_set_lnk table.
	
	-- Before Inserting do all Validation.
	-- a) 1.  The certificate to be inserted should not be a part of another set
	-- b) 2.  The Certificate to be inserted have same usages and property.
	
	-- Validation 1
	
	-- Check whether the Primary Certificate belongs to some other Set.
	
	BEGIN
	
	SELECT certsetid 
	INTO v_certsetid
	FROM pr_cert a, pr_cert_set_lnk l
	WHERE a.certid = l.certid
	AND coalesce(a.deleteflag,0) <> 1
	AND l.certid = COALESCE(P_Pricertid,0)
	AND a.prisecflg = 1
        ;
        
        EXCEPTION
        	When NO_DATA_FOUND THEN
        	NULL;        
        
        END;
                
	IF v_certsetid <> P_CERTSETID THEN
		-- The Primary Certificate belongs to some other set
		RAISE CERT_BELONGS_TO_ANOTHER_SET;
	END IF;
	
	-- Check whether the Secondary Certificate belongs to some other Set.
	BEGIN
	
	SELECT certsetid 
	INTO v_certsetid
	FROM pr_cert a, pr_cert_set_lnk l
	WHERE a.certid = l.certid
	AND coalesce(a.deleteflag,0) <> 1
	AND l.certid = COALESCE(P_SECCERTID,0)
	AND a.prisecflg = 2
	;
	
	EXCEPTION
	
		When NO_DATA_FOUND THEN
		NULL;
	END;
       
	IF v_certsetid <> P_CERTSETID THEN
		-- The secondary Certificate belongs to some other set
		RAISE CERT_BELONGS_TO_ANOTHER_SET;
				
	END IF
       	;
       	
       	-- Don't Insert in pr_cert_set_lnk table if Primary Certificateid id NULL
       	
       	IF P_Pricertid IS NOT NULL THEN
       	
       		-- Validation 2a
		-- For the given Primary Certid check the following
		-- a) Is it really a Primary Certificate
		-- b) Is the Primary Certificate property and set property matches
		-- c) Is the Primary Certificate and Set belongs to same Partner.
		
		BEGIN
      	
		SELECT 'x' into v_exists_pri
		FROM pr_cert
		WHERE certid = P_Pricertid
		AND coalesce(deleteflag,0) <> 1
		AND SSLCLIENTCERTFLG = p_sslclientflg
		AND DIGSIGCERTFLG = p_digsigflg
		AND ENCRYPTCERTFLG = p_encryptflg
		AND PARTNERID = p_partnerid
		AND PRISECFLG = 1;
		
		EXCEPTION
		
		When NO_DATA_FOUND THEN
		NULL;

		END;
	
       		IF v_exists_pri = 'x' THEN
       	
			DELETE FROM pr_cert_set_lnk where certsetid = P_CERTSETID 
			AND certid = P_Pricertid;
       
       
			Insert into PR_CERT_SET_LNK
       			(
       			CERTSETLNKID	
       			,CERTSETID	
       			,CERTID			
       			) 
       			Values 	
       			(
       			PR_CERT_SET_LNK_SEQ.nextval
       			,P_CERTSETID
       			,P_Pricertid
       			)RETURNING CERTSETLNKID INTO v_certsetlnkid1
			;
		v_RowCnt := sql%ROWCOUNT;
		p_ReturnCd := 0;
			
		ELSE
	
			RAISE CERT_SET_PROPERTY_MISMATCH;
	
		END IF;
	
	END IF;
	
	-- Don't Insert in pr_cert_set_lnk table if Secondary Certificateid id NULL
	
	IF P_SECCERTID IS NOT NULL THEN
	
		-- For the given Secondary Certid check the following
		-- a) Is it really a Secondary Certificate
		-- b) Is the Secondary Certificate property and set property matches
		-- c) Is the Secondary Certificate and Set belongs to same Partner.
		
		BEGIN
		
		SELECT 'y' into v_exists_sec
		FROM pr_cert
		WHERE certid = P_SECCERTID
		AND coalesce(deleteflag,0) <> 1
		AND SSLCLIENTCERTFLG = p_sslclientflg
		AND DIGSIGCERTFLG = p_digsigflg
		AND ENCRYPTCERTFLG = p_encryptflg
		AND PARTNERID = p_partnerid
		AND PRISECFLG = 2;
		
		EXCEPTION
				
				When NO_DATA_FOUND THEN
				NULL;		
				
		END;
	
       		IF v_exists_sec = 'y' THEN
       	
 			DELETE FROM pr_cert_set_lnk where certsetid = P_CERTSETID 
 			AND certid = P_SECCERTID;
        
        		Insert into PR_CERT_SET_LNK
	       			(
	       			CERTSETLNKID	
	       			,CERTSETID	
	       			,CERTID			
	       			) 
	       			Values 	
	       			(
	       			PR_CERT_SET_LNK_SEQ.nextval
	       			,P_CERTSETID
	       			,P_SECCERTID
	       			)RETURNING CERTSETLNKID INTO v_certsetlnkid2
			;
		
        		v_RowCnt := sql%ROWCOUNT;
        		p_ReturnCd := 0;	
        		
        	ELSE
        
        		RAISE CERT_SET_PROPERTY_MISMATCH;
        
        	END IF;
        
        END IF;
        
	IF RC1%ISOPEN THEN 
	       	CLOSE RC1;
	END IF;	
	
	OPEN RC1 FOR SELECT 	
		coalesce(v_certsetlnkid1,0) CERTSETLNKID1, 
		coalesce(v_certsetlnkid2,0)  CERTSETLNKID2,
		p_returncd RETURNSETLNK
		FROM Dual;
		
	p_ReturnCd := 0;
		    	    
DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);		    	    
        
       
EXCEPTION
			
	WHEN CERT_BELONGS_TO_ANOTHER_SET THEN
		p_ReturnCd := 4;
		v_ExceptionFlg :=1 ;
		OPEN RC1 FOR SELECT 	
				coalesce(v_certsetlnkid1,0) CERTSETLNKID1, 
				coalesce(v_certsetlnkid2,0)  CERTSETLNKID2,
				p_returncd RETURNSETLNK
		FROM Dual;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'CERT_BELONGS_TO_ANOTHER_SET,'||v_ParamsTxt, p_DebugFlg, 1);
				
				                
	WHEN CERT_SET_PROPERTY_MISMATCH THEN
		p_ReturnCd := 5;
		v_ExceptionFlg := 1;
		OPEN RC1 FOR SELECT 	
				coalesce(v_certsetlnkid1,0) CERTSETLNKID1, 
				coalesce(v_certsetlnkid2,0)  CERTSETLNKID2,
				p_returncd RETURNSETLNK
		FROM Dual;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'CERT_SET_PROPERTY_MISMATCH, '||v_ParamsTxt, p_DebugFlg, 1);
				
	WHEN DUP_VAL_ON_INDEX THEN
		p_ReturnCd := 2;
		v_ExceptionFlg :=1 ;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, 1);
				                
	WHEN OTHERS THEN
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SET_LNKI_AH1:'||SQLERRM);
		        
END PR_CERT_SET_LNKI_AH1;
/


















-- PR_CERT_SET_LNKS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SET_LNKS_AH1
	(
	 p_ReturnCd OUT     PLS_Integer
	,p_DebugFlg IN     PLS_Integer
	,RC1  OUT  DB_REF.RCT
	,P_CERTSETID  IN  NUMBER
	)
AS
/*


	================================================================================
	Proc Name:       PR_CERT_SET_LNKS_AH1
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	Returns the list of certificate from the given certsetid
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


 v_BEGINTS           TIMESTAMP;   -- Performance related variables
 v_ExceptionFlg      INTEGER      DEFAULT 0;
 v_ProcName          VARCHAR2(30) DEFAULT 'PR_CERT_SET_LNKS_AH1';
 v_sqlerrm           VARCHAR2(255);
 v_Chkpt             CHAR(1)      DEFAULT '0';
 v_Msg               VARCHAR2(1024);
 v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', P_CERTSETID=>'||P_CERTSETID||CHR(10)
       ;
 v_partnerid         pr_partner.partnerid%TYPE;
 v_searchflg         INTEGER;
 
 BEGIN
 
	SELECT  ps.partnerid,
 		CASE ppt.PARTNERTYPENAME 
 			WHEN 'HubOperator' THEN 0
 			WHEN 'HubOwner' THEN 1
 			WHEN 'HubPartner' THEN 2
 		END into v_partnerid,v_searchflg 
 		from pr_cert_set ps ,pr_partner pp , pr_partnertype ppt
 		WHERE ps.partnerid = pp.partnerid 
 		AND pp.partnertypeid = ppt.partnertypeid
 		AND ps.certsetid = P_CERTSETID ;


OPEN RC1 FOR
	SELECT  certsetid,certsetname,certsetdesc,encryptflg,digsigflg,sslclientflg,status,rowts,
	       max(pricertid) AS pricertid ,max(seccertid) AS Seccertid ,max(pricertname) AS pricertname,
		max(seccertname) as SECCERTNAME,defflg from
	(SELECT           
	        b.certsetname , b.certsetdesc, b.certsetid, b.ENCRYPTFLG ,
	        b.DIGSIGFLG , b.SSLCLIENTFLG, b.status,b.rowts,
	        case when a.prisecflg=1 then a.certid end AS pricertid ,
	        case when a.prisecflg=2 then a.certid end AS seccertid ,
	        case when a.prisecflg=1 then a.CERTNAME END AS pricertname,
	        case when a.prisecflg=2 then a.CERTNAME end AS seccertname,
		FN_SET_DEFORNONDEF(P_CERTSETID,v_partnerid,v_searchflg) AS defflg
	    FROM pr_cert_set_lnk l
		right outer join pr_cert_set b on b.certsetid = l.certsetid
    		left outer join pr_cert a on a.certid = l.certid
	    where b.CERTSETID = P_CERTSETID
	    AND coalesce(a.deleteflag,0) <> 1) 
   	group by certsetname,certsetdesc,certsetid,encryptflg,digsigflg,sslclientflg,status,rowts,defflg
   	;
   	
   	
   p_returncd :=0;
   
   EXCEPTION
   
		WHEN NO_DATA_FOUND then
   			p_ReturnCd := 3;
   			v_ExceptionFlg := 1;
   			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'FOR NOT FOUND,'||v_ParamsTxt, p_DebugFlg, 1);
   			  			
   		WHEN OTHERS then
   			p_ReturnCd := 1;
   			v_ExceptionFlg := 1;
   			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
   			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SET_LNKS_AH1:'||SQLERRM);
   			
   			
END PR_CERT_SET_LNKS_AH1;

/
    	
   



-- PR_CERT_SET_LNKS_AH2.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SET_LNKS_AH2
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 out db_ref.RCT
	,P_CERTID IN NUMBER
	)
AS

/*


	================================================================================
	Proc Name:       PR_CERT_SET_LNKS_AH2
	Create Date:     2007-09-19 00:01:44
	Author:          BHARANI J
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-09-19 AMIT KUAMR    6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	Returns the list of certificate sets where given certid is used
	
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERT_SET_LNKS_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', P_CERTID=>'||P_CERTID||CHR(10)
       ;



---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

   	

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	Open RC1 FOR 	SELECT
    				b.certsetid,
				b.certsetname ,
				b.certsetdesc,
				b.ENCRYPTFLG ,
				b.DIGSIGFLG ,
				b.SSLCLIENTFLG
			FROM
        		PR_CERT a, PR_CERT_SET b, PR_CERT_SET_lnk l
			where
				a.certid = l.certid 
			AND coalesce(a.deleteflag,0) <> 1
			and b.certsetid = l.certsetid
			and l.CERTID = P_CERTID ;	
        	
	p_ReturnCd := 0;
	   	    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	   	
	EXCEPTION
	
	WHEN OTHERS then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SET_LNKS_AH2:'||SQLERRM);
				        
END PR_CERT_SET_LNKS_AH2;
/
                     

-- PR_CERT_SET_LNKS_AH3.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SET_LNKS_AH3
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 out db_ref.RCT
	,P_CERTSETNAME IN VARCHAR2
	,p_PARTNERID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_CERT_SET_LNKS_AH3
	Create Date:     2007-09-19 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-09-19 BHARANI J    6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	USED BY CONSOLE TO AVOID INSERTING DUPLICATE RECORD
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERT_SET_LNKS_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', P_CERTSETNAME=>'||P_CERTSETNAME||CHR(10)
       ||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
       ;

BEGIN

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	Open RC1 FOR 	SELECT
    				a.certsetid,
				a.certsetname 			
			FROM
				PR_CERT_SET a
			where 
				a.CERTSETNAME = P_CERTSETNAME
				and a.partnerid = p_PARTNERID;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	   	
	EXCEPTION
			WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SET_LNKS_AH3:'||SQLERRM);
				        
END PR_CERT_SET_LNKS_AH3;
/
                     

-- PR_CERT_SET_USAGEI_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SET_USAGEI_AH1
	(
	p_ReturnCd OUT PLS_Integer
	,p_DebugFlg IN PLS_Integer
	,RC1 out db_ref.RCT
	,p_frpartnerid IN NUMBER
	,p_topartnerid IN NUMBER
	,p_frroutingobjid IN NUMBER
	,p_toroutingobjid IN NUMBER
	,p_desttypeid IN NUMBER
	,p_certsetid_encrypt IN NUMBER
	,p_certsetid_digsign IN NUMBER
	,p_certsetid_sslauth IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CERT_SET_USAGEI_AH1
	Create Date:     2007-09-19 15:16:24
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-19 BHARANI J    6.1.1  Created for Multiple Certificate Feature

	================================================================================
	List and show set of certificates used by partners and packages
	
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	4= PACKAGE_NOT_SAME_TYPE 5= SET_USAGE_ALREADY_EXISTS
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERT_SET_USAGEI_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_frpartnerid=>'||p_frpartnerid||CHR(10)
       ||', p_topartnerid=>'||p_topartnerid||CHR(10)
       ||', p_frroutingobjid=>'||p_frroutingobjid||CHR(10)
       ||', p_toroutingobjid=>'||p_toroutingobjid||CHR(10)
       ||', p_desttypeid=>'||p_desttypeid||CHR(10)
       ||', p_certsetid_encrypt=>'||p_certsetid_encrypt||CHR(10)
       ||', p_certsetid_digsign=>'||p_certsetid_digsign||CHR(10)
       ||', p_certsetid_sslauth=>'||p_certsetid_sslauth||CHR(10)
       ;
v_RowTS        		integer :=0;

PACKAGE_NOT_SAME_TYPE 	Exception;

v_certsetid 		INTEGER;
v_function        	VARCHAR2(60);
v_partnerpairid 	PR_PARTNER_PAIR.PARTNERPAIRID%Type DEFAULT NULL;
v_routingobjpairid 	PR_ROUTINGOBJ_PAIR.ROUTINGOBJPAIRID%Type DEFAULT NULL;
v_partnerrtobjlnkid 	PR_PARTNERPAIR_RTOBJPAIR_LNK.PARTNERRTOBJLNKID%Type DEFAULT NULL;
v_routingobjtypeid 	cf_routingobject.routingobjtypeid%Type DEFAULT NULL;
v_certsetusageid 	PR_CERT_SET_USAGE.CERTSETUSAGEID%Type DEFAULT NULL;
v_RC1 			db_ref.RCT;
v_RC2 			db_ref.RCT;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	PR_PARTNER_PAIRI(p_returncd, p_debugflg, v_RC1, p_frpartnerid,p_topartnerid);
	    
        Fetch v_RC1 into v_partnerpairid,
	        		 v_rowts
        		 ;        		 
        		 
        BEGIN
        	SELECT distinct coalesce(routingobjtypeid,-1) INTO v_routingobjtypeid 
        	FROM cf_routingobject where routingobjid in(coalesce(p_frroutingobjid,-1),coalesce(p_toroutingobjid,-1)); 
        	
        	EXCEPTION 
        	
        		WHEN NO_DATA_FOUND THEN
        		NULL;
        		
        					
			WHEN OTHERS THEN 
			p_ReturnCd := 4;
			v_rowts := null;
			RAISE PACKAGE_NOT_SAME_TYPE;
			
		END;

                
        IF v_routingobjtypeid is null then
        	SELECT Routingobjtypeid INTO v_routingobjtypeid from cf_routingobjtype 
            	WHERE Routingobjtypekey='Package';
        END IF;
	
	PR_ROUTINGOBJ_PAIRI(p_returncd, p_debugflg, v_RC2, v_routingobjtypeid,p_frroutingobjid,p_toroutingobjid);
	
	        
	Fetch v_RC2 into v_routingobjpairid,
			 v_rowts
        		 ;
        		 	    		
        
	BEGIN
	
	SELECT PARTNERRTOBJLNKID into v_partnerrtobjlnkid from PR_PARTNERPAIR_RTOBJPAIR_LNK 
	                where PARTNERPAIRID = v_partnerpairid
	                and ROUTINGOBJPAIRID = v_routingobjpairid
			; 
	EXCEPTION
	WHEN NO_DATA_FOUND THEN
	NULL;
	END;
				
	
	IF v_partnerrtobjlnkid IS NULL THEN
	
		INSERT INTO PR_PARTNERPAIR_RTOBJPAIR_LNK
			(
			 PARTNERRTOBJLNKID
			,PARTNERPAIRID		
			,ROUTINGOBJPAIRID
			)
			VALUES
			(
			PR_PARPAIR_RTOBJPAIR_LNK_SEQ.nextval,
			v_partnerpairid,
			v_routingobjpairid
			)RETURNING PARTNERRTOBJLNKID INTO v_partnerrtobjlnkid;
	
	END IF;	
	
	BEGIN
	SELECT CERTSETUSAGEID into v_certsetusageid from PR_CERT_SET_USAGE 
		                where PARTNERRTOBJLNKID = v_partnerrtobjlnkid
		                and DESTTYPEID = p_desttypeid
				; 				
				
	EXCEPTION
	WHEN NO_DATA_FOUND THEN
	NULL;
	END;
	
	IF v_certsetusageid IS NULL THEN
	
		INSERT INTO PR_CERT_SET_USAGE
			(
			CERTSETUSAGEID,
			PARTNERRTOBJLNKID,
			DESTTYPEID,
			CERTSETID_ENCRYPTION,
			CERTSETID_SIGNING,
			CERTSETID_SSLCLIENT
			)
			VALUES
			(
			PR_CERT_SET_USAGE_SEQ.nextval,
			v_partnerrtobjlnkid,
			p_desttypeid,
			p_certsetid_encrypt,
			p_certsetid_digsign,
	        	p_certsetid_sslauth
	        	)RETURNING CERTSETUSAGEID INTO v_certsetusageid
	        	;
	        	
	ELSE
	
		UPDATE PR_CERT_SET_USAGE SET
		CERTSETID_ENCRYPTION = p_certsetid_encrypt,
		CERTSETID_SIGNING = p_certsetid_digsign,
		CERTSETID_SSLCLIENT = p_certsetid_sslauth,
		rowts = rowts + 1
		WHERE
		PARTNERRTOBJLNKID = v_partnerrtobjlnkid
		and DESTTYPEID = p_desttypeid
		;
		
	END IF;	
	
	DELETE FROM PR_CERT_SET_USAGE
	WHERE 
	CERTSETID_ENCRYPTION is null and
	CERTSETID_SIGNING is null and
	CERTSETID_SSLCLIENT is null;
	
	BEGIN
	
	SELECT coalesce(rowts,0) into v_rowts from PR_CERT_SET_USAGE 
		where PARTNERRTOBJLNKID = v_partnerrtobjlnkid
		and DESTTYPEID = p_desttypeid
	;
	
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
		NULL;
	END;
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;		        
	Open RC1 FOR SELECT 	v_partnerpairid PARTNERPAIRID,
				v_routingobjpairid ROUTINGOBJPAIRID,
				v_rowts ROWTS
		     FROM Dual;
		     
	p_ReturnCd := 0;		    	    
	
EXCEPTION
			WHEN PACKAGE_NOT_SAME_TYPE Then
				-- ROLLBACK work;
				p_ReturnCd := 4;
				v_ExceptionFlg :=1 ;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'PACKAGE_NOT_SAME_TYPE,'||v_ParamsTxt, p_DebugFlg, 1);
						          
			WHEN DUP_VAL_ON_INDEX Then
				p_ReturnCd := 2;
			        v_ExceptionFlg :=1 ;
			        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, 1);
				                
			WHEN OTHERS then
				-- ROLLBACK work;
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				Commit;
				RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SET_USAGEI_AH1:'||SQLERRM);
		        

		    	    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	
END PR_CERT_SET_USAGEI_AH1;
/



-- PR_CERT_SET_USAGES_AH1.sql


CREATE OR REPLACE PROCEDURE PR_CERT_SET_USAGES_AH1
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 out db_ref.RCT
	,P_FRPARTNERID IN NUMBER
	,P_TOPARTNERID IN NUMBER
	,P_FRROUTINGOBJID IN NUMBER
	,P_TOROUTINGOBJID IN NUMBER
	)
AS
/*


	================================================================================
	Proc Name:       PR_CERT_SET_USAGES_AH1
	Create Date:     2007-09-19 00:01:44
	Author:          AMIT KUMAR VIJAY

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-09-19 AMIT KUMAR    6.1.1  Feature 55795
	================================================================================
	List and show set of certificates used by partners and packages
	
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERT_SET_USAGES_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', P_FRPARTNERID=>'||P_FRPARTNERID||CHR(10)
       ||', P_TOPARTNERID=>'||P_TOPARTNERID||CHR(10)
       ||', P_FRROUTINGOBJID=>'||P_FRROUTINGOBJID||CHR(10)
       ||', P_TOROUTINGOBJID=>'||P_TOROUTINGOBJID||CHR(10)
       ;
v_RowTS        		integer :=0;


v_partnerpairid 	PR_PARTNER_PAIR.PARTNERPAIRID%Type DEFAULT NULL;
v_routingobjpairid 	PR_ROUTINGOBJ_PAIR.ROUTINGOBJPAIRID%Type DEFAULT NULL;
v_partnerrtobjlnkid 	PR_PARTNERPAIR_RTOBJPAIR_LNK.PARTNERRTOBJLNKID%Type DEFAULT NULL;




---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	Open RC1 FOR 	SELECT
    				csu.certsetusageid,
				csu.desttypeid,
				csu.certsetid_encryption,
				(select s.certsetname from pr_cert_set s where s.certsetid = csu.certsetid_encryption) certsetname_encryption,
				FN_SET_VALORINVAL(csu.certsetid_encryption)  VALIDITY_ENC,
				csu.certsetid_signing,
				(select s.certsetname from pr_cert_set s where s.certsetid = csu.certsetid_signing) certsetname_signing,
				FN_SET_VALORINVAL(csu.certsetid_signing)  VALIDITY_SGN,
				csu.certsetid_sslclient,
				(select s.certsetname from pr_cert_set s where s.certsetid = csu.certsetid_sslclient) certsetname_sslclient,
				FN_SET_VALORINVAL(csu.certsetid_sslclient)  VALIDITY_SSL,
				pp.partnerpairid,
	       			rp.routingobjpairid,
	       			prl.PARTNERRTOBJLNKID,
	       			(select partnername from pr_partner p where p.partnerid = pp.frpartnerid) FRPARTNERNAME,
	       			(select partnername from pr_partner p where p.partnerid = pp.topartnerid) TOPARTNERNAME,
	       			(select routingobjname from cf_routingobject c where c.routingobjid = rp.frroutingobjid) FRROUTINGOBJNAME,
	       			(select routingobjname from cf_routingobject c where c.routingobjid = rp.toroutingobjid) TOROUTINGOBJNAME
	       		FROM
    	      		PR_CERT_SET_USAGE csu
    	      		INNER JOIN PR_PARTNERPAIR_RTOBJPAIR_LNK prl on csu.partnerrtobjlnkid = prl.partnerrtobjlnkid 
    	      		INNER JOIN PR_PARTNER_PAIR pp on prl.partnerpairid=pp.partnerpairid
    	      		INNER JOIN PR_ROUTINGOBJ_PAIR RP on prl.routingobjpairid = rp.routingobjpairid 
	      		WHERE COALESCE(pp.frpartnerid,-1) =COALESCE(P_FRPARTNERID,-1) AND
    	      		    COALESCE(pp.topartnerid,-1) = COALESCE(P_TOPARTNERID,-1) AND
    	      		    COALESCE(rp.frroutingobjid,-1) = COALESCE(P_FRROUTINGOBJID,-1) AND
    	      		    COALESCE(rp.toroutingobjid,-1) = COALESCE(P_TOROUTINGOBJID,-1)
    	      	    ;
    	      	    
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	    	      	    
    	
    	
EXCEPTION
				                
			WHEN OTHERS then
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				Commit;
				RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SET_USAGES_AH1:'||SQLERRM);

    	    
END PR_CERT_SET_USAGES_AH1;
/


-- PR_CERT_SET_USAGES_AH2.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SET_USAGES_AH2
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, RC2  OUT  DB_REF.RCT
	, RC3  OUT  DB_REF.RCT
	,p_SortKey IN VARCHAR2
	,p_pagesizenbr IN NUMBER
	,p_pagenbr IN NUMBER
	,P_CERTSETID IN NUMBER
	,p_searchflg IN NUMBER           -- 0 or 1
	)
AS 
/*
	================================================================================
	Proc Name:       PR_CERT_SET_USAGES_AH2
	Create Date:     2007-09-19 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-09-19 BHARANI J     6.1.1  Feature 55795
        2008-02-27 BHARANI J     6.1.1  Feature 105965
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          	integer :=0;
v_BeginTS         	timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        	varchar2(30) := 'PR_CERT_SET_USAGES_AH2';
v_ExceptionFlg    	integer := 0;
v_ParamsTxt       	varchar2(4000) :=
       			'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       			||', p_SortKey=>'||p_SortKey||CHR(10)
       			||', p_pagesizenbr=>'||p_pagesizenbr||CHR(10)
       			||', p_pagenbr=>'||p_pagenbr||CHR(10)
       			||', P_CERTSETID=>'||P_CERTSETID||CHR(10)
       			||', p_searchflg=>'||p_searchflg||CHR(10)
       			;
v_RowTS        		integer :=0;

PACKAGE_NOT_SAME_TYPE 	Exception;

v_certsetid 		INTEGER;
v_function        	VARCHAR2(60);
v_partnerpairid 	PR_PARTNER_PAIR.PARTNERPAIRID%Type DEFAULT NULL;
v_routingobjpairid 	PR_ROUTINGOBJ_PAIR.ROUTINGOBJPAIRID%Type DEFAULT NULL;
v_partnerrtobjlnkid 	PR_PARTNERPAIR_RTOBJPAIR_LNK.PARTNERRTOBJLNKID%Type DEFAULT NULL;
v_routingobjtypeid 	cf_routingobject.routingobjtypeid%Type DEFAULT NULL;
v_certsetusageid 	PR_CERT_SET_USAGE.CERTSETUSAGEID%Type DEFAULT NULL;
v_RC1 			db_ref.RCT;
v_RC2 			db_ref.RCT;
v_Chkpt             	VARCHAR2(1)      DEFAULT '0';

v_DySQL            	VARCHAR2(4000);
v_sql1		        VARCHAR2(1000);

v_where 		VARCHAR2(500);
sql_stmt 		VARCHAR2(4000);
sql_stmt1 		VARCHAR2(4000);
sql_stmt2 		VARCHAR2(4000);
v_stmt			VARCHAR2(4000);
v_Stmt2			VARCHAR2(4000);
v_Stmt3			VARCHAR2(4000);
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	 v_where := ' where 1=1 ';
		
	IF P_SEARCHFLG = 0 then     -- SSLCLIENT AND SIGNATURE (Internal + HUB )
		 v_where := v_where 
		      ||' AND CERTSETID_SSLCLIENT = ' || (P_CERTSETID)
		      ||' OR CERTSETID_SIGNING = ' || (P_CERTSETID) 
	;
	
	ELSE 	IF P_SEARCHFLG = 1 then -- For Encryption ( External )
			v_where := v_where 
			      ||' AND CERTSETID_ENCRYPTION = ' || (P_CERTSETID)
			      ;
		END IF;	
    	END IF; 
    	
    	sql_stmt := 'SELECT distinct
			pp.frpartnerid,
    			(select partnername from pr_partner where partnerid=pp.frpartnerid) FROMPARTNERNAME,
   			pp.topartnerid,
			(select partnername from pr_partner where partnerid=pp.topartnerid) TOPARTNERNAME,
			rp.frroutingobjid,
			(select routingobjname from cf_routingobject where routingobjid= rp.frroutingobjid) FROMROUTINGOBJECTNAME,
			rp.toroutingobjid,
			(select routingobjname from cf_routingobject where routingobjid= rp.toroutingobjid) TOROUTINGOBJECTNAME,
			prl.partnerrtobjlnkid
    		  FROM PR_CERT_SET_USAGE csu 
			INNER JOIN PR_PARTNERPAIR_RTOBJPAIR_LNK prl on csu.partnerrtobjlnkid = prl.partnerrtobjlnkid 
			INNER JOIN PR_PARTNER_PAIR pp on prl.partnerpairid=pp.partnerpairid
			INNER JOIN PR_ROUTINGOBJ_PAIR RP on prl.routingobjpairid = rp.routingobjpairid '
		; 
		
	sql_stmt := sql_stmt || v_where ;
	
	IF p_sortkey IS NOT NULL THEN
		sql_stmt := sql_stmt || ' ORDER BY ' || p_SortKey ;		        		
	END IF;
	
	-- Paging logic
	sql_stmt1 := FN_PAGING_LOGIC(p_pageNbr, p_pageSizeNbr, sql_stmt);
	
	BEGIN
	        v_stmt:= sql_stmt1;
		v_Chkpt:='2';
		IF RC1%isOpen THEN CLOSE RC1; END IF;
   		OPEN RC1 FOR v_stmt;
		
	END;     
	
	IF COALESCE(p_pageNbr,0) in (0,1) THEN
			v_DySQL := '';
			v_Chkpt:='3';
	
	END IF;
	
	BEGIN
					                        
	v_DySQL := 'SELECT COUNT(*) TotalRowsNbr FROM '
			|| ' ( ' ||  sql_stmt ||
			' )  A ';
						
	v_Stmt2:= v_Dysql;
	v_Chkpt:='4';
	IF RC2%isOpen THEN CLOSE RC1; END IF;
   	OPEN RC2 FOR v_DySQL;
	END;
	
	sql_stmt2 := 'SELECT PARTNERRTOBJLNKID,
			csu.DESTTYPEID,
			(select destname from pr_desttype where desttypeid= csu.DESTTYPEID) DESTNAME,
			(select resourcekey from pr_desttype where desttypeid= csu.DESTTYPEID) RESOURCEKEY,'
			|| '(CASE when CERTSETID_ENCRYPTION =  ''' || P_CERTSETID || ''' then 1 else 0 end ) as CERTSETID_ENCRYPTION,'
			|| '(CASE when CERTSETID_SIGNING =  ''' || P_CERTSETID || ''' then 1 else 0 end ) as CERTSETID_SIGNING,'
			|| '(CASE when CERTSETID_SSLCLIENT =  ''' || P_CERTSETID || ''' then 1 else 0 end ) as CERTSETID_SSLCLIENT'
			|| ' FROM PR_CERT_SET_USAGE csu';
			
	sql_stmt2 := sql_stmt2 ||v_where;	
					
	BEGIN
		        v_stmt3:=sql_stmt2;
			v_Chkpt:='5';
			IF RC3%isOpen THEN CLOSE RC1; END IF;
   			OPEN RC3 FOR v_stmt3;
	END; 
	
	p_returncd :=0;
	      		 	 
EXCEPTION
				                
	WHEN OTHERS then
			
	p_ReturnCd := 1;
	v_ExceptionFlg := 1;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
	RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SET_USAGES_AH2:'||SQLERRM);
	
END PR_CERT_SET_USAGES_AH2;
/


-- PR_CERT_SET_USAGES_AH3.sql

CREATE OR REPLACE PROCEDURE PR_CERT_SET_USAGES_AH3
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 out db_ref.RCT
	,P_FRPARTNERID IN NUMBER
	,P_TOPARTNERID IN NUMBER
	)
AS
/*


	================================================================================
	Proc Name:       PR_CERT_SET_USAGES_AH3
	Create Date:     2008-03-28 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-09-19 AMIT KUMAR    6.1.1  Feature 55795
	================================================================================
	List and show set of certificates used by partners and packages

	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CERT_SET_USAGES_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', P_FRPARTNERID=>'||P_FRPARTNERID||CHR(10)
       ||', P_TOPARTNERID=>'||P_TOPARTNERID||CHR(10)
       ;
v_RowTS        		integer :=0;


v_partnerpairid 	PR_PARTNER_PAIR.PARTNERPAIRID%Type DEFAULT NULL;
v_routingobjpairid 	PR_ROUTINGOBJ_PAIR.ROUTINGOBJPAIRID%Type DEFAULT NULL;
v_partnerrtobjlnkid 	PR_PARTNERPAIR_RTOBJPAIR_LNK.PARTNERRTOBJLNKID%Type DEFAULT NULL;




---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	IF P_FRPARTNERID IS NULL AND P_TOPARTNERID IS NULL THEN

		Open RC1 FOR 	
		SELECT 
		csu.desttypeid,
		csu.certsetid_encryption,
		(select s.certsetname from pr_cert_set s where s.certsetid = csu.certsetid_encryption) certsetname_encryption,
		csu.certsetid_signing,
		(select s.certsetname from pr_cert_set s where s.certsetid = csu.certsetid_signing) certsetname_signing,
		csu.certsetid_sslclient,
		(select s.certsetname from pr_cert_set s where s.certsetid = csu.certsetid_sslclient) certsetname_sslclient,
		pp.frpartnerid,
	       	(select partnername from pr_partner p where p.partnerid = pp.frpartnerid) FRPARTNERNAME,
	       	pp.topartnerid,
	       	(select partnername from pr_partner p where p.partnerid = pp.topartnerid) TOPARTNERNAME,
	       	rp.frroutingobjid,
	       	(select routingobjname from cf_routingobject c where c.routingobjid = rp.frroutingobjid) FRROUTINGOBJNAME,
	       	rp.toroutingobjid,
	       	(select routingobjname from cf_routingobject c where c.routingobjid = rp.toroutingobjid) TOROUTINGOBJNAME
	        FROM
    	        PR_CERT_SET_USAGE csu
    	        INNER JOIN PR_PARTNERPAIR_RTOBJPAIR_LNK prl on csu.partnerrtobjlnkid = prl.partnerrtobjlnkid 
    	        INNER JOIN PR_PARTNER_PAIR pp on prl.partnerpairid=pp.partnerpairid
    	        INNER JOIN PR_ROUTINGOBJ_PAIR RP on prl.routingobjpairid = rp.routingobjpairid 
    	        WHERE pp.frpartnerid IS NOT NULL  OR
                pp.topartnerid IS NOT NULL  OR
                rp.frroutingobjid IS NOT NULL   OR
                rp.toroutingobjid IS NOT NULL
    	        ;
    	      
    	      	    
	ELSE
	
		OPEN RC1 FOR 
		SELECT 
		csu.desttypeid,
		csu.certsetid_encryption,
		(select s.certsetname from pr_cert_set s where s.certsetid = csu.certsetid_encryption) certsetname_encryption,
		csu.certsetid_signing,
		(select s.certsetname from pr_cert_set s where s.certsetid = csu.certsetid_signing) certsetname_signing,
		csu.certsetid_sslclient,
		(select s.certsetname from pr_cert_set s where s.certsetid = csu.certsetid_sslclient) certsetname_sslclient,
		pp.frpartnerid,
		(select partnername from pr_partner p where p.partnerid = pp.frpartnerid) FRPARTNERNAME,
		pp.topartnerid,
		(select partnername from pr_partner p where p.partnerid = pp.topartnerid) TOPARTNERNAME,
		rp.frroutingobjid,
		(select routingobjname from cf_routingobject c where c.routingobjid = rp.frroutingobjid) FRROUTINGOBJNAME,
		rp.toroutingobjid,
		(select routingobjname from cf_routingobject c where c.routingobjid = rp.toroutingobjid) TOROUTINGOBJNAME
		FROM
		PR_CERT_SET_USAGE csu
		INNER JOIN PR_PARTNERPAIR_RTOBJPAIR_LNK prl on csu.partnerrtobjlnkid = prl.partnerrtobjlnkid 
		INNER JOIN PR_PARTNER_PAIR pp on prl.partnerpairid=pp.partnerpairid
		INNER JOIN PR_ROUTINGOBJ_PAIR RP on prl.routingobjpairid = rp.routingobjpairid
		WHERE COALESCE(pp.frpartnerid,-1) = COALESCE(p_frpartnerid,-1) AND
		COALESCE(pp.topartnerid,-1) = COALESCE(p_topartnerid,-1) 
		;
		
	END IF;		
    	      

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);



EXCEPTION

			WHEN OTHERS then
				p_ReturnCd := 1;
				v_ExceptionFlg := 1;
				DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
				Commit;
				RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CERT_SET_USAGES_AH3:'||SQLERRM);


END PR_CERT_SET_USAGES_AH3;
/

-- PR_CHANNEL_CERTSETI_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CHANNEL_CERTSETI_AH1
	(
	 p_ReturnCd OUT PLS_Integer
	,  p_DebugFlg  IN pls_integer
	,  RC1 out db_ref.RCT
	,  P_CHANNELID IN INTEGER
	,  P_DESTTYPEID IN INTEGER
	,  P_CERTSETID_ENCRYPT IN INTEGER
	,  P_CERTSETID_DIGSIGN IN INTEGER
	,  P_CERTSETID_SSLAUTH IN INTEGER
	,  P_ROWTS IN INTEGER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CHANNEL_CERTSETI_AH1
	Create Date:     2007-09-25 15:16:24
	Author:          BHARANI J 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-25 BHARANI J     6.1.1  Created for Multiple Certificate Feature

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/




 v_BeginTS         TimeStamp  ; -- Performance related variables
 v_ProcName        varchar(30)  Default 'PR_CHANNEL_CERTSETI_AH1';
 v_ExceptionFlg    integer  Default 0;
 v_ExceptionCode   varchar(30);
 v_ParamsTxt       varchar(4000) ;
 v_SQLErrm         Varchar(255) ;
 v_msg             Varchar(500);
 v_RowTS           Integer Default 0;
 v_temp INTEGER;
 v_temp1 INTEGER;
 v_temp2 INTEGER;

 v_chcertsetlnkid   INTEGER;
 
 ROWTS_NOT_MATCHING EXCEPTION;



BEGIN

	BEGIN
	SELECT chcertsetlnkid into v_chcertsetlnkid from PR_CHANNEL_CERTSET_LNK 
	WHERE channelid = p_channelid and DESTTYPEID = p_desttypeid 
	;
	EXCEPTION
	WHEN NO_DATA_FOUND THEN
	NULL;
	END;
	
	IF v_chcertsetlnkid is not null THEN
	
	BEGIN
	select chcertsetlnkid into v_temp from PR_CHANNEL_CERTSET_LNK 
		where channelid = p_channelid and DESTTYPEID = p_desttypeid 
		and rowts = p_rowts
	;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
		NULL;
	END;
	
		IF v_temp is NULL THEN
			RAISE ROWTS_NOT_MATCHING;
		END IF
		;		
	
	UPDATE PR_CHANNEL_CERTSET_LNK
	set CERTSETID_ENCRYPTION = P_CERTSETID_ENCRYPT,
	CERTSETID_SIGNING = P_CERTSETID_DIGSIGN,
	CERTSETID_SSLCLIENT = P_CERTSETID_SSLAUTH,
	rowts = rowts + 1
	where channelid = p_channelid 
	and DESTTYPEID = p_desttypeid
	and rowts = P_ROWTS
	;
	
	else 
	
	INSERT into PR_CHANNEL_CERTSET_LNK
		(
		 CHCERTSETLNKID	
		,CHANNELID	
		,DESTTYPEID
		,CERTSETID_ENCRYPTION
		,CERTSETID_SIGNING
		,CERTSETID_SSLCLIENT
		) 
		Values 	
		(
		 PR_CHANNEL_CERTSET_LNK_SEQ.NEXTVAL
		,p_channelid
		,p_desttypeid
		,P_CERTSETID_ENCRYPT
		,P_CERTSETID_DIGSIGN
		,P_CERTSETID_SSLAUTH
		) RETURNING CHCERTSETLNKID INTO v_chcertsetlnkid
		;		

     	END IF;
     	
     	DELETE from PR_CHANNEL_CERTSET_LNK 
	WHERE
	CERTSETID_ENCRYPTION is null and
	CERTSETID_SIGNING is null and
	CERTSETID_SSLCLIENT is null;
	
	commit;

	BEGIN
	SELECT chcertsetlnkid,rowts into v_temp1,v_temp2 from PR_CHANNEL_CERTSET_LNK 
	WHERE channelid = p_channelid and DESTTYPEID = p_desttypeid;
	EXCEPTION
			WHEN NO_DATA_FOUND THEN
			NULL;
	END;
	
	v_chcertsetlnkid := coalesce(v_temp1,-1);
	v_rowts := coalesce(v_temp2,-1);    	

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;	
	
	OPEN RC1 FOR
	SELECT v_chcertsetlnkid chcertsetlnkid, v_rowts RowTS from DUAL;
	
	p_ReturnCd := 0;
	
	EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX Then
	p_ReturnCd := 2;
	v_ExceptionFlg :=1 ;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, 1);
	
	WHEN ROWTS_NOT_MATCHING Then
	p_ReturnCd := 3;
	v_ExceptionFlg :=1 ;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ROWTS_NOT_MATCHING,'||v_ParamsTxt, p_DebugFlg, 1);
	
	WHEN OTHERS then
	ROLLBACK work;
	p_ReturnCd := 1;
	v_ExceptionFlg := 1;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
	RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CHANNEL_CERTSETI_AH1:'||SQLERRM);
				              
END PR_CHANNEL_CERTSETI_AH1;
/

-- PR_CHANNEL_CERTSETS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CHANNEL_CERTSETS_AH1
(
	p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1			out		DB_REF.RCT
	, p_ChannelID		in		INTEGER
	, p_DestTypeID		in		INTEGER
) AS
/*
	================================================================================
	Proc Name:       PR_CHANNEL_CERTSETS_AH1
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

	v_RowCnt          integer Default 0;
	v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
	v_ProcName        varchar2(30) Default 'PR_USERS_AH3';
	v_ExceptionFlg    integer Default 0;
	v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ChannelID=>'||p_ChannelID||CHR(10)
		||', p_DestTypeID=>'||p_DestTypeID||CHR(10)
		;

	v_RowTS			integer :=0;
	v_frpartnerid		PR_PARTNER.PARTNERID%TYPE;
	v_topartnerid		PR_PARTNER.PARTNERID%TYPE;
	v_frroutingobjid	CF_ROUTINGOBJECT.ROUTINGOBJID%TYPE;
	v_toroutingobjid	CF_ROUTINGOBJECT.ROUTINGOBJID%TYPE;
	v_frroutingobjname      CF_ROUTINGOBJECT.ROUTINGOBJNAME%TYPE;
	v_toroutingobjname      CF_ROUTINGOBJECT.ROUTINGOBJNAME%TYPE;
	v_channelid             integer;
	
	v_channeldesttypeid PR_DESTTYPE.DESTTYPEID%TYPE;
	v_encryption_flg       NUMBER;
	v_certsetid_encryption pr_channel_certset_lnk.certsetid_encryption%TYPE;
	v_certsetid_encryption_name pr_cert_set.certsetname%TYPE;
	v_sslclient_flg        NUMBER;
	v_certsetid_sslclient pr_channel_certset_lnk.certsetid_signing%TYPE;
	v_certsetid_sslclient_name pr_cert_set.certsetname%TYPE;
	v_digsigflg            NUMBER;
	v_certsetid_signing    pr_channel_certset_lnk.certsetid_signing%TYPE;
	v_certsetid_signing_name pr_cert_set.certsetname%TYPE;
	v_temppartnerid        NUMBER;
	v_temproutingobjid     NUMBER;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	WITH RTOBJVW AS
		(SELECT	R.ROUTINGOBJLEVEL,
			R.ROUTINGOBJTYPEID,
			R.ROUTINGOBJID,
			F.ROUTINGOBJREFID,
			F.ROUTINGOBJPARENTREFID,
			CASE R.ROUTINGOBJLEVEL
				WHEN 1 THEN R.ROUTINGOBJID
				WHEN 2 THEN R2.ROUTINGOBJID
				WHEN 3 THEN R3.ROUTINGOBJID
				WHEN 4 THEN R4.ROUTINGOBJID
				WHEN 5 THEN R5.ROUTINGOBJID
				WHEN 6 THEN R6.ROUTINGOBJID
				ELSE -1
			END PackageID
		FROM	cf_routingobject01_vw R
		INNER JOIN CF_ROUTINGOBJREF_LNK F ON F.ROUTINGOBJID = R.ROUTINGOBJID
		LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F2 ON F2.ROUTINGOBJREFID = F.ROUTINGOBJPARENTREFID
		LEFT OUTER JOIN cf_routingobject01_vw R2 ON R2.ROUTINGOBJID = F2.ROUTINGOBJID
		LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F3 ON F3.ROUTINGOBJREFID = F2.ROUTINGOBJPARENTREFID
		LEFT OUTER JOIN cf_routingobject01_vw R3 ON R3.ROUTINGOBJID = F3.ROUTINGOBJID
		LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F4 ON F4.ROUTINGOBJREFID = F3.ROUTINGOBJPARENTREFID
		LEFT OUTER JOIN cf_routingobject01_vw R4 ON R4.ROUTINGOBJID = F4.ROUTINGOBJID
		LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F5 ON F5.ROUTINGOBJREFID = F4.ROUTINGOBJPARENTREFID
		LEFT OUTER JOIN cf_routingobject01_vw R5 ON R5.ROUTINGOBJID = F5.ROUTINGOBJID
		LEFT OUTER JOIN CF_ROUTINGOBJREF_LNK F6 ON F6.ROUTINGOBJREFID = F5.ROUTINGOBJPARENTREFID
		LEFT OUTER JOIN cf_routingobject01_vw R6 ON R6.ROUTINGOBJID = F6.ROUTINGOBJID
		)
	SELECT	pc.frpartnerid
		, pc.topartnerid
		, frrol.Packageid as frroutingobjid
		, torol.Packageid as toroutingobjid
	INTO	v_frpartnerid
		,v_topartnerid
		,v_frroutingobjid
		,v_toroutingobjid
	FROM	pr_routingchannel pc	
	INNER JOIN pr_capability frcap ON frcap.capabilityid = pc.frcapabilityid
	INNER JOIN pr_capability tocap ON tocap.capabilityid = pc.tocapabilityid
	INNER JOIN rtobjvw frrol ON frcap.routingobjrefid = frrol.routingobjrefid
	INNER JOIN rtobjvw torol ON tocap.routingobjrefid = torol.routingobjrefid
	WHERE	pc.channelid = p_channelid
	;
	
	SELECT routingobjname into v_frroutingobjname from cf_routingobject01_vw where 
				  routingobjid = v_frroutingobjid;
				  
	SELECT routingobjname INTO v_toroutingobjname from cf_routingobject01_vw where 
				  routingobjid = v_toroutingobjid;
	-- FIX FOR RNIF 2 WAY PIP.
	
	IF v_frroutingobjname ='RNIF' AND v_toroutingobjname='Backend Integration' THEN
	
		v_temppartnerid := v_frpartnerid;
		v_frpartnerid := v_topartnerid;
		v_topartnerid := v_temppartnerid;
		v_temproutingobjid := v_frroutingobjid;
		v_frroutingobjid := v_toroutingobjid;
		v_toroutingobjid := v_temproutingobjid;
	
	END IF;
				  

	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 FOR
		WITH	CSLVL AS
		(SELECT 	desttypeid,
			certsetid_encryption,
			certsetid_signing,
			certsetid_sslclient,
			9999 sortlevel
		FROM	pr_channel_certset_lnk pccl
		WHERE	channelid=p_channelid
		AND	((p_desttypeid is not null AND desttypeid = p_desttypeid ) OR ( p_desttypeid IS NULL))
		UNION	
		SELECT	desttypeid
			,pcu.certsetid_encryption
			,pcu.certsetid_signing
			,pcu.certsetid_sslclient
			,case when pp.frpartnerid is null then 0 else 1000 end
			 + case when pp.topartnerid is null then 0 else 100 end
			 + case when rp.toroutingobjid is null then 0 else 10 end
			 + case when rp.toroutingobjid is null then 0 else 1 end AS sortlevel
		FROM	PR_CERT_SET_USAGE pcu
		INNER JOIN PR_PARTNERPAIR_RTOBJPAIR_LNK ppl ON pcu.partnerrtobjlnkid = ppl.partnerrtobjlnkid
		INNER JOIN PR_PARTNER_PAIR PP ON PP.PARTNERPAIRID = ppl.PARTNERPAIRID
		INNER JOIN PR_ROUTINGOBJ_PAIR RP ON RP.ROUTINGOBJPAIRID = PPL.ROUTINGOBJPAIRID
		WHERE	COALESCE(pp.frpartnerid,v_frpartnerid) = v_frpartnerid
		AND  	COALESCE(pp.topartnerid,v_topartnerid) = v_topartnerid
		AND 	COALESCE(rp.frroutingobjid,v_frroutingobjid) = v_frroutingobjid
		AND  	COALESCE(rp.toroutingobjid,v_toroutingobjid) = v_toroutingobjid
		AND	((p_desttypeid is not null AND pcu.desttypeid = p_desttypeid ) OR ( p_desttypeid IS NULL))
		)
	SELECT	b.desttypeid
		,b.ENC AS encryption_flg
		,x.certsetid_encryption
		,csx.certsetname certsetid_encryption_name
		,b.SSL AS sslclient_flg
		,y.certsetid_sslclient
		,csy.certsetname certsetid_sslclient_name
		,b.SGN AS signing_flg
		,z.certsetid_signing
		,csz.certsetname certsetid_signing_name
	FROM	(SELECT	desttypeid, 
			MAX(CASE WHEN certsetid_encryption IS NOT NULL THEN sortlevel ELSE -1 END) AS ENC
			,MAX(CASE WHEN certsetid_sslclient IS NOT NULL THEN sortlevel ELSE -1 END) AS SSL
			,MAX(CASE WHEN certsetid_signing IS NOT NULL THEN sortlevel ELSE -1 END) AS SGN
		FROM	CSLVL
		GROUP BY desttypeid) B
	LEFT OUTER JOIN CSLVL x ON x.sortlevel = b.ENC AND x.desttypeid = b.desttypeid
	LEFT OUTER JOIN PR_CERT_SET csx ON csx.certsetid = x.certsetid_encryption  
	LEFT OUTER JOIN CSLVL y ON y.sortlevel = b.SSL AND y.desttypeid = b.desttypeid
	LEFT OUTER JOIN PR_CERT_SET csy ON csy.certsetid = y.certsetid_sslclient  
	LEFT OUTER JOIN CSLVL z ON z.sortlevel = b.SGN AND z.desttypeid = b.desttypeid
	LEFT OUTER JOIN PR_CERT_SET csz ON csz.certsetid = z.certsetid_signing  
	;
	
	p_returncd := 0;
	
	
    	
EXCEPTION
	WHEN OTHERS then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
END;
/


-- PR_CHANNEL_CERTSETS_AH2.sql

CREATE OR REPLACE PROCEDURE PR_CHANNEL_CERTSETS_AH2
	(
	 p_ReturnCd    OUT  PLS_INTEGER
	,p_DebugFlg   IN   PLS_INTEGER
	,RC1 out db_ref.RCT
	,RC2 out db_ref.RCT
	,P_sortkey    IN   VARCHAR2
	,p_pagesizenbr  IN INTEGER
	,p_pagenbr   IN    INTEGER
	,p_certsetid   IN  INTEGER
	)
AS
/*

	================================================================================
	Proc Name:       PR_CHANNEL_CERTSETS_AH2
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


 v_BEGINTS           TIMESTAMP;   -- Performance related variables
 v_ExceptionFlg      INTEGER      DEFAULT 0;
 v_ProcName          VARCHAR (30) DEFAULT 'PR_CHANNEL_CERTSETS_AH2';
 v_sqlerrm           VARCHAR(255);
 v_ParamsTxt         VARCHAR(4000);
 v_Chkpt             CHAR(1)      DEFAULT '0';
 v_Msg              VARCHAR(1024);
 v_where            VARCHAR(1024);
 sqlstmt           VARCHAR(4000);
 sqlstmt1           VARCHAR(4000);
 v_DySQL            VARCHAR(4000);
    		
BEGIN

	p_ReturnCd := 0;
	v_BEGINTS := FN_Get_Date();

	sqlstmt := '	SELECT distinct   
			pc.frpartnerid,
			(select partnername from pr_partner  where partnerid = pc.frpartnerid) FRPARTNERNAME,
			pc.topartnerid,
			(select partnername from pr_partner  where partnerid = pc.topartnerid) TOPARTNERNAME,
			frcap.routingobjrefid FRROUTINGOBJREFID,
			tocap.routingobjrefid TOROUTINGOBJREFID
        FROM
        pr_channel_certset_lnk a
        INNER JOIN pr_routingchannel pc ON a.channelid = pc.channelid
        INNER JOIN pr_capability frcap ON frcap.capabilityid = pc.frcapabilityid
	INNER JOIN pr_capability tocap ON tocap.capabilityid = pc.tocapabilityid'
		;
		
	 v_where := ' WHERE  '
		    || ' ( CERTSETID_ENCRYPTION = ' || p_certsetid
		    || ' OR CERTSETID_SIGNING = ' || p_certsetid
		    || ' OR CERTSETID_SSLCLIENT = ' || p_certsetid
	   	    || '  ) '
	;	   	    
		
	
	sqlstmt := sqlstmt || v_where 
	;
	
	
	
	IF p_sortkey IS NOT NULL THEN
		 sqlstmt := sqlstmt ||' ORDER BY '|| p_sortkey;       		
	END IF;
	
	-- Paging logic
	 sqlstmt1 := FN_PAGING_LOGIC(p_pageNbr, p_pageSizeNbr, sqlstmt);
	
	v_Chkpt :='1';

	BEGIN
	
	IF rc1%ISOPEN THEN
		CLOSE rc1;
	END IF;
		
	OPEN rc1 FOR sqlstmt1;
	
	END;       
           
           
	IF COALESCE(p_pageNbr,0) in (0,1) THEN
		 v_DySQL := '';
		 v_Chkpt :='3';
		
	BEGIN                      
		
		v_DySQL := 'SELECT COUNT(*) TotalRowsNbr FROM '
			    || ' ( ' ||  sqlstmt ||
						' )  A ';
		OPEN rc2 FOR v_DySQL;
		
		dbms_output.put_line(v_DySQL);
		
	END;

	END IF; 
	
	EXCEPTION
	
	WHEN OTHERS THEN
	p_ReturnCd := 1;
	v_ExceptionFlg := 1;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
	Commit;
	RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CHANNEL_CERTSETS_AH2:'||SQLERRM);

END PR_CHANNEL_CERTSETS_AH2;
/                        

-- PR_CHANNEL_CERTSETS_AH3.sql

CREATE OR REPLACE Procedure PR_CHANNEL_CERTSETS_AH3
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_channelid 		in		INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       PR_CHANNEL_CERTSETS_AH3
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CHANNEL_CERTSETS_AH3';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_channelid=>'||p_channelid||CHR(10)
		;


BEGIN
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	OPEN RC1 FOR
		SELECT 	CHCERTSETLNKID,
				channelid,
				desttypeid,
				certsetid_encryption,
				certsetid_signing,
				certsetid_sslclient,
				rowts
			FROM	pr_channel_certset_lnk pccl
			WHERE	channelid=p_channelid;
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_CHANNEL_CERTSETS_AH3' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CHANNEL_CERTSETS_AH3' || SQLCODE|| ' '||SQLERRM);
END PR_CHANNEL_CERTSETS_AH3;
/

-- PR_CHANNEL_GATEWAY_LNKD.sql

CREATE OR REPLACE Procedure PR_CHANNEL_GATEWAY_LNKD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_CHANNELGATEWAYID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CHANNEL_GATEWAY_LNKD
	Create Date:     2003-09-26 08:24:13
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CHANNEL_GATEWAY_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CHANNELGATEWAYID=>'||p_CHANNELGATEWAYID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_CHANNEL_GATEWAY_LNK
	Where
		CHANNELGATEWAYID = p_CHANNELGATEWAYID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CHANNEL_GATEWAY_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_CHANNEL_GATEWAY_LNKD;
/


-- PR_CHANNEL_GATEWAY_LNKD_AH1.sql

CREATE OR REPLACE PROCEDURE pr_channel_gateway_lnkd_ah1
(
   p_returncd    OUT      PLS_INTEGER,
   p_debugflg    IN       PLS_INTEGER,
   p_gatewayid   IN       NUMBER,
   p_partnerid   IN       NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       PR_CHANNEL_GATEWAY_LNKD_AH1
    Create Date:     2007-08-20 08:24:13
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-20 BHARANI           1  Created

    This procedure is used to update the channel with the default gateways
    and if there is no defautt gateway for a destination type then the record is removed.

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  
   v_begints        TIMESTAMP       DEFAULT fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   DEFAULT 'PR_CHANNEL_GATEWAY_LNKD_AH1';
   v_exceptionflg   INTEGER         DEFAULT 0;
   v_paramstxt      VARCHAR2 (4000) DEFAULT    'p_DebugFlg=>'|| p_debugflg || CHR (10) ||
                                               ', p_gatewayid=>'  || p_gatewayid || CHR (10) ||
                                               ', p_partnerid=> ||p_partnerid||CHR(10)';


BEGIN

--  Update frgatewayid and togatewayid only if the defaultgatewayid of the destype is known.

   UPDATE pr_channel_gateway_lnk a
      SET a.frgatewayid =
             (SELECT b.gatewayid
                FROM pr_gatewaydefault b
               WHERE b.partnerid = p_partnerid AND b.desttypeid = a.desttypeid)
    WHERE a.frgatewayid = p_gatewayid
      AND EXISTS (
               SELECT b.gatewayid
                 FROM pr_gatewaydefault b
                WHERE b.partnerid = p_partnerid
                      AND b.desttypeid = a.desttypeid);
                      
                       

   UPDATE pr_channel_gateway_lnk a
      SET a.togatewayid =
             (SELECT b.gatewayid
                FROM pr_gatewaydefault b
               WHERE b.partnerid = p_partnerid AND b.desttypeid = a.desttypeid)
    WHERE a.togatewayid = p_gatewayid
      AND EXISTS (
               SELECT b.gatewayid
                 FROM pr_gatewaydefault b
                WHERE b.partnerid = p_partnerid
                      AND b.desttypeid = a.desttypeid);
                      
                      

--  If the default destype is not know for a given gatewayid then delete.

   DELETE FROM pr_channel_gateway_lnk a
         WHERE a.frgatewayid = p_gatewayid
           AND NOT EXISTS (
                  SELECT b.gatewayid
                    FROM pr_gatewaydefault b
                   WHERE b.partnerid = p_partnerid
                     AND b.desttypeid = a.desttypeid);
                     
                        

   DELETE FROM pr_channel_gateway_lnk a
         WHERE a.togatewayid = p_gatewayid
           AND NOT EXISTS (
                  SELECT b.gatewayid
                    FROM pr_gatewaydefault b
                   WHERE b.partnerid = p_partnerid
                     AND b.desttypeid = a.desttypeid);
                     
                     
                     
                   

   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
   p_returncd := 0;    
   
EXCEPTION

   WHEN OTHERS
   THEN
      p_returncd := 1;
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            SQLERRM || CHR (10) || v_paramstxt,
                            p_debugflg,
                            1
                           );
      raise_application_error (-20999, '20000 ' || SQLCODE || ' ' || SQLERRM);
      
END pr_channel_gateway_lnkd_ah1;
/

-- PR_CHANNEL_GATEWAY_LNKI.sql

CREATE OR REPLACE Procedure PR_CHANNEL_GATEWAY_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CHANNELID in NUMBER
	,p_FRGATEWAYID in NUMBER
	,p_TOGATEWAYID in NUMBER
	,p_FRSCHEDULEID in NUMBER
	,p_TOSCHEDULEID in NUMBER
	,p_DESTTYPEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CHANNEL_GATEWAY_LNKI
	Create Date:     2003-09-26 00:01:42
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CHANNEL_GATEWAY_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
		||', p_FRGATEWAYID=>'||p_FRGATEWAYID||CHR(10)
		||', p_TOGATEWAYID=>'||p_TOGATEWAYID||CHR(10)
		||', p_FRSCHEDULEID=>'||p_FRSCHEDULEID||CHR(10)
		||', p_TOSCHEDULEID=>'||p_TOSCHEDULEID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_CHANNEL_GATEWAY_LNK(
		CHANNELGATEWAYID
		,CHANNELID
		,FRGATEWAYID
		,TOGATEWAYID
		,FRSCHEDULEID
		,TOSCHEDULEID
		,DESTTYPEID
	) Values (
		PR_CHANNEL_GATEWAY_LNK_SEQ.nextval
		,p_CHANNELID
		,p_FRGATEWAYID
		,p_TOGATEWAYID
		,p_FRSCHEDULEID
		,p_TOSCHEDULEID
		,p_DESTTYPEID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_CHANNEL_GATEWAY_LNK_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_CHANNEL_GATEWAY_LNKI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_CHANNEL_GATEWAY_LNKI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CHANNEL_GATEWAY_LNKI' || SQLCODE|| ' '||SQLERRM);
END PR_CHANNEL_GATEWAY_LNKI;
/


-- PR_CHANNEL_GATEWAY_LNKS.sql

CREATE OR REPLACE PROCEDURE PR_CHANNEL_GATEWAY_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CHANNELGATEWAYID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CHANNEL_GATEWAY_LNKS
	Create Date:     2003-09-26 00:06:41
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CHANNEL_GATEWAY_LNKS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CHANNELGATEWAYID=>'||p_CHANNELGATEWAYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_CHANNELGATEWAYID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		CHANNELGATEWAYID
		,CHANNELID
		,FRGATEWAYID
		,TOGATEWAYID
		,FRSCHEDULEID
		,TOSCHEDULEID
		,DESTTYPEID
		,ROWTS
		,CREATEDATE
	FROM
		PR_CHANNEL_GATEWAY_LNK
	Where
		CHANNELGATEWAYID = p_CHANNELGATEWAYID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		CHANNELGATEWAYID
		,CHANNELID
		,FRGATEWAYID
		,TOGATEWAYID
		,FRSCHEDULEID
		,TOSCHEDULEID
		,DESTTYPEID
		,ROWTS
		,CREATEDATE
	FROM
		PR_CHANNEL_GATEWAY_LNK
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CHANNEL_GATEWAY_LNKS;
/


-- PR_CHANNEL_GATEWAY_LNKS_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_Channel_Gateway_LnkS_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_CHANNELID in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_Channel_Gateway_LnkS_AH1
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_CHANNEL_GATEWAY_LNKS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
         Open RC1 for
         select
                   ChannelGatewayId
                   ,ChannelId
                   ,FrGatewayId
                   ,ToGatewayId
                   ,FrScheduleId
                   ,ToScheduleId
                   ,DestTypeId
                   ,RowTS
         From
                   PR_CHANNEL_GATEWAY_LNK
         Where
                   CHANNELID = p_CHANNELID;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION
         When OTHERS Then
 --           Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --          Commit;
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_Channel_Gateway_LnkS_AH1' || SQLCODE|| ' '||SQLERRM);
End PR_Channel_Gateway_LnkS_AH1;
/


-- PR_CHANNEL_GATEWAY_LNKS_AH2.sql

CREATE OR REPLACE PROCEDURE pr_channel_gateway_lnks_ah2
	(
   p_returncd      OUT      INTEGER,
   p_debugflg      IN       PLS_INTEGER,
   RC1             OUT      db_ref.rct,
   RC2             OUT      db_ref.RCT,
   p_sortkey       IN       VARCHAR2,
   p_pagesizenbr   IN       PLS_INTEGER,
   p_pagenbr       IN       PLS_INTEGER,
   p_gatewayid     IN       NUMBER
	)
AS
/*
    ================================================================================
    Proc Name:       PR_Channel_Gateway_LnkS_AH2
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Return a list of channels for given gateway.
    2007-08-20 BHARANI J      6.1.1 Modified for Feature F104631

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_begints        TIMESTAMP       := fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   := 'PR_CHANNEL_GATEWAY_LNKS_AH2';
   v_exceptionflg   INTEGER         := 0;
   v_paramstxt      VARCHAR2 (4000) :=    'p_DebugFlg=>' || p_debugflg || CHR (10) ||
                                          ', p_GatewayID=>' || p_gatewayid || CHR (10) ||
                                          ', p_sortkey=>' || p_sortkey || CHR (10) ||
                                          ', p_PageSizeNbr=>' || p_pagesizenbr || CHR (10) ||
                                          ', p_PageNbr=>' || p_pagenbr || CHR (10)
                                          ;
   sqlstmt          VARCHAR2 (4000);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    sqlstmt :=
            'SELECT distinct FRCAPABILITYID,c.TOCAPABILITYID,'
         || '(select partnername from pr_partner where partnerid=c.frpartnerid) FROMPARTNERNAME,'
         || '(select partnername from pr_partner where partnerid=c.topartnerid) TOPARTNERNAME'
         || ' FROM PR_Channel_Gateway_Lnk l '
         || ' INNER JOIN PR_RoutingChannel c ON l.ChannelId = c.ChannelId'
         || ' WHERE l.FrGatewayId = '
         || ''''
         || p_gatewayid
         || ''''
         || ' OR l.toGatewayId = '
         || ''''
         || p_gatewayid
         || '''';



    if RC2%IsOpen then close RC2; end if;
       if NVL(p_pageNbr,0) in (0,1) then
         open RC2 for 'select count(*) TotalRowsNbr from (' || sqlstmt || ')';
    end if;

--------------
-- Add SortKey
--------------

  -- Order by Logic
      if p_sortkey is not null then
         sqlstmt := sqlstmt ||' order by '||p_sortkey;
      end if;

-------------------
 -- Add Paging logic
 -------------------

   sqlstmt := fn_paging_logic (p_pagenbr, p_pagesizenbr, sqlstmt);

   IF rc1%ISOPEN
   THEN
      CLOSE rc1;
   END IF;

   OPEN rc1 FOR sqlstmt;



   p_returncd := 0;
   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
EXCEPTION
   WHEN OTHERS
   THEN
      p_returncd := 1;
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            v_paramstxt,
                            p_debugflg,
                            v_exceptionflg
                           );
      raise_application_error (-20999,
                                  '20000-PR_Channel_Gateway_LnkS_AH2'
                               || SQLCODE
                               || ' '
                               || SQLERRM
                              );
END pr_channel_gateway_lnks_ah2;
/

-- PR_CHANNEL_GATEWAY_LNKU.sql

CREATE OR REPLACE Procedure PR_CHANNEL_GATEWAY_LNKU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_CHANNELGATEWAYID in NUMBER
	,p_CHANNELID in NUMBER
	,p_FRGATEWAYID in NUMBER
	,p_TOGATEWAYID in NUMBER
	,p_FRSCHEDULEID in NUMBER
	,p_TOSCHEDULEID in NUMBER
	,p_DESTTYPEID in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_CHANNEL_GATEWAY_LNKU
	Create Date:     2003-09-26 08:21:12
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CHANNEL_GATEWAY_LNKU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CHANNELGATEWAYID=>'||p_CHANNELGATEWAYID||CHR(10)
		||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
		||', p_FRGATEWAYID=>'||p_FRGATEWAYID||CHR(10)
		||', p_TOGATEWAYID=>'||p_TOGATEWAYID||CHR(10)
		||', p_FRSCHEDULEID=>'||p_FRSCHEDULEID||CHR(10)
		||', p_TOSCHEDULEID=>'||p_TOSCHEDULEID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_CHANNEL_GATEWAY_LNK
	SET
		CHANNELID=p_CHANNELID
		,      FRGATEWAYID=p_FRGATEWAYID
		,      TOGATEWAYID=p_TOGATEWAYID
		,      FRSCHEDULEID=p_FRSCHEDULEID
		,      TOSCHEDULEID=p_TOSCHEDULEID
		,      DESTTYPEID=p_DESTTYPEID
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		CHANNELGATEWAYID = p_CHANNELGATEWAYID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_CHANNEL_GATEWAY_LNKU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CHANNEL_GATEWAY_LNKU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CHANNEL_GATEWAY_LNKU:'||SQLERRM);
End PR_CHANNEL_GATEWAY_LNKU;
/


-- PR_CHECKDESTS.sql

CREATE OR REPLACE PROCEDURE         PR_CHECKDESTS
(
      p_ReturnCd          out pls_integer
    , p_debugFlg             in  pls_integer
    , rc1                    out db_ref.rct
    , rc2                    out db_ref.rct
    , p_FrPartnerId          out pls_integer
    , p_ToPartnerid          out pls_integer
    , p_FrPartnerId2         out pls_integer
    , p_ToPartnerId2         out pls_integer
    , p_FrBusinessId         in varchar2
    , p_ToBusinessId         in varchar2
    , p_CN                   in varchar2
    , p_L                    in varchar2
    , p_ST                   in varchar2
    , p_O                    in varchar2
    , p_OU                   in varchar2
    , p_C                    in varchar2
    , p_IPAddress            in varchar2
    , p_FrBusinessId2        in varchar2
    , p_ToBusinessId2        in varchar2
    )
AS
/*
================================================================================
    Proc Name:      PR_CheckDestS
    Create Date:    2002-10-07
    Author          J.Chen
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_ToPartnerId pr_partner.partnerid%Type; --number(10);
v_FrPartnerId pr_partner.partnerid%Type; --number(10);
v_sqlstmt varchar2(2000);
v_where varchar2(2000);
--
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_CHECKDESTS',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
                ||','||'RC1'
                ||','||'RC2'
                ||','||p_FrBusinessId
                ||','||p_ToBusinessId
                ||','||p_CN
                ||','||p_L
                ||','||p_ST
                ||','||p_O
                ||','||p_OU
                ||','||p_C
                --||','||P_serialnbr
                ||','||p_IPAddress
                ||','||p_FrBusinessId2
                ||','||p_ToBusinessId2
                );
End ProcAudit;
--
Function get_partnerid (p_busID in varchar2) return integer is
v_PartnerId pr_partner.partnerid%type;
begin
    begin
        select partnerId into v_PartnerId
        from pr_BusIdentifier
        where businessId = p_BusId;
        Exception
            when NO_DATA_FOUND then   v_PartnerId := -1;
            When Others Then
                p_ReturnCd := 1;
                Raise_application_error (-20999,'20000-PR_CHECKDESTS: Get_partnerid:'||p_BusId||': ' || SQLERRM);
    end;
    Return   v_PartnerId;
end get_partnerid;
--------
-- Main
--------
begin
p_ReturnCd := 0;
---------------------
-- PartnerId lookup
---------------------
    p_FrPartnerId := get_partnerid (p_FrBusinessId);
    p_ToPartnerId := get_partnerid (p_ToBusinessId);
If p_FrBusinessId2 Is Not Null Then
    p_FrPartnerId2 := get_partnerid (p_FrBusinessId2);
End If;
If p_ToBusinessId2 Is Not Null Then
    p_ToPartnerId2 := get_partnerid (p_ToBusinessId2);
End If;
----------------------
-- CertDestTypeId lookup
----------------------
v_where := FN_Build_Where('c.CN',  'VARCHAR2', p_CN)
                ||FN_BUILD_WHERE('c.L', 'VARCHAR2', p_L)
                ||FN_BUILD_WHERE('c.ST', 'VARCHAR2', p_ST)
                ||FN_BUILD_WHERE('c.O', 'VARCHAR2', p_O)
                ||FN_BUILD_WHERE('c.OU', 'VARCHAR2', p_OU)
                ||FN_BUILD_WHERE('c.C', 'VARCHAR2', p_C);
v_sqlstmt :=
        ' SELECT t.DestName '||
        ' FROM  pr_Cert c '||
        ' INNER JOIN pr_BusIdentifier fb on fb.PartnerId = c.PartnerId '||
        ' INNER JOIN pr_CertDestType d on d.CertId = c.Certid '||
        ' INNER JOIN pr_DestType t on t.desttypeid = d.desttypeid '||
        ' WHERE fb.BusinessId = '''|| p_FrBusinessId ||''''||
        ' AND   SSLCLIENTCERTFLG = 1 '||
        ' AND   EnabledFlg = 1 '||
        ' AND coalesce(c.deleteflag,0) <> 1 '||
        v_where;
IF RC1%isOpen THEN
    Close RC1;
END IF;
    Begin
    open rc1 for v_sqlstmt;
/*    Select
          t.DestName
    From  pr_Cert c
          inner join pr_BusIdentifier fb on fb.PartnerId = c.PartnerId
          inner join pr_CertDestType d on d.CertId = c.Certid
          inner join pr_DestType t on t.desttypeid = d.desttypeid
    where fb.BusinessId = p_FrBusinessId
        and   SSLCLIENTCERTFLG = 1
        and   c.CN = p_CN
        AND coalesce(a.deleteflag,0) <> 1;
*/
    EXCEPTION
        When NO_DATA_FOUND Then Null;
        When Others Then
        p_ReturnCd := 1;
        Raise_application_error (-20999,'20000-PR_CHECKDESTS: CertDest lookup:' || SQLERRM);
    End;
--------------------
-- IPDestTypeId lookup
--------------------
IF RC2%isOpen THEN
    Close RC2;
END IF;
    Begin
    open rc2 for
    Select
           t.DestName
    From  pr_IPAddress i
          inner join pr_BusIdentifier fb on fb.PartnerId = i.PartnerId
          inner join pr_IPDestType d on d.IPAddressId = i.IPAddressId
          inner join pr_DestType t on t.desttypeid = d.desttypeid
    where fb.BusinessId = p_FrBusinessId
    and   i.IPAddress = p_IPAddress;
    EXCEPTION
        When NO_DATA_FOUND Then Null;
        When Others Then
            p_ReturnCd := 1;
            Raise_application_error (-20999,'20000-PR_CHECKDESTS: IPDest lookup:' || SQLERRM);
    End;
   ProcAudit;
end;
/


-- PR_CLASSIFICATIONS_ALL.sql

CREATE OR REPLACE PROCEDURE         PR_CLASSIFICATIONS_ALL
(p_ReturnCd out integer
, p_DebugFlg in pls_integer
, RC1 out db_ref.RCT
, p_Locale in Varchar2
) AS

/*
    ===============================================================================
=
    Proc Name:       PR_CLASSIFICATIONS_ALL
    Create Date:     2003-01-21
    Author:          L.Ling

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ===============================================================================
=
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

PROCEDURE ProcAudit IS
BEGIN
   DB_ProcAuditLogI_Ah1('PR_CLASSIFICATIONS_ALL',v_beginTS,
   'p_ReturnCd'||','||p_DebugFlg||','||p_Locale
   ||','||'RC1');
END ProcAudit;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
---    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(v_Locale);

   IF RC1%isOpen THEN Close RC1; END IF;

   OPEN RC1 FOR
   SELECT
        CLASSIFICATIONID
   ,    RTRIM(RESOURCEKEY)  RESOURCEKEY
   ,    DESCRIPTION
   FROM PR_CLASSIFICATION
	;

---   EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';

   p_ReturnCd := 0;
   procAudit;
End PR_CLASSIFICATIONS_ALL;
/


-- PR_CONTACTD.sql

CREATE OR REPLACE Procedure PR_CONTACTD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_CONTACTID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CONTACTD
	Create Date:     2003-09-26 08:24:14
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTACTD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONTACTID=>'||p_CONTACTID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_CONTACT
	Where
		CONTACTID = p_CONTACTID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CONTACTD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_CONTACTD;
/


-- PR_CONTACTD_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_CONTACTd_ah1
(p_ReturnCd out integer, p_DebugFlg in pls_integer
,p_CONTACTID in NUMBER
) As
/*
================================================================================
Proc Name:      PR_CONTACTd
Create Date:    03/12/02
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_CONTACTd_ah1',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_CONTACTID
    );
End ProcAudit;
BEGIN
  begin
     delete from pr_alert_contact
     Where CONTACTID = p_CONTACTID
     ;
   Exception
     When NO_DATA_FOUND Then Null;
     When Others Then ProcAudit;
        Raise_application_error (-20999,'20000 PR_CONTACTd_ah1:' || SQLCODE|| ' '||SQLERRM);
        p_ReturnCd := 1;
  end;
Delete From PR_CONTACT
Where CONTACTID = p_CONTACTID
;
v_rowcnt := SQL%ROWCOUNT;
ProcAudit;
p_ReturnCd := 0;
Exception
    When NO_DATA_FOUND Then ProcAudit;
    Raise_application_error (-20999,'20001 PR_CONTACTd_ah1: No Data Found');
    When Others Then ProcAudit;
    Raise_application_error (-20999,'20002 PR_CONTACTd_ah1:' || SQLCODE|| ' '||SQLERRM);
    Rollback work;
    p_ReturnCd := 1;
End PR_CONTACTd_ah1;
/


-- PR_CONTACTI.sql

CREATE OR REPLACE Procedure PR_CONTACTI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_NAME1 in VARCHAR2
	,p_NAME2 in VARCHAR2
	,p_NAME3 in VARCHAR2
	,p_TITLE in VARCHAR2
	,p_EMAIL in VARCHAR2
	,p_ADDRESS in VARCHAR2
	,p_PHONEBUS in VARCHAR2
	,p_PHONEMBL in VARCHAR2
	,p_PHONEPGR in VARCHAR2
	,p_PHONEFAX in VARCHAR2
	,p_NOTIFYFLG in NUMBER
	,p_VISIBILITYFLG in NUMBER
	,p_LANGUAGELOCALE in VARCHAR2
	,p_FORMATLOCALE in VARCHAR2
	,p_TIMEZONE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_CONTACTI
	Create Date:     2003-09-26 00:01:43
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTACTI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_NAME1=>'||p_NAME1||CHR(10)
		||', p_NAME2=>'||p_NAME2||CHR(10)
		||', p_NAME3=>'||p_NAME3||CHR(10)
		||', p_TITLE=>'||p_TITLE||CHR(10)
		||', p_EMAIL=>'||p_EMAIL||CHR(10)
		||', p_ADDRESS=>'||p_ADDRESS||CHR(10)
		||', p_PHONEBUS=>'||p_PHONEBUS||CHR(10)
		||', p_PHONEMBL=>'||p_PHONEMBL||CHR(10)
		||', p_PHONEPGR=>'||p_PHONEPGR||CHR(10)
		||', p_PHONEFAX=>'||p_PHONEFAX||CHR(10)
		||', p_NOTIFYFLG=>'||p_NOTIFYFLG||CHR(10)
		||', p_VISIBILITYFLG=>'||p_VISIBILITYFLG||CHR(10)
		||', p_LANGUAGELOCALE=>'||p_LANGUAGELOCALE||CHR(10)
		||', p_FORMATLOCALE=>'||p_FORMATLOCALE||CHR(10)
		||', p_TIMEZONE=>'||p_TIMEZONE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_CONTACT(
		CONTACTID
		,PARTNERID
		,NAME1
		,NAME2
		,NAME3
		,TITLE
		,EMAIL
		,ADDRESS
		,PHONEBUS
		,PHONEMBL
		,PHONEPGR
		,PHONEFAX
		,NOTIFYFLG
		,VISIBILITYFLG
		,LANGUAGELOCALE
		,FORMATLOCALE
		,TIMEZONE
	) Values (
		PR_CONTACT_SEQ.nextval
		,p_PARTNERID
		,p_NAME1
		,p_NAME2
		,p_NAME3
		,p_TITLE
		,p_EMAIL
		,p_ADDRESS
		,p_PHONEBUS
		,p_PHONEMBL
		,p_PHONEPGR
		,p_PHONEFAX
		,p_NOTIFYFLG
		,p_VISIBILITYFLG
		,p_LANGUAGELOCALE
		,p_FORMATLOCALE
		,p_TIMEZONE
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_CONTACT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_CONTACTI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_CONTACTI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CONTACTI' || SQLCODE|| ' '||SQLERRM);
END PR_CONTACTI;
/


-- PR_CONTACTS.sql

CREATE OR REPLACE PROCEDURE PR_CONTACTS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CONTACTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CONTACTS
	Create Date:     2003-09-26 00:06:39
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTACTS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONTACTID=>'||p_CONTACTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_CONTACTID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		CONTACTID
		,PARTNERID
		,RTRIM(NAME1) NAME1
		,RTRIM(NAME2) NAME2
		,RTRIM(NAME3) NAME3
		,RTRIM(TITLE) TITLE
		,RTRIM(EMAIL) EMAIL
		,RTRIM(ADDRESS) ADDRESS
		,RTRIM(PHONEBUS) PHONEBUS
		,RTRIM(PHONEMBL) PHONEMBL
		,RTRIM(PHONEPGR) PHONEPGR
		,RTRIM(PHONEFAX) PHONEFAX
		,NOTIFYFLG
		,VISIBILITYFLG
		,ROWTS
		,CREATEDATE
		,RTRIM(LANGUAGELOCALE) LANGUAGELOCALE
		,RTRIM(FORMATLOCALE) FORMATLOCALE
		,RTRIM(TIMEZONE) TIMEZONE
	FROM
		PR_CONTACT
	Where
		CONTACTID = p_CONTACTID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		CONTACTID
		,PARTNERID
		,RTRIM(NAME1) NAME1
		,RTRIM(NAME2) NAME2
		,RTRIM(NAME3) NAME3
		,RTRIM(TITLE) TITLE
		,RTRIM(EMAIL) EMAIL
		,RTRIM(ADDRESS) ADDRESS
		,RTRIM(PHONEBUS) PHONEBUS
		,RTRIM(PHONEMBL) PHONEMBL
		,RTRIM(PHONEPGR) PHONEPGR
		,RTRIM(PHONEFAX) PHONEFAX
		,NOTIFYFLG
		,VISIBILITYFLG
		,ROWTS
		,CREATEDATE
		,RTRIM(LANGUAGELOCALE) LANGUAGELOCALE
		,RTRIM(FORMATLOCALE) FORMATLOCALE
		,RTRIM(TIMEZONE) TIMEZONE
	FROM
		PR_CONTACT
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CONTACTS;
/


-- PR_CONTACTS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_CONTACTS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PARTNERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CONTACTS_AH1
	Create Date:     2003-09-26 00:06:39
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTACTS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PARTNERID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		c.CONTACTID
		,c.PARTNERID
                ,RTRIM(p.PartnerName) PartnerName
		,RTRIM(c.NAME1) NAME1
		,RTRIM(c.NAME2) NAME2
		,RTRIM(c.NAME3) NAME3
		,RTRIM(c.TITLE) TITLE
		,RTRIM(c.EMAIL) EMAIL
		,RTRIM(c.ADDRESS) ADDRESS
		,RTRIM(c.PHONEBUS) PHONEBUS
		,RTRIM(c.PHONEMBL) PHONEMBL
		,RTRIM(c.PHONEPGR) PHONEPGR
		,RTRIM(c.PHONEFAX) PHONEFAX
		,c.NOTIFYFLG
		,c.VISIBILITYFLG
		,c.ROWTS
		,c.CREATEDATE
		,RTRIM(c.LANGUAGELOCALE) LANGUAGELOCALE
		,RTRIM(c.FORMATLOCALE) FORMATLOCALE
		,RTRIM(c.TIMEZONE) TIMEZONE
	FROM
		PR_CONTACT c
                Left outer join pr_partner p ON p.partnerId = c.partnerId
	Where
		c.PARTNERID = p_PartnerId
		AND notifyFlg=1
		AND visibilityFlg=1
		AND email is not null;
	ELSE
	OPEN RC1 FOR
	SELECT
		CONTACTID
		,PARTNERID
		,RTRIM(NAME1) NAME1
		,RTRIM(NAME2) NAME2
		,RTRIM(NAME3) NAME3
		,RTRIM(TITLE) TITLE
		,RTRIM(EMAIL) EMAIL
		,RTRIM(ADDRESS) ADDRESS
		,RTRIM(PHONEBUS) PHONEBUS
		,RTRIM(PHONEMBL) PHONEMBL
		,RTRIM(PHONEPGR) PHONEPGR
		,RTRIM(PHONEFAX) PHONEFAX
		,NOTIFYFLG
		,VISIBILITYFLG
		,ROWTS
		,CREATEDATE
		,RTRIM(LANGUAGELOCALE) LANGUAGELOCALE
		,RTRIM(FORMATLOCALE) FORMATLOCALE
		,RTRIM(TIMEZONE) TIMEZONE
	FROM
		PR_CONTACT
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CONTACTS_AH1;
/

-- PR_CONTACTS_ANYALERT.sql

CREATE OR REPLACE PROCEDURE PR_CONTACTS_ANYALERT
(
   p_ReturnCd out integer,
   p_DebugFlg in pls_integer,
   RC1 out db_ref.RCT,
   p_CONTACTID in NUMBER
) As
/*
================================================================================
Proc Name:      PR_CONTACTs
Create Date:    03/12/02 11:22:34 AM
Author          SYSTEM
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME                  DESCRIPTION
---------  --------------------  ---------------------------------------------
03/12/02   V Goel                Add any alerts for this contact
2003-07-01 M.Mount               Changed input parameters to standard.
2003-08-21 L.Ling                ECR: 1-1L1VD. Combined Address1, Address2, Address3 to Address.
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_count pls_integer:= 0;
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_CONTACTs_AnyAlert',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_CONTACTID
    );
End ProcAudit;
BEGIN
select count(*) into v_count from pr_alert_contact
Where CONTACTID = p_CONTACTID;
Open RC1 for
    select
        CONTACTID
    ,   PARTNERID
    ,   RTRIM(NAME1)  NAME1
    ,   RTRIM(NAME2)  NAME2
    ,   RTRIM(NAME3)  NAME3
    ,   RTRIM(TITLE)  TITLE
    ,   RTRIM(EMAIL)  EMAIL
    ,   RTRIM(ADDRESS)  ADDRESS
    ,   RTRIM(PHONEBUS)  PHONEBUS
    ,   RTRIM(PHONEMBL)  PHONEMBL
    ,   RTRIM(PHONEPGR)  PHONEPGR
    ,   RTRIM(PHONEFAX)  PHONEFAX
    ,   NOTIFYFLG
    ,   VISIBILITYFLG
    ,   ROWTS
    ,   CREATEDATE
    ,   case v_count
            when 0 then 0
            else 1
        end  AnyAlerts
    From PR_CONTACT
    Where CONTACTID = p_CONTACTID;

p_ReturnCd := 0;
procAudit;
End PR_CONTACTs_AnyAlert;
/


-- PR_CONTACTS_BYPID.sql

CREATE OR REPLACE PROCEDURE PR_CONTACTS_BYPID
(
    p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_PartnerID in NUMBER
    , p_Locale in varchar2
) As
/*
================================================================================
Proc Name:      PR_CONTACTs_PID
Create Date:    02/28/02 11:22:34 AM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_CONTACTs',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_PartnerID
    );
End ProcAudit;
BEGIN
    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);

    Open RC1 for
    SELECT
        CONTACTID
    ,   PARTNERID
    ,   RTRIM(NAME1)  NAME1
    ,   RTRIM(NAME2)  NAME2
    ,   RTRIM(NAME3)  NAME3
    ,   RTRIM(TITLE)  TITLE
    ,   RTRIM(EMAIL)  EMAIL
    ,   RTRIM(ADDRESS)  ADDRESS
    ,   RTRIM(PHONEBUS)  PHONEBUS
    ,   RTRIM(PHONEMBL)  PHONEMBL
    ,   RTRIM(PHONEPGR)  PHONEPGR
    ,   RTRIM(PHONEFAX)  PHONEFAX
    ,   NOTIFYFLG
    ,   VISIBILITYFLG
    ,   ROWTS
    ,   CREATEDATE
    ,RTRIM(LanguageLocale) LanguageLocale
    ,RTRIM(FormatLocale) FormatLocale
    ,RTRIM(TimeZone) TimeZone
    FROM PR_CONTACT
    WHERE PARTNERID = p_PARTNERID
    ORDER BY UPPER(NAME1), UPPER(NAME2), UPPER(NAME3);

    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';

    p_ReturnCd := 0;
    procAudit;
End PR_CONTACTs_byPID;
/


-- PR_CONTACTS_BYPID_FILTER.sql

CREATE OR REPLACE PROCEDURE PR_CONTACTS_BYPID_FILTER
(
    p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_PartnerID in NUMBER
    , p_Locale in varchar2
)
AS
/*
================================================================================
Proc Name:      PR_CONTACTs_PID
Create Date:    02/28/02 11:22:34 AM
Author          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2006-04-12 Shishir N        6.1 60217 Partner Migration

================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
DB_ProcAuditLogI_Ah1('PR_CONTACTs',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
||','||p_PartnerID
);
End ProcAudit;
BEGIN
    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);
  IF p_PartnerID is NOT NULL Then
    OPEN RC1 FOR
    SELECT
		        CONTACTID
		    ,   pc.PARTNERID
		    ,   RTRIM(NAME1)  NAME1
		    ,   RTRIM(NAME2)  NAME2
		    ,   RTRIM(NAME3)  NAME3
		    ,   RTRIM(TITLE)  TITLE
		    ,   RTRIM(EMAIL)  EMAIL
		    ,   RTRIM(ADDRESS)  ADDRESS
		    ,   RTRIM(PHONEBUS)  PHONEBUS
		    ,   RTRIM(PHONEMBL)  PHONEMBL
		    ,   RTRIM(PHONEPGR)  PHONEPGR
		    ,   RTRIM(PHONEFAX)  PHONEFAX
		    ,   NOTIFYFLG
		    ,   VISIBILITYFLG
		    ,   pc.ROWTS
		    ,   pc.CREATEDATE
		    ,   RTRIM(LanguageLocale)  LanguageLocale
		    ,   RTRIM(FormatLocale)  FormatLocale
		    ,   RTRIM(TimeZone)  TimeZone
		    ,   pp.PARTNERLOGIN
		  FROM  PR_CONTACT pc, PR_PARTNER pp
		  WHERE pc.PARTNERID = p_PARTNERID
		  AND   pc.PARTNERID = pp.PARTNERID
		  AND   contactid NOT IN (select contactid from pr_user where partnerid = p_partnerid)
		  ORDER BY Upper(NAME1),Upper(NAME2), Upper(NAME3);

		ELSE
		OPEN RC1 FOR
		SELECT
		        CONTACTID
		    ,   pc.PARTNERID
		    ,   RTRIM(NAME1)  NAME1
		    ,   RTRIM(NAME2)  NAME2
		    ,   RTRIM(NAME3)  NAME3
		    ,   RTRIM(TITLE)  TITLE
		    ,   RTRIM(EMAIL)  EMAIL
		    ,   RTRIM(ADDRESS)  ADDRESS
		    ,   RTRIM(PHONEBUS)  PHONEBUS
		    ,   RTRIM(PHONEMBL)  PHONEMBL
		    ,   RTRIM(PHONEPGR)  PHONEPGR
		    ,   RTRIM(PHONEFAX)  PHONEFAX
		    ,   NOTIFYFLG
		    ,   VISIBILITYFLG
		    ,   pc.ROWTS
		    ,   pc.CREATEDATE
		    ,   RTRIM(LanguageLocale)  LanguageLocale
		    ,   RTRIM(FormatLocale)  FormatLocale
		    ,   RTRIM(TimeZone)  TimeZone
		    ,   pp.PARTNERLOGIN
		  FROM  PR_CONTACT pc, PR_PARTNER pp
		  WHERE pc.PARTNERID = pp.PARTNERID
		  ORDER BY Upper(NAME1),Upper(NAME2), Upper(NAME3);

		
		END IF;

    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';

    p_ReturnCd := 0;
    procAudit;
End PR_CONTACTS_BYPID_FILTER;
/


-- PR_CONTACTTYPES_ALL.sql

CREATE OR REPLACE PROCEDURE         PR_CONTACTTYPES_ALL
(p_ReturnCd out integer
, p_DebugFlg in pls_integer
, RC1 out db_ref.RCT
, p_Locale in Varchar2
)
AS

/*
    ================================================================================
    Proc Name:       PR_CONTACTTYPES_ALL
    Create Date:     2003-01-20 13:51:39
    Author:          SYSTEM

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

PROCEDURE ProcAudit IS
BEGIN
   DB_ProcAuditLogI_Ah1('PR_CONTACTTYPES_ALL',v_beginTS,
   'p_ReturnCd'||','||p_DebugFlg
   ||','||p_Locale
   ||','||'RC1'
   );
END ProcAudit;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    SELECT
        CONTACTTYPEID
        ,  RTRIM(RESOURCEKEY) RESOURCEKEY
        ,  DESCRIPTION
    FROM PR_CONTACTTYPE
;

    p_ReturnCd := 0;
    procAudit;
END PR_CONTACTTYPES_ALL;
/


-- PR_CONTACTU.sql

CREATE OR REPLACE Procedure PR_CONTACTU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_CONTACTID in NUMBER
	,p_PARTNERID in NUMBER
	,p_NAME1 in VARCHAR2
	,p_NAME2 in VARCHAR2
	,p_NAME3 in VARCHAR2
	,p_TITLE in VARCHAR2
	,p_EMAIL in VARCHAR2
	,p_ADDRESS in VARCHAR2
	,p_PHONEBUS in VARCHAR2
	,p_PHONEMBL in VARCHAR2
	,p_PHONEPGR in VARCHAR2
	,p_PHONEFAX in VARCHAR2
	,p_NOTIFYFLG in NUMBER
	,p_VISIBILITYFLG in NUMBER
	,p_ROWTS in NUMBER
	,p_LANGUAGELOCALE in VARCHAR2
	,p_FORMATLOCALE in VARCHAR2
	,p_TIMEZONE in VARCHAR2
) as

/*
	================================================================================
	Proc Name:       PR_CONTACTU
	Create Date:     2003-09-26 08:21:14
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTACTU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONTACTID=>'||p_CONTACTID||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_NAME1=>'||p_NAME1||CHR(10)
		||', p_NAME2=>'||p_NAME2||CHR(10)
		||', p_NAME3=>'||p_NAME3||CHR(10)
		||', p_TITLE=>'||p_TITLE||CHR(10)
		||', p_EMAIL=>'||p_EMAIL||CHR(10)
		||', p_ADDRESS=>'||p_ADDRESS||CHR(10)
		||', p_PHONEBUS=>'||p_PHONEBUS||CHR(10)
		||', p_PHONEMBL=>'||p_PHONEMBL||CHR(10)
		||', p_PHONEPGR=>'||p_PHONEPGR||CHR(10)
		||', p_PHONEFAX=>'||p_PHONEFAX||CHR(10)
		||', p_NOTIFYFLG=>'||p_NOTIFYFLG||CHR(10)
		||', p_VISIBILITYFLG=>'||p_VISIBILITYFLG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_LANGUAGELOCALE=>'||p_LANGUAGELOCALE||CHR(10)
		||', p_FORMATLOCALE=>'||p_FORMATLOCALE||CHR(10)
		||', p_TIMEZONE=>'||p_TIMEZONE||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_CONTACT
	SET
		PARTNERID=p_PARTNERID
		,      NAME1=p_NAME1
		,      NAME2=p_NAME2
		,      NAME3=p_NAME3
		,      TITLE=p_TITLE
		,      EMAIL=p_EMAIL
		,      ADDRESS=p_ADDRESS
		,      PHONEBUS=p_PHONEBUS
		,      PHONEMBL=p_PHONEMBL
		,      PHONEPGR=p_PHONEPGR
		,      PHONEFAX=p_PHONEFAX
		,      NOTIFYFLG=p_NOTIFYFLG
		,      VISIBILITYFLG=p_VISIBILITYFLG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      LANGUAGELOCALE=p_LANGUAGELOCALE
		,      FORMATLOCALE=p_FORMATLOCALE
		,      TIMEZONE=p_TIMEZONE
	Where
		CONTACTID = p_CONTACTID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_CONTACTU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CONTACTU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CONTACTU:'||SQLERRM);
End PR_CONTACTU;
/


-- PR_CONTACT_CONTACTTYPED.sql

CREATE OR REPLACE Procedure PR_CONTACT_CONTACTTYPED

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_CONTACTID in NUMBER
	,p_CONTACTTYPEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CONTACT_CONTACTTYPED
	Create Date:     2003-09-26 08:24:15
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTACT_CONTACTTYPED';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONTACTID=>'||p_CONTACTID||CHR(10)
		||', p_CONTACTTYPEID=>'||p_CONTACTTYPEID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_CONTACT_CONTACTTYPE
	Where
		CONTACTID = p_CONTACTID
		AND CONTACTTYPEID = p_CONTACTTYPEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CONTACT_CONTACTTYPED: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_CONTACT_CONTACTTYPED;
/


-- PR_CONTACT_CONTACTTYPED_ID.sql

CREATE OR REPLACE PROCEDURE         PR_CONTACT_CONTACTTYPED_ID

(p_ReturnCd out integer
, p_DebugFlg in pls_integer
,p_CONTACTID in NUMBER
)
AS

/*
    ================================================================================
    Proc Name:       PR_CONTACT_CONTACTTYPED_ID
    Create Date:     2003-01-20 13:57:09
    Author:          L.Ling
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

PROCEDURE ProcAudit IS
BEGIN
   DB_ProcAuditLogI_Ah1('PR_CONTACT_CONTACTTYPED_ID',v_beginTS,
   'p_ReturnCd'||','||p_DebugFlg
   ||','||'RC1'
   ||','||p_CONTACTID
   );
END ProcAudit;

BEGIN
   DELETE FROM PR_CONTACT_CONTACTTYPE
   WHERE CONTACTID = p_CONTACTID;


   ProcAudit;
   p_ReturnCd := 0;
   Exception
   WHEN NO_DATA_FOUND THEN ProcAudit;
       Raise_application_error (-20999,'20002 PR_CONTACT_CONTACTTYPED_ID: No Data Found');
   WHEN OTHERS THEN ProcAudit;
       Raise_application_error (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
       p_ReturnCd := 1;
END PR_CONTACT_CONTACTTYPED_ID;
/


-- PR_CONTACT_CONTACTTYPEI.sql

CREATE OR REPLACE Procedure PR_CONTACT_CONTACTTYPEI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_CONTACTID in NUMBER
	,p_CONTACTTYPEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CONTACT_CONTACTTYPEI
	Create Date:     2003-09-26 00:03:48
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTACT_CONTACTTYPEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONTACTID=>'||p_CONTACTID||CHR(10)
		||', p_CONTACTTYPEID=>'||p_CONTACTTYPEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_CONTACT_CONTACTTYPE(
		CONTACTID
		,CONTACTTYPEID
	) Values (
		p_CONTACTID
		,p_CONTACTTYPEID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_CONTACT_CONTACTTYPEI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_CONTACT_CONTACTTYPEI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CONTACT_CONTACTTYPEI' || SQLCODE|| ' '||SQLERRM);
END PR_CONTACT_CONTACTTYPEI;
/


-- PR_CONTACT_CONTACTTYPES_ID.sql

CREATE OR REPLACE PROCEDURE         PR_CONTACT_CONTACTTYPES_ID
    (p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_Locale in Varchar2
    , p_CONTACTID in NUMBER
    )
AS
/*
    ================================================================================
    Proc Name:       PR_CONTACT_CONTACTTYPES_ID
    Create Date:     2003-01-23
    Author:          L.Ling

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
begin
   DB_ProcAuditLogI_Ah1('PR_CONTACT_CONTACTTYPES_ID',v_beginTS,
   'p_ReturnCd'||','||p_DebugFlg
   ||','||p_Locale
   ||','||'RC1'
   ||','||p_CONTACTID
   );
End ProcAudit;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
---    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(v_Locale);

    IF RC1%isOpen THEN Close RC1; END IF;
    OPEN RC1 FOR
    SELECT
       c.CONTACTTYPEID
    ,   RTRIM(c.RESOURCEKEY) RESOURCEKEY
    ,   C.DESCRIPTION
    ,   cp.ROWTS
    FROM   PR_CONTACTTYPE c, PR_CONTACT_CONTACTTYPE cp
    WHERE  c.CONTACTTYPEID = cp.CONTACTTYPEID
    AND    cp.CONTACTID = p_CONTACTID
    ORDER BY c.CONTACTTYPEID 
	;

---    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';

    p_ReturnCd := 0;
    procAudit;
END PR_CONTACT_CONTACTTYPES_ID;
/


-- PR_CONTACT_CONTACTTYPEU.sql

CREATE OR REPLACE Procedure PR_CONTACT_CONTACTTYPEU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_CONTACTID in NUMBER
	,p_CONTACTTYPEID in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_CONTACT_CONTACTTYPEU
	Create Date:     2003-09-26 08:21:16
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTACT_CONTACTTYPEU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CONTACTID=>'||p_CONTACTID||CHR(10)
		||', p_CONTACTTYPEID=>'||p_CONTACTTYPEID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_CONTACT_CONTACTTYPE
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		CONTACTID = p_CONTACTID
		AND CONTACTTYPEID = p_CONTACTTYPEID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_CONTACT_CONTACTTYPEU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CONTACT_CONTACTTYPEU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CONTACT_CONTACTTYPEU:'||SQLERRM);
End PR_CONTACT_CONTACTTYPEU;
/


-- PR_CONTACT_CONTACTTYPEU_AH1.sql

CREATE OR REPLACE PROCEDURE  PR_CONTACT_CONTACTTYPEU_AH1
(
    p_ReturnCd out integer
    ,p_DebugFlg in pls_integer
    ,RC1 out db_ref.RCT
    ,p_CONTACTID in NUMBER
    ,p_CONTACTTYPEID in NUMBER
    ,p_ROWTS in NUMBER
) as

/*
    ================================================================================
    Proc Name:       PR_CONTACT_CONTACTTYPEU_AH1
    Create Date:     2003-01-20 13:57:08
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-01-20 SYSTEM           3.4 Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_GET_DATE(); -- Performance related variables
v_rowTS integer :=0;
zeroUpdate  Exception;

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_CONTACT_CONTACTTYPEU_AH1',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_CONTACTID
    ||','||p_CONTACTTYPEID
    ||','||p_ROWTS
    );
End ProcAudit;

BEGIN
    v_rowcnt := 0;
    Update PR_CONTACT_CONTACTTYPE SET
        CONTACTID=p_CONTACTID
        ,      CONTACTTYPEID=p_CONTACTTYPEID
        ,      ROWTS= p_ROWTS + 1
        ,      UPDATEDATE= FN_GET_DATE()
    Where
        CONTACTID = p_CONTACTID
        and CONTACTTYPEID = p_CONTACTTYPEID
        And ROWTS = p_ROWTS returning ROWTS into v_rowTS;
    v_rowcnt := SQL%rowcount;
    if (v_rowcnt = 0) then  raise ZeroUpdate ; end if;
    IF RC1%isOpen THEN Close RC1; END IF;
    OPEN RC1 for select v_rowTS ROWTS from Dual;
    p_ReturnCd := 0;
    ProcAudit;
EXCEPTION
    WHEN TOO_MANY_ROWS then ProcAudit;
    raise_application_error(-20999, '20100 - PR_CONTACT_CONTACTTYPEU_AH1: Too Many
    Rows');
    WHEN ZeroUpdate then ProcAudit;
    raise_application_error(-20999,'20100 - PR_CONTACT_CONTACTTYPEU_AH1: No Data
    Found');
    WHEN OTHERS then ProcAudit;
    p_ReturnCd := 1;
    raise_application_error(-20999,'20100 - PR_CONTACT_CONTACTTYPEU_AH1:'||sqlerrm);

End;
/


-- PR_CONTROLNUMD.sql

Create or Replace Procedure PR_CONTROLNUMD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer   
	,p_PARTNERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CONTROLNUMD
	Create Date:     2004-08-09 11:10:38
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTROLNUMD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_CONTROLNUM
	Where 
		PARTNERID = p_PARTNERID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CONTROLNUMD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_CONTROLNUMD;
/


-- PR_CONTROLNUMI.sql

Create or Replace Procedure PR_CONTROLNUMI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PartnerID IN INTEGER
   ,p_INTCTLNUM in VARCHAR2
	,p_GRPCTLNUM in VARCHAR2
	,p_TXCTLNUM in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_CONTROLNUMI
	Create Date:     2004-08-09 11:09:27
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTROLNUMI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
      ||', p_PartnerID=>'||p_PartnerID||CHR(10)
		||', p_INTCTLNUM=>'||p_INTCTLNUM||CHR(10)
		||', p_GRPCTLNUM=>'||p_GRPCTLNUM||CHR(10)
		||', p_TXCTLNUM=>'||p_TXCTLNUM||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_CONTROLNUM(
		PARTNERID
		,INTCTLNUM
		,GRPCTLNUM
		,TXCTLNUM
      ,UPDATEDATE
	   ,CREATEDATE
	) Values (
		p_Partnerid
		,p_INTCTLNUM
		,p_GRPCTLNUM
		,p_TXCTLNUM
      ,v_BeginTS
	   ,v_BeginTS
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_CONTROLNUMI' || SQLCODE|| ' '||SQLERRM);
END PR_CONTROLNUMI;
/


-- PR_CONTROLNUMS.sql

CREATE OR REPLACE PROCEDURE PR_CONTROLNUMS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PARTNERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CONTROLNUMS
	Create Date:     2004-08-09 11:10:51
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created
  2006-04-12 Shishir N     6.1    60217 Partner Migration

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTROLNUMS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PARTNERID IS NULL 
 THEN
	OPEN RC1 FOR 
	SELECT pc.partnerid
	, pc.rowts
	, intctlnum
	, grpctlnum
	, txctlnum
	, pr.PARTNERLOGIN
		FROM 
			PR_CONTROLNUM PC,
			PR_partner pr
			where pr.partnerid = pc.partnerid
	;
	ELSE
	OPEN RC1 FOR 
	SELECT pc.partnerid
	, pc.rowts
	, intctlnum
	, grpctlnum
	, txctlnum
	, pr.PARTNERLOGIN
		FROM 
			PR_CONTROLNUM pc,
			pr_partner pr
	   WHERE pc.partnerid = p_PartnerID
	   and pr.partnerid = pc.partnerid
		;

	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CONTROLNUMS;
/


-- PR_CONTROLNUMU.sql

Create or Replace Procedure PR_CONTROLNUMU
	(
	 p_ReturnCd out integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
   ,p_INTCTLNUM in VARCHAR2
	,p_GRPCTLNUM in VARCHAR2
	,p_TXCTLNUM in VARCHAR2
   ,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_CONTROLNUMU
	Create Date:     2004-08-09 11:10:23
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-08-09 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CONTROLNUMU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_INTCTLNUM=>'||p_INTCTLNUM||CHR(10)
		||', p_GRPCTLNUM=>'||p_GRPCTLNUM||CHR(10)
		||', p_TXCTLNUM=>'||p_TXCTLNUM||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_CONTROLNUM
	SET
		UPDATEDATE= v_BeginTS
		,      ROWTS= p_RowTS + 1
		,      INTCTLNUM=p_INTCTLNUM
		,      GRPCTLNUM=p_GRPCTLNUM
		,      TXCTLNUM=p_TXCTLNUM
	Where 
		PARTNERID = p_PARTNERID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_CONTROLNUMU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CONTROLNUMU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_CONTROLNUMU:'||SQLERRM);
End PR_CONTROLNUMU;
/


-- PR_CRLS.SQL

CREATE OR REPLACE PROCEDURE PR_CRLS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CRLID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_CRLS
	Create Date:     2004-06-26 00:07:06
	Author:          Shishir N

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM         4.3   Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_CRLS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CRLID=>'||p_CRLID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_CRLID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		CRLID
		,CRL
		,ISSUERDN
		,ROWTS
		,CREATEDATE
		,UPDATEDATE
	FROM
		CF_CRL
	Where
		CRLID = p_CRLID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		CRLID
		,CRL
		,ISSUERDN
		,ROWTS
		,CREATEDATE
		,UPDATEDATE
	FROM
		CF_CRL
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_CRLS;
/


-- PR_DestTypeI_AH1.SQL

CREATE OR REPLACE PROCEDURE PR_DESTTYPEI_AH1
    (
    p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,RC1 out db_ref.RCT
    ,p_Locale in varchar2
    ,p_DESTNAME in VARCHAR2
    ,p_DESTDISPLAYNAME in Varchar2
    ,p_DEFAULTFLG in NUMBER
    ,p_DIRFLG in NUMBER
    )
AS
/*
    ================================================================================
    Proc Name:       PR_DESTTYPEI_AH1
    Create Date:     2003-05-08 10:05:58
    Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-01 S.Priani      4.2.2  Removed NLS
    
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_DESTTYPEI';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_DESTNAME=>'||p_DESTNAME||CHR(10)
        ||', p_DEFAULTFLG=>'||p_DEFAULTFLG||CHR(10)
        ||', p_DIRFLG=>'||p_DIRFLG||CHR(10)
        ;
v_RowTS  integer;
v_RowTS_loc integer;
v_DestTypeId number;
--
Procedure Get_DestTypeID is
Begin
    Select DestTypeID into v_DestTypeID
    From PR_DestType
    Where DestNAME = p_DestNAME;
End Get_DestTypeID;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    Insert into PR_DestType(
        DESTTYPEID
        ,DESTNAME
        ,DEFAULTFLG
        ,DIRFLG
        ,DESCRIPTION
    ) Values (
        PR_DESTTYPE_SEQ.nextval
        ,p_DESTNAME
        ,p_DEFAULTFLG
        ,p_DIRFLG
        ,p_DestDisplayName
    ) RETURNING DestTypeId, RowTS INTO v_DestTypeId, v_RowTS;
    p_ReturnCd := 0;
    v_RowCnt := sql%ROWCOUNT;
    Open RC1 FOR Select
                  v_DestTypeId As ID,
                   v_RowTS As RowTS
                 From Dual;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
            RAISE_APPLICATION_ERROR (-20999,'20000 PR_DestTypeI_AH1' || SQLCODE|| ' '||SQLERRM);
        When DUP_VAL_ON_INDEX then
            Get_DestTypeId;
        When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR (-20999,'20002 PR_DestTypeI_AH1' || SQLCODE|| ' '||SQLERRM);
END PR_DESTTYPEI_AH1;
/


-- PR_DESTTYPES.sql

CREATE OR REPLACE PROCEDURE PR_DESTTYPES
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  Db_Ref.RCT
	, p_lastUpdtTime  IN  TIMESTAMP
	)
AS

/*
	================================================================================
	Proc Name:       PR_DESTTYPES
	Create Date:     2004-06-26 00:10:33
	Author:          Shishir Narain

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'PR_DESTTYPES';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_lastUpdtTime=>'||p_lastUpdtTime||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_lastUpdtTime IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		DESTTYPEID
		,RTRIM(DESTNAME) DESTNAME
		,DEFAULTFLG
		,ROWTS
		,CREATEDATE
		,DIRFLG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,DESCRIPTION

	FROM
		PR_DESTTYPE
	WHERE
		updatedate > p_lastupdttime AND
		hiddenflg <> 1
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		DESTTYPEID
		,RTRIM(DESTNAME) DESTNAME
		,DEFAULTFLG
		,ROWTS
		,CREATEDATE
		,DIRFLG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,DESCRIPTION
	FROM
		PR_DESTTYPE
	WHERE
		hiddenflg <> 1
	;
	END IF;
	p_ReturnCd := 0;
	Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
END PR_DESTTYPES;
/


-- PR_DESTTYPEU_AH1.SQL

CREATE OR REPLACE PROCEDURE PR_DESTTYPEU_AH1
(
   p_ReturnCd out INTEGER
   ,p_DebugFlg in PLS_INTEGER
   ,RC1 out db_ref.RCT
   ,p_LOCALE in VARCHAR2
   ,p_DESTTYPEID in NUMBER
   ,p_DESTNAME in VARCHAR2
   ,p_DEFAULTFLG in NUMBER
   ,p_ROWTS in NUMBER
   ,p_DESTDISPLAYNAME in VARCHAR2
   ,p_LOC_ROWTS in NUMBER
   ,p_DIRFLG in NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       PR_DestTypeU_AH1
    Create Date:     2003-05-01 09:43:31
    Author:          SYSTEM
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_LocRowCnt       integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_DestTypeU_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
       ||', p_DestNAME=>'||p_DestNAME||CHR(10)
       ||', p_DEFAULTFLG=>'||p_DEFAULTFLG||CHR(10)
       ||', p_ROWTS=>'||p_ROWTS||CHR(10)
       ||', p_LOCALE=>'||p_LOCALE||CHR(10)
       ||', p_DESTDISPLAYNAME=>'||p_DESTDISPLAYNAME||CHR(10)
       ||', p_LOC_ROWTS=>'||p_ROWTS||CHR(10)
       ||', p_DIRFLG=>'||p_DIRFLG||CHR(10)
        ;
v_RowTS        integer :=0;
v_LocRowTS     integer :=0;
zeroUpdate     Exception;
v_PR_DestType_LOC_ROWTS  integer :=0;
v_PR_DestType_ROWID      rowid;
v_PR_DestType_LOC_ROWID  rowid;
v_PR_DestType_ROWTS  integer :=0;
v_DestNAME   PR_DestType.DestNAME%Type;
v_DefaultFLG       PR_DestType.DefaultFLG%Type;
v_DESTDISPLAYNAME   PR_DestType.DESCRIPTION%Type;
RC2           db_ref.RCT;
v_ReturnCd    integer;
v_DirFlg       PR_DestType.DirFlg%Type;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
   p_ReturnCd := 0;
    v_RowCnt := 0;
   SELECT  ROWID, ROWTS, DestNAME, DefaultFLG, DirFlg,Description
   INTO    v_PR_DestType_ROWID, v_PR_DestType_ROWTS, v_DestNAME, v_DefaultFLG, v_DirFlg,v_DESTDISPLAYNAME
   FROM    PR_DestType
   WHERE   DestTypeID = p_DestTypeId
   FOR UPDATE; -- row level locking

   IF NVL(v_DestNAME, ' ') <> NVL(p_DestNAME, ' ')  OR
      NVL(v_DefaultFLG, -1)   <> NVL(p_DefaultFLG, -1) OR
      NVL(v_DESTDISPLAYNAME, -1)   <> NVL(p_DESTDISPLAYNAME, -1) OR
      NVL(v_DirFlg, -1)   <> NVL(p_DirFlg, -1) THEN -- data changes
       IF NVL(v_PR_DestType_ROWTS, -1) <> NVL(p_ROWTS, -1) THEN
           ROLLBACK;
           p_ReturnCd := 1;
           v_ExceptionFlg := 1;
           DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'ROWTS Changes in PR_DestType,'|| v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
           RAISE_APPLICATION_ERROR(-20999,'20100 - PR_DestTypeU_AH1 in PR_DestType: ROWTS Changes');
       END IF;
       BEGIN
            UPDATE PR_DestType
            SET
                DestNAME=p_DestNAME
                ,      DefaultFLG=p_DefaultFLG
                ,      DIRFLG=p_DIRFLG
                ,      DESCRIPTION=p_DESTDISPLAYNAME
                ,      ROWTS= p_RowTS + 1
                ,      UPDATEDATE= FN_GET_DATE()
            WHERE   ROWID = v_PR_DestType_ROWID returning RowTS into v_RowTS;
            v_RowCnt := SQL%rowcount;
            if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
            p_ReturnCd := 0;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
           EXCEPTION
                WHEN TOO_MANY_ROWS Then
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                    Commit;
                    RAISE_APPLICATION_ERROR(-20999, '20100 - PR_DestTypeU_AH1: Too Many Rows');
                WHEN ZeroUpdate then
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                    Commit;
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_DestTypeU_AH1: No Data Found');
                WHEN OTHERS then
                    ROLLBACK work;
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
                    Commit;
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_DestTypeU_AH1:'||SQLERRM);
       END;
   END IF;
   IF RC1%isOpen THEN Close RC1; END IF;
   OPEN RC1 FOR
   SELECT v_RowTS As RowTS,
          v_LocRowTS As Loc_RowTS
   FROM Dual;
End PR_DestTypeU_AH1;
/


-- PR_DESTTYPE_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE PR_DESTTYPE_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_DESTTYPEID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_DESTTYPE_LOC_VWS
	Create Date:     2003-09-26 00:10:33
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
    	2004-03-02 S.Priani	 4.2.2  Removed NLS

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_DESTTYPE_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_DESTTYPEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		DESTTYPEID
		,RTRIM(DESTNAME) DESTNAME
		,DEFAULTFLG
		,ROWTS
		,CREATEDATE
		,DIRFLG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,DESCRIPTION

	FROM
		PR_DESTTYPE
	Where
		DESTTYPEID = p_DESTTYPEID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		DESTTYPEID
		,RTRIM(DESTNAME) DESTNAME
		,DEFAULTFLG
		,ROWTS
		,CREATEDATE
		,DIRFLG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,DESCRIPTION
	FROM
		PR_DESTTYPE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_DESTTYPE_LOC_VWS;
/


-- PR_EMAILD.sql

Create or Replace Procedure PR_EMAILD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_BUSINESSEMAILID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_EMAILD
	Create Date:     2006-09-01 18:13:01
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_EMAILD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSINESSEMAILID=>'||p_BUSINESSEMAILID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_EMAIL
	Where 
		BUSINESSEMAILID = p_BUSINESSEMAILID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
Exception
	When ZeroDelete Then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
	When OTHERS Then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
End PR_EMAILD;
/


-- PR_EMAILI.sql

CREATE OR REPLACE PROCEDURE PR_EMAILI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_EMAILADDRESS in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_EMAILI
	Create Date:     2006-09-01 18:13:01
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_EMAILI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_EMAILADDRESS=>'||p_EMAILADDRESS||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_EMAIL(
		BUSINESSEMAILID
		,PARTNERID
		,EMAILADDRESS
	) Values (
		PR_EMAIL_SEQ.nextval
		,p_PARTNERID
		,LOWER(p_EMAILADDRESS)
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		PR_EMAIL_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
END PR_EMAILI;
/


-- PR_EMAILS.sql

CREATE OR REPLACE PROCEDURE PR_EMAILS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_BUSINESSEMAILID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_EMAILS
	Create Date:     2006-09-01 18:13:02
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_EMAILS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSINESSEMAILID=>'||p_BUSINESSEMAILID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_BUSINESSEMAILID IS NOT NULL THEN
	OPEN RC1 FOR 
	SELECT
		BUSINESSEMAILID
		,PARTNERID
		,RTRIM(EMAILADDRESS) EMAILADDRESS
		,ROWTS
		,CREATEDATE
	FROM 
		PR_EMAIL
	Where 
		BUSINESSEMAILID = p_BUSINESSEMAILID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		BUSINESSEMAILID
		,PARTNERID
		,RTRIM(EMAILADDRESS) EMAILADDRESS
		,ROWTS
		,CREATEDATE
	FROM 
		PR_EMAIL
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_EMAILS;
/


-- PR_EMAILS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_EMAILS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PARTNERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_EMAILS_AH1
	Create Date:     2006-09-01 18:13:02
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_EMAILS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PARTNERID IS NOT NULL THEN
	OPEN RC1 FOR 
	SELECT
		BUSINESSEMAILID
		,PARTNERID
		,RTRIM(EMAILADDRESS) EMAILADDRESS
		,ROWTS
		,CREATEDATE
	FROM 
		PR_EMAIL
	Where 
		PARTNERID = p_PARTNERID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		BUSINESSEMAILID
		,PARTNERID
		,RTRIM(EMAILADDRESS) EMAILADDRESS
		,ROWTS
		,CREATEDATE
	FROM 
		PR_EMAIL
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_EMAILS_AH1;
/


-- PR_EMAILU.sql

CREATE OR REPLACE PROCEDURE PR_EMAILU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_BUSINESSEMAILID in NUMBER
	,p_PARTNERID in NUMBER
	,p_EMAILADDRESS in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_EMAILU
	Create Date:     2006-09-01 18:13:02
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-09-01 S.Priani	6.1	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_EMAILU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSINESSEMAILID=>'||p_BUSINESSEMAILID||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_EMAILADDRESS=>'||p_EMAILADDRESS||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_EMAIL
	SET
		      EMAILADDRESS=LOWER(p_EMAILADDRESS)
		,	PARTNERID=p_PARTNERID
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		BUSINESSEMAILID = p_BUSINESSEMAILID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN ZeroUpdate then
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
	WHEN OTHERS then
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE;
End PR_EMAILU;
/


-- PR_ERRORFLOWD.sql

CREATE OR REPLACE Procedure PR_ERRORFLOWD
(
	 p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, p_ErrorFlowID		in		INTEGER
)
AS

/*


	================================================================================
	Proc Name:       PR_ERRORFLOWU
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ERRORFLOWU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;
v_RowTS INTEGER;


v_wsattrvalueid    INTEGER;
ZeroUpdate EXCEPTION;


BEGIN
	
	
		v_RowCnt := 0;
		
		DELETE FROM PR_ERRORFLOW
		WHERE
		errorflowid = p_ErrorFlowID
		;
		
		p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ERRORFLOWU' || SQLCODE|| ' '||SQLERRM);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GWYATTRU: No Data Found');
					
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ERRORFLOWU' || SQLCODE|| ' '||SQLERRM);
END PR_ERRORFLOWd;
/

-- PR_ERRORFLOWI.sql

CREATE OR REPLACE Procedure PR_ERRORFLOWI
(
	p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1     		out             db_ref.RCT
	, p_errorflowname       in    		VARCHAR2
	, p_SendingPartnerID	in		INTEGER
	, p_eventid 		in		INTEGER
	, p_ErrorHandlerTask	in		INTEGER
	, p_SendTo		in		INTEGER
	, p_WSRoutingObjId	in		INTEGER
)
AS

/*


	================================================================================
	Proc Name:       PR_ERRORFLOWI
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ERRORFLOWI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;
v_ERRORFLOWID    INTEGER;


BEGIN
	
	
	INSERT INTO PR_ERRORFLOW
		(
		ERRORFLOWID,
		ERRORFLOWNAME,				
		SENDERID,		
		EVENTID,
		ERRORHANDLERTASK,		
		SENDTO,
		WSROUTINGOBJID
		)
		VALUES
		(
		PR_ERRORFLOW_SEQ.nextval,
		p_errorflowname,
		p_SendingPartnerID,
		p_eventid,
		p_ErrorHandlerTask,
		p_SendTo,
		p_WSRoutingObjId	
		)RETURNING ERRORFLOWID INTO v_ERRORFLOWID
		;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	OPEN RC1 FOR
		SELECT v_ERRORFLOWID,
			1 ROWTS
		FROM DUAL 
		;
		
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ERRORFLOWI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ERRORFLOWI' || SQLCODE|| ' '||SQLERRM);
END PR_ERRORFLOWI;
/

-- PR_ERRORFLOWS.sql

CREATE OR REPLACE Procedure PR_ERRORFLOWS
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_ErrorFlowID		in		INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       PR_ERRORFLOWS
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  ERROR FLOW FEATURE
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ERRORFLOWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) ;

v_where           varchar2(500);
v_sqlstmt	  varchar2(2000);


	
BEGIN

IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	IF p_ErrorFlowID IS NULL THEN

		OPEN RC1 FOR 
	                     SELECT 
					a.ERRORFLOWID
					,a.ERRORFLOWNAME
					,a.RESOURCEKEY
					,a.SENDERID
					,b.PARTNERNAME
					,a.EVENTID
					,a.ERRORHANDLERTASK
					,a.SENDTO
					,a.WSROUTINGOBJID
					,a.rowts
					,c.EVENTCODE ERRORDETAIL
					,c.EVENTNAMEKEY
					,d.ROUTINGOBJNAME
				FROM 
			              PR_ERRORFLOW a
				      INNER JOIN PR_PARTNER b ON a.senderid=b.partnerid
				      INNER JOIN lg_eventcd c ON a.eventid=c.eventcd
				      LEFT OUTER JOIN cf_routingobject d ON a.WSROUTINGOBJID=d.ROUTINGOBJID ;

	
	ELSE
		OPEN RC1 FOR 
     				SELECT 
     					a.ERRORFLOWID
     					,a.ERRORFLOWNAME
     					,a.RESOURCEKEY
     					,a.SENDERID
     					,b.PARTNERNAME
     					,a.EVENTID
     					,a.ERRORHANDLERTASK
     					,a.SENDTO
     					,a.WSROUTINGOBJID
     					,a.rowts
     					,c.EVENTCODE ERRORDETAIL
     					,c.EVENTNAMEKEY
     					,d.ROUTINGOBJNAME
     				FROM 
     			              PR_ERRORFLOW a
				      INNER JOIN PR_PARTNER b ON a.senderid=b.partnerid
				      INNER JOIN lg_eventcd c ON a.eventid=c.eventcd 
				      LEFT OUTER JOIN cf_routingobject d ON a.WSROUTINGOBJID=d.ROUTINGOBJID
				      WHERE a.errorflowid = p_ErrorFlowID;
	END IF;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ERRORFLOWS' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ERRORFLOWS' || SQLCODE|| ' '||SQLERRM);
END PR_ERRORFLOWS;
/

-- PR_ERRORFLOWS_ah1.SQL

CREATE OR REPLACE Procedure PR_ERRORFLOWS_ah1
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT 
	, rc2                   out             DB_REF.rct
	, RC3                   OUT      	DB_REF.rct 
	, P_SENDINGPARTNERBUSINESSID IN		VARCHAR2
	, p_eventcode IN INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       PR_ERRORFLOWS_ah1
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  ERROR FLOW Feature
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ERRORFLOWS_ah1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', P_SENDINGPARTNERBUSINESSID=>'||P_SENDINGPARTNERBUSINESSID||CHR(10)
		||', p_eventcode =>'||p_eventcode||CHR(10)
		;
		



BEGIN
	
	 OPEN RC1 FOR 
	 SELECT a.errorflowid,
         a.errorflowname,
         a.resourcekey,
         a.senderid,
         a.eventid,
         a.errorhandlertask,
         a.sendto,
         a.wsroutingobjid,
         b.ROUTINGOBJTYPEID,
         b.ROUTINGOBJLEVEL,
        CASE to_char(b.ROUTINGOBJLEVEL)
            WHEN '1' THEN b.ROUTINGOBJCD
            WHEN '2' THEN b.PARENTOBJCD
            WHEN '3' THEN b.GRPARENTOBJCD
            WHEN '4' THEN b.GRPARENTOBJCD2
            WHEN '5' THEN b.GRPARENTOBJCD3
            ELSE '-1'
        END PACKAGENAME,
        CASE to_char(b.ROUTINGOBJLEVEL)
            WHEN '1' THEN NULL
            WHEN '2' THEN b.ROUTINGOBJCD
            WHEN '3' THEN b.PARENTOBJCD
            WHEN '4' THEN b.GRPARENTOBJCD
            WHEN '5' THEN b.GRPARENTOBJCD2
            ELSE '-1'
        END PROTOCOLNAME,
        CASE to_char(b.ROUTINGOBJLEVEL)
            WHEN '1' THEN NULL
            WHEN '2' THEN NULL
            WHEN '3' THEN b.ROUTINGOBJCD
            WHEN '4' THEN b.PARENTOBJCD
            WHEN '5' THEN b.GRPARENTOBJCD
            ELSE '-1'
        END PROCESSOBJNAME,
        CASE to_char(b.ROUTINGOBJLEVEL)
            WHEN '1' THEN NULL
            WHEN '2' THEN NULL
            WHEN '3' THEN NULL
            WHEN '4' THEN b.ROUTINGOBJCD
            WHEN '5' THEN b.PARENTOBJCD
            ELSE '-1'
        END ACTIVITYOBJNAME,
        CASE to_char(b.ROUTINGOBJLEVEL)
            WHEN '1' THEN NULL
            WHEN '2' THEN NULL
            WHEN '3' THEN NULL
            WHEN '4' THEN NULL
            WHEN '5' THEN b.ROUTINGOBJCD
            ELSE '-1'
        END ACTIONOBJNAME,
            d.TEXTKEY,
            e.RESBUNDLEFILENAME TEXTKEYRESOURCEBUNDENAME
        FROM pr_errorflow a 
        LEFT OUTER JOIN CF_ROUTINGOBJECT06_VW b ON a.wsroutingobjid =  b.routingobjid
        INNER JOIN PR_BUSIDENTIFIER c ON a.senderid = c.partnerid
        INNER JOIN LG_EVENTCD D ON a.eventid=d.eventcd
        INNER JOIN PR_RESOURcEBUNDLES E ON d.TEXTKEYRESOURCEBUNDLEID=e.resourcebundleid
        WHERE 
	C.Businessid = P_SENDINGPARTNERBUSINESSID AND
        a.EventID = p_eventcode;
	
      	OPEN RC2 FOR
      		  SELECT  a.WSATTRID,
		            a.WSATTRKEY,
		            a.WSATTRDESC,
		            a.RESOURCEKEY,
		            b.WSATTRVALUE,
		            b.WSROUTINGOBJID,
		            c.ERRORFLOWID,
		            c.ERRORFLOWNAME,
		            d.ROUTINGOBJCD,
		            d.ROUTINGOBJLEVEL
		            FROM cf_webservice_attr a, CF_WEBSERVICE_ATTRVALUE b,pr_errorflow c ,CF_ROUTINGOBJECT06_VW d
		            WHERE a.WSATTRID = b.WSATTRID and
		            b.WSROUTINGOBJID=c.WSROUTINGOBJID and
		            c.WSROUTINGOBJID=d.ROUTINGOBJID and
		            c.EventID = p_eventcode;
			
	OPEN RC3 FOR
		SELECT businessid,
			partnerid
		FROM PR_BUSIDENTIFIER d
		WHERE d.Businessid= P_SENDINGPARTNERBUSINESSID;
		
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ERRORFLOWS_ah1' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ERRORFLOWS_ah1' || SQLCODE|| ' '||SQLERRM);
END PR_ERRORFLOWS_ah1;
/

-- PR_ERRORFLOWS_AH2.sql

CREATE OR REPLACE Procedure PR_ERRORFLOWS_AH2
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_SenderId            in              INTEGER
	, p_EventId		in		INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       PR_ERRORFLOWS_AH2
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  ERROR FLOW FEATURE
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ERRORFLOWS_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) ;


BEGIN

IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	

		OPEN RC1 FOR
	                     SELECT
					a.ERRORFLOWID
					,a.ERRORFLOWNAME
					,a.RESOURCEKEY
					,a.SENDERID
					,b.PARTNERNAME
					,a.EVENTID
					,a.ERRORHANDLERTASK
					,a.SENDTO
					,a.WSROUTINGOBJID
					,a.rowts
					,c.EVENTCODE || ' - ' || c.EVENTNAME ERRORDETAIL
					,d.ROUTINGOBJNAME
				FROM
			              PR_ERRORFLOW a
				      INNER JOIN PR_PARTNER b ON a.senderid=b.partnerid
				      INNER JOIN lg_eventcd c ON a.eventid=c.eventcd
				      LEFT OUTER JOIN cf_routingobject d ON a.WSROUTINGOBJID=d.ROUTINGOBJID 
				      WHERE a.senderid=p_SenderId 
				      AND a.eventid=p_EventId;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ERRORFLOWS_AH2' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ERRORFLOWS_AH2' || SQLCODE|| ' '||SQLERRM);
END PR_ERRORFLOWS_AH2;
/

-- PR_ERRORFLOWS_AH3.sql

CREATE OR REPLACE Procedure PR_ERRORFLOWS_AH3
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_ErrorFlowName	in		VARCHAR2
	)
AS

/*


	================================================================================
	Proc Name:       PR_ERRORFLOWS_AH3
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  ERROR FLOW FEATURE
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ERRORFLOWS_AH3';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) ;

v_where           varchar2(500);
v_sqlstmt	  varchar2(2000);



BEGIN

IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	
		OPEN RC1 FOR
	                     SELECT
					a.ERRORFLOWID
					
				FROM
			              PR_ERRORFLOW a where 
			              a.errorflowname = p_ErrorFlowName;
				      


	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ERRORFLOWS_AH3' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ERRORFLOWS_AH3' || SQLCODE|| ' '||SQLERRM);
END PR_ERRORFLOWS_AH3;
/

-- PR_ERRORFLOWS_AH5.sql

CREATE OR REPLACE Procedure PR_ERRORFLOWS_AH5
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_SenderID		in		INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       PR_ERRORFLOWS_AH5
	Create Date:     2008-08-29 15:00:44
	Author:          AMIT KUMAR VIJAY

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.2   ERROR FLOW FOR PARTNER MIGRATION
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ERRORFLOWS_AH5';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) ;

v_where           varchar2(500);
v_sqlstmt	  varchar2(2000);


	
BEGIN

IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	IF p_SenderID IS NULL THEN

		OPEN RC1 FOR 
	                     SELECT 
					a.ERRORFLOWID
					,a.ERRORFLOWNAME
					,a.RESOURCEKEY
					,a.SENDERID
					,b.PARTNERNAME
					,a.EVENTID
					,a.ERRORHANDLERTASK
					,a.SENDTO
					,a.WSROUTINGOBJID
					,a.rowts
					,c.EVENTCODE ERRORDETAIL
					,c.EVENTNAMEKEY
					,d.ROUTINGOBJNAME
				FROM 
			              PR_ERRORFLOW a
				      INNER JOIN PR_PARTNER b ON a.senderid=b.partnerid
				      INNER JOIN lg_eventcd c ON a.eventid=c.eventcd
				      LEFT OUTER JOIN cf_routingobject d ON a.WSROUTINGOBJID=d.ROUTINGOBJID ;

	
	ELSE
		OPEN RC1 FOR 
     				SELECT 
     					a.ERRORFLOWID
     					,a.ERRORFLOWNAME
     					,a.RESOURCEKEY
     					,a.SENDERID
     					,b.PARTNERNAME
     					,a.EVENTID
     					,a.ERRORHANDLERTASK
     					,a.SENDTO
     					,a.WSROUTINGOBJID
     					,a.rowts
     					,c.EVENTCODE ERRORDETAIL
     					,c.EVENTNAMEKEY
     					,d.ROUTINGOBJNAME
     				FROM 
     			              PR_ERRORFLOW a
				      INNER JOIN PR_PARTNER b ON a.senderid=b.partnerid
				      INNER JOIN lg_eventcd c ON a.eventid=c.eventcd 
				      LEFT OUTER JOIN cf_routingobject d ON a.WSROUTINGOBJID=d.ROUTINGOBJID
				      WHERE a.SENDERID = p_SenderID;
	END IF;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ERRORFLOWS_AH5' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ERRORFLOWS_AH5' || SQLCODE|| ' '||SQLERRM);
END PR_ERRORFLOWS_AH5;
/

-- PR_ERRORFLOWU.sql

CREATE OR REPLACE Procedure PR_ERRORFLOWU
(
	 p_ReturnCd		in out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_errorflowname       in              VARCHAR
	, p_SendingPartnerID	in		INTEGER
	, p_EventID		in		INTEGER
	, p_ErrorHandlerTask	in		INTEGER
	, p_SendTo		in		INTEGER
	, p_WSRoutingObjId	in		INTEGER
	, p_ErrorFlowId		in		INTEGER
	, p_RowTS		in		INTEGER
)
AS


/*


	================================================================================
	Proc Name:       PR_ERRORFLOWU
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ERRORFLOWU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000);
		
v_RowTS INTEGER;

v_wsattrvalueid    INTEGER;
ZeroUpdate EXCEPTION;


BEGIN
	
	
	v_RowCnt := 0;
		Update PR_ERRORFLOW
		SET
			errorflowname = p_errorflowname,
			senderid = p_SendingPartnerID,
			eventid = p_EventID,
			errorhandlertask = p_ErrorHandlerTask,
			sendto = p_SendTo,
			wsroutingobjid = p_WSRoutingObjId,			
			ROWTS= p_RowTS + 1,
			UPDATEDATE= v_BeginTS
		Where
			errorflowid = p_ErrorFlowId
			And RowTS = p_RowTS returning RowTS into v_RowTS;
			
		v_RowCnt := SQL%rowcount;
		
		if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
		
		IF RC1%isOpen THEN Close RC1; END IF;
		OPEN RC1 for select v_RowTS RowTS from Dual;
		p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ERRORFLOWU' || SQLCODE|| ' '||SQLERRM);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ERRORFLOWU: No Data Found');
					
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ERRORFLOWU' || SQLCODE|| ' '||SQLERRM);
END PR_ERRORFLOWU;
/

-- PR_FIXEDWORKFLOWD.sql

CREATE OR REPLACE Procedure PR_FIXEDWORKFLOWD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_FIXEDWORKFLOWID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWD
	Create Date:     2004-03-16 16:11:04
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWORKFLOWID=>'||p_FIXEDWORKFLOWID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_FIXEDWORKFLOW
	Where 
		FIXEDWORKFLOWID = p_FIXEDWORKFLOWID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_FIXEDWORKFLOWD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_FIXEDWORKFLOWD;
/


-- PR_FIXEDWORKFLOWHANDLER_CATD.sql

CREATE OR REPLACE Procedure PR_FIXEDWORKFLOWHANDLER_CATD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_FIXEDWOKFLOWSTEPID in VARCHAR2
	,p_HANDLERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWHANDLER_CATD
	Create Date:     2004-03-12 17:45:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWHANDLER_CATD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_where := ' WHERE 1=1 ' ;
    v_where := v_where || FN_BUILD_WHERE('FIXEDWOKFLOWSTEPID', 'VARCHAR2', p_FIXEDWOKFLOWSTEPID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	
    v_sqlstmt := 
	   'Delete From PR_FIXEDWORKFLOWHANDLER_CAT ' || v_where  ;
	
	execute immediate v_sqlstmt ;
	
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_FIXEDWORKFLOWHANDLER_CATD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_FIXEDWORKFLOWHANDLER_CATD;
/


-- PR_FIXEDWORKFLOWHANDLER_CATI.sql

CREATE OR REPLACE Procedure PR_FIXEDWORKFLOWHANDLER_CATI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT    
	,p_FIXEDWOKFLOWSTEPID in VARCHAR2
	,p_HANDLERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWHANDLER_CATI
	Create Date:     2004-03-12 18:02:37
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWHANDLER_CATI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_FIXEDWORKFLOWHANDLER_CAT(
		FIXEDWOKFLOWSTEPID
		,HANDLERID
	) Values (
		p_FIXEDWOKFLOWSTEPID
		,p_HANDLERID
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_FIXEDWORKFLOWHANDLER_CATI' || SQLCODE|| ' '||SQLERRM);
END PR_FIXEDWORKFLOWHANDLER_CATI;
/


-- PR_FIXEDWORKFLOWHANDLER_CATS.sql

CREATE OR REPLACE PROCEDURE PR_FIXEDWORKFLOWHANDLER_CATS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FIXEDWOKFLOWSTEPID IN VARCHAR2
	, p_HANDLERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWHANDLER_CATS
	Create Date:     2004-03-12 18:08:25
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWHANDLER_CATS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;                                
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('FIXEDWOKFLOWSTEPID', 'VARCHAR2', p_FIXEDWOKFLOWSTEPID) ||
       FN_Build_Where('c.HANDLERID', 'NUMBER', p_HANDLERID);
	
	v_sqlstmt := 
		'SELECT '
		||'RTRIM(c.FIXEDWOKFLOWSTEPID) FIXEDWOKFLOWSTEPID'
		||',c.HANDLERID'   
		||',h.PROVIDERFLG'                                                                                                                         
		||',RTRIM(h.RESOURCEKEY) RESOURCEKEY'                                               
		||',RTRIM(h.DESCRIPTION) DESCRIPTION'                                               
		||',RTRIM(h.CLASSNAME) CLASSNAME'    
		||',c.CREATEDATE'
		||',c.ROWTS '     
    	||'FROM ' 
		||'PR_FIXEDWORKFLOWHANDLER_CAT c, PR_HANDLER h'
		|| v_where
		|| ' AND c.HANDLERID = h.HANDLERID'
	;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_FIXEDWORKFLOWHANDLER_CATS;
/


-- PR_FIXEDWORKFLOWI.sql

CREATE OR REPLACE Procedure PR_FIXEDWORKFLOWI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROVIDERFLAG in NUMBER
	,p_WORKFLOWNAME in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWI
	Create Date:     2004-03-16 16:12:36
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_WORKFLOWNAME=>'||p_WORKFLOWNAME||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_FIXEDWORKFLOW(
		FIXEDWORKFLOWID
		,PROVIDERFLAG
		,WORKFLOWNAME
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		PR_FIXEDWORKFLOW_SEQ.nextval
		,p_PROVIDERFLAG
		,p_WORKFLOWNAME
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		PR_FIXEDWORKFLOW_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_FIXEDWORKFLOWI' || SQLCODE|| ' '||SQLERRM);
END PR_FIXEDWORKFLOWI;
/


-- PR_FIXEDWORKFLOWS.sql

CREATE OR REPLACE PROCEDURE PR_FIXEDWORKFLOWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FIXEDWORKFLOWID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWS
	Create Date:     2004-03-16 16:11:56
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWORKFLOWID=>'||p_FIXEDWORKFLOWID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_FIXEDWORKFLOWID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		FIXEDWORKFLOWID
		,PROVIDERFLAG
		,ROWTS
		,CREATEDATE
		,RTRIM(WORKFLOWNAME) WORKFLOWNAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
	FROM 
		PR_FIXEDWORKFLOW
	Where 
		FIXEDWORKFLOWID = p_FIXEDWORKFLOWID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		FIXEDWORKFLOWID
		,PROVIDERFLAG
		,ROWTS
		,CREATEDATE
		,RTRIM(WORKFLOWNAME) WORKFLOWNAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
	FROM 
		PR_FIXEDWORKFLOW
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_FIXEDWORKFLOWS;
/


-- PR_FIXEDWORKFLOWSTEPD.sql

CREATE OR REPLACE Procedure PR_FIXEDWORKFLOWSTEPD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_FIXEDWOKFLOWSTEPID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWSTEPD
	Create Date:     2004-03-16 16:41:36
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWSTEPD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_FIXEDWORKFLOWSTEP
	Where 
		FIXEDWOKFLOWSTEPID = p_FIXEDWOKFLOWSTEPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_FIXEDWORKFLOWSTEPD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_FIXEDWORKFLOWSTEPD;
/


-- PR_FIXEDWORKFLOWSTEPI.sql

CREATE OR REPLACE Procedure PR_FIXEDWORKFLOWSTEPI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT 
	,p_FIXEDWOKFLOWSTEPID in VARCHAR2
	,p_CONFIGURABLE in NUMBER
	,p_PROVIDERFLAG in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_CLASSNAME in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWSTEPI
	Create Date:     2004-03-16 16:42:29
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWSTEPI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_CONFIGURABLE=>'||p_CONFIGURABLE||CHR(10)
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		||', p_CLASSNAME=>'||p_CLASSNAME||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_FIXEDWORKFLOWSTEP(
		FIXEDWOKFLOWSTEPID
		,CONFIGURABLE
		,PROVIDERFLAG
		,RESOURCEKEY
		,DESCRIPTION
		,CLASSNAME
	) Values (
		p_FIXEDWOKFLOWSTEPID
		,p_CONFIGURABLE
		,p_PROVIDERFLAG
		,p_RESOURCEKEY
		,p_DESCRIPTION
		,p_CLASSNAME
	);
	p_ReturnCd := 0;
    IF RC1%isOpen THEN Close RC1; END IF;                                          
	Open RC1 FOR Select p_FIXEDWOKFLOWSTEPID, 1 RowTS From Dual; 
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_FIXEDWORKFLOWSTEPI' || SQLCODE|| ' '||SQLERRM);
END PR_FIXEDWORKFLOWSTEPI;
/


-- PR_FIXEDWORKFLOWSTEPS.sql

CREATE OR REPLACE PROCEDURE PR_FIXEDWORKFLOWSTEPS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FIXEDWOKFLOWSTEPID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWSTEPS
	Create Date:     2004-03-16 16:42:04
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWSTEPS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_FIXEDWOKFLOWSTEPID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		RTRIM(FIXEDWOKFLOWSTEPID) FIXEDWOKFLOWSTEPID
		,CONFIGURABLE
		,PROVIDERFLAG
		,ROWTS
		,CREATEDATE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(CLASSNAME) CLASSNAME
	FROM 
		PR_FIXEDWORKFLOWSTEP
	Where 
		FIXEDWOKFLOWSTEPID = p_FIXEDWOKFLOWSTEPID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		RTRIM(FIXEDWOKFLOWSTEPID) FIXEDWOKFLOWSTEPID
		,CONFIGURABLE
		,PROVIDERFLAG
		,ROWTS
		,CREATEDATE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(CLASSNAME) CLASSNAME
	FROM 
		PR_FIXEDWORKFLOWSTEP
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_FIXEDWORKFLOWSTEPS;
/


-- PR_FIXEDWORKFLOWSTEPU.sql

CREATE OR REPLACE Procedure PR_FIXEDWORKFLOWSTEPU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_FIXEDWOKFLOWSTEPID in VARCHAR2
	,p_CONFIGURABLE in NUMBER
	,p_PROVIDERFLAG in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_CLASSNAME in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWSTEPU
	Create Date:     2004-03-16 16:41:50
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWSTEPU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWOKFLOWSTEPID=>'||p_FIXEDWOKFLOWSTEPID||CHR(10)
		||', p_CONFIGURABLE=>'||p_CONFIGURABLE||CHR(10)
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		||', p_CLASSNAME=>'||p_CLASSNAME||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_FIXEDWORKFLOWSTEP
	SET
		CONFIGURABLE=p_CONFIGURABLE
		,      PROVIDERFLAG=p_PROVIDERFLAG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
		,      CLASSNAME=p_CLASSNAME
	Where 
		FIXEDWOKFLOWSTEPID = p_FIXEDWOKFLOWSTEPID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_FIXEDWORKFLOWSTEPU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_FIXEDWORKFLOWSTEPU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_FIXEDWORKFLOWSTEPU:'||SQLERRM);
End PR_FIXEDWORKFLOWSTEPU;
/


-- PR_FIXEDWORKFLOWU.sql

CREATE OR REPLACE Procedure PR_FIXEDWORKFLOWU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_FIXEDWORKFLOWID in NUMBER
	,p_PROVIDERFLAG in NUMBER
	,p_WORKFLOWNAME in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_FIXEDWORKFLOWU
	Create Date:     2004-03-16 16:11:35
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_FIXEDWORKFLOWU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FIXEDWORKFLOWID=>'||p_FIXEDWORKFLOWID||CHR(10)
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_WORKFLOWNAME=>'||p_WORKFLOWNAME||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_FIXEDWORKFLOW
	SET
		PROVIDERFLAG=p_PROVIDERFLAG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      WORKFLOWNAME=p_WORKFLOWNAME
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
	Where 
		FIXEDWORKFLOWID = p_FIXEDWORKFLOWID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_FIXEDWORKFLOWU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_FIXEDWORKFLOWU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_FIXEDWORKFLOWU:'||SQLERRM);
End PR_FIXEDWORKFLOWU;
/


-- PR_GATEWAYD.sql

CREATE OR REPLACE Procedure PR_GATEWAYD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_Gatewayid in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GATEWAYD
	Create Date:     2007-08-20 16:40:12
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-08-20 BHARANI  J         1  Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GATEWAY_D';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_Gatewayid=>'||p_Gatewayid||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_GATEWAY
	Where gatewayid = p_Gatewayid
	;
	
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	
	p_ReturnCd := 0;
	
	
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			p_ReturnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_GATEWAYD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_GATEWAYD;
/

-- PR_GATEWAYDEFAULTD.SQL

CREATE OR REPLACE Procedure PR_GATEWAYDEFAULTD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_GATEWAYDEFAULTID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GATEWAYDEFAULTD
	Create Date:     2003-09-26 08:24:17
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GATEWAYDEFAULTD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYDEFAULTID=>'||p_GATEWAYDEFAULTID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_GATEWAYDEFAULT
	Where
		GATEWAYDEFAULTID = p_GATEWAYDEFAULTID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_GATEWAYDEFAULTD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_GATEWAYDEFAULTD;
/


-- PR_GATEWAYDEFAULTI.SQL

CREATE OR REPLACE Procedure PR_GATEWAYDEFAULTI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_GATEWAYID in NUMBER
	,p_DESTTYPEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GATEWAYDEFAULTI
	Create Date:     2003-09-26 00:01:43
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GATEWAYDEFAULTI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_GATEWAYDEFAULT(
		GATEWAYDEFAULTID
		,PARTNERID
		,GATEWAYID
		,DESTTYPEID
	) Values (
		PR_GATEWAYDEFAULT_SEQ.nextval
		,p_PARTNERID
		,p_GATEWAYID
		,p_DESTTYPEID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_GATEWAYDEFAULT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_GATEWAYDEFAULTI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_GATEWAYDEFAULTI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_GATEWAYDEFAULTI' || SQLCODE|| ' '||SQLERRM);
END PR_GATEWAYDEFAULTI;
/


-- PR_GATEWAYDEFAULTS.SQL

CREATE OR REPLACE PROCEDURE PR_GATEWAYDEFAULTS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_GATEWAYDEFAULTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GATEWAYDEFAULTS
	Create Date:     2003-09-26 00:06:37
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GATEWAYDEFAULTS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYDEFAULTID=>'||p_GATEWAYDEFAULTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_GATEWAYDEFAULTID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		GATEWAYDEFAULTID
		,PARTNERID
		,GATEWAYID
		,DESTTYPEID
		,ROWTS
		,CREATEDATE
	FROM
		PR_GATEWAYDEFAULT
	Where
		GATEWAYDEFAULTID = p_GATEWAYDEFAULTID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		GATEWAYDEFAULTID
		,PARTNERID
		,GATEWAYID
		,DESTTYPEID
		,ROWTS
		,CREATEDATE
	FROM
		PR_GATEWAYDEFAULT
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_GATEWAYDEFAULTS;
/


-- PR_GATEWAYDEFAULTS_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_GATEWAYDEFAULTS_AH1
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_PARTNERID IN NUMBER
    , p_GatewayId In Number
    , p_DestTypeId in number
    )
AS
/*
    ================================================================================
    Proc Name:       PR_GATEWAYDEFAULTS_AH1
    Create Date:     2003-05-06 23:21:01
    Author:          SYSTEM
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_GATEWAYDEFAULTS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
         ||', p_GatewayID=>'||p_PARTNERID||CHR(10)
         ||', p_DestTypeID=>'||p_PARTNERID||CHR(10)
        ;
v_sqlstmt          varchar2(4000);
v_where            varchar2(1000);
---------------
-- Where_Clause
---------------
PROCEDURE Where_Clause AS
BEGIN
    v_where := ' Where 1=1 '||
    FN_BUILD_WHERE('PartnerId', 'NUMBER', p_PartnerId)||
    FN_BUILD_WHERE('GatewayId', 'NUMBER', p_GatewayId)||
    FN_BUILD_WHERE('DestTypeId', 'NUMBER', p_DestTypeId);

END Where_Clause;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    Where_Clause;

    v_sqlstmt := ' SELECT
                      GatewayDefaultId
                      ,PARTNERID
                      ,GATEWAYID
                      ,DESTTYPEID
                      ,ROWTS
                  FROM
                      PR_GATEWAYDEFAULT' ||
                   v_where;


    IF RC1%ISOPEN THEN CLOSE RC1; END IF;

    OPEN RC1 FOR v_sqlstmt;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION
         When OTHERS Then
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_GatewayDefaultS_AH1' || SQLCODE|| ' '||SQLERRM);
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_GATEWAYDEFAULTS_AH1;
/


-- PR_GATEWAYDEFAULTU.SQL

CREATE OR REPLACE Procedure PR_GATEWAYDEFAULTU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_GATEWAYDEFAULTID in NUMBER
	,p_PARTNERID in NUMBER
	,p_GATEWAYID in NUMBER
	,p_DESTTYPEID in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_GATEWAYDEFAULTU
	Create Date:     2003-09-26 08:21:21
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GATEWAYDEFAULTU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYDEFAULTID=>'||p_GATEWAYDEFAULTID||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_GATEWAYDEFAULT
	SET
		PARTNERID=p_PARTNERID
		,      GATEWAYID=p_GATEWAYID
		,      DESTTYPEID=p_DESTTYPEID
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		GATEWAYDEFAULTID = p_GATEWAYDEFAULTID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_GATEWAYDEFAULTU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GATEWAYDEFAULTU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GATEWAYDEFAULTU:'||SQLERRM);
End PR_GATEWAYDEFAULTU;
/


-- PR_GATEWAYD_AH1.sql

CREATE OR REPLACE PROCEDURE PR_GATEWAYD_AH1 
(
   p_returncd    OUT      PLS_INTEGER,
   p_debugflg    IN       PLS_INTEGER,
   p_gatewayid   IN       NUMBER,
   p_partnerid   IN       NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       PR_GATEWAYD_AH1
    Create Date:     2007-08-20 16:40:12
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-09 BHARANI J          1  Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   
   v_begints        TIMESTAMP       DEFAULT fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   DEFAULT 'PR_GATEWAYD_AH1';
   v_exceptionflg   INTEGER         DEFAULT 0;
   v_paramstxt      VARCHAR2 (4000) DEFAULT  'p_DebugFlg=>' || p_debugflg || CHR (10)  ||
                                             ', p_gatewayid=>' || p_gatewayid || CHR (10) ||
                                             ', p_partnerid=> ||p_partnerid||CHR(10)'
                                             ;
  
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

   DELETE FROM cf_gwyhandlerattr_lnk
         WHERE gatewayid = p_gatewayid;
         
           

   DELETE FROM cf_gwyhandler_xref
         WHERE gatewayid = p_gatewayid;
         
   
   cf_proxy_gateway_lnkd_ah1 (p_returncd       => p_returncd,
                              p_debugflg       => p_debugflg,
                              p_gatewayid      => p_gatewayid
                             );
                             
                                 

   DELETE FROM pr_gwyattr
         WHERE gatewayid = p_gatewayid;
         
           

   pr_channel_gateway_lnkd_ah1 (p_returncd       => p_returncd,
                                p_debugflg       => p_debugflg,
                                p_gatewayid      => p_gatewayid,
                                p_partnerid      => p_partnerid
                               );
                               
                                 
                               
   pr_gatewayd (p_returncd       => p_returncd,
                p_debugflg       => p_debugflg,
                p_gatewayid      => p_gatewayid
               );
               
   
   
   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
   p_returncd := 0;
   
EXCEPTION
  
   WHEN OTHERS
   THEN
      p_returncd := 1;
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            SQLERRM || CHR (10) || v_paramstxt,
                            p_debugflg,
                            1
                           );
      raise_application_error (-20999, '20000 ' || SQLCODE || ' ' || SQLERRM);
      
END pr_gatewayd_ah1;
/

-- PR_GATEWAYI.sql

CREATE OR REPLACE Procedure PR_GATEWAYI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_GATEWAYNAME in VARCHAR2
	,p_PARTNERID in NUMBER
	,p_TRANSPORTID in NUMBER
	,p_THREADNBR in NUMBER
	,p_URI in VARCHAR2
	,p_URILOGIN in VARCHAR2
	,p_URIPASSWORD in RAW
	,p_TRANSPORTRETRY in NUMBER
	,p_TRANSPORTINTERVAL in NUMBER
	,p_ENABLEDFLG in NUMBER
	,p_OFFLINEFLG in NUMBER
	,p_GATEWAYDESC in VARCHAR2
	,p_VALIDATECLIENTSSLCERT in NUMBER
	,p_VALIDATECLIENTIP in NUMBER
	,p_AUTOCONFIGFLG in NUMBER
	,p_AUTHENTICATIONRQDFLG in NUMBER
	,p_CONNECTIONTIMEOUT in NUMBER
	,p_JMSQUEUENAME in VARCHAR2
	,p_JMSFACTORYNAME in VARCHAR2
	,p_JNDIFACTORYNAME in VARCHAR2
	,p_PROVIDERURLPKG in VARCHAR2
	,p_JMSMESSAGECLASS in VARCHAR2
	,p_JMSMESSAGETYPE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_GATEWAYI
	Create Date:     2003-09-26 00:01:43
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GATEWAYI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYNAME=>'||p_GATEWAYNAME||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_THREADNBR=>'||p_THREADNBR||CHR(10)
		||', p_URI=>'||p_URI||CHR(10)
		||', p_URILOGIN=>'||p_URILOGIN||CHR(10)
		||', p_URIPASSWORD=>'||p_URIPASSWORD||CHR(10)
		||', p_TRANSPORTRETRY=>'||p_TRANSPORTRETRY||CHR(10)
		||', p_TRANSPORTINTERVAL=>'||p_TRANSPORTINTERVAL||CHR(10)
		||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
		||', p_OFFLINEFLG=>'||p_OFFLINEFLG||CHR(10)
		||', p_GATEWAYDESC=>'||p_GATEWAYDESC||CHR(10)
		||', p_VALIDATECLIENTSSLCERT=>'||p_VALIDATECLIENTSSLCERT||CHR(10)
		||', p_VALIDATECLIENTIP=>'||p_VALIDATECLIENTIP||CHR(10)
		||', p_AUTOCONFIGFLG=>'||p_AUTOCONFIGFLG||CHR(10)
		||', p_AUTHENTICATIONRQDFLG=>'||p_AUTHENTICATIONRQDFLG||CHR(10)
		||', p_CONNECTIONTIMEOUT=>'||p_CONNECTIONTIMEOUT||CHR(10)
		||', p_JMSQUEUENAME=>'||p_JMSQUEUENAME||CHR(10)
		||', p_JMSFACTORYNAME=>'||p_JMSFACTORYNAME||CHR(10)
		||', p_JNDIFACTORYNAME=>'||p_JNDIFACTORYNAME||CHR(10)
		||', p_PROVIDERURLPKG=>'||p_PROVIDERURLPKG||CHR(10)
		||', p_JMSMESSAGECLASS=>'||p_JMSMESSAGECLASS||CHR(10)
		||', p_JMSMESSAGETYPE=>'||p_JMSMESSAGETYPE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_GATEWAY(
		GATEWAYID
		,GATEWAYNAME
		,PARTNERID
		,TRANSPORTID
		,THREADNBR
		,URI
		,URILOGIN
		,URIPASSWORD
		,TRANSPORTRETRY
		,TRANSPORTINTERVAL
		,ENABLEDFLG
		,OFFLINEFLG
		,GATEWAYDESC
		,VALIDATECLIENTSSLCERT
		,VALIDATECLIENTIP
		,AUTOCONFIGFLG
		,AUTHENTICATIONRQDFLG
		,CONNECTIONTIMEOUT
		,JMSQUEUENAME
		,JMSFACTORYNAME
		,JNDIFACTORYNAME
		,PROVIDERURLPKG
		,JMSMESSAGECLASS
		,JMSMESSAGETYPE
	) Values (
		PR_GATEWAY_SEQ.nextval
		,p_GATEWAYNAME
		,p_PARTNERID
		,p_TRANSPORTID
		,p_THREADNBR
		,p_URI
		,p_URILOGIN
		,p_URIPASSWORD
		,p_TRANSPORTRETRY
		,p_TRANSPORTINTERVAL
		,p_ENABLEDFLG
		,p_OFFLINEFLG
		,p_GATEWAYDESC
		,p_VALIDATECLIENTSSLCERT
		,p_VALIDATECLIENTIP
		,p_AUTOCONFIGFLG
		,p_AUTHENTICATIONRQDFLG
		,p_CONNECTIONTIMEOUT
		,p_JMSQUEUENAME
		,p_JMSFACTORYNAME
		,p_JNDIFACTORYNAME
		,p_PROVIDERURLPKG
		,p_JMSMESSAGECLASS
		,p_JMSMESSAGETYPE
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_GATEWAY_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_GATEWAYI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_GATEWAYI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_GATEWAYI' || SQLCODE|| ' '||SQLERRM);
END PR_GATEWAYI;
/


-- PR_GATEWAYS.sql

CREATE OR REPLACE PROCEDURE PR_GATEWAYS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_GATEWAYID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GATEWAYS
	Create Date:     2003-09-26 00:06:35
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GATEWAYS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_GATEWAYID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		GATEWAYID
		,RTRIM(GATEWAYNAME) GATEWAYNAME
		,PARTNERID
		,TRANSPORTID
		,THREADNBR
		,RTRIM(URI) URI
		,RTRIM(URILOGIN) URILOGIN
		,URIPASSWORD
		,TRANSPORTRETRY
		,TRANSPORTINTERVAL
		,ENABLEDFLG
		,OFFLINEFLG
		,RTRIM(GATEWAYDESC) GATEWAYDESC
		,ROWTS
		,CREATEDATE
		,VALIDATECLIENTSSLCERT
		,VALIDATECLIENTIP
		,AUTOCONFIGFLG
		,AUTHENTICATIONRQDFLG
		,CONNECTIONTIMEOUT
		,RTRIM(JMSQUEUENAME) JMSQUEUENAME
		,RTRIM(JMSFACTORYNAME) JMSFACTORYNAME
		,RTRIM(JNDIFACTORYNAME) JNDIFACTORYNAME
		,RTRIM(PROVIDERURLPKG) PROVIDERURLPKG
		,RTRIM(JMSMESSAGECLASS) JMSMESSAGECLASS
		,RTRIM(JMSMESSAGETYPE) JMSMESSAGETYPE
	FROM
		PR_GATEWAY
	Where
		GATEWAYID = p_GATEWAYID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		GATEWAYID
		,RTRIM(GATEWAYNAME) GATEWAYNAME
		,PARTNERID
		,TRANSPORTID
		,THREADNBR
		,RTRIM(URI) URI
		,RTRIM(URILOGIN) URILOGIN
		,URIPASSWORD
		,TRANSPORTRETRY
		,TRANSPORTINTERVAL
		,ENABLEDFLG
		,OFFLINEFLG
		,RTRIM(GATEWAYDESC) GATEWAYDESC
		,ROWTS
		,CREATEDATE
		,VALIDATECLIENTSSLCERT
		,VALIDATECLIENTIP
		,AUTOCONFIGFLG
		,AUTHENTICATIONRQDFLG
		,CONNECTIONTIMEOUT
		,RTRIM(JMSQUEUENAME) JMSQUEUENAME
		,RTRIM(JMSFACTORYNAME) JMSFACTORYNAME
		,RTRIM(JNDIFACTORYNAME) JNDIFACTORYNAME
		,RTRIM(PROVIDERURLPKG) PROVIDERURLPKG
		,RTRIM(JMSMESSAGECLASS) JMSMESSAGECLASS
		,RTRIM(JMSMESSAGETYPE) JMSMESSAGETYPE
	FROM
		PR_GATEWAY
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_GATEWAYS;
/


-- PR_GATEWAYS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_GATEWAYS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_TRANSPORTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GATEWAYS_AH1
	Create Date:     2006-09-28 00:06:35
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2006-09-28 SYSTEM        6.1.0 	Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GATEWAYS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_TRANSPORTID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		GATEWAYID
		,RTRIM(GATEWAYNAME) GATEWAYNAME
		,PARTNERID
		,TRANSPORTID
		,THREADNBR
		,RTRIM(URI) URI
		,RTRIM(URILOGIN) URILOGIN
		,URIPASSWORD
		,TRANSPORTRETRY
		,TRANSPORTINTERVAL
		,ENABLEDFLG
		,OFFLINEFLG
		,RTRIM(GATEWAYDESC) GATEWAYDESC
		,ROWTS
		,CREATEDATE
		,VALIDATECLIENTSSLCERT
		,VALIDATECLIENTIP
		,AUTOCONFIGFLG
		,AUTHENTICATIONRQDFLG
		,CONNECTIONTIMEOUT
		,RTRIM(JMSQUEUENAME) JMSQUEUENAME
		,RTRIM(JMSFACTORYNAME) JMSFACTORYNAME
		,RTRIM(JNDIFACTORYNAME) JNDIFACTORYNAME
		,RTRIM(PROVIDERURLPKG) PROVIDERURLPKG
		,RTRIM(JMSMESSAGECLASS) JMSMESSAGECLASS
		,RTRIM(JMSMESSAGETYPE) JMSMESSAGETYPE
	FROM
		PR_GATEWAY
	Where
		TRANSPORTID = p_TRANSPORTID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		GATEWAYID
		,RTRIM(GATEWAYNAME) GATEWAYNAME
		,PARTNERID
		,TRANSPORTID
		,THREADNBR
		,RTRIM(URI) URI
		,RTRIM(URILOGIN) URILOGIN
		,URIPASSWORD
		,TRANSPORTRETRY
		,TRANSPORTINTERVAL
		,ENABLEDFLG
		,OFFLINEFLG
		,RTRIM(GATEWAYDESC) GATEWAYDESC
		,ROWTS
		,CREATEDATE
		,VALIDATECLIENTSSLCERT
		,VALIDATECLIENTIP
		,AUTOCONFIGFLG
		,AUTHENTICATIONRQDFLG
		,CONNECTIONTIMEOUT
		,RTRIM(JMSQUEUENAME) JMSQUEUENAME
		,RTRIM(JMSFACTORYNAME) JMSFACTORYNAME
		,RTRIM(JNDIFACTORYNAME) JNDIFACTORYNAME
		,RTRIM(PROVIDERURLPKG) PROVIDERURLPKG
		,RTRIM(JMSMESSAGECLASS) JMSMESSAGECLASS
		,RTRIM(JMSMESSAGETYPE) JMSMESSAGETYPE
	FROM
		PR_GATEWAY
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_GATEWAYS_AH1;
/


-- PR_GATEWAYs_change.SQL

CREATE OR REPLACE PROCEDURE         PR_GATEWAYs_change
(
    p_returncd out integer,
    p_debugFlg in pls_integer,
    RC1 out db_ref.RCT,
    p_Locale in varchar2,
    p_TS in Timestamp
) As
/*
================================================================================
Proc Name:      PR_GATEWAYs_change
Create Date:    04/12/02 02:34:37 PM
Author          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
     2004-03-05 S.Priani      4.2.2  Removed NLS
     2004-04-15 P Dominick    4.2.2  Added code to accept gatewayid and return info based on id
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

v_sqlstmt         varchar2(2000) ;
v_where           varchar2(2000) ;
v_TSString        VARCHAR2(1000);

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_ah1('PR_GATEWAYs_change',v_beginTS, 'p_returncd'||','||p_debugFlg
    ||','||p_Locale
    ||','||'RC1'
    ||','||p_TS
    );
End ProcAudit;
--
-- Get Partner Name
--
BEGIN

v_TSString := TO_CHAR(p_TS, 'DD-MON-YY HH.MI.SS PM');
v_where := ' Where 1=1 AND g.updatedate > '''|| v_TSString ||''''; 
--       FN_Build_Where('GATEWAYID', 'NUMBER', p_GATEWAYID);

--v_where := ' Where g.updatedate > p_TS '||
--       FN_Build_Where('GATEWAYID', 'NUMBER', p_gatewayid);

V_sqlstmt := 'SELECT GATEWAYID '||
',   RTRIM(GATEWAYNAME)  GATEWAYNAME'||
',   g.PARTNERID'||
',   RTRIM(p.PartnerName) PARTNERNAME'||
',   g.TRANSPORTID'||
',   RTRIM(w.TRANSPORTTYPE) TRANSPORTTYPE'||
',   THREADNBR'||
',   RTRIM(g.URI)  URI'||
',   RTRIM(URILOGIN)  URILOGIN'||
',   URIPASSWORD'||
',   TRANSPORTRETRY'||
',   TRANSPORTINTERVAL'||
',   g.ENABLEDFLG'||
',   OFFLINEFLG'||
',   RTRIM(GATEWAYDESC)  GATEWAYDESC'||
',   g.UpdateDATE'||
',   AutoConfigFlg'||
',   g.AuthenticationRqdFlg'||
',   g.ConnectionTimeOut'||
',   RTRIM(g.JMSQueueName) JMSQueueName'||
',   RTRIM(g.JMSFactoryName) JMSFactoryName'||
',   RTRIM(g.JNDIFactoryName) JNDIFactoryName'||
',   RTRIM(g.ProviderURLPkg) ProviderURLPkg'||
',   RTRIM(g.JMSMessageClass) JMSMessageClass'||
',   RTRIM(g.JMSMessageType) JMSMessageType'||
' From PR_GATEWAY g'||
' left outer join pr_partner p    on g.PARTNERID   = p.PARTNERID '||
' left outer join PR_TRANSPORT  w on g.TRANSPORTID = w.TRANSPORTID '||
v_where;

IF RC1%ISOPEN THEN CLOSE RC1; END IF;
OPEN RC1 FOR v_sqlstmt ;


p_returncd := 0;
procAudit;
EXCEPTION
        WHEN NO_DATA_FOUND THEN NULL;
        WHEN OTHERS then
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GATEWAYs_change:'||SQLERRM);
End PR_GATEWAYs_change;
/


-- PR_GATEWAYs_change_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_GATEWAYs_change_AH1
(
    p_returncd out integer,
	p_debugFlg IN integer,
    RC1 out db_ref.RCT,
    p_gatewayid IN INTEGER
) As
/*
================================================================================
Proc Name:      PR_GATEWAYs_change_AH1
Create Date:    04/12/02 02:34:37 PM
Author          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
	2004-04-14 Paul D.	4.2.2	Created
	2006-10-31 S.Priani	6.0	JR24838
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

v_sqlstmt         varchar2(2000) ;
v_where           varchar2(2000) ;
v_TSString        VARCHAR2(1000);

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_ah1('PR_GATEWAYs_change_AH1',v_beginTS, 'p_returncd'||','||p_debugFlg
    ||','||'RC1'
    );
End ProcAudit;
BEGIN

v_where := ' Where 1=1 '|| FN_Build_Where('GATEWAYID', 'NUMBER', p_GATEWAYID);

V_sqlstmt := 'SELECT GATEWAYID '||
',   RTRIM(GATEWAYNAME)  GATEWAYNAME'||
',   g.PARTNERID'||
',   RTRIM(p.PartnerName) PARTNERNAME'||
',   g.TRANSPORTID'||
',   RTRIM(w.TRANSPORTTYPE) TRANSPORTTYPE'||
',   THREADNBR'||
',   RTRIM(g.URI)  URI'||
	',   RTRIM(g.URILOGIN)  URILOGIN'||
	',   g.URIPASSWORD'||
	',   g.TRANSPORTRETRY'||
	',   g.TRANSPORTINTERVAL'||
',   g.ENABLEDFLG'||
',   OFFLINEFLG'||
',   RTRIM(GATEWAYDESC)  GATEWAYDESC'||
',   g.UpdateDATE'||
	',   g.VALIDATECLIENTSSLCERT '||
	',   g.VALIDATECLIENTIP '||
	',   g.AutoConfigFlg'||
',   g.AuthenticationRqdFlg'||
',   g.ConnectionTimeOut'||
',   RTRIM(g.JMSQueueName) JMSQueueName'||
',   RTRIM(g.JMSFactoryName) JMSFactoryName'||
',   RTRIM(g.JNDIFactoryName) JNDIFactoryName'||
',   RTRIM(g.ProviderURLPkg) ProviderURLPkg'||
',   RTRIM(g.JMSMessageClass) JMSMessageClass'||
',   RTRIM(g.JMSMessageType) JMSMessageType'||
' From PR_GATEWAY g'||
' left outer join pr_partner p    on g.PARTNERID   = p.PARTNERID '||
' left outer join PR_TRANSPORT  w on g.TRANSPORTID = w.TRANSPORTID '||
v_where;

IF RC1%ISOPEN THEN CLOSE RC1; END IF;
OPEN RC1 FOR v_sqlstmt ;


p_returncd := 0;
procAudit;
EXCEPTION
        WHEN NO_DATA_FOUND THEN NULL;
        WHEN OTHERS then
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GATEWAYs_change:'||SQLERRM);
End PR_GATEWAYs_change_AH1;
/


-- PR_GATEWAYS_CHANGE_AH2.sql

CREATE OR REPLACE PROCEDURE PR_GATEWAYS_CHANGE_AH2
(
    p_returncd out integer,
	p_debugFlg IN integer,
    RC1 out db_ref.RCT,
    p_partnerid IN INTEGER
) As
/*
================================================================================
Proc Name:      PR_GATEWAYs_change_AH2
Create Date:    04/12/02 02:34:37 PM
Author          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
     2006-11-15 C. Nestor    6.1.0  created
================================================================================*/

v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

v_sqlstmt         varchar2(2000) ;
v_where           varchar2(2000) ;
v_TSString        VARCHAR2(1000);

Procedure ProcAudit is
begin
    DB_ProcAuditLogI_ah1('PR_GATEWAYS_CHANGE_AH2',v_beginTS, 'p_returncd'||','||p_debugFlg
    ||','||'RC1'
    --||','||p_TS
    );
End ProcAudit;
--
-- Get Partner Name
--
BEGIN

v_where := ' Where 1=1 '|| FN_Build_Where('g.PARTNERID', 'NUMBER', p_PARTNERID);


V_sqlstmt := 'SELECT GATEWAYID '||
',   RTRIM(GATEWAYNAME)  GATEWAYNAME'||
',   g.PARTNERID'||
',   RTRIM(p.PartnerName) PARTNERNAME'||
',   g.TRANSPORTID'||
',   RTRIM(w.TRANSPORTTYPE) TRANSPORTTYPE'||
',   THREADNBR'||
',   RTRIM(g.URI)  URI'||
',   RTRIM(URILOGIN)  URILOGIN'||
',   URIPASSWORD'||
',   TRANSPORTRETRY'||
',   TRANSPORTINTERVAL'||
',   g.ENABLEDFLG'||
',   OFFLINEFLG'||
',   RTRIM(GATEWAYDESC)  GATEWAYDESC'||
',   g.UpdateDATE'||
',   AutoConfigFlg'||
',   g.AuthenticationRqdFlg'||
',   g.ConnectionTimeOut'||
',   RTRIM(g.JMSQueueName) JMSQueueName'||
',   RTRIM(g.JMSFactoryName) JMSFactoryName'||
',   RTRIM(g.JNDIFactoryName) JNDIFactoryName'||
',   RTRIM(g.ProviderURLPkg) ProviderURLPkg'||
',   RTRIM(g.JMSMessageClass) JMSMessageClass'||
',   RTRIM(g.JMSMessageType) JMSMessageType'||
' From PR_GATEWAY g'||
' left outer join pr_partner p    on g.PARTNERID   = p.PARTNERID '||
' left outer join PR_TRANSPORT  w on g.TRANSPORTID = w.TRANSPORTID '||
v_where;

IF RC1%ISOPEN THEN CLOSE RC1; END IF;

OPEN RC1 FOR v_sqlstmt ;


p_returncd := 0;
procAudit;
EXCEPTION
        WHEN NO_DATA_FOUND THEN NULL;
        WHEN OTHERS then
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GATEWAYs_change:'||SQLERRM);
End PR_GATEWAYS_CHANGE_AH2;
/

-- PR_GATEWAYS_PARTNERID.sql

CREATE OR REPLACE PROCEDURE         PR_GATEWAYS_partnerID
(
    p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_PARTNERID in NUMBER
    , p_Locale in varchar2
) As
/*
================================================================================
Proc Name:        PR_GATEWAYS
Create Date:     2002-07-08 14:55:32
Author:            SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_GATEWAYS_PartnerID',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_PARTNERID
    );
End ProcAudit;
BEGIN
    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);

    OPEN RC1 FOR
    SELECT
        GATEWAYID
    ,    RTRIM(GATEWAYNAME)  GATEWAYNAME
    ,    PARTNERID
    ,    TRANSPORTID
    ,    THREADNBR
    ,    RTRIM(URI)  URI
    ,    RTRIM(URILOGIN)  URILOGIN
    ,    URIPASSWORD
    ,    TRANSPORTRETRY
    ,    TRANSPORTINTERVAL
    ,    ENABLEDFLG
    ,    OFFLINEFLG
    ,    RTRIM(GATEWAYDESC)  GATEWAYDESC
    ,    ROWTS
    ,    CREATEDATE
    ,    VALIDATECLIENTSSLCERT
    ,    VALIDATECLIENTIP
    ,    AutoConfigFlg
    ,    CONNECTIONTIMEOUT
    ,    AUTHENTICATIONRQDFLG
    ,    RTRIM(JMSQUEUENAME) JMSQUEUENAME
    ,    RTRIM(JMSFACTORYNAME) JMSFACTORYNAME
    ,    RTRIM(JNDIFACTORYNAME) JNDIFACTORYNAME
    ,    RTRIM(PROVIDERURLPKG) PROVIDERURLPKG
    ,    RTRIM(JMSMessageClass) JMSMessageClass
    ,    RTRIM(JMSMessageType) JMSMessageType
    FROM PR_GATEWAY
    WHERE PARTNERID = NVL(p_PARTNERID, PARTNERID)
    ORDER BY UPPER(GATEWAYNAME);

    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';

    p_ReturnCd := 0;
    procAudit;
End PR_GATEWAYS_PARTNERID;
/


-- PR_GATEWAYU.sql

CREATE OR REPLACE Procedure PR_GATEWAYU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_GATEWAYID in NUMBER
	,p_GATEWAYNAME in VARCHAR2
	,p_PARTNERID in NUMBER
	,p_TRANSPORTID in NUMBER
	,p_THREADNBR in NUMBER
	,p_URI in VARCHAR2
	,p_URILOGIN in VARCHAR2
	,p_URIPASSWORD in RAW
	,p_TRANSPORTRETRY in NUMBER
	,p_TRANSPORTINTERVAL in NUMBER
	,p_ENABLEDFLG in NUMBER
	,p_OFFLINEFLG in NUMBER
	,p_GATEWAYDESC in VARCHAR2
	,p_ROWTS in NUMBER
	,p_VALIDATECLIENTSSLCERT in NUMBER
	,p_VALIDATECLIENTIP in NUMBER
	,p_AUTOCONFIGFLG in NUMBER
	,p_AUTHENTICATIONRQDFLG in NUMBER
	,p_CONNECTIONTIMEOUT in NUMBER
	,p_JMSQUEUENAME in VARCHAR2
	,p_JMSFACTORYNAME in VARCHAR2
	,p_JNDIFACTORYNAME in VARCHAR2
	,p_PROVIDERURLPKG in VARCHAR2
	,p_JMSMESSAGECLASS in VARCHAR2
	,p_JMSMESSAGETYPE in VARCHAR2
) as

/*
	================================================================================
	Proc Name:       PR_GATEWAYU
	Create Date:     2003-09-26 08:21:18
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GATEWAYU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_GATEWAYNAME=>'||p_GATEWAYNAME||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_THREADNBR=>'||p_THREADNBR||CHR(10)
		||', p_URI=>'||p_URI||CHR(10)
		||', p_URILOGIN=>'||p_URILOGIN||CHR(10)
		||', p_URIPASSWORD=>'||p_URIPASSWORD||CHR(10)
		||', p_TRANSPORTRETRY=>'||p_TRANSPORTRETRY||CHR(10)
		||', p_TRANSPORTINTERVAL=>'||p_TRANSPORTINTERVAL||CHR(10)
		||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
		||', p_OFFLINEFLG=>'||p_OFFLINEFLG||CHR(10)
		||', p_GATEWAYDESC=>'||p_GATEWAYDESC||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_VALIDATECLIENTSSLCERT=>'||p_VALIDATECLIENTSSLCERT||CHR(10)
		||', p_VALIDATECLIENTIP=>'||p_VALIDATECLIENTIP||CHR(10)
		||', p_AUTOCONFIGFLG=>'||p_AUTOCONFIGFLG||CHR(10)
		||', p_AUTHENTICATIONRQDFLG=>'||p_AUTHENTICATIONRQDFLG||CHR(10)
		||', p_CONNECTIONTIMEOUT=>'||p_CONNECTIONTIMEOUT||CHR(10)
		||', p_JMSQUEUENAME=>'||p_JMSQUEUENAME||CHR(10)
		||', p_JMSFACTORYNAME=>'||p_JMSFACTORYNAME||CHR(10)
		||', p_JNDIFACTORYNAME=>'||p_JNDIFACTORYNAME||CHR(10)
		||', p_PROVIDERURLPKG=>'||p_PROVIDERURLPKG||CHR(10)
		||', p_JMSMESSAGECLASS=>'||p_JMSMESSAGECLASS||CHR(10)
		||', p_JMSMESSAGETYPE=>'||p_JMSMESSAGETYPE||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_GATEWAY
	SET
		GATEWAYNAME=p_GATEWAYNAME
		,      PARTNERID=p_PARTNERID
		,      TRANSPORTID=p_TRANSPORTID
		,      THREADNBR=p_THREADNBR
		,      URI=p_URI
		,      URILOGIN=p_URILOGIN
		,      URIPASSWORD=p_URIPASSWORD
		,      TRANSPORTRETRY=p_TRANSPORTRETRY
		,      TRANSPORTINTERVAL=p_TRANSPORTINTERVAL
		,      ENABLEDFLG=p_ENABLEDFLG
		,      OFFLINEFLG=p_OFFLINEFLG
		,      GATEWAYDESC=p_GATEWAYDESC
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      VALIDATECLIENTSSLCERT=p_VALIDATECLIENTSSLCERT
		,      VALIDATECLIENTIP=p_VALIDATECLIENTIP
		,      AUTOCONFIGFLG=p_AUTOCONFIGFLG
		,      AUTHENTICATIONRQDFLG=p_AUTHENTICATIONRQDFLG
		,      CONNECTIONTIMEOUT=p_CONNECTIONTIMEOUT
		,      JMSQUEUENAME=p_JMSQUEUENAME
		,      JMSFACTORYNAME=p_JMSFACTORYNAME
		,      JNDIFACTORYNAME=p_JNDIFACTORYNAME
		,      PROVIDERURLPKG=p_PROVIDERURLPKG
		,      JMSMESSAGECLASS=p_JMSMESSAGECLASS
		,      JMSMESSAGETYPE=p_JMSMESSAGETYPE
	Where
		GATEWAYID = p_GATEWAYID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_GATEWAYU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GATEWAYU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GATEWAYU:'||SQLERRM);
End PR_GATEWAYU;
/


-- PR_GROUPD.sql

CREATE OR REPLACE Procedure PR_GROUPD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_GROUPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GROUPD
	Create Date:     2003-09-26 08:24:18
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GROUPD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GROUPID=>'||p_GROUPID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_GROUP
	Where
		GROUPID = p_GROUPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_GROUPD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_GROUPD;
/


-- PR_GROUPI_NODUP.sql

CREATE OR REPLACE PROCEDURE pr_GROUPi_nodup
(
     p_ReturnCd out integer
    ,p_debugFlg in pls_integer
    ,RC1 out db_ref.RCT
    ,p_PARTNERID in NUMBER
    ,p_GROUPNAME in VARCHAR2
    ,p_GROUPDESC in VARCHAR2
) as
/*
================================================================================
Proc Name:      pr_GROUPi_nodup
Create Date:    2002-07-11
Author          V Goel
================================================================================
    Return code  = 0  Good
                   1  Something went wrong
                   2  Dup
                   3  No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
Procedure ProcAudit is
v_diffTS Timestamp;
begin
    DB_ProcAuditLogI_Ah1('pr_GROUPi_nodup',v_beginTS,'RETURN_CODE'||','||p_debugFlg||','||
    P_PARTNERID||','||P_GROUPNAME||','||P_GROUPDESC||','||'RC1');
End ProcAudit;

------------
--Main
------------

BEGIN
  p_ReturnCd := 0;
  BEGIN
    Insert into pr_GROUP(
    GROUPID
    ,   PARTNERID
    ,   GROUPNAME
    ,   GROUPDESC
    ) Values (
    PR_GROUP_SEQ.nextval
    ,   p_PARTNERID
    ,   p_GROUPNAME
    ,   p_GROUPDESC
    );
    v_rowcnt := sql%ROWCOUNT;
    
    EXCEPTION
      When Dup_val_on_index Then
        ProcAudit;
        p_returncd := 2;
      When Others Then
        ProcAudit;
        Raise_application_error (-20999,'20000 - pr_GROUPi_nodup' || SQLCODE|| ' '||SQLERRM);
        p_returncd := 1;
  END;
  
  BEGIN
  
    IF p_ReturnCd > 0 THEN 
       OPEN RC1 For Select NULL, 1 RowTs From Dual;
       ProcAudit;
    ELSE
       Open RC1 FOR Select 1 ROWTS,PR_GROUP_SEQ.currval Id From Dual;
       p_returncd := 0;
       ProcAudit;
    END IF;        
    
    EXCEPTION
      When NO_DATA_FOUND Then NULL;
  END;
END PR_GROUPI_nodup;
/


-- PR_GROUPS.sql

CREATE OR REPLACE PROCEDURE PR_GROUPS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_GROUPID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GROUPS
	Create Date:     2003-09-26 00:06:33
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GROUPS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GROUPID=>'||p_GROUPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_GROUPID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		GROUPID
		,PARTNERID
		,RTRIM(GROUPNAME) GROUPNAME
		,RTRIM(GROUPDESC) GROUPDESC
		,ROWTS
		,CREATEDATE
	FROM
		PR_GROUP
	Where
		GROUPID = p_GROUPID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		GROUPID
		,PARTNERID
		,RTRIM(GROUPNAME) GROUPNAME
		,RTRIM(GROUPDESC) GROUPDESC
		,ROWTS
		,CREATEDATE
	FROM
		PR_GROUP
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_GROUPS;
/


-- PR_GROUPS_BYPARTNER.sql

CREATE OR REPLACE PROCEDURE PR_GROUPS_BYPARTNER
(
   p_ReturnCd   OUT pls_INTEGER
   ,p_DebugFlg  IN  pls_INTEGER
   ,rc1         OUT DB_REF.rct
   ,p_partnerid IN  NUMBER
   ,p_groupname IN  VARCHAR2
   ,p_Locale    IN  VARCHAR2
)
AS
/*
================================================================================
Proc Name:      pr_GROUPs_byPartner
Create Date:    12/27/01 07:38:46 AM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_begints timestamp := FN_Get_Date;
   PROCEDURE procaudit
   IS
   BEGIN
         DB_ProcAuditLogI_AH1 (
            'pr_GROUPs_byPartner',
            v_begints,
            'p_ReturnCd' ||
            ',' ||p_DebugFlg||
            ',' ||p_partnerid ||
            ',' ||p_groupname ||
            ',' ||'RC1'
         );
   END procaudit;

BEGIN
    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);

    OPEN rc1 FOR
    SELECT groupid, partnerid, RTRIM (groupname) groupname,
    RTRIM (groupdesc) groupdesc, rowts, createdate
    FROM pr_group
    WHERE partnerid = p_partnerid
    AND groupname LIKE NVL (p_groupname, '%')
    ORDER BY UPPER(groupname);

    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';

    p_ReturnCd := 0;
    procaudit;
END PR_GROUPS_BYPARTNER;
/


-- PR_GROUPU.sql

CREATE OR REPLACE Procedure PR_GROUPU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_GROUPID in NUMBER
	,p_PARTNERID in NUMBER
	,p_GROUPNAME in VARCHAR2
	,p_GROUPDESC in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_GROUPU
	Create Date:     2003-09-26 08:21:22
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GROUPU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GROUPID=>'||p_GROUPID||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_GROUPNAME=>'||p_GROUPNAME||CHR(10)
		||', p_GROUPDESC=>'||p_GROUPDESC||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_GROUP
	SET
		PARTNERID=p_PARTNERID
		,      GROUPNAME=p_GROUPNAME
		,      GROUPDESC=p_GROUPDESC
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		GROUPID = p_GROUPID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_GROUPU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GROUPU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GROUPU:'||SQLERRM);
End PR_GROUPU;
/


-- PR_GWYATTRD.sql

CREATE OR REPLACE Procedure PR_GWYATTRD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_GATEWAYID in NUMBER
	,p_TRANSPORTATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GWYATTRD
	Create Date:     2004-03-13 22:42:27
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GWYATTRD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' WHERE 1=1 ' ;
    v_where := v_where || FN_BUILD_WHERE('GATEWAYID', 'NUMBER', p_GATEWAYID) ;
	v_where := v_where || FN_BUILD_WHERE('TRANSPORTATTRID', 'NUMBER', p_TRANSPORTATTRID) ;
	
    v_sqlstmt := 
	   'Delete From PR_GWYATTR ' || v_where ;
	   
	execute immediate v_sqlstmt ;
	
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_GWYATTRD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_GWYATTRD;
/


-- PR_GWYATTRI.sql

CREATE OR REPLACE Procedure PR_GWYATTRI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT   
	,p_GATEWAYID in NUMBER
	,p_TRANSPORTATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_GWYATTRI
	Create Date:     2004-03-13 22:44:28
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GWYATTRI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_GWYATTR(
		GATEWAYID
		,TRANSPORTATTRID
		,ATTRVALUE
	) Values (
		p_GATEWAYID
		,p_TRANSPORTATTRID
		,p_ATTRVALUE
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_GWYATTRI' || SQLCODE|| ' '||SQLERRM);
END PR_GWYATTRI;
/


-- PR_GWYATTRS.sql

CREATE OR REPLACE PROCEDURE PR_GWYATTRS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_GATEWAYID IN NUMBER
	, p_TRANSPORTATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_GWYATTRS
	Create Date:     2004-03-13 22:43:51
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GWYATTRS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;           
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)
		;
v_gwyid    integer Default 0;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('GATEWAYID', 'NUMBER', p_GATEWAYID) ||
       FN_Build_Where('RA.TRANSPORTATTRID', 'NUMBER', p_TRANSPORTATTRID);
	
    if p_TRANSPORTATTRID is not null then
       v_sqlstmt :=
       'SELECT '
	   ||'GATEWAYID'
	   ||',RA.TRANSPORTATTRID'
       ||',RTRIM(ATTRNAME) ATTRNAME'
       ||',RTRIM(ATTRVALUE) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',RA.ROWTS'
       ||',RA.CREATEDATE '
       ||'FROM '
       ||'PR_GWYATTR RA INNER JOIN PR_TRANSPORT_ATTR TA '
       ||'ON RA.TRANSPORTATTRID = TA.TRANSPORTATTRID '
       || v_where ;
    ELSE
       BEGIN

       SELECT NVL(transportid,0) INTO v_gwyid FROM pr_gateway WHERE gatewayid = p_GATEWAYID ;
       
       v_sqlstmt :=
       'SELECT '
       ||'GATEWAYID'
       ||',NVL(ra.transportattrid, ta.transportattrid) transportattrid '
       ||',RTRIM(attrname) ATTRNAME'
       ||',RTRIM(NVL(attrvalue, defaultattrvalue)) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',ra.ROWTS'
       ||',ra.CREATEDATE '
       ||'from '
       ||'(select GATEWAYID, transportattrid, attrvalue, rowts, createdate from pr_GWYATTR where GATEWAYID = ' || p_GATEWAYID ||') ra '
       ||'right outer join pr_transport_attr ta '
       ||'on ra.transportattrid = ta.transportattrid '
       ||'where ta.transportid = ' ||v_gwyid
       ;
       END;
    end if ;     
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_GWYATTRS;
/


-- PR_GWYATTRU.sql

CREATE OR REPLACE Procedure PR_GWYATTRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_GATEWAYID in NUMBER
	,p_TRANSPORTATTRID in NUMBER
	,p_ROWTS in NUMBER
	,p_ATTRVALUE in VARCHAR2
) as

/*
	================================================================================
	Proc Name:       PR_GWYATTRU
	Create Date:     2004-03-13 22:43:05
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_GWYATTRU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_GATEWAYID=>'||p_GATEWAYID||CHR(10)
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_GWYATTR
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      ATTRVALUE=p_ATTRVALUE
	Where 
		GATEWAYID = p_GATEWAYID
		AND TRANSPORTATTRID = p_TRANSPORTATTRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_GWYATTRU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GWYATTRU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_GWYATTRU:'||SQLERRM);
End PR_GWYATTRU;
/


-- PR_HANDLERATTRD.sql

CREATE OR REPLACE Procedure PR_HANDLERATTRD
                                    
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer                      
   ,p_HANDLERATTRID in NUMBER                                                     
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_HANDLERATTRD                                               
	Create Date:     2004-03-11 16:30:28                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_HANDLERATTRD';                       
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)                             
		;
                                                                            
zeroDelete  Exception;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
   
   Delete From pr_handlerattr                                                     
	Where 
		HANDLERATTRID = p_HANDLERATTRID                                       
	;                                                                              
	v_RowCnt := SQL%ROWCOUNT;                                                      
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;                             
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                
	p_ReturnCd := 0;                                                               
	Exception                                                                      
		When ZeroDelete Then                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt,p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20002 pr_handlerattrD: No Data Found');     
		When OTHERS Then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);         
End pr_handlerattrD;
/


-- PR_HANDLERATTRI.sql

CREATE OR REPLACE Procedure PR_HANDLERATTRI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT                    
	,p_HANDLERID in NUMBER                                                         
	,p_ATTRNAME in VARCHAR2                                                        
	,p_RESOURCEKEY in VARCHAR2                                                     
	,p_DEFAULTATTRVALUE in VARCHAR2                                                
	,p_DESCRIPTION in VARCHAR2                                                     
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_HANDLERATTRI                                               
	Create Date:     2004-03-11 16:32:38                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             
v_PK_Value        number := 0;                                                  

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_HANDLERATTRI';                       
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)                                     
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)                                       
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)                                 
		||', p_DEFAULTATTRVALUE=>'||p_DEFAULTATTRVALUE||CHR(10)                       
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)                                 
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	Insert into pr_handlerattr(                                                    
		HANDLERATTRID                                                                 
		,HANDLERID                                                                    
		,ATTRNAME                                                                     
		,RESOURCEKEY                                                                  
		,DEFAULTATTRVALUE                                                             
		,DESCRIPTION                                                                  
	) Values (                                                                     
		PR_HANDLERATTR_SEQ.nextval                                                    
		,p_HANDLERID                                                                  
		,p_ATTRNAME                                                                   
		,p_RESOURCEKEY                                                                
		,p_DEFAULTATTRVALUE                                                           
		,p_DESCRIPTION                                                                
	);                                                                             
	p_ReturnCd := 0;                                                               
	v_RowCnt := sql%ROWCOUNT;                                                      
	IF RC1%isOpen THEN Close RC1; END IF;                                          
	Open RC1 FOR Select 
		pr_handlerattr_SEQ.currval ID, 1 RowTS From Dual;       
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                
	EXCEPTION                                                                      
		When NO_DATA_FOUND THEN                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt,p_DebugFlg, 0);                                                                 
			p_returnCd := 3;                                                             
		When DUP_VAL_ON_INDEX then                                                    
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                    
			p_returnCd := 2;                                                             
		When OTHERS Then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20002 pr_handlerattrI' || SQLCODE|| ' '||SQLERRM);                                                                    
END PR_HANDLERATTRI;
/


-- PR_HANDLERATTRS.sql

CREATE OR REPLACE PROCEDURE PR_HANDLERATTRS                                     
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT DB_REF.RCT   
	, p_HANDLERATTRID IN NUMBER                                                  
    , p_HANDLERID IN NUMBER  DEFAULT NULL                                                                 
	)                                                                              
AS                                                                              

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_HANDLERATTRS                                               
	Create Date:     2004-03-11 16:32:15                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             
v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_HANDLERATTRS';                       
v_ExceptionFlg    integer Default 0; 
v_sqlstmt         varchar2(4000) ;                                           
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_HANDLERID=>'||p_HANDLERID||CHR(10)                                           
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)                             
		;
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                             
    v_where := ' Where 1=1'||
    FN_Build_Where('HANDLERID', 'NUMBER', p_HANDLERID) ||
    FN_Build_Where('HANDLERATTRID', 'NUMBER', p_HANDLERATTRID);
    
    v_sqlstmt :=
	'SELECT                                                          
		HANDLERATTRID                                                                 
		,HANDLERID                                                                    
		,ROWTS                                                                        
		,CREATEDATE                                                                   
		,RTRIM(ATTRNAME) ATTRNAME                                                     
		,RTRIM(RESOURCEKEY) RESOURCEKEY                                               
		,RTRIM(DEFAULTATTRVALUE) DEFAULTATTRVALUE                                     
		,RTRIM(DESCRIPTION) DESCRIPTION 
        FROM 
		pr_handlerattr' || v_where ;
    
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
	OPEN RC1 FOR v_sqlstmt ; 
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                    
End PR_HANDLERATTRS;
/


-- PR_HANDLERATTRU.sql

CREATE OR REPLACE Procedure PR_HANDLERATTRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT                                 
	,p_HANDLERATTRID in NUMBER                                                     
	,p_HANDLERID in NUMBER                                                         
	,p_ROWTS in NUMBER                                                             
	,p_ATTRNAME in VARCHAR2                                                        
	,p_RESOURCEKEY in VARCHAR2                                                     
	,p_DEFAULTATTRVALUE in VARCHAR2                                                
	,p_DESCRIPTION in VARCHAR2                                                     
) as                                                                            

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_HANDLERATTRU                                               
	Create Date:     2004-03-11 16:30:50                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related  variables                                                                       
v_ProcName        varchar2(30) Default 'PR_HANDLERATTRU';                       
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_HANDLERATTRID=>'||p_HANDLERATTRID||CHR(10)                             
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)                                     
		||', p_ROWTS=>'||p_ROWTS||CHR(10)                                             
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)                                       
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)                                 
		||', p_DEFAULTATTRVALUE=>'||p_DEFAULTATTRVALUE||CHR(10)                       
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)                                 
		;
                                                                            
v_RowTS      integer :=0;                                                       
zeroUpdate  Exception;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	v_RowCnt := 0;                                                                 
	Update pr_handlerattr
	SET                                                     
		HANDLERID=p_HANDLERID                                                         
		,      ROWTS= p_RowTS + 1                                                     
		,      UPDATEDATE= v_BeginTS                                                  
		,      ATTRNAME=p_ATTRNAME                                                    
		,      RESOURCEKEY=p_RESOURCEKEY                                              
		,      DEFAULTATTRVALUE=p_DEFAULTATTRVALUE                                    
		,      DESCRIPTION=p_DESCRIPTION                                              
	Where 
		HANDLERATTRID = p_HANDLERATTRID                                       
		And RowTS = p_RowTS returning RowTS into v_RowTS;                             
	v_RowCnt := SQL%rowcount;                                                      
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;                             
	IF RC1%isOpen THEN Close RC1; END IF;                                          
	OPEN RC1 for select v_RowTS RowTS from Dual;                                   
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                
	EXCEPTION                                                                      
		WHEN TOO_MANY_ROWS Then                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt,p_DebugFlg, 1);                                                                 
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);                                                                
			RAISE_APPLICATION_ERROR(-20999, '20100 - pr_handlerattrU: Too Many Rows');   
		WHEN ZeroUpdate then                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt,p_DebugFlg, 1);                                                                 
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,v_ExceptionFlg);                                                                
			RAISE_APPLICATION_ERROR(-20999,'20100 - pr_handlerattrU: No Data Found');    
		WHEN OTHERS then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR(-20999,'20100 - pr_handlerattrU:'||SQLERRM);         
End PR_HANDLERATTRU;
/


-- PR_HANDLERD.sql

CREATE OR REPLACE Procedure PR_HANDLERD
                                        
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer                      
	,p_HANDLERID in NUMBER                                                         
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_HANDLERD                                                   
	Create Date:     2004-03-11 15:28:33                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_HANDLERD';                           
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)                                     
		;
                                                                            
zeroDelete  Exception;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	Delete From PR_HANDLER                                                         
	Where 
		HANDLERID = p_HANDLERID                                               
	;                                                                              
	v_RowCnt := SQL%ROWCOUNT;                                                      
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;                             
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,           
v_ExceptionFlg);                                                                
	p_ReturnCd := 0;                                                               
	Exception                                                                      
		When ZeroDelete Then                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt,     
p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_HANDLERD: No Data Found');         
		When OTHERS Then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,   
p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);         
End PR_HANDLERD;
/


-- PR_HANDLERI.sql

CREATE OR REPLACE Procedure PR_HANDLERI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT                    
	,p_PROVIDERFLG in NUMBER                                                       
	,p_RESOURCEKEY in VARCHAR2                                                     
	,p_DESCRIPTION in VARCHAR2                                                     
	,p_CLASSNAME in VARCHAR2                                                       
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_HANDLERI                                                   
	Create Date:     2004-03-11 15:28:26                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             
v_PK_Value        number := 0;                                                  

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related  variables                                                                       
v_ProcName        varchar2(30) Default 'PR_HANDLERI';                           
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_PROVIDERFLG=>'||p_PROVIDERFLG||CHR(10)                                 
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)                                 
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)                                 
		||', p_CLASSNAME=>'||p_CLASSNAME||CHR(10)                                     
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	Insert into PR_HANDLER(                                                        
		HANDLERID                                                                     
		,PROVIDERFLG                                                                  
		,RESOURCEKEY                                                                  
		,DESCRIPTION                                                                  
		,CLASSNAME                                                                    
	) Values (                                                                     
		PR_HANDLER_SEQ.nextval                                                        
		,p_PROVIDERFLG                                                                
		,p_RESOURCEKEY                                                                
		,p_DESCRIPTION                                                                
		,p_CLASSNAME                                                                  
	);                                                                             
	p_ReturnCd := 0;                                                               
	v_RowCnt := sql%ROWCOUNT;                                                      
	IF RC1%isOpen THEN Close RC1; END IF;                                          
	Open RC1 FOR Select 
		PR_HANDLER_SEQ.currval ID, 1 RowTS From Dual;           
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,           
v_ExceptionFlg);                                                                
	EXCEPTION                                                                      
		When NO_DATA_FOUND THEN                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt,  
p_DebugFlg, 0);                                                                 
			p_returnCd := 3;                                                             
		When DUP_VAL_ON_INDEX then                                                    
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX,               
'||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                    
			p_returnCd := 2;                                                             
		When OTHERS Then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,   
p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_HANDLERI' || SQLCODE|| ' '||SQLERRM);                                                                    
END PR_HANDLERI;
/


-- PR_HANDLERS.sql

CREATE OR REPLACE PROCEDURE PR_HANDLERS                                         
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT DB_REF.RCT                                                                      
	, p_HANDLERID IN NUMBER                                                        
	)                                                                              
AS                                                                              

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_HANDLERS                                                   
	Create Date:     2004-03-11 15:28:42                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_HANDLERS';                           
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)                                     
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
IF p_HANDLERID IS NOT NULL 
 THEN                                               
	OPEN RC1 FOR 
	SELECT                                                          
		HANDLERID                                                                     
		,PROVIDERFLG                                                                  
		,ROWTS                                                                        
		,CREATEDATE                                                                   
		,RTRIM(RESOURCEKEY) RESOURCEKEY                                               
		,RTRIM(DESCRIPTION) DESCRIPTION                                               
		,RTRIM(CLASSNAME) CLASSNAME                                                   
	FROM 
		PR_HANDLER                                                             
	Where 
		HANDLERID = p_HANDLERID                                               
	;                                                                              
	ELSE                                                                           
	OPEN RC1 FOR 
	SELECT                                                          
		HANDLERID                                                                     
		,PROVIDERFLG                                                                  
		,ROWTS                                                                        
		,CREATEDATE                                                                   
		,RTRIM(RESOURCEKEY) RESOURCEKEY                                               
		,RTRIM(DESCRIPTION) DESCRIPTION                                               
		,RTRIM(CLASSNAME) CLASSNAME                                                   
	FROM 
		PR_HANDLER                                                             
	;                                                                              
	END IF;                                                                        
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt,     
p_DebugFlg, v_ExceptionFlg);                                                    
End PR_HANDLERS;
/


-- PR_HANDLERS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_HANDLERS_AH1                                        
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT                                                                      
	, p_ClassName IN VARCHAR2                                                        
	)                                                                              
AS                                                                              

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_HANDLERS                                                   
	Create Date:     2004-03-11 15:28:42                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_HANDLERS_AH1';                           
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_CLASSNAME=>'||p_ClassName||CHR(10)                                     
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
IF p_ClassName IS NOT NULL 
 THEN                                               
	OPEN RC1 FOR 
	SELECT                                                          
		HANDLERID                                                                     
		,PROVIDERFLG                                                                  
		,ROWTS                                                                        
		,CREATEDATE                                                                   
		,RTRIM(RESOURCEKEY) RESOURCEKEY                                               
		,RTRIM(DESCRIPTION) DESCRIPTION                                               
		,RTRIM(CLASSNAME) CLASSNAME                                                   
	FROM 
		PR_HANDLER                                                             
	Where 
		CLASSNAME = RTRIM(p_ClassName)                                               
	;                                                                              
	ELSE                                                                           
	OPEN RC1 FOR 
	SELECT                                                          
		HANDLERID                                                                     
		,PROVIDERFLG                                                                  
		,ROWTS                                                                        
		,CREATEDATE                                                                   
		,RTRIM(RESOURCEKEY) RESOURCEKEY                                               
		,RTRIM(DESCRIPTION) DESCRIPTION                                               
		,RTRIM(CLASSNAME) CLASSNAME                                                   
	FROM 
		PR_HANDLER                                                             
	;                                                                              
	END IF;                                                                        
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt,     
p_DebugFlg, v_ExceptionFlg);                                                    
End PR_HANDLERS_AH1
;
/


-- PR_HANDLERU.sql

CREATE OR REPLACE Procedure PR_HANDLERU
	(
	 p_ReturnCd out integer
	,p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT                                 
	,p_HANDLERID in NUMBER                                                         
	,p_PROVIDERFLG in NUMBER                                                       
	,p_ROWTS in NUMBER                                                             
	,p_RESOURCEKEY in VARCHAR2                                                     
	,p_DESCRIPTION in VARCHAR2                                                     
	,p_CLASSNAME in VARCHAR2                                                       
) as                                                                            

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_HANDLERU                                                   
	Create Date:     2004-03-11 15:28:16                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_HANDLERU';                           
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)                                     
		||', p_PROVIDERFLG=>'||p_PROVIDERFLG||CHR(10)                                 
		||', p_ROWTS=>'||p_ROWTS||CHR(10)                                             
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)                                 
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)                                 
		||', p_CLASSNAME=>'||p_CLASSNAME||CHR(10)                                     
		;
                                                                            
v_RowTS      integer :=0;                                                       
zeroUpdate  Exception;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	v_RowCnt := 0;                                                                 
	Update PR_HANDLER
	SET                                                         
		PROVIDERFLG=p_PROVIDERFLG                                                     
		,      ROWTS= p_RowTS + 1                                                     
		,      UPDATEDATE= v_BeginTS                                                  
		,      RESOURCEKEY=p_RESOURCEKEY                                              
		,      DESCRIPTION=p_DESCRIPTION                                              
		,      CLASSNAME=p_CLASSNAME                                                  
	Where 
		HANDLERID = p_HANDLERID                                               
		And RowTS = p_RowTS returning RowTS into v_RowTS;                             
	v_RowCnt := SQL%rowcount;                                                      
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;                             
	IF RC1%isOpen THEN Close RC1; END IF;                                          
	OPEN RC1 for select v_RowTS RowTS from Dual;                                   
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,           
v_ExceptionFlg);                                                                
	EXCEPTION                                                                      
		WHEN TOO_MANY_ROWS Then                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt,    
p_DebugFlg, 1);                                                                 
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,         
v_ExceptionFlg);                                                                
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_HANDLERU: Too Many Rows');       
		WHEN ZeroUpdate then                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt,      
p_DebugFlg, 1);                                                                 
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,         
v_ExceptionFlg);                                                                
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_HANDLERU: No Data Found');        
		WHEN OTHERS then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,   
p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_HANDLERU:'||SQLERRM);             
End PR_HANDLERU;
/


-- PR_IPADDRESSD.sql

CREATE OR REPLACE Procedure PR_IPADDRESSD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_IPADDRESSID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_IPADDRESSD
	Create Date:     2003-09-26 08:24:19
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_IPADDRESSD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_IPADDRESSID=>'||p_IPADDRESSID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_IPADDRESS
	Where
		IPADDRESSID = p_IPADDRESSID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_IPADDRESSD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_IPADDRESSD;
/


-- PR_IPADDRESSI.sql

CREATE OR REPLACE Procedure PR_IPADDRESSI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_IPADDRESS in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_IPADDRESSI
	Create Date:     2003-09-26 00:01:44
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_IPADDRESSI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_IPADDRESS=>'||p_IPADDRESS||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_IPADDRESS(
		IPADDRESSID
		,PARTNERID
		,IPADDRESS
	) Values (
		PR_IPADDRESS_SEQ.nextval
		,p_PARTNERID
		,p_IPADDRESS
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_IPADDRESS_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_IPADDRESSI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_IPADDRESSI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_IPADDRESSI' || SQLCODE|| ' '||SQLERRM);
END PR_IPADDRESSI;
/


-- PR_IPADDRESSS_PARTNERID.sql

CREATE OR REPLACE PROCEDURE         PR_IPADDRESSS_PartnerId
(p_ReturnCd out integer, p_DebugFlg in pls_integer, RC1 out db_ref.RCT
, p_PARTNERID in NUMBER
) As
/*
================================================================================
Proc Name:        PR_IPADDRESSS_PartnerId
Create Date:     2002-07-08 14:43:08
Purpose:        Return all the IP address for given partnerId
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_IPADDRESSS_PartnerId',v_beginTS,
    'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_PARTNERID
    );
--End if ;
End ProcAudit;
BEGIN
Open RC1 for select
    RTRIM(IPADDRESS)  IPADDRESS
    , IPADDRESSID
From PR_IPADDRESS
Where PARTNERID = p_PARTNERID
;
p_ReturnCd := 0;
procAudit;
End PR_IPADDRESSS_PartnerId;
/


-- PR_IPDESTTYPED.sql

CREATE OR REPLACE Procedure PR_IPDESTTYPED

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_IPDESTTYPEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_IPDESTTYPED
	Create Date:     2003-09-26 08:24:21
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_IPDESTTYPED';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_IPDESTTYPEID=>'||p_IPDESTTYPEID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_IPDESTTYPE
	Where
		IPDESTTYPEID = p_IPDESTTYPEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_IPDESTTYPED: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_IPDESTTYPED;
/


-- PR_IPDESTTYPEI.sql

CREATE OR REPLACE Procedure PR_IPDESTTYPEI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_IPADDRESSID in NUMBER
	,p_DESTTYPEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_IPDESTTYPEI
	Create Date:     2003-09-26 00:01:44
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_IPDESTTYPEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_IPADDRESSID=>'||p_IPADDRESSID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_IPDESTTYPE(
		IPDESTTYPEID
		,IPADDRESSID
		,DESTTYPEID
	) Values (
		PR_IPDESTTYPE_SEQ.nextval
		,p_IPADDRESSID
		,p_DESTTYPEID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_IPDESTTYPE_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_IPDESTTYPEI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_IPDESTTYPEI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_IPDESTTYPEI' || SQLCODE|| ' '||SQLERRM);
END PR_IPDESTTYPEI;
/


-- PR_IPDESTTYPES_DESTTYPELIST.sql

CREATE OR REPLACE PROCEDURE         PR_IPDESTTYPES_DESTTYPELIST
(
    p_ReturnCd     out integer
    , p_DebugFlg    in pls_integer
    , RC1           out db_ref.RCT
    , p_Locale      in Varchar2
    , p_ipaddressid in NUMBER
) As
/*
================================================================================
Proc Name:        PR_IPDESTTYPES_DESTTYPEList
Create Date:     2002-10-07 09:52:55
Author:            SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
    	2004-03-05 S.Narain      4.2.2  Removed NLS
================================================================================
Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer     :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

PROCEDURE ProcAudit IS
BEGIN
   DB_ProcAuditLogI_Ah1('PR_IPDESTTYPES_DESTTYPEList',v_beginTS,
   'p_ReturnCd'||','||p_DebugFlg
   ||','||p_Locale
   ||','||'RC1'
   ||','||p_ipaddressid
   );
END ProcAudit;

BEGIN
   IF RC1%isOpen THEN Close RC1; END IF;

   OPEN RC1 FOR SELECT
        ipd.IPDESTTYPEID
   ,    ipd.IPADDRESSID
   ,    ipd.DESTTYPEID
   ,   d.Resourcekey
   ,   d.DEFAULTFLG
   ,   d.Description
   FROM PR_IPDESTTYPE ipd
   INNER JOIN PR_DESTTYPE d on d.DESTTYPEID = ipd.DESTTYPEID
   WHERE ipd.ipaddressid = p_ipaddressid
   ;
   p_ReturnCd := 0;
   procAudit;

END PR_IPDESTTYPES_DESTTYPELIST;
/


-- PR_LOCALES_AH1.sql

CREATE OR REPLACE PROCEDURE PR_LOCALES_AH1
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        , p_LOCALE IN VARCHAR2
        )
AS

/*
        ================================================================================
        Proc Name:       PR_LOCALES_AH1
        Create Date:     2003-10-13 13:32:37
        Author:          SYSTEM 

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-10-13 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_LOCALES_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_LOCALE=>'||p_LOCALE||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
        IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF  P_LOCALE IS NOT NULL THEN
        OPEN RC1 FOR 
        SELECT
                RTRIM(LOCALE) LOCALE
                ,RTRIM(LOCALEDESC) LOCALEDESC
                ,ROWTS
                ,CREATEDATE
                ,DEFAULTFLG
                ,RTRIM(NLSLANGUAGE) NLSLANGUAGE
                ,RTRIM(NLSSORTCD) NLSSORTCD
        FROM 
                PR_LOCALE
        Where 
                LOCALE = P_LOCALE
        ;
        ELSE
        OPEN RC1 FOR 
        SELECT
                RTRIM(LOCALE) LOCALE
                ,RTRIM(LOCALEDESC) LOCALEDESC
                ,ROWTS
                ,CREATEDATE
                ,DEFAULTFLG
                ,RTRIM(NLSLANGUAGE) NLSLANGUAGE
                ,RTRIM(NLSSORTCD) NLSSORTCD
        FROM 
                PR_LOCALE
        ;
        END IF;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_LOCALES_AH1;
/


-- PR_MDNSTATES_AH1.sql

CREATE OR REPLACE PROCEDURE PR_MDNSTATES_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_MDNSTATEKEY IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_MDNSTATES_AH1
	Create Date:     2006-08-05 00:06:43
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2006-08-05 Sujit Nair	    6.1	
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_MDNSTATES_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_MDNSTATEKEY=>'||p_MDNSTATEKEY||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

IF RC1%ISOPEN THEN CLOSE RC1; END IF;

IF p_MDNSTATEKEY IS NOT NULL THEN
	OPEN RC1 FOR
	SELECT
		 MDNSTATEKEY		
		,NAMENLSKEY		
		,ICONNAME
	FROM
		PR_MDNSTATE
	WHERE
		MDNSTATEKEY = p_MDNSTATEKEY
	;
ELSE
	OPEN RC1 FOR
	SELECT
		 MDNSTATEKEY		
		,NAMENLSKEY		
		,ICONNAME
	FROM
		PR_MDNSTATE
	;
END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_MDNSTATES_AH1;
/


-- PR_NOTIFICATIONI.sql

CREATE OR REPLACE Procedure PR_NOTIFICATIONI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_ALERTID in NUMBER
	,p_BATCHTYPE in NUMBER
	,p_BATCHCNT in NUMBER
	,p_FREQUENCY in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_NOTIFICATIONI
	Create Date:     2003-09-26 00:03:50
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_NOTIFICATIONI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ALERTID=>'||p_ALERTID||CHR(10)
		||', p_BATCHTYPE=>'||p_BATCHTYPE||CHR(10)
		||', p_BATCHCNT=>'||p_BATCHCNT||CHR(10)
		||', p_FREQUENCY=>'||p_FREQUENCY||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_NOTIFICATION(
		ALERTID
		,BATCHTYPE
		,BATCHCNT
		,FREQUENCY
	) Values (
		p_ALERTID
		,p_BATCHTYPE
		,p_BATCHCNT
		,p_FREQUENCY
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_NOTIFICATIONI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_NOTIFICATIONI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_NOTIFICATIONI' || SQLCODE|| ' '||SQLERRM);
END PR_NOTIFICATIONI;
/


-- PR_NOTIFICATIONS.sql

CREATE OR REPLACE PROCEDURE PR_NOTIFICATIONS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ALERTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_NOTIFICATIONS
	Create Date:     2003-09-26 00:06:32
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_NOTIFICATIONS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ALERTID=>'||p_ALERTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ALERTID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		ALERTID
		,BATCHTYPE
		,BATCHCNT
		,ROWTS
		,CREATEDATE
		,FREQUENCY
	FROM
		PR_NOTIFICATION
	Where
		ALERTID = p_ALERTID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		ALERTID
		,BATCHTYPE
		,BATCHCNT
		,ROWTS
		,CREATEDATE
		,FREQUENCY
	FROM
		PR_NOTIFICATION
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_NOTIFICATIONS;
/


-- PR_NOTIFICATIONU.sql

CREATE OR REPLACE Procedure PR_NOTIFICATIONU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_ALERTID in NUMBER
	,p_BATCHTYPE in NUMBER
	,p_BATCHCNT in NUMBER
	,p_ROWTS in NUMBER
	,p_FREQUENCY in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_NOTIFICATIONU
	Create Date:     2003-09-26 08:21:24
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_NOTIFICATIONU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ALERTID=>'||p_ALERTID||CHR(10)
		||', p_BATCHTYPE=>'||p_BATCHTYPE||CHR(10)
		||', p_BATCHCNT=>'||p_BATCHCNT||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_FREQUENCY=>'||p_FREQUENCY||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_NOTIFICATION
	SET
		BATCHTYPE=p_BATCHTYPE
		,      BATCHCNT=p_BATCHCNT
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      FREQUENCY=p_FREQUENCY
	Where
		ALERTID = p_ALERTID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_NOTIFICATIONU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_NOTIFICATIONU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_NOTIFICATIONU:'||SQLERRM);
End PR_NOTIFICATIONU;
/


-- PR_PARTNERI.sql

CREATE OR REPLACE Procedure PR_PARTNERI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERNAME in VARCHAR2
	,p_GLOBALPARTNERID in NUMBER
	,p_PARENTPARTNERID in NUMBER
	,p_ENABLED in NUMBER
	,p_PARTNERTYPEID in NUMBER
	,p_OLSID in VARCHAR2
	,p_LOCALE in VARCHAR2
	,p_BILLINGID in VARCHAR2
	,p_URI in VARCHAR2
	,p_CLASSIFICATIONID in NUMBER
	,p_PARTNERLOGIN in VARCHAR2
	,p_DIRRCVACCESSFLG in NUMBER
	,P_ISDEFAULTINTERNALPARTNER in  NUMBER -- 0 --> False  1 --> True
	)
AS

/*
	================================================================================
	Proc Name:       PR_PARTNERI
	Create Date:     2003-09-26 00:01:44
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
        2007-08-25  BHARANI J     6.1.1  Feature 101102
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PARTNERI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERNAME=>'||p_PARTNERNAME||CHR(10)
		||', p_GLOBALPARTNERID=>'||p_GLOBALPARTNERID||CHR(10)
		||', p_PARENTPARTNERID=>'||p_PARENTPARTNERID||CHR(10)
		||', p_ENABLED=>'||p_ENABLED||CHR(10)
		||', p_PARTNERTYPEID=>'||p_PARTNERTYPEID||CHR(10)
		||', p_OLSID=>'||p_OLSID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		||', p_BILLINGID=>'||p_BILLINGID||CHR(10)
		||', p_URI=>'||p_URI||CHR(10)
		||', p_CLASSIFICATIONID=>'||p_CLASSIFICATIONID||CHR(10)
		||', p_PARTNERLOGIN=>'||p_PARTNERLOGIN||CHR(10)
		||', p_DIRRCVACCESSFLG=>'||p_DIRRCVACCESSFLG||CHR(10)
		||', P_ISDEFAULTINTERNALPARTNER=>'||P_ISDEFAULTINTERNALPARTNER||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_PARTNER(
		PARTNERID
		,PARTNERNAME
		,GLOBALPARTNERID
		,PARENTPARTNERID
		,ENABLED
		,PARTNERTYPEID
		,OLSID
		,LOCALE
		,BILLINGID
		,URI
		,CLASSIFICATIONID
		,PARTNERLOGIN
		,DIRRCVACCESSFLG
	) Values (
		PR_PARTNER_SEQ.nextval
		,p_PARTNERNAME
		,p_GLOBALPARTNERID
		,p_PARENTPARTNERID
		,p_ENABLED
		,p_PARTNERTYPEID
		,p_OLSID
		,p_LOCALE
		,p_BILLINGID
		,p_URI
		,p_CLASSIFICATIONID
		,p_PARTNERLOGIN
		,p_DIRRCVACCESSFLG
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	
		
	
	
	IF P_ISDEFAULTINTERNALPARTNER=1 THEN
		Update PR_COMMUNITYMANAGERDEFAULT
		SET 
		partnerid=PR_PARTNER_SEQ.currval;
	END IF;
	        
	
	
	
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_PARTNER_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_PARTNERI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_PARTNERI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PARTNERI' || SQLCODE|| ' '||SQLERRM);
END PR_PARTNERI;
/


-- PR_PARTNERS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_PARTNERS_AH1
(
      p_RETURNCD        in out    pls_INTEGER
    , p_DebugFlg        in    pls_integer
    , RC1               in out  db_ref.rct          -- rows
    , RC2               in out  db_ref.rct          -- count
    , p_sortkey         in      varchar2
    , p_PageSizeNbr     in    pls_Integer
    , p_PageNbr         in    pls_Integer
    , p_PartnerName     in    varchar2
    , p_BusinessID      in    Varchar2
    , p_businessidType  in    varchar2
    , p_Locale          in    varchar2
    , p_partnertype     in    varchar2   -- ALL, 'INTERNAL PARTNER' ,'EXTERNAL PARTNER','HUB ADMINISTRATOR'
) AS
/*
================================================================================
Proc Name:      PR_PARTNERS_AH1
Create Date:    2002-07-11
Author          V Goel
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE        NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-04-07  S.Narain      4.2.2  BGTRK00041845
    2007-08-25  BHARANI J     6.1.1  Feature 101102
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  v_beginTS          timestamp(6):= FN_Get_Date;
  StartRow           pls_integer;
  EndRow             pls_integer;
  sqlstmt            varchar2(4000);
  sql_inner          varchar2(4000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(4000);
  sql_Where varchar2(4000) := ' Where 1=1';
  v_Locale           varchar2(8) := '';
  v_partnertype   varchar2(4000);
  
  
--
-- Procedures
--
Procedure ProcAudit is
v_diffTS Timestamp;
begin
    --If p_debugflg =1 Then
    DB_ProcAuditLogI_AH1('pr_PARTNERs_ah1',v_beginTS,
    'p_return_code'||','||
    p_debugflg||','||
    'RC1'||','||
    'RC2'||','||
    p_sortkey||','||
    p_PageSizeNbr||','||
    p_PageNbr||','||
    p_partnerName||','||
    p_BusinessID||','||
    p_businessidType
    ,0,0);
    --End if ;
End ProcAudit;
--

-----------------------------------------------------------
--              M A I N                                  --
-----------------------------------------------------------
BEGIN

    IF p_Locale IS NULL THEN
        v_Locale := FN_GET_DEFAULT_LOC();
    ELSE
        v_Locale := p_Locale;        
    END IF;    

     
    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(v_Locale);
    
 
    
    -- BGTRK00041845
    --sql_where := ' Where 1 =1 AND p.Locale = ''' || v_Locale || '''' || ' ' ||
    
    sql_where := ' Where 1 =1 '||
    ' AND p.ENABLED <> 2 ' || ------------ added for Delete Partner requirement------------
    FN_Build_Where('p.PARTNERNAME',   'VARCHAR2_LIKE', p_PartnerName)||
    FN_Build_Where('b.BUSINESSID',    'VARCHAR2_LIKE', p_BusinessID);
    
    If p_BusinessID is not null then
        sql_where := sql_where || FN_Build_Where('b.BUSINESSIDTYPE','NUMBER', p_businessidType);
    End If;
    
      
      
	If upper(p_partnertype) = 'ALL' then
		v_partnertype := '';
	elsif upper(p_partnertype) = 'INTERNAL PARTNER' then
		v_partnertype := 'HubOwner';
	elsif upper(p_partnertype) = 'EXTERNAL PARTNER' then
		v_partnertype:= 'HubPartner';
	elsif upper(p_partnertype) = 'HUB ADMINISTRATOR' then
		v_partnertype := 'HubOperator';
	ELSE 
		v_partnertype := '';
	End if;
    
	If p_partnertype is not null then
		sql_where := sql_where || FN_Build_Where('PARTNERTYPENAME','VARCHAR2', v_partnertype);
	End if; 
    
   
    
    --DB_Print('Sql_Where ', sql_Where);
    
    if RC1%ISOPEN then close RC1; end if;
    if RC2%ISOPEN then close RC2; end if;
    sqlstmt :=            'select Distinct p.PARTNERID '               ||chr(10);
    sqlstmt := sqlstmt || ', RTRIM(p.PARTNERNAME)  PARTNERNAME '||chr(10);
    sqlstmt := sqlstmt || ', p.GLOBALPARTNERID '                ||chr(10);
    sqlstmt := sqlstmt || ', GP.PartnerName GlobalPartnerName'||chr(10);
    sqlstmt := sqlstmt || ', p.PARENTPARTNERID '                ||chr(10);
    sqlstmt := sqlstmt || ', PP.PartnerName ParentPartnerName'||chr(10);
    sqlstmt := sqlstmt || ', p.ENABLED '                        ||chr(10);
    sqlstmt := sqlstmt || ', p.PARTNERTYPEID '                  ||chr(10);
    sqlstmt := sqlstmt || ', RTRIM(p.OLSID)  OLSID '            ||chr(10);
    sqlstmt := sqlstmt || ', RTRIM(p.BILLINGID) BILLINGID '     ||chr(10);
    sqlstmt := sqlstmt || ', RTRIM(p.PARTNERLOGIN) PARTNERLOGIN '     ||chr(10);
    sqlstmt := sqlstmt || ', p.DIRRCVACCESSFLG '     ||chr(10);
    sqlstmt := sqlstmt || ', DECODE(d.partnerid,null,0,1 ) ISDEFAULTINTERNALPARTNER ' || chr(10);


--    sqlstmt := sqlstmt || ', b.BUSINESSID';
    sql_inner := ' From pr_PARTNER p
                 Left outer Join pr_busIdentifier b ON p.PARTNERID =  b.PARTNERID';
    sql_outer := ' Left outer join pr_partner GP on p.GlobalPARTNERID =  GP.PARTNERID
                   Left outer join pr_partner PP on p.ParentPARTNERID =  PP.PARTNERID 
                   Left Outer Join PR_COMMUNITYMANAGERDEFAULT d on d.partnerid = p.partnerid
                   Left Outer join pr_partnertype pt on pt.PARTNERTYPEID = p.PARTNERTYPEID';
    
    open RC2 for 'select count(distinct p.partnerName) TotalRowsNbr  '||sql_inner||sql_outer||sql_where;
-- Order by Logic
    if p_sortkey is not null then
       sql_Where := sql_Where ||' order by '||p_sortkey;
    end if;
-- Set Paging Logic
    sqlstmt := sqlstmt ||sql_inner||sql_outer||sql_where;
    
   

    sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);
    --DB_Print('sqlstmt: ', sqlstmt);
    Open RC1 for  sqlstmt;
    p_returnCd := 0;
    procAudit;
    
    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';
    
Exception
    WHEN NO_DATA_FOUND THEN 
        p_ReturnCd := 3;
    WHEN OTHERS THEN
        procAudit;
        p_returnCd := 1;
        Raise_application_error (-20999,'20000 - pr_partnerS_ah1' || SQLCODE|| ' '||SQLERRM);
End PR_PARTNERS_AH1;
/


-- PR_PARTNERS_AH4.sql

CREATE OR REPLACE PROCEDURE PR_PARTNERS_AH4
(
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_PARTNERTYPENAME IN VARCHAR2
)
AS
/*
        ================================================================================
        Proc Name:       PR_PARTNERS_AH4
        Create Date:     2003-05-30 16:20:59
        Author:          SYSTEM

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-05-30 SYSTEM           3.5 Created

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_PARTNERS_AH4';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
                'p_DebugFlg=>'||p_DebugFlg||CHR(10);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;

    OPEN RC1 FOR
    SELECT
        RTRIM(BusinessID)  BusinessID
    FROM
    PR_PARTNER p
    INNER JOIN PR_BusIdentifier B ON p.partneriD = B.partneriD
    INNER JOIN PR_PARTNERTYPE t ON p.PARTNERTYPEID = t.PARTNERTYPEID
    WHERE t.PARTNERTYPENAME = p_PARTNERTYPENAME;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   --     COMMIT;
END PR_PARTNERS_AH4;
/


-- PR_PARTNERS_AH5.sql

CREATE OR REPLACE PROCEDURE         PR_PARTNERS_AH5
        (
        p_ReturnCd  OUT  PLS_INTEGER
        , p_DebugFlg  IN  PLS_INTEGER
        , RC1  OUT  DB_REF.RCT
        )
AS

/*
        ================================================================================
        Proc Name:       PR_PARTNERS_AH5
        Create Date:     2003-05-30 16:20:59
        Author:          SYSTEM

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2003-05-30 SYSTEM           3.5 Return enabled partnerId.

        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_PARTNERS_AH5';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    Open RC1 for
    Select
        PartnerID
    FROM
        PR_PARTNER
    Where ENABLED = 1
    ;
        p_ReturnCd := 0;
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --      COMMIT;
End PR_PARTNERS_AH5;
/


-- PR_PARTNERS_BUSINESSID.sql

CREATE OR REPLACE PROCEDURE PR_PARTNERS_BUSINESSID
(
    p_ReturnCd out pls_integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_Locale in Varchar2
    , p_BusinessID in NUMBER
    , p_businessidName in varchar2
)
AS
/*
================================================================================
Proc Name:      pr_PARTNERs_businessId
Create Date:    12/20/01 12:43:29 PM
Author          SYSTEM
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  -------------  -----  ---------------------------------------------
DATE       NAME           Ver    DESCRIPTION
---------  -------------  -----  ---------------------------------------------
2001-12-20 SYSTEM                Created
2003-04-23 V.Goel         3.5    Globalization
2003-04-25 L.Ling         3.5    Added PARTNERLOGIN.
2003-05-30 L.Ling         3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
2004-03-05 S.Narain      4.2.2  Removed NLS
2006-06-17 S.Priani       6.1    CR 101607
2007-08-25 BHARANI J     6.1.1  Feature 101102
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

PROCEDURE ProcAudit IS
v_diffTS Timestamp;
BEGIN
 --  IF debug ='Y' THEN
   DB_ProcAuditLogI_AH1('pr_PARTNERs_businessid',v_beginTS,'RETURN_CODE'||','||p_DEBUGFlg||','||
   p_BusinessID||','||p_businessidName||','||p_Locale||','||'RC1');
 --  END IF;
END ProcAudit;

BEGIN
   OPEN RC1 FOR
   SELECT
   p.PARTNERID
   ,   RTRIM(PARTNERNAME)  PARTNERNAME
   ,   GLOBALPARTNERID
   ,   PARENTPARTNERID
   ,   ENABLED
   ,   PARTNERTYPEID
   ,   RTRIM(OLSID)  OLSID
   ,   RTRIM(p.LOCALE)  LOCALE
   ,   b.BUSINESSID
   ,   RTRIM(bt.Resourcekey) Resourcekey
   ,   RTRIM(PARTNERLOGIN) PARTNERLOGIN
   ,   bt.Description	Description
   ,   DECODE(d.partnerid,null,0,1) ISDEFAULTINTERNALPARTNER
   FROM pr_PARTNER p ,  PR_BUSIDENTIFIER b,  PR_BUSINESSIDTYPE bt, PR_COMMUNITYMANAGERDEFAULT d
   WHERE p.PARTNERID =  b.PARTNERID
   AND   b.BUSINESSIDTYPE =  bt.BUSINESSIDTYPE
   AND   b.BUSINESSID = RTRIM(TO_CHAR(p_BUSINESSID))
   AND   bt.BUSINESSIDNAME = p_BUSINESSIDNAME
   AND   d.partnerid(+) = p.partnerid 
;

   p_ReturnCd := 0;
   procAudit;
 --  COMMIT;

END PR_PARTNERS_BUSINESSID;
/


-- PR_PARTNERS_BUSINESSID_AH1.sql

CREATE OR REPLACE PROCEDURE PR_PARTNERS_BUSINESSID_AH1
(
    p_ReturnCd out pls_integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_BUSINESSID in VARCHAR2
)
AS
/*
================================================================================
Proc Name:      pr_PARTNERs_businessId_AH1
Create Date:    12/20/01 12:43:29 PM
Author          SYSTEM
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  -------------  -----  ---------------------------------------------
DATE       NAME           Ver    DESCRIPTION
---------  -------------  -----  ---------------------------------------------
2004-03-29 T. Dang               Created
2006-10-05 S.Priani              CR 102881

================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found      
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_ProcName        varchar2(30) Default 'PR_PARTNERS_BUSINESSID_AH1';
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_ExceptionFlg    integer Default 0; 
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_BUSINESSID=>'||p_BUSINESSID||CHR(10)
		;

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF; 
   OPEN RC1 FOR
   SELECT p.PARTNERID 
	,p.PARTNERTYPEID
	,RTRIM(p.PARTNERNAME)  PARTNERNAME
   FROM pr_PARTNER p INNER JOIN PR_BUSIDENTIFIER b
   ON p.PARTNERID =  b.PARTNERID
   WHERE   b.BUSINESSID = p_BUSINESSID
   ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --  COMMIT;
END PR_PARTNERS_BUSINESSID_AH1;
/


-- PR_PARTNERS_CHANNEL.sql

CREATE OR REPLACE PROCEDURE         PR_PARTNERs_channel
(p_ReturnCd out integer
,p_DebugFlg in pls_integer
,RC1 out db_ref.RCT
,p_PARTNERID in NUMBER
) As
/*
===========================================================================
Proc Name:      PR_PARTNERs_channel
Create Date:    2002-07-03 02:20:34 PM
Author          SYSTEM
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ------------------------------------------
DATE       NAME       Ver        DESCRIPTION
---------  --------------------  ------------------------------------------
2002-07-03 J.Chen     3.3        Created
2002-07-09 J.Chen     3.3        Use dynamic sql for rc1 otherwise getting ORA - 600[qctcte1]
2003-05-15 V.Goel     3.5        Use new channel logic
2003-07-01 M.Mount    3.5        Changed input paramters to standard.
2007-08-25 BHARANI J  6.1.1      Feature 101102
===========================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
--If p_DebugFlg in ('1', 'Y') Then
DB_ProcAuditLogI_AH1('PR_PARTNERs',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
||','||p_PARTNERID
);
--End if ;
End ProcAudit;
BEGIN
Open RC1 for
select
    p.PARTNERID
,   RTRIM(PARTNERNAME)  PARTNERNAME
,   GLOBALPARTNERID
,   PARENTPARTNERID
,   ENABLED
,   PARTNERTYPEID
,   RTRIM(OLSID)  OLSID
,   RTRIM(LOCALE)  LOCALE
,   RTRIM(BILLINGID)  BILLINGID
,   ROWTS
,   DECODE(d.partnerid,null,0,1) ISDEFAULTINTERNALPARTNER
--,   CREATEDATE
From PR_PARTNER p
Left Outer Join PR_COMMUNITYMANAGERDEFAULT d on d.partnerid = p.partnerid
Where p.PARTNERID IN (
select
Case
    When frpartnerid = p_PARTNERID Then ToPartnerID
    When ToPartnerID = p_PARTNERID Then frpartnerid
    End as partnerid
        from PR_RoutingChannel
)
;
p_ReturnCd := 0;
procAudit;
/*
'select
    PARTNERID
,   RTRIM(PARTNERNAME)  PARTNERNAME
,   GLOBALPARTNERID
,   PARENTPARTNERID
,   ENABLED
,   PARTNERTYPEID
,   RTRIM(OLSID)  OLSID
,   RTRIM(LOCALE)  LOCALE
,   RTRIM(BILLINGID)  BILLINGID
,   ROWTS
,   CREATEDATE
From PR_PARTNER p
Where PARTNERID IN (
select distinct partnerids from (
        select frpartnerid as partnerids
        from pr_channel
        where topartnerid = '||p_partnerId ||
        ' union all
        select topartnerid as partnerids
        from pr_channel
        where frpartnerid = '||p_partnerId || '))'
*/
End PR_PARTNERs_channel;
/


-- PR_PARTNERS_CHILDREN.sql

CREATE OR REPLACE PROCEDURE         PR_PARTNERs_Children
(p_ReturnCd out integer
,p_DebugFlg in pls_integer
,RC1 out db_ref.RCT
,p_PARTNERID in NUMBER
) As
/*
===========================================================================
Proc Name:      PR_PARTNERs_Children
Create Date:    2002-07-03 02:20:34 PM
Purpose:        Return all the children of the input partnerID.
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ------------------------------------------
DATE       NAME       Ver        DESCRIPTION
---------  --------------------  ------------------------------------------
2002-07-03 J.Chen     3.3        Created
2003-07-01 M.Mount    3.5        Changed input parameters to standard.
===========================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
--If p_DebugFlg in ('1', 'Y') Then
DB_ProcAuditLogI_AH1('PR_PARTNERs',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
||','||'RC1'
||','||p_PARTNERID
);
--End if ;
End ProcAudit;
BEGIN
Open RC1 for select
    PARTNERID
,   RTRIM(PARTNERNAME)  PARTNERNAME
,   GLOBALPARTNERID
,   PARENTPARTNERID
,   ENABLED
,   PARTNERTYPEID
,   RTRIM(OLSID)  OLSID
,   RTRIM(LOCALE)  LOCALE
,   RTRIM(BILLINGID)  BILLINGID
,   ROWTS
,   CREATEDATE
From PR_PARTNER
Connect by prior PartnerId = ParentPartnerId
Start with ParentPartnerId = p_PartnerId
;
p_ReturnCd := 0;
procAudit;
End PR_PARTNERs_Children;
/


-- PR_PARTNERS_DEAFULTINTERNAL.sql

CREATE OR REPLACE PROCEDURE PR_PARTNERS_DEFAULTINTERNAL
(
    p_ReturnCd out integer,
    p_DebugFlg in pls_integer,
    RC1 out db_ref.RCT
    
) 
AS
/*
================================================================================
Proc Name:      PR_PARTNERS_DEFAULTINTERNAL
Create Date:    11/10/07 15:11:29 PM
Author          AMIT KUMAR VIJAY
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME                  DESCRIPTION
---------  --------------------  ---------------------------------------------
11-10-2007 AMIT KUMAR            6.1.1   Feature 101102                                 
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PARTNERS_DEFAULTINTERNAL';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
        SELECT
            a.PARTNERID,
            RTRIM(PARTNERNAME) PARTNERNAME
        FROM    pr_PARTNER a
        INNER JOIN PR_COMMUNITYMANAGERDEFAULT C on C.partnerid = a.partnerid
	Left Outer Join PR_PARTNERTYPE d on a.partnertypeid = d.partnertypeid
        WHERE   d.PARTNERTypeNAME = 'HubOwner';
        
        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

        EXCEPTION
            WHEN NO_DATA_FOUND THEN NULL;
            WHEN OTHERS THEN
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                RAISE_APPLICATION_ERROR (-20999, '20000' || SQLCODE || ' ' || SQLERRM);
                p_ReturnCd := 1;
END PR_PARTNERS_DEFAULTINTERNAL;
/


-- PR_PARTNERS_LOGIN.sql

CREATE OR REPLACE PROCEDURE         PR_PARTNERS_LOGIN
(
   p_ReturnCd out pls_integer,
   p_DebugFlg in pls_integer,
   RC1 out db_ref.RCT,
   p_PARTNERLOGIN in varchar2
)
As
/*
===========================================================================
Proc Name:      PR_PARTNERS_LOGIN
Create Date:    04/28/03 02:20:34 PM
Author          L.Ling

::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ------------------------------------------
DATE       NAME       Ver        DESCRIPTION
---------  --------------------  ------------------------------------------
2003-04-23   L.Ling     3.5        Created
2003-05-30   V.Goel     3.5        Add BusinessID -- if multiple choose any one
2003-07-01   M.Mount    3.5        Changed input parameters to standard.
2004-03-30   S.Priani	4.2.2	   Filter on ENABLED flag not to show "DELETED" (=2)
2007-08-25   BHARANI J   6.1.1      Feature 101102
===========================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

PROCEDURE ProcAudit IS
BEGIN
--   IF p_DebugFlg IN ('1', 'Y') THEN
   DB_ProcAuditLogI_AH1('PR_PARTNERS_LOGIN',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
   ||','||'RC1'
   ||','||p_PARTNERLOGIN
   );
--   END IF;
END ProcAudit;

BEGIN
   OPEN RC1 FOR
   SELECT  p.PARTNERID, B.BusinessID , 
   DECODE(d.partnerid,null,0,1) ISDEFAULTINTERNALPARTNER
   FROM    PR_PARTNER P
   Left outer Join PR_BusIdentifier B on P.PartnerID = B.PartnerID
   Left Outer Join PR_COMMUNITYMANAGERDEFAULT d on d.partnerid = p.partnerid
   WHERE   PARTNERLOGIN = p_PARTNERLOGIN
	AND ENABLED <> 2  ------------ added for Delete Partner requirement------------
   And rownum <2;

   p_ReturnCd := 0;
   procAudit;
/*
   SELECT  PARTNERID
   FROM    PR_PARTNER
   WHERE   PARTNERLOGIN = p_PARTNERLOGIN;

*/
END PR_PARTNERS_LOGIN;
/


-- PR_PARTNERS_NAME.sql

CREATE OR REPLACE PROCEDURE PR_PARTNERS_NAME (
        p_ReturnCd out pls_integer,
        p_DebugFlg in pls_integer,
        RC1 out db_ref.RCT,
        p_PartnerName in varchar2
    ) As
/*
================================================================================
Proc Name:      pr_PARTNERs_name
Create Date:    12/20/01 12:43:29 PM
Author          SYSTEM
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME                  DESCRIPTION
---------  --------------------  ---------------------------------------------
2001-12-20   SYSTEM             Created
2003-06-05   V.Goel     3.5     Wild card is no longer needed, remove it and allow for appostrophy in p_PartnerName
2003-07-01   M.Mount    3.5     Changed input parameters to standard.
2004-03-30   S.Priani	4.2.2	Filter on ENABLED flag not to show "DELETED" (=2)
2007-08-25   BHARANI J  6.1.1   Feature 101102
================================================================================

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
      v_error INTEGER := 0;
      v_errmsg VARCHAR2 (255);
      v_rowcnt INTEGER := 0;
      v_begints timestamp := FN_Get_Date;
      sqlarea VARCHAR2 (1000);
      v_value pr_partner.partnerName%TYpe ;

      PROCEDURE procaudit
      IS
--         v_diffts timestamp;
      BEGIN
--         IF p_DebugFlg = 'Y'         THEN
            DB_ProcAuditLogI_ah1 (   --db_general_pkg.audit_log (
               'PR_PARTNERS_NAME',
               v_begints,
               'p_ReturnCd' ||',' ||
               p_DebugFlg    ||',' ||
               p_PartnerName ||',' ||
               'RC1'
            );
--         END IF;
      END procaudit;
------------------------------------------------------------------------
--                            M  A  I  N
------------------------------------------------------------------------
   BEGIN
    IF rc1%ISOPEN THEN CLOSE rc1; END IF;
    OPEN rc1 FOR
    SELECT a.PARTNERID,
        RTRIM(PARTNERNAME) PARTNERNAME
        , GLOBALPARTNERID
        , PARENTPARTNERID
        , ENABLED
        , PARTNERTYPEID
        , RTRIM(OLSID) OLSID
        , RTRIM(LOCALE) LOCALE
        , ROWTS
        , DECODE(d.partnerid,null,0,1) ISDEFAULTINTERNALPARTNER
    From pr_PARTNER a
         Left Outer Join PR_COMMUNITYMANAGERDEFAULT d on d.partnerid = a.partnerid
    Where PartnerName = p_PartnerName
	AND ENABLED <> 2; ------------ added for Delete Partner requirement------------
      p_ReturnCd := 0;
      procaudit;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         NULL;
      WHEN OTHERS
      THEN
         procaudit;
         raise_application_error (-20999, '20000 PR_PARTNERS_NAME' || SQLCODE || ' ' || SQLERRM);
         ROLLBACK WORK;
         p_ReturnCd := 1;


--vc_dba_execution_time.more_than_nn_sec('pr_PARTNERs', v_beginTime);
End pr_partnerS_name;
/


-- PR_PARTNERS_PARENT.sql

CREATE OR REPLACE PROCEDURE PR_PARTNERS_PARENT
(p_ReturnCd out pls_integer
, p_DebugFlg in pls_integer
, RC1 out db_ref.RCT
, p_Locale in Varchar2
, p_PARTNERID in NUMBER
)
 AS
/*
===========================================================================
Proc Name:      PR_PARTNERS_PARENT
Create Date:    04/17/02 02:20:34 PM
Author          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-03-05 S.Priani      4.2.2   Removed NLS
    2007-08-25  BHARANI J     6.1.1  Feature 101102
===========================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

PROCEDURE ProcAudit IS
BEGIN
   DB_ProcAuditLogI_ah1('PR_PARTNERs',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
   ||','||p_Locale
   ||','||'RC1'
   ||','||p_PARTNERID
   );
END ProcAudit;

BEGIN
   OPEN RC1 FOR
   SELECT
       a.PARTNERID
   ,   RTRIM(a.PARTNERNAME)  PARTNERNAME
   ,   a.GLOBALPARTNERID
   ,   a.PARENTPARTNERID
   ,   a.ENABLED
   ,   a.PARTNERTYPEID
   ,   RTRIM(a.OLSID)  OLSID
   ,   RTRIM(a.LOCALE)  LOCALE
   ,   RTRIM(a.BILLINGID)  BILLINGID
   ,   a.ROWTS
   ,   a.CREATEDATE
   ,   b.partnername ParentPartnerName
   ,   c.partnername GlobalPartnerName
   ,   a.URI
   ,   a.CLASSIFICATIONID
   ,   Loc.RESOURCEKEY
   ,   Loc.Description
   ,   RTRIM(a.PARTNERLOGIN) PARTNERLOGIN
   ,   a.DIRRCVACCESSFLG
   ,   DECODE(d.partnerid,null,0,1) ISDEFAULTINTERNALPARTNER
   FROM
       PR_PARTNER a
   Left Outer Join PR_CLASSIFICATION loc on a.CLASSIFICATIONID = Loc.CLASSIFICATIONID
   Left Outer Join pr_partner b on b.partnerid = a.parentPartnerId
   Left Outer Join pr_partner c on c.partnerid = a.globalpartnerId
   Left Outer Join PR_COMMUNITYMANAGERDEFAULT d on d.partnerid = a.partnerid
   WHERE  a.PARTNERID = p_PARTNERID
   ;

   p_ReturnCd := 0;
   procAudit;
END PR_PARTNERS_PARENT;
/


-- PR_PARTNERS_TYPEID.sql

CREATE OR REPLACE PROCEDURE PR_PARTNERS_TYPEID
(
    p_ReturnCd out integer,
    p_DebugFlg in pls_integer,
    RC1 out db_ref.RCT,
    p_PARTNERTypeId in integer
) AS
/*
================================================================================
Proc Name:      PR_PARTNERS_TYPEID
Create Date:    12/20/01 12:43:29 PM
Author          SYSTEM
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME                  DESCRIPTION
---------  --------------------  ---------------------------------------------
12/20/01   SYSTEM                Created
2003-07-01 M.Mount               Chagned input parameters to standard.
2003-07-29 L.Ling                Changed data type from varchar to integer for p_PARTNERTypeId.
2006-04-12 Shishir N        6.1 60217 Partner Migration
                                 Restructured this proc and removed dynamic sql.
2007-08-25 BHARANI J     6.1.1   Feature 101102                                 
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_PARTNERS_TYPEID';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :='p_DebugFlg=>'||p_DebugFlg||CHR(10)||',p_PARTNERTypeId=>'||TO_CHAR(p_PARTNERTypeId);
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	  IF p_PARTNERTypeId is NOT NULL THEN
    OPEN RC1 FOR
        SELECT
            a.PARTNERID,
            RTRIM(PARTNERNAME) PARTNERNAME
            ,GLOBALPARTNERID,
            PARENTPARTNERID,
            ENABLED,
            PARTNERTYPEID,
            RTRIM(OLSID) OLSID,
            RTRIM(LOCALE) LOCALE,
            RowTS,
            DECODE(d.partnerid,null,0,1) ISDEFAULTINTERNALPARTNER
        FROM    pr_PARTNER a
        Left Outer Join PR_COMMUNITYMANAGERDEFAULT d on d.partnerid = a.partnerid
        WHERE   PARTNERTypeId = p_PARTNERTypeId
        AND a.ENABLED <> 2 
        ORDER BY PARTNERTypeId;
     ELSE
       OPEN RC1 FOR
       SELECT
			 					            a.PARTNERID,
			 					            RTRIM(PARTNERNAME) PARTNERNAME
			 					            ,GLOBALPARTNERID,
			 					            PARENTPARTNERID,
			 					            ENABLED,
			 					            PARTNERTYPEID,
			 					            RTRIM(OLSID) OLSID,
			 					            RTRIM(LOCALE) LOCALE,
			 					            RowTS,
			 					             DECODE(d.partnerid,null,0,1) ISDEFAULTINTERNALPARTNER
					        FROM    pr_PARTNER a
					        Left Outer Join PR_COMMUNITYMANAGERDEFAULT d on d.partnerid = a.partnerid
					        where a.ENABLED <> 2 
					        ;
     END IF;

        p_ReturnCd := 0;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

        EXCEPTION
            WHEN NO_DATA_FOUND THEN NULL;
            WHEN OTHERS THEN
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                RAISE_APPLICATION_ERROR (-20999, '20000' || SQLCODE || ' ' || SQLERRM);
                p_ReturnCd := 1;
END PR_PARTNERS_TYPEID;
/


-- PR_PARTNERTYPE_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE PR_PARTNERTYPE_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_PARTNERTYPEID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PARTNERTYPE_LOC_VWS
	Create Date:     2003-09-26 00:10:31
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-03-05 S.Priani      4.2.2  Removed NLS

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PARTNERTYPE_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERTYPEID=>'||p_PARTNERTYPEID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PARTNERTYPEID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		PARTNERTYPEID
		,RTRIM(PARTNERTYPENAME) PARTNERTYPENAME
		,OPERATORFLG
		,OWNERFLG
		,TYPEFLG
		,ROWTS
		,CREATEDATE
		,VISIBILITY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
	FROM
		PR_PARTNERTYPE
	Where
		PARTNERTYPEID = p_PARTNERTYPEID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		PARTNERTYPEID
		,RTRIM(PARTNERTYPENAME) PARTNERTYPENAME
		,OPERATORFLG
		,OWNERFLG
		,TYPEFLG
		,ROWTS
		,CREATEDATE
		,VISIBILITY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
	FROM
		PR_PARTNERTYPE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_PARTNERTYPE_LOC_VWS;
/


-- PR_PARTNERU.sql

CREATE OR REPLACE Procedure PR_PARTNERU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_PARTNERNAME in VARCHAR2
	,p_GLOBALPARTNERID in NUMBER
	,p_PARENTPARTNERID in NUMBER
	,p_ENABLED in NUMBER
	,p_PARTNERTYPEID in NUMBER
	,p_OLSID in VARCHAR2
	,p_LOCALE in VARCHAR2
	,p_BILLINGID in VARCHAR2
	,p_ROWTS in NUMBER
	,p_URI in VARCHAR2
	,p_CLASSIFICATIONID in NUMBER
	,p_PARTNERLOGIN in VARCHAR2
	,p_DIRRCVACCESSFLG in NUMBER
	,P_ISDEFAULTINTERNALPARTNER in  pls_integer -- 0 --> False  1 --> True
) as

/*
	================================================================================
	Proc Name:       PR_PARTNERU
	Create Date:     2003-09-26 08:21:26
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM        3.5    Created
	2004-12-22 S.Priani	 4.3	BGTRK00047098
	2006-12-17 S.Priani	 6.1	CR103585
	2007-08-25 BHARANI J     6.1.1  Feature 101102
	2009-06-19 Amit Vijay	62 FP1 defect 108618

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PARTNERU';
v_ExceptionFlg    integer Default 0;
v_DGATEWAYID	  integer Default 0;
CURSOR CUR_GATEWAY 
	IS 
		SELECT GatewayID FROM PR_GATEWAY
			WHERE partnerid=p_PARTNERID
;

v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_PARTNERNAME=>'||p_PARTNERNAME||CHR(10)
		||', p_GLOBALPARTNERID=>'||p_GLOBALPARTNERID||CHR(10)
		||', p_PARENTPARTNERID=>'||p_PARENTPARTNERID||CHR(10)
		||', p_ENABLED=>'||p_ENABLED||CHR(10)
		||', p_PARTNERTYPEID=>'||p_PARTNERTYPEID||CHR(10)
		||', p_OLSID=>'||p_OLSID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		||', p_BILLINGID=>'||p_BILLINGID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_URI=>'||p_URI||CHR(10)
		||', p_CLASSIFICATIONID=>'||p_CLASSIFICATIONID||CHR(10)
		||', p_PARTNERLOGIN=>'||p_PARTNERLOGIN||CHR(10)
		||', p_DIRRCVACCESSFLG=>'||p_DIRRCVACCESSFLG||CHR(10)
		||', P_ISDEFAULTINTERNALPARTNER=>'||P_ISDEFAULTINTERNALPARTNER ||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_PARTNER
	SET
		PARTNERNAME=p_PARTNERNAME
		,      GLOBALPARTNERID=p_GLOBALPARTNERID
		,      PARENTPARTNERID=p_PARENTPARTNERID
		,      ENABLED=p_ENABLED
		,      PARTNERTYPEID=p_PARTNERTYPEID
		,      OLSID=p_OLSID
		,      LOCALE=p_LOCALE
		,      BILLINGID=p_BILLINGID
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      URI=p_URI
		,      CLASSIFICATIONID=p_CLASSIFICATIONID
		,      PARTNERLOGIN=p_PARTNERLOGIN
		,      DIRRCVACCESSFLG=p_DIRRCVACCESSFLG
	Where
		PARTNERID = p_PARTNERID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	
	
	if P_ISDEFAULTINTERNALPARTNER= 1 THEN
	Update PR_COMMUNITYMANAGERDEFAULT
	SET 
	partnerid=p_PARTNERID;
	end if;
	
	

	-- For deleted partners change businessid to unique string for possible reuse
	IF p_ENABLED = 2 THEN
	
		SELECT COUNT(1) INTO v_DGATEWAYID FROM PR_GATEWAYDEFAULT 
				WHERE partnerid=p_partnerid;
						
		IF v_DGATEWAYID<>0 THEN 
				DELETE FROM PR_GATEWAYDEFAULT WHERE partnerid=p_partnerid;
				commit;
		END IF;
	
		FOR GATEWAYDELETE IN CUR_GATEWAY LOOP
			BEGIN
				
				PR_GATEWAYD_AH1 (p_returncd       => p_returncd,
									p_debugflg       => p_debugflg,
									p_gatewayid      => GATEWAYDELETE.GATEWAYID,
									p_partnerid      => p_partnerid
									);
				
			END;
		END LOOP;
		
		UPDATE	pr_busidentifier
		SET	BUSINESSID = TO_CHAR(BUSIDENTIFIERID) || TO_CHAR(SYS_EXTRACT_UTC(SYSTIMESTAMP))
			, ROWTS= p_RowTS + 1
		WHERE	PARTNERID = p_PARTNERID
		;
		UPDATE	pr_email
		SET	EMAILADDRESS = TO_CHAR(BUSINESSEMAILID) || TO_CHAR(SYS_EXTRACT_UTC(SYSTIMESTAMP))
			, ROWTS= p_RowTS + 1
		WHERE	PARTNERID = p_PARTNERID
		;
		UPDATE pr_User
		SET USERNAME=TO_CHAR(USERID) || TO_CHAR(SYS_EXTRACT_UTC(SYSTIMESTAMP))
			, ROWTS= p_RowTS + 1
		WHERE	PARTNERID = p_PARTNERID
		;
	END IF;

	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End PR_PARTNERU;
/


-- PR_PARTNER_PAIRI.sql

CREATE OR REPLACE PROCEDURE PR_PARTNER_PAIRI
	(
	p_ReturnCd OUT PLS_Integer
	,p_DebugFlg IN PLS_Integer
	,RC1 out db_ref.RCT
	,p_FrPartnerid IN NUMBER
	,p_ToPartnerId IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PARTNER_PAIRI
	Create Date:     2007-09-19 15:16:24
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-19 BHARANI J    6.1.1  Created for Multiple Certificate Feature

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	CREATES A PARTNER PAIR.  NULL IMPLIES 'ALL'
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/



v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_PARTNER_PAIRI';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_FrPartnerid=>'||p_FrPartnerid||CHR(10)
       ||', p_ToPartnerId=>'||p_ToPartnerId||CHR(10)
       ;
v_RowTS        integer :=0;

v_partnerpairid   PR_PARTNER_PAIR.partnerpairid%Type DEFAULT NULL;
v_insert          Varchar2(1);


BEGIN
	BEGIN
	
	SELECT partnerpairid into v_partnerpairid from PR_PARTNER_PAIR 
                where COALESCE(FRPARTNERID,-1) = COALESCE(p_FrPartnerId,-1)
                and COALESCE(TOPARTNERID,-1) = COALESCE(p_ToPartnerId,-1)
	; 
	
	EXCEPTION
	WHEN NO_DATA_FOUND THEN
	NULL;	
	END;
	               
	IF v_partnerpairid IS NULL THEN

		Insert into PR_PARTNER_PAIR
			(
			PARTNERPAIRID	
			,FRPARTNERID	
			,TOPARTNERID			
			) 
			Values 	
			(
			PR_PARTNER_PAIR_SEQ.nextval
			,p_FrPartnerId
			,p_ToPartnerId
			)RETURNING PARTNERPAIRID INTO v_partnerpairid 
		;
			
		v_rowts := 1;

	ELSE
		v_rowts := 0;
	
	END IF;	
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;		        
	Open RC1 FOR SELECT 	v_partnerpairid PARTNERPAIRID, 
				v_rowts RowTS 
		FROM DUAL;
		
	p_ReturnCd := 0;		
	
EXCEPTION

	WHEN OTHERS then
		--ROLLBACK work;
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PARTNER_PAIRI:'||SQLERRM);
			    	   
	
END PR_PARTNER_PAIRI;
/

		        

-- PR_PARTNER_PAIRS_AH1.sql

CREATE OR REPLACE Procedure PR_PARTNER_PAIRS_AH1
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_FrPartnerId		in		INTEGER
	, p_ToPartnerId		in		INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       PR_PARTNER_PAIRS_AH1
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PARTNER_PAIRS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FrPartnerId=>'||p_FrPartnerId||CHR(10)
		||', p_ToPartnerId=>'||p_ToPartnerId||CHR(10)
		;


BEGIN
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	OPEN RC1 FOR
		SELECT PARTNERPAIRID  
		FROM PR_PARTNER_PAIR 
		where COALESCE(FRPARTNERID,-1) = COALESCE(p_FrPartnerId,-1)
		and COALESCE(TOPARTNERID,-1) = COALESCE(p_ToPartnerId,-1)
		;
		
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_PARTNER_PAIRS_AH1' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PARTNER_PAIRS_AH1' || SQLCODE|| ' '||SQLERRM);
END PR_PARTNER_PAIRS_AH1;
/

-- PR_PARTNER_PAIRS_AH2.sql

CREATE OR REPLACE Procedure PR_PARTNER_PAIRS_AH2
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, p_FrPartnerId		in		INTEGER
	, p_ToPartnerId		in		INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       PR_PARTNER_PAIRS_AH2
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PARTNER_PAIRS_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FrPartnerId=>'||p_FrPartnerId||CHR(10)
		||', p_ToPartnerId=>'||p_ToPartnerId||CHR(10)
		;
sqlstmt      varchar2(1000);
sql_where    varchar2(100);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
        
        sql_where := ' Where 1 =1 '||	    
	FN_Build_Where('FRPARTNERID',   'NUMBER', p_FrPartnerId)||
	FN_Build_Where('TOPARTNERID',   'NUMBER', p_ToPartnerid);
         
        sqlstmt := 'SELECT PARTNERPAIRID,
			FRPARTNERID,
			TOPARTNERID
			FROM
			PR_PARTNER_PAIR ' ;
	
	sqlstmt := sqlstmt || sql_where ;
	
	Open RC1 for  sqlstmt;
	
	p_ReturnCd := 0;
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_PARTNER_PAIRS_AH2' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PARTNER_PAIRS_AH2' || SQLCODE|| ' '||SQLERRM);
END PR_PARTNER_PAIRS_AH2;
/

-- PR_PERMISSIOND_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_PERMISSIOND_AH1
(
   p_ReturnCd out INTEGER
   ,p_DebugFlg in PLS_INTEGER
   ,p_PERMISSIONID in NUMBER
)
AS

/*
    ================================================================================
    Proc Name:       PR_PERMISSIOND_AH1
    Create Date:     2003-04-29 10:56:59
    Author:          L.Ling

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-29 L.Ling        3.5    Created.
    2003-09-23 M.Mount       4.2.1  Removed Commit.
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_PERMISSIOND_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_PERMISSIONID=>'||p_PERMISSIONID||CHR(10)
        ;
/******
PROCEDURE PR_PERMISSION_LOCD AS
BEGIN
    Delete From PR_PERMISSION_LOC
    Where PERMISSIONID = p_PERMISSIONID;

    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    Exception
        When NO_DATA_FOUND Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND when deleting PR_PERMISSION_LOC,'||v_ParamsTxt, p_DebugFlg, 1);
   --         Commit;
            RAISE_APPLICATION_ERROR (-20999,'20002 PR_PERMISSION_LOCD when deleting PR_PERMISSION_LOC: No Data Found');
        When OTHERS Then
   --         Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
  --          Commit;
            RAISE_APPLICATION_ERROR (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
END PR_PERMISSION_LOCD;
*******/

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   p_ReturnCd := 0;
---   PR_PERMISSION_LOCD;

   BEGIN
       Delete From PR_PERMISSION
       Where PERMISSIONID = p_PERMISSIONID;

       DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

       Exception
        When NO_DATA_FOUND Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND when deleting PR_PERMISSION,'||v_ParamsTxt, p_DebugFlg, 1);
   --         Commit;
            RAISE_APPLICATION_ERROR (-20999,'20002 PR_PERMISSIOND_AH1 when deleting PR_PERMISSION: No Data Found');
        When OTHERS Then
   --         Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
   --         Commit;
            RAISE_APPLICATION_ERROR (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
   END;
  -- COMMIT;
END PR_PERMISSIOND_AH1;
/


-- PR_PERMISSIONI_AH1.sql

CREATE OR REPLACE PROCEDURE PR_PERMISSIONI_AH1
(
p_ReturnCd out pls_integer
,p_DebugFlg in pls_integer
,RC1 out db_ref.RCT
,p_locale in Varchar2
,p_PERMISSIONNAME in VARCHAR2
,p_PERMISSIONDESC in VARCHAR2
,p_ENABLEDFLG in NUMBER
) as
/*
================================================================================
Proc Name:      PR_PERMISSIONI_AH1
Create Date:    12/19/01 04:16:23 PM
    Insert record Into PR_Permission and PR_Permission_Loc table, one exists in the
    PR_permission table then insert into PR_Permission_Loc table
Author          SYSTEM
::::::::::::::::::::::::::::::
M O D I F I C A T I O N
::::::::::::::::::::::::::::::
---------  --------------------  ---------------------------------------------
DATE       NAME                  DESCRIPTION
---------  --------------------  ---------------------------------------------
12/19/01   SYSTEM                Created
2003-04-17 V.Goel       3.5      Globalization Effort
2003-07-01 M.Mount      3.5      Changed input paramters to standard.
2003-09-23 M.Mount      4.2.1    Removed rollback statement
2004-03-05 S.Priani     4.2.2    Removed NLS
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
--
v_PERMISSIONID pls_integer;

v_ExceptionFlg pls_integer;
v_ProcName      Varchar2(30) := 'PR_PERMISSIONI_AH1';
v_ParamsTxt     Varchar2(4000):= 'PR_PERMISSIONI_AH1 '||','||' p_ReturnCd '
||','||p_DebugFlg
||','||p_locale
||','||P_PERMISSIONNAME
||','||P_PERMISSIONDESC
||','||p_ENABLEDFLG
||','||'RC1'
;
--
-- Procedures
--
/*
Procedure ProcAudit is
v_diffTS Timestamp;
begin
If p_DebugFlg ='Y' Then
db_general_Pkg.Audit_Log('PR_PERMISSIONI_AH1',v_beginTS,'p_ReturnCd'
||','||p_DebugFlg
||','||p_locale
||','||P_PERMISSIONNAME
||','||P_PERMISSIONDESC
||','||p_ENABLEDFLG
||','||'RC1');
End if ;
End ProcAudit;
*/
--
--
--
Procedure Get_PermissionID is
Begin
    Select PermissionID into v_PermissionID
    From PR_Permission
    Where PERMISSIONNAME = p_PERMISSIONNAME;
End Get_PermissionID;
--                                        M  A  I  N
BEGIN
Begin
    Insert into pr_PERMISSION(
    PERMISSIONID
    ,   PERMISSIONNAME
    ,   ENABLEDFLG
    ,   DESCRIPTION
    ) Values (
    PR_PERMISSION_SEQ.nextval
    ,   p_PERMISSIONNAME
    ,   p_ENABLEDFLG
    ,   p_PERMISSIONDESC 
    ) returning PERMISSIONID INTO v_PERMISSIONID;
    v_rowcnt := sql%ROWCOUNT;
---    Permission_LocI;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 0);
 --   Commit;
--    ProcAudit;
--    vc_dba_execution_time.more_than_nn_sec('PR_PERMISSIONI_AH1', v_beginTime);
    EXCEPTION
        When NO_DATA_FOUND Then Null;
        When DUP_VAL_ON_INDEX then
            Get_PermissionID;
        When Others Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
--            p_SQLERRM := SQLERRM;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR (-20999,'20000 PR_PERMISSION_LOCI' || SQLCODE|| ''||SQLERRM);
End;
    If RC1%IsOpen Then Close RC1; End If;
    Open RC1 FOR Select v_PERMISSIONID  Id,1 ROWTS,1 Loc_RowTS From Dual;
END PR_PERMISSIONI_AH1;
/


-- PR_PERMISSIONS_AH2.sql

CREATE OR REPLACE PROCEDURE         PR_PERMISSIONS_AH2 (
    p_ReturnCd in out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_Locale in Varchar2
    , p_PERMISSIONNAME in Varchar2
    ) As
/*
================================================================================
Proc Name:      PR_PERMISSIONS_AH2
Create Date:    12/20/01 02:19:48 PM
Author          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-03-05 S.Priani      4.2.2  Removed NLS
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
v_diffTS Timestamp;
begin
    DB_ProcAuditLogI_Ah1('PR_PERMISSIONS_AH2',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||','||
    P_PERMISSIONNAME||','||'RC1');
End ProcAudit;
BEGIN
   Open RC1 for select
   PERMISSIONID
   ,   RTRIM(PERMISSIONNAME)  PERMISSIONNAME
   ,   RTRIM(RESOURCEKEY)  RESOURCEKEY
   ,   ENABLEDFLG
   ,   ROWTS
   ,   CREATEDATE
   ,   DESCRIPTION
   From   PR_PERMISSION
   Where  PERMISSIONName Like nvl(p_PERMISSIONNAME, '%')
   order by permissionname
   ;
   p_ReturnCd := 0;
   procAudit;
end  PR_PERMISSIONS_AH2;
/


-- PR_PERMISSIONS_BYGROUP.sql

CREATE OR REPLACE PROCEDURE         PR_PERMISSIONS_BYGROUP (
    p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    , p_Locale in Varchar2
    , p_GroupID in NUMBER
    ) As
/*
================================================================================
Proc Name:      pr_Permissions_ByGroup
Create Date:    12/20/01 02:19:48 PM
Author          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2004-03-05 S.Priani      4.2.2  Removed NLS
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
v_diffTS Timestamp;
BEGIN
DB_ProcAuditLogI_Ah1('pr_PERMISSIONs_ByGroup',v_beginTS,'p_ReturnCd'||','||p_DebugFlg||','||P_GroupID||','||'RC1');
End ProcAudit;
BEGIN
   Open RC1 for select
   p.PERMISSIONID
   ,   RTRIM(PERMISSIONNAME)  PERMISSIONNAME
   ,   RTRIM(RESOURCEKEY)  RESOURCEKEY
   ,   DESCRIPTION
   ,   pg.ACCESSTYPE
   From pr_PERMISSION p
   Inner Join PR_PERMISSION_GROUP pg on p.PERMISSIONID= pg.PERMISSIONID
   Where pg.groupId = p_groupID
   ;
   p_ReturnCd := 0;
   procAudit;
END  PR_PERMISSIONS_BYGROUP;
/


-- PR_PERMISSIONU_AH1.sql

CREATE OR REPLACE PROCEDURE PR_PERMISSIONU_AH1
(
   p_ReturnCd out INTEGER
   ,p_DebugFlg in PLS_INTEGER
   ,RC1 out db_ref.RCT
   ,p_LOCALE in VARCHAR2
   ,p_PERMISSIONID in NUMBER
   ,p_PERMISSIONNAME in VARCHAR2
   ,p_ENABLEDFLG in NUMBER
   ,p_ROWTS in NUMBER
   ,p_PERMISSIONDESC in VARCHAR2
   ,p_LOC_ROWTS in NUMBER
)
AS

/*
    ================================================================================
    Proc Name:       PR_PERMISSIONU_AH1
    Create Date:     2003-05-01 09:43:31
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-02 L.Ling        3.5    Created.
    2003-09-23 M.Mount       4.2.1  Removed Commit and rollback
    2004-03-05 S.Priani      4.2.2  Removed NLS

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_LocRowCnt       integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_PERMISSIONU_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_PERMISSIONID=>'||p_PERMISSIONID||CHR(10)
       ||', p_PERMISSIONNAME=>'||p_PERMISSIONNAME||CHR(10)
       ||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
       ||', p_ROWTS=>'||p_ROWTS||CHR(10)
       ||', p_LOCALE=>'||p_LOCALE||CHR(10)
       ||', p_PERMISSIONDESC=>'||p_PERMISSIONDESC||CHR(10)
       ||', p_LOC_ROWTS=>'||p_ROWTS||CHR(10)
        ;

v_RowTS        integer :=0;
v_LocRowTS     integer :=0;
zeroUpdate     Exception;
v_PR_PERMISSION_LOC_ROWTS  integer :=0;
v_PR_PERMISSION_ROWID      rowid;
v_PR_PERMISSION_LOC_ROWID  rowid;
v_PR_PERMISSION_ROWTS  integer :=0;
v_PERMISSIONNAME   PR_PERMISSION.PERMISSIONNAME%Type;
v_ENABLEDFLG       PR_PERMISSION.ENABLEDFLG%Type;
v_PERMISSIONDESC   PR_PERMISSION_LOC.PERMISSIONDESC%Type;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
   p_ReturnCd := 0;
    v_RowCnt := 0;

   SELECT  ROWID, ROWTS, PERMISSIONNAME, ENABLEDFLG
   INTO    v_PR_PERMISSION_ROWID, v_PR_PERMISSION_ROWTS, v_PERMISSIONNAME, v_ENABLEDFLG
   FROM    PR_PERMISSION
   WHERE   PERMISSIONID = p_PERMISSIONID
   FOR UPDATE; -- row level locking


   IF NVL(v_PERMISSIONNAME, ' ') <> NVL(p_PERMISSIONNAME, ' ')  OR
      NVL(v_ENABLEDFLG, -1)   <> NVL(p_ENABLEDFLG, -1) THEN -- data changes

       IF NVL(v_PR_PERMISSION_ROWTS, -1) <> NVL(p_ROWTS, -1) THEN
   --        ROLLBACK;
           p_ReturnCd := 1;
           v_ExceptionFlg := 1;
           DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'ROWTS Changes in PR_PERMISSION,'|| v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   --        COMMIT;
           RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PERMISSIONU_AH1 in PR_PERMISSION: ROWTS Changes');
       END IF;

       BEGIN
            UPDATE PR_PERMISSION
            SET
                PERMISSIONNAME=p_PERMISSIONNAME
                ,      ENABLEDFLG=p_ENABLEDFLG
                ,      DESCRIPTION=p_PERMISSIONDESC
                ,      ROWTS= p_RowTS + 1
                ,      UPDATEDATE= FN_GET_DATE()
            WHERE   ROWID = v_PR_PERMISSION_ROWID returning RowTS into v_RowTS;

            v_RowCnt := SQL%rowcount;
            if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
            p_ReturnCd := 0;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

           EXCEPTION
                WHEN TOO_MANY_ROWS Then
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   --                 Commit;
                    RAISE_APPLICATION_ERROR(-20999, '20100 - PR_PERMISSIONU_AH1: Too Many Rows');
                WHEN ZeroUpdate then
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
   --                 Commit;
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PERMISSIONU_AH1: No Data Found');
                WHEN OTHERS then
   --                 ROLLBACK work;
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
    --                Commit;
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PERMISSIONU_AH1:'||SQLERRM);
       END;
   END IF;

---   PR_PERMISSION_LOCU;

   IF RC1%isOpen THEN Close RC1; END IF;
   OPEN RC1 FOR
   SELECT v_RowTS As RowTS
   FROM Dual;
End PR_PERMISSIONU_AH1;
/


-- PR_PERMISSION_GROUPD.sql

CREATE OR REPLACE Procedure PR_PERMISSION_GROUPD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_PERMISSIONID in NUMBER
	,p_GROUPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PERMISSION_GROUPD
	Create Date:     2003-09-26 08:24:22
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PERMISSION_GROUPD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PERMISSIONID=>'||p_PERMISSIONID||CHR(10)
		||', p_GROUPID=>'||p_GROUPID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_PERMISSION_GROUP
	Where
		PERMISSIONID = p_PERMISSIONID
		AND GROUPID = p_GROUPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PERMISSION_GROUPD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_PERMISSION_GROUPD;
/


-- PR_PERMISSION_GROUPD_AH1.sql

CREATE OR REPLACE PROCEDURE PR_PERMISSION_GROUPD_AH1 
(
p_ReturnCd out pls_integer
,p_DebugFlg in pls_integer
,p_groupid in number
) as
/*
	================================================================================
	Proc Name:       PR_PERMISSION_GROUPD_AH1
	Create Date:     2007-08-20 16:44:38
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-08-10 BHARANI J      6.1.1 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt     integer    :=0;
v_BeginTS    timestamp := FN_Get_Date; -- Performance related variables
v_rowTS      integer :=0;
zeroUpdate   Exception;
v_contactid     pr_user.contactid%type;
v_contact_rowTS pr_contact.rowts%type;


Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_PERMISSION_GROUPD_AH1',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_groupid
     );
End ProcAudit;
BEGIN
   
   
    delete from pr_permission_group
    Where groupid = p_groupid;
    
   
    p_ReturnCd := 0;
    
    
    
    
EXCEPTION
    WHEN TOO_MANY_ROWS then ProcAudit;
    raise_application_error(-20999, '20100 - PR_PERMISSION_GROUPD_AH1: Too Many Rows'||' groupid: '||p_groupid);
    WHEN OTHERS then ProcAudit;
    raise_application_error(-20999,'20102 - PR_PERMISSION_GROUPD_AH1:'||sqlerrm||' groupid: '||p_groupid);
    p_ReturnCd := 1;
End PR_PERMISSION_GROUPD_AH1;
/

-- PR_PERMISSION_GROUPI.sql

CREATE OR REPLACE Procedure PR_PERMISSION_GROUPI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PERMISSIONID in NUMBER
	,p_GROUPID in NUMBER
	,p_ACCESSTYPE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_PERMISSION_GROUPI
	Create Date:     2003-09-26 00:03:51
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PERMISSION_GROUPI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PERMISSIONID=>'||p_PERMISSIONID||CHR(10)
		||', p_GROUPID=>'||p_GROUPID||CHR(10)
		||', p_ACCESSTYPE=>'||p_ACCESSTYPE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_PERMISSION_GROUP(
		PERMISSIONID
		,GROUPID
		,ACCESSTYPE
	) Values (
		p_PERMISSIONID
		,p_GROUPID
		,p_ACCESSTYPE
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_PERMISSION_GROUPI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_PERMISSION_GROUPI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PERMISSION_GROUPI' || SQLCODE|| ' '||SQLERRM);
END PR_PERMISSION_GROUPI;
/


-- PR_PERMISSION_GROUPS.sql

CREATE OR REPLACE PROCEDURE PR_PERMISSION_GROUPS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PERMISSIONID IN NUMBER
	, p_GROUPID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PERMISSION_GROUPS
	Create Date:     2003-09-26 00:06:30
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PERMISSION_GROUPS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PERMISSIONID=>'||p_PERMISSIONID||CHR(10)
		||', p_GROUPID=>'||p_GROUPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PERMISSIONID IS NOT NULL
	 AND p_GROUPID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		PERMISSIONID
		,GROUPID
		,RTRIM(ACCESSTYPE) ACCESSTYPE
		,ROWTS
		,CREATEDATE
	FROM
		PR_PERMISSION_GROUP
	Where
		PERMISSIONID = p_PERMISSIONID
		AND GROUPID = p_GROUPID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		PERMISSIONID
		,GROUPID
		,RTRIM(ACCESSTYPE) ACCESSTYPE
		,ROWTS
		,CREATEDATE
	FROM
		PR_PERMISSION_GROUP
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_PERMISSION_GROUPS;
/


-- PR_PERMISSION_GROUPU.sql

CREATE OR REPLACE Procedure PR_PERMISSION_GROUPU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PERMISSIONID in NUMBER
	,p_GROUPID in NUMBER
	,p_ACCESSTYPE in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_PERMISSION_GROUPU
	Create Date:     2003-09-26 08:21:28
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PERMISSION_GROUPU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PERMISSIONID=>'||p_PERMISSIONID||CHR(10)
		||', p_GROUPID=>'||p_GROUPID||CHR(10)
		||', p_ACCESSTYPE=>'||p_ACCESSTYPE||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_PERMISSION_GROUP
	SET
		ACCESSTYPE=p_ACCESSTYPE
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		PERMISSIONID = p_PERMISSIONID
		AND GROUPID = p_GROUPID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_PERMISSION_GROUPU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PERMISSION_GROUPU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PERMISSION_GROUPU:'||SQLERRM);
End PR_PERMISSION_GROUPU;
/


-- PR_PERMISSION_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE PR_PERMISSION_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_PERMISSIONID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PERMISSION_LOC_VWS
	Create Date:     2003-09-26 00:10:29
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-03-05 S.Priani      4.2.2  Removed NLS

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PERMISSION_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PERMISSIONID=>'||p_PERMISSIONID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PERMISSIONID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		PERMISSIONID
		,RTRIM(PERMISSIONNAME) PERMISSIONNAME
		,ENABLEDFLG
		,ROWTS
		,CREATEDATE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
	FROM
		PR_PERMISSION
	Where
		PERMISSIONID = p_PERMISSIONID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		PERMISSIONID
		,RTRIM(PERMISSIONNAME) PERMISSIONNAME
		,ENABLEDFLG
		,ROWTS
		,CREATEDATE
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
	FROM
		PR_PERMISSION
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_PERMISSION_LOC_VWS;
/


-- PR_PROXYATTRD.sql

Create or Replace Procedure PR_PROXYATTRD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_TRANSPORTPROXYATTRID in NUMBER
	,p_PROXYID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PROXYATTRD
	Create Date:     2005-03-10 16:40:12
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PROXYATTRD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTPROXYATTRID=>'||p_TRANSPORTPROXYATTRID||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_PROXYATTR
	Where
		TRANSPORTPROXYATTRID = p_TRANSPORTPROXYATTRID
	AND	PROXYID = p_PROXYID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PROXYATTRD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_PROXYATTRD;
/


-- PR_PROXYATTRI.sql

Create or Replace Procedure PR_PROXYATTRI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROXYID in NUMBER
	,p_TRANSPORTPROXYATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_PROXYATTRI
	Create Date:     2005-03-10 16:40:13
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PROXYATTRI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTPROXYATTRID=>'||p_TRANSPORTPROXYATTRID||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_PROXYATTR(
		TRANSPORTPROXYATTRID
		,PROXYID
		,ATTRVALUE
	) Values (
		p_TRANSPORTPROXYATTRID
		,p_PROXYID
		,p_ATTRVALUE
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PROXYATTRI' || SQLCODE|| ' '||SQLERRM);
END PR_PROXYATTRI;
/


-- PR_PROXYATTRS.sql

CREATE OR REPLACE PROCEDURE PR_PROXYATTRS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROXYID IN NUMBER
	, p_TRANSPORTPROXYATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PROXYATTRS
	Create Date:     2005-03-10 16:40:14
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PROXYATTRS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTPROXYATTRID=>'||p_TRANSPORTPROXYATTRID||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
	SELECT
		pr.PROXYID
		,pr.TRANSPORTID
		,pta.TRANSPORTPROXYATTRID
		,pta.attrname
		,COALESCE(prat.attrvalue, pta.attrvalue) ATTRVALUE
		,pta.resourcekey
		,prat.rowts
	FROM
		PR_PROXY   pr
		INNER JOIN PR_TRANSPORT_PROXY_ATTR pta ON pr.TRANSPORTID = pta.TRANSPORTID
		LEFT OUTER JOIN PR_PROXYATTR prat ON pr.PROXYID = prat.PROXYID AND pta.TRANSPORTPROXYATTRID = prat.TRANSPORTPROXYATTRID
	Where
		((pr.PROXYID = p_PROXYID AND p_PROXYID IS NOT NULL) OR (p_PROXYID IS NULL))
	AND	((pta.TRANSPORTPROXYATTRID = p_TRANSPORTPROXYATTRID AND p_TRANSPORTPROXYATTRID IS NOT NULL) OR (p_TRANSPORTPROXYATTRID IS NULL))
	;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_PROXYATTRS;
/

-- PR_PROXYATTRU.sql

Create or Replace Procedure PR_PROXYATTRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_TRANSPORTPROXYATTRID in NUMBER
	,p_PROXYID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_PROXYATTRU
	Create Date:     2005-03-10 16:40:16
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PROXYATTRU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTPROXYATTRID=>'||p_TRANSPORTPROXYATTRID||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_PROXYATTR
	SET
		ATTRVALUE=p_ATTRVALUE
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where
		TRANSPORTPROXYATTRID = p_TRANSPORTPROXYATTRID
		AND PROXYID = p_PROXYID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_PROXYATTRU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PROXYATTRU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PROXYATTRU:'||SQLERRM);
End PR_PROXYATTRU;
/


-- PR_PROXYD.sql

CREATE OR REPLACE Procedure PR_PROXYD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_PROXYID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PROXYD
	Create Date:     2005-03-10 16:40:11
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PROXYD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

	 update pr_gwyattr
	set attrvalue = 'Use default forward proxy'
	where transportattrid = (select transportattrid from pr_transport_attr where transportid = 
	(select transportid from pr_proxy where proxyid = p_proxyid)
	and attrname = 'bcg.fwdproxy.proxyName');
	
	Delete From CF_PROXY_GATEWAY_LNK cfgl
	Where
		cfgl.PROXYID = p_PROXYID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt || ' - no linked Gateways', p_DebugFlg, v_ExceptionFlg);
	end if;

	Delete From PR_PROXYATTR pa
	Where
		pa.PROXYID = p_PROXYID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt || ' - no linked Attributes', p_DebugFlg, v_ExceptionFlg);
	end if;

	Delete From PR_PROXY
	Where
		PROXYID = p_PROXYID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	
	
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PROXYD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_PROXYD;
/

-- PR_PROXYI.sql

CREATE OR REPLACE Procedure PR_PROXYI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_TRANSPORTID in NUMBER
	,p_PROXYNAME in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_DEFAULTFLG in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PROXYI
	Create Date:     2005-03-10 16:40:12
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PROXYI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_PROXYNAME=>'||p_PROXYNAME||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		||', p_DEFAULTFLG=>'||p_DEFAULTFLG||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF p_DEFAULTFLG = 1 THEN
		update pr_proxy
		set defaultflg = 0
		where transportid = p_transportid
		and defaultflg = 1;
	END IF;
	Insert into PR_PROXY(
		PROXYID
		,TRANSPORTID
		,PROXYNAME
		,DESCRIPTION
		,DEFAULTFLG
	) Values (
		PR_PROXY_SEQ.nextval
		,p_TRANSPORTID
		,p_PROXYNAME
		,p_DESCRIPTION
		--,v_PK_Value
		,p_DEFAULTFLG
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_PROXY_SEQ.currval PROXYID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PROXYI' || SQLCODE|| ' '||SQLERRM);
END PR_PROXYI;
/

-- PR_PROXYS.sql

CREATE OR REPLACE PROCEDURE PR_PROXYS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROXYID IN NUMBER
	, p_TRANSPORTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PROXYS
	Create Date:     2005-03-10 16:40:13
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PROXYS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
	SELECT
		pr.PROXYID
		,pr.TRANSPORTID
		,RTRIM(pr.PROXYNAME) PROXYNAME
		,RTRIM(pr.DESCRIPTION) DESCRIPTION
		,pr.DEFAULTFLG
		,pr.ROWTS
		,pr.CREATEDATE
		,pr.UPDATEDATE
	FROM
		PR_PROXY   pr
	Where
		((pr.PROXYID = p_PROXYID AND p_PROXYID IS NOT NULL) OR (p_PROXYID IS NULL))
	AND	((pr.TRANSPORTID = p_TRANSPORTID AND p_TRANSPORTID IS NOT NULL) OR (p_TRANSPORTID IS NULL))
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_PROXYS;
/


-- PR_PROXYS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_PROXYS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_TRANSPORTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PROXYS
	Create Date:     2005-03-10 16:40:13
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PROXYS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR
	SELECT
		pr.PROXYID
		,pr.TRANSPORTID
		,pt.TRANSPORTTYPE
		,RTRIM(pr.PROXYNAME) PROXYNAME
		,RTRIM(pr.DESCRIPTION) DESCRIPTION
		,pr.ROWTS
		,pr.CREATEDATE
		,pr.UPDATEDATE
	FROM
		PR_PROXY   pr
		INNER JOIN PR_TRANSPORT pt ON pt.TRANSPORTID = pr.TRANSPORTID
	Where
		pt.TRANSPORTID = p_TRANSPORTID
	AND	pr.DEFAULTFLG = 1
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_PROXYS_AH1;
/


-- PR_PROXYU.sql

CREATE OR REPLACE Procedure PR_PROXYU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_PROXYID in NUMBER
	,p_TRANSPORTID in NUMBER
	,p_PROXYNAME in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_DEFAULTFLG in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_PROXYU
	Create Date:     2005-03-10 16:40:15
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_PK_Value	NUMBER;
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PROXYU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROXYID=>'||p_PROXYID||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_PROXYNAME=>'||p_PROXYNAME||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		||', p_DEFAULTFLG=>'||p_DEFAULTFLG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF p_DEFAULTFLG = 1 THEN
		update pr_proxy
		set defaultflg = 0
		where transportid = p_transportid
		and defaultflg = 1;
	END IF;
	v_RowCnt := 0;
	Update PR_PROXY
	SET
		TRANSPORTID=p_TRANSPORTID
		,      PROXYNAME=p_PROXYNAME
		,      DESCRIPTION=p_DESCRIPTION
		--,      DEFAULTFLG=v_PK_Value
		,      DEFAULTFLG= p_defaultflg
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where
		PROXYID = p_PROXYID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_PROXYU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PROXYU: No Data Found');
		WHEN Dup_Val_On_Index then
			p_REturnCd := 2;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PROXYU: Dup Val On Index');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PROXYU:'||SQLERRM);
End PR_PROXYU;
/

-- PR_PWDHISTORYD_PWDUNIQUE.sql

CREATE OR REPLACE PROCEDURE         PR_PWDHISTORYd_pwdUnique
(p_ReturnCd in out integer, p_DebugFlg in pls_integer
,p_UserID in pls_integer
,p_PwdUnique in pls_integer
) As
/*
================================================================================
Proc Name:      PR_PWDHISTORYd_pwdUnique
Create Date:    01/24/02 10:02:20 AM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_PWDHISTORYd_pwdUnique',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_UserID
    ||','||p_PwdUnique
    );
End ProcAudit;
BEGIN
    Delete From PR_PWDHISTORY
    Where UserID = p_UserID
    and PwdID in  (select F.pwdid from
                   (select rownum rnbr, E.pwdid from
                       (select pwdid from pr_pwdHistory order by pwdid desc) E )F
                where rnbr > p_PwdUnique)
    ;
    v_rowcnt := SQL%ROWCOUNT;
    ProcAudit;
    p_ReturnCd := 0;
Exception
    When NO_DATA_FOUND Then ProcAudit;
    Raise_application_error (-20999,'20002 PR_PWDHISTORYd: No Data Found');
    When Others Then ProcAudit;
    Raise_application_error (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
    p_ReturnCd := 1;
End PR_PWDHISTORYd_pwdUnique;
/


-- PR_PWDHISTORYI.sql

CREATE OR REPLACE Procedure PR_PWDHISTORYI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_USERID in NUMBER
	,p_PASSWORD in RAW
	)
AS

/*
	================================================================================
	Proc Name:       PR_PWDHISTORYI
	Create Date:     2003-09-26 00:01:44
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PWDHISTORYI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_USERID=>'||p_USERID||CHR(10)
		||', p_PASSWORD=>'||p_PASSWORD||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_PWDHISTORY(
		PWDID
		,USERID
		,PASSWORD
	) Values (
		PR_PWDHISTORY_SEQ.nextval
		,p_USERID
		,p_PASSWORD
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_PWDHISTORY_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_PWDHISTORYI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_PWDHISTORYI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PWDHISTORYI' || SQLCODE|| ' '||SQLERRM);
END PR_PWDHISTORYI;
/


-- PR_PWDHISTORYS.sql

CREATE OR REPLACE PROCEDURE PR_PWDHISTORYS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PWDID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PWDHISTORYS
	Create Date:     2003-09-26 00:06:28
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PWDHISTORYS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PWDID=>'||p_PWDID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_PWDID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		PWDID
		,USERID
		,PASSWORD
		,ROWTS
		,CREATEDATE
	FROM
		PR_PWDHISTORY
	Where
		PWDID = p_PWDID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		PWDID
		,USERID
		,PASSWORD
		,ROWTS
		,CREATEDATE
	FROM
		PR_PWDHISTORY
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_PWDHISTORYS;
/


-- PR_PWDHISTORYS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_PWDHISTORYs_AH1
(
    p_ReturnCd in out integer
    , p_DebugFlg in pls_integer
    , RC1 in out db_ref.RCT
    , p_USERID in NUMBER
    , p_PASSWORD in RAW
) As
/*
================================================================================
Proc Name:      PR_PWDHISTORYs_AH1
Create Date:    01/24/02 10:03:34 AM
Author          S.HUTCHINSON
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_PWDHISTORYs_AH1',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_USERID
    ||','||p_PASSWORD
    );
End ProcAudit;
BEGIN
Open RC1 for

    SELECT
        PWDID,
        USERID,
        PASSWORD,
        ROWTS,
        CREATEDATE
    FROM
        PR_PWDHISTORY
    WHERE
        USERID = p_USERID
        AND PASSWORD = p_PASSWORD;

p_ReturnCd := 0;
procAudit;
End PR_PWDHISTORYs_AH1;
/


-- PR_PWDHISTORYS_USERID.sql

CREATE OR REPLACE PROCEDURE         PR_PWDHISTORYs_USERID
(
    p_ReturnCd in out integer
    , p_DebugFlg in pls_integer
    , RC1 in out db_ref.RCT
    , p_USERID in NUMBER
) As
/*
================================================================================
Proc Name:      PR_PWDHISTORYs_USERID
Create Date:    01/24/02 10:03:34 AM
Author          S.HUTCHINSON
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_PWDHISTORYs_USERID',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_USERID
    );
End ProcAudit;
BEGIN
Open RC1 for
    SELECT
        PWDID
        , USERID
        , PASSWORD
        , ROWTS
        , CREATEDATE
    FROM
        PR_PWDHISTORY
    WHERE
        USERID = p_USERID
;
p_ReturnCd := 0;
procAudit;
End PR_PWDHISTORYs_USERID;
/


-- PR_PWDPOLICYI.sql

CREATE OR REPLACE Procedure PR_PWDPOLICYI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PARTNERID in NUMBER
	,p_PWDMINLENGTH in NUMBER
	,p_PWDEXPIREDAYS in NUMBER
	,p_PWDUNIQUE in NUMBER
	,p_PWDSPECIALCHARS in NUMBER
	,p_PWDNAMECHECK in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PWDPOLICYI
	Create Date:     2003-09-26 00:01:45
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PWDPOLICYI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_PWDMINLENGTH=>'||p_PWDMINLENGTH||CHR(10)
		||', p_PWDEXPIREDAYS=>'||p_PWDEXPIREDAYS||CHR(10)
		||', p_PWDUNIQUE=>'||p_PWDUNIQUE||CHR(10)
		||', p_PWDSPECIALCHARS=>'||p_PWDSPECIALCHARS||CHR(10)
		||', p_PWDNAMECHECK=>'||p_PWDNAMECHECK||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_PWDPOLICY(
		POLICYID
		,PARTNERID
		,PWDMINLENGTH
		,PWDEXPIREDAYS
		,PWDUNIQUE
		,PWDSPECIALCHARS
		,PWDNAMECHECK
	) Values (
		PR_PWDPOLICY_SEQ.nextval
		,p_PARTNERID
		,p_PWDMINLENGTH
		,p_PWDEXPIREDAYS
		,p_PWDUNIQUE
		,p_PWDSPECIALCHARS
		,p_PWDNAMECHECK
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_PWDPOLICY_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_PWDPOLICYI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_PWDPOLICYI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_PWDPOLICYI' || SQLCODE|| ' '||SQLERRM);
END PR_PWDPOLICYI;
/


-- PR_PWDPOLICYS.sql

CREATE OR REPLACE PROCEDURE PR_PWDPOLICYS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_POLICYID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_PWDPOLICYS
	Create Date:     2003-09-26 00:06:26
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PWDPOLICYS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_POLICYID=>'||p_POLICYID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_POLICYID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		POLICYID
		,PARTNERID
		,PWDMINLENGTH
		,PWDEXPIREDAYS
		,PWDUNIQUE
		,PWDSPECIALCHARS
		,PWDNAMECHECK
		,ROWTS
		,CREATEDATE
	FROM
		PR_PWDPOLICY
	Where
		POLICYID = p_POLICYID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		POLICYID
		,PARTNERID
		,PWDMINLENGTH
		,PWDEXPIREDAYS
		,PWDUNIQUE
		,PWDSPECIALCHARS
		,PWDNAMECHECK
		,ROWTS
		,CREATEDATE
	FROM
		PR_PWDPOLICY
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_PWDPOLICYS;
/


-- PR_PWDPOLICYU.sql

CREATE OR REPLACE Procedure PR_PWDPOLICYU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_POLICYID in NUMBER
	,p_PARTNERID in NUMBER
	,p_PWDMINLENGTH in NUMBER
	,p_PWDEXPIREDAYS in NUMBER
	,p_PWDUNIQUE in NUMBER
	,p_PWDSPECIALCHARS in NUMBER
	,p_PWDNAMECHECK in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_PWDPOLICYU
	Create Date:     2003-09-26 08:21:29
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_PWDPOLICYU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_POLICYID=>'||p_POLICYID||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_PWDMINLENGTH=>'||p_PWDMINLENGTH||CHR(10)
		||', p_PWDEXPIREDAYS=>'||p_PWDEXPIREDAYS||CHR(10)
		||', p_PWDUNIQUE=>'||p_PWDUNIQUE||CHR(10)
		||', p_PWDSPECIALCHARS=>'||p_PWDSPECIALCHARS||CHR(10)
		||', p_PWDNAMECHECK=>'||p_PWDNAMECHECK||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_PWDPOLICY
	SET
		PARTNERID=p_PARTNERID
		,      PWDMINLENGTH=p_PWDMINLENGTH
		,      PWDEXPIREDAYS=p_PWDEXPIREDAYS
		,      PWDUNIQUE=p_PWDUNIQUE
		,      PWDSPECIALCHARS=p_PWDSPECIALCHARS
		,      PWDNAMECHECK=p_PWDNAMECHECK
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		POLICYID = p_POLICYID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_PWDPOLICYU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PWDPOLICYU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_PWDPOLICYU:'||SQLERRM);
End PR_PWDPOLICYU;
/


-- PR_RCVATTRD.sql

CREATE OR REPLACE Procedure PR_RCVATTRD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_RCVCONFIGID in NUMBER
	,p_TRANSPORTATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_RCVATTRD
	Create Date:     2004-03-13 22:42:27
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVATTRD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' WHERE 1=1 ' ;
    v_where := v_where || FN_BUILD_WHERE('RCVCONFIGID', 'NUMBER', p_RCVCONFIGID) ;
	v_where := v_where || FN_BUILD_WHERE('TRANSPORTATTRID', 'NUMBER', p_TRANSPORTATTRID) ;
	
    v_sqlstmt := 
	   'Delete From PR_RCVATTR ' || v_where ;
	   
	execute immediate v_sqlstmt ;
	
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_RCVATTRD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_RCVATTRD;
/


-- PR_RCVATTRI.sql

--------------------------------------------------------------------------------
CREATE OR REPLACE Procedure PR_RCVATTRI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT   
	,p_RCVCONFIGID in NUMBER
	,p_TRANSPORTATTRID in NUMBER
	,p_ATTRVALUE in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_RCVATTRI
	Create Date:     2004-03-13 22:44:28
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVATTRI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_RCVATTR(
		RCVCONFIGID
		,TRANSPORTATTRID
		,ATTRVALUE
	) Values (
		p_RCVCONFIGID
		,p_TRANSPORTATTRID
		,p_ATTRVALUE
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_RCVATTRI' || SQLCODE|| ' '||SQLERRM);
END PR_RCVATTRI;
/


-- PR_RCVATTRS.sql

CREATE OR REPLACE PROCEDURE PR_RCVATTRS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_RCVCONFIGID IN NUMBER
	, p_TRANSPORTATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_RCVATTRS
	Create Date:     2004-03-13 22:43:51
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVATTRS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;           
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)
		;
v_TransportID     Integer Default 0 ;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('RCVCONFIGID', 'NUMBER', p_RCVCONFIGID) ||
       FN_Build_Where('RA.TRANSPORTATTRID', 'NUMBER', p_TRANSPORTATTRID);
	
    if p_TRANSPORTATTRID is not null then
       v_sqlstmt :=
       'SELECT '
	   ||'RCVCONFIGID'
	   ||',RA.TRANSPORTATTRID'
       ||',RTRIM(ATTRNAME) ATTRNAME'
       ||',RTRIM(ATTRVALUE) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',RA.ROWTS'
       ||',RA.CREATEDATE '
       ||'FROM '
       ||'PR_RCVATTR RA INNER JOIN PR_TRANSPORT_ATTR TA '
       ||'ON RA.TRANSPORTATTRID = TA.TRANSPORTATTRID '
       || v_where ;
    ELSE
       BEGIN

       SELECT NVL(transportid,0) INTO v_transportid FROM pr_rcvconfig WHERE rcvconfigid = p_RCVCONFIGID ;

       v_sqlstmt :=
       'SELECT '
       ||'rcvconfigid'
       ||',NVL(ra.transportattrid, ta.transportattrid) transportattrid '
       ||',RTRIM(attrname) ATTRNAME'
       ||',RTRIM(NVL(attrvalue, defaultattrvalue)) ATTRVALUE'
       ||',RTRIM(RESOURCEKEY) RESOURCEKEY'
       ||',RTRIM(DESCRIPTION) DESCRIPTION'
       ||',ra.ROWTS'
       ||',ra.CREATEDATE '
       ||'from '
       ||'(select rcvconfigid, transportattrid, attrvalue, rowts, createdate from pr_rcvattr where rcvconfigid = ' || p_RCVCONFIGID ||') ra '
       ||'right outer join pr_transport_attr ta '
       ||'on ra.transportattrid = ta.transportattrid '
       ||'where ta.transportid = ' || v_transportid
       ;
       END;
    end if ;     
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_RCVATTRS;
/


-- PR_RCVATTRU.sql

CREATE OR REPLACE Procedure PR_RCVATTRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_RCVCONFIGID in NUMBER
	,p_TRANSPORTATTRID in NUMBER
	,p_ROWTS in NUMBER
	,p_ATTRVALUE in VARCHAR2
) as

/*
	================================================================================
	Proc Name:       PR_RCVATTRU
	Create Date:     2004-03-13 22:43:05
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVATTRU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_RCVATTR
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      ATTRVALUE=p_ATTRVALUE
	Where 
		RCVCONFIGID = p_RCVCONFIGID
		AND TRANSPORTATTRID = p_TRANSPORTATTRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_RCVATTRU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVATTRU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVATTRU:'||SQLERRM);
End PR_RCVATTRU;
/


-- PR_RCVCONFIGD.sql

CREATE OR REPLACE Procedure PR_RCVCONFIGD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_RCVCONFIGID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_RCVCONFIGD
	Create Date:     2006-08-29 16:40:11
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2006-08-29 S.Nair        6.1.0  Created
	2007-07-31 Bharani J     6.1.1  Modified for Feature F104473

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVCONFIGD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		;

--zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	
	Delete From CF_RCVHANDLER_XREF crx
	Where
		crx.RCVCONFIGID = p_RCVCONFIGID
	;
	--v_RowCnt := SQL%ROWCOUNT;
	--if (v_RowCnt = 0) then  raise ZeroDelete ; end if;

        Delete From PR_RCVCONFIGdtl a
			Where a.RCVCONFIGID = p_RCVCONFIGID
	;
	
	Delete From PR_RCVCONFIG pr
	Where
		pr.RCVCONFIGID = p_RCVCONFIGID
	;
	
	
	--v_RowCnt := SQL%ROWCOUNT;
	--if (v_RowCnt = 0) then  raise ZeroDelete ; end if;	
	
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			--RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_RCVCONFIGD;
/


-- PR_RCVCONFIGDTLI.sql

 CREATE OR REPLACE Procedure PR_RCVCONFIGDTLI
  	(
  	p_ReturnCd out pls_integer
  	,p_DebugFlg in pls_integer
  	,RC1 out db_ref.RCT
  	,p_RCVCONFIGID in NUMBER
  	,p_MAXSYNCTIMEOUT in NUMBER
  	,p_MAXSYNCSIMULCONNECTION in NUMBER
  	,P_BASICAUTHFLAG in NUMBER  --   should be '0' or '1'
  	)
  AS
  
  /*
  	================================================================================
  	Proc Name:       PR_RCVCONFIGDTLI
  	Create Date:     2003-09-26 00:01:45
  	Author:          SYSTEM
  
  	                   ::::::::::::::::::::::::::::::
  	                         M O D I F I C A T I O N
  	                   ::::::::::::::::::::::::::::::
  
  	---------- ------------- ------ ---------------------------------------------
  	DATE       NAME          Ver    DESCRIPTION
  	---------- ------------- ------ ---------------------------------------------
  	2003-09-26 SYSTEM        3.5    Created
  	2007-07-31 Bharani J     6.1.1  Modified for Feature F104473
  	2007-08-24 Bharani J     6.1.1  Modified for Feature F104489
  
  	================================================================================
  	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
 */
  
  v_PK_Value        number := 0;
  
  v_RowCnt          integer Default 0;
  v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
  v_ProcName        varchar2(30) Default 'PR_RCVCONFIGDTLI';
  v_ExceptionFlg    integer Default 0;
  v_ParamsTxt       varchar2(4000) Default
  		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
  		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
  		||', p_MAXSYNCTIMEOUT=>'||p_MAXSYNCTIMEOUT||CHR(10)
  		||', p_MAXSYNCSIMULCONNECTION=>'||p_MAXSYNCSIMULCONNECTION||CHR(10)
  		||', P_BASICAUTHFLAG=>'||P_BASICAUTHFLAG||CHR(10)
  		;
  
  ---------------------------------------------------------------------------
  --                         M    A    I    N                              --
  ---------------------------------------------------------------------------
  
  BEGIN
  	Insert into PR_RCVCONFIGDTL(
  		RCVCONFIGDTLID
  		,rcvconfigid
  		,MAXSYNCTIMEOUT
  		,MAXSYNCSIMULCONNECTION
  		,BASICAUTHFLAG
  	) Values (
  		PR_RCVCONFIGDTL_SEQ.nextval
  		,p_RCVCONFIGID
  		,p_MAXSYNCTIMEOUT
  		,p_MAXSYNCSIMULCONNECTION
  		,P_BASICAUTHFLAG
  	);
  	p_ReturnCd := 0;
  	v_RowCnt := sql%ROWCOUNT;
  	IF RC1%isOpen THEN Close RC1; END IF;
  	Open RC1 FOR Select
  		PR_RCVCONFIGDTL_SEQ.currval ID, 1 RowTS From Dual;
  	p_ReturnCd := 0;
  	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  	EXCEPTION
  		When NO_DATA_FOUND THEN
  			v_ExceptionFlg := 1;
  			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
  			RAISE_APPLICATION_ERROR (-20999,'20000 PR_RCVCONFIGDTLI' || SQLCODE|| ' '||SQLERRM);
  		When DUP_VAL_ON_INDEX then
  			v_ExceptionFlg := 1;
  			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  			p_returnCd := 2;
  			RAISE_APPLICATION_ERROR (-20999,'20001 PR_RCVCONFIGDTLI' || SQLCODE|| ' '||SQLERRM);
  		When OTHERS Then
  			p_ReturnCd := 1;
  			v_ExceptionFlg := 1;
  			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
  			RAISE_APPLICATION_ERROR (-20999,'20002 PR_RCVCONFIGDTLI' || SQLCODE|| ' '||SQLERRM);
  END PR_RCVCONFIGDTLI;
/

-- PR_RCVCONFIGDTLS.sql

CREATE OR REPLACE PROCEDURE PR_RCVCONFIGDTLS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_RCVCONFIGDTLID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_RCVCONFIGDTLS
	Create Date:     2003-09-26 00:06:25
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2007-07-31 Bharani J     6.1.1  Modified for Feature F104473
	2007-08-24 Bharani J     6.1.1  Modified for Feature F104489

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVCONFIGDTLS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGDTLID=>'||p_RCVCONFIGDTLID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_RCVCONFIGDTLID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		RCVCONFIGDTLID
		,RCVCONFIGID
		,MAXSYNCTIMEOUT
		,MAXSYNCSIMULCONNECTION
		,BASICAUTHFLAG
		,ROWTS
		,CREATEDATE
	FROM
		PR_RCVCONFIGDTL
	Where
		RCVCONFIGDTLID = p_RCVCONFIGDTLID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		RCVCONFIGDTLID
		,RCVCONFIGID
		,MAXSYNCTIMEOUT
		,MAXSYNCSIMULCONNECTION
		,BASICAUTHFLAG
		,ROWTS
		,CREATEDATE
	FROM
		PR_RCVCONFIGDTL
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_RCVCONFIGDTLS;
/

-- PR_RCVCONFIGDTLS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_RCVCONFIGDTLS_AH1
	(
	p_ReturnCd          OUT  PLS_INTEGER
	, p_DebugFlg        IN  PLS_INTEGER
	, RC1               OUT  DB_REF.RCT
	, p_TransportType   IN Varchar2
    , p_TransportID     IN PLS_INTEGER
	)
AS

/*
	================================================================================
	Proc Name:       PR_RCVCONFIGDTLS_ah1
	Create Date:     2003-07-11
	Author:          V.Goel
        This proc retrieves data from PR_RcvConfigDTL based upon TransportType

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVCONFIGDTLS_AH1';
v_Where           Varchar2 (1000);
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TransportType=>'||p_TransportType||CHR(10)
        ||', p_TransportID=>'||p_TransportID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_where := ' Where 1 = 1 '
                ||FN_Build_Where('TransportType', 'VARCHAR2', p_TransportType)
                ||FN_Build_Where('D.TransportID'  , 'NUMBER'  , p_TransportID);

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR
	'SELECT
		D.RCVCONFIGDTLID
        ,D.TransportID
		,T.TransportType
		,D.MAXSYNCTIMEOUT
		,D.MAXSYNCSIMULCONNECTION
	FROM
		PR_RCVCONFIGDTL D
        Inner Join PR_Transport T on D.TransportId = T.TransportId'||v_where ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_RCVCONFIGDTLS_ah1;
/


-- PR_RCVCONFIGDTLU.sql

CREATE OR REPLACE Procedure PR_RCVCONFIGDTLU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_RCVCONFIGDTLID in NUMBER
	,p_RCVCONFIGID in NUMBER
	,p_MAXSYNCTIMEOUT in NUMBER
	,p_MAXSYNCSIMULCONNECTION in NUMBER
	,p_BASICAUTHFLAG in NUMBER
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_RCVCONFIGDTLU
	Create Date:     2003-09-26 08:21:34
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2007-07-31 Bharani J     6.1.1  Modified for Feature F104473
	2007-08-24 Bharani J     6.1.1  Modified for Feature F104489
	

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVCONFIGDTLU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGDTLID=>'||p_RCVCONFIGDTLID||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_MAXSYNCTIMEOUT=>'||p_MAXSYNCTIMEOUT||CHR(10)
		||', p_MAXSYNCSIMULCONNECTION=>'||p_MAXSYNCSIMULCONNECTION||CHR(10)
		||', p_BASICAUTHFLAG=>'||p_BASICAUTHFLAG||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_RCVCONFIGDTL
	SET
		RCVCONFIGID=p_RCVCONFIGID
		,      MAXSYNCTIMEOUT=p_MAXSYNCTIMEOUT
		,      MAXSYNCSIMULCONNECTION=p_MAXSYNCSIMULCONNECTION
		,      BASICAUTHFLAG=P_BASICAUTHFLAG
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
	Where
		RCVCONFIGDTLID = p_RCVCONFIGDTLID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_RCVCONFIGDTLU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGDTLU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGDTLU:'||SQLERRM);
End PR_RCVCONFIGDTLU;
/


-- PR_RCVCONFIGI.SQL

CREATE OR REPLACE PROCEDURE PR_RCVCONFIGI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_RCVCONFIGNAME in VARCHAR2
	,p_RCVCONFIGDESC in VARCHAR2
	,p_PARTNERID in NUMBER
	,p_TRANSPORTID in NUMBER
	,p_DESTTYPEID in NUMBER
	,p_THREADNBR in NUMBER
	,p_URI in VARCHAR2
	,p_URILOGIN in VARCHAR2
	,p_URIPASSWORD in RAW
	,p_ACTIVESTATUSCD in NUMBER
	,p_JMSQUEUENAME in VARCHAR2
	,p_JMSFACTORYNAME in VARCHAR2
	,p_PROVIDERURLPKG in VARCHAR2
	,p_JNDIFACTORYNAME in VARCHAR2
	,p_DAYOFMONTH in VARCHAR2
	,p_DAYOFWEEK in VARCHAR2
	,p_HOUROPT in VARCHAR2
	,p_MINUTEOPT in VARCHAR2
	,p_PORTNBR in NUMBER
	,p_TIMEOUT_MS in NUMBER
	,p_FILEEXTFILTER in VARCHAR2
	,p_FREQDAYOFMONTH in NUMBER
	,p_FREQDAYOFWEEK in NUMBER
	,p_POLLINTERVAL in NUMBER
	,p_FILEUNCHNGINTERVAL in NUMBER
        ,p_MAXSYNCTIMEOUT in NUMBER
 	,p_MAXSYNCSIMULCONNECTION in NUMBER
 	,P_BASICAUTHFLAG in NUMBER  --   should be '0' or '1'
	)
AS

/*
	================================================================================
	Proc Name:       PR_RCVCONFIGI
	Create Date:     2003-09-26 00:01:45
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2007-07-31 Bharani J     6.1.1  Modified for Feature F104473
	2007-08-24 Bharani J     6.1.1  Modified for Feature F104489

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVCONFIGI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGNAME=>'||p_RCVCONFIGNAME||CHR(10)
		||', p_RCVCONFIGDESC=>'||p_RCVCONFIGDESC||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		||', p_THREADNBR=>'||p_THREADNBR||CHR(10)
		||', p_URI=>'||p_URI||CHR(10)
		||', p_URILOGIN=>'||p_URILOGIN||CHR(10)
		||', p_URIPASSWORD=>'||p_URIPASSWORD||CHR(10)
		||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
		||', p_JMSQUEUENAME=>'||p_JMSQUEUENAME||CHR(10)
		||', p_JMSFACTORYNAME=>'||p_JMSFACTORYNAME||CHR(10)
		||', p_PROVIDERURLPKG=>'||p_PROVIDERURLPKG||CHR(10)
		||', p_JNDIFACTORYNAME=>'||p_JNDIFACTORYNAME||CHR(10)
		||', p_DAYOFMONTH=>'||p_DAYOFMONTH||CHR(10)
		||', p_DAYOFWEEK=>'||p_DAYOFWEEK||CHR(10)
		||', p_HOUROPT=>'||p_HOUROPT||CHR(10)
		||', p_MINUTEOPT=>'||p_MINUTEOPT||CHR(10)
		||', p_PORTNBR=>'||p_PORTNBR||CHR(10)
		||', p_TIMEOUT_MS=>'||p_TIMEOUT_MS||CHR(10)
		||', p_FILEEXTFILTER=>'||p_FILEEXTFILTER||CHR(10)
		||', p_FREQDAYOFMONTH=>'||p_FREQDAYOFMONTH||CHR(10)
		||', p_FREQDAYOFWEEK=>'||p_FREQDAYOFWEEK||CHR(10)
		||', p_POLLINTERVAL=>'||p_POLLINTERVAL||CHR(10)
		||', p_FILEUNCHNGINTERVAL=>'||p_FILEUNCHNGINTERVAL||CHR(10)
        ||',p_MAXSYNCTIMEOUT=>'||p_MAXSYNCTIMEOUT||CHR(10)
 	    ||',p_MAXSYNCSIMULCONNECTION=>'||p_MAXSYNCSIMULCONNECTION||CHR(10)
 	    ||',P_BASICAUTHFLAG=>'||P_BASICAUTHFLAG||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_RCVCONFIG(
		RCVCONFIGID
		,RCVCONFIGNAME
		,RCVCONFIGDESC
		,PARTNERID
		,TRANSPORTID
		,DESTTYPEID
		,THREADNBR
		,URI
		,URILOGIN
		,URIPASSWORD
		,ACTIVESTATUSCD
		,JMSQUEUENAME
		,JMSFACTORYNAME
		,PROVIDERURLPKG
		,JNDIFACTORYNAME
		,DAYOFMONTH
		,DAYOFWEEK
		,HOUROPT
		,MINUTEOPT
		,PORTNBR
		,TIMEOUT_MS
		,FILEEXTFILTER
		,FREQDAYOFMONTH
		,FREQDAYOFWEEK
		,POLLINTERVAL
		,FILEUNCHNGINTERVAL
	) Values (
		PR_RCVCONFIG_SEQ.nextval
		,p_RCVCONFIGNAME
		,p_RCVCONFIGDESC
		,p_PARTNERID
		,p_TRANSPORTID
		,p_DESTTYPEID
		,p_THREADNBR
		,p_URI
		,p_URILOGIN
		,p_URIPASSWORD
		,p_ACTIVESTATUSCD
		,p_JMSQUEUENAME
		,p_JMSFACTORYNAME
		,p_PROVIDERURLPKG
		,p_JNDIFACTORYNAME
		,p_DAYOFMONTH
		,p_DAYOFWEEK
		,p_HOUROPT
		,p_MINUTEOPT
		,p_PORTNBR
		,p_TIMEOUT_MS
		,p_FILEEXTFILTER
		,p_FREQDAYOFMONTH
		,p_FREQDAYOFWEEK
		,p_POLLINTERVAL
		,p_FILEUNCHNGINTERVAL
	);
	p_ReturnCd := 0;

                              
  
	v_RowCnt := sql%ROWCOUNT;
    
	
	INSERT INTO pr_rcvconfigdtl
		            (rcvconfigdtlid, rcvconfigid, maxsynctimeout,
		             maxsyncsimulconnection,BASICAUTHFLAG
		            )
		     VALUES (pr_rcvconfigdtl_seq.NEXTVAL,pr_rcvconfig_seq.CURRVAL,  p_maxsynctimeout,
		             p_maxsyncsimulconnection,P_BASICAUTHFLAG
            );
	
	IF RC1%isOpen THEN Close RC1; END IF;
		Open RC1 FOR Select
			PR_RCVCONFIG_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
   			p_returnCd := 3;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
--			RAISE_APPLICATION_ERROR (-20999,'20000 PR_RCVCONFIGI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
          	Open RC1 FOR Select NULL, 1 RowTS From Dual;
--			RAISE_APPLICATION_ERROR (-20999,'20001 PR_RCVCONFIGI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_RCVCONFIGI' || SQLCODE|| ' '||SQLERRM);
END PR_RCVCONFIGI;
/

-- PR_RCVCONFIGS_AH1.SQL

CREATE OR REPLACE PROCEDURE PR_RCVCONFIGS_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_ATTRVALUE VARCHAR2
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RCVCONFIGS_AH1
    Create Date:     2008-11-11 16:26:14
    Author:          Amit Kumar Vijay

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2008-11-11 Amit Vijay	 6.2	To select receiver and destination with given attrvalue.
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RCVCONFIGS_AH1';
v_ExceptionFlg    integer := 0;

v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
	SELECT 
  	   pd.destname
  	   ,prc.rcvconfigname
  	FROM pr_desttype pd
		inner join pr_rcvconfig prc on pd.desttypeid=prc.desttypeid
	inner join pr_rcvattr pr on pr.rcvconfigid=prc.rcvconfigid
		inner join pr_transport_attr pta on pr.transportAttrId=pta.transportAttrId
	WHERE pr.attrvalue=p_ATTRVALUE
		AND pta.attrname='bcg.receiver.activationSpecJNDIName'; 

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);


EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGS_AH1:'||SQLERRM);
End PR_RCVCONFIGS_AH1;
/

-- PR_RCVCONFIGS_AH2.SQL

CREATE OR REPLACE PROCEDURE         PR_RCVCONFIGS_AH2
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in varchar2
    , p_RCVCONFIGID in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RCVCONFIGS_AH2
    Create Date:     2003-04-24 09:26:14
    Author:          M.Mount

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-23 M.Mount	 4.2.1  Removed Commit and Rollback
	2004-03-01 S.Priani      4.2.2  Removed NLS
	2007-07-31 Bharani J     6.1.1  Modified for Feature F104473
	2007-08-24 Bharani J     6.1.1  Modified for Feature F104489
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RCVCONFIGS_AH2';
v_ExceptionFlg    integer := 0;

v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_Locale=>'||p_Locale||CHR(10)
        ||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
          c.RCVCONFIGID
        ,c.RCVCONFIGNAME
        ,c.RCVCONFIGDESC
        ,c.PARTNERID
       ,c.TRANSPORTID
        ,RTRIM(t.TRANSPORTTYPE) TRANSPORTTYPE
        ,c.DESTTYPEID
        ,c.THREADNBR
        ,RTRIM(c.URI) URI
        ,RTRIM(c.URILOGIN) URILOGIN
        ,c.URIPASSWORD
        ,c.ACTIVESTATUSCD
        ,RTRIM(c.JMSQUEUENAME) JMSQUEUENAME
        ,RTRIM(c.JMSFACTORYNAME) JMSFACTORYNAME
        ,RTRIM(c.PROVIDERURLPKG) PROVIDERURLPKG
        ,RTRIM(c.JNDIFACTORYNAME) JNDIFACTORYNAME
        ,c.PORTNBR
        ,c.TIMEOUT_MS
        ,RTRIM(c.DAYOFMONTH) DAYOFMONTH
        ,RTRIM(c.DAYOFWEEK) DAYOFWEEK
        ,RTRIM(c.HOUROPT) HOUROPT
        ,RTRIM(c.MINUTEOPT) MINUTEOPT
        ,RTRIM(c.FILEEXTFILTER) FILEEXTFILTER
        ,c.ROWTS
        ,c.CREATEDATE
        ,c.FREQDAYOFMONTH
        ,c.FREQDAYOFWEEK
        ,c.POLLINTERVAL
        ,c.FILEUNCHNGINTERVAL
        ,p.MAXSYNCTIMEOUT
        ,p.MAXSYNCSIMULCONNECTION
        ,p.BASICAUTHFLAG BASICAUTHFLAG
    From
        PR_RCVCONFIG c
        INNER JOIN PR_TRANSPORT t ON t.TransportId = c.TransportId
        INNER JOIN PR_RCVCONFIGdtl p on c.RCVCONFIGID = p.RCVCONFIGID
    Where
        C.RCVCONFIGID = p_RCVCONFIGID
        and c.ACTIVESTATUSCD != 2
	;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --   COMMIT;

EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
 --           ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --           Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGS_AH2:'||SQLERRM);
End PR_RCVCONFIGS_AH2;
/

-- PR_RCVCONFIGS_AH3.SQL

CREATE OR REPLACE PROCEDURE PR_RCVCONFIGS_AH3
    (
        p_ReturnCd  out  integer
        , p_DebugFlg  in  pls_integer
        , RC1  out  db_ref.RCT
        , p_Locale in varchar2
        , p_URI in varchar
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RCVCONFIGS_AH3
    Create Date:     2003-04-24 09:26:14
    Author:          M.Mount

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-29 M.Mount       3.5    Created.
    2003-06-02 L.Ling        3.5    Added LOCALE feature.
    2003-09-23 M.Mount       4.2.1  Removed rollback and commit.
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RCVCONFIGS_AH3';

v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_Locale=>'||p_Locale||CHR(10)
        ||', p_URI =>'||p_URI ||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
          RTRIM(c.URI) URI
        ,c.RcvConfigId RcvConfigId
        ,c.DesttypeId DestTypeId
        ,RTRIM(d.DestName) DestName
        ,t.TransPortId TransPortiId
        ,RTRIM(t.TransPortType) TransPortType
          ,c.ROWTS
        ,c.CREATEDATE
    From
        PR_RCVCONFIG c
        INNER JOIN PR_TRANSPORT t ON t.TransPortId = c.TransPortId
        INNER JOIN PR_DESTTYPE d ON d.DestTypeId = c.DestTypeId
    Where
        c.UrI =  p_URI
	;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --  COMMIT;

EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
 --           ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --           Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGS_AH3:'||SQLERRM);
End PR_RCVCONFIGS_AH3;
/


-- PR_RCVCONFIGS_AH4.sql

CREATE OR REPLACE PROCEDURE  PR_RCVCONFIGS_AH4
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , RC2  out  db_ref.RCT
    , p_Locale in varchar2
    , p_RCVCONFIGID in NUMBER
    )
AS
/*
    ================================================================================
    Proc Name:       PR_RCVCONFIGS_AH4
    Create Date:     2003-04-24 09:26:14
    Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-01 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RCVCONFIGS';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_Locale=>'||p_Locale||CHR(10)
        ||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
          c.RCVCONFIGID
        ,c.RCVCONFIGNAME
        ,c.RCVCONFIGDESC
        ,c.PARTNERID
        ,RTRIM(t.TRANSPORTTYPE) TRANSPORTTYPE
        ,D.DestName     --c.DESTTYPEID
        ,c.THREADNBR
        ,RTRIM(c.URI) URI
        ,RTRIM(c.URILOGIN) URILOGIN
        ,c.URIPASSWORD
        ,c.ACTIVESTATUSCD
        ,RTRIM(c.JMSQUEUENAME) JMSQUEUENAME
        ,RTRIM(c.JMSFACTORYNAME) JMSFACTORYNAME
        ,RTRIM(c.PROVIDERURLPKG) PROVIDERURLPKG
        ,RTRIM(c.JNDIFACTORYNAME) JNDIFACTORYNAME
        ,c.PORTNBR
        ,c.TIMEOUT_MS
        ,RTRIM(c.DAYOFMONTH) DAYOFMONTH
        ,RTRIM(c.DAYOFWEEK) DAYOFWEEK
        ,RTRIM(c.HOUROPT) HOUROPT
        ,RTRIM(c.MINUTEOPT) MINUTEOPT
        ,RTRIM(c.FileextFilter) FILEEXTFILTER
        ,c.ROWTS
        ,c.CREATEDATE
        ,c.POLLINTERVAL
        ,c.FILEUNCHNGINTERVAL
    From
        PR_RCVCONFIG c
        INNER JOIN PR_TRANSPORT t ON t.TransportId = c.TransportId
        LEFT OUTER JOIN PR_DestType D ON c.DESTTYPEID = D.DESTTYPEID
    Where
        RCVCONFIGID = p_RCVCONFIGID
        ;
--
--  Result set
--
/*    IF RC2%isOpen THEN Close RC2; END IF;
    Open RC2 for
    select
        PARTNERID
        , PARTNERLOGIN
    from PR_PARTNER
    where DIRRCVACCESSFLG = 1
    ;
*/
--
-- Second Result set
--
    IF RC2%isOpen THEN Close RC2; END IF;
    Open RC2 for
    select
        DESTTYPEID
        , DESTNAME
    from PR_DESTTYPE
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGS:'||SQLERRM);
End PR_RCVCONFIGS_AH4;
/


-- PR_RCVCONFIGS_AH5.sql

CREATE OR REPLACE PROCEDURE PR_RCVCONFIGS_AH5
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RCVCONFIGS_AH5
    Create Date:     2003-05-30 15:05:43
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-05-30 V.Goel           3.5 Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RCVCONFIGS_AH5';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    SELECT
        RTRIM(RCVCONFIGNAME) RCVCONFIGNAME
        ,D.DestName
        ,RTRIM(URI) URI
    FROM
        PR_RCVCONFIG R
        INNER JOIN PR_DestType D on D.DestTypeId = R.DestTypeId
    Where
        UPPER(DestName) like 'VTP%';

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_RCVCONFIGS_AH5;
/


-- PR_RCVCONFIGS_AH6.sql

CREATE OR REPLACE PROCEDURE         PR_RCVCONFIGS_AH6
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , RC2  out  db_ref.RCT
    , p_Locale in varchar2
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RCVCONFIGS_AH6
    Create Date:     2003-07-21 09:26:14
    Author:          V.Goel
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-01 S.Priani      4.2.2  Removed NLS

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RCVCONFIGS_AH1';
v_ExceptionFlg    integer := 0;

v_ParamsTxt       varchar2(4000) :=
        ', p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_Locale=>'||p_Locale||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
       c.RCVCONFIGID
       ,c.RCVCONFIGNAME
       ,c.RCVCONFIGDESC
       ,c.PARTNERID
       ,c.TRANSPORTID
       ,RTRIM(t.TRANSPORTTYPE) TRANSPORTTYPE
       ,D.DestName     --c.DESTTYPEID
       ,c.THREADNBR
       ,RTRIM(c.URI) URI
       ,RTRIM(c.URILOGIN) URILOGIN
       ,c.URIPASSWORD
       ,c.ACTIVESTATUSCD
       ,RTRIM(c.JMSQUEUENAME) JMSQUEUENAME
       ,RTRIM(c.JMSFACTORYNAME) JMSFACTORYNAME
       ,RTRIM(c.PROVIDERURLPKG) PROVIDERURLPKG
       ,RTRIM(c.JNDIFACTORYNAME) JNDIFACTORYNAME
       ,c.PORTNBR
       ,c.TIMEOUT_MS
       ,RTRIM(c.DAYOFMONTH) DAYOFMONTH
       ,RTRIM(c.DAYOFWEEK) DAYOFWEEK
       ,RTRIM(c.HOUROPT) HOUROPT
       ,RTRIM(c.MINUTEOPT) MINUTEOPT
       ,RTRIM(c.FILEEXTFILTER) FILEEXTFILTER
       ,c.ROWTS
       ,c.CREATEDATE
        ,POLLINTERVAL
        ,FILEUNCHNGINTERVAL
    From
        PR_RCVCONFIG c
        INNER JOIN PR_TRANSPORT t ON t.TransportId = c.TransportId
        LEFT OUTER JOIN PR_DestType D ON c.DESTTYPEID = D.DESTTYPEID
    WHERE   c.ACTIVESTATUSCD != 2
    ;
--
-- Second Result set
--
    IF RC2%isOpen THEN Close RC2; END IF;
    Open RC2 for
    select
        DESTTYPEID
        , DESTNAME
    from PR_DESTTYPE
    ;

--
-- Result set
--
/*    IF RC3%isOpen THEN Close RC3; END IF;
    Open RC3 for
    select
        PARTNERID
        , PARTNERLOGIN
    from PR_PARTNER
    where DIRRCVACCESSFLG = 1
    ;
*/

    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);


    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGS_AH6:'||SQLERRM);
End PR_RCVCONFIGS_AH6;
/


-- PR_RCVCONFIGS_AH7.sql

CREATE OR REPLACE PROCEDURE PR_RCVCONFIGS_AH7
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , RC2  out  db_ref.RCT
    , p_TransportID in INTEGER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RCVCONFIGS_AH7
    Create Date:     2004-03-19 09:26:14
    Author:          T. Dang
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-07-31 Bharani J     6.1.1  Modified for Feature F104473
        2007-08-24 Bharani J     6.1.1  Modified for Feature F104489
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RCVCONFIGS_AH7';
v_ExceptionFlg    integer := 0;

v_ParamsTxt       varchar2(4000) :=
        ', p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_TransportID=>'||p_TransportID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
       c.RCVCONFIGID
       ,c.RCVCONFIGNAME
       ,c.RCVCONFIGDESC
       ,c.PARTNERID
       ,c.TRANSPORTID
       ,RTRIM(t.TRANSPORTTYPE) TRANSPORTTYPE
       ,D.DestName     --c.DESTTYPEID
       ,c.THREADNBR
       ,RTRIM(c.URI) URI
       ,RTRIM(c.URILOGIN) URILOGIN
       ,c.URIPASSWORD
       ,c.ACTIVESTATUSCD
       ,RTRIM(c.JMSQUEUENAME) JMSQUEUENAME
       ,RTRIM(c.JMSFACTORYNAME) JMSFACTORYNAME
       ,RTRIM(c.PROVIDERURLPKG) PROVIDERURLPKG
       ,RTRIM(c.JNDIFACTORYNAME) JNDIFACTORYNAME
       ,c.PORTNBR
       ,c.TIMEOUT_MS
       ,RTRIM(c.DAYOFMONTH) DAYOFMONTH
       ,RTRIM(c.DAYOFWEEK) DAYOFWEEK
       ,RTRIM(c.HOUROPT) HOUROPT
       ,RTRIM(c.MINUTEOPT) MINUTEOPT
       ,RTRIM(c.FILEEXTFILTER) FILEEXTFILTER
       ,c.ROWTS
       ,c.CREATEDATE
        ,POLLINTERVAL
        ,FILEUNCHNGINTERVAL
        ,p.MAXSYNCTIMEOUT 
        ,p.MAXSYNCSIMULCONNECTION 
        ,p.BASICAUTHFLAG BASICAUTHFLAG
    From
        PR_RCVCONFIG c
        INNER JOIN PR_TRANSPORT t ON t.TransportId = c.TransportId
        LEFT OUTER JOIN PR_DestType D ON c.DESTTYPEID = D.DESTTYPEID
        INNER JOIN PR_RCVCONFIGdtl p on c.RCVCONFIGID = p.RCVCONFIGID
    WHERE   c.ACTIVESTATUSCD != 2
            AND c.TRANSPORTID = p_TransportID
    ;
--
-- Second Result set
--
    IF RC2%isOpen THEN Close RC2; END IF;
    Open RC2 for
    select
        DESTTYPEID
        , DESTNAME
    from PR_DESTTYPE
    ;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);


    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGS_AH7:'||SQLERRM);
End PR_RCVCONFIGS_AH7 ;
/

-- PR_RCVCONFIGS_AH8.sql

CREATE OR REPLACE PROCEDURE PR_RCVCONFIGS_AH8
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, p_Locale  IN  VARCHAR2
	, p_value IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_RCVCONFIGS_AH8
	Create Date:     2004-04-07 00:10:27
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 P Dominick    4.2.2  Created
	================================================================================
	This procedure will check for the presence of a value (p_value) the URI column
		and return 0 if the value is not present, or 1 if the value is present.  
		The procedure will be used to help ensure uniqueness of none key columns.
	Return Code : 0 = unique, 1 = duplicate
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVCONFIGS_AH8';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_value=>'||p_value||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	SELECT  COUNT(DISTINCT URI) INTO v_RowCnt
	FROM
		 PR_RCVCONFIG
	WHERE
		URI = p_value
	;

	p_ReturnCd := 0;

	IF (v_RowCnt > 0) THEN p_ReturnCd := 1;
	END IF;

--dbms_output.put_line('v_RowCnt '||v_RowCnt);
--dbms_output.put_line('p_value '||p_value);

	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_RCVCONFIGS_AH8;
/


-- PR_RCVCONFIGS_AH9.sql

CREATE OR REPLACE PROCEDURE PR_RCVCONFIGS_AH9
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_TransportID in INTEGER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RCVCONFIGS_AH9
    Create Date:     2006-06-29 09:26:14
    Author:          S.Nair
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RCVCONFIGS_AH9';
v_ExceptionFlg    integer := 0;

v_ParamsTxt       varchar2(4000) :=
        ', p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_TransportID=>'||p_TransportID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
	RCVCONFIGID,         
	RCVCONFIGNAME,       
	RCVCONFIGDESC,       
	PARTNERID,           
	TRANSPORTID,         
	DESTTYPEID,          
	THREADNBR,          
	URI,                 
	URILOGIN,            
	URIPASSWORD,         
	ACTIVESTATUSCD,      
	JMSQUEUENAME,        
	JMSFACTORYNAME,      
	PROVIDERURLPKG,      
	JNDIFACTORYNAME,     
	DAYOFMONTH,          
	DAYOFWEEK,          
	HOUROPT,         
	MINUTEOPT,          
	PORTNBR,             
	TIMEOUT_MS,          
	FILEEXTFILTER,       
	ROWTS,               
	CREATEDATE,          
	UPDATEDATE,          
	FREQDAYOFMONTH,      
	FREQDAYOFWEEK,       
	POLLINTERVAL,        
	FILEUNCHNGINTERVAL  
    From
        PR_RCVCONFIG
    WHERE   
            TRANSPORTID = p_TransportID
    ;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            --RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGS_AH9:'||SQLERRM);
End PR_RCVCONFIGS_AH9;
/


-- PR_RCVCONFIGS_ALL.SQL

CREATE OR REPLACE PROCEDURE         PR_RCVCONFIGS_ALL
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_Locale in varchar2
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RCVCONFIGS_ALL
    Create Date:     2003-04-24 09:26:14
    Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-01 S.Priani      4.2.2  Removed NLS

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RCVCONFIGS_ALL';
v_ExceptionFlg    integer := 0;

v_ParamsTxt       varchar2(4000) :=
        ', p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_Locale=>'||p_Locale||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
       c.RCVCONFIGID
       ,c.RCVCONFIGNAME
       ,c.RCVCONFIGDESC
       ,c.PARTNERID
       ,c.TRANSPORTID
       ,RTRIM(t.TRANSPORTTYPE) TRANSPORTTYPE
       ,c.DESTTYPEID
       ,c.THREADNBR
       ,RTRIM(c.URI) URI
       ,RTRIM(c.URILOGIN) URILOGIN
       ,c.URIPASSWORD
       ,c.ACTIVESTATUSCD
       ,RTRIM(c.JMSQUEUENAME) JMSQUEUENAME
       ,RTRIM(c.JMSFACTORYNAME) JMSFACTORYNAME
       ,RTRIM(c.PROVIDERURLPKG) PROVIDERURLPKG
       ,RTRIM(c.JNDIFACTORYNAME) JNDIFACTORYNAME
       ,c.PORTNBR
       ,c.TIMEOUT_MS
       ,RTRIM(c.DAYOFMONTH) DAYOFMONTH
       ,RTRIM(c.DAYOFWEEK) DAYOFWEEK
       ,RTRIM(c.HOUROPT) HOUROPT
       ,RTRIM(c.MINUTEOPT) MINUTEOPT
       ,RTRIM(c.FILEEXTFILTER) FILEEXTFILTER
       ,c.ROWTS
       ,c.CREATEDATE
       ,c.POLLINTERVAL
       ,c.FILEUNCHNGINTERVAL
    From
        PR_RCVCONFIG c
        INNER JOIN PR_TRANSPORT t ON t.TransportId = c.TransportId
    WHERE   c.ACTIVESTATUSCD != 2
    ;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);


    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGS_ALL:'||SQLERRM);
End PR_RCVCONFIGS_ALL;
/


-- PR_RCVCONFIGU.SQL

CREATE OR REPLACE Procedure PR_RCVCONFIGU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_RCVCONFIGID in NUMBER
	,p_RCVCONFIGNAME in VARCHAR2
	,p_RCVCONFIGDESC in VARCHAR2
	,p_PARTNERID in NUMBER
	,p_TRANSPORTID in NUMBER
	,p_DESTTYPEID in NUMBER
	,p_THREADNBR in NUMBER
	,p_URI in VARCHAR2
	,p_URILOGIN in VARCHAR2
	,p_URIPASSWORD in RAW
	,p_ACTIVESTATUSCD in NUMBER
	,p_JMSQUEUENAME in VARCHAR2
	,p_JMSFACTORYNAME in VARCHAR2
	,p_PROVIDERURLPKG in VARCHAR2
	,p_JNDIFACTORYNAME in VARCHAR2
	,p_DAYOFMONTH in VARCHAR2
	,p_DAYOFWEEK in VARCHAR2
	,p_HOUROPT in VARCHAR2
	,p_MINUTEOPT in VARCHAR2
	,p_PORTNBR in NUMBER
	,p_TIMEOUT_MS in NUMBER
	,p_FILEEXTFILTER in VARCHAR2
	,p_ROWTS in NUMBER
	,p_FREQDAYOFMONTH in NUMBER
	,p_FREQDAYOFWEEK in NUMBER
	,p_POLLINTERVAL in NUMBER
	,p_FILEUNCHNGINTERVAL in NUMBER
        ,p_MAXSYNCTIMEOUT in NUMBER
 	,p_MAXSYNCSIMULCONNECTION in NUMBER
 	,P_BASICAUTHFLAG in NUMBER  --   should be '0' or '1'
) as

/*
	================================================================================
	Proc Name:       PR_RCVCONFIGU
	Create Date:     2003-09-26 08:21:31
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM        3.5    Created
	2007-07-31 Bharani J     6.1.1  Modified for Feature F104473
	2007-08-24 Bharani J     6.1.1  Modified for Feature F104489

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RCVCONFIGU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RCVCONFIGID=>'||p_RCVCONFIGID||CHR(10)
		||', p_RCVCONFIGNAME=>'||p_RCVCONFIGNAME||CHR(10)
		||', p_RCVCONFIGDESC=>'||p_RCVCONFIGDESC||CHR(10)
		||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
		||', p_THREADNBR=>'||p_THREADNBR||CHR(10)
		||', p_URI=>'||p_URI||CHR(10)
		||', p_URILOGIN=>'||p_URILOGIN||CHR(10)
		||', p_URIPASSWORD=>'||p_URIPASSWORD||CHR(10)
		||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
		||', p_JMSQUEUENAME=>'||p_JMSQUEUENAME||CHR(10)
		||', p_JMSFACTORYNAME=>'||p_JMSFACTORYNAME||CHR(10)
		||', p_PROVIDERURLPKG=>'||p_PROVIDERURLPKG||CHR(10)
		||', p_JNDIFACTORYNAME=>'||p_JNDIFACTORYNAME||CHR(10)
		||', p_DAYOFMONTH=>'||p_DAYOFMONTH||CHR(10)
		||', p_DAYOFWEEK=>'||p_DAYOFWEEK||CHR(10)
		||', p_HOUROPT=>'||p_HOUROPT||CHR(10)
		||', p_MINUTEOPT=>'||p_MINUTEOPT||CHR(10)
		||', p_PORTNBR=>'||p_PORTNBR||CHR(10)
		||', p_TIMEOUT_MS=>'||p_TIMEOUT_MS||CHR(10)
		||', p_FILEEXTFILTER=>'||p_FILEEXTFILTER||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_FREQDAYOFMONTH=>'||p_FREQDAYOFMONTH||CHR(10)
		||', p_FREQDAYOFWEEK=>'||p_FREQDAYOFWEEK||CHR(10)
		||', p_POLLINTERVAL=>'||p_POLLINTERVAL||CHR(10)
		||', p_FILEUNCHNGINTERVAL=>'||p_FILEUNCHNGINTERVAL||CHR(10)
                ||', p_MAXSYNCTIMEOUT=>'||p_MAXSYNCTIMEOUT||CHR(10)
 	        ||', p_MAXSYNCSIMULCONNECTION=>'||p_MAXSYNCSIMULCONNECTION||CHR(10)
 	        ||', P_BASICAUTHFLAG=>'||P_BASICAUTHFLAG||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_RCVCONFIG
	SET
		RCVCONFIGNAME=p_RCVCONFIGNAME
		,      RCVCONFIGDESC=p_RCVCONFIGDESC
		,      PARTNERID=p_PARTNERID
		,      TRANSPORTID=p_TRANSPORTID
		,      DESTTYPEID=p_DESTTYPEID
		,      THREADNBR=p_THREADNBR
		,      URI=p_URI
		,      URILOGIN=p_URILOGIN
		,      URIPASSWORD=p_URIPASSWORD
		,      ACTIVESTATUSCD=p_ACTIVESTATUSCD
		,      JMSQUEUENAME=p_JMSQUEUENAME
		,      JMSFACTORYNAME=p_JMSFACTORYNAME
		,      PROVIDERURLPKG=p_PROVIDERURLPKG
		,      JNDIFACTORYNAME=p_JNDIFACTORYNAME
		,      DAYOFMONTH=p_DAYOFMONTH
		,      DAYOFWEEK=p_DAYOFWEEK
		,      HOUROPT=p_HOUROPT
		,      MINUTEOPT=p_MINUTEOPT
		,      PORTNBR=p_PORTNBR
		,      TIMEOUT_MS=p_TIMEOUT_MS
		,      FILEEXTFILTER=p_FILEEXTFILTER
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= FN_Get_Date
		,      FREQDAYOFMONTH=p_FREQDAYOFMONTH
		,      FREQDAYOFWEEK=p_FREQDAYOFWEEK
		,      POLLINTERVAL=p_POLLINTERVAL
		,      FILEUNCHNGINTERVAL=p_FILEUNCHNGINTERVAL
	Where
		RCVCONFIGID = p_RCVCONFIGID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
		
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	
	
	Update PR_RCVCONFIGDTL
		SET
			
			     MAXSYNCTIMEOUT=p_MAXSYNCTIMEOUT
			,    MAXSYNCSIMULCONNECTION=p_MAXSYNCSIMULCONNECTION
			,    BASICAUTHFLAG=P_BASICAUTHFLAG
			,    ROWTS= p_RowTS + 1
			,    UPDATEDATE= FN_Get_Date
		Where
			RCVCONFIGID = p_RCVCONFIGID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
		
        IF RC1%isOpen THEN Close RC1; END IF;
        OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_RCVCONFIGU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RCVCONFIGU:'||SQLERRM);
End PR_RCVCONFIGU;
/

-- PR_RESOURCEBUNDLESD.sql

CREATE OR REPLACE Procedure PR_RESOURCEBUNDLESD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 

	,p_RESOURCEBUNDLEID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_RESOURCEBUNDLESD
	Create Date:     2004-03-26 10:12:51
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RESOURCEBUNDLESD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RESOURCEBUNDLEID=>'||p_RESOURCEBUNDLEID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_RESOURCEBUNDLES
	Where 
		RESOURCEBUNDLEID = p_RESOURCEBUNDLEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_RESOURCEBUNDLESD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_RESOURCEBUNDLESD;
/


-- PR_RESOURCEBUNDLESI.sql

CREATE OR REPLACE Procedure PR_RESOURCEBUNDLESI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_PROVIDERFLAG in NUMBER
	,p_RESBUNDLEFILENAME in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_RESOURCEBUNDLESI
	Create Date:     2004-03-26 10:14:22
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RESOURCEBUNDLESI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_RESBUNDLEFILENAME=>'||p_RESBUNDLEFILENAME||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_RESOURCEBUNDLES(
		RESOURCEBUNDLEID
		,PROVIDERFLAG
		,RESBUNDLEFILENAME
	) Values (
		PR_RESOURCEBUNDLES_SEQ.nextval
		,p_PROVIDERFLAG
		,p_RESBUNDLEFILENAME
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		PR_RESOURCEBUNDLES_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_RESOURCEBUNDLESI' || SQLCODE|| ' '||SQLERRM);
END PR_RESOURCEBUNDLESI;
/


-- PR_RESOURCEBUNDLESS.sql

CREATE OR REPLACE PROCEDURE PR_RESOURCEBUNDLESS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_RESOURCEBUNDLEID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_RESOURCEBUNDLESS
	Create Date:     2004-03-26 10:13:49
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RESOURCEBUNDLESS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RESOURCEBUNDLEID=>'||p_RESOURCEBUNDLEID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_RESOURCEBUNDLEID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		RESOURCEBUNDLEID
		,PROVIDERFLAG
		,RTRIM(RESBUNDLEFILENAME) RESBUNDLEFILENAME
		,ROWTS
		,CREATEDATE
	FROM 
		PR_RESOURCEBUNDLES
	Where 
		RESOURCEBUNDLEID = p_RESOURCEBUNDLEID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		RESOURCEBUNDLEID
		,PROVIDERFLAG
		,RTRIM(RESBUNDLEFILENAME) RESBUNDLEFILENAME
		,ROWTS
		,CREATEDATE
	FROM 
		PR_RESOURCEBUNDLES
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_RESOURCEBUNDLESS;
/


-- PR_RESOURCEBUNDLESU.sql

CREATE OR REPLACE Procedure PR_RESOURCEBUNDLESU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_RESOURCEBUNDLEID in NUMBER
	,p_PROVIDERFLAG in NUMBER
	,p_RESBUNDLEFILENAME in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_RESOURCEBUNDLESU
	Create Date:     2004-03-26 10:13:32
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_RESOURCEBUNDLESU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_RESOURCEBUNDLEID=>'||p_RESOURCEBUNDLEID||CHR(10)
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_RESBUNDLEFILENAME=>'||p_RESBUNDLEFILENAME||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_RESOURCEBUNDLES
	SET
		PROVIDERFLAG=p_PROVIDERFLAG
		,      RESBUNDLEFILENAME=p_RESBUNDLEFILENAME
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
	Where 
		RESOURCEBUNDLEID = p_RESOURCEBUNDLEID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_RESOURCEBUNDLESU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RESOURCEBUNDLESU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RESOURCEBUNDLESU:'||SQLERRM);
End PR_RESOURCEBUNDLESU;
/


-- PR_ROUTINGATTR_LNKD.sql

CREATE OR REPLACE Procedure PR_ROUTINGATTR_LNKD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_ROUTINGATTRLNKID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_ROUTINGATTR_LNKD
	Create Date:     2003-09-26 08:24:23
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ROUTINGATTR_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTRLNKID=>'||p_ROUTINGATTRLNKID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_ROUTINGATTR_LNK
	Where
		ROUTINGATTRLNKID = p_ROUTINGATTRLNKID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ROUTINGATTR_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_ROUTINGATTR_LNKD;
/


-- PR_ROUTINGATTR_LNKD_AH1.sql

CREATE OR REPLACE Procedure PR_ROUTINGATTR_LNKD_AH1

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_ROUTINGATTRID in NUMBER
	,p_ROUTINGOBJREFID in NUMBER
	,p_PartnerId in NUMBER
	,p_ChannelId in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_ROUTINGATTR_LNKD_AH1
	Create Date:     2006-04-26 08:24:23
	Author:          Shishir N
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ROUTINGATTR_LNKD_AH1';
v_ExceptionFlg    integer Default 0;
v_where           varchar2(4000) ;
v_sqlstmt         varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTRID=>'||p_ROUTINGATTRID||CHR(10)
		||', p_ROUTINGOBJREFID=>'||p_ROUTINGOBJREFID||CHR(10)
		||', p_PartnerId=>'||p_PartnerId||CHR(10)
		||', p_ChannelId=>'||p_ChannelId||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	    v_where := ' WHERE 1=1 ' ;
		  v_where := v_where || FN_Build_Where('PartnerId','NUMBER', p_PartnerId)||
		                        FN_Build_Where('ROUTINGATTRID','NUMBER', p_ROUTINGATTRID)||
		                        FN_Build_Where('ROUTINGOBJREFID','NUMBER', p_ROUTINGOBJREFID)||
                            FN_Build_Where('ChannelId','NUMBER', p_ChannelId);
		 	
		    v_sqlstmt := ' DELETE FROM PR_ROUTINGATTR_LNK '||v_where;
		 	
	execute immediate v_sqlstmt ;

	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ROUTINGATTR_LNKD_AH1: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_ROUTINGATTR_LNKD_AH1;
/


-- PR_ROUTINGATTR_LNKI_AH1.sql

CREATE OR REPLACE PROCEDURE PR_RoutingAttr_LnkI_AH1
    (
    p_ReturnCd out integer
    ,p_DebugFlg in pls_integer
    ,RC1 out db_ref.RCT
    ,p_Locale in varchar2
    ,p_ROUTINGATTRID in NUMBER
    ,p_ROUTINGOBJRefID in NUMBER
    ,p_PARTNERID in NUMBER
    ,p_CHANNELID in NUMBER
    ,p_DESTTYPEID in NUMBER
    ,p_ATTRVALUE in VARCHAR2
    ,p_GuidelineMapId in NUMBER
    ,p_FAMapId in NUMBER
    ,p_ProfileId in NUMBER
    ,p_AttrOptId in Number
    ,P_BUSIDENTIFIERID in Number
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RoutingAttr_LnkI_AH1
    Create Date:     2003-04-22 10:07:32
    Author:          J.Chen

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RoutingAttr_LnkI_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_ROUTINGATTRID=>'||p_ROUTINGATTRID||CHR(10)
        ||', p_ROUTINGOBJRefID=>'||p_ROUTINGOBJRefID||CHR(10)
        ||', p_PARTNERID=>'||p_PARTNERID||CHR(10)
        ||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
        ||', p_DESTTYPEID=>'||p_DESTTYPEID||CHR(10)
        ||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
        ||', p_GuidelineMapId=>'||p_GuidelineMapId||CHR(10)
        ||', p_FAMapId=>'||p_FAMapId||CHR(10)
        ||', p_ProfileId=>'||p_ProfileId||CHR(10)
        ||', p_AttrOptId=>'||p_AttrOptId||CHR(10)
        ||', P_BUSIDENTIFIERID=>'||P_BUSIDENTIFIERID
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    Insert into PR_ROUTINGATTR_LNK(
        ROUTINGATTRLNKID
        ,ROUTINGATTRID
        ,ROUTINGOBJRefID
        ,PARTNERID
        ,CHANNELID
        ,DESTTYPEID
        ,ATTRVALUE
        ,GuidelineMapId
        ,FAMapId
        ,ProfileId
        ,AttrOptId
        ,BUSIDENTIFIERID
    ) Values (
        PR_ROUTINGATTR_LNK_SEQ.nextval
        ,p_ROUTINGATTRID
        ,p_ROUTINGOBJRefID
        ,p_PARTNERID
        ,p_CHANNELID
        ,p_DESTTYPEID
        ,p_ATTRVALUE
        ,p_GuidelineMapId
        ,p_FAMapId
        ,p_ProfileId
        ,p_AttrOptId
        ,P_BUSIDENTIFIERID
    );
    p_ReturnCd := 0;
    v_RowCnt := sql%ROWCOUNT;
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 FOR Select
        PR_ROUTINGATTR_LNK_SEQ.currval RoutingAttrLnkId, 1 RowTS From Dual;
    p_ReturnCd := 0;
    Commit;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
        When DUP_VAL_ON_INDEX then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            p_returnCd := 2;
        When OTHERS Then
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_RoutingAttr_LnkI_AH1' || SQLCODE|| ' '||SQLERRM);
            Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
END PR_RoutingAttr_LnkI_AH1;
/


-- PR_ROUTINGATTR_LNKS.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGATTR_LNKS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ROUTINGATTRLNKID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_ROUTINGATTR_LNKS
	Create Date:     2003-09-26 00:06:23
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ROUTINGATTR_LNKS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ROUTINGATTRLNKID=>'||p_ROUTINGATTRLNKID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_ROUTINGATTRLNKID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		ROUTINGATTRLNKID
		,ROUTINGOBJREFID
		,ROUTINGATTRID
		,PARTNERID
		,CHANNELID
		,DESTTYPEID
		,GUIDELINEMAPID
		,FAMAPID
		,PROFILEID
		,ATTROPTID
		,RTRIM(ATTRVALUE) ATTRVALUE
		,ROWTS
		,CREATEDATE
		,BUSIDENTIFIERID
	FROM
		PR_ROUTINGATTR_LNK
	Where
		ROUTINGATTRLNKID = p_ROUTINGATTRLNKID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		ROUTINGATTRLNKID
		,ROUTINGOBJREFID
		,ROUTINGATTRID
		,PARTNERID
		,CHANNELID
		,DESTTYPEID
		,GUIDELINEMAPID
		,FAMAPID
		,PROFILEID
		,ATTROPTID
		,RTRIM(ATTRVALUE) ATTRVALUE
		,ROWTS
		,CREATEDATE
		,BUSIDENTIFIERID
	FROM
		PR_ROUTINGATTR_LNK
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_ROUTINGATTR_LNKS;
/


-- PR_ROUTINGATTR_LNKS_AH2.sql

CREATE OR REPLACE PROCEDURE PR_RoutingAttr_LnkS_AH2
    (
    p_ReturnCd              out integer
    ,p_DebugFlg             in pls_integer
    ,RC1                    out DB_Ref.RCT
    ,rc2                    out DB_Ref.RCT
    ,p_Locale               in varchar2
    ,p_RoutingObjRefId      in number
    ,p_PartnerId            in number
    ,p_ChannelId            in number
    ,p_DestTypeId           in number
    )
as
/*
    ================================================================================
    Proc Name:       PR_RoutingAttr_LnkS_AH2
    Create Date:     2003-05-05 10:07:31
    Author:          J.Chen
    Dependency:      FN_Get_AttrScope
                     FN_Get_RTObjAttrVL
                     FN_GET_RTOBJTYPE
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_ROUTINGATTR_LNKS_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||',p_RoutingObjRefId=>'||p_RoutingObjRefId||CHR(10)
        ||',p_PartnerId=>'||p_PartnerId||CHR(10)
        ||',p_ChannelId=>'||p_ChannelId||CHR(10)
        ||',p_DestTypeId=>'||p_DestTypeId||CHR(10);
v_ScopeKey         varchar2(512);
v_ScopeRank        varchar2(30);
v_SumScopeRank     Integer;
v_CurrScopeRank    Integer;
v_AttrRank         Integer;
v_RoutingObjAttrRank Integer;
v_where             varchar2(1000);
v_where2             varchar2(1000);
v_sqlstmt           varchar2(4000);
v_RoutingObjTypeKey varchar2(60);
TYPE v_RoutingAttrLnk_Rec is Record
(
    RoutingAttrId           number(10),
    RoutingObjRefId         number(10),
    RoutingObjId            number(10),
    RoutingObjAttrRank      number(6)
);
Type v_RoutingAttrLnk_T is table of v_RoutingAttrLnk_Rec index by pls_integer;
v_WkRoutingAttrLnk_Tbl v_RoutingAttrLnk_T;
v_RoutingAttrLnk_Tbl PR_RoutingAttrLnk_Tbl := PR_RoutingAttrLnk_Tbl();

----------------
-- Build_WhereIn
----------------
PROCEDURE Build_WhereIn(p_column in varchar2, p_dataType in varchar2, p_param in varchar2) IS
BEGIN
    IF p_param IS NOT NULL THEN
         IF p_dataType = 'NUMBER_LIST' THEN
              v_where := v_where||' and '||p_column||' in '||p_param;
         END IF;
         IF p_dataType = 'NUMBER' THEN
              Case
                    when p_Column = 'pl.PartnerId' Then
                        v_where := v_where||' Or ('||p_column||' ='|| to_char(p_param) ||' and pl.ChannelID IS NULL)';
                    when p_Column = 'pl.ChannelId' Then
                        v_where := v_where||' Or ('||p_column||' ='|| to_char(p_param) ||' and pl.DestTypeId is NULL)';
                    when p_Column = 'pl.DestTypeId' Then
                        v_where := v_where||' Or ('||p_column||' ='|| to_char(p_param)|| ' And pl.ChannelId = '||p_ChannelId ||')';
                    Else
                        null;
              End Case;
         END IF;
         IF p_dataType = 'VARCHAR2' THEN
              v_where := v_where||' and '||p_column||' ='''||p_param||'''' ;
         END IF;
         IF p_dataType = 'VARCHAR2_LIKE' THEN
              v_where := v_where||' and '||p_column||' LIKE '''||p_param||'''' ;
         END IF;
    END IF;
END Build_WhereIn;
---------------
-- Where_Clause
---------------
PROCEDURE Where_Clause AS
BEGIN
    Build_WhereIn('pl.ChannelId', 'NUMBER', p_ChannelId);
    Build_WhereIn('pl.PartnerId', 'NUMBER', p_PartnerId);
    Build_WhereIn('pl.DestTypeId', 'NUMBER', p_DestTypeId);
END Where_Clause;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
     Where_Clause;
     v_sqlstmt := '
     Select  Distinct al.RoutingAttrId, al.RoutingObjRefId, x.RoutingObjId, tp.RoutingObjAttrRank
     From
          (
           Select   RoutingObjId, RoutingObjRefId
           From     CF_RoutingObjRef_Lnk
           Start with RoutingObjRefId = '||p_RoutingObjRefId ||
          ' Connect by RoutingObjRefId = PRIOR RoutingObjParentRefId
           ) x
     Inner Join PR_RoutingAttr_Lnk pl On pl.RoutingObjRefId = x.RoutingObjRefId
     Inner Join CF_RoutingObjAttr_Lnk al On al.RoutingObjRefId = pl.RoutingObjRefId
                                         And al.RoutingAttrId = pl.RoutingAttrId
     Inner Join CF_RoutingObject o On o.RoutingObjId = x.RoutingObjId
     Inner Join CF_RoutingObjType tp On tp.RoutingObjTypeId = o.RoutingObjTypeId
     where   (pl.PartnerId IS NULL and pl.ChannelID is NULL and pl.DestTypeId IS NULL) '||
     v_where;
   db_Print('Sql =', v_sqlstmt);
   Execute Immediate v_Sqlstmt Bulk collect into v_WkRoutingAttrLnk_Tbl;
   For i IN 1..v_WkRoutingAttrLnk_Tbl.count LOOP
    v_RoutingAttrLnk_Tbl.extend;
                    v_RoutingAttrLnk_Tbl(i) := PR_RoutingAttrLnk_Rec
                                        (
                                        v_WkRoutingAttrLnk_Tbl(i).RoutingAttrId,
                                        v_WkRoutingAttrLnk_Tbl(i).RoutingObjRefId,
                                        v_WkRoutingAttrLnk_Tbl(i).RoutingObjId,
                                        v_WkRoutingAttrLnk_Tbl(i).RoutingObjAttrRank
                                        );
   End Loop;
   Select   t.RoutingObjTypeKey into v_RoutingObjTypeKey
   From     CF_RoutingObject o
            Inner join CF_RoutingObjType t On t.RoutingObjTypeId = o.RoutingObjTypeId
            Inner Join CF_RoutingObjRef_lnk l On l.RoutingObjId = o.RoutingObjId
   Where    l.RoutingObjRefId = p_RoutingObjRefId;
   Begin
   If rc1%isOpen then
        close rc1;
   End If;
   Open rc1 For
   Select
              T2.RoutingAttrId,
              v.Resourcekey,
              v.AttrDescKey,
              a.AttributeKey,
              v.ValidationRuleKey,
              v.Description,
              a.AttrDefLevel,
              a.RequiredFlg,
              a.VisibilityCd,
              T2.RoutingObjRefId,
              v_RoutingObjTypeKey RoutingObjTypeKey,
              T2.ValueSourceROTypeKey,
              T2.ValueSourceROTypeCd,
              T2.ValueSourceScopeKey,
              T2.ValueSourceScopeCd,
              T2.AttrValue,
              TO_NUMBER(T2.AttrOptId) AS AttrOptId,
              TO_NUMBER(T2.GuidelineMapId) AS GuidelineMapId,
              TO_NUMBER(T2.FAMapId) AS FAMapId,
              TO_NUMBER(T2.ProfileId) AS ProfileId,
              TO_NUMBER(T2.RoutingAttrLnkId) AS RoutingAttrLnkId,
              TO_NUMBER(T2.BusIdentifierId) AS BusIdentifierId,
              al.RowTS
		,T2.RoutingAttr_GrpID
  From
  (Select
              FN_Get_AttrScope(FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId, p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'AttrRank'),'LevelKey',p_locale) As ValueSourceROTypeKey,
              FN_Get_AttrScope(FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'AttrRank'),'LevelCd',p_locale) As ValueSourceROTypeCd,
              FN_Get_AttrScope(FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'AttrRank'),'ScopeCd',p_locale) As ValueSourceScopeCd,
              FN_Get_AttrScope(FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'AttrRank'),'ScopeKey',p_locale) As ValueSourceScopeKey,
              FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'AttrValue') As AttrValue,
              FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'AttrOptId') As AttrOptId,
              FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'GuidelineMapId') As GuidelineMapId,
              FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'FAMapId') As FAMapId,
              FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'ProfileId') As ProfileId,
              FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'RoutingAttrLnkId') As RoutingAttrLnkId,
              FN_Get_RTObjAttrVL(x.RoutingObjId, x.RoutingObjRefId,  p_PartnerId, p_ChannelId, p_DestTypeId, z.AttributeKey, 'BusIdentifierId') As BusIdentifierId,
              x.RoutingObjRefId,
              x.RoutingAttrId
		,z.RoutingAttr_GrpID
      From
            TABLE(CAST (v_RoutingAttrLnk_Tbl AS PR_RoutingAttrLnk_Tbl)) x,
            (Select     RoutingAttrId, MAX(RoutingObjAttrRank) As RoutingObjAttrRank
             From       TABLE(CAST(v_RoutingAttrLnk_Tbl AS PR_RoutingAttrLnk_Tbl))
             Group by   RoutingAttrId) y,
             CF_RoutingAttr z
      Where  x.RoutingAttrId = y.RoutingAttrId
      And    x.RoutingObjAttrRank = y.RoutingObjAttrRank
      And    x.RoutingAttrId = z.RoutingAttrId
     ) T2
    Inner Join pr_RoutingAttr_Lnk al On al.RoutingAttrLnkId = T2.RoutingAttrLnkId
    Inner Join CF_RoutingObjAttr_Lnk l On l.RoutingObjRefId = T2.RoutingObjRefId
                                      And l.RoutingAttrId = T2.RoutingAttrId
    Inner Join CF_RoutingAttr a On a.RoutingAttrId = l.RoutingAttrId
    Inner Join CF_RoutingAttr v On v.RoutingAttrId = a.RoutingAttrId
    Where     t2.RoutingAttrLnkId = al.RoutingAttrLnkId
 ;
    If rc2%isOpen then
        close rc2;
    end If;
    Open rc2 For
      Select  v.RoutingAttrId,
              v.AttrOptId,
              v.AttrValueKey,
              v.DefaultFlg,
              v.AttrSeqNbr,
              v.Resourcekey,
              v.Description,
              v.RowTS
    From      CF_RoutingObjAttr_Lnk l
              Inner Join CF_RoutingAttrOpt v On v.RoutingAttrId = l.RoutingAttrId
    Where
              l.RoutingObjRefId = p_RoutingObjRefId
    Order by 1;
    EXCEPTION
            WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
		p_ReturnCd := 1;
		RAISE;
    END;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt||CHR(10)||'SQLERRM='||SQLERRM, p_DebugFlg, v_ExceptionFlg);
END;
/


-- PR_ROUTINGATTR_LNKS_AH4.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGATTR_LNKS_AH4
    (
      p_ReturnCd            out pls_integer
    , p_debugFlg            in  pls_integer
    , RC1                   out db_ref.rct   --(single row channel data)
    , p_PartnerId           in number
    , p_FrPackageName       in varchar2
    , p_FrPackageVer        in varchar2
    , p_FrProtocolName      in varchar2
    , p_FrProtocolVer       in varchar2
    , p_FrProcessCd         in varchar2
    , p_FrProcessVer        in varchar2
    )
AS
/*
    ================================================================================
    Proc Name:      PR_ROUTINGATTR_LNKS_AH4.sql
    Create Date:    2004-10-10
    Author:         Shishir Narain
    ================================================================================
    Return Code : 0 = OK, 1 = Error Exception, 2 = Dup,
                  3= No Channel row Found, 4= 1 or more Capabilities NOT found (i.e. v_CapabilityCnt < 5),
                  5= Channel found, but missing 1 or more data lookups (i.e. Gateway or Workflow)

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt            integer :=0;
v_BeginTS           timestamp := FN_Get_Date; -- Performance related variables
v_ProcName          varchar2(30) := 'PR_ROUTINGATTR_LNKS_AH4.sql';
v_ExceptionFlg      integer := 0;
v_ReturnCd          pls_integer := 0;

v_DebugFlg          number(1) := p_DebugFlg;
v_RoutingObjCd      CF_RoutingObject.RoutingObjCd%TYPE;
v_RoutingObjVer     CF_RoutingObject.RoutingObjVer%TYPE;
v_FrPackageName     CF_RoutingObject.RoutingObjCd%TYPE;
v_FrPackageVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_FrPkgRefId        CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrProtocolName    CF_RoutingObject.RoutingObjCd%TYPE;
v_FrProtocolVer     CF_RoutingObject.RoutingObjVer%TYPE;
v_FrProtocolRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrProcessCd       CF_RoutingObject.RoutingObjCd%TYPE;
v_FrProcessVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_FrProcessRefId    CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_Msg               varchar2(128);
j                   number(6) := 0;
v_DestTypeId        PR_DESTTYPE.DESTTYPEID%TYPE := NULL;

p_FrRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
v_ChannelId                 PR_RoutingChannel.ChannelId%TYPE := NULL;
v_RoutingObjRefId           CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
TYPE v_RoutingAttr_Rec is Record
(
    AttrLabel               Varchar2(2),
    AttributeKey            varchar2(60),
    AttrRank                number(6),
    AttrValue               varchar2(255)
);
TYPE v_RoutingAttr_T is table of v_RoutingAttr_Rec index by pls_integer;
v_RoutingAttr_Tbl   v_RoutingAttr_T;
p_RoutingAttr_Tbl PR_RoutingAttr_Tbl := PR_RoutingAttr_Tbl();
v_RA_Rec            v_RoutingAttr_Rec;
v_StartMsg          varchar2(80) := CHR(10)||'-----------------------  Start Attribute Search -----------------------';
v_EndMsg            varchar2(80) := CHR(10)||'-----------------------  END   Attribute Search -----------------------';
v_ParamsTxt         varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
    ||', p_PartnerId=>'''||p_PartnerId||''''||CHR(10)
    ||', p_FrPackageName=>'''||p_FrPackageName||''''||CHR(10)
    ||', p_FrPackageVer=>'''||p_FrPackageVer||''''||CHR(10)
    ||', p_FrProtocolName=>'''||p_FrProtocolName||''''||CHR(10)
    ||', p_FrProtocolVer=>'''||p_FrProtocolVer||''''||CHR(10)
    ||', p_FrProcessCd=>'''||p_FrProcessCd||''''||CHR(10)
    ||', p_FrProcessVer=>'''||p_FrProcessVer||''''||CHR(10);
	Procedure FrAttrValue(v_RoutingObjTypeKey in varchar2, v_FrName out varchar2, v_FrVer out varchar2, v_FrRefId out pls_integer)
	       AS
	       BEGIN
	       
	           SELECT RoutingObjCd, RoutingObjVer, RoutingObjRefId INTO v_FrName, v_FrVer, v_FrRefId
	           FROM TABLE(CAST (p_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
	           WHERE RoutingObjTypeKey = v_RoutingObjTypeKey;
	       
	           EXCEPTION
	           WHEN OTHERS THEN NULL;              
	       END;
	BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    p_ReturnCd := 0;
    select
                            routingobjrefid into v_RoutingObjRefID
                            from
                            cf_routingobject03_vw
                            where RoutingObjCd = p_FrProcessCd 
                            and RoutingObjVer = p_FrProcessVer
                            and ParentObjCd = p_FrProtocolName
                            and ParentObjVer = p_FrProtocolVer
                            and GrParentObjCd = p_FrPackageName
                            and GrParentObjVer = p_FrPackageVer ;  
               
                PR_CapabilityS_AH5
                    (
                    v_ReturnCd
                    ,v_DebugFlg
                    ,p_PartnerId
                    ,v_RoutingObjRefId
                    ,p_FrRoutingCapability_Tbl
                    ,1
                    );
         
            FrAttrValue('Package', v_FrPackageName, v_FrPackageVer, v_FrPkgRefId);

            FrAttrValue('Protocol', v_FrProtocolName, v_FrProtocolVer, v_FrProtocolRefId);

            FrAttrValue('Process', v_FrProcessCd, v_FrProcessVer, v_FrProcessRefId);

    
        Begin
            select AttrLabel, AttributeKey, AttrRank, AttrValue
            BULK COLLECT into v_RoutingAttr_Tbl
            From
            (
                select /*+ FIRST_ROWS */ distinct
                    'FR' AttrLabel,
                    AttributeKey,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  p_PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrRank') AttrRank,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  p_PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrValue') AttrValue
                from
                    TABLE(CAST (p_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
                    INNER JOIN PR_RoutingAttr_Lnk  Ra on Ra.RoutingObjRefId = T.RoutingObjRefId
                    INNER JOIN CF_RoutingAttr A on Ra.RoutingAttrId = A.RoutingAttrId
                Where T.RoutingObjRefId in (v_FrPkgRefId, v_FrProtocolRefId, v_FrProcessRefId)
                      AND Ra.PartnerId IS NULL 
                           OR Ra.PartnerId = p_PartnerId
                                                     );
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'RC1 Attribute Select NO_DATA_FOUND: '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                    commit;
                WHEN OTHERS then
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGATTR_LNKS_AH4.sql:'||SQLERRM);
                    ROLLBACK work;
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, '>> Attribute Rank Lookup failed << '||CHR(10)||v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
                    Commit;
        END;
        For i in 1..v_RoutingAttr_Tbl.count LOOP
            p_RoutingAttr_Tbl.extend;
            p_RoutingAttr_Tbl(i) := PR_RoutingAttr_Rec(v_RoutingAttr_Tbl(i).AttrLabel, v_RoutingAttr_Tbl(i).AttributeKey,
                                                       v_RoutingAttr_Tbl(i).AttrRank,v_RoutingAttr_Tbl(i).AttrValue);
        END LOOP;
    Open RC1 for
        select distinct
            --T.AttrLabel,
            T.AttributeKey,
            T.AttrValue
        from
            TABLE(CAST (p_RoutingAttr_Tbl AS PR_RoutingAttr_Tbl)) T,
            (Select AttrLabel, AttributeKey, MAX(TO_NUMBER(AttrRank)) AttrRank from TABLE(CAST (p_RoutingAttr_Tbl AS PR_RoutingAttr_Tbl))
             group by AttrLabel, AttributeKey) T2
        Where T.AttrLabel = T2.AttrLabel and T.AttributeKey = T2.AttributeKey and T.AttrRank = T2.AttrRank;
       DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_EndMsg, p_DebugFlg, v_ExceptionFlg);
End;
/

-- pr_routingattr_lnks_ah5.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGATTR_LNKS_AH5
        (
        p_ReturnCd      OUT PLS_INTEGER
        , p_DebugFlg    IN  PLS_INTEGER
        , RC1           OUT DB_REF.RCT
        , p_partnerid   IN PLS_INTEGER
        , p_channelid   IN PLS_INTEGER
        )
AS

/*
        ================================================================================
        Proc Name:       PR_ROUTINGATTR_LNKS_AH5
        Create Date:     2006-05-17 08:59:37
        Author:          S.Nair 

                           ::::::::::::::::::::::::::::::
                                 M O D I F I C A T I O N
                           ::::::::::::::::::::::::::::::

        ---------- ------------- ------ ---------------------------------------------
        DATE       NAME          Ver    DESCRIPTION
        ---------- ------------- ------ ---------------------------------------------
        2006-06-13 domnickp      1.0    Created for defect 101520
        ================================================================================
        Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; 
v_ProcName        varchar2(30) Default 'PR_ROUTINGATTR_LNKS_AH5';
v_ExceptionFlg    integer Default 0;
v_ActivityID      varchar2(65);
v_ParamsTxt       varchar2(4000) Default
                'p_DebugFlg=>'||p_DebugFlg||CHR(10)
                ||', p_partnerid=>'||p_partnerid||CHR(10)
                ||', p_channelid=>'||p_channelid||CHR(10)
                ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   IF RC1%ISOPEN THEN CLOSE RC1; END IF;

   IF p_ChannelID IS NOT NULL THEN
        OPEN RC1 FOR 
        SELECT 
           ROUTINGATTRLNKID
		    ,ROUTINGOBJREFID
		    ,ROUTINGATTRID
		    ,PARTNERID
		    ,CHANNELID
    		 ,DESTTYPEID
	       ,GUIDELINEMAPID
    		 ,FAMAPID
	       ,PROFILEID
    		 ,ATTROPTID
    		 ,RTRIM(ATTRVALUE) ATTRVALUE
		    ,ROWTS
		    ,CREATEDATE
	     FROM
		    PR_ROUTINGATTR_LNK
	     WHERE
		    CHANNELID = p_ChannelID
	     ;
   ELSE
        OPEN RC1 FOR 
        SELECT 
           ROUTINGATTRLNKID
		    ,ROUTINGOBJREFID
		    ,ROUTINGATTRID
		    ,PARTNERID
		    ,CHANNELID
    		 ,DESTTYPEID
	       ,GUIDELINEMAPID
    		 ,FAMAPID
	       ,PROFILEID
    		 ,ATTROPTID
    		 ,RTRIM(ATTRVALUE) ATTRVALUE
		    ,ROWTS
		    ,CREATEDATE
	     FROM
		    PR_ROUTINGATTR_LNK
	     WHERE
		    PARTNERID = p_PartnerID 
          AND CHANNELID IS NULL
        ;
    END IF;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);      
    
    Exception
      WHEN NO_DATA_FOUND THEN
         p_ReturnCd := 3;
      WHEN OTHERS THEN
        p_ReturnCd := 1;
        v_ExceptionFlg := 1;
        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
        RAISE_APPLICATION_ERROR (-20999,'20001 Get_ActivityID_From_VUID'||v_ProcName ||' '|| SQLCODE|| ' '||SQLERRM);                    
    
End PR_ROUTINGATTR_LNKS_AH5;
/

-- PR_ROUTINGATTR_LNKS_ENVELOPE.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGATTR_LNKS_ENVELOPE
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROFILEID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_ROUTINGATTR_LNKS_ENVELOPE
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ROUTINGATTR_LNKS_ENVELOPE';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
		SELECT distinct
		RAL.ROUTINGOBJREFID
		,P1.PARTNERID FROMPARTNERID
		,P1.PARTNERNAME FROMPARTNERNAME
		,P2.PARTNERID TOPARTNERID
		,P2.PARTNERNAME TOPARTNERNAME
	FROM
		PR_ROUTINGATTR_LNK RAL
		INNER JOIN PR_ROUTINGCHANNEL RC ON RAL.CHANNELID = RC.CHANNELID
		INNER JOIN CF_ROUTINGATTR RA ON RA.ATTRIBUTEKEY = 'bcg.ro.edi.EnvelopeProfile'
		INNER JOIN PR_PARTNER P1 ON RC.FRPARTNERID = P1.PARTNERID
		INNER JOIN PR_PARTNER P2 ON RC.TOPARTNERID = P2.PARTNERID
	WHERE
		RAL.PROFILEID = p_PROFILEID;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		              WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
		              WHEN OTHERS then
		                   p_ReturnCd := 1;
		                   v_ExceptionFlg := 1;
		                   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                   RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGATTR_LNKS_ENVELOPE:'||SQLERRM);
End PR_ROUTINGATTR_LNKS_ENVELOPE;
/


-- PR_ROUTINGATTR_LNKS_FAMAP.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGATTR_LNKS_FAMAP
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_FAMAPID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_ROUTINGATTR_LNKS_FAMAP
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ROUTINGATTR_LNKS_FAMAP';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_FAMAPID=>'||p_FAMAPID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
		SELECT distinct
				RAL.ROUTINGOBJREFID
				,P1.PARTNERID FROMPARTNERID
				,P1.PARTNERNAME FROMPARTNERNAME
				,P2.PARTNERID TOPARTNERID
				,P2.PARTNERNAME TOPARTNERNAME
			FROM
				PR_ROUTINGATTR_LNK RAL
				INNER JOIN PR_ROUTINGCHANNEL RC ON RAL.CHANNELID = RC.CHANNELID
				INNER JOIN CF_ROUTINGATTR RA ON RA.ATTRIBUTEKEY = 'bcg.ro.edi.FAMAPNAME'
				INNER JOIN PR_PARTNER P1 ON RC.FRPARTNERID = P1.PARTNERID
				INNER JOIN PR_PARTNER P2 ON RC.TOPARTNERID = P2.PARTNERID
			WHERE
		RAL.FAMAPID = p_FAMAPID;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		              WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
		              WHEN OTHERS then
		                   p_ReturnCd := 1;
		                   v_ExceptionFlg := 1;
		                   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                   RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGATTR_LNKS_FAMAP:'||SQLERRM);
End PR_ROUTINGATTR_LNKS_FAMAP;
/


-- PR_ROUTINGATTR_LNKU_AH2.sql

CREATE OR REPLACE PROCEDURE PR_RoutingAttr_LnkU_AH2
    (
     p_ReturnCd out integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    ,p_RoutingAttrLnkId in NUMBER
    ,p_GuidelineMapId in number
    ,p_FAMapId in number
    ,p_ProfileId in number
    ,p_AttrValue in varchar2
    ,p_AttrOptId in Number
    ,P_BUSIDENTIFIERID in number
    ,p_ROWTS in NUMBER
) as

/*
    ================================================================================
    Proc Name:       PR_RoutingAttr_LnkU_AH2
    Create Date:     2003-04-22 10:07:32
    Author:          J.Chen

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RoutingAttr_LnkU_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_RoutingAttrLnkId=>'||p_RoutingAttrLnkId||CHR(10)
        ||', p_GuidelineMapId=>'||p_GuidelineMapId||chr(10)
        ||', p_FAMapId=>'||p_FAMapId||chr(10)
        ||', p_ProfileId=>'||p_ProfileId||chr(10)
        ||', p_AttrValue=>'||p_AttrValue||chr(10)
        ||', p_AttrOptId=>'||p_AttrOptId||chr(10)
        ||', P_BUSIDENTIFIERID=>'||P_BUSIDENTIFIERID||chr(10)
        ||', p_ROWTS=>'||p_ROWTS||CHR(10) ;


v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_RowCnt := 0;
    Update PR_ROUTINGATTR_LNK
    SET
        GuidelineMapId=p_GuidelineMapId
        ,      FAMapId=p_FAMapId
        ,      ProfileId=p_ProfileId
        ,      ATTRVALUE=p_ATTRVALUE
        ,      AttrOptId=p_AttrOptId
        ,      BusidentifierID=P_BUSIDENTIFIERID
        ,      ROWTS= p_RowTS + 1
        ,      UPDATEDATE= FN_GET_DATE()
    Where
        ROUTINGATTRLNKID = p_ROUTINGATTRLNKID
        And RowTS = p_RowTS returning RowTS into v_RowTS;
    v_RowCnt := SQL%rowcount;
    if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
    IF RC1%isOpen THEN Close RC1; END IF;
    OPEN RC1 for select v_RowTS RowTS from Dual;


    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        WHEN TOO_MANY_ROWS Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999, '20100 - PR_RoutingAttr_LnkU_AH2: Too Many Rows');
        WHEN ZeroUpdate then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingAttr_LnkU_AH2: No Data Found');
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingAttr_LnkU_AH2:'||SQLERRM);
End PR_RoutingAttr_LnkU_AH2;
/


-- PR_ROUTINGATTR_LNKU_AH3.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGATTR_LNKU_AH3 
(
   p_returncd         OUT      INTEGER,
   p_debugflg         IN       PLS_INTEGER,
   p_guidelinemapid   IN       NUMBER
)

AS
/*
    ================================================================================
    Proc Name:       PR_RoutingAttr_LnkU_AH3
    Create Date:     2007-08-20 16:40:12
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-10 BHARANI J         1  Created

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
  
   v_begints        TIMESTAMP       := fn_get_date ();
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   := 'PR_RoutingAttr_LnkU_AH3';
   v_exceptionflg   INTEGER         := 0;
   v_paramstxt      VARCHAR2 (4000)
      :=    'p_DebugFlg=>'
         || p_debugflg
         || CHR (10)
         || ', p_GuidelineMapId=>'
         || p_guidelinemapid
         || CHR (10);
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
   UPDATE pr_routingattr_lnk
      SET guidelinemapid = NULL
    WHERE guidelinemapid = p_guidelinemapid;

   
   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
                        
   p_returncd := 0;
   
EXCEPTION
  
   WHEN OTHERS
   THEN
      p_returncd := 1;
      v_exceptionflg := 1;
      db_procauditlogi_ah1 (v_procname, v_begints, v_paramstxt, p_debugflg,
                            1);
      raise_application_error (-20999,
                               '20100 - PR_RoutingAttr_LnkU_AH3:' || SQLERRM
                              );
END pr_routingattr_lnku_ah3;
/

-- PR_ROUTINGCHANNELD.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELD
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,p_CHANNELID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_ROUTINGCHANNELD
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        NUMBER := 0;
v_count           INTEGER DEFAULT 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'PR_ROUTINGCHANNELD';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
		;
zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

       select count(*) into v_count
	   from pr_routingchannel
	   where channelid = p_channelid and
	   parentchannelid != channelid;
	   if v_count > 0 then
	   begin
	        delete from PR_CHANNEL_GATEWAY_LNK
		 			               where
		    channelid = p_CHANNELID;
	   		delete from PR_ROUTINGATTR_LNK
		 			               where
		    channelid = p_CHANNELID;

		    delete from PR_ROUTINGCHANNEL
		 			               where
		    channelid = p_CHANNELID;
		end;
		else
		raise ZeroDelete ; 
		end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ROUTINGCHANNELD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_ROUTINGCHANNELD;
/

-- PR_ROUTINGCHANNELI_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_RoutingChannelI_AH1 (
 p_ReturnCd out integer
,p_DebugFlg in pls_integer
,RC1 out db_ref.RCT
,p_FrCapabilityId in NUMBER
,p_ToCapabilityId in NUMBER
,p_WorkFlowId in NUMBER
,p_ActiveStatusCd in NUMBER
,p_MultiCastFlg in NUMBER
,p_TransformMapId in NUMBER
,p_ProfileId in NUMBER
) AS
/*
================================================================================
Proc Name:        PR_RoutingChannelI_AH1
Create Date:      2003-04-21
Author:           J.Chen
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
---------- ------------- ------ ---------------------------------------------
DATE         NAME             Ver     DESCRIPTION
---------- ------------- ------ ---------------------------------------------
2003-04-21 J.Chen          3.5    Add a new channel record
2008-09-12 Amit Kumar      6.2	  Defect 47980, 106065 and 106066
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_rowcnt integer     :=0;
v_BeginTS timestamp  := FN_Get_Date; -- Performance related variables
v_pk_value number    := 0;
v_ProcName varchar2(30) := 'PR_RoutingChannelI_AH1';
v_ExceptionFlg integer;
v_ParamsTxt       varchar2(4000) :=
 --  db_general_Pkg.Audit_Log('PR_RoutingChannelI_AH1',v_beginTS, 'p_ReturnCd'||','||
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', RC1=>'||'RC1'||CHR(10)
        ||', p_FrCapabilityId=>'||p_FrCapabilityId||CHR(10)
        ||', p_ToCapabilityId=>'||p_ToCapabilityId||CHR(10)
        ||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
        ||', p_ActiveStatusCd=>'||p_ActiveStatusCd||CHR(10)
        ||', p_MultiCastFlg=>'||p_MultiCastFlg||CHR(10)
        ||', p_TransformMapId=>'||p_TransformMapId||CHR(10)
        ||', p_ProfileId=>'||p_ProfileId||CHR(10)
        ;
v_FrPartnerId Number(10);
v_ToPartnerId Number(10);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

--  Get FrPartnerId, ToPartnerId

    Select    PartnerId
    Into      v_FrPartnerId
    From      PR_Capability
    Where     CapabilityId = p_FrCapabilityId
    ;

    Select    PartnerId
    Into      v_ToPartnerId
    From      PR_Capability
    Where     CapabilityId = p_ToCapabilityId
    ;

    INSERT INTO PR_RoutingChannel(
        CHANNELID
        ,PARENTCHANNELID        -- For EDI which is not implemented in 3.5
        ,FRCAPABILITYID
        ,TOCAPABILITYID
        ,FRPARTNERID
        ,TOPARTNERID
        ,WORKFLOWID
        ,ACTIVESTATUSCD
        ,MULTICASTFLG          -- For EDI
        ,TransformMapId
        ,ProfileId
    )
    VALUES (
        PR_RoutingChannel_Seq.nextval
        ,PR_RoutingChannel_Seq.currval
        ,p_FrCapabilityId
        ,p_ToCapabilityId
        ,v_FrPartnerId
        ,v_ToPartnerId
        ,P_WorkFlowId
        ,CASE WHEN p_ActiveStatusCd=1 then 63 else 31 end
        ,p_MultiCastFlg
        ,p_TransformMapId
        ,p_ProfileId
    );

    v_rowcnt := sql%ROWCOUNT;

    OPEN RC1 FOR
    Select  PR_RoutingChannel_Seq.currval  ChannelId,
            1 RowTS
    From    Dual;


    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
 --   Commit;


    EXCEPTION
    When NO_DATA_FOUND Then p_ReturnCd := 3;
    When Others Then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
            Commit;
            Raise_application_error (-20999,'20000 PR_RoutingChannelI_AH1' || SQLCODE|| ' '||SQLERRM);
END PR_RoutingChannelI_AH1;
/


-- PR_ROUTINGCHANNELI_COPY.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELI_COPY
	(
	p_ReturnCd OUT PLS_INTEGER
	,p_DebugFlg IN PLS_INTEGER
	,RC1 OUT Db_Ref.RCT
	,p_CHANNELID IN NUMBER	
	,p_PROFILEID IN VARCHAR2
	)
AS
/*
	================================================================================
	Proc Name:       PR_ROUTINGCHANNELI_COPY
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        NUMBER := 0;
v_RowCnt          INTEGER DEFAULT 0;
v_BeginTS         TIMESTAMP DEFAULT Fn_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'PR_ROUTINGCHANNELI_COPY';
v_ExceptionFlg    INTEGER DEFAULT 0;
v_ParamsTxt       VARCHAR2(4000) DEFAULT
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	INSERT INTO PR_RoutingChannel(
		      CHANNELID
		      ,PARENTCHANNELID        
		      ,FRCAPABILITYID
		      ,TOCAPABILITYID
		      ,FRPARTNERID
		      ,TOPARTNERID
		      ,WORKFLOWID
		      ,ACTIVESTATUSCD
		      ,MULTICASTFLG          
	              ,PROFILEID
		      ,TransformMapId)
		      SELECT
			      PR_RoutingChannel_SEQ.NEXTVAL
			      ,PARENTCHANNELID 
			      ,FRCAPABILITYID
			      ,TOCAPABILITYID
			      ,FRPARTNERID
			      ,TOPARTNERID
			      ,WORKFLOWID
			      ,ACTIVESTATUSCD
			      ,MULTICASTFLG
			      ,p_PROFILEID
		    	      ,TransformMapId
			FROM
				PR_RoutingChannel
			Where
				channelid = p_CHANNELID;
				
	v_RowCnt := SQL%ROWCOUNT;
	IF RC1%isOpen THEN CLOSE RC1; END IF;
	OPEN RC1 FOR SELECT 
		PR_RoutingChannel_SEQ.CURRVAL NewChannelId, 1 RowTS FROM Dual;
		
	INSERT INTO PR_ROUTINGATTR_LNK 
		               (ROUTINGATTRLNKID,
		               ROUTINGATTRID,
		                ROUTINGOBJREFID,
		                PARTNERID,
		                CHANNELID,
		                DESTTYPEID,
		                ATTRVALUE,
		                GUIDELINEMAPID,
		                ATTROPTID,
		                BUSIDENTIFIERID)
			SELECT	 PR_ROUTINGATTR_LNK_SEQ.nextval,
				 ROUTINGATTRID,
				 ROUTINGOBJREFID,
				 PARTNERID,
				 PR_RoutingChannel_SEQ.CURRVAL,
				 DESTTYPEID,
				 ATTRVALUE,
				 GUIDELINEMAPID,
				 ATTROPTID,
	                	 BUSIDENTIFIERID
		         from
			               PR_ROUTINGATTR_LNK
			               where
		channelid = p_CHANNELID;
		
		INSERT INTO PR_CHANNEL_GATEWAY_LNK
			            ( channelgatewayid
			            , channelid
			            , frgatewayid
			            , togatewayid
			            , frscheduleid
			            , toscheduleid
			            , desttypeid)
			         SELECT PR_CHANNEL_GATEWAY_LNK_SEQ.NEXTVAL
			            , PR_RoutingChannel_SEQ.CURRVAL
			            , frgatewayid
			            , togatewayid
			            , frscheduleid
			            , toscheduleid
			            , desttypeid
			         FROM  PR_CHANNEL_GATEWAY_LNK
            where channelid = p_CHANNELID
            and Exists (select 'x' from PR_CHANNEL_GATEWAY_LNK where channelid = p_CHANNELID);
            
		p_ReturnCd := 0;
	Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		WHEN DUP_VAL_ON_INDEX THEN
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		WHEN OTHERS THEN
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			Db_Procauditlogi_Ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ROUTINGCHANNELI_COPY' || SQLCODE|| ' '||SQLERRM);
END PR_ROUTINGCHANNELI_COPY;
/

-- PR_ROUTINGCHANNELS_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_RoutingChannelS_AH1
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
--    , p_Locale in Varchar2
    , RC1  out  db_ref.RCT
    , p_CHANNELID in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RoutingChannelS_AH1
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Get channel info
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_ROUTINGCHANNELS';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
        CHANNELID
        ,FRCAPABILITYID
        ,TOCAPABILITYID
        ,CASE WHEN ACTIVESTATUSCD=63 THEN 1 ELSE 0 END ACTIVESTATUSCD
        ,MULTICASTFLG
        ,WorkFlowId
        ,TransformMapId
        ,RowTS
        ,ProfileId
    From
        PR_ROUTINGCHANNEL
    Where
        CHANNELID = p_CHANNELID
    ;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --  Commit;

    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH1:'||SQLERRM);

End PR_RoutingChannelS_AH1;
/


-- PR_ROUTINGCHANNELS_AH10.sql

CREATE OR REPLACE PROCEDURE pr_routingchannels_ah10 (
   p_secreturncd      OUT      PLS_INTEGER,
   p_partnerid        IN       PLS_INTEGER,
   p_frpackagename    IN       VARCHAR2,
   p_frpackagever     IN       VARCHAR2,
   p_frprotocolname   IN       VARCHAR2,
   p_frprotocolver    IN       VARCHAR2,
   p_frprocesscd      IN       VARCHAR2,
   p_frprocessver     IN       VARCHAR2,
   p_activityname     IN       VARCHAR2,
   p_actionname       IN       VARCHAR2
)
AS
/*
   ================================================================================
   Proc Name:       PR_ROUTINGCHANNELS_AH10
   Create Date:     07/30/2005
   Author:          Shishir N

                      ::::::::::::::::::::::::::::::
                            M O D I F I C A T I O N
                      ::::::::::::::::::::::::::::::

   ---------- ------------- ------ ---------------------------------------------
   DATE       NAME          VER    DESCRIPTION
   ---------- ------------- ------ ---------------------------------------------
   07/30/2005 Shishir N       6.0 Created
   2008-09-12 Amit Kumar      6.2 Defect 47980, 106065 and 106066

   ================================================================================
   Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

   ---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
   v_rowcnt                  INTEGER         DEFAULT 0;
   v_begints                 TIMESTAMP       DEFAULT fn_get_date;
                                             -- Performance related variables
   v_procname                VARCHAR2 (30)  DEFAULT 'PR_ROUTINGCHANNELS_AH10';
   v_exceptionflg            INTEGER         DEFAULT 0;
   v_chkpt                   INTEGER         DEFAULT 0;
   v_returncd                INTEGER         DEFAULT 0;
   v_paramstxt               VARCHAR2 (4000);
   v_sqlerrm                 VARCHAR2 (2000);
   v_msg                     VARCHAR2 (512);
   v_temp                    VARCHAR (255);
   v_routingobjrefid         INT             DEFAULT 0;
   v_routingobjparentrefid   INT             DEFAULT 0;
   v_grparentobjrefid        INT             DEFAULT 0;
   v_capabilityid            INT             DEFAULT 0;
   v_interactid              INT             DEFAULT 0;
BEGIN
   --
   -- step1: Verify that all the routing objects exists or not
   --
   v_chkpt := 0;

   SELECT 'x'
     INTO v_temp
     FROM cf_routingobject
    WHERE routingobjcd = p_frpackagename
      AND routingobjver = p_frpackagever
      AND routingobjtypeid = 1;

   v_chkpt := 1;

   SELECT 'x'
     INTO v_temp
     FROM cf_routingobject
    WHERE routingobjcd = p_frprotocolname
      AND routingobjver = p_frprotocolver
      AND routingobjtypeid = 2;

   v_chkpt := 2;

   SELECT 'x'
     INTO v_temp
     FROM cf_routingobject
    WHERE routingobjcd = p_frprocesscd
      AND routingobjver = p_frprocessver
      AND routingobjtypeid = 3;

   v_chkpt := 3;

   IF p_activityname IS NOT NULL
   THEN
      SELECT 'x'
        INTO v_temp
        FROM cf_routingobject
       WHERE routingobjcd = p_activityname AND routingobjtypeid = 4;
   END IF;

   v_chkpt := 4;

   IF p_actionname IS NOT NULL
   THEN
      SELECT 'x'
        INTO v_temp
        FROM cf_routingobject
       WHERE routingobjcd = p_actionname AND routingobjtypeid = 5;
   END IF;

   --
   -- step 2: the object hierarchy must exist
   --
   v_chkpt := 5;

   SELECT routingobjrefid, routingobjparentrefid, grparentobjrefid
     INTO v_routingobjrefid, v_routingobjparentrefid, v_grparentobjrefid
     FROM cf_routingobject03_vw
    WHERE routingobjcd = p_frprocesscd
      AND routingobjver = p_frprocessver
      AND parentobjcd = p_frprotocolname
      AND parentobjver = p_frprotocolver
      AND grparentobjcd = p_frpackagename
      AND grparentobjver = p_frpackagever;

   --
   -- step 3: the b2b capabilities for that routing object must be enabled
   --
   v_chkpt := 6;

   SELECT capabilityid
     INTO v_capabilityid
     FROM pr_capability
    WHERE partnerid = p_partnerid
      AND routingobjrefid = v_grparentobjrefid
      AND roletypecd IN (1, 3)
      AND CASE WHEN activestatuscd=3 THEN 1 ELSE 0 END = 1;

   v_chkpt := 7;

   SELECT capabilityid
     INTO v_capabilityid
     FROM pr_capability
    WHERE partnerid = p_partnerid
      AND routingobjrefid = v_routingobjparentrefid
      AND roletypecd IN (1, 3)
      AND CASE WHEN activestatuscd=3 THEN 1 ELSE 0 END = 1;

   v_chkpt := 8;

   SELECT capabilityid
     INTO v_capabilityid
     FROM pr_capability
    WHERE partnerid = p_partnerid
      AND routingobjrefid = v_routingobjrefid
      AND roletypecd IN (1, 3)
      AND CASE WHEN activestatuscd=3 THEN 1 ELSE 0 END = 1;
EXCEPTION
   WHEN NO_DATA_FOUND
   THEN
      BEGIN
         CASE v_chkpt
            WHEN 0
            THEN
               p_secreturncd := 9;
            WHEN 1
            THEN
               p_secreturncd := 10;
            WHEN 2
            THEN
               p_secreturncd := 11;
            WHEN 6
            THEN
               p_secreturncd := 13;
            WHEN 7
            THEN
               p_secreturncd := 14;
            WHEN 8
            THEN
               p_secreturncd := 15;
            ELSE
               p_secreturncd := 5;
         END CASE;
      END;
   WHEN OTHERS
   THEN
      --p_ReturnCd := 1;
      --v_ExceptionFlg := 1;
      db_procauditlogi_ah1 (v_procname,
                            v_begints,
                            SQLERRM || CHR (10) || v_paramstxt,
                            1,
                            1
                           );
      raise_application_error (-20999,
                                  '20002 PR_ROUTINGCHANNELS_AH10'
                               || SQLCODE
                               || ' '
                               || SQLERRM
                              );
END pr_routingchannels_ah10;
/


-- PR_ROUTINGCHANNELS_AH2.sql

CREATE OR REPLACE PROCEDURE         PR_RoutingChannelS_AH2
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , p_PartnerId in NUMBER
    , p_RoleTypeCd in integer        -- 1 = Initiator, 2 = receiver, 3 = both
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RoutingChannelS_AH2
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Return a list of channels for a given partner
                                    and role type. RC2 returns WorkFlowId which
                                    could be more than one for given channelId.
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066                              
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RoutingChannelS_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_PartnerId=>'||p_PartnerId||CHR(10)
        ||', p_RoleTypeCd=>'||p_RoleTypeCd
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    IF p_RoleTypeCd = 1 OR p_RoleTypeCd = 2 THEN

         Open RC1 for
         select
              c.CHANNELID
              ,c.FRCAPABILITYID
              ,c.TOCAPABILITYID
              ,CASE WHEN c.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END ACTIVESTATUSCD
              ,c.MULTICASTFLG
              ,c.ProfileId
              ,c.WorkFlowId
              ,c.TransformMapId
              ,c.RowTS
         From
              PR_RoutingChannel c
              INNER JOIN PR_Capability p ON p.CapabilityId = CASE WHEN p_RoleTypeCd = 1 THEN c.FrCapabilityId
                                                                  WHEN p_RoleTypeCd = 2 THEN c.ToCapabilityId
                                                             END
         Where
              p.PartnerId = p_PartnerId
         And  p.RoleTypeCd = p_RoleTypeCd
         ;

    ELSIF p_RoleTypeCd = 3 THEN
         Open RC1 for
         select
              c.CHANNELID
              ,c.FRCAPABILITYID
              ,c.TOCAPABILITYID
              ,CASE WHEN c.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END ACTIVESTATUSCD
              ,c.MULTICASTFLG
              ,c.ProfileId
              ,c.WorkFlowId
              ,c.TransformMapId
              ,c.RowTS
         From
              PR_RoutingChannel c
              INNER JOIN PR_Capability p ON p.CapabilityId = c.FrCapabilityId
                                         OR p.CapabilityId = c.ToCapabilityId
         Where
              p.PartnerId = p_PartnerId
         And  p.RoleTypeCd = p_RoleTypeCd
         ;

    END IF;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --  Commit;

    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH2:'||SQLERRM);

End PR_RoutingChannelS_AH2;
/


-- PR_ROUTINGCHANNELS_AH3.sql

CREATE OR REPLACE PROCEDURE         PR_RoutingChannelS_AH3
    (
    p_ReturnCd  out  pls_integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    , RC2 out  db_ref.rct
    , p_sortkey         in  varchar2
    , p_PageSizeNbr     in  pls_Integer
    , p_PageNbr         in  pls_Integer
    , p_FrPartnerIdList in Varchar2  -- list of partnerId (Id1,Id2,Id3)
    , p_ToPartnerIdList in Varchar2
    , p_InitRoutingObjRefId in Number
    , p_RecvRoutingObjRefId in Number
    , p_ActiveStatusCd in varchar2  -- 0=disabled, 1=enabled, 2=deleted
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RoutingChannelS_AH3
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns a list of active channels match input criteria
                                    including all the channels associated with all
                                    the children of the given Fr/ToRoutingObjRefId
    2003-06-02 J.Chen        3.5    Call pr_CapabilityS_AH5 with p_StatusOption = 0
    2003-06-13 J.Chen        3.5    Get all the children of input RoutingObjId - Defect #1-1G289
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
--v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RoutingChannelS_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_sortkey=>'||p_sortkey||CHR(10)
        ||', p_PageSizeNbr=>'||p_PageSizeNbr||CHR(10)
        ||', p_PageNbr=>'||p_PageNbr||CHR(10)
        ||', p_FrPartnerIdList=>'||p_FrPartnerIdList ||CHR(10)
        ||', p_ToPartnerIdList=>'||p_ToPartnerIdList ||CHR(10)
        ||', p_InitRoutingObjRefId=>'||p_InitRoutingObjRefId ||CHR(10)
        ||', p_RecvRoutingObjRefId=>'||p_RecvRoutingObjRefId||CHR(10)
        ||', p_ActiveStatusCd=>'||p_ActiveStatusCd||CHR(10)
        ;

  v_beginTS          timestamp(6):= FN_Get_Date;
  v_where            varchar2(1000);
  v_count            pls_integer;
  v_count2           pls_integer;
  v_sortKey          varchar2(1000);
  StartRow           pls_integer;
  EndRow             pls_integer;
  i                  pls_integer;
  sqlstmt            varchar2(4000);
  sqlstmt2           varchar2(4000);
  sql_Column         varchar2(2000);
  sql_inner          varchar2(1000); -- includes from clause, inner joins and some outer joins that effect where clause
  sql_outer          varchar2(4000);

  v_Select           varchar2(1000);
  v_SqlFr            varchar2(4000);
  v_SqlTo            varchar2(4000);
  v_NullFlg          integer := 0;
  v_FrCapabilityId   varchar2(4000);
  v_ToCapabilityId   varchar2(4000);
  v_where_Fr         varchar2(1000);
  v_where_To         varchar2(1000);
  v_FrFlg            Integer := 0;
  v_ToFlg            Integer := 0;

 Type v_Input_Rec is Record
(
    RoutingObjRefId     number(10),
    PartnerId           number(10)
);

TYPE v_Input_T is table of v_Input_Rec index by pls_integer;
v_WkInput_Tbl v_Input_T;

p_FrRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
p_ToRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
v_FrRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
v_ToRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();

----- Types
/*
Type char65Tab is table of varchar2(65);
Type num10Tab  is table of number(10);
array_ChannelId char65Tab := char65Tab();
array_rnbr       num10Tab  := num10Tab();
--
ChannelIdTab  char65_Table := char65_Table();
*/
/*
------------
-- Set pages
------------
Procedure Set_Page_Parameters(p_pageNumber in pls_integer,
                              p_pageSize   in pls_integer,
                              p_sqlstmt    in out varchar2) as
StartRow binary_integer:=0;
EndRow   binary_integer:=0;
Begin
    If nvl(p_pageNumber,0) in (0, 1) then
         StartRow := 1;
         EndRow   := nvl(p_PageSize,10);
    else
         StartRow := ((nvl(p_PageSize,10) * p_pageNumber)+1) -  nvl(p_PageSize,10) ;
         EndRow   :=   nvl(p_PageSize,10) * p_pageNumber;
    end if;
    p_sqlstmt := 'select E.* from (select rownum rnbr, F.*  from ('||p_sqlstmt||') F ) E where rnbr between '||startRow||' and '||endRow;
End Set_page_parameters;

*/
----------------
-- Build_WhereIn
----------------
PROCEDURE Build_WhereIn(p_column in varchar2, p_dataType in varchar2, p_param in varchar2, p_DirFlg in varchar2) IS
BEGIN
    IF p_param IS NOT NULL THEN

         IF p_dataType = 'NUMBER_LIST' THEN
              IF p_DirFlg = 'Fr' Then
                   v_where_Fr := v_Where_Fr ||' and '||p_column||' in '||p_param;
              ElsIf p_DirFlg = 'To' Then
                   v_where_To := v_Where_To ||' and '||p_column||' in '||p_param;
              Else
                   v_where := v_where||' and '||p_column||' in '||p_param;
              End If;
         END IF;

         IF p_dataType = 'NUMBER' THEN
             IF p_DirFlg = 'Fr' Then
                   v_where_Fr := v_Where_Fr ||' and '||p_column||' ='||to_char(p_param) ;
             ElsIf p_DirFlg = 'To' Then
                   v_where_To := v_Where_To ||' and '|| p_column||' ='||to_char(p_param) ;
             Else
                   v_where := v_where||' and '||p_column||' ='||to_char(p_param) ;
             End If;
         END IF;

         IF p_dataType = 'VARCHAR2' THEN
              v_where := v_where||' and '||p_column||' ='''||p_param||'''' ;
         END IF;

         IF p_dataType = 'VARCHAR2_LIKE' THEN

              v_where := v_where||' and '||p_column||' LIKE '''||p_param||'''' ;
         END IF;
    END IF;
END Build_WhereIn;

---------------
-- Where_Clause
---------------
PROCEDURE Where_Clause AS
BEGIN

    Build_WhereIn('CASE WHEN ACTIVESTATUSCD=63 THEN 1 ELSE 0 END ', 'NUMBER_LIST', p_ActiveStatusCd,'');

END Where_Clause;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

--------------------------------
-- Select columns for result set
--------------------------------

    v_select := 'Select  c.ChannelId
                        ,CASE WHEN c.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END ACTIVESTATUSCD
                        ,c.MulticastFlg
                        ,c.FrCapabilityId
                        ,c.ToCapabilityId
                        ,c.FrPartnerId
                        ,c.ToPartnerId
                        ,c.WorkFlowId
                        ,c.TransformMapId
                        ,c.RowTS ';

----------------------
--  Build Where clause
----------------------

    Build_WhereIn('c.PartnerId',  'NUMBER_LIST', p_FrPartnerIdList,'Fr');
    Build_WhereIn('c.PartnerId',  'NUMBER_LIST', p_ToPartnerIdList,'To');
    Build_WhereIn('r.RoutingObjRefId', 'NUMBER', p_InitRoutingObjRefId,'Fr');
    Build_WhereIn('r.RoutingObjRefId', 'NUMBER', p_RecvRoutingObjRefId,'To');


---------------------------------
-- Get CapabilityId List by input
---------------------------------
    Case
        When p_InitRoutingObjRefId is null and p_RecvRoutingObjRefId is Null        -- All Null input
              and p_FrPartnerIdList is null and p_ToPartnerIdList is null Then
              If p_ActiveStatusCd is not null Then
                   SqlStmt := v_Select || ' From PR_RoutingChannel c '||
                                                 ' Inner Join pr_Partner frp on frp.PartnerId = c.FrPartnerId
                                                   Inner Join pr_Partner top on top.PartnerId = c.ToPartnerId
                                            Where CASE WHEN ACTIVESTATUSCD=63 THEN 1 ELSE 0 END in '|| p_ActiveStatusCd;
              Else
                   SqlStmt := v_Select || ' From PR_RoutingChannel c
                                                 Inner Join pr_Partner frp on frp.PartnerId = c.FrPartnerId
                                                 Inner Join pr_Partner top on top.PartnerId = c.ToPartnerId ';
              End If;
              v_NullFlg := 1;

        When p_InitRoutingObjRefId is null and p_RecvRoutingObjRefId is Null Then    -- Null input on both ObjectRefId

               Build_WhereIn('c.FrPartnerId',  'NUMBER_LIST', p_FrPartnerIdList,'');
               Build_WhereIn('c.ToPartnerId',  'NUMBER_LIST', p_ToPartnerIdList,'');
               Build_WhereIn('CASE WHEN c.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END', 'NUMBER_LIST', p_ActiveStatusCd,'');

              SqlStmt := v_Select || ' From PR_RoutingChannel c
                                            Inner Join pr_Partner frp on frp.PartnerId = c.FrPartnerId
                                            Inner Join pr_Partner top on top.PartnerId = c.ToPartnerId
                                            where 1=1 '|| v_Where ;
              v_NullFlg := 1;
        Else

              v_sqlFr := ' Select distinct
                                  c.RoutingObjRefId,
                                  c.PartnerId
                             From
                                  CF_RoutingObject04_vw r
                                  INNER JOIN PR_Capability c on r.RoutingObjRefId = c.RoutingObjRefId ' ||
                             --     Inner Join pr_RoutingChannel ch on ch.FrcapabilityId = c.CapabilityId '||
                            'Where 1=1 '|| v_where_Fr;

              v_sqlTo := ' Select distinct
                                  c.RoutingObjRefId,
                                  c.PartnerId
                             From
                                  CF_RoutingObject04_vw r
                                  INNER JOIN PR_Capability c on r.RoutingObjRefId = c.RoutingObjRefId '||
                                --  Inner Join pr_RoutingChannel ch on ch.FrcapabilityId = c.CapabilityId '||
                             'Where 1=1 '||v_Where_To;


        End Case;

    If  v_NullFlg = 0 Then        -- At least one of the RoutingObjRefid is not null

---------------------
-- Get FrCapabilityId
---------------------
        If (p_FrPartnerIdList is not null) OR (p_InitRoutingObjRefId is not null) Then

              v_FrFlg := 1;

              Execute Immediate v_sqlFr Bulk Collect into v_WkInput_Tbl;

              For i IN 1..v_WkInput_Tbl.count LOOP

              PR_CapabilityS_AH5
              (
              p_ReturnCd
              ,p_DebugFlg
              ,v_WkInput_Tbl(i).PartnerId
              ,v_WkInput_Tbl(i).RoutingObjRefId
              ,p_FrRoutingCapability_Tbl
              ,0
              );

              For j IN 1..p_FrRoutingCapability_Tbl.count LOOP
                    v_FrRoutingCapability_Tbl.extend;
                    v_FrRoutingCapability_Tbl(j) := PR_RoutingCapability_Rec
                                        (
                                        p_FrRoutingCapability_Tbl(j).PartnerId,
                                        p_FrRoutingCapability_Tbl(j).RoutingObjLevel,
                                        p_FrRoutingCapability_Tbl(j).RoutingObjId,
                                        p_FrRoutingCapability_Tbl(j).RoutingObjTypeKey,
                                        p_FrRoutingCapability_Tbl(j).RoutingObjCd,
                                        p_FrRoutingCapability_Tbl(j).RoutingObjVer,
                                        p_FrRoutingCapability_Tbl(j).RoutingObjRefId,
                                        p_FrRoutingCapability_Tbl(j).CapabilityId
                                        );

               End Loop;
            End Loop;
        End If;

---------------------
-- Get ToCapabilityId
---------------------
        If (p_ToPartnerIdList is not null) OR (p_RecvRoutingObjRefId is not null) Then

              v_ToFlg := 1;
              execute immediate v_sqlTo Bulk Collect into v_WkInput_Tbl;

              For i IN 1..v_WkInput_Tbl.count LOOP
               --    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Channels being searched .... '||v_WkRoutingCapability_Tbl(i).ChannelId, p_DebugFlg, v_ExceptionFlg);
                --   DBMS_OUTPUT.Put_Line('from refid and partnerid'||v_WkInput_Tbl(i).RoutingObjRefId || ',' ||v_WkInput_Tbl(i).PartnerId );
              PR_CapabilityS_AH5
              (
              p_ReturnCd
              ,p_DebugFlg
              ,v_WkInput_Tbl(i).PartnerId
              ,v_WkInput_Tbl(i).RoutingObjRefId
              ,p_ToRoutingCapability_Tbl
              ,0
              );

              For j IN 1..p_ToRoutingCapability_Tbl.count LOOP
                    v_ToRoutingCapability_Tbl.extend;
                    v_ToRoutingCapability_Tbl(j) := PR_RoutingCapability_Rec
                                        (
                                        p_ToRoutingCapability_Tbl(j).PartnerId,
                                        p_ToRoutingCapability_Tbl(j).RoutingObjLevel,
                                        p_ToRoutingCapability_Tbl(j).RoutingObjId,
                                        p_ToRoutingCapability_Tbl(j).RoutingObjTypeKey,
                                        p_ToRoutingCapability_Tbl(j).RoutingObjCd,
                                        p_ToRoutingCapability_Tbl(j).RoutingObjVer,
                                        p_ToRoutingCapability_Tbl(j).RoutingObjRefId,
                                        p_ToRoutingCapability_Tbl(j).CapabilityId
                                        );
              END Loop;
           End Loop;
        End If;

----------------------------------
-- Where clause for ActiveStatusCd
----------------------------------
    v_where := ' where 1 = 1 ';
    Where_clause;

    Case
        When v_FrFlg = 1 and v_ToFlg = 1 Then
        dbms_output.put_line('both : sqlstmt set');
        sqlstmt := v_Select ||
               ' From PR_RoutingChannel c
                      Inner Join pr_Partner frp on frp.PartnerId = c.FrPartnerId
                      Inner Join pr_Partner top on top.PartnerId = c.ToPartnerId
                      Inner Join TABLE(CAST(:FrTb1 AS PR_RoutingCapability_Tbl)) FrT On c.FrCapabilityId = FrT.CapabilityId
                      Inner Join TABLE(CAST(:ToTbl AS PR_RoutingCapability_Tbl)) ToT On c.ToCapabilityId = ToT.CapabilityId '||
               v_where;
        -- From is not null
        When v_FrFlg = 1 Then
               sqlstmt := v_Select ||
               ' From PR_RoutingChannel c
                      Inner Join pr_Partner frp on frp.PartnerId = c.FrPartnerId
                      Inner Join pr_Partner top on top.PartnerId = c.ToPartnerId
                      Inner Join TABLE(CAST(:FrTb1 AS PR_RoutingCapability_Tbl)) FrT On c.FrCapabilityId = FrT.CapabilityId '||
               v_where;
        -- To is not null
        When v_ToFlg = 1 Then

        sqlstmt := v_Select ||
               ' From PR_RoutingChannel c
                      Inner Join pr_Partner frp on frp.PartnerId = c.FrPartnerId
                      Inner Join pr_Partner top on top.PartnerId = c.ToPartnerId
                      Inner Join TABLE(CAST(:ToTbl AS PR_RoutingCapability_Tbl)) ToT On c.ToCapabilityId = ToT.CapabilityId '||
               v_where;
        Else
            null;
    End Case;

    End If;  -- End of (If  v_NullFlg = 0 )

--    DB_Print('SqlSTMT', sqlSTMT);
--    DB_Print('v_sqlFr', v_Sqlfr);


    If p_sortkey is not null then
       v_sortkey := upper(p_sortkey);
       v_sortkey := replace(v_sortkey,'FRPARTNERNAME','frp.PartnerName');
       v_sortkey := replace(v_sortkey,'TOPARTNERNAME','top.PartnerName');
       sqlstmt := sqlstmt ||' order by '||v_sortkey;
    End If;

    --DB_Print('Sql_STMT', sqlSTMT);
-------------------------
-- Total row count Cursor
-------------------------
    If RC2%IsOpen then close RC2; end if;
    If NVL(p_pageNbr,0) in (0,1) then
        Case
            When v_FrFlg =1 and v_ToFlg = 1 Then
                Execute Immediate 'select count(*) From ( '||sqlstmt ||')' INTO v_count Using v_FrRoutingCapability_Tbl, v_ToRoutingCapability_Tbl;
            When v_FrFlg =1 Then
                Execute Immediate 'select count(*) From ( '||sqlstmt ||')' INTO v_count Using v_FrRoutingCapability_Tbl;
            When v_ToFlg =1 Then
                Execute Immediate 'select count(*) From ( '||sqlstmt ||')' INTO v_count Using v_ToRoutingCapability_Tbl;
            Else
                Execute Immediate 'select count(*) From ( '||sqlstmt ||')' INTO v_count ;
        End Case;

        Open RC2 for select v_count TotalRowsNbr from dual;
    End if;

---------------
-- Paging logic
---------------
--   Set_page_parameters(p_pageNbr, p_pageSizeNbr, sqlstmt);
   sqlstmt := FN_Paging_Logic(p_pageNbr, p_pageSizeNbr, sqlstmt);

     DB_Print('Sql paging', sqlstmt);
   --  dbms_output.put_line('sql pageing= '|| sqlstmt);
/*
    execute immediate sqlstmt Bulk Collect INTO array_rnbr, array_ChannelId;
--    dbms_output.put_line('Last rnbr '||array_rnbr.LAST );
-- Transfer activityID from array to table

--     v_count2 := array_ActivityId.count;
    dbms_output.put_line('******** number of rows  ********** '||v_count2);
    For i IN 1..array_ChannelId.count LOOP
      ChannelIdTab.extend;
      ChannelIdTab(i) := char65_rec(array_rnbr(i), array_ChannelId(i));
      --dbms_output.put_line('ChannelID '||array_ChannelId(i) );
    End Loop;
*/
    IF RC1%isOpen THEN Close RC1; END IF;

        Case
            When v_FrFlg = 1 and v_ToFlg = 1 Then
                open rc1 for sqlstmt using v_FrRoutingCapability_Tbl,v_ToRoutingCapability_Tbl;
                dbms_output.put_line('both');
            When v_FrFlg = 1 Then
                open rc1 for sqlstmt using v_FrRoutingCapability_Tbl;
                dbms_output.put_line('from');

            When v_ToFlg = 1 Then
                Open rc1 for sqlstmt using v_ToRoutingCapability_Tbl;
                dbms_output.put_line('to');

            Else
                Open rc1 For sqlstmt;
                dbms_output.put_line('none');

         End Case;


         p_ReturnCd := 0;
         DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

         EXCEPTION
              WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
              WHEN OTHERS then
                   p_ReturnCd := 1;
                   v_ExceptionFlg := 1;
                   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                   RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH3:'||SQLERRM);
End PR_RoutingChannelS_AH3;
--/
/


-- PR_ROUTINGCHANNELS_AH4.sql

CREATE OR REPLACE PROCEDURE PR_RoutingChannelS_AH4
    (
      p_ReturnCd            out pls_integer
    , p_debugFlg            in  pls_integer
    , RC1                   out db_ref.rct   --(single row channel data)
    , RC2                   out db_ref.rct   --(To/Fr attribute data)
    , RC3                   out db_ref.rct   --(To/Fr Services)
    , p_InitBusinessId      in varchar2
    , p_FrBusinessId        in varchar2
    , p_ToBusinessId        in varchar2
    , p_FrPackageName       in varchar2
    , p_FrPackageVer        in varchar2
    , p_FrProtocolName      in varchar2
    , p_FrProtocolVer       in varchar2
    , p_FrProcessCd         in varchar2
    , p_FrProcessVer        in varchar2
    , p_DestName            in varchar2
    , p_ActivityName        in varchar2 DEFAULT NULL
    , p_ActionName          in varchar2 DEFAULT NULL
    , p_ChannelId           in number Default NULL     -- Optional
    , p_Signal              in varchar2 default null   -- Optional
    )
AS
/*
    ================================================================================
    Proc Name:      PR_RoutingChannelS_AH4
    Create Date:    2003-04-23
    Author:         B.Moffat
    Description:    Retrieve From/To Channel information based upon Package, Protocol
                    Process, Activity and Action inputs.

            Returns the following Result sets:
            ---------------------------------
            1.  Channel attribures in a single row
            2.  All to/from services associated with this channel
            3.  All to/from TPA attributes
            Dependencies:  PR_CapabilityS_AH5
                           CF_RoutingObject04_vw
                           PR_Capability01_vw
                           PR_Capability03_vw
                           PR_Partner01_vw
                           PR_RoutingChannel03_vw
            Logic Steps
            -----------
            1.  Lookup Partner info and flip PartnerId's if necessary
            2.  Look for the root PR_RoutingChannel row at either the
                Process, Activity or action Level.
            3.  Using the previous channel result set, loop thru and find a channel
                that has all of matching capabilities using the input
                params passed in.
            4.  Return result set 1 with channel data.
            5.  Return result set 2 with service data.
            6.  Return result set 3 with TPA attribute data.  This data is retrieved based
                upon attributes tied to the process, activity or actions set at either
                the global, partner or channel override levels.
            NOTE:  A filter was added for all the differing spellings of Receipt
                   Acknowledgement.  This action will be queried using IN
                   'Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge'
                   to catch any spelling variations.

            IMPORTANT:  Currently the max number of routing object levels
            ---------   necessary for channel validation is 5.  This value
                        is stored in v_CapabilityMaxCnt.

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	
	2008-09-12 Amit Kumar 	 6.2	Defect 47980, 106065 and 106066
	
    ================================================================================
    Return Code : 0 = OK, 1 = Error Exception, 2 = Dup,
                  3= No Channel row Found, 4= 1 or more Capabilities NOT found (i.e. v_CapabilityCnt < 5),
                  5= Channel found, but missing 1 or more data lookups (i.e. Gateway or Workflow)

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt            integer :=0;
v_BeginTS           timestamp := FN_Get_Date; -- Performance related variables
v_ProcName          varchar2(30) := 'PR_RoutingChannelS_AH4';
v_ExceptionFlg      integer := 0;
v_ReturnCd          pls_integer := 0;

v_CapabilityMaxCnt  NUMBER(2) := 5; -- Max Number Channel Object levels
v_FrCapabilityId    PR_Capability.CapabilityId%TYPE;
v_ToCapabilityId    PR_Capability.CapabilityId%TYPE;
v_FrPrtRec          PR_Partner01_vw%ROWTYPE;
v_ToPrtRec          PR_Partner01_vw%ROWTYPE;
v_ChanRec           PR_RoutingChannel01_vw%ROWTYPE := NULL;
v_SvcRec            PR_Capability03_vw%ROWTYPE;
v_ChanDirection     varchar2(2);
v_DebugFlg          number(1) := p_DebugFlg;
v_RoutingObjCd      CF_RoutingObject.RoutingObjCd%TYPE;
v_RoutingObjVer     CF_RoutingObject.RoutingObjVer%TYPE;
v_FrPackageName     CF_RoutingObject.RoutingObjCd%TYPE;
v_FrPackageVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_FrPkgRefId        CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrProtocolName    CF_RoutingObject.RoutingObjCd%TYPE;
v_FrProtocolVer     CF_RoutingObject.RoutingObjVer%TYPE;
v_FrProtocolRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrProcessCd       CF_RoutingObject.RoutingObjCd%TYPE;
v_FrProcessVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_FrProcessRefId    CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrActivityRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrActionRefId     CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrSignalRefId     CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToPackageName     CF_RoutingObject.RoutingObjCd%TYPE;
v_ToPackageVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_ToPkgRefId        CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToProtocolName    CF_RoutingObject.RoutingObjCd%TYPE;
v_ToProtocolVer     CF_RoutingObject.RoutingObjVer%TYPE;
v_ToProcessCd       CF_RoutingObject.RoutingObjCd%TYPE;
v_ToProcessVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_ToActivityName    CF_RoutingObject.RoutingObjCd%TYPE := p_ActivityName;
v_ToActionName      CF_RoutingObject.RoutingObjCd%TYPE := p_ActionName;
v_ToSignal         CF_RoutingObject.RoutingObjCd%TYPE := p_ActionName;
v_ToProtocolRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToProcessRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToActivityRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToActionRefId     CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToSignalRefId     CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_Msg               varchar2(128);
j                   number(6) := 0;
v_DestTypeId        PR_DESTTYPE.DESTTYPEID%TYPE;

TYPE v_RoutingCapability_Rec is Record
(
    ChannelId               number(10),
    FrPartnerId             number(10),
    ToPartnerId             number(10),
    RoutingObjLevel         number(6),
    RoutingObjId            number(10),
    RoutingObjTypeKey       varchar2(60),
    RoutingObjCd            varchar2(255),
    RoutingObjVer           varchar2(60),
    RoutingObjRefId         number(10),
    CapabilityId            number(10)
);
TYPE v_RoutingCapability_T is table of v_RoutingCapability_Rec index by pls_integer;
v_WkRoutingCapability_Tbl v_RoutingCapability_T;
p_FrRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
p_ToRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
v_TmpRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
v_CapabilityCnt             number(6) := 0;
v_ChannelId                 PR_RoutingChannel.ChannelId%TYPE;
v_RoutingObjRefId           CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
TYPE v_RoutingAttr_Rec is Record
(
    AttrLabel               Varchar2(2),
    AttributeKey            varchar2(60),
    AttrRank                number(6),
    AttrValue               varchar2(255)
);
TYPE v_RoutingAttr_T is table of v_RoutingAttr_Rec index by pls_integer;
v_RoutingAttr_Tbl   v_RoutingAttr_T;
p_RoutingAttr_Tbl PR_RoutingAttr_Tbl := PR_RoutingAttr_Tbl();
v_RA_Rec            v_RoutingAttr_Rec;
v_StartMsg          varchar2(80) := CHR(10)||'-----------------------  Start Channel Search -----------------------';
v_EndMsg            varchar2(80) := CHR(10)||'-----------------------  END   Channel Search -----------------------';
v_ParamsTxt         varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
    ||', p_InitBusinessId=>'''||p_InitBusinessId||''''||CHR(10)
    ||', p_FrBusinessId=>'''||p_FrBusinessId||''''||CHR(10)
    ||', p_ToBusinessId=>'''||p_ToBusinessId||''''||CHR(10)
    ||', p_FrPackageName=>'''||p_FrPackageName||''''||CHR(10)
    ||', p_FrPackageVer=>'''||p_FrPackageVer||''''||CHR(10)
    ||', p_FrProtocolName=>'''||p_FrProtocolName||''''||CHR(10)
    ||', p_FrProtocolVer=>'''||p_FrProtocolVer||''''||CHR(10)
    ||', p_FrProcessCd=>'''||p_FrProcessCd||''''||CHR(10)
    ||', p_FrProcessVer=>'''||p_FrProcessVer||''''||CHR(10)
    ||', p_ActivityName=>'''||p_ActivityName||''''||CHR(10)
    ||', p_ActionName=>'''||p_ActionName||''''||CHR(10)
    ||', p_DestName=>'||p_DestName||CHR(10)
    ||', p_ChannelId=>'||p_ChannelId||CHR(10);
    
Procedure FrAttrValue(v_RoutingObjTypeKey in varchar2, v_FrName out varchar2, v_FrVer out varchar2, v_FrRefId out pls_integer)
AS
BEGIN

    SELECT RoutingObjCd, RoutingObjVer, RoutingObjRefId INTO v_FrName, v_FrVer, v_FrRefId
    FROM TABLE(CAST (p_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
    WHERE RoutingObjTypeKey = v_RoutingObjTypeKey;

    EXCEPTION
    WHEN OTHERS THEN NULL;              
END;

Procedure ToAttrValue(v_RoutingObjTypeKey in varchar2, v_ToName out varchar2, v_ToVer out varchar2, v_ToRefId out pls_integer)
AS
BEGIN
    SELECT RoutingObjCd, RoutingObjVer, RoutingObjRefId into v_ToName, v_ToVer, v_ToRefId
    FROM TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
    Where RoutingObjTypeKey = v_RoutingObjTypeKey;

    EXCEPTION
    WHEN OTHERS THEN NULL;              
END;

Procedure FrAttrValue2(v_RoutingObjTypeKey in varchar2, p_ObjCd in varchar2, v_FrRefId out pls_integer)
AS
BEGIN
    IF p_ObjCd IS NOT NULL then
    BEGIN
        select RoutingObjRefId into v_FrRefId
        FROM TABLE(CAST (p_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
        Where RoutingObjTypeKey = v_RoutingObjTypeKey and RoutingObjCd = p_ObjCd  and rownum < 2;
        exception
            When NO_DATA_FOUND Then v_FrRefId := NULL;
            when others then
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'SQLERRM: ' || SQLERRM);
    END;
    END IF;
END;
    
Procedure Get_Channel as
BEGIN
    BEGIN
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'Direction = '||v_ChanDirection||'  Fr/To Partner='||v_FrPrtRec.PartnerId||'/'||v_ToPrtRec.PartnerId, p_DebugFlg, v_ExceptionFlg);
        -----------------------------------------------------------
        --  2. Look for the root PR_RoutingChannel row at either the
        --     Process, Activity or action Level
        -----------------------------------------------------------
        Select /*+ FIRST_ROWS */
            Ch.ChannelId,
            Ch.FrPartnerId FrPartnerId,
            Ch.ToPartnerId ToPartnerId,
            RoutingObjLevel,
            RoutingObjId,
            RoutingObjTypeKey,
            RoutingObjCd,
            RoutingObjVer,
            R.RoutingObjRefId,
            C.CapabilityId
        BULK COLLECT INTO v_WkRoutingCapability_Tbl
        From
            CF_RoutingObject04_vw R
            INNER JOIN PR_Capability C on C.RoutingObjRefId = R.RoutingObjRefId
            INNER JOIN PR_RoutingChannel Ch on C.CapabilityId = CASE WHEN v_ChanDirection = 'FR' THEN Ch.FrCapabilityId ELSE Ch.ToCapabilityId END
                                            and v_FrPrtRec.PartnerId = CASE WHEN v_ChanDirection = 'FR' THEN Ch.FrPartnerId ELSE Ch.ToPartnerId END
                                            and v_ToPrtRec.PartnerId = CASE WHEN v_ChanDirection = 'TO' THEN Ch.FrPartnerId ELSE Ch.ToPartnerId END
        Where
            R.DocumentFlg = 1
            AND ((RoutingObjTypeKey = 'Process' and RoutingObjCd = p_FrProcessCd and RoutingObjVer = p_FrProcessVer) OR
            (RoutingObjTypeKey = 'Activity' and RoutingObjCd = p_ActivityName) OR
                    (RoutingObjTypeKey = 'Action' and
                    CASE WHEN RoutingObjCd IN ('Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge') THEN 'Receipt Ack' ELSE RoutingObjCd END =
                         CASE WHEN p_ActionName IN ('Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge') THEN 'Receipt Ack' ELSE p_ActionName END
            ))
            and CASE WHEN ch.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END = 1 AND C.PartnerId = v_FrPrtRec.PartnerId;
        Exception
            WHEN NO_DATA_FOUND THEN
                p_ReturnCd := 3;
            WHEN OTHERS then
                RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH4:'||SQLERRM);
                ROLLBACK work;
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
                Commit;
    END;
    IF p_ReturnCd = 0 THEN
        IF p_DebugFlg = 1 THEN
            BEGIN
                FOR i IN 1..v_WkRoutingCapability_Tbl.count LOOP
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,
                        'ChannelId       =>'||v_WkRoutingCapability_Tbl(i).ChannelId||CHR(10)||
                        'FrPartnerId     =>'||v_WkRoutingCapability_Tbl(i).FrPartnerId||CHR(10)||
                        'ToPartnerId     =>'||v_WkRoutingCapability_Tbl(i).ToPartnerId||CHR(10)||
                        'RoutingLevel    =>'||v_WkRoutingCapability_Tbl(i).RoutingObjLevel||CHR(10)||
                        'RoutingObjId    =>'||v_WkRoutingCapability_Tbl(i).RoutingObjId||CHR(10)||
                        'RoutingObjType  =>'||v_WkRoutingCapability_Tbl(i).RoutingObjTypeKey||CHR(10)||
                        'RoutingObjCd    =>'||v_WkRoutingCapability_Tbl(i).RoutingObjCd||CHR(10)||
                        'RoutingObjVer   =>'||v_WkRoutingCapability_Tbl(i).RoutingObjVer||CHR(10)||
                        'RoutingObjRefId =>'||v_WkRoutingCapability_Tbl(i).RoutingObjRefId||CHR(10)||
                        'CapabilityId    =>'||v_WkRoutingCapability_Tbl(i).CapabilityId
                        , p_DebugFlg, v_ExceptionFlg);
                END LOOP;
                COMMIT;
            END;
        END IF;
        -----------------------------------------------------------
        --  3.  Using the previous channel result set, find a channel
        --      that has all of matching capabilities using the input
        --      params passed in.
        -----------------------------------------------------------
        BEGIN
            For i IN 1..v_WkRoutingCapability_Tbl.count LOOP
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'--> Channels being searched .... '||v_WkRoutingCapability_Tbl(i).ChannelId, p_DebugFlg, v_ExceptionFlg);
                PR_CapabilityS_AH5
                    (
                    v_ReturnCd
                    ,v_DebugFlg
                    ,v_FrPrtRec.PartnerId
                    ,v_WkRoutingCapability_Tbl(i).RoutingObjRefId
                    ,p_FrRoutingCapability_Tbl
                    ,1
                    );
                ---- Count how many parameters match the capabilities returned from PR_CapabilityS_AH5
                v_CapabilityCnt := 0;
                SELECT /*+ FIRST_ROWS */ count(*) ChanChk into v_CapabilityCnt
                FROM TABLE(CAST (p_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl))
                Where
                    (RoutingObjTypeKey = 'Package' and RoutingObjCd = p_FrPackageName AND RoutingObjVer = p_FrPackageVer) OR
                    (RoutingObjTypeKey = 'Protocol' and RoutingObjCd = p_FrProtocolName and RoutingObjVer = p_FrProtocolVer) OR
                    (RoutingObjTypeKey = 'Process' and RoutingObjCd = p_FrProcessCd and RoutingObjVer = p_FrProcessVer) OR
                    (RoutingObjTypeKey = 'Activity' and RoutingObjCd = p_ActivityName) OR
                    (RoutingObjTypeKey = 'Action' and
                    CASE WHEN RoutingObjCd IN ('Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge') THEN 'Receipt Ack' ELSE RoutingObjCd END =
                         CASE WHEN p_ActionName IN ('Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge') THEN 'Receipt Ack' ELSE p_ActionName END
                    );
                IF p_ActionName IS NULL then v_CapabilityCnt := v_CapabilityCnt + 1; END IF;
                IF p_ActivityName IS NULL then v_CapabilityCnt := v_CapabilityCnt + 1; END IF;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'v_CapabilityCnt='||v_CapabilityCnt, p_DebugFlg, v_ExceptionFlg);
                v_ChannelId := v_WkRoutingCapability_Tbl(i).ChannelId;
                EXIT When v_CapabilityCnt = v_CapabilityMaxCnt;
            END Loop;
        END;
    END IF;
END Get_Channel;
Procedure No_Channel as
BEGIN
    IF p_ReturnCd = 3 or v_WkRoutingCapability_Tbl.count = 0 THEN
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'CheckChannel Status:  Channel row NOT Found Fr='||p_FrBusinessId||
                        '  To='||p_ToBusinessId||'  ProcessCd='||p_FrProcessCd||'  ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||']'
                        , p_DebugFlg, v_ExceptionFlg);
        commit;
        p_ReturnCd  := 3;
    ELSIF p_ReturnCd = 5 THEN
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'CheckChannel Status:  Missing lookup Channel data - Channel='||NVL(v_ChannelId, 0)||'  Fr='||p_FrBusinessId||
                                        '  To='||p_ToBusinessId||'  ProcessCd='||p_FrProcessCd||'  ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||']'
                                        , p_DebugFlg, v_ExceptionFlg);
    ELSIF v_WkRoutingCapability_Tbl.count < v_CapabilityMaxCnt THEN
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'CheckChannel Status:  Capabilities NOT Found - Channel='||NVL(v_ChannelId, 0)||'  Fr='||p_FrBusinessId||
                                        '  To='||p_ToBusinessId||'  ProcessCd='||p_FrProcessCd||'  ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||']'
                                        , p_DebugFlg, v_ExceptionFlg);
        p_ReturnCd := 4;
    END IF;
    v_FrPrtRec  := NULL;
    v_ToPrtRec  := NULL;
    v_ChanRec   := NULL;
END No_Channel;
Procedure Debug_Output as
BEGIN
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,'Debug_Output: ');
    FOR i IN 1..p_FrRoutingCapability_Tbl.count LOOP
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,
            CHR(10)||'FROM'||CHR(10)||'------------------------------------------'||CHR(10)||
            'ChannelId       =>'||v_ChannelId||CHR(10)||
            'PartnerId       =>'||p_FrRoutingCapability_Tbl(i).PartnerId||CHR(10)||
            'RoutingLevel    =>'||p_FrRoutingCapability_Tbl(i).RoutingObjLevel||CHR(10)||
            'RoutingObjId    =>'||p_FrRoutingCapability_Tbl(i).RoutingObjId||CHR(10)||
            'RoutingObjType  =>'||p_FrRoutingCapability_Tbl(i).RoutingObjTypeKey||CHR(10)||
            'RoutingObjCd    =>'||p_FrRoutingCapability_Tbl(i).RoutingObjCd||CHR(10)||
            'RoutingObjVer   =>'||p_FrRoutingCapability_Tbl(i).RoutingObjVer||CHR(10)||
            'RoutingObjRefId =>'||p_FrRoutingCapability_Tbl(i).RoutingObjRefId||CHR(10)||
            'CapabilityId    =>'||p_FrRoutingCapability_Tbl(i).CapabilityId
            , p_DebugFlg, v_ExceptionFlg);
    END LOOP;

    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'step 1');

    FOR i IN 1..p_ToRoutingCapability_Tbl.count LOOP
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,
            CHR(10)||'TO  '||CHR(10)||'------------------------------------------'||CHR(10)||
            'ChannelId       =>'||v_ChannelId||CHR(10)||
            'PartnerId       =>'||p_ToRoutingCapability_Tbl(i).PartnerId||CHR(10)||
            'RoutingLevel    =>'||p_ToRoutingCapability_Tbl(i).RoutingObjLevel||CHR(10)||
            'RoutingObjId    =>'||p_ToRoutingCapability_Tbl(i).RoutingObjId||CHR(10)||
            'RoutingObjType  =>'||p_ToRoutingCapability_Tbl(i).RoutingObjTypeKey||CHR(10)||
            'RoutingObjCd    =>'||p_ToRoutingCapability_Tbl(i).RoutingObjCd||CHR(10)||
            'RoutingObjVer   =>'||p_ToRoutingCapability_Tbl(i).RoutingObjVer||CHR(10)||
            'RoutingObjRefId =>'||p_ToRoutingCapability_Tbl(i).RoutingObjRefId||CHR(10)||
            'CapabilityId    =>'||p_ToRoutingCapability_Tbl(i).CapabilityId
            , p_DebugFlg, v_ExceptionFlg);
    END LOOP;

    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'step 2');

    For i in 1..v_RoutingAttr_Tbl.count LOOP
        DBMS_OUTPUT.Put_Line(RPAD(v_RoutingAttr_Tbl(i).AttrLabel, 5)||RPAD(v_RoutingAttr_Tbl(i).AttributeKey, 30)||
            RPAD(v_RoutingAttr_Tbl(i).AttrRank, 5)||RPAD(v_RoutingAttr_Tbl(i).AttrValue, 5));
    END LOOP;

    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'step 3');

    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,CHR(10)||
    'ChannelId                  =>'||  v_ChanRec.CHANNELID||CHR(10)||
    'FrPartnerId                =>'||  v_FrPrtRec.PartnerId||CHR(10)||
    'ToPartnerId                =>'||  v_ToPrtRec.PartnerId||CHR(10)||
    'FrBusinessId               =>'||  v_FrPrtRec.BusinessId||CHR(10)||
    'ToBusinessId               =>'||  v_ToPrtRec.BusinessId||CHR(10)||
    'FrProcessCd                =>'||  v_FrProcessCd||CHR(10)||
    'FrProcessVer               =>'||  v_FrProcessVer||CHR(10)||
    'FrProtocolName             =>'||  v_FrProtocolName||CHR(10)||
    'FrProtocolVer              =>'||  v_FrProtocolVer||CHR(10)||
    'FrPackageName              =>'||  v_FrPackageName||CHR(10)||
    'FrPackageVer               =>'||  v_FrPackageVer||CHR(10)||
    'ToProcessCd                =>'||  v_ToProcessCd||CHR(10)||
    'ToProcessVer               =>'||  v_ToProcessVer||CHR(10)||
    'ToProtocolName             =>'||  v_ToProtocolName||CHR(10)||
    'ToProtocolVer              =>'||  v_ToProtocolVer||CHR(10)||
    'ToPackageName              =>'||  v_ToPackageName||CHR(10)||
    'ToPackageVer               =>'||  v_ToPackageVer||CHR(10)||
    'FrPartnerEnabled           =>'||  v_FrPrtRec.Enabled||CHR(10)||
    'ToPartnerEnabled           =>'||  v_ToPrtRec.Enabled||CHR(10)||
    'InitPartnerTypeName        =>'||  v_FrPrtRec.PartnerTypeName||CHR(10)||
    'InitOwnerFlg               =>'||  v_ToPrtRec.OwnerFlg||CHR(10)||
    'FrPartnerTypeName          =>'||  v_FrPrtRec.PartnerTypeName||CHR(10)||
    'FrOwnerFlg                 =>'||  v_FrPrtRec.OwnerFlg||CHR(10)||
    'ToPartnerTypeName          =>'||  v_ToPrtRec.PartnerTypeName||CHR(10)||
    'ToOwnerFlg                 =>'||  v_ToPrtRec.OwnerFlg||CHR(10)||
    'Enabled                    =>'||  v_ChanRec.Enabled||CHR(10)||
    'ToGatewayEnabled           =>'||  v_ChanRec.ToGatewayEnabled||CHR(10)||
    'FrGatewayId                =>'||  v_ChanRec.FRGATEWAYID||CHR(10)||
    'ToGatewayId                =>'||  v_ChanRec.TOGATEWAYID||CHR(10)||
    'FrGatewayName              =>'||  v_ChanRec.FRGATEWAYNAME||CHR(10)||
    'ToGatewayName              =>'||  v_ChanRec.TOGATEWAYNAME||CHR(10)||
    'FrGatewayEnabled           =>'||  v_ChanRec.FrGatewayEnabled||CHR(10)||
    'ToGatewayEnabled           =>'||  v_ChanRec.ToGatewayEnabled||CHR(10)||
    'ToURI                      =>'||  v_ChanRec.ToURI||CHR(10)||
    'ToURILogin                 =>'||  v_ChanRec.ToURILOGIN||CHR(10)||
    'ToURIPassword              =>'||  v_ChanRec.ToURIPASSWORD||CHR(10)||
    'WorkflowId                 =>'||  v_ChanRec.WORKFLOWID||CHR(10)||
    'ProfileId                  =>'||  v_ChanRec.PROFILEID||CHR(10)||
    'WorkflowName               =>'||  substr(v_ChanRec.WORKFLOWNAME, 255)||CHR(10)||
    'WorkflowDef                =>'||  substr(v_ChanRec.WORKFLOWDEF, 255)||CHR(10)||
    'TransportType              =>'||  v_ChanRec.TRANSPORTTYPE||CHR(10)||
    'FrValidateClientSSLCert    =>'||  v_ChanRec.FRVALIDATECLIENTSSLCERT||CHR(10)||
    'FrValidateClientIP         =>'||  v_ChanRec.FRVALIDATECLIENTIP||CHR(10)||
    'TransformMapId             =>'||  v_ChanRec.TransformMapId||CHR(10)||
    'ToActivityName             =>'||  v_ToActivityName||CHR(10)||
    'ToActionName               =>'||  v_ToActionName
    , p_DebugFlg, v_ExceptionFlg);
END Debug_Output;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_StartMsg, p_DebugFlg, v_ExceptionFlg);
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    IF RC1%isOpen THEN Close RC1; END IF;
    IF RC2%isOpen THEN Close RC2; END IF;
    IF RC3%isOpen THEN Close RC3; END IF;
    p_ReturnCd := 0;

    -- -----------------------------------------------------------
    -- 1.  Lookup Partner info and flip PartnerId's if necessary
    -- -----------------------------------------------------------

    BEGIN
        v_Msg := '------------>> DestType look up FAILED << ';
        SELECT DestTypeId INTO v_DestTypeId FROM PR_DESTTYPE WHERE DestName = p_DestName;
        IF p_InitBusinessId = p_FrBusinessId and p_InitBusinessId != p_ToBusinessId THEN
            v_ChanDirection := 'FR';
        ELSE
            v_ChanDirection := 'TO';
        END IF;
        v_Msg := '------------>> Partner look up FAILED << ';
        select * into v_FrPrtRec from PR_Partner01_vw where BusinessId = p_FrBusinessId;
        select * into v_ToPrtRec from PR_Partner01_vw where BusinessId = p_ToBusinessId;

        IF p_ChannelId IS NULL THEN
            Get_Channel;
        ELSE
            v_ChannelId := p_ChannelId;
            select /*+ FIRST_ROWS */ RoutingObjRefId into v_RoutingObjRefId
            from PR_RoutingChannel Ch
            INNER JOIN PR_Capability C on C.CapabilityId = CASE WHEN v_ChanDirection = 'FR' THEN Ch.FrCapabilityId ELSE Ch.ToCapabilityId END
                                            and v_FrPrtRec.PartnerId = CASE WHEN v_ChanDirection = 'FR' THEN Ch.FrPartnerId ELSE Ch.ToPartnerId END
                                            and v_ToPrtRec.PartnerId = CASE WHEN v_ChanDirection = 'TO' THEN Ch.FrPartnerId ELSE Ch.ToPartnerId END
            Where ChannelId = p_ChannelId;
            PR_CapabilityS_AH5
                (
                v_ReturnCd
                ,p_DebugFlg
                ,v_FrPrtRec.PartnerId
                ,v_RoutingObjRefId
                ,p_FrRoutingCapability_Tbl
                ,1
                );
            v_CapabilityCnt := 5;
        END IF;
        Exception
            WHEN NO_DATA_FOUND THEN
                p_ReturnCd := 3;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_Msg, p_DebugFlg, v_ExceptionFlg);
    END;
    IF p_ReturnCd = 3 OR NVL(v_CapabilityCnt, 0) < 5 THEN --- If v_CapabilityCnt = 5, then all capabilities were found
        No_Channel;
    ELSE
        ----- NOTE: the row coming back from PR_RoutingChannel01_vw is in the original Initiating/From/To representation
        BEGIN
            select /*+ FIRST_ROWS */  * into v_ChanRec
            from PR_RoutingChannel01_vw C
            Where ChannelDirection = v_ChanDirection and ChannelID = v_ChannelId and DestTypeId = v_DestTypeId;
            Exception
                When NO_DATA_FOUND THEN
                    p_ReturnCd := 5;
                    No_Channel;
                WHEN OTHERS then
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH4:'||SQLERRM);
                    ROLLBACK work;
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, '>> Channel view select failed << '||v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
                    Commit;
        END;
        IF p_ReturnCd = 0 THEN
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'CheckChannel Status:  Channel Found = '||v_ChannelId||'  Fr='||p_FrBusinessId||
                                            '  To='||p_ToBusinessId||'  ProcessCd='||p_FrProcessCd||'  ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||']'
                                            , p_DebugFlg, v_ExceptionFlg);
            v_FrCapabilityId := v_ChanRec.FrCapabilityId;
            v_ToCapabilityId := v_ChanRec.ToCapabilityId;
            IF v_ChanDirection = 'TO' THEN
                v_RoutingObjRefId := v_ChanRec.FrRoutingObjRefId;
            ELSE
                v_RoutingObjRefId := v_ChanRec.ToRoutingObjRefId;
            END IF;

            PR_CapabilityS_AH5
                (
                p_ReturnCd
                ,v_DebugFlg
                ,v_ToPrtRec.PartnerId
                ,v_RoutingObjRefId              --v_ChanRec.ToRoutingObjRefId
                ,p_ToRoutingCapability_Tbl
                ,1
                );
                
            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt: Package');
            FrAttrValue('Package', v_FrPackageName, v_FrPackageVer, v_FrPkgRefId);
            --dbms_output.put_line('v_FrPackageName2:'||v_FrPackageName||', v_FrPackageVer2:' || v_FrPackageVer||', v_FrPkgRefId2:' ||v_FrPkgRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt: Protocol');
            FrAttrValue('Protocol', v_FrProtocolName, v_FrProtocolVer, v_FrProtocolRefId);
            --dbms_output.put_line('v_FrProtocolName2:'||v_FrProtocolName||', v_FrProtocolVer2:' || v_FrProtocolVer||', v_FrProtocolRefId2:' ||v_FrProtocolRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt: Process');
            FrAttrValue('Process', v_FrProcessCd, v_FrProcessVer, v_FrProcessRefId);
            --dbms_output.put_line('v_FrProcessCd2:'||v_FrProcessCd||', v_FrProcessVer2:' || v_FrProcessVer||', v_FrProcessRefId2:' ||v_FrProcessRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt:Activity');
            FrAttrValue2('Activity', p_ActivityName, v_FrActivityRefId);
            --dbms_output.put_line('v_FrActivityRefId2:'||v_FrActivityRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt:Action');
            FrAttrValue2('Action', p_ActionName, v_FrActionRefId);
            --dbms_output.put_line('v_FrActionRefId2:'||v_FrActionRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt:Signal');
            FrAttrValue2('Signal', p_Signal, v_FrSignalRefId);
            --dbms_output.put_line('v_FrSignalRefId2:'||v_FrSignalRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt2: Package');
            ToAttrValue('Package', v_ToPackageName, v_ToPackageVer, v_ToPkgRefId);
            --dbms_output.put_line('v_ToPackageName2:'||v_ToPackageName||', v_ToPackageVer2:' || v_ToPackageVer||', v_ToPkgRefId2:' ||v_ToPkgRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt2: Protocol');
            ToAttrValue('Protocol', v_ToProtocolName, v_ToProtocolVer, v_ToProtocolRefId);
            --dbms_output.put_line('v_ToProtocolName2:'||v_ToProtocolName||', v_ToProtocolVe2:' || v_ToProtocolVer||', v_ToProtocolRefId2:' ||v_ToProtocolRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt2: Process');
            ToAttrValue('Process', v_ToProcessCd, v_ToProcessVer, v_ToProcessRefId);
            --dbms_output.put_line('v_ToProcessCd2:'||v_ToProcessCd||', v_ToProcessVer2:' || v_ToProcessVer||', v_ToProcessRefId2:' ||v_ToProcessRefId);
        END IF;
    END IF;


    --IF p_DebugFlg = 1 THEN Debug_Output; END IF;
    -------------------------------------------------------------------------
    -- 4.  Return result set 1 with channel data.
    -------------------------------------------------------------------------
    Open RC1 for
    select
        v_ChanRec.CHANNELID             ChannelId,
        v_FrPrtRec.PartnerId            FrPartnerId,
        v_ToPrtRec.PartnerId            ToPartnerId,
        v_FrPrtRec.BusinessId           FrBusinessId,
        v_ToPrtRec.BusinessId           ToBusinessId,
        v_FrProcessCd                   FrProcessCd,
        v_FrProcessVer                  FrProcessVer,
        v_FrProtocolName                FrProtocolName,
        v_FrProtocolVer                 FrProtocolVer,
        v_FrPackageName                 FrPackageName,
        v_FrPackageVer                  FrPackageVer,
        v_ToProcessCd                   ToProcessCd,
        v_ToProcessVer                  ToProcessVer,
        v_ToProtocolName                ToProtocolName,
        v_ToProtocolVer                 ToProtocolVer,
        v_ToPackageName                 ToPackageName,
        v_ToPackageVer                  ToPackageVer,
        v_FrPrtRec.Enabled              FrPartnerEnabled,
        v_ToPrtRec.Enabled              ToPartnerEnabled,
        v_FrPrtRec.PartnerTypeName      InitPartnerTypeName,
        v_ToPrtRec.OwnerFlg             InitOwnerFlg,
        v_FrPrtRec.PartnerTypeName      FrPartnerTypeName,
        v_FrPrtRec.OwnerFlg             FrOwnerFlg,
        v_ToPrtRec.PartnerTypeName      ToPartnerTypeName,
        v_ToPrtRec.OwnerFlg             ToOwnerFlg,
        v_ChanRec.Enabled               Enabled,
        v_ChanRec.ToGatewayEnabled      ToGatewayEnabled,
        v_ChanRec.FRGATEWAYID           FrGatewayId,
        v_ChanRec.TOGATEWAYID           ToGatewayId,
        v_ChanRec.FRGATEWAYNAME         FrGatewayName,
        v_ChanRec.TOGATEWAYNAME         ToGatewayName,
        v_ChanRec.FrGatewayEnabled      FrGatewayEnabled,
        v_ChanRec.ToGatewayEnabled      ToGatewayEnabled,
        v_ChanRec.ToURI                 ToURI,
        v_ChanRec.ToURILOGIN            ToURILogin,
        v_ChanRec.ToURIPASSWORD         ToURIPassword,
        v_ChanRec.WORKFLOWID            WorkflowId,
        v_ChanRec.ProfileID             ProfileId,
        v_ChanRec.WORKFLOWNAME          WorkflowName,
        v_ChanRec.WORKFLOWDEF           WorkflowDef,
        v_ChanRec.TRANSPORTTYPE         TransportType,
        v_ChanRec.FRVALIDATECLIENTSSLCERT   FrValidateClientSSLCert,
        v_ChanRec.FRVALIDATECLIENTIP        FrValidateClientIP,
        v_ChanRec.TransformMapId        TransformMapId
    from dual;
    -------------------------------------------------------------------------
    --  5.  Return result set 2 with service data.
    -------------------------------------------------------------------------
    Open RC2 for
        select /*+ FIRST_ROWS */
            'FR' SvcSrc,
            RoutingObjCd ServiceName,
            RoutingObjVer ServiceVer
        from
            PR_Capability01_vw
        where PartnerId = v_ChanRec.FrPartnerId and RoutingObjTypeKey = 'Service'
        UNION
        select /*+ FIRST_ROWS */
            'TO' SvcSrc,
            RoutingObjCd ServiceName,
            RoutingObjVer ServiceVer
        from
            PR_Capability01_vw
        where PartnerId = v_ChanRec.ToPartnerId and RoutingObjTypeKey = 'Service';
    -- NOTE: If there is no Transform Map, then use the From Activity and Action, else use the actual TO Activity and Action.
    --       It is assumed here that if a Transform Map is present, then specific attributes and guidelines should be
    --       returned in the RC3 result set.
    --  6.  Return result set 3 with TPA attribute data.  This data is retrieved based
    --      upon attributes tied to the process, activity or actions set at either
    --      the global, partner or channel override levels.
--    IF v_ChanRec.TransformMapId IS NOT NULL THEN
        BEGIN
            select RoutingObjCd, RoutingObjRefId into v_ToActivityName, v_ToActivityRefId
            from TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
            Where RoutingObjTypeKey = 'Activity' and rownum < 2;
        Exception
            When NO_DATA_FOUND Then
                v_ToActivityName := NULL;
                v_ToActivityRefId := NULL;
            WHEN OTHERS then
                RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH4:'||SQLERRM);
                ROLLBACK work;
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
        END;
        BEGIN
            select RoutingObjCd, RoutingObjRefId  into v_ToActionName, v_ToActionRefId
            from TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
            Where RoutingObjTypeKey = 'Action' and rownum < 2;
        Exception
            When NO_DATA_FOUND Then
                v_ToActionName := NULL;
                v_ToActionRefId := NULL;
            WHEN OTHERS then
                RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH4:'||SQLERRM);
                ROLLBACK work;
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
        END;
        BEGIN
            select RoutingObjCd, RoutingObjRefId into v_ToSignal, v_ToSignalRefId
            from TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
            Where RoutingObjTypeKey = 'Signal' and RoutingObjCd = p_Signal and rownum < 2;
        Exception
            When NO_DATA_FOUND Then
                v_ToSignalRefId := NULL;
            WHEN OTHERS then
                RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH4:'||SQLERRM);
                ROLLBACK work;
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
        END;
--    END IF;
    IF p_ReturnCd = 0 THEN
        Begin
            select AttrLabel, AttributeKey, AttrRank, AttrValue
            BULK COLLECT into v_RoutingAttr_Tbl
            From
            (
                select /*+ FIRST_ROWS */ distinct
                    'FR' AttrLabel,
                    AttributeKey,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  v_FrPrtRec.PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrRank') AttrRank,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  v_FrPrtRec.PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrValue') AttrValue
                from
                    TABLE(CAST (p_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
                    INNER JOIN PR_RoutingAttr_Lnk  Ra on Ra.RoutingObjRefId = T.RoutingObjRefId
                    INNER JOIN CF_RoutingAttr A on Ra.RoutingAttrId = A.RoutingAttrId
                Where T.RoutingObjRefId in (v_FrPkgRefId, v_FrProtocolRefId, v_FrProcessRefId, v_FrActivityRefId, v_FrActionRefId, v_FrSignalRefId)
                      AND (   (Ra.PartnerId IS NULL and Ra.ChannelID is NULL and Ra.DestTypeId IS NULL)
                           OR (Ra.PartnerId = v_FrPrtRec.PartnerId and Ra.ChannelID IS NULL)
                           OR (Ra.ChannelId = v_ChannelId)
                           OR (Ra.ChannelId = v_ChannelId AND Ra.DestTypeId = v_DestTypeId)
                          )
                UNION ALL
                select /*+ FIRST_ROWS */ distinct
                    'TO' AttrLabel,
                    AttributeKey,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  v_ToPrtRec.PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrRank') AttrRank,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  v_ToPrtRec.PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrValue') AttrValue
                from
                    TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
                    INNER JOIN PR_RoutingAttr_Lnk  Ra on Ra.RoutingObjRefId = T.RoutingObjRefId
                    INNER JOIN CF_RoutingAttr A on Ra.RoutingAttrId = A.RoutingAttrId
                Where T.RoutingObjRefId in (v_ToPkgRefId, v_ToProtocolRefId, v_ToProcessRefId, v_ToActivityRefId, v_ToActionRefId, v_ToSignalRefId)
                      AND (   (Ra.PartnerId IS NULL and Ra.ChannelID is NULL and Ra.DestTypeId IS NULL)
                           OR (Ra.PartnerId = v_ToPrtRec.PartnerId and Ra.ChannelID IS NULL)
                           OR (Ra.ChannelId = v_ChannelId)
                           OR (Ra.ChannelId = v_ChannelId AND Ra.DestTypeId = v_DestTypeId)
                          )
            );
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'RC3 Attribute Select NO_DATA_FOUND: '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                    commit;
                WHEN OTHERS then
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH4:'||SQLERRM);
                    ROLLBACK work;
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, '>> Attribute Rank Lookup failed << '||CHR(10)||v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
                    Commit;
        END;
        For i in 1..v_RoutingAttr_Tbl.count LOOP
            p_RoutingAttr_Tbl.extend;
            p_RoutingAttr_Tbl(i) := PR_RoutingAttr_Rec(v_RoutingAttr_Tbl(i).AttrLabel, v_RoutingAttr_Tbl(i).AttributeKey,
                                                       v_RoutingAttr_Tbl(i).AttrRank,v_RoutingAttr_Tbl(i).AttrValue);
        END LOOP;
    END IF;
    Open RC3 for
        select distinct
            T.AttrLabel,
            T.AttributeKey,
            T.AttrValue
        from
            TABLE(CAST (p_RoutingAttr_Tbl AS PR_RoutingAttr_Tbl)) T,
            (Select AttrLabel, AttributeKey, MAX(TO_NUMBER(AttrRank)) AttrRank from TABLE(CAST (p_RoutingAttr_Tbl AS PR_RoutingAttr_Tbl))
             group by AttrLabel, AttributeKey) T2
        Where T.AttrLabel = T2.AttrLabel and T.AttributeKey = T2.AttributeKey and T.AttrRank = T2.AttrRank;
    IF p_DebugFlg = 1 THEN Debug_Output; END IF;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_EndMsg, p_DebugFlg, v_ExceptionFlg);
End;
/


-- PR_ROUTINGCHANNELS_AH5.sql

CREATE OR REPLACE PROCEDURE         PR_RoutingChannelS_AH5
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
    , RC1  out  db_ref.RCT
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RoutingChannelS_AH5
    Create Date:     2006-04-22 10:07:30
    Author:          S Narain

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	
	2008-09-12 Amit Kumar 	 6.2	Defect 47980, 106065 and 106066

     ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_ROUTINGCHANNELS';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
  	Select
          pr.CHANNELID
					,pr.PARENTCHANNELID
					,pr.FRCAPABILITYID 
					,pr.TOCAPABILITYID 
					,pr.FRPARTNERID    
					,pr.TOPARTNERID    
					,pr.TRANSFORMMAPID 
					,pr.WORKFLOWID     
					,(CASE WHEN pr.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END) ACTIVESTATUSCD 
					,pr.MULTICASTFLG   
					,pr.ROWTS          
					,pr.CREATEDATE     
					,pr.UPDATEDATE     
					,pr.PROFILEID      

      From
          PR_ROUTINGCHANNEL pr,pr_partner pp, pr_partner pp1
      Where
          (CASE WHEN pr.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END) in (0,1)
and pp1.partnerid=pr.topartnerid
and pp.enabled<>2
and pp1.enabled<>2
and pp.partnerid=pr.frpartnerid;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --  Commit;

    EXCEPTION
        WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_RoutingChannelS_AH5:'||SQLERRM);

End PR_RoutingChannelS_AH5;
/


-- PR_ROUTINGCHANNELS_AH6.sql

CREATE OR REPLACE PROCEDURE         PR_RoutingChannelS_AH6
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
  --  , p_Locale in Varchar2
    , RC1  out  db_ref.RCT
    , p_FrCapabilityId in NUMBER
    , p_ToCapabilityId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RoutingChannelS_AH6
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns count of existing channels by given input
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RoutingChannelS_AH6';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
--        ||','||p_Locale||CHR(10)
        ||', p_FrCapabilityId =>'||p_FrCapabilityId ||CHR(10)
        ||', p_ToCapabilityId=>'||p_ToCapabilityId ||CHR(10)
        ;

--v_Locale PR_Locale.Locale%Type := nvl(p_Locale, FN_GET_DEFAULT_LOCALE);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
              ChannelId,
              FrCapabilityId,
              ToCapabilityId,
              WorkFlowId,
              TransformMapId,
              ProfileId,
              CASE WHEN ACTIVESTATUSCD=63 THEN 1 ELSE 0 END ACTIVESTATUSCD,
              RowTS
    From      PR_RoutingChannel
    Where     FrCapabilityId = p_FrCapabilityId
    And       ToCapabilityId = p_ToCapabilityId
    ;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --   Commit;

    EXCEPTION
         When OTHERS Then
            Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_RoutingChannelS_AH6' || SQLCODE|| ' '||SQLERRM);

End PR_RoutingChannelS_AH6;
/


-- PR_ROUTINGCHANNELS_AH7.sql

CREATE OR REPLACE PROCEDURE PR_RoutingChannelS_AH7
    (
    p_ReturnCd  out  integer
    , p_DebugFlg  in  pls_integer
 --   , p_Locale in Varchar2
    , RC1  out  db_ref.RCT
    , p_FrPartnerId in NUMBER
    , p_ToPartnerId in NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_RoutingChannelS_AH7
    Create Date:     2003-04-22 10:07:30
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Returns all channels by given Fr/To Partner
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_RoutingChannelS_AH7';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
 --       ||','||p_Locale||CHR(10)
        ||', p_FrPartnerId =>'||p_FrPartnerId||CHR(10)
        ||', p_ToPartnerId=>'||p_ToPartnerId ||CHR(10)
        ;

-- v_Locale PR_Locale.Locale%Type := nvl(p_Locale, FN_GET_DEFAULT_LOC);

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 for
    select
            c.ChannelId
            ,c.FrCapabilityId
            ,c.ToCapabilityId
            ,c.WorkFlowId
            ,c.TransformMapId
            ,c.ProfileId
            ,CASE WHEN c.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END ACTIVESTATUSCD
            ,c.MultiCastFlg
            ,c.RowTS
    From   PR_RoutingChannel c
         Inner Join PR_Capability fp On fp.CapabilityId = c.FrCapabilityId
         Inner Join PR_Capability tp On tp.CapabilityId = c.ToCapabilityId
         INNER JOIN cf_routingobjref_lnk cl on cl.ROUTINGOBJREFID = fp.ROUTINGOBJREFID 
	 INNER JOIN cf_routingobject cr on cr.ROUTINGOBJID = cl.ROUTINGOBJID
    where 
          cr.activestatuscd=1 
     AND  fp.PartnerId = p_FrPartnerId
     And  tp.PartnerId = p_ToPartnerId
    ;


    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
  --  Commit;

    EXCEPTION
         When OTHERS Then
            Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_RoutingChannelS_AH7' || SQLCODE|| ' '||SQLERRM);

End PR_RoutingChannelS_AH7;
/


-- PR_ROUTINGCHANNELS_AH8.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELS_AH8
	(
	  p_ReturnCd		OUT PLS_INTEGER
	, p_debugFlg		IN  PLS_INTEGER
	, RC1			OUT DB_REF.RCT   --(single row channel data)
	, p_InitBusinessId	IN VARCHAR2
	, p_FrBusinessId	IN VARCHAR2
	, p_ToBusinessId	IN VARCHAR2
	, p_FrPackageName	IN VARCHAR2
	, p_FrPackageVer	IN VARCHAR2
	, p_FrProtocolName	IN VARCHAR2
	, p_FrProtocolVer	IN VARCHAR2
	, p_FrProcessCd		IN VARCHAR2
	, p_FrProcessVer	IN VARCHAR2
	, p_DestName		IN VARCHAR2
	, p_ActivityName	IN VARCHAR2 DEFAULT NULL
	, p_ActionName		IN VARCHAR2 DEFAULT NULL
	)
AS
/*
	================================================================================
	Proc Name:	  PR_ROUTINGCHANNELS_AH8
	Create Date:	2004-09-23
	================================================================================
	Return Code : 0 = OK, 1 = Error Exception, 2 = Dup,
		  3= No Channel row Found, 4= 1 or more Capabilities NOT found (i.e. v_CapabilityCnt < 5),
		  5= Channel found, but missing 1 or more data lookups (i.e. Gateway or Workflow)
                  6= From Partner Business ID is not correct
		  7= To Partner Business ID is not correct
		  8= Both From and To Partner Business ID are not correct
		  9= Package Name/Version Incorrect
		 10= Protocol Name/Version Incorrect
		 11= Process Name/Version Incorrect
		 13= Package Capability not enabled
		 14= Protocol Capability not enabled
                 15= Process Capability not enabled

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	
	2008-09-12 Amit Kumar 	 6.2	Defect 47980, 106065 and 106066
                 
--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
	v_RowCnt		INTEGER := 0;
	v_BeginTS		TIMESTAMP := FN_Get_Date; -- Performance related variables
	v_ProcName		VARCHAR2(30) := 'PR_ROUTINGCHANNELS_AH8';
	v_ExceptionFlg		INTEGER := 0;
	v_ReturnCd		PLS_INTEGER := 0;
	v_chkPt		 INTEGER :=0;
	v_SumchkPt	      INTEGER := 5;
	v_CapabilityMaxCnt	NUMBER(2) := 5; -- Max Number Channel Object levels
	v_FrPartnerId		PR_Partner01_vw.PartnerId%TYPE;
	v_ToPartnerId		PR_Partner01_vw.PartnerId%TYPE;
	v_ChanDirection		VARCHAR2(2);
	v_DebugFlg		NUMBER(1) := p_DebugFlg;
	v_SecReturnCd       INTEGER :=0;
	IncorrectData		EXCEPTION;
	
	TYPE v_RoutingCapability_Rec is RECORD
	(
		ChannelId		NUMBER(10),
		FrPartnerId		NUMBER(10),
		ToPartnerId		NUMBER(10),
		RoutingObjLevel		NUMBER(6),
		RoutingObjId		NUMBER(10),
		RoutingObjTypeKey	VARCHAR2(60),
		RoutingObjCd		VARCHAR2(255),
		RoutingObjVer		VARCHAR2(60),
		RoutingObjRefId		NUMBER(10),
		CapabilityId		NUMBER(10)
	);
	TYPE v_RoutingCapability_T IS TABLE OF v_RoutingCapability_Rec INDEX BY PLS_INTEGER;

	v_WkRoutingCapability_Tbl	v_RoutingCapability_T;
	vt_RtCap			IDTable := IDTable();
	v_FrRoutingCapability_Tbl	PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
	v_CapabilityCnt			number(6) := 0;
	v_ChannelId			PR_RoutingChannel.ChannelId%TYPE;
	v_RoutingObjRefId		CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;

	v_Msg				varchar2(128) := CHR(10)||'-----------------------  Start Channel Search -----------------------';
	v_ParamsTxt			varchar2(4000) :=
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_InitBusinessId=>'''||p_InitBusinessId||''''||CHR(10)
		||', p_FrBusinessId=>'''||p_FrBusinessId||''''||CHR(10)
		||', p_ToBusinessId=>'''||p_ToBusinessId||''''||CHR(10)
		||', p_FrPackageName=>'''||p_FrPackageName||''''||CHR(10)
		||', p_FrPackageVer=>'''||p_FrPackageVer||''''||CHR(10)
		||', p_FrProtocolName=>'''||p_FrProtocolName||''''||CHR(10)
		||', p_FrProtocolVer=>'''||p_FrProtocolVer||''''||CHR(10)
		||', p_FrProcessCd=>'''||p_FrProcessCd||''''||CHR(10)
		||', p_FrProcessVer=>'''||p_FrProcessVer||''''||CHR(10)
		||', p_ActivityName=>'''||p_ActivityName||''''||CHR(10)
		||', p_ActionName=>'''||p_ActionName||''''||CHR(10);

PROCEDURE Get_Channel AS
BEGIN
	BEGIN
	DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'Direction = '||v_ChanDirection||'  Fr/To Partner='||v_FrPartnerId||'/'||v_ToPartnerId, p_DebugFlg, v_ExceptionFlg);
	-----------------------------------------------------------
	--  2. Look for the root PR_RoutingChannel row at either the
	--	 Process, Activity or action Level
	-----------------------------------------------------------
	SELECT /*+ FIRST_ROWS */
		Ch.ChannelId,
		Ch.FrPartnerId FrPartnerId,
		Ch.ToPartnerId ToPartnerId,
		RoutingObjLevel,
		RoutingObjId,
		RoutingObjTypeKey,
		RoutingObjCd,
		RoutingObjVer,
		R.RoutingObjRefId,
		C.CapabilityId
	BULK COLLECT INTO v_WkRoutingCapability_Tbl
	FROM
		CF_RoutingObject04_vw R
		INNER JOIN PR_Capability C ON C.RoutingObjRefId = R.RoutingObjRefId
		INNER JOIN PR_RoutingChannel Ch ON C.CapabilityId = CASE WHEN v_ChanDirection = 'FR' THEN Ch.FrCapabilityId ELSE Ch.ToCapabilityId END
						AND v_FrPartnerId = CASE WHEN v_ChanDirection = 'FR' THEN Ch.FrPartnerId ELSE Ch.ToPartnerId END
						AND v_ToPartnerId = CASE WHEN v_ChanDirection = 'TO' THEN Ch.FrPartnerId ELSE Ch.ToPartnerId END
	WHERE
		R.DocumentFlg = 1
		AND	((RoutingObjTypeKey = 'Process' AND RoutingObjCd = p_FrProcessCd AND RoutingObjVer = p_FrProcessVer)
		OR	(RoutingObjTypeKey = 'Activity' AND RoutingObjCd = p_ActivityName)
		OR	(RoutingObjTypeKey = 'Action' AND
			CASE WHEN RoutingObjCd IN ('Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge') THEN 'Receipt Ack' ELSE RoutingObjCd END =
			CASE WHEN p_ActionName IN ('Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge') THEN 'Receipt Ack' ELSE p_ActionName END
		))
		AND	CASE WHEN ch.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END = 1
		AND	C.PartnerId = v_FrPartnerId;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			p_ReturnCd := 3;
	END;
	IF p_ReturnCd = 0 THEN

	-----------------------------------------------------------
	--  3.  Using the previous channel result set, find a channel
	--	that has all of matching capabilities using the input
	--	params passed in.
	-----------------------------------------------------------
	BEGIN
		FOR i IN 1..v_WkRoutingCapability_Tbl.count LOOP
		DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'--> Channels being searched .... '||v_WkRoutingCapability_Tbl(i).ChannelId, p_DebugFlg, v_ExceptionFlg);
		PR_CapabilityS_AH5
			(
			v_ReturnCd
			,v_DebugFlg
			,v_FrPartnerId
			,v_WkRoutingCapability_Tbl(i).RoutingObjRefId
			,v_FrRoutingCapability_Tbl
			,1
			);
		--- Count how many parameters match the capabilities returned from PR_CapabilityS_AH5
		v_CapabilityCnt := 0;
		v_ChannelId := v_WkRoutingCapability_Tbl(i).ChannelId;

		SELECT  COUNT(*)
		INTO	v_CapabilityCnt
		FROM	TABLE(CAST (v_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl))
		WHERE
			(RoutingObjTypeKey = 'Package' and RoutingObjCd = p_FrPackageName AND RoutingObjVer = p_FrPackageVer) OR
			(RoutingObjTypeKey = 'Protocol' and RoutingObjCd = p_FrProtocolName and RoutingObjVer = p_FrProtocolVer) OR
			(RoutingObjTypeKey = 'Process' and RoutingObjCd = p_FrProcessCd and RoutingObjVer = p_FrProcessVer) OR
			(RoutingObjTypeKey = 'Activity' and RoutingObjCd = p_ActivityName) OR
			(RoutingObjTypeKey = 'Action' and
			CASE WHEN RoutingObjCd IN ('Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge') THEN 'Receipt Ack' ELSE RoutingObjCd END =
			 CASE WHEN p_ActionName IN ('Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge') THEN 'Receipt Ack' ELSE p_ActionName END
			);
		IF p_ActionName IS NULL THEN v_CapabilityCnt := v_CapabilityCnt + 1; END IF;
		IF p_ActivityName IS NULL THEN v_CapabilityCnt := v_CapabilityCnt + 1; END IF;
		DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'v_CapabilityCnt='||v_CapabilityCnt, p_DebugFlg, v_ExceptionFlg);
		IF v_CapabilityCnt < v_CapabilityMaxCnt then
			-- Delete this channel
			v_WkRoutingCapability_Tbl(i).ChannelID := -1;
		ELSE
			-- increment count of valid channels
			v_RowCnt := v_RowCnt + 1;
		END IF;
		END Loop;
	END;
	END IF;
END Get_Channel;

PROCEDURE No_Channel AS
BEGIN
	IF p_ReturnCd = 3 THEN
		DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'CheckChannel Status:  Channel row NOT Found Fr='||p_FrBusinessId||
			'  To='||p_ToBusinessId||'  ProcessCd='||p_FrProcessCd||'  ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||']'
			, p_DebugFlg, v_ExceptionFlg);
	ELSIF p_ReturnCd = 5 THEN
		DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'CheckChannel Status:  Missing lookup Channel data - Channel='||NVL(v_ChannelId, 0)||'  Fr='||p_FrBusinessId||
					'  To='||p_ToBusinessId||'  ProcessCd='||p_FrProcessCd||'  ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||']'
					, p_DebugFlg, v_ExceptionFlg);
	ELSIF v_RowCnt < 1 THEN
		DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'CheckChannel Status:  Capabilities NOT Found - Channel='||NVL(v_ChannelId, 0)||'  Fr='||p_FrBusinessId||
					'  To='||p_ToBusinessId||'  ProcessCd='||p_FrProcessCd||'  ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||']'
					, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 4;
	END IF;
END No_Channel;

---------------------------------------------------------------------------
--			 M	A	I	N				  --
---------------------------------------------------------------------------
BEGIN
	DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_Msg, p_DebugFlg, v_ExceptionFlg);
	DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	IF RC1%isOpen THEN Close RC1; END IF;
	p_ReturnCd := 0;

	-- -----------------------------------------------------------
	-- 1.  Lookup Partner info and flip PartnerId's if necessary
	-- -----------------------------------------------------------

	IF p_InitBusinessId = p_FrBusinessId AND p_InitBusinessId != p_ToBusinessId THEN
		v_ChanDirection := 'FR';
	ELSE
		v_ChanDirection := 'TO';
	END IF;
	v_Msg := '------------>> Partner look up FAILED << ';
		-- Check count initialised to 5. If FROM business id is wrong add 1 to check count and continue
	        -- If TO business id is wrong add 2 to check count
                -- So 6 means wrong FROM, 7 means wrong TO, 8 means both FROM and TO wrong
	BEGIN
		SELECT	PartnerId
		INTO	v_FrPartnerId
		FROM	PR_Partner01_vw
		WHERE	BusinessId = p_FrBusinessId;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			v_SumchkPt := v_SumchkPt + 1;
	END;
	BEGIN
		SELECT	PartnerId
		INTO	v_ToPartnerId
		FROM	PR_Partner01_vw
		WHERE	BusinessId = p_ToBusinessId;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			v_SumchkPt := v_SumchkPt + 2;
	END;
	IF v_SumChkPt > 5 then
		p_ReturnCd := v_SumChkPt;
		RAISE IncorrectData;
	END IF;

	Get_Channel;
	No_Channel;

	IF p_ReturnCd <> 0 then RAISE IncorrectData; END IF;
	-------------------------------------------------------------------------
	-- 4.  Return result set 1 with channel data.
	-------------------------------------------------------------------------
	For i IN v_WkRoutingCapability_Tbl.FIRST..v_WkRoutingCapability_Tbl.LAST LOOP
		BEGIN
			vt_RtCap.extend;
			vt_RtCap(i) := IDREC(null);
			vt_RtCap(i).ID := v_WkRoutingCapability_Tbl(i).ChannelId;
		EXCEPTION
			WHEN Subscript_Beyond_Count THEN
				NULL;
		END;
	END LOOP;

	OPEN RC1 FOR
		SELECT	id AS channelid
		FROM	TABLE(CAST(vt_RtCap as IDTable))
		WHERE	id <> -1;
EXCEPTION
	WHEN IncorrectData THEN
		IF p_ReturnCd IN (3,4) THEN
			PR_RoutingChannelS_AH10(v_SecReturnCd
					, v_FrPartnerId
					, p_FrPackageName
					, p_FrPackageVer
					, p_FrProtocolName
					, p_FrProtocolVer
					, p_FrProcessCd
					, p_FrProcessVer
					, p_ActivityName
					, p_ActionName);
			IF v_SecReturnCd > 8 THEN
				p_ReturnCd := v_SecReturnCd;
			END IF;
		END IF;
	WHEN OTHERS then
		RAISE_APPLICATION_ERROR(-20999,'20100 - '||v_ProcName||': '||SQLERRM);
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
END;
/

-- PR_ROUTINGCHANNELS_AH9.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELS_AH9
    (
      p_ReturnCd            out pls_integer
    , p_debugFlg            in  pls_integer
    , RC1                   out db_ref.rct   --(single row channel data)
    , RC2                   out db_ref.rct   --(To/Fr attribute data)
    , RC3                   out db_ref.rct   --(To/Fr Services)
    , p_InitBusinessId      in varchar2
    , p_FrBusinessId        in varchar2
    , p_ToBusinessId        in varchar2
    , p_DestName            in varchar2
    , p_ActivityName        in varchar2 DEFAULT NULL
    , p_ActionName          in varchar2 DEFAULT NULL
    , p_ChannelId           in number      
    , p_Signal              in varchar2 default null   -- Optional
    )
AS
/*
    ================================================================================
    Proc Name:      PR_ROUTINGCHANNELS_AH9
    Create Date:    2003-04-23
    Author:         B.Moffat
    Description:    Retrieve From/To Channel information based upon Package, Protocol
                    Process, Activity and Action inputs.
    2006-09-27      S Narain  6.1  CR102859
            Returns the following Result sets:
            ---------------------------------
            1.  Channel attribures in a single row
            2.  All to/from services associated with this channel
            3.  All to/from TPA attributes
            Dependencies:  PR_CapabilityS_AH5
                           CF_RoutingObject04_vw
                           PR_Capability01_vw
                           PR_Capability03_vw
                           PR_Partner01_vw
                           PR_RoutingChannel03_vw
            Logic Steps
            -----------
            1.  Lookup Partner info and flip PartnerId's if necessary
            2.  Look for the root PR_RoutingChannel row at either the
                Process, Activity or action Level.
            3.  Using the previous channel result set, loop thru and find a channel
                that has all of matching capabilities using the input
                params passed in.
            4.  Return result set 1 with channel data.
            5.  Return result set 2 with service data.
            6.  Return result set 3 with TPA attribute data.  This data is retrieved based
                upon attributes tied to the process, activity or actions set at either
                the global, partner or channel override levels.
            NOTE:  A filter was added for all the differing spellings of Receipt
                   Acknowledgement.  This action will be queried using IN
                   'Receipt Acknowledgment', 'Receipt Acknowledgement', 'Receipt Acknowledge'
                   to catch any spelling variations.

            IMPORTANT:  Currently the max number of routing object levels
            ---------   necessary for channel validation is 5.  This value
                        is stored in v_CapabilityMaxCnt.

    ================================================================================
    Return Code : 0 = OK, 1 = Error Exception, 2 = Dup,
                  3= No Channel row Found, 4= 1 or more Capabilities NOT found (i.e. v_CapabilityCnt < 5),
                  5= Channel found, but missing 1 or more data lookups (i.e. Gateway or Workflow)

--================================================================================
-- *
-- *
--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt            integer :=0;
v_BeginTS           timestamp := FN_Get_Date; -- Performance related variables
v_ProcName          varchar2(30) := 'PR_ROUTINGCHANNELS_AH9';
v_ExceptionFlg      integer := 0;
v_ReturnCd          pls_integer := 0;

v_CapabilityMaxCnt  NUMBER(2) := 5; -- Max Number Channel Object levels
v_FrCapabilityId    PR_Capability.CapabilityId%TYPE;
v_ToCapabilityId    PR_Capability.CapabilityId%TYPE;
v_FrPrtRec          PR_Partner01_vw%ROWTYPE;
v_ToPrtRec          PR_Partner01_vw%ROWTYPE;
v_ChanRec           PR_RoutingChannel01_vw%ROWTYPE := NULL;
v_SvcRec            PR_Capability03_vw%ROWTYPE;
v_ChanDirection     varchar2(2);
v_DebugFlg          number(1) := p_DebugFlg;
v_RoutingObjCd      CF_RoutingObject.RoutingObjCd%TYPE;
v_RoutingObjVer     CF_RoutingObject.RoutingObjVer%TYPE;
v_FrPackageName     CF_RoutingObject.RoutingObjCd%TYPE;
v_FrPackageVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_FrPkgRefId        CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrProtocolName    CF_RoutingObject.RoutingObjCd%TYPE;
v_FrProtocolVer     CF_RoutingObject.RoutingObjVer%TYPE;
v_FrProtocolRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrProcessCd       CF_RoutingObject.RoutingObjCd%TYPE;
v_FrProcessVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_FrProcessRefId    CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrActivityRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrActionRefId     CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_FrSignalRefId     CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToPackageName     CF_RoutingObject.RoutingObjCd%TYPE;
v_ToPackageVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_ToPkgRefId        CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToProtocolName    CF_RoutingObject.RoutingObjCd%TYPE;
v_ToProtocolVer     CF_RoutingObject.RoutingObjVer%TYPE;
v_ToProcessCd       CF_RoutingObject.RoutingObjCd%TYPE;
v_ToProcessVer      CF_RoutingObject.RoutingObjVer%TYPE;
v_ToActivityName    CF_RoutingObject.RoutingObjCd%TYPE := p_ActivityName;
v_ToActionName      CF_RoutingObject.RoutingObjCd%TYPE := p_ActionName;
v_ToSignal         CF_RoutingObject.RoutingObjCd%TYPE := p_ActionName;
v_ToProtocolRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToProcessRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToActivityRefId   CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToActionRefId     CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_ToSignalRefId     CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
v_Msg               varchar2(128);
j                   number(6) := 0;
v_DestTypeId        PR_DESTTYPE.DESTTYPEID%TYPE;
v_FrActivityName    CF_RoutingObject.RoutingObjCd%TYPE ;
v_FrActionName      CF_RoutingObject.RoutingObjCd%TYPE ;
v_frbusinessidname  pr_businessidtype.BUSINESSIDNAME%TYPE;
v_Tobusinessidname  pr_businessidtype.BUSINESSIDNAME%TYPE;

v_certsetid_encryption_temp INTEGER;
v_certsetid_sslclient_temp  INTEGER;
v_certsetid_signing_temp    INTEGER;

v_channeldesttypeid INTEGER;
v_certsetid_encryption VARCHAR(10) ;
v_certsetid_signing VARCHAR(10);
v_certsetid_sslclient VARCHAR(10) ;
v_encryption_flg INTEGER;
v_certsetid_encryption_name VARCHAR(60) ;
v_sslclient_flg INTEGER;
v_certsetid_sslclient_name VARCHAR(60) ;
v_digsigflg INTEGER;
v_certsetid_signing_name VARCHAR(60);
 
v_RC1 		    db_ref.RCT;


TYPE v_RoutingCapability_Rec is Record
(
    ChannelId               number(10),
    FrPartnerId             number(10),
    ToPartnerId             number(10),
    RoutingObjLevel         number(6),
    RoutingObjId            number(10),
    RoutingObjTypeKey       varchar2(60),
    RoutingObjCd            varchar2(255),
    RoutingObjVer           varchar2(60),
    RoutingObjRefId         number(10),
    CapabilityId            number(10)
);
TYPE v_RoutingCapability_T is table of v_RoutingCapability_Rec index by pls_integer;
v_WkRoutingCapability_Tbl v_RoutingCapability_T;
p_FrRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
p_ToRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
v_TmpRoutingCapability_Tbl PR_RoutingCapability_Tbl := PR_RoutingCapability_Tbl();
v_CapabilityCnt             number(6) := 0;
v_ChannelId                 PR_RoutingChannel.ChannelId%TYPE;
v_RoutingObjRefId           CF_RoutingObjRef_Lnk.RoutingObjRefId%TYPE;
TYPE v_RoutingAttr_Rec is Record
(
    AttrLabel               Varchar2(2),
    AttributeKey            varchar2(60),
    AttrRank                number(6),
    AttrValue               varchar2(255)
);
TYPE v_RoutingAttr_T is table of v_RoutingAttr_Rec index by pls_integer;
v_RoutingAttr_Tbl   v_RoutingAttr_T;
p_RoutingAttr_Tbl PR_RoutingAttr_Tbl := PR_RoutingAttr_Tbl();
v_RA_Rec            v_RoutingAttr_Rec;
v_StartMsg          varchar2(80) := CHR(10)||'-----------------------  Start Channel Search -----------------------';
v_EndMsg            varchar2(80) := CHR(10)||'-----------------------  END   Channel Search -----------------------';
v_ParamsTxt         varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
    ||', p_InitBusinessId=>'''||p_InitBusinessId||''''||CHR(10)
    ||', p_FrBusinessId=>'''||p_FrBusinessId||''''||CHR(10)
    ||', p_ToBusinessId=>'''||p_ToBusinessId||''''||CHR(10)
    ||', p_ActivityName=>'''||p_ActivityName||''''||CHR(10)
    ||', p_ActionName=>'''||p_ActionName||''''||CHR(10)
    ||', p_DestName=>'||p_DestName||CHR(10)
    ||', p_ChannelId=>'||p_ChannelId||CHR(10);
    
Procedure FrAttrValue(v_RoutingObjTypeKey in varchar2, v_FrName out varchar2, v_FrVer out varchar2, v_FrRefId out pls_integer)
AS
BEGIN

    SELECT RoutingObjCd, RoutingObjVer, RoutingObjRefId INTO v_FrName, v_FrVer, v_FrRefId
    FROM TABLE(CAST (p_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
    WHERE RoutingObjTypeKey = v_RoutingObjTypeKey;

    EXCEPTION
    WHEN OTHERS THEN NULL;              
END;

Procedure ToAttrValue(v_RoutingObjTypeKey in varchar2, v_ToName out varchar2, v_ToVer out varchar2, v_ToRefId out pls_integer)
AS
BEGIN
    SELECT RoutingObjCd, RoutingObjVer, RoutingObjRefId into v_ToName, v_ToVer, v_ToRefId
    FROM TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
    Where RoutingObjTypeKey = v_RoutingObjTypeKey;

    EXCEPTION
    WHEN OTHERS THEN NULL;              
END;

Procedure FrAttrValue2(v_RoutingObjTypeKey in varchar2, p_ObjCd in varchar2, v_FrRefId out pls_integer)
AS
BEGIN
    IF p_ObjCd IS NOT NULL then
    BEGIN
        select RoutingObjRefId into v_FrRefId
        FROM TABLE(CAST (p_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
        Where RoutingObjTypeKey = v_RoutingObjTypeKey and RoutingObjCd = p_ObjCd  and rownum < 2;
        exception
            When NO_DATA_FOUND Then v_FrRefId := NULL;
            when others then
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'SQLERRM: ' || SQLERRM);
    END;
    END IF;
END;
    

Procedure Debug_Output as
BEGIN
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,'Debug_Output: ');
    FOR i IN 1..p_FrRoutingCapability_Tbl.count LOOP
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,
            CHR(10)||'FROM'||CHR(10)||'------------------------------------------'||CHR(10)||
            'ChannelId       =>'||v_ChannelId||CHR(10)||
            'PartnerId       =>'||p_FrRoutingCapability_Tbl(i).PartnerId||CHR(10)||
            'RoutingLevel    =>'||p_FrRoutingCapability_Tbl(i).RoutingObjLevel||CHR(10)||
            'RoutingObjId    =>'||p_FrRoutingCapability_Tbl(i).RoutingObjId||CHR(10)||
            'RoutingObjType  =>'||p_FrRoutingCapability_Tbl(i).RoutingObjTypeKey||CHR(10)||
            'RoutingObjCd    =>'||p_FrRoutingCapability_Tbl(i).RoutingObjCd||CHR(10)||
            'RoutingObjVer   =>'||p_FrRoutingCapability_Tbl(i).RoutingObjVer||CHR(10)||
            'RoutingObjRefId =>'||p_FrRoutingCapability_Tbl(i).RoutingObjRefId||CHR(10)||
            'CapabilityId    =>'||p_FrRoutingCapability_Tbl(i).CapabilityId
            , p_DebugFlg, v_ExceptionFlg);
    END LOOP;

    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'step 1');

    FOR i IN 1..p_ToRoutingCapability_Tbl.count LOOP
        DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,
            CHR(10)||'TO  '||CHR(10)||'------------------------------------------'||CHR(10)||
            'ChannelId       =>'||v_ChannelId||CHR(10)||
            'PartnerId       =>'||p_ToRoutingCapability_Tbl(i).PartnerId||CHR(10)||
            'RoutingLevel    =>'||p_ToRoutingCapability_Tbl(i).RoutingObjLevel||CHR(10)||
            'RoutingObjId    =>'||p_ToRoutingCapability_Tbl(i).RoutingObjId||CHR(10)||
            'RoutingObjType  =>'||p_ToRoutingCapability_Tbl(i).RoutingObjTypeKey||CHR(10)||
            'RoutingObjCd    =>'||p_ToRoutingCapability_Tbl(i).RoutingObjCd||CHR(10)||
            'RoutingObjVer   =>'||p_ToRoutingCapability_Tbl(i).RoutingObjVer||CHR(10)||
            'RoutingObjRefId =>'||p_ToRoutingCapability_Tbl(i).RoutingObjRefId||CHR(10)||
            'CapabilityId    =>'||p_ToRoutingCapability_Tbl(i).CapabilityId
            , p_DebugFlg, v_ExceptionFlg);
    END LOOP;

    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'step 2');

    For i in 1..v_RoutingAttr_Tbl.count LOOP
        DBMS_OUTPUT.Put_Line(RPAD(v_RoutingAttr_Tbl(i).AttrLabel, 5)||RPAD(v_RoutingAttr_Tbl(i).AttributeKey, 30)||
            RPAD(v_RoutingAttr_Tbl(i).AttrRank, 5)||RPAD(v_RoutingAttr_Tbl(i).AttrValue, 5));
    END LOOP;

    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'step 3');

    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,CHR(10)||
    'ChannelId                  =>'||  v_ChanRec.CHANNELID||CHR(10)||
    'FrPartnerId                =>'||  v_FrPrtRec.PartnerId||CHR(10)||
    'ToPartnerId                =>'||  v_ToPrtRec.PartnerId||CHR(10)||
    'FrBusinessId               =>'||  v_FrPrtRec.BusinessId||CHR(10)||
    'ToBusinessId               =>'||  v_ToPrtRec.BusinessId||CHR(10)||
    'FrProcessCd                =>'||  v_FrProcessCd||CHR(10)||
    'FrProcessVer               =>'||  v_FrProcessVer||CHR(10)||
    'FrProtocolName             =>'||  v_FrProtocolName||CHR(10)||
    'FrProtocolVer              =>'||  v_FrProtocolVer||CHR(10)||
    'FrPackageName              =>'||  v_FrPackageName||CHR(10)||
    'FrPackageVer               =>'||  v_FrPackageVer||CHR(10)||
    'ToProcessCd                =>'||  v_ToProcessCd||CHR(10)||
    'ToProcessVer               =>'||  v_ToProcessVer||CHR(10)||
    'ToProtocolName             =>'||  v_ToProtocolName||CHR(10)||
    'ToProtocolVer              =>'||  v_ToProtocolVer||CHR(10)||
    'ToPackageName              =>'||  v_ToPackageName||CHR(10)||
    'ToPackageVer               =>'||  v_ToPackageVer||CHR(10)||
    'FrPartnerEnabled           =>'||  v_FrPrtRec.Enabled||CHR(10)||
    'ToPartnerEnabled           =>'||  v_ToPrtRec.Enabled||CHR(10)||
    'InitPartnerTypeName        =>'||  v_FrPrtRec.PartnerTypeName||CHR(10)||
    'InitOwnerFlg               =>'||  v_ToPrtRec.OwnerFlg||CHR(10)||
    'FrPartnerTypeName          =>'||  v_FrPrtRec.PartnerTypeName||CHR(10)||
    'FrOwnerFlg                 =>'||  v_FrPrtRec.OwnerFlg||CHR(10)||
    'ToPartnerTypeName          =>'||  v_ToPrtRec.PartnerTypeName||CHR(10)||
    'ToOwnerFlg                 =>'||  v_ToPrtRec.OwnerFlg||CHR(10)||
    'Enabled                    =>'||  v_ChanRec.Enabled||CHR(10)||
    'ToGatewayEnabled           =>'||  v_ChanRec.ToGatewayEnabled||CHR(10)||
    'FrGatewayId                =>'||  v_ChanRec.FRGATEWAYID||CHR(10)||
    'ToGatewayId                =>'||  v_ChanRec.TOGATEWAYID||CHR(10)||
    'FrGatewayName              =>'||  v_ChanRec.FRGATEWAYNAME||CHR(10)||
    'ToGatewayName              =>'||  v_ChanRec.TOGATEWAYNAME||CHR(10)||
    'FrGatewayEnabled           =>'||  v_ChanRec.FrGatewayEnabled||CHR(10)||
    'ToGatewayEnabled           =>'||  v_ChanRec.ToGatewayEnabled||CHR(10)||
    'ToURI                      =>'||  v_ChanRec.ToURI||CHR(10)||
    'ToURILogin                 =>'||  v_ChanRec.ToURILOGIN||CHR(10)||
    'ToURIPassword              =>'||  v_ChanRec.ToURIPASSWORD||CHR(10)||
    'WorkflowId                 =>'||  v_ChanRec.WORKFLOWID||CHR(10)||
    'ProfileId                  =>'||  v_ChanRec.PROFILEID||CHR(10)||
    'WorkflowName               =>'||  substr(v_ChanRec.WORKFLOWNAME, 255)||CHR(10)||
    'WorkflowDef                =>'||  substr(v_ChanRec.WORKFLOWDEF, 255)||CHR(10)||
    'TransportType              =>'||  v_ChanRec.TRANSPORTTYPE||CHR(10)||
    'FrValidateClientSSLCert    =>'||  v_ChanRec.FRVALIDATECLIENTSSLCERT||CHR(10)||
    'FrValidateClientIP         =>'||  v_ChanRec.FRVALIDATECLIENTIP||CHR(10)||
    'TransformMapId             =>'||  v_ChanRec.TransformMapId||CHR(10)||
    'ToActivityName             =>'||  v_ToActivityName||CHR(10)||
    'ToActionName               =>'||  v_ToActionName
    , p_DebugFlg, v_ExceptionFlg);
END Debug_Output;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_StartMsg, p_DebugFlg, v_ExceptionFlg);
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    IF RC1%isOpen THEN Close RC1; END IF;
    IF RC2%isOpen THEN Close RC2; END IF;
    IF RC3%isOpen THEN Close RC3; END IF;
    p_ReturnCd := 0;

    -- -----------------------------------------------------------
    -- 1.  Lookup Partner info and flip PartnerId's if necessary
    -- -----------------------------------------------------------

    BEGIN
        v_Msg := '------------>> DestType look up FAILED << ';
        SELECT DestTypeId INTO v_DestTypeId FROM PR_DESTTYPE WHERE DestName = p_DestName;
        IF p_InitBusinessId = p_FrBusinessId and p_InitBusinessId != p_ToBusinessId THEN
            v_ChanDirection := 'FR';
        ELSE
            v_ChanDirection := 'TO';
        END IF;
        v_Msg := '------------>> Partner look up FAILED << ';
        select * into v_FrPrtRec from PR_Partner01_vw where BusinessId = p_FrBusinessId and enabled = 1;
        select * into v_ToPrtRec from PR_Partner01_vw where BusinessId = p_ToBusinessId and enabled = 1;
        
		select businessidname into v_frbusinessidname from pr_businessidtype, PR_BUSIDENTIFIER
		where pr_businessidtype.BUSINESSIDTYPE = PR_BUSIDENTIFIER.BUSINESSIDTYPE
		and PR_BUSIDENTIFIER.BUSINESSID = p_FrBusinessId;
		
		select businessidname into v_tobusinessidname from pr_businessidtype, PR_BUSIDENTIFIER
		where pr_businessidtype.BUSINESSIDTYPE = pr_busidentifier.BUSINESSIDTYPE
		and pr_busidentifier.BUSINESSID = p_ToBusinessId;
		

            v_ChannelId := p_ChannelId;
            select /*+ FIRST_ROWS */ RoutingObjRefId into v_RoutingObjRefId
            from PR_RoutingChannel Ch
            INNER JOIN PR_Capability C on C.CapabilityId = CASE WHEN v_ChanDirection = 'FR' THEN Ch.FrCapabilityId ELSE Ch.ToCapabilityId END
                                            and v_FrPrtRec.PartnerId = CASE WHEN v_ChanDirection = 'FR' THEN Ch.FrPartnerId ELSE Ch.ToPartnerId END
                                            and v_ToPrtRec.PartnerId = CASE WHEN v_ChanDirection = 'TO' THEN Ch.FrPartnerId ELSE Ch.ToPartnerId END
            Where ChannelId = p_ChannelId;
            PR_CapabilityS_AH5
                (
                v_ReturnCd
                ,p_DebugFlg
                ,v_FrPrtRec.PartnerId
                ,v_RoutingObjRefId
                ,p_FrRoutingCapability_Tbl
                ,1
                );
            v_CapabilityCnt := 5;
            Exception
            WHEN NO_DATA_FOUND THEN
                p_ReturnCd := 3;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_Msg, p_DebugFlg, v_ExceptionFlg);
    END;
    IF p_ReturnCd = 3 OR NVL(v_CapabilityCnt, 0) < 5 THEN --- If v_CapabilityCnt = 5, then all capabilities were found
        null;
    ELSE
        ----- NOTE: the row coming back from PR_RoutingChannel01_vw is in the original Initiating/From/To representation
        BEGIN
            select /*+ FIRST_ROWS */  * into v_ChanRec
            from PR_RoutingChannel01_vw C
            Where ChannelDirection = v_ChanDirection and ChannelID = v_ChannelId and DestTypeId = v_DestTypeId;
            Exception
                When NO_DATA_FOUND THEN
                    p_ReturnCd := 5;
                    null;
                WHEN OTHERS then
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELS_AH9:'||SQLERRM);
                    ROLLBACK work;
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, '>> Channel view select failed << '||v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
                    Commit;
        END;
        IF p_ReturnCd = 0 THEN
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, CHR(10)||'CheckChannel Status:  Channel Found = '||v_ChannelId||'  Fr='||p_FrBusinessId||
                                            '  To='||p_ToBusinessId||'  ['||to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS')||']'
                                            , p_DebugFlg, v_ExceptionFlg);
            v_FrCapabilityId := v_ChanRec.FrCapabilityId;
            v_ToCapabilityId := v_ChanRec.ToCapabilityId;
            IF v_ChanDirection = 'TO' THEN
                v_RoutingObjRefId := v_ChanRec.FrRoutingObjRefId;
            ELSE
                v_RoutingObjRefId := v_ChanRec.ToRoutingObjRefId;
            END IF;

            PR_CapabilityS_AH5
                (
                p_ReturnCd
                ,v_DebugFlg
                ,v_ToPrtRec.PartnerId
                ,v_RoutingObjRefId              --v_ChanRec.ToRoutingObjRefId
                ,p_ToRoutingCapability_Tbl
                ,1
                );
                
            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt: Package');
            FrAttrValue('Package', v_FrPackageName, v_FrPackageVer, v_FrPkgRefId);
            --dbms_output.put_line('v_FrPackageName2:'||v_FrPackageName||', v_FrPackageVer2:' || v_FrPackageVer||', v_FrPkgRefId2:' ||v_FrPkgRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt: Protocol');
            FrAttrValue('Protocol', v_FrProtocolName, v_FrProtocolVer, v_FrProtocolRefId);
            --dbms_output.put_line('v_FrProtocolName2:'||v_FrProtocolName||', v_FrProtocolVer2:' || v_FrProtocolVer||', v_FrProtocolRefId2:' ||v_FrProtocolRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt: Process');
            FrAttrValue('Process', v_FrProcessCd, v_FrProcessVer, v_FrProcessRefId);
            --dbms_output.put_line('v_FrProcessCd2:'||v_FrProcessCd||', v_FrProcessVer2:' || v_FrProcessVer||', v_FrProcessRefId2:' ||v_FrProcessRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt:Activity');
            FrAttrValue2('Activity', p_ActivityName, v_FrActivityRefId);
            --dbms_output.put_line('v_FrActivityRefId2:'||v_FrActivityRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt:Action');
            FrAttrValue2('Action', p_ActionName, v_FrActionRefId);
            --dbms_output.put_line('v_FrActionRefId2:'||v_FrActionRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt:Signal');
            FrAttrValue2('Signal', p_Signal, v_FrSignalRefId);
            --dbms_output.put_line('v_FrSignalRefId2:'||v_FrSignalRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt2: Package');
            ToAttrValue('Package', v_ToPackageName, v_ToPackageVer, v_ToPkgRefId);
            --dbms_output.put_line('v_ToPackageName2:'||v_ToPackageName||', v_ToPackageVer2:' || v_ToPackageVer||', v_ToPkgRefId2:' ||v_ToPkgRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt2: Protocol');
            ToAttrValue('Protocol', v_ToProtocolName, v_ToProtocolVer, v_ToProtocolRefId);
            --dbms_output.put_line('v_ToProtocolName2:'||v_ToProtocolName||', v_ToProtocolVe2:' || v_ToProtocolVer||', v_ToProtocolRefId2:' ||v_ToProtocolRefId);

            --DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ChkPt2: Process');
            ToAttrValue('Process', v_ToProcessCd, v_ToProcessVer, v_ToProcessRefId);
            --dbms_output.put_line('v_ToProcessCd2:'||v_ToProcessCd||', v_ToProcessVer2:' || v_ToProcessVer||', v_ToProcessRefId2:' ||v_ToProcessRefId);
        END IF;
    END IF;
---- following code moved up here from -- 6 for ebXML
        BEGIN
            select RoutingObjCd, RoutingObjRefId into v_ToActivityName, v_ToActivityRefId
            from TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
            Where RoutingObjTypeKey = 'Activity' and rownum < 2;
        Exception
            When NO_DATA_FOUND Then
                v_ToActivityName := NULL;
                v_ToActivityRefId := NULL;
            WHEN OTHERS then
                RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELS_AH9:'||SQLERRM);
                ROLLBACK work;
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
        END;
        BEGIN
            select RoutingObjCd, RoutingObjRefId  into v_ToActionName, v_ToActionRefId
            from TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
            Where RoutingObjTypeKey = 'Action' and rownum < 2;
        Exception
            When NO_DATA_FOUND Then
                v_ToActionName := NULL;
                v_ToActionRefId := NULL;
            WHEN OTHERS then
                RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELS_AH9:'||SQLERRM);
                ROLLBACK work;
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
        END;
        BEGIN
            select RoutingObjCd, RoutingObjRefId into v_ToSignal, v_ToSignalRefId
            from TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
            Where RoutingObjTypeKey = 'Signal' and RoutingObjCd = p_Signal and rownum < 2;
        Exception
            When NO_DATA_FOUND Then
                v_ToSignalRefId := NULL;
            WHEN OTHERS then
                RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELS_AH9:'||SQLERRM);
                ROLLBACK work;
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
        END;

    --IF p_DebugFlg = 1 THEN Debug_Output; END IF;
    
   		 
       
    -------------------------------------------------------------------------
    -- 4.  Return result set 1 with channel data.
    -------------------------------------------------------------------------
    Open RC1 for
    select
        v_ChanRec.CHANNELID             ChannelId,
        v_FrPrtRec.PartnerId            FrPartnerId,
        v_ToPrtRec.PartnerId            ToPartnerId,
        v_FrPrtRec.BusinessId           FrBusinessId,
        v_ToPrtRec.BusinessId           ToBusinessId,
        v_FrProcessCd                   FrProcessCd,
        v_FrProcessVer                  FrProcessVer,
        v_FrProtocolName                FrProtocolName,
        v_FrProtocolVer                 FrProtocolVer,
        v_FrPackageName                 FrPackageName,
        v_FrPackageVer                  FrPackageVer,
        v_ToProcessCd                   ToProcessCd,
        v_ToProcessVer                  ToProcessVer,
        v_ToProtocolName                ToProtocolName,
        v_ToProtocolVer                 ToProtocolVer,
        v_ToPackageName                 ToPackageName,
        v_ToPackageVer                  ToPackageVer,
        v_FrPrtRec.Enabled              FrPartnerEnabled,
        v_ToPrtRec.Enabled              ToPartnerEnabled,
        v_FrPrtRec.PartnerTypeName      InitPartnerTypeName,
        v_ToPrtRec.OwnerFlg             InitOwnerFlg,
        v_FrPrtRec.PartnerTypeName      FrPartnerTypeName,
        v_FrPrtRec.OwnerFlg             FrOwnerFlg,
        v_ToPrtRec.PartnerTypeName      ToPartnerTypeName,
        v_ToPrtRec.OwnerFlg             ToOwnerFlg,
        v_ChanRec.Enabled               Enabled,
        v_ChanRec.ToGatewayEnabled      ToGatewayEnabled,
        v_ChanRec.FRGATEWAYID           FrGatewayId,
        v_ChanRec.TOGATEWAYID           ToGatewayId,
        v_ChanRec.FRGATEWAYNAME         FrGatewayName,
        v_ChanRec.TOGATEWAYNAME         ToGatewayName,
        v_ChanRec.FrGatewayEnabled      FrGatewayEnabled,
        v_ChanRec.ToGatewayEnabled      ToGatewayEnabled,
        v_ChanRec.ToURI                 ToURI,
        v_ChanRec.ToURILOGIN            ToURILogin,
        v_ChanRec.ToURIPASSWORD         ToURIPassword,
        v_ChanRec.WORKFLOWID            WorkflowId,
        v_ChanRec.ProfileID             ProfileId,
        v_ChanRec.WORKFLOWNAME          WorkflowName,
        v_ChanRec.WORKFLOWDEF           WorkflowDef,
        v_ChanRec.TRANSPORTTYPE         TransportType,
        v_ChanRec.FRVALIDATECLIENTSSLCERT   FrValidateClientSSLCert,
        v_ChanRec.FRVALIDATECLIENTIP        FrValidateClientIP,
        v_ChanRec.TransformMapId        TransformMapId
        ,v_frbusinessidname     FrBusinessIDName
		,v_tobusinessidname     ToBusinessIDName
		,v_FrActivityName		FrActivityName
		,v_FrActionName			FrActionName
		,v_ToActivityName		ToActivityName
		,v_ToActionName			ToActionName	
    from dual;
    -------------------------------------------------------------------------
    --  5.  Return result set 2 with service data.
    -------------------------------------------------------------------------
    Open RC2 for
        select /*+ FIRST_ROWS */
            'FR' SvcSrc,
            RoutingObjCd ServiceName,
            RoutingObjVer ServiceVer
        from
            PR_Capability01_vw
        where PartnerId = v_ChanRec.FrPartnerId and RoutingObjTypeKey = 'Service'
        UNION
        select /*+ FIRST_ROWS */
            'TO' SvcSrc,
            RoutingObjCd ServiceName,
            RoutingObjVer ServiceVer
        from
            PR_Capability01_vw
        where PartnerId = v_ChanRec.ToPartnerId and RoutingObjTypeKey = 'Service';
    -- NOTE: If there is no Transform Map, then use the From Activity and Action, else use the actual TO Activity and Action.
    --       It is assumed here that if a Transform Map is present, then specific attributes and guidelines should be
    --       returned in the RC3 result set.
    --  6.  Return result set 3 with TPA attribute data.  This data is retrieved based
    --      upon attributes tied to the process, activity or actions set at either
    --      the global, partner or channel override levels.
--    IF v_ChanRec.TransformMapId IS NOT NULL THEN
---- code moved up to before RC1 from -- 6 for ebXML
--    END IF;

	 BEGIN
	          
	        	PR_CHANNEL_CERTSETS_AH1
	        	(v_ReturnCd
	        	,p_DebugFlg
	        	,v_rc1
	        	,p_ChannelId
	        	,v_DestTypeId);
	        	
	        	FETCH v_RC1 
					INTO
				       	v_channeldesttypeid,
				       	v_encryption_flg,
				       	v_certsetid_encryption_temp,
				       	v_certsetid_encryption_name,
				       	v_sslclient_flg,
				       	v_certsetid_sslclient_temp,
				       	v_certsetid_sslclient_name,
				       	v_digsigflg,
				       	v_certsetid_signing_temp,
	       		                v_certsetid_signing_name;
	       		                
	v_certsetid_encryption := COALESCE(to_char(v_certsetid_encryption_temp),'-1');
	v_certsetid_sslclient := COALESCE(to_char(v_certsetid_sslclient_temp),'-1');
	v_certsetid_signing := COALESCE(to_char(v_certsetid_signing_temp),'-1');
		       		                
	EXCEPTION
	            	WHEN OTHERS THEN
	    	
	        	NULL;		        		 
				
	END; 
	
    IF p_ReturnCd = 0 THEN
        Begin
            select AttrLabel, AttributeKey, AttrRank, AttrValue
            BULK COLLECT into v_RoutingAttr_Tbl
            From
            (
                select /*+ FIRST_ROWS */ distinct
                    'FR' AttrLabel,
                    AttributeKey,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  v_FrPrtRec.PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrRank') AttrRank,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  v_FrPrtRec.PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrValue') AttrValue
                from
                    TABLE(CAST (p_FrRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
                    INNER JOIN PR_RoutingAttr_Lnk  Ra on Ra.RoutingObjRefId = T.RoutingObjRefId
                    INNER JOIN CF_RoutingAttr A on Ra.RoutingAttrId = A.RoutingAttrId
                Where T.RoutingObjRefId in (v_FrPkgRefId, v_FrProtocolRefId, v_FrProcessRefId, v_FrActivityRefId, v_FrActionRefId, v_FrSignalRefId)
                      AND (   (Ra.PartnerId IS NULL and Ra.ChannelID is NULL and Ra.DestTypeId IS NULL)
                           OR (Ra.PartnerId = v_FrPrtRec.PartnerId and Ra.ChannelID IS NULL)
                           OR (Ra.ChannelId = v_ChannelId and Ra.PartnerId = v_FrPrtRec.PartnerId and Ra.DestTypeId IS NULL)
                           OR (Ra.ChannelId = v_ChannelId AND Ra.DestTypeId = v_DestTypeId)
                          )
                UNION ALL
                select /*+ FIRST_ROWS */ distinct
                    'TO' AttrLabel,
                    AttributeKey,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  v_ToPrtRec.PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrRank') AttrRank,
                    FN_GET_RTObjAttrVl(T.RoutingObjId, T.RoutingObjRefId,  v_ToPrtRec.PartnerId, v_ChannelId, v_DestTypeId, A.AttributeKey, 'AttrValue') AttrValue
                from
                    TABLE(CAST (p_ToRoutingCapability_Tbl AS PR_RoutingCapability_Tbl)) T
                    INNER JOIN PR_RoutingAttr_Lnk  Ra on Ra.RoutingObjRefId = T.RoutingObjRefId
                    INNER JOIN CF_RoutingAttr A on Ra.RoutingAttrId = A.RoutingAttrId
                Where T.RoutingObjRefId in (v_ToPkgRefId, v_ToProtocolRefId, v_ToProcessRefId, v_ToActivityRefId, v_ToActionRefId, v_ToSignalRefId)
                      AND (   (Ra.PartnerId IS NULL and Ra.ChannelID is NULL and Ra.DestTypeId IS NULL)
                           OR (Ra.PartnerId = v_ToPrtRec.PartnerId and Ra.ChannelID IS NULL)
                           OR (Ra.ChannelId = v_ChannelId and Ra.PartnerId = v_ToPrtRec.PartnerId and Ra.DestTypeId IS NULL)
                           OR (Ra.ChannelId = v_ChannelId AND Ra.DestTypeId = v_DestTypeId)
                          )
	   	UNION ALL
		Select 'TO' Attrlabel,
		       'bcg.ro.certsetidencryption' AttributeKey,
		       '1' AttrRank,
		       coalesce(v_certsetid_encryption,'-1') AttrValue
		FROM DUAL
		
		UNION ALL
		Select 'FR' Attrlabel,
		'bcg.ro.certsetidsslclient' AttributeKey,
		'1' AttrRank,
		coalesce(v_certsetid_sslclient,'-1') AttrValue
		FROM DUAL
		
		UNION ALL
		Select 'FR' Attrlabel,
	       'bcg.ro.certsetidsign' AttributeKey,
		'1' AttrRank,
		coalesce(v_certsetid_signing,'-1') AttrValue
		FROM DUAL                           
            );
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'RC3 Attribute Select NO_DATA_FOUND: '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                    commit;
                WHEN OTHERS then
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELS_AH9:'||SQLERRM);
                    ROLLBACK work;
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, '>> Attribute Rank Lookup failed << '||CHR(10)||v_ParamsTxt||' SQLERRM: '||CHR(10)||SQLERRM, p_DebugFlg, v_ExceptionFlg);
                    Commit;
        END;
        For i in 1..v_RoutingAttr_Tbl.count LOOP
            p_RoutingAttr_Tbl.extend;
            p_RoutingAttr_Tbl(i) := PR_RoutingAttr_Rec(v_RoutingAttr_Tbl(i).AttrLabel, v_RoutingAttr_Tbl(i).AttributeKey,
                                                       v_RoutingAttr_Tbl(i).AttrRank,v_RoutingAttr_Tbl(i).AttrValue);
        END LOOP;
    END IF;
    Open RC3 for
        select distinct
            T.AttrLabel,
            T.AttributeKey,
            T.AttrValue           
        from
            TABLE(CAST (p_RoutingAttr_Tbl AS PR_RoutingAttr_Tbl)) T,
            (Select AttrLabel, AttributeKey, MAX(TO_NUMBER(AttrRank)) AttrRank from TABLE(CAST (p_RoutingAttr_Tbl AS PR_RoutingAttr_Tbl))
             group by AttrLabel, AttributeKey) T2
        Where T.AttrLabel = T2.AttrLabel and T.AttributeKey = T2.AttributeKey and T.AttrRank = T2.AttrRank;
    IF p_DebugFlg = 1 THEN Debug_Output; END IF;
    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_EndMsg, p_DebugFlg, v_ExceptionFlg);
End;
/

-- PR_ROUTINGCHANNELS_LIKE.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELS_LIKE
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_ChannelId  IN  PLS_INTEGER
	)
AS

/*
	================================================================================
	Proc Name:       PR_ROUTINGCHANNELS_LIKE
	Create Date:     2006-06-23 11:17:48
	Author:          SYSTEM 

Get similar channels for given ChannelID from routing tables 
	to send the message on the (A)sync channel
	                   ::::::::::::::::::::::::::::::
	                      M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------	-------	---------------------------------------------
	DATE       NAME		VER	DESCRIPTION
	---------- ------------	-------	---------------------------------------------
	2006-06-23 S.Priani	6.1	Created
	2008-09-12 Amit Kumar 	6.2	Defect 47980, 106065 and 106066

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          INTEGER Default 0;
v_BeginTS         TIMESTAMP Default FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) Default 'PR_ROUTINGCHANNELS_LIKE';
v_ExceptionFlg    INTEGER Default 0;
v_ParamsTxt       VARCHAR2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_ChannelID=>'||p_ChannelID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
	SELECT	rc.CHANNELID
		, rc.PARENTCHANNELID
		, rc.FRCAPABILITYID
		, rc.TOCAPABILITYID
		, rc.FRPARTNERID
		, rc.TOPARTNERID
		, rc.TRANSFORMMAPID
		, rc.WORKFLOWID
		, rc.PROFILEID
		, CASE WHEN rc.ACTIVESTATUSCD=63 THEN 1 ELSE 0 END ACTIVESTATUSCD
		, rc.MULTICASTFLG
		, rc.ROWTS
		, rc.CREATEDATE
		, rc.UPDATEDATE
		, p.ProfileName
	FROM	PR_ROUTINGCHANNEL rc
	LEFT OUTER JOIN CF_PROFILE p ON p.ProfileId = rc.ProfileID
	WHERE	parentChannelId = (SELECT	parentChannelId
				FROM		pr_routingChannel
				WHERE		ChannelId = p_ChannelID
				)
	AND	ChannelID <> p_ChannelId
	;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
EXCEPTION
	WHEN Others THEN
		DB_ProcAuditLOGI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
		p_ReturnCd := 1;
		RAISE;
End PR_ROUTINGCHANNELS_LIKE;
/


-- PR_ROUTINGCHANNELS_PROFILE.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELS_PROFILE
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_PROFILEID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_ROUTINGCHANNELS_PROFILE
	Create Date:     2004-09-06 18:06:05
	Author:          SYSTEM 
	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::
	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-09-06 SYSTEM           4.3 Created
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ROUTINGCHANNELS_PROFILE';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_PROFILEID=>'||p_PROFILEID||CHR(10)
		;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR 
		SELECT
				CHANNELID
				,p1.PARTNERID FROMPARTNERID
				,RTRIM(p1.PARTNERNAME) FROMPARTNERNAME
				,p2.PARTNERID TOPARTNERID
				,RTRIM(p2.PARTNERNAME) TOPARTNERNAME
			FROM 
				PR_ROUTINGCHANNEL r 
				INNER JOIN PR_PARTNER p1 ON r.FRPARTNERID = p1.PARTNERID 
				INNER JOIN PR_PARTNER p2 ON r.TOPARTNERID = p2.PARTNERID   
			Where 
		r.PROFILEID = p_PROFILEID;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		              WHEN NO_DATA_FOUND THEN p_ReturnCd := 3;
		              WHEN OTHERS then
		                   p_ReturnCd := 1;
		                   v_ExceptionFlg := 1;
		                   DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
                   RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELS_PROFILE:'||SQLERRM);
End PR_ROUTINGCHANNELS_PROFILE;
/


-- PR_ROUTINGCHANNELS_WORKFLOW.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELS_WORKFLOW
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_WORKFLOWID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_ROUTINGCHANNELS_WORKFLOW
	Create Date:     2004-03-13 00:10:40
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-31 T. Dang        4.2.2 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ROUTINGCHANNELS_WORKFLOW';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;

	OPEN RC1 FOR 
	SELECT
		CHANNELID
		,p1.PARTNERID FROMPARTNERID
		,RTRIM(p1.PARTNERNAME) FROMPARTNERNAME
		,p2.PARTNERID TOPARTNERID
		,RTRIM(p2.PARTNERNAME) TOPARTNERNAME
	FROM 
		PR_ROUTINGCHANNEL r 
		INNER JOIN PR_PARTNER p1 ON r.FRPARTNERID = p1.PARTNERID 
		INNER JOIN PR_PARTNER p2 ON r.TOPARTNERID = p2.PARTNERID   
	Where 
		r.WORKFLOWID = p_WORKFLOWID
	;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);


END PR_ROUTINGCHANNELS_WORKFLOW ;
/


-- PR_ROUTINGCHANNELU_AH1.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELU_AH1
(
     p_ReturnCd out pls_integer
    , p_DebugFlg in pls_integer
    , RC1 out db_ref.RCT
    ,p_CHANNELID in NUMBER
    ,p_ACTIVESTATUSCD in NUMBER
    ,p_MULTICASTFLG in NUMBER
    ,p_WorkFlowId in NUMBER
    ,p_ProfileId in NUMBER
    ,p_ROWTS in NUMBER
) as

/*
    ================================================================================
    Proc Name:       PR_ROUTINGCHANNELU_AH1
    Create Date:     2003-04-22 06:32:57
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-22 J.Chen        3.5    Update channel ActiveStatusCd,MultiCastFlg
    2003-05-22 J.Chen        3.5    Use ActiveStatusCd_Seq when p_ActiveStatusCd = 2
    2008-09-12 Amit Kumar    6.2    Defect 47980, 106065 and 106066
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_ROUTINGCHANNELU_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_CHANNELID=>'||p_CHANNELID||CHR(10)
        ||', p_ACTIVESTATUSCD=>'||p_ACTIVESTATUSCD||CHR(10)
        ||', p_MULTICASTFLG=>'||p_MULTICASTFLG||CHR(10)
        ||', p_ProfileId=>'||p_ProfileId||CHR(10)
        ||', p_ROWTS=>'||p_ROWTS
        ;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_RowCnt := 0;
    IF p_ACTIVESTATUSCD = 2 THEN   --- Delete the Channel (Deactivate Condition)
    
	DELETE FROM pr_channel_gateway_lnk
	WHERE channelid = p_channelid;
	    
	DELETE FROM pr_routingattr_lnk
	WHERE channelid = p_channelid;
	    
	DELETE FROM pr_channel_certset_lnk
	WHERE channelid = p_channelid;
	    
	DELETE FROM pr_routingchannel
	WHERE channelid = p_channelid;
    
    ELSIF p_ACTIVESTATUSCD = 1 THEN
    	BEGIN
    		Update PR_ROUTINGCHANNEL
   		SET
        	ACTIVESTATUSCD= CASE WHEN ACTIVESTATUSCD=31 THEN 63 ELSE ACTIVESTATUSCD END
        	,      MULTICASTFLG=p_MULTICASTFLG
        	,      WORKFLOWID = p_WorkFlowId
        	,      PROFILEID = p_ProfileId
        	,      ROWTS= p_RowTS + 1
        	,      UPDATEDATE= sysdate
    		Where
        	CHANNELID = p_CHANNELID
        	And RowTS = p_RowTS returning RowTS into v_RowTS;
        
    		v_RowCnt := SQL%rowcount;
	    	EXCEPTION
			When NO_DATA_FOUND THEN
				v_RowCnt := 0;
    	
    	END;
    	
    
    ELSE
    	BEGIN
    		Update PR_ROUTINGCHANNEL
		   	SET
		        ACTIVESTATUSCD= CASE WHEN ACTIVESTATUSCD=63 THEN 31 ELSE ACTIVESTATUSCD END
		        ,      MULTICASTFLG=p_MULTICASTFLG
		        ,      WORKFLOWID = p_WorkFlowId
		        ,      PROFILEID = p_ProfileId
		        ,      ROWTS= p_RowTS + 1
		        ,      UPDATEDATE= sysdate
		    	Where
		        CHANNELID = p_CHANNELID
		        And RowTS = p_RowTS returning RowTS into v_RowTS;
	        
	    	v_RowCnt := SQL%rowcount;
	    	EXCEPTION
			When NO_DATA_FOUND THEN
				v_RowCnt := 0;
				
	END;    
    		
    
    END IF;
    
    IF RC1%isOpen THEN Close RC1; END IF;
    
    if (v_RowCnt = 0) then  
    	OPEN RC1 for select p_RowTS RowTS from Dual;
    else
    	OPEN RC1 for select v_RowTS RowTS from Dual;
    end if;
    
    

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);


    EXCEPTION
        WHEN TOO_MANY_ROWS Then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999, '20100 - PR_ROUTINGCHANNELU_AH1: Too Many Rows');
        WHEN ZeroUpdate then
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELU_AH1: No Data Found');
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
            Commit;
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELU_AH1:'||SQLERRM);
End PR_ROUTINGCHANNELU_AH1;
/

-- PR_ROUTINGCHANNELU_AH2.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELU_AH2
(
     p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,RC1 out db_ref.RCT
    ,p_InteractId in Number
    ,p_TransformMapId in number
) as

/*
    ================================================================================
    Proc Name:       PR_ROUTINGCHANNELU_AH1
    Create Date:     2003-04-22 06:32:57
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-09-22 J.Chen        3.5    Update TransformMapId by given InteractId
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_ROUTINGCHANNELU_AH2';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_TransformMapId =>'||p_TransformMapId ||CHR(10)
        ||', p_InteractId =>'||p_InteractId;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_RowCnt := 0;

    Update PR_ROUTINGCHANNEL
    SET TransformMapId = p_TransformMapId
        ,      ROWTS = RowTS + 1
        ,      UPDATEDATE = v_BeginTS
    Where
         CHANNELID in (  Select c.ChannelId
                            From    cf_ValidObjInteract v
                            Inner join Pr_Capability f on v.FrRoutingObjRefId = f.RoutingObjRefId
                            Inner join Pr_Capability t on v.ToRoutingObjRefId = t.RoutingObjRefId
                            Inner join pr_RoutingChannel c on c.frCapabilityId = f.CapabilityId
                                                          and c.ToCapabilityId = t.CapabilityId
                            Where   v.InteractId = p_InteractId
                            And     c.TransformMapId is null);

    v_RowCnt := SQL%rowcount;
    
    if (v_RowCnt = 0) then  
        p_ReturnCd := 3; 
    end if;
    
    IF RC1%isOpen THEN Close RC1; END IF;
    OPEN RC1 for select NVL(v_RowTS, 0) RowTS from Dual;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION
        WHEN ZeroUpdate then
            p_ReturnCd := 3;
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELU_AH2:'||SQLERRM);
End PR_ROUTINGCHANNELU_AH2;
/


-- PR_ROUTINGCHANNELU_AH3.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGCHANNELU_AH3
(
     p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,p_ChannelId in Number
    ,p_TransformMapId in number
) as

/*
    ================================================================================
    Proc Name:       PR_ROUTINGCHANNELU_AH3
    Create Date:     2004-09-08 06:32:57
    Author:          S.Narain

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-09-22 J.Chen        3.5    Update TransformMapId by given InteractId
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_ROUTINGCHANNELU_AH3';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_TransformMapId =>'||p_TransformMapId ||CHR(10)
        ||', p_ChannelId =>'||p_ChannelId;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_RowCnt := 0;

    Update PR_ROUTINGCHANNEL
        SET TransformMapId = p_TransformMapId
            ,      ROWTS = RowTS + 1
            ,      UPDATEDATE = v_BeginTS
        Where
         CHANNELID = p_ChannelID;  

    v_RowCnt := SQL%rowcount;
    
    if (v_RowCnt = 0) then  
        p_ReturnCd := 3; 
    end if;
    
    
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

    EXCEPTION
        WHEN ZeroUpdate then
            p_ReturnCd := 3;
        WHEN OTHERS then
            ROLLBACK work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGCHANNELU_AH3:'||SQLERRM);
End PR_ROUTINGCHANNELU_AH3;
/


-- PR_ROUTINGOBJ_PAIRI.sql

CREATE OR REPLACE PROCEDURE PR_ROUTINGOBJ_PAIRI
	(
	p_ReturnCd OUT PLS_Integer
	,p_DebugFlg IN PLS_Integer
	,RC1 out db_ref.RCT
	,P_ROUTINGOBJTYPEID IN NUMBER
	,P_FRROUTINGOBJID IN NUMBER
	,P_TOROUTINGOBJID IN NUMBER
	)
AS
/*
	================================================================================
	Proc Name:       PR_ROUTINGOBJ_PAIRI
	Create Date:     2007-09-19 15:16:24
	Author:          AMIT KUMAR VIJAY 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          VER    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2007-09-19 AMIT KUMAR    6.1.1  Created for Multiple Certificate Feature

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
	
	CREATES A ROUTING OBJ PAIR.  NULL IMPLIES 'ALL'
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_ROUTINGOBJ_PAIRI';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', P_ROUTINGOBJTYPEID=>'||P_ROUTINGOBJTYPEID||CHR(10)
       ||', P_FRROUTINGOBJID=>'||P_FRROUTINGOBJID||CHR(10)
       ||', P_TOROUTINGOBJID=>'||P_TOROUTINGOBJID||CHR(10)
       ;
v_RowTS        integer :=0;

v_routingobjpairid   PR_ROUTINGOBJ_PAIR.routingobjpairid%Type DEFAULT NULL;

BEGIN

	BEGIN
	
	SELECT routingobjpairid into v_routingobjpairid from PR_ROUTINGOBJ_PAIR 
                where COALESCE(FRROUTINGOBJID,-1) = COALESCE(P_FRROUTINGOBJID,-1)
                and COALESCE(TOROUTINGOBJID,-1) = COALESCE(P_TOROUTINGOBJID,-1)
	; 
	
	EXCEPTION
	
	WHEN NO_DATA_FOUND THEN
	NULL;
	END;
	        
	if v_routingobjpairid IS NULL then                
	
		Insert into PR_ROUTINGOBJ_PAIR
			(
			ROUTINGOBJPAIRID,	
 			ROUTINGOBJTYPEID,	
			FRROUTINGOBJID,		
			TOROUTINGOBJID			
			) 
			Values 	
			(
			PR_ROUTINGOBJ_PAIR_SEQ.NEXTVAL
			,P_ROUTINGOBJTYPEID
			,P_FRROUTINGOBJID
			,P_TOROUTINGOBJID
			)RETURNING ROUTINGOBJPAIRID INTO v_routingobjpairid
		;
		
		v_rowts := 1;
		
	ELSE
		v_rowts := 0;
	
	END IF;			
		
	p_ReturnCd := 0;
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;		        
	Open RC1 FOR SELECT 	v_routingobjpairid ROUTINGOBJPAIRID, 
				v_RowTS RowTS
	FROM DUAL;	    

EXCEPTION
	
	WHEN DUP_VAL_ON_INDEX Then
		p_ReturnCd := 2;
	        v_ExceptionFlg :=1 ;
	        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'DUP_VAL_ON_INDEX,'||v_ParamsTxt, p_DebugFlg, 1);
				       
	
	WHEN OTHERS then
		ROLLBACK work;
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		RAISE_APPLICATION_ERROR(-20999,'20100 - PR_ROUTINGOBJ_PAIRI:'||SQLERRM);
			    	    
DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	
END PR_ROUTINGOBJ_PAIRI;
/	

-- PR_ROUTINGOBJ_PAIRS_AH1.sql

CREATE OR REPLACE Procedure PR_ROUTINGOBJ_PAIRS_AH1
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, P_FRROUTINGOBJID	in		INTEGER
	, P_TOROUTINGOBJID	in		INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       PR_ROUTINGOBJ_PAIRS_AH1
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ROUTINGOBJ_PAIRS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', P_FRROUTINGOBJID=>'||P_FRROUTINGOBJID||CHR(10)
		||', P_TOROUTINGOBJID=>'||P_TOROUTINGOBJID||CHR(10)
		;

BEGIN
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	
	OPEN RC1 FOR
	SELECT 	ROUTINGOBJPAIRID 
	FROM pr_routingobj_pair a
	where COALESCE(a.frroutingobjid,-1) = COALESCE(P_FRROUTINGOBJID,-1)
	and COALESCE(a.toroutingobjid,-1) = COALESCE(P_TOROUTINGOBJID,-1)
		;
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  


	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ROUTINGOBJ_PAIRS_AH1' || SQLCODE|| ' '||SQLERRM);
			
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ROUTINGOBJ_PAIRS_AH1' || SQLCODE|| ' '||SQLERRM);
END PR_ROUTINGOBJ_PAIRS_AH1;
/

-- PR_ROUTINGOBJ_PAIRS_AH2.sql

CREATE OR REPLACE Procedure PR_ROUTINGOBJ_PAIRS_AH2
	(
	  p_ReturnCd		out		INTEGER
	, p_DebugFlg		in		INTEGER
	, RC1                   out             db_ref.RCT
	, P_FRROUTINGOBJID	in		INTEGER
	, P_TOROUTINGOBJID	in		INTEGER
	)
AS

/*


	================================================================================
	Proc Name:       PR_ROUTINGOBJ_PAIRS_AH2
	Create Date:     2007-09-10 00:01:44
	Author:          BHARANI J

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
        2007-08-25  BHARANI J     6.1.1  Feature 55795
	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_ROUTINGOBJ_PAIRS_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', P_FRROUTINGOBJID=>'||P_FRROUTINGOBJID||CHR(10)
		||', P_TOROUTINGOBJID=>'||P_TOROUTINGOBJID||CHR(10)
		;
sqlstmt      varchar2(1000);
sql_where    varchar2(100);


BEGIN

	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
        
        sql_where := ' Where 1 =1 '||	    
	FN_Build_Where('frroutingobjid',   'NUMBER', P_FRROUTINGOBJID)||
	FN_Build_Where('toroutingobjid',   'NUMBER', P_TOROUTINGOBJID);
         
        sqlstmt :=  'SELECT routingobjpairid,
	         		routingobjtypeid,
				frroutingobjid,
				toroutingobjid
				FROM
			pr_routingobj_pair ' ;
	
	sqlstmt := sqlstmt || sql_where ;
	
	Open RC1 for  sqlstmt;
	
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	  
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_ROUTINGOBJ_PAIRS_AH2' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_ROUTINGOBJ_PAIRS_AH2' || SQLCODE|| ' '||SQLERRM);
END PR_ROUTINGOBJ_PAIRS_AH2;
/

-- PR_TRANSPORTCFGPOINTD.sql

CREATE OR REPLACE Procedure PR_TRANSPORTCFGPOINTD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_CFGPOINTID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTCFGPOINTD
	Create Date:     2004-03-12 15:48:55
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTCFGPOINTD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_TRANSPORTCFGPOINT
	Where 
		CFGPOINTID = p_CFGPOINTID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORTCFGPOINTD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_TRANSPORTCFGPOINTD;
/


-- PR_TRANSPORTCFGPOINTI.sql

CREATE OR REPLACE Procedure PR_TRANSPORTCFGPOINTI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT 
	,p_CFGPOINTID in VARCHAR2
	,p_TRANSPORTID in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTCFGPOINTI
	Create Date:     2004-03-12 16:16:43
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTCFGPOINTI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_TRANSPORTCFGPOINT(
		CFGPOINTID
		,TRANSPORTID
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		p_CFGPOINTID
		,p_TRANSPORTID
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
    IF RC1%isOpen THEN Close RC1; END IF;                                          
	Open RC1 FOR Select p_CFGPOINTID, 1 RowTS From Dual;    
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORTCFGPOINTI' || SQLCODE|| ' '||SQLERRM);
END PR_TRANSPORTCFGPOINTI;
/


-- PR_TRANSPORTCFGPOINTS.sql

CREATE OR REPLACE PROCEDURE PR_TRANSPORTCFGPOINTS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CFGPOINTID IN VARCHAR2
	, p_TRANSPORTID IN NUMBER DEFAULT NULL
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTCFGPOINTS
	Create Date:     2004-03-12 15:49:35
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTCFGPOINTS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;      
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_where := ' Where 1=1'||
    FN_Build_Where('TRANSPORTID', 'NUMBER', p_TRANSPORTID) ||
    FN_Build_Where('CFGPOINTID', 'VARCHAR2', p_CFGPOINTID);
    
    v_sqlstmt := 
		'SELECT '
    	||'RTRIM(CFGPOINTID) CFGPOINTID'
	    ||',TRANSPORTID'
		||',ROWTS'
		||',CREATEDATE'
		||',RTRIM(RESOURCEKEY) RESOURCEKEY'
		||',RTRIM(DESCRIPTION) DESCRIPTION '
	    ||'FROM ' 
		||'PR_TRANSPORTCFGPOINT' || v_where 
	;
	
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ; 
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_TRANSPORTCFGPOINTS;
/


-- PR_TRANSPORTCFGPOINTU.sql

CREATE OR REPLACE Procedure PR_TRANSPORTCFGPOINTU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_CFGPOINTID in VARCHAR2
	,p_TRANSPORTID in NUMBER
	,p_ROWTS in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
) as

/*
	================================================================================
	Proc Name:       PR_TRANSPORTCFGPOINTU
	Create Date:     2004-03-12 15:49:18
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTCFGPOINTU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_TRANSPORTCFGPOINT
	SET
		TRANSPORTID=p_TRANSPORTID
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
	Where 
		CFGPOINTID = p_CFGPOINTID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_TRANSPORTCFGPOINTU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORTCFGPOINTU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORTCFGPOINTU:'||SQLERRM);
End PR_TRANSPORTCFGPOINTU;
/


-- PR_TRANSPORTD.sql

CREATE OR REPLACE Procedure PR_TRANSPORTD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_TRANSPORTID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTD
	Create Date:     2004-03-13 00:09:51
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_TRANSPORT
	Where 
		TRANSPORTID = p_TRANSPORTID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORTD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_TRANSPORTD;
/


-- PR_TRANSPORTD_AH1.sql

CREATE OR REPLACE Procedure PR_TRANSPORTD_AH1
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_TRANSPORTID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTD_AH1
	Create Date:     2006-08-29 16:40:11
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2006-08-29 S.Nair        6.1.0  Created
	2007-07-31 Bharani J     6.1.1  Modified for Feature F104473

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	
	Delete From PR_GATEWAY 
	Where
		TRANSPORTID = p_TRANSPORTID
	;
	Delete From PR_PROXY
	Where
		TRANSPORTID = p_TRANSPORTID
	;	
	Delete From PR_RCVCONFIG
	Where
		TRANSPORTID = p_TRANSPORTID
	;
	/* Delete From PR_RCVCONFIGDTL
	Where
		TRANSPORTID = p_TRANSPORTID
	;*/
	Delete From PR_TRANSPORTCFGPOINT
	Where
		TRANSPORTID = p_TRANSPORTID
	;
	Delete From PR_TRANSPORT_ATTR
	Where
		TRANSPORTID = p_TRANSPORTID
	;
	Delete From PR_TRANSPORT_GLOBAL_ATTR
	Where
		TRANSPORTID = p_TRANSPORTID
	;
	Delete From PR_TRANSPORT_LOC
	Where
		TRANSPORTID = p_TRANSPORTID
	;	
	Delete From PR_TRANSPORT
	Where
		TRANSPORTID = p_TRANSPORTID
	;	
		
	
	Commit;
	
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			--RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_TRANSPORTD_AH1;
/

-- PR_TRANSPORTHANDLER_CATD.sql

CREATE OR REPLACE Procedure PR_TRANSPORTHANDLER_CATD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTHANDLER_CATD
	Create Date:     2004-03-12 17:45:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTHANDLER_CATD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_where := ' WHERE 1=1 ' ;
    v_where := v_where || FN_BUILD_WHERE('CFGPOINTID', 'VARCHAR2', p_CFGPOINTID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	
    v_sqlstmt := 
	   'Delete From PR_TRANSPORTHANDLER_CAT ' || v_where  ;
	
	execute immediate v_sqlstmt ;
	
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORTHANDLER_CATD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_TRANSPORTHANDLER_CATD;
/


-- PR_TRANSPORTHANDLER_CATI.sql

CREATE OR REPLACE Procedure PR_TRANSPORTHANDLER_CATI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT    
	,p_CFGPOINTID in VARCHAR2
	,p_HANDLERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTHANDLER_CATI
	Create Date:     2004-03-12 18:02:37
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTHANDLER_CATI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_TRANSPORTHANDLER_CAT(
		CFGPOINTID
		,HANDLERID
	) Values (
		p_CFGPOINTID
		,p_HANDLERID
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORTHANDLER_CATI' || SQLCODE|| ' '||SQLERRM);
END PR_TRANSPORTHANDLER_CATI;
/


-- PR_TRANSPORTHANDLER_CATS.sql

CREATE OR REPLACE PROCEDURE PR_TRANSPORTHANDLER_CATS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_CFGPOINTID IN VARCHAR2
	, p_HANDLERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTHANDLER_CATS
	Create Date:     2004-03-12 18:08:25
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTHANDLER_CATS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;                                
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_CFGPOINTID=>'||p_CFGPOINTID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('CFGPOINTID', 'VARCHAR2', p_CFGPOINTID) ||
       FN_Build_Where('c.HANDLERID', 'NUMBER', p_HANDLERID);
	
	v_sqlstmt := 
		'SELECT '
		||'RTRIM(c.CFGPOINTID) CFGPOINTID'
		||',c.HANDLERID'   
		||',h.PROVIDERFLG'                                                                                                                         
		||',RTRIM(h.RESOURCEKEY) RESOURCEKEY'                                               
		||',RTRIM(h.DESCRIPTION) DESCRIPTION'                                               
		||',RTRIM(h.CLASSNAME) CLASSNAME'    
		||',c.CREATEDATE'
		||',c.ROWTS '     
    	||'FROM ' 
		||'PR_TRANSPORTHANDLER_CAT c, PR_HANDLER h'
		|| v_where
		|| ' AND c.HANDLERID = h.HANDLERID'
	;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_TRANSPORTHANDLER_CATS;
/


-- PR_TRANSPORTI.sql

CREATE OR REPLACE Procedure PR_TRANSPORTI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_TRANSPORTTYPE in VARCHAR2
	,p_VISIBILITY in NUMBER
	,p_UI_VISIBILITY in NUMBER
	,p_PROVIDERFLAG in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_CLASSNAME in VARCHAR2
	,p_TRANSPORTTYPEDESC in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTI
	Create Date:     2004-03-13 00:11:16
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTTYPE=>'||p_TRANSPORTTYPE||CHR(10)
		||', p_VISIBILITY=>'||p_VISIBILITY||CHR(10)
		||', p_UI_VISIBILITY=>'||p_UI_VISIBILITY||CHR(10)
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		||', p_CLASSNAME=>'||p_CLASSNAME||CHR(10)
		||', p_TRANSPORTTYPEDESC=>'||p_TRANSPORTTYPEDESC||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_TRANSPORT(
		TRANSPORTID
		,TRANSPORTTYPE
		,VISIBILITY
		,UI_VISIBILITY
		,PROVIDERFLAG
		,RESOURCEKEY
		,DESCRIPTION
		,CLASSNAME
		,TRANSPORTTYPEDESC
	) Values (
		PR_TRANSPORT_SEQ.nextval
		,p_TRANSPORTTYPE
		,p_VISIBILITY
		,p_UI_VISIBILITY
		,p_PROVIDERFLAG
		,p_RESOURCEKEY
		,p_DESCRIPTION
		,p_CLASSNAME
		,p_TRANSPORTTYPEDESC
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select 
		PR_TRANSPORT_SEQ.currval ID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORTI' || SQLCODE|| ' '||SQLERRM);
END PR_TRANSPORTI;
/


-- PR_TRANSPORTS.sql

CREATE OR REPLACE PROCEDURE PR_TRANSPORTS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_TRANSPORTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTS
	Create Date:     2004-03-13 00:10:40
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_TRANSPORTID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		TRANSPORTID
		,RTRIM(TRANSPORTTYPE) TRANSPORTTYPE
		,VISIBILITY
		,UI_VISIBILITY
		,ROWTS
		,CREATEDATE
		,PROVIDERFLAG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(CLASSNAME) CLASSNAME
		,RTRIM(TRANSPORTTYPEDESC) TRANSPORTTYPEDESC
	FROM 
		PR_TRANSPORT
	Where 
		TRANSPORTID = p_TRANSPORTID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		TRANSPORTID
		,RTRIM(TRANSPORTTYPE) TRANSPORTTYPE
		,VISIBILITY
		,UI_VISIBILITY
		,ROWTS
		,CREATEDATE
		,PROVIDERFLAG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(CLASSNAME) CLASSNAME
		,RTRIM(TRANSPORTTYPEDESC) TRANSPORTTYPEDESC
	FROM 
		PR_TRANSPORT
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_TRANSPORTS;
/


-- PR_TRANSPORTS_AH1_RCV.sql

CREATE OR REPLACE PROCEDURE PR_TRANSPORTS_AH1_RCV
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_TRANSPORTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTS
	Create Date:     2004-03-13 00:10:40
	Author:          T. Dang 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_TRANSPORTID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		TRANSPORTID
		,RTRIM(TRANSPORTTYPE) TRANSPORTTYPE
		,VISIBILITY
		,UI_VISIBILITY
		,ROWTS
		,CREATEDATE
		,PROVIDERFLAG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(CLASSNAME) CLASSNAME
	FROM 
		PR_TRANSPORT
	Where 
		TRANSPORTID = p_TRANSPORTID AND VISIBILITY < 7
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		TRANSPORTID
		,RTRIM(TRANSPORTTYPE) TRANSPORTTYPE
		,VISIBILITY
		,UI_VISIBILITY
		,ROWTS
		,CREATEDATE
		,PROVIDERFLAG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(CLASSNAME) CLASSNAME
	FROM 
		PR_TRANSPORT
	WHERE
	    VISIBILITY < 7
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_TRANSPORTS_AH1_RCV ;
/


-- PR_TRANSPORTS_AH2_GWY.sql

CREATE OR REPLACE PROCEDURE PR_TRANSPORTS_AH2_GWY
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_TRANSPORTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORTS
	Create Date:     2004-03-13 00:10:40
	Author:          T.Dang

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_TRANSPORTID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		TRANSPORTID
		,RTRIM(TRANSPORTTYPE) TRANSPORTTYPE
		,VISIBILITY
		,UI_VISIBILITY
		,ROWTS
		,CREATEDATE
		,PROVIDERFLAG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(CLASSNAME) CLASSNAME
	FROM 
		PR_TRANSPORT
	Where 
		TRANSPORTID = p_TRANSPORTID AND ( VISIBILITY = 7 OR VISIBILITY = 3 )
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		TRANSPORTID
		,RTRIM(TRANSPORTTYPE) TRANSPORTTYPE
		,VISIBILITY
		,UI_VISIBILITY
		,ROWTS
		,CREATEDATE
		,PROVIDERFLAG
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(CLASSNAME) CLASSNAME
	FROM 
		PR_TRANSPORT
	WHERE
	    VISIBILITY = 7 OR VISIBILITY = 3
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_TRANSPORTS_AH2_GWY ;
/


-- PR_TRANSPORTU.sql

CREATE OR REPLACE Procedure PR_TRANSPORTU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_TRANSPORTID in NUMBER
	,p_TRANSPORTTYPE in VARCHAR2
	,p_VISIBILITY in NUMBER
	,p_UI_VISIBILITY in NUMBER
	,p_ROWTS in NUMBER
	,p_PROVIDERFLAG in NUMBER
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_CLASSNAME in VARCHAR2
	,p_TRANSPORTTYPEDESC in VARCHAR2
) as

/*
	================================================================================
	Proc Name:       PR_TRANSPORTU
	Create Date:     2004-03-13 00:10:16
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-13 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORTU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_TRANSPORTTYPE=>'||p_TRANSPORTTYPE||CHR(10)
		||', p_VISIBILITY=>'||p_VISIBILITY||CHR(10)
		||', p_UI_VISIBILITY=>'||p_UI_VISIBILITY||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_PROVIDERFLAG=>'||p_PROVIDERFLAG||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		||', p_CLASSNAME=>'||p_CLASSNAME||CHR(10)
		||', p_TRANSPORTTYPEDESC=>'||p_TRANSPORTTYPEDESC||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_TRANSPORT
	SET
		TRANSPORTTYPE=p_TRANSPORTTYPE
		,      VISIBILITY=p_VISIBILITY
		,      UI_VISIBILITY=p_UI_VISIBILITY
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      PROVIDERFLAG=p_PROVIDERFLAG
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
		,      CLASSNAME=p_CLASSNAME
		,      TRANSPORTTYPEDESC=p_TRANSPORTTYPEDESC
	Where 
		TRANSPORTID = p_TRANSPORTID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_TRANSPORTU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORTU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORTU:'||SQLERRM);
End PR_TRANSPORTU;
/


-- PR_TRANSPORT_ATTRD.sql

CREATE OR REPLACE Procedure PR_TRANSPORT_ATTRD
                                 
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer                       
   ,p_TRANSPORTATTRID in NUMBER                                                   
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_TRANSPORT_ATTRD                                            
	Create Date:     2004-03-12 14:25:28                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-12 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                               
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_ATTRD';                    
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)                         
		;
                                                                            
zeroDelete  Exception;                                                          
                                                                                
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
   
   Delete From PR_TRANSPORT_ATTR                                                  
	Where 
		TRANSPORTATTRID = p_TRANSPORTATTRID                                   
	;                                                                              
	v_RowCnt := SQL%ROWCOUNT;                                                      
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;                             
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                          
	p_ReturnCd := 0;                                                               
	Exception                                                                      
		When ZeroDelete Then                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);                                                                     
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORT_ATTRD: No Data Found');  
		When OTHERS Then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);                                                                   
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);         
End PR_TRANSPORT_ATTRD;
/


-- PR_TRANSPORT_ATTRI.sql

CREATE OR REPLACE Procedure PR_TRANSPORT_ATTRI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT                                  
	,p_TRANSPORTID in NUMBER                                                       
	,p_ATTRNAME in VARCHAR2                                                        
	,p_RESOURCEKEY in VARCHAR2                                                     
	,p_DEFAULTATTRVALUE in VARCHAR2                                                
	,p_DESCRIPTION in VARCHAR2                                                     
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_TRANSPORT_ATTRI                                            
	Create Date:     2004-03-12 14:26:53                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-12 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             
v_PK_Value        number := 0;                                                  
v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                               
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_ATTRI';                    
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)                                 
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)                                       
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)                                 
		||', p_DEFAULTATTRVALUE=>'||p_DEFAULTATTRVALUE||CHR(10)                       
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)                                 
		;
                                                                            
                                                                                
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	Insert into PR_TRANSPORT_ATTR(                                                 
		TRANSPORTATTRID                                                               
		,TRANSPORTID                                                                  
		,ATTRNAME                                                                     
		,RESOURCEKEY                                                                  
		,DEFAULTATTRVALUE                                                             
		,DESCRIPTION                                                                  
	) Values (                                                                     
		PR_TRANSPORT_ATTR_SEQ.nextval                                                 
		,p_TRANSPORTID                                                                
		,p_ATTRNAME                                                                   
		,p_RESOURCEKEY                                                                
		,p_DEFAULTATTRVALUE                                                           
		,p_DESCRIPTION                                                                
	);                                                                             
	p_ReturnCd := 0;                                                               
	v_RowCnt := sql%ROWCOUNT;                                                      
	IF RC1%isOpen THEN Close RC1; END IF;                                          
	Open RC1 FOR Select 
		PR_TRANSPORT_ATTR_SEQ.currval ID, 1 RowTS From Dual;    
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                          
	EXCEPTION                                                                      
		When NO_DATA_FOUND THEN                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);                                                                  
			p_returnCd := 3;                                                             
		When DUP_VAL_ON_INDEX then                                                    
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                  
			p_returnCd := 2;                                                             
		When OTHERS Then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);                                                                   
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORT_ATTRI' || SQLCODE|| ' '|| SQLERRM);                                                                       
END PR_TRANSPORT_ATTRI;
/


-- PR_TRANSPORT_ATTRS.sql

CREATE OR REPLACE PROCEDURE pr_transport_attrs (
   p_returncd          OUT      PLS_INTEGER,
   p_debugflg          IN       PLS_INTEGER,
   rc1                 OUT      db_ref.rct,
   p_transportattrid   IN       NUMBER,
   p_transportid       IN       NUMBER DEFAULT NULL
)
AS
/*
   ===============================================================================
=
   Proc Name:       PR_TRANSPORT_ATTRS
   Create Date:     2004-03-12 14:26:16
   Author:          SYSTEM

                      ::::::::::::::::::::::::::::::
                            M O D I F I C A T I O N
                      ::::::::::::::::::::::::::::::

   ---------- ------------- ------ ---------------------------------------------
   DATE       NAME          Ver    DESCRIPTION
   ---------- ------------- ------ ---------------------------------------------
   2004-03-12 SYSTEM           3.5 Created

   ===============================================================================
=
   Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
   v_rowcnt         INTEGER         DEFAULT 0;
   v_begints        TIMESTAMP       DEFAULT fn_get_date;
                                             -- Performance related variables
   v_procname       VARCHAR2 (30)   DEFAULT 'PR_TRANSPORT_ATTRS';
   v_exceptionflg   INTEGER         DEFAULT 0;
   v_sqlstmt        VARCHAR2 (4000);
   v_where          VARCHAR2 (4000);
   v_paramstxt      VARCHAR2 (4000)
      DEFAULT    'p_DebugFlg=>'
              || p_debugflg
              || CHR (10)
              || ', p_TRANSPORID=>'
              || p_transportid
              || CHR (10)
              || ', p_TRANSPORTATTRID=>'
              || p_transportattrid
              || CHR (10);
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
   v_where :=
         ' Where 1=1'
      || fn_build_where ('TRANSPORTID', 'NUMBER', p_transportid)
      || fn_build_where ('TRANSPORTATTRID', 'NUMBER', p_transportattrid);
	  
   v_sqlstmt :=
        'SELECT                                                     
		 TRANSPORTATTRID                                                               
		,TRANSPORTID                                                                  
		,ROWTS                                                                        
		,RTRIM(ATTRNAME) ATTRNAME                                                     
		,CREATEDATE                                                                   
		,RTRIM(RESOURCEKEY) RESOURCEKEY                                               
		,RTRIM(DEFAULTATTRVALUE) DEFAULTATTRVALUE                                     
		,RTRIM(DESCRIPTION) DESCRIPTION                                               
	     FROM 
		 PR_TRANSPORT_ATTR'
         || v_where;

   IF rc1%ISOPEN
   THEN
      CLOSE rc1;
   END IF;

   OPEN rc1
    FOR v_sqlstmt;

   p_returncd := 0;
   db_procauditlogi_ah1 (v_procname,
                         v_begints,
                         v_paramstxt,
                         p_debugflg,
                         v_exceptionflg
                        );
END pr_transport_attrs;
/


-- PR_TRANSPORT_ATTRU.sql

CREATE OR REPLACE Procedure PR_TRANSPORT_ATTRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT                                   
	,p_TRANSPORTATTRID in NUMBER                                                   
	,p_TRANSPORTID in NUMBER                                                       
	,p_ROWTS in NUMBER                                                             
	,p_ATTRNAME in VARCHAR2                                                        
	,p_RESOURCEKEY in VARCHAR2                                                     
	,p_DEFAULTATTRVALUE in VARCHAR2                                                
	,p_DESCRIPTION in VARCHAR2                                                     
) as                                                                            

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_TRANSPORT_ATTRU                                            
	Create Date:     2004-03-12 14:25:54                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-12 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                               
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_ATTRU';                    
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_TRANSPORTATTRID=>'||p_TRANSPORTATTRID||CHR(10)                         
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)                                 
		||', p_ROWTS=>'||p_ROWTS||CHR(10)                                             
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)                                       
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)                                 
		||', p_DEFAULTATTRVALUE=>'||p_DEFAULTATTRVALUE||CHR(10)                       
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)                                 
		;
                                                                            
v_RowTS      integer :=0;                                                       
zeroUpdate  Exception;                                                          
                                                                                
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	v_RowCnt := 0;                                                                 
	Update PR_TRANSPORT_ATTR
	SET                                                  
		TRANSPORTID=p_TRANSPORTID                                                     
		,      ROWTS= p_RowTS + 1                                                     
		,      ATTRNAME=p_ATTRNAME                                                    
		,      UPDATEDATE= v_BeginTS                                                  
		,      RESOURCEKEY=p_RESOURCEKEY                                              
		,      DEFAULTATTRVALUE=p_DEFAULTATTRVALUE                                    
		,      DESCRIPTION=p_DESCRIPTION                                              
	Where 
		TRANSPORTATTRID = p_TRANSPORTATTRID                                   
		And RowTS = p_RowTS returning RowTS into v_RowTS;                             
	v_RowCnt := SQL%rowcount;                                                      
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;                             
	IF RC1%isOpen THEN Close RC1; END IF;                                          
	OPEN RC1 for select v_RowTS RowTS from Dual;                                   
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                          
	EXCEPTION                                                                      
		WHEN TOO_MANY_ROWS Then                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);                                                                    
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                        
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_TRANSPORT_ATTRU: Too Many Rows');
		WHEN ZeroUpdate then                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);                                                                      
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                        
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORT_ATTRU: No Data Found'); 
		WHEN OTHERS then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);                                                                   
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORT_ATTRU:'||SQLERRM);      
End PR_TRANSPORT_ATTRU;
/


-- PR_TRANSPORT_GLOBAL_ATTRD.sql

CREATE OR REPLACE Procedure PR_TRANSPORT_GLOBAL_ATTRD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_TRANSPORTGLOBALATTRID in NUMBER                                             
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_TRANSPORT_GLOBAL_ATTRD                                     
	Create Date:     2004-03-11 22:48:53                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_GLOBAL_ATTRD';             
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_TRANSPORTGLOBALATTRID=>'||p_TRANSPORTGLOBALATTRID||CHR(10)             
		;
                                                                            
zeroDelete  Exception;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	Delete From PR_TRANSPORT_GLOBAL_ATTR                                           
	Where 
		TRANSPORTGLOBALATTRID = p_TRANSPORTGLOBALATTRID                       
	;                                                                              
	v_RowCnt := SQL%ROWCOUNT;                                                      
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;                             
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg,           
v_ExceptionFlg);                                                                
	p_ReturnCd := 0;                                                               
	Exception                                                                      
		When ZeroDelete Then                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORT_GLOBAL_ATTRD: No Data Found');                                                                        
		When OTHERS Then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt,p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);         
End PR_TRANSPORT_GLOBAL_ATTRD;
/


-- PR_TRANSPORT_GLOBAL_ATTRI.sql

CREATE OR REPLACE Procedure PR_TRANSPORT_GLOBAL_ATTRI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT                    
	,p_TRANSPORTID in NUMBER                                                       
	,p_ATTRNAME in VARCHAR2                                                        
	,p_ATTRVALUE in VARCHAR2                                                       
	,p_RESOURCEKEY in VARCHAR2                                                     
	,p_DESCRIPTION in VARCHAR2                                                     
	)
AS                                                                           

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_TRANSPORT_GLOBAL_ATTRI                                     
	Create Date:     2004-03-11 22:50:26                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             
v_PK_Value        number := 0;                                                  

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_GLOBAL_ATTRI';             
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)                                 
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)                                       
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)                                     
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)                                 
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)                                 
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	Insert into PR_TRANSPORT_GLOBAL_ATTR(                                          
		TRANSPORTID                                                                   
		,TRANSPORTGLOBALATTRID                                                        
		,ATTRNAME                                                                     
		,ATTRVALUE                                                                    
		,RESOURCEKEY                                                                  
		,DESCRIPTION                                                                  
	) Values (                                                                     
		p_TRANSPORTID                                                                 
		,PR_TRANSPORT_GLOBAL_ATTR_SEQ.nextval                                         
		,p_ATTRNAME                                                                   
		,p_ATTRVALUE                                                                  
		,p_RESOURCEKEY                                                                
		,p_DESCRIPTION                                                                
	);                                                                             
	p_ReturnCd := 0;                                                               
	v_RowCnt := sql%ROWCOUNT;                                                      
	IF RC1%isOpen THEN Close RC1; END IF;                                          
	Open RC1 FOR Select 
		PR_TRANSPORT_GLOBAL_ATTR_SEQ.currval ID, 1 RowTS From Dual;                                                                           
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                
	EXCEPTION                                                                      
		When NO_DATA_FOUND THEN                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);                                                                 
			p_returnCd := 3;                                                             
		When DUP_VAL_ON_INDEX then                                                    
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                    
			p_returnCd := 2;                                                             
		When OTHERS Then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORT_GLOBAL_ATTRI' || SQLCODE|| ' '||SQLERRM);                                                        
END PR_TRANSPORT_GLOBAL_ATTRI;
/


-- PR_TRANSPORT_GLOBAL_ATTRS.sql

CREATE OR REPLACE PROCEDURE PR_TRANSPORT_GLOBAL_ATTRS                           
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT   DB_REF.RCT            
	, p_TRANSPORTGLOBALATTRID IN NUMBER                                            
    , p_TRANSPORTID IN NUMBER  DEFAULT NULL                                                        
	)                                                                              
AS                                                                              

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_TRANSPORT_GLOBAL_ATTRS                                     
	Create Date:     2004-03-11 22:49:49                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_GLOBAL_ATTRS';             
v_ExceptionFlg    integer Default 0;    
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;                                        
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)     
        ||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)                                      
		||', p_TRANSPORTGLOBALATTRID=>'||p_TRANSPORTGLOBALATTRID||CHR(10)             
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN     
    v_where := ' Where 1=1'||
    FN_Build_Where('TRANSPORTID', 'NUMBER', p_TRANSPORTID) ||
    FN_Build_Where('TRANSPORTGLOBALATTRID', 'NUMBER', p_TRANSPORTGLOBALATTRID);
    
    v_sqlstmt := 
	'SELECT                                                          
		TRANSPORTGLOBALATTRID                                                        
		,TRANSPORTID                                                                   
		,ROWTS                                                                        
		,CREATEDATE                                                                   
		,RTRIM(ATTRNAME) ATTRNAME                                                     
		,RTRIM(ATTRVALUE) ATTRVALUE                                                   
		,RTRIM(RESOURCEKEY) RESOURCEKEY                                               
		,RTRIM(DESCRIPTION) DESCRIPTION                                               
	 FROM 
		PR_TRANSPORT_GLOBAL_ATTR ' || v_where 
     ;
    
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
	OPEN RC1 FOR v_sqlstmt ;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                    
End PR_TRANSPORT_GLOBAL_ATTRS;
/


-- PR_TRANSPORT_GLOBAL_ATTRU.sql

CREATE OR REPLACE Procedure PR_TRANSPORT_GLOBAL_ATTRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT                      
	,p_TRANSPORTID in NUMBER                                                       
	,p_TRANSPORTGLOBALATTRID in NUMBER                                             
	,p_ROWTS in NUMBER                                                             
	,p_ATTRNAME in VARCHAR2                                                        
	,p_ATTRVALUE in VARCHAR2                                                       
	,p_RESOURCEKEY in VARCHAR2                                                     
	,p_DESCRIPTION in VARCHAR2                                                     
) as                                                                            

/*                                                                             
	===============================================================================
=                                                                               
	Proc Name:       PR_TRANSPORT_GLOBAL_ATTRU                                     
	Create Date:     2004-03-11 22:49:23                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2004-03-11 SYSTEM           3.5 Created
                                       
	===============================================================================
=                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          integer Default 0;                                           
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables                                                                       
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_GLOBAL_ATTRU';             
v_ExceptionFlg    integer Default 0;                                            
v_ParamsTxt       varchar2(4000) Default                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)                                           
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)                                 
		||', p_TRANSPORTGLOBALATTRID=>'||p_TRANSPORTGLOBALATTRID||CHR(10)             
		||', p_ROWTS=>'||p_ROWTS||CHR(10)                                             
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)                                       
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)                                     
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)                                 
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)                                 
		;
                                                                            
v_RowTS      integer :=0;                                                       
zeroUpdate  Exception;                                                          
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	v_RowCnt := 0;                                                                 
	Update PR_TRANSPORT_GLOBAL_ATTR
	SET                                           
		TRANSPORTID=p_TRANSPORTID                                                     
		,      ROWTS= p_RowTS + 1                                                     
		,      UPDATEDATE= v_BeginTS                                                  
		,      ATTRNAME=p_ATTRNAME                                                    
		,      ATTRVALUE=p_ATTRVALUE                                                  
		,      RESOURCEKEY=p_RESOURCEKEY                                              
		,      DESCRIPTION=p_DESCRIPTION                                              
	Where 
		TRANSPORTGLOBALATTRID = p_TRANSPORTGLOBALATTRID                       
		And RowTS = p_RowTS returning RowTS into v_RowTS;                             
	v_RowCnt := SQL%rowcount;                                                      
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;                             
	IF RC1%isOpen THEN Close RC1; END IF;                                          
	OPEN RC1 for select v_RowTS RowTS from Dual;                                   
	p_ReturnCd := 0;                                                               
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                
	EXCEPTION                                                                      
		WHEN TOO_MANY_ROWS Then                                                       
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);                                                                 
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_TRANSPORT_GLOBAL_ATTRU: Too Many Rows');                                                                         
		WHEN ZeroUpdate then                                                          
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt,p_DebugFlg, 1);                                                                 
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);                                                                
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORT_GLOBAL_ATTRU: No Data Found');                                                                        
		WHEN OTHERS then                                                              
			p_ReturnCd := 1;                                                             
			v_ExceptionFlg := 1;                                                         
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);                                                                 
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORT_GLOBAL_ATTRU:'||SQLERRM);                                          
End PR_TRANSPORT_GLOBAL_ATTRU;
/


-- PR_TRANSPORT_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE PR_TRANSPORT_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_TRANSPORTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORT_LOC_VWS
	Create Date:     2003-09-26 00:10:27
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-03-05 S.Priani      4.2.2  Removed NLS

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_TRANSPORTID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		TRANSPORTID
		,RTRIM(TRANSPORTTYPE) TRANSPORTTYPE
		,ROWTS
		,CREATEDATE
		,VISIBILITY
		,UI_VISIBILITY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
	FROM
		PR_TRANSPORT
	Where
		TRANSPORTID = p_TRANSPORTID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		TRANSPORTID
		,RTRIM(TRANSPORTTYPE) TRANSPORTTYPE
		,ROWTS
		,CREATEDATE
		,VISIBILITY
		,UI_VISIBILITY
		,RTRIM(DESCRIPTION) DESCRIPTION
		,RTRIM(RESOURCEKEY) RESOURCEKEY
	FROM
		PR_TRANSPORT
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_TRANSPORT_LOC_VWS;
/


-- PR_TRANSPORT_PROXY_ATTRD.sql

Create or Replace Procedure PR_TRANSPORT_PROXY_ATTRD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,p_TRANSPORTPROXYATTRID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORT_PROXY_ATTRD
	Create Date:     2005-03-10 16:40:12
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_PROXY_ATTRD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTPROXYATTRID=>'||p_TRANSPORTPROXYATTRID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_TRANSPORT_PROXY_ATTR
	Where
		TRANSPORTPROXYATTRID = p_TRANSPORTPROXYATTRID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORT_PROXY_ATTRD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_TRANSPORT_PROXY_ATTRD;
/


-- PR_TRANSPORT_PROXY_ATTRI.sql

Create or Replace Procedure PR_TRANSPORT_PROXY_ATTRI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_TRANSPORTID in NUMBER
	,p_ATTRNAME in VARCHAR2
	,p_ATTRVALUE in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORT_PROXY_ATTRI
	Create Date:     2005-03-10 16:40:13
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_PROXY_ATTRI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_TRANSPORT_PROXY_ATTR(
		TRANSPORTPROXYATTRID
		,TRANSPORTID
		,ATTRNAME
		,ATTRVALUE
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
		,p_TRANSPORTID
		,p_ATTRNAME
		,p_ATTRVALUE
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		PR_TRANSPORT_PROXY_ATTR_SEQ.currval TRANSPORTPROXYATTRID, 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_TRANSPORT_PROXY_ATTRI' || SQLCODE|| ' '||SQLERRM);
END PR_TRANSPORT_PROXY_ATTRI;
/


-- PR_TRANSPORT_PROXY_ATTRS.sql

CREATE OR REPLACE PROCEDURE PR_TRANSPORT_PROXY_ATTRS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_TRANSPORTPROXYATTRID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORT_PROXY_ATTRS
	Create Date:     2005-03-10 16:40:14
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_PROXY_ATTRS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTPROXYATTRID=>'||p_TRANSPORTPROXYATTRID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_TRANSPORTPROXYATTRID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		TRANSPORTPROXYATTRID
		,TRANSPORTID
		,ROWTS
		,CREATEDATE
		,RTRIM(ATTRNAME) ATTRNAME
		,RTRIM(ATTRVALUE) ATTRVALUE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
	FROM
		PR_TRANSPORT_PROXY_ATTR
	Where
		TRANSPORTPROXYATTRID = p_TRANSPORTPROXYATTRID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		TRANSPORTPROXYATTRID
		,TRANSPORTID
		,ROWTS
		,CREATEDATE
		,RTRIM(ATTRNAME) ATTRNAME
		,RTRIM(ATTRVALUE) ATTRVALUE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
	FROM
		PR_TRANSPORT_PROXY_ATTR
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_TRANSPORT_PROXY_ATTRS;
/


-- PR_TRANSPORT_PROXY_ATTRS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_TRANSPORT_PROXY_ATTRS_AH1
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_TRANSPORTID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_TRANSPORT_PROXY_ATTRS
	Create Date:     2005-03-10 16:40:14
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_PROXY_ATTRS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_TRANSPORTID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		pr.TRANSPORTPROXYATTRID
		,pr.TRANSPORTID
		,RTRIM(pr.DESCRIPTION) DESCRIPTION
		,pr.ROWTS
		,pr.CREATEDATE
		,pr.UPDATEDATE
		,pr.attrname
		,pr.attrname
		,pr.resourcekey
	FROM
		PR_TRANSPORT_PROXY_ATTR   pr
	Where
		(pr.TRANSPORTID = p_TRANSPORTID AND p_TRANSPORTID IS NOT NULL) 
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		pr.TRANSPORTPROXYATTRID
		,pr.TRANSPORTID
		,RTRIM(pr.DESCRIPTION) DESCRIPTION
		,pr.ROWTS
		,pr.CREATEDATE
		,pr.UPDATEDATE
		,pr.attrname
		,pr.attrname
		,pr.resourcekey
	FROM
		PR_TRANSPORT_PROXY_ATTR   pr
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_TRANSPORT_PROXY_ATTRS_AH1;
/


-- PR_TRANSPORT_PROXY_ATTRU.sql

Create or Replace Procedure PR_TRANSPORT_PROXY_ATTRU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_TRANSPORTPROXYATTRID in NUMBER
	,p_TRANSPORTID in NUMBER
	,p_ROWTS in NUMBER
	,p_ATTRNAME in VARCHAR2
	,p_ATTRVALUE in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
) as

/*
	================================================================================
	Proc Name:       PR_TRANSPORT_PROXY_ATTRU
	Create Date:     2005-03-10 16:40:16
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2005-03-10 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_TRANSPORT_PROXY_ATTRU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_TRANSPORTPROXYATTRID=>'||p_TRANSPORTPROXYATTRID||CHR(10)
		||', p_TRANSPORTID=>'||p_TRANSPORTID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_ATTRNAME=>'||p_ATTRNAME||CHR(10)
		||', p_ATTRVALUE=>'||p_ATTRVALUE||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_TRANSPORT_PROXY_ATTR
	SET
		TRANSPORTID=p_TRANSPORTID
		,      ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      ATTRNAME=p_ATTRNAME
		,      ATTRVALUE=p_ATTRVALUE
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
	Where
		TRANSPORTPROXYATTRID = p_TRANSPORTPROXYATTRID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_TRANSPORT_PROXY_ATTRU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORT_PROXY_ATTRU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_TRANSPORT_PROXY_ATTRU:'||SQLERRM);
End PR_TRANSPORT_PROXY_ATTRU;
/


-- PR_USERI_CONTACT.sql

CREATE OR REPLACE PROCEDURE PR_USERI_CONTACT
(
    p_ReturnCd out pls_integer
    ,p_debugflg in pls_integer
    ,RC1 out db_ref.RCT
    ,p_PARTNERID in NUMBER
    ,p_USERNAME in varchar2
    ,p_PASSWORD in RAW
    ,p_NAME1 in varchar2
    ,p_NAME2 in varchar2
    ,p_NAME3 in varchar2
    ,p_ENABLEDFLG in NUMBER
    ,p_EMAIL in VARCHAR2
    ,p_PWDRESETFLG in NUMBER
    ,p_LanguageLocale in varchar2
    ,p_FormatLocale in varchar2
    ,p_TimeZone in varchar2
    ,p_PHONEBUS varchar2
    ,p_PHONEFAX varchar2          
    ,p_NOTIFYFLG pls_integer
    ,p_VISIBILITYFLG pls_integer
) AS
/*
================================================================================
Proc Name:      pr_USERi_contact
Create Date:    12/19/01 01:44:24 PM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
v_pk_value number := 0;
--- For Contact info
v_r_code     pls_integer;
v_debug      pls_integer;
RC4          db_ref.RCT;
v_contactid  pls_integer;
v_rowts      pls_integer;
v_chkpt pls_integer;
---
Procedure ProcAudit is
v_diffTS Timestamp;
begin
    DB_ProcAuditLogI_Ah1('pr_USERi',v_beginTS,'p_RETURNCD'||','||p_DEBUGflg||','||'RC1'||','||
    P_PARTNERID||','||P_USERNAME||','||P_PASSWORD||','||P_NAME1||','||P_NAME2||','||
    P_NAME3||','||P_ENABLEDFLG||','||P_EMAIL||','||P_PWDRESETFLG||','||p_LanguageLocale||
    ','||p_FormatLocale||','||p_TimeZone||
    ','||p_PHONEBUS||','||p_PHONEFAX||','||p_NOTIFYFLG||','||p_VISIBILITYFLG||' checkpoint: '||v_chkpt);
End ProcAudit;
--
Procedure Insert_contact is
begin
    Insert into PR_CONTACT(
        CONTACTID
        ,PARTNERID
        ,NAME1
        ,NAME2
        ,NAME3
        ,TITLE
        ,EMAIL
        ,ADDRESS
        ,PHONEBUS
        ,PHONEMBL
        ,PHONEPGR
        ,PHONEFAX
        ,NOTIFYFLG
        ,VISIBILITYFLG
        ,LanguageLocale
        ,FormatLocale
        ,TimeZone
    ) Values (
        PR_CONTACT_SEQ.nextval
        ,p_PARTNERID
        ,p_NAME1
        ,p_NAME2
        ,p_NAME3
        ,NULL  --p_TITLE
        ,p_EMAIL
        ,NULL  --p_ADDRESS
        ,p_PHONEBUS
        ,NULL  --p_PHONEMBL
        ,NULL  --p_PHONEPGR
        ,p_PHONEFAX
        ,p_NOTIFYFLG
        ,p_VISIBILITYFLG
        ,p_LanguageLocale
        ,p_FormatLocale
        ,p_TimeZone
    ) Returning CONTACTID INTO v_contactid;

EXCEPTION
  When Dup_val_on_index Then
       v_chkpt := 2;
       ProcAudit;
       p_returnCd :=2;
       PR_CONTACTd (p_ReturnCd =>v_r_code,
                     p_debugFlg    =>v_debug,
                     p_CONTACTID =>v_contactid);
  When Others Then
       v_chkpt := 3;
       ProcAudit;
       Raise_application_error (-20999,'20000 pr_userI --Insert_contact ' || SQLCODE|| ' '||SQLERRM);
end insert_contact;
--
-- Main
--
BEGIN
v_chkpt := 1;
Procaudit;
insert_contact;
  begin
      Insert into pr_USER(
        USERID
        ,   contactId
        ,   PARTNERID
        ,   USERNAME
        ,   PASSWORD
        ,   ENABLEDFLG
        ,   PWDRESETFLG
          ) Values (
            PR_USER_SEQ.nextval
        ,   v_contactid
        ,   p_PARTNERID
        ,   p_USERNAME
        ,   p_PASSWORD
        ,   p_ENABLEDFLG
        ,   p_PWDRESETFLG);
   
       EXCEPTION
          When Dup_val_on_index Then
              v_chkpt := 5;
              p_returnCd :=2;
              ProcAudit;
              PR_CONTACTd (p_ReturnCd =>v_r_code,
                           p_debugFlg    =>v_debug,
                           p_CONTACTID =>v_contactid);
           When Others Then
             v_chkpt := 6;
             ProcAudit;
             PR_CONTACTd (p_ReturnCd =>v_r_code,
                          p_debugFlg    =>v_debug,
                          p_CONTACTID =>v_contactid);
             Raise_application_error (-20999,'20001' || SQLCODE|| ' '||SQLERRM);
             p_returnCd := 1;                           
  END;

  Begin
    IF p_ReturnCd > 0 THEN 
       OPEN RC1 For Select NULL, 1 RowTs From Dual;
    ELSE
       Open RC1 FOR Select PR_USER_SEQ.currval Id,1 ROWTS From Dual;
        p_returnCd := 0;
        ProcAudit;
    END IF;        
    
          EXCEPTION
          When NO_DATA_FOUND THEN NULL; 

   END;
     
END PR_USERI_contact;
/


-- PR_USERS.sql

CREATE OR REPLACE PROCEDURE PR_USERS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_USERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_USERS
	Create Date:     2003-09-26 00:06:21
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2007-08-20 BHARANI J      6.1.1 Modified for Feature F104631

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_USERS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_USERID=>'||p_USERID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_USERID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		USERID
		,PARTNERID
		,CONTACTID
		,RTRIM(USERNAME) USERNAME
		,PASSWORD
		,ENABLEDFLG
		,PWDRESETFLG
		,ROWTS
		,CREATEDATE
	FROM
		PR_USER
	Where
		USERID = p_USERID
		 AND coalesce(deleteflag,0) <> 1 ;
	
	ELSE
	OPEN RC1 FOR
	SELECT
		USERID
		,PARTNERID
		,CONTACTID
		,RTRIM(USERNAME) USERNAME
		,PASSWORD
		,ENABLEDFLG
		,PWDRESETFLG
		,ROWTS
		,CREATEDATE
	FROM
		PR_USER
		 where coalesce(deleteflag,0) <> 1;
		 
		 
END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_USERS;
/


-- PR_USERS_AH1.sql

CREATE OR REPLACE PROCEDURE PR_USERS_AH1
    (
    p_ReturnCd      OUT PLS_INTEGER
    , p_DebugFlg    IN  PLS_INTEGER
    , RC1           OUT DB_REF.RCT
    , P_BUSINESSID  IN  VARCHAR2
    , P_USERNAME    IN  VARCHAR2
    )
AS

/*
    ================================================================================
    Proc Name:       PR_USERS_AH1
    Create Date:     2003-10-08 00:06:21
    Author:          V.Goel

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-10-08 V.Goel           3.5 Created
    2007-08-20 BHARANI J      6.1.1 Modified for Feature F104631

    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_USERS_AH1';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', P_BUSINESSID=>'||P_BUSINESSID||CHR(10)
        ||', p_UserName=>'||p_UserName||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    SELECT
        PASSWORD
    FROM
        PR_USER U
        Inner join pr_busidentifier B on U.PARTNERID = B.PARTNERID
    WHERE
        U.UserName = p_UserName
    AND B.BUSINESSID   = P_BUSINESSID
    AND coalesce(U.deleteflag,0) <> 1;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_USERS_AH1;
/


-- PR_USERS_AH2.sql

CREATE OR REPLACE PROCEDURE PR_USERS_AH2
    (
    p_ReturnCd      OUT PLS_INTEGER
    , p_DebugFlg    IN  PLS_INTEGER
    , RC1           OUT DB_REF.RCT
    , P_USERNAME    IN  VARCHAR2
    )
AS

/*
    ================================================================================
    Proc Name:       PR_USERS_AH2
    Create Date:     02/27/06 14:51:56 PM
    Author:          Shishir N

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-20 BHARANI J      6.1.1 Modified for Feature F104631
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_USERS_AH2';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_UserName=>'||p_UserName||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;
    OPEN RC1 FOR
    Select
		                    pu.userid
		                    , pu.username
		                    ,   ppt.partnertypename
		                    ,   ppt.Operatorflg
		            From    pr_USER pu, pr_partner  pp, pr_partnertype ppt
		            Where   pp.partnertypeid= ppt.partnertypeid
		            and     pu.partnerId = pp.partnerId
            and     pu.username = p_username
            AND coalesce(pu.deleteflag,0) <> 1;
    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_USERS_AH2;
/


-- PR_USERS_AH3.sql

CREATE OR REPLACE PROCEDURE PR_USERS_AH3
	(
	 p_ReturnCd	out	integer
	, p_DebugFlg	in	pls_integer
	, RC1		out	db_ref.RCT
	,p_USERNAME	in	VARCHAR2
) as

/*
	================================================================================
	Proc Name:       PR_USERS_AH3
	Create Date:     2007-08-24 08:21:26
	Author:          S.Priani

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

        ---------  ------------ -------	---------------------------------------------
        DATE       NAME         Version	DESCRIPTION
        ---------  ------------ -------	---------------------------------------------
	2007-08-24 S.Priani	6.1.1	Created for 104489

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


	v_RowCnt          integer Default 0;
	v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
	v_ProcName        varchar2(30) Default 'PR_USERS_AH3';
	v_ExceptionFlg    integer Default 0;
	v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_USERNAME=>'||p_USERNAME||CHR(10)
		;

	v_RowTS      integer :=0;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 FOR
		SELECT	u.userid
			,u.partnerid
			,u.contactid
			,u.password
			,b.busidentifierid
			,b.businessid
			,b.businessidtype
		FROM	pr_user u
		LEFT OUTER JOIN pr_busidentifier b ON b.partnerid = u.partnerid
		WHERE	u.username = p_username
		;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE;
End PR_USERS_AH3;
/


-- PR_USERS_BYPARTNER.sql

CREATE OR REPLACE PROCEDURE PR_USERS_BYPARTNER
(
    p_ReturnCd      out pls_integer
    , p_debugFlg    in pls_integer
    , RC1           out db_ref.RCT
    , p_PartnerId   in Number
    , p_Locale      in varchar2
) As
/*
    ================================================================================
    Proc Name:       PR_USERS_BYPARTNER
    Create Date:     12/20/01 02:41:58 PM
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-20 BHARANI J      6.1.1 Modified for Feature F104631
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
v_diffTS Timestamp;
begin
    DB_ProcAuditLogI_Ah1('pr_users_ByPartner',v_beginTS,'p_return_code'||','||p_debugFlg||','||
    P_PARTNERID||','||'RC1');
End ProcAudit;
BEGIN
    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);

    OPEN RC1 FOR
    SELECT
    USERID
    ,   u.PARTNERID
    ,   RTRIM(USERNAME)  USERNAME
    ,   PASSWORD
    ,   RTRIM(c.NAME1)  NAME1
    ,   RTRIM(c.NAME2)  NAME2
    ,   RTRIM(c.NAME3)  NAME3
    ,   u.ENABLEDFLG
    ,   RTRIM(c.EMAIL)  EMAIL
    ,   PWDRESETFLG
    ,   c.NotifyFlg
    ,   c.contactid
    ,   c.Title
    ,   RTRIM(c.LanguageLocale) LanguageLocale
    ,   RTRIM(c.FormatLocale) FormatLocale
    ,   RTRIM(c.TimeZone) TimeZone
    FROM pr_USER u
    left outer join pr_contact c on c.contactid =u.contactid
    WHERE u.PARTNERID = p_PARTNERID
    AND coalesce(U.deleteflag,0) <> 1
    ORDER BY UPPER(USERNAME);

    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';

    p_returnCd := 0;
    procAudit;
    Exception
     when others then
      ProcAudit;
      p_returnCd := 1;
END PR_USERS_BYPARTNER;
/


-- PR_USERS_FTPUSER.sql

CREATE OR REPLACE PROCEDURE PR_USERS_FTPUSER
    (
    p_ReturnCd	OUT pls_INTEGER
    ,p_DebugFlg	IN pls_INTEGER
    ,RC1 out db_ref.RCT
    ,RC2 out db_ref.RCT
    ,p_sortkey	IN varchar2
    ,p_PageSizeNbr IN NUMBER
    ,p_PageNbr IN NUMBER
    ,P_USERID IN VARCHAR2
    )
    AS
    
/*
================================================================================
    Proc Name:       PR_USERS_FTPUSER
    Create Date:     07/09/13 11:06:12 AM
    Author:          Amit Kumar Vijay

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ --------------------------------------------
-
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ --------------------------------------------
-

    ============================================================================
====
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--==============================================================================
==
-- *
-- *
--==============================================================================
==
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
 v_RowCnt          integer :=0;
 v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
 v_ProcName        varchar2(30) := 'PR_USERS_FTPUSER';
 v_ExceptionFlg    integer := 0;
 v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_sortkey=>'||p_sortkey||CHR(10)
        ||', p_PageSizeNbr=>'||p_PageSizeNbr||CHR(10)
        ||', p_PageNbr=>'||p_PageNbr||CHR(10)
        ||', P_USERID=>'||P_USERID||CHR(10)
        ;
 v_RowTS        		integer :=0;

  sqlstmt            varchar2(4000) DEFAULT '';
  sqlstmt2           varchar2(4000) DEFAULT '';
  v_where           varchar2(1000) DEFAULT '';
  v_where1           varchar2(1000) DEFAULT '';
  v_RC1		    db_ref.RCT;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN

--    where_clause;
   v_where := ' where 1=1'||
        FN_Build_Where('pu.USERNAME'      ,'VARCHAR2_LIKE'   , P_USERID);


     sqlstmt := 'SELECT '
		|| ' fu.FTPUSERID '
		|| ', fu.USERNAME AS USERNAME '
		|| ', fu.enableflag '
		|| ', fu.homedirectory '
		|| ', fu.writepermission '
		|| ', pp.partnerid '
		|| ', pp.partnername '
		|| ' From      CF_FTP_USER fu '
		|| ' INNER JOIN pr_USER pu ON fu.USERNAME = pu.USERNAME '
		|| ' INNER JOIN pr_partner pp ON pp.partnerid = pu.partnerid ' ||v_where;

sqlstmt2 := sqlstmt;
    if p_sortkey is not null then
        sqlstmt := sqlstmt ||' order by '||p_sortkey;
    end if;
    
-- total row count Cursor
    if RC2%IsOpen then close RC2; end if;
    if NVL(p_pageNbr,0) in (0,1) then
      open RC2 for 'select count(*) TotalRowsNbr from ('||sqlstmt||')';
     end if;


-- Paging logic
     sqlstmt := FN_Paging_Logic(p_pagenbr, p_pagesizenbr,sqlstmt);
     DB_Print('Sql ', sqlstmt);

-- Open Cursor
    if RC1%IsOpen then close RC1; end if;
        open RC1 for  sqlstmt;

        P_RETURNCD := 0;
        
EXCEPTION
	WHEN OTHERS then
		ROLLBACK work;
		p_ReturnCd := 1;
		v_ExceptionFlg := 1;
		DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
		Commit;
		RAISE_APPLICATION_ERROR(-20999,'20100 - PR_USERS_FTPUSER:'||SQLERRM);

END PR_USERS_FTPUSER;
/
  

-- PR_USERS_GROUP.sql

CREATE OR REPLACE PROCEDURE PR_USERS_GROUP
(
    p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,RC1 in out db_ref.RCT
    ,p_UserID in NUMBER
    , p_Locale      in varchar2
)
AS
/*
================================================================================
Proc Name:      pr_Users_GROUP
Create Date:    12/20/01 04:51:56 PM
Author          SYSTEM
================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
v_diffTS Timestamp;
begin
    DB_ProcAuditLogI_Ah1('pr_USERs',v_beginTS,'p_RETURNCd'||','||p_DEBUGFLG||','||
    P_UserID||','||'RC1');
End ProcAudit;
BEGIN
    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);

    OPEN RC1 FOR
    SELECT
        ug.userid
        ,g.GROUPID
        ,PARTNERID
        ,RTRIM(GROUPNAME)  GROUPNAME
        ,RTRIM(GROUPDESC)  GROUPDESC
    FROM pr_GROUP g, pr_user_group  ug
    WHERE g.GROUPID= ug.GROUPID
    AND   ug.userid = p_userId
    ORDER BY GROUPNAME;

    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';

    p_ReturnCd := 0;
    procAudit;
End PR_USERS_GROUP;
/


-- PR_USERS_GROUPMEMBER.sql

CREATE OR REPLACE PROCEDURE PR_USERS_GROUPMEMBER
(
    p_ReturnCd      out pls_integer
    , p_debugFlg    in pls_integer
    , RC1           out db_ref.RCT
    , p_GroupId   in Number
    , p_Locale      in varchar2
) As
/*
    ================================================================================
    Proc Name:       pr_USERs_GroupMember
    Create Date:     12/20/01 02:41:58 PM
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-20 BHARANI J      6.1.1 Modified for Feature F104631
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
v_diffTS Timestamp;
begin
    DB_ProcAuditLogI_Ah1('pr_USERs',v_beginTS,'p_RETURNCD'||','||p_DEBUGFLg||','||
    P_GroupID||','||'RC1');
End ProcAudit;
BEGIN
    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ' || FN_GET_NLS_SORT(p_Locale);

    OPEN RC1 FOR
    SELECT
        u.USERID
    ,   u.PARTNERID
    ,   RTRIM(u.USERNAME)  USERNAME
    ,   u.PASSWORD
    ,   RTRIM(c.NAME1)  NAME1
    ,   RTRIM(c.NAME2)  NAME2
    ,   RTRIM(c.NAME3)  NAME3
    ,   u.ENABLEDFLG
    ,   RTRIM(c.EMAIL)  EMAIL
    ,   u.PWDRESETFLG
    ,   RTRIM(c.LanguageLocale) LanguageLocale
    ,   RTRIM(c.FormatLocale) FormatLocale
    ,   RTRIM(c.TimeZone) TimeZone
    FROM pr_USER U
    inner join pr_user_Group ug on u.USERID = ug.USERID
    left outer join pr_contact c on c.contactid = u.contactid
    WHERE ug.GroupId = p_groupId
    AND coalesce(U.deleteflag,0) <> 1
    ORDER BY USERNAME;

    EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_SORT = ''binary''';

    p_ReturnCd := 0;
    procAudit;
    Exception
     when others then
      ProcAudit;
      p_ReturnCD := 1;
END  PR_USERS_GROUPMEMBER;
/


-- PR_USERS_NAME.sql

CREATE OR REPLACE PROCEDURE         PR_USERS_NAME
(
   p_ReturnCd out pls_integer
   ,p_DebugFlg in pls_integer
   ,RC1 out db_ref.RCT
   ,p_PartnerId in NUMBER
   ,p_UserName in varchar2
) As
/*
    ================================================================================
    Proc Name:       pr_USERs_name
    Create Date:     12/27/01 07:34:17 AM
    Author:          SYSTEM

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-20 BHARANI J      6.1.1 Modified for Feature F104631
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables
Procedure ProcAudit is
begin
  DB_ProcAuditLogI_Ah1('Pr_UserS_Name',v_beginTS,'p_RETURNCD'||','||p_DEBUGFlg||','||
                            P_PARTNERID||','||P_USERNAME||','||'RC1');
End ProcAudit;
BEGIN
 Open RC1 for
   Select
            USERID
        ,   u.PARTNERID
        ,   RTRIM(USERNAME)  USERNAME
        ,   PASSWORD
        ,   RTRIM(c.NAME1)  NAME1
        ,   RTRIM(c.NAME2)  NAME2
        ,   RTRIM(c.NAME3)  NAME3
        ,   ENABLEDFLG
        ,   RTRIM(c.EMAIL)  EMAIL
        ,   PWDRESETFLG
        ,   RTRIM(c.LanguageLocale)  LanguageLocale
        ,   RTRIM(c.FormatLocale)  FormatLocale
        ,   RTRIM(c.TimeZone)  TimeZone
        ,   u.CONTACTID
        ,   u.RowTS
   From      pr_USER u
            left outer join pr_contact c on u.contactId = c.contactid
   Where     u.PARTNERID = p_PARTNERID
   And       USERName Like p_USERName
   AND coalesce(U.deleteflag,0) <> 1;

  p_ReturnCd := 0;
  procAudit;
  Exception
 when others then
  ProcAudit;
  p_ReturnCd := 1;
End pr_USERs_name;
/


-- PR_USERS_ROLE.sql

CREATE OR REPLACE PROCEDURE PR_USERS_ROLE
(
    p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,RC1 in out db_ref.RCT
    ,p_UserID in NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       PR_USERS_ROLE
    Create Date:     02/27/06 14:51:56 PM
    Author:          Shishir N

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2007-08-20 BHARANI J      6.1.1 Modified for Feature F104631
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_error integer     := 0;
v_errmsg varchar2(255);
v_rowcnt integer    :=0;
v_BeginTS timestamp := FN_Get_Date; -- Performance related variables

Procedure ProcAudit is
v_diffTS Timestamp;
begin
    DB_ProcAuditLogI_Ah1('pr_USERs',v_beginTS,'p_RETURNCd'||','||p_DEBUGFLG||','||
    P_UserID||','||'RC1');
End ProcAudit;
BEGIN
    OPEN RC1 FOR
    Select
		                    pu.userid
		                    , pu.username
		                    ,   ppt.partnertypename
		                    ,   ppt.Operatorflg
		            From    pr_USER pu, pr_partner  pp, pr_partnertype ppt
		            Where   pp.partnertypeid= ppt.partnertypeid
		            and     pu.partnerId = pp.partnerId
            and     pu.userid = p_userId
            AND coalesce(pu.deleteflag,0) <> 1;

    p_ReturnCd := 0;
    procAudit;
End PR_USERS_ROLE;
/


-- PR_USERU_AH1.sql

CREATE OR REPLACE PROCEDURE PR_USERU_AH1 
(
p_ReturnCd out pls_integer
,p_DebugFlg in pls_integer
,p_userid in number
) as
/*
    ================================================================================
    Proc Name:       PR_USERU_AH1
    Create Date:     2007-08-20
    Author:          BHARANI J

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          VER    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt     integer    :=0;
v_ProcName        varchar2(30) Default 'PR_USERU_AH1';
v_BeginTS    timestamp := FN_Get_Date; -- Performance related variables
v_rowTS      integer :=0;
zeroUpdate   Exception;
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_userid=>'||p_userid||CHR(10)
		;

v_username  varchar2(64);

BEGIN
   
     
	DELETE from pr_pwdhistory
	WHERE USERID = p_USERID;
    
	DELETE from pr_user_group
	WHERE USERID = p_USERID;
    
	SELECT USERNAME into v_username from PR_USER where USERID = p_USERID;
        
	UPDATE cf_ftp_user SET
	username=TO_CHAR(username) ||' deleted on ' ||  TO_CHAR(SYS_EXTRACT_UTC(SYSTIMESTAMP))
	WHERE username = v_username;
    
	Update PR_USER SET
        deleteflag=1,username=TO_CHAR(username) ||' deleted on ' ||  TO_CHAR(SYS_EXTRACT_UTC(SYSTIMESTAMP))
	WHERE USERID = p_USERID;
    
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
  
    
	db_procauditlogi_ah1 (v_procname,
                             v_begints,
                             v_paramstxt,
                             p_debugflg,
                             v_exceptionflg
                        );
	p_ReturnCd := 0;
    
    
    
EXCEPTION
    WHEN TOO_MANY_ROWS then 
    raise_application_error(-20999, '20100 - PR_USERU_AH1: Too Many Rows'||' Userid: '||p_userid);
    WHEN ZeroUpdate then 
    raise_application_error(-20999,'20101 - PR_USERU_AH1: No Data Found'||' Userid: '||p_userid);
    WHEN OTHERS then 
    raise_application_error(-20999,'20102 - PR_USERU_AH1:'||sqlerrm||' Userid: '||p_userid);
    p_ReturnCd := 1;
End PR_USERU_AH1;
/

-- PR_USERU_CONTACT.sql

CREATE OR REPLACE PROCEDURE PR_USERU_CONTACT (
p_ReturnCd out pls_integer
,p_DebugFlg in pls_integer
,RC1 out db_ref.RCT
,p_USERID in NUMBER
,p_PARTNERID in NUMBER
,p_USERNAME in varchar2
,p_PASSWORD in RAW
,p_NAME1 in varchar2
,p_NAME2 in varchar2
,p_NAME3 in varchar2
,p_ENABLEDFLG in NUMBER
,p_EMAIL in VARCHAR2
,p_PWDRESETFLG in NUMBER
,p_LanguageLocale in varchar2
,p_FormatLocale in varchar2
,p_TimeZone in varchar2
,p_ROWTS in NUMBER
,p_PHONEBUS varchar2
,p_PHONEFAX varchar2          
,p_NOTIFYFLG pls_integer
,p_VISIBILITYFLG pls_integer
) as
/*
================================================================================
Proc Name:      PR_USERU_CONTACT
Create Date:    01/21/02 12:22:50 PM
Author          SYSTEM
================================================================================
			   ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

---------- ------------ ------- ---------------------------------------------
DATE       NAME		Ver	DESCRIPTION
---------- ------------ ------- ---------------------------------------------
2007-10-03 AMIT KUMAR	6.1.1	FTU USER UPDATE
================================================================================
Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found

--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_rowcnt	integer    :=0;
v_BeginTS	timestamp := FN_Get_Date; -- Performance related variables
v_rowTS		integer :=0;
zeroUpdate	Exception;
NO_DATA_FOUND	Exception;
v_contactid     pr_user.contactid%type;
v_contact_rowTS pr_contact.rowts%type;
v_USERNAME	PR_USER.USERNAME%type Default NULL;


Procedure ProcAudit is
begin
    DB_ProcAuditLogI_Ah1('PR_USERU_CONTACT',v_beginTS, 'p_ReturnCd'||','||p_DebugFlg
    ||','||'RC1'
    ||','||p_USERID
    ||','||p_PARTNERID
    ||','||p_USERNAME
    ||','||p_PASSWORD
    ||','||p_NAME1
    ||','||p_NAME2
    ||','||p_NAME3
    ||','||p_ENABLEDFLG
    ||','||p_EMAIL
    ||','||p_PWDRESETFLG
    ||','||p_ROWTS
    ||','||p_LanguageLocale
    ||','||p_FormatLocale
    ||','||p_TimeZone
    ||','||p_PHONEBUS
    ||','||p_PHONEFAX
    ||','||p_NOTIFYFLG
    ||','||p_VISIBILITYFLG
    );
End ProcAudit;
BEGIN
    v_rowcnt := 0;
    
-- Update FTP USER
    BEGIN
    		SELECT USERNAME INTO v_USERNAME FROM PR_USER
    		WHERE USERID=p_USERID;
    		EXCEPTION
    			WHEN NO_DATA_FOUND THEN
    			v_USERNAME:=NULL;
    END;
    IF v_USERNAME IS NOT NULL THEN
    Update CF_FTP_USER SET
    USERNAME=p_USERNAME
    ,     UPDATEDATE= FN_Get_Date
    Where USERNAME=v_USERNAME;
    END IF;
    
    Update PR_USER SET
    USERID=p_USERID
    ,     PARTNERID=p_PARTNERID
    ,     USERNAME=p_USERNAME
    ,     PASSWORD=p_PASSWORD
    ,     ENABLEDFLG=p_ENABLEDFLG
    ,     PWDRESETFLG=p_PWDRESETFLG
    ,     ROWTS= p_ROWTS + 1
    ,     UPDATEDATE= FN_Get_Date
    Where USERID = p_USERID
    And ROWTS = p_ROWTS returning ROWTS, contactid into v_rowTS, v_contactid;
    v_rowcnt := SQL%rowcount;
    if (v_rowcnt = 0) then  raise ZeroUpdate ; end if;
--
-- Update Contact
--
    select rowTS into v_rowTS from pr_contact
    where contactId = v_contactId;
    Update pr_contact set
          NAME1=p_NAME1
    ,     NAME2=p_NAME2
    ,     NAME3=p_NAME3
    ,     EMAIL=p_EMAIL
    ,     ROWTS= v_rowTS + 1
    ,     UPDATEDATE= FN_Get_Date
    ,     LanguageLocale = p_LanguageLocale
    ,     FormatLocale = p_FormatLocale
    ,     TimeZone = p_TimeZone
    ,     PhoneBus = p_PHONEBUS
    ,     PhoneFax = p_PHONEFAX 
    ,     NotifyFlg = p_NOTIFYFLG
    ,     VISIBILITYFLG = p_VISIBILITYFLG
    Where contactId = v_contactId
    and   rowTS = v_rowTS;
    v_rowcnt := SQL%rowcount;
    if (v_rowcnt = 0) then  raise ZeroUpdate ; end if;
    OPEN RC1 for select v_rowTS ROWTS from Dual;
    p_ReturnCd := 0;
    ProcAudit;
EXCEPTION
    WHEN TOO_MANY_ROWS then ProcAudit;
    raise_application_error(-20999, '20100 - PR_USERU_CONTACT: Too Many Rows'||' Userid: '||p_userid||' Partnerid: '||p_partnerID);
    WHEN ZeroUpdate then ProcAudit;
    raise_application_error(-20999,'20101 - PR_USERU_CONTACT: No Data Found'||' Userid: '||p_userid||' Partnerid: '||p_partnerID);
    WHEN NO_DATA_FOUND then ProcAudit;
        raise_application_error(-20999,'20101 - PR_USERU_CONTACT: No Data Found'||' Userid: '||p_userid||' Partnerid: '||p_partnerID);
    WHEN OTHERS then ProcAudit;
    raise_application_error(-20999,'20102 - PR_USERU_CONTACT:'||sqlerrm||' Userid: '||p_userid||' Partnerid: '||p_partnerID);
    p_ReturnCd := 1;
End PR_USERu_contact;
/


-- PR_USER_GROUPD.sql

CREATE OR REPLACE Procedure PR_USER_GROUPD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_USERID in NUMBER
	,p_GROUPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_USER_GROUPD
	Create Date:     2003-09-26 08:24:25
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_USER_GROUPD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_USERID=>'||p_USERID||CHR(10)
		||', p_GROUPID=>'||p_GROUPID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_USER_GROUP
	Where
		USERID = p_USERID
		AND GROUPID = p_GROUPID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_USER_GROUPD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_USER_GROUPD;
/


-- PR_USER_GROUPI.sql

CREATE OR REPLACE Procedure PR_USER_GROUPI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_USERID in NUMBER
	,p_GROUPID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_USER_GROUPI
	Create Date:     2003-09-26 00:03:53
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_USER_GROUPI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_USERID=>'||p_USERID||CHR(10)
		||', p_GROUPID=>'||p_GROUPID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_USER_GROUP(
		USERID
		,GROUPID
	) Values (
		p_USERID
		,p_GROUPID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_USER_GROUPI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_USER_GROUPI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_USER_GROUPI' || SQLCODE|| ' '||SQLERRM);
END PR_USER_GROUPI;
/


-- PR_VARWORKFLOWHANDLERTYPED.sql

CREATE OR REPLACE Procedure PR_VARWORKFLOWHANDLERTYPED
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_VARWORKFLOWHANDLERTYPEID in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_VARWORKFLOWHANDLERTYPED
	Create Date:     2004-03-16 10:23:57
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_VARWORKFLOWHANDLERTYPED';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VARWORKFLOWHANDLERTYPEID=>'||p_VARWORKFLOWHANDLERTYPEID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_VARWORKFLOWHANDLERTYPE
	Where 
		VARWORKFLOWHANDLERTYPEID = p_VARWORKFLOWHANDLERTYPEID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_VARWORKFLOWHANDLERTYPED: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_VARWORKFLOWHANDLERTYPED;
/


-- PR_VARWORKFLOWHANDLERTYPEI.sql

CREATE OR REPLACE Procedure PR_VARWORKFLOWHANDLERTYPEI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
    ,RC1 out db_ref.RCT 
	,p_VARWORKFLOWHANDLERTYPEID in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_VARWORKFLOWHANDLERTYPEI
	Create Date:     2004-03-16 10:25:08
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_VARWORKFLOWHANDLERTYPEI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VARWORKFLOWHANDLERTYPEID=>'||p_VARWORKFLOWHANDLERTYPEID||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_VARWORKFLOWHANDLERTYPE(
		VARWORKFLOWHANDLERTYPEID
		,RESOURCEKEY
		,DESCRIPTION
	) Values (
		p_VARWORKFLOWHANDLERTYPEID
		,p_RESOURCEKEY
		,p_DESCRIPTION
	);
	p_ReturnCd := 0;
    IF RC1%isOpen THEN Close RC1; END IF;                                          
	Open RC1 FOR Select p_VARWORKFLOWHANDLERTYPEID, 1 RowTS From Dual;    
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_VARWORKFLOWHANDLERTYPEI' || SQLCODE|| ' '||SQLERRM);
END PR_VARWORKFLOWHANDLERTYPEI;
/


-- PR_VARWORKFLOWHANDLERTYPES.sql

CREATE OR REPLACE PROCEDURE PR_VARWORKFLOWHANDLERTYPES
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_VARWORKFLOWHANDLERTYPEID IN VARCHAR2
	)
AS

/*
	================================================================================
	Proc Name:       PR_VARWORKFLOWHANDLERTYPES
	Create Date:     2004-03-16 10:24:46
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_VARWORKFLOWHANDLERTYPES';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VARWORKFLOWHANDLERTYPEID=>'||p_VARWORKFLOWHANDLERTYPEID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_VARWORKFLOWHANDLERTYPEID IS NOT NULL 
 THEN
	OPEN RC1 FOR 
	SELECT
		RTRIM(VARWORKFLOWHANDLERTYPEID) VARWORKFLOWHANDLERTYPEID
		,ROWTS
		,CREATEDATE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
	FROM 
		PR_VARWORKFLOWHANDLERTYPE
	Where 
		VARWORKFLOWHANDLERTYPEID = p_VARWORKFLOWHANDLERTYPEID
	;
	ELSE
	OPEN RC1 FOR 
	SELECT
		RTRIM(VARWORKFLOWHANDLERTYPEID) VARWORKFLOWHANDLERTYPEID
		,ROWTS
		,CREATEDATE
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,RTRIM(DESCRIPTION) DESCRIPTION
	FROM 
		PR_VARWORKFLOWHANDLERTYPE
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_VARWORKFLOWHANDLERTYPES;
/


-- PR_VARWORKFLOWHANDLERTYPEU.sql

CREATE OR REPLACE Procedure PR_VARWORKFLOWHANDLERTYPEU
	(
	 p_ReturnCd out integer
	, p_DebugFlg in pls_integer
	, RC1 out db_ref.RCT
	,p_VARWORKFLOWHANDLERTYPEID in VARCHAR2
	,p_RESOURCEKEY in VARCHAR2
	,p_DESCRIPTION in VARCHAR2
	,p_ROWTS in NUMBER
) as

/*
	================================================================================
	Proc Name:       PR_VARWORKFLOWHANDLERTYPEU
	Create Date:     2004-03-16 10:24:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-16 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_VARWORKFLOWHANDLERTYPEU';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VARWORKFLOWHANDLERTYPEID=>'||p_VARWORKFLOWHANDLERTYPEID||CHR(10)
		||', p_ROWTS=>'||p_ROWTS||CHR(10)
		||', p_RESOURCEKEY=>'||p_RESOURCEKEY||CHR(10)
		||', p_DESCRIPTION=>'||p_DESCRIPTION||CHR(10)
		;

v_RowTS      integer :=0;
zeroUpdate  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	v_RowCnt := 0;
	Update PR_VARWORKFLOWHANDLERTYPE
	SET
		ROWTS= p_RowTS + 1
		,      UPDATEDATE= v_BeginTS
		,      RESOURCEKEY=p_RESOURCEKEY
		,      DESCRIPTION=p_DESCRIPTION
	Where 
		VARWORKFLOWHANDLERTYPEID = p_VARWORKFLOWHANDLERTYPEID
		And RowTS = p_RowTS returning RowTS into v_RowTS;
	v_RowCnt := SQL%rowcount;
	if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
	IF RC1%isOpen THEN Close RC1; END IF;
	OPEN RC1 for select v_RowTS RowTS from Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		WHEN TOO_MANY_ROWS Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999, '20100 - PR_VARWORKFLOWHANDLERTYPEU: Too Many Rows');
		WHEN ZeroUpdate then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS,'ZeroUpdate, '||v_ParamsTxt, p_DebugFlg, 1);
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_VARWORKFLOWHANDLERTYPEU: No Data Found');
		WHEN OTHERS then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR(-20999,'20100 - PR_VARWORKFLOWHANDLERTYPEU:'||SQLERRM);
End PR_VARWORKFLOWHANDLERTYPEU;
/


-- PR_VARWORKFLOWHANDLER_CATD.sql

CREATE OR REPLACE Procedure PR_VARWORKFLOWHANDLER_CATD
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer 
	,p_VARWORKFLOWHANDLERTYPEID in VARCHAR2
	,p_HANDLERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_VARWORKFLOWHANDLER_CATD
	Create Date:     2004-03-12 17:45:30
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_VARWORKFLOWHANDLER_CATD';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VARWORKFLOWHANDLERTYPEID=>'||p_VARWORKFLOWHANDLERTYPEID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;

zeroDelete  Exception;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    v_where := ' WHERE 1=1 ' ;
    v_where := v_where || FN_BUILD_WHERE('VARWORKFLOWHANDLERTYPEID', 'VARCHAR2', p_VARWORKFLOWHANDLERTYPEID) ;
	v_where := v_where || FN_BUILD_WHERE('HANDLERID', 'NUMBER', p_HANDLERID) ;
	
    v_sqlstmt := 
	   'Delete From PR_VARWORKFLOWHANDLER_CAT ' || v_where  ;
	
	execute immediate v_sqlstmt ;
	
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_VARWORKFLOWHANDLER_CATD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_VARWORKFLOWHANDLER_CATD;
/


-- PR_VARWORKFLOWHANDLER_CATI.sql

CREATE OR REPLACE Procedure PR_VARWORKFLOWHANDLER_CATI
	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT    
	,p_VARWORKFLOWHANDLERTYPEID in VARCHAR2
	,p_HANDLERID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_VARWORKFLOWHANDLER_CATI
	Create Date:     2004-03-12 18:02:37
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_VARWORKFLOWHANDLER_CATI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VARWORKFLOWHANDLERTYPEID=>'||p_VARWORKFLOWHANDLERTYPEID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_VARWORKFLOWHANDLER_CAT(
		VARWORKFLOWHANDLERTYPEID
		,HANDLERID
	) Values (
		p_VARWORKFLOWHANDLERTYPEID
		,p_HANDLERID
	);
	p_ReturnCd := 0;
	Open RC1 FOR Select 1 RowTS From Dual;  
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			p_returnCd := 3;
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_VARWORKFLOWHANDLER_CATI' || SQLCODE|| ' '||SQLERRM);
END PR_VARWORKFLOWHANDLER_CATI;
/


-- PR_VARWORKFLOWHANDLER_CATS.sql

CREATE OR REPLACE PROCEDURE PR_VARWORKFLOWHANDLER_CATS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_VARWORKFLOWHANDLERTYPEID IN VARCHAR2
	, p_HANDLERID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_VARWORKFLOWHANDLER_CATS
	Create Date:     2004-03-12 18:08:25
	Author:          SYSTEM 

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2004-03-12 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_VARWORKFLOWHANDLER_CATS';
v_ExceptionFlg    integer Default 0;
v_sqlstmt         varchar2(4000) ;
v_where           varchar2(4000) ;                                
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_VARWORKFLOWHANDLERTYPEID=>'||p_VARWORKFLOWHANDLERTYPEID||CHR(10)
		||', p_HANDLERID=>'||p_HANDLERID||CHR(10)
		;


---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN

    v_where := ' Where 1=1'||
       FN_Build_Where('VARWORKFLOWHANDLERTYPEID', 'VARCHAR2', p_VARWORKFLOWHANDLERTYPEID) ||
       FN_Build_Where('c.HANDLERID', 'NUMBER', p_HANDLERID);
	
	v_sqlstmt := 
		'SELECT '
		||'RTRIM(c.VARWORKFLOWHANDLERTYPEID) VARWORKFLOWHANDLERTYPEID'
		||',c.HANDLERID'   
		||',h.PROVIDERFLG'                                                                                                                         
		||',RTRIM(h.RESOURCEKEY) RESOURCEKEY'                                               
		||',RTRIM(h.DESCRIPTION) DESCRIPTION'                                               
		||',RTRIM(h.CLASSNAME) CLASSNAME'    
		||',c.CREATEDATE'
		||',c.ROWTS '     
    	||'FROM ' 
		||'PR_VARWORKFLOWHANDLER_CAT c, PR_HANDLER h'
		|| v_where
		|| ' AND c.HANDLERID = h.HANDLERID'
	;
		
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
	OPEN RC1 FOR v_sqlstmt ;

	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_VARWORKFLOWHANDLER_CATS;
/


-- PR_WORKFLOWD_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_WORKFLOWD_AH1
(
   p_ReturnCd out pls_integer
    ,p_DebugFlg in pls_integer
    ,p_WORKFLOWID in NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       PR_WORKFLOWD
    Create Date:     2003-04-29 13:46:04
    Author:          L.Ling

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-29 L.Ling        3.5    Created.
    2003-09-29 M.Mount       4.2.1  Removed commits and Rollbacks
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_WORKFLOWD';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
        ;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
   p_ReturnCd := 0;

   BEGIN
        Delete From PR_WORKFLOW
        Where   WORKFLOWID = p_WORKFLOWID;

        DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);

        Exception
            When NO_DATA_FOUND Then
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND when deleting PR_WORKFLOW,'||v_ParamsTxt, p_DebugFlg, 1);
--                Commit;
                RAISE_APPLICATION_ERROR (-20999,'20002 PR_WORKFLOWD when deleting PR_WORKFLOW : No Data Found');
            When OTHERS Then
--                Rollback work;
                p_ReturnCd := 1;
                v_ExceptionFlg := 1;
                DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
 --               Commit;
                RAISE_APPLICATION_ERROR (-20999,'20000' || SQLCODE|| ' '||SQLERRM);
   END;
 --  COMMIT;
End PR_WORKFLOWD_AH1;
/


-- PR_WORKFLOWINTERACT_LNKD.sql

CREATE OR REPLACE Procedure PR_WORKFLOWINTERACT_LNKD

	(p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer

	,p_WORKFLOWID in NUMBER
	,p_INTERACTID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_WORKFLOWINTERACT_LNKD
	Create Date:     2003-09-26 08:24:26
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_WORKFLOWINTERACT_LNKD';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_INTERACTID=>'||p_INTERACTID||CHR(10)
		;

zeroDelete  Exception;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Delete From PR_WORKFLOWINTERACT_LNK
	Where
		WORKFLOWID = p_WORKFLOWID
		AND INTERACTID = p_INTERACTID
	;
	v_RowCnt := SQL%ROWCOUNT;
	if (v_RowCnt = 0) then  raise ZeroDelete ; end if;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	p_ReturnCd := 0;
	Exception
		When ZeroDelete Then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'Zero Delete,'||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_WORKFLOWINTERACT_LNKD: No Data Found');
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20000 ' || SQLCODE|| ' '||SQLERRM);
End PR_WORKFLOWINTERACT_LNKD;
/


-- PR_WORKFLOWINTERACT_LNKI.sql

CREATE OR REPLACE Procedure PR_WORKFLOWINTERACT_LNKI
	(
	p_ReturnCd out pls_integer
	,p_DebugFlg in pls_integer
	,RC1 out db_ref.RCT
	,p_WORKFLOWID in NUMBER
	,p_INTERACTID in NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_WORKFLOWINTERACT_LNKI
	Create Date:     2003-09-26 00:03:54
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/

v_PK_Value        number := 0;

v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_WORKFLOWINTERACT_LNKI';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_INTERACTID=>'||p_INTERACTID||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	Insert into PR_WORKFLOWINTERACT_LNK(
		WORKFLOWID
		,INTERACTID
	) Values (
		p_WORKFLOWID
		,p_INTERACTID
	);
	p_ReturnCd := 0;
	v_RowCnt := sql%ROWCOUNT;
	IF RC1%isOpen THEN Close RC1; END IF;
	Open RC1 FOR Select
		 1 RowTS From Dual;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
	EXCEPTION
		When NO_DATA_FOUND THEN
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
			RAISE_APPLICATION_ERROR (-20999,'20000 PR_WORKFLOWINTERACT_LNKI' || SQLCODE|| ' '||SQLERRM);
		When DUP_VAL_ON_INDEX then
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'DUP_VAL_ON_INDEX, '||v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
			p_returnCd := 2;
			RAISE_APPLICATION_ERROR (-20999,'20001 PR_WORKFLOWINTERACT_LNKI' || SQLCODE|| ' '||SQLERRM);
		When OTHERS Then
			p_ReturnCd := 1;
			v_ExceptionFlg := 1;
			DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
			RAISE_APPLICATION_ERROR (-20999,'20002 PR_WORKFLOWINTERACT_LNKI' || SQLCODE|| ' '||SQLERRM);
END PR_WORKFLOWINTERACT_LNKI;
/


-- PR_WORKFLOWINTERACT_LNKS_AH1.sql

CREATE OR REPLACE PROCEDURE         PR_WORKFLOWINTERACT_LNKS_AH1
    (
    p_ReturnCd  OUT  PLS_INTEGER
    , p_DebugFlg  IN  PLS_INTEGER
    , RC1  OUT  DB_REF.RCT
    , p_Locale IN Varchar2
    , p_INTERACTID IN NUMBER
    )
AS

/*
    ================================================================================
    Proc Name:       PR_WORKFLOWINTERACT_LNKS_AH1
    Create Date:     2003-05-03 12:28:36
    Author:          J.Chen

                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::

    ---------- -------  ------ ---------------------------------------------
    DATE       NAME     Ver    DESCRIPTION
    ---------- -------  ------ ---------------------------------------------
    2003-05-03 J.Chen   3.5    Returns all workflows for given InteractId.
    2003-05-06 J.Chen   3.5    Changed p_locale to v_locale for null checking on p_locale.
    2003-05-30 L.Ling   3.5    Changed FN_GET_DEFAULT_LOC to FN_CHK_DEFAULT_LOCALE.
    2003-09-29 M.Mount  4.2.1  Removed Commit and rollback.
    2004-03-01 S.Priani 4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) := 'PR_WORKFLOWINTERACT_LNKS_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_INTERACTID=>'||p_INTERACTID||CHR(10)
        ;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
    IF RC1%ISOPEN THEN CLOSE RC1; END IF;

    OPEN RC1 FOR
    SELECT
        v.WORKFLOWID
        ,v.WorkFlowName
        ,v.WorkFlowDef
        ,v.EnabledFlg
    FROM
        PR_WORKFLOWINTERACT_LNK l
         Inner Join PR_WORKFLOW v On v.WorkFlowId = l.WorkFlowId
    Where
         l.INTERACTID = p_INTERACTID
    ;

    p_ReturnCd := 0;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --   commit;
    EXCEPTION
         When OTHERS Then
            RAISE_APPLICATION_ERROR (-20999,'20000-PR_WORKFLOWINTERACT_LNKS_AH1' || SQLCODE|| ' '||SQLERRM);
--            Rollback work;
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
--            Commit;
End PR_WORKFLOWINTERACT_LNKS_AH1;
/


-- PR_WORKFLOWI_AH1.sql

CREATE OR REPLACE PROCEDURE PR_WORKFLOWI_AH1
(
   p_ReturnCd out integer
   ,p_DebugFlg in pls_integer
   ,RC1 out db_ref.RCT
   ,p_LOCALE in VARCHAR2
   ,p_WORKFLOWNAME in VARCHAR2
   ,p_WORKFLOWDEF in VARCHAR2
   ,p_ENABLEDFLG in NUMBER
   ,p_WORKFLOWTYPEID in NUMBER
   ,p_WORKFLOWDISPLAYNAME in VARCHAR2
   ,p_RESOURCEKEY in VARCHAR2
   ,p_PROVIDERFLAG in NUMBER   
)
AS
/*
    ================================================================================
    Proc Name:       PR_WORKFLOWI_AH1
    Create Date:     2003-04-28 16:36:19
    Author:          L.Ling
                       ::::::::::::::::::::::::::::::
                             M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-28 L.Ling        3.5    Created
    2003-05-17 J.Chen        3.5    Replace p_WorkflowId with sequence Number for WorkflowId
    2004-03-05 S.Priani      4.2.2  Removed NLS
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_PK_Value        number := 0;
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_Get_Date; -- Performance related variables
v_WORKFLOWID       NUMBER;
v_ProcName        varchar2(30) := 'PR_WORKFLOWI_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
       'p_DebugFlg=>'||p_DebugFlg||CHR(10)
       ||', p_WORKFLOWNAME=>'||p_WORKFLOWNAME||CHR(10)
       ||', p_WORKFLOWDEF=>'||p_WORKFLOWDEF||CHR(10)
       ||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
       ||', p_WORKFLOWTYPEID=>'||p_WORKFLOWTYPEID||CHR(10)
       ||', p_LOCALE=>'||p_LOCALE||CHR(10)
       ||', p_WORKFLOWDISPLAYNAME=>'||p_WORKFLOWDISPLAYNAME||CHR(10);
PROCEDURE Get_WORKFLOWID AS
BEGIN
   Select WORKFLOWID
   into   v_WORKFLOWID
   From   PR_WORKFLOW
   Where  WORKFLOWNAME = p_WORKFLOWNAME;
END Get_WORKFLOWID;
---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
    Insert into PR_WORKFLOW(
         WorkflowId
        ,WORKFLOWNAME
        ,WORKFLOWDEF
        ,ENABLEDFLG
        ,WORKFLOWTYPEID
	,DESCRIPTION
  	,RESOURCEKEY
	,PROVIDERFLAG
    ) Values (
        pr_Workflow_Seq.Nextval
        ,p_WORKFLOWNAME
        ,p_WORKFLOWDEF
        ,p_ENABLEDFLG
        ,p_WORKFLOWTYPEID
	,p_WORKFLOWDISPLAYNAME
	,p_RESOURCEKEY
	,p_PROVIDERFLAG
    )RETURNING WORKFLOWID INTO v_WORKFLOWID;
    p_ReturnCd := 0;
  -- Commit;
    IF RC1%isOpen THEN Close RC1; END IF;
    Open RC1 FOR Select    v_WORKFLOWID ID, 1 RowTS From Dual;
    DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    EXCEPTION
        When NO_DATA_FOUND THEN
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, 'NO_DATA_FOUND, '||v_ParamsTxt, p_DebugFlg, 0);
            RAISE_APPLICATION_ERROR (-20999,'20000 PR_WORKFLOWI_AH1' || SQLCODE|| ' '||SQLERRM);
        When DUP_VAL_ON_INDEX THEN
           Get_WORKFLOWID;
        When OTHERS Then
            p_ReturnCd := 1;
            v_ExceptionFlg := 1;
            DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
            RAISE_APPLICATION_ERROR (-20999,'20000 PR_WORKFLOWI_AH1' || SQLCODE|| ' '||SQLERRM);
END PR_WORKFLOWI_AH1;
/


-- pr_workflowS_ah1.sql

CREATE OR REPLACE PROCEDURE PR_WORKFLOWS_AH1                                        
	(
	  p_ReturnCd   OUT PLS_INTEGER
	, p_DebugFlg   IN  PLS_INTEGER
	, RC1          OUT DB_REF.RCT                                                                      
	, p_fromsyntax IN  VARCHAR2
   , p_tosyntax   IN  VARCHAR2
	)                                                                              
AS                                                                              

/*                                                                             
   This proc selects workflows and sets a default flag of 1 in the record is present in the dis_imp_edi_syntax_def_actn
   table and also ensures that they are supported and not present in the dis_imp_edi_syntax_action_lnk table

	================================================================================                                                                               
	Proc Name:       PR_WORKFLOWS_AH1                                                  
	Create Date:     2005-12-06 13:01:53                                           
	Author:          SYSTEM 
                                                      
	                   ::::::::::::::::::::::::::::::                              
	                         M O D I F I C A T I O N                               
	                   ::::::::::::::::::::::::::::::
                             
	---------- ------------- ------ ---------------------------------------------  
	DATE       NAME          Ver    DESCRIPTION                                    
	---------- ------------- ------ ---------------------------------------------  
	2005-12-06 SYSTEM           3.5 Created
                                       
	================================================================================                                                                               
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found                  
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
                                                                             

v_RowCnt          INTEGER DEFAULT 0;                                           
v_BeginTS         timestamp DEFAULT FN_Get_Date; -- Performance related variables
v_ProcName        VARCHAR2(30) DEFAULT 'PR_WORKFLOWS_AH1';                          
v_ExceptionFlg    INTEGER DEFAULT 0;                                            
v_ParamsTxt       VARCHAR2(4000) DEFAULT                                        
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_fromsyntax=>'||p_fromsyntax||CHR(10)
      ||', p_tosyntax=>'||p_tosyntax||CHR(10)
		;
                                                                            
---------------------------------------------------------------------------     
--                         M    A    I    N                              --     
---------------------------------------------------------------------------     

BEGIN                                                                          
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;                                          
   OPEN RC1 FOR
   SELECT WORKFLOWID                                                                    
      ,RTRIM(RESOURCEKEY) RESOURCEKEY                                               
		,RTRIM(DESCRIPTION) DESCRIPTION                                                   
      ,coalesce((CASE WHEN (SELECT actionid 
                            FROM dis_imp_edi_syntax_def_actn def
                            WHERE def.actionid = wf.workflowid
                              AND def.fromsyntax = p_fromsyntax
                              AND def.tosyntax = p_tosyntax)
                 IS NOT NULL THEN (SELECT 1 FROM dual)END), 0) defaultflag
	FROM         
		PR_WORKFLOW wf    
	WHERE wf.workflowid NOT IN (SELECT DISTINCT actionid
                               FROM  dis_imp_edi_syntax_action_lnk lnk
                               WHERE lnk.fromsyntax = p_fromsyntax
                                 AND lnk.tosyntax = p_tosyntax)
	;                                                                                

   --hardcoded sql above because p_fromsyntax and p_tosyntax will never be null
	
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt,     
p_DebugFlg, v_ExceptionFlg);                                                    
END PR_WORKFLOWS_AH1;
/


-- PR_WORKFLOWU_AH1.sql

CREATE OR REPLACE PROCEDURE PR_WORKFLOWU_AH1
(
   p_ReturnCd out INTEGER
   ,p_DebugFlg in PLS_INTEGER
   ,RC1 out db_ref.RCT
   ,p_LOCALE in VARCHAR2
   ,p_WORKFLOWID in NUMBER
   ,p_WORKFLOWNAME in VARCHAR2
   ,p_WORKFLOWDEF in VARCHAR2
   ,p_ENABLEDFLG in NUMBER
   ,p_ROWTS in NUMBER
   ,p_WORKFLOWTYPEID in NUMBER
   ,p_WORKFLOWDISPLAYNAME in VARCHAR2
   ,p_LOC_ROWTS in NUMBER
   ,p_RESOURCEKEY in VARCHAR2
   ,p_PROVIDERFLAG in NUMBER
)
AS
/*
    ================================================================================
    Proc Name:       PR_WORKFLOWU_AH1
    Create Date:     2003-04-29 08:30:11
    Author:          L.Ling
                       ::::::::::::::::::::::::::::::
                          M O D I F I C A T I O N
                       ::::::::::::::::::::::::::::::
    ---------- ------------- ------ ---------------------------------------------
    DATE       NAME          Ver    DESCRIPTION
    ---------- ------------- ------ ---------------------------------------------
    2003-04-29 L.Ling        3.5    Created.
    2003-07-01 J.Chen        3.5    Moved p_Locale after RC1
    2003-09-29 M.Mount       4.2.1  Removed Commits and Rollbacks
    2004-03-02 S.Priani	     4.2.2  Removed NLS
    2004-04-12 P Dominick    4.2.2  Added resourcekey column to update
    ================================================================================
    Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
v_RowCnt          integer :=0;
v_BeginTS         timestamp := FN_GET_DATE(); -- Performance related variables
v_ProcName        varchar2(30) := 'PR_WORKFLOWU_AH1';
v_ExceptionFlg    integer := 0;
v_ParamsTxt       varchar2(4000) :=
        'p_DebugFlg=>'||p_DebugFlg||CHR(10)
        ||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
        ||', p_WORKFLOWNAME=>'||p_WORKFLOWNAME||CHR(10)
        ||', p_WORKFLOWDEF=>'||p_WORKFLOWDEF||CHR(10)
        ||', p_ENABLEDFLG=>'||p_ENABLEDFLG||CHR(10)
        ||', p_ROWTS=>'||p_ROWTS||CHR(10)
        ||', p_WORKFLOWTYPEID=>'||p_WORKFLOWTYPEID||CHR(10)
        ||', p_LOCALE=>'||p_LOCALE||CHR(10)
        ||', p_WORKFLOWDISPLAYNAME=>'||p_WORKFLOWDISPLAYNAME||CHR(10)
        ||', p_LOC_ROWTS=>'||p_ROWTS||CHR(10)
        ;
v_Locale        PR_Locale.Locale%Type := nvl(p_Locale, FN_GET_DEFAULT_LOC);
v_RowTS        integer :=0;
v_LocRowCnt    integer := 0;
zeroUpdate     Exception;
--
v_PR_WORKFLOW_ROWID rowid;
v_PR_WORKFLOW_LOC_ROWID rowid;
v_LocRowTS     integer :=0;
v_PR_WORKFLOW_ROWTS integer :=0;
v_PR_WORKFLOW_LOC_ROWTS integer :=0;
v_WORKFLOWNAME  PR_WORKFLOW.WORKFLOWNAME%Type;
v_WORKFLOWDEF   PR_WORKFLOW.WORKFLOWDEF%Type;
v_ENABLEDFLG    PR_WORKFLOW.ENABLEDFLG%Type;
v_WORKFLOWTYPEID PR_WORKFLOW.WORKFLOWTYPEID%Type;
v_DESCRIPTION	PR_WORKFLOW.DESCRIPTION%Type;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------
BEGIN
   p_ReturnCd := 0;
   v_RowCnt := 0;
   SELECT  ROWID, ROWTS, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, WORKFLOWTYPEID, DESCRIPTION
   INTO    v_PR_WORKFLOW_ROWID, v_PR_WORKFLOW_ROWTS, v_WORKFLOWNAME, v_WORKFLOWDEF, v_ENABLEDFLG, v_WORKFLOWTYPEID, v_DESCRIPTION
   FROM    PR_WORKFLOW
   WHERE   WORKFLOWID = p_WORKFLOWID
   FOR UPDATE; -- row level locking
   IF NVL(v_WORKFLOWNAME, ' ') <> NVL(p_WORKFLOWNAME, ' ')  OR
      NVL(v_WORKFLOWDEF, ' ')  <> NVL(p_WORKFLOWDEF, ' ')   OR
      NVL(v_ENABLEDFLG, -1)   <> NVL(p_ENABLEDFLG, -1)    OR
      NVL(v_DESCRIPTION, ' ')   <> NVL(p_WORKFLOWDISPLAYNAME, ' ')    OR
      NVL(v_WORKFLOWTYPEID, -1) <> NVL(p_WORKFLOWTYPEID, -1)  THEN     -- data changes
       IF NVL(v_PR_WORKFLOW_ROWTS, -1) <> NVL(p_ROWTS, -1) THEN
           p_ReturnCd := 1;
           v_ExceptionFlg := 1;
           DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ROWTS Changes in PR_WORKFLOW,'|| v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
    --       COMMIT;
           RAISE_APPLICATION_ERROR(-20999,'20100 - PR_WORKFLOW_LOCU in PR_WORKFLOW: ROWTS Changes');
       END IF;
       BEGIN
           Update PR_WORKFLOW
            SET
                WORKFLOWNAME=p_WORKFLOWNAME
                ,      WORKFLOWDEF=p_WORKFLOWDEF
                ,      ENABLEDFLG=p_ENABLEDFLG
                ,      ROWTS= p_RowTS + 1
                ,      UPDATEDATE= FN_GET_DATE()
                ,      WORKFLOWTYPEID=p_WORKFLOWTYPEID
		,	DESCRIPTION = p_WORKFLOWDISPLAYNAME
		,	RESOURCEKEY = p_RESOURCEKEY
		,	PROVIDERFLAG = p_PROVIDERFLAG
            Where   ROWID = v_PR_WORKFLOW_ROWID returning RowTS into v_RowTS;
            v_RowCnt := SQL%rowcount;
            if (v_RowCnt = 0) then  raise ZeroUpdate ; end if;
            p_ReturnCd := 0;
            DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
            EXCEPTION
                WHEN TOO_MANY_ROWS Then
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS,'TOO_MANY_ROWS,'|| v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --                   Commit;
                    RAISE_APPLICATION_ERROR(-20999, '20100 - PR_WORKFLOWU_AH1 in PR_WORKFLOW: Too Many Rows');
                WHEN ZeroUpdate Then
                   v_ExceptionFlg := 1;
                   DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, 'ZeroUpdate in PR_WORKFLOW,'|| v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
 --                  Commit;
                   RAISE_APPLICATION_ERROR(-20999,'20100 - PR_WORKFLOWU_AH1 in PR_WORKFLOW: ZeroUpdate');
                WHEN OTHERS Then
 --                   ROLLBACK WORK;
                    p_ReturnCd := 1;
                    v_ExceptionFlg := 1;
                    DB_ProcAuditLogI_AH1(v_ProcName, v_BeginTS, SQLERRM||CHR(10)||v_ParamsTxt, p_DebugFlg, 1);
 --                   Commit;
                    RAISE_APPLICATION_ERROR(-20999,'20100 - PR_WORKFLOWU_AH1 in PR_WORKFLOW: '||SQLERRM);
       END;
   END IF;
----   PR_WORKFLOW_LOCU;
   IF RC1%isOpen THEN Close RC1; END IF;
   dbms_output.put_line('v_RowCnt = ' || v_RowCnt || ' v_LocRowCnt = ' || v_LocRowCnt || ' p_ReturnCd = ' || p_ReturnCd);
   OPEN RC1 FOR
   SELECT
        v_RowTS as RowTS,
        v_LocRowTS as Loc_RowTS
   FROM Dual;
END;
/


-- PR_WORKFLOW_LOC_VWS.sql

CREATE OR REPLACE PROCEDURE PR_WORKFLOW_LOC_VWS
	(
	p_ReturnCd  OUT  PLS_INTEGER
	, p_DebugFlg  IN  PLS_INTEGER
	, RC1  OUT  DB_REF.RCT
	, p_Locale  IN  VARCHAR2
	, p_WORKFLOWID IN NUMBER
	)
AS

/*
	================================================================================
	Proc Name:       PR_WORKFLOW_LOC_VWS
	Create Date:     2003-09-26 00:10:24
	Author:          SYSTEM

	                   ::::::::::::::::::::::::::::::
	                         M O D I F I C A T I O N
	                   ::::::::::::::::::::::::::::::

	---------- ------------- ------ ---------------------------------------------
	DATE       NAME          Ver    DESCRIPTION
	---------- ------------- ------ ---------------------------------------------
	2003-09-26 SYSTEM           3.5 Created
	2004-03-05 S.Priani      4.2.2  Removed NLS

	================================================================================
	Return Code : 0 = OK, 1 = Not Good, 2 = Dup, 3= No Data Found
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/


v_RowCnt          integer Default 0;
v_BeginTS         timestamp Default FN_Get_Date; -- Performance related variables
v_ProcName        varchar2(30) Default 'PR_WORKFLOW_LOC_VWS';
v_ExceptionFlg    integer Default 0;
v_ParamsTxt       varchar2(4000) Default
		'p_DebugFlg=>'||p_DebugFlg||CHR(10)
		||', p_WORKFLOWID=>'||p_WORKFLOWID||CHR(10)
		||', p_LOCALE=>'||p_LOCALE||CHR(10)
		;

---------------------------------------------------------------------------
--                         M    A    I    N                              --
---------------------------------------------------------------------------

BEGIN
	IF RC1%ISOPEN THEN CLOSE RC1; END IF;
IF p_WORKFLOWID IS NOT NULL
 THEN
	OPEN RC1 FOR
	SELECT
		WORKFLOWID
		,RTRIM(WORKFLOWNAME) WORKFLOWNAME
		,RTRIM(WORKFLOWDEF) WORKFLOWDEF
		,ENABLEDFLG
		,ROWTS
		,CREATEDATE
		,WORKFLOWTYPEID
		,RTRIM(DESCRIPTION) WORKFLOWDISPLAYNAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,PROVIDERFLAG
	FROM
		PR_WORKFLOW
	Where
		WORKFLOWID = p_WORKFLOWID
	;
	ELSE
	OPEN RC1 FOR
	SELECT
		WORKFLOWID
		,RTRIM(WORKFLOWNAME) WORKFLOWNAME
		,RTRIM(WORKFLOWDEF) WORKFLOWDEF
		,ENABLEDFLG
		,ROWTS
		,CREATEDATE
		,WORKFLOWTYPEID
		,RTRIM(DESCRIPTION) WORKFLOWDISPLAYNAME
		,RTRIM(RESOURCEKEY) RESOURCEKEY
		,PROVIDERFLAG
	FROM
		PR_WORKFLOW
	;
	END IF;
	p_ReturnCd := 0;
	DB_ProcAuditLogI_ah1(v_ProcName, v_BeginTS, v_ParamsTxt, p_DebugFlg, v_ExceptionFlg);
End PR_WORKFLOW_LOC_VWS;
/


-- # ============================================================== # 
-- Metadata 
-- # ============================================================== # 

-- pr_Locale.sql


-- English
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg)
   select 'en_US', 'English United States', 1 from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'en_US')
/

--German
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg, nlssortcd)
   select 'de', 'German', 0, 'XGERMAN' from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'de')
/

--Spanish
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg, nlssortcd)
   select 'es', 'Spanish', 0, 'SPANISH_M' from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'es')
/

--French
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg, nlssortcd)
   select 'fr', 'French', 0, 'FRENCH_M' from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'fr')
/

--Italian
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg, nlssortcd)
   select 'it', 'Italian', 0, '' from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'it')
/

--Japanese
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg, nlssortcd)
   select 'ja', 'Japanese', 0, 'JAPANESE_M' from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'ja')
/

--Korean
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg, nlssortcd)
   select 'ko', 'Korean', 0, 'KOREAN_M' from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'ko')
/

--Portugese(Brazil)
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg, nlssortcd)
   select 'pt_BR', 'Portugese(Brazil)', 0, '' from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'pt_BR')
/

--Chinese
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg, nlssortcd)
   select 'zh', 'Chinese', 0, 'SCHINESE_RADICAL_M' from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'zh')
/

--Traditional Chinese/Taiwanese
Insert Into PR_LOCALE(LOCALE ,LOCALEDESC, DefaultFlg, nlssortcd)
   select 'zh_TW', 'Traditional Chinese', 0, 'TCHINESE_RADICAL_M' from Dual
   Where Not Exists (select 'x' from PR_LOCALE where LOCALE = 'zh_TW')
/


-- pr_Transport.sql


Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'Http1.1', 7, 1, 'Http1.1', 'com.ibm.bcg.delivery.HttpSender' FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'Http1.1')
/
  

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'Https1.0', 7, 1, 'Https1.0', 'com.ibm.bcg.delivery.HttpSender' FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'Https1.0')
/
  

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'Https1.1', 7, 1, 'Https1.1', 'com.ibm.bcg.delivery.HttpSender' FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'Https1.1')
/
 

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'Ftp', 7, 1, 'Ftp', 'com.ibm.bcg.delivery.FtpSender'  FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'Ftp')
/
  

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'FtpDirectory', 1, 2, 'FtpDirectory', 'com.ibm.bcg.server.receiver.FTPDirReceiverImpl'  FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'FtpDirectory')
/
  

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'JMS', 1, 2, 'JMSQueue', 'com.ibm.bcg.server.receiver.JmsReceiver'  FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'JMS' AND VISIBILITY = 1 AND
                     UI_VISIBILITY = 2)
/
 

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'Smtp', 7, 1, 'Smtp', 'com.ibm.bcg.delivery.SmtpSender' FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'Smtp')
/
    

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'POP3', 1, 2, 'POP3', 'com.ibm.bcg.server.receiver.Pop3Receiver'  FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'POP3')
/
    

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'HttpS', 1, 2, 'HttpS', 'com.ibm.bcg.servlet.HTTPRcvrImpl' FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'HttpS')
/
 

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'FileDirectory', 1, 2, 'FileDirectory', 'com.ibm.bcg.server.receiver.DirectoryReceiverImpl'  FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'FileDirectory')
/


Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'JMS', 7, 1, 'JMS', 'com.ibm.bcg.delivery.JmsSender'  FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'JMS' AND VISIBILITY = 7 AND
                     UI_VISIBILITY = 1)
/
 

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
   SELECT  PR_TRANSPORT_SEQ.Nextval,'FileDirectoryGateway', 7, 1, 'FileDirectoryGateway', 'com.ibm.bcg.delivery.FileSender'  FROM DUAL
   Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'FileDirectoryGateway')
/


Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY ) 
	select PR_TRANSPORT_SEQ.Nextval,'FtpSDirectory', 0, 0, 'FtpSDirectory' from DUAL
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'FtpSDirectory')
/


Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
	select PR_TRANSPORT_SEQ.Nextval,'FtpS', 7, 1, 'FTPS', 'com.ibm.bcg.delivery.FtpsSender' from DUAL
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'FtpS')
/

--FTP Scripting Metadata

Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
	select PR_TRANSPORT_SEQ.Nextval,'FTPScriptingReceiver', 1, 2, 'FTPScripting', 'com.ibm.bcg.server.receiver.FTPScriptingReceiver' from DUAL
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'FTPScriptingReceiver')
/


Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
	select PR_TRANSPORT_SEQ.Nextval,'FTPScriptingGateway', 7, 1, 'FTPScripting', 'com.ibm.bcg.delivery.FTPScriptingSender' from DUAL
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'FTPScriptingGateway')
/

-- F104595 start
-- SFTP receiver
Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
	select PR_TRANSPORT_SEQ.Nextval, 'SFTPReceiver', 1, 2, 'SFTP', 'com.ibm.bcg.server.receiver.SFTPReceiver' from DUAL
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'SFTPReceiver')
/


-- SFTP destination
Insert Into PR_TRANSPORT(TRANSPORTID, TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY, RESOURCEKEY, CLASSNAME ) 
	select PR_TRANSPORT_SEQ.Nextval, 'SFTPSender', 7, 1, 'SFTP', 'com.ibm.bcg.delivery.SFTPSender' from DUAL
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'SFTPSender')
/

-- F104595 end


-- pr_Transport_Loc_ENUS.sql

-- pr_Transport_loc 

Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'HTTP/1.1' from PR_TRANSPORT where TRANSPORTTYPE = 'Http1.1'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'HTTP/1.1'	And LOCALE = 'en_US')
/
	   
Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'HTTPS/1.0' from PR_TRANSPORT where TRANSPORTTYPE = 'Https1.0'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'HTTPS/1.0'	And LOCALE = 'en_US')
/
	
Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'FTP' from PR_TRANSPORT where TRANSPORTTYPE = 'Ftp'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'FTP'	And LOCALE = 'en_US')
/
	
Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'SMTP' from PR_TRANSPORT where TRANSPORTTYPE = 'Smtp'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'SMTP'	And LOCALE = 'en_US')
/
	
Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'HTTPS/1.1' from PR_TRANSPORT where TRANSPORTTYPE = 'Https1.1'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'HTTPS/1.1' And LOCALE = 'en_US')
/
	   
Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'FTP Directory' from PR_TRANSPORT where TRANSPORTTYPE = 'FtpDirectory'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'FTP Directory'	And LOCALE = 'en_US')
/

Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'JMS' from PR_TRANSPORT where TRANSPORTTYPE = 'JMS'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'JMS'	And LOCALE = 'en_US')
/
	       
INSERT into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'POP3' from PR_TRANSPORT where TRANSPORTTYPE = 'POP3'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'POP3'	And LOCALE = 'en_US')
/

Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'HTTP/S' from PR_TRANSPORT where TRANSPORTTYPE = 'HttpS'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'HTTP/S'	And LOCALE = 'en_US')
/	 
      
Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'File Directory' from PR_TRANSPORT where TRANSPORTTYPE = 'FileDirectory'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'File Directory'	And LOCALE = 'en_US')
/	 


/*  *************************** 3.6 **********************************

Insert Into PR_TRANSPORT(TRANSPORTID ,TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY) 
	select PR_TRANSPORT_SEQ.Nextval, 'Sftp2.0', 7, 1 from Dual
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'Sftp2.0')
/ 
Insert Into PR_TRANSPORT(TRANSPORTID ,TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY) 
	select PR_TRANSPORT_SEQ.Nextval, 'SSH', 7, 1 from Dual
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'SSH')
/    

Insert Into PR_TRANSPORT(TRANSPORTID ,TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY) 
	select PR_TRANSPORT_SEQ.Nextval, 'SmtpDirectory', 1, 1 from Dual
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'SmtpDirectory')
/


Insert Into PR_TRANSPORT(TRANSPORTID ,TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY) 
	select PR_TRANSPORT_SEQ.Nextval, 'Directory', 3, 3 from Dual
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'Directory')
/  

Insert Into PR_TRANSPORT(TRANSPORTID ,TRANSPORTTYPE ,VISIBILITY, UI_VISIBILITY) 
	select PR_TRANSPORT_SEQ.Nextval, 'POP3Secure', 1, 1 from Dual
	Where Not Exists (select 'x' from PR_TRANSPORT where TRANSPORTTYPE = 'POP3Secure')
/   

---- Loc 
Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'SSH' from PR_TRANSPORT where TRANSPORTTYPE = 'SSH'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'SSH'	And LOCALE = 'en_US')
/

Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'SMTP Directory' from PR_TRANSPORT where TRANSPORTTYPE = 'SmtpDirectory'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'SMTP Directory'	And LOCALE = 'en_US')
/
	       

Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'Directory' from PR_TRANSPORT where TRANSPORTTYPE = 'Directory'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'Directory'	And LOCALE = 'en_US')
/

Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'POP3 Secure' from PR_TRANSPORT where TRANSPORTTYPE = 'POP3Secure'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'POP3 Secure'	And LOCALE = 'en_US')
/

Insert into PR_TRANSPORT_LOC(TRANSPORTID ,LOCALE,TRANSPORTTYPEDISPLAY) 
select TRANSPORTID, 'en_US', 'SFTP/2.0' from PR_TRANSPORT where TRANSPORTTYPE = 'Sftp2.0'
and NOT Exists (select 'x' from  PR_TRANSPORT_LOC Where TRANSPORTTYPEDISPLAY = 'SFTP/2.0'	And LOCALE = 'en_US')
/	
*/  




 






 

-- pr_DestType.sql

-- pr_DestType

-- Note the script hard codes the primary keys for the rows inserted
-- and does not rely on the sequence. The create sequence script
-- will start the seed at the 20 to leave space for future types.
/*
JAVA Destination Values
public static final String RECEIVER_DESTINATION = "ReceiverDestinationType";
public static final String DESTINATION = "DestinationType";
public static final String DESTINATION_PRODUCTION = "1";
public static final String DESTINATION_TEST = "2";
public static final String DESTINATION_VTP_PARTNER = "3";
public static final String DESTINATION_VTP_BUILDER = "4";
public static final String DESTINATION_FAIL = "0";
public static final String DESTINATION_NONE = "None";
*/
--Change to auto gen desttypeid. 07-10-2003

Insert Into PR_DESTTYPE(DESTTYPEID ,DESTNAME, HIDDENFLG, DIRFLG, RESOURCEKEY )
   select PR_DESTTYPE_SEQ.nextval, 'Unknown', 1, 0 , 'Unknown' From Dual
   Where Not Exists (select 'x' from PR_DESTTYPE where DESTNAME = 'Unknown')
/
--
Insert Into PR_DESTTYPE(DESTTYPEID ,DESTNAME, HIDDENFLG, DefaultFlg, DIRFLG, RESOURCEKEY )
   select PR_DESTTYPE_SEQ.nextval, 'Production', 0, 1, 1 , 'Production' From Dual
   Where Not Exists (select 'x' from PR_DESTTYPE where DESTNAME = 'Production')
/ 
--
Insert Into PR_DESTTYPE(DESTTYPEID ,DESTNAME, HIDDENFLG, DIRFLG, RESOURCEKEY )
   select PR_DESTTYPE_SEQ.nextval, 'Test', 0, 1, 'Test' From Dual
   Where Not Exists (select 'x' from PR_DESTTYPE where DESTNAME = 'Test')
/ 
--
Insert Into PR_DESTTYPE(DESTTYPEID ,DESTNAME, HIDDENFLG, DIRFLG, RESOURCEKEY )
   select PR_DESTTYPE_SEQ.nextval, 'VtpPartner', 0, 0, 'VtpPartner' From Dual
   Where Not Exists (select 'x' from PR_DESTTYPE where DESTNAME = 'VtpPartner')
/ 
--
Insert Into PR_DESTTYPE(DESTTYPEID ,DESTNAME, HIDDENFLG, DIRFLG, RESOURCEKEY )
   select PR_DESTTYPE_SEQ.nextval, 'VtpHubOwner', 0, 0, 'VtpHubOwner' From Dual
   Where Not Exists (select 'x' from PR_DESTTYPE where DESTNAME = 'VtpHubOwner')
/ 







-- pr_DestType_Loc_ENUS.sql

-- pr_DestType_loc

Insert into PR_DESTTYPE_LOC(DESTTYPEID ,LOCALE, DESTDISPLAYNAME) 
select DESTTYPEID, 'en_US', 'Production' from PR_DESTTYPE where DESTNAME = 'Production'
AND NOT Exists (select 'x' from  PR_DESTTYPE_LOC Where DESTDISPLAYNAME = 'Production' And  LOCALE = 'en_US')
/
--     
Insert into PR_DESTTYPE_LOC(DESTTYPEID ,LOCALE, DESTDISPLAYNAME) 
select DESTTYPEID, 'en_US', 'Test' from PR_DESTTYPE where DESTNAME = 'Test'
AND NOT Exists (select 'x' from  PR_DESTTYPE_LOC Where DESTDISPLAYNAME = 'Test' And  LOCALE = 'en_US')
/
--
Insert into PR_DESTTYPE_LOC(DESTTYPEID ,LOCALE, DESTDISPLAYNAME) 
select DESTTYPEID, 'en_US', 'CPS Participant' from PR_DESTTYPE where DESTNAME = 'VtpPartner'
AND NOT Exists (select 'x' from  PR_DESTTYPE_LOC Where DESTDISPLAYNAME = 'CPS Participant' And  LOCALE = 'en_US')
/
--
Insert into PR_DESTTYPE_LOC(DESTTYPEID ,LOCALE, DESTDISPLAYNAME) 
select DESTTYPEID, 'en_US', 'CPS Manager' from PR_DESTTYPE where DESTNAME = 'VtpHubOwner'
AND NOT Exists (select 'x' from  PR_DESTTYPE_LOC Where DESTDISPLAYNAME = 'CPS Manager' And  LOCALE = 'en_US')
/
--     
Insert into PR_DESTTYPE_LOC(DESTTYPEID ,LOCALE, DESTDISPLAYNAME) 
select DESTTYPEID, 'en_US', 'Unknown' from PR_DESTTYPE where DESTNAME = 'Unknown'
AND NOT Exists (select 'x' from  PR_DESTTYPE_LOC Where DESTDISPLAYNAME = 'Unknown' And  LOCALE = 'en_US')
/


--Update

UPDATE pr_desttype_loc 
   SET destdisplayname = 'Unknown' 
 WHERE desttypeid = (SELECT d.desttypeid 
                       FROM pr_desttype_loc t
                          , pr_desttype d 
                      WHERE d.desttypeid = t.desttypeid 
                        AND d.destname = 'Unknown'
                        AND t.locale = 'en_US')
/
 

UPDATE pr_desttype_loc
   SET destdisplayname = 'Production'
 WHERE desttypeid = (SELECT d.desttypeid
                       FROM pr_desttype_loc t
                          , pr_desttype d
                      WHERE d.desttypeid = t.desttypeid 
                        AND d.destname = 'Production'
                        AND t.locale = 'en_US')
/

UPDATE pr_desttype_loc
   SET destdisplayname = 'Test'
 WHERE desttypeid = (SELECT d.desttypeid
                       FROM pr_desttype_loc t
                          , pr_desttype d
                      WHERE d.desttypeid = t.desttypeid 
                        AND d.destname = 'Test'
                        AND t.locale = 'en_US')
/

UPDATE pr_desttype_loc
   SET destdisplayname = 'CPS Participant'
 WHERE desttypeid = (SELECT d.desttypeid
                       FROM pr_desttype_loc t
                          , pr_desttype d
                      WHERE d.desttypeid = t.desttypeid 
                        AND d.destname = 'VtpPartner'
                        AND t.locale = 'en_US')
/

UPDATE pr_desttype_loc
   SET destdisplayname = 'CPS Manager'
 WHERE desttypeid = (SELECT d.desttypeid
                       FROM pr_desttype_loc t
                          , pr_desttype d
                      WHERE d.desttypeid = t.desttypeid 
                        AND d.destname = 'VtpHubOwner'
                        AND t.locale = 'en_US')
/


-- pr_ContactType.sql

-- pr_ContactType

Insert Into PR_CONTACTTYPE(CONTACTTYPEID ,CONTACTTYPENAME, RESOURCEKEY )
	select PR_CONTACTTYPE_SEQ.Nextval, 'ProjectManager', 'ProjectManager' from Dual
	Where Not Exists (select 'x' from PR_CONTACTTYPE where CONTACTTYPENAME = 'ProjectManager')
/ 
--
Insert Into PR_CONTACTTYPE(CONTACTTYPEID ,CONTACTTYPENAME, RESOURCEKEY )
	select PR_CONTACTTYPE_SEQ.Nextval, 'BusinessLead', 'BusinessLead' from Dual
	Where Not Exists (select 'x' from PR_CONTACTTYPE where CONTACTTYPENAME = 'BusinessLead')
/ 
--
Insert Into PR_CONTACTTYPE(CONTACTTYPEID ,CONTACTTYPENAME, RESOURCEKEY )
	select PR_CONTACTTYPE_SEQ.Nextval, 'TechnicalLead', 'TechnicalLead' from Dual
	Where Not Exists (select 'x' from PR_CONTACTTYPE where CONTACTTYPENAME = 'TechnicalLead')
/ 
--
Insert Into PR_CONTACTTYPE(CONTACTTYPEID ,CONTACTTYPENAME, RESOURCEKEY )
	select PR_CONTACTTYPE_SEQ.Nextval, 'B2BLead', 'B2BLead' from Dual
	Where Not Exists (select 'x' from PR_CONTACTTYPE where CONTACTTYPENAME = 'B2BLead')
/
--
Insert Into PR_CONTACTTYPE(CONTACTTYPEID ,CONTACTTYPENAME, RESOURCEKEY )
	select PR_CONTACTTYPE_SEQ.Nextval, 'DataContentLead', 'DataContentLead' from Dual
	Where Not Exists (select 'x' from PR_CONTACTTYPE where CONTACTTYPENAME = 'DataContentLead')
/ 
--
Insert Into PR_CONTACTTYPE(CONTACTTYPEID ,CONTACTTYPENAME, RESOURCEKEY )
	select PR_CONTACTTYPE_SEQ.Nextval, 'BackendApplicationLead', 'BackendApplicationLead' from Dual
	Where Not Exists (select 'x' from PR_CONTACTTYPE where CONTACTTYPENAME = 'BackendApplicationLead')
/ 
--
Insert Into PR_CONTACTTYPE(CONTACTTYPEID ,CONTACTTYPENAME, RESOURCEKEY )
	select PR_CONTACTTYPE_SEQ.Nextval, 'NetworkFirewallLead', 'NetworkFirewallLead' from Dual
	Where Not Exists (select 'x' from PR_CONTACTTYPE where CONTACTTYPENAME = 'NetworkFirewallLead')
/ 








-- pr_ContactType_Loc_ENUS.sql

-- pr_ContactType_loc

Insert into PR_CONTACTTYPE_LOC(CONTACTTYPEID ,LOCALE,CONTACTTYPEDISPLAYNAME) 
select CONTACTTYPEID, 'en_US', 'Project Manager' from PR_CONTACTTYPE t WHERE CONTACTTYPENAME = 'ProjectManager'
AND NOT Exists (select 'x' from PR_CONTACTTYPE_loc where CONTACTTYPEDISPLAYNAME = 'Project Manager')
/ 
--	   
Insert into PR_CONTACTTYPE_LOC(CONTACTTYPEID ,LOCALE,CONTACTTYPEDISPLAYNAME) 
select CONTACTTYPEID, 'en_US', 'Business Lead' from PR_CONTACTTYPE t WHERE CONTACTTYPENAME = 'BusinessLead'
AND NOT Exists (select 'x' from PR_CONTACTTYPE_loc where CONTACTTYPEDISPLAYNAME = 'Business Lead')
/ 
--	   
Insert into PR_CONTACTTYPE_LOC(CONTACTTYPEID ,LOCALE,CONTACTTYPEDISPLAYNAME) 
select CONTACTTYPEID, 'en_US', 'Technical Lead' from PR_CONTACTTYPE t WHERE CONTACTTYPENAME = 'TechnicalLead'
AND NOT Exists (select 'x' from PR_CONTACTTYPE_loc where CONTACTTYPEDISPLAYNAME = 'Technical Lead')
/ 
--	   
Insert into PR_CONTACTTYPE_LOC(CONTACTTYPEID ,LOCALE,CONTACTTYPEDISPLAYNAME) 
select CONTACTTYPEID, 'en_US', 'B2B Lead' from PR_CONTACTTYPE t WHERE CONTACTTYPENAME = 'B2BLead'
AND NOT Exists (select 'x' from PR_CONTACTTYPE_loc where CONTACTTYPEDISPLAYNAME = 'B2B Lead')
/ 
--	   
Insert into PR_CONTACTTYPE_LOC(CONTACTTYPEID ,LOCALE,CONTACTTYPEDISPLAYNAME) 
select CONTACTTYPEID, 'en_US', 'Data Content Lead' from PR_CONTACTTYPE t WHERE CONTACTTYPENAME = 'DataContentLead'
AND NOT Exists (select 'x' from PR_CONTACTTYPE_loc where CONTACTTYPEDISPLAYNAME = 'Data Content Lead')
/ 
--	  
Insert into PR_CONTACTTYPE_LOC(CONTACTTYPEID ,LOCALE,CONTACTTYPEDISPLAYNAME) 
select CONTACTTYPEID, 'en_US', 'Backend Application Lead' from PR_CONTACTTYPE t WHERE CONTACTTYPENAME = 'BackendApplicationLead'
AND NOT Exists (select 'x' from PR_CONTACTTYPE_loc where CONTACTTYPEDISPLAYNAME = 'Backend Application Lead')
/ 
--	   
Insert into PR_CONTACTTYPE_LOC(CONTACTTYPEID ,LOCALE,CONTACTTYPEDISPLAYNAME) 
select CONTACTTYPEID, 'en_US', 'Network/Firewall Lead' from PR_CONTACTTYPE t WHERE CONTACTTYPENAME = 'NetworkFirewallLead'
AND NOT Exists (select 'x' from PR_CONTACTTYPE_loc where CONTACTTYPEDISPLAYNAME = 'Network/Firewall Lead')
/ 
--	     

-- pr_Classification.sql

-- pr_Classification

Insert Into PR_CLASSIFICATION(CLASSIFICATIONID ,CLASSIFICATIONNAME, RESOURCEKEY )
	select PR_CLASSIFICATION_SEQ.Nextval, 'Supplier', 'Supplier' from Dual
	Where Not Exists (select 'x' from PR_CLASSIFICATION where CLASSIFICATIONNAME = 'Supplier')
/ 
--
Insert Into PR_CLASSIFICATION(CLASSIFICATIONID ,CLASSIFICATIONNAME, RESOURCEKEY )
	select PR_CLASSIFICATION_SEQ.Nextval, 'ContractManufacturer', 'ContractManufacturer' from Dual
	Where Not Exists (select 'x' from PR_CLASSIFICATION where CLASSIFICATIONNAME = 'ContractManufacturer')
/ 
--
Insert Into PR_CLASSIFICATION(CLASSIFICATIONID ,CLASSIFICATIONNAME, RESOURCEKEY )
	select PR_CLASSIFICATION_SEQ.Nextval, 'Distributor', 'Distributor' from Dual
	Where Not Exists (select 'x' from PR_CLASSIFICATION where CLASSIFICATIONNAME = 'Distributor')
/
--
Insert Into PR_CLASSIFICATION(CLASSIFICATIONID ,CLASSIFICATIONNAME, RESOURCEKEY )
	select PR_CLASSIFICATION_SEQ.Nextval, 'LogisticsProvider', 'LogisticsProvider' from Dual
	Where Not Exists (select 'x' from PR_CLASSIFICATION where CLASSIFICATIONNAME = 'LogisticsProvider')
/ 
--
Insert Into PR_CLASSIFICATION(CLASSIFICATIONID ,CLASSIFICATIONNAME, RESOURCEKEY )
	select PR_CLASSIFICATION_SEQ.Nextval, 'Other', 'Other' from Dual
	Where Not Exists (select 'x' from PR_CLASSIFICATION where CLASSIFICATIONNAME = 'Other')
/ 




-- pr_Classification_loc_ENUS.sql

-- pr_Classification_loc

Insert into PR_CLASSIFICATION_LOC(CLASSIFICATIONID ,LOCALE,CLASSIFICATIONDISPLAYNAME) 
select CLASSIFICATIONID, 'en_US', 'Supplier' from PR_CLASSIFICATION  WHERE CLASSIFICATIONNAME = 'Supplier'
AND NOT Exists (select 'x' from PR_CLASSIFICATION_loc where CLASSIFICATIONDISPLAYNAME = 'Supplier')
/ 
--	   
Insert into PR_CLASSIFICATION_LOC(CLASSIFICATIONID ,LOCALE,CLASSIFICATIONDISPLAYNAME) 
select CLASSIFICATIONID, 'en_US', 'Contract Manufacturer' from PR_CLASSIFICATION  WHERE CLASSIFICATIONNAME = 'ContractManufacturer'
AND NOT Exists (select 'x' from PR_CLASSIFICATION_loc where CLASSIFICATIONDISPLAYNAME = 'Contract Manufacturer')
/ 
--	   
Insert into PR_CLASSIFICATION_LOC(CLASSIFICATIONID ,LOCALE,CLASSIFICATIONDISPLAYNAME) 
select CLASSIFICATIONID, 'en_US', 'Distributor' from PR_CLASSIFICATION  WHERE CLASSIFICATIONNAME = 'Distributor'
AND NOT Exists (select 'x' from PR_CLASSIFICATION_loc where CLASSIFICATIONDISPLAYNAME = 'Distributor')
/ 
--	   
Insert into PR_CLASSIFICATION_LOC(CLASSIFICATIONID ,LOCALE,CLASSIFICATIONDISPLAYNAME) 
select CLASSIFICATIONID, 'en_US', 'Logistics Provider' from PR_CLASSIFICATION  WHERE CLASSIFICATIONNAME = 'LogisticsProvider'
AND NOT Exists (select 'x' from PR_CLASSIFICATION_loc where CLASSIFICATIONDISPLAYNAME = 'Logistics Provider')
/ 
--
Insert into PR_CLASSIFICATION_LOC(CLASSIFICATIONID ,LOCALE,CLASSIFICATIONDISPLAYNAME) 
select CLASSIFICATIONID, 'en_US', 'Other' from PR_CLASSIFICATION  WHERE CLASSIFICATIONNAME = 'Other'
AND NOT Exists (select 'x' from PR_CLASSIFICATION_loc where CLASSIFICATIONDISPLAYNAME = 'Other')
/ 



-- pr_BusinessIdType.sql

-- pr_BusinessIdType

-- PR_BUSINESSIDTYPE
Insert Into PR_BUSINESSIDTYPE(BUSINESSIDTYPE ,BUSINESSIDNAME,ValidationRule, RESOURCEKEY,BUSINESSIDDESCKEY)
	select PR_BUSINESSIDTYPE_SEQ.Nextval, 'DUNS', 'numeric|minlen=9|maxlen=9|basic-latin', 'DUNS', 'DUNSdesckey' from Dual
	Where Not Exists (select 'x' from PR_BUSINESSIDTYPE where BUSINESSIDNAME = 'DUNS')
/ 
--
Insert Into PR_BUSINESSIDTYPE(BUSINESSIDTYPE ,BUSINESSIDNAME,ValidationRule, RESOURCEKEY,BUSINESSIDDESCKEY)
	select PR_BUSINESSIDTYPE_SEQ.Nextval, 'DUNS+4', 'numeric|minlen=13|maxlen=13|basic-latin', 'DUNS4', 'DUNS4desckey' from Dual
	Where Not Exists (select 'x' from PR_BUSINESSIDTYPE where BUSINESSIDNAME = 'DUNS+4')
/ 

--
Insert Into PR_BUSINESSIDTYPE(BUSINESSIDTYPE ,BUSINESSIDNAME, RESOURCEKEY,BUSINESSIDDESCKEY)
	select PR_BUSINESSIDTYPE_SEQ.Nextval, 'Freeform', 'Freeform', 'Freeformdesckey' from Dual
	Where Not Exists (select 'x' from PR_BUSINESSIDTYPE where BUSINESSIDNAME = 'Freeform')
/




-- pr_BusinessIdType_Loc_ENUS.sql

-- pr_BusinessIdType_loc

Insert into PR_BUSINESSIDTYPE_LOC(BUSINESSIDTYPE ,LOCALE, BUSINESSIDDISPLAYNAME, BUSINESSIDDESC) 
select BUSINESSIDTYPE, 'en_US', BUSINESSIDNAME,'The DnB D-U-N-S Number is a unique nine-digit identification sequence, which provides unique identifiers of single business entities, while linking corporate family structures together.' 
from PR_BUSINESSIDTYPE T where BusinessIdName = 'DUNS'
AND NOT Exists (
	select 1 from  PR_BUSINESSIDTYPE_LOC L
	Where l.BUSINESSIDTYPE = T.BUSINESSIDTYPE
	And   l.LOCALE = 'en_US'
	)
/
	   
Insert into PR_BUSINESSIDTYPE_LOC(BUSINESSIDTYPE ,LOCALE, BUSINESSIDDISPLAYNAME, BUSINESSIDDESC) 
select BUSINESSIDTYPE, 'en_US', BUSINESSIDNAME,'DUNS+4, a more specific 13-character identifier than the original 9-digit DUNS, is currently being used to identify subsidiaries and/or divisions at separate addresses from the registered "main" or "parent" company.' 
from PR_BUSINESSIDTYPE T where BusinessIdName = 'DUNS+4'
AND NOT Exists (
	select 1 from  PR_BUSINESSIDTYPE_LOC L
	Where l.BUSINESSIDTYPE = T.BUSINESSIDTYPE
	And   l.LOCALE = 'en_US'
	)
/

Insert into PR_BUSINESSIDTYPE_LOC(BUSINESSIDTYPE ,LOCALE, BUSINESSIDDISPLAYNAME, BUSINESSIDDESC) 
select BUSINESSIDTYPE, 'en_US', BUSINESSIDNAME,'Partner defined businessID' 
from PR_BUSINESSIDTYPE T where BusinessIdName = 'Freeform'
AND NOT Exists (
	select 1 from  PR_BUSINESSIDTYPE_LOC L
	Where l.BUSINESSIDTYPE = T.BUSINESSIDTYPE
	And   l.LOCALE = 'en_US'
	)
/


-- pr_AddressType.sql

-- pr_AddressType

-- Insert into Header table

Insert Into PR_ADDRESSTYPE(ADDRESSTYPEID ,ADDRESSTYPENAME, RESOURCEKEY )
	select PR_ADDRESSTYPE_SEQ.Nextval, 'Corporate' , 'Corporate' from Dual
	Where Not Exists (select 'x' from PR_ADDRESSTYPE where ADDRESSTYPENAME = 'Corporate')
/ 
--
Insert Into PR_ADDRESSTYPE(ADDRESSTYPEID ,ADDRESSTYPENAME, RESOURCEKEY )
	select PR_ADDRESSTYPE_SEQ.Nextval, 'Billing', 'Billing' from Dual
	Where Not Exists (select 'x' from PR_ADDRESSTYPE where ADDRESSTYPENAME = 'Billing')
/ 
--
Insert Into PR_ADDRESSTYPE(ADDRESSTYPEID ,ADDRESSTYPENAME, RESOURCEKEY )
	select PR_ADDRESSTYPE_SEQ.Nextval, 'Technical', 'Technical' from Dual
	Where Not Exists (select 'x' from PR_ADDRESSTYPE where ADDRESSTYPENAME = 'Technical')
/ 



-- pr_AddressType_Loc_ENUS.sql

-- pr_AddressType_loc 
  
Insert into PR_ADDRESSTYPE_LOC(ADDRESSTYPEID ,LOCALE,ADDRESSTYPEDISPLAYNAME) 
select ADDRESSTYPEID, 'en_US', ADDRESSTYPENAME from PR_ADDRESSTYPE  WHERE ADDRESSTYPENAME = 'Corporate'
and NOT Exists (select 'x' from  PR_ADDRESSTYPE_LOC Where ADDRESSTYPEDISPLAYNAME = 'Corporate')
/
--
Insert into PR_ADDRESSTYPE_LOC(ADDRESSTYPEID ,LOCALE,ADDRESSTYPEDISPLAYNAME) 
select ADDRESSTYPEID, 'en_US', ADDRESSTYPENAME from PR_ADDRESSTYPE  WHERE ADDRESSTYPENAME = 'Billing'
and NOT Exists (select 'x' from  PR_ADDRESSTYPE_LOC Where ADDRESSTYPEDISPLAYNAME = 'Billing')
/
--
Insert into PR_ADDRESSTYPE_LOC(ADDRESSTYPEID ,LOCALE,ADDRESSTYPEDISPLAYNAME) 
select ADDRESSTYPEID, 'en_US', ADDRESSTYPENAME from PR_ADDRESSTYPE  WHERE ADDRESSTYPENAME = 'Technical'
and NOT Exists (select 'x' from  PR_ADDRESSTYPE_LOC Where ADDRESSTYPEDISPLAYNAME = 'Technical')
/



-- pr_PartnerType.sql

-- PR_PARTNERTYPE

Insert Into PR_PARTNERTYPE(PARTNERTYPEID ,PARTNERTYPENAME ,OPERATORFLG ,OWNERFLG ,TYPEFLG ,VISIBILITY, RESOURCEKEY )
   select PR_PARTNERTYPE_SEQ.Nextval, 'HubOperator',1,0,1,1, 'HubOperator' from Dual
   Where Not Exists (select 'x' from PR_PARTNERTYPE where PARTNERTYPENAME = 'HubOperator')
/   

--
Insert Into PR_PARTNERTYPE(PARTNERTYPEID ,PARTNERTYPENAME ,OPERATORFLG ,OWNERFLG ,TYPEFLG ,VISIBILITY, RESOURCEKEY )
   select PR_PARTNERTYPE_SEQ.Nextval, 'HubPartner',0,0,4,1, 'HubPartner' from Dual
   Where Not Exists (select 'x' from PR_PARTNERTYPE where PARTNERTYPENAME = 'HubPartner')
/   

--
Insert Into PR_PARTNERTYPE(PARTNERTYPEID ,PARTNERTYPENAME ,OPERATORFLG ,OWNERFLG ,TYPEFLG ,VISIBILITY, RESOURCEKEY )
   select PR_PARTNERTYPE_SEQ.Nextval, 'HubOwner',0,1,2,1, 'HubOwner' from Dual
   Where Not Exists (select 'x' from PR_PARTNERTYPE where PARTNERTYPENAME = 'HubOwner')
/   

-- pr_PartnerType_loc_ENUS.sql

-- PR_PARTNERTYPE_LOC

Insert into PR_PARTNERTYPE_LOC(PARTNERTYPEID ,LOCALE,PARTNERTYPEDISPLAYNAME) 
select PARTNERTYPEID, 'en_US', 'Community Operator' from PR_PARTNERTYPE where PARTNERTYPENAME = 'HubOperator'
and NOT Exists (select 'x' from  PR_PARTNERTYPE_LOC   Where PARTNERTYPEDISPLAYNAME = 'Community Operator' And LOCALE = 'en_US')
/
--
Insert into PR_PARTNERTYPE_LOC(PARTNERTYPEID ,LOCALE,PARTNERTYPEDISPLAYNAME) 
select PARTNERTYPEID, 'en_US', 'Community Participant' from PR_PARTNERTYPE where PARTNERTYPENAME = 'HubPartner'
and NOT Exists (select 'x' from  PR_PARTNERTYPE_LOC   Where PARTNERTYPEDISPLAYNAME = 'Community Participant' And LOCALE = 'en_US')
/
--
Insert into PR_PARTNERTYPE_LOC(PARTNERTYPEID ,LOCALE,PARTNERTYPEDISPLAYNAME) 
select PARTNERTYPEID, 'en_US', 'Community Manager' from PR_PARTNERTYPE where PARTNERTYPENAME = 'HubOwner'
and NOT Exists (select 'x' from  PR_PARTNERTYPE_LOC   Where PARTNERTYPEDISPLAYNAME = 'Community Manager' And LOCALE = 'en_US')
/

--Update
UPDATE pr_partnertype_loc
   SET partnertypedisplayname = 'Community Operator'
 WHERE partnertypeid = (SELECT d.partnertypeid
                          FROM pr_partnertype_loc t, pr_partnertype d
                         WHERE d.partnertypeid = t.partnertypeid
                           AND d.partnertypename = 'HubOperator'
						         AND t.locale = 'en_US')
/
			  
UPDATE pr_partnertype_loc
   SET partnertypedisplayname = 'Community Participant'
 WHERE partnertypeid = (SELECT d.partnertypeid
                          FROM pr_partnertype_loc t, pr_partnertype d
                         WHERE d.partnertypeid = t.partnertypeid
                           AND d.partnertypename = 'HubPartner'
						         AND t.locale = 'en_US')
/
			  
UPDATE pr_partnertype_loc
   SET partnertypedisplayname = 'Community Manager'
 WHERE partnertypeid = (SELECT d.partnertypeid
                          FROM pr_partnertype_loc t, pr_partnertype d
                         WHERE d.partnertypeid = t.partnertypeid
                           AND d.partnertypename = 'HubOwner'
  						         AND t.locale = 'en_US')
/


-- pr_Permission.sql

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'password-module' AS PERMISSIONNAME                                                                                                 
        ,0 AS ENABLEDFLG,'password-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'password-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'eventcode-module' AS PERMISSIONNAME                                                                                                
    ,0 AS ENABLEDFLG ,'eventcode-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'eventcode-module')
/                                                           
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'document-module' AS PERMISSIONNAME                                                                                                 
    ,1 AS ENABLEDFLG ,'document-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'document-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'eventlog-module' AS PERMISSIONNAME                                                                                                 
        ,1 AS ENABLEDFLG,'eventlog-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'eventlog-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'volume-summary-module' AS PERMISSIONNAME                                                                                           
    ,1 AS ENABLEDFLG,'volume-summary-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'volume-summary-module')
/                                                      
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'partner-module' AS PERMISSIONNAME                                                                                                  
    ,1 AS ENABLEDFLG,'partner-moduled' 
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'partner-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'user-module' AS PERMISSIONNAME                                                                                                     
    ,1 AS ENABLEDFLG,'user-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'user-module')
/                                                                
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'contact-module' AS PERMISSIONNAME                                                                                                  
    ,1 AS ENABLEDFLG,'contact-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'contact-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'volume-analysis-module' AS PERMISSIONNAME                                                                                          
    ,1 AS ENABLEDFLG,'volume-analysis-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'volume-analysis-module')
/                                                     
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'alerts-module' AS PERMISSIONNAME                                                                                                   
    ,1 AS ENABLEDFLG ,'alerts-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'alerts-module')
/                                                              
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'group-module' AS PERMISSIONNAME                                                                                                    
    ,1 AS ENABLEDFLG ,'group-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'group-module')
/                                                               
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'channel-module' AS PERMISSIONNAME                                                                                                  
    ,1 AS ENABLEDFLG ,'channel-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'channel-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'certificate-module' AS PERMISSIONNAME                                                                                              
    ,1 AS ENABLEDFLG ,'certificate-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'certificate-module')
/                                                         
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'process-module' AS PERMISSIONNAME                                                                                                  
    ,1 AS ENABLEDFLG ,'process-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'process-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'gateway-module' AS PERMISSIONNAME                                                                                                  
    ,1 AS ENABLEDFLG ,'gateway-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'gateway-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'workflow-module' AS PERMISSIONNAME
    ,0 AS ENABLEDFLG ,'workflow-moduled'                                                                                                                  
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'workflow-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'businessid-module' AS PERMISSIONNAME                                                                                               
    ,1 AS ENABLEDFLG ,'businessid-moduled'                                                                                                                  
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'businessid-module')
/                                                          
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'testpartnerconnection-module' AS PERMISSIONNAME                                                                                    
    ,1 AS ENABLEDFLG ,'testpartnerconnection-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'testpartnerconnection-module')
/                                               
                                                                                                                                    

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'gw-queue-module' AS PERMISSIONNAME                                                                                                 
    ,1 AS ENABLEDFLG ,'gw-queue-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'gw-queue-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'address-module' AS PERMISSIONNAME                                                                                                  
    ,1 AS ENABLEDFLG,'address-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'address-module')
/                                                             

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'capability-module' AS PERMISSIONNAME                                                                                               
    ,1 AS ENABLEDFLG ,'capability-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'capability-module')
/         
                                                 
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'xml-protocol-module' AS PERMISSIONNAME                                                                                             
    ,0 AS ENABLEDFLG,'xml-protocol-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'xml-protocol-module')
/                                                        
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'receiver-module' AS PERMISSIONNAME                                                                                                 
    ,0 AS ENABLEDFLG ,'receiver-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'receiver-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'maps-module' AS PERMISSIONNAME                                                                                                     
    ,0 AS ENABLEDFLG ,'maps-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'maps-module')
/                                                                
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'routing-object-module' AS PERMISSIONNAME                                                                                           
    ,0 AS ENABLEDFLG ,'routing-object-moduled'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'routing-object-module')
/                                                      
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'vtt-upload' AS PERMISSIONNAME                                                                                                      
    ,1 AS ENABLEDFLG,'vtt-uploadd'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'vtt-upload')
/                                                                 
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'vtt-open-processes' AS PERMISSIONNAME                                                                                              
    ,1 AS ENABLEDFLG ,'vtt-open-processesd'                                                                                                                   
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'vtt-open-processes')
/                                                         
                                                                                                                                    
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'doc-pkg-module' AS PERMISSIONNAME                                                                                                  
    ,1 AS ENABLEDFLG,'doc-pkg-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'doc-pkg-module')
/       

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'locale-module' AS PERMISSIONNAME                                                                                                  
    ,1 AS ENABLEDFLG,'locale-moduled'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'locale-module')
/

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'fixedworkflow-module' AS PERMISSIONNAME                                                                                                  
    ,0 AS ENABLEDFLG,'fixedworkflow-module-manager'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'fixedworkflow-module')
/

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'fixedworkflow-out-module' AS PERMISSIONNAME                                                                                                  
    ,0 AS ENABLEDFLG,'fixedworkflow-out-module-manager'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'fixedworkflow-out-module')
/

--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--SELECT PR_Permission_SEQ.Nextval,'systemadmin-summaryservice-module' AS PERMISSIONNAME                                                                                                  
--    ,0 AS ENABLEDFLG,'systemadmin-summaryservice-module-manager'                                                                                                                    
-- FROM DUAL WHERE NOT EXISTS                                                                                                         
-- (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'systemadmin-summaryservice-module')
--/

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'systemadmin-publishinginfo-module' AS PERMISSIONNAME                                                                                                  
    ,0 AS ENABLEDFLG,'systemadmin-publishinginfo-module-manager'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'systemadmin-publishinginfo-module')
/ 

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'handler-action-module' AS PERMISSIONNAME                                                                                                  
    ,0 AS ENABLEDFLG,'handler-action-module-manager'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'handler-action-module')
/

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'handler-target-module' AS PERMISSIONNAME                                                                                                  
    ,0 AS ENABLEDFLG,'handler-target-module-manager'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'handler-target-module')
/

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'handler-gateway-module' AS PERMISSIONNAME                                                                                                  
    ,0 AS ENABLEDFLG,'handler-gateway-module-manager'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'handler-gateway-module')
/

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
SELECT PR_Permission_SEQ.Nextval,'handler-workflow-module' AS PERMISSIONNAME                                                                                                  
    ,0 AS ENABLEDFLG,'handler-workflow-module-manager'                                                                                                                    
 FROM DUAL WHERE NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'handler-workflow-module')
/

--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--SELECT PR_Permission_SEQ.Nextval,'systemadmin-publicapi-module' AS PERMISSIONNAME                                                                                                  
--    ,0 AS ENABLEDFLG,'systemadmin-publicapi-module-manager'                                                                                                                    
-- FROM DUAL WHERE NOT EXISTS                                                                                                         
-- (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'systemadmin-publicapi-module')
--/


--54523
--1) PermissionName : transformation-maps
--   ResourceKey : transformation-maps-moduled
--   enabledflg=0
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
SELECT PR_Permission_SEQ.Nextval,'transformation-maps' AS PERMISSIONNAME
    ,0 AS ENABLEDFLG,'transformation-maps-moduled'
 FROM DUAL WHERE NOT EXISTS
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'transformation-maps')
/
--2) PermissionName : edi-fa-maps
--   ResourceKey : edi-fa-maps-moduled
--   enabledflg=0
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
SELECT PR_Permission_SEQ.Nextval,'edi-fa-maps' AS PERMISSIONNAME
    ,0 AS ENABLEDFLG,'edi-fa-maps-moduled'
 FROM DUAL WHERE NOT EXISTS
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'edi-fa-maps')
/
--3) PermissionName : enveloper-profile
--   ResourceKey : enveloper-profile-moduled
--   enabledflg=0
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
SELECT PR_Permission_SEQ.Nextval,'enveloper-profile' AS PERMISSIONNAME
    ,0 AS ENABLEDFLG,'enveloper-profile-moduled'
 FROM DUAL WHERE NOT EXISTS
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'enveloper-profile')
/
--4) PermissionName : connection-profile
--   ResourceKey : connection-profile-moduled
--   enabledflg=0
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
SELECT PR_Permission_SEQ.Nextval,'connection-profile' AS PERMISSIONNAME
    ,0 AS ENABLEDFLG,'connection-profile-moduled'
 FROM DUAL WHERE NOT EXISTS
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'connection-profile')
/
--5) PermissionName : control-number-initialization
--   ResourceKey : control-number-initialization-moduled
--   enabledflg=0
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
SELECT PR_Permission_SEQ.Nextval,'control-number-initialization' AS PERMISSIONNAME
    ,0 AS ENABLEDFLG,'control-number-initialization-moduled'
 FROM DUAL WHERE NOT EXISTS
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'control-number-initialization')
/
--6) PermissionName : current-control-numbers
--    ResourceKey : current-control-numbers-moduled
--   enabledflg=0
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
SELECT PR_Permission_SEQ.Nextval,'current-control-numbers' AS PERMISSIONNAME
    ,0 AS ENABLEDFLG,'current-control-numbers-moduled'
 FROM DUAL WHERE NOT EXISTS
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'current-control-numbers')
/
--7) PermissionName : enveloper
--    ResourceKey : enveloper-moduled
--    enabledflg=0
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
SELECT PR_Permission_SEQ.Nextval,'enveloper' AS PERMISSIONNAME
    ,0 AS ENABLEDFLG,'enveloper-moduled'
 FROM DUAL WHERE NOT EXISTS
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'enveloper')
/

--54756
-- Insert commented. CR Num BGTRK00049086
-- INSERT INTO pr_permission (permissionid, permissionname, enabledflg, resourcekey)
-- SELECT pr_permission_seq.nextval, 'ftpconfig-module' AS permissionname
--   ,0 AS enabledflg, 'ftpconfig-moduled'
-- FROM dual WHERE NOT EXISTS
--   (SELECT * FROM pr_permission WHERE permissionname = 'ftpconfig-module')
-- /
 
--54756
-- Insert commented. CR Num BGTRK00049086
-- INSERT INTO pr_permission (permissionid, permissionname, enabledflg, resourcekey)
-- SELECT pr_permission_seq.nextval, 'branding-module' AS permissionname
--   ,0 AS enabledflg, 'branding-moduled'
-- FROM dual WHERE NOT EXISTS
--   (SELECT * FROM pr_permission WHERE permissionname = 'branding-module')
-- /

-- PermissionName : menu.ediReportRejected-module
-- ResourceKey : menu.ediReportRejected-module
-- enabledflg=1
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
SELECT PR_Permission_SEQ.Nextval,'menu.ediReportRejected-module' AS PERMISSIONNAME
    ,1 AS ENABLEDFLG,'menu.ediReportRejected-module'
 FROM DUAL WHERE NOT EXISTS
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'menu.ediReportRejected-module')
/

-- PermissionName : menu.ediReportFAOverdue-module
-- ResourceKey : menu.ediReportFAOverdue-module
-- enabledflg=1
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
SELECT PR_Permission_SEQ.Nextval,'menu.ediReportFAOverdue-module' AS PERMISSIONNAME
    ,1 AS ENABLEDFLG,'menu.ediReportFAOverdue-module'
 FROM DUAL WHERE NOT EXISTS
 (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'menu.ediReportFAOverdue-module')
/

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'permission-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'permission-moduled'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'permission-module')
/

--49486
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY  )
--SELECT PR_Permission_SEQ.Nextval,'menu.systemadmin.docmgrgrp' AS PERMISSIONNAME
--    ,0 AS ENABLEDFLG,'menu.systemadmin.docmgrgrp'
-- FROM DUAL WHERE NOT EXISTS
-- (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'menu.systemadmin.docmgrgrp')
--/

--55630

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'resend-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'resend-moduled'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'resend-module')
/                                                          


-- Start 102380


INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-common-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-common-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-common-module')
/                                                          
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--  SELECT PR_Permission_SEQ.Nextval, 'consoleadmin-group' AS PERMISSIONNAME                                                                                               
--        ,0 AS ENABLEDFLG , 'consoleadmin-group'                                                                                                                   
--    FROM DUAL 
--   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'consoleadmin-group')
--/                                                          
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--  SELECT PR_Permission_SEQ.Nextval, 'docmgradmin-group' AS PERMISSIONNAME                                                                                               
--        ,0 AS ENABLEDFLG , 'docmgradmin-group'                                                                                                                   
--    FROM DUAL 
--   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'docmgradmin-group')
--/                                                          
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--  SELECT PR_Permission_SEQ.Nextval, 'featureadmin-group' AS PERMISSIONNAME                                                                                               
--        ,0 AS ENABLEDFLG , 'featureadmin-group'                                                                                                                   
--    FROM DUAL 
--   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'featureadmin-group')
--/                                                          
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--  SELECT PR_Permission_SEQ.Nextval, 'receiveradmin-group' AS PERMISSIONNAME                                                                                               
--        ,0 AS ENABLEDFLG , 'receiveradmin-group'                                                                                                                   
--    FROM DUAL 
--   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'receiveradmin-group')
--/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-console-general-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-console-general-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-console-general-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-console-jms-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-console-jms-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-console-jms-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-console-vtp-RosettaNet-Simulator-props-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-console-vtp-RosettaNet-Simulator-props-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-console-vtp-RosettaNet-Simulator-props-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-alert-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-alert-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-alert-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'docmgr-asstateengine-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'docmgr-asstateengine-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'docmgr-asstateengine-module')
/                                                          
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--  SELECT PR_Permission_SEQ.Nextval, 'bpe-dae-group' AS PERMISSIONNAME                                                                                               
--        ,0 AS ENABLEDFLG , 'bpe-dae-group'                                                                                                                   
--    FROM DUAL 
--   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'bpe-dae-group')
--/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-deliveryMgr-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-deliveryMgr-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-deliveryMgr-module')
/                                                          
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-eventengine-module' AS PERMISSIONNAME                                                                                               
--        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-eventengine-module'                                                                                                                   
--    FROM DUAL 
--   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-eventengine-module')
--/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-jms-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-jms-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-jms-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-packaging-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-packaging-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-packaging-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-rn-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-rn-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-rn-module')
/                                                          
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-process-module' AS PERMISSIONNAME                                                                                               
--        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-process-module'                                                                                                                   
--    FROM DUAL 
--   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-process-module')
--/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-security-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-security-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-security-module')
/                                                          
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-others-module' AS PERMISSIONNAME                                                                                               
--        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-others-module'                                                                                                                   
--    FROM DUAL 
--   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-others-module')
--/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-bpe-main-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-bpe-main-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-bpe-main-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-bpe-signal-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-bpe-signal-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-bpe-signal-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-bpe-synch-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-bpe-synch-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-bpe-synch-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'systemadmin-eventenginegeneral-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'systemadmin-eventenginegeneral-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'systemadmin-eventenginegeneral-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'systemadmin-publishinginfo-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'systemadmin-publishinginfo-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'systemadmin-publishinginfo-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-rm-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-rm-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-rm-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-docmgr-ebms-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-docmgr-ebms-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-docmgr-ebms-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'systemadmin-summaryengine-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'systemadmin-summaryengine-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'systemadmin-summaryengine-module')
/                                                          

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'resend-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'resend-moduled'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'resend-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'systemadmin-archiver-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'systemadmin-archiver-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'systemadmin-archiver-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'systemadmin-publicapiinfo-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'systemadmin-publicapiinfo-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'systemadmin-publicapiinfo-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-edi-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-edi-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-edi-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-receiver-jms-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-receiver-jms-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-receiver-jms-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-receiver-sync-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-receiver-sync-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-receiver-sync-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-receiver-dir-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-receiver-dir-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-receiver-dir-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'sysadmin-receiver-others-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'sysadmin-receiver-others-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'sysadmin-receiver-others-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'ebms-viewer-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'ebms-viewer-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'ebms-viewer-module')
/                                                          
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'ebms' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'ebms'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'ebms')
/                                                          
--INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
--  SELECT PR_Permission_SEQ.Nextval, 'connections-group' AS PERMISSIONNAME                                                                                               
--        ,0 AS ENABLEDFLG , 'connections-group'                                                                                                                   
--    FROM DUAL 
--   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'connections-group')
--/                                                          
-- End 102380
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'migration-module' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'migration-module'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'migration-module')
/  
-- 101952 permission for Stop Processing function
INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'stop-document-processing-function' AS PERMISSIONNAME                                                                                               
        ,0 AS ENABLEDFLG , 'stop-document-processing-functiond'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'stop-document-processing-function')
/   

-- Added for 105256 

INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'ftp-stats-module' AS PERMISSIONNAME                                                                                               
        ,1 AS ENABLEDFLG , 'ftp-stats-moduled'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'ftp-stats-module')
/


INSERT INTO PR_Permission (PERMISSIONID, PERMISSIONNAME, ENABLEDFLG, RESOURCEKEY )                                                                      
  SELECT PR_Permission_SEQ.Nextval, 'ftp-connections-module' AS PERMISSIONNAME                                                                                               
        ,1 AS ENABLEDFLG , 'ftp-connections-moduled'                                                                                                                   
    FROM DUAL 
   WHERE NOT Exists (SELECT * FROM PR_Permission WHERE PERMISSIONNAME = 'ftp-connections-module')
/



-- pr_Permission_loc_ENUS.sql

-- PR_PERMISSION_LOC_ENUS 
--select * from PR_PERMISSION_LOC/

INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
  SELECT PERMISSIONID 
        ,'en_US' AS LOCALE                                                                                                                     
        ,'Permission Manager' AS PERMISSIONDESC                                                                                            
    FROM PR_PERMISSION 
   WHERE PERMISSIONNAME = 'permission-module'
   AND NOT Exists 
   (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME = 'permission-module')
/
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID                                                                                                                              
,'en_US' AS LOCALE                                                                                                                     
,'Password Policy Manager' AS PERMISSIONDESC                                                                                        
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'password-module'
 AND    NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'password-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID                                                                                                                              
,'en_US' AS LOCALE                                                                                                                   
,'Event Code Manager' AS PERMISSIONDESC                                                                                             
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'eventcode-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'eventcode-module')
/                                                           
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID                                                                                                                             
,'en_US' AS LOCALE                                                                                                                     
,'Document Viewer' AS PERMISSIONDESC                                                                                                
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'document-module'
 AND NOT EXISTS                                                                                                            
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'document-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID                                                                                                                             
,'en_US' AS LOCALE                                                                                                                     
,'Event Viewer' AS PERMISSIONDESC                                                                                               
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'eventlog-module'
 AND NOT EXISTS
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'eventlog-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Document Volume Report' AS PERMISSIONDESC                                                                                          
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'volume-summary-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'volume-summary-module')
/                                                      
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Community Participant Manager' AS PERMISSIONDESC                                                                                        
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'partner-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'partner-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'User Manager' AS PERMISSIONDESC                                                                                           
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'user-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'user-module')
/                                                                
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Contact Manager' AS PERMISSIONDESC                                                                                       
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'contact-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'contact-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Document Analysis' AS PERMISSIONDESC                                                                                         
 FROM PR_PERMISSION 
  WHERE PERMISSIONNAME = 'volume-analysis-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'volume-analysis-module')
/                                                     
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Alert Manager' AS PERMISSIONDESC                                                                                             
 FROM PR_PERMISSION 
   WHERE PERMISSIONNAME = 'alerts-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'alerts-module')
/                                                              
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Group Manager' AS PERMISSIONDESC                                                                                                  
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'group-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'group-module')
/                                                               
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Participant Connection Manager' AS PERMISSIONDESC                                                                                                 
 FROM PR_PERMISSION 
  WHERE PERMISSIONNAME = 'channel-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'channel-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Certificate Manager' AS PERMISSIONDESC                                                                                            
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'certificate-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'certificate-module')
/                                                         
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'RosettaNet Viewer' AS PERMISSIONDESC                                                                                                 
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'process-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'process-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Gateway Manager' AS PERMISSIONDESC                                                                                                       
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'gateway-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'gateway-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Action Manager' AS PERMISSIONDESC                                                                                     
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'workflow-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'workflow-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Exclusion List ID Manager' AS PERMISSIONDESC                                                                     
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'businessid-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'businessid-module')
/                                                          
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Test Participant Connection' AS PERMISSIONDESC                                                                                        
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'testpartnerconnection-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'testpartnerconnection-module')
/                                               
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Gateway Queue' AS PERMISSIONDESC                                                                                                
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'gw-queue-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'gw-queue-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Address Manager' AS PERMISSIONDESC                                                                                         
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'address-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'address-module')
/                                                             
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'FTP Manager' AS PERMISSIONDESC                                                                                      
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'ftpconfig-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'ftpconfig-module')
/                                                           
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Console Branding Manager' AS PERMISSIONDESC                                                                                               
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'branding-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'branding-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'B2B Capability Manager' AS PERMISSIONDESC                                                                                    
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'capability-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'capability-module')
/         
                                                 
                                                                                                                                   
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'XML Format Manager' AS PERMISSIONDESC                                                                                            
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'xml-protocol-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'xml-protocol-module')
/                                                        
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Target Manager' AS PERMISSIONDESC                                                                                                
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'receiver-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'receiver-module')
/                                                            
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Validation Map Manager' AS PERMISSIONDESC                                                                                                    
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'maps-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'maps-module')
/                                                                
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Document Flow Definition Manager' AS PERMISSIONDESC                                                                                          
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'routing-object-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'routing-object-module')
/                                                      
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Initiate Document Flow' AS PERMISSIONDESC                                                                                                     
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'vtt-upload'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'vtt-upload')
/                                                                 
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'View Document Flow' AS PERMISSIONDESC                                                                                             
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'vtt-open-processes'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'vtt-open-processes')
/                                                         
                                                                                                                                    
INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'AS1/AS2 Viewer' AS PERMISSIONDESC                                                                                                 
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'doc-pkg-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'doc-pkg-module')
/       

INSERT INTO PR_Permission_loc (PERMISSIONID, LOCALE, PERMISSIONDESC)
SELECT PERMISSIONID
,'en_US' AS LOCALE                                                                                                                     
,'Locale Manager' AS PERMISSIONDESC                                                                                                 
 FROM PR_PERMISSION 
 WHERE PERMISSIONNAME = 'locale-module'
 AND NOT EXISTS                                                                                                         
 (SELECT * FROM PR_Permission p, PR_PERMISSION_LOC l WHERE p.PERMISSIONID = l.PERMISSIONID AND p.PERMISSIONNAME =  'locale-module')
/       



-- OperatorInit.sql

-- This is a script to initializ an operator account in the data base.

-- The tables updated by this script are:
--
--     pr_Partner
--     pr_Contact
--     pr_User
--     pr_Group
--     pr_User_Group
--     pr_Permission_Group
--

-- PR_Partner

Insert Into PR_PARTNER( PartnerId, PartnerName, Enabled, PartnerTypeId, PartnerLogin, locale)
  Select PR_PARTNER_SEQ.Nextval, 'Hub Operator', 1, p.PartnerTypeId, 'Operator', 'en_US'
    From pr_PartnerType p 
   Where p.PartnerTypeName = 'HubOperator'
     And NOT Exists ( Select 'x' from PR_PARTNER where PartnerName = 'Hub Operator' )
/

--PR_CONTACT
--Contact information for Hub Administrator

Insert Into PR_CONTACT( ContactId, PartnerId, Name1, NotifyFlg, VisibilityFlg, LanguageLocale, FormatLocale, TimeZone )
  Select PR_CONTACT_SEQ.Nextval
       , (Select PartnerId From Pr_Partner Where PartnerName = 'Hub Operator' )
       , 'Hub Administrator'
       , 0, 0 , 'en_US', 'en_US', 'UTC'
    From Dual 
   Where NOT Exists ( Select 'x' From PR_CONTACT Where Name1 = 'Hub Administrator')
/

--Contact information for Administrator
Insert Into PR_CONTACT( ContactId, PartnerId, Name1, NotifyFlg, VisibilityFlg, LanguageLocale, FormatLocale, TimeZone )
  Select PR_CONTACT_SEQ.Nextval
       , (Select PartnerId From Pr_Partner Where PartnerName = 'Hub Operator' )
       , 'Administrator for Hub Operator'
       , 0, 0, 'en_US', 'en_US', 'UTC' 
   From Dual 
  Where NOT Exists ( Select 'x' From PR_CONTACT Where Name1 = 'Administrator for Hub Operator')
/

-- PR_USER
-- NOTE: Password is an encrypted string and is based on the following: 'Pa55word'.
--#1 
Insert Into PR_USER(USERID,PARTNERID,CONTACTID,USERNAME,PASSWORD,ENABLEDFLG,PWDRESETFLG)
Select PR_USER_SEQ.Nextval, p.PartnerId,c.ContactId,'hubadmin','4DC515D55FA80E036EBA47924BD31378',1,1
from pr_Partner p
Inner Join pr_Contact c on  c.partnerId = p.partnerId
WHERE p.Partnername = 'Hub Operator'
AND c.Name1 = 'Hub Administrator'
And Not Exists (Select 'x' from pr_User where UserName = 'hubadmin')
/

--#2 
Insert Into PR_USER(USERID,PARTNERID,CONTACTID,USERNAME,PASSWORD,ENABLEDFLG,PWDRESETFLG)
Select PR_USER_SEQ.Nextval, p.PartnerId,c.ContactId,'Admin','4DC515D55FA80E036EBA47924BD31378',1,1
from pr_Partner p
Inner Join pr_Contact c on c.partnerId = p.partnerId
WHERE p.Partnername = 'Hub Operator'
AND c.Name1 = 'Administrator for Hub Operator'
And Not Exists (Select 'x' from pr_User where UserName = 'Admin')
/

--#3
Insert Into PR_USER(USERID,PARTNERID,CONTACTID,USERNAME,PASSWORD,ENABLEDFLG,PWDRESETFLG)
Select PR_USER_SEQ.Nextval,p.PartnerId,c.ContactId,'ftpadmin','4DC515D55FA80E036EBA47924BD31378',1,1
from pr_Partner p
Inner Join pr_Contact c on  c.partnerId = p.partnerId
WHERE p.Partnername = 'Hub Operator'
AND c.Name1 = 'Hub Administrator'
And Not Exists (Select 'x' from pr_User where UserName = 'ftpadmin')
/

--PR_GROUP

-- #1
Insert Into PR_GROUP( GROUPID, PARTNERID, GROUPNAME, GROUPDESC )
  Select PR_GROUP_SEQ.Nextval
       , (Select PartnerId From Pr_Partner Where PartnerName = 'Hub Operator' )
       , 'Administrators'
       , 'Administrator Group for Operator' 
    From Dual 
   Where NOT Exists ( Select 'x' From PR_GROUP Where GROUPNAME = 'Administrators')
/

-- #2
Insert Into PR_GROUP( GROUPID, PARTNERID, GROUPNAME, GROUPDESC )
  Select PR_GROUP_SEQ.Nextval
       , (Select PartnerId From Pr_Partner Where PartnerName = 'Hub Operator' )
       , 'Default'
       , 'Default Users Group for Operator'
    From Dual 
   Where NOT Exists ( Select 'x' From PR_GROUP Where GROUPNAME = 'Default')
/
--55763
Insert Into PR_GROUP( GROUPID, PARTNERID, GROUPNAME, GROUPDESC )
  Select PR_GROUP_SEQ.Nextval
       , (Select PartnerId From Pr_Partner Where PartnerName = 'Hub Operator' )
       , 'Hubadmin'
       , 'Hubadmin Group for Operator'
    From Dual 
   Where NOT Exists ( Select 'x' From PR_GROUP Where GROUPNAME = 'Hubadmin')
/
-- PR_USER_GROUP

INSERT INTO PR_USER_GROUP ( UserID, GroupId )
  Select u.UserID, GroupId 
    From PR_USER u 
   Inner Join PR_PARTNER p On p.PartnerId = u.PartnerId 
   Inner Join PR_Group g On g.PartnerId = p.PartnerId
   Where u.USERNAME = 'hubadmin' 
     And p.PartnerName = 'Hub Operator' 
     And g.Groupname = 'Administrators'
     And NOT Exists ( Select 'x' From PR_USER_GROUP ug Where ug.UserID = u.UserID)
/


INSERT INTO PR_USER_GROUP ( UserID, GroupId )
  Select u.UserID, GroupId 
    From PR_USER u 
   Inner Join PR_PARTNER p ON p.PartnerId = u.PartnerId 
   Inner Join PR_Group g On g.PartnerId = p.PartnerId
   Where u.USERNAME = 'Admin' 
     And p.PartnerName = 'Hub Operator' 
     And g.Groupname='Administrators'
     And NOT Exists ( Select 'x' From PR_USER_GROUP ug Where ug.UserID = u.UserID)
/   
--55763
INSERT INTO PR_USER_GROUP ( UserID, GroupId )
  Select u.UserID, GroupId 
    From PR_USER u 
   Inner Join PR_PARTNER p On p.PartnerId = u.PartnerId 
   Inner Join PR_Group g On g.PartnerId = p.PartnerId
   Where u.USERNAME = 'hubadmin' 
     And p.PartnerName = 'Hub Operator' 
     And g.Groupname = 'Hubadmin'
     And NOT Exists ( Select 'x' From PR_USER_GROUP ug Where ug.UserID = u.UserID and ug.groupid = g.groupid)
/
--PR_PERMISSION_GROUP

-- #1
--Insert Into PR_PERMISSION_GROUP(PERMISSIONID,GROUPID,ACCESSTYPE)
--Select (PR_PERMISSION_GROUP_SEQ.Nextval, g.GroupId,'rw' from pr_Group g WHERE g.GroupName = --'Administrator' AND g.GroupDesc = 'Administrator Group for Operator'
/

--Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
--   Select (Select PermissionId From pr_Permission Where PermissionName = 'password-module')
--        , g.GroupId 
--        , 'rw' 
--     from pr_Group g
--    inner join pr_Partner p on p.PartnerId = g.PartnerId
--    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
--      And NOT Exists (Select 'x' From pr_Permission_Group pg
--                       Where PermissionId = (Select PermissionId From pr_Permission 
--                                              Where PermissionName = 'password-module')
--                         And pg.GroupId = g.GroupId
--                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'document-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'document-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'eventlog-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'eventlog-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'volume-summary-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'volume-summary-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'partner-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'partner-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'user-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'user-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'contact-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'contact-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'volume-analysis-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'volume-analysis-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'alerts-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'alerts-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'group-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'group-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'channel-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'channel-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'certificate-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'certificate-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'process-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'process-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'gateway-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'gateway-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'businessid-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'businessid-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'testpartnerconnection-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'testpartnerconnection-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'gw-queue-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'gw-queue-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'address-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'address-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

--Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
--   Select (Select PermissionId From pr_Permission Where PermissionName = 'ftpconfig-module')
--        , g.GroupId 
--        , 'rw' 
--     from pr_Group g
--    inner join pr_Partner p on p.PartnerId = g.PartnerId
--    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
--      And NOT Exists (Select 'x' From pr_Permission_Group pg 
--                       Where PermissionId = (Select PermissionId From pr_Permission 
--                                              Where PermissionName = 'ftpconfig-module')
--                         And pg.GroupId = g.GroupId
--                         And pg.AccessType = 'rw' )
/

--Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
--   Select (Select PermissionId From pr_Permission Where PermissionName = 'branding-module')
--        , g.GroupId 
--        , 'rw' 
--     from pr_Group g
--    inner join pr_Partner p on p.PartnerId = g.PartnerId
--    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
--      And NOT Exists (Select 'x' From pr_Permission_Group pg 
--                       Where PermissionId = (Select PermissionId From pr_Permission 
--                                              Where PermissionName = 'branding-module')
--                         And pg.GroupId = g.GroupId
--                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'capability-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'capability-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/


--#2
--Insert Into PR_PERMISSION_GROUP(PERMISSIONID,GROUPID,ACCESSTYPE)
--Select PR_PERMISSION_GROUP_SEQ.Nextval, g.GroupId,'r' from pr_Group g WHERE g.GroupName =  'Default' --AND  --g.GroupDesc = 'Default Users Group for Operator'
/

--Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
--   Select (Select PermissionId From pr_Permission Where PermissionName = 'password-module')
--        , g.GroupId 
--        , 'r' 
--     from pr_Group g
--    inner join pr_Partner p on p.PartnerId = g.PartnerId
--    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
--      And NOT Exists (Select 'x' From pr_Permission_Group pg 
--                       Where PermissionId = (Select PermissionId From pr_Permission 
--                                              Where PermissionName = 'password-module')
--                         And pg.GroupId = g.GroupId
--                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'document-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'document-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'eventlog-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'eventlog-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'volume-summary-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'volume-summary-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'partner-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'partner-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'user-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'user-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'contact-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'contact-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'volume-analysis-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'volume-analysis-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'alerts-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'alerts-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'group-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'group-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/
 
Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'channel-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'channel-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'certificate-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'certificate-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'process-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'process-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'gateway-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'gateway-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'businessid-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'businessid-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'testpartnerconnection-module')
        , g.GroupId 
        , 'r' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'testpartnerconnection-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'r' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'gw-queue-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'gw-queue-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'address-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'address-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/

--Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
--   Select (Select PermissionId From pr_Permission Where PermissionName = 'ftpconfig-module')
--        , g.GroupId 
--        , 'rw' 
--     from pr_Group g
--    inner join pr_Partner p on p.PartnerId = g.PartnerId
--    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
--      And NOT Exists (Select 'x' From pr_Permission_Group pg 
--                       Where PermissionId = (Select PermissionId From pr_Permission 
--                                              Where PermissionName = 'ftpconfig-module')
--                         And pg.GroupId = g.GroupId
--                         And pg.AccessType = 'rw' )
/

--Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
--   Select (Select PermissionId From pr_Permission Where PermissionName = 'branding-module')
--        , g.GroupId 
--        , 'rw' 
--     from pr_Group g
--    inner join pr_Partner p on p.PartnerId = g.PartnerId
--    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
--      And NOT Exists (Select 'x' From pr_Permission_Group pg 
--                       Where PermissionId = (Select PermissionId From pr_Permission 
--                                              Where PermissionName = 'branding-module')
--                         And pg.GroupId = g.GroupId
--                         And pg.AccessType = 'rw' )
/

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'capability-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'capability-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/
                         
                         
-- Added for 105256 

Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'ftp-stats-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'ftp-stats-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/
                         
Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'ftp-stats-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'ftp-stats-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/
                         
Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'ftp-connections-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Administrators' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'ftp-connections-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/
                         
Insert Into pr_Permission_Group ( PermissionId, GroupId, AccessType )
   Select (Select PermissionId From pr_Permission Where PermissionName = 'ftp-connections-module')
        , g.GroupId 
        , 'rw' 
     from pr_Group g
    inner join pr_Partner p on p.PartnerId = g.PartnerId
    Where g.GroupName = 'Default' And p.PartnerName = 'Hub Operator'
      And NOT Exists (Select 'x' From pr_Permission_Group pg 
                       Where PermissionId = (Select PermissionId From pr_Permission 
                                              Where PermissionName = 'ftp-connections-module')
                         And pg.GroupId = g.GroupId
                         And pg.AccessType = 'rw' )
/
                         
-- End of fix for 105256 

-- cf_ftp_user



Insert into cf_ftp_user (ftpuserid,USERNAME, HomeDirectory,ENABLEFLAG,WRITEPERMISSION,
              IDLETIME,UPLOADRATE,DOWNLOADRATE,MAXLOGINPERIP,
              MODIFYDIR)
Select cf_ftp_user_seq.nextval,'ftpadmin','conf/ftpadmin',1,1,0,0,0,0,1
 From DUAL 
where NOT Exists ( Select 'x' from cf_ftp_user where USERNAME = 'ftpadmin')
/

-- pr_PwdPolicy.sql

-- PR_PwdPolicy
        

Insert Into PR_PWDPOLICY( PolicyId, PartnerID, PwdMinLength, PwdExpireDays, PwdUnique, PwdSpecialChars, PwdNameCheck )
  Select PR_PWDPOLICY_SEQ.Nextval
       , (Select PartnerId From Pr_Partner Where PartnerName = 'Hub Operator' )
       , 8, 30, 10, 0, 1 
    From Dual 
   Where NOT Exists ( Select 'x' From PR_PWDPOLICY Where PartnerID = (Select PartnerId From Pr_Partner Where PartnerName = 'Hub Operator' ))
/   


-- pr_RcvConfigDtl.sql

-- Meta data for PR_RcvConfigDtl

--  Do not Populate this table.  F104773.

/* insert into PR_RcvConfigDtl (RCVCONFIGDTLID, TRANSPORTID, MAXSYNCTIMEOUT, MAXSYNCSIMULCONNECTION) 
Select PR_RcvConfigDtl_SEQ.nextval 
                   ,(Select TRANSPORTID from pr_transport where TransportType = 'HttpS')
		   ,600000
                   ,100 from dual
 Where Not exists (select * from PR_RcvConfigDtl where TRANSPORTID = 
                    (Select TRANSPORTID from pr_transport where TransportType = 'HttpS')
                  )
      ;
*/



-- lg_Media.sql

-- LG_MEDIA LG_LEDIA_CFG
-- Install Script to load the Non-Repudiation and Message Store URI's.
-- The Input parameters denoted with a $ character will be replaced at install.
-- The URI MUST be prefixed with file:
   
--INSERT INTO LG_MEDIA ( MEDIAID, LINKMEDIAID, URI, TYPEFLG, PURGEFLG, ARCHIVEFLG, ROWTS ) 
--  Select LG_MEDIA_SEQ.NEXTVAL, NULL, 'file://opt/data/vcrouter/vms/non_rep', 1, 0, 0, 1
--    From dual
--   Where Not Exists (Select 'x' From LG_MEDIA Where URI = 'file://opt/data/vcrouter/vms/non_rep')
--/
--
--INSERT INTO LG_MEDIA ( MEDIAID, LINKMEDIAID, URI, TYPEFLG, PURGEFLG, ARCHIVEFLG, ROWTS ) 
--  Select LG_MEDIA_SEQ.NEXTVAL, NULL, 'file://opt/data/vcrouter/vms/msg_store', 2, 0, 0, 1
--    From dual
--   Where Not Exists (Select 'x' From LG_MEDIA Where URI = 'file://opt/data/vcrouter/vms/msg_store')
--/
INSERT INTO LG_MEDIA (MediaId, LinkMediaId, URI, TYPEFLG, PURGEFLG, ARCHIVEFLG, ROWTS ) 
  Select LG_MEDIA_SEQ.NEXTVAL, NULL,'$non_rep$', 1, 0, 0, 1
    From dual
   Where Not Exists (Select 'x' From LG_MEDIA Where TYPEFLG = 1)
/

INSERT INTO LG_MEDIA (MediaId,LinkMediaId, URI, TYPEFLG, PURGEFLG, ARCHIVEFLG, ROWTS ) 
  Select LG_MEDIA_SEQ.NEXTVAL, NULL,'$msg_store$', 2, 0, 0, 1
    From dual
   Where Not Exists (Select 'x' From LG_MEDIA Where TYPEFLG = 2)
/

INSERT INTO LG_MEDIA_CFG ( MEDIACFGID, MEDIAID, TYPEFLG, ROWTS )
  Select LG_MEDIA_CFG_SEQ.NEXTVAL, MEDIAID, 1, 1 
    From LG_MEDIA 
   Where TYPEFLG = 1
     And NOT Exists (Select 'x' From LG_MEDIA_CFG Where TYPEFLG = 1)
/

INSERT INTO LG_MEDIA_CFG ( MEDIACFGID, MEDIAID, TYPEFLG, ROWTS )
  Select LG_MEDIA_CFG_SEQ.NEXTVAL, MEDIAID, 2, 1 
    From LG_MEDIA 
   Where TYPEFLG = 2
     And NOT Exists (Select 'x' From LG_MEDIA_CFG Where TYPEFLG = 2)
/


-- db_SchemaVer.sql

-- DB_SCHEMAVER

--should only be 1 row in table
DELETE FROM db_schemaver
/

Insert Into DB_SCHEMAVER( SCHEMAVERID, VERSION, BUILD )
     select DB_SCHEMAVER_SEQ.nextval
          , '6.2.0'
          ,'' 
       from dual
      where NOT Exists (select 'x' from db_schemaver where version = '6.2.0')
/



-- db_MsgLength.sql


--DB_MSGLENGTH

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '0-1K', 1024 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '0-1K' and SIZECATEGORY = 1024)
/ 
--
Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '1-2MB',	2048000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '1-2MB' and SIZECATEGORY = 2048000)
/ 

--
Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '1-5K', 5120 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '1-5K' and SIZECATEGORY = 5120)
/

--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '10-20K', 20480 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '10-20K' and SIZECATEGORY = 20480)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '10-20MB', 20480000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '10-20MB' and SIZECATEGORY = 20480000)
/

--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '100-200K', 204800 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '100-200K' and SIZECATEGORY = 204800)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '100-200MB', 204800000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '100-200MB' and SIZECATEGORY = 204800000)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '2-5MB', 5120000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '2-5MB' and SIZECATEGORY = 5120000)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '20-50K', 51200 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '20-50K' and SIZECATEGORY = 51200)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '20-50MB', 51200000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '20-50MB' and SIZECATEGORY = 51200000)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '200-300K', 307200 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '200-300K' and SIZECATEGORY = 307200)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '200-300MB', 307200000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '200-300MB' and SIZECATEGORY = 307200000)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '300-500K', 512000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '300-500K' and SIZECATEGORY = 512000)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '300-500MB', 512000000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '300-500MB' and SIZECATEGORY = 512000000)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '5-10K', 10240 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '5-10K' and SIZECATEGORY = 10240)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '5-10MB', 10240000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '5-10MB' and SIZECATEGORY = 10240000)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '50-100K', 102400 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '50-100K' and SIZECATEGORY = 102400)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '50-100MB', 102400000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '50-100MB' and SIZECATEGORY = 102400000)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '500-1024K', 1024000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '500-1024K' and SIZECATEGORY = 1024000)
/
--

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select 'N/A', -1 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = 'N/A' and SIZECATEGORY = -1)
/



-- Added for CR105789

Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '500MB-1GB', 1024000000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '500MB-1GB' and SIZECATEGORY = 1024000000)
/



Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '1GB-2GB', 2048000000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '1GB-2GB' and SIZECATEGORY = 2048000000)
/


Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '2GB-3GB', 3072000000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '2GB-3GB' and SIZECATEGORY = 3072000000)
/


Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '3GB-5GB', 5120000000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '3GB-5GB' and SIZECATEGORY = 5120000000)
/


Insert Into DB_MSGLENGTH(SIZECATEGORYID ,SIZECATEGORY )
	select '5GB-10GB', 10240000000 from Dual
	Where Not Exists (select 'x' from DB_MSGLENGTH where SIZECATEGORYID = '5GB-10GB' and SIZECATEGORY = 10240000000)
/





-- cf_StructureType.sql


Insert Into CF_StructureType (STRUCTURETYPEID,STRUCTURETYPEKEY, RESOURCEKEY)
       Select CF_StructureType_Seq.nextval,'XML', 'XML' from Dual
       Where Not Exists (select 'x' from CF_StructureType   where STRUCTURETYPEKEY = 'XML')
/       

 




-- cf_StructureType_Loc_ENUS.sql

-- CF_StructureType_Loc

Insert into CF_StructureType_Loc(STRUCTURETYPEID ,LOCALE,STRUCTURETYPENAME) 
select STRUCTURETYPEID, 'en_US', 'XML' from CF_StructureType where STRUCTURETYPEKEY = 'XML'
and NOT Exists (select 'x' from  CF_StructureType_Loc Where STRUCTURETYPENAME = 'XML' And LOCALE = 'en_US')
/


-- CF_RNPIPATTR_GRP.sql

-- bcg.grp.ro.rnif.0A1
INSERT INTO CF_RNPIPATTR_GRP (
   RNPIPATTR_GRPID, RNPIPATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.0A1'
		,180
		,'bcg.grp.ro.rnif.0A1'
		,'bcg.grp.ro.rnif.0A1'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_RNPIPATTR_GRP WHERE RNPIPATTR_GRPKEY = 'bcg.grp.ro.rnif.0A1')
	)
/

-- bcg.grp.ro.rnif.CONTACTINFO
INSERT INTO CF_RNPIPATTR_GRP (
   RNPIPATTR_GRPID, RNPIPATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.CONTACTINFO'
		,130
		,'bcg.grp.ro.rnif.CONTACTINFO'
		,'bcg.grp.ro.rnif.CONTACTINFO'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_RNPIPATTR_GRP WHERE RNPIPATTR_GRPKEY = 'bcg.grp.ro.rnif.CONTACTINFO')
	)
/

-- bcg.grp.ro.rnif.GENERAL
INSERT INTO CF_RNPIPATTR_GRP (
   RNPIPATTR_GRPID, RNPIPATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.GENERAL'
		,100
		,'bcg.grp.ro.rnif.GENERAL'
		,'bcg.grp.ro.rnif.GENERAL'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_RNPIPATTR_GRP WHERE RNPIPATTR_GRPKEY = 'bcg.grp.ro.rnif.GENERAL')
	)
/

-- bcg.grp.ro.rnif.ROLE
INSERT INTO CF_RNPIPATTR_GRP (
   RNPIPATTR_GRPID, RNPIPATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.ROLE'
		,120
		,'bcg.grp.ro.rnif.ROLE'
		,'bcg.grp.ro.rnif.ROLE'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_RNPIPATTR_GRP WHERE RNPIPATTR_GRPKEY = 'bcg.grp.ro.rnif.ROLE')
	)
/

-- bcg.grp.ro.rnif.DUNS
INSERT INTO CF_RNPIPATTR_GRP (
   RNPIPATTR_GRPID, RNPIPATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.DUNS'
		,110
		,'bcg.grp.ro.rnif.DUNS'
		,'bcg.grp.ro.rnif.DUNS'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_RNPIPATTR_GRP WHERE RNPIPATTR_GRPKEY = 'bcg.grp.ro.rnif.DUNS')
	)
/

-- bcg.grp.ro.rnif.DOCID
INSERT INTO CF_RNPIPATTR_GRP (
   RNPIPATTR_GRPID, RNPIPATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.DOCID'
		,140
		,'bcg.grp.ro.rnif.DOCID'
		,'bcg.grp.ro.rnif.DOCID'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_RNPIPATTR_GRP WHERE RNPIPATTR_GRPKEY = 'bcg.grp.ro.rnif.DOCID')
	)
/

-- bcg.grp.ro.rnif.GLBDOCCD
INSERT INTO CF_RNPIPATTR_GRP (
   RNPIPATTR_GRPID, RNPIPATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.GLBDOCCD'
		,150
		,'bcg.grp.ro.rnif.GLBDOCCD'
		,'bcg.grp.ro.rnif.GLBDOCCD'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_RNPIPATTR_GRP WHERE RNPIPATTR_GRPKEY = 'bcg.grp.ro.rnif.GLBDOCCD')
	)
/

-- bcg.grp.ro.rnif.GLBSUPPCD
INSERT INTO CF_RNPIPATTR_GRP (
   RNPIPATTR_GRPID, RNPIPATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.GLBSUPPCD'
		,160
		,'bcg.grp.ro.rnif.GLBSUPPCD'
		,'bcg.grp.ro.rnif.GLBSUPPCD'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_RNPIPATTR_GRP WHERE RNPIPATTR_GRPKEY = 'bcg.grp.ro.rnif.GLBSUPPCD')
	)
/

-- bcg.grp.ro.rnif.GLBCLASSCD
INSERT INTO CF_RNPIPATTR_GRP (
   RNPIPATTR_GRPID, RNPIPATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.GLBCLASSCD'
		,170
		,'bcg.grp.ro.rnif.GLBCLASSCD'
		,'bcg.grp.ro.rnif.GLBCLASSCD'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_RNPIPATTR_GRP WHERE RNPIPATTR_GRPKEY = 'bcg.grp.ro.rnif.GLBCLASSCD')
	)
/




-- cf_RNPipMetaDef.sql

-- CF_RNPIPMetaDef

Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'DTDName','DTDName' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY = 'DTDName')
/ 

Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'FrService','FrService' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY = 'FrService')
/ 

Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'ToService','ToService' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY = 'ToService')
/ 

Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'FrRole','FrRole' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY = 'FrRole')
/

Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select  CF_RNPIPMetaDef_Seq.nextval,'ToRole','ToRole' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY = 'ToRole')
/

Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'RootTag','RootTag' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY = 'RootTag')
/

Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'ResponseFrActionName','ResponseFrActionName' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY = 'ResponseFrActionName')
/
-- added for 49576
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.0', 'bcg.pip.xpath.frRole.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.1', 'bcg.pip.xpath.frRole.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.2', 'bcg.pip.xpath.frRole.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.3', 'bcg.pip.xpath.frRole.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.4', 'bcg.pip.xpath.frRole.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.5', 'bcg.pip.xpath.frRole.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.6', 'bcg.pip.xpath.frRole.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.7', 'bcg.pip.xpath.frRole.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.8', 'bcg.pip.xpath.frRole.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frRole.9', 'bcg.pip.xpath.frRole.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frRole.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.0', 'bcg.pip.xpath.frGlbSuppCD.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.1', 'bcg.pip.xpath.frGlbSuppCD.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.2', 'bcg.pip.xpath.frGlbSuppCD.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.3', 'bcg.pip.xpath.frGlbSuppCD.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.4', 'bcg.pip.xpath.frGlbSuppCD.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.5', 'bcg.pip.xpath.frGlbSuppCD.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.6', 'bcg.pip.xpath.frGlbSuppCD.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.7', 'bcg.pip.xpath.frGlbSuppCD.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.8', 'bcg.pip.xpath.frGlbSuppCD.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbSuppCD.9', 'bcg.pip.xpath.frGlbSuppCD.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbSuppCD.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.0', 'bcg.pip.xpath.frDuns.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.1', 'bcg.pip.xpath.frDuns.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.2', 'bcg.pip.xpath.frDuns.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.3', 'bcg.pip.xpath.frDuns.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.4', 'bcg.pip.xpath.frDuns.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.5', 'bcg.pip.xpath.frDuns.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.6', 'bcg.pip.xpath.frDuns.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.7', 'bcg.pip.xpath.frDuns.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.8', 'bcg.pip.xpath.frDuns.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frDuns.9', 'bcg.pip.xpath.frDuns.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frDuns.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.0', 'bcg.pip.xpath.frGlbClassCD.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.1', 'bcg.pip.xpath.frGlbClassCD.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.2', 'bcg.pip.xpath.frGlbClassCD.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.3', 'bcg.pip.xpath.frGlbClassCD.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.4', 'bcg.pip.xpath.frGlbClassCD.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.5', 'bcg.pip.xpath.frGlbClassCD.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.6', 'bcg.pip.xpath.frGlbClassCD.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.7', 'bcg.pip.xpath.frGlbClassCD.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.8', 'bcg.pip.xpath.frGlbClassCD.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.frGlbClassCD.9', 'bcg.pip.xpath.frGlbClassCD.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.frGlbClassCD.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.0', 'bcg.pip.xpath.docId.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.1', 'bcg.pip.xpath.docId.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.2', 'bcg.pip.xpath.docId.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.3', 'bcg.pip.xpath.docId.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.4', 'bcg.pip.xpath.docId.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.5', 'bcg.pip.xpath.docId.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.6', 'bcg.pip.xpath.docId.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.7', 'bcg.pip.xpath.docId.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.8', 'bcg.pip.xpath.docId.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.docId.9', 'bcg.pip.xpath.docId.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.docId.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.0', 'bcg.pip.xpath.toDuns.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.1', 'bcg.pip.xpath.toDuns.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.2', 'bcg.pip.xpath.toDuns.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.3', 'bcg.pip.xpath.toDuns.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.4', 'bcg.pip.xpath.toDuns.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.5', 'bcg.pip.xpath.toDuns.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.6', 'bcg.pip.xpath.toDuns.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.7', 'bcg.pip.xpath.toDuns.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.8', 'bcg.pip.xpath.toDuns.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toDuns.9', 'bcg.pip.xpath.toDuns.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toDuns.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.0', 'bcg.pip.xpath.dts.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.1', 'bcg.pip.xpath.dts.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.2', 'bcg.pip.xpath.dts.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.3', 'bcg.pip.xpath.dts.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.4', 'bcg.pip.xpath.dts.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.5', 'bcg.pip.xpath.dts.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.6', 'bcg.pip.xpath.dts.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.7', 'bcg.pip.xpath.dts.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.8', 'bcg.pip.xpath.dts.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.dts.9', 'bcg.pip.xpath.dts.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.dts.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.0', 'bcg.pip.xpath.toRole.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.1', 'bcg.pip.xpath.toRole.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.2', 'bcg.pip.xpath.toRole.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.3', 'bcg.pip.xpath.toRole.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.4', 'bcg.pip.xpath.toRole.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.5', 'bcg.pip.xpath.toRole.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.6', 'bcg.pip.xpath.toRole.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.7', 'bcg.pip.xpath.toRole.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.8', 'bcg.pip.xpath.toRole.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toRole.9', 'bcg.pip.xpath.toRole.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toRole.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.0', 'bcg.pip.xpath.glbDocCD.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.1', 'bcg.pip.xpath.glbDocCD.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.2', 'bcg.pip.xpath.glbDocCD.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.3', 'bcg.pip.xpath.glbDocCD.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.4', 'bcg.pip.xpath.glbDocCD.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.5', 'bcg.pip.xpath.glbDocCD.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.6', 'bcg.pip.xpath.glbDocCD.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.7', 'bcg.pip.xpath.glbDocCD.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.8', 'bcg.pip.xpath.glbDocCD.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.glbDocCD.9', 'bcg.pip.xpath.glbDocCD.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.glbDocCD.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.0', 'bcg.pip.xpath.toGlbSuppCD.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.1', 'bcg.pip.xpath.toGlbSuppCD.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.2', 'bcg.pip.xpath.toGlbSuppCD.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.3', 'bcg.pip.xpath.toGlbSuppCD.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.4', 'bcg.pip.xpath.toGlbSuppCD.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.5', 'bcg.pip.xpath.toGlbSuppCD.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.6', 'bcg.pip.xpath.toGlbSuppCD.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.7', 'bcg.pip.xpath.toGlbSuppCD.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.8', 'bcg.pip.xpath.toGlbSuppCD.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbSuppCD.9', 'bcg.pip.xpath.toGlbSuppCD.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbSuppCD.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.0', 'bcg.pip.xpath.reqDocId.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.1', 'bcg.pip.xpath.reqDocId.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.2', 'bcg.pip.xpath.reqDocId.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.3', 'bcg.pip.xpath.reqDocId.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.4', 'bcg.pip.xpath.reqDocId.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.5', 'bcg.pip.xpath.reqDocId.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.6', 'bcg.pip.xpath.reqDocId.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.7', 'bcg.pip.xpath.reqDocId.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.8', 'bcg.pip.xpath.reqDocId.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.reqDocId.9', 'bcg.pip.xpath.reqDocId.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.reqDocId.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.0', 'bcg.pip.xpath.toGlbClassCD.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.1', 'bcg.pip.xpath.toGlbClassCD.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.2', 'bcg.pip.xpath.toGlbClassCD.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.3', 'bcg.pip.xpath.toGlbClassCD.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.4', 'bcg.pip.xpath.toGlbClassCD.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.5', 'bcg.pip.xpath.toGlbClassCD.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.6', 'bcg.pip.xpath.toGlbClassCD.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.7', 'bcg.pip.xpath.toGlbClassCD.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.8', 'bcg.pip.xpath.toGlbClassCD.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.xpath.toGlbClassCD.9', 'bcg.pip.xpath.toGlbClassCD.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.xpath.toGlbClassCD.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.0', 'bcg.pip.0A1.toGlbClassCD.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.1', 'bcg.pip.0A1.toGlbClassCD.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.2', 'bcg.pip.0A1.toGlbClassCD.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.3', 'bcg.pip.0A1.toGlbClassCD.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.4', 'bcg.pip.0A1.toGlbClassCD.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.5', 'bcg.pip.0A1.toGlbClassCD.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.6', 'bcg.pip.0A1.toGlbClassCD.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.7', 'bcg.pip.0A1.toGlbClassCD.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.8', 'bcg.pip.0A1.toGlbClassCD.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.toGlbClassCD.9', 'bcg.pip.0A1.toGlbClassCD.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.toGlbClassCD.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.0', 'bcg.pip.0A1.frGlbClassCD.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.1', 'bcg.pip.0A1.frGlbClassCD.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.2', 'bcg.pip.0A1.frGlbClassCD.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.3', 'bcg.pip.0A1.frGlbClassCD.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.4', 'bcg.pip.0A1.frGlbClassCD.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.5', 'bcg.pip.0A1.frGlbClassCD.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.6', 'bcg.pip.0A1.frGlbClassCD.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.7', 'bcg.pip.0A1.frGlbClassCD.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.8', 'bcg.pip.0A1.frGlbClassCD.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.pip.0A1.frGlbClassCD.9', 'bcg.pip.0A1.frGlbClassCD.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.pip.0A1.frGlbClassCD.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.0', 'bcg.0A1.fromContactName.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.1', 'bcg.0A1.fromContactName.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.2', 'bcg.0A1.fromContactName.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.3', 'bcg.0A1.fromContactName.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.4', 'bcg.0A1.fromContactName.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.5', 'bcg.0A1.fromContactName.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.6', 'bcg.0A1.fromContactName.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.7', 'bcg.0A1.fromContactName.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.8', 'bcg.0A1.fromContactName.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromContactName.9', 'bcg.0A1.fromContactName.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromContactName.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.0', 'bcg.0A1.fromEMailAddr.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.1', 'bcg.0A1.fromEMailAddr.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.2', 'bcg.0A1.fromEMailAddr.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.3', 'bcg.0A1.fromEMailAddr.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.4', 'bcg.0A1.fromEMailAddr.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.5', 'bcg.0A1.fromEMailAddr.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.6', 'bcg.0A1.fromEMailAddr.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.7', 'bcg.0A1.fromEMailAddr.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.8', 'bcg.0A1.fromEMailAddr.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromEMailAddr.9', 'bcg.0A1.fromEMailAddr.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromEMailAddr.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.0', 'bcg.0A1.fromPhoneNbr.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.1', 'bcg.0A1.fromPhoneNbr.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.2', 'bcg.0A1.fromPhoneNbr.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.3', 'bcg.0A1.fromPhoneNbr.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.4', 'bcg.0A1.fromPhoneNbr.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.5', 'bcg.0A1.fromPhoneNbr.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.6', 'bcg.0A1.fromPhoneNbr.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.7', 'bcg.0A1.fromPhoneNbr.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.8', 'bcg.0A1.fromPhoneNbr.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromPhoneNbr.9', 'bcg.0A1.fromPhoneNbr.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromPhoneNbr.9')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.0', 'bcg.0A1.fromFaxNbr.0' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.0')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.1', 'bcg.0A1.fromFaxNbr.1' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.1')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.2', 'bcg.0A1.fromFaxNbr.2' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.2')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.3', 'bcg.0A1.fromFaxNbr.3' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.3')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.4', 'bcg.0A1.fromFaxNbr.4' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.4')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.5', 'bcg.0A1.fromFaxNbr.5' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.5')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.6', 'bcg.0A1.fromFaxNbr.6' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.6')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.7', 'bcg.0A1.fromFaxNbr.7' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.7')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.8', 'bcg.0A1.fromFaxNbr.8' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.8')
/
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY)
    select CF_RNPIPMetaDef_Seq.nextval,'bcg.0A1.fromFaxNbr.9', 'bcg.0A1.fromFaxNbr.9' from Dual
    Where Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'bcg.0A1.fromFaxNbr.9')
/


UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'DTDName'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'FrService'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'ToService'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'FrRole'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'ToRole'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'RootTag'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'ResponseFrActionName'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.frRole.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbSuppCD.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.frDuns.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.frGlbClassCD.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	metadefkey = 'bcg.pip.xpath.docId.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DUNS')
WHERE	metadefkey = 'bcg.pip.xpath.toDuns.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.dts.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.ROLE')
WHERE	metadefkey = 'bcg.pip.xpath.toRole.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBDOCCD')
WHERE	metadefkey = 'bcg.pip.xpath.glbDocCD.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBSUPPCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbSuppCD.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.DOCID')
WHERE	metadefkey = 'bcg.pip.xpath.reqDocId.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GLBCLASSCD')
WHERE	metadefkey = 'bcg.pip.xpath.toGlbClassCD.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.toGlbClassCD.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.0A1')
WHERE	metadefkey = 'bcg.pip.0A1.frGlbClassCD.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromContactName.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromEMailAddr.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromPhoneNbr.9'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.0'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.1'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.2'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.3'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.4'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.5'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.6'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.7'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.8'
/
UPDATE cf_rnpipmetadef
SET	rnpipattr_grpid = (SELECT rnpipattr_grpid 
				FROM cf_rnpipattr_grp 
				WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.CONTACTINFO')
WHERE	metadefkey = 'bcg.0A1.fromFaxNbr.9'
/
-- 102237
Insert Into CF_RNPIPMetaDef (METADEFID, METADEFKEY, RESOURCEKEY,rnpipattr_grpid)
    select CF_RNPIPMetaDef_Seq.nextval,'GlobalAdministeringAuthorityCode','bcg.ro.rnif.GlobalAdminAuthCd',
    rnpipattr_grpid from cf_rnpipattr_grp 
    WHERE rnpipattr_grpkey = 'bcg.grp.ro.rnif.GENERAL'
    and Not Exists (select 'x' from CF_RNPIPMetaDef where METADEFKEY =  'GlobalAdministeringAuthorityCode')
/
-- 102237

-- cf_RNPipMetaDef_Loc_ENUS.sql

-- CF_RNPIPMetaDef_Loc 

Insert into CF_RNPIPMetaDef_Loc(METADEFID ,LOCALE, METADEFNAME) 
    select METADEFID, 'en_US', 'DTD Name' from CF_RNPIPMetaDef where METADEFKEY = 'DTDName'
    AND NOT Exists(select 'x' from  CF_RNPIPMetaDef_Loc Where METADEFNAME =  'DTD Name' And   LOCALE = 'en_US')
/

Insert into CF_RNPIPMetaDef_Loc(METADEFID ,LOCALE, METADEFNAME) 
    select METADEFID, 'en_US', 'From Service' from CF_RNPIPMetaDef where METADEFKEY = 'FrService'
    AND NOT Exists(select 'x' from  CF_RNPIPMetaDef_Loc Where METADEFNAME =  'From Service' And  LOCALE = 'en_US')
/

Insert into CF_RNPIPMetaDef_Loc(METADEFID ,LOCALE, METADEFNAME) 
    select METADEFID, 'en_US', 'To Service' from CF_RNPIPMetaDef where METADEFKEY = 'ToService'
    AND NOT Exists(select 'x' from  CF_RNPIPMetaDef_Loc Where METADEFNAME =  'To Service' And  LOCALE = 'en_US')
/

Insert into CF_RNPIPMetaDef_Loc(METADEFID ,LOCALE, METADEFNAME) 
    select METADEFID, 'en_US', 'From Role' from CF_RNPIPMetaDef where METADEFKEY = 'FrRole'
    AND NOT Exists(select 'x' from  CF_RNPIPMetaDef_Loc Where METADEFNAME =  'From Role' And  LOCALE = 'en_US')
/

Insert into CF_RNPIPMetaDef_Loc(METADEFID ,LOCALE, METADEFNAME) 
    select METADEFID, 'en_US', 'To Role' from CF_RNPIPMetaDef where METADEFKEY = 'ToRole'
    AND NOT Exists(select 'x' from  CF_RNPIPMetaDef_Loc Where METADEFNAME =  'To Role' And  LOCALE = 'en_US')
/

Insert into CF_RNPIPMetaDef_Loc(METADEFID ,LOCALE, METADEFNAME) 
    select METADEFID, 'en_US', 'Root Tag' from CF_RNPIPMetaDef where METADEFKEY = 'RootTag'
    AND NOT Exists(select 'x' from  CF_RNPIPMetaDef_Loc Where METADEFNAME =  'Root Tag' And  LOCALE = 'en_US')
/

Insert into CF_RNPIPMetaDef_Loc(METADEFID ,LOCALE, METADEFNAME) 
    select METADEFID, 'en_US', 'Response From Action Name' from CF_RNPIPMetaDef where METADEFKEY = 'ResponseFrActionName'
    AND NOT Exists(select 'x' from  CF_RNPIPMetaDef_Loc Where METADEFNAME =  'Response From Action Name' And  LOCALE = 'en_US')
/


-- cf_ProtocolDefType.sql

-- CF_ProtocolDefType
Insert Into CF_ProtocolDefType (METHODTYPEID,METHODTYPEKEY, RESOURCEKEY )
       Select CF_ProtocolDefType_Seq.Nextval,'DTD','DTD' from Dual
       Where Not Exists (select 'x' from CF_ProtocolDefType   where METHODTYPEKEY = 'DTD')
/ 
--
Insert Into CF_ProtocolDefType (METHODTYPEID,METHODTYPEKEY, RESOURCEKEY ) 
       Select CF_ProtocolDefType_Seq.Nextval,'NameSpace','NameSpace' from Dual
       Where Not Exists (select 'x' from CF_ProtocolDefType   where METHODTYPEKEY = 'NameSpace')
/ 
--
Insert Into CF_ProtocolDefType (METHODTYPEID,METHODTYPEKEY, RESOURCEKEY ) 
       Select CF_ProtocolDefType_Seq.Nextval,'RootTag','RootTag' from Dual
       Where Not Exists (select 'x' from CF_ProtocolDefType   where METHODTYPEKEY = 'RootTag')
/ 



   


-- cf_ProtocolDefType_Loc_ENUS.sql

-- CF_ProtocolDefType_Loc
Insert into CF_ProtocolDefType_Loc(METHODTYPEID ,LOCALE,METHODTYPENAME) 
select METHODTYPEID, 'en_US', 'DTD' from CF_ProtocolDefType where METHODTYPEKEY = 'DTD'
and NOT Exists (select 'x' from  CF_ProtocolDefType_Loc  Where METHODTYPENAME = 'DTD' And LOCALE = 'en_US')
/      
--            
Insert into CF_ProtocolDefType_Loc(METHODTYPEID ,LOCALE,METHODTYPENAME) 
select METHODTYPEID, 'en_US', 'Name Space' from CF_ProtocolDefType where METHODTYPEKEY = 'NameSpace'
and NOT Exists (select 'x' from  CF_ProtocolDefType_Loc  Where METHODTYPENAME = 'Name Space' And LOCALE = 'en_US')
/      
--           
Insert into CF_ProtocolDefType_Loc(METHODTYPEID ,LOCALE,METHODTYPENAME) 
select METHODTYPEID, 'en_US', 'Root Tag' from CF_ProtocolDefType where METHODTYPEKEY = 'RootTag'
and NOT Exists (select 'x' from  CF_ProtocolDefType_Loc  Where METHODTYPENAME = 'Root Tag' And LOCALE = 'en_US')
/      
--     

-- cf_ProtocolAttrValueType.sql

-- CF_ProtocolAttrValueType

--100499
--Insert Into CF_ProtocolAttrValueType ( ATTRVALUETYPEID, ATTRVALUETYPEKEY, RESOURCEKEY )
--       Select CF_ProtocolAttrValueType_Seq.Nextval
--            , 'XPath'  , 'XPath' 
--         From Dual
--        Where Not Exists (select 'x' from CF_ProtocolAttrValueType where ATTRVALUETYPEKEY = 'XPath')/  

Insert Into CF_ProtocolAttrValueType ( ATTRVALUETYPEID, ATTRVALUETYPEKEY, RESOURCEKEY ) 
       Select CF_ProtocolAttrValueType_Seq.Nextval
            , 'Constant' , 'Constant' 
         From Dual
        Where Not Exists (select 'x' from CF_ProtocolAttrValueType where ATTRVALUETYPEKEY= 'Constant')
/        

--71468
Insert Into CF_ProtocolAttrValueType ( ATTRVALUETYPEID,ATTRVALUETYPEKEY, RESOURCEKEY  )
       Select CF_ProtocolAttrValueType_Seq.Nextval,'Text' , 'Text'
         From dual
        Where Not Exists (select 'x' from CF_ProtocolAttrValueType where ATTRVALUETYPEKEY = 'Text')
/  

Insert Into CF_ProtocolAttrValueType ( ATTRVALUETYPEID,ATTRVALUETYPEKEY, RESOURCEKEY  )
       Select CF_ProtocolAttrValueType_Seq.Nextval,'TagName' , 'TagName'
         From dual
        Where Not Exists (select 'x' from CF_ProtocolAttrValueType where ATTRVALUETYPEKEY = 'TagName')
/

--101792
Insert Into CF_ProtocolAttrValueType ( ATTRVALUETYPEID,ATTRVALUETYPEKEY, RESOURCEKEY  )
       Select CF_ProtocolAttrValueType_Seq.Nextval,'SubTree' , 'SubTree'
         From dual
        Where Not Exists (select 'x' from CF_ProtocolAttrValueType where ATTRVALUETYPEKEY = 'SubTree')
/


-- cf_ProtocolAttrValueType_Loc_ENUS.sql

-- Insert into LOC tables 

Insert Into CF_ProtocolAttrValueType_Loc ( ATTRVALUETYPEID, LOCALE, ATTRVALUETYPENAME ) 
       Select ATTRVALUETYPEID
            , 'en_US'
            , 'Element Path' 
         From CF_ProtocolAttrValueType 
        Where ATTRVALUETYPEKEY = 'XPath'
          And NOT Exists (select 'x' from  CF_ProtocolAttrValueType_Loc 
                           Where ATTRVALUETYPENAME = 'Element Path' And LOCALE = 'en_US')
/                  
--
Insert Into CF_ProtocolAttrValueType_Loc( ATTRVALUETYPEID, LOCALE, ATTRVALUETYPENAME ) 
       Select ATTRVALUETYPEID
            , 'en_US'
            , 'Constant' 
         From CF_ProtocolAttrValueType 
        Where  ATTRVALUETYPEKEY = 'Constant'
          And NOT Exists (select 'x' from  CF_ProtocolAttrValueType_Loc 
                           Where ATTRVALUETYPENAME = 'Constant' And LOCALE = 'en_US')
/                  

--Update

UPDATE CF_PROTOCOLATTRVALUETYPE_LOC 
   SET ATTRVALUETYPENAME = 'Element Path' 
 WHERE ATTRVALUETYPEID = (SELECT d.ATTRVALUETYPEID 
                            FROM CF_PROTOCOLATTRVALUETYPE_LOC t
                               , CF_PROTOCOLATTRVALUETYPE d 
                           WHERE d.ATTRVALUETYPEID = t.ATTRVALUETYPEID 
                             AND d.ATTRVALUETYPEKEY = 'XPath'
                             AND t.locale = 'en_US')
/

UPDATE CF_PROTOCOLATTRVALUETYPE_LOC 
   SET ATTRVALUETYPENAME = 'Constant' 
 WHERE ATTRVALUETYPEID = (SELECT d.ATTRVALUETYPEID 
                            FROM CF_PROTOCOLATTRVALUETYPE_LOC t
                               , CF_PROTOCOLATTRVALUETYPE d 
                           WHERE d.ATTRVALUETYPEID = t.ATTRVALUETYPEID 
                             AND d.ATTRVALUETYPEKEY = 'Constant'
                             AND t.locale = 'en_US')
/





-- cf_ProtocolAttrType.sql

-- CF_ProtocolAttrType
Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'FrBusinessId',1,0,1,'FrBusinessId' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'FrBusinessId')
/ 
     
     
Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY) 
       Select CF_ProtocolAttrType_Seq.Nextval,'ToBusinessId',1,0,2,'ToBusinessId' from Dual
        Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'ToBusinessId')
/  

Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'FrProcessCd',1,0,3,'FrProcessCd' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'FrProcessCd')
/  

Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'FrProcessVer',1,0,4,'FrProcessVer' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'FrProcessVer')
/ 

Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY) 
       Select CF_ProtocolAttrType_Seq.Nextval,'DocId',0,1,5,'DocId' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'DocId')
/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'RefDocId',0,0,6,'RefDocId' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'RefDocId')
/
       
Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'DocTS',0,1,7,'DocTS' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'DocTS')
/ 

Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'DupCheckKey1',0,0,8,'DupCheckKey1' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'DupCheckKey1')
/ 

Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY) 
       Select CF_ProtocolAttrType_Seq.Nextval,'DupCheckKey2',0,0,9,'DupCheckKey2' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'DupCheckKey2')
/ 

Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY) 
       Select CF_ProtocolAttrType_Seq.Nextval,'DupCheckKey3',0,0,10,'DupCheckKey3' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'DupCheckKey3')
/ 

Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY) 
       Select CF_ProtocolAttrType_Seq.Nextval,'DupCheckKey4',0,0,11,'DupCheckKey4' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'DupCheckKey4')
/ 

Insert Into CF_ProtocolAttrType (ATTRTYPEID,ATTRTYPEKEY,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY) 
       Select CF_ProtocolAttrType_Seq.Nextval,'DupCheckKey5',0,0,12,'DupCheckKey5' from Dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'DupCheckKey5')
/ 

--71468
Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SyncFlag',0,0,13,'SyncFlag' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where attrtypekey = 'SyncFlag')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'ValidationRoot',0,0,14,'ValidationRoot' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'ValidationRoot')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget0',0,0,15,'SearchTarget0' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget0')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget1',0,0,16,'SearchTarget1' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget1')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget2',0,0,17,'SearchTarget2' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget2')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget3',0,0,18,'SearchTarget3' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget3')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget4',0,0,19,'SearchTarget4' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget4')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget5',0,0,20,'SearchTarget5' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget5')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget6',0,0,21,'SearchTarget6' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget6')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget7',0,0,22,'SearchTarget7' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget7')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget8',0,0,23,'SearchTarget8' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget8')

/

Insert Into CF_ProtocolAttrType (ATTRTYPEID, attrtypekey,REQUIREDFLG,CONSTANTFLG,SEQNBR, RESOURCEKEY)
       Select CF_ProtocolAttrType_Seq.Nextval,'SearchTarget9',0,0,24,'SearchTarget9' from dual
       Where Not Exists (select 'x' from CF_ProtocolAttrType  where ATTRTYPEKEY = 'SearchTarget9')

/

UPDATE cf_protocolattrtype
SET providerflag = 1

/

UPDATE cf_protocolattrtype
SET seqnbr = 1
WHERE attrtypekey = 'FrProcessCd'

/

UPDATE cf_protocolattrtype
SET seqnbr = 2
WHERE attrtypekey = 'FrProcessVer'

/

UPDATE cf_protocolattrtype
SET seqnbr = 3
WHERE attrtypekey = 'FrBusinessId'

/

UPDATE cf_protocolattrtype
SET seqnbr = 4
WHERE attrtypekey = 'ToBusinessId'

/

--100906 - change order of refdocid to be after doc id
--UPDATE cf_protocolattrtype
--SET seqnbr = 7
--WHERE attrtypekey = 'RefDocId'
--
/

--UPDATE cf_protocolattrtype
--SET seqnbr = 8
--WHERE attrtypekey = 'DupCheckKey1'
--
/

--UPDATE cf_protocolattrtype
--SET seqnbr = 9
--WHERE attrtypekey = 'DupCheckKey2'
--
/

--UPDATE cf_protocolattrtype
--SET seqnbr = 10
--WHERE attrtypekey = 'DupCheckKey3'
--
/

--UPDATE cf_protocolattrtype
--SET seqnbr = 11
--WHERE attrtypekey = 'DupCheckKey4'
--
/

--UPDATE cf_protocolattrtype
--SET seqnbr = 12
--WHERE attrtypekey = 'DupCheckKey5'
--
/

--UPDATE cf_protocolattrtype
--SET seqnbr = 13
--WHERE attrtypekey = 'SyncFlag'
--
/

--UPDATE cf_protocolattrtype
--SET seqnbr = 14
--WHERE attrtypekey = 'ValidationRoot'
--
/


-- cf_ProtocolAttrType_Loc_ENUS.sql

--Insert into  CF_PROTOCOLATTRTYPE_LOC
Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Source BusinessId' from CF_ProtocolAttrType where ATTRTYPEKEY = 'FrBusinessId'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Source BusinessId' And   LOCALE = 'en_US')
/
          
Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Target BusinessId' from CF_ProtocolAttrType where ATTRTYPEKEY = 'ToBusinessId'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Target BusinessId' And   LOCALE = 'en_US')
/
          
Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Source Document Flow' from CF_ProtocolAttrType where ATTRTYPEKEY = 'FrProcessCd'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Source Document Flow' And   LOCALE = 'en_US')
/
          
Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Source Document Flow Version' from CF_ProtocolAttrType where ATTRTYPEKEY = 'FrProcessVer'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Source Document Flow Version' And   LOCALE = 'en_US')
/

Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Document Identifier' from CF_ProtocolAttrType where ATTRTYPEKEY = 'DocId'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Document Identifier' And   LOCALE = 'en_US')
/
          
Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Document Timestamp' from CF_ProtocolAttrType where ATTRTYPEKEY = 'DocTS'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Document Timestamp' And   LOCALE = 'en_US')
/
         
Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Duplicate Check Key 1' from CF_ProtocolAttrType where ATTRTYPEKEY = 'DupCheckKey1'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Duplicate Check Key 1' And   LOCALE = 'en_US')
/
          
Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Duplicate Check Key 2' from CF_ProtocolAttrType where ATTRTYPEKEY = 'DupCheckKey2'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Duplicate Check Key 2' And   LOCALE = 'en_US')
/
          
Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Duplicate Check Key 3' from CF_ProtocolAttrType where ATTRTYPEKEY = 'DupCheckKey3'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Duplicate Check Key 3' And   LOCALE = 'en_US')
/
         
Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Duplicate Check Key 4' from CF_ProtocolAttrType where ATTRTYPEKEY = 'DupCheckKey4'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Duplicate Check Key 4' And   LOCALE = 'en_US')
/

Insert into CF_ProtocolAttrType_Loc(ATTRTYPEID ,LOCALE,ATTRTYPENAME) 
select ATTRTYPEID, 'en_US', 'Duplicate Check Key 5' from CF_ProtocolAttrType where ATTRTYPEKEY = 'DupCheckKey5'
AND NOT Exists (select 'x' from  CF_ProtocolAttrType_Loc Where ATTRTYPENAME = 'Duplicate Check Key 5' And   LOCALE = 'en_US')
/                               

-- cf_BusinessLogo.sql

Create or Replace procedure metadata_insert as                                                                                                                             
BRANDINGLogoIMAGE_1 raw (32676) :=                                                                                                  
'FFD8FFE000104A46494600010201004800480000FFED0B0C50686F746F73686F7020332E30003842494D03ED0A5265736F6C7574696F6E0000000010'          
||'004800000001000100480000000100013842494D040D18465820476C6F62616C204C69676874696E6720416E676C6500000000040000001E3842494D'        
||'041912465820476C6F62616C20416C74697475646500000000040000001E3842494D03F30B5072696E7420466C616773000000090000000000000000'        
||'01003842494D040A0E436F7079726967687420466C6167000000000100003842494D2710144A6170616E657365205072696E7420466C616773000000'        
||'000A000100000000000000023842494D03F517436F6C6F722048616C66746F6E652053657474696E677300000048002F66660001006C666600060000'        
||'00000001002F6666000100A1999A0006000000000001003200000001005A00000006000000000001003500000001002D000000060000000000013842'        
||'494D03F817436F6C6F72205472616E736665722053657474696E6773000000700000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF03E80000'        
||'0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF03E800000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF03E800000000FFFF'        
||'FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF03E800003842494D04000B4C617965722053746174650000000200013842494D04020C4C61796572'        
||'2047726F7570730000000004000000003842494D0408064775696465730000000010000000010000024000000240000000003842494D041E0D55524C'        
||'206F766572726964657300000004000000003842494D041A06536C696365730000000071000000060000000000000000000000500000007000000008'        
||'00690062006D005F006C006F0067006F0000000100000000000000000000000000000000000000010000000000000000000000700000005000000000'        
||'0000000000000000000000000000000000000000000000000000000000003842494D04111149434320556E74616767656420466C6167000000010100'        
||'3842494D0414174C617965722049442047656E657261746F72204261736500000004000000023842494D040C154E65772057696E646F777320546875'        
||'6D626E61696C0000073400000001000000700000005000000150000069000000071800180001FFD8FFE000104A46494600010201004800480000FFEE'        
||'000E41646F626500648000000001FFDB0084000C08080809080C09090C110B0A0B11150F0C0C0F1518131315131318110C0C0C0C0C0C110C0C0C0C0C'        
||'0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C010D0B0B0D0E0D100E0E10140E0E0E14140E0E0E0E14110C0C0C0C0C11110C0C0C0C0C0C11'        
||'0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0CFFC00011080050007003012200021101031101FFDD00040007FFC4013F000001'        
||'0501010101010100000000000000030001020405060708090A0B0100010501010101010100000000000000010002030405060708090A0B1000010401'        
||'030204020507060805030C33010002110304211231054151611322718132061491A1B14223241552C16233347282D14307259253F0E1F163733516A2'        
||'B283264493546445C2A3743617D255E265F2B384C3D375E3F3462794A485B495C4D4E4F4A5B5C5D5E5F55666768696A6B6C6D6E6F637475767778797'        
||'A7B7C7D7E7F711000202010204040304050607070605350100021103213112044151617122130532819114A1B14223C152D1F0332462E17282924353'        
||'15637334F1250616A2B283072635C2D2449354A317644555367465E2F2B384C3D375E3F34694A485B495C4D4E4F4A5B5C5D5E5F55666768696A6B6C6'        
||'D6E6F62737475767778797A7B7C7FFDA000C03010002110311003F00F2A4924925292492494A4924925292492494A4924925292492494A49249253FF'        
||'D0F2A49249252924974985D1FA55B894DB6E36EB1EC05CEFDB5D3A89247FDC5C8C77DF8FFF00156BBD4624A79B497459DD23A5D34B5D5E36C71B18D2'        
||'7F6CF4FBF4739AD70F4B1E8658DF6FF87FE6B1FF009EB7F46AC7EC2E8FFF00713FF87FD2FF00F799253CAA4BA4CBE8184FA0B70A9AE9BF4DB65DD6FA'        
||'6DAC027DD3556CC573BDBFF0C83D27A574DC9C165B7E3FA9612E977ED6C1C3E0C7F43CEA6CC8AFFB6FFD224A70525ADD73A6E36232AB316B6D4C712D'        
||'78FDA3899EE27969DBD3994BE9669FE1188DD1FA4E1E4627AD954B6E2E71D84755C2C2200F6C3F1B3ABBEFFA5F9FFA3494E1A4B67ADF4EC0C4C763F1'        
||'A9F49CE7ED27F6961E7C883FE03A75555B57FC6BFF0046B1925292492494FF00FFD1F2A4924925296C7D57A6FBBA858DA6ABAE70A492DA302AEA6E03'        
||'757EE38798E6575B3FEECFD3FF0005FE1963ADAE9787D2AA0DC8CACEE9B7FA8CD7172DBD446C71877B9DD3B1EAFD2B2367B726DA525357AFD7657D5A'        
||'F65B5D953C6C965B8B5E0BC4B19F4BA7E317538FFD8FE7BF9FFF0008B6FA274FCACBE8A1ADC6CDB2AB7D461B31BA263E60825CD77A5D52DB6ACADFFD'        
||'5FE8FF00E0965751C1E9AEB0DF8FD47A7B1AF731A31B19B9FB580ED63AC0ECFC57D9B1BFCF5BFAC58FFF0043FE895CAB17A25786EC77E6747B6D21C0'        
||'65BC7561682E9DAF6B6AA2BC5DD57F83DD8DFF001BEA24A7530FA164F4FA9ECAB13AA9613BDCEC8FABD8F791A7FA4CCCABDEC669F4772C1FABE3232F'        
||'AB58F652FB6DB1AF7B998BD3E9CE89734BB674DBBD1C5A6BD7E9D7FCCFF375AB5838BD131AB732ECCE8F9CE719165E3AB35C047D06FD8E8C466DFEB3'        
||'1070B1BA40C9B726ECBE96EADCFB1ACC3BC752F4DA377E8ECA9D8940C8F4F60FD07AF95EAFA5FD27F4C929D7CEE9B999D8D938B4E267DB7D501F5D7F'        
||'57F1719CDB21B7575DD938577DA71B7B1CC7FD1FE6BFC1A2D183938E2AC118B9FEB0AF732A7FD5BC3B2D2C66D63ECDF7DEEC8B9AC758C6BEE7FEFAC3'        
||'A5FD20756C863EBE96EC6735A6BB1FFB4BECCD21ADDCDC7F49DFB4F7BDCEFD27DAABD9BFF9AFD1EC4AA7F487757B43EBE975E2B6ADAC2FFDA5F6573A'        
||'5A7D4ABD373FAA7AFAEDFD36CC6FA7FF0006929A3D6D9657D53219631F5BC112CB719984F1ED6FD3C0C72EA71FFAACFF008C54568F57C6C163CDF899'        
||'78768B1D03170C65C3047D2DDD528ADFB3FF00422D7ADCCFFA97562E13981EE39B536F73AFF503AA7BB13EDDF6CA7ECACC76D984CFF2565FD8B26FCC'        
||'B3F687A75FA7895FF943F63A53C924924929FFD2F2A4924925296951F597EB1E3D2CA31FAAE6D34D6036BAEBC8B5AD6B47D16B18D7ED6B566A4929D1'        
||'BFEB27D62C860AF23AA665CC0E6BC36CC8B5C039877D6FDAE79F7B1EDDEC44FF009D9F5ABFF2E73FFF0062AEFF00D28B292494E95DF59FEB25F53E9B'        
||'BAB66DB55AD2CB2B7E45AE6B9AE1B5EC7B1D66D731CD50C4FAC1D7B0686E3E1752CBC6A1B25B5537D95B012773A2BADED6FB9CA824929B47AAF53767'        
||'7ED239979CFE7ED66C7FAD21BE90FD3EEF57F9BFD1FD3FA0963755EA78993665E2665F8F93748B6FAAC7B2C7071DEFF52D639AF7EF7B77B955492537'        
||'337AD759EA35B6AEA19F93995B0EE6B2FB9F6B43A36EE6B6D73BDC9EDEB3D46DC2184FB1A6ADAD639C2BAC5AFAD91E9517E635832EFC7ABD3ABD2C7B'        
||'AFB29ABD0C7D95FEAF47A7492494A49249253FFFD3F2A4924925292492494A4924925292492494A4924925292492494A49249253FFD93842494D0421'        
||'1A56657273696F6E20636F6D7061746962696C69747920696E666F000000005500000001010000000F00410064006F00620065002000500068006F00'        
||'74006F00730068006F00700000001300410064006F00620065002000500068006F0074006F00730068006F007000200036002E003000000001003842'        
||'494D04060C4A504547205175616C69747900000000070008000000010100FFEE000E41646F626500644000000001FFDB008400010101010101010101'        
||'010101010101010101010101010101010101010101010101010101010101010101010202020202020202020202030303030303030303030101010101'        
||'010101010101020201020203030303030303030303030303030303030303030303030303030303030303030303030303030303030303030303030303'        
||'FFC00011080050007003011100021101031101FFDD0004000EFFC401A20000000602030100000000000000000000070806050409030A0201000B0100'        
||'000603010101000000000000000000060504030702080109000A0B100002010304010303020303030206097501020304110512062107132200083114'        
||'4132231509514216612433175271811862912543A1B1F02634720A19C1D13527E1533682F192A24454734546374763285556571AB2C2D2E2F2648374'        
||'938465A3B3C3D3E3293866F3752A393A48494A58595A6768696A767778797A85868788898A9495969798999AA4A5A6A7A8A9AAB4B5B6B7B8B9BAC4C5'        
||'C6C7C8C9CAD4D5D6D7D8D9DAE4E5E6E7E8E9EAF4F5F6F7F8F9FA110002010302040403050404040606056D0102031104211205310600221341510732'        
||'61147108428123911552A162163309B124C1D14372F017E18234259253186344F1A2B226351954364564270A7383934674C2D2E2F255657556378485'        
||'A3B3C3D3E3F3291A94A4B4C4D4E4F495A5B5C5D5E5F52847576638768696A6B6C6D6E6F667778797A7B7C7D7E7F7485868788898A8B8C8D8E8F83949'        
||'5969798999A9B9C9D9E9F92A3A4A5A6A7A8A9AAABACADAEAFAFFDA000C03010002110311003F00F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7B'        
||'F75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAFFD0F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEF'        
||'DD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAFFD1F9FF00FBF75EEBDEFDD7BAF7BF'        
||'75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBD6F7EEBDD7AC7FA1E3EBFE1EFDD7BAF7BF75EEBD6F7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75FFFD2F9'        
||'FF00FBF75EEBDEFDD7BAEC7FC48E7FA7D7F1F9F7EEBDD5D774AFC3EF8B7BC3A8FAE774EE9E91FE37B8F3FB3F0795CE663FE8227FE4E5F1DBF8964EB6'        
||'8229EAEB07457717C7EDC1DA9D482A2572DFC0370D755E5B1BFE66A65791589F75EEA07767C45F8C3B376862F2BB5BA57F81E46AB7FF005C60EA2B3F'        
||'E8212FE501F233CD87DC1BDB0786CDD07F70BA6FA0F6EEEBC37F12C556CB07F7A2A677C16D1F27F17CB433E3A8EA627F75EE85EFF6467E1E7FDE3EFF'        
||'00EDD1BFC8A3FF00B99BDFBAF741C76BFC07E9ACC6C9C95074675AF5BF5DF63CB3E39B0FBAFB63FE1473FC93FB9F64D0D2475D0C9968725D7FB43657'        
||'42E772B515B8D5922A7922DCD46B4B3BACCF1D42A985FDD7BA0E3E287C5AF8E7D9DD25B6F78F61F50FF7B775642BF71415B9C1FCF5BF956FC1FF00BA'        
||'8687375D454518F8E9F28BA477AF72ED1FB7A38523FBAC86425832D6FBCA558E9A68917DD7BA07FE6F7C6EEB5E9EC56C7DCFD4DB2F6FEC4C06432191'        
||'C0EE1C749FCD7BF97CFF0031CDD595CC4D4EB90C3D6D1E07E1A6D0EB8DC7D7180A1A2C7D5A54D6663195F4355513411A55534BA61A8F75EE840F877F'        
||'14BA77B2BAA64DEFDBDD77B7F7ED5E6B7064976CD562BF9D87F2BEF80B97C4E0F165317518EDCFD1FF002B7AF7B47B5FF88BE6692A27832937F09A3A'        
||'DA19616829A48ED553FBAF74C1F367E3BF4574EEC0DAB9DEACEB2FEE3E6329BBD71390C81FE707FCB9FF0098C7DE637F82652B0D17FA2EF86FD49D7D'        
||'BEB63DEAA9637FE3F94AA9B12BA7ED0C5F71530B2FBAF7559DEFDD7BAF7BF75EEBDEFDD7BAFFD3F9FF00FBF75EEBDEFDD7BAF7BF75EEAC83F95FED1D'        
||'E9BD7BF777E2F62EC0EE0EC7CBC1D419FC854E13A57F95FF00417F365DD74B8E877A6C0A7972B90F8E9F22F3FB7364ECCC04153550C32EF6A59DB2B8'        
||'EA9A883191A1A7CC54B2FBAF740FFCF5C06E2DAFF2C3B5B05BB36A7606C9CFD11D8DF7FB63B47E15755FF2EFDF98CFB9EB6D9D5949FC73E1D74A6573'        
||'7D65D3FF007945531D452FF0CAA93FBC3473439BA8D357929D17DD7BAB47F849D03DA5DBFF000CB1F89C4F477CD6DDDB2F7AD3F666D6AADE1F1F3FE1'        
||'38DF0E7E7450B51657339EC2E5E4D91F3C77CF6AF5FF007ECFBA31CB3CA91D4515463B21B47211FDBE36A626A38265F75EE8C3F4EFC14EC7F8E1B573'        
||'F88DA7F1EFF9AEE43035B90A8DCF98CAF7C7FC2503E1AFC8ECAE39A0C753D3D4B51EF3F929F29FB4B716DAC053D163D657A1A3ADA5C7249E49FC2259'        
||'2591FDD7BAA9AFE5FD0F61772FCB3DE99AC0F5E6F8DEDBBF74ED7DFF00BC731B57E37FF2AEF8C7FCC4E5A19731B9B0991CA4DB7FE16764BF5DF46F5E'        
||'6C9A0ADC8F8A1C96129A862DB313C18FC7D2454952D1C7EEBDD59EF78FC71EE3EFEEB6EE7EA6D8DF1E7E7BEF2EC2D8B3E2B1B9ED95B0BFE12B9F043E'        
||'2E6E4DB7D8670380EC7D97B3BB13BB7E3176FD6777F4AD3EE6DBF9CC56467F050D55455E072314CD415B4754B1CBEEBDD081B17A3BB2BAEA3D81D090'        
||'F457CF71BE687AE27CEEDFD8FBC7FE1217FCB57B4BB5F33B2364D56D9DB7B9777CFB8BB3FBCB33DB9D9588DBB9DDD78AA2C96E5CAC95550D5993A415'        
||'932D455C6AFEEBDD5017CD9C2E7B6DFCA4EDDC2EE7DB3BE367E768331878F23B6BB23E22F5B7C0CDED8B924DA9819E28B3FF00123A832597EB8E99A8'        
||'969E54963A2C5D4CB15740E9909089AAE551EEBDD157B7FBEFF8DFD3DFBAF75D7BF75EEBDEFDD7BAFFD4F9FF00FBF75EEBDEFDD7BAF7BF75EEACE3E2'        
||'F7507C57DAB1E33B17B5FE537F2C8ECA6DD1B1D21ABE86F941B7BF9CB50C9D719EC9D562326D5794CC7C2EF8E5B061AEDF1B762A09B1CC28B7A67B6D'        
||'BC55B50CA95522D2D541EEBDD25FE447477C6CCAEE4C9760F5AFCC7FE5E1B67199FDC1B230D4BD1BF1E36C7F362A8DB7B271B5AF81DAD9EDD98DCBFC'        
||'B1F8AD9FDDF57B7F1016A3726723ACDE197CAB06AA8B0F4531FB2C58F75EE8C3ED4EAEF847B77A7F29D6798F921FC9A77EEF4AFC3EEAC6527C85DD78'        
||'3FF8509D1F6FE12B770A57A62770E3309B1BA1766740CD98D8AD59136292B763D5E3E634917F12A5C883389BDD7BAEFA4BABBE127556DFCAE177DFC8'        
||'FF00E4D3F2632391CC1C9D1EECEECC1FFC284B039FC0D11A2A5A51B77134DF1C7A1BE3FED79B0CB3D3BD4EBAEC6D6E43CF3B8FB93088E28FDD7BA0E3'        
||'A6FAD3E2553765EFFECFDEDF213F958E4B6BE477776861B6EFC74EE4C2FF003B44EB5DBF868F7BD645B5376EC1CDFC7DE8FA0EE29F67CD80C7A4BB5E'        
||'3DD1BEAB73E3075B18DC746B9859520F75EEBAD9998F89549F2BBB83059CDA5FCAC725D4598DBFB4EB365EF2DE753FCED23F8A3B5B2388DB3B7D72F8'        
||'9E9FA5EBEDCB41F38EA3706EACA64EA1B2CFBFB1591C54793C7557F0C9A8F1ED422A7DD7BAF6D7CCFC4ACBFCB8DF5065B69FF2B1DAFD3984EAFF00E0'        
||'D80ACDD153FCED27F88FBC775A6736B6446ECD8CBB2B726E2F9DF4BD9F363F235741E2DCB162B658A1A0AD6FB25AC6C6D454FBAF74007CB5EB4E8FC1'        
||'E5EA7B0FA73E427C30DE745BA77052D1C3D11F12B0BFCC3A9B0FD6D8E83067CD96A6CC7CEDE8EDA99FAFDBF535F4179566DE59FCA8ADAEFDB8528D6D'        
||'4FEEBDD5A777E7F25EDABD43D3596C1D2EE5DC13FC83D93B7FE4C66F29D9F1F6763B77F4DEFEDD5F0C6A3E74B7C96EB4ADE8AC07C7FC66F6F8CDB7E9'        
||'E9BE00761A757EF8DD5D8D9AA9EDE93118E7C76D3C68AAEC04EA0F75EEB5E0F7EEBDD7BDFBAF75FFD5F9FF00FBF75EEBDEFDD7BAEC1B1BFD3FDE7DFB'        
||'AF7475B657F32AFE633D6BB476E75FF5D7CFCF9AFB0761ECEC3D0EDEDA3B2B657CA9EF4DABB476AEDFC5C094B8DC1EDCDB782DF74387C1E1F1F4B1AC'        
||'705352C31410C6A15540007BF75EEB06F6FE641FCC3FB33114BB7FB1FE7A7CCFEC0C0D06E0DB7BB28709BDFE51F786EBC451EEAD9B9BA1DCBB437352'        
||'E373DBE6BE8A9F706D5DC78CA6C8636B1116A686BA9E29E174963561EEBDD2BFFE1D8BF9A6FF00DECB3E7FFF00E9647C8AFF00ED8DEFDD7BA60DD9FC'        
||'CDBF991EFDDABB9762EF9FE60DF37B7A6C9DE9B7F33B4F78ECDDD9F2BFBE771ED5DD9B577163AA70FB836D6E5DBD98DFD5989CF6DFCEE22B26A5ADA2'        
||'AA865A6AAA695E3951D1994FBAF7497EA8F9FF00F3BFA1B64637ACFA33E6BFCB8E98EB8C34F90AAC475FF53FC90EE3EB9D918AAACBD74F94CB54E376'        
||'A6CFDE587C0D0D465327552D4543C502B4F3C8F23967624FBAF7480AAF94FF00272B7BC97E4F567C8CEF7AAF92893D2D527C85A9EDDEC0A8EF14AAA0'        
||'DB10EC9A1A95ED99770BEFE13D16CDA78F110BFF0010D71E3116954881427BF75EEBAEB6F94DF273A6BB2B7677374FFC8CEF7EA9EDFDFB4F9DA4DF3D'        
||'ABD6DDBDD83B1BB277A52EE8CF506EADCF4DBB37D6D8DC38BDD1B8A9F71EE8C5D364AB92B2AA65ABAFA68AA250F3468E3DD7BA507767CD2F98DF25B0'        
||'189DA9F23BE597C97F901B5B03981B8B07B6BBB7BDBB4BB5B0186DC0B43578D5CE62B0FBEF756771D8ECC0C7574F4E2AA18D27104D2206D2EC0FBAF7'        
||'52372FCC9F90DBBBA6E9BA273BBC30151B2536FED3D9997CE51F56F5261BB937B75EEC09F0D51D7FD57DA1F22F0BB171DF213B67A7FAFE4DA980180D'        
||'A1B9B7465B6CE123DABB7A3A2A18136EE0D683DD7BA2BFEFDD7BAF7BF75EEBFFD6F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7'        
||'BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAFFD7F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75E'        
||'EBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAFFD0F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7B'        
||'AF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAFFD970686F746F73686F703A6631316163'        
||'6336612D383632622D313164372D613864662D6636643135386464356333383C2F7861704D4D3A446F63756D656E7449443E0A203C2F7264663A4465'        
||'736372697074696F6E3E0A0A3C2F7264663A5244463E0A3C2F783A7861706D6574613E0A202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020200A20202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A2020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020202020202020202020202020202020202020202020202020202020200A202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020200A20202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'0A2020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020202020202020202020202020202020202020202020202020202020202020200A202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020200A20202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020200A2020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020200A20202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020200A2020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020200A20202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020200A2020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020200A20202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020202020200A2020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020200A20202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020202020202020200A2020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020200A20202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020202020202020202020200A2020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020200A20202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020202020202020202020202020200A2020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20200A202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A20202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020202020202020202020202020202020200A2020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020200A202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A20202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020202020202020202020202020202020202020200A2020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020200A202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A20202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020202020202020202020202020202020202020202020200A2020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020200A202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A20202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'2020202020202020202020202020202020202020202020202020202020202020200A2020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'20202020202020202020202020200A202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020'        
||'202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A20202020'        
||'2020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020202020200A3C3F787061636B65'        
||'7420656E643D2777273F3EFFEE000E41646F626500644000000001FFDB00840001010101010101010101010101010101010101010101010101010101'        
||'010101010101010101010101010101020202020202020202020203030303030303030303010101010101010101010102020102020303030303030303'        
||'0303030303030303030303030303030303030303030303030303030303030303030303030303030303FFC00011080050007003011100021101031101'        
||'FFDD0004000EFFC401A20000000602030100000000000000000000070806050409030A0201000B010000060301010100000000000000000006050403'        
||'0702080109000A0B10000201030401030302030303020609750102030411051206210713220008311441322315095142166124331752718118629125'        
||'43A1B1F02634720A19C1D13527E1533682F192A24454734546374763285556571AB2C2D2E2F2648374938465A3B3C3D3E3293866F3752A393A48494A'        
||'58595A6768696A767778797A85868788898A9495969798999AA4A5A6A7A8A9AAB4B5B6B7B8B9BAC4C5C6C7C8C9CAD4D5D6D7D8D9DAE4E5E6E7E8E9EA'        
||'F4F5F6F7F8F9FA110002010302040403050404040606056D010203110421120531060022134151073261147108428123911552A162163309B124C1D1'        
||'4372F017E18234259253186344F1A2B226351954364564270A7383934674C2D2E2F255657556378485A3B3C3D3E3F3291A94A4B4C4D4E4F495A5B5C5'        
||'D5E5F52847576638768696A6B6C6D6E6F667778797A7B7C7D7E7F7485868788898A8B8C8D8E8F839495969798999A9B9C9D9E9F92A3A4A5A6A7A8A9A'        
||'AABACADAEAFAFFDA000C03010002110311003F00F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75E'        
||'EBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAFFD0F9FF00FBF75EEBDEFDD7BAEC7FC4FD3EBF5FE80FD7DFBAF74763657C03EF4DFDB476E6F6'        
||'C16FBF84F4386DD587A1CE6328B7AFF32AFE5CDD69BBA968B2302D4D3C1B8FAFFB17E54ED5DF7B37311C6E04F8ECBE3687214AF78E686370547BAF75'        
||'1F7B7C0BEF0D8188A5CDE777CFC30AFA3ADDC1B6F6D4306C8FE63FFCBC3B332EB91DD39BA1DBF8CA9AADBDD6FF0028F75E7E83014D5F908E4C965A6A'        
||'68F1786A1596B2BEA29A8E19A78FDD7BA577FC369FC8AFF9F8DF003FF4EC5FCACBFF00BB23DFBAF741CF6BFC28EE2E98D9392EC0DE1BCFE23E63058A'        
||'9F1D4D5541D51F3FBE08F7D6F6964CA56C38EA66C6F5A7467C90EC4EC7CCD3C53D42B54CB478A9E2A38034D3B470A3BAFBAF75EEA8F851DC7DCFB231'        
||'BD81B43797C47C3E0B2B5191A6A6A0ED7F9FDF043A177B452632BA7C7D4B64BAD3BCFE4875D763E1A9E5A8A766A696AF15045590159E9DA485D246F7'        
||'5EE901DE1F1CBB03E3D556DDA3DFDB83A1F3F36E9A7C954E31FA3BE537C63F93B494B1E2A4A38AA937157FC6CEDFED9A2D9F512B57A1A68B2F2514B5'        
||'8AB2353ACAB14A53DD7BA7FE92F89BDA3F2030195DCBB13757C68C0E3B0B98FE05594FDDBF34BE1CFC68CFCD5AB454B9033E276A7C8EEF7EAADD39EC'        
||'3F86AD14646868EA71E660F089BCD1491A7BAF75DF767C4DED2F8FF80C56E5DF5BA7E33E7B1D99CB8C1D1D37497CD1F873F2633F0D735155643CF96D'        
||'A9F1CBBDFB5774E070E20A275391AEA3A7C7898A4266F2CB1A3FBAF74597DFBAF75EF7EEBDD7BDFBAF75FFD1F9FF00FBF75EEBDEFDD7BAEC7FC4FD7E'        
||'9F4FE84FD3DFBAF7572FFCBD7B7BBCFBCB72C7D0982E9EC46F8C375E7544D99C647F17FF0090D7F2E6FE637F2064A1DBB9CDA5B660C9761637B17607'        
||'59EFBC9ED7F1EE00B91DE197DD396AFF00E2B25153CD0CEF5E6A20F75EE837F9B5F227BC3ADFB7B77F40E77AB7ABB6851EC4DC1D6BB921C77767F263'        
||'FE5E1F053E46D3E469713B47B27194FD87D5BD71D47BC33DB570192AFAB8CAE266DD594C56ECDB32C2F5F4ED47919B1EBEEBDD1FFF008F727CB3F90D'        
||'F1A9BB9B627C58ECFEC5DED93C7EFB8F67527C7CFF0084AE7F2C4EF8F8D3B8F74EDCABCBE376F60DBE4E61B6B6DBCBD2E3EB72D430D2E7AAE9762642'        
||'A7012B542C547927A7549FDD7BA11FA4BA7FF988E7B6FE5AA7E477C04F92FD55BA61CC1830786E92FF0084927F2EBF90180C86005152BAE4F2DB977D'        
||'EC2F8D190C3660E45E788D0C38AAF84431C72FDD9691A18BDD7BAAF1F817BF77CEF4ECFCEFC67E96D8798EE3EABDB14FD87BA362E7BAEBF9187F2FFF'        
||'00E617F2F370ED9A7DDB4FFC1F707606C0EDA4C9EF1C1E1EAA87351C991BF666E3A1DB3532C18DA392B298C53AFBAF7467FE7CFC54EFBCFF00C7BCFE'        
||'EAAFF8D1F3DF0F17584F1EFCAACA6E0FF84D77C4AFE5AFD7589C163E19A9B74E6FB2FE47FC5EECCCAEEEC6ED6C0EDCAAA9AC14993C5E430F2D543134'        
||'C299952AE1F75EE860F8CDF117E4675B7486C1DB551F14FF0098B45916C3C79ACCD3663FE1287F087E608A3CCEE176CC64E8F09F227E4576B2F6D764'        
||'ED7A6ACAC65C65664E9B1E05178D21A2A5855214F75EEAA0BE7EF6AF68CFD9DBA7E3FEFBD81B5F6263BACF7463ABE8E8F727F2C0F873FCB6BE46435B'        
||'5BB5696A3ED7B4FAFF00E38EC68F74E069A4833AF253E1EBB75E671F534E696BCA24AD1AC1EEBDD57AFBF75EEBDEFDD7BAF7BF75EEBFFFD2F9FF00FB'        
||'F75EEBDEFDD7BAF7BF75EE8E07C29FF431FE94F3FF00E9CFFD93FF00EE97FA3FCAFF000EFF006757FD9F9FF459FDE2FEF16D6FB4FE01FF000DD1FF00'        
||'19B7FD207F0CFBCFB5FE29FEFD5FE1DF7DF73FE5FF00C33DFBAF741FFCA3FF00475FE9DB7CFF00A27FF65FFF00D1FDF6CFF00FF6573FD9A6FF00413F'        
||'F1E7EDEFE2BFDC6FF6753FE726ADFC6BEE7F89FF00797FE5F1F77FC3BFDC57D8FBF75EE8E87C7CCCFC4ADBBF1A1B70F63ED3FE561BF7B4B038FDF799'        
||'1B1BE41D4FF3B4A3F92FBD2A7195597ADDBFB4D6ABE2A6E5D97F0EA9721B8E1A7868F0529CC6269238A6A76CD56D2CA2AA74F75EE840E9DECBF851DB'        
||'DB573D90DE9F1EFF00938FC58CBD3642A30D8EC177166BFE140FB8774E469A4C6D3CF1EEDC04DD01DE1F23764A63E9EAAA9A18E3CA564157F734CE5E'        
||'8DA9CA3CBEEBDD141F8C786E8FDB9F217786DEED4DD9F0BB7EF5BE071DBCB0D86DF5F2729BF98751FC7ADE9518BCFE3E8709BAF67D27C32DB5B2FE57'        
||'52E4771E3A192B30D1E7F0F89A54C7CD2AE5A8E96B843027BAF746F762663E1449DBFDE3B7F706D3FE4E306CBA5C86C8CCEC1DF5BEEA7FE140E7A7F2'        
||'34F5FB1701439EDA9D1D47D67B9A5EFD4C7E173F8DA8ACCE4BD958682A9F3B90AA5C3D65461968A0A6F75EEBB399F85155F2320C18DA7FC9C71FD678'        
||'8EA0CD56C9BC7EE7FE140EFF001CF74EF4CBEEDDAA94389A8A53B9ABFE5AD3F706D5C3636B1A89A2C563B60C988C8D77DDCF59945C68A6F75EEAB83E'        
||'48FF0070BFD36EFEFF00463FE83FFB89FC431DFDDEFF0065BBFD9913D25F6DFC0F19F73FDC1FF66F7FE723FF0087FDFF0097EE3FBD1FE55F7DE7F07F'        
||'91FDB7BF75EE963B97E1B7C87DA1D354DDED9ED9D80A7D92FB7F69EF4CB60E8FB47A9733DC9B2BAF37FD461A9BAFBB53B3FE3A61B7CE47E4175374FF'        
||'006049BAB02701BC3736D7C4ED9CDA6EADBCF455D3A6E2C1B643DD7BA2BBEFDD7BAF7BF75EEBFFD3F9FF00FBF75EEBDEFDD7BAEC7D7F3FEC3EBEFDD7'        
||'BABAEE94FE6CDFE89BA8FAE7ACFF00D2A7F3ADC57F71367E0F6BFF000DE94FE74BFE85BA9287F8450C547F6DD71D4BFEC88EFAFF00475B3E131DA870'        
||'FF00C6729F630058BEE65D3ACFBAF74DFDDBFCD67FD2DED1C56D9FF4A3FCE7F2BFC37B03ADF7AFDB776FF397FF004D7B463FEE1EF6C1EEDFBDC56D1F'        
||'F6467617F0BEC0A1FE13E5DB79FF00BD9BFBB99B4A6C97D9D6FDB7DACBEEBDD0BE7F9D47FDFE3FE7FDFF00A7F4B7FBC7FC3727BF75EE987767F39FDD'        
||'359B5B72526C6EFDFE7BBB777B556DFCCD36CEDC5BB3F9E66477A6D5C16EA9B1B5316DFCCEE5D9D87F821B072FBB3018BCB3C33D6632973B85A9AFA6'        
||'47823AFA37905447EEBDD05FF1F3F9AEEE9EA9EBC6DBDBE7BB3F9C6E4B7B65B77EFBDF3BC739F1F3F9C0647E3AF5E6E5DD5D81BB32FBBB70EE96EB5C'        
||'C7C3DEEECC43BBF7265F312D6E772955B92BEA337979A7AE97C6F398D7DD7BA0029FE79EE9C3FCDBCFFCB1DB5DA9FCC3B0547BB71F8DC06EEDC54BFC'        
||'C0F231FCD9DD3B5A8B636DDDB92EDDCFFCD8A0E83C6C798C736736B50CB4F15575FCD4F4D83A1A4C5986492963C87BF75EEBAE94F9E7BA7AEFE5576D'        
||'7C9ADE7DA9FCC3B239EECCDBF9FDB8FBE3A4FF00982647A3BE54E531557B8768D6EDBC4F6D7CA1CAF41F6E56F6C60317B73685252E4284ED9C3C792C'        
||'8D15056C468A2A14A193DD7BA507CD9F9F9FECDB6C1DA9B2FF00BF7FCCBF75FF0076F782EE8FB3F9B3FCCA3FD9D6D834C170995C4FDD6D4D85FECABF'        
||'437F757780FE23A4663F88D669A269E9FED8F9FCB1FBAF747BFBF3F9D0ED5EDEE9BCB6729F6D6E087E41EF6DBFF2630B93EB04EB1C6ED1E9AD83BABE'        
||'67CDF3A47C96ECCAEEF6DBFF0020727BDBE4D6DFA8A6F9FF00D88FD61B1F7575CE12A7A824CBE3931DBB324293B01FB7FDD7BAD783DFBAF75EF7EEBD'        
||'D7FFD4F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75E'        
||'EBDEFDD7BAFFD5A83FFA06AF3DFF007969FF00B01A8FFF00B7BFB13FF537993FE8DBFF005522FF00A0FA32FDD1B8FF00CA3FFC697FE82EBDFF0040D5'        
||'E7BFEF2D3FF60351FF00F6F7F7EFEA6F327FD1B7FEAA45FF0041F5EFDD1B8FFCA3FF00C697FE82EBDFF40D5E7BFEF2D3FF0060351FFF006F7F7EFEA6'        
||'F327FD1B7FEAA45FF41F5EFDD1B8FF00CA3FFC697FE82EBDFF0040D5E7BFEF2D3FF60351FF00F6F7F7EFEA6F327FD1B7FEAA45FF0041F5EFDD1B8FFC'        
||'A3FF00C697FE82EBDFF40D5E7BFEF2D3FF0060351FFF006F7F7EFEA6F327FD1B7FEAA45FF41F5EFDD1B8FF00CA3FFC697FE82EBDFF0040D5E7BFEF2D'        
||'3FF60351FF00F6F7F7EFEA6F327FD1B7FEAA45FF0041F5EFDD1B8FFCA3FF00C697FE82EBDFF40D5E7BFEF2D3FF0060351FFF006F7F7EFEA6F327FD1B'        
||'7FEAA45FF41F5EFDD1B8FF00CA3FFC697FE82EBC3FE13599D371FECDA723EBFF00181A8FF238FF009AEFEFDFD4DE65A0276DFF00AA917FD07D7BF73E'        
||'E3E76DFF001A5FF3F5EFFA06AF3DFF007969FF00B01A8FFF00B7BFBF7F537993FE8DBFF5522FFA0FAF7EE8DC7FE51FFE34BFF4175EFF00A06AF3DFF7'        
||'969FFB01A8FF00FB7BFBF7F537993FE8DBFF005522FF00A0FAF7EE8DC7FE51FF00E34BFF004175EFFA06AF3DFF007969FF00B01A8FFF00B7BFBF7F53'        
||'7993FE8DBFF5522FFA0FAF7EE8DC7FE51FFE34BFF4175EFF00A06AF3DFF7969FFB01A8FF00FB7BFBF7F537993FE8DBFF005522FF00A0FAF7EE8DC7FE'        
||'51FF00E34BFF004175EFFA06AF3DFF007969FF00B01A8FFF00B7BFBF7F537993FE8DBFF5522FFA0FAF7EE8DC7FE51FFE34BFF4175EFF00A06AF3DFF7'        
||'969FFB01E8FF00FB7BFBF7F537993FE8DBFF005522FF00A0FADFEE7DC7FE51FF00E34BFF004175FFD6B48F793DD4A1D7BDFBAF75EF7EEBDD7BDFBAF7'        
||'5EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75FFD7B48F793DD4A1D7BDFBAF75EF7EEBDD'        
||'7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75FFD0B48F793DD4A1D7BDFBAF75'        
||'EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75FFD9'                      
;                                                                                                                                   
BRANDINGBackgroundIMAGE_1 raw (32676) :=                                                                                            
'FFD8FFE000104A46494600010201004800480000FFE1053A4578696600004D4D002A000000080007011200030000000100010000011A000500000001'          
||'00000062011B0005000000010000006A0128000300000001000200000131000200000014000000720132000200000014000000868769000400000001'        
||'0000009C000000C80000004800000001000000480000000141646F62652050686F746F73686F7020372E3000323030333A30353A31342030373A3430'        
||'3A31320000000003A00100030000000100010000A00200040000000100000640A0030004000000010000003400000000000000060103000300000001'        
||'00060000011A00050000000100000116011B0005000000010000011E0128000300000001000200000201000400000001000001260202000400000001'        
||'0000040C0000000000000048000000010000004800000001FFD8FFE000104A46494600010201004800480000FFED000C41646F62655F434D0001FFEE'        
||'000E41646F626500648000000001FFDB0084000C08080809080C09090C110B0A0B11150F0C0C0F1518131315131318110C0C0C0C0C0C110C0C0C0C0C'        
||'0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C010D0B0B0D0E0D100E0E10140E0E0E14140E0E0E0E14110C0C0C0C0C11110C0C0C0C0C0C11'        
||'0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0CFFC00011080004008003012200021101031101FFDD00040008FFC4013F000001'        
||'0501010101010100000000000000030001020405060708090A0B0100010501010101010100000000000000010002030405060708090A0B1000010401'        
||'030204020507060805030C33010002110304211231054151611322718132061491A1B14223241552C16233347282D14307259253F0E1F163733516A2'        
||'B283264493546445C2A3743617D255E265F2B384C3D375E3F3462794A485B495C4D4E4F4A5B5C5D5E5F55666768696A6B6C6D6E6F637475767778797'        
||'A7B7C7D7E7F711000202010204040304050607070605350100021103213112044151617122130532819114A1B14223C152D1F0332462E17282924353'        
||'15637334F1250616A2B283072635C2D2449354A317644555367465E2F2B384C3D375E3F34694A485B495C4D4E4F4A5B5C5D5E5F55666768696A6B6C6'        
||'D6E6F62737475767778797A7B7C7FFDA000C03010002110311003F00E4717EDFFB3AADBFB1367A436FABF63F5E20FF003BEA7E9BD7FF008CFD22A67F'        
||'6547B7ECDBA073EA472DDDFF007E588927CFA6FF0055D2E9BFD5D0CEFB3ECFD5BD1DBAFA9B7E96ED3E8EFF00CDFF008B5DB7F8ACD9FB3BA9EEDD3EB5'        
||'3B76733B2FFA5B7DDB579CA4963F990377DB33F7FD8B27D0F527D17EDDB3E1F9BEA7B7FCC5CE37ED1FB2B07D4DD3BC7A331FCD6E3E9ED9FE4FF34BCD'        
||'92524F696FF2949F987987D42CF5BD2CDDF11E933D788DDEA6DD3F99F77A9EA7EF7B156C5FE82CF579F4CEFE3883BE767B7E87D35E7092BB8BE4E9B4'        
||'76FEEACE6FFEEBF97F84FAC53FF373D0CBF5B66EDA3D5DDB276FA2DDBE9FFE83A6A3D1D9FABFF37357EEFF00A2A7D2FA1FE13D1F4BF9A5E50928398F'        
||'E6CF98FE4164377D17A9ECD95FAF1C3E7D48DBCFBFD4FB0FBFE96DFE7968F4EDFF00B39BF66F53640F47D09DDE9FF86F4FD7F67D3DFE9FABFA6FA7E8'        
||'7E99794A49D1D86ECB97F9996DFA3F37CBFCE3EABD1FD1FD916FEC78FB0FE96388DFB1DEAFD3FD6BFEDCFD2AC8CCD9FB3A8F57D5DBBEA8D9BB74459F'        
||'477FB7E82E0524C1FCFE3F3598BF4BCFFC2FF09F68FAAFF6EFF9BB67A93B3EDB67A1B767F37B6BF43D4FCDFA1FBFFA2567267EDF891E97ABEB59BBD6'        
||'9F57D281EA7D97D7FD0EFF005BF9CFB07E93D0F4D787249C3F4B7FD2FEF6DFA6B67FCE7D46EFFFD9FFED115E50686F746F73686F7020332E30003842'        
||'494D03ED0A5265736F6C7574696F6E0000000010004800000001000100480000000100013842494D040D18465820476C6F62616C204C69676874696E'        
||'6720416E676C650000000004000000783842494D041912465820476C6F62616C20416C74697475646500000000040000001E3842494D03F30B507269'        
||'6E7420466C61677300000009000000000000000001003842494D040A0E436F7079726967687420466C6167000000000100003842494D2710144A6170'        
||'616E657365205072696E7420466C616773000000000A000100000000000000023842494D03F517436F6C6F722048616C66746F6E652053657474696E'        
||'677300000048002F66660001006C66660006000000000001002F6666000100A1999A0006000000000001003200000001005A00000006000000000001'        
||'003500000001002D000000060000000000013842494D03F817436F6C6F72205472616E736665722053657474696E6773000000700000FFFFFFFFFFFF'        
||'FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF03E800000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF03E800000000FFFFFFFFFFFFFFFFFFFF'        
||'FFFFFFFFFFFFFFFFFFFFFFFF03E800000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF03E800003842494D04000B4C6179657220537461'        
||'74650000000200093842494D04020C4C617965722047726F75707300000000180000000000000000000000000000000000000000000000003842494D'        
||'040806477569646573000000001A00000001000002400000024000000002000005BE0100000CA0003842494D041E0D55524C206F7665727269646573'        
||'00000004000000003842494D041A06536C69636573000000008D00000006000000000000000000000050000006400000001600770062006900630032'        
||'002800380030007000780029005F007200650076002E00620061006E006E006500720000000100000000000000000000000000000000000000010000'        
||'0000000000000000064000000050000000000000000000000000000000000000000000000000000000000000000000003842494D0414174C61796572'        
||'2049442047656E657261746F72204261736500000004000000343842494D040C154E65772057696E646F7773205468756D626E61696C000003FF0000'        
||'0001000000700000000600000150000007E0000003E300180001FFD8FFE000104A46494600010201004800480000FFEE000E41646F62650064800000'        
||'0001FFDB0084000C08080809080C09090C110B0A0B11150F0C0C0F1518131315131318110C0C0C0C0C0C110C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C'        
||'0C0C0C0C0C0C0C0C0C0C0C010D0B0B0D0E0D100E0E10140E0E0E14140E0E0E0E14110C0C0C0C0C11110C0C0C0C0C0C110C0C0C0C0C0C0C0C0C0C0C0C'        
||'0C0C0C0C0C0C0C0C0C0C0C0C0C0C0C0CFFC00011080006007003012200021101031101FFDD00040007FFC4013F000001050101010101010000000000'        
||'0000030001020405060708090A0B0100010501010101010100000000000000010002030405060708090A0B1000010401030204020507060805030C33'        
||'010002110304211231054151611322718132061491A1B14223241552C16233347282D14307259253F0E1F163733516A2B283264493546445C2A37436'        
||'17D255E265F2B384C3D375E3F3462794A485B495C4D4E4F4A5B5C5D5E5F55666768696A6B6C6D6E6F637475767778797A7B7C7D7E7F7110002020102'        
||'04040304050607070605350100021103213112044151617122130532819114A1B14223C152D1F0332462E1728292435315637334F1250616A2B28307'        
||'2635C2D2449354A317644555367465E2F2B384C3D375E3F34694A485B495C4D4E4F4A5B5C5D5E5F55666768696A6B6C6D6E6F62737475767778797A7'        
||'B7C7FFDA000C03010002110311003F00F3D6F52E91F656D0EE9357AA181AECA16DFBC91F4AEF4BD6F437BBF7367A6806EE97AC63BF5EFB8E9FF49534'        
||'9137D6BE95FF007293F4FA24BDF53DFF00A2AC56C130249244E9BB739CBDDF31AC6BCED73DF249960B0341F06AF04494B87AEFD3640DFA7D5F5CCEFD'        
||'B3FB6EB7B7ED5B054775637ED2D1EAFA9EEFE6FDDFA2FA6A54FEDD6D38FF00D29E4D81D54FA9A0878756E9F4DDF4BFE82F214938571C76FD2F997C36'        
||'9F90FCFF0049F55B8F54F59E48CD04B896877AC4813F4674FCEFDD5A1D3EAAF7B1D6DF93EB1AB4639B7E8DF50EE97BBF97FC95E3492B72F947EC73FF'        
||'00CAE4FEF1FCDF5FF4EC6BA70AECAB01F5340DB9A1BEFF007EEDDEC77BD0330753FB1DAEDD99B4B4457535FB86BF9B607FAFF4FF0072BFE6D793A4AB'        
||'1FE7BE8D9C5B0F37D47A67ED17E2670A8BEABDEC2C73B205C5EC6C5917D433277D9BBD4FD163AD7FAAEFC76606037A7D74BDADA4B711C5CC3B981C7E'        
||'D169F53D17B6F7646FF51AEAFD45E2E927CB7FD1EBFF00458E5FCF4AAF78F97CBFD6FF0098FB3E736E77D6BC2F59D632F6D4C7542A2C2D2E16D9E9D4'        
||'FD8CF43D373B77A9F4323FD15EAE5807DA46E73F47B4E3B4B1825DFA4DCC75B7DCE7EDFF004BF66C6AEE5E18923FF783FBBF45F93FC9F9F5F9BEAFFF'        
||'D9003842494D04211A56657273696F6E20636F6D7061746962696C69747920696E666F000000005500000001010000000F00410064006F0062006500'        
||'2000500068006F0074006F00730068006F00700000001300410064006F00620065002000500068006F0074006F00730068006F007000200036002E00'        
||'3000000001003842494D0FA016506C75672D696E207265736F7572636520626C6F636B00000004826D6F707400000003000000010000000000000007'        
||'000000010100000000010000FFFFFFFFFFFFFFFF0000006400000001FFFFFFFF0000000000000000000000000000000600000000FFFFFFFF00000000'        
||'04FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF00000000FFFFFFFF0000000000000020FF000000FFFFFFFFFF9C9A9CFF848284FF9492ADFFBDBC'        
||'CDFF31309CFF313063FF0000FFFF00009CFF000084FF000063FFDEDEE6FF63659CFFBDBECEFFDEDFE7FF00FF3100000000FF636563FFB3B3B3FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF0000'        
||'00FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000FF000000000000000000000000000000003842494D0FA1'        
||'16506C75672D696E207265736F7572636520626C6F636B00000004A46D7365740000000000000010000000010000000000046E756C6C000000030000'        
||'000756657273696F6E6C6F6E67000000000000000C48544D4C53657474696E67734F626A63000000010000000000046E756C6C000000100000001246'        
||'696C65536176696E6753657474696E67734F626A63000000010000000000046E756C6C0000000800000010496E636C756465436F7079726967687462'        
||'6F6F6C0100000018536176696E6746696C654E616D65436F6D706F6E656E7473566C4C73000000096C6F6E67000000016C6F6E67000000146C6F6E67'        
||'000000026C6F6E67000000136C6F6E67000000066C6F6E67000000186C6F6E67000000186C6F6E67000000186C6F6E6700000016000000114E616D65'        
||'436F6D7061746962696C6974794F626A63000000010000000000046E756C6C00000003000000114E616D65436F6D70617457696E646F7773626F6F6C'        
||'010000000D4E616D65436F6D7061744D6163626F6F6C010000000E4E616D65436F6D706174554E4958626F6F6C0100000012496D616765537562666F'        
||'6C6465724E616D6554455854000000070069006D00610067006500730000000000194475706C696361746546696C654E616D654265686176696F726C'        
||'6F6E67000000010000000E436F70794261636B67726F756E64626F6F6C0100000011557365496D616765537562666F6C646572626F6F6C0100000017'        
||'536C69636546696C654E616D65436F6D706F6E656E7473566C4C73000000066C6F6E67000000006C6F6E67000000136C6F6E67000000096C6F6E6700'        
||'0000186C6F6E67000000186C6F6E670000001800000010496D6167654D61704C6F636174696F6E6C6F6E67000000020000000D544457696474684865'        
||'696768746C6F6E67000000010000000C496D6167654D6170547970656C6F6E670000000000000010476F4C697665436F6D70617469626C65626F6F6C'        
||'000000001153706163657273456D70747943656C6C736C6F6E67000000000000000F496E636C756465436F6D6D656E7473626F6F6C01000000115370'        
||'6163657273486F72697A6F6E74616C6C6F6E67000000010000000C5374796C6573466F726D61746C6F6E67000000000000001251756F7465416C6C41'        
||'747472696275746573626F6F6C000000000F53706163657273566572746963616C6C6F6E670000000100000006557365435353626F6F6C000000000D'        
||'417474726962757465436173656C6F6E67000000000000000B4C696E65456E64696E67736C6F6E670000000100000006496E64656E746C6F6E67FFFF'        
||'FFFF00000007546167436173656C6F6E67000000000000001648544D4C4261636B67726F756E6453657474696E67734F626A63000000010000000000'        
||'046E756C6C00000006000000124261636B67726F756E64436F6C6F725265646C6F6E67000000FF000000144261636B67726F756E64436F6C6F724772'        
||'65656E6C6F6E67000000FF000000134261636B67726F756E64436F6C6F72426C75656C6F6E67000000FF000000134261636B67726F756E64496D6167'        
||'65506174685445585400000001000000000014557365496D61676541734261636B67726F756E64626F6F6C00000000144261636B67726F756E64436F'        
||'6C6F7253746174656C6F6E67000000013842494D04060C4A504547205175616C69747900000000070008000000010100FFE20C584943435F50524F46'        
||'494C4500010100000C484C696E6F021000006D6E74725247422058595A2007CE00020009000600310000616373704D53465400000000494543207352'        
||'47420000000000000000000000000000F6D6000100000000D32D48502020000000000000000000000000000000000000000000000000000000000000'        
||'00000000000000000000000000000000001163707274000001500000003364657363000001840000006C77747074000001F000000014626B70740000'        
||'0204000000147258595A00000218000000146758595A0000022C000000146258595A0000024000000014646D6E640000025400000070646D64640000'        
||'02C400000088767565640000034C0000008676696577000003D4000000246C756D69000003F8000000146D6561730000040C00000024746563680000'        
||'04300000000C725452430000043C0000080C675452430000043C0000080C625452430000043C0000080C7465787400000000436F7079726967687420'        
||'2863292031393938204865776C6574742D5061636B61726420436F6D70616E790000646573630000000000000012735247422049454336313936362D'        
||'322E31000000000000000000000012735247422049454336313936362D322E3100000000000000000000000000000000000000000000000000000000'        
||'0000000000000000000000000000000000000000000058595A20000000000000F35100010000000116CC58595A200000000000000000000000000000'        
||'000058595A200000000000006FA2000038F50000039058595A2000000000000062990000B785000018DA58595A2000000000000024A000000F840000'        
||'B6CF64657363000000000000001649454320687474703A2F2F7777772E6965632E636800000000000000000000001649454320687474703A2F2F7777'        
||'772E6965632E636800000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000646573630000'        
||'00000000002E4945432036313936362D322E312044656661756C742052474220636F6C6F7572207370616365202D2073524742000000000000000000'        
||'00002E4945432036313936362D322E312044656661756C742052474220636F6C6F7572207370616365202D2073524742000000000000000000000000'        
||'0000000000000000000064657363000000000000002C5265666572656E63652056696577696E6720436F6E646974696F6E20696E2049454336313936'        
||'362D322E3100000000000000000000002C5265666572656E63652056696577696E6720436F6E646974696F6E20696E2049454336313936362D322E31'        
||'000000000000000000000000000000000000000000000000000076696577000000000013A4FE00145F2E0010CF140003EDCC0004130B00035C9E0000'        
||'000158595A2000000000004C09560050000000571FE76D6561730000000000000001000000000000000000000000000000000000028F000000027369'        
||'6720000000004352542063757276000000000000040000000005000A000F00140019001E00230028002D00320037003B00400045004A004F00540059'        
||'005E00630068006D00720077007C00810086008B00900095009A009F00A400A900AE00B200B700BC00C100C600CB00D000D500DB00E000E500EB00F0'        
||'00F600FB01010107010D01130119011F0125012B01320138013E0145014C0152015901600167016E0175017C0183018B0192019A01A101A901B101B9'        
||'01C101C901D101D901E101E901F201FA0203020C0214021D0226022F02380241024B0254025D02670271027A0284028E029802A202AC02B602C102CB'        
||'02D502E002EB02F50300030B03160321032D03380343034F035A03660372037E038A039603A203AE03BA03C703D303E003EC03F9040604130420042D'        
||'043B0448045504630471047E048C049A04A804B604C404D304E104F004FE050D051C052B053A05490558056705770586059605A605B505C505D505E5'        
||'05F6060606160627063706480659066A067B068C069D06AF06C006D106E306F507070719072B073D074F076107740786079907AC07BF07D207E507F8'        
||'080B081F08320846085A086E0882089608AA08BE08D208E708FB09100925093A094F09640979098F09A409BA09CF09E509FB0A110A270A3D0A540A6A'        
||'0A810A980AAE0AC50ADC0AF30B0B0B220B390B510B690B800B980BB00BC80BE10BF90C120C2A0C430C5C0C750C8E0CA70CC00CD90CF30D0D0D260D40'        
||'0D5A0D740D8E0DA90DC30DDE0DF80E130E2E0E490E640E7F0E9B0EB60ED20EEE0F090F250F410F5E0F7A0F960FB30FCF0FEC1009102610431061107E'        
||'109B10B910D710F511131131114F116D118C11AA11C911E81207122612451264128412A312C312E31303132313431363138313A413C513E514061427'        
||'1449146A148B14AD14CE14F01512153415561578159B15BD15E0160316261649166C168F16B216D616FA171D17411765178917AE17D217F7181B1840'        
||'1865188A18AF18D518FA19201945196B199119B719DD1A041A2A1A511A771A9E1AC51AEC1B141B3B1B631B8A1BB21BDA1C021C2A1C521C7B1CA31CCC'        
||'1CF51D1E1D471D701D991DC31DEC1E161E401E6A1E941EBE1EE91F131F3E1F691F941FBF1FEA20152041206C209820C420F0211C2148217521A121CE'        
||'21FB22272255228222AF22DD230A23382366239423C223F0241F244D247C24AB24DA250925382568259725C725F726272657268726B726E827182749'        
||'277A27AB27DC280D283F287128A228D429062938296B299D29D02A022A352A682A9B2ACF2B022B362B692B9D2BD12C052C392C6E2CA22CD72D0C2D41'        
||'2D762DAB2DE12E162E4C2E822EB72EEE2F242F5A2F912FC72FFE3035306C30A430DB3112314A318231BA31F2322A3263329B32D4330D3346337F33B8'        
||'33F1342B3465349E34D83513354D358735C235FD3637367236AE36E937243760379C37D738143850388C38C839053942397F39BC39F93A363A743AB2'        
||'3AEF3B2D3B6B3BAA3BE83C273C653CA43CE33D223D613DA13DE03E203E603EA03EE03F213F613FA23FE24023406440A640E74129416A41AC41EE4230'        
||'427242B542F7433A437D43C044034447448A44CE45124555459A45DE4622466746AB46F04735477B47C04805484B489148D7491D496349A949F04A37'        
||'4A7D4AC44B0C4B534B9A4BE24C2A4C724CBA4D024D4A4D934DDC4E254E6E4EB74F004F494F934FDD5027507150BB51065150519B51E65231527C52C7'        
||'5313535F53AA53F65442548F54DB5528557555C2560F565C56A956F75744579257E0582F587D58CB591A596959B85A075A565AA65AF55B455B955BE5'        
||'5C355C865CD65D275D785DC95E1A5E6C5EBD5F0F5F615FB36005605760AA60FC614F61A261F56249629C62F06343639763EB6440649464E9653D6592'        
||'65E7663D669266E8673D679367E9683F689668EC6943699A69F16A486A9F6AF76B4F6BA76BFF6C576CAF6D086D606DB96E126E6B6EC46F1E6F786FD1'        
||'702B708670E0713A719571F0724B72A67301735D73B87414747074CC7528758575E1763E769B76F8775677B37811786E78CC792A798979E77A467AA5'        
||'7B047B637BC27C217C817CE17D417DA17E017E627EC27F237F847FE5804780A8810A816B81CD8230829282F4835783BA841D848084E3854785AB860E'        
||'867286D7873B879F8804886988CE8933899989FE8A648ACA8B308B968BFC8C638CCA8D318D988DFF8E668ECE8F368F9E9006906E90D6913F91A89211'        
||'927A92E3934D93B69420948A94F4955F95C99634969F970A977597E0984C98B89924999099FC9A689AD59B429BAF9C1C9C899CF79D649DD29E409EAE'        
||'9F1D9F8B9FFAA069A0D8A147A1B6A226A296A306A376A3E6A456A4C7A538A5A9A61AA68BA6FDA76EA7E0A852A8C4A937A9A9AA1CAA8FAB02AB75ABE9'        
||'AC5CACD0AD44ADB8AE2DAEA1AF16AF8BB000B075B0EAB160B1D6B24BB2C2B338B3AEB425B49CB513B58AB601B679B6F0B768B7E0B859B8D1B94AB9C2'        
||'BA3BBAB5BB2EBBA7BC21BC9BBD15BD8FBE0ABE84BEFFBF7ABFF5C070C0ECC167C1E3C25FC2DBC358C3D4C451C4CEC54BC5C8C646C6C3C741C7BFC83D'        
||'C8BCC93AC9B9CA38CAB7CB36CBB6CC35CCB5CD35CDB5CE36CEB6CF37CFB8D039D0BAD13CD1BED23FD2C1D344D3C6D449D4CBD54ED5D1D655D6D8D75C'        
||'D7E0D864D8E8D96CD9F1DA76DAFBDB80DC05DC8ADD10DD96DE1CDEA2DF29DFAFE036E0BDE144E1CCE253E2DBE363E3EBE473E4FCE584E60DE696E71F'        
||'E7A9E832E8BCE946E9D0EA5BEAE5EB70EBFBEC86ED11ED9CEE28EEB4EF40EFCCF058F0E5F172F1FFF28CF319F3A7F434F4C2F550F5DEF66DF6FBF78A'        
||'F819F8A8F938F9C7FA57FAE7FB77FC07FC98FD29FDBAFE4BFEDCFF6DFFFFFFEE000E41646F626500644000000001FFDB008400010101010101010101'        
||'010101010101010101010101010101010101010201010101010102020202020202020202020202020303030303030303030303030303030101010101'        
||'010102010102030202020303030303030303030303030303030303030303030303030303030303030303030303030303030303030303030303030303'        
||'FFC00011080050064003011100021101031101FFDD000400C8FFC401A20000000602030100000000000000000000070806050409030A0201000B0100'        
||'000603010101000000000000000000060504030702080109000A0B100002010304010303020303030206097501020304110512062107132200083114'        
||'4132231509514216612433175271811862912543A1B1F02634720A19C1D13527E1533682F192A24454734546374763285556571AB2C2D2E2F2648374'        
||'938465A3B3C3D3E3293866F3752A393A48494A58595A6768696A767778797A85868788898A9495969798999AA4A5A6A7A8A9AAB4B5B6B7B8B9BAC4C5'        
||'C6C7C8C9CAD4D5D6D7D8D9DAE4E5E6E7E8E9EAF4F5F6F7F8F9FA110002010302040403050404040606056D0102031104211205310600221341510732'        
||'61147108428123911552A162163309B124C1D14372F017E18234259253186344F1A2B226351954364564270A7383934674C2D2E2F255657556378485'        
||'A3B3C3D3E3F3291A94A4B4C4D4E4F495A5B5C5D5E5F52847576638768696A6B6C6D6E6F667778797A7B7C7D7E7F7485868788898A8B8C8D8E8F83949'        
||'5969798999A9B9C9D9E9F92A3A4A5A6A7A8A9AAABACADAEAFAFFDA000C03010002110311003F00F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7B'        
||'F75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF'        
||'75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF7'        
||'5EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAD867FE134FFF0065D9DA9FF8A9BBF3FF007EF6C5'        
||'F723FB61FF0025F9BFE79DFF00EAE45D06B9A7FE49C9FF003517FE3ADD6F09FEBFE7E87F3EE74E801D7BFE2BC5FDFBAF75EFCFFB0FF6DEFDD6BAF7FC'        
||'53FDE7DFBAF74627A4BFE2D5B83FED6747FF00B867D857987FB68BFD29FF000F477B4FF66FF68FF074360049B0049FE805CFFBC7B0F746DD2573DBE3'        
||'676D752DB8373E1312C0E930D4D7C26AB55B511F650979BE9C9F47B7E0B6B8B934B742FF0060C57D2BC2BD3E96B73250A46C6A2BC08143C0D4F91F23'        
||'E7D03B9AF949D578A94C5473E777005B7F94E271263A1249B0095595929B50FAF28847B304D96F5B0C029FE1AD5BF62D69F9D3A308764BE9543B058D'        
||'4F9B35067CFE7F957A4057FCCADB91B15C56C4DC75E79D3F7391C750DC81FD2259CDBFC7DBCBCBBB8D6B3B471A7A9724FEC0B4FF008D74BD361B564A'        
||'0B96793F8521247FBD33A9FF008C749C9BE67CB224A83AB2AF4696576FEF5D3AB2DC736F25128FF5B9F6A24D8A1868DF54A49C81A18FFC70B1FE5D7A'        
||'1E599A7A8D4D41C4E951FF001F651FCFAE343F3076F54CB1C555B42B71973691EAF3D8E10C7FE0268627BDF9FD4AA3FA917F682E6C376B54693E9DAE'        
||'1470F0353391FE92448857E41C93E55E97C1C95B65D32C69BAA5BB9E3F511848C1F2AC90CB3903CAA630079D3A58D2FCA0D8CD2D2A65705BBB0B0561'        
||'3F6F91969F0992C64A80D8CD054E32B646910717F1C4CC3F2A0F1ED3C57161702416F70ACF110248CA4CB2464FE1911A2050FF00A6A03E448CF4CDDF'        
||'B6BCD565A1A48A329302D1389A2F0E651E7149AF4B8C8AD0D4568C01C742BE0FB3FAF7724AD061B77E16AEA5503BD34950D4550A87807C55EB113C9B'        
||'10B7F7E6923450ECCA0370AB015FC89AFF002E83B71CB7CC16A584D67376712A85C0FCD350A7CEB4E977F550C395600AB0E558117055BE8411F423DD'        
||'BA25A50907047F2E81FED39DE09F09A2464D54F58490D6FD3320B9F653BAAEA58F1E6DFF003EF431E5170AD700F988FF00C327411C95B3DFFCF3FD78'        
||'3A88E3F37F64E10798E8768EA4557AE9B20E0732C9C807F530B9FEA7FC7DFB40AF0EBC694E99AB7232588D6F7FEBA8DC9FE86E7DBA8A3853A4732AAD'        
||'64AD6BC01F2FB3A0CF72D6CCD04A9F712A1E34313A9756AFD2DFF15F66369A54D280F41ADD95A5A3FC3F3FF60F44DFB32AA6923C82A4B2B85A2AB528'        
||'23AC92F2185D74A252912B13F4D3110E7E88435AC6F66078E31F953A0C6E63FC55457CC6719C8FC87E7D5517C91158313F1268ABCBB55D1EDBED27AA'        
||'8DE8F7763962A86E88C82C852837FD4556721E081A32F512550E7CCC5BDA5DC92965681B8F7F911F87D1B3FB73D1F6D441DDAFB48F24F4FE2FE8F6FE'        
||'CE83CF8EB1A41D1FF2FA36D0A2AFA3BAB2945D5497BFC98D99378D6E38BE807FD87B0FB2F7C74F5FF21E842DF1291EA7FC07AE1F1729E383E49746CC'        
||'B14484C9DFEB70238D9946D2DAC1EC4823E87FA7FB7F620D83FE4AC95FE097FE383A25E622C36A2578F8B01FFAA9D5BCFC58C1CD49B0EA3EEF189455'        
||'0779EFDA8FB75DB19EDACA52A378D6CB4F3A62F725657D5B7950AB9AA35021A8BF969E28A068E35359E206DE30E286AFE4570589E049FDBC0F1000C7'        
||'4109F709ADF7AB8FA72581098A86C85FE881F3A839518249E8E2D163509219103F8CB950A2C05C5C93FF0011ECA7C36AEA4E15E3E75E842772468960'        
||'B8AEAD218A8C002B9A9F5F41D40AEA28023EA11DD0D8DD505895E41D5F83FD7DB89148FC7CFA4F36ED6900AC182AC071A606726BC2B4F514CF9F418E'        
||'6E0863571188AE108644B0FA73A15BFA8F667040DA425081E5F67415DC7728E498DC3B07623BA9C0B60D0547963F67406663C0D3900F1A893600FD3F'        
||'1A87FBD7B334DBE48417D3F67FC57A744373BD25D958DDF1E63ECF2AFAFF002E822EC4A5925DA5BA29E14F2B4FB77330C680A825E4C7C8882EC78E7F'        
||'26D6F64DBEC24ECF7DE279412D0D3CFC363D08F95AF5577BDB920A55EE6104630A654181EBE67AAEAA4DB996124EA293F3A5879E0B1B3116376F78AA'        
||'429EB3452371FEA1D0BB88DA95F7669288DC4B0B0FF29849E0DFF0FEDB3A474AD226F4FF000743A6D4C3553B47491D23CB555591A78A969E328EF34D'        
||'2B2C50451AA9FAB31000F6C4823035360015E8D6D9249184518AB330000E249C01F99EAD63AE7E2FEC8DBD8BA4A8DE9411EE9DCB2C492D747533D42E'        
||'1F1B33ADDA8A8A929D904A63FD2D34A5B5104A85161EC0D7BBE5CCD211687C341C29C4FCC9F2FB075911B0FB73B458DAABEF082E6E18558127C343FC'        
||'2AA08AD38166AD4F00074F3BC7E3AECBC9D0CB51B471CBB73374F1C9252C14934E7155D20436A5A9A69D9FC65FE8B2C6459BF50617F7AB3DEEE22900'        
||'BA3E221E3FC43E60F9FD87ABEF9EDEED1776CCFB420B69C0254027C373FC25493A6BE4CB4A1E20F54BBBF7E37EE2F923F3030BD678CA97C063A87AFF'        
||'001399DEDB825A7FB96DB5B7B1B93A8A6AE963A662049572CCE94B4B13300656D4C74237BCFAF6C7DCEDAFDA7FBBEDC7345DC7F512BEE13C56B0574F'        
||'8D3BA465416C958D54179180242AD07730EB0B375F6EB72F723DDB8B96AD98DBC71DAA49732915F06247656A0E0646621114E0B1A9ED07AB5ADA7FCB'        
||'E3E236D5C245876EA4C76E8A85804353B877764F3396DC15B215B4952F570CF0C70B31F569A58A355FEC81EF18378FBCBFBD5BBEE0D7CBBD3D9A9355'        
||'86DA38A38547928528CCC07AC8CE4F99EB2BB6CFBBEFB49B6D8AD936D097440A196E1E47958FA960CA14FC91540F21D5677CEBF82781E92C10EDEEA1'        
||'6C91D823234D8EDD9B532555364E7DA53E4E5F06332B89C9CF79A5C7CB315A7963A86692191E32247472132CFEEEFF0078ADC7DC0DC7FA93CEE23FDE'        
||'5A19EDAE514462E4462B24724628AB32A82EAC802C8A1BB5597BB183DF8F61AC3922C0738727EBFDDFAD52E2DDD8B9B72E689246E7B9A266A232B92C'        
||'8C54862A7B6A648E4FFAE7FDEFDE7929ED1F60EB921703FC624FF4EDFF001E3D0BDD7200C6579B73F7E2E6DCDBEDD4D813EC31BE93F509FE97FCA7A9'        
||'3B90BFE49D38FF00867FCFA3A35DF1BEDFE9EBAA2D6FF8FBA97FA7FCA9CDF8F605E68AFF00572F7FE699FF0008EA54E5BFF92F5A7FCD41FE03D5E75F'        
||'FD6FF6C3DE3E7539F48FC8FF00C0DA8FF830FF00A107B571FC03AA9AF5CB187FCA7FE9D3FE383F4F7A93E1EBDD3FB9F43F3FEEB7FC0FF507DB34F2EA'        
||'C38F5542DFADFF00E5A3FF00D0E7DC8DD1B0E1D4DA43E86FF837FC47B6DB8F4EA9A0EB164EA60A4A0A892A6A296929C858249EBA4922A38D6B255A52'        
||'2A248E394AA317D2C4A15E7D565B9063B38AEE31FF00B6FF008E37D9D449EFB4BE17B55BAC86B8FA6E1C7FDCCB7F91FCFA62A8A583214D4936E6CC6D'        
||'CC9410D20C4E2E932598CA9AEA5C4C4C27A18B199FC623B35343200D4F0CE2431D8AC722A8E4685D62F8453571A28E3F61F33E74A7CFAE760B1129D5'        
||'157421C771C9A791AF0F303D49C81D49DEF88C36E7C6B4DFDEBBD4426928F13515BB86A6AFEFE6919454D247B8246A28A6116B2C03C4D322DD672C46'        
||'B158F53E952BC6A49A69141C3B73F61CD0F97A7495C47B7C8EC6B5A80189D4C49F2C100E6A6B4A8FC55E3D03BBB30BB841C36067C464687214B4CD0C'        
||'34F5745A22AD84224F1E423AB5D723C7223ABF91449001CACCA2EA140D041752B41E878797D80D7F3F974D9B86771090C481C48A93C0D4D7BE841E34'        
||'2BE87A75D9557B8305254E0325F75B76AA19294E2EBEBF253E3D70D5D1D5A55B56434AAE927EEC492409F6F32C722C9AD8B585997485DAB251948351'        
||'4AD7CBE63E7EB8A74BEDD2E3452222360568C7068082450F06AE07950D687A14F3391CF6F7AAA7A6CC6236E6F9C84E18C7970F4D49B86AE3A0A6FB51'        
||'0536E45956B0A05D0DF6D91FB86D6BFB6E01B7B6218A28EAB0128A07C3F8479D42F007E6B4FB3A592ABC21A6215AA685A849624D2A5B55471C020D07'        
||'ECE90953B6A92BDE8E8B2D4F59848B1EADF6D1D74944B35225D62ADF35646206F0EA01DE4AE8D847F5D6059898221F089C329CD4571E87D6BE54073D'        
||'2291C2B89678C86F8456A0369F2A9214815AF70C60D7A53D6ED4C88DA3267A9E1933D82A7AFABC2A54455A95B5D049046D22549868E70C94722C7AE1'        
||'33137D4A51143709F5A097C08C80D407E5F6678915A1A7EDE94A0668CDCCB19D1DC6BC71E4DDB4A2B711AA9EB8E83ADBDBB23DA35B4F5B8E41064696'        
||'A8C853F884B8BAB87EE10C65659673501A375FF7591A0FD40BFB70852195F81F50687ECA53AAEA69555A03A4641A15A8CF9D49F4E1907A5BD7EE49B2'        
||'DA67C960FEE82C32A955CDCF3540495BCDFF0003A87C4748FAAAA301CF373C7BA290D528687F97EC35E9D92328034BF0904571535CE0281A7F2C74F8'        
||'9DAFB566923A0DA9B2F726C8DCB2D07F077CED267729BF726B8B24544F1D163376C55A608DDE26691E04575FECB0173EDB5B210D24BB0B2AD6A01509'        
||'9F5ED22A73806A3A49F593DC87B7B791E22DE6496A0028683240E2350C8A9FB7A62CAEC7E94EC5757EE9ED1EB2828B0D4E926DEADEFBABA4867ADAAA'        
||'D6072636BC13E2DDA311A4718AA88C2A46A4F5369F655BB35B4010DC59098B3350058CE8A538F88EA2A7C8A93C3CB1D485C886EE17B936770402B182'        
||'416EE39CD5013FB40F99AD7A6B1D3DFCBFA949FBDEE2F86106824145759CB16FA59E1DB4E0FF0088B1FA8B587B219370DBD0FF00C93457E6B683FC33'        
||'F523ABEF0C317127FD573FE04E8C27C6ED8BF07B01DFBD2797D89DA7F17725BB683B63AF24DA90ED2C564C6726CF26E7A54C7C787A91B5E982D4CD21'        
||'11425A78D751B971FA815EF174B36C97890D87861A0986A1F4D81E1B67B272703D013E83CBA536A7725BC88CD3B30D69507C6A11A863B900FDBD2EBA'        
||'6B11D3949F26361BE2775EC59F3F45DC3451B6270D88CD8C963373BEE4792960AE9131914093D157C88F2CCF5253F6A4935F8ECDECC378BABA9B95EE'        
||'912160BF4ED56D51D0A84AB1A072DDE95A0D3AB2315E92D9C6F1EE50991EB5714C360D68388A501A0E34C7A74B3FE7EDD0192EF2F923D10F0D65260B'        
||'21B6BAEFBF2A6AAA2A2392B28E9445DA92E42B298B5394B879A6A78519780097B102DEE2EF69B6F86F3669E60D94FA60848E20DBD0FCF21413E9D0F3'        
||'9A2F5ED2EBC365F89E6AD0F9F8B5FB31523AD5CA7DAF5114AB0875D2195785362435898DD7820FF5B723DC9AD66C18007A0F2CE08D5FE5E9FA9F697A'        
||'6206259588B8075DEE47F8FF004F6A058E062BD326E6BF674B3C76CFA83194FB445505482D723FC2C0736FEBED6C360E45294E92BDD0078F476FF97F'        
||'ED59713F367E2B57BC70AC50F78EC20EEA5BC8DF739514AE101BDFFCE1F659CD360D1F2BEE0C68296F29FF008C1E9DB0B90FB840BEB22FF8475B0766'        
||'B7AF546CFF0085FF0015A9FB9772E63014590DE1F21176B57607654BBC9563A0DD9494F53419254C8639A9645D45E3F1B4A1D3EA54A9041F0DAEEF73'        
||'EE06F4FB2429232C563E20925F0B262720AFE9C81811C6BA687D6BD2994DA8E58DBBEB1D9017BAD3A103E35A541ABA50D6B4A56A3D3A2B7FE9C3E16C'        
||'F3D506EE6ECE81EEADA13A46BC4700F1985C4328DCDCEABDEC79BFFBC8ABE879CD4051656C69FF002F473FF66DD1215DA4E4CF37FCE15FFADE7AE0BD'        
||'F9F07A213AC9DB9DBF2A4B24657ED7A3AA4B9655085E413EE5E6FCFAB9361F4F7A367CEA402B676B8F5BA6FF0025B75B54DA2B99A6FF009C29FE59BA'        
||'E527C80F83F4F1DC773F71220702CFD2558253A6331960E33A78B9D4413FE1EEA6D39CFF001595B7FD94B7FDB3F5BD1B5568B2CA7FE6D27FD6EE8CB7'        
||'C5FEDEF8E5BDAAFBFF006FF51EFEEC0DC5BA8FC4FEFEA9A8FEF4F5C3ED5C45361A8B69451CD54F93932354CD27919163896325F592C542DC8539AED3'        
||'9823FDD92EEB6F0C710DCACBFB3999D8B19780062414A7135C5300F47BB19B256BC103C858DA5CFC48AA00F0FF00A2EC7AA1DFE669B4AA725F333BCA'        
||'A29E02C8729B33417B05D09D67848355C7D6DA6DF5F631E57B279F962D645F3127FD5E93A4B7972B1EE12293FC3FF1D5EAB8A7D835F2EA5114770186'        
||'A6161EA6B0B2F0783ECC0ED523631FEA3D685E463CCF48DCD75FD543C69BC87E9A838BB7E01E383ED05D6D0CBC38F4B21BE5271D31E37ADFEEAB29E4'        
||'AA8A3658DD1C878DDA42D19B802F6FA5B8BFB62DB64134AA640081F2F4E9F7DC550123AB6FFE5A1B7E887CD3F8A11D440EA60EE9D9759118C98E213D'        
||'35619E944CE3E881956E0FD7E9F9F677CE812CF907715A50B5BC83F6A9FF00263A29DB5DEE77FB735C788BFC88EAE2FF009B5E5F676D5EE7D8153BDA'        
||'8775EE9A57EA8C64D8EA1DB4B82823C6C51632885753D42E5A0AA7699D453538946953E06B26A43EC15ED0C972DB05CFD2058CFD43EA2C19B577BD29'        
||'A4AD05751A64F771C8E94F36C43F78421CEA1E0C7402829D8B8CD6A460571F671E89FF00CA8F95DF15BADBB8B27B2FB17A93BEF716EEC1F5AF45D565'        
||'735B4B746C1C06D7A88F2FD1F80CB6228E8A933589AAA959A2A59624A85698DDD5997482B65FB0DF6F46C0FD1CD0A4425B8203C323B66E6626AC2640'        
||'493534D228306BD52E3698E6943B54B154E1228188D056863634F9D73D14E7FE617F056923603E38FC95AA21D998C9DB1D63086208F578E2DBA6DC5E'        
||'C41E2E3FC6EB9B79E6019FAA807D96B21FF0DDF571CAE0D706BFF3587FD69E8E27C60EE1DE7DF3B177B776FC2DF8F7BEE6D9BD5792CAECCDEDB4B787'        
||'606DDDC9BCB31BA72B4989CED24BB170F84A5C7356CF251F884B04304CEBE15D6523916D8E1EF1DFDD5EEFF6AF7522CAC96F4AAC6630078AE694324B'        
||'5FB6A3D29E7D49BC8FB69DBB6E9E13F8A5AFC5ABF028FE04FF0001E869A6ED5EEADDE9B1B1398E98DC7B8778EE4CABD16E3D852F5B50EE5DC5D753ED'        
||'4467CFEE6CA6222A6A831D553CC69E9D649DCF8EAAA613A9E58D41892B81D0D286B4E87FDF196F895DF3F1E727DEDDD3B397B5B69EC0CA56EDAEB9A1'        
||'CB66773E1696937E4D908F63EE1ACFB5C1D5D0CAD34590923C3C952CD23410C390F0DD9DC9AB51B3D5863A65C4F5EEDFA7E856E8FF008B7BA30B4B91'        
||'C8ED8AEC26D5DC7B9B3ED5B8D9327BA73AD99DF595DCFF00C262A8A98A3A5A2FE2939904451237A680DD97C926FCA83AAF1353D22761F5852D7BF696'        
||'E3F9E9B53AA329B33616D2183DA391DC91E1F73E3CE36B4D46ECDE3BFF006DE790C72E3A18E3A4A78E59FF0062B22F0D98289083A39F8BCBAB0FE8F4'        
||'573AEFA4F17D91B170F55F147B8AB7A87A6F21D9136F1CEED3AF9ABB7A6F4DC341F6318C1ED2933D5F590B51E3B6DD2CD93A869B2465D590CAC81DDD'        
||'A92173AA7983D6C9F23D43ADEE1E93A6EEF87A4BAEBA171FB0B3FD6FB8EA71306F11B522DBDDBFDFDB97772C0991DE09BD776410646B30D54CF2FF00'        
||'B94C8D4C895C91492AF8689238E3DD457AD1074D7A70E97E9DEF4A3DDDDEFD05F209B69653AAF7F6E3ACDC5DC7375AEFDAEC9E3F716D8A5C60FEECF4'        
||'5C7B9D29B1954AF4B0CFF759F812086314D513CB348064807D0AF06EB7514A8E86CE99F8A5B0F2DD82D1F407746F6EADF8DBB4EBAB769F676D3D8390'        
||'CB9DAF96DD86B24FBBDABD2BBF321929A4C34D0D3078771C941038A33E34A0969EA5A511EE9FC3D6ABEA3FD5F3E90F0ECCF80737756F2CC750ECDC7F'        
||'51E27A2AA2AB6BF666FF00D8DBC6BB6B60FB5B329409366B6DEE3AA9DEAAB249317354D39ABCFC551FC52BAA454D0C73369594EB1E58EB793F175EEA'        
||'CF97FD09DF1479FC2FF75E8769744F586ED836AEDBC567F670A7DFDBCFB0EAA169F115F87DA73C22AB0B8DA78FCD252D38865CA656A2A63352618454'        
||'D3D56C1AFD9D79853AEBB0F6B6DDEE8DCF5FBD32DD57B8BA9638E4830585DBFB81707156E6B038DA38AA287774747B76A6A29A923AC79E58C521712C'        
||'4F03ACA91B028B903ED9BD396694FF004697FC09D209E4649348F4FF003F495A7E81D831124D2330B5CEA00862780A035FD8FCBFC8755599C9AFFABF'        
||'C1D2961E87EBC5D0DFC2E36FD3766869886E412394BDCD8DBFDEBDD3C43E83A70CF21E07A50D374A75EC5E32B8484B795058470724A953C841F4FAFD'        
||'3DD0CADF2FD9D6C4F2538F0F9F4ABA3E9DD86A351C1C67EB6B3051A413F42A0FD07FADEDB33B8FF8AEBDE349E7D067D99D5DB067FB1C20C0C456770E'        
||'EBE42A4EA3F460BF8B0F6E23B3AF767A536F34A0EA53C3A21141B1E9BB7BB3F3584D9B46989DA3B563AAA1A492989613C3492BD249509371A64AF9E2'        
||'9195FF00B14F10B726FECFC38DAEC415C4D30AD69955F2FCFD3F6F97455A8EEF7C65B925A084D29E4C7FD9E27E541D176C6632B303B4BE42D16D3A8F'        
||'3FF0FCCE023C4E458006B7234BBDAA68A67BD672D198C2A0327A48E6FC9F646ED3787AD3E234A1F3275678FA8A753BEE8D74373E5696D90444A8D2A1'        
||'4280C5A3D540000015A79533D34F6EFDCD4F7D6D5CA52657210D65050EC9FEE24546D0363E6C5CB91ADA7ECCA5CAD34C51E32693D504A1246670B1E9'        
||'552B22249D25FDE31D38E299F2A9D58F9F436DDE3DC47385898CD23C63CC825B5D3CA94E35F97CBA1ABE26750E17B6F647CAFCDD3C7B4F6D76274876'        
||'CECEEC8C56E9DCD579AA5CC6E9C74BE2A7CCEC5C053AE8A69BCF8E669044278A556BB26A76054BAF2EBC1DC4424355E5A5479768C91E9F31F9F40AE6'        
||'C4BC3BF5C0695BE9CEB568F046AF0C32B518D00385AE40F3E82CD9BD5DB6A6AFC061EA72D5F4B90C'                                                
;                                                                                                                                   
BRANDINGBackgroundIMAGE_2 raw (32676) :=                                                                                            
'C8C36436BC14B5B17DCE5333BDF058BADCAD152C0C42942238404670141E4D8FB59682268CB4C74E966A7EC15FF0751F6E525DC57016D143FE9287C5'          
||'68A0B5327867CE9D21FE6E7C5ACC75C751EE3EC6C961B3D4F0D051CF401F20986F12CB9E45C3C1A5A9EA5A50C2792228115B535AE2C2E136E5F48D6E'        
||'CF138660380F9E3A456735DC928495348AFF00ABCBCFA0A709B79B6F6D7F8EDD7AE7F72871359BD3291816F1FD95098E332A1FED19EA837FAF73EE44'        
||'BD87E8766DB7693C40D6DFED573FF1A6E8F3D9FB65DDFDC3DC77702A96C0440FCD540247E669F975C1C4D26C0DC556CC3EE3B17B4BEC29891A9A4C7D'        
||'0D6438D6278E5445148D6FE87DDE3ADBF2B4B21C35CCA69F61609FE004F4B37263CC3EF7C702E56CC2D7E59A9FE487A5234C98C97B637133294DB5B1'        
||'E9B074D6FA2D649492D4E8523FAF922B8F6EECC05BDB5F6E0DC234A0FB5549FF009F874A3DEA9DEF39A767D862C92EAC47D86BFE41D49EBFC408372F'        
||'5B6265D27FBBDB52BB2F53F5D22414B163D5DBFA12647E7DB5CA9016DCA3AFFA1C458FDA68BFE7E8CBDFCB91B7F21C1B621A195D100FB053FCBD191F'        
||'8115ACFF00267A6EA8C8639F74EF4EE3CB42C8583B45275BEE4969CF1CD8C6A2E7FA71EC2DBF4DE2453DC31C34ACDF68D7A47F90FE5D05FDE4B45DB3'        
||'EED6DB6B10ACEB64057F8BEA2197F3C2B7F87AD997AE23C9C7BF36A54BD54D198B71E2A46F54A8404AC5D7A6407EBC71A6DFD47F5F71F732DC5BB6C3'        
||'731A8AD6270335E2B8C7F9FAE79724595D47CD963348694B88C9EDA707CD0FF9A9F2E9E6B71796355395A996D1D4D4F3E4721CF90D9B93F43FEC47E2'        
||'F7F64F6DB9DA2C2A08E2A3FC1D082F364BC7B872A4503B1E1F33EBF6FCFEDAF093B728329FDF4C5B9ADA851161B2CA3F75918311195D254FFBC8E7E9'        
||'7F75DCEF6D9F97A68D5454CB11FCBBBFD5FB7A7365DAAEA3E6EB69246C2C130F206BDA470FE7F9748DED5C264E7DF18AAA5AD9C8A7A19CE825DD8196'        
||'18B966727E8179B7D3D88B92B75B783972E2DD972CEBE74F36FF003F412F71763BA9B9B2DA743811B79548C2FAFD99F4A74D3251D7D1886696BDD011'        
||'617D47512962580205EFC923DAD4BD8AE35451A57F3A533FEAA745D26D72DA689E571E83B6B5C7134C57D4FCBA9152F90CA51C3490D5319A32618D75'        
||'B6BB6B901D401E0313C9FF0061F5F6D5BB456572D348A406CF1C7E1FDBD3B731BEE5642DA365D4BDA319A7771F41FE0F3EB580FE619DCF4FBDFB977E'        
||'D647907C8E03AF71D16C3C0C818CD4F3C780D71656A6941E04751907AB941BFA859B936F790D249172B72334528D133C66675CD75CF9556F9AA69535'        
||'FE12075277B1BB14E6CD79818556EA562AD81FA308F0E229E747D3E2003FDF9534A9EA90F35B8BF8C55C4554224B3338B2946E38D1AAC41B9FC7F4BD'        
||'FDE344F70D3B6A6F3EB2654B035CF4DF93A7822A715C2088C2CA140544BDCF25547F4B8FC7F87B6997B6A3ADEA03CFA63825A0AA2169A920FB87D11F'        
||'EE207E59BC6889A81BF3F51F9FADFDD0D3AD1341A81E1D6CFF00FC95BF9774F95DD187F95DD9B888CED6EBCCA524DD718BAEA589A3DC5D8EAC924597'        
||'304886F498089FEE11AD635AD00E7C2E3DA2DE3764DAB6D6B28BFDCABA561F38A220866F93C9945F30BADBF84F41CDEEE6492D8408485665D7F35AD7'        
||'463F8B05BCB4E381EB6AADB78FC93491482A67F463D9A57677E65350E6466170C49BDFF1FE3FD7DC6D7D710D08A716C7D9414E82BB4D84DA95879267'        
||'1E75353EBE7C3A2F9F2EB19946E8DECA2F3CEF1FF08C30789A491C5DB73D0FEA0C4EA03FDF03C1F726FB297100F723690067C497FEAC4BD461EF9EDF'        
||'72DEDEEE8DE5E12541E1FDAA71AE29D51A478B9C00ED4F222791955ECECCC2DC298EC3FA5C7161C7E7DF450CCA7B7554D33FF17FE1F5EB9C5F44EBDE'        
||'534A12687249A8040A0E27048000518F307A153ABBADBFD206E58F195130A3C5C4A95594AF010D453C68D6D3179B8D5239013D3CF3C707D85F9A3984'        
||'6C1B6FD4C635CADDA8BE44FCE9E4071CF5297B5BEDEC9CFF00CC476F9D84769180F2C800D6801E0355402ED4D069522A78F13AB97F8A7D694786C551'        
||'50D5E5A9F73D5D34AB254FDDB552C750F30923A9A8A28805BC7195529AD4585CF3CFB876D7DCFE6296EE49A6446B756074E9A54529A431A9C9F3A13D'        
||'65CEE1F76AF6F60DB21B4B39278AF5D0AF8BE216A396A891A31A41201029502993539E8C2F51FC5DEA9DA0DF759BC645BD322DE3A887239E45969A95'        
||'625FD106352F08B31D459C31FA58F1EC15CCBEE3F336ED58AD643691E4158CD09FB5BE2E18A0A7F3EA4EE46F613DBFE54D33DDC03719C10C24B80195'        
||'3D34C7F00CE6A41249E381D0F1BF3E19ED3EEFA148315B3E936EE6E08E0316E4C16321A49D1634D1043554EBE38E44373A830BF00DF8F618D83DCFDE'        
||'39566D52DCB4F09AD62918B03EA413520FA53A15F3EFB1DC91CFB66236B38ECEE4529710A2A38A700401A594F9820F0078F4B2E97FE5FDD6FD274F5F'        
||'9FCF653FBD9BB5A8DE966FE2D414029B191CA0199319404CCAB3336951248E4E9B8B0E47B49CD9EEFEFBCD452D2DD3E9AD83540466ABFA6B6C12295C'        
||'014AFAF4C7B71EC1F297B7ECF7B9BEBC9174F8B2A2512A6A7C34A10B902A6A4D00E03A395B0D23C1E1E9292498554D0431C724C608E1123A8B6B68E2'        
||'01549E7802DEE2ADC9CDCDC34806904E056B8FF2F59016F6E22B748C9A90295A019F3C75372797852793492CDAACE74FFAAE4820FE47B6A3898AF4F8'        
||'5A52BD262BEBEB50487EDE55B91CE872B76E47238FA7E2FED547121C13D79A400517E5D2332293E48DDE771A4826304D8F1F91FD7EBECC61658B007E'        
||'7D23995A5E27FD5F3EB9438010247598C75FB955BB34EBAD758E180D5FEC4016F7A375A8F8730C7CBAA9B5D3DC98FF000F5C66C96E68ED1D4CB1188B'        
||'5E20823678CE8B58C9FA8293CD8587F87BBAC7687B9467A64F8EBE94FE7D3A47535A68A08AA4C6BA65F278D9B99109B36927F48FF7BF6C109E2129E9'        
||'D2854200D7FEAFB7AD50BF9EBF824F98DD712469095FF65C76296755D52DC7686F0B877FF52000540FA73FD7DE577B16D28E4B9D549FF7325F323FD0'        
||'2DBA26DDA58609F548C1404D47E4054938F403AAB9393A0B922AA3B5CFD03DBFD87A7DCADF41BA0C686FDBFECF4061EE4F229E1B9474FB24FF00A03A'        
||'31BD5DDA7B1B6DED48F1998DD54B8DAE5C9E46A1A9A45AF2E219D94C2F782265B3007F37E3D8277FD977A9B7132470B91A17351F3F9F52D725F3572F'        
||'EE3B18B9B1BB59233248350D54A822A32A0E3A536E3ECED99BAB18D87C06E9832992927A7A94A38057AC861A562F3C979E345B20B13CFF00B0F68B6E'        
||'DB376B1B913DCC6C88011524713C3813C7A5BCCBBA58DD6D4D0C12876D48682BC01CF11D24F19455793ABA7C6C534A8D592784BF91C88D1949964B5F'        
||'FB2B736F67335CB431998B1EDCF13D012DA233CCB027163FCBCFA39395EC9DF99814A959BB73C94F4349494547474394AEC7D15353D153AD2C022A6A'        
||'49145F4A0D4CC4926E49F6038EDA08892AA2AC4924E4924D4E4FDBD0DA3892350ABE5D0A7D4FDC3BBE1CF62F6D6773B96CC62B2F530E32924AEC854C'        
||'F598CAEAA71151BC3552B16689E42A9223B102E196D6372EDCAC6330B5C43D8CA2A69C081C71EB4E14E9B9A252BA970475B34FC2DF8FDFE90F0387DC'        
||'5D85435276BE070D86A7A7C54D3953B8F2925309DBEF2485F5FDB429A59D6EA6466517D0183427CF3CDEFB35BA59ED4E3EA26A92F4AF86BC315C6A63'        
||'C0E6801F3A5204B6E588F7AE6CDCAF371526DE29DC05AD35B124E699D20711E648F2AD6D166EAFEB79F1670B2EC3DA2F8A31F8BECBFBBF8B10841FEA'        
||'74C4083FED40DFFC7DC2CBBF6F693FD52DDCC24AD75788F5FF000F43F3B2ED0D0FD39B58B470A786B4FF00074457E48D5617E3B4B84FEEEED435589D'        
||'D0B5FF00C1E9A29A9E87158DAEC718DAA68AAA540D3380932C90FA2FA750D7E9BFB91797F7B9B7B818DEB933454D4789607837A038A1A6381A67A8E3'        
||'7EE5E8F6BB95166290CB52A3C908E2BEA4798FCC571D107DCFDDFD89BA126A66CC9C163A6255B1DB743E36368F90239EB159AA65B0241D72D8FF0041'        
||'EC461110D556A7D4E4FF009BF97454B028A54E3D07FAAA7F6F410B3DDDBD4649663A9CDDE59E577362C7EACCC7FA9E7DDC43737A74282581A633F675'        
||'E6786142E5801F974ABC36C3CDE5B5CEC20C6D3836F2573133BAB8B9F1D1C576FF0062E57D9AC7B5485145D35386064D47F2E8BEE375455A42351F5E'        
||'03CFF3FE5D08F89D89B7F1F2C6F5CB2662504006B068A5BAD8F14709D247FCB42FECCA3B4862EE8D457D4E4FE5E43F21F9F4552DDCB3295624034C0E'        
||'029FE1FCFA59EE27A2A3C19961860A7829A3B08A08E38635502F7548C05F6A6D159EE02F127A433B911B127A2A596DF5471D6983CE05F5B2FA81045F'        
||'D8CA0DBE429A80E8827B98C3697CF46C3E2A666944FD91D913E87A4EB3D8F92CBC4CF668D727554B2B40C49E011141228FF837B0373D46C20B5DA97E'        
||'2BB9829FF4A08FF291FB3A13F2A91F513DFB7C36F1923FD3107FC80F5595DFDDD583DCDD7D53B405679EAF2F264B3192AD0EAE6B321573B4F909E4D4'        
||'6ECCCE6CB7FEC81EE56E59D827B0BC5BAA7C1A540F40303A056F5B9457B03424D7C40C49AE4F540BBE31D3D6656531292892CA18DAF6E401623EBC8F'        
||'C7BC96DAAE121B7EEE269D6207396D177B85FA9B65C296A9FB69FB7874C543B32AAA00721B4B5BFB1C0FEA58FE3FD8FB5136ED1A7A0A745367C91717'        
||'4002CD53E9D1F4F877F153B6B3BDABB03B4709B1370CFD7FD7FB9B19B8B74EF19319352E071B8DA22669E6390ACF1C73B8162B0D3B3B9FC2FB8A3DC5'        
||'E78D9A1E5BBED9E7B841737513A471020B16380282A40F990075903ECDFB5DBA5B7B83B26F71C07C1B4B98E47738C035C93C4E300757E34DF6484148'        
||'A6A9627F54CC218CDCFE238EEDFEDDBDE2136B3C481D74C0D7874A6A695C52B7892287C3327F9B894D9255209BC9737BA8B9F699855B39A8EAA4D38F'        
||'52965A871779A523F00C8C00FF001207BA517C875A00713D7FFFD0F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBD'        
||'EFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDE'        
||'FDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEF'        
||'DD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAD867FE134DFF0065D9DABFF8A9BBF3FF007EF6C5F7237B63FF0025E9BFE79DFF'        
||'00EAE45D06B9ABFE49C9FF003517FE3ADD6F09FEFB9FEBEE75EA3FEBDFD4F3F8B71EFDD6FAEADC7FBEFA7BF57AF13D60A9AAA6A285EA6B2782969E31'        
||'779AA2458A25E3F2CF6FF6C3DFA95C0E9C8A1967711C2A5DBD064FFABEDEA4ED4EFCA1DB78ACDD16D7A08B335F3D7C0E95B90157062A211D318DB4C7'        
||'0A896537B15E554FF5F649BAD84F7177199233E185E2192B93FC24D71E7E7E83A1EEC7CBA82D5A5BDB958DC9AF861198E302AE01515F96AA7991D079'        
||'B97B37B2376B3AE6372D6454042BB62F0AC7138D611F2A92474255DAE7F579643FE3EDC4B3B1863D102E976ED0CD42C09E257560902A4501E15E84B6'        
||'9B7DB5B4BE208848A9DC78B023D18F902680F0E34E8313427CCF5527AEA9DB5B4F27AA4D6792C0B7E47E2F7B7E07B581634845B422918C516BFCC8C9'        
||'AF99AD4E4B1CF4B8CD2C931B898D5CF9B0141E945A69007E114A0140A283AE5252255ACB29119AC8E3669352190CF08E5A544234874FED587239FA83'        
||'EEB468956389182570148403ED35D441FDB5E3D6C48B239926914B11C5834873E829A47CBC87974CB35399081E595AC34AAAA08C69FC2844205BFD87'        
||'B7A28CA0A8890126A6AC58FDB52A7FC3D6E5B8D58699D80141440A3ECA071D409F0CED148DE0942E9FD462551FEBDE4651ED589E60405D3FEF47FC8A'        
||'7A45A2DC83ACB0FB141CFE6E3A677C253153E4AEA784DB859A32FABF3A755219483FF20FB6DEFB7043DB6BE203E692AD47E5204AFEDE95C56BB5CAA6'        
||'B78622380785E87F38CC847FBCF4F3B7B19154D167F0D3565254D04D88ADC8AD3B89DA2A1AFC701514F90896A11156E6F1C8411A95AC7F1EC15CE575'        
||'FBBAFF006AE63B78A486E96F20B6660AA4CD6F704A490B94662FA712C633A592AB4A9AC81C8B6CDBAEDDBCF2BDCCB1CF66F637176A0B9020B9B55124'        
||'53A09154216CC321AA874928F5A0A2166DB2501A8A36A9882A966FB78266BE91790C3242DC8039209B8FF102FEC78773DBAE58DB4EF1B91F864A03F2'        
||'AAB81FB69D47A961BB59A8BA8A39230783C7523E7DF1923F9F4A3DB5D8FD89B31D1B6D6FFCD52471E81F6154D3D7E399636D4B149459059A32BC9160'        
||'071C5FDEA4DA36C9726DC2D4D6A874FCAB4520797A7486E257BD1A6F29350115914330FF006E46B19CFC55E8CC6D2EDDDD1DA74B2A6E9A5C34559B66'        
||'2A5897218959E9864972924ACF255514ACCB1327816C623A4DCFA45B903F34ED7159180DB1621CBE1B34A68E0471E3E7D24B216BB549230A81205E27'        
||'034EAF3E3F8BE7D2BF46A00860CA4F0411C31FA8B8FF0079F61131303420F4709BB5BD350714A7AF58240A96FDC5BF1C161C30F7610487C8F4D36FD0'        
||'572C07E7D33D711A4A865BF27EAA6E6DC5C7B752171E5D219B7785C921A8483E7D03BB9EB18A489E9D23FD4825AC0FEA047F4FC8F66705A8075641E8'        
||'8AE77795D444482A3F33D169EC18649E86A9C441C49055472C7F6F535424D74AC02FDBD11595EE0F31C24484F08431041B5AC5DC58D781E8AEF6E15F'        
||'420A5095F4F51EBD5577C99A77A26F8BD4628C5108F6C76823501C76E1C30A48E3E91AA055315BB669F2D4E01254C1929A4A94BDA6769351251BA1A5'        
||'B5AA7A07F223F0FA1C8FCFA15ED2A7EBAF253E6545707CFD571FB3A0BBA294AF4D7CA20001E4EA5EAB4FE97FF9C8BDA8D6FF0078F646455D3EDFF274'        
||'7CD4AAFDA7FC07A95F1ADA9E8BBFFA56AEA3C87449DF29098943CAB23ED8DB3730AFEA2C74A82154F1CFE3D9C6CCC53738CAF12B201F9A81D17EE70A'        
||'CF64CB23055564724D4E109622832700E3CFAB8FF8DD8CCBE23AEDBF86EDEACA086A7766F599E21B4F2FB4D9E41BA6AE59AA97139AAAAD9DC4CF2793'        
||'EF7CC23A9BF9628E289923519DBEDF6F756EA25214C65C50A95E2D5AD2A7CB35AE7880387513EFD7DB9DB6E4F71640B24EB190C08624283804014A1C'        
||'50E4702491D187155B8A240DFC35CBBC7246AEF19366760D73A4FD05BF3EDF8F69B12D89052A381F218FDBD10CFB86FA001242DAA84641E2735FF357'        
||'ECE996B8E527935363EA15E3F1B34641F1B3442CC4B1176D47E83E9ED545B6C28A57C55A1AD0F9E7CBD053A2F9AE3709A40C6160453143434F3F535F'        
||'D9D22B278FC8D54CE648A440FA81406E471C0BDF8E3FA7B590DB59C118D4C0D3A6C41B85C4A6A8C35797FAB863A0FF0029B79D253E408806A3A8F02D'        
||'F9BFBBB5D5AD351E03A368F97AF64916355C9E917B836FC3262F2710F0B8AAA0AC8B4EBF43F929CC61750FA024DAE3D91731DDDB372E5F84E26DA7FE'        
||'713F42DE53E5ABF839AB6E7923202DD415F4A2CA87A2AB16C4A449A7FF0071D483F71B915121E55CDFF3EF0C0B13C3ACEBFA7A1C81D0918CDA54E598'        
||'0A28399231CCCFFD7FD7F6D163EBD2C8E01E83A1A3AFB6F50E17786D6CAD45152A5263772622B6A58CCCDA22A7AD8DDE5D2C79D006AFF61ED1DDAB4B'        
||'6B2C69C59180FB687A3CD8FC1B3DDED6EE703447346CDF2018127F2E3D5AE5C1B32B2B2B00C8EA414656E55D5870411620FB8D3EDEB2B6A0E41A8FF0'        
||'8FF67AF5C0F533040A0B33B1B2AAA8259893F402D727DEB3C075EA851A89A019FB3E7D13BE9E3B661F905DA7938E8E829B3FBBF6FD27D8E456C2AF21'        
||'86C467EA6B92815C9B154139A808A2FF00A8F207139F31D96E92FB23B4CEBA9ADACB71B8122FE1479A08955CFDA50A54E2A40F315C7BE42DCB6CFF00'        
||'5DFDEA21A565BEB64684F9BAC53CACCA3D7B5C3D079027CBA3897BFF00B1BDBDC194A75909D151F9B999C162BE31F6852674D3BAEE6C6D0ED9C551D4'        
||'152D5D98C8E560969A3A78C9BB344B13D4311FA5632DF8F738FDDC36BDC373F797657B05245A48F712B0E090C713862C7C83165415E2580EA0EFBC8E'        
||'F365B37B31BCFD5B847BA8D2DE1069569A4950A851E65555A434E0AA4F974497A7FE34F4067A388E6BA9768648B5241231AAA7AD6BBBC0AECDE99C7D'        
||'4927DF42778E73E6BB724417F2AD09E047AFFA5EB9AF0FB73C892C624936A80B30049A36491527E2F5CF4AFDEFF1E7A3B6BD254A6DEEAEDAB88562D2'        
||'B0A3A7AB50640BA43FEE4CDCD801ED15A735F325EC81AEEF64908C648FF374610726F2AEDD1B476161144AC6A428393C2B963E5D176EB2DB5B7F17DD'        
||'5B165C6E228E8E4A6DC70C903C2AE0C4E29E50193513CD8FB3FDDAFAF26D8EE165959814CFCF23A72C767DAA0DC22922811595AA08071C7E7D59F7DC'        
||'4DFF001D1BFDE3DC5153D0CFC38FD3A8CF1C72333BA2B3B105988E4DB8F760EE0501EBDE147E83A6EAD634A85E9C985C291A938363F8E7DB88CCC68C'        
||'6A3AD18E31F84741767F71E7A9C49F6F96AB8783FA1D0704588375F66D6F0C4D4D4A0F499C01C3A29BB8F1D414A2534D490C245CDD011C937FC9F62B'        
||'B562FF0011AF49A5B89D7839E802DC39ACB51191692BEA29C5FE91941CFF005E41F622B6B5B67A17407A2B9770BD5F86561FEAFB3A0C1B37BA370D67'        
||'F039B393CD4F5493B3C159574B4B4D2FD9C2D5C88D3D478D03168C78C338BBE91F9B7B3B82CECE12268E3018702012738F9F5137BC17F7D37B79B845'        
||'2CACCA7E9EA2A3FE52A13F2E1C78F4AA8A0922A6868A3A77A797C5A64A1C808E39649E50CB15442B56442CB20BE9911AC4DEE5AC2CBA8DA6AC4D78FE'        
||'5FEC75852B242C4C51286A12BE5527CB071C33C7F6F4F4F454B95C2D162B2B9B4A38F1BF729153255348680CB2ACD2C3F66AC4307692420B3065D3FA'        
||'F4D94D0382D915FF00571EAED6EAA3547DAC4668787C8F9D0D4E307A55D03E6766ED48A36CC52EE2EBDC9B0957172D035761D3210C851A7AF82AFCDF'        
||'C3EB4F8F8902C6CCB631B953EDB6589E42E01D6052BC0D3D314A8CF0C8F51D54295511C8540275672350186AE6871E9E94AF50EAEBB6A6F8C5E3E0CD'        
||'4797C0D2634E4D30993A7FB7CBCB471D7D41AB7C6432D64F1D4B51A485A45A5AAF215BB986488C8FAF6A84121456B4A8C530295C0A5698C53CAA0D05'        
||'190A4832A10092D935AD1B88EF2315CF70AD4920AE6A9C9BA733B0435155B5F796D5DC18BAB708BE2CF7F776BDDB5DD692AF1B9FF008DCF3E859E452'        
||'05C137B7BB160594B6A5A7F4491F6D56BFCC0E98B733461A1D23BFCF58D38F4040CFE669EB5EA51C17636217FDCE61AA6A6314A291D2A6A86501A65B'        
||'0568DE8A49245D1E9D256E00B585B8F770C69A636041E23D7FC1D3CAE8A6B3D548C7F481FB3247F9B3D40A56AB8A6817EE68689219F52CF96865869E'        
||'926E0456CCE196392362C08F2B423EBEA37F6DE82A09404547979FAE0FF83A7E67034BDC11205355D40769F2A30034FDB407A1CB11BE732291A0CB62'        
||'B099A8E4C7CB49354C59D9F34D3CB6F11AA5A6CF091D1910041A94230B685B73EDA10075A2B6920D462847FC59CF5602196413490921C777760D0FCB'        
||'481E95F3FF000E3C76F4EAEC5E32BA9B7475AE1372D3AD5252C75D434787DB75D8981A068E6767C1C14E6A6A0310E8B34B19E0D9813A82531DDACFE2'        
||'4729A11906AC09AD6BDC703CB00FF93A5D70B692DA08D53C1D2D9D2421A50D14955351520F91F9F49BA2CF758623F8856612B851C4D052494F419AA7'        
||'ECB79F215288AD1F8EA3039BAC681D0860CF355B29B03A17E9ED635C34B45656526B565111D3F2A32A823EC5AFA1E8846D9796E0CED721D7553C2264'        
||'1515C3094177AD0706620E6A07996DF929F1CB23F2A68B66566DDED3DB6D5B80AFCC4F5986C8E237B57D5E0E97310C10D151AD4E699E49C33412B871'        
||'3CAC2E15980D2007F7EDB137458A003C21083DCCA08727891A4E0E322807A79F5287B67BA3ED8B76F77FAA642BA42B02516A680F62D73C092E4F99E8'        
||'AC47FCAF379C6F0BD47666D931891418E2DA7B86492E42B69D111620D8F208FC7173EC813945037F6EA699F87FE861D4A9FD6C87FDF4DFEF43FCDD1B'        
||'EF8CFF0006B72F5F7787476E7FEFCE3EB86D4EDDEB0DC069A8B6EE5A964230FBD28EBA58849330D3C47A4B11C137B71ED75EEDF0DBED17485EA3C196'        
||'9803FD0DBE7E7D263BF25D5C2858C8AB28E3FD21F2E8F1F526C4A8D8FF002C7155F16231198ABAAF91B922478EAA1AC7C3557615553CF4CDE4919259'        
||'60C7E8A789E3853C86204F321F76BD58DB93A7EFD21AD2B50299F0815AFE7404FA549E8BE098B6EF1269274C94C9FE91069F6E481EB8E8E2FF003C2A'        
||'9AEC76F3EA8A0C365A9E187716DDEDEC3556E095923CD6571F4F9DC7E5F4D24C41545A9F29A89CC607257D457D3EE32F674B49B5DDA1013FB02403F0'        
||'9F0DC507D9A69F9742EE700897C8D52C03499FE2EE5353F6D41FCFAD71136252CF51022D2F91A109190C9FA7C6355F8B7205BDCEB05A472468695C53'        
||'F674097B8742457CFF00C3D29D760219E245A606CA028119BF3C8BFE7FD8FD3DA9FA440D403A63EA188E3D0BF80EAD4FB24926A49199F93E80AB1DFE'        
||'A091CFF88B7FB1F771E0C7DBC7AA932BF77468FE2175EA527CA8F8E3928E029151F75F5C4C8E6320B5B7453A86D24FE09B7FBC1F61CE719623CA9B90'        
||'A67E966FFAB6DD2FDA15FF007A5B54FF00A2A7FC78756C1BB3A253B87E1C7C6C831B8ADCD999F6C764FC897AFA4C0D1FF1134994ADDC74B0A43508CA'        
||'4D3C72C5144E8801F533B7E7DC6967CC0BB373F6EFF50F1C625B7B0D25C85AA88DEB4FE2209209F40074773594973CB560604690A497418005A84BA5'        
||'300601001E1EA7A2B0DFCBFE4C8C9533CBB27B5A2722400478830C274C46446626909B91652B7363CDF9B0179E7FB34007D4DB9FF6E2BFF1EE893F74'        
||'5EB7FC4793FDE1BFCDD611FCBDA6447F0EC4ED4F56865F262A56D28600E75B250106CC0FD3F1C7E6FEF63DC0B10456EADBFDED7D7FD3FA75EFDD1B81'        
||'FF0088F2FF00BC3FFD03D4A93F97BC8D0E89B61F66D4DCEA8CC58AAA465260B4A7D58F1FAAC0D872093F5F75FF005C1B23C2EADC7DAEBFF41F5BFDD1'        
||'7E3FE23CA7FDA3FF00D03D188F8D3F1269BA468BE466FA936C760E2525F89BDE5829A7DCB0353E3DDF27B7E9E46A78192929C890B2AF8C6BB9B32D8F'        
||'3EC27CD3CD70EF336D36104D0C87F79D9B011904D164392351C0F3C7A1E8EF65DBEE2D92FA79E2745167702ACACA3283152067F3EAB73E70EC99F3BF'        
||'273B7AAC41196932B826BE825CC71EC9C6425DF4DF8E3F1FEC07B91F92648872A592B792BFFD5D93A23DDD5CEE73328E247FC757A23B59D572C550DA'        
||'A485741F568A72C49B6AD20B587D3F0071CFB14688D8EA53FCBA42242B865AFE7D20B72F5BA42469D41CC4EFAD6046801B0F4A5AD63F96B13FEC3DB7'        
||'2DB230C7F83A7229DC1E1FCFA48D36D0F151A538A71555CD294631AE9F12DF500ADC86B0E6C5BDB705BAA2E922A7A79DCB36A181F3F5E8FDFF002E4C'        
||'0245F32FE3889F4472D2768E12BA28A412092418F826AE95A20BC591539D4DCDEDF9BFB0BFB8CDFF0020DBF4FF008438FDB41D2DE5FF00F92C5BBFF4'        
||'C7F94E7AB3FF00E6A7B373BBC3BAF1F16D2C54ED478AEADC3AEE193EFE9A855ABA1A39EB1ABE99B24E2506774AA90F80AAE8F105BF03D837DA0BBB7B'        
||'5E5F7FAF7CBDC3E8C126848143A45300A8CE6B5E8C39C159F71516E3E1892B9F30B5273EA7570C529D168F95FF0006B21DD7DBF91ECA7D855F9B8327'        
||'B0FA8F17455F55BD452D0AD062BA8F03472C347899B234E94A7C9081314A7469186A6D57041972D6EDCBD6BB6882FA4FD50F7008A4A680DCCAD41405'        
||'45789238F99C749EF1B7459F55AE174C7C346488D0649C9FCF879745567FE57D8E317EEF4EE214D86B93FBFF004D246C1FD458467361945BD3F536FA'        
||'7279F678371E506C60FF00B49BFE81E98FDE1CC4B9F108A7FCD3E1D18BF8B7F12F37F1DBB1B60EF3C2764EC6E9BEB7DA9D854DBAB7AF53CD9CCDE533'        
||'FD9D57534D0E30458BABA29ABA863157F6F45460D4CD19558586908CE5F1A7DED3B6BF3159B6D434A1B6EEC30AB78AF9EE00F0A70C63A957DBEB9BCB'        
||'9DB2E1EF9B5B09683E1E1A17F871C6BD19AD838EED1EADEB1EF9CCF5EEEBC2F71F71764E4B395BD558ADA7594781C664E877AEE09AA767E167AACDBD'        
||'1D32D5515209EB335A675874D3158A40A81CC35903D7A1ED413E83A4061F786E9DFB4FD73F1B7E487C45CD6036A6036C4D9ECECD9B91705D751B61B0'        
||'CB86C3D5534DB5D9291AAE1A99AA1F1B8CA7AC6B4D2ACCC3F6038D7C5823AF1C641E845F8F782F889B3376766EE2D955894F51435190D8B2E0C6E7AF'        
||'CD4F4390DA95BFC4B7BD46DEC6D7BC8D0793240621A213C81DE959DC80F18F7E14E23AF6781E809DC7B4BB67AF773489F2F875D663AD320D93EC99F1'        
||'3FC6A9371526E42F93ABCE6236DEE1C25708845498CF068ACA7AEA614CF3C51C5125546086F0A9F8BAF70E1D39FC5CCAFC79C9D554F6B6C8DAAD4B8E'        
||'CF1C8B6D3D83B623A9FE0996CB52E46464CF8DB8CC94ED95ABAC0898EC4D23478F8140ABC84E630EEBE14E23AF6463A1AB6266FB6BB8F7A7623EF1EA'        
||'AACE97EC7C561771D27406F6DF7B1F6A6FCDCDD51D6399A34C0D7761A6E2A0ACAAC7E4B339CC909A6A4C5C593962A78E1B3A168AA7CFEC9CF5EC0C0C'        
||'8E89D6DDF8DBDB091EF3E9EDE7DD3364BA125D87976CC76AEDBDDE3677647646E2873334E366C8C2A6AAB36C402AE4ACC8EE7A982AE3356DF6E56B95'        
||'5DD06A87875BD59D43A7EF8B1D2994DB54349B7BA7FB97752FC66C151CB885C7642B16AAAF3594C74F27F1FC56CFDEF8EFB69A4DB40A551973C94D0D'        
||'43CA1E9E827A842D905F014E1C3AF13E6467FD5C7A153A57A5FE0E6CC96BFE55ECD8319D63D59D78FB9F374F01DD1938BACA9F378BAA9E872BDDD955'        
||'CED5D44F5159572239C3C9E5FDE895671035554B30F50711C3AD8A9C1E98E1EECF89DF28B3DB2BBFB0D8FCD5663F61D364536C6664A4A9A2C876751B'        
||'49F6787ADC8ED0C5472D756C71209BF826350264A4F314610C0CF4C7D5073D6A8460F4EB8EEDE83BD692ABB029F0595DB74899CCCED2A1C66E1630EE'        
||'555DA757FC22A2AB72611A285B1558F3A4AA714E9AE9A2112CA4CA5CFB9F7DB435E5AFF9BD27F813A4173A44943E83A7B86104AA9366FEA2DC1FAFD0'        
||'FE6FFD7D8F89E98402BD3CC718B7007E09B5810781F5FEA7DD2A6BD5F0474ED0440F8FF5310F7D248275143604FF00C47F87BA13D7857FC9D28E853F'        
||'4861723E838038B102C3EBFF001AF6D3756E8ABF77E764C426F5CB42E619B0FB57332D23053AE3AF7A134B42FA4FD0899908F6BED2332B2443F1301F'        
||'B4F4EC8E21B39253F8549FE5D243E106C5861D919FCBD4C618D5FF001422571EA34D4717F05A7B4A2C7FDD323DFF00AB9FCDFDA8DFA7ADF145E0A428'        
||'FB07FB3D25DB14C7B6444F193B8FE67FCDD56563A7AAC5E03E4BC7B152A33B1E02A258B6AB56D15549575AF4BBE6BA8A5A4A98A57F24CEB18650CCC1'        
||'980121B036F693C697C2D508A95E18E243508EA77DC771BE96F3956EE040D2696A00305BF4C1141F2FF553A59EE78B7AD2EFFC0C7854860C2D2CDB62'        
||'4C4A4B47E59B343255221DF54F0D4A48803D1D122D486953D163E23E46B16E796637029C3FD5ABF60EA45DDAF7775DFED62B750606A8271FEDF04E68'        
||'B4F234E3D217AD71F89DBBDFD4086AE0A67EC7EF5A7D9B92A5971143918F378DDB5B1AAF214780A896B2191A9D2A0BA167A764F21044A182A5B52AC4'        
||'B78B29C34869F6D070E82BEE625B45B2DCDD2AD279C69A8A8A85D353C69503862BC7A4CB3EE0EB6DD1D6585DDCB93ADA0EC1DE1B8713B57295C9150C'        
||'91F5E64365622A767D2E16B4C7F6F5B0C3490FD934BE8923A8F22BEAD03517C12A45208D721A47FCAA063F97516CD03382E4F7343FF1CAB57F3AF41F'        
||'FCB4DBD80CFF006A74FF004BEDDA6CD7DDE57349B97730C86428AB601B770A44A8047471A15F2CDE905D88B2116F66B0598DCB7AB4DB145417D6FF00'        
||'E913B8FEDE1D076EAF0ED5B4DC6E7734011091F6D31FCE9D316E4CDC759BCBB5F785369FB0D9D89A4D83B6CA0558A4ABA78BCF5D152A8000D7512C51'        
||'1D23F52DBF16F628DFEE4DC6EB33479102AC4BFE98E481F9903F2EA55F6336AFDC1C8B36FB7834BCC0CAE4F1382C6A4FDB4FCBA9D5D8C18CAFEA7D8B'        
||'22A91B476BCBBAF3A47D1723514FE20F290796779A623FE0A7DADDEC2DB4165B50FF00435D6DFED453F9B127F2E83BEC9C326FBCDDBB736CDDC1A470'        
||'A4FA0240A7EC6E9A7290492F58C27C80D776876652A885AFE59F0D4B546A58D80B145A4A537FF5C0FAFBACF5B5E5509C1AEE41F9863A8FFC617A4A66'        
||'6E69F7DFC3505E3B25E38201522BFB6941F6F4AEC4CA289FB7377A9BC7B736C516DAC7328BE9AB14ED533A291C5DA59E05B7F85BFA7B57B069B6B4BE'        
||'DC9BF02E85FF006AA58FED2475EF7CAE1B79E76D8F94E1CD6456603E67D3EC1D184F81D8A64F995F1E70E814AED3DA1BFF002B5DAAC02D464BAE7238'        
||'B8DEC48BB307A8F4FF004D47E8A7D8037988B6CF70D427C28E327FD33CF101F67E2E9EFBD0DC4367EDFDAED1A82D668881EBA148A7EC24FA63ADA036'        
||'253C71EE4C34C082132F44FC2F3A9240355C7E78E6DFEBFF005F712EFD33BD84AA7CD0FF003F2FF8BFB3AC14E538234DE6075F2954F0F31E7FB3D3E4'        
||'7A144E3239343B345EA5325C5ECC4A96274B70411F93FEBFB0778ECB8EA443691BD0D4673FEAAFAFECEB9E17191479EA397485D1475CAA5816501C82'        
||'43587FADF4FAFBF5CDC39B368EBC594FECAF5EB1B38D3738E4238238FDB4AFFABF2E9A37C61E927CED24B720AD3B282134B728ACC483F41FEC2FED7E'        
||'CF7F34165246BC091FE5E8B39976AB7B9DCE299B0403E59F2FF57A9F5A741AEF4C5C098A46460191895641E9FD3EB22D627F1F8FADC9F627E5BBD90D'        
||'FE86C823CFA03F38EDD17EEAD486854E08FF005026B8F2FB7A287DF1DCB0F4CF517646FF009258FF0088E3B0EF85DBA8EC079B7267D1F15878E2D26F'        
||'78E591AA980BD9623F81EE61E5BE5A8F7BDEECAD2714855BC697FE69C455C8FF006D454FF6DD44297BB85E4B36D5B677DCDDB7D3467C97C4560EE73D'        
||'A2340F26700A815E03AD3D7B57372D5623774B34AECEED1B34923B1767F25E496426FEA6BDC93724FF008FB3AE7FDE5EFE4BF5AD14BAD07C87CBCBFD'        
||'8EB3B396B658361D82CB6BB7002DBC288294F251E9D110C84ED0450C258891A433DEFCA29B98E2171C587EABFD49F70BD3A3B26BC3A555157A57E3DE'        
||'0902BBA2B6A0FA41BDB56A8FFE283FA8E3DEEB8A757C3034E8DDFC03F87BBCFE5D77E6DCD85B6A17A4C5452CB9FDDBB9E7A479B1BB4B6961DD67CD6E'        
||'4AC0A082D1829051C26C66AA961887EA242BB6FA6B685F74DC3FB08298E06490FC112FF498F1FE150CC703A2CDD6FA3DB6CDE6A55DAA117F88D09C9F'        
||'E10012C7C80F523AFA1BEC5D87B67AC7AE366F5EEC8A08F13B4B67EDCC76070543759255829A237ACAB987F9DA9A890BD4554DF5795D9B81C0886F6F'        
||'66DC3719AF2E72F239269C00E0147A2A8C28F41D06A6494DAA4ACC0D4027E64F9FF93E429D0D9B6A9E9D69298B0D44D3AB30F5F2A64FED8E4DCF26FE'        
||'C3F7CEE652070AFF0093A3CDAA38D604272699AF9E7A09FE4C605F27D2FBFE8E89E16ACA9C7617C4F281A548DCF42CC646FA7E917B5B9FF7A1DFB477'        
||'AB6DEE16D93CC0E95792B4FF009A12FF0097A097BA1B3CDBA724DFD8DA15F16444D25A941FA884D6B8E02BD53465B66494A8D14B0455938209FB4540'        
||'6C058D9989E2F7FA0BFBCFAB3DE04A7523685FE957AC25DE7934DB425278FC590115118515F982491407D3273D4BD9B8DC96DB92A2BE2A0FB182701C'        
||'27DC8122B427C90B18803E92D63CF3FE22FED8DEA5B5DCD5207935B2F13A78D7073EB4E96F23D9EE5CAD24F7B15B9B74932078991420AD467B49A1FE'        
||'58AF4A5A8DC1BAEA2BA7AD83333A54CA59E40B532A82F35964F1196E45C7D4927F3F8B0F65D16D9B4C702C32420A8F3A0E038569D092EF9979AAE6F1'        
||'EEED6F1959B8AEA600134A8526A4034CF1CD7ECE84ADAFBB3B70E5B0588A1ABC9D642B598F925A489E47867A65AB4A891B212ADC68650753335B4DED'        
||'FD3D87371DA39585ACD7722AA1D2C013C435081A47A83C00F3E3D0F763E68F715F72B4DB919E45D5196005414D4092E7E62B524F0E03CBAD9DB64D5D'        
||'1CB86C6350C54C23A9A5827D54EA04452588328563C9FADC13EF07AFD2459DFC426A0919F507ACF88D83AF6FFA874E992D89B72BA66A8AA92146A978'        
||'DE64709189A553A8BB37D091FE23FD8DBDD23DC2E1174AF97F21D320383DB519FF0051E9156DA94D2643191625E24A2934C957E54649D1CE83287FAE'        
||'91626C2FED67F8DB6990B83ABCBA78656A49FF0067A41FDBED3AECDC71E3464A578D958C8B4734D8F2CADCC736A504FD7F5580F6BCB5DA425A4A0AFC'        
||'C57AF02DC09AF979FF008787426536D98AA56A21A998BC13F8D02A6953169FF5008B83F8FF005BD95BDD94A15191D59633C7A43673A5EA91E7ABA3AF'        
||'1F69A0CCBAD48913FAA92B7B9FE961FEC3DAFB7DED0808EB9EAACA751A648F4FF67A01920DC10669F0B1514F3DA59142847620162178E2C7F3CFB109'        
||'FA6683C76603A4EB34CAFA003EB5F9742F62FAFA5AA83CB9015341522266D3222145720596C7927EB7F64B36E2A8DA63A30E9528272457A416E6C1E4'        
||'718B2C710358EB70116C1F4D880E97E0F36FA9FEBECC2D278A5235768EA93332A191334F2EB514FE75EB5D1FCB5D8A990A5A8A491BE3E6CE9A24A9B7'        
||'94D39ECADDD12E9D248D2191B4FF00B1F797DECAF85FD5297C22180BC92A470AF836DD01F7B795E299A552A7C17C1E34D2FD55213F5FF5FDCFFD60BA'        
||'FC23EC1D32D77FC083FF00044FF63F5F647B87FB93F90EB2CBD9AFF95257FE7A27FF000AF4B7EAE17DDD4E3FEADD92FF00AD439F643BA7FB867ED5FF'        
||'000F5265CFF65F98E8DA6DCAD8F1B99A0AB9CDA1495A399CFD122990C4D21FF05BDCFF0087B085E4466B678D38D31F32334E9BB0992DEED257E15A1F'        
||'9038E87EB7F5FC1E41BDFE97FA1F614E870339E84CEA1DA392DE7D85B6F1B8F82578687274399CBD5A231871D8AC6D52554F515122F0BACA08A217BB'        
||'3B003F362FDD2E52D6C6491CE482AA3D588A003ECE27E5D372BE88C93E78EB708F807DD589DC180C8755E423A2C4E73054F435D8088D41336E2C6C54'        
||'2B49939A212D819A9DE2569228EE446E1B90AC4628F3FD8C9F5697F1D4A53437F448248FC8D71F31D45DB55C247BB6E164E68E6E2475F983407F314C'        
||'FC8F563FEE3AE849D54F7F30CEC4C265735B27AEF153C55D96DAF264B3BB88C3223478D9B294B1D26331B3B83E999A312CF247F55531923D5EE5CF6E'        
||'F96373BB825DC4A688A5A2A336036924B11E645700814AD479751973C7316DD6B3C760CDAA44AB305C95240001F43E647A53D7AADD2CCC7D4E42DF85'        
||'880078FEAEDCFF005FA01EE64B3E58B4806AB963237A701FE73FB47513DC7325E4AE56DD4227AD6A7F2F4CF4A3C678A02AD1848D8B23338E59B9E6F2'        
||'1B93FEDFD9B7831C29E1C2A147A014E8AF53CAE5E5626BEA4F42EE0F2AB68D5D805528082FF520727FD7F64F71052A4746114F5EDF2EA6E672D12956'        
||'460401F836FCF3FF001AF6DC10B703D39248295E83FDF9B8FC1B3B3952A3CCB4D8CAA94C6D214560909B0D6391CFE7D996DF6DAAF51785587F87A497'        
||'328F00EACE0FFAABD53A657B66A5730F0255068E39348632B1B6B7E6EC7FC4916FF0F732C5B5AF83A88E808F77DE406E1F3EAD37E216EDAECBFC26F9'        
||'BB9F4916A3218DC356C102DEC16969F68C93807F363AE43C7F436F70DF3DDBC31F3E6C16ED846704FDA6403FCDD48FCA8D39E5BDCE502AC148FCB412'        
||'7FCBD525D3E3378EEFCBD36070185CDEE5CD56EAA7C7E230F8FACCAE46B0C83D420A2A05924600124955B01F523EBEE75379B7595B9B8B9912245C96'        
||'660A07DA4D00EA307B5BEBBB9F0ADD0BB1AD0004923EC153C3A3B9D31FCA23BF3B3A5A7CA761478DE9EC1CE56461B9A3FE25BB268E4F57F936D5C7BE'        
||'A8891C2FDF5443FF00056F602DEFDE9D8F6B062DACB5E3FF0040E98FF376E3FED43746F63ED55CDF378BB8010A1F222AE7EC51C3F323AB4DD89F057E'        
||'0F7C54829F2BBEE7C5EF2DE34512CEB57BFE4A6CCD59AA55D492E23AF71AA608C961746A8826607E8E3EBEE1BDE7DCCE73E63AAC521B784F9444A8A7'        
||'A34A7B8FCF491F67525ED7C91CB3B28A242B24828497018FDA100A0FCC1A7AF4E5D89F31B6CEE2A56EB2EBEDA124589CD8FE035396CA85A7968E8E7F'        
||'448D8DC36307869BD22CBE694DB8BA7E3D856DEC9C4E2699CBB035ED069F696393FCBEDE873B03A1DE6D828D3FA8A06AF4A8C00303A0B29072A3FC6D'        
||'FED87B347EA653EBD2AE83D51D4C7FEAA02E07F8C5207FF7ABFB4AFC41EA8DD4C160A07F403FDEBFC3DD3AB79F5FFFD1F9FF00FBF75EE95FB2FAF77F'        
||'76464EA709D77B2377EFDCCD1E3E4CBD662365EDACCEE9C9D262A1AA86866C9D4D060E19E58E9D26A8821699D020792352753A82AAD2C6F77097C0B0'        
||'85E7702BA634676A70AD14134A902B4A6474F416F7172FE1DB46D2352B4552C69EB4009E84BFF655BE4FF3FF0038E1DF3C7D7FE310760FFADFF3AEF6'        
||'65FD58E65FFA375CFF00CE097FE80E957EE9DD695FA697FE71BFF9BAEBFD958F93DFF78E3DF1C7FDFA1EC1FF005FFE75FEFDFD59E64C7FBAFB9CFF00'        
||'C225FF00A07ADFEE8DDB8FD2CDFF0038DFFE81EBDFECABFC9EFF00BC71EF9FFD141D823FF91DEFDFD58E65FF00A37DCFFCE097FE81EB4369DD4F0B69'        
||'7FE71BFF009BAF7FB2AFF27BFEF1C7BE3FF450F60FFF005BFDFBFAAFCCBFF46EBAFF00B2797FE80EBDFBA375FF0094597FE71BFF00D03D7BFD957F93'        
||'C3FEE5C7BE3FF450F60FFF005BBDFBFAB1CCBFF46FB9FF009C12FF00D03D7BF746EBFF0028B37FCE37FF00A07AF7FB2AFF0027BFEF1C7BE7FA7FCCA0'        
||'EC1FA8FF00C877BF7F563997FE8DF73FF3825FFA03AF7EE8DD7FE5165FF9C6FF00E6EBDFECABFC9EFA7FB2E3DF3F8FF9A41D83F9FA7FCBBBDFBFAB1C'        
||'C9FF0046FB9FF9C12FFD01D7BF74EEBFF28D2E7FE16FFE6EBAFF006563E4EFFDE38F7C71F5FF008C41D83C7FEB3BDFBFAB1CCBFF0046EB9FF9C12FFD'        
||'01D7BF746EB5A7D2CBFF0038DFFE81EBBFF6563E4EFF00DE38F7C7FE8A1EC1FF005BFE75DEFDFD58E65FFA375CFF00CE097FE80EB7FBA376FF009459'        
||'7FE71BFF009BAF0F8AFF00278F03E38F7C924D87FC620EC1FAFF004FF8B77BF7F563997FE8DF73FF003825FF00A07AD7EE9DD7FE51A5FF009C6FFE6E'        
||'813C8E372387C8576272D41598BCAE2EB2AB1D93C66469A6A2C863B21433B5356D057D154AACB0CD0C8AD1CB148A191815600823D933C6F1BB47202A'        
||'CA48208A10471041E0450D7A40CACAC558508E20F11E59F4EA17BA75AEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBD6F7EEBDD7'        
||'BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD77636BFBF75EEBAF7EEBDD7BDFBAF75EFF7DFF13EFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75'        
||'EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEB61EFF0084D0C13D47CEEED54A78269DC7C4BDF8C5208A499C'        
||'28EDFD880B158C136B91CFB917DB2655DFA62C40FF00176E3FF3522E835CD5FF0024E4FF009A8BFF001D6EB78CFE1B93FF009D6E43FF00386A8FF8FE'        
||'53DCE7E247FC43F68EA3FEBC31993FA0C6E4493FD286ACFF00BC68F7EF123F361FB475EFB7A0FB726F08F0FAA9282964C8E4B95744495E9A95BE9FBE'        
||'F1FEA6BFFBAD48FF00123DF83192BE1D283CCE07CE9EB4F963A136DBCBB24FA67DC098A322A14535B03C31F841F56C9F21E7D03B925CE66C4B5D9715'        
||'75121748208A48A531C0AE0C920A6A3401105805E56E41E58FB71658C1A2BD695248A28FCD8E7F61E8676F676F66821B7882569C6ACC7E74F33F68FB'        
||'3A516D4C4CAB4B580D2CE1854463FCD3AD8087F2A8A00FF793ECB6F6E1415706AA7879926BF849353F6E001F2E06B6C8CD5461461C7D140F36A0A0FB'        
||'32C4E389E94A71958E1FC54F52DEA008F0C9622E5AF2022D6BFF005E3DA3325BB5C471CAABAC2B3D704AFC29DA7883434A8A547CBA583C65B596689D'        
||'8465D6323203FC4F465E040D20D1AA01A79E7A85262A4E758951C292D14314929207FA922CA08E6E357FADFD3DD9EE63B66FF18A18CE04869DA4F93F'        
||'C89E0FC2B86A1A13E8A196ED7FC4EA2519318AF701E71FA9038C7C6994A8A810843340CAF050CACE849579E379083F4F4AA2ADB8E2C49F6A4C10B1AE'        
||'A2BFE95CAFF2AD3F3A74985D4FA74B00E3FA48ADFCC8AFE5503AE35B4B2AF8E58296A21A6A84D49F6D1BC26291789A16B2FAAC7E9723820FBA2A5C2E'        
||'1270D4E22450D51E5DCA508FD8DFE5E9FF001ACE4AF8D6E549E062665A7AF6BF880D7D015CF9818E982A71534CAFE3596472BFA278A48A4624DB4866'        
||'BA13FF0021FF00B0F7B5DCA6B7FF0073A12A01F8E23E2A5295A90009147FB4207AF4F1DBADAE7FE49F70AC695D128F05EB5A690496898FD92027F847'        
||'0E9395188AD89995E8AA9594D8A3534DC1FF000E3FDE47B33B5DC2C6F53C4B599245F556069F6E6AA7E4403D175DEDF7F6127837B0BC4DE41948AD30'        
||'749A5187CD491F3E9E25C6CF81C0D44629E76CC6E285229A248642F8EC28612B4729504092A182DD4F2107F5F71E41769CF1CDD15DC343B46C8ECC92'        
||'5469BABFA14AC7EB15A82DDC095694FA0EA549ED4FB7BC93358DC5577BE608D1648BF1D9EDD51269907E19AF1829D04065846684E50071F5F0B0922A'        
||'6AD8DD486578E1A88DD4837055D4023DCA3235B4E9E1CDA5D4F10DA581FB41A83D44B11B981C490EA461C0AD54FED143D49920ADAA8DA6931ED532C5'        
||'76A9867A07667522C6AA29D15251FED6A1EC0FA8704D8BFE82CA33FE2B235B934A18DE8B8F2F0DB545FF0018C8C7A7464374BE7016F116E545712C7A'        
||'9B3FF0C5D337D9FA9C734E3D2AB65B3D09C91A68F218DF32D0F953EE678E290A798294E1188E4F0E5ADFD79F614E6837682059A6594564D25574353B'        
||'30C031527850A85AE7B7A417A9613BA3436ED19A772B1122FCCA1650CA3D436B3C28C73D2D24CA6600291E42BA3527592B5954AA4DF48B806D7B7B08'        
||'192663C4F568ED3698D35344A6BE545F4E3D42A9CDE552CD264F20156ECE56AEA4BBA837218061C9FA7B75649011A98D3EDE929B5B275610C29A9B00'        
||'90B407D7D7A41E677865115D93319A885CE90B5F52A2F70511BD47F37FA1BFBD78AE72091D28876BB70423448DC2BDA3F323CFFC9F6F48CADDDF99A8'        
||'8E21165F310B33C6659E6CAD4B8637B148E366E03717045FDA88E472000C54FA927A6EE6C6DE1572D0A38A1A2AA2D47CC9F5EA84BE76F6B76BE3FE48'        
||'67F078CED5ECCC2E1A8BAD3612C789DBDD89BB7058B924CCE433EF95A99B1D88AC863692654863795935144540748B7B63729A74BA28AEC0695E0C40'        
||'CD6B807AB6DD676636E86511212751A94526A0E324138E8A6755CF92CF766ED8ACCF6E0DD1B8AA36F6C7EC0C1E164DC9BAB706E3FE0F84A7EB4CD3C3'        
||'8BC6266AA671040A4B158E3007B411166901624D0102A49F23EBD191345214015CE001535F90E8D8F56A6E48BAABE4A2E1DB0A296A3A8FA922A41911'        
||'5DE63944F927B5A4AE6A834E6DF6E6855FC5A46BF3DB57A2E3DBAE1F5AE9A7FABFD8FE7D50E9246AAF13FF001D3FE5FE5D173EECA68D36E756D54725'        
||'652D5D0EF4ECEAEC657D0642BB1391A0ADA7A0DAC91D5D257E3248A649155D802B2585EFFE3EF53E0A918209C8A82387A75E0C4034F97A1F5F5E824C'        
||'16E8DE188CB6C8A4C46FDEC7C75243D95B1271434DD83BCE2C64D36537E514592FBDA35AEF1CAB5026944EB22B2C9ADB502493EF51CB209128CDF129'        
||'F88F9B0AF9F4E5BAA3CE03AA906A3E15E14FB3AD94F65D555CBB6F0F2D455D6339A7943992BAA5C378EAE48A3E59EE785FF7A1EE44B8955635180428'        
||'FF0007502CA6E9F759C2BBE932380351A001BC856829D095455F550D9454552596E6D53312C6E1FD459BFDB5BD91CB70C1F24E3A125A5BCF2DBEA462'        
||'4B6324E334FF008BF4EB256D64D511091A79D810C3D734878B58DF51BFD39B8F6EC172EC4549A749EFECE7B605759A9A7E23E7F9FCA9F2E822DC594A'        
||'986490A544EA1406F4CD2837FECD8DFF001EDF69E4D55A9C7CFA4912CA5002ED5273DC7CBCAB5E80'                                                
;                                                                                                                                   
BRANDINGBackgroundIMAGE_3 raw (32676) :=                                                                                            
'4DCDBDAB69A8321506B72064A6A5AA9CF8EA245274C2F21D0DA80BFA0DBFD87B29DE6E99F64BB424D4C328FDB1B0E847CBF6D711732D8C9ADB48B884'          
||'FC4DE52A9F5F9FF2E8B0A76D54896626AF709D52311FE5EDF990FD7F73DE3294CF59882F4D4D6BFB7A1171BDA75258DAAB3E09922FF94E6FEBF5FF00'        
||'39EE85074A16F18799FDBD09B8BECEAC3FF29B9E3FBCA39AE73C71F4BC9EE8631E7D2C5BC6F53FB7A343D75F2F3756CCC74189AF8A1DC38785CC54AB'        
||'B8A73155D0C7FA8410656390398C5FD29287D3F452071ECA2E79721DCA62D02B890E4F860B57E65403FB71D0F763F72776D86D56DA629340B85129A1'        
||'51E8AF50683C81AD3CA831D2EF3BF27772F60D00A1A34A4C1626A4B25545839CCF3552A827C35391772C10DAE5102DFE8D71C7BA43CBB16DB36A9D58'        
||'C838788A569F30A40FDB9E9FDDBDC5DD3982D8DBC452181B0C223A8B0F467249A7C852BE751D15CDEBDA95BB5777D05650AE4932B4349435F4593A7A'        
||'F105552CCB23F8CC4C558F163717B1048208247BCDDF63397ED37EF6D67B2DC1125B79AE278E48DD752BA958EA08FF005104020839EB15FDC3E63BAD'        
||'8B9CA1BCB0678E78628A48E446D2C8C0B5083FEA0460E31D0D107CF5EE74DB22B315D2EBBD668584326769B1FB97EC8AAC4C5A7A98F050BC25C1035F'        
||'8D917EBC0F616DD7EEBBED847BBF872F303EDAAF916EF2DB16153C14CCCB201E435073F33D4A3B37DE5FDCCBADAF541B0A6E0C8337222B90869E6C21'        
||'5285BCCE92A3E43AAE4EE7F923D8DDEBB8A2C86FFAB063C43544187DB98D89B1F82C079180A94A5C6B977F3BE90B34F3BBCAC0052C1405193BEDD7B6'        
||'9C9DED8ED6D65CAB01D53E932DC48C249A7A7C25A4000D02A4A22058C54900935EB1BBDC7E65E77F74B725DC39AEF974C1A8436F1C6D1C1057E2D119'        
||'62759A00CF216734A540000343D65F2EF19B612349362642B34C114374CE52C37F1C423D5EA80FD6D7F69772E4D96ED8917016A49F80FF009FA646EE'        
||'B6F184284D001C7D07D9D0B959F20E8BB17195B5306D6ABC508A634DA26CA415458B4424D60A44BFD6D6F6523971F6C9551E50F515C291E7F6F4A2DA'        
||'FD6FA269154AD0D38D7CBA06B6A67531BD9BB6330699E65A4CCC7398048A8D2010C8BA0484103EBF5B7B37BCB732ED53435A55695FCC754865097895'        
||'1E7D1FBC2EFD8730C8AB8C9A9F55B96A98E4B5FF00D651EE3B9B6F684575D7F2E8422E011C3A1329A8CD444B2890286FC15BDBFDB1F65AC749A1EADE'        
||'301E5D30E6A3F0C6C09D5607E82DF4F6A205A9EAAD718E1D005BAAB84624FDB26DABE8C07FC47B105A424D33D2392E07A7457F78EE38E0F28348ED6B'        
||'F3E555FF0088F62BB2B4269DDFCBA2D9AE4034D3D15DDCDB9A396475FB3704B117F329B7E3FD4FB15DADA1001D5FCBA289AE013F0F4C9B2B25434FBB'        
||'B195B90A492A68D1324B3C11D77D84B69F173C11CD1559F4ABC4CCB22870C8C5423ABA332932A155A034FCABFCBFD5F2A1EA2FF767537206E1A0E93F'        
||'A19FFA8987FD59C7A83C3A34DB6B0FD7DB8E88613716ED86686AE6993113DB158ACF6DC325A7334743929A829658A561695292A5A273FB9E059407F7'        
||'5B8B976216304500AD6B46A632406229E5515F2AD31D616FD3FD3A9BA75D6F4385A79904F6F6835F3D38FC5A7567A83B9FE3E56D3566267D9DBC76BE'        
||'F75A848A968BECEBA2C565FCF1A05FB3C9E06AA7FB95A82C7C4A695EA63770012970BED3C5788CAC66568801525B2A7EC618A79E42F576322B0D4A4B'        
||'0341A431F98D74155238798C648E1D06E361EFEDB12E5AA576F6EBA165C855E37214D4F8EAE325357D3C11D4D4D156C1571C913E9591249227516575'        
||'242823DA9D68DA58302A4541A8A11522A29FB2BF2F975E528CCEABF10F89695209008AD4647027ED1C2A3A5DEDE6EA5AFADA0A3DE796A7D9D59FC3C4'        
||'F364719B7EBD3CF903CAC7594F89AEA7A6420FAA46D2748B91139F4962E8DF471936C9A8D68035076FC8E9C9F4F53E63A556C2CA72B1CAFDD92590B1'        
||'0588FF004CD4507E6748CD3A566EDC162170382CA1DF34D9AD9265AC876F55D7CF3D4D326469159E6C6C31414F1D643EAB37AA8E58975ABEA5B9BD2C'        
||'E49248D848BA1FCC528687CCE4A9FF007A0714A756B92915DA88C8750451B56A5D40127E15D429E9A1B8D7A001F7D537F1411C9532AC1029A534EB1C'        
||'35143344AC54BA4B48D704DC1591620D6B7D3E9ED588F4AD13F6E7FCBD36F7B03CA55B8FA507A7A8CFE74E863DB15F89C8C5F775E30196C6C83C1340'        
||'BB868F1195489F892A225C91A662E181F43F9636FD2501B3AE892AB86A37CC123ECA007F6E0FCFAD3091D98C22AA2808AA8A5680915D25B35A8A9007'        
||'91F26BC9759ECBAA9CB6237C6031333C855A9F3D8BDDF8E312923C4F1E476ED464A116E59B4D3851C11AAFED31B99D9BBE2A8F55299FC8853FCFA74D'        
||'A787089A06A3601525B07D7E22A01E34A5079F4DABD5B93A68F2530DC9B47311C2A25D38AEC1DB9354E469E321038C6EE118AABA861C1485D4BD8595'        
||'09E3DA813C44A8A302D8CAB003ED203003E7C2BC7A4327D54719668CB1E34015C9F95032963E74F4E1538E9275FB326AF8E8A4A4C1C1570D3CE63A8C'        
||'726D5A4C5D74FE3B24B22D7612A1FD3637066991EE0E95BFD5DC2EAAB15FF4CC187CA9F6FD9F6F4945AF8A530AF5269A55D0F0CD469C69F3A91C0D2B'        
||'D21F7BF697C92E81A1C43FC6AC2EE2DB196DC93642937BD745B664DEF26568A8E34970548D067E9AA969561334E0342C0C97B393A47B0E731D9B5F2C'        
||'44C0B205248146C70CF6B0E3FB3D29D4A9ED947630C9770CD377058EA438A9CB60D5703D3CF8D49E91986F987FCCEA5666A8DC3BE68C965911A3EA8D'        
||'B946582D8F9A37FE180FD6C0107FD7F61F8363825A89ED1283D437FD07D4A52BEDF18FD39CFF00BD0FF20E8C874D7CA4FE60593EC2EBA87787607620'        
||'C44FBFB64439BA2936B61B1B14B867DCF4B1E4A1A87A7A24658DE9CC8AEC083626C6FECC66E57D806D7712496D109045215C643686208EEE20F45CD7'        
||'EFF551A452315D4A3E2C52A2A387A746CB74F6276F6D8F9A58DDB543BDF7C627AC0FC96C96DEACA6A199BF8460A8A8FB726C3A2D5AC3EBFB27492392'        
||'789B50FB6491458393ECB0D8D8CDCA11DE0B649657B1563DA0B163029AD4F06CE0FA8AF1EAB1CAF1EEAF0B4A5024E40EE34A0908A538531C3D0F42B7'        
||'FC2853250D06EAF8FACD4992A2ABA88BB9FF008761B172454F4F4953151ECFAB9F09529246623120A9FB4118650A09205BE91B7B41204B1BB5D45DB4'        
||'4241AFC746B85A9AFF00152BF9FE7D0D79A50BDDA9A051A9EA0FE1AAC4683FD2D7F97556BF1B30345BEF1799A9AFA758EAB1ED4F52E35792085AA202'        
||'B3530A87FD5E2788A97B8D57B8F73BC578F1C215719FB7247007A00CF0278BA8E7A16E5D9D454F96312AD3732D97C6F724B703535CDB836E491ED60B'        
||'A7299E9288406E874A4D898B6C3D2205689EDA955AA74C64FD18925B93F4FAFF00BC7B2C6B96F10D7A53A140F97435FC76C062B17DF9D255029E1496'        
||'3EDBEBCF1B99CC8A643BAE910494E1AC09BD81B0FC01CFB20E6991DF9637203FE5167FFAB4DC7A5FB5851BA5B0FF0086C7FF001F1D1D4EC3A2EC1C7F'        
||'C64E848F6765737846C6F6FF00C99C7CF3EDCDD957B73CB8FF00EFFC55748320F45534DADA39448889216D006A1656F602DA5B6A979CF7437E8926BB'        
||'5DB9809231250F80C0E9AAB50114A914AF0F2E8DAE9AE1797AC4A314A4B762AAC57FD11695A119AD78F4516A6B3E427DDCD252F67EFBA386D2302FDB'        
||'B938903A3F8C2B46B916B006C3FD717F6391072C691AACE13FF50CBFF407443F55723FD19FFE7237FD05D3EC759DEC94513CDDB7BBDA48B5485DFB57'        
||'73BB33A7AB433C755602FF00560DF90791ED9F0796CBD16C62FF00B268FF00E811D78DD5CD3FB67FF9C8FF00E7E9D2A66EDDA8A2F476BEE7592C9E74'        
||'6EC6DE1513C6CCAAECEEC642180371C1B7D6D73EDA54D855E86C62F97E84407F83AF09EE5BFD19CD38F7BFF9FA1E3E3763F7ED455778CD97DE99DDC4'        
||'E9F18BBC852D1E6B76EE1CC63E3A8AADAF1C5F7661CA93106559182B04D435716B93EC2FCDF26D489B62DBDBA440EE365529122B102538EDCE683CE9'        
||'D1DEC6D72EF784B96A5A5CE19988FECFE75E8A77C97DB54D37C8CECE959B54926728A3F17DAF95F4C7B7E89028624B1E1748F49FF6DEC67CA331FEAB'        
||'5993C3437FC7DFA2BDCD40DC255E24118F3F84745B3716D16D4CD4F46ADF6C4B342F053B8A9005C1545219B926CA56C39FF62278E7C71E9032D470E8'        
||'38DCFB1A8AA29FCED430C7321F3344B1B2310D624116D3C7371C1FC7D47B570DCB29A1351F3E9A7881EE1C7A6AD81B0B0B3E4E55AAA38C42F2BE9865'        
||'01ECE48D32AEBFA01C5EC6DFEB7BBDCDCBAA5233FB3AAC71EA357C8E19E967F14E06A3FE649F1CF051414D8EC663FB29EA28DA78AACCB5F2A6CFAB97'        
||'EDE927A64D2A436A2A24611B00C08FA5C0BEE04F237285E963FE86066BE6E8314F3CF9F421D8225FDE7105AF12714C5149FD98E8DFFF0036DC1E5F3D'        
||'F2BB6FAD4C18FA8C7E13A9F135AB2C3355495F511536359A9683ED8C446985128A9E4B37A66F285D5F525DECCB247CAC5E3240370F50719D5938C10C'        
||'4B3027F0E9EABCEA6BB9957C9F0D00C571A4507DA0531C2B5EABBBE757C7CDEFBD7E55F6AEE2A1C8E3E831791A3EB24A4A7A9AECAC1242D8EE9EDBD8'        
||'D9A36A458F4C65E48D88BFE0DC817F626E4FCF2EC04C872663827CEE2538A1A749372B88E3BC64F0EBA420C81FC0BC6B9E88BE57E1BF65545424D0EE'        
||'3C1C718594B6AC8E7D50000160CD1407F4DEE0FD07D4DBD9B5C58C933EA8E761F225BFCFD5E0DE2D625D2F0827D74AF4637A3B64EF1E8FEB5DF9D75B'        
||'BE9779E7B01BFB3C770556EFEB1C4EE2CBD7E00D261E9A8E963A4CCC98EA98E3AC8648A39A282589A32258D981D5638A9EF95B4B6BCC56692C864D56'        
||'D5072481E2B8A66BF6F52D7235E417DB74B2C09A02C942280027429AE3AB10D9B88DAFD318BE9EDF1276476CEEAC06F382ADF67F5C7686D7C5E1B3BB'        
||'57C94135351D6D4EE4C254E3D9279228DE1A07A8C5AF922A83FB618C4EB0B0E03D3A1AF1EB8779F67FCC8DCBB89376EC7E9E5EDBE9A0953B7B6B3EC3'        
||'CBE2F21BE572F86C74B92CBCB97C0D4D4C1511D09AC59A8F17570A4A6696096C231A4B78961C063AD002993D7B6FE33A0329B73ABFB5FBF3ACE93657'        
||'7FE6B73EC8A4C86E8199CA6C80FBB6821937263B61E5F66E26A5689E2A5C7E2CC7352D6529662B234EC259581D50609E3D6EA73E83A5B6F6DB1B4B7D'        
||'EE1EC6F957F23EAE1DDBD174F518D4EA7E8EC360EBF3F5DD8388DAF4D1D3E2EBB73E26146FE3477164E4A9A9DBDB62920904F14D1D457B3461A9DBC6'        
||'9C4F0EBC3D071E8BFE0F3BB033D43D8757F12FA976AE6BE47F666169AA76CF50ED4AEA1D93B67AF76752B430CF435D5B939E083154B4D52D09DC4D49'        
||'494CF555468A829221108623A1EA38F56352687A19FA0335DD995D97BDBAEB7860F73F43E4F05B6695BB67B8F73BE3972F80CF4D8EB6E3AFD9791A91'        
||'3638E56786335D0E52A09C662E89A9D561AAD291AD813C3AAE2BEBD075FECB0F5EEDBEB6AEDBDDE3F2220CEFC38C3EC3C766B6FC9B7F2D9BD99D81DA'        
||'699293CF1D5EE94C1913D550494BE1929463AA8364EAE75A86F0A45A3DEA98A1E1D6EB435F3E9A77964FE39FC6BEB5C7D26E8DFDB977F753762E662D'        
||'A1D6BD0DBF37345B5B766ED5C84D0E2062371EF0C6548AFCAE360D7E3A82B251D3C7158D7FDC8658A6F1A019E1D7A84E7A6ECE7C9CF87FF0BB6DE036'        
||'8F5D6DACFF00684CD97A04DAFB0303555190DA5D735DBA1524C20C9E73767DE536DBC6CB33AD3E3E4AD1535B244D1BC61D5D5574481C3ADD09CF5377'        
||'AFC85E8AF8698AAAEC1C4753E37786FDDC7B8B07B8FB03697460A48309B4B31BCAB6238D8B79EE8A912516CFC554D5C8560431092A64769BEDE49252'        
||'EDE242F5A00B71E9937CF7936FCED2391A0ADDA11E5379E3E2DC1260B6478A9A829B238FA58E872B868F273CA1F395F8D8A1A7A6C8659E2A792A5D35'        
||'AD2C30A20F7909ED8B29E5AD0695334BFE04E989E2A466E083A56953A4903D2A7CABD0F1D69BAA8F7760E2AF8A78E7531C6C26762923C72A5D75C725'        
||'99180B70C01F63A90645381E92BA146AF42A2E85504CD0B06600D9D0103D4C2CA4F249B03FEDFDB078F5E071D3B53B403C3E49516F2C4B632A5CB38E'        
||'41B1FF00607DD4D7CBAF74FD4B2D2A2826A699546954B4C96B10142804FE4FE3DB4D53D5B3D11DF9155B0CB8ADE9024F048F5753B771EE1268CB88EA'        
||'771D39943283C02AAC1AFF008F677B42937B0FC8D7F6027A4FBC398F6796B8AAD3F69EB5FDF913F277BCF66F68B6C0D91F20A5E9FDA38ED8782AC4A5'        
||'869EB67A6CA57E51A5A9ACA56FE1745592898971CB68400137D4790CEF77D70BBA4A91CA235049E15A9A9F4073D0BB6BB2B65DB2DFC45CF86BE55EA3'        
||'FC4EDF7BAB2DD1DF392A727DB15B5553B6761EDFCBE2375AD157BB6027AEADC856E4B7050C0D1473C8CF36A0632BAD992E000C3DA7B0B998DA5CD663'        
||'D8010D4F86B5248FCFA36BADC2E96F76D55BA28B6EEC23214FE90216ACA315F4A7134E815DADDBBBDF73E177A65371FCE4DCFB5AA76B521AFC4632BB'        
||'159E7C96F3A719E83134D4BB5E9691C192B66A798E40D249242B0C314DE494BAAAB978BC94825EE5810718F2F5E238FF002E8FDF7EDC1EE135EE1291'        
||'56ABE9CA8F2602A0F78E201047035E9F3E1CEF2C96E3F961F1D2AF7576CEE7CFE4A7EE0C6E4EAF0D984A99F19513C989AAA4A9CBD465AA6B8A2CA22B'        
||'465BEDEFA0D8B003976C6769AEE1F126676D5F09069C08E35FF2744DBE5F35EECD29B9BC96693492236525457068C5CF97F47E5D18AECCF917BA3B93'        
||'ABFE34E377EEE8A68301F1E3E586EEEB2EB97929686928B05B6D6BB1DB832B4559598A829F5C12092A2B279AA44B2F055E471EDCD300D374A34B34F4'        
||'63539D21727D31E9E5D25967496E0C68349F016B4F37A95249F20540C0C7F3AE1EBFDCF2E7B2BDC7F2CB2F033C55D2FF0072FAB69AA02A34F484B506'        
||'2C50A102ED3B7EFF00A45C33F3F5F722F2A225ADADD7345C8F8814887F45718FF4EF41F3E80DBFC12EFDBB58725D977199D5E503242291407FD337F2'        
||'E94FB57681FBBD8FB03232F9A4A23376576356359109A798E5121A927E827A96BA6A3F45B7E3DA9D92D1EEAFA313776826690FAB93551F9B9A81E8BD'        
||'4F1EEC6ED17247B771F2F58B0171721614038D5B0C69C7CEBF99E9AEB3353E7FFBDBBBA02A2BFB033F0ECFDA30C6CACEB8A8A734304F4F7E594969AA'        
||'B8FEC11FD3DD6F247DD773731E7C4710A7FA55342DF9F737D9D2BE49B083DBAF6B9B71BA3A5CC66424E09257B47F83F69E955B95B1D47BCE8B170CD0'        
||'0DBFD3BB39239983A988EE1AEA60D30637B068608C5EFCDA6F661CC72A2DDC5649FD9DAC7A8FFA661403F2515FF6DD04FEEFFB5CB751EE1CEF7E28D7'        
||'5239527F86B5AD7D387F3EB3C54725075FECCDBB333264FB1F724FBBF3D1BA857FE150D40CBB2CA8DC955229E9C5FE9A47B557519DBF9721B23F1DC3'        
||'6A6F5CF7BD7EC145E839C96ADCFDEF95E6FE7BE0DBAA10F1008ED5A7A633D1AFFE5E30C75DF28B1FBD8B80991CFEE7DA9843216119A3DB9D6B9CFBD6'        
||'84A9E54CCF35C8FE83FA0F61CDC6DBFE41DB85EB0FED5E255FF4B1CD18C7AD5ABFB3A027DE93988DF6EE76A84D56D517579852CC295F31C48A8CF975'        
||'B25ECC9B4EE2C49009B6528558136E04A140B1E3E86D7FF6FEE0DDEE3AD8C84FF031FF0057F87AC5BE56982EED00F4912BFB683E5E74E85086A5CC7A'        
||'46A54B0E095252F6512707EA6CC3EBFEF3EC18F1AD6BD48D14CFA2847FABFD40F5328AA4C79357F512B04DA5AD6FD56D0093F5E05FFD8DEFF4F6DBA6'        
||'A808F523A7A09B4DE06CE01CFDBD40DC323CF569228054280581259ECBE96622DF4B7FB1E07B7EC82A4655BA49BA334B3875CFCFD683FC9FCF1D069B'        
||'D642D8B366B0258722C785086FFE17E071FEF5EC59CB4A05F678E3A01F38BB1DB085F53FE6FF008AC7F83AD7FF00F99876E53CB96C0F5050D5332602'        
||'63BA772C31BEB8DB3991A56A7C0504A14FEAA5A279EA5C1FA1A84B8045FDE5CF22D943B76C4FBF5CD355C011C42992AA7B88F9161FF19078751BFB5D'        
||'B35FEF5CE57134287C2B62C8CC48D207697A7F4CB08E307E532D01AF5AEF770E7E3A7ACA8C423A892BA38AA275D5A478D1FCA884FD3D440BFF00803F'        
||'D7DC23CE5755DD258D3F190C7F3CF59D68851163FE1007ECE8B3BC704B54F1D5EAD27F6C1561F937D563CFD6D63EC155F3EAF9D543F9F4B2DB1B46BB'        
||'3999C6E0F0CB575D93CBE42971F8FC6D0432D456E46B6B265A4A2A3A6822059E596491238957D4CC401C9F6F410BDC48B0C7C5B1F2FB4FA01C49F21D'        
||'78E9452EE74A8049AF000713F65327ADFF007F97F7C23DABF067E3547B7E6A3867EE3DF987C3E6FB7F70108F514F9334BE7A5D818D95402B4388799A'        
||'3942BFEFD619A626C22080EDEF78FDEFBAC3616469676AC447FF000C7AD1E76F52F4ECAFC31803896263BDEB703736D2DDBD72B44507E14F2C1FC4D5'        
||'05B23C97F0F561953528D83336AFF374F1DACD6E16D6D238E2F6FF008AFB0AA4645E69A7127A573CC876B2E0F05078F4AFDBF91D1434C5E4B30A78F4'        
||'282A482CDEB06C09FA5FF3ECB6F21ACCD41C4F4BB6CBAFF17424D0803F9F48EEE795ABFAD37753BEA6F352631845A8032DB3B48DA89537B0009047E7'        
||'D8ABDB91E0F3A5838C519F34E1FA327F97A47CCEDF53B34F1302DA803A6B4AF703E46BC3D0FCB1D123C175FEDBC8A9A8A9A7581D092AD532BB217FAB'        
||'9284F279FED1F795573BDDFC1D88D51F219FF57D9D45F6BCA7B35CFEA3C63567E22695F3A83EBF3EA5E5FAB6824612533D1B46157FCDC6CA594716BA'        
||'9207F51F4F75B5E62B85146D5F9F55DC792EC24C8541C320797903FE4FF074317C7BF8BD83EC2DD22A2AD6A1A830B53473D6687212771289529D5802'        
||'0065521FFC3E9CF3EC33CE5CFF007BB4D8F811101E5040F90A52BF979742CE44F6AF65DC6F7F78DC2B32C0CB824904F100797967E5D5D1EC4E95EBED'        
||'AB49251E2F6D6269A3AD945455AAD0C05AAA646F2A4951395D44A9E556F61F803DE376E5CC7BADFCA24B89DD8AE0558E07C8701FE13D6475AECBB76D'        
||'D115B58113564E9502A7E6464FCBD3CBA12A0DBD4F4702C78BA610C312E88E182311468ABF41108FF1FD07B266B9795AB31A93E67FCBD190D1150607'        
||'9752136E0AFF0055634D18493CBA44B717D2035A361C71EF42E0A7C34EAB2C81480B9FF0750A4C5612967712411CE8DC4A6645694E936D25B8E3FC3D'        
||'DBC79DF81A74EA2131F0009EA6544D858A322868E9E06541A4451AA716E75151FEC6DEE83C663DE49EBD144EB976AF491395869A620E84049F4FA458'        
||'7F523F1ED4F82CE3D7A500E3A711B9A0D015A68F4FD194E92181E2C01F6D7D33035A75BD20F9749BC864B08B2354AC34AB391A5A7554125AF7B71F8E'        
||'7DAB8D272349269E9D53401C07491C8EF2A3446413800DD43137FEB7B7E7DAC8ACD89AD3AD152B8E93432189CAC8DE62960C1AE09D4CC7EB73FEBFB5'        
||'4639621DBD794107D6BD6A4DFCFDD685BE66759AD322B2A7C62D8CA4102F63DADBD48FF89FAFF8FBCBAF600BFF00526E4B71FAE97FED1ED7A03F33E6'        
||'F953F0B44011EA0B3835FB463AA62F0C57E234FA7FA9FF001F736FD5DCFF00BF1BF6F51DFF00557967FE5020FF009C63A6BAB820331FDA43E84FAA8F'        
||'E87F3ECE6C55678049300ED5393938E1D44BCE5BB6E9CB7BE1DAF97AE24B2B611A3F850B144D4D5D4DA462AD4153E741D2D3AD6185775C0563507F87'        
||'E4402001F588717F65DCC50C49B5B32280752640F99E8CBDB6E65E61DCF9A92D371BE9A788C53128EE596A1450D0FA7974639A2D513AAA6A2D1BA855'        
||'17258A9B2803EA6FF4F71FD723AC8C1C29D0D5DF5FCCEFF96F752E671FB2F71A66FB5F7B6131787C76E99FA7B6FAE4F1B88C9D262E186AF1F92DD52D'        
||'7E368AAEA21915A39D69649F43AB2338616F78BDBAFB99B7ECB7D2DA473BCA55DC11180EABDC700B1031FD124742B8A77440AA49A01C0540C7F87A37'        
||'7F12FE637C57F92982C88F8C3B9E3C7E47114B0D76EFD8193C64FB5F7EE2A369045064B2B8AAC795EB6983308856D2D4D442AC42B3A3300576D1CCFB'        
||'7F3402F0CDE2489928F8751EBA7852BC4AD470AF5BF13C43DC4D7D0F97E5D5A5F5F2EE498EDAAADBD5390A7DC09153D6E3ABB17533419582AA34D42B'        
||'61AB8590C257EAD26B161F5FAFB5373B2582432DFEEF2AA5BB6483C287829AF127C94026BD6397306E57F3F355CEDFB346CF389DC0D3C6A0E4D7C80F'        
||'36240A71E8ECCDD99F2D5B6FFF000C3DA95722888A930CB86873CEA45BC7FC6E3A349357FB509835FF00B7F9F61181BDAF8EEC37D0B501F8995CA7DA'        
||'535F0FF6BF9746F36D5EE735A3017695A1ED5740FF00607F0C67FDBF1F3E89DE4A3C9A64AB57302B3F8B1A9924C83641E596BA5AA94EB966AA9E62CD'        
||'23393A8C8CC755EF73EE63B696D26B5496C595A12A3414A69A0F2007003852829C29D4317105E5B5D3DBEE0AEB32B1D41EBA813C6B5C9AF1AF9F1A9E'        
||'BA8A9891AC8F4FF89E08B1E6E3DD99B34EB6ABE7D33E4B3706258349288D148552C45B9FA5EFC0FA5BDBD140D28C0EA8F2842057AE31760D3D3216FB'        
||'8441F51736FF00783FEF7EF476E2C323ADADC02D553FCFAC5FE95696AE48A891CCAF24E5358BB286D37B165E40F7EFDD0CB590E283A7FEB86911F1CF'        
||'4C7D97BEA9F1FB37311D408FC3598E9A0669D99101990AAF2390C5B807FAFB7B6DDBD9AED1978A9AE38E3A6EE2E15A33F319FD9D5777567C50F909DF'        
||'39B8EA7AE7AEF2F5F8596A7F7373E5D1705B5600CE7D6D9ECB98A1974589B53195AE2DA6FC7B1D6F1CEDCB7CBF098F70B951201F02F7B9FF006AB522'        
||'BFD2A0E839B772BEF3BA4BE2C111F0DB83B502FED3C7F2AF5B017C31F86B5FD11B43B17A6FB4F70E3778D0F7DE0AA5B378CDBF1E4686871C98C87F81'        
||'65B194799AAF1CB3C9253D7C72199208B46825035B57BC6CE79E754E62BAB7DEF6C88C0DB7B8D258824EA3AD58A8A8001422956E39A7533F2D6C0DB5'        
||'42FB75DB8916E410DA6A0034D0454D09C3035A0E97C3B0BE2CFC67A3ABDBDD51B330C3274A1A8EAE9F64E321A43354D30F0326737AE483544ECB621F'        
||'F72737B8007B0BDD5D6F7BDB7D46EB70CDE63C462C4573DB18ED5FE5D1CDBDAD8EDEA52CA303D4200A3E619CE4FA79F450FB3BE657706F4AA970DB40'        
||'47B56864493FC8F6D99A2AEFB7553E49F259F935D4D82DCB98427FAD6F6FD9EDAAD4F0535B83966C81C3CB0A29F9F5E965882FF8C1F0D29F0AE0D3E6'        
||'7E23F953A275BA77762769D24FB97B237A5363D26D724F5392AAD12D54A7D4F1D2C4C64A8A873FD6ECDF936FA7B13EDFB36E1BBDC0B0DB2DDAF26E18'        
||'07427CC9340BE9D116E5B8EDFB4C2D797F225BC6057B8804FD9E67A007677CC5EBBDC1DC1D7DD7FB370592CAFF007A776E3305FC7EB658B1F053AD5C'        
||'AC8D574F8F224964D2471AD909BFD3DC9137B51CC363CBD73BDEEB70B08B689A4F0505756903B59F1C7CE95A53A0772E7BB7CBB75CF5B56C1B6A34D2'        
||'5DDCC7107A8017512355389A5387CFAB44A520103FDAAC7DC50FD65F74ADC4A49354A450C6F2B3AC91B2C685C8592364BB69FA0E7EA7D94EE9B96DDB'        
||'4DA1BEDD6E23B68578BCAEA8B8F20588A9F90A9E9C82D6EAF641059C6D2B9FC28A58FF002E03E6683A52A62A0A5556CBD6A529001FB3A7B54D6B7F81'        
||'09754BFF00B51F71EB73E6E3BEB187DBFDB5EF9787D65C6AB6B25F9AB3012CF4F48D403FC47A117EE0B7DBC6BE60B85B73FEF98E924E7E4403A63FF6'        
||'C7F2EBFFD2F9FF00FBF75EEAD77F93F123E47761693623A17729FCFF00CFC5DAA0F1C7B963D9D20733CD5AFF00B8B27E5FAB0F431E4815DDA4FF009A'        
||'2DFF001F8FADA4B637436FFEC5EA3EEBEE9DB353817DA9D0316CE9B7DE36B7255706E6A8837D64DB0F86AADBD431D3BC13C6932486A5A6A984A2A390'        
||'1C803DE474DB86DD6FB85AED73B38B9BE2EB0284D48ED128760EFAD4A305202698E4D64E93A7A93DEE2D9278ED64244B704888052CA4A80586AD555A'        
||'0229456AD470E87EDD7FCBC7E466CADE7BE3626E79F60E332BD7BD067E496E4AB977265DB150F5EC75B2E366C7D3541C7076CC45530CB0BD1B46B15C'        
||'6AFB8D2412821E68E57BAB786EAD6EA59629EE3E9158419FA915A21066A784568E260D523FD07563A4E9BAED32849219599647F04111D0F8BC68007F'        
||'83CF5D72382138E89CEDFDB1B937164B65E3A9B1B94C72760E730FB776B657354196C760B235D9CAF8F1901A7CB3C2629E38A5957EE0D2994A283C12'        
||'2DECF625124BF4EC4028097A7150B199092B55604AA9D1AA9AC95A366BD184685DCC65854024D734142C4915040A0A8A8CE0631D081D83D17D97D6FD'        
||'C594E8DCA630E637AD0760C3D5B455781FE2726D5DC9BD2A2B69B1D0E2B6EEE1CB53D2C731F354C11C9AD51A2D5795500BFB2DB4DCF6CDC36FFDEB62'        
||'EC6DCA78BA9D74308C066ABC6AD2683456340CC28ADA49D27A4D0DCC1716DF5B6E58C7A7582C2874509AB282DA4D149A54D687269D183EEAF80DDB5D'        
||'2DD73BABB1AA77EF5276253F5A6E0C16D0EEADADD73BA6BB31B9FA77756E430C789C2EE982A608E2A82D2D4414F23D29056470A22650EEA5361CD3B5'        
||'EE3B8C7611473C46E10C96EF2C5A16E63503BE3A3330D55D403852568E09D41421B5DDED6EEE45B46B22B4953133A5048829DCB92699040205568C3A'        
||'2758ADB3B8B3192C16321C764684EE2DC149B5B1991CB63B2B8FC43E6EAE68E11432642687C6648FC88F34285A444B394B11EC4CAACB208A51A5BCC1'        
||'1DC17C8953434C1D2680120F0A1E8DCAD1B4360D2B42286849CD08AF01C7CF871E971DD9D2FBF7A07B237AF596FC8A0A8CB6C2CDC3B6F35B8F6F266E'        
||'AF6555E62A30D4F9F14389DC591A5A449A55A6AA85E48991245E498C01A8A0DBB73B0DDED63DC76B62D04C0B21910472680C56AF1AC9268EE05451D8'        
||'1A503134E925BDDDADE42B73684B4720AA96011A952B5650EE145452BA8E7A0D2A31F99A3871B3D6E273B4516622FB8C34D5B88CA51C39A83C823FB8'        
||'C34D53122D6477206BA52E2FC5EFED722891FC28FB9AB4D2A4960695A151520D33422B4CF4A5575B78494671E4284D78D31E74F2E3D66A5C467EBA5C'        
||'7D3D160F71D754E596A5F134D4384CD56D66623A291A1AE93114B4D0BBD5240E92473B53AB88D9595CA956029234511712B04F0FE2D4C17470F8AB4D'        
||'2723E2A75566D048720538D68283FA55F87E55A13D3787759591FCC8F14AD14914A258A58E48E431CD1491BD9959082AEAC352904102DEEEF1942410'        
||'453D4E7F6797CEB4EAE4006847F2FCFF00C19FB33D6ACDB57E2A76E7CC7F981DFDD5BD371ECB5DC382CC7C82ED6DC996EC6EC3D9DD59B276CF5EF59E'        
||'5721B9F7DEEDDD3BF37FD650E271D458DA08E5ACABA9ADAB8E38E24791D82AB11849BE9FF777795FF7FCDFF571BCFA81371FF9284F5FF7E3F1E3F11E'        
||'3D06BF297E1CF7AFC3BCF6C5C4773623673E2FB4F64C5D8DD57D81D63D9DD75DCDD57D9DB225CB54E024DC5B1BB2FAAB2797C3642186BE8EAA8EA638'        
||'AB3CB0CB191246AAD1B394F48FA2B5EFDD7BA11E1EB6AC9BA8F25DBEBBBBAEA3C7E2FB1B09D6D2EC49B7DE021EDCABADCF6D9C86E88376E37ACE4946'        
||'52A36E53A63A4A3AECEC509A5A6AD9A92926759AAA057F75EE838F7EEBDD7BDFBAF75EF7EEBDD093D37D4FBDBBEBB7BAABA33AD31D4D98EC7EE8EC9D'        
||'8DD4DD7F89ACC8D16228F29BDBB1773D2ED0DA98EAACB64DE3A6A58E7AFAD8227A8A891628D58BC8CAA091EEBDD26777ED6CCEC5DD9BA364EE3823A5'        
||'DC3B3B7166F6AE769609E1AA869F33B7B232E2329045554E5A39152785D564425580D40906FEFDD7BA5DEE9E87ED6D97D39D49DFFB9B6AFF000DEA3E'        
||'F5DC3DA7B53AAF76FF001CDB759FDE9CFF004ACB8487B3683F80E3EB25C9D0FF000C6DC5861E5C95153C751F71FE48F388A7F1FBAF7410FBF75EEBDE'        
||'FDD7BAEEC7DFBAF746A1BE1CF73E336E6C2DDFBE0F5D757ED6ED9F8F5D91F26FA973BD93DB1D75B668BB3BAE3AC32B98DBB98C76D25932324ADB8EBF'        
||'2982CA62F05B66B23832592AA8D23A4A6944F034BEEBDD156F7EEBDD7BDFBAF746D3E2FF00C30EE3F9654FD999CD835BD65B27AEBA6315B6B2FDB7DC'        
||'3DDBDA1B37A77A97AF63DEF9A6DB7B171DB837B6F5A9A683EFF399049293158FA659AA2768E6904620A7A8963F75EE830F905D05DADF16FB977EF40F'        
||'77ED77D9DDA3D6B978F0FBA705FC471799A547ABC7C399C464F119CC24D51455F8FC8D05552E431D90A39E482A696786785DE39158FBAF740D7BF75E'        
||'EBDEFDD7BAF7BF75EEBDFEFBFDF1F7EEBDD0BDDF1D0FDADF197B5B75748F77ED5FEE4F686C9FE07FDE7DB1FC736DEE4FE19FDE4DB747BBB0BFEE6B68'        
||'D657E3E6F363ABE92A3FC9EAE4D1E4F1C9A655745F75EE821F7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD6CF7FF09397D3'        
||'FCC67B8406219BE17764AAD890588EEBEBC7205BFC013FEC3D8BF92E9FBD24AFFBE9BFE3F1F41AE69FF92727FCD41FF1D6EBE84F3552D3432D454542'        
||'C10411B4B3CF34BE38A18906A792491880001F527DC99451C7A01C6924AEB1440B33100002A493C00038927CBA287D99DC394DC32CF82DA55B538FC0'        
||'28786B32B1B4B05765CFE9923A77043C54FF008F4D99FF00B4557826563022FEACA9A98D0AA630BE4CF5C2D78A839A791381206DDCB7159289AF8A99'        
||'071AE51187E05A57C471C188ED0700D06A204474623168C0402F7626EF63F924703FD61FEF3ECCA590200D727513808A2A09F40BC5A9E65B0064E91D'        
||'08220D2B116C34281567634A0F566FC35F255C9381A8F4E0685FEC94EAF1AB55B12F21652E0C20028002C79BFD07B6CB0159EF080AA2BA7F0A01E64F'        
||'E234FC81C28F33E5625C5BD802CCE69AA9DEE4F90FE1527CBE2232C7C828B6D2C11D35588D0CAE2A12F24D7D0C7C7CE98471C7E0B13ED05CF8B2CAB7'        
||'138D3A81D2BE6ABE55FE93716F4C2F91A984463890DAC07505235B0E0EF4CD3FA0BF0AFAE5A99149F51E594E9766BAAF03E8A05F80116C07FB6F6950'        
||'A7D61006446BFCD9FF00CDD2E3A86DEADE46561F9844FF0006AE9BA4808FEA78FF00917B5950C0AB0041C1045411E608F314F2E92A92AC1D09520D41'        
||'06841190411C08E20F11D439223F5BD89FCFE0FF00AFECABC5FDD122A4A7FC51C85563FE82C4D02B1FF7D3134463F035109D25483E58BF7F44CF10FF'        
||'001F8C16651FF12100AB3A01FE8E8013220FED5019146B570DE853EE229E8CFEB6066807E45444BCA8FF00832DC5BFC07B396ED21FF23D1006E07CBA'        
||'6508C655001BDFE9CF047E7DB57B7969616725E5F48B0C518259DC85503E64FF008389F2E97EDFB7DF6E77F1586DB0BCF3CA4058E352CCC7E407F33C'        
||'071240EA635A05B93AE5B7A53EA8841E1987F51EE2D7177CFF00700D9C4F65B58F8EE88315CDDAD0831C3C1920618691F2C2A140EA6984D97B5B68CB'        
||'B84C9B86F47E0B2044D6762D5044B71C524B9422A914740AC01763D36C8B14F7D4ED0CAC6E759678DC9FCEBFAA9FF5F8F628B78F76E5C852D60856F2'        
||'C2250A8B0AAC771128E03C3148A6503CD34484E743939085DCFB17364EF7D733BD86E53316769DDA5B5998F16329066B7663C04824894635A28E9A2A'        
||'69E584E895596FCAB5C9461F86471C1FF607D8836EDD36FDD62336DF209029A30C8746F35911807461E8C01E837BA6D1B9ECD38837389A2661A91AA1'        
||'9245F278E45252453FC48C474D81DE2956453668CFE4920FE19587E411C1FF000F66540453A2EE23A6ECC44893413420F826883A062C446E2461242C'        
||'7FDA4FD3FC2C7F3EC97763DB1EAC905BFC0B9E89F732FE22229A541A9F3A57CBA689649541FE86C7805801FDA171EC9323A4089132F916151834A9FB'        
||'3FCF53D3064A66F190A1C6A52430057907804B7FBD9F7767AE3A6AD6CC46DAC9AD0D2983C7FCDF2E820CF64744CDE32A1FE9A8B963A48BE9FF0003F5'        
||'E7DD2BFC3D1F431E0894D01F2F9F494A6D1354D3310FE033C7E437628C4BDAF7B124FF00AC38F6E44054754BE122A38C13A714E34A70F4F9F5427F3C'        
||'6348BE556EF427FE699755952013E9157B83FAFD6FC7B6B7435BC6F9AA7F97A47B6FFC92E01E9A87F3E80EE91889EC1A69154E94DB7D8A0B15FA91D6'        
||'799FE9F51ED1C1FDA0FCFF00C07A54787FABD7A3AFD470DFAA7E40022C5BAF7AC540FEBA7BD36EBDB9FF005BDAA6F8D3AA3715FB7FC87A2B1DEB0E9D'        
||'A3D557FABEE8EE12BC5CFA28B690BF1C7B6AE3F0D7E7FE4EAD5E3F97F97A01F1D12B64F6A96B368DF5D7732AB5C02D0EFCC6BA5CDC7E473EEB10AC88'        
||'3FA4BFF1E1D5EDCD2E17F3FF0001EB637DA19C44C1632069118A0AA8CDCE8B0FBF947A85FF00C78B7B912E238DCA8269DA2BF6F5014D24A97B70D1AE'        
||'A3E2311EB4AD49FF0063A18F1F345550DD1C92C961C836FECDF9FE80FB24B983345CF424DA3736886B900007CA84E2A6BF69EBD936F0C5F43C2B3B93'        
||'73A48FA117FEBA47BA2C650D3D33F974A25BC5BA8C3B713DA31C5A95A53F33D03F9E066FB8D23516760A2C0DC5EF61F9F6A5860D38F45D1B9D4A5B00'        
||'63F3F53FE0E807DE9816381CD30FDA6930F924622E7F4513AEA2BFE039FAFF005F651BC269D9EEBC8F8327FC71BA126C3378BBF59A83502E22CFDB22'        
||'FF00ABF6744B26A214E79656BB3282100FD2793EF1C3893D659529D2F70B06B32723FCEC2395FCFF005F6D9E94C79E845810514334AEF758F54CC402'        
||'0811AEA36FF1F6F5ADB4979751DA45F14ACAA3ED634FE5C7A726916DE079DFE1452C7F215E903595B515F2B4B51233F24A464931C4BF858D4F03FD7F'        
||'CFBC8FDAB6AB2D9AD56D2C5428032DF89CF9B31E249F4E0380EA20BEBEB8DC6733DCB56BC07928F403CA9FCF89E9C76F6E2C9ED7C94193C5CED1BC4E'        
||'A66A7D47EDEB21BFEE53D444382185C03F553C8208F7ADDB68B2DEED1ACEF943023B5BF121F2653C411FB0F038EAFB76E575B55CADD5A352872BF858'        
||'79861E75FDA388EAC1BA2FA9F03DE3DC94D94DC34FF79B376D6C9C26E9C862DD984797AAC854BC586C455B2104C25F5C950A08D6B1E8FA39F614939F'        
||'B75F6C7D947B1DA1FC3DCAF772BAB68E51C6258D23334A95FC74D2B19FC25F57151D4B3CA3C83B67B95EEAADDEEE9E26DD636305C4919E12BBBB08A2'        
||'6FE8575338FC4134F063D5BDD1C10636960A1C741063E8A962586968E8218A8E929A18C694869E9A9C2A22A8160AAA00F7873732CB793B5D5E334D2C'        
||'84967725DD89C92CCD5624FA927ACE5B78A2B4816DAD116289000A880222818015540007A003AADDFE60FF001CB6B6E7EBACB774EDDC352E337EECA6'        
||'A3ABCFD563295206DD7B66A2AD282B172B0D32812D551B4A93C1504790A2C91B160574E57FDD7FDD4DE768E6C83DBFDD2769B6DDC352C2AEC5BE9EE1'        
||'54BAF8649AAC728528E83B751560010D5C6CFBC3FB71B5EE9CB3373A6DB02C77F63A5A5645A78F0960AC1C01DCF1D43ABFC5A4329245294DF8247420'        
||'3A3A7D2FA9197E9F5FA8F7D109483907AE7EDC82719E8CD75DCB0AE0B26AF2C48C7220AAB488AC47DAAFD031E7D85B76FF007253FD2FF94F469B2C52'        
||'B5AC85518F779027C87A0E9D712C9FDEFC2B6B4D22BD496D4BA40D0DC96FA7B4F311F4520AF974A9609C5CAB18DA95E3A5BFCDD1CED975B431C91792'        
||'B68A3B11FE72AE9D3F3FED6C3D822F55C8C027F23D1DAA3FA1FD87A3418AC8E35E922D192C7C84807D15D4AF7E3F1A5FD86668E4D67B4FEC3D58FA1E'        
||'9933A0CD1B984198106C620651FEDD2FED45BE1BBB1D51BA2E7BBA8EB4896D47586E1BF4D2CE7F3FE0BEC4766E98EE1FB47491C1F4E8A4EF9C7E40F9'        
||'AD8FAF3726D6A2A93FF447B17D8491E3B87ED1D164EAD5E1FCBA2B5B871F90133138FAF1EAE6F45543F3FF0004F62CB7923D3F10FDA3A2B911EBC0FE'        
||'CEB0ED6DB39EDC19DA2C462B1D24D90AA5AC34F0D43C78F494D350CB552449515C638CC8C88C218B56A964D31461A47552F493C31A6B918003D33C4D'        
||'380AFE67C864E0751BFBAD0BCBC837E8A327C0E38E17309F3FE5EA7033D2CA1DBB9EA59E4A134D594D534E0473632AE1929EA2163707C71547AA3B1F'        
||'A0B7BB2E9906B8C823D47F9FCFAC31313438507F6F0FF6B923E5FCBA95431E4E1BC1995AE7A3766B32C4F34006A00C325CD97903D573623FC3DFAA14'        
||'D7CFA7A37728526145238818FCFA1931DB8972D8B9286BB72E6A99D615141515592C94B087863F0405183388A54508565475B28D27D24A8A9111A768'        
||'1C6BE99FF579D7AD344DDD221AB1A01E468380F3E1E54A75DE0F1B5CB5744F96DD382C851C923D355516F5A39B318F612B18D5AA6AA38EB55805B689'        
||'E3F14B1FF659AD6F6E55343692D5F950547F4723F6641F41D16BC0EAC3C28D02D2A32FAAB9A863DCB438A76AB0CD09C74E15BD61BAA2C7E431D80A7C'        
||'06536F56562D649FC2379E3F3D4FA9DFC1472A491B2BC46CC5141A8BAA584A8A40F69D6EE190515A84628C854FCF8F1FC852BC2BD2C5B56834B5C643'        
||'7A38635A79815181C096269C40E1D04397EBFA0DB1185DDDB4F74D1CD7D7FC4B0D9BA4FB3917C974668B2343347F5E352561563C7B73FC65C6A84A90'        
||'3C8A927F9107F9749C2EDEB282EC46AC8C8514E1C4AD38FF004BA9D430EC7ABA7A6828F73458F98B469F6BBAB1955885555BB348D9DC1B57D3D89E35'        
||'4B1456E0F22F665A59C559A3240FE120D7FDA9D27FC3D1842D6E9A623229A9F3A8A71E2D561FE0A79F5CAAF6ECF320FE1EA6AE9E1D2C6AB0B98C767E'        
||'1440BAF5ACB8D766B5B90648D6DF903DBB1B03F69F2383FB0D33F675A591586030AFE78F5C5683CC569D3853C3051C2D38A8A9137A164FE2619E34F1'        
||'FABC91D3CE847ABF240E4F1F4F7A60C68AE283E5FEC1E95C0F12333C64D48CEAF97DA3A5475ED0E18D6646A32FD9FF00C1CCD571D451D2E4769D46E2'        
||'C6962ECEC6478608DA04B90850413023FB3C7B72E5F2AD0C64D050D095F2031526A7CF8AFDBD17C492985A391CB6A381F10A12493845A015A528D5AD'        
||'7CBA5A6EBF901F1EBA4735143DBEF9AEC4C1E768A1A3DB32F4060F19B724A6C9E3EF26567CF52F62409E04944D098A2A7848D68E49170006B799B74F'        
||'021FA16589CB36AF1C338A62817C261F992C78F523722DAADCCD76B329D0AB1D0210AD5EEA962E0E3C9400294F9F52311F357E0AE4268218BA7FE4F5'        
||'6B49AA34126E5EA3A0BA32851AE56C6D415E7D46DF9FA5BD96259F394F886EAD149F586E0FFD661D0E5EDF6C84132C729FB248FF00EB59E867C37C8A'        
||'F8775324191C5F4777E87A5A8A2AAA77ABEC7EB0807DC52CCB510EB34FB69DAFA901E38B5C1BDFDB8DB0F3BBA1492FECE8C0834B79F8114F3B8E9912'        
||'ECE8C19629AA0823F5538FFCE2E846DD5DF755DC1BC71FB9B63F56F5B6077264776E2374E3A7A9D99B357315FBF33156B357D66732D8AA1A492A29AA'        
||'EB5D254A5BA10CEEB24B2022C5F63CB2BB2EDAD697D7B3C90A46D1B0F1652AB0A8A28456760ACA98D59C00428E945CEE8F7978258A18D1D9830A2202'        
||'64273A8855A82D9D3E55E27A49FF0032A4DEDDF38DE98CD76FE57696497132EF697099AD838CC9E1AB37561B72ED9DA5355CF9DDAB9FAEA84A699168'        
||'28D1258320C0CE6A03A0BAE92AE45D8F69B19AEA0DB04C94544749195D50ACB7184755527249CAE174D0F1E8CF76DD6F6E3C396E0A3D4965600A920C'        
||'7164825BC801C7E2D5F2E8AB750E43A7FACF06982A0A8DC38E86ABD7595195A4D35390A89D411255D7C41E00A9E90AAB200AB60A07D7DC812595C2A5'        
||'210A699E3FEA35E88CDCA48DA9EA3A1421187DC19686BB19534B5519923489286AA0AC91907A19C88DDF49FAF2C3FDEBDF8EB8A3A3E3EDEB60AB1AA9'        
||'AF433D36355DD55CE88E144448D9256D46D7BDDACA0DC7E3FDE3D95C93955C649FB3A715474B6C7761767759D2A55F576EA380AAA6A9AEDCB3C32627'        
||'11582BB2388C534D46F1D7D641254C062F1EA8C53CA8A2408CCADF92AB8DAB69DE58AEEF00941013E2614566A104060A6B5CD4134AE7A73EBAF6C007'        
||'B390A1AD714E2060F0271D1C99BB2315B5FE297C759B2384C06F0ACDF1D81DD91C91EE9CE64F0B2D3EE1738DADAC188FE0ED1999AA211F772C4F76D4'        
||'CEC874B580046D535DF3CEEC914AF6CB041674F0D15B525240356B069A4F6823C80078747A2744E59B1D4A242F2DCD7596146AA60692BC464D6B9248'        
||'E820A8F90183C33BC2DD43D7E5CD49D0916E8DF66FAC5A5D4BF7C3F038007F8DB923D8817962E6E06AFAF9E94FF7D41FF40745BF59147FF11E3FF7A9'        
||'BFEB674B1C3F7CE3AAE0793FD16ECE801521847B977E4900466175301C9005B48FA8205BF1ED04DCB32ABD3EBA63FF0036E0AFEDF0FA7D2F50834823'        
||'1FEDA5FF00AD9D62C9F7D1A6A2A88A3EB9D93189E5942CCF9AEC0D6617B4BA58A6695C7D780AD606D6F768F96034819AF2634F2D107FD69A75A7BE3A'        
||'48104401F9CDFF005B7A5B74677043BB1BBBF0326D9DA9818297E38F74D5CB3E1B2DBCA7C94D1AE22928E28A9E3CFE4AAE00AD2C90A4AC202CA480AE'        
||'BA8DCAF997616B23B65C09A5959B70B31475882FC6C493A2346C0048EEA7A8C74BB6BBB564BD4D0883E92E32A64AFC20506B91C71A797E63A211B6FB'        
||'93B57B4E6CDD47606EFAC9F6D4B9BDC5846C63083EE33136CECB4384C266F239048FCCA5453B08A0819230A417D47DC8ADB1ED1B5AA3EDD6E1240010'        
||'73DA1EACC00AD335A92413E9D0763DC2F2E9CC7712165A0AD7CC8A015FB380E99EBB1D8DFBB93C26A2A6A1518959EA5D5A5E0CCDCB81AFEA54585F8F'        
||'A9F6631C92E81AA807C87E5F975A20571D055BC863E8A86A66AEAFA1C3511BB892B65820629A6EE696A2775E18F04DBD98C152C00049F97F97A68D00'        
||'353D0094FD9982C2D44D5344D2E6A7E7C71E2691E587D11E947FBC97C74E430049D2E6FC91ED7FD33CB45E1D33E2AA54F4207C7ADD991DD1F26BA8B7'        
||'0E3A86936A6E8C4E6B275FB73755563E8F70E6B1990936CD6C144B8FC3D604A269242E2058EAD6A5417D410B280C47CD3B7DBBF2EDCADD47E2A516AA'        
||'58A0A788B52594EA000EEC53852A2B5E96EDD772477A851F41EEC8018E55A80038C9C70E877EE3DE3D95B7BB766EC3A8ED0DDBBC7319095571F5BBA3'        
||'3558F5126062CDCF8FA8C2E4E2DB92E3A98D054CF41354352D1470232687562AEA422D8F6DDAA7D8FF00770B58E1451DCB1A0003940C1D758721D55C'        
||'0D4C588350720F4CDE5D5CFD789CCACE49AD598934048A1A5304A9C0A023860F483DE1F3137CD1CF0355751FC7EDCF3C819F219FDC7D759BCD66B232'        
||'213FBD95CBD4674CD532904289A563E9555FA28F6A2DB936D156915EDEC4A38224EAA8A3D15445451F21E753E7D3926E4CD9786173EA50927E65B5E7'        
||'F3CD3A027717CF8EC8C2B552D3744FC5A9A253E48A293A9F25272BE9D2193357D26C6E0704FD6FEDE9395624C8BEBE247FCBC8CFFD53EAB1DDABF186'        
||'1C9FF7D9C7D9DDD19FF8A1F2B7B4BB9BADFB2773D7EDCD95D7B41B177153CD47B77A7367D6E062DD195A5C17DF252CB40F555B51555B2BB5341188EC'        
||'BA02AB29E08C5DF7A6CC59EFF671F8934B5B726B33F88DFDAB8A03A5683CE94E353D4C3C84AABB74FA5556B2F051A47C0BE553FB7A42E27BAF726FEE'        
||'C2DB3B1FE4EFC3AECF8DC554950CFD97B4374E170F89AEACC4CD8BCBEE6872F91A3A67A55C753C85A1AB8EA8B2BAB2D3FEECB1DE1A0DAB04743C20AE'        
||'474E99DDB193D879A7D8DD27BBAA7194C71F4F36DED83B9F78E52AEA307BA771AB53D0D0267B3B592BC9164A4586512329FB795AA9565903944DF0C0'        
||'EB5F33D3775DF486F2EC5DEBBA309F3E7A8B0528C4E0293706DEDE180DE30D6EC3C665A0A7A7A6ACA8D93B936C57A7DAE5AA4E87AF9A7816714304B4'        
||'D2C71C72C9E7D71F8BAF70183D0C98DEEAD9DF1EFAFB79773EDFCB6F3EE5EBAD8FB6D737D4F8BA18A923CCA631843B7FEDD5F1D0B4D97CC6465534F8'        
||'AACA7A35FB7C54861749259A676DF015F2EBD424E9E93BB33E4CE2778ECCDC3BDBE3C75AEC3DF1F2BBB931D5FB8B71D4B56D2ED0C0E3AA3175430F87'        
||'C97656EE789A4C0ED5A6F1474D88DB903FDE64EA5E49245FB86C85553E81AF0193D6E9E470074CF8AEB5F915F25B646FEE91EEECDF6274F6FAC3D1ED'        
||'EDE797DE158F4793D97BEF3147955CC4D89C78A05971551B6249299DEBE831D359A95E380E88A097C9EC9C1C75EAE6A3A1276D751ED0DDDB129B60FC'        
||'D8DA35197CE63373A6CAEBADAD8BDDD519F9F7264B2B1C55C77DE2B3BB66A96A6A047471D4D7E29AA96187174CB154CF12CD25977F6F5E06991D3F6F'        
||'0C27C6FF008E3B0297AABB0E08F7F56F6EEE8CA6D9EB6D9994C2C75DBC37FD6C15104785DAF491D6'                                                
;                                                                                                                                   
BRANDINGBackgroundIMAGE_4 raw (32676) :=                                                                                            
'4B549F6F422446C8E5AA62A5A283524B1434B3931CFEC0C1EB5939E8AF6C0DABD23F1CF3B47F1A0E67B0B7267FB2EA376E5DBACAB22832B80A8A886B'          
||'9321A3B6BB17214A98CA8A5A183EC69129614AAC855BC2B2CFE059BF6B4283B7ADE5BB8F4FFBCB79FC7DF8BF97D91F1A70F98DB357B8F7E653706E2D'        
||'C741B981CA6CEDB9B4A2A392A77257F62E732517DAE46B628920A3A6C5DE7AE9C3C54EF152C72493C9EA818EB5466CF55EFDDBD9DB8BA9FBC71DBA76'        
||'0F5064F66F5DE3F636DBD9981C4E7CD3367B71602096AEB177457EDF79A45A37AE96A6A7ECA86A6A0E569E140F2B5E528275F6DF545B009B46AA4D27'        
||'DBF87F6FD9D2C5712D8C961E218C49F1119C020D0F9D2B4AD280D07A74B4E925EC3DC184ADDCFB7E486970F5F5B341052CD9574AA8BED488C0AAA22A'        
||'AF1106FE891411C1E4104C950091D75D307857FD58E915CDD5843A6DA42752015A0C1F983FEAF4E877483B4502A4B5B441D6E496CA3F173C7D7DBE63'        
||'6A701D2517FB7134AB7ECEA6247D964ADF2741E96426F936D4486E6C7FD6FA7F5F7AF0DBD075717DB78FE2FD9D3AC11763F1AB2D8D5B5BFE5E87EAA6'        
||'EA7D47F1FD7DEB41F41D6FF78589F26FD9D04B94EA5ED7CBE5EA6A8E636BD5E3EB729495B530556E1FB795E3A59CCEA01911D6E090403FD3DBF632FD'        
||'2DE2CF265457038E411D31BC5CDA5FED4D670556434A12318209AD33C0745577E7426D8A3CA64693B17A73AAFB273F8DA2C0C15BB92B33791823C253'        
||'E79274C3C191CDC38C6A58F5988C74C25A9579183044F431F64778B6F25CB19635666248A9CD2BF679743BD92D67DC36A866B5721631A08280E500D5'        
||'4A9AD3CF87DB4E931D73F1F778ECAA2EE9DA9B5B6374C50E07B5F0F8BC10C6D46E1A882962A38AA1E5A7A45271CE323E612BAC2B3581663A439247B6'        
||'E3D16EAC822401FCBD47A1C67F3E84A56CAE65DBA75894FD1485A5A22FEB0200A3790183C6BF675CF1DF16B7FE67294741B77A43A65E6A3ABA88842F'        
||'95D9D8F972F2D3D31A7A9868A83238B4AA6116B1210B186B2EAE50DFDEDE6B5143E02003E43FE81E872FBBECADA255B245D26A404435C114F80E3CF1'        
||'9E9EB6E7C3AF911B0BB2B19BD29FA4FA906429F74D1EEAC56DBFEF66D35CB1A2FE0B518B5A5A0C6FD86A7490CDF70024650AC6E594A82DED93756E1F'        
||'5A44828C0D7D28284574F1E8977BBADBB7BDAE7DBED20481A6A51E8015A52B4A203E5E4471E8EDECEF815D3D8EACC564B707566DA96BF17B963DEB4F'        
||'8CA0EEDC9C384A6DDE7154F455B929284500F24D2CD13F95CB85643E364642EACA8DA5B141A069CEAF5EE3E7C47963867F3EA0B7DF2E16620B020294'        
||'AF8601D209A0E3EBE7C7A35B45D29B1E2C2E3B05174574D3E3B175CD5F8EC6FF0078762AD25254CAE6792B23806D5F124DE5B32B784B2FF61D012A54'        
||'34B31885BF8CE235E09DDA453851755053A416BB83D8DD9DCED1BC3BA34FD55521E80529AEBABF2AF4FD4BD1DB420ADADC80E89EAD92A33302C394AA'        
||'3BDF0ECD94D0BAE3A6C8A478105E20C40551208C0E4440129EDB4B9B9843086E1D3571A54540AF9EAF9F4BB71DEEE3799E39B7691AE5A11D8645D4CA'        
||'69C56A719F3E3D2CA9BA7F1B8FA5C3D3637A07A56866C734F2D1490EE0DA6A31F0A2471532D069DBAA11C02E7D11C66C4FAD81E13C72786DAD6771A7'        
||'8501047FC6B1E9D56EF98771BFB5FA2BD9E5962E051CBB253CBB4B1040F4E9B721D2F889E6C9C959D09D10F4F928EA8CE9264F6BD47DFD43C5A69AA3'        
||'24AD840D23F0B779257360010CA2DEEE65D64B34CEC588A93E7F69AE718CF5BB5E62DC6CA03656573341010408D0BAA50E3E10C169EB41D42AAEB4A7'        
||'AB8F16F53D47D332D450D3CB4934EF93DA33554141254291434AEF86E2208BCA0082E4FD7821F79E495B54B2C8D4E152C69EBC4E2BF2F4E936DFBACF'        
||'B41276A99ED59F2E62D51EAFB4A904FE7D60DA5D71B77696FADB79AC7755758ED68B139BCF4B83A8DAD91C2AD4620E636D65692AE4A3C6E2A8E98079'        
||'926314CDAD8E9637BF2DEF570F3DC6DC6C7C673180A749274D43A9E15F5A9FB7E7D475EE96E71272AEE1B94C7C49A630A9765ABB7EB21EE739C00724'        
||'D3CBCFA36382CD447214491D848D5D008DB5100B3CCB6B86B703F1EC25BA6DB20B691DB8043FC81FF57A758CFB16F70B5EC4919EEF1169C7CD87FA87'        
||'9F42B53EE3A7541AF822EA2C410AA3F49078E4900DBFD71FE1EC0B2ED536AC0AF528DBEFB6FE1E71F9FF00ABFD551D651B9298396063BF894AD98690'        
||'57D2F6205C5EDCFF00ADF93EDBFDD5705781A57A706F96C1AB515D3EBF91F98F4EB8D4EE280AA39BB2ADD558B5CDAFEAD47F55BFA0FF006FF4F768B6'        
||'A9D89518F3FF005797559F7CB60039FB3FD5E78E811EE7ED1C16C3D83B9F7CE6D07F09DAB89AFCDD6468FEA9C53C2A29686027EAF5551E3A78D6F7D6'        
||'FF00EBFB1E72672D5D6E1BC45B7C668D290B5F4192C7FDAAD4FECEA3EE6EDF966B358AC935CB238445F2D6E42AF98056BDCD914018F5A8B7726FBC96'        
||'E3C96E3ED9DFD5B3C12EE4DD55F2D4D58A2A8968AAB2F5ECD95CD52D1D4DC46131B42B689351BC688B61C1F7965BF5CD86CD61044CFE0DBDBE985411'        
||'9A05ABB0A501291826A704E3891D0EBDB7B48B6668F97B6E55BABA9633733112A23C710D62191D092F4BAB9D45A80682E49240A1AB5DC5983BBB7165'        
||'B3153078A1A895969A91A421E8E822F4D246F25C8328403C9A4DB516B71EF13774BD1B8EE335EA82AB2312A0E485E0A0FCE94AFCEBD6492860A15C82'        
||'C29522B427E55F2F4E91E94934B5378575DDB507FD37BD81057F16FEB7F65E3AD64E71D6CB7FC893E1DE0B399FCB7CC1EC6A6A4ABA7EBBDC155B4FA9'        
||'76F54AF9D5F7EC58C8AB72FBEEB84C0A95C443511C38D8CA9BD54AF37A4D3445936F0D756B6220814AB5D29AB7A440E9603CC162284FF0823CFA0273'        
||'6F30436D22ECE0D0B28793EC35D0BF612A59BE414799EB66EDD7BDE39E8AA686227F763BBEA7D7C860CAC45FFC08E4FD79F64FB2EC0E2E12793C8F51'        
||'2731F36466D24B58CD2A33E7FEAE14E9794F552D66C69A73EA95A851958D94D828626C0F16FC9FF63FEB87DE358B7CF0FC8391FCFA1EC33B5CF2A19B'        
||'8B18811FB2BD62DBBBC28E6A28FC725DA33E320C87492A48FE839FF89FF1F7BDC7699E0B821C71CFEDE8BB66E62B7B8B4051BE1C71F31FB3FE2FE7D4'        
||'DDCB948739B672B898E45325547468A471A7456C5201620F164E45FDAEE5281ED399AD2E186159FF00EADBFF009FA3E17D16E119B3422AE295FC89FF'        
||'0027E5D02E9B332F34494B1D453C4A1EFE48CE82A2DC6A2C49373F4B1B8FE9EE7FFDE96CA75904FCBA43FB9AF1D3C30C17D08F4F9FFB1D26EBF69EEC'        
||'A495637AF962A5120569A36575F11600B024FD403F4FCFF5F6BA2DC76E752C1016A607CFA2E9F69DDC48A8662AB5156001C573F9D3AB48F8DE9B6F67'        
||'ED9C7D1E3F226BA5C828AC96A6AC4095AEC4FEE09962038537038E07E7DC03CE12DEEE1B8492DC268D3DA00AE91F657D7AC95E56DBECB6FD9E2B7B29'        
||'3C4046A2C69A893EA001E7D1C6A4DD142538A88B82A490DF83CDBD809ED640787423A5788E94F4BBB693C634C91B0B82483F407EB6B7B61ADE4AF0E9'        
||'2B5AA13C4F50325BA69D25F20A81103C00081C37D751FF0013CFF87BBA5B39F2E9D8E10AB4E3D341ADC7D6B19A6A8460F72C124B5EFF004B95FF0088'        
||'F6E699130A3A7B46287AC35B98C6D226A85A350ABA7F503650B6E357E7DD92291CF775AD2470E805DF5BE71D4A934B155C50B2724DC02CC0581E48FC'        
||'FB3FB0B091C80571D559D6204B1C8F2E8A7EE3EFCA9A29248D2AB508D81B2901B4F3F426FF00D2E3D8B6DF97D2400D38F45B2EEA22C0201E826CB7C9'        
||'7C9339FF002B758C0D2ABA94006D6B96FA9FEBF5FE9ECE21E558C0F86BD164BCC6A0E8634FF57D9D246A7E4456C962F5248D57F59626FF00E1FD4DCD'        
||'FEB6F6B979640C05E90373142DF8ABF9F1C74E18CF9252D1BA133BB00C350B8B9B1B91EAB707F1FEDBDD25E57325469EB6BCC76F150EAA7CFCBFD5E5'        
||'D536FF00334D9D2FC8EEF0DB1D8D4DB8E9F0D1E07A6F09B524A0971F356B492E1B77EE0DC0F309639902865C82A01A38D37B9FA0997DB7BA3CADB1BE'        
||'D724258CB72F203A80C3C70A70A1FE0AF4925922DFD1B7082414881438AE52AC471F461D54C1D872001BF8A47EA17FF80AFF004FADAFAFDCBC2E07A7'        
||'4033BD2D2BE19CFCFF00D8E97DB63E3B54EEEA04C9A6EDA5A00F513D37864C44F5047DB900BF91275FD57FA5B8F69E6E704DA1BE91ADCC94EEA86038'        
||'F95349F4E88371F6DA5E79BAFDFD1DE2DB06511E831973FA78AEA0CA335E14C7433ECFF88F5B85C8AE65B7E50D4AC54D510FDB2E02AA366FB840A1BC'        
||'86A481A6D7FA7B20DDBDC08AFAD4DA0B465A906BE203C3E5A7A3BE51F68E6E57DE97787BF59C2A3A6811153DE00AEA2E787D99E815F9BD80DC3D3BF1'        
||'7BB737AEDECF16CB5261283114F5347473D354D0C7B9B354DB7AB2BA09C484C6F1C351214702EAC4116201F714FB89CD37563C957F7162A639590461'        
||'81CA891951985060E96201F2AD7A9663B10B20766AD3E5D6AAE0050001603E83DE0DF0C7479D1A2F855DA3BB7A77E56F42EF7D9957534F948BB33696'        
||'DFAFA4A777099DDB9BAF350EDDDC7B7AB625204B0D65254CB1146B80FA1C7AD1483BE5BBCB8B1DF6D67B624378A8A69E6ACC15948F304123AA38186F'        
||'423F9FF93CFF002EBEA2BF1C68693FB9AD9644BD54CD4F8F5662AEF0D1D353ACAB1071FEAD9B535BEB61FD3DCDDEE25C4CB756DB757F4D23D74F562C'        
||'56A7E600A0F4A9EA2FE5EB5806F5BD5F800C8D772475F30AB46A7E64D4FAD07A7461BDC75D0BBA04BB771304D2E0B211A2AD54AF53433BA8B34B0448'        
||'B34264B7D7412C17FC0DBE9EE51F6E6F650B7564C6B1A85703D1892A69F6D013F675107BA563096B3DC1452462F1B1F32AA032D7D749240F91A7A741'        
||'E57EDF923C1BD5C24332C52310BA8E9D22E431B707FAFB90A3B906E0237AF517343FA3A97D3AAE7EE9ED1931D254E1ED2B48AEBAD9184643C4D709A8'        
||'FE3FAFF88F723ECFB6075139A53A0C5E5DE96318AFCBA0AB6AEECDE3BEABF1586DBB8BCE6E2CE55830418AC2D0D6E56B66D4D6D1F674292487E972DA'        
||'6C0737007B31BC8AC36F8DE7BA7489064B390A3F692074D402EEE5956156763E4A2A7F60EACF3A67E07F79EEC8B1794DF9578DEB3C60453514554DFC'        
||'5F75CE8EC5CC63138E90450480597FCA2A9581FAAF047B8AB7DF71F97ECCB45B72B5DBF911DB18FF006CC2A47FA553F6F437DB7953739943DE110027'        
||'81EE7CF900A680FDADD1F66E97F8C3D3743413760FF04CFE5F1D1A4D4AFBCCC3B932B34C97612526D3810C37B8050BD310A79D77E7DC5F73CD9CC9BB'        
||'330B5730C6FC4464A0A7CDEBA8FCE8DF97431B7D836AB12A6650EEA38B80C491FD1181F2C7E7D23B787CCA86955E87AD76AC54D1C319A7A5CDEE68E3'        
||'D31C6A34A1A2DBD44C238D47F644B29007FBAFD928DBC07A5CB6A278853F9FC4727E78AFCFA3569C835451E80B0C8FB0701F2E806DB9F2437B507666'        
||'D4DF5BD7756432F4F8DCA8827A29A78E9E8A3C5E4D4D0E529B178D83C70A3B44E590469A999579240F6BADEC56E2392C608C2AC8BE95A11956639E04'        
||'649C52BD332CA534DD3B650F9FA70341C05470A74BBF903D17B9F7477CED9A4EBBC61A9D9DDCC959BBCEEF0047B5B67D2D1C695DBD32198AB8CD942A'        
||'4A95D494E2DE6FB858D58598AA4DB1615DBE77BD9446F6B45F0F8C9216348D501F982ACDF84AD69D184EDFA91BDBA97598D4B700A40AB6AFCB2079EA'        
||'A0E883FCC9F955D6FD1F8C9BAABA3B17460C70AC53E5A641266B7954C3E897756F1CA8FDD344D2AB363F168E91BF123284D36953DBFF006FAFF989C5'        
||'DEE8592DD4F769C67FDF683D7F8E4C91C0751CF3BF3A58F2EC4C2D82BDC483B01E2053E293D17F813CF8F5AECF657686EEDFF9CABCA6E3CDD7656AE7'        
||'959F5554C4C54E9ABD30D353ADA38D07E2345007BCC8E5EE5BDAB62B25B5DBE158954530327E64F127E673D609F39F3B6EFBEEE0D35E4ACC48A52A74'        
||'81E8070E96FF0011D7CFF287A1D659844ADD99B6C19A52C52206AEE5D945CDBFD6F64BEE95CCB63EDC6F5776D035CBC7692958A3D21E421708A5885A'        
||'938A9200F3E8F3D825FAEF7AB96A0B8996157DC2DC191EA550190558D2A680570075B5553C981A26FD35597981E357F91515FF00AE9179187FAFEF9A'        
||'8C9EE76FCBDCD6DB0407F87FC72EE9F69D3021FB3550FAF5DAF7FEA4ED6C447E36EB28F33FE2F6F5FB3BA561F6D2BF2E9F23DC55AC12281A2C7D2865'        
||'3F6F411AC0A4022E24907ADBFC6E7DB963EDBF2E5B5C8DC775126EB783FD1EF9CCE41FE846690C63D02A63D7A4179CD3BA5C446D6CF459407FD0EDD7'        
||'C307FD330ABB1F996EBC65D12CAA39FDC93EA7EBEBFADCFF00C4FB1BD0690060003EC1F21E9D06B233C6BFB7AFFFD3F9FF00FBF75EEAD6FF0093F7FD'        
||'947F61FF00E204DCBFFBF176B1F72BFB3A09E669E9FF0028B27FD5C87A19F23506ED257FDF2DFF001F8FADE27F946E336EF60F6D7787416EBC99A0C2'        
||'F71F4CD0D4AC32E9929ABB2BD67D858BDCD4911826FDB66F04D517D43840D6BDEDEE63E7DBBB9D9F6EB6E64B68FC63B7CFA88AE9204F0C9013A8696E'        
||'D668E946A16A54015E86BCC534B676B1EE9128736D2645698911A30714350D42287EDC0C9F3EEDEC2C4EF2F89FF23BE4DE6374C4D5FBA772F637C47C'        
||'851CD36BAD4DA357F316AEA28615A9494934F0EDEAEFB3F1088114E8AE0E93EC836682DEC799B6BE5EFA56A5B2C37C5D4294A2EDC206A214004CD711'        
||'893534853C463AB80A20B28A28375B5DACC47F4825C1614A516D821A291890CAA1AA58AEB26A3C82AFE5E647214B47D89856EBBECA5EA37EEAF8810F'        
||'516F4EC0EC2D97274FE29A1DC3895DB937C5DD9988C6C55C88601343968DF20EC1C179594322FB7BDB96B3937AB09E49E06BBFA2DC4DC476D14893B3'        
||'98EE357EF095A4646646D2C9A2350C34AA0A827AB72C3C2F7F6F2BC9199BC1BA322C4ACAE58AC849B9666209534A6955C502799E9D7E4AD1F70566DB'        
||'F96507706E48F6753D5FCCCF8ECFF09F77EF34A16DA9B733AD98C59C5673022962959A8A298564995D31B48C7CE24FAB9F64DCA636737BB58D9F4155'        
||'DA6E8EE2232FE238087546E35A132BB51101754350AA40A0E916CFF426E2D16CE85459CBF5416BAB8641AB025CFC280900161A453A0CFBDFAA25DE9B'        
||'4F7FF6B7C8BE8BC2F487C82EA5EFBF8FF87C3774F5B652BF0FD6BF2E6A331DA588C43E5A0D9756E24AB5FB63E659AAD26929D9A375AA0B154D3A3DB4'        
||'6E30DB4906D7CBFB8BDDD85C5ADE1FA7982B4F602380BC60C8A6803B111088B920AB0D24F86E6F677291BC769B75CB4D6F24331F05C0325BE888B2F7'        
||'8C02DF069A83514A534B1157E5AC1DAD1E3FE709EEA8F231F56C7DE1F0FA5F8B6B995A48F0D16506E2C6AE7A5D8490D8EA6AD61FC54FEA690B87F4EB'        
||'F6879024DB7F78EC2BB594376D05F0BBA54920C6DE1890E4EA040F081A05E2A323A4DCBA6D0DCEDC2D286631CFE350F9693A759A935A81A41A6483C3'        
||'A62EEEDE936FBA9FE639B57B7370CFB8FAD3AD3E4BFC4A8B6BE1B2F251CD8AD9D86C8C787C9E69F091326989659E4F2D4C8751219F5B5AD64FCBF6E2'        
||'D0F2DDD6DF188E6B8B6BF5908521A5D224081CA9D4C47E12738006074D6DD178236B96D9023CB15C062A0D5E9AA80FA9F30388F2F4E95DDEF94CE9EC'        
||'ADBD87DCBD7DD9F49B38FCFF00E8E5EB2DE9DABBF769566DCA74831C02AFC78DA18CC6C15506D69A8D6459964AC902CFAD998D8AFBDF2CC50C7B55C5'        
||'C43730F8FF00BAAEFC58A289D5EA453FC664F10A99C3569A955993550004577B62462CA4659632FF004731648D086D5EB2B1623C507854574FDBD02F'        
||'D5FDC1BF3B2FB27F99C7DDC9D9FBBBB03ACB33B3361F4A6D2F8F351B4365F6DEDBEB1A2EDCC9C59CC475164AA686A96056658AA7392494D3BCF0AB5E'        
||'4F2BA3299EFBB55A6D36BB0784D1C714FE23DCB5D2B4D6EF21B58CC725C6A65D721632041A80560A54018E96EE9656F649B7A21558DF5199A60CF1B3'        
||'7849A5A4CE49AB051E440201E1D55CFF00331A7CC54FCBBDEFB932FD7F1F5F5467B1FB52932B4B439BC76E8C765B77E0F6BD1A6E3AD8F736169E9A8A'        
||'AAB562A8C78CA4504778A73FB875B9F63EE4CB436DCAB66AB319E361218A431B447C3F1640ABA5C97A02182549AC7A40C0143FD8E168B678591CCA86'        
||'A55CA14AA6B22801F42081439140053AD67BF941E1325B97F9947CF6DB986EA9FF004EF97DC1F0D3F9A461315D1FE5DD94E7B972595EABDC14141D53'        
||'E7D87598FCE47FDE29644C3EBC2D7D3570FB8BD25443502391712B7DFF0092DDE1FF0087CBFF00571BA86F71FF009284FF00F351FF00E3C7AB458FAB'        
||'B2DB877B741F461E8AA2F8F7DF79FF00E47BFCDBFA7B6A7F2C8DBF94AADEF97E90DF15985DF79CEAB87686DFDD9364F7A1DC5D95575F57B9D71DBA32'        
||'993CDBD54724F14E290D2C3015748FAC7B1367F70FC6ADDDF1A31F55F1EFBE17B0B6AFFC27EBAAB6BF6ED17C62ABA3D97FCC53E3F546EAFE615BD303'        
||'57DB7F1DF67E428E6ABAFDCD829E3C762F76E2D8515752E22A2AD27AEC7AC13A9F75EEA2F7BF5C4FB57B17B070FBD33FB2BB337449FF000A50FE5613'        
||'67377613ACB0FD67256C5D83F14B736F1CCED9DDDD7F472554588DC34B255478EDF18B49D93F8FD2D79906A165F75EE977B77B1705F22FB4FA4283E6'        
||'4E6B6BEEEEACE94FF853D761F4675ED1EF6C2ED6C56CAEB7EA29766D5E670BD4587A1A2A3A7A1C7ED6ACCFD0E225ADC4F8D68657064A85F5CAE7DD7B'        
||'AC58FEC3F9112F63FC49DC1DA9F11FE566EFEFCD83B0BF9A4E531B96F927DF9B2AB7F998E5FAAEA362E2B696E6CA7C63C6E1762505452E576065E5CB'        
||'657A7DEA3135504D34598C7E024A6A5A390C3EEBDD4AC3C5DC5D65F307E526F4A3A7F961DD598EC0F87DFCB672DB9BBC3E3C75E6DEEBBFE6E7F1BA93'        
||'7C6729EBF6CD476FF4950AE54EE8DC39087110E1FBACBE468E4CBD0D7E2CD55640723E26F75EEA93F6BEC8C975C7FC296BA8F64E6B79F5DF6266B6FF'        
||'00F38BE89A5CCEF6EA8DB38AD91B037165A6F96583ACC964301B2B0535451E1AF3C9225562696A258A8EA44D4C9248B1076F75EEAEFBE1676B6FDD87'        
||'DB5FCA3BAA7696723C3F5EFCA3FE697FCE03AB7E43ED84C3E0EAA8FB8BACA0DE5B0F1CBD79BEA6AFA5966ADC1BC798AF32E29E5FB596578E792269E9'        
||'E9E487DD7BA04FE006E1EC0EE3F89BFC80B0FDAB5757BD7E2CEC9F9FBF30FAEFB1E9B2F49B325EBFC2F6961F6C6D1CC7C23D87BD2B772D2CB8EA7A8A'        
||'CCDFF174C2D3E595A1AA0D38A849A3000F75EE97BB5376F666F1EF7F836BD8DD3FDEFB47E6F52FC44FE6E537C48DDDF3BBB4B66F617CCAEC8EFC93AC'        
||'A6A5F8978EDDD80CAED9DAF538C5C1EF0A3DC74BD5B539F8267AAA89BC38B95131D448BEEBDD3D766EF2EFFE98DE7B87B7BB2E930389F979B17FE12B'        
||'94DBF7776E1CD6D4D8BB833F82EF9A5FE675161ABB7BEE6A0A8A5A9A06DEB42EC4E52B6A617AC872D1CCD527EEE3900F75EE9D7F96D6C4F901DA5F0C'        
||'FAD60DF12F79FC82DA3F35BE227F352DEB5B4BD7788D9FB4FE29E57BF77A4DBE862F6A7C9979686AEA3B43BA2B33F884AEDA340ED8BACC1D14B1C98B'        
||'FBBA4C75796F75EE911B5B7877A76DFC24F877B9B7B55E677AF46C1FF09F0FE6D787A7CFD762B175DB429FBCFADF6AF69EDFA2C44795A7834C79CC46'        
||'DAA4C32FDB090490C290CACA2421CFBAF74126E4A1DF5FF0DEDB8F1357B77271FF002AD8FF00915752EF0D9F9E9F03808FA47FE1CC6A77A6DC837064'        
||'F07BB5E3101ED27EC839FC456D3455073470E1E296138E58641EEBDD3DFF00308ED6DFBBAFA6FF009D77C7CCFE723C874B747FC2AFE49FDA5D4BD747'        
||'0F82A7DBFB0FB337A53742506EEEC3C0C5474B1C899CCA52EE4CD50E432AD2B554F4953F692CAF4D141145EEBDD5477C16EB3EC6F92FFCABFF0098BF'        
||'C6BF8EDB1775769FC841F223E0E7783F53F5DE16B372F64EFF00E9CD9B47D89B2372E436AECFDBCB3E57350EDDCCEE0C2D4E4E9A0A590524759155D8'        
||'A895A3F75EEAD8FA0BA87E677C7CC4FCA3EBAF8A7DBBBF7E57FF00328DA1F307F9626CEF93DBB767455BD89DB9B53E2BD57C77CFC3D8BD2BB86B7395'        
||'F99AC9F67EC9DD836D75FEFD921AB38CA5871B4B45944A3C7532C50FBAF74727A93B3703D3DF23FE1D6C2F88D9DC36D7F8F3DF9FF0A24FE60FD37B9E'        
||'836762B6CCDB7BB2BA06A331D7B881D5CF9414B23D5EC99E0C8D5A53E2639FECAA69FED25923964A7A6963F75EE8BDFC12EC5DD9D7B4BFC9EFE3FECA'        
||'ACC6EDEE9CF91F98FE735B13BEF61506DBDB4311DBDB1B6D67F72C5B7B656FF9A5A47A9C962282C24A3C654CED4D0CA04D1C6B280E3DD7BA48750669'        
||'365FF2D5F8FF008BEAFE9CEEEED3F8BDBB3F9477CB8CBFC92A8A2ED9D95B03E0950F7BD5F62F60E237CEECEF6A3CB6D5C8565676E6DAAF6D9CFB34B6'        
||'7A9B26E0E07198058D6A256AAF75EE80DFE6259EF9A3B03E38E6F70F4B47B1305FCAC7607407F2A9DE5F11375EF2C04DFC1E2EDF8BAD36A65F72E5BE'        
||'1FE7A3A66C5D67617F7D6977A4DDA74B5323D57D84156D9587CF1D191EEBDD118FF851E64BBB339FCCFB7BEE2EDF6DC157B7B72F4E7436E1E82CD662'        
||'92961C46E4E92CBF5CD2D652E7B6557514691576266DD0DBA156B11E406A92AA2D7FB5A13DD7BAA20F7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEB'        
||'DD7BDFBAF75EF7EEBDD6CEDFF093F6862FE62DDC7533C890C54BF0B3B32A24964758E28A35EE6EBD496495DB80AA8CC49278FAFB17F250AEEB201C7C'        
||'26FF008FC7D06F9A016DBD1545499147FC65BADDAFB2BB0AA777D4C98DC549245B629DC2C4A0B44F97994F390AA5E0F87FE38447F1EB6F515D324BD2'        
||'6DCE1DA236A1D2D34C788D08CA8B18F52D23558E568854D6A47425E56E5F5D8761B8E68BE40D70CE96D00AFF0066F22348F27C8A44B40410D59432D2'        
||'81BA0C61A15B296D2ABCAEB7B8416FAAAA8FE9FD05FD9DDC5E25ABADB42A64964A9541C4D38BBB1E0831A9CF9D15433517ABDBD94D7A8F773B88E086'        
||'819CF004FC31C6A3E276CE945A0A02CC55416133C71A002155247FBB64404DFF00AC719B81FEB9B9FF005BDDE085E3AC93B6B91B89181FE9547928F4'        
||'E278B127A4F7370B29115BAF870AFC2B5A93FD2738D4E4713803828031D6392964928E763EA295311666259BF723643CFF00B0F6C1905DDF7823FB28'        
||'285BD1A5C32AFD91821D87F1141F84F4AD57E82C04ED89AE4109EAB0D4ABBFC8CA41443FC0B21FC40F4F1B6A84FDAD6DD79FB88FE83FE6D7F4F76BF7'        
||'FD45FB0FF87A6EC7E061F31D39C94F6964B8D568A3B9B722ECDF8FF61EC8217FF773381E50C1FF001F9FA14CEA7FABB6AC7CEE2E7F9476BFE7E9BE68'        
||'AD7E38FA8FF5BFA7B3806BD1270343D354B1D89E3EBFEF7FD4FBADC35BADBBB5E1510E921F590134914218B76D08E35C74A6D16EDEEE35DBC39B8D40'        
||'C6230C64D60D54A05EED40E45335EA1E928F1CEAC54C443DC5CF286E2C47B8B21E79960BA7D879713F7AAC8E23B598B148509524C134EE0095A3D354'        
||'319679128A4EB524CE371EDAC373609CCBCDF27EE468E332DF5B04125CC8A1940B9B7B54626159B501209424714B570046C1578D6031CB13C423D154'        
||'AB36B44B0F5925E300DEDA4DC7D7D9FD9F27DC5F4C373E74B8FDE17119D51C2069B484F9688BFD1187FBF24A9A7979F418BCF702D76EB67D97DBBB53'        
||'B4DAB8D12DC3307BFB91C0F8938FEC9185098A1D22BF88F0E9BE553A8FD4DF9E3927D8D070C79751F7CCFF00ABE7D45928AA5B9482620F2088D80FEB'        
||'FA8F1FEF3EEE1D7D7AF02063AC7E2AB894ABAC0D093FB94F515106865FEA016BA91F82BCFB28DC765B1DC65178A5A0BB5144B887B655F931A11227AC'        
||'728743E401CF47BB57325FED509B0216EAC9CD5ED6605E163FC4A2A1A293D248991C7A918306A3151CEFAA82A2066D05A4A47983CD169173A1D01D63'        
||'FD617FF0F6861DFAF3682B6BCD6A1149D29791822DE424D144A0926DA43C28E4C4C7E193217A379B62B0DE55AF393D9DC81A9ECA435BA8C015631300'        
||'16EA31C6A804CA3E38A80B7490CBC24341189E271A1C811BB911B7900624328B5EC3FDB7B38DCFB96334FE2FCFE1FDBD459BEC8824883035EEF2CE0E'        
||'7ECA74DFF6FE822F723F3F5000FA0E3D92B29D3D14473A78C1684FDBFE5E9339A8CAD348C5C202874DF906FF009FF8D9F74A9033D19C0A1E60D1824F'        
||'134F2F5FF50E8AAEE5C94B0E4DE148A6917CABADF52C634EABB1058FD00FF5BFC3DE971F9F47E478B18756155F5071FCBA91808AB32956869E5378A6'        
||'8551A360CB4EC1C58A8FF5ADF51EDC4917C415E1D317769751D9380DDC41391C71C3FCDD5187CF5A09E97E566EAA7A87F2CCBD5BD4ED23F3CB355E7D'        
||'588FF1241E3DB5BA10D7CC47F0AFF97A2EDA811B5421BFA7FE1E824E948C0DF54C0051A76CF6371C1E4759664FFC47B4B07F683F3FF01E959E1FEAF5'        
||'E8E3F58268EACEEDE387D9BD628C6E7FE7F3E0DC03FF0024FB507E35FCFAA9E23FD5E47A2D7DF74C1364750BD97F7B73F7359AFF00EA13698363C7D6'        
||'D6F749FF000FE7FE4EB75C9FCBFCBD175A4A2A8ACAEDB94944D1255546F2D890D349292214A8937AE3FC2D290090B7B6AB0FA7F5F7584132C6A38974'        
||'FF008F0EAD0B2C72891B82D49FB003D5D443BD31D8AC6D0C751532A55C29209C46A5E3323D6C8058FD6C6FFD3DC972496A6DD44FF1002B415F3EA0AD'        
||'D2003769CD9123B8FA01EBEB8C7435EC5DFB43915063AB5263255D89914AE8E189FEB6245EDFD3D944C76D762C18823E47A53670730A46238A2D6AF4'        
||'03233E9FE0FD9D09990CC53D6C2CCB588434609B386F4E9205C11F91ED14AF6A49A3F90E8E6D2CB7E54A1B53863E5E751C3FD5C3E5D2562A17AD92CB'        
||'3EBD22F61C90A4E8E6DF4E073FE3EEABA09A249D3B28BB893C49ED34D4FA11E87FC26BF67490DFF40B06D2DD0CD246F245B7739327D53568C6CA740F'        
||'AF3F5FF63ECBF796FF0075574A5EA7C197FE38DFCFA32E5C8DBF7CD9CA2DCAAFD4435A7E1FD44CFD9C3F2EAB7953CF23F36B3DCD86AFAB1F78DFC3AC'        
||'B219E87CDBD406D29BBF12407FCDFE6C7DB44D7A318D69D08BFC2064692AA95A468FEE124A7D5A2E01963D21B8FE84DFDA9B1BB6B0BE86F5454C4EAF'        
||'4F5D24123F31D39716C2EED64B6269E2295FDA295E800AEA1AAC6D5CD435B1341534EE5248DC117B7D1D2FF5561CAB0E08F792F67796DB85B25E5A30'        
||'78E415047F80FA11C083907A84EE6DA7B39DADAE174BA1A11FE51EA0F91F3EB963B1D5996ADA7C763E169EAEAA411C48A2F6B9B19243FD9451EA663C'        
||'003DFAF2F2DB6FB57BDBC6091C62A49FF00F527801C49EBD6D6B3DECEB6B6CBA9DCD00FF0029F403893D5957C5BEC6C0F57770AED0DC35D063F11BC7'        
||'63EDEDB54B96AB9121A4A6CFE1EAA497162AE792CB1C755AE6843B10A1DA304804911A730728EEBCF5ECB1E61DA22696E36EDCEEEE1A2504BB412A46'        
||'25D0A32CD1D11C8192A1A95A53A9FBDB4E6CDB3933DCBFDC5BAC8B141B858DBC0B2B10156689D8C6198FC2B255941380C56BC7AB6B2194D8823E9616'        
||'FF006208F789808391D666114E3D14BF983D9388DA9D5B95D9A6AA197736FB861C6526315D4D4D3E1855A5464F2D5318E638B4C7E189980D6EDE9B85'        
||'6B4F7F77AE4DDC37DE7883988215B2DAC991E423B5A6D2563894F066AB6B702BA55734A8AC51EEF73159ED5CB12ECFA81B9BE01152B911EA05E461E4'        
||'2834A93C58E2B43D51349FE724E49FDC7FF7973EFA72BF00FB07F83AE67CBFDAB7FA63FE1E95BB6C0FB5A8E07FC08FF78F18F64BBAFF006CBFE97FCB'        
||'D485C9A4FD0CBFF353FE7D1D2CF116FE2545F9FDE17E2DFD93F8F650FF0009E851744FD33E7CBA11F4A9FAAAFF00B61ED3741FE870D84A820C7D9578'        
||'893F03FD59F65B784D0E7A20B8FF00725EBEBD1D0DABC50ADB8F4AFD3FD6FF000F60FBAF8FAAF963AEB3ACC1186A23EBF427FA7BDC14AF556E8BCEED'        
||'66B4BEB3F46FC9FF001F622B4A63A4CFD157DD8CDAE4BB37D4FF0068FF00ADEC576801E90CA7A0B244A79F5C358B55253C97475A431FDC066E217884'        
||'A0A9D2FA58AB0E40238BDFD9FD957EA569F3FF0001EA31F76846DEDF6E0252429F03867FE24C34FE7D389C76E0A6929852E4B1FBDB1889FF001EFE66'        
||'223394311E7C14337996A61005EDF65583EBCA8FA7B3A9208DBBE23A09F4F3FB7143F98EB08629248894910B9A0CD69402BC0121B3F238F3EA4D54BB'        
||'2B26C63A27DCBB5EA845214C665AAA4CCE3A9DF4DA6A78F231C31CFA41BE94A8A52DF832B10496234B88BB6E74BFF494153F2A8269FB0FE5D3F2C91C'        
||'ADAADD59462A19810319228357E447E7D2566D9F9EF226422A496BB10EDE34A8A7A6FF00718EAA9A844F91A466A68640390B2CA8D7FA8F6FAD1AAA08'        
||'AFDB9FD8727F674C6A58DC33367D38797F17C20FE7D4CA9C66E0A3A492A69B0592CA63112F2D4A51CB5A945120D044F5B405D0588B062F6FAFD47B70'        
||'A329A7C3FE5FB2BFE0EB7F53038A9D26B9A120E3D4FF009FA48B656B6B5E14789648E31286668935C17E7D62C2F7B0E1D6F7FCFE7DD326BFEAAF552C'        
||'573A40E39F4FB3EDE1D29A2DE992C6535362B2796CDCB435930618CACA51B8B0F2CC2E81D6833419A1620907EDEA2C47E4036F7AAF8757CA91E60D3F'        
||'6D3A4D25AA5CB80FA18D70A6A083EA2BC3FC3C734E963549D61050D3C5BCFAA644ABAD9520A3C96DDDCD95D81957798011AC78CAA199C51249FA3C31'        
||'29FA7A471ED281773378D6F2D557243A86047FA605187DB53D3B3451C6861910C7231015D1ABA4FCD583A1F98A00470E9A2BB6AF435055C3F659DEEA'        
||'D97534F34306469B3B84D91BD64C749A7C93C74D3622B30928D487C917921D2C34D8904B7BD86BD8DBF551181C8D25D6BE9921C1F9D0F56D13DC5B6B'        
||'8A50C5090752D29EBA9415207023B720D454758278B6C4D5D3C3B7FB6F212E369AE6924DD9B2F398D9929492D1BD545849F28A808B6A50CC01FA71CF'        
||'B7D198A00C8439E20107F6542F568DA7423C561A3C89D4003F6A972C29FD119F967A5861B664595D5154773F5861166D2A17731DD14B85A8674F2425'        
||'9EAB1529883D85B5424026E78BDA8F732406A91C8D4A9ED0A48F5C6ACFEDE9C92159A12F2F865890BA7BEADE9466455A74A3A6F86E9DCB24B1E5A4EA'        
||'4ECB8B6D471D5D054F5FF6762AB1F16D90BC552F90A43538A31AC82251134C8D7604280035C8772DF76D8BC36BE122925A82486519C70A2B57A1EF25'        
||'DBEE31497296EAAA34C75D2F11F5C61C9C70A9D3F674B1C5FF002EEA1C64D195DA9B5B172C7A55449D8DB365909624A3792B73E56C2DEAB0FF005C00'        
||'09F69179C76A41FA25CFD90CFF00F5ABA1A3596E4E7F500FF7B8BFE82E84BC47C345C68963FE25D7D046BAA430AF6875688894B205F3499A25896E74'        
||'DFF06C6FC7BABF3AC0C068598FFD43DCFF0080458EAABB65D54EAD1FF3961FFA0F3D73AFEABABEB49F1D591E7F6755C34B554D3247B5B7EEC6DC95F4'        
||'D12D4FDC43530536DCC8D4CA7C1226BD4A848D3A9C468B72EDB6F116ECAF17872AEA047EA433460E28413246A320FAF9D057A665B59AD486665C1046'        
||'99236FF8E31FF571E9CBE563543ED9D8B8A96925094BBBB7BE6A0AEAE531D5D126EAC460EB0E19E06512A47108A275560109D65554EAF697959545DC'        
||'F3822AD144842E41F09A61AEBC093535F3E1527A5174E4C221A615DD813C46B0874FE54C797A744B66C30FE1F532C6857C3253CAAC5490B1194D2C8C'        
||'17E80FEEC7C01C5BFC3D8CD64AB0073FEAFF0063A421699E15E99A2DB7451D6A5408129AA5ACBF7D46F2535679A3248D1554E51C358DB83F93F91EEC'        
||'64AA9FF0757D3DD43E7FE4FF0063A3DDD4905566360E29AAAA66ACABA19AB71155357CA6AAB643455A4C324F51505A466F198C0666248B73EC1DB9B2'        
||'4378C14501018538647953E7D18C00B479FB3F6742557E0565A45A36F2BCB9269E80C8C863318C850CB8F0DA8923EB20240FAF1CFB430CF47D5C028A'        
||'FEC21BFC9D56F17F4AA339E8C86CDA2EADDE3F11FA9715DA3BC7AAB65CFB57B1777D7E0EA3B36A692861972792DA186AE0D869EAA92AFF007E1D2FE5'        
||'75D0C140E485B282F737DDAC79E6F2E76AB6B8B913DB42184035502CB30EE1A970714E39AFAF471662DAE797618679A284C33CB4F10915D4911C515B'        
||'8533C3A728BA8FAB6B20F38EF2E93AB86A236969EA8E62B6F289FF007A0961923C2116208B4809B81717BFBD36FDBD21D3FBB2F011C4685F2F5FD6FE'        
||'5D786DB6EC357D6C14FF004CFF00F5AFA7EA2EA8EB68E09205EF2EA4B98E431FDBE6B3034B32050CDA70962793CDAE2DFEC3DA67DF3792DABF75DDFE'        
||'689FF5BBA706DB001FEE6C1FEF6FFF005AFACE3AD3AA7449056F77750C8BA64B1932B944B155216F236179B311F43C0B58703DD7F7D6FB50D1ED9763'        
||'FDA2FF00D6EEBDFBB6DB835E41FEF4FF00F5AFAC386C174F6D6C4771CBB4FB43AAB78EF797A632BB7A7C5ED5CB495BBAE970D90DDD827ABA8929E6C7'        
||'D2BA523903CA4C803164215AE2D59AE77CBFBDDB96FACAE6DE0176AE1A55010B2C53505448DDDE98F23C3AB086D2D2CEF1A3B98A573032D10B16EE92'        
||'3CE5463F3EAB93AEB6DD42ED6C7CD1DC196AF7065E7D2ECAEDFC5B71D4D70286C6D78F43027FAFB96370BA513953FD151F9201FE1E82F6519259BFD5'        
||'C7A10A9F6FC991C92A0A2A8A6114B24534CD3128EF27EA484394915181D2E0006FFD47D4BDAE7C38BE206B43C3F9FA13E9FE7E97E9A9CE3A209D9BB7'        
||'B1B96DCFBC3273D3A54D44FB82A7194424FDD22186AD7094FE1129214698C6A55B0FEDFB196DEE561890FF000863F6D351E8B2E00AB30F5A0FDB4E92'        
||'55DB77ECDE4A78E1568A2A7D0A523B6A320F046AAA07D4026C01E2C7DAC8E60DDC7A61C7051E67A5EF52BE476BF666D9C962A01365A05C852D23482E'        
||'29A4CA612AB122AA116243D38A832C6C2CC1901163C841BA470DED84904E688C549F985756A1F91D343F23D388ED03F8A9C4034FCC11FCAB8E875DDD'        
||'49B3B29BEAB293756F6A5D9B81DB9062B078DAF93099ECD2554186C74781768A3DBD04CC815E332BEB522433B1FA6BB14DBC97B058092D6033C92967'        
||'65D68942EDAC7C6403834F969FB3AD948A5974BBE8550AA0E92DC053F08FCFE75FB7AC55FB73E372422AAA7BF71F13028F233F5B764F8A30BE956B26'        
||'2D89006AB9503824023EBED94BEE64AE95DB5A9FF35E0FFAD9D3A6DAD00CDC0FF9C727F9BA4E3D07C58A75695BE416DF8EDE29151FAE3B1C52C86393'        
||'4D8B3E2083FD7916078FCDFDBFF5BCC9E7B7301FF35E0FFAD9D53E9ED88EDB815FF9A727FD03D38D36C2DB3B96920DE5D31F2EB07B7B6D60F3F83C2E'        
||'771184EB7DCA6A6BB3D2E5E95A8BEDF2591FB16A67916AA1A65718AA845D48EF2811B04C60F7CA5BF9B98AC9AFA13030B6A00591C91E2BE6A8CC0671'        
||'439C134A1EA64F6DA38E3DA6E02B6BACDC68C3F02E3B80AFE5D0E9F2B7B3FB62AB7074A6036E6CCECDEC3D914391C84F968F6C6529EB31EBBAE0C724'        
||'3B1A0DC46AAA6284A643C395A8D6D1BC718A53348D1A7A9A163C7875220C8CF485ECACDF5775BEE9E9ECDD66D8DA783EF5DC94F94CE6F0EC4CDD7C95'        
||'F9B32458FA1A4C8C74F0D7D54D44906284E989A24A480B78A35584092A4C8FA3407AF648A790E929BC7A37E4AE6FB2B2B90DE5B77ACF70F50F6D6EF1'        
||'16EDC7EDFDE8D5397D8BB3931A20A3DBFBE31758F05455C95B47124756308D2D33D44CF4EB2847D67C6B5CF0EBD8031E5D32D72742FC71DC1B9FB37A'        
||'476D64EA37974AE1B72E0F6874C61F70265764D0F68653134D4B8F936CE3F70133D1E62776971BE28AB168E92396A6448E1442C35DA323ADD49C1E8B'        
||'7EC3EFEDDBDBD5194C774E740D1527CD0ECADBF518EDEF86AAA4C76CFAACDEE3C162D9EA775F61D4EE47A3A2A7911DEA6A9CCF1C4C217A4A38C4A593'        
||'DD437A71EB67E7C3A38DB5FAF3B2FE4C7C76CE74B56E5F737C758315B867AEDC35ECCF9DCCEF5DD380CA19B70E3F3196CA54797F83545642D2E45A8D'        
||'87DD82B1D21308F27BB7C43D3AD5687A689FE3A6E2ED9DAD06EEEF3CCEE3EB4EF8EBFDC75F88E9BDDC37A52C5B21666869E2C7E5B1FB631151E0ABA4'        
||'CC5479965C8D5C26ACDE15A648841F6EFEA572707AF569802B5EA23EF5E92DD7BCF6FF005E5475DD67CB2F947D55FC3DB7B6F9D9B1D1516E9977E478'        
||'D9AA323B2309BBAA0C34BB4B69E2A074A3AF97EEE956B679820826A912A45EA8AFA9EBD4C7CBA06F7A77CEECEFCDC9B8B617CA7EAA87E3B61B606E08'        
||'A9B11D5B8DCFE4F21D9FBB3294F4C6AB6FE669B78EDF8CC445451D42263136E50C951577951AA50218FDE89AE1B1D7A94C8CF48EC76EEF8CFDD9FDEE'        
||'ADECDEBADE1D45B27A477EE0F1F83C76EE8329B7BB3B2FB8F6E50C592FBEA1DAB309B70E36148A48569D31AB3555553B3B4C618F81EA83F975BA107E'        
||'DE8A8F73F71E6BE527762F6A7C67C46706DAA7C0E07ABA4C3EF0D951D3ECDDDD53B2EB6A6933188CD61F2CF52B2C3199E259BF8D85AB21BC84C4C015'        
||'9D3DBA8A697976B0D69E2C833C0E17FD59E952CD6715BE9BA22A093407B857811F23F23D2F76FCFF001FB2B4DB836FD5EF7EE9F8F5BC36CD451CD9FD'        
||'81D39D7797ED3DAF1EEEADA048F3F8CA2DE3BA3155F2C58C0F1A4F434A4C8D0195A269E408188FCCB2A2AC085B5293ACA818AF019520E3D29D16FD3C'        
||'F2C9E388A39918510C8FA0903CE8ACB9AE2BF2EA2CD41B1CDFEDBBE7E5BCD10BE863D3F8386E0FFAA0DB62FF00EC3DBA1988CCB2FF00BC8FFA07A704'        
||'3203FEE25B7FCE56FF00AD9D61188DAB20E3BDBE5D27D3F4F56E1540FF006036CF1EF67553FB697FDE47FD03D39E1B8FF8876DFF00391BFEB6759C60'        
||'3693F2DDFF00F30A3FC5D3AD71CB7FF5826D9F7AA37FBFA5FD9FF42F4E77AD2B676BFEF67FEB674F182C375FE0F294798AFF00915F2F2BA0A6AA89DB'        
||'1BB83AE607C054B5437DAC54D5C916DC84F8CBBADBF747A82F3FD5CB65912E1595E4973F0B0AA9AE284051FE11D21DC35359C9AADE08452A5E37EF50'        
||'3355AB9FF01C753F69EF4E9FEBDEC3AA7DD7B83E45F6CF5B6E3DCB84CB6FBD9F96EAFDE15387ECEC56272AF38D93989F6FE1922FB4C5B24F1E1DD164'        
||'7A72F2312C4820AB750C92B8CAC8756962A4E827145F90C62BC3CFA1EF296E8D0EC412055F13BB4104315D4B4F1327E26F30680D3874396CCF905D0D'        
||'97EDDC86FB93A4B7CEE5DA35195ADCDD0F49C3D49DB98BD9898682A8B6D6C2571A6C6412CF4F8CAD3049FB327ADD0AC7E212B2842ECF2DA8B68DDC48'        
||'AA17C4D3DDE5539C54E69E9F9746303DC244F08254BD3B8E8A83E646A34A9CF1E15E9BFAF7BF3A4F0B59BFF2FBC23ED37CFD6EF2A9DC7B5312BD4BDF'        
||'14D80DA59634F518A6CE4B838B113A54D6D3D0D64B4F4A66953C692B6A5762A51A9AE175A815A2E1B02A456A05714C8A9A0CF464C259625F0D082010'        
||'4D2B8A8A006B4191E55E85CDBFDEBD1557B8B31BAAAEB7B1B2FBB6BB1945434AD374AFC806C651498E8E6A2C2EE44C0C783FF81F49055D5C54B2BCC2'        
||'3F580E926951EDA96E3C521635602B52069A91E95AF0278F5A8105BC681815A572748E392327CFCFCE83CBA2D32768FC4EABDD3BC6A693E5BFC9FA5C'        
||'A4D90AFDBF53B3B19B3DEBA830196C72474B918B69E02A70734ED2A359CD9A6B33DC2AFA4014D8DCC891B4F128955B0358D5A480090A31C0115E3C7A'        
||'00BF29F30EFD23CB61690031392C914CAAEDA78D535EA2ADC6A2809E07A9549BD7E24D461F194127CE0F96F0536172095196CCD16CA92A727E4D4B30'        
||'A1DE3908F6EBFD9C560ABE29120B87B7E7DBCB06E250491C5E20522A464123BA8D438047118C7A756927BFBA27971B69B1B2B8D20032D6098FCD5A59'        
||'00663E6457E5D2D20DF7F0DAA331515137CE7F95D46D9880418CA23B0C52E11FD2FE23B58B6DD1155CE46AFD324AF75E471EEA64DC6E67FD3816A400'        
||'156801AD4F68AF71FDB8EB7FBB77FE45B32375D9ADEE11B3E3DC6B9C0F2CC88FA57D734FD9D428F787C3A82822C4CBF3ABE608FE13918EA32B5953B1'        
||'AAA4DD2AB2CC1920DC108DACB2D352DC588F146A14905B4FBABCBB9476EB6CF6E148209254073934D449A853EB40283D3ADEDFCB1CC73CA79A6C6C2D'        
||'6FA003578514C258739FECA37AD457E1E22831D3DAEEDF86106425C8C7F3CFE54D541B9B1CB4D8569F63ACDB614B2B48ADB6EA64DB8B4D2D758DF419'        
||'A47E07A6DEDD89B75B6ACCF6AACB201A4B0AA0E34D2755093F9D69E9D207B1DD39DEFBE9F6AB2B4B5960235410CA2DE434A9EE8DDF5907CC8F4E3D26'        
||'C6EBF8830D3513AFCE8F9752C7B6AAD24CAAC9B09CE4A66B18D3FBDD4BFDDEF2C74A7962E5238CDAFAB48F7AB74DD182CC96E24119EE14A8F3F8C035'        
||'03CFC87E5D2BDE1B722EBCA979B4D9EDF72E34AC84985DE9E6B33BE86248AD56B53D181E9AC1749E5CD4F696C2F96FBFBB3A6C76E714541D7BD8B95D'        
||'9FB6AB6B66C8ED8A9B54ED5D9551478FCAD7D153D3B4D50D598F8E7895D584CC006B28B2967B9DD96D25882F8C188D2A7400A2B404554701826A7247'        
||'50F7DE36C7983947D95BEB3DCF6B8A342F687EAEA6494837515144C18A6580AAD2B4F2A57A37DB3B72A49B930372181CD6301551A8806B116CA003F8'        
||'B86E3817F6AF98B6529B45C9A644527CBF09FF0063EDEB9E9CA1CC5E273158863869E2E1E9AD47A1F9838C67A5447B96689DD3CACC51E48C904281E3'        
||'76049BDEDF9B7FB7E7D9036C91BA870B4A807F681D0B63E689A32C85ABA491C69C0B7DBFEACE7AE71EEA9249586BB5D75F0C1B428E40B37D7E9C1FF8'        
||'9F757D85523AD2B9A7DBFEAFF563A762E6D7967214F9570787CB3FEAFCC8EB155EEA9D418BC8059AE1989012EA09D4CDF53C71C7F5BF3EDCB6E5F8DB'        
||'BA9FEAFCB87FAA9D3377CDD3A0281871FD9F6D78FCB14E35CD3AAA7FE64BDE0D1E076A74CD26660A21B86A23DE7BD6B6AEADA8E8E8F0188691301495'        
||'D3C6AEC2392A525AB75084DA18CDB900CB1C8DB25BED514DBE4C1534831A92748540354AF5FB00514F9D3247425E408EEB7FDCE6DF6E524B8B7DBD7F'        
||'4D523F11E4BA9005508BA94168A27627510A0CA092341EB5A8EEDDECBB93315345878960C5C54D4D4C0F9ABE466A0A690D4504AEF906325EBE4635D2'        
||'A78E31E25A4464043FB8CB9F79986EF7C61B527C20028D41B5680750A963ABF509D64694ED1102B507ACCBE4FD8EEF66DA826E6E25B9999A476F0A28'        
||'8A86358E12B10A5214A2D599CEA2E751047405E2A9EA2777F1DC92E43955B2162353A8E3F51BF1CFB8E941271D0B0D0E3A1B3606C59B3DB836F6DDC2'        
||'6365CBEE1DC394A1C1E0F174F66A8C86632D5694745471A30B03249222B3B5955496621549F671B75909E74880D4CC6807F33FB064FA0CF48EEEF2DA'        
||'C2DA5BDBD611C30A33BB1E0A8835313F6007F3C75BAAFC5BEB7A5F8D7D17B1FA7E8E682B2BF034D5792DD390A6D2D4D91DE19DA8FE25B8EA29AC0168'        
||'62988A3A5661CC11231E4FB3BDCB695BDB812FE1450883D1457F9124B1F3A9EB0977DF719B7DDEEE774452892B7629352B1200B1861FC7A40D4386A2'        
||'7A1BAA736CEACCCAA5ACFF00A883C15B806E2D7B9FC1FF008AFB6A1DAC2B051C31C3E5D062EB7E2E359A16A1E27D47D94AFF0097CBA363B3E44AAD83'        
||'4EC39F3E3D1F92350F24652C01FC0FF5BDC29BDA9837F957F86423F61EB29F95E45B9E4F8245E0F0A9CF1CAFF93ECE8A46D7DD53F8EA1A1917C4D5D5'        
||'F1C6ACE42E84AB74B8FADBE9F9BFF4BDFDCDDBDEC51168FC45EE0884D3D4A83FEAFDBD62C72EF33CF1B4CB13F6F8B281527807618E3FEAC57A1270DB'        
||'96B25C840B3C91AD3B108F7620900D96DF527903D92596CD6D15DA3C60EA0491F6D0FF009FA95395398AF6E3986D56523412E0E7D2373F69C81FEC74'        
||'32E1EB71193A61119584DA9AC2E382A6CA416E6C4FD3D9D5C453C2F5A63AC80B59E1990F7770F2EB056606AAA35082AC809AB5EB0CE09FE9A8F1F5FF'        
||'007DF9F7786EA34A6B5E3D35756B2C80E9623FC03A7CC2E4B72EDB63250D5A473BE884311E4578C9B90AA080B7B7207B417B6767B8624069C7F3F9F4'        
||'75B5EEF79B51AA50F01E74A7A81FE1E94359DA5D978B45318FBCA69E4D293451B3FA826A0DA5589503FC7F3FEDBD940E5BDB246343423C8F4278F9CE'        
||'F11034A873C08EB35077676AB4890C345302C080248CC7AADC3017205FFAFF005F6CC9CABB6E9D4EC07E75E952F3A3310A8858FD94FE47A9D5DD93DD'        
||'B5A100A58A25E49D5345A80B1B0D1A89FC71C7BA47CBFB0A717AFE47AF3F35EE80552022BFE97A659FB43BC28699DE3824758D847E2113F9189E4344'        
||'492AC0FF0050DFEF5EDF5E5CD8647A6B1EBFF17E9D51B9C37244AB427D3D73F2E92357DC9DE35B14'                                                
;                                                                                                                                   
BRANDINGBackgroundIMAGE_5 raw (32676) :=                                                                                            
'9A317937446314AF1C5E9D64E95504B7F5E0F1FEC6DED4A72BECD1B005D7E59E9A3CE974E9889B8FF09E813CD761F62662AA6A79A1C94D511C8C9344'          
||'8929F1B81670C149008FCFE3D9EC1B159C080AD00F2E8A6E39B0CAC6B5F98A647FABFD47A415541BEABE403F86E4CBCEFA13F66462EE39D3CDF93F81'        
||'ECCA3B6B48C54B28A7CC745536F52CADA5351FC8F9FF00ABF3E91B578BDC62A65A7969AA85446C54C4EAE1838E0A053CDFD9A46902A8714A1F3E88A4'        
||'BF9A5629A8820F0C8A7DBD73A7A1AC5A77D74957F72A58C8244FDA082DA1947D7FAFE3DA94F0852B43D135CCD745CAA3507C8647A927FC1D72C7D3CD'        
||'56AC26A4AC5362A9511E868E21C956313AF3FD7EBCFB71A589083A14D3CB8749BE9EE65470972CA585385727CE9FE1CF1E8AEFC8E4A1C066E8A9EB72'        
||'D41E7ACD966AA312544304922495B5B0C4E639487B92BA6DC8E2C3F3ECD6CA4379A6482360AAFA785694D24E7F3EA4CE43B5936DE5E9EDEF25579249'        
||'246AD685AB146A307CEAB4EAAD5E589546A923040E6EEA2C40B107DCA01871AF411FA1BD200113FF00BC9FF374613AB73D83A3DBD0C55799C5D2CA32'        
||'15CE63A8AFA58640ACCBA1B448E0D8FE0DBD83F7D82696F4B46858695C804FF83A92F9554DBED223B8EC6D6F86C1CD3C8F467B079FC055C2F0D2E6F1'        
||'3512988B08A0C852CB2155176211189E3F3C7B07DC5B5C21D4F1B01EA54F42B49E163A55C13E808AFF0087A0F7B5F68ECDECED89BC3AEF7834159B63'        
||'79EDFC8603330C7550C750B4D591DA3AAA399AE127A79024F03906D2229208B8F69EF76A8B77DBE6DAEF10B453A946A0C8AF023E6A6854FA81D59DD0'        
||'7123F6F5AE1F747F27DF985D77B964A7EB4DA10F7DEC3AEB54EDFDE5B0B2B838EB1E8A550F0D3EE4DB397AB82AA8AAD01D3288C4D0122E931BD86286'        
||'EFEDBF326DF78D05A426E6304E964A024571A9490CA7D788F427A52AEC0770AFCC7FA8747BBF974FF294ECBD87D8FB6BBFBE4DE3A836ECFB1EB5737B'        
||'0FA960ABA2CF66EA375D292711B8778D5E2DE7A3A586824B5553514534B2C93AC665F12A14715F24FB7D77697F1EEFBF011F827524550CC587067A54'        
||'00A7B80A924D2B4033EEE66A9C01E5FE53FE6FCFD3ADCA3A0F774DB5F0385A5AB8659F1B5F89C6FDE44A3FCA29AA628B4A55448F6B90095742413FD6'        
||'E399439AB97177FB38A58182CF103A49F8594E4A92386720E687CB3D63C2F34FF5739CF754B852F6D35C3EB0BF12B06203A83838C32E2A2943519370'        
||'372E11A015095CAC8C2EABE2984A7FDA7C4CA0DFFD7F7137F5637DF17C136E41F5D4BA7EDD55A53A90BFADFCB862F196E4114AD02BEAFB34E9E3F6FE'        
||'DE838CF7DDEEAC9C12449E3A4A2474A5858FA897379A590FD353900003E80017F723EC1B6C5B059346EDAA59482EC3863828F3A0CE7CC927A8A79977'        
||'89798AF96541A2084108A78E4E59A98AB500A7900075DECEDB9BEF71D5E7B0943B37211D14350D043B872C9FC3B012AC8963E1AAAB00CB6B692B4F1C'        
||'841FC7B34BFBCDBED5239DE752C454A2F73E3E4387FB623A21B6826999D0A11426848D2BFB48CFFB507A6CA7FE5CDD5591CCE4779F6CE7B219AA2A96'        
||'82B2AF03415C36DEDAA49E243F706BF3B3115722311EA589A9AE05B573ED9B9F733774B75B1DA5044460391ADC8F2A2FC23F3D5D3F6FCABB748E6E2F'        
||'2AE0D0951DAA3ED6E26BF2D38E86CA3ECCF8E3F1FF001276EF516D0C2318E3F1CB47B171B498BC6CEE82C4E5373D4AB4F546F724869FFE0D7F605BE9'        
||'F76DDA5FA8DD67676E23C462CC3EC5AD17F97D9D0A2DA1B4B18BC0B54014F10802823E678B7E75E80BDEDF29BB3775A4D4989AC836662A6D519A1DB4'        
||'248EB9E23FD9A9CCCD7A8627F3E2F18FF0F6992DEDE335D3AC8E05B3FB070FE47A7FC597821A0A528A29FCF8F4493B13BBF60EC3FBAACDE9BB699727'        
||'26B964A1150D96CFD5C846ABB53465E5B926DAE7651FE3EC5FB272A7306FD22FD040ECBFC4C34A01FE98E0FD82BD11EE9BFECDB2445EFE74527F0D41'        
||'7FD833F99A74407B2BE7FD5B0A9A1EBDDBB063E352D08CCE7E58EBB2054F0B353E2E0FD8436E4791E4FF005BDCDBCBFEC843A96E37D98C84E74460AA'        
||'FD858F71F9D00EA18E63F7BADADC98B678B530342EE47E44018FDA4F458719DB9BEB7C6E0A2CEEE1DCD98C9E4292AE2AAA296A6B18C3453453092292'        
||'9A9A3D314654807D0A3DC8D372C6CFB2DAB5A584091A3021805C90466A724FE67A066D3CEBBC7305F8B8BA99B5D4533DA0038A0E1FCBADB4FE356FED'        
||'C19CE8ADB5D45DC93576CF83B83AD69727B23704150AB3D3E3770A30A5A212391E36F27EFD3412B2868A46A57651221F7861CD1616D6DCC125EED444'        
||'AD6731561FC4578FDB8C3100E4061520F595FB25E48FB6ADBDF028B7280A9C62A707E54391C304A9E23AD6CFE6EFC77ED0E90ED2CF6D8ECDA77A9ADC'        
||'A54D56476EEEE856A3F80EF7C309B45365F03573F168C5A3A8A227CB4B203148A2CACD95BEDE7336D3BC6D515CED3D8B180AF1635C4F4CAB81EBC55B'        
||'838C8F3031D7DCDD83718EF64178031932920C248060153F2E0CBC54E0FAF55C95BB3AB9EA99152256FA962C141BFE7FD7FF0079F731C3BB40B1D493'        
||'4FB3AC6ADCF942F6E2E74A0029C4D7D7A389F0CFA2778E77B7BAFF007A60B6BE7F3789D8FBA6833FB9B398DC65554E1B078EC73992A6AB2B91443140'        
||'A80FF6E4049E00278F71CFB93CD367172EDEEDD2CA88D710B222310198B700A2B53FB3A9BBD90F6FA7B6E78D9B7528CE6DEEA37671F08D26B9C63F6F'        
||'5B0243273FEC6FEF10C8F23D74BDB3D3B413FD02DCB7E00B93FEDBDB4CBD57EDE9F9D6A5E42CB14966546BB008A4B2066E5EC3EBED80540C9EBD4EBF'        
||'FFD4F9FF00FBF75EE8FD7F2EBEF8EB5F8F5DCDBBF78F69662B30B81CBF5466B6A5054D161F259B964CD56EF2C0E629E06A5C5C72488A60A1A9732101'        
||'4150A4DD8023DF6EB7FDB79777C92FB7562B1342C950A58D4BC647004F0539E1D08F96372B5DAF707B9BB24298CA8A0AE4B29F2AF903D5CA52FF0034'        
||'3F895413AD4E3BB3779636A903AA5662F68EFAC65622B8D2EB1D6504514CA187A5C07B11F507F335B7BA9C96E6AF2B9FF9B2C7D7D47A9FCBCB893D0E'        
||'FF00ADDB111DEE4F03F031FE4569FB38D3AF1FE689F135E99A8A4ECFDE8F40F50D56F8D9369EFC7C64958FC3573E39A23034E7EAD398FC8DC5DB8F7E'        
||'FF005D6E4DE3E2BD462BE13D695F502B4F3A5695CD3AD8E6FD8C67C43FF38DB153C07A01FEACF5226FE69FF162AA0A2A5ACEDCEC3ADA4C66838BA2AE'        
||'C1763D6D0624C76319C450D5ABC34856DE834E8857FB247E1D3EEDF2898C45E3C815492A044C295AD721456B5CD78F0EAE79CF662823323514D47636'        
||'3C8F966BE7FE0EB8D5FF0034DF8AD90A614791EDAEC2C9D109E5AA14395DBFD8994A15ABA8FF008115D1D164164884EFFDB9826B63C963F5F75FF5D9'        
||'E50D2ABE33F670FD26047C81A569FEAF9F5A3CE3B29A0F11B1C3B1B190718F2E1F214A529D3D637F9B5FC66C7E6B6BE6E7EDCDFD9C93676E0C2EE5C1'        
||'6377160BB2F3588A6C8E07211E428963A0C87952242D12A3F8423682CA0816B52E3DD4E4BB880DBC92C9A590A6222182B6AD401D35A1A9E24D09273D'        
||'55F9BB62923F05DCD082BF035749AD40C7CC9F3CE4D6B40B8EEFFE753F1EBE40765EEFED3DF3D99B8E9331BCF334D9EAFDBF81DA5D9306D0C6E4E931'        
||'54F868AAB0784AD33253C861A642D2A812172CC1C5FDA7B1F72B9176CB0876DB292558ADD42A563667A0C773854D44FAD00F2A5074CC3CD3B0DB5BC7'        
||'696CECB1C628B5562C07F48E91527CFCB1C3A0B25FE697F152A16B56A7B637F54AE4DA37CB254EDBEC2A94CC4912858A4CCAD42B8AC2A0008D53AD97'        
||'FB36FA7B50BEEA725AD34C8E349C7E8B507FA51A68B5F3A50E78F4E7F5B763A8A391A785108A70E145FB7383E87ACD3FF354F8BD542896AFB87B1EB4'        
||'62F4FF0007FBDC2F65560C2E90349C2AD5EBFB322C029A6D056C2D6B0F6EC9EED7284AAAAF3C8428A50C4D4A71238641AD0D6A7D6A3A78F39ECCD4D5'        
||'2B300294287E7E74CE7D6BE99AF58A9BF9A67C55A1AD6C963BB677F63328DE5D796C66DBEC2C7659FCEC1EA3C995A148EA5BC8C019754843917604F3'        
||'EF49EECF2823AC89349A932A4C4C6840C100A9038D3C8FCFD5B1CE1B2820890D479E8627852B90780269E9D619FF009A17C49AB64359D97BC2B4A4B3'        
||'4EAD5DB3F7C57949EA5C49553AB56C2E75CCC034CE2C5D802C5AC2DA6F767941904467974AFC23C37A2834AD05315238799C9CF1B3F38ECCC0069598'        
||'2F01A5A993520628056A7863AD7677362F33DE5F21771E1FA9B059DDF79EEDDEE3CCD0F5BEDAC0E22BAB7726EECA6FBDE92C5B4B0D88C0C0AD512D6D'        
||'749550430D2AA191A4708016E3DE31EE93C775BA5CDCC26A924B2329A1150CE483439E0781CF513DE48B35DCB327067623EC2C48E8E17CB4FE553F3D'        
||'FF0097E75FED1EF2F901D6D8AD8FB372BD953753C3BBF62771F54F644BB1FBA71B859F74547596F093AA73B949F07B8A0A1A4AAAA34755E365104B67'        
||'D71B2841D26EABAB199ACC61725166B0D96CA627314E677832D8CAFAAA0C942F550BC150D15752B2CAA648E47472AE352B303C123DFBAF74DD23BC8E'        
||'F23B3BC9231777918BBBB31BB33335C924F26E6FFD7DFBAF75C3FDF7FC40F7EEBDD3E4DB9B725465E0DC13EE0CDCFB8299A95E9B39365ABE5CC53B50'        
||'A2C742D4F929243321855544455FD0000B6B0F7EEBDD754BB9B725065AAF3D43B8339459CAF6AC7AECCD2E56BE9F2B58F9025F22D57908641348672C'        
||'C662EE75924BDEE7DFBAF74C9C7FC8FF00DB7BF75EE8C57FB2BBDAC7E3AEDAF93E91ECD9BAD7787711E8CDB18B837F6D1A8ECECC6FD185973C7ED7A9'        
||'E0AB6DC1FC2BC30490FF001B6C7AD01A9D34BF71F72F1C4DEEBDD35FC94F8D1DE1F103B93757C7EF91DB0EABAC3B8F6443809F766C5C86636E66F258'        
||'14DD3B7693776DF5AFABDAB595F48AF538CAEA3AD8E31505845346CCABA87BF75EEA474AFC8BDE7D2BDA87B686DFEBDEDFC9D76DFDC3B4F73ED9EFBD'        
||'9D41DB1B3B78ED9DD3B79F6C65F139FC7E7DBEE91FED1C2D1E471D5B4B90A378E3968EAE0740DEFDD7BAB35DA7F0DFF99B7F3A25DB7DDBD59D17D694'        
||'3D5DD6586DAFF12BA4B178EDD7D59F1DBA8718FB33195DBCF6EFC7AE8FA4ED8CFD154E7F2EB155E5B3B5891D6642ADA59AAAB325582497537BAF754F'        
||'7BC307BD761E6F727576F4A5CDEDDCCEC5DDF9FC26E7D979596785B6EEF5C1D5B6DDDC54B5B8CD6628EB609A95A96760357ED842C428F7EEBDD243EB'        
||'FEDAFF008FF5BDFBAF74E1FC5728716309FC4ABCE196BDB2AB89FBCA8FE16B93929D6924C88A0D5E2FB83122C666D1ACA00A4D85BDFBAF74DFFEF89F'        
||'F78F7EEBDD1A0F8BFF0017FE4D7C92C876FE63E336D2C8E7EB3E37F47EFCF91FDB7B8A8B7C6D1EBB8BAF3A6360410D3EF8DE551B8F7965711149F6F0'        
||'D62838EA0A896BAA22331829A68E29CA7BAF745BB1F95C9E2659AA31592AFC64F514953413CD8FABA8A3966A1AD88C159452C94CCA5A295095922274'        
||'B8243023DFBAF7502FFEFBFAFBF75EEBD7FF008A1FA0F7EEBDD38AE632A98B9304994C8A6127AD4C94D875ADA95C54B908E2F0475F2E3C3789A6543A'        
||'04A50B05E2F6E3DFBAF75C24CAE4E7C7D26267C8D7CD8AA09AA6A287192D5D4498FA39EB34FDE4F4946CC638DE5D09E464505B4AEA26C3DFBAF7503F'        
||'DF7D3DFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD6C3FF00F09A192A07CEBEDBA78279A18EB7E246FCA5AC5898'        
||'A8A8A4FF004C7B0AA1E0980FD49AA3462BF92A01B8B8238E415D7BC4A2B4FD16FB7E38F81F23E55F215A668451FC30564740E50D56B901A8402479D0'        
||'138F5E38C1DE2562F1B5DC06258B3437234DF850E57F3F4F4837B7D48F621E6E9B73D9376B1E64DA2137212392DE6B74FED24849128789403530B296'        
||'6C703E75EA51E418365E65E5FDCB933982E458B492C3796D77253C18AE14342D14CC48D22E11C2A67E25AF97593C524CC0B7F4B2A802CABFD1547007'        
||'F80F675CB9BCEC5BA42D75B75DA5C4D2D0CA6BA6407C90C4D478D5321548F56356662435CDFCBBCCDB0DC258EEF632D9DBC5510E35C4C0D2B22CC958'        
||'A57930CEEAD9C28A2AAA898B8F7201D26E485502C4B31E00E7FDB9FF000F671B85EB595BF891AEA918848D786A76C28CD3032CDE88AC7CBA0DED5631'        
||'EE177E14B27870A2B4933F1D112658E01EE3844F591901E3D3C52E218C357110C4BD3F907079785C487FDE2FEFD6D1FD25BA44096D3F1379B3354B39'        
||'F9B3127F3A7974C6E1782FAF1EE48081880A808A220015107C914051EB4AF99E9DB07053D1D255B544F040A674F54F2C712FF9AFEAE47B2CDE374DBE'        
||'CCAC97B3C71281C5DD57CFE64746DB1ECDBC6EAC60DB2D26B97270228A490F0FE829EA15555D1C95537DA992B0FDBC247D9C4F32EAF249A94BA80BF9'        
||'1C96B7B03C7CE1B47EF99FE83C4BE26184016D1B4B565796A3500107C4325C0F2AD6BD4A6DEDDF31A72DDAFEF810ED4A2E6E096BE9E3B7A2B456F421'        
||'0932B1251BB5632D4152284559275A9726D02C0B7BFEF32BB9FEA0245C7FB76F666F7DCE9B9131EDD6716DE98FD5BA712C94F32B0424AD7FD3C94F51'        
||'D17C3B6FB6FB3A89777DC67DDE4A13E058C4D0455AE03DD5CA872A7CFC382BE87A80F4350CA5CAB95FA99652B0C43F1E9D761FEF67DD20E47B3B99C5'        
||'DF345D4BBB4A321653A6DD4FF46DD3B31FD2D5D29B8F73AFACADDAC3926CA0D8A06142F6E0BDDB8FF865DC95933E7A02750DA28626264A8F237E5204'        
||'2E091F5064974AD8FF0085FD8AF70DA6CB73DADB6891344240D3A28863619478F4FC2F1B00CA45323D3A04ED5BF6E7B46F29BEC0E649D492DE292E25'        
||'5604491CB524BA4AA595C13904F9D0F5E022349AA2A7497EDA4F32F98B4815656D328D2856DA4D8D8DC7E7D95EC7BADE4B24BB26F642DFDAA8D44616'        
||'E22388EE631E8F42245E31C9553823A39E63D96C60F0398B97896DB2F98855396B59C77496729F33183589F84B110C321A8DF355D411E8758473E981'        
||'1221FEB7A003FEF3ECF82AF9E7A0ED287A66A8692404BBBBDFEBADD98DFF00D76F6F2D070EAC78F4D8CA08B585AC47FB1B7E3DB80F5E3D37DCAB0652'        
||'559482ACA486523E8548FA11EF72471CB1B433287470432B004303C4107041F307A76292585D26858A3A10559490CA470208C8238820D7A66DC90643'        
||'2E71F253E42A68A5A7958D5C70880D3655458C52D746D1B49E6879F1C90C91DC3BABEB04821FB4B0B5D803DBC4CCF6D254C50B50AC0694758D8F7088'        
||'92AC919AF8654E821485529E7FBEBAE654B695E28D6F143092E412AF2F74655DD076B4D40C8F20D3E22B02E0B02CD9E2C781110D77247361C31FCFD6'        
||'F6F681E40CD51C3A0FDB5A3C710491A8C335F3FF000E6BD273298932A3155F4321524F054DEE4953F5E3DB5AF15E8FA085CC812B50BE5C3FC1FE5E8B'        
||'06EFDA063C8CACA85848CCC4920917047A557FDE2FF4FE9EFC0FAF4751396F86B83D2C36C99CD3E3A96AEA669A3C6858E9A39293134D051532DBC749'        
||'4A68618A5652D79247A992472FCEAB962C602E2D3E9D6248F4B035278D4E7FC9D06370D9F785BE7BA69068228002D5D341C7F0E0D723D7F6D1CFF314'        
||'DA0F45F2265DCBE4493FBCBB176AD00A648D7EE29A2DB15D9245798899AFE46AB7D2A69E323493AA50C3C68F705FF18D7FC417F90EADB71A584719E0'        
||'357EDD46BFEAA9E8B274FD1AC1BE2959413AB6D762FEA5B5D8758E6B5707E9F8B7B4D10A38FCFF00C07A58787FABD7A367D6C40EADEEA161EBDA9D5E'        
||'A07E6C3B8712E78FF61EDE6AEB4FCFFC1D54F15FB4FF0083A2FF00DF34C64EB8E977017536E3EF070C6C47A6A76A4405FF00C7FE29EE938AE9FCFF00'        
||'C9D6FD7F2FF2F408EC5C44194DD3B3E96AAA128E33BB76D5489DE332A2CD8ECE53E4608B4ABC7CCB242B106D7E92C0D980D2CE59286BB841C77A7FC7'        
||'81E93DC31585C815ED6FF8E91D5A8EF1DD788CD759EC4D9D49B603E731192CE54D76511286196BDB293F8E9A896AA9E14A80B1692584B50EACC410AB'        
||'A41F636755899A5762FAC0016A68B4FCE99F901D46D7B2594F2986186922B1ABFDBF2196F2A6A2748141C70EDD3583A99AAA75929A6A24859A092333'        
||'BC8D11F2686322FD793F520FFAFCFB2D796C893AE323CB07A516B61BD8557B3956872032D41A7CFD40AF1E8E3C3B7A8969E3F22C9EA5BDD1883A00BE'        
||'A20FE05C023DA097E88F00C3F3E847672F3645540D11A710471F90351434C8FCFCBA98D8DA08A30B4CDE35F0B32950097D46FF00A8DB807923EBFF00'        
||'12D28B50C3B8D3A5D2CDCCF35B485A18F5574F9E0D3071EB815E1D24FB66926A9EB4DCF3D4449513613AF370471D5250E3E927AB968F0134CF51592E'        
||'3E2896669640D228118F1826353A0280B77FB2DB7FABF713DBC9A9FC290D335A686E35C71F4C7419E5EE60DDA3E68B5B0685A3479A2570CA28B57518'        
||'35FB739A8A798A9A93DB33BD6CD5C24511F8FC4DE9BF25A46041D5EF189B07ACB58324D7A3ABB5B050CF1D51124E4ABC160814F254917E3DA673D1F4'        
||'31820D3A10A8706D1BA232D4A97992C1A3B3106C2E05BDD0B573D2B48A9EBD71DC38BDA0516977053C15B5118BA472D309AB200FC8F5A00C80FD402C'        
||'3FD6F722F25F287B83BD446F795D1E1809A78AF208A26238D3557591C095534E15F2E8837DBCE5AB7221DDC2C920FC3A753807D48CAFD848AFA75130'        
||'18EDAD4F2B47B7E9A1A491D4EA8929C43532A8176BBB82CE3F360C7FD6F7EE74E50F707658C5E73423CD00200955C4B1293815D34D04F005956BC01E'        
||'B5B1DE72E4AC62DA02C6E78AE9D2E47E796A7C89FB3A44EFAC1C55D9F851E2A992A25A3A4A78A08958C9333C8CB1471C2AA599989B28517278F7911E'        
||'C15F4D6FC88CB1D0017531CFFA58FA0A7396C1B7EE3BA7D55E1652235068C14051A8D4D47ED3D183A5D9DF3BB0DB2A929BABB775762151A258303BD3'        
||'77522C9062BEDDC98E8A1AD59A4A693578C246D2A69171A57D8BDF91BD9CDF7787BFE6ED9E0766A932430956692A32E10AAB0A5492549269527A8EF9'        
||'87DE9E7DE54D9936DF6CF7A966951C294B863240B0E96AF84EE010C1B405D2DA74EAA755B1DB5D87F23B606F1ADC576F610E3777552FDE4951B8692A'        
||'321265E9B598D2BF1F991532C55705C6912432B2A9F49D278F73D72FF23F21CBB6471F2DF6DAC7DAA9115454F91408349F335009E39E3D6386F5F789'        
||'F772DEFD8EF896E6E1F25A489DCB8F50E653A87D8683863A15E0C150CF0413B99F5CD04333E99401AE68C48FA45BE97278F60A7DC6E124645A514903'        
||'1E40D3D7ACA6B6E57DB6E2DE3B890BEA91158D1852ACA18D31C2A71D0B1B0F64E16BB1B5B2CED5BA92B420F1D4228B1815AE4693EC9372DC2E1E55AD'        
||'387A7CFEDE8DACEC61DA50C1695A31D4751A9AD29E83A59546C9C3636096BE99AB4D452219A1F24EAD1961E91AD748B8E7FAFB4715DCB248B1B5284D'        
||'3A76E667303F0E1D32799FFC3FDB1F6B740E88BC56E9C6877FE7B06EB0D12D014A7B247E7A6690D873EA21C5FEBEDC3B7DBCE819EB9F43FEC744772E'        
||'DF50E7E7D0C9B37BFF007E4D34F46D1E03C50D30912D8C943EAF204E5BCDFF0011EC9770D8AC634122EAA934E3F2FB3A51628B71214938015C7DBD2D'        
||'5FB7B7657CB0C33A6202CB3451B78E85D4E991C2B58990F363ECB46DB6F182CB5C7CFF00D8E8D4EDF6E7D7F6FF00B1D0C990DA188C9EA1526B006B83'        
||'E39953EBFF00209F6531EE17117C34FD9D6CED16A7896FDBFEC75553F3377CE6FA77B2B0DB5368AD0BE2F25B331FB82A0E629DABAA857D565EB68651'        
||'14F1BC4047A29E3B2E9FADCDF9F7387B75B359F31ECD25FEE3A84893346341D2348446151439AB1EB1BFDDFE78DEF9279921DAB651118A4B7494F888'        
||'5DB5349229A1D4314418A71AF401F50EFF00DE5DADD8FB6F60D77F7760A4DC12E45679DE07A048531D88A8CB999AADE460814D3DCFA4DED61C9F633D'        
||'CB96B6BDAEC9EFA1F10B474A55ABF1305E14F9F507F30FB93CC7CCDB44DB26E5F4E904DA7510854F63AC828DA8D0964000A66B4F3E8D655E170DB719'        
||'A39F3B949A481D75454A639E1D02FA2402A5835C5AE556453FEBFB0E968FE0F3F5EA215463A644355238356B8F97A7E7F6F589AA7115B2948F2C33F4'        
||'F5040779E882656903AFAF4CD1389B5211F4921914DAC3FAFBF0208A1E14EBC03BB12CC6A3F0E0FD86BC40A701E98EB3C780DE5B5A18772F5F6EE8D8'        
||'BBF8E53439BA9DB79D8B4D9C2CB14EF4CB3800F2118DC7F60D9BDD258A19478522F69E218547E5C7F9FEDEB4CF3DBB2BA0D6C090ACA32071EE269EB8'        
||'C907D2BD3149D9DD8472BE4CFB19AA25722A2A6BB1D150E42B236E1C26771AB4B552FF00AC6A1BFD63F4F745863853447C0600AD547E46A07ECEA875'        
||'BCDAA40433649C8353C7C854FAE69D2C1770EDFAE92082AA92782AE68E36398D74F503192CC7C5E096AE48BEE82FE4790B5BFD5DAFEDD0C48D2787A7'        
||'99FB0607E63A71688DD82A28C158D0006A3E2393F603F91A74F751D75B96A76BC9BD36965A8737498BCA50D2D7D34D1C3459BC357D757251E25EAA9D'        
||'4DE48E6A864892565684860CE3C6C7DA6792169440E695A915AD0D32687E4334E3E42A7AAC931806AD26B4D2CCA2A338151FD2268ADC08C9A0E8BB54'        
||'E2DB3B31A7CC6125C83E3EADEA6310CF23D650C93CBF733474F574AC268630795804461E7488C0B7B5BE2CEACCB8CE0F97FA8FCF8FCFA612CAD82C72'        
||'2EA254874F322B91F97F40F6F953A16325DA388CB62F0D83DF9B45F3949828568F1B97094187DCB498A0A63FB18323414502346BC32255C732861AB4'        
||'EA37F69444C9ABC23A41A5412585479D09E34C54107D4F4ECAF2A2914AB668591548041A0AAA03A6A7831614C815E830A9DB5B3ABE599B03BAFEDA3A'        
||'A064869B70626BA9DE00490B119A87EEA95DFF00A1F2229FC589B054930AEA0BA194F1047FB07FC3D32D04DE0AAD0C8196B4F43E7414C8AD789141D3'        
||'FA6D8DC147451C8F90C5EE0C7C4D184141570C6F0C68BE9F353D1EB9907D46A74B0FA03EEA74BB9607B8D6BC6BF3E200FD9D3B0B345100E815569400'        
||'500A0A0A658FCB3C3A45E4FA1F71F6AD4838ACAE2F0D16DD0EC127355919EA5726DA20445A42B2058DA3B5E50A0DD42F37F696E6E56028589A9AFA8F'        
||'4FB3FCBD0E793F4A3DC9600860878D4F16E35FB7A57ED4F8AFB8B00C16BF25495EE23772F0627266EF1F26237D45413FA493C93FEBDAC9BC44AB4AE7'        
||'ED1FB727A17BD19B853FD5F2E876C674B64D1A0FB79E1A78E38416738EA963184D20F8DDC0D44DC8E0D85BD447D7DA497768883AB353EA3AB2902941'        
||'8E96D84D8598C6D4D3D3C71CF5B3D7D75252D398A9C44CF2D4D57DA411A79993D4C5CA006C0EAB026F7F6925BF80A99188014126A78002A7D7EDE9CA'        
||'F8BFA6A0D49A7EDC743C7CB2D9598D9799A4EBFCE33E7725B52870753FDE58A9A7A58EBF1B5987A382385A39D99974D3BE3666B3B0D52F241247B0F7'        
||'286E36FB9D91DCED87849333FE992090C1DAB914FC4241C06074AF70B796CAF8DACFDCC1573C2A3482B8FF004A41FCFA2C94F87FE2128C752A4BA6BE'        
||'2A9A489A54287EE6480AD16B44E7998460D85BFA7B14B49E18D6DE59FCBCFF00974C32D4103CBAC781D9F575D245A6026725D8978CBB864A76624A02'        
||'4FA4C6C481FED77F7E96E5101A9C7FB3FEAFE5D682F06E8E37C5DD8F94DC31761E15236965C45660F3A29C3C32353D2E6E859269D803C1D54EA5803C'        
||'5C7F5F60FE67DC60B36B79A4340FA96B9E2A41A7F3E8CACE2966D5A3CA87F68E8C66E3EBCCBE33190D5FF0E9D238A4A6AD33BF880B53D42331911492'        
||'AB7FD43FA71F9F6416BBB5A4F37841C57801F683C3A72E6DA758599863FCC7A68ECDF8934DBDFE2F753C736E6CBE32AE5EC9DD39DC850BD0D03C1468'        
||'767538A0FB077258A350E469A4935DC96416B027DA5DB79CDAD79C6F6331295482355604D5BF55C36AF2C3A3529E473D29B8D9826C56F721CD6495F5'        
||'2D050511082A789AAB8AD7D3A8D83EA9CA6176C60F0177C83E0B191D08AF9275866ABFE12829E3925448B4A931806C0F1FE3ED64DBDC135C34DF0EB3'        
||'5A5386ACFAFAF491125450A05683CFE5D2B64DA39A4A38C261E910C0277154B909164BC710996E822B7E7EBED3AEE76FE265CE6829A7F2F5EB656622'        
||'BA47DB5E982A364E7A4B4B263D54F9433A8ACBAF8D29C310ADA38622C4F17FEB7B7B7C6E76B5A06FE5F3FB7AAFEBAF15FE7F9F4F7D19F199770765F6'        
||'E6ECC867F27899B7374FD546F434B498FAA8619F1B96C17D863A19640AFA2696289DD88D42F6039BFB2DE64E6E165B758C10C6B278774A2A4B0C324C'        
||'19B1E601341C3A5FB56D27719AE04AC63FD166C50D4AB2155CF9134AF9F40A6D9D999CA6C42D16323734D89A99B10C1E281DA74C54B252C324CACAD6'        
||'668D839506DAB8FC7B145C5F41238925201750DC4E35004FF3C74556F0B46CEA0134623F2069D2EA9F68E6697159BCC4B4D3C74D82C45765E4AA74A6'        
||'5FB7FE1944F592CCF2CA52CDA63B1BF1F4B7D7942D7B6BE2A4258169182819CD480062BEBD2BF0A520B053402A6B8EAB869366D6E671FB772020FBD6'        
||'ADC84392AE962FDC20C74F357B473373EA77E6E3EA09FA91EE446BA4859D2BA68283F681FE0E89DD18E9C5735E9B725B52A23812A5A99E1FBEC94F02'        
||'B34441F1D1469E472BF4BEB93903E841BFD6E1C4B8535506B403F9F4D15EFA1F2FF2F4A9D8182CB1DD741F694C6AEA2B664C2D3808BE645AD8E492A1'        
||'A251F491608A6D173F51F9F69EF26896D5DA4340A357EC229F9548AF5E916A55466A7A17FE447C6BDCDB67B3536BEDAC36E4CC60E4DB3B7B36F50D41'        
||'5793AA86BEB639A3CAD24B538CA6586F1D5C7353AA05D40441896D4BEC3FCB9CD369B8ED5F5B772471B891D29A8282011A480CD5CA956FCE9E5D2DBE'        
||'DBE6B1B8F023466ED53C09C9E2303C8823F2E80E9FE326FF0092068A5DAFBB64B33E9A8FE0F97B0620BF0C6000587D05AD6BEAB7D3D9EAF31ED40E27'        
||'8C7CB5A7F9FA466DEECFFA137FBCB7F9BA4D56FC4CDE351084ADDABB91230910FB94C257B924F2A5E5FB7FA8B7A81E7837B0B7B787326D64D04D19F9'        
||'6B5FFA0BAD781734AF86DFEF27FCDD3A758F496CEA3DE74FD03BDA4EC5DB34BBB28EA3B2777D4E0164C4E761D978165C6539C66526A6D34AF57928A9'        
||'E86169207D7E670A18A9518B7EFC5E417DCC764F6EEAC05AD0E920D3F59F8D09EA65F6D43AED371E229149BCC11FE86BEBD0BCDD4185DA5B83B636B7'        
||'4EF6F6E6DC15B5741B736AEDDC2770EEFC23CF57D96B4B24F3E1F19350C5470AD39F3E2A867AB7875441EAD4058BFCEC2001CF522F1E93D0F60CFD75'        
||'BCA193E5EF456DBDBCF156E328FADF71EE5C641BCABEB37A6DDC89AF66EBACB2D2CAB513C951FE4F490D257C72544E4BCB4E62113A6B89CF5EA53874'        
||'F1BA76DF6C63F7A51768759F6EF5D453EE0CB3E30757EE6C16531351D57B229258DB2997DB7BAEAA6A93B8EAA28E69F2791FB8A5A2864758853FF934'        
||'112CBBEEAF5EA83D61A2E9FF008CDB83B4F03BAB07B5739B13746D0AE6DEBB9725B5B2B07F0ADCD519D3263EA37DEEBDAD9F79E9687275A63A996972'        
||'3005A9AA9E5A9925A6969C3483400AD475ED4781E919D99DA5B3FE4255F6E677E1EA6C0C2F66E3E86A3A6773F74E6B3298CC967F6ED3D5A56761EDEC'        
||'1EE9C6097FCBCA41FC170B93B32C730ABA9FB88A990237AB5E1C7ADE460F5137EE43E4C6E2DA7B2A3DA394D85D5DB4F6456E0F72E4FAB11E5CB677B8'        
||'45253452E3BAFF0025D89495D4F0631249E5D34B8C4A4AA8E79638854CF244FE35F1A9E1D6853CFAEBE4CF5ECDBBB01B23BDB7C76B6E8EB6DC5D598A'        
||'8B7457756E5774E1701D7F3D65646296BF6D4B1E3A1356F9C916A529E9320F5B24493B1A358228EA2665F115C9F2EB63D3A7197B4FB9F6B74CC5B8BE'        
||'297C7EC6CBB5C54E1F2BBCFB2F7D5126CE924C3E2E963A6CF6F0C1EC7C7250E5775CB8EA2093D55545534EB2C292AC0D56CADEFD902AA3AF507E2E84'        
||'CEC7DEBD7FD3BD2196F92FB6F6CF637C8FEC7A5DAD489B83B1F1F894DCBB836E60CC895B929AAF74A038DDAD8622635151418680986091E79691C219'        
||'3DF8E33C7AF64E063A29998EEBEF5AAC4FFA45EA8D83B6F7F7686EBCAECFEBBDEBBB30388FEF050ECBD91B8327054E470587C766E532EE27C64539A9'        
||'01A5F0F93D4C0C5A293DFAA7CBAF0D3E7D39EECD9195C6771E37B3B6BEC7191CBC3B2E6DAB989E85D76DE0B3F91ABAD6972FB81B19363E29E6CA0291'        
||'525564D1B454787459A3863227DF6D6343CBC25A9A89A4F3C7E0F2A7FC5F4E19DCDA4962C5343D0E52AC0F0C37A7A03C3A41E1BAFF00BAB135F9DC9E'        
||'3F616DDA66CEE5AB72523BE2EB2AEB648E6935C695B90A49298CEC808552CB602C17DC8AB285AD282BFB7F6F484DAC2CAA8D21214538903F21902BFC'        
||'FA5D26D7F90CE888765EDE2002D75C365C9B1B7FD36F1F51EFC6E0572C3FD5F9F4D8B2B7A503E3FD5F2E9C63D9BF224A131EC9DBBE9B5ED86C95893F'        
||'4BB495B6FF006FEEBF50BE6C3AB7D1DBFF0018FF0057E5D484DA9F25B9F175E6DF974F378F1DA49B1FA5E5AB6FF7ABFBA9B95072DFCC7571676C7064'        
||'1FB0FF009BA0B3B293E46C18AC9EDEAFD8987C752CF44990CAD79A6A78920C1D03A64EBAA286A2295984E91C65A322E2FEDFB59DBEA6264FE35CD47A'        
||'E7AA5D6DD6CD632B9941EC6C51AA4D0D067A91D50BF21B73D3E5717B2768D1E7B0382C8661B1392A8871D1F9691EB0D4D4C524F590CCED2899E404FD'        
||'34E916E3DB9BA4820BF955FD706A07100F49F69B6866DA2DE66942D569A68C6942479103CBAEFAE7B5F7BD3768760C9D95B50E317AC364D1D74989A5'        
||'5C61AFAE924CF5364CD2432D1470A8590471AC6D2C7E866D46E011ED032C9221D182C282A6B9F5FB3A1E45CACEB16D77026574BFB8F0D68181040209'        
||'6AD715F4E8A777D677ADF6AD0EE9ECBECCE8B8F2F53BA924DD1B8B2B3F77E5696A66C9E5A359696963C78898318D8C70525340BCAA2A2A05E3DB6EC9'        
||'6D6ACD41A545092C454FCB1924F90EA419F62E75DAECDE41BC4315B438D1E10002838452509248ED19A93F9F406F4F546EAEC0EFCDB9DF59AE9B8B0F'        
||'87EC1193D99B1FA5B65FDBE3DDB15534E2B6A73DBC6A7ECDE6AA5A8FDF699E38A1D0F2442230AAF2E5BED37D636F16F379FA22625163392174D43B57'        
||'812470E3E7D42BCC3CC316F57126D11B34D25AAAB935ED5AB534D47E2CD48F41D28B73F593FC7BEC6C975B6D9C651B6F2EC7C8E5A5EA7C90CBAE6E97'        
||'6D75FD593515705654EB95A1CAE3E1BD318E5A869E48A2592CAACA6230DBA133DC47B629A135D2CD8054658E78903CB25A8A0768A093B95BDC2E5BE5'        
||'CE4DBADC66874DF400075515F109A88C8A705A9AB83F8C93535EB8D60DAF478893AF76F6B8B656D790E5FB23783D5486A772E615FCB509535A481233'        
||'B1650A7D363F4B31082DDCEEAD76FB5FDD563DA00EF35C8072413E6EE72E7C863A28F6A793F75E6BDF65F72B9D98B8624C31B0EDE350403C15070F22'        
||'7EC3D73C266B1B4513F756E4C0F08836DF486C97F30D72497A6A6CAAD13F209D3AFCA45D57548DC9947BA6D96F0ED966776BD5EF6148D7CE8DC053C9'        
||'E4C7FA58FD2A7A2BF72F7FDCFDCCE6A83DBDE5473F4E8DFAEEBF09CF7313C28A2B41E7FCBA4EC187AADC5911B11AF3E7B74D4BEE7ED3DC3119D5A870'        
||'F21FDFC6455049318940FB78109BAA02DF58B94563692EF1B8913E43774C470D3E483D2B8403C9413D49BCF5BE6D7ECF7B7A9B26CF4172E9E144BF89'        
||'9C8CC840C935C9F9E3CBA5567F72E0AA6A06EDA5C2450EC2EB5A67DABD75878438833FBAB4AD24D91A4A76367F5B2C7169B0B5C7E9957DBDBE5F0BBB'        
||'91696C2A911D2A079BFC341F25FECD7FDB1E88FD92E4F6E52E5EB8E78E696FF1CBC532B33F148B8F9E6AE73FE0C7516A283F86EDBA1EBD9A9E37DDDB'        
||'F679777F66654CEED363F025048F8D33020A269514A8A7EAA1CFD54DCC6F34ECFB52EDA9FDB5C0AC841E00FC67ECE11AFC81E81BC8F6777EEEFBA93F'        
||'3A6E153B6EDCFA6153F096534503CA83E227CE83D7AB10DBFF0018F2FD45D57D0FF29376F506F596AFB3723BB68F62EE4931B051ED2EAAD814B85868'        
||'B6F659E7A7AC96796A378C15551E1FBBA1851218468919A550435C9FB85A6E1CF316D2855BC35954104D43089D9811400534D01049F90EA37FBF66FC'        
||'77BF6A777DA76E94783B7C96219707C476BB883D323FB234520712483419E8C1F5476353E537F6CCC7F9D58CFB8F049C3EA251B2F0AEB7901FAADCF1'        
||'7E7FAFD7DCA9CDFB2AA72DDFCA05088263C3CC46C69D71FF009525B9839A76B72752FD5DA826A301A74524D2BC0E00AE4F9F42DE4774AD2E5F314A67'        
||'17A7CBE462F530D778AA99416247D09E78FF000F614B3D984DB7DBCFA7E28A33FB547F303A3EBCDD278370B98B556934A0124705908AF0E0491C3CE8'        
||'3AE382DD0B59946A659859A9267043292E629578BF3F4D5CFBB6E5B3886CC494C861F9541FF0D3A6F6CDDE47BB2A4D069FDBDDE67E46A08FF3759721'        
||'5B9ECBEE5A2DB7B631392CDE525892A268E8A212C54149A2ED5F97AC974D3D253A2A9633D54C880024B1B5BDB56D1ED5B76D4FB9EF13A41154A82C68'        
||'5DABF04683BE4724D34C6ACC4E29D2B65DEB7ADD976BD82DE4BA9C50B2A6742D2BAE573FA714600F8E57541C751A53AD65FF0098177954C3DC9DA781'        
||'6929770EE8C965B2FD7F4CB8DABAB9B6C63B6CE05D318CD2E4E208678CC31A4AD146C85CC9A9F4A380D6E73E6CDB767D8A2DA2D2366B8B88BE16D482'        
||'2590066F13491FA84115881C0235E3073B3D92E4992C7956C66B9D2635905C96A06325C55BB9091511C66A88E455C2E3CC8AC6A379A692692BA596AE'        
||'AA79A6A8ABA96BB3D4D454483CF3965E14B1B703D200B0FA7BC7A2ED239790D589A93EA4F13D64552B91E7D2C31D4D494C81521545E35484DD85CF24'        
||'004DFF00C0DBFC3DA98C22F97E7D50EA39EAD97F95D757C59EEE88BB5ABF12E36CF59D3D54F4397A98CFD9D6EF9C8C2683158BA39641A2492952492B'        
||'66111FDB2912B90594190F94B6B6B849EF910E955281BE6DF1007E4B834E15EB1F3EF07CE36FB1F2B2F2D4120FADDC99018C1ABADB29D4EEC0700EEA'        
||'B1AEAA6AABD2A14F5B21AE513C61D58129FA88000B1172759FEBCF3ECCDAC9B569A63FD5E5D61A35F803392389C7E66BFB7A8D36614AB8D474942149'        
||'6BA836B9B91FED801FEBFD3DBB16DC410699AE7FD8E934BBA6A565AF1141FECD3FD4307A3CFD6150D57D7180995AE66C4217FA3D86961753F43CDBE9'        
||'F81F9F78D5CE11781CD57719FC3337F93ACDEF6EE76B9F6FB6F9ABF15B293E74C1F3E1FE1FCFA207D6759535B8549E50016CB672320DDF494CBCB130'        
||'057936D3C9FF0078F7935CD71450DD685F28E2F97FA1A9FB3AC29E58796552E461A4909F3A558D463E7E7E7D18EDA148D3662892489A557950E950ED'        
||'FA92C06803E82C2FCF03FD63EE3CF195675D2684547F23D4FBC876EEDCCD6819750D4DEBFEFA7F2A7CBECCFDBD1865C00A492290D3988C9665D0BE83'        
||'FDA68CAB01623EB6FAF3EDCFABF1411AAB4FDBF6F593C6C8DBD0E9D3ABF6638FFB3D4F48F43B68924550B631A950A4DEC3506E00FF0001EDB2411903'        
||'A706AC8A9EB1551A8B46CD0BBAA13CA5B49278FC91FEF67DDE309915A74CCD23E299A53C87FAB1D47A7C856D34E218675657B6AA694599587E4153F9'        
||'FF005FDDDE18DD3511C3CC7558EE2647D21B8F967F97FAB1D2AE39B229A1E286375B5982901416FC31FE97B7B2F68E02086343D18A4F70AE0AA83FB3'        
||'FD43F3E9C29B2F5504EA66A0975EA05950992F63CFD7F16BFB4F25AA3A7638E96C37CCAF49233FE1FF000F4A48F3F4B23FEDC4C4FABD3A498D2C2DE3'        
||'64E2C6FCF1FEF5ED1359CAA327A5C2FE276A50D787EDF9791E9CA1CB44418E4A2255BF5B045D2C840BFD07FC4FB4CD6EDF106FE7D2A1720608EA1554'        
||'D8082363151D3A6BBEBF1C4AA5AE3FB7A41373FE3F5F6F46974CDDCC4F4CCB25A22D42804F1A01D7541538ED0445046C8F7F51D20DD4F074FF005FF8'        
||'8F7E9A39B50D44D475B82684A1D0053F2EB3B51E201595F1B44AE5BCA6578935EAFF00541985C5CFD4FBA092E29A439A70A57A73C2B63DC5057D683A'        
||'8CF8AC24C8D0CF8BA1D1511B47AD2084C9A0F0D76D22C08FC1F6E09EE5486576C6789E9A305AB828C8B46C701D771ED0DB8CAA29F194102A82880534'        
||'4A083E90145ADC8BF1FEF1EEA771BC072EC7F33D5C6D564476A28FC87F2EA833F99C6369319DED84A3A6A314891F5561895F1C69E7277566FF00CA97'        
||'47F65ADA07FC14F00DFDCE5EDE4EF3EC2D23B6A3E3BFE5D9163FD5EBD27482382531C62801FF0051EAA8AB6C449FD6F2FF00BD1F723AFF009BA39809'        
||'0DFB3A8146AA7412A0FD3F009E3DD8FCBA767E3D194EA245FE3ABE903FDC7D7FE07FC721EC3BBC9FF15FCD7A676ECEE03FD2B7466361F586EAEE7EC6'        
||'D97D4DB1E922ABDDBD85B86836C6112A0B251D3D457CA7CD91C84A8094A6A48165AAA9700911C6D604D87B0BDEEE96BB36DD36ED7A6915BA176A7120'        
||'0F847F498D147CC8E8DAE16BDA3ADC73A33F966FC66EA6D8D85DBBBA768D276CEE7A5C7D3C599DD7BC8544D156578880AA7C460229452D153EBBF891'        
||'51E50B60F34879F787FBE7B91CC9BADF3DC5ACA6D62249548E9503CB53D3531F5381E8074A3C7940001A01D16DF997FCBE3636D4D9399EDBE89C7546'        
||'DD6DA74EF97DD5B0A3A9ACC8E1EB36FD30F264B2BB77EF1A59A967A34BD44B4E2430BC4AFA1637550E22E4FE7EBDBABD4DA77C61278A74A4B401831F'        
||'855E940C18E01A541A56A382882E58B6893CFCFA251B2E0FB0DB1B7EB008DE4AAC5D2CB1123500A6304BFF00C401ECC7DDBF741B90F69B7B2DB5164D'        
||'C2ED4F861F29122D0195D45351A9D28B500B5493414293D94FBBF2FBCFEE36F5B8EFD2BDBEC7B65C9F1CC44096E2690964B68DC82231A417964A12AA'        
||'5428D4E08562E5B22A6E2A9EC0DC2109E2E0DEDE3B5ADFE1EF1357DDBF71D6EFEB0EEB2B356BA4AC6623F2F0F469D3F202BF3AF5D0393EECBEC449B6'        
||'7EEA1CB9022529E22BCE2E07F484FE2F89AFE64907CC118E8CFF005CF60F566236D9CD6E134384DC54337DAD4A5509B275B917F1EB4AAC1D28576547'        
||'1E960883C6C2CD25883EF21B9279FAEB9F36B77B9023B8B72166443A63C825245A9AE97A1ED24D18102A29D73BBDF4F64DBD9CE688ECEC256B8DAEFD'        
||'5E5B49A400CA341025B7908014C916A521942878DD5880750E99B777CB098B4D4BB1B6EAC44031C79BDCADF7131005BC94F89A57D0B6FAA89666FF00'        
||'15FC7B16688D40D46B5F25C0FB2BC7F653A85123AD4D3EC2D9FE5C3A297BEFB33716E549729BE375D4D551C3EB91B275D1D0E228D40BDA2A5BC74D1A'        
||'81F4012FED45B43713CA20B44CB1C05049FE5535E9D631C69AA66141E66807F9BA257BFBE5BF5BED559E97006A37B64E22CBA3127EDB0F1BF2ABE6CA'        
||'4E3D42FF005F046FFEBFB91364F6C77DDCA925F916919A7C797FC9470F96A23ECE827BBF3CECDB62784959E45F24E1F21ABFCC3A22BD95F2BBB5B760'        
||'9A968EBC6D6C3CC0A0C76DDF2D354BC67FE56F2A4F9DF81F456553FD3DCC9CBBEDA72CED8C2678FEA261F8A5C8AFF453E11F6904F510731FB8BBDCE8'        
||'56DEB0464F04F888CF16E3C3D283A27199CC656AA7A99DA4AB96A2676324B505A692624FA9A577B963CFEA249F72DD9D9DAC68A9450A38018A7D9E9D'        
||'42DB9EEDBA5C48EFA9CD6BC454F9FCBA0EAA29AAE592EC1D803C58373FEBFB10A3C6AB8E80D3DB5D4AFC091F9F46A7E32F5FD56FCEC8EBBD931ACBAF'        
||'786F2DBBB74801B58872D988A92A1813FD23776FF0F71FF3A6E4963B75CDE9FF00408DDFF355247F31D4C7EDBECD2CD770479FD4755FDAC07F83ABF0'        
||'F9F9DDD99D85F28309B4F6B44B2EDBEB5D9DB6B6BCB851304A5A9A39A95664A00A9FA248D4C66175E5580E08B8F78C7C91CB76BBC72AC9777AD49677'        
||'770F4C86AFC55F4F5F5EB297987776DBB755B7B700AA05520D48D2470C79E3A36F8ADD1D7BF22FA7F0FD73F2776065F33B0F70D326476CE6F7262EBF'        
||'0DBBB6D56C0BF6CB93C764658D6A62A8A7E635C95196D7180B325442C47B0693B972BEF4770E5EB9115CC64AB1421A39071D2CB52A437128D8072A55'        
||'B3D1A48F67BD6DC2CB788BC5B77CA86055D0F0D4A70C0A8C065CF910C3A61EABFE537F15B6A4D36F2CBE5731DD98E8E6A9C8E1E8370C9052E16931D1'        
||'7EF53C3578BDB1EACC4A882CFF00B81643F5A41F4F671BA7BBFCDD709F4A156C8D00668EA493E6417FECC7A631E4FD15D97B75CB4ADE3256E40AE90F'        
||'41A47A517127EDFF0069D283B17E48F57E13095BD23D4DB25B1D475D149B7A430E1A2D9585C04730114B3536DBA6863943285B87AA8E2FEA41F61285'        
||'A7BBBB5BABE98CD213526A6424FA1735FD809FB7A1FF002FD9476DBADB242AB1A09134AD02FE6A8381FB403F2E8BCD3D7534205E596B2416B8576869'        
||'D4FE7D67D6DFEC2C0FB3B68D8F0141FB4F53357F8BA728F35536D314829D7F0B4E044D6FE8CE3D47FD8B7B6FC25FC59FB7AF15078759A6AE95FC4CD3'        
||'C8C4C29CB3B13717424937FE9EF4AAB9C0EA9A0F01D7FFD5F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7B'        
||'AF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAD863FE12DDB2B65EEDFE731F1DF29BC562AF7EB7D9DDE3D97B476D1FE1EF59BB77BEDCEA5CAD3E031386A'        
||'7C84D0096BA9BEEA5CBD1A23EA12D12B9D31AC8E9EEBDD592F467C27ECEDC9F363F953FC3AFE621F26EB33A9F393E4DFC84FE621F283F96AEEEDAB8C'        
||'C2617A1FB0B1382DD1BF3AE2A7B6F705566AA2B72B9CECEA48E6C6362B2188A2A88609EA71CE673E1925F75EE8D07CE9EBB97E4DF697F23FF875F233'        
||'AA3AE766FCC1EEBF9F1DC5DB5D85D4F86EBCEBEDA1BAFE3A7C0FA5EF8A883ACBA773584D93050C631157B1E846431F4D948A7AA91B1D399A68DC4BE7'        
||'F75EEADBFAE7B0A9FB5FB1FE3862765F4FFC7A5A2F90FF00CF0FE51E1FA2F209F19FA1FC3B6FE13FC18EABCFEC3ED5DC5B67253EDD9244AFCDE6B65D'        
||'7D145B9EAA4FE330C39C6A0A3ADA68E1A0A5A7F75EEA9DFF0096D51ED4DDBB7FF980FCC4EABEAFE9F4DC9FCC2BF9E8FC79F881F166AB33D37D6BBD36'        
||'E75B6C3A4EEF97BAFB33706CCDA1BCF1B5D8F8218FAF73195A89EA21A2FF003D8F5A84D5514A027BAF74ADFE75BBDF6B766FC09F953B7B0DD6DD19B5'        
||'E5F933FCF4AB3E2C7C6CAADB9D41D63B2AAF11B2FE3EED6A3D87DA1DB1FDF4DBF8BA0AEC8D56537F6DDCFD3E4F3791AEA89DE9F21F6D14FF006168E2'        
||'F75EE85CFE747F1EB62FC33F89DFCDCFB523E8BEB9D97B7B2BB27F9747F2E6F8579B9F6EEC35C8D46D9DB5B6A0ECDEF5DDFB76930A827A1ADC9D36E8'        
||'CBD054D5CB1C5533C9880FCC08DABDD7BA5AFC80C26CEF86BF053F9807434BD57D2D8CD8BF097F9207C47E82ECCACCA74FF574FBC373FCE2F977FC67'        
||'6B536E8CDEF9A8C43656A2AF165B11908B1EF9034E2B7291D4BD3C92B53BA7BAF755E7D5BD3F86EA6ED9FF00849B7C36DABB6F62ED5ED0DC3899FE71'        
||'77A66B37B4363576E2DDFB53E406F86EC8C160776CBB9A86659E783018FDD184C64757AEA692478528C53D6D340C3DD7BA1CFE324196FE66BFCE63F9'        
||'F87C76F9454BD7FB6A6CFF00C7CEE5EA8D979ADDDB1F61E333BD37D39D5DF21319B036EEEF87298FA58923CA6336D67BEEEB7295933573CCF1473D62'        
||'470809EEBDD6A49FCD37E54EC3F981F36FB7FB47A77646D9EB6E81C157517557C78D8FB4F018FDB98AC2749759C276CEC6AB9A8282353F7B968A2933'        
||'99069DE59054D6CB1F95D234B7BAF75B6363BE2B7686E3FE5C3FC8DF29D2BF22321F0BFE307C4EF8A1F283E7C77BFCEDC56CCC1EFF00DB9D7DDE3BC3'        
||'214F538CD994388AFDC185A6C9EF4192C967B018CC749918AAD04D20A7469698D2C7EEBDD0ADFC91FAC22DA1F0CFE14FCCFC9E2BAF72FB7BE55FC8DF'        
||'9D5F233F9B4FCB8EC9DA7D75BCEAF65F49746E03766DBDB1D3BBFF0072EF6A3AC7C563B76EE35C6E7A6A6C7AC3593BD44D3A930C8E1BDD7BAC9FC93F'        
||'60ECDC6F4AFC24DDBD5DD39D5383DC3FCCAFF9BC7CA4EE8C1D5EECEA9D81BCB21B0FE11FC6AEB5DCB3EE5D89B36A37950E5A7C3C1165F0306069EB69'        
||'2A16A29A4C8C8B0554734914BEFDD7BA0DFF0092BED0C0F7876A7F319F94BB63E3E41B27647CA0F9E9D8D174B7CEAEAAD8DF19BB4367FC4EDB3D7BB8'        
||'72DDF598EBDEE0EAEECD612ED4EB6CCE0A6A0A5CCE7B154F4EF938A78684D752CE696B22F75EE9CBF93CF4C49BBBAFBB5BE6CF41ECDE80DDFDC7DE5F'        
||'CEDB19D5BDF3DDDD73D51B0317D65F1E7F97CF5851C9DE5DD9BCBAFF006AF62AE5293626DEDE1494'                                                
;                                                                                                                                   
BRANDINGBackgroundIMAGE_6 raw (32676) :=                                                                                            
'D5B8C772FAD69B2B8DA7491E786123DD7BA23F5BD83D45B33E0AFF00C2993E7774160B03B5BAB7E5B7CB9DA9F083E30B6D0A15C0ED5C86CECDF65556'          
||'F0ED0FEEF50D35D69E0CC6D0CD53665A8A109046AC90C71450B08E3F75EEAC5BF986EC9E89F80BFCA6FB57A736FF00C7EDA7FECBF567C0DF8D1D53D7'        
||'9BEF71D6748E3763FC88F963DE54F2EF3CAFC98E97AE6DB15BBD778EF6C2BA8C8E6A75DC94F8BC7E35607A182358DBC1EEBDD7CF47DFBAF75EF7EEBD'        
||'D7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD7BDFBAF75EF7EEBDD6C49FF00099C2C3E7776C8407C8FF1237E'        
||'24657F52B37716C30CC0FE3D3A85FF00C7D8CB922E20B3DC6E2EEE982471DBBB313E403C7FB4D6940324D00CF4EC169737F731D959AEB96560AABEA4'        
||'FCF801E649C0009381D6F231536A2113903EADC9D4DF436FF88F7276DD1CEF236ED7EBA669400887FD062AD553FD3B1EF988E2D45CAA2F465BB4F6D1'        
||'449B1EDAC1EDE0259E41FF001227228D2FFCD341D9003C1353901A56E95343863300193506B7E0DF9FCDFD95EF3CB9CB9BCB19373B38A473F8F4E993'        
||'FE722697FE7D186C3CEFCDFCB2A22D93719E08D7FD0B56B87F3864D51FFC67A5252ED4495DC7F94058C2A9D134A099586B6B10780AA40E3FA9F60D8B'        
||'947656BF922824BA486D80514BB9FF00B56153A496340919031E6EC0F0E86F75EE673247B5453DD43612DC5E16725B6FB4FEC11B4AEB02315324AACD'        
||'DC3846AC3E2EA5AED0A21345E7FBB78CB85757ACAA5051BD26FA1871ECC1F92B6199693BDCCA3D1EEEE181FB46B15E8A13DD6E6EB7CD9C7636EDE4D1'        
||'6DD66AC3EC262620FCFA9B88DAD4510AD55C6D3BCB05488D59E9D2591542F366704FF437F6A97967962C0C66D2C6052071281DB8FF0013EA627F3E8B'        
||'E7F7039E7775906E1BBDD3AB1CA899A34FB02465100F905EA0D7512419748A56D2B262656D10E97378AB174A123D22DACF3CFBAACC539A638A3002BD'        
||'9BFC8764E94C7CB59FDBD2858C4DC8135C392CF1EE710A9C9A4969254D789AF863F6749FAE8C445BC10AC67F123DA5941FEA19B85FF9047B15A1AE58'        
||'D7A087492AB47762F2333B0F4B1762D7FCFD4FE3DAB4206075B229C3A619E3D26C3E9F517FC7E45FDBC0F5BF2AF5829A6F0D52A1B18E7262914DF4DA'        
||'51A2FF00EB8BFB0EF326D33DE431EEBB6505FD892F0D4D048A47EA5BB9FE0994507F0BE97F2E857CAFBCDB594936CFBB54EDDB80549E82AD1329AC57'        
||'318E1E240D93FC71974AF77516AE27A791E291486462083F91F83FEC7DA9D9F75B5DEF6F8F72B22744832A70C8E0D1E371F85D181561E447A53A4DBC'        
||'6D575B26E52EDB794D71F065CA3A30052443F8924521948F23EB5E9A25FA7FB7F66ABD169E9B9881FEDFFDBFF5F6E81E9D6CF4DEE39B7F8F1FD7DDFA'        
||'DAE4752A8E8E7AB77586169742C65CAA3B690EEC072BF4FD3FEC7D93EECBA950FF00A6FF009F7A20DF2E7E9A68734A86F3A5694F3FCFA7D8B095409B'        
||'40EA47D418DEC3FD7B8F643A581AD0F45E6FA078F4975D55AE4F0FCFD3E5D7A5DBD51557090B90CA149F1332DEFF00EB7D7DD1D181A914E9659EEB6B'        
||'18D0AC0F9D41FF003F1FE5D079B9760D4D429B53CC2C3E8602072DC81F43F4FF0063EFC030141D1AC3B9DAB30919947CEB93F223A46D06D093175563'        
||'49346A6E599E02CAABAAE1C7F436BDBFC3DB891BB30D3D3B77BC5ABC2DE330229800FECFB475AF5FCF7C639F97DBF26516A79FACFA8A4894B92E91BB'        
||'E7834862E4AEA238E39FEBFD2D780FD5B7C953F6D0D7A2BDB9B56DB09A52BAC8F98D583F3E8BD758535B7B529D240FEEE76320B925AFFE8BB34C5CDF'        
||'DB318EF1F9FF0080F4A8FF00ABF6F4683AEA9ADD5BDC8C40046D9EAE5161FEABB671A49E7FE0A3DBADFDA20F5AFF0083AD37C4BF9FF83A03FBD29CFF'        
||'00A2FE8DB01EBDC5DEACC6DF85CA6D7417FF00623DD26FC3F9FF0093AD799FCBFCBD17AA0804391DAB238D289BF3AED1CAB68FF39BFB1A8CDA8D80FA'        
||'FE4FFAFEEF6C74DC44C78074FF008F8EA8D1B4A8F147F132B81F69534FE7D5B2E1F68D6D535155D322BD33CB3C91CFE32F6BD5C8AEC814FD41BDAE3F'        
||'A11EC717369792D26B652CA7D057A8B59E3DBAE6587714286B504E01C02334FF00570E8C96D5C4D0E320D33346B56E41795902999880EE59AC2C18FD'        
||'47FC4FB299B6FBE350D13D7D749FF3746B67BF6DD04C9224EBA064AEA029F91F31D2CCE46155B1994E9478C59C1D1A4925B48362B6B7B2D6B2BB5C34'        
||'6FFEF2DFE6E8631730ECD25185C45DD9F8D057E4722878FEDEA30C9C249819E31E32ACA5265F1FD6CC0163F9BDC0F7436575C0C6DFEF27FCDD2C4E63'        
||'DA69E2A5D4441AA9EE5AF0C573E5EBD24BB3B394F1F5AEFC8848BA86CDDD118B4C0F2F839C5B4DFD40DF9B73ED1EE905CA6D573A91852292B83FC07E'        
||'5D56C770DA6E77981A3962667923D342A4925C500CD410780FCBAA78DBF9E928EA2B4BC94E824280175363A6563C73FE3EE002BD4ED1C9A6BC3A3C9D'        
||'7DBD6393EED5EB2805EAA814591C5C35C1FCFB4ACBD086DAE3E63CBA31B2646014957908E54966A18E5961553E932431F9630C3F3CDAFECC797B6C8F'        
||'79DFECB6890D12E678A363E8AEE0353E74AD3E7D2CBDBAFA5B19AED32624661F68048FE7D0252CB2CF2C93CD234B34CED2CB239259E47376624FF5F7'        
||'D18B6B6B7B2B74B3B4411C51284455140AAA28001E807F9FA80649249A469A562CEE4924F124E49EBA8E5921912689CC72C4E248DD7828EA6EAC0FBF'        
||'5CDB5B5EDB496778824865528EAC2A195850823E63FCFD7A396486459A262AC86A08E208E07A365F1DA9303BA3B733190AC55A8CBECFD89B73354D49'        
||'2C2C62A5AADCB553D2C1918DD8692F1AC132A58DD0B06E0D8FB887DBCD9E6D87956EECA33585373BD815AB92212828471E056BEA7A43EEC737C17124'        
||'3CBF6CE56E65B78279D402008A40DA406E0753A9AA835000AE0F47E3D8AFA83FA271F3B7ABF17D91F1E376D6CD474F26E3D8294DBB36AE41C469514D'        
||'3C55B152E571E952DCAC3574D23A4884E92EB1B7D507B197226F0FB4F30C5562219EA920E208A12A69EAAC011F2A8F3E837CD5B13EFF00B535A5BA06'        
||'B8521A2A903BAB9153C032D41AE387A755ED494152B4B48A55432D2D32B0D6BF510AA917F77964432B9F563FE13D667D9A14B2851F8AC680FDA1003D'        
||'087B5371E236E5154D2E5A79209AA2A85444B1C12CE0C5E211DCB44081C83C7B492D95CDE1125B8040C1C819E8BF73DD2CEC2558EE9882C2A2809C54'        
||'8F2E94751BDB6EE520971D47573495558861811A9278D5A43EA00BB8B0160793EE91ED77B0B8964501572723A29937EDB268CC71B9AB70ED3D327DB4'        
||'BFD07FC943DAAA1E90FD641EBFCBACA3646E3AF02B29A9217A7A8024859AAE9D0B21E2E518DC7D3E87DDC5EDBC43C3726A38E0F45F2D64919D78138E'        
||'9F701B673181A99EA3274F1C314F0082364A88A6264120908D31924703EA7DA4BDBA86E23558CD4835E04797461B52113313E9FE5E95F0CF1C534534'        
||'8C42452C72B9009212370EE401F5E07D3D9595245079F47868057CBA1DFF00D987EA9FF9DE643FF3C792FC1FF827B29FEAD6F1FEFB1FEF6BFE7E91FE'        
||'F9DBFF008CFF00BC9EAB23E6863EABB97B4305B9FAFD172986A1D8D8EC254CF56EB8A913234F9AAFAD9615A7AED2E54473C6438163723EA0FB9ABDB8'        
||'BEB5E5DD925B2DD98C7234CCE000586928801AAE38A9C758CFEF3727EFFCE5CCF06E7CBF12CB0A5B244C59D50EB124AC4518834A32E7A003ACFA8FB0'        
||'F19BDF0B5FF650D37DBAE51BCD4F5B4957326AC3D425929E32CCFAAFA4803E87D8CB72E61DA770B27B3B672CEFA680A915A3027270300F503F327B77'        
||'CD7CBFB34DBBEE9022410E8D444A8C46B9111700927B980C0F9F461E2A7A88EB27A5C9C7530D5B4A51FD3E46672095B445A3717018B81CF1EC26CA54'        
||'70C751A46CB5D2C6B5F4CFF9BF3E9C319B3F295F560E0EAA95D643CAD54F1D0E962D628F3571448EE7804C87FC4FE7DD68482E782FA7F97D7AF76DB3'        
||'045C17AE48AF03E47CBF2AD7A13729D6DDAF83C609A6DA7B8A8A26869EB054D3C2F94C34E93BB462AA97238F32C4C1AF63691FFC6DC7B696E2097E07'        
||'56A1230456A3C88E3FCBA50B494B7875D6402568F807009EDF3A71E912E3254356D4B571D4E16BB490F4B5346CD4F386425FEEB195E8C18724FF009A'        
||'07FD707DBBA59454F0F5FF0031E9B511DC26A560C69C01A8E246718FB0D2BE87A6B9E9A992692714F8FA8913C4DE4C6D6643112AEA3771124E5A153C'        
||'FA50446DF416FA7BF0614A75A285781071EA453ECF2FE47A5F606972B4B2534981DDD5BB5EAEBDA589FF00BC4950312C1135457CF6DB8A72C492D1DA'        
||'A69230188BB052486598906885E9E4A0379FA123FC3D50A36AA4C54293C4B693C302BC09AFCBCFA5E9ABEC0A3DBD5DB832FB6A8F338AA2962872FBC3'        
||'6FFF0077374D25319C32C6B9A58E2A9A8A60EC8409E468949E2E4FA7DD0C96AD2ADB9256420955AB21C71A024569E99EB491CBA0CF4A460A86620B01'        
||'5AF980CAA38D0D47A53CBA06D72185ADC83549C363B251C923068E3873D4954E587E831E2EBA387FAD8246B61F8F6A46A2028047DA01FF000E7F9F5A'        
||'68D114CEB29603810E467C87F0834E1DA71D08189CAFC728E8D21DE3D3FBF172730A9592BB6BEFA9F134825D40524A28F3B4192F269176957CC80FD0'        
||'7F5F68A58B700C4DACE84630F0D7EDCABA9FB38F4EC67490972ADC72524E23CBE25A1A9E3850070AF9A5AA68FAE6791EAF6ED7E66828E3A8444A4CC5'        
||'501530467FB2F94C4452C4CCA01043532823FA11ED6C6EA00F14669922A057E409AFF33D69E3BC00E965C1C00171F69C8A81F21D29B6DF6C767F4DD4'        
||'55CDD274BB1B7BCFB9DA9A9B702E6767E0FB31B09498E266C7544516E0A241466669E58E43A0EB2A01B851ECAF76DAB6EDDC46B7CD2A84A91A25922E'        
||'34AD4A32EAE18A934F5E3D0C791AEE58E4BA2BA31A077246D9EEE1A8C94FD8BF653A309B6FE4F7C8AA881A5C8617A8B1EE213AA969FA37A9A0D409D3'        
||'1A086A71721363A8FD783F4F6473F29F2FD68AF72DF337975FE497A9063DC6E3CC43F67830FF00D01D2EB6FF00C90EFEAB8C4F58DD7343189245063E'        
||'92E9AA51108868B031E188D42DCE917238F6965E54E5F53A7F5CFDB77747FEB2F4EAEE531353E157FE68C3FF00407420E33BDBB27255D86C464B74EC'        
||'FC7455B99C3502D451F57758514B422A3291245534B350E1A2756859848195D4AE91622D7F68E6E5CD9E08A49E38A572A8ED43737043514E08694835'        
||'029420FCFA752FE691D63D6AB520544718A64641080E0FCFAB3AF91DD239AEDCED8C86D4C46CA8A68A8F65D10CE57D5D4D3D07F75E966EBBC745532B'        
||'0AC7333AA6428696AA048E390B25950FA943441CA1CC96BB16C297D75734D73368500B7887C772294141547656248A1C9E048166FDB4DCEE1BC1B7B5'        
||'8B291A16A9A6902200D6B9A0650471A8229C4745E7657F2E8DE558D8BCDE6378ED2C0CD8FC8636A5A99E7852D0D3BACC2AE9CD54AB2B8D5A878DE994'        
||'F36620FD051B8FBB5B6C45ADEDE09650CAC2A0139E1434141F6863D17DB7285FCA9AE4748F23E22063D4799FB0A8F9F46776FF00F2F7EBEC1E5EAF2C'        
||'7B1F126596B6BAA218A2ABC7C54F08AE77964A48B5CD1F08247442F11F493E9F60EB9F763729E1108B37A000124312698A9ED3C69E47F3E8EE3E4B81'        
||'7E3BA4FE5FB38FF93F2E85DDADF157AABACE5AEAFDB1BDB62ED49F25494B8ECAD7C396C5495990A2A67BD1C556F515518628410AE46AFADC9FC915D7'        
||'3E6F3BB858EE6D2698292CA34B501F3228A7F674B53966CED2ACB7714608A13A96B4F2AD5879F52F7875B75F64B0F99C4C3DB5B432B9CC860B394B41'        
||'8A83706065C9E4667C6CBF6F4F49454D57E69646700461159AFF008361EF5B7EFDBB457114EDB7CB1C4AE8598A3E951A85496294000E35A0E9ABBD9E'        
||'C1A1745BD89DCAB51432549D268000D527D280F1E8AEFCE6C8FC97E99F8DFF001E60F8FD4F550F6164A48F1D9F18BCF6370F8EA857EAFC54432B94FE'        
||'272D22579A5AAA730C3134A84120B0923528C22E451CBBBF734EF126F0165856A63F110B95FF00189B0B40DA350209E23EC3D27DFD770DAF64DBA3B7'        
||'2D1330EF28DA757E94433C35508207D9E9D057F1FF00B0FB9E0E9FEBFA5EEFEE0DD993ED98B6F81BFF002516F04829EAF35F76ED55353C580921A20A'        
||'17446A69E255B2FF0052C49EEE3B26DCB76C2CACA26885349F057D3E6B5E3EBD13C7B9CDA3BEE24AE7FD11BFCFD1816EC08BEDD64FF4B5BBA24F1B99'        
||'1D77BEE0B0F1280E5E37A9207D6F603E83FA5BD947EE842D4FA088FF00CD94F3FF006BD3DFBCA502BF54FF00F391FF00E82E9925ECC8E0495A7EDDDF'        
||'12235405121DE59B45F1051E12816A2FCFA8DCDAE3EB616F6A46CA8C405B0841A7FBE93F3FC3D367749071B9929FF351FF00CFD277E186EFF903BB7E'        
||'647746DADE7BB32FBE7A0EBBAFE5CFF58C194DC14B947C3D4D2D7EDFA3AAA196844C65923F3FDE57C32D6A3C896B24A1488821E7DDBB65DB793ADAF6'        
||'DA2482F3EA115CAC7A0D0C72D46A0A38E0100D0D723CFA36E55BABBBEDEDEDDA47962F0988058B0AEB8E8684F971CF0A7429EC4E9DDA5B271B9CA1EC'        
||'ADFDB6B66E7EAB7A6FAAF4C6E4F71636929EB71EB9F9051D7E3E7AD9219E584C49A3578C0D408FC11EDBDCF98EF7719629366B492E6258615D4B1B12'        
||'ADA055580054104D78D69D24B7DA2DEDC489B85D240E6490E96600101A9A85482463D3A7BADDA7F1F278EB68F23DE1B01A82AE29A09A8ABF74D0D6D0'        
||'55C73C661962A9A5472B665B128E08FA8B91ED326E7CD29A648B6C9F529AD446430A645091FE0E9C363B39AAB6E11D0FF4EA0FDBE9F9F41F567487C3'        
||'EA9A2112F7075DE0E512A98D30D9C925849BBFDBB98FEDDC37A1D8A840381A381CFB338F9A39F51EBFBBA79053F12007CABF88798F3FB7A4EDB472F1'        
||'4AB5F44ADF224E3FDE7FD5C3A083757C74F8CF95A0A0A3C3FC8ED8D1AE21327251D3BFF1759A77C8D6B4EC1D8D1CE1849210A1CBA58581E1793DB2E7'        
||'0E6E8E466B8D9E7EFD35234505053F8D787A50FF003C17CFB26CD4FD1DC2224570438FB3F0919F9D3A4DECAF8D70E2A9AAF786CCDC5B4B77E07696EA'        
||'C853EE4C9D25549F7F42725B0EB6871F5F88A78A19D5A38D7EF268A599A27D7EA289A412BB70E6FF0015D76EBF865825B88D5A3523B4E9990B2B1246'        
||'49D2081A853CCD7A4706CCE11AFE091244858A350D68591A8C3070002413A4D4701D20BF989516FDDBFF002772506D0DEDBDB69A65F626DACBC98CDB'        
||'DB9373E228698C793C96331D2BD1E0E714ECD2D252534F2B69D4CEEE5FD5EDFF006B0EDD7BC9C8D7B045378733A0678E3727B519B2E2B877651E4001'        
||'4C74BB9B84969BDB88DD935AAB514B2D05582F034CAA863F327A21991AFF0090D2465E8BB4BB62AE30198CB0F63761519FDD4063090FDD0D216CC57D'        
||'2491F5047B9192D39701A359DB0FB6084FFCFBD065AF2EFCA693FE723FFD05D327F10F91EC6174ED0ED8084EB9C41D8DD8331BD8A33489F704917040'        
||'B0F51FA7D7DBBF47CB42A1ACEDABE558211FF3EF55FACBBE3E349FEF6FFE7E977D7BBD3E446DA7CBE6F21D73DA9DB78CA374A2A1ECACA672AB21B730'        
||'39DCFD553E2713B6B7154EE790D635351395CC3434B1B2C2923CC23964368F1A3DF28F6F87986C976F8922536B911A2A827C5932740009A799CF52E7'        
||'B7324936D770D2BB3FEB506A249A685E153D0739BD97F10701DCDD7F3EE6FEF76CAEC5C52CBDAD53916EC7CC66B1BBD61A2CBB63E4CFE4B039C6A891'        
||'F5E55C4D2D453CD4D079606863431811AC2840AF521549E8C7673BCB3AD3BEE4A5D992EEAF8FB86A6C2E3F6D65B1D9EA0DC5968BB03275C895DBDB39'        
||'B32709554A678FEDA97059267761125548560F383EF75CE7875AFB3A0820E93DEDBBF7B9ECCD9DD934392C9EFF00C762CEE4EB0DE1B1654DADD53B2E'        
||'6A9922C554603334954CD5F9178637AC8A82BA155ACA8896577828A1A789B59AFDBD5BE5E9D2577275075352EEADD51F487636EB8F7AE1374E23FD99'        
||'98B299993B1F0DBD2A6BF1108A3DA32265268A0A0CDAD24548678B154A9434F4739A6A887C922A2EBCF07EDEBD5F51D38D675FEC9DCFB93C3D492ECB'        
||'D9BDCBB568B04377ED5C2514349B3AB76D55E20D250276063F6C528868F2B053D33C941E084CF3C4A29A58D29CC73D36FE63AF57F67481A7EA8CC6EA'        
||'EF1C3765E13E40C9BE37DEC2CB6536BEE0EBCDEF8AA2DBBD25B1726D045164ABB63D3ED28AAEB137451D3CC8AD1CF515D593412494AD59492B73A22A'        
||'6B5EBD5F2A74EAFD7196EBCF96181ED3AAEC7A8EFD9B78BE1F656D7EBADCFB6F1384ABE9ACC54C6EF06E9D8581A5FBAD388AD52F1D5E4A5A76AEA5B0'        
||'612CD0CB214F5286A4D7AF56A2831FE5E86CDD59EEF0D85DFDB5370F6F6F7D97BDBA2F7450566DFDA7B4FAFE833D5B574BD942B94D3C5BF9AAA2A99B'        
||'3134943AEA314D137DB065A91F681A28E6F7E35AF5AC53A554BF22ABB7CF73B7C585E96DE9D718FAFD879BCECBD83DBF86A0DBDD7398DA55539C0645'        
||'B636C5C3D46BCB8984AB4F245355E3D612C1AA15231A64F66B4EB6694AF4597275FF0016BA668A9BE3F627E596DFAEECB7C8F870786D99B669F786E8'        
||'C450D5E4160A6C1D2ED8D9957514906889A64693259150154BBE80678E4D54791EBD4AE698E873F8EFB23756C6EA918EEC8DEF3766EE98F7A6F8CAD5'        
||'EF2AC9E3AC9F274193CBFDD6247DB8A58168A48A98A9A9A185A658AA0CC7EE252C4FB9D3DBA2DFD5EA7FC3A4FF0002F482E333D01A60746269C3385D'        
||'315E3D2BA5478EEAE6EDF4BD802BA48B7F5F63827A64A81C7A755695A44856072CE5540D496525C28D6C5BFDBD87B6CD295AF561C3D7A73856A5A164'        
||'30BFF9F47D4026811ACDAC5EEDCFA396E38FC7B6CD2B5EAC29C3AE12EE2C2E2A454C8D7434AEE0DC12C151A43636641F8E0DBDFBC391C768AF5BD2E7'        
||'E11C3A28FDF9BDB6AE40E6A828F2504934FB3772D22C562106BC2CA89E3160B6E401ECD2C637428587E207F98E9F31486D64A8FC27FC07A4BFC1DDF5'        
||'B7B05B7B70D1657290515425665E57A797C8CF1C15094F52640145BFB65881CFB55CCB0BC97CDA457E1FF0745BB34523ECB0E9CD0B0AFD8C7A245BEF'        
||'7862EBBB17E626E3C2492D6FF0EE99ABCD253D0A89AA2A0E0D23668A923B8D6EE51742DFEAC07D6FED904C5086A124790E269E407A9E03A9A2E5A4D9'        
||'F97B976E2F54A78576D2118A94CB022BEA0D47967AAD8389CA6F9ADC076FFC8D8A6A6C56360A44EAFE8BA2A8FBBA99F21152AC74D96CFBB5C4F592E9'        
||'D7233204897D0814062047B6EC5F4EABBD73100BA4D63808A853C4161F8E43E4BC1789E809CC7CE9CC5EE3EF5FD58E50427B882EBF0440D6A6BC19E9'        
||'C5BC8616833D0CD93E81EC9ECB9BABB75EEEC76E4C063733D87B521A2CA62F69EE0CD6D3EBEDA06B0C593A99E0C144F535354616697C7491B48FE2D0'        
||'BA752828398374977460E7FD09832C60D302B51538D4786A38AE063A12DC729726726F274FB2D95D4773BACAE86570C19F5E75027C9057BB3C7CBAB3'        
||'EDF5F107A3BBEF07B1BCBDF3B9B21B5F13529924DCD8DF8BDDDF8ACCD6B22E9927C366A9EAA99A89574B379C82DA7F52817B862EB74DCE6A22DA6865'        
||'2083E3286520E0A914A11D47163B7AD949E2BCF1B5450AEB4A303F8581D55AFA7AF4517B4BE136DDEB4C9F546DADFF00DCBBD313D5FD9394C8E4F634'        
||'359D37B8BAF9B7BD362C25453D4D6EE3DC53CE9942CB3D2AD42428855E5D4C02CA10BF6FB935E170D28592120B28D24EBAD48726B5E049F53FB3A913'        
||'77E7CE6B8AC22DB634823B670502C207C01400AC54D57F9573C7A2778FDBBD73B97B07AF3239DF96F9EAAABA4CCE13078CA24D99888E8A1A1C857C14'        
||'72D1D3E2E955211E55621D8465AFCDC9247B32BDBFB8BA91669EF99D92A5568A16A78E0002A78578F40BE54DDF78E57BB6BAE5EB211339D4ED467D59'        
||'FC45CB12BF2AF4B0E97C7756EE8DC1BDF62D47C97CCE36932F439DCC6E5DD18CE85CBEE7CD2262E545AAA393214195A15A68CD2AC91C2ECD65360A8D'        
||'238F6AED7982FACED1ED76ED33190E4E9D0C0B6356BE15030A298AD470E848DCBBCC9CFF00CD36FCC5BE6DCF39B620843308E13A72168C1B49273EA6'        
||'9D19A8BAB7E275545B2777537C97DD53EC8DB3514984DA9B7C7C66DCB45453EE29EAD71B1E526AAACDC0BF7123CAE7C64C410123D6551487EC24DEAC'        
||'69BAC9B7878954B29328007906A52A6830B8F324549E9773C739DF73ACCDEDBD8E9B1B9D45648E393C462147C1508114003D481D0FDB77E08759EE2D'        
||'8FB8FB76BFE4EE55E9B77766C7D7382C356742EE74DFFD8FB929A93F8A41D77B1369E1B2B5D552C8B4AAFE79C2F8A10B34F33471AB37B20DD39AAF1F'        
||'702B3DBD5DD43160E02C683038F90F2CD493C2BD0B795AF370E45D88F276DFB7A433449A9E569C305D6011348420515A8A0A9380067AB14F931B9B7B'        
||'6DAF87197EACCAED5ED0A1C1565375F62F115FBCBA333DD5D4987C6E0F74E37254F514BB8EA3726628668EF4CB46B17DAC4D31903A6858D943BED94D'        
||'F57EE15A5DAE802333B1D2FA89ADBCCA011A54FE2AF13C3E7D62A7DE3765B24F6AF7198CCB2492CB6E1809564249B88D9BB7C3561F09A9AD05735247'        
||'555BD1549911DB1B0D5A653147BBB6E33B2C9CB85CDC2E9AD0F3F516E38FEB73EF28799EE233CB7B87CEDA7FFAB4E3FD5E7D7319B69863DCAC481A4F'        
||'D5DAD0014AD2E6261839C107CC035048E8DF6F3C5D52EF5DD8D4F21D32EE0AEAB2C19569CA5446953E826C6C3512483604DF9F607D82F236D86CF58C'        
||'88545389C557FC94EABBCD988B7AB9CA8066735180751D64F1F206A4D307249AD7A143A2FA932D92A9ACDE1B95B258DDB30E327AAA1868E9E139EDC9'        
||'45E441535F8A4AF64A5A2C4456D351B83212474911368CCCEA5003FDC0E78B4B185764DAC2CF7ACE03549F0A034345934D5E49DB8A5AC40CAC055B42'        
||'904C85EDD7B5979CD172379BF2F67B524608914564B8CD49875D512000F75E4A7C252C044B2BEA00AAFCDCFE6278AC66D6AEF8F1F16ABE84ECE96BA7'        
||'6EC1DF98647C9E3F2B968A13492E2766CB4A8B5195A944FDB9323505C18C1606246BA86393F916EA2DCD39E79B81B8DC42948958176813E2A089491E'        
||'296EEF0630A917191EA2BD65F6DDB1D85BECC3956CE06B3DA43ACE2D50885AEE6145FA8B99A40196DC2D144B3B196634089C13AA53A4A4EBEEC5A193'        
||'6BE4B1987DC799AAAB9728F8ECA554997DC1265AA488EA6B68FF00834D7F2B0089245133E9BD9B57D4CB507F56798213B6DDAC73B826578DDD5E4D67'        
||'05C98CD14D000CA8D45F84D7A5DBAC3CC9B2DC8DD5247B6B755118686331DBA44B9084CCB42A0963E23005F882A30067DB9FCBF769C98F8F33BC30D8'        
||'8DBA2A02B51E068EA72557952854309F254F154AA5200005F0091A424DDDA322DED00E48E4E795956C81238E969156A6981DC4D053D6A6A73C2803DD'        
||'BDF3BEB36106D644A33FAB26800015C29D3FA85AB5D6555052815A84912735F0FBA8762F4E63BB5B6EEC9A0ABADA8DC159B6EBA3CAC47251D037D91A'        
||'8A5AC8E5AC92630EB647470C5882140E49F6BF6DE5FE56B1DEA4DAE2B08448918915994B573A4FF685FE4715F3E83365EEB736F316EA2CA7BA78A020'        
||'3288884634A16074AC7A80191554A8A57A31DF1CA97271F546C0A8A795A931D5D3658AC34348B4545475143BDAB28F2147042A4AEA489A99C9162048'        
||'A589241F66BF511785246D1A864C69002800A02B451F0A9A1A56B5A1CF5197B9FB7C7FD7EB888CAC434709ABB6B67AA77333506A21988341A4500000'        
||'A756091CB594AB1C52A489AAE5188D5E408A543155E40FF7BB7F4F60C0209AACA47CFE553D45DBA5B5C413760A6718241A020E06695F9D7AC92CF369'        
||'1FA81914058DAE0D996E435FE9E9B707FD7FA7BBA471D7CB1E63E5FECFFAABD13CD2480026B56A60F1CF91FCBD7FC1D58B74E1D5D5FB619C69230F4E'        
||'A46804BBAC8C46961F8161EAFF001F7893CF834F38DF53FDFCDFE4EB3FFDAC3ABDBADB4B62B6E8283CF27F90A71E89E757E1E3A4DB0F55512470454F'        
||'9CDD93D44D3C90C11C30C3B82A419EA26908544D3625998000F36F7397386E0CFBA0863C930DBD29539685300799AFCB8F58BFCA1B5A9DBFEA9FF0CB'        
||'30CD07C12BA8AE4F901E607D94E8D37C71CF6DADC5DAFB5A9B1998C2E7604ACAA8EAA6C5E4E8327053B3636A3C69506864711EA20001EDFEF5EE3DE6'        
||'0DBF70B3DB5EE278DE2C02352B2939031A80AF53D7B4D7B6B2FB8BB659AD198BC9F31FEE3CC703ECFF00570EACAB39B776FE4E8248E7A4A769E22602'        
||'FA54DE36E5594A8BDF802F7B8B5AE7D81AD770BCB7941463439EB38EEF69B4BB8CF8883FD5E5D162DE3B3ABB17515535363929688CE89141F7BFBEB0'        
||'C97F0B30909B93C06E7823D8F76BDDE09D02CCF56A64D315F3E1D471BC72FDEDBBF89691D23AD29AB3F6E7FD55E815CFCB9CA498D3C3388F458909A2'        
||'78C2F05BD56E7FDEBD8BAC4594B1EB615AFE5D47FB9FEF3B69FC35EDA50E05453D6B4FDBD22EA2BF391B330AEA3F3B0E75C0637BFF00640F1F1CFE7D'        
||'9B2DBDA1006934FB6BD1035E6E152CAEBC4FA83F975161DDBBD28D99855D3CD186E6356D41C2F0545F9E7FC3DDDB6CDB24E2A41F5E9A5DE77984E086'        
||'F9547F87A55E3BB3B241D63AFA6109616D614F8C5C003903926DF5FF0079F65B36C1095D51357A3A839AA7D5A674D3FEA1FCBA7993B1541668FC1724'        
||'166857539207F6907239FF000F6946C86801AFE7D2DFEB0A9248A54FA1FF00553FD55EB8C5DBF474C3455C5372743141AB4DB9FF00023FD8FBD9E5A9'        
||'5F31D3AAB738C1190931A0F3F3FF00374B1C2761ED1ACD72AE4523690DC89E440AAF6FA11273FE07D965D6C9B8C745284D3D3A38B2E64DA652645900'        
||'AD3048A7FABF9F4B44CC6DFC8C01E9EB689896E0432C45B83C9D2878BF03D951B6BC85A8EADF983FE5E8EFEAAC2601A265A9CE0FF9BAED7298A765A6'        
||'69189E4AB24AC0DCDC1BB1278E2D63EF46DEE00D607F2E9C5BAB727C3D46A7E7D71ABA5A9252A286AA430A8BC91382415D37F4F00DFF001EFD1BC7F0'        
||'4AB9F23D7A559077C4F81E5FE51D778FACAB967B3C52E95B1D4AC5D4B922D75FADF8E0FBF4F0C48B8233E5D7AD6E6791AAC0FDBFEAFD9D5237F33D89'        
||'A6EFBD9AB503523F50EDD8F967D6CA77BE7C3AB7F4163C7E7EB7FC7B96BDBD6D1B04BA3FDFF27FD5A8BA3EB38239E44330AD59471F2A8C63EDE888CD'        
||'B076A3EA0D8C7372D7FF002CABFCFD7FB7ECE577ADCBFDF9FF00191FE6EA621CABB12376C3C3FA6FFE7E9019BDAF83C7650D351D218A15A7A7709F71'        
||'3C96670751BC8C4FB3CB1BDB99EDBC495AA6A4700387D9D02398AC2D2CF70F02D974AE953C49C9AD724F4B2DA0ED8AAF13501F0C9E0A88F51024F43A'        
||'D985A4BFFB7F69F74767B621BD4749F972C6D6E37754956A343F99F4F91EAC63F97476FE07AABE6C741EF3DFB5D4B45B5FFBCB91DB391CB56F829E8B'        
||'0B2EF3DBF57B5F1995ACA87B2C50C55555089652404462CC4004FB8C39FEC2EF75E4EBEB2B305A4D01C28E2DE1B2B9503CC90A683CCF434DC761B216'        
||'523DAC7491454658F0E2295F31D6F600DF91C83C823DE167402E807F93DBEF6FF5CF4176B6E5DC75304149FDCBCF61E8A9E665D794CD67B1B261F0D8'        
||'8A68988324951513469A56F65D4E7D2AC41EF2D58DC6E3BF5ADB5B824F88AC4FF0AA10CCC7D00009FE5C7A72152D2A81EA3AA3C7C35451F58749E4C4'        
||'0451CFD6D80A39EA638218A9CE4D6135322EA87F549223EB66717241B924701CFBC1585D7F5920DE284C0E8D0D7242BA3B38524E06A56AA8F3D269C3'        
||'ACB2FBA27306DAF69CCBCA65D56F22DC1AEC26033C12A245AC0E2DE1BC7A5CF96B5AF11D277FE47EE00EB31BA52E27ABF7CEFAA35AEDB1825AFA282A'        
||'E4A492B2A3218DC740B388D5DD11B212C65C28235140403C1E78F73FFB1F633A1BFDCE4AA41208E253A588665259B80C85A804FAB53D7AC05FBEDF30'        
||'ED8EDB0F2B44E8D79034F7522EA50D1C7222C5106A9C194ABB01E6A81B811D2957E35767B50564D262A8A4C82A68A2A28770E0E38DA571C4D2D4BCC4'        
||'155FADAC2FEF252CC72FC722FD64ACC389A248294F21DB5EB00277DC3416B754AF902E86BF3E38F5F9755EFD9BF00BE64EF4CBD53CF835CFD07DCCED'        
||'46951BF36941411D397B451D3D0BD6A22800D85E3048E48B9F72AED7CE9C8FB5269B3062269522190B13EA58AD4FEDF9740ADC760E62DD0FF8CB8233'        
||'41E2A53F2A30E81F1FCACFE5C480DBAEF0B18FA5DBB0F658B13F41615BECF07BABCAABFE8EFF00F3864FFA07A2593DBEDDA5FC284FFCD45FF3F585BF'        
||'94FF00CB4A8D45B63EDC85AFA3D7D8FB3495FC06F4D5916F6E8F77795A2A52590FFCD893FE81E92B7B63BAB8CAC7FF003997FCFD751FF28CF962FABC'        
||'BB636A861CA86EC0DA66EA7FA9151EDEFF005E5E540B8796BE9E0C9FE6E9AFF5A9DD18558257FE6AA63F9E0F5CE5FE503F2AFC49226DAD9A6F7F2093'        
||'B0B6AA00D7BDC7EFDAD6FAF3EEDFEBD3CB029DF357FE68C9FE6E9A3ED35FB3685F0C1FF9A887A39FF0DBF95C76E758F6BEC8ED1EC9CCEC9DB94BB0F7'        
||'4E2B70D36DFA1CD45B9B2D9C7C7CE25F1ACB8B1F6B4B17E7C924ECE48B088DEFEC0DCEDEEAED5BBED53ED7B6C72C8674642E54A2AD411C1BB98FCA94'        
||'F9F42CE57F6FE7DAAEE3BC9A48D3C3607486D4C686BE42807E75C70EAE507C7BEA6837FEE5EDAA8D9DB7F37D879ECA49939B3F9454CB64D7C34EB474'        
||'34D848F2FF00E4D4368638D75C411B55D9A4E7DC2BFD60DD4EDF16D4923A5BA2E9D03B57D496D26AD9AF1A8F975269DB2D0DCBDE4811E42C4D4E5BD0'        
||'05AE171F67DBD059DC781EFF00DF78EA8C06D9D8FD7789C3B4BAE0ADDC7BB6832D9E2E84159E9A75510D139B0178F5B58E9D447B6A29E18731B3B311'        
||'C42D17FDE789FCCF4F18964A78C10806B42D56E1EBC2BF671E911D71D63F23367346C2BB67E1EA88D7571D0EEB35787AB201BF9A825A770C7F01AD70'        
||'4FA4A8F6A26BA8658C070CE78653FC06B51F657A4DF4C15BFC5D8201C0EAC9FF004C29427F9FA7422770616BB33D57BD731BFB676C2ACDC983DAF5D5'        
||'B8ADCF8DA85A9CBE3ABA9D55A3A9A49AA2956752A40B46B53A2D70508F75DB022DEC4212F1EA71A97F09FB687FC2B5AF9F477B41964DE6D8CC51F4C8'        
||'B43524FE55151F6034A797557AB35C0996C15AC2451C08E5239B7FB4B7D57FA723F1EC70410749E3FE1EA5FA57E63A991D491F53C1FF0003FD7DD0A8'        
||'EB5A4F97538D55E384827F43ADF9FC4A4FFC4FB6F40A9EB6350E3D7FFFD6F9FF00FBF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF'        
||'75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BA9349595541554B5D435351455B45510D5D156524D2D3555255D34826A7AAA6A884ABC7246'        
||'E15D1D482A4020823DFBAF75926C8E42A3212E5A7AEAC9F293D649919B25354CD2E426C84B3FDCCB5D2D6C8C64699A426469598B162589BF3EFDD7BA'        
||'F6432590CB5654E47295B5792C8564865ABAFAFA99EB2B6AA5234996A2AAA59A476B5812CC7FC7DFBAF750EFFF0013FEF3EFDD7BAF5FFDF7FBEFF5FD'        
||'FBAF75D7BF75EEA6D7E4B2195AB96BF295D5992AE9F479EB6BEAA7ACAB9BC510823F2D4D4333B6945555D4C6C00038007BF75EEA1DFF00D6FF006DEF'        
||'DD7BAF5FFDE7DFBAF74767A23F9807C83F8CFF001EBBD7E39F483F5CEC2C37C90C6E6B6AF6E76A62FAD36A3FC80DC3D67B96831F8FDC9D3949DCF550'        
||'3E668769640636196B30B4534514B2493BB1BCCF7F75EE8935FDFBAF753DB2D957C647847C9E41B0D0D63E461C435654B6322C84917824AE8E80B789'        
||'6664F419426A2BC136F7EEBDD7A3CBE561C6D4E1E1C9E422C456544357578A8EB6A531D55574E34C1555342AC2279105C23B292BF823DFBAF74DFEFD'        
||'D7BA9D4F94C95251E431D4B90AEA6C7E5969932B434F57510D1E4D28A715546B90A68D824C219409221229D0C352D8F3EFDD7BAE74B97CAD0D264282'        
||'8B2790A3A0CBC704595A1A5ACA9A7A3CA454B2F9A9A2C8D342CA932C6FEB459010A79163EFDD7BA6FBFBF75EEA755653255D4F41475B90ADABA4C5C3'        
||'253E3296AAAE7A8A6C753CD31A89A0A082662B0A3C8CCECB1850589622FCFBF75EEA07BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF'        
||'7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAD897FE1333ABFD9F1ED70BF56F891BF56FF9507B8361DC8FF61C7B16F2659417BBC7F8'        
||'C54AC49E205AE1995D34861E61588703F8954F953A5967B85C6DE2636B40D346D11623B955E9ACA1FC2CCA0C64FF0003B8F3A8DEF70D8C32B29D24DC'        
||'8B7FB7E49F72F4F2D3A42AB4E867C16DBBC46674223442EC6DCB69173C7FC47B0EEE1B9476B0BDC49F0C6A58D3D00AE3EDA63A5D676525EDCC76917C'        
||'72B2A0AF0058D2A7E42B53D08F8EDB090D3C68622D2BDE493483CC921D6F61FD0136FF0061EC8AC1A5B7B4512FF68F5793FD3C84B37EC2683D001D19'        
||'6F17305DEE0ED09FD28C2C510F48A25089F990BA89F3249EA0E53030451B1605D80FF3719FC8FF00572FFC42DFD9A4372CC7A2974A67CBA638E99FED'        
||'AA1D4058E5911CAA8B5DD63D12072396371F93EED33F7AFC87F97A76D90686FB7FC9D0739BA730E7714C07A2A2932B4D7FE8EA22AA4FFA10FB0F5E48'        
||'D17376D8FE52C37719FB408E41FF001D3D485B3A25CFB73BE43F8EDEE76F9C7D84CF037FD5C1D3264201A5AE3E80FD7FD6F6378DBA8FFA43D6A0526F'        
||'F5B1FF00783ED6A1EB7C73D262A4F3FEF3FF0011EDF5EADD33391E55E6DC8E7FD6FA1F6EB7F667A713FB407A7FAC8C6529A364FF00818294CF0A8FAD'        
||'4083D3574E2DF5616F22FE4DC8F71B5D7FC8477FFDECA29B76E522A5D7F0C172D4586E7E492E229BC836873E7D0FED65FEB3EC636A94D6FB6D466B62'        
||'78CD6C2AD25BFCDA2CCB1712575A0E03A44CBC8FF602DCFF0051EE43E183D037E2351C3A812D96FCFD0585CFD6FF00E1EDC5EBC467A89A803FEC3FDE'        
||'FDDBCBABAF953A08BB06BBB60EE2C5E33AC3BAB79F501A6D83D83BD72D4DB529F6AC906F4AADA793C152617159C9771E3EBA54862192ABD2289E0918'        
||'4AE4C97542B516B05DCD1C738A82D41DA8C33A467529A0FF004B4FF07408E73BA5B56B52F1ABAB16075348A40C574E865A93FD2A8C7DBD04F47D8BF3'        
||'126A4972337C8BDF58FC6D397F26472726DFC751BBA58986905452896A1F9E23A78E46FF000F670DCBBB52B787A5598FE154427F3A2500F9923A0BFD'        
||'55A3656034FF009AB28FF9FF00A05762FC83F9C9BBF76F7F4357F2B3716336CF547646CCD9782A1A0936453672AE8373F556237CA575750E4A192BAA'        
||'4B54D656219E089628951637B35AE5D072F594FB95CD93281E132E91E1253498D18EA7F0C8A966340482470069D2B926DBE2B386E56166F14356934B'        
||'821D970049C28B5AF4C1F237E5AFCCBE9CE8AEC3ED1C07C96DEB96CB6CAC7E2324B06462C14D11A4A9DCF4389CA4E69A3A645768A96A2591158E9D6A'        
||'A48205BDB9BBF2D59EDFB5CD7B1A2931807E08C8CB28269A07004F5EB29ECEEEF63B468997C42403E2CBC429207C7C2A07561180D99F263737F07972'        
||'1F3C7BF31905751D2554AD8DDB5D46D032D640932A32D661A4361A8D8871FE37F65775CBF143119206524791862FFA07A0B58F3643717021B9B40AA4'        
||'D0B09A734FDAFD6B79F32B686F6D89F3A7E54EDBDE7D91B87B7EB69BFD0AD76337CEE7A6C1D0E5EB3099DEB91964A3ACC7EDD8A9E8E0FB7AC96B5112'        
||'1810B2DA492F248CC426E275B8904F4D544A500000A5060507530D9CB692EDF0B5913E18D4054962287854D4F9FAFF002E82CEB88826EDA6600DDB05'        
||'D90A0DBE80756E6C5BFD8FBDA7C43FD5E47A51C463FD5C3A307B20D4275A76824418A4B8BEAF8EA482085893B229A405FF00D77551FEBFB71BFB44FC'        
||'FF00C1D51B883F6F414F7753EBEAFE81FF006BCCF7EB3722D719EDB71AFF00BD7BACBC47E7FE4EBDEBF97F97A2D990C766AB61C36376F644E1F3D91D'        
||'E9D7B4388CBAC7492BE2F235FBF71B4B4D92A78EBE29A067819C4A82685D3501A9585C1D04790AC719D2CCCA01F42580AFE5D50CE96CAD72C2A23567'        
||'23D74A93FE4EB61CEEC7EF8E9EE9BEC1EC5DA7F26BB519BAF7646E1DC38DC3E4713D4D3622A860B172545163E6A5A6DB90982225141F1C8348E00B7B'        
||'125CDBC302C87C4766404E5500247E5D016D37C9F72B88F5D95B012B004F792351C9F8A9E7D030FD95F32C5263E5ADF9247CD5B8BC6D7CA28B67F5EE'        
||'5A855EBA8A3AC31D3D7D3D3489205D7A6E18FD2C6C411ECFE1D83C48966590AEA00E5403915F315E845FEEA2462BF468749A713E5D04594EF9F994BD'        
||'B789EB587BFE9A2A3C8757E6FB0A5CBC9D71B2E5AF3558CDE149B623A142D49E13094A82ECA135860096D240F699B6A946E0B60B2E3C232574A93F18'        
||'51E5C0D7D3A73E9367688CE6CD6BA82D2A47956BFCBA5753EF7F9853D5C0B27C8B82A91EA29C3447AD361C65D5A40ACA5928C117048BDFDABFDC5702'        
||'959CD3FD2AFF00D03D50C3B2A293F4280FA924F406F747C86F908FF306BBE1A8ECF4CA6C9ABE89CBF636F1CBD76CBD998FAFCB504F40ED90DB304F45'        
||'4A5A1867A413D3B4D115983C88F1C8853903EEB712CE2EF6D898E91677129242D7B10F6E07039E94F2EC16824B6BDBBB584486E6244F0C30A1D6B46E'        
||'E27209A8FB3A0A53AEB6B2492FF90D403ACFFCA7D4F1663F5F57BC72321AF5910228BA1836953ECDA533189E45B54D23B12D5AD6F19BDF91F8F6CB16'        
||'AF4BA09201C0FA7AF46128F76EC5A782A22ADC93243297F28FB5C8B7ECB2689082919FC5FDB9697375637715F5A1D32C0EB221F46460CBFCC0E8C1EE'        
||'2CE485E198F6B8208A1C82287F9748126927135462AABF896312792186BD20A8855C258DA48EA15595802355C5AFF427DE76F28FB8DCB9CD96292A5C'        
||'476F75A478B048EA8EADF8B4EA203A13F0B2D7140C01C750E5FED175652B04532455ED7504823CAB8A834E20FE55EB8C5263D268864EBE3C751175FB'        
||'8AA68E69CC515C06290D3ABB963F45E2D7FA903DFB9BFDC7E5DE52B0799A78EE2EF49F0A08DD5D99FF000EBD24844072CCD4C61413D5B6CD96EB719D'        
||'5194C71D7B9D81000F3A7A9F403CF8D3A6BC57C97DB3D19F28E8F73531ABC9F5BEE4EBDC16D0DE0B474F506BA8E9E1AF9AAA8B3347495211A5928A50'        
||'AED101792279550EB2A3DADF62365BBE71F67AE259581BC3B9DDCEAC7019DD622EA4F907A9A1F2205715EB1DBDFADFADF97BDDF89854DB36DD6B1300'        
||'3202B4BA580F32BE63CC13E7D5C16D8ED2EB4DE988833DB537FECFCE626A2113AD5D1EE1C6031215D4455D34F224D03AFD1D278D194F0403EDFBBDAB'        
||'73B098C1796F246E0D2851BF91A50FDA09E8AECEFECF708C4B632ACA08AF6904FE6388FCC0E890FCBBF91BB5B2BB5EAFAAF61E569B3F5196A8A6FEF6'        
||'66F1B28A8C4D163E8AA16A970D475A9E89E69A544F33444A22295D459ECA27E5BD92E23B81B85DA940B5D0A789271A88F20070AE49E8FACED5C378D2'        
||'8A5380FF002F459E1FF330FF00CB18BFEB58B7B4AFF19FB4FF0087AC9F841F053FD2AFF80749ACEFFC0987FE58FF00D743ECE76CFEC5BEDFF2751FF3'        
||'97FB9F0D7FDF7FF3F1EB86DFFF008BDE33FEA297FE846F6AEE7FB06FB3A0AC3FDAAFDBD0CFEC8FA36E866DBFFF00164C67FD42AFFD0C7D91DC7F6CDF'        
||'6F4AD3E01D61DC17FB683FE5B9FF00AD67DB69D19EDDFDAB7D9FE5E91F37F999BFE58CBFF5ACFB7169A87DA3FC3D1B3FC07EC3D16EFAFF00BE1EC67D'        
||'003AF7BF75EEBB10C93B2C5155B5048CCBE3AC452ED03AB6B570AACA7F163637E7804D8155654FA95AFCFF00C07A8D3DDF23FD6EB71F2FF71FFED2A1'        
||'E84B9D3B7B218F8AB12A713BEA9B1F494F45535724984CFD7C48B1914B1CF8BCCC6993F1AAAFF661645B5B50FA934FD10C6104AD73C0807D48395AFE'        
||'75EB08119E16133C6A09ED1F8B1E5C28DA7E74031C7A414B9DDC8BA639E35C34F24F2C13D353E368696988916FACC10A68E6D6B37FBC7B71838A2938'        
||'A74A22481D7C44401B571AD687D7E79F5FB2BD083D61DC7D87D3F1EE8FE01060ABB09BC7015DB5B73E13358FA934594C3574B1CF50B1458F9606A6AB'        
||'57891E9AB606596222E8C2E414377636F7AD134BA8344E1D0A90086151E60D54824152287CF874F2497102CBE105AC88C95D3915A51B04654804647A'        
||'5454F4B4CD773E7F726360C464B74EF7C5EDDA65A5A61B5F7CE417B176F500A52248171B9DCB407234F0DCEA48EF2FA783AED7378EDA0818CDA17C46'        
||'A92CAA1092706A01D24FCF1F974DC31DC8A043D89419209A0FB40615F30A28471E812CB186BB4AD2CD479248AA988F03D254C71BDBD669F5E8658DAD'        
||'C5E34617FA5C1B295535A9E14FB3ADB4B149D848A834F23C3D3FCB5A74B5A7EA8CAE4E2C7566D4C46EEA0AEA9C617AD490E0E1A7FE21029D53E172F8'        
||'EAE77962923D2C60AA892456042338217DD4C96E4307755D27CC9CFDA08C67CC123CC81D5B49B763E0C6CFAFD00016838EA0486AF9834A7004F0E836'        
||'9721DA9B4F7025553D76469B2542A601255D40A2C9240495920735A6398A30056488DD587D41171EEEC91B2E96A107CA951FE6FB0FECE93299A31545'        
||'2C4F020D07A7AE0FE59E84293B0371C180AA869E01B4EBB2349E0CB55D3E2E8A9AAB274CC4B3C2B99C5431D4C314A749743232B00036A160372C6B20'        
||'5F1496D26A01AD01F5A134C7558A0103F890AE8A0A37A1F3A600E3E75AF0A70E810A5C8E697206A20C8E4E21E4F224D49949E250DC68B31641F8E4DB'        
||'DD19031A1E9D4B86451A327D01A53FC1D0D90656AB378AA8A7DC55B82CEC9574B22AFF0016C4EDE6CD4269D55A9DA0CE53B9A866B8B82CD722EA6FEF'        
||'680C4A284D01F534E19C7A75E1044F33395A171422B9E208C024123D48AD389E86EF875B3F65EE1CF6FBA0DD797DE3B7A7306163C3D0EC6EBFCD6F1C'        
||'C66116AE78EAB5AE36B290D3C6A0AB2BAEA275926C12E429CDB79B8D8C10CB65142E096D465996155C0A71560C4E7D3879D7A1CF23416D3CF7293BBA'        
||'D02050885D8E581AF70A7E5F3EAC167E8BF8E78257A8DFBDCBBEFACE929A8A3AA8E977AEDA8F0797AB85E558B447B763CF54E4D186B0F69E8E346537'        
||'05EE01022F327365C636EB086EC934AC5216418FF7E189633C29DAC4D7881D48FF00BAB6843FE317124541C190063FED43961F981F9F4DD36DBF8990'        
||'57494B43BE7B877052D0A98864F05B0309FC26568E3D7648B3DB82098B456026492052857F4A7D7DBCB7BCF2D18692DAD632DF85E77D43FDE21233E4'        
||'4135AF13D37F4FCBE1B4F8D3B81E6238C03F66A92BFB40EA7ED3EBFF008E9BE3796D6DB3B72ABBD2AF379CDCB87A1C6495DB33AEE86852BDEA5452CD'        
||'24E72D572A471B28632242EC50315D763ED9BEDD79AEC36F9AEEF12CD638E362D496763A403500786A0923C8902A4569D3D6B69B44F751C16E672CEC'        
||'00AAC40549C706634FCBABA7F90783C7617AE3BF377EE1AD38A9FB13FB8BB6A5969298E66A28A93198DA3C4D4D243048F48267A80B3EBBC8A152EE49'        
||'B11EF1E794EE66B9DE36BDBED17C4169E348013A012C5D812406A05EDF23538EA59E628A3876CBFBCBA3A0CFE1478EE2000A08CE9AD686B9E1D559E2'        
||'687ADE311ACD5F9AAA8A48248A9A9AA7ADB661FD16697EE3EF72D2EA250B3347A880BEB5FE9EE709E4DDE874A229AD491712FF002D310F3F3FC8F514'        
||'471D80C6A638FF007D27F95FA58D3C3D6D03475B1FF1CA732C8FA95767EC3A5F14D1C6242CA63A9606448C175617BC7C0BDB92F67DE1AB11D069FF00'        
||'0D9CE0FF00B5E04E3EDE9404DBC10E4B8AFF004107FCFDD2D2AF19B370FB7E6DF5B8371653696D04A8868E4DC19CA1DBD87C6543D5C889F67868A92A'        
||'D24AE766292451D247210CD6254DC7B2D5B8BF9AE46DD044934F4AE84676614AE5C94A279825C8AF957A5A2DAD162FAA76648C9A6A21403C3E101EAD'        
||'EBDB5A799E859E8AD9BB3B7276675D6636CEFBDC3BC2820CE6524A94C86DECBEDF44143B64659249A8B2558C4C3343341A1D629175A9B91C1F643CCF'        
||'B96E367B2DDC37B6A96CC51694915C9D5269A02A832086A8A8C70E8EF61DBACEE376B792DA769975B56A8CA3B535641738A119A115E8CFFCA6A5D951'        
||'ECFD8837B60F71E6E9A9F2F5D4188A1DA3B80EDEAAA7D54E29CD454CAD149AE28E25896D61A59AE2E3D82B91A4DCCEE37676C96388945676963F101C'        
||'D6805450924FE43A15F37258AD8DB0BF8DA40198208DF4118A54E0D400063D7A2393D5F44C74F0483AE7B71D1EC9185EC974626B434D621296C74BAB'        
||'2F3F4FF1F7262AF34972BF596A0FFCF3FA7FB6F4E8055E5F0A0FD24FFF0039FD7FDA7CBA900F4408'                                                
;                                                                                                                                   
BRANDINGBackgroundIMAGE_7 raw (32676) :=                                                                                            
'9D875CF6A30D0926A6ECBA8660B5548CCEDCD3704042BFEBFD0F3EEA3FAD05856F6D7D3FDC61E47FD37CFADEAD83CAD67FF9CE7CC7FA4EBD27FA0734'          
||'6E475D76DB09622E443DA55224B1C7FDE68B2C00D82FF87D79FC7BF7FC89F5D0DE5A8A7FCBB0FE2A7AFAFF002EBC1B6002A2D67A9FF9783E95FE1E87'        
||'FF0089547D5B55BFB3D5DB0F6B7606DECB51ECEABC6544DBB77A4DB871AD45592527EC4141E15B4CB22C27CBAAE14381F5F612E7E937B8F6989374B8'        
||'8268DA60C0450F86DA806C96A9C52B8A7A7425E4F3B5CBB9486C2196371130264975AD0D380D2335A66BEBD203E64E3BAC692B7686537EEE7DDFB60E'        
||'1F6696CBD66D4DB52E671B353BCA33D93CEE5258B338C929E2F2CE636D34F512598B5B8B7B35F6F6E77A68AE21DB208A6F126ED1249A1853B1514785'        
||'206345AFC4A31D20E6FB7DB3C581EEE478CAC62BA1350627BC9275A69C9A7063E7D14ACE75B6D2A1A1C55464DF7D57E333D8E932F84CE63EB7EFF079'        
||'FC4D4C4265AEC4E4E8F38F1CBE5BA208B509413EB55045C7B6DBD5F4CEEB07801A33A5D08A3230C5194C408A64D781F23D03A5B1B68555A512697155'        
||'2285587CBBCFECE23CC749AA7DA9B0E5829E96B22DEBF7116A570D4544E667D0A8CD0CB51936BA99078D783C2923F3ED6BDE6E952E861A1F9B607CC0'        
||'8C7967F3E92882CE9A595C1FB169FCCFAF506B701B06952632D26F191974485DB05B58A5486366313CCD284D60370CDC2297E2F7F778EE7737A51A2F'        
||'F7B971F6D295A7F84D3AD3C766A2BA5FF647FE63FEACF471BE14E0F6FE620EF0D8B8EFE2D454FBAB6A52688F254384A50278E0ACC65364E857151432'        
||'99225AC28F13DC78D92C4166F71FFB8D71756EDB6EE92E96304A72A5CE2A8C54EA245094AD47983E83A18F26C705C7D6D82540963A8A85E235004690'        
||'386AA11E87EDE925F39E93AE713D99B637DF6374CB761E4F78ED2A3A18B3B4BBDDB68D26329F6A33353639A866C757890DEA1EA7CC2651A4E964F4EA'        
||'659EDAB6ED3ECF36DBB46E02D52DE524A1844A58C94AB6AD694F874D2878541CD0539C5AD23DC23BBBFB6F18CB18A3090C6004F2A696AF1AD6A38D29'        
||'E7D13CA9EC6F8ED89A3FBEAFF8C75A6969442A668BB6E0A9A9689C88BC3143360E30F2C92328557750CC555996F6F63E5DAB9B257D09BC2D4D71F4B4'        
||'15FCA634007A038AF1E829F5BB228EEB16A7FCD73FF4074F90F6CFC18CD54A61F6F75A470EFE85AA6AF29B6F7AF608DAE8D8E86A063457E2F72E2B13'        
||'95A2A85FBB57A3956A4D304655D258300117EEAF70E1626E2F8340476B450090D7CC346D246E2828415D75AF4A5AEB968C4ACB68DAEB90D3301F2A30'        
||'56049C82085E82DEE8A1CC64136D5074BF5FED0D8F9583310EF88F686E3EDFC567B6B64E8B034D90A18BB060CEE0B1AEB3494D5551488B8E50B378A2'        
||'95D0973121853DCA6BBFDED6E979746EE4586953034053BDBB4AB1249F3A8C797525F239B66DBE67B687C14327012896BD8B9A818C6287EDE8A16FAE'        
||'C3F8DFB3FB1F6ED37CA0D99D4996EC7A9D9516F3DD9DBFD89D6D3254E4531B472536070582DCAB4F2AC14F40B4759591ED78E696A1D27A79AA96499E'        
||'69678E3B41CF436EE231C3AEBB2371F756D8A6ADF913D59D7FB5BB0BA2705B5972DB1BA630F93C855F69D4BE6E9DF3126F9DD9B63EC51254A9A28695'        
||'86271ED535B8FC7EB91E9D19EA120F54F11C3AD81E5E7D0829D3BD8F9FC760F79EC4EE4DC9B2F717764A33BF2B70DBE9AB5F736D9C941828B035D55D'        
||'315D4D149061E17A7A6C6E32871D12DA8294C12AC82AA398FBD8AD7AD1C64F41076CED1F8E9D6D04B27C6BC6FF00A11A7EB419A5F901DCB16E6ABC7C'        
||'704D41466B33745BA729936A89F29B8BF792BF299F82A7EEA27D342CE929D306B03875BA93C7A0532DBAFE2DF5A756D57716D1C067BAC3039EA3A48F'        
||'73D7EDBA8CF41BC7B523C9E4625C57636EBDA3B6089627CDACBE28B2F5995A77A9A19E26AD646D023F638F5BC934F4E975D7DD2FB33ADB0936E3C56E'        
||'AC3F41F543EDBAADC14DB1856E371994DBDB932A17F8B641FB3B3D909A929F1E5D64AB869A897EE1EAA7A848EB1E18C29F529F21D54E7E67A10FE3CE'        
||'C5CDED4D87BFB3BF1870945BF3696F8CFE6F78A769775EF8CF6D9873198AED189C8D16EDDCFB931F53B977052522C52CF8CC80A5581688F896B47324'        
||'BEFB3ADD7356E85AF8BDBCBBAE6EA1AACDF6DEC7920ED2C5E4B74C391A7DAD59B6F66ECFCED6E2B27518AA2C96CEC8EE9991A9B0A6052D4998AC6592'        
||'A6269EA64A86F2BB0F29C75E619C745CFB93E467456EADAF801F25176DE3F79ED1CB6577567FA3B0788DDBDA5BEBAEE9F19949705578CEC63B423AC8'        
||'6A237586315C2A0D263A56440AAF0C31BC9E2C3CFAF696F2E848A7EA8EA4F9398AEABF975D6DB1B05D25252E264C2AF63EFCDB3B2F63D3EE2EB0A9A3'        
||'108AC6DAB83CA9A4829E19A290E28E61D6E25330942CE633EC1C8C75E3518E8CB6D2DCBD73B8B135793D85D838DEC0DBA33593A34DC3455D41518B93'        
||'234421A4C9D063A4A0A6A4A57829658FC6AF4B1B44CC5F4B7055671F6F2A797AAB9FD693FC0BD20B907C4FCBA57D2D7E26340895F47185165027802A'        
||'2AAE8D2A2F6B01F4F63760FC69D3401269D3F4596C323C0C6BA8CB699107F94C37E006205CFB68A3FA1EAD46FF0050EA6D56770EB8FAF1157D17963A'        
||'690283510A86262FAABB1B1FF123FD6FC7BA2C6FA8541A756D2C7B8F44B772EE7C5673BA366EC9CC6E5A0A1D9D9BD99BCF359BADFBFA4C63516770B9'        
||'CC3D26197F8C4EAE881E1ABAA3E0E0BE9D40FA0FB513C93DBC7AA25F2F4AF423D9EC21B90E66A8D34A7CF8D7A1E76B7C76F8879BAF8B33B87B053219'        
||'495A5A4C5534BDB7B728F1F3C35B4ED44527A2924592A03B388B4A91707800FB259B77DD96402300529F80D78F4697364896ED1A44D420EA273C07FA'        
||'BCBA1022F87FF09368EE19A967ECDC26CBAFCA80B578293BF36CD1E443C712D3B5453E3339319D125444656D24358902DED2DC732EFB3485DC6A7207'        
||'188E71E83A476167656D6620B68EB1AB352A7193EB5FF8AE88677C7C59F87BF1C3B2932FD0FD8D26E5CBF707506F8C7765D467BBAB6EEFB8317498BD'        
||'F383A2C2D5D3C0A63831D1442BEAEA2452B7A82A402580F663B0EE3BB3CD35EDDA378B108CA02856A149380788040AFF003E94EF538DFA1820DD6401'        
||'35A40B46A058BC375A2E4807006AF226ADD558F506C8C7D5E570BD9FD8F93837465F346AD365EDAC3541ABA8DD195A09C53A6DBDBD063CC8D052D04A'        
||'629B273A279A4023861064995A21DDFEE573B94BE2330A9FCD635E3403CCF913E67E5D1AEE37DB0FB77B09E55E4FEEB89005B9B95C925877468E2BDC'        
||'6BDCEA4851804B1A8E3DB0BBEB13B85F6936648DD795D1BBF7A6FC5C8345B736DD5D0E40C3351ED88A9247869A5A12A9AE3A5D46291C2334AE0790C5'        
||'2F36F5DB3E9E38034873A9D437901AF2056B90AB80BC4FCC3BEDCF24DE736EFEFB9DDDD88ED2D00061898A300C4F6B01F16AA65CD549AD33C022C46D'        
||'CC06E3439BD698CEAED9F5C331B8B7FE7D4D466F7B67219FEE67FB292A8BCCE92CCA488B90CA4EBD5A98FBB6D9B2413C7F537C816000900E3501F889'        
||'E21479B716385C743AF733DC0DBF6888726729C0971B8CC04602A2B7875C01906AFF003F23F3E96392DE0D5DD914DF247774198A2A48A5AEC7F56EC8'        
||'15B575593DE99D964923199C963AB1E45851E69D4D4D4471A48EAA2152AC09A627BDB2B0BBBB58B6E874E15050519CD7E22A31A8F0507217B9CF46BC'        
||'A3C96BCB9CB526E1CFD3896523C47F10D52D90F144FE91A7C2300F9672D9B5760D2F4E57D166A9F68E0E0F901BD991F64ED8C4E2B1C94FB131F91522'        
||'2AFA98994A412C6268DD24E1C7054ADA33EC4B1EDFB46C96064B88A39259011C2A09535D20FF000211591FF11ED15AD3A8AA59B78F77F9B976FE5653'        
||'B7ECF666B2488341294A1662B4259C7C2B920671C7ACD41B7E3C88CBEC4C5E5E37C3D3D42663BAF7EA98D62C9D7538FB89B6F63EBE33CC2BAAC5A221'        
||'0D830F4AC3A58DB2C24DDAE4DDDDE6304EA3C351A7C206298F8BF813B7893D49FEE2F3A6D5ED8F2DC3CABCB09FE3F2A787046B9640D8F15FFA6C7857'        
||'24F4D999DCD86AB106EEC8F8F0FD75B5215C775E616A19204C855026920CDC914800D059D62A78D802C4F03D6745B7BDD639CE94602DA1F3A503118A'        
||'81E9F8507E7E63A77DA0F6F5791F69979BF9A883B8DD03248CD931AB50E8FF004CDE74FF004BD5D27C49EE2EA5EA4E83F88BD81BDDD27C8544DF2EF7'        
||'06468AA465D25ADDC1BDE2DB783D83141908D1152E8D9048AAA924FD9F14C55B5AFB8977182FB74BE9992A91BF8214E2800D4587FC76A08CD7D3A47C'        
||'C1B8C5BAEE7B84EA68645B6AA8A6AD2BAF07EC0071E15E9435D574FBBBB136CEE1DF1BC37465A5DC9904A1876E653030D661C6DBC76DCCA9DBDB7F1B'        
||'1D7527DBA410BA32D1AD6D434A65BD448EF2B166116D2D26DB2E9DAB48968C41D5A724773B3277D156ACC178A8D231D634FBF304175ED8EE72EE0AD1'        
||'4486D6A5002D9BCB70AAAAE421691A8953805813414E8C2E031DD73B6723869A1EBEDA34D38C9E3AA21972D4B8D81F194A95E2A24AE558120D2E5544'        
||'91EA636D4AC54E9E5EDCAE399B72B49C49BA4FA046EA4445A92314D3A2A4B54674B500AD185738C12DA072CD8EE36A9FBA2296579A16517043322893'        
||'5EB08341D5DBAE3A9254156D38CA77E727CCAC67C40EBAAACD374AECDCC7667693D4CDD61BB774EDCA4AB8EBF6E50D63D1D5E722D8D3430BB250C660'        
||'686A6AA168E5919B4EB544B82F93F95CEF71FEF1B9DE2E86DDB5694B8B4B699B50B8650FE178CAE78E4C8B136A4402A012475953B2DBED3BAEE326D5'        
||'36C36C370DC03DC2DEDC5A29FF00115731C532C12A280CC4148CCD1F73AB152EAAAC35A8EE3FE651F207BDB078EEAFDC55B9FCE50C1F77579C8B0B2C'        
||'1B7A1DF390ABC84B574D5DBF711B7A35ADC9351C4C9494714F571C490468B1C518D579076FDD363DA2F5E4E5CDA0999C00B231722252285214CB2292'        
||'2ADA591E6624B9CF525A726A4320692E488C508D6AACE18000C95348D9CD2A1A4493C2140980003B7F1C7E24FC06ED7DA385DC9DEDFCCF317B26AEBA'        
||'92096BBACE3E94DC9B0A2DBD208D25AFDBFF00C533D21A19A3A7958C6CF0ACA9291E46B93C6B9837DF76EC5DA1B7E5593778083A59770B6822D3C541'        
||'B58D567514CB4729C9E35E3D2032725DB6B857733B4DE0704CB3EDB7BB8BC8694322CB1B7852311455911FF4FE10A387568DD6BBA7F91FFC5E8AAB0B'        
||'B43E4CF5DEE4C89A43FEFE77EABDC7D97BC68A611088B546E18A91E9C5296D463A4A7A38A22080C5C8D4634BBDC3DFFDE9142ED30EDA9161A18B71B7'        
||'810A711FA713348AD4A0321727D2831D22DDF96F907759C8DE375DC7784986B84B6CB3E889C8A1CCC52290120B2C6F19230493407A5FCFF3CFF93649'        
||'8E78E0EC9C86F298CC5F2388DB5F1AB71CB539E984CA824A66FE1AA29180D44F8E78C1B13A755812CB5B6F7F26BA260B9B7840C2C877396458850E18'        
||'46AE64CE328C7E74C82EBEE40F65EC76F4BABDDB2F6535AB47F43656F24A6A0551A69A258706B412A0A57B7550140F607CE3FE513BB7AEF71F5555FF'        
||'00A6ADA785DC940F062F2980E839308FB76B7EE164C665A931DE3413C94D2246CEF247791752391AC9028D9362F7F6CB7F8398BEBAC2F2488832472C'        
||'FB8482642086466FA53A438274E9C23508074F417B9B0F6623DAE4DB368D9B74B09687C09A23B12989C7C2DA7F797EA01401C31AC8BA8135351587F0'        
||'EFE6BF5A742767EE7EBCECDDB998DF7F15F7065370EE4C85754ED286AF74D065F1F49E2C166B616224324D4B559330D2C75141255468A8CFE4924283'        
||'5CC3EE0EC9BDF30278DCB1E3586EB1C60C7705A6FA660D4F12DE51144CF220AB98D8C219586A0006344B6FB3EC33ED967B8F3725BEE280C715C5A406'        
||'DDAE34866D13C6D34E8B0CB11A7885676578898C890AAF56819BFE675FCACE0ACABC78D87F27B3751108B4E6B03F1EE0A7C44F45574C9534D4D05357'        
||'D7C019D56454925F0A9D618292A35183EDF93BEF0318568EE6DC2D0120B5F3F7799122DA9073F84310381C83D0B0727FB037A1DA7B67575775075EDE'        
||'9554764AF842E081C3E360AF4E216B4EA2E43F9937F2A7C83B0ABDBFF2E71EFA63005074997789428531948E692ED15B4B017FF02473ED58E5EFBC0E'        
||'D828B36DF25726A2F8D0F983FA0335F4F3E89EEB90BD83DD18C8D61B946060325CED02AB80083F547B6991AA869C4038E86FDB1FCD13F961E376ED2E'        
||'2B1794F96B0C14F0B454B495BD05BB7EFCAA925595E3A6F092E7851AED7FE9EC097FEDF7BCFBAEE325FCE36BD721AB77DCA0AFA51A207F2E87FB44BE'        
||'CC6C9B445B459C97EB1C234AD6EF6B66A0F33A6E287E7F3F2E80DDE3F27FF945EFEA0A0C5EFBDAFF003A77CED1C664E5CDC3B14F52EEEC4ED9AFCCCD'        
||'5AD5CF90DC549867A26C9A21934414B592BD3C6351F11760E0669B77DE2E393EB2CC6CD14C422F8C16595D55142A051223A29C54B05258D2A6829D03'        
||'6C7967EEFBB36B86DEE374319F12A82FF6D8C9F198B4A04892AC8B52D4051D1900ED604E46DF8C5BC3F948C7D87B6AABE38F58F69ED7EF39EB6B93AF'        
||'729BDBADBB8F684F8913E3E4397A0CA65AB6A9F15342F422B02B643C8A599020122C4420DDA5FBC9EE8F1FF5E26864DA12BF51E19B6EFE3A58294598'        
||'7EA14ED4A0005694AF524726EDBF77ED96F87F552679775908FA6596E2598C7A63A10155DADF5840F5969ADAADA9DAB9B48937865256A88E111CA40F'        
||'505A808C6F723402473FD3FD8FB2F5B084509C7E5D4A2D7F2693E7D60ADC847B8A8D23AE7459E288B3C6EDA2658EDE9918B7279FA1E7DBB143F48F58'        
||'F81FD9D249A5FAC4A31029C6A6840E816DCD450413C890C91D4451C62799B5AA8E3D2B1ABB5BEB63F9B7F5F62ADB6463426A0934A7408DF215F0D970'        
||'6809AD40A7FABFC9D056BB836B5689606C9474D3091A22B2A5AFA5B4F1201FED8EAB1FF79F6335B3DC6201F41614AE3A88E5DC7659DDE212AAB062B9'        
||'3FE0FE74A71E9A6AA8A83F7244A96AD8D35B4469DA39579B9F488C122D6FCFB571C92E011A4F9D6BD227484D4A1D4323B69FE4E931595F5B4E614A7C'        
||'7B4A030B34DA645B0370081F5F6BA282292AD23D3ECE8B6E2FA58B4A411EBAF99E1FE73FE0E98ABB20ECDAE7C3780A1F53C2E54936B7D41FEBED6436'        
||'CB4A2CB5FB4745373B9CAAD592DB47CC1C1FCC7FAB8F4DA722CE96149258B7D2478DC007D2793C9FF5EFED47D2806A1C0FC8F488EE6CD4D51311F683'        
||'4F5F981FEAE3D6394D2AA87929A442E080F1F1636B7AEC7EA473EFC8B313446069E47ADCF716AAA0CA8CB5C12A294FB698FCFA6D6192A7227C464E68'        
||'AD61E379583AF279B03720FF00BEFF0005205AC9D979103F3031D17993738499F67B96F9AB1C8FDB4AF4F745BBB7DD1344EF55154682B6F22CCCDA40'        
||'E751FEA3DA29B6AD966AAA02BFB3A37839939AED6924C03FE46B4FB4573D2F293BC776635447363D6A42FA469692C45B95D320B7B279793F6EB8358E'        
||'4D3FEAF9742087DC8DDECC11716C5A9E8789C63870F3EA6D3FC8E481DBEF3132514F2068CBAEAD361C72A78E0DED6F6DBF214AEBFA5207033D3917BB'        
||'D630BD2E6368D9AA0823CC75583F39B3ADD83DA781DD34B342D4F8EEB8C6514C5F524C4E3770E5B252C61003F45981524F24DBF1EC47B1D9FEE2B3FD'        
||'D93290D2485870A778441FCC753E7B6F71FD70E5F9B992CA4468AD66757E21AB0C51CCC00A53E1614CD2BD57C1F90BB35C5C63371FA85F9A4A2FA1E7'        
||'FE3BFB18FF00ADEEF231E2C38FE937FD03D3DFF04CF22B770B4BECFF00C2E2F3FF009BBD3E632AE3EC2806E6C32BD2D03C8F8F58B22A22AAF3D010B2'        
||'B1484BAE93A869F55FFD6F65D750372F49FBB2F4877035D532B46E193435C671D0BB65DCA0F732C07346C40C36ECCD0859FB5F54540C68A5C693A869'        
||'CFAD40E843DA9B53235D944A48A7A4595A9EA6405DE40968D012090A4DFF00A71ECA770DCA036E681B88F4FF003F423DA360BCDAEFC5E4EC8542B0A2'        
||'935A9141C40E8483D6D9A6055AA7165581560659CA9045886063FA7F5F647FBC211E4DFCBFCFD0B7C55F9F56F5F187E6D7F318D87D7F8ADAFB39B19D'        
||'83B270F48B41B72B7B67150D64D49414A9F6D4F4587DC73D65057565342142C1E779955404570A2C202E7DBCF6476DDD9D37EBD16D784FEA476BADD8'        
||'31C93224692246C7891DAD5C95E9FD9FDA5E6DE6C84EE3B4EDCE616A912332428F9E29E211A87CD4693EBD03BF20BE617C8CDF39EA78BE52CDB8AF4F'        
||'50D5BB536CE13118FC4EC7A08D21FB7A9ABC453514CD0D54E790D512CF34AAA74EA456208D3916C791B75DBDAEF902E21B8414595F531981390B22B0'        
||'0F18F405554F9569D05799795378E4EBB4B2DEECE4B57704A9700AB807252404A301E614E3CC0EACDFA6FBF3A7731D21D6B87CD5366AB56A7646021A'        
||'9C64B816A82251460E9578A4E1D4F2B246D707907D83B9BADB68737B61BF889ED959BC5F10D10693F154D3491E4C0820F035EA04B29B9C76BF700EE9'        
||'C8F35C45B9FD430B7FA652D2B331A68541ABC4571878D959596BA85074F91CDF1EA2A85AB7A7DFB2C21F59C798E43095D5711B0565948B704792FEF1'        
||'CA3DA3D923B868FAE948AFC2CF2887ECF13C2074F954B529E7D66E5C7347DF6BFABA653B45A2BE9CC890DB1BC18F8BC0FA864D7E7411935FC3E5D0A9'        
||'8EF921D3189A683174306728A8E894C3051D3E00411D3AADEC8B1F96F7BF2C4F25AE4924DFDCF76316DD6B631C1B518E3B7551E188FE0D2720A90284'        
||'1E35A9AF1249EB01B7DFEB0EEBBD5C5F734C92CF7EF21370D3EAF18C830438600A95A6909401000A00000E9EA2F949D42AA2CFB94916BFFB824BFE6F'        
||'F59B91F4F6FBABB7061FCFA2D4B464AF69F975287CA8EA1576774DD0F61736C2C3A4FE45879EFF00D3DB251DB83A8FDBFE6E9E48581A0438FB3FCFD7'        
||'33F2B3A8430222DD6CA086B7F06A70188FC1BD47F4F7EF0242051D7F9FF9BA7E2B72D52EA4529E9FE0AF59E1F967D463F5D2EEBFA97BA61E90104AE9'        
||'3FF291CFD78F6D3C12F930F4F3FF00374E2C14F238FB3FCFD674F975D451DED47BB4937D446268FE85AE17D551F8FC7BA1B5978965FE7FE6EB613C82'        
||'9FE5FE7EB21F97DD44574FD86EF3602E062A840621AE6C3EE7FDEBDD7E96519D6BFF001AFF00375B11D4E50FF2FF003F5D0F983D4E83D38BDDEC78B1'        
||'5C663D3E86FF009A9FF616F7AFA4909CBAFF003FF3756D14E0A7F97F9FA927E63754585B13BC8F209271F8E5E3E961FE53FE1F5F7436520AD5D7F9FF'        
||'009BAD9C0F85BF97F9FA8F27CC8EAA00DF0FBC8FF41F638C55BFE38155C7BF2DA48461C7F3FF0037551526850FF2FF003F501BE63F560D2570BBCECA'        
||'CCC00A4C58B6A07800D4DC8F6E7D3B004975E1E8DFE6EBC50F100FF2FF003F487ECDF953D77BBBAF7796D7C5E1F74435FB83035D8CA39AAE9B1AB4D1'        
||'5454C6155E668EA19820B1BE9527FC3DAEDBAD5D2FE262E0D187AE7A5FB386FDED6F553FDA2FA755E70CA6336E1948D2CA7E8CB7E47FC483F83EC7CC'        
||'818752DEBEEAF975283147011B52372B71FD9FE87FA11F9F6D680467AF0615A7538B9D09CD802FFEF62DF4F75007563C7AFFD7F9FF00FBF75EEBDEFD'        
||'D7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD'        
||'7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7'        
||'BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAD8EFFE13034B1D5F'        
||'CFDED78E404AAFC44DFD2706DCAF726C11F5FF0063EC61C92E63DD252BFEF96FF8FC7D3B122C8F46EBE823B4B6BD34CC8591EC0027D478FF000FA7B1'        
||'D5EDEC8A2808E97476B11C9AF46071BB6A97451C0C8C91CCE5DF4B7AFC14C3CAC40FE85B42927FAFB056EF70F73243644D7C470CD4FE08A8E7F22C11'        
||'4FC8F46DB669B45B9DC23C18632895E1E24D58C53E6A86471F3507A58C989A511B2A232291C8D47530BFD59BF3FEB7D3DABFA894B55B3D108822E02B'        
||'FB7A47657154C038B311FF0006FC5BDAA8E77A86E94451AB763741EE42014747551D3F1A661290E039D322E86B13FD0D8FB5C8DE338327A53AB84586'        
||'A17CFA05773554B1D56166B21099178AFA3F4FDD51C90DC9FF005EDECA77F22DB72D9AE40A8FAB311F909A1913FC34E873C9B10BCD97996C49A13B70'        
||'9C0F536D730C9FF1DD5D25B295F304720A7E7FB1FD7D8FA2852BD4745C8E83AC8E46A016E53FE49B7D7D9847047D6C3B1207493A9C8CF73CA5C02785'        
||'FEA781ED4AC09D5B5B74CDF7F31990314E5C0365B1B7D7EBEDD681341E3D595D830A74F50642A63A669A170B538EA98AB617D3FA55888DF8FC8B85B8'        
||'FF001F6537DB4D8EE76B2EDD7E9E241708D1BA93C55867EC34E07C880467A5D6BB8DDD8DDC57968DA2589832B0F22A6A3F2F51E62A3AEB3F0DE1A4CF'        
||'63C2AE372C5D5A251738ECA4435565049FD05CF9213F943FE1EC31C9B79324D71C9DBE396BFDB02D2438FAAB46C41723D5A83C39FD255FE9747BBFA4'        
||'6EB16FDB6A85B6BC24941FE8338CCB17C854978FD50FCBA47C933DCDC8FF00623F16F63E16F1F957F6F41CF1DC8A9A74A5C36D5C857529CCE56A20DB'        
||'FB714FAF3190420D4FFCD9C5D1F1254487FB3A069FCDFD8039879E76ADAEFBFABDB1C2FBB6F0785A4041F0FF00A7732FC16F18FC45CEBF20B91D0876'        
||'ED9AF2EA2FDE17CEB696838CAE3E2F9449F148C7CA98F9F4087716F1D99B7B79EC1AA8337B636963A3DBDBE70122EF3A0C96432DBDE832B93C0D5E62'        
||'1A59F1B94C5AD1C8051C29192CE104A2DA0DEEAF96B6FE63DACC9B9F35DE2CD757653F46214B6B454A909056ACED56FD495C132155C506405EE33D8E'        
||'E2F636BB35BB2C76E643AC91E24A494EE9314031DA8B40A09CD4F55FB9CFE673FCB8E8EA725889F07450E4E96BEA29EA2480EF8A1C952D7521347511'        
||'19EB32F59AF4C89721CB02C3EBA4D88C177DB589F3BA1C1C8F094839F3FD34CF9797444395F7C91750B5521862B29E07D327EDE8B86D4F9ABF0AFF00'        
||'D2DF7CF6267FB5B3FB7B6A6FC1D1DFC09E4C2FF78AB8E636A6C9AFDA5968659229229134C74346CBAA331956552DA801EF76BCC1B6DADE5CCE932159'        
||'8C6D521D46A0A548A287CE0127E7D5E7E59DF1ED6DED843DD1090515969A4B0232DA7D48F5F3E949F27FE727C18EDDF8BFDC7D4FB37B62BABF736E6E'        
||'AECDE0B6E8ACD9D43894CAE6E9E05AEC5D3E43214512488B34D0AC6656D6412189622FED36E1BC585DD9DC442E6226456A2D66E3C401A9680D453C87'        
||'D9D7AC79737BB5BD82792DD82C6C093AA3E1C09A0353F667AB49D85DF9B123C36DAFE2DDA5D794B2C185C379EA462B32B445A2C7441CF816B5A402E3'        
||'90589FF1FCFB3A9D6EFE9498743D470AD0FE553D455B6D8EDD25F81347711F7713A48E3E81475435F2F77B6D8ECFF99BF22F3DB5EBF019A8E04EA4C6'        
||'56E7F6D4F976C5E6DA9761EBA19928F2CBFB0D144E61648A6995994B9704E858DEF4B9BC6128A38550456BE4695F2E1E95FDB8EA7BB0B682CEC6386D'        
||'8931D5882789A904F9D78FAF417ECBA35A7DCD4654127F8176313F45D27FD1666F571FEB7B4EA28C3F3FF01E9567CBFD5C3A1E36063EA6A7A93B8EB2'        
||'0432458DC5F514F56C1953C74F3F65C34E1AC482D790A0005FEB7B585C79C812C60F9D7FC1D69B0C3F3FF07417F70401FAABA0895E1AABBEE543CDB5'        
||'9DDF8188DC7F869F767C9FDBFE4EAA7CE9F2FF002F4563339F7D9D4F8ADD6B8EA0CACBB7B78EC0C9418ECA574F8CC5545641BE31E687F8A57D32B491'        
||'53473F8E59D9149D08C00E7DE925304893000956520134150C2953E95E3D313C02E6DE5B7662A1D1D4902A4028D5A0F5A70F9F5723F25BB9E3DC9F1E'        
||'FBA76F647B83E35D7626BFAEB76E3F2B47B7B29BCEA3272D04F88962ADFE132CC4A794293E27742B7B1616BFB14DFDF413452EA54AB2B0246AF4C9E3'        
||'FE7E801B272BDFC5716F244660AACAD46F0F80208AD067ECE8B7C1F3B7F977D0189A0F9139B8996831F47221D91553D3AAD0D308235829E9E7545039'        
||'E5145FEA7D9AAF36EC71803EAA2A8007193CB1FC1D0EBF74EEA46968C7DBAD6BFE1E83ECCFCC6F86553DE941D8192EDC962EBBC97C748B67E0B354BB'        
||'76AF175799CE9ED9AAAECCC6315A65745862C7C426A8D719919945986A3ED1FF0058768FDE7F5AF3A04687483DE413E21240ED0714F4A67ABA6D1BA0'        
||'B7F0D535307D5F10A01A401906878F0E871D9DF34FE0EEF2DCB84DAFB47BC7295DB9B379182870B4353B53C34D3D7B1F2422AEB9E78845082BAA6908'        
||'1A5031FA8B7B325E69DA656F0E19D1D8D680092A4FA0ECA7ED3D352ECDBB143AE3007AEA18AF9F45E64C851D3FF346D89BE7B737D74F7DBF69744CFD'        
||'6183A8D8992DCAF81DC7B8B2FBA69368E076353D367D1AA972354F546EEB278D93F3191EA00DEBA4135D5D5DE9883D8DCA2E4F73321555008E249FB3'        
||'A32DA769B9B7169054CFA2EE076381450EA589A53000FB7EDEAD6A4E8AD810BCE65DA58DE5A6D3A6B2BDB9524F3FBBEF194CCE58827CCF592AB696DA'        
||'4557C87AF409F4FF0059ED7CA479C393C151D5782BE8122D73D50D11B090C8078DC7D6C3EBEDC965614A1E91D941036AD6B5C8F5E943DE5D7DB4B6DE'        
||'D2DD391C0E12971D518FD95B83254F3C72D53986B6928A6960A85133B0254A820116E391EFD6CECF228638A8FD9D57774861B495E11A488D88FB4034'        
||'E866F885F10773667A5F1994EED57C2E537454D56731D8B6884B9E8B6E64A0864C4B65A2568E3A592455328A7656915597C9A58951296DBEDBCDB922'        
||'DE5FCBF4C8C015455D4FF69D468B5F2193EB4EA3B839C1ACAD3E9C47E349A892CC68A062805327F97E7D2F7B77E18E3F09B4B2F95D8F4D1E7E4A1816'        
||'AA6C74F1CF4D955A5A79566AA9E8FC72B473948C33347A55C81E90C78F6C6F9EDC5E6DB6ED79B5CDF50882AC8542C941C4AD2A1A9C48C1F4AF467B77'        
||'38DA5DB0B7BC8BC176C2B024AD7D0D72BF6E47D9D536766ED04ABDF906228F15495157518DA2F0218E3942A12ECCEF3480E945176624D80F793DF776'        
||'DC2EED7DBA7F0E6745FAB9CD1588152B1F90F33D0039F361D9378E601717D650DC4BE122EA922476D20B505581341534CF4F588E97C6E3C354C95D14'        
||'15F3426295B1B8FA78A30A48731991ECEEB703EB6BFF004F7334FBDDFCE024923B2835019D8FE7C69D07ED394760B190CD656904121142D1C28848E3'        
||'4A800915A1A7CBA61DC7B6B27B6DE3692735343331482AE22E8BAC0BF8668893A1ADC8E483CD8FBAC776F2FE235FB7AB5C6DD140728A41F3D23F6742'        
||'6C55757E28BFCA6A3FCD47FEED7FF503FC7DBDF4F6E73A17F60E8ACEEBBA0341732803FA6DFE7E94B8744AA8257AA55A875974ABCE04ACABA01D20BD'        
||'EC2FF8F653B81304AAB09D008AD1719AFCBA1C72B2AEE36724BB8013BABD0193BC81A41A02D5A0AE69D3B18208419618628A58C168E48D151D1BFD52'        
||'3AF20FF88F69229A579555D8904F024F46FBB5858C5B6CD2C50A2B2AD410A0106A320D3AC7F7B59FF2B551FF0053A4FF008AFB32F0D3D0751C6B7F53'        
||'FB7A991E7F39122C51667291C6834A4695D52A8A3FA2A86B0F7436F6E4D4A2D7EC1D6FC597C98FEDEA4C19CCD4ECCB3E5B253285D4165ADA8750D7B5'        
||'C066FAFB627820550551467C80E8CF6A965F19AAC7E1F5F9F59DF235E51C1AEABB156B8FB8948208B5BEBED27851FF0008FD9D1FABB960093FB7A0FE'        
||'E7FA9FF6FED56B7F53FB7A3DFA0B1FF7CA7FBCAFF9BA72A3556898B00C7C87F500780A0FD4FBA348F5E27F6F4C4965661B1127FBC8FF003751B334B4'        
||'F3E3E68A486291247A752AF3352A026A92CED511952814D98B5EDC737171ED6EDAEE6F50124FC5F3FC27A887DF3B5B65F6AF74291AA9FF0015A10003'        
||'FEE65BF9D31F3F97485CE64327B3720F4F86CB355471244E69EAC89853CE544924319750ADA5890B2AA82C2C7E87D89EA41A0C53AE7C65A3CD083820'        
||'7953CB3C7FCC7A77A2EF0CCA81064A18AA9F40826798A55C1690021238324B3AA90795D1F43C803DE8B6294F3AFF00AB874E88D5B26950295AD3D283'        
||'874AAA7EC2A2AA32CD3D2631C205411F89686563F53221A3290961F421A1E7DB7A8D6BEBD3AA1E3148C574F95707EDAD3FCB9E959B7FB229A5068678'        
||'7135545E39A55C56EDDBD459BC3961CE95928169EBA9D5F9065A698E9BDCC6CA0FBABA9720B1208F3069C7C8D6A0FE63AAC71A6A245079F0A9F3A914'        
||'A1FC871F4E95D80ADE862B96C9EE4F8FBBBF3353E0924A41D6FDE7260717415213FCE7F09CEE1B33512D3EA20954AD62071E8E0045730EF4405B1B88'        
||'9541EE2F0176A7C8ACB1A834F3D3F974F41FBBD583DD2CACC78049235A9F32DAA17623F3045684F0E81CC8EEAD8531AB787606E1C5C6646681B19BEA'        
||'A05653140444646AAC74904C54FEBBC084F3C8FA058AB7208D6E09FC5D8457EC1AAA3F69E933E8119105554574D4827FDB10A15BEC0ABEB5E9E71136'        
||'D5CA5550C157D9198DAED3C71AC4BBFB0194DC18E8BCA2CA24CC6CF6AFAC58F9E58635828E4816F7795DE35668E3D647929504FD81B42D7FDB74C422'        
||'51246CCE7493421B50A5786754869E802E2BD09198EB6DD0E2193079AEAEEC3C63D3C77ABD95D9FB77EEA2D49C8930BB8DF1B9384FD2F0CB44ADFD01'        
||'23DA383728E63A2649616F4922703F265D6847CC353A35BAB59224322B24894AD639149FCD5B4B79F0A0E824C9E0E2C3D79A3CDE1F218DAC5F4BD3C9'        
||'56C913175B8918F88070D6E195EC47D0FB3204D350C83E75E8B42461878A0AB11C294FF57EDEA75060B05B83462E5ADA3C3CACCFF6D5F58F53538E49'        
||'5C69535CB470B5446973FE70249A7EA411EFC5BB6BA6BF2C57F2AD07E5D38547F68A476F019C9F4C6457D73D186E98DA9BE763C5BBF1987DDD053C99'        
||'BA5C54B4F1EC0EC0A5ADC5E65B0D5AF5468B7245B7AA56A2259E3668E38AB638AEAD29552CA7486B7596C6E5A369A1AF864D7C588EA4D400AA6B5A1A'        
||'7994AF9797433E548AE61F1A4D434C94A6891483427042B1238FE200FF003E87097ABF1F9DA0DCBB97CB005DB5490E494557DEA495988DD99EC762A5'        
||'C63551D2D2B5364DA9AA8FDC3DC2DD16ECADECB937268678AD581266250529868D1DC1A790640CB81C78E08E85BE096864950FC0031F982CABFC8907'        
||'A51E023A89678E2295D593D442B2FEDC356D4F157D02C54B2CA5DDC07F32182604496B8654B49E4BA6B921549C28534E22B4353E9E46A387A138A75B'        
||'8C6AF9D7AB0EF80DD7BFDE0F90188CC6415E78362E0F25B92596489E28B1F3CB18C7E3C92598162F3B5AE3FB2C74A157062AF73F76FA4E559208F06E'        
||'5D63001CB0AEA6FCA83F98E351D0D791EC3EAB7F476F86105C9F4A0A0FE67A3D5F35F3759FE8CB6560BD72566E2CED76E1ADA7658F4C54A2F4F8F470'        
||'AF1F117DF2A06D45494D24956BFB8E3DB7B78FF7C5CDD0C2C282307E7C5BC8F1D15FCEBC4742FE789E4FDD76F093DD239908C70FC3E6386AA57E5F3E'        
||'889ED6EA5DC59FC44FB8E35C3EDDDBD47E19721BDF7652C784DA38A1180C516BEAEA434AE8BA9A3A7A312CD70D1B210C0FB93AF77DB4B59C599D52CC'        
||'D50B0C44BCADFED42D003C0B3955F30707A00DA6D7717119B8C4710A5647EC41F993934E0A818F953A4C643B2BAB3654490ED1C5C3DB79CA78836437'        
||'957E1BF876CAC6DC57D72C9B5763D43BD456099F13928E8A7C8C8CAD2C6A8D4E239158A94DAF7BBF6ADF39B143C2157D52B7C23F567034A504885D63'        
||'1501AA1EA08EBC6E36EB6FF71878E4719196883891A2226A6B43A4C87CBE1A1E8B8F64B6E9EC6DCB8EADEC6CDD75756D3550A4A7C56B4931DB6E46C5'        
||'4B488C91D37ECD398320F10A48E2E23448F51245C0AB668ED36D84C5B6C6AB1B0356F39320F9E5AA8096272493D155F4924C7C5B8725D48F9E9F5E1C'        
||'28694030053AB3FF0080559054D76CFC9554AB512D760772D18A9448D34D461B6BE261AFA895214558FCCF4F5A224E58888D8955BFB867DD28DD2DE7'        
||'854534BC469F26925D2326A68192A7033EA7A91791590DDC521FC4B253F244A9C7AD1A9C787462BE55FDEE436AF5B23CAD0541A5C8E4A5F1D55453B3'        
||'D5C9414D301FB60B904C4D6BFF005FA73EC27C8A521BDBC34A8AAAF0071A9879E071E8EF9BB5CB6B68BC0D19B04F120744AE971F24B92A383EFEA9E2'        
||'86649D87F10AEB0892BA1AC8F91C7E8AAB1E3E83FC7891DE655859B40048A7C2BC7491FE15E80CB11F142963407D4F0A83EBFD2E9473E3E45A323EFE'        
||'B1C88E950DAAEB402638EB62D5215FADB48E3FC3E9F4F69126065CA28E3E4BFD0FF3F4FBC4421A31F2F33FD2E3D457C5CD2515A3A8CA03F6F32B5E7C'        
||'9DB5BD3418F431C80823FCE137B5BDBC2E1449DCA9C4792FAB37F93A60C0C57B4B70F56F90FF002F43E7C40A29F13D959DA59DF20FE4C15694FBC9AB'        
||'254445C95D74F9BD26EB1A916E79FE9EC29EE14A93EC9148A1451D78003F0FCBE67A127252343BC3AB13F0371AFAFF00B1D053F3145565B0D5D055F8'        
||'EA6B29F118ED859289243ABC3925C2FDFD54736924BC71348EA06937049BF03D9DFB761239E3A02AAEED30F4C19A83EC2680F1FCBA2EE716628C7898'        
||'D421FF00794FE63AAE0EA0EE1CEF4CD1AEC5CED2D16E6EAE9B375F141B22412C716DEDBB3D696A0C8D14E8B7A19A3864150B2C4E1B4A31F51BC4F2AF'        
||'306C70EF337D7DA0315D2A8FD614AB3532A41C38A8D2411435F2F880376DBE7B48843390F1B64C66B4FB453E13E6181AE3CC547479B1BB7F6CF676D1'        
||'A5EC5EAACDD66E3D9F2D2C53D651C9575149BAF69472810987358B8103BD3807C30D5C4817C93379D21705447F2DDDE6CD7A76BDEE3114F5ED6A0314'        
||'BE75463C1BCD909AD14692C33D1EFD1437B6FF0059B612F181DCB5FD48FCBB80E2BE41C62A7B829C748DC9632F4EF155543CCAAA59E75A8AD3435479'        
||'2411CE9560AE5AD6BC5180BFE72C4CA09FB834629F2A0D43FCE45471FC473C3A2D962A0A486BF3CD0FFAB3F90C71E855F8995F51B63BDB054F2493D4'        
||'516E2C665716CF3999D44E631514E647234FA9E3646FA7ADC806D17047CF7125EF2BCACA0068591B14E15A1C7D86A3E43E7D1CF28486D37F8ABF0C81'        
||'9735F31FE7143F327D3A14BE74ECA9B2DD2BB7F2F4D467255BB03B07298198D4289AA5B1D5F553D25348C75056BA4746CBA89550CBAC7A4FB21F6CB7'        
||'25877F9ADDDB42DD40AE29C352804FF85FE7834E8FF9E2CB5ED51CC82A609593E7A589A7F817ECAF54B99B8248161FB480A0811EBE4C7BD1B421E646'        
||'5FE194CE925547612CAC5829243AABB8266318190B6EC1EBACD6BDB506B8FC44769E031F2240A69AF511B82BC3F653F679FF00AB3E7D05BB6F0BB4EA'        
||'B0B97DCB454B3CDBAA8778D5D1574CD4123E3EAB6FEDFDBF4357B3F132CD22952F5996AA9AA4B3C8B2034E09701267F6697B2DDC77696AD411188119'        
||'1A83BBBF8ADEB458D546011DDC2BA47558842D6FACD4BEA3EB4D200D23ED249F9D07DA7A43D6F430ECFEC0C26231FF002977474D6F7C16DF9E6A4D9F'        
||'8AD998FCFED7DC74D8CAD972FBCF76E7A4ABC9D34DF691D7D4D1D0858AAE2F024651FC8110A6367BD1219B7FB37AD2B6D50BC68BE2B85AE06481F9D3'        
||'57E23D4C5EDBA84DAAE053FD1B8FFB45FB7A55676AA6D8FD79B27AA3B6F2F45F27715DA398CC64B7564B03D6B9CABEB5ADC650E31B70EC6CEE5F135D'        
||'F7B4D4149AA5A55C4BE42A041573D33BC8743C8BEE1DF2CE6BD485F6629D47DC198ED3DBDBCDB7B60F77F4DEF8D9F95C4E12931DB1B1996ACC147D2B'        
||'F6F51254B6E0DDF99951CE5AAF2503454822C5414C6082355D34F4539AB6F7757ADF69E90ADB63B8B17D998BDEDB17B2B7277161B7E64A1DB9DB1B6F'        
||'6D47B7E74EAAF262172382AFDA7151D3C54188C508C2D34F4F909A5C8C151E3C83B54196414FEC838F3EBD83D0A7F7941F1EF3BD6FB2762FC7BDBBD5'        
||'BD55D93D81594951BCA1DFD84C965767E5F7051D6E6F2F53B9BAC726959575994CCD5C4E46566C85750D3B54C62A62133085BDC303AF71C9EA041DDF'        
||'D4DB1376E23E23F596D1DBB94C857E133F94ACEBEC36029B298DA3D839CAD91ABE5CFE73703458BA8A09669D8414B2CBA34DA18A162C22F7EAE69D7B'        
||'2457A2B2BF09FA8F784B3E43B2F35BB33BBCF65EF06DD7F1FA0DD5D8B90DCF82EB4DAF8DAD8733B1B696E2EB6C543062AB218DAA21A3CCC9411564E6'        
||'3155234DE581957C1479FE5D6F51E03A305DBF85C266B0BB736A6E3DF5D859ADE185ADDA9BAE5E85EA5C05455ED2AFCDC13CB94C46DDEE4C5E322ABA'        
||'CA7DB95B252901F2196A5130513454952E8606F1A75A15E3D059D839DF96C7BB3AC76C6FEDFF00D39D45D1BD9473385DDFB97A7F6E64F75F63637746'        
||'322FB9A1EBF8E3EC3A14151539764A8829B294F81A9856686659630EA909D770E381D6FB69EA7A18F6BF607C47EB8ED61D03D4D57BE7B03BF7B09966'        
||'CFD164F6BEEECA768EE09320CF4F4F92DFF94DCB89C5E2528A9A2A78E0FBDACAB8608A9D6391651A7CD36C15EB5A49CF4E955F1E21DC5D19D8BD73D8'        
||'388DC387DAB91ABDC78DEA1DA192CA7F0E4D834294558769CD59B2769D40A68EAE966961820ADA9A9AE8D1A38E58D5226741BA5475AAD0F448B3B99C'        
||'F53E3F68D16269B3BB76868B66E0B1E31F879B0B8CA379F1F0B63EB2A852E4A1D619A489935C65D1C2870C59980C81F6CD6BCB15A81FAD2791F44EA6'        
||'5E43E4ED877DD8FF00786E285A5F15D6BA9876AE9A6063CCF4C11E6376007FDC8EEE36E6C739B6179FF1D34E7DC81A47F10FD87A1AFF00ADBF290FF4'        
||'135FF4EFFE7EA62E5B773B2DEB7761B1247FBF8F6F2F2458DC2D29F7A2A3F8BF91EB63DBAE521FE827FDEDFF00CFD3AC391DD8EA5649775C8A54AB0F'        
||'EF560631A48E7EB447DD1801C1C7EC3D54FB79CA23843FF1B93FE82E91F9CEB3C46ED46FE3580DD95E8E75B8FF004814B146CC0F24A525228F6CBA2B'        
||'61DEBF91FF003F4A21E4DE5AB6CC5181F9BFF95BA263DFBB2FA2FAE72587C552F5F6EADC5DC3B923825DB1B7B1FD89573CF8BC6D2D4032EEBDC50C34'        
||'EAB063A98AB30F210D3B8F1C57B3156D2D965B848AD50CD29A1D22A28A38B124D028F9F1E033D1073649CADCB3B44924E017756089A9865811A89270'        
||'013F69381F28FBE715B5E8FB53726E3C36DFA6DC5DBD9AC7E0E88663294CB518EDAFB6E9B0F19A6C8CF34F7B3485A46F1A10CDA45CE91751F89AC36F'        
||'B892F52357BB90055AFE150304FF000AD6B81963D639F24721F30F3F3A6DE2430ED50B132B8C177D4494AF9D07E42BE5C7A4DECDC4E06AF72F60A41B'        
||'8F0AF9C8BA8B7B8DC3B9F70CD5B4D4799C91C9E34AE331B250534EA168AC59D06958C48A2F727D85EFE69A7B8FAA98B48CE8E357E1A82B81E402FA0E'        
||'15F33D491EE86CFCB5B06D9B6F2E6C3A11ADE42640284F7A80A58F1A9D2695FC853A89D7543BE766EC5C3C49BB2830580A4359BA1FB236DE7056D452'        
||'525657D4C143B576D66A9DD952A69CC33FDCC7069689E59239406572BEDAEF117656DBF483A9D8B31A120023485F98F53C2B8049E94727FB6761CC37'        
||'CBCCFBDCA4DAC49E1A4418A8D54FD463C280D46789A0A500E86EDBB361FB836EFF007577BD0E470591C9B5454ED2CCD74068E2DC14F44F65CB514284'        
||'0490A1515F48D61221D453430214C96D730AA35DC657577216186A7A8F234F890F107853A8C2FEEE1E59E65BB8794EFCCB1C44A6A43A5B43D7B1E986'        
||'A640618246A143D26B75C53E0F354F8EEC4C6A55BE02AA9A87AE7AA36D435868775D63C41A9F34ED1001A9C3692E818C8ADE92540F305F7FBADE6ECE'        
||'B671A6818EC5E0C479E6981E40F6A0CB571D497ED3EDDC93B06CF71CDD7B75E35F2926679A9E2455274845FC45C7065C711404105FDC65F68E7A8B74'        
||'EE9868F7877CEE28A35DA5B268630715D714042C149E4A485043098942D93470A02851FA49AC1059EC36C659CEB99C1033C41E2149C84AFC721CB1C2'        
||'F415DC772E67F7CF7D1B46CBAAD765B76ABB9AD0D0FC4E7F131C90B9F9F5871B4194C86673185C567866F7D66CBD476BF6BD4CDE4C7EDCA0957C9598'        
||'1C156C8CC19D81D0F246DEABE80590BB4E5B696B73BD5D78F71511A900D0528070545F2A0E03F0FC4DDD41D4A3CCDCCBCB3ECB72BA6D1B322BDE38FD'        
||'28B8BC921C78D379D2B903F67CB034745BCAA31BB2763E3ABCF5DD3E56931B053E3299E5CD767EE99A74829F138F4A75F24F1CF504283C995985EEB7'        
||'322DDD3728E2B7FDDF6642451821D81C5056A01FE11F8DBF19C70AF41FF6ABDB6BFBBBD6F733DC26325DCBFA9124B4FD3073AD81C0A0F81701475743'        
||'D01FCBD3796077362EA7B597AC72EB9BC763A9B70ECA9F6E6DCDFF000ED54A22990A0D8D87C8E7686AE9B1D92A71A5B3398A212351852A19A5652F15'        
||'EE5BE595DDB160AFE1A1AA924AEB3FC5404123F841A57F2C0AF9C39E2D77343B46CA8EF38350CD88E35E1E2BAD46BAFF00A146D4D6686800E8C6EEAF'        
||'8D3D8C9567EE37CF54C388C7C74587DABB468763E272D80D8BB6E818418FDBFB393298DD5052522334D2C8583C8DE495BD4C14150DDECE503F49FEDD'        
||'5A6A7D480789E03F21D31B2A6C1B7DA2C115B4B239AB492B91AE4739666C9E27014614500F33D46DC39CDF3B337BEC4DFBF293B877BF7DF57EDACC64'        
||'AB2B3AB76852CC2B32E2AB6765B1D83FE0788A2FE0F4091D157CD495754CF550AC54B14B23B189194B4D657DBA5A49B7F248876DDC1E9A2E27234200'        
||'478BA8B07CB45AC2F69AB914CD080BFBA537B6769EDC5E8E66D99A4DB8BDB09D519848EC2E23686AE8DE2D0CEB1D687BBE020EA20869BBFF0098EEE6'        
||'C9C953B5BE32F55741F40E35E5FB65CE9C3D27C8FED4829652CC6AAB70DB58D36D6C748506978F27BA252A4D8AB037020DAFD8ABABB3F59CDBB8EE3C'        
||'C128A120C8F65640D3D4B2BB00720A01519A7975819BC7BD7C99C990ADAEC3B5EDBCB10387D091411B5F38539A34B14D724915040B351AAA3C51C7AA'        
||'F7A3EA8ABDFDDABBEB716FAEBC8BBD739B3315B331982DE7F26722D0EC7C3546731D91CFE4B00BD2DD7D5543857868C54D3D5E2F17F7F569451CF12E'        
||'89FF006D925B6D976EB981368DDEE859C5185610EDA2BE2BB3659E67462640115659CC61A4A905EA2A625BDF7AB70E5CDA62DDB916DE411EE535C78C'        
||'6F4E89628EDD228E331096692431C8CF27830B4A151A2252140581301D69D3BDAF3D33ED3A6CC627AC369FD9BC7275C755F5E41B5E8733836651578F'        
||'C865B63517DF3C6FCB346DB82628180996E34FBAEF2DC89B3B47B95BC3E3B230FD6967842A4838378520515C769F096A6B4A8CF40DDBB9EFDCAE6333'        
||'6DAD7122C8F1C834086F6693C36180B3AC920142DFABA70401570D856DEF5F81DB6FB7B2783FF483D83DA1B2EA297114D85C152EDD06971F151D2C5A'        
||'6238BDA39AC6CD474D24AAA5AB1A99A212C8BE520162A0A25E69B5DDB6B9A3D9068A3969248658EA6A4D448C1C96C9A2B11A80ED24D01E857C8FCD7C'        
||'FDEDA5D0B5DD04776B28AA2DE5B4FE382428AC32D03AC745CC459A353DC817511D16EC2FF285DBDB752BA2C77C90DE2B8DC94EAF58D5FD638C9F313B'        
||'053E38E4C94391854680C40294E05BF1720FB24DAF7A9767496D76CD4167CCA4B46646A0230F414193414E0789EA5C9FDE6BBBC686F373DAA1925849'        
||'D07C49F4066E3D8D1935A0153AB88F2E95B86FE537D7B80A4824A5EEEEC6A7B20A6967C66071F8F69949D4501A7AE524D89F5CBA985ED7F6F59EF96F'        
||'683E96DEDD85148FF7200A8AF16020EE35272C58D3CFA4F7DEF06E1780DCCDB6DB32960DDCAF210C47E12EE298A021428F515E876C2FF29AE94CF613'        
||'175F86CFF70555753DE93239CCEF6425649908E4B54491C383AEA7969E885F494FB72001C1D5F5F6863F7320E58DCA58EFA25909421549B8603BBCD8'        
||'48A0D294ED55E3F97489B79E7FE6CDBD6F3615B4B68D9C1245AD9A9000AE95AC6CC6B5AEA777A91514EB8D4FF2B5F8B58FC854E37347B9EB3218D6A1'        
||'7AC969BB2292929E433C42A10C51D2E3F41041FA004023DD27E79DDF778E3DD2DBC28A397569004BF80D320B9F31F98E948E7AE65D9E69369B9581A4'        
||'83406616B6B9D4A1B07480706A481C6BC3A69C6FF2B9F8A13C30D46E4C576AD5E4E825AE68E4A4ED8C853D1B5243249418A74A6869D2D2350AC1E51C'        
||'8130774209D5ED9BCE60DCA5B88E589928BA7483E2E1B4056341253D7ECC3001BA661F72798D63741E181DEC48B7B5F362C40250D46A248AD7D0D474'        
||'F54DFCAC3E1C53421536BF64BA075D119ED9DCCD220500DDA455B9B8B104DF9FE9ED28E63BA148D6'                                                
;                                                                                                                                   
BRANDINGBackgroundIMAGE_8 raw (32676) :=                                                                                            
'38801E7FAE6A7F3B8F2E9C3CFBCDA5351B85E3C05B59529E64FF008B1C91E951F9F4F791FE4D1D5796A2195C16672B82C5D5470C948A3726EBCCD72C'          
||'72480289EA7239454623572DE05B01C0F684FBA3B04131B5BAB02D229218EA71523CE9ABF664FDBD0AED07B92F631EE027B592290022915B455048A6'        
||'56D8918E39C9A9A0E1D0335BFCA7BAAE864A84FF00493DA2F5B0FDDA0831B96929627920661E389AAAB96352C45819240BC8B91EC4D1735D8CD49E2B'        
||'66442070B89852A2B5D2093D07E4F71B992D9FE925B5B3675620B3C28E7B4D0D58A0C799A0FB33D2BBE2F7C22DABD4DDC7B4BB592A7BC70551B0EB6B'        
||'AB2869BB0F35B56AF1D997AFC4D4608DA0DBB9ECA48D1E8AB7911E58946A55FCFBA6E3BA5B5ED8BD9421D99E83FB795C541A9AABA80787AFCFCBA14F'        
||'297B83BADDF34D958DCC1622398BEA786028E80452302AED1A532A148F43C73D5B5D5764482755A5AC205CAA36A37B7E6F6E6FF4BDFD9243B048CBA9'        
||'D71D4CD79CD9671C8635906A1C7FC3C457A5660FB2F1D3CBAABAAC2656998426A9A6229F4C51FEDC5246FC103936FA73EE975CB9791C63C342636150'        
||'299CF98E91D973A6D37170E8F2A89636D2D52028A0AE7A42762762455CAD063A7A6323891259E097D4F1BA9051556E0822F7239E7D893967972556F1'        
||'6ED4851C011E7D473EE273ED9C69F4BB548ACE6A188604D08F21C3F3F2AF40A51C759545C2A04540AF23AA066B7E9D441209E3EBEC7D3781080389E1'        
||'43D42166F7D7AE5A9A538920571C2A4541C0AD298FCBAC1356D751B0FB2AD9524D4F710B1034AAF05C2DFEBF5200F6E476F0CC3F5E314C71F9FF009B'        
||'E7D25B8DC6E2CCFF0088DC36AEEA85F414A5467272683CBCFA153ADF159BDCB51E4CB6467A5C5C4C6E1A3BD456054E440D2D956E48F5153C5F8FA7B0'        
||'9732DE58ED6BA6CD03C87E7DAB5F5A64FD951D4A7C81B56F7CC5FAFBB4C62814D2857BDE838A934001F5A1F9743EE03AF68774E3E7A49B1B4D422269'        
||'6D3545533D5337221613C23F36E401A45C7B01DE7305C6DD3896394BD6980B45F98A1FF8B34EA64B1E4CDBB78B36826B711D2BDCCF57F40430FDBF2A'        
||'F49AA7F8F9514B5CCF90AB8EAE333C6D454D45505D4C2AE19854065B9B8B823FD8FB33979F565842DBAE934EE661E7F2FB38D7A0DD9FB426DE72F7B2'        
||'191757622B54115CEAC0E3C298E92395E96DD345539AAEAD968B078BA6F35551A7DC34FE4899898E08624074AA5C2FAF93C0FF001F6696FCE5B6C914'        
||'3044AD34AF40D8A50F9927CEBF2FB7A21BBF6BF7B8EE6EAF2E1D2DEDA3AB44351355E2140A600E06B93C3A66A2EBCC8CD838F24CF05464AB24686971'        
||'EEBA046A2408F546A7E9F5E141E3EBF9B7B533F31DAADF7D30AAC6A2ACDE67E54E92D97216E4DB4FD749A6499FB550F05A9F8ABFE0F2E85AC2FC7FA0'        
||'780B667754A92CD120823C745A1629B40D7E769F586F55C050071CDFD862F39EE4D74B5B604026BA8F115C5294A63CFD7A1D6D9ED246B1EABDBC2188'        
||'00681C0D33AAB50687000F2E3D7559F1D2BA82569A5CFC15541E35788C546454CB6604D336A6D21ED7B30B8E2DC71EEE9CFD0C89A5202AFF0036141F'        
||'3FB3A4E7DA1BA8A60D35CA98FD554EAA7A649151C41C83D4B1D5BD7490B0AAA1F34BF6C69E57AD77768A42850D4468A4286079E781FD3DA0FEB66FFA'        
||'BF4E4D22B5EDF315E07E5D080FB67C9EE0F8D0F88E534D5F39A50B018CF98F2F975529F38769E0F64EFF00A2C5E1AAD25A6ABEB45C9132C91ACE659F'        
||'2997A724A020588885885E4EA1722C00FF0062DDEF37C8E3BBBA5A324A1300F0050FCFF88F529FB71CB3B5F277256E3B5D838092BDCCB9201AB5AC69'        
||'818F241F9F542A91C9A13F6E4FD2BFD87BFE9FF5BDE53332D4E47EDEB9ED1C72681DADC0791F4E8E77462B0D81102AC0FF001CCCF05587F6E3FEBEE1'        
||'8E7ACF301A7FBEA3FF002F59E7F777047B6A80823FC6EEB8E3CD3D7A321B0ACBB9212C428FB2AEE58851FE6C7E4FB025F7FB8E40F51D4D9391A2A7D4'        
||'746E7A936E63F77F62ED7C1644C7363AA2B64AAAEA7D6A4555363699EBDE908537D3218C237FB493EE2AF73F982F395B90F72DE76FAADC2461236FE0'        
||'795D630E3E69A8B0F981D09FDBDD9ACF98B9CEC369BDA342EE59D7F89635690A7D8DA403F227AB4D5544448E3448A28D1638A38D1638A28E35D11C71'        
||'C6B60AAA000AA0580E07BE6DB333B1790966624924D4927249272493924E49C9EB3D942AA84401554000014000C0000C0006001803A0A3BCB6562B7E'        
||'756EEFC46529E2925A1C2E4B3B86AB64533E333387A17AEA3ABA6908BAEAD0619403668D994FE2D20FB55CCFB8729F3EEDBB8D8390B2CD1C13202692'        
||'C333AA3A30E0695D6B5CABA823A01FB99CB761CD1C93B8585EA82D1C324D139198E6890BAB29F2AE9D2DEA8C41E93BF1BE914F536C6C8491DA6936AE'        
||'121895802D145F6092BD81FA162C01FF0001FD3DC9DF791BDDC6D79B3F70BD52072F7073893548C884D3885D24807CCD695A758D9F748E5BD9AE537C'        
||'E7A2AB2DE2DDB58A3119811512594293C1A43228661928BA6B424743D7BC6FEB343A456EC8A280C190034164992A59549BAC31897CAE17EA40BF3FEB'        
||'7BC8EF62A6DDB778AF760855A64B6092A0A8EC123146515228A580600703AA9C7AE7DFDF4760D8F66BED9F9D542C171B834D6D39029E33428B245235'        
||'38BAA16466392A101E1D23B059BC6EE39628703549969E590C31C544B2BBCB30B5E28C10BA9B91E91CFBC80979637E85099ED5929E64A8C7AF1EB071'        
||'77BDADB29303F603FE6E8658FA7FB4E5489E3D85B8889115D09A68118A37D094790153FED2C01FF0F648D0BC6C55A9F3C83D2E4BA82550F19241C834'        
||'3D326E3D93BBF6740936EADBF5F814949584645E8E19A53FF36A956532B01F9654207E4DBDD3C37A6AA607CC74E25C42EC023549F91AF48BFBFA7516'        
||'2F6B9E2EA6F7B723E9EFD96C8231F31D3D4A1C834FB0F497DC7D91B176847136E9DD785C089D888BF8956253C929FA9D1072E40B1B9D36FF001F66DB'        
||'76C3BDEF008DB6DA4980C9D2A48FDB81D16EE1BB6D7B551B709D210786A34AD7F9F48FFF00661FA447A8F68ECF02F627F899E47FADA3D9A7F5179C08'        
||'A7EEE9EBFE947F9FA2D5E70E5661517F07FBD8EB247F20FA49E431C7DA3B39DFEBA7F89DC8E2FCFA78F7EFEA273928AB6DB381E5DBFECF4D8E77E527'        
||'6D31EE10B11C407151E95E86EDAF04FBD76667FB0B69A2EE0D93B5A967AEDC1B97193433E3F17454B134F5355508584BE38D11DD9D22616563FD96B1'        
||'1DC6D9B85A5D2D85E44639DC8558DBE224F01E95351415F31D1DC3BA58DD426E6D9FC48D454BAFC2052B5AF98E81FAAEF0EA1A72527EC7DA90B0FC49'        
||'948D0F03F208FE9ECDD39339AA4EE4B0988FF4BD1749CCDCBF18225BB8D7ED34E9ACF7FF004A8B0FF4A3B285CF1AB354EB7FF0F57B78F21F391CAED9'        
||'3E38D13A2E3CF7C9B1F6CDB9DBAFA56403A918CEE8EA8CF646930D85EC2DA395CB642A168E8B1B4799A79EAAAEAA4368E9E086337666FC01EDE8B93F'        
||'9A2C6417779B7CF145190CCCC840551C493E40746FCBBCE3CA97FBDDA5A596E56F2CB2C8AA88B202CCDE8A3CCF424208EDA8DD1B9B0FD4A09FF1FAFF'        
||'00B707FD7F6BF3C0753AF0153C7A71A5865905A38DA50792514B683F40C0DB8FF636F65DB96EBB66D31F8BBA5C476EBFF0C755AFD809A9FC813E9D1A'        
||'ED5B2EEFBDC9E0ECF6B2DD37A448CE07DAC06951EA58803CCF4E2B4C89A7EE668D2C49F147FBD29FF69217D23E9F96F641FD63BABFC72F584D700FFA'        
||'34DFE2B6FF0068320F19C7FA48483EBD088F2A596D64B734EE705A30E305BD2F6E7CF05616F0233E549270479AF5FFD0F9FF00FBF75EEBDEFDD7BAF7'        
||'BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7B'        
||'F75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF'        
||'75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAF7BF75EEBDEFDD7BAD943FE12D101A8FE60DDB6'        
||'AA3515F87DBFDAC3EA6FDD5D7D1D85BFE0DEC57CA0DA77193FE6937FC7D3A7EDFF00B4FCBAFA28ED1A28E08A32CAAF312381CA476FF5447EA3FE1F41'        
||'FE3EC4F76E59BE5D1A9348FE5D0B58E225ACA862757DAD3D3D3826D6F3540FB9997FA5C2F8BFD6F6175227DDA67F2851631F6BF7BFE7411F4AEE95AD'        
||'B61B78C61AE649253FE963FD28CFD858CBF691D3C4A4052491C03F9FCFF87B30E2683A25881F3E91596907AEC6FCDBFD7B7B54831D2987E227A0BF2B'        
||'3A09195CFA24D513FE789174DC8FF0363ECD20534EAD21E80CDE05A08096177A2ACA49987D0DA2AA50CDFED89F655CE721B7D8177041536D716B2FD8'        
||'166404FF00BCB1AF436F6CE34BBE687DAE56D2B7B677D6FF00697B590A8FF7B55A7CFA4266A7014A8FCDFE9EE4589738EA375A902BC69D0695F35D9B'        
||'F16BFF00C6FD9822F5707CFA4D4CD70CC7FB449FF8A0E7DA91E9D580238751A1A4AA9658A4584AC5AB99642228EDF93AE4B0F7B6650A4139F4EBC4F7'        
||'803CBA7EA5861498C3255C2C6A217A768E20F20BCCB65BCA405B8363F5F6C31256AA381AF5B393F674A8D93889732993C39C7564B82AE422B32B52D1'        
||'43478BC9D1A97A1AD590D85EFF00B522AB9628C78F7117BA7BF6D7CB37161CC515CA2EED66F48ED949696EED65216E2DCA28660A47EAA3B28559101A'        
||'8A9E867CAD6777B8C371B6BC64D9CE3BA4E0B14A82B1C818D057F03006A5588A7582BCED3D9D3CF4871B16E5DCD49234532D4C13D3EDFC65547C11E2'        
||'A83E5AB64363EAD287DDACE5E7AF71ED63BD866FDC5B2DC2874313ACBB85C44DC3F516B15AAB8AD74EA917AD3A6CBCBD2B42EA6FAF23343A815B78D8'        
||'7F47E29083EB453D23B29B933F9CA85A8C8D6198A0D34F178A14A5A48FE822A4A655D31A01C00A2F6FA93EE43E5EE54E5FE55B3363B15BAC2AD977CB'        
||'4B2B79BCB2B12F2313925891526807447B8EE979BA4C26BE90C84600E0AA3D15461401E83EDEA7EDBF8FF83EF7C8B56EE6A8A72FB2A0098F5AAC1D16'        
||'662D3B95D8D5D92A6C23B1C7C4415E4DBF16F76DEB751B488CA2062E5B8D0D34E9F50479FF0083A09EF3B7C9B9688D095515D5A49526B4A64107C8F0'        
||'3F6F4B16FE5D9D2923B4B3ED9EBFA996472EF2547516D399E4763ADA496495092C5AE5893737BF3CFB0F1E6AD5C608FEDD29FF00407448397EE233FE'        
||'E44E07FCD6931FF1BE9D60FE5E3D311111B6CCEA1969AEAE6097A53699058023530375B8BFD74DF9E3EBED3B73246EB9B71FF19FF22F4B23DA2FE261'        
||'4B9933FD263FCCB1E9BF33FCB4FA2B2D185A4D89D3D4322C904E5BFD0EEDD01DA1904AAADF69E260A481A803623D278247B65798A2187B706B518D03'        
||'FC287A338F6ABF5EF8AE581F3D4D230FD9AC0EA3E63E03C52432438A6E97A11A7F6D9FA4B173AC4C8BFB60254D44805B8E0580FCFD3DAA5E6685C00C'        
||'927FCE4FF301D169D86E6D8F881A3507CD53FCADABF6F5AEAFCC9EB2C86D3ED4ABA79F6FED3A3C94390DE1B53215DB23686376BD266BFB83BB6A76D5'        
||'3646B61C54117964644D68D282D18631AB32A82545C4F6D72A93C0BA7556B5E248F53E7D19ED49771ABADD3EBAE965CE002082070A647E7C7A2AB88C'        
||'4D6D26629E4FB2A904EDFEC8F57826B81FE8B33771F4FCDF8F69715E3FEAA1E8D47CBFD5C3A14361ED8DC993EA0EDECD515551D1E236DD0F524F9DC7'        
||'562D4264F2E994DFFF00678B5C3C4A34B3413832CE1BE91F239F7577412221C96AD3D05066BF68E1D6890180FB7FC1D233B3B1B553F50F40C9F67544'        
||'22F7AB9021958FAF7FE2145F8FF9B7C5FDD8915FF57CBAD362B5F9741375DED14DC9D8BB076F64F1905650E5776E1A0ABA2CAE253278EAC8D6A7CCB0'        
||'D5D0552347202CAA406520101AD71ED76D6914DB8C114942ACE0106847ECE8AF78675DAEE5A22CAC23620A92181A7104647574DD3FFCB7A97B37AFF1'        
||'BD932E5F67E2A8B76D6675B19B51BAE3195D4F88C5E273551818A9A591E38D5DA5FB579DD0C6CA3585D6C05FDDB70DEEDB6DBF96D7C1F128D4195000'        
||'F4A686AFCEBD2FE4CD8BFAC9B447705CC4AA3B9DA49DA47639E09222A815A0A790CE7A10731FCB036A4104134F075630A2A60911A1E9BDAD8E69C799'        
||'A56FBC30D2959A5258FAE4BB05B28B2A8F6EDBEEF6971DC6D50548E0507F823E8E2EF97ADAD1842D732B100E4997CBC87EB83F2CD49E90992F819D79'        
||'4069E9EB705D7350A0BC30FF00C62EC130A68F5EB6644920B2A9258B041C937E6E4FB3E55B568838B5561F6AE3FE33D05D96C9A76B7372E85479B4D9'        
||'F2E225FB38F489C87C29EB8C6C89514FB7B63D34B1CB659E8BAD36BD2CF0337A498A5148580B1B0208FC8BFB511D9C0E6A96E01C11429FCBB3A4B2CB'        
||'6118EEB97F307BA7C11EBFAFD20B37D15B23602B6F9A8DB3B7AB25D92137A51E4E3EBDDAE37262A5DA321CEC0D86CCBD2AC94CE5E156BC6E8750560E'        
||'A45FDA4E61DBA24D96EDBC304AC13107F4EA0F86C7F801FD99F43D3BB0DE5A9BF8196E24AF8D1F6F893107BC70D521FD871E441E8139BE68601A4983'        
||'54EFC7F5C805D7187EAC41FF0094CF78A5F4209268BD4F0DCC2C383C9FCBFCFD25B627C9ADBFB7C57AC87758155534B28FB44C7904461837935D4AF3'        
||'CFBF359863C0754837DF081EE7CFD9FE7E8C1EC4EFDD8BD97D97D7FB732F16E0930D9DDD9B7701928B331D00A29A9F23978E1682AF454BFECC85C249'        
||'C7D09E3D98ECD6111DDED84C17478B1EAFB350E99BEDE9A7B591033925180AD3CC1F9F57E8F7D6D7001D4410058020DAC00FA5BFA7BC993C7A8DFAF2'        
||'5F5AD85C96000B5EF73F423DF871EBDD6B93BEE3C3B7C81EDE38A9527A1C6E77338BC41D6AC9150536E2A889E3A6652478D4A845B7F640FEBEC59ED1'        
||'C7141CAD750C02918DC2EB4D3853B29FB3CBA38BB99A7BA47932DE0C7527F3AF5C7FE29F8FEBF4F726F4DF49ADE10453EDACA2CD6B2C49244C6C74CE'        
||'93AF88ADFF0037E3FD8FBBA3E860DE9D7BE9BEB0FD3D69ABCE95A79D698FF0F413AEF2D0AABFC3AFA1552FF756BE916BFF009BFF000F62C5B5AA83AB'        
||'F97FB3D470FDB232FA13FE1E854D8791FE358DAD9FC3F6FE1AE10E9F279757EC2BEAD565FEB6FA7B22DDAD2932F77E1F4F99F9F42CE5EDEFF765ABC3'        
||'E1EBD4D5AEAA7901E87A51E72A3F8561F2591D1E7FB3A569FC3ABC7E4B305D3AECD6FAFD6C7DA1B5B3D7728BAB89F4FF0067A32DC799BEA2C6583C1A'        
||'6A5A5757FD0BD047FE927FEACFFF00ABFF00F607D88FF767F4FF0097FB3D023EB3FA3FCFA90BD87A941FE136B8BDBEF7FEC17BD7EECFE9FF002FF67A'        
||'A9BEA1A69FE7FEC74A1DBDBBFF0089D54F17D8783C7009357DCF92FF00B816D6F1AFF5F65DB95A7D3C4ADAB554D3853CBED3D08F963FDD85E491FC1A'        
||'52BEBF880F974AD393BF1E1FADC7F9CFEBC7F4F64D5E86C36AA1AEBE1F2FF67A85FC3FFE6F73FD3C7FF1BF76D7F2E9778BF2EA0D4D77F0B714FE2F39'        
||'75136AD7E3B6ABA69D366FE9F5BFB5B6D67F54864D5A6869C2BFE51D05B7CE63FDD774B0783E26A40D5D54F323F84FA75928374C549590544F81C6E6'        
||'208998CD8CCB4B5E71F5713214912A062E5A69FD20964F1CCA75017D42EA5741B718A50E243515E005781F5A8FDA3A86BDE2E69FDE1EDCEE367E0F87'        
||'ACDBF76A2D4A5D40DC00539A5388E94F0EECEB3DEC2976EE4BAC22A69229E434D161B74578AAA68B5F91A0A4ADC924B54205B964479E545B90A83D98'        
||'AA5C541692B8E2547F9282BF90FB7AC1E74452CD09A93424134AFA9CD7EDE27ECE92F90E9BD9191C85645B5F7B55E1AA23AA2F0D16EEA487298C90B0'        
||'FD9853746D90C43A9363E4C7D87F6981E3DE9BEAC11D8AE3FA274B7FBCB63FE35D3D03C0632676208E04519318CB01FE003E743D25F71F44768E1E78'        
||'E857094D9A112F99DF6BE73139B324721BB4AB43492FDD8241BE87A6561C5C0F745B98972F54FF004C08FE7C3F9F4E50C874266868699FE42A7F9749'        
||'25C56E8DBF501F218FCCE0EA216658E3CBE3AA28AA12CA4E9105620275716E3E87DA849239949521BECCFF0083A6DA39229405054E00AD071FB7A938'        
||'5CE545564754CF1D1548935ACF1BCF43331D602B48D09B024702C05C13C7BF2D6B55E1D38EE8158381AA878F9FE7FE6A742164AA2BE2A3F3647C990A'        
||'636F1CD3BD0D554A2B0D5CD4CF1B4A47D47A80FF005FDD8960D5A57F97F83A6962468C1614A71A640FDBC3A464151B7EAAA523A89053AA4A8CA3EE24'        
||'8844E58FA74CD0B445BEBCAC8BF5FA8B7BB281A73D6A5244A1714C799FF0007F953A5C2D12C4EEF87CC607231B0493ECB2B22E26AD8E9D0891644493'        
||'50B9E05B5D5C66FF008FEBAA80050FECFF00352BFC8F57732692CC8401E7E54F5A93FC8907D3A7CAFCDA0C053E0375EDFCFE1BED6A4CF8DDC1454FF7'        
||'691C322789E8E2AD76103D313FB9A21A8B5C061C7D6D1C91B1A3509F4AF9FAD38D7ED1D32ED33AF8B6ED8029503CBCF271F3E3F9F50296836DC74ED9'        
||'293712E4A17D4AA8EB2D356E868C8D0F23111A006D60ACE48E3FC7DDCF0FF57FC5FF0083AA055AD5F20F99E23E5E43F99F9F4673E2E5151662B37947'        
||'454B5D53353C7809A85EA2767A0154B2D49304B24765679220E967FAC61D74BAB32FB0CF31CBE0A44D5017B837AD3191F2069F9D0D460F432E4F0AED'        
||'7016AC404A1F2E2D8FF57CFA3A3FC33114981DD52D719050A6268B250454713D37A2A774E3E9721446298E865A5626A5639B4A89522628E617002625'        
||'9DEE6158BE3D454D73C2372A7D6ADF0D4674922A3503D0F5553C3766E14AFA7165041F5A64E781031823A6AC1E28E3EADA0AF42D594556AE29A829E9'        
||'87DCD4D2C4F33912BC123B0ACA496ABC37D5248248E1554462C8FDECC258C4917C2E29524E0134E1A8534B05AF0028CD52450B76E9A18C727143E5FF'        
||'0014788AD3CCE070E175DF09768536D7EBBEC9DF55B226DEC7E6BC586C7E5735253D0528C6D053BC990C9BD4B1002F9651EA67243259F532B138E5EE'        
||'2DF4979BB59ED91032BC757654058EA6234AD3CF03C87038A023A97B936D52D36EBADC266112B8D01988028064D7F3FDB839E911F217E4D74764F70E'        
||'2EB3696DE8FB6329B7ABF07D7F439ACCEB8FAA76664B715498F135D96C72149725E47A8811010D4FFB910E4306534E53E4DE63B5B374DC66FA14983C'        
||'CC91E6EA5541DCAAD91101424F07C1FB3A43CC3CC5B3DD5D23D943F52D1011AB49510A5781D183213514AF6F0C1E890EFAC9769F715654536F7CED76'        
||'4B271C5BFF006751E2E8A2A0A6DBFB5F786D9C8C1BA3AEF31B330B05A9F1D154E3A9F4491AB1676A9814BB0B83236DF6FB3EC710FDDD1AC719F06566'        
||'258BC91480C73095CF74855CF138011B03A07DCDC5F6E526ABB73238D4A0634A32F72E851DA80AF90F55CF4CF85A0C6D2C516422A64A384426591E38'        
||'29DE831D4D1CF1EE996A64AC4BEA99680656082303EAD2580B100C2F5E59085AEA271C4EA66A18E801E03C4F0989F90E8B2CC052CBFC26B5A0A0186A'        
||'D7CC85D600FB7A8FBA71DE1C6352D409A8DE10CD0E2211475B91AEAEC4CFF712D565AAA3FCBD68995D579F40B5C8E3561356E04A946048AB9A855561'        
||'40147C9294FB734E9FB88E9118DAA0D0F6E092466AC7EDA83F6746E7E2166EAB0797DBF4953217A45CFEF7828E200C463353B28D55251B450A8D4C0D'        
||'7546900950A783AB9F603F706CD27B79E48C50E8849F3E13509A9F2EC5F9D7E5D0A7946ECC53C2AE704CA07FCE3C0FB7B8F4717E50BD3D2D4ECBA39E'        
||'6511E3B00F2BEA9EA226614750A032A44AD70EB4EE0EA37FF5FDC73C9219E3B875197929C07983FE02C3A19F3515592043C113CEBE447F8403D145A4'        
||'A7A602568929DE66A79215F254654A9923865A60404B0E7ED16E3FD8FF00AE3E91DC90189A56BC178541FF009F8F40F550016502B4A79FA1FF00A07A'        
||'74991A41305A5A05B544A458E6A4B04A8A9D245B91C4A0DBDB2A460966E03F83D17FCDD38578800649FE2F56FF003F4F789C5896087C94F4614332B0'        
||'14D9A26E6B69E50071F8083F3ED89E6D2E684FED4F461FE5E9D8A2250023F937A83FE4E871F8F542947D8B4F50638236A8C3641078A9F23192AB8F86'        
||'572ED54748F5331B5AF7FA7E7D8639B65326CCC95268EBE6A7F111C07D9D1EF2DC6137556A50946F23FC23D7A2DDF26AB6BF25B3FB96B162969A7DBD'        
||'F2021C147137859AAA831586A79669915D4E902A31E1D5873E323F171EC6DC930C516E5B6A543096C0C95C8A333B507EC7A7DBD0679A1DA5B3BFD5DA'        
||'52EB47E5A40FF0AD7ECEABE2931538A5A1A78701455B4992A4AD867C855AC6B357536429DA41245E41E458E39144F168171774B922FEE50926469496'        
||'94AB2953A470041E07CAA41D26BF234E81AAAD1C24E80C28727CF151FCC571F67423F5F0DF7D79BA2BF7A756D59C7E46A2BF1D88C7E37EF85362F279'        
||'0A7C861B6A51E0EB699119742C796CAE4A44507EE0C31C6E196223D94EF836DDCE2165BB2F889A49634A955D32C85C1AE0831A203F87512327A7F666'        
||'BBB58C5C5AB1492A4A9AF13545D047983A8923CE9F2E8EB6C9DCBD71DF3454915255603667676522ACC9E330F5B50B41B67B071EB9397138FDCF87A6'        
||'866618C6AC9A8D9A1847ECCA14682977558E770B4DDB96DDA4224B8B24A0660354B6E748668DCD3F50286CB7C40935AE0917DB3D8EEE046DA20BA352'        
||'BE514D9A065CFE9B123E1F84E294A9E9830981CBF59F696D6AFAFC71C55562F70E31F2147571E5249638259479DC331752AF11244FA8AB8F2BADFC83'        
||'DABB8BA8378D9278A27D6AF1B69234D2B4C7A1C1FC3C47683C3A490DB4DB56EB14B22E96475241AD695FE608F3E0684F9F5605DE9B521DD3D73DC3B6'        
||'9150B54E0B19BBE80BA0A85FBCA187C6D53E16E180FE19EB41F50DA89BC9EE27E58BF6B1DDEC2F0F93B44DE5863C3FEAA71F97CBA9337FB45BBDBAF6'        
||'DD7CD1651F68C57FEA9E47CFE7D6BA7DA54468282B667A2964AEC84821A51A2B68E20C647A4C551CA950A02B48C6AE678E5F4B164AA85CABAFBCB7D8'        
||'DC492A8D402C62A7E13E40B1A8F4ED008C8A1461507AC7CBF521700D5CD0711F203F3C9CF1AEA0723A4C6CCAC4AAD933EDAC7FDBD40D99D879E917CB'        
||'48E573152D84C361B1334F24AAD123BD7BD7CE2333958830490985EC546E0853701753547D4428307E11AA577028413D9A16B4AB52A3B875A8496B6F'        
||'0A2FC0E4E471A8555FE609E341E78E82DEC7DD1D4F4196ABE81DF5B8F72F5F673BD76ED06C1A2EC6DA7B5629F74D4E12B3215926E7DBB8ADD39CA7AB'        
||'A55964D3574AB1CC6410AD524F3233C6A0E3C7BBD21937AB47240D501200A540F11A9C29C4528283352304752E7B769A36CB8001ED9A9F6F62FF0096'        
||'BE7FE0E9D36D6D6DD5D335BDA3D4F4FDEB86EDBEAAC6E1CFF1ADE7366EBA2C8EC4DBF049F75062F726069E0A8C27DED0E38ABC755555C286332BC828'        
||'91D7ED9E2815EA403F6740EF497C56A7C1EECDE18FC5EE1DC5BEFE24EED7CC6ECCCD56FAADACA0EF5DCB9DCF542FF10C51DC9B562A0ADFEEF40E055C'        
||'55FF006B4120BAC5129A55479754F2EB75AE7A93D6D91DF9D7F926F8E9F1F325B6F6F7C66D9785AFCCD2FC854C5673B53274D92DD1BD2AA5A7DA482B'        
||'AA28B1F97AAA592A67126E496AF24C74C0B5E86A9C89755F25EB64799E3D65E94DAF9AEBAC9F67ED2DFDB47737C98EE9A7A8C864B72F7DEF7A2D95D7'        
||'985DC7B3F339291FAED76CD7765483F84432A88693F836168452415D4F3CF2BB6A599B631F3EBC7A9DB7B179CEDDDBBB77B57B2B7AED8F8DF94A6C96'        
||'E2A0D93D7734385DC52EC3A3AC95A1DC8BDA5D9B9B1534924B5712514B458EC7629A83EE90C5144E2269D7DC7E5D6B03E7D05F3F5B75F567C8CA5DE3'        
||'B3BBA7706D3C66FEC456ED0DC392ED4DC1B533504193DBB0AE7E1C8F5CD4D0560830E9978E7AA59D3232CA2111D310A59FC31FA99AD7AD571C3A7CEB'        
||'9C66CEF8FDB5B77EC5EA1EF3871349BFBB2F726FDCC6F4EE1836CEECDC199C8EE568E38E15CBD4D49A9CC42860861A349A196478C2D8A85057C318EB'        
||'64D7A476C0DCD5F8EDAFBA3B2BE627C99C549B2FA93B22589F3FB4F6E6DDC26D4A98B6CE6123D9B9596970B86C9E65AAEB281A478D283271B9ADD31D'        
||'5CB4A4D2563E8E324F5BC1E03A16AAF6BECBACDEFB3BE46EDFED5DE5B432188DB1BEE5CAEEFDC55D49B7B25B8BAEB3CB167EA7054516F39A3A9DBD43'        
||'3CE296AC5153E3A82A4495666A782A49995774F3EB5C31D25768D16E1DCBF28F6FE6373FCA5ED8ED1F8EB3ED76ADEB9DA53672965ADA5DE3419C8692'        
||'8E922CEF5AD3D057EE8C1C54B347546AFC3E37600C8CF116D5E1C78F5A3C294E993B632FD6B5DBE33347D5394C2CF86DBB50DB77700A4D9753B2CD36'        
||'F5A566AECF415988AEA7A29249F4D4D3C9256344DE7D61C4B27D7DCF9EDB15FEADF03FDB49FE04EB233DACD4395B813FAF2F0FB13A0ED669C1F4D750'        
||'7D2C4185383F9FF76FB1F76FA1EA45A9AE41FF0057E5D4D8AA2A548FF7258E16E7FCCC77FF000E0CBEEB453E47FD5F97546FF4A7FD5F974F14D5F540'        
||'5FF8962C580E5A08C9E3FC3CDEE8CABE87FD5F9749DBEC6FE5FE6E814EEDF9095FD6D150ED3D93161F78F6E6E981DF6E6DCFB376C66168351867DDBB'        
||'BAAE0988A7A1A73768A1244954EBE38C690EEBBB7B19EFEE16D2CD0B48DEA68AA3CD98D30A3F69381D03B9BF9BB6AE51DB1EFAFDE8D4EC424549FB00'        
||'AD3F99381E645792FDF50576EE6C76764DD9D99B884F90ED7EE3CA84610CCD07F946270E16D1451A27EC5352D3811C31D9557F0C350B65CBD68F6160'        
||'44972E3F5653E469FE4FC2830389CF50772E7286FF00EEB6E2DCCDCCC5A0DB90978E2382E545549E1FB2945E14AF4BDDF78FAEAECF645B21908F6BEC'        
||'CFE09B66BB70E7A171FC573025C144F160F1C13D60E900B30208D5C5BF52A85DB96EA76B9BA6090285D46B42C40C8AF9281C4F13C07AF45BCB9EE55E'        
||'F2E7278E52E5C84CBB9DC5C4C4100D1159800C4E735069E9E84D3A4B62E17CBCB994A44DBDB076AD074F762C1B6B179DA9A9A2AFC843F718D6A9CC54'        
||'2C104B29697C60333A0008B0B1F4FB24DF37086E5922B55D31449205E009AE9CD3C9718AE7AF6F9C89BA72E6CF6FBD6FD71E35F6E170A6504D748552'        
||'55789A658FF9493D27BAC7666D9DA9B571D95CFEF6C6F65621F315759B07696D0A892AE8F259AAFA7866CCD4D7412E868DA96B04B48C93A466F10959'        
||'543237B779561B1B5DB7F7ADE771D6C1169C581FC23F137F48F6A7DBD2CDC6FF009DB98845EDAF28A1894AEB9E6A90116435EE6FC200181C5AB806BD'        
||'2DF3272B93CED3D3564126E3EC8AB4BEDDDB18BAA929F1DB0607F453D7D4D652E958A68948F5102ED75286E62F6AEE27BDE60B9F0946064283DB18FE'        
||'266F5F9F13C1453A921795FDBAF68B93665DF824EF3A52567CC93B1C9082A5867208351824F0E8D9F5C6F2DBD9143B53B1F31B6EAFB9764BE3DB62E4'        
||'B10C4AD7E727A0776CD1337EDC6D8D1A8D5E39E47154C635506132E82FBEB6B8DBEE859B480B11F18C12A45082BC416C82A78AE7D298EB15BC973B71'        
||'E60B6B5956C6490AAF8AA4AD43554330A02280153835F9835042AF6E6EEC7E7F33B6DCAE0370E4A8A5C976676D57C38DA4A6AAC22CBF6BE2DA14B441'        
||'1618A5882A4C814156255FD07F754EDD6736EB747EA9CF650924D5987969A9E03D7E18C7A9A7534C9EEC72BF28724DBC3CAD67A6EDCF86B6E1490B2F'        
||'9BBB01DF5394A71E1C46139535BB7F29856C46165AADB3D29836924CC66E46F1673B1AB20064A9582598EA14F211FB92B120A9B9250FEE98EE3BAA2C'        
||'3F41B710B1A8A330C0A798078E9FE27E2E6A0633D3BEDCFB5DB86E3B89F70BDC5266BC90F891C52708EB90CE380A7E14F2F3EAFF003F9687C2EC9D23'        
||'50F7B7606D5FE05B966A4A7A5EB0DB359430B9EACDB5530254D3D7C18FAC42A775E4A9D836B9D3FDC45233545504AA99618A1EE63DF63923FA585BF4'        
||'471A60CA47AFA440F90F8CE06013D08F9F79D24BC7FDC3B2BF601591FCBF3A7E1FE14FC669F8466F46936163F6E7DCE2E9F158BAA5869E0592714D4D'        
||'2450C3525AA66C7D14F326B7844B7792473AE690B4925C9F71FCF7B2DCB091988F419C53E5FEAA0C0EA35B231C310646605892C493A9DBF89F3C48E0'        
||'061460741A6EDDB78DF0CFA3194348B2AA2CED4B1524492471542D42C5378B412BAD15CA9E0D85C1F6FC12B13924D3A11DA5CCBA85589FB49F439FB7'        
||'CBA213F23F66E2B7A7516F2C16F3865A8DB347BE360CB4DB5456474D87DC150F8EABACAB7960087C92524914535957FCDB91AC0523DC83C8D3B5B734'        
||'D9DC404091566219943D0F86CA304D321891E84569D43BF7A2BBB8B5F64F747B1D42493E995A44768A48D7EBADB848BDCA1C551A84545479F45576F6'        
||'0707B5E6C563F0D82C36128D9A90B26228A0A49963793C7E38AAE7596456080FA832FE081F5B4B7B85EDDEE36D34B3CCF2940D40E494A81FEFB4D094'        
||'AD3143E9D7296C6CE2B3BD8999151A468F5B27F6AC19B359A4F1252D4AF71607CC7C8C347B676DED1C867B1F83C553E3E0FE3195AD99CB4D555D555F'        
||'5B397A8C956E46B5A4A8A89E4B0D724929242A80422A85879771DCB75B4827BB94B9645C0A2AD3F842A80AA3D0018F4AF590D3EDDB5ED7B84F05AC21'        
||'044C5413DCE00C2F73558E38E726AC4E4F4FDB3328B06F5C6C9AACC7099B8ED7B2DAF112CBC1E38BD8FF00B7F6877FB466E5C985384D11FF008F74BF'        
||'9577054E6E8181CFD3CE3E5C533F9815E987BAF211C9BAF6ACDCB37951432E9D5CC72A28555E2D63C0FAFE6FECD3DBBB771B25EAFDBFE153D17FBA17'        
||'6AFCC76727F4467F6D29FE41C7A4E4B510BD3B18D4FA18824DCDC1200005C717B037F6689148B30D5E7FEAFDBE63A0DCB3C26DE899A1CFF219CF0F22'        
||'3CB1D4BA9F462A999C21D751655B8BDC444822DC9E78B8FF008D7BA4557BD651E4BFE51F97CFABCC746D885B8B3D0673F09A7954FA638678F0E86BEB'        
||'3AF5A7DB0D19172956785F538D74E86F637E07E3FD6B7B8E79C2DDA4DE357AAFE5C4FF0087A97B906F161E5FF0FF0085E9F3F847FA879F41EEE8AD59'        
||'37B6E595FD378B08C8069B12B42B61A41BD8FF005FC7B14ECF6ECBCB966ABEB2D7FDECF409E60BB57E6CBF67C621A0F5FD35F2FF005533D33C934416'        
||'A0B80A1A09748520F1E3662AA3EA6E7EA7F16FA5FDAE447AA01E4C2BFB47FAA9D15C9346AAECC3054D3F61C0F5FF0038EA62D4A3468CA2CABE30A2F7'        
||'31B347728D70383FF15F6C9898310789AFE62BFE1E9409D4C618600A0FB0D3FC1FECF1E8DB624B45B1F1456EA22A6A59640353DD3CEBA83702DFD78F'        
||'C0F70E6E146DEE6AF9B11FCBCBAC91DB18272B5B05C694563E7E7E7E9FEC745232CE83215FA58B2B56D5302A49521A6637FF001FF1E3F1EE62B35736'        
||'F1D46422FF0083AC70DC65896F26D2705DCD6BFD23FE5FF074CD989DCE2AA592CA238E2D214DF53FDCA0F57D3FD6BFB3CE5EB74937B862933A89AFD9'        
||'A58E3ADDBEE335A491DE5B90A63D447A54A919E1E58E907E5ACE64B3006C357D02961753F4FC916B9FAFB9812D6D1691E2BE9C787FABF2E944FBDEF5'        
||'36BBA0480715000001FB3D4D295E3E5D71D324977672C5BEAC4F248E09F57F5E47D78F6B515100550283A0DCF34F34AD24ACC598D49278F9FEDFB7FC'        
||'9D742025B5DD8950793E92B7FD40151EDCD402E9F5E9251DDF5EAA81C41C1F9D315F5C8F5E94D4F938E9920D38FA779102316A879250E6C2C19491F9'        
||'1C01F4FA7B2992C6498B6A958035F8401D0AADF7FB7B20822B546650BF1B31AFE44D71D34CDE5AA9A6AA6215E590BB69D3182C4DB80BC5BFC0702DEC'        
||'C11163884232145339E83D2CE64BB6BD383231614A0153E801028787A0E1D2D63DEB9A86969A0A778E28E962318B0B172C08B902DC01F81FEBFB0EBF'        
||'2ED94933C92549635FB3E5D48B1FB8BBBDB5AC3040AA163523871F2AE29C3FD9E84DDA1D9E9478DA84CFE4A459507F92C14B0BA30B1BB34B22F0CC4F'        
||'F5FC0F612DEF949E5B95FDDD1D57CD89FF0000F21FE7EA53E4EF74608B6E66DFE75492BDA8A0D69C4927CC9AFF002AF53AA7BCEAF1E0B61E79E7768C'        
||'3C5F72AA6CC49177E09FA7F4FF0089F696DF909A734BA1A479D3D3E5D186E7EF5D85AA6ADBFF0055A9DA0E335E07D31D37CDD9559BCE06837165CE3C'        
||'3012253C2EA12478D8044A97717D27F51506DFD7DAA1CAEBB34824B28CCA7854838AFA0E15F2AF4847B88BCD10B43B8CCB6838E9571DD43F88F100FA'        
||'7020E7A79C4F65619123A6A82852902C1132596E225E1845F95279040FC7B2EBDE57DC2A648EB57C9AFCFE7D1EED1EE2EC0E05BCEEA045DA082298F3'        
||'03CF852B91F3E977176961E94C329AFF00118D755872254917E92211C7F51FF14F61E9396F707254465B3FCC743B879D7618D7C4370AA05389F23F6F'        
||'536A7BDF6EABABAD7D4CCC6355685A34F09B2E960A00FADBF3EFD17266ECC3FB1A0F5F3E98BAF72B956334378A49F20453FD9FF0F48BABEDBC4CC647'        
||'8E78D4C841314A402EA4D8E91C9BD8F03D9A45CA9B881468CE3CE9D11DC7B89CBAAD55B9525B396031F21E6475AFFF00F340CDC59DF913B0EA69AA04'        
||'8B0F546D581D63661A5C6FDCFC84B0536170C069B585BFC481919ED758BEDFCA9710CCBA4B5C4CD423C8C108AFF23D42DCEBBD41BDF345ADD5ACBE22'        
||'AA4284A9C022791A84034068C2BD016546A3E85FA9FEC0FEBFEB7B422B41C7AC9B60351C0E3E83A19F6229FEEFAD947FC0EAD3C2DBFB4BEC21BE7FB9'        
||'DFED57FCBD4B9C95FF0024214FF7E49FE11D3D676E31EFF507CB07D2EA7F573C8FEBED36DDFEE50FB0F4AB9AFF00E48EDFE9D3FC27AE5D7DBD72BD77'        
||'BCF6D6F7C2DA5C86DBC9C3908E9E577582B6000C55B413329B849E16922247D355F9B7BA736F2D58738F2D5EF2BEE5510DEC4632C002C8D82922FCD1'        
||'C2B01E74A79F412E55E62BEE52E61B3E64DBF32D9C81C29268EB90E8D4CD1D0B293E55AF9757A1B33BBBACF7C62A97278FDD189C5CF3D3C73556173B'        
||'5F4D8BCAE36578C49253CF155B22C812F612C259180B83F81CBFE68F69B9FB953707B1BDDBA6B840C424F6F1BCD0CA2B40CA5012B5F3470ACA70479F'        
||'5D25E59F7539179A6C12F2D37086072017867912296334CAB2B90180F2642CAC320F900FBB5FBB76B1C065B6D6CFCCD167F2F9189F195D5B8C996AB1'        
||'D88A4A98CAD55EB52F1C93C91928891960B72CC41001933DA2F66798A7DFADB9979AAD5ECACED584B1C72A94967950D63FD33DC912B00CCCE06AA055'        
||'04124467EEE7BC5CBF6FB15C72DF2BDCA5E5E5DA98A492221A282371490F883B5A4652555549D352CC410016EEAFECFC1E071F88C3E46A0A52CB8AC5'        
||'C12BC68D2FF0EABA7A7F089248A3BB088AAFEE102E0D8D88BFB9D7DEFF0067F74E7FDA2D37EE5E50FB8D92B2F86485FA885CEBD0ACD402547AB46188'        
||'560CCB5069D6157DDFBEF07B47B45EE1EFDCABCE2E5365DD6E8C9E32AB3FD1DCA7E9F88E8A0B18258E8B2950590A23E92BABA359B7D1B76368DAE06E'        
||'02080ED8822B6388B7D04F2C24AC67FE0E45BF3EF0826E47E73B7BCFA0B8DA6EE39AB4D2F048A7F6950B4F9D69F3EBA45FEBB9ED5FEEB3BD2F326DAD'        
||'6A057C45BC85B14AD34AB17D5FD1D3ABCA95E8C26D5E80A8CA24FF00DF0AA8A3A59A9EDFC3B17A27A98AE4331AAC8CAA638F8FA88D1C7F57F7957ED4'        
||'72ECDEDE6CF2CF7343B8DEB29973558A34AF8710A7C4D566676AD351016A16A7987F798F77ACFDE9E68B6B3D815C6CDB4ABADBB30D2F7134A57C6B82'        
||'A455148454895806D0A59802FA43C51E63E3BFC7FA6A9A2DB18EC53E61E4925A8A4DB91C79ACCCD56E2CDF7D99A9764A7249234F9940FC47EE43DC77'        
||'7DCF767ADF4C481E4C6807D8A3FCBFB7AC78B1DAA08068894823D3248F9B1FB4F9FE5D025BCFE51EF8CC7DCD2ED88E9366E389244F4856B338621F97'        
||'C9D48D1091F5260894FF00B57B227954B69442E7867857EC1C7EC35E8450592448B23B690466B4C7E7D57E765FC92D83B5B253FF0078B734FB8F734C'        
||'419A928EA24CBE459A4360D5990958C712DCFF006E527EB653EC63B27B79CD5CC604A9118213F8A4AA8A7C97E238F41E9D11EF1CEBCB5CB9DB712076'        
||'C9A2D19BF33F69E903076DE5F725354D4CA94985C3D463A69A9A5A1AB91ABE02558C6D3D538F4BDB4D8228049FAF1EE4BDB7DB8D9F68753356EA6520'        
||'7781A3F251C7F3278740CBFE7ADC3708AB6E3E9E371EB56A53CCF9571D5697C8BEC9A1DC392AA6156F25763658F1F4BA0EA7A88A11A6592A2726FF00'        
||'5E45BFC7DCF7CB3B2B5B44A02854615E1403D001D6347B8FCDB0CF1BDB8909951E8057E23E678FCEBD1379F2F9095C93572850C485121D239B8B01EC'        
||'6C96D6EA29A07ECEA0EFDE37EC7578AC3EC271D2936C656B17294C65A86952594238247258695B83F5FAFB47B859C4D68DA140A0E841CB9BDDDA6E8B'        
||'E3B960E456B4A9C63ADA53E25EF2DB7F18BA2BE2DEE9C8EB79BBAFFD2050EF1C4D569AAA2DC1B74E4F5D1914B517469A089BF611814955DE123F70DF'        
||'0AB9F6D2E77CE61DCBC05AA5A7879182869C719E3C7CC60F9759FF00C8B3AD972FD9BC8692481B48FE315F869C30BC38F983C7AAF5FE651F1671BD25'        
||'9C4ED1EAEC85366FA3FB3E56C9ED96A2A88EAE4D9D575D1FDCC9B5EB555BCA94CA7C871B5322FAA21E09089E170653F6A79AA5DF611B5EE634DE5B00'        
||'09A504AA38483CB5114D6A381EE1DAC3A8BBDDCD8A3DAE19373B2FEC26071FEFB241254F9E9AD7413C4769C8EA946BB2B5666910390A921B58952083'        
||'E9E6FEF22E2822080D38F5869717334B211AB00E29D0E3F142B249BE4974D2D4F9258CEFBC5B48A8FA599516477219AE2F61F5F60FF71D27FEA26EA2'        
||'C19526FA77D05D4B206C50B2A95247C83027D4752CFDDFD6CFFD7AB96DF72124900BE88C8B1B2AC856A6A1199595588F32AC07A1EB6405AE801D54F4'        
||'1021FC3D433D430B7E46B213FD7F4FBC166D8373BAC6EDBACECBE696CA96A87E45903CE47FCDD1D767BFACDB3D91FF0074BB2DB230E125DB497B20A7'        
||'984731DB03FF00360F521B213540D324EC40E7C6A047128FF044B0FF0078F6AB6FE59D8F6A93C7B2B5412FFBF5819253F332C85E427FDB748774E6FE'        
||'64DEA2FA6DC6F6430F942A44508F9086209101E9DBD742560AA351D1EB2AB7FA03F52A3E9CFE7D9D1AB1A9C9E83A1428A2E075FFD9'                      
;                                                                                                                                   

-------------------------------
    V_Count pls_integer;
    lob_loc        BLOB;
    lob_loc_back   BLOB;
    amt            BINARY_INTEGER ;
    pos            INTEGER := 1;
    active_cnt     INTEGER := 0;
begin
--  Insert Logo Image
       
    SELECT count(*) 
      INTO active_cnt
      FROM CF_BUSINESSLOGO
     WHERE BUSINESSLOGONAME = 'active';

    IF active_cnt = 0 THEN
       amt := length(BRANDINGLOGOIMAGE_1)/2;
       dbms_output.put_line(' Logo Image Length '|| amt);

       INSERT INTO CF_BUSINESSLOGO ( BUSINESSLOGOID
                                   , BUSINESSLOGONAME
                                   , BRANDINGLOGOIMAGE
                                   , BRANDINGBACKGROUNDIMAGE )
                            VALUES ( 1
                                   , 'active'
                                   , empty_blob()
                                   , empty_blob() )      
              Returning BRANDINGLOGOIMAGE
                      , BRANDINGBACKGROUNDIMAGE 
                   INTO lob_loc
                      , lob_loc_back;
 
       dbms_lob.write (lob_loc, amt, pos, BRANDINGLOGOIMAGE_1);
       -- BackGround Image
       pos := 1;
       amt := 16000;
       DBMS_LOB.WRITE (lob_loc_back, amt, pos, BRANDINGBACKGROUNDIMAGE_1 );    
       pos := Pos + Amt ;
       DBMS_LOB.WRITE (lob_loc_back, amt, pos, BRANDINGBACKGROUNDIMAGE_2);
       pos := Pos + Amt ;
       DBMS_LOB.WRITE (lob_loc_back, amt, pos, BRANDINGBACKGROUNDIMAGE_3);
       pos := Pos + Amt ;
       DBMS_LOB.WRITE (lob_loc_back, amt, pos, BRANDINGBACKGROUNDIMAGE_4);
       pos := Pos + Amt ;
       DBMS_LOB.WRITE (lob_loc_back, amt, pos, BRANDINGBACKGROUNDIMAGE_5);
       pos := Pos + Amt ;
       DBMS_LOB.WRITE (lob_loc_back, amt, pos, BRANDINGBACKGROUNDIMAGE_6);
       pos := Pos + Amt ;
       DBMS_LOB.WRITE (lob_loc_back, amt, pos, BRANDINGBACKGROUNDIMAGE_7);
       pos := Pos + Amt ;
       amt := length(BRANDINGBACKGROUNDIMAGE_8)/2;                         -- adjust for length for the last variable
       dbms_output.put_line(' BackGround Image length '||( 16000*7 + amt));
       DBMS_LOB.WRITE (lob_loc_back, amt, pos, BRANDINGBACKGROUNDIMAGE_8);          
    END IF;

    EXCEPTION
        WHEN Others then 
        raise_application_error(-20999, 20000||'-'||'Failed'||Sqlerrm);
        
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 


-- pr_WorkFlow.sql

-- PR_WORKFLOW

--select * from pr_workflow
INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'PassThru',
'com.ibm.bcg.passthrough.No_op|com.ibm.bcg.outbound.OutboundDocFactory',1,
'user-exit.variableWorkflow.PassThru'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'PassThru')
/


INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'HubOwnerCancellationOfRNProcess',
'com.ibm.bcg.server.pkg.UnPackagingFactory|com.ibm.bcg.validation.ValidationFactory|com.ibm.bcg.outbound.OutboundDocFactory',1, 'user-exit.variableWorkflow.HubOwnerCancellationOfRNProcess'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'HubOwnerCancellationOfRNProcess')
/


INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'RNPassThruWithProcessLogging',
'com.ibm.bcg.rosettanet.passthru.ProcessLoggingFactory|com.ibm.bcg.passthrough.No_op|com.ibm.bcg.outbound.OutboundDocFactory',1,
'user-exit.variableWorkflow.RNPassThruWithProcessLogging'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'RNPassThruWithProcessLogging')
/


INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'BiDirectionalTranslationOfRNAndRNSC',
'com.ibm.bcg.server.pkg.UnPackagingFactory|com.ibm.bcg.validation.ValidationFactory|com.ibm.bcg.translation.protocol.StdRNandRNSCProtFactory|com.ibm.bcg.sponsor.SponsorBusProcessFactory|com.ibm.bcg.outbound.OutboundDocFactory',
1, 'user-exit.variableWorkflow.BiDirectionalTranslationOfRNAndRNSC'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'BiDirectionalTranslationOfRNAndRNSC')
/

--new for 43564
INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
PR_WORKFLOW_SEQ.Nextval,
'BiDirectionalTranslationOfRNAndRNSC Without Content validation',
'com.ibm.bcg.server.pkg.UnPackagingFactory|com.ibm.bcg.validation.ValidationWithoutContentFactory|com.ibm.bcg.translation.protocol.StdRNandRNSCProtFactory|com.ibm.bcg.sponsor.SponsorBusProcessFactory|com.ibm.bcg.outbound.OutboundDocFactory'
,1
,'user-exit.variableWorkflow.BiDirectionalTranslationOfRNAndRNSCWithoutContentValidation'
From DUAL
Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation')
/

INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation',
'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory|com.ibm.bcg.server.pkg.UnPackagingFactory|com.ibm.bcg.validation.ValidationFactory|com.ibm.bcg.translation.protocol.RNXsltProtFactory|com.ibm.bcg.validation.OutboundValidationFactory|com.ibm.bcg.sponsor.SponsorBusProcessFactory|com.ibm.bcg.outbound.OutboundDocFactory',                       1, 'user-exit.variableWorkflow.BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation')
/
 
INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'BiDirectionalTranslationOfRNAndXML',
'com.ibm.bcg.server.pkg.UnPackagingFactory|com.ibm.bcg.validation.ValidationFactory|com.ibm.bcg.translation.protocol.RNXsltProtFactory|com.ibm.bcg.validation.OutboundValidationFactory|com.ibm.bcg.sponsor.SponsorBusProcessFactory|com.ibm.bcg.outbound.OutboundDocFactory',1,
'user-exit.variableWorkflow.BiDirectionalTranslationOfRNAndXML'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'BiDirectionalTranslationOfRNAndXML')
/


INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation',
'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory|com.ibm.bcg.validation.ValidationFactory|com.ibm.bcg.translation.protocol.translators.xslt.XSLTranslationFactory|com.ibm.bcg.validation.OutboundValidationFactory|com.ibm.bcg.outbound.OutboundDocFactory', 1 ,
'user-exit.variableWorkflow.BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation')
/


INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'CustomXMLPassThruWithDupCheckAndValidation',
'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory|com.ibm.bcg.validation.ValidationFactory|com.ibm.bcg.passthrough.No_op|com.ibm.bcg.outbound.OutboundDocFactory',  1,
'user-exit.variableWorkflow.CustomXMLPassThruWithDupCheckAndValidation'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'CustomXMLPassThruWithDupCheckAndValidation')
/


INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'CustomXMLPassThruWithDupCheck',
'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory|com.ibm.bcg.passthrough.No_op|com.ibm.bcg.outbound.OutboundDocFactory', 1,
'user-exit.variableWorkflow.CustomXMLPassThruWithDupCheck'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'CustomXMLPassThruWithDupCheck')
/


INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'BiDirectionalTranslationOfCustomXMLWithValidation',
'com.ibm.bcg.validation.ValidationFactory|com.ibm.bcg.translation.protocol.translators.xslt.XSLTranslationFactory|com.ibm.bcg.validation.OutboundValidationFactory|com.ibm.bcg.outbound.OutboundDocFactory',1, 'user-exit.variableWorkflow.BiDirectionalTranslationOfCustomXMLWithValidation'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation')
/


INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'BiDirectionalTranslationOfCustomXML',
'com.ibm.bcg.translation.protocol.translators.xslt.XSLTranslationFactory|com.ibm.bcg.outbound.OutboundDocFactory',  1, 'user-exit.variableWorkflow.BiDirectionalTranslationOfCustomXML'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'BiDirectionalTranslationOfCustomXML')
/


INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'CustomXMLPassThruWithValidation',
'com.ibm.bcg.validation.ValidationFactory|com.ibm.bcg.passthrough.No_op|com.ibm.bcg.outbound.OutboundDocFactory',   1, 'user-exit.variableWorkflow.CustomXMLPassThruWithValidation'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'CustomXMLPassThruWithValidation')
/



--new work flows for 4.3
INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF,ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'EDIDeenvelope',
'com.ibm.bcg.edi.business.process.EDIDenvFactory|com.ibm.bcg.outbound.OutboundDocFactory',
1,
'user-exit.variableWorkflow.EDIDeenvelope'
From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EDIDeenvelope')
/

INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF,ENABLEDFLG, RESOURCEKEY )
 Select
PR_WORKFLOW_SEQ.Nextval,
'EDIvalidationAndEDI Translation',
'com.ibm.bcg.edi.business.process.EDISourceValidationFactory|com.ibm.bcg.edi.business.process.EDITranslatorFactory|com.ibm.bcg.outbound.OutboundDocFactory',
1,
'user-exit.variableWorkflow.EDIvalidationAndEDITranslation'
From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EDIvalidationAndEDI Translation')
/

INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF,ENABLEDFLG, RESOURCEKEY )
 Select
PR_WORKFLOW_SEQ.Nextval,
'XMLTranslationAndEDIValidation',
'com.ibm.bcg.edi.business.process.XMLTranslatorFactory | com.ibm.bcg.edi.business.process.EDITargetValidationFactory | com.ibm.bcg.outbound.OutboundDocFactory',
1,
'user-exit.variableWorkflow.XMLTranslationAndEDIValidation'
From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLTranslationAndEDIValidation')
/

INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
PR_WORKFLOW_SEQ.Nextval,
'RODTranslationAndEDIValidation',
'com.ibm.bcg.edi.business.process.RODTranslatorFactory|com.ibm.bcg.edi.business.process.EDITargetValidationFactory|com.ibm.bcg.outbound.OutboundDocFactory',
1,
'user-exit.variableWorkflow.RODTranslationAndEDIValidation'
From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'RODTranslationAndEDIValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'EDI Validation',
--1,
--'user-exit.variableWorkflow.EDIValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EDI Validation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'EDIValidationEDI TranslationAndEDIValidation',
--1,
--'user-exit.variableWorkflow.EDIValidationEDI TranslationAndEDIValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EDIValidationEDI TranslationAndEDIValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'EDIValidationEDI TranslationAndXMLValidation',
--1,
--'user-exit.variableWorkflow.EDIValidationEDI TranslationAndXMLValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EDIValidationEDI TranslationAndXMLValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'EDITranslation',
--1,
--'EDITranslationuser-exit.variableWorkflow'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EDITranslation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'EDITranslationEDIValidation',
--1,
--'user-exit.variableWorkflow.EDITranslationEDIValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EDITranslationEDIValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLValidationXMLTranslationAndEDIValidation',
--1,
--'user-exit.variableWorkflow.XMLValidationXMLTranslationAndEDIValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLValidationXMLTranslationAndEDIValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLValidationAndXML Translation',
--1,
--'user-exit.variableWorkflow.XMLValidationAndXML Translation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLValidationAndXML Translation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLValidationXMLTranslationAndXMLValidation',
--1,
--'user-exit.variableWorkflow.XMLValidationXMLTranslationAndXMLValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLValidationXMLTranslationAndXMLValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLValidationWithXMLDupCheckXML TranslationAndEDIValidation',
--1,
--'user-exit.variableWorkflow.XMLValidationWithXMLDupCheckXML TranslationAndEDIValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLValidationWithXMLDupCheckXML TranslationAndEDIValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLValidationWithXMLDupCheckAndXMLTranslation',
--1,
--'user-exit.variableWorkflow.XMLValidationWithXMLDupCheckAndXMLTranslation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLValidationWithXMLDupCheckAndXMLTranslation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLValidationWithXMLDupCheckXMLTranslationnAndXMLValidation',
--1,
--'User-exit.variableWorkflow.XMLValidationWithXMLDupCheckXMLTranslationnAndXMLValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLValidationWithXMLDupCheckXMLTranslationnAndXMLValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XML Translation',
--1,
--'user-exit.variableWorkflow.XML Translation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XML Translation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLTranslationAndXMLValidation',
--1,
--'user-exit.variableWorkflow.XMLTranslationAndXMLValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLTranslationAndXMLValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLDupCheckAndXMLTranslationAndEDIValidation',
--1,
--'user-exit.variableWorkflow.XMLDupCheckAndXMLTranslationAndEDIValidation'
--From DUAL
---Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLDupCheckAndXMLTranslationAndEDIValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLDupCheckAndXMLTranslation',
--1,
--'user-exit.variableWorkflow.XMLDupCheckAndXMLTranslation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLDupCheckAndXMLTranslation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLDupCheckWithXMLValidationXMLTranslationAndEDIValidation',
--1,
--'user-exit.variableWorkflow.XMLDupCheckWithXMLValidationXMLTranslationAndEDIValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLDupCheckWithXMLValidationXMLTranslationAndEDIValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLDupCheckWithXMLValidationAndXMLTranslation',
--1,
--'user-exit.variableWorkflow.XMLDupCheckWithXMLValidationAndXMLTranslation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLDupCheckWithXMLValidationAndXMLTranslation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'XMLDupCheckWithXMLValidationAndXMLTranslationWithXMLValidation',
--1,
--'USER-exit.variableWorkflow.XMLDupCheckWithXMLValidationAndXMLTranslationWithXMLValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'XMLDupCheckWithXMLValidationAndXMLTranslationWithXMLValidation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'RODTranslation',
--1,
--'user-exit.variableWorkflow.RODTranslation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'RODTranslation')
/

--INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, ENABLEDFLG, RESOURCEKEY )
-- Select
--PR_WORKFLOW_SEQ.Nextval,
--'RODTranslationAndXMLValidation',
--1,
--'user-exit.variableWorkflow.RODTranslationAndXMLValidation'
--From DUAL
-- Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'RODTranslationAndXMLValidation')
/

--CR60181
INSERT INTO PR_WORKFLOW (WORKFLOWID, WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
PR_WORKFLOW_SEQ.Nextval,
'EBMSSplitAndParse',
'com.ibm.bcg.server.EBMSSplitAndParse|com.ibm.bcg.outbound.OutboundDocFactory'
,1
,'user-exit.variableWorkflow.EBMSSplitAndParse'
From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EBMSSplitAndParse')

/


INSERT INTO PR_WORKFLOW (WORKFLOWID,WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'Validate SOAP Envelope',
'com.ibm.bcg.validation.soap.WebserviceFactory|com.ibm.bcg.outbound.OutboundDocFactory',1,
'user-exit.variableWorkflow.ValidateSOAPEnvelope'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'Validate SOAP Envelope')

/

INSERT INTO PR_WORKFLOW (WORKFLOWID,WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'Validate SOAP Body',
'com.ibm.bcg.validation.ValidationFactory|com.ibm.bcg.outbound.OutboundDocFactory',1,
'user-exit.variableWorkflow.ValidateWebservicePayload'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'Validate SOAP Body')

/

INSERT INTO PR_WORKFLOW (WORKFLOWID,WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'De-Envelope SOAP',
'com.ibm.bcg.validation.soap.SOAPDeEnveloperFactory |com.ibm.bcg.outbound.OutboundDocFactory',1,
'user-exit.variableWorkflow.SOAPDe-Enveloper'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'De-Envelope SOAP')

/

INSERT INTO PR_WORKFLOW (WORKFLOWID,WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
PR_WORKFLOW_SEQ.Nextval,
'EDIInterchangeValidation',
'',1,
'user-exit.variableWorkflow.EDIInterchangeValidation'
 From Dual
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EDIInterchangeValidation')

/


INSERT INTO PR_WORKFLOW (WORKFLOWID,WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
PR_WORKFLOW_SEQ.Nextval,
'WTXTransformation',
'',1,
'user-exit.variableWorkflow.WTXTransformation'
 From Dual
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'WTXTransformation')

/




INSERT INTO PR_WORKFLOW (WORKFLOWID,WORKFLOWNAME, WORKFLOWDEF, ENABLEDFLG, RESOURCEKEY )
 Select
 PR_WORKFLOW_SEQ.Nextval,
'EDI ReEnvelope Action',
'com.ibm.bcg.wtx.business.process.EDIReEnveloperFactory',1,
'user-exit.variableWorkflow.EDIReEnveloper'
 From DUAL
 Where Not Exists (Select 'x' From pr_Workflow where workflowname = 'EDI ReEnvelope Action')

/


-- pr_WorkFlow_loc_ENUS.sql

-- PR_WORKFLOW_LOC_ENUS

--select * from PR_WORKFLOW_LOC/

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Pass Through'                                                                 
 From PR_Workflow
 Where WorkflowName = 'PassThru'                                                                  
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Pass Through' And locale = 'en_US')
/                                                                   


INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                      
'en_US',                                                                        
'Community Manager Cancellation of RosettaNet Process'                                 
 From PR_Workflow                                                                      
 Where WorkflowName = 'HubOwnerCancellationOfRNProcess' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Community Manager Cancellation of RosettaNet Process' And locale = 'en_US')
/   


INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'RosettaNet Pass Through with Process Logging'                                
 From PR_Workflow                                                                      
 Where WorkflowName = 'RNPassThruWithProcessLogging' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'RosettaNet Pass Through with Process Logging' And locale = 'en_US')
/                                   


INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Bi-Directional Translation of RosettaNet and RosettaNet Service Content with Validation'     
 From PR_Workflow                                                                     
 Where WorkflowName = 'BiDirectionalTranslationOfRNAndRNSC' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Bi-Directional Translation of RosettaNet and RosettaNet Service Content with Validation' And Locale = 'en_US')
/        
                                                     

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 workflowId,                                                       
'en_US',                                                                        
'Bi-Directional Translation of RosettaNet and XML with Validation'                             
 From PR_Workflow                                                                     
 Where WorkflowName = 'BiDirectionalTranslationOfRNAndXML' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Bi-Directional Translation of RosettaNet and XML with Validation' And Locale = 'en_US')
/            

    

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Bi-Directional Translation of Custom XML with Duplicate Check and Validation'
 From pr_Workflow                                                                     
 Where WorkflowName = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Bi-Directional Translation of Custom XML with Duplicate Check and Validation' And locale='en_US')
/   

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Custom XML Pass Through with Duplicate Check and Validation'                  
 From pr_Workflow                                                                      
 Where WorkflowName = 'CustomXMLPassThruWithDupCheckAndValidation' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Custom XML Pass Through with Duplicate Check and Validation' and locale = 'en_US')
/ 


INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Custom XML Pass Through with Duplicate Check'                                
 From pr_Workflow                                                                      
 Where WorkflowName = 'CustomXMLPassThruWithDupCheck' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Custom XML Pass Through with Duplicate Check' and locale = 'en_US')
/ 

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Bi-Directional Translation of Custom XML with Validation'                     
 From pr_Workflow                                                                     
 Where  WorkflowName = 'BiDirectionalTranslationOfCustomXMLWithValidation'
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Bi-Directional Translation of Custom XML with Validation' and locale = 'en_US')
/

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Bi-Directional Translation of Custom XML'                                     
 From pr_Workflow                                                                     
 Where WorkflowName = 'BiDirectionalTranslationOfCustomXML'
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Bi-Directional Translation of Custom XML' and locale = 'en_US')
/  

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Custom XML Pass Through with Validation'                      
 From pr_Workflow                                                                     
 Where  WorkflowName = 'CustomXMLPassThruWithValidation'
 and Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Custom XML Pass Through with Validation' and locale = 'en_US')
/ 

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Bi-Directional Translation of Community Manager Custom XML to RosettaNet with Content Duplicate Check and Validation'                        
 From pr_Workflow                                                                     
 Where WorkflowName = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Bi-Directional Translation of Community Manager Custom XML to RosettaNet with Content Duplicate Check and Validation' and locale = 'en_US')
/

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Validate SOAP Envelope'                        
 From pr_Workflow                                                                     
 Where WorkflowName = 'Validate SOAP Envelope' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Validate SOAP Envelope' and locale = 'en_US')

/

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'Validate SOAP Body'                        
 From pr_Workflow                                                                     
 Where WorkflowName = 'Validate SOAP Body' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'Validate SOAP Body' and locale = 'en_US')

/

INSERT INTO PR_WORKFLOW_LOC (WorkFlowId, locale, WORKFLOWDisplayName)           
 Select                                                                         
 WorkflowId,                                                       
'en_US',                                                                        
'De-envelope SOAP Envelope'                        
 From pr_Workflow                                                                     
 Where WorkflowName = 'De-Envelope SOAP' 
 And Not Exists (Select * From pr_Workflow_loc where WorkflowDisplayName = 'De-envelope SOAP Envelope' and locale = 'en_US')

/



-- CF_ROUTINGATTR_GRP.sql

-- bcg.grp.ro.common.ACKNOWLEDGEMENT
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.common.ACKNOWLEDGEMENT'
		,110
		,'bcg.grp.ro.common.ACKNOWLEDGEMENT'
		,'bcg.grp.ro.common.ACKNOWLEDGEMENT'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
	)
/

-- bcg.grp.ro.common.COMPRESSION
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.common.COMPRESSION'
		,140
		,'bcg.grp.ro.common.COMPRESSION'
		,'bcg.grp.ro.common.COMPRESSION'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.common.COMPRESSION')
	)
/

-- bcg.grp.ro.common.ENCRYPTION
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.common.ENCRYPTION'
		,130
		,'bcg.grp.ro.common.ENCRYPTION'
		,'bcg.grp.ro.common.ENCRYPTION'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.common.ENCRYPTION')
	)
/

-- bcg.grp.ro.common.GENERAL
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.common.GENERAL'
		,100
		,'bcg.grp.ro.common.GENERAL'
		,'bcg.grp.ro.common.GENERAL'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.common.GENERAL')
	)
/

-- bcg.grp.ro.common.SIGNATURE
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.common.SIGNATURE'
		,120
		,'bcg.grp.ro.common.SIGNATURE'
		,'bcg.grp.ro.common.SIGNATURE'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.common.SIGNATURE')
	)
/

-- bcg.grp.ro.edi.DELIMITER
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.edi.DELIMITER'
		,310
		,'bcg.grp.ro.edi.DELIMITER'
		,'bcg.grp.ro.edi.DELIMITER'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.edi.DELIMITER')
	)
/

-- bcg.grp.ro.edi.GENERAL
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.edi.GENERAL'
		,300
		,'bcg.grp.ro.edi.GENERAL'
		,'bcg.grp.ro.edi.GENERAL'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.edi.GENERAL')
	)
/

-- bcg.grp.ro.edi.VALIDATION
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.edi.VALIDATION'
		,320
		,'bcg.grp.ro.edi.VALIDATION'
		,'bcg.grp.ro.edi.VALIDATION'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.edi.VALIDATION')
	)
/

-- bcg.grp.ro.rnif.GENERAL
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.rnif.GENERAL'
		,200
		,'bcg.grp.ro.rnif.GENERAL'
		,'bcg.grp.ro.rnif.GENERAL'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.rnif.GENERAL')
	)
/

-- bcg.grp.ro.ebxml.GENERAL
INSERT INTO CF_ROUTINGATTR_GRP (
   ROUTINGATTR_GRPID, ROUTINGATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_ROUTINGATTR_grp_seq.NEXTVAL
		,'bcg.grp.ro.ebxml.GENERAL'
		,400
		,'bcg.grp.ro.ebxml.GENERAL'
		,'bcg.grp.ro.ebxml.GENERAL'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_ROUTINGATTR_GRP WHERE ROUTINGATTR_GRPKEY = 'bcg.grp.ro.ebxml.GENERAL')
	)
/



-- cf_RoutingAttr.sql

-- CF_RoutingAttr

INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY, Validescription)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'TimeToAcknowledge',1, 116,1,7, 'TimeToAcknowledge' , 'TimeToAcknowledge', 'integer|minValue=1|maxValue=10080' From dual                                                                                                                                   
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'TimeToAcknowledge')
/                                                                                                              
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY, Validescription)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'TimeToPerform',1, 52,1,7, 'TimeToPerform' , 'TimeToPerform', 'integer|minValue=1|maxValue=10080' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'TimeToPerform')
/                                                                                                                  
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY, Validescription)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'RetryCount',1, 52,1,7, 'RetryCount', 'RetryCount', 'integer|minValue=0|maxValue=10' From dual                                                                                                                                           
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'RetryCount')
/                                                                                                                     
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'DigitalSignatureRequired',1, 52,1,7, 'DigitalSignatureRequired', 'DigitalSignatureRequired' From dual                                                                                                                             
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'DigitalSignatureRequired')
/                                                                                                       
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'Encryption',1, 52,1,7, 'Encryption', 'Encryption' From dual                                                                                                                                           
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'Encryption')
/                                                                                                                     
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'GuidelineMap',1, 80,1,7, 'GuidelineMap', 'GuidelineMap' From dual                                                                                                                                         
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'GuidelineMap')
/                                                                                                                   
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ASCompressBeforeSign',1, 4,1,7, 'ASCompressBeforeSign', 'ASCompressBeforeSign' From dual                                                                                                                                  
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASCompressBeforeSign')
/                                                                                                           
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ASCompressed',1, 4,1,7, 'ASCompressed', 'ASCompressed' From dual                                                                                                                                          
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASCompressed')
/                                                                                                                   
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ASEncrypted',1, 4,1,7, 'ASEncrypted', 'ASEncrypted' From dual                                                                                                                                           
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASEncrypted')
/                                                                                                                    
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY, Validescription)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ASMdnHttpUrl',1, 4,1,7, 'ASMdnHttpUrl' , 'ASMdnHttpUrl', 'URL' From dual                                                                                                                                              
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASMdnHttpUrl')             --new attribute
/                                                                                                                       
                 
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY, Validescription)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ASMdnEmailAddress',1, 4,1,7, 'ASMdnEmailAddress', 'ASMdnEmailAddress', 'email' From dual                                                                                                                                              
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASMdnEmailAddress')        --new attribute
/                                                                                                                       
                                                                                                                                                                                                                         
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ASMDNAsynchronous',1, 4,1,7, 'ASMDNAsynchronous', 'ASMDNAsynchronous' From dual                                                                                                                                     
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASMDNAsynchronous')
/                                                                                                              
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ASMDNRequested',1, 4,1,7, 'ASMDNRequested', 'ASMDNRequested' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASMDNRequested')
/                                                                                                                 
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                  --      ASMessageDigestAlgorithm                                                                                                                                                                         
 CF_RoutingAttr_SEQ.Nextval, 'ASMessageDigestAlgorithm',1, 4,1,7, 'ASMessageDigestAlgorithm', 'ASMessageDigestAlgorithm' From dual                                                                                                                                           
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASMessageDigestAlgorithm')   --new attribute
/                

INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ASMDNSigned',1, 4,1,7, 'ASMDNSigned', 'ASMDNSigned' From dual                                                                                                                                           
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASMDNSigned')
/                                                                                                                    
                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ASSigned',1, 4,1,7, 'ASSigned', 'ASSigned' From dual                                                                                                                                              
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASSigned')
/                                                                                                                       
                                                                                                                                                                                                                                                                                                 
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'NonRepudiationRequired',1, 116,1,7, 'NonRepudiationRequired', 'NonRepudiationRequired' From dual                                                                                                                              
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'NonRepudiationRequired')
/                                                                                                         
                                                                                                                                                                                                        
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'NonRepudiationOfReceipt',1, 116,1,7, 'NonRepudiationOfReceipt', 'NonRepudiationOfReceipt' From dual                                                                                                                             
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'NonRepudiationOfReceipt')
/   
--- V Goel 6/9/03                                                                                                     
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'GlobalSupplyChainCode',1, 2,1,7 , 'GlobalSupplyChainCode', 'GlobalSupplyChainCode' From dual                                                                                                                             
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'GlobalSupplyChainCode')
/  
--47766
--INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
-- Select                                                                                                                                                                                                 
-- CF_RoutingAttr_SEQ.Nextval, 'OneWay',0,32,1,7, 'OneWay', 'OneWay' From dual                                                                                                                            
-- Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'OneWay')
--/

INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval,'SyncSupported',0, 16,1,7,'SyncSupported','SyncSupported' From dual                                                                                                                            
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'SyncSupported')
/

INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval,'SyncAckRequired',0, 16,1,7,'SyncAckRequired','SyncAckRequired' From dual                                                                                                                            
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'SyncAckRequired')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval,'ASBusinessId',0, 16,1,7,'ASBusinessId','ASBusinessId' From dual                                                                                                                            
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ASBusinessId')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select
  CF_RoutingAttr_SEQ.Nextval, 'RNMessageDigestAlgorithm',1, 4,1,7 , 'RNMessageDigestAlgorithm', 'RNMessageDigestAlgorithm' From DUAL                                                                                                                                           
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'RNMessageDigestAlgorithm')
 
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select
 CF_RoutingAttr_SEQ.Nextval, 'RNEncryptionAlgorithm',1, 4,1,7, 'RNEncryptionAlgorithm', 'RNEncryptionAlgorithm' From DUAL                                                                                                                                           
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'RNEncryptionAlgorithm')
/

INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'EnvelopeFlag',1, 4,1,7, 'EnvelopeFlag', 'EnvelopeFlag' From dual                                                                                                                                   
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'EnvelopeFlag')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.EDISEGDLM',0,4,1,7,'bcg.ro.edi.EDISEGDLM','bcg.ro.edi.EDISEGDLM' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.EDISEGDLM')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.EDIDEDLM',0,4,1,7,'bcg.ro.edi.EDIDEDLM','bcg.ro.edi.EDIDEDLM' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.EDIDEDLM')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.EDISEDLM',0,4,1,7,'bcg.ro.edi.EDISEDLM','bcg.ro.edi.EDISEDLM' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.EDISEDLM')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.EDIRLSCHAR',0,4,1,7,'bcg.ro.edi.EDIRLSCHAR','bcg.ro.edi.EDIRLSCHAR' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.EDIRLSCHAR')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.EDIDESEP',0,4,1,7,'bcg.ro.edi.EDIDESEP','bcg.ro.edi.EDIDESEP' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.EDIDESEP')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.EDIDECNOT',0,4,1,7,'bcg.ro.edi.EDIDECNOT','bcg.ro.edi.EDIDECNOT' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.EDIDECNOT')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.SEGOUTPUT',0,4,1,7,'bcg.ro.SEGOUTPUT','bcg.ro.SEGOUTPUT' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.SEGOUTPUT')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.ALLOWDUPLICATES',0,4,1,7,'bcg.ro.edi.ALLOWDUPLICATES','bcg.ro.edi.ALLOWDUPLICATES' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.ALLOWDUPLICATES')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.TRANSFORMATIONERRLEVEL',0,4,1,7,'bcg.ro.TRANSFORMATIONERRLEVEL','bcg.ro.TRANSFORMATIONERRLEVEL' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.TRANSFORMATIONERRLEVEL')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.FAMAPNAME',0,4,1,7,'bcg.ro.edi.FAMAPNAME','bcg.ro.edi.FAMAPNAME' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.FAMAPNAME')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.EnvelopeProfile',1,4,1,7,'bcg.ro.edi.EnvelopeProfile','bcg.ro.edi.EnvelopeProfile' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.EnvelopeProfile')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.xml.XMLNSACTIVE',0,4,1,7,'bcg.ro.xml.XMLNSACTIVE','bcg.ro.xml.XMLNSACTIVE' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.xml.XMLNSACTIVE')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.VALERRLEVEL',0,4,1,7,'bcg.ro.edi.VALERRLEVEL','bcg.ro.edi.VALERRLEVEL' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.VALERRLEVEL')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.VALLEVEL',0,4,1,7,'bcg.ro.edi.VALLEVEL','bcg.ro.edi.VALLEVEL' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.VALLEVEL')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.VALCHARSET',0,4,1,7,'bcg.ro.edi.VALCHARSET','bcg.ro.edi.VALCHARSET' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.VALCHARSET')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.VALALPHANUM',0,4,1,7,'bcg.ro.edi.VALALPHANUM','bcg.ro.edi.VALALPHANUM' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.VALALPHANUM')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.GRPLVLFA',0,4,1,7,'bcg.ro.edi.GRPLVLFA','bcg.ro.edi.GRPLVLFA' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.GRPLVLFA')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY,Validescription)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.VALCTRLYY',0,4,1,7,'bcg.ro.edi.VALCTRLYY','bcg.ro.edi.VALCTRLYY', 'integer|minValue=0|maxValue=10' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.VALCTRLYY')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.VALSERVSEGVAL',0,4,1,7,'bcg.ro.edi.VALSERVSEGVAL','bcg.ro.edi.VALSERVSEGVAL' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.VALSERVSEGVAL')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.TA1OVERRIDE',0,4,1,7,'bcg.ro.edi.TA1OVERRIDE','bcg.ro.edi.TA1OVERRIDE' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.TA1OVERRIDE')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.DISCARDONERROR',1,4,1,7,'bcg.ro.DISCARDONERROR','bcg.ro.DISCARDONERROR' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.DISCARDONERROR')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.CONNECTIONQUALIFIER',1,4,1,7,'bcg.ro.edi.CONNECTIONQUALIFIER','bcg.ro.edi.CONNECTIONQUALIFIER' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.CONNECTIONQUALIFIER')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.INTERCHANGEQUAL',1,4,1,7,'bcg.ro.edi.INTERCHANGEQUAL','bcg.ro.edi.INTERCHANGEQUAL' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.INTERCHANGEQUAL')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.INTERCHANGEID',1,4,1,7,'bcg.ro.edi.INTERCHANGEID','bcg.ro.edi.INTERCHANGEID' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.INTERCHANGEID')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.ICHGUSAGEINDICATOR',1,4,1,7,'bcg.ro.edi.ICHGUSAGEINDICATOR','bcg.ro.edi.ICHGUSAGEINDICATOR' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.ICHGUSAGEINDICATOR')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.GRPAPPSENDERID',1,4,1,7,'bcg.ro.edi.GRPAPPSENDERID','bcg.ro.edi.GRPAPPSENDERID' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.GRPAPPSENDERID')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.GRPAPPRECEIVERID',1,4,1,7,'bcg.ro.edi.GRPAPPRECEIVERID','bcg.ro.edi.GRPAPPRECEIVERID' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.GRPAPPRECEIVERID')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.ICHGREVROUTE',1,4,1,7,'bcg.ro.edi.ICHGREVROUTE','bcg.ro.edi.ICHGREVROUTE' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.ICHGREVROUTE')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.ICHGROUTEADDR',1,4,1,7,'bcg.ro.edi.ICHGROUTEADDR','bcg.ro.edi.ICHGROUTEADDR' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.ICHGROUTEADDR')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.GRPAPPSENDERQUAL',1,4,1,7,'bcg.ro.edi.GRPAPPSENDERQUAL','bcg.ro.edi.GRPAPPSENDERQUAL' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.GRPAPPSENDERQUAL')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.GRPAPPRECEIVERQUAL',1,4,1,7,'bcg.ro.edi.GRPAPPRECEIVERQUAL','bcg.ro.edi.GRPAPPRECEIVERQUAL' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.GRPAPPRECEIVERQUAL')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
 Select CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.edi.GRPAPPPASSWORD',1,4,1,7,'bcg.ro.edi.GRPAPPPASSWORD','bcg.ro.edi.GRPAPPPASSWORD' From dual 
   Where Not Exists (Select * From CF_RoutingAttr where AttributeKey =  'bcg.ro.edi.GRPAPPPASSWORD')
/

--55533 -> add new for EDI reports
INSERT INTO CF_RoutingAttr (routingattrid, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY,Validescription)
SELECT  CF_RoutingAttr_SEQ.Nextval,'bcg.ro.edi.farequired',1,4,1,7,'bcg.ro.edi.farequired','bcg.ro.edi.farequired.desc','integer|minValue=0'
   FROM dual 
   WHERE NOT EXISTS (SELECT * 
                     FROM CF_RoutingAttr 
                     WHERE AttributeKey =  'bcg.ro.edi.farequired')
/

--54887_1
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
                            ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY,Validescription)
   SELECT CF_RoutingAttr_SEQ.NEXTVAL, 'ASMdnFtpAddress',1, 4,1,7, 'bcg.ro.ediint.as3.mdnFTPAddress' , 'bcg.ro.ediint.as3.mdnFTPAddress', 'FTP' 
   FROM dual
   WHERE NOT EXISTS (SELECT * FROM CF_RoutingAttr WHERE AttributeKey = 'ASMdnFtpAddress')
/
-- JR23074/100606
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.rnif.MSGSTANDARD',1, 4,1,7, 'bcg.ro.rnif.MSGSTANDARD', 'bcg.ro.rnif.MSGSTANDARD' From dual
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.rnif.MSGSTANDARD')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.rnif.MSGSTANDARD_VER',1, 4,1,7, 'bcg.ro.rnif.MSGSTANDARD_VER', 'bcg.ro.rnif.MSGSTANDARD_VER' From dual
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.rnif.MSGSTANDARD_VER')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)                                                                                                
 Select
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.rnif.PIPPAYLOADBINDINGID',1, 4,1,7, 'bcg.ro.rnif.PIPPAYLOADBINDINGID', 'bcg.ro.rnif.PIPPAYLOADBINDINGID' From dual
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.rnif.PIPPAYLOADBINDINGID')
/


--56000
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User01',1, 4,1,7, 'bcg.ro.user.User01','bcg.ro.user.User01' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User01')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User02',1, 4,1,7, 'bcg.ro.user.User02','bcg.ro.user.User02' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User02')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User03',1, 4,1,7, 'bcg.ro.user.User03','bcg.ro.user.User03' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User03')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User04',1, 4,1,7, 'bcg.ro.user.User04','bcg.ro.user.User04' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User04')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User05',1, 4,1,7, 'bcg.ro.user.User05','bcg.ro.user.User05' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User05')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User06',1, 4,1,7, 'bcg.ro.user.User06','bcg.ro.user.User06' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User06')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User07',1, 4,1,7, 'bcg.ro.user.User07','bcg.ro.user.User07' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User07')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User08',1, 4,1,7, 'bcg.ro.user.User08','bcg.ro.user.User08' From dual
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User08')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User09',1, 4,1,7, 'bcg.ro.user.User09','bcg.ro.user.User09' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User09')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.ro.user.User10',1, 4,1,7, 'bcg.ro.user.User10','bcg.ro.user.User10' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.user.User10')
/

-- 56000 end

--101468
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'FromGlobalPartnerClassificationCodeFromPIP',1, 2,1,7, 'FromGlobalPartnerClassificationCodeFromPIP','FromGlobalPartnerClassificationCodeFromPIP' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'FromGlobalPartnerClassificationCodeFromPIP')
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, 
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'ToGlobalPartnerClassificationCodeFromPIP',1, 2,1,7, 'ToGlobalPartnerClassificationCodeFromPIP','ToGlobalPartnerClassificationCodeFromPIP' From dual                                                                                                                                        
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'ToGlobalPartnerClassificationCodeFromPIP')
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'FromGlobalPartnerClassificationCodeFromPIP'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'ToGlobalPartnerClassificationCodeFromPIP'
/
--101468

-- CR60181 start
-- bcg.ro.ebxml.ACKREQUESTED
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ACKREQUESTED'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ACKREQUESTED'
		, 'bcg.ro.ebxml.ACKREQUESTED' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ACKREQUESTED')
/
-- bcg.ro.ebxml.ACKSIGNATUREREQUESTED
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ACKSIGNATUREREQUESTED'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ACKSIGNATUREREQUESTED'
		, 'bcg.ro.ebxml.ACKSIGNATUREREQUESTED' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ACKSIGNATUREREQUESTED')
/
-- bcg.ro.ebxml.ACTOR
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ACTOR'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ACTOR'
		, 'bcg.ro.ebxml.ACTOR' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ACTOR')
/
-- bcg.ro.ebxml.COMPRESSIONREQUIRED
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.COMPRESSIONREQUIRED'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.COMPRESSIONREQUIRED'
		, 'bcg.ro.ebxml.COMPRESSIONREQUIRED' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.COMPRESSIONREQUIRED')
/
-- bcg.ro.ebxml.DUPLICATEELIMINATION
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.DUPLICATEELIMINATION'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.DUPLICATEELIMINATION'
		, 'bcg.ro.ebxml.DUPLICATEELIMINATION' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.DUPLICATEELIMINATION')
/
-- bcg.ro.ebxml.ENCMIMECONSTITUENT
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ENCMIMECONSTITUENT'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ENCMIMECONSTITUENT'
		, 'bcg.ro.ebxml.ENCMIMECONSTITUENT' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ENCMIMECONSTITUENT')
/
-- bcg.ro.ebxml.ENCMIMEPARAM
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ENCMIMEPARAM'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ENCMIMEPARAM'
		, 'bcg.ro.ebxml.ENCMIMEPARAM' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ENCMIMEPARAM')
/
-- bcg.ro.ebxml.ENCMIMETYPE
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ENCMIMETYPE'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ENCMIMETYPE'
		, 'bcg.ro.ebxml.ENCMIMETYPE' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ENCMIMETYPE')
/
-- bcg.ro.ebxml.ENCRYPTIONREQUIRED
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ENCRYPTIONREQUIRED'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ENCRYPTIONREQUIRED'
		, 'bcg.ro.ebxml.ENCRYPTIONREQUIRED' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONREQUIRED')
/
-- bcg.ro.ebxml.ENCRYPTIONTRANSFORMATION
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ENCRYPTIONTRANSFORMATION'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ENCRYPTIONTRANSFORMATION'
		, 'bcg.ro.ebxml.ENCRYPTIONTRANSFORMATION' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONTRANSFORMATION')
/
-- bcg.ro.ebxml.EXCLUDEFRSIGN
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.EXCLUDEFRSIGN'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.EXCLUDEFRSIGN'
		, 'bcg.ro.ebxml.EXCLUDEFRSIGN' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.EXCLUDEFRSIGN')
/
-- bcg.ro.ebxml.HASHFUNCTION
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.HASHFUNCTION'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.HASHFUNCTION'
		, 'bcg.ro.ebxml.HASHFUNCTION' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.HASHFUNCTION')
/
-- bcg.ro.ebxml.MESSAGEORDERSEMANTICS
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.MESSAGEORDERSEMANTICS'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.MESSAGEORDERSEMANTICS'
		, 'bcg.ro.ebxml.MESSAGEORDERSEMANTICS' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.MESSAGEORDERSEMANTICS')
/
-- bcg.ro.ebxml.PARTYROLE
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.PARTYROLE'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.PARTYROLE'
		, 'bcg.ro.ebxml.PARTYROLE' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.PARTYROLE')
/
-- bcg.ro.ebxml.PERSISTDURATION
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.PERSISTDURATION'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.PERSISTDURATION'
		, 'bcg.ro.ebxml.PERSISTDURATION' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.PERSISTDURATION')
/
-- bcg.ro.ebxml.PKGMIMECONSTITUENT
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.PKGMIMECONSTITUENT'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.PKGMIMECONSTITUENT'
		, 'bcg.ro.ebxml.PKGMIMECONSTITUENT' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.PKGMIMECONSTITUENT')
/
-- bcg.ro.ebxml.PKGMIMEPARAM
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.PKGMIMEPARAM'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.PKGMIMEPARAM'
		, 'bcg.ro.ebxml.PKGMIMEPARAM' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.PKGMIMEPARAM')
/
-- bcg.ro.ebxml.ENCRYPTIONALGORITHM
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ENCRYPTIONALGORITHM'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ENCRYPTIONALGORITHM'
		, 'bcg.ro.ebxml.ENCRYPTIONALGORITHM' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONALGORITHM')
/
-- bcg.ro.ebxml.ENCRYPTIONPROTOCOL
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ENCRYPTIONPROTOCOL'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ENCRYPTIONPROTOCOL'
		, 'bcg.ro.ebxml.ENCRYPTIONPROTOCOL' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONPROTOCOL')
/
-- bcg.ro.ebxml.RETRYINTERVAL
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.RETRYINTERVAL'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.RETRYINTERVAL'
		, 'bcg.ro.ebxml.RETRYINTERVAL' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.RETRYINTERVAL')
/
-- bcg.ro.ebxml.SIGNATUREALGORITHM
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.SIGNATUREALGORITHM'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.SIGNATUREALGORITHM'
		, 'bcg.ro.ebxml.SIGNATUREALGORITHM' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.SIGNATUREALGORITHM')
/
-- bcg.ro.ebxml.SIGNATURETRANSFORMATION
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.SIGNATURETRANSFORMATION'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.SIGNATURETRANSFORMATION'
		, 'bcg.ro.ebxml.SIGNATURETRANSFORMATION' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.SIGNATURETRANSFORMATION')
/
-- bcg.ro.ebxml.SYNCREPLYMODE
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.SYNCREPLYMODE'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.SYNCREPLYMODE'
		, 'bcg.ro.ebxml.SYNCREPLYMODE' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.SYNCREPLYMODE')
/
-- bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED'
		, 'bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED')
/
-- bcg.ro.ebxml.CANONICALIZATIONMETHOD
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.CANONICALIZATIONMETHOD'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.CANONICALIZATIONMETHOD'
		, 'bcg.ro.ebxml.CANONICALIZATIONMETHOD' From dual
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.CANONICALIZATIONMETHOD')
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'TimeToAcknowledge'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'TimeToPerform'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'RetryCount'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.SIGNATURE')
WHERE	attributekey = 'DigitalSignatureRequired'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'Encryption'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'GuidelineMap'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.COMPRESSION')
WHERE	attributekey = 'ASCompressBeforeSign'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.COMPRESSION')
WHERE	attributekey = 'ASCompressed'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'ASEncrypted'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
WHERE	attributekey = 'ASMdnHttpUrl'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
WHERE	attributekey = 'ASMdnEmailAddress'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
WHERE	attributekey = 'ASMDNAsynchronous'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
WHERE	attributekey = 'ASMDNRequested'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.SIGNATURE')
WHERE	attributekey = 'ASMessageDigestAlgorithm'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.SIGNATURE')
WHERE	attributekey = 'ASMDNSigned'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.SIGNATURE')
WHERE	attributekey = 'ASSigned'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'NonRepudiationRequired'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'NonRepudiationOfReceipt'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'GlobalSupplyChainCode'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
WHERE	attributekey = 'SyncSupported'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
WHERE	attributekey = 'SyncAckRequired'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'ASBusinessId'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.SIGNATURE')
WHERE	attributekey = 'RNMessageDigestAlgorithm'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'RNEncryptionAlgorithm'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'EnvelopeFlag'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.DELIMITER')
WHERE	attributekey = 'bcg.ro.edi.EDISEGDLM'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.DELIMITER')
WHERE	attributekey = 'bcg.ro.edi.EDIDEDLM'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.DELIMITER')
WHERE	attributekey = 'bcg.ro.edi.EDISEDLM'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.DELIMITER')
WHERE	attributekey = 'bcg.ro.edi.EDIRLSCHAR'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.DELIMITER')
WHERE	attributekey = 'bcg.ro.edi.EDIDESEP'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.DELIMITER')
WHERE	attributekey = 'bcg.ro.edi.EDIDECNOT'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.SEGOUTPUT'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.ALLOWDUPLICATES'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.FAMAPNAME'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.EnvelopeProfile'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.xml.XMLNSACTIVE'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION')
WHERE	attributekey = 'bcg.ro.edi.VALERRLEVEL'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION')
WHERE	attributekey = 'bcg.ro.edi.VALLEVEL'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION')
WHERE	attributekey = 'bcg.ro.edi.VALCHARSET'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION')
WHERE	attributekey = 'bcg.ro.edi.VALALPHANUM'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION')
WHERE	attributekey = 'bcg.ro.edi.GRPLVLFA'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION')
WHERE	attributekey = 'bcg.ro.edi.VALCTRLYY'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.VALSERVSEGVAL'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.TA1OVERRIDE'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.DISCARDONERROR'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.CONNECTIONQUALIFIER'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.INTERCHANGEQUAL'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.INTERCHANGEID'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.ICHGUSAGEINDICATOR'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.GRPAPPSENDERID'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.GRPAPPRECEIVERID'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.ICHGREVROUTE'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.ICHGROUTEADDR'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.GRPAPPSENDERQUAL'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.GRPAPPRECEIVERQUAL'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.GRPAPPPASSWORD'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.GENERAL')
WHERE	attributekey = 'bcg.ro.edi.farequired'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
WHERE	attributekey = 'ASMdnFtpAddress'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	attributekey = 'bcg.ro.rnif.MSGSTANDARD'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	attributekey = 'bcg.ro.rnif.MSGSTANDARD_VER'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.rnif.GENERAL')
WHERE	attributekey = 'bcg.ro.rnif.PIPPAYLOADBINDINGID'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
WHERE	attributekey = 'bcg.ro.ebxml.ACKREQUESTED'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ACKNOWLEDGEMENT')
WHERE	attributekey = 'bcg.ro.ebxml.ACKSIGNATUREREQUESTED'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.ACTOR'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.COMPRESSION')
WHERE	attributekey = 'bcg.ro.ebxml.COMPRESSIONREQUIRED'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.DUPLICATEELIMINATION'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'bcg.ro.ebxml.ENCMIMECONSTITUENT'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'bcg.ro.ebxml.ENCMIMEPARAM'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'bcg.ro.ebxml.ENCMIMETYPE'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'bcg.ro.ebxml.ENCRYPTIONREQUIRED'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'bcg.ro.ebxml.ENCRYPTIONTRANSFORMATION'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.SIGNATURE')
WHERE	attributekey = 'bcg.ro.ebxml.EXCLUDEFRSIGN'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.SIGNATURE')
WHERE	attributekey = 'bcg.ro.ebxml.HASHFUNCTION'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.MESSAGEORDERSEMANTICS'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.PARTYROLE'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.PERSISTDURATION'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.PKGMIMECONSTITUENT'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.PKGMIMEPARAM'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.RETRYINTERVAL'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.SIGNATURE')
WHERE	attributekey = 'bcg.ro.ebxml.SIGNATUREALGORITHM'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.SIGNATURE')
WHERE	attributekey = 'bcg.ro.ebxml.SIGNATURETRANSFORMATION'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.SYNCREPLYMODE'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'bcg.ro.ebxml.ENCRYPTIONPROTOCOL'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.ENCRYPTION')
WHERE	attributekey = 'bcg.ro.ebxml.ENCRYPTIONALGORITHM'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
WHERE	attributekey = 'bcg.ro.ebxml.CANONICALIZATIONMETHOD'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User01'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User02'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User03'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User04'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User05'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User06'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User07'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User08'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User09'
/
UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
WHERE	attributekey = 'bcg.ro.user.User10'
/
-- CR60181 end
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY, ROUTINGATTR_GRPID)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.COMPRESSMIMECONSTITUENT'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.COMPRESSMIMECONSTITUENT'
		, 'bcg.ro.ebxml.COMPRESSMIMECONSTITUENT'
		, g.routingattr_grpid 
	From CF_ROUTINGATTR_GRP g
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.COMPRESSMIMECONSTITUENT')
	AND	g.routingattr_grpkey = 'bcg.grp.ro.common.COMPRESSION'
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY, ROUTINGATTR_GRPID)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.CPAID'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.CPAID'
		, 'bcg.ro.ebxml.CPAID'
		, g.routingattr_grpid 
	From CF_ROUTINGATTR_GRP g
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.CPAID')
	AND	g.routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL'
/
INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY, ROUTINGATTR_GRPID)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.SERVICETYPE'
		,1
		,4
		,1
		,7
		, 'bcg.ro.ebxml.SERVICETYPE'
		, 'bcg.ro.ebxml.SERVICETYPE'
		, g.routingattr_grpid 
	From CF_ROUTINGATTR_GRP g
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.SERVICETYPE')
	AND	g.routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL'
/
UPDATE CF_RoutingAttr Set ATTRDESCKEY = ATTRDESCKEY||'.desc' where ATTRDESCKEY <> 'bcg.ro.edi.farequired.desc'
and ATTRDESCKEY NOT LIKE '%.desc%'
/

--104470

INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY, ROUTINGATTR_GRPID)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.RNIF20.Compression'
		,1
		,2
		,7
		, 'bcg.ro.RNIF20.Compression'
		, 'bcg.ro.RNIF20.Compression.desc'
		, g.routingattr_grpid 
	From CF_ROUTINGATTR_GRP g
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.RNIF20.Compression')
	AND	g.routingattr_grpkey = 'bcg.grp.ro.common.COMPRESSION'
/

INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY, ROUTINGATTR_GRPID)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.RNIF20.CompressContentType'
		,0
		,2
		,7
		, 'bcg.ro.RNIF20.CompressContentType'
		, 'bcg.ro.RNIF20.CompressContentType.desc'
		, g.routingattr_grpid 
	From CF_ROUTINGATTR_GRP g
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.RNIF20.CompressContentType')
	AND	g.routingattr_grpkey = 'bcg.grp.ro.common.COMPRESSION'
/


INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY, ROUTINGATTR_GRPID)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.RNIF20.CompressSize'
		,0
		,2
		,7
		, 'bcg.ro.RNIF20.CompressSize'
		, 'bcg.ro.RNIF20.CompressSize.desc'
		, g.routingattr_grpid 
	From CF_ROUTINGATTR_GRP g
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.RNIF20.CompressSize')
	AND	g.routingattr_grpkey = 'bcg.grp.ro.common.COMPRESSION'
/

-- JR26780 This file adds a record for Routing attribute 'bcg.ro.ebxml.PARTYIDTYPE' in cf_routingattr Table */

INSERT INTO cf_routingattr (routingattrid, attributekey, requiredflg, attrdeflevel,
             attrseqnbr, visibilitycd, resourcekey, attrdesckey)
   SELECT cf_routingattr_seq.NEXTVAL, 'bcg.ro.ebxml.PARTYIDTYPE', 1, 4, 1, 7,
          'bcg.ro.ebxml.PARTYIDTYPE', 'bcg.ro.ebxml.PARTYIDTYPE'
     FROM DUAL
    WHERE NOT EXISTS (SELECT 'x'
                        FROM cf_routingattr
                       WHERE attributekey = 'bcg.ro.ebxml.PARTYIDTYPE')
/

UPDATE cf_routingattr
   SET routingattr_grpid =
                      (SELECT routingattr_grpid
                         FROM cf_routingattr_grp
                        WHERE routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL')
 WHERE attributekey = 'bcg.ro.ebxml.PARTYIDTYPE'
/

-- 105155 Encryption key length to be added as an attribute.


INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY, ROUTINGATTR_GRPID)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'bcg.ro.ebxml.EncKeylength'
		,1
		,4
		,7
		, 'bcg.ro.ebxml.EncKeylength'
		, 'bcg.ro.ebxml.EncKeylength.desc'
		, g.routingattr_grpid 
	From CF_ROUTINGATTR_GRP g
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'bcg.ro.ebxml.EncKeylength')
	AND	g.routingattr_grpkey = 'bcg.grp.ro.ebxml.GENERAL'
/

-- End 105155

INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY,
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
  CF_RoutingAttr_SEQ.Nextval,'bcg.soap.ContentValidation',1, 2,1,7, 'bcg.soap.ContentValidation' ,
  'bcg.soap.ContentValidation.desc' From DUAL                                                                                                                                   
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.soap.ContentValidation')
/


UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION')
WHERE	attributekey = 'bcg.soap.ContentValidation'
/


INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY,
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
  CF_RoutingAttr_SEQ.Nextval,'bcg.soap.ResponseValidation',1, 2,1,7, 'bcg.soap.ResponseValidation' ,
  'bcg.soap.ResponseValidation.desc' From DUAL                                                                                                                                   
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.soap.ResponseValidation')
/


UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION')
WHERE	attributekey = 'bcg.soap.ResponseValidation'
/


INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY,
ATTRDESCKEY)                                                                                                
 Select                                                                                                                                                                                                 
  CF_RoutingAttr_SEQ.Nextval,'bcg.soap.Re-RouteDe-EnvelopedDocument',1, 2,1,7, 'bcg.soap.Re-RouteDe-EnvelopedDocument' ,
  'bcg.soap.Re-RouteDe-EnvelopedDocument.desc' From DUAL                                                                                                                                   
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.soap.Re-RouteDe-EnvelopedDocument')
/


UPDATE cf_routingattr
SET	routingattr_grpid = (SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION')
WHERE	attributekey = 'bcg.soap.Re-RouteDe-EnvelopedDocument'
/

-- Added for ACR102199

INSERT INTO CF_RoutingAttr (ROUTINGATTRID, ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
VISIBILITYCD,RESOURCEKEY,ATTRDESCKEY, ROUTINGATTR_GRPID)
	SELECT CF_RoutingAttr_SEQ.Nextval
		, 'MsgStoreRequired'
		,1
		,116
		,7
		, 'MsgStoreRequired'
		, 'MsgStoreRequired.desc'
		, g.routingattr_grpid 
	From CF_ROUTINGATTR_GRP g
	WHERE NOT EXISTS (SELECT 'x' FROM CF_RoutingAttr WHERE AttributeKey = 'MsgStoreRequired')
	AND	g.routingattr_grpkey = 'bcg.grp.ro.common.GENERAL'
/

-- Feature 104551

INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY,routingattr_grpid)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval, 'bcg.soap.ConsumePayload',1, 2,1,7, 'bcg.soap.ConsumePayload' , 'bcg.soap.ConsumePayload'
 ,(SELECT routingattr_grpid 
				FROM cf_routingattr_grp 
				WHERE routingattr_grpkey = 'bcg.grp.ro.edi.VALIDATION') From dual                                                                                                                                   
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.soap.ConsumePayload')
/

-- Added for JR28004

INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,
ATTRSEQNBR,VISIBILITYCD,RESOURCEKEY, ATTRDESCKEY, Validescription,routingattr_grpid) 
Select  CF_RoutingAttr_seq.nextval,'bcg.ro.OUTPUTCONTENTTYPE',1, 4,
1,7, 'bcg.ro.OUTPUTCONTENTTYPE' , 'bcg.ro.OUTPUTCONTENTTYPE.desc', 'bcg.ro.OUTPUTCONTENTTYPE',
(SELECT routingattr_grpid FROM cf_routingattr_grp 
WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
From DUAL Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.ro.OUTPUTCONTENTTYPE')
/

-- JR30414

INSERT INTO CF_RoutingAttr (ROUTINGATTRID,ATTRIBUTEKEY,REQUIREDFLG,ATTRDEFLEVEL,ATTRSEQNBR,VISIBILITYCD, RESOURCEKEY, ATTRDESCKEY,ROUTINGATTR_GRPID)                                                                                                
 Select                                                                                                                                                                                                 
 CF_RoutingAttr_SEQ.Nextval,'bcg.as2.payload.encoding',1, 2,1,7, 'bcg.as2.payload.encoding' , 'bcg.as2.payload.encoding.desc',
 (SELECT routingattr_grpid FROM cf_routingattr_grp 
 WHERE routingattr_grpkey = 'bcg.grp.ro.common.GENERAL')
 From DUAL                                                                                                                                 
 Where Not Exists (Select * From CF_RoutingAttr where AttributeKey = 'bcg.as2.payload.encoding')
/


-- cf_RoutingAttr_loc_ENUS.sql

--CF_RoutingAttr_Loc
--Insert 
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Time To Acknowledge' , 'Time To Acknowledge' , 'integer|minValue=1|maxValue=10080' FROM CF_RoutingAttr WHERE AttributeKey = 'TimeToAcknowledge' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'TimeToAcknowledge')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Time To Perform(Minutes)' , 'Time To Perform' , 'integer|minValue=1|maxValue=10080' FROM CF_RoutingAttr WHERE AttributeKey = 'TimeToPerform' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'TimeToPerform')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Retry Count' , 'Retry Count' , 'integer|minValue=0|maxValue=10' FROM CF_RoutingAttr WHERE AttributeKey = 'RetryCount' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'RetryCount')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Digital Signature Required' , 'Digital Signature Required' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'DigitalSignatureRequired' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'DigitalSignatureRequired')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Encryption' , 'Encryption' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'Encryption' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'Encryption')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Validation Map' , 'Validation Map' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'GuidelineMap' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'GuidelineMap')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS Compress Before Sign' , 'AS Compress Before Sign' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASCompressBeforeSign' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASCompressBeforeSign')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS Compressed' , 'AS Compressed' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASCompressed' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASCompressed')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS Digest Algorithm' , 'AS Digest Algorithm' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASDigestAlgorithm' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASDigestAlgorithm')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS Encrypted' , 'AS Encrypted' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASEncrypted' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASEncrypted')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS Hub URI' , 'AS Hub URI' , 'url | no-mailto' FROM CF_RoutingAttr WHERE AttributeKey = 'ASHubURI' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASHubURI')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS MDN Asynchronous' , 'AS MDN Asynchronous' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASMDNAsynchronous' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMDNAsynchronous')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS MDN Requested' , 'AS MDN Requested' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASMDNRequested' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMDNRequested')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS MDN Signature Algorithm' , 'AS MDN Signature Algorithm' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASMDNSigAlgorithm' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMDNSigAlgorithm')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS MDN Signed' , 'AS MDN Signed' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASMDNSigned' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMDNSigned')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS Signed' , 'AS Signed' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASSigned' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASSigned')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Check Signing Certificate To Business Identifier' , 'Check Signing Certificate To Business Identifier' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'CheckSigningCertificateToDuns' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'CheckSigningCertificateToDuns')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Non-Repudiation Required' , 'Non-Repudiation Required' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'NonRepudiationRequired' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'NonRepudiationRequired')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Non-Repudiation of Receipt' , 'Non-Repudiation of Receipt' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'NonRepudiationOfReceipt' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'NonRepudiationOfReceipt')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Global Supply Chain Code' , 'Global Supply Chain Code' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'GlobalSupplyChainCode' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'GlobalSupplyChainCode')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS Message Digest Algorithm' , 'AS Message Digest Algorithm' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASMessageDigestAlgorithm' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMessageDigestAlgorithm')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS MDN HTTP URL (AS2)' , 'AS MDN HTTP URL (AS2)' , 'URL' FROM CF_RoutingAttr WHERE AttributeKey = 'ASMdnHttpUrl' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMdnHttpUrl')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS MDN E-Mail Address (AS1)' , 'AS MDN E-Mail Address (AS1)' , 'email' FROM CF_RoutingAttr WHERE AttributeKey = 'ASMdnEmailAddress' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMdnEmailAddress')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'AS BusinessId' , 'AS BusinessId' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'ASBusinessId' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASBusinessId')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Sync Supported' , 'Sync Supported' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'SyncSupported' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'SyncSupported')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Sync Ack Required' , 'Sync Ack Required' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'SyncAckRequired' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'SyncAckRequired')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'RN Message Digest Algorithm' , 'RN Message Digest Algorithm' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'RNMessageDigestAlgorithm' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'RNMessageDigestAlgorithm')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'RN Encryption Algorithm' , 'RN Encryption Algorithm' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'RNEncryptionAlgorithm' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'RNEncryptionAlgorithm')
/
INSERT INTO CF_RoutingAttr_Loc (ROUTINGATTRID,LOCALE,ATTRNAME,ATTRDESC,VALIDATIONRULE) SELECT ROUTINGATTRID, 'en_US' , 'Envelope Flag' , 'Envelope Flag' , '' FROM CF_RoutingAttr WHERE AttributeKey = 'EnvelopeFlag' AND NOT EXISTS (SELECT * FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'EnvelopeFlag')
/


--Update
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Time To Acknowledge' ,ATTRDESC = 'Time To Acknowledge' ,VALIDATIONRULE = 'integer|minValue=1|maxValue=10080' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'TimeToAcknowledge' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Time To Perform(Minutes)' ,ATTRDESC = 'Time To Perform' ,VALIDATIONRULE = 'integer|minValue=1|maxValue=10080' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'TimeToPerform' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Retry Count' ,ATTRDESC = 'Retry Count' ,VALIDATIONRULE = 'integer|minValue=0|maxValue=10' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'RetryCount' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Digital Signature Required' ,ATTRDESC = 'Digital Signature Required' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'DigitalSignatureRequired' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Encryption' ,ATTRDESC = 'Encryption' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'Encryption' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Validation Map' ,ATTRDESC = 'Validation Map' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'GuidelineMap' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS Compress Before Sign' ,ATTRDESC = 'AS Compress Before Sign' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASCompressBeforeSign' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS Compressed' ,ATTRDESC = 'AS Compressed' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASCompressed'  and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS Digest Algorithm' ,ATTRDESC = 'AS Digest Algorithm' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASDigestAlgorithm'  and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS Encrypted' ,ATTRDESC = 'AS Encrypted' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASEncrypted'  and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS Hub URI' ,ATTRDESC = 'AS Hub URI' ,VALIDATIONRULE = 'url | no-mailto' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASHubURI'  and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS MDN Asynchronous' ,ATTRDESC = 'AS MDN Asynchronous' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMDNAsynchronous'  and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS MDN Requested' ,ATTRDESC = 'AS MDN Requested' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMDNRequested'  and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS MDN Signature Algorithm' ,ATTRDESC = 'AS MDN Signature Algorithm' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMDNSigAlgorithm'  and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS MDN Signed' ,ATTRDESC = 'AS MDN Signed' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMDNSigned'  and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS Signed' ,ATTRDESC = 'AS Signed' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASSigned' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Check Signing Certificate To Business Identifier' ,ATTRDESC = 'Check Signing Certificate To Business Identifier' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'CheckSigningCertificateToDuns' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Non-Repudiation Required' ,ATTRDESC = 'Non-Repudiation Required' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'NonRepudiationRequired' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Non-Repudiation of Receipt' ,ATTRDESC = 'Non-Repudiation of Receipt' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'NonRepudiationOfReceipt' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Global Supply Chain Code' ,ATTRDESC = 'Global Supply Chain Code' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'GlobalSupplyChainCode' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS Message Digest Algorithm' ,ATTRDESC = 'AS Message Digest Algorithm' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMessageDigestAlgorithm' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS MDN HTTP URL (AS2)' ,ATTRDESC = 'AS MDN HTTP URL (AS2)' ,VALIDATIONRULE = 'URL' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMdnHttpUrl' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS MDN E-Mail Address (AS1)' ,ATTRDESC = 'AS MDN E-Mail Address (AS1)' ,VALIDATIONRULE = 'email' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASMdnEmailAddress' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'AS BusinessId' ,ATTRDESC = 'AS BusinessId' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'ASBusinessId' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Sync Supported' ,ATTRDESC = 'Sync Supported' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'SyncSupported' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Sync Ack Required' ,ATTRDESC = 'Sync Ack Required' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'SyncAckRequired' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'RN Message Digest Algorithm' ,ATTRDESC = 'RN Message Digest Algorithm' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'RNMessageDigestAlgorithm' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'RN Encryption Algorithm' ,ATTRDESC = 'RN Encryption Algorithm' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'RNEncryptionAlgorithm' and t.locale = 'en_US')
/
UPDATE CF_RoutingAttr_Loc SET ATTRNAME = 'Envelope Flag' ,ATTRDESC = 'Envelope Flag' ,VALIDATIONRULE = '' WHERE ROUTINGATTRID = (SELECT d.ROUTINGATTRID FROM CF_RoutingAttr_Loc t, CF_RoutingAttr d WHERE d.ROUTINGATTRID = t.ROUTINGATTRID AND d.ATTRIBUTEKEY = 'EnvelopeFlag' and t.locale = 'en_US')
/

-- cf_RoutingAttrOpt.sql

--CF_RoutingAttrOpt 

INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'DigitalSignatureRequired' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'DigitalSignatureRequired' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '0', 1, 1 , '0'
  From CF_RoutingAttr a   
 Where AttributeKey = 'Encryption' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '0')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '1', 2, 0 , '1'
  From CF_RoutingAttr a   
 Where AttributeKey = 'Encryption' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '1')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '2', 3, 0 , '2'
  From CF_RoutingAttr a   
 Where AttributeKey = 'Encryption' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '2')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASEncrypted' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASEncrypted' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASCompressed' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASCompressed' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 1 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASCompressBeforeSign' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 0 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASCompressBeforeSign' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 1 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASMDNAsynchronous' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 0 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASMDNAsynchronous' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASMDNSigned' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 0 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASMDNSigned' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )   -- new attr option
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'sha1', 2, 1 , 'sha1'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASMessageDigestAlgorithm' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'sha1')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )     -- new attr option
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'md5', 1, 0 , 'md5'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASMessageDigestAlgorithm' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'md5')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 1 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASMDNRequested' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 0 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASMDNRequested' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASSigned' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASSigned' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'NonRepudiationRequired' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'NonRepudiationRequired' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'NonRepudiationOfReceipt' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'NonRepudiationOfReceipt' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
-- GlobalSupplyChainCode = Information Technology
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Information Technology', 2, 1, 'InformationTechnology' 
  From CF_RoutingAttr a   
 Where AttributeKey = 'GlobalSupplyChainCode' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Information Technology')
/
-- GlobalSupplyChainCode = Electronic Components
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Electronic Components', 1, 1, 'ElectronicComponents' 
  From CF_RoutingAttr a   
 Where AttributeKey = 'GlobalSupplyChainCode' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Electronic Components')
/
-- GlobalSupplyChainCode =  Semiconductor Manufacturing  
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Semiconductor Manufacturing', 3, 1 , 'SemiconductorManufacturing'
  From CF_RoutingAttr a   
 Where AttributeKey = 'GlobalSupplyChainCode' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Semiconductor Manufacturing')
/
--47766
--INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
--Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0, 'Y' 
--  From CF_RoutingAttr a   
-- Where AttributeKey = 'OneWay' 
--   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
--/
--47766
--INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
--Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 1, 1 , 'N'
--  From CF_RoutingAttr a   
-- Where AttributeKey = 'OneWay' 
--   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
--/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 2, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'SyncSupported' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 1, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'SyncSupported' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 2, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'SyncAckRequired' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 1, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'SyncAckRequired' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASBusinessId' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 1, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'ASBusinessId' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )   
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'sha1', 2, 1 , 'sha1'
  From CF_RoutingAttr a   
 Where AttributeKey = 'RNMessageDigestAlgorithm' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'sha1')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )    
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'md5', 1, 0 , 'md5'
  From CF_RoutingAttr a   
 Where AttributeKey = 'RNMessageDigestAlgorithm' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'md5')
   
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )   
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '3DES', 1, 1, '3DES' 
  From CF_RoutingAttr a   
 Where AttributeKey = 'RNEncryptionAlgorithm' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '3DES')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )    
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'RC2-40', 3, 0, 'RC2-40' 
  From CF_RoutingAttr a   
 Where AttributeKey = 'RNEncryptionAlgorithm' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'RC2-40')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1, 'N' 
  From CF_RoutingAttr a   
 Where AttributeKey = 'EnvelopeFlag' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/   
   
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0, 'Y' 
  From CF_RoutingAttr a   
 Where AttributeKey = 'EnvelopeFlag' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , ',', 1, 0 ,'common.char.comma'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.EDIDECNOT' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = ',')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '.', 2, 0 ,'common.char.period'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.EDIDECNOT' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '.')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 2, 0 ,'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.SEGOUTPUT' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 1, 1 ,'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.SEGOUTPUT' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 2, 0 ,'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 1, 1 ,'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 ,'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.xml.XMLNSACTIVE' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 0 ,'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.xml.XMLNSACTIVE' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 ,'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.GRPLVLFA' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1 ,'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.GRPLVLFA' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 ,'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1 ,'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 ,'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.DISCARDONERROR' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 1 ,'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.DISCARDONERROR' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 1 ,'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.TA1OVERRIDE' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 2, 0 ,'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.TA1OVERRIDE' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '0', 1, 1 , 'Zero'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '0')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '1', 2, 0 , 'One'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '1')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '2', 3, 0 , 'Two'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '2')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '0', 1, 1 , 'Zero'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.VALERRLEVEL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '0')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '1', 2, 0 , 'One'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.VALERRLEVEL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '1')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '2', 3, 0 , 'Two'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.VALERRLEVEL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '2')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '0', 1, 1 , 'Zero'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.VALLEVEL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '0')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '1', 2, 0 , 'One'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.VALLEVEL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '1')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , '2', 3, 0 , 'Two'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.ro.edi.VALLEVEL' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = '2')
/

--100424
UPDATE cf_routingattropt
SET defaultflg = 1
WHERE RoutingAttrId = (SELECT RoutingAttrId 
                       FROM cf_routingattr
                       WHERE attributekey = 'NonRepudiationRequired') 
AND AttrValueKey = 'Y'
/

UPDATE cf_routingattropt
SET defaultflg = 0
WHERE RoutingAttrId = (SELECT RoutingAttrId 
                       FROM cf_routingattr
                       WHERE attributekey = 'NonRepudiationRequired') 
AND AttrValueKey = 'N'
/
--end 100424
-- CR60181 start
-- bcg.ro.ebxml.ACKREQUESTED
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.ALWAYS', 1, 1 , 'bcg.ro.attropt.value.ALWAYS'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ACKREQUESTED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.ALWAYS')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.NEVER', 2, 0 , 'bcg.ro.attropt.value.NEVER'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ACKREQUESTED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.NEVER')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.PERMESSAGE', 3, 0 , 'bcg.ro.attropt.value.PERMESSAGE'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ACKREQUESTED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.PERMESSAGE')
/
-- bcg.ro.ebxml.ACKSIGNATUREREQUESTED
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.ALWAYS', 1, 1 , 'bcg.ro.attropt.value.ALWAYS'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ACKSIGNATUREREQUESTED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.ALWAYS')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.NEVER', 2, 0 , 'bcg.ro.attropt.value.NEVER'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ACKSIGNATUREREQUESTED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.NEVER')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.PERMESSAGE', 3, 0 , 'bcg.ro.attropt.value.PERMESSAGE'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ACKSIGNATUREREQUESTED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.PERMESSAGE')
/
-- bcg.ro.ebxml.COMPRESSIONREQUIRED
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'Y', 1, 1 , 'Y'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.COMPRESSIONREQUIRED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'N', 2, 0 , 'N'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.COMPRESSIONREQUIRED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'N')
/
-- bcg.ro.ebxml.ENCRYPTIONREQUIRED
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'Y', 1, 1 , 'Y'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONREQUIRED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'N', 2, 0 , 'N'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONREQUIRED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'N')
/
-- bcg.ro.ebxml.CANONICALIZATIONMETHOD
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.INCLUSIVE_WITH_COMMENTS', 1, 1 , 'bcg.ro.attropt.value.INCLUSIVE_WITH_COMMENTS'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.CANONICALIZATIONMETHOD'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.INCLUSIVE_WITH_COMMENTS')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.INCLUSIVE', 2, 0 , 'bcg.ro.attropt.value.INCLUSIVE'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.CANONICALIZATIONMETHOD'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.INCLUSIVE')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.EXCLUSIVE', 3, 0 , 'bcg.ro.attropt.value.EXCLUSIVE'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.CANONICALIZATIONMETHOD'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.EXCLUSIVE')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.EXCLUSIVE_WITH_COMMENTS', 4, 0 , 'bcg.ro.attropt.value.EXCLUSIVE_WITH_COMMENTS'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.CANONICALIZATIONMETHOD'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.EXCLUSIVE_WITH_COMMENTS')
/
-- bcg.ro.ebxml.DUPLICATEELIMINATION
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.ALWAYS', 1, 1 , 'bcg.ro.attropt.value.ALWAYS'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.DUPLICATEELIMINATION'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.ALWAYS')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.NEVER', 2, 0 , 'bcg.ro.attropt.value.NEVER'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.DUPLICATEELIMINATION'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.NEVER')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.PERMESSAGE', 3, 0 , 'bcg.ro.attropt.value.PERMESSAGE'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.DUPLICATEELIMINATION'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.PERMESSAGE')
/
-- bcg.ro.ebxml.HASHFUNCTION
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.SHA1', 1, 1 , 'bcg.ro.attropt.value.SHA1'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.HASHFUNCTION'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.SHA1')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.SHA256', 2, 0 , 'bcg.ro.attropt.value.SHA256'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.HASHFUNCTION'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.SHA256')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.SHA512', 3, 0 , 'bcg.ro.attropt.value.SHA512'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.HASHFUNCTION'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.SHA512')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.RIPEMD160', 4, 0 , 'bcg.ro.attropt.value.RIPEMD160'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.HASHFUNCTION'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.RIPEMD160')
/
-- bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'Y', 1, 1 , 'Y'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'Y')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'N', 2, 0 , 'N'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'N')
/
-- bcg.ro.ebxml.MESSAGEORDERSEMANTICS
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.GUARANTEED', 1, 1 , 'bcg.ro.attropt.value.GUARANTEED'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.MESSAGEORDERSEMANTICS'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.GUARANTEED')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.NOTGUARANTEED', 2, 0 , 'bcg.ro.attropt.value.NOTGUARANTEED'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.MESSAGEORDERSEMANTICS'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.NOTGUARANTEED')
/
-- bcg.ro.ebxml.ENCRYPTIONALGORITHM
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.3DES-CBC', 1, 1 , 'bcg.ro.attropt.value.3DES-CBC'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONALGORITHM'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.3DES-CBC')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.AES-128-CBC', 2, 0 , 'bcg.ro.attropt.value.AES-128-CBC'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONALGORITHM'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.AES-128-CBC')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.AES-192-CBC', 3, 0 , 'bcg.ro.attropt.value.AES-192-CBC'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONALGORITHM'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.AES-192-CBC')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.AES-256-CBC', 4, 0 , 'bcg.ro.attropt.value.AES-256-CBC'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONALGORITHM'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.AES-256-CBC')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.DES', 5, 0 , 'bcg.ro.attropt.value.DES'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONALGORITHM'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.DES')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.RC2', 6, 0 , 'bcg.ro.attropt.value.RC2'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONALGORITHM'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.RC2')
/
-- bcg.ro.ebxml.ENCRYPTIONPROTOCOL
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.SMIME', 1, 1 , 'bcg.ro.attropt.value.SMIME'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONPROTOCOL'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.SMIME')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.XMLENCRYPTION', 2, 0 , 'bcg.ro.attropt.value.XMLENCRYPTION'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.ENCRYPTIONPROTOCOL'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.XMLENCRYPTION')
/
-- bcg.ro.ebxml.SIGNATUREALGORITHM
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.DSA-SHA1', 1, 0 , 'bcg.ro.attropt.value.DSA-SHA1'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.SIGNATUREALGORITHM'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.DSA-SHA1')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.HMAC-SHA1', 2, 0 , 'bcg.ro.attropt.value.HMAC-SHA1'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.SIGNATUREALGORITHM'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.HMAC-SHA1')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.RSA-SHA1', 3, 1 , 'bcg.ro.attropt.value.RSA-SHA1'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.SIGNATUREALGORITHM'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.RSA-SHA1')
/
-- bcg.ro.ebxml.SYNCREPLYMODE
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.MSHSIGNALSONLY', 1, 1 , 'bcg.ro.attropt.value.MSHSIGNALSONLY'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.SYNCREPLYMODE'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.MSHSIGNALSONLY')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.SIGNALSONLY', 2, 0 , 'bcg.ro.attropt.value.SIGNALSONLY'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.SYNCREPLYMODE'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.SIGNALSONLY')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.RESPONSEONLY', 3, 0 , 'bcg.ro.attropt.value.RESPONSEONLY'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.SYNCREPLYMODE'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.RESPONSEONLY')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.SIGNALSANDRESPONSE', 4, 0 , 'bcg.ro.attropt.value.SIGNALSANDRESPONSE'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.SYNCREPLYMODE'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.SIGNALSANDRESPONSE')
/
INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.NONE', 5, 0 , 'bcg.ro.attropt.value.NONE'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.ebxml.SYNCREPLYMODE'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.NONE')
/
-- CR60181 end

--104470

INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, ResourceKey, DefaultFlg )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.NONE', 'bcg.ro.attropt.value.NONE', 1
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.RNIF20.Compression'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.NONE')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, ResourceKey, DefaultFlg )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.PAYLOAD', 'bcg.ro.attropt.value.PAYLOAD', 0
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.RNIF20.Compression'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.PAYLOAD')
/


INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, ResourceKey, DefaultFlg )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'bcg.ro.attropt.value.PAYLOAD_AND_ATTACHMENT', 'bcg.ro.attropt.value.PAYLOAD_AND_ATTACHMENT', 0
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.ro.RNIF20.Compression'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'bcg.ro.attropt.value.PAYLOAD_AND_ATTACHMENT')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID,RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval,a.RoutingAttrId 
		, 'Y', 2, 1 , 'Y'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.soap.ContentValidation'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'Y')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID,RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval,a.RoutingAttrId 
		, 'N', 1, 0, 'N'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.soap.ContentValidation'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'N')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID,RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval,a.RoutingAttrId 
		, 'Y', 2, 1 , 'Y'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.soap.ResponseValidation'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'Y')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID,RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval,a.RoutingAttrId 
		, 'N', 1, 0, 'N'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.soap.ResponseValidation'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'N')
/


INSERT INTO CF_RoutingAttrOpt (AttrOptID,RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval,a.RoutingAttrId 
		, 'Y', 2, 1 , 'Y'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.soap.Re-RouteDe-EnvelopedDocument'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'Y')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID,RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval,a.RoutingAttrId 
		, 'N', 1, 0, 'N'
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'bcg.soap.Re-RouteDe-EnvelopedDocument'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'N')
/

-- CR102199


INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, ResourceKey, DefaultFlg )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'Y', 'Y', 1
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'MsgStoreRequired'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'Y')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, ResourceKey, DefaultFlg )  
	SELECT CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId 
		, 'N', 'N', 0
	FROM CF_RoutingAttr a   
	WHERE AttributeKey = 'MsgStoreRequired'
	AND NOT EXISTS (SELECT 'x' FROM CF_RoutingAttrOpt p
			WHERE p.RoutingAttrId = a.RoutingAttrId 
			AND p.AttrValueKey = 'N')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'Y', 1, 0 , 'Y'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.soap.ConsumePayload' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'Y')
/

INSERT INTO CF_RoutingAttrOpt (AttrOptID, RoutingAttrId, AttrValueKey, AttrSeqNbr, DefaultFlg, ResourceKey )  
Select CF_RoutingAttrOpt_SEQ.Nextval, a.RoutingAttrId , 'N', 1, 1 , 'N'
  From CF_RoutingAttr a   
 Where AttributeKey = 'bcg.soap.ConsumePayload' 
   And Not Exists (Select * From CF_RoutingAttrOpt p where p.RoutingAttrId = a.RoutingAttrId And p.AttrValueKey = 'N')
/





-- cf_RoutingAttrOpt_loc_ENUS.sql

-- CF_RoutingAttrOpt_loc 

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'DigitalSignatureRequired')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'DigitalSignatureRequired')
                                           And AttrValueKey = 'N' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'DigitalSignatureRequired')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'DigitalSignatureRequired')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'None' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'Encryption')
   And AttrValueKey = '0'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'Encryption')
                                           And AttrValueKey = '0' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Payload' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'Encryption')
   And AttrValueKey = '1'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'Encryption')
                                           And AttrValueKey = '1' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Payload and Container' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'Encryption')
   And AttrValueKey = '2'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'Encryption')
                                           And AttrValueKey = '2' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASEncrypted')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASEncrypted')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASEncrypted')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASEncrypted')
                                           And AttrValueKey = 'N' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASCompressed')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASCompressed')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASCompressed')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASCompressed')
                                           And AttrValueKey = 'N' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASCompressBeforeSign')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASCompressBeforeSign')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASCompressBeforeSign')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASCompressBeforeSign')
                                           And AttrValueKey = 'N' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASMDNAsynchronous')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASMDNAsynchronous')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASMDNAsynchronous')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASMDNAsynchronous')
                                           And AttrValueKey = 'N' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASMDNSigned')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASMDNSigned')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASMDNSigned')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASMDNSigned')
                                           And AttrValueKey = 'N' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'sha1' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASMessageDigestAlgorithm')
   And AttrValueKey = 'sha1'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASMessageDigestAlgorithm')
                                           And AttrValueKey = 'sha1' ))
/                                           
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'md5' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASMessageDigestAlgorithm')
   And AttrValueKey = 'md5'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASMessageDigestAlgorithm')
                                           And AttrValueKey = 'md5' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASMDNRequested')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASMDNRequested')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASMDNRequested')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASMDNRequested')
                                           And AttrValueKey = 'N' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASSigned')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASSigned')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASSigned')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASSigned')
                                           And AttrValueKey = 'N' ))
/

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'NonRepudiationRequired')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'NonRepudiationRequired')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'NonRepudiationRequired')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'NonRepudiationRequired')
                                           And AttrValueKey = 'N' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'NonRepudiationOfReceipt')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'NonRepudiationOfReceipt')
                                           And AttrValueKey = 'Y' ))
/
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'NonRepudiationOfReceipt')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'NonRepudiationOfReceipt')
                                           And AttrValueKey = 'N' ))
/
-- GlobalSupplyChainCode = Information Technology
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Information Technology' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'GlobalSupplyChainCode')
   And AttrValueKey = 'Information Technology'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'GlobalSupplyChainCode')
                                           And AttrValueKey = 'Information Technology' ))
/                                           
-- GlobalSupplyChainCode = Electronic Components
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Electronic Components' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'GlobalSupplyChainCode')
   And AttrValueKey = 'Electronic Components'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'GlobalSupplyChainCode')
                                           And AttrValueKey = 'Electronic Components' ))                                           
/                                           
-- GlobalSupplyChainCode =  Semiconductor Manufacturing  
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Semiconductor Manufacturing' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'GlobalSupplyChainCode')
   And AttrValueKey = 'Semiconductor Manufacturing'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'GlobalSupplyChainCode')
                                           And AttrValueKey = 'Semiconductor Manufacturing' ))
/

--47766
--INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
--Select AttrOptID, 'en_US', 'Yes' 
--  From CF_RoutingAttrOpt 
-- Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'OneWay')
--   And AttrValueKey = 'Y'
--   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
--                    Where Locale = 'en_US' 
--                      And AttrOptID = ( Select AttrOptID 
--                                          From CF_RoutingAttrOpt 
--                                         Where RoutingAttrId = (Select RoutingAttrId 
--                                                                  From CF_RoutingAttr 
--                                                                 Where AttributeKey = 'OneWay')
--                                           And AttrValueKey = 'Y' ))
--/


--47766
--INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
--Select AttrOptID, 'en_US', 'No' 
--  From CF_RoutingAttrOpt 
-- Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'OneWay')
--   And AttrValueKey = 'N'
--   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
--                    Where Locale = 'en_US' 
--                      And AttrOptID = ( Select AttrOptID 
--                                          From CF_RoutingAttrOpt 
--                                         Where RoutingAttrId = (Select RoutingAttrId 
--                                                                  From CF_RoutingAttr 
--                                                                 Where AttributeKey = 'OneWay')
--                                           And AttrValueKey = 'N' ))
--                                           
--/                                          
INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'SyncSupported')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'SyncSupported')
                                           And AttrValueKey = 'Y' ))
/

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'SyncSupported')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'SyncSupported')
                                           And AttrValueKey = 'N' ))
/          

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'SyncAckRequired')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'SyncAckRequired')
                                           And AttrValueKey = 'Y' ))
/


INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'SyncAckRequired')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'SyncAckRequired')
                                           And AttrValueKey = 'N' ))
/ 

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASBusinessId')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASBusinessId')
                                           And AttrValueKey = 'Y' ))
/     

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'ASBusinessId')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'ASBusinessId')
                                           And AttrValueKey = 'N' ))
/

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'sha1' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'RNMessageDigestAlgorithm')
   And AttrValueKey = 'sha1'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'RNMessageDigestAlgorithm')
                                           And AttrValueKey = 'sha1' ))
/
                                           

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'md5' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'RNMessageDigestAlgorithm')
   And AttrValueKey = 'md5'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'RNMessageDigestAlgorithm')
                                           And AttrValueKey = 'md5' ))
/

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', '3DES' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'RNEncryptionAlgorithm')
   And AttrValueKey = '3DES'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'RNEncryptionAlgorithm')
                                           And AttrValueKey = '3DES' ))
/
                                           
--47923
--INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
--Select AttrOptID, 'en_US', 'RC5' 
--  From CF_RoutingAttrOpt 
-- Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'RNEncryptionAlgorithm')
--   And AttrValueKey = 'RC5'
--   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
--                    Where Locale = 'en_US' 
--                      And AttrOptID = ( Select AttrOptID 
--                                          From CF_RoutingAttrOpt 
--                                         Where RoutingAttrId = (Select RoutingAttrId 
--                                                                  From CF_RoutingAttr 
--                                                                 Where AttributeKey = 'RNEncryptionAlgorithm')
--                                           And AttrValueKey = 'RC5' ))
--/

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'RC2-40' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'RNEncryptionAlgorithm')
   And AttrValueKey = 'RC2-40'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'RNEncryptionAlgorithm')
                                           And AttrValueKey = 'RC2-40' ))
/

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'No' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'EnvelopeFlag')
   And AttrValueKey = 'N'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'EnvelopeFlag')
                                           And AttrValueKey = 'N' ))
/

INSERT INTO CF_RoutingAttrOpt_Loc ( AttrOptID, Locale, AttrValue )  
Select AttrOptID, 'en_US', 'Yes' 
  From CF_RoutingAttrOpt 
 Where RoutingAttrId = (Select RoutingAttrId From CF_RoutingAttr Where AttributeKey = 'EnvelopeFlag')
   And AttrValueKey = 'Y'
   And Not Exists (Select * From CF_RoutingAttrOpt_Loc 
                    Where Locale = 'en_US' 
                      And AttrOptID = ( Select AttrOptID 
                                          From CF_RoutingAttrOpt 
                                         Where RoutingAttrId = (Select RoutingAttrId 
                                                                  From CF_RoutingAttr 
                                                                 Where AttributeKey = 'EnvelopeFlag')
                                           And AttrValueKey = 'Y' ))
/


-- cf_RoutingObjType.sql

--CF_RoutingObjType

Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, 1, 2,16,'Package' ,'Package'
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'Package')
/
Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, 2, 1,32,'Protocol' ,'Protocol'
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'Protocol')
/
Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, 3, 1,64,'Process' ,'Process' 
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'Process')
/
Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, 4, 1,128,'Activity' ,'Activity' 
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'Activity')
/
Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, 5, 1,256,'Action' ,'Action'
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'Action')
/
Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, -1, 1, 1,'Global' ,'Global'
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'Global')
/
Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, -1, 1, 2,'Partner' ,'Partner'
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'Partner')
/
Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, -1, 1, 4,'Channel' ,'Channel'
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'Channel')
/
Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, -1, 1, 8,'DestType' ,'DestType'
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'DestType')
/
Insert into CF_RoutingObjType (RoutingObjTypeId, RoutingObjLevel, RoutingObjSubLevel, RoutingObjAttrRank, RoutingObjTypeKey, ResourceKey )
    Select CF_RoutingObjType_Seq.nextval, 6, 1, 512,'Signal' ,'Signal'
      From dual Where Not Exists (Select * From CF_RoutingObjType Where RoutingObjTypeKey = 'Signal')
/

-- cf_RoutingObjType_loc_ENUS.sql

--CF_RoutingObjType_Loc

Insert into CF_RoutingObjType_Loc (RoutingObjTypeId, Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'Package'), 'en_US', 'Package'
     From dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Package') 
/
Insert into CF_RoutingObjType_Loc (RoutingObjTypeId, Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'Protocol'), 'en_US', 'Protocol'
     From dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Protocol') 
/
Insert into CF_RoutingObjType_Loc (ROUTINGOBJTYPEID,  Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'Process'), 'en_US', 'Document Flow'
     From Dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Document Flow')
/ 
Insert into CF_RoutingObjType_Loc (RoutingObjTypeId, Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'Activity'), 'en_US', 'Activity'
     From dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Activity') 
/
Insert into CF_RoutingObjType_Loc (RoutingObjTypeId, Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'Action'), 'en_US', 'Action'
     From dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Action') 
/
Insert into CF_RoutingObjType_Loc (RoutingObjTypeId, Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'Global'), 'en_US', 'Global'
     From dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Global') 
/
Insert into CF_RoutingObjType_Loc (ROUTINGOBJTYPEID,  Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'Partner'), 'en_US', 'Participant'
     From dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Participant')
/ 
Insert into CF_RoutingObjType_Loc (ROUTINGOBJTYPEID,  Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'Channel'), 'en_US', 'Connection'
     From dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Connection')
/ 
Insert into CF_RoutingObjType_Loc (ROUTINGOBJTYPEID,  Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'DestType'), 'en_US', 'Gateway Type'
     From dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Gateway Type')
/ 
Insert into CF_RoutingObjType_Loc (ROUTINGOBJTYPEID,  Locale, RoutingObjTypeCd) 
    Select (select RoutingObjTypeId from CF_RoutingObjType where RoutingObjTypeKey = 'Signal'), 'en_US', 'Signal'
     From dual Where Not Exists (Select * From CF_RoutingObjType_Loc Where RoutingObjTypeCd = 'Signal')
/  

-- db_ProcAuditNames.sql

Insert Into db_procAuditNames (PROCNAME, AUDITFLG, FILEAUDITFLG) 
   Select '/tmp', 0, 0
     From dual
    Where NOT Exists (Select 'x' From db_procAuditNames Where PROCNAME = '/tmp')
/

Insert Into db_procAuditNames (PROCNAME, AUDITFLG, FILEAUDITFLG)
   Select object_name, 0, 0 
     From user_Objects
    Where object_type = 'PROCEDURE' 
      And object_name <> 'DB_PROCAUDITLOGI'
      And object_name NOT In (Select PROCNAME From db_procAuditNames)
/



-- BaseObjectInit.sql

/*
 ================================================================================
 Script Name:    Meta data script for f
                   CF_RoutingObject
                  , CF_RoutingObjRef_Lnk
                  , CF_ValidobjInteract
                  , Pr_WorkFlowInteract_lnk
                  , CF_RoutingObjAttr_Lnk
                  , PR_RoutingAttr_Lnk

 Create Date:    2003-04-23
 Author:         M..Mount
 ================================================================================
*/

--CF_RoutingObject 
-- insert packages

  Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'AS', 'AS', 'N/A', 1, 'AS Packaging', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Package'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'AS'
                   and   o.Routingobjver = 'N/A'
                   and   t.routingobjtypekey = 'Package'))
/                   
    
  Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'None', 'None', 'N/A', 1, 'No Packaging', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Package'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'None'
                   and   o.Routingobjver = 'N/A'
                   and   t.routingobjtypekey = 'Package'))
/                   

  Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'Backend Integration', 'Backend Integration', '1.0', 1, 'Backend Integration Packaging', 1, 3, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Package'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'Backend Integration'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Package'))
/                   
  Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'N/A', 'N/A', 'N/A', 1, 'N/A', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Package'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'N/A'
                   and   o.Routingobjver = 'N/A'
                   and   t.routingobjtypekey = 'Package'))
/                   

  Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
         CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'ebMS', 'ebMS', '2.0', 1, 'ebXML Packaging', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Package'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'ebMS'
                   and   o.Routingobjver = '2.0'
                   and   t.routingobjtypekey = 'Package'))
/                   
    
------
--    Insert into CF_RoutingObjRef_Lnk  null records for services and packages
------
Create or Replace procedure metadata_insert as 
   v_RoutingObjId number; 
   v_InterActObjId number; 
BEGIN
    Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId, RoutingObjId, RoutingObjParentRefId) 
     select CF_RoutingObjRef_Lnk_Seq.nextval, RoutingObjId, null
    from  CF_RoutingObject r
    inner join CF_RoutingObjType t on t.RoutingObjTypeId = r.RoutingObjTypeId
    WHERE t.routingobjtypekey in('Package','Service')
    AND r.RoutingObjCd in('AS','Backend Integration','None')
    AND r.RoutingObjName in('AS','Backend Integration','None')
    AND NOT Exists(Select * from cf_routingObjref_lnk l
                   inner join cf_routingobject r on r.RoutingObjId = l.RoutingObjId
                   inner join cf_routingobjtype t on t.RoutingObjTypeId = r.RoutingObjTypeId
                   WHERE t.routingobjtypekey in('Package','Service')
                   AND r.RoutingObjCd in('AS','Backend Integration','None')
                   AND r.RoutingObjName in('AS','Backend Integration','None'));

--had to fix for migration, give N/A its own insert statement o/w N/A does not get
--get inserted during migration
    Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId, RoutingObjId, RoutingObjParentRefId) 
     select CF_RoutingObjRef_Lnk_Seq.nextval, RoutingObjId, null
    from  CF_RoutingObject r
    inner join CF_RoutingObjType t on t.RoutingObjTypeId = r.RoutingObjTypeId
    WHERE t.routingobjtypekey in('Package','Service')
    AND r.RoutingObjCd in('N/A')
    AND r.RoutingObjName in('N/A')
    AND NOT Exists(Select * from cf_routingObjref_lnk l
                   inner join cf_routingobject r on r.RoutingObjId = l.RoutingObjId
                   inner join cf_routingobjtype t on t.RoutingObjTypeId = r.RoutingObjTypeId
                   WHERE t.routingobjtypekey in('Package','Service')
                   AND r.RoutingObjCd in('N/A')
                   AND r.RoutingObjName in('N/A'));

    Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId, RoutingObjId, RoutingObjParentRefId) 
     select CF_RoutingObjRef_Lnk_Seq.nextval, RoutingObjId, null
    from  CF_RoutingObject r
    inner join CF_RoutingObjType t on t.RoutingObjTypeId = r.RoutingObjTypeId
    WHERE t.routingobjtypekey in('Package')
    AND r.RoutingObjCd in('ebMS')
    AND r.RoutingObjVer in('2.0')
    AND NOT Exists(Select * from cf_routingObjref_lnk l
                   inner join cf_routingobject r on r.RoutingObjId = l.RoutingObjId
                   inner join cf_routingobjtype t on t.RoutingObjTypeId = r.RoutingObjTypeId
                   WHERE t.routingobjtypekey in('Package')
                   AND r.RoutingObjCd in('ebMS')
                   AND r.RoutingObjVer in('2.0'));

--Insert records for Binary  protocol  ------------------1st procotol

    v_RoutingObjId := '';
    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'Binary', 'Binary', '1.0', 1, 'Binary Protocol', 1, 7,0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'Binary'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Protocol'));

     Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'Binary'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Protocol'
                   and rownum < 2;
                   

-- insert AS protocol record in lnk table for routing object record just entered
   BEGIN
     If NVL(v_RoutingObjId,0) != 0 THEN 

       Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId,  L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'AS'
                    and R.RoutingObjCd = 'AS'
                    and R.RoutingObjVer = 'N/A';
    end if;

      EXCEPTION
         When DUP_VAL_ON_INDEX then NULL;
   end;
  
    BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'None'
                    and R.RoutingObjCd = 'None'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;

    BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval, v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'Backend Integration'
                    and R.RoutingObjCd = 'Backend Integration'
                    and R.RoutingObjVer = '1.0';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then null; --  DBMS_OUTPUT.PUT_LINE('v_RoutingObjId = ' || NVL(v_RoutingObjId, ''));
--                                    DBMS_OUTPUT.PUT_LINE('' || NVL(v_RoutingObjId, ''));
   END;
--end;


--   Insert records for XMLEVENT protocol  ------------------2nd procotol  

    v_RoutingObjId := '';

    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'XMLEvent', 'XMLEvent', '1.0', 1, 'XML Event Definition', 1, 3, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'XMLEvent'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Protocol'));
                   
     Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'XMLEvent'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Protocol'
                   and rownum < 2;


    BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'Backend Integration'
                    and R.RoutingObjCd = 'Backend Integration'
                    and R.RoutingObjVer = '1.0';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;

--   Insert records for EDI-X12 protocol  ------------------3rd procotol  

    v_RoutingObjId := '';

    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'EDI-X12', 'EDI-X12', 'ALL', 1, 'EDI-X12', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'EDI-X12'
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Protocol'));
     
                   
    Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
           inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
           Where o.RoutingObjCd = 'EDI-X12'
           and   o.Routingobjver = 'ALL'
           and   t.routingobjtypekey = 'Protocol'
           and rownum < 2;

    BEGIN
      If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'AS'
                    and R.RoutingObjCd = 'AS'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;
   
   BEGIN
      If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'None'
                    and R.RoutingObjCd = 'None'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;
   
   BEGIN
         If NVL(v_RoutingObjId,0) != 0 THEN   
   
           Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                   select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                   from 
                       CF_RoutingObject R
                       INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                       INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                   where
                       T.RoutingObjTypeKey = 'Package'
                       and R.RoutingObjName = 'N/A'
                       and R.RoutingObjCd = 'N/A'
                       and R.RoutingObjVer = 'N/A';
        end if;       
   
        EXCEPTION
           When DUP_VAL_ON_INDEX then NULL;
   END;
   
   --   Insert records for EDI-EDIFACT protocol  ------------------4th procotol  

    v_RoutingObjId := '';

    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'EDI-EDIFACT', 'EDI-EDIFACT', 'ALL', 1, 'EDI-EDIFACT', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'EDI-EDIFACT'
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Protocol'));
                   
                   
     Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                  inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                  Where o.RoutingObjCd = 'EDI-EDIFACT'
                  and   o.Routingobjver = 'ALL'
                  and   t.routingobjtypekey = 'Protocol'                   
                  and rownum < 2;
     
    BEGIN
      If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'AS'
                    and R.RoutingObjCd = 'AS'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;
   
  BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'None'
                    and R.RoutingObjCd = 'None'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;
   
   BEGIN
       If NVL(v_RoutingObjId,0) != 0 THEN   
   
           Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                   select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                   from 
                       CF_RoutingObject R
                       INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                       INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                   where
                       T.RoutingObjTypeKey = 'Package'
                       and R.RoutingObjName = 'N/A'
                       and R.RoutingObjCd = 'N/A'
                       and R.RoutingObjVer = 'N/A';
        end if;       
   
        EXCEPTION
           When DUP_VAL_ON_INDEX then NULL;
   END;

  --   Insert records for EDI-Consent protocol  ------------------ 5th procotol  

    v_RoutingObjId := '';

    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'EDI-Consent', 'EDI-Consent', 'ALL', 1, 'EDI-Consent', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'EDI-Consent'
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Protocol'));
                   
    Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'EDI-Consent'
                   and   o.Routingobjver = 'ALL'
                   and   t.routingobjtypekey = 'Protocol'
                   and rownum < 2;

    BEGIN
      If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'AS'
                    and R.RoutingObjCd = 'AS'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;
   
  BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'None'
                    and R.RoutingObjCd = 'None'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;
	
	BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'N/A'
                    and R.RoutingObjCd = 'N/A'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;

  --   Insert records for cXML protocol  ------------------ 6th procotol  

    v_RoutingObjId := '';

    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'cXML', 'cXML', '1.2.009', 1, 'cXML', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'cXML'
                   and   o.Routingobjver = '1.2.009'
                   and   t.routingobjtypekey = 'Protocol'));
                   
     Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'cXML'
                   and   o.Routingobjver = '1.2.009'
                   and   t.routingobjtypekey = 'Protocol'
                   and rownum < 2;
      
  BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'None'
                    and R.RoutingObjCd = 'None'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;  
   
    --   Insert records for Web Service protocol  ------------------ 7th procotol  

    v_RoutingObjId := '';

    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'Web Service', 'Web Service', '1.0', 1, 'Web Service', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'Web Service'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Protocol'));

    Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
     inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'Web Service'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Protocol'
                   and rownum < 2;
      
  BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjName = 'None'
                    and R.RoutingObjCd = 'None'
                    and R.RoutingObjVer = 'N/A';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END; 
 
    --   Insert records for ebXML MS protocol  ------------------ 8th procotol  

    v_RoutingObjId := 0;

    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'ebMS', 'ebMS', '2.0', 1, 'ebXML Message Service Protocol', 1, 7, 0
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Protocol'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'ebMS'
                   and   o.Routingobjver = '2.0'
                   and   t.routingobjtypekey = 'Protocol'));
                   
    Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
     inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'ebMS'
                   and   o.Routingobjver = '2.0'
                   and   t.routingobjtypekey = 'Protocol'
                   and rownum < 2;
      
  BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjCd = 'ebMS'
                    and R.RoutingObjVer = '2.0';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END; 
  BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Package'
                    and R.RoutingObjCd = 'Backend Integration'
                    and R.RoutingObjVer = '1.0';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END; 
 
------------------
---1ST PROCESS  Binary 1.0
------------------

    v_RoutingObjId := '';

--Insert process  object
  
    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'Binary', 'Binary', '1.0', 1, 'Binary Process', 1, 7,1
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Process'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'Binary'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Process'));
                   
       Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'Binary'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Process'
                   and rownum < 2;
   BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Protocol'
                    and R.RoutingObjName = 'Binary'
                    and R.RoutingObjCd = 'Binary'
                    and R.RoutingObjVer = '1.0';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;

-- END;

------------------
---2nd PROCESS  XMLEvent 1.0
------------------

    v_RoutingObjId := '';

    Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
    (select
        CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'XMLEvent', 'XMLEvent', '1.0', 1, 'XMLEvent Process Definition', 1, 3, 1
     from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Process'
     AND Not exists(Select * From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'XMLEvent'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Process'));
                   
      Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                   inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                   Where o.RoutingObjCd = 'XMLEvent'
                   and   o.Routingobjver = '1.0'
                   and   t.routingobjtypekey = 'Process'
                   and rownum < 2;

   -- insert Process record in lnk table for routing object record just entered
  BEGIN
    If NVL(v_RoutingObjId,0) != 0 THEN   

        Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                from 
                    CF_RoutingObject R
                    INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                    INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                where
                    T.RoutingObjTypeKey = 'Protocol'
                    and R.RoutingObjName = 'XMLEvent'
                    and R.RoutingObjCd = 'XMLEvent'
                    and R.RoutingObjVer = '1.0';
     end if;       

     EXCEPTION
        When DUP_VAL_ON_INDEX then NULL;
   END;
   
 
  ------------------
  ---WDI/EDI related processes
  ------------------
  -- ISA Process for EDI-X12 Protocol
  
      v_RoutingObjId := '';
  
      Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
      (select
          CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'ISA', 'ISA', 'ALL', 1, 'ISA', 1, 7, 1
       from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Process'
       AND Not exists(Select * From CF_RoutingObject o 
                     inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                     Where o.RoutingObjCd = 'ISA'
                     and   o.Routingobjver = 'ALL'
                     and   t.routingobjtypekey = 'Process'));
                     
       Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                     inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                     Where o.RoutingObjCd = 'ISA'
                     and   o.Routingobjver = 'ALL'
                     and   t.routingobjtypekey = 'Process'
                     and rownum < 2;
  
     -- insert Process record in lnk table for routing object record just entered
    BEGIN
      If NVL(v_RoutingObjId,0) != 0 THEN   
  
          Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                  select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                  from 
                      CF_RoutingObject R
                      INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                      INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                  where
                      T.RoutingObjTypeKey = 'Protocol'
                      and R.RoutingObjName = 'EDI-X12'
                      and R.RoutingObjCd = 'EDI-X12'
                      and R.RoutingObjVer = 'ALL';
       end if;       
  
       EXCEPTION
          When DUP_VAL_ON_INDEX then NULL;
     END;
 
   -- UNB Process for EDI-EDIFACT Protocol
   
       v_RoutingObjId := '';
   
       Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
       (select
           CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'UNB', 'UNB', 'ALL', 1, 'UNB', 1, 7, 1
        from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Process'
        AND Not exists(Select * From CF_RoutingObject o 
                      inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                      Where o.RoutingObjCd = 'UNB'
                      and   o.Routingobjver = 'ALL'
                      and   t.routingobjtypekey = 'Process'));
                      
        Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                      inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                      Where o.RoutingObjCd = 'UNB'
                      and   o.Routingobjver = 'ALL'
                      and   t.routingobjtypekey = 'Process'
                      and rownum < 2;
   
      -- insert Process record in lnk table for routing object record just entered
     BEGIN
       If NVL(v_RoutingObjId,0) != 0 THEN   
   
           Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                   select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                   from 
                       CF_RoutingObject R
                       INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                       INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                   where
                       T.RoutingObjTypeKey = 'Protocol'
                       and R.RoutingObjName = 'EDI-EDIFACT'
                       and R.RoutingObjCd = 'EDI-EDIFACT'
                       and R.RoutingObjVer = 'ALL';
        end if;       
   
        EXCEPTION
           When DUP_VAL_ON_INDEX then NULL;
   END;
   
   
   -- BG Process for EDI-Consent Protocol
   
       v_RoutingObjId := '';
   
       Insert into CF_RoutingObject (RoutingObjId, RoutingObjTypeId, RoutingObjName, RoutingObjCd, RoutingObjVer, ChorgSeqNbr, RoutingObjDesc, ActiveStatusCd, VisibilityCd, DocumentFlg)
       (select
           CF_RoutingObject_Seq.nextval, RoutingObjTypeId,  'BG', 'BG', 'ALL', 1, 'BG', 1, 7, 1
        from  CF_RoutingObjType  Where RoutingObjTypeKey = 'Process'
        AND Not exists(Select * From CF_RoutingObject o 
                      inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                      Where o.RoutingObjCd = 'BG'
                      and   o.Routingobjver = 'ALL'
                      and   t.routingobjtypekey = 'Process'));
                      
        Select RoutingObjId into v_RoutingObjId From CF_RoutingObject o 
                      inner join CF_RoutingObjType t on t.RoutingObjTypeId = o.RoutingObjTypeId
                      Where o.RoutingObjCd = 'BG'
                      and   o.Routingobjver = 'ALL'
                      and   t.routingobjtypekey = 'Process'
                      and rownum < 2;
   
      -- insert Process record in lnk table for routing object record just entered
     BEGIN
       If NVL(v_RoutingObjId,0) != 0 THEN   
   
           Insert into CF_RoutingObjRef_Lnk (RoutingObjRefId ,RoutingObjId, RoutingObjParentRefId) 
                   select CF_RoutingObjRef_Lnk_Seq.nextval,v_RoutingObjId, L.RoutingObjRefId
                   from 
                       CF_RoutingObject R
                       INNER JOIN CF_RoutingObjType T on T.RoutingObjTypeId = R.RoutingObjTypeId
                       INNER JOIN CF_RoutingObjRef_Lnk L on L.RoutingObjId = R.RoutingObjId
                   where
                       T.RoutingObjTypeKey = 'Protocol'
                       and R.RoutingObjName = 'EDI-Consent'
                       and R.RoutingObjCd = 'EDI-Consent'
                       and R.RoutingObjVer = 'ALL';
        end if;       
   
        EXCEPTION
           When DUP_VAL_ON_INDEX then NULL;
   END;
   ------this is for CF_VALIDOBJINTERACT Section  AS to Backend Integration

     v_InterActObjId :='';
 
    Insert into CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
    Select  CF_ValidobjInterAct_Seq.nextval,(select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'    and RoutingObjVer = '1.0'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS'     and GrParentObjVer = 'N/A'),
                                            (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'Backend Integration' and GrParentObjVer = '1.0'),7 from Dual
    Where Not Exists(Select * from CF_ValidobjInteract
                     where FrRoutingObjRefId =
                                             (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS'     and GrParentObjVer = 'N/A')
                      and  ToRoutingObjRefId =  (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'Backend Integration' and GrParentObjVer = '1.0'));

   Select InteractId into v_InterActObjId from CF_ValidobjInteract
          where FrRoutingObjRefId = (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS'     and GrParentObjVer = 'N/A')
          and   ToRoutingObjRefId =  (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'Backend Integration' and GrParentObjVer = '1.0');  
                                        
                                                                                             
-----Insert into Pr_WorkFlowInteract_Lnk
 BEGIN
  Insert into Pr_WorkFlowInteract_lnk(WorkflowId,InterActId)
  Select WorkflowId, v_InterActObjId from pr_workflow where WorkFlowName = 'PassThru';

  EXCEPTION
     When DUP_VAL_ON_INDEX then NULL;
   END;

   ------this is for CF_VALIDOBJINTERACT Section  Backend Integration to AS

     v_InterActObjId :='';
 
    Insert into CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
    Select  CF_ValidobjInterAct_Seq.nextval,(select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary' and RoutingObjVer = '1.0'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'Backend Integration' and GrParentObjVer = '1.0'),
                                            (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package' 
                                                and GrParentObjCd = 'AS' and GrParentObjVer = 'N/A'),7 from Dual
    Where Not Exists(Select * from CF_ValidobjInteract
                     where FrRoutingObjRefId =
                                             (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'Backend Integration'     and GrParentObjVer = '1.0')
                      and  ToRoutingObjRefId =  (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS' and GrParentObjVer = 'N/A'));
                                            

   Select InteractId into v_InterActObjId from CF_ValidobjInteract
    where FrRoutingObjRefId =  (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'Backend Integration'     and GrParentObjVer = '1.0')
    and  ToRoutingObjRefId =  (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS' and GrParentObjVer = 'N/A')
    and rownum < 2;                                                


-----Insert into Pr_WorkFlowInteract_Lnk
 BEGIN
  Insert into Pr_WorkFlowInteract_lnk(WorkflowId,InterActId)
  Select WorkflowId, v_InterActObjId from pr_workflow where WorkFlowName = 'PassThru';

  EXCEPTION
     When DUP_VAL_ON_INDEX then NULL;
   END;

  
    ------this is for CF_VALIDOBJINTERACT Section  AS to None

     v_InterActObjId :='';
 
    Insert into CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
    Select  CF_ValidobjInterAct_Seq.nextval,(select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'   and RoutingObjVer = '1.0'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS'     and GrParentObjVer = 'N/A'),
                                            (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'None' and GrParentObjVer = 'N/A'),7 from Dual
    Where Not Exists(Select * from CF_ValidobjInteract
                     where FrRoutingObjRefId =
                                             (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS'     and GrParentObjVer = 'N/A')
                      and  ToRoutingObjRefId =  (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'None' and GrParentObjVer = 'N/A'));

 Select InteractId into v_InterActObjId from CF_ValidobjInteract
                             where FrRoutingObjRefId =
                                             (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS'     and GrParentObjVer = 'N/A')
                             and  ToRoutingObjRefId =  (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'None' and GrParentObjVer = 'N/A')
                             and rownum < 2;                                                
   
-----Insert into Pr_WorkFlowInteract_Lnk
 BEGIN
  Insert into Pr_WorkFlowInteract_lnk(WorkflowId,InterActId)
  Select WorkflowId, v_InterActObjId from pr_workflow where WorkFlowName = 'PassThru';

  EXCEPTION
     When DUP_VAL_ON_INDEX then NULL;
   END;

   ------this is for CF_VALIDOBJINTERACT Section  None to AS

     v_InterActObjId :='';
 
    Insert into CF_ValidobjInteract (InteractId,FrRoutingObjRefId,ToRoutingObjRefId,EnabledCd)
    Select  CF_ValidobjInterAct_Seq.nextval,(select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'  and RoutingObjVer = '1.0'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'None' and GrParentObjVer = 'N/A'),
                                            (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package' 
                                                and GrParentObjCd = 'AS' and GrParentObjVer = 'N/A'),7 from dual
    Where Not Exists(Select * from CF_ValidobjInteract
                     where FrRoutingObjRefId =
                                             (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'None' and GrParentObjVer = 'N/A')
                      and  ToRoutingObjRefId =  (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS' and GrParentObjVer = 'N/A'));

  Select InteractId into v_InterActObjId from CF_ValidobjInteract
                             where FrRoutingObjRefId =
                                         (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'None' and GrParentObjVer = 'N/A')
                             and  ToRoutingObjRefId =  (select  RoutingObjRefId
                                             from 
                                                CF_ROUTINGOBJECT03_VW 
                                             where
                                                RoutingObjTypeKey = 'Process' and RoutingObjCd = 'Binary'
                                                and RoutingObjVer = '1.0'     and GrParentObjTypeKey = 'Package'
                                                and GrParentObjCd = 'AS' and GrParentObjVer = 'N/A')
                             and rownum < 2;  

-----Insert into Pr_WorkFlowInteract_Lnk
 BEGIN
  Insert into Pr_WorkFlowInteract_lnk(WorkflowId,InterActId)
  Select WorkflowId, v_InterActObjId from pr_workflow where WorkFlowName = 'PassThru';

  EXCEPTION
     When DUP_VAL_ON_INDEX then NULL;
   END;

 EXCEPTION
    when no_data_found then null;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 
/*
Creating Routing Objects
	================================================================================
	Proc Name:       CF_ROUTINGOBJECTI_AH3
	Desc:	*Inserts metadata for RoutingObject into  CF_RoutingObject
		*Inserts metadata into CF_RoutingObjRef_Lnk such that
		- Links the RO to given parentRO 
		- needs to be invoked for each "RO -> ParentRO" combination
		*Output lists the RO and its ParentRefIds
	================================================================================
	---RoutingObjType, RoutingObjCd, RoutingObjVer, RoutingObjName, RoutingObjDesc, ChorgSeqNbr, ActiveStatusCd, VisibilityCd, DocumentFlg
	------ ParentType, ParentCode, ParentVer
*/
-- SET SERVEROUTPUT ON SIZE 1000000
Create or Replace procedure metadata_insert as
------ Main --------
BEGIN
	CF_RoutingObjectI_AH3('Package', 'AS', 'N/A', 'AS', 'AS Packaging', 1, 1, 7, 0, Null, Null, Null);
	CF_RoutingObjectI_AH3('Package', 'None', 'N/A', 'None', 'No Packaging', 1, 1, 7, 0, Null, Null, Null);
	CF_RoutingObjectI_AH3('Package', 'Backend Integration', '1.0', 'Backend Integration', 'Backend Integration Packaging', 1, 1, 3, 0, Null, Null, Null);
	CF_RoutingObjectI_AH3('Package', 'N/A', 'N/A', 'N/A', 'N/A', 1, 1, 7, 0, Null, Null, Null);
	CF_RoutingObjectI_AH3('Package', 'ebMS', '2.0', 'ebMS', 'ebXML Packaging', 1, 1, 7, 0, Null, Null, Null);

	CF_RoutingObjectI_AH3('Protocol', 'Binary', '1.0', 'Binary', 'Binary Protocol', 1, 1, 7, 0, 'Package', 'AS', 'N/A');
	CF_RoutingObjectI_AH3('Protocol', 'Binary', '1.0', 'Binary', 'Binary Protocol', 1, 1, 7, 0, 'Package', 'None', 'N/A');
	CF_RoutingObjectI_AH3('Protocol', 'ErrorFlowDocument', '1.0', 'ErrorFlowDocument', 'ErrorFlowDocument Protocol', 1, 1, 7, 0, 'Package', 'None', 'N/A');
	CF_RoutingObjectI_AH3('Protocol', 'Binary', '1.0', 'Binary', 'Binary Protocol', 1, 1, 7, 0, 'Package', 'Backend Integration', '1.0');
	CF_RoutingObjectI_AH3('Protocol', 'Binary', '1.0', 'Binary', 'Binary Protocol', 1, 1, 7, 0, 'Package', 'N/A', 'N/A');

	CF_RoutingObjectI_AH3('Protocol', 'XMLEvent', '1.0', 'XMLEvent', 'XML Event Definition', 1, 1, 3, 0, 'Package', 'Backend Integration', '1.0');

	CF_RoutingObjectI_AH3('Protocol', 'EDI-X12', 'ALL', 'EDI-X12', 'EDI-X12', 1, 1, 7, 0, 'Package', 'AS', 'N/A');
	CF_RoutingObjectI_AH3('Protocol', 'EDI-X12', 'ALL', 'EDI-X12', 'EDI-X12', 1, 1, 7, 0, 'Package', 'None', 'N/A');
	CF_RoutingObjectI_AH3('Protocol', 'EDI-X12', 'ALL', 'EDI-X12', 'EDI-X12', 1, 1, 7, 0, 'Package', 'N/A', 'N/A');

	CF_RoutingObjectI_AH3('Protocol', 'EDI-EDIFACT', 'ALL', 'EDI-EDIFACT', 'EDI-EDIFACT', 1, 1, 7, 0, 'Package', 'AS', 'N/A');
	CF_RoutingObjectI_AH3('Protocol', 'EDI-EDIFACT', 'ALL', 'EDI-EDIFACT', 'EDI-EDIFACT', 1, 1, 7, 0, 'Package', 'None', 'N/A');
	CF_RoutingObjectI_AH3('Protocol', 'EDI-EDIFACT', 'ALL', 'EDI-EDIFACT', 'EDI-EDIFACT', 1, 1, 7, 0, 'Package', 'N/A', 'N/A');

	CF_RoutingObjectI_AH3('Protocol', 'EDI-Consent', 'ALL', 'EDI-Consent', 'EDI-Consent', 1, 1, 7, 0, 'Package', 'AS', 'N/A');
	CF_RoutingObjectI_AH3('Protocol', 'EDI-Consent', 'ALL', 'EDI-Consent', 'EDI-Consent', 1, 1, 7, 0, 'Package', 'None', 'N/A');
	CF_RoutingObjectI_AH3('Protocol', 'EDI-Consent', 'ALL', 'EDI-Consent', 'EDI-Consent', 1, 1, 7, 0, 'Package', 'N/A', 'N/A');

	CF_RoutingObjectI_AH3('Protocol', 'cXML', '1.2.009', 'cXML', 'cXML', 1, 1, 7, 0, 'Package', 'None', 'N/A');

	CF_RoutingObjectI_AH3('Protocol', 'Web Service', '1.0', 'Web Service', 'Web Service', 1, 1, 7, 0, 'Package', 'None', 'N/A');

	CF_RoutingObjectI_AH3('Protocol', 'ebMS', '2.0', 'ebMS', 'ebXML Message Service Protocol', 1, 1, 7, 0, 'Package', 'ebMS', '2.0');
	CF_RoutingObjectI_AH3('Protocol', 'ebMS', '2.0', 'ebMS', 'ebXML Message Service Protocol', 1, 1, 7, 0, 'Package', 'Backend Integration', '1.0');
	CF_RoutingObjectI_AH3('Protocol', 'ebMS', '2.0', 'ebMS', 'ebXML Message Service Protocol', 1, 1, 7, 0, 'Package', 'N/A', 'N/A');

	CF_RoutingObjectI_AH3('Process', 'Binary', '1.0', 'Binary', 'Binary Process', 1, 1, 7, 1, 'Protocol', 'Binary', '1.0');
	CF_RoutingObjectI_AH3('Process', 'ErrorFlowDocument', '1.0', 'ErrorFlowDocument', 'ErrorFlowDocument Process', 1, 1, 7, 1, 'Protocol', 'ErrorFlowDocument', '1.0');
	
	CF_RoutingObjectI_AH3('Process', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:BCGErrorFlow', '1.0', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:BCGErrorFlow', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:BCGErrorFlow  Process', 1, 1, 7, 0, 'Protocol', 'Web Service', '1.0');

	CF_RoutingObjectI_AH3('Process', 'XMLEvent', '1.0', 'XMLEvent', 'XMLEvent Process Definition', 1, 1, 3, 1, 'Protocol', 'XMLEvent', '1.0');

	CF_RoutingObjectI_AH3('Process', 'ISA', 'ALL', 'ISA', 'ISA', 1, 1, 7, 1, 'Protocol', 'EDI-X12', 'ALL');
	CF_RoutingObjectI_AH3('Process', 'UNB', 'ALL', 'UNB', 'UNB', 1, 1, 7, 1, 'Protocol', 'EDI-EDIFACT', 'ALL');
	CF_RoutingObjectI_AH3('Process', 'BG', 'ALL', 'BG', 'BG', 1, 1, 7, 1, 'Protocol', 'EDI-Consent', 'ALL');

	CF_RoutingObjectI_AH3('Process', 'MSHService', '2.0', 'MSHService', 'MSHService', 1, 1, 7, 0, 'Protocol', 'ebMS', '2.0');

	CF_RoutingObjectI_AH3('Activity', 'MSHService', '2.0', 'MSHService', 'MSHService', 1, 1, 7, 0, 'Process', 'MSHService', '2.0');
	
	CF_RoutingObjectI_AH3('Activity', 'ErrorFlowDocumentOperation', '1.0', 'ErrorFlowDocumentOperation', 'ErrorFlowDocumentOperation', 1, 1, 7, 0, 'Process', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:BCGErrorFlow', '1.0');
	
	CF_RoutingObjectI_AH3('Activity', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument', '1.0', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument', 1, 1, 7, 0, 'Process', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:BCGErrorFlow', '1.0');

	CF_RoutingObjectI_AH3('Action', 'Ping', 'N/A', 'Ping', 'ebXML Ping', 1, 1, 7, 1, 'Activity', 'MSHService', '2.0');
	CF_RoutingObjectI_AH3('Action', 'StatusRequest', 'N/A', 'StatusRequest', 'ebXML Status Request', 1, 1, 7, 1, 'Activity', 'MSHService', '2.0');
	CF_RoutingObjectI_AH3('Action', 'MessageError', 'N/A', 'MessageError', 'ebXML Message Error', 1, 1, 7, 1, 'Activity', 'MSHService', '2.0');
	CF_RoutingObjectI_AH3('Action', 'Acknowledgment', 'N/A', 'Acknowledgment', 'ebXML Acknowledgment', 1, 1, 7, 1, 'Activity', 'MSHService', '2.0');
	CF_RoutingObjectI_AH3('Action', 'ErrorFlowDocument', '1.0', 'ErrorFlowDocument', 'ErrorFlowDocument', 1, 1, 7, 1, 'Activity', 'ErrorFlowDocumentOperation', '1.0');
	CF_RoutingObjectI_AH3('Action', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument', '1.0', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument', 1, 1, 7, 1, 'Activity', '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument', '1.0');
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 
/*
	================================================================================
	Proc Name:       CF_ValidObjInteractI_AH1

	Desc:	*Inserts metadata for linking a pair of RoutingObjects into CF_ValidobjInteract
		- ROHeir accepts Routing Object Heirarchy upto 5 levels (sDl is Separator):
			Package sDl Protocol sDl Process sDl Activity sDl Action
		- Each ROHeir is string of format:
			ROType sTp ROCode sVr ROVersion
			(sTp is delimiter for RO Type, sVr is delimiter for ROCode)
		- String can contain SQL wildcards ("%") and uses LIKE operator
		- ROHeir should uniquely identify a RoutingObjectRefId

		*Inserts metadata into Pr_WorkFlowInteract_lnk such that
		- Links the Interaction to WorkFlow
		- Workflow parameter accepts the workflowname (string)

*/
Create or Replace procedure metadata_insert as
	sTp	VARCHAR2(1)	 := '$';
	sVr	VARCHAR2(1)	 := '@';
	sDl	VARCHAR2(1)	 := '+';
BEGIN
	CF_ValidObjInteractI_AH1('Package'||sTp||'AS'||sVr||'N/A'||'%'||sDl||'Process'||sTp||'Binary'||sVr||'1.0'
				,'Package'||sTp||'Backend Integration'||sVr||'1.0'||'%'||sDl||'Process'||sTp||'Binary'||sVr||'1.0'
				,'PassThru');
	CF_ValidObjInteractI_AH1('Package'||sTp||'Backend Integration'||sVr||'1.0'||'%'||sDl||'Process'||sTp||'Binary'||sVr||'1.0'
				,'Package'||sTp||'AS'||sVr||'N/A'||'%'||sDl||'Process'||sTp||'Binary'||sVr||'1.0'
				,'PassThru');


	CF_ValidObjInteractI_AH1('Package'||sTp||'AS'||sVr||'N/A'||'%'||sDl||'Process'||sTp||'Binary'||sVr||'1.0'
				,'Package'||sTp||'None'||sVr||'N/A'||'%'||sDl||'Process'||sTp||'Binary'||sVr||'1.0'
				,'PassThru');
	CF_ValidObjInteractI_AH1('Package'||sTp||'None'||sVr||'N/A'||'%'||sDl||'Process'||sTp||'Binary'||sVr||'1.0'
				,'Package'||sTp||'AS'||sVr||'N/A'||'%'||sDl||'Process'||sTp||'Binary'||sVr||'1.0'
				,'PassThru');
				
	CF_ValidObjInteractI_AH1('Package'||sTp||'None'||sVr||'N/A'||'%'||sDl||'Process'||sTp||'ErrorFlowDocument'||sVr||'1.0'
				,'Package'||sTp||'None'||sVr||'N/A'||'%'||sDl||'Process'||sTp||'ErrorFlowDocument'||sVr||'1.0'
				,'PassThru');
				

CF_ValidObjInteractI_AH1('Package'||sTp||'ebMS'||sVr||'2.0'||'%'||sDl||'Action'||sTp||'Ping'||sVr||'N/A'
			,'Package'||sTp||'N/A'||sVr||'N/A'||'%'||sDl||'Action'||sTp||'Ping'||sVr||'N/A'
			,'PassThru');
CF_ValidObjInteractI_AH1('Package'||sTp||'N/A'||sVr||'N/A'||'%'||sDl||'Action'||sTp||'Ping'||sVr||'N/A'
			,'Package'||sTp||'ebMS'||sVr||'2.0'||'%'||sDl||'Action'||sTp||'Ping'||sVr||'N/A'
			,'PassThru');
CF_ValidObjInteractI_AH1('Package'||sTp||'ebMS'||sVr||'2.0'||'%'||sDl||'Action'||sTp||'StatusRequest'||sVr||'N/A'
			,'Package'||sTp||'N/A'||sVr||'N/A'||'%'||sDl||'Action'||sTp||'StatusRequest'||sVr||'N/A'
			,'PassThru');
CF_ValidObjInteractI_AH1('Package'||sTp||'N/A'||sVr||'N/A'||'%'||sDl||'Action'||sTp||'StatusRequest'||sVr||'N/A'
			,'Package'||sTp||'ebMS'||sVr||'2.0'||'%'||sDl||'Action'||sTp||'StatusRequest'||sVr||'N/A'
			,'PassThru');
CF_ValidObjInteractI_AH1('Package'||sTp||'ebMS'||sVr||'2.0'||'%'||sDl||'Action'||sTp||'MessageError'||sVr||'N/A'
			,'Package'||sTp||'N/A'||sVr||'N/A'||'%'||sDl||'Action'||sTp||'MessageError'||sVr||'N/A'
			,'PassThru');
CF_ValidObjInteractI_AH1('Package'||sTp||'N/A'||sVr||'N/A'||'%'||sDl||'Action'||sTp||'MessageError'||sVr||'N/A'
			,'Package'||sTp||'ebMS'||sVr||'2.0'||'%'||sDl||'Action'||sTp||'MessageError'||sVr||'N/A'
			,'PassThru');
CF_ValidObjInteractI_AH1('Package'||sTp||'ebMS'||sVr||'2.0'||'%'||sDl||'Action'||sTp||'Acknowledgment'||sVr||'N/A'
			,'Package'||sTp||'N/A'||sVr||'N/A'||'%'||sDl||'Action'||sTp||'Acknowledgment'||sVr||'N/A'
			,'PassThru');
CF_ValidObjInteractI_AH1('Package'||sTp||'N/A'||sVr||'N/A'||'%'||sDl||'Action'||sTp||'Acknowledgment'||sVr||'N/A'
			,'Package'||sTp||'ebMS'||sVr||'2.0'||'%'||sDl||'Action'||sTp||'Acknowledgment'||sVr||'N/A'
			,'PassThru');

END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 
-------------------------------------------------------------------------------------------
--***********************************
-------------------------------------------------------------------------------------------

---Inserts for CF_RoutingObjAttr_lnk 

---- NONE N/A
   Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASBusinessId')
--                                               AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
--                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                                              where a.AttributeKey = 'ASBusinessId'))
    from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'None' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'None' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'None'
                                                                             and a.AttributeKey = 'ASBusinessId')
/
---- AS N/A
   Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASCompressBeforeSign') 
    --                                           AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
    --                                                          INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
    --                                                          where a.AttributeKey = 'ASCompressBeforeSign'))
    from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             and a.AttributeKey = 'ASCompressBeforeSign')
/
   
------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
     select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASCompressed')
--                                               AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
--                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                                              where a.AttributeKey = 'ASCompressed'))
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASCompressed')
/
   
    
------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASMessageDigestAlgorithm') 
 --                                              AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
 --                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
 --                                                             where a.AttributeKey = 'ASMessageDigestAlgorithm')) 
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASMessageDigestAlgorithm')
/
------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASEncrypted') 
 --                                              AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
 --                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
 --                                                             where a.AttributeKey = 'ASEncrypted'))   
      from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASEncrypted')
/
   

------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
     select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASMdnEmailAddress') 
  --                                             AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
  --                                                            INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
  --                                                            where a.AttributeKey = 'ASMdnEmailAddress')) 
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASMdnEmailAddress')
/
   

------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
     select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASMdnHttpUrl') 
 --                                              AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
 --                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
 --                                                             where a.AttributeKey = 'ASMdnHttpUrl')) 
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASMdnHttpUrl')
/
   
------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASMDNAsynchronous') 
--                                               AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
--                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                                              where a.AttributeKey = 'ASMDNAsynchronous'))   
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASMDNAsynchronous')
/
------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
     select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASMDNRequested' )
--                                               AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
--                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                                              where a.AttributeKey = 'ASMDNRequested'))  
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASMDNRequested')
/

------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    Select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASMDNSigned')
--                                               AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
--                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                                              where a.AttributeKey = 'ASMDNSigned'))   
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASMDNSigned')
/
                                                                 
------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASSigned')
  --                                             AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
  --                                                            INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
  --                                                            where a.AttributeKey = 'ASSigned')) 
    from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASSigned')
/
   
 
------
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'TimeToAcknowledge') 
                                              -- AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
                                               --               INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                               --               where a.AttributeKey = 'TimeToAcknowledge'))      
    from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'TimeToAcknowledge')
/
                                                               
------
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'RetryCount')
                                             --  AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
                                             --                 INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                             --                 where a.AttributeKey = 'RetryCount'))      
    from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'RetryCount')
/
                                                                             
--47766
--Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
--    select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'OneWay')
--  --                                             AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
--  --                                                            INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--  --                                                            where a.AttributeKey = 'OneWay'))      
--    from cf_RoutingObjRef_lnk l
--     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
--     Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
--                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
--                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
--                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                                                             Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' and R.RoutingobjName = 'Web Service'
--                                                                             AND a.AttributeKey = 'OneWay')
/
                                                                             
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    Select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASBusinessId')
--                                               AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
--                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                                              where a.AttributeKey = 'ASBusinessId'))   
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'ASBusinessId')
/
 
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'EnvelopeFlag') 
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'Backend Integration' and r.RoutingObjver = '1.0' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'Backend Integration' and r.RoutingObjver = '1.0' and R.RoutingobjName = 'Backend Integration'
                                                                             AND a.AttributeKey = 'EnvelopeFlag')
/   
--- Changes for EDI Protocol Attributes 4.3
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.ALLOWDUPLICATES'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL' and R.RoutingobjName = 'EDI-X12' 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.ALLOWDUPLICATES'	)	 
/
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.ALLOWDUPLICATES'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL' and R.RoutingobjName = 'EDI-EDIFACT' 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.ALLOWDUPLICATES'	)	 
/
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.ALLOWDUPLICATES'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL' and R.RoutingobjName = 'EDI-Consent' 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.ALLOWDUPLICATES'	)	 
/
-- XML
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.SEGOUTPUT'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'XMLEvent' and r.RoutingObjver = '1.0' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'XMLEvent' and r.RoutingObjver = '1.0' and R.RoutingobjName = 'XMLEvent' 
                                                                              AND a.AttributeKey = 	'bcg.ro.SEGOUTPUT'	)	 
/
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.TRANSFORMATIONERRLEVEL'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'XMLEvent' and r.RoutingObjver = '1.0' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'XMLEvent' and r.RoutingObjver = '1.0' and R.RoutingobjName = 'XMLEvent' 
                                                                              AND a.AttributeKey = 	'bcg.ro.TRANSFORMATIONERRLEVEL'	)	 
/
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.xml.XMLNSACTIVE'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'XMLEvent' and r.RoutingObjver = '1.0' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'XMLEvent' and r.RoutingObjver = '1.0' and R.RoutingobjName = 'XMLEvent' 
                                                                              AND a.AttributeKey = 	'bcg.ro.xml.XMLNSACTIVE'	)	 
/
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.edi.TA1OVERRIDE'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'ISA' and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'ISA' and r.RoutingObjver = 'ALL' and R.RoutingobjName = 'ISA' 
                                                                              AND a.AttributeKey = 	'bcg.ro.edi.TA1OVERRIDE'	)	 
/
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.DISCARDONERROR'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'ISA' and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'ISA' and r.RoutingObjver = 'ALL' and R.RoutingobjName = 'ISA' 
                                                                              AND a.AttributeKey = 	'bcg.ro.DISCARDONERROR'	)	 
/
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.DISCARDONERROR'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'UNB' and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'UNB' and r.RoutingObjver = 'ALL' and R.RoutingobjName = 'UNB' 
                                                                              AND a.AttributeKey = 	'bcg.ro.DISCARDONERROR'	)	 
/
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'bcg.ro.DISCARDONERROR'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'BG' and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'BG' and r.RoutingObjver = 'ALL' and R.RoutingobjName = 'BG' 
                                                                              AND a.AttributeKey = 	'bcg.ro.DISCARDONERROR'	)	 
/
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 	'GuidelineMap'	) 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'XMLEvent' and r.RoutingObjver = 'ALL' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'XMLEvent' and r.RoutingObjver = 'ALL' and R.RoutingobjName = 'XMLEvent' 
									      AND a.AttributeKey = 	'GuidelineMap' )
/

--62289
--INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
--SELECT DISTINCT RoutingobjrefId,(SELECT RoutingAttrId  
--                         FROM cf_routingAttr 
--                         WHERE AttributeKey = 'NonRepudiationRequired') 
--FROM cf_RoutingObjRef_lnk lnk
--/

INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
Select RoutingObjRefId, RoutingAttrId from
(SELECT DISTINCT RoutingobjrefId,(SELECT RoutingAttrId  
                         FROM cf_routingAttr 
                         WHERE AttributeKey = 'NonRepudiationRequired') RoutingAttrId
FROM cf_RoutingObjRef_lnk lnk) a 
Where Not Exists (Select 'x' from CF_RoutingObjAttr_Lnk b where a.RoutingObjRefId = b.RoutingObjRefId
and a.RoutingAttrId =b.RoutingAttrId)
/


Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'bcg.soap.ConsumePayload') 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' and R.RoutingobjName = 'Web Service' 
                                                                              AND a.AttributeKey = 'bcg.soap.ConsumePayload')
/

Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
    select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'bcg.as2.payload.encoding') 
    from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             and a.AttributeKey = 'bcg.as2.payload.encoding')
/


--- End EDI Protocol Attributes requirement changes
------ Insert in PR_RoutingAttr_Lnk 

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId,RoutingAttrId,AttrOptId)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'ASCompressBeforeSign'),
a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'ASCompressBeforeSign' 
and o.AttrvalueKey = 'Y'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'ASCompressBeforeSign'
               and o.AttrvalueKey = 'Y'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'ASCompressBeforeSign'))
/ 

------

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId,RoutingAttrId, AttrOptId)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'ASCompressed'),
a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'ASCompressed' 
and o.AttrvalueKey = 'N'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'ASCompressed'
               and o.AttrvalueKey = 'N'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'ASCompressed'))
/ 


------

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrOptId)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'ASMessageDigestAlgorithm'),
a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'ASMessageDigestAlgorithm' 
and o.AttrvalueKey = 'sha1'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'ASMessageDigestAlgorithm'
               and o.AttrvalueKey = 'sha1'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'ASMessageDigestAlgorithm'))
/ 

------

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId,  AttrOptId)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'ASEncrypted'),
a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'ASEncrypted' 
and o.AttrvalueKey = 'N'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'ASEncrypted'
               and o.AttrvalueKey = 'N'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' 
                                         AND a.AttributeKey = 'ASEncrypted'))
/ 

	------

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrOptId)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'ASMDNAsynchronous'),
a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'ASMDNAsynchronous' 
and o.AttrvalueKey = 'Y'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'ASMDNAsynchronous'
               and o.AttrvalueKey = 'Y'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'ASMDNAsynchronous'))
/ 

------

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrOptId)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'ASMDNRequested'),
a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'ASMDNRequested' 
and o.AttrvalueKey = 'Y'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'ASMDNRequested'
               and o.AttrvalueKey = 'Y'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'ASMDNRequested'))
/ 

------

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrOptId)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'ASMDNSigned'),
a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'ASMDNSigned' 
and o.AttrvalueKey = 'N'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'ASMDNSigned'
               and o.AttrvalueKey = 'N'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'ASMDNSigned'))
/ 
------

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrOptId)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'ASSigned'),
a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'ASSigned' 
and o.AttrvalueKey = 'N'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'ASSigned'
               and o.AttrvalueKey = 'N'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'ASSigned'))
/ 

------ with 3, 30 just do insert of values into PR_RoutingAttr_Lnk table

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrOptId, AttrValue)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'RetryCount'),
a.RoutingAttrId,'','3'
From cf_RoutingAttr a
Where a.AttributeKey = 'RetryCount' 
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               Where  a.AttributeKey = 'RetryCount'
               and l.AttrValue = '3'
               and l.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'RetryCount'))
/ 

------ with 3, 30 just do insert of values into PR_RoutingAttr_Lnk table


Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrOptId, AttrValue)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'TimeToAcknowledge'),
a.RoutingAttrId,'','30'
From cf_RoutingAttr a
Where a.AttributeKey = 'TimeToAcknowledge' 
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               Where  a.AttributeKey = 'TimeToAcknowledge'
               and l.AttrValue = '30'
               and l.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'TimeToAcknowledge'))
/ 
---

-- This is tied to a protocol and not a package.
--47766
--Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrOptId)
--select 
--    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
--                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
--                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
--                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                    Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
--                                    AND a.AttributeKey = 'OneWay'),
--a.RoutingAttrId, o.AttrOptid
--From cf_RoutingAttr a
--INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
--Where a.AttributeKey = 'OneWay' 
--and o.AttrvalueKey = 'N'
--And Not Exists(Select * from PR_RoutingAttr_Lnk l
--               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
--               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
--               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
--               Where  a.AttributeKey = 'OneWay'
--               and o.AttrvalueKey = 'N'
----               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
--                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
--                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
--                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                         Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
--                                         AND a.AttributeKey = 'OneWay'))
/ 

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId, RoutingObjRefId, RoutingAttrId, AttrOptId)
select 
    PR_RoutingAttr_Lnk_Seq.nextval,(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'Backend Integration' and r.RoutingObjver = '1.0'
                                    AND a.AttributeKey = 'EnvelopeFlag'),
a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'EnvelopeFlag' 
and o.AttrvalueKey = 'N'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'EnvelopeFlag'
               and o.AttrvalueKey = 'N'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'Backend Integration' and r.RoutingObjver = '1.0'
                                         AND a.AttributeKey = 'EnvelopeFlag'))
/

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, '10' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALCTRLYY'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALCTRLYY'
               and prl.AttrValue = '10'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, 'CHARSET' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALCHARSET'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALCHARSET'
               and prl.AttrValue = 'CHARSET'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, 'ALPHANUM' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALALPHANUM'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALALPHANUM'
               and prl.AttrValue = 'ALPHANUM'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, '10' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALCTRLYY'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALCTRLYY'
               and prl.AttrValue = '10'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, 'CHARSET' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALCHARSET'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALCHARSET'
               and prl.AttrValue = 'CHARSET'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, 'ALPHANUM' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALALPHANUM'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALALPHANUM'
               and prl.AttrValue = 'ALPHANUM'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, '10' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALCTRLYY'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALCTRLYY'
               and prl.AttrValue = '10'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, 'CHARSET' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALCHARSET'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALCHARSET'
               and prl.AttrValue = 'CHARSET'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, 'ALPHANUM' from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALALPHANUM'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'bcg.ro.edi.VALALPHANUM'
               and prl.AttrValue = 'ALPHANUM'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/ 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.ALLOWDUPLICATES'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'ISA' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.TA1OVERRIDE'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.TA1OVERRIDE'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'ISA' and r.RoutingObjver = 'ALL'))
/

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'ISA' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.DISCARDONERROR'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.DISCARDONERROR'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'ISA' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'UNB' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.DISCARDONERROR'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.DISCARDONERROR'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'UNB' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from 
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'BG' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.DISCARDONERROR'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.DISCARDONERROR'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'BG' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.SEGOUTPUT'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.SEGOUTPUT'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/ 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.SEGOUTPUT'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.SEGOUTPUT'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.SEGOUTPUT'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.SEGOUTPUT'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/ 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.TRANSFORMATIONERRLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/ 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/ 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALLEVEL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALLEVEL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.GRPLVLFA'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.GRPLVLFA'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/ 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.GRPLVLFA'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.GRPLVLFA'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.GRPLVLFA'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.GRPLVLFA'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/ 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.edi.VALSERVSEGVAL'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.DISCARDONERROR'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.DISCARDONERROR'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-Consent' and r.RoutingObjver = 'ALL'))
/ 
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.DISCARDONERROR'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.DISCARDONERROR'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-X12' and r.RoutingObjver = 'ALL'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptid)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid, cro.attroptid from  
                                    cf_RoutingObjref_lnk l , cf_routingattr a, cf_RoutingObject r, cf_routingattropt cro
                                    where r.RoutingObjId = l.RoutingObjId
                                    and r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'
                                    AND a.AttributeKey = 'bcg.ro.DISCARDONERROR'
                                    and cro.routingattrid = a.routingattrid
                                    and cro.defaultflg = 1
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
				                   inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId
				                   inner join cf_routingattropt cro on cro.attroptid = prl.attroptid
				                   Where  a.AttributeKey = 'bcg.ro.DISCARDONERROR'
				                   and cro.defaultflg = 1
				                   and cro.routingattrid = a.routingattrid
				                   and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_RoutingObjref_Lnk l
				                   INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
				                   Where r.RoutingObjCd = 'EDI-EDIFACT' and r.RoutingObjver = 'ALL'))
/

Update CF_ROUTINGOBJECT Set PROTOCOLTYPE =1 Where ROUTINGOBJNAME IN 
('EDI-X12','ISA','EDI-EDIFACT','UNB','EDI-Consent','BG')	
/

--100535
 Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
     select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'NonRepudiationRequired')
--                                               AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
--                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                                              where a.AttributeKey = 'ASMDNRequested'))  
     from cf_RoutingObjRef_lnk l
     Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                                                             INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                                                             INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                                                             Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
                                                                             AND a.AttributeKey = 'NonRepudiationRequired')
/


Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'bcg.soap.ContentValidation') 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' and R.RoutingobjName = 'Web Service' 									      AND a.AttributeKey = 'bcg.soap.ContentValidation' )
/

Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'bcg.soap.ResponseValidation') 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' and R.RoutingobjName = 'Web Service' 									      AND a.AttributeKey = 'bcg.soap.ResponseValidation' )
/

Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId) 
     select  RoutingobjrefId,(Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'bcg.soap.Re-RouteDe-EnvelopedDocument') 
     from cf_RoutingObjRef_lnk l 
      Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId 
      Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l 
                                                                              INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId 
                                                                              INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId 
                                                                              INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId 
                                                                              Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0' and R.RoutingobjName = 'Web Service' 									      AND a.AttributeKey = 'bcg.soap.Re-RouteDe-EnvelopedDocument')
/

-- End of JR27207
   
   
--WPG-WTX INTEGRATION

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.GRPLVLFA')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.RoutingObjCd='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.GRPLVLFA')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.FAMAPNAME')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.RoutingObjCd='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.FAMAPNAME')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALCTRLYY')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.RoutingObjCd='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALCTRLYY')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALALPHANUM')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.RoutingObjCd='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALALPHANUM')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALCHARSET')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.RoutingObjCd='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALCHARSET')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALERRLEVEL')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.RoutingObjCd='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALLEVEL')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-X12' AND CRO.RoutingObjCd='EDI-X12' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALLEVEL')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.GRPLVLFA')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.RoutingObjCd='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.GRPLVLFA')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.FAMAPNAME')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.RoutingObjCd='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.FAMAPNAME')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALCTRLYY')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.RoutingObjCd='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALCTRLYY')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALALPHANUM')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.RoutingObjCd='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALALPHANUM')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALCHARSET')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.RoutingObjCd='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALCHARSET')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALERRLEVEL')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.RoutingObjCd='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALLEVEL')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-EDIFACT' AND CRO.RoutingObjCd='EDI-EDIFACT' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALLEVEL')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.GRPLVLFA')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.RoutingObjCd='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.GRPLVLFA')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.FAMAPNAME')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.RoutingObjCd='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.FAMAPNAME')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALCTRLYY')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.RoutingObjCd='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALCTRLYY')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALALPHANUM')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.RoutingObjCd='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALALPHANUM')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALCHARSET')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.RoutingObjCd='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALCHARSET')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALERRLEVEL')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.RoutingObjCd='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALERRLEVEL')
/

INSERT INTO CF_ROUTINGOBJATTR_LNK(
	ROUTINGOBJREFID
	,ROUTINGATTRID
	)
SELECT	CRL.ROUTINGOBJREFID,
	(SELECT CRA.ROUTINGATTRID FROM CF_ROUTINGATTR CRA WHERE CRA.ATTRIBUTEKEY='bcg.ro.edi.VALLEVEL')
FROM CF_ROUTINGOBJREF_LNK CRL
		INNER JOIN CF_ROUTINGOBJECT CRO ON CRL.ROUTINGOBJID=CRO.ROUTINGOBJID 
		WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
And Not Exists(Select CAL.RoutingObjRefId from CF_ROUTINGOBJATTR_LNK CAL
           INNER JOIN CF_ROUTINGOBJREF_LNK CRL on CRL.RoutingObjrefId = CAL.RoutingObjRefId
           INNER JOIN CF_ROUTINGOBJECT CRO ON CRO.RoutingObjId = CRL.RoutingObjId
           INNER JOIN cf_routingAttr a on a.RoutingAttrId = CAL.RoutingAttrId
           WHERE CRO.ROUTINGOBJNAME='EDI-Consent' AND CRO.RoutingObjCd='EDI-Consent' AND CRO.ROUTINGOBJVER='ALL'
           AND a.AttributeKey = 'bcg.ro.edi.VALLEVEL')
/

-- END WPG-WTX Integration

--100836
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptId)
select PR_RoutingAttr_Lnk_Seq.nextval,
    (Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'NonRepudiationRequired'),
    a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'NonRepudiationRequired' 
and o.AttrvalueKey = 'Y'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'NonRepudiationRequired'
               and o.AttrvalueKey = 'Y'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'NonRepudiationRequired'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptId)
select PR_RoutingAttr_Lnk_Seq.nextval,
    (Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'Backend Integration' and r.RoutingObjver = '1.0'
                                    AND a.AttributeKey = 'NonRepudiationRequired'),
    a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'NonRepudiationRequired' 
and o.AttrvalueKey = 'Y'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'NonRepudiationRequired'
               and o.AttrvalueKey = 'Y'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'Backend Integration' and r.RoutingObjver = '1.0'
                                         AND a.AttributeKey = 'NonRepudiationRequired'))
/
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptId)
select PR_RoutingAttr_Lnk_Seq.nextval,
    (Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'None' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'NonRepudiationRequired'),
    a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'NonRepudiationRequired' 
and o.AttrvalueKey = 'Y'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'NonRepudiationRequired'
               and o.AttrvalueKey = 'Y'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'None' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'NonRepudiationRequired'))
/

--101305 changing default for non rep attribute on N/A package to NO
Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptId)
select PR_RoutingAttr_Lnk_Seq.nextval,
    (Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'N/A' and r.RoutingObjver = 'N/A'
                                    AND a.AttributeKey = 'NonRepudiationRequired'),
    a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'NonRepudiationRequired' 
and o.AttrvalueKey = 'N'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'NonRepudiationRequired'
               and o.AttrvalueKey = 'N'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'N/A' and r.RoutingObjver = 'N/A'
                                         AND a.AttributeKey = 'NonRepudiationRequired'))
/


-- JR27207

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptId)
select PR_RoutingAttr_Lnk_Seq.nextval,
    (Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
                                    AND a.AttributeKey = 'bcg.soap.ContentValidation'),
    a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'bcg.soap.ContentValidation' 
and o.AttrvalueKey = 'Y'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'bcg.soap.ContentValidation'
               and o.AttrvalueKey = 'Y'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
                                         AND a.AttributeKey = 'bcg.soap.ContentValidation'))                                      
/

Insert into PR_RoutingAttr_Lnk ( RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptId)
select PR_RoutingAttr_Lnk_Seq.nextval,
    (Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
                                    AND a.AttributeKey = 'bcg.soap.ResponseValidation'),
    a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'bcg.soap.ResponseValidation' 
and o.AttrvalueKey = 'Y'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'bcg.soap.ResponseValidation'
               and o.AttrvalueKey = 'Y'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
                                         AND a.AttributeKey = 'bcg.soap.ResponseValidation'))                                                                                
/                                      
                                         
                                         
Insert into PR_RoutingAttr_Lnk ( RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptId)
select PR_RoutingAttr_Lnk_Seq.nextval,
    (Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
                                    AND a.AttributeKey = 'bcg.soap.Re-RouteDe-EnvelopedDocument'),
    a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'bcg.soap.Re-RouteDe-EnvelopedDocument' 
and o.AttrvalueKey = 'N'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'bcg.soap.Re-RouteDe-EnvelopedDocument'
               and o.AttrvalueKey = 'N'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
                                         AND a.AttributeKey = 'bcg.soap.Re-RouteDe-EnvelopedDocument'))                                                                       
/        

-- CR102199

-- Defect 108746.  Attrvalue 'Y' gets populated by Console for RNIF PIP. For EDI Attrvalue is null.
--  So Check only for key columns

Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId,attroptid, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid,cp.attroptid, 'YES' from 
                                    CF_ROUTINGOBJECT06_VW l , cf_routingattr a, cf_RoutingObject r,cf_routingattropt cp
                                    where r.RoutingObjId = l.RoutingObjId
                                    AND a.ROUTINGATTRID=cp.ROUTINGATTRID
                                    and l.routingobjlevel =1
                                    AND a.AttributeKey = 'MsgStoreRequired'
                                    AND cp.ATTRVALUEKEY = 'Y'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'MsgStoreRequired'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_ROUTINGOBJECT06_VW l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where l.routingobjlevel = 1 ))
/                                         


/* Insert into PR_RoutingAttr_Lnk (RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId,attroptid, AttrValue)
Select PR_RoutingAttr_Lnk_Seq.nextval,l.RoutingObjRefId, a.routingattrid,cp.attroptid, 'NO' from 
                                    CF_ROUTINGOBJECT06_VW l , cf_routingattr a, cf_RoutingObject r,cf_routingattropt cp
                                    where r.RoutingObjId = l.RoutingObjId
                                    AND a.ROUTINGATTRID=cp.ROUTINGATTRID
                                    and l.routingobjlevel in (2,3)
                                    AND a.AttributeKey = 'MsgStoreRequired'
                                    AND cp.ATTRVALUEKEY = 'N'
And Not Exists(                                 
                                    Select * from PR_RoutingAttr_Lnk prl
               inner join cf_RoutingAttr a on a.RoutingAttrId = prl.RoutingAttrId 
               Where  a.AttributeKey = 'MsgStoreRequired'
               and prl.AttrValue = 'NO'
               and prl.RoutingObjRefId in(Select l.RoutingObjRefId from CF_ROUTINGOBJECT06_VW l
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = l.RoutingObjId
                                         Where l.routingobjlevel in (2,3)))
*/        


-- End of JR27207

-- Feature 104551

Insert into PR_RoutingAttr_Lnk ( RoutingAttrLnkId,RoutingObjRefId, RoutingAttrId, AttrOptId)
select PR_RoutingAttr_Lnk_Seq.nextval,
    (Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                    INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                    INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                    INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                    Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
                                    AND a.AttributeKey = 'bcg.soap.ConsumePayload'),
    a.RoutingAttrId, o.AttrOptid
From cf_RoutingAttr a
INNER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
Where a.AttributeKey = 'bcg.soap.ConsumePayload' 
and o.AttrvalueKey = 'N'
And Not Exists(Select * from PR_RoutingAttr_Lnk l
               inner join cf_RoutingAttr a on a.RoutingAttrId = l.RoutingAttrId 
               inner join cf_RoutingAttropt o on  o.RoutingAttrId = a.RoutingAttrId
               inner join cf_RoutingObjAttr_lnk al on al.RoutingObjrefId = l.RoutingObjRefId
               Where  a.AttributeKey = 'bcg.soap.ConsumePayload'
               and o.AttrvalueKey = 'N'
               and al.RoutingObjRefId =(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
                                         INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
                                         INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
                                         INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
                                         Where r.RoutingObjCd = 'Web Service' and r.RoutingObjver = '1.0'
                                         AND a.AttributeKey = 'bcg.soap.ConsumePayload'))                                                                       
/


--100865
Insert into CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
select  RoutingobjrefId, (Select RoutingAttrId  from cf_routingAttr where AttributeKey = 'ASMdnFtpAddress') 
--                                              AND Not Exists(Select l.RoutingAttrId  from cf_routingObjAttr_Lnk l
--                                                             INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
--                                                             where a.AttributeKey = 'ASMdnFtpAddress')) 
from cf_RoutingObjRef_lnk l
Inner Join cf_RoutingObject r on r.RoutingObjId = l.RoutingObjId
Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' And Not Exists(Select l.RoutingObjRefId from CF_RoutingObjAttr_Lnk l
								     INNER JOIN cf_RoutingObjref_lnk rl on rl.RoutingObjrefId = l.RoutingObjRefId
								     INNER JOIN cf_RoutingObject r ON r.RoutingObjId = rl.RoutingObjId
								     INNER JOIN cf_routingAttr a on a.RoutingAttrId = l.RoutingAttrId
								     Where r.RoutingObjCd = 'AS' and r.RoutingObjver = 'N/A' and R.RoutingobjName = 'AS'
								     AND a.AttributeKey = 'ASMdnFtpAddress')
/

-- CR102199

INSERT INTO CF_RoutingObjAttr_Lnk (RoutingObjRefId, RoutingAttrId)
Select RoutingObjRefId, RoutingAttrId from
(SELECT DISTINCT RoutingobjrefId,(SELECT RoutingAttrId  
                         FROM cf_routingAttr 
                         WHERE AttributeKey = 'MsgStoreRequired'
                         ) RoutingAttrId
FROM CF_ROUTINGOBJECT06_VW lnk where routingobjlevel in (1,2,3)) a 
Where Not Exists (Select 'x' from CF_RoutingObjAttr_Lnk b where a.RoutingObjRefId = b.RoutingObjRefId
and a.RoutingAttrId =b.RoutingAttrId)
/


--100865
COMMIT
/

-- SET SERVEROUTPUT ON SIZE 1000000

Create or Replace procedure metadata_insert as
	v_returncd		NUMBER(1);
	v_refcur		db_ref.RCT;
	v_cnta			NUMBER(10);
	v_cntb			NUMBER(10);
	CURSOR c_ro_ebms IS
		SELECT	l.routingobjrefid refID
		FROM	cf_routingobjref_lnk l
		INNER JOIN cf_routingobject o ON o.routingobjid = l.routingobjid
		INNER JOIN cf_routingobjtype t ON t.routingobjtypeid = o.routingobjtypeid
		WHERE	o.routingobjcd = 'ebMS'
		AND	o.routingobjver = '2.0'
		AND	t.routingobjtypekey IN ('Package', 'Protocol')
		;
	PROCEDURE SetDefaultROAttrValue(p_RORefID	NUMBER
				, p_AttrKey	VARCHAR2
				, p_AttrVal	VARCHAR2)
	IS
		v_ValueString	pr_RoutingAttr_Lnk.AttrOptId%TYPE;
		CURSOR c_attr IS
			SELECT	al.RoutingAttrId ROAttrID
				, o.AttrOptId AttrOptId
				, pl.RoutingAttrLnkId
				, pl.AttrOptId Id_Default
				, pl.AttrValue AttrVal
			FROM	cf_RoutingObjAttr_lnk al
			INNER JOIN cf_RoutingAttr a ON a.RoutingAttrId = al.RoutingAttrId
			LEFT OUTER JOIN cf_RoutingAttropt o on o.RoutingAttrId = a.RoutingAttrId
						AND	o.AttrvalueKey = p_AttrVal
			LEFT OUTER JOIN PR_RoutingAttr_Lnk pl ON pl.RoutingObjRefId = al.RoutingObjRefId
						AND	pl.RoutingAttrId = al.RoutingAttrId
			WHERE	al.RoutingObjRefId = p_RORefId
			AND	a.AttributeKey = p_AttrKey
			;
	BEGIN
		FOR r_attr IN c_attr
		LOOP
			IF r_Attr.RoutingAttrLnkId IS NULL THEN
				IF r_Attr.AttrOptId IS NOT NULL THEN
					INSERT INTO pr_RoutingAttr_Lnk (RoutingAttrLnkId ,RoutingObjRefId, RoutingAttrId, AttrOptId)
					SELECT	PR_RoutingAttr_Lnk_Seq.NEXTVAL
						, p_ROrefId
						, r_attr.ROAttrId
						, r_attr.AttrOptId
					FROM	Dual;
				ELSE
					INSERT INTO pr_RoutingAttr_Lnk (RoutingAttrLnkId ,RoutingObjRefId, RoutingAttrId, AttrValue)
					SELECT	PR_RoutingAttr_Lnk_Seq.NEXTVAL
						, p_ROrefId
						, r_attr.ROAttrId
						, p_AttrVal
					FROM	Dual;
				END IF;
			ELSE
				IF r_Attr.AttrOptId IS NOT NULL THEN
					UPDATE	pr_RoutingAttr_Lnk pl
					SET	AttrOptId = r_attr.AttrOptId
					WHERE	pl.RoutingObjRefId = p_RORefId
					AND	pl.RoutingAttrId = r_attr.ROAttrId;
				ELSE
					UPDATE	pr_RoutingAttr_Lnk pl
					SET	AttrValue = p_AttrVal
					WHERE	pl.RoutingObjRefId = p_RORefId
					AND	pl.RoutingAttrId = r_attr.ROAttrId;
				END IF;
			END IF;
		IF SQL%RowCount > 0 THEN
			dbms_output.put_line( 'RO:' || p_ROrefid || '|Attr:' || p_AttrKey || '|Value:' || p_AttrVal);
		END IF;
		END LOOP;

	END;
BEGIN
	FOR r_ref IN c_ro_ebms
	LOOP
		cf_routingattr_lnkI_cpa( p_returncd => v_returncd
					, p_DebugFlg => 0
					, RC1 => v_refcur
					, p_routingobjrefid => r_ref.refid
			);
		FETCH v_refcur INTO v_cnta, v_cntb;
		dbms_output.put_line( 'RO -- ' || r_ref.refid || ' New ' || v_cnta || ' Exist ' || v_cntb);

		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.ACKREQUESTED',		'bcg.ro.attropt.value.NEVER');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.ACKSIGNATUREREQUESTED',	'bcg.ro.attropt.value.NEVER');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.ACTOR',			'http://schemas.xmlsoap.org/soap/actor/next');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.COMPRESSIONREQUIRED',		'N');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.DUPLICATEELIMINATION',	'bcg.ro.attropt.value.NEVER');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.ENCMIMECONSTITUENT',		'application/xml;text/xml;application/EDI-X12;application/EDI-CONSENT;application/EDIFACT;application/binary;application/octet-stream;application/gzip');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.ENCRYPTIONREQUIRED',		'N');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.HASHFUNCTION',		'bcg.ro.attropt.value.SHA1');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.MESSAGEORDERSEMANTICS',	'bcg.ro.attropt.value.NOTGUARANTEED');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.PERSISTDURATION',		'0');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.RETRYINTERVAL',		'270');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.SYNCREPLYMODE',		'bcg.ro.attropt.value.NONE');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.ISINTELLIGIBLECHECKREQUIRED',	'N');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.CANONICALIZATIONMETHOD',	'bcg.ro.attropt.value.INCLUSIVE_WITH_COMMENTS');
		SetDefaultROAttrValue( r_ref.refid, 'bcg.ro.ebxml.SIGNATUREALGORITHM',	        'bcg.ro.attropt.value.RSA-SHA1');
	END LOOP;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 


-- PR_FIXEDWORKFLOW.sql

-- PR_FIXEDWORKFLOW

Insert Into PR_FIXEDWORKFLOW(FIXEDWORKFLOWID, WORKFLOWNAME, RESOURCEKEY) 
   SELECT 1, 'Inbound', 'user-exit.fixedWorkflow.Input'  FROM DUAL
   Where Not Exists (select 'x' from PR_FIXEDWORKFLOW where 
   FIXEDWORKFLOWID = 1 AND 
   WORKFLOWNAME = 'Inbound')
/   
  

Insert Into PR_FIXEDWORKFLOW(FIXEDWORKFLOWID, WORKFLOWNAME, RESOURCEKEY) 
   SELECT 2,'Outbound', 'user-exit.fixedWorkflow.Output'  FROM DUAL
   Where Not Exists (select 'x' from PR_FIXEDWORKFLOW where 
   FIXEDWORKFLOWID = 2 AND
   WORKFLOWNAME = 'Outbound')
/   
  

-- lg_summaryengine.sql

INSERT INTO LG_SUMMARYENGINE(
    LastRunTS
	)
(select SYS_EXTRACT_UTC(SYSTIMESTAMP) from dUAL
where NOT EXISTS (select 'X' from lg_summaryengine))
/
commit
/




-- DB_SYSPROPERTIES_GROUP.sql

INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'common','common' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'common')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'edi','edi' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'edi')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'consoleGeneral','consoleGeneral' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'consoleGeneral')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'consoleJMS','consoleJMS' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'consoleJMS')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'consoleVTP','consoleVTP' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'consoleVTP')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrAlert','docmgrAlert' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrAlert')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrAsState','docmgrAsState' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrAsState')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrBpeMain','docmgrBpeMain' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrBpeMain')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrBpeSignal','docmgrBpeSignal' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrBpeSignal')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrBpeSync','docmgrBpeSync' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrBpeSync')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrDelivery','docmgrDelivery' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrDelivery')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrJMS','docmgrJMS' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrJMS')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrPackaging','docmgrPackaging' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrPackaging')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrRN','docmgrRN' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrRN')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrSecurity','docmgrSecurity' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrSecurity')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrOther','docmgrOther' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrOther')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'receiverJMS','receiverJMS' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'receiverJMS')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'receiverSync','receiverSync' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'receiverSync')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'receiverDir','receiverDir' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'receiverDir')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'receiverManager','receiverManager' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'receiverManager')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'headerDef','headerDef' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'headerDef')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrSE','docmgrSE' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrSE')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrSS','docmgrSS' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrSS')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'docmgrARC','docmgrARC' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrARC')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'receiverOther','receiverOther' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'receiverOther')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL, 'docmgrEBMS','docmgrEBMS' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrEBMS')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL, 'docmgrRM','docmgrRM' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'docmgrRM')
/

-- Start 55055
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL, 'eventGeneral','eventGeneral' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'eventGeneral')
/
-- End   55055

--START 104485

INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL, 'DEFAULT LISTENER','DEFAULT LISTENER' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'DEFAULT LISTENER')
/

INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL, 'CLIENTAUTH LISTENER','CLIENTAUTH LISTENER' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'CLIENTAUTH LISTENER')
/

INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL, 'CONNECTION PROPERTIES','CONNECTION PROPERTIES' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'CONNECTION PROPERTIES')
/

INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL, 'FTP OTHER PROPERTIES','FTP OTHER PROPERTIES' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'FTP OTHER PROPERTIES')
/

INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL, 'FTP EVENT PROPERTIES','FTP EVENT PROPERTIES' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'FTP EVENT PROPERTIES')
/
INSERT into db_sysproperties_group (SYSPROPGRPID, GROUPNAME, RESOURCEKEY)
Select db_sysproperties_SEQ.NEXTVAL,'externalevent','externalevent' from dual
where not exists (select 'x' from db_sysproperties_group where groupname = 'externalevent')
/
--END F104485

-- db_sysproperties.sql

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.queueName', 'jms/bcg/queue/deliveryQ', 'event.delivery.jms.queueName' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.queueName')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.jndiFactoryName', 'com.ibm.websphere.naming.WsnInitialContextFactory', 'event.delivery.jms.jndiFactoryName' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.jndiFactoryName')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.messageClass', 'defaultmessage', 'event.delivery.jms.messageClass' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.messageClass')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.messageType', 'byte', 'event.delivery.jms.messageType' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.messageType')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.providerURLPackages', '', 'event.delivery.jms.providerURLPackages' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.providerURLPackages')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.queueFactoryName', 'WBIC//QCF', 'event.delivery.jms.queueFactoryName' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.queueFactoryName')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.serverName', 'localhost', 'event.delivery.jms.serverName' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.serverName')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.serverPort', '9999', 'event.delivery.jms.serverPort' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.serverPort')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.on', 'No', 'event.delivery.on' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.on')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.summary.processingInterval', '15', 'event.summary.processingInterval' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME in ('event.summary.processingInterval','bcg.summary.processingInterval'))
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'publicapi.xmlBasedApi', 'false', 'publicapi.xmlBasedApi' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'publicapi.xmlBasedApi')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.messageCharSet', 'UTF-8', 'event.delivery.jms.messageCharSet' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.messageCharSet')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.jms.providerURL', '', 'event.delivery.jms.providerURL' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.providerURL')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.userId', '', 'event.delivery.userId' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.userId')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'event.delivery.password', '', 'event.delivery.password' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.password')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.enveloper.edi.schedule.DAYOFWEEK', '', 'bcg.enveloper.edi.schedule.DAYOFWEEK' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.enveloper.edi.schedule.DAYOFWEEK')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.enveloper.edi.schedule.HOUROPT', '', 'bcg.enveloper.edi.schedule.HOUROPT' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.enveloper.edi.schedule.HOUROPT')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.enveloper.edi.schedule.MINUTEOPT', '', 'bcg.enveloper.edi.schedule.MINUTEOPT' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.enveloper.edi.schedule.MINUTEOPT')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.enveloper.edi.schedule.DAYOFMONTH', '', 'bcg.enveloper.edi.schedule.DAYOFMONTH' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.enveloper.edi.schedule.DAYOFMONTH')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.enveloper.edi.schedule.BATCHMODE', 'true', 'bcg.enveloper.edi.schedule.BATCHMODE' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.enveloper.edi.schedule.BATCHMODE')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.enveloper.edi.schedule.POLLINTERVAL', '60000', 'bcg.enveloper.edi.schedule.POLLINTERVAL' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.enveloper.edi.schedule.POLLINTERVAL')
/

--54425 changed propvalue to false
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'publicapi.soapBasedApi', 'false', 'publicapi.soapBasedApi' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'publicapi.soapBasedApi')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.enveloper.edi.maxlockholdtime', '240000', 'bcg.enveloper.edi.maxlockholdtime' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.enveloper.edi.maxlockholdtime')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.enveloper.edi.maxqueueage', '740000', 'bcg.enveloper.edi.maxqueueage' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.enveloper.edi.maxqueueage')
/

--58747 Perf: Change inbound queue directory lock
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.dae.main.maxLockAge', '180000', 'bcg.dae.main.maxLockAge' FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.dae.main.maxLockAge')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.dae.synchronous.maxLockAge', '180000', 'bcg.dae.synchronous.maxLockAge' FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.dae.synchronous.maxLockAge')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.dae.signal.maxLockAge', '180000', 'bcg.dae.signal.maxLockAge' FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.dae.signal.maxLockAge')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.dae.main.maxfiles.perPass', '5', 'bcg.dae.main.maxfiles.perPass' FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.dae.main.maxfiles.perPass')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.dae.synchronous.maxfiles.perPass', '5', 'bcg.dae.synchronous.maxfiles.perPass' FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.dae.synchronous.maxfiles.perPass')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.dae.signal.maxfiles.perPass', '5', 'bcg.dae.signal.maxfiles.perPass' FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.dae.signal.maxfiles.perPass')
/

--58723
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.docmgr.channelCache.maxSize', '20', 'bcg.docmgr.channelCache.maxSize' FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.docmgr.channelCache.maxSize')
/
--55763
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ldap.containerauth','False', 'bcg.ldap.containerauth' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ldap.containerauth')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ldap.jaaslogin', 'WSLogin', 'bcg.ldap.jaaslogin' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ldap.jaaslogin')
/

-- Start 100289
--UPDATE db_sysproperties SET PROPVALUE = 'WBIC//DeliveryQ' WHERE PROPNAME = 'event.delivery.jms.queueName'
--/
--UPDATE db_sysproperties SET PROPVALUE = 'WBIC//QCF' WHERE PROPNAME = 'event.delivery.jms.queueFactoryName'
--/
-- End 100289

-- Start 103977
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.console.file.encodings', 'us-ascii|ascii|646|iso_646.irv:1983|ansi_x3.4-1968|iso646-us|default|ascii7|utf-8|utf8|unicode-1-1-utf-8|utf-16|utf16|unicode|sjis|\u30B7\u30D5\u30C8\u7B26\u53F7\u5316\u8868\u73FE|pck|gb18030|big5|windows-1255|windows-1256|ISO8859-8|IBM856|ISO8869-6|IBM1046',    'bcg.console.file.encodings',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.file.encodings')
/
-- End 103977


--start 49576
UPDATE db_sysproperties set 
		COMPONENT = 3, 
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain')
  WHERE PROPNAME = 'bcg.dae.main.maxLockAge'
/
UPDATE db_sysproperties set 
		COMPONENT = 3, 
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSync')
  WHERE PROPNAME = 'bcg.dae.synchronous.maxLockAge'
/
UPDATE db_sysproperties set 
		COMPONENT = 3, 
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSignal')
  WHERE PROPNAME = 'bcg.dae.signal.maxLockAge'
/
UPDATE db_sysproperties set 
		COMPONENT = 3, 
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain')
  WHERE PROPNAME = 'bcg.dae.main.maxfiles.perPass'
/
UPDATE db_sysproperties set 
		COMPONENT = 3, 
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSync')
  WHERE PROPNAME = 'bcg.dae.synchronous.maxfiles.perPass'
/
UPDATE db_sysproperties set 
		COMPONENT = 3, 
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSignal')
  WHERE PROPNAME = 'bcg.dae.signal.maxfiles.perPass'
/
UPDATE db_sysproperties set 
		COMPONENT = 3, 
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain')
  WHERE PROPNAME = 'bcg.docmgr.channelCache.maxSize'
/
UPDATE db_sysproperties set 
		COMPONENT = 0, 
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common')
  WHERE PROPNAME = 'bcg.ldap.containerauth'
/
UPDATE db_sysproperties set 
		COMPONENT = 0, 
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common')
  WHERE PROPNAME = 'bcg.ldap.jaaslogin'
/
UPDATE db_sysproperties set
    PROPNAME = 'bcg.summary.processingInterval',
		COMPONENT = 3,
		RESOURCEKEY = 'bcg.summary.processingInterval',
    SYSPROPGRPID = (select SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrSE')
  WHERE PROPNAME = 'event.summary.processingInterval'
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.queue.factory','jms/bcg/cf/RCVRCF','bcg.jms.queue.factory',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.queue.factory' and COMPONENT = 2)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.topic.factory','jms/bcg/cf/RCVRCF','bcg.jms.topic.factory',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.topic.factory' and COMPONENT = 2)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.jndi_factory','com.ibm.websphere.naming.WsnInitialContextFactory','bcg.jms.jndi_factory',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.jndi_factory' and COMPONENT = 2)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.context_url','corbaloc:iiop:<ip-address>:<jndi-port>','bcg.jms.context_url',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.context_url' and COMPONENT = 2)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.oaq_log_q','jms/bcg/queue/datalogQ','bcg.oaq_log_q',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.oaq_log_q' and COMPONENT = 2)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.sync.responseURL','/bcgsyncreceiver/SyncResponse','bcg.receiver.sync.responseURL',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverSync' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.sync.responseURL')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.sync.responseURL.port','$RECEIVER_HTTP_PORT$','bcg.receiver.sync.responseURL.port',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverSync' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.sync.responseURL.port')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.configpath','$SHARED_DATA_DIR$/receiver/config','bcg.receiver.configpath',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverDir' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.configpath')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.vms_receiver_reject_dir','$SHARED_DATA_DIR$/receiver/reject','bcg.vms_receiver_reject_dir',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverDir' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.vms_receiver_reject_dir')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.vms_receiver_tmp_dir','$SHARED_DATA_DIR$/receiver/tmp','bcg.vms_receiver_tmp_dir',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverDir' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.vms_receiver_tmp_dir')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.queue.factory','jms/bcg/cf/DOCMGRCF','bcg.jms.queue.factory',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.queue.factory' and COMPONENT = 3)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.topic.factory','jms/bcg/cf/DOCMGRCF','bcg.jms.topic.factory',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.topic.factory' and COMPONENT = 3)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.jndi_factory','com.ibm.websphere.naming.WsnInitialContextFactory','bcg.jms.jndi_factory',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.jndi_factory' and COMPONENT = 3)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.context_url','corbaloc:iiop:<ip-address>:<jndi-port>','bcg.jms.context_url',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.context_url' and COMPONENT = 3)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.oaq_bpe_in.main','jms/bcg/queue/main_InboundQ','bcg.oaq_bpe_in.main',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.oaq_bpe_in.main')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.oaq_bpe_out.main','jms/bcg/queue/deliveryManagerQ','bcg.oaq_bpe_out.main',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.oaq_bpe_out.main')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.oaq_bpe_in.signal','jms/bcg/queue/signal_InboundQ','bcg.oaq_bpe_in.signal',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.oaq_bpe_in.signal')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.oaq_bpe_out.signal','jms/bcg/queue/deliveryManagerQ','bcg.oaq_bpe_out.signal',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.oaq_bpe_out.signal')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.oaq_bpe_in.synchronous','jms/bcg/queue/sync_InboundQ','bcg.oaq_bpe_in.synchronous',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.oaq_bpe_in.synchronous')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.oaq_bpe_out.synchronous','jms/bcg/queue/deliveryManagerQ','bcg.oaq_bpe_out.synchronous',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.oaq_bpe_out.synchronous')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.queue','jms/bcg/queue/deliveryManagerQ','bcg.delivery.queue',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.queue')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertQueue.queue','jms/bcg/queue/alertQ','bcg.alertQueue.queue',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertQueue.queue')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.eventAlertQReceiver.queue','jms/bcg/queue/alertEventQ','bcg.eventAlertQReceiver.queue',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.eventAlertQReceiver.queue')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.syncdelivery.queue','jms/bcg/queue/syncDeliveryManagerQ','bcg.syncdelivery.queue',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.syncdelivery.queue')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.logReceiver.queue','jms/bcg/queue/datalogQ','bcg.logReceiver.queue',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.logReceiver.queue')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.dberrors.queue','jms/bcg/queue/datalogErrorQ','bcg.dberrors.queue',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.dberrors.queue')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.retry_connect_interval','300000','bcg.jms.retry_connect_interval',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.retry_connect_interval' and COMPONENT = 3)
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.in_thread_count.main','2','bcg.in_thread_count.main',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.in_thread_count.main')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.inbound_poll_interval.main','1000','bcg.inbound_poll_interval.main',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.inbound_poll_interval.main')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.destination.destination_class','com.ibm.bcg.destination.H2DestinationProcess','bcg.destination.destination_class',3 FROM DUAL WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.destination.destination_class')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.bpe_max_file_size','0','bcg.bpe_max_file_size',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.bpe_max_file_size')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.inbound_files_per_pass.main','5','bcg.inbound_files_per_pass.main',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.inbound_files_per_pass.main')
/

-- Start 104206
--INSERT INTO db_sysproperties(
--		SYSPROPID,
--		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
--   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.docmgr.channelCache.expiry', '', 'bcg.docmgr.channelCache.expiry' ,3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.docmgr.channelCache.expiry')
--/
-- End 104206

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField1','x-aux-system-msg-id','bcg.duplicate.DupField1',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField1')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField2','none','bcg.duplicate.DupField2',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField2')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField3','none','bcg.duplicate.DupField3',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField3')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField4','none','bcg.duplicate.DupField4',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField4')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField5','none','bcg.duplicate.DupField5',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField5')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField6','none','bcg.duplicate.DupField6',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField6')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField7','none','bcg.duplicate.DupField7',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField7')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField8','none','bcg.duplicate.DupField8',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField8')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField9','none','bcg.duplicate.DupField9',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField9')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.duplicate.DupField10','none','bcg.duplicate.DupField10',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.duplicate.DupField10')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.inbound_poll_interval.signal','1000','bcg.inbound_poll_interval.signal',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSignal' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.inbound_poll_interval.signal')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.in_thread_count.signal','2','bcg.in_thread_count.signal',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSignal' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.in_thread_count.signal')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.inbound_files_per_pass.signal','5','bcg.inbound_files_per_pass.signal',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSignal' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.inbound_files_per_pass.signal')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.inbound_poll_interval.synchronous','1000','bcg.inbound_poll_interval.synchronous',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSync' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.inbound_poll_interval.synchronous')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.in_thread_count.synchronous','2','bcg.in_thread_count.synchronous',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSync' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.in_thread_count.synchronous')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.inbound_files_per_pass.synchronous','5','bcg.inbound_files_per_pass.synchronous',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeSync' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.inbound_files_per_pass.synchronous')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertQReceiver.maxRetries','100','bcg.alertQReceiver.maxRetries',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertQReceiver.maxRetries')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertQReceiver.retryInterval','60000','bcg.alertQReceiver.retryInterval',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertQReceiver.retryInterval')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.volumeAlertScheduler.allowanceForProcessingReceivedDocInMins','10','bcg.volumeAlertScheduler.allowanceForProcessingReceivedDocInMins',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.volumeAlertScheduler.allowanceForProcessingReceivedDocInMins')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertNotifications.maxNotificationsInInterval','10','bcg.alertNotifications.maxNotificationsInInterval',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertNotifications.maxNotificationsInInterval')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertNotifications.maxNotificationIntervalInMins','30','bcg.alertNotifications.maxNotificationIntervalInMins',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertNotifications.maxNotificationIntervalInMins')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertNotifications.minNotificationQuietIntervalInMins','30','bcg.alertNotifications.minNotificationQuietIntervalInMins',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertNotifications.minNotificationQuietIntervalInMins')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertNotifications.heldAlertsBatchTimeInMins','30','bcg.alertNotifications.heldAlertsBatchTimeInMins',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertNotifications.heldAlertsBatchTimeInMins')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertNotifications.mailHost','unknown','bcg.alertNotifications.mailHost',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertNotifications.mailHost')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertNotifications.mailFrom','unknown@unknown.com','bcg.alertNotifications.mailFrom',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertNotifications.mailFrom')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertNotifications.mailReplyTo','unknown@unknown.com','bcg.alertNotifications.mailReplyTo',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertNotifications.mailReplyTo')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alertNotifications.mailEnvelopeFrom','unknown@unknown.com','bcg.alertNotifications.mailEnvelopeFrom',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alertNotifications.mailEnvelopeFrom')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.alert.eventGenerator.schedule','13 1 CertificateExpiration','bcg.alert.eventGenerator.schedule',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.alert.eventGenerator.schedule')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.gatewayDirectory','$SHARED_DATA_DIR$/gateways','bcg.delivery.gatewayDirectory',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.gatewayDirectory')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.smtpHost','$ROUTER.DM.SMTP_RELAY$','bcg.delivery.smtpHost',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.smtpHost')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.smtpHostPort','$ROUTER.DM.SMTP_RELAY.PORT$','bcg.delivery.smtpHostPort',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.smtpHostPort')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.responseDir','$SHARED_DATA_DIR$/sync_in','bcg.delivery.responseDir',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.responseDir')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.msMaxFileLockLife','180000','bcg.delivery.msMaxFileLockLife',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.msMaxFileLockLife')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.threadPoolMaxThreads','50','bcg.delivery.threadPoolMaxThreads',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.threadPoolMaxThreads')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.gatewayMaxThreads','20','bcg.delivery.gatewayMaxThreads',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.gatewayMaxThreads')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.gwTransportMaxRetries','3','bcg.delivery.gwTransportMaxRetries',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.gwTransportMaxRetries')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.gwTransportRetryInterval','3000','bcg.delivery.gwTransportRetryInterval',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.gwTransportRetryInterval')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.numberOfLoggers','10','bcg.delivery.numberOfLoggers',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.numberOfLoggers')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.jmstimeout','60000','bcg.delivery.jmstimeout',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.jmstimeout')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.http.socketTimeout','120000','bcg.http.socketTimeout',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.http.socketTimeout')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.http.version','1.1','bcg.http.version',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.http.version')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.retryWaitTmMS','5000','bcg.rosettanet.retryWaitTmMS',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.retryWaitTmMS')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.strictBoundaryParse','FALSE','bcg.rosettanet.strictBoundaryParse',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.strictBoundaryParse')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.mimeBoundaryValidate','FALSE','bcg.rosettanet.mimeBoundaryValidate',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.mimeBoundaryValidate')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.globalUsageCode','Literal','bcg.rosettanet.globalUsageCode',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.globalUsageCode')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.defaultUsageCdOnErr','1','bcg.rosettanet.defaultUsageCdOnErr',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.defaultUsageCdOnErr')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.defaultGlbUsageCd','Test','bcg.rosettanet.defaultGlbUsageCd',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.defaultGlbUsageCd')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.useBuilderProcessInstanceId','1','bcg.rosettanet.useBuilderProcessInstanceId',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.useBuilderProcessInstanceId')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.genProcessInstanceIdOnError','1','bcg.rosettanet.genProcessInstanceIdOnError',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.genProcessInstanceIdOnError')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rne.inbound_poll_interval','10000','bcg.rne.inbound_poll_interval',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rne.inbound_poll_interval')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rne.in_thread_count','2','bcg.rne.in_thread_count',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rne.in_thread_count')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rne.work_size','50','bcg.rne.work_size',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rne.work_size')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.0A1.fromContactName','$ROUTER.CONTACT_NAME$','bcg.0A1.fromContactName',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.0A1.fromContactName')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.0A1.fromEMailAddr','$ROUTER.CONTACT.MAIL_FROM$','bcg.0A1.fromEMailAddr',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.0A1.fromEMailAddr')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.0A1.fromPhoneNbr','$ROUTER.CONTACT.PHONE_NO$','bcg.0A1.fromPhoneNbr',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.0A1.fromPhoneNbr')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.0A1.fromFaxNbr','$ROUTER.CONTACT.FAX_NO$','bcg.0A1.fromFaxNbr',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.0A1.fromFaxNbr')
/

--101461
--INSERT INTO db_sysproperties(
--		SYSPROPID,
--		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
--   SELECT db_sysproperties_seq.NEXTVAL,'bcg.as.inbound_poll_interval','60000','bcg.as.inbound_poll_interval',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAsState' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.as.inbound_poll_interval')
--/
--INSERT INTO db_sysproperties(
--		SYSPROPID,
--		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
--   SELECT db_sysproperties_seq.NEXTVAL,'bcg.as.in_thread_count','2','bcg.as.in_thread_count',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAsState' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.as.in_thread_count')
--/
--INSERT INTO db_sysproperties(
--		SYSPROPID,
--		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
--   SELECT db_sysproperties_seq.NEXTVAL,'bcg.as.batch_size','1','bcg.as.batch_size',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAsState' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.as.batch_size')
--/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.CRLDir','$SHARED_DATA_DIR$/security/crl/','bcg.CRLDir',0, SYSPROPGRPID 
	FROM db_sysproperties_group
	WHERE groupname = 'common' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.CRLDir')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.checkRevocationStatus','TRUE','bcg.checkRevocationStatus',0, SYSPROPGRPID
	FROM db_sysproperties_group 
	WHERE groupname = 'common' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.checkRevocationStatus')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.http.SSLDebug','FALSE','bcg.http.SSLDebug',0, SYSPROPGRPID
	FROM db_sysproperties_group
	WHERE groupname = 'common' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.http.SSLDebug')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.signature.DigestAlgorithm','SHA1','bcg.rosettanet.signature.DigestAlgorithm',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrSecurity' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.signature.DigestAlgorithm')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.signature.RejectIfFailVal','TRUE','bcg.rosettanet.signature.RejectIfFailVal',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrSecurity' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.signature.RejectIfFailVal')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.signature.VerifySigner','TRUE','bcg.rosettanet.signature.VerifySigner',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrSecurity' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.signature.VerifySigner')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.encrypt.CertDbRefreshInterval','60000','bcg.rosettanet.encrypt.CertDbRefreshInterval',0, SYSPROPGRPID 
	FROM db_sysproperties_group 
	WHERE groupname = 'common' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.encrypt.CertDbRefreshInterval')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rosettanet.encrypt.Algorithm','3des','bcg.rosettanet.encrypt.Algorithm',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrSecurity' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rosettanet.encrypt.Algorithm')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.certs.vtp.CertificateDir','$SHARED_DATA_DIR$/security/vtp','bcg.certs.vtp.CertificateDir',0, SYSPROPGRPID
	FROM db_sysproperties_group 
	WHERE groupname = 'common' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.certs.vtp.CertificateDir')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'Wbipackaging_version','1.1','Wbipackaging_version',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrPackaging' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'Wbipackaging_version')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.sponsor.inbound_poll_interval','10000','bcg.sponsor.inbound_poll_interval',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrSS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.sponsor.inbound_poll_interval')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.sponsor.in_thread_count','1','bcg.sponsor.in_thread_count',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrSS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.sponsor.in_thread_count')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.sponsor.work_size','10','bcg.sponsor.work_size',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrSS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.sponsor.work_size')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.sponsor.eventMsgClass','com.ibm.bcg.delivery.sponsor.SponsorEventMessage','bcg.delivery.sponsor.eventMsgClass',3 FROM DUAL WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.sponsor.eventMsgClass')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'DBProcDebug','1','DBProcDebug',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrPackaging' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'DBProcDebug')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'GlobalStateEngInstanceId','Bcg','GlobalStateEngInstanceId',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrPackaging' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'GlobalStateEngInstanceId')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.ediint.reportingUA','WPG','bcg.ediint.reportingUA',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrPackaging' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ediint.reportingUA')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.ediint.retryWaitTmMS','5000','bcg.ediint.retryWaitTmMS',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrPackaging' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ediint.retryWaitTmMS')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.event_log_exclude','','bcg.event_log_exclude',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.event_log_exclude')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.maxBatchSize','1000','bcg.maxBatchSize',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrPackaging' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.maxBatchSize')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.archiver.maxSubVolFiles','70000','bcg.archiver.maxSubVolFiles',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrARC' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.archiver.maxSubVolFiles')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.archiver.runinterval','1500','bcg.archiver.runinterval',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrARC' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.archiver.runinterval')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.summaryEng.thread_count','1','bcg.summaryEng.thread_count',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrSE' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.summaryEng.thread_count')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.outbound.gatewayDirectory','$SHARED_DATA_DIR$/gateways','bcg.console.outbound.gatewayDirectory',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.outbound.gatewayDirectory')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.db.debugLevel','0','bcg.console.db.debugLevel',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.db.debugLevel')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.appserver.mgmt.pool.maxsize','20','bcg.console.appserver.mgmt.pool.maxsize',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.appserver.mgmt.pool.maxsize')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.EAIDocDir','Documents','bcg.console.EAIDocDir',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.EAIDocDir')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.specialChars','!#:;\\& /?.,','bcg.console.specialChars',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.specialChars')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.specialCharsDir','!#;& ?.,','bcg.console.specialCharsDir',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.specialCharsDir')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.help.host','$IES-HELP-HOST$','bcg.console.help.host',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.help.host')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.help.port','$IES-HELP-PORT$','bcg.console.help.port',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.help.port')
/
--Start 101712
--INSERT INTO db_sysproperties(
--		SYSPROPID,
--		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
--   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.locale.lookup.host','$CONSOLE_HOST$','bcg.console.locale.lookup.host',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.locale.lookup.host')
--/
--INSERT INTO db_sysproperties(
--		SYSPROPID,
--		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
--   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.locale.lookup.port','$CONSOLE_PORT$','bcg.console.locale.lookup.port',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.locale.lookup.port')
--/
--End 101712
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.queue.factory','jms/bcg/cf/CONCF','bcg.jms.queue.factory',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.queue.factory' and COMPONENT = 1)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.topic.factory','jms/bcg/cf/CONCF','bcg.jms.topic.factory',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.topic.factory' and COMPONENT = 1)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.jndi_factory','com.ibm.websphere.naming.WsnInitialContextFactory','bcg.jms.jndi_factory',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.jndi_factory' and COMPONENT = 1)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.context_url','corbaloc:iiop:<ip-address>:<jndi-port>','bcg.jms.context_url',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.context_url' and COMPONENT = 1)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.oaq_log_q','jms/bcg/queue/datalogQ','bcg.oaq_log_q',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.oaq_log_q' and COMPONENT = 3)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.topic.name','jms/bcg/topic/reloadCacheT','bcg.jms.topic.name',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.topic.name')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.certs.vtp.CertificateDir','$SHARED_DATA_DIR$/security/vtp','bcg.console.certs.vtp.CertificateDir',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleVTP' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.certs.vtp.CertificateDir')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.certs.vtp.Certificate','','bcg.console.certs.vtp.Certificate',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleVTP' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.certs.vtp.Certificate')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.certs.vtp.PrivateKey','','bcg.console.certs.vtp.PrivateKey',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleVTP' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.certs.vtp.PrivateKey')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.certs.vtp.Passwd','','bcg.console.certs.vtp.Passwd',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleVTP' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.certs.vtp.Passwd')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.certs.vtp.VerifySig','FALSE','bcg.console.certs.vtp.VerifySig',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleVTP' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.certs.vtp.VerifySig')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.vtp.RouterIn','$SHARED_DATA_DIR$/router_in','bcg.console.vtp.RouterIn',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleVTP' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.vtp.RouterIn')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.All','0','traceLevel.All',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.All')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.Transformation','0','traceLevel.Transformation',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.Transformation')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.Validation','0','traceLevel.Validation',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.Validation')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.Enveloper','0','traceLevel.Enveloper',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.Enveloper')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.Deenveloper','0','traceLevel.Deenveloper',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.Deenveloper')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.EDI-Parser','0','traceLevel.EDI-Parser',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.EDI-Parser')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.XML-Parser','0','traceLevel.XML-Parser',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.XML-Parser')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.ROD-Parser','0','traceLevel.ROD-Parser',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.ROD-Parser')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.EDI-Serializer','0','traceLevel.EDI-Serializer',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.EDI-Serializer')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.XML-Serializer','0','traceLevel.XML-Serializer',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.XML-Serializer')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.ROD-Serializer','0','traceLevel.ROD-Serializer',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.ROD-Serializer')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.EDI-Splitter','0','traceLevel.EDI-Splitter',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.EDI-Splitter')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.XML-Splitter','0','traceLevel.XML-Splitter',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.XML-Splitter')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.ROD-Splitter','0','traceLevel.ROD-Splitter',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.ROD-Splitter')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.ROD-Scanner','0','traceLevel.ROD-Scanner',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.ROD-Scanner')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.FTP-Scripting','0','traceLevel.FTP-Scripting',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.FTP-Scripting')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.IBMVanAckProcessor','0','traceLevel.IBMVanAckProcessor',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.IBMVanAckProcessor')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.EDIAckProcessor','0','traceLevel.EDIAckProcessor',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.EDIAckProcessor')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'traceLevel.Utility','0','traceLevel.Utility',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'traceLevel.Utility')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'transcript.file.option','N','transcript.file.option',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'transcript.file.option')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'database.encoding','UTF-16','database.encoding',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'database.encoding')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.persistpath','$SHARED_DATA_DIR$/router_in/','bcg.receiver.persistpath',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.persistpath')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.sync.persistpath','$SHARED_DATA_DIR$/sync_in','bcg.receiver.sync.persistpath',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.sync.persistpath')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.signal.persistpath','$SHARED_DATA_DIR$/signal_in','bcg.receiver.signal.persistpath',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.signal.persistpath')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.vms_inbound_directory.main','$SHARED_DATA_DIR$/router_in','bcg.vms_inbound_directory.main',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.vms_inbound_directory.main')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.bpe_temp_directory.main','$SHARED_DATA_DIR$/data','bcg.bpe_temp_directory.main',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.bpe_temp_directory.main')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.vms_inbound_directory.signal','$SHARED_DATA_DIR$/signal_in','bcg.vms_inbound_directory.signal',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.vms_inbound_directory.signal')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.bpe_temp_directory.signal','$SHARED_DATA_DIR$/data','bcg.bpe_temp_directory.signal',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.bpe_temp_directory.signal')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.vms_inbound_directory.synchronous','$SHARED_DATA_DIR$/sync_in','bcg.vms_inbound_directory.synchronous',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.vms_inbound_directory.synchronous')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.bpe_temp_directory.synchronous','$SHARED_DATA_DIR$/data','bcg.bpe_temp_directory.synchronous',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.bpe_temp_directory.synchronous')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.scheduler_initial_pool_size','10','bcg.scheduler_initial_pool_size',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.scheduler_initial_pool_size')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.scheduler_max_pool_size','50','bcg.scheduler_max_pool_size',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.scheduler_max_pool_size')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.OriginalFileName','x-aux-in-file-name','bcg.hdrdef.OriginalFileName',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.OriginalFileName')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.fromID','x-aux-sender-id','bcg.hdrdef.fromID',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.fromID')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.toID','x-aux-receiver-id','bcg.hdrdef.toID',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.toID')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.protocol','x-aux-protocol','bcg.hdrdef.protocol',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.protocol')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.protocolVersion','x-aux-protocol-version','bcg.hdrdef.protocolVersion',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.protocolVersion')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.process','x-aux-process-type','bcg.hdrdef.process',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.process')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.processVersion','x-aux-process-version','bcg.hdrdef.processVersion',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.processVersion')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.contentType','content-type','bcg.hdrdef.contentType',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.contentType')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.systemMsgId','x-aux-system-msg-id','bcg.hdrdef.systemMsgId',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.systemMsgId')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.RNResponseType','x-rn-response-type','bcg.hdrdef.RNResponseType',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.RNResponseType')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.RNVersion','x-rn-version','bcg.hdrdef.RNVersion',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.RNVersion')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.productionFlag','x-aux-production','bcg.hdrdef.productionFlag',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.productionFlag')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.provSessionId','x-aux-prov-session-id','bcg.hdrdef.provSessionId',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.provSessionId')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.processInstanceId','x-aux-process-instance-id','bcg.hdrdef.processInstanceId',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.processInstanceId')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.contentLength','Content-Length','bcg.hdrdef.contentLength',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.contentLength')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.as2From','AS2-From','bcg.hdrdef.as2From',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.as2From')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.as2To','AS2-To','bcg.hdrdef.as2To',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.as2To')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.as2Version','AS2-Version','bcg.hdrdef.as2Version',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.as2Version')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.mimeVersion','Mime-Version','bcg.hdrdef.mimeVersion',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.mimeVersion')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.messageId','Message-ID','bcg.hdrdef.messageId',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.messageId')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.date','Date','bcg.hdrdef.date',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.date')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.from','From','bcg.hdrdef.from',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.from')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.subject','Subject','bcg.hdrdef.subject',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.subject')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.contentTransferEncoding','Content-Transfer-Encoding','bcg.hdrdef.contentTransferEncoding',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.contentTransferEncoding')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.contentDisposition','Content-Disposition','bcg.hdrdef.contentDisposition',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.contentDisposition')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.dispositionNotificationTo','Disposition-Notification-To','bcg.hdrdef.dispositionNotificationTo',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.dispositionNotificationTo')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.dispositionNotificationOptions','Disposition-Notification-Options','bcg.hdrdef.dispositionNotificationOptions',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.dispositionNotificationOptions')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.receiptDeliveryOption','Receipt-Delivery-Option','bcg.hdrdef.receiptDeliveryOption',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.receiptDeliveryOption')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.toPackagingName','ToPackagingName','bcg.hdrdef.toPackagingName',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.toPackagingName')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.asDocType','ASDocType','bcg.hdrdef.asDocType',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.asDocType')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.recipientAddress','Recipient-Address','bcg.hdrdef.recipientAddress',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.recipientAddress')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.authorization','Authorization','bcg.hdrdef.authorization',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.authorization')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.soapAction','SOAPAction','bcg.hdrdef.soapAction',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.soapAction')
/
-- end 49576
--start 82799
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.global.common.introduce.document.transport','JMS','bcg.global.common.introduce.document.transport',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.global.common.introduce.document.transport')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.global.common.introduce.document.transport.unavailable.timeout','60000','bcg.global.common.introduce.document.transport.unavailable.timeout',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.global.common.introduce.document.transport.unavailable.timeout')
/
--end 82799
--start 60181
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.ebXML.language','en-US','bcg.ebXML.language',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrEBMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ebXML.language')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.AddKeyInfo','true','bcg.AddKeyInfo',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrEBMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.AddKeyInfo')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.ebXML.version','2.0','bcg.ebXML.version',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrEBMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ebXML.version')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.ebms.xsd.schemaName','ebXMLSchema.xsd','bcg.ebms.xsd.schemaName',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrEBMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ebms.xsd.schemaName')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rm.pollInterval','300000','bcg.rm.pollInterval',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRM' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rm.pollInterval')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rm.thread_count','3','bcg.rm.thread_count',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRM' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rm.thread_count')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.TimeToLive','x-aux-TimeToLive','bcg.hdrdef.TimeToLive',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.TimeToLive')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.GlobalBusinessActionCode','x-aux-action','bcg.hdrdef.GlobalBusinessActionCode',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.GlobalBusinessActionCode')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.ActivityName','x-aux-activity','bcg.hdrdef.ActivityName',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.ActivityName')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.SOAPAction','SOAPAction','bcg.hdrdef.SOAPAction',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.SOAPAction')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.seqnumber','x-aux-seq-number','bcg.hdrdef.seqnumber',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.seqnumber')
/
-- end 60181
--start 101250
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.version','','bcg.console.version',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.version')
/
--end 101250
--start 101359
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.console.appserver.mgmt.ctx.instancepolicy','singleton','bcg.console.appserver.mgmt.ctx.instancepolicy',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.appserver.mgmt.ctx.instancepolicy')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.console.jmsPosterInstance','com.ibm.bcg.shared.event.MQSeriesPoster','bcg.console.jmsPosterInstance',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.console.jmsPosterInstance')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.reloadCache.name','','bcg.jms.reloadCache.name',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.jms.reloadCache.name')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.oaq_log_q','jms/bcg/queue/datalogQ','bcg.oaq_log_q',1, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'consoleJMS' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.oaq_log_q' and COMPONENT = 1)
/
-- end 101359
--start 100566
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.senderLocationID','x-aux-rnif-sender-location-id','bcg.hdrdef.senderLocationID',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.senderLocationID')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.receiverLocationID','x-aux-rnif-receiver-location-id','bcg.hdrdef.receiverLocationID',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.receiverLocationID')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.build_complete_certpath','true','bcg.build_complete_certpath',0, SYSPROPGRPID
	FROM db_sysproperties_group
	WHERE groupname = 'common' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.build_complete_certpath')
/
-- end 100566
--101376/101461
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.asstate.thread_count','1','bcg.asstate.thread_count',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrAsState' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.asstate.thread_count'
               and COMPONENT = 3)
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.asstate.batchSize','1','bcg.asstate.batchSize',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrAsState' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.asstate.batchSize'
               and COMPONENT = 3)
/
   
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.asstate.runinterval','1000','bcg.asstate.runinterval',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrAsState' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.asstate.runinterval'
               and COMPONENT = 3)
/
--end 101376
--101896
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.ebms.validate','false','bcg.ebms.validate',3, SYSPROPGRPID i
	FROM db_sysproperties_group 
	WHERE groupname = 'docmgrEBMS' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ebms.validate')
/
--end 101896
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.ipv6','','bcg.receiver.ipv6',2, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'receiverOther' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.ipv6' and COMPONENT = 2)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.router.ipv6.address','','bcg.router.ipv6.address',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.router.ipv6.address' and COMPONENT = 3)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.rnif.pip.twoaction.correlation','documentid','bcg.rnif.pip.twoaction.correlation',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrRN' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.rnif.pip.twoaction.correlation')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.loggerTimeOut','10000','bcg.delivery.loggerTimeOut',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrDelivery' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.delivery.loggerTimeOut')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.destinationCache.expiry','3600','bcg.destinationCache.expiry',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBPEMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.destinationCache.expiry')
/

-- Start 102094
UPDATE db_sysproperties SET SYSPROPGRPID = NULL WHERE PROPNAME = 'bcg.console.appserver.mgmt.ctx.instancepolicy'
/
-- End   102094
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.MessageId','x-aux-msg-id','bcg.hdrdef.MessageId',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'headerDef' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.hdrdef.MessageId'
               and COMPONENT = 0)
/

-- Start 55055
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.eventeng.alertscache.size','100','bcg.eventeng.alertscache.size',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'eventGeneral' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.eventeng.alertscache.size')
/
-- End   55055
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.VolumeAlertScheduler.scheduleTime','10','bcg.VolumeAlertScheduler.scheduleTime',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.VolumeAlertScheduler.scheduleTime')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.BatchAlertScheduler.scheduleTime','10','bcg.BatchAlertScheduler.scheduleTime',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.BatchAlertScheduler.scheduleTime')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.NotificationAlertScheduler.scheduleTime','10','bcg.NotificationAlertScheduler.scheduleTime',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrAlert' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.NotificationAlertScheduler.scheduleTime')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.install.commonloc', '$SHARED_DATA_DIR$', 'bcg.install.commonloc' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.install.commonloc')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.freshInstallation', 'true', 'bcg.freshInstallation',3 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.freshInstallation')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
	SELECT db_sysproperties_seq.NEXTVAL,'bcg.duns', NULL, 'bcg.duns', 1, SYSPROPGRPID 
	FROM db_sysproperties_group 
	WHERE groupname = 'consoleGeneral' 
	AND NOT EXISTS (SELECT 'x' 
			FROM db_sysproperties 
			WHERE PROPNAME = 'bcg.duns')
/
-- INSERT INTO db_sysproperties(
--		SYSPROPID,
--		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
--	SELECT db_sysproperties_seq.NEXTVAL,'bcg.useCRLDP' ,'true', 'bcg.useCRLDP', 0, SYSPROPGRPID 
--	FROM db_sysproperties_group 
--	WHERE groupname = 'common' 
--	AND NOT EXISTS (SELECT 'x' 
--			FROM db_sysproperties 
--			WHERE PROPNAME = 'bcg.useCRLDP')
--/

-- Start 103417
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.ebms.MaxFileSize','20','bcg.ebms.MaxFileSize',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrEBMS' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.ebms.MaxFileSize'
               and COMPONENT = 3)
/
-- End   103417

-- Start 103352
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.role','x-aux-role','bcg.hdrdef.role',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'headerDef' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.hdrdef.role'
               and COMPONENT = 0)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.roleSchema','x-aux-role-schema','bcg.hdrdef.roleSchema',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'headerDef' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.hdrdef.roleSchema'
               and COMPONENT = 0)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.roleSchemaVer','x-aux-role-schema-version','bcg.hdrdef.roleSchemaVer',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'headerDef' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.hdrdef.roleSchemaVer'
               and COMPONENT = 0)
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.description','x-aux-description','bcg.hdrdef.description',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'headerDef' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.hdrdef.description'
               and COMPONENT = 0)
/
-- End 103352

-- Start 103618
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.delivery.allowDMLoadBalance','Yes','bcg.delivery.allowDMLoadBalance',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrDelivery' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.delivery.allowDMLoadBalance'
               and COMPONENT = 3)
/
-- End   103618

-- Start 103635
Update db_sysproperties SET SYSPROPGRPID = NULL WHERE PROPNAME = 'bcg.receiver.sync.responseURL' and COMPONENT = 2
/
-- End   103635
-- Start 103823
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.default.encoding','UTF-8','bcg.receiver.default.encoding',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'receiverOther' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.receiver.default.encoding'
               and COMPONENT = 0)
/
-- End   103823 
-- Start 104139
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.global.common.deletetempfiles','Yes','bcg.global.common.deletetempfiles',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'common' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.global.common.deletetempfiles'
               and COMPONENT = 0)
/
-- End 104139

-- Start 104192
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'ftp.data.connection.timeout','18000000','ftp.data.connection.timeout',4, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'edi' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'ftp.data.connection.timeout'
               and COMPONENT = 4)
/
-- End   104192

-- Start JR26399

INSERT INTO db_sysproperties(
		syspropid,
		PROPNAME,RESOURCEKEY) 
SELECT db_sysproperties_seq.NEXTVAL,'event.delivery.jms.password','event.delivery.jms.password' FROM DUAL
WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.password')
/


INSERT INTO db_sysproperties(
		syspropid,
		PROPNAME,RESOURCEKEY) 
SELECT db_sysproperties_seq.NEXTVAL,'event.delivery.jms.userName','event.delivery.jms.userName' FROM DUAL
WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'event.delivery.jms.userName')
/

-- End JR26399

-- JR25898
UPDATE db_sysproperties SET PROPVALUE = 'WBIC\\DeliveryQ' WHERE PROPNAME = 'event.delivery.jms.queueName'
and PROPVALUE = 'WBIC/DeliveryQ'
/
UPDATE db_sysproperties SET PROPVALUE = 'WBIC\\QCF' WHERE PROPNAME = 'event.delivery.jms.queueFactoryName'
and PROPVALUE = 'WBIC/QCF'
/
-- JR25898

-- Feature F104473
INSERT INTO db_sysproperties
            (syspropid, propname, propvalue, resourcekey, component
             )
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.sync.cleanUpPollInterval',
          '300000', 'bcg.sync.cleanUpPollInterval', 2
     FROM dual
      where NOT EXISTS (
             SELECT 'x'
               FROM db_sysproperties
              WHERE propname = 'bcg.sync.cleanUpPollInterval'
                    AND component = 2)
/ 

-- Feature F56098
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.securityProvider', 'IBMPKCS', 'bcg.securityProvider' FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.securityProvider')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.maximumFileSizeForByteArrays','15','bcg.maximumFileSizeForByteArrays',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.maximumFileSizeForByteArrays')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.noasretry.iffailed','false','bcg.noasretry.iffailed',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrAsState' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.noasretry.iffailed'
               and COMPONENT = 3)
/                    

--Feature 104485

--Default listener


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.port','$defaultPort$','bcg.ftp.config.listeners.default.port',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'DEFAULT LISTENER' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.port')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.implicit-ssl', 'false', 'bcg.ftp.config.listeners.default.implicit-ssl', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.implicit-ssl')
/   


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.class', 'org.apache.ftpserver.listener.mina.MinaListener', 'bcg.ftp.config.listeners.default.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.class')
/   

   
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.address','$hostIp$','bcg.ftp.config.listeners.default.address',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'DEFAULT LISTENER' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.address')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.class', 'com.ibm.bcg.ftpserver.ssl.BCGSslImpl', 'bcg.ftp.config.listeners.default.ssl.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.class')
/   

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.keystore-file', './conf/ftpKeyStore.jks', 'bcg.ftp.config.listeners.default.ssl.keystore-file', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.keystore-file')
/   

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.keystore-password', 'Pa55word', 'bcg.ftp.config.listeners.default.ssl.keystore-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.keystore-password')
/   

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.keystore-type', 'JKS', 'bcg.ftp.config.listeners.default.ssl.keystore-type', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.keystore-type')
/   

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.keystore-algorithm', 'IbmX509', 'bcg.ftp.config.listeners.default.ssl.keystore-algorithm', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.keystore-algorithm')
/   
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.ssl-protocol', 'TLSv1', 'bcg.ftp.config.listeners.default.ssl.ssl-protocol', 5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'DEFAULT LISTENER' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.ssl-protocol')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.client-authentication', 'none', 'bcg.ftp.config.listeners.default.ssl.client-authentication', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.client-authentication')
/ 

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.key-password', 'Pa55word', 'bcg.ftp.config.listeners.default.ssl.key-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.key-password')
/   

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.truststore-file', './conf/ftpTrustStore.jks', 'bcg.ftp.config.listeners.default.ssl.truststore-file', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.truststore-file')
/   

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.truststore-password', 'Pa55word', 'bcg.ftp.config.listeners.default.ssl.truststore-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.truststore-password')
/   

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.truststore-type', 'JKS', 'bcg.ftp.config.listeners.default.ssl.truststore-type', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.truststore-type')
/   

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.ssl.truststore-algorithm', 'IbmX509', 'bcg.ftp.config.listeners.default.ssl.truststore-algorithm', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.ssl.truststore-algorithm')
/   


--Default listener data connection



INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.passive.address', '$hostIp$', 'bcg.ftp.config.listeners.default.data-connection.passive.address', 5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'DEFAULT LISTENER' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.passive.address')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.passive.ports','0','bcg.ftp.config.listeners.default.data-connection.passive.ports',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'DEFAULT LISTENER' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.passive.ports')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.class', 'org.apache.ftpserver.DefaultDataConnectionConfig', 'bcg.ftp.config.listeners.default.data-connection.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.class')
/ 

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.idle-time','10','bcg.ftp.config.listeners.default.data-connection.idle-time',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'DEFAULT LISTENER' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.idle-time')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.active.enable', 'true', 'bcg.ftp.config.listeners.default.data-connection.active.enable', 5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'DEFAULT LISTENER' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.active.enable')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.active.local-address', '$hostIp$', 'bcg.ftp.config.listeners.default.data-connection.active.local-address', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.active.local-address')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.active.local-port', '0', 'bcg.ftp.config.listeners.default.data-connection.active.local-port', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.active.local-port')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.passive.external-address', '$hostIp$', 'bcg.ftp.config.listeners.default.data-connection.passive.external-address', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.passive.external-address')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.active.ip-check', 'false', 'bcg.ftp.config.listeners.default.data-connection.active.ip-check', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.active.ip-check')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.class', 'com.ibm.bcg.ftpserver.ssl.BCGSslImpl', 'bcg.ftp.config.listeners.default.data-connection.ssl.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.class')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-file', './conf/ftpKeyStore.jks', 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-file', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-file')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-password', 'Pa55word', 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-password')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-type', 'JKS', 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-type', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-type')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-algorithm', 'IbmX509', 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-algorithm', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.keystore-algorithm')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-algorithm', 'IbmX509', 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-algorithm', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-algorithm')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.ssl-protocol', 'TLSv1', 'bcg.ftp.config.listeners.default.data-connection.ssl.ssl-protocol', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.ssl-protocol')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.client-authentication', 'none', 'bcg.ftp.config.listeners.default.data-connection.ssl.client-authentication', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.client-authentication')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.key-password', 'Pa55word', 'bcg.ftp.config.listeners.default.data-connection.ssl.key-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.key-password')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-file', './conf/ftpKeyStore.jks', 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-file', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-file')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-password', 'Pa55word', 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-password')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-type', 'JKS', 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-type', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.default.data-connection.ssl.truststore-type')
/


--Client auth listener

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.port', '$clientAuthPort$', 'bcg.ftp.config.listeners.clientauth.port', 5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'CLIENTAUTH LISTENER' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.port')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.implicit-ssl', 'false', 'bcg.ftp.config.listeners.clientauth.implicit-ssl', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.implicit-ssl')
/ 

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.class', 'org.apache.ftpserver.listener.mina.MinaListener', 'bcg.ftp.config.listeners.clientauth.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.class')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.address', '$hostIp$', 'bcg.ftp.config.listeners.clientauth.address', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.address')
/ 


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.class', 'com.ibm.bcg.ftpserver.ssl.BCGSslImpl', 'bcg.ftp.config.listeners.clientauth.ssl.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.class')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.keystore-file', './conf/ftpKeyStore.jks', 'bcg.ftp.config.listeners.clientauth.ssl.keystore-file', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.keystore-file')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.keystore-password', 'Pa55word', 'bcg.ftp.config.listeners.clientauth.ssl.keystore-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.keystore-password')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.keystore-type', 'JKS', 'bcg.ftp.config.listeners.clientauth.ssl.keystore-type', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.keystore-type')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.keystore-algorithm', 'IbmX509', 'bcg.ftp.config.listeners.clientauth.ssl.keystore-algorithm', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.keystore-algorithm')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.ssl-protocol', 'TLSv1', 'bcg.ftp.config.listeners.clientauth.ssl.ssl-protocol', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.ssl-protocol')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.client-authentication', 'YES', 'bcg.ftp.config.listeners.clientauth.ssl.client-authentication', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.client-authentication')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.key-password', 'Pa55word', 'bcg.ftp.config.listeners.clientauth.ssl.key-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.key-password')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.truststore-file', './conf/ftpTrustStore.jks', 'bcg.ftp.config.listeners.clientauth.ssl.truststore-file', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.truststore-file')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.truststore-password', 'Pa55word', 'bcg.ftp.config.listeners.clientauth.ssl.truststore-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.truststore-password')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.truststore-type', 'JKS', 'bcg.ftp.config.listeners.clientauth.ssl.truststore-type', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.truststore-type')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.ssl.truststore-algorithm', 'IbmX509', 'bcg.ftp.config.listeners.clientauth.ssl.truststore-algorithm', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.ssl.truststore-algorithm')
/


--Client auth listener data connection


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.idle-time', '10', 'bcg.ftp.config.listeners.clientauth.data-connection.idle-time', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.idle-time')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.passive.address', '$hostIp$', 'bcg.ftp.config.listeners.clientauth.data-connection.passive.address', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.passive.address')
/ 

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.active.enable', 'true', 'bcg.ftp.config.listeners.clientauth.data-connection.active.enable', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.active.enable')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.active.local-address', '$hostIp$', 'bcg.ftp.config.listeners.clientauth.data-connection.active.local-address', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.active.local-address')
/ 


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.class', 'org.apache.ftpserver.DefaultDataConnectionConfig', 'bcg.ftp.config.listeners.clientauth.data-connection.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.class')
/ 

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.active.local-port', '0', 'bcg.ftp.config.listeners.clientauth.data-connection.active.local-port', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.active.local-port')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.active.ip-check', 'false', 'bcg.ftp.config.listeners.clientauth.data-connection.active.ip-check', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.active.ip-check')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.class', 'com.ibm.bcg.ftpserver.ssl.BCGSslImpl', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.class')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-file', './conf/ftpKeyStore.jks', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-file', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-file')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-password', 'Pa55word', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-password')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-type', 'JKS', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-type', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-type')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-algorithm', 'IbmX509', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-algorithm', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.keystore-algorithm')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-algorithm', 'IbmX509', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-algorithm', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-algorithm')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.ssl-protocol', 'TLSv1', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.ssl-protocol', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.ssl-protocol')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.client-authentication', 'need', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.client-authentication', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.client-authentication')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.key-password', 'Pa55word', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.key-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.key-password')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.passive.ports', '0', 'bcg.ftp.config.listeners.clientauth.data-connection.passive.ports', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.passive.ports')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-type', 'JKS', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-type', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-type')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-password', 'Pa55word', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-password', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-password')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-file', './conf/ftpKeyStore.jks', 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-file', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.listeners.clientauth.data-connection.ssl.truststore-file')
/

--Connection manager

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.connection-manager.max-connection','100','bcg.ftp.config.connection-manager.max-connection',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'CONNECTION PROPERTIES' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.connection-manager.max-connection')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.connection-manager.max-login','100','bcg.ftp.config.connection-manager.max-login',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'CONNECTION PROPERTIES' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.connection-manager.max-login')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.connection-manager.default-idle-time','60','bcg.ftp.config.connection-manager.default-idle-time',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'CONNECTION PROPERTIES' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.connection-manager.default-idle-time')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.connection-manager.timeout-poll-interval', '60', 'bcg.ftp.config.connection-manager.timeout-poll-interval', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.connection-manager.timeout-poll-interval')
/ 

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.connection-manager.anonymous-login-enabled', 'false', 'bcg.ftp.config.connection-manager.anonymous-login-enabled', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.connection-manager.anonymous-login-enabled')
/ 

--Ftplet



INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.ftplets', 'bcgFtpLet', 'bcg.ftp.config.ftplets', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.ftplets')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.ftplet.bcgFtpLet.class', 'com.ibm.bcg.ftplet.BCGFtplet', 'bcg.ftp.config.ftplet.bcgFtpLet.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.ftplet.bcgFtpLet.class')
/

--Other




INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.rootdirectory','$installLocation$','bcg.ftp.config.rootdirectory',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'FTP OTHER PROPERTIES' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.rootdirectory')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.ip-restrictor.class', 'com.ibm.bcg.ftpserver.mgmt.DBIPRestrictor', 'bcg.ftp.config.ip-restrictor.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.ip-restrictor.class')
/



INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.config.ftpserver.FTPInstallLocation', '$installLocation$', 'bcg.config.ftpserver.FTPInstallLocation', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.config.ftpserver.FTPInstallLocation')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.config.ftpserver.FTPInstallHostIp', '$hostIp$', 'bcg.config.ftpserver.FTPInstallHostIp', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.config.ftpserver.FTPInstallHostIp')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.create-default-user', 'false', 'bcg.ftp.config.create-default-user', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.create-default-user')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.ftp.config.user-manager.class', 'com.ibm.bcg.ftpserver.usermanager.FTPServerUserManager', 'bcg.ftp.config.user-manager.class', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.ftp.config.user-manager.class')
/





--Event


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.config.ftpserver.FTPSerializeFileInterval', '10000', 'bcg.config.ftpserver.FTPSerializeFileInterval',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'FTP EVENT PROPERTIES' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.config.ftpserver.FTPSerializeFileInterval')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.config.ftpserver.eventPersistThreads', '5', 'bcg.config.ftpserver.eventPersistThreads',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'FTP EVENT PROPERTIES' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.config.ftpserver.eventPersistThreads')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.config.ftpserver.FTPEventThreshold','500','bcg.config.ftpserver.FTPEventThreshold',5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'FTP EVENT PROPERTIES' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.config.ftpserver.FTPEventThreshold')
/



INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.config.ftpserver.FTPEventStoreInterval', '15000', 'bcg.config.ftpserver.FTPEventStoreInterval', 5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'FTP EVENT PROPERTIES' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.config.ftpserver.FTPEventStoreInterval')
/


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.config.ftpserver.FTPEventLoggingLevel', '2', 'bcg.config.ftpserver.FTPEventLoggingLevel', 5, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'FTP EVENT PROPERTIES' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.config.ftpserver.FTPEventLoggingLevel')
/

--END F104485

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.FIPSMode','FALSE','bcg.FIPSMode',0, SYSPROPGRPID 
   FROM db_sysproperties_group WHERE groupname = 'docmgrSecurity' 
   AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.FIPSMode')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.archive.maxThreads','4','bcg.archive.maxThreads',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrARC' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.archive.maxThreads'
               )
/

-- Start 105273

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.config.ftpmanagement.allowedIPs', '*', 'bcg.config.ftpmanagement.allowedIPs', 5 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.config.ftpmanagement.allowedIPs')
/

-- End 105273


INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY,COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.soap.SOAP1_1Namespace', 'http://schemas.xmlsoap.org/soap/envelope/',
   'bcg.soap.SOAP1_1Namespace',3 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.soap.SOAP1_1Namespace')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY,COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.soap.SOAP1_1Grammar', 'soapEnvelope.xsd',
   'bcg.soap.SOAP1_1Grammar',3 FROM DUAL
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.soap.SOAP1_1Grammar')
/

-- Start 105609

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.jms.retry_connect_interval','30000','bcg.jms.retry_connect_interval',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'common' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.jms.retry_connect_interval' and COMPONENT = 0
               )
/

-- End 105609

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.jms.retry_connect_interval','300000','bcg.jms.retry_connect_interval',1, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'consoleJMS' AND 
   NOT EXISTS (SELECT 'x' 
   	       FROM db_sysproperties 
   	       WHERE PROPNAME = 'bcg.jms.retry_connect_interval' and COMPONENT = 1
   	       )
/

-- Added for CR102199

INSERT INTO db_sysproperties(SYSPROPID, PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.messagestore.threshold','100000','bcg.messagestore.threshold',3, SYSPROPGRPID 
   FROM db_sysproperties_group WHERE groupname = 'common' 
   AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE 
   		   PROPNAME = 'bcg.messagestore.threshold')
/


-- Added for CR 105462

INSERT INTO db_sysproperties
            (syspropid, propname, propvalue, resourcekey, component,
             syspropgrpid)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.allow.partner.type.edit',
          'false', 'bcg.allow.partner.type.edit', 0, syspropgrpid
     FROM db_sysproperties_group
    WHERE groupname = 'common'
      AND NOT EXISTS (SELECT 'x'
                        FROM db_sysproperties
                       WHERE propname = 'bcg.allow.partner.type.edit')
/

-- Added for CR 105648f_1

INSERT INTO db_sysproperties
            (syspropid, propname, propvalue, resourcekey, component,
             syspropgrpid)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.rosettanet.encrypt.Algorithm.keysize',
          '168', 'bcg.rosettanet.encrypt.Algorithm.keysize', 0, syspropgrpid
     FROM db_sysproperties_group
    WHERE groupname = 'docmgrSecurity'
      AND NOT EXISTS (SELECT 'x'
                        FROM db_sysproperties
                       WHERE propname = 'bcg.rosettanet.encrypt.Algorithm.keysize')
/

INSERT INTO db_sysproperties
            (syspropid, propname, propvalue, resourcekey, component,
             syspropgrpid)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.jms.idle_connect_timeout',
          '1800000', 'bcg.jms.idle_connect_timeout', 0, syspropgrpid
     FROM db_sysproperties_group
    WHERE groupname = 'common'
      AND NOT EXISTS (
             SELECT 'x'
               FROM db_sysproperties
              WHERE propname = 'bcg.jms.idle_connect_timeout'
                    AND component = 0)
/   
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.archive.db.rowsPerStatement','1000','bcg.archive.db.rowsPerStatement',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrARC' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.archive.db.rowsPerStatement'
               )
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.archive.db.rowsPerBatch','100000','bcg.archive.db.rowsPerBatch',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrARC' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.archive.db.rowsPerBatch'
               )
/

-- 105964

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'PageThreshold','1000','PageThreshold',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'PageThreshold')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'PageFileDir','','PageFileDir',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'PageFileDir')
/

-- END 105964
-- 106307
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.archive.db.paralellism','4','bcg.archive.db.paralellism',0, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'docmgrARC' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'bcg.archive.db.paralellism'
              ) 
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'wtx.rmihostname', 'localhost', 'wtx.rmihostname',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'wtx.rmihostname')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'wtx.rmiport', '2500', 'wtx.rmiport',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'wtx.rmiport')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'rmiuseserver', 'no', 'rmiuseserver',3, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'common' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'rmiuseserver')
/

-- 106554

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'timezone','GMT','timezone',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'timezone')
/

-- END 106554

--106815
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY,COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.wtx.mapLocation', '$SHARED_DATA_DIR$/maps', 'bcg.wtx.mapLocation',0, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'common' and NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.wtx.mapLocation')
/
-- APAR JR29895
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.fail.expired_signals','false','bcg.fail.expired_signals',3, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'docmgrBpeMain' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.fail.expired_signals')
/
-- APAR JR29895

INSERT INTO db_sysproperties(
		SYSPROPID,PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hdrdef.docSyntax','x-aux-docSyntax','bcg.hdrdef.docSyntax',0, 
SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'headerDef'
 AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hdrdef.docSyntax')
/

-- Added for 107224
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'ftp.encoding.option','UTF-8','ftp.encoding.option',4, SYSPROPGRPID 
   FROM db_sysproperties_group 
   WHERE groupname = 'edi' AND 
   NOT EXISTS (SELECT 'x' 
               FROM db_sysproperties 
               WHERE PROPNAME = 'ftp.encoding.option'
               and COMPONENT = 4)
/ 
INSERT INTO DB_SYSPROPERTIES(
        syspropid,
        rowts
        , component
        , syspropgrpid
        , propname
        , propvalue
        , resourcekey
        )
        SELECT db_sysproperties_seq.NEXTVAL,1
               ,3
               ,syspropgrpid
               ,'bcg.archive.db.eventRatio'
               ,'2'
               ,'bcg.archive.db.eventRatio'
        FROM db_sysproperties_group
	WHERE	groupname = 'docmgrARC'
        AND NOT EXISTS (SELECT syspropid
                        FROM DB_SYSPROPERTIES
                        where propname = 'bcg.archive.db.eventRatio'
                        )

/

INSERT INTO DB_SYSPROPERTIES(
        syspropid,
        rowts
        , component
        , syspropgrpid
        , propname
        , propvalue
        , resourcekey
        )
        SELECT db_sysproperties_seq.NEXTVAL,1
               ,3
               ,syspropgrpid
               ,'bcg.archive.activityParallelism'
               ,'4'
               ,'bcg.archive.activityParallelism'
        FROM db_sysproperties_group
	WHERE	groupname = 'docmgrARC'
        AND NOT EXISTS (SELECT syspropid
                        FROM DB_SYSPROPERTIES
                        where propname = 'bcg.archive.activityParallelism'
                        )
/
UPDATE db_sysproperties
SET syspropgrpid = (SELECT syspropgrpid FROM db_sysproperties_group WHERE groupname = 'docmgrARC')
WHERE PROPNAME = 'bcg.archive.maxThreads'
/
UPDATE db_sysproperties
SET syspropgrpid = (SELECT syspropgrpid FROM db_sysproperties_group WHERE groupname = 'docmgrARC')
WHERE PROPNAME = 'bcg.archive.db.rowsPerStatement'
/
UPDATE db_sysproperties
SET syspropgrpid = (SELECT syspropgrpid FROM db_sysproperties_group WHERE groupname = 'docmgrARC')
WHERE PROPNAME = 'bcg.archive.db.rowsPerBatch'
/
UPDATE db_sysproperties
SET syspropgrpid = (SELECT syspropgrpid FROM db_sysproperties_group WHERE groupname = 'docmgrARC')
WHERE PROPNAME = 'bcg.archive.db.paralellism'
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.application.name','','bcg.receiver.application.name',1, SYSPROPGRPID 
	FROM db_sysproperties_group
	WHERE groupname = 'externalevent' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.application.name')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.documentmanager.application.name','','bcg.documentmanager.application.name',1, SYSPROPGRPID 
	FROM db_sysproperties_group
	WHERE groupname = 'externalevent' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.documentmanager.application.name')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.hub.mode','','bcg.hub.mode',1, SYSPROPGRPID 
	FROM db_sysproperties_group
	WHERE groupname = 'externalevent' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.hub.mode')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.common.sftp','$SHARED_DATA_DIR$/sftp','bcg.receiver.common.sftp',1, SYSPROPGRPID 
	FROM db_sysproperties_group
	WHERE groupname = 'receiverDir' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.common.sftp')
/

INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.receiver.common.receiver','$SHARED_DATA_DIR$/receiver','bcg.receiver.common.receiver',1, SYSPROPGRPID 
	FROM db_sysproperties_group
	WHERE groupname = 'receiverDir' 
	AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.receiver.common.receiver')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.edifastate.thread_count','0','bcg.edifastate.thread_count',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.edifastate.thread_count')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'bcg.edifastate.runinterval','60000','bcg.edifastate.runinterval',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.edifastate.runinterval')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY, COMPONENT, SYSPROPGRPID)
   SELECT db_sysproperties_seq.NEXTVAL,'EDIFaStateEngInstanceId','EDIFaStateEngInstanceId','EDIFaStateEngInstanceId',4, SYSPROPGRPID FROM db_sysproperties_group WHERE groupname = 'edi' AND NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'EDIFaStateEngInstanceId')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY,COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'bcg.archiver.dataRestorePath','$SHARED_DATA_DIR$/dataRestore', 'bcg.archiver.dataRestorePath',3 FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME = 'bcg.archiver.dataRestorePath')
/
INSERT INTO db_sysproperties(
		SYSPROPID,
		PROPNAME, PROPVALUE, RESOURCEKEY,COMPONENT)
   SELECT db_sysproperties_seq.NEXTVAL, 'excludedContentTypesForCanonicalization','application/xml:text/xml', 'excludedContentTypesForCanonicalization',0 FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM db_sysproperties WHERE PROPNAME ='excludedContentTypesForCanonicalization')
/

--JR30558
UPDATE db_sysproperties SET PROPVALUE='1000000' WHERE  PROPNAME = 'bcg.messagestore.threshold'
/
-- 108120

update db_sysproperties set syspropgrpid = (select syspropgrpid from db_sysproperties_group where groupname = 'common')
where propname like '%excludedContentTypesForCanonicalization%'
/

UPDATE db_sysproperties SET PROPVALUE='application/xml:text/xml:application/octet-stream' 
WHERE  PROPNAME = 'excludedContentTypesForCanonicalization'
/

--JR32362
update db_sysproperties set COMPONENT='0', 
SYSPROPGRPID=(select syspropgrpid from db_sysproperties_group where groupname = 'common') where PROPNAME='bcg.duns'
/

-- PR_HANDLER.sql

-- PR_HANDLER

--1
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.cxml.cXMLChannelParseHandler',
     'user-exit.handler.cxml.cXMLChannelParseHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE 
   CLASSNAME = 'com.ibm.bcg.cxml.cXMLChannelParseHandler')
/

--2
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.cxml.cXMLPackagingHandler',
     'user-exit.handler.cxml.cXMLPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.cxml.cXMLPackagingHandler')
/

--3
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory',
     'user-exit.handler.duplicate.ContentDuplicateProcessFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory')
/

--4
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.eai.EAIPackagingHandler',
     'user-exit.handler.eai.EAIPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.eai.EAIPackagingHandler')
/

--5
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.eai.EAIUnPackagingHandler',
     'user-exit.handler.eai.EAIUnPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.eai.EAIUnPackagingHandler')
/

--6
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.EDIRouterBizProcessHandler',
     'user-exit.handler.edi.EDIRouterBizProcessHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.EDIRouterBizProcessHandler')
/

--7
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.ediint.ASUnPackagingHandler',
     'user-exit.handler.ediint.ASUnPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.ediint.ASUnPackagingHandler')
/

--8
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.outbound.OutboundDocFactory',
     'user-exit.handler.outbound.OutboundDocFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory')
/

--9
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.passthrough.No_op',
     'user-exit.handler.passthrough.No_op'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.passthrough.No_op')
/

--10
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.rosettanet.passthru.ProcessLoggingFactory',
     'user-exit.handler.rosettanet.passthru.ProcessLoggingFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.rosettanet.passthru.ProcessLoggingFactory')
/

--11
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.rosettanet.pkg.RNOPackagingHandler',
     'user-exit.handler.rosettanet.pkg.RNOPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.rosettanet.pkg.RNOPackagingHandler')
/

--12
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.BinaryChannelParseHandler',
     'user-exit.handler.server.BinaryChannelParseHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.BinaryChannelParseHandler')
/

--13
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.RNOChannelParseHandler',
     'user-exit.handler.server.RNOChannelParseHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.RNOChannelParseHandler')
/

--14
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.RNSCChannelParseHandler',
     'user-exit.handler.server.RNSCChannelParseHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.RNSCChannelParseHandler')
/

--15
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.RNSignalChannelParseHandler',
     'user-exit.handler.server.RNSignalChannelParseHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.RNSignalChannelParseHandler')
/

--16
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.XMLRouterBizProcessHandler',
     'user-exit.handler.server.XMLRouterBizProcessHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.XMLRouterBizProcessHandler')
/

--17
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.pkg.MimeMultipartUnpackagingHandler',
     'user-exit.handler.server.pkg.MimeMultipartUnpackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.pkg.MimeMultipartUnpackagingHandler')
/

--18
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.pkg.NullUnPackagingHandler',
     'user-exit.handler.server.pkg.NullUnPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.pkg.NullUnPackagingHandler')
/

--19
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.pkg.RNPassThruPackagingHandler',
     'user-exit.handler.server.pkg.RNPassThruPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.pkg.RNPassThruPackagingHandler')
/

--20
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.pkg.UnPackagingFactory',
     'user-exit.handler.server.pkg.UnPackagingFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.pkg.UnPackagingFactory')
/

--21
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.soap.SOAPChannelParseHandler',
     'user-exit.handler.soap.SOAPChannelParseHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.soap.SOAPChannelParseHandler')
/

--22   
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.soap.SOAPPackagingHandler',
     'user-exit.handler.soap.SOAPPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.soap.SOAPPackagingHandler')
/

--23
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.sponsor.SponsorBusProcessFactory',
     'user-exit.handler.sponsor.SponsorBusProcessFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.sponsor.SponsorBusProcessFactory')
/

--24
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.translation.protocol.RNXsltProtFactory',
     'user-exit.handler.translation.protocol.RNXsltProtFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.translation.protocol.RNXsltProtFactory')
/

--25
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.translation.protocol.StdRNandRNSCProtFactory',
     'user-exit.handler.translation.protocol.StdRNandRNSCProtFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.translation.protocol.StdRNandRNSCProtFactory')
/

--26
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.translation.protocol.translators.xslt.XSLTranslationFactory',
     'user-exit.handler.translation.protocol.translators.xslt.XSLTranslationFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.translation.protocol.translators.xslt.XSLTranslationFactory')
/

--27
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.validation.OutboundValidationFactory',
     'user-exit.handler.validation.OutboundValidationFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.validation.OutboundValidationFactory')
/

--28
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.validation.ValidationFactory'  ,
     'user-exit.handler.validation.ValidationFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory')
/

--30
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.sync.As2SyncHdlr',
     'user-exit.handler.server.sync.As2SyncHdlr'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.sync.As2SyncHdlr')
/

--31
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.sync.CxmlSyncHdlr',
     'user-exit.handler.server.sync.CxmlSyncHdlr'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.sync.CxmlSyncHdlr')
/

--32
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.sync.RnifSyncHdlr',
     'user-exit.handler.server.sync.RnifSyncHdlr'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.sync.RnifSyncHdlr')
/

--33
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.sync.SoapSyncHdlr',
     'user-exit.handler.server.sync.SoapSyncHdlr'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.sync.SoapSyncHdlr')
/

--34
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.sync.DefaultAsynchronousSyncCheckHandler',
     'user-exit.handler.server.sync.DefaultAsynchronousSyncCheckHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.sync.DefaultAsynchronousSyncCheckHandler')
/

--35
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.sync.DefaultSynchronousSyncCheckHandler',
     'user-exit.handler.server.sync.DefaultSynchronousSyncCheckHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.sync.DefaultSynchronousSyncCheckHandler')
/

--36
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.pkg.NullPackagingHandler',
     'user-exit.handler.server.pkg.NullPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.pkg.NullPackagingHandler')
/

--37
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.ediint.ASPackagingHandler',
     'user-exit.handler.ediint.ASPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.ediint.ASPackagingHandler')
/

--new handlers for 4.3
--38
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.business.process.EDISourceValidationFactory',
     'user-exit.handler.validation.edi.SourceValidationFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.business.process.EDISourceValidationFactory')
/

--39
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.business.process.EDITargetValidationFactory',
     'user-exit.handler.validation.edi.TargetValidationFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.business.process.EDITargetValidationFactory')
/

--40
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.business.process.XMLTranslatorFactory',
     'user-exit.handler.translation.protocol.XMLTranslatorFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.business.process.XMLTranslatorFactory')
/

--41
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.business.process.EDITranslatorFactory',
     'user-exit.handler.translation.protocol.EDITranslatorFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.business.process.EDITranslatorFactory')
/

--42
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.business.process.RODTranslatorFactory',
     'user-exit.handler.ediint.ASPackagingHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.business.process.RODTranslatorFactory')
/

--43
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.business.process.EDIDenvFactory',
     'user-exit.handler.edi.EDIDenvFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.business.process.EDIDenvFactory')
/
--new for 43564
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.validation.ValidationWithoutContentFactory',
     'user-exit.handler.validation.ValidationWithoutContentFactory'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.validation.ValidationWithoutContentFactory')
/

--45
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler',
     'user-exit.handler.EDI.preprocess.EDISplitterHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler')
/

--46
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler',
     'user-exit.handler.EDI.preprocess.XMLSplitterHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler')
/

--47
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler',
     'user-exit.handler.EDI.preprocess.RODSplitterHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler')
/

--48
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler',
     'user-exit.handler.edi.GenericDocumentFlowHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler')
/

--49
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.server.EDITransactionHandler',
     'user-exit.handler.edi.EDITransactionHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.server.EDITransactionHandler')
/

--50
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.business.process.RODScannerHandler',
     'user-exit.handler.edi.RODScannerHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.business.process.RODScannerHandler')
/

--51
Insert Into PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.edi.business.process.NetworkAckHandler',
     'user-exit.handler.edi.NetworkAckHandler'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.edi.business.process.NetworkAckHandler')
/
--100624
--52
INSERT INTO PR_HANDLER(HANDLERID, CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval, 'com.ibm.bcg.server.receiver.preprocesshandler.FileNamePartnerId',
     'user-exit.handler.server.preprocess.FileNamePartnerId',
     'This Handler is applicable to extract the partner information from the file name.The file name needs to conform with the WPG defined format.  The defined format is <bcgrcv><Receiver ID><bcgsdr><Sender ID><bcgend> where the Receiver ID and the Sender ID are business ids as configured in the partner profile. The string <bcgrcv> determines the start of Receiver Id. The string <bcgsdr> determines the end of Receiver Id and Start of the Sender Id. The string <bcgend> determines the end of Sender Id.'
   FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM PR_HANDLER WHERE 
   CLASSNAME = 'com.ibm.bcg.server.receiver.preprocesshandler.FileNamePartnerId')
/

--100624
--53
INSERT INTO PR_HANDLER(HANDLERID, CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval, 'com.ibm.bcg.delivery.preprocesshandler.OutFileName',
     'user-exit.handler.gateway.preprocess.OutFileName',
     'This Handler adds partner information as part of the file name.The file name must conform to WPG format which is <bcgrcv><ReceiverId><bcgsdr><SenderId><bcgend><Filename> where ReceiverId and SenderId are business ids as configured in the partner profile.The string <bcgrcv> determines the start of ReceiverId.The string <bcgsdr> determines the end of ReceiverId and Start of the SenderId.The string <bcgend> determines the end of SenderId.The string <Filename> is the filename before this handler is applied.'
   FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM PR_HANDLER WHERE 
   CLASSNAME = 'com.ibm.bcg.delivery.preprocesshandler.OutFileName')
/


--CR60181 start
INSERT INTO PR_HANDLER(HANDLERID, CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval, 'com.ibm.bcg.server.sync.EBMSSyncCheckHandler',
     'user-exit.handler.server.sync.EBMSSyncCheckHandler',
     'This Handler checks for syncReply element in the ebXML message. If it is present, then it sets a flag for synchronous reply to be sent.'
   FROM dual
   WHERE NOT EXISTS (SELECT 'x' FROM PR_HANDLER WHERE 
   CLASSNAME = 'com.ibm.bcg.server.sync.EBMSSyncCheckHandler')
/

INSERT INTO PR_HANDLER(HANDLERID, CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.EBMSProtocolParseHandler',
     'user-exit.handler.server.EBMSProtocolParseHandler'
	,'This handler is invoked for an incoming ebXML message. It verifies the Signature for a signed message, decrypts an encrypted message and decompresses a compressed message.'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.EBMSProtocolParseHandler')
/

INSERT INTO PR_HANDLER(HANDLERID, CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.BackendChannelParseHandler',
     'user-exit.handler.server.BackendChannelParseHandler'
	,'This handler is invoked for ebXML documents sent from backend. It sets up the routing attributes on the Business Document.'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.BackendChannelParseHandler')
/

INSERT INTO PR_HANDLER(HANDLERID, CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.ebms.pkg.EBMSPackagingHandler',
     'user-exit.handler.ebms.pkg.EBMSPackagingHandler'
	,'This handler is invoked as part of Fixed OutBound workflow to add the ebMS packaging to the outbound document.'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.ebms.pkg.EBMSPackagingHandler')
/
--CR60181 end
--- 101707
INSERT INTO PR_HANDLER(HANDLERID, CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.EBMSSplitAndParseFactory',
     'user-exit.handler.ebms.business.process.EBMSSplitAndParseHandler'
	,'This handler is invoked as part of Variable workflow to reintroduce the payloads of ebXML document into the flow.'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.EBMSSplitAndParseFactory')
/

--102093
INSERT INTO PR_HANDLER(HANDLERID, CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.server.receiver.preprocesshandler.Pop3PreProcessHandler',
     'user-exit.handler.pop3.receiver.preprocesshandler.Pop3PreProcessHandler'
	,'This handler is used for removing the FRPackaging which is set as AS when the document comes from SMTP. This handler should not be used if AS document is coming from SMTP server.'
   FROM dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.server.receiver.preprocesshandler.Pop3PreProcessHandler')
/

-- Added by Bharani for F104464

INSERT INTO PR_HANDLER(HANDLERID,CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.validation.soap.WebserviceFactory',
     'user-exit.handler.validation.WebserviceFactory'
	,'This handler is used for Validating WebserviceFactory'
   FROM DUAL
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.validation.soap.WebserviceFactory')
/

INSERT INTO PR_HANDLER( HANDLERID,CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.validation.ValidationFactory',
     'user-exit.handler.validation.ValidationFactory'
	,'This handler is used for Validating ValidationFactory'
   FROM DUAL
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory')
/

INSERT INTO PR_HANDLER( HANDLERID,CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_HANDLER_SEQ.nextval,'com.ibm.bcg.validation.soap.SOAPDeEnveloperFactory',
     'user-exit.handler.soap.SOAPDe-Enveloper'
	,'This handler is used for Deenveloping SOAP'
   FROM DUAL
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.validation.soap.SOAPDeEnveloperFactory')
/

INSERT INTO PR_HANDLER( HANDLERID,CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_WORKFLOW_SEQ.Nextval,'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory',
     'user-exit.handler.EDIInterchangeValidationFactory'
	,'Action to validate interchange which will generate functional ack'
   FROM Dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory')
/

INSERT INTO PR_HANDLER( HANDLERID,CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_WORKFLOW_SEQ.Nextval,'com.ibm.bcg.wtx.business.process.WTXTransformationFactory',
     'user-exit.handler.WTXTransformationFactory'
	,'Sample Action handler for invoking a WTX Map. Attributes: WTX_RMIServer_Use_Server - Use WTX RMI Server yes (default)/no). WTX_RMIServer_Hostname - WTX RMI Server hostname location (localhost is default). WTX_RMIServer_Port - WTX RMI Server port number (2500 is default). WTX_Map_Root - WTX maps root directory location (no root directory is default). Slashes must be forward slashes (i.e."/"). Trailing slash is not required.'
   FROM Dual
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.wtx.business.process.WTXTransformationFactory')
/


INSERT INTO PR_HANDLER( HANDLERID,CLASSNAME, RESOURCEKEY, DESCRIPTION)
   SELECT PR_WORKFLOW_SEQ.Nextval,'com.ibm.bcg.wtx.business.process.EDIReEnveloperFactory',
     'user-exit.handler.WTXEDIReEnveloperFactory'
	,'This Handler will envelope the transaction. The envelope details will be picked from De-enveloped Transaction.'
   FROM DUAL
   Where Not Exists (select 'x' from PR_HANDLER WHERE
   CLASSNAME = 'com.ibm.bcg.wtx.business.process.EDIReEnveloperFactory')
/

-- PR_VARWORKFLOWHANDLERTYPE.sql

-- PR_VARWORKFLOWHANDLERTYPE

Insert Into PR_VARWORKFLOWHANDLERTYPE(VARWORKFLOWHANDLERTYPEID, RESOURCEKEY) 
   SELECT 'ACTION.VALIDATION',
   'user-exit-handler.varWorkflowType.VALIDATION'  FROM DUAL Where Not Exists 
   (select 'x' from PR_VARWORKFLOWHANDLERTYPE where 
   VARWORKFLOWHANDLERTYPEID = 'ACTION.VALIDATION' AND
   RESOURCEKEY = 'user-exit-handler.varWorkflowType.VALIDATION')
/

Insert Into PR_VARWORKFLOWHANDLERTYPE(VARWORKFLOWHANDLERTYPEID, RESOURCEKEY) 
   SELECT 'ACTION.TRANSFORMATION',
   'user-exit-handler.varWorkflowType.TRANSFORMATION'  FROM DUAL Where Not Exists 
   (select 'x' from PR_VARWORKFLOWHANDLERTYPE where 
   VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' AND
   RESOURCEKEY = 'user-exit-handler.varWorkflowType.TRANSFORMATION')
/

Insert Into PR_VARWORKFLOWHANDLERTYPE(VARWORKFLOWHANDLERTYPEID, RESOURCEKEY) 
   SELECT 'ACTION.DUPLICATECHECK',
   'user-exit-handler.varWorkflowType.DUPLICATECHECK'  FROM DUAL Where Not Exists 
   (select 'x' from PR_VARWORKFLOWHANDLERTYPE where 
   VARWORKFLOWHANDLERTYPEID = 'ACTION.DUPLICATECHECK' AND
   RESOURCEKEY = 'user-exit-handler.varWorkflowType.DUPLICATECHECK')
/
   
Insert Into PR_VARWORKFLOWHANDLERTYPE(VARWORKFLOWHANDLERTYPEID, RESOURCEKEY) 
   SELECT 'ACTION.SPLIT',
   ''  FROM DUAL Where Not Exists 
   (select 'x' from PR_VARWORKFLOWHANDLERTYPE where 
   VARWORKFLOWHANDLERTYPEID = 'ACTION.SPLIT' AND
   RESOURCEKEY is null)
/
    

-- PR_VARWORKFLOWHANDLER_CAT.sql

--PR_VARWORKFLOWHANDLER_CAT
INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.VALIDATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.validation.OutboundValidationFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.VALIDATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.VALIDATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.validation.ValidationFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.VALIDATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.outbound.OutboundDocFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.passthrough.No_op') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.rosettanet.passthru.ProcessLoggingFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.server.pkg.UnPackagingFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.sponsor.SponsorBusProcessFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.translation.protocol.RNXsltProtFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.translation.protocol.StdRNandRNSCProtFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.translation.protocol.translators.xslt.XSLTranslationFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.DUPLICATECHECK', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.DUPLICATECHECK'
                        AND HANDLERID = hid)
/

--EDI handlers
INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.VALIDATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.edi.business.process.EDISourceValidationFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.VALIDATION'
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.VALIDATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.edi.business.process.EDITargetValidationFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.VALIDATION'
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.edi.business.process.XMLTranslatorFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.edi.business.process.EDITranslatorFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.edi.business.process.RODTranslatorFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.edi.business.process.EDIDenvFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/

--100493

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID)
   SELECT 'ACTION.VALIDATION', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.validation.ValidationWithoutContentFactory')
   WHERE NOT EXISTS (SELECT 'x'
                     FROM PR_VARWORKFLOWHANDLER_CAT
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.VALIDATION'
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.SPLIT', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.server.EBMSSplitAndParseFactory')  subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.SPLIT' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.VALIDATION', hid
   FROM DUAL, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.validation.soap.WebserviceFactory')  subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.VALIDATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.VALIDATION', hid
   FROM DUAL, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.validation.ValidationFactory')  subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.VALIDATION' 
                        AND HANDLERID = hid)
/
                        
INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.SPLIT', hid
   FROM DUAL, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.validation.soap.SOAPDeEnveloperFactory')  subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.SPLIT' 
                        AND HANDLERID = hid)
/
                        
INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.VALIDATION', hid
   FROM Dual, (SELECT handlerid  hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory')  subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.VALIDATION' 
                        AND HANDLERID = hid)
/

INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM Dual, (SELECT handlerid  hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.wtx.business.process.WTXTransformationFactory')  subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/


INSERT INTO PR_VARWORKFLOWHANDLER_CAT(VARWORKFLOWHANDLERTYPEID, HANDLERID) 
   SELECT 'ACTION.TRANSFORMATION', hid
   FROM DUAL, (SELECT handlerid AS hid
                           FROM pr_handler 
                           WHERE classname = 'com.ibm.bcg.wtx.business.process.EDIReEnveloperFactory') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_VARWORKFLOWHANDLER_CAT 
                     WHERE VARWORKFLOWHANDLERTYPEID = 'ACTION.TRANSFORMATION' 
                        AND HANDLERID = hid)
/



-- CF_VARWORKFLOWHANDLER_XREF.sql

-- CF_VARWORKFLOWHANDLER_XREF

--workflow 1
--handler 9   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.passthrough.No_op') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'PassThru') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 1
--handler 8   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'PassThru') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 2
--handler 20   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.pkg.UnPackagingFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'HubOwnerCancellationOfRNProcess') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 2
--handler 28 
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'HubOwnerCancellationOfRNProcess') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 2
--handler 8   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'HubOwnerCancellationOfRNProcess') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 3
--handler 10   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.rosettanet.passthru.ProcessLoggingFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'RNPassThruWithProcessLogging') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 3
--handler 9   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.passthrough.No_op') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'RNPassThruWithProcessLogging') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 3
--handler 8   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'RNPassThruWithProcessLogging') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 4
--handler 20   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.pkg.UnPackagingFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 4
--handler 28   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 4
--handler 25   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.translation.protocol.StdRNandRNSCProtFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 4
--handler 23   
--ordering 3
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 3
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.sponsor.SponsorBusProcessFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 3)
/

--workflow 4
--handler 8   
--ordering 4
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 4
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 4)
/

--workflow 5
--handler 20   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.pkg.UnPackagingFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 5
--handler 28   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 5
--handler 24   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.translation.protocol.RNXsltProtFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 5
--handler 27   
--ordering 3
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 3
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.OutboundValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 3)
/

--workflow 5
--handler 23   
--ordering 4
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 4
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.sponsor.SponsorBusProcessFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 4)
/

--workflow 5
--handler 8   
--ordering 5
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 5
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 5)
/

--workflow 6
--handler 3   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 6
--handler 28   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 6
--handler 26   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.translation.protocol.translators.xslt.XSLTranslationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 6
--handler 27   
--ordering 3
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 3
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.OutboundValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 3)
/
           
--workflow 6
--handler 8   
--ordering 4
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 4
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 4)
/

--workflow 7
--handler 3   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 7
--handler 28   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 7
--handler 9   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.passthrough.No_op') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 7
--handler 8   
--ordering 3
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 3
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 3)
/

--workflow 8
--handler 3   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithDupCheck') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 8
--handler 9   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.passthrough.No_op') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithDupCheck') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 8
--handler 8   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithDupCheck') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 9
--handler 28   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 9
--handler 26   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.translation.protocol.translators.xslt.XSLTranslationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 9
--handler 27   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.OutboundValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 9
--handler 8   
--ordering 3
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 3
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 3)
/

--workflow 10
--handler 26   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.translation.protocol.translators.xslt.XSLTranslationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXML') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/



-- BGTRK00044541 - This handler is not needed
--workflow 10
--handler 27   
--ordering 1
--INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
--SELECT wf.workflowid, handler.handlerid, 1
--FROM (SELECT handlerid 
--      FROM PR_HANDLER 
--      WHERE  CLASSNAME = 'com.ibm.bcg.validation.OutboundValidationFactory') handler,
--     (SELECT workflowid 
--      FROM pr_Workflow 
--      WHERE workflowname = 'BiDirectionalTranslationOfCustomXML') wf
--WHERE NOT EXISTS
--   (SELECT 'x' 
--    FROM CF_VARWORKFLOWHANDLER_XREF 
--    WHERE WORKFLOWID = wf.workflowid AND
--      HANDLERID = handler.handlerid AND
--      ORDERING = 1)
--/




-- BGTRK00044541 - Change the ordering of this handler
--workflow 10
--handler 8   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfCustomXML') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 11
--handler 28   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 11
--handler 9   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.passthrough.No_op') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 11
--handler 8   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'CustomXMLPassThruWithValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 12
--handler 3   
--ordering 0
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.duplicate.ContentDuplicateProcessFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--workflow 12
--handler 20   
--ordering 1
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.pkg.UnPackagingFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--workflow 12
--handler 28   
--ordering 2
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--workflow 12
--handler 24   
--ordering 3
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 3
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.translation.protocol.RNXsltProtFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 3)
/

--workflow 12
--handler 27
--ordering 4
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 4
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.OutboundValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 4)
/

--workflow 12
--handler 23   
--ordering 5
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 5
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.sponsor.SponsorBusProcessFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 5)
/

--workflow 12
--handler 8   
--ordering 6
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 6
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 6)
/

--new for 43564
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING)
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid
      FROM PR_HANDLER
      WHERE  CLASSNAME = 'com.ibm.bcg.server.pkg.UnPackagingFactory') handler,
     (SELECT workflowid
      FROM pr_Workflow
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation') wf
WHERE NOT EXISTS
   (SELECT 'x'
    FROM CF_VARWORKFLOWHANDLER_XREF
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING)
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid
      FROM PR_HANDLER
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationWithoutContentFactory') handler,
     (SELECT workflowid
      FROM pr_Workflow
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation') wf
WHERE NOT EXISTS
   (SELECT 'x'
    FROM CF_VARWORKFLOWHANDLER_XREF
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING)
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid
      FROM PR_HANDLER
      WHERE  CLASSNAME = 'com.ibm.bcg.translation.protocol.StdRNandRNSCProtFactory') handler,
     (SELECT workflowid
      FROM pr_Workflow
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation') wf
WHERE NOT EXISTS
   (SELECT 'x'
    FROM CF_VARWORKFLOWHANDLER_XREF
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING)
SELECT wf.workflowid, handler.handlerid, 3
FROM (SELECT handlerid
      FROM PR_HANDLER
      WHERE  CLASSNAME = 'com.ibm.bcg.sponsor.SponsorBusProcessFactory') handler,
     (SELECT workflowid
      FROM pr_Workflow
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation') wf
WHERE NOT EXISTS
   (SELECT 'x'
    FROM CF_VARWORKFLOWHANDLER_XREF
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 3)
/

--new Actions for 43
--EDIDeenvelope
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.business.process.EDIDenvFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'EDIDeenvelope') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'EDIDeenvelope') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

--EDIvalidationAndEDI Translation
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.business.process.EDISourceValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'EDIvalidationAndEDI Translation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.business.process.EDITranslatorFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'EDIvalidationAndEDI Translation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'EDIvalidationAndEDI Translation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--XMLTranslationAndEDIValidation
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.business.process.XMLTranslatorFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'XMLTranslationAndEDIValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.business.process.EDITargetValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'XMLTranslationAndEDIValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'XMLTranslationAndEDIValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

--RODTranslationAndEDIValidation
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.business.process.RODTranslatorFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'RODTranslationAndEDIValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.business.process.EDITargetValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'RODTranslationAndEDIValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'RODTranslationAndEDIValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 2)
/

-- BGTRK00059953
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 4
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 4)
/
--- 101707 for CR60181
INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.EBMSSplitAndParseFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'EBMSSplitAndParse') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.soap.WebserviceFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'Validate SOAP Envelope') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/


INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.ValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'Validate SOAP Body') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.validation.soap.SOAPDeEnveloperFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'De-Envelope SOAP') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'Validate SOAP Body') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'Validate SOAP Envelope') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'De-Envelope SOAP') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'EDIInterchangeValidation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.wtx.business.process.WTXTransformationFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'WTXTransformation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.wtx.business.process.EDIReEnveloperFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'EDI ReEnvelope Action') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 0)
/

--  Added for 106658

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'WTXTransformation') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

INSERT INTO CF_VARWORKFLOWHANDLER_XREF(WORKFLOWID, HANDLERID, ORDERING) 
SELECT wf.workflowid, handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.outbound.OutboundDocFactory') handler,
     (SELECT workflowid 
      FROM pr_Workflow 
      WHERE workflowname = 'EDI ReEnvelope Action') wf
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_VARWORKFLOWHANDLER_XREF 
    WHERE WORKFLOWID = wf.workflowid AND
      HANDLERID = handler.handlerid AND
      ORDERING = 1)
/

-- PR_RESOURCEBUNDLES.sql

-- PR_RESOURCEBUNDLES

Insert Into PR_RESOURCEBUNDLES(RESOURCEBUNDLEID, RESBUNDLEFILENAME) 
   SELECT 1, 
   'com.ibm.bcg.consoleUI.bundles.event-name-resource' 
   FROM DUAL Where Not Exists (select 'x' from PR_RESOURCEBUNDLES 
   where RESOURCEBUNDLEID = 1 AND
   RESBUNDLEFILENAME = 'com.ibm.bcg.consoleUI.bundles.event-name-resource')
/

Insert Into PR_RESOURCEBUNDLES(RESOURCEBUNDLEID, RESBUNDLEFILENAME) 
   SELECT 2, 
   'com.ibm.bcg.consoleUI.bundles.event-text-resource' 
   FROM DUAL Where Not Exists (select 'x' from PR_RESOURCEBUNDLES 
   where RESOURCEBUNDLEID = 2 AND
   RESBUNDLEFILENAME = 'com.ibm.bcg.consoleUI.bundles.event-text-resource')
/ 

Insert Into PR_RESOURCEBUNDLES(RESOURCEBUNDLEID, RESBUNDLEFILENAME) 
   SELECT 3, 
   'com.ibm.bcg.consoleUI.bundles.global-text-resource' 
   FROM DUAL Where Not Exists (select 'x' from PR_RESOURCEBUNDLES 
   where RESOURCEBUNDLEID = 3 AND
   RESBUNDLEFILENAME = 'com.ibm.bcg.consoleUI.bundles.global-text-resource')
/ 

Insert Into PR_RESOURCEBUNDLES(RESOURCEBUNDLEID, RESBUNDLEFILENAME) 
   SELECT 4, 
   'com.ibm.bcg.consoleUI.bundles.edi-event-name-resource' 
   FROM DUAL Where Not Exists (select 'x' from PR_RESOURCEBUNDLES 
   where RESOURCEBUNDLEID = 4 AND
   RESBUNDLEFILENAME = 'com.ibm.bcg.consoleUI.bundles.edi-event-name-resource')
/ 

Insert Into PR_RESOURCEBUNDLES(RESOURCEBUNDLEID, RESBUNDLEFILENAME) 
   SELECT 5, 
   'com.ibm.bcg.consoleUI.bundles.edi-event-text-resource' 
   FROM DUAL Where Not Exists (select 'x' from PR_RESOURCEBUNDLES 
   where RESOURCEBUNDLEID = 5 AND
   RESBUNDLEFILENAME = 'com.ibm.bcg.consoleUI.bundles.edi-event-text-resource')
/ 


-- PR_TRANSPORTCFGPOINT.sql

-- PR_TRANSPORTCFGPOINT
Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.PREPROCESS.Http1.1',
    tid,
    'handler-selection.config-point.gateway.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Http1.1'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.Http1.1'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.POSTPROCESS.Http1.1',
    tid,
    'handler-selection.config-point.gateway.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Http1.1'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.POSTPROCESS.Http1.1'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.PREPROCESS.Https1.0',
    tid,
    'handler-selection.config-point.gateway.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Https1.0'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.Https1.0'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.POSTPROCESS.Https1.0',
    tid,
    'handler-selection.config-point.gateway.postprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Https1.0'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.POSTPROCESS.Https1.0'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.PREPROCESS.Https1.1',
    tid,
    'handler-selection.config-point.gateway.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Https1.1'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.Https1.1'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.POSTPROCESS.Https1.1',
    tid,
    'handler-selection.config-point.gateway.postprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Https1.1'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.POSTPROCESS.Https1.1'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.PREPROCESS.Ftp',
    tid,
    'handler-selection.config-point.gateway.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Ftp'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.Ftp'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.POSTPROCESS.Ftp',
    tid,
    'handler-selection.config-point.gateway.postprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Ftp'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.POSTPROCESS.Ftp'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.PREPROCESS.FtpDirectory',
    tid,
    'handler-selection.config-point.receiver.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'FtpDirectory'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FtpDirectory'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.SYNCCHECK.FtpDirectory',
    tid,
    'handler-selection.config-point.receiver.syncCheck'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'FtpDirectory'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FtpDirectory'
                     AND transportid = tid)
/

--54922
--Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
--  SELECT
--    'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory',
--    tid,
--    'handler-selection.config-point.receiver.postResponse'
--  FROM dual, (SELECT transportid AS tid
--                          FROM pr_transport
--                          WHERE transporttype = 'FtpDirectory'
--                           AND visibility = 1 
--                           AND ui_visibility = 2) subq
--  WHERE NOT EXISTS (SELECT 'X' 
--                    FROM PR_TRANSPORTCFGPOINT 
--                    WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory'
--                     AND transportid = tid)
--/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.PREPROCESS.JMS',
    tid,
    'handler-selection.config-point.receiver.postResponse'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'JMS'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.JMS'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.POSTPROCESS.JMS',
    tid,
    'handler-selection.config-point.gateway.postprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'JMS'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.POSTPROCESS.JMS'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.PREPROCESS.JMS',
    tid,
    'handler-selection.config-point.receiver.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'JMS'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.JMS'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.SYNCCHECK.JMS',
    tid,
    'handler-selection.config-point.receiver.syncCheck'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'JMS'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.JMS'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.SYNCRESPONSEPROCESS.JMS',
    tid,
    'handler-selection.config-point.receiver.postResponse'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'JMS'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.JMS'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.PREPROCESS.Smtp',
    tid,
    'handler-selection.config-point.gateway.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Smtp'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.Smtp'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.POSTPROCESS.Smtp',
    tid,
    'handler-selection.config-point.gateway.postprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'Smtp'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.POSTPROCESS.Smtp'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.PREPROCESS.Smtp',
    tid,
    'handler-selection.config-point.receiver.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'POP3'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.Smtp'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.SYNCCHECK.Smtp',
    tid,
    'handler-selection.config-point.receiver.syncCheck'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'POP3'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.Smtp'
                     AND transportid = tid)
/

--54922
--Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
--  SELECT
--    'RECEIVER.SYNCRESPONSEPROCESS.Smtp',
--    tid,
--    'handler-selection.config-point.receiver.postResponse'
--  FROM dual, (SELECT transportid AS tid
--                          FROM pr_transport
--                          WHERE transporttype = 'POP3'
--                           AND visibility = 1 
--                           AND ui_visibility = 2) subq
--  WHERE NOT EXISTS (SELECT 'X' 
--                    FROM PR_TRANSPORTCFGPOINT 
--                    WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.Smtp'
--                     AND transportid = tid)
--/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.PREPROCESS.HttpS',
    tid,
    'handler-selection.config-point.receiver.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'HttpS'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.HttpS'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.SYNCCHECK.HttpS',
    tid,
    'handler-selection.config-point.receiver.syncCheck'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'HttpS'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.SYNCRESPONSEPROCESS.HttpS',
    tid,
    'handler-selection.config-point.receiver.postResponse'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'HttpS'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.HttpS'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.PREPROCESS.FileDirectory',
    tid,
    'handler-selection.config-point.gateway.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'FileDirectoryGateway'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.FileDirectory'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'GATEWAY.POSTPROCESS.FileDirectory',
    tid,
    'handler-selection.config-point.gateway.postprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'FileDirectoryGateway'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.POSTPROCESS.FileDirectory'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.PREPROCESS.FileDirectory',
    tid,
    'handler-selection.config-point.receiver.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'FileDirectory'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FileDirectory'
                     AND transportid = tid)
/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT
    'RECEIVER.SYNCCHECK.FileDirectory',
    tid,
    'handler-selection.config-point.receiver.syncCheck'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'FileDirectory'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FileDirectory'
                     AND transportid = tid)
/

--54922
--Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
--  SELECT
--    'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory',
--    tid,
--    'handler-selection.config-point.receiver.postResponse'
--  FROM dual, (SELECT transportid AS tid
--                          FROM pr_transport
--                          WHERE transporttype = 'FileDirectory'
--                           AND visibility = 1 
--                           AND ui_visibility = 2) subq
--  WHERE NOT EXISTS (SELECT 'X' 
--                    FROM PR_TRANSPORTCFGPOINT 
--                    WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory'
--                     AND transportid = tid)
--/

Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
   SELECT
      'RECEIVER.PREPROCESS.FTPScriptingReceiver',
      tid,
      'handler-selection.config-point.receiver.postResponse'
   FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'FTPScriptingReceiver'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTCFGPOINT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FTPScriptingReceiver'
                      AND transportid = tid)
/
Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
   SELECT
      'GATEWAY.PREPROCESS.Ftps',
      tid,
      'handler-selection.config-point.gateway.preprocess'
   FROM dual, (SELECT transportid AS tid
                           FROM pr_transport
                           WHERE transporttype = 'FtpS'
                            AND visibility = 7 
                            AND ui_visibility = 1)  subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTCFGPOINT 
                     WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.Ftps'
                      AND transportid = tid)
/
Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
   SELECT
      'GATEWAY.POSTPROCESS.Ftps',
      tid,
      'handler-selection.config-point.gateway.postprocess'
   FROM dual, (SELECT transportid AS tid
                           FROM pr_transport
                           WHERE transporttype = 'FtpS'
                            AND visibility = 7 
                            AND ui_visibility = 1)  subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTCFGPOINT 
                     WHERE CFGPOINTID = 'GATEWAY.POSTPROCESS.Ftps'
                      AND transportid = tid)
/

--JR29780
--Insert into PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
--   SELECT
--      'GATEWAY.PREPROCESS.FtpScripting',
--      tid,
--      'handler-selection.config-point.gateway.preprocess'
--   FROM dual, (SELECT transportid AS tid
--                           FROM pr_transport
--                           WHERE transporttype = 'FTPScriptingGateway'
--                            AND visibility = 7 
--                            AND ui_visibility = 1)  subq
--   WHERE NOT EXISTS (SELECT 'X' 
--                     FROM PR_TRANSPORTCFGPOINT 
--                     WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.FtpScripting'
--                      AND transportid = tid)
-- /
--100624

INSERT INTO PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT 'GATEWAY.PREPROCESS.FTPScriptingGateway',
    tid, 'handler-selection.config-point.gateway.preprocess'
  FROM dual, (SELECT transportid AS tid
                          FROM pr_transport
                          WHERE transporttype = 'FTPScriptingGateway'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.FTPScriptingGateway'
                     AND transportid = tid)
/

-- Start 104397
Update PR_TRANSPORTCFGPOINT Set RESOURCEKEY = 'handler-selection.config-point.gateway.postResponse'
WHERE transportid = (SELECT transportid 
                          FROM pr_transport
                          WHERE transporttype = 'Https1.0'
                           AND visibility = 7 
                           AND ui_visibility = 1)
AND CFGPOINTID = 'GATEWAY.POSTPROCESS.Https1.0'
/
Update PR_TRANSPORTCFGPOINT Set RESOURCEKEY = 'handler-selection.config-point.gateway.postResponse'
WHERE transportid = (SELECT transportid 
                          FROM pr_transport
                          WHERE transporttype = 'Https1.1'
                           AND visibility = 7 
                           AND ui_visibility = 1)
AND CFGPOINTID = 'GATEWAY.POSTPROCESS.Https1.1'
/
Update PR_TRANSPORTCFGPOINT Set RESOURCEKEY = 'handler-selection.config-point.gateway.postResponse'
WHERE transportid = (SELECT transportid 
                          FROM pr_transport
                          WHERE transporttype = 'Ftp'
                           AND visibility = 7 
                           AND ui_visibility = 1)
AND CFGPOINTID = 'GATEWAY.POSTPROCESS.Ftp'
/
Update PR_TRANSPORTCFGPOINT Set RESOURCEKEY = 'handler-selection.config-point.gateway.postResponse'
WHERE transportid = (SELECT transportid 
                          FROM pr_transport
                          WHERE transporttype = 'JMS'
                           AND visibility = 7 
                           AND ui_visibility = 1)
AND CFGPOINTID = 'GATEWAY.POSTPROCESS.JMS'
/
Update PR_TRANSPORTCFGPOINT Set RESOURCEKEY = 'handler-selection.config-point.gateway.postResponse'
WHERE transportid = (SELECT transportid 
                          FROM pr_transport
                          WHERE transporttype = 'Smtp'
                           AND visibility = 7 
                           AND ui_visibility = 1)
AND CFGPOINTID = 'GATEWAY.POSTPROCESS.Smtp'
/
Update PR_TRANSPORTCFGPOINT Set RESOURCEKEY = 'handler-selection.config-point.gateway.postResponse'
WHERE transportid = (SELECT transportid 
                          FROM pr_transport
                          WHERE transporttype = 'FileDirectoryGateway'
                           AND visibility = 7 
                           AND ui_visibility = 1)
AND CFGPOINTID = 'GATEWAY.POSTPROCESS.FileDirectory'
/
Update PR_TRANSPORTCFGPOINT Set RESOURCEKEY = 'handler-selection.config-point.gateway.postResponse'
WHERE transportid = (SELECT transportid 
                          FROM pr_transport
                          WHERE transporttype = 'FtpS'
                           AND visibility = 7 
                           AND ui_visibility = 1)
AND CFGPOINTID = 'GATEWAY.POSTPROCESS.Ftps'
/
-- End   104397
-- Start 108414
INSERT INTO PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT 'RECEIVER.PREPROCESS.SFTP',
    tid, 'handler-selection.config-point.receiver.preprocess'
  FROM DUAL, (SELECT transportid tid
                          FROM pr_transport
                          WHERE transporttype = 'SFTPReceiver'
                           AND visibility = 1 
                           AND ui_visibility = 2) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.SFTP'
                     AND transportid = tid)
/
INSERT INTO PR_TRANSPORTCFGPOINT(CFGPOINTID, TRANSPORTID, RESOURCEKEY)
  SELECT 'GATEWAY.PREPROCESS.SFTP',
    tid, 'handler-selection.config-point.gateway.preprocess'
  FROM DUAL, (SELECT transportid tid
                          FROM pr_transport
                          WHERE transporttype = 'SFTPSender'
                           AND visibility = 7 
                           AND ui_visibility = 1) subq
  WHERE NOT EXISTS (SELECT 'X' 
                    FROM PR_TRANSPORTCFGPOINT 
                    WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.SFTP'
                     AND transportid = tid)
/
-- End 108414


-- PR_FIXEDWORKFLOWSTEP.sql

-- PR_FIXEDWORKFLOWSTEP

Insert Into PR_FIXEDWORKFLOWSTEP(FIXEDWOKFLOWSTEPID, CLASSNAME, RESOURCEKEY, CONFIGURABLE)
   SELECT 
   'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING','com.ibm.bcg.server.transport.TransportUnPackagingFactory',
   'user-exit.fixedWorkflow.server.transport.TransportUnPackagingFactory',
   1
   FROM DUAL WHERE NOT EXISTS
   (SELECT 'x' FROM PR_FIXEDWORKFLOWSTEP WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' 
   AND CLASSNAME = 'com.ibm.bcg.server.transport.TransportUnPackagingFactory')
/

Insert Into PR_FIXEDWORKFLOWSTEP(FIXEDWOKFLOWSTEPID, CLASSNAME, RESOURCEKEY, CONFIGURABLE)
   SELECT 
   'FIXEDWORKFLOW.PROTOCOL.PARSE','com.ibm.bcg.server.ChannelParseFactory',
   'user-exit.fixedWorkflow.server.ChannelParseFactory',
   1   
   FROM DUAL WHERE NOT EXISTS
   (SELECT 'x' FROM PR_FIXEDWORKFLOWSTEP WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND 
   CLASSNAME = 'com.ibm.bcg.server.ChannelParseFactory')
/

Insert Into PR_FIXEDWORKFLOWSTEP(FIXEDWOKFLOWSTEPID, CLASSNAME, RESOURCEKEY, CONFIGURABLE)
   SELECT 
   'FIXEDWORKFLOW.DESTINATION.TYPE','com.ibm.bcg.destination.DestinationParseFactory',
   'user-exit.fixedWorkflow.destination.DestinationParseFactory',
   0
   FROM DUAL WHERE NOT EXISTS
   (SELECT 'x' FROM PR_FIXEDWORKFLOWSTEP WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.DESTINATION.TYPE' AND 
   CLASSNAME = 'com.ibm.bcg.destination.DestinationParseFactory')
/

Insert Into PR_FIXEDWORKFLOWSTEP(FIXEDWOKFLOWSTEPID, CLASSNAME, RESOURCEKEY, CONFIGURABLE)
   SELECT 
   'FIXEDWORKFLOW.DESTINATION.PROCESS','com.ibm.bcg.destination.DestinationProcessFactory',
   'user-exit.fixedWorkflow.destination.DestinationProcessFactory',
   0
   FROM DUAL WHERE NOT EXISTS
   (SELECT 'x' FROM PR_FIXEDWORKFLOWSTEP WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.DESTINATION.PROCESS' AND 
   CLASSNAME = 'com.ibm.bcg.destination.DestinationProcessFactory')
/

Insert Into PR_FIXEDWORKFLOWSTEP(FIXEDWOKFLOWSTEPID, CLASSNAME, RESOURCEKEY, CONFIGURABLE)
   SELECT 
   'FIXEDWORKFLOW.CHANNEL.CHECK','com.ibm.bcg.server.ChannelCheckFactory',
   'user-exit.fixedWorkflow.server.ChannelCheckFactory',
   0
   FROM DUAL WHERE NOT EXISTS
   (SELECT 'x' FROM PR_FIXEDWORKFLOWSTEP WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.CHANNEL.CHECK' AND 
   CLASSNAME = 'com.ibm.bcg.server.ChannelCheckFactory')
/

Insert Into PR_FIXEDWORKFLOWSTEP(FIXEDWOKFLOWSTEPID, CLASSNAME, RESOURCEKEY, CONFIGURABLE)
   SELECT 
   'FIXEDWORKFLOW.TRANSPORT.STATE','com.ibm.bcg.server.transport.TransportLoggingFactory',
   'user-exit.fixedWorkflow.server.transport.TransportLoggingFactory',
   0
   FROM DUAL WHERE NOT EXISTS
   (SELECT 'x' FROM PR_FIXEDWORKFLOWSTEP WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.TRANSPORT.STATE' AND 
   CLASSNAME = 'com.ibm.bcg.server.transport.TransportLoggingFactory')
/

Insert Into PR_FIXEDWORKFLOWSTEP(FIXEDWOKFLOWSTEPID, CLASSNAME, RESOURCEKEY, CONFIGURABLE)
   SELECT 
   'FIXEDWORKFLOW.DUPLICATE.CHECK','com.ibm.bcg.duplicate.DuplicateProcessFactory',
   'user-exit.fixedWorkflow.duplicate.DuplicateProcessFactory',
   0
   FROM DUAL WHERE NOT EXISTS
   (SELECT 'x' FROM PR_FIXEDWORKFLOWSTEP WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.DUPLICATE.CHECK' AND 
   CLASSNAME = 'com.ibm.bcg.duplicate.DuplicateProcessFactory')
/

Insert Into PR_FIXEDWORKFLOWSTEP(FIXEDWOKFLOWSTEPID, CLASSNAME, RESOURCEKEY, CONFIGURABLE)
   SELECT 
   'FIXEDWORKFLOW.PROTOCOL.PACKAGING','com.ibm.bcg.server.pkg.ProtocolPackagingFactory',
   'user-exit.fixedWorkflow.server.pkg.PackagingFactory',
   1
   FROM DUAL WHERE NOT EXISTS
   (SELECT 'x' FROM PR_FIXEDWORKFLOWSTEP WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND 
   CLASSNAME = 'com.ibm.bcg.server.pkg.ProtocolPackagingFactory')
/

-- CF_FIXEDWORKFLOWHANDLER_XREF.sql

-- CF_FIXEDWORKFLOWHANDLER_XREF
-- Changed for 108524.  Only FIXEDWOKFLOWSTEPID and Handlerid is Unique.  No need to check Ordering
-- ON NOT EXISTS Clause


INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING', handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.ediint.ASUnPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' AND
      HANDLERID = handler.handlerid 
      )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING', handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.pkg.NullUnPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING', handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.pkg.MimeMultipartUnpackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING', handler.handlerid, 3
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.eai.EAIUnPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.RNOChannelParseHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.RNSignalChannelParseHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.RNSCChannelParseHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /
-- handlers after this moved down to make way for ebMS handlers (at Ordering=3,4)
INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 5
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.BinaryChannelParseHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )
 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 6
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.cxml.cXMLChannelParseHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /
-- handlers after this moved down to make way for ebMS handler
INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 7
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.soap.SOAPChannelParseHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 8
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.XMLRouterBizProcessHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 9
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.EDIRouterBizProcessHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /

--added ordering 8 9 for 47515
INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 10
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.business.process.RODScannerHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 11
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.business.process.NetworkAckHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /


INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', handler.handlerid, 1
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.rosettanet.pkg.RNOPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', handler.handlerid, 2
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.pkg.RNPassThruPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', handler.handlerid, 3
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.cxml.cXMLPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', handler.handlerid, 4
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.soap.SOAPPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', handler.handlerid, 5
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.eai.EAIPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.pkg.NullPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND
      HANDLERID = handler.handlerid )

 /

INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.ediint.ASPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND
      HANDLERID = handler.handlerid )

 /

--added for 47512
INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', handler.handlerid, 0
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.edi.server.EDITransactionHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND
      HANDLERID = handler.handlerid )

 /
--- CR60181 Start
--- Need to add BackendChannelParseHandler and EBMSProtocolParse handler BEFORE BinaryChannelParseHandler
--- Push all the handlers down by 2 positions- if BinaryChannelParseHandler is at ORDERING=3
UPDATE CF_FIXEDWORKFLOWHANDLER_XREF
SET ORDERING = ORDERING + 2
 ,updateDate = Fn_Get_Date
WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE'
AND ORDERING >= (SELECT ordering
  FROM cf_fixedworkflowhandler_xref fs
  INNER JOIN pr_handler h ON h.handlerid = fs.handlerid
  WHERE fixedwokflowstepid = 'FIXEDWORKFLOW.PROTOCOL.PARSE'
  AND h.classname = 'com.ibm.bcg.server.BinaryChannelParseHandler'
  )

 /
--- Insert the new handler at position 3
INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 3
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.BackendChannelParseHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid)

 /
--- Insert the new handler at position 4
INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', handler.handlerid, 4
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.server.EBMSProtocolParseHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' AND
      HANDLERID = handler.handlerid )

 /
--- Insert new handler for packaging
INSERT INTO CF_FIXEDWORKFLOWHANDLER_XREF(FIXEDWOKFLOWSTEPID, HANDLERID, ORDERING)
SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', handler.handlerid, 6
FROM (SELECT handlerid 
      FROM PR_HANDLER 
      WHERE  CLASSNAME = 'com.ibm.bcg.ebms.pkg.EBMSPackagingHandler') handler
WHERE NOT EXISTS
   (SELECT 'x' 
    FROM CF_FIXEDWORKFLOWHANDLER_XREF
    WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' AND
      HANDLERID = handler.handlerid )

 /

--- CR60181 End


-- CF_FIXEDWORKFLOWSTEP_XREF.sql

-- CF_FIXEDWORKFLOWSTEP_XREF
INSERT INTO cf_fixedworkflowstep_xref(fixedworkflowid, fixedwokflowstepid, ordering) 
   SELECT fwid,'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING', 0
   FROM dual, (SELECT fixedworkflowid AS fwid
                           FROM pr_fixedworkflow
                           WHERE workflowname = 'Inbound') subq 
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM cf_fixedworkflowstep_xref
                     WHERE fixedworkflowid = fwid 
                        AND fixedwokflowstepid = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' 
                        )

 /

INSERT INTO cf_fixedworkflowstep_xref(fixedworkflowid, fixedwokflowstepid, ordering) 
   SELECT fwid,'FIXEDWORKFLOW.PROTOCOL.PARSE', 1
   FROM dual, (SELECT fixedworkflowid AS fwid
                           FROM pr_fixedworkflow
                           WHERE workflowname = 'Inbound') subq 
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM cf_fixedworkflowstep_xref
                     WHERE fixedworkflowid = fwid 
                        AND fixedwokflowstepid = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        )

 /

INSERT INTO cf_fixedworkflowstep_xref(fixedworkflowid, fixedwokflowstepid, ordering) 
   SELECT fwid,'FIXEDWORKFLOW.DESTINATION.TYPE', 2
   FROM dual, (SELECT fixedworkflowid AS fwid
                           FROM pr_fixedworkflow
                           WHERE workflowname = 'Inbound') subq 
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM cf_fixedworkflowstep_xref
                     WHERE fixedworkflowid = fwid 
                        AND fixedwokflowstepid = 'FIXEDWORKFLOW.DESTINATION.TYPE' 
                        )

 /

INSERT INTO cf_fixedworkflowstep_xref(fixedworkflowid, fixedwokflowstepid, ordering) 
   SELECT fwid,'FIXEDWORKFLOW.DESTINATION.PROCESS', 3
   FROM dual, (SELECT fixedworkflowid AS fwid
                           FROM pr_fixedworkflow
                           WHERE workflowname = 'Inbound') subq 
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM cf_fixedworkflowstep_xref
                     WHERE fixedworkflowid = fwid 
                        AND fixedwokflowstepid = 'FIXEDWORKFLOW.DESTINATION.PROCESS' 
                        )

 /

INSERT INTO cf_fixedworkflowstep_xref(fixedworkflowid, fixedwokflowstepid, ordering) 
   SELECT fwid,'FIXEDWORKFLOW.CHANNEL.CHECK', 4
   FROM dual, (SELECT fixedworkflowid AS fwid
                           FROM pr_fixedworkflow
                           WHERE workflowname = 'Inbound') subq 
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM cf_fixedworkflowstep_xref
                     WHERE fixedworkflowid = fwid 
                        AND fixedwokflowstepid = 'FIXEDWORKFLOW.CHANNEL.CHECK' 
                        )

 /

/*  -- CR 100632    
INSERT INTO cf_fixedworkflowstep_xref(fixedworkflowid, fixedwokflowstepid, ordering) 
   SELECT fwid,'FIXEDWORKFLOW.TRANSPORT.STATE', 5
   FROM dual, (SELECT fixedworkflowid AS fwid
                           FROM pr_fixedworkflow
                           WHERE workflowname = 'Inbound') subq 
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM cf_fixedworkflowstep_xref
                     WHERE fixedworkflowid = fwid 
                        AND fixedwokflowstepid = 'FIXEDWORKFLOW.TRANSPORT.STATE' 
                        AND ordering = 5)

 /
*/

INSERT INTO cf_fixedworkflowstep_xref(fixedworkflowid, fixedwokflowstepid, ordering) 
   SELECT fwid,'FIXEDWORKFLOW.DUPLICATE.CHECK', 6
   FROM dual, (SELECT fixedworkflowid AS fwid
                           FROM pr_fixedworkflow
                           WHERE workflowname = 'Inbound') subq 
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM cf_fixedworkflowstep_xref
                     WHERE fixedworkflowid = fwid 
                        AND fixedwokflowstepid = 'FIXEDWORKFLOW.DUPLICATE.CHECK' 
                        )  

 /

INSERT INTO cf_fixedworkflowstep_xref(fixedworkflowid, fixedwokflowstepid, ordering) 
   SELECT fwid,'FIXEDWORKFLOW.PROTOCOL.PACKAGING', 0
   FROM dual, (SELECT fixedworkflowid AS fwid
                           FROM pr_fixedworkflow
                           WHERE workflowname = 'Outbound') subq 
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM cf_fixedworkflowstep_xref
                     WHERE fixedworkflowid = fwid 
                        AND fixedwokflowstepid = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        )

 /


-- PR_FIXEDWORKFLOWHANDLER_CAT.sql

-- PR_FIXEDWORKFLOWHANDLER_CAT
INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.ediint.ASUnPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.pkg.NullUnPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.pkg.MimeMultipartUnpackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.eai.EAIUnPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.UNPACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.RNOChannelParseHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.RNSignalChannelParseHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.RNSCChannelParseHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.BinaryChannelParseHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.cxml.cXMLChannelParseHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.soap.SOAPChannelParseHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.XMLRouterBizProcessHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.EDIRouterBizProcessHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.rosettanet.pkg.RNOPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.pkg.RNPassThruPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.cxml.cXMLPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.soap.SOAPPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.eai.EAIPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.pkg.NullPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.ediint.ASPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        AND HANDLERID = hid)
/

--47512 47515
INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.server.EDITransactionHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.business.process.RODScannerHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/

INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.business.process.NetworkAckHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE' 
                        AND HANDLERID = hid)
/
--- CR60181
INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PACKAGING', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.ebms.pkg.EBMSPackagingHandler') subq
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PACKAGING' 
                        AND HANDLERID = hid)
/
INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.EBMSProtocolParseHandler')  subq
   WHERE NOT EXISTS (SELECT 'x'
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE'
                        AND HANDLERID = hid)
/
INSERT INTO pr_fixedworkflowhandler_cat(fixedwokflowstepid, handlerid)
   SELECT 'FIXEDWORKFLOW.PROTOCOL.PARSE', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.BackendChannelParseHandler') subq
   WHERE NOT EXISTS (SELECT 'x'
                     FROM PR_FIXEDWORKFLOWHANDLER_CAT
                     WHERE FIXEDWOKFLOWSTEPID = 'FIXEDWORKFLOW.PROTOCOL.PARSE'
                        AND HANDLERID = hid)
/


-- PR_TRANSPORTHANDLER_CAT.sql

--PR_TRANSPORTHANDLER_CAT
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.sync.As2SyncHdlr') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS' 
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.sync.CxmlSyncHdlr') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.sync.RnifSyncHdlr') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.sync.SoapSyncHdlr') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.sync.DefaultAsynchronousSyncCheckHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.sync.DefaultSynchronousSyncCheckHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.SYNCCHECK.JMS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.sync.DefaultAsynchronousSyncCheckHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.JMS'
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.SYNCCHECK.JMS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.sync.DefaultSynchronousSyncCheckHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.JMS'
                     AND HANDLERID = hid)
/

--need to associate the splitter handlers with ALL PREPROCESS receivers
--and the Generic Document Flow Handler
--RECEIVER.PREPROCESS.FTPScriptingReceiver
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.FTPScriptingReceiver', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FTPScriptingReceiver' 
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.FTPScriptingReceiver', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FTPScriptingReceiver' 
                     AND HANDLERID = hid)
/
   
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.FTPScriptingReceiver', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FTPScriptingReceiver' 
		               AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.FTPScriptingReceiver', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FTPScriptingReceiver' 
		               AND HANDLERID = hid)
/

--RECEIVER.PREPROCESS.FileDirectory
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.FileDirectory', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FileDirectory' 
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.FileDirectory', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FileDirectory' 
                     AND HANDLERID = hid)
/
   
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.FileDirectory', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FileDirectory' 
		               AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.FileDirectory', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FileDirectory' 
		               AND HANDLERID = hid)
/

--RECEIVER.PREPROCESS.FtpDirectory
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.FtpDirectory', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FtpDirectory' 
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.FtpDirectory', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FtpDirectory' 
                     AND HANDLERID = hid)
/
   
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.FtpDirectory', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FtpDirectory' 
		               AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.FtpDirectory', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FtpDirectory' 
		               AND HANDLERID = hid)
/

--RECEIVER.PREPROCESS.HttpS
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.HttpS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.HttpS' 
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.HttpS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.HttpS' 
                     AND HANDLERID = hid)
/
   
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.HttpS', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.HttpS' 
		               AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.HttpS', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.HttpS' 
		               AND HANDLERID = hid)
/

--RECEIVER.PREPROCESS.JMS
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.JMS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.JMS' 
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.JMS', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.JMS' 
                     AND HANDLERID = hid)
/
   
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.JMS', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.JMS' 
		               AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.JMS', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.JMS' 
		               AND HANDLERID = hid)
/

--RECEIVER.PREPROCESS.Smtp
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.Smtp', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.Smtp' 
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.Smtp', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.Smtp' 
                     AND HANDLERID = hid)
/
   
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.Smtp', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.Smtp' 
		               AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.Smtp', hid
	FROM dual, (SELECT handlerid AS hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.Smtp' 
		               AND HANDLERID = hid)
/

--RECEIVER.SYNCCHECK.FileDirectory
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.FileDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FileDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.FileDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FileDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.FileDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FileDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.FileDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FileDirectory'
--                      AND HANDLERID = hid)
-- /

-- --RECEIVER.SYNCCHECK.FtpDirectory
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.FtpDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FtpDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.FtpDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FtpDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.FtpDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FtpDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.FtpDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.FtpDirectory'
--                      AND HANDLERID = hid)
-- /

--RECEIVER.SYNCCHECK.HttpS
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS'
--                      AND HANDLERID = hid)
-- /

--RECEIVER.SYNCCHECK.JMS
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.JMS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.JMS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.JMS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.JMS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.JMS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.JMS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.JMS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.JMS'
--                      AND HANDLERID = hid)
-- /

--RECEIVER.SYNCCHECK.Smtp
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.Smtp', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.Smtp'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.Smtp', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.Smtp'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.Smtp', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.Smtp'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCCHECK.Smtp', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.Smtp'
--                      AND HANDLERID = hid)
-- /

--RECEIVER.SYNCRESPONSEPROCESS.FileDirectory
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FileDirectory'
--                      AND HANDLERID = hid)
-- /

--RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.FtpDirectory'
--                      AND HANDLERID = hid)
-- /

--RECEIVER.SYNCRESPONSEPROCESS.HttpS
--    INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.HttpS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.HttpS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.HttpS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.HttpS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.HttpS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.HttpS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.HttpS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.HttpS'
--                      AND HANDLERID = hid)
-- /

--RECEIVER.SYNCRESPONSEPROCESS.JMS
--    INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.JMS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.JMS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.JMS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.JMS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.JMS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.JMS'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.JMS', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.JMS'
--                      AND HANDLERID = hid)
-- /

--RECEIVER.SYNCRESPONSEPROCESS.Smtp
--    INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.Smtp', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.Smtp'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.Smtp', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.Smtp'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.Smtp', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.Smtp'
--                      AND HANDLERID = hid)
-- /
--
-- INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
--    SELECT 'RECEIVER.SYNCRESPONSEPROCESS.Smtp', hid
--    FROM dual, (SELECT handlerid AS hid
--                            FROM pr_handler
--                            WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
--    WHERE NOT EXISTS (SELECT 'X'
--                      FROM PR_TRANSPORTHANDLER_CAT
--                      WHERE CFGPOINTID = 'RECEIVER.SYNCRESPONSEPROCESS.Smtp'
--                      AND HANDLERID = hid)
-- /
--100624
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.FileDirectory', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.receiver.preprocesshandler.FileNamePartnerId') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FileDirectory' 
                     AND HANDLERID = hid)
/

--100624
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.FTPScriptingReceiver', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.server.receiver.preprocesshandler.FileNamePartnerId') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.FTPScriptingReceiver' 
                     AND HANDLERID = hid)
/

--100624
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'GATEWAY.PREPROCESS.FileDirectory', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.delivery.preprocesshandler.OutFileName') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.FileDirectory' 
                     AND HANDLERID = hid)
/

--100624
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'GATEWAY.PREPROCESS.FTPScriptingGateway', hid
   FROM dual, (SELECT handlerid AS hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.delivery.preprocesshandler.OutFileName') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.FTPScriptingGateway' 
                     AND HANDLERID = hid)
/
--CR60181
--PR_TRANSPORTHANDLER_CAT
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.SYNCCHECK.HttpS', hid
   FROM (SELECT handlerid AS hid
           FROM pr_handler
          WHERE classname = 'com.ibm.bcg.server.sync.EBMSSyncCheckHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.SYNCCHECK.HttpS' 
                     AND HANDLERID = hid)
/

--54698 Document splitting preprocess not applicable to POP3 target 
--RECEIVER.PREPROCESS.Smtp = POP3
--adding delete statements so there won't have to be a change for migration
DELETE FROM pr_transporthandler_cat
WHERE cfgpointid = 'RECEIVER.PREPROCESS.Smtp'
   AND handlerid = (SELECT handlerid AS hid
                    FROM pr_handler
                    WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler')
/

DELETE FROM pr_transporthandler_cat
WHERE cfgpointid = 'RECEIVER.PREPROCESS.Smtp'
   AND handlerid = (SELECT handlerid AS hid
                    FROM pr_handler
                    WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler')
/

DELETE FROM pr_transporthandler_cat
WHERE cfgpointid = 'RECEIVER.PREPROCESS.Smtp'
   AND handlerid = (SELECT handlerid AS hid
                    FROM pr_handler
                    WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler')
/
--end 54698

--102093
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.Smtp', hid
   FROM (SELECT handlerid AS hid
           FROM pr_handler
          WHERE classname = 'com.ibm.bcg.server.receiver.preprocesshandler.Pop3PreProcessHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.Smtp' 
                     AND HANDLERID = hid)
/
-- Start 108414
--need to associate the splitter handlers with ALL PREPROCESS receivers
--as well as the Generic Document Flow Handler
--RECEIVER.PREPROCESS.SFTP
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.SFTP', hid
   FROM DUAL, (SELECT handlerid hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.SFTP' 
                     AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'RECEIVER.PREPROCESS.SFTP', hid
   FROM DUAL, (SELECT handlerid hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.SFTP' 
                     AND HANDLERID = hid)
/
   
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.SFTP', hid
   	FROM DUAL, (SELECT handlerid hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.SFTP' 
		               AND HANDLERID = hid)
/

INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
	SELECT 'RECEIVER.PREPROCESS.SFTP', hid
   	FROM DUAL, (SELECT handlerid hid
		                     FROM pr_handler
		                     WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') subq
	WHERE NOT EXISTS (SELECT 'X' 
	                  FROM PR_TRANSPORTHANDLER_CAT 
		               WHERE CFGPOINTID = 'RECEIVER.PREPROCESS.SFTP' 
		               AND HANDLERID = hid)
/
INSERT INTO PR_TRANSPORTHANDLER_CAT(CFGPOINTID, HANDLERID)
   SELECT 'GATEWAY.PREPROCESS.SFTP', hid
   FROM DUAL, (SELECT handlerid hid
                           FROM pr_handler
                           WHERE classname = 'com.ibm.bcg.delivery.preprocesshandler.OutFileName') subq
   WHERE NOT EXISTS (SELECT 'X' 
                     FROM PR_TRANSPORTHANDLER_CAT 
                     WHERE CFGPOINTID = 'GATEWAY.PREPROCESS.SFTP' 
                     AND HANDLERID = hid)
/
-- END 108414


-- PR_TRANSPORTATTR.sql

-- PR_TRANSPORT_ATTR

--select * from PR_TRANSPORT_ATTR
--receiver attributes
INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.SERVERIP','bcg.transport.SERVERIP' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.SERVERIP' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.USERID','bcg.transport.USERID' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.USERID' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.PASSWORD','bcg.transport.PASSWORD' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.PASSWORD' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.FTPSMODE','bcg.transport.FTPSMODE' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.FTPSMODE' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.SCRIPT_FILE','bcg.transport.SCRIPT_FILE' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.SCRIPT_FILE' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.DAYOFWEEK','bcg.transport.DAYOFWEEK' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.DAYOFWEEK' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.HOUROPT','bcg.transport.HOUROPT' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.HOUROPT' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.MINUTEOPT','bcg.transport.MINUTEOPT' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.MINUTEOPT' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.DAYOFMONTH','bcg.transport.DAYOFMONTH' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.DAYOFMONTH' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey,defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.POLLINTERVAL','bcg.transport.POLLINTERVAL','60000'
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.POLLINTERVAL' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.LOCKUSER','bcg.transport.LOCKUSER' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.LOCKUSER' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

--new bcg.receiver.CONNECTIONTIMEOUT

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey,defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.CONNECTIONTIMEOUT','bcg.transport.CONNECTIONTIMEOUT',120000 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.CONNECTIONTIMEOUT' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/



--gateway attributes
INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.SERVERIP','bcg.transport.SERVERIP' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.SERVERIP' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.USERID','bcg.transport.USERID' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.USERID' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.PASSWORD','bcg.transport.PASSWORD' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.PASSWORD' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.FTPSMODE','bcg.transport.FTPSMODE' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.FTPSMODE' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.SCRIPT_FILE','bcg.transport.SCRIPT_FILE' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.SCRIPT_FILE' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.RETRY_COUNT','bcg.transport.RETRY_COUNT','3'
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.RETRY_COUNT' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.RETRY_INTERVAL','bcg.transport.RETRY_INTERVAL','300000'
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.RETRY_INTERVAL' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.DAYOFWEEK','bcg.transport.DAYOFWEEK' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.DAYOFWEEK' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.HOUROPT','bcg.transport.HOUROPT' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.HOUROPT' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.MINUTEOPT','bcg.transport.MINUTEOPT' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.MINUTEOPT' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.DAYOFMONTH','bcg.transport.DAYOFMONTH' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.DAYOFMONTH' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey,defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.POLLINTERVAL','bcg.transport.POLLINTERVAL','60000'
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.POLLINTERVAL' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.LOCKUSER','bcg.transport.LOCKUSER' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.LOCKUSER' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/



--new bcg.gateway.CONNECTIONTIMEOUT

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey,defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.CONNECTIONTIMEOUT','bcg.transport.CONNECTIONTIMEOUT',120000 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'), DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.CONNECTIONTIMEOUT' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/


INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'Unique FileName', 'bcg.transport.uniqueFileName', 'Yes'
   FROM pr_transport WHERE transporttype = 'FileDirectoryGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'Unique FileName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FileDirectoryGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'Unique FileName', 'bcg.transport.uniqueFileName', 'Yes'
   FROM pr_transport WHERE transporttype = 'Ftp'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'Unique FileName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'Ftp'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'Unique FileName', 'bcg.transport.uniqueFileName', 'Yes'
   FROM pr_transport WHERE transporttype = 'FtpS'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'Unique FileName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FtpS'))
/

--new
INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option1', 'bcg.transport.Option1'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option1' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option2', 'bcg.transport.Option2'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option2' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option3', 'bcg.transport.Option3'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option3' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option4', 'bcg.transport.Option4'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option4' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option5', 'bcg.transport.Option5'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option5' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option6', 'bcg.transport.Option6'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option6' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option7', 'bcg.transport.Option7'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option7' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option8', 'bcg.transport.Option8'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option8' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option9', 'bcg.transport.Option9'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option9' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.Option10', 'bcg.transport.Option10'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.Option10' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

--now do ftpscripting gateways
INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option1', 'bcg.transport.Option1'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option1' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option2', 'bcg.transport.Option2'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option2' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option3', 'bcg.transport.Option3'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option3' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option4', 'bcg.transport.Option4'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option4' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option5', 'bcg.transport.Option5'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option5' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option6', 'bcg.transport.Option6'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option6' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option7', 'bcg.transport.Option7'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option7' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option8', 'bcg.transport.Option8'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option8' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option9', 'bcg.transport.Option9'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option9' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.Option10', 'bcg.transport.Option10'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.Option10' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/
INSERT INTO pr_transport_attr (
	TRANSPORTATTRID
	,TRANSPORTID
	,ATTRNAME
	,DEFAULTATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,UPDATEDATE
	,CREATEDATE
) (SELECT
	PR_TRANSPORT_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'bcg.fwdproxy.proxyName'
	,'Use no forward proxy'
	,'bcg.fwdproxy.proxyName'
	,'Use no forward proxy'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE IN ('Http1.1')
  AND	NOT EXISTS (SELECT 'x'
		FROM	pr_transport_attr
		WHERE	ATTRNAME = 'bcg.fwdproxy.proxyName'
		AND	TRANSPORTID = (SELECT TRANSPORTID
				FROM	pr_transport
				WHERE	transporttype = 'Http1.1'
				)
		)
)
/
INSERT INTO pr_transport_attr (
	TRANSPORTATTRID
	,TRANSPORTID
	,ATTRNAME
	,DEFAULTATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,UPDATEDATE
	,CREATEDATE
) (SELECT
	PR_TRANSPORT_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'bcg.fwdproxy.proxyName'
	,'Use no forward proxy'
	,'bcg.fwdproxy.proxyName'
	,'Use no forward proxy'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE IN ('Https1.0')
  AND	NOT EXISTS (SELECT 'x'
		FROM	pr_transport_attr
		WHERE	ATTRNAME = 'bcg.fwdproxy.proxyName'
		AND	TRANSPORTID = (SELECT TRANSPORTID
				FROM	pr_transport
				WHERE	transporttype = 'Https1.0'
				)
		)
)
/
INSERT INTO pr_transport_attr (
	TRANSPORTATTRID
	,TRANSPORTID
	,ATTRNAME
	,DEFAULTATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,UPDATEDATE
	,CREATEDATE
) (SELECT
	PR_TRANSPORT_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'bcg.fwdproxy.proxyName'
	,'Use no forward proxy'
	,'bcg.fwdproxy.proxyName'
	,'Use no forward proxy'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE IN ('Https1.1')
  AND	NOT EXISTS (SELECT 'x'
		FROM	pr_transport_attr
		WHERE	ATTRNAME = 'bcg.fwdproxy.proxyName'
		AND	TRANSPORTID = (SELECT TRANSPORTID
				FROM	pr_transport
				WHERE	transporttype = 'Https1.1'
				)
		)
)
/
INSERT INTO pr_transport_attr (
	TRANSPORTATTRID
	,TRANSPORTID
	,ATTRNAME
	,DEFAULTATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,UPDATEDATE
	,CREATEDATE
) (SELECT
	PR_TRANSPORT_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'bcg.fwdproxy.proxyName'
	,'Use no forward proxy'
	,'bcg.fwdproxy.proxyName'
	,'Use no forward proxy'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE IN ('HttpS')
  AND	NOT EXISTS (SELECT 'x'
		FROM	pr_transport_attr
		WHERE	ATTRNAME = 'bcg.fwdproxy.proxyName'
		AND	TRANSPORTID = (SELECT TRANSPORTID
				FROM	pr_transport
				WHERE	transporttype = 'HttpS'
				)
		)
)
/
---55723

UPDATE  pr_transport_attr 
    SET defaultattrvalue = 'No'
    WHERE attrname = 'bcg.gateway.FTPSMODE' AND 
    transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway')
/

UPDATE  pr_transport_attr 
    SET defaultattrvalue = 'No'
    WHERE attrname = 'bcg.receiver.FTPSMODE' AND 
    transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver')
/


---JR22965 103698

INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'Unique FileName', 'bcg.transport.uniqueFileName', 'Yes'
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway') prtrnsprt, dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'Unique FileName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))

/
--- JR24570

INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.JMSPASSWORD', 'bcg.receiver.JMSPASSWORD', ''
   FROM (SELECT transportid  transid FROM pr_transport WHERE resourcekey = 'JMSQueue') prtrnsprt, dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.JMSPASSWORD' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE resourcekey  = 'JMSQueue'))
/

INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.receiver.JMSUSERNAME', 'bcg.receiver.JMSUSERNAME', ''
   FROM (SELECT transportid  transid FROM pr_transport WHERE resourcekey = 'JMSQueue') prtrnsprt, dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.JMSUSERNAME' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE resourcekey  = 'JMSQueue'))
/

INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.JMSPASSWORD', 'bcg.gateway.JMSPASSWORD', ''
   FROM (SELECT transportid  transid FROM pr_transport WHERE resourcekey = 'JMS') prtrnsprt, dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.JMSPASSWORD' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE resourcekey = 'JMS'))
/

INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transid,'bcg.gateway.JMSUSERNAME', 'bcg.gateway.JMSUSERNAME', ''
   FROM (SELECT transportid  transid FROM pr_transport WHERE resourcekey = 'JMS') prtrnsprt, dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.JMSUSERNAME' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE resourcekey = 'JMS'))
/
                        


                        
INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)   
   SELECT pr_transport_attr_seq.NEXTVAL,transid,'bcg.gateway.JMSREPLYTOQUEUENAME', 'bcg.gateway.JMSREPLYTOQUEUENAME' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE resourcekey = 'JMS') prtrnsprt, Dual
   WHERE NOT EXISTS (SELECT 'x'
   		     FROM pr_transport_attr 
		     WHERE attrname = 'bcg.gateway.JMSREPLYTOQUEUENAME' AND 
			transportid = (SELECT transportid FROM pr_transport WHERE resourcekey = 'JMS'))
/
			

INSERT INTO pr_transport_attr( transportattrid,transportid, attrname, defaultattrvalue ,resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,transid,'bcg.ftp.is.retry','No','bcg.ftp.is.retry' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway') prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.ftp.is.retry' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/
                        

-- F104595 start
-- SFTP Receiver attributes start
INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.FTPPollFrequency','bcg.transport.FTPPollFrequency' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.FTPPollFrequency' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.HostName','bcg.transport.HostName' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.HostName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.LocalEventDirectory','bcg.transport.LocalEventDirectory' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.LocalEventDirectory' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.MaximumConnections','bcg.transport.MaximumConnections' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.MaximumConnections' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.MinimumConnections','bcg.transport.MinimumConnections' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.MinimumConnections' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.SFTPServerPollPeriod','bcg.transport.SFTPServerPollPeriod' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.SFTPServerPollPeriod' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.PollQuantity','bcg.transport.PollQuantity' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.PollQuantity' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.PortNumber','bcg.transport.PortNumber' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.PortNumber' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.RetryInterval','bcg.transport.RetryInterval' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.RetryInterval' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.RetryLimit','bcg.transport.RetryLimit' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.RetryLimit' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.EventDirectory','bcg.transport.EventDirectory' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.EventDirectory' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.SocksProxyHost','bcg.transport.SocksProxyHost' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.SocksProxyHost' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.SocksProxyPort','bcg.transport.SocksProxyPort' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.SocksProxyPort' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.SocksProxyPassword','bcg.transport.SocksProxyPassword' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.SocksProxyPassword' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.SocksProxyUserName','bcg.transport.SocksProxyUserName' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.SocksProxyUserName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.EISEncoding','bcg.transport.EISEncoding' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.EISEncoding' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.privateKeyFilePath','bcg.transport.privateKeyFilePath' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.privateKeyFilePath' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.passphrase','bcg.transport.passphrase' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.passphrase' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.EP_TableName','bcg.transport.EP_TableName' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.EP_TableName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.LocalArchiveDirectory','bcg.transport.LocalArchiveDirectory' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.LocalArchiveDirectory' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.activationSpecJNDIName','bcg.transport.activationSpecJNDIName' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.activationSpecJNDIName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.MDBApplicationName','bcg.transport.MDBApplicationName' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.MDBApplicationName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/

-- SFTPReceiver attributes end

-- SFTPDestination attributes start
INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.OutputDirectory','bcg.destination.OutputDirectory' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.OutputDirectory' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.HostName','bcg.destination.HostName' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.HostName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.PortNumber','bcg.destination.PortNumber' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.PortNumber' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.SocksProxyHost','bcg.destination.SocksProxyHost' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.SocksProxyHost' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.SocksProxyPort','bcg.destination.SocksProxyPort' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.SocksProxyPort' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.SocksProxyPassword','bcg.destination.SocksProxyPassword' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.SocksProxyPassword' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.SocksProxyUserName','bcg.destination.SocksProxyUserName' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.SocksProxyUserName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.EISEncoding','bcg.destination.EISEncoding' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.EISEncoding' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.privateKeyFilePath','bcg.destination.privateKeyFilePath' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.privateKeyFilePath' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.passphrase','bcg.destination.passphrase' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.passphrase' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.connectionFactoryJNDIName','bcg.destination.connectionFactoryJNDIName' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.connectionFactoryJNDIName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/


-- SFTPDestination attributes end

-- F104595 end


-- New attributes for authType for receiver and destination has to be added to database metadata.

--107174 start
INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.receiver.authType','bcg.transport.authType' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.authType' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/

INSERT INTO pr_transport_attr(transportattrid,transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL,prtrnsprt.transid,'bcg.destination.authType','bcg.destination.authType' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPSender')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.authType' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/

--107174 end

-- Added for 107224
INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.receiver.ENCODING', 'bcg.transport.ENCODING'
   FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.ENCODING' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver'))
/

--107224 end 

-- Added for 107474
INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'bcg.gateway.ENCODING', 'bcg.transport.ENCODING'
   FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.gateway.ENCODING' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway'))
/

--107474 end   

--Added for 107222 


-- insert the enableServerVerification Attribute

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, prtrnsprt.transid,'bcg.receiver.enableServerVerification','bcg.transport.enableServerVerification' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.enableServerVerification' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/

-- Insert the host key file

INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, prtrnsprt.transid,'bcg.receiver.hostKeyFile','bcg.transport.hostKeyFile' 
   FROM (SELECT transportid  transid FROM pr_transport WHERE transporttype = 'SFTPReceiver')  prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.receiver.hostKeyFile' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPReceiver'))
/

-- END 107224
-- Desh IFIX for SFTP
INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, prtrnsprt.transid,'bcg.destination.enableServerVerification','bcg.destination.enableServerVerification' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'SFTPSender') prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.enableServerVerification' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/
INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey)
   SELECT pr_transport_attr_seq.NEXTVAL, prtrnsprt.transid,'bcg.destination.hostKeyFile','bcg.destination.hostKeyFile' 
   FROM (SELECT transportid transid FROM pr_transport WHERE transporttype = 'SFTPSender') prtrnsprt, DUAL
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'bcg.destination.hostKeyFile' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/
-- End IFIX
-- Start 108414
INSERT INTO pr_transport_attr(transportattrid, transportid, attrname, resourcekey, defaultattrvalue)
   SELECT pr_transport_attr_seq.NEXTVAL, transportid,'Unique FileName', 'bcg.transport.uniqueFileName', 'Yes'
   FROM pr_transport WHERE transporttype = 'SFTPSender'
   AND NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_attr 
                     WHERE attrname = 'Unique FileName' AND
                        transportid = (SELECT transportid FROM pr_transport WHERE transporttype = 'SFTPSender'))
/
-- END 108414

-- CF_PROFILETYPE.sql

Insert Into CF_PROFILETYPE(PROFILETYPEID, PROFILETYPENAME, RESOURCEKEY ) 
   SELECT  1,'EDI','bcg.edi.profiletype.connectionprofile'  FROM dual
   Where Not Exists (select 'x' from CF_PROFILETYPE where PROFILETYPENAME = 'EDI')
/
Insert Into CF_PROFILETYPE(PROFILETYPEID, PROFILETYPENAME, RESOURCEKEY ) 
   SELECT  2,'X12','bcg.edi.profiletype.x12'  FROM dual
   Where Not Exists (select 'x' from CF_PROFILETYPE where PROFILETYPENAME = 'X12')
/ 
Insert Into CF_PROFILETYPE(PROFILETYPEID, PROFILETYPENAME, RESOURCEKEY ) 
   SELECT  3,'UCS','bcg.edi.profiletype.ucs'  FROM dual
   Where Not Exists (select 'x' from CF_PROFILETYPE where PROFILETYPENAME = 'UCS')
/
Insert Into CF_PROFILETYPE(PROFILETYPEID, PROFILETYPENAME, RESOURCEKEY ) 
   SELECT  4,'EDIFACT','bcg.edi.profiletype.edifact'  FROM dual
   Where Not Exists (select 'x' from CF_PROFILETYPE where PROFILETYPENAME = 'EDIFACT')
/
--UPDATE CF_PROFILETYPE
--SET	profiletypename = 'EDI'
--WHERE	profiletypename = 'CONNECTION'
--/
Insert Into CF_PROFILETYPE(PROFILETYPEID, PROFILETYPENAME, RESOURCEKEY ) 
   SELECT  5,'ebMS','bcg.ebms.profiletype.connection'
  FROM dual
   Where Not Exists (select 'x' from CF_PROFILETYPE where PROFILETYPENAME = 'ebMS')
/


-- CF_PROFILEATTR_GRP.sql

-- bcg.grp.edi.env.GROUP
INSERT INTO CF_PROFILEATTR_GRP (
   PROFILEATTR_GRPID, PROFILEATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.edi.env.GROUP'
		,130
		,'bcg.grp.edi.env.GROUP'
		,'bcg.grp.edi.env.GROUP'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_PROFILEATTR_GRP WHERE PROFILEATTR_GRPKEY = 'bcg.grp.edi.env.GROUP')
	)
/

-- bcg.grp.edi.env.INTERCHANGE
INSERT INTO CF_PROFILEATTR_GRP (
   PROFILEATTR_GRPID, PROFILEATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.edi.env.INTERCHANGE'
		,120
		,'bcg.grp.edi.env.INTERCHANGE'
		,'bcg.grp.edi.env.INTERCHANGE'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_PROFILEATTR_GRP WHERE PROFILEATTR_GRPKEY = 'bcg.grp.edi.env.INTERCHANGE')
	)
/

-- bcg.grp.edi.env.TRANSACTION
INSERT INTO CF_PROFILEATTR_GRP (
   PROFILEATTR_GRPID, PROFILEATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.edi.env.TRANSACTION'
		,140
		,'bcg.grp.edi.env.TRANSACTION'
		,'bcg.grp.edi.env.TRANSACTION'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_PROFILEATTR_GRP WHERE PROFILEATTR_GRPKEY = 'bcg.grp.edi.env.TRANSACTION')
	)
/

-- bcg.grp.edi.env.MESSAGE
INSERT INTO CF_PROFILEATTR_GRP (
   PROFILEATTR_GRPID, PROFILEATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.edi.env.MESSAGE'
		,110
		,'bcg.grp.edi.env.MESSAGE'
		,'bcg.grp.edi.env.MESSAGE'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_PROFILEATTR_GRP WHERE PROFILEATTR_GRPKEY = 'bcg.grp.edi.env.MESSAGE')
	)
/

-- bcg.grp.edi.env.GENERAL
INSERT INTO CF_PROFILEATTR_GRP (
   PROFILEATTR_GRPID, PROFILEATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.edi.env.GENERAL'
		,100
		,'bcg.grp.edi.env.GENERAL'
		,'bcg.grp.edi.env.GENERAL'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_PROFILEATTR_GRP WHERE PROFILEATTR_GRPKEY = 'bcg.grp.edi.env.GENERAL')
	)
/

-- bcg.grp.ebms.GENERAL
INSERT INTO CF_PROFILEATTR_GRP (PROFILEATTR_GRPID, PROFILEATTR_GRPKEY, ORDERING, RESOURCEKEY, DESCRIPTION) 
	(SELECT	cf_profileattr_grp_seq.NEXTVAL
		,'bcg.grp.ebms.GENERAL'
		,200
		,'bcg.grp.ebms.GENERAL'
		,'bcg.grp.ebms.GENERAL'
	FROM	DUAL
	WHERE	Not Exists (SELECT 'x' FROm CF_PROFILEATTR_GRP WHERE PROFILEATTR_GRPKEY = 'bcg.grp.ebms.GENERAL')
	)
/


-- CF_PROFILEATTR.sql

---1
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 1,1,255,'bcg.connection.profile.edi.qualifier1','bcg.connection.profile.edi.qualifier1'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.connection.profile.edi.qualifier1' 
AND PROFILETYPEID = 1)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 1,1,255,'bcg.connection.profile.edi.usage','bcg.connection.profile.edi.usage'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.connection.profile.edi.usage' 
AND PROFILETYPEID = 1)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 1,1,255,'bcg.connection.profile.edi.appsenderid','bcg.connection.profile.edi.appsenderid'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.connection.profile.edi.appsenderid' 
AND PROFILETYPEID = 1)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 1,1,255,'bcg.connection.profile.edi.appreceiverid','bcg.connection.profile.edi.appreceiverid'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.connection.profile.edi.appreceiverid' 
AND PROFILETYPEID = 1)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 1,1,255,'bcg.connection.profile.edi.password','bcg.connection.profile.edi.password'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.connection.profile.edi.password' 
AND PROFILETYPEID = 1)
/
---2
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,2,'bcg.envelope.profile.edi.ISA01','bcg.envelope.profile.edi.ISA01'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA01' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,10,'bcg.envelope.profile.edi.ISA02','bcg.envelope.profile.edi.ISA02'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA02' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,2,'bcg.envelope.profile.edi.ISA03','bcg.envelope.profile.edi.ISA03'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA03' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,10,'bcg.envelope.profile.edi.ISA04','bcg.envelope.profile.edi.ISA04'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA04' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,2,'bcg.envelope.profile.edi.ISA05','bcg.envelope.profile.edi.ISA05'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA05' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,15,'bcg.envelope.profile.edi.ISA06','bcg.envelope.profile.edi.ISA06'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA06' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,2,'bcg.envelope.profile.edi.ISA07','bcg.envelope.profile.edi.ISA07'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA07' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,15,'bcg.envelope.profile.edi.ISA08','bcg.envelope.profile.edi.ISA08'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA08' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.ISA09','bcg.envelope.profile.edi.ISA09'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA09' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.ISA10','bcg.envelope.profile.edi.ISA10'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA10' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,1,'bcg.envelope.profile.edi.ISA11','bcg.envelope.profile.edi.ISA11'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA11' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,5,'bcg.envelope.profile.edi.ISA12','bcg.envelope.profile.edi.ISA12'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA12' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.ISA13','bcg.envelope.profile.edi.ISA13'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA13' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,1,'bcg.envelope.profile.edi.ISA14','bcg.envelope.profile.edi.ISA14'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA14' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,1,'bcg.envelope.profile.edi.ISA15','bcg.envelope.profile.edi.ISA15'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA15' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.ISA16','bcg.envelope.profile.edi.ISA16'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ISA16' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,2,'bcg.envelope.profile.edi.GS01','bcg.envelope.profile.edi.GS01'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS01' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,15,'bcg.envelope.profile.edi.GS02','bcg.envelope.profile.edi.GS02'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS02' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,15,'bcg.envelope.profile.edi.GS03','bcg.envelope.profile.edi.GS03'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS03' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.GS04','bcg.envelope.profile.edi.GS04'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS04' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.GS05','bcg.envelope.profile.edi.GS05'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS05' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.GS06','bcg.envelope.profile.edi.GS06'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS06' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,2,'bcg.envelope.profile.edi.GS07','bcg.envelope.profile.edi.GS07'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS07' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,12,'bcg.envelope.profile.edi.GS08','bcg.envelope.profile.edi.GS08'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS08' 
AND PROFILETYPEID = 2)
/
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.ST01','bcg.envelope.profile.edi.ST01'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ST01' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.ST02','bcg.envelope.profile.edi.ST02'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ST02' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,35,'bcg.envelope.profile.edi.ST03','bcg.envelope.profile.edi.ST03'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ST03' 
AND PROFILETYPEID = 2)
/
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,255,'bcg.envelope.profile.edi.INTCTLLEN','bcg.envelope.profile.edi.INTCTLLEN','9'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.INTCTLLEN' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,255,'bcg.envelope.profile.edi.GRPCTLLEN','bcg.envelope.profile.edi.GRPCTLLEN','9'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GRPCTLLEN' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,255,'bcg.envelope.profile.edi.TRXCTLLEN','bcg.envelope.profile.edi.TRXCTLLEN','9'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.TRXCTLLEN' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,0,255,'bcg.envelope.profile.edi.ENVTYPE','bcg.envelope.profile.edi.ENVTYPE'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ENVTYPE' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,255,'bcg.envelope.profile.edi.EDIFACTGRP','bcg.envelope.profile.edi.EDIFACTGRP','N' 
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.EDIFACTGRP' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,255,'bcg.envelope.profile.edi.MAXDOCS','bcg.envelope.profile.edi.MAXDOCS'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.MAXDOCS' 
AND PROFILETYPEID = 2)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 2,1,255,'bcg.envelope.profile.edi.CTLNUMFLAG','bcg.envelope.profile.edi.CTLNUMFLAG' ,'N' 
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.CTLNUMFLAG' 
AND PROFILETYPEID = 2)
/  
---3
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,2,'bcg.envelope.profile.edi.GS01','bcg.envelope.profile.edi.GS01'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS01' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,15,'bcg.envelope.profile.edi.GS02','bcg.envelope.profile.edi.GS02'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS02' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,15,'bcg.envelope.profile.edi.GS03','bcg.envelope.profile.edi.GS03'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS03' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,255,'bcg.envelope.profile.edi.GS04','bcg.envelope.profile.edi.GS04'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS04' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,255,'bcg.envelope.profile.edi.GS05','bcg.envelope.profile.edi.GS05'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS05' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,255,'bcg.envelope.profile.edi.GS06','bcg.envelope.profile.edi.GS06'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS06' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,2,'bcg.envelope.profile.edi.GS07','bcg.envelope.profile.edi.GS07'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS07' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,12,'bcg.envelope.profile.edi.GS08','bcg.envelope.profile.edi.GS08'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GS08' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,255,'bcg.envelope.profile.edi.ST01','bcg.envelope.profile.edi.ST01'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ST01' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,255,'bcg.envelope.profile.edi.ST02','bcg.envelope.profile.edi.ST02'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ST02' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,35,'bcg.envelope.profile.edi.ST03','bcg.envelope.profile.edi.ST03'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ST03' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,10,'bcg.envelope.profile.edi.BG01','bcg.envelope.profile.edi.BG01'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.BG01' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,10,'bcg.envelope.profile.edi.BG02','bcg.envelope.profile.edi.BG02'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.BG02' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,15,'bcg.envelope.profile.edi.BG03','bcg.envelope.profile.edi.BG03'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.BG03' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,15,'bcg.envelope.profile.edi.BG04','bcg.envelope.profile.edi.BG04'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.BG04' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,255,'bcg.envelope.profile.edi.BG05','bcg.envelope.profile.edi.BG05'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.BG05' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,255,'bcg.envelope.profile.edi.BG06','bcg.envelope.profile.edi.BG06'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.BG06' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,255,'bcg.envelope.profile.edi.BG07','bcg.envelope.profile.edi.BG07'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.BG07' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,255,'bcg.envelope.profile.edi.INTCTLLEN','bcg.envelope.profile.edi.INTCTLLEN','5'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.INTCTLLEN' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,255,'bcg.envelope.profile.edi.GRPCTLLEN','bcg.envelope.profile.edi.GRPCTLLEN' ,'9' 
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GRPCTLLEN' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ,DEFAULTATTRVALUE) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,255,'bcg.envelope.profile.edi.TRXCTLLEN','bcg.envelope.profile.edi.TRXCTLLEN','9' 
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.TRXCTLLEN' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,0,255,'bcg.envelope.profile.edi.ENVTYPE','bcg.envelope.profile.edi.ENVTYPE'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ENVTYPE' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,255,'bcg.envelope.profile.edi.EDIFACTGRP','bcg.envelope.profile.edi.EDIFACTGRP','N'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.EDIFACTGRP' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,255,'bcg.envelope.profile.edi.MAXDOCS','bcg.envelope.profile.edi.MAXDOCS'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.MAXDOCS' 
AND PROFILETYPEID = 3)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ,DEFAULTATTRVALUE) 
SELECT CF_PROFILEATTR_SEQ.nextval , 3,1,255,'bcg.envelope.profile.edi.CTLNUMFLAG','bcg.envelope.profile.edi.CTLNUMFLAG' ,'N' 
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.CTLNUMFLAG' 
AND PROFILETYPEID = 3)
/  
---4
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,255,'bcg.envelope.profile.edi.INTCTLLEN','bcg.envelope.profile.edi.INTCTLLEN','9'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.INTCTLLEN' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,255,'bcg.envelope.profile.edi.GRPCTLLEN','bcg.envelope.profile.edi.GRPCTLLEN','9'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.GRPCTLLEN' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,255,'bcg.envelope.profile.edi.TRXCTLLEN','bcg.envelope.profile.edi.TRXCTLLEN','9' 
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.TRXCTLLEN' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,255,'bcg.envelope.profile.edi.ENVTYPE','bcg.envelope.profile.edi.ENVTYPE'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.ENVTYPE' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,255,'bcg.envelope.profile.edi.EDIFACTGRP','bcg.envelope.profile.edi.EDIFACTGRP' ,'N' 
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.EDIFACTGRP' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,255,'bcg.envelope.profile.edi.MAXDOCS','bcg.envelope.profile.edi.MAXDOCS'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.MAXDOCS' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,255,'bcg.envelope.profile.edi.CTLNUMFLAG','bcg.envelope.profile.edi.CTLNUMFLAG','N'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.CTLNUMFLAG' 
AND PROFILETYPEID = 4)
/
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,4,'bcg.envelope.profile.edi.UNB0101','bcg.envelope.profile.edi.UNB0101'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0101' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,1,'bcg.envelope.profile.edi.UNB0102','bcg.envelope.profile.edi.UNB0102'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0102' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,35,'bcg.envelope.profile.edi.UNB0201','bcg.envelope.profile.edi.UNB0201'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0201' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,4,'bcg.envelope.profile.edi.UNB0202','bcg.envelope.profile.edi.UNB0202'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0202' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,14,'bcg.envelope.profile.edi.UNB0203','bcg.envelope.profile.edi.UNB0203'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0203' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,35,'bcg.envelope.profile.edi.UNB0301','bcg.envelope.profile.edi.UNB0301'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0301' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,4,'bcg.envelope.profile.edi.UNB0302','bcg.envelope.profile.edi.UNB0302'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0302' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,14,'bcg.envelope.profile.edi.UNB0303','bcg.envelope.profile.edi.UNB0303'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0303' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,255,'bcg.envelope.profile.edi.UNB0401','bcg.envelope.profile.edi.UNB0401'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0401' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,255,'bcg.envelope.profile.edi.UNB0402','bcg.envelope.profile.edi.UNB0402'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0402' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,255,'bcg.envelope.profile.edi.UNB05','bcg.envelope.profile.edi.UNB05'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB05' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,14,'bcg.envelope.profile.edi.UNB0601','bcg.envelope.profile.edi.UNB0601'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0601' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,2,'bcg.envelope.profile.edi.UNB0602','bcg.envelope.profile.edi.UNB0602'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB0602' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,14,'bcg.envelope.profile.edi.UNB07','bcg.envelope.profile.edi.UNB07'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB07' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,1,'bcg.envelope.profile.edi.UNB08','bcg.envelope.profile.edi.UNB08'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB08' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,1,'bcg.envelope.profile.edi.UNB09','bcg.envelope.profile.edi.UNB09'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB09' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,35,'bcg.envelope.profile.edi.UNB10','bcg.envelope.profile.edi.UNB10'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB10' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,1,'bcg.envelope.profile.edi.UNB11','bcg.envelope.profile.edi.UNB11'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNB11' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,6,'bcg.envelope.profile.edi.UNG01','bcg.envelope.profile.edi.UNG01'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG01' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,35,'bcg.envelope.profile.edi.UNG0201','bcg.envelope.profile.edi.UNG0201'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG0201' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,4,'bcg.envelope.profile.edi.UNG0202','bcg.envelope.profile.edi.UNG0202'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG0202' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,35,'bcg.envelope.profile.edi.UNG0301','bcg.envelope.profile.edi.UNG0301'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG0301' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,4,'bcg.envelope.profile.edi.UNG0302','bcg.envelope.profile.edi.UNG0302'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG0302' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,255,'bcg.envelope.profile.edi.UNG0401','bcg.envelope.profile.edi.UNG0401'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG0401' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,255,'bcg.envelope.profile.edi.UNG0402','bcg.envelope.profile.edi.UNG0402'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG0402' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,255,'bcg.envelope.profile.edi.UNG05','bcg.envelope.profile.edi.UNG05'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG05' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,2,'bcg.envelope.profile.edi.UNG06','bcg.envelope.profile.edi.UNG06'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG06' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,3,'bcg.envelope.profile.edi.UNG0701','bcg.envelope.profile.edi.UNG0701'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG0701' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,3,'bcg.envelope.profile.edi.UNG0702','bcg.envelope.profile.edi.UNG0702'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG0702' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,5,'bcg.envelope.profile.edi.UNG0703','bcg.envelope.profile.edi.UNG0703'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG0703' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,14,'bcg.envelope.profile.edi.UNG08','bcg.envelope.profile.edi.UNG08'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNG08' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,255,'bcg.envelope.profile.edi.UNH01','bcg.envelope.profile.edi.UNH01'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNH01' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,6,'bcg.envelope.profile.edi.UNH0201','bcg.envelope.profile.edi.UNH0201'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNH0201' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY, DEFAULTATTRVALUE  ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,3,'bcg.envelope.profile.edi.UNH0202','bcg.envelope.profile.edi.UNH0202', 'D'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNH0202' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,3,'bcg.envelope.profile.edi.UNH0203','bcg.envelope.profile.edi.UNH0203'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNH0203' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ,DEFAULTATTRVALUE ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,2,'bcg.envelope.profile.edi.UNH0204','bcg.envelope.profile.edi.UNH0204' ,'UN' 
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNH0204' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,6,'bcg.envelope.profile.edi.UNH0205','bcg.envelope.profile.edi.UNH0205'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNH0205' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,1,35,'bcg.envelope.profile.edi.UNH03','bcg.envelope.profile.edi.UNH03'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNH03' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,2,'bcg.envelope.profile.edi.UNH0401','bcg.envelope.profile.edi.UNH0401'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNH0401' 
AND PROFILETYPEID = 4)
/  
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
SELECT CF_PROFILEATTR_SEQ.nextval , 4,0,1,'bcg.envelope.profile.edi.UNH0402','bcg.envelope.profile.edi.UNH0402'  
FROM dual
Where Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.envelope.profile.edi.UNH0402' 
AND PROFILETYPEID = 4)
/ 
-- CR60181 start
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.connection.profile.edi.qualifier1'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.connection.profile.edi.usage'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.connection.profile.edi.appsenderid'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.connection.profile.edi.appreceiverid'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.connection.profile.edi.password'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA01'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA02'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA03'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA04'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA05'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA06'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA07'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA08'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA09'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA10'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA11'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA12'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA13'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA14'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA15'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.ISA16'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS01'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS02'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS03'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS04'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS05'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS06'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS07'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS08'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.TRANSACTION')
WHERE	attrname = 'bcg.envelope.profile.edi.ST01'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.TRANSACTION')
WHERE	attrname = 'bcg.envelope.profile.edi.ST02'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.TRANSACTION')
WHERE	attrname = 'bcg.envelope.profile.edi.ST03'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.INTCTLLEN'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.GRPCTLLEN'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.TRXCTLLEN'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.ENVTYPE'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.EDIFACTGRP'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.MAXDOCS'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.CTLNUMFLAG'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS01'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS02'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS03'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS04'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS05'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS06'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS07'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.GS08'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.TRANSACTION')
WHERE	attrname = 'bcg.envelope.profile.edi.ST01'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.TRANSACTION')
WHERE	attrname = 'bcg.envelope.profile.edi.ST02'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.TRANSACTION')
WHERE	attrname = 'bcg.envelope.profile.edi.ST03'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.BG01'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.BG02'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.BG03'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.BG04'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.BG05'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.BG06'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.BG07'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.INTCTLLEN'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.GRPCTLLEN'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.TRXCTLLEN'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.ENVTYPE'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.EDIFACTGRP'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.MAXDOCS'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.CTLNUMFLAG'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.INTCTLLEN'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.GRPCTLLEN'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.TRXCTLLEN'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.ENVTYPE'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.EDIFACTGRP'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.MAXDOCS'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GENERAL')
WHERE	attrname = 'bcg.envelope.profile.edi.CTLNUMFLAG'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0101'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0102'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0201'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0202'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0203'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0301'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0302'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0303'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0401'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0402'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB05'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0601'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB0602'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB07'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB08'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB09'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB10'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.INTERCHANGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNB11'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG01'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG0201'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG0202'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG0301'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG0302'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG0401'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG0402'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG05'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG06'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG0701'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG0702'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG0703'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.GROUP')
WHERE	attrname = 'bcg.envelope.profile.edi.UNG08'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.MESSAGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNH01'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.MESSAGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNH0201'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.MESSAGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNH0202'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.MESSAGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNH0203'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.MESSAGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNH0204'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.MESSAGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNH0205'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.MESSAGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNH03'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.MESSAGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNH0401'
/
UPDATE cf_profileattr
SET	profileattr_grpid = (SELECT profileattr_grpid 
				FROM cf_profileattr_grp 
				WHERE profileattr_grpkey = 'bcg.grp.edi.env.MESSAGE')
WHERE	attrname = 'bcg.envelope.profile.edi.UNH0402'
/
-- CR60181 end
Insert Into CF_PROFILEATTR(PROFILEATTRID, PROFILETYPEID,PROFILEATTR_GRPID, EDITFLAG, MAXLEN, ATTRNAME, RESOURCEKEY ) 
	SELECT CF_PROFILEATTR_SEQ.nextval 
		,pt.ProfileTypeID
		,g.ProfileAttr_GrpId
		,1
		,14
		,'bcg.ebms.profileattr.responsetype'
		,'bcg.ebms.profileattr.responsetype'
	FROM 	cf_profiletype pt, cf_profileattr_grp g
	WHERE	pt.PROFILETYPENAME = 'ebMS'
	AND	g.PROFILEATTR_GRPKEY = 'bcg.grp.ebms.GENERAL'
	AND Not Exists (select 'x' from CF_PROFILEATTR where ATTRNAME = 'bcg.ebms.profileattr.responsetype' AND profiletypeid = pt.profiletypeid)
/


-- PR_TRANSPORT_GLOBAL_ATTR.sql

/* INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.target.transport.https.synctimeout', '300000' ,
         'bcg.target.transport.https.synctimeout'
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'HttpS')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.target.transport.https.synctimeout')
/

INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.target.transport.https.maxsyncconnections', '100' ,
         'bcg.target.transport.https.maxsyncconnections'
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'HttpS')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.target.transport.https.maxsyncconnections')
/
*/                        

INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.target.transport.ftpscripting.maxqueueage', '740000' ,
         'bcg.target.transport.ftpscripting.maxqueueage'
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.target.transport.ftpscripting.maxqueueage')
/

INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.target.transport.ftpscripting.lockretries', '3' ,
         'bcg.target.transport.ftpscripting.lockretries'
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.target.transport.ftpscripting.lockretries')
/

INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.target.transport.ftpscripting.lockretryinterval', '260000' ,
         'bcg.target.transport.ftpscripting.lockretryinterval'
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.target.transport.ftpscripting.lockretryinterval')
/

INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.target.transport.ftpscripting.maxlockholdtime', '240000' ,
         'bcg.target.transport.ftpscripting.maxlockholdtime'
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'FTPScriptingReceiver')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.target.transport.ftpscripting.maxlockholdtime')
/

INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.gateway.transport.ftpscripting.maxqueueage', '740000' ,
         'bcg.gateway.transport.ftpscripting.maxqueueage'
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.gateway.transport.ftpscripting.maxqueueage')
/

INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.gateway.transport.ftpscripting.lockretries', '3' ,
         'bcg.gateway.transport.ftpscripting.lockretries'
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.gateway.transport.ftpscripting.lockretries')
/

INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.gateway.transport.ftpscripting.lockretryinterval', '260000' ,
         'bcg.gateway.transport.ftpscripting.lockretryinterval'      
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.gateway.transport.ftpscripting.lockretryinterval')
/

INSERT INTO pr_transport_global_attr(transportglobalattrid, transportid, 
                                     attrname, attrvalue, resourcekey)
   SELECT pr_transport_global_attr_seq.NEXTVAL, from_transportid, 
         'bcg.gateway.transport.ftpscripting.maxlockholdtime', '240000' ,
         'bcg.gateway.transport.ftpscripting.maxlockholdtime'
   FROM dual, (SELECT transportid from_transportid FROM pr_transport WHERE transporttype = 'FTPScriptingGateway')
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_transport_global_attr 
                     WHERE transportid = from_transportid
                        AND attrname = 'bcg.gateway.transport.ftpscripting.maxlockholdtime')
/


-- PR_HANDLERATTR.sql


--Generic Document Flow Handler Attributes   
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromPackagingName', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromPackagingName'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromPackagingVersion', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromPackagingVersion'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromProtocolName', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromProtocolName'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromProtocolVersion', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromProtocolVersion'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

--54530 changed handlerattrname from FromProcessCode to FromDocumentType
--47952 changed handlerattrname back
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromProcessCode', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromProcessCode'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

--54530 changed attrname from FromProcessVersion to FromDocumentFlowVersion
--47952 changed handlerattrname back
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromProcessVersion', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromProcessVersion'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'METADICTIONARY', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'METADICTIONARY'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'METADOCUMENT', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'METADOCUMENT'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'METASYNTAX', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'METASYNTAX'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'ENCODING', 'ascii'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'ENCODING'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'BCG_BATCHDOCS', 'ON'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'BCG_BATCHDOCS'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/

--ROD Splitter Attributes   
--com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromPackagingName', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromPackagingName'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromPackagingVersion', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromPackagingVersion'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromProtocolName', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromProtocolName'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromProtocolVersion', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromProtocolVersion'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

--54530 changed handlerattrname from FromProcessCode to FromDocumentType
--47952 changed handlerattrname back
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromProcessCode', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromProcessCode'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

--54530 changed attrname from FromProcessVersion to FromDocumentFlowVersion
--47952 changed handlerattrname back
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'FromProcessVersion', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'FromProcessVersion'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'METADICTIONARY', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'METADICTIONARY'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'METADOCUMENT', NULL
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'METADOCUMENT'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

--54997 added 'rod' as default value
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'METASYNTAX', 'rod'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'METASYNTAX'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'ENCODING', 'ascii'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'ENCODING'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'BCG_BATCHDOCS', 'ON'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'BCG_BATCHDOCS'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.RODSplitterHandler'))
/

--XML Splitter Attributes
--com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'BCG_BATCHDOCS', 'ON'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'BCG_BATCHDOCS'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.XMLSplitterHandler'))
/

--EDI Splitter Attributes
--com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'ENCODING', 'ascii'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'ENCODING'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.receiver.preprocesshandler.EDISplitterHandler'))
/

--55246

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'Content-Types', 'application/binary,application/octet-stream'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.server.BinaryChannelParseHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'Content-Types'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.server.BinaryChannelParseHandler'))
/                                       
                                       
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'Content-Types', 'application/xml,text/xml'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.cxml.cXMLChannelParseHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'Content-Types'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.cxml.cXMLChannelParseHandler'))
/                                       
                                      
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'Content-Types', 'application/xml,text/xml'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.server.XMLRouterBizProcessHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'Content-Types'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.server.XMLRouterBizProcessHandler'))
/

INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname, defaultattrvalue)
SELECT pr_handlerattr_seq.nextval, handlerid, 'Content-Types', 'application/EDI-X12,application/EDI-CONSENT,application/EDIFACT'
   FROM (SELECT handlerid 
         FROM pr_handler
         WHERE classname = 'com.ibm.bcg.edi.EDIRouterBizProcessHandler'), 
   dual
   WHERE NOT EXISTS (SELECT 'x' 
                     FROM pr_handlerattr
                     WHERE attrname = 'Content-Types'
                     AND handlerid = (SELECT handlerid hid
                                      FROM pr_handler
                                      WHERE classname = 
                                       'com.ibm.bcg.edi.EDIRouterBizProcessHandler'))
/
--100624
INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname)
 SELECT pr_handlerattr_seq.nextval, handlerid, 'SenderId'
    FROM (SELECT handlerid
          FROM pr_handler
          WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') hid,
    dual
    WHERE NOT EXISTS (SELECT 'x'
                      FROM pr_handlerattr
                      WHERE attrname = 'SenderId'
                      AND handlerid = (SELECT handlerid
                                       FROM pr_handler
                                       WHERE classname =
                                        'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/
  INSERT INTO pr_handlerattr (handlerattrid, handlerid, attrname)
  SELECT pr_handlerattr_seq.nextval, handlerid, 'ReceiverId'
     FROM (SELECT handlerid
           FROM pr_handler
           WHERE classname = 'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler') hid,
     dual
     WHERE NOT EXISTS (SELECT 'x'
                       FROM pr_handlerattr
                       WHERE attrname = 'ReceiverId'
                       AND handlerid = (SELECT handlerid
                                        FROM pr_handler
                                        WHERE classname =
                                         'com.ibm.bcg.edi.receiver.preprocesshandler.GenericDocumentFlowHandler'))
/


INSERT INTO pr_handlerattr (handlerattrid,handlerid, attrname)
  SELECT pr_handlerattr_seq.nextval, handlerid, 'WTX_Map_Root' 
     FROM (SELECT handlerid
           FROM pr_handler
           WHERE classname = 'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory')  hid,
     Dual
     WHERE NOT EXISTS (SELECT 'x'
                       FROM pr_handlerattr
                       WHERE attrname = 'WTX_Map_Root'
                       AND handlerid = (SELECT handlerid
                                        FROM pr_handler
                                        WHERE classname =
                                         'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory'))
/

INSERT INTO pr_handlerattr (handlerattrid,handlerid, attrname, defaultattrvalue)
  SELECT pr_handlerattr_seq.nextval, handlerid, 'WTX_RMIServer_Use_Server', 'no'
     FROM (SELECT handlerid
           FROM pr_handler
           WHERE classname = 'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory')  hid,
     Dual
     WHERE NOT EXISTS (SELECT 'x'
                       FROM pr_handlerattr
                       WHERE attrname = 'WTX_RMIServer_Use_Server'
                       AND handlerid = (SELECT handlerid
                                        FROM pr_handler
                                        WHERE classname =
                                         'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory'))
/

INSERT INTO pr_handlerattr (handlerattrid,handlerid, attrname, defaultattrvalue)
  SELECT pr_handlerattr_seq.nextval, handlerid, 'WTX_RMIServer_Hostname', 'localhost'
     FROM (SELECT handlerid
           FROM pr_handler
           WHERE classname = 'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory')  hid,
     Dual
     WHERE NOT EXISTS (SELECT 'x'
                       FROM pr_handlerattr
                       WHERE attrname = 'WTX_RMIServer_Hostname'
                       AND handlerid = (SELECT handlerid
                                        FROM pr_handler
                                        WHERE classname =
                                         'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory'))
/

INSERT INTO pr_handlerattr (handlerattrid,handlerid, attrname, defaultattrvalue)
  SELECT pr_handlerattr_seq.nextval, handlerid, 'WTX_RMIServer_Port', '5000'
     FROM (SELECT handlerid
           FROM pr_handler
           WHERE classname = 'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory')  hid,
     Dual
     WHERE NOT EXISTS (SELECT 'x'
                       FROM pr_handlerattr
                       WHERE attrname = 'WTX_RMIServer_Port'
                       AND handlerid = (SELECT handlerid
                                        FROM pr_handler
                                        WHERE classname =
                                         'com.ibm.bcg.wtx.business.process.EDIInterchangeValidationFactory'))
/


 INSERT INTO pr_handlerattr (handlerattrid,handlerid, attrname)
  SELECT pr_handlerattr_seq.nextval, handlerid, 'WTX_Map_Root' 
     FROM (SELECT handlerid
           FROM pr_handler
           WHERE classname = 'com.ibm.bcg.wtx.business.process.WTXTransformationFactory')  hid,
     Dual
     WHERE NOT EXISTS (SELECT 'x'
                       FROM pr_handlerattr
                       WHERE attrname = 'WTX_Map_Root'
                       AND handlerid = (SELECT handlerid
                                        FROM pr_handler
                                        WHERE classname =
                                         'com.ibm.bcg.wtx.business.process.WTXTransformationFactory'))
/

INSERT INTO pr_handlerattr (handlerattrid,handlerid, attrname, defaultattrvalue)
  SELECT pr_handlerattr_seq.nextval, handlerid, 'WTX_RMIServer_Use_Server', 'no'
     FROM (SELECT handlerid
           FROM pr_handler
           WHERE classname = 'com.ibm.bcg.wtx.business.process.WTXTransformationFactory')  hid,
     Dual
     WHERE NOT EXISTS (SELECT 'x'
                       FROM pr_handlerattr
                       WHERE attrname = 'WTX_RMIServer_Use_Server'
                       AND handlerid = (SELECT handlerid
                                        FROM pr_handler
                                        WHERE classname =
                                         'com.ibm.bcg.wtx.business.process.WTXTransformationFactory'))
/

INSERT INTO pr_handlerattr (handlerattrid,handlerid, attrname, defaultattrvalue)
  SELECT pr_handlerattr_seq.nextval, handlerid, 'WTX_RMIServer_Hostname', 'localhost'
     FROM (SELECT handlerid
           FROM pr_handler
           WHERE classname = 'com.ibm.bcg.wtx.business.process.WTXTransformationFactory')  hid,
     Dual
     WHERE NOT EXISTS (SELECT 'x'
                       FROM pr_handlerattr
                       WHERE attrname = 'WTX_RMIServer_Hostname'
                       AND handlerid = (SELECT handlerid
                                        FROM pr_handler
                                        WHERE classname =
                                         'com.ibm.bcg.wtx.business.process.WTXTransformationFactory'))
/

INSERT INTO pr_handlerattr (handlerattrid,handlerid, attrname, defaultattrvalue)
  SELECT pr_handlerattr_seq.nextval, handlerid, 'WTX_RMIServer_Port', '5000'
     FROM (SELECT handlerid
           FROM pr_handler
           WHERE classname = 'com.ibm.bcg.wtx.business.process.WTXTransformationFactory')  hid,
     Dual
     WHERE NOT EXISTS (SELECT 'x'
                       FROM pr_handlerattr
                       WHERE attrname = 'WTX_RMIServer_Port'
                       AND handlerid = (SELECT handlerid
                                        FROM pr_handler
                                        WHERE classname =
                                         'com.ibm.bcg.wtx.business.process.WTXTransformationFactory'))
/


-- lg_EventCd.sql

---COMMENT ON COLUMN LG_EVENTCD.SEVERITY IS 'Severity Indicator: 0 = Debug, 1 = Information, 3 = Warning, 6 = Error, 9 = Critical'
--
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '101000', 'BCG101000',  'Password Under Minimum Length'
        , 'The password provided does not meet minimum length requirement as defined in the password policy'
        , 'event.name.101000', 'event.text.101000'
        , 7, 3, 1
        , 'This event is generated when the user enters a password that does not meet the minimum length specified in the password policy.  Reference the password policy.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 101000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '101001', 'BCG101001',  'Password Already Exists'
        , 'The password supplied has already been used by this user'
        , 'event.name.101001', 'event.text.101001'
        , 7, 3, 0
        , 'This event is generated when the user tries to update the user profile using a password that exists in the users password history list.  Reference the  password policy.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 101001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '101002', 'BCG101002', 'Participant Name Already Exists'
        , 'An attempt was made to create a duplicate participant name: {0}'
        , 'event.name.101002', 'event.text.101002'
        , 1, 3, 0
        , 'This event is generated when the user tries to create a participant using the same name as an existing participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 101002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '101003', 'BCG101003', 'Connection Already Exists'
        , 'An attempt was made to create a duplicate connection'
        , 'event.name.101003', 'event.text.101003'
        , 1, 3, 0
        , 'This event is generated when the user tries to create a connection with the same setup as an existing connection.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 101003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '101004', 'BCG101004', 'Community Operator Already Exists'
        , 'An attempt was made to create a Community Operator when one already exists'
        , 'event.name.101004', 'event.text.101004'
        , 1, 3, 0
        , 'This event is generated when the user tries to create a second Community Operator.  The system will only allow one Community Operator to exist for each hub.  The Community Operator is automatically created when the hub is initialized.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 101004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '101005', 'BCG101005', 'Community Manager Already Exists'
        , 'An attempt was made to create a Community Manager when one already exists'
        , 'event.name.101005', 'event.text.101005'
        , 1, 3, 0
        , 'This event is generated when the user tried to create a second Community Manager.  Only one Community Manager can be created per hub.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 101005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '101006', 'BCG101006', 'Password Missing Special Characters'
        , 'The password provided does not meet the requirements defined in the password policy because it does not contain 3 of the 4 special characters'
        , 'event.name.101006', 'event.text.101006'
        , 7, 3, 0
        , 'If the password policy''s "special characters" flag is set, the system will generate this event whenever a user tries to update a user profile with a password that does not contain the valid characters.When the flag is set to true, the password must contain at least 3 of the following 4 types of characters - uppercase, lowercase, numeric, and non-alphanumeric characters (e.g., #&*, etc.)'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 101006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '101007', 'BCG101007', 'Password Contains User Name'
        , 'The password provided contains a variation of the user name'
        , 'event.name.101007', 'event.text.101007'
        , 7, 3, 0
        , 'When the password policy''s "name variation" flag is set to true, the system will generate this event whenever a user tries to update a user profile with a password that contains a variation of the user''s name.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 101007)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '102001', 'BCG102001', 'User Login failed'
        , 'An attempt to logon to the hub has failed due to invalid user name or password'
        , 'event.name.102001', 'event.text.102001'
        , 7, 3, 1
        , 'This event is generated each time a user login attempt has failed.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 102001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '102002', 'BCG102002', 'User Login was successful'
        , 'User {0} has successfully logged onto the system'
        , 'event.name.102002', 'event.text.102002'
        , 7, 1, 0
        , 'This event is generated each time a user has successfully logged onto the console.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 102002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '102003', 'BCG102003', 'Hub Initialized'
        , 'Hub initialized: hub admin and Community Operator have been successfully created'
        , 'event.name.102003', 'event.text.102003'
        , 1, 1, 0
        , 'Before the hub can be used, it must be initialized by creating the hub admin, Community Operator, and Operator''s admin, default user groups and permissions.  This event is generated when a user requests the system to initialize.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 102003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103001', 'BCG103001', 'Database Failure'
        , 'Database Error: {0} failed in {1} with exception {3}'
        , 'event.name.103001', 'event.text.103001'
        , 1, 9, 1
        , 'This event is generated when the database generates an error. This, in turn, causes the system to terminate its process and react by throwing its own exception. The end result of this event is dependent on where the error occurred.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103004', 'BCG103004', 'XML Format Published'
        , 'Publish XML Format: Request the system to reload the XML formats'
        , 'event.name.103004', 'event.text.103004'
        , 1, 1, 1
        , 'This event is generated when the XML Formats are published and reloaded by the system.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103005', 'BCG103005', 'Validation Map Published'
        , 'Publish Validation Map: Request the system to reload the validation maps'
        , 'event.name.103005', 'event.text.103005'
        , 1, 1, 1
        , 'This event is generated when the Validation Maps are published and reloaded by the system.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103006', 'BCG103006', 'XSLT Transformation Map Published'
        , 'Publish XSLT Transformation Map: Request the system to reload the XSLT transformation maps'
        , 'event.name.103006', 'event.text.103006'
        , 1, 1, 1
        , 'This event is generated when the XSLT Transformation Maps are published and reloaded by the system.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103007', 'BCG103007', 'RosettaNet Process Information Published'
        , 'Publish RosettaNet process information:  Request the system to reload the RosettaNet process information'
        , 'event.name.103007', 'event.text.103007'
        , 1, 1, 1
        , 'This event is generated when RosettaNet process information is published and reloaded by the system.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103007)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103009', 'BCG103009', 'Target Configuration Published'
        , 'Published Target Configurations: Request Targets to reload the configuration'
        , 'event.name.103009', 'event.text.103009'
        , 1, 1, 0
        , 'This event is generated when the Target configurations are published and reloaded by the system.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103009)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103010', 'BCG103010', 'Synchronous HTTP/S Configuration Published'
        , 'Published Synchronous HTTP/S Configuration: Request CacheEngine to reload the configuration'
        , 'event.name.103010', 'event.text.103010'
        , 1, 1, 0
        , 'This event is generated when the Synchronous HTTP/S configurations  are published and reloaded by the CacheEngine.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103010)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103011', 'BCG103011', 'Attachment Content-Type Filter Published'
        , 'Published Attachment Content-Type Filter: Request CacheEngine to reload the filter list'
        , 'event.name.103011', 'event.text.103011'
        , 1, 1, 0
        , 'This event is generated when the Attachment Content Type Filter List is published and reloaded by the CacheEngine.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103011)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103012', 'BCG103012', 'Document Flow Definition Updated'
        , 'Publish Web Services process information: Request the system to reload the Web Service process information.'
        , 'event.name.103012', 'event.text.103012'
        , 1, 1, 1
        , 'This event is generated when the system publishes Web Services process information and requests the system to reload the Web Services process information.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103012)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103101', 'BCG103101', 'Cache Engine Error'
        , 'Cache Engine instanceId {0} on host {1} failed to initialized, please correct the problem and restart the service, error reason:{2}'
        , 'event.name.103101', 'event.text.103101'
        , 1, 9, 1
        , 'This event is generated each time the system encounters an error in the cache engine.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103101)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103201', 'BCG103201', 'Hub Owner State Engine Error'
        , 'Error Reason:{0}'
        , 'event.name.103201', 'event.text.103201'
        , 3, 6, 1
        , 'Hub Owner State Engine Error'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103201)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103202', 'BCG103202', 'XML Encoding Warning'
        , 'The XML encoding ''{0}'' does not match to transport Content-Type charset ''{0}''.'
        , 'event.name.103202', 'event.text.103202'
        , 1, 3, 1
        , 'XML Encoding Warning'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103202)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103203', 'BCG103203', 'Target Processing Error'
        , 'Target ''{0},{1}'' failed to processing document, error: {2}.'
        , 'event.name.103203', 'event.text.103203'
        , 7, 6, 1
        , 'Target Processing Error'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103203)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103204', 'BCG103204', 'Target Processing Warning'
        , 'Target ''{0},{1}'' processing document warning, reason: {2}.'
        , 'event.name.103204', 'event.text.103204'
        , 7, 3, 1
        , 'Target Processing Warning'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103204)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103205', 'BCG103205', 'Target Error'
        , 'Target ''{0},{1}'' failed to process target: {2}.'
        , 'event.name.103205', 'event.text.103205'
        , 7, 6, 1
        , 'Target Processing Error'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103205)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103206', 'BCG103206', 'Target with supplied name already exists'
        , 'When trying to create a target entry, the name already exists'
        , 'event.name.103206', 'event.text.103206'
        , 7, 3, 0
        , 'This event is generated each time a user attempts to create a target name that already exists'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103206)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104001', 'BCG104001', 'Participant Created'
        , 'Participant was successfully created: {0}'
        , 'event.name.104001', 'event.text.104001'
        , 1, 1, 1
        , 'This event is generated each time a user successfully creates a new participant profile.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104002', 'BCG104002', 'Participant Updated'
        , 'Participant was successfully updated: {0}'
        , 'event.name.104002', 'event.text.104002'
        , 1, 1, 1
        , 'This event is generated each time a user successfully updates an existing participant profile.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104003', 'BCG104003', 'User Created'
        , 'User was successfully created: {0}'
        , 'event.name.104003', 'event.text.104003'
        , 7, 1, 1
        , 'This event is generated each time a user successfully creates a new user profile for an existing participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104004', 'BCG104004', 'User Updated'
        , 'User successfully updated: {0}'
        , 'event.name.104004', 'event.text.104004'
        , 7, 1, 1
        , 'This event is generated each time a user successfully updates an existing user''s profile.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104005', 'BCG104005', 'User Disabled'
        , 'User disabled: {0}'
        , 'event.name.104005', 'event.text.104005'
        , 7, 1, 1
        , 'This event is generated when a user disables an existing user. A disabled user cannot log onto the console.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104006', 'BCG104006', 'Participant Disabled'
        , 'Participant disabled: {0}'
        , 'event.name.104006', 'event.text.104006'
        , 1, 1, 1
        , 'This event is generated when a user disables an existing participant. When the participant has been disabled, all connections, gateways and users are also disabled.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104007', 'BCG104007', 'Business ID Already Exists'
        , 'An attempt was made to create a duplicate Business ID: {0}'
        , 'event.name.104007', 'event.text.104007'
        , 1, 3, 0
        , 'This event is generated when a user attempts to update a participant profile with a Business ID that already exists. A Business ID is considered a duplicate when the type and ID match an existing ID.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104007)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104008', 'BCG104008', 'User Already Exists'
        , 'An attempt was made to create a duplicate user name: {0}'
        , 'event.name.104008', 'event.text.104008'
        , 7, 3, 0
        , 'This event is generated when the user attempts to create a user using a name that matches another user for the same participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104008)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '105000', 'BCG105000', 'Group Name Already Exists'
        , 'A group with the name "{0}" already exists for participant {1}'
        , 'event.name.105000', 'event.text.105000'
        , 7, 3, 0
        , 'This event is generated when a user attempts to create a group using a name that matches another group for the same participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 105000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106001', 'BCG106001', 'Connection Created'
        , 'A connection was successfully created between participant {0} and participant {1}'
        , 'event.name.106001', 'event.text.106001'
        , 1, 1, 1
        , 'This event is generated when a user successfully creates a new connection between two participants.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106002', 'BCG106002', 'Connection Updated'
        , 'A connection was successfully updated between participant {0} and participant {1}'
        , 'event.name.106002', 'event.text.106002'
        , 1, 1, 1
        , 'This event is generated when a user successfully updates an existing connection between two participants.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106003', 'BCG106003', 'Connection Disabled'
        , 'A connection was disabled between participant {0} and participant {1}'
        , 'event.name.106003', 'event.text.106003'
        , 1, 1, 1
        , 'This event is generated when a user disabled an existing connection between two participants.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106004', 'BCG106004', 'No Default Gateway Pair'
        , 'Connection create failed.  Pair of default gateways does not exist between participants: {0} and {1}'
        , 'event.name.106004', 'event.text.106004'
        , 1, 6, 0
        , 'This event is generated when the system fails to create a connection because a pair of default gateways does not exist for the selected participants.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106005', 'BCG106005', 'No Action Found'
        , 'A connection could not be created for the B2B capability because no actions are associated with the interaction.'
        , 'event.name.106005', 'event.text.106005'
        , 1, 6, 0
        , 'This event is generated when the system fails to create a connection for the B2B capability because no action is associated with the interaction.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106500', 'BCG106500', 'Gateway Created'
        , 'Gateway "{0}" was created for participant {1}'
        , 'event.name.106500', 'event.text.106500'
        , 1, 1, 0
        , 'This event is generated when a user successfully creates a new gateway.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106500)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106505', 'BCG106505', 'Gateway Updated'
        , 'Gateway "{0}" was updated for participant {1}'
        , 'event.name.106505', 'event.text.106505'
        , 1, 1, 0
        , 'This event is generated when a user successfully updates an existing gateway.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106505)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106507', 'BCG106507', 'Gateway Disable Failed'
        , 'Gateway could not be disabled.  It is a default gateway for at least one gateway type.'
        , 'event.name.106507', 'event.text.106507'
        , 1, 3, 0
        , 'This event is generated when a user unsuccessfully disables a gateway.  It is a default gateway for at least one gateway type.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106507)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106600', 'BCG106600', 'Document Flow Definition Create Error'
        , 'Child level = {0} greater than or equal to parent level = {1}'
        , 'event.name.106600', 'event.text.106600'
        , 1, 6, 0
        , 'This event is generated when the user is not able to create a document flow definition.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106600)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '107001', 'BCG107001', 'Page Generated JSP Error'
        , 'A JSP Error occurred:  {0} at {1}'
        , 'event.name.107001', 'event.text.107001'
        , 1, 3, 1
        , 'This event is generated when a JSP page has generated an error.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 107001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '108001', 'BCG108001', 'Not a Certificate'
        , 'The file that was uploaded is either not a certificate or not a format currently supported.'
        , 'event.name.108001', 'event.text.108001'
        , 7, 3, 0
        , 'This event is generated during the certificate creation process whenever the uploaded file is not recognized as a one of the supported certificates.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '108002', 'BCG108002', 'Certificate Created'
        , 'Certificate {1} was successfully created for participant {0}'
        , 'event.name.108002', 'event.text.108002'
        , 7, 1, 1
        , 'This event is generated when a user successfully uploaded and saved a certificate for a participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '108003', 'BCG108003', 'Certificate Updated'
        , 'Certificate {1} was successfully updated for participant {0}'
        , 'event.name.108003', 'event.text.108003'
        , 7, 1, 1
        , 'This event is generated when a user successfully updated an existing certificate for a participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '108005', 'BCG108005', 'Certificate Expiration in 60 Days'
        , 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
        , 'event.name.108005', 'event.text.108005'
        , 7, 3, 1
        , 'The certificate will expire in 60 days and should be replaced.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '108006', 'BCG108006', 'Certificate Expiration in 30 Days'
        , 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
        , 'event.name.108006', 'event.text.108006'
        , 7, 3, 1
        , 'The certificate will expire in 30 days and should be replaced.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '108007', 'BCG108007', 'Certificate Expiration in 15 Days'
        , 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
        , 'event.name.108007', 'event.text.108007'
        , 7, 3, 1
        , 'The certificate will expire in 15 days and should be replaced.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108007)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '108008', 'BCG108008', 'Certificate Expiration in 7 Days'
        , 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
        , 'event.name.108008', 'event.text.108008'
        , 7, 3, 1
        , 'The certificate will expire in 7 days and should be replaced.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108008)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '108009', 'BCG108009', 'Certificate Expiration in 2 Days'
        , 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
        , 'event.name.108009', 'event.text.108009'
        , 7, 3, 1
        , 'The certificate will expire in 2 days and should be replaced.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108009)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '108010', 'BCG108010', 'Certificate Expiration Check'
        , 'Installed certificates were checked for approaching expiration date'
        , 'event.name.108010', 'event.text.108010'
        , 1, 1, 0
        , 'This event is generated when the system checks the expiration date of installed certificates.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108010)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '109001', 'BCG109001', 'Activation Step Completed'
        , 'Activation step {0} was completed for participant {1} by {2} on {3}'
        , 'event.name.109001', 'event.text.109001'
        , 7, 1, 1
        , 'This event is generated each time an Activation step is completed by a Participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 109001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '109002', 'BCG109002', 'Activation Phase Late'
        , 'Activation phase {0} for participant {1}  is past its expected completion date of {2}'
        , 'event.name.109002', 'event.text.109002'
        , 7, 1, 1
        , 'This event is generated each time an Activation phase is past due for a Participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 109002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '110001', 'BCG110001', 'Cache Refresh Internal Error'
        , 'Cached object ''{0}'' on instanceId ''{1}'' of host ''{2}'', error reason:{3}'
        , 'event.name.110001', 'event.text.110001'
        , 1, 3, 1
        , 'This event is generated when the console refresh generates an error.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 110001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '110002', 'BCG110002', 'Cached Object Refreshed Successfully'
        , 'Cached object={0} on cache engine instance={1} of host={2} refreshed successfully.'
        , 'event.name.110002', 'event.text.110002'
        , 1, 1, 0
        , 'This event is generated when the console refresh is successful.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 110002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '111001', 'BCG111001', 'FTP Account Create Error'
        , 'FTP account create failed for participant {0}. Error message: {1}'
        , 'event.name.111001', 'event.text.111001'
        , 1, 6, 1
        , 'This event is generated when the system fails to create a FTP Account.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 111001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '111002', 'BCG111002', 'FTP Account Created'
        , 'An FTP account for participant {0} has been created'
        , 'event.name.111002', 'event.text.111002'
        , 7, 1, 1
        , 'This event is generated when the system successfully creates a FTP Account.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 111002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '111003', 'BCG111003', 'FTP Account Updated'
        , 'The  FTP account for participant {0} has been updated.'
        , 'event.name.111003', 'event.text.111003'
        , 7, 1, 1
        , 'This event is generated when the FTP Account is updated.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 111003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '112002', 'BCG112002', 'Failed to Create Directory'
        , 'Failed to create directory: {0}'
        , 'event.name.112002', 'event.text.112002'
        , 1, 6, 1
        , 'This event is generated when the system fails to create a directory.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '112003', 'BCG112003', 'Document Root Directory Exists'
        , 'Document root directory {0} already exists'
        , 'event.name.112003', 'event.text.112003'
        , 1, 6, 1
        , 'This event is generated when the document root directory already exists.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112003)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '112005', 'BCG112005', 'Destination Type Directory Structure Created'
        , 'Destination type directory created for destination type {0}'
        , 'event.name.112005', 'event.text.112005'
        , 1, 3, 0
        , 'This event is generated when the destination type directory structure is created.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112005)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '112006', 'BCG112006', 'Target Directory Structure Created'
        , 'Target directory {0}  created for Target {1}'
        , 'event.name.112006', 'event.text.112006'
        , 1, 1, 1
        , 'This event is generated when the Target directory structure is created.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '123456', 'BCG123456', 'Participant Name Already Exists'
        , 'An attempt was made to create a duplicate participant name: {0}'
        , 'event.name.123456', 'event.text.123456'
        , 1, 3, 0
        , 'This event is generated when the user tries to create a participant using the same name as an existing participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 123456)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200000', 'BCG200000', 'Get Protocol Transformer Business Process Successful'
        , 'Factory successfully got an instance of the protocol transformer ({0}) business process'
        , 'event.name.200000', 'event.text.200000'
        , 1, 1, 0
        , 'This event is generated when the factory successfully gets an instance of the transformer business process.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200001', 'BCG200001', 'Get Protocol Transformer Business Process Failed'
        , 'Factory failed to get an instance of the protocol transformer business process because {0}'
        , 'event.name.200001', 'event.text.200001'
        , 1, 9, 1
        , 'This event is generated due to system failure when attempting to locate an instance of the protocol transformer business process.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200002', 'BCG200002', 'Protocol Transformer Entrance'
        , 'Protocol transformer business process ({0}) entrance'
        , 'event.name.200002', 'event.text.200002'
        , 1, 1, 0
        , 'This event is generated when a document enters transformation.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200003', 'BCG200003', 'Protocol Transformer Exit'
        , 'Protocol transformer business process ({0}) exit'
        , 'event.name.200003', 'event.text.200003'
        , 1, 1, 0
        , 'This event is generated when a document exits transformation.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200004', 'BCG200004', 'Document Successfully Transformed'
        , '{0}'
        , 'event.name.200004', 'event.text.200004'
        , 1, 1, 0
        , 'This event is generated when a document is successfully transformed.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200005', 'BCG200005', 'Document Transformation Failure'
        , 'Document failed transformation due to {0}'
        , 'event.name.200005', 'event.text.200005'
        , 7, 6, 1
        , 'This event is generated due to a failure during document transformation.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200006', 'BCG200006', 'Protocol Transformer Input File Failure'
        , 'Protocol transformer input file error: {0}'
        , 'event.name.200006', 'event.text.200006'
        , 1, 9, 1
        , 'This event is generated due to a failure with the input file during action processing.  For example, the file is corrupted.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200007', 'BCG200007', 'Protocol Transformer Output File Failure'
        , 'Protocol transformer output file error: {0}'
        , 'event.name.200007', 'event.text.200007'
        , 1, 9, 1
        , 'This event is generated due to a failure when attempting to write to the output file directory.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200007)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200009', 'BCG200009', 'Failed to Parse Document'
        , 'Failed to parse: {0}'
        , 'event.name.200009', 'event.text.200009'
        , 1, 6, 1
        , 'This event is generated due to failure when attempting to parse the document.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200009)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200013', 'BCG200013', 'Community Manager Provided RN Process-Instance-ID Error'
        , '{0}'
        , 'event.name.200013', 'event.text.200013'
        , 7, 6, 1
        , 'This event is generated when an invalid Process Instance ID is received and the configuration property indicates that the system will not generate a new Process Instance ID.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200013)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200014', 'BCG200014', 'Community Manager Provided RN Process-Instance-ID Warning'
        , '{0}'
        , 'event.name.200014', 'event.text.200014'
        , 7, 3, 1
        , 'This event is generated when an invalid Process Instance ID is received and the configuration property indicates that the system will generate a new Process Instance ID.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200014)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200015', 'BCG200015', 'Community Manager Provided RosettaNet GlobalUsageCode Error'
        , '{0}'
        , 'event.name.200015', 'event.text.200015'
        , 7, 6, 1
        , 'This event is generated when the x-aux-production header value is invalid and the configuration property indicates that the system will not use the default value on error.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200015)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200016', 'BCG200016', 'Community Manager Provided RosettaNet GlobalUsageCode Warnin'
        , '{0}'
        , 'event.name.200016', 'event.text.200016'
        , 7, 3, 1
        , 'This event is generated when the received x-aux-production header value is invalid and the configuration property indicates that the system will use the default value on error.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200016)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '200017', 'BCG200017', 'Community Manager Provided RN Process-Instance-ID Error'
        , 'Community Manager Provided RN Process-Instance-ID verification error: {0}'
        , 'event.name.200017', 'event.text.200017'
        , 7, 3, 1
        , 'This event is generated when WBIC failed to verify the Process Instance ID because of system error.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 200017)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210000', 'BCG210000', 'System Error in Channel Check Process'
        , 'System Error in Channel Check Process'
        , 'event.name.210000', 'event.text.210000'
        , 7, 6, 1
        , 'This event is generated when there is a check channel related error.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210001', 'BCG210001', 'Check Channel Error'
        , 'Check Channel Error'
        , 'event.name.210001', 'event.text.210001'
        , 7, 6, 1
        , 'This event is generated when data required to lookup a connection is available but the matching connection is not found.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210002', 'BCG210002', 'Connection Lookup Failed'
        , 'Connection lookup failed {0}'
        , 'event.name.210002', 'event.text.210002'
        , 7, 6, 1
        , 'This event is generated when data required to lookup a connection is not available.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210003', 'BCG210003', 'Document Sent to BPE'
        , 'Document sent to the Business Process Engine'
        , 'event.name.210003', 'event.text.210003'
        , 1, 1, 0
        , 'This event is generated after the inbound processor finished processing the document and successfully puts into the queue for the Business Process Engine.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210004', 'BCG210004', 'Document Rejected'
        , 'Document was rejected'
        , 'event.name.210004', 'event.text.210004'
        , 7, 1, 1
        , 'This event is generated upon initial document receipt to indicate that the document was rejected due to an unreadable or unsupported document type.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210005', 'BCG210005', 'Document Sent to Outbound Processor'
        , 'Document sent to Outbound Processor'
        , 'event.name.210005', 'event.text.210005'
        , 1, 1, 0
        , 'This event is generated after the Business Process Engine processed the document and successfully puts into the queue for the outbound processor.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210006', 'BCG210006', 'Document Sent to Delivery Manager'
        , 'Document sent to Delivery Manager'
        , 'event.name.210006', 'event.text.210006'
        , 1, 1, 0
        , 'This event is generated after the outbound processor processed the document and successfully puts into the queue for the delivery manager.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210007', 'BCG210007', 'Outbound Document Cannot be Packaged'
        , 'Error in Outbound Processor'
        , 'event.name.210007', 'event.text.210007'
        , 1, 9, 1
        , 'This event is generated when a packager is not available for an outbound document.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210007)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210008', 'BCG210008', 'IP Address Validation Failure'
        , 'From IP address is not in the participant profile {0}'
        , 'event.name.210008', 'event.text.210008'
        , 7, 6, 1
        , 'This event is generated when  a document is posted from an unapproved IP Address for that participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210008)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210009', 'BCG210009', 'SSL Certificate Validation Failure'
        , 'Client SSL certificate name is not in the participant profile {0}'
        , 'event.name.210009', 'event.text.210009'
        , 7, 6, 1
        , 'This event is generated when the SSL Certificate used to post the document is not in the approved certificate list for that participant.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210009)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210010', 'BCG210010', 'Document Too Large'
        , 'Document too large: {0} bytes'
        , 'event.name.210010', 'event.text.210010'
        , 7, 6, 1
        , 'This event is generated when the document received is too large to be processed.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210010)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210011', 'BCG210011', 'Community Manager Transport Unpackage Failure'
        , 'Insufficient Community Manager transport information provided: {0}'
        , 'event.name.210011', 'event.text.210011'
        , 3, 6, 1
        , 'This event is generated when insufficient transport information is provided.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210011)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210012', 'BCG210012', 'B2B Capability Not Found'
        , 'B2B capability not found {0}'
        , 'event.name.210012', 'event.text.210012'
        , 7, 6, 1
        , 'This event is generated when the B2B capability required to route the document is not enabled.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210012)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210013', 'BCG210013', 'Connection Not Fully Configured'
        , 'Connection not fully configured {0}'
        , 'event.name.210013', 'event.text.210013'
        , 7, 6, 1
        , 'This event is generated when the connection for the document is not fully configured.  Most likely the destination for the document does not have a configured gateway.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210013)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210014', 'BCG210014', 'MIME Multipart Unpackaging Failure'
        , 'Failed to unpackage a MIME multipart document: {0}'
        , 'event.name.210014', 'event.text.210014'
        , 7, 6, 1
        , 'This event is generated when the system failed to unpackage a MIME multipart document.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210014)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210015', 'BCG210015', 'cXML Packaging Failure'
        , 'Failed to packaing a cXML document: {0}'
        , 'event.name.210015', 'event.text.210015'
        , 7, 6, 1
        , 'cXML Packaging Failure'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210015)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210016', 'BCG210016', 'cXML Channel Parse Failure'
        , 'Failed to parse cXML routing information: {0}'
        , 'event.name.210016', 'event.text.210016'
        , 7, 6, 1
        , 'cXML Channel Parse Failure'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210016)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210017', 'BCG210017', 'EDI Connection Parse Failure'
        , 'Failed to parse EDI routing information: {0}'
        , 'event.name.210017', 'event.text.210017'
        , 7, 6, 1
        , 'This event is generated when the system failed to parse EDI routing information.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210017)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210018', 'BCG210018', 'User Defined Duplicate Check Field(s) not Found'
        , 'User defined duplicate check field(s) not found in document: {0}'
        , 'event.name.210018', 'event.text.210018'
        , 7, 3, 1
        , 'This event is generated when user defined duplicate check fields are not found.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210018)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210019', 'BCG210019', 'Synchronous Operation not Supported on this Connection'
        , 'Synchronous Operation not Supported on this Connection'
        , 'event.name.210019', 'event.text.210019'
        , 7, 6, 1
        , 'This event is generated when the document requests synchronous operation but the connection does not support synchronous operations.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210019)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210020', 'BCG210020', 'Document Moving Error'
        , 'Error encountered while moving a document to its working folder'
        , 'event.name.210020', 'event.text.210020'
        , 1, 0, 0
        , 'This event is generated when the system encounters an error while trying to move a document into its working folder.  The system will re-try when this occurs.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210020)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210021', 'BCG210021', 'Processing cancelled by user'
        , 'The Operator requested that processing of this document be cancelled'
        , 'event.name.210021', 'event.text.210021'
        , 3, 9, 1
        , 'The Operator requested that processing of this document be cancelled.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210021)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210022', 'BCG210022', 'Process transaction rolled back.'
        , 'An error occurred and the processing transaction has rolled back.  The time in process was {0} seconds.'
        , 'event.name.210022', 'event.text.210022'
        , 3, 9, 1
        , 'An error occurred and the processing transaction has rolled back.  The time in process was {0} seconds.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210022)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210030', 'BCG210030', 'Not all Files Present for Document'
        , 'Files missing for doc {0}'
        , 'event.name.210030', 'event.text.210030'
        , 1, 3, 1
        , 'This event is generated when files are missing for the document.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210030)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210031', 'BCG210031', 'Unable to Non-Rep document'
        , 'Unable to Non-Rep document {0}'
        , 'event.name.210031', 'event.text.210031'
        , 1, 9, 1
        , 'This event is generated when the system is unable to non-repudiate the document.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210031)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210032', 'BCG210032', 'System Error in the Inbound Processor'
        , 'System error in the Inbound Processor for document: {0}'
        , 'event.name.210032', 'event.text.210032'
        , 1, 9, 1
        , 'This event is generated when the system encounters an error in the inbound processor.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210032)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210033', 'BCG210033', 'Message Store Failed'
        , 'Unable to store document plain text.'
        , 'event.name.210033', 'event.text.210033'
        , 1, 6, 1
        , 'This event is generated when the system is unable to store the document in plain text.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210033)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210034', 'BCG210034', 'System Error in the BPE'
        , 'System error in the BPE for document: {0}'
        , 'event.name.210034', 'event.text.210034'
        , 1, 9, 1
        , 'This event is generated when the system encounters an error in the BPE.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210034)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210050', 'BCG210050', 'Duplicate Check Successful'
        , 'Passed Duplicate Check'
        , 'event.name.210050', 'event.text.210050'
        , 1, 0, 0
        , 'This event is generated when a document successfully passes duplicate checking.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210050)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210051', 'BCG210051', 'Duplicate Processing Failure'
        , 'System error - failure in duplicate process'
        , 'event.name.210051', 'event.text.210051'
        , 1, 9, 1
        , 'This event is generated when the system is unable to contact the database server during duplicate processing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210051)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210052', 'BCG210052', 'Duplicate Document Received'
        , 'This document appears to be a duplication of a document sent on {2}'
        , 'event.name.210052', 'event.text.210052'
        , 7, 6, 1
        , 'This event is generated when a document received is a duplicate and rejected.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210052)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210060', 'BCG210060', 'Destination Parse Successful'
        , 'Passed destination parse'
        , 'event.name.210060', 'event.text.210060'
        , 1, 1, 0
        , 'This event is generated when a document successfully passes destination parse.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210060)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210061', 'BCG210061', 'Destination Parse Failure'
        , 'Error in destination Parse:{0}'
        , 'event.name.210061', 'event.text.210061'
        , 1, 9, 1
        , 'This event is generated when destination parse fails.  Usually due to a database problem.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210061)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210062', 'BCG210062', 'Destination Process Successful'
        , 'Destination Process successful'
        , 'event.name.210062', 'event.text.210062'
        , 1, 1, 0
        , 'This event is generated when destination processing is successful.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210062)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210063', 'BCG210063', 'Destination Process Failure'
        , 'Destination Process failed'
        , 'event.name.210063', 'event.text.210063'
        , 1, 9, 1
        , 'This event is generated when destination processing fails.  Usually due to a database problem.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210063)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210065', 'BCG210065', 'Destination Determination Failure'
        , '{0}'
        , 'event.name.210065', 'event.text.210065'
        , 7, 6, 1
        , 'This event is generated when there are conflicting inputs when processing  the destination.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210065)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210066', 'BCG210066', 'Package and Content Business Id''s map to different partners.'
        , 'From Partner ID = {0}, To Partner ID = {1}, From Package Partner ID = {2}, To Package Partner ID = {3}'
        , 'event.name.210066', 'event.text.210066'
        , 7, 6, 1
        , 'This event is generated when there is a mismatch between the content and package routing information'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210066)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210100', 'BCG210100', 'Timing Event - Start'
        , 'Timing event - Start {0}'
        , 'event.name.210100', 'event.text.210100'
        , 1, 0, 0
        , 'This event is generated at the start of each action step.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210100)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210101', 'BCG210101', 'Timing Event - Finish'
        , 'Timing event - Finish'
        , 'event.name.210101', 'event.text.210101'
        , 1, 0, 0
        , 'This event is generated at the end of each action step.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210101)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210200', 'BCG210200', 'Doctype Processing Completed Successfully'
        , 'Doctype processing completed successfully'
        , 'event.name.210200', 'event.text.210200'
        , 1, 1, 0
        , 'This event is generated when RN Service Content is received and the system successfully inserts the DocType tag.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210200)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210201', 'BCG210201', 'PIP Load During Doctype Processing Failure'
        , 'Unable to load PIP for a document during Doctype processing'
        , 'event.name.210201', 'event.text.210201'
        , 1, 9, 1
        , 'This event is generated when a spec for the PIP cannot be found.  Should not occur unless there is a configuration problem.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210201)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210202', 'BCG210202', 'Exception in Doctype Processing'
        , 'Exception during Doctype Processing: {0}'
        , 'event.name.210202', 'event.text.210202'
        , 1, 9, 1
        , 'This event is generated when the system fails when attempting to insert the DocType tag.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210202)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '210203', 'BCG210203', 'DoctypeProcess Error - No Action Found'
        , 'DoctypeProcess Error - No action found'
        , 'event.name.210203', 'event.text.210203'
        , 1, 9, 1
        , 'This event is generated when a spec for the PIP DocType cannot be found.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210203)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230000', 'BCG230000', 'Validation Business Process Entrance'
        , 'Validation business process ({0}) entrance'
        , 'event.name.230000', 'event.text.230000'
        , 1, 1, 0
        , 'This event is generated when a document enters validation of the  business process.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230001', 'BCG230001', 'Field Validation Error'
        , 'Field Validation Error'
        , 'event.name.230001', 'event.text.230001'
        , 1, 6, 0
        , 'System description of location of validation errors so that the console can highlight for data validation.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230002', 'BCG230002', 'Parser Error'
        , 'Parser Error'
        , 'event.name.230002', 'event.text.230002'
        , 1, 0, 0
        , 'System description of location of validation errors so that the console can highlight for data validation.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230003', 'BCG230003', 'Validation Business Process Exit'
        , 'Validation business process ({0}) exit'
        , 'event.name.230003', 'event.text.230003'
        , 1, 1, 0
        , 'This event is generated when the document exits business validation processing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230004', 'BCG230004', 'Validation Internal Error'
        , '{0}'
        , 'event.name.230004', 'event.text.230004'
        , 1, 9, 1
        , 'This event is generated due to internal system failure during validation processing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230005', 'BCG230005', 'Validation Successful'
        , '{0}'
        , 'event.name.230005', 'event.text.230005'
        , 7, 1, 0
        , 'This event is generated when a document successfully completes process validation.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230006', 'BCG230006', 'Validation Database Error'
        , '{0}'
        , 'event.name.230006', 'event.text.230006'
        , 1, 9, 1
        , 'This event is generated due to a database error during validation processing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230007', 'BCG230007', 'Validation Business Process Factory Error'
        , '{0}'
        , 'event.name.230007', 'event.text.230007'
        , 1, 6, 1
        , 'This event is generated when the system is unable to determine the process to send to the validation engine.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230007)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230008', 'BCG230008', 'Validation Warning'
        , '{0}'
        , 'event.name.230008', 'event.text.230008'
        , 7, 3, 1
        , 'This event is generated when a document is received and the initiating process cannot be found.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230008)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230009', 'BCG230009', 'RosettaNet Validation Error'
        , '{0}'
        , 'event.name.230009', 'event.text.230009'
        , 7, 6, 1
        , 'This event is generated when a document fails to complete RosettaNet process validation.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230009)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230010', 'BCG230010', 'Data Validation Error'
        , 'Document failed data validation: {0}'
        , 'event.name.230010', 'event.text.230010'
        , 7, 6, 1
        , 'This event is generated when a document fails data validation and is rejected.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230010)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230011', 'BCG230011', 'Document Sequence Validation Successful'
        , '{0}'
        , 'event.name.230011', 'event.text.230011'
        , 1, 1, 0
        , 'This event is generated when a document successfully passes sequence validation.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230011)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230012', 'BCG230012', 'AS Sequence Validation Error'
        , '{0}'
        , 'event.name.230012', 'event.text.230012'
        , 7, 6, 1
        , 'This event is generated when a document fails to complete EDIINT process validation.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230012)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '230015', 'BCG230015', 'Failed to Log RN State due to Missing Key Data'
        , 'Unable to log RN state for RNSC validation error for synchronous response.  Missing either initiate Business ID = {0} and/or Process Instance ID {1}'
        , 'event.name.230015', 'event.text.230015'
        , 1, 1, 0
        , 'This event is generated when the system fails to log the RN state because the Business ID or the Process Instance ID is missing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 230015)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240003', 'BCG240003', 'RosettaNet Unpackaging Error'
        , 'RosettaNet Unpackaging Error'
        , 'event.name.240003', 'event.text.240003'
        , 7, 6, 1
        , 'This event is generated when the system is unable to parse the RosettaNet preamble during unpackaging.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240005', 'BCG240005', 'RNPackager Delivery Header Parser Failure'
        , 'Delivery Header Parser Error: {0}'
        , 'event.name.240005', 'event.text.240005'
        , 7, 6, 1
        , 'This event is generated when the system is unable to parse the RosettaNet delivery header during unpackaging.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240007', 'BCG240007', 'RNPackager Service Header Failure'
        , 'Service Header parser error: {0}'
        , 'event.name.240007', 'event.text.240007'
        , 7, 6, 1
        , 'This event is generated when the system is unable to parse the RosettaNet service header during unpackaging.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240007)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240009', 'BCG240009', 'RNPackager Mime Parsing Failure'
        , 'Mime parsing error: {0}'
        , 'event.name.240009', 'event.text.240009'
        , 7, 6, 1
        , 'This event is generated when an error occurs in Mime parsing of the RosettaNet message during unpackaging.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240009)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240011', 'BCG240011', 'RNPackager Signature Failed'
        , 'Digital Signature validation failed: {0}'
        , 'event.name.240011', 'event.text.240011'
        , 7, 6, 1
        , 'This event is generated when digital signature validation fails during unpackaging.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240011)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240012', 'BCG240012', 'RN Unpackaging State Update Error'
        , 'Database access failure: Could not update the RosettaNet state'
        , 'event.name.240012', 'event.text.240012'
        , 1, 9, 1
        , 'This event is generated when the unpackager encounters database communication errors when updating the RosettaNet state.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240012)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240013', 'BCG240013', 'Participant Certificate Did Not Match Signer'
        , 'Name/serial on signer certificate did not match database entry'
        , 'event.name.240013', 'event.text.240013'
        , 7, 6, 1
        , 'This event is generated when Certificate to DUNS check fails for digital signature.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240013)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240014', 'BCG240014', 'Missing Signature in Document'
        , 'Signature not found in document'
        , 'event.name.240014', 'event.text.240014'
        , 7, 6, 1
        , 'This event is generated when a signature is required by the TPA, but not found in the document.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240014)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240015', 'BCG240015', 'RosettaNet Document Creation Failure'
        , '{0}'
        , 'event.name.240015', 'event.text.240015'
        , 1, 9, 1
        , 'This event is generated when an attempt to construct a RosettaNet document fails.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240015)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240016', 'BCG240016', 'RosettaNet Non-Repudiation Error'
        , '{0}'
        , 'event.name.240016', 'event.text.240016'
        , 7, 6, 1
        , 'This event is generated when the Receipt Ack does not contain correct digest of previous message, or the digest is missing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240016)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240017', 'BCG240017', 'Synchronous Acknowledgement Not Received'
        , 'Synchronous acknowledgement is required but was not received in the synchronous response'
        , 'event.name.240017', 'event.text.240017'
        , 1, 6, 0
        , 'This event is generated when a synchronous acknowledgement is required but was not received in the synchronous response.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240017)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240018', 'BCG240018', 'Digital Signature Key Not Loaded for Operator'
        , 'Digital Signature Key Not Loaded for Operator'
        , 'event.name.240018', 'event.text.240018'
        , 1, 1, 0
        , 'This event is generated when digital signature keys are not loaded for the Community Operator.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240018)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240019', 'BCG240019', 'Encryption Key Not Loaded for Operator'
        , 'Encryption Key Not Loaded for Operator'
        , 'event.name.240019', 'event.text.240019'
        , 1, 1, 0
        , 'This event is generated when encryption keys are not loaded for the Community Operator.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240019)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240020', 'BCG240020', 'Synchronous Response Not Received'
        , 'Synchronous response is required but received an empty synchronous response content.'
        , 'event.name.240020', 'event.text.240020'
        , 1, 1, 0
        , 'This event is generated when a synchronous acknowledgement is required and empty synchronous response was received'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240020)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240021', 'BCG240021', 'Digital Signature Certificate has been revoked'
        , 'Digital Signature Certificate has been revoked: {0}'
        , 'event.name.240021', 'event.text.240021'
        , 1, 1, 0
        , 'This event is generated when digital signature certificate has been revoked.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240021)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240022', 'BCG240022', 'Encryption Certificate has been revoked'
        , 'Encryption Certificate has been revoked: {0}'
        , 'event.name.240022', 'event.text.240022'
        , 1, 1, 0
        , 'This event is generated when encryption certificate has been revoked.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240022)
/

INSERT INTO lg_EventCd ( EVENTCD
    ,EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC )
SELECT 240028, 'BCG240028', 'Certificate has been revoked'
    , 'Certificate has been revoked:  Serial number: {0}, Subject DN: {1}, Issuer DN: {2}'
    , 'event.name.240028', 'event.text.240028'
    , 7, 9, 0
    , 'This event is generated when a certificate is revoked'
FROM DUAL
WHERE NOT EXISTS (SELECT * FROM lg_EventCd WHERE EVENTCD = 240028)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240031', 'BCG240031', 'Packaging Instance Error'
        , 'Error: {0}'
        , 'event.name.240031', 'event.text.240031'
        , 1, 9, 1
        , 'This event is generated when the system is unable to find a packager for the supplied document type.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240031)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240034', 'BCG240034', 'XML Packager Successful'
        , 'XML parsing successful'
        , 'event.name.240034', 'event.text.240034'
        , 1, 1, 0
        , 'This event is generated when the system successfully completes XML parsing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240034)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240036', 'BCG240036', 'Unpackaging Instance Error'
        , 'Error: {0}'
        , 'event.name.240036', 'event.text.240036'
        , 1, 6, 1
        , 'This event is generated when the system cannot find an unpackager for a document.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240036)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240037', 'BCG240037', 'XML Format Selection'
        , 'XML format selected. Doc fam/type/Fmt id/ver: {0}'
        , 'event.name.240037', 'event.text.240037'
        , 1, 1, 0
        , 'This event is generated when an xml format is used.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240037)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240060', 'BCG240060', 'Connection Parse Factory Success'
        , 'Factory using {0}'
        , 'event.name.240060', 'event.text.240060'
        , 1, 0, 0
        , 'This event is generated when the system successfully completes connection parsing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240060)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240065', 'BCG240065', 'Connection Parse XML Failure'
        , 'XML connection parsing failed: {0}'
        , 'event.name.240065', 'event.text.240065'
        , 7, 6, 1
        , 'This event is generated when connection info for an XML message could not be found.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240065)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240068', 'BCG240068', 'Connection Parser RosettaNet Failure'
        , 'Connection Parse RosettaNet Failure'
        , 'event.name.240068', 'event.text.240068'
        , 7, 6, 1
        , 'This event is generated when connection info could not be found in a RosettaNet document.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240068)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240069', 'BCG240069', 'XML Connection Parse Succeeded'
        , 'XML connection parse succeeded'
        , 'event.name.240069', 'event.text.240069'
        , 1, 1, 0
        , 'This event is generated when the system successfully completes XML Connection parsing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240069)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240070', 'BCG240070', 'XML Connection Parse Failure'
        , 'XML connection parse failed'
        , 'event.name.240070', 'event.text.240070'
        , 7, 6, 1
        , 'This event is generated when the system is unable to find connection information for an XML file.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240070)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240071', 'BCG240071', 'Flat File Connection Parse Failure'
        , 'Flat File connection parse failed: {0}'
        , 'event.name.240071', 'event.text.240071'
        , 7, 6, 1
        , 'This event is generated when the system is unable to find connection information for a Flat File.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240071)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240072', 'BCG240072', 'MIME Structure Validation Warning'
        , 'MIME Document contains trailing data'
        , 'event.name.240071', 'event.text.240071'
        , 7, 3, 1
        , 'This event is generated when extra data is found at the end of an RNIF 2.0 document and the system generates a MIME structure validation warning.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240072)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240077', 'BCG240077', 'Web Service Connection Parse Succeeded'
        , 'Web Service connection parse succeeded'
        , 'event.name.240077', 'event.text.240077'
        , 1, 1, 0
        , 'This event is generated when the system finds connection information for a SOAP message.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240077)
/

--61817 changed internal description to "Web Service Connection Parse Failed. Reason {0}"
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240078', 'BCG240078', 'Web Service Connection Parse Failed'
        , 'Web Service Connection Parse Failed. Reason {0}'
        , 'event.name.240078', 'event.text.240078'
        , 7, 6, 1
        , 'This event is generated when the system is unable to find connection information for a SOAP message.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240078)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240409', 'BCG240409', 'AS Unpackager Failure'
        , 'AS Unpackager Error: {0}'
        , 'event.name.240409', 'event.text.240409'
        , 7, 6, 1
        , 'This event is generated when the AS unpackager fails.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240409)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240411', 'BCG240411', 'AS Signature Failure'
        , 'AS Signature Validation Error: {0}'
        , 'event.name.240411', 'event.text.240411'
        , 7, 6, 1
        , 'This event is generated when AS signature validation fails.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240411)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240412', 'BCG240412', 'AS State Engine DB Failure'
        , 'AS State Engine DB error: {0}'
        , 'event.name.240412', 'event.text.240412'
        , 1, 9, 1
        , 'This event is generated when the AS state engine database fails.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240412)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240415', 'BCG240415', 'AS Packager Failure'
        , 'AS Packager Error: {0}'
        , 'event.name.240415', 'event.text.240415'
        , 1, 9, 1
        , 'This event is generated when the AS packager fails.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240415)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240416', 'BCG240416', 'AS Non-Repudiation Error'
        , '{0}'
        , 'event.name.240416', 'event.text.240416'
        , 7, 6, 1
        , 'This event is generated when AS Non-Repudiation fails.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240416)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240417', 'BCG240417', 'Decryption Failed'
        , '{0}'
        , 'event.name.240417', 'event.text.240417'
        , 7, 6, 1
        , 'This event is generated when decryption fails.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240417)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240418', 'BCG240418', 'Unable to Generate Message Digest'
        , '{0}'
        , 'event.name.240418', 'event.text.240418'
        , 7, 6, 1
        , 'This event is generated when the system is unable to generate a message digest.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240418)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240419', 'BCG240419', 'Unsupported Signature Format'
        , '{0}'
        , 'event.name.240419', 'event.text.240419'
        , 7, 6, 1
        , 'This event is generated when the system receives an unsupported signature format.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240419)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240420', 'BCG240420', 'Unsupported Signature Algorithm'
        , '{0}'
        , 'event.name.240420', 'event.text.240420'
        , 7, 6, 1
        , 'This event is generated when the system receives unsupported signature algorithm.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240420)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240421', 'BCG240421', 'Unexpected Error'
        , '{0}'
        , 'event.name.240421', 'event.text.240421'
        , 1, 6, 1
        , 'This event is generated when the system encounters an unexpected error.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240421)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240422', 'BCG240422', 'AS document not found for this MDN'
        , '{0}'
        , 'event.name.240422', 'event.text.240422'
        , 7, 6, 1
        , 'This event is generated when a MDN is received and the system is unable to locate the corresponding document.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240422)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240423', 'BCG240423', 'Input File Failure'
        , 'Invalid input file passed in the document'
        , 'event.name.240423', 'event.text.240423'
        , 1, 6, 0
        , 'This event is generated when the system encounters an invalid input file.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240423)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240424', 'BCG240424', 'Insufficient Message Security'
        , '{0}'
        , 'event.name.240424', 'event.text.240424'
        , 7, 6, 1
        , 'This event is generated when the system encounters insufficient message security.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240424)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240425', 'BCG240425', 'MDN Not Signed'
        , 'MDN is supposed to be signed but not signed'
        , 'event.name.240425', 'event.text.240425'
        , 7, 3, 1
        , 'This event is generated when an inbound MDN is supposed to be signed but not signed'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240425)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240500', 'BCG240500', 'RosettaNet State Engine Error'
        , 'RosettaNet State Engine Error'
        , 'event.name.240500', 'event.text.240500'
        , 1, 9, 1
        , 'This event is generated when the RosettaNet State Engine encounters a system error.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240500)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240501', 'BCG240501', 'RN State Engine Generated 0A1'
        , 'RosettaNet State Engine Generated 0A1 for PIP {0} Process Instance ID = {1}'
        , 'event.name.240501', 'event.text.240501'
        , 7, 1, 1
        , 'This event is generated when the RosettaNet State Engine initiated a 0A1. (PIP is cancelled per RosettaNet standard)'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240501)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240550', 'BCG240550', 'POP3 Poll Error'
        , 'Error polling POP3 server: {0}; rejected message VUID: {1}'
        , 'event.name.240550', 'event.text.240550'
        , 1, 6, 1
        , 'Event generated when trying to poll a message from a POP3 server.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240550)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240600', 'BCG240600', 'AS State Engine Error'
        , 'AS State Engine Error: {0}'
        , 'event.name.240600', 'event.text.240600'
        , 1, 9, 1
        , 'This event is generated when the RosettaNet State Engine encounters a system error.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240600)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '240601', 'BCG240601', 'AS Retry Failure'
        , 'AS Attribute max retry limit reached'
        , 'event.name.240601', 'event.text.240601'
        , 7, 6, 1
        , 'This event is generated when the system fails AS retries. The maximum retry limit may have been reached.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240601)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '250001', 'BCG250001', 'Document Delivery Failed'
        , 'Document delivery to participant gateway failed: {0}'
        , 'event.name.250001', 'event.text.250001'
        , 7, 6, 1
        , 'This event is generated  when document delivery to a participant''s gateway fails and the document is set to a failed state.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 250001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '250002', 'BCG250002', 'Delivery Scheduler Failed'
        , 'An internal error occurred in the Delivery Scheduler: {0}'
        , 'event.name.250002', 'event.text.250002'
        , 1, 9, 1
        , 'This event is generated when an uncategorized internal error occurred within the Delivery Manager, due to bad gateway or document data, rather than failure to deliver.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 250002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '250003', 'BCG250003', 'Delivery Scheduler Warning'
        , 'Warning: {0}'
        , 'event.name.250003', 'event.text.250003'
        , 7, 3, 1
        , 'This event is generated when a non-fatal abnormal condition occurred in the Delivery Manager, e.g. a retry failed but there are still more retries allowed.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 250003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '250004', 'BCG250004', 'Document Delivered'
        , 'Document was delivered successfully, response: {0}'
        , 'event.name.250004', 'event.text.250004'
        , 7, 1, 0
        , 'This event is generated when a document is successfully delivered.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 250004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '250005', 'BCG250005', 'FTP Delivery Failed'
        , 'FTP delivery to participant gateway failed with exception: {0}'
        , 'event.name.250005', 'event.text.250005'
        , 7, 6, 1
        , 'This event is generated  when the FTP protocol document delivery failed but more retries may be possible. Final failure will generate event 250001.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 250005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '250006', 'BCG250006', 'Document Delivered to Exception Directory'
        , 'Document delivered to directory: {0}'
        , 'event.name.250006', 'event.text.250006'
        , 7, 3, 1
        , 'This event is generated when the document was delivered to the participant''s default "exception" directory, not a normal destination directory.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 250006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '250010', 'BCG250010', 'Auto Gateway Set Offline'
        , 'Gateway {0} set offline automatically due to delivery failure'
        , 'event.name.250010', 'event.text.250010'
        , 7, 3, 1
        , 'This event is generated when the Gateway is automatically set offline due to failure to deliver.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 250010)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '250011', 'BCG250011', 'First Delivery Attempt Failed'
        , 'First delivery attempt failed for message {0} due to {1}, on gateway {2}'
        , 'event.name.250011', 'event.text.250011'
        , 3, 3, 1
        , 'This event is generated when the first attempt to deliver a document fails.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 250011)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '250012', 'BCG250012', 'Delivery Retry Failed'
        , '{0} retry {1} for message {2} failed due to {3}, on gateway {4}'
        , 'event.name.250012', 'event.text.250012'
        , 3, 3, 1
        , 'This event is generated when a retry to deliver a document fails.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 250012)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '260001', 'BCG260001', 'RosettaNet Pass Through Logging Warning'
        , 'Unsupported RosettaNet protocol version ({0}) for pass through process view logging'
        , 'event.name.260001', 'event.text.260001'
        , 7, 3, 0
        , 'This event is generated when unsupported RosettaNet protocol version is received.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 260001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '260002', 'BCG260002', 'RosettaNet Pass Through Logging Failed'
        , 'RosettaNet pass through process view logging failed: {0}'
        , 'event.name.260002', 'event.text.260002'
        , 1, 6, 1
        , 'This event is generated when a document fails RN pass through logging.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 260002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '270000', 'BCG270000', 'Unexpected Exception in Backend Integration Unpackaging'
        , 'Encountered unexpected exception while unpackaging backend integration document: {0}'
        , 'event.name.270000', 'event.text.270000'
        , 1, 1, 0
        , 'This event is generated when the system encounters an unexpected error during backend integration unpackaging.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 270000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '270001', 'BCG270001', 'Missing Headers in Backend Integration'
        , 'Backend integration missing required headers: {0}'
        , 'event.name.270001', 'event.text.270001'
        , 1, 1, 0
        , 'This event is generated when the  backend integration package is missing required headers.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 270001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '270002', 'BCG270002', 'Unexpected Exception in Backend Integration Packaging'
        , 'Encountered unexpected exception while packaging backend integration document: {0}'
        , 'event.name.270002', 'event.text.270002'
        , 1, 6, 0
        , 'This event is generated when the system encounters an unexpected error during backend integration packaging.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 270002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '270003', 'BCG270003', 'Backend Integration Unpackage Parse Error'
        , 'Unpackaging backend integration envelope encountered XML parser error: {0}'
        , 'event.name.270003', 'event.text.270003'
        , 1, 9, 0
        , 'This event is generated when the system encounters an XML parser error during backend integration unpackaging.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 270003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '300001', 'BCG300001', 'Adapter Received Duplicate Message'
        , '{0}'
        , 'event.name.300001', 'event.text.300001'
        , 7, 3, 0
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 300001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '300002', 'BCG300002', 'Adapter Internal Error'
        , '{0}'
        , 'event.name.300002', 'event.text.300002'
        , 7, 3, 0
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 300002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '300003', 'BCG300003', 'Adapter Backend Interface Failed'
        , '{0}'
        , 'event.name.300003', 'event.text.300003'
        , 7, 3, 0
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 300003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '300004', 'BCG300004', 'Adapter Received Malformed Data'
        , '{0}'
        , 'event.name.300004', 'event.text.300004'
        , 7, 3, 0
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 300004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '300005', 'BCG300005', 'Adapter Backend Integration Failed due to Participant Data'
        , '{0}'
        , 'event.name.300005', 'event.text.300005'
        , 7, 3, 0
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 300005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '300006', 'BCG300006', 'Adapter Received Unknown Requested Document Handler'
        , '{0}'
        , 'event.name.300006', 'event.text.300006'
        , 7, 3, 0
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 300006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '800000', 'BCG800000', 'Get Community Manager Business Process Failed'
        , 'Failed to get an instance of the Community Manager business process because {0}'
        , 'event.name.800000', 'event.text.800000'
        , 1, 9, 1
        , 'This event is generated when the system fails to locate the Community Manager action for business processing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 800000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '800001', 'BCG800001', 'Community Manager Business Process Entrance'
        , 'Community Manager business process ({0}) entrance'
        , 'event.name.800001', 'event.text.800001'
        , 1, 0, 0
        , 'This event is generated when the document enters the Community Manager action for business processing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 800001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '800002', 'BCG800002', 'Community Manager Business Process Exit'
        , 'Community Manager business process ({0}) exit'
        , 'event.name.800002', 'event.text.800002'
        , 1, 0, 0
        , 'This event is generated when the document exits the Community Manager action for business processing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 800002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '800003', 'BCG800003', 'Community Manager Business Process Successful'
        , 'Community Manager business process successful'
        , 'event.name.800003', 'event.text.800003'
        , 1, 1, 0
        , 'This event is generated when the document successfully completes the Community Manager action for business processing.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 800003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '800004', 'BCG800004', 'Community Manager Business Process Encounters Database Error'
        , '{0}'
        , 'event.name.800004', 'event.text.800004'
        , 1, 9, 1
        , 'This event is generated due to database error while processing the Community Manager''s action.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 800004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '800005', 'BCG800005', 'Community Manager Process Encounters Internal Error'
        , '{0}'
        , 'event.name.800005', 'event.text.800005'
        , 1, 9, 1
        , 'This event is generated due to internal system error while processing the Community Manager''s action.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 800005)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '800006', 'BCG800006', 'Community Manger Business Process Warning'
        , '{0}'
        , 'event.name.800006', 'event.text.800006'
        , 1, 3, 1
        , 'This event is generated when the file map cannot be found and the Delivery Manager is notified to deliver the document to the exception directory.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 800006)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '800007', 'BCG800007', 'Discard MSA File'
        , 'Discard MSA File'
        , 'event.name.800007', 'event.text.800007'
        , 1, 0, 0
        , 'This event is generated when the system discards the MSA File Map.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 800007)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '900001', 'BCG900001', '0A1 Service Content Received'
        , '0A1 service content received'
        , 'event.name.900001', 'event.text.900001'
        , 3, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 900001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106000', 'BCG106000', 'Publishing Information Updated'
        , 'Publishing Information Updated'
        , 'event.name.106000', 'event.text.106000'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104010', 'BCG104010', 'FixedWorkFlow Handler Added'
        , 'FixedWorkFlow Handler Added'
        , 'event.name.104010', 'event.text.104010'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104010)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104011', 'BCG104011', 'Administrative API status updated'
        , 'Administrative API status updated'
        , 'event.name.104011', 'event.text.104011'
        , 1, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104011)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104000', 'BCG104000', 'Processing Interval is updated'
        , 'Processing Interval is updated'
        , 'event.name.104000', 'event.text.104000'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '105001', 'BCG105001', 'Gateway handler metadata Modified'
        , 'Gateway Handler Updated'
        , 'event.name.105001', 'event.text.105001'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 105001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '105002', 'BCG105002', 'Gateway transport meta data modified'
        , 'Gateway transport metadata modified'
        , 'event.name.105002', 'event.text.105002'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 105002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '105003', 'BCG105003', 'Gateway information updated'
        , 'Gateway information updated'
        , 'event.name.105003', 'event.text.105003'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 105003)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '111111', 'BCG111111', 'Event Delivery Failure'
        , 'Error occured during event delivery. No future events will be delivered. Error details: {0}'
        , 'event.name.111111', 'event.text.111111'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 111111)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104013', 'BCG104013', 'Receiver Exit Event Published'
        , 'Receiver Exit Event Published. Request CacheEngine to reload the configuration'
        , 'event.name.104013', 'event.text.104013'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104013)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104012', 'BCG104012', 'Variable Workflow Event Published'
        , 'Variable Workflow Event Published. Request CacheEngine to reload the configuration'
        , 'event.name.104012', 'event.text.104012'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104012)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '104009', 'BCG104009', 'Participant Deleted'
        , 'Participant Deleted'
        , 'event.name.104009', 'event.text.104009'
        , 1, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104009)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106501', 'BCG106501', 'Gateway has been disabled'
        , 'Gateway "{0}" has been disabled by participant {1}'
        , 'event.name.106501', 'event.text.106501'
        , 1, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106501)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106502', 'BCG106502', 'Gateway has been enabled'
        , 'Gateway "{0}" has been enabled by participant {1}'
        , 'event.name.106502', 'event.text.106502'
        , 1, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106502)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106503', 'BCG106503', 'Gateway has been made Offline'
        , 'Gateway "{0}" has been made offline by participant {1}'
        , 'event.name.106503', 'event.text.106503'
        , 1, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106503)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '106504', 'BCG106504', 'Gateway has been made Online'
        , 'Gateway "{0}" has been made online by participant {1}'
        , 'event.name.106504', 'event.text.106504'
        , 1, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106504)
/

--59043
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240603, 'BCG240603', 'Packaging Business Process Entry'
        , 'Packaging business process {0} entry'
        , 'event.name.240603', 'event.text.240603'
        ,7,1,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240603)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240604, 'BCG240604', 'Packaging Business Process Exit'
        , 'Packaging business process {0} exit'
        , 'event.name.240604', 'event.text.240604'
        ,7,1,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240604)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240607, 'BCG240607', 'Unpackaging Business Process Entry'
        , 'Unpackaging business process {0} entry'
        , 'event.name.240607', 'event.text.240607'
        ,7,1,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240607)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240608, 'BCG240608', 'Unpackaging Business Process Exit'
        , 'Unpackaging business process {0} exit'
        , 'event.name.240608', 'event.text.240608'
        ,7,1,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240608)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240612, 'BCG240612', 'Protocol Parse Business Process Entry'
        , 'Protocol parse business process {0} entry'
        , 'event.name.240612', 'event.text.240612'
        ,7,1,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240612)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240613, 'BCG240613', 'Protocol Parse Business Process Exit'
        , 'Protocol parse business process {0} exit'
        , 'event.name.240613', 'event.text.240613'
        ,7,1,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240613)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240605, 'BCG240605', 'Packaging Warning'
        , 'Packaging warning {0}'
        , 'event.name.240605', 'event.text.240605'
        ,7,3,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240605)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240609, 'BCG240609', 'Unpackaging Warning'
        , 'Unpackaging warning {0}'
        , 'event.name.240609', 'event.text.240609'
        ,7,3,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd WHERE EVENTCD = 240609)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240614, 'BCG240614', 'Protocol Parse Warning'
        , 'Protocol parse warning {0}'
        , 'event.name.240614', 'event.text.240614'
        ,7,3,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240614)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240606, 'BCG240606', 'Packaging Error'
        , 'Packaging error {0}'
        , 'event.name.240606', 'event.text.240606'
        ,7,6,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd WHERE EVENTCD = 240606)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240611, 'BCG240611', 'Encryption Failure'
        , 'Encryption failure {0}'
        , 'event.name.240611', 'event.text.240611'
        ,7,1,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd WHERE EVENTCD = 240611)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240610, 'BCG240610', 'Unpackaging Error'
        , 'Unpackaging error {0}'
        , 'event.name.240610', 'event.text.240610'
        ,7,6,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd WHERE EVENTCD = 240610)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240615, 'BCG240615', 'Protocol Parse Error'
        , 'Protocol parse error {0}'
        , 'event.name.240615', 'event.text.240615'
        ,7,6,1
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd WHERE EVENTCD = 240615)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID )
Select 240616, 'BCG240616', 'MDN not received.'
, 'A requested MDN was not received for the document with document reference id {0} and AS Message Id {1}.'
, 'event.name.240616', 'event.text.240616'
, 7, 3, 1
, 'This event is generated when the MDN was not received even after retries'
, 1
, 2
, 3
From DUAL
Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240616)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC,EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID )
Select 240617, 'BCG240617', 'MDN with error disposition.'
, 'An MDN with a disposition error of [{0}] is processed. The MDN document reference id is {1}, and AS Message Id is {2}.'
, 'event.name.240617', 'event.text.240617'
, 7, 3, 1
, 'This event is generated when a MDN is received with error disposition'
, 1
, 2
, 3
From DUAL
Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240617)
/

--47940 changed severity to 9
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240700, 'BCG240700', 'Workflow Handler Runtime Error'
        , 'Runtime exception in workflow handler execution {0}'
        , 'event.name.240700', 'event.text.240700'
        ,7,9,0
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd WHERE EVENTCD = 240700)
/

--47121
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 240701, 'BCG240701', 'Activity Logging Error'
        , 'Error occured while logging activity details {0}'
        , 'event.name.240701', 'event.text.240701'
        ,7,6,0
        , '{0}'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd WHERE EVENTCD = 240701)
/

Update lg_EventCd
   Set InternalDesc = 'The password provided does not meet minimum length requirement as defined in the password policy'
     , EventName    = 'Password Under Minimum Length'
     , EventNameKey = 'event.name.101000'
     , TextKey      = 'event.text.101000'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the user enters a password that does not meet the minimum length specified in the password policy.  Reference the password policy.'
 Where EventCd = 101000
/

Update lg_EventCd
   Set InternalDesc = 'The password supplied has already been used by this user'
     , EventName    = 'Password Already Exists'
     , EventNameKey = 'event.name.101001'
     , TextKey      = 'event.text.101001'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the user tries to update the user profile using a password that exists in the users password history list.  Reference the  password policy.'
 Where EventCd = 101001
/

Update lg_EventCd
   Set InternalDesc = 'An attempt was made to create a duplicate participant name: {0}'
     , EventName    = 'Participant Name Already Exists'
     , EventNameKey = 'event.name.101002'
     , TextKey      = 'event.text.101002'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the user tries to create a participant using the same name as an existing participant.'
 Where EventCd = 101002
/

Update lg_EventCd
   Set InternalDesc = 'An attempt was made to create a duplicate connection'
     , EventName    = 'Connection Already Exists'
     , EventNameKey = 'event.name.101003'
     , TextKey      = 'event.text.101003'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the user tries to create a connection with the same setup as an existing connection.'
 Where EventCd = 101003
/

Update lg_EventCd
   Set InternalDesc = 'An attempt was made to create a Community Operator when one already exists'
     , EventName    = 'Community Operator Already Exists'
     , EventNameKey = 'event.name.101004'
     , TextKey      = 'event.text.101004'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the user tries to create a second Community Operator.  The system will only allow one Community Operator to exist for each hub.  The Community Operator is automatically created when the hub is initialized.'
 Where EventCd = 101004
/

Update lg_EventCd
   Set InternalDesc = 'An attempt was made to create a Community Manager when one already exists'
     , EventName    = 'Community Manager Already Exists'
     , EventNameKey = 'event.name.101005'
     , TextKey      = 'event.text.101005'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the user tried to create a second Community Manager.  Only one Community Manager can be created per hub.'
 Where EventCd = 101005
/

Update lg_EventCd
   Set InternalDesc = 'The password provided does not meet the requirements defined in the password policy because it does not contain 3 of the 4 special characters'
     , EventName    = 'Password Missing Special Characters'
     , EventNameKey = 'event.name.101006'
     , TextKey      = 'event.text.101006'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'If the password policy''s "special characters" flag is set, the system will generate this event whenever a user tries to update a user profile with a password that does not contain the valid characters.When the flag is set to true, the password must contain at least 3 of the following 4 types of characters - uppercase, lowercase, numeric, and non-alphanumeric characters (e.g., #&*, etc.)'
 Where EventCd = 101006
/

Update lg_EventCd
   Set InternalDesc = 'The password provided contains a variation of the user name'
     , EventName    = 'Password Contains User Name'
     , EventNameKey = 'event.name.101007'
     , TextKey      = 'event.text.101007'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'When the password policy''s "name variation" flag is set to true, the system will generate this event whenever a user tries to update a user profile with a password that contains a variation of the user''s name.'
 Where EventCd = 101007
/

Update lg_EventCd
   Set InternalDesc = 'An attempt to logon to the hub has failed due to invalid user name or password'
     , EventName    = 'User Login failed'
     , EventNameKey = 'event.name.102001'
     , TextKey      = 'event.text.102001'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated each time a user login attempt has failed.'
 Where EventCd = 102001
/

Update lg_EventCd
   Set InternalDesc = 'User {0} has successfully logged onto the system'
     , EventName    = 'User Login was successful'
     , EventNameKey = 'event.name.102002'
     , TextKey      = 'event.text.102002'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated each time a user has successfully logged onto the console.'
 Where EventCd = 102002
/

Update lg_EventCd
   Set InternalDesc = 'Hub initialized: hub admin and Community Operator have been successfully created'
     , EventName    = 'Hub Initialized'
     , EventNameKey = 'event.name.102003'
     , TextKey      = 'event.text.102003'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'Before the hub can be used, it must be initialized by creating the hub admin, Community Operator, and Operator''s admin, default user groups and permissions.  This event is generated when a user requests the system to initialize.'
 Where EventCd = 102003
/

Update lg_EventCd
   Set InternalDesc = 'Database Error: {0} failed in {1} with exception {3}'
     , EventName    = 'Database Failure'
     , EventNameKey = 'event.name.103001'
     , TextKey      = 'event.text.103001'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the database generates an error. This, in turn, causes the system to terminate its process and react by throwing its own exception. The end result of this event is dependent on where the error occurred.'
 Where EventCd = 103001
/

Update lg_EventCd
   Set InternalDesc = 'Publish XML Format: Request the system to reload the XML formats'
     , EventName    = 'XML Format Published'
     , EventNameKey = 'event.name.103004'
     , TextKey      = 'event.text.103004'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the XML Formats are published and reloaded by the system.'
 Where EventCd = 103004
/

Update lg_EventCd
   Set InternalDesc = 'Publish Validation Map: Request the system to reload the validation maps'
     , EventName    = 'Validation Map Published'
     , EventNameKey = 'event.name.103005'
     , TextKey      = 'event.text.103005'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the Validation Maps are published and reloaded by the system.'
 Where EventCd = 103005
/

Update lg_EventCd
   Set InternalDesc = 'Publish XSLT Transformation Map: Request the system to reload the XSLT transformation maps'
     , EventName    = 'XSLT Transformation Map Published'
     , EventNameKey = 'event.name.103006'
     , TextKey      = 'event.text.103006'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the XSLT Transformation Maps are published and reloaded by the system.'
 Where EventCd = 103006
/

Update lg_EventCd
   Set InternalDesc = 'Publish RosettaNet process information:  Request the system to reload the RosettaNet process information'
     , EventName    = 'RosettaNet Process Information Published'
     , EventNameKey = 'event.name.103007'
     , TextKey      = 'event.text.103007'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when RosettaNet process information is published and reloaded by the system.'
 Where EventCd = 103007
/

Update lg_EventCd
   Set InternalDesc = 'Published Target Configurations: Request Targets to reload the configuration'
     , EventName    = 'Target Configuration Published'
     , EventNameKey = 'event.name.103009'
     , TextKey      = 'event.text.103009'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the Target configurations are published and reloaded by the system.'
 Where EventCd = 103009
/

Update lg_EventCd
   Set InternalDesc = 'Published Synchronous HTTP/S Configuration: Request CacheEngine to reload the configuration'
     , EventName    = 'Synchronous HTTP/S Configuration Published'
     , EventNameKey = 'event.name.103010'
     , TextKey      = 'event.text.103010'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the Synchronous HTTP/S configurations  are published and reloaded by the CacheEngine.'
 Where EventCd = 103010
/

Update lg_EventCd
   Set InternalDesc = 'Published Attachment Content-Type Filter: Request CacheEngine to reload the filter list'
     , EventName    = 'Attachment Content-Type Filter Published'
     , EventNameKey = 'event.name.103011'
     , TextKey      = 'event.text.103011'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the Attachment Content Type Filter List is published and reloaded by the CacheEngine.'
 Where EventCd = 103011
/

Update lg_EventCd
   Set InternalDesc = 'Publish Web Services process information: Request the system to reload the Web Service process information.'
     , EventName    = 'Document Flow Definition Updated'
     , EventNameKey = 'event.name.103012'
     , TextKey      = 'event.text.103012'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the system publishes Web Services process information and requests the system to reload the Web Services process information.'
 Where EventCd = 103012
/

Update lg_EventCd
   Set InternalDesc = 'Cache Engine instanceId {0} on host {1} failed to initialized, please correct the problem and restart the service, error reason:{2}'
     , EventName    = 'Cache Engine Error'
     , EventNameKey = 'event.name.103101'
     , TextKey      = 'event.text.103101'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated each time the system encounters an error in the cache engine.'
 Where EventCd = 103101
/

Update lg_EventCd
   Set InternalDesc = 'Error Reason:{0}'
     , EventName    = 'Hub Owner State Engine Error'
     , EventNameKey = 'event.name.103201'
     , TextKey      = 'event.text.103201'
     , Visibility   = 3
     , Severity     = 6
     , ExternalDesc = 'Hub Owner State Engine Error'
 Where EventCd = 103201
/

Update lg_EventCd
   Set InternalDesc = 'The XML encoding ''{0}'' does not match to transport Content-Type charset ''{0}''.'
     , EventName    = 'XML Encoding Warning'
     , EventNameKey = 'event.name.103202'
     , TextKey      = 'event.text.103202'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'XML Encoding Warning'
 Where EventCd = 103202
/

Update lg_EventCd
   Set InternalDesc = 'Target ''{0},{1}'' failed to processing document, error: {2}.'
     , EventName    = 'Target Processing Error'
     , EventNameKey = 'event.name.103203'
     , TextKey      = 'event.text.103203'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'Target Processing Error'
 Where EventCd = 103203
/

Update lg_EventCd
   Set InternalDesc = 'Target ''{0},{1}'' processing document warning, reason: {2}.'
     , EventName    = 'Target Processing Warning'
     , EventNameKey = 'event.name.103204'
     , TextKey      = 'event.text.103204'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'Target Processing Warning'
 Where EventCd = 103204
/

Update lg_EventCd
   Set InternalDesc = 'Target ''{0},{1}'' failed to process target: {2}.'
     , EventName    = 'Target Error'
     , EventNameKey = 'event.name.103205'
     , TextKey      = 'event.text.103205'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'Target Processing Error'
 Where EventCd = 103205
/

Update lg_EventCd
   Set InternalDesc = 'When trying to create a target entry, the name already exists'
     , EventName    = 'Target with supplied name already exists'
     , EventNameKey = 'event.name.103206'
     , TextKey      = 'event.text.103206'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated each time a user attempts to create a target name that already exists'
 Where EventCd = 103206
/

Update lg_EventCd
   Set InternalDesc = 'Participant was successfully created: {0}'
     , EventName    = 'Participant Created'
     , EventNameKey = 'event.name.104001'
     , TextKey      = 'event.text.104001'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated each time a user successfully creates a new participant profile.'
 Where EventCd = 104001
/

Update lg_EventCd
   Set InternalDesc = 'Participant was successfully updated: {0}'
     , EventName    = 'Participant Updated'
     , EventNameKey = 'event.name.104002'
     , TextKey      = 'event.text.104002'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated each time a user successfully updates an existing participant profile.'
 Where EventCd = 104002
/

Update lg_EventCd
   Set InternalDesc = 'User was successfully created: {0}'
     , EventName    = 'User Created'
     , EventNameKey = 'event.name.104003'
     , TextKey      = 'event.text.104003'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated each time a user successfully creates a new user profile for an existing participant.'
 Where EventCd = 104003
/

Update lg_EventCd
   Set InternalDesc = 'User successfully updated: {0}'
     , EventName    = 'User Updated'
     , EventNameKey = 'event.name.104004'
     , TextKey      = 'event.text.104004'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated each time a user successfully updates an existing user''s profile.'
 Where EventCd = 104004
/

Update lg_EventCd
   Set InternalDesc = 'User disabled: {0}'
     , EventName    = 'User Disabled'
     , EventNameKey = 'event.name.104005'
     , TextKey      = 'event.text.104005'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a user disables an existing user. A disabled user cannot log onto the console.'
 Where EventCd = 104005
/

Update lg_EventCd
   Set InternalDesc = 'Participant disabled: {0}'
     , EventName    = 'Participant Disabled'
     , EventNameKey = 'event.name.104006'
     , TextKey      = 'event.text.104006'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a user disables an existing participant. When the participant has been disabled, all connections, gateways and users are also disabled.'
 Where EventCd = 104006
/

Update lg_EventCd
   Set InternalDesc = 'An attempt was made to create a duplicate Business ID: {0}'
     , EventName    = 'Business ID Already Exists'
     , EventNameKey = 'event.name.104007'
     , TextKey      = 'event.text.104007'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when a user attempts to update a participant profile with a Business ID that already exists. A Business ID is considered a duplicate when the type and ID match an existing ID.'
 Where EventCd = 104007
/

Update lg_EventCd
   Set InternalDesc = 'An attempt was made to create a duplicate user name: {0}'
     , EventName    = 'User Already Exists'
     , EventNameKey = 'event.name.104008'
     , TextKey      = 'event.text.104008'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the user attempts to create a user using a name that matches another user for the same participant.'
 Where EventCd = 104008
/

Update lg_EventCd
   Set InternalDesc = 'A group with the name "{0}" already exists for participant {1}'
     , EventName    = 'Group Name Already Exists'
     , EventNameKey = 'event.name.105000'
     , TextKey      = 'event.text.105000'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when a user attempts to create a group using a name that matches another group for the same participant.'
 Where EventCd = 105000
/

Update lg_EventCd
   Set InternalDesc = 'A connection was successfully created between participant {0} and participant {1}'
     , EventName    = 'Connection Created'
     , EventNameKey = 'event.name.106001'
     , TextKey      = 'event.text.106001'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a user successfully creates a new connection between two participants.'
 Where EventCd = 106001
/

Update lg_EventCd
   Set InternalDesc = 'A connection was successfully updated between participant {0} and participant {1}'
     , EventName    = 'Connection Updated'
     , EventNameKey = 'event.name.106002'
     , TextKey      = 'event.text.106002'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a user successfully updates an existing connection between two participants.'
 Where EventCd = 106002
/

Update lg_EventCd
   Set InternalDesc = 'A connection was disabled between participant {0} and participant {1}'
     , EventName    = 'Connection Disabled'
     , EventNameKey = 'event.name.106003'
     , TextKey      = 'event.text.106003'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a user disabled an existing connection between two participants.'
 Where EventCd = 106003
/

Update lg_EventCd
   Set InternalDesc = 'Connection create failed.  Pair of default gateways does not exist between participants: {0} and {1}'
     , EventName    = 'No Default Gateway Pair'
     , EventNameKey = 'event.name.106004'
     , TextKey      = 'event.text.106004'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system fails to create a connection because a pair of default gateways does not exist for the selected participants.'
 Where EventCd = 106004
/

Update lg_EventCd
   Set InternalDesc = 'A connection could not be created for the B2B capability because no actions are associated with the interaction.'
     , EventName    = 'No Action Found'
     , EventNameKey = 'event.name.106005'
     , TextKey      = 'event.text.106005'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system fails to create a connection for the B2B capability because no action is associated with the interaction.'
 Where EventCd = 106005
/

Update lg_EventCd
   Set InternalDesc = 'Gateway "{0}" was created for participant {1}'
     , EventName    = 'Gateway Created'
     , EventNameKey = 'event.name.106500'
     , TextKey      = 'event.text.106500'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a user successfully creates a new gateway.'
 Where EventCd = 106500
/

Update lg_EventCd
   Set InternalDesc = 'Gateway "{0}" was updated for participant {1}'
     , EventName    = 'Gateway Updated'
     , EventNameKey = 'event.name.106505'
     , TextKey      = 'event.text.106505'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a user successfully updates an existing gateway.'
 Where EventCd = 106505
/

Update lg_EventCd
   Set InternalDesc = 'Gateway could not be disabled.  It is a default gateway for at least one gateway type.'
     , EventName    = 'Gateway Disable Failed'
     , EventNameKey = 'event.name.106507'
     , TextKey      = 'event.text.106507'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when a user unsuccessfully disables a gateway.  It is a default gateway for at least one gateway type.'
 Where EventCd = 106507
/

Update lg_EventCd
   Set InternalDesc = 'Child level = {0} greater than or equal to parent level = {1}'
     , EventName    = 'Document Flow Definition Create Error'
     , EventNameKey = 'event.name.106600'
     , TextKey      = 'event.text.106600'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the user is not able to create a document flow definition.'
 Where EventCd = 106600
/

Update lg_EventCd
   Set InternalDesc = 'A JSP Error occurred:  {0} at {1}'
     , EventName    = 'Page Generated JSP Error'
     , EventNameKey = 'event.name.107001'
     , TextKey      = 'event.text.107001'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when a JSP page has generated an error.'
 Where EventCd = 107001
/

Update lg_EventCd
   Set InternalDesc = 'The file that was uploaded is either not a certificate or not a format currently supported.'
     , EventName    = 'Not a Certificate'
     , EventNameKey = 'event.name.108001'
     , TextKey      = 'event.text.108001'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated during the certificate creation process whenever the uploaded file is not recognized as a one of the supported certificates.'
 Where EventCd = 108001
/

Update lg_EventCd
   Set InternalDesc = 'Certificate {1} was successfully created for participant {0}'
     , EventName    = 'Certificate Created'
     , EventNameKey = 'event.name.108002'
     , TextKey      = 'event.text.108002'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a user successfully uploaded and saved a certificate for a participant.'
 Where EventCd = 108002
/

Update lg_EventCd
   Set InternalDesc = 'Certificate {1} was successfully updated for participant {0}'
     , EventName    = 'Certificate Updated'
     , EventNameKey = 'event.name.108003'
     , TextKey      = 'event.text.108003'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a user successfully updated an existing certificate for a participant.'
 Where EventCd = 108003
/

Update lg_EventCd
   Set InternalDesc = 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
     , EventName    = 'Certificate Expiration in 60 Days'
     , EventNameKey = 'event.name.108005'
     , TextKey      = 'event.text.108005'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'The certificate will expire in 60 days and should be replaced.'
 Where EventCd = 108005
/

Update lg_EventCd
   Set InternalDesc = 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
     , EventName    = 'Certificate Expiration in 30 Days'
     , EventNameKey = 'event.name.108006'
     , TextKey      = 'event.text.108006'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'The certificate will expire in 30 days and should be replaced.'
 Where EventCd = 108006
/

Update lg_EventCd
   Set InternalDesc = 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
     , EventName    = 'Certificate Expiration in 15 Days'
     , EventNameKey = 'event.name.108007'
     , TextKey      = 'event.text.108007'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'The certificate will expire in 15 days and should be replaced.'
 Where EventCd = 108007
/

Update lg_EventCd
   Set InternalDesc = 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
     , EventName    = 'Certificate Expiration in 7 Days'
     , EventNameKey = 'event.name.108008'
     , TextKey      = 'event.text.108008'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'The certificate will expire in 7 days and should be replaced.'
 Where EventCd = 108008
/

Update lg_EventCd
   Set InternalDesc = 'Certificate {0}, type {1}, for participant {2}, issued to {3}, issued by {6}, serial number = {7}, created on {4}, will expire on {5}'
     , EventName    = 'Certificate Expiration in 2 Days'
     , EventNameKey = 'event.name.108009'
     , TextKey      = 'event.text.108009'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'The certificate will expire in 2 days and should be replaced.'
 Where EventCd = 108009
/

Update lg_EventCd
   Set InternalDesc = 'Installed certificates were checked for approaching expiration date'
     , EventName    = 'Certificate Expiration Check'
     , EventNameKey = 'event.name.108010'
     , TextKey      = 'event.text.108010'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the system checks the expiration date of installed certificates.'
 Where EventCd = 108010
/

Update lg_EventCd
   Set InternalDesc = 'Activation step {0} was completed for participant {1} by {2} on {3}'
     , EventName    = 'Activation Step Completed'
     , EventNameKey = 'event.name.109001'
     , TextKey      = 'event.text.109001'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated each time an Activation step is completed by a Participant.'
 Where EventCd = 109001
/

Update lg_EventCd
   Set InternalDesc = 'Activation phase {0} for participant {1}  is past its expected completion date of {2}'
     , EventName    = 'Activation Phase Late'
     , EventNameKey = 'event.name.109002'
     , TextKey      = 'event.text.109002'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated each time an Activation phase is past due for a Participant.'
 Where EventCd = 109002
/

Update lg_EventCd
   Set InternalDesc = 'Cached object ''{0}'' on instanceId ''{1}'' of host ''{2}'', error reason:{3}'
     , EventName    = 'Cache Refresh Internal Error'
     , EventNameKey = 'event.name.110001'
     , TextKey      = 'event.text.110001'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the console refresh generates an error.'
 Where EventCd = 110001
/

Update lg_EventCd
   Set InternalDesc = 'Cached object={0} on cache engine instance={1} of host={2} refreshed successfully.'
     , EventName    = 'Cached Object Refreshed Successfully'
     , EventNameKey = 'event.name.110002'
     , TextKey      = 'event.text.110002'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the console refresh is successful.'
 Where EventCd = 110002
/

Update lg_EventCd
   Set InternalDesc = 'FTP account create failed for participant {0}. Error message: {1}'
     , EventName    = 'FTP Account Create Error'
     , EventNameKey = 'event.name.111001'
     , TextKey      = 'event.text.111001'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system fails to create a FTP Account.'
 Where EventCd = 111001
/

Update lg_EventCd
   Set InternalDesc = 'An FTP account for participant {0} has been created'
     , EventName    = 'FTP Account Created'
     , EventNameKey = 'event.name.111002'
     , TextKey      = 'event.text.111002'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the system successfully creates a FTP Account.'
 Where EventCd = 111002
/

Update lg_EventCd
   Set InternalDesc = 'The  FTP account for participant {0} has been updated.'
     , EventName    = 'FTP Account Updated'
     , EventNameKey = 'event.name.111003'
     , TextKey      = 'event.text.111003'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the FTP Account is updated.'
 Where EventCd = 111003
/

Update lg_EventCd
   Set InternalDesc = 'Failed to create directory: {0}'
     , EventName    = 'Failed to Create Directory'
     , EventNameKey = 'event.name.112002'
     , TextKey      = 'event.text.112002'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system fails to create a directory.'
 Where EventCd = 112002
/

Update lg_EventCd
   Set InternalDesc = 'Document root directory {0} already exists'
     , EventName    = 'Document Root Directory Exists'
     , EventNameKey = 'event.name.112003'
     , TextKey      = 'event.text.112003'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the document root directory already exists.'
 Where EventCd = 112003
/

Update lg_EventCd
   Set InternalDesc = 'Target directory {0}  created for Target {1}'
     , EventName    = 'Target Directory Structure Created'
     , EventNameKey = 'event.name.112006'
     , TextKey      = 'event.text.112006'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the Target directory structure is created.'
 Where EventCd = 112006
/

Update lg_EventCd
   Set InternalDesc = 'An attempt was made to create a duplicate participant name: {0}'
     , EventName    = 'Participant Name Already Exists'
     , EventNameKey = 'event.name.123456'
     , TextKey      = 'event.text.123456'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the user tries to create a participant using the same name as an existing participant.'
 Where EventCd = 123456
/

Update lg_EventCd
   Set InternalDesc = 'Factory successfully got an instance of the protocol transformer ({0}) business process'
     , EventName    = 'Get Protocol Transformer Business Process Successful'
     , EventNameKey = 'event.name.200000'
     , TextKey      = 'event.text.200000'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the factory successfully gets an instance of the transformer business process.'
 Where EventCd = 200000
/

Update lg_EventCd
   Set InternalDesc = 'Factory failed to get an instance of the protocol transformer business process because {0}'
     , EventName    = 'Get Protocol Transformer Business Process Failed'
     , EventNameKey = 'event.name.200001'
     , TextKey      = 'event.text.200001'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated due to system failure when attempting to locate an instance of the protocol transformer business process.'
 Where EventCd = 200001
/

Update lg_EventCd
   Set InternalDesc = 'Protocol transformer business process ({0}) entrance'
     , EventName    = 'Protocol Transformer Entrance'
     , EventNameKey = 'event.name.200002'
     , TextKey      = 'event.text.200002'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when a document enters transformation.'
 Where EventCd = 200002
/

Update lg_EventCd
   Set InternalDesc = 'Protocol transformer business process ({0}) exit'
     , EventName    = 'Protocol Transformer Exit'
     , EventNameKey = 'event.name.200003'
     , TextKey      = 'event.text.200003'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when a document exits transformation.'
 Where EventCd = 200003
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Document Successfully Transformed'
     , EventNameKey = 'event.name.200004'
     , TextKey      = 'event.text.200004'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a document is successfully transformed.'
 Where EventCd = 200004
/

Update lg_EventCd
   Set InternalDesc = 'Document failed transformation due to {0}'
     , EventName    = 'Document Transformation Failure'
     , EventNameKey = 'event.name.200005'
     , TextKey      = 'event.text.200005'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated due to a failure during document transformation.'
 Where EventCd = 200005
/

Update lg_EventCd
   Set InternalDesc = 'Protocol transformer input file error: {0}'
     , EventName    = 'Protocol Transformer Input File Failure'
     , EventNameKey = 'event.name.200006'
     , TextKey      = 'event.text.200006'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated due to a failure with the input file during action processing.  For example, the file is corrupted.'
 Where EventCd = 200006
/

Update lg_EventCd
   Set InternalDesc = 'Protocol transformer output file error: {0}'
     , EventName    = 'Protocol Transformer Output File Failure'
     , EventNameKey = 'event.name.200007'
     , TextKey      = 'event.text.200007'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated due to a failure when attempting to write to the output file directory.'
 Where EventCd = 200007
/

Update lg_EventCd
   Set InternalDesc = 'Failed to parse: {0}'
     , EventName    = 'Failed to Parse Document'
     , EventNameKey = 'event.name.200009'
     , TextKey      = 'event.text.200009'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated due to failure when attempting to parse the document.'
 Where EventCd = 200009
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Community Manager Provided RN Process-Instance-ID Error'
     , EventNameKey = 'event.name.200013'
     , TextKey      = 'event.text.200013'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when an invalid Process Instance ID is received and the configuration property indicates that the system will not generate a new Process Instance ID.'
 Where EventCd = 200013
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Community Manager Provided RN Process-Instance-ID Warning'
     , EventNameKey = 'event.name.200014'
     , TextKey      = 'event.text.200014'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when an invalid Process Instance ID is received and the configuration property indicates that the system will generate a new Process Instance ID.'
 Where EventCd = 200014
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Community Manager Provided RosettaNet GlobalUsageCode Error'
     , EventNameKey = 'event.name.200015'
     , TextKey      = 'event.text.200015'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the x-aux-production header value is invalid and the configuration property indicates that the system will not use the default value on error.'
 Where EventCd = 200015
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Community Manager Provided RosettaNet GlobalUsageCode Warnin'
     , EventNameKey = 'event.name.200016'
     , TextKey      = 'event.text.200016'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the received x-aux-production header value is invalid and the configuration property indicates that the system will use the default value on error.'
 Where EventCd = 200016
/

Update lg_EventCd
   Set InternalDesc = 'Community Manager Provided RN Process-Instance-ID verification error: {0}'
     , EventName    = 'Community Manager Provided RN Process-Instance-ID Error'
     , EventNameKey = 'event.name.200017'
     , TextKey      = 'event.text.200017'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when WBIC failed to verify the Process Instance ID because of system error.'
 Where EventCd = 200017
/

Update lg_EventCd
   Set InternalDesc = 'System Error in Channel Check Process'
     , EventName    = 'System Error in Channel Check Process'
     , EventNameKey = 'event.name.210000'
     , TextKey      = 'event.text.210000'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when there is a check channel related error.'
 Where EventCd = 210000
/

Update lg_EventCd
   Set InternalDesc = 'Check Channel Error'
     , EventName    = 'Check Channel Error'
     , EventNameKey = 'event.name.210001'
     , TextKey      = 'event.text.210001'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when data required to lookup a connection is available but the matching connection is not found.'
 Where EventCd = 210001
/

Update lg_EventCd
   Set InternalDesc = 'Connection lookup failed {0}'
     , EventName    = 'Connection Lookup Failed'
     , EventNameKey = 'event.name.210002'
     , TextKey      = 'event.text.210002'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when data required to lookup a connection is not available.'
 Where EventCd = 210002
/

Update lg_EventCd
   Set InternalDesc = 'Document sent to the Business Process Engine'
     , EventName    = 'Document Sent to BPE'
     , EventNameKey = 'event.name.210003'
     , TextKey      = 'event.text.210003'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated after the inbound processor finished processing the document and successfully puts into the queue for the Business Process Engine.'
 Where EventCd = 210003
/

Update lg_EventCd
   Set InternalDesc = 'Document was rejected'
     , EventName    = 'Document Rejected'
     , EventNameKey = 'event.name.210004'
     , TextKey      = 'event.text.210004'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated upon initial document receipt to indicate that the document was rejected due to an unreadable or unsupported document type.'
 Where EventCd = 210004
/

Update lg_EventCd
   Set InternalDesc = 'Document sent to Outbound Processor'
     , EventName    = 'Document Sent to Outbound Processor'
     , EventNameKey = 'event.name.210005'
     , TextKey      = 'event.text.210005'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated after the Business Process Engine processed the document and successfully puts into the queue for the outbound processor.'
 Where EventCd = 210005
/

Update lg_EventCd
   Set InternalDesc = 'Document sent to Delivery Manager'
     , EventName    = 'Document Sent to Delivery Manager'
     , EventNameKey = 'event.name.210006'
     , TextKey      = 'event.text.210006'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated after the outbound processor processed the document and successfully puts into the queue for the delivery manager.'
 Where EventCd = 210006
/

Update lg_EventCd
   Set InternalDesc = 'Error in Outbound Processor'
     , EventName    = 'Outbound Document Cannot be Packaged'
     , EventNameKey = 'event.name.210007'
     , TextKey      = 'event.text.210007'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when a packager is not available for an outbound document.'
 Where EventCd = 210007
/

Update lg_EventCd
   Set InternalDesc = 'From IP address is not in the participant profile {0}'
     , EventName    = 'IP Address Validation Failure'
     , EventNameKey = 'event.name.210008'
     , TextKey      = 'event.text.210008'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when  a document is posted from an unapproved IP Address for that participant.'
 Where EventCd = 210008
/

Update lg_EventCd
   Set InternalDesc = 'Client SSL certificate name is not in the participant profile {0}'
     , EventName    = 'SSL Certificate Validation Failure'
     , EventNameKey = 'event.name.210009'
     , TextKey      = 'event.text.210009'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the SSL Certificate used to post the document is not in the approved certificate list for that participant.'
 Where EventCd = 210009
/

Update lg_EventCd
   Set InternalDesc = 'Document too large: {0} bytes'
     , EventName    = 'Document Too Large'
     , EventNameKey = 'event.name.210010'
     , TextKey      = 'event.text.210010'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the document received is too large to be processed.'
 Where EventCd = 210010
/

Update lg_EventCd
   Set InternalDesc = 'Insufficient Community Manager transport information provided: {0}'
     , EventName    = 'Community Manager Transport Unpackage Failure'
     , EventNameKey = 'event.name.210011'
     , TextKey      = 'event.text.210011'
     , Visibility   = 3
     , Severity     = 6
     , ExternalDesc = 'This event is generated when insufficient transport information is provided.'
 Where EventCd = 210011
/

Update lg_EventCd
   Set InternalDesc = 'B2B capability not found {0}'
     , EventName    = 'B2B Capability Not Found'
     , EventNameKey = 'event.name.210012'
     , TextKey      = 'event.text.210012'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the B2B capability required to route the document is not enabled.'
 Where EventCd = 210012
/

Update lg_EventCd
   Set InternalDesc = 'Connection not fully configured {0}'
     , EventName    = 'Connection Not Fully Configured'
     , EventNameKey = 'event.name.210013'
     , TextKey      = 'event.text.210013'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the connection for the document is not fully configured.  Most likely the destination for the document does not have a configured gateway.'
 Where EventCd = 210013
/

Update lg_EventCd
   Set InternalDesc = 'Failed to unpackage a MIME multipart document: {0}'
     , EventName    = 'MIME Multipart Unpackaging Failure'
     , EventNameKey = 'event.name.210014'
     , TextKey      = 'event.text.210014'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system failed to unpackage a MIME multipart document.'
 Where EventCd = 210014
/

Update lg_EventCd
   Set InternalDesc = 'Failed to packaing a cXML document: {0}'
     , EventName    = 'cXML Packaging Failure'
     , EventNameKey = 'event.name.210015'
     , TextKey      = 'event.text.210015'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'cXML Packaging Failure'
 Where EventCd = 210015
/

Update lg_EventCd
   Set InternalDesc = 'Failed to parse cXML routing information: {0}'
     , EventName    = 'cXML Channel Parse Failure'
     , EventNameKey = 'event.name.210016'
     , TextKey      = 'event.text.210016'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'cXML Channel Parse Failure'
 Where EventCd = 210016
/

Update lg_EventCd
   Set InternalDesc = 'Failed to parse EDI routing information: {0}'
     , EventName    = 'EDI Connection Parse Failure'
     , EventNameKey = 'event.name.210017'
     , TextKey      = 'event.text.210017'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system failed to parse EDI routing information.'
 Where EventCd = 210017
/

Update lg_EventCd
   Set InternalDesc = 'User defined duplicate check field(s) not found in document: {0}'
     , EventName    = 'User Defined Duplicate Check Field(s) not Found'
     , EventNameKey = 'event.name.210018'
     , TextKey      = 'event.text.210018'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when user defined duplicate check fields are not found.'
 Where EventCd = 210018
/

Update lg_EventCd
   Set InternalDesc = 'Synchronous Operation not Supported on this Connection'
     , EventName    = 'Synchronous Operation not Supported on this Connection'
     , EventNameKey = 'event.name.210019'
     , TextKey      = 'event.text.210019'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the document requests synchronous operation but the connection does not support synchronous operations.'
 Where EventCd = 210019
/

Update lg_EventCd
   Set InternalDesc = 'Error encountered while moving a document to its working folder'
     , EventName    = 'Document Moving Error'
     , EventNameKey = 'event.name.210020'
     , TextKey      = 'event.text.210020'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when the system encounters an error while trying to move a document into its working folder.  The system will re-try when this occurs.'
 Where EventCd = 210020
/

Update lg_EventCd
   Set InternalDesc = 'The Operator requested that processing of this document be cancelled'
     , EventName    = 'Processing cancelled by user'
     , EventNameKey = 'event.name.210021'
     , TextKey      = 'event.text.210021'
     , Visibility   = 3
     , Severity     = 9
     , ExternalDesc = 'The Operator requested that processing of this document be cancelled.'
 Where EventCd = 210021
/

Update lg_EventCd
   Set InternalDesc = 'An error occurred and the processing transaction has rolled back.  The time in process was {0} seconds.'
     , EventName    = 'Process transaction rolled back.'
     , EventNameKey = 'event.name.210022'
     , TextKey      = 'event.text.210022'
     , Visibility   = 3
     , Severity     = 9
     , ExternalDesc = 'An error occurred and the processing transaction has rolled back.  The time in process was {0} seconds.'
 Where EventCd = 210022
/

Update lg_EventCd
   Set InternalDesc = 'Files missing for doc {0}'
     , EventName    = 'Not all Files Present for Document'
     , EventNameKey = 'event.name.210030'
     , TextKey      = 'event.text.210030'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when files are missing for the document.'
 Where EventCd = 210030
/

Update lg_EventCd
   Set InternalDesc = 'Unable to Non-Rep document {0}'
     , EventName    = 'Unable to Non-Rep document'
     , EventNameKey = 'event.name.210031'
     , TextKey      = 'event.text.210031'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the system is unable to non-repudiate the document.'
 Where EventCd = 210031
/

Update lg_EventCd
   Set InternalDesc = 'System error in the Inbound Processor for document: {0}'
     , EventName    = 'System Error in the Inbound Processor'
     , EventNameKey = 'event.name.210032'
     , TextKey      = 'event.text.210032'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the system encounters an error in the inbound processor.'
 Where EventCd = 210032
/

Update lg_EventCd
   Set InternalDesc = 'Unable to store document plain text: {0}'
     , EventName    = 'Message Store Failed'
     , EventNameKey = 'event.name.210033'
     , TextKey      = 'event.text.210033'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system is unable to store the document in plain text.'
 Where EventCd = 210033
/

Update lg_EventCd
   Set InternalDesc = 'System error in the BPE for document: {0}'
     , EventName    = 'System Error in the BPE'
     , EventNameKey = 'event.name.210034'
     , TextKey      = 'event.text.210034'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the system encounters an error in the BPE.'
 Where EventCd = 210034
/

Update lg_EventCd
   Set InternalDesc = 'Passed Duplicate Check'
     , EventName    = 'Duplicate Check Successful'
     , EventNameKey = 'event.name.210050'
     , TextKey      = 'event.text.210050'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when a document successfully passes duplicate checking.'
 Where EventCd = 210050
/

Update lg_EventCd
   Set InternalDesc = 'System error - failure in duplicate process'
     , EventName    = 'Duplicate Processing Failure'
     , EventNameKey = 'event.name.210051'
     , TextKey      = 'event.text.210051'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the system is unable to contact the database server during duplicate processing.'
 Where EventCd = 210051
/

Update lg_EventCd
   Set InternalDesc = 'This document appears to be a duplication of a document sent on {2}'
     , EventName    = 'Duplicate Document Received'
     , EventNameKey = 'event.name.210052'
     , TextKey      = 'event.text.210052'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when a document received is a duplicate and rejected.'
 Where EventCd = 210052
/

Update lg_EventCd
   Set InternalDesc = 'Passed destination parse'
     , EventName    = 'Destination Parse Successful'
     , EventNameKey = 'event.name.210060'
     , TextKey      = 'event.text.210060'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a document successfully passes destination parse.'
 Where EventCd = 210060
/

Update lg_EventCd
   Set InternalDesc = 'Error in destination Parse'
     , EventName    = 'Destination Parse Failure'
     , EventNameKey = 'event.name.210061'
     , TextKey      = 'event.text.210061'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when destination parse fails.  Usually due to a database problem.'
 Where EventCd = 210061
/

Update lg_EventCd
   Set InternalDesc = 'Destination Process successful'
     , EventName    = 'Destination Process Successful'
     , EventNameKey = 'event.name.210062'
     , TextKey      = 'event.text.210062'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when destination processing is successful.'
 Where EventCd = 210062
/

Update lg_EventCd
   Set InternalDesc = 'Destination Process failed'
     , EventName    = 'Destination Process Failure'
     , EventNameKey = 'event.name.210063'
     , TextKey      = 'event.text.210063'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when destination processing fails.  Usually due to a database problem.'
 Where EventCd = 210063
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Destination Determination Failure'
     , EventNameKey = 'event.name.210065'
     , TextKey      = 'event.text.210065'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when there are conflicting inputs when processing  the destination.'
 Where EventCd = 210065
/

Update lg_EventCd
   Set InternalDesc = 'From Partner ID = {0}, To Partner ID = {1}, From Package Partner ID = {2}, To Package Partner ID = {3}'
     , EventName    = 'Package and Content Business Id''s map to different partners.'
     , EventNameKey = 'event.name.210066'
     , TextKey      = 'event.text.210066'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when there is a mismatch between the content and package routing information'
 Where EventCd = 210066
/

Update lg_EventCd
   Set InternalDesc = 'Timing event - Start {0}'
     , EventName    = 'Timing Event - Start'
     , EventNameKey = 'event.name.210100'
     , TextKey      = 'event.text.210100'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated at the start of each action step.'
 Where EventCd = 210100
/

Update lg_EventCd
   Set InternalDesc = 'Timing event - Finish'
     , EventName    = 'Timing Event - Finish'
     , EventNameKey = 'event.name.210101'
     , TextKey      = 'event.text.210101'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated at the end of each action step.'
 Where EventCd = 210101
/

Update lg_EventCd
   Set InternalDesc = 'Doctype processing completed successfully'
     , EventName    = 'Doctype Processing Completed Successfully'
     , EventNameKey = 'event.name.210200'
     , TextKey      = 'event.text.210200'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when RN Service Content is received and the system successfully inserts the DocType tag.'
 Where EventCd = 210200
/

Update lg_EventCd
   Set InternalDesc = 'Unable to load PIP for a document during Doctype processing'
     , EventName    = 'PIP Load During Doctype Processing Failure'
     , EventNameKey = 'event.name.210201'
     , TextKey      = 'event.text.210201'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when a spec for the PIP cannot be found.  Should not occur unless there is a configuration problem.'
 Where EventCd = 210201
/

Update lg_EventCd
   Set InternalDesc = 'Exception during Doctype Processing: {0}'
     , EventName    = 'Exception in Doctype Processing'
     , EventNameKey = 'event.name.210202'
     , TextKey      = 'event.text.210202'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the system fails when attempting to insert the DocType tag.'
 Where EventCd = 210202
/

Update lg_EventCd
   Set InternalDesc = 'DoctypeProcess Error - No action found'
     , EventName    = 'DoctypeProcess Error - No Action Found'
     , EventNameKey = 'event.name.210203'
     , TextKey      = 'event.text.210203'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when a spec for the PIP DocType cannot be found.'
 Where EventCd = 210203
/

Update lg_EventCd
   Set InternalDesc = 'Validation business process ({0}) entrance'
     , EventName    = 'Validation Business Process Entrance'
     , EventNameKey = 'event.name.230000'
     , TextKey      = 'event.text.230000'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when a document enters validation of the  business process.'
 Where EventCd = 230000
/

Update lg_EventCd
   Set InternalDesc = 'Field Validation Error'
     , EventName    = 'Field Validation Error'
     , EventNameKey = 'event.name.230001'
     , TextKey      = 'event.text.230001'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'System description of location of validation errors so that the console can highlight for data validation.'
 Where EventCd = 230001
/

Update lg_EventCd
   Set InternalDesc = 'Parser Error'
     , EventName    = 'Parser Error'
     , EventNameKey = 'event.name.230002'
     , TextKey      = 'event.text.230002'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'System description of location of validation errors so that the console can highlight for data validation.'
 Where EventCd = 230002
/

Update lg_EventCd
   Set InternalDesc = 'Validation business process ({0}) exit'
     , EventName    = 'Validation Business Process Exit'
     , EventNameKey = 'event.name.230003'
     , TextKey      = 'event.text.230003'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when the document exits business validation processing.'
 Where EventCd = 230003
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Validation Internal Error'
     , EventNameKey = 'event.name.230004'
     , TextKey      = 'event.text.230004'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated due to internal system failure during validation processing.'
 Where EventCd = 230004
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Validation Successful'
     , EventNameKey = 'event.name.230005'
     , TextKey      = 'event.text.230005'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a document successfully completes process validation.'
 Where EventCd = 230005
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Validation Database Error'
     , EventNameKey = 'event.name.230006'
     , TextKey      = 'event.text.230006'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated due to a database error during validation processing.'
 Where EventCd = 230006
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Validation Business Process Factory Error'
     , EventNameKey = 'event.name.230007'
     , TextKey      = 'event.text.230007'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the system is unable to determine the process to send to the validation engine.'
 Where EventCd = 230007
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Validation Warning'
     , EventNameKey = 'event.name.230008'
     , TextKey      = 'event.text.230008'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when a document is received and the initiating process cannot be found.'
 Where EventCd = 230008
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'RosettaNet Validation Error'
     , EventNameKey = 'event.name.230009'
     , TextKey      = 'event.text.230009'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when a document fails to complete RosettaNet process validation.'
 Where EventCd = 230009
/

Update lg_EventCd
   Set InternalDesc = 'Document failed data validation: {0}'
     , EventName    = 'Data Validation Error'
     , EventNameKey = 'event.name.230010'
     , TextKey      = 'event.text.230010'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when a document fails data validation and is rejected.'
 Where EventCd = 230010
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Document Sequence Validation Successful'
     , EventNameKey = 'event.name.230011'
     , TextKey      = 'event.text.230011'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a document successfully passes sequence validation.'
 Where EventCd = 230011
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'AS Sequence Validation Error'
     , EventNameKey = 'event.name.230012'
     , TextKey      = 'event.text.230012'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when a document fails to complete EDIINT process validation.'
 Where EventCd = 230012
/

Update lg_EventCd
   Set InternalDesc = 'Unable to log RN state for RNSC validation error for synchronous response.  Missing either initiate Business ID = {0} and/or Process Instance ID {1}'
     , EventName    = 'Failed to Log RN State due to Missing Key Data'
     , EventNameKey = 'event.name.230015'
     , TextKey      = 'event.text.230015'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the system fails to log the RN state because the Business ID or the Process Instance ID is missing.'
 Where EventCd = 230015
/

Update lg_EventCd
   Set InternalDesc = 'RosettaNet Unpackaging Error'
     , EventName    = 'RosettaNet Unpackaging Error'
     , EventNameKey = 'event.name.240003'
     , TextKey      = 'event.text.240003'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system is unable to parse the RosettaNet preamble during unpackaging.'
 Where EventCd = 240003
/

Update lg_EventCd
   Set InternalDesc = 'Delivery Header Parser Error: {0}'
     , EventName    = 'RNPackager Delivery Header Parser Failure'
     , EventNameKey = 'event.name.240005'
     , TextKey      = 'event.text.240005'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system is unable to parse the RosettaNet delivery header during unpackaging.'
 Where EventCd = 240005
/

Update lg_EventCd
   Set InternalDesc = 'Service Header parser error: {0}'
     , EventName    = 'RNPackager Service Header Failure'
     , EventNameKey = 'event.name.240007'
     , TextKey      = 'event.text.240007'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system is unable to parse the RosettaNet service header during unpackaging.'
 Where EventCd = 240007
/

Update lg_EventCd
   Set InternalDesc = 'Mime parsing error: {0}'
     , EventName    = 'RNPackager Mime Parsing Failure'
     , EventNameKey = 'event.name.240009'
     , TextKey      = 'event.text.240009'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when an error occurs in Mime parsing of the RosettaNet message during unpackaging.'
 Where EventCd = 240009
/

Update lg_EventCd
   Set InternalDesc = 'Digital Signature validation failed: {0}'
     , EventName    = 'RNPackager Signature Failed'
     , EventNameKey = 'event.name.240011'
     , TextKey      = 'event.text.240011'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when digital signature validation fails during unpackaging.'
 Where EventCd = 240011
/

Update lg_EventCd
   Set InternalDesc = 'Database access failure: Could not update the RosettaNet state'
     , EventName    = 'RN Unpackaging State Update Error'
     , EventNameKey = 'event.name.240012'
     , TextKey      = 'event.text.240012'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the unpackager encounters database communication errors when updating the RosettaNet state.'
 Where EventCd = 240012
/

Update lg_EventCd
   Set InternalDesc = 'Name/serial on signer certificate did not match database entry'
     , EventName    = 'Participant Certificate Did Not Match Signer'
     , EventNameKey = 'event.name.240013'
     , TextKey      = 'event.text.240013'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when Certificate to DUNS check fails for digital signature.'
 Where EventCd = 240013
/

Update lg_EventCd
   Set InternalDesc = 'Signature not found in document'
     , EventName    = 'Missing Signature in Document'
     , EventNameKey = 'event.name.240014'
     , TextKey      = 'event.text.240014'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when a signature is required by the TPA, but not found in the document.'
 Where EventCd = 240014
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'RosettaNet Document Creation Failure'
     , EventNameKey = 'event.name.240015'
     , TextKey      = 'event.text.240015'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when an attempt to construct a RosettaNet document fails.'
 Where EventCd = 240015
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'RosettaNet Non-Repudiation Error'
     , EventNameKey = 'event.name.240016'
     , TextKey      = 'event.text.240016'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the Receipt Ack does not contain correct digest of previous message, or the digest is missing.'
 Where EventCd = 240016
/

Update lg_EventCd
   Set InternalDesc = 'Synchronous acknowledgement is required but was not received in the synchronous response'
     , EventName    = 'Synchronous Acknowledgement Not Received'
     , EventNameKey = 'event.name.240017'
     , TextKey      = 'event.text.240017'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when a synchronous acknowledgement is required but was not received in the synchronous response.'
 Where EventCd = 240017
/

Update lg_EventCd
   Set InternalDesc = 'Digital Signature Key Not Loaded for Operator'
     , EventName    = 'Digital Signature Key Not Loaded for Operator'
     , EventNameKey = 'event.name.240018'
     , TextKey      = 'event.text.240018'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when digital signature keys are not loaded for the Community Operator.'
 Where EventCd = 240018
/

Update lg_EventCd
   Set InternalDesc = 'Encryption Key Not Loaded for Operator'
     , EventName    = 'Encryption Key Not Loaded for Operator'
     , EventNameKey = 'event.name.240019'
     , TextKey      = 'event.text.240019'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when encryption keys are not loaded for the Community Operator.'
 Where EventCd = 240019
/

Update lg_EventCd
   Set InternalDesc = 'Synchronous response is required but received an empty synchronous response content.'
     , EventName    = 'Synchronous Response Not Received'
     , EventNameKey = 'event.name.240020'
     , TextKey      = 'event.text.240020'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a synchronous acknowledgement is required and empty synchronous response was received'
 Where EventCd = 240020
/

Update lg_EventCd
   Set InternalDesc = 'Digital Signature Certificate has been revoked: {0}'
     , EventName    = 'Digital Signature Certificate has been revoked'
     , EventNameKey = 'event.name.240021'
     , TextKey      = 'event.text.240021'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when digital signature certificate has been revoked.'
 Where EventCd = 240021
/

Update lg_EventCd
   Set InternalDesc = 'Encryption Certificate has been revoked: {0}'
     , EventName    = 'Encryption Certificate has been revoked'
     , EventNameKey = 'event.name.240022'
     , TextKey      = 'event.text.240022'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when encryption certificate has been revoked.'
 Where EventCd = 240022
/

Update lg_EventCd
   Set InternalDesc = 'Error: {0}'
     , EventName    = 'Packaging Instance Error'
     , EventNameKey = 'event.name.240031'
     , TextKey      = 'event.text.240031'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the system is unable to find a packager for the supplied document type.'
 Where EventCd = 240031
/

Update lg_EventCd
   Set InternalDesc = 'XML parsing successful'
     , EventName    = 'XML Packager Successful'
     , EventNameKey = 'event.name.240034'
     , TextKey      = 'event.text.240034'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the system successfully completes XML parsing.'
 Where EventCd = 240034
/

Update lg_EventCd
   Set InternalDesc = 'Error: {0}'
     , EventName    = 'Unpackaging Instance Error'
     , EventNameKey = 'event.name.240036'
     , TextKey      = 'event.text.240036'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system cannot find an unpackager for a document.'
 Where EventCd = 240036
/

Update lg_EventCd
   Set InternalDesc = 'Factory using {0}'
     , EventName    = 'Connection Parse Factory Success'
     , EventNameKey = 'event.name.240060'
     , TextKey      = 'event.text.240060'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when the system successfully completes connection parsing.'
 Where EventCd = 240060
/

Update lg_EventCd
   Set InternalDesc = 'XML connection parsing failed: {0}'
     , EventName    = 'Connection Parse XML Failure'
     , EventNameKey = 'event.name.240065'
     , TextKey      = 'event.text.240065'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when connection info for an XML message could not be found.'
 Where EventCd = 240065
/

Update lg_EventCd
   Set InternalDesc = 'Connection Parse RosettaNet Failure'
     , EventName    = 'Connection Parser RosettaNet Failure'
     , EventNameKey = 'event.name.240068'
     , TextKey      = 'event.text.240068'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when connection info could not be found in a RosettaNet document.'
 Where EventCd = 240068
/

Update lg_EventCd
   Set InternalDesc = 'XML connection parse succeeded'
     , EventName    = 'XML Connection Parse Succeeded'
     , EventNameKey = 'event.name.240069'
     , TextKey      = 'event.text.240069'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the system successfully completes XML Connection parsing.'
 Where EventCd = 240069
/

Update lg_EventCd
   Set InternalDesc = 'XML connection parse failed Exception Message: {0}'
     , EventName    = 'XML Connection Parse Failure'
     , EventNameKey = 'event.name.240070'
     , TextKey      = 'event.text.240070'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system is unable to find connection information for an XML file.'
 Where EventCd = 240070
/

Update lg_EventCd
   Set InternalDesc = 'Flat File connection parse failed: {0}'
     , EventName    = 'Flat File Connection Parse Failure'
     , EventNameKey = 'event.name.240071'
     , TextKey      = 'event.text.240071'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system is unable to find connection information for a Flat File.'
 Where EventCd = 240071
/

Update lg_EventCd
   Set InternalDesc = 'MIME Document contains trailing data'
     , EventName    = 'MIME Structure Validation Warning'
     , EventNameKey = 'event.name.240071'
     , TextKey      = 'event.text.240071'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when extra data is found at the end of an RNIF 2.0 document and the system generates a MIME structure validation warning.'
 Where EventCd = 240072
/

Update lg_EventCd
   Set InternalDesc = 'Web Service connection parse succeeded'
     , EventName    = 'Web Service Connection Parse Succeeded'
     , EventNameKey = 'event.name.240077'
     , TextKey      = 'event.text.240077'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the system finds connection information for a SOAP message.'
 Where EventCd = 240077
/

--61817
--removed setting of InternalDesc = 'Web Service connection parse failed'
Update lg_EventCd
   Set 
       EventName    = 'Web Service Connection Parse Failed'
     , EventNameKey = 'event.name.240078'
     , TextKey      = 'event.text.240078'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system is unable to find connection information for a SOAP message.'
 Where EventCd = 240078
/

Update lg_EventCd
   Set InternalDesc = 'AS Unpackager Error: {0}'
     , EventName    = 'AS Unpackager Failure'
     , EventNameKey = 'event.name.240409'
     , TextKey      = 'event.text.240409'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the AS unpackager fails.'
 Where EventCd = 240409
/

Update lg_EventCd
   Set InternalDesc = 'AS Signature Validation Error: {0}'
     , EventName    = 'AS Signature Failure'
     , EventNameKey = 'event.name.240411'
     , TextKey      = 'event.text.240411'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when AS signature validation fails.'
 Where EventCd = 240411
/

Update lg_EventCd
   Set InternalDesc = 'AS State Engine DB error: {0}'
     , EventName    = 'AS State Engine DB Failure'
     , EventNameKey = 'event.name.240412'
     , TextKey      = 'event.text.240412'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the AS state engine database fails.'
 Where EventCd = 240412
/

Update lg_EventCd
   Set InternalDesc = 'AS Packager Error: {0}'
     , EventName    = 'AS Packager Failure'
     , EventNameKey = 'event.name.240415'
     , TextKey      = 'event.text.240415'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the AS packager fails.'
 Where EventCd = 240415
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'AS Non-Repudiation Error'
     , EventNameKey = 'event.name.240416'
     , TextKey      = 'event.text.240416'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when AS Non-Repudiation fails.'
 Where EventCd = 240416
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Decryption Failed'
     , EventNameKey = 'event.name.240417'
     , TextKey      = 'event.text.240417'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when decryption fails.'
 Where EventCd = 240417
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Unable to Generate Message Digest'
     , EventNameKey = 'event.name.240418'
     , TextKey      = 'event.text.240418'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system is unable to generate a message digest.'
 Where EventCd = 240418
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Unsupported Signature Format'
     , EventNameKey = 'event.name.240419'
     , TextKey      = 'event.text.240419'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system receives an unsupported signature format.'
 Where EventCd = 240419
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Unsupported Signature Algorithm'
     , EventNameKey = 'event.name.240420'
     , TextKey      = 'event.text.240420'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system receives unsupported signature algorithm.'
 Where EventCd = 240420
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Unexpected Error'
     , EventNameKey = 'event.name.240421'
     , TextKey      = 'event.text.240421'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the system encounters an unexpected error.'
 Where EventCd = 240421
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'AS document not found for this MDN'
     , EventNameKey = 'event.name.240422'
     , TextKey      = 'event.text.240422'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when a MDN is received and the system is unable to locate the corresponding document.'
 Where EventCd = 240422
/

Update lg_EventCd
   Set InternalDesc = 'Invalid input file passed in the document'
     , EventName    = 'Input File Failure'
     , EventNameKey = 'event.name.240423'
     , TextKey      = 'event.text.240423'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system encounters an invalid input file.'
 Where EventCd = 240423
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Insufficient Message Security'
     , EventNameKey = 'event.name.240424'
     , TextKey      = 'event.text.240424'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system encounters insufficient message security.'
 Where EventCd = 240424
/

Update lg_EventCd
   Set InternalDesc = 'MDN is supposed to be signed but not signed'
     , EventName    = 'MDN Not Signed'
     , EventNameKey = 'event.name.240425'
     , TextKey      = 'event.text.240425'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when an inbound MDN is supposed to be signed but not signed'
 Where EventCd = 240425
/

Update lg_EventCd
   Set InternalDesc = 'RosettaNet State Engine Error'
     , EventName    = 'RosettaNet State Engine Error'
     , EventNameKey = 'event.name.240500'
     , TextKey      = 'event.text.240500'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the RosettaNet State Engine encounters a system error.'
 Where EventCd = 240500
/

Update lg_EventCd
   Set InternalDesc = 'RosettaNet State Engine Generated 0A1 for PIP {0} Process Instance ID = {1}'
     , EventName    = 'RN State Engine Generated 0A1'
     , EventNameKey = 'event.name.240501'
     , TextKey      = 'event.text.240501'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the RosettaNet State Engine initiated a 0A1. (PIP is cancelled per RosettaNet standard)'
 Where EventCd = 240501
/

Update lg_EventCd
   Set InternalDesc = 'Error polling POP3 server: {0}; rejected message VUID: {1}'
     , EventName    = 'POP3 Poll Error'
     , EventNameKey = 'event.name.240550'
     , TextKey      = 'event.text.240550'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'Event generated when trying to poll a message from a POP3 server.'
 Where EventCd = 240550
/

Update lg_EventCd
   Set InternalDesc = 'AS State Engine Error: {0}'
     , EventName    = 'AS State Engine Error'
     , EventNameKey = 'event.name.240600'
     , TextKey      = 'event.text.240600'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the RosettaNet State Engine encounters a system error.'
 Where EventCd = 240600
/

Update lg_EventCd
   Set InternalDesc = 'AS Attribute max retry limit reached'
     , EventName    = 'AS Retry Failure'
     , EventNameKey = 'event.name.240601'
     , TextKey      = 'event.text.240601'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system fails AS retries. The maximum retry limit may have been reached.'
 Where EventCd = 240601
/

Update lg_EventCd
   Set InternalDesc = 'Document delivery to participant gateway failed: {0}'
     , EventName    = 'Document Delivery Failed'
     , EventNameKey = 'event.name.250001'
     , TextKey      = 'event.text.250001'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated  when document delivery to a participant''s gateway fails and the document is set to a failed state.'
 Where EventCd = 250001
/

Update lg_EventCd
   Set InternalDesc = 'An internal error occurred in the Delivery Scheduler: {0}'
     , EventName    = 'Delivery Scheduler Failed'
     , EventNameKey = 'event.name.250002'
     , TextKey      = 'event.text.250002'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when an uncategorized internal error occurred within the Delivery Manager, due to bad gateway or document data, rather than failure to deliver.'
 Where EventCd = 250002
/

Update lg_EventCd
   Set InternalDesc = 'Warning: {0}'
     , EventName    = 'Delivery Scheduler Warning'
     , EventNameKey = 'event.name.250003'
     , TextKey      = 'event.text.250003'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when a non-fatal abnormal condition occurred in the Delivery Manager, e.g. a retry failed but there are still more retries allowed.'
 Where EventCd = 250003
/

Update lg_EventCd
   Set InternalDesc = 'Document was delivered successfully, response: {0}'
     , EventName    = 'Document Delivered'
     , EventNameKey = 'event.name.250004'
     , TextKey      = 'event.text.250004'
     , Visibility   = 7
     , Severity     = 1
     , ExternalDesc = 'This event is generated when a document is successfully delivered.'
 Where EventCd = 250004
/

Update lg_EventCd
   Set InternalDesc = 'FTP delivery to participant gateway failed with exception: {0}'
     , EventName    = 'FTP Delivery Failed'
     , EventNameKey = 'event.name.250005'
     , TextKey      = 'event.text.250005'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'This event is generated  when the FTP protocol document delivery failed but more retries may be possible. Final failure will generate event 250001.'
 Where EventCd = 250005
/

Update lg_EventCd
   Set InternalDesc = 'Document delivered to directory: {0}'
     , EventName    = 'Document Delivered to Exception Directory'
     , EventNameKey = 'event.name.250006'
     , TextKey      = 'event.text.250006'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the document was delivered to the participant''s default "exception" directory, not a normal destination directory.'
 Where EventCd = 250006
/

Update lg_EventCd
   Set InternalDesc = 'Gateway {0} set offline automatically due to delivery failure'
     , EventName    = 'Auto Gateway Set Offline'
     , EventNameKey = 'event.name.250010'
     , TextKey      = 'event.text.250010'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the Gateway is automatically set offline due to failure to deliver.'
 Where EventCd = 250010
/

Update lg_EventCd
   Set InternalDesc = 'First delivery attempt failed for message {0} due to {1}, on gateway {2}'
     , EventName    = 'First Delivery Attempt Failed'
     , EventNameKey = 'event.name.250011'
     , TextKey      = 'event.text.250011'
     , Visibility   = 3
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the first attempt to deliver a document fails.'
 Where EventCd = 250011
/

Update lg_EventCd
   Set InternalDesc = '{0} retry {1} for message {2} failed due to {3}, on gateway {4}'
     , EventName    = 'Delivery Retry Failed'
     , EventNameKey = 'event.name.250012'
     , TextKey      = 'event.text.250012'
     , Visibility   = 3
     , Severity     = 3
     , ExternalDesc = 'This event is generated when a retry to deliver a document fails.'
 Where EventCd = 250012
/

Update lg_EventCd
   Set InternalDesc = 'Unsupported RosettaNet protocol version ({0}) for pass through process view logging'
     , EventName    = 'RosettaNet Pass Through Logging Warning'
     , EventNameKey = 'event.name.260001'
     , TextKey      = 'event.text.260001'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = 'This event is generated when unsupported RosettaNet protocol version is received.'
 Where EventCd = 260001
/

Update lg_EventCd
   Set InternalDesc = 'RosettaNet pass through process view logging failed: {0}'
     , EventName    = 'RosettaNet Pass Through Logging Failed'
     , EventNameKey = 'event.name.260002'
     , TextKey      = 'event.text.260002'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when a document fails RN pass through logging.'
 Where EventCd = 260002
/

Update lg_EventCd
   Set InternalDesc = 'Encountered unexpected exception while unpackaging backend integration document: {0}'
     , EventName    = 'Unexpected Exception in Backend Integration Unpackaging'
     , EventNameKey = 'event.name.270000'
     , TextKey      = 'event.text.270000'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the system encounters an unexpected error during backend integration unpackaging.'
 Where EventCd = 270000
/

Update lg_EventCd
   Set InternalDesc = 'Backend integration missing required headers: {0}'
     , EventName    = 'Missing Headers in Backend Integration'
     , EventNameKey = 'event.name.270001'
     , TextKey      = 'event.text.270001'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the  backend integration package is missing required headers.'
 Where EventCd = 270001
/

Update lg_EventCd
   Set InternalDesc = 'Encountered unexpected exception while packaging backend integration document: {0}'
     , EventName    = 'Unexpected Exception in Backend Integration Packaging'
     , EventNameKey = 'event.name.270002'
     , TextKey      = 'event.text.270002'
     , Visibility   = 1
     , Severity     = 6
     , ExternalDesc = 'This event is generated when the system encounters an unexpected error during backend integration packaging.'
 Where EventCd = 270002
/

Update lg_EventCd
   Set InternalDesc = 'Unpackaging backend integration envelope encountered XML parser error: {0}'
     , EventName    = 'Backend Integration Unpackage Parse Error'
     , EventNameKey = 'event.name.270003'
     , TextKey      = 'event.text.270003'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the system encounters an XML parser error during backend integration unpackaging.'
 Where EventCd = 270003
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Adapter Received Duplicate Message'
     , EventNameKey = 'event.name.300001'
     , TextKey      = 'event.text.300001'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = '{0}'
 Where EventCd = 300001
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Adapter Internal Error'
     , EventNameKey = 'event.name.300002'
     , TextKey      = 'event.text.300002'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = '{0}'
 Where EventCd = 300002
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Adapter Backend Interface Failed'
     , EventNameKey = 'event.name.300003'
     , TextKey      = 'event.text.300003'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = '{0}'
 Where EventCd = 300003
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Adapter Received Malformed Data'
     , EventNameKey = 'event.name.300004'
     , TextKey      = 'event.text.300004'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = '{0}'
 Where EventCd = 300004
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Adapter Backend Integration Failed due to Participant Data'
     , EventNameKey = 'event.name.300005'
     , TextKey      = 'event.text.300005'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = '{0}'
 Where EventCd = 300005
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Adapter Received Unknown Requested Document Handler'
     , EventNameKey = 'event.name.300006'
     , TextKey      = 'event.text.300006'
     , Visibility   = 7
     , Severity     = 3
     , ExternalDesc = '{0}'
 Where EventCd = 300006
/

Update lg_EventCd
   Set InternalDesc = 'Failed to get an instance of the Community Manager business process because {0}'
     , EventName    = 'Get Community Manager Business Process Failed'
     , EventNameKey = 'event.name.800000'
     , TextKey      = 'event.text.800000'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated when the system fails to locate the Community Manager action for business processing.'
 Where EventCd = 800000
/

Update lg_EventCd
   Set InternalDesc = 'Community Manager business process ({0}) entrance'
     , EventName    = 'Community Manager Business Process Entrance'
     , EventNameKey = 'event.name.800001'
     , TextKey      = 'event.text.800001'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when the document enters the Community Manager action for business processing.'
 Where EventCd = 800001
/

Update lg_EventCd
   Set InternalDesc = 'Community Manager business process ({0}) exit'
     , EventName    = 'Community Manager Business Process Exit'
     , EventNameKey = 'event.name.800002'
     , TextKey      = 'event.text.800002'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when the document exits the Community Manager action for business processing.'
 Where EventCd = 800002
/

Update lg_EventCd
   Set InternalDesc = 'Community Manager business process successful'
     , EventName    = 'Community Manager Business Process Successful'
     , EventNameKey = 'event.name.800003'
     , TextKey      = 'event.text.800003'
     , Visibility   = 1
     , Severity     = 1
     , ExternalDesc = 'This event is generated when the document successfully completes the Community Manager action for business processing.'
 Where EventCd = 800003
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Community Manager Business Process Encounters Database Error'
     , EventNameKey = 'event.name.800004'
     , TextKey      = 'event.text.800004'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated due to database error while processing the Community Manager''s action.'
 Where EventCd = 800004
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Community Manager Process Encounters Internal Error'
     , EventNameKey = 'event.name.800005'
     , TextKey      = 'event.text.800005'
     , Visibility   = 1
     , Severity     = 9
     , ExternalDesc = 'This event is generated due to internal system error while processing the Community Manager''s action.'
 Where EventCd = 800005
/

Update lg_EventCd
   Set InternalDesc = '{0}'
     , EventName    = 'Community Manger Business Process Warning'
     , EventNameKey = 'event.name.800006'
     , TextKey      = 'event.text.800006'
     , Visibility   = 1
     , Severity     = 3
     , ExternalDesc = 'This event is generated when the file map cannot be found and the Delivery Manager is notified to deliver the document to the exception directory.'
 Where EventCd = 800006
/

Update lg_EventCd
   Set InternalDesc = 'Discard MSA File'
     , EventName    = 'Discard MSA File'
     , EventNameKey = 'event.name.800007'
     , TextKey      = 'event.text.800007'
     , Visibility   = 1
     , Severity     = 0
     , ExternalDesc = 'This event is generated when the system discards the MSA File Map.'
 Where EventCd = 800007
/

Update lg_EventCd
   Set InternalDesc = '0A1 service content received'
     , EventName    = '0A1 Service Content Received'
     , EventNameKey = 'event.name.900001'
     , TextKey      = 'event.text.900001'
     , Visibility   = 3
     , Severity     = 1
     , ExternalDesc = '{0}'
 Where EventCd = 900001
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103020', 'BCG103020', 'Security Information for the partner has changed'
        , 'Security Information has changed for the partner {0}'
        , 'event.name.103020', 'event.text.103020'
        , 7, 1, 0
        , 'This event is generated when the partner security information has changed.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103020)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '103022', 'BCG103022', 'Partner has been deleted'
        , 'Partner {0} has been deleted'
        , 'event.name.103022', 'event.text.103022'
        , 7, 3, 1
        , 'This event is generated when a partner is deleted.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103022)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '280001', 'BCG280001', 'ResendEngine Entrance'
        , 'ResendEngine ({0}) entrance.'
        , 'event.name.280001', 'event.text.280001'
        , 7, 1, 0
        , 'This event is generated when the ReSend Engine is invoked for a document'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 280001)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '280002', 'BCG280002', 'ResendEngine Exit'
        , 'ResendEngine ({0}) Exit.'
        , 'event.name.280002', 'event.text.280002'
        , 7, 1, 0
        , 'This event is generated when the ReSend Engine completes the processing'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 280002)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '280003', 'BCG280003', 'Document resent successfully'
        , 'Document ({0}) resent successfully.'
        , 'event.name.280003', 'event.text.280003'
        , 7, 1, 0
        , 'This event is generated when the ReSend Engine successfully processes a document'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 280003)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '280004', 'BCG280004', 'Processing error'
        , 'Document ({0}) processing failed. Exception message ({1}).'
        , 'event.name.280004', 'event.text.280004'
        , 7, 1, 0
        , 'This event is generated when the ReSend Engine encounters an error'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 280004)
/
--BGTRK00060472
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '280005', 'BCG280005', 'Processing error'
        , 'Child  document ({0}) can not be resent.'
        , 'event.name.280005', 'event.text.280005'
        , 7, 1, 0
        , 'This event is generated when the ReSend Engine encounters an error'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 280005)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '280006', 'BCG280006', 'Processing error'
        , 'Unable to find content, meta-data and header files in {0} reject and oversize folders for document: {1}'
        , 'event.name.280006', 'event.text.280006'
        , 7, 6, 0
        , 'This event is generated when the Async Document fails in DAE and is being resent'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 280006)
/

--EDI Event Codes
INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990001, 'BCGEDICM0001', 'Unexpected Exception Occurred', 0, 'An unexpected exception occurred in component: {0}.  Exception text: {1}', 'ediEvent.name.CM0001', 4, 'ediEvent.text.CM0001', 5, 7, 6, 1, 1, 'An unexpected exception occurred in component: {0}.  Exception text: {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0001')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990002, 'BCGEDICM0003', 'Missing Required Property', 0, 'Invalid input for component {0}. Missing required property {1}', 'ediEvent.name.CM0003', 4, 'ediEvent.text.CM0003', 5, 7, 6, 1, 1, 'Invalid input for component {0}. Missing required property {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0003')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990003, 'BCGEDICM0004', 'Invalid Property Value', 0, 'Invalid input for component {0}. Value {1} is invalid for property {2}', 'ediEvent.name.CM0004', 4, 'ediEvent.text.CM0004', 5, 7, 6, 1, 1, 'Invalid input for component {0}. Value {1} is invalid for property {2}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0004')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990004, 'BCGEDICM0005', 'Unusupported Character Set', 0, 'Invalid input for component {0}. Character set {1} specified in property {2} is not supported', 'ediEvent.name.CM0005', 4, 'ediEvent.text.CM0005', 5, 7, 6, 1, 1, 'Invalid input for component {0}. Character set {1} specified in property {2} is not supported', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0005')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990005, 'BCGEDICM0006', 'Invalid Document Syntax for Component', 0, 'Invalid input for component {0}. The document syntax {1} is not valid for this component', 'ediEvent.name.CM0006', 4, 'ediEvent.text.CM0006', 5, 7, 6, 1, 1, 'Invalid input for component {0}. The document syntax {1} is not valid for this component', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0006')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990006, 'BCGEDICM0010', 'I/O Error Occurred ', 0, 'An I/O error occurred in component {0}.  The exception text is: {1}', 'ediEvent.name.CM0010', 4, 'ediEvent.text.CM0010', 5, 7, 6, 1, 1, 'An I/O error occurred in component {0}.  The exception text is: {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0010')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990007, 'BCGEDICM0011', 'File Open Failed', 0, 'Component {0} could not open file: {1}', 'ediEvent.name.CM0011', 4, 'ediEvent.text.CM0011', 5, 7, 6, 1, 1, 'Component {0} could not open file: {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0011')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990008, 'BCGEDICM0012', 'Failure Accessing Memory Buffer', 0, 'Component {0} could not access the memory buffer', 'ediEvent.name.CM0012', 4, 'ediEvent.text.CM0012', 5, 7, 6, 1, 1, 'Component {0} could not access the memory buffer', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0012')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990009, 'BCGEDICM0013', 'Missing Input Data Source', 0, 'No input data source was specified for component {0}', 'ediEvent.name.CM0013', 4, 'ediEvent.text.CM0013', 5, 7, 6, 1, 1, 'No input data source was specified for component {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0013')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990010, 'BCGEDICM0014', 'Missing Output Data Source', 0, 'No output data source was specified for component {0}', 'ediEvent.name.CM0014', 4, 'ediEvent.text.CM0014', 5, 7, 6, 1, 1, 'No output data source was specified for component {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0014')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990011, 'BCGEDICM0020', 'Parsing Error in Component', 0, 'Component {0} failed due to errors parsing the input data', 'ediEvent.name.CM0020', 4, 'ediEvent.text.CM0020', 5, 7, 6, 1, 1, 'Component {0} failed due to errors parsing the input data', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0020')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990012, 'BCGEDICM0021', 'Database Error', 0, 'An error occurred while trying to access the database.  Class name: {0} Method: {1} Exception: {2}', 'ediEvent.name.CM0021', 4, 'ediEvent.text.CM0021', 5, 7, 6, 1, 1, 'An error occurred while trying to access the database.  Class name: {0} Method: {1} Exception: {2}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0021')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990013, 'BCGEDICM0022', 'Unexpected Database Exception', 0, 'An unexpected exception occurred while trying to access the database.  Class name: {0} Method: {1} Exception: {2}', 'ediEvent.name.CM0022', 4, 'ediEvent.text.CM0022', 5, 7, 6, 1, 1, 'An unexpected exception occurred while trying to access the database.  Class name: {0} Method: {1} Exception: {2}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0022')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990014, 'BCGEDICM0023', 'No Database Connection', 0, 'The database connection manager class {0} did not return a valid connection', 'ediEvent.name.CM0023', 4, 'ediEvent.text.CM0023', 5, 7, 9, 1, 1, 'The database connection manager class {0} did not return a valid connection', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0023')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990015, 'BCGEDICM0101', 'Missing or Invalid Object for Component', 0, 'An internal error occurred.  The object passed to component {0} was missing or invalid', 'ediEvent.name.CM0101', 4, 'ediEvent.text.CM0101', 5, 7, 6, 1, 1, 'An internal error occurred.  The object passed to component {0} was missing or invalid', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0101')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990016, 'BCGEDICM0102', 'Class Load Failure', 0, 'A dynamically configured class could not be loaded.  Configuration key: {0}', 'ediEvent.name.CM0102', 4, 'ediEvent.text.CM0102', 5, 7, 9, 1, 1, 'A dynamically configured class could not be loaded.  Configuration key: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0102')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990017, 'BCGEDICM0103', 'Invalid Function Parameter', 0, 'An internal error occurred in component {0}. An invalid value ''{1}'' was passed to function {2}', 'ediEvent.name.CM0103', 4, 'ediEvent.text.CM0103', 5, 7, 6, 1, 1, 'An internal error occurred in component {0}. An invalid value ''{1}'' was passed to function {2}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0103')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990018, 'BCGEDICM0104', 'Invalid Source Document', 0, 'The source document is not applicable to component {0}', 'ediEvent.name.CM0104', 4, 'ediEvent.text.CM0104', 5, 7, 6, 1, 1, 'The source document is not applicable to component {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDICM0104')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990019, 'BCGEDIEV0003', 'Interchange Begin Not Found', 0, 'An attempt to envelope or deenvelope a message failed.  The syntax was {0}', 'ediEvent.name.EV0003', 4, 'ediEvent.text.EV0003', 5, 7, 6, 1, 1, 'An attempt to envelope or deenvelope a message failed.  The syntax was {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0003')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990020, 'BCGEDIEV0009', 'Trading Partner Nickname Lookup Failed', 0, 'Unable to find trading partner nickname: {0}', 'ediEvent.name.EV0009', 4, 'ediEvent.text.EV0009', 5, 7, 6, 1, 1, 'Unable to find trading partner nickname: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0009')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990021, 'BCGEDIEV0010', 'Internal Error for Function', 0, 'Internal error occurred.  Function: {0}', 'ediEvent.name.EV0010', 4, 'ediEvent.text.EV0010', 5, 7, 6, 1, 1, 'Internal error occurred.  Function: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0010')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990022, 'BCGEDIEV0011', 'Database transaction failed', 0, 'Database transaction failed.  SQL Error: {0}', 'ediEvent.name.EV0011', 4, 'ediEvent.text.EV0011', 5, 7, 6, 1, 1, 'Database transaction failed.  SQL Error: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0011')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990023, 'BCGEDIEV0018', 'Envelope Segment Not Found', 0, 'The {0} enveloper or de-enveloper detected an error:  The {1} segment was not found', 'ediEvent.name.EV0018', 4, 'ediEvent.text.EV0018', 5, 7, 6, 1, 1, 'The {0} enveloper or de-enveloper detected an error:  The {1} segment was not found', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0018')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990024, 'BCGEDIEV0050', 'Translation Table Lookup Failed ', 0, 'A translation table lookup failed while enveloping or deenveloping a message.  Translation table: {0}', 'ediEvent.name.EV0050', 4, 'ediEvent.text.EV0050', 5, 7, 6, 1, 1, 'A translation table lookup failed while enveloping or deenveloping a message.  Translation table: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0050')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990025, 'BCGEDIEV0051', 'Envelope Segment Not Found', 0, 'The {0} deenveloper detected an error:  {1} found with no {2}', 'ediEvent.name.EV0051', 4, 'ediEvent.text.EV0051', 5, 7, 6, 1, 1, 'The {0} deenveloper detected an error:  {1} found with no {2}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0051')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990026, 'BCGEDIEV0052', 'Empty Message to Envelope', 0, 'The {0} enveloper detected an error:  Received an empty message to envelope', 'ediEvent.name.EV0052', 4, 'ediEvent.text.EV0052', 5, 7, 6, 1, 1, 'The {0} enveloper detected an error:  Received an empty message to envelope', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0052')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990027, 'BCGEDIEV0053', 'Maximum Groups Exceeded for Control Number Mask', 0, 'The {0} enveloper detected an error:  Total groups greater than number allowed by control number mask', 'ediEvent.name.EV0053', 4, 'ediEvent.text.EV0053', 5, 7, 6, 1, 1, 'The {0} enveloper detected an error:  Total groups greater than number allowed by control number mask', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0053')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990028, 'BCGEDIEV0057', 'Envelope Failed', 0, 'An attempt to envelope a message failed.  The envelope type was {2}', 'ediEvent.name.EV0057', 4, 'ediEvent.text.EV0057', 5, 7, 6, 1, 1, 'An attempt to envelope a message failed.  The envelope type was {2}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0057')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990029, 'BCGEDIEV0058', 'Deenvelope Failed', 0, 'An attempt to deenvelope a message failed', 'ediEvent.name.EV0058', 4, 'ediEvent.text.EV0058', 5, 7, 6, 1, 1, 'An attempt to deenvelope a message failed', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEV0058')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990030, 'BCGEDIFT0100', 'Expected Argument Missing', 0, 'Syntax error on command: {0}. An expected argument was missing', 'ediEvent.name.FT0100', 4, 'ediEvent.text.FT0100', 5, 7, 6, 1, 1, 'Syntax error on command: {0}. An expected argument was missing', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0100')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990031, 'BCGEDIFT0110', 'FTP script Processing Stopped', 0, 'Error caused FTP script processing to stop', 'ediEvent.name.FT0110', 4, 'ediEvent.text.FT0110', 5, 7, 6, 1, 1, 'Error caused FTP script processing to stop', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0110')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990032, 'BCGEDIFT0111', 'Missing File Base Name ', 0, 'No base name provided for retrieved files', 'ediEvent.name.FT0111', 4, 'ediEvent.text.FT0111', 5, 7, 6, 1, 1, 'No base name provided for retrieved files', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0111')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990033, 'BCGEDIFT0112', 'Missing or Invalid Object for Component', 0, 'Unknown object on input terminal', 'ediEvent.name.FT0112', 4, 'ediEvent.text.FT0112', 5, 7, 6, 1, 1, 'Unknown object on input terminal', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0112')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990034, 'BCGEDIFT0113', 'Unexpected Error Executing Command', 0, 'Unexpected error executing command: {0}', 'ediEvent.name.FT0113', 4, 'ediEvent.text.FT0113', 5, 7, 6, 1, 1, 'Unexpected error executing command: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0113')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990035, 'BCGEDIFT0114', 'Unexpected Error Downloading File', 0, 'Unexpected error downloading file: {0}', 'ediEvent.name.FT0114', 4, 'ediEvent.text.FT0114', 5, 7, 6, 1, 1, 'Unexpected error downloading file: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0114')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990036, 'BCGEDIFT0115', 'FTP Script File Not Found', 0, 'FTP script file not found', 'ediEvent.name.FT0115', 4, 'ediEvent.text.FT0115', 5, 7, 6, 1, 1, 'FTP script file not found', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0115')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990037, 'BCGEDIFT0116', 'IO Exception Reading Script', 0, 'IO Exception caught when reading script', 'ediEvent.name.FT0116', 4, 'ediEvent.text.FT0116', 5, 7, 6, 1, 1, 'IO Exception caught when reading script', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0116')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990038, 'BCGEDIFT0117', 'Unexpected Exception Parsing FTP Script', 0, 'Unexpected exception caught while parsing FTP Script. Contact your system administrator. Further details about the exception and a stacktrace can be found in the trace file', 'ediEvent.name.FT0117', 4, 'ediEvent.text.FT0117', 5, 7, 6, 1, 1, 'Unexpected exception caught while parsing FTP Script. Contact your system administrator. Further details about the exception and a stacktrace can be found in the trace file', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0117')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990039, 'BCGEDIFT0118', 'File Upload Failed', 0, 'Unable to upload file.  File name was: {0}', 'ediEvent.name.FT0118', 4, 'ediEvent.text.FT0118', 5, 7, 6, 1, 1, 'Unable to upload file.  File name was: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0118')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990040, 'BCGEDIFT0119', 'No File for MPUT ', 0, 'MPUT issued but no file was found to be sent. Filename was: {0}. Directory was: {1}', 'ediEvent.name.FT0119', 4, 'ediEvent.text.FT0119', 5, 7, 6, 1, 1, 'MPUT issued but no file was found to be sent. Filename was: {0}. Directory was: {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0119')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990041, 'BCGEDIFT0120', 'FTP Command Timed Out', 0, 'FTP Command timed out.  The command being sent was: {0}', 'ediEvent.name.FT0120', 4, 'ediEvent.text.FT0120', 5, 7, 6, 1, 1, 'FTP Command timed out.  The command being sent was: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0120')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990042, 'BCGEDIFT0200', 'IO Exception ', 0, 'An IO Exception has occured.Exception Text {0}', 'ediEvent.name.FT0200', 4, 'ediEvent.text.FT0200', 5, 7, 6, 1, 1, 'An IO Exception has occured.Exception Text {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0200')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990043, 'BCGEDIFT0201', 'Data Socket Create Failed', 0, 'Data Socket could not be created. Connection or ControlSocket is null', 'ediEvent.name.FT0201', 4, 'ediEvent.text.FT0201', 5, 7, 6, 1, 1, 'Data Socket could not be created. Connection or ControlSocket is null', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0201')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990044, 'BCGEDIFT0202', 'Reply Codes Are Null', 0, 'Null Pointer Exception: StringBuffer that has replycodes for processing is null', 'ediEvent.name.FT0202', 4, 'ediEvent.text.FT0202', 5, 7, 6, 1, 1, 'Null Pointer Exception: StringBuffer that has replycodes for processing is null', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0202')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990045, 'BCGEDIFT0203', 'Invalid Argument Values', 0, 'Invalid values for the arguments', 'ediEvent.name.FT0203', 4, 'ediEvent.text.FT0203', 5, 7, 6, 1, 1, 'Invalid values for the arguments', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0203')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990046, 'BCGEDIFT0204', 'Control Socket Not Created', 0, 'Control Socket not created', 'ediEvent.name.FT0204', 4, 'ediEvent.text.FT0204', 5, 7, 6, 1, 1, 'Control Socket not created', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0204')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990047, 'BCGEDIFT0205', 'Required File Not Found', 0, 'Required File Not Found', 'ediEvent.name.FT0205', 4, 'ediEvent.text.FT0205', 5, 7, 6, 1, 1, 'Required File Not Found', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0205')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990048, 'BCGEDIFT0206', 'Exception occured', 0, 'Exception occured', 'ediEvent.name.FT0206', 4, 'ediEvent.text.FT0206', 5, 7, 6, 1, 1, 'Exception occured', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0206')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990049, 'BCGEDIFT0207', 'Active Data Socket Is Null', 0, 'Active Data Socket is null', 'ediEvent.name.FT0207', 4, 'ediEvent.text.FT0207', 5, 7, 6, 1, 1, 'Active Data Socket is null', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0207')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990050, 'BCGEDIFT0208', 'SocketException Has Occurred', 0, 'SocketException has occurred', 'ediEvent.name.FT0208', 4, 'ediEvent.text.FT0208', 5, 7, 6, 1, 1, 'SocketException has occurred', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0208')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990051, 'BCGEDIFT0209', 'Passive Data Socket Is Null', 0, 'Passive Data Socket is null', 'ediEvent.name.FT0209', 4, 'ediEvent.text.FT0209', 5, 7, 6, 1, 1, 'Passive Data Socket is null', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0209')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990052, 'BCGEDIFT0210', 'Data Socket Is Null', 0, 'Data Socket is null', 'ediEvent.name.FT0210', 4, 'ediEvent.text.FT0210', 5, 7, 6, 1, 1, 'Data Socket is null', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0210')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990053, 'BCGEDIFT0211', 'Load Private Key Failed', 0, 'Could not load private key file from filename - {0}  Alias not found', 'ediEvent.name.FT0211', 4, 'ediEvent.text.FT0211', 5, 7, 6, 1, 1, 'Could not load private key file from filename - {0}  Alias not found', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0211')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990054, 'BCGEDIFT0212', 'Client Certificate Validation Failed', 0, 'Client Certificate (local certificate) validation has failed', 'ediEvent.name.FT0212', 4, 'ediEvent.text.FT0212', 5, 7, 6, 1, 1, 'Client Certificate (local certificate) validation has failed', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0212')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990055, 'BCGEDIFT0220', 'OPEN Command Failed', 0, 'OPEN command failed. Reason: {0}', 'ediEvent.name.FT0220', 4, 'ediEvent.text.FT0220', 5, 7, 6, 1, 1, 'OPEN command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0220')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990056, 'BCGEDIFT0221', 'CWD Command Failed', 0, 'CWD command failed. Reason: {0}', 'ediEvent.name.FT0221', 4, 'ediEvent.text.FT0221', 5, 7, 6, 1, 1, 'CWD command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0221')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990057, 'BCGEDIFT0222', 'DELE Command Failed', 0, 'DELE command failed. Reason: {0}', 'ediEvent.name.FT0222', 4, 'ediEvent.text.FT0222', 5, 7, 6, 1, 1, 'DELE command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0222')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990058, 'BCGEDIFT0223', 'PUT Command Failed', 0, 'PUT command failed. Reason: {0}', 'ediEvent.name.FT0223', 4, 'ediEvent.text.FT0223', 5, 7, 6, 1, 1, 'PUT command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0223')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990059, 'BCGEDIFT0224', 'GET Command Failed', 0, 'GET command failed. Reason: {0}', 'ediEvent.name.FT0224', 4, 'ediEvent.text.FT0224', 5, 7, 6, 1, 1, 'GET command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0224')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990060, 'BCGEDIFT0225', 'LIST Command Failed', 0, 'LIST command failed. Reason: {0}', 'ediEvent.name.FT0225', 4, 'ediEvent.text.FT0225', 5, 7, 6, 1, 1, 'LIST command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0225')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990061, 'BCGEDIFT0226', 'QUIT Command Failed', 0, 'QUIT command failed. Reason: {0}', 'ediEvent.name.FT0226', 4, 'ediEvent.text.FT0226', 5, 7, 6, 1, 1, 'QUIT command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0226')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990062, 'BCGEDIFT0227', 'RMD Command Failed', 0, 'RMD command failed. Reason: {0}', 'ediEvent.name.FT0227', 4, 'ediEvent.text.FT0227', 5, 7, 6, 1, 1, 'RMD command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0227')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990063, 'BCGEDIFT0228', 'MKD Command Failed', 0, 'MKD command failed. Reason: {0}', 'ediEvent.name.FT0228', 4, 'ediEvent.text.FT0228', 5, 7, 6, 1, 1, 'MKD command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0228')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990064, 'BCGEDIFT0229', 'PASV Command Failed', 0, 'PASV command failed. Reason: {0}', 'ediEvent.name.FT0229', 4, 'ediEvent.text.FT0229', 5, 7, 6, 1, 1, 'PASV command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0229')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990065, 'BCGEDIFT0230', 'GETDEL Command Failed', 0, 'GETDEL command failed. Reason: {0}', 'ediEvent.name.FT0230', 4, 'ediEvent.text.FT0230', 5, 7, 6, 1, 1, 'GETDEL command failed. Reason: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0230')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990066, 'BCGEDIFT0231', 'FTP Command Failed', 0, 'FTP command {0} failed. Reason: {1}', 'ediEvent.name.FT0231', 4, 'ediEvent.text.FT0231', 5, 7, 6, 1, 1, 'FTP command {0} failed. Reason: {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0231')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990067, 'BCGEDIFT0232', 'Null Reply From FTP Server', 0, 'The reply from the FTP Server is null', 'ediEvent.name.FT0232', 4, 'ediEvent.text.FT0232', 5, 7, 6, 1, 1, 'The reply from the FTP Server is null', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIFT0232')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990068, 'BCGEDIMD0001', 'Unexpected Exception Reading Metadata', 0, 'An unexpected exception occurred while reading the metadata.  Syntax: {0}', 'ediEvent.name.MD0001', 4, 'ediEvent.text.MD0001', 5, 7, 6, 1, 1, 'An unexpected exception occurred while reading the metadata.  Syntax: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIMD0001')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990069, 'BCGEDIMD0002', 'Metadata Control String Invalid', 0, 'The metadata control string is invalid', 'ediEvent.name.MD0002', 4, 'ediEvent.text.MD0002', 5, 7, 6, 1, 1, 'The metadata control string is invalid', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIMD0002')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990070, 'BCGEDIMD0003', 'Metadata Control String Read Failed', 0, 'The metadata control string could not be read from the database.  Syntax: {0}', 'ediEvent.name.MD0003', 4, 'ediEvent.text.MD0003', 5, 7, 6, 1, 1, 'The metadata control string could not be read from the database.  Syntax: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIMD0003')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990071, 'BCGEDINK0001', 'Invalid Network Acknowledgement ', 0, 'The document passed to the IBM VAN network acknowledgement component is not a valid network acknowledgement', 'ediEvent.name.NK0001', 4, 'ediEvent.text.NK0001', 5, 7, 6, 1, 1, 'The document passed to the IBM VAN network acknowledgement component is not a valid network acknowledgement', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDINK0001')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990072, 'BCGEDINK0002', 'Invalid Attribute Value', 0, 'The attribute {0} has an invalid value {1}', 'ediEvent.name.NK0002', 4, 'ediEvent.text.NK0002', 5, 7, 6, 1, 1, 'The attribute {0} has an invalid value {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDINK0002')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990073, 'BCGEDIUP0001', 'Fatal XML Parsing Error ', 0, 'A fatal error occured during parsing of XML document {0} at line {1}', 'ediEvent.name.UP0001', 4, 'ediEvent.text.UP0001', 5, 7, 6, 1, 1, 'A fatal error occured during parsing of XML document {0} at line {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0001')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990074, 'BCGEDIUP0002', 'Severe XML Parsing Error ', 0, 'A severe error occured during parsing of XML document {0} at line {1}', 'ediEvent.name.UP0002', 4, 'ediEvent.text.UP0002', 5, 7, 6, 1, 1, 'A severe error occured during parsing of XML document {0} at line {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0002')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990075, 'BCGEDIUP0003', 'XML Parsing Warning', 0, 'A warning occured during parsing of XML document {0} at line {1}', 'ediEvent.name.UP0003', 4, 'ediEvent.text.UP0003', 5, 7, 3, 1, 1, 'A warning occured during parsing of XML document {0} at line {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0003')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990076, 'BCGEDIUP0015', 'Metadata Read Failed', 0, 'An attempt to get the metadata for the message failed', 'ediEvent.name.UP0015', 4, 'ediEvent.text.UP0015', 5, 7, 6, 1, 1, 'An attempt to get the metadata for the message failed', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0015')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990077, 'BCGEDIUP0021', 'Unable to Identify Input Data Record', 0, 'Unable to identify the input data record.  Record number was {0}.  Data Image {1}', 'ediEvent.name.UP0021', 4, 'ediEvent.text.UP0021', 5, 7, 6, 1, 1, 'Unable to identify the input data record.  Record number was {0}.  Data Image {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0021')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990078, 'BCGEDIUP0023', 'Record Exceeded Maximum Repetitions specified', 0, 'The data received has exceeded the maximum repetitions specified.  Record number was {0}.  Data Identification was {1} and maximum reptitions was {2}', 'ediEvent.name.UP0023', 4, 'ediEvent.text.UP0023', 5, 7, 6, 1, 1, 'The data received has exceeded the maximum repetitions specified.  Record number was {0}.  Data Identification was {1} and maximum reptitions was {2}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0023')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990079, 'BCGEDIUP0033', 'Missing Dictionary or Document values ', 0, 'The Dictionary or Document values used for parsing were not specified or are blank', 'ediEvent.name.UP0033', 4, 'ediEvent.text.UP0033', 5, 7, 6, 1, 1, 'The Dictionary or Document values used for parsing were not specified or are blank', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0033')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990080, 'BCGEDIUP0034', 'Invalid Structure Usage', 0, 'Character separated data is not a supported option for data formats that contain structures', 'ediEvent.name.UP0034', 4, 'ediEvent.text.UP0034', 5, 7, 6, 1, 1, 'Character separated data is not a supported option for data formats that contain structures', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0034')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990081, 'BCGEDIUP0038', 'Missing Record Delimiter', 0, 'End of record reached without record delimiter detected.  Record delimiter expected was {0}', 'ediEvent.name.UP0038', 4, 'ediEvent.text.UP0038', 5, 7, 6, 1, 1, 'End of record reached without record delimiter detected.  Record delimiter expected was {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0038')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990082, 'BCGEDIUP0039', 'Character Conversion Failed', 0, 'An attempt to convert data to unicode characters failed.  The input data was {0}', 'ediEvent.name.UP0039', 4, 'ediEvent.text.UP0039', 5, 7, 6, 1, 1, 'An attempt to convert data to unicode characters failed.  The input data was {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0039')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990083, 'BCGEDIUP0040', 'Invalid Data for Data Type', 0, 'Invalid data found attempting to convert {0} type data.  Invalid data was {1}', 'ediEvent.name.UP0040', 4, 'ediEvent.text.UP0040', 5, 7, 6, 1, 1, 'Invalid data found attempting to convert {0} type data.  Invalid data was {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0040')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990084, 'BCGEDIUP0041', 'Unsupported Character Set', 0, 'The character set used for the ROD data is not supported.  The character set was {0}', 'ediEvent.name.UP0041', 4, 'ediEvent.text.UP0041', 5, 7, 6, 1, 1, 'The character set used for the ROD data is not supported.  The character set was {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0041')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990085, 'BCGEDIUP0042', 'Unsupported Record Found', 0, 'An unsupported record was found processing C and D records.  The character C', 'ediEvent.name.UP0042', 4, 'ediEvent.text.UP0042', 5, 7, 6, 1, 1, 'An unsupported record was found processing C and D records.  The character C', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0042')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990086, 'BCGEDIUP0045', 'Multiple Start Records', 0, 'Another record oriented data start record was found after the start record had already been processed.  The start record name was {0}', 'ediEvent.name.UP0045', 4, 'ediEvent.text.UP0045', 5, 7, 3, 1, 1, 'Another record oriented data start record was found after the start record had already been processed.  The start record name was {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0045')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990087, 'BCGEDIUP0046', 'Data After End Record', 0, 'Another record oriented data record was found after the end record had already been processed.  The record name was {0}', 'ediEvent.name.UP0046', 4, 'ediEvent.text.UP0046', 5, 7, 3, 1, 1, 'Another record oriented data record was found after the end record had already been processed.  The record name was {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0046')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990088, 'BCGEDIUP0052', 'Unexpected Serialization Exception ', 0, 'An unexpected exception occurred while serializing the document. Exception text is: {0}', 'ediEvent.name.UP0052', 4, 'ediEvent.text.UP0052', 5, 7, 6, 1, 1, 'An unexpected exception occurred while serializing the document. Exception text is: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0052')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990089, 'BCGEDIUP0053', 'Parser or Serializer Creation Failed', 0, 'No parser or serializer could be created for syntax {0}', 'ediEvent.name.UP0053', 4, 'ediEvent.text.UP0053', 5, 7, 6, 1, 1, 'No parser or serializer could be created for syntax {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0053')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990090, 'BCGEDIUP0055', 'Empty Document for Serialization', 0, 'The document could not be serialized because it is empty', 'ediEvent.name.UP0055', 4, 'ediEvent.text.UP0055', 5, 7, 6, 1, 1, 'The document could not be serialized because it is empty', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0055')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990091, 'BCGEDIUP0057', 'Invalid Document for Serialization', 0, 'The document could not be serialized because its internal structure is invalid', 'ediEvent.name.UP0057', 4, 'ediEvent.text.UP0057', 5, 7, 6, 1, 1, 'The document could not be serialized because its internal structure is invalid', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0057')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990092, 'BCGEDIUP0099', 'No Recognized Input Data', 0, 'Parser found no recognizable input data.  Parser component {2}', 'ediEvent.name.UP0099', 4, 'ediEvent.text.UP0099', 5, 7, 6, 1, 1, 'Parser found no recognizable input data.  Parser component {2}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0099')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990093, 'BCGEDIUP0100', 'MetaData Details Not Found', 0, 'MetaData details not found in document. Dictionary', 'ediEvent.name.UP0100', 4, 'ediEvent.text.UP0100', 5, 7, 6, 1, 1, 'MetaData details not found in document. Dictionary', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0100')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990094, 'BCGEDIUP0101', 'MetaData Control String Not Found', 0, 'MetaData Control String not Found. Dictionary', 'ediEvent.name.UP0101', 4, 'ediEvent.text.UP0101', 5, 7, 6, 1, 1, 'MetaData Control String not Found. Dictionary', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0101')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990095, 'BCGEDIUP0106', 'Invalid ROD Data Format', 0, 'Invalid ROD DataFormat. No Child Nodes (STRUCTURES or FIELDS) found in Record node. Record Name: {0}', 'ediEvent.name.UP0106', 4, 'ediEvent.text.UP0106', 5, 7, 6, 1, 1, 'Invalid ROD DataFormat. No Child Nodes (STRUCTURES or FIELDS) found in Record node. Record Name: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0106')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990096, 'BCGEDIUP0107', 'Missing Record Name in Record', 0, 'NULL RecordName found in document for D Record', 'ediEvent.name.UP0107', 4, 'ediEvent.text.UP0107', 5, 7, 6, 1, 1, 'NULL RecordName found in document for D Record', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0107')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990097, 'BCGEDIUP0108', 'Unexpected Nodes Under Root Node', 0, 'Invalid ROD DataFormat. ROD ROOT node: {0} has child nodes other than RECORD and LOOP', 'ediEvent.name.UP0108', 4, 'ediEvent.text.UP0108', 5, 7, 6, 1, 1, 'Invalid ROD DataFormat. ROD ROOT node: {0} has child nodes other than RECORD and LOOP', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0108')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990098, 'BCGEDIUP0109', 'Missing Record Name in Node', 0, 'NULL or empty RecordName found in RECORD node', 'ediEvent.name.UP0109', 4, 'ediEvent.text.UP0109', 5, 7, 6, 1, 1, 'NULL or empty RecordName found in RECORD node', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0109')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990099, 'BCGEDIUP0110', 'Error Getting Metadata Information', 0, 'Unable to get RODMetaDataElement from MetaData for the record: {0}', 'ediEvent.name.UP0110', 4, 'ediEvent.text.UP0110', 5, 7, 6, 1, 1, 'Unable to get RODMetaDataElement from MetaData for the record: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0110')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990100, 'BCGEDIUP0111', 'Empty Record Found', 0, 'Child Elements not found in MetaDataElement: {0}', 'ediEvent.name.UP0111', 4, 'ediEvent.text.UP0111', 5, 7, 6, 1, 1, 'Child Elements not found in MetaDataElement: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0111')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990101, 'BCGEDIUP0112', 'Unexpected Nodes Under Record Node ', 0, 'Invalid ROD DataFormat. ROD RECORD node: {0} has child nodes other than STRUCTURE and FIELD', 'ediEvent.name.UP0112', 4, 'ediEvent.text.UP0112', 5, 7, 6, 1, 1, 'Invalid ROD DataFormat. ROD RECORD node: {0} has child nodes other than STRUCTURE and FIELD', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0112')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990102, 'BCGEDIUP0113', 'Unexpected Nodes Under Loop Node ', 0, 'Invalid ROD DataFormat. ROD LOOP node: {0} has child nodes other than LOOP and RECORD', 'ediEvent.name.UP0113', 4, 'ediEvent.text.UP0113', 5, 7, 6, 1, 1, 'Invalid ROD DataFormat. ROD LOOP node: {0} has child nodes other than LOOP and RECORD', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0113')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990103, 'BCGEDIUP0114', 'Unexpected Nodes Under Structure Node ', 0, 'Invalid ROD DataFormat. ROD STRUCTURE node: {0} has child nodes other than STRUCTURE and FIELD', 'ediEvent.name.UP0114', 4, 'ediEvent.text.UP0114', 5, 7, 6, 1, 1, 'Invalid ROD DataFormat. ROD STRUCTURE node: {0} has child nodes other than STRUCTURE and FIELD', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0114')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990104, 'BCGEDIUP0115', 'Empty Structure Found ', 0, 'Child Elements not found in MetaDataElement: {0}', 'ediEvent.name.UP0115', 4, 'ediEvent.text.UP0115', 5, 7, 6, 1, 1, 'Child Elements not found in MetaDataElement: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0115')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990105, 'BCGEDIUP0116', 'Invalid Character in Data Format', 0, 'Invalid Character found with {0} data format. Character is: {1}', 'ediEvent.name.UP0116', 4, 'ediEvent.text.UP0116', 5, 7, 6, 1, 1, 'Invalid Character found with {0} data format. Character is: {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUP0116')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990106, 'BCGEDISP0002', 'Unable to Determine Encoding', 0, 'The XML splitter could not determine the encoding of the XML input data', 'ediEvent.name.SP0002', 4, 'ediEvent.text.SP0002', 5, 7, 6, 1, 1, 'The XML splitter could not determine the encoding of the XML input data', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDISP0002')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990107, 'BCGEDISP0003', 'Invalid XML Data', 0, 'The data passed to the XML splitter is not valid XML data', 'ediEvent.name.SP0003', 4, 'ediEvent.text.SP0003', 5, 7, 6, 1, 1, 'The data passed to the XML splitter is not valid XML data', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDISP0003')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990108, 'BCGEDISP0005', 'Interchange Iterator is Null', 0, 'An internal error occurred.  The interchange iterator was not set during a previous call', 'ediEvent.name.SP0005', 4, 'ediEvent.text.SP0005', 5, 7, 6, 1, 1, 'An internal error occurred.  The interchange iterator was not set during a previous call', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDISP0005')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990109, 'BCGEDISP0006', 'End of Input Data', 0, 'The splitter has reached the end of the input data', 'ediEvent.name.SP0006', 4, 'ediEvent.text.SP0006', 5, 7, 6, 1, 1, 'The splitter has reached the end of the input data', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDISP0006')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990110, 'BCGEDIUT0008', 'Current Map Name', 0, 'Map name being processed: {0}', 'ediEvent.name.UT0008', 4, 'ediEvent.text.UT0008', 5, 7, 1, 1, 1, 'Map name being processed: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0008')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990111, 'BCGEDIUT0011', 'Control String Instruction Failed', 0, 'The transformation node (DTC) was unable to process a control string instruction.  The control string instruction was {0}', 'ediEvent.name.UT0011', 4, 'ediEvent.text.UT0011', 5, 7, 6, 1, 1, 'The transformation node (DTC) was unable to process a control string instruction.  The control string instruction was {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0011')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990112, 'BCGEDIUT0023', 'Output Document Creation Failed', 0, 'An attempt to create an output document failed.  The root node name was {0}', 'ediEvent.name.UT0023', 4, 'ediEvent.text.UT0023', 5, 7, 6, 1, 1, 'An attempt to create an output document failed.  The root node name was {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0023')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990113, 'BCGEDIUT0033', 'User Specified Message Text', 0, 'User specified message text: {0}. This message was logged with severity code {1} and user code {2}', 'ediEvent.name.UT0033', 4, 'ediEvent.text.UT0033', 5, 7, 6, 1, 1, 'User specified message text: {0}. This message was logged with severity code {1} and user code {2}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0033')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990114, 'BCGEDIUT0034', 'HexDecode String Length Invalid', 0, 'The transformation component attempted to HexDecode a string', 'ediEvent.name.UT0034', 4, 'ediEvent.text.UT0034', 5, 7, 6, 1, 1, 'The transformation component attempted to HexDecode a string', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0034')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990115, 'BCGEDIUT0035', 'HexDecode Character Invalid', 0, 'The transformation component was executing a HexDecode command', 'ediEvent.name.UT0035', 4, 'ediEvent.text.UT0035', 5, 7, 6, 1, 1, 'The transformation component was executing a HexDecode command', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0035')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990116, 'BCGEDIUT0061', 'Invalid Value for Imbedded Map', 0, 'Imbedded map requires a byte array element. the instruction stream offset was {1}', 'ediEvent.name.UT0061', 4, 'ediEvent.text.UT0061', 5, 7, 6, 1, 1, 'Imbedded map requires a byte array element. the instruction stream offset was {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0061')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990117, 'BCGEDIUT0100', 'User Exit Not Found', 0, 'User exit {0} was not found', 'ediEvent.name.UT0100', 4, 'ediEvent.text.UT0100', 5, 7, 6, 1, 1, 'User exit {0} was not found', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0100')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990118, 'BCGEDIUT0101', 'Unexpected Exception in User Exit', 0, 'User exit {0} had an unexpected exception: {1}', 'ediEvent.name.UT0101', 4, 'ediEvent.text.UT0101', 5, 7, 6, 1, 1, 'User exit {0} had an unexpected exception: {1}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0101')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990119, 'BCGEDIUT0401', 'Map Control String Not Found', 0, 'The control string for map {0} was not found in the database', 'ediEvent.name.UT0401', 4, 'ediEvent.text.UT0401', 5, 7, 6, 1, 1, 'The control string for map {0} was not found in the database', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0401')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990120, 'BCGEDIUT0402', 'Map Control String Invalid', 0, 'The control string for map {0} is invalid', 'ediEvent.name.UT0402', 4, 'ediEvent.text.UT0402', 5, 7, 6, 1, 1, 'The control string for map {0} is invalid', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0402')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990121, 'BCGEDIUT0403', 'Global Variable Not Found', 0, 'The global variable {0} was not found.  The map control string {1} could not be loaded', 'ediEvent.name.UT0403', 4, 'ediEvent.text.UT0403', 5, 7, 6, 1, 1, 'The global variable {0} was not found.  The map control string {1} could not be loaded', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0403')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990122, 'BCGEDIUT0404', 'Global Variable has Invalid Initial Value', 0, 'The global variable {0} has an invalid initial value.  The map control string {1} could not be loaded', 'ediEvent.name.UT0404', 4, 'ediEvent.text.UT0404', 5, 7, 6, 1, 1, 'The global variable {0} has an invalid initial value.  The map control string {1} could not be loaded', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0404')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990123, 'BCGEDIUT0405', 'Unexpected Exception Reading Map Control String', 0, 'An unexpected exception occurred while reading the map control string from the database.  Map name: {0}', 'ediEvent.name.UT0405', 4, 'ediEvent.text.UT0405', 5, 7, 6, 1, 1, 'An unexpected exception occurred while reading the map control string from the database.  Map name: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0405')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990124, 'BCGEDIUT0406', 'Unexpected Exception Reading Global Variable ', 0, 'An unexpected exception occurred while reading the global variable from the database.  Variable name: {0}', 'ediEvent.name.UT0406', 4, 'ediEvent.text.UT0406', 5, 7, 6, 1, 1, 'An unexpected exception occurred while reading the global variable from the database.  Variable name: {0}', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0406')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990125, 'BCGEDIUT0407', 'Database Error Reading Map Control String', 0, 'The control string for map {0} could not be loaded because of a database error', 'ediEvent.name.UT0407', 4, 'ediEvent.text.UT0407', 5, 7, 6, 1, 1, 'The control string for map {0} could not be loaded because of a database error', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0407')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990126, 'BCGEDIUT0501', 'Input Document for Transformation is Empty', 0, 'The input document for the transformation is empty', 'ediEvent.name.UT0501', 4, 'ediEvent.text.UT0501', 5, 7, 6, 1, 1, 'The input document for the transformation is empty', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIUT0501')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990127, 'BCGEDIVA0001', 'Mandatory Data Element Missing', 0, 'Mandatory data element is missing', 'ediEvent.name.VA0001', 4, 'ediEvent.text.VA0001', 5, 7, 6, 1, 1, 'Mandatory data element is missing', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0001')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990128, 'BCGEDIVA0002', 'Data Element Too Long', 0, 'Data element is too long', 'ediEvent.name.VA0002', 4, 'ediEvent.text.VA0002', 5, 7, 6, 1, 1, 'Data element is too long', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0002')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990129, 'BCGEDIVA0003', 'Data element Too Short', 0, 'Data element is too short', 'ediEvent.name.VA0003', 4, 'ediEvent.text.VA0003', 5, 7, 6, 1, 1, 'Data element is too short', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0003')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990130, 'BCGEDIVA0004', 'Coded Value Not Found in Validation Table', 0, 'Coded value not found in validation table', 'ediEvent.name.VA0004', 4, 'ediEvent.text.VA0004', 5, 7, 6, 1, 1, 'Coded value not found in validation table', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0004')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990131, 'BCGEDIVA0010', 'Failed ''Paired'' (P) Condition', 0, 'Failed ''paired'' (P) condition', 'ediEvent.name.VA0010', 4, 'ediEvent.text.VA0010', 5, 7, 6, 1, 1, 'Failed ''paired'' (P) condition', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0010')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990132, 'BCGEDIVA0011', 'Failed ''Required'' (R) Condition', 0, 'Failed ''required'' (R) condition', 'ediEvent.name.VA0011', 4, 'ediEvent.text.VA0011', 5, 7, 6, 1, 1, 'Failed ''required'' (R) condition', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0011')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990133, 'BCGEDIVA0012', 'Failed ''Exclusive'' (E) Condition', 0, 'Failed ''exclusive'' (E) condition', 'ediEvent.name.VA0012', 4, 'ediEvent.text.VA0012', 5, 7, 6, 1, 1, 'Failed ''exclusive'' (E) condition', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0012')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990134, 'BCGEDIVA0013', 'Failed ''Conditional'' (C) Condition', 0, 'Failed ''conditional'' (C) condition', 'ediEvent.name.VA0013', 4, 'ediEvent.text.VA0013', 5, 7, 6, 1, 1, 'Failed ''conditional'' (C) condition', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0013')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990135, 'BCGEDIVA0014', 'Failed ''List Conditional'' (L) Condition', 0, 'Failed ''list conditional'' (L) condition', 'ediEvent.name.VA0014', 4, 'ediEvent.text.VA0014', 5, 7, 6, 1, 1, 'Failed ''list conditional'' (L) condition', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0014')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990136, 'BCGEDIVA0015', 'Mandatory Composite Element Missing', 0, 'Mandatory composite element is missing', 'ediEvent.name.VA0015', 4, 'ediEvent.text.VA0015', 5, 7, 6, 1, 1, 'Mandatory composite element is missing', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0015')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990137, 'BCGEDIVA0025', 'Duplicate Transaction or Message in Interchange or Group', 0, 'Duplicate transction set or message within current interchange or functional group', 'ediEvent.name.VA0025', 4, 'ediEvent.text.VA0025', 5, 7, 6, 1, 1, 'Duplicate transction set or message within current interchange or functional group', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0025')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990138, 'BCGEDIVA0030', 'Data Element Failed Character Set Validation', 0, 'Data element failed character set validation', 'ediEvent.name.VA0030', 4, 'ediEvent.text.VA0030', 5, 7, 6, 1, 1, 'Data element failed character set validation', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0030')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990139, 'BCGEDIVA0031', 'Invalid Numeric Element', 0, 'Invalid numeric element', 'ediEvent.name.VA0031', 4, 'ediEvent.text.VA0031', 5, 7, 6, 1, 1, 'Invalid numeric element', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0031')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990140, 'BCGEDIVA0032', 'Invalid Real Numeric Element', 0, 'Invalid real numeric element', 'ediEvent.name.VA0032', 4, 'ediEvent.text.VA0032', 5, 7, 6, 1, 1, 'Invalid real numeric element', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0032')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990141, 'BCGEDIVA0033', 'Invalid Date Element', 0, 'Invalid date element', 'ediEvent.name.VA0033', 4, 'ediEvent.text.VA0033', 5, 7, 6, 1, 1, 'Invalid date element', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0033')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990142, 'BCGEDIVA0034', 'Invalid Time Element', 0, 'Invalid time element', 'ediEvent.name.VA0034', 4, 'ediEvent.text.VA0034', 5, 7, 6, 1, 1, 'Invalid time element', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0034')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990143, 'BCGEDIVA0050', 'Too Many Elements or Unexpected Element in Segment', 0, 'Too many elements or unexpected element in segment', 'ediEvent.name.VA0050', 4, 'ediEvent.text.VA0050', 5, 7, 6, 1, 1, 'Too many elements or unexpected element in segment', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0050')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990144, 'BCGEDIVA0051', 'Unrecognized Segment Id', 0, 'Unrecognized segment id', 'ediEvent.name.VA0051', 4, 'ediEvent.text.VA0051', 5, 7, 6, 1, 1, 'Unrecognized segment id', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0051')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990145, 'BCGEDIVA0052', 'Mandatory Segment Missing', 0, 'Mandatory segment missing', 'ediEvent.name.VA0052', 4, 'ediEvent.text.VA0052', 5, 7, 6, 1, 1, 'Mandatory segment missing', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0052')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990146, 'BCGEDIVA0054', 'Loop Repeats More Times than Defined by Standard', 0, 'Loop repeats more times than defined by standard', 'ediEvent.name.VA0054', 4, 'ediEvent.text.VA0054', 5, 7, 6, 1, 1, 'Loop repeats more times than defined by standard', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0054')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990147, 'BCGEDIVA0055', 'Segment Repeats More Times than Defined by Standard', 0, 'Segment repeats more times than defined by standard', 'ediEvent.name.VA0055', 4, 'ediEvent.text.VA0055', 5, 7, 6, 1, 1, 'Segment repeats more times than defined by standard', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0055')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990148, 'BCGEDIVA0101', 'Transaction Set or Message Control Numbers Mismatch', 0, 'Transaction set or message control numbers do not match in header and trailer', 'ediEvent.name.VA0101', 4, 'ediEvent.text.VA0101', 5, 7, 6, 1, 1, 'Transaction set or message control numbers do not match in header and trailer', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0101')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990149, 'BCGEDIVA0102', 'Transaction Set or Message Trailer Missing or Invalid', 0, 'Transaction set or message trailer missing or invalid', 'ediEvent.name.VA0102', 4, 'ediEvent.text.VA0102', 5, 7, 6, 1, 1, 'Transaction set or message trailer missing or invalid', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0102')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990150, 'BCGEDIVA0103', 'Transaction Set or Message Trailer Count Invalid', 0, 'Transaction set or message trailer contains an invalid segment count', 'ediEvent.name.VA0103', 4, 'ediEvent.text.VA0103', 5, 7, 6, 1, 1, 'Transaction set or message trailer contains an invalid segment count', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0103')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990151, 'BCGEDIVA0151', 'Functional Group Control Numbers Mismatch', 0, 'Functional group control numbers do not match in header and trailer', 'ediEvent.name.VA0151', 4, 'ediEvent.text.VA0151', 5, 7, 6, 1, 1, 'Functional group control numbers do not match in header and trailer', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0151')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990152, 'BCGEDIVA0152', 'Functional Group Trailer Missing or Invalid', 0, 'Functional group trailer missing or invalid', 'ediEvent.name.VA0152', 4, 'ediEvent.text.VA0152', 5, 7, 6, 1, 1, 'Functional group trailer missing or invalid', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0152')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990153, 'BCGEDIVA0153', 'Functional Group Trailer Count Invalid', 0, 'Functional group trailer contains invalid transaction set or message count', 'ediEvent.name.VA0153', 4, 'ediEvent.text.VA0153', 5, 7, 6, 1, 1, 'Functional group trailer contains invalid transaction set or message count', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0153')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990154, 'BCGEDIVA0158', 'Duplicate Group in Interchange', 0, 'Duplicate group detected within current interchange', 'ediEvent.name.VA0158', 4, 'ediEvent.text.VA0158', 5, 7, 6, 1, 1, 'Duplicate group detected within current interchange', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0158')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990155, 'BCGEDIVA0202', 'Interchange Trailer Missing or Invalid', 0, 'Interchange trailer missing or invalid', 'ediEvent.name.VA0202', 4, 'ediEvent.text.VA0202', 5, 7, 6, 1, 1, 'Interchange trailer missing or invalid', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0202')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990156, 'BCGEDIVA0203', 'Interchange Control Numbers Mismatch', 0, 'Interchange control numbers do not match in header and trailer', 'ediEvent.name.VA0203', 4, 'ediEvent.text.VA0203', 5, 7, 6, 1, 1, 'Interchange control numbers do not match in header and trailer', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0203')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990157, 'BCGEDIVA0205', 'Interchange Trailer Count Invalid', 0, 'Interchange trailer contains an invalid group or message count', 'ediEvent.name.VA0205', 4, 'ediEvent.text.VA0205', 5, 7, 6, 1, 1, 'Interchange trailer contains an invalid group or message count', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0205')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990158, 'BCGEDIVA0211', 'Duplicate Interchange', 0, 'Duplicate interchange detected', 'ediEvent.name.VA0211', 4, 'ediEvent.text.VA0211', 5, 7, 6, 1, 1, 'Duplicate interchange detected', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0211')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990159, 'BCGEDIVA0991', 'Required Property or Content Element Not Found', 0, 'Required property or content element {0} not found', 'ediEvent.name.VA0991', 4, 'ediEvent.text.VA0991', 5, 7, 6, 1, 1, 'Required property or content element {0} not found', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0991')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990160, 'BCGEDIVA0992', 'No Message Properties Found', 0, 'No message properties found', 'ediEvent.name.VA0992', 4, 'ediEvent.text.VA0992', 5, 7, 6, 1, 1, 'No message properties found', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0992')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990161, 'BCGEDIVA0993', 'Metadata Not Found', 0, 'Metadata not found', 'ediEvent.name.VA0993', 4, 'ediEvent.text.VA0993', 5, 7, 6, 1, 1, 'Metadata not found', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0993')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990162, 'BCGEDIVA0994', 'Empty Transaction Set or Message', 0, 'EDI transaction set or message is empty', 'ediEvent.name.VA0994', 4, 'ediEvent.text.VA0994', 5, 7, 6, 1, 1, 'EDI transaction set or message is empty', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0994')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990163, 'BCGEDIVA0995', 'Fatal Parser Error', 0, 'Fatal parser error encountered', 'ediEvent.name.VA0995', 4, 'ediEvent.text.VA0995', 5, 7, 6, 1, 1, 'Fatal parser error encountered', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0995')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990164, 'BCGEDIVA0997', 'Unknown Flow Direction ', 0, 'Unknown flow direction {0} specified', 'ediEvent.name.VA0997', 4, 'ediEvent.text.VA0997', 5, 7, 6, 1, 1, 'Unknown flow direction {0} specified', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0997')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990165, 'BCGEDIVA0998', 'Unsupported Syntax Type ', 0, 'Unsupported syntax type {0} specified', 'ediEvent.name.VA0998', 4, 'ediEvent.text.VA0998', 5, 7, 6, 1, 1, 'Unsupported syntax type {0} specified', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0998')
/

INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, eventnameresourcebundleid, textkey, textkeyresourcebundleid, visibility, severity, alertableflg, rowts, externaldesc, externaldescresourcebundleid)
SELECT 990166, 'BCGEDIVA0999', 'Unknown Object Received', 0, 'Unknown object of type {0} received', 'ediEvent.name.VA0999', 4, 'ediEvent.text.VA0999', 5, 7, 6, 1, 1, 'Unknown object of type {0} received', 3
FROM dual 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIVA0999')
/

--BGTRK00047349
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )   
Select '240029', 'BCG240029', 'Certificate not found'        , 'Certificate not found'        , 
'event.name.240029', 'event.text.240029'        , 7, 9, 0        , 
'This event is generated when there is no Valid certificate in the database related to Encryption/Decryption/Signing/ Verification.'     
From dual    
Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240029)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC ) 
Select '240023', 'BCG240023', 'Certificate not found'        , 
'CertPath build failed for certificate with serial number: {0}; SubjectDN: {1} and IssuerDN: {2}' , 
'event.name.240023', 'event.text.240023'        , 7, 9, 0        , 
'This event is generated when Certificate path could not be built.'     
From dual    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240023)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC ) 
Select '240025', 'BCG240025', 'WBICSecurityManager initialization failed. \n Exception: {1}'  , 
'WBICSecurityManager initialization failed. \n Exception: {1}' , 'event.name.240025', 'event.text.240025'        , 
7, 9, 1        , 'This event is generated when WBICSecurity Manager could not be initialized.'     
From dual    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240025)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )   
Select '240026', 'BCG240026', 'Certificate is not yet valid'  , 'Certificate is not yet valid: Serial number: {0}, 
Subject DN: {1}, Issuer DN: {2}' , 'event.name.240026', 'event.text.240026'        , 7, 9, 1        , 
'This event is generated when Certificate is not yet valid.'     
From dual    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240026)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )   
Select '240027', 'BCG240027', 'Certificate is expired'  , 'Certificate is expired:  Serial number: {0}, Subject DN: {1}, 
Issuer DN: {2}' , 'event.name.240027', 'event.text.240027'        , 7, 1, 0        , 
'This event is generated when Certificate has expired.'    
From dual    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240027)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )   Select '108011', 'BCG108011', 'Secondary has been changed to Primary'  , 'The Secondary certificate Serial No = {1} and Issuer Name = {2} has been changed to primary Certificate' , 'event.name.108011', 'event.text.108011'        , 7, 1, 0        , 'This event is generated when Secondary Certificate has been changed to Primary.'     From dual    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108011)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )   Select '240030', 'BCG240030', 'No valid signing certificate found'        , 'No valid signing certificate found'        , 'event.name.240030', 'event.text.240030'        , 7, 9, 1        , 'This event is generated when there is no Valid sIGNING certificate in the database '     From dual    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240030)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )   Select '240032', 'BCG240032', 'No valid encryption certificate found'        , 'No valid encryption certificate found'        , 'event.name.240032', 'event.text.240032'        , 7, 9, 1        , 'This event is generated when there is no Valid encryption certificate in the database '     From dual    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240032)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )   Select '240033', 'BCG240033', 'No valid SSL client certificate found'        , 'No valid encryption certificate found'        , 'event.name.240033', 'event.text.240033'        , 7, 9, 1        , 'This event is generated when there is no Valid SSL client certificate in the database '     From dual    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240033)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '107000', 'BCG107000', 'Submit  Document for Resend'
        , 'Submit  Document for Resend: Request the system to resend the documents'
        , 'event.name.107000', 'event.text.107000'
        , 1, 1, 1
        , 'This event is generated when the console submits the documents for resend'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 107000)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 281001, 'BCG281001', 'Console resend Placed Document In Queue: {0}'
        , 'Console resend Placed Document In Queue: {0}'
        , 'event.name.281001', 'event.text.281001'
        , 7, 9, 1
        , 'This event is generated when the console resend places a document in a queue.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 281001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 281002, 'BCG281002', 'Console Resend Document Already In Queue :{0}'
        , 'Console Resend Document Already In Queue :{0}'
        , 'event.name.281002', 'event.text.281002'
        , 7, 9, 1
        , 'This event is generated when the console resend document is already in a queue.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 281002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 103025, 'BCG103025', 'New Gateway Type added'
        , 'A new gateway type has been added'
        , 'event.name.103025', 'event.text.103025'
        , 7, 1, 1
        , 'This event is generated when a new gateway type has been added.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103025)
/

--47410
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310004, 'BCG310004', 'Enveloper configuration changed'
        , 'Enveloper configuration changed'
        , 'event.name.310004', 'event.text.310004'
        , 1, 1, 1
        , 'This event is generated when the Enveloper Configuration information is updated.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310004)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310005, 'BCG310005', 'Enveloper scehdule information changed'
        , 'Enveloper scehdule information changed'
        , 'event.name.310005', 'event.text.310005'
        , 1, 1, 1
        , 'This event is generated when the Enveloper Schedule information is updated.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310005)
/

--47408
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310011, 'BCG310011', 'Unable to work with any Trading Partner pair'
        , 'Unable to work with any Trading Partner pair'
        , 'event.name.310011', 'event.text.310011'
        , 7, 1, 0
        , 'Unable to work with any Trading Partner pair.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310011)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310012, 'BCG310012', 'Error in removing the transactions from the Envelope Queue'
        , 'Error in removing the transactions from the Envelope Queue'
        , 'event.name.310012', 'event.text.310012'
        , 7, 1, 0
        , 'Error in removing the transactions from the Envelope Queue.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310012)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310013, 'BCG310013', 'Error while working with transactions'
        , 'Error while working with transactions'
        , 'event.name.310013', 'event.text.310013'
        , 7, 1, 0
        , 'Error while working with transactions.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310013)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310014, 'BCG310014', 'Error while getting the serialized transaction'
        , 'Error while getting the serialized transaction'
        , 'event.name.310014', 'event.text.310014'
        , 7, 1, 0
        , 'Error while getting the serialized transaction.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310014)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310015, 'BCG310015', 'Error in introducing envelope into the flow'
        , 'Error in introducing envelope into the flow'
        , 'event.name.310015', 'event.text.310015'
        , 7, 1, 0
        , 'Error in introducing envelope into the flow.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310015)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310016, 'BCG310016', 'Unable to start the worker for enveloping transcations'
        , 'Unable to start the worker for enveloping transcations'
        , 'event.name.310016', 'event.text.310016'
        , 7, 1, 0
        , 'Unable to start the worker for enveloping transcations.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310016)
/
--47475
INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
SELECT 880100, 'BCGEDIEM0100','Transcript File Contents
',0,'Transcript File Contents. {0}
','ediEvent.name.EM0100','ediEvent.text.EM0100',7, 3, 1, 1, 'Transcript File Contents. {0}
' FROM DUAL 
 WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0100')
/
  INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
 SELECT 880101, 'BCGEDIEM0101','An exception occcured while retrieving certificates
 ',0,'An exception occcured while retrieving certificates . Details : {0}
 ','ediEvent.name.EM0101','ediEvent.text.EM0101',7, 6, 1, 1, 'An exception occcured while retrieving certificates . Details : {0}' FROM DUAL 
  WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0101')
/
   INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
  SELECT 880102, 'BCGEDIEM0102','Exception when reading the transcript file
  ',0,'Exception when reading the transcript file . Details : {0}
  ','ediEvent.name.EM0102','ediEvent.text.EM0102',7, 6, 1, 1, 'Exception when reading the transcript file . Details : {0}
'  FROM DUAL 
   WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0102')
/
  INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
   SELECT 880103, 'BCGEDIEM0103','Required attribute is null
   ',0,'Required attribute {0} is null .','ediEvent.name.EM0103','ediEvent.text.EM0103',7, 6, 1, 1, 'Required attribute {0} is null .'  FROM DUAL 
    WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0103')
/ 
  INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880104, 'BCGEDIEM0104','Exception occured when trying to write the file to be sent',0,
    'Exception occured when trying to write the file to be sent to a temporary location .  Details : {0}
','ediEvent.name.EM0104','ediEvent.text.EM0104',7, 6, 1, 1, 'Exception occured when trying to write the file to be sent to a temporary location .  Details : {0}'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0104')
/ 
  INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880105, 'BCGEDIEM0105','Certificates need to be uploaded to certificate repository',0,'Certificates need to be uploaded to the certificate repository .
','ediEvent.name.EM0105','ediEvent.text.EM0105',7, 6, 1, 1, 'Certificates need to be uploaded to the certificate repository .'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0105')
/ 
   INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880106, 'BCGEDIEM0106','Could not load private key file.  Alias not found
    ',0,'Could not load private key file.  Alias not found.
','ediEvent.name.EM0106','ediEvent.text.EM0106',7, 6, 1, 1, 'Could not load private key file.  Alias not found.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0106')
/
INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
   SELECT 880107, 'BCGEDIEM0107','Client Certificate (local certificate) validation has failed',0,'Client Certificate (local certificate) validation has failed,certificate could be invalid or revoked.
','ediEvent.name.EM0107','ediEvent.text.EM0107',7, 6, 1, 1, 'Client Certificate (local certificate) validation has failed,certificate could be invalid or revoked.
'  FROM DUAL 
    WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0107')
/    
  INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
   SELECT 880108, 'BCGEDIEM0108','Security Exception
   ',0,'Security Exception. Details : {0}
','ediEvent.name.EM0108','ediEvent.text.EM0108',7, 6, 1, 1, 'Security Exception. Details : {0}
'  FROM DUAL 
    WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0108')
/ 
     INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880109, 'BCGEDIEM0109','The temporary directory value provided for receiver is null',0,'The temporary directory value provided for the receiver is null 
','ediEvent.name.EM0109','ediEvent.text.EM0109',7, 6, 1, 1, 'The temporary directory value provided for the receiver is null '  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0109')
/
  INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
   SELECT 880110, 'BCGEDIEM0110','Required attribute is null
   ',0,'The BusinessDoucument Array passed is null 
','ediEvent.name.EM0110','ediEvent.text.EM0110',7, 6, 1, 1, 'The BusinessDoucument Array passed is null 
'  FROM DUAL 
    WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0110')
/
INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
   SELECT 880111, 'BCGEDIEM0111','Handler Config is null
',0,'The handler config object passed to mediation is null.
','ediEvent.name.EM0111','ediEvent.text.EM0111',7, 6, 1, 1, 'The handler config object passed to mediation is null.
'  FROM DUAL 
    WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0111')
/ 
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880112, 'BCGEDIEM0112','Input file is null
    ',0,'The input file is null.
','ediEvent.name.EM0112','ediEvent.text.EM0112',7, 6, 1, 1, 'The input file is null.
'  FROM DUAL      WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0112')
/ 
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880113, 'BCGEDIEM0113','A splitter exception was received.
    ',0,'Splitter Exception was received. Details : {0}','ediEvent.name.EM0113','ediEvent.text.EM0113',7, 6, 1, 1, 'Splitter Exception was received. Details : {0}
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0113')
/ 
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880114, 'BCGEDIEM0114','Cannot find reader.
    ',0,'Cannot find reader.','ediEvent.name.EM0114','ediEvent.text.EM0114',7, 6, 1, 1, 'Cannot find reader.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0114')
/ 
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880120, 'BCGEDIEM0120','Error initializing RODScanner. Details : {0}
    ',0,'Error initializing RODScanner','ediEvent.name.EM0120','ediEvent.text.EM0120',7, 6, 1, 1, 'Error initializing RODScanner. Details : {0}
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0120')
/
INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
   SELECT 880125, 'BCGEDIEM0125','Network delivery acknowledgement received from IBM VAN.
   ',0,'Network delivery acknowledgement received from IBM VAN.',
'ediEvent.name.EM0125','ediEvent.text.EM0125',7, 3, 1, 1, 'Network delivery acknowledgement received from IBM VAN. Details are Message Id = {0}, Sender Account = {1}, Sender User Id = {2}, Message Type = {3}, Message Text = {4}, User Message Name = {5}, User Message Class = {6}, User Message Sequence Number = {7}, Date = {8}, Time = {9}, Message Receiver = {10}
'  FROM DUAL 
    WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0125')
/
--changed severity to 1 - 47946
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
     SELECT 880126, 'BCGEDIEM0126','Network receipt acknowledgement received from IBM VAN.
     ',0,'Network receipt acknowledgement received from IBM VAN.',
'ediEvent.name.EM0126','ediEvent.text.EM0126',7, 1, 1, 1, 'Network receipt acknowledgement received from IBM VAN.  Details are Message Id = {0}, Sender Account = {1}, Sender User Id = {2}, Message Type = {3}, Message Text = {4}, User Message Name = {5}, User Message Class = {6}, User Message Sequence Number = {7}, Date = {8}, Time = {9}, Message Receiver = {10}
'  FROM DUAL 
      WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0126')
/
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880127, 'BCGEDIEM0127','Network purge acknowledgement received from IBM VAN.
    ',0,'Network purge acknowledgement received from IBM VAN.',
'ediEvent.name.EM0127','ediEvent.text.EM0127',7, 3, 1, 1, 'Network purge acknowledgement received from IBM VAN. Details are Message Id = {0}, Sender Account = {1}, Sender User Id = {2}, Message Type = {3}, Message Text = {4}, User Message Name = {5}, User Message Class = {6}, User Message Sequence Number = {7}, Date = {8}, Time = {9}, Purge Text = {10}, Message Receiver = {11}
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0127')
/
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880128, 'BCGEDIEM0128','Network error message received from IBM VAN. 
    ',0,'Network error message received from IBM VAN. Details are Message Id = {0}, Message Description = {1}, Severity Code = {2}.
','ediEvent.name.EM0128','ediEvent.text.EM0128',7, 6, 1, 1, 'Network error message received from IBM VAN. Details are Message Id = {0}, Message Description = {1}, Severity Code = {2}.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0128')
/ 
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880200, 'BCGEDIEM0200','Database Connection Error.
    ',0,'Invalid or missing database connection object in the context.
','ediEvent.name.EM0200','ediEvent.text.EM0200',7, 6, 1, 1, 'Invalid or missing database connection object in the context.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0200')
/
   INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
      SELECT 880201, 'BCGEDIEM0201','I/O Error While Writting to File.
      ',0,'Unable to Create a File in PROCESS DIR {0}
','ediEvent.name.EM0201','ediEvent.text.EM0201',7, 6, 1, 1, 'Unable to Create a File in PROCESS DIR {0}
'  FROM DUAL 
       WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0201')
/
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880202, 'BCGEDIEM0202','Unable to Serialize AbsDocument.
    ',0,'Parser Exception Occured while trying to serialize the AbsDocument.
','ediEvent.name.EM0202','ediEvent.text.EM0202',7, 6, 1, 1, 'Parser Exception Occured while trying to serialize the AbsDocument.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0202')
/ 
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880203, 'BCGEDIEM0203','Exception occurred while serializing AbsDocument.
    ',0,'Exception Occured while trying to serialize the AbsDocument.
','ediEvent.name.EM0203','ediEvent.text.EM0203',7, 6, 1, 1, 'Exception Occured while trying to serialize the AbsDocument.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0203')
/
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880204, 'BCGEDIEM0204','Unable To Introduce Business Document
    ',0,'Unable to Introduce Business Document with ID {0} back into workflow.
','ediEvent.name.EM0204','ediEvent.text.EM0204',7, 6, 1, 1, 'Unable to Introduce Business Document with ID {0} back into workflow.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0204')
/
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880205, 'BCGEDIEM0205','Unable to find state information.
    ',0,'Unable to find state information in the state management service.
','ediEvent.name.EM0205','ediEvent.text.EM0205',7, 6, 1, 1, 'Unable to find state information in the state management service.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0205')
/ 
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880150, 'BCGEDIEM0150','The document passed does not apply to EDIAckHandler
    ',0,'The document passed does not apply to EDIAckHandler
','ediEvent.name.EM0150','ediEvent.text.EM0150',7, 6, 1, 1, 'The document passed does not apply to EDIAckHandler
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0150')
/ 
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880151, 'BCGEDIEM0151','Error processing edi ack
    ',0,'Error processing edi ack. Message got in Error terminal.
','ediEvent.name.EM0151','ediEvent.text.EM0151',7, 6, 1, 1, 'Error processing edi ack. Message got in Error terminal.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0151')
/ 
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880152, 'BCGEDIEM0152','Cannot get database Connection from context
    ',0,'Cannot get database Connection from context
','ediEvent.name.EM0152','ediEvent.text.EM0152',7, 6, 1, 1, 'Cannot get database Connection from context
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0152')
/ 
--changed from sev 6 to sev 3 for 60826
--71174 - changed internal description
 INSERT INTO LG_EVENTCD (eventcd, eventcode, eventname, providerflag, internaldesc, eventnamekey, textkey, visibility, severity, alertableflg, rowts, externaldesc)
    SELECT 880153, 'BCGEDIEM0153','FA Reconciliation Failure
    ',0,'No documents reconciled for EDI Acknowledgement with envelope activity id = {0}, control numbers interchange = {1}, group = {2}, transaction = {3}. Cannot locate documents for control numbers interchange = {4}, group = {5}, transaction = {6}.
 .','ediEvent.name.EM0153','ediEvent.text.EM0153',7, 3, 1, 1, 'FA Reconciliation Failure. No activity ids found for the transactions found in the EDI Acknowledgement.
'  FROM DUAL 
     WHERE NOT EXISTS (SELECT * FROM lg_eventcd WHERE eventcode = 'BCGEDIEM0153')
/

--47512
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310001, 'BCG310001', 'EDI Transaction Queued for Enveloping'
        , 'EDI Transaction Queued for Enveloping'
        , 'event.name.310001', 'event.text.310001'
        , 1, 1, 0
        , 'EDI Transaction Queued for Enveloping.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310001)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310002, 'BCG310002', 'EDI Transaction Enveloped'
        , 'EDI Transaction Enveloped'
        , 'event.name.310002', 'event.text.310002'
        , 1, 1, 0
        , 'EDI Transaction Enveloped.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310002)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310003, 'BCG310003', 'EDI Transaction Enveloping Failed'
        , 'EDI Transaction Enveloping Failed'
        , 'event.name.310003', 'event.text.310003'
        , 1, 1, 0
        , 'EDI Transaction Enveloping Failed.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310003)
/

--58747
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 310021, 'BCG310021', 'DocMgr Administration Information Updated'
        , 'Updated document acquisition engine properties: {0}'
        , 'event.name.310021', 'event.text.310021'
        , 1, 3, 0
        , 'Document Manager Administration Information Updated.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 310021)
/

--61496
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 117, 'BCGEDIUP0117', 'Character Decoding Error'
        , 'Character decoding error at offset {0}, hex data is: "{1}".'
        , 'ediEvent.name.UP0117', 'ediEvent.text.UP0117'
        , 7, 6, 1
        , 'Character decoding error at offset {0}, hex data is: "{1}".'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 117)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 41, 'BCGEDIUT0041', 'Translation Table Lookup Failed'
        , 'Translation table lookup entry {0} not found in {1}.  The default value returned was {2}.'
        , 'ediEvent.name.UT0041', 'ediEvent.text.UT0041'
        , 7, 6, 1
        , 'Translation table lookup entry {0} not found in {1}.  The default value returned was {2}.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 41)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 16, 'BCGEDIVA0016', 'Composite Data Element Maximum Repetitions Exceeded'
        , 'Composite element repeats more times than allowed, name = {0}, segment name = {1}, segment position = {2}, sending partner id/qualfier = {3}/{4}, receiving partner id/qualifier = {5}/{6}, control numbers = {7}. Rep NUMBER = {8}, max defined reps = {9}.'
        , 'ediEvent.name.VA0016', 'ediEvent.text.VA0016'
        , 7, 6, 1
        , 'Composite element repeats more times than defined by standard, element name = {0}, segment name = {1}, segment position = {2}, sending trading partner id/qualfier = {3}/{4}, receiving trading partner id/qualifier = {5}/{6}, control numbers = {7}. Repetition number = {8}, maximum defined repetitions = {9}.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 16)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 35, 'BCGEDIVA0035', 'Data Element Maximum Repetitions Exceeded'
        , 'Element repeats more times than allowed, element name = {0}, segment name = {1}, segment position = {2}, sending partner id/qualfier = {3}/{4}, receiving partner id/qualifier = {5}/{6}, control numbers = {7}. Rep number = {8}, max defined reps = {9}.'
        , 'ediEvent.name.VA0035', 'ediEvent.text.VA0035'
        , 7, 6, 1
        , 'Element repeats more times than defined by standard, element name = {0}, segment name = {1}, segment position = {2}, sending trading partner id/qualfier = {3}/{4}, receiving trading partner id/qualifier = {5}/{6}, control numbers = {7}. Repetition number = {8}, maximum defined repetitions = {9}.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 35)
/

--60763
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 981, 'BCGEDIVA0981', 'User-specified validation map not found.'
        , 'User-specified validation map {0} not found.'
        , 'ediEvent.name.VA0981', 'ediEvent.text.VA0981'
        , 7, 6, 1
        , 'User-specified validation map {0} not found.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 981)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 982, 'BCGEDIVA0982', 'Service segment validation map not found.'
        , 'Service segment validation map {0} not found.'
        , 'ediEvent.name.VA0982', 'ediEvent.text.VA0982'
        , 7, 6, 1
        , 'Service segment validation map {0} not found.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 982)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 983, 'BCGEDIVA0983', 'Service segment validation not supported for document syntax'
        , 'Service segment validation not supported for document syntax.'
        , 'ediEvent.name.VA0983', 'ediEvent.text.VA0983'
        , 7, 6, 1
        , 'Service segment validation not supported for document syntax.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 983)
/

--61817
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 210204, 'BCG210204', 'Connection Lookup Successful'
        , 'The participant connection lookup is successful'
        , 'event.name.210204', 'event.text.210204'
        , 7, 1, 0
        , 'This event is generated when a participant connection lookup is successful.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210204)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 106506, 'BCG106506', 'Proxy Configuration Changed'
        , ' Proxy :{0} has been changed.'
        , 'event.name.106506', 'event.text.106506'
        , 7, 3, 0
        , 'Proxy Configuration Changed.'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106506)
/

--54973
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 118, 'BCGEDIUP0118', 'Character encoding error'
        , 'error encoding "{2}" into character set {3}'
        , 'ediEvent.name.UP0118', 'ediEvent.text.UP0118'
        , 7, 6, 1
        , 'error encoding "{2}" into character set {3}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 118)
/

--48414, 48608
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, 
VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, PROVIDERFLAG )
   Select '210205', 'BCG210205', 'Document Processing Cancelled'
        , 'Document processing cancelled. Reason: Processing for one or more associated documents failed'
        , 'event.name.210205', 'event.text.210205'
        , 7, 6, 0
        , 'This event is generated when the document processing is cancelled due to 
           the "Discard envelope on Error" attribute set to "Yes".'
        , 0
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210205)
/

--55194
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, 
VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, PROVIDERFLAG )
   Select 54, 'BCGEDIEV0054', 'Multiple Interchanges Error'
        , 'The {0} deenveloper detected an error:  Multiple interchanges were detected but not allowed.'
        , 'ediEvent.name.EV0054', 'ediEvent.text.EV0054'
        , 7, 6, 1
        , 'The {0} deenveloper detected an error:  Multiple interchanges were detected but not allowed.'
        , 0
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 54)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, 
VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, PROVIDERFLAG )
   Select 55, 'BCGEDIEV0055', 'Translation Table Lookup Warning'
        , 'A translation table lookup returned no entry while enveloping a message.  Translation table: {0}, value: {1}.'
        , 'ediEvent.name.EV0055', 'ediEvent.text.EV0055'
        , 7, 3, 1
        , 'A translation table lookup returned no entry while enveloping a message.  Translation table: {0}, value: {1}.'
        , 0
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 55)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, 
VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, PROVIDERFLAG )
   Select 56, 'BCGEDIEV0056', 'Translation Table Lookup Warning'
        , 'A translation table lookup returned no entry while deenveloping a message.  Translation table: {0}, value: {1}, group/transaction control number {2}.'
        , 'ediEvent.name.EV0056', 'ediEvent.text.EV0056'
        , 7, 3, 1
        , 'A translation table lookup returned no entry while deenveloping a message.  Translation table: {0}, value: {1}, group/transaction control number {2}.'
        , 0
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 56)
/

--55395 NEW event codes
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, 
VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, PROVIDERFLAG )
   Select 21, 'BCGEDIVA0021', 'Transaction rejected due to interchange envelope error'
        , 'Transaction set or message rejected due to interchange envelope error, sending trading partner id/qualfier = {0}/{1}, receiving trading partner id/qualifier ={2}/{3}, control numbers = {4}.'
        , 'ediEvent.name.VA0021', 'ediEvent.text.VA0021'
        , 7, 6, 1
        , 'Transaction set or message rejected due to interchange envelope error, sending trading partner id/qualfier = {0}/{1}, receiving trading partner id/qualifier ={2}/{3}, control numbers = {4}.'
        , 0
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 21)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, 
VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, PROVIDERFLAG )
   Select 22, 'BCGEDIVA0022', 'Transaction rejected due to group envelope error'
        , 'Transaction set or message rejected due to group envelope error, sending trading partner id/qualfier = {0}/{1}, receiving trading partner id/qualifier ={2}/{3}, control numbers = {4}.'
        , ' ediEvent.name.VA0022', 'ediEvent.text.VA0022'
        , 7, 6, 1
        , 'Transaction set or message rejected due to group envelope error, sending trading partner id/qualfier = {0}/{1}, receiving trading partner id/qualifier ={2}/{3}, control numbers = {4}.'
        , 0
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 22)
/

--48608 - Description for BCG210205 needs to change
UPDATE lg_eventcd
   SET INTERNALDESC = 'Document processing cancelled. Reason: Processing for one or more associated documents failed.'
   Where EVENTCD = 210205
/


--49072 - Event BCG240024 included
INSERT INTO lg_EventCd ( EVENTCD
    ,EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC )
SELECT 240024, 'BCG240024', 'CertPath validation failed'
    , 'CertPath validation failed'
    , 'event.name.240024', 'event.text.240024'
    , 7, 9, 0
    , 'Certpath validation failed for the certificate with Serial number: {0}, Subject DN: {1}, Issuer DN: {2}, Reason: {3}'
FROM DUAL
WHERE NOT EXISTS (SELECT * FROM lg_EventCd WHERE EVENTCD = 240024)
/

--55533 edi report events
INSERT INTO lg_EventCd ( EVENTCD
    ,EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC )
SELECT 1, 'BCGEDIFA0001', 'Functional Acknowledgement with Reject status code received'
    , 'Functional Acknowledgement with Reject status code from partner {0} to partner {1}, FA Control Numbers [Interchange {2}, Group {3}, Transaction {4}], reference ID {5}'
    , 'ediEvent.name.FA0001', 'ediEvent.text.FA0001'
    , 7, 6, 1
    , 'Functional Acknowledgement with Reject status code from partner {0} to partner {1}, FA Control Numbers [Interchange {2}, Group {3}, Transaction {4}], reference ID {5}'
FROM dual
WHERE NOT EXISTS (SELECT * FROM lg_EventCd WHERE EVENTCD = 1)
/

INSERT INTO lg_EventCd ( EVENTCD
    ,EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC )
SELECT 2, 'BCGEDIFA0002', 'Transaction rejected'
    , 'Transaction Reference ID {0} from Partner {1} to Partner {2}, Control Numbers [Interchange {3}, Group {4}, Transaction {5}] rejected with Functional Acknowledgement Status code {6} and Status text {7}'
    , 'ediEvent.name.FA0002', 'ediEvent.text.FA0002'
    , 7, 6, 1
    , 'Transaction Reference ID {0} from Partner {1} to Partner {2}, Control Numbers [Interchange {3}, Group {4}, Transaction {5}] rejected with Functional Acknowledgement Status code {6} and Status text {7}'
FROM dual
WHERE NOT EXISTS (SELECT * FROM lg_EventCd WHERE EVENTCD = 2)
/

--55598 
INSERT INTO lg_EventCd ( EVENTCD
    ,EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC )
SELECT 100, 'BCGEDIVA0100', 'Control number missing in transaction Set or message header'
    , 'Control number missing in transaction set header, interchange control number = {0}, group control number = {1}, sending trading partner id/qualfier = {2}/{3}, receiving trading partner id/qualifier = {4}/{5}.'
    , 'ediEvent.name.VA0100', 'ediEvent.text.VA0100'
    , 7, 6, 1
    , 'Control number missing in transaction set header, interchange control number = {0}, group control number = {1}, sending trading partner id/qualfier = {2}/{3}, receiving trading partner id/qualifier = {4}/{5}.'
FROM DUAL
WHERE NOT EXISTS (SELECT * FROM lg_EventCd WHERE EVENTCD = 100)
/

INSERT INTO lg_EventCd ( EVENTCD
    ,EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC )
SELECT 150, 'BCGEDIVA0150', 'Control number missing in group header'
    , 'Control number missing in group header, interchange control number = {0}, sending trading partner id/qualfier = {1}/{2}, receiving trading partner id/qualifier = {3}/{4}.'
    , 'ediEvent.name.VA0150', 'ediEvent.text.VA0150'
    , 7, 6, 1
    , 'Control number missing in group header, interchange control number = {0}, sending trading partner id/qualfier = {1}/{2}, receiving trading partner id/qualifier = {3}/{4}.'
FROM DUAL
WHERE NOT EXISTS (SELECT * FROM lg_EventCd WHERE EVENTCD = 150)
/

INSERT INTO lg_EventCd ( EVENTCD
    ,EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC )
SELECT 200, 'BCGEDIVA0200', 'Control number missing in interchange header'
    , 'Control number missing in interchange header, sending trading partner id/qualfier = {0}/{1}, receiving trading partner id/qualifier = {2}/{3}.'
    , 'ediEvent.name.VA0200', 'ediEvent.text.VA0200'
    , 7, 6, 1
    , 'Control number missing in interchange header, sending trading partner id/qualfier = {0}/{1}, receiving trading partner id/qualifier = {2}/{3}.'
FROM DUAL
WHERE NOT EXISTS (SELECT * FROM lg_EventCd WHERE EVENTCD = 200)
/
-- Start JR23016(100616)
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 210206, 'BCG210206', 'Stopping document processing as requested by user.'
        , 'Processing of document is being stopped. Reason: User Requested stopping the processing for document {0}.'
        , 'event.name.210206', 'event.text.210206'
        , 7, 3, 0
        , 'Processing of document is being stopped. Reason: User Requested stopping the processing for document {0}.'
     FROM DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210206)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 210207, 'BCG210207', 'Failed to stop the processing.'
        , 'Stop Process was invoked but failed to stop the processing for this document as processing is already done {0}.'
        , 'event.name.210207', 'event.text.210207'
        , 7, 3, 0
        , 'Stop Process was invoked but failed to stop the processing for this document as processing is already done {0}.'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210207)
/
-- End JR23016(100616)
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 210208, 'BCG210208', 'Unable to stop document from Destination Queue.'
        , 'Unable to stop document {0} from destination queue at this moment as user requested.'
        , 'event.name.210208', 'event.text.210208'
        , 7, 6, 0
        , 'Unable to stop document {0} from destination queue at this moment as user requested.'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210208)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 210209, 'BCG210209', 'Document stopped as requested by user.'
        , 'Document  {0} stopped as requested by user.'
        , 'event.name.210209', 'event.text.210209'
        , 7, 6, 0
        , 'Document  {0} stopped as requested by user.'
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210209)
/
-- Start 100163
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 112011, 'BCG112011', 'An error occurred while sending the synchronous response.'
        , 'A network or input/output related error occurred while sending the synchronous response.'
        , 'event.name.112011', 'event.text.112011'
        , 7, 6, 1
        , 'A network or input/output related error occurred while sending the synchronous response.'
     FROM DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112011)
/
-- End 100163

-- Start 100193(100616)
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select 210053, 'BCG210053', 'Document stop request submitted by user.'
        , 'Document with ID {0} is requested for stop processing'
        , 'event.name.210053', 'event.text.210053'
        , 7, 1, 0
        , 'Processing of document is being stopped. Reason: User Requested stopping the processing for document {0}.'
     FROM DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210053)
/
-- End 100193(100616)

Update lg_EventCd
   Set InternalDesc = 'Invalid segment count in transaction set or message trailer,control number = {0}, sending trading partner id/qualfier = {1}/{2}, receiving trading partner id/qualifier = {3}/{4}.  Value from trailer = {5}, actual number received = {6}.'
     , EventName    = 'Transaction Set or Message Trailer Count Invalid'
     , EventNameKey = 'ediEvent.name.VA0103'
     , EventCd      = 103
     , TextKey      = 'ediEvent.text.VA0103'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'Invalid segment count in transaction set or message trailer,control number = {0}, sending trading partner id/qualfier = {1}/{2}, receiving trading partner id/qualifier = {3}/{4}.  Value from trailer = {5}, actual number received = {6}.'
 Where EventCd = 990150
/

Update lg_EventCd
   Set InternalDesc = 'Translation table lookup entry {0} not found in translation table {1} (or translation table is missing).  The default value returned was {2}.'
     , EventName    = 'Translation Table Lookup Failed'
     , EventNameKey = 'ediEvent.name.UT0041'
     , TextKey      = 'ediEvent.text.UT0041'
     , Visibility   = 7
     , Severity     = 6
     , ExternalDesc = 'Translation table lookup entry {0} not found in translation table {1} (or translation table is missing).  The default value returned was {2}.'
 Where EventCd = 41
/

--55870
UPDATE lg_eventcd
SET EVENTNAMEKEY = 'ediEvent.name.VA0022'
WHERE eventcode = 'BCGEDIVA0022'
/

--49516
UPDATE lg_eventcd
SET Severity = 6
WHERE eventcode = 'BCG310003'
/
--101825
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '410025', 'BCG410025',  'Collaboration Partner Agreement Created'
        , 'Collaboration Partner Agreement {0} has been created'
        , 'event.name.410025', 'event.text.410025'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 410025)
/
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '410026', 'BCG410026',  'Collaboration Partner Agreement Created'
        , 'Collaboration Partner Agreement {0} has been created'
        , 'event.name.410026', 'event.text.410026'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 410026)
/
--100855
UPDATE lg_eventcd
SET severity = 6,
   internaldesc = 'View the event and server logs for more information.',
   externaldesc = 'View the event and server logs for more information.'
WHERE eventcd = 210004
/
-- CR60181 start
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410000'
    , 'BCG410000'
    , 'Signature Verification Failed'
    , 'Signature Verification Failed{0}'
    , 'event.name.410000'
    , 'event.text.410000'
    , 1, 9, 1
    , 'If Signature verification failed'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410000)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410001'
    , 'BCG410001'
    , 'Signature Verification Successful'
    , 'Signature Verification Successful{0}'
    , 'event.name.410001'
    , 'event.text.410001'
    , 1, 1, 1
    , 'If Signature verification is successful'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410001)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410002'
    , 'BCG410002'
    , 'Message De-Compression Failed'
    , 'Message De-Compression Failed{0}'
    , 'event.name.410002'
    , 'event.text.410002'
    , 1, 9, 1
    , 'If decompression of attachments failed'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410002)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410003'
    , 'BCG410003'
    , 'Message Compression Failed'
    , 'Message Compression Failed{0}'
    , 'event.name.410003'
    , 'event.text.410003'
    , 1, 9, 1
    , 'If compression of message failed'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410003)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410004'
    , 'BCG410004'
    , 'Ack Not Received'
    , 'Unable to obtain the Ack{0} for {1} Retry'
    , 'event.name.410004'
    , 'event.text.410004'
    , 1, 1, 1
    , 'If acknowledgement is not obtained and message is reintroduced in the flow'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410004)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410005'
    , 'BCG410005'
    , 'Unable To Receive the Ack'
    , 'Unable to obtain the Ack and all the retries are exhausted{0}'
    , 'event.name.410005'
    , 'event.text.410005'
    , 1, 6, 1
    , 'If acknowledgement is not obtained and all retries by reliable messaging engine is exhausted'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410005)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410006'
    , 'BCG410006'
    , 'Ack Already Received'
    , 'The Ack for the MessageID{0} is already received'
    , 'event.name.410006'
    , 'event.text.410006'
    , 1, 6, 1
    , 'If duplicate ack is received for the same message'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410006)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410007'
    , 'BCG410007'
    , 'Ack Not Requested'
    , 'The Ack for the message {0} is not requested'
    , 'event.name.410007'
    , 'event.text.410007'
    , 1, 6, 1
    , 'If ack is not requested from partner but ack is obtained'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410007)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410008'
    , 'BCG410008'
    , 'Ack not located'
    , 'Unable to locate the ack in persistent storage {0}'
    , 'event.name.410008'
    , 'event.text.410008'
    , 1, 6, 1
    , 'If ack is not located for resending to the partner and duplicate message is obtained'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410008)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410009'
    , 'BCG410009'
    , 'Message Out Of Order'
    , 'The Message {0} is out of order.'
    , 'event.name.410009'
    , 'event.text.410009'
    , 7, 3, 1
    , 'If a message is received with a higher sequence number and messages having lower sequence number is not received'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410009)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410010'
    , 'BCG410010'
    , 'CPA Invocation Limit Over'
    , 'The CPA Invocation limit is over{0}'
    , 'event.name.410010'
    , 'event.text.410010'
    , 1, 6, 1
    , 'If engine tries to invoke CPA after its maximum invocation limit is reached'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410010)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410011'
    , 'BCG410011'
    , 'CPA Expired'
    , 'The CPA is expired. {0}'
    , 'event.name.410011'
    , 'event.text.410011'
    , 1, 6, 1
    , 'If CPA is invoked before the its start date or after its end date'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410011)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410012'
    , 'BCG410012'
    , 'CPA Conncurrent Connection Limit Reached'
    , 'No Conncurrent Connection Possible for CPA. {0}'
    , 'event.name.410012'
    , 'event.text.410012'
    , 1, 6, 1
    , 'If maximum number of concurrent invocation of CPA is reached'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410012)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410013'
    , 'BCG410013'
    , 'EBMSVersion not supported'
    , 'The EBMS Version {0} is not supported {1}'
    , 'event.name.410013'
    , 'event.text.410013'
    , 1, 6, 1
    , 'If ebXML message with version other then 2.0 is obtained'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410013)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410014'
    , 'BCG410014'
    , 'Inconsistent Message'
    , 'The EBXML Message is Inconsistent {0}'
    , 'event.name.410014'
    , 'event.text.410014'
    , 1, 6, 1
    , 'If the received ebXML is inconsistent with CPA configuration'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410014)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410017'
    , 'BCG410017'
    , 'Signing Failed'
    , 'Unable to sign the message. {0}'
    , 'event.name.410017'
    , 'event.text.410017'
    , 1, 9, 1
    , 'If engine is unable to sign the message using the required signature algorithm'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410017)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410018'
    , 'BCG410018'
    , 'SaopValidationMap not found'
    , 'Unable to locate the validation map{0}'
    , 'event.name.410018'
    , 'event.text.410018'
    , 1, 6, 1
    , 'If validation map is not found in the database'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410018)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410019'
    , 'BCG410019'
    , 'Extra attachments found (Warning)'
    , '{0}'
    , 'event.name.410019'
    , 'event.text.410019'
    , 1, 6, 1
    , 'If a attachment is found in the ebXML message for which no reference is present in Soap Headers'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410019)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410020'
    , 'BCG410020'
    , 'Not Enough Information to generate error'
    , 'Missing {0}'
    , 'event.name.410020'
    , 'event.text.410020'
    , 1, 6, 1
    , 'If enough information is not obtained for generating a ebXML error'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410020)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410021'
    , 'BCG410021'
    , 'Ack Introduced successfully in the flow'
    , '{0}'
    , 'event.name.410021'
    , 'event.text.410021'
    , 1, 1, 1
    , 'If acknowledgement is successfully generated and introduced to flow.'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410021)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410027'
    , 'BCG410027'
    , 'Message Expired'
    , 'Message has expired {1}'
    , 'event.name.410027'
    , 'event.text.410027'
    , 7, 6, 1
    , 'Message has expired'
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410027)
/
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , INTERNALDESC
    , EVENTNAMEKEY
    , TEXTKEY
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EVENTNAMERESOURCEBUNDLEID 
    , TEXTKEYRESOURCEBUNDLEID 
    , EXTERNALDESCRESOURCEBUNDLEID )
Select '410028'
    , 'BCG410028'
    , 'Unable to generate Ack Message'
    , 'Unable to generate Ack Message {0}'
    , 'event.name.410028'
    , 'event.text.410028'
    , 7, 6, 1
    , 'Unable to generate Ack Message '
    , 1, 2, 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 410028)
/

--JR24914
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC,
	eventnameresourcebundleid ,textkeyresourcebundleid, externaldescresourcebundleid )
   Select 240509, 'BCG240509', 'RN State Engine 0A1 generation is disabled.'
        , '0A1 could not be generated because 0A1 generation is disabled for business ID {0}'
        , 'event.name.240509', 'event.text.240509'
        , 7, 3, 1
        , 'This event is generated when the RosettaNet State Engine initiated a 0A1, but generation of 0A1 id disabled for the partner.'
	,1,2,3
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240509)
/

-- CR60181 end
--101514 using update statements to make migration neater
UPDATE lg_eventcd
SET eventname = 'Partner Name Already Exists'
WHERE eventcd = 101002
/

UPDATE lg_eventcd
SET eventname = 'Hub Administrator Already Exists'
WHERE eventcd = 101004
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Already Exists'
WHERE eventcd = 101005
/

UPDATE lg_eventcd
SET eventname = 'XML Family or Format Updated'
WHERE eventcd = 103004
/

UPDATE lg_eventcd
SET eventname = 'Receiver Configuration Published'
WHERE eventcd = 103009
/

UPDATE lg_eventcd
SET eventname = 'Document Definition Updated'
WHERE eventcd = 103012
/

UPDATE lg_eventcd
SET eventname = 'New Operation Mode Added'
WHERE eventcd = 103025
/

UPDATE lg_eventcd
SET eventname = 'Receiver Processing Error'
WHERE eventcd = 103203
/

UPDATE lg_eventcd
SET eventname = 'Receiver Processing Warning'
WHERE eventcd = 103204
/

UPDATE lg_eventcd
SET eventname = 'Receiver Error'
WHERE eventcd = 103205
/

UPDATE lg_eventcd
SET eventname = 'Receiver with supplied name already exists'
WHERE eventcd = 103206
/

UPDATE lg_eventcd
SET eventname = 'Partner Created'
WHERE eventcd = 104001
/

UPDATE lg_eventcd
SET eventname = 'Partner Updated'
WHERE eventcd = 104002
/

UPDATE lg_eventcd
SET eventname = 'Partner Disabled'
WHERE eventcd = 104006
/

UPDATE lg_eventcd
SET eventname = 'Partner Deleted'
WHERE eventcd = 104009
/

UPDATE lg_eventcd
SET eventname = 'Destination handler metadata modified'
WHERE eventcd = 105001
/

UPDATE lg_eventcd
SET eventname = 'Destination transport metadata modified'
WHERE eventcd = 105002
/

UPDATE lg_eventcd
SET eventname = 'Destination information upated'
WHERE eventcd = 105003
/

UPDATE lg_eventcd
SET eventname = 'No Default Destination Pair'
WHERE eventcd = 106004
/

UPDATE lg_eventcd
SET eventname = 'Destination Created'
WHERE eventcd = 106500
/

UPDATE lg_eventcd
SET eventname = 'Destination has been disabled'
WHERE eventcd = 106501
/

UPDATE lg_eventcd
SET eventname = 'Destination has been enabled'
WHERE eventcd = 106502
/

UPDATE lg_eventcd
SET eventname = 'Destination has been made Offline'
WHERE eventcd = 106503
/

UPDATE lg_eventcd
SET eventname = 'Destination has been made Online'
WHERE eventcd = 106504
/

UPDATE lg_eventcd
SET eventname = 'Destination Updated'
WHERE eventcd = 106505
/

UPDATE lg_eventcd
SET eventname = 'Destination Disable Failed'
WHERE eventcd = 106507
/

UPDATE lg_eventcd
SET eventname = 'Document Definition Create Error'
WHERE eventcd = 106600
/

UPDATE lg_eventcd
SET eventname = 'Receiver Directory Structure Created'
WHERE eventcd = 112006
/

UPDATE lg_eventcd
SET eventname = 'Parnter Name Already Exists'
WHERE eventcd = 123456
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Provided RN Process-Instance-ID Error'
WHERE eventcd = 200013
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Provided RN Process-Instance-ID Warning'
WHERE eventcd = 200014
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Provided RosettaNet GlobalUsageCode Error'
WHERE eventcd = 200015
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Provied RosettaNet GlobalUsageCode Warning'
WHERE eventcd = 200016
/

UPDATE lg_eventcd
SET eventname = 'Inertnal Partner Provied RN Process-Instance-ID Error'
WHERE eventcd = 200017
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Transport Unpackage Failure'
WHERE eventcd = 210011
/

UPDATE lg_eventcd
SET eventname = 'Partner Certificate Did Not Match Signer'
WHERE eventcd = 240013
/

UPDATE lg_eventcd
SET eventname = 'Digital Signature Key Not Loaded for Hub Administrator'
WHERE eventcd = 240018
/

UPDATE lg_eventcd
SET eventname = 'Encryption Key Not Loaded for Hub Administrator'
WHERE eventcd = 240019
/

UPDATE lg_eventcd
SET eventname = 'Auto Destination Set Offline'
WHERE eventcd = 250010
/

UPDATE lg_eventcd
SET eventname = 'Adapter Backend Integration Failed due to Partner Data'
WHERE eventcd = 300005
/

UPDATE lg_eventcd
SET eventname = 'Enveloper schedule changed'
WHERE eventcd = 310005
/

UPDATE lg_eventcd
SET eventname = 'Configuration Data Updated'
WHERE eventcd = 310021
/

UPDATE lg_eventcd
SET eventname = 'Get Internal Partner Business Process Failed'
WHERE eventcd = 800000
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Business Process Entrance'
WHERE eventcd = 800001
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Business Process Exit'
WHERE eventcd = 800002
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Business Process Successful'
WHERE eventcd = 800003
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Business Process Encounters Database Error'
WHERE eventcd = 800004
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Process Encounters Internal Error'
WHERE eventcd = 800005
/

UPDATE lg_eventcd
SET eventname = 'Internal Partner Business Process Warning'
WHERE eventcd = 800006
/

UPDATE lg_eventcd
SET eventname = 'Receiver Exit Event Published'
WHERE eventcd = 104011
/
--101514 end
UPDATE lg_eventcd
SET SEVERITY = 1
WHERE eventcd = 281001
/
-- Start 102261
-- Start 104352f_1
UPDATE lg_eventcd a SET ALERTABLEFLG = 0 where not exists (select 'x' from pr_alert b where
a.eventcd = b.eventcd)
/
-- End   104352f_1
-- End   102261
-- 102376
UPDATE lg_eventcd
SET SEVERITY = 6
WHERE eventcd = 240611
/

--update for EDI event codes
UPDATE lg_eventcd 
SET eventnameresourcebundleid = 4, textkeyresourcebundleid = 5, externaldescresourcebundleid = 3
WHERE eventcd BETWEEN 990000 AND 990165
   OR eventcode = 'BCGEDIFA0001'
   OR eventcode = 'BCGEDIFA0002'
/

--JR24939
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC )
   Select '105004', 'BCG105004', 'Global Transport Attributes updated'
        , 'Global Transport Attributes updated'
        , 'event.name.105003', 'event.text.105003'
        , 7, 1, 1
        , '{0}'
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 105004)
/

--103097
--101825
-- Note that the following 2 updates that set resource bundle ids MUST always be done last so that the
-- resource bundle columns are filled in properly for all rows added by this file.
--added update to lg_eventcd since the values are dependant on the pr_resourcebundles inserts
UPDATE lg_eventcd 
SET eventnameresourcebundleid = 1, textkeyresourcebundleid = 2, externaldescresourcebundleid = 3
WHERE eventnameresourcebundleid IS NULL 
   AND textkeyresourcebundleid IS NULL 
   AND externaldescresourcebundleid IS NULL
/

-- Start 104448
UPDATE lg_eventcd SET SEVERITY=6 where EVENTCD = 280005
/
-- END 104448

-- Addded for BASIC AUTHENTICATION FEATURE

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY,
 VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC,eventnameresourcebundleid , textkeyresourcebundleid , externaldescresourcebundleid  )
   Select '210210', 'BCG210210',  'Basic Authentication Failed'
        , 'Basic Authentication Failed, {0}'
        , 'event.name.210210', 'event.text.210210'
        , 7, 6, 0
        , 'This event is generated when Basic Authentication Failed',1,2,3
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210210)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY,
 VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC,eventnameresourcebundleid , textkeyresourcebundleid , externaldescresourcebundleid )
   Select '210211', 'BCG210211',  'Authorization Failed'
        , 'Authorization Failed, {0}'
        , 'event.name.210211', 'event.text.210211'
        , 7, 6, 0
        , ' This event is generated when Authorization Failed',1,2,3
     From dual
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 210211)
/

-- Added for Delete Artifacts Feature

INSERT INTO LG_EVENTCD ( EVENTCD, EVENTCODE, EVENTNAME,INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, EVENTNAMERESOURCEBUNDLEID, ALERTABLEFLG, TEXTKEYRESOURCEBUNDLEID,
			EXTERNALDESC,EXTERNALDESCRESOURCEBUNDLEID ) 
	SELECT 	'106508','BCG106508','Destination deleted'
		,'Destination :"{0}" has been deleted for partner {1}'
		,'event.name.106508','event.text.106508', 7,1,1,0,2
		,'This event is generated when destination is deleted.',3
	From DUAL
        Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 106508)
/

INSERT INTO LG_EVENTCD ( EVENTCD, EVENTCODE, EVENTNAME,INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, EVENTNAMERESOURCEBUNDLEID, ALERTABLEFLG, TEXTKEYRESOURCEBUNDLEID,
			EXTERNALDESC,EXTERNALDESCRESOURCEBUNDLEID ) 
     	SELECT '104014','BCG104014', 'User Deleted','User deleted {0}'
		,'event.name.104014', 'event.text.104014',7,1,1,0,2  
		,'This event is generated when user is deleted.',3 
	From DUAL
        Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 104014)
/        

 
INSERT INTO LG_EVENTCD ( EVENTCD, EVENTCODE, EVENTNAME,INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, EVENTNAMERESOURCEBUNDLEID, ALERTABLEFLG, TEXTKEYRESOURCEBUNDLEID,
			EXTERNALDESC,EXTERNALDESCRESOURCEBUNDLEID )
	SELECT  '103002', 'BCG103002', 'Validation Map Deleted'
		,'Validation Map :"{0}" has been deleted'
		,'event.name.103002','event.text.103002',1,1,1,0,2 
		,'This event is generated when Validation Map is deleted.',3
	From DUAL
        Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103002)
/        

 
 
INSERT INTO LG_EVENTCD ( EVENTCD, EVENTCODE, EVENTNAME,INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, EVENTNAMERESOURCEBUNDLEID, ALERTABLEFLG, TEXTKEYRESOURCEBUNDLEID,
			EXTERNALDESC,EXTERNALDESCRESOURCEBUNDLEID ) 
	SELECT 	'103301', 'BCG103301', 'Routing object deleted',
		'Routing Object :"{0}" of type "{1}" and Document level "{2}" has been deleted'
		,'event.name.103301', 'event.text.103301', 1,1,1,0,2
		, 'This event is generated when Routing object deleted is deleted.',3
	From DUAL
        Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103301)
/        

 
INSERT INTO LG_EVENTCD ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, EVENTNAMERESOURCEBUNDLEID, ALERTABLEFLG, TEXTKEYRESOURCEBUNDLEID,
			 EXTERNALDESC,EXTERNALDESCRESOURCEBUNDLEID ) 
	SELECT '103302', 'BCG103302', 'Interaction deleted','Interaction :"{0}"  to "{1}" has been deleted'
		,'event.name.103302', 'event.text.103302',1,1,1,0,2 
		,'This event is generated when Interaction is deleted.',3
	From DUAL
        Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103302)
/        

 -- F104485
 
 
 Insert Into lg_EventCd ( EVENTCD
 	, EVENTCODE
 	, EVENTNAME
 	, PROVIDERFLAG
 	, INTERNALDESC
 	, EXTERNALDESC
 	, EVENTNAMEKEY
 	, TEXTKEY
 	, VISIBILITY
 	, SEVERITY
 	, ALERTABLEFLG
 	, EVENTNAMERESOURCEBUNDLEID
 	, TEXTKEYRESOURCEBUNDLEID
 	, EXTERNALDESCRESOURCEBUNDLEID
 	)
 Select 	610001
 	, 'BCG610001'
 	, 'Connection to FTP Server successful'
    	, 0
    	, 'FTP connection is successful {0}. FTP Server response status is:  {1}'
    	, 'This event is generated when connection is established with the FTP Server'
    	, 'event.name.610001'
    	, 'event.text.610001'
    	, 7
    	, 1
    	, 0
    	, 1
    	, 2
    	, 3
      From DUAL
     Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 610001)
/
 
 
 Insert Into lg_EventCd ( EVENTCD
 	, EVENTCODE
 	, EVENTNAME
 	, PROVIDERFLAG
 	, INTERNALDESC
 	, EXTERNALDESC
 	, EVENTNAMEKEY
 	, TEXTKEY
 	, VISIBILITY
 	, SEVERITY
 	, ALERTABLEFLG
 	, EVENTNAMERESOURCEBUNDLEID
 	, TEXTKEYRESOURCEBUNDLEID
 	, EXTERNALDESCRESOURCEBUNDLEID
 	)
    Select 610002,
    	'BCG610002'
    	, 'Failed to connect to FTP Server'
    	, 0
    	, 'FTP connection {0} is not obtained. FTP Server response status is:  {1}'
    	, 'This event is generated when connection to FTP server cannot be established due to either of IP restriction, number of connections exceeded'
    	, 'event.name.610002'
    	, 'event.text.610002'
    	, 7
    	, 9
    	, 0
    	, 1
    	, 2
    	, 3
      From DUAL
     Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 610002)
/
 
 
 
 Insert Into lg_EventCd ( EVENTCD
 	, EVENTCODE
 	, EVENTNAME
 	, PROVIDERFLAG
 	, INTERNALDESC
 	, EXTERNALDESC
 	, EVENTNAMEKEY
 	, TEXTKEY
 	, VISIBILITY
 	, SEVERITY
 	, ALERTABLEFLG
 	, EVENTNAMERESOURCEBUNDLEID
 	, TEXTKEYRESOURCEBUNDLEID
 	, EXTERNALDESCRESOURCEBUNDLEID
 	)
    Select 610003,
    	'BCG610003'
    	, 'User logged in to FTP Server'
    	, 0
    	, 'User {0} successfully logged in to FTP server from host {1}. FTP Server response status is: {2}'
    	, 'This event is generated for every successful FTP Server login attempt'
    	, 'event.name.610003'
    	, 'event.text.610003'
    	, 7
    	, 1
    	, 0
    	, 1
    	, 2
    	, 3
      From DUAL
     Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 610003)
/
 
 
 
 Insert Into lg_EventCd ( EVENTCD
 	, EVENTCODE
 	, EVENTNAME
 	, PROVIDERFLAG
 	, INTERNALDESC
 	, EXTERNALDESC
 	, EVENTNAMEKEY
 	, TEXTKEY
 	, VISIBILITY
 	, SEVERITY
 	, ALERTABLEFLG
 	, EVENTNAMERESOURCEBUNDLEID
 	, TEXTKEYRESOURCEBUNDLEID
 	, EXTERNALDESCRESOURCEBUNDLEID
 	)
    Select 610004,
    	'BCG610004'
    	, 'Unable to login on FTP Server'
    	, 0
    	, 'Cannot login to FTP server {0}. FTP Server response status is: {1}'
    	, 'This event is generated for every unsuccessful login attempt'
    	, 'event.name.610004'
    	, 'event.text.610004'
    	, 7
    	, 1
    	, 0
    	, 1
    	, 2
    	, 3
      From DUAL
     Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 610004)
/
 
 
 
 Insert Into lg_EventCd ( EVENTCD
 	, EVENTCODE
 	, EVENTNAME
 	, PROVIDERFLAG
 	, INTERNALDESC
 	, EXTERNALDESC
 	, EVENTNAMEKEY
 	, TEXTKEY
 	, VISIBILITY
 	, SEVERITY
 	, ALERTABLEFLG
 	, EVENTNAMERESOURCEBUNDLEID
 	, TEXTKEYRESOURCEBUNDLEID
 	, EXTERNALDESCRESOURCEBUNDLEID
 	)
    Select 610005,
    	'BCG610005'
    	, 'Document successfully uploaded'
    	, 0
    	, 'File {0} is successfully uploaded {1}. FTP Server response status is {2}'
    	, 'This event is generated for every successful upload of document to the FTP Server'
    	, 'event.name.610005'
    	, 'event.text.610005'
    	, 7
    	, 1
    	, 0
    	, 1
    	, 2
    	, 3
      From DUAL
     Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 610005)
/
 
 
 
 Insert Into lg_EventCd ( EVENTCD
 	, EVENTCODE
 	, EVENTNAME
 	, PROVIDERFLAG
 	, INTERNALDESC
 	, EXTERNALDESC
 	, EVENTNAMEKEY
 	, TEXTKEY
 	, VISIBILITY
 	, SEVERITY
 	, ALERTABLEFLG
 	, EVENTNAMERESOURCEBUNDLEID
 	, TEXTKEYRESOURCEBUNDLEID
 	, EXTERNALDESCRESOURCEBUNDLEID
 	)
    Select 610006,
    	'BCG610006'
    	, 'Document successfully downloaded'
    	, 0
    	, 'File {0} is successfully downloaded. FTP Server response status is {1}'
    	, 'This event is generated for every successful download of document to the FTP Server'
    	, 'event.name.610006'
    	, 'event.text.610006'
    	, 7
    	, 1
    	, 0
    	, 1
    	, 2
    	, 3
      From DUAL
     Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 610006)
/
 
 
 
 Insert Into lg_EventCd ( EVENTCD
 	, EVENTCODE
 	, EVENTNAME
 	, PROVIDERFLAG
 	, INTERNALDESC
 	, EXTERNALDESC
 	, EVENTNAMEKEY
 	, TEXTKEY
 	, VISIBILITY
 	, SEVERITY
 	, ALERTABLEFLG
 	, EVENTNAMERESOURCEBUNDLEID
 	, TEXTKEYRESOURCEBUNDLEID
 	, EXTERNALDESCRESOURCEBUNDLEID
 	)
    Select 610007,
    	'BCG610007'
    	, 'Successfully disconnected from FTP Server'
    	, 0
    	, '{0} is successfully disconnected from FTP Server'
    	, 'This event is generated for every successful disconnection'
    	, 'event.name.610007'
    	, 'event.text.610007'
    	, 7
    	, 1
    	, 0
    	, 1
    	, 2
    	, 3
      From DUAL
     Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 610007)
/

--END F104485


Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC,
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
   			Select 108012, 'BCG108012', 'Certificate Deleted'
        		, 'Certificate {1} is Deleted for partner {0}'
        		, 'event.name.108012', 'event.text.108012'
        		, 7, 1, 0
        		, 'This event is generated when the Certificate is deleted',
       			 1,2,3
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108012)
/

 

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC,
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
   Select 108013, 'BCG108013', 'Set Created'
        , 'Set {0} was successfully created for partner {1} with Primary certificate {2} and secondry certificate {3}'
        , 'event.name.108013', 'event.text.108013'
        , 7, 1, 0
        , 'This event is generated when the Set is Created'
        ,1,2,3
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108013)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
   Select 108014, 'BCG108014', 'Set Updated'
        , 'Set {0} was successfully updated for partner {1} with Primary certificate {2} and secondry certificate {3}'
        , 'event.name.108014', 'event.text.108014'
        , 7, 1, 0
        , 'This event is generated when the Set is Updated'
        ,1,2,3
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108014)
/

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
   Select 108015, 'BCG108015', 'Set Deleted'
        , 'Set {0} having  Primary certificate {1} and secondry certificate {2} is Deleted for partner {0}'
        , 'event.name.108015', 'event.text.108015'
        , 7, 1, 0
        , 'This event is generated when the Set is deleted'
        ,1,2,3
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 108015)
/

 

Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
   Select 240082, 'BCG240082', 'Error Flow Document Creation failed for the event '
        , 'Error Flow Document Creation failed for the event : {0} for the failed document sending partner: {1}, document reference id  : {2}. Failure details : {3}'
        , 'event.name.240082', 'event.text.240082'
        , 7, 6, 0
        , 'This event is generated when the Error Flow Document Creation failed for the event'
        ,1,2,3
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240082)
/

-- Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
--			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
--			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
--   Select 240038, 'BCG240038', 'FIPS mode status '
--        , 'The product is running in {0} mode.'
--        , 'event.name.240038', 'event.text.240038'
--        , 1, 1, 0
--        , 'This event indicates whether the product is running in FIPS mode or not.'
--        ,1,2,3
--     From DUAL
--    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240038)
-- /


Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
   Select 240035, 'BCG240035', ' FIPS error '
        , '{0} is not FIPS approved.'
        , 'event.name.240035', 'event.text.240035'
        , 1, 6, 0
        , 'This event is generated when a non-FIPS approved algorithm is used in FIPS mode.'
        ,1,2,3
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 240035)
/    

--Start JR26947 added by Amit Kumar Vijay
INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , PROVIDERFLAG
    , INTERNALDESC
    , EVENTNAMEKEY
    , EVENTNAMERESOURCEBUNDLEID
    , TEXTKEY
    , TEXTKEYRESOURCEBUNDLEID
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EXTERNALDESCRESOURCEBUNDLEID 
     )
Select 110173
    , 'BCG110173'
    , 'Document ID is used as the correlation attribute'
    , 0
    , 'Document ID is used as the correlation attribute'
    , 'event.name.110173'
    , 1
    , 'event.text.110173'
    , 2
    , 7
    , 1
    , 0
    , 'Document ID used as the correlation attribute'
    , 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 110173)
/

-- END JR26947

INSERT INTO lg_EventCd ( EVENTCD
    , EVENTCODE
    , EVENTNAME
    , PROVIDERFLAG
    , INTERNALDESC
    , EVENTNAMEKEY
    , EVENTNAMERESOURCEBUNDLEID
    , TEXTKEY
    , TEXTKEYRESOURCEBUNDLEID
    , VISIBILITY
    , SEVERITY
    , ALERTABLEFLG
    , EXTERNALDESC
    , EXTERNALDESCRESOURCEBUNDLEID 
     )
Select 240079
    , 'BCG240079'
    , 'SOAP De-Envelope failed.'
    , 0
    , 'SOAP De-Envelope failed. [Reason : {0}]'
    , 'event.name.240079'
    , 1
    , 'event.text.240079'
    , 2
    , 7
    , 6
    , 0
    , 'Document ID is used as the correlation attribute'
    , 3
    FROM DUAL
    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 240079)
/

-- 104775f_1

-- INSERT INTO lg_EventCd ( EVENTCD
--    , EVENTCODE
--    , EVENTNAME
--    , PROVIDERFLAG
--    , INTERNALDESC
--    , EVENTNAMEKEY
--    , EVENTNAMERESOURCEBUNDLEID
--    , TEXTKEY
--    , TEXTKEYRESOURCEBUNDLEID
--    , VISIBILITY
--    , SEVERITY
--    , ALERTABLEFLG
--    , EXTERNALDESC
--    , EXTERNALDESCRESOURCEBUNDLEID 
--     )
-- Select 240426
--    , 'BCG240426'
--    , 'Temp file could not be deleted'
--    , 0
--    , 'Temp file could not be deleted'
--    , 'event.name.240426'
--    , 1
--    , 'event.text.240426'
--    , 2
--    , 7
--    , 3
--    , 0
--    , 'Temp file could not be deleted'
--    , 3
--    FROM DUAL
--    WHERE NOT Exists (SELECT 'x' FROM lg_EventCd WHERE eventcd = 240426)
--/

-- 104775f_1
INSERT INTO lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
	Select 700001, 'BCG700001'
		,  'Archiver Successful','Archiver Successful for task {0}. Execution Time {1}'
		, 'event.name.700001', 'event.text.700001'
		, 7, 1, 0
		, 'Event is generated when archiver finishes successfully'
		,1,2,3
	From	DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 700001)
/
INSERT INTO lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
	Select 700002, 'BCG700002'
		, 'Archiver Task Error','Archiver Error for task {0}. Execution Time {1}. Error Reason {2}'
		, 'event.name.700002', 'event.text.700002'
		, 7, 6, 0
		, 'Event is generated when archiver finishes with an error'
		,1,2,3
	From	DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 700002)
/
INSERT INTO lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
	Select 700003, 'BCG700003'
		, 'Archiver Task Stopped By User','Archiver task stopped by user {0}. Stop time {1}'
		, 'event.name.700003', 'event.text.700003'
		, 7, 3, 0
		, 'Event is generated when user stops running archiver task'
		,1,2,3
	From	DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 700003)
/

INSERT INTO lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
	Select 3, 'BCGEDIFA0003'
		, 'Functional Acknowledgement is Overdue','Functional acknowledgement is overdue for document sent from partner {0} to partner{1} with transaction control number {2} group control number {3} interchange control number {4} and reference id as {5}. FA is overdue by {6} minutes.'
		, 'ediEvent.name.FA0003', 'ediEvent.text.FA0003'
		, 7, 3, 1
		, 'Functional acknowledgement is overdue for document sent from partner {0} to partner{1} with transaction control number {2} group control number {3} interchange control number {4} and reference id as {5}. FA is overdue by {6} minutes.'
		,1,2,3
	From	DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 3)
/

INSERT INTO lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
	Select 700004, 'BCG700004'
		, 'Restore Successful','Restore Successful for task {0}. Restore Time {1}'
		, 'event.name.700004', 'event.text.700004'
		, 7, 1, 0
		, 'Event is generated when restore task completes successfully.'
		,1,2,3
	From	DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 700004)
/
INSERT INTO lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
	Select 700005, 'BCG700005'
		, 'Restore Failed','Restore Failed. Error Reason{1}'
		, 'event.name.700005', 'event.text.700005'
		, 7, 6, 0
		, 'Event is generated when restore task completes successfully.'
		,1,2,3
	From	DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 700005)
/
INSERT INTO lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY,
			TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, 
			EVENTNAMERESOURCEBUNDLEID,TEXTKEYRESOURCEBUNDLEID,EXTERNALDESCRESOURCEBUNDLEID)
	Select 700006, 'BCG700006'
		, 'Restore Warning','Restore Warning. Reason {0}'
		, 'event.name.700006', 'event.text.700006'
		, 7, 3, 0
		, 'Event is generated when restore task completes with warnings.'
		,1,2,3
	From	DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 700006)
/

		
INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112031
		,'BCG112031'
		,'SFTP J2C Activation specification creation failed'
		,0
		,'SFTP J2C Activation specification {0} creation failed'
		,'event.name.112031'
		,1
		,'event.text.112031'
		,2
		,7
		,3
		,0
		,'This event is generated during receiver creation, when creation of activation specification falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112031)
/
INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112032
		,'BCG112032'
		,'SFTP J2C Activation specification updation failed'
		,0
		,'SFTP J2C Activation specification {0} updation failed'
		,'event.name.112032'
		,1
		,'event.text.112032'
		,2
		,7
		,3
		,0
		,'This event is generated during receiver updation,when upating of activation specification falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112032)
/
INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112033
		,'BCG112033'
		,'SFTP J2C Activation specification deletion failed'
		,0
		,'SFTP J2C Activation specification {0} deletion failed'
		,'event.name.112033'
		,1
		,'event.text.112033'
		,2
		,7
		,3
		,0
		,'This event is generated during receiver deletion, when deletion of activation specification falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112033)
/
INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112034
	,'BCG112034'
	,'SFTP MDB Module Insallation Failed'
	,0
	,'SFTP MDB Module {0} Insallation Failed'
	,'event.name.112034'
	,1
	,'event.text.112034'
	,2
	,7
	,3
	,0
	,'This event is generated during receiver creation/updation, when Installation of MDB module falied.'
	,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112034)
/
INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112035
		,'BCG112035'
		,'SFTP MDB Module Un-Installation Failed'
		,0
		,'SFTP MDB Module {0} Un-Installation Failed'
		,'event.name.112035'
		,1
		,'event.text.112035'
		,2
		,7
		,3
		,0
		,'This event is generated during receiver creation/updation, when Uninstallation of MDB module falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112035)
/

INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112036
		,'BCG112036'
		,'SFTP Local Event Directory creation failed'
		,0
		,'SFTP Local Event Directory {0} creation failed'
		,'event.name.112036'
		,1
		,'event.text.112036'
		,2
		,7
		,3
		,0
		,'This event is generated during receiver creation/updation, when Local event directory creation falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112036)
/

INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112037
		,'BCG112037'
		,'SFTP Local Event Directory deletion failed'
		,0
		,'SFTP Local Event Directory {0} deletion failed'
		,'event.name.112037'
		,1
		,'event.text.112037'
		,2
		,7
		,3
		,0
		,'This event is generated during receiver creation/updation, when Local event directory deletion falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112037)
/
		

INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112038
		,'BCG112038'
		,'SFTP Local Archive Directory creation failed'
		,0
		,'SFTP Local Archive Directory {0} creation failed'
		,'event.name.112038'
		,1
		,'event.text.112038'
		,2
		,7
		,3
		,0
		,'This event is generated during receiver creation/updation, when Local archive directory creation falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112038)
/

INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112039
		,'BCG112039'
		,'SFTP Local Archive Directory deletion failed'
		,0
		,'SFTP Local Archive Directory {0} deletion failed'
		,'event.name.112039'
		,1
		,'event.text.112039'
		,2
		,7
		,3
		,0
		,'This event is generated during receiver creation/updation, when Local archive directory creation falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112039)
/

INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112040
		,'BCG112040'
		,'SFTP J2C Connection Factory creation failed'
		,0
		,'SFTP J2C Connection Factory {0} creation failed'
		,'event.name.112040'
		,1
		,'event.text.112040'
		,2
		,7
		,3
		,0
		,'This event is generated during destination creation, when creation of connection factory falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112040)
/

INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112041
		,'BCG112041'
		,'SFTP J2C Connection Factory updation failed'
		,0
		,'SFTP J2C Connection Factory {0} updation failed'
		,'event.name.112041'
		,1
		,'event.text.112041'
		,2
		,7
		,3
		,0
		,'This event is generated during destination updation, when updation of connection factory falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112041)
/

INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112042
		,'BCG112042'	
		,'SFTP J2C Connection Factory deletion failed'
		,0
		,'SFTP J2C Connection Factory {0} deletion failed'
		,'event.name.112042'
		,1
		,'event.text.112042'
		,2
		,7
		,3
		,0
		,'This event is generated during destination deletion, when deletion of connection factory falied.'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112042)
/

-- Added for 107904
-- Edi Events should have eventnameresourcebundleid as '4' and textkeyresourcebundleid as '5'

UPDATE lg_eventcd 
SET eventnameresourcebundleid = 4, textkeyresourcebundleid = 5 
WHERE textkey LIKE 'ediEvent%'
 AND eventnameresourcebundleid = 1
 AND textkeyresourcebundleid=2
/ 
-- START 32386
Insert Into lg_EventCd ( EVENTCD, EVENTCODE, EVENTNAME, INTERNALDESC, EVENTNAMEKEY, TEXTKEY, VISIBILITY, SEVERITY, ALERTABLEFLG, EXTERNALDESC, EVENTNAMERESOURCEBUNDLEID, TEXTKEYRESOURCEBUNDLEID, EXTERNALDESCRESOURCEBUNDLEID )
   Select 103013, 'BCG103013', 'SQL injection detected'
        , 'SQL injection detected'
        , 'event.name.103013', 'event.text.103013'
        , 1, 9, 1
        , 'This event is generated when the malicious User tries to inject some sql query, causing the security hazard'
		, 1,2,3
     From DUAL
    Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 103013)
/
-- END 32386
-- Start JR33020
INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112043
		,'BCG112043'
		,'SFTP Local Event Directory could not be deleted.'
		,0
		,'SFTP Local Event Directory {0} could not be deleted as there are files in the directory which may need to be processed manually.'
		,'event.name.112043'
		,1
		,'event.text.112043'
		,2
		,7
		,3
		,0
		,'The event is generated when deleting the Local event directory in the process of deleting the SFTP receiver'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112043)
/

INSERT INTO lg_EventCd (EVENTCD
			,EVENTCODE
			,EVENTNAME
			,PROVIDERFLAG
			,INTERNALDESC
			,EVENTNAMEKEY
			,EVENTNAMERESOURCEBUNDLEID
			,TEXTKEY
			,TEXTKEYRESOURCEBUNDLEID
			,VISIBILITY
			,SEVERITY
			,ALERTABLEFLG
			,EXTERNALDESC		
			,EXTERNALDESCRESOURCEBUNDLEID
			)
	Select 	112044
		,'BCG112044'
		,'SFTP Local Archive Directory could not be deleted.'
		,0
		,'SFTP Local Archive Directory {0} could not be deleted as there are files in the directory which may need to be processed manually.'
		,'event.name.112044'
		,1
		,'event.text.112044'
		,2
		,7
		,3
		,0
		,'The event is generated when deleting the Local archive directory in the process of deleting the SFTP receiver'
		,3
	From DUAL
	Where NOT Exists (Select * From lg_EventCd Where EVENTCD = 112044)
/

-- END JR33020


-- PR_TRANSPORT_PROXY_ATTR.sql

-- proxyHost Http1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyHost'
	,null
	,'proxyHost'
	,'proxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Http1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyHost'
								FROM	pr_transport
								WHERE	transporttype = 'Http1.1'
							)
			)
)
/
-- proxyPort Http1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyPort'
	,null
	,'proxyPort'
	,'proxyPort'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Http1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyPort'
								FROM	pr_transport
								WHERE	transporttype = 'Http1.1'
							)
			)
)
/
-- proxyUserName Http1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyUserName'
	,null
	,'proxyUserName'
	,'proxyUserName'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Http1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyUserName'
								FROM	pr_transport
								WHERE	transporttype = 'Http1.1'
							)
			)
)
/
-- proxyUserPassword Http1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyUserPassword'
	,null
	,'proxyUserPassword'
	,'proxyUserPassword'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Http1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyUserPassword'
								FROM	pr_transport
								WHERE	transporttype = 'Http1.1'
							)
			)
)
/
-- socksProxyHost Http1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'socksProxyHost'
	,null
	,'socksProxyHost'
	,'socksProxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Http1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'socksProxyHost'
								FROM	pr_transport
								WHERE	transporttype = 'Http1.1'
							)
			)
)
/
-- socksProxyPort Http1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'socksProxyPort'
	,null
	,'socksProxyPort'
	,'socksProxyPort'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Http1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'socksProxyPort'
								FROM	pr_transport
								WHERE	transporttype = 'Http1.1'
							)
			)
)
/
-- proxyHost Https1.0
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyHost'
	,null
	,'proxyHost'
	,'proxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.0'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyHost'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.0'
							)
			)
)
/
-- proxyPort Https1.0
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyPort'
	,null
	,'proxyPort'
	,'proxyPort'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.0'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyPort'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.0'
							)
			)
)
/
-- proxyUserName Https1.0
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyUserName'
	,null
	,'proxyUserName'
	,'proxyUserName'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.0'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyUserName'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.0'
							)
			)
)
/
-- proxyUserPassword Https1.0
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyUserPassword'
	,null
	,'proxyUserPassword'
	,'proxyUserPassword'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.0'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyUserPassword'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.0'
							)
			)
)
/
-- socksProxyHost Https1.0
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'socksProxyHost'
	,null
	,'socksProxyHost'
	,'socksProxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.0'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'socksProxyHost'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.0'
							)
			)
)
/
-- socksProxyPort Https1.0
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'socksProxyPort'
	,null
	,'socksProxyPort'
	,'socksProxyPort'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.0'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'socksProxyPort'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.0'
							)
			)
)
/
-- proxyHost Https1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyHost'
	,null
	,'proxyHost'
	,'proxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyHost'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.1'
							)
			)
)
/
-- proxyPort Https1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyPort'
	,null
	,'proxyPort'
	,'proxyPort'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyPort'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.1'
							)
			)
)
/
-- proxyUserName Https1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyUserName'
	,null
	,'proxyUserName'
	,'proxyUserName'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyUserName'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.1'
							)
			)
)
/
-- proxyUserPassword Https1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyUserPassword'
	,null
	,'proxyUserPassword'
	,'proxyUserPassword'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyUserPassword'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.1'
							)
			)
)
/
-- socksProxyHost Https1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'socksProxyHost'
	,null
	,'socksProxyHost'
	,'socksProxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'socksProxyHost'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.1'
							)
			)
)
/
-- socksProxyPort Https1.1
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'socksProxyPort'
	,null
	,'socksProxyPort'
	,'socksProxyPort'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'socksProxyPort'
								FROM	pr_transport
								WHERE	transporttype = 'Https1.1'
							)
			)
)
/
-- proxyHost HttpS
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyHost'
	,null
	,'proxyHost'
	,'proxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'HttpS'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyHost'
								FROM	pr_transport
								WHERE	transporttype = 'HttpS'
							)
			)
)
/
-- proxyPort HttpS
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyPort'
	,null
	,'proxyPort'
	,'proxyPort'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'HttpS'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyPort'
								FROM	pr_transport
								WHERE	transporttype = 'HttpS'
							)
			)
)
/
-- proxyUserName HttpS
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyUserName'
	,null
	,'proxyUserName'
	,'proxyUserName'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'HttpS'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyUserName'
								FROM	pr_transport
								WHERE	transporttype = 'HttpS'
							)
			)
)
/
-- proxyUserPassword HttpS
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'proxyUserPassword'
	,null
	,'proxyUserPassword'
	,'proxyUserPassword'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'HttpS'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'proxyUserPassword'
								FROM	pr_transport
								WHERE	transporttype = 'HttpS'
							)
			)
)
/
-- socksProxyHost HttpS
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'socksProxyHost'
	,null
	,'socksProxyHost'
	,'socksProxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'HttpS'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'socksProxyHost'
								FROM	pr_transport
								WHERE	transporttype = 'HttpS'
							)
			)
)
/
-- socksProxyPort HttpS
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'socksProxyPort'
	,null
	,'socksProxyPort'
	,'socksProxyPort'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'HttpS'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	(transportid, attrname) = (SELECT	transportid, 'socksProxyPort'
								FROM	pr_transport
								WHERE	transporttype = 'HttpS'
							)
			)
)
/
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'nonProxyHost'
	,''
	,'nonProxyHost'
	,'nonProxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Http1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	transportid = (SELECT	transportid
						FROM	pr_transport
						WHERE	transporttype = 'Http1.1'
							)
			AND	attrname = 'nonProxyHost'
			)
)
/
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'nonProxyHost'
	,''
	,'nonProxyHost'
	,'nonProxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.0'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	transportid = (SELECT	transportid
						FROM	pr_transport
						WHERE	transporttype = 'Https1.0'
							)
			AND	attrname = 'nonProxyHost'
			)
)
/
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'nonProxyHost'
	,''
	,'nonProxyHost'
	,'nonProxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'Https1.1'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	transportid = (SELECT	transportid
						FROM	pr_transport
						WHERE	transporttype = 'Https1.1'
							)
			AND	attrname = 'nonProxyHost'
			)
)
/
INSERT INTO PR_TRANSPORT_PROXY_ATTR(
	TRANSPORTPROXYATTRID
	,TRANSPORTID
	,ATTRNAME
	,ATTRVALUE
	,RESOURCEKEY
	,DESCRIPTION
	,ROWTS
	,CREATEDATE
	,UPDATEDATE
) (SELECT
	PR_TRANSPORT_PROXY_ATTR_SEQ.nextval
	,pt.TRANSPORTID
	,'nonProxyHost'
	,''
	,'nonProxyHost'
	,'nonProxyHost'
	,1
	,sysdate
	,sysdate
  FROM PR_TRANSPORT pt
  WHERE TRANSPORTTYPE = 'HttpS'
  AND	NOT EXISTS (SELECT 'x'
			FROM	pr_transport_proxy_attr
			WHERE	transportid = (SELECT	transportid
						FROM	pr_transport
						WHERE	transporttype = 'HttpS'
							)
			AND	attrname = 'nonProxyHost'
			)
)
/

-- CF_PROFILE.sql

INSERT into cf_profile (profileid, profiletypeid, profilename, description)
Select CF_PROFILE_SEQ.NEXTVAL,2, 'WPG-Default-X12', 'Predefined Envelope Profile for use by the Administrator. 
For those attributes that have default values, the default values are filled in.' from dual
where not exists (select 'x' from cf_profile where profilename IN ('WPG-Default-X12','WPG-Defined-X12'))
/
INSERT into cf_profile (profileid, profiletypeid, profilename, description)
Select CF_PROFILE_SEQ.NEXTVAL,3, 'WPG-Default-UCS' , 'Predefined Envelope Profile for use by the Administrator. 
For those attributes that have default values, the default values are filled in.' from dual
where not exists (select 'x' from cf_profile where profilename  IN ('WPG-Default-UCS','WPG-Defined-UCS'))
/
INSERT into cf_profile (profileid, profiletypeid, profilename, description)
Select CF_PROFILE_SEQ.NEXTVAL,4, 'WPG-Default-EDIFACT' , 'Predefined Envelope Profile for use by the Administrator. 
For those attributes that have default values, the default values are filled in.' from dual
where not exists (select 'x' from cf_profile where profilename IN ('WPG-Default-EDIFACT','WPG-Defined-EDIFACT'))
/
INSERT into cf_profile (profileid, profiletypeid, profilename, description, candelete)
	SELECT CF_PROFILE_SEQ.NEXTVAL
		,pt.PROFILETYPEID
		, 'WPG-ebMS-Synchronous' 
		, 'Predefined profile for ebXML synchronous response'
		, 1
	FROM	cf_profiletype pt
	WHERE	pt.profiletypename = 'ebMS'
	AND NOT EXISTS (SELECT 'x' FROM cf_profile WHERE profilename = 'WPG-ebMS-Synchronous' )
/
INSERT into cf_profile (profileid, profiletypeid, profilename, description, candelete)
	SELECT CF_PROFILE_SEQ.NEXTVAL
		,pt.PROFILETYPEID
		, 'WPG-ebMS-ASynchronous' 
		, 'Predefined profile for ebXML asynchronous response'
		, 1
	FROM	cf_profiletype pt
	WHERE	pt.profiletypename = 'ebMS'
	AND NOT EXISTS (SELECT 'x' FROM cf_profile WHERE profilename = 'WPG-ebMS-ASynchronous' )
/
--102086
update cf_profile
set profilename = 'WPG-Defined-X12'
where profilename = 'WPG-Default-X12'
/
update cf_profile
set profilename = 'WPG-Defined-UCS'
where profilename = 'WPG-Default-UCS'
/
update cf_profile
set profilename = 'WPG-Defined-EDIFACT'
where profilename = 'WPG-Default-EDIFACT'
/


-- edi_standard_package_lnk.sql

--edi_standard_package_lnk
   

INSERT INTO edi_standard_package_lnk
            (profiletypeid)
   SELECT profiletypeid
     FROM cf_profiletype
    WHERE profiletypename = 'X12'
      AND NOT EXISTS (SELECT 'x'
                        FROM cf_profiletype a INNER JOIN edi_standard_package_lnk b
                             ON a.profiletypeid = b.profiletypeid
                       WHERE a.profiletypename = 'X12')
/

INSERT INTO edi_standard_package_lnk
            (profiletypeid)
   SELECT profiletypeid
     FROM cf_profiletype
    WHERE profiletypename = 'UCS'
      AND NOT EXISTS (SELECT 'x'
                        FROM cf_profiletype a INNER JOIN edi_standard_package_lnk b
                             ON a.profiletypeid = b.profiletypeid
                       WHERE a.profiletypename = 'UCS')
 /                      
                       
INSERT INTO edi_standard_package_lnk
            (profiletypeid)
   SELECT profiletypeid
     FROM cf_profiletype
    WHERE profiletypename = 'EDIFACT'
      AND NOT EXISTS (SELECT 'x'
                        FROM cf_profiletype a INNER JOIN edi_standard_package_lnk b
                             ON a.profiletypeid = b.profiletypeid
                       WHERE a.profiletypename = 'EDIFACT')
 /
 
UPDATE edi_standard_package_lnk
SET protocolid = (SELECT routingobjid 
                  FROM cf_routingobject
                  WHERE routingobjcd = 'EDI-X12')
WHERE profiletypeid = (SELECT profiletypeid
                       FROM cf_profiletype
                       WHERE profiletypename = 'X12')
/

UPDATE edi_standard_package_lnk
SET protocolid = (SELECT routingobjid 
                  FROM cf_routingobject
                  WHERE routingobjcd = 'EDI-Consent')
WHERE profiletypeid = (SELECT profiletypeid
                       FROM cf_profiletype
                       WHERE profiletypename = 'UCS')
/

UPDATE edi_standard_package_lnk
SET protocolid = (SELECT routingobjid 
                  FROM cf_routingobject
                  WHERE routingobjcd = 'EDI-EDIFACT')
WHERE profiletypeid = (SELECT profiletypeid
                       FROM cf_profiletype
                       WHERE profiletypename = 'EDIFACT')
/

UPDATE edi_standard_package_lnk
SET doctypeid = (SELECT routingobjid 
                  FROM cf_routingobject
                  WHERE routingobjcd = 'ISA')
WHERE profiletypeid = (SELECT profiletypeid
                       FROM cf_profiletype
                       WHERE profiletypename = 'X12')
/

UPDATE edi_standard_package_lnk
SET doctypeid = (SELECT routingobjid 
                  FROM cf_routingobject
                  WHERE routingobjcd = 'BG')
WHERE profiletypeid = (SELECT profiletypeid
                       FROM cf_profiletype
                       WHERE profiletypename = 'UCS')
/

UPDATE edi_standard_package_lnk
SET doctypeid = (SELECT routingobjid 
                  FROM cf_routingobject
                  WHERE routingobjcd = 'UNB')
WHERE profiletypeid = (SELECT profiletypeid
                       FROM cf_profiletype
                       WHERE profiletypename = 'EDIFACT')
/


-- dis_imp_edi_syntax_def_actn.sql

--dis_imp_edi_syntax_def_actn
INSERT INTO dis_imp_edi_syntax_def_actn (fromsyntax, tosyntax, actionid)
   SELECT 'E','E', workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIvalidationAndEDI Translation'
     AND NOT EXISTS (SELECT 'x'
                        FROM dis_imp_edi_syntax_def_actn a INNER JOIN pr_workflow b
                             ON a.ACTIONID=b.WORKFLOWID
                       WHERE a.FROMSYNTAX='E' and
                             a.TOSYNTAX='E' and
                            b.WORKFLOWNAME = 'EDIvalidationAndEDI Translation')
/                             
                             

INSERT INTO dis_imp_edi_syntax_def_actn (fromsyntax, tosyntax, actionid)
   SELECT 'E','X', workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIvalidationAndEDI Translation'
     AND NOT EXISTS (SELECT 'x'
                        FROM dis_imp_edi_syntax_def_actn a INNER JOIN pr_workflow b
                             ON a.ACTIONID=b.WORKFLOWID
                       WHERE a.FROMSYNTAX='E' and
                             a.TOSYNTAX='X' and
                             b.WORKFLOWNAME = 'EDIvalidationAndEDI Translation')
/

INSERT INTO dis_imp_edi_syntax_def_actn (fromsyntax, tosyntax, actionid)
   SELECT 'E','F', workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIvalidationAndEDI Translation'
     AND NOT EXISTS (SELECT 'x'
                        FROM dis_imp_edi_syntax_def_actn a INNER JOIN pr_workflow b
                             ON a.ACTIONID=b.WORKFLOWID
                       WHERE a.FROMSYNTAX='E' and
                             a.TOSYNTAX='F' and
                             b.WORKFLOWNAME = 'EDIvalidationAndEDI Translation')
/

INSERT INTO dis_imp_edi_syntax_def_actn (fromsyntax, tosyntax, actionid)
   SELECT 'X','E', workflowid
   FROM pr_workflow
   WHERE workflowname = 'XMLTranslationAndEDIValidation'
     AND NOT EXISTS (SELECT 'x'
                        FROM dis_imp_edi_syntax_def_actn a INNER JOIN pr_workflow b
                             ON a.ACTIONID=b.WORKFLOWID
                       WHERE a.FROMSYNTAX='X' and
                             a.TOSYNTAX='E' and
                             b.WORKFLOWNAME = 'XMLTranslationAndEDIValidation')
/


INSERT INTO dis_imp_edi_syntax_def_actn (fromsyntax, tosyntax, actionid)
   SELECT 'F','E', workflowid
   FROM pr_workflow
   WHERE workflowname = 'RODTranslationAndEDIValidation'
     AND NOT EXISTS (SELECT 'x'
                        FROM dis_imp_edi_syntax_def_actn a INNER JOIN pr_workflow b
                             ON a.ACTIONID=b.WORKFLOWID
                       WHERE a.FROMSYNTAX='F' and
                             a.TOSYNTAX='E' and
                             b.WORKFLOWNAME = 'RODTranslationAndEDIValidation')
/

-- dis_imp_edi_syntax_action_lnk.sql

--dis_imp_edi_syntax_action_lnk

INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXML'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXML'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXML'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXML'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXML'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXML'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXML'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXML'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXML'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXML'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/
 

INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithDupCheckAndValidation'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/
 


INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfCustomXMLWithValidation'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/


INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfOwnerCustomXMLToRNWithDupCheckAndValidation'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/


INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/


INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndRNSC Without Content validation'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/


INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndXML'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndXML'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndXML'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndXML'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'BiDirectionalTranslationOfRNAndXML'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'BiDirectionalTranslationOfRNAndXML'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/


INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheck'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheck'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheck'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheck'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheck'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheck'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheck'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheck'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheck'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheck'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/

INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithDupCheckAndValidation'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/



INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithValidation'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithValidation'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'CustomXMLPassThruWithValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'CustomXMLPassThruWithValidation'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/             
    
    
INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIDeenvelope'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'EDIDeenvelope'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIDeenvelope'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'EDIDeenvelope'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIDeenvelope'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'EDIDeenvelope'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIDeenvelope'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'EDIDeenvelope'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIDeenvelope'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'EDIDeenvelope'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/                 
             
INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIvalidationAndEDI Translation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'EDIvalidationAndEDI Translation'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'EDIvalidationAndEDI Translation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'EDIvalidationAndEDI Translation'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/                 
             

INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'HubOwnerCancellationOfRNProcess'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'HubOwnerCancellationOfRNProcess'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'HubOwnerCancellationOfRNProcess'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'HubOwnerCancellationOfRNProcess'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'HubOwnerCancellationOfRNProcess'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'HubOwnerCancellationOfRNProcess'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'HubOwnerCancellationOfRNProcess'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'HubOwnerCancellationOfRNProcess'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'HubOwnerCancellationOfRNProcess'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'HubOwnerCancellationOfRNProcess'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/                 


INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'RNPassThruWithProcessLogging'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'RNPassThruWithProcessLogging'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'RNPassThruWithProcessLogging'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'RNPassThruWithProcessLogging'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'RNPassThruWithProcessLogging'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'RNPassThruWithProcessLogging'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'RNPassThruWithProcessLogging'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'RNPassThruWithProcessLogging'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'RNPassThruWithProcessLogging'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'RNPassThruWithProcessLogging'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/    


INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'RODTranslationAndEDIValidation'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'RODTranslationAndEDIValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'RODTranslationAndEDIValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'RODTranslationAndEDIValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'RODTranslationAndEDIValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'RODTranslationAndEDIValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'X','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'RODTranslationAndEDIValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'RODTranslationAndEDIValidation'
             and b.FROMSYNTAX='X' and b.TOSYNTAX='E'))
/    


INSERT INTO dis_imp_edi_syntax_action_lnk (fromsyntax, tosyntax, actionid)
   (SELECT 'E','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'XMLTranslationAndEDIValidation'
   and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'XMLTranslationAndEDIValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='E'
             ))
   UNION ALL
   (SELECT 'E','X',workflowid
   FROM pr_workflow
   WHERE workflowname = 'XMLTranslationAndEDIValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'XMLTranslationAndEDIValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='X'))
   UNION ALL
   (SELECT 'E','F',workflowid
   FROM pr_workflow
   WHERE workflowname = 'XMLTranslationAndEDIValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'XMLTranslationAndEDIValidation'
             and b.FROMSYNTAX='E' and b.TOSYNTAX='F'))
   UNION ALL
   (SELECT 'F','E',workflowid
   FROM pr_workflow
   WHERE workflowname = 'XMLTranslationAndEDIValidation'
     and Not Exists (select 'x' from pr_workflow a INNER JOIN dis_imp_edi_syntax_action_lnk b
                   ON a.WORKFLOWID=b.ACTIONID
             where a.workflowname = 'XMLTranslationAndEDIValidation'
             and b.FROMSYNTAX='F' and b.TOSYNTAX='E'))
/    


-- dis_imp_edi_syntax_def_pkg.sql

--dis_imp_edi_syntax_pkg
INSERT INTO dis_imp_edi_syntax_def_pkg (syntax, pkgid)
   (SELECT 'X', routingobjid
    FROM cf_routingobject
    WHERE routingobjname = 'None'
      AND routingobjtypeid = (SELECT routingobjtypeid
                              FROM cf_routingobjtype
                              WHERE routingobjtypekey = 'Package')
and Not Exists (select 'x' from dis_imp_edi_syntax_def_pkg a 
                 INNER JOIN cf_routingobject b ON a.PKGID=b.ROUTINGOBJID
                 INNER JOIN cf_routingobjtype c ON b.ROUTINGOBJTYPEID=c.ROUTINGOBJTYPEID
             where a.SYNTAX = 'X' 
             and b.ROUTINGOBJNAME= 'None' 
             and c.routingobjtypekey = 'Package'
             ))     
/             

INSERT INTO dis_imp_edi_syntax_def_pkg (syntax, pkgid)
   (SELECT 'F', routingobjid
    FROM cf_routingobject
    WHERE routingobjname = 'None'
      AND routingobjtypeid = (SELECT routingobjtypeid
                              FROM cf_routingobjtype
                              WHERE routingobjtypekey = 'Package')
and Not Exists (select 'x' from dis_imp_edi_syntax_def_pkg a 
                 INNER JOIN cf_routingobject b ON a.PKGID=b.ROUTINGOBJID
                 INNER JOIN cf_routingobjtype c ON b.ROUTINGOBJTYPEID=c.ROUTINGOBJTYPEID
             where a.SYNTAX = 'F' 
             and b.ROUTINGOBJNAME= 'None' 
             and c.routingobjtypekey = 'Package'
             ))     
/             

INSERT INTO dis_imp_edi_syntax_def_pkg (syntax, pkgid)
   (SELECT 'E', routingobjid
    FROM cf_routingobject
    WHERE routingobjname = 'N/A'
      AND routingobjtypeid = (SELECT routingobjtypeid
                              FROM cf_routingobjtype
                              WHERE routingobjtypekey = 'Package')
and Not Exists (select 'x' from dis_imp_edi_syntax_def_pkg a 
                 INNER JOIN cf_routingobject b ON a.PKGID=b.ROUTINGOBJID
                 INNER JOIN cf_routingobjtype c ON b.ROUTINGOBJTYPEID=c.ROUTINGOBJTYPEID
             where a.SYNTAX = 'E' 
             and b.ROUTINGOBJNAME= 'N/A' 
             and c.routingobjtypekey = 'Package'
             ))                 
/        

-- CF_GUIDELINEMAP.sql

Create or Replace procedure metadata_insert as

v_ebxmlXSD VARCHAR2(16000) := '<?xml version="1.0" encoding="UTF-8"?>
<!-- Some parsers may require explicit declaration of xmlns:xml="http://www.w3.org/XML/1998/namespace".
In that case, a copy of this schema augmented with the above declaration should be cached and used
for the purpose of schema validation on ebXML messages. -->
<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema" targetNamespace="http://www.oasis-open.org/committees/ebxml-msg/schema/msg-header-2_0.xsd"
xmlns:tns="http://www.oasis-open.org/committees/ebxml-msg/schema/msg-header-2_0.xsd"
xmlns:ds="http://www.w3.org/2000/09/xmldsig#"
xmlns:xlink="http://www.w3.org/1999/xlink"
xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
xmlns="http://www.w3.org/2001/XMLSchema"
elementFormDefault="qualified"
attributeFormDefault="qualified"
version="2.0">
<import namespace="http://www.w3.org/2000/09/xmldsig#"
schemaLocation="http://www.w3.org/TR/xmldsig-core/xmldsig-core-schema.xsd"/>
<import namespace="http://www.w3.org/1999/xlink"
schemaLocation="http://www.oasis-open.org/committees/ebxml-msg/schema/xlink.xsd"/>
<import namespace="http://schemas.xmlsoap.org/soap/envelope/"
schemaLocation="http://www.oasis-open.org/committees/ebxml-msg/schema/envelope.xsd"/>
<import namespace="http://www.w3.org/XML/1998/namespace"
schemaLocation="http://www.w3.org/2001/03/xml.xsd"/>
<!-- MANIFEST, for use in soap:Body element -->
<element name="Manifest">
<complexType>
<sequence>
<element ref="tns:Reference" maxOccurs="unbounded"/>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attributeGroup ref="tns:bodyExtension.grp"/>
</complexType>
</element>
<element name="Reference">
<complexType>
<sequence>
<element ref="tns:Schema" minOccurs="0" maxOccurs="unbounded"/>
<element ref="tns:Description" minOccurs="0" maxOccurs="unbounded"/>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attribute ref="tns:id"/>
<attribute ref="xlink:type" fixed="simple"/>
<attribute ref="xlink:href" use="required"/>
<attribute ref="xlink:role"/>
<anyAttribute namespace="##other" processContents="lax"/>
</complexType>
</element>
<element name="Schema">
<complexType>
<attribute name="location" type="anyURI" use="required"/>
<attribute name="version" type="tns:non-empty-string"/>
</complexType>

</element>
<!-- MESSAGEHEADER, for use in soap:Header element -->
<element name="MessageHeader">
<complexType>
<sequence>
<element ref="tns:From"/>
<element ref="tns:To"/>
<element ref="tns:CPAId"/>
<element ref="tns:ConversationId"/>
<element ref="tns:Service"/>
<element ref="tns:Action"/>
<element ref="tns:MessageData"/>
<element ref="tns:DuplicateElimination" minOccurs="0"/>
<element ref="tns:Description" minOccurs="0" maxOccurs="unbounded"/>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attributeGroup ref="tns:headerExtension.grp"/>
</complexType>
</element>
<element name="CPAId" type="tns:non-empty-string"/>
<element name="ConversationId" type="tns:non-empty-string"/>
<element name="Service">
<complexType>
<simpleContent>
<extension base="tns:non-empty-string">
<attribute name="type" type="tns:non-empty-string"/>
</extension>
</simpleContent>
</complexType>
</element>
<element name="Action" type="tns:non-empty-string"/>
<element name="MessageData">
<complexType>
<sequence>
<element ref="tns:MessageId"/>
<element ref="tns:Timestamp"/>
<element ref="tns:RefToMessageId" minOccurs="0"/>
<element ref="tns:TimeToLive" minOccurs="0"/>
</sequence>
</complexType>
</element>
<element name="MessageId" type="tns:non-empty-string"/>
<element name="TimeToLive" type="dateTime"/>
<element name="DuplicateElimination">
</element>
<!-- SYNC REPLY, for use in soap:Header element -->
<element name="SyncReply">
<complexType>
<sequence>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attributeGroup ref="tns:headerExtension.grp"/>
<attribute ref="soap:actor" use="required"/>
</complexType>
</element>
<!-- MESSAGE ORDER, for use in soap:Header element -->
<element name="MessageOrder">
<complexType>
<sequence>
<element ref="tns:SequenceNumber"/>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attributeGroup ref="tns:headerExtension.grp"/>
</complexType>
</element>
<element name="SequenceNumber" type="tns:sequenceNumber.type"/>
<!-- ACK REQUESTED, for use in soap:Header element -->
<element name="AckRequested">
<complexType>
<sequence>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>

</sequence>
<attributeGroup ref="tns:headerExtension.grp"/>
<attribute ref="soap:actor"/>
<attribute name="signed" type="boolean" use="required"/>
</complexType>
</element>
<!-- ACKNOWLEDGMENT, for use in soap:Header element -->
<element name="Acknowledgment">
<complexType>
<sequence>
<element ref="tns:Timestamp"/>
<element ref="tns:RefToMessageId"/>
<element ref="tns:From" minOccurs="0"/>
<element ref="ds:Reference" minOccurs="0" maxOccurs="unbounded"/>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attributeGroup ref="tns:headerExtension.grp"/>
<attribute ref="soap:actor"/>
</complexType>
</element>
<!-- ERROR LIST, for use in soap:Header element -->
<element name="ErrorList">
<complexType>
<sequence>
<element ref="tns:Error" maxOccurs="unbounded"/>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attributeGroup ref="tns:headerExtension.grp"/>
<attribute name="highestSeverity" type="tns:severity.type" use="required"/>
</complexType>
</element>
<element name="Error">
<complexType>
<sequence>
<element ref="tns:Description" minOccurs="0"/>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attribute ref="tns:id"/>
<attribute name="codeContext" type="anyURI"
default="urn:oasis:names:tc:ebxml-msg:service:errors"/>
<attribute name="errorCode" type="tns:non-empty-string" use="required"/>
<attribute name="severity" type="tns:severity.type" use="required"/>
<attribute name="location" type="tns:non-empty-string"/>
<anyAttribute namespace="##other" processContents="lax"/>
</complexType>
</element>
<!-- STATUS RESPONSE, for use in soap:Body element -->
<element name="StatusResponse">
<complexType>
<sequence>
<element ref="tns:RefToMessageId"/>
<element ref="tns:Timestamp" minOccurs="0"/>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attributeGroup ref="tns:bodyExtension.grp"/>
<attribute name="messageStatus" type="tns:messageStatus.type" use="required"/>
</complexType>
</element>
<!-- STATUS REQUEST, for use in soap:Body element -->
<element name="StatusRequest">
<complexType>
<sequence>
<element ref="tns:RefToMessageId"/>
<any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
</sequence>
<attributeGroup ref="tns:bodyExtension.grp"/>
</complexType>
</element>
<!-- COMMON TYPES -->
<complexType name="sequenceNumber.type">
<simpleContent>
<extension base="nonNegativeInteger">
<attribute name="status" type="tns:status.type" default="Continue"/>
</extension>
</simpleContent>
</complexType>
<simpleType name="status.type">
<restriction base="NMTOKEN">
<enumeration value="Reset"/>
<enumeration value="Continue"/>
</restriction>
</simpleType>
<simpleType name="messageStatus.type">
<restriction base="NMTOKEN">
<enumeration value="UnAuthorized"/>
<enumeration value="NotRecognized"/>
<enumeration value="Received"/>
<enumeration value="Processed"/>
<enumeration value="Forwarded"/>
</restriction>
</simpleType>
<simpleType name="non-empty-string">
<restriction base="string">
<minLength value="1"/>
</restriction>
</simpleType>
<simpleType name="severity.type">
<restriction base="NMTOKEN">
<enumeration value="Warning"/>
<enumeration value="Error"/>
</restriction>
</simpleType>
<!-- COMMON ATTRIBUTES and ATTRIBUTE GROUPS -->
<attribute name="id" type="ID"/>
<attribute name="version" type="tns:non-empty-string"/>
<attributeGroup name="headerExtension.grp">
<attribute ref="tns:id"/>
<attribute ref="tns:version" use="required"/>
<attribute ref="soap:mustUnderstand" use="required"/>
<anyAttribute namespace="##other" processContents="lax"/>
</attributeGroup>
<attributeGroup name="bodyExtension.grp">
<attribute ref="tns:id"/>
<attribute ref="tns:version" use="required"/>
<anyAttribute namespace="##other" processContents="lax"/>
</attributeGroup>
<!-- COMMON ELEMENTS -->
<element name="PartyId">
<complexType>
<simpleContent>
<extension base="tns:non-empty-string">
<attribute name="type" type="tns:non-empty-string"/>
</extension>
</simpleContent>
</complexType>
</element>
<element name="To">
<complexType>
<sequence>
<element ref="tns:PartyId" maxOccurs="unbounded"/>
<element name="Role" type="tns:non-empty-string" minOccurs="0"/>
</sequence>
</complexType>
</element>
<element name="From">
<complexType>
<sequence>
<element ref="tns:PartyId" maxOccurs="unbounded"/>
<element name="Role" type="tns:non-empty-string" minOccurs="0"/>
</sequence>
</complexType>
</element>
<element name="Description">
<complexType>
<simpleContent>
<extension base="tns:non-empty-string">
<attribute ref="xml:lang" use="required"/>
</extension>
</simpleContent>
</complexType>
</element>
<element name="RefToMessageId" type="tns:non-empty-string"/>
<element name="Timestamp" type="dateTime"/>
</xsd:schema>';

v_fil		VARCHAR2(60)	:= 'ebXMLSchema.xsd';
v_lob		BLOB;
v_loc		RAW(32000)	:= UTL_RAW.CAST_TO_RAW(v_ebXMLxsd);
v_len		INTEGER		:= LENGTHB(v_loc);
v_cnt		INTEGER		:= 0;

BEGIN
	dbms_output.put_line(v_fil || ':' || substr(v_ebxmlXSD,1,40) );
	SELECT	count(*)
	INTO	v_cnt
	FROM	cf_guidelinemap
	WHERE	guidelinemapfilename = v_fil
	;
	IF v_cnt = 0 THEN
		INSERT INTO CF_GUIDELINEMAP (
			GUIDELINEMAPID
			, GUIDELINEMAPFILENAME
			, GUIDELINEMAPDESC
			, GUIDELINEMAP
			, COMMONMAPFLG
			, GUIDELINEMAPSIZE) 
		VALUES (CF_GuideLineMap_Seq.NExtVal
			,v_fil
			,'XML Schema to validate the ebXML message format'
			,empty_blob()
			,0
			,v_len/2)
		RETURNING GuideLineMap INTO v_lob	;

		dbms_lob.write (v_lob, v_len/2, 1, v_loc);

		dbms_output.put_line( v_len/2 || ' bytes written for ' || v_fil );
	END IF;
	COMMIT;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 

--- XMLEvent_v1.0ns.xsd

Create or Replace procedure metadata_insert as

v_XMLEVENTxsd VARCHAR2(16000) := '<?xml version="1.0" encoding="UTF-8"?>
<xsd:schema targetNamespace="http://www.ibm.com/websphere/bcg/2003/v1.0/xmleventnotification" xmlns:evntf="http://www.ibm.com/websphere/bcg/2003/v1.0/xmleventnotification" xmlns:xsd="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified">
	<!-- EventNotification version 1.0 document element -->
	<xsd:element name="EventNotification">
		<xsd:complexType>
			<xsd:all>
				<xsd:element ref="evntf:StatusCode"/>
				<xsd:element ref="evntf:StatusMessage"/>
				<xsd:element ref="evntf:EventMessageID"/>
				<xsd:element ref="evntf:BusinessObjectID"/>
				<xsd:element ref="evntf:GlobalMessageID"/>
				<xsd:element ref="evntf:Timestamp"/>
			</xsd:all>
		</xsd:complexType>
	</xsd:element>
	<!-- StatusCode element -->
	<xsd:element name="StatusCode">
		<xsd:simpleType>
			<xsd:restriction base="xsd:string">
				<xsd:maxLength value="3"/>
<!--				<xsd:enumeration value="100"/>
				<xsd:enumeration value="800"/>
				<xsd:enumeration value="900"/>
				<xsd:enumeration value="901"/>
				<xsd:enumeration value="902"/>
				<xsd:enumeration value="903"/>
				<xsd:enumeration value="904"/> -->
			</xsd:restriction>
		</xsd:simpleType>
	</xsd:element>
	<!-- StatusMessage element -->
	<xsd:element name="StatusMessage">
		<xsd:simpleType>
			<xsd:restriction base="xsd:string"/>
		</xsd:simpleType>
	</xsd:element>
	<!-- EventMessageID element -->
	<xsd:element name="EventMessageID">
		<xsd:simpleType>
			<xsd:restriction base="xsd:string"/>
		</xsd:simpleType>
	</xsd:element>
	<!-- BusinessObjectID element -->
	<xsd:element name="BusinessObjectID">
		<xsd:simpleType>
			<xsd:restriction base="xsd:string"/>
		</xsd:simpleType>
	</xsd:element>
	<!-- GlobalMessageID element -->
	<xsd:element name="GlobalMessageID">
		<xsd:simpleType>
			<xsd:restriction base="xsd:string"/>
		</xsd:simpleType>
	</xsd:element>
	<!-- Timestamp element -->
	<xsd:element name="Timestamp">
		<xsd:simpleType>
			<xsd:restriction base="xsd:dateTime"/>
		</xsd:simpleType>
	</xsd:element>
</xsd:schema>';

v_fil		VARCHAR2(60)	:= 'XMLEvent_v1.0ns.xsd';
v_lob		BLOB;
v_loc		RAW(32000)	:= UTL_RAW.CAST_TO_RAW(v_XMLEVENTxsd);
v_len		INTEGER		:= LENGTHB(v_loc);
v_cnt		INTEGER		:= 0;

BEGIN
	dbms_output.put_line(v_fil || ':' || substr(v_XMLEVENTxsd,1,40) );
	SELECT	count(*)
	INTO	v_cnt
	FROM	cf_guidelinemap
	WHERE	guidelinemapfilename = v_fil
	;
	IF v_cnt = 0 THEN
		INSERT INTO CF_GUIDELINEMAP (
			GUIDELINEMAPID
			, GUIDELINEMAPFILENAME
			, GUIDELINEMAPDESC
			, GUIDELINEMAP
			, COMMONMAPFLG
			, GUIDELINEMAPSIZE) 
		VALUES (CF_GuideLineMap_Seq.NExtVal
			,v_fil
			,'XML Schema to validate the XML Event format'
			,empty_blob()
			,0
			,v_len/2)
		RETURNING GuideLineMap INTO v_lob	;

		dbms_lob.write (v_lob, v_len/2, 1, v_loc);

		dbms_output.put_line( v_len/2 || ' bytes written for ' || v_fil );
	END IF;
	COMMIT;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 

Create or Replace procedure metadata_insert as
v_XMLEVENTxml	VARCHAR2(16000) := 
'<?xml version="1.0" encoding="UTF-8" ?>
<evntf:EventNotification xmlns:evntf="http://www.ibm.com/websphere/bcg/2003/v1.0/xmleventnotification">
	<evntf:StatusCode/>
	<evntf:StatusMessage/>
	<evntf:EventMessageID/>
	<evntf:BusinessObjectID/>
	<evntf:GlobalMessageID/>
	<evntf:Timestamp/>
</evntf:EventNotification>';

v_fil		VARCHAR2(60)	:= 'XMLEvent_1.0.xml';
v_lob		BLOB;
v_loc		RAW(32000)	:= UTL_RAW.CAST_TO_RAW(v_XMLEVENTxml);
v_len		INTEGER		:= LENGTHB(v_loc);
v_cnt		INTEGER		:= 0;

BEGIN
	dbms_output.put_line(v_fil || ':' || substr(v_XMLEVENTxml,1,40) );
	SELECT	count(*)
	INTO	v_cnt
	FROM	cf_guidelinemap
	WHERE	guidelinemapfilename = v_fil
	;
	IF v_cnt = 0 THEN
		INSERT INTO CF_GUIDELINEMAP (
			GUIDELINEMAPID
			, GUIDELINEMAPFILENAME
			, GUIDELINEMAPDESC
			, GUIDELINEMAP
			, COMMONMAPFLG
			, GUIDELINEMAPSIZE) 
		VALUES (CF_GuideLineMap_Seq.NExtVal
			,v_fil
			,'WPG XML Event notification message format'
			,empty_blob()
			,0
			,v_len/2)
		RETURNING GuideLineMap INTO v_lob	;

		dbms_lob.write (v_lob, v_len/2, 1, v_loc);

		dbms_output.put_line( v_len/2 || ' bytes written for ' || v_fil );
	END IF;
	COMMIT;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 

Create or Replace procedure metadata_insert as
v_XMLxsd	VARCHAR2(16000) := 
'<?xml version="1.0" encoding="UTF-8" ?>
<xsd:schema
	xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
	targetNamespace="http://www.w3.org/XML/1998/namespace">

	<xsd:attribute	name="lang"		type="xsd:language"/>
</xsd:schema>';

v_fil		VARCHAR2(60)	:= 'xml.xsd';
v_lob		BLOB;
v_loc		RAW(32000)	:= UTL_RAW.CAST_TO_RAW(v_XMLxsd);
v_len		INTEGER		:= LENGTHB(v_loc);
v_cnt		INTEGER		:= 0;

BEGIN
	dbms_output.put_line(v_fil || ':' || substr(v_XMLxsd,1,40) );
	SELECT	count(*)
	INTO	v_cnt
	FROM	cf_guidelinemap
	WHERE	guidelinemapfilename = v_fil
	;
	IF v_cnt = 0 THEN
		INSERT INTO CF_GUIDELINEMAP (
			GUIDELINEMAPID
			, GUIDELINEMAPFILENAME
			, GUIDELINEMAPDESC
			, GUIDELINEMAP
			, COMMONMAPFLG
			, GUIDELINEMAPSIZE) 
		VALUES (CF_GuideLineMap_Seq.NExtVal
			,v_fil
			,'XML schema for message format'
			,empty_blob()
			,0
			,v_len/2)
		RETURNING GuideLineMap INTO v_lob	;

		dbms_lob.write (v_lob, v_len/2, 1, v_loc);

		dbms_output.put_line( v_len/2 || ' bytes written for ' || v_fil );
	END IF;
	COMMIT;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 

Create or Replace procedure metadata_insert as
v_XMLxsd	VARCHAR2(16000) := 
'<?xml version="1.0" encoding="UTF-8"?>
<schema targetNamespace="http://schemas.xmlsoap.org/soap/envelope/" 
                                   xmlns="http://www.w3.org/2001/XMLSchema" 
                                   xmlns:tns="http://schemas.xmlsoap.org/soap/envelope/">
	<include schemaLocation="http://schemas.xmlsoap.org/soap/envelope/" />
</schema>';

v_fil		VARCHAR2(60)	:= 'envelope.xsd';
v_lob		BLOB;
v_loc		RAW(32000)	:= UTL_RAW.CAST_TO_RAW(v_XMLxsd);
v_len		INTEGER		:= LENGTHB(v_loc);
v_cnt		INTEGER		:= 0;

BEGIN
	dbms_output.put_line(v_fil || ':' || substr(v_XMLxsd,1,40) );
	SELECT	count(*)
	INTO	v_cnt
	FROM	cf_guidelinemap
	WHERE	guidelinemapfilename = v_fil
	;
	IF v_cnt = 0 THEN
		INSERT INTO CF_GUIDELINEMAP (
			GUIDELINEMAPID
			, GUIDELINEMAPFILENAME
			, GUIDELINEMAPDESC
			, GUIDELINEMAP
			, COMMONMAPFLG
			, GUIDELINEMAPSIZE) 
		VALUES (CF_GuideLineMap_Seq.NExtVal
			,v_fil
			,'XML schema for message format'
			,empty_blob()
			,0
			,v_len/2)
		RETURNING GuideLineMap INTO v_lob	;

		dbms_lob.write (v_lob, v_len/2, 1, v_loc);

		dbms_output.put_line( v_len/2 || ' bytes written for ' || v_fil );
	END IF;
	COMMIT;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 


Create or Replace procedure metadata_insert as
v_XMLxsd	VARCHAR2(16000) := 
'<?xml version="1.0" encoding="UTF-8" ?>

<!-- Schema for the SOAP/1.1 envelope

Portions (c) 2001 DevelopMentor. 
(c) 2001 W3C (Massachusetts Institute of Technology, Institut National de Recherche en Informatique et en Automatique, Keio University). All Rights Reserved.  
 
This document is governed by the W3C Software License [1] as described in the FAQ [2].
[1] http://www.w3.org/Consortium/Legal/copyright-software-19980720
[2] http://www.w3.org/Consortium/Legal/IPR-FAQ-20000620.html#DTD 
By obtaining, using and/or copying this work, you (the licensee) agree that you have read, understood, and will comply with the following terms and conditions:

Permission to use, copy, modify, and distribute this software and its documentation, with or without modification,  for any purpose and without fee or royalty is hereby granted, provided that you include the following on ALL copies of the software and documentation or portions thereof, including modifications, that you make:

1.  The full text of this NOTICE in a location viewable to users of the redistributed or derivative work. 

2.  Any pre-existing intellectual property disclaimers, notices, or terms and conditions. If none exist, a short notice of the following form (hypertext is preferred, text is permitted) should be used within the body of any redistributed or derivative code: "Copyright (c) 2001 World Wide Web Consortium, (Massachusetts Institute of Technology, Institut National de Recherche en Informatique et en Automatique, Keio University). All Rights Reserved. http://www.w3.org/Consortium/Legal/" 

3.  Notice of any changes or modifications to the W3C files, including the date changes were made. (We recommend you provide URIs to the location from which the code is derived.)   

Original W3C files; http://www.w3.org/2001/06/soap-envelope
Changes made: 
     - reverted namespace to http://schemas.xmlsoap.org/soap/envelope/
     - reverted mustUnderstand to only allow 0 and 1 as lexical values
	 - made encodingStyle a global attribute 20020825
	 - removed default value from mustUnderstand attribute declaration

THIS SOFTWARE AND DOCUMENTATION IS PROVIDED "AS IS," AND COPYRIGHT HOLDERS MAKE NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF THE SOFTWARE OR DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS.

COPYRIGHT HOLDERS WILL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF ANY USE OF THE SOFTWARE OR DOCUMENTATION.

The name and trademarks of copyright holders may NOT be used in advertising or publicity pertaining to the software without specific, written prior permission. Title to copyright in this software and any associated documentation will at all times remain with copyright holders.

-->
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
           xmlns:tns="http://schemas.xmlsoap.org/soap/envelope/"
           targetNamespace="http://schemas.xmlsoap.org/soap/envelope/" >

     
  <!-- Envelope, header and body -->
  <xs:element name="Envelope" type="tns:Envelope" />
  <xs:complexType name="Envelope" >
    <xs:sequence>
      <xs:element ref="tns:Header" minOccurs="0" />
      <xs:element ref="tns:Body" minOccurs="1" />
      <xs:any namespace="##other" minOccurs="0" maxOccurs="unbounded" processContents="lax" />
    </xs:sequence>
    <xs:anyAttribute namespace="##other" processContents="lax" />
  </xs:complexType>

  <xs:element name="Header" type="tns:Header" />
  <xs:complexType name="Header" >
    <xs:sequence>
      <xs:any namespace="##other" minOccurs="0" maxOccurs="unbounded" processContents="lax" />
    </xs:sequence>
    <xs:anyAttribute namespace="##other" processContents="lax" />
  </xs:complexType>
  
  <xs:element name="Body" type="tns:Body" />
  <xs:complexType name="Body" >
    <xs:sequence>
      <xs:any namespace="##any" minOccurs="0" maxOccurs="unbounded" processContents="lax" />
    </xs:sequence>
    <xs:anyAttribute namespace="##any" processContents="lax" >
	  <xs:annotation>
	    <xs:documentation>
		  Prose in the spec does not specify that attributes are allowed on the Body element
		</xs:documentation>
	  </xs:annotation>
	</xs:anyAttribute>
  </xs:complexType>

       
  <!-- Global Attributes.  The following attributes are intended to be usable via qualified attribute names on any complex type referencing them.  -->
  <xs:attribute name="mustUnderstand" >	
     <xs:simpleType>
     <xs:restriction base="xs:boolean">
	   <xs:pattern value="0|1" />
	 </xs:restriction>
   </xs:simpleType>
  </xs:attribute>
  <xs:attribute name="actor" type="xs:anyURI" />

  <xs:simpleType name="encodingStyle" >
    <xs:annotation>
	  <xs:documentation>
	    "encodingStyle" indicates any canonicalization conventions followed in the contents of the containing element.  For example, the value "http://schemas.xmlsoap.org/soap/encoding/" indicates the pattern described in SOAP specification
	  </xs:documentation>
	</xs:annotation>
    <xs:list itemType="xs:anyURI" />
  </xs:simpleType>

  <xs:attribute name="encodingStyle" type="tns:encodingStyle" />
  <xs:attributeGroup name="encodingStyle" >
    <xs:attribute ref="tns:encodingStyle" />
  </xs:attributeGroup>

  <xs:element name="Fault" type="tns:Fault" />
  <xs:complexType name="Fault" final="extension" >
    <xs:annotation>
	  <xs:documentation>
	    Fault reporting structure
	  </xs:documentation>
	</xs:annotation>
    <xs:sequence>
      <xs:element name="faultcode" type="xs:QName" />
      <xs:element name="faultstring" type="xs:string" />
      <xs:element name="faultactor" type="xs:anyURI" minOccurs="0" />
      <xs:element name="detail" type="tns:detail" minOccurs="0" />      
    </xs:sequence>
  </xs:complexType>

  <xs:complexType name="detail">
    <xs:sequence>
      <xs:any namespace="##any" minOccurs="0" maxOccurs="unbounded" processContents="lax" />
    </xs:sequence>
    <xs:anyAttribute namespace="##any" processContents="lax" /> 
  </xs:complexType>

</xs:schema>';

v_fil		VARCHAR2(60)	:= 'soapEnvelope.xsd';
v_lob		BLOB;
v_loc		RAW(32000)	:= UTL_RAW.CAST_TO_RAW(v_XMLxsd);
v_len		INTEGER		:= LENGTHB(v_loc);
v_cnt		INTEGER		:= 0;

BEGIN
	dbms_output.put_line(v_fil || ':' || substr(v_XMLxsd,1,40) );
	SELECT	count(*)
	INTO	v_cnt
	FROM	cf_guidelinemap
	WHERE	guidelinemapfilename = v_fil
	;
	IF v_cnt = 0 THEN
		INSERT INTO CF_GUIDELINEMAP (
			GUIDELINEMAPID
			, GUIDELINEMAPFILENAME
			, GUIDELINEMAPDESC
			, GUIDELINEMAP
			, COMMONMAPFLG
			, GUIDELINEMAPSIZE) 
		VALUES (CF_GuideLineMap_Seq.NExtVal
			,v_fil
			,'XML schema for message format'
			,empty_blob()
			,0
			,v_len/2)
		RETURNING GuideLineMap INTO v_lob	;

		dbms_lob.write (v_lob, v_len/2, 1, v_loc);

		dbms_output.put_line( v_len/2 || ' bytes written for ' || v_fil );
	END IF;
	COMMIT;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 

Create or Replace procedure metadata_insert as
v_XMLxsd	VARCHAR2(16000) := 
'<?xml version="1.0" encoding="UTF-8"?>
<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
xmlns:xlink="http://www.w3.org/1999/xlink"
targetNamespace="http://www.w3.org/1999/xlink"
elementFormDefault="qualified"
attributeFormDefault="qualified">
	<xsd:element name="root">
		<xsd:annotation>
			<xsd:documentation>Comment describing your root element</xsd:documentation>
		</xsd:annotation>
	</xsd:element>
	<xsd:attribute name="type">
	<xsd:simpleType>
		<xsd:restriction base="xsd:NMTOKEN">
			<xsd:enumeration value="simple"/>
			<xsd:enumeration value="extended"/>
			<xsd:enumeration value="locator"/>
			<xsd:enumeration value="arc"/>
			<xsd:enumeration value="resource"/>
			<xsd:enumeration value="title"/>
			<xsd:enumeration value="none"/>
		</xsd:restriction>
	</xsd:simpleType>
	</xsd:attribute>
	<xsd:attribute name="href" type="xsd:anyURI">
	</xsd:attribute>
	<xsd:attribute name="role" type="xsd:anyURI">
	</xsd:attribute>
	<xsd:attribute name="arcrole" type="xsd:anyURI">
	</xsd:attribute>
	<xsd:attribute name="title" type="xsd:string">
	</xsd:attribute>
	<xsd:attribute name="label" type="xsd:NMTOKEN">
	</xsd:attribute>
	<xsd:attribute name="show">
		<xsd:simpleType>
			<xsd:restriction base="xsd:NMTOKEN">
				<xsd:enumeration value="new"/>
				<xsd:enumeration value="replace"/>
				<xsd:enumeration value="embed"/>
				<xsd:enumeration value="other"/>
				<xsd:enumeration value="none"/>
			</xsd:restriction>
		</xsd:simpleType>
	</xsd:attribute>
</xsd:schema>';

v_fil		VARCHAR2(60)	:= 'xlink.xsd';
v_lob		BLOB;
v_loc		RAW(32000)	:= UTL_RAW.CAST_TO_RAW(v_XMLxsd);
v_len		INTEGER		:= LENGTHB(v_loc);
v_cnt		INTEGER		:= 0;

BEGIN
	dbms_output.put_line(v_fil || ':' || substr(v_XMLxsd,1,40) );
	SELECT	count(*)
	INTO	v_cnt
	FROM	cf_guidelinemap
	WHERE	guidelinemapfilename = v_fil
	;
	IF v_cnt = 0 THEN
		INSERT INTO CF_GUIDELINEMAP (
			GUIDELINEMAPID
			, GUIDELINEMAPFILENAME
			, GUIDELINEMAPDESC
			, GUIDELINEMAP
			, COMMONMAPFLG
			, GUIDELINEMAPSIZE) 
		VALUES (CF_GuideLineMap_Seq.NExtVal
			,v_fil
			,'XML schema for message format'
			,empty_blob()
			,0
			,v_len/2)
		RETURNING GuideLineMap INTO v_lob	;

		dbms_lob.write (v_lob, v_len/2, 1, v_loc);

		dbms_output.put_line( v_len/2 || ' bytes written for ' || v_fil );
	END IF;
	COMMIT;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 
Create or Replace procedure metadata_insert as
v_XMLxsd	VARCHAR2(16000) := 
'<?xml version="1.0" encoding="utf-8"?>
<!-- Schema for XML Signatures
    http://www.w3.org/2000/09/xmldsig#
    $Revision: 1.1 $ on $Date: 2002/02/08 20:32:26 $ by $Author: reagle $

    Copyright 2001 The Internet Society and W3C (Massachusetts Institute
    of Technology, Institut National de Recherche en Informatique et en
    Automatique, Keio University). All Rights Reserved.
    http://www.w3.org/Consortium/Legal/

    This document is governed by the W3C Software License [1] as described
    in the FAQ [2].

    [1] http://www.w3.org/Consortium/Legal/copyright-software-19980720
    [2] http://www.w3.org/Consortium/Legal/IPR-FAQ-20000620.html#DTD
-->


<schema xmlns="http://www.w3.org/2001/XMLSchema"
        xmlns:ds="http://www.w3.org/2000/09/xmldsig#"
        targetNamespace="http://www.w3.org/2000/09/xmldsig#"
        version="0.1" elementFormDefault="qualified"> 

<!-- Basic Types Defined for Signatures -->

<simpleType name="CryptoBinary">
  <restriction base="base64Binary">
  </restriction>
</simpleType>

<!-- Start Signature -->

<element name="Signature" type="ds:SignatureType"/>
<complexType name="SignatureType">
  <sequence> 
    <element ref="ds:SignedInfo"/> 
    <element ref="ds:SignatureValue"/> 
    <element ref="ds:KeyInfo" minOccurs="0"/> 
    <element ref="ds:Object" minOccurs="0" maxOccurs="unbounded"/> 
  </sequence>  
  <attribute name="Id" type="ID" use="optional"/>
</complexType>

  <element name="SignatureValue" type="ds:SignatureValueType"/> 
  <complexType name="SignatureValueType">
    <simpleContent>
      <extension base="base64Binary">
        <attribute name="Id" type="ID" use="optional"/>
      </extension>
    </simpleContent>
  </complexType>

<!-- Start SignedInfo -->

<element name="SignedInfo" type="ds:SignedInfoType"/>
<complexType name="SignedInfoType">
  <sequence> 
    <element ref="ds:CanonicalizationMethod"/> 
    <element ref="ds:SignatureMethod"/> 
    <element ref="ds:Reference" maxOccurs="unbounded"/> 
  </sequence>  
  <attribute name="Id" type="ID" use="optional"/> 
</complexType>

  <element name="CanonicalizationMethod" type="ds:CanonicalizationMethodType"/> 
  <complexType name="CanonicalizationMethodType" mixed="true">
    <sequence>
      <any namespace="##any" minOccurs="0" maxOccurs="unbounded"/>
      <!-- (0,unbounded) elements from (1,1) namespace -->
    </sequence>
    <attribute name="Algorithm" type="anyURI" use="required"/> 
  </complexType>

  <element name="SignatureMethod" type="ds:SignatureMethodType"/>
  <complexType name="SignatureMethodType" mixed="true">
    <sequence>
      <element name="HMACOutputLength" minOccurs="0" type="ds:HMACOutputLengthType"/>
      <any namespace="##other" minOccurs="0" maxOccurs="unbounded"/>
      <!-- (0,unbounded) elements from (1,1) external namespace -->
    </sequence>
    <attribute name="Algorithm" type="anyURI" use="required"/> 
  </complexType>

<!-- Start Reference -->

<element name="Reference" type="ds:ReferenceType"/>
<complexType name="ReferenceType">
  <sequence> 
    <element ref="ds:Transforms" minOccurs="0"/> 
    <element ref="ds:DigestMethod"/> 
    <element ref="ds:DigestValue"/> 
  </sequence>
  <attribute name="Id" type="ID" use="optional"/> 
  <attribute name="URI" type="anyURI" use="optional"/> 
  <attribute name="Type" type="anyURI" use="optional"/> 
</complexType>

  <element name="Transforms" type="ds:TransformsType"/>
  <complexType name="TransformsType">
    <sequence>
      <element ref="ds:Transform" maxOccurs="unbounded"/>  
    </sequence>
  </complexType>

  <element name="Transform" type="ds:TransformType"/>
  <complexType name="TransformType" mixed="true">
    <choice minOccurs="0" maxOccurs="unbounded"> 
      <any namespace="##other" processContents="lax"/>
      <!-- (1,1) elements from (0,unbounded) namespaces -->
      <element name="XPath" type="string"/> 
    </choice>
    <attribute name="Algorithm" type="anyURI" use="required"/> 
  </complexType>

<!-- End Reference -->

<element name="DigestMethod" type="ds:DigestMethodType"/>
<complexType name="DigestMethodType" mixed="true"> 
  <sequence>
    <any namespace="##other" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
  </sequence>    
  <attribute name="Algorithm" type="anyURI" use="required"/> 
</complexType>

<element name="DigestValue" type="ds:DigestValueType"/>
<simpleType name="DigestValueType">
  <restriction base="base64Binary"/>
</simpleType>

<!-- End SignedInfo -->

<!-- Start KeyInfo -->

<element name="KeyInfo" type="ds:KeyInfoType"/> 
<complexType name="KeyInfoType" mixed="true">
  <choice maxOccurs="unbounded">     
    <element ref="ds:KeyName"/> 
    <element ref="ds:KeyValue"/> 
    <element ref="ds:RetrievalMethod"/> 
    <element ref="ds:X509Data"/> 
    <element ref="ds:PGPData"/> 
    <element ref="ds:SPKIData"/>
    <element ref="ds:MgmtData"/>
    <any processContents="lax" namespace="##other"/>
    <!-- (1,1) elements from (0,unbounded) namespaces -->
  </choice>
  <attribute name="Id" type="ID" use="optional"/> 
</complexType>

  <element name="KeyName" type="string"/>
  <element name="MgmtData" type="string"/>

  <element name="KeyValue" type="ds:KeyValueType"/> 
  <complexType name="KeyValueType" mixed="true">
   <choice>
     <element ref="ds:DSAKeyValue"/>
     <element ref="ds:RSAKeyValue"/>
     <any namespace="##other" processContents="lax"/>
   </choice>
  </complexType>

  <element name="RetrievalMethod" type="ds:RetrievalMethodType"/> 
  <complexType name="RetrievalMethodType">
    <sequence>
      <element ref="ds:Transforms" minOccurs="0"/> 
    </sequence>  
    <attribute name="URI" type="anyURI"/>
    <attribute name="Type" type="anyURI" use="optional"/>
  </complexType>

<!-- Start X509Data -->

<element name="X509Data" type="ds:X509DataType"/> 
<complexType name="X509DataType">
  <sequence maxOccurs="unbounded">
    <choice>
      <element name="X509IssuerSerial" type="ds:X509IssuerSerialType"/>
      <element name="X509SKI" type="base64Binary"/>
      <element name="X509SubjectName" type="string"/>
      <element name="X509Certificate" type="base64Binary"/>
      <element name="X509CRL" type="base64Binary"/>
      <any namespace="##other" processContents="lax"/>
    </choice>
  </sequence>
</complexType>

<complexType name="X509IssuerSerialType"> 
  <sequence> 
    <element name="X509IssuerName" type="string"/> 
    <element name="X509SerialNumber" type="integer"/> 
  </sequence>
</complexType>

<!-- End X509Data -->

<!-- Begin PGPData -->

<element name="PGPData" type="ds:PGPDataType"/> 
<complexType name="PGPDataType"> 
  <choice>
    <sequence>
      <element name="PGPKeyID" type="base64Binary"/> 
      <element name="PGPKeyPacket" type="base64Binary" minOccurs="0"/> 
      <any namespace="##other" processContents="lax" minOccurs="0"
       maxOccurs="unbounded"/>
    </sequence>
    <sequence>
      <element name="PGPKeyPacket" type="base64Binary"/> 
      <any namespace="##other" processContents="lax" minOccurs="0"
       maxOccurs="unbounded"/>
    </sequence>
  </choice>
</complexType>

<!-- End PGPData -->

<!-- Begin SPKIData -->

<element name="SPKIData" type="ds:SPKIDataType"/> 
<complexType name="SPKIDataType">
  <sequence maxOccurs="unbounded">
    <element name="SPKISexp" type="base64Binary"/>
    <any namespace="##other" processContents="lax" minOccurs="0"/>
  </sequence>
</complexType> 

<!-- End SPKIData -->

<!-- End KeyInfo -->

<!-- Start Object (Manifest, SignatureProperty) -->

<element name="Object" type="ds:ObjectType"/> 
<complexType name="ObjectType" mixed="true">
  <sequence minOccurs="0" maxOccurs="unbounded">
    <any namespace="##any" processContents="lax"/>
  </sequence>
  <attribute name="Id" type="ID" use="optional"/> 
  <attribute name="MimeType" type="string" use="optional"/> <!-- add a grep facet -->
  <attribute name="Encoding" type="anyURI" use="optional"/> 
</complexType>

<element name="Manifest" type="ds:ManifestType"/> 
<complexType name="ManifestType">
  <sequence>
    <element ref="ds:Reference" maxOccurs="unbounded"/> 
  </sequence>
  <attribute name="Id" type="ID" use="optional"/> 
</complexType>

<element name="SignatureProperties" type="ds:SignaturePropertiesType"/> 
<complexType name="SignaturePropertiesType">
  <sequence>
    <element ref="ds:SignatureProperty" maxOccurs="unbounded"/> 
  </sequence>
  <attribute name="Id" type="ID" use="optional"/> 
</complexType>

   <element name="SignatureProperty" type="ds:SignaturePropertyType"/> 
   <complexType name="SignaturePropertyType" mixed="true">
     <choice maxOccurs="unbounded">
       <any namespace="##other" processContents="lax"/>
       <!-- (1,1) elements from (1,unbounded) namespaces -->
     </choice>
     <attribute name="Target" type="anyURI" use="required"/> 
     <attribute name="Id" type="ID" use="optional"/> 
   </complexType>

<!-- End Object (Manifest, SignatureProperty) -->

<!-- Start Algorithm Parameters -->

<simpleType name="HMACOutputLengthType">
  <restriction base="integer"/>
</simpleType>

<!-- Start KeyValue Element-types -->

<element name="DSAKeyValue" type="ds:DSAKeyValueType"/>
<complexType name="DSAKeyValueType">
  <sequence>
    <sequence minOccurs="0">
      <element name="P" type="ds:CryptoBinary"/>
      <element name="Q" type="ds:CryptoBinary"/>
    </sequence>
    <element name="G" type="ds:CryptoBinary" minOccurs="0"/>
    <element name="Y" type="ds:CryptoBinary"/>
    <element name="J" type="ds:CryptoBinary" minOccurs="0"/>
    <sequence minOccurs="0">
      <element name="Seed" type="ds:CryptoBinary"/>
      <element name="PgenCounter" type="ds:CryptoBinary"/>
    </sequence>
  </sequence>
</complexType>

<element name="RSAKeyValue" type="ds:RSAKeyValueType"/>
<complexType name="RSAKeyValueType">
  <sequence>
    <element name="Modulus" type="ds:CryptoBinary"/> 
    <element name="Exponent" type="ds:CryptoBinary"/> 
  </sequence>
</complexType> 

<!-- End KeyValue Element-types -->

<!-- End Signature -->

</schema>';

v_fil		VARCHAR2(60)	:= 'xmldsig-core-schema.xsd';
v_lob		BLOB;
v_loc		RAW(32000)	:= UTL_RAW.CAST_TO_RAW(v_XMLxsd);
v_len		INTEGER		:= LENGTHB(v_loc);
v_cnt		INTEGER		:= 0;

BEGIN
	dbms_output.put_line(v_fil || ':' || substr(v_XMLxsd,1,40) );
	SELECT	count(*)
	INTO	v_cnt
	FROM	cf_guidelinemap
	WHERE	guidelinemapfilename = v_fil
	;
	IF v_cnt = 0 THEN
		INSERT INTO CF_GUIDELINEMAP (
			GUIDELINEMAPID
			, GUIDELINEMAPFILENAME
			, GUIDELINEMAPDESC
			, GUIDELINEMAP
			, COMMONMAPFLG
			, GUIDELINEMAPSIZE) 
		VALUES (CF_GuideLineMap_Seq.NExtVal
			,v_fil
			,'XML schema for message format'
			,empty_blob()
			,0
			,v_len/2)
		RETURNING GuideLineMap INTO v_lob	;

		dbms_lob.write (v_lob, v_len/2, 1, v_loc);

		dbms_output.put_line( v_len/2 || ' bytes written for ' || v_fil );
	END IF;
	COMMIT;
END metadata_insert;
/
Call Metadata_Insert()
/
drop procedure metadata_insert
/ 




-- CF_PROFILEATTR_LNK.sql

Insert Into CF_PROFILEATTR_LNK(PROFILEID, PROFILEATTRID, ATTRVALUE)
	SELECT	p.ProfileId
		,a.ProfileAttrId
		,'Asynchronous'
	FROM 	cf_profile p, cf_profileattr a
	WHERE	p.PROFILENAME = 'WPG-ebMS-ASynchronous'
	AND	a.ATTRNAME = 'bcg.ebms.profileattr.responsetype'
	AND Not Exists (select 'x' from CF_PROFILEATTR_LNK where ProfileID = p.ProfileID and ProfileAttrID = a.ProfileAttrId and AttrValue = 'Asynchronous')
/
Insert Into CF_PROFILEATTR_LNK(PROFILEID, PROFILEATTRID, ATTRVALUE)
	SELECT	p.ProfileId
		,a.ProfileAttrId
		,'Synchronous'
	FROM 	cf_profile p, cf_profileattr a
	WHERE	p.PROFILENAME = 'WPG-ebMS-Synchronous'
	AND	a.ATTRNAME = 'bcg.ebms.profileattr.responsetype'
	AND Not Exists (select 'x' from CF_PROFILEATTR_LNK where ProfileID = p.ProfileID and ProfileAttrID = a.ProfileAttrId and AttrValue = 'Synchronous')
/


-- PR_MDNSTATE.sql

Insert Into PR_MDNSTATE (MDNSTATEKEY,NAMENLSKEY,ICONNAME)
	Select 0,'doc-pkg-viewer.icon.unknownMDNState','unknownMDN.gif' from Dual
	Where Not Exists (select 'x' from PR_MDNSTATE where MDNSTATEKEY  = 0)  
/
Insert Into PR_MDNSTATE (MDNSTATEKEY,NAMENLSKEY,ICONNAME)
	Select 1,'doc-pkg-viewer.icon.mdnnotreq','notrequired.gif' from Dual
	Where Not Exists (select 'x' from PR_MDNSTATE where MDNSTATEKEY  = 1)  
/
Insert Into PR_MDNSTATE (MDNSTATEKEY,NAMENLSKEY,ICONNAME)
	Select 2,'doc-pkg-viewer.icon.mdnWaiting','waiting.gif' from Dual
	Where Not Exists (select 'x' from PR_MDNSTATE where MDNSTATEKEY  = 2)  
/
Insert Into PR_MDNSTATE (MDNSTATEKEY,NAMENLSKEY,ICONNAME)
	Select 3,'doc-pkg-viewer.icon.mdnNotReceived','icon_retries_exceeded.gif' from Dual
	Where Not Exists (select 'x' from PR_MDNSTATE where MDNSTATEKEY  = 3)  
/
Insert Into PR_MDNSTATE (MDNSTATEKEY,NAMENLSKEY,ICONNAME)
	Select 4,'doc-pkg-viewer.icon.mdnDispositionError','mdnDispError.gif' from Dual
	Where Not Exists (select 'x' from PR_MDNSTATE where MDNSTATEKEY  = 4)  
/
Insert Into PR_MDNSTATE (MDNSTATEKEY,NAMENLSKEY,ICONNAME)
	Select 5,'doc-pkg-viewer.icon.mdnprocessed','received.gif' from Dual
	Where Not Exists (select 'x' from PR_MDNSTATE where MDNSTATEKEY  = 5)
/
Insert Into PR_MDNSTATE (MDNSTATEKEY,NAMENLSKEY,ICONNAME)
    Select 6,'doc-pkg-viewer.icon.mdnnotexpected','error.gif'  from Dual 
    Where Not Exists (select 'x' from PR_MDNSTATE where MDNSTATEKEY  = 6)  
/
Insert Into PR_MDNSTATE (MDNSTATEKEY,NAMENLSKEY,ICONNAME)
	Select 7,'doc-pkg-viewer.icon.mdnexpired','mdnexpired.gif' from Dual
	Where Not Exists (select 'x' from PR_MDNSTATE where MDNSTATEKEY  = 7) 
/
INSERT INTO PR_MDNSTATE (MDNSTATEKEY,NAMENLSKEY,ICONNAME)
  Select 8,'doc-pkg-viewer.icon.mdnprocessingerror','mdnprocError.gif'
   From DUAL 
   Where Not Exists (Select 'x' From PR_MDNSTATE Where MDNSTATEKEY =8) 
/   

-- PR_ALERTNOTIFYMODE.sql

INSERT into PR_ALERTNOTIFYMODE(ID,MODENAME, RESOURCEKEY,ROWTS)
Select PR_ALERTNOTIFYMODE_SEQ.NEXTVAL,'subscribed', ' ',1 from dual
where not exists (select 'x' from PR_ALERTNOTIFYMODE where MODENAME = 'subscribed')
/
INSERT into PR_ALERTNOTIFYMODE(ID,MODENAME, RESOURCEKEY,ROWTS)
Select PR_ALERTNOTIFYMODE_SEQ.NEXTVAL,'allRelated', ' ',1 from dual
where not exists (select 'x' from PR_ALERTNOTIFYMODE where MODENAME = 'allRelated')
/


-- PR_COMMUNITYMANAGERDEFAULT.sql

-- For Migration.  The existing old hubowner should be made as Default.

INSERT INTO PR_COMMUNITYMANAGERDEFAULT(                       
                     DEFAULTCOMMUNITYMANAGERID,  PARTNERID
                       ) 
                       select PR_COMMUNITYMANAGERDEFAULT_SEQ.nextval,partnerid from PR_PARTNER_HUBOWNER_VW
                       where Not Exists (select 'x' from PR_COMMUNITYMANAGERDEFAULT)
                       
/                       

-- For Internal Purpose

INSERT INTO PR_COMMUNITYMANAGERDEFAULT(                       
                     DEFAULTCOMMUNITYMANAGERID,  PARTNERID
                       ) 
                       select PR_COMMUNITYMANAGERDEFAULT_SEQ.nextval,partnerid from pr_partner where 
                       partnername='Hub Operator'
                       AND Not Exists (select 'x' from PR_COMMUNITYMANAGERDEFAULT)

/       

-- CF_WEBSERVICE_ATTR.sql

Insert into CF_WEBSERVICE_ATTR (wsattrid,wsattrkey,wsattrdesc,resourcekey)
SELECT CF_WEBSERVICE_ATTR_seq.nextval,'bcg.webservice.attr.style','bcg.webservice.attr.style','bcg.webservice.attr.style'
From DUAL
Where Not Exists (select 'x' from CF_WEBSERVICE_ATTR where wsattrkey = 'bcg.webservice.attr.style')
/

Insert into CF_WEBSERVICE_ATTR (wsattrid,wsattrkey,wsattrdesc,resourcekey)
SELECT CF_WEBSERVICE_ATTR_seq.nextval,'bcg.webservice.attr.operationType','bcg.webservice.attr.operationType','bcg.webservice.attr.operationType'
From DUAL
Where Not Exists (select 'x' from CF_WEBSERVICE_ATTR where wsattrkey = 'bcg.webservice.attr.operationType')
/

Insert into CF_WEBSERVICE_ATTR (wsattrid,wsattrkey,wsattrdesc,resourcekey)
SELECT CF_WEBSERVICE_ATTR_seq.nextval,'bcg.webservice.attr.parameterType','bcg.webservice.attr.parameterType','bcg.webservice.attr.parameterType'
From DUAL
Where Not Exists (select 'x' from CF_WEBSERVICE_ATTR where wsattrkey = 'bcg.webservice.attr.parameterType')
/

Insert into CF_WEBSERVICE_ATTR (wsattrid,wsattrkey,wsattrdesc,resourcekey)
SELECT CF_WEBSERVICE_ATTR_seq.nextval,'bcg.webservice.attr.parameterName','bcg.webservice.attr.parameterName','bcg.webservice.attr.parameterName'
From DUAL
Where Not Exists (select 'x' from CF_WEBSERVICE_ATTR where wsattrkey = 'bcg.webservice.attr.parameterName')
/

-- CF_WEBSERVICE_ATTRVALUE.sql

INSERT INTO cf_webservice_attrvalue
            (wsattrvalueid, wsattrid, wsroutingobjid, wsattrvalue)
   SELECT cf_webservice_attr_seq.NEXTVAL, b.wsattrid, a.routingobjid,
          'Document'
     FROM cf_routingobject a, cf_webservice_attr b
    WHERE a.routingobjname = 'ErrorFlowDocument'
      AND routingobjtypeid = 5
      AND b.wsattrkey = 'bcg.webservice.attr.style'
        AND NOT EXISTS(select 'x' from cf_routingobject a,cf_webservice_attr b,cf_webservice_attrvalue c
            where a.ROUTINGOBJID=c.WSROUTINGOBJID 
            AND b.WSATTRID=c.WSATTRID
            AND c.WSATTRVALUE= 'Document' 
            and b.wsattrkey = 'bcg.webservice.attr.style'
            and a.routingobjname = 'ErrorFlowDocument')
/  

INSERT INTO cf_webservice_attrvalue
            (wsattrvalueid, wsattrid, wsroutingobjid, wsattrvalue)
   SELECT cf_webservice_attr_seq.NEXTVAL, b.wsattrid, a.routingobjid,
          '1'
     FROM cf_routingobject a, cf_webservice_attr b
    WHERE a.routingobjname = 'ErrorFlowDocument'
      AND routingobjtypeid = 5
      AND b.wsattrkey = 'bcg.webservice.attr.operationType'
        AND NOT EXISTS(select 'x' from cf_routingobject a,cf_webservice_attr b,cf_webservice_attrvalue c
            where a.ROUTINGOBJID=c.WSROUTINGOBJID 
            AND b.WSATTRID=c.WSATTRID
            AND c.WSATTRVALUE= '1' 
            and b.wsattrkey = 'bcg.webservice.attr.operationType'
       and a.routingobjname = 'ErrorFlowDocument')
/      


INSERT INTO cf_webservice_attrvalue
            (wsattrvalueid, wsattrid, wsroutingobjid, wsattrvalue)
   SELECT cf_webservice_attr_seq.NEXTVAL, b.wsattrid, a.routingobjid,
          'ErrorFlowDocumentType'
     FROM cf_routingobject a, cf_webservice_attr b
    WHERE a.routingobjname = 'ErrorFlowDocument'
      AND routingobjtypeid = 5
      AND b.wsattrkey = 'bcg.webservice.attr.parameterType'
       AND NOT EXISTS(select 'x' from cf_routingobject a,cf_webservice_attr b,cf_webservice_attrvalue c
                  where a.ROUTINGOBJID=c.WSROUTINGOBJID 
                  AND b.WSATTRID=c.WSATTRID
                  AND c.WSATTRVALUE= 'ErrorFlowDocumentType' 
                  and b.wsattrkey = 'bcg.webservice.attr.parameterType'
       and a.routingobjname = 'ErrorFlowDocument')
/      

INSERT INTO cf_webservice_attrvalue
            (wsattrvalueid, wsattrid, wsroutingobjid, wsattrvalue)
   SELECT cf_webservice_attr_seq.NEXTVAL, b.wsattrid, a.routingobjid,
          'ErrorFlowDocumentMessagePart'
     FROM cf_routingobject a, cf_webservice_attr b
    WHERE a.routingobjname = 'ErrorFlowDocument'
      AND routingobjtypeid = 5
      AND b.wsattrkey = 'bcg.webservice.attr.parameterName'
      AND NOT EXISTS(select 'x' from cf_routingobject a,cf_webservice_attr b,cf_webservice_attrvalue c
                        where a.ROUTINGOBJID=c.WSROUTINGOBJID 
                        AND b.WSATTRID=c.WSATTRID
                        AND c.WSATTRVALUE= 'ErrorFlowDocumentMessagePart' 
                        and b.wsattrkey = 'bcg.webservice.attr.parameterName'
       and a.routingobjname = 'ErrorFlowDocument')
/      

INSERT INTO cf_webservice_attrvalue
            (wsattrvalueid, wsattrid, wsroutingobjid, wsattrvalue)
   SELECT cf_webservice_attr_seq.NEXTVAL, b.wsattrid, a.routingobjid,
          'RPC'
     FROM cf_routingobject a, cf_webservice_attr b
    WHERE a.routingobjname = '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument'
      AND routingobjtypeid = 5
      AND b.wsattrkey = 'bcg.webservice.attr.style'
       AND NOT EXISTS(select 'x' from cf_routingobject a,cf_webservice_attr b,cf_webservice_attrvalue c
                              where a.ROUTINGOBJID=c.WSROUTINGOBJID 
                              AND b.WSATTRID=c.WSATTRID
                              AND c.WSATTRVALUE= 'RPC' 
                              and b.wsattrkey = 'bcg.webservice.attr.style'
       and a.routingobjname = '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument')
/      

INSERT INTO cf_webservice_attrvalue
            (wsattrvalueid, wsattrid, wsroutingobjid, wsattrvalue)
   SELECT cf_webservice_attr_seq.NEXTVAL, b.wsattrid, a.routingobjid,
          '1'
     FROM cf_routingobject a, cf_webservice_attr b
    WHERE a.routingobjname = '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument'
      AND routingobjtypeid = 5
      AND b.wsattrkey = 'bcg.webservice.attr.operationType'
      AND NOT EXISTS(select 'x' from cf_routingobject a,cf_webservice_attr b,cf_webservice_attrvalue c
                                    where a.ROUTINGOBJID=c.WSROUTINGOBJID 
                                    AND b.WSATTRID=c.WSATTRID
                                    AND c.WSATTRVALUE= '1' 
                                    and b.wsattrkey = 'bcg.webservice.attr.operationType'
       and a.routingobjname = '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument')
/      

INSERT INTO cf_webservice_attrvalue
            (wsattrvalueid, wsattrid, wsroutingobjid, wsattrvalue)
   SELECT cf_webservice_attr_seq.NEXTVAL, b.wsattrid, a.routingobjid,
          'ErrorFlowDocumentType'
     FROM cf_routingobject a, cf_webservice_attr b
    WHERE a.routingobjname = '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument'
      AND routingobjtypeid = 5
      AND b.wsattrkey = 'bcg.webservice.attr.parameterType'
      AND NOT EXISTS(select 'x' from cf_routingobject a,cf_webservice_attr b,cf_webservice_attrvalue c
                                          where a.ROUTINGOBJID=c.WSROUTINGOBJID 
                                          AND b.WSATTRID=c.WSATTRID
                                          AND c.WSATTRVALUE= 'ErrorFlowDocumentType' 
                                          and b.wsattrkey = 'bcg.webservice.attr.parameterType'
       and a.routingobjname = '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument')
/      

INSERT INTO cf_webservice_attrvalue
            (wsattrvalueid, wsattrid, wsroutingobjid, wsattrvalue)
   SELECT cf_webservice_attr_seq.NEXTVAL, b.wsattrid, a.routingobjid,
          'ErrorFlowDocumentMessagePart'
     FROM cf_routingobject a, cf_webservice_attr b
    WHERE a.routingobjname = '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument'
      AND routingobjtypeid = 5
      AND b.wsattrkey = 'bcg.webservice.attr.parameterName'
       AND NOT EXISTS(select 'x' from cf_routingobject a,cf_webservice_attr b,cf_webservice_attrvalue c
                                                where a.ROUTINGOBJID=c.WSROUTINGOBJID 
                                                AND b.WSATTRID=c.WSATTRID
                                                AND c.WSATTRVALUE= 'ErrorFlowDocumentMessagePart' 
                                                and b.wsattrkey = 'bcg.webservice.attr.parameterName'
       and a.routingobjname = '{http://www.ibm.com/websphere/bcg/2007/v1.0/xmlerrorflow}:InvokeErrorFlowDocument')
/      

-- CF_ROUTINGATTR_PIP.sql

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
		,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  'bcg.ro.attropt.value.NONE'
		, 1
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'bcg.ro.RNIF20.Compression'
	AND NOT EXISTS 
	(SELECT 'x' 
	FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
	WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
	                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='bcg.ro.attropt.value.NONE' 
	                       AND  cv.ROUTINGOBJTYPEID = 1))
                       
		
/


INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT  CF_ROUTINGATTR_PIP_SEQ.nextval
		,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  'All'
		, 1
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'bcg.ro.RNIF20.CompressContentType'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='All' 
	                       AND  cv.ROUTINGOBJTYPEID = 1))
/

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  '0KB'
		, 1
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'bcg.ro.RNIF20.CompressSize'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='0KB' 
	                       AND  cv.ROUTINGOBJTYPEID = 1))
/

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
		,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  'bcg.ro.attropt.value.NONE'
		, 2
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'bcg.ro.RNIF20.Compression'
	AND NOT EXISTS 
	(SELECT 'x' 
	FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
	WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
	                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='bcg.ro.attropt.value.NONE' 
	                       AND  cv.ROUTINGOBJTYPEID = 2 ))
                       
		
/


INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT  CF_ROUTINGATTR_PIP_SEQ.nextval
		,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  'All'
		, 2
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'bcg.ro.RNIF20.CompressContentType'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='All' 
	                       AND  cv.ROUTINGOBJTYPEID = 2))
/

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  '0KB'
		, 2
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'bcg.ro.RNIF20.CompressSize'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='0KB' 
	                       AND  cv.ROUTINGOBJTYPEID =2))
/



INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
		,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  'bcg.ro.attropt.value.NONE'
		, 3
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'bcg.ro.RNIF20.Compression'
	AND NOT EXISTS 
	(SELECT 'x' 
	FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
	WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
	                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='bcg.ro.attropt.value.NONE' 
	                       AND  cv.ROUTINGOBJTYPEID = 3))
                       
		
/


INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT  CF_ROUTINGATTR_PIP_SEQ.nextval
		,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  'All'
		, 3
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'bcg.ro.RNIF20.CompressContentType'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='All' 
	                       AND  cv.ROUTINGOBJTYPEID = 3))
/

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  '0KB'
		, 3
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'bcg.ro.RNIF20.CompressSize'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='0KB' 
	                       AND  cv.ROUTINGOBJTYPEID = 3))
/


INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  'Y'
		, 1
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'MsgStoreRequired'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='Y' 
	                       AND  cv.ROUTINGOBJTYPEID = 1))
/

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  'Y'
		, 2
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'MsgStoreRequired'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='Y' 
	                       AND  cv.ROUTINGOBJTYPEID = 2))
/

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNIFV02.00'
		,  'Y'
		, 3
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'MsgStoreRequired'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIFV02.00' AND cv.ROUTINGOBJDEFAULT ='Y' 
	                       AND  cv.ROUTINGOBJTYPEID = 3))
/

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNIF1.1'
		,  'Y'
		, 1
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'MsgStoreRequired'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIF1.1' AND cv.ROUTINGOBJDEFAULT ='Y' 
	                       AND  cv.ROUTINGOBJTYPEID = 1))
/

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNIF1.1'
		,  'Y'
		, 2
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'MsgStoreRequired'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIF1.1' AND cv.ROUTINGOBJDEFAULT ='Y' 
	                       AND  cv.ROUTINGOBJTYPEID = 2))
/

INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNIF1.1'
		,  'Y'
		, 3
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'MsgStoreRequired'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNIF1.1' AND cv.ROUTINGOBJDEFAULT ='Y' 
	                       AND  cv.ROUTINGOBJTYPEID = 3))
/



INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNSC1.0'
		,  'Y'
		, 3
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'MsgStoreRequired'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNSC1.0' AND cv.ROUTINGOBJDEFAULT ='Y' 
	                       AND  cv.ROUTINGOBJTYPEID = 3))
/




INSERT INTO CF_ROUTINGATTR_PIP(ROUTINGATTRPIP_ID,ROUTINGATTRID,ROUTINGOBJVER,ROUTINGOBJDEFAULT,ROUTINGOBJTYPEID)
	SELECT CF_ROUTINGATTR_PIP_SEQ.nextval
	        ,cr.ROUTINGATTRID
		, 'RNSC1.0'
		,  'Y'
		, 2
	FROM CF_ROUTINGATTR cr
	WHERE cr.attributekey = 'MsgStoreRequired'
	AND NOT EXISTS 
		(SELECT 'x' 
		FROM CF_ROUTINGATTR cr INNER JOIN CF_ROUTINGATTR_PIP cv ON cr.ROUTINGATTRID=cv.routingattrid
		WHERE cv.ROUTINGATTRID = (SELECT cv.ROUTINGATTRID FROM CF_ROUTINGATTR_PIP cv
		                       WHERE cv.ROUTINGOBJVER='RNSC1.0' AND cv.ROUTINGOBJDEFAULT ='Y' 
	                       AND  cv.ROUTINGOBJTYPEID = 2))
/



-- CF_ARCHIVER_CONFIG.sql

INSERT INTO cf_archiver_config( 
	configid, 
	configname, configdesc
	, statusflag, keep_numdays, archiveflag, sched_type 
	,dayofmonth, dayofweek, houropt, minuteopt, freqdayofmonth, freqdayofweek, lockflag
	,archdest_fl, archdest_db
	)
	SELECT	CF_ARCHIVER_CONFIG_SEQ.nextval,
		'ARCHIVER'
		, 'Archive and Purge Data From the System'
		, 6 , 14, 0, 1
		, NULL ,'1', '22','0', 1, 1, 0
		, '$COMMON_LOC$/dataBackup', '$DBLOADER_INSTALL_PATH$/dataBackup'
	FROM	DUAL
	WHERE	NOT EXISTS (SELECT configid
			FROM	cf_archiver_config
			WHERE	configname = 'ARCHIVER'
			)
/
INSERT INTO cf_archiver_config( 
	configid, 
	configname, configdesc
	, statusflag, keep_numdays, archiveflag, sched_type 
	,dayofmonth, dayofweek, houropt, minuteopt, freqdayofmonth, freqdayofweek, lockflag
	,archdest_fl, archdest_db
	)
	SELECT	CF_ARCHIVER_CONFIG_SEQ.nextval,
		'RESTORE'
		, 'Archive and Purge Data From the System'
		, 6 , 14, 0, 1
		, NULL ,'1', '22','0', 1, 1, 0
		, '$COMMON_LOC$/dataBackup', '$DBLOADER_INSTALL_PATH$/dataBackup'
	FROM	DUAL
	WHERE	NOT EXISTS (SELECT configid
			FROM	cf_archiver_config
			WHERE	configname = 'RESTORE'
			)
/


-- UPDATE BUILD NUMBER

Update DB_SCHEMAVER Set BUILD = '6.2.0.1.333', UPDATEDATE = CURRENT_TIMESTAMP 
 / 
Update DB_SYSPROPERTIES Set PROPVALUE = '6.2.0.1.333', UPDATEDATE = CURRENT_TIMESTAMP Where PROPNAME = 'bcg.console.version' 
 / 
Commit 
 / 

-- # ============================================================== # 
-- Create Packages 
-- # ============================================================== # 

-- DB_REF.sql

CREATE OR REPLACE PACKAGE DB_REF
as
/*
    =============================================================================
    Proc Name:      DB_REF
    Create Date:    2002-04-07
    Author:         V.Goel
    =============================================================================
--================================================================================
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-E75,5724-E87,5724-L68,5724-L69
-- * (C) Copyright IBM Corp. 2001,2008 - All Rights Reserved.
-- * US Government Users Restricted Rights - Use, duplication or disclosure 
-- * restricted by GSA ADP Schedule Contract with IBM Corp.
-- *
--================================================================================*/
TYPE RCT IS REF CURSOR;

Type ReturnCount is record
    (TotalRowsNbr number);
TYPE RCT_Count IS REF CURSOR return ReturnCount;

END DB_REF;
/


-- # ============================================================== # 
-- Create Package Bodies 
-- # ============================================================== # 

-- -- # ============================================================== # 
-- # Update statements 
-- -- # ============================================================== # 

-- 620FP1MigrationTail.sql

CREATE OR REPLACE PROCEDURE JR32944
AS
	v_rowcnt1			NUMBER :=0;
	v_chkpt				NUMBER :=0;
	v_BeginTS			timestamp := FN_GET_DATE(); -- Performance related variables
	v_ProcName			varchar2(30) := 'JR32944';
	v_ExceptionFlg		integer := 0;
	v_transportattrid	pr_transport_attr.transportattrid%type;
	v_transportattrid1	pr_transport_attr.transportattrid%type;
	v_transportid		pr_transport.transportid%type;
	
BEGIN

		BEGIN
	   		
	   		Select transportid into v_transportid FROM pr_transport where transporttype='SFTPSender';
			EXCEPTION
				When NO_DATA_FOUND THEN
					v_chkpt := 1;
		END;
		
		BEGIN
	   		
			select transportattrid into v_transportattrid from pr_transport_attr pta 
				where resourcekey='bcg.destination.enableServerVerification';
			EXCEPTION
				When NO_DATA_FOUND THEN
					v_chkpt := 1;
		END;
		
		IF v_chkpt=0 THEN

			select count(pg.gatewayid) INTO	v_rowcnt1 from pr_gateway pg 
				where pg.transportid=v_transportid
			and not exists (select 'x' from pr_gwyattr pga where pga.gatewayid=pg.gatewayid 
								and pga.transportattrid=v_transportattrid);
	
		 	

			IF v_rowcnt1<>0 THEN
				
				select transportattrid into v_transportattrid1 from pr_transport_attr pta 
					where resourcekey='bcg.destination.hostKeyFile';
				
				BEGIN
					INSERT INTO pr_gwyattr( transportattrid
										,gatewayid
										,attrvalue
										,createdate
										,updatedate
										,rowts
										)
								(SELECT v_transportattrid
										,pg.gatewayid
										,'false'
										,v_BeginTS
										,v_BeginTS
										,1
								FROM PR_GATEWAY pg
								WHERE transportid=v_transportid 
								AND NOT EXISTS (SELECT 'X' FROM pr_gwyattr pga WHERE pga.transportattrid=v_transportattrid
													AND gatewayid=pg.gatewayid));
					
					INSERT INTO pr_gwyattr( transportattrid
										,gatewayid
										,attrvalue
										,createdate
										,updatedate
										,rowts
										)
								(SELECT v_transportattrid1
										,pg.gatewayid
										,''
										,v_BeginTS
										,v_BeginTS
										,1
								FROM PR_GATEWAY pg
								WHERE transportid=v_transportid 
								AND NOT EXISTS (SELECT 'X' FROM pr_gwyattr pga WHERE pga.transportattrid=v_transportattrid1
													AND gatewayid=pg.gatewayid));
					COMMIT;
				END;
			END IF;
		END IF;
	EXCEPTION
		WHEN Others THEN
			ROLLBACK;
		raise_application_error (-20999, '20002 SFTP METADATA INSERTION WITH INTERNAL ERROR ' || SQLERRM);
END;
/
Call JR32944()
/
Drop procedure JR32944
/
update db_sysproperties set propvalue=(select propvalue from db_sysproperties where propname='bcg.ftp.config.listeners.clientauth.data-connection.active.local-address')
	where propname='bcg.ftp.config.listeners.default.data-connection.active.local-address'
/

-- UPDATE BUILD NUMBER

Update DB_SCHEMAVER Set BUILD = '6.2.0.1.333', UPDATEDATE = CURRENT_TIMESTAMP 
 / 
Update DB_SYSPROPERTIES Set PROPVALUE = '6.2.0.1.333', UPDATEDATE = CURRENT_TIMESTAMP Where PROPNAME = 'bcg.console.version' 
 / 
Commit 
 / 

--  -- # ============================================================== # !
--  # Recompile !
--  -- # ============================================================== # !

-- Recompile.sql

Create or Replace Procedure recompile as

v_compile_failures varchar2(4000) := 'DB_recompile: Failed for following objects : ';
v_fail_flag INTEGER := 0;
COMPILE_FAIL exception;
v_object_type user_objects.object_type%type;
v_object_name user_objects.object_name%type;
v_retrycnt integer := 0;
v_retry integer := 0;
v_error number;
v_errmsg varchar2(250);
v_cnt integer;

PROCEDURE object_recompile as

sql_stmt  varchar2(200);

cursor C_object is  select object_type, object_name from user_objects where status='INVALID';

begin
 
     Open c_object;
     Loop
        Fetch c_object into v_object_type, v_object_name;
        Exit When c_object%NotFound;
           sql_stmt := 'alter '|| v_object_type || ' ' || v_object_name || ' compile';
           begin
              execute immediate sql_stmt;
                 exception
                    when OTHERS then
                       NULL;
                    v_fail_flag := 1;
                 end;   
     End Loop;
     Close c_object;
     
end object_recompile;

---------------------------------------------------------
--                     M  A  I  N                      --
---------------------------------------------------------

BEGIN
    v_cnt := 0;
    Loop       
       
       v_cnt := v_cnt + 1;
       object_recompile;
       Exit when v_cnt >= 10;
    End Loop;   
     
    EXCEPTION
    WHEN COMPILE_FAIL THEN
         raise_application_error(-20999,20002||'-'||v_compile_failures);

END recompile;
/
Call recompile()
/
drop procedure recompile
/
-- spool off;
-- EXIT;

